/*---------------------------------------------------------------------
 *                                                                     
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                       
 *      COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *      THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                       
 *                         INTEL  CORPORATION                            
 *                                                                      
 *                      2200 MISSION COLLEGE BLVD                        
 *                                                                       
 *                SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                       
 *---------------------------------------------------------------------
 *
 *        File Name: dl_system_ingress_a.h
 *
 *        Purpose: This file contains system wide parameters
 *                 that are customized for a given application.
 *        History:
 *
 *        Date            Comment                         By
 *        ---------------------------------------------------------------
 *
 *        11/11/2002      Created                         asv
 *
 *-------------------------------------------------------------------*/
#ifndef	__DL_SYSTEM_INGRESS_A__H__
#define	__DL_SYSTEM_INGRESS_A__H__

/* Define chip. */
#ifndef IXP2800
#define IXP2800
#endif

/* Define project as microcode based. */
#ifndef IXP_MICROCODE
#define IXP_MICROCODE
#endif

/* PHY mode for TX device. */
#ifndef	TX_PHY_MODE
#define TX_PHY_MODE 			SPHY_1_32
#endif

/* Put IPv4 counters in scratch. */
#define IPV4_COUNTERS_SCRATCH

/* Put IPv4 DBcast table in local memory */
#define LM_DBCAST_TABLE

 /* Used compressed next hop id entry format */
#define COMPRESSED_NEXTHOP_INFO

/* IngressA doesn't have QM so don't need to split metadata */
//#define	DL_SPLIT_META_DATA

#ifdef	DL_SPLIT_META_DATA

#define	BUF_SRAM_SPLIT_LW		7
#define	BUF_SRAM_SPLIT_CHANNEL	0

#endif // DL_SPLIT_META_DATA

/* Define next block id's for PPP decap */
#define	BID_PPP_DECAP_NEXT1		BID_IPV4
#define	BID_PPP_DECAP_NEXT2		BID_MPLSILM
#define	BID_PPP_DECAP_NEXT3		BID_IPV6

#define	IPV4_NEXT1				BID_MPLSFTN

/* Include default settings. */

/*
 * Default size for PKT HDR to be cached in long words:
 *	6 or 8 longwords
 */
#ifndef	PKT_HDR_CACHE_SIZE
#define	PKT_HDR_CACHE_SIZE		6
#endif

/* Heading caching in Local Memory
 * Starting LW: 512
 * Size: 8 (threads) * 16 LWs = 128 LWs
 * LM memory map used by MPLS hdr caching: from LW 512 to LW 1024
 */
#define HDR_CACHE_LM_BASE		512
#define HDR_CACHE_LM_SIZE		64

#define LM_LABEL_SPACE_ADDR     0
#define INTERFACE_MAX           16

/* Define Label Space Type:
 * 	PER_PLATFORM_LABEL_SPACE (default) or PER_INTERFACE_LABEL_SPACE
 */
#ifndef PER_INTERFACE_LABEL_SPACE
    #ifndef PER_PLATFORM_LABEL_SPACE
	    #define PER_PLATFORM_LABEL_SPACE
    #endif
#endif

/* SRAM channel allocations */
#ifndef		USE_IMPORT_VAR

/* SRAM channel 0 includes MetaData and NextHopID */
#ifndef		NEXTHOP_TABLE_SRAM_BASE
#define		NEXTHOP_TABLE_SRAM_BASE		(0x00000000 + 0x300000)
#endif

/* SRAM channel 1 includes TRIE */
#ifndef		TRIE_TABLE_SRAM_BASE
#define		TRIE_TABLE_SRAM_BASE		(0x40000000 + 0x02000000)
#endif

/* SRAM channel 2 includes DBcast (init only) and MPLS/IPv6 */

#ifndef 	MPLS_FTN_TABLE_BASE
#define 	MPLS_FTN_TABLE_BASE				(0x80000000 + 0x4000)
#endif

#ifndef 	MPLS_ILM_TABLE_BASE
#define 	MPLS_ILM_TABLE_BASE				(0x80000000 + 0x204000)
#endif

#ifndef 	MPLS_ILM_NHLFE_SET_TABLE_BASE
#define 	MPLS_ILM_NHLFE_SET_TABLE_BASE	(0x80000000 + 0x404000)
#endif

#ifndef 	MPLS_ILM_PARAM_TABLE_BASE
#define 	MPLS_ILM_PARAM_TABLE_BASE		(0x80000000 + 0x484000)
#endif

#ifdef __COUNTERS__
#define 	MPLS_STATS_FULL
#define 	MPLS_INPUT_FTN_STAT_BASE		(0xC0000000 + 0x18000)
#define 	MPLS_INPUT_ILM_STAT_BASE		(0xC0000000 + 0x19000)
#define 	MPLS_ILM_NHLFE_SET_STAT_BASE   	(0xC0000000 + 0x25000)
#define 	MPLS_INPUT_PORT_STAT_BASE   	(0xC0000000 + 0x27000)
#endif

#endif	/* USE_IMPORT_VAR for SRAM channel allocations */

#ifdef __COUNTERS__
#define 	MPLS_STATS_FULL
#endif

/* This file is also used by script files that setup the system when run 
 * in simulation mode. When symbol IXP_SCRIPTS is defined, it indicates
 * that scripts are using this file. Since script files cannot access
 * path definitions, they should not encounter '#include <file>'. Instead
 * they include the <file> using Simulation -> Options -> Startup.
 */
#ifndef	IXP_SCRIPTS
#include <dl_ingress_system_default_2800.h>
#endif

/* Static bindings for MPLS blocks */
#ifdef		BID_MPLSFTN_NEXT1
#undef 		BID_MPLSFTN_NEXT1
#endif
#define		BID_MPLSFTN_NEXT1		BID_QM

#ifdef		BID_MPLSILM_NEXT1
#undef		BID_MPLSILM_NEXT1
#endif
#define		BID_MPLSILM_NEXT1		BID_QM

#ifdef		BID_MPLSILM_NEXT2
#undef		BID_MPLSILM_NEXT2
#endif
#define		BID_MPLSILM_NEXT2		BID_IPV4

#ifdef		BID_MPLSILM_NEXT3
#undef		BID_MPLSILM_NEXT3
#endif
#define		BID_MPLSILM_NEXT3		BID_IPV6

#endif	/* __DL_SYSTEM_INGRESS_A__H__ */