///////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                 
//                                                                      
///////////////////////////////////////////////////////////////////////
//
//
//      File Name: packet_proc_ingress_a.uc
//
//      Purpose: This file contains packet processing microblocks.
//
//      History:
//
//      Date            Comment                         By
//      ---------------------------------------------------------------
//
//      04/25/2003      Created                         asv
//
///////////////////////////////////////////////////////////////////////

#ifndef	__PACKET_PROC_INGRESS_A__UC__
#define	__PACKET_PROC_INGRESS_A__UC__

#define			USE_DL_THREAD_ID

//	Include Files

// include system.h from dispatch loop
#include	"dl_system.h"

#include	"definitions.h"

// include system_init.h from dispatch loop for inter ME signal
#include	"system_init.uc"

// include dispatch_loop.uc from dispatch loop 
#include	"dispatch_loop.uc"

// include ipv4_fwder_dl.uc from IPV4 forwarder 
#include	"ipv4_fwder_init.uc"

// include ipv4_fwder.uc from IPV4 forwarder 
#include	"ipv4_fwder.uc"

// include ppp.uc from rx
#include 	"ppp.uc"

// include MPLS microblocks
#include	"mpls_ilm.uc"
#include	"mpls_ftn.uc"

// define header cache parameters
#define_eval	HDR_OFFSET			16
#define_eval 	HDR_CACHE			lmem_buf0

//-----------------------------------------------------------
// Global variables
//-----------------------------------------------------------
.reg global volatile ctx_rel_lm_addr 		// dispatch loop cache LM address
.reg global volatile ctx_rel_xfer_addr 		// first per ctx xfer address
.reg global volatile base_indirect
.reg global volatile labels_in_cache
.reg global volatile labels_to_write

//-----------------------------------------------------------
// Initialize the blocks
//-----------------------------------------------------------

init#:
	// allocate cache to store the packet header
	dl_iphdr_cache_init[]
#ifndef	__PKTHDR
#define	__PKTHDR	$$iphdr	; for convenience. DL defines this.
#else
#warning	"__PKTHDR is already defined."
#endif

	; init buffer meta data cache to Zero.
	init_meta_cache_to_zero[]

	; Temporary : Initialize EOP handle default.
	_dl_eop_buf_handle_init[]

	; header caching initialization
	_dl_hdr_caching_init(HDR_CACHE)

	; mpls initialization
	_mpls_init()

	; ipv4fwder initialization
	_ipv4_fwder_init()

	; now kick start the IPV4 Forwarder
	_ipv4_fwder_kick_start()


//-----------------------------------------------------------
// dispatch loop
//-----------------------------------------------------------
.begin
.reg sig_mask
	immed32[sig_mask, 0]

	// get packets in order.
	dl_source[DL_THREAD_ORDER, sig_mask, HDR_CACHE]
.end

// Thread 0 of the IPV4_START_ME  from here on
// will process only exception packets. There 
// is enough headroom available and will not affect
// performance

#ifdef	IPV4_START_ME
	br=ctx[0,_handle_exception_pkt#]
#endif

loop#:

	_ppp_decap_classify(__PKTHDR,PPP_HEADER_OFFSET)

	// process MPLS packets: LSR and Egress LER
	// HDR_CACHE points to cache buffer
	// HDR_OFFSET indicates where packet header begins in cache buffer
	mpls_ilm(HDR_CACHE, HDR_OFFSET)

	// process IP packets.
	_ipv4_fwder(HDR_CACHE, HDR_CACHE, HDR_OFFSET, HDR_OFFSET)

	// process MPLS packets: Ingress LER
	mpls_ftn(HDR_CACHE, HDR_OFFSET)

	// Now call DL macros to send the current packet
	// and received the next packet. Depending on the dl_next_block, these
	// macros can drop/send exception/send to next block. Also, we need to
	// maintain packet order when we send/receive packets.

.reg sig_mask
.sig sig_scr_put
.set_sig sig_scr_put

	// allocate xfer registers to be used by dl_qm_sink.
	xbuf_alloc[$wxfer, 3, write]
	immed32[sig_mask, 0]

	// send packet.
	dl_qm_sink[sig_mask, sig_scr_put, $wxfer, HDR_CACHE]

	// receive packet
	dl_source[DL_THREAD_NO_ORDER, sig_mask, HDR_CACHE]

.io_completed sig_scr_put

	// free the xfer registers allocated
	xbuf_free[$wxfer]

	// start again.
	br[loop#]
	
#ifdef	IPV4_START_ME

_handle_exception_pkt#:
	
		dl_get_exception_pkt()

	    br[_handle_exception_pkt#]	

#endif

// We never execute the below code. This is here for completeness only.

term#:

	// frees the IP header cache
	dl_iphdr_cache_fini[]

	// Control should never come here.
kill#:
	nop
	nop

#endif	// __PACKET_PROC_INGRESS_A__UC__