/*------------------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//------------------------------------------------------------------------------------
*/

#ifndef __PKTHDR_CACHE_UC__
#define __PKTHDR_CACHE_UC__

#ifndef			PKT_HDR_DRAM_RD_SIZE
#define_eval	PKT_HDR_DRAM_RD_SIZE	(PKT_HDR_CACHE_SIZE >> 1)	// read 3 quadwords
#endif

///////////////////////////////////////////////////////////////////////////////
// dl_load_iphdr_cache:
//	 	Description: 
//			Reads the IP header (32 bytes) of the packet from SDRAM and keeps 
//			it in a cache (DRAM transfer registers). 
//
//	 	Outputs:
//			The $$dl_iphdr# global dram transfer registers will be set with appropriate
//			header data.
//
//		Inputs:
//			buf_handle	:			Handle of buffer.
//			req_sig		:			Signal to use in the I/O (DRAM read) operation.
//			sig_action	:			What to do with I/O operation - 
//									- wait for signal(s) as specified by sig_action
//									- do not wait for signal, just return (SIG_NONE)
//		Size:   				:
//
#define	dl_load_iphdr_cache		dl_iphdr_load_cache

#macro dl_iphdr_load_cache[buf_handle, req_sig, sig_action]
.begin
.reg 	sdram_offset, buf_offset

	//	Read 32 bytes of IP Header from SDRAM

	dl_buf_get_data[sdram_offset, buf_handle]
	dl_meta_get_offset[buf_offset]
	dram[read, $$iphdr0, sdram_offset, buf_offset, PKT_HDR_DRAM_RD_SIZE], sig_done[req_sig]	; DRAM reads are in QWORD (8 bytes)

#if (!streq('sig_action', 'SIG_NONE'))
	ctx_arb[sig_action]
#endif

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_flush_iphdr_cache:
//	 	Description: 
//			Writes back the IP header (32 bytes) from the cache to the packet in SDRAM. 
//
//	 	Outputs:
//
//		Inputs:
//			buf_handle	:			Handle of buffer.
//			req_sig		:			Signal to use in the I/O (DRAM read) operation.
//			sig_action	:			What to do with I/O operation - 
//									- wait for signal(s) as specified by sig_action
//									- do not wait for signal, just return (SIG_NONE)
//			The $$dl_iphdr# global dram transfer registers will be the set of registers 
//			from which data will be written back to SDRAM.
//
//		Size:   				:
//
#define	dl_flush_iphdr_cache		dl_iphdr_flush_cache

#macro dl_iphdr_flush_cache[buf_handle, req_sig, sig_action]
.begin
.reg 	sdram_offset, buf_offset

	//	Write 32 bytes of IP Header from SDRAM

	dl_buf_get_data[sdram_offset, buf_handle]
	dl_meta_get_offset[buf_offset]


#ifdef	PPP_RECEIVE
	alu[buf_offset,buf_offset,-,2]
#endif


	dram[write, $$iphdr0, sdram_offset, buf_offset, PKT_HDR_DRAM_RD_SIZE], sig_done[req_sig]	; DRAM reads are in QWORD (8 bytes)

#if (!streq('sig_action', 'SIG_NONE'))
	ctx_arb[sig_action]
#endif

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_iphdr_cache_init:
//	 	Description: 
//			This macro needs to be called before any of the dl_iphdr_* can be called.
//			It allocates a set of DRAM xfer registers to be used for caching. This macro
//			doesn't generate any code (instructions). It only produces some assembler
//			directives.
//
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//			0 Instructions

#macro dl_iphdr_cache_init[]

	xbuf_alloc[$$iphdr, PKT_HDR_CACHE_SIZE, read_write]

#endm

///////////////////////////////////////////////////////////////////////////////
// dl_iphdr_cache_fini:
//	 	Description: 
//			This macro needs to be called at the end when dl_iphdr_macros are no longer
//			required. It frees a set of DRAM xfer registers that were allocated using
//			dl_iphdr_cache_init. This macro	doesn't generate any code (instructions). 
//			It only produces some assembler	directives.
//
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//			0 Instructions

#macro dl_iphdr_cache_fini[]

	xbuf_free[$$iphdr]

#endm

///////////////////////////////////////////////////////////////////////////////
// 		_dl_hdr_caching_init()
//
//	 	Description:
//				Allocate local memory buffer for caching packet headers.
//				Prepare base register for DRAM indirect format for copying
//					packet headers from/to DRAM and local memory.
//				
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//
//		Notes:
// 			+ This macro should be called before dl_source_mpls
//			+ LM index 0 is used

#macro _dl_hdr_caching_init(HDR_CACHE)
.begin

	.reg ctx_no

	// Initiate dl variables
	immed32(dl_next_block, 0)
	immed32(dl_buf_handle, 0)
	immed32(dl_exception_reg, 0)

	// Read the active context number.
	local_csr_rd[active_ctx_sts]
	immed[ctx_no, 0]
	alu[ctx_no, ctx_no, and, 7]

	// Set per context xfer reg base address
	alu_shf[ctx_rel_xfer_addr, --, B, ctx_no, <<6]

	// Prepare base register for DRAM indirect format
	#define DRAM_INDIRECT_REF_REF_CNT_OV_OFFSET			25
	#define DRAM_INDIRECT_REF_XFER_REG_ADDR_OV_OFFSET	4

	// Set OV bit for ref cnt
	alu[base_indirect, --, b, 1, <<DRAM_INDIRECT_REF_REF_CNT_OV_OFFSET]

	// Set OV bit for xfer reg address
	alu_shf[base_indirect, base_indirect, \
						OR, 1, <<DRAM_INDIRECT_REF_XFER_REG_ADDR_OV_OFFSET]

	// The base address should be (ctx_number * 16) moved to bits 11:5
	// ctx_rel_xfer_addr contains (ctx_number * 64), so we must shift it
	// left by three bits
	alu[base_indirect, base_indirect, OR, ctx_rel_xfer_addr, <<3]

	#undef DRAM_INDIRECT_REF_REF_CNT_OV_OFFSET
	#undef DRAM_INDIRECT_REF_XFER_REG_ADDR_OV_OFFSET

	// There is 16 LW for packet caching reserved in local memory for every
	// context. Calculate cache base address for current thread.
.reg hdr_cache_lm_temp_base
	immed32(hdr_cache_lm_temp_base, HDR_CACHE_LM_BASE)
	alu[ctx_rel_lm_addr, ctx_rel_xfer_addr, +, hdr_cache_lm_temp_base]

	// init cache in LM
	xbuf_alloc(HDR_CACHE, 16, read_write)
	xbuf_bind_address(HDR_CACHE, HDR_CACHE_LM_BASE, HDR_CACHE_LM_SIZE, 0)
	xbuf_activate(HDR_CACHE, 0, ctx_no, 0)

.end
#endm

///////////////////////////////////////////////////////////////////////////////
// 		dl_load_hdr_cache_lm(dl_packet_offset, HDR_CACHE)
//
//	 	Description:
//				Reads packet header from DRAM,
//				and caches aligned in local memory.
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//

#macro dl_load_hdr_cache_lm(HDR_CACHE)
.begin

	.reg lm_address
	.reg t_idx_byte_idx

	// Register T_INDEX_BYTE_INDEX holds:
	//	- transfer register address on bits 8:2
	//  - quadword offset on bits 7:0
	// Read transfer register address and move it to bits 8:2.
	// ctx_rel_xfer_addr holds context-relative transfer register base addres.

	alu_shf[t_idx_byte_idx, ctx_rel_xfer_addr, OR , &$$iphdr0, <<2]

	// Add quadword offset on bits 2:0
	//
	// For now, assume that quadword offset is equal to PPP_HEADER_LENGTH.
	// This is because (packet offset % 8) == 0, check
	// _packet_get_rand_buf_offset macro in packet_rx_util.uc.
	//
	// Generally, if quadword offset is between 4-7 (and thus bit 2 is set),
	// we will add one to T_INDEX field and we will start reading data
	// from $$ip_header_read[1].

	alu[t_idx_byte_idx, t_idx_byte_idx, +, PPP_HEADER_LENGTH]

	local_csr_wr[T_INDEX_BYTE_INDEX, t_idx_byte_idx]

	// there must be three instructions between
	// local_csr_wr and byte_align_be
	nop

	// these are default conditions (cache IPv4 header)
	immed32(labels_in_cache, (PKT_HDR_CACHE_SIZE - 1))
	immed32(labels_to_write, 3)

#if (!isnum(PKT_HDR_CACHE_SIZE))
    #error "PKT_HDR_CACHE_SIZE must be a numeric constant."
#endif
#if ((PKT_HDR_CACHE_SIZE != 6) && (PKT_HDR_CACHE_SIZE != 8))
    #error "PKT_HDR_CACHE_SIZE must not be either 6 or 8."
#endif

.reg tmp_cache0
.reg tmp_cache1
#if (PKT_HDR_CACHE_SIZE == 8)
.reg tmp_cache2
.reg tmp_cache3
#endif

	// Here we align and load labeled packet or ipv4 packet header.
	byte_align_be[--, *$$index++]
	byte_align_be[HDR_CACHE[4], *$$index++]
	byte_align_be[HDR_CACHE[5], *$$index++]
	byte_align_be[HDR_CACHE[6], *$$index++]
	byte_align_be[HDR_CACHE[7], *$$index++]
	byte_align_be[tmp_cache0, *$$index++]
#if (PKT_HDR_CACHE_SIZE == 6)
	byte_align_be[tmp_cache1, 0]
#endif
#if (PKT_HDR_CACHE_SIZE == 8)
	byte_align_be[tmp_cache1, *$$index++]
	byte_align_be[tmp_cache2, *$$index++]
	byte_align_be[tmp_cache3, 0]
#endif

	move(HDR_CACHE[8], tmp_cache0)
	move(HDR_CACHE[9], tmp_cache1)
#if (PKT_HDR_CACHE_SIZE == 8)
	move(HDR_CACHE[10], tmp_cache2)
	move(HDR_CACHE[11], tmp_cache3)
#endif

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// 		dl_flush_hdr_cache_lm(sig_cache, HDR_CACHE)
//
//	 	Description:
//				Flushes packet header from LM cache into DRAM.
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//

#macro dl_flush_hdr_cache_lm(sig_cache, HDR_CACHE)

.reg dram_offset, qword_offset
.reg xfer_addr
.reg t_idx_byte_idx

	xbuf_alloc($$header, 10, write)

	dl_meta_get_offset(dram_offset)
	alu[qword_offset, dram_offset, AND, 0x7]
	alu_shf[xfer_addr, ctx_rel_xfer_addr, OR , &$$header0, <<2]
	alu[t_idx_byte_idx, xfer_addr, +, qword_offset]

	local_csr_wr[T_INDEX_BYTE_INDEX, t_idx_byte_idx]

.reg temp_hdr8 temp_hdr9
	move(temp_hdr8, HDR_CACHE[8])
	move(temp_hdr9, HDR_CACHE[9])

	byte_align_be[--, 0]
	byte_align_be[*$$index++, HDR_CACHE[0]]
	byte_align_be[*$$index++, HDR_CACHE[1]]
	byte_align_be[*$$index++, HDR_CACHE[2]]
	byte_align_be[*$$index++, HDR_CACHE[3]]
	byte_align_be[*$$index++, HDR_CACHE[4]]
	byte_align_be[*$$index++, HDR_CACHE[5]]
	byte_align_be[*$$index++, HDR_CACHE[6]]
	byte_align_be[*$$index++, HDR_CACHE[7]]
	byte_align_be[*$$index++, temp_hdr8]
	byte_align_be[*$$index++, temp_hdr9]

	.set $$header0
	.set $$header1
	.set $$header2
	.set $$header3
	.set $$header4
	.set $$header5
	.set $$header6
	.set $$header7
	.set $$header8
	.set $$header9

.reg dram_handle
	dl_buf_get_data(dram_handle, dl_buf_handle)

	// calculate reference count value to be set in indirect word
.reg ref_count
.set labels_to_write
	alu_shf[ref_count, --, B, labels_to_write, <<2]
	alu[ref_count, ref_count, +, qword_offset]
	alu[ref_count, ref_count, -, 2]
	alu_shf[ref_count, --, B, ref_count, >>3]

.reg indirect_word
.set labels_in_cache
	alu[indirect_word, (PKT_HDR_CACHE_SIZE + 3), - , labels_in_cache]
	alu[indirect_word, indirect_word, +, &$$header0]
	alu_shf[indirect_word, base_indirect, OR, indirect_word, <<5]
	alu_shf[indirect_word, indirect_word, OR, ref_count, <<21]

	dram[write, $$header0, dram_handle, dram_offset, max_4], \
			indirect_ref, sig_done[sig_cache]

	xbuf_free($$header)

#endm

#endif	//__PKTHDR_CACHE_UC__