/*---------------------------------------------------------------------
 *                                                                     
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                       
 *      COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *      THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                       
 *                         INTEL  CORPORATION                            
 *                                                                      
 *                      2200 MISSION COLLEGE BLVD                        
 *                                                                       
 *                SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                       
 *---------------------------------------------------------------------
 *
 *        File Name: dl_system_ingress_a.h
 *
 *        Purpose: This file contains system wide parameters
 *                 that are customized for a given application.
 *        History:
 *
 *        Date            Comment                         By
 *        ---------------------------------------------------------------
 *
 *        11/11/2002      Created                         asv
 *
 *-------------------------------------------------------------------*/
#ifndef	__DL_SYSTEM_INGRESS_A__H__
#define	__DL_SYSTEM_INGRESS_A__H__

/* Define chip. */
#ifndef IXP2800
#define IXP2800
#endif

/* Define project as microcode based. */
#ifndef IXP_SCRIPTS
#ifndef IXP_MICROCODE
#define IXP_MICROCODE
#endif
#endif

/* PHY mode for TX device. */
#ifndef	TX_PHY_MODE
#define TX_PHY_MODE 				SPHY_1_32
#endif

/* Size of the packet header to be cached (in long words) */ 
#define PKT_HDR_CACHE_SIZE   		16

/* Put IPv4 counters in scratch. */
#define IPV4_COUNTERS_SCRATCH

/* Define next block id's for PPP decap */
#define	BID_PPP_DECAP_NEXT1			BID_IPV4
#define	BID_PPP_DECAP_NEXT2			BID_MPLSILM
#define	BID_PPP_DECAP_NEXT3			BID_IPV6

#define BID_V6V4_TUNNEL_DECAP		0x29
#define BID_V6V4_TUNNEL_ENCAP		0x2a

#define IPV6_NEXT_BLOCK				BID_V6V4_TUNNEL_ENCAP
#define V6V4_NEXT_IPV4				BID_IPV4
#define V6V4_NEXT_IPV6				BID_IPV6
#define V6V4_NEXT_NO_ENCAP			BID_QM
#define V6V4_NEXT_NO_DECAP			BID_QM

/*
 * Tunnel Encapsulation Flag for Header Type.  Must not conflict with
 * any of the header types defined in definitions.h
 */
#define TUNNEL_ENCAP_FLAG			0x10

/* Values possible for nexthop_id_type */
#define NEXTHOP_TYPE_FORWARD		0x0
#define NEXTHOP_TYPE_TUNNEL			0x2

/* Signal used by timer for refreshing IPv6 trie-cache */
#define IPV6_CACHE_REFRESH_SIGNAL	11


/*---------------------------------------------------------------------
 * SRAM memory map for channel 0.
 *-------------------------------------------------------------------*/

/* 
 * SRAM location where router's common IPv6 prefix will be
 * stored by XScale core.
 */

#define IPV6_ROUTER_PREFIX_SRAM_BASE			(SRAM_CH0_BASE + 0x300000)
#define IPV6_ROUTER_PREFIX_SRAM_SIZE			0x10

/* Local memory location where router's IPv6 prefix is cached
 * with trie-entries.
 */ 

#define IPV6_TRIE_CACHE_LM_BASE					0x418
#define IPV6_TRIE_CACHE_LM_SIZE					0x10


/*---------------------------------------------------------------------
 * SRAM memory map for channel 1.
 *-------------------------------------------------------------------*/

/* The prefix table for validating source addresses on tunneled
 * packets requiring decapsulation can be a simple linear list
 * or a trie table.  There are performance tradeoffs for using
 * each option.  The linear list is designed to be used in cases
 * where each tunnel supports a small number of prefixes (about 12
 * or less).  The trie table can be used if each tunnel can have a
 * large number of source prefixes.  However, since the trie table
 * is global for all ending tunnel endpoints, if more than one
 * tunnel allows a single prefix, a separate list needs to be used
 * to hold the tunnels that allow a particular prefix.
 */

/* Tunnel Ingress Source Block Array using Linear List */
#ifndef USE_IMPORT_VAR

#ifndef	TUNNEL_INGRESS_LIST_SRAM_BASE
#define TUNNEL_INGRESS_LIST_SRAM_BASE			(SRAM_CH1_BASE + 0x2000)
#endif

/* Base address for IPv6 counters in sram. */
#ifndef IPV6_STATS_TABLE_SRAM_BASE
#define IPV6_STATS_TABLE_SRAM_BASE  			(SRAM_CH1_BASE + 0x6000)
#endif

#endif /* USE_IMPORT_VAR */


#ifdef V6V4_INGRESS_LIST_TRIE

/* Tunnel Ingress Source List using Trie Table */
#define TUNNEL_INGRESS_LIST_NUM_TRIE_BLOCKS   	512

/* The Ingress List trie entry contains an index, which can be either
 * a index of the tunnel next hop entry that supports the prefix, or it
 * can be 0xffff to indicate the prefix is valid for all tunnels, or
 * it can be an index into an array containing indexes of tunnel next hop
 * entries that support the prefix.  If a single prefix is allowed for
 * only one tunnel, V6V4_INGRESS_LIST_INDEX_BLOCK_SIZE should be set to
 * zero to avoid using a list.
 */

#define V6V4_INGRESS_LIST_INDEX_BLOCK_SIZE		8
#define V6V4_INGRESS_LIST_NUM_INDEX_BLOCKS    	256

#else

/* Tunnel Ingress Source Block Array using Linear List */
#define V6V4_NUM_INGRESS_LIST_BLOCKS            256
#define V6V4_INGRESS_LIST_LW                    16

/* The maximum number of addresses per tunnel can be set to a value
 * from 1-12. If V6V4_INGRESS_LIST_MAX_ADDR is not set, then multiple
 * blocks can be used to store additional prefixes for a single tunnel,
 * if more are needed.  If V6V4_INGRESS_LIST_MAX_ADDR is set, then
 * only a single block can be used to store the addresses, and only the
 * specified number of addresses will be checked when validating a
 * source address.
 */

#define V6V4_INGRESS_LIST_MAX_ADDR  12

#endif   /* V6V4_INGRESS_LIST_TRIE */

/* 
 * IPv6 Definitions
 */
#define PORTS_PER_BLADE_SHIFT               4
#define NUM_BLADES                          0x40


/*---------------------------------------------------------------------
 * SRAM memory map for channel 2.
 *-------------------------------------------------------------------*/

#ifndef USE_IMPORT_VAR

#ifndef	IPV6_L3_NEXTHOP_TABLE_SRAM_BASE
#define IPV6_L3_NEXTHOP_TABLE_SRAM_BASE		(SRAM_CH2_BASE + 0x8000)
#endif

/* Tunnel Next-Hop Arrays */

#ifndef	TUNNEL_ENCAP_NEXTHOP_SRAM_BASE
#define TUNNEL_ENCAP_NEXTHOP_SRAM_BASE		0x80003000
#endif

#ifndef	TUNNEL_DECAP_NEXTHOP_SRAM_BASE
#define TUNNEL_DECAP_NEXTHOP_SRAM_BASE		0x80005000
#endif

#endif /* USE_IMPORT_VAR */

#define IPV6_NUM_L3_NEXTHOPS               	NUM_NEXTHOPS
#define	IPV6_SIZE_OF_L3_NEXTHOP_LW         	SIZE_OF_NEXTHOP_LW

#define NUM_START_TUNNEL_NEXTHOPS          	256
#define START_TUNNEL_NEXTHOP_LW            	8
#define TUNNEL_ENCAP_NEXTHOP_SRAM_SIZE     	0x2000

#define NUM_END_TUNNEL_NEXTHOPS        		128
#define END_TUNNEL_NEXTHOP_LW              	1
#define TUNNEL_DECAP_NEXTHOP_SRAM_SIZE     	0x200


/*---------------------------------------------------------------------
 * SRAM memory map for channel 3.
 *-------------------------------------------------------------------*/

#ifndef USE_IMPORT_VAR

#ifndef	IPV6_TRIE_TABLE_SRAM_BASE

#ifdef SPLIT_IPV6_TRIE_TABLE
#define IPV6_TRIE_TABLE_SRAM_BASE_16		(SRAM_CH2_BASE + 0x10000)
#define IPV6_TRIE_TABLE_SRAM_BASE_8			(SRAM_CH3_BASE + 0x02200000)
#else
#define IPV6_TRIE_TABLE_SRAM_BASE			(SRAM_CH3_BASE + 0x02200000)
#endif

#endif

#endif /* USE_IMPORT_VARS */

#define	SIZE_OF_IPV6_TRIE_SET    			SIZE_OF_TRIE_SET
#define	NUM_IPV6_TRIE_SETS	                NUM_TRIE_SETS

/* Include default settings. */

/* This file is also used by script files that setup the system when run 
 * in simulation mode. When symbol IXP_SCRIPTS is defined, it indicates
 * that scripts are using this file. Since script files cannot access
 * path definitions, they should not encounter '#include <file>'. Instead
 * they include the <file> using Simulation -> Options -> Startup.
 */
#ifndef	IXP_SCRIPTS
#include <dl_ingress_system_default_2800.h>
#endif

#endif	/* __DL_SYSTEM_INGRESS_A__H__ */