///////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                 
//                                                                      
///////////////////////////////////////////////////////////////////////
//
//
//      File Name: packet_proc_ingress_a.uc
//
//      Purpose: This file contains packet processing microblocks.
//
//      History:
//
//      Date            Comment                         By
//      ---------------------------------------------------------------
//
//      04/25/2003      Created                         asv
//
///////////////////////////////////////////////////////////////////////

#ifndef	__PACKET_PROC_INGRESS_A__UC__
#define	__PACKET_PROC_INGRESS_A__UC__

//	Include Files

// include system.h from dispatch loop
#include	"dl_system.h"

#include	"definitions.h"

// include system_init.h from dispatch loop for inter ME signal
#include	"system_init.uc"

// include dispatch_loop.uc from dispatch loop 
#include	"dispatch_loop.uc"

// include pkthdr_cache.uc from dispatch loop 
#include	"pkthdr_cache_a.uc"

// include ipv4_fwder.uc from IPV4 forwarder 
#include	"ipv4_fwder.uc"

// include ipv6_fwder.uc from IPV4 forwarder 
#include	"ipv6_fwder.uc"

// include tunneling blocks
#include    "v6v4_tunnel_decap.uc"
#include    "v6v4_tunnel_encap.uc"

// include ppp.uc from rx
#include 	"ppp.uc"

// constants
// None.


//-----------------------------------------------------------
// Initialize the blocks
//-----------------------------------------------------------

init#:
	// allocate cache to store the packet header
	dl_iphdr_cache_init[]
#ifndef	__PKTHDR
#define	__PKTHDR	$$iphdr	; for convenience. DL defines this.
#else
#warning	"__PKTHDR is already defined."
#endif

	; init buffer meta data cache to Zero.
	init_meta_cache_to_zero[]

	; Temporary : Initialize EOP handle default.
	_dl_eop_buf_handle_init[]

	; ipv4fwder initialization
	_ipv4_fwder_init()

	; ipv6fwder initialization
	ipv6_forwarder_init()

	; tunneling initialization
	v6v4_tunnel_decap_init()
	v6v4_tunnel_encap_init()

	; now kick start the IPV4 Forwarder
	_ipv4_fwder_kick_start()


//-----------------------------------------------------------
// dispatch loop
//-----------------------------------------------------------
.begin
.reg sig_mask
	immed32[sig_mask, 0]

	// get packets in order.
	dl_source[DL_THREAD_ORDER, sig_mask]

.end

// Thread 0 of the IPV4_START_ME  from here on
// will process only exception packets. There 
// is enough headroom available and will not affect
// performance

#ifdef	IPV4_START_ME
	br=ctx[0,_handle_exception_pkt#]
#endif

loop#:

	_ppp_decap_classify(__PKTHDR,PPP_HEADER_OFFSET)

    // Align and copy the IP header into local memory.  The
	// header will be copied to a known local memory address
	// depending on the header type (IPv4 or IPv6).

	dl_copy_iphdr_to_lmem()

ppp_decap_classify#:

	// If the packet is IPv4, the IPv4 header will now be in the local
	// memory buffer lmem_ipv4_hdr at offset 0.  First set the value 
	// of the IPv4 default output to the tunnel decap block.


	#ifdef IPV6_IPV4_ONLY_PIPELINE
	#define_eval IPV4_NEXT1 	BID_QM
	#else
	#define_eval IPV4_NEXT1		BID_V6V4_TUNNEL_DECAP
	#endif

	_ipv4_fwder(LM_IPV4_HDR, LM_IPV4_HDR, 0, 0)

	#ifdef IPV6_IPV4_ONLY_PIPELINE

	// Skip the tunneling portion of the pipeline

	#undef	IPV6_NEXT_BLOCK
	#define IPV6_NEXT_BLOCK				BID_QM

	#else

    // If the packet is IPv4, it might be a tunneled IPv6 packet.  The
	// tunnel decapsulate block will remove the IPv4 header if necessary.
	// If decapsulation is required, this block will also call the DL macro
	// dl_load_ipv6_hdr to load the IPv6 header into the local memory buffer
	// lmem_ipv6_hdr.

	v6v4_tunnel_decap(LM_IPV6_HDR, LM_IPV6_HDR, LM_IPV4_HDR, 0, 0, 0)

	#endif

	// If the packet is IPv6 or decapsulated IPv6, the IPv6 header is now
	// in the local memory buffer lmem_ipv6_hdr at offset 0.

	ipv6_forwarder(LM_IPV6_HDR, 0, LM_IPV6_HDR, 0)

	#ifdef IPV6_IPV4_ONLY_PIPELINE

	// Skip the tunneling portion of the pipeline

	#else

	// If the packet is IPv6 or decapsulated IPv6, it might need to be
	// encapsulated (or re-encapsulated) in an IPv4 packet to be forwarded
	// to the next hop.  v6v4_tunnel_encap determines if this needs to
	// occur.  If so, it creates the encapsulating header in the local
	// memory buffer lmem_ipv4_hdr.

	v6v4_tunnel_encap(LM_IPV4_HDR, LM_IPV6_HDR, 0, 0)

    // If the packet was encapsulated it needs to be handled by the IPv4
	// forwarder.  Reset the value of the IPv4 forwarder default output so
	// that the packet will be directed to the next processing stage instead
	// of the tunnel decapsulation block.

	#define_eval IPV4_NEXT1		BID_QM

	_ipv4_fwder(LM_IPV4_HDR, LM_IPV4_HDR, 0, 0)

	#endif

	// Now call DL macros to send the current packet
	// and received the next packet. Depending on the dl_next_block, these
	// macros can drop/send exception/send to next block. Also, we need to
	// maintain packet order when we send/receive packets.

.reg sig_mask
.sig sig_scr_put
.set_sig sig_scr_put

	// allocate xfer registers to be used by dl_qm_sink.
	xbuf_alloc[$wxfer, 3, write]
	immed32[sig_mask, 0]

	// send packet.
	dl_qm_sink[sig_mask, sig_scr_put, $wxfer]

	// receive packet
	dl_source[DL_THREAD_NO_ORDER, sig_mask]

.io_completed sig_scr_put

	// free the xfer registers allocated
	xbuf_free[$wxfer]

	// start again.
	br[loop#]
	
#ifdef	IPV4_START_ME

_handle_exception_pkt#:
	
		dl_get_exception_pkt()

	    br[_handle_exception_pkt#]	

#endif

// We never execute the below code. This is here for completeness only.

term#:

	// frees the IP header cache
	dl_iphdr_cache_fini[]

	// Control should never come here.
kill#:
	nop
	nop

#endif	// __PACKET_PROC_INGRESS_A__UC__