///////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                 
//                                                                      
///////////////////////////////////////////////////////////////////////
//
//
//      File Name: packet_proc_ingress_b.uc
//
//      Purpose: This file contains packet processing microblocks.
//
//      History:
//
//      Date            Comment                         By
//      ---------------------------------------------------------------
//
//      04/25/2003      Created                         asv
//
///////////////////////////////////////////////////////////////////////

#ifndef	__PACKET_PROC_INGRESS_B__UC__
#define	__PACKET_PROC_INGRESS_B__UC__

//	Include Files

// include system.h from dispatch loop
#include	"dl_system.h"

#include	"definitions.h"

// include system_init.h from dispatch loop for inter ME signal
#include	"system_init.uc"

// include dispatch_loop.uc from dispatch loop 
#include	"dispatch_loop.uc"

// include pkthdr_cache.uc from dispatch loop 
#include	"pkthdr_cache_b.uc"

//-----------------------------------------------------------
// Initialize the blocks
//-----------------------------------------------------------

init#:

	dl_pphdr_cache_init()

	; init buffer meta data cache to Zero.
	init_meta_cache_to_zero[]

	; Temporary : Initialize EOP handle default.
	_dl_eop_buf_handle_init[]

	; now kick start the Forwarder
	_fwder_kick_start()

//-----------------------------------------------------------
// dispatch loop
//-----------------------------------------------------------

.begin
.reg sig_mask
	immed32[sig_mask, 0]

	// get packets in order.
	dl_source[DL_THREAD_ORDER, sig_mask]

.end

// Thread 0 of the FWDER_START_ME  from here on
// will process only exception packets. There 
// is enough headroom available and will not affect
// performance

#ifdef	FWDER_START_ME
	br=ctx[0,_handle_exception_pkt#]
#endif

loop#:

	#define_eval DL_SOURCE_NEXT1   BID_QM

	dl_pphdr_set_meta()

.reg sig_mask
.sig sig_scr_put
.set_sig sig_scr_put

	// allocate xfer registers to be used by dl_qm_sink.
	xbuf_alloc[$wxfer, 3, write]
	immed32[sig_mask, 0]

	// send packet.
	dl_qm_sink[sig_mask, sig_scr_put, $wxfer]

	// receive packet
	dl_source[DL_THREAD_NO_ORDER, sig_mask]

.io_completed sig_scr_put

	// free the xfer registers allocated
	xbuf_free[$wxfer]

	// start again.
	br[loop#]

#ifdef	FWDER_START_ME

_handle_exception_pkt#:
	
	dl_get_exception_pkt()

    br[_handle_exception_pkt#]	

#endif

// We never execute the below code. This is here for completeness only.

term#:

	dl_pphdr_fini()

	// Control should never come here.
kill#:
	nop
	nop

#endif	// __PACKET_PROC_INGRESS_B__UC__