/*------------------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//------------------------------------------------------------------------------------
*/

#ifndef __PKTHDR_CACHE_B_UC__
#define __PKTHDR_CACHE_B_UC__

#include "definitions.h"
#include "ppp.h"

///////////////////////////////////////////////////////////////////////////////
// dl_load_pphdr_cache:
//	 	Description: 
//			Reads the initial portion of the packet header from DRAM into 
//			DRAM transfer registers. 
//
//	 	Outputs:
//			The $$pphdr# global dram transfer registers will be set with appropriate
//			header data.
//
//		Inputs:
//			buf_handle	:			Handle of buffer.
//			req_sig		:			Signal to use in the I/O (DRAM read) operation.
//			sig_action	:			What to do with I/O operation - 
//									- wait for signal(s) as specified by sig_action
//									- do not wait for signal, just return (SIG_NONE)
//		Size:   				:
//

#define	dl_load_pphdr_cache		dl_pphdr_load_cache

#macro dl_pphdr_load_cache[buf_handle, req_sig, sig_action]
.begin
	.reg 	sdram_offset, buf_offset

	dl_buf_get_data[sdram_offset, buf_handle]
	dl_meta_get_offset[buf_offset]

	#define_eval	QW_COUNT	(PREPEND_HDR_BYTES >> 2)

	dram[read, $$pphdr[0], sdram_offset, buf_offset, QW_COUNT], sig_done[req_sig]

#if (!streq('sig_action', 'SIG_NONE'))
	ctx_arb[sig_action]
#endif

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_pphdr_cache_init:
//	 	Description: 
//			This macro needs to be called before any of the dl_pphdr_* can be called.
//			It allocates a set of DRAM xfer registers to be used for caching. This macro
//			doesn't generate any code (instructions). It only produces some assembler
//			directives.
//
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//			0 Instructions

#macro dl_pphdr_cache_init[]

	xbuf_alloc[$$pphdr, 4, read]

	.set $$pphdr[0], $$pphdr[1], $$pphdr[2], $$pphdr[3]

#endm

///////////////////////////////////////////////////////////////////////////////
// dl_pphdr_cache_fini:
//	 	Description: 
//			This macro needs to be called at the end when dl_pphdr_macros are no longer
//			required. It frees a set of DRAM xfer registers that were allocated using
//			dl_pphdr_cache_init. This macro	doesn't generate any code (instructions). 
//			It only produces some assembler	directives.
//
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//			0 Instructions

#macro dl_pphdr_fini[]

	xbuf_free[$$pphdr]

#endm


///////////////////////////////////////////////////////////////////////////////
// dl_pphdr_set_meta:
//	 	Description: 
//
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//			0 Instructions

#macro dl_pphdr_set_meta[]
.begin

	.reg fabric_port
	.reg size, offset
	.reg pphdr_0, pphdr_1

	br!=byte[dl_next_block, 0, BID_QM, exit#]

	xbuf_extract(pphdr_0, $$pphdr, 0, 0, 4)
	alu[fabric_port, 0x0f, and, pphdr_0]
	dl_meta_set_fabric_port[fabric_port]

	#define_eval BYTES_ADJUST		(PREPEND_HDR_BYTES)

	dl_meta_get_offset[offset]
	alu[offset, offset, +, BYTES_ADJUST]
	dl_meta_set_offset[offset]

	dl_meta_get_packet_size[size]
	alu[size, size, -, BYTES_ADJUST]
	dl_meta_set_packet_size[size]

	dl_meta_get_buffer_size[size]
	alu[size, size, -, BYTES_ADJUST]
	dl_meta_set_buffer_size[size]

exit#:

.end
#endm

#endif	//__PKTHDR_CACHE_B_UC__