/*===========================================================================
 * = COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 * ===========================================================================
 */

/* =========================================================================*/
/**
 * @file   ix_sa_cc_list.c
 *
 * Definition of CC_INITIALIZATION funciton.
 *
 **/
/*=========================================================================*/
/* $Id: ix_sa_cc_list.c,v 1.4.4.1 2004/03/12 11:22:39 wneubaue Exp $ */
#include "ix_cc_error.h"
#include "sa/ix_sa_cc_list.h"
#include "cc/ix_cc_atmtm41_cci.h"
#include "cc/ix_cc_atmsar_main_cci.h"
#include "cc/ix_cc_atmsartest_cci.h"
#include "cc/ix_cc_atmtx_cci.h"
#include "cc/ix_cc_csix_rx.h"
#include "cc/ix_cc_qm.h"
#include "cc/ix_cc_qm_frmwrk.h"
#include "cc/ix_cc_wred.h"
#include "ix_cc_llc_snap_encap.h"

#ifdef INCLUDE_PDK
#include "cc/ix_cc_fpm_proxy.h"
#endif /* INCLUDE_PDK */

extern int ix_sa_create(ix_uint32 arg_bladeID);
#ifdef IXF_INIT
extern media_init();
extern Egress_Set_PLL();
#endif;
char g_sa_uofFile[256];

void egress_init(ix_int32 bladeId, char* gUofName)
{
#ifdef IXF_INIT
    Egress_Set_PLL();
    media_init();
#endif
    strcpy(g_sa_uofFile,gUofName);
    ix_sa_create(bladeId);
} /* egress_init */

/*  _ix_sa_cc_list */
/*  ---------------------------------------- */
/**
 * This function generates and returns the ix_sa_ccfunc array.
 *
 * An array containing pointers to various cc functions is returned
 * to be used during cc initialization.
 *
 * @return IX_SUCCESS, a pass through ix_error, or an ix_error with one
 *         of the following values.
 * @retval IX_CC_ERROR_NULL arg_pCC_List is NULL
**/
ix_error
_ix_sa_cc_list(ix_sa_ccfunc *arg_pCC_List)
{
    IX_ERROR_CHECK_ARG_NULL(1, arg_pCC_List);

    /* add one entry for each CC in this configuration.  example below.
     *   _ix_sa_set_cc
     *          Param #1: the cclist passed to _ix_sa_cc_list (arg_pCC_List)
     *          Param #2: Identifier of CC as defined in bindings.h
     *          Param #3: The CCs init function (include header in this file)
     *          Param #4: The CCs fini function
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_DUMMY, _ix_sa_dummy_cc_init, _ix_sa_dummy_cc_fini);
    */
    /* QM cell */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_QM, ix_cc_qm_init, ix_cc_qm_fini);

    /* CSIX RX */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_CSIX_RX, ix_cc_csix_rx_init, ix_cc_csix_rx_fini);

    /* ATM RX */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_ATMTX, ix_cc_atmtx_init, ix_cc_atmtx_fini);

    /* SAR CTRL MAIN */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_ATMSAR_MAIN, ix_cc_atmsar_init, ix_cc_atmsar_fini);

    /* SAR CTRL TEST */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_ATMSAR_TEST, ix_cc_atmsartest_init, ix_cc_atmsartest_fini); 

    /* ATMTM41*/
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_ATMTM41, ix_cc_atmtm41_init, ix_cc_atmtm41_fini);

    /* WRED */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_WRED, ix_cc_wred_init, ix_cc_wred_fini);

    /* LLC_SNAP_ENCAP */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_LLC_SNAP_ENCAP, ix_cc_llc_snap_encap_init, ix_cc_llc_snap_encap_fini);

#ifdef INCLUDE_PDK
    /* FPM PROXY CC */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_FPM_PROXY, ix_cc_fpm_proxy_init, ix_cc_fpm_proxy_fini);
#endif /* INCLUDE_PDK */

    return IX_SUCCESS;
}
