///////////////////////////////////////////////////////////////////////
//                                                                   
//                  I N T E L   P R O P R I E T A R Y                
//                                                                   
//     COPYRIGHT (c)  2001-2003 BY  INTEL  CORPORATION.  ALL RIGHTS  
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY   
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A 
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER 
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL, 
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT 
//     THE PRIOR WRITTEN PERMISSION OF :                             
//                                                                   
//                        INTEL  CORPORATION                         
//                                                                   
//                     2200 MISSION COLLEGE BLVD                     
//                                                                   
//               SANTA  CLARA,  CALIFORNIA  95052-8119               
//                                                                   
///////////////////////////////////////////////////////////////////////
//                                                                   
//                                                                   
//      File Name: llc_encap.uc                                         
//                                                                   
//      Description: This is the main LLC/SNAP encapsulation file.
//                   It does the following.
//                   1. Add the 8-byte LLC/SNAP header to start of
//                      packet payload.
//                   2. Update sop buffer meta-data due to 1.
//                   3. Update sop buffer cell count due to 1 and
//                      additional ATM cell consideration for 8-byte
//                      trailer.
//                   4. Compute packet cell count.
//                   5. Update vcq# for the packet if necessary.
//                   6. Optionally fetch outgoing VPI/VCI information
//                      for the packet in the sop buffer meta-data.
//                                                                                                                                 
///////////////////////////////////////////////////////////////////////
#ifndef	_LLC_ENCAP_UC_
#define	_LLC_ENCAP_UC_

//---------------------------------------------------------------------
// Preprocessor definitions.
//---------------------------------------------------------------------

// Scratch ring entry size in lw's.
#define SCR_RING_CACHE_SIZE 	3
#define BID_NEXT_BLOCK          BID_QM
//---------------------------------------------------------------------
// Including needed files.
//---------------------------------------------------------------------

#include <xbuf.uc>
#include <dl_system.h>
#include <dispatch_loop.uc>
#include <llc_encap_util.uc>
#include <wred.uc>



#ifdef USE_IMPORT_VAR
  .import_var QD_SRAM_BASE
#endif
//---------------------------------------------------------------------
// Signal definitions.
//---------------------------------------------------------------------

	; Define some signals.
	.sig scr_put_sig

	.sig meta_rd_sig
	.sig meta_wr_sig

//---------------------------------------------------------------------
// Register definitions.
//---------------------------------------------------------------------

	; Holds the information needed to signal the next thread.
	.reg next_thread

	; Defines the next thread signal.
	.sig volatile prev_thread_done_sig
	.addr prev_thread_done_sig			PREV_THREAD_DONE_SIG_VAL

	.reg self_sig_val
    

	; Defines SCR_RING_CACHE_SIZE gpr's.

	.reg src_sink_data[SCR_RING_CACHE_SIZE]

	; Define signal mask gpr's.
	.reg default_sigmask_a
	.reg sigmask_a
	.reg default_sigmask_b
	.reg sigmask_b

	; Initialize some gpr's that store constants.
	; These gpr's save cycle in the critical path.

	.reg OxFFFF
	move(OxFFFF, 0xFFFF)

	.reg OxE003FFFF
	move(OxE003FFFF, 0xE003FFFF)

    .reg zero
    immed[zero, 0]

	.reg NEXT_HOP_TABLE_BASE_SRAM_GPR

	immed32(NEXT_HOP_TABLE_BASE_SRAM_GPR, NEXT_HOP_TABLE_SRAM_BASE)

	; Allocate s-xfer and d-xfer registers.

	; Allocate a register to increment eop buffer size when additional
	; ATM cell is detected.
	.reg write $eop_buf_size_incr

	;-----------------------------------------------------------
	; One-time initialization phase.
	;-----------------------------------------------------------

    dl_source_init[]

	_llc_encap_init()

    wred_init()

	; signal will be sent to the same context we are now in.
	local_csr_rd[ACTIVE_CTX_STS]
	immed[self_sig_val, 0]
	; current context is held on the lowest 3 bits.
	alu[self_sig_val, self_sig_val, and, 7]


    /* This signal is declared in system_init() macro */
    .sig visible sys_init_signal

	; Wait for system initialization.
	.if(ctx()==0)
		; Thread 0 waits for the ME_INIT_SIGNAL indicating
		; completion of system initialization.
	    signal_first_ctx(WRED_NEXT_THREAD_SIG)
		ctx_arb[sys_init_signal]

    .else
		; Threads 1-7 wait for a prev_thread_done_sig signal.
		ctx_arb[prev_thread_done_sig]
	.endif

   	signal_ctx(next_thread)

	alu[dl_next_block, --, B, IX_NULL]
   	dl_meta_init_cache(zero, zero, zero, zero, zero, zero, zero, zero)
    
    immed32[dl_buf_handle,IX_NULL]
    immed32[dl_eop_buf_handle, IX_NULL]

	#define_eval i	0
	#while(i < SCR_RING_CACHE_SIZE)
    	immed[src_sink_data[i], 0]
	#define_eval i	(i+1)
	#endloop	
	#undef i

    ;-----------------------------------------------------------
	; Recurring processing phase.
	;-----------------------------------------------------------

.while (1)

	; Signals that we could have waited on in _llc_encap_source()
	.io_completed eop_meta_wr_sig

	; Run llc encap.
llc_encap#:

   	; Allocate 2 lw's to read the first qw of a non 8-byte aligned
   	; start of payload
   	; Allocate 4 lw's to write the 8-byte LLC/SNAP header along with
   	; the first qw of the non 8-byte aligned start of payload.
   	xbuf_alloc($$payload, 4, read_write)

	_llc_encap()

	; ctx_arb on sigmask_b
	local_csr_wr[active_ctx_wakeup_events, sigmask_b]

	; Reset sigmask_b to its default value.
	alu[sigmask_b, --, B, default_sigmask_b]
	ctx_arb[--]

    .io_completed dram_wr_sig
    signal_ctx(next_thread)
    xbuf_free($$payload)

    /* setup meta data required by WRED */
    dl_update_diffserv_meta_data()

wred#:    
    wred()

	; Allocate META_CACHE_SIZE lw's to read-modify-write sop buffer
	; meta-data.
	xbuf_alloc($dl_meta, META_CACHE_SIZE, read_write)

	; Allocate SCR_RING_CACHE_SIZE lw's to read from the source
	; scratch ring and write to the sink scratch ring.
	xbuf_alloc($sink, SCR_RING_CACHE_SIZE, write)

	; Write scratch ring cache contents to the sink scratch ring.
	_dl_diffserv_sink(SINK_RING_NO, scr_put_sig, self_sig_val, meta_wr_sig)
    xbuf_free($sink)

	xbuf_alloc($src, SCR_RING_CACHE_SIZE, read)
	; Read the source scratch ring for the next iteration.
	_dl_diffserv_source(SRC_RING_NO, scr_put_sig, meta_wr_sig)

    xbuf_free($src)
	xbuf_free($dl_meta)
.endw

#endif // _LLC_ENCAP_UC_