#ifndef DL_SOURCE_UC
#define DL_SOURCE_UC


/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

// FILE  egress_dl_csix_rx_sink.uc
//



#include <stdmac.uc>
#include <sig_macros.uc>
#include <dl_system.h>


/////////////////////////////
//	Some local defines
/////////////////////////////

#if (DL_RING_NUMBER > 11)
#error			"For Ring Number Greater than 11, we cannot use the SCR_Ring#_Full input state. Check RING_NUMBER"
#else
#define_eval    RING 	DL_RING_NUMBER
#define_eval	DL_RING_FULL			SCR_Ring/**/RING/**/_Full
#undef			RING
#endif

///////////////////////////////////////////////////////////////////////////////
// _dl_pos_sink_init:
//	 	Description: 
//			Initialisation for POS dl_sink. Should be called before using dl_sink.
//			It simply sets a global variable dl_ring to the corresponding ring number
//			This will later save one instruction in dl_sink (for POS).
//
//			[Note: This is an internal macro not meant to be used by the user.
//			This macro will undergo some more changes.]
//
//	 	Outputs:
//			
//		Inputs:
//		
//		Size:   				:
//			1 instruction
//

#macro _dl_pos_sink_init[]
.begin

	alu_shf[dl_ring, --, B, DL_RING_NUMBER, <<2]	; ring number to be used in get/put 

.end
#endm

/////////////////////////////////////////////////////////////////////////////
// dl_qm_sink
//	
//	Description:
//		Send received packet to the qm block by writing scratch ring
//
//	Outputs:	
//		None
//		
//	Inputs:
//		s_hl:			SOP buffer handle register, this register value will be changed
//						after macroc is excuted.  If the caller want to keep this value  
//						the value should be stored some where 
//		e_e_hl:			Encoded EOP buffer handle: NULL for one buffer cframe
//						otherwise will be real EOP handle  
//		cls_n:			Class number
//		o_n:			Output port number
//		p_size:			packet size
// 		flow_color_id	flow and color encoded as flow_id[0..23] and color[24..31]
//		rx_sig:			Signal used to wait receving logid wake up  
//		scr_sig:		Scratch ring write signal
//		rt_lbl:			return label after waked up by receiving logic
//		qm_ring_num:	QM receiving ring number
//
//	CONSTANTS:	
//		None
//	
//
//	Size: 8 instructions
/////////////////////////////////////////////////////////////////////////////

#macro dl_qm_sink[s_hl, e_e_hl, cls_n, o_n, p_size, flow_color_id, rx_sig, scr_sig,\
					 rt_lbl, qm_ring_num]
.begin
.reg tmp_0 tmp_1

	xbuf_alloc[$sink, 3, write]				; alloc $sink
	alu[$sink0, --, B, s_hl]				; set SOP hndl to $sink0
	alu[$sink1, --, B, e_e_hl]				; set EOP hndl to $sink1

	alu[tmp_0, 0xf0, AND, o_n, <<4 ]		; set output port/class num
	alu[tmp_1, 0x0f, AND, class_id]			; set output port/class num
    alu[tmp_0, tmp_0, OR, tmp_1]
	ld_field[tmp_0, 1000, flow_color_id]	; set color ID
	ld_field[tmp_0, 0010, p_size, <<8]

	alu[$sink2, tmp_0, or,1, <<31 ]				; set MSB in $sink1

check_ring_full#:
	; Check if ring is full and loop
 	br_inp_state[DL_RING_FULL, ring_full#]

	scratch[put, $sink0, 0, qm_ring_num, 3],\
			sig_done[scr_sig]								; put to SCR ring
	ctx_arb[rx_sig, scr_sig],and						; ctx swap
	xbuf_free[$sink]										; free $sink
	br[rt_lbl]

ring_full#:
	nop
	br[check_ring_full#]
.end
#endm

#endif	// DL_SOURCE_UC