#ifndef	DL_MEM_MAP_H
#define	DL_MEM_MAP_H

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanpathical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

/* FILE  dl_mem_map.h
 *
 * This file contains memory map customised for IXP2400 hardware/application.
 */


/*******************************************************************************
 *                           INCLUDE SECTION
 *******************************************************************************/
#include <atm_tm_defs.h>
#include <aal5_tx_defs.h>
#include <qm_atm_defs.h>
#include <wred_defs.h>



/*------------------------------------------------------------------------------
 * Hw/os dependent definitions
 */

/* SRAM memory controlers base addresses */
#define	SRAM_MEMORY_CHANNEL_0_BASE		(0x00000000 + 16 * 4)
#define	SRAM_MEMORY_CHANNEL_1_BASE		(0x40000000 + 16 * 4)

/* SDRAM available memory base address */
#define	SDRAM_MEMORY_AVAIL_BASE         0x00000000





/*------------------------------------------------------------------------------
 * SDRAM map
 */

/*
 *	Base Address in DRAM for the the actual packet buffers.
 */
#define	BUF_SDRAM_BASE		SDRAM_MEMORY_AVAIL_BASE


/*------------------------------------------------------------------------------
 * WRED definitions
 */

/*
 * Base address in sram for the wred table
 */
#define WRED_TABLE_RAM_BASE		    (BUF_SDRAM_BASE + BUF_SDRAM_SIZE)
#define WRED_TABLE_RAM_SIZE  		(WRED_ENTRY_SRAM_SIZE * NUMBER_OF_VC)
                                    


/* Start of unused SDRAM memory area */
#define	SDRAM_REST_BASE     (WRED_TABLE_RAM_BASE + WRED_TABLE_RAM_SIZE)



/*------------------------------------------------------------------------------
 * SRAM channel 0 map
 */


/*
 *	Base address in SRAM for the meta data (buffer descriptors).
 */
#define	BUF_SRAM_BASE				SRAM_MEMORY_CHANNEL_0_BASE



/*------------------------------------------------------------------------------
 * QUEUE definitions
 */



/*
 *	Base Address for Queue Descriptors.
 */
#define	QD_SRAM_BASE		(BUF_SRAM_BASE + BUF_SRAM_SIZE)
#define	QD_SRAM_SIZE		(QD_TOTAL * QD_SIZE)


/*------------------------------------------------------------------------------
 * Queue Manager definitions
 */

/* 
 * SRAM channel number
 */
#define BUF_SRAM_BASE_LW 					SRAM_MEMORY_CHANNEL_0_BASE




/*
 * Beginning of channel0 unused area
 */
#define	CHANNEL0_REST_SRAM_BASE		(QD_SRAM_BASE + QD_SRAM_SIZE)




/*---------------------------------------------------------------------------
 * SRAM channel 1 map
 */

/*------------------------------------------------------------------------------
 * CSIX RX definitions
 */

/*
 * Base address CSIX Receive Contexts in SRAM 
 */
#define CSIX_RX_CTX_SRAM_BASE	SRAM_MEMORY_CHANNEL_1_BASE
#define CSIX_RX_CTX_SRAM_SIZE	(CSIX_RX_CTX_SIZE * CSIX_RX_CTX_TOTAL)



/*
 * CSIX_RX needs CSIX_RX_CTX_TOTAL elements CSIX_RX_COUNTER_SIZE each
 */
#define	CSIX_RX_COUNTERS_SRAM_BASE	  (CSIX_RX_CTX_SRAM_BASE + CSIX_RX_CTX_SRAM_SIZE)
#define CSIX_RX_COUNTERS_SRAM_SIZE	  (CSIX_RX_COUNTER_SIZE * CSIX_RX_CTX_TOTAL)

/*------------------------------------------------------------------------------
 * LLC Encap definitions
 */

#define	NEXT_HOP_TABLE_SRAM_BASE 	(CSIX_RX_COUNTERS_SRAM_BASE + CSIX_RX_COUNTERS_SRAM_SIZE)
#define	NEXT_HOP_TABLE_SRAM_SIZE 	(NEXT_HOP_ENTRY_SIZE * NUMBER_OF_NEXT_HOP_ENTRIES)



/* 
 * defines the base address of the table where the most significant 
 * long words of meter statistics are held.
 */
#define WRED_64BIT_STAT_SRAM_BASE			(NEXT_HOP_TABLE_SRAM_BASE + NEXT_HOP_TABLE_SRAM_SIZE)
#define WRED_64BIT_STAT_TABLE_SRAM_SIZE     (WRED_64BIT_STAT_SRAM_SIZE * MAX_ATM_PORTS)


/* DQ rings */
#define	DQ_SRAM_BASE	(WRED_64BIT_STAT_SRAM_BASE + WRED_64BIT_STAT_TABLE_SRAM_SIZE)
#define	DQ_SRAM_SIZE	(DQ_ELEMENT_SIZE * MAX_ATM_PORTS)



/*
 *  Port Shaping table base
 */
#define	PORTSHAPING_SRAM_BASE		(DQ_SRAM_BASE + DQ_SRAM_SIZE) 
#define	PORTSHAPING_SRAM_SIZE		(PORTSHAPING_ENTRY_SIZE * MAX_ATM_PORTS)



#define	PORTINFO_SRAM_BASE			(PORTSHAPING_SRAM_BASE + PORTSHAPING_SRAM_SIZE)
#define	PORTINFO_SRAM_SIZE			(PORTINFO_SRAM_ENTRY_SIZE * MAX_ATM_PORTS)


#define	GCRA_PARAM_SRAM_BASE		(PORTINFO_SRAM_BASE + PORTINFO_SRAM_SIZE)
#define	GCRA_PARAM_SRAM_SIZE		(NUMBER_OF_VC * GCRA_ENTRY_SIZE)


/* base address for the HBR TQ in SRAM */
#define HBR_TQ_SRAM_BASE		(GCRA_PARAM_SRAM_BASE + GCRA_PARAM_SRAM_SIZE)
#define HBR_TQ_TABLE_SIZE		(HBR_TQ_SRAM_SIZE)



/* HBR Indicator longword */
#define	HBR_TQ_SRAM_INDICATOR		(HBR_TQ_SRAM_BASE + HBR_TQ_TABLE_SIZE)

/* Time queue base for ATM TM scheduler */
#define	RTQLEN_SRAM_BASE		(HBR_TQ_SRAM_INDICATOR + HBR_TQ_INDICATOR_SIZE)
#define RTQLEN_TABLE_SIZE		(TQ_TOT * TQLEN_ENTRY_SIZE)

/* Time queue base for ATM TM scheduler */
#define	NRTQLEN_SRAM_BASE		(RTQLEN_SRAM_BASE + RTQLEN_TABLE_SIZE)
#define NRTQLEN_TABLE_SIZE		(TQ_TOT * TQLEN_ENTRY_SIZE)

/* Time queue base for ATM TM scheduler */
#define	UBRTQLEN_SRAM_BASE		(NRTQLEN_SRAM_BASE + NRTQLEN_TABLE_SIZE)
#define UBRTQLEN_TABLE_SIZE		(TQ_TOT * TQLEN_ENTRY_SIZE)

/* Base addresses for queues required in ATM TM */
#define	RTQ_SRAM_BASE			(UBRTQLEN_SRAM_BASE + UBRTQLEN_TABLE_SIZE)
#define	RTQ_TABLE_SIZE			(TQ_SIZE * TQ_TOT)


/* Base address for real-time time queue in SRAM */
#define	NRTQ_SRAM_BASE			(RTQ_SRAM_BASE + RTQ_TABLE_SIZE)
#define	NRTQ_TABLE_SIZE			(TQ_SIZE * TQ_TOT)


/* Base address for ubr time queue in SRAM */
#define	UBRTQ_SRAM_BASE			(NRTQ_SRAM_BASE + NRTQ_TABLE_SIZE)
#define	UBRTQ_TABLE_SIZE		(TQ_SIZE * TQ_TOT)



/*------------------------------------------------------------------------------
 * AAL5 TX definitions
 */
/*	Base address AAL5 Transmit Contexts in SRAM. */
#define	AAL5_TX_CTX_SRAM_BASE		(UBRTQ_SRAM_BASE + UBRTQ_TABLE_SIZE)
#define	AAL5_TX_CTX_SRAM_SIZE		(AAL5_TX_CTX_SIZE * AAL5_TX_CTX_TOTAL)
                                    


/*  Base address for AAL5 TX counters */ 
#define	AAL5_TX_COUNTER_BASE		(AAL5_TX_CTX_SRAM_BASE + AAL5_TX_CTX_SRAM_SIZE)	


#define	AAL5_TX_PKTS_PER_VCQ_COUNTER_SRAM_BASE			AAL5_TX_COUNTER_BASE
#define	AAL5_TX_PKTS_PER_VCQ_COUNTER_TABLE_SIZE		(AAL5_TX_PKTS_PER_VCQ_COUNTER_SIZE * AAL5_TX_CTX_TOTAL)


#define	AAL5_TX_CELLS_PER_VCQ_COUNTER_SRAM_BASE			(AAL5_TX_PKTS_PER_VCQ_COUNTER_SRAM_BASE + AAL5_TX_PKTS_PER_VCQ_COUNTER_TABLE_SIZE)
#define	AAL5_TX_CELLS_PER_VCQ_COUNTER_TABLE_SIZE	(AAL5_TX_CELLS_PER_VCQ_COUNTER_SIZE * AAL5_TX_CTX_TOTAL)

#define	AAL5_TX_PKTS_PER_OUTPORT_COUNTER_SRAM_BASE		(AAL5_TX_CELLS_PER_VCQ_COUNTER_SRAM_BASE + AAL5_TX_CELLS_PER_VCQ_COUNTER_TABLE_SIZE)
#define	AAL5_TX_PKTS_PER_OUTPORT_COUNTER_TABLE_SIZE	(AAL5_TX_PKTS_PER_OUTPORT_COUNTER_SIZE * MAX_ATM_PORTS)

#define	AAL5_TX_CELLS_PER_OUTPORT_COUNTER_SRAM_BASE			(AAL5_TX_PKTS_PER_OUTPORT_COUNTER_SRAM_BASE + AAL5_TX_PKTS_PER_OUTPORT_COUNTER_TABLE_SIZE)
#define	AAL5_TX_CELLS_PER_OUTPORT_COUNTER_TABLE_SIZE	(AAL5_TX_CELLS_PER_OUTPORT_COUNTER_SIZE * MAX_ATM_PORTS)


/*
 * Beginning of channel1 unused area
 */
#define	CHANNEL1_REST_SRAM_BASE		(AAL5_TX_CELLS_PER_OUTPORT_COUNTER_SRAM_BASE + AAL5_TX_CELLS_PER_OUTPORT_COUNTER_TABLE_SIZE)



/*--------------------------------------------------------------------------
 *  SCRATCHPAD map
 */
#define SCRATCHPAD_MEMORY_SIZE      0x4000


/*
 * Scratch ring for communicating between CSIX RX and LLC encap block
 */
#define	CSIX_TO_L2_ENCAP_SCR_RING_BASE	0
#define	CSIX_TO_L2_ENCAP_SCR_RING_SIZE	512


/*
 * Scratch ring for communicating between LLC encap and DiffServ pipeline
 */
#define	L2_ENCAP_TO_DIFFSERV_SCR_RING_BASE	CSIX_TO_L2_ENCAP_SCR_RING_BASE
#define	L2_ENCAP_TO_DIFFSERV_SCR_RING_SIZE	CSIX_TO_L2_ENCAP_SCR_RING_SIZE


/*
 * Scratch ring for communicating between DiffServ pipeline and ATM Queue Manager
 */
#define	DIFFSERV_TO_QM_SCR_RING_BASE	(L2_ENCAP_TO_DIFFSERV_SCR_RING_BASE + 4 * L2_ENCAP_TO_DIFFSERV_SCR_RING_SIZE)
#define	DIFFSERV_TO_QM_SCR_RING_SIZE	512


/*
 * Scratch ring for communicating between Scheduler and ATM Queue Manager Blocks
 */
#define	SCHEDULER_TO_QM_SCR_RING_BASE	(DIFFSERV_TO_QM_SCR_RING_BASE + 4 * DIFFSERV_TO_QM_SCR_RING_SIZE)
#define	SCHEDULER_TO_QM_SCR_RING_SIZE	256


/*
 * First scractch ring for communicating between Queue Manager and ATM TX
 */
#define	QM_TO_AAL5_TX_SCR_RING_0_BASE	(SCHEDULER_TO_QM_SCR_RING_BASE + 4 * SCHEDULER_TO_QM_SCR_RING_SIZE)
#define	QM_TO_AAL5_TX_SCR_RING_0_SIZE	256



#define	QM_TO_AAL5_TX_SCR_RING_1_BASE	(QM_TO_AAL5_TX_SCR_RING_0_BASE + 4 * QM_TO_AAL5_TX_SCR_RING_0_SIZE)
#define	QM_TO_AAL5_TX_SCR_RING_1_SIZE	256


#define	QM_TO_AAL5_TX_SCR_RING_2_BASE	(QM_TO_AAL5_TX_SCR_RING_1_BASE + 4 * QM_TO_AAL5_TX_SCR_RING_1_SIZE)
#define	QM_TO_AAL5_TX_SCR_RING_2_SIZE	256


#define	QM_TO_AAL5_TX_SCR_RING_3_BASE	(QM_TO_AAL5_TX_SCR_RING_2_BASE + 4 * QM_TO_AAL5_TX_SCR_RING_2_SIZE)
#define	QM_TO_AAL5_TX_SCR_RING_3_SIZE	256



/*
 *	Scratchpad for AAL5 Tx data
 */

/* Stores next available TBUF element */
#define	AAL5_TX_AVAIL_TBUF_ELEMENT_SCRATCH_BASE		(QM_TO_AAL5_TX_SCR_RING_3_BASE + 4 * QM_TO_AAL5_TX_SCR_RING_3_SIZE)
#define	AAL5_TX_AVAIL_TBUF_ELEMENT_TABLE_SIZE			(AAL5_TX_PARTITIONS_NUMBER * AAL5_TX_AVAIL_TBUF_ELEMENT_SIZE)

/* Stores cells transmitted per partition */
#define	AAL5_TX_CELLS_TX_SCRATCH_BASE				(AAL5_TX_AVAIL_TBUF_ELEMENT_SCRATCH_BASE + AAL5_TX_AVAIL_TBUF_ELEMENT_TABLE_SIZE)
#define	AAL5_TX_CELLS_TX_TABLE_SIZE					(AAL5_TX_PARTITIONS_NUMBER * AAL5_TX_CELLS_TX_ELEMENT_SIZE)

/*
 *  These defines are used by the individual microcode to identify their 
 *  scratch rings. The reason for these defines is to improve the portability 
 *  of microblocks. User just has to make changes here and the microblocks will
 *  remain untouched. For the naming convention, the microblock name goes first,
 *  then the _RING_ specifier. Following this is the direction indicator. '_OUT'
 *  would imply the microblock is writing to this scratch ring and '_IN' would
 *  imply the microblock is reading from this scratch ring. If there is more 
 *  than one 'IN' or 'OUT' ring the name will have '_<digit>' to 
 *  differentiate the rings.
 */
#define			CSIX_RX_RING_OUT_BASE			CSIX_TO_L2_ENCAP_SCR_RING_BASE
#define			CSIX_RX_RING_OUT_SIZE			CSIX_TO_L2_ENCAP_SCR_RING_SIZE

#define 		L2_ENCAP_RING_IN_0_BASE			CSIX_TO_L2_ENCAP_SCR_RING_BASE
#define 		L2_ENCAP_RING_IN_0_SIZE			CSIX_TO_L2_ENCAP_SCR_RING_SIZE

#define 		L2_ENCAP_RING_OUT_0_BASE		L2_ENCAP_TO_DIFFSERV_SCR_RING_BASE
#define 		L2_ENCAP_RING_OUT_0_SIZE		L2_ENCAP_TO_DIFFSERV_SCR_RING_SIZE

#define			DIFFSERV_RING_IN_0_BASE			L2_ENCAP_TO_DIFFSERV_SCR_RING_BASE
#define			DIFFSERV_RING_IN_0_SIZE			L2_ENCAP_TO_DIFFSERV_SCR_RING_SIZE

#define			DIFFSERV_RING_OUT_0_BASE		DIFFSERV_TO_QM_SCR_RING_BASE
#define			DIFFSERV_RING_OUT_0_SIZE		DIFFSERV_TO_QM_SCR_RING_SIZE

#define			QM_RING_IN_0_BASE				DIFFSERV_TO_QM_SCR_RING_BASE
#define			QM_RING_IN_0_SIZE				DIFFSERV_TO_QM_SCR_RING_SIZE

/* Defined since the QM microblock currently uses this format */
#define			QM_RING_IN_BASE					QM_RING_IN_0_BASE
#define			QM_RING_IN_SIZE					QM_RING_IN_0_SIZE

#define			QM_RING_IN_1_BASE				SCHEDULER_TO_QM_SCR_RING_BASE
#define			QM_RING_IN_1_SIZE				SCHEDULER_TO_QM_SCR_RING_SIZE

#define			QM_RING_OUT_0_BASE				QM_TO_AAL5_TX_SCR_RING_0_BASE
#define			QM_RING_OUT_0_SIZE				QM_TO_AAL5_TX_SCR_RING_0_SIZE

#define			QM_RING_OUT_1_BASE				QM_TO_AAL5_TX_SCR_RING_1_BASE
#define			QM_RING_OUT_1_SIZE				QM_TO_AAL5_TX_SCR_RING_1_SIZE

#define			QM_RING_OUT_2_BASE				QM_TO_AAL5_TX_SCR_RING_2_BASE
#define			QM_RING_OUT_2_SIZE				QM_TO_AAL5_TX_SCR_RING_2_SIZE

#define			QM_RING_OUT_3_BASE				QM_TO_AAL5_TX_SCR_RING_3_BASE
#define			QM_RING_OUT_3_SIZE				QM_TO_AAL5_TX_SCR_RING_3_SIZE

/* Defined since the QM microblock currently uses this format */
#define			QM_RING_OUT_BASE				QM_RING_OUT_0_BASE
#define			QM_RING_OUT_SIZE				QM_RING_OUT_0_SIZE

#define			SCHEDULER_RING_OUT_0_BASE		SCHEDULER_TO_QM_SCR_RING_BASE			
#define			SCHEDULER_RING_OUT_0_SIZE		SCHEDULER_TO_QM_SCR_RING_SIZE
								
#define			AAL5_TX_IN_0_BASE				QM_TO_AAL5_TX_SCR_RING_0_BASE
#define			AAL5_TX_IN_0_SIZE				QM_TO_AAL5_TX_SCR_RING_0_SIZE

#define			AAL5_TX_IN_1_BASE				QM_TO_AAL5_TX_SCR_RING_1_BASE
#define			AAL5_TX_IN_1_SIZE				QM_TO_AAL5_TX_SCR_RING_1_SIZE

#define			AAL5_TX_IN_2_BASE				QM_TO_AAL5_TX_SCR_RING_2_BASE
#define			AAL5_TX_IN_2_SIZE				QM_TO_AAL5_TX_SCR_RING_2_SIZE

#define			AAL5_TX_IN_3_BASE				QM_TO_AAL5_TX_SCR_RING_3_BASE
#define 		AAL5_TX_IN_3_SIZE				QM_TO_AAL5_TX_SCR_RING_3_SIZE


#endif		 /*  DL_MEM_MAP_H  */


