#ifndef DL_DIFFSERV_SOURCE_UC
#define DL_DIFFSERV_SOURCE_UC

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

// FILE  egress_dl_diffserv_source.uc
//


//------------------------------------------------------------------------------
// Included files:

#include <stdmac.uc>
#include <sig_macros.uc>
#include <dl_csix_rx_sink.uc>

//------------------------------------------------------------------------------
// Global definitions

#if (DIFFSERV_RING_OUT_0 > 11)
#error			"For Ring Number Greater than 11, we cannot use the SCR_Ring#_Full input state. Check DIFFSERV_RING_OUT_0"
#else
#define_eval    _RING_NO 	DIFFSERV_RING_OUT_0
#define_eval	DIFFSERV_RING_OUT_0_FULL	SCR_Ring/**/_RING_NO/**/_Status
#undef 			_RING_NO
#endif


//------------------------------------------------------------------------------
// Global signals

.sig dl_source_next_thread_signal
#ifdef DL_SOURCE_NEXT_THREAD_SIG
.addr dl_source_next_thread_signal DL_SOURCE_NEXT_THREAD_SIG
#endif

///////////////////////////////////////////////////////////////////////////////
// dl_source_init:
//	 	Description: 
//
//			Initialisation for dl_source. Should be called before using dl_source.
//			Currently not used/required. Place holder.
//
//	 	Outputs:
//
//		Inputs:
//		
//		Size:   				:
//	
//
#macro dl_source_init[]

    #define SRC_RING_NO (DIFFSERV_RING_IN_0 <<2)
    #define SINK_RING_NO (DIFFSERV_RING_OUT_0 <<2)

#endm


///////////////////////////////////////////////////////////////////////////////
// _dl_eop_buf_handle_init:
//	 	Description: 
//			THIS IS NOT AN EXPORTED MACRO. PLS DO NOT USE IT. THIS WILL CHANGE.
//			THIS IS ONLY TEMPORARY.
//
//	 	Outputs:
//
//		Inputs:
//		
//		Size:   				:
//	
//
#macro _dl_eop_buf_handle_init[]
.begin

	immed32[dl_eop_buf_handle, IX_NULL]

.end
#endm

///////////////////////////////////////////////////////////////////////////////
// dl_sink_init:
//	 	Description: 
//
//			Initialisation for dl_sink. Should be called before using dl_sink.
//			Currently not used/required. Place holder.
//
//	 	Outputs:
//
//		Inputs:
//		
//		Size:   				:
//	
//
#macro dl_sink_init[]


#endm



///////////////////////////////////////////////////////////////////////////////
// _dl_diffserv_source:
//	 	Description: 
//
//
#macro _dl_diffserv_source(in_src_ring, in_scr_put_sig, in_meta_wr_sig)
.begin
	.sig scr_get_sig  ; signal for scratch get
	.sig meta_rd_sig
//    .reg src_ring

dl_source#:
	; Set incoming and outgoing scratch rings.
//	alu[src_ring, --, B, DIFFSERV_RING_IN_0, <<2]
//	alu[sink_ring, --, B, DIFFSERV_RING_OUT_0, <<2]

	; Get 3 Long words from ring 
	scratch[get, $src[0], zero, in_src_ring, 3], sig_done[scr_get_sig]

	ctx_arb[scr_get_sig, in_scr_put_sig, in_meta_wr_sig]

check_ring_empty#:

	//	Check if Ring is empty. ==> it would have returned zero
	alu[--, $src[0], -, 0]; Check for 0. i.e check if ring empty
	beq[ring_empty#]							

	; Copy source scratch ring contents to the scratch ring cache.
	#define_eval i	0
	#while(i < SCR_RING_CACHE_SIZE)
	alu[src_sink_data[i], --, B, $src[i]]
	#define_eval i	(i+1)
	#endloop	
	#undef i
    	
	; LW0 from ring is the current buffer handle (containign SOP)
	move[dl_buf_handle, src_sink_data[0]]
	; LW1 from ring is last buffer handle (for large packets)
	move[dl_eop_buf_handle, src_sink_data[1]]
	; LW2 from ring

	dl_meta_load_cache(dl_buf_handle, $dl_meta, meta_rd_sig, 0, 8)

    ctx_arb[prev_thread_done_sig, meta_rd_sig]

	signal_ctx(next_thread)

	; Copy sop buffer meta-data to the meta-data cache.
	dl_meta_init_cache($dl_meta[0], $dl_meta[1], $dl_meta[2],
		$dl_meta[3], $dl_meta[4], $dl_meta[5], $dl_meta[6], $dl_meta[7])

	move[dl_next_block, DL_SOURCE_NEXT2]
	br[end_source#]

//	Handle Ring empty Condition.
ring_empty#:

.begin

	alu[dl_next_block, --, b, IX_NULL]
	alu[dl_buf_handle, --, b, IX_NULL]
	alu[dl_eop_buf_handle, --, b, IX_NULL]

	ctx_arb[prev_thread_done_sig]

	signal_ctx(next_thread)

	dl_meta_init_cache(zero, zero, zero, zero, zero, zero, zero, zero)

	br[end_source#]

.end

end_source#:

.end
#endm



///////////////////////////////////////////////////////////////////////////////
// _dl_diffserv_sink:
//	 	Description: 
//			Called by IPv4 Block to send the packet to Queue Manager thro' a scratch ring.
//			This macro will do thread ordering always. 
//		
//			This macro will produce the following on to the scratch ring.
//			dl_buf_handle		// The current buffer containing SOP 
//			dl_eop_buf_handle	// for large packets, the buffer containing EOP. Otherwise NULL
//			Queue Number		// Queue number into which this packet is to be queued.
//								// Queue number = 16 * output Fabric id. (available in meta data)
//	 	Outputs:
//			sig_mask			- if the I/O operation is made (scratch put) then the sig_mask
//								  will have the bit corresponding to sig_scr set. (sig_scr is an
//								  input parameter)
//
//		Inputs:
//			sig_scr				- Signal to use in the I/O (scracth put ) operation
//			wxfer				- SRAM write xfer regsiter to be used in I/O.
//								  Should have been allocated using xbuf_alloc. 
//								  Atleast 3 regsiters should be allocated.
//
//			The following Global variables will also act as inputs.
//			dl_next_block		- IX_EXCEPTION, for exception packets
//								- IX_NULL, for null packets.
//								- BID_NEXT_BLOCK, block ID of the next block to handle this packet.
//			dl_buf_handle		: Buffer handle of the buffer in the start of chain (contianinf SOP)
//			dl_buf_eop_handle	: Buffer handle of the last buffer in the chain
//		
//
//
#macro _dl_diffserv_sink(in_sink_ring, in_scr_put_sig, in_self_sig_val, in_meta_wr_sig)
.begin
.reg sig_myself_csr_val

dl_sink#:
	// now check for next block
	br!=byte[dl_next_block, 0, BID_NEXT_BLOCK, check_exception#]  

	; Write sop buffer meta-data cache to sram.

	dl_meta_flush_cache[$dl_meta, dl_buf_handle, in_meta_wr_sig, SIG_NONE, 0, 8]

check_ring_full#:
	; Check if ring is full and loop
 	br_inp_state[DIFFSERV_RING_OUT_0_FULL, ring_is_full#]

	; 3 3 2 2 2 2 2 2 2 2 2 2 1 1 1 1 1 1 1 1 1 1 
	; 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0 9 8 7 6 5 4 3 2 1 0
	;+---------------+---------------+---------------+---------------+
	;|                         SOP buffer handle                     |
	;+---------------+---------------+---------------+---------------+
	;|                         EOP buffer handle                     |
	;+---------------+---------------+---------------+---------------+
	;|V|R|C| cell_count(11bits)  | R |              VCQ              |
	;+---------------------------------------------------------------+
	; V - Valid bit (1)
	; R - reserved
	; C - CLP bit 

	; Copy the scratch ring cache contents into s-xfer-wr registers.
	#define_eval i	0
	#while(i < SCR_RING_CACHE_SIZE)
	alu[$sink[i], --, B, src_sink_data[i]]
	#define_eval i	(i+1)
	#endloop	
	#undef i

	; Produce 3 Long words on the ring
	scratch[put, $sink[0], zero, in_sink_ring, 3], sig_done[in_scr_put_sig]


	// To improve performance, we do not ctx_swap on scratch[put..] above. 
	// We'll wait on this signal in dl_source macro. However this(dl_qm_sink) 
	// macro has multiple paths in which we do not issue this scratch[put..]
	// at all. e.g exception packets, or NULL packets etc. So if we issue
	// scratch[put..], we set the sig_mask to indicate that.

	br[end#]


ring_is_full#:

	nop						; to set break point
	br[check_ring_full#]	; go back and check again


// 	Check for exception Packets. Exception packets are sent to the core (Xscale)
//	on a separate ring. 
check_exception#:

	br=byte[dl_next_block, 0, IX_EXCEPTION, drop_packet#]

	br!=byte[dl_next_block, 0, IX_DROP, check_ix_null#]

	// 	Drop the packet. Two cases to handle. Single buffer or a bufffer chain.
	//	It is determined by looking at the dl_eop_buf_handle

drop_packet#:

	alu[--, dl_eop_buf_handle, -, IX_NULL]	; if dl_eop_buf_handle == IX_NULL
	beq[drop_single_buffer#]				; then it is single buffer.

	.begin
		.reg	queue_number
		.reg 	ring

	check_drop_ring_full#:
		; Check if ring is full and loop
	 	br_inp_state[DIFFSERV_RING_OUT_0_FULL, check_drop_ring_full#]

		// At this point, ring has enough space. Write the data

		move[$sink[0], dl_buf_handle]		; LW0 on the ring is current buffer handle
		move[$sink[1], dl_eop_buf_handle]	; LW1 on the ring is the last buffer in the chain,
											; for big pkts only. Othereise IX_NULL

		immed[queue_number, QM_DROP_QUEUE]				; LW2 on the ring is Queue number for Queue Manager
		alu_shf[$sink[2], queue_number, OR, 1, <<31]		; The msb is always set to 1, to prevent producing a value of 0 on to the ring.
		alu_shf[ring, --, b, DL_DROP_RING, <<2]			; ring nos in get/put is always multiple of 4.

		scratch[put, $sink[0], zero, in_sink_ring, 3], sig_done[in_scr_put_sig]; Produce 3 Long words on the ring

	.end
    
    ; Signal fake-meta-write
    .set_sig in_meta_wr_sig
   	alu_shf[sig_myself_csr_val, in_self_sig_val, OR, &in_meta_wr_sig, <<3]
	local_csr_wr[SAME_ME_SIGNAL, sig_myself_csr_val]

	
	br[end#]



drop_single_buffer#:


	dl_drop_buffer[dl_buf_handle]
    ; Signal scratch-put
    .set_sig in_scr_put_sig
	alu_shf[sig_myself_csr_val, in_self_sig_val, OR, &in_scr_put_sig, <<3]
	local_csr_wr[SAME_ME_SIGNAL, sig_myself_csr_val]
    ; Signal fake-meta-write
    .set_sig in_meta_wr_sig
   	alu_shf[sig_myself_csr_val, in_self_sig_val, OR, &in_meta_wr_sig, <<3]
	local_csr_wr[SAME_ME_SIGNAL, sig_myself_csr_val]

	br[end#]


check_ix_null#:
    .set_sig in_scr_put_sig
	alu_shf[sig_myself_csr_val, in_self_sig_val, OR, &in_scr_put_sig, <<3]
	local_csr_wr[SAME_ME_SIGNAL, sig_myself_csr_val]
    .set_sig in_meta_wr_sig
   	alu_shf[sig_myself_csr_val, in_self_sig_val, OR, &in_meta_wr_sig, <<3]
	local_csr_wr[SAME_ME_SIGNAL, sig_myself_csr_val]
end#:

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_update_diffserv_meta_data:
//	 	Description: 
//			Called to properly setup meta data required by WRED.		
//
//
#macro dl_update_diffserv_meta_data()

    .reg class_id  dl_vcq

    /* WRED abolute number of a queue is calculated as follows */
    /* queue_id = (output_port_id << WRED_MAX_QUEUES) + class_id */
    /* WRED_MAX_QUEUES is set to 0 */

    // Set class to 0 because SoftSAR/DiffServ isn't using it
	alu[class_id, --, B, 0] 
    dl_meta_set_class_id(class_id)
    
    // Output port is set to VCQ
    alu[dl_vcq, OxFFFF, and, src_sink_data[2]]
 	dl_meta_set_output_port(dl_vcq)

#endm
 

#endif	// DL_DIFFSERV_SOURCE_UC
