#ifndef	DL_SYSTEM_H
#define	DL_SYSTEM_H

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

/* FILE  dl_system.h
 *
 * This file contains the system wide parameters that can be
 * customised for a given hardware/application
 */
#ifndef IXP2400
    #ifdef IXP2800
	    #error "Define only IXP2400."
    #endif
#endif

#ifndef   USE_IMPORT_VAR
    #ifndef	SIMULATION
 	    #define	SIMULATION
    #endif
#endif
/*******************************************************************************
 *                           INCLUDE SECTION
 *******************************************************************************/
#include <ix_cc_microengines_bindings.h>

#include <dl_system_config.h>
#include <definitions.h>


#define UBRWPRI


/* Size of the TBUF elements is common for both ATM TXes  */
#define TBUF_ELE_SIZE_LOG2	    6
#define TBUF_ELE_SIZE	        (1<<TBUF_ELE_SIZE_LOG2)

/* TBUF relaed constants */
#define TOTAL_TBUF_SIZE 		8192


/* Size of TCW in bytes */
#define TCW_SIZE_LOG2			3

#define NUM_OF_THREADS_PER_CHANNEL  4


/*
 *	Base address in SRAM for the meta data (buffer descriptors).
 */
#ifdef	USE_IMPORT_VAR 	
	.import_var BUF_SRAM_BASE
#endif

#define 		NUM_OF_BUFFERS		8192


/*------------------------------------------------------------------------------
 * DRAM BUFEERS definitions
 */

/*	Size of packet buffer. */
#define	BUF_SDRAM_SIZE		(BUFFER_SIZE * NUM_OF_BUFFERS)
#define BUF_SRAM_SIZE		(META_DATA_SIZE * NUM_OF_BUFFERS)


/*------------------------------------------------------------------------------
 * DL Source definitions
 */
#define   DL_SOURCE_NEXT_THREAD_SIG		10


/*------------------------------------------------------------------------------
 * ATM SoftSAR config
 */


/* Maximum number of ports in the system */
#define 	TX_MAX_PORTS						4


/* Number of VC's */
#define 	AAL5_TX_NUMBER_OF_VC				NUMBER_OF_VC
#define		TX_MAX_VCS							NUMBER_OF_VC


/*------------------------------------------------------------------------------
 * If ATM_OC48 is defined - It's possible to use HBR at aggregated OC48 else
 * OC24 is used and LBRs can be used.
 */
#ifdef ATM_OC48
 #define			OC48
#else
 #define			OC12
#endif


/*------------------------------------------------------------------------------
 * QUEUE definitions
 */


/*
 *	Size of each Queue Descriptor Entry (in Bytes)
 */

#define		QD_SIZE					(1<<QD_LOG_SRAM_SIZE)

/*------------------------------------------------------------------------------
 * Queue Manager definitions
 */


/*
 * SRAM channel number
 */
#define 	QD_SRAM_CHANNEL_BASE_ADDRESS				SRAM_MEMORY_CHANNEL_0_BASE



#ifndef	USE_IMPORT_VAR
#define   QD_TOTAL			NUMBER_OF_VC
#endif



/*------------------------------------------------------------------------------
 * CSIX RX definitions
 */


/*
 * Base address CSIX Receive Contexts in SRAM
 */


/*
 * Size of each CSIX Receive Context (in Bytes)
 */

#define			CSIX_RX_CTX_SIZE		32

/*
 * Size of each CSIX Receiver Counter (in Bytes)
 */
#define			CSIX_RX_COUNTER_SIZE	16


/*
 * Total number of CSIX Receive Contexts.
 */
#define			CSIX_RX_CTX_TOTAL		1024


/*------------------------------------------------------------------------------
 * LLC Encap definitions
 */


/* 2 ^ PO2_NEXT_HOP_ENTRY_SIZE = NEXT_HOP_ENTRY_SIZE */
#define		PO2_NEXT_HOP_ENTRY_SIZE					4


/* Size of each next hop entry in bytes. */
#define		NEXT_HOP_ENTRY_SIZE						(1 << PO2_NEXT_HOP_ENTRY_SIZE)


/* Number of next hop entries */
#define		NUMBER_OF_NEXT_HOP_ENTRIES				NUMBER_OF_VC



/*------------------------------------------------------------------------------
 * WRED definitions
 */

#define   WRED_DISABLE_FIN_SYNC


#define AAL5_TX_NUMBER_OF_FP_MEs		1

/*
 * signal used to synchronize wred threads on particular microengines
 *
 */
#define	  WRED_NEXT_THREAD_SIG     	    14


/* signal used to synchronize wred threads on particular microengines */

#define	PREV_THREAD_DONE_SIG_VAL		5


/*
 * signal used to synchronize wred crtitical sections on different microengines
 */

#define   WRED_NEXT_ME_SIG				12

/*
 * value used to determine the number of queues per port in the system
 * queue_id = (output_port_id << WRED_MAX_QUEUE) + class_id
 */
#define     WRED_MAX_QUEUE                   0


/*------------------------------------------------------------------------------
 * QM cell definitions
 */

#define NUM_PORT_BITS				2
#define NUM_CLASS_BITS              0

/*------------------------------------------------------------------------------
 * ATM TM 4.1 definitions
 */

/* Traffic Classes to SAR queue map */
#define			CLASS_CBR						RTQUEUE
#define			CLASS_RTVBR						RTQUEUE
#define			CLASS_NRTVBR					NRTQUEUE
#define			CLASS_UBR_PCR					UBRQUEUE
#define			CLASS_UBR_MDCR					UBRQUEUE
#define			CLASS_GFR						NRTQUEUE


/* Aggegation level in a time queue*/
#define 	AGGREGATION_LOG2					3
#define 	AGGREGATION 						(1<<AGGREGATION_LOG2)
#define		AGGREGATION_MASK 					(AGGREGATION - 1)


/* size of PortInfo entry in bytes */
#define 	PORTINFO_SRAM_ENTRY_SIZE			(16 * 4)

/*------------------------------------------------------------------------------
 * AAL5 TX definitions
 */
#define	AAL5_TX_ME01							0x13

#define	CELL_PADDING							0x6A6A6A6A

#define	TX_TBUF_ELE_SIZE 						64
#define	TX_TOTAL_TBUF_SIZE 						8192


/*
 * First 16 (0-15) SRAM Q-Array entries are allocated for the Queue Manager
 */
#define	QM_QARRAY_BASE					0
#define	QM_QARRAY_SIZE					16


#ifndef	USE_IMPORT_VAR

#define	BUF_QARRAY_BASE		16
#define	BUF_QARRAY_SIZE					4
#endif

/*
 *  Next 1 (20) SRAM Q-Array entry is used for Drop Queue in Queue Manager.
 */
#ifndef	USE_IMPORT_VAR

#define	QM_DROP_QUEUE_ENTRY				20
#endif

/*
 *	Queue Number for Drop Queue. 
 *  (0-NUMBER_OF_VC-1 are the normal queues used in forwarding pkts)
 */

#define	QM_DROP_QUEUE					NUMBER_OF_VC

/*------------------------------------------------------------------------------
 * SOURCE & SINK definitions
 */


/*	specify if next neighbor ring is used. If not, default is scratch rings.
 *	NOTE: DL_RING_* is now obsolete. Will be changed to use CSIX_RX_RING_IN, 
 *	which is defined somwehere above.
 */



/*
 *	These tell if a the executing threads need to work in order. i.e wait for
 *	a signal from prev thread, do some work and signal next thread.
 */
#define	DL_THREAD_NO_ORDER				0x0
#define	DL_THREAD_ORDER					0x1


/*	All the IX_* constants should be between 0 and 255 (i.e 1 byte) to 
 *	enable using them as immed operands in instructions.
 *
 *	IX_NULL is used to denote a NULL (or empty) packet handle, It should not
 *  be zero, as it would interfere with ring empty (0) condition.
 *	Note: IX_NULL is too generic a name and may clash with core components code.
 *	So it's marked for deletion.  Start using UC_NULL
 */

#define	IX_NULL				0xFF
#define	IX_EXCEPTION		0x1
#define	IX_DROP				0x2
#define	UC_NULL				0xFF	/* NULL for microcode */
#define	DL_BUF_NULL			UC_NULL /* NULL Buffer representation */


/*
 *	Block IDs for various blocks. Block IDs Start from 0x20.
 *  (0 to 0x1f and 0xFF are reserved for IX_* )
 */
#define BID_WRED            IX_CC_WRED_PKT_MICROBLOCK_ID
#define BID_QM				0x24
#define	BID_LLC_ENCAP		0x25

#define	BID_NULL			UC_NULL	/* NULL Block ID      */
#define	BID_EXCEPTION		0x1		/* Exception Block ID */
#define	BID_DROP			0x2		/* Drop Block ID      */


/*
 * 	Static Bindings (a.k.a targets) for various microblocks. 
 *	Should this go into project/application specific global header file?
 */
#define	DL_SOURCE_NEXT2				BID_LLC_ENCAP
#define	WRED_NEXT1					BID_QM
#define	WRED_NEXT2					IX_DROP
#define	BID_LLC_ENCAP_NEXT1			BID_WRED


/* used to initialize meter & WRED signals */
#define	FIRST_WRED_ME					0x02


/*	BLADE ID  */
#ifndef	USE_IMPORT_VAR
#define	THIS_BLADE_ID					0
#endif

/*
 *	Signals being a precious resource, we use a single signal for 
 *	inter-thread as well as any other inter-me signalling (init over signal)
 */
#define	COMMON_INTER_THD_ME_SIGNAL		10


/*
 *	We are not able to use a single signal before of limitations in assembler.
 *	until then, we need a separate signal for inter-thread signalling.
 */
#define	INTER_THD_SIGNAL				12


/*
 *	ME_NIT_SIGNAL signal is used to let everyone know that initialisation 
 *	is over and all blocks in all MEs can start running. This signal will 
 *	be typically issued by the system initialisation code and all other MEs
 *	should wait for this signal before starting to run.
 */
#define	ME_INIT_SIGNAL					COMMON_INTER_THD_ME_SIGNAL


/*
 * 	Signal number to use in inter thread signalling. 
 */
#define	DL_SIG_WAKE						INTER_THD_SIGNAL

/* free list id that is stored in meta data */
#ifndef	USE_IMPORT_VAR
#define	FREE_LIST_ID               		0x0
#endif

#ifdef	USE_IMPORT_VAR
.import_var DL_REL_BASE
.import_var BUF_FREE_LIST0
.import_var	NEXT_HOP_TABLE_SRAM_BASE
#endif



#define	L2_ENCAP_TO_DIFFSERV_SCR_RING	CSIX_TO_L2_ENCAP_SCR_RING
#define	CSIX_RX_RING_OUT				CSIX_TO_L2_ENCAP_SCR_RING
#define	L2_ENCAP_RING_IN_0				CSIX_TO_L2_ENCAP_SCR_RING
#define	L2_ENCAP_RING_OUT_0				L2_ENCAP_TO_DIFFSERV_SCR_RING
#define	DIFFSERV_RING_IN_0				L2_ENCAP_TO_DIFFSERV_SCR_RING
#define	DIFFSERV_RING_OUT_0				DIFFSERV_TO_QM_SCR_RING
#define	QM_RING_IN_0					DIFFSERV_TO_QM_SCR_RING
#define	QM_RING_IN						QM_RING_IN_0
#define	QM_RING_IN_1					SCHEDULER_TO_QM_SCR_RING
#define	QM_RING_OUT_0					QM_TO_AAL5_TX_SCR_RING_0
#define	QM_RING_OUT_1					QM_TO_AAL5_TX_SCR_RING_1
#define	QM_RING_OUT_2					QM_TO_AAL5_TX_SCR_RING_2
#define	QM_RING_OUT_3					QM_TO_AAL5_TX_SCR_RING_3
#define	QM_RING_OUT						QM_RING_OUT_0
#define	SCHEDULER_RING_OUT_0			SCHEDULER_TO_QM_SCR_RING
#define	AAL5_TX_IN_0					QM_TO_AAL5_TX_SCR_RING_0
#define	AAL5_TX_IN_1					QM_TO_AAL5_TX_SCR_RING_1
#define	AAL5_TX_IN_2					QM_TO_AAL5_TX_SCR_RING_2
#define	AAL5_TX_IN_3					QM_TO_AAL5_TX_SCR_RING_3


/* 	Ring number to use. 0-15. */
#define	DL_RING_NUMBER			CSIX_RX_RING_OUT
#define	DL_DROP_RING			5 /* CSIX_RX_RING_OUT */


#ifndef	USE_IMPORT_VAR
    #ifdef	MICROCODE

        #include "dl_mem_map.h"

    #endif
#endif


#define	SRAM_REST_CHANNEL_NUMBER			0


#endif		 /*  DL_SYSTEM_H  */



