/*===========================================================================
 * = COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000 - 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 * ===========================================================================
 */

/* =========================================================================*/
/**
 * @file   ix_sa_cc_list.c
 *
 * Definition of CC_INITIALIZATION funciton.
 *
 **/
/*=========================================================================*/
/* $Id: ix_sa_cc_list.c,v 1.3.4.1 2004/03/12 11:15:08 wneubaue Exp $ */
#include "ix_cc_error.h"
#include "sa/ix_sa_cc_list.h"

#include "cc/ix_cc_atmsar_agent_cci.h"
#include "cc/ix_cc_atmsartest_cci.h"
#include "ix_cc_csix_tx.h"
#include "cc/ix_cc_atmrx_cci.h"

#include "cc/ix_cc_ipv4.h"
#include "cc/ix_cc_stkdrv.h"
#include "cc/ix_cc_qm_frmwrk.h"
#include "cc/ix_cc_sched.h"

#include "cc/ix_cc_dscp_classifier.h"
#include "cc/ix_cc_classifier_6t.h"
#include "cc/ix_cc_tc_meter.h"

#ifdef INCLUDE_PDK
#include "cc/ix_cc_fpm_core.h"
#endif

extern int ix_sa_create(ix_uint32 arg_bladeID);
#ifdef IXF_INIT
extern Ingress_Set_PLL();
#endif

char g_sa_uofFile[256];

void ingress_init(ix_int32 bladeId, char* gUofName)
{
#ifdef IXF_INIT
    Ingress_Set_PLL();
#endif
    strcpy(g_sa_uofFile, gUofName);
    ix_sa_create(bladeId);
} /* ingress_init */

/*  _ix_sa_cc_list */
/*  ---------------------------------------- */
/**
 * This function generates and returns the ix_sa_ccfunc array.
 *
 * An array containing pointers to various cc functions is returned
 * to be used during cc initialization.
 *
 * @return IX_SUCCESS, a pass through ix_error, or an ix_error with one
 *         of the following values.
 * @retval IX_CC_ERROR_NULL arg_pCC_List is NULL
**/
ix_error
_ix_sa_cc_list(ix_sa_ccfunc *arg_pCC_List)
{
    IX_ERROR_CHECK_ARG_NULL(1, arg_pCC_List);

    /* add one entry for each CC in this configuration.  example below.
     *   _ix_sa_set_cc
     *          Param #1: the cclist passed to _ix_sa_cc_list (arg_pCC_List)
     *          Param #2: Identifier of CC as defined in bindings.h
     *          Param #3: The CCs init function (include header in this file)
     *          Param #4: The CCs fini function
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_DUMMY, _ix_sa_dummy_cc_init, _ix_sa_dummy_cc_fini);
    */
    /* Stack Driver */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_STKDRV, ix_cc_stkdrv_init, ix_cc_stkdrv_fini);

    /* CSIX TX */
    _ix_sa_set_cc(arg_pCC_List,
		  IX_CC_CSIX_TX, ix_cc_csix_tx_init, ix_cc_csix_tx_fini);

   /* Queue Manager*/
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_QM, ix_cc_qm_init, ix_cc_qm_fini);

    /* IPV4 Forwarder */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_IPV4, ix_cc_ipv4_init, ix_cc_ipv4_fini);

    /* Classifier DSCP */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_CLASSIFIER_DSCP, ix_cc_classifier_dscp_init, ix_cc_classifier_dscp_fini);

    /* Classifier 6t */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_CLASSIFIER_6T, ix_cc_classifier_6t_init, ix_cc_classifier_6t_fini);

    /* TC Meter */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_TC_METER, ix_cc_tc_meter_init, ix_cc_tc_meter_fini);

    /* Scheduler */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_SCHEDULER, ix_cc_sched_init, ix_cc_sched_fini);
    /* ATM RX */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_ATMRX, ix_cc_atmrx_init, ix_cc_atmrx_fini);

    /* SAR CTRL MAIN */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_ATMSAR_AGENT, ix_cc_atmsar_agent_init, ix_cc_atmsar_agent_fini);

#ifdef INCLUDE_PDK
    /* FPM CC */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_FPM, ix_cc_fpm_init, ix_cc_fpm_fini);
#endif

    return IX_SUCCESS;
}
