#ifndef INGRESS_DIFFSERV_PIPELINE_UC
#define INGRESS_DIFFSERV_PIPELINE_UC

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

// FILE:  ingress_diffserv_pipeline.uc
//


#define			USE_DL_THREAD_ID

//	Include Files

// include system.h from dispatch loop
#include	"dl_system.h"

#include	<definitions.h>

// include system_init.h from dispatch loop for inter ME signal
#include	"system_init.uc"

// include dispatch_loop.uc from dispatch loop 
#include	"dispatch_loop.uc"

// include ipv4_fwder_dl.uc from IPV4 forwarder 
#include	<ipv4_fwder_init.uc>

// include ipv4_fwder.uc from IPV4 forwarder 
#include	<ipv4_fwder.uc>

// include llc_snap_decap.uc from rx
#include 	<llc_snap_decap.uc>

// include DiffServ definitions

#include    <diffserv.h>

#include    <classifier_6t.uc>
#include	<dscp_classifier.uc>
#include    <dscp_marker.uc>
#include 	<tc_meter.uc>

#ifdef	EXCEPTION_HANDLING_ME
// signal used to read data from CC->ME exception ring
.sig global volatile exc_sig 

 xbuf_alloc[$$exc_data, 3, read]  // data from CC->ME exception ring
#endif 

.reg ip_hdr_offset_reg

//-----------------------------------------------------------
// Initialize the blocks
//-----------------------------------------------------------

init#:
	// allocate cache to store the packet header
	dl_iphdr_cache_init[]
#ifndef	__PKTHDR
#define	__PKTHDR	$$iphdr	; for convenience. DL defines this.
#else
#warning	"__PKTHDR is already defined."
#endif

    alu[ip_hdr_offset_reg, --, B, 0]

	classifier_6t_init()

	tc_meter_init()

	; init buffer meta data cache to Zero.
	init_meta_cache_to_zero[]

	; Temporary : Initialize EOP handle default.
	_dl_eop_buf_handle_init[]

	; ipv4fwder initialization
	_ipv4_fwder_init()

	; now kick start the IPV4 Forwarder
	_ipv4_fwder_kick_start()

	.if (ctx()==7)
	    signal_first_ctx(TC_METER_NEXT_THREAD_SIG)
	.endif

	dl_source_init[]
//-----------------------------------------------------------
// dispatch loop
//-----------------------------------------------------------
.begin
.reg sig_mask
	immed32[sig_mask, 0]

	// get packets in order.
	dl_source[DL_THREAD_ORDER, sig_mask]

.end

loop#:
///////////////////////////////////////////////////////////

    /* 
     * Decapsulation and classification
     */    
	_llc_snap_decap_classify(ip_hdr_offset_reg, __PKTHDR, AAL5_HDR_OFFSET)

    /* 
     * DSCP classifier
     */
    dscp_classifier(__PKTHDR, ip_hdr_offset_reg)

    /* 
     * 6-tuple cassifier
     */
	classifier_6t(__PKTHDR, ip_hdr_offset_reg)

    /* 
     * TCM
     */
    tc_meter()

    ;process the IP header. The modified packet is in cache
	;registers starting at IP_HDR_OFFSET.

	xbuf_alloc(iphdr, PKT_HDR_CACHE_SIZE, read_write)
	.set iphdr[0], iphdr[1], iphdr[2], iphdr[3], iphdr[4], iphdr[5], iphdr[6], iphdr[7], iphdr[8], iphdr[9]

    /* Check IP header offset */
    alu[--, IP_HDR_OFFSET_8, -, ip_hdr_offset_reg]
    bne[not_8B_iphdr_offset#], defer[3]
	// The IPv4 forwarder macro copies only 20 bytes of IP header to the
	// destination register buffer.
	alu[__PKTHDR/**/7, --, B, __PKTHDR/**/7]
	alu[__PKTHDR/**/8, --, B, __PKTHDR/**/8]
	alu[__PKTHDR/**/9, --, B, __PKTHDR/**/9]

    /*
     * DSCP marker
     */
	dscp_marker(iphdr, __PKTHDR, IP_HDR_OFFSET_8, IP_HDR_OFFSET_8)

    /* 
     * IPv4 forwarder 
     */
    _ipv4_fwder(__PKTHDR, iphdr, IP_HDR_OFFSET_8, IP_HDR_OFFSET_8)

    br[sink#]

not_8B_iphdr_offset#:
        
    /*
     * DSCP marker
     */
	dscp_marker(iphdr, __PKTHDR, IP_HDR_OFFSET_0, IP_HDR_OFFSET_0)

    /* 
     * IPv4 forwarder 
     */
    _ipv4_fwder(__PKTHDR, iphdr, IP_HDR_OFFSET_0, IP_HDR_OFFSET_0)

    alu[__PKTHDR/**/5, --, B, __PKTHDR/**/5]
	alu[__PKTHDR/**/6, --, B, __PKTHDR/**/6]

sink#:

	xbuf_free(iphdr)

.reg sig_mask
.sig sig_scr_put
.set_sig sig_scr_put

	// allocate xfer registers to be used by dl_qm_sink.
	xbuf_alloc[$wxfer, 3, write]
	immed32[sig_mask, 0]

	// send packet.
	dl_qm_sink[sig_mask, sig_scr_put, $wxfer, ip_hdr_offset_reg]

	// receive packet
	dl_source[DL_THREAD_NO_ORDER, sig_mask]

.io_completed sig_scr_put

	// free the xfer registers allocated
	xbuf_free[$wxfer]

	// start again.
	br[loop#]
	
// We never execute the below code. This is here for completeness only.

term#:

	// frees the IP header cache
	dl_iphdr_cache_fini[]

	classifier_6t_term[]

    tc_meter_term[]

	// Control should never come here.
kill#:
	nop
	nop

#endif	
