#ifndef	DL_MEM_MAP_H
#define	DL_MEM_MAP_H

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

/* FILE  dl_mem_map.h
 *
 * This file contains memory map customised for IXP2400 hardware/application.
 */

#ifndef MICROCODE
#define MICROCODE
#endif

/*******************************************************************************
 *                           INCLUDE SECTION
 *******************************************************************************/
#include <aal5_rx_defs.h>
#include <classifier_6t_defs.h>
#include <tc_meter_defs.h>

/*------------------------------------------------------------------------------
 * SRAM memory controlers base addresses - extra gap (64B) is for RxC on AAL5Rx
 */
#define	SRAM_MEMORY_CHANNEL_0_BASE		(0x00000000 + 16 * 4)
#define	SRAM_MEMORY_CHANNEL_1_BASE		(0x40000000 + 16 * 4)

#define	SDRAM_MEMORY_BASE			0x00000000 

/*
 *	Base address in SRAM for the meta data (buffer descriptors).
 */
#define	BUF_SRAM_BASE			SRAM_MEMORY_CHANNEL_0_BASE
#define	BUF_CHANNEL_NUMBER		0

/* Meta data is not split */
#define BUF_REST_CHANNEL_NUMBER 	BUF_CHANNEL_NUMBER

/*
 *	Base address in DRAM for the the actual packet buffer.
 */
/* Starts at the begining of the DRAM memory */
#define	BUF_SDRAM_BASE			SDRAM_MEMORY_BASE


/*==============================================================================
 * 							SRAM MEMORY CHANNEL 0
 *
 *------------------------------------------------------------------------------
 */

/*------------------------------------------------------------------------------
 * 6-tuple CLASSIFIER definitions
 */

#define CLASSIFIER_6T_HASH_TABLE_BASE   (BUF_SRAM_BASE + BUF_SRAM_SIZE)

#define CLASSIFIER_6T_HASH_TABLE_SIZE   (CLASSIFIER_6T_HASH_TABLE_ENTRY_SIZE * \
                                        (1 << CLASSIFIER_6T_HASH_INDEX_SIZE))

#ifdef __COUNTERS__
	
	#define CLASSIFIER_6T_STATS_TABLE_SIZE                                   \
                                    (CLASSIFIER_6T_STATISTICS_ENTRY_SIZE *     \
                                    (1 << CLASSIFIER_6T_HASH_INDEX_SIZE))

	#define CLASSIFIER_6T_STATS_SRAM_BASE                                    \
                                    (CLASSIFIER_6T_HASH_TABLE_BASE +           \
				                    CLASSIFIER_6T_HASH_TABLE_SIZE)

	#define CLASSIFIER_6T_DEFAULT_STATS_ADDRESS                              \
                                    (CLASSIFIER_6T_STATS_SRAM_BASE +           \
                                    CLASSIFIER_6T_STATS_TABLE_SIZE)
#endif

/*------------------------------------------------------------------------------
 * TC METER definitions
 */

/* Base address in sram for the meter table  */
#ifdef __COUNTERS__

#define TC_METER_TABLE_SRAM_BASE  	(CLASSIFIER_6T_DEFAULT_STATS_ADDRESS + \
                                    CLASSIFIER_6T_STATISTICS_ENTRY_SIZE)
#else

#define TC_METER_TABLE_SRAM_BASE    (CLASSIFIER_6T_HASH_TABLE_BASE +       \
                                    CLASSIFIER_6T_HASH_TABLE_SIZE)
#endif

#ifdef __COUNTERS__

/* 
 * defines the base address of the table where the most significant 
 * long words of meter statistics are held.
 */
#define TC_METER_64BIT_STAT_SRAM_BASE   (TC_METER_TABLE_SRAM_BASE +  \
	                                    TC_METER_TABLE_SRAM_SIZE)

#endif


/*------------------------------------------------------------------------------
 * DSCP Classifier definitions
 */

#ifdef __COUNTERS__

#define CLASSIFIER_DSCP_CONFIG_SRAM_BASE    (TC_METER_64BIT_STAT_SRAM_BASE + \
                                            TC_METER_64BIT_STAT_TABLE_SRAM_SIZE)
#else

#define CLASSIFIER_DSCP_CONFIG_SRAM_BASE    (TC_METER_TABLE_SRAM_BASE + \
                                            TC_METER_TABLE_SRAM_SIZE)
#endif

#define CLASSIFIER_DSCP_NUMBER_OF_CONFIG_ENTRIES    (CLASSIFIER_DSCP_MAX_NUMBER_OF_PORTS << 6)

#define CLASSIFIER_DSCP_CONFIG_SRAM_SIZE    (CLASSIFIER_DSCP_CONFIG_ENTRY_SIZE * \
							                CLASSIFIER_DSCP_NUMBER_OF_CONFIG_ENTRIES)

#ifdef __COUNTERS__
    
#define CLASSIFIER_DSCP_STATS_SRAM_BASE     (CLASSIFIER_DSCP_CONFIG_SRAM_BASE + \
                                            CLASSIFIER_DSCP_CONFIG_SRAM_SIZE)

#define CLASSIFIER_DSCP_STATS_SRAM_SIZE     (CLASSIFIER_DSCP_STATS_ENTRY_SIZE * \	
    				                        CLASSIFIER_DSCP_NUMBER_OF_CONFIG_ENTRIES)

#endif /* __COUNTERS__ */


/*------------------------------------------------------------------------------
 * QUEUE definitions
 */

#ifdef __COUNTERS__
#define QD_SRAM_BASE    (CLASSIFIER_DSCP_STATS_SRAM_BASE + CLASSIFIER_DSCP_STATS_SRAM_SIZE)
#else
#define QD_SRAM_BASE    (CLASSIFIER_DSCP_CONFIG_SRAM_BASE + CLASSIFIER_DSCP_CONFIG_SRAM_SIZE)
#endif

#define QD_SRAM_CHANNEL_BASE_ADDRESS    QD_SRAM_BASE
#define QD_SRAM_SIZE	                (QD_SIZE * QD_TOTAL)


/*------------------------------------------------------------------------------
 * CSIX TX  definitions
 */
#define CSIX_TX_CTX_SRAM_BASE	      (QD_SRAM_BASE + QD_SRAM_SIZE)
#define CSIX_TX_CTX_SRAM_SIZE	      (CSIX_TX_CTX_SIZE * CSIX_TX_CTX_TOTAL)
#define CSIX_TX_COUNTERS_SRAM_BASE  (CSIX_TX_CTX_SRAM_BASE + CSIX_TX_CTX_SRAM_SIZE)
#define CSIX_TX_COUNTERS_SRAM_SIZE  (NO_OF_CSIX_TX_COUNTERS * CSIX_TX_COUNTERS_SIZE)


/*------------------------------------------------------------------------------
 * IPv4 Forwarder definitions
 */

/* for better DRAM bank utilization, we put the nexthop info in
 * SRAM at this address. We also have a segment in DRAM reserved
 * to keep this info, should we decide to move back to DRAM. This
 * would a compile time option for IPV4 forwarder.
 */
#define	NEXTHOP_TABLE_SRAM_BASE    (CSIX_TX_COUNTERS_SRAM_BASE + CSIX_TX_COUNTERS_SRAM_SIZE)
#define	NEXTHOP_TABLE_SDRAM_BASE   (BUF_SDRAM_BASE + BUF_SDRAM_SIZE)
#define	NEXTHOP_TABLE_SIZE         (NO_OF_NEXTHOP * SIZE_OF_NEXTHOP)
#define	NEXTHOP_TABLE_SRAM_SIZE    NEXTHOP_TABLE_SIZE
#define	TRIE_TABLE_SRAM_BASE       (NEXTHOP_TABLE_SRAM_BASE + NEXTHOP_TABLE_SIZE)
#define	TRIE_TABLE_SIZE            (SIZE_OF_TRIE_SET * NO_OF_TRIE_SETS)
#define	TRIE_TABLE_SRAM_SIZE       (TRIE_TABLE_SIZE + TRIE_RESERVED)
#define DBCAST_TABLE_SRAM_BASE     (TRIE_TABLE_SRAM_BASE + TRIE_TABLE_SRAM_SIZE)
#define DBCAST_TABLE_SIZE          0x2000	
#define	CONTROL_BLOCK_SRAM_BASE    (DBCAST_TABLE_SRAM_BASE + DBCAST_TABLE_SIZE)
#define	CONTROL_BLOCK_SRAM_SIZE    16

/*---------------------------------------------------------------------------
 * AAL5 RX defines	
 */
#define AAL5_RX_PORT_STAT_SRAM_BASE   		(CONTROL_BLOCK_SRAM_BASE + CONTROL_BLOCK_SRAM_SIZE)
#define AAL5_RX_PORT_STAT_TABLE_SIZE	  	(RX_MAX_PORTS * AAL5_RX_PORT_STAT_ENTRY_SIZE)
#define AAL5_RX_VC_INFO_SRAM_BASE	  	(AAL5_RX_PORT_STAT_SRAM_BASE + AAL5_RX_PORT_STAT_TABLE_SIZE)
#define	AAL5_RX_VC_INFO_TABLE_SIZE     		(RX_MAX_VCS * AAL5_RX_VC_INFO_ENTRY_SIZE)
#define	SRAM_ZERO_BLOCK               		(AAL5_RX_VC_INFO_SRAM_BASE + AAL5_RX_VC_INFO_TABLE_SIZE)
#define SRAM_ZERO_BLOCK_SIZE          		8
#define AAL5_RX_VC_HASH_PRI_SRAM_BASE 		(SRAM_ZERO_BLOCK + SRAM_ZERO_BLOCK_SIZE)
#define	AAL5_RX_VC_HASH_PRI_TABLE_SIZE         	(AAL5_RX_VC_HASH_PRI_BKT_ENTRY_SIZE * AAL5_RX_VC_HASH_PRI_SIZE)
#define	AAL5_RX_VC_HASH_SEC_SRAM_BASE 		(AAL5_RX_VC_HASH_PRI_SRAM_BASE + AAL5_RX_VC_HASH_PRI_TABLE_SIZE)
#define	AAL5_RX_VC_HASH_SEC_TABLE_SIZE         	(AAL5_RX_VC_HASH_SEC_BKT_ENTRY_SIZE * AAL5_RX_VC_HASH_SEC_SIZE)
#define	AAL5_RX_VC_HASH_SEC_END               	(AAL5_RX_VC_HASH_SEC_SRAM_BASE + AAL5_RX_VC_HASH_SEC_TABLE_SIZE)
										
/*
 *	Scratch for communicating between AAL5 RX and IPv4 Blocks
 *	Ring number 0, base 0, and size 128 Long Words
 */				
#define			AAL5_RX_TO_IPV4_SCR_RING_BASE	0
#define			AAL5_RX_TO_IPV4_SCR_RING_SIZE	1024


#define			RX_TO_FWD_SCR_RING_SIZE			512
/*
 *	Scratch for communicating between IPV4 and Queue Manager Blocks
 *	Ring number 1, base 512, and size 128 Long Words
 */
#define			IPV4_TO_QM_SCR_RING_BASE		(AAL5_RX_TO_IPV4_SCR_RING_BASE + 4*AAL5_RX_TO_IPV4_SCR_RING_SIZE)
#define			IPV4_TO_QM_SCR_RING_SIZE		512

/*
 *	Scratch for communicating between Scheduler and Queue Manager Blocks
 *	Ring number 2, base 1024, and size 128 Long Words
 */
#define			SCHEDULER_TO_QM_SCR_RING_BASE	(IPV4_TO_QM_SCR_RING_BASE + 4*IPV4_TO_QM_SCR_RING_SIZE)
#define			SCHEDULER_TO_QM_SCR_RING_SIZE	512

/*
 *	Scratch for communicating between Queue Manager and CSIX Transmit Blocks
 *	Ring number 3, base 1536, and size 128 Long Words
 */
#define			QM_TO_CSIX_TX_SCR_RING_BASE		(SCHEDULER_TO_QM_SCR_RING_BASE + 4*SCHEDULER_TO_QM_SCR_RING_SIZE)
#define			QM_TO_CSIX_TX_SCR_RING_SIZE		512


#define         IPV4_STATS_TABLE_BASE           (QM_TO_CSIX_TX_SCR_RING_BASE + 4*QM_TO_CSIX_TX_SCR_RING_SIZE)
#define         IPV4_STATS_TABLE_SIZE           (IPV4_NUMBER_OF_COUNTERS * IPV4_COUNTER_SIZE)


#define			IPV4_RING_IN_BASE				AAL5_RX_TO_IPV4_SCR_RING_BASE
#define			IPV4_RING_IN_SIZE				AAL5_RX_TO_IPV4_SCR_RING_SIZE

#define			IPV4_RING_OUT_BASE				IPV4_TO_QM_SCR_RING_BASE
#define			IPV4_RING_OUT_SIZE				IPV4_TO_QM_SCR_RING_SIZE

#define			QM_RING_IN_BASE					IPV4_TO_QM_SCR_RING_BASE
#define			QM_RING_IN_SIZE					IPV4_TO_QM_SCR_RING_SIZE

#define			QM_RING_OUT_BASE				QM_TO_CSIX_TX_SCR_RING_BASE
#define			QM_RING_OUT_SIZE				QM_TO_CSIX_TX_SCR_RING_SIZE

#define			SCHEDULER_RING_OUT_BASE			SCHEDULER_TO_QM_SCR_RING_BASE			
#define			SCHEDULER_RING_OUT_SIZE			SCHEDULER_TO_QM_SCR_RING_SIZE

#define			CSIX_TX_RING_IN_BASE			QM_TO_CSIX_TX_SCR_RING_BASE
#define			CSIX_TX_RING_IN_SIZE			QM_TO_CSIX_TX_SCR_RING_SIZE



#endif		 /*  DL_MEM_MAP_H  */
