#ifndef DL_SOURCE_UC
#define DL_SOURCE_UC

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

// FILE  ingress_dl_source.uc
//



#include <stdmac.uc>
#include <sig_macros.uc>
#include <dl_system.h>
#include <dl_misc.uc>

/////////////////////////////
//	Some local defines
/////////////////////////////

#if (DL_RING_NUMBER > 11)
#error			"For Ring Number Greater than 11, we cannot use the SCR_Ring#_Full input state. Check RING_NUMBER"
#else
#define_eval     	DL_RING_NUMBER_EVALUATED			    DL_RING_NUMBER	
#define_eval     	DL_RING_FULL			SCR_Ring/**/DL_RING_NUMBER_EVALUATED/**/_Full	
#endif


#if (IPV4_TO_QM_SCR_RING > 11)
#error			"For Ring Number Greater than 11, we cannot use the SCR_Ring#_Full input state. Check IPV4_TO_QM_SCR_RING"
#else
#define_eval     	DL_IPV4_QM_RING_FULL	SCR_Ring/**/IPV4_TO_QM_SCR_RING /**/_Full
#endif

/////////////////////////////
// Global variables/Registers
/////////////////////////////


.reg 	global	dl_ring					// scratch ring number for aal5 rx ==>ipv4. This can be made an absolut register
.reg 	global	dl_buf_handle			// The current Buffer handle.
.reg	global	dl_eop_buf_handle		// For large packets, this is the last buffer in the chain
.reg	global	dl_next_block			// Next block that should process the buffer/packet
.reg    global  dl_exception_reg 		// Exception register used to send pkts to CORE
.set	dl_exception_reg
//	Global Variable for a Prev/Next thread signaling. 

.sig volatile 	sig_prev				
.addr			sig_prev				DL_SIG_WAKE


///////////////////////////////////////////////////////////////////////////////
// _dl_aal5_sink_init:
//	 	Description: 
//			Initialisation for AAL5 dl_sink. Should be called before using dl_sink.
//			It simply sets a global variable dl_ring to the corresponding ring number
//			This will later save one instruction in dl_sink (for AAL5).
//
//			[Note: This is an internal macro not meant to be used by the user.
//			This macro will undergo some more changes.]
//
//	 	Outputs:
//			
//		Inputs:
//		
//		Size:   				:
//			1 instruction
//

#macro _dl_aal5_sink_init[]
.begin

	alu_shf[dl_ring, --, B, DL_RING_NUMBER, <<2]	; ring number to be used in get/put 

.end
#endm

///////////////////////////////////////////////////////////////////////////////
// dl_source_init:
//	 	Description: 
//
//			Initialisation for dl_source. Should be called before using dl_source.
//			Currently not used/required. Place holder.
//
//	 	Outputs:
//
//		Inputs:
//		
//		Size:   				:
//	
//
#macro dl_source_init[]

#ifdef	EXCEPTION_HANDLING_ME
        .reg exc_ring // CC->ME exception ring
        alu[exc_ring, --,b, EXCEPTION_RING_IN_0, <<2]

        // Get data from scratch ring.
    	// We will check data in the next loop
    	scratch[get, $$exc_data[0], 0, exc_ring, 3], sig_done[exc_sig]
        // For now execute fast path forwarding
#endif

#endm


///////////////////////////////////////////////////////////////////////////////
// _dl_eop_buf_handle_init:
//	 	Description: 
//			THIS IS NOT AN EXPORTED MACRO. PLS DO NOT USE IT. THIS WILL CHANGE.
//			THIS IS ONLY TEMPORARY.
//
//	 	Outputs:
//
//		Inputs:
//		
//		Size:   				:
//	
//
#macro _dl_eop_buf_handle_init[]
.begin

	immed32[dl_eop_buf_handle, IX_NULL]

.end
#endm

///////////////////////////////////////////////////////////////////////////////
// dl_sink_init:
//	 	Description: 
//
//			Initialisation for dl_sink. Should be called before using dl_sink.
//			Currently not used/required. Place holder.
//
//	 	Outputs:
//
//		Inputs:
//		
//		Size:   				:
//	
//
#macro dl_sink_init[]


#endm

///////////////////////////////////////////////////////////////////////////////
// dl_source:
//	 	Description: 
//
//			This macro is used by IPv4 Block (for now) to get packets  and other info
//			from the AAL5 RX block. It receives them thro a scratch ring (AAL5_RX_IPV4_RING)
//			The following info is available from the ring.
//			dl_buf_handle		// The current buffer containing SOP 
//			dl_eop_buf_handle	// for large packets, the buffer containing EOP. Otherwise NULL
//			meta1				// Long Word 1 from the Meta data (starting from 0)
//			meta2				// Long Word 2 from the Meta Data
//
//			THREAD_ORDER means if the execution of the threads need to be in
//			order. i.e wait for a signal from prev. thread, do some work and 
//			signal next thread. (This thread ordering is required, if the 
//			threads access some shared data, in order to get exclusive access 
//			to that shared data).
//
//	 	Outputs:
//
//		Inputs:
//			THREAD_ORDER	:	Specifies if the execution of the threads need to be in
//								order. Possible (constant) values are 
//								DL_THREAD_ORDER
//								DL_THREAD_NO_ORDER.
//			
//			sig_mask		:	Signal mask to wait on. This is the bitmask tobe used in a
//								ctx_arb[--] instrcution. (The bitmask contains a set of signals
//								used in previous I/O on which we want to wait here)
//		Size:   			:
//			N/A	
//		
// 		Notes:
//		dl_source is meant to be used along with dl_qm_sink in the following way (only).
//
//		dl_source[DL_THREAD_ORDER, 0]
//		.while (1)
//			do_what_ever[]
//			dl_qm_sink[...]
//			dl_source[DL_THREAD_NO_ORDER, ...]
//		.endw
//
//		dl_qm_sink is always ordered, meaning, it will always wait for a signal from a previous
//		thread, do its job and then signal the next thread. Because dl_qm_sink is always ordered,
//		we don't need the dl_source just following it to be ordered. (but the first time we call
//		dl_source it needs to be ordered).
//
//		Performance optimisation: 
//		-------------------------
//		original code:
//		dl_qm_sink signals the next thread after producing on to the scratch ring, which was
//		at the end of that macro. Then dl_source is called (no thread order). At the very top
//		we had a scratch[get...] ctx_swap[..]. This ctx_swap happened too soon (because the signal
//		we sent to the next thread takes about 10 cycles to reach) so the next thread is still
//		not ready to execute. This lead to large "Idle" time in the pipeline affecting throughput.
//		
//		The following changes were made to improve performance.
//		1. In dl_qm_sink we signal the next thread at the first available oppurtunity, even
//		before the scratch[put...]. This is ok because the next thread (in the same ME) cannot
//		run until we sleep. So make sure there is no ctx_swap between this signal and scratch_put.
//
//		2. If the next thread is another ME, then 1 is a wrong solution. So if we have to
//		signal another ME (because we are in thread 7), we postpone the signalling until 
//		later (in dl_source macro). Thus, for thread 7, dl_source will always signal the 
//		next ME.
//
//		3. soon after we make a scratch[put...] in dl_qm_sink, we make scratch[get...] in 
//		dl_source. So we wait for the combined signals (in dl_source macro) rather wait
//		for wait for each of them separately. (This is the reason for having 3 paramters
//		to dl_qm_sink, and the second paramter to dl_source).
//
//		For ease of understanding, look at dl_qm_sink and dl_source as one BIG combined 
//		macro in the while loop.


#define	RX_TO_FUNC_MSG_SIZE		6		// To get 6 longwords from scratch.

#macro dl_source[THREAD_ORDER, sig_mask]
.begin

.sig scr_get									; signal for scratch get
.reg ring

#ifdef EXCEPTION_HANDLING_ME
	br!=ctx[7, standard_path#]	; Special case for thread 7.

    // Thread 7 checks exception ring from CORE
    .begin
        .reg pkt_proto_type

        // Check if data from exception scratch ring has already been read.
        br_!signal[exc_sig, standard_path#]

        // We check transfer registers in the next loop, so .set must be used 
    	// because of warning (level 3) 4701.
        .set $$exc_data[0]
        .set $$exc_data[1]
        .set $$exc_data[2]

        .reg exc_ring // CC->ME exception ring
        alu[exc_ring, --,b, EXCEPTION_RING_IN_0, <<2]

        alu[--, $$exc_data[0], -, 0]
    	beq[exc_bypass#]

        // Thread 7, So Signal Next ME
    	signal_me[DL_NEXT_ME, DL_SIG_WAKE]

        // Read meta data
    	.sig signal_meta_exc
    	.reg $meta_exc0 $meta_exc1 $meta_exc2 $meta_exc3
    	.reg $meta_exc4 $meta_exc5 $meta_exc6 $meta_exc7
	    .xfer_order $meta_exc0 $meta_exc1 $meta_exc2 $meta_exc3 \
	                $meta_exc4 $meta_exc5 $meta_exc6 $meta_exc7

    	immed[dl_exception_reg, 0]

    	move(dl_buf_handle,     $$exc_data[0])
    	move(dl_eop_buf_handle, $$exc_data[1])
        move(dl_next_block,     $$exc_data[2])

        dl_meta_load_cache($$exc_data[0], $meta_exc, signal_meta_exc, 0, 8)

	    //	Add the signal to the signal mask on which to wait
    	alu_shf[sig_mask, sig_mask, OR, 1, <<(&signal_meta_exc)]
    	local_csr_wr[active_ctx_wakeup_events, sig_mask]
    	ctx_arb[--]

        .io_completed signal_meta_exc

    	dl_meta_init_cache[$meta_exc[0], $meta_exc[1], $meta_exc[2], $meta_exc[3], \
	                       $meta_exc[4], $meta_exc[5], $meta_exc[6], $meta_exc[7]]

    	// Read the packet header and cache it in local memory
        .reg dl_packet_offset
        .sig	sig_ip
    	dl_meta_get_offset(dl_packet_offset)
       	dl_load_iphdr_cache[dl_buf_handle, sig_ip, sig_ip]

        // it is assumed that original packet offset (before decapsulation) is
        // always even multiplication of 8 bytes
        alu[ip_hdr_offset_reg, dl_packet_offset, AND, 0xF]

        // Get data from scratch ring.
    	// We will check data in the next loop
    	scratch[get, $$exc_data[0], 0, exc_ring, 3], sig_done[exc_sig]
            	
        br[nothing_to_do#]

    exc_bypass#:

        // Get data from scratch ring.
    	// We will check data in the next loop
    	scratch[get, $$exc_data[0], 0, exc_ring, 3], sig_done[exc_sig]
        // For now execute fast path forwarding

    .end

#endif /* EXCEPTION_HANDLING_ME */

standard_path#:

	xbuf_alloc[$rdata, RX_TO_FUNC_MSG_SIZE, read]		; allocate xfer register. No Code generated.

#if	(THREAD_ORDER == DL_THREAD_ORDER)

	// Wait for signal from previous thread to ensure thread ordering

	ctx_arb[sig_prev]

#endif
	
	//	Get Data from Scratch Ring.
	//	Instead of Hardcoding the ring (AAL5_RX_TO_IPV4_SCR_RING) we should 
	//	use import variable here. ???

	alu_shf[ring, --, B, AAL5_RX_TO_IPV4_SCR_RING, <<2]		; Ring from which to consume
	scratch[get, $rdata0, 0, ring, RX_TO_FUNC_MSG_SIZE], sig_done[scr_get]	; Get 6 Long words from ring 

	// 	Signal Next Thread. See Notes as why this code is like the way itis shown here.

	br=ctx[7, thread_7#]									; Special case for thread 7.

#if	(THREAD_ORDER == DL_THREAD_ORDER)
	signal_next_ctx[DL_SIG_WAKE]							; !7, so signal the next thread
#endif

	br[wait_for_signals#]

thread_7#:
	signal_me[DL_NEXT_ME, DL_SIG_WAKE]						; Thread 7, So Signal Next ME


	//	We wait for the above scratch[get...] and the scratch[put...] in dl_qm_sink.
	//	See Notes above.

wait_for_signals#:
.reg tmp

	//	Add the signal to the signal mask on which to wait

	alu[tmp, --, B, &scr_get]
	alu[--, tmp, OR, 0]										; tmp will be used  by "indirect" in next instruction
	alu_shf[sig_mask, sig_mask, OR, 1, <<indirect]			; shift 1 by scr_get times

	local_csr_wr[active_ctx_wakeup_events, sig_mask]		; tells which signals to wait on.
	ctx_arb[--]

	//	Assembler doesn't know that we actually wait on the I/O (scratch[get..])
.io_completed scr_get

check_ring_empty#:

	//	Check if Ring is empty. ==> it would have returned zero

	alu[--, $rdata0, -, 0]									; Check for 0. i.e check if ring empty
	beq[ring_empty#]								


	// $rdata2 = dl_meta1
	// $rdata3 = dl_meta2
	// $rdata4 = dl_meta3
	// $rdata5 = dl_meta5

	dl_meta_init_cache[0, $rdata2, $rdata3, $rdata4, 0, $rdata5, 0, 0]
      dl_meta_set_nexthop_id[0xffff]	// set nexthop ID to -1
      immed[dl_exception_reg,0x0]		// clear exception register

	// 	We branch here so that we can take advantage of the defer slots here.
	//	This branch with defer, will finally save a branch to the end of this macro 
	//	(where 2 cycles may be wasted)

	br[load_ip_cache#], defer[2]
	move[dl_buf_handle, $rdata0]				; LW0 from ring is the current buffer handle (containign SOP)
	move[dl_eop_buf_handle, $rdata1]			; LW1 from ring is last buffer handle (for large packets)

	// 	program flow continues in load_ip_cache#:

	//	Handle Ring empty Condition.
ring_empty#:

	//	If ring is empty, just set dl_buf_handle and dl_next_block to IX_NULL;
	// 	Is this the right thing to do ? TBD ???

	move[dl_buf_handle, IX_NULL]				; Note IX_NULL is NOT ZERO.
	move[dl_next_block, IX_NULL]
	br[nothing_to_do#]

load_ip_cache#:
.sig	sig_ip

	// Read the IP header of the packet and cache it in some place

	dl_load_iphdr_cache[dl_buf_handle, sig_ip, sig_ip]
	move[dl_next_block, DL_SOURCE_NEXT1]

nothing_to_do#:

	xbuf_free[$rdata]							; Free xfer registers. No Code generated.


//	The OC48 project spits out 100s of warnings.
//	We'll investigate. until then....

 .set	$$iphdr0, $$iphdr1, $$iphdr2, $$iphdr3, $$iphdr4, $$iphdr5, $$iphdr6, $$iphdr7 , $$iphdr8, $$iphdr9

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_sink:
//	 	Description: 
//			Called by AAL5 RX to handover the packet to the next block (IPv4) in the
//			ingress pipeline. We do not do thread ordering in this macro. (It is done
//			by AAL5 RX itself because of tight budget)
//			
//			This macro will produce the following on to the scratch ring.
//			dl_buf_handle		// The current buffer containing SOP 
//			dl_eop_buf_handle	// for large packets, the buffer containing EOP. Otherwise NULL
//			meta1				// Long Word 1 from the Meta data (starting from 0)
//			meta2				// Long Word 2 from the Meta Data
//
//			This info will be used by the IPV4 block (by calling dl_source)
//
//			Note:
//			sig_mask is a bit mask of signals to wait on. This is passed in a regiter.
//			If there is no need to wait for any signal, pass SIG_NONE. 
//
//	 	Outputs:
//
//		Inputs:
//			wxfer_prefix:		SRAM Write Transfer Register name prefix (using xbuf_alloc).
//			req_sig			:	signal to be used in the I/O (scratch put)
//			sig_mask		:	What to do with I/O operation - 
//								- wait for signal(s) as specified by sig_mask (register)
//								- do not wait for signal, just return (SIG_NONE - constant)
//			The following Global variables will also act as inputs.
//			dl_next_block		- IX_EXCEPTION, for exception packets
//								- BID_NEXT_BLOCK, block ID of the next block to handle this packet.
//			dl_buf_handle		: Buffer handle of the buffer in the start of chain (contianinf SOP)
//			dl_buf_eop_handle	: Buffer handle of the last buffer in the chain
//		Size:   				:
//			N/A
//
#macro dl_sink[wxfer_prefix, req_sig, sig_mask]
.begin

	// We need to act only on EOP. otherwise just return.

	br=byte[dl_next_block, 0, AAL5_RX_NEXT1, sink_it#], defer[3]

	//	Move required data to xfer registers in the defer slot.

		move[wxfer_prefix/**/0, dl_buf_handle]		; dl_buf_handle -> xfer0
		move[wxfer_prefix/**/1, dl_eop_buf_handle]	; dl_eop_buf_handle -> xfer1
		move[wxfer_prefix/**/3, dl_meta[2]]			; LW2 of meta data -> xfer3.

	// If next block is NULL then, nothing to do
	br=byte[dl_next_block, 0, IX_NULL, nothing_to_do#]


    // Check for IX_EXCEPTION and IX_DROP
	
	// Handle IX_DROP by dropping packets.
	
	// In the case of an exception packet, it needs to be sent to the core (Xscale)
	// through a different ring.
	// Currently IX_EXCEPTION is also handled by dropping packets.

drop_pkt#:

	// 	Drop the packet. Two cases to handle. Single buffer or a buffer chain.
	//	It is determined by looking at the dl_eop_buf_handle

	alu[--, dl_eop_buf_handle, -, IX_NULL]					; if dl_eop_buf_handle == IX_NULL
	beq[drop_single_buffer#]								; then it is single buffer.

	// Has a ctx_arb on scratch(drop ring) put to drop the buffer chain
	dl_drop_buffer_chain[dl_buf_handle, dl_eop_buf_handle]	; otherwise, buffer chain.
	br[nothing_to_do#]

drop_single_buffer#:

	dl_drop_buffer[dl_buf_handle]
	br[nothing_to_do#]


sink_it#:

	// Check if the buffer is a NULL buffer. (paranoid check for now).	

	alu_shf[--, --, B, dl_buf_handle, <<8]					; 24 LSBits contain the address
	beq[nothing_to_do#]

	// Ok. We can sink. Check if the ring is full and then produce the data 
	// onto the ring. If ring full, drop the packet and continue.

 	br_inp_state[DL_RING_FULL, drop_pkt#]				; Check if ring is full.

	// Ring Not Full. Put the packet on the ring.

	// put 6 LWs on scratch
	move[wxfer_prefix/**/2, dl_meta[1]]			; buffersize:offset -> xfer2
	move[wxfer_prefix/**/4, dl_meta[3]]			; inport | 0x0 -> xfer4
	move[wxfer_prefix/**/5, dl_meta[5]]			; vc_flowid = vc_key (vpi/vci/input_port) -> xfer5

	scratch[put, wxfer_prefix/**/0, 0, dl_ring, 6], sig_done[req_sig]	; Produce 6 Long words on the ring


#if (!streq('sig_mask', 'SIG_NONE'))
.reg tmp

	//	Add the signal to the signal mask on which to wait

	alu[tmp, --, B, &req_sig]
	alu[--, tmp, OR, 0]									; tmp used  by "indirect" 
	alu_shf[sig_mask, sig_mask, OR, 1, <<indirect]		; shift 1 by req_sig times
#endif


nothing_to_do#:


#ifdef TWO_ME_RX_FP

	br!=ctx[7, NOT_THD_7_1#]

	; Signal next ME with enter_cs3_inter_me_sig.
	alu[--, --, B, lm_cs3_inter_me_sig_val]
	cap[fast_wr, ALU, INTERTHREAD_SIG]	


NOT_THD_7_1#:


#endif  // TWO_ME_RX_FP


#if (!streq('sig_mask', 'SIG_NONE'))
	local_csr_wr[active_ctx_wakeup_events, sig_mask]
	ctx_arb[--]
#endif

	
.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_qm_sink:
//	 	Description: 
//			Called by IPv4 Block to send the packet to Queue Manager thro' a scratch ring.
//			This macro will do thread ordering always. 
//		
//			This macro will produce the following on to the scratch ring.
//			dl_buf_handle		// The current buffer containing SOP 
//			dl_eop_buf_handle	// for large packets, the buffer containing EOP. Otherwise NULL
//			Queue Number		// Queue number into which this packet is to be queued.
//								// Queue number = 16 * output Fabric id. (available in meta data)
//	 	Outputs:
//			sig_mask			- if the I/O operation is made (scratch put) then the sig_mask
//								  will have the bit corresponding to sig_scr set. (sig_scr is an
//								  input parameter)
//
//		Inputs:
//			sig_scr				- Signal to use in the I/O (scracth put ) operation
//			wxfer				- SRAM write xfer regsiter to be used in I/O.
//								  Should have been allocated using xbuf_alloc. 
//								  Atleast 3 regsiters should be allocated.
//          ip_hdr_offset       - IP header offset in wxfer buffer.
//
//			The following Global variables will also act as inputs.
//			dl_next_block		- IX_EXCEPTION, for exception packets
//								- IX_NULL, for null packets.
//								- BID_NEXT_BLOCK, block ID of the next block to handle this packet.
//			dl_buf_handle		: Buffer handle of the buffer in the start of chain (contianinf SOP)
//			dl_buf_eop_handle	: Buffer handle of the last buffer in the chain
//		
//		Size:   				:
//			N/A
//
//		Notes:
//
//		See notes in dl_source macro

#macro dl_qm_sink[sig_mask, sig_scr, wxfer, ip_hdr_offset]
.begin
.sig	sig_meta, sig_ip

	xbuf_alloc[$meta_cache, 8, write]					; This line doesn't produce any code.

	//	This macro can be seen in two parts. The normal packet case (which is the critical
	//	path) and other(drop, null pktetc) cases

	//	First the normal case.

	// compute the cell count

	compute_sop_buf_cframes(dl_buf_handle)

	// now check for next block

	br!=byte[dl_next_block, 0, BID_NEXT_BLOCK, check_exception#]  
	

	//	Write back Meta data. LW0 and LW1 were already written by RX.
	//	So we need to write the rest. (LW2 ...LW7)


//	The 1st LW of the meta data is next buffer pointer and should not be updated
//  when doing dl_meta_flush_cache
#define_eval META_SIZE (META_CACHE_SIZE -1)
	dl_meta_flush_cache[$meta_cache, dl_buf_handle, sig_meta, SIG_NONE, 1, META_SIZE]		; Write back Meta data to SRAM
	dl_flush_iphdr_cache[dl_buf_handle, sig_ip, ip_hdr_offset, SIG_NONE]					; Write back IP hdr to DRAM

	//	Wait for the prev. I/O as well as signal from previous thread.

	ctx_arb[sig_meta, sig_ip, sig_prev]


	// 	Signal Next Thread. Signaling next thread has a latency of 10 cycles. 
	//  So we signal at the first available oppurtunity. (see notes in dl_source macro).
	//	It is IMPORTANT that there should be not ctx_swap/arb from here to scratch[put...]
	//	below.

	br=ctx[7, thread_7#]								; Special case for thread 7.
	signal_next_ctx[DL_SIG_WAKE]						; !7, so signal the next thread

thread_7#:

	// 	If thread 7 we have to signal next ME. But we cannot do that until we have issued
	//	the scratch[put..] command. We'll do it in dl_source macro. (see notes in dl_source).


	// Check if Ring is FUll. If Full, just stall the pipeline. (i.e keep looping)

check_ring_full#:
 	br_inp_state[DL_IPV4_QM_RING_FULL, ring_is_full#]; Check if ring is full and loop

	// At this point, ring has enough space. Sink the data.

	move[wxfer/**/0, dl_buf_handle]						; LW0 on the ring is current buffer hande
	move[wxfer/**/1, dl_eop_buf_handle]					; LW1 on the ring is the last buffer in the chain, 
														; for big pkts only. Othereise IX_NULL
.reg	queue_number

	dl_meta_get_fabric_port[queue_number]				; LW2 on the ring is Queue number for Queue Manager
	alu_shf[queue_number, --, B, queue_number, <<4]		; fabric * 16 is the actual queue number
	alu_shf[wxfer/**/2, queue_number, OR, 1, <<31]		; The msb is always set to 1, to prevent producing a value of 0 on to the ring.

.reg ring

	alu_shf[ring, --, b, IPV4_TO_QM_SCR_RING, <<2]		; ring nos in get/put is always multiple of 4.

	scratch[put, wxfer/**/0, 0, ring, 3], sig_done[sig_scr]	; Produce 3 Long words on the ring

	//	To improve performance, we do not ctx_swap on scratch[put..] above. We'll wait
	//	on this signal in dl_source macro. (see notes in dl_source macro). However this(dl_qm_sink) 
	//	macro has multiple paths in which we do not issue this scratch[put..] at all. 
	//	e.g exception packets, or NULL packets etc. So if we issue scratch[put..], 
	//	we set the sig_mask to indicate that.

.reg tmp

	//	Add the signal to the signal mask on which to wait
	br[end#], defer[1]
	alu_shf[sig_mask, sig_mask, OR, 1, <<(&sig_scr)]

ring_is_full#:
	nop													; to set break point
	br[check_ring_full#]								; go back and check again

	// 	Check for exception Packets. Exception packets are sent to the core (Xscale)
	//	on a separate ring. 

check_exception#:



	br!=byte[dl_next_block, 0, IX_EXCEPTION, check_drop#]

#ifndef DEBUG
	;xbuf_alloc[$meta_cache, 8, write]					; This line doesn't produce any code.

	//	Write back Meta data. LW0 and LW1 were already written by RX.
	//	So we need to write the rest. (LW2 ...LW7)


//	The 1st LW of the meta data is next buffer pointer and should not be updated
//  when doing dl_meta_flush_cache
#define_eval META_SIZE (META_CACHE_SIZE -1)
	dl_meta_flush_cache[$meta_cache, dl_buf_handle, sig_meta, SIG_NONE, 1, META_SIZE]		; Write back Meta data to SRAM    
    dl_flush_iphdr_cache[dl_buf_handle, sig_ip, ip_hdr_offset, SIG_NONE]					; Write back IP hdr to DRAM

	//	Wait for the prev. I/O as well as signal from previous thread.

	ctx_arb[sig_meta, sig_ip, sig_prev]
#else
	;xbuf_free[$meta_cache]								; free xfer register. No Code generated.

	// 	Wait for signal from prev thread and then signal next context.
	
	ctx_arb[sig_prev]
#endif

#ifndef USE_IMPORT_VAR

	// 	Drop the packet. Two cases to handle. Single buffer or a bufffer chain.
	//	It is determined by looking at the dl_eop_buf_handle

	alu[--, dl_eop_buf_handle, -, IX_NULL]					; if dl_eop_buf_handle == IX_NULL
	beq[drop_single_buffer#]								; then it is single buffer.

	dl_drop_buffer_chain[dl_buf_handle, dl_eop_buf_handle]	; otherwise, buffer chain.
	br[signal_next#]

drop_single_buffer#:
	dl_drop_buffer[dl_buf_handle]
	br[signal_next#]

#else /* USE_IMPORT_VAR */

    //This code will send pkts to the CORE on two rings based on priority.
    dl_exception_send(dl_buf_handle)
    br[signal_next#]

#endif /* USE_IMPORT_VAR */

check_drop#:

	// 	Wait for signal from prev thread and then signal next context.
	
	ctx_arb[sig_prev]

	br!=byte[dl_next_block, 0, IX_DROP, check_ix_null#]

	// 	Drop the packet. Two cases to handle. Single buffer or a bufffer chain.
	//	It is determined by looking at the dl_eop_buf_handle

	alu[--, dl_eop_buf_handle, -, IX_NULL]					; if dl_eop_buf_handle == IX_NULL
	beq[drop_single_buffer_1#]								; then it is single buffer.

	dl_drop_buffer_chain[dl_buf_handle, dl_eop_buf_handle]	; otherwise, buffer chain.
	br[signal_next#]

drop_single_buffer_1#:
	dl_drop_buffer[dl_buf_handle]
	br[signal_next#]

check_ix_null#:

signal_next#:

	br=ctx[7, thread_7_1#]

	; For threads 0-6, signal next thread in the same ME.
	signal_next_ctx(DL_SIG_WAKE)

thread_7_1#:

	// Do something to handle null packet. TBD
	nop

end#:
	xbuf_free[$meta_cache]								; free xfer register. No Code generated.

.end
#endm


////////////////////////////////////////////////////////////////////////
// Macro Name  : dl_send_exception
// Description : Macro to write the two longwords of exception
//				 and trigger interrupt to activate RM ISR for 
//				 exception packets
// Input       : in_buffer_handle, in_exception_code
// Constant    : nil
// Size        : 4
// Branches    : 0
////////////////////////////////////////////////////////////////////////
#macro dl_send_exception(in_buffer_handle, in_exception_code, IN_NUM, IN_INTERRUPT)

.begin
.reg $excep_lw0, $excep_lw1, ring
.xfer_order $excep_lw0, $excep_lw1
.sig scratch_wr_done

   // set the ring number

   alu[ring, --,b, IN_NUM, <<2]

   // set the exception longwords

   alu[$excep_lw0, --,b, in_buffer_handle]
   alu[$excep_lw1, --,b, in_exception_code]

   // check if transmit ring is full

   #define_eval RING_NUM IN_NUM

   scr_write#:
	
   br_inp_state[SCR_RING/**/RING_NUM/**/_FULL, scratch_ring_full#]

   // write the two longwords to the exception ring

   scratch[put, $excep_lw0, ring, 0,2], sig_done[scratch_wr_done]
   ctx_arb[scratch_wr_done]

   // trigger the interrupt

   cap[fast_wr, 0, IN_INTERRUPT]

   // take three  cycles for the CSR to be updated
   nop
   nop
   nop
   br[end#]

scratch_ring_full#:
	// 	Drop the packet. Two cases to handle. Single buffer or a bufffer chain.
	//	It is determined by looking at the dl_eop_buf_handle

	alu[--, dl_eop_buf_handle, -, IX_NULL]					; if dl_eop_buf_handle == IX_NULL
	beq[drop_single_buffer_1#]								; then it is single buffer.

	dl_drop_buffer_chain[dl_buf_handle, dl_eop_buf_handle]	; otherwise, buffer chain.
	br[end#]

drop_single_buffer_1#:
	dl_drop_buffer[dl_buf_handle]

end#:
.end

#undef RING_NUM

#endm 

#endif	// DL_SOURCE_UC