#ifndef	DL_SYSTEM_H
#define	DL_SYSTEM_H

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

/* FILE  dl_system.h
 *
 * This file contains the system wide parameters that can be 
 * customised for a given hardware/application
 */
#ifndef IXP2400
    #ifdef IXP2800
	    #error "Define only IXP2400."
    #endif
#endif

#ifndef   USE_IMPORT_VAR
    #ifndef	SIMULATION
	    #define	SIMULATION
    #endif
#endif
/*******************************************************************************
 *                           INCLUDE SECTION
 *******************************************************************************/

#include <ix_cc_microengines_bindings.h>

#include <dl_system_config.h>
#include <definitions.h>

/*******************************************************************************
 *                           SRAM/DRAM SECTION
 *******************************************************************************/
/*------------------------------------------------------------------------------
 * BUFFER related definitions
 */

/*	Amount of SDRAM space available for packet buffers.*/
#define BUF_SDRAM_SIZE					(BUFFER_SIZE * NUMBER_OF_BUFFERS)

/*
 *	Amount of SRAM space available for (meta data of) packet buffers.
 */
#define BUF_SRAM_SIZE					(META_DATA_SIZE * NUMBER_OF_BUFFERS)

#ifdef USE_IMPORT_VAR
.import_var DL_REL_BASE
.import_var BUF_SDRAM_BASE
.import_var	BUF_SRAM_BASE
.import_var	BUF_CHANNEL_NUMBER
#endif


/*------------------------------------------------------------------------------
 * 6-tuple CLASSIFIER definitions
 */

#ifndef USE_IMPORT_VAR

/*#define CLASSIFIER_6T_HASH_BY_CRC*/

#define	CLASSIFIER_6T_DEFAULT_FLOW_ID			0x1
#define	CLASSIFIER_6T_DEFAULT_COLOR_ID			0x1
#define	CLASSIFIER_6T_DEFAULT_CLASS_ID			0x00
#define	CLASSIFIER_6T_DEFAULT_NEXTHOP_ID		0xFFFF
#define CLASSIFIER_6T_DEFAULT_NEXTHOP_ID_TYPE	0x00

#endif

/*------------------------------------------------------------------------------
 * TC METER definitions
 */

/*
 * signal used to synchronize meter threads on particular microengines
 */
#define TC_METER_NEXT_THREAD_SIG        13

/*
 * signal used to initialize timestamp on every timestamp
 */
#define TC_METER_TIMESTAMP_SIG          11

/*
 * signal used to synchronize meter crtitical sections 
 * on different microengines
 */
#define TC_METER_NEXT_ME_SIG            15

/* used to initialize TC_METER meter signals */
#define FIRST_TC_METER_ME			    0x02


/*------------------------------------------------------------------------------
 * DSCP Marker definitions
 */

#define DSCP_MARKER_IPV4_PACKET 		PKT_PROTO_IPv4
#define DSCP_MARKER_IPV6_PACKET 		PKT_PROTO_IPv6
#define DSCP_MARKER_MPLS_PACKET 		0x03

/*------------------------------------------------------------------------------
 * DSCP Classifier definitions
 */
#define     CLASSIFIER_DSCP_IPV4_HEADER_TYPE 		PKT_PROTO_IPv4
#define     CLASSIFIER_DSCP_IPV6_HEADER_TYPE 		PKT_PROTO_IPv6


/*------------------------------------------------------------------------------
 * QUEUE definitions
 */

/* Queue descriptor size for Downstream, because of TIMESTAMPS */
#define QD_LOG_SRAM_SIZE                5

/*
 *	Total number of Queue Descriptors
 */
#ifndef USE_IMPORT_VAR
#define QD_TOTAL            RX_MAX_VCS
#endif
/*
 *	Size of each Queue Descriptor Entry (in Bytes)
 */
#define	QD_SIZE				(1<<QD_LOG_SRAM_SIZE)

#define QM_DROP_QUEUE       RX_MAX_VCS

/*
* IPv4 Forwarder definitions
 */

/* 
/*---------------------------------------------------------------------------
 * AAL5 RX defines	
 */
#ifdef USE_IMPORT_VAR
    .import_var BUF_FREE_LIST0
    .import_var FREE_LIST_ID
#else
    #define	FREE_LIST_ID                0x0
    /* To create a free list for secondary hash table  */
    #define BUF_FREE_LIST1		(BUF_FREE_LIST0+1)
#endif

/* PHY mode */
#define	RX_PHY_MODE    				SPHY_4_8

/* RBUF relaed constants */
#define TOTAL_RBUF_SIZE         8192

/* Size of the RBUF elements is common for both ATM TXes  */
#define RBUF_ELE_SIZE_LOG2	    6
#define RBUF_ELE_SIZE	        (1<<RBUF_ELE_SIZE_LOG2)

/* CLP bit location in meta4 */
#define CLP_BIT_OFFSET_IN_DL_META4	3


/*
 *  These defines are used by the individual microcode to identify their 
 *  scratch rings. The reason for these defines is to improve the portability 
 *  of microblocks. User just has to make changes here and the microblocks will
 *  remain untouched. For the naming convention, the microblock name goes first,
 *  then the _RING_ specifier. Following this is the direction indicator. '_OUT'
 *  would imply the microblock is writing to this scratch ring and '_IN' would
 *  imply the microblock is reading from this scratch ring. If there is more 
 *  than one 'IN' or 'OUT' ring the name will have '_<digit>' to 
 *  differentiate the rings.
 */


/*
 *	The Ring on which AAL5 RX sends out msg to next block
 */
#define 		AAL5_RX_RING_OUT				AAL5_RX_TO_IPV4_SCR_RING

/*
 *	The input ring ring on which IPV4 receives msgs from prev block
 */
#define			IPV4_RING_IN					AAL5_RX_TO_IPV4_SCR_RING

/*
 * The ring on which IPV4 sends out msg to next block
 * (Queue Manager in this case)
 */
#define			IPV4_RING_OUT					IPV4_TO_QM_SCR_RING

/*
 * The input ring on which QM receives msgs from prev. block
 */
#define			QM_RING_IN						IPV4_TO_QM_SCR_RING

/*
 * The output ring in which QM Sends msgs to next block 
 * (CSIX TX in this case)
 */
#define			QM_RING_OUT						QM_TO_CSIX_TX_SCR_RING


#define			SCHEDULER_RING_OUT				SCHEDULER_TO_QM_SCR_RING

/*
 * The input ring in which CSIX TX receives msgs from QM
 */
#define			CSIX_TX_RING_IN					QM_TO_CSIX_TX_SCR_RING




/*	
 * First 16 (0-15) SRAM Q-Array entries are allocated for the Queue Manager
 */
#define			QM_QARRAY_BASE					0
#define			QM_QARRAY_SIZE					16

/*
 *	Next 4 (16-19) SRAM Q-Array entries are allocated for buffer free list
 */
#define			BUF_QARRAY_BASE					16
#define			BUF_QARRAY_SIZE					4

/*	
 *  Next 1 (20) SRAM Q-Array entry is used for Drop Queue in Queue Manager.
 */
#ifndef USE_IMPORT_VAR
#define			QM_DROP_QUEUE_ENTRY				20
#endif

/*
 * QD_SIZE = 2 ^ QD_LOG_SIZE
 */
#define			QD_LOG_SIZE         5 


/*------------------------------------------------------------------------------
 * SOURCE & SINK definitions
 */

/*	specify if next neighbor ring is used. If not, default is scratch rings.
 *	NOTE: DL_RING_* is now obsolete. Will be changed to use RX_RING_IN, 
 *	which is defined somwehere above.
 */

/*
 *	Valid values for RING_SIZE are 128, 256, 512 or 1024.
 */
#define 	DL_RING_SIZE			AAL5_RX_TO_IPV4_SCR_RING_SIZE

/*
 * 	Base Address for Scratch Ring is 0x1000.
 *  Should be aligned on a 4 x RING_SIZE byte boundary.
 */
#define 	DL_RING_BASE			AAL5_RX_TO_IPV4_SCR_RING_BASE

/* 	Ring number to use. 0-15. */
#define		DL_RING_NUMBER			AAL5_RX_TO_IPV4_SCR_RING

#ifndef MICROCODE
#define	DL_DROP_RING			IPV4_TO_QM_SCR_RING
#else

#ifdef CORE_CODE
#define DL_DROP_RING			IPV4_TO_QM_SCR_RING
#else
#define_eval DL_DROP_RING		IPV4_TO_QM_SCR_RING
#endif

#endif


/*
 *	These tell if a the executing threads need to work in order. i.e wait for
 *	a signal from prev thread, do some work and signal next thread.
 */
#define			DL_THREAD_NO_ORDER				0x0
#define			DL_THREAD_ORDER					0x1




/*
 *	All the IX_* constants should be between 0 and 255 (i.e 1 byte) to 
 *	enable using them as immed operands in instructions.
 *
 *	IX_NULL is used to denote a NULL (or empty) packet handle, It should not
 *  be zero, as it would interfere with ring empty (0) condition.
 *	Note: IX_NULL is too generic a name and may clash with core components code.
 *	So it's marked for deletion.  Start using UC_NULL
 */
#define			IX_NULL				0xFF
#define			IX_EXCEPTION		0x1
#define			IX_DROP				0x2
#define			UC_NULL				0xFF	/* NULL for microcode */
#define			DL_BUF_NULL			UC_NULL /* NULL Buffer representation */

/*
 *	Block IDs for various blocks. Block IDs Start from 0x20.
 *  (0 to 0x1f and 0xFF are reserved for IX_* )
 */
#define			BID_LLC_SNAP_DECAP	0x20
#define         BID_CLASSIFIER_6T	IX_CC_CLASSIFIER_6T_PKT_MICROBLOCK_ID
#define         BID_IPV4			IX_CC_IPV4_PKT_MICROBLOCK_ID
#define         BID_TC_METER        IX_CC_TC_METER_PKT_MICROBLOCK_ID
#define         BID_DSCP_MARKER     IX_CC_DSCP_MARKER_PKT_MICROBLOCK_ID
#define			BID_QM				IX_CC_QM_CORE_TO_MICROBLOCK_ID
#define		    BID_CLASSIFIER_DSCP IX_CC_CLASSIFIER_DSCP_PKT_MICROBLOCK_ID

#define			BID_NULL			UC_NULL	/* NULL Block ID      */
#define			BID_EXCEPTION		0x1		/* Exception Block ID */
#define			BID_DROP			0x2		/* Drop Block ID      */

/*
 * 	Static Bindings (a.k.a targets) for various microblocks. 
 *	Should this go into project/application specific global header file?
 */
#define			AAL5_RX_NEXT1						BID_LLC_SNAP_DECAP
#define			DL_SOURCE_NEXT1						BID_LLC_SNAP_DECAP

#define		    CLASSIFIER_6T_INV_IP_NEXT_BLOCK     IX_EXCEPTION
#define		    CLASSIFIER_6T_DEFAULT_NEXT_BLOCK	BID_IPV4

#define			IPV4_NEXT1							BID_QM
#define         IPV4_LOCAL_NEXT1                    IX_EXCEPTION


#define		    TC_METER_NEXT1				        BID_DSCP_MARKER
#define		    TC_METER_NEXT2				        BID_IPV4
#define		    TC_METER_NEXT3				        BID_DROP

#define		    CLASSIFIER_DSCP_METER_OUTPUT		BID_TC_METER
#define		    CLASSIFIER_DSCP_MARKER_OUTPUT		BID_DSCP_MARKER
#define		    CLASSIFIER_DSCP_IP4_FWD_OUTPUT		BID_IPV4
#define		    CLASSIFIER_DSCP_IP6_FWD_OUTPUT		BID_DROP
#define		    CLASSIFIER_DSCP_INV_IP_OUTPUT		BID_DROP
#define		    CLASSIFIER_DSCP_IP4_DEFAULT_OUTPUT	BID_CLASSIFIER_6T
#define		    CLASSIFIER_DSCP_IP6_DEFAULT_OUTPUT	BID_DROP

#define			DSCP_MARKER_NEXT1					BID_IPV4

#define	        TARGET_LLCSNAP_IPV4                 BID_CLASSIFIER_DSCP
#define	        TARGET_LLCSNAP_IPV6                 BID_DROP
#define	        TARGET_LLCSNAP_ARP                  BID_DROP
#define	        TARGET_LLCSNAP_PPP_DISCOVERY        BID_DROP
#define	        TARGET_LLCSNAP_PPP_SESSION          BID_DROP
#define	        TARGET_LLCSNAP_MPLS_UNICAST         BID_DROP
#define	        TARGET_LLCSNAP_ETHER                BID_DROP

/*	BLADE ID  */
#ifndef USE_IMPORT_VAR
#define			THIS_BLADE_ID					2
#endif

/*
 *	Signals being a precious resource, we use a single signal for 
 *	inter-thread as well as any other inter-me signalling (init over signal)
 */
#define			COMMON_INTER_THD_ME_SIGNAL		10

/*
 *	We are not able to use a single signal before of limitations in assembler.
 *	until then, we need a separate signal for inter-thread signalling.
 */
#define			INTER_THD_SIGNAL				12

/*
 *	ME_NIT_SIGNAL signal is used to let everyone know that initialisation 
 *	is over and all blocks in all MEs can start running. This signal will 
 *	be typically issued by the system initialisation code and all other MEs
 *	should wait for this signal before starting to run.
 */
#define			ME_INIT_SIGNAL					COMMON_INTER_THD_ME_SIGNAL

/*
 * 	Signal number to use in inter thread signalling. 
 */
#define 		DL_SIG_WAKE						INTER_THD_SIGNAL
 
/*
 * The DDR (Double Data Rate DRAM) bank size is used to randomize buffer offset.
 */
#define			DDR_BANK_SIZE					128

#define	  		META_CACHE_SIZE					7


/*
 * Number of DDR banks
 */
#define			NO_OF_DDR_BANKS					4

#define			MAX_BUFFER_OFFSET				(DDR_BANK_SIZE*NO_OF_DDR_BANKS)

/*---------------------------------------------------------------------
 * 7) Scratch and scratch ring memory map.
 *-------------------------------------------------------------------*/

/*  There are two exception ring to send pkts up */
#ifndef		EXCEPTION_RING_OUT_0
#define		EXCEPTION_RING_OUT_0			0
#endif

#ifndef		EXCEPTION_RING_OUT_1
#define		EXCEPTION_RING_OUT_1			1
#endif

/*  There are two exception ring to send pkts down */
#ifndef		EXCEPTION_RING_IN_0
#define		EXCEPTION_RING_IN_0				2
#endif

#ifndef		EXCEPTION_RING_IN_1
#define		EXCEPTION_RING_IN_1				3
#endif

/* In the order of D_BASE, D_SIZE, S_BASE, S_SIZE  */
#define	VC_HASH_SEC_TABLE_POOL		0, 0, VC_HASH_SEC_BASE_SRAM_LW, AAL5_RX_VC_HASH_SEC_BKT_ENTRY_SIZE



#define IPV4_NUMBER_OF_COUNTERS     9
#define IPV4_COUNTER_SIZE           4



#define			RX_TO_FWD_MSG_SIZE				8		// 8 longwords


#ifndef USE_IMPORT_VAR
#ifdef MICROCODE
#include <dl_mem_map.h>
#endif
#endif

#endif
/*  DL_SYSTEM_H  */

