#ifndef	DL_SYSTEM_CONFIG_H
#define	DL_SYSTEM_CONFIG_H

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

/* FILE  dl_system_config.h
 *
 * This file contains general system configuration parameters
 *
 */

/* BUFFERS RELATED DEFINITIONS */
/*------------------------------------------------------------------------------
 * Number of buffers
 */
#define	NUMBER_OF_BUFFERS			2048


/*------------------------------------------------------------------------------
 * DRAM BUFEERS definitions
 */

/*
 *	Size of Meta Data (Buffer Descriptors) in SRAM. 32 entries per buffer.
 * 	Should be a power of 2.
 */
#define	META_DATA_SIZE					32

/*	Size of packet buffer. */
#define	BUFFER_SIZE					2048

/* High Level system Configuration parameters */

/* VC configuration */
#define RX_MAX_VCS		8192

#define NUMBER_OF_VC    RX_MAX_VCS

/* Port configuration */
#define RX_MAX_PORTS	4
#define MAX_ATM_PORTS   RX_MAX_PORTS

/*-----------------------------------------------------------------------------
 * AAL5 RX defines
 */
#define NO_PER_PORT_STAT_COUNTERS	    8
#define ATMRX_VC_STAT_COUNTERS_NO       5
#define ATMRX_VC_STAT_COUNTERS_FIRST_LW 10

#define AAL5_RX_VC_HASH_PRI_SIZE	(RX_MAX_VCS )
#define AAL5_RX_VC_HASH_SEC_SIZE	(AAL5_RX_VC_HASH_PRI_SIZE>>2)

#ifndef USE_IMPORT_VAR
    #define AAL5_RX_VC_HASH_PRI_INDEX_MASK      (AAL5_RX_VC_HASH_PRI_SIZE - 1)
#endif

/* This is a pattern used to fill the VCs */
#define AAL5_RX_VC_HASH_FILLING 0x00000000

/*------------------------------------------------------------------------------
 * 6-tuple CLASSIFIER definitions
 */

#define     CLASSIFIER_6T_SRAM_HASH_TABLE

#ifdef __COUNTERS__
	#define   CLASSIFIER_6T_PACKET_COUNTER_FEATURE
#endif

/*
 * bit size of Hash Table index (used to mask 
 * out result of hash operation)
 */
#define     CLASSIFIER_6T_HASH_INDEX_SIZE           8

/*------------------------------------------------------------------------------
 * DSCP Classifier definitions
 */
#define CLASSIFIER_DSCP_CONFIG_ENTRY_SIZE           0x8
#define CLASSIFIER_DSCP_MAX_NUMBER_OF_PORTS         256
#define CLASSIFIER_DSCP_STATS_ENTRY_SIZE            0x10

/*------------------------------------------------------------------------------
 * TC METER definitions
 */

#define TC_METER_TRTCM_FEATURE
#define TC_METER_SRTCM_FEATURE

#ifdef __COUNTERS__
#define TC_METER_PACKET_COUNTER_FEATURE
#endif

/*
 * Amount of SDRAM space taken by meter table
 * in simulation we use 255 entries
 */
#define   TC_METER_TABLE_SRAM_SIZE  		   (TC_METER_ENTRY_SRAM_SIZE * 0xFF)
/*
 * defines the SDRAM space taken by meter's 64 bit supplementary statistics
 */
#define   TC_METER_64BIT_STAT_TABLE_SRAM_SIZE  (0xFF * \
                                                  TC_METER_64BIT_STAT_SRAM_SIZE)

/*------------------------------------------------------------------------------
 * IPv4 Forwarder definitions
 */
/* 2 ^ PO2_NEXT_HOP_ENTRY_SIZE = NEXT_HOP_ENTRY_SIZE */
#define	PO2_NEXT_HOP_ENTRY_SIZE			4
/*
 *	Each Next Hop entry is 16 bytes.
 */
#define NEXTHOP_TABLE_ENTRY_SIZE		(1<<PO2_NEXT_HOP_ENTRY_SIZE)
#define NEXTHOP_TABLE_NUMBER_OF_ENTRIES	4096 /* 256 */


/*------------------------------------------------------------------------------
 * CSIX TX definitions
 */
/* Size of each CSIX Transmit Context (in Bytes) */
#define	  CSIX_TX_CTX_SIZE           64

/* Total number of CSIX Transmit Contexts. */
#define   CSIX_TX_CTX_TOTAL          1024

/* I have no idea how many counters are used !!!!! */
#define   NO_OF_CSIX_TX_COUNTERS   		1
#define   CSIX_TX_COUNTERS_SIZE   		16
    
/*------------------------------------------------------------------------------
 * IPv4 Forwarder definitions
 */
#define 		SIZE_OF_NEXTHOP					16
#define 		NO_OF_NEXTHOP					8192
#define			NO_OF_TRIE_SETS					(8192*2)
#define			SIZE_OF_TRIE_SET				16
#define 		TRIE_RESERVED					((4<<16) + (4<<8) + (4<<8) + (4<<8))

/*
 *	SCRATCH RINGS
 */

#define		    EXCEPTION_RING_OUT_0			0
#define		    EXCEPTION_RING_OUT_1			1
#define		    EXCEPTION_RING_IN_0				2
#define	    	EXCEPTION_RING_IN_1				3

#define			AAL5_RX_TO_IPV4_SCR_RING		4

#define			IPV4_TO_QM_SCR_RING				5

#define			SCHEDULER_TO_QM_SCR_RING		6
    
#define			QM_TO_CSIX_TX_SCR_RING			7

/* There is no support for FPGA header */
#define AAL5_FPGA_PORT_BITS        0
    
#endif		 /*  DL_SYSTEM_CONFIG_H  */


