#ifndef PKTHDR_CACHE_UC
#define PKTHDR_CACHE_UC


/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

// FILE  pkthdr_cache.uc
//


#ifndef	PKT_HDR_CACHE_SIZE
#define	PKT_HDR_CACHE_SIZE		10//8	// 6 longwords
#endif

#ifndef			PKT_HDR_DRAM_RD_SIZE
#define_eval	PKT_HDR_DRAM_RD_SIZE	(PKT_HDR_CACHE_SIZE >> 1)	// read 4 quadwords
#endif

///////////////////////////////////////////////////////////////////////////////
// dl_load_iphdr_cache:
//	 	Description: 
//			Reads the IP header (32 bytes) of the packet from SDRAM and keeps 
//			it in a cache (DRAM transfer registers). 
//
//	 	Outputs:
//			The $$dl_iphdr# global dram transfer registers will be set with appropriate
//			header data.
//
//		Inputs:
//			buf_handle	:			Handle of buffer.
//			req_sig		:			Signal to use in the I/O (DRAM read) operation.
//			sig_action	:			What to do with I/O operation - 
//									- wait for signal(s) as specified by sig_action
//									- do not wait for signal, just return (SIG_NONE)
//		Size:   				:
//
#define	dl_load_iphdr_cache		dl_iphdr_load_cache

#macro dl_iphdr_load_cache[buf_handle, req_sig, sig_action]
.begin
.reg 	sdram_offset, buf_offset

	//	Read 32 bytes of IP Header from SDRAM

	dl_buf_get_data[sdram_offset, buf_handle]
	dl_meta_get_offset[buf_offset]
	dram[read, $$iphdr0, sdram_offset, buf_offset, PKT_HDR_DRAM_RD_SIZE], sig_done[req_sig]	; DRAM reads are in QWORD (8 bytes)

#if (!streq('sig_action', 'SIG_NONE'))
	ctx_arb[sig_action]
#endif

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_flush_iphdr_cache:
//	 	Description: 
//			Writes back the IP header (32 bytes) from the cache to the packet in SDRAM. 
//
//	 	Outputs:
//
//		Inputs:
//			buf_handle	:			Handle of buffer.
//			req_sig		:			Signal to use in the I/O (DRAM read) operation.
//			sig_action	:			What to do with I/O operation - 
//									- wait for signal(s) as specified by sig_action
//									- do not wait for signal, just return (SIG_NONE)
//          ip_hdr_offset :         IP header offset in transfer registers
//			The $$dl_iphdr# global dram transfer registers will be the set of registers 
//			from which data will be written back to SDRAM.
//
//		Size:   				:
//
#define	dl_flush_iphdr_cache		dl_iphdr_flush_cache

#macro dl_iphdr_flush_cache[buf_handle, req_sig, ip_hdr_offset, sig_action]
.begin
.reg 	sdram_offset, buf_offset

	//	Write 32 bytes of IP Header from SDRAM

	dl_buf_get_data[sdram_offset, buf_handle]
	dl_meta_get_offset[buf_offset]

    .if (ip_hdr_offset == IP_HDR_OFFSET_0)
        
	    dram[write, $$iphdr0, sdram_offset, buf_offset, PKT_HDR_DRAM_RD_SIZE], sig_done[req_sig]

    .else

         /* IP_HDR_OFFSET_8 */
        dram[write, $$iphdr2, sdram_offset, buf_offset, (PKT_HDR_DRAM_RD_SIZE - 1)], sig_done[req_sig]

    .endif

#if (!streq('sig_action', 'SIG_NONE'))
	ctx_arb[sig_action]
#endif

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_iphdr_cache_init:
//	 	Description: 
//			This macro needs to be called before any of the dl_iphdr_* can be called.
//			It allocates a set of DRAM xfer registers to be used for caching. This macro
//			doesn't generate any code (instructions). It only produces some assembler
//			directives.
//
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//			0 Instructions

#macro dl_iphdr_cache_init[]

	xbuf_alloc[$$iphdr, PKT_HDR_CACHE_SIZE, read_write]

#endm

///////////////////////////////////////////////////////////////////////////////
// dl_iphdr_cache_fini:
//	 	Description: 
//			This macro needs to be called at the end when dl_iphdr_macros are no longer
//			required. It frees a set of DRAM xfer registers that were allocated using
//			dl_iphdr_cache_init. This macro	doesn't generate any code (instructions). 
//			It only produces some assembler	directives.
//
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//			0 Instructions

#macro dl_iphdr_cache_fini[]

	xbuf_free[$$iphdr]

#endm

#endif	//__PKTHDR_CACHE_UC__