#ifndef SYSTEM_INIT_UC
#define SYSTEM_INIT_UC

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

// FILE  ingress_system_init.uc
//

#include <dispatch_loop.uc>

#define SRAM_Q_ARRAY_NUM_FIELD	24		

////////////////////////////////////////////////////////////////////////
// Macro Name  : _rm_read_q_array
// Description : Macro to read and cache a Queue Descriptor entry from 
//				 SRAM to Q_Array. $QDArray is actually packet count.
// Output      : out_qd_array
// Input       : in_queue_ptr, in_qarray_entry, in_qa_signal
// Constant    : nil
// Size        : 4
// Branches    : 0
////////////////////////////////////////////////////////////////////////
#macro _rm_read_q_array[ in_sram_base, in_queue_ptr, in_qarray_entry, in_qa_signal, out_qd_array  ]

.begin

.reg  qarray_sram_addr, dummy	

	// multiply the pointer by 4 to get the offset

	alu[dummy, --, b, in_queue_ptr, <<2]

	// get the sram QD address by adding the offset to the base
	// the address has to be LW alighned( ie byte address >> 2)

	alu[qarray_sram_addr,dummy , +, in_sram_base]

	// read the complete queue descriptor from the sram.
	
	sram[rd_qdesc_head, out_qd_array, in_qarray_entry,qarray_sram_addr,2], sig_done[in_qa_signal]
	sram[rd_qdesc_other, --, in_qarray_entry, qarray_sram_addr]


.end
#endm

////////////////////////////////////////////////////////////////////////
// Macro Name  : rm_qarray_init
// Description : Macro to read and cache a Queue Descriptor entry from 
//				 SRAM to Q_Array. The macro initializes all 64 Q_arrays
//				 for a given channel
// Output      : 
// Input       : in_sram_base
// Constant    : nil
// Size        : 
// Branches    : 0
////////////////////////////////////////////////////////////////////////
#macro rm_qarray_init[in_qarray_sram_base]

.begin

.reg count
.reg channel_mask
.reg channel_num
.reg sram_base
.reg qarray_entry
.reg qarray_sram_add
.reg $qd_return
.sig qarray_signal

// move sram_Base into a register

move[sram_base, in_qarray_sram_base]

// mask used to mask out everything except
// the two most significant bits, which are
// the channel numbers

move[channel_mask, 0xC0000000]

// using the mask obtain the channel num from
// the given sram address

alu[channel_num, sram_base, AND, channel_mask]

// initialize the count register to the
// num of qarrays in a channel
immed[count, 63]

.while(count != 0 )


	// setting the channelnum and q array entry. This is done before the branch
	// since it might be required for enqueue and dequeue operations in hit case.

	alu[qarray_entry, channel_num, OR, count, <<SRAM_Q_ARRAY_NUM_FIELD]

	_rm_read_q_array[ sram_base, count, qarray_entry, qarray_signal, $qd_return ]

	ctx_arb[qarray_signal]

	alu[count, count, -, 1]
.endw
nop
.end
#endm


///////////////////////////////////////////////////////////////////////////////
// init_meta_cache_to_zero:
//	 	Description: 
//			Just sets the registers used for meta data caching to zero. [This 
//			helps a lot when debugging, because all the unused meta data regsiters
//			will be zero instead of something like 0x5a5a5a).
//
//		Outputs:
//
//	 	Inputs:
//
//		Size:   				
//
#macro	init_meta_cache_to_zero[]
.begin
.reg zero

	immed32[zero, 0]
	dl_meta_init_cache[zero, zero, zero, zero, zero, zero, zero, zero]	

.end
#endm

///////////////////////////////////////////////////////////////////////////////
// system_init:
//	 	Description: 
//			Does all the initialisation to get the system up and running.
//			Typically it initialises all the various rings, creates the buffer
//			free list, initialises the various cache (meta, ip header etc).
//			Once initialisation is done, a signal is sent to all other MEs
//			so that rest of the code, that depends on this initialisation, 
//			to run.
//
//		Outputs:
//
//	 	Inputs:
//
//		Size:   				
//
//
#macro	system_init[]

#ifdef USE_IMPORT_VAR

.import_var QARRAY_INIT_SRAM_BASE_CH0
.import_var QARRAY_INIT_SRAM_BASE_CH1

	.reg sram_base_ch
	.set sram_base_ch
	
	immed32(sram_base_ch, QARRAY_INIT_SRAM_BASE_CH0)
	rm_qarray_init[sram_base_ch]

	immed32(sram_base_ch, QARRAY_INIT_SRAM_BASE_CH1)
	rm_qarray_init[sram_base_ch]


#else

	// Initialise the various scratch rings used in the ingress pipeline

	init_scratch_ring[AAL5_RX_TO_IPV4_SCR_RING_BASE, AAL5_RX_TO_IPV4_SCR_RING_SIZE, AAL5_RX_TO_IPV4_SCR_RING]
	init_scratch_ring[IPV4_TO_QM_SCR_RING_BASE, IPV4_TO_QM_SCR_RING_SIZE, IPV4_TO_QM_SCR_RING]
	init_scratch_ring[SCHEDULER_TO_QM_SCR_RING_BASE, SCHEDULER_TO_QM_SCR_RING_SIZE, SCHEDULER_TO_QM_SCR_RING]
	init_scratch_ring[QM_TO_CSIX_TX_SCR_RING_BASE, QM_TO_CSIX_TX_SCR_RING_SIZE, QM_TO_CSIX_TX_SCR_RING]


	// Initialise the free buffer list

	dl_buffer_init[]

#endif

	.sig cap_write
	.reg $enable_timestamp

	immed[$enable_timestamp, 0x300]
   	cap[write, $enable_timestamp, MISC_CONTROL], ctx_swap[cap_write] ; set 7th bit(Timestamp Enable)

    
	signal_me[FIRST_TC_METER_ME, TC_METER_TIMESTAMP_SIG]

	//	To ease debugging. Not absoultely essential for proper execution.

	init_meta_cache_to_zero[]		; all the dl_meta* gprs will be set to 0.

	//	signal all MEs, except ME 0 (ME0 is running UTOPIA RX & this code)

        .sig remote sys_init_signal 

#if (RX_PHY_MODE == SPHY_1_32)
	signal_me[0x00, (&remote(sys_init_signal))]	; ME 0
	signal_me[0x01, (&remote(sys_init_signal))]	; ME 1
#else
	#ifdef SINGLE_4_8
		signal_me[0x00, (&remote(sys_init_signal) + (0 << 4))]	; ME 0
		signal_me[0x00, (&remote(sys_init_signal) + (2 << 4))]	; ME 0
		signal_me[0x00, (&remote(sys_init_signal) + (4 << 4))]	; ME 0
		signal_me[0x00, (&remote(sys_init_signal) + (6 << 4))]	; ME 0
	#else
		signal_me[0x00, (&remote(sys_init_signal) + (0 << 4))]	; ME 0
		signal_me[0x00, (&remote(sys_init_signal) + (4 << 4))]	; ME 0
		signal_me[0x01, (&remote(sys_init_signal) + (0 << 4))]	; ME 1
		signal_me[0x01, (&remote(sys_init_signal) + (4 << 4))]	; ME 1
	#endif
#endif

	signal_me[0x02, ME_INIT_SIGNAL]	; ME 2
	signal_me[0x03, ME_INIT_SIGNAL]	; ME 3
	signal_me[0x10, ME_INIT_SIGNAL]	; ME 4
	signal_me[0x11, ME_INIT_SIGNAL]	; ME 5 
	signal_me[0x12, ME_INIT_SIGNAL]	; ME 6
	signal_me[0x13, ME_INIT_SIGNAL]	; ME 7 

	.sig remote tc_meter_next_me_signal
	signal_me[FIRST_TC_METER_ME, &remote(tc_meter_next_me_signal, FIRST_TC_METER_ME)]
 	
#endm

#endif	//__SYSTEM_INIT_UC__