/*
 * ============================================================================
 * COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2003 Intel Corporation. All rights reserved.
 *
 * PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *                                                                          */
/** \file      egress_init.c
 *
 *  \brief     The file contains examplery code to configure SAR on hardware
 *
 */

#include <ix_error.h>
#include <ix_types.h>
#include <ix_ossl.h>

ix_error 
ix_cc_llc_snap_encap_cb_add_next_hop_callback(ix_error  arg_Result,
                                              void     *arg_pContext)
{
    /* print the received results */
    printf("Callback reached for ix_cc_llc_snap_encap_async_add_next_hop; Received values:\n"
           "result                      : %u\n"
           "arg_pContext                : 0x%4x\n"
           "*arg_pContext == Next Hop ID: %u\n\n",
           arg_Result, arg_pContext, *((ix_uint16*)arg_pContext));

    /* free context (no longer needed) */
    ix_ossl_free(arg_pContext);

    return IX_SUCCESS;
}




void egress_init()
{
    ix_error   err = IX_SUCCESS;
    ix_uint16 *pContext;

    /* add cbr profile */
    SarUt_TPCBRAdd(11, 707547, 0);

    ix_ossl_sleep(50);

    /* Add VC. parameter order:   /direction/vcid/port/vpi/vci/tpid/type/  */
    SarUt_VcDef( 0, 1, 0, 0, 33, 11,  4);

    /* Create Vc. Parameter order: /vcid/ */
    SarUt_VcCreate(1);
                      
    ix_ossl_sleep(50);

    pContext = (ix_uint16*)ix_ossl_malloc(sizeof(ix_uint16));
    *pContext = 1;

    /* actually issue the command */
    err = ix_cc_llc_snap_encap_async_add_next_hop(
             1,
             1,
             0,
             33,
             1,
             0xaa,
             0xaa,
             3,
             0,
             0x800,
             ix_cc_llc_snap_encap_cb_add_next_hop_callback,
             pContext);

    if (err != IX_SUCCESS ) {
      /* free context (no longer needed) */
      ix_ossl_free(pContext);

      printf("Error in ix_cc_llc_snap_encap_async_add_next_hop\n");
    }

    ix_ossl_sleep(50);

    SarUt_VcUpdate(1, 2, 1);

    return;
}

