/*
 * ============================================================================
 * COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2003 Intel Corporation. All rights reserved.
 *
 * PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *                                                                          */
/** \file      ingress_init.c
 *
 *  \brief     The file contains examplery code to configure SAR on hardware
 *
 */


/*********************************************************************************************************/
/*                                    configure forwarder                                                */
/*********************************************************************************************************/
#include <ix_error.h>
#include <ix_types.h>
#include <ix_ossl.h>

#include "cc/ix_cc_rtmv4.h"
#include "cc/ix_cc_ipv4.h"

ix_error
common_callbacak_ingress(ix_error  arg_Result,
                         void     *arg_pUserContext)
{
    if ( arg_Result != IX_SUCCESS )
    {
        printf( "\nCallback: common_callbacak_ingress: \n" \
                                         "\t_status = 0x%08x \n" \
                                         "\tcontext = %s\n\n",
                                         IX_ERROR_GET_CODE( arg_Result ),
                                         arg_pUserContext );
        printf( "\n Ingress initialization FAILED\n" );
        ix_error_dump( stderr, arg_Result );
        return IX_SUCCESS;

    } else {

        printf( "\nCallback: common_callbacak_ingress: OK\n" );
        printf( "User context: %s\n", (int) arg_pUserContext );

    }
    return IX_SUCCESS;
}



void ingress_init()
{
    ix_error _status;
    /* Next hop data */
    ix_cc_rtmv4_next_hop_info ipv4_next_hop_data_rtm;

    memset(&ipv4_next_hop_data_rtm, 0, sizeof(ix_cc_rtmv4_next_hop_info));

    ipv4_next_hop_data_rtm.mtu         = 2048;
    ipv4_next_hop_data_rtm.bladeID     = 1;
    ipv4_next_hop_data_rtm.ipAddr      = 0x0A442A21;
    ipv4_next_hop_data_rtm.l2IndexType = 0;
    ipv4_next_hop_data_rtm.l2Index     = 1;

    if ( ( _status = ix_cc_ipv4_async_add_next_hop(
                        0x1,
                        &ipv4_next_hop_data_rtm,
                        common_callbacak_ingress,
                        "Add next hop" ) ) != IX_SUCCESS )
    {
        printf( "Add next hop RTM on Ingress FAILED 0x%08x\n",
                                         IX_ERROR_GET_CODE( _status ) );
        return;
    }
    ix_ossl_sleep(1000);

    if ( ( _status = ix_cc_ipv4_async_add_route(
                       0x0A452A21,
                       0xFFFFFFFF,
                       0x1,
                       common_callbacak_ingress,
                       "Add route RTM" ) ) != IX_SUCCESS )
    {
        printf( "Add route RTM on Ingress FAILED 0x%08x\n",
                                         IX_ERROR_GET_CODE( _status ) );
        return;
    }
    ix_ossl_sleep(1000);

    return;
}


