/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2000 - 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILENAME
 *      ix_sa_cc_list.c
 *
 * ============================================================================
 */

#define IX_ERROR_FILE_IDENT \
  "$Id: ix_sa_cc_list.c,v 1.9.4.2 2004/03/12 11:06:43 wneubaue Exp $"

/**
 * @file  _ix_sa_cc_list.c
 * @brief Definition of CC_INITIALIZATION function.
 **/
#include "ix_ossl.h"
#include "ix_cc_error.h"
#include "sa/ix_sa_cc_list.h"
#include "cc/ix_cc_tc_meter.h"
#include "cc/ix_cc_wred.h"
#include "cc/ix_cc_csix_rx.h"
#include "cc/ix_cc_atm_pos_tx.h"
#include "cc/ix_cc_qm_frmwrk.h"
#include "cc/ix_cc_sched.h"
#ifdef INCLUDE_PDK
#include "cc/ix_cc_fpm_proxy.h"
#endif

#if !defined(IX_CONFIGURATION_oc48_sphy_pos_egress)
#error IX_CONFIGURATION_oc48_sphy_pos_egress not defined! Wrong configuration header.
#endif /* IX_CONFIGURATION_oc48_sphy_pos_egress */


#if !defined(IX_USE_ENV_VAR)
    char g_sa_uofFile[256] = "";
#endif





/* -------------------------------------------------------- _ix_sa_cc_list -- */
ix_error _ix_sa_cc_list( ix_sa_ccfunc *arg_pCC_List )
{
    IX_ERROR_CHECK_ARG_NULL( 1, arg_pCC_List );


    /* Pos ATM tx */
    _ix_sa_set_cc(arg_pCC_List,
        IX_CC_ATM_POS_TX, ix_cc_atm_pos_tx_init, ix_cc_atm_pos_tx_fini);

    /* add CSIX_RX core component */
    _ix_sa_set_cc( arg_pCC_List	,
        IX_CC_CSIX_RX, ix_cc_csix_rx_init, ix_cc_csix_rx_fini );

    /* add Queue Manager core component */
    _ix_sa_set_cc( arg_pCC_List,
        IX_CC_QM, ix_cc_qm_init, ix_cc_qm_fini );

    /* add Scheduler core component */
    _ix_sa_set_cc( arg_pCC_List,
        IX_CC_SCHEDULER, ix_cc_sched_init, ix_cc_sched_fini );

    /* add TC Meter core component */
    _ix_sa_set_cc( arg_pCC_List,
        IX_CC_TC_METER, ix_cc_tc_meter_init, ix_cc_tc_meter_fini );

    /* add WRED core component */
    _ix_sa_set_cc( arg_pCC_List,
        IX_CC_WRED, ix_cc_wred_init, ix_cc_wred_fini );
#ifdef INCLUDE_PDK
    _ix_sa_set_cc( arg_pCC_List,
        IX_CC_FPM_PROXY, ix_cc_fpm_proxy_init, ix_cc_fpm_proxy_fini );
#endif


    return IX_SUCCESS;
}
