/*
 * Intel(r) IXA SDK 3.0 for the IXP2x00 Network Processor, Release 6
 *
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a licence agreement or
 * nondisclosure agreement with Intel Corporation and may not be copied
 * or disclosed except in accordance with the terms in that agreement.
 *
 *    Copyright (C) 2000-2003 Intel Corporation.All rights reserved.
 */

#define IX_ERROR_FILE_IDENT \
  "$Id: ix_sa_symbols.c,v 1.7.2.1 2004/03/12 11:06:44 wneubaue Exp $"

/**
 * @file  ix_sa_symbols.c
 * @brief Patches misc symbols for this configuration.
 */

#include "ix_ossl.h"
#include "ix_cc_error.h"
#include "sa/internal/internal_sa.h"
#include "sa/ix_sa_cc_list.h"


#if !defined( IX_CONFIGURATION_oc48_sphy_pos_egress )
#error IX_CONFIGURATION_oc48_sphy_pos_egress not defined! Wrong configuration header.
#endif /* !IX_CONFIGURATION_oc48_sphy_pos_egress */

/* ------------------------------------------------- _ix_sa_patch_symbols -- */

ix_error _ix_sa_patch_symbols( ix_sa_ctrl * arg_sa_ctrl )
  {
    ix_imported_symbol       _Symbols[7];
    ix_uint32                _base;
    ix_buffer_free_list_info _fl_info;
    ix_uint32                _fl_channel;
    ix_memory_type           _fl_type;
    void *                   _zb_ptr;
    ix_uint32                _phys_meta_addr;
    ix_uint32                _phys_data_addr;
    void *                   _phys_ptr;

    /* validity check for arg_sa_crtl */
    IX_ERROR_CHECK_ARG_NULL( 1, arg_sa_ctrl );

    IX_ERROR_CR( ix_rm_mem_alloc(
        IX_MEMORY_TYPE_SRAM, 0, sizeof( ix_uint32 ), &_zb_ptr )
      );

    IX_ERROR_CR( ix_rm_buffer_free_list_get_info(
        arg_sa_ctrl->freelist[1], &_fl_info )
      );

    IX_ERROR_CR( ix_rm_get_phys_offset( (void *)_fl_info.m_pMetaBaseAddress,
        &_fl_type, &_fl_channel, &_phys_meta_addr, &_phys_ptr )
      );

    IX_ERROR_CR ( ix_rm_get_phys_offset( (void *)_fl_info.m_pDataBaseAddress,
        &_fl_type, &_fl_channel, &_phys_data_addr, &_phys_ptr )
      );

    IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET( 0, _phys_meta_addr );
    IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET( 0, _phys_data_addr );

    _base = _phys_data_addr - (_phys_meta_addr *
      (_fl_info.m_DataElementSize / _fl_info.m_MetaElementSize));

    _Symbols[0].m_Value =
      (ix_uint32)((ix_uint32)_fl_info.m_pMetaBaseAddress & 0x3fffffff);
    _Symbols[0].m_Name  = "BUF_SRAM_BASE";

    _Symbols[1].m_Value = (ix_uint32)_fl_info.m_FreeListInfo;
    _Symbols[1].m_Name  = "BUF_FREE_LIST0";

    IX_ERROR_CR( ix_rm_ueng_patch_symbols(
      g_aMicroengineNumberMapping[1], 2, _Symbols ) );
    IX_ERROR_CR( ix_rm_ueng_patch_symbols(
      g_aMicroengineNumberMapping[2], 2, _Symbols ) );
    IX_ERROR_CR( ix_rm_ueng_patch_symbols(
      g_aMicroengineNumberMapping[5], 2, _Symbols ) );
    IX_ERROR_CR( ix_rm_ueng_patch_symbols(
      g_aMicroengineNumberMapping[7], 2, _Symbols ) );

    _Symbols[2].m_Value = _base;
    _Symbols[2].m_Name  = "DL_REL_BASE";

    IX_ERROR_CR( ix_rm_ueng_patch_symbols(
      g_aMicroengineNumberMapping[6], 1, &_Symbols[2] ) );

    _Symbols[3].m_Value = 0;
    _Symbols[3].m_Name  = "FREE_LIST_ID";

    IX_ERROR_CR( ix_rm_ueng_patch_symbols(
      g_aMicroengineNumberMapping[0], 1, _Symbols ) );

    IX_ERROR_CR( ix_rm_ueng_patch_symbols(
      g_aMicroengineNumberMapping[0], 4, _Symbols ) );

    IX_ERROR_CR( ix_rm_ueng_patch_symbols(
      g_aMicroengineNumberMapping[4], 3, _Symbols ) );
/*


	#if defined  (__COUNTERS__)
        if ( ix_rm_mem_alloc( IX_MEMORY_TYPE_SRAM,
               0, 48 * 4, (void **)&_base ) != IX_SUCCESS )
        {
             ix_ossl_message_log( "ix_rm_mem_alloc - failed again:-(\n" );
             _base = 0;
        }
        IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET( 0, _base );

        _Symbols[0].m_Value = _base;
        _Symbols[0].m_Name  = "PACKET_TX_COUNTER_BASE";

        IX_ERROR_CR( ix_rm_ueng_patch_symbols(
            g_aMicroengineNumberMapping[4], 1, _Symbols ) );
    #endif
*/
    return IX_SUCCESS;
  }
