#ifndef DIFFSERV_PIPELINE_UC
#define DIFFSERV_PIPELINE_UC

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

// FILE:  diffserv_pipeline.uc
//


//------------------------------------------------------------------------------
//    Global definitions

#define	 META_CACHE_SIZE	8
#define  BID_NEXT_BLOCK   	BID_QM	// block next to dispatch loop

#define  DL_DIFFSERV_DEBUG


//------------------------------------------------------------------------------
//    Include files

#include "dl_system.h"
#include "diffserv.h"
#include "system_init.uc"
#include "dispatch_loop.uc"

// wred uses queue decriptor constats from queue manager.
#include <qm_diffserv.h>
#include <tc_meter.uc>
#include <wred.uc>


//------------------------------------------------------------------------------
//    Global registers
.reg @system_shut_down


#ifdef DL_DIFFSERV_DEBUG

.reg @debug_eg_dl_cycles
.reg @debug_eg_dl_packets_received
.reg @debug_eg_dl_packets_sent
.reg @debug_eg_dl_ring_full
.reg @debug_eg_dl_drop_path
.reg @debug_eg_dl_null_path

#endif




.reg dl_queue_num

;-----------------------------------------------------------
; Initialize the blocks
;-----------------------------------------------------------
init#:


	; init buffer meta data cache to Zero.
	init_meta_cache_to_zero[]

	; Temporary : Initialize EOP handle default.
	_dl_eop_buf_handle_init[]

	dl_source_init()
	tc_meter_init()
	wred_init()

	.begin
	.sig     sys_init_signal
	.addr    sys_init_signal	ME_INIT_SIGNAL
	.set_sig sys_init_signal

	.if (ctx()==0)
		move(@system_shut_down, 0)

		#ifdef DL_DIFFSERV_DEBUG
			alu[@debug_eg_dl_packets_received, --, B, 0]
			alu[@debug_eg_dl_packets_sent, --, B, 0]
			alu[@debug_eg_dl_cycles, --, B, 0]
			alu[@debug_eg_dl_ring_full, --, B, 0]
			alu[@debug_eg_dl_drop_path, --, B, 0]
			alu[@debug_eg_dl_null_path, --, B, 0]
		#endif

		ctx_arb[sys_init_signal]
        signal_next_ctx[&sys_init_signal]

	.else
		ctx_arb[sys_init_signal]
		.if (ctx()==7)
		    signal_first_ctx(TC_METER_NEXT_THREAD_SIG)
		    signal_first_ctx(WRED_NEXT_THREAD_SIG)
		.else
	        signal_next_ctx(&sys_init_signal)
		.endif
	.endif
	.end



;---------------------------------------------------------
; dispatch loop
;-----------------------------------------------------------
	.sig sig_scratch_put
	.reg reg_scratch_put

start#:
	; signal will be sent to the same context we are now in.
	local_csr_rd[ACTIVE_CTX_STS]
	immed[reg_scratch_put, 0]


	; current context is held on the lowest 3 bits.
	alu[reg_scratch_put, reg_scratch_put, and, 7]
	alu_shf[reg_scratch_put, reg_scratch_put, OR, &sig_scratch_put, <<3]

	local_csr_wr[SAME_ME_SIGNAL, reg_scratch_put]
	.set_sig sig_scratch_put

	// receive packet
	dl_source(sig_scratch_put)

loop#:

	tc_meter()

	wred()

	#ifdef DL_DIFFSERV_DEBUG
		alu[@debug_eg_dl_cycles, @debug_eg_dl_cycles, +, 1]
	#endif

	// send packet.
	dl_qm_sink(sig_scratch_put,
			   reg_scratch_put)

	dl_source(sig_scratch_put)


	alu[--, --, B, @system_shut_down]
	beq[loop#]


;-----------------------------------------------------------
; terminate. We never execute this. We keep this as a
; place holder.
;-----------------------------------------------------------
term#:

    tc_meter_term()

	wred_term()


kill#:
	nop
	ctx_arb[kill]
	nop



#endif // DIFFSERV_PIPELINE_UC

