#ifndef DL_DIFFSERV_SOURCE_UC
#define DL_DIFFSERV_SOURCE_UC

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

// FILE  egress_dl_diffserv_source.uc
//


//------------------------------------------------------------------------------
// Included files:
#include <ix_cc_microengines_bindings.h>
#include <stdmac.uc>
#include <sig_macros.uc>
#include <dl_csix_rx_sink.uc>

//------------------------------------------------------------------------------
// Global definitions

#if (DIFFSERV_RING_OUT_0 > 11)
#error			"For Ring Number Greater than 11, we cannot use the SCR_Ring#_Full input state. Check DIFFSERV_RING_OUT_0"
#else
#define_eval    _RING_NO 	DIFFSERV_RING_OUT_0
#define_eval	DIFFSERV_RING_OUT_0_FULL	SCR_Ring/**/_RING_NO/**/_Status
#undef 			_RING_NO
#endif


///////////////////////////////////////////////////////////////////////////////
// dl_source_init:
//	 	Description:
//
//			Initialisation for dl_source. Should be called before using dl_source.
//			Currently not used/required. Place holder.
//
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//
//
#macro dl_source_init[]

#endm


///////////////////////////////////////////////////////////////////////////////
// _dl_eop_buf_handle_init:
//	 	Description:
//			THIS IS NOT AN EXPORTED MACRO. PLS DO NOT USE IT. THIS WILL CHANGE.
//			THIS IS ONLY TEMPORARY.
//
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//
//
#macro _dl_eop_buf_handle_init[]
.begin

	immed32[dl_eop_buf_handle, IX_NULL]

.end
#endm

///////////////////////////////////////////////////////////////////////////////
// dl_sink_init:
//	 	Description:
//
//			Initialisation for dl_sink. Should be called before using dl_sink.
//			Currently not used/required. Place holder.
//
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//
//
#macro dl_sink_init[]


#endm





///////////////////////////////////////////////////////////////////////////////
// dl_source:
//	 	Description:
//
//
#macro dl_source(sig_scr_put)
.begin

dl_source_start#:

	xbuf_alloc[$rdata, 3, read]	 ; allocate xfer register. No Code generated.

	.reg ring
	; Ring from which to consume
	alu_shf[ring, --, B, DIFFSERV_RING_IN_0, <<2]

	.sig sig_scr_get  ; signal for scratch get
	; Get 3 Long words from ring
	scratch[get, $rdata[0], 0, ring, 3], sig_done[sig_scr_get]

	// We wait for the above scratch[get...] and
	// the scratch[put...] in dl_qm_sink.
	ctx_arb[sig_scr_get, sig_scr_put]


check_ring_empty#:

	//	Check if Ring is empty. ==> it would have returned zero
	alu[--, $rdata[0], -, 0]; Check for 0. i.e check if ring empty
	beq[ring_empty#]

	#ifdef DL_DIFFSERV_DEBUG
	alu[@debug_eg_dl_packets_received, @debug_eg_dl_packets_received, +, 1]
	#endif

	; LW0 from ring is the current buffer handle (containign SOP)
	move[dl_buf_handle, $rdata[0]]

	; LW1 from ring is last buffer handle (for large packets)
	move[dl_eop_buf_handle, $rdata[1]]

	.reg color_id
	alu[color_id, COLOR_ID_MASK, AND, $rdata[2], >>24]
	dl_meta_set_color(color_id)

	.reg packet_size
	alu[packet_size, 0xFF, AND, $rdata[2], >>8]
	alu[dl_queue_num, $rdata[2], AND~, 0xff, <<8]
	dl_meta_set_packet_size(packet_size)

	.reg class_id
	alu[class_id, 0x0F, AND, $rdata[2]]

	.reg class_priority
	alu[--, class_id, OR,0]
	alu_shf[class_priority, --, B, 1, <<indirect]
	alu[--, HIGH_PRIORITY_MASK, AND, class_priority]
	bne[high_priority_queue#]

low_priority_queue#:

	.reg port_id
	alu[port_id, 0x0F, AND, $rdata[2], >>4]
	dl_meta_set_output_port(port_id)
	dl_meta_set_class_id(class_id);
	move[dl_next_block, DL_SOURCE_NEXT2]
	br[end_source#]


high_priority_queue#:
	.reg  queue_id
	alu[queue_id, 0xFF, AND, $rdata[2]]
	dl_meta_set_flow_id(queue_id)
	move[dl_next_block, DL_SOURCE_NEXT1]
	br[end_source#]


//	Handle Ring empty Condition.
ring_empty#:

	// If ring is empty, just set dl_buf_handle and dl_next_block to IX_NULL;
	// Note IX_NULL is NOT ZERO.
	.set dl_queue_num
	move[dl_buf_handle, IX_NULL]
	move[dl_next_block, IX_NULL]

end_source#:

	xbuf_free[$rdata]; Free xfer registers. No Code generated.


.end
#endm



///////////////////////////////////////////////////////////////////////////////
// dl_qm_sink:
//	 	Description:
//			Called by IPv4 Block to send the packet to Queue Manager thro' a scratch ring.
//			This macro will do thread ordering always.
//
//			This macro will produce the following on to the scratch ring.
//			dl_buf_handle		// The current buffer containing SOP
//			dl_eop_buf_handle	// for large packets, the buffer containing EOP. Otherwise NULL
//			Queue Number		// Queue number into which this packet is to be queued.
//								// Queue number = 16 * output Fabric id. (available in meta data)
//	 	Outputs:
//			sig_mask			- if the I/O operation is made (scratch put) then the sig_mask
//								  will have the bit corresponding to sig_scr set. (sig_scr is an
//								  input parameter)
//
//		Inputs:
//			sig_scr				- Signal to use in the I/O (scracth put ) operation
//			wxfer				- SRAM write xfer regsiter to be used in I/O.
//								  Should have been allocated using xbuf_alloc.
//								  Atleast 3 regsiters should be allocated.
//
//			The following Global variables will also act as inputs.
//			dl_next_block		- IX_EXCEPTION, for exception packets
//								- IX_NULL, for null packets.
//								- BID_NEXT_BLOCK, block ID of the next block to handle this packet.
//			dl_buf_handle		: Buffer handle of the buffer in the start of chain (contianinf SOP)
//			dl_buf_eop_handle	: Buffer handle of the last buffer in the chain
//
//
//
#macro dl_qm_sink(sig_scr_put, reg_scr_put)
.reg write $wxfer0, $wxfer1, $wxfer2
.xfer_order $wxfer0, $wxfer1, $wxfer2
.begin

	// now check for next block
	br!=byte[dl_next_block, 0, BID_NEXT_BLOCK, check_exception#]

check_ring_full#:
	; Check if ring is full and loop
 	br_inp_state[DIFFSERV_RING_OUT_0_FULL, ring_is_full#]

	; At this point, ring has enough space. Sink the data.
	; LW0 on the ring is current buffer hande
	move($wxfer/**/0, dl_buf_handle)
	; LW1 on the ring is the last buffer in the chain,
	; for big pkts only. Othereise IX_NULL
	move($wxfer/**/1, dl_eop_buf_handle)

	move($wxfer/**/2, dl_queue_num)

	.reg ring

	; ring nos in get/put is always multiple of 4.
	alu_shf[ring, --, b, DIFFSERV_RING_OUT_0, <<2]

	; Produce 3 Long words on the ring
	scratch[put, $wxfer/**/0, 0, ring, 3], sig_done[sig_scr_put]


	#ifdef DL_DIFFSERV_DEBUG
	alu[@debug_eg_dl_packets_sent, @debug_eg_dl_packets_sent, +, 1]
	#endif

	// To improve performance, we do not ctx_swap on scratch[put..] above.
	// We'll wait on this signal in dl_source macro. However this(dl_qm_sink)
	// macro has multiple paths in which we do not issue this scratch[put..]
	// at all. e.g exception packets, or NULL packets etc. So if we issue
	// scratch[put..], we set the sig_mask to indicate that.

	br[end#]


ring_is_full#:
	#ifdef DL_DIFFSERV_DEBUG
	alu[@debug_eg_dl_ring_full, @debug_eg_dl_ring_full, +, 1]
	#endif

	nop						; to set break point
	br[check_ring_full#]	; go back and check again


// 	Check for exception Packets. Exception packets are sent to the core (Xscale)
//	on a separate ring.
check_exception#:

	br=byte[dl_next_block, 0, IX_EXCEPTION, drop_packet#]

	br!=byte[dl_next_block, 0, IX_DROP, check_ix_null#]

	// 	Drop the packet. Two cases to handle. Single buffer or a bufffer chain.
	//	It is determined by looking at the dl_eop_buf_handle

drop_packet#:
	alu[@debug_eg_dl_drop_path, @debug_eg_dl_drop_path, +, 1]

	alu[--, dl_eop_buf_handle, -, IX_NULL]	; if dl_eop_buf_handle == IX_NULL
	beq[drop_single_buffer#]				; then it is single buffer.

	.begin
		.reg	queue_number
		.reg 	ring

	check_drop_ring_full#:
		; Check if ring is full and loop
	 	br_inp_state[DIFFSERV_RING_OUT_0_FULL, check_drop_ring_full#]

		// At this point, ring has enough space. Write the data

		move[$wxfer0, dl_buf_handle]		; LW0 on the ring is current buffer handle
		move[$wxfer1, dl_eop_buf_handle]	; LW1 on the ring is the last buffer in the chain,
											; for big pkts only. Othereise IX_NULL

		immed[queue_number, QM_DROP_QUEUE]				; LW2 on the ring is Queue number for Queue Manager
		alu_shf[$wxfer2, queue_number, OR, 1, <<31]		; The msb is always set to 1, to prevent producing a value of 0 on to the ring.
		alu_shf[ring, --, b, DL_DROP_RING, <<2]			; ring nos in get/put is always multiple of 4.

		scratch[put, $wxfer0, 0, ring, 3], sig_done[sig_scr_put]; Produce 3 Long words on the ring

	.end

	br[end#]



drop_single_buffer#:

	dl_drop_buffer[dl_buf_handle]

	.set_sig sig_scr_put
	local_csr_wr[SAME_ME_SIGNAL, reg_scr_put]

	br[end#]



check_ix_null#:

	.set_sig sig_scr_put
	local_csr_wr[SAME_ME_SIGNAL, reg_scr_put]
	alu[@debug_eg_dl_null_path, @debug_eg_dl_null_path, +, 1]

end#:

.end
#endm



#endif	// DL_DIFFSERV_SOURCE_UC
