#ifndef	DL_SYSTEM_H
#define	DL_SYSTEM_H

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

/* FILE  dl_system.h
 *
 * This file contains the system wide parameters that can be 
 * customised for a given hardware/application
 */

#ifdef IXP_MICROCODE
#include <ix_cc_microengines_bindings.h>
#endif

/*------------------------------------------------------------------------------
 * Include global definition and configuration files
 *----------------------------------------------------------------------------*/

#include <system.h>
#include <wred_defs.h>
#include <tc_meter_defs.h>

/*------------------------------------------------------------------------------
 * 1) Miscellaneous settings.
 *----------------------------------------------------------------------------*/

/* PHY mode for TX device. */
#ifndef		TX_PHY_MODE
#define		TX_PHY_MODE			SPHY_1_32
#endif

#ifndef		USE_IMPORT_VAR

/* Freelist id that is stored in meta data */
#ifndef		FREE_LIST_ID
#define		FREE_LIST_ID		0
#endif

/* This blade's id */
#ifndef		THIS_BLADE_ID
#define		THIS_BLADE_ID		0
#endif

#endif /* USE_IMPORT_VAR */


#ifdef		USE_IMPORT_VAR

/* Patched by core */
.import_var DL_REL_BASE
.import_var BUF_FREE_LIST0
.import_var BUF_SRAM_BASE
.import_var L2_TABLE_SRAM_BASE

#endif /* USE_IMPORT_VAR */

/*
 *	These tell if a the executing threads need to work in order.
 *  i.e wait for a signal from prev thread, do some work
 *  and signal next thread.
 */
#define			DL_THREAD_NO_ORDER				0x0
#define			DL_THREAD_ORDER					0x1

/* used to initialize meter & WRED signals */
#define 		FIRST_TC_METER_ME               0x01
#define 		FIRST_WRED_ME					0x01
#define         FIRST_PIPELINE_ME               0x01

/* DL Source definitions. */
#define   DL_SOURCE_NEXT_THREAD_SIG		10


/*------------------------------------------------------------------------------
 * 2) Microblock specific settings.
 *----------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
 * i) CSIX RX microblock.
 *----------------------------------------------------------------------------*/

/*
 * Size of each CSIX Receive Context (in Bytes)
 */
#ifndef		CSIX_RX_CTX_SIZE
#define		CSIX_RX_CTX_SIZE	32
#endif

/*
 * Total number of CSIX Receive Contexts.
 */
#ifndef		CSIX_RX_CTX_TOTAL
#define		CSIX_RX_CTX_TOTAL	1024
#endif

/* 
 * Size of each CSIX Receiver Counter (in Bytes)
 */
#ifndef		CSIX_RX_COUNTER_SIZE
#define		CSIX_RX_COUNTER_SIZE	16
#endif


/*------------------------------------------------------------------------------
 * ii) L2 encap microblock.
 *----------------------------------------------------------------------------*/

/*
 * Following definition enable this microblock add L2 header before payload 
 * so L2_ENCAP microblock can be removed from pipeline for performance 
 * optimization
 */
#ifdef SKIP_L2_ENCAP
#ifndef		ADD_L2_HEADER
#define  	ADD_L2_HEADER		SWITCH_ENABLE	
#endif
#endif


/*------------------------------------------------------------------------------
 * iii) Queue manager microblock.
 *----------------------------------------------------------------------------*/

/* Size of each queue descriptor entry in bytes. */
#define     QD_SRAM_SIZE           (1 << QD_LOG_SRAM_SIZE)

#define     QM_INIT_TIMESTAMP_LOW     0x00000000
#define     QM_INIT_TIMESTAMP_HIGH    0x00000000

/* Queue number for drop queue. (0-1023 are the normal queues used
   in forwarding pkts) */
#ifndef		QM_DROP_QUEUE
#define		QM_DROP_QUEUE		1024
#endif

/*------------------------------------------------------------------------------
 * iv) TC METER microblock.
 *----------------------------------------------------------------------------*/


#define   TC_METER_DISABLE_FIN_SYNC


/* Base address in sram for the meter table  */
#ifndef	  USE_IMPORT_VAR
#define   TC_METER_TABLE_SRAM_BASE         (BUF_SRAM_BASE + BUF_SRAM_SIZE)
#endif

/*
 * Amount of SDRAM space taken by meter table
 * We use 256 entries (one for each queue)
 */
#define   TC_METER_TABLE_SRAM_SIZE         (0x100 * \
                                           TC_METER_ENTRY_SRAM_SIZE)

#ifndef	  USE_IMPORT_VAR
#ifdef    TC_METER_PACKET_COUNTER_FEATURE
/* 
 * defines the base address of the table where the most significant 
 * long words of meter statistics are held.
 */
#define   TC_METER_64BIT_STAT_SRAM_BASE    (TC_METER_TABLE_SRAM_BASE + \
                                           TC_METER_TABLE_SRAM_SIZE)

/* 
 * defines the SDRAM space taken by meter's 64 bit supplementary statistics
 */
#define   TC_METER_64BIT_STAT_TABLE_SRAM_SIZE   (0x100 * \
                                                 TC_METER_64BIT_STAT_SRAM_SIZE)
#endif
#endif



/* signal used to synchronize meter threads on particular microengines */
#define   TC_METER_NEXT_THREAD_SIG         13

/*
 * signal used to initialize timestamp on every timestamp
 */
#define   TC_METER_TIMESTAMP_SIG           15

/*
 * signal used to synchronize meter crtitical sections 
 * on different microengines
 */
#define   TC_METER_NEXT_ME_SIG             11


/*------------------------------------------------------------------------------
 * iv) WRED microblock.
 *----------------------------------------------------------------------------*/


#define   WRED_DISABLE_FIN_SYNC

/*
 * value used to determine the number of queues per port in the system
 * queue_id = (output_port_id << WRED_MAX_QUEUE) + class_id
 */
#define   WRED_MAX_QUEUE                   4


/*
 * Base address in sram for the wred table
 */
#ifndef	USE_IMPORT_VAR
#ifdef  TC_METER_PACKET_COUNTER_FEATURE
#define   WRED_TABLE_RAM_BASE			   (TC_METER_64BIT_STAT_SRAM_BASE + \
                                            TC_METER_64BIT_STAT_TABLE_SRAM_SIZE)
#else
#define   WRED_TABLE_RAM_BASE			   (TC_METER_TABLE_SRAM_BASE + \
                                            TC_METER_TABLE_SRAM_SIZE)
#endif
#endif


/*
 * Amount of SDRAM space taken by wred table
 * in simulation we use 16 queueus on each of 16 ports entries
 */
#define   WRED_TABLE_RAM_SIZE  		      (WRED_ENTRY_RAM_SIZE * 16*16)




#ifndef	  USE_IMPORT_VAR
#ifdef    WRED_PACKET_COUNTER_FEATURE

/* 
 * defines the base address of the table where the most significant 
 * long words of meter statistics are held.
 */
#define   WRED_64BIT_STAT_SRAM_BASE		   (WRED_TABLE_RAM_BASE + \
                                            WRED_TABLE_RAM_SIZE)

/* 
 * defines the SDRAM space taken by wred's 64 bit supplementary statistics
 */
#define   WRED_64BIT_STAT_TABLE_SRAM_SIZE  (WRED_64BIT_STAT_SRAM_SIZE * 16*16)

#endif
#endif



/*
 * signal used to synchronize wred threads on particular microengines
 *
 * #define	  WRED_NEXT_THREAD_SIG     	    14
 *
 * let's compiler define this
 */

/*
 * signal used to synchronize wred crtitical sections on different microengines
 */
#define   WRED_NEXT_ME_SIG				12


/*------------------------------------------------------------------------------
 * iv) DiffServ SCHEDULER microblock.
 *----------------------------------------------------------------------------*/




/* MTU for the media */
#define   MTU							(9 * 1024)

/*
 * Mask defining high priority queues on ports
 */
#define   HIGH_PRIORITY_MASK			0x000000FF

#define   NUMBER_OF_PORTS                    16
#define   NUMBER_OF_BITS_FOR_PORT             4
#define   PORT_LM_BASE_OFFSET                 0
#define   SCHEDULER_MAX_NO_PENDING_REQUESTS  12



/*------------------------------------------------------------------------------
 * 3) SRAM memory map for channel 0.
 *----------------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

 /* Area of SRAM which is zero'ed out. */
#ifndef		SRAM_ZERO_BLOCK
#define		SRAM_ZERO_BLOCK 	0x0
#endif

#endif /* USE_IMPORT_VAR */


/*------------------------------------------------------------------------------
 * i) Base address for meta data (buffer descriptors) of packet 
 *    buffers.
 *----------------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

#ifndef		BUF_SRAM_BASE
#define		BUF_SRAM_BASE		4 /* first available non zero address */
#endif

#endif /* USE_IMPORT_VAR */

/* Total SRAM size allocated in bytes for meta data.
   In simulation we use only 10KB. */
#ifndef		BUF_SRAM_SIZE
#ifdef 		IXP_SIMULATION
#define		BUF_SRAM_SIZE		(1024 * 10)
#else
#define		BUF_SRAM_SIZE		(1024 * 10 * 2 * 20)
#endif
#endif

/* Size of meta-data (buffer descriptor) in bytes for each buffer.
   Should be a power of 2. */
#ifndef		META_DATA_SIZE
#define		META_DATA_SIZE		32
#endif

/* Allocating 1 MB of SRAM for meta-data. */


/*------------------------------------------------------------------------------
 * ii) Base address for queue descriptors.
 *----------------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR


#ifndef		QD_SRAM_BASE
#ifdef      WRED_PACKET_COUNTER_FEATURE
#define		QD_SRAM_BASE		(WRED_64BIT_STAT_SRAM_BASE + \
                                WRED_64BIT_STAT_TABLE_SRAM_SIZE)
#else
#define		QD_SRAM_BASE		(WRED_TABLE_RAM_BASE +       \
                                 WRED_TABLE_RAM_SIZE)
#endif
#endif

/* Total number of queue descriptors. */
#ifndef		QD_TOTAL
#define		QD_TOTAL			256
#endif

#define     QD_SIZE             (1 << QD_LOG_SRAM_SIZE)



/* Allocating 1 MB of SRAM for queue descriptors. */
#endif /* USE_IMPORT_VAR */

#define     NUM_CLASS_BITS      4

/*------------------------------------------------------------------------------
 * iii) Base address for CSIX RX contexts.
 *----------------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

#ifndef		CSIX_RX_CTX_SRAM_BASE
#define		CSIX_RX_CTX_SRAM_BASE	(QD_SRAM_BASE + (QD_TOTAL * QD_SIZE))
#endif

#endif /* USE_IMPORT_VAR */


/*------------------------------------------------------------------------------
 * iv) Base address for counters.
 *----------------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

/*
 * CSIX_RX needs 1024 * 4 counters and each counter is 1LW 
 * This implies 1024*4*4 bytes
 */
#ifndef		CSIX_RX_COUNTERS_SRAM_BASE
#define		CSIX_RX_COUNTERS_SRAM_BASE	0x300000
#endif

#ifndef		CSIX_RX_COUNTER_BASE
#define		CSIX_RX_COUNTER_BASE		0x300000
#endif

/*
 * Packet TX needs 16*3 counters and each counter is 1LW 
 * this implies 48*4 bytes. Though CSIX RX counter does not require 
 * 0x10000 bytes but we start Packet TX counters from 0x310000, 
 * to provide memory for any additional counters in future.
 */
#ifndef		PACKET_TX_COUNTER_BASE
#define		PACKET_TX_COUNTER_BASE		0x310000	
#endif

#endif /* USE_IMPORT_VAR */


/*------------------------------------------------------------------------------
 * v) Base address for L2 table.
 *----------------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

/*
 * L2 Table (each entry has 32 bytes, bit 31 of first long word is
 * valid bit requires 64K * 32 = 2M 
 */
#ifndef		L2_TABLE_SRAM_BASE
#define 	L2_TABLE_SRAM_BASE			0x320000
#endif

#endif /* USE_IMPORT_VAR */


/*------------------------------------------------------------------------------
 * 4) SRAM memory map for channel 1.
 *----------------------------------------------------------------------------*/


/*------------------------------------------------------------------------------
 * 5) SRAM Q-array memory map.
 *----------------------------------------------------------------------------*/

/* SRAM q-array entries 0-15 are used by the queue manager to cache queues */
#ifndef		QM_QARRAY_BASE
#define		QM_QARRAY_BASE			0
#endif

#ifndef		QM_QARRAY_SIZE
#define		QM_QARRAY_SIZE			16
#endif

#ifndef		USE_IMPORT_VAR
//asv

/* SRAM q-array entries allocated for buffer free list. */
#ifndef		BUF_QARRAY_BASE
#define		BUF_QARRAY_BASE			16
#endif

#ifndef		BUF_QARRAY_SIZE
#define		BUF_QARRAY_SIZE			4
#endif

/* SRAM q-array entry allocated for drop queue in queue manager. */
#ifndef		QM_DROP_QUEUE_ENTRY
#define		QM_DROP_QUEUE_ENTRY		20
#endif

#endif /* USE_IMPORT_VAR */


/*------------------------------------------------------------------------------
 * 6) DRAM memory map.
 *----------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
 * i) Base address for packet buffers. This is the actual packet data.
 *----------------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

#ifndef		BUF_SDRAM_BASE
#define		BUF_SDRAM_BASE		0
#endif

#endif /* USE_IMPORT_VAR */

/* Total DRAM size allocated in bytes for packet buffers.
   In simulation we use only 1280KB (320 buffers). */
#ifndef		BUF_SDRAM_SIZE
#ifdef 		IXP_SIMULATION
#define		BUF_SDRAM_SIZE		(640 * 1024)
#else
#define		BUF_SDRAM_SIZE		(64 * 1024 * 10 * 2 * 20)
#endif
#endif

/*	Size of one packet buffer in bytes.  */
#ifndef		BUFFER_SIZE
#define		BUFFER_SIZE			2048
#endif

/* Allocating 64 MB of DRAM for packet buffers. */


/*------------------------------------------------------------------------------
 * 7) Scratch and scratch ring memory map.
 *----------------------------------------------------------------------------*/

/*  There are two exception ring to send pkts up */
#ifndef		EXCEPTION_RING_OUT_0
#define		EXCEPTION_RING_OUT_0			0
#endif

#ifndef		EXCEPTION_RING_OUT_1
#define		EXCEPTION_RING_OUT_1			1
#endif

/*  There are two exception ring to send pkts down */
#ifndef		EXCEPTION_RING_IN_0
#define		EXCEPTION_RING_IN_0				11
#endif

#ifndef		EXCEPTION_RING_IN_1
#define		EXCEPTION_RING_IN_1				12
#endif


/*
 * Scratch ring for communicating between CSIX RX and Diffserv dispatch loop
 * Ring number 0, base 0, and size 512 long words
 */
#define			CSIX_TO_L2_ENCAP_SCR_RING		5
#define			CSIX_TO_L2_ENCAP_SCR_RING_BASE	0
#define			CSIX_TO_L2_ENCAP_SCR_RING_SIZE	512

/*
 * Third scractch ring for communicating between Queue Manager and Packet TX
 * microblocks Ring number 4, base 6144, and size 256 long words
 */
#define			L2_ENCAP_TO_QM_SCR_RING			6
#define			L2_ENCAP_TO_QM_SCR_RING_BASE	2048
#define			L2_ENCAP_TO_QM_SCR_RING_SIZE	512

/*
 * Scratch ring for communicating between Scheduler and Queue Manager Blocks
 * Ring number 1, base 2048, and size 512 long words
 */
#define			SCHEDULER_TO_QM_SCR_RING		7
#define			SCHEDULER_TO_QM_SCR_RING_BASE	4096
#define			SCHEDULER_TO_QM_SCR_RING_SIZE	512

/*
 * First scractch ring for communicating between Queue Manager and Packet TX
 * microblocks.	Ring number 2, base 4096, and size 256 long words
 */
#define 		QM_TO_PACKET_TX_SCR_RING_0		8
#define 		QM_TO_PACKET_TX_SCR_RING_0_BASE	6144
#define 		QM_TO_PACKET_TX_SCR_RING_0_SIZE	256

/*
 * Second scractch ring for communicating between Queue Manager and Packet TX
 * microblocks Ring number 3, base 5120, and size 256 long words
 */
#define 		QM_TO_PACKET_TX_SCR_RING_1		9
#define 		QM_TO_PACKET_TX_SCR_RING_1_BASE	7168
#define 		QM_TO_PACKET_TX_SCR_RING_1_SIZE	256

/*
 * Third scractch ring for communicating between Queue Manager and Packet TX
 * microblocks Ring number 4, base 6144, and size 256 long words
 */
#define			DIFFSERV_SCR_RING				10
#define			DIFFSERV_SCR_RING_BASE			8192
#define			DIFFSERV_SCR_RING_SIZE			512


/*
 * Scratch address required by Packet TX for updating available Transmit Buffers.
 */
#ifndef		PACKET_TX_AVAIL_TBUF_ELE_SCR_ADDR
#define 	PACKET_TX_AVAIL_TBUF_ELE_SCR_ADDR	14336
#endif 


/* These defines are used by the individual microcode to identify their
 * scratch rings. The reason for these defines is to improve the portability
 * of microblocks. User just has to make changes here and the microblocks will
 * remain untouched. For the naming convention, the microblock name goes first,
 * then the _RING_ specifier. Following this is the direction indicator.
 * '_OUT' would imply the microblock is writing to this scratch ring and
 * '_IN' would imply the microblock is reading from this scratch ring.
 * If there is more than one 'IN' or 'OUT' ring the name will have '_<digit>'
 * to differentiate the rings.
 */

#define 		CSIX_RX_RING_OUT				CSIX_TO_L2_ENCAP_SCR_RING
#define			CSIX_RX_RING_OUT_BASE			CSIX_TO_L2_ENCAP_SCR_RING_BASE
#define			CSIX_RX_RING_OUT_SIZE			CSIX_TO_L2_ENCAP_SCR_RING_SIZE

#define			DIFFSERV_RING_IN_0				CSIX_TO_L2_ENCAP_SCR_RING
#define			DIFFSERV_RING_IN_0_BASE			CSIX_TO_L2_ENCAP_SCR_RING_BASE
#define			DIFFSERV_RING_IN_0_SIZE			CSIX_TO_L2_ENCAP_SCR_RING_SIZE

#define			DIFFSERV_RING_OUT_0				DIFFSERV_SCR_RING
#define			DIFFSERV_RING_OUT_0_BASE		DIFFSERV_SCR_RING_BASE
#define			DIFFSERV_RING_OUT_0_SIZE		DIFFSERV_SCR_RING_SIZE

#define 		L2_ENCAP_SOURCE_SCR_RING		DIFFSERV_SCR_RING
#define 		L2_ENCAP_SOURCE_SCR_RING_BASE	DIFFSERV_SCR_RING_BASE
#define 		L2_ENCAP_SOURCE_SCR_RING_SIZE	DIFFSERV_SCR_RING_SIZE

#define			L2_ENCAP_SINK_SCR_RING			6 /* L2_ENCAP_TO_QM_SCR_RING */
#define			L2_ENCAP_SINK_SCR_RING_BASE		L2_ENCAP_TO_QM_SCR_RING_BASE
#define			L2_ENCAP_SINK_SCR_RING_SIZE		L2_ENCAP_TO_QM_SCR_RING_SIZE

#define			QM_RING_IN_0					L2_ENCAP_TO_QM_SCR_RING
#define			QM_RING_IN_0_BASE				L2_ENCAP_TO_QM_SCR_RING_BASE
#define			QM_RING_IN_0_SIZE				L2_ENCAP_TO_QM_SCR_RING_SIZE

#define			QM_RING_IN_1					SCHEDULER_TO_QM_SCR_RING
#define			QM_RING_IN_1_BASE				SCHEDULER_TO_QM_SCR_RING_BASE
#define			QM_RING_IN_1_SIZE				SCHEDULER_TO_QM_SCR_RING_SIZE

#define			QM_RING_OUT_0					QM_TO_PACKET_TX_SCR_RING_0
#define			QM_RING_OUT_0_BASE				QM_TO_PACKET_TX_SCR_RING_0_BASE
#define			QM_RING_OUT_0_SIZE				QM_TO_PACKET_TX_SCR_RING_0_SIZE

#define			QM_RING_OUT_1					QM_TO_PACKET_TX_SCR_RING_1	
#define			QM_RING_OUT_1_BASE				QM_TO_PACKET_TX_SCR_RING_1_BASE
#define			QM_RING_OUT_1_SIZE				QM_TO_PACKET_TX_SCR_RING_1_SIZE

#define			QM_RING_OUT_2					QM_TO_PACKET_TX_SCR_RING_2
#define			QM_RING_OUT_2_BASE				QM_TO_PACKET_TX_SCR_RING_2_BASE
#define			QM_RING_OUT_2_SIZE				QM_TO_PACKET_TX_SCR_RING_2_SIZE

#define			QM_RING_OUT_3					QM_TO_PACKET_TX_SCR_RING_3	
#define			QM_RING_OUT_3_BASE				QM_TO_PACKET_TX_SCR_RING_3_BASE
#define			QM_RING_OUT_3_SIZE				QM_TO_PACKET_TX_SCR_RING_3_SIZE


#define			SCHEDULER_RING_OUT				SCHEDULER_TO_QM_SCR_RING
#define			SCHEDULER_RING_OUT_BASE			SCHEDULER_TO_QM_SCR_RING_BASE			
#define			SCHEDULER_RING_OUT_SIZE			SCHEDULER_TO_QM_SCR_RING_SIZE

								
#define			PACKET_TX_IN_0					QM_TO_PACKET_TX_SCR_RING_0
#define			PACKET_TX_IN_0_BASE				QM_TO_PACKET_TX_SCR_RING_0_BASE
#define			PACKET_TX_IN_0_SIZE				QM_TO_PACKET_TX_SCR_RING_0_SIZE

#define 		PACKET_TX_IN_1					QM_TO_PACKET_TX_SCR_RING_1
#define			PACKET_TX_IN_1_BASE				QM_TO_PACKET_TX_SCR_RING_1_BASE
#define			PACKET_TX_IN_1_SIZE				QM_TO_PACKET_TX_SCR_RING_1_SIZE
								
#define			PACKET_TX_IN_2					QM_TO_PACKET_TX_SCR_RING_2
#define			PACKET_TX_IN_2_BASE				QM_TO_PACKET_TX_SCR_RING_2_BASE
#define			PACKET_TX_IN_2_SIZE				QM_TO_PACKET_TX_SCR_RING_2_SIZE

#define 		PACKET_TX_IN_3					QM_TO_PACKET_TX_SCR_RING_3
#define			PACKET_TX_IN_3_BASE				QM_TO_PACKET_TX_SCR_RING_3_BASE
#define			PACKET_TX_IN_3_SIZE				QM_TO_PACKET_TX_SCR_RING_3_SIZE


/*------------------------------------------------------------------------------
 * SOURCE & SINK definitions
 */

/*	specify if next neighbor ring is used. If not, default is scratch rings.
 *	NOTE: DL_RING_* is now obsolete. Will be changed to use CSIX_RX_RING_IN, 
 *	which is defined somwehere above.
 */

/* 	Ring number to use. 0-15. */
#define		 	DL_RING_NUMBER			CSIX_RX_RING_OUT


#define			DL_DROP_RING			5 /* CSIX_RX_RING_OUT */


/*------------------------------------------------------------------------------
 * 8) System-wide signals.
 *----------------------------------------------------------------------------*/

/* Signals being a precious resource, we use a single signal for 
 * inter-thread as well as any other inter-me signalling
 * (init over signal).
 */
#ifndef		COMMON_INTER_THD_ME_SIGNAL
#define		COMMON_INTER_THD_ME_SIGNAL		10
#endif

/* We are not able to use a single signal before of limitations in
 * assembler. Until then, we need a separate signal for inter-thread
 * signalling.
 */
#ifndef		INTER_THD_SIGNAL
#define		INTER_THD_SIGNAL				12
#endif

/* ME_INIT_SIGNAL signal is used to let all ME's know that system
 * initialization is over and all blocks in all MEs can start running.
 * This signal will be typically issued by the system initialization code
 * and all other MEs should wait for this signal before starting to run. 
 */
#ifndef		ME_INIT_SIGNAL
#define		ME_INIT_SIGNAL					COMMON_INTER_THD_ME_SIGNAL
#endif

/*
 * Signal number to use in inter thread signalling. 
 */
#ifndef		DL_SIG_WAKE
#define 	DL_SIG_WAKE						INTER_THD_SIGNAL
#endif


/*------------------------------------------------------------------------------
 * 9) System communication ID's and block ID's.
 *----------------------------------------------------------------------------*/

/* These tell if the executing threads need to work in order.
 * i.e wait for	a signal from prev thread, do some work and signal
 * next thread. */
#ifndef		DL_THREAD_NO_ORDER
#define		DL_THREAD_NO_ORDER	0x0
#endif

#ifndef		DL_THREAD_ORDER
#define		DL_THREAD_ORDER		0x1
#endif

/* All the IX_* constants should be between 0 and 255 (i.e 1 byte)
 * to enable using them as immed operands in instructions.
 *
 * IX_NULL is used to denote a NULL (or empty) packet handle. 
 * It should not be zero, as it would interfere with ring empty (0)
 * condition.
 * Note: IX_NULL is too generic a name and may clash with core
 *  components code. So it's marked for deletion. Start using UC_NULL.
 */
#ifndef		IX_NULL
#define		IX_NULL				0xFF
#endif

#ifndef		IX_EXCEPTION
#define		IX_EXCEPTION		0x1
#endif

#ifndef		IX_DROP
#define		IX_DROP				0x2
#endif

#ifndef		UC_NULL
#define		UC_NULL				0xFF		/* NULL for microcode */
#endif

#ifndef		DL_BUF_NULL
#define		DL_BUF_NULL			UC_NULL 	/* NULL buffer */
#endif

#ifndef		BID_NULL
#define		BID_NULL			UC_NULL		/* NULL block ID */
#endif

#ifndef		BID_EXCEPTION
#define		BID_EXCEPTION		0x1			/* Exception block ID */
#endif

#ifndef		BID_DROP
#define		BID_DROP			0x2			/* Drop block ID */
#endif


/* Block IDs for various blocks. Block IDs Start from 0x20.
 * (0x0 to 0x1f and 0xFF are reserved for IX_*).
 */
#ifndef		BID_TC_METER
#define		BID_TC_METER			IX_CC_TC_METER_PKT_MICROBLOCK_ID
#endif

#ifndef		BID_WRED
#define		BID_WRED				IX_CC_WRED_PKT_MICROBLOCK_ID
#endif

#ifndef		BID_QM
#define		BID_QM					IX_CC_QM_CORE_TO_MICROBLOCK_ID
#endif


/*------------------------------------------------------------------------------
 * 10) Static Bindings (a.k.a targets) for various microblocks.
 *----------------------------------------------------------------------------*/

#ifndef		DL_SOURCE_NEXT1
#define		DL_SOURCE_NEXT1		BID_TC_METER
#endif

#ifndef		DL_SOURCE_NEXT2
#define		DL_SOURCE_NEXT2		BID_WRED
#endif

#ifndef		TC_METER_NEXT1
#define		TC_METER_NEXT1		BID_QM
#endif

#ifndef		TC_METER_NEXT2
#define		TC_METER_NEXT2		IX_DROP
#endif

#ifndef		TC_METER_NEXT3
#define		TC_METER_NEXT3		IX_DROP
#endif

#ifndef		WRED_NEXT1
#define		WRED_NEXT1			BID_QM
#endif

#ifndef		WRED_NEXT2
#define		WRED_NEXT2			IX_DROP
#endif

/*
 * To be redefined as an import variable. And its value should be
 * one of the above block ID's.
 */

#ifndef		BID_NEXT_BLOCK
#define		BID_NEXT_BLOCK		IX_NULL					
#endif


#endif		 /*  DL_SYSTEM_H  */
