#ifndef DIFFSERV_PIPELINE_UC
#define DIFFSERV_PIPELINE_UC

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

// FILE:  diffserv_pipeline.uc
//

#define      USE_DL_THREAD_ID
#define      FLUSH_EXCP_PKT_HDR

// read ipv4 header 6 longwords
#define	     PKT_HDR_CACHE_SIZE	 8

//	Include Files
#include    <diffserv.h>

// include system.h from dispatch loop
#include	"dl_system.h"

#include	"definitions.h"

// include system_init.h from dispatch loop for inter ME signal
#include	"system_init.uc"

// include dispatch_loop.uc from dispatch loop 
#include	"dispatch_loop.uc"

// include ipv4_fwder_dl.uc from IPV4 forwarder 
#include	<ipv4_fwder_init.uc>

// include ipv4_fwder.uc from IPV4 forwarder 
#include	<ipv4_fwder.uc>

// include ppp.uc from rx
#include    <classifier_6t.uc>
#include	<dscp_classifier.uc>
#include    <dscp_marker.uc>
#include 	<tc_meter.uc>
#include 	<ppp.uc>


//-----------------------------------------------------------
// Initialize the blocks
//-----------------------------------------------------------


init#:
	// allocate cache to store the packet header
	dl_iphdr_cache_init[]

	dl_source_init[]

	classifier_6t_init[]

	tc_meter_init[]


	; init buffer meta data cache to Zero.
	init_meta_cache_to_zero[]

	; Temporary : Initialize EOP handle default.
	_dl_eop_buf_handle_init[]

	; ipv4fwder initialization
	_ipv4_fwder_init()

	; now kick start the IPV4 Forwarder
	_ipv4_fwder_kick_start()

	.if (ctx()==7)
	    signal_first_ctx(TC_METER_NEXT_THREAD_SIG)
	.endif


//-----------------------------------------------------------
// dispatch loop
//-----------------------------------------------------------

loop#:

	_ppp_decap_classify($$iphdr, PPP_HEADER_OFFSET)

	dscp_classifier($$iphdr, IP_HDR_OFFSET)

	classifier_6t($$iphdr, IP_HDR_OFFSET)

	tc_meter()

	;process the IP header. The modified packet is in cache
	;registers starting at IP_HDR_OFFSET.
	;its enough for us to have 7 elements in this table not 8

	xbuf_alloc(iphdr, 6, read_write)

	dscp_marker(iphdr, $$iphdr, IP_HDR_OFFSET, IP_HDR_OFFSET)

	_ipv4_fwder($$iphdr, iphdr, IP_HDR_OFFSET, IP_HDR_OFFSET)

	// The above macro copies only 20 bytes of IP header to the
	// destination register buffer. we need to copy the registers untouched
	// by the above macro. The registers modified is dependent on
	// where the IP Hdr starts in the buffer. 

	#if	(IP_HDR_OFFSET == 4)
	// copy untouched registers
    alu[$$iphdr[0], --, B, iphdr[0]]
 	#endif

	alu[$$iphdr[6], --, B, $$iphdr[6]]
	alu[$$iphdr[7], --, B, $$iphdr[7]]

	xbuf_free(iphdr)

	// Now call DL macros to send the current packet
	// and received the next packet. Depending on the dl_next_block, these
	// macros can drop/send exception/send to next block. Also, we need to
	// maintain packet order when we send/receive packets.

	.reg sig_mask
	.sig sig_scr_put
	.set_sig sig_scr_put

	// allocate xfer registers to be used by dl_qm_sink.
	xbuf_alloc[$wxfer, 3, write]

	immed32[sig_mask, 0]

	// send packet.
	dl_qm_sink[sig_mask, sig_scr_put, $wxfer]

	// receive packet
	dl_source[DL_THREAD_NO_ORDER, sig_mask]

	.io_completed sig_scr_put

	// free the xfer registers allocated
	xbuf_free[$wxfer]

	// start again.
	br[loop#]
	
	// We never execute the below code. This is here for completeness only.

term#:

	// frees the IP header cache
	dl_iphdr_cache_fini[]

	classifier_6t_term[]

    tc_meter_term[]

	// Control should never come here.
kill#:

	nop
	nop

#endif
