#ifndef DL_SOURCE_UC
#define DL_SOURCE_UC

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

// FILE  dl_source.uc
//

#include	<dl_msg_if_default.uc>


#ifdef	EXCEPTION_HANDLING_ME
// signal used to read data from CC->ME exception ring
.sig global volatile exc_sig 
 xbuf_alloc[$$exc_data, 3, read]  // data from CC->ME exception ring
#endif 



#macro dl_source_init[]
	dl_rx_from_pkt_rx_init_default[]
    #ifdef	EXCEPTION_HANDLING_ME
        .reg src_init_signal
        alu[src_init_signal, --, B, &exc_sig, <<3]
		alu[src_init_signal, src_init_signal, OR, 7]
        signal_ctx(src_init_signal)
    #endif
    ; Note IX_NULL is NOT ZERO.
    move[dl_buf_handle, IX_NULL]
	move[dl_next_block, IX_NULL]
	#define_eval i 0
	#while (i < PKT_HDR_CACHE_SIZE)
		.set $$iphdr[i]
		#define_eval i (i + 1)
	#endloop
#endm



#macro dl_sink_init[]
	dl_pkt_proc_to_qm_sink_init_default[]
#endm



#macro dl_source[THREAD_ORDER, sig_mask]

#ifdef EXCEPTION_HANDLING_ME
	br!=ctx[7, standard_path#]	; Special case for thread 7.

    // Thread 7 checks exception ring from CORE
    .begin
        .reg exc_ring 
        alu[exc_ring, --,b, EXCEPTION_RING_IN_0, <<2]

        // Check if data from exception scratch ring has already been read.
        br_!signal[exc_sig, standard_path#]

        // We check transfer registers in the next loop, so .set must be used 
    	// because of warning (level 3) 4701.
        .set $$exc_data[0]
        .set $$exc_data[1]
        .set $$exc_data[2]

	    alu[--, $$exc_data[0], -, 0]
    	beq[exc_bypass#]

        // Thread 7, So Signal Next ME
    	signal_me[DL_NEXT_ME, DL_SIG_WAKE]

        // Read meta data
    	.sig signal_meta_exc
    	.reg $meta_exc0 $meta_exc1 $meta_exc2 $meta_exc3
    	.reg $meta_exc4 $meta_exc5 $meta_exc6 $meta_exc7
	    .xfer_order $meta_exc0 $meta_exc1 $meta_exc2 $meta_exc3 \
	                $meta_exc4 $meta_exc5 $meta_exc6 $meta_exc7

    	immed[dl_exception_reg, 0]


    	move(dl_buf_handle,     $$exc_data[0])
    	move(dl_eop_buf_handle, $$exc_data[1])
	    move(dl_next_block,     $$exc_data[2])

        dl_meta_load_cache($$exc_data[0], $meta_exc, signal_meta_exc, 0, 8)

	    //	Add the signal to the signal mask on which to wait
    	alu_shf[sig_mask, sig_mask, OR, 1, <<(&signal_meta_exc)]
    	local_csr_wr[active_ctx_wakeup_events, sig_mask]
    	ctx_arb[--]

        .io_completed signal_meta_exc

    	dl_meta_init_cache[$meta_exc[0], $meta_exc[1], $meta_exc[2], $meta_exc[3], \
	                       $meta_exc[4], $meta_exc[5], $meta_exc[6], $meta_exc[7]]

    	// Read the packet header and cache it in local memory
        .reg dl_packet_offset
        .sig	sig_ip
    	dl_meta_get_offset(dl_packet_offset)
       	dl_load_iphdr_cache[dl_buf_handle, sig_ip, sig_ip]

        #define_eval i 0
        #while (i < PKT_HDR_CACHE_SIZE)
            .set $$iphdr[i]
            #define_eval i (i + 1)
        #endloop

        // Get data from scratch ring.
    	// We will check data in the next loop
    	scratch[get, $$exc_data[0], 0, exc_ring, 3], sig_done[exc_sig]

        br[nothing_to_do#]

    exc_bypass#:

        // Get data from scratch ring.
    	// We will check data in the next loop
    	scratch[get, $$exc_data[0], 0, exc_ring, 3], sig_done[exc_sig]
	    // For now execute fast path forwarding

    .end

#endif /* EXCEPTION_HANDLING_ME */



standard_path#:
	dl_rx_from_pkt_rx_default[THREAD_ORDER, sig_mask]

nothing_to_do#:

#endm






#macro dl_sink[wxfer_prefix, req_sig, sig_mask]
	dl_pkt_rx_sink_default[wxfer_prefix, req_sig, sig_mask]
#endm


#macro dl_qm_sink[sig_mask, sig_scr, wxfer]

    // save only 6 longwords
    #define_eval  SAVE_PKT_HDR_CACHE_SIZE (PKT_HDR_CACHE_SIZE)
	#undef   PKT_HDR_CACHE_SIZE
    #undef   PKT_HDR_DRAM_RD_SIZE
    #define	      PKT_HDR_CACHE_SIZE       6
	#define_eval  PKT_HDR_DRAM_RD_SIZE    (PKT_HDR_CACHE_SIZE >> 1)

	dl_pkt_proc_to_qm_sink_default[sig_mask, sig_scr, wxfer]

	#undef   PKT_HDR_CACHE_SIZE
    #undef   PKT_HDR_DRAM_RD_SIZE
    #define_eval  PKT_HDR_CACHE_SIZE      (SAVE_PKT_HDR_CACHE_SIZE)
	#define_eval  PKT_HDR_DRAM_RD_SIZE    (PKT_HDR_CACHE_SIZE >> 1)
	#undef   SAVE_PKT_HDR_CACHE_SIZE

#endm


#endif	// DL_SOURCE_UC