#ifndef	__DL_SYSTEM_H__
#define	__DL_SYSTEM_H__

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/
/* "$Id: dl_system.h,v 1.21.2.1 2004/03/12 11:00:02 wneubaue Exp $" */

/* FILE  dl_system.h
 *
 * This file contains the system wide parameters that can be 
 * customised for a given hardware/application
 */


#ifndef		USE_IMPORT_VAR
#define		SIMULATION
#endif


/*
 *******************************************************************************
 * Include global configuration files
 */

#ifdef IXP_MICROCODE
#include <ix_cc_microengines_bindings.h>
#endif

#include <system.h>
#include <tc_meter_defs.h>
#include <classifier_6t_defs.h>


/*
 *******************************************************************************
 * C A U T I O N
 *
 * The code below may not be changed by the user. If any of the below stuff
 * has to be overwritten, redefine it in a separate file and then include this
 * file.
 *******************************************************************************
*/

/*------------------------------------------------------------------------------
 * 1) Miscellaneous settings.
 *----------------------------------------------------------------------------*/

/* PHY mode for RX device. */
#ifndef		RX_PHY_MODE
#define 	RX_PHY_MODE 		SPHY_1_32
#endif

/* Default size for PKT HDR to be cached in long words. */
#ifndef		PKT_HDR_CACHE_SIZE
#define		PKT_HDR_CACHE_SIZE	8
#endif

#ifndef		USE_IMPORT_VAR

/* This defines the number of queues/class per port */
#ifndef		NUM_CLASS_BITS
#define 	NUM_CLASS_BITS		4
#endif

/* Freelist id that is stored in meta data */
#ifndef		FREE_LIST_ID
#define		FREE_LIST_ID		0x0
#endif

/* This blade's id */
#ifndef		THIS_BLADE_ID
#define		THIS_BLADE_ID		1
#endif

#endif /* USE_IMPORT_VAR */

#ifdef		USE_IMPORT_VAR

/* Patched by core */
.import_var DL_REL_BASE
.import_var BUF_FREE_LIST0
.import_var BUF_SRAM_BASE

#endif /* USE_IMPORT_VAR */





/*------------------------------------------------------------------------------
 * 2) Microblock specific settings.
 *----------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
 * i) Packet Rx microblock.
 *----------------------------------------------------------------------------*/

/* Base address for Packet Rx contexts in local memory. */
#ifndef		POS_RX_CTX_LM_BASE
#define		POS_RX_CTX_LM_BASE		0x0
#endif

/* Size of each Packet Rx context in bytes. */
#ifndef		POS_RX_CTX_SIZE
#define		POS_RX_CTX_SIZE			64
#endif

/* Total number of Packet Rx contexts. */
#ifndef		POS_RX_CTX_TOTAL
#define		POS_RX_CTX_TOTAL		16
#endif


/*------------------------------------------------------------------------------
 * iii) Queue manager microblock.
 *----------------------------------------------------------------------------*/

/* Size of each queue descriptor entry in bytes. */
#ifndef		QD_SIZE
#define		QD_SIZE					16
#endif

/* Queue number for drop queue. (0-1023 are the normal queues used
   in forwarding pkts) */
#ifndef		QM_DROP_QUEUE
#define		QM_DROP_QUEUE			1024
#endif


/*------------------------------------------------------------------------------
 * iv) CSIX TX microblock.
 *----------------------------------------------------------------------------*/

/*	Size of each CSIX TX context in bytes.  */
#ifndef		CSIX_TX_CTX_SIZE
#define		CSIX_TX_CTX_SIZE		64
#endif

/*	Total number of CSIX TX contexts. */
#ifndef		CSIX_TX_CTX_TOTAL
#define		CSIX_TX_CTX_TOTAL		1024
#endif


/*------------------------------------------------------------------------------
 * v) 6-tuple Classifier microblock.
 *----------------------------------------------------------------------------*/

#ifndef USE_IMPORT_VAR
   /*
    *  Base address in sram for the hash table
    */
	#define   CLASSIFIER_6T_HASH_TABLE_BASE           (BUF_SRAM_BASE +         \
	                                                  BUF_SRAM_SIZE)
    #define   CLASSIFIER_6T_COLLISION_ENTRIES         32
    #define   CLASSIFIER_6T_HASH_TABLE_SIZE                                    \
	                                    (CLASSIFIER_6T_HASH_TABLE_ENTRY_SIZE * \
                                        (CLASSIFIER_6T_COLLISION_ENTRIES +     \
										(1 << CLASSIFIER_6T_HASH_INDEX_SIZE)))

	#define   CLASSIFIER_6T_DEFAULT_FLOW_ID           0x50
	#define   CLASSIFIER_6T_DEFAULT_COLOR_ID          0x1
	#define   CLASSIFIER_6T_DEFAULT_CLASS_ID          0x8
	#define   CLASSIFIER_6T_DEFAULT_NEXTHOP_ID        0xFFFF
	#define   CLASSIFIER_6T_DEFAULT_NEXTHOP_ID_TYPE   0x0

#endif


#ifndef   USE_IMPORT_VAR
#ifdef	  CLASSIFIER_6T_PACKET_COUNTER_FEATURE

	#define	  CLASSIFIER_6T_STATS_TABLE_SIZE                                   \
                                    (CLASSIFIER_6T_STATISTICS_ENTRY_SIZE *     \
                                    (CLASSIFIER_6T_COLLISION_ENTRIES +         \
                                    (1 << CLASSIFIER_6T_HASH_INDEX_SIZE)))

	#define   CLASSIFIER_6T_STATS_SRAM_BASE                                    \
                                    (CLASSIFIER_6T_HASH_TABLE_BASE +           \
				                    CLASSIFIER_6T_HASH_TABLE_SIZE)

	#define   CLASSIFIER_6T_DEFAULT_STATS_ADDRESS                              \
                                    (CLASSIFIER_6T_STATS_SRAM_BASE +           \
                                    CLASSIFIER_6T_STATS_TABLE_SIZE)
#endif
#endif



/*------------------------------------------------------------------------------
 * vi) TC_METER microblock.
 *----------------------------------------------------------------------------*/

/*
 * Amount of SDRAM space taken by meter table
 * in simulation we use 255 entries
 */
#define   TC_METER_TABLE_SRAM_SIZE  		   (TC_METER_ENTRY_SRAM_SIZE * 0xFF)


/* Base address in sram for the meter table  */
#ifndef	  USE_IMPORT_VAR
#ifdef    TC_METER_PACKET_COUNTER_FEATURE
#ifdef    CLASSIFIER_6T_PACKET_COUNTER_FEATURE
#define   TC_METER_TABLE_SRAM_BASE      (CLASSIFIER_6T_DEFAULT_STATS_ADDRESS + \
                                         CLASSIFIER_6T_STATISTICS_ENTRY_SIZE)
#else
#define   TC_METER_TABLE_SRAM_BASE      (CLASSIFIER_6T_HASH_TABLE_BASE +       \
                                         CLASSIFIER_6T_HASH_TABLE_SIZE)
#endif
#endif
#endif


#ifndef	USE_IMPORT_VAR
#ifdef  TC_METER_PACKET_COUNTER_FEATURE

	/* 
	 * defines the SDRAM space taken by meter's 64 bit supplementary statistics
	 */
	#define   TC_METER_64BIT_STAT_TABLE_SRAM_SIZE  (0xFF * \
                                                  TC_METER_64BIT_STAT_SRAM_SIZE)
	/* 
	 * defines the base address of the table where the most significant 
	 * long words of meter statistics are held.
	 */
	#define   TC_METER_64BIT_STAT_SRAM_BASE       (TC_METER_TABLE_SRAM_BASE +  \
                                                   TC_METER_TABLE_SRAM_SIZE)
#endif
#endif


/*
 * signal used to synchronize meter threads on particular microengines
 */
#define     TC_METER_NEXT_THREAD_SIG        13

/*
 * signal used to initialize timestamp on every timestamp
 */
#define     TC_METER_TIMESTAMP_SIG          11

/*
 * signal used to synchronize meter crtitical sections 
 * on different microengines
 */
#define     TC_METER_NEXT_ME_SIG            15


/* used to initialize TC_METER meter signals */
#define     FIRST_TC_METER_ME			    0x01


/*------------------------------------------------------------------------------
 * vii) DSCP Marker microblock.
 *----------------------------------------------------------------------------*/

#define     DSCP_MARKER_IPV4_PACKET 		PPP_IPV4_TYPE
#define     DSCP_MARKER_IPV6_PACKET 		PPP_IPV6_TYPE
#define     DSCP_MARKER_MPLS_PACKET 		0x03


/*------------------------------------------------------------------------------
 * viii) DSCP Classifier microblock.
 *----------------------------------------------------------------------------*/

#define     CLASSIFIER_DSCP_IPV4_HEADER_TYPE 		PPP_IPV4_TYPE
#define     CLASSIFIER_DSCP_IPV6_HEADER_TYPE 		PPP_IPV6_TYPE

#ifndef USE_IMPORT_VAR

	#define CLASSIFIER_DSCP_CONFIG_SRAM_BASE           0xA800
	#define CLASSIFIER_DSCP_CONFIG_ENTRY_SIZE          0x8
	#define CLASSIFIER_DSCP_MAX_NUMBER_OF_PORTS        256
	#define CLASSIFIER_DSCP_NUMBER_OF_CONFIG_ENTRIES   (CLASSIFIER_DSCP_MAX_NUMBER_OF_PORTS << 6)
	#define CLASSIFIER_DSCP_CONFIG_SRAM_SIZE           (CLASSIFIER_DSCP_CONFIG_ENTRY_SIZE * \
													CLASSIFIER_DSCP_NUMBER_OF_CONFIG_ENTRIES)

	#define CLASSIFIER_DSCP_STATS_ENTRY_SIZE       0x10
    #define CLASSIFIER_DSCP_STATS_SRAM_SIZE        (CLASSIFIER_DSCP_STATS_ENTRY_SIZE * \
													CLASSIFIER_DSCP_NUMBER_OF_CONFIG_ENTRIES)
	#ifdef CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE
	    #define CLASSIFIER_DSCP_STATS_SRAM_BASE        (CLASSIFIER_DSCP_CONFIG_SRAM_BASE + \
	                                                    CLASSIFIER_DSCP_STATS_SRAM_SIZE)
	#endif

#endif /* USE_IMPORT_VAR */


/*------------------------------------------------------------------------------
 * 3) SRAM memory map for channel 0.
 *----------------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

 /* Area of SRAM which is zero'ed out. */
#ifndef		SRAM_ZERO_BLOCK
#define		SRAM_ZERO_BLOCK 	0x0
#endif

#endif /* USE_IMPORT_VAR */


/*------------------------------------------------------------------------------
 * i) Base address for meta data (buffer descriptors) of packet
 *    buffers.
 *----------------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

#ifndef		BUF_SRAM_BASE
#define		BUF_SRAM_BASE		4 /* first nonzero available address */
#endif

#endif /* USE_IMPORT_VAR */

/* Total SRAM size allocated in bytes for meta data.
   In simulation we use only 10KB. */
#ifndef		BUF_SRAM_SIZE
#ifdef 		IXP_SIMULATION
#define		BUF_SRAM_SIZE		(1024 * 10)
#else
#define		BUF_SRAM_SIZE		(1024 * 10 * 20)
#endif
#endif

/* Size of meta-data (buffer descriptor) in bytes for each buffer.
   Should be a power of 2. */
#ifndef		META_DATA_SIZE
#define		META_DATA_SIZE		32
#endif

/* Allocating 1 MB of SRAM for meta-data. */


/*------------------------------------------------------------------------------
 * ii) Base address for queue descriptors.
 *----------------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

#ifndef		QD_SRAM_BASE
#define		QD_SRAM_BASE		0x100000
#endif

/* Total number of queue descriptors. */
#ifndef		QD_TOTAL
#define		QD_TOTAL			1024
#endif

/* Allocating 1 MB of SRAM for queue descriptors. */

#endif /* USE_IMPORT_VAR */


/*------------------------------------------------------------------------------
 * iii) Base address for CSIX TX contexts.
 *----------------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

#ifndef		CSIX_TX_CTX_SRAM_BASE
#define		CSIX_TX_CTX_SRAM_BASE	0x200000
#endif

/* Allocating 1 MB of SRAM for CSIX TX contexts. */

#endif /* USE_IMPORT_VAR */


/*------------------------------------------------------------------------------
 * iv) Base address for next hop information.
 *----------------------------------------------------------------------------*/

/* For better DRAM bank utilization, we put the nexthop info in
 * SRAM at this address. We also have a segment in DRAM reserved
 * to keep this info, should we decide to move back to DRAM. This
 * would be a compile time option for IPV4 forwarder.
 */

#ifndef		USE_IMPORT_VAR

#ifndef		NEXTHOP_TABLE_SRAM_BASE
#define		NEXTHOP_TABLE_SRAM_BASE		0x300000
#endif

/*	Each next hop entry is 16 bytes. So 4KB is sufficient */
#ifndef		NEXTHOP_TABLE_SIZE
#define		NEXTHOP_TABLE_SIZE			0x1000
#endif

#endif /* USE_IMPORT_VAR */


/*------------------------------------------------------------------------------
 * 4) SRAM memory map for channel 1.
 *----------------------------------------------------------------------------*/

/*------------------------------------------------------------------------------
 * i) Base address for trie table.
 *----------------------------------------------------------------------------*/

#ifndef		USE_IMPORT_VAR

#ifndef		TRIE_TABLE_SRAM_BASE
#define		TRIE_TABLE_SRAM_BASE	0x40000000
#endif

/*	Trie table size. 2MB */
#ifndef		TRIE_TABLE_SIZE
#define		TRIE_TABLE_SIZE			0x200000
#endif

#endif /* USE_IMPORT_VAR */


/*------------------------------------------------------------------------------
 * ii) Base address for IPv4 counters.
 *----------------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

#ifdef 		IPV4_COUNTERS_SCRATCH

#ifndef		IPV4_STATS_TABLE_BASE
#define		IPV4_STATS_TABLE_BASE			11264
#endif

/* Space allocated to maintain these counters */
#ifndef		IPV4_STATS_TABLE_SIZE
#define		IPV4_STATS_TABLE_SIZE			0x800
#endif

#else

/* Counters in SRAM	*/
#ifndef		STATS_TABLE_SRAM_BASE
#define		STATS_TABLE_SRAM_BASE			0x40200000
#endif

/* Space allocated to maintain these counters */
#ifndef		STATS_TABLE_SRAM_SIZE
#define		STATS_TABLE_SRAM_SIZE			0x800
#endif

#endif  /* IPV4_COUNTERS_SCRATCH */

#endif /* USE_IMPORT_VAR */


/*------------------------------------------------------------------------------
 * iii) Base address for POS RX counters.
 *----------------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

/* Base for POS RX counters */
#ifndef		PACKET_COUNTERS_SRAM_BASE
#define		PACKET_COUNTERS_SRAM_BASE	0x40200800
#endif

#endif /* USE_IMPORT_VAR */


/*------------------------------------------------------------------------------
 * iv) Base address for directed broadcast table.
 *----------------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

#ifndef		DBCAST_TABLE_SRAM_BASE
#define		DBCAST_TABLE_SRAM_BASE		0x40201000
#endif

#ifndef		DBCAST_TABLE_SIZE
#define		DBCAST_TABLE_SIZE			0x2000
#endif

#endif /* USE_IMPORT_VAR */


/*------------------------------------------------------------------------------
 * v) Base address for control block table.
 *----------------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

#ifndef		CONTROL_BLOCK_SRAM_BASE
#define		CONTROL_BLOCK_SRAM_BASE		0x40203000
#endif

#endif /* USE_IMPORT_VAR */


/*------------------------------------------------------------------------------
 * vi) Base address for CSIX TX counters.
 *----------------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

#ifndef		CSIX_TX_COUNTERS_SRAM_BASE
#define   	CSIX_TX_COUNTERS_SRAM_BASE   	0x40300000
#endif

#endif /* USE_IMPORT_VAR */


/*------------------------------------------------------------------------------
 * 5) SRAM Q-array memory map.
 *----------------------------------------------------------------------------*/

/* SRAM q-array entries 0-15 are used by the queue manager to cache queues */
#ifndef		QM_QARRAY_BASE
#define		QM_QARRAY_BASE			0
#endif

#ifndef		QM_QARRAY_SIZE
#define		QM_QARRAY_SIZE			16
#endif

#ifndef		USE_IMPORT_VAR

/* SRAM q-array entries allocated for buffer free list. */
#ifndef		BUF_QARRAY_BASE
#define		BUF_QARRAY_BASE			16
#endif

#ifndef		BUF_QARRAY_SIZE
#define		BUF_QARRAY_SIZE			4
#endif

/* SRAM q-array entry allocated for drop queue in queue manager. */
#ifndef		QM_DROP_QUEUE_ENTRY
#define		QM_DROP_QUEUE_ENTRY		20
#endif

#endif /* USE_IMPORT_VAR */


/*------------------------------------------------------------------------------
 * 6) DRAM memory map.
 *----------------------------------------------------------------------------*/

/* The DDR (Double Data Rate) DRAM bank size is used to randomize
   buffer offset. */
#ifndef		DDR_BANK_SIZE
#define		DDR_BANK_SIZE			128
#endif

/* Number of DDR banks. */
#ifndef		NO_OF_DDR_BANKS
#define		NO_OF_DDR_BANKS			4
#endif

#define		MAX_BUFFER_OFFSET		(DDR_BANK_SIZE * NO_OF_DDR_BANKS)


/*------------------------------------------------------------------------------
 * i) Base address for packet buffers. This is the actual packet data.
 *----------------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

#ifndef		BUF_SDRAM_BASE
#define		BUF_SDRAM_BASE		0
#endif

#endif /* USE_IMPORT_VAR */

/* Total DRAM size allocated in bytes for packet buffers.
   In simulation we use only 640KB. */
#ifndef		BUF_SDRAM_SIZE
#ifdef 		IXP_SIMULATION
#define		BUF_SDRAM_SIZE		(64 * 1024 * 10)
#else
#define		BUF_SDRAM_SIZE		(64 * 1024 * 10 * 20)
#endif
#endif

/*	Size of one packet buffer in bytes.  */
#ifndef		BUFFER_SIZE
#define		BUFFER_SIZE			2048
#endif

/* Allocating 64 MB of DRAM for packet buffers. */


/*------------------------------------------------------------------------------
 * ii) Base address for next hop table. See comment under next hop
 *     table maintained in SRAM.
 *----------------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

#ifndef		NEXTHOP_TABLE_SDRAM_BASE
#define		NEXTHOP_TABLE_SDRAM_BASE	0x1000000
#endif

#endif /* USE_IMPORT_VAR */


/*------------------------------------------------------------------------------
 * 7) Scratch and scratch ring memory map.
 *----------------------------------------------------------------------------*/

/*  There are two exception ring to send pkts up */
#ifndef		EXCEPTION_RING_OUT_0
#define		EXCEPTION_RING_OUT_0			0
#endif

#ifndef		EXCEPTION_RING_OUT_1
#define		EXCEPTION_RING_OUT_1			1
#endif

/*  There are two exception ring to send pkts down */
#ifndef		EXCEPTION_RING_IN_0
#define		EXCEPTION_RING_IN_0				2
#endif

#ifndef		EXCEPTION_RING_IN_1
#define		EXCEPTION_RING_IN_1				3
#endif


/*	Scratch for communicating between POS RX and IPv4 Blocks */
#ifndef		POS_RX_TO_IPV4_SCR
#define		POS_RX_TO_IPV4_SCR
#define		POS_RX_TO_IPV4_SCR_RING			4
#define		POS_RX_TO_IPV4_SCR_RING_BASE	0
#define		POS_RX_TO_IPV4_SCR_RING_SIZE	1024
#endif

/*	Scratch for communicating between IPV4 and Queue Manager Blocks */
#ifndef		IPV4_TO_QM_SCR
#define		IPV4_TO_QM_SCR
#define		IPV4_TO_QM_SCR_RING				5
#define		IPV4_TO_QM_SCR_RING_BASE		4096
#define		IPV4_TO_QM_SCR_RING_SIZE		512
#endif

/*	Scratch for communicating between Scheduler and Queue Manager Blocks */
#ifndef		SCHEDULER_TO_QM_SCR
#define		SCHEDULER_TO_QM_SCR
#define		SCHEDULER_TO_QM_SCR_RING		6
#define		SCHEDULER_TO_QM_SCR_RING_BASE	6144
#define		SCHEDULER_TO_QM_SCR_RING_SIZE	512
#endif

/*	Scratch for communicating between Queue Manager and CSIX Transmit Blocks */
#ifndef		QM_TO_CSIX_TX_SCR
#define		QM_TO_CSIX_TX_SCR
#define		QM_TO_CSIX_TX_SCR_RING			7
#define		QM_TO_CSIX_TX_SCR_RING_BASE		8192
#define		QM_TO_CSIX_TX_SCR_RING_SIZE		512
#endif


/* These defines are used by the individual microcode to identify their scratch rings.
 * The reason for these defines is to improve the portability of microblocks. User just
 * has to make changes here and the microblocks will remain untouched.
 * For the naming convention, the microblock name goes first, then the _RING_ specifier.
 * Following this is the direction indicator. '_OUT' would imply the microblock is writing
 * to this scratch ring and '_IN' would imply the microblock is reading from this scratch
 * ring. If there is more than one 'IN' or 'OUT' ring the name will have '_<digit>' to
 * differentiate the rings.
 *
 * An '_IND' at the end indicates the compilation switch.
 */


/*	The Ring on which POS RX sends out msg to next block */
#ifndef		POS_RX_RING_OUT_IND
#define		POS_RX_RING_OUT_IND
#define 	POS_RX_RING_OUT				POS_RX_TO_IPV4_SCR_RING
#define		POS_RX_RING_OUT_BASE		POS_RX_TO_IPV4_SCR_RING_BASE
#define		POS_RX_RING_OUT_SIZE		POS_RX_TO_IPV4_SCR_RING_SIZE
#endif

/*	The input ring ring on which IPV4 receives msgs from prev block */
#ifndef		IPV4_RING_IN_IND
#define		IPV4_RING_IN_IND
#define		IPV4_RING_IN				POS_RX_TO_IPV4_SCR_RING
#define		IPV4_RING_IN_BASE			POS_RX_TO_IPV4_SCR_RING_BASE
#define		IPV4_RING_IN_SIZE			POS_RX_TO_IPV4_SCR_RING_SIZE
#endif

/* The ring on which IPV4 sends out msg to next block (Queue Manager in this case) */
#ifndef		IPV4_RING_OUT_IND
#define		IPV4_RING_OUT_IND
#define		IPV4_RING_OUT				IPV4_TO_QM_SCR_RING
#define		IPV4_RING_OUT_BASE			IPV4_TO_QM_SCR_RING_BASE
#define		IPV4_RING_OUT_SIZE			IPV4_TO_QM_SCR_RING_SIZE
#endif

/* The input ring on which QM receives msgs from prev. block */
#ifndef		QM_RING_IN_IND
#define		QM_RING_IN_IND
#define		QM_RING_IN					IPV4_TO_QM_SCR_RING
#define		QM_RING_IN_BASE				IPV4_TO_QM_SCR_RING_BASE
#define		QM_RING_IN_SIZE				IPV4_TO_QM_SCR_RING_SIZE
#endif

/*	The output ring in which QM Sends msgs to next block (CSIX TX in this case) */
#ifndef		QM_RING_OUT_IND
#define		QM_RING_OUT_IND
#define		QM_RING_OUT					QM_TO_CSIX_TX_SCR_RING
#define		QM_RING_OUT_BASE			QM_TO_CSIX_TX_SCR_RING_BASE
#define		QM_RING_OUT_SIZE			QM_TO_CSIX_TX_SCR_RING_SIZE
#endif

/*	The input ring in which CSIX TX receives msgs from QM */
#ifndef		CSIX_TX_RING_IN_IND
#define		CSIX_TX_RING_IN_IND
#define		CSIX_TX_RING_IN				QM_TO_CSIX_TX_SCR_RING
#define		CSIX_TX_RING_IN_BASE		QM_TO_CSIX_TX_SCR_RING_BASE
#define		CSIX_TX_RING_IN_SIZE		QM_TO_CSIX_TX_SCR_RING_SIZE
#endif


/* NOTE: DL_RING_* is now obsolete. Will be changed to use
 * POS_RX_RING_IN, which is defined above.
 */
#ifdef IXP_MICROCODE

/* Ring number to use. 0-15. */
#ifndef			DL_RING_NUMBER
#define_eval 	DL_RING_NUMBER		POS_RX_TO_IPV4_SCR_RING
#endif

#ifndef			DL_DROP_RING
#define_eval	DL_DROP_RING		IPV4_TO_QM_SCR_RING
#endif

#else	/* Used for micro-c */

/* Ring number to use. 0-15. */
#ifndef			DL_RING_NUMBER
#define		 	DL_RING_NUMBER		POS_RX_TO_IPV4_SCR_RING
#endif

#endif	/* IXP_MICROCODE */

/* Valid values for RING_SIZE are 128, 256, 512 or 1024. */
#ifndef			DL_RING_SIZE
#define 		DL_RING_SIZE		POS_RX_TO_IPV4_SCR_RING_SIZE
#endif

/* Base Address for Scratch Ring is 0x1000. Should be aligned
 * on a 4 x RING_SIZE byte boundary.
 */
#ifndef			DL_RING_BASE
#define 		DL_RING_BASE		POS_RX_TO_IPV4_SCR_RING_BASE
#endif


/*------------------------------------------------------------------------------
 * 8) System-wide signals.
 *----------------------------------------------------------------------------*/

/* Signals being a precious resource, we use a single signal for
 * inter-thread as well as any other inter-me signalling
 * (init over signal).
 */
#ifndef		COMMON_INTER_THD_ME_SIGNAL
#define		COMMON_INTER_THD_ME_SIGNAL		10
#endif

/* We are not able to use a single signal before of limitations in
 * assembler. Until then, we need a separate signal for inter-thread
 * signalling.
 */
#ifndef		INTER_THD_SIGNAL
#define		INTER_THD_SIGNAL				12
#endif

/* ME_INIT_SIGNAL signal is used to let all ME's know that system
 * initialization is over and all blocks in all MEs can start running.
 * This signal will be typically issued by the system initialization code
 * and all other MEs should wait for this signal before starting to run.
 */
#ifndef		ME_INIT_SIGNAL
#define		ME_INIT_SIGNAL					COMMON_INTER_THD_ME_SIGNAL
#endif

/* Signal number to use in inter thread signalling. */
#ifndef		DL_SIG_WAKE
#define 	DL_SIG_WAKE						INTER_THD_SIGNAL
#endif


/*------------------------------------------------------------------------------
 * 9) System communication ID's and block ID's.
 *----------------------------------------------------------------------------*/

/* These tell if the executing threads need to work in order.
 * i.e wait for	a signal from prev thread, do some work and signal
 * next thread. */
#ifndef		DL_THREAD_NO_ORDER
#define		DL_THREAD_NO_ORDER	0x0
#endif

#ifndef		DL_THREAD_ORDER
#define		DL_THREAD_ORDER		0x1
#endif


/*	All the IX_* constants should be between 0 and 255 (i.e 1 byte) to enable
 *  using them as immed operands in instructions.
 *
 *	IX_NULL is used to denote a NULL (or empty) packet handle, It should not be
 *  zero, as it would interfere with ring empty (0) condition.
 *	Note: IX_NULL is too generic a name and may clash with core components code.
 *	So it's marked for deletion.  Start using UC_NULL
 */

#ifndef		IX_NULL
#define		IX_NULL					255
#endif

#ifndef		IX_EXCEPTION
#define		IX_EXCEPTION			1
#endif

#ifndef		IX_DROP
#define		IX_DROP					2
#endif

#ifndef		UC_NULL
#define		UC_NULL					255			/* NULL for microcode */
#endif

#ifndef		DL_BUF_NULL
#define		DL_BUF_NULL				UC_NULL 	/* NULL Buffer representation */
#endif

#ifndef		BID_NULL
#define		BID_NULL				UC_NULL		/* NULL Block ID */
#endif

#ifndef		BID_EXCEPTION
#define		BID_EXCEPTION	        1			/* Exception Block ID */
#endif

#ifndef		BID_DROP
#define		BID_DROP                2			/* Drop Block ID */
#endif


/* Block IDs for various blocks. Block IDs Start from 0x20.
 * (0x0 to 0x1f and 0xFF are reserved for IX_*).
 */


#ifndef		BID_IPV4
#define		BID_IPV4		IX_CC_IPV4_PKT_MICROBLOCK_ID
#endif

#ifndef		BID_POS
#define		BID_POS			IX_CC_POS_RX_PKT_ID
#endif

#ifndef		BID_QM
#define     BID_QM          IX_CC_QM_CORE_TO_MICROBLOCK_ID
#endif

#ifndef		BID_ETHER
#define 	BID_ETHER		IX_CC_ETH_RX_PKT_ID
#endif

#ifndef		BID_IPV6
#define		BID_IPV6		IX_DROP
#endif

#ifndef		BID_ARP
#define		BID_ARP                 IX_EXCEPTION
#endif

#ifndef		BID_ETHER_PPP_DISCOVERY
#define		BID_ETHER_PPP_DISCOVERY	IX_EXCEPTION
#endif

#ifndef		BID_ETHER_PPP_SESSION
#define		BID_ETHER_PPP_SESSION	IX_EXCEPTION
#endif

#ifndef		BID_CLASSIFIER_6T
#define		BID_CLASSIFIER_6T		IX_CC_CLASSIFIER_6T_PKT_MICROBLOCK_ID
#endif

#ifndef		BID_TC_METER
#define		BID_TC_METER		    IX_CC_TC_METER_PKT_MICROBLOCK_ID
#endif

#ifndef		BID_DSCP_MARKER
#define		BID_DSCP_MARKER		    IX_CC_DSCP_MARKER_PKT_MICROBLOCK_ID
#endif

#ifndef		BID_CLASSIFIER_DSCP
#define		BID_CLASSIFIER_DSCP		IX_CC_CLASSIFIER_DSCP_PKT_MICROBLOCK_ID
#endif


/*------------------------------------------------------------------------------
 * 10) Static Bindings (a.k.a targets) for various microblocks.
 *----------------------------------------------------------------------------*/

#ifdef		ETHER_WITHOUT_L2
	#ifndef		DL_SOURCE_NEXT1
	#define		DL_SOURCE_NEXT1			BID_ETHER
	#endif
#else

	#ifdef		PPP_RECEIVE
		#ifndef		DL_SOURCE_NEXT1
		#define		DL_SOURCE_NEXT1			BID_POS
		#endif
	#else
		#ifndef		DL_SOURCE_NEXT1
		#define		DL_SOURCE_NEXT1			BID_IPV4
		#endif

	#endif
#endif

#ifndef     POS_RX_NEXT1
#define     POS_RX_NEXT1                        BID_IPV4
#endif

#ifndef     RX_NEXT1
#define     RX_NEXT1                            BID_POS
#endif


#ifndef     IPV4_NEXT1
#define     IPV4_NEXT1                          BID_QM
#endif

#ifndef     IPV4_LOCAL_NEXT1
#define     IPV4_LOCAL_NEXT1                    IX_EXCEPTION
#endif

#ifndef		CLASSIFIER_6T_INV_IP_NEXT_BLOCK
#define		CLASSIFIER_6T_INV_IP_NEXT_BLOCK	    IX_EXCEPTION
#endif

#ifndef		CLASSIFIER_6T_DEFAULT_NEXT_BLOCK
#define		CLASSIFIER_6T_DEFAULT_NEXT_BLOCK    BID_IPV4
#endif

#ifndef		TC_METER_NEXT1
#define		TC_METER_NEXT1                      BID_DSCP_MARKER
#endif

#ifndef		TC_METER_NEXT2
#define		TC_METER_NEXT2	                    BID_IPV4
#endif

#ifndef		TC_METER_NEXT3
#define		TC_METER_NEXT3                      IX_DROP
#endif

#ifndef		CLASSIFIER_DSCP_METER_OUTPUT
#define		CLASSIFIER_DSCP_METER_OUTPUT		BID_TC_METER
#endif

#ifndef		CLASSIFIER_DSCP_MARKER_OUTPUT
#define		CLASSIFIER_DSCP_MARKER_OUTPUT		BID_DSCP_MARKER
#endif

#ifndef		CLASSIFIER_DSCP_IP4_FWD_OUTPUT
#define		CLASSIFIER_DSCP_IP4_FWD_OUTPUT		BID_IPV4
#endif

#ifndef		CLASSIFIER_DSCP_IP6_FWD_OUTPUT
#define		CLASSIFIER_DSCP_IP6_FWD_OUTPUT		BID_IPV6
#endif

#ifndef		CLASSIFIER_DSCP_INV_IP_OUTPUT
#define		CLASSIFIER_DSCP_INV_IP_OUTPUT		IX_DROP
#endif

#ifndef		CLASSIFIER_DSCP_IP4_DEFAULT_OUTPUT
#define		CLASSIFIER_DSCP_IP4_DEFAULT_OUTPUT		BID_CLASSIFIER_6T
#endif

#ifndef		CLASSIFIER_DSCP_IP6_DEFAULT_OUTPUT
#define		CLASSIFIER_DSCP_IP6_DEFAULT_OUTPUT		IX_DROP
#endif

#ifndef		DSCP_MARKER_NEXT1
#define		DSCP_MARKER_NEXT1		BID_IPV4
#endif

#ifndef		BID_PPP_DECAP_NEXT1
#define		BID_PPP_DECAP_NEXT1		BID_CLASSIFIER_DSCP
#endif

#ifndef		BID_PPP_DECAP_NEXT2
#define		BID_PPP_DECAP_NEXT2		IX_DROP
#endif

#ifndef		BID_PPP_DECAP_NEXT3
#define		BID_PPP_DECAP_NEXT3		IX_DROP
#endif


/*	To be Redefined as an import variable. And its value should be one of the above
 *	Block IDs.
 */

#ifndef     BID_NEXT_BLOCK
#define     BID_NEXT_BLOCK          BID_QM
#endif


#endif
