/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILENAME
 *      egress_test_conf.c
 *
 * ============================================================================
 */

#define IX_ERROR_FILE_IDENT "$Id: egress_test_conf.c,v 1.2.2.1 2004/03/12 09:03:48 wneubaue Exp $";

#include <stdarg.h>
#include <stdio.h>
#include <string.h>

#include <ix_error.h>
#include <ix_types.h>
#include <ix_ossl.h>

#include <ix_cc.h>

#if !defined( IX_EXCLUDE_CCI )
#include <ix_cci.h>
#endif /* IX_EXCLUDE_CCI */

#include <bindings.h>

#include "cc/ix_cc_tc_meter_msg_helpers.h"
#include "cc/internal/ix_cc_wred_api.h"
#include "cc/ix_cc_wred_msghlp.h"




/**
 * NAME: common_callback_egress
 */
ix_error
common_callbacak_egress(ix_error  arg_Result,
                         void     *arg_pUserContext)
{
    if ( arg_Result != IX_SUCCESS )
    {
        printf( "\nCallback: common_callbacak_egress: \n" \
                                         "\t_status = 0x%08x \n" \
                                         "\tcontext = %s\n\n",
                                         IX_ERROR_GET_CODE( arg_Result ),
                                         arg_pUserContext );
        printf( "\n Egress initialization FAILED\n" );
        ix_error_dump( stderr, arg_Result );
        return IX_SUCCESS;

    } else {

        printf( "\nCallback: common_callbacak_egress: OK\n" );
        printf( "User context: %s\n", (int) arg_pUserContext );

    }
    return IX_SUCCESS;
}



/************************************************************************************************************/
/*                                  Three Color Meter                                                       */
/************************************************************************************************************/


void
add_tcm_entry(
        ix_uint32 instance,
        ix_uint32 meter_type_id,
        ix_uint32 cir,
        ix_uint32 pir,
        ix_uint32 cbs,
        ix_uint32 ebs,
        ix_uint8 greenDscp,  ix_uint8 greenOutput,
        ix_uint8 yellowDscp, ix_uint8 yellowOutput,
        ix_uint8 redDscp,    ix_uint8 redOutput)

{
    ix_error _status;
    ix_cc_tc_meter_parameters tcm_entry;
    tcm_entry.meter_type_id = meter_type_id;
    if(meter_type_id==IX_CC_TC_METER_SRTCM_TYPE)
    {
        tcm_entry.meter_type.srtcm.cir = cir;
        tcm_entry.meter_type.srtcm.cbs = cbs;
        tcm_entry.meter_type.srtcm.ebs = ebs;
    }
    else
    {
        tcm_entry.meter_type.trtcm.cir = cir;
        tcm_entry.meter_type.trtcm.pir = pir;
        tcm_entry.meter_type.trtcm.cbs = cbs;
        tcm_entry.meter_type.trtcm.pbs = ebs;
    }
    tcm_entry.greenDscp    = greenDscp;
    tcm_entry.greenOutput  = greenOutput;
    tcm_entry.yellowDscp   = yellowDscp;
    tcm_entry.yellowOutput = yellowOutput;
    tcm_entry.redDscp      = redDscp;
    tcm_entry.redOutput    = redOutput;
    tcm_entry.statsEnabled = 1;

    if ( ( _status = ix_cc_tc_meter_async_add_entry(
                        instance,
                        &tcm_entry,
                        common_callbacak_egress,
                        "Add TCMeter entry" ) ) != IX_SUCCESS )

       printf(" Message sending error = 0x%08x\n",IX_ERROR_GET_CODE(_status));
    else
       printf(" ix_cc_tc_meter_async_add_entry() sent messsage to add %d entry\n", instance);
    return;
}



void
update_tcm_entry(
       ix_uint8 instance,
       ix_uint8 meter_type_id,
       ix_uint32 cir,
       ix_uint32 pir,
       ix_uint32 cbs,
       ix_uint32 ebs,
       ix_uint8 greenDscp,  ix_uint8 greenOutput,
       ix_uint8 yellowDscp, ix_uint8 yellowOutput,
       ix_uint8 redDscp,    ix_uint8 redOutput)
{
    ix_error _status;
    ix_cc_tc_meter_parameters tcm_entry;
    tcm_entry.meter_type_id = meter_type_id;
    if(meter_type_id==IX_CC_TC_METER_SRTCM_TYPE)
    {
        tcm_entry.meter_type.srtcm.cir = cir;
        tcm_entry.meter_type.srtcm.cbs = cbs;
        tcm_entry.meter_type.srtcm.ebs = ebs;
    }
    else
    {
        tcm_entry.meter_type.trtcm.cir = cir;
        tcm_entry.meter_type.trtcm.pir = pir;
        tcm_entry.meter_type.trtcm.cbs = cbs;
        tcm_entry.meter_type.trtcm.pbs = ebs;
    }
    tcm_entry.greenDscp    = greenDscp;
    tcm_entry.greenOutput  = greenOutput;
    tcm_entry.yellowDscp   = yellowDscp;
    tcm_entry.yellowOutput = yellowOutput;
    tcm_entry.redDscp      = redDscp;
    tcm_entry.redOutput    = redOutput;

    if ( ( _status = ix_cc_tc_meter_async_update_entry(
                        instance,
                        &tcm_entry,
                        common_callbacak_egress,
                        "Update TCMeter entry" ) ) != IX_SUCCESS )
       printf(" Message sending error = 0x%08x\n",IX_ERROR_GET_CODE(_status));
    else
       printf(" ix_cc_tc_meter_async_update_entry() sent messsage to update %d entry\n", instance);
    return;
}



void remove_tcm_entry(ix_uint8 instance)
{
    ix_error _status;
    if ( ( _status = ix_cc_tc_meter_async_remove_entry(
                        instance,
                        common_callbacak_egress,
                        "Remove TCMeter entry" ) ) != IX_SUCCESS )
       printf(" Message sending error = 0x%08x\n",IX_ERROR_GET_CODE(_status));
    else
       printf(" ix_cc_tc_meter_async_update_entry() sent messsage to remove %d entry\n", instance);
    return;
}



#define TCM_EGRESS_CONF_SIZE 8

/* TCM parameters */
ix_cc_tc_meter_parameters tcm_configuration[TCM_EGRESS_CONF_SIZE] =
{
	{/*algoritm*/ IX_CC_TC_METER_SRTCM_TYPE, {/*cir*/ 8, /*cbs*/ 1000, /*ebs*/ 1000}, /*green DSCP*/ 0, IX_CC_TC_METER_NEXT1, /*yellow DCSP*/ 0, IX_CC_TC_METER_NEXT1, /*red DSCP*/ 0, IX_CC_TC_METER_NEXT1, 1},
	{/*algoritm*/ IX_CC_TC_METER_SRTCM_TYPE, {/*cir*/ 8, /*cbs*/ 1000, /*ebs*/ 1000}, /*green DSCP*/ 0, IX_CC_TC_METER_NEXT1, /*yellow DCSP*/ 0, IX_CC_TC_METER_NEXT1, /*red DSCP*/ 0, IX_CC_TC_METER_NEXT1, 1},
	{/*algoritm*/ IX_CC_TC_METER_SRTCM_TYPE, {/*cir*/ 0xFFFFFFFF, /*cbs*/ 0xFFFFFFFF, /*ebs*/ 0xFFFFFFFF}, /*green DSCP*/ 0, IX_CC_TC_METER_NEXT1, /*yellow DCSP*/ 0, IX_CC_TC_METER_NEXT1, /*red DSCP*/ 0, IX_CC_TC_METER_NEXT1, 1},
	{/*algoritm*/ IX_CC_TC_METER_SRTCM_TYPE, {/*cir*/ 0xFFFFFFFF, /*cbs*/ 0xFFFFFFFF, /*ebs*/ 0xFFFFFFFF}, /*green DSCP*/ 0, IX_CC_TC_METER_NEXT1, /*yellow DCSP*/ 0, IX_CC_TC_METER_NEXT1, /*red DSCP*/ 0, IX_CC_TC_METER_NEXT1, 1},
	{/*algoritm*/ IX_CC_TC_METER_SRTCM_TYPE, {/*cir*/ 0xFFFFFFFF, /*cbs*/ 0xFFFFFFFF, /*ebs*/ 0xFFFFFFFF}, /*green DSCP*/ 0, IX_CC_TC_METER_NEXT1, /*yellow DCSP*/ 0, IX_CC_TC_METER_NEXT1, /*red DSCP*/ 0, IX_CC_TC_METER_NEXT1, 1},
	{/*algoritm*/ IX_CC_TC_METER_SRTCM_TYPE, {/*cir*/ 0xFFFFFFFF, /*cbs*/ 0xFFFFFFFF, /*ebs*/ 0xFFFFFFFF}, /*green DSCP*/ 0, IX_CC_TC_METER_NEXT1, /*yellow DCSP*/ 0, IX_CC_TC_METER_NEXT1, /*red DSCP*/ 0, IX_CC_TC_METER_NEXT1, 1},
	{/*algoritm*/ IX_CC_TC_METER_SRTCM_TYPE, {/*cir*/ 8000, /*cbs*/ 1000000, /*ebs*/ 1000000}, /*green DSCP*/ 0, IX_CC_TC_METER_NEXT1, /*yellow DCSP*/ 0, IX_CC_TC_METER_NEXT3, /*red DSCP*/ 0, IX_CC_TC_METER_NEXT3, 1},
	{/*algoritm*/ IX_CC_TC_METER_SRTCM_TYPE, {/*cir*/ 800000, /*cbs*/ 100000000, /*ebs*/ 0}, /*green DSCP*/ 0, IX_CC_TC_METER_NEXT1, /*yellow DCSP*/ 0, IX_CC_TC_METER_NEXT3, /*red DSCP*/ 0, IX_CC_TC_METER_NEXT3, 1}
};



void add_tcm_cfg_entry(ix_uint8 instance)
{
    ix_cc_tc_meter_parameters tcm_entry = tcm_configuration[instance];

    printf( "Meter type = %x \n",tcm_entry.meter_type_id );
    if(tcm_entry.meter_type_id == IX_CC_TC_METER_SRTCM_TYPE)
    {
        printf( "CIR = %d \n",tcm_entry.meter_type.srtcm.cir );
        printf( "CBS = %d \n",tcm_entry.meter_type.srtcm.cbs );
        printf( "EBS = %d \n",tcm_entry.meter_type.srtcm.ebs );
    }
    else
    {
        printf( "PIR = %d \n",tcm_entry.meter_type.trtcm.pir );
        printf( "CIR = %d \n",tcm_entry.meter_type.trtcm.cir );
        printf( "CBS = %d \n",tcm_entry.meter_type.trtcm.cbs );
        printf( "PBS = %d \n",tcm_entry.meter_type.trtcm.pbs );
    }
    printf( "green DSCP = %d \n",tcm_entry.greenDscp );
    printf( "yellow DSCP = %d \n",tcm_entry.yellowDscp );
    printf( "red DSCP = %d \n",tcm_entry.redDscp );

    if(tcm_entry.meter_type_id == IX_CC_TC_METER_SRTCM_TYPE)
    {
        add_tcm_entry(instance,
                      tcm_entry.meter_type_id,
                      tcm_entry.meter_type.srtcm.cir, 0,
                      tcm_entry.meter_type.srtcm.cbs,
                      tcm_entry.meter_type.srtcm.ebs,
                      tcm_entry.greenDscp,
                      tcm_entry.greenOutput,
                      tcm_entry.yellowDscp,
                      tcm_entry.yellowOutput,
                      tcm_entry.redDscp,
                      tcm_entry.redOutput);
    }
    else
    {
        add_tcm_entry(instance,
                      tcm_entry.meter_type_id,
                      tcm_entry.meter_type.trtcm.cir,
                      tcm_entry.meter_type.trtcm.pir,
                      tcm_entry.meter_type.trtcm.cbs,
                      tcm_entry.meter_type.trtcm.pbs,
                      tcm_entry.greenDscp,
                      tcm_entry.greenOutput,
                      tcm_entry.yellowDscp,
                      tcm_entry.yellowOutput,
                      tcm_entry.redDscp,
                      tcm_entry.redOutput);
    }
    return;
}


ix_cc_tc_meter_statistics tcm_stats;


ix_error tcm_callback(
       ix_error                  arg_Result,
       ix_cc_tc_meter_statistics *arg_pInstanceStatistics,
       void                      *arg_pContext)
{
     printf( "TCM statistics: \n greenPacketCount = %d \n",arg_pInstanceStatistics->greenPacketCount);

     printf("green : %d,   %d\n",(ix_uint32)(arg_pInstanceStatistics->greenPacketCount), (ix_uint32)(arg_pInstanceStatistics->greenByteCount));
     printf("yellow: %d,   %d\n",(ix_uint32)(arg_pInstanceStatistics->yellowPacketCount),(ix_uint32)(arg_pInstanceStatistics->yellowByteCount));
     printf("red   : %d,   %d\n",(ix_uint32)(arg_pInstanceStatistics->redPacketCount),   (ix_uint32)(arg_pInstanceStatistics->redByteCount ));
     return IX_SUCCESS;
}



void print_tcm_statistics(ix_uint32 instance)
{
    ix_error _status;
    if ( ( _status = ix_cc_tc_meter_async_get_statistics(
               instance,
               &tcm_stats,
               tcm_callback,
               "Statistics" ) ) != IX_SUCCESS )
    {
        printf( "Get statistics TCM1 on Egress FAILED 0x%08x\n",
                                             IX_ERROR_GET_CODE( _status ) );
        return;

    }
    return;
}




void create_tcm_cfg()
{
    int i;
    for(i=0; i < TCM_EGRESS_CONF_SIZE; i++)
    {
       add_tcm_cfg_entry(i);
       ix_ossl_sleep(1000);
    }
    return;
}


void remove_tcm_cfg()
{
    int i;
    for(i=0; i < TCM_EGRESS_CONF_SIZE; i++)
    {
       remove_tcm_entry(i);
       ix_ossl_sleep(1000);
    }
    return;
}




/************************************************************************************************************/
/*                                  WRED                                                      */
/************************************************************************************************************/


void
add_wred_entry(
       ix_uint16 instance,
       ix_uint32 service_rate,
       ix_uint8 weight,
       ix_uint8 green_min_th,  ix_uint8 green_max_th,  ix_uint16 green_max_prob,
       ix_uint8 yellow_min_th, ix_uint8 yellow_max_th, ix_uint16 yellow_max_prob,
       ix_uint8 red_min_th,    ix_uint8 red_max_th,    ix_uint16 red_max_prob,
       ix_uint8 statsEnabled)
{
    ix_error              _status;
    ix_cc_wred_parameters wred_entry;
    ix_generic_context    Context;

    Context.hCC = IX_INSTANCE_TO_HANDLE(0, 0);

    wred_entry.service_rate = service_rate;
    wred_entry.weight = weight;
    wred_entry.green.min_th = green_min_th;
    wred_entry.green.max_th = green_max_th;
    wred_entry.green.max_prob = green_max_prob;
    wred_entry.yellow.min_th = yellow_min_th;
    wred_entry.yellow.max_th = yellow_max_th;
    wred_entry.yellow.max_prob = yellow_max_prob;
    wred_entry.red.min_th = red_min_th;
    wred_entry.red.max_th = red_max_th;
    wred_entry.red.max_prob = red_max_prob;
    wred_entry.statsEnabled = statsEnabled;

    if ( ( _status = ix_cc_wred_async_add_entry(
               instance,
               &wred_entry,
               common_callbacak_egress,
               (ix_generic_context*) &Context) ) != IX_SUCCESS )
       printf(" Message sending error = 0x%08x\n",IX_ERROR_GET_CODE(_status));
    else
       printf(" add_wred_entry() sent messsage to add %d entry\n", instance);
    return;
}


void update_wred_entry(
        ix_uint16 instance,
        ix_uint32 service_rate,
        ix_uint8 weight,
        ix_uint8 green_min_th,  ix_uint8 green_max_th,  ix_uint16 green_max_prob,
        ix_uint8 yellow_min_th, ix_uint8 yellow_max_th, ix_uint16 yellow_max_prob,
        ix_uint8 red_min_th,    ix_uint8 red_max_th,    ix_uint16 red_max_prob,
        ix_uint8 statsEnabled)
{
    ix_error              _status;
    ix_cc_wred_parameters wred_entry;
    ix_generic_context    Context;

    Context.hCC = IX_INSTANCE_TO_HANDLE(0, 0);

    wred_entry.service_rate = service_rate;
    wred_entry.weight = weight;
    wred_entry.green.min_th = green_min_th;
    wred_entry.green.max_th = green_max_th;
    wred_entry.green.max_prob = green_max_prob;
    wred_entry.yellow.min_th = yellow_min_th;
    wred_entry.yellow.max_th = yellow_max_th;
    wred_entry.yellow.max_prob = yellow_max_prob;
    wred_entry.red.min_th = red_min_th;
    wred_entry.red.max_th = red_max_th;
    wred_entry.red.max_prob = red_max_prob;
    wred_entry.statsEnabled = statsEnabled;
    if ( ( _status = ix_cc_wred_async_update_entry(
                instance,
                &wred_entry,
                common_callbacak_egress,
                (ix_generic_context*) &Context ) ) != IX_SUCCESS )
       printf(" Message sending error = 0x%08x\n",IX_ERROR_GET_CODE(_status));
    else
       printf(" update_wred_entry() sent messsage to update %d entry\n", instance);
    return;
}


void remove_wred_entry(ix_uint8 instance)
{
    ix_error _status;
    ix_generic_context    Context;

    Context.hCC = IX_INSTANCE_TO_HANDLE(0, 0);

    if ( ( _status = ix_cc_wred_async_remove_entry(
               instance,
               common_callbacak_egress,
               (ix_generic_context*) &Context ) ) != IX_SUCCESS )
       printf(" Message sending error = 0x%08x\n",IX_ERROR_GET_CODE(_status));
    else
       printf(" remove_wred_entry() sent messsage to remove %d entry\n", instance);
    return;
}



#define WRED_CONF_SIZE 6

/* WRED parameters */
ix_cc_wred_parameters wred_configuration[WRED_CONF_SIZE] =
{
    {/*8  service rate*/ 100, /*weight*/ 15, /*green*/ { 255, 255, 0xFFFF}, /*yellow*/ { 255, 255, 0xFFFF}, /*red*/ { 255, 255, 0xFFFF}, 1},
    {/*9  service rate*/ 100, /*weight*/ 0,  /*green*/ { 0,     0, 0xFFFF}, /*yellow*/ { 0,     0, 0xFFFF}, /*red*/ { 0,     0, 0xFFFF}, 1},
    {/*10 service rate*/ 100, /*weight*/ 5,  /*green*/ { 1,    10, 0xFFFF}, /*yellow*/ { 1,    10, 0xFFFF}, /*red*/ { 1,    10, 0xFFFF}, 1},
    {/*11 service rate*/ 100, /*weight*/ 15, /*green*/ { 1,    10, 0xFFFF}, /*yellow*/ { 1,    10, 0xFFFF}, /*red*/ { 1,    10, 0xFFFF}, 1},
    {/*12 service rate*/ 100, /*weight*/ 8,  /*green*/ { 14,   20, 0xFFFF}, /*yellow*/ { 8,    14, 0xFFFF}, /*red*/ { 2,     8, 0xFFFF}, 1},
    {/*13 service rate*/ 100, /*weight*/ 0,  /*green*/ { 4,    10,      0}, /*yellow*/ { 4,    10, 0x7FFF}, /*red*/ { 4,    10, 0xFFFF}, 1}
};


void add_wred_cfg_entry(ix_uint8 instance)
{
    ix_cc_wred_parameters wred_entry = wred_configuration[instance];

    printf( "service rate = %d \n",wred_entry.service_rate );
    printf( "weight = %d \n",wred_entry.weight );
    printf( "green_min_th = %d \n",wred_entry.green.min_th );
    printf( "green_max_th = %d \n",wred_entry.green.max_th );
    printf( "green_max_prob = %d \n",wred_entry.green.max_prob );
    printf( "yellow_min_th = %d \n",wred_entry.yellow.min_th );
    printf( "yellow_max_th = %d \n",wred_entry.yellow.max_th );
    printf( "yellow_max_prob = %d \n",wred_entry.yellow.max_prob );
    printf( "red_min_th = %d \n",wred_entry.red.min_th );
    printf( "red_max_th = %d \n",wred_entry.red.max_th );
    printf( "red_max_prob = %d \n",wred_entry.red.max_prob );

    add_wred_entry(instance + 8,
                   wred_entry.service_rate,
                   wred_entry.weight,
                   wred_entry.green.min_th,
                   wred_entry.green.max_th,
                   wred_entry.green.max_prob,
                   wred_entry.yellow.min_th,
                   wred_entry.yellow.max_th,
                   wred_entry.yellow.max_prob,
                   wred_entry.red.min_th,
                   wred_entry.red.max_th,
                   wred_entry.red.max_prob,
                   wred_entry.statsEnabled);
    return;
}


void create_wred_cfg()
{
    int i;
    for(i=0; i<WRED_CONF_SIZE; i++)
    {
       add_wred_cfg_entry(i);
       ix_ossl_sleep(1000);
    }
    return;
}


void remove_wred_cfg()
{
    int i;
    for(i=0; i<WRED_CONF_SIZE; i++)
    {
       remove_wred_entry(i);
       ix_ossl_sleep(1000);
    }
    return;
}



ix_cc_wred_statistics wred_stats;


ix_error wred_callback(ix_error arg_Result,
                       ix_cc_wred_statistics *arg_pInstanceStatistics,
                       void* arg_pContext)
{
    printf( "WRED statistics: \n greenPacketCount = %d \n",(ix_uint32)(arg_pInstanceStatistics->greenPacketCount));
    printf( " greenByteCount = %d \n",                     (ix_uint32)(arg_pInstanceStatistics->greenByteCount));
    printf( " yellowPacketCount = %d \n",                  (ix_uint32)(arg_pInstanceStatistics->yellowPacketCount));
    printf( " yellowByteCount = %d \n",                    (ix_uint32)(arg_pInstanceStatistics->yellowByteCount));
    printf( " redPacketCount = %d \n",                     (ix_uint32)(arg_pInstanceStatistics->redPacketCount));
    printf( " redByteCount = %d \n",                       (ix_uint32)(arg_pInstanceStatistics->redByteCount));
    return IX_SUCCESS;
}


void
print_wred_statistics(ix_uint32 instance)
{
    ix_error _status;
    ix_generic_context    Context;

    Context.hCC = IX_INSTANCE_TO_HANDLE(0, 0);

    if ( ( _status = ix_cc_wred_async_get_statistics(
               instance,
               &wred_stats,
               wred_callback,
               (ix_generic_context*) &Context ) ) != IX_SUCCESS )
    {
        printf( "Get statistics TCM1 on Egress FAILED 0x%08x\n",
                                             IX_ERROR_GET_CODE( _status ) );
        return;
    }
    return;
}





void create_cfg()
{
    create_tcm_cfg();
    create_wred_cfg();
    return;
}


void remove_cfg()
{
    remove_wred_cfg();
    remove_tcm_cfg();
    return;
}





