/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILENAME
 *      ingress_test_conf.c
 *
 * ============================================================================
 */

#define IX_ERROR_FILE_IDENT "$Id: ingress_test_conf.c,v 1.1.4.1 2004/03/12 09:03:48 wneubaue Exp $";

#include <stdarg.h>
#include <stdio.h>
#include <string.h>

#include <ix_error.h>
#include <ix_types.h>
#include <ix_ossl.h>

#include <ix_cc.h>

#if !defined( IX_EXCLUDE_CCI )
#include <ix_cci.h>
#endif /* IX_EXCLUDE_CCI */


#include <bindings.h>
#include "cc/ix_cc_dscp_classifier.h"
#include "cc/ix_cc_tc_meter_msg_helpers.h"
#include "cc/ix_cc_dscp_classifier_msghlp.h"

#include "cc/ix_cc_classifier_6t_stats.h"
#include "cc/ix_cc_classifier_6t_api.h"
#include "cc/ix_cc_classifier_6t_msg.h"
#include "cc/ix_cc_classifier_6t.h"

#include "cc/ix_cc_rtmv4.h"
#include "cc/ix_cc_ipv4.h"
#include "cc/ipv4/internal/header_defs.h"

extern void show_mem(void* arg_address, int byteNum);



/**
 * NAME: common_callback_ingress
 */
ix_error
common_callbacak_ingress(ix_error  arg_Result,
                         void     *arg_pUserContext)
{
    if ( arg_Result != IX_SUCCESS )
    {
        printf( "\nCallback: common_callbacak_ingress: \n" \
                                         "\t_status = 0x%08x \n" \
                                         "\tcontext = %s\n\n",
                                         IX_ERROR_GET_CODE( arg_Result ),
                                         arg_pUserContext );
        printf( "\n Ingress initialization FAILED\n" );
        ix_error_dump( stderr, arg_Result );
        return IX_SUCCESS;

    } else {

        printf( "\nCallback: common_callbacak_ingress: OK\n" );
        printf( "User context: %s\n", (int) arg_pUserContext );

    }
    return IX_SUCCESS;
}


/************************************************************************************************************/
/*                                  6-tuple Classifier                                                      */
/************************************************************************************************************/


/**
 * NAME: classifier_6t_callback_search
 */

ix_error classifier_6t_callback_search(
  ix_error arg_Result,
  ix_cc_classifier_6t_result * arg_pResult,
  void * arg_pUserContext )
{
    if ( arg_pUserContext == NULL)
    {
        printf("\nCallback: classifier_6t_callback_search(): \n" \
                                        "\t_status = 0x%08x context = NULL\n",
                                        arg_Result);
        return IX_CC_ERROR_NULL;
    }

    printf("\nCallback: classifier_6t_callback_search(): \n" \
	                                "\t_status = 0x%08x arg_pResult = 0x%08x \n" \
                                    "\tcontext = %s\n\n",
                                    arg_Result, arg_pResult, arg_pUserContext);

    printf( "\n flow Id = %d\n",  arg_pResult->result.qos.flowId );
    printf( "\n class Id = %d\n", arg_pResult->result.qos.classId );
    printf( "\n color Id = %d\n", arg_pResult->result.qos.colorId );

    return IX_SUCCESS;
}


void add_6t_classifier_qos_entry(ix_uint32 srcAddress, ix_uint32 dstAddress, ix_uint32 inputPort,ix_uint32 dscp,
                             ix_uint32 protocol, ix_uint32 srcPort, ix_uint32 dstPort,
                             ix_uint32 flowId, ix_uint16 classId, ix_uint8 colorId)
{
    ix_error _status;
    ix_cc_classifier_6t_pattern classifier_pattern_entry;
    ix_cc_classifier_6t_result classifier_result_entry;

    classifier_pattern_entry.ipv4SourceAddress = srcAddress;
    classifier_pattern_entry.ipv4DestinationAddress = dstAddress;
    classifier_pattern_entry.inputPort = inputPort;
    classifier_pattern_entry.padding = 0;
    classifier_pattern_entry.dscp = dscp;
    classifier_pattern_entry.protocol = protocol;
    classifier_pattern_entry.l4SourcePort = srcPort;
    classifier_pattern_entry.l4DestinationPort = dstPort;

    classifier_result_entry.type = IX_CC_CLASSIFIER_6T_QOS;
    classifier_result_entry.result.qos.flowId = flowId;
    classifier_result_entry.result.qos.classId = classId;
    classifier_result_entry.result.qos.colorId = colorId;
    classifier_result_entry.result.qos.OutputId = IX_CC_CLASSIFIER_6T_METER_BID;
    classifier_result_entry.stats_enable = 1;

    printf( "Src addr = %x \n",classifier_pattern_entry.ipv4SourceAddress );
    printf( "Dest addr = %x \n",classifier_pattern_entry.ipv4DestinationAddress );
    printf( "L4src = %d \n",classifier_pattern_entry.l4SourcePort );
    printf( "L4dst = %d \n",classifier_pattern_entry.l4DestinationPort );
    printf( "Input port = %d \n",classifier_pattern_entry.inputPort );
    printf( "Dscp = %d \n",classifier_pattern_entry.dscp );
    printf( "Protocol = %d \n",classifier_pattern_entry.protocol );


    if ( ( _status = ix_cc_classifier_6t_async_add_entry( &classifier_pattern_entry,
                                                          &classifier_result_entry,
                                                          common_callbacak_ingress,
                                                          "Add CLASSIFIER_6T 6TC1" ) ) != IX_SUCCESS )
    {
        printf( "Add CLASSIFIER_6T on Ingress FAILED 0x%08x\n",
                                         IX_ERROR_GET_CODE( _status ) );
        return;
    }
    printf( "Added CLASSIFIER_6T entry, flowId = %d, colorId = %d,\n dscp = %x \n", flowId, colorId, dscp );
    return;
}




void update_6t_classifier_qos_entry(ix_uint32 srcAddress, ix_uint32 dstAddress, ix_uint32 inputPort,ix_uint32 dscp,
                             ix_uint32 protocol, ix_uint32 srcPort, ix_uint32 dstPort,
                             ix_uint32 flowId, ix_uint16 classId, ix_uint8 colorId)
{
    ix_error _status;
    ix_cc_classifier_6t_pattern classifier_pattern_entry;
    ix_cc_classifier_6t_result classifier_result_entry;
    classifier_pattern_entry.ipv4SourceAddress = srcAddress;
    classifier_pattern_entry.ipv4DestinationAddress = dstAddress;
    classifier_pattern_entry.inputPort = inputPort;
    classifier_pattern_entry.padding = 0;
    classifier_pattern_entry.dscp = dscp;
    classifier_pattern_entry.protocol = protocol;
    classifier_pattern_entry.l4SourcePort = srcPort;
    classifier_pattern_entry.l4DestinationPort = dstPort;

    classifier_result_entry.type = IX_CC_CLASSIFIER_6T_QOS;
    classifier_result_entry.result.qos.flowId = flowId;
    classifier_result_entry.result.qos.classId = classId;
    classifier_result_entry.result.qos.colorId = colorId;
    classifier_result_entry.result.qos.OutputId = IX_CC_CLASSIFIER_6T_METER_BID;


    if ( ( _status = ix_cc_classifier_6t_async_update_entry(&classifier_pattern_entry,
                                                            &classifier_result_entry,
                                                            common_callbacak_ingress,
                                                            "Update CLASSIFIER_6T 6TC1" ) ) != IX_SUCCESS )
    {
        printf( "Update CLASSIFIER_6T on Ingress FAILED 0x%08x\n",
                                         IX_ERROR_GET_CODE( _status ) );
        return;
    }
    printf( "Updated CLASSIFIER_6T entry flowId = %d, classId = %d \n", flowId, classId );
    return;
}



void remove_6t_classifier_qos_entry(ix_uint32 srcAddress, ix_uint32 dstAddress, ix_uint32 inputPort, ix_uint32 dscp,
                                ix_uint32 protocol, ix_uint32 srcPort, ix_uint32 dstPort)
{
    ix_error _status;
    ix_cc_classifier_6t_pattern classifier_pattern_entry;
    classifier_pattern_entry.ipv4SourceAddress = srcAddress;
    classifier_pattern_entry.ipv4DestinationAddress = dstAddress;
    classifier_pattern_entry.inputPort = inputPort;
    classifier_pattern_entry.padding = 0;
    classifier_pattern_entry.dscp = dscp;
    classifier_pattern_entry.protocol = protocol;
    classifier_pattern_entry.l4SourcePort = srcPort;
    classifier_pattern_entry.l4DestinationPort = dstPort;

    if ( ( _status = ix_cc_classifier_6t_async_remove_entry( &classifier_pattern_entry,
                                                          IX_CC_CLASSIFIER_6T_QOS,
                                                          common_callbacak_ingress,
                                                          "Delete CLASSIFIER_6T 6TC1" ) ) != IX_SUCCESS )
    {
        printf( "Delete CLASSIFIER_6T on Ingress FAILED 0x%08x\n",
                                         IX_ERROR_GET_CODE( _status ) );
        return;
    }
    printf( "Deleted CLASSIFIER_6T entry, srcPort = %d \n", srcPort);
    return;
}


#define CLASSIFIER_CONF_SIZE 24

/* CLASSIFIER_6T QOS key pattern */
ix_cc_classifier_6t_pattern classifier_pattern_configuration[CLASSIFIER_CONF_SIZE] =
{
    {/*0  src addr*/ 0x01000002, /*dst addr*/ 0x02000002, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*1  src addr*/ 0x01000002, /*dst addr*/ 0x02000003, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*2  src addr*/ 0x01000002, /*dst addr*/ 0x02000004, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*3  src addr*/ 0x01000002, /*dst addr*/ 0x02000005, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*4  src addr*/ 0x01000002, /*dst addr*/ 0x02000006, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*5  src addr*/ 0x01000002, /*dst addr*/ 0x02000007, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*6  src addr*/ 0x01000002, /*dst addr*/ 0x02000008, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*7  src addr*/ 0x01000002, /*dst addr*/ 0x02000009, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*8  src addr*/ 0x01000002, /*dst addr*/ 0x0200000A, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*9  src addr*/ 0x01000002, /*dst addr*/ 0x0200000B, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*10 src addr*/ 0x01000002, /*dst addr*/ 0x0200000C, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*11 src addr*/ 0x01000002, /*dst addr*/ 0x0200000D, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*12 src addr*/ 0x01000002, /*dst addr*/ 0x0200000E, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*13 src addr*/ 0x01000002, /*dst addr*/ 0x0200000F, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*14 src addr*/ 0x01000002, /*dst addr*/ 0x02000010, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*15 src addr*/ 0x01000002, /*dst addr*/ 0x02000011, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*16 src addr*/ 0x01000002, /*dst addr*/ 0x02000012, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*17 src addr*/ 0x01000002, /*dst addr*/ 0x02000013, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*18 src addr*/ 0x01000002, /*dst addr*/ 0x02000014, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*19 src addr*/ 0x01000002, /*dst addr*/ 0x02000015, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*20 src addr*/ 0x01000002, /*dst addr*/ 0x02000016, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*21 src addr*/ 0x01000002, /*dst addr*/ 0x02000017, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*22 src addr*/ 0x01000002, /*dst addr*/ 0x02000018, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20},
    {/*23 src addr*/ 0x01000002, /*dst addr*/ 0x02000019, /*input port*/ 0, /*dscp*/ 0, /*padding*/ 0, /*protocol*/ 0x11, /*L4 src*/ 20, /*L4 dst*/ 20}
};


/* CLASSIFIER_6T QOS parameters */
ix_cc_classifier_6t_result classifier_result_configuration[CLASSIFIER_CONF_SIZE]=
{
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 1, /*class ID 1*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 2, /*class ID 1*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 3, /*class ID 1*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 4, /*class ID 1*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 2, /*class ID 2*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 2, /*class ID 2*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 2, /*class ID 2*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 2, /*class ID 2*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 2, /*class ID 3*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 3, /*class ID 3*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 3, /*class ID 3*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 3, /*class ID 3*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 3, /*class ID 4*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 3, /*class ID 4*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 4, /*class ID 4*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 4, /*class ID 4*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 4, /*class ID 5*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 4, /*class ID 5*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 4, /*class ID 5*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 5, /*class ID 5*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 6, /*class ID 6*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 6, /*class ID 7*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 6, /*class ID 8*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID},
    { IX_CC_CLASSIFIER_6T_QOS, /*flow ID*/ 6, /*class ID 9*/ 0, /*color*/ 0, /*next block*/ IX_CC_CLASSIFIER_6T_METER_BID}
};




void add_6t_classifier_cfg_entry(ix_uint8 EntryNo)
{
    add_6t_classifier_qos_entry(
         classifier_pattern_configuration[EntryNo].ipv4SourceAddress,
         classifier_pattern_configuration[EntryNo].ipv4DestinationAddress,
         classifier_pattern_configuration[EntryNo].inputPort,
         classifier_pattern_configuration[EntryNo].dscp,
         classifier_pattern_configuration[EntryNo].protocol,
         classifier_pattern_configuration[EntryNo].l4SourcePort,
         classifier_pattern_configuration[EntryNo].l4DestinationPort,
         classifier_result_configuration[EntryNo].result.qos.flowId,
         classifier_result_configuration[EntryNo].result.qos.classId,
         classifier_result_configuration[EntryNo].result.qos.colorId);
    return;
}



void remove_6t_classifier_cfg_entry(ix_uint8 EntryNo)
{
    remove_6t_classifier_qos_entry(
         classifier_pattern_configuration[EntryNo].ipv4SourceAddress,
         classifier_pattern_configuration[EntryNo].ipv4DestinationAddress,
         classifier_pattern_configuration[EntryNo].inputPort,
         classifier_pattern_configuration[EntryNo].dscp,
         classifier_pattern_configuration[EntryNo].protocol,
         classifier_pattern_configuration[EntryNo].l4SourcePort,
         classifier_pattern_configuration[EntryNo].l4DestinationPort);
}



void print_6t_classifier_cfg_entry(ix_uint8 EntryNo)
{
    ix_error _status = IX_SUCCESS;
    ix_cc_classifier_6t_result classifier_result_entry;
	classifier_result_entry.type = IX_CC_CLASSIFIER_6T_QOS;
    _status = ix_cc_classifier_6t_async_search_entry(&classifier_pattern_configuration[EntryNo],
                                                     IX_CC_CLASSIFIER_6T_QOS,
                                                     &classifier_result_entry,
                                                     classifier_6t_callback_search,
                                                     "Search CLASSIFIER_6t params");
    if ( IX_ERROR_GET_CODE(_status) != IX_SUCCESS )
       printf(" Message sending error = 0x%08x\n",IX_ERROR_GET_CODE(_status));
    else
       printf("print_6t_classifier_cfg_entry() : returned status 0x%08x\n", _status );

    return;
}


void create_6t_classifier_cfg()
{
    int i;
    for(i=0;i < CLASSIFIER_CONF_SIZE; i++)
    {
       add_6t_classifier_cfg_entry(i);
       ix_ossl_sleep(1000);
    }
    return;
}


void remove_6t_classifier_cfg()
{
    int i;
    for(i=0; i<CLASSIFIER_CONF_SIZE; i++)
    {
       remove_6t_classifier_cfg_entry(i);
       ix_ossl_sleep(1000);
    }
    return;
}




/************************************************************************************************************/
/*                                  Three Color Meter                                                       */
/************************************************************************************************************/


void
add_tcm_entry(
        ix_uint32 instance,
        ix_uint32 meter_type_id,
        ix_uint32 cir,
        ix_uint32 pir,
        ix_uint32 cbs,
        ix_uint32 ebs,
        ix_uint8 greenDscp,  ix_uint8 greenOutput,
        ix_uint8 yellowDscp, ix_uint8 yellowOutput,
        ix_uint8 redDscp,    ix_uint8 redOutput)

{
    ix_error _status;
    ix_cc_tc_meter_parameters tcm_entry;
    tcm_entry.meter_type_id = meter_type_id;
    if(meter_type_id==IX_CC_TC_METER_SRTCM_TYPE)
    {
        tcm_entry.meter_type.srtcm.cir = cir;
        tcm_entry.meter_type.srtcm.cbs = cbs;
        tcm_entry.meter_type.srtcm.ebs = ebs;
    }
    else
    {
        tcm_entry.meter_type.trtcm.cir = cir;
        tcm_entry.meter_type.trtcm.pir = pir;
        tcm_entry.meter_type.trtcm.cbs = cbs;
        tcm_entry.meter_type.trtcm.pbs = ebs;
    }
    tcm_entry.greenDscp    = greenDscp;
    tcm_entry.greenOutput  = greenOutput;
    tcm_entry.yellowDscp   = yellowDscp;
    tcm_entry.yellowOutput = yellowOutput;
    tcm_entry.redDscp      = redDscp;
    tcm_entry.redOutput    = redOutput;
    tcm_entry.statsEnabled = 1;

    if ( ( _status = ix_cc_tc_meter_async_add_entry(
                        instance,
                        &tcm_entry,
                        common_callbacak_ingress,
                        "Add TCMeter entry" ) ) != IX_SUCCESS )

       printf(" Message sending error = 0x%08x\n",IX_ERROR_GET_CODE(_status));
    else
       printf(" ix_cc_tc_meter_async_add_entry() sent messsage to add %d entry\n", instance);
    return;
}



void
update_tcm_entry(
       ix_uint8 instance,
       ix_uint8 meter_type_id,
       ix_uint32 cir,
       ix_uint32 pir,
       ix_uint32 cbs,
       ix_uint32 ebs,
       ix_uint8 greenDscp,  ix_uint8 greenOutput,
       ix_uint8 yellowDscp, ix_uint8 yellowOutput,
       ix_uint8 redDscp,    ix_uint8 redOutput)
{
    ix_error _status;
    ix_cc_tc_meter_parameters tcm_entry;
    tcm_entry.meter_type_id = meter_type_id;
    if(meter_type_id==IX_CC_TC_METER_SRTCM_TYPE)
    {
        tcm_entry.meter_type.srtcm.cir = cir;
        tcm_entry.meter_type.srtcm.cbs = cbs;
        tcm_entry.meter_type.srtcm.ebs = ebs;
    }
    else
    {
        tcm_entry.meter_type.trtcm.cir = cir;
        tcm_entry.meter_type.trtcm.pir = pir;
        tcm_entry.meter_type.trtcm.cbs = cbs;
        tcm_entry.meter_type.trtcm.pbs = ebs;
    }
    tcm_entry.greenDscp    = greenDscp;
    tcm_entry.greenOutput  = greenOutput;
    tcm_entry.yellowDscp   = yellowDscp;
    tcm_entry.yellowOutput = yellowOutput;
    tcm_entry.redDscp      = redDscp;
    tcm_entry.redOutput    = redOutput;

    if ( ( _status = ix_cc_tc_meter_async_update_entry(
                        instance,
                        &tcm_entry,
                        common_callbacak_ingress,
                        "Update TCMeter entry" ) ) != IX_SUCCESS )
       printf(" Message sending error = 0x%08x\n",IX_ERROR_GET_CODE(_status));
    else
       printf(" ix_cc_tc_meter_async_update_entry() sent messsage to update %d entry\n", instance);
    return;
}



void remove_tcm_entry(ix_uint8 instance)
{
    ix_error _status;
    if ( ( _status = ix_cc_tc_meter_async_remove_entry(
                        instance,
                        common_callbacak_ingress,
                        "Remove TCMeter entry" ) ) != IX_SUCCESS )
       printf(" Message sending error = 0x%08x\n",IX_ERROR_GET_CODE(_status));
    else
       printf(" ix_cc_tc_meter_async_update_entry() sent messsage to remove %d entry\n", instance);
    return;
}



#define TCM_INGRESS_CONF_SIZE 10

/* TCM parameters */
ix_cc_tc_meter_parameters tcm_configuration[TCM_INGRESS_CONF_SIZE] =
{
	{/*algoritm*/ IX_CC_TC_METER_SRTCM_TYPE, {/*cir*/ 8, /*cbs*/ 1000, /*ebs*/ 1000}, /*green DSCP*/ 0x04, IX_CC_TC_METER_NEXT1, /*yellow DCSP*/ 0x08, IX_CC_TC_METER_NEXT1, /*red DSCP*/ 0x0C, IX_CC_TC_METER_NEXT1, 1},
	{/*algoritm*/ IX_CC_TC_METER_SRTCM_TYPE, {/*cir*/ 8, /*cbs*/ 1000, /*ebs*/ 1000}, /*green DSCP*/ 0x04, IX_CC_TC_METER_NEXT1, /*yellow DCSP*/ 0x08, IX_CC_TC_METER_NEXT1, /*red DSCP*/ 0x0C, IX_CC_TC_METER_NEXT1, 1},
	{/*algoritm*/ IX_CC_TC_METER_SRTCM_TYPE, {/*cir*/ 8000, /*cbs*/ 10000, /*ebs*/ 10000}, /*green DSCP*/ 0x10, IX_CC_TC_METER_NEXT1, /*yellow DCSP*/ 0x14, IX_CC_TC_METER_NEXT1, /*red DSCP*/ 0x18, IX_CC_TC_METER_NEXT1, 1},
	{/*algoritm*/ IX_CC_TC_METER_SRTCM_TYPE, {/*cir*/ 800,  /*cbs*/ 1000, /*ebs*/ 5000}, /*green DSCP*/ 0x1C, IX_CC_TC_METER_NEXT1, /*yellow DCSP*/ 0x20, IX_CC_TC_METER_NEXT1, /*red DSCP*/ 0x24, IX_CC_TC_METER_NEXT1, 1},
	{/*algoritm*/ IX_CC_TC_METER_SRTCM_TYPE, {/*cir*/ 800000, /*cbs*/ 100000, /*ebs*/ 100000}, /*green DSCP*/ 0x28, IX_CC_TC_METER_NEXT1, /*yellow DCSP*/ 0x2C, IX_CC_TC_METER_NEXT1, /*red DSCP*/ 0x30, IX_CC_TC_METER_NEXT1, 1},
	{/*algoritm*/ IX_CC_TC_METER_SRTCM_TYPE, {/*cir*/ 8, /*cbs*/ 0, /*ebs*/ 1000}, /*green DSCP*/ 0x34, IX_CC_TC_METER_NEXT1, /*yellow DCSP*/ 0x38, IX_CC_TC_METER_NEXT1, /*red DSCP*/ 0x3C, IX_CC_TC_METER_NEXT1, 1},
	{/*algoritm*/ IX_CC_TC_METER_SRTCM_TYPE, {/*cir*/ 0xFFFFFFFF, /*cbs*/ 0xFFFFFFFF, /*ebs*/ 0xFFFFFFFF}, /*green DSCP*/ 0x32, IX_CC_TC_METER_NEXT1, /*yellow DCSP*/ 0x34, IX_CC_TC_METER_NEXT1, /*red DSCP*/ 0x36, IX_CC_TC_METER_NEXT1, 1},
	{/*algoritm*/ IX_CC_TC_METER_SRTCM_TYPE, {/*cir*/ 0xFFFFFFFF, /*cbs*/ 0xFFFFFFFF, /*ebs*/ 0xFFFFFFFF}, /*green DSCP*/ 0x4C, IX_CC_TC_METER_NEXT1, /*yellow DCSP*/ 0x4C, IX_CC_TC_METER_NEXT1, /*red DSCP*/ 0x4C, IX_CC_TC_METER_NEXT1, 1},
	{/*algoritm*/ IX_CC_TC_METER_SRTCM_TYPE, {/*cir*/ 8000, /*cbs*/ 8000, /*ebs*/ 8000}, /*green DSCP*/ 0x12, IX_CC_TC_METER_NEXT1, /*yellow DCSP*/ 0x16, IX_CC_TC_METER_NEXT1, /*red DSCP*/ 0x1A, IX_CC_TC_METER_NEXT3, 1},
	{/*algoritm*/ IX_CC_TC_METER_SRTCM_TYPE, {/*cir*/ 500000, /*cbs*/ 1200, /*ebs*/ 1200}, /*green DSCP*/ 0x22, IX_CC_TC_METER_NEXT1, /*yellow DCSP*/ 0x26, IX_CC_TC_METER_NEXT1, /*red DSCP*/ 0x2A, IX_CC_TC_METER_NEXT1, 1}
};



void add_tcm_cfg_entry(ix_uint8 instance)
{
    ix_cc_tc_meter_parameters tcm_entry = tcm_configuration[instance];

    printf( "Meter type = %x \n",tcm_entry.meter_type_id );
    if(tcm_entry.meter_type_id == IX_CC_TC_METER_SRTCM_TYPE)
    {
        printf( "CIR = %d \n",tcm_entry.meter_type.srtcm.cir );
        printf( "CBS = %d \n",tcm_entry.meter_type.srtcm.cbs );
        printf( "EBS = %d \n",tcm_entry.meter_type.srtcm.ebs );
    }
    else
    {
        printf( "PIR = %d \n",tcm_entry.meter_type.trtcm.pir );
        printf( "CIR = %d \n",tcm_entry.meter_type.trtcm.cir );
        printf( "CBS = %d \n",tcm_entry.meter_type.trtcm.cbs );
        printf( "PBS = %d \n",tcm_entry.meter_type.trtcm.pbs );
    }
    printf( "green DSCP = %d \n",tcm_entry.greenDscp );
    printf( "yellow DSCP = %d \n",tcm_entry.yellowDscp );
    printf( "red DSCP = %d \n",tcm_entry.redDscp );

    if(tcm_entry.meter_type_id == IX_CC_TC_METER_SRTCM_TYPE)
    {
        add_tcm_entry(instance,
                      tcm_entry.meter_type_id,
                      tcm_entry.meter_type.srtcm.cir, 0,
                      tcm_entry.meter_type.srtcm.cbs,
                      tcm_entry.meter_type.srtcm.ebs,
                      tcm_entry.greenDscp,
                      tcm_entry.greenOutput,
                      tcm_entry.yellowDscp,
                      tcm_entry.yellowOutput,
                      tcm_entry.redDscp,
                      tcm_entry.redOutput);
    }
    else
    {
        add_tcm_entry(instance,
                      tcm_entry.meter_type_id,
                      tcm_entry.meter_type.trtcm.cir,
                      tcm_entry.meter_type.trtcm.pir,
                      tcm_entry.meter_type.trtcm.cbs,
                      tcm_entry.meter_type.trtcm.pbs,
                      tcm_entry.greenDscp,
                      tcm_entry.greenOutput,
                      tcm_entry.yellowDscp,
                      tcm_entry.yellowOutput,
                      tcm_entry.redDscp,
                      tcm_entry.redOutput);
    }
    return;
}


void create_tcm_cfg()
{
    int i;
    for(i=0;i<TCM_INGRESS_CONF_SIZE ;i++)
    {
       add_tcm_cfg_entry(i);
       ix_ossl_sleep(1000);
    }
    return;
}


void remove_tcm_cfg()
{
    int i;
    for(i=0;i<TCM_INGRESS_CONF_SIZE ;i++)
    {
       remove_tcm_entry(i);
       ix_ossl_sleep(1000);
    }
    return;
}


/*********************************************************************************************************/
/*                                    configure forwarder                                                */
/*********************************************************************************************************/


/* Next hop Ids */
#define IPV4_NEXT_HOP_RTM 10

/* IPv4 addresses */
#define IPV4_IP_ADDR_RTM  0x02000000 /* 2.0.0.0 */

/* IPv4 net masks */
#define IPV4_NET_MASK_RTM 0xFFFF0000


/* TMP defines ++ */
#define MPLS_FLAG_E_LSP        0x10
#define NEXT_HOP_ID_IPv4_TYPE  0


void create_rtm()
{
    ix_error _status;
    /* Next hop data */
    ix_cc_rtmv4_next_hop_info ipv4_next_hop_data_rtm;

    memset(&ipv4_next_hop_data_rtm, 0, sizeof(ix_cc_rtmv4_next_hop_info));

    ipv4_next_hop_data_rtm.mtu = 1500;
    ipv4_next_hop_data_rtm.bladeID = 2;
    ipv4_next_hop_data_rtm.ipAddr = IPV4_IP_ADDR_RTM;
    ipv4_next_hop_data_rtm.l2IndexType = NEXT_HOP_ID_IPv4_TYPE;

    if ( ( _status = ix_cc_ipv4_async_add_next_hop(
                        IPV4_NEXT_HOP_RTM,
                        &ipv4_next_hop_data_rtm,
                        common_callbacak_ingress,
                        "Add next hop" ) ) != IX_SUCCESS )
    {
        printf( "Add next hop RTM on Ingress FAILED 0x%08x\n",
                                         IX_ERROR_GET_CODE( _status ) );
        return;
    }
    ix_ossl_sleep(1000);

    if ( ( _status = ix_cc_ipv4_async_add_route(
                       IPV4_IP_ADDR_RTM,
                       IPV4_NET_MASK_RTM,
                       IPV4_NEXT_HOP_RTM,
                       common_callbacak_ingress,
                       "Add route RTM" ) ) != IX_SUCCESS )
    {
        printf( "Add route RTM on Ingress FAILED 0x%08x\n",
                                         IX_ERROR_GET_CODE( _status ) );
        return;
    }
    ix_ossl_sleep(1000);
}







void create_cfg()
{
  create_rtm();
  create_6t_classifier_cfg();
  create_tcm_cfg();
  return;
}


void remove_cfg()
{
  remove_6t_classifier_cfg();
  remove_tcm_cfg();
  return;
}



