/*---------------------------------------------------------------------
 *                                                                     
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                       
 *      COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *      THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                       
 *                         INTEL  CORPORATION                            
 *                                                                      
 *                      2200 MISSION COLLEGE BLVD                        
 *                                                                       
 *                SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                       
 *---------------------------------------------------------------------
 *
 *        File Name: dl_system.h
 *
 *        Purpose: This file contains system wide parameters
 *                 that are customized for a given application.
 *
 *-------------------------------------------------------------------*/
#ifndef	__DL_SYSTEM__H__
#define	__DL_SYSTEM__H__

/* Define chip. */
#ifndef 	IXP2400
#define 	IXP2400
#endif

/* Define project as microcode based. */
#ifndef 	IXP_MICROCODE
#define 	IXP_MICROCODE
#endif

/* Put IPv4 counters in scratch. */
#define 	IPV4_COUNTERS_SCRATCH

#define		PKT_HDR_CACHE_SIZE			10	

/* Receive Phy mode */
#define 	RX_PHY_MODE					SPHY_4_8

/*---------------------------------------------------------------------
 * SRAM memory map for channel 0.
 *-------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * i) Base address for next hop information.
 *-------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR
#ifdef		IXP_SIMULATION
#define		NEXTHOP_TABLE_SRAM_BASE		0x1000000
#else		/* hardware */
#define		NEXTHOP_TABLE_SRAM_BASE		0x300000
#endif
#endif		/* USE_IMPORT_VAR */

/* Trie Table size. 4MB */
#define		TRIE_TABLE_SIZE				0x400000


/*---------------------------------------------------------------------
 * SRAM memory map for channel 1.
 *-------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * i) Base address for IPv4 counters.
 *-------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

#ifdef 		IPV4_COUNTERS_SCRATCH

/* Counters in scratch */
#define		IPV4_STATS_TABLE_BASE		11264 

#else

/* Counters in SRAM	*/
#ifdef 		IXP_SIMULATION
#define		STATS_TABLE_SRAM_BASE		0x40400000
#else		/* hardware */
#define		STATS_TABLE_SRAM_BASE		0x40200000
#endif

#endif		/* IPV4_COUNTERS_SCRATCH */

#endif		/* USE_IMPORT_VAR */

/*---------------------------------------------------------------------
 * ii) Base address for POS RX counters.
 *-------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

/* Base for POS RX counters */

#ifdef 		IXP_SIMULATION
#define		PACKET_COUNTERS_SRAM_BASE	0x40400800
#else 		/* hardware */
#define		PACKET_COUNTERS_SRAM_BASE	0x40200800
#endif 		/* IXP_SIMULATION */

#endif /* USE_IMPORT_VAR */

/*---------------------------------------------------------------------
 * iii) Base address for directed broadcast table.
 *-------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

#ifdef 		IXP_SIMULATION
#define		DBCAST_TABLE_SRAM_BASE		0x40401000
#else 		/* hardware */
#define		DBCAST_TABLE_SRAM_BASE		0x40201000
#endif		 /* IXP_SIMULATION */

#endif /* USE_IMPORT_VAR */

/*---------------------------------------------------------------------
 * iv) Base address for control block table.
 *-------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

#ifdef		 IXP_SIMULATION
#define		CONTROL_BLOCK_SRAM_BASE		0x40403000
#else 		/* hardware */
#define		CONTROL_BLOCK_SRAM_BASE		0x40203000
#endif 		/* IXP_SIMULATION */

#endif /* USE_IMPORT_VAR */

/*---------------------------------------------------------------------
 * v) Base address for Ethernet MAC filtering hash table.
 *-------------------------------------------------------------------*/
#ifdef		USE_IMPORT_VAR
.import_var	SRAM_FLT_TABLE_BASE
#else
#ifdef		IXP_SIMULATION
#define 	SRAM_FLT_TABLE_BASE			0x41000000
#else 		/* hardware */
#define		SRAM_FLT_TABLE_BASE			0x40302000
#endif
#endif		/* USE_IMPORT_VAR */

/* Size of an entry in bytes */
#define		SRAM_FLT_ENTRY_SIZE			8

/* Size of the filtering table in bytes */
#define		SRAM_FLT_TABLE_SIZE			1024

/*---------------------------------------------------------------------
 * vi) Base address for port bit map base.
 *-------------------------------------------------------------------*/

/* Input port bit map base: should be be set and updated by core
 * Each port uses 1 bits to define the port type
 *	bit 0 for port 0, bit 1 for port 1,..., bit 15 for port 15
 *	Bit definition:
 *		0 - the port is defined as unit cast mode
 *		1 - the port is defined promisc mode
 */
#ifdef		USE_IMPORT_VAR
.import_var	SRAM_PORT_BIT_MAP_BASE
#else
#ifdef		IXP_SIMULATION
#define 	SRAM_PORT_BIT_MAP_BASE		0x41100000
#else 		/* hardware */
#define		SRAM_PORT_BIT_MAP_BASE		0x40301000
#endif
#endif		/* USE_IMPORT_VAR */

/* Size of entry in LW's in the port bit map */
#define		SRAM_PORT_BIT_ENTRY			0x1


/* Hash table size */
#define		HASH_TABLE_SIZE				1024*8

/* Static Bindings (a.k.a targets) for various microblocks. */
#ifdef		ETHER_WITHOUT_L2
#define		DL_SOURCE_NEXT1				BID_ETHER
#else
#define		DL_SOURCE_NEXT1				BID_IPV4
#endif

/* Define blocks id's for ether decap */
#define		BID_ETHER_DECAP_NEXT1		ETHER_IPV4_TYPE
#define		BID_ETHER_DECAP_NEXT2		ETHER_IPV6_TYPE
#define		BID_ETHER_DECAP_NEXT3		ETHER_ARP_TYPE

/* Include default settings */

/* This file is also used by script files that setup the system when run
 * in simulation mode. When symbol IXP_SCRIPTS is defined, it indicates
 * that scripts are using this file. Since script files cannot access
 * path definitions, they should not encounter '#include <file>'. Instead
 * they include the <file> using Simulation -> Options -> Startup.
 */
#ifndef IXP_SCRIPTS
#include <dl_ingress_system_default.h>
#endif

#endif /* __DL_SYSTEM__H__ */