/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      xscale_me_media.c
 *
 * = DESCRIPTION
 * 		Setup the Pt Lone media card.
 *
 * = AUTHOR
 *      Eswar Eduri
 *      eswar.m.eduri@intel.com
 *
 * = CHANGE HISTORY
 *      05/21/03 	Eswar Eduri	Created.
 * ============================================================================
 * $Id: xscale_me_media.c,v 1.7 2003/09/30 20:32:50 emeduri Exp $
 */

#include	"xscale_me_init.h"

/* device driver common includes */
#ifdef	__linux
#include 	"ixf1104ce_linux_driver.h"
#else
#include <vxworks.h>
#include <stdLib.h>
#include <fioLib.h>
#include <stdio.h>
#include <intLib.h>
#include <iv.h>
#include	"ixf1104ce_driver_api.h"
#endif

#include 	"ixf1104ce_ioctl.h"
#include 	"ixf1104ce_internal.h"

#define	SPHY_4X8

#define SPHY_MODE 	0x10
#define MPHY_MODE 	0x0
#define EVEN_PARITY 0x8
#define ODD_PARITY 	0x0
#define FIBER 		0x0


/**
 * Static function prototypes
 */

/**
 * Variable declarations global to this file only.
 */


/**
 * function definitions.
 */

#ifdef __linux

uint32 GbEMAC_DeviceStart(
    uint32 arg_PortMask,
    uint32 *arg_Handle,
    uint32 arg_Mode)				
{

    uint32 s_DeviceNumber = 0; /* intended for only one card on media side */
    uint32 i;
    uint32 s_Result;
    gbe_mac_ioctl_ptr g_GbeMacIoctlPtr;
    gbe_mac_ioctl_ptr_64 g_GbeMacIoctlPtr64;

    for (i = 0; i < 4; i++)
    {
        /* Enable each port */
        if ((arg_PortMask & (1 << i)) == 0)
        {
            /* This port is not to be enabled */
            continue; 
        }

        /* Fill the Port number and the value in the IOCTL structure */
        g_GbeMacIoctlPtr64.portIndex = i;
        g_GbeMacIoctlPtr64.valueHigh = arg_Mode;
        g_GbeMacIoctlPtr64.valueLow = 0; /* not used */

        /* Configure each port */
        s_Result = gbe_mac_config_handler(s_DeviceNumber,
                                          SET_CONFIG_MODE,
                                          (void *)&g_GbeMacIoctlPtr64);

        if(s_Result != SUCCESS)
        {
            DBG_MSG("Media port %d configuration failed.\n", i);
            return(s_Result);
        }


        /* Fill the Port number and the value in the IOCTL structure */
        g_GbeMacIoctlPtr.portIndex = i;
        g_GbeMacIoctlPtr.value = (1 << i);

        /* Call the driver handler routine for opening the port */
        s_Result = gbe_mac_config_handler(s_DeviceNumber,
                                          SET_PORT_INIT_ENABLE,
                                          (void *)&g_GbeMacIoctlPtr);

        if(s_Result != SUCCESS)
        {
            DBG_MSG("Media port %d initialization failed.\n", i);
            return(s_Result);
        }
    }

    return(s_Result);

} /* GbEMAC_DeviceStart */


uint32 GbEMAC_Ioctl(
    uint32 *arg_Handle, 
    uint32 ioctl_command, 
    void *ioctl_struct)
{
    uint32 s_Result;
    uint32 s_deviceNumber=0; /* intended for only 1 card */
    uint32 i=0;


    /* Check for NULL pointer */
    if ((void *)ioctl_struct == NULL)
    {
        DBG_MSG("IOCTL_BUFFER_POINTER_NULL\n");
        return IOCTL_BUFFER_POINTER_NULL;
    }

    s_Result = gbe_mac_config_handler(s_deviceNumber, ioctl_command, ioctl_struct);
    if (s_Result != SUCCESS)
    {
        DBG_MSG("Media port %d Ioctl failed.\n", i);
        return(s_Result);
    }

    return s_Result;

} /* GbEMAC_Ioctl */

#endif /* __linux */

typedef struct driv_app_s_ioctl_ptr {
    uint32  portIndex;             /* Port Number */
    uint32  value;                 /* Buffer which hold the the value to be set/get */
} driv_app_ioctl_ptr;

/*
 * This function can only be called from EGRESS
 */
void fiber_cfg(void)
{
    uint32 s_ReturnValue;
    uint32 i=0;
    uint32 handle=0,portMask=0;

    driv_app_ioctl_ptr ioctlPtr;
    driv_app_ioctl_ptr appIoctl;	
    driv_app_ioctl_ptr *pIoctlPtr=&ioctlPtr;
    driv_app_ioctl_ptr *pAppIoctl=&appIoctl;	


	/*
	 * start in SPHY 4X8 mode
	 */
    portMask=0xf;   
#ifdef	SPHY_4X8
    s_ReturnValue=GbEMAC_DeviceStart(portMask, &handle, SPHY_MODE | EVEN_PARITY |FIBER); 
	DBG_MSG("Devices started in SPHY 4X8 mode\n");
#else
    s_ReturnValue=GbEMAC_DeviceStart(portMask, &handle, EVEN_PARITY |FIBER); 
	DBG_MSG("Devices started in MPHY 4 mode\n");
#endif

    if(s_ReturnValue == SUCCESS)
    { 	
		/* The call returns SUCCESS */
  
     	for(i=0;i<4;i++)
     	{
	  		/*
	    	 * Set parameters for port 0, 1, 2, and 3
 	    	 */

		/* max frame size = 9216 bytes */
 	   	pAppIoctl->portIndex = i;
   	 	pAppIoctl->value = 0x2400;
    	if (GbEMAC_Ioctl(&handle, SET_MAX_FRAME_SIZE, pAppIoctl) != SUCCESS)
		{
			DBG_MSG("IOCTL SET_MAX_FRAME_SIZE=%d failed for port %d\n",
						pAppIoctl->value,i);
		}
		else
		{
			DBG_MSG("IOCTL SET_MAX_FRAME_SIZE=%d for port %d\n",
						pAppIoctl->value,i);
		}

		/* TFIFO high watermark = 0x3E0 (default)*/
 	   	pAppIoctl->portIndex = i;
   	 	pAppIoctl->value = ((1024 * 7.75)/8);
    	if (GbEMAC_Ioctl(&handle, SET_TX_FIFO_HIGH_WATERMARK, pAppIoctl) != SUCCESS)
		{
			DBG_MSG("IOCTL SET_TX_FIFO_HIGH_WATERMARK=%d failed for port %d\n",pAppIoctl->value,i);
		}
		else
		{
			DBG_MSG("IOCTL SET_TX_FIFO_HIGH_WATERMARK=%d for port %d\n",pAppIoctl->value,i);
		}

		/* TFIFO low watermark */
	 	pAppIoctl->portIndex = i;
   	 	pAppIoctl->value = ((1024 * 5.625)/8);
    	if(GbEMAC_Ioctl(&handle, SET_TX_FIFO_LOW_WATERMARK, pAppIoctl) != SUCCESS)
		{
			DBG_MSG("IOCTL SET_TX_FIFO_LOW_WATERMARK=%d failed for port %d\n",pAppIoctl->value,i);
		}
		else
		{
			DBG_MSG("IOCTL SET_TX_FIFO_LOW_WATERMARK=%d for port %d\n",pAppIoctl->value,i);
		}

		/* MAC Threshold = 0x1BE (default) */
	 	pAppIoctl->portIndex = i;
   	 	pAppIoctl->value = ((1024 * 3.485)/8);
    	if(GbEMAC_Ioctl(&handle, SET_MAC_THRESHOLD, pAppIoctl) != SUCCESS)
		{
			DBG_MSG("IOCTL SET_MAC_THRESHOLD=%d failed for port %d\n",pAppIoctl->value,i);
		}
		else
		{
			DBG_MSG("IOCTL SET_MAC_THRESHOLD=%d for port %d\n",pAppIoctl->value,i);
		}

		} /* end for */
 
		/*
		 * Enable all ports
		 */
    	pIoctlPtr->portIndex = 0;
    	pIoctlPtr->value = 0xFF;		/* enable ports */
    	GbEMAC_Ioctl(&handle, SET_PORT_STATUS, pIoctlPtr);

    }
    else
    { 
		/* Device Starts call fails */
  		DBG_MSG("Device start FAILED for portmask 0x%x",portMask);
	}

 
}




