//---------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//---------------------------------------------------------------------
///////////////////////////////////////////////////////////////////////
//
//
//      File Name: 	dl_source.uc
//
//      Purpose: 	Contains macros for dl_sink and dl_source for POS
//                  and IPv4 Blocks.
//
///////////////////////////////////////////////////////////////////////

#ifndef _DL_SOURCE_UC_
#define _DL_SOURCE_UC_

#include <stdmac.uc>
#include <sig_macros.uc>
#include <dispatch_loop.h>


/////////////////////////////
//	Some local defines
/////////////////////////////

#if (DL_RING_NUMBER > 11)
#error			"For Ring Number Greater than 11, we cannot use the \
					SCR_Ring#_Full input state. Check RING_NUMBER"
#else
#define_eval	DL_RING_FULL		SCR_Ring/**/DL_RING_NUMBER/**/_Full	
#endif


/////////////////////////////
// Global variables/Registers
/////////////////////////////


.reg 	global	dl_ring					// scratch ring number for pos rx ==>ipv4. This can be made an absolut register
.reg 	global	dl_buf_handle			// The current Buffer handle.
.reg	global	dl_eop_buf_handle		// For large packets, this is the last buffer in the chain
.reg	global	dl_next_block			// Next block that should process the buffer/packet

///////////////////////////////////////////////////////////////////////////////
// _dl_pos_sink_init:
//	 	Description: 
//			Initialisation for POS dl_sink. Should be called before using dl_sink.
//			It simply sets a global variable dl_ring to the corresponding ring number
//			This will later save one instruction in dl_sink (for POS).
//
//			[Note: This is an internal macro not meant to be used by the user.
//			This macro will undergo some more changes.]
//
//	 	Outputs:
//			
//		Inputs:
//		
//		Size:   				:
//			1 instruction
//

#macro _dl_pos_sink_init()
.begin

	alu_shf[dl_ring, --, B, DL_RING_NUMBER, <<2]	; ring number to be used in get/put 

.end
#endm

/////////////////////////////////////////////////////////////////////////////
// dl_qm_sink
//	
//	Description:
//		Send received packet to the qm block by writing scratch ring
//		For ATM the third long word has following format
//			   31 30   28 27		 12 11			0
//			   ---------------------------------------
//			   |1|no use | output port | Class ID    |
//			   ---------------------------------------
//
//		For None ATM the third long word has following format
//			   31 30	  21 20		       4 3		0
//			   ---------------------------------------
//			   |1| no use   | output port	|Class ID|
//			   ---------------------------------------
//	Outputs:	
//		None
//		
//	Inputs:
//		s_hl:			SOP buffer handle register, this register value will be changed
//						after macroc is excuted.  If the caller want to keep this value  
//						the value should be stored some where 
//		e_e_hl:			Encoded EOP buffer handle: NULL for one buffer cframe
//						otherwise will be real EOP handle  
//		cls_n:			Class number
//		o_n:			Output port number
//		p_size:			packet size
//		flow_color_id 	flow and color information 
//		rx_sig:			Signal used to wait receving logid wake up  
//		scr_sig:		Scratch ring write signal
//		rt_lbl:			return lable after waked up by receiving logic
//		qm_ring_num:	QM receiving ring number
//	CONSTANTS:	
//		None
//	
//
//	Size: 8 instructions
/////////////////////////////////////////////////////////////////////////////

#macro dl_qm_sink(s_hl, e_e_hl, cls_n, o_n, p_size, flow_color_id, rx_sig, scr_sig,\
					 rt_lbl, qm_ring_num)
.begin
/////////////////////////////////////////////////////////////////////////////
// Local register declaration

.reg tmp_0													; temporary gpr 0

	xbuf_alloc[$sink, 3, write]								; alloc $sink
	alu[$sink[0], --, B, s_hl]								; set SOP hndl to $sink0
	alu[$sink[1], --, B, e_e_hl]								; set EOP hndl to $sink1
	alu[tmp_0, cls_n, or, o_n, <<NUM_CLASS_BITS]
	alu[$sink[2], tmp_0, or,1, <<31 ]							; set MSB in $sink1
	scratch[put, $sink[0], 0, qm_ring_num, 3],\
			sig_done[scr_sig]								; put to SCR ring
	ctx_arb[rx_sig, scr_sig],and,br[rt_lbl]					; ctx swap
	xbuf_free[$sink]										; free $sink
.end
#endm


#if (L2_ENCAP_RING_OUT_0 > 11)
#error "For ring number > 11, we cannot check for scratch ring \
		full condition."
#else
#define_eval RING_NO    				L2_ENCAP_RING_OUT_0
#define_eval L2_ENCAP_RING_OUT_0_FULL	SCR_Ring/**/RING_NO/**/_Full 
#undef RING_NO
#endif

//------------------------------------------------------------------
// _llc_encap_source()
//
//    Description: Read from the source scratch ring.
//
//    Parameters:
//        Outputs:
//		  out_scr_ring_lw :
//			  s-xfer-rd registers containing the result of scratch[get].
//        Inputs:
//        ring_in :
//            Source scratch ring.
//        scr_ring_sig:
//            Signal used to indicate completion of scratch[get].
//		  NUM_LW:
//			  Number of lw's to read.
//
//------------------------------------------------------------------
#macro _llc_encap_source(src_sink_data, src_ring, src_ring_sig, sigmask, NUM_LW)
.begin

	scratch[get, $src[0], src_ring, 0, NUM_LW], \
		sig_done[src_ring_sig]

	; Wait on all pending signals
	ctx_arb[--], defer[2]

	; Add src_ring_sig to sigmask.
	alu[sigmask, sigmask, OR, 1, <<(&src_ring_sig)]

	; ctx_arb on sigmask.
	local_csr_wr[active_ctx_wakeup_events, sigmask]

	.io_completed src_ring_sig

	signal_ctx(next_thread)

	; Copy source scratch ring contents to the scratch ring cache.
	#define_eval i	0
	#while(i < NUM_LW)
	alu[src_sink_data[i], --, B, $src[i]]
	#define_eval i	(i+1)
	#endloop	
	#undef i

	; If source scratch ring is empty, execute null path.
	alu[--, $src[0], -, 0]
	beq[null_path#]

	; If the packet needs to be dropped, execute drop queue path.
	.reg queue
	alu[queue, 0, +16, $src[2]]
	alu[--, drop_queue, -, queue]
	beq[drop_queue_path#]

	; Normal path.
	alu[dl_next_block, --, b, BID_LLC_ENCAP]
	alu[dl_buf_handle, --, b, $src[0]]
	alu[dl_eop_buf_handle, --, b, $src[1]]

	dl_meta_load_cache(dl_buf_handle, $dl_meta, meta_rd_sig, 0, 5)

	ctx_arb[prev_thread_done_sig, meta_rd_sig]

	signal_ctx(next_thread)

	; Copy sop buffer meta-data to the meta-data cache.
	dl_meta_init_cache($dl_meta[0], $dl_meta[1], $dl_meta[2],
		$dl_meta[3], $dl_meta[4], 0, 0, 0)

	br[end#]

null_path#:

	.begin

	alu[dl_next_block, --, b, IX_NULL]
	alu[dl_buf_handle, --, b, IX_NULL]
	alu[dl_eop_buf_handle, --, b, IX_NULL]

	ctx_arb[prev_thread_done_sig]

	signal_ctx(next_thread)

	.reg zero
	immed[zero, 0]
	dl_meta_init_cache(zero, zero, zero, zero, zero, 0, 0, 0)

	br[end#]

	.end

drop_queue_path#:

	.begin

	alu[dl_next_block, --, b, BID_QM]
	alu[dl_buf_handle, --, b, IX_NULL]
	alu[dl_eop_buf_handle, --, b, IX_NULL]

	ctx_arb[prev_thread_done_sig]

	signal_ctx(next_thread)

	.reg zero
	immed[zero, 0]
	dl_meta_init_cache(zero, zero, zero, zero, zero, 0, 0, 0)

	.end

end#:

	; Reset sigmask_b to its default value.
	alu[sigmask_b, --, B, default_sigmask_b]

.end
#endm


//------------------------------------------------------------------
// _llc_encap_sink()
//
//    Description: Write to the sink scratch ring.
//
//    Parameters:
//        Inputs:
//        ring_out :
//            Sink scratch ring.
//		  in_scr_ring_lw :
//			  s-xfer-wr registers used for scratch[put].
//        scr_ring_sig:
//            Signal used to indicate completion of scratch[put].
//		  NUM_LW:
//			  Number of lw's to write.
//
//------------------------------------------------------------------
#macro _llc_encap_sink(src_sink_data, sink_ring, sink_ring_sig, sigmask, NUM_LW)
.begin

	alu[--, dl_next_block, -, BID_QM]
	bne[end#]

	; Write sop buffer meta-data cache to sram.

	dl_meta_flush_cache[$dl_meta, dl_buf_handle, meta_wr_sig, SIG_NONE, 0, 3]

	alu[sigmask_b, sigmask_b, OR, 1, <<(&meta_wr_sig)]

check_ring_full#:

	; Check if ring is full.
 	br_inp_state[L2_ENCAP_RING_OUT_0_FULL, check_ring_full#]

	; Copy the scratch ring cache contents into s-xfer-wr registers.
	#define_eval i	0
	#while(i < NUM_LW)
	alu[$sink[i], --, B, src_sink_data[i]]
	#define_eval i	(i+1)
	#endloop	
	#undef i

	alu[sigmask, sigmask, OR, 1, <<(&sink_ring_sig)]

	; If not full, write to scratch ring.	
	scratch[put, $sink[0], sink_ring, 0, NUM_LW], \
		sig_done[sink_ring_sig]

end#:

.end
#endm

#endif // _DL_SOURCE_UC_