/*--------------------------------------------------------------------*/
/*                                                                    */   
/*                   I N T E L   P R O P R I E T A R Y                */    
/*                                                                    */    
/*      COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS       */   
/*      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY   */   
/*      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A */   
/*      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER */   
/*      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL, */   
/*      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT */   
/*      THE PRIOR WRITTEN PERMISSION OF :                             */   
/*                                                                    */   
/*                         INTEL  CORPORATION                         */   
/*                                                                    */  
/*                      2200 MISSION COLLEGE BLVD                     */   
/*                                                                    */   
/*                SANTA  CLARA,  CALIFORNIA  95052-8119               */   
/*                                                                    */    
/*--------------------------------------------------------------------*/
/*                                                                    */
/*                                                                    */
/*      File Name: 	dl_system.h                                       */
/*                                                                    */
/*      Purpose: This file contains the system wide parameters that   */
/*               can be customized for a given hardware/application.  */
/*                                                                    */
/*--------------------------------------------------------------------*/

#ifndef	_DL_SYSTEM_H_
#define	_DL_SYSTEM_H_

#include <definitions.h>

/* Run in simulation mode */
#ifndef	SIMULATION
#define	SIMULATION
#endif

/* Chip is defined as IXP2400 */
#ifndef IXP2400
#define IXP2400
#endif

/* Defines TX_PHY_MODE as PHY_1_32 or PHY_4_8 */
#define	TX_PHY_MODE					SPHY_4_8

/* use signal number 'ME_INIT_SIGNAL' to wait for system initialization */
#define USE_ME_INIT_SIGNAL

/* Define number AAL5 TX ME's in a functional pipeline */
/* If this design does not have an AAL5 TX functional pipeline, set to 1 */
#define	AAL5_TX_NUMBER_OF_FP_MEs	1

/* Total TBUF size in bytes */
#define TOTAL_TBUF_SIZE 			8192

/* Size of the TBUF element used by AAL5 TX  */
#define TBUF_ELE_SIZE_LOG2	    	6
#define TBUF_ELE_SIZE	        	(1<<TBUF_ELE_SIZE_LOG2)

/* Size of TCW in bytes */
#define TCW_SIZE_LOG2				3

/* Define number of partitions for TBUF */
#define AAL5_TX_PARTITIONS_NUMBER   4

/* Define number of threads allocated for each channel (or partition) */
#define NUM_OF_THREADS_PER_CHANNEL  4

/* Do MSF TX initialization in AAL5 TX microblock */
#define AAL5_TX_ME_INIT_MSF

/* Define AAL5 cell padding value */
#define AAL5_TX_CELL_PADDING		0x6A6A6A6A

#define	SRAM_REST_CHANNEL_NUMBER	0


/*  First lw in SRAM is zero'ed out.
    Used by some microblocks for initialization */

#define         SRAM_ZERO_BLOCK 		0x0

/* Buffer free list id used by the core. */

#define         FREE_LIST_ID            0x0

/* Base address in SRAM for the packet buffers.
   This is for the meta data (buffer descriptors). */

#define			BUF_SRAM_BASE			0

/* Amount of SRAM space available for meta data of packet buffers.
   1 MB.  In simulation we use only 20K bytes. */

#ifdef IXP_SIMULATION
#define_eval	BUF_SRAM_SIZE			(1024 * 20 )
#else
#define_eval	BUF_SRAM_SIZE			(1024 * 20 * 20 * 2)
#endif

/* Base Address in DRAM for the packet buffers.
   This is for the actual packet data. */

#define			BUF_SDRAM_BASE			0

/* Amount of SDRAM space available for packet buffers. 64 MB.
   In simulation we use only 1280K bytes. */

#ifdef IXP_SIMULATION
#define_eval	BUF_SDRAM_SIZE			(64 * 1024 * 20 )
#else
#define_eval	BUF_SDRAM_SIZE			(64 * 1024 * 20 * 20 * 2)
#endif

/* Size of meta data (Buffer Descriptors) in SRAM.
   32 entries per buffer. Should be a power of 2. */

#define			META_DATA_SIZE			32

/* Size of packet buffer. */

#define			BUFFER_SIZE				2048

/* Base Address for Queue Descriptors. */

#define			QD_SRAM_BASE			0x100000

/* Total number of Queue Descriptors. [0..65534] + 1 */

#define			QD_TOTAL				65535

/* Size of each Queue Descriptor Entry (in bytes) */

#define			QD_SIZE					16

/* Base address AAL5 Transmit Contexts in SRAM. */

#define			AAL5_TX_CTX_SRAM_BASE	0x200000

/* Size of each AAL5 Transmit Context (in bytes). */

#define			AAL5_TX_CTX_SIZE		64

/* Total number of AAL5 Transmit Contexts. */
#ifdef IXP_SIMULATION
#define			AAL5_TX_CTX_TOTAL		65536
#else
#define			AAL5_TX_CTX_TOTAL		16384
#endif
/* Base address CSIX Receive Contexts in SRAM */
#ifdef IXP_SIMULATION
#define			CSIX_RX_CTX_SRAM_BASE	0x600000
#else
#define			CSIX_RX_CTX_SRAM_BASE	0x300000
#endif
/* Size of each CSIX Receive Context (in bytes) */

#define			CSIX_RX_CTX_SIZE		32

/* Total number of CSIX Receive Contexts. */

#define			CSIX_RX_CTX_TOTAL		1024

/* Following definitions are used as counters by
   respective microblocks in SRAM */

/* CSIX_RX needs 1024 * 4 counters.
   Each counter is 1 LW.
   This implies 1024*4*4 bytes. */
#ifdef IXP_SIMULATION
#define			CSIX_RX_COUNTER_BASE					0x700000
#else
#define			CSIX_RX_COUNTER_BASE					0x308000
#endif

/* Base address for AAL5 TX counters */
#ifdef IXP_SIMULATION
#define			AAL5_TX_COUNTER_BASE					0x704000
#else
#define			AAL5_TX_COUNTER_BASE					0x30C000
#endif
/* AAL5_TX counts #pkts tx per VCQ.
   There are AAL5_TX_CTX_TOTAL VCQ's.
   Each counter is 1 LW (4 bytes).
   This implies 65536*4 bytes */

#ifdef IXP_SIMULATION
#define			AAL5_TX_PKTS_PER_VCQ_COUNTER_BASE		0x704000
#else
#define			AAL5_TX_PKTS_PER_VCQ_COUNTER_BASE		0x30C000
#endif

/* AAL5_TX counts #cells tx per VCQ.
   There are AAL5_TX_CTX_TOTAL VCQ's.
   Each counter is 1 LW (4 bytes).
   This implies 65536*4 bytes */

#ifdef IXP_SIMULATION
#define			AAL5_TX_CELLS_PER_VCQ_COUNTER_BASE		0x744000
#else
#define			AAL5_TX_CELLS_PER_VCQ_COUNTER_BASE		0x31C000
#endif

/* Total number of output ports. */

#ifdef IXP_SIMULATION
#define			AAL5_TX_OUTPORT_TOTAL					65536
#else
#define			AAL5_TX_OUTPORT_TOTAL					16384
#endif

/* AAL5_TX counts #pkts tx per output port.
   There are AAL5_TX_OUTPORT_TOTAL output ports.
   Each counter is 1 LW (4 bytes).
   This implies 65536*4 bytes */

#ifdef IXP_SIMULATION
#define			AAL5_TX_PKTS_PER_OUTPORT_COUNTER_BASE	0x784000
#else
#define			AAL5_TX_PKTS_PER_OUTPORT_COUNTER_BASE	0x32C000
#endif

/* AAL5_TX counts #cells tx per output port.
   There are AAL5_TX_OUTPORT_TOTAL output ports.
   Each counter is 1 LW (4 bytes).
   This implies 65536*4 bytes */

#ifdef IXP_SIMULATION
#define			AAL5_TX_CELLS_PER_OUTPORT_COUNTER_BASE	0x7C4000
#else
#define			AAL5_TX_CELLS_PER_OUTPORT_COUNTER_BASE	0x33C000
#endif

/* Stores the next hop table.
   Size of each entry is 8LW's
   There could be 65536 entries.
   This implies 65536*32 bytes. */

#ifdef IXP_SIMULATION
#define			NEXT_HOP_TABLE_BASE_SRAM				0x804000
#else
#define			NEXT_HOP_TABLE_BASE_SRAM				0x34C000
#endif

/* Size of each next hop entry in bytes. */

#define			NEXT_HOP_ENTRY_SIZE						32

/* 2 ^ PO2_NEXT_HOP_ENTRY_SIZE = NEXT_HOP_ENTRY_SIZE */

#define			PO2_NEXT_HOP_ENTRY_SIZE					5

/* Number of next hop entries */

#ifdef IXP_SIMULATION
#define			NUMBER_OF_NEXT_HOP_ENTRIES				65536
#else
#define			NUMBER_OF_NEXT_HOP_ENTRIES				16384
#endif

/* This defines the number of queue# bits that make up the vcq# */

#define			NUM_CLASS_BITS					16

/*	Scratch for communicating between CSIX RX and L2 encap blocks
	Ring number 0, base 0, and size 512 Long Words */

#define			CSIX_RX_TO_L2_ENCAP_SCR_RING		0
#define			CSIX_RX_TO_L2_ENCAP_SCR_RING_BASE	0
#define			CSIX_RX_TO_L2_ENCAP_SCR_RING_SIZE	512

/*	Scratch for communicating between L2 encap and QM blocks
	Ring number 1, base 2048, and size 512 Long Words */

#define			L2_ENCAP_TO_QM_SCR_RING				1
#define			L2_ENCAP_TO_QM_SCR_RING_BASE		2048
#define			L2_ENCAP_TO_QM_SCR_RING_SIZE		512

/*	Scratch for communicating between Scheduler and Queue Manager Blocks
	Ring number 2, base 4096, and size 512 Long Words */

#define			SCHEDULER_TO_QM_SCR_RING			2
#define			SCHEDULER_TO_QM_SCR_RING_BASE		4096
#define			SCHEDULER_TO_QM_SCR_RING_SIZE		512

/*	Scratch for communicating between Queue Manager and AAL5 Transmit Blocks
	Ring number 3, base 6144, and size 512 Long Words */

#if (TX_PHY_MODE == SPHY_1_32)

#define 		QM_TO_AAL5_TX_SCR_RING_0			3
#define 		QM_TO_AAL5_TX_SCR_RING_0_BASE		6144
#define 		QM_TO_AAL5_TX_SCR_RING_0_SIZE		512

#endif /* (TX_PHY_MODE == SPHY_1_32) */

#if ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4))

#define 		QM_TO_AAL5_TX_SCR_RING_0			3
#define 		QM_TO_AAL5_TX_SCR_RING_0_BASE		6144
#define 		QM_TO_AAL5_TX_SCR_RING_0_SIZE		128

#define 		QM_TO_AAL5_TX_SCR_RING_1			4
#define 		QM_TO_AAL5_TX_SCR_RING_1_BASE		6656
#define 		QM_TO_AAL5_TX_SCR_RING_1_SIZE		128

#define 		QM_TO_AAL5_TX_SCR_RING_2			5
#define 		QM_TO_AAL5_TX_SCR_RING_2_BASE		7168
#define 		QM_TO_AAL5_TX_SCR_RING_2_SIZE		128

#define 		QM_TO_AAL5_TX_SCR_RING_3			6
#define 		QM_TO_AAL5_TX_SCR_RING_3_BASE		7680
#define 		QM_TO_AAL5_TX_SCR_RING_3_SIZE		128

#endif /* ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4)) */

/*	Scratch locations used by AAL5 TX */

/* Stores next available TBUF element */

#define			AAL5_TX_AVAIL_TBUF_ELEMENT_SCRATCH_BASE		10240

#define			AAL5_TX_AVAIL_TBUF_ELEMENT_0		10240
#define			AAL5_TX_AVAIL_TBUF_ELEMENT_1		10244
#define			AAL5_TX_AVAIL_TBUF_ELEMENT_2		10248
#define			AAL5_TX_AVAIL_TBUF_ELEMENT_3		10252

/* Stores cells transmitted per partition */
/* There are 4 partitions. Total bytes needed = 4*4 = 16 */

#define			AAL5_TX_CELLS_TX_SCRATCH_BASE		10256

#define			AAL5_TX_CELLS_TX_0					10256
#define			AAL5_TX_CELLS_TX_1					10260
#define			AAL5_TX_CELLS_TX_2					10264
#define			AAL5_TX_CELLS_TX_3					10268


/* These defines are used by the individual microcode to identify
 their scratch rings. The reason for these defines is to improve
 the portability of microblocks. User just has to make changes
 here and the microblocks will remain untouched.
 For the naming convention, the microblock name goes first, then
 the _RING_ specifier. Following this is the direction indicator.
 '_OUT' would imply the microblock is writing to this scratch ring
 and '_IN' would imply the microblock is reading from this scratch 
 ring. If there is more than one 'IN' or 'OUT' ring the name will
 have '_<digit>' to differentiate the rings. */

#define_eval 	CSIX_RX_RING_OUT			CSIX_RX_TO_L2_ENCAP_SCR_RING
#define_eval	CSIX_RX_RING_OUT_BASE		CSIX_RX_TO_L2_ENCAP_SCR_RING_BASE
#define_eval	CSIX_RX_RING_OUT_SIZE		CSIX_RX_TO_L2_ENCAP_SCR_RING_SIZE

#define_eval	L2_ENCAP_RING_IN_0			CSIX_RX_TO_L2_ENCAP_SCR_RING
#define_eval	L2_ENCAP_RING_IN_0_BASE		CSIX_RX_TO_L2_ENCAP_SCR_RING_BASE
#define_eval	L2_ENCAP_RING_IN_0_SIZE		CSIX_RX_TO_L2_ENCAP_SCR_RING_SIZE

#define_eval	L2_ENCAP_RING_OUT_0			L2_ENCAP_TO_QM_SCR_RING
#define_eval	L2_ENCAP_RING_OUT_0_BASE	L2_ENCAP_TO_QM_SCR_RING_BASE
#define_eval	L2_ENCAP_RING_OUT_0_SIZE	L2_ENCAP_TO_QM_SCR_RING_SIZE
	 
#define_eval	QM_RING_IN_0				L2_ENCAP_TO_QM_SCR_RING
#define_eval	QM_RING_IN_0_BASE			L2_ENCAP_TO_QM_SCR_RING_BASE
#define_eval	QM_RING_IN_0_SIZE			L2_ENCAP_TO_QM_SCR_RING_SIZE

/* Defined since the QM microblock currently uses this format */

#define_eval	QM_RING_IN					QM_RING_IN_0
#define_eval	QM_RING_IN_BASE				QM_RING_IN_0_BASE
#define_eval	QM_RING_IN_SIZE				QM_RING_IN_0_SIZE

#define_eval	QM_RING_IN_1				SCHEDULER_TO_QM_SCR_RING
#define_eval	QM_RING_IN_1_BASE			SCHEDULER_TO_QM_SCR_RING_BASE
#define_eval	QM_RING_IN_1_SIZE			SCHEDULER_TO_QM_SCR_RING_SIZE

#define_eval	QM_RING_OUT_0				QM_TO_AAL5_TX_SCR_RING_0
#define_eval	QM_RING_OUT_0_BASE			QM_TO_AAL5_TX_SCR_RING_0_BASE
#define_eval	QM_RING_OUT_0_SIZE			QM_TO_AAL5_TX_SCR_RING_0_SIZE

/* Defined since the QM microblock currently uses this format */

#define_eval	QM_RING_OUT					QM_RING_OUT_0
#define_eval	QM_RING_OUT_BASE			QM_RING_OUT_0_BASE
#define_eval	QM_RING_OUT_SIZE			QM_RING_OUT_0_SIZE

#define_eval	QM_RING_OUT_1				QM_TO_AAL5_TX_SCR_RING_1
#define_eval	QM_RING_OUT_1_BASE			QM_TO_AAL5_TX_SCR_RING_1_BASE
#define_eval	QM_RING_OUT_1_SIZE			QM_TO_AAL5_TX_SCR_RING_1_SIZE

#define_eval	QM_RING_OUT_2				QM_TO_AAL5_TX_SCR_RING_2
#define_eval	QM_RING_OUT_2_BASE			QM_TO_AAL5_TX_SCR_RING_2_BASE
#define_eval	QM_RING_OUT_2_SIZE			QM_TO_AAL5_TX_SCR_RING_2_SIZE

#define_eval	QM_RING_OUT_3				QM_TO_AAL5_TX_SCR_RING_3
#define_eval	QM_RING_OUT_3_BASE			QM_TO_AAL5_TX_SCR_RING_3_BASE
#define_eval	QM_RING_OUT_3_SIZE			QM_TO_AAL5_TX_SCR_RING_3_SIZE

#define_eval	SCHEDULER_RING_OUT_0		SCHEDULER_TO_QM_SCR_RING
#define_eval	SCHEDULER_RING_OUT_0_BASE	SCHEDULER_TO_QM_SCR_RING_BASE			
#define_eval	SCHEDULER_RING_OUT_0_SIZE	SCHEDULER_TO_QM_SCR_RING_SIZE
								
#define_eval	AAL5_TX_IN_0				QM_TO_AAL5_TX_SCR_RING_0
#define_eval	AAL5_TX_IN_0_BASE			QM_TO_AAL5_TX_SCR_RING_0_BASE
#define_eval	AAL5_TX_IN_0_SIZE			QM_TO_AAL5_TX_SCR_RING_0_SIZE

#define_eval	AAL5_TX_IN_1				QM_TO_AAL5_TX_SCR_RING_1
#define_eval	AAL5_TX_IN_1_BASE			QM_TO_AAL5_TX_SCR_RING_1_BASE
#define_eval	AAL5_TX_IN_1_SIZE			QM_TO_AAL5_TX_SCR_RING_1_SIZE

#define_eval	AAL5_TX_IN_2				QM_TO_AAL5_TX_SCR_RING_2
#define_eval	AAL5_TX_IN_2_BASE			QM_TO_AAL5_TX_SCR_RING_2_BASE
#define_eval	AAL5_TX_IN_2_SIZE			QM_TO_AAL5_TX_SCR_RING_2_SIZE

#define_eval	AAL5_TX_IN_3				QM_TO_AAL5_TX_SCR_RING_3
#define_eval	AAL5_TX_IN_3_BASE			QM_TO_AAL5_TX_SCR_RING_3_BASE
#define_eval	AAL5_TX_IN_3_SIZE			QM_TO_AAL5_TX_SCR_RING_3_SIZE

/* First 16 (0-15) SRAM Q-Array entries are allocated for the Queue Manager */

#define			QM_QARRAY_BASE				0
#define			QM_QARRAY_SIZE				16

/* Next 4 (16-19) SRAM Q-Array entries are allocated for buffer free list */

#define			BUF_QARRAY_BASE				16
#define			BUF_QARRAY_SIZE				4

/* Next 1 (20) SRAM Q-Array entry is used for Drop Queue in Queue Manager. */

#define			QM_DROP_QUEUE_ENTRY			20

/* Queue Number for Drop Queue.
   (0-65534 are the normal queues used in forwarding pkts) */

#define			QM_DROP_QUEUE				QD_TOTAL //65535 //1024

/*	These tell if a the executing threads need to work in order.
    i.e wait for a signal from prev thread, do some work and
    signal next thread. */

#define			DL_THREAD_NO_ORDER			0x0
#define			DL_THREAD_ORDER				0x1


/* All the IX_* constants should be between 0 and 255 (i.e 1 byte)
   to enable using them as immed operands in instructions. */

/* IX_NULL is used to denote a NULL (or empty) packet handle.
   It should not be zero, as it would interfere with ring empty
   (0) condition. */

#define			IX_NULL						0xFF
#define			IX_EXCEPTION				0x1
#define			IX_DROP						0x2

/* Block IDs for various blocks. Block IDs Start from 0x20.
   (0 to 0x1f and 0xFF are reserved for IX_* ) */

#define			BID_POS						0x21
#define			BID_IPV4					0x22
#define			BID_QM						0x23
#define			BID_LLC_ENCAP				0x24

/* Static Bindings (a.k.a targets) for various microblocks. 
   Should this go into project/application specific global
   header file? */

#define			DL_SOURCE_NEXT1				BID_IPV4
#define			POS_RX_NEXT1				BID_IPV4
#define			IPV4_NEXT1					BID_QM
#define			BID_LLC_ENCAP_NEXT1			BID_QM


/* To be Redefined as an import variable. And its value should
   be one of the above Block IDs. */

#ifndef			BID_NEXT_BLOCK
#define			BID_NEXT_BLOCK				IX_NULL					
#endif

/*	BLADE ID */
#define			THIS_BLADE_ID				1

/* Signals being a precious resource, we use a single signal
   for inter-thread as well as any other inter-me signalling
   (init over signal) */

#define			COMMON_INTER_THD_ME_SIGNAL	10

/* We are not able to use a single signal before of limitations
   in assembler. Until then, we need a separate signal for
   inter-thread signalling. */

#define			INTER_THD_SIGNAL			12

/* ME_NIT_SIGNAL signal is used to let everyone know that
   initialisation is over and all blocks in all MEs can start
   running. This signal will be typically issued by the system
   initialisation code and all other MEs should wait for this
   signal before starting to run. */

#define			ME_INIT_SIGNAL				COMMON_INTER_THD_ME_SIGNAL

/*	Source and Sink related defines */

/* Specify if next neighbor ring is used.
   If not, default is scratch rings. */

/* 	Valid values for RING_SIZE are 128, 256, 512 or 1024. */

#define 		DL_RING_SIZE				CSIX_RX_RING_OUT_SIZE

/* Base Address for Scratch Ring is 0x1000.
   Should be aligned on a 4 x RING_SIZE byte boundary. */

#define 		DL_RING_BASE				CSIX_RX_RING_OUT_BASE


/* 	Ring number to use. 0-15. */

#define_eval 	DL_RING_NUMBER				CSIX_RX_RING_OUT 

/* 	Signal number to use in inter thread signalling.  */

#define 		DL_SIG_WAKE					INTER_THD_SIGNAL

/* Drop buffer chain ring */

#define_eval 	DL_DROP_RING				QM_RING_IN_0


/*-----------------------------------------------------------------*/
/*                                                                 */
/* _LN                                                             */
/*                                                                 */
/*	 	Description: Returns log (input) to base 2.                */ 
/*      i.e 2 ** result = input. In other words, given a value     */
/*      (it should be a power of 2) this macro will return the     */
/*      shift amount.                                              */
/*                                                                 */
/*		This is not like normal macros. This does not produce any  */
/*      code and is expected to be used in pre-processing.         */
/*                                                                 */
/*		Parameters:                                                */
/*	 	Outputs:                                                   */
/*			The result will be available in LN_RESULT              */
/*          (it's a #define)                                       */
/*                                                                 */
/*		Inputs:                                                    */
/*			value		: This should be a power of 2.             */
/*                                                                 */
/*		Size:                                                      */
/*			0 instructions                                         */
/*                                                                 */
/*-----------------------------------------------------------------*/
#macro	_POWER_OF_2(value)

.begin

#define_eval			LN_RESULT		0
#define_eval			LN_VALUE	 	value	

	/*	Find the bit position where a '1' (and only one '1') is set
		in value. */

#while (LN_VALUE)

	#define_eval		IS_SET		(LN_VALUE & 0x1)

	#if	(IS_SET == 0)

		#define_eval	LN_RESULT	(LN_RESULT+1)
		#define_eval	LN_VALUE	(LN_VALUE>>1)
	#else

		#define_eval	LN_VALUE	0

	#endif

#endloop

#undef 	IS_SET
#undef	LN_VALUE

	/*  At the end, LN_RESULT will contain the result. */

.end
#endm

#endif /* _DL_SYSTEM_H_ */