//---------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//---------------------------------------------------------------------
///////////////////////////////////////////////////////////////////////
//
//
//      File Name: system_init.uc
//
//      Purpose: Contains macros used once during system intialization.
//
///////////////////////////////////////////////////////////////////////

#ifndef _SYSTEM_INIT_UC_
#define _SYSTEM_INIT_UC_

#include <dispatch_loop.uc>

///////////////////////////////////////////////////////////////////////
// init_meta_cache_to_zero:
//	 	Description: 
//			Just sets the registers used for meta data caching to zero.
//          [This helps a lot when debugging, because all the unused
//          meta data regsiters will be zero instead of something
//          like 0x5a5a5a].
//
//		Outputs:
//
//	 	Inputs:
//
//		Size:   				
//
#macro	init_meta_cache_to_zero()
.begin
.reg zero

	immed32[zero, 0]
	dl_meta_init_cache(zero, zero, zero, zero, zero, zero, zero, zero)

.end
#endm

///////////////////////////////////////////////////////////////////////
// system_init:
//	 	Description: 
//			Does all the initialization to get the system up and running.
//			Typically it initializes all the various rings, creates the
//          buffer free list, initializes the various cache
//          (meta, ip header etc).
//			Once initialization is done, a signal is sent to all other MEs
//			so that rest of the code, that depends on this initialization, 
//			to run.
//
//		Outputs:
//
//	 	Inputs:
//
//		Size:   				
//
//
#macro	system_init()

	// Initialize the various scratch rings used in the ingress pipeline

	init_scratch_ring(CSIX_RX_TO_L2_ENCAP_SCR_RING_BASE,
	                  CSIX_RX_TO_L2_ENCAP_SCR_RING_SIZE,
					  CSIX_RX_TO_L2_ENCAP_SCR_RING)

	init_scratch_ring(L2_ENCAP_TO_QM_SCR_RING_BASE,
	                  L2_ENCAP_TO_QM_SCR_RING_SIZE,
					  L2_ENCAP_TO_QM_SCR_RING)

	init_scratch_ring(SCHEDULER_TO_QM_SCR_RING_BASE,
	                  SCHEDULER_TO_QM_SCR_RING_SIZE,
					  SCHEDULER_TO_QM_SCR_RING)

	init_scratch_ring(QM_TO_AAL5_TX_SCR_RING_0_BASE,
	                  QM_TO_AAL5_TX_SCR_RING_0_SIZE,
					  QM_TO_AAL5_TX_SCR_RING_0)

#if ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4))

	init_scratch_ring(QM_TO_AAL5_TX_SCR_RING_1_BASE,
	                  QM_TO_AAL5_TX_SCR_RING_1_SIZE,
					  QM_TO_AAL5_TX_SCR_RING_1)

	init_scratch_ring(QM_TO_AAL5_TX_SCR_RING_2_BASE,
	                  QM_TO_AAL5_TX_SCR_RING_2_SIZE,
					  QM_TO_AAL5_TX_SCR_RING_2)

	init_scratch_ring(QM_TO_AAL5_TX_SCR_RING_3_BASE,
	                  QM_TO_AAL5_TX_SCR_RING_3_SIZE,
					  QM_TO_AAL5_TX_SCR_RING_3)

#endif // ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4))

	// Initialize the free buffer list

	dl_buf_init()

	//	To ease debugging. Not absoultely essential for proper execution.

	init_meta_cache_to_zero()		; all the dl_meta* gprs will be set to 0.

	//	Signal all MEs, except ME 0 (ME0 is running POS RX & this code)

	signal_me(0x01, ME_INIT_SIGNAL)	; ME 1
	signal_me(0x02, ME_INIT_SIGNAL)	; ME 2
	signal_me(0x03, ME_INIT_SIGNAL)	; ME 3

	// The Next 4 MEs are in Cluster 1. 

	signal_me(0x10, ME_INIT_SIGNAL)	; ME 4
	signal_me(0x11, ME_INIT_SIGNAL)	; ME 5
	signal_me(0x12, ME_INIT_SIGNAL)	; ME 6
	signal_me(0x13, ME_INIT_SIGNAL)	; ME 7

#endm

#endif	// _SYSTEM_INIT_UC_