////////////////////////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                 
//                                                                      
////////////////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: aal5_ipv4_dl.uc
//
//      Purpose: This file contains dispatch loop for IPV4 Forwarding application over
//				 AAL5 (AAL5 frames may or may not have LLC/SNAP header)
//
//      History:
//
//
//      Date            Comment                         			By
//      --------------------------------------------------------------------------------	
//
//      06/10/2002      Created                         			Eswar Eduri
//
//
/////////////////////////////////////////////////////////////////////////////////////////

#ifndef	_AAL5_IPV4_DL_UC_
#define	_AAL5_IPV4_DL_UC_


;-----------------------------------------------------------
;	Include Files
;-----------------------------------------------------------

// File with dispatch loop and system constants.
#include	"dl_system.h"

// File with system init macros.
#include	"system_init.uc"

// File with dispatch loop macros.
#include	"dispatch_loop.uc"

// File with IPV4 forwarder init macros
#include	"ipv4_fwder_init.uc"

// File with IPV4 forwarder macros
#include	"ipv4_fwder.uc"

// File with llc_snap_decap macros
#include 	"llc_snap_decap.uc"

// constants
// None.

;-----------------------------------------------------------
; AAL5 - IPV4 Macro
;-----------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
//
// _aal5_ipv4_fwder
//
//	 	Description: 
//
//			Its a wrapper macro around  _ipv4_fwder() to satisfy the AAL5 
//			specific offset requirements.			
//
//	 	Outputs:
//			N/A
//		
//		Inputs:
//			ip_hdr_offset	-	the offset to point to the start of IP header
//								(If the frame has LLC/SNAP header, offset = 8,
//								 else if frame is not encapsulated, offset = 0)
//
//			__PACKETHDR		- 	buffer containing packet header.
//
//		Size: 
//		
///////////////////////////////////////////////////////////////////////////////

#macro _aal5_ipv4_fwder(ip_hdr_offset, __PACKETHDR)
.begin

		
	alu[--, ip_hdr_offset, -, 8]
	bne[hdr_offset_is_zero#], defer[3]
	// The macro copies only 20 bytes of IP header to the
	// destination register buffer. we need to copy the registers untouched
	// by the above macro. The registers modified is dependent on
	// where the IP Hdr starts in the buffer. 
	alu[__PACKETHDR[7],--,B,__PACKETHDR[7]]
	alu[__PACKETHDR[8],--,B,__PACKETHDR[8]]
	alu[__PACKETHDR[9],--,B,__PACKETHDR[9]]

	; Here the frame has LLC/SNAP header encapsulation.
	 _ipv4_fwder(__PKTHDR,__PKTHDR, IP_HDR_OFFSET_8, IP_HDR_OFFSET_8)

	br[end#], defer[2]	
	alu[__PACKETHDR[0],--,B,__PACKETHDR[0]]
	alu[__PACKETHDR[1],--,B,__PACKETHDR[1]]

hdr_offset_is_zero#:

	; Here the frame is not encapsulated.
	 _ipv4_fwder(__PKTHDR,__PKTHDR, IP_HDR_OFFSET_0, IP_HDR_OFFSET_0)

 	alu[__PACKETHDR[5],--,B,__PACKETHDR[5]]
	alu[__PACKETHDR[6],--,B,__PACKETHDR[6]]


end#:

.end
#endm


;-----------------------------------------------------------
; MAIN CODE starts here
;-----------------------------------------------------------





;-----------------------------------------------------------
; Initialize the blocks
;-----------------------------------------------------------

init#:
	; allocate cache to store the packet header
	dl_iphdr_cache_init[]

#ifndef	__PKTHDR
#define	__PKTHDR	$$iphdr	; for convenience. DL defines this.
#else
#warning	"__PKTHDR is already defined."
#endif

	; init buffer meta data cache to Zero.
	init_meta_cache_to_zero[]

	; Temporary : Initialize EOP handle default.
	_dl_eop_buf_handle_init[]

	; ipv4_fwder initialization.
	_ipv4_fwder_init()

	; now kick start the IPV4 Forwarder.
	_ipv4_fwder_kick_start()

;-----------------------------------------------------------
; dispatch loop
;-----------------------------------------------------------
.begin
.reg sig_mask

	immed32[sig_mask, 0]

	; Get packets in order.
	dl_source[DL_THREAD_ORDER, sig_mask]

.end

loop#:

.reg ip_hdr_offset_reg


	; Perform classification and decapsulation of headers accordingly.
	 alu[ip_hdr_offset_reg, --, b, 0]
	_llc_snap_decap_classify(ip_hdr_offset_reg, __PKTHDR, AAL5_HDR_OFFSET)

	;Process the IP header. The modified packet is in cache
	;registers starting at ip_hdr_offset_reg.

	_aal5_ipv4_fwder(ip_hdr_offset_reg, __PKTHDR)


sink_the_packet#:


.reg sig_mask
.sig sig_scr_put
.set_sig sig_scr_put

	// allocate xfer registers to be used by dl_qm_sink.
	xbuf_alloc[$wxfer, 3, write]
	immed32[sig_mask, 0]

	; send packet.
	dl_qm_sink[sig_mask, ip_hdr_offset_reg, sig_scr_put, $wxfer]

	; receive packet
	dl_source[DL_THREAD_NO_ORDER, sig_mask]

.io_completed sig_scr_put

	// free the xfer registers allocated
	xbuf_free[$wxfer]

	; start again.
	br[loop#]	

// We never execute the below code. This is here for completeness only.

term#:

	// frees the IP header cache
	dl_iphdr_cache_fini[]

	// Control should never come here.
kill#:
	nop
	nop

;-----------------------------------------------------------
; MAIN CODE ends here
;-----------------------------------------------------------



#endif	// 	_AAL5_IPV4_DL_UC_