/*------------------------------------------------------------------------------------
                                                                      
                   I N T E L   P R O P R I E T A R Y                   
                                                                       
      COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
      THE PRIOR WRITTEN PERMISSION OF :                                
                                                                       
                         INTEL  CORPORATION                            
                                                                      
                      2200 MISSION COLLEGE BLVD                        
                                                                       
                SANTA  CLARA,  CALIFORNIA  95052-8119                  
                                                                       
------------------------------------------------------------------------------------



      File Name: 	dl_system.h

      Purpose: 	This file contains the system wide parameters that can be 
					customised for a given hardware/application 

      History:


      Date            Comment                         By
      ---------------------------------------------------------------------

      11/19/2001      Created                         Senthil Nathan


-----------------------------------------------------------------------------------*/

#ifndef	__SYSTEM__H__
#define	__SYSTEM__H__

#include <definitions.h>

#include <aal5_rx_defs.h>

/* Run in simulation mode */
#ifndef	SIMULATION
#define	SIMULATION
#endif


/*	Chip is IXP2400 */
#ifndef IXP2400
#define IXP2400
#endif	

#define 		RX_PHY_MODE 		SPHY_4_8

/* The size of the message put onto scratch ring by AAL5 Rx for the next block */
#define			RX_TO_FWD_MSG_SIZE					6

/* Size of the RBUF elements */
#define 		RBUF_ELE_SIZE_LOG2	    			6
#define 		RBUF_ELE_SIZE	        			(1<<RBUF_ELE_SIZE_LOG2)
#define			TOTAL_RBUF_SIZE						8192


#define 		IPV4_COUNTERS_SCRATCH		

/*	Base address in SRAM for the packet buffers. This is for the meta data (buffer descriptors). */

#define			BUF_SRAM_BASE		0

/*	Amount of SRAM space available for (meta data of) packet buffers. 1 MB.  */
/*	In simulation we use only 10K bytes.		*/
#ifdef IXP_SIMULATION
#define_eval	BUF_SRAM_SIZE		(1024 * 10)
#else
#define_eval	BUF_SRAM_SIZE		(1024 * 10 * 40 * 2)
#endif

/*	Base Address in DRAM for the packet buffers. This is for the actual packet data. */

#define			BUF_SDRAM_BASE		0

/*	Amount of SDRAM space available for Packet buffers. 64 MB. */
/*	In simulation we use only 640K bytes.	*/
#ifdef IXP_SIMULATION
#define_eval	BUF_SDRAM_SIZE		(64 * 1024 * 10 )
#else
#define_eval	BUF_SDRAM_SIZE		(64 * 1024 * 10 * 40 * 2)
#endif

/*	Size of Meta Data (Buffer Descriptors) in SRAM. 32 entries per buffer. */
/* 	Should be a power of 2. */

#define			META_DATA_SIZE		32

/*	Size of packet buffer. */

#define			BUFFER_SIZE			2048




/*---------------------------------------------------------------------------*/
/* AAL5 RX defines										   		     		 */			
/*---------------------------------------------------------------------------*/

/* Port STAT Table (32 bits counters) - SRAM channel 0 */
#define 	COUNTER_SZ_BYTES				4

#ifdef IXP_SIMULATION
#define 	AAL5_RX_PORT_STAT_SRAM_BASE		0x280000 			 /* @ 2.5MB */
#else
#define 	AAL5_RX_PORT_STAT_SRAM_BASE		0x112000			 /* @ 2.5MB */
#endif

#define 	NO_OF_PORTS						16
#define 	NO_PER_PORT_STAT_COUNTERS		8
#define 	AAL5_RX_PORT_STAT_SHF_VAL		5        		



/* VC INFO TABLE,(RXC table & VC STAT table (32 bits counters) - SRAM channel 0 )  */
#ifdef IXP_SIMULATION
#define 	AAL5_RX_VC_INFO_SRAM_BASE	0x300000   				/* @ 3MB */
#define 	VC_INFO_TOTAL 				(64*1024)
#else
#define 	AAL5_RX_VC_INFO_SRAM_BASE	0x112200   				/* @ 3MB */
#define 	VC_INFO_TOTAL 				(32*1024)
#endif

#define 	AAL5_RX_VC_INFO_SZ_BYTES	(16*4)	
#define 	AAL5_RX_VC_INFO_SHF_VAL		6						



/* Primary Hash Table -	(SRAM channel 1)  */
#ifdef IXP_SIMULATION
#define 	AAL5_RX_VC_HASH_PRI_SRAM_BASE		0x40500000      /* @ 5MB  */
#else
#define 	AAL5_RX_VC_HASH_PRI_SRAM_BASE		0x40300000         		/* @ 5MB  */
#endif

#define 	VC_HASH_PRI_BKT_SZ_BYTES	(4*4) 
#define 	VC_HASH_PRI_BKT_TOTAL		(1024*64)

#define 	VC_HASH_PRI_END				(AAL5_RX_VC_HASH_PRI_SRAM_BASE+(VC_HASH_PRI_BKT_SZ_BYTES*\
									 	VC_HASH_PRI_BKT_TOTAL))

#define 	VC_HASH_PRI_SHF_VAL			4  						

#define 	AAL5_RX_VC_HASH_PRI_INDEX_MASK		0xffff

/* Secondary Hash Table   -  (SRAM channel 1)  */
#ifdef IXP_SIMULATION
#define 	AAL5_RX_VC_HASH_SEC_SRAM_BASE		VC_HASH_PRI_END  /* @ 6MB  */
#else
#define 	AAL5_RX_VC_HASH_SEC_SRAM_BASE		0x40204000   			 /* @ 6MB  */
#endif

#define 	VC_HASH_SEC_BKT_SZ_BYTES	(8*4) 
/*#define VC_HASH_SEC_BKT_TOTAL	(1024*8) - can be changed later to have 8K/16K bkts */
#define 	VC_HASH_SEC_BKT_TOTAL		(1024*1)	

#define 	VC_HASH_SEC_END				(AAL5_RX_VC_HASH_SEC_SRAM_BASE+(VC_HASH_SEC_BKT_SZ_BYTES*\
										VC_HASH_SEC_BKT_TOTAL))

#define 	VC_HASH_SEC_SHF_VAL			5						


/* To create a free list for secondary hash table  */
#define 	BUF_FREE_LIST1				(BUF_FREE_LIST0+1)


/*  While comparing the bkt_next value in the primary or the secondary table, we
	need to have bkt_index in the secondary table other than 0, to differentiate
	between bkt_index - not initilized and 0. So we will waste little memory worth 
	one bucket bytes (16 bytes) by not using bkt_index = 0. */

#define_eval VC_HASH_SEC_BASE_SRAM_LW	AAL5_RX_VC_HASH_SEC_SRAM_BASE + VC_HASH_SEC_BKT_SZ_BYTES

/* In the order of D_BASE, D_SIZE, S_BASE, S_SIZE  */
#define		VC_HASH_SEC_TABLE_POOL		0, 0, VC_HASH_SEC_BASE_SRAM_LW, VC_HASH_SEC_BKT_SZ_BYTES



/*--------------------------------------------------------------------------*/

/*	Base Address for Queue Descriptors. @ 1MB in SRAM (channel 0 [31:30]) */

#define			QD_SRAM_BASE		0x100000

/*	Total number of Queue Descriptors */

#define			QD_TOTAL			1024

/*	Size of each Queue Descriptor Entry (in Bytes) */

#define			QD_SIZE				16

/*	Base address CSIX Transmit Contexts in SRAM @ 2MB (channel 0 [31:30]) */
#ifdef IXP_SIMULATION
#define			CSIX_TX_CTX_SRAM_BASE	0x200000
#else
#define			CSIX_TX_CTX_SRAM_BASE	0x101000
#endif

/*	Size of each CSIX Transmit Context (in Bytes) */

#define			CSIX_TX_CTX_SIZE	64

/*	Total number of CSIX Transmit Contexts. */

#define			CSIX_TX_CTX_TOTAL	1024

/* Counter base for CSIX TX */
#ifdef IXP_SIMULATION
#define   		CSIX_TX_COUNTERS_SRAM_BASE   0x40300000
#else
#define   		CSIX_TX_COUNTERS_SRAM_BASE   0x40203000
#endif

/*  Area of SRAM which is zero'ed out */

#define         SRAM_ZERO_BLOCK 				0x0

/* free list id that is stored in meta data  in POS RX */
#define         FREE_LIST_ID                    0x0

/* 
 for better DRAM bank utilization, we put the nexthop info in
 SRAM at this address. We also have a segment in DRAM reserved
 to keep this info, should we decide to move back to DRAM. This
 would a compile time option for IPV4 forwarder. */

#ifdef IXP_SIMULATION
#define			NEXTHOP_TABLE_SRAM_BASE		0x1000000
#else
#define			NEXTHOP_TABLE_SRAM_BASE		0x111000
#endif

/*	Trie Table is in channel 1 [31:30] at address 0 in SRAM */

#define			TRIE_TABLE_SRAM_BASE			0x40000000

/*	Trie Table size. 4MB */

#define			TRIE_TABLE_SIZE					0x400000

/* base for IPv4 32bit counters */
#ifdef IPV4_COUNTERS_SCRATCH	

#define 		IPV4_STATS_TABLE_BASE			11264

#define 		IPV4_STATS_TABLE_SIZE			0x800

#else

#ifdef IXP_SIMULATION
#define         STATS_TABLE_SRAM_BASE			0x40400000
#else
#define         STATS_TABLE_SRAM_BASE			0x40200000
#endif

/*	Base for AAL5 RX counters */

//#define			AAL5_COUNTERS_SRAM_BASE			0x40400800

/* space allocated to maintain these counters */

#define         STATS_TABLE_SRAM_SIZE			0x800

#endif

/*  Directed broadcast table base in SRAM */
#ifdef IXP_SIMULATION
#define         DBCAST_TABLE_SRAM_BASE			0x40401000
#else
#define         DBCAST_TABLE_SRAM_BASE			0x40200800
#endif

/* directed broadcast table size */

#define         DBCAST_TABLE_SIZE				0x2000

/* IPV4 control block */
#ifdef IXP_SIMULATION
#define			CONTROL_BLOCK_SRAM_BASE			0x40403000
#else
#define			CONTROL_BLOCK_SRAM_BASE			0x40202800
#endif

/*	Next Hop Table @ 32MB in DRAM. 
	Will be moved to 64MB when the workbench supports it. */

/* #define			NEXTHOP_TABLE_SDRAM_BASE		0x2000000 */

/* with new DDR model, we ave only 32MB. Hence, moved the 
   nexthop base to 16MB. */
#define			NEXTHOP_TABLE_SDRAM_BASE		0x1000000

/*	Each Next Hop entry is 16 bytes. So 4K is sufficient */

#define			NEXTHOP_TABLE_SIZE				0x1000

/*	Scratch for communicating between AAL5 RX and IPv4 Blocks
	Ring number 0, base 0, and size 128 Long Words */

#define			AAL5_RX_TO_IPV4_SCR_RING		0
#define			AAL5_RX_TO_IPV4_SCR_RING_BASE	0
#define			AAL5_RX_TO_IPV4_SCR_RING_SIZE	1024

/*	Scratch for communicating between IPV4 and Queue Manager Blocks
	Ring number 1, base 512, and size 128 Long Words */

#define			IPV4_TO_QM_SCR_RING				1
#define			IPV4_TO_QM_SCR_RING_BASE		4096
#define			IPV4_TO_QM_SCR_RING_SIZE		512

/*	Scratch for communicating between Scheduler and Queue Manager Blocks
	Ring number 2, base 1024, and size 128 Long Words */

#define			SCHEDULER_TO_QM_SCR_RING		2
#define			SCHEDULER_TO_QM_SCR_RING_BASE	6144
#define			SCHEDULER_TO_QM_SCR_RING_SIZE	512

/*	Scratch for communicating between Queue Manager and CSIX Transmit Blocks
	Ring number 3, base 1536, and size 128 Long Words */

#define			QM_TO_CSIX_TX_SCR_RING			3
#define			QM_TO_CSIX_TX_SCR_RING_BASE		8192
#define			QM_TO_CSIX_TX_SCR_RING_SIZE		512


/*  These defines are used by the individual microcode to identify their scratch rings.
	The reason for these defines is to improve the portability of microblocks. User just
	has to make changes here and the microblocks will remain untouched.
 	For the naming convention, the microblock name goes first, then the _RING_ specifier. 
    Following this is the direction indicator. '_OUT' would imply the microblock is writing 
	to this scratch ring and '_IN' would imply the microblock is reading from this scratch 
	ring. If there is more than one 'IN' or 'OUT' ring the name will have '_<digit>' to 
    differentiate the rings.  */


/*	The Ring on which AAL5 RX sends out msg to next block */

#define 		AAL5_RX_RING_OUT				AAL5_RX_TO_IPV4_SCR_RING
#define			AAL5_RX_RING_OUT_BASE			AAL5_RX_TO_IPV4_SCR_RING_BASE
#define			AAL5_RX_RING_OUT_SIZE			AAL5_RX_TO_IPV4_SCR_RING_SIZE

/*	The input ring ring on which IPV4 receives msgs from prev block */

#define			IPV4_RING_IN					AAL5_RX_TO_IPV4_SCR_RING
#define			IPV4_RING_IN_BASE				AAL5_RX_TO_IPV4_SCR_RING_BASE
#define			IPV4_RING_IN_SIZE				AAL5_RX_TO_IPV4_SCR_RING_SIZE

/* The ring on which IPV4 sends out msg to next block (Queue Manager in this case) */

#define			IPV4_RING_OUT					IPV4_TO_QM_SCR_RING
#define			IPV4_RING_OUT_BASE				IPV4_TO_QM_SCR_RING_BASE
#define			IPV4_RING_OUT_SIZE				IPV4_TO_QM_SCR_RING_SIZE

/* The input ring on which QM receives msgs from prev. block */

#define			QM_RING_IN						IPV4_TO_QM_SCR_RING
#define			QM_RING_IN_BASE					IPV4_TO_QM_SCR_RING_BASE
#define			QM_RING_IN_SIZE					IPV4_TO_QM_SCR_RING_SIZE

/*	The output ring in which QM Sends msgs to next block (CSIX TX in this case) */

#define			QM_RING_OUT						QM_TO_CSIX_TX_SCR_RING
#define			QM_RING_OUT_BASE				QM_TO_CSIX_TX_SCR_RING_BASE
#define			QM_RING_OUT_SIZE				QM_TO_CSIX_TX_SCR_RING_SIZE

/*	The input ring in which CSIX TX receives msgs from QM */

#define			CSIX_TX_RING_IN					QM_TO_CSIX_TX_SCR_RING
#define			CSIX_TX_RING_IN_BASE			QM_TO_CSIX_TX_SCR_RING_BASE
#define			CSIX_TX_RING_IN_SIZE			QM_TO_CSIX_TX_SCR_RING_SIZE


/*	First 16 (0-15) SRAM Q-Array entries are allocated for the Queue Manager */

#define			QM_QARRAY_BASE					0
#define			QM_QARRAY_SIZE					16

/*	Next 4 (16-19) SRAM Q-Array entries are allocated for buffer free list */

#define			BUF_QARRAY_BASE					16
#define			BUF_QARRAY_SIZE					4

/*	Next 1 (20) SRAM Q-Array entry is used for Drop Queue in Queue Manager */

#define			QM_DROP_QUEUE_ENTRY				20

/*	Queue Number for Drop Queue. (0-1023 are the normal queues used in forwarding pkts) */

#define			QM_DROP_QUEUE					1024

/*	These tell if a the executing threads need to work in order. i.e wait for
	a signal from prev thread, do some work and signal next thread. */

#define			DL_THREAD_NO_ORDER				0x0
#define			DL_THREAD_ORDER					0x1


/*	All the IX_* constants should be between 0 and 255 (i.e 1 byte) to enable using
	them as immed operands in instructions. */

/*	IX_NULL is used to denote a NULL (or empty) packet handle, It should not be zero, 
	as it would interfere with ring empty (0) condition.
	Note: IX_NULL is too generic a name and may clash with core components code.
	So it's marked for deletion.  Start using UC_NULL  */

#define			IX_NULL							0xFF
#define			IX_EXCEPTION					0x1
#define			IX_DROP							0x2

#define			UC_NULL							0xFF		/* NULL for microcode 	*/

#define			BID_NULL						UC_NULL		/* NULL Block ID 		*/
#define			BID_EXCEPTION					0x1			/* Exception Block ID 	*/
#define			BID_DROP						0x2			/* Drop Block ID 		*/

#define			DL_BUF_NULL						UC_NULL 	/* NULL Buffer representation */

/*	Block IDs for various blocks. Block IDs Start from 0x20. (0 to 0x1f and 0xFF are 
	reserved for IX_* ) */

#define 		BID_LLC_SNAP_DECAP				0x21
#define			BID_IPV4						0x22
#define			BID_QM							0x23
#define 		BID_ETHER						BID_DROP
#define			BID_IPV6						BID_DROP
#define			BID_ARP							BID_DROP
#define			BID_PPP_DISCOVERY				BID_DROP
#define			BID_PPP_SESSION					BID_DROP
#define			BID_MPLS_UNICAST				BID_DROP

/* 	Static Bindings (a.k.a targets) for LLCSNAP microblock. 
 	Should this go into project/application specific global header file? */

#define 		TARGET_LLCSNAP_IPV4				BID_IPV4
#define 		TARGET_LLCSNAP_IPV6				BID_IPV6
#define 		TARGET_LLCSNAP_ARP				BID_ARP
#define 		TARGET_LLCSNAP_PPP_DISCOVERY	BID_PPP_DISCOVERY
#define 		TARGET_LLCSNAP_PPP_SESSION		BID_PPP_SESSION
#define 		TARGET_LLCSNAP_ETHER			BID_ETHER
#define			TARGET_LLCSNAP_MPLS_UNICAST		BID_MPLS_UNICAST

/* 	Static Bindings (a.k.a targets) for various microblocks. 
 	Should this go into project/application specific global header file? */

#ifdef			AAL5_WITH_L2_DECAP
#define			AAL5_RX_NEXT1					BID_LLC_SNAP_DECAP
#else 
#define			AAL5_RX_NEXT1					BID_IPV4
#endif


#ifdef			AAL5_WITH_L2_DECAP
#define			DL_SOURCE_NEXT1					BID_LLC_SNAP_DECAP
#else
#define			DL_SOURCE_NEXT1					BID_IPV4
#endif

#define			IPV4_NEXT1						BID_QM


#ifndef			IPV4_LOCAL_NEXT1
#define			IPV4_LOCAL_NEXT1				IX_EXCEPTION
#endif


/*	To be Redefined as an import variable. And its value should be one of the above
	Block IDs. */

#ifndef			BID_NEXT_BLOCK
#define			BID_NEXT_BLOCK					IX_NULL					
#endif

/*	BLADE ID */
#define			THIS_BLADE_ID					0x3F

/*	Signals being a precious resource, we use a single signal for 
	inter-thread as well as any other inter-me signalling (init over signal) */

#define			COMMON_INTER_THD_ME_SIGNAL		10

/*	We are not able to use a single signal before of limitations in assembler.
	until then, we need a separate signal for inter-thread signalling. */

#define			INTER_THD_SIGNAL				12

/*	ME_NIT_SIGNAL signal is used to let everyone know that initialisation is over
	and all blocks in all MEs can start running. This signal will be typically issued
	by the system initialisation code and all other MEs should wait for this signal
	before starting to run. */

#define			ME_INIT_SIGNAL					COMMON_INTER_THD_ME_SIGNAL



/*---------------------------------------------------*/
/*	Source and Sink related defines				 	 */
/*---------------------------------------------------*/

/*	specify if next neighbor ring is used. If not, default is scratch rings.
	NOTE: DL_RING_* is now obsolete. Will be changed to use AAL5_RX_RING_IN, 
	which is defined somwehere above. */

/* 	Valid values for RING_SIZE are 128, 256, 512 or 1024. */

#define 		DL_RING_SIZE			AAL5_RX_TO_IPV4_SCR_RING_SIZE

/* 	Base Address for Scratch Ring is 0x1000. 
	Should be aligned on a 4 x RING_SIZE byte boundary.*/

#define 		DL_RING_BASE			AAL5_RX_TO_IPV4_SCR_RING_BASE

/* 	Ring number to use. 0-15. */

#define_eval 	DL_RING_NUMBER			AAL5_RX_TO_IPV4_SCR_RING


/* 	Signal number to use in inter thread signalling. */

#define 		DL_SIG_WAKE				INTER_THD_SIGNAL


/* 	NEXT ME NUMBER to signal. Remember ME numbers to include the cluster information.
 	i.e MEs are numbered 0x0, 1, 2, 3,0x10, 0x11, 0x12, 0x13 for sausalito (8 MEs) and
 	it should be numbered 0x0, 1, 2,..7, 0x10, 0x11,....0x17 for Castine (16 MEs)
	This will become a imported variable later.  */
/*
#ifndef			DL_NEXT_ME
#define			DL_NEXT_ME			0
#warning 		"DL_NEXT_ME is not defined. Defaulting to ME 0."
#endif
*/

#define_eval	DL_DROP_RING			IPV4_TO_QM_SCR_RING


/*	The DDR (Double Data Rate DRAM) bank size is used to randomize buffer 
	offset. */

#define			DDR_BANK_SIZE		128

/*	Number of DDR banks */

#define			NO_OF_DDR_BANKS		4

#define			MAX_BUFFER_OFFSET	(DDR_BANK_SIZE * NO_OF_DDR_BANKS)

#endif		// __SYSTEM__H__