/* Target Server Initialization Code
 *
 * --------------------------------------------------------------------------
 *                                                                      
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  1998-2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *---------------------------------------------------------------------------
 *
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#define PRINT_STRING	printf
#include "hw_support_routines.c"


/**********************************************************************************************/
/**********************************************************************************************/
/******************************************		ATM Ingress Init   ****************************/
/**********************************************************************************************/
/**********************************************************************************************/

/******************************************CODE FROM IND FILE START****************************/

/******************************************CODE FROM aal5_rx_init.ind ****************************/

//----------------------------------------------------------------------------
// This file initializes required SRAM memory to zero.
//
// It initializes VC INFO (which has RXC and VC STAT counters) to zero.
// It initializes HASH Table and PORT STAT counters to zero.

// Ideally this would be done by Xscale core. But for simulation purposes
// we do it here.
//----------------------------------------------------------------------------



// Ths should match with the ones described in dl_system.h


//============================================================================
// SRAM memory usage( Channel 0):

// (1) PORT_STAT_COUNTERS 		= 16   * (8*4) 	 = 512 B   	@ 2.5 MB (0x280000) 

// (2) VC INFO TABLE 			= 64 K * (16*4)	 =     4 MB	@ 	3 MB (0x300000)
// 	 (i)  RXC TABLE  	 		= 64 K * (9*4)	 =  2304 KB 	
// 	 (ii) VC STAT COUNTERS  	= 64 K * (7*4)	 =  1792 KB 	
//============================================================================

//----------------------------------------------------------------------------
// Port Statistics Counters 												  			
//----------------------------------------------------------------------------

#define COUNTER_SZ_BYTES			4


// Base for PORT STATISTICS 32 bits counters  

// mbp
#define AAL5_RX_PORT_STAT_SRAM_BASE		0x112000

#define NO_OF_PORTS						16
#define NO_PER_PORT_STAT_COUNTERS		8
#define AAL5_RX_PORT_STAT_SHF_VAL		5        // in bytes 

#define AAL5_RX_PORT_STAT_TABLE_SIZE		(NO_OF_PORTS*(NO_PER_PORT_STAT_COUNTERS*COUNTER_SZ_BYTES))
#define AAL5_RX_PORT_STAT_END			(AAL5_RX_PORT_STAT_SRAM_BASE+AAL5_RX_PORT_STAT_TABLE_SIZE)



//-------------------------------------------------------------------------------
// VC INFO TABLE,(RXC table & VC STAT table)			 						 
//-------------------------------------------------------------------------------


#define AAL5_RX_VC_INFO_SHF_VAL		6		// in bytes 

// mbp
#define AAL5_RX_VC_INFO_SRAM_BASE	0x112200

// mbp
#define VC_INFO_TOTAL 				(32*1024)
#define AAL5_RX_VC_INFO_SZ_BYTES		(16*4)		// (VC_INFO 16 LWs each)

#define AAL5_RX_VC_INFO_TABLE_SIZE	(VC_INFO_TOTAL*AAL5_RX_VC_INFO_SZ_BYTES)

#define AAL5_RX_VC_INFO_END			(AAL5_RX_VC_INFO_SRAM_BASE+AAL5_RX_VC_INFO_TABLE_SIZE)


//============================================================================
//  SRAM memory usage( Channel 1):

// (1) HASH PRI TABLE  = 64K * (4*4)	     =  1 MB 	@ 5MB (0x40500000)
// (2) HASH SEC TABLE  =  1K * (8*4)		 = 32 KB 	@ 6MB (0x40600000) 

// Total number of secondary table buckets
//  Based on my calculation, 8k secondary table with 4 keys in every table, 
//  we are ok for 16k VC 
//============================================================================


//  Base address in SRAM for VC primary hash table  

#define VC_HASH_PRI_BASE_SRAM		0x40300000         // @ 5MB  
#define VC_HASH_PRI_BKT_SZ_BYTES	(4*4) 
#define VC_HASH_PRI_BKT_TOTAL		(1024*64)

#define VC_HASH_PRI_END				(VC_HASH_PRI_BASE_SRAM+(VC_HASH_PRI_BKT_SZ_BYTES*VC_HASH_PRI_BKT_TOTAL))

//--------------------------------------------------------------------------

//  Base address in SRAM for VC secondary hash table  

#define VC_HASH_SEC_BASE_SRAM		0x40204000   // @ 6MB  
#define VC_HASH_SEC_BKT_SZ_BYTES	(8*4) 

// To make init faster allocate just 1k secondary buckets 
// #define VC_HASH_SEC_BKT_TOTAL	(1024*8) 
#define VC_HASH_SEC_BKT_TOTAL		(1024*1)	


#define VC_HASH_SEC_END				(VC_HASH_SEC_BASE_SRAM+(VC_HASH_SEC_BKT_SZ_BYTES*VC_HASH_SEC_BKT_TOTAL))

//--------------------------------------------------------------------


int aal5_rx_port_stat_counters_init_zero()  {

	int addr_lo, addr_hi;

	addr_lo = AAL5_RX_PORT_STAT_SRAM_BASE;
	addr_hi = AAL5_RX_PORT_STAT_END;
	init_sram(0x0, addr_lo, addr_hi);

	return 1;

}


int aal5_rx_vc_info_init_zero()  {

	int addr_lo, addr_hi;

	addr_lo = AAL5_RX_VC_INFO_SRAM_BASE;
	addr_hi = AAL5_RX_VC_INFO_END;
	init_sram(0x0, addr_lo, addr_hi);

	return 1;

}


int aal5_rx_pri_init_zero()  {

	int addr_lo, addr_hi;
	
	addr_lo = VC_HASH_PRI_BASE_SRAM;
	addr_hi = VC_HASH_PRI_END;
	init_sram(0x0, addr_lo, addr_hi);

	return 1;

}

int aal5_rx_sec_init_zero()  {

	int addr_lo, addr_hi;
	
	addr_lo = VC_HASH_SEC_BASE_SRAM;
	addr_hi = VC_HASH_SEC_END;
	init_sram(0x0, addr_lo, addr_hi);

	return 1;

}

/******************************************CODE FROM aal5_rx_hash_table_init.ind ****************************/

//----------------------------------------------------------------------------
// This file sets up hash table (primary and secondary) and RXC (part of VC_info)
// in SRAM memory for 107 VCs.
//
//
//	1. Range of VCs to test normal cases:
//
//     VCI: VPI =  0 : 0
// 	   VCI: VPI =  1 : 1
//	   VCI: VPI =  2 : 2
//
//	   ...
//	   ...
//	   ...
//
//	   VCI: VPI = 100 : 100
//
//	
//	2. Range of VCs to test special cases:
//
//	   error#:
//
//	     vc_not_valid -     VCI: VPI =  101: 101
//	     nonAAL5 - 			VCI: VPI =  102 :102
//	     hash_miss -        VCI: VPI =  106+ : 106+
//
//	   non_error#:
//
//	     signalling cells -  	VCI: VPI = 103 : 103
//	     No LLC -			    VCI: VPI = 104 : 104 / VCI: VPI = 105 : 105
//	     signalling & no LLC - 	VCI: VPI = 106 : 106
//
// Note: VPI/VCI should be in hex.
//
// Ideally this would be done by Xscale core. But for simulation purposes
// we do it here.
//----------------------------------------------------------------------------


int aal5_rx_hash_rxc_init() {

	// done from MEs
return -1;

}

/******************************************CODE FROM dbcast_init.ind ****************************/

//
//	Some constants
//

#define	SRAM_READ_BLOCK_COUNT	8	// in number of long words
#define	SRAM_READ_SHIFT			3	// block count is pow of 2

// mbp
#define	DBCAST_SRAM_BASE		0x40200800			

#define	DBCAST_NUM_BLOCKS		256
#define	DBCAST_BLOCK_SIZE		SRAM_READ_BLOCK_COUNT


int dbcast_init(){

	int size, low, high;

	size = DBCAST_NUM_BLOCKS * SRAM_READ_BLOCK_COUNT * 4;

	low = DBCAST_SRAM_BASE;
	high = low+size;

	init_sram(0x0,low,high);

	return 1;
}

//
//	This function adds an address to dbcast 
// 	table. 
//	NOTE : Add utmost SRAM_READ_BLOCK_COUNT-1 entries
//  per block. Each block looks as follows.
//
//	 ---------------  BASE
//	|				|
//	|---------------
//	|	x			|
//	 ---------------
//	|	 			|
//	|---------------
//	|<Next addr>	|-----> contains the full address of the next block
//	 ---------------
//	|				|
//	|---------------
//	|				|
//	 ---------------
//
int	dbcast_add(int ipAddr){

	int hash_result, base, index, value;

	//  Given an IP address of form A.B.C.D produce
	//   a hash_result of form A.A^B.A^B^C.A^B^C^D
	

	hash_result = (ipAddr) ^ (ipAddr >> 8) ^ (ipAddr >> 16) ^ (ipAddr>>24);

	// get the key which is 8 bits
	hash_result = hash_result & 0xff;

	// calculate the base
	base = DBCAST_SRAM_BASE + (hash_result << SRAM_READ_SHIFT) * 4;

	for (index=0;index<SRAM_READ_BLOCK_COUNT-1;index++)
	{
		value = get_sram(base+(index*4));
		if (value)
			continue;
		else
		{
			set_sram(base+(index*4),ipAddr);
			PRINT_STRING("Added 0x%x to DBCAST table at 0x%x\n",
				ipAddr,base+(index*4));
			break;
		}

	}

	if (index == (SRAM_READ_BLOCK_COUNT-1))
		PRINT_STRING("DBCAST table is full. Can't add 0x%x\n",ipAddr);

	return 1;
}

/******************************************CODE FROM qm_init.ind ****************************/

#define			QD_SRAM_BASE		0x100000
#define 		NUM_OF_Q			1025

//---------------------------------
//	Macros to Read/Write memory
//---------------------------------

int qmInit() {
	
	int finaladdr;
	int baseaddr;

	PRINT_STRING("qmInit()	: Initializing QM SRAM memory :=..\n");

	baseaddr = QD_SRAM_BASE;
	finaladdr = baseaddr + NUM_OF_Q*16 ;

	PRINT_STRING("Clearing SRAM ...");
	init_sram(0x0,QD_SRAM_BASE,finaladdr);

	PRINT_STRING("QM memory initialization COMPLETE.\n\n");

	return 1;
}

/******************************************CODE FROM rtm_init.ind ****************************/

//-------------------------------------------------------------
//	The following constants are taken from RTM code
//-------------------------------------------------------------

#define	_SRAM_LOOKUP_BASE	0x40000000

#define	_DRAM_NEXTHOP_BASE	0x1000000
// mbp
#define	_SRAM_NEXTHOP_BASE	0x111000

#define	SIZE_OF_TRIE_SET	16
#define	NUM_TRIE_SETS		1024
#define	NUM_NEXTHOPS		512
#define	SIZE_OF_NEXTHOP		4	// long words

//---------------------------------
//	Macros to Read/Write memory
//---------------------------------

int rtmInit() {
	
	int cnt, addr;
	int baseaddr;

	PRINT_STRING("rtmInit()	: Initializing RTM memory :=..\n");

	baseaddr = _SRAM_LOOKUP_BASE;
	cnt = 64 * 1024 + 256 + SIZE_OF_TRIE_SET * NUM_TRIE_SETS;
	addr = baseaddr + (cnt * 4);

	PRINT_STRING("Clearing SRAM ...");
	init_sram(0x0,baseaddr,addr);
	

//	PRINT_STRING("Clearing DRAM ...");
//	baseaddr = _DRAM_NEXTHOP_BASE;
//	cnt = NUM_NEXTHOPS * SIZE_OF_NEXTHOP;
//	for (index=0; index < cnt ; index++) {
//		addr = baseaddr + (index<<2);
//		set_dram(addr,0x0);
//	}

	baseaddr = _SRAM_NEXTHOP_BASE;
	cnt = NUM_NEXTHOPS * SIZE_OF_NEXTHOP * 4;
	addr = baseaddr + cnt;

	PRINT_STRING("Clearing SRAM nexthop info...");
	init_sram(0x0,baseaddr,addr);

	PRINT_STRING("RTM memory initialization COMPLETE.\n\n");

	return 1;
}


//------------------------------------------------------
//	Initialize the counters to zeros
//------------------------------------------------------
// mbp
#define	_SRAM_COUNTER_BASE	0x40200000
// mbp
#define	_SRAM_COUNTER_SIZE	0x800

int ipv4ClearCounters(){
	int addr_lo,addr_hi;

	addr_lo = _SRAM_COUNTER_BASE;
	addr_hi = _SRAM_COUNTER_BASE + _SRAM_COUNTER_SIZE;

	init_sram(0x0,addr_lo,addr_hi);

	return 1;
}

/******************************************CODE FROM rtm_routes.ind ****************************/


int l3CreateRoutes() {


set_sram(0x111000,0xffffffff);

// 
// #=======================================================================
// # Assumption :-							 
// #=======================================================================
// 
// #	This AI's Ingress Blade_id = 1
// #	This AI's Egress Blade_id = 0
// #	Angel Island has one port (port = 0x0)
// #
// #	Hence for Redirect outport = {bladeid, outport} = {0x01, 0x000000}
// #	For other cases, bladeid = {0x00, 0x02-0x3f}, outport = {0x000000}
// 
// 
// #=======================================================================
// #  rtm.exe requires following parameters to generate the rtm routes
// #=======================================================================
// 
// #	This file contains the routes to be added for running the unit tests
// #	
// #	The syntax is understood by RTM executable which is used to generate
// #	the rtm_routes.ind file.
// #
// #	The syntax for the add command is as follows.
// #
// #	add ip-addr netmask nexthopid flags mtu outport
// #
// #	How to run the rtm.exe
// #	----------------------
// #	At command prompt, run 'rtm' on the routes.txt file
// #
// #	prompt:> rtm routes.txt
// #
// #	Then, give a file name for the script to be generated, say 
// #	rtm_routes.ind. Then replace this file in the Ingress project.
// #
// #	ip-addr 	: IP address in A.B.C.D format
// #	netmask		: Netmask in A.B.C.D format
// #	nexthopid	: Nexthop ID. Only least significant 16 bits taken
// #	flags		: Normally, ZERO. See IPV4 LLD for more options
// #	mtu		: Max tramission unit
// #	outport		: Encoded value as follows
// #				bladeid  : bits (31:24)
// #				outport  : bits (23:0)
// #
// 
// 
// #=======================================================================
// # Flags specify the quick-action to be carried on the packet.		
// # Flags have higher priority to anything				
// #=======================================================================
// 
// #
// #  The bit positions are relative to the starting bit position = 0.
// #
// #    Bit pos  	Flag val  Desc		Action
// #   ---------------------------------------------------------------------------------------
// #	0	0x1	Local		Exception
// #	1	0x2	I/f Down		Exception
// #	2	0x4	Drop		Drop
// #	3	0x8	Broadcast	Exception
// #	4	0x10	Multicast		Exception
// #		0x0	Forward		Forward
// #
// 
// 
// #=======================================================================
// #       TEST  CASES 							 
// #=======================================================================
// 
// # NOTE :- You cannot have Out going blade id as 1 (Ingress blade id = 1) 
// 
// 
// 
// 
// # db cast init
// # 
// # SA {<prefix>, -1} 16.1.255.255 (0x1001ffff)
// # SA {<prefex>, 0}  16.1.0.0     (0x10010000)
// # SA {0, <host>}    0.0.1.3	 (0x00000103)
// 
// 
// #-----------------------------------------------
// # Limited broadcast - exception 		
// #-----------------------------------------------
// 
// add 255.255.255.255 255.255.255.255 0xd255 0x8 9192 0x02000000

// @@@ Adding dest=0xffffffff, mask = 0xffffffff, blade_nh_id=0x200d255,mtu_port=0x23e80000,flags=0x8

	set_sram(0x4004053c,0x10000);

// writing nexthop entry with flags 0x8

set_sram(0x111010,0x8);

// blade 0x2 and NH 0xd255

set_sram(0x111014,0x200d255);

// MTU is 9192, outport is 0x0

set_sram(0x111018,0x23e80000);


set_sram(0x11101c,0x0);


	set_sram(0x400404fc,0x4);


	set_sram(0x400404bc,0x3);


	set_sram(0x4004047c,0x2);


	set_sram(0x4003fffc,0x1);

// 
// 
// #-----------------------------------------------
// #  Invalid destination addresses 	 
// #----------------------------------------------
// 
// # dst==0 - drop
// #add 0.0.0.0 255.255.255.255 0xd000 0x4 9212 0x02000000
// 
// # dst==127.*.*.* - drop
// add 127.0.0.0 255.0.0.0 0xd127 0x4 9212 0x02000000

// @@@ Adding dest=0x7f000000, mask = 0xff000000, blade_nh_id=0x300d127,mtu_port=0x23fc0000,flags=0x4

	set_sram(0x400401fc,0x20000);

// writing nexthop entry with flags 0x4

set_sram(0x111020,0x4);

// blade 0x2 and NH 0xd127

set_sram(0x111024,0x200d127);

// MTU is 9212, outport is 0x0

set_sram(0x111028,0x23fc0000);


set_sram(0x11102c,0x0);

// 
// # dst==classE - drop
// add 240.0.0.0 240.0.0.0 0xd240 0x4 9212 0x02000000

// @@@ Adding dest=0xf0000000, mask = 0xf0000000, blade_nh_id=0x400d240,mtu_port=0x23fc0000,flags=0x4

	set_sram(0x400403c0,0x30000);


	set_sram(0x400403c4,0x30000);


	set_sram(0x400403c8,0x30000);


	set_sram(0x400403cc,0x30000);


	set_sram(0x400403d0,0x30000);


	set_sram(0x400403d4,0x30000);


	set_sram(0x400403d8,0x30000);


	set_sram(0x400403dc,0x30000);


	set_sram(0x400403e0,0x30000);


	set_sram(0x400403e4,0x30000);


	set_sram(0x400403e8,0x30000);


	set_sram(0x400403ec,0x30000);


	set_sram(0x400403f0,0x30000);


	set_sram(0x400403f4,0x30000);


	set_sram(0x400403f8,0x30000);


	set_sram(0x400403fc,0x30000);

// writing nexthop entry with flags 0x4

set_sram(0x111030,0x4);

// blade 0x2 and NH 0xd240

set_sram(0x111034,0x200d240);

// MTU is 9212, outport is 0x0

set_sram(0x111038,0x23fc0000);


set_sram(0x11103c,0x0);

// 
// # dest==multicast - exception
// add 224.0.0.0 224.0.0.0 0xd224 0x10 9192 0x02000000

// @@@ Adding dest=0xe0000000, mask = 0xe0000000, _nh_id=0x500d224,mtu_port=0x23e80000,flags=0x10

	set_sram(0x40040380,0x40000);


	set_sram(0x40040384,0x40000);


	set_sram(0x40040388,0x40000);


	set_sram(0x4004038c,0x40000);


	set_sram(0x40040390,0x40000);


	set_sram(0x40040394,0x40000);


	set_sram(0x40040398,0x40000);


	set_sram(0x4004039c,0x40000);


	set_sram(0x400403a0,0x40000);


	set_sram(0x400403a4,0x40000);


	set_sram(0x400403a8,0x40000);


	set_sram(0x400403ac,0x40000);


	set_sram(0x400403b0,0x40000);


	set_sram(0x400403b4,0x40000);


	set_sram(0x400403b8,0x40000);


	set_sram(0x400403bc,0x40000);

// writing nexthop entry with flags 0x10

set_sram(0x111040,0x10);

// blade 0x2 and NH 0xd224

set_sram(0x111044,0x200d224);

// MTU is 9192, outport is 0x0

set_sram(0x111048,0x23e80000);


set_sram(0x11104c,0x0);

// 
// 
// 
// #-----------------------------------------------
// # Local delivery - exception 			
// #-----------------------------------------------
// 
// add 100.0.0.1 255.255.255.255 0x1001 0x1 9212 0x02000000 

// @@@ Adding dest=0x64000001, mask = 0xffffffff, blade_nh_id=0x2001001,mtu_port=0x23fc0000,flags=0x1

	set_sram(0x40040604,0x50000);

// writing nexthop entry with flags 0x1

set_sram(0x111050,0x1);

// blade 0x2 and NH 0x1001

set_sram(0x111054,0x2001001);

// MTU is 9212, outport is 0x0

set_sram(0x111058,0x23fc0000);


set_sram(0x11105c,0x0);


	set_sram(0x400405c0,0x8);


	set_sram(0x40040580,0x7);


	set_sram(0x40040540,0x6);


	set_sram(0x40019000,0x5);

// 
// #-----------------------------------------------
// # redirect - exception  			
// #-----------------------------------------------
// 
// add 100.0.0.2 255.255.255.255 0x1002 0x0 9192 0x01000000

// @@@ Adding dest=0x64000002, mask = 0xffffffff, blade_nh_id=0x1001002,mtu_port=0x23e80000,flags=0x0

	set_sram(0x40040608,0x60000);

// writing nexthop entry with flags 0x0

set_sram(0x111060,0x0);

// blade 0x1 and NH 0x1002

set_sram(0x111064,0x1001002);

// MTU is 9192, outport is 0x0

set_sram(0x111068,0x23e80000);


set_sram(0x11106c,0x0);

// 
// #-----------------------------------------------
// # fragment - exception  			
// #-----------------------------------------------
// 
// add 100.0.0.3 255.255.255.255 0x1003 0x0 5  0x03000000

// @@@ Adding dest=0x64000003, mask = 0xffffffff, blade_nh_id=0x3001003,mtu_port=0x50000,flags=0x0

	set_sram(0x4004060c,0x70000);

// writing nexthop entry with flags 0x0

set_sram(0x111070,0x0);

// blade 0x3 and NH 0x1003

set_sram(0x111074,0x3001003);

// MTU is 5, outport is 0x0

set_sram(0x111078,0x50000);


set_sram(0x11107c,0x0);

// 
// #-----------------------------------------------
// # Interface/host down - exception  			
// #-----------------------------------------------
// 
// add 100.0.0.4 255.255.255.255 0x1004 0x2 9192 0x04000000

// @@@ Adding dest=0x64000004, mask = 0xffffffff, blade_nh_id=0x4001004,mtu_port=0x23e80000,flags=0x2

	set_sram(0x40040610,0x80000);

// writing nexthop entry with flags 0x2

set_sram(0x111080,0x2);

// blade 0x4 and NH 0x1004

set_sram(0x111084,0x4001004);

// MTU is 9192, outport is 0x0

set_sram(0x111088,0x23e80000);


set_sram(0x11108c,0x0);

// 
// 
// 
// #-----------------------------------------------
// # Directed broadcast - Exception		 
// #-----------------------------------------------
// 
// # dest={<prefix>,-1} 
// add 192.1.2.255 255.255.255.0 0xcab1 0x8 9212 0x02000000

// @@@ Adding dest=0xc00102ff, mask = 0xffffff00, blade_nh_id=0x200cab1,mtu_port=0x23fc0000,flags=0x8

	set_sram(0x40040688,0x90000);

// writing nexthop entry with flags 0x8

set_sram(0x111090,0x8);

// blade 0x2 and NH 0xcab1

set_sram(0x111094,0x200cab1);

// MTU is 9212, outport is 0x0

set_sram(0x111098,0x23fc0000);


set_sram(0x11109c,0x0);


	set_sram(0x40040640,0xa);


	set_sram(0x40030004,0x9);

// 
// # dest={<prefix>,0} 
// add 192.1.2.0 255.255.255.0 0xcab2 0x8 9212 0x02000000

// @@@ Adding dest=0xc0010200, mask = 0xffffff00, blade_nh_id=0x300cab2,mtu_port=0x23fc0000,flags=0x8
// writing nexthop entry with flags 0x8

set_sram(0x111090,0x8);

// blade 0x2 and NH 0xcab2

set_sram(0x111094,0x200cab2);

// MTU is 9212, outport is 0x0

set_sram(0x111098,0x23fc0000);


set_sram(0x11109c,0x0);

// 
// # dest={0,<host>} 
// #add 0.0.0.3 255.255.255.0 0xcab3 0x8 9212 0x04000000
// 
// 
// 
// #-----------------------------------------------
// # 32-bit routes - Forward			 
// #-----------------------------------------------
// 
//  add 32.0.0.1 255.255.255.255 0x0001 0x0 9212 0x02000000

// @@@ Adding dest=0x20000001, mask = 0xffffffff, blade_nh_id=0x01,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040784,0xa0000);

// writing nexthop entry with flags 0x0

set_sram(0x1110a0,0x0);

// blade 0x2 and NH 0x01

set_sram(0x1110a4,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x1110a8,0x23fc0000);


set_sram(0x1110ac,0x0);


	set_sram(0x40040740,0xe);


	set_sram(0x40040700,0xd);


	set_sram(0x400406c0,0xc);


	set_sram(0x40008000,0xb);

//  add 32.0.0.2 255.255.255.255 0x0020 0x0 9212 0x02000001

// @@@ Adding dest=0x20000002, mask = 0xffffffff, blade_nh_id=0x2000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040788,0xb0000);

// writing nexthop entry with flags 0x0

set_sram(0x1110b0,0x0);

// blade 0x2 and NH 0x20

set_sram(0x1110b4,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x1110b8,0x23fc0001);


set_sram(0x1110bc,0x0);

//  add 32.0.0.3 255.255.255.255 0x0040 0x0 9212 0x02000002

// @@@ Adding dest=0x20000003, mask = 0xffffffff, blade_nh_id=0x3000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x4004078c,0xc0000);

// writing nexthop entry with flags 0x0

set_sram(0x1110c0,0x0);

// blade 0x2 and NH 0x40

set_sram(0x1110c4,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x1110c8,0x23fc0002);


set_sram(0x1110cc,0x0);

//  add 32.0.0.4 255.255.255.255 0x0060 0x0 9212 0x02000003

// @@@ Adding dest=0x20000004, mask = 0xffffffff, blade_nh_id=0x4000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x40040790,0xd0000);

// writing nexthop entry with flags 0x0

set_sram(0x1110d0,0x0);

// blade 0x2 and NH 0x60

set_sram(0x1110d4,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x1110d8,0x23fc0003);


set_sram(0x1110dc,0x0);

//  add 32.0.0.5 255.255.255.255 0x0001 0x0 9212 0x02000000

// @@@ Adding dest=0x20000005, mask = 0xffffffff, blade_nh_id=0x5000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040794,0xe0000);

// writing nexthop entry with flags 0x0

set_sram(0x1110e0,0x0);

// blade 0x2 and NH 0x01

set_sram(0x1110e4,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x1110e8,0x23fc0000);


set_sram(0x1110ec,0x0);

//  add 32.0.0.6 255.255.255.255 0x0020 0x0 9212 0x02000001

// @@@ Adding dest=0x20000006, mask = 0xffffffff, blade_nh_id=0x6000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040798,0xf0000);

// writing nexthop entry with flags 0x0

set_sram(0x1110f0,0x0);

// blade 0x2 and NH 0x20

set_sram(0x1110f4,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x1110f8,0x23fc0001);


set_sram(0x1110fc,0x0);

//  add 32.0.0.7 255.255.255.255 0x0040 0x0 9212 0x02000002

// @@@ Adding dest=0x20000007, mask = 0xffffffff, blade_nh_id=0x7000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x4004079c,0x100000);

// writing nexthop entry with flags 0x0

set_sram(0x111100,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111104,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111108,0x23fc0002);


set_sram(0x11110c,0x0);

//  add 32.0.0.8 255.255.255.255 0x0060 0x0 9212 0x02000003

// @@@ Adding dest=0x20000008, mask = 0xffffffff, blade_nh_id=0x8000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x400407a0,0x110000);

// writing nexthop entry with flags 0x0

set_sram(0x111110,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111114,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111118,0x23fc0003);


set_sram(0x11111c,0x0);

//  add 32.0.0.9 255.255.255.255 0x0001 0x0 9212 0x02000000

// @@@ Adding dest=0x20000009, mask = 0xffffffff, blade_nh_id=0x9000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x400407a4,0x120000);

// writing nexthop entry with flags 0x0

set_sram(0x111120,0x0);

// blade 0x2 and NH 0x01

set_sram(0x111124,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x111128,0x23fc0000);


set_sram(0x11112c,0x0);

//  add 32.0.0.10 255.255.255.255 0x0020 0x0 9212 0x02000001

// @@@ Adding dest=0x2000000a, mask = 0xffffffff, blade_nh_id=0xa000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x400407a8,0x130000);

// writing nexthop entry with flags 0x0

set_sram(0x111130,0x0);

// blade 0x2 and NH 0x20

set_sram(0x111134,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x111138,0x23fc0001);


set_sram(0x11113c,0x0);

//  add 32.0.0.11 255.255.255.255 0x0040 0x0 9212 0x02000002

// @@@ Adding dest=0x2000000b, mask = 0xffffffff, blade_nh_id=0xb000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x400407ac,0x140000);

// writing nexthop entry with flags 0x0

set_sram(0x111140,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111144,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111148,0x23fc0002);


set_sram(0x11114c,0x0);

//  add 32.0.0.12 255.255.255.255 0x0060 0x0 9212 0x02000003

// @@@ Adding dest=0x2000000c, mask = 0xffffffff, blade_nh_id=0xc000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x400407b0,0x150000);

// writing nexthop entry with flags 0x0

set_sram(0x111150,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111154,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111158,0x23fc0003);


set_sram(0x11115c,0x0);

//  add 32.0.0.13 255.255.255.255 0x0001 0x0 9212 0x02000000

// @@@ Adding dest=0x2000000d, mask = 0xffffffff, blade_nh_id=0xd000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x400407b4,0x160000);

// writing nexthop entry with flags 0x0

set_sram(0x111160,0x0);

// blade 0x2 and NH 0x01

set_sram(0x111164,0xd000001);

// MTU is 9212, outport is 0x0

set_sram(0x111168,0x23fc0000);


set_sram(0x11116c,0x0);

//  add 32.0.0.14 255.255.255.255 0x0020 0x0 9212 0x02000001

// @@@ Adding dest=0x2000000e, mask = 0xffffffff, blade_nh_id=0xe000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x400407b8,0x170000);

// writing nexthop entry with flags 0x0

set_sram(0x111170,0x0);

// blade 0x2 and NH 0x20

set_sram(0x111174,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x111178,0x23fc0001);


set_sram(0x11117c,0x0);

//  add 32.0.0.15 255.255.255.255 0x0040 0x0 9212 0x02000002

// @@@ Adding dest=0x2000000f, mask = 0xffffffff, blade_nh_id=0xf000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x400407bc,0x180000);

// writing nexthop entry with flags 0x0

set_sram(0x111180,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111184,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111188,0x23fc0002);


set_sram(0x11118c,0x0);

//  add 32.0.0.16 255.255.255.255 0x0060 0x0 9212 0x02000003

// @@@ Adding dest=0x20000010, mask = 0xffffffff, blade_nh_id=0x10000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x400407c0,0x190000);

// writing nexthop entry with flags 0x0

set_sram(0x111190,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111194,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111198,0x23fc0003);


set_sram(0x11119c,0x0);


	set_sram(0x40040744,0xf);

//  add 32.0.0.17 255.255.255.255 0x0001 0x0 9212 0x02000000

// @@@ Adding dest=0x20000011, mask = 0xffffffff, blade_nh_id=0x11000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x400407c4,0x1a0000);

// writing nexthop entry with flags 0x0

set_sram(0x1111a0,0x0);

// blade 0x2 and NH 0x01

set_sram(0x1111a4,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x1111a8,0x23fc0000);


set_sram(0x1111ac,0x0);

//  add 32.0.0.18 255.255.255.255 0x0020 0x0 9212 0x02000001

// @@@ Adding dest=0x20000012, mask = 0xffffffff, blade_nh_id=0x12000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x400407c8,0x1b0000);

// writing nexthop entry with flags 0x0

set_sram(0x1111b0,0x0);

// blade 0x2 and NH 0x20

set_sram(0x1111b4,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x1111b8,0x23fc0001);


set_sram(0x1111bc,0x0);

//  add 32.0.0.19 255.255.255.255 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x20000013, mask = 0xffffffff, blade_nh_id=0x13000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x400407cc,0x1c0000);

// writing nexthop entry with flags 0x0

set_sram(0x1111c0,0x0);

// blade 0x2 and NH 0x40

set_sram(0x1111c4,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x1111c8,0x23fc0002);


set_sram(0x1111cc,0x0);

//  add 32.0.0.20 255.255.255.255 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x20000014, mask = 0xffffffff, blade_nh_id=0x14000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x400407d0,0x1d0000);

// writing nexthop entry with flags 0x0

set_sram(0x1111d0,0x0);

// blade 0x2 and NH 0x60

set_sram(0x1111d4,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x1111d8,0x23fc0003);


set_sram(0x1111dc,0x0);

//  add 32.0.0.21 255.255.255.255 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x20000015, mask = 0xffffffff, blade_nh_id=0x15000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x400407d4,0x1e0000);

// writing nexthop entry with flags 0x0

set_sram(0x1111e0,0x0);

// blade 0x2 and NH 0x01

set_sram(0x1111e4,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x1111e8,0x23fc0000);


set_sram(0x1111ec,0x0);

//  add 32.0.0.22 255.255.255.255 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x20000016, mask = 0xffffffff, blade_nh_id=0x16000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x400407d8,0x1f0000);

// writing nexthop entry with flags 0x0

set_sram(0x1111f0,0x0);

// blade 0x2 and NH 0x20

set_sram(0x1111f4,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x1111f8,0x23fc0001);


set_sram(0x1111fc,0x0);

//  add 32.0.0.23 255.255.255.255 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x20000017, mask = 0xffffffff, blade_nh_id=0x17000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x400407dc,0x200000);

// writing nexthop entry with flags 0x0

set_sram(0x111200,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111204,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111208,0x23fc0002);


set_sram(0x11120c,0x0);

//  add 32.0.0.24 255.255.255.255 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x20000018, mask = 0xffffffff, blade_nh_id=0x18000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x400407e0,0x210000);

// writing nexthop entry with flags 0x0

set_sram(0x111210,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111214,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111218,0x23fc0003);


set_sram(0x11121c,0x0);

//  add 32.0.0.25 255.255.255.255 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x20000019, mask = 0xffffffff, blade_nh_id=0x19000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x400407e4,0x220000);

// writing nexthop entry with flags 0x0

set_sram(0x111220,0x0);

// blade 0x2 and NH 0x01

set_sram(0x111224,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x111228,0x23fc0000);


set_sram(0x11122c,0x0);

//  add 32.0.0.26 255.255.255.255 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x2000001a, mask = 0xffffffff, blade_nh_id=0x1a000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x400407e8,0x230000);

// writing nexthop entry with flags 0x0

set_sram(0x111230,0x0);

// blade 0x2 and NH 0x20

set_sram(0x111234,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x111238,0x23fc0001);


set_sram(0x11123c,0x0);

//  add 32.0.0.27 255.255.255.255 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x2000001b, mask = 0xffffffff, blade_nh_id=0x1b000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x400407ec,0x240000);

// writing nexthop entry with flags 0x0

set_sram(0x111240,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111244,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111248,0x23fc0002);


set_sram(0x11124c,0x0);

//  add 32.0.0.28 255.255.255.255 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x2000001c, mask = 0xffffffff, blade_nh_id=0x1c000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x400407f0,0x250000);

// writing nexthop entry with flags 0x0

set_sram(0x111250,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111254,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111258,0x23fc0003);


set_sram(0x11125c,0x0);

//  add 32.0.0.29 255.255.255.255 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x2000001d, mask = 0xffffffff, blade_nh_id=0x1d000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x400407f4,0x260000);

// writing nexthop entry with flags 0x0

set_sram(0x111260,0x0);

// blade 0x2 and NH 0x01

set_sram(0x111264,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x111268,0x23fc0000);


set_sram(0x11126c,0x0);

//  add 32.0.0.30 255.255.255.255 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x2000001e, mask = 0xffffffff, blade_nh_id=0x1e000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x400407f8,0x270000);

// writing nexthop entry with flags 0x0

set_sram(0x111270,0x0);

// blade 0x2 and NH 0x20

set_sram(0x111274,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x111278,0x23fc0001);


set_sram(0x11127c,0x0);

//  add 32.0.0.31 255.255.255.255 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x2000001f, mask = 0xffffffff, blade_nh_id=0x1f000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x400407fc,0x280000);

// writing nexthop entry with flags 0x0

set_sram(0x111280,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111284,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111288,0x23fc0002);


set_sram(0x11128c,0x0);

//  add 32.0.0.32 255.255.255.255 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x20000020, mask = 0xffffffff, blade_nh_id=0x20000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x40040800,0x290000);

// writing nexthop entry with flags 0x0

set_sram(0x111290,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111294,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111298,0x23fc0003);


set_sram(0x11129c,0x0);


	set_sram(0x40040748,0x10);

//  add 32.0.0.33 255.255.255.255 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x20000021, mask = 0xffffffff, blade_nh_id=0x21000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040804,0x2a0000);

// writing nexthop entry with flags 0x0

set_sram(0x1112a0,0x0);

// blade 0x2 and NH 0x01

set_sram(0x1112a4,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x1112a8,0x23fc0000);


set_sram(0x1112ac,0x0);

//  add 32.0.0.34 255.255.255.255 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x20000022, mask = 0xffffffff, blade_nh_id=0x22000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040808,0x2b0000);

// writing nexthop entry with flags 0x0

set_sram(0x1112b0,0x0);

// blade 0x2 and NH 0x20

set_sram(0x1112b4,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x1112b8,0x23fc0001);


set_sram(0x1112bc,0x0);

//  add 32.0.0.35 255.255.255.255 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x20000023, mask = 0xffffffff, blade_nh_id=0x23000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x4004080c,0x2c0000);

// writing nexthop entry with flags 0x0

set_sram(0x1112c0,0x0);

// blade 0x2 and NH 0x40

set_sram(0x1112c4,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x1112c8,0x23fc0002);


set_sram(0x1112cc,0x0);

//  add 32.0.0.36 255.255.255.255 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x20000024, mask = 0xffffffff, blade_nh_id=0x24000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x40040810,0x2d0000);

// writing nexthop entry with flags 0x0

set_sram(0x1112d0,0x0);

// blade 0x2 and NH 0x60

set_sram(0x1112d4,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x1112d8,0x23fc0003);


set_sram(0x1112dc,0x0);

//  add 32.0.0.37 255.255.255.255 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x20000025, mask = 0xffffffff, blade_nh_id=0x25000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040814,0x2e0000);

// writing nexthop entry with flags 0x0

set_sram(0x1112e0,0x0);

// blade 0x2 and NH 0x01

set_sram(0x1112e4,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x1112e8,0x23fc0000);


set_sram(0x1112ec,0x0);

//  add 32.0.0.38 255.255.255.255 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x20000026, mask = 0xffffffff, blade_nh_id=0x26000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040818,0x2f0000);

// writing nexthop entry with flags 0x0

set_sram(0x1112f0,0x0);

// blade 0x2 and NH 0x20

set_sram(0x1112f4,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x1112f8,0x23fc0001);


set_sram(0x1112fc,0x0);

//  add 32.0.0.39 255.255.255.255 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x20000027, mask = 0xffffffff, blade_nh_id=0x27000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x4004081c,0x300000);

// writing nexthop entry with flags 0x0

set_sram(0x111300,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111304,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111308,0x23fc0002);


set_sram(0x11130c,0x0);

//  add 32.0.0.40 255.255.255.255 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x20000028, mask = 0xffffffff, blade_nh_id=0x28000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x40040820,0x310000);

// writing nexthop entry with flags 0x0

set_sram(0x111310,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111314,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111318,0x23fc0003);


set_sram(0x11131c,0x0);

//  add 32.0.0.41 255.255.255.255 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x20000029, mask = 0xffffffff, blade_nh_id=0x29000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040824,0x320000);

// writing nexthop entry with flags 0x0

set_sram(0x111320,0x0);

// blade 0x2 and NH 0x01

set_sram(0x111324,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x111328,0x23fc0000);


set_sram(0x11132c,0x0);

//  add 32.0.0.42 255.255.255.255 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x2000002a, mask = 0xffffffff, blade_nh_id=0x2a000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040828,0x330000);

// writing nexthop entry with flags 0x0

set_sram(0x111330,0x0);

// blade 0x2 and NH 0x20

set_sram(0x111334,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x111338,0x23fc0001);


set_sram(0x11133c,0x0);

//  add 32.0.0.43 255.255.255.255 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x2000002b, mask = 0xffffffff, blade_nh_id=0x2b000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x4004082c,0x340000);

// writing nexthop entry with flags 0x0

set_sram(0x111340,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111344,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111348,0x23fc0002);


set_sram(0x11134c,0x0);

//  add 32.0.0.44 255.255.255.255 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x2000002c, mask = 0xffffffff, blade_nh_id=0x2c000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x40040830,0x350000);

// writing nexthop entry with flags 0x0

set_sram(0x111350,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111354,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111358,0x23fc0003);


set_sram(0x11135c,0x0);

//  add 32.0.0.45 255.255.255.255 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x2000002d, mask = 0xffffffff, blade_nh_id=0x2d000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040834,0x360000);

// writing nexthop entry with flags 0x0

set_sram(0x111360,0x0);

// blade 0x2 and NH 0x01

set_sram(0x111364,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x111368,0x23fc0000);


set_sram(0x11136c,0x0);

//  add 32.0.0.46 255.255.255.255 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x2000002e, mask = 0xffffffff, blade_nh_id=0x2e000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040838,0x370000);

// writing nexthop entry with flags 0x0

set_sram(0x111370,0x0);

// blade 0x2 and NH 0x20

set_sram(0x111374,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x111378,0x23fc0001);


set_sram(0x11137c,0x0);

//  add 32.0.0.47 255.255.255.255 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x2000002f, mask = 0xffffffff, blade_nh_id=0x2f000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x4004083c,0x380000);

// writing nexthop entry with flags 0x0

set_sram(0x111380,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111384,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111388,0x23fc0002);


set_sram(0x11138c,0x0);

//  add 32.0.0.48 255.255.255.255 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x20000030, mask = 0xffffffff, blade_nh_id=0x30000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x40040840,0x390000);

// writing nexthop entry with flags 0x0

set_sram(0x111390,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111394,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111398,0x23fc0003);


set_sram(0x11139c,0x0);


	set_sram(0x4004074c,0x11);

//  add 32.0.0.49 255.255.255.255 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x20000031, mask = 0xffffffff, blade_nh_id=0x31000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040844,0x3a0000);

// writing nexthop entry with flags 0x0

set_sram(0x1113a0,0x0);

// blade 0x2 and NH 0x01

set_sram(0x1113a4,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x1113a8,0x23fc0000);


set_sram(0x1113ac,0x0);

//  add 32.0.0.50 255.255.255.255 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x20000032, mask = 0xffffffff, blade_nh_id=0x32000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040848,0x3b0000);

// writing nexthop entry with flags 0x0

set_sram(0x1113b0,0x0);

// blade 0x2 and NH 0x20

set_sram(0x1113b4,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x1113b8,0x23fc0001);


set_sram(0x1113bc,0x0);

//  add 32.0.0.51 255.255.255.255 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x20000033, mask = 0xffffffff, blade_nh_id=0x33000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x4004084c,0x3c0000);

// writing nexthop entry with flags 0x0

set_sram(0x1113c0,0x0);

// blade 0x2 and NH 0x40

set_sram(0x1113c4,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x1113c8,0x23fc0002);


set_sram(0x1113cc,0x0);

//  add 32.0.0.52 255.255.255.255 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x20000034, mask = 0xffffffff, blade_nh_id=0x34000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x40040850,0x3d0000);

// writing nexthop entry with flags 0x0

set_sram(0x1113d0,0x0);

// blade 0x2 and NH 0x60

set_sram(0x1113d4,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x1113d8,0x23fc0003);


set_sram(0x1113dc,0x0);

//  add 32.0.0.53 255.255.255.255 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x20000035, mask = 0xffffffff, blade_nh_id=0x35000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040854,0x3e0000);

// writing nexthop entry with flags 0x0

set_sram(0x1113e0,0x0);

// blade 0x2 and NH 0x01

set_sram(0x1113e4,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x1113e8,0x23fc0000);


set_sram(0x1113ec,0x0);

//  add 32.0.0.54 255.255.255.255 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x20000036, mask = 0xffffffff, blade_nh_id=0x36000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040858,0x3f0000);

// writing nexthop entry with flags 0x0

set_sram(0x1113f0,0x0);

// blade 0x2 and NH 0x20

set_sram(0x1113f4,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x1113f8,0x23fc0001);


set_sram(0x1113fc,0x0);

//  add 32.0.0.55 255.255.255.255 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x20000037, mask = 0xffffffff, blade_nh_id=0x37000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x4004085c,0x400000);

// writing nexthop entry with flags 0x0

set_sram(0x111400,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111404,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111408,0x23fc0002);


set_sram(0x11140c,0x0);

//  add 32.0.0.56 255.255.255.255 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x20000038, mask = 0xffffffff, blade_nh_id=0x38000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x40040860,0x410000);

// writing nexthop entry with flags 0x0

set_sram(0x111410,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111414,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111418,0x23fc0003);


set_sram(0x11141c,0x0);

//  add 32.0.0.57 255.255.255.255 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x20000039, mask = 0xffffffff, blade_nh_id=0x39000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040864,0x420000);

// writing nexthop entry with flags 0x0

set_sram(0x111420,0x0);

// blade 0x2 and NH 0x01

set_sram(0x111424,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x111428,0x23fc0000);


set_sram(0x11142c,0x0);

//  add 32.0.0.58 255.255.255.255 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x2000003a, mask = 0xffffffff, blade_nh_id=0x3a000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040868,0x430000);

// writing nexthop entry with flags 0x0

set_sram(0x111430,0x0);

// blade 0x2 and NH 0x20

set_sram(0x111434,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x111438,0x23fc0001);


set_sram(0x11143c,0x0);

//  add 32.0.0.59 255.255.255.255 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x2000003b, mask = 0xffffffff, blade_nh_id=0x3b000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x4004086c,0x440000);

// writing nexthop entry with flags 0x0

set_sram(0x111440,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111444,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111448,0x23fc0002);


set_sram(0x11144c,0x0);

//  add 32.0.0.60 255.255.255.255 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x2000003c, mask = 0xffffffff, blade_nh_id=0x3c000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x40040870,0x450000);

// writing nexthop entry with flags 0x0

set_sram(0x111450,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111454,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111458,0x23fc0003);


set_sram(0x11145c,0x0);

//  add 32.0.0.61 255.255.255.255 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x2000003d, mask = 0xffffffff, blade_nh_id=0x3d000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040874,0x460000);

// writing nexthop entry with flags 0x0

set_sram(0x111460,0x0);

// blade 0x2 and NH 0x01

set_sram(0x111464,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x111468,0x23fc0000);


set_sram(0x11146c,0x0);

//  add 32.0.0.62 255.255.255.255 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x2000003e, mask = 0xffffffff, blade_nh_id=0x3e000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040878,0x470000);

// writing nexthop entry with flags 0x0

set_sram(0x111470,0x0);

// blade 0x2 and NH 0x20

set_sram(0x111474,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x111478,0x23fc0001);


set_sram(0x11147c,0x0);

//  add 32.0.0.63 255.255.255.255 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x2000003f, mask = 0xffffffff, blade_nh_id=0x3f000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x4004087c,0x480000);

// writing nexthop entry with flags 0x0

set_sram(0x111480,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111484,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111488,0x23fc0002);


set_sram(0x11148c,0x0);

//  add 32.0.0.64 255.255.255.255 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x20000040, mask = 0xffffffff, blade_nh_id=0x3f000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x40040880,0x490000);

// writing nexthop entry with flags 0x0

set_sram(0x111490,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111494,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111498,0x23fc0003);


set_sram(0x11149c,0x0);


	set_sram(0x40040750,0x12);

// 
// 
// 
// #-----------------------------------------------
// # 16-bit routes - Forward			 
// #-----------------------------------------------
// 
//  add 16.1.0.0 255.255.0.0 0x0001 0x0 9212 0x02000000

// @@@ Adding dest=0x10010000, mask = 0xffff0000, blade_nh_id=0x01,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040904,0x4a0000);

// writing nexthop entry with flags 0x0

set_sram(0x1114a0,0x0);

// blade 0x2 and NH 0x01

set_sram(0x1114a4,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x1114a8,0x23fc0000);


set_sram(0x1114ac,0x0);


	set_sram(0x400408c0,0x14);


	set_sram(0x40040040,0x13);

//  add 16.2.0.0 255.255.0.0 0x0020 0x0 9212 0x02000001

// @@@ Adding dest=0x10020000, mask = 0xffff0000, blade_nh_id=0x2000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040908,0x4b0000);

// writing nexthop entry with flags 0x0

set_sram(0x1114b0,0x0);

// blade 0x2 and NH 0x20

set_sram(0x1114b4,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x1114b8,0x23fc0001);


set_sram(0x1114bc,0x0);

//  add 16.3.0.0 255.255.0.0 0x0040 0x0 9212 0x02000002

// @@@ Adding dest=0x10030000, mask = 0xffff0000, blade_nh_id=0x3000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x4004090c,0x4c0000);

// writing nexthop entry with flags 0x0

set_sram(0x1114c0,0x0);

// blade 0x2 and NH 0x40

set_sram(0x1114c4,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x1114c8,0x23fc0002);


set_sram(0x1114cc,0x0);

//  add 16.4.0.0 255.255.0.0 0x0060 0x0 9212 0x02000003

// @@@ Adding dest=0x10040000, mask = 0xffff0000, blade_nh_id=0x4000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x40040910,0x4d0000);

// writing nexthop entry with flags 0x0

set_sram(0x1114d0,0x0);

// blade 0x2 and NH 0x60

set_sram(0x1114d4,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x1114d8,0x23fc0003);


set_sram(0x1114dc,0x0);

//  add 16.5.0.0 255.255.0.0 0x0001 0x0 9212 0x02000000

// @@@ Adding dest=0x10050000, mask = 0xffff0000, blade_nh_id=0x5000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040914,0x4e0000);

// writing nexthop entry with flags 0x0

set_sram(0x1114e0,0x0);

// blade 0x2 and NH 0x01

set_sram(0x1114e4,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x1114e8,0x23fc0000);


set_sram(0x1114ec,0x0);

//  add 16.6.0.0 255.255.0.0 0x0020 0x0 9212 0x02000001

// @@@ Adding dest=0x10060000, mask = 0xffff0000, blade_nh_id=0x6000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040918,0x4f0000);

// writing nexthop entry with flags 0x0

set_sram(0x1114f0,0x0);

// blade 0x2 and NH 0x20

set_sram(0x1114f4,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x1114f8,0x23fc0001);


set_sram(0x1114fc,0x0);

//  add 16.7.0.0 255.255.0.0 0x0040 0x0 9212 0x02000002

// @@@ Adding dest=0x10070000, mask = 0xffff0000, blade_nh_id=0x7000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x4004091c,0x500000);

// writing nexthop entry with flags 0x0

set_sram(0x111500,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111504,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111508,0x23fc0002);


set_sram(0x11150c,0x0);

//  add 16.8.0.0 255.255.0.0 0x0060 0x0 9212 0x02000003

// @@@ Adding dest=0x10080000, mask = 0xffff0000, blade_nh_id=0x8000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x40040920,0x510000);

// writing nexthop entry with flags 0x0

set_sram(0x111510,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111514,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111518,0x23fc0003);


set_sram(0x11151c,0x0);

//  add 16.9.0.0 255.255.0.0 0x0001 0x0 9212 0x02000000

// @@@ Adding dest=0x10090000, mask = 0xffff0000, blade_nh_id=0x9000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040924,0x520000);

// writing nexthop entry with flags 0x0

set_sram(0x111520,0x0);

// blade 0x2 and NH 0x01

set_sram(0x111524,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x111528,0x23fc0000);


set_sram(0x11152c,0x0);

//  add 16.10.0.0 255.255.0.0 0x0020 0x0 9212 0x02000001

// @@@ Adding dest=0x100a0000, mask = 0xffff0000, blade_nh_id=0xa000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040928,0x530000);

// writing nexthop entry with flags 0x0

set_sram(0x111530,0x0);

// blade 0x2 and NH 0x20

set_sram(0x111534,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x111538,0x23fc0001);


set_sram(0x11153c,0x0);

//  add 16.11.0.0 255.255.0.0 0x0040 0x0 9212 0x02000002

// @@@ Adding dest=0x100b0000, mask = 0xffff0000, blade_nh_id=0xb000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x4004092c,0x540000);

// writing nexthop entry with flags 0x0

set_sram(0x111540,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111544,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111548,0x23fc0002);


set_sram(0x11154c,0x0);

//  add 16.12.0.0 255.255.0.0 0x0060 0x0 9212 0x02000003

// @@@ Adding dest=0x100c0000, mask = 0xffff0000, blade_nh_id=0xc000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x40040930,0x550000);

// writing nexthop entry with flags 0x0

set_sram(0x111550,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111554,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111558,0x23fc0003);


set_sram(0x11155c,0x0);

//  add 16.13.0.0 255.255.0.0 0x0001 0x0 9212 0x02000000

// @@@ Adding dest=0x100d0000, mask = 0xffff0000, blade_nh_id=0xd000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040934,0x560000);

// writing nexthop entry with flags 0x0

set_sram(0x111560,0x0);

// blade 0x2 and NH 0x01

set_sram(0x111564,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x111568,0x23fc0000);


set_sram(0x11156c,0x0);

//  add 16.14.0.0 255.255.0.0 0x0020 0x0 9212 0x02000001

// @@@ Adding dest=0x100e0000, mask = 0xffff0000, blade_nh_id=0xe000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040938,0x570000);

// writing nexthop entry with flags 0x0

set_sram(0x111570,0x0);

// blade 0x2 and NH 0x20

set_sram(0x111574,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x111578,0x23fc0001);


set_sram(0x11157c,0x0);

//  add 16.15.0.0 255.255.0.0 0x0040 0x0 9212 0x02000002

// @@@ Adding dest=0x100f0000, mask = 0xffff0000, blade_nh_id=0xf000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x4004093c,0x580000);

// writing nexthop entry with flags 0x0

set_sram(0x111580,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111584,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111588,0x23fc0002);


set_sram(0x11158c,0x0);

//  add 16.16.0.0 255.255.0.0 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x10100000, mask = 0xffff0000, blade_nh_id=0x10000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x40040940,0x590000);

// writing nexthop entry with flags 0x0

set_sram(0x111590,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111594,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111598,0x23fc0003);


set_sram(0x11159c,0x0);


	set_sram(0x400408c4,0x15);

//  add 16.17.0.0 255.255.0.0 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x10110000, mask = 0xffff0000, blade_nh_id=0x11000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040944,0x5a0000);

// writing nexthop entry with flags 0x0

set_sram(0x1115a0,0x0);

// blade 0x2 and NH 0x01

set_sram(0x1115a4,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x1115a8,0x23fc0000);


set_sram(0x1115ac,0x0);

//  add 16.18.0.0 255.255.0.0 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x10120000, mask = 0xffff0000, blade_nh_id=0x12000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040948,0x5b0000);

// writing nexthop entry with flags 0x0

set_sram(0x1115b0,0x0);

// blade 0x2 and NH 0x20

set_sram(0x1115b4,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x1115b8,0x23fc0001);


set_sram(0x1115bc,0x0);

//  add 16.19.0.0 255.255.0.0 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x10130000, mask = 0xffff0000, blade_nh_id=0x13000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x4004094c,0x5c0000);

// writing nexthop entry with flags 0x0

set_sram(0x1115c0,0x0);

// blade 0x2 and NH 0x40

set_sram(0x1115c4,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x1115c8,0x23fc0002);


set_sram(0x1115cc,0x0);

//  add 16.20.0.0 255.255.0.0 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x10140000, mask = 0xffff0000, blade_nh_id=0x14000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x40040950,0x5d0000);

// writing nexthop entry with flags 0x0

set_sram(0x1115d0,0x0);

// blade 0x2 and NH 0x60

set_sram(0x1115d4,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x1115d8,0x23fc0003);


set_sram(0x1115dc,0x0);

//  add 16.21.0.0 255.255.0.0 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x10150000, mask = 0xffff0000, blade_nh_id=0x15000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040954,0x5e0000);

// writing nexthop entry with flags 0x0

set_sram(0x1115e0,0x0);

// blade 0x2 and NH 0x01

set_sram(0x1115e4,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x1115e8,0x23fc0000);


set_sram(0x1115ec,0x0);

//  add 16.22.0.0 255.255.0.0 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x10160000, mask = 0xffff0000, blade_nh_id=0x16000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040958,0x5f0000);

// writing nexthop entry with flags 0x0

set_sram(0x1115f0,0x0);

// blade 0x2 and NH 0x20

set_sram(0x1115f4,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x1115f8,0x23fc0001);


set_sram(0x1115fc,0x0);

//  add 16.23.0.0 255.255.0.0 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x10170000, mask = 0xffff0000, blade_nh_id=0x17000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x4004095c,0x600000);

// writing nexthop entry with flags 0x0

set_sram(0x111600,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111604,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111608,0x23fc0002);


set_sram(0x11160c,0x0);

//  add 16.24.0.0 255.255.0.0 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x10180000, mask = 0xffff0000, blade_nh_id=0x18000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x40040960,0x610000);

// writing nexthop entry with flags 0x0

set_sram(0x111610,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111614,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111618,0x23fc0003);


set_sram(0x11161c,0x0);

//  add 16.25.0.0 255.255.0.0 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x10190000, mask = 0xffff0000, blade_nh_id=0x19000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040964,0x620000);

// writing nexthop entry with flags 0x0

set_sram(0x111620,0x0);

// blade 0x2 and NH 0x01

set_sram(0x111624,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x111628,0x23fc0000);


set_sram(0x11162c,0x0);

//  add 16.26.0.0 255.255.0.0 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x101a0000, mask = 0xffff0000, blade_nh_id=0x1a000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040968,0x630000);

// writing nexthop entry with flags 0x0

set_sram(0x111630,0x0);

// blade 0x2 and NH 0x20

set_sram(0x111634,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x111638,0x23fc0001);


set_sram(0x11163c,0x0);

//  add 16.27.0.0 255.255.0.0 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x101b0000, mask = 0xffff0000, blade_nh_id=0x1b000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x4004096c,0x640000);

// writing nexthop entry with flags 0x0

set_sram(0x111640,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111644,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111648,0x23fc0002);


set_sram(0x11164c,0x0);

//  add 16.28.0.0 255.255.0.0 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x101c0000, mask = 0xffff0000, blade_nh_id=0x1c000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x40040970,0x650000);

// writing nexthop entry with flags 0x0

set_sram(0x111650,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111654,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111658,0x23fc0003);


set_sram(0x11165c,0x0);

//  add 16.29.0.0 255.255.0.0 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x101d0000, mask = 0xffff0000, blade_nh_id=0x1d000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040974,0x660000);

// writing nexthop entry with flags 0x0

set_sram(0x111660,0x0);

// blade 0x2 and NH 0x01

set_sram(0x111664,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x111668,0x23fc0000);


set_sram(0x11166c,0x0);

//  add 16.30.0.0 255.255.0.0 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x101e0000, mask = 0xffff0000, blade_nh_id=0x1e000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040978,0x670000);

// writing nexthop entry with flags 0x0

set_sram(0x111670,0x0);

// blade 0x2 and NH 0x20

set_sram(0x111674,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x111678,0x23fc0001);


set_sram(0x11167c,0x0);

//  add 16.31.0.0 255.255.0.0 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x101f0000, mask = 0xffff0000, blade_nh_id=0x1f000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x4004097c,0x680000);

// writing nexthop entry with flags 0x0

set_sram(0x111680,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111684,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111688,0x23fc0002);


set_sram(0x11168c,0x0);

//  add 16.32.0.0 255.255.0.0 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x10200000, mask = 0xffff0000, blade_nh_id=0x20000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x40040980,0x690000);

// writing nexthop entry with flags 0x0

set_sram(0x111690,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111694,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111698,0x23fc0003);


set_sram(0x11169c,0x0);


	set_sram(0x400408c8,0x16);

//  add 16.33.0.0 255.255.0.0 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x10210000, mask = 0xffff0000, blade_nh_id=0x21000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040984,0x6a0000);

// writing nexthop entry with flags 0x0

set_sram(0x1116a0,0x0);

// blade 0x2 and NH 0x01

set_sram(0x1116a4,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x1116a8,0x23fc0000);


set_sram(0x1116ac,0x0);

//  add 16.34.0.0 255.255.0.0 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x10220000, mask = 0xffff0000, blade_nh_id=0x22000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040988,0x6b0000);

// writing nexthop entry with flags 0x0

set_sram(0x1116b0,0x0);

// blade 0x2 and NH 0x20

set_sram(0x1116b4,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x1116b8,0x23fc0001);


set_sram(0x1116bc,0x0);

//  add 16.35.0.0 255.255.0.0 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x10230000, mask = 0xffff0000, blade_nh_id=0x23000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x4004098c,0x6c0000);

// writing nexthop entry with flags 0x0

set_sram(0x1116c0,0x0);

// blade 0x2 and NH 0x40

set_sram(0x1116c4,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x1116c8,0x23fc0002);


set_sram(0x1116cc,0x0);

//  add 16.36.0.0 255.255.0.0 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x10240000, mask = 0xffff0000, blade_nh_id=0x24000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x40040990,0x6d0000);

// writing nexthop entry with flags 0x0

set_sram(0x1116d0,0x0);

// blade 0x2 and NH 0x60

set_sram(0x1116d4,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x1116d8,0x23fc0003);


set_sram(0x1116dc,0x0);

//  add 16.37.0.0 255.255.0.0 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x10250000, mask = 0xffff0000, blade_nh_id=0x25000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040994,0x6e0000);

// writing nexthop entry with flags 0x0

set_sram(0x1116e0,0x0);

// blade 0x2 and NH 0x01

set_sram(0x1116e4,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x1116e8,0x23fc0000);


set_sram(0x1116ec,0x0);

//  add 16.38.0.0 255.255.0.0 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x10260000, mask = 0xffff0000, blade_nh_id=0x26000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040998,0x6f0000);

// writing nexthop entry with flags 0x0

set_sram(0x1116f0,0x0);

// blade 0x2 and NH 0x20

set_sram(0x1116f4,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x1116f8,0x23fc0001);


set_sram(0x1116fc,0x0);

//  add 16.39.0.0 255.255.0.0 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x10270000, mask = 0xffff0000, blade_nh_id=0x27000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x4004099c,0x700000);

// writing nexthop entry with flags 0x0

set_sram(0x111700,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111704,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111708,0x23fc0002);


set_sram(0x11170c,0x0);

//  add 16.40.0.0 255.255.0.0 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x10280000, mask = 0xffff0000, blade_nh_id=0x28000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x400409a0,0x710000);

// writing nexthop entry with flags 0x0

set_sram(0x111710,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111714,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111718,0x23fc0003);


set_sram(0x11171c,0x0);

//  add 16.41.0.0 255.255.0.0 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x10290000, mask = 0xffff0000, blade_nh_id=0x29000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x400409a4,0x720000);

// writing nexthop entry with flags 0x0

set_sram(0x111720,0x0);

// blade 0x2 and NH 0x01

set_sram(0x111724,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x111728,0x23fc0000);


set_sram(0x11172c,0x0);

//  add 16.42.0.0 255.255.0.0 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x102a0000, mask = 0xffff0000, blade_nh_id=0x2a000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x400409a8,0x730000);

// writing nexthop entry with flags 0x0

set_sram(0x111730,0x0);

// blade 0x2 and NH 0x20

set_sram(0x111734,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x111738,0x23fc0001);


set_sram(0x11173c,0x0);

//  add 16.43.0.0 255.255.0.0 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x102b0000, mask = 0xffff0000, blade_nh_id=0x2b000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x400409ac,0x740000);

// writing nexthop entry with flags 0x0

set_sram(0x111740,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111744,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111748,0x23fc0002);


set_sram(0x11174c,0x0);

//  add 16.44.0.0 255.255.0.0 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x102c0000, mask = 0xffff0000, blade_nh_id=0x2c000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x400409b0,0x750000);

// writing nexthop entry with flags 0x0

set_sram(0x111750,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111754,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111758,0x23fc0003);


set_sram(0x11175c,0x0);

//  add 16.45.0.0 255.255.0.0 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x102d0000, mask = 0xffff0000, blade_nh_id=0x2d000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x400409b4,0x760000);

// writing nexthop entry with flags 0x0

set_sram(0x111760,0x0);

// blade 0x2 and NH 0x01

set_sram(0x111764,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x111768,0x23fc0000);


set_sram(0x11176c,0x0);

//  add 16.46.0.0 255.255.0.0 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x102e0000, mask = 0xffff0000, blade_nh_id=0x2e000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x400409b8,0x770000);

// writing nexthop entry with flags 0x0

set_sram(0x111770,0x0);

// blade 0x2 and NH 0x20

set_sram(0x111774,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x111778,0x23fc0001);


set_sram(0x11177c,0x0);

//  add 16.47.0.0 255.255.0.0 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x102f0000, mask = 0xffff0000, blade_nh_id=0x2f000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x400409bc,0x780000);

// writing nexthop entry with flags 0x0

set_sram(0x111780,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111784,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111788,0x23fc0002);


set_sram(0x11178c,0x0);

//  add 16.48.0.0 255.255.0.0 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x10300000, mask = 0xffff0000, blade_nh_id=0x30000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x400409c0,0x790000);

// writing nexthop entry with flags 0x0

set_sram(0x111790,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111794,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111798,0x23fc0003);


set_sram(0x11179c,0x0);


	set_sram(0x400408cc,0x17);

//  add 16.49.0.0 255.255.0.0 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x10310000, mask = 0xffff0000, blade_nh_id=0x31000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x400409c4,0x7a0000);

// writing nexthop entry with flags 0x0

set_sram(0x1117a0,0x0);

// blade 0x2 and NH 0x01

set_sram(0x1117a4,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x1117a8,0x23fc0000);


set_sram(0x1117ac,0x0);

//  add 16.50.0.0 255.255.0.0 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x10320000, mask = 0xffff0000, blade_nh_id=0x32000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x400409c8,0x7b0000);

// writing nexthop entry with flags 0x0

set_sram(0x1117b0,0x0);

// blade 0x2 and NH 0x20

set_sram(0x1117b4,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x1117b8,0x23fc0001);


set_sram(0x1117bc,0x0);

//  add 16.51.0.0 255.255.0.0 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x10330000, mask = 0xffff0000, blade_nh_id=0x33000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x400409cc,0x7c0000);

// writing nexthop entry with flags 0x0

set_sram(0x1117c0,0x0);

// blade 0x2 and NH 0x40

set_sram(0x1117c4,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x1117c8,0x23fc0002);


set_sram(0x1117cc,0x0);

//  add 16.52.0.0 255.255.0.0 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x10340000, mask = 0xffff0000, blade_nh_id=0x34000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x400409d0,0x7d0000);

// writing nexthop entry with flags 0x0

set_sram(0x1117d0,0x0);

// blade 0x2 and NH 0x60

set_sram(0x1117d4,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x1117d8,0x23fc0003);


set_sram(0x1117dc,0x0);

//  add 16.53.0.0 255.255.0.0 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x10350000, mask = 0xffff0000, blade_nh_id=0x35000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x400409d4,0x7e0000);

// writing nexthop entry with flags 0x0

set_sram(0x1117e0,0x0);

// blade 0x2 and NH 0x01

set_sram(0x1117e4,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x1117e8,0x23fc0000);


set_sram(0x1117ec,0x0);

//  add 16.54.0.0 255.255.0.0 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x10360000, mask = 0xffff0000, blade_nh_id=0x36000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x400409d8,0x7f0000);

// writing nexthop entry with flags 0x0

set_sram(0x1117f0,0x0);

// blade 0x2 and NH 0x20

set_sram(0x1117f4,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x1117f8,0x23fc0001);


set_sram(0x1117fc,0x0);

//  add 16.55.0.0 255.255.0.0 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x10370000, mask = 0xffff0000, blade_nh_id=0x37000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x400409dc,0x800000);

// writing nexthop entry with flags 0x0

set_sram(0x111800,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111804,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111808,0x23fc0002);


set_sram(0x11180c,0x0);

//  add 16.56.0.0 255.255.0.0 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x10380000, mask = 0xffff0000, blade_nh_id=0x38000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x400409e0,0x010000);

// writing nexthop entry with flags 0x0

set_sram(0x111810,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111814,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111818,0x23fc0003);


set_sram(0x11181c,0x0);

//  add 16.57.0.0 255.255.0.0 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x10390000, mask = 0xffff0000, blade_nh_id=0x39000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x400409e4,0x200000);

// writing nexthop entry with flags 0x0

set_sram(0x111820,0x0);

// blade 0x2 and NH 0x01

set_sram(0x111824,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x111828,0x23fc0000);


set_sram(0x11182c,0x0);

//  add 16.58.0.0 255.255.0.0 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x103a0000, mask = 0xffff0000, blade_nh_id=0x3a000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x400409e8,0x400000);

// writing nexthop entry with flags 0x0

set_sram(0x111830,0x0);

// blade 0x2 and NH 0x20

set_sram(0x111834,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x111838,0x23fc0001);


set_sram(0x11183c,0x0);

//  add 16.59.0.0 255.255.0.0 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x103b0000, mask = 0xffff0000, blade_nh_id=0x3b000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x400409ec,0x600000);

// writing nexthop entry with flags 0x0

set_sram(0x111840,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111844,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111848,0x23fc0002);


set_sram(0x11184c,0x0);

//  add 16.60.0.0 255.255.0.0 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x103c0000, mask = 0xffff0000, blade_nh_id=0x3c000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x400409f0,0x850000);

// writing nexthop entry with flags 0x0

set_sram(0x111850,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111854,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111858,0x23fc0003);


set_sram(0x11185c,0x0);

//  add 16.61.0.0 255.255.0.0 0x0001 0x0 9212 0x2000000

// @@@ Adding dest=0x103d0000, mask = 0xffff0000, blade_nh_id=0x3d000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x400409f4,0x860000);

// writing nexthop entry with flags 0x0

set_sram(0x111860,0x0);

// blade 0x2 and NH 0x01

set_sram(0x111864,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x111868,0x23fc0000);


set_sram(0x11186c,0x0);

//  add 16.62.0.0 255.255.0.0 0x0020 0x0 9212 0x2000001

// @@@ Adding dest=0x103e0000, mask = 0xffff0000, blade_nh_id=0x3e000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x400409f8,0x870000);

// writing nexthop entry with flags 0x0

set_sram(0x111870,0x0);

// blade 0x2 and NH 0x20

set_sram(0x111874,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x111878,0x23fc0001);


set_sram(0x11187c,0x0);

//  add 16.63.0.0 255.255.0.0 0x0040 0x0 9212 0x2000002

// @@@ Adding dest=0x103f0000, mask = 0xffff0000, blade_nh_id=0x3f000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x400409fc,0x880000);

// writing nexthop entry with flags 0x0

set_sram(0x111880,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111884,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x111888,0x23fc0002);


set_sram(0x11188c,0x0);

//  add 16.64.0.0 255.255.0.0 0x0060 0x0 9212 0x2000003

// @@@ Adding dest=0x10400000, mask = 0xffff0000, blade_nh_id=0x3f000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x40040a00,0x890000);

// writing nexthop entry with flags 0x0

set_sram(0x111890,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111894,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x111898,0x23fc0003);


set_sram(0x11189c,0x0);


	set_sram(0x400408d0,0x18);

// 
// 
// 
// #-----------------------------------------------
// # 32-bit routes - Forward			 
// #-----------------------------------------------
// 
//  add 48.2.0.1 255.255.255.255 0x0001 0x0 9212 0x02000000

// @@@ Adding dest=0x30020001, mask = 0xffffffff, blade_nh_id=0x01,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040b04,0x8a0000);

// writing nexthop entry with flags 0x0

set_sram(0x1118a0,0x0);

// blade 0x2 and NH 0x01

set_sram(0x1118a4,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x1118a8,0x23fc0000);


set_sram(0x1118ac,0x0);


	set_sram(0x40040ac0,0x1c);


	set_sram(0x40040a80,0x1b);


	set_sram(0x40040a40,0x1a);


	set_sram(0x4000c008,0x19);

//  add 48.2.0.2 255.255.255.255 0x0020 0x0 9212 0x02000001

// @@@ Adding dest=0x30020002, mask = 0xffffffff, blade_nh_id=0x2000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040b08,0x8b0000);

// writing nexthop entry with flags 0x0

set_sram(0x1118b0,0x0);

// blade 0x2 and NH 0x20

set_sram(0x1118b4,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x1118b8,0x23fc0001);


set_sram(0x1118bc,0x0);

//  add 48.2.0.3 255.255.255.255 0x0040 0x0 9212 0x02000002

// @@@ Adding dest=0x30020003, mask = 0xffffffff, blade_nh_id=0x3000040,mtu_port=0x23fc0002,flags=0x0

	set_sram(0x40040b0c,0x8c0000);

// writing nexthop entry with flags 0x0

set_sram(0x1118c0,0x0);

// blade 0x2 and NH 0x40

set_sram(0x1118c4,0x2000040);

// MTU is 9212, outport is 0x2

set_sram(0x1118c8,0x23fc0002);


set_sram(0x1118cc,0x0);

//  add 48.2.0.4 255.255.255.255 0x0060 0x0 9212 0x02000003

// @@@ Adding dest=0x30020004, mask = 0xffffffff, blade_nh_id=0x4000060,mtu_port=0x23fc0003,flags=0x0

	set_sram(0x40040b10,0x8d0000);

// writing nexthop entry with flags 0x0

set_sram(0x1118d0,0x0);

// blade 0x2 and NH 0x60

set_sram(0x1118d4,0x2000060);

// MTU is 9212, outport is 0x3

set_sram(0x1118d8,0x23fc0003);


set_sram(0x1118dc,0x0);

//  add 48.2.0.5 255.255.255.255 0x0001 0x0 9212 0x02000000

// @@@ Adding dest=0x30020005, mask = 0xffffffff, blade_nh_id=0x5000001,mtu_port=0x23fc0000,flags=0x0

	set_sram(0x40040b14,0x8e0000);

// writing nexthop entry with flags 0x0

set_sram(0x1118e0,0x0);

// blade 0x2 and NH 0x01

set_sram(0x1118e4,0x2000001);

// MTU is 9212, outport is 0x0

set_sram(0x1118e8,0x23fc0000);


set_sram(0x1118ec,0x0);

// 
// #-----------------------------------------------
// # 16-bit routes - Forward			 
// #-----------------------------------------------
// 
//  add 48.2.0.0 255.255.0.0 0x0020 0x0 9212 0x02000001

// @@@ Adding dest=0x30020000, mask = 0xffff0000, blade_nh_id=0x6000020,mtu_port=0x23fc0001,flags=0x0

	set_sram(0x40040b88,0x8f0000);

// writing nexthop entry with flags 0x0

set_sram(0x1118f0,0x0);

// blade 0x2 and NH 0x20

set_sram(0x1118f4,0x2000020);

// MTU is 9212, outport is 0x1

set_sram(0x1118f8,0x23fc0001);


set_sram(0x1118fc,0x0);


	set_sram(0x40040b40,0x1e);


	set_sram(0x400400c0,0x1d);

// 
// 
// 
// #-----------------------------------------------
// # 8-bit routes  - Forward			 
// #-----------------------------------------------
// 
// add 8.0.0.0 255.0.0.0 0x0040 0x0 9192 0x02000002

// @@@ Adding dest=0x8000000, mask = 0xff000000, blade_nh_id=0x2000040,mtu_port=0x23e80002,flags=0x0

	set_sram(0x40040020,0x900000);

// writing nexthop entry with flags 0x0

set_sram(0x111900,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111904,0x2000040);

// MTU is 9192, outport is 0x2

set_sram(0x111908,0x23e80002);


set_sram(0x11190c,0x0);

// 
// 
// #-----------------------------------------------
// # 11-bit routes  - Forward			 
// #-----------------------------------------------
// 
// add 11.32.0.0 255.224.0.0 0x0060 0x0 9192 0x2000003

// @@@ Adding dest=0xb200000, mask = 0xffe00000, blade_nh_id=0x20000060,mtu_port=0x23e80003,flags=0x0

	set_sram(0x40040bc8,0x910000);


	set_sram(0x40040bcc,0x910000);

// writing nexthop entry with flags 0x0

set_sram(0x111910,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111914,0x2000060);

// MTU is 9192, outport is 0x3

set_sram(0x111918,0x23e80003);


set_sram(0x11191c,0x0);


	set_sram(0x4004002c,0x1f);

// add 11.64.0.0 255.224.0.0 0x0001 0x0 9192 0x2000000

// @@@ Adding dest=0xb400000, mask = 0xffe00000, blade_nh_id=0x3f000001,mtu_port=0x23e80000,flags=0x0

	set_sram(0x40040bd0,0x920000);


	set_sram(0x40040bd4,0x920000);

// writing nexthop entry with flags 0x0

set_sram(0x111920,0x0);

// blade 0x2 and NH 0x01

set_sram(0x111924,0x2000001);

// MTU is 9192, outport is 0x0

set_sram(0x111928,0x23e80000);


set_sram(0x11192c,0x0);

// 
// 
// #-----------------------------------------------
// # 15-bit routes - Forward			 
// #-----------------------------------------------
// 
// add 15.2.0.0 255.254.0.0 0x0020 0x0 9192 0x02000001

// @@@ Adding dest=0xf020000, mask = 0xfffe0000, blade_nh_id=0x2000020,mtu_port=0x23e80001,flags=0x0

	set_sram(0x40040c48,0x930000);


	set_sram(0x40040c4c,0x930000);

// writing nexthop entry with flags 0x0

set_sram(0x111930,0x0);

// blade 0x2 and NH 0x20

set_sram(0x111934,0x2000020);

// MTU is 9192, outport is 0x1

set_sram(0x111938,0x23e80001);


set_sram(0x11193c,0x0);


	set_sram(0x40040c00,0x21);


	set_sram(0x4004003c,0x20);

// add 15.4.0.0 255.254.0.0 0x0040 0x0 9192 0x02000002

// @@@ Adding dest=0xf040000, mask = 0xfffe0000, blade_nh_id=0x4000040,mtu_port=0x23e80002,flags=0x0

	set_sram(0x40040c50,0x940000);


	set_sram(0x40040c54,0x940000);

// writing nexthop entry with flags 0x0

set_sram(0x111940,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111944,0x2000040);

// MTU is 9192, outport is 0x2

set_sram(0x111948,0x23e80002);


set_sram(0x11194c,0x0);

// add 15.6.0.0 255.254.0.0 0x0060 0x0 9192 0x02000003

// @@@ Adding dest=0xf060000, mask = 0xfffe0000, blade_nh_id=0x6000060,mtu_port=0x23e80003,flags=0x0

	set_sram(0x40040c58,0x950000);


	set_sram(0x40040c5c,0x950000);

// writing nexthop entry with flags 0x0

set_sram(0x111950,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111954,0x2000060);

// MTU is 9192, outport is 0x3

set_sram(0x111958,0x23e80003);


set_sram(0x11195c,0x0);

// add 15.8.0.0 255.254.0.0 0x0001 0x0 9192 0x02000000

// @@@ Adding dest=0xf080000, mask = 0xfffe0000, blade_nh_id=0x8000001,mtu_port=0x23e80000,flags=0x0

	set_sram(0x40040c60,0x960000);


	set_sram(0x40040c64,0x960000);

// writing nexthop entry with flags 0x0

set_sram(0x111960,0x0);

// blade 0x2 and NH 0x01

set_sram(0x111964,0x2000001);

// MTU is 9192, outport is 0x0

set_sram(0x111968,0x23e80000);


set_sram(0x11196c,0x0);

// add 15.10.0.0 255.254.0.0 0x0020 0x0 9192 0x02000001

// @@@ Adding dest=0xf0a0000, mask = 0xfffe0000, blade_nh_id=0xa000020,mtu_port=0x23e80001,flags=0x0

	set_sram(0x40040c68,0x970000);


	set_sram(0x40040c6c,0x970000);

// writing nexthop entry with flags 0x0

set_sram(0x111970,0x0);

// blade 0x2 and NH 0x20

set_sram(0x111974,0x2000020);

// MTU is 9192, outport is 0x1

set_sram(0x111978,0x23e80001);


set_sram(0x11197c,0x0);

// 
// 
// #-----------------------------------------------
// # 20-bit routes - Forward			 
// #-----------------------------------------------
// 
// add 20.0.16.0 255.255.240.0 0x0040 0x0 9192 0x2000002

// @@@ Adding dest=0x14001000, mask = 0xfffff000, blade_nh_id=0x10000040,mtu_port=0x23e80002,flags=0x0

	set_sram(0x40040c84,0x980000);

// writing nexthop entry with flags 0x0

set_sram(0x111980,0x0);

// blade 0x2 and NH 0x40

set_sram(0x111984,0x2000040);

// MTU is 9192, outport is 0x2

set_sram(0x111988,0x23e80002);


set_sram(0x11198c,0x0);


	set_sram(0x40005000,0x22);

// add 20.0.32.0 255.255.240.0 0x0060 0x0 9192 0x2000003

// @@@ Adding dest=0x14002000, mask = 0xfffff000, blade_nh_id=0x20000060,mtu_port=0x23e80003,flags=0x0

	set_sram(0x40040c88,0x990000);

// writing nexthop entry with flags 0x0

set_sram(0x111990,0x0);

// blade 0x2 and NH 0x60

set_sram(0x111994,0x2000060);

// MTU is 9192, outport is 0x3

set_sram(0x111998,0x23e80003);


set_sram(0x11199c,0x0);

// add 20.0.48.0 255.255.240.0 0x0021 0x0 9192 0x2000000

// @@@ Adding dest=0x14003000, mask = 0xfffff000, blade_nh_id=0x30000001,mtu_port=0x23e80000,flags=0x0

	set_sram(0x40040c8c,0x9a0000);

// writing nexthop entry with flags 0x0

set_sram(0x1119a0,0x0);

// blade 0x2 and NH 0x21

set_sram(0x1119a4,0x2000021);

// MTU is 9192, outport is 0x0

set_sram(0x1119a8,0x23e80000);


set_sram(0x1119ac,0x0);

// add 20.0.64.0 255.255.240.0 0x0022 0x0 9192 0x2000001

// @@@ Adding dest=0x14004000, mask = 0xfffff000, blade_nh_id=0x3f000020,mtu_port=0x23e80001,flags=0x0

	set_sram(0x40040c90,0x9b0000);

// writing nexthop entry with flags 0x0

set_sram(0x1119b0,0x0);

// blade 0x2 and NH 0x22

set_sram(0x1119b4,0x2000022);

// MTU is 9192, outport is 0x1

set_sram(0x1119b8,0x23e80001);


set_sram(0x1119bc,0x0);

// 
// 
// #-----------------------------------------------
// # 24-bit routes - Forward			 
// #-----------------------------------------------
// 
// add 24.0.1.0 255.255.255.0 0x0023 0x0 9192 0x02000002

// @@@ Adding dest=0x18000100, mask = 0xffffff00, blade_nh_id=0x40,mtu_port=0x23e80002,flags=0x0

	set_sram(0x40040d04,0x9c0000);

// writing nexthop entry with flags 0x0

set_sram(0x1119c0,0x0);

// blade 0x2 and NH 0x23

set_sram(0x1119c4,0x2000023);

// MTU is 9192, outport is 0x2

set_sram(0x1119c8,0x23e80002);


set_sram(0x1119cc,0x0);


	set_sram(0x40040cc0,0x24);


	set_sram(0x40006000,0x23);

// add 24.0.2.0 255.255.255.0 0x0024 0x0 9192 0x02000003

// @@@ Adding dest=0x18000200, mask = 0xffffff00, blade_nh_id=0x2000060,mtu_port=0x23e80003,flags=0x0

	set_sram(0x40040d08,0x9d0000);

// writing nexthop entry with flags 0x0

set_sram(0x1119d0,0x0);

// blade 0x2 and NH 0x24

set_sram(0x1119d4,0x2000024);

// MTU is 9192, outport is 0x3

set_sram(0x1119d8,0x23e80003);


set_sram(0x1119dc,0x0);

// add 24.0.3.0 255.255.255.0 0x0001 0x0 9192 0x02000000

// @@@ Adding dest=0x18000300, mask = 0xffffff00, blade_nh_id=0x3000001,mtu_port=0x23e80000,flags=0x0

	set_sram(0x40040d0c,0x9e0000);

// writing nexthop entry with flags 0x0

set_sram(0x1119e0,0x0);

// blade 0x2 and NH 0x01

set_sram(0x1119e4,0x2000001);

// MTU is 9192, outport is 0x0

set_sram(0x1119e8,0x23e80000);


set_sram(0x1119ec,0x0);

// 
// 
// # default route
// # send pkt with 0x1b000001
// # 0xdcba - next_hop id
// # 0x0 - flag (forward)
// # 9192 - MTU size
// # 0x03000003  - LS 24 bits 24 - output_port, 8 bits of blade_id
// # default 0xabcddcba 0x0 9192 0x03000003
// 
// # To have no-default route
// default 0xffff 0x0 0x0 0x0
// writing nexthop entry with flags 0x0

set_sram(0x111000,0x0);

// blade 0x2 and NH 0xffff

set_sram(0x111004,0x200ffff);

// MTU is 0, outport is 0x0

set_sram(0x111008,0x0);


set_sram(0x11100c,0x0);

// 
// 



return 1;


}


/******************************************CODE FROM IND FILE END****************************/

int
ATM_Ingress_init()
{
		//======================================================
		//	This is the top level file, which calls various
		//	functions from other scripts to setup the OC48 AAL5
		//	Ingress system.
		//	This should be the last script to be executed.
		//======================================================


		PRINT_STRING("\n**********************************************\n");
		PRINT_STRING("\n  OC48 AAL5 INGRESS System setup begin ...    \n");
		PRINT_STRING("\n**********************************************\n");

		//----------------------
		//	Init Scratch to 0
		//----------------------
			init_scratch(0, 0, 16384);

			init_sram(0, 0, 0x100000);


		//-----------------------------------------------------
		//	AAL5 RX - VC INFO Init (has RXC and VC STAT counters)
		//-----------------------------------------------------

		   PRINT_STRING("Initializing VC info table(RXC and Rx VC Stat counters) to zero ...\n");
		   aal5_rx_vc_info_init_zero();


		//-----------------------------------------------------
		//	AAL5 RX - Port STAT COUNTERS Init
		//-----------------------------------------------------

			PRINT_STRING("Initializing Rx Port Stat counters to zero ...\n");
			aal5_rx_port_stat_counters_init_zero();

		//-----------------------------------------------------
		//	AAL5 RX - HASH TABLE and RXC Init
		//-----------------------------------------------------

			PRINT_STRING("Initializing Pri & Sec hash tables to zero ...\n");
			aal5_rx_pri_init_zero();

			aal5_rx_sec_init_zero();

			// PRINT_STRING("Setting up RXC, Pri & Sec hash tables ...\n");
			// aal5_rx_hash_rxc_init();
			// Hash table init done by microcode

		//-----------------------------------------------------



		// ======= From OC_48 POS INGRESS system_setup =====

		//-----------------------------------------------------
		//	IPV4 Forwarder stuff
		//-----------------------------------------------------

		PRINT_STRING("Clearing route table ...\n");

		rtmInit();

		PRINT_STRING("Initializing with some routes ...\n");

		l3CreateRoutes();

		PRINT_STRING("Clearing counters in SRAM ...\n");

		ipv4ClearCounters();

		PRINT_STRING("Initializing the directed broadcast table ...\n");

		dbcast_init();

		//
		//	here add dbast entries using addresses from rtm_routes.ind
		//  file (take the destination IP address for which the 'LOCAL'
		//	flag is set, and then add corresponding network broadcast 
		//	address	for that network in the table).
		//

		dbcast_add(0x1affffff);

		dbcast_add(0x1a86ffff);

		dbcast_add(0x1a877fff);

		dbcast_add(0x1887ffff);

		dbcast_add(0x18870aff);

		dbcast_add(0x18870a00);

		// here, set up the control block to enable only 0-31 ports
		// for forwarding

		// mbp
		#define	CONTROL_BLOCK_SRAM_BASE	0x40202800
		set_sram(CONTROL_BLOCK_SRAM_BASE,0xffffffff);

		//-----------------------------------------------------
		//	QM Forwarder stuff
		//-----------------------------------------------------

		qmInit();

		//-----------------------------------------------------
		//	Done
		//------------------------------------------------------

		return 1;

}


/**********************************************************************************************/
/**********************************************************************************************/
/******************************************		ATM Engress Init   ****************************/
/**********************************************************************************************/
/**********************************************************************************************/

/******************************************CODE FROM IND FILE START****************************/
//-------------------------------------------------------------------
// The definitions here should match those in dl_system.h
//-------------------------------------------------------------------

// CSIX RX
#define			CSIX_RX_CTX_SRAM_BASE					0x300000
#define			CSIX_RX_CTX_TOTAL						1024
#define			CSIX_RX_COUNTER_BASE					0x308000

// LLC ENCAP
#define			NEXT_HOP_TABLE_BASE_SRAM				0x34C000
#define			NEXT_HOP_ENTRY_SIZE						32
#define			NUMBER_OF_NEXT_HOP_ENTRIES				512 //16384

// QM
#define			QD_SRAM_BASE							0x100000
#define 		QD_TOTAL								65536
#define			QD_SIZE									16

// TX
#define			AAL5_TX_ME01							5
#define			AAL5_TX_ME02							6
#define			AAL5_TX_ME03							7

#define 		NUMBER_OF_VC							512 //65536

#define			CELL_PADDING							0x6A6A6A6A

#define			AAL5_TX_CTX_SIZE						64
#define			AAL5_TX_CTX_TOTAL						16384
#define			AAL5_TX_OUTPORT_TOTAL					16384

#define			AAL5_TX_CTX_SRAM_BASE 					0x200000

#define			AAL5_TX_PKTS_PER_VCQ_COUNTER_BASE		0x30C000
#define			AAL5_TX_CELLS_PER_VCQ_COUNTER_BASE		0x31C000
#define			AAL5_TX_PKTS_PER_OUTPORT_COUNTER_BASE	0x32C000
#define			AAL5_TX_CELLS_PER_OUTPORT_COUNTER_BASE	0x33C000

#define			AAL5_TX_AVAIL_TBUF_ELEMENT_0			10240
#define			AAL5_TX_AVAIL_TBUF_ELEMENT_1			10244
#define			AAL5_TX_AVAIL_TBUF_ELEMENT_2			10248
#define			AAL5_TX_AVAIL_TBUF_ELEMENT_3			10252

#define			AAL5_TX_CELLS_TX_0						10256
#define			AAL5_TX_CELLS_TX_1						10260
#define			AAL5_TX_CELLS_TX_2						10264
#define			AAL5_TX_CELLS_TX_3						10268

/******************************************CODE FROM aal5_tx_init.ind ****************************/
int aal5_tx_init() {

	int sram_addr;
	int i;

	PRINT_STRING("Executing file: aal5_tx_init.ind\n");

	PRINT_STRING("Initializing outgoing cell header field in TXC of each vcq.\n");

	for(i=0; i<NUMBER_OF_VC; i++) {

		sram_addr = AAL5_TX_CTX_SRAM_BASE + i*AAL5_TX_CTX_SIZE;

		// For now outgoing cell header is same as the vcq#.
		set_sram(sram_addr+28, (i<<4));
	}

	//PRINT_STRING("For all TX ME's, initializing LM locations to the cell padding value.\n");
	
	//init_lmem(AAL5_TX_ME01, CELL_PADDING);
	//init_lmem(AAL5_TX_ME02, CELL_PADDING);
	//init_lmem(AAL5_TX_ME03, CELL_PADDING);

	PRINT_STRING("Initializing SRAM per VCQ counters to 0.\n");

	sram_addr = AAL5_TX_PKTS_PER_VCQ_COUNTER_BASE + ((AAL5_TX_CTX_TOTAL * 4) - 4);
	init_sram(0, AAL5_TX_PKTS_PER_VCQ_COUNTER_BASE, sram_addr);

	sram_addr = AAL5_TX_CELLS_PER_VCQ_COUNTER_BASE + ((AAL5_TX_CTX_TOTAL * 4) - 4);
	init_sram(0, AAL5_TX_CELLS_PER_VCQ_COUNTER_BASE, sram_addr);

	PRINT_STRING("Initializing SRAM per output port counters to 0.\n");

	sram_addr = AAL5_TX_PKTS_PER_OUTPORT_COUNTER_BASE + ((AAL5_TX_OUTPORT_TOTAL * 4) - 4);
	init_sram(0, AAL5_TX_PKTS_PER_OUTPORT_COUNTER_BASE, sram_addr);

	sram_addr = AAL5_TX_CELLS_PER_OUTPORT_COUNTER_BASE + ((AAL5_TX_OUTPORT_TOTAL * 4) - 4);
	init_sram(0, AAL5_TX_CELLS_PER_OUTPORT_COUNTER_BASE, sram_addr);

	PRINT_STRING("Initializing scratch locations that contains next available TBUF element to 0.\n");

	set_scratch(AAL5_TX_AVAIL_TBUF_ELEMENT_0, 0);
	set_scratch(AAL5_TX_AVAIL_TBUF_ELEMENT_1, 0);
	set_scratch(AAL5_TX_AVAIL_TBUF_ELEMENT_2, 0);
	set_scratch(AAL5_TX_AVAIL_TBUF_ELEMENT_3, 0);

	PRINT_STRING("Initializing scratch locations that contains cells tx info to 0.\n");

    set_scratch(AAL5_TX_CELLS_TX_0, 0x0);
    set_scratch(AAL5_TX_CELLS_TX_1, 0x0);
    set_scratch(AAL5_TX_CELLS_TX_2, 0x0);
    set_scratch(AAL5_TX_CELLS_TX_3, 0x0);

	PRINT_STRING("Finished executing file: aal5_tx_init.ind\n");

	return 1;
}


/******************************************CODE FROM csix_rx_init.ind ****************************/

int csix_rx_init() {
	
	int finaladdr;

	PRINT_STRING("Executing file: csix_rx_init.ind\n");


	PRINT_STRING("Initializing CRX SRAM memory..\n");

	finaladdr = CSIX_RX_CTX_SRAM_BASE + CSIX_RX_CTX_TOTAL*32 ;

	init_sram(0x0,CSIX_RX_CTX_SRAM_BASE,finaladdr);


	PRINT_STRING("Initializing COUNTER SRAM memory..\n");

	finaladdr = CSIX_RX_COUNTER_BASE + CSIX_RX_CTX_TOTAL*16 ;

	init_sram(0x0,CSIX_RX_COUNTER_BASE,finaladdr);


	PRINT_STRING("Finished executing file: csix_rx_init.ind\n");

	return 1;
}

/******************************************CODE FROM llc_encap.ind ****************************/

int llc_encap_init() {

	int i;
	int next_hop_addr;

	PRINT_STRING("Executing file: llc_encap_init.ind\n");


	PRINT_STRING("Initializing next hop table in SRAM..\n");

	for(i=0; i<NUMBER_OF_NEXT_HOP_ENTRIES; i++) {

		next_hop_addr = NEXT_HOP_TABLE_BASE_SRAM + i*NEXT_HOP_ENTRY_SIZE;

		// Not used.
		//set_sram(next_hop_addr, i);

		// Set vcq#.
//		set_sram(next_hop_addr+4, 0xFFFFFFFF);
		set_sram(next_hop_addr+4, i);

		// Set llc/snap header.
//		set_sram(next_hop_addr+8, 0xFFFFFFFF);
		set_sram(next_hop_addr+8, 0xAAAA0300);
		set_sram(next_hop_addr+12, 0x00000800);
	}

	PRINT_STRING("Finished executing file: llc_encap_init.ind\n");

	return 1;
}

/******************************************CODE FROM qm_cell_init.ind ****************************/
int qm_cell_init() {
	
	int finaladdr;
	int baseaddr;

	PRINT_STRING("Executing file: qm_init.ind\n");


	PRINT_STRING("Initializing QM SRAM memory..\n");

	baseaddr = QD_SRAM_BASE;
	finaladdr = baseaddr + QD_TOTAL*16 ;

	init_sram(0x0,baseaddr,finaladdr);


	PRINT_STRING("Finished executing file: qm_init.ind\n");

	return 1;
}

/******************************************CODE FROM IND FILE END****************************/

int
ATM_Egress_init()
{
	PRINT_STRING("\n******************************************************\n");
	PRINT_STRING("\n     OC48 egress ATM system setup begin ...           \n");
	PRINT_STRING("\n******************************************************\n");
	
	//----------------------
	//	Init Scratch to 0
	//----------------------
	init_scratch(0, 0, 16384);

	init_sram(0, 0, 0x100000);


	//----------------------
	//	CSIX RX
	//----------------------

	csix_rx_init();

	//----------------------
	//	LLC ENCAP
	//----------------------

	llc_encap_init();

	//----------------------
	//	QM
	//----------------------

	qm_cell_init();

	//----------------------
	//	ATM AAL5 TX
	//----------------------

	aal5_tx_init();

	//----------------------
	//	COMMON
	//----------------------
	
	return 1;
}


/*****************************************************
             SOME UTILITIES
*****************************************************/
/* Macro to write and read one longword from memory 
   the macro assumes that the memory being read or written to
   is mapped 
 */

#define GET32(address)      (*(volatile unsigned int *)(address))
#define PUT32(address,val)  (*(volatile unsigned int *)(address) = (val))

/* Base address for SRAM and DRAM memory */

#define SRAM_CH0_BEGIN      0x80000000
#define SRAM_CH1_BEGIN      0x90000000
#define SDRAM_BEGIN         0x2000000
#define SCRATCH_BEGIN       0xca000000

#define SRAM_CH0_BASE	SRAM_CH0_BEGIN 
#define SRAM_CH1_BASE	SRAM_CH1_BEGIN
#define SDRAM_BASE	SDRAM_BEGIN
#define SCRATCH_BASE    SCRATCH_BEGIN

#define IPV4_COUNTERS_SCRATCH

#ifdef IPV4_COUNTERS_SCRATCH
	#define _SCRATCH_COUNTER_BASE	11264
	#define IPV4_COUNTERS_BASE	(_SCRATCH_COUNTER_BASE + SCRATCH_BASE)
#else
	#define  IPV4_COUNTERS_BASE           0x90200000
#endif

#define BUFFER_SIZE                     2048
#define META_SIZE                       32
#define BUF_SDRAM_BASE                  SDRAM_BEGIN

/* offsets of counters in big_endian format.*/
/*/	Packets Rxed */
#define	OFFSET_PKTS_RX			0
/*/	Packets Fwd */	
#define	OFFSET_PKTS_FWD			4
/*/	Packets dropped */
#define	OFFSET_PKTS_DROP		8
/*	Packets excp */
#define	OFFSET_PKTS_EXCP		12
/*/	pkts bad cksum */
#define	OFFSET_PKTS_BAD_HDR		16
/*/	pkts bad len */
#define	OFFSET_PKTS_BAD_LEN		20
/*/	pkts with bad TTL */
#define	OFFSET_PKTS_BAD_TTL		24
/*/ packets with no route */
#define	OFFSET_PKTS_NO_ROUTE    	28
/*/ packets too small */
#define	OFFSET_PKTS_TOO_SMALL   	32

#define STATS_PER_PORT_SHF_VAL		6

/**
 * NAME: read_ipv4_counters
 *
 * DESCRIPTION: This function reads IPv4 counters 
 */
int read_ipv4_counters(unsigned int port)
{	
	unsigned int ipv4_base = IPV4_COUNTERS_BASE + (port<<STATS_PER_PORT_SHF_VAL);

	printf("NUM_PKTS_RX       =%d \n",( unsigned int) *(( unsigned int *) (ipv4_base + OFFSET_PKTS_RX      ) ) ) ;
	printf("NUM_PKTS_FWD      =%d \n",( unsigned int)*(( unsigned int *) (ipv4_base + OFFSET_PKTS_FWD     ) ) ) ;
	printf("NUM_PKTS_DRO      =%d \n",( unsigned int)*(( unsigned int *) (ipv4_base + OFFSET_PKTS_DROP    ) ) ) ;
	printf("NUM_PKTS_EXCEP    =%d \n",( unsigned int)*(( unsigned int *) (ipv4_base + OFFSET_PKTS_EXCP    ) ) ) ;
	printf("NUM_PKTS_BAD_HDR  =%d \n",( unsigned int)*(( unsigned int *) (ipv4_base + OFFSET_PKTS_BAD_HDR ) ) ) ;
	printf("NUM_PKTS_BAD_LE   =%d \n",( unsigned int)*(( unsigned int *) (ipv4_base + OFFSET_PKTS_BAD_LEN ) ) ) ;
	printf("NUM_PKTS_BAD_TTL  =%d \n",( unsigned int)*(( unsigned int *) (ipv4_base + OFFSET_PKTS_BAD_TTL ) ) ) ;
	printf("NUM_PKTS_NO_ROUTE =%d \n",( unsigned int)*(( unsigned int *) (ipv4_base + OFFSET_PKTS_NO_ROUTE) ) ) ;
	printf("NUM_PKTS_TOO_SMALL=%d \n",( unsigned int)*(( unsigned int *) (ipv4_base + OFFSET_PKTS_TOO_SMALL) ) ) ; 
	
	return 0;
}

/**
 * NAME: read_meta_data
 *
 * DESCRIPTION: This function reads the meta data. The input parameter is the buffer handle or SRAM address in LW.
 */
int read_meta_data( unsigned int *bufhandle )
{

	if( bufhandle == NULL )
	{
		printf("Enter buffer handle \n") ;
		return 1 ;
	}	

	bufhandle = (unsigned int * )( (( (unsigned int )bufhandle) & ((unsigned int ) 0x00ffffff)) << 2 );
	bufhandle = (unsigned int * ) (  (unsigned int )bufhandle |  (unsigned int ) SRAM_CH0_BASE ) ;

	printf("The SRAM Address for meta data is = 0x%x\n", (unsigned int) bufhandle) ;
	
	printf("Buffer_NEXT        =0x%x\n",  *(bufhandle +0)                     ) ;

	printf("Buffer_SIZE        =%d\n",   (*(bufhandle+1) >> 16)               ) ;
	printf("Buffer_Data_Offset =0x%x\n", (*(bufhandle+1) & 0x0000ffff)        ) ;

	printf("Buffer_PKT_SIZE    =%d\n",   (*(bufhandle+2) >> 16 )              ) ;
	printf("Buffer_FREE_LIST_ID=%d\n",   (*(bufhandle+2) & 0x0000f000) >> 12  ) ;
	printf("Buffer_RX_STAT     =%d\n",   (*(bufhandle+2) &0x00000f00) >> 8    ) ;
	printf("Buffer_HEADER_TYPE =%d\n",   (*(bufhandle+2) & 0x000000ff)        ) ;

	printf("Buffer_INPUT_PORT  =%d\n",   (*(bufhandle+3) >> 16)               ) ;
	printf("Buffer_OUTPUT_PORT =%d\n",   (*(bufhandle+3) & 0x0000ffff)        ) ;
 
	printf("Buffer_NEXTHOP_ID  =%d\n",   (*(bufhandle+4) >> 16)               ) ;
	printf("Buffer_FABRIC_PORT =%d\n",   (*(bufhandle+4) & 0x0000ff00) >> 8   ) ;

	printf("Buffer_FLOW_ID     =%d\n",   *(bufhandle +5 )                     ) ;

	printf("Buffer_CLASS_ID    =%d\n",   *(bufhandle+6) >> 16                 ) ;

	printf("Buffer_PKT_NEXT    =0x%x\n", *(bufhandle+7)                       ) ;

	return 0 ;

}

/**
 * NAME: read_pkt_data
 *
 * DESCRIPTION: This function reads the packet data. The input parameter is the buffer handle or SRAM address in LW.
 *              If the buffer size or meta data size or dram_base change just change the definitions above. 
 */
int read_pkt_data( unsigned int *bufhandle )
{

	unsigned int * temp ;
        unsigned int ratio  ;
        unsigned int * dataptr ;
	unsigned int offset = 0 ;
	int i ;

	if( bufhandle == NULL )
	{
		printf("Enter buffer handle \n") ;
		return 1 ;
	}

	
        /* calculate the data offset from meta data */

	temp = (unsigned int * ) ((( (unsigned int )bufhandle) & ((unsigned int ) 0x00ffffff)) << 2 );
	temp = (unsigned int * ) (  (unsigned int )temp |  (unsigned int ) SRAM_CH0_BASE ) ;
	offset = (*(temp+1) & 0x0000ffff) ;

	printf("Offset at given dram address is 0x%x \n", offset) ;

        /* calculate the ratio. It is 0x100 in the default configuration */
        ratio = (BUFFER_SIZE/META_SIZE) * 4 ;

        /* calculate the data pointer in DRAM */
        dataptr = bufhandle ;
	dataptr = (unsigned int * ) (( (unsigned int )dataptr) & ((unsigned int ) 0x00ffffff)) ;
	dataptr = (unsigned int * ) ((unsigned int ) dataptr *  (unsigned int ) ratio) ;
	dataptr = (unsigned int * ) ((unsigned int )BUF_SDRAM_BASE + (unsigned int )dataptr + offset) ;

	printf("Pkt data address in DRAM is 0x%x \n", (unsigned int ) bufhandle ) ;

	printf("Printing the first 64  Bytes......\n");
	for(i=0; i<16; i++ )
	{
		printf("0x%x\n", *(dataptr) );
		bufhandle += 1 ;
	} 

	return 0 ;

	
}

/**
 * NAME: read_pkt
 *
 * DESCRIPTION: This function reads the meta data and also prints the first 64bytes of the packet.
 *              The input parameter is the buffer handle or SRAM address in LW.
 */
int read_pkt( unsigned int *bufhandle)
{
	read_meta_data(bufhandle) ;
	read_pkt_data(bufhandle) ;

	return 0 ;

}

/**
 * NAME: read_scratch_reg
 *
 * DESCRIPTION: This function reads the scratch ring CSR register given the ring number
 */
int read_scratch_reg(unsigned int *ring_number)
{
   unsigned int *scratch_base, *ring_address ;

   if( (unsigned int ) ring_number < 0x10 )
   {
	scratch_base = (unsigned int *) 0xc0004800 ;

	ring_address = (unsigned int *) ((unsigned int ) scratch_base + (( unsigned int ) ring_number * 16) );

	printf(" Scratch Ring %d.....\n", (unsigned int ) ring_number) ;
	printf("    Base Address...0x%x\n", *(ring_address) )  ;
	ring_address += 1 ;
	printf("    Head Pointer...0x%x\n", *(ring_address) ) ;
	ring_address += 1 ;
	printf("    Tail Pointer...0x%x\n", *(ring_address) ) ;


   }


   return 0 ; 
	
}

/**
 * NAME: read_rbuf
 *
 * DESCRIPTION: This function reads rbuf given the element size and the element number
 *              By default it will read element 0
 */
int read_rbuf( unsigned int *element_num, unsigned int rbuf_size)
{

	int i ;

	unsigned int *rbuf_base, *rbuf_add ;

	if( rbuf_size  <= 64  ) 
	{
		printf("Assuming rbuf_size to be 128\n") ;
		rbuf_size = 128 ;
	}

	rbuf_base = (unsigned int *) 0xc8002000 ;

	rbuf_add = ( unsigned int * ) ( ( unsigned int ) rbuf_base + ( ((unsigned int ) element_num ) * rbuf_size));

	printf("Printing the Rbuf data at address 0x%x......\n", (unsigned int) rbuf_add);

	for(i=0; i<32; i++ )
	{
		printf("0x%x\n", *(rbuf_add) );
		rbuf_add += 1 ;
	} 

	return 0 ;
}

/**
 * NAME: read_scratch_full_reg
 *
 * DESCRIPTION: This function reads the scratch full CSR register
 */
int read_scratch_full_reg()
{ 
	int i;
	
	i = GET32(0xd6000058) ;

	printf("Scratch ring full register value is = 0x%x\n", i) ;

	return 0 ;

}

/**
 * NAME: clear_ipv4_counters
 *
 * DESCRIPTION: This function clears the ipv4 counters
 */
int clear_ipv4_counters(unsigned int port)
{
	unsigned int ipv4_base = IPV4_COUNTERS_BASE + (port<<STATS_PER_PORT_SHF_VAL);

        *(( unsigned int *) (ipv4_base + OFFSET_PKTS_RX      ) )  = 0 ;
        *(( unsigned int *) (ipv4_base + OFFSET_PKTS_FWD     ) )  = 0 ;
        *(( unsigned int *) (ipv4_base + OFFSET_PKTS_DROP    ) )  = 0 ;
        *(( unsigned int *) (ipv4_base + OFFSET_PKTS_EXCP    ) )  = 0 ;
        *(( unsigned int *) (ipv4_base + OFFSET_PKTS_BAD_HDR ) )  = 0 ;
        *(( unsigned int *) (ipv4_base + OFFSET_PKTS_BAD_LEN ) )  = 0 ;
        *(( unsigned int *) (ipv4_base + OFFSET_PKTS_BAD_TTL ) )  = 0 ;
        *(( unsigned int *) (ipv4_base + OFFSET_PKTS_NO_ROUTE) )  = 0 ;
        *(( unsigned int *) (ipv4_base + OFFSET_PKTS_TOO_SMALL) ) = 0  ;

        printf("IPV4 Counters Reset \n") ;
 

        return 0 ;

}

/**
 * NAME: read_all_utils
 *
 * DESCRIPTION: This function prints all the utilities available
 */
int read_all_utils()
{
	printf("1. read_ipv4_counters(port_number)...........\n");
	printf("2. read_meta_data(buf_handle).....\n");
	printf("3. read_pkt_data(buf_handle)......\n");
	printf("4. read_pkt(buf_handle).....................\n");
	printf("5. read_rbuf(element_num, element_size)..default element size is 128....\n");
	printf("6. read_scratch_reg()... \n");
	printf("7. read_scratch_full_reg()..................\n") ;
	printf("8. clear_ipv4_counters(port_number)....................\n");

	return 0 ;
}

