/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2004 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

#define XSCALE_SRAM_BASE 	0x80000000
#define XSCALE_DRAM_BASE 	0x02000000
#define XSCALE_SCRATCH_BASE	0xca000000
	
int get_sram_address(int address) {
  return XSCALE_SRAM_BASE + (address & 0x3FFFFFFF) + 0x10000000 * (address >> 30);
}

int get_dram_address(int address) {
  return XSCALE_DRAM_BASE + address;
}

int get_scratch_address(int address) {
  return XSCALE_SCRATCH_BASE + address;
}

int set_sram(int address, int value)  {
   *((volatile unsigned int*) get_sram_address(address)) = value;
   return 1;
}

int set_dram(int address, int value)  {
   *((volatile unsigned int*) get_dram_address(address)) = value;
   return 1;
}

int set_scratch(int address, int value)  {
   *((volatile unsigned int*) get_scratch_address(address)) = value;
   return 1;
}

int get_sram(int address)  {
   return    *((volatile unsigned int*) get_sram_address(address));
}

int get_dram(int address)  {
   return    *((volatile unsigned int*) get_dram_address(address));
}

int get_scratch(int address)  {
   return    *((volatile unsigned int*) get_scratch_address(address));
}

int init_sram(int value, int add_begin, int add_end) {
   int address = add_begin;
   printf("init_sram(): value = %d,  start address = %d, stop address = %d \n",
          value, (int) get_sram_address(add_begin), (int) get_sram_address(add_end));
   for (;address <= add_end; address = address + 4)
	set_sram(address, value);
   return 1;
};


int dram_search(int start, int end) {
   int address = start;
   for (;address <= end; address = address + 4) {
	if (get_dram(address) == 0xf0f0) {
	  printf("Address found :%d \n",address);
	  return 1;
	};
        if ((address % 512) == 0)  printf("Looking at :%d \n",address);
   }
   return 0;
}

int init_scratch(int value, int add_begin, int add_end) {
   int address = add_begin;
   printf("init_scratch(): value = %d,  start address = %d, stop address = %d \n",
          value, (int) get_scratch_address(add_begin), (int) get_scratch_address(add_end));
   for (;address <= add_end; address = address + 4)
	set_scratch(address, value);
   return 1;
};
