/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      media.c
 *
 * = DESCRIPTION
 *     This file has routines to initialize the 6048 media
 *     card to receive and transmit POS packets
 *
 * = AUTHOR
 *      Suman Sharma
 *
 * = CHANGE HISTORY
 *      12/01/2002  - creation time 
 * ============================================================================
 */

#define PUT32(address,val)  (*(volatile unsigned int *)(address) = (val))
#define GET32(address)	    (*(volatile unsigned int *)(address))

typedef struct initRegTable
{
	unsigned short* offset;
	unsigned short value;
} InitRegTable_t;

typedef unsigned short * bb_RegPointer_type;

const InitRegTable_t RegInitIXF6048Table[] =
{
/* Note: Channel applies depending of the Chip Block as:
   Channel could represent the DataLine
   Channel could represent the group
   Channel could represent the Tributary
   Channel could represent the Line */

    /* IXF6048 Global Cnfg Register Block */
    { (bb_RegPointer_type) 0x0001,    0x0000 },
    { (bb_RegPointer_type) 0x0003,    0x0000 },
    { (bb_RegPointer_type) 0x0004,    0x0000 },
    { (bb_RegPointer_type) 0x0008,    0x2078 },
    { (bb_RegPointer_type) 0x000C,    0x0000 },
    { (bb_RegPointer_type) 0x000D,    0x0000 },
    { (bb_RegPointer_type) 0x000E,    0x0000 },
    { (bb_RegPointer_type) 0x000F,    0xF001 },
    { (bb_RegPointer_type) 0x0010,    0x0000 },
    { (bb_RegPointer_type) 0x0011,    0x0000 },
    { (bb_RegPointer_type) 0x0014,    0x0010 },
    { (bb_RegPointer_type) 0x0015,    0x0000 },
    { (bb_RegPointer_type) 0x001E,    0x0000 },
    { (bb_RegPointer_type) 0x001B,    0x0000 },
    { (bb_RegPointer_type) 0x001D,    0x0000 },
    { (bb_RegPointer_type) 0x0020,    0x0000 },
    /* IXF6048 Channel Confg Register Block Channel = 0 */
    { (bb_RegPointer_type) 0x0009,    0x0006 },
    { (bb_RegPointer_type) 0x000A,    0x00EA },
    { (bb_RegPointer_type) 0x000B,    0x050A },
    /* IXF6048 Channel Confg Register Block Channel = 1 */
    { (bb_RegPointer_type) 0x0109,    0x0006 },
    { (bb_RegPointer_type) 0x010A,    0x00EA },
    { (bb_RegPointer_type) 0x010B,    0x050A },
    /* IXF6048 Channel Confg Register Block Channel = 2 */
    { (bb_RegPointer_type) 0x0209,    0x0006 },
    { (bb_RegPointer_type) 0x020A,    0x00EA },
    { (bb_RegPointer_type) 0x020B,    0x050A },
    /* IXF6048 Channel Confg Register Block Channel = 3 */
    { (bb_RegPointer_type) 0x0309,    0x0006 },
    { (bb_RegPointer_type) 0x030A,    0x00EA },
    { (bb_RegPointer_type) 0x030B,    0x050A },
    /* IXF6048 Glb Utopia Register Block */
    { (bb_RegPointer_type) 0x0070,    0x0328 },
    { (bb_RegPointer_type) 0x0071,    0x0002 },
    { (bb_RegPointer_type) 0x0050,    0x00A9 },
    { (bb_RegPointer_type) 0x0051,    0x0002 },
    { (bb_RegPointer_type) 0x0073,    0x0000 },
    { (bb_RegPointer_type) 0x0053,    0x0000 },
    /* IXF6048 Channel Utopia Register Block Channel = 0 */
    { (bb_RegPointer_type) 0x0060,    0x0119 },
    { (bb_RegPointer_type) 0x0061,    0x000F },
    { (bb_RegPointer_type) 0x0040,    0x0119 },
    { (bb_RegPointer_type) 0x0041,    0x001F },
    { (bb_RegPointer_type) 0x0042,    0x001F },
    { (bb_RegPointer_type) 0x0043,    0x00FF },
    /* IXF6048 Channel Utopia Register Block Channel = 1 */
    { (bb_RegPointer_type) 0x0160,    0x0119 },
    { (bb_RegPointer_type) 0x0161,    0x000F },
    { (bb_RegPointer_type) 0x0140,    0x0119 },
    { (bb_RegPointer_type) 0x0141,    0x001F },
    { (bb_RegPointer_type) 0x0142,    0x001F },
    { (bb_RegPointer_type) 0x0143,    0x00FF },
    /* IXF6048 Channel Utopia Register Block Channel = 2 */
    { (bb_RegPointer_type) 0x0260,    0x0119 },
    { (bb_RegPointer_type) 0x0261,    0x000F },
    { (bb_RegPointer_type) 0x0240,    0x0119 },
    { (bb_RegPointer_type) 0x0241,    0x001F },
    { (bb_RegPointer_type) 0x0242,    0x001F },
    { (bb_RegPointer_type) 0x0243,    0x00FF },
    /* IXF6048 Channel Utopia Register Block Channel = 3 */
    { (bb_RegPointer_type) 0x0360,    0x0119 },
    { (bb_RegPointer_type) 0x0361,    0x000F },
    { (bb_RegPointer_type) 0x0340,    0x0119 },
    { (bb_RegPointer_type) 0x0341,    0x001F },
    { (bb_RegPointer_type) 0x0342,    0x001F },
    { (bb_RegPointer_type) 0x0343,    0x00FF },
    /* IXF6048 RST Register Block Channel = 0 */
    { (bb_RegPointer_type) 0x0480,    0x8002 },
    { (bb_RegPointer_type) 0x0481,    0x0000 },
    { (bb_RegPointer_type) 0x0487,    0x0002 },
    { (bb_RegPointer_type) 0x04D0,    0x0000 },
    { (bb_RegPointer_type) 0x04D4,    0x000E },
    { (bb_RegPointer_type) 0x04D8,    0x0000 },
    /* IXF6048 RST Register Block Channel = 1 */
    { (bb_RegPointer_type) 0x0580,    0x8002 },
    { (bb_RegPointer_type) 0x0581,    0x0000 },
    { (bb_RegPointer_type) 0x0587,    0x0002 },
    { (bb_RegPointer_type) 0x05D0,    0x0000 },
    { (bb_RegPointer_type) 0x05D4,    0x000E },
    { (bb_RegPointer_type) 0x05D8,    0x0000 },
    /* IXF6048 RST Register Block Channel = 2 */
    { (bb_RegPointer_type) 0x0680,    0x8002 },
    { (bb_RegPointer_type) 0x0681,    0x0000 },
    { (bb_RegPointer_type) 0x0687,    0x0002 },
    { (bb_RegPointer_type) 0x06D0,    0x0000 },
    { (bb_RegPointer_type) 0x06D4,    0x0000 },
    { (bb_RegPointer_type) 0x06D8,    0x0000 },
    /* IXF6048 RST Register Block Channel = 3 */
    { (bb_RegPointer_type) 0x0780,    0x8002 },
    { (bb_RegPointer_type) 0x0781,    0x0000 },
    { (bb_RegPointer_type) 0x0787,    0x0002 },
    { (bb_RegPointer_type) 0x07D0,    0x0000 },
    { (bb_RegPointer_type) 0x07D4,    0x0000 },
    { (bb_RegPointer_type) 0x07D8,    0x0000 },
    /* IXF6048 MST Register Block Channel = 0 */
    { (bb_RegPointer_type) 0x0490,    0x0510 },
    { (bb_RegPointer_type) 0x04B0,    0x0000 },
    { (bb_RegPointer_type) 0x04B1,    0x0000 },
    { (bb_RegPointer_type) 0x04B2,    0x0000 },
    { (bb_RegPointer_type) 0x04B3,    0x0000 },
    { (bb_RegPointer_type) 0x04B4,    0x0000 },
    { (bb_RegPointer_type) 0x04B5,    0x0000 },
    { (bb_RegPointer_type) 0x04B8,    0x0000 },
    { (bb_RegPointer_type) 0x04B9,    0x0000 },
    { (bb_RegPointer_type) 0x04BC,    0x0000 },
    { (bb_RegPointer_type) 0x04BD,    0x0000 },
    { (bb_RegPointer_type) 0x04D1,    0x0000 },
    { (bb_RegPointer_type) 0x04D5,    0x3000 },
    { (bb_RegPointer_type) 0x04D9,    0x0000 },
    /* IXF6048 MST Register Block Channel = 1 */
    { (bb_RegPointer_type) 0x0590,    0x0510 },
    { (bb_RegPointer_type) 0x05B0,    0x0000 },
    { (bb_RegPointer_type) 0x05B1,    0x0000 },
    { (bb_RegPointer_type) 0x05B2,    0x0000 },
    { (bb_RegPointer_type) 0x05B3,    0x0000 },
    { (bb_RegPointer_type) 0x05B4,    0x0000 },
    { (bb_RegPointer_type) 0x05B5,    0x0000 },
    { (bb_RegPointer_type) 0x05B8,    0x0000 },
    { (bb_RegPointer_type) 0x05B9,    0x0000 },
    { (bb_RegPointer_type) 0x05BC,    0x0000 },
    { (bb_RegPointer_type) 0x05BD,    0x0000 },
    { (bb_RegPointer_type) 0x05D1,    0x0000 },
    { (bb_RegPointer_type) 0x05D5,    0x7FFF },
    { (bb_RegPointer_type) 0x05D9,    0x0000 },
    /* IXF6048 MST Register Block Channel = 2 */
    { (bb_RegPointer_type) 0x0690,    0x0510 },
    { (bb_RegPointer_type) 0x06B0,    0x0000 },
    { (bb_RegPointer_type) 0x06B1,    0x0000 },
    { (bb_RegPointer_type) 0x06B2,    0x0000 },
    { (bb_RegPointer_type) 0x06B3,    0x0000 },
    { (bb_RegPointer_type) 0x06B4,    0x0000 },
    { (bb_RegPointer_type) 0x06B5,    0x0000 },
    { (bb_RegPointer_type) 0x06B8,    0x0000 },
    { (bb_RegPointer_type) 0x06B9,    0x0000 },
    { (bb_RegPointer_type) 0x06BC,    0x0000 },
    { (bb_RegPointer_type) 0x06BD,    0x0000 },
    { (bb_RegPointer_type) 0x06D1,    0x0000 },
    { (bb_RegPointer_type) 0x06D5,    0x0000 },
    { (bb_RegPointer_type) 0x06D9,    0x0000 },
    /* IXF6048 MST Register Block Channel = 3 */
    { (bb_RegPointer_type) 0x0790,    0x0510 },
    { (bb_RegPointer_type) 0x07B0,    0x0000 },
    { (bb_RegPointer_type) 0x07B1,    0x0000 },
    { (bb_RegPointer_type) 0x07B2,    0x0000 },
    { (bb_RegPointer_type) 0x07B3,    0x0000 },
    { (bb_RegPointer_type) 0x07B4,    0x0000 },
    { (bb_RegPointer_type) 0x07B5,    0x0000 },
    { (bb_RegPointer_type) 0x07B8,    0x0000 },
    { (bb_RegPointer_type) 0x07B9,    0x0000 },
    { (bb_RegPointer_type) 0x07BC,    0x0000 },
    { (bb_RegPointer_type) 0x07BD,    0x0000 },
    { (bb_RegPointer_type) 0x07D1,    0x0000 },
    { (bb_RegPointer_type) 0x07D5,    0x0000 },
    { (bb_RegPointer_type) 0x07D9,    0x0000 },
    /* IXF6048 RX MSA Register Block Channel = 0 */
    { (bb_RegPointer_type) 0x04A0,    0x0020 },
    { (bb_RegPointer_type) 0x04D2,    0x0000 },
    { (bb_RegPointer_type) 0x04D6,    0x0004 },
    { (bb_RegPointer_type) 0x04DA,    0x0000 },
    /* IXF6048 RX MSA Register Block Channel = 1 */
    { (bb_RegPointer_type) 0x05A0,    0x0020 },
    { (bb_RegPointer_type) 0x05D2,    0x0000 },
    { (bb_RegPointer_type) 0x05D6,    0x0004 },
    { (bb_RegPointer_type) 0x05DA,    0x0000 },
    /* IXF6048 RX MSA Register Block Channel = 2 */
    { (bb_RegPointer_type) 0x06A0,    0x0020 },
    { (bb_RegPointer_type) 0x06D2,    0x0000 },
    { (bb_RegPointer_type) 0x06D6,    0x0000 },
    { (bb_RegPointer_type) 0x06DA,    0x0000 },
    /* IXF6048 RX MSA Register Block Channel = 3 */
    { (bb_RegPointer_type) 0x07A0,    0x0020 },
    { (bb_RegPointer_type) 0x07D2,    0x0000 },
    { (bb_RegPointer_type) 0x07D6,    0x0000 },
    { (bb_RegPointer_type) 0x07DA,    0x0000 },
    /* IXF6048 RX HPM Register Block Channel = 0 */
    { (bb_RegPointer_type) 0x04A4,    0x219C },
    { (bb_RegPointer_type) 0x04A5,    0x0000 },
    { (bb_RegPointer_type) 0x04D7,    0x0000 },
    /* IXF6048 RX HPM Register Block Channel = 1 */
    { (bb_RegPointer_type) 0x05A4,    0x219C },
    { (bb_RegPointer_type) 0x05A5,    0x0000 },
    { (bb_RegPointer_type) 0x05D7,    0x0000 },
    /* IXF6048 RX HPM Register Block Channel = 2 */
    { (bb_RegPointer_type) 0x06A4,    0x21FC },
    { (bb_RegPointer_type) 0x06A5,    0x0000 },
    { (bb_RegPointer_type) 0x06D7,    0x0000 },
    /* IXF6048 RX HPM Register Block Channel = 3 */
    { (bb_RegPointer_type) 0x07A4,    0x21FC },
    { (bb_RegPointer_type) 0x07A5,    0x0000 },
    { (bb_RegPointer_type) 0x07D7,    0x0000 },
    /* IXF6048 RMST Register Block Channel = 0 */
    { (bb_RegPointer_type) 0x04E0,    0x8000 },
    { (bb_RegPointer_type) 0x04E1,    0x0124 },
    { (bb_RegPointer_type) 0x04E2,    0x880A },
    /* IXF6048 RMST Register Block Channel = 1 */
    { (bb_RegPointer_type) 0x05E0,    0x8000 },
    { (bb_RegPointer_type) 0x05E1,    0x0124 },
    { (bb_RegPointer_type) 0x05E2,    0x880A },
    /* IXF6048 RMST Register Block Channel = 2 */
    { (bb_RegPointer_type) 0x06E0,    0x8000 },
    { (bb_RegPointer_type) 0x06E1,    0x0124 },
    { (bb_RegPointer_type) 0x06E2,    0x800A },
    /* IXF6048 RMST Register Block Channel = 3 */
    { (bb_RegPointer_type) 0x07E0,    0x8000 },
    { (bb_RegPointer_type) 0x07E1,    0x0124 },
    { (bb_RegPointer_type) 0x07E2,    0x800A },
    /* IXF6048 TX HPM Register Block Channel = 0 */
    { (bb_RegPointer_type) 0x04E8,    0x4142 },
    { (bb_RegPointer_type) 0x04EF,    0x06C0 },
    /* IXF6048 TX HPM Register Block Channel = 1 */
    { (bb_RegPointer_type) 0x05E8,    0x4142 },
    { (bb_RegPointer_type) 0x05EF,    0x06C0 },
    /* IXF6048 TX HPM Register Block Channel = 2 */
    { (bb_RegPointer_type) 0x06E8,    0x4142 },
    { (bb_RegPointer_type) 0x06EF,    0x06C0 },
    /* IXF6048 TX HPM Register Block Channel = 3 */
    { (bb_RegPointer_type) 0x07E8,    0x4142 },
    { (bb_RegPointer_type) 0x07EF,    0x06C0 },
    /* IXF6048 RX ATM Register Block Channel = 0 */
    { (bb_RegPointer_type) 0x0420,    0x0183 },
    { (bb_RegPointer_type) 0x0421,    0xFF01 },
    { (bb_RegPointer_type) 0x042C,    0x03FF },
    /* IXF6048 RX ATM Register Block Channel = 1 */
    { (bb_RegPointer_type) 0x0520,    0x0183 },
    { (bb_RegPointer_type) 0x0521,    0xFF01 },
    { (bb_RegPointer_type) 0x052C,    0x03FF },
    /* IXF6048 RX ATM Register Block Channel = 2 */
    { (bb_RegPointer_type) 0x0620,    0x0383 },
    { (bb_RegPointer_type) 0x0621,    0xFF01 },
    { (bb_RegPointer_type) 0x062C,    0x0000 },
    /* IXF6048 RX ATM Register Block Channel = 3 */
    { (bb_RegPointer_type) 0x0720,    0x0383 },
    { (bb_RegPointer_type) 0x0721,    0xFF01 },
    { (bb_RegPointer_type) 0x072C,    0x0000 },
    /* IXF6048 TX ATM Register Block Channel = 0 */
    { (bb_RegPointer_type) 0x0410,    0x0023 },
    { (bb_RegPointer_type) 0x0411,    0x6A01 },
    { (bb_RegPointer_type) 0x0417,    0x0003 },
    /* IXF6048 TX ATM Register Block Channel = 1 */
    { (bb_RegPointer_type) 0x0510,    0x0023 },
    { (bb_RegPointer_type) 0x0511,    0x6A01 },
    { (bb_RegPointer_type) 0x0517,    0x0000 },
    /* IXF6048 TX ATM Register Block Channel = 2 */
    { (bb_RegPointer_type) 0x0610,    0x0023 },
    { (bb_RegPointer_type) 0x0611,    0x6A01 },
    { (bb_RegPointer_type) 0x0617,    0x0000 },
    /* IXF6048 TX ATM Register Block Channel = 3 */
    { (bb_RegPointer_type) 0x0710,    0x0023 },
    { (bb_RegPointer_type) 0x0711,    0x6A01 },
    { (bb_RegPointer_type) 0x0717,    0x0000 },
    /* IXF6048 RX POS Register Block Channel = 0 */
    { (bb_RegPointer_type) 0x0460,    0x006F },
    { (bb_RegPointer_type) 0x0461,    0x0002 },
    { (bb_RegPointer_type) 0x0462,    0x0600 },
    { (bb_RegPointer_type) 0x0470,    0x0000 },
    /* IXF6048 RX POS Register Block Channel = 1 */
    { (bb_RegPointer_type) 0x0560,    0x006F },
    { (bb_RegPointer_type) 0x0561,    0x0002 },
    { (bb_RegPointer_type) 0x0562,    0x0600 },
    { (bb_RegPointer_type) 0x0570,    0x0000 },
    /* IXF6048 RX POS Register Block Channel = 2 */
    { (bb_RegPointer_type) 0x0660,    0x006F },
    { (bb_RegPointer_type) 0x0661,    0x0002 },
    { (bb_RegPointer_type) 0x0662,    0x0600 },
    { (bb_RegPointer_type) 0x0670,    0x0000 },
    /* IXF6048 RX POS Register Block Channel = 3 */
    { (bb_RegPointer_type) 0x0760,    0x006F },
    { (bb_RegPointer_type) 0x0761,    0x0002 },
    { (bb_RegPointer_type) 0x0762,    0x0600 },
    { (bb_RegPointer_type) 0x0770,    0x0000 },
    /* IXF6048 TX POS Register Block Channel = 0 */
    { (bb_RegPointer_type) 0x0440,    0x004F },
    { (bb_RegPointer_type) 0x0441,    0x0401 },
    { (bb_RegPointer_type) 0x044A,    0x0000 },
    /* IXF6048 TX POS Register Block Channel = 1 */
    { (bb_RegPointer_type) 0x0540,    0x004F },
    { (bb_RegPointer_type) 0x0541,    0x0401 },
    { (bb_RegPointer_type) 0x054A,    0x0000 },
    /* IXF6048 TX POS Register Block Channel = 2 */
    { (bb_RegPointer_type) 0x0640,    0x004F },
    { (bb_RegPointer_type) 0x0641,    0x0401 },
    { (bb_RegPointer_type) 0x064A,    0x0000 },
    /* IXF6048 TX POS Register Block Channel = 3 */
    { (bb_RegPointer_type) 0x0740,    0x004F },
    { (bb_RegPointer_type) 0x0741,    0x0401 },
    { (bb_RegPointer_type) 0x074A,    0x0000 },
               
    { (bb_RegPointer_type) 0xFFFF,    0xA5A5 }
};

typedef struct  /* Complete Data for a Chip */
{
   unsigned short*   BaseAddress;	   /* Base Address of chip */
   unsigned long        ChipType;      /* Type of Chip */
   void*                pChipCfg;      /* Pointer to Chip Specific Configuration */
   void*                pAlarmCfg;     /* Pointer to Chip Specific Alarm Config */
   void*                funcPtr;       /* Pointer to the chip's api func's */
} bb_ChipData_t;

#if 0
typedef struct /* Used as a generic parameter for many API routines */
{
   struct  /* Denotes which block of the chip (chipSection_t) */
   {
       ulong           block;
       bb_ChipSide_t   side;
       bb_ChipFlow_t   flow;
       bb_ChipPath_t   path;
   } block;

   union u_group /* Denotes the section within the block */
   {
       unsigned short  group;
       unsigned short  channel;
       unsigned short  tributary;      
   } group;

   union u_indirect /* Denotes a location within the group */
   {
       uchar   slot;
   } indirect;

   struct /* Misc. variables used for varies function parameters */
   {
       ushort  source;
       uchar   autoIncrement;
       uchar   bitOrBlock;
       uchar   flag;       
   } generic;

   ulong       value;

} bb_ChipSegment_t;


/**********************************************************************
 * For Reset Function
 **********************************************************************/

typedef enum   /* Specifies Chip or Channel Reset */
{  
   bb_RESET_CHIP =  10,        /* Reset entire chip; then reconfig chip */
                               /* Starting from 10 so not to overlap legacy code */
   bb_RESET_RX_CHAN,           /* Reset Rx side of Channel; then recfg Rx Side */
   bb_RESET_TX_CHAN,           /* Reset Tx side of Channel; then recfg Tx Side */
   bb_RESET_CHAN,              /* Reset Channel */
   bb_RESET_RL_MSA_FIFO,       /* Reset the receive FIFO */
   bb_RESET_TT_MSA_FIFO,       /* Reset the transmit FIFO */
   bb_RESET_HPC,               /* Reset the HPC */
   bb_RESET_TL_SKEW_FIFO,      /* Reset the Transmit Line Skew Compensation FIFO */
   bb_RESET_RT_SKEW_FIFO,      /* Reset the Receive Tributary Skew Compensation FIFO */
   bb_RESET_RX_CNTR_INT_SRC,   /* Reset all the performance Counters and Interrupt Sources in the Rx Side */
   bb_RESET_TX_CNTR_INT_SRC,   /* Reset all the performance Counters and Interrupt Sources in the Tx Side */
   bb_RESET_LINE_INTF,
   bb_RESET_SPI_INTF,
   bb_RESET_APS_INTF,

   bb_RESET_GFP_TX_INTF = IXF_API_GFP_BASE,
   bb_RESET_GFP_RX_INTF,
   bb_RESET_GFP_CPU_INTF,
   bb_RESET_GFP_TX_FCS,
   bb_RESET_GFP_RX_FCS,
   bb_RESET_GFP_RX_FSM,
   bb_RESET_GFP,
   bb_MAX_SELRESET                           /* Always make this one last */
} bb_SelResetType_e;

#endif

extern unsigned long				
IxfApiInit(void *baseAddress, unsigned long chipType,
           bb_ChipData_t *pChipData);

extern unsigned long 
Ixf6048InitChip(bb_ChipData_t *pChipData, InitRegTable_t *pTable);


media_init()
{
	bb_ChipData_t pc;

	printf("Initialising media card. Please wait...\n");

	/* initializing slow port */

		PUT32(0xC0080000, 0x3);
		PUT32(0xC0080004, 0x68);
		PUT32(0xC0080008, 0x91);
		PUT32(0xC008000C, 0x68);
		PUT32(0xC0080010, 0x91);
		PUT32(0xC0080014, 0);
		PUT32(0xC0080018, 0x3);
		PUT32(0xC008001C, 0x13);


IxfApiInit((unsigned short*) 0xc6000000, (unsigned long)3, &pc);
Ixf6048InitChip(&pc, (InitRegTable_t *)RegInitIXF6048Table);


}

media_reset()
{
	/* Setting RESET_0 CAP CSR. */
	PUT32(0xc0004a0c,0x208080);
	PUT32(0xc0004a0c,0x208000);
	

	/* This is a Vxworks delay routine 
        and is OS dependent */
	taskDelay(1) ;
}


Ingress_Set_PLL()
{
	int tries, i;

	/*
	 * set the MCCR to 104/125MHz. The register is at 0xc0004a08
	 * -----------------------------------------------------
	 */
	/* put PLL in PLL disable mode */
	PUT32(0xc0004a08, 0x000000f0);
	/* PLL in bypass mode */
	PUT32(0xc0004a08, 0x0000f0f0);
	/* setup MSF clock to 125MHz (both Tx and Rx) */
	PUT32(0xc0004a08,0x00faf0f0);
	/* disable PLL bypass */
	PUT32(0xc0004a08,0x00fa00f0);
	/* turn on MSF */
	PUT32(0xc0004a08,0x00fa0000);
	printf("media_reset : Waiting for MSF PLL lock ...");
	while (GET32(0xc0004a08) != 0xf0fa0000)
	{
		printf("# ");
		for(i=0;i<20000;i++);
	}
	printf(" Done.\n");

}

Egress_Set_PLL()
{
	int tries, i;

	/*
	 * set the MCCR to 125/104MHz. The register is at 0xc0004a08
	 * -----------------------------------------------------
	 */
	/* put PLL in PLL disable mode */
	PUT32(0xc0004a08, 0x000000f0);
	/* PLL in bypass mode */
	PUT32(0xc0004a08, 0x0000f0f0);
	/* setup MSF clock to 125MHz (both Tx and Rx) */
	PUT32(0xc0004a08,0x00aff0f0);
	/* disable PLL bypass */
	PUT32(0xc0004a08,0x00af00f0);
	/* turn on MSF */
	PUT32(0xc0004a08,0x00af0000);
	printf("media_reset : Waiting for MSF PLL lock ...");
	while (GET32(0xc0004a08) != 0xf0af0000)
	{
		printf("# ");
		for(i=0;i<20000;i++);
	}
	printf(" Done.\n");

}

