/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmitted,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/
/*
 * pkthdr_cache.uc
 * Contains IP Header Caching Macros. The macros supports PPP (POS)
 * and Ethernet encapsulations.
 */

#ifndef __PKTHDR_CACHE_UC__
#define __PKTHDR_CACHE_UC__

#ifndef	PKT_HDR_CACHE_SIZE_ETH
#define	PKT_HDR_CACHE_SIZE_ETH		10	// 10 longwords
#endif

#ifndef	PKT_HDR_CACHE_SIZE_POS
#define	PKT_HDR_CACHE_SIZE_POS		6	// 6 longwords
#endif


#ifndef			PKT_HDR_DRAM_RD_SIZE_ETH
#define_eval	PKT_HDR_DRAM_RD_SIZE_ETH	(PKT_HDR_CACHE_SIZE_ETH >> 1)// read 5 quadwords for Ethernet
#endif

#ifndef			PKT_HDR_DRAM_RD_SIZE_POS
#define_eval	PKT_HDR_DRAM_RD_SIZE_POS	(PKT_HDR_CACHE_SIZE_POS >> 1)// read 3 quadwords for POS
#endif

#ifdef			PKT_HDR_CACHE_SIZE
#undef			PKT_HDR_CACHE_SIZE
#endif
#define_eval	PKT_HDR_CACHE_SIZE          PKT_HDR_CACHE_SIZE_ETH// get the greater value


///////////////////////////////////////////////////////////////////////////////
// dl_load_iphdr_cache_eth:
//	 Description: 
//		Reads the IP header (32 bytes) of the Ethernet packet from SDRAM and keeps 
//		it in a cache (DRAM transfer registers). 
//
//	 Outputs:
//		The $$dl_iphdr# global dram transfer registers will be set with appropriate
//		header data.
//
//	Inputs:
//		buf_handle	:			Handle of buffer.
//		req_sig		:			Signal to use in the I/O (DRAM read) operation.
//		sig_action	:			What to do with I/O operation - 
//									- wait for signal(s) as specified by sig_action
//									- do not wait for signal, just return (SIG_NONE)
//	Size:   				:
//
#define	dl_load_iphdr_cache_eth		dl_iphdr_load_cache_eth

#macro dl_iphdr_load_cache_eth[buf_handle, req_sig, sig_action]
.begin
.reg 	sdram_offset, buf_offset

	//	Read 32 bytes of IP Header from SDRAM

	dl_buf_get_data[sdram_offset, buf_handle]
	dl_meta_get_offset[buf_offset]
#ifndef DISABLE_MAC_FILTERING
	dram[read, $$iphdr0, sdram_offset, buf_offset, PKT_HDR_DRAM_RD_SIZE_ETH],\
        sig_done[req_sig]
#else // DISABLE_MAC_FILTERING
	alu[buf_offset, buf_offset, +, 8]
	dram[read, $$iphdr2, sdram_offset, buf_offset, 4], sig_done[req_sig]
#endif // DISABLE_MAC_FILTERING

#if (!streq('sig_action', 'SIG_NONE'))
	ctx_arb[sig_action]
#endif

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_load_iphdr_cache_pos:
//	 Description: 
//		Reads the IP header (32 bytes) of the POS packet from SDRAM and keeps 
//		it in a cache (DRAM transfer registers). 
//
//	 Outputs:
//		The $$dl_iphdr# global dram transfer registers will be set with appropriate
//		header data.
//
//	 Inputs:
//		buf_handle	:			Handle of buffer.
//		req_sig		:			Signal to use in the I/O (DRAM read) operation.
//		sig_action	:			What to do with I/O operation - 
//								- wait for signal(s) as specified by sig_action
//								- do not wait for signal, just return (SIG_NONE)
//	 Size:   				:
//
#define	dl_load_iphdr_cache_pos		dl_iphdr_load_cache_pos

#macro dl_iphdr_load_cache_pos[buf_handle, req_sig, sig_action]
.begin
.reg 	sdram_offset, buf_offset

	//	Read 32 bytes of IP Header from SDRAM

	dl_buf_get_data[sdram_offset, buf_handle]
	dl_meta_get_offset[buf_offset]
	dram[read, $$iphdr0, sdram_offset, buf_offset, PKT_HDR_DRAM_RD_SIZE_POS],\
        sig_done[req_sig]

#if (!streq('sig_action', 'SIG_NONE'))
	ctx_arb[sig_action]
#endif

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_flush_iphdr_cache_eth_pos:
//	 Description: 
//		Writes back the IP header (32 bytes) from the cache to the packet in SDRAM. 
//      The macro supports the both encapsulations such as Ethernet and POS.
//
//	 Outputs:
//
//	 Inputs:
//		buf_handle	:			Handle of buffer.
//		req_sig		:			Signal to use in the I/O (DRAM read) operation.
//		sig_action	:			What to do with I/O operation - 
//								- wait for signal(s) as specified by sig_action
//								- do not wait for signal, just return (SIG_NONE)
//		The $$dl_iphdr# global dram transfer registers will be the set of registers 
//		from which data will be written back to SDRAM.
//
//	 Size:   				:
//
#define	dl_flush_iphdr_cache_eth_pos		dl_iphdr_flush_cache_eth_pos



///////////////////////////////////////////////////////////////////////////////
// dl_flush_iphdr_cache_eth_pos:
//	 Description: 
//		Writes back the IP header (32 bytes) from the cache to the packet in SDRAM. 
//      The macro supports the case when frame was received as Ethernet packet and 
//      is to be sent as POS packet. The macro also insert POS L2 header.
//
//   Assumptions:
//      Register iphdr3 contains 2 bytes of POS hdr and 2 bytes of IP hdr
//      buf_offset in dl_meta set to the begining of L2 hdr
//
//	 Inputs:
//		buf_handle	:			Handle of buffer.
//		req_sig		:			Signal to use in the I/O (DRAM read) operation.
//		sig_action	:			What to do with I/O operation - 
//								- wait for signal(s) as specified by sig_action
//								- do not wait for signal, just return (SIG_NONE)

#macro dl_iphdr_flush_cache_eth_pos[buf_handle, req_sig, sig_action]
.begin
.reg 	sdram_offset, buf_offset, input_port, port_bit

	//	Write 32 bytes of IP Header from SDRAM
    dl_buf_get_data[sdram_offset, buf_handle]

    dl_meta_get_offset[buf_offset]

   	alu[buf_offset,buf_offset,-,4]
	dram[write, $$iphdr2, sdram_offset, buf_offset, 3], sig_done[req_sig]

#if (!streq('sig_action', 'SIG_NONE'))
	ctx_arb[sig_action]
#endif

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_flush_iphdr_cache_eth_eth:
//	 Description: 
//		Writes back the IP header from the cache to the packet in SDRAM. 
//      The macro supports the case when frame was received as Ethernet packet and 
//      is to be sent as Ethernet packet. The macro also insert Ethernet L2 header.
//
//   Assumptions:
//      Register $$iphdr0, $$iphdr1, $$iphdr2, iphdr3 contains 12 bytes of MAC hdr 
//      and 2 bytes of Ether type and 2 bytes of IP hdr
//      buf_offset in dl_meta set to the begining of L2 hdr
//
//   Macro writes 32 bytes into DRAM
//
//	 Inputs:
//		buf_handle	:			Handle of buffer.
//		req_sig		:			Signal to use in the I/O (DRAM read) operation.
//		sig_action	:			What to do with I/O operation - 
//								- wait for signal(s) as specified by sig_action
//								- do not wait for signal, just return (SIG_NONE)

#macro dl_iphdr_flush_cache_eth_eth[buf_handle, req_sig, sig_action]
.begin
.reg 	sdram_offset, buf_offset, input_port, port_bit

	//	Write 32 bytes of IP Header from SDRAM
    dl_buf_get_data[sdram_offset, buf_handle]

    dl_meta_get_offset[buf_offset]

	dram[write, $$iphdr0, sdram_offset, buf_offset, 4], sig_done[req_sig]

#if (!streq('sig_action', 'SIG_NONE'))
	ctx_arb[sig_action]
#endif

.end
#endm

///////////////////////////////////////////////////////////////////////////////
// dl_flush_iphdr_cache_pos_pos:
//	 Description: 
//		Writes back the IP header from the cache to the packet in SDRAM. 
//      The macro supports the case when frame was received as POS packet and 
//      is to be sent as POS packet. The macro also insert POS L2 header.
//
//   Assumptions:
//      Register $$iphdr0 contains 2 bytes of POS hdr and 2 bytes of IP hdr
//      buf_offset in dl_meta set to the begining of L2 hdr
//
//   Macro writes 24 bytes into DRAM
//
//	 Inputs:
//		buf_handle	:			Handle of buffer.
//		req_sig		:			Signal to use in the I/O (DRAM read) operation.
//		sig_action	:			What to do with I/O operation - 
//								- wait for signal(s) as specified by sig_action
//								- do not wait for signal, just return (SIG_NONE)

#macro dl_iphdr_flush_cache_pos_pos[buf_handle, req_sig, sig_action]
.begin
.reg 	sdram_offset, buf_offset, input_port, port_bit

	//	Write 32 bytes of IP Header from SDRAM
    dl_buf_get_data[sdram_offset, buf_handle]

    dl_meta_get_offset[buf_offset]

	dram[write, $$iphdr0, sdram_offset, buf_offset, PKT_HDR_DRAM_RD_SIZE_POS],\
        sig_done[req_sig]

#if (!streq('sig_action', 'SIG_NONE'))
	ctx_arb[sig_action]
#endif

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_flush_iphdr_cache_pos_eth:
//	 Description: 
//		Writes back the IP header (32 bytes) from the cache to the packet in SDRAM. 
//      The macro supports the case when frame was received as POS packet and 
//      is to be sent as Ethernet packet. The macro also insert Ethernet L2 header.
//
//   Assumptions:
//      Register $$l2hdr1, $$l2hdr2, $$l2hdr3, $$iphdr0 contains 12 bytes of MAC hdr 
//      and 2 bytes of Ether type and 2 bytes of IP hdr, $$l2hdr0 is meaningless
//      buf_offset in dl_meta set to the begining of L2 hdr
//
//   Macro writes 40 bytes into DRAM
//
//	 Inputs:
//		buf_handle	:			Handle of buffer.
//		req_sig		:			Signal to use in the I/O (DRAM read) operation.
//		sig_action	:			What to do with I/O operation - 
//								- wait for signal(s) as specified by sig_action
//								- do not wait for signal, just return (SIG_NONE)

#macro dl_iphdr_flush_cache_pos_eth[buf_handle, req_sig, sig_action]
.begin
.reg 	sdram_offset, buf_offset, input_port, port_bit

	//	Write 32 bytes of IP Header from SDRAM
    dl_buf_get_data[sdram_offset, buf_handle]

    dl_meta_get_offset[buf_offset]

	alu[buf_offset,buf_offset,-,4]	
	dram[write, $$l2hdr0, sdram_offset, buf_offset, 5],\
        sig_done[req_sig]

#if (!streq('sig_action', 'SIG_NONE'))
	ctx_arb[sig_action]
#endif

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_iphdr_cache_init_eth_pos:
//	 Description: 
//		This macro needs to be called before any of the dl_iphdr_* can be called.
//		It allocates a set of DRAM xfer registers to be used for caching. This macro
//		doesn't generate any code (instructions). It only produces some assembler
//		directives.
//
//	 Outputs:
//
//	 Inputs:
//
//	 Size:   				:
//		4 Instructions

#macro dl_iphdr_cache_init_eth_pos[]

	.reg $$l2hdr0, $$l2hdr1, $$l2hdr2, $$l2hdr3
	.reg $$iphdr0, $$iphdr1, $$iphdr2, $$iphdr3, $$iphdr4
	.reg $$iphdr5, $$iphdr6, $$iphdr7, $$iphdr8, $$iphdr9

	.xfer_order $$l2hdr0, $$l2hdr1, $$l2hdr2, $$l2hdr3,\
		$$iphdr0, $$iphdr1, $$iphdr2, $$iphdr3, $$iphdr4,\
		$$iphdr5, $$iphdr6, $$iphdr7 , $$iphdr8, $$iphdr9

	/*
	 * Below definition to enable $$iphdr in the XFER namespace
	 */
	#ifndef _xbuf_d1_name
		#define_eval _xbuf_d1_name ($$iphdr)
		#define_eval _xbuf_d1_size (10)
	#else
		#ifndef _xbuf_d2_name
			#define_eval _xbuf_d2_name ($$iphdr)
			#define_eval _xbuf_d2_size (10)
		#else
			#ifndef _xbuf_d3_name
				#define_eval _xbuf_d3_name ($$iphdr)
				#define_eval _xbuf_d3_size (10)
			#else
				#ifndef _xbuf_d4_name
					#define_eval _xbuf_d4_name ($$iphdr)
					#define_eval _xbuf_d4_size (10)
				#else
					#error 2 "_xbuf_d_alloc: No more dram xfer bufs available."
				#endif
			#endif
		#endif
	#endif


	immed[$$l2hdr0,0]
	immed[$$l2hdr1,0]
	immed[$$l2hdr2,0]
	immed[$$l2hdr3,0]
#endm

///////////////////////////////////////////////////////////////////////////////
// dl_iphdr_cache_fini_eth_pos:
//	 Description: 
//		This macro needs to be called at the end when dl_iphdr_macros are no longer
//		required. It frees a set of DRAM xfer registers that were allocated using
//		dl_iphdr_cache_init. This macro	doesn't generate any code (instructions). 
//		It only produces some assembler	directives.
//
//	 Outputs:
//
//	 Inputs:
//
//	 Size:   				:
//		0 Instructions

#macro dl_iphdr_cache_fini_eth_pos[]

/*
 * Below disable definition of $$iphdr in the XFER namespace
 */
#define_eval _this_xbuf_d_name ($$iphdr)
#if (streq(_this_xbuf_d_name, _xbuf_d1_name))
	#undef _xbuf_d1_name
	#undef _xbuf_d1_size
	#ifdef _xbuf_d1_next
		#undef _xbuf_d1_next
	#endif

#elif (streq(_this_xbuf_d_name, _xbuf_d2_name))
	#undef _xbuf_d2_name
	#undef _xbuf_d2_size
	#ifdef _xbuf_d2_next
		#undef _xbuf_d2_next
	#endif
#elif (streq(_this_xbuf_d_name, _xbuf_d3_name))
	#undef _xbuf_d3_name
	#undef _xbuf_d3_size
	#ifdef _xbuf_d3_next
		#undef _xbuf_d3_next
	#endif
#elif (streq(_this_xbuf_d_name, _xbuf_d4_name))
	#undef _xbuf_d4_name
	#undef _xbuf_d4_size
	#ifdef _xbuf_d4_next
		#undef _xbuf_d4_next
	#endif
#else
    #error 2 "_xbuf_d_free: Invalid str_xbuf_d_name."
#endif

#endm

#endif	//__PKTHDR_CACHE_UC__