/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmitted,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/
/*
 * pos_ethernet_dl_system.h
 * System wide parameters customised for a given hardware/application
 */
#ifndef	__POS_ETHERNET_DL_SYSTEM__H__
#define	__POS_ETHERNET_DL_SYSTEM__H__

#include <definitions.h>

#include "ix_cc_microengines_bindings.h"

#include "pos_ethernet_memory.h"

#pragma warning(disable: 5145)

/*	Platform is IXDP2801 */
#ifndef IX_PLATFORM_2801
#define IX_PLATFORM_2801
#endif

/*	Chip is IXP2800 */
#ifndef IXP2800
#define IXP2800
#endif	

/////////////////////////////////////////////
///  	RX & TX modes
/////////////////////////////////////////////

/*	Receive Phy mode */
#ifdef RX_PHY_MODE
#undef RX_PHY_MODE
#endif
#define RX_PHY_MODE		MPHY_16
//#define RX_PHY_MODE		SPHY_1_32
/*SPHY_1_32*//*MPHY_16*/////this is to be replaced with MPHY_16 
						//when the appropriate mode will be supported by RX

/* Transmit Phy mode */
#ifdef TX_PHY_MODE
#undef TX_PHY_MODE
#endif
#define TX_PHY_MODE 	MPHY_16


//------------------------------------------------------------------------------
/* PORT type assignment */
#define 	PORT_0_TYPE		PPP_TYPE
#define 	PORT_1_TYPE 	PPP_TYPE
#define 	PORT_2_TYPE 	PPP_TYPE
#define 	PORT_3_TYPE 	PPP_TYPE
#define 	PORT_4_TYPE 	ETHER_TYPE
#define 	PORT_5_TYPE 	ETHER_TYPE
#define 	PORT_6_TYPE 	ETHER_TYPE
#define 	PORT_7_TYPE 	ETHER_TYPE
#define 	PORT_8_TYPE 	ETHER_TYPE
#define 	PORT_9_TYPE 	ETHER_TYPE
#define 	PORT_10_TYPE 	ETHER_TYPE
#define 	PORT_11_TYPE 	ETHER_TYPE
#define 	PORT_12_TYPE 	ETHER_TYPE
#define 	PORT_13_TYPE 	ETHER_TYPE
#define 	PORT_14_TYPE 	ETHER_TYPE
#define 	PORT_15_TYPE 	ETHER_TYPE

/////////////////////////////////////////////
///  	Meta data spliting if any
/////////////////////////////////////////////

/* Split LW0 and LW1-7 of meta-data */
/* Channel 0 contains LW0 of meta-data. */
/* Channel 2 contains LW1-7 of meta-data. */

//do not use #define DL_SPLIT_META_DATA - to enable spliting define it upon compilation

#ifdef	DL_SPLIT_META_DATA

#define	BUF_SRAM_SPLIT_LW		1
#define	BUF_SRAM_SPLIT_CHANNEL	2 /* we have only channels 0,1,2 in our HW, channel 3 left for TCAM */

#endif // DL_SPLIT_META_DATA

/////////////////////////////////////////////
///  	Various constants
/////////////////////////////////////////////

//#define IPV4_COUNTERS_SCRATCH
#define IP_HDR_OFFSET   14 //only necessary for compilation purposes

/* only temporarly here */
#define IX_CC_RTMV4_L2INDEX_POS   	   0xfffe

/////////////////////////////////////////////
///  	Various constants normally patched
/////////////////////////////////////////////

#ifndef USE_IMPORT_VAR

// free list id that is stored in meta data  in POS RX 
#define     FREE_LIST_ID                    0x0

//	Next 1 (20) SRAM Q-Array entry is used for Drop Queue in Queue Manager.
#define		QM_DROP_QUEUE_ENTRY				20

//	BLADE ID
#define		THIS_BLADE_ID					1

// Port mask for Ethernet ports
#define ETH_PORT_MASK						0xff0

// Port mask for POS ports					
#define POS_PORT_MASK						0x00f

#else /* #ifdef USE_IMPORT_VAR */

.import_var ETH_PORT_MASK
.import_var POS_PORT_MASK

#endif	/* #ifdef USE_IMPORT_VAR */

/////////////////////////////////////////////
///  	Scratch rings names mapping
/////////////////////////////////////////////

//  These defines are used by the individual microcode to identify their scratch rings.
//	The reason for these defines is to improve the portability of microblocks. User just
//	has to make changes here and the microblocks will remain untouched.
// 	For the naming convention, the microblock name goes first, then the _RING_ specifier. 
//  Following this is the direction indicator. '_OUT' would imply the microblock is writing 
//	to this scratch ring and '_IN' would imply the microblock is reading from this scratch 
//	ring. If there is more than one 'IN' or 'OUT' ring the name will have '_<digit>' to 
//  differentiate the rings.

#define			QM_RING_IN_0					IPV4_TO_QM_SCR_RING
#define			QM_RING_IN_0_BASE				IPV4_TO_QM_SCR_RING_BASE
#define			QM_RING_IN_0_SIZE				IPV4_TO_QM_SCR_RING_SIZE

#define			QM_RING_IN_1					SCHEDULER_TO_QM_SCR_RING
#define			QM_RING_IN_1_BASE				SCHEDULER_TO_QM_SCR_RING_BASE
#define			QM_RING_IN_1_SIZE				SCHEDULER_TO_QM_SCR_RING_SIZE

#define			QM_RING_OUT_0					QM_TO_PACKET_TX_SCR_RING_0
#define			QM_RING_OUT_0_BASE				QM_TO_PACKET_TX_SCR_RING_0_BASE
#define			QM_RING_OUT_0_SIZE				QM_TO_PACKET_TX_SCR_RING_0_SIZE

#define			QM_RING_OUT_1					QM_TO_PACKET_TX_SCR_RING_1	
#define			QM_RING_OUT_1_BASE				QM_TO_PACKET_TX_SCR_RING_1_BASE
#define			QM_RING_OUT_1_SIZE				QM_TO_PACKET_TX_SCR_RING_1_SIZE


#define			QM_RING_OUT_2					QM_TO_PACKET_TX_SCR_RING_2
#define			QM_RING_OUT_2_BASE				QM_TO_PACKET_TX_SCR_RING_2_BASE
#define			QM_RING_OUT_2_SIZE				QM_TO_PACKET_TX_SCR_RING_2_SIZE

#define			QM_RING_OUT_3					QM_TO_PACKET_TX_SCR_RING_3
#define			QM_RING_OUT_3_BASE				QM_TO_PACKET_TX_SCR_RING_3_BASE
#define			QM_RING_OUT_3_SIZE				QM_TO_PACKET_TX_SCR_RING_3_SIZE


#define			SCHEDULER_RING_OUT				SCHEDULER_TO_QM_SCR_RING
#define			SCHEDULER_RING_OUT_BASE			SCHEDULER_TO_QM_SCR_RING_BASE			
#define			SCHEDULER_RING_OUT_SIZE			SCHEDULER_TO_QM_SCR_RING_SIZE
								
#define			PACKET_TX_IN_0					QM_TO_PACKET_TX_SCR_RING_0
#define			PACKET_TX_IN_0_BASE				QM_TO_PACKET_TX_SCR_RING_0_BASE
#define			PACKET_TX_IN_0_SIZE				QM_TO_PACKET_TX_SCR_RING_0_SIZE

#define 		PACKET_TX_IN_1					QM_TO_PACKET_TX_SCR_RING_1
#define			PACKET_TX_IN_1_BASE				QM_TO_PACKET_TX_SCR_RING_1_BASE
#define			PACKET_TX_IN_1_SIZE				QM_TO_PACKET_TX_SCR_RING_1_SIZE

								
#define			PACKET_TX_IN_2					QM_TO_PACKET_TX_SCR_RING_2
#define			PACKET_TX_IN_2_BASE				QM_TO_PACKET_TX_SCR_RING_2_BASE
#define			PACKET_TX_IN_2_SIZE				QM_TO_PACKET_TX_SCR_RING_2_SIZE

#define 		PACKET_TX_IN_3					QM_TO_PACKET_TX_SCR_RING_3
#define			PACKET_TX_IN_3_BASE				QM_TO_PACKET_TX_SCR_RING_3_BASE
#define			PACKET_TX_IN_3_SIZE				QM_TO_PACKET_TX_SCR_RING_3_SIZE

//	Scratch address required by Packet TX for updating available Transmit Buffers.
//#define 		PACKET_TX_AVAIL_TBUF_ELE_SCR_ADDR	14336//ok


/////////////////////////////////////////////
///  	QUEUE MANAGER DEFINITIONS
/////////////////////////////////////////////
/*
 *	Size of each Queue Descriptor Entry (in Bytes = 16)
 */
#define			QD_LOG_SRAM_SIZE				4
#define			QD_SIZE							(1<<QD_LOG_SRAM_SIZE)

/////////////////////////////////////////////
///  	Q_ARRAY DEFINITIONS
/////////////////////////////////////////////
//	First 16 (0-15) SRAM Q-Array entries are allocated for the Queue Manager

#define			QM_QARRAY_BASE					0
#define			QM_QARRAY_SIZE					16

//	Next 4 (16-19) SRAM Q-Array entries are allocated for buffer free list

#define			BUF_QARRAY_BASE					16
#define			BUF_QARRAY_SIZE					4


//	Queue Number for Drop Queue. (0-1023 are the normal queues used in forwarding pkts)

#define			QM_DROP_QUEUE					1024

/////////////////////////////////////////////////
///  		SYSTEM DEFINITIONS
////////////////////////////////////////////////
//	These tell if a the executing threads need to work in order. i.e wait for
//	a signal from prev thread, do some work and signal next thread.

#define			DL_THREAD_NO_ORDER				0x0
#define			DL_THREAD_ORDER					0x1


//	All the IX_* constants should be between 0 and 255 (i.e 1 byte) to enable using
//	them as immed operands in instructions.

//	IX_NULL is used to denote a NULL (or empty) packet handle, It should not be zero, 
//	as it would interfere with ring empty (0) condition.

#define			IX_NULL							0xFF
#define			IX_EXCEPTION					0x1
#define			IX_DROP							0x2

#define			UC_NULL							0xFF		/* NULL for microcode */
#define			DL_BUF_NULL						UC_NULL 	/* NULL Buffer representation */


#define			BID_NULL						UC_NULL		/* NULL Block ID */
#define			BID_EXCEPTION					0x1			/* Exception Block ID */
#define			BID_DROP						0x2			/* Drop Block ID */


//	Block IDs for various blocks. Block IDs Start from 0x20. (0 to 0x1f and 0xFF are 
//	reserved for IX_* )

#define			BID_POS							IX_CC_POS_RX_PKT_ID
#define			BID_IPV4						IX_CC_IPV4_PKT_MICROBLOCK_ID
#define			BID_QM							0x23
#define 		BID_ETHER						IX_CC_ETH_TX_PKT_ID
#define         BID_WRED                        IX_CC_WRED_PKT_MICROBLOCK_ID
#define			BID_IPV6						IX_DROP
#define			BID_ARP							IX_EXCEPTION
#define			BID_ETHER_PPP_DISCOVERY			0x27
#define			BID_ETHER_PPP_SESSION			0x28
#define			BID_MPLSILM						0x29

/*---------------------------------------------------------------------
 * DL processing constansts
 *-------------------------------------------------------------------*/
// 	Static Bindings (a.k.a targets) for various microblocks. 
//	Should this go into project/application specific global header file?

#define			DL_SOURCE_NEXT1					BID_ETHER
#define			POS_RX_NEXT1					BID_ETHER

#ifdef USE_WRED
#define			IPV4_NEXT1						BID_WRED
#else
#define			IPV4_NEXT1						BID_QM
#endif


//	To be Redefined as an import variable. And its value should be one of the above
//	Block IDs.

//#ifdef			BID_NEXT_BLOCK
//#undef 			BID_NEXT_BLOCK
//#endif

#ifndef			BID_NEXT_BLOCK
#define			BID_NEXT_BLOCK					BID_QM
#endif


/*---------------------------------------------------------------------
 * WRED constansts
 *-------------------------------------------------------------------*/
//here WRED passing dl_next_block
#define			WRED_NEXT1						BID_QM
//here WRED dropping dl_next_block
#define			WRED_NEXT2						BID_DROP

#define         FIRST_WRED_ME                   0x02

/*---------------------------------------------------------------------
 * Signals
 *-------------------------------------------------------------------*/
//	Signals being a precious resource, we use a single signal for 
//	inter-thread as well as any other inter-me signalling (init over signal)

#define			COMMON_INTER_THD_ME_SIGNAL		10

//	We are not able to use a single signal before of limitations in assembler.
//	until then, we need a separate signal for inter-thread signalling.

#define			INTER_THD_SIGNAL				12

//	ME_NIT_SIGNAL signal is used to let everyone know that initialisation is over
//	and all blocks in all MEs can start running. This signal will be typically issued
//	by the system initialisation code and all other MEs should wait for this signal
//	before starting to run.

#define			ME_INIT_SIGNAL					COMMON_INTER_THD_ME_SIGNAL

//
// WRED_NEXT_ME_SIG is a signal used to synchronize wred crititical sections
// on different MEs
//

#define			WRED_NEXT_ME_SIG                11


//////////////////////////////////////////////
//	Source and Sink related defines
//////////////////////////////////////////////

//	specify if next neighbor ring is used. If not, default is scratch rings.

// 	Valid values for RING_SIZE are 128, 256, 512 or 1024.

#define 		DL_RING_SIZE			ETH_RX_TO_IPV4_SCR_RING_SIZE
//#define 		DL_RING_SIZE			QM_TO_PACKET_TX_SCR_RING_0_SIZE

// 	Base Address for Scratch Ring is 0x1000. Should be aligned on a 4 x RING_SIZE byte boundary.

#define 		DL_RING_BASE			ETH_RX_TO_IPV4_SCR_RING_BASE
//#define 		DL_RING_BASE			QM_TO_PACKET_TX_SCR_RING_0_BASE

// 	Ring number to use. 0-15.

#define_eval 	DL_RING_NUMBER			ETH_RX_TO_IPV4_SCR_RING	
//#define_eval 	DL_RING_NUMBER			QM_TO_PACKET_TX_SCR_RING_0

// 	Signal number to use in inter thread signalling. 

#define 		DL_SIG_WAKE				INTER_THD_SIGNAL

// Drop buffer chain ring

#define_eval 	DL_DROP_RING			IPV4_TO_QM_SCR_RING

// some definition to keep compatibel with PR3 code
#define_eval QM_TO_POS_TX_SCR_RING_0 	QM_TO_PACKET_TX_SCR_RING_0
#define_eval QM_TO_POS_TX_SCR_RING_1 	QM_TO_PACKET_TX_SCR_RING_1


//the below define is to obsolete old assembler
#define_eval _FULL 						_STATUS


/*---------------------------------------------------------------------
 * DRAM memory settings
 *-------------------------------------------------------------------*/

/* The DDR (Double Data Rate) DRAM bank size is used to randomize 
   buffer offset. */
#ifndef		DDR_BANK_SIZE
#define		DDR_BANK_SIZE			128
#endif

/* Number of DDR banks. */
#ifndef		NO_OF_DDR_BANKS
#define		NO_OF_DDR_BANKS			4
#endif

#define		MAX_BUFFER_OFFSET		(DDR_BANK_SIZE * NO_OF_DDR_BANKS)

/* Changes for PR6 compatibility */
#define		BID_ETHER_DECAP_NEXT1	(BID_IPV4)
#define		BID_ETHER_DECAP_NEXT2	(BID_IPV6)
#define		BID_ETHER_DECAP_NEXT3	(BID_ARP)

/* these defines are in a diffrent way !*/
#define		BID_PPP_DECAP_NEXT1	    (BID_IPV4)
#define		BID_PPP_DECAP_NEXT2	    (IX_DROP)
#define		BID_PPP_DECAP_NEXT3	    (BID_IPV6)

/* changes for PR7 compatibility */
#define	TARGET_ETHER_DECAP_IPV4				(BID_IPV4)
#define	TARGET_ETHER_DECAP_IPV6 			(BID_IPV6)
#define TARGET_ETHER_DECAP_ARP  			(BID_ARP)

#define TARGET_ETHER_DECAP_PPP_DISCOVERY	(IX_DROP)//(BID_ETHER_PPP_DISCOVERY)
#define TARGET_ETHER_DECAP_PPP_SESSION		(IX_DROP)//(BID_ETHER_PPP_SESSION)

#define IPV4_LOCAL_NEXT1					(IX_EXCEPTION)

#endif		// __POS_ETHERNET_DL_SYSTEM__H__

