/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmitted,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/
/*
 * pos_ethernet_ipv4.uc
 */

#ifndef __POS_ETHERNET_IPV4_UC__
#define __POS_ETHERNET_IPV4_UC__

//  Include Files

// include system.h from dispatch loop
#include    "dl_system.h"

#include    "definitions.h"

// include system_init.h from dispatch loop for inter ME signal
#include    "system_init.uc"

// include dispatch_loop.uc from dispatch loop 
#include    "dispatch_loop.uc"

// include ipv4_fwder_dl.uc from IPV4 forwarder 
#include    "ipv4_fwder_init.uc"

// include ipv4_fwder.uc from IPV4 forwarder 
#include    "ipv4_fwder.uc"

// include ether.uc from rx/l2
#include    "ether.uc"

// include ppp.uc from rx
#include 	"ppp.uc"

//include ethernet_arp constants
#include    "ethernet_arp.h"

#ifdef USE_WRED
#include    "wred_defines.h"
//include wred microblock
#include    "wred.uc"
#endif

// constants and defines
#ifndef __PKTHDR
#define __PKTHDR    			$$iphdr ; for convenience. DL defines this.
#else
#warning    "__PKTHDR is already defined."
#endif


//-----------------------------------------------------------
// Initialize the blocks
//-----------------------------------------------------------

init#:
    // allocate cache to store the packet header
    dl_iphdr_cache_init_eth_pos[]
    ; init buffer meta data cache to Zero.
    init_meta_cache_to_zero[]

    ; Temporary : Initialize EOP handle default.
    _dl_eop_buf_handle_init[]

#ifdef USE_IMPORT_VAR
    ; The code below is only to patch all standard constants
    .begin
    .reg tmp0

#ifdef COUNTERS

    // PACKET_TX_COUNTER_BASE is patched here because we must patch
    // L2_TABLE_SRAM_BASE and the easiest way is to
    // expand the UE mask for ethernet TX core component which has also
    // PACKET_TX_COUNTER patched.
    .import_var PACKET_TX_COUNTER_BASE

        immed32[tmp0, PACKET_TX_COUNTER_BASE]
        .use tmp0

#endif //COUNTERS
#ifndef PROCESS_CONTROL_BLOCK
		//Simulate that we have a process control block
//		immed32[tmp0, CONTROL_BLOCK_SRAM_BASE]
//		.use tmp0
#endif
        immed32[tmp0, BUF_FREE_LIST0]
        .use tmp0
        immed32[tmp0, DL_REL_BASE]
        .use tmp0
        immed32[tmp0, BUF_SRAM_BASE]
        .use tmp0
    .end
#endif

#ifdef USE_WRED

	; initialize wred stage
	wred_init()


//#if 0 //#ifndef USE_IMPORT_VAR

#if (DL_NEXT_ME == FIRST_WRED_ME)

	br!=ctx[0,skip_wred_me_signal#]

    // Signal the thread 0 on FIRST_WRED_ME with wred_next_me_signal
    // signal_me[FIRST_WRED_ME, &wred_next_me_signal]
    cap[fast_wr,((&wred_next_me_signal) | (FIRST_WRED_ME <<7)), INTERTHREAD_SIG]
skip_wred_me_signal#:

#endif /* (DL_NEXT_ME == FIRST_WRED_ME) */

//#else /* USE_IMPORT_VAR version */

//#ifdef IPV4_START_ME

//	br!=ctx[0,skip_wred_signal#]

    // Signal the thread 1 on FIRST_WRED_ME with wred_next_me_signal
    // signal_me[FIRST_WRED_ME, &wred_next_me_signal]
//	signal_next_ctx[(&wred_next_thread_signal)]

//skip_wred_signal#:

//#endif /* IPV4_START_ME */
//#endif /* USE_IMPORT_VAR */

#endif /* USE_WRED */

    ; for us L2 gprs are unused so set them to 0
    _ether_init(0, 0)

	//; initialize DL registers
	//immed[dl_exception_reg, 0]
	immed[dl_encap_type, DL_ENCAP_UNKNOWN]

    ; initialize l2 validation stage
    dl_init_l2_encap()

    ; ipv4fwder initialization
    _ipv4_fwder_init()

    ; now kick start the IPV4 Forwarder
    _ipv4_fwder_kick_start()

//-----------------------------------------------------------
// dispatch loop
//-----------------------------------------------------------
.begin
.reg sig_mask
    immed32[sig_mask, 0]

    // get packets in order.
    dl_source[DL_THREAD_ORDER, sig_mask]

.end

// Thread 0 of the IPV4_START_ME  from here on
// will process only exception packets. There 
// is enough headroom available and will not affect
// performance

#ifdef HANDLE_XSCALE_SINK
#ifdef  IPV4_START_ME
    br=ctx[0,_handle_exception_pkt#]
#endif
#endif


loop#:
	; set dl_exception_reg to 0
	immed[dl_exception_reg, 0]

    .if (dl_next_block == BID_ETHER)

        _ether_decap_classify(__PKTHDR,ETHER_HEADER_OFFSET)

        ;process the IP header. The modified packet is in cache
        ;registers starting at IP_HDR_OFFSET.

        _ipv4_fwder(__PKTHDR,__PKTHDR,ETHER_HEADER_LENGTH,ETHER_HEADER_LENGTH)

    .else

    	_ppp_decap_classify(__PKTHDR,PPP_HEADER_OFFSET)

        ;process the IP header. The modified packet is in cache
        ;registers starting at PPP_HEADER_LENGTH.

        _ipv4_fwder(__PKTHDR,__PKTHDR,PPP_HEADER_LENGTH,PPP_HEADER_LENGTH)

    .endif

    //Encapsulate L2 layer
	//If packet is to be sent via Ethernet check whether L2 address for 
	//the next_hop_id is valid. If invalid send the packet to the core for
	//ARP resolution. Otherwise (valid case) encapsulate Ethernet L2 header
	//in DRAM XFERs. 
	//If packet is to be sent via POS encapsulate PPP header in DRAM XFERs.
	//Note that DRAM XFERs are flushed in dl_qm_sink macro in DRAM
    dl_l2_encap()

#ifdef USE_WRED

	//Perform WRED algorithm for Default Forwarding
	//always color_id and class_id equal 0
	//alu[--,dl_next_block,-,76]
	//alu[--,dl_next_block,-,76]
	//.if (dl_next_block == BID_WRED)
		wred()
	//.endif
#endif

    // Now call DL macros to send the current packet
    // and received the next packet. Depending on the dl_next_block, these
    // macros can drop/send exception/send to next block. Also, we need to
    // maintain packet order when we send/receive packets.

.reg sig_mask
.sig sig_scr_put
.set_sig sig_scr_put

    // allocate xfer registers to be used by dl_qm_sink.
    xbuf_alloc[$wxfer, 3, write]
    immed32[sig_mask, 0]

    // send packet.
    dl_qm_sink[sig_mask, sig_scr_put, $wxfer]

    // receive packet
    dl_source[DL_THREAD_NO_ORDER, sig_mask]

.io_completed sig_scr_put

    // free the xfer registers allocated
    xbuf_free[$wxfer]

    // start again.
    br[loop#]   

#ifdef HANDLE_XSCALE_SINK
#ifdef IPV4_START_ME

_handle_exception_pkt#:
    
        dl_get_exception_pkt()

        br[_handle_exception_pkt#]  

#endif //IPV4_START_ME
#endif //HANDLE_XSCALE_SINK

// We never execute the below code. This is here for completeness only.

term#:
    // frees the IP header cache
    dl_iphdr_cache_fini_eth_pos[]

    // Control should never come here.
kill#:
    nop
    nop

#endif  // __POS_ETHERNET_IPV4_UC__


