/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmitted,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/
/*
 * wred_defines.h
 *
 * This file contains the WRED parameters that had to be 
 * customised for a Default Forwarding WRED application
 */
#ifndef WRED_DEFINES_H
#define WRED_DEFINES_H

//currently feature unused 
#ifdef WRED_PACKET_COUNTER_FEATURE
#undef WRED_PACKET_COUNTER_FEATURE
#endif

/*---------------------------------------------------------------------
 * WRED constansts
 *-------------------------------------------------------------------*/
//necessary renames
#define WRED_TABLE_RAM_BASE			WRED_TABLE_SRAM_BASE

/*
 * value used to determine the number of queues per port in the system
 * queue_id = (output_port_id << WRED_MAX_QUEUE) + class_id
 */
#define	WRED_MAX_QUEUE				4


#if 0
#define	WRED_ENTRY_LOG_RAM_SIZE		WRED_ENTRY_LOG_SRAM_SIZE


#ifndef WRED_READ_XFER
#define WRED_READ_XFER 				$wred_xfer
#endif

#ifndef WRED_WRITE_XFER
#define WRED_WRITE_XFER				$wred_xfer
#endif

#ifndef WRED_PACKET_COUNTER_FEATURE

#ifndef WRED_READ_MAX_SIZE
#define WRED_READ_MAX_SIZE 			8
#endif

#ifndef WRED_READ_SIZE
#define WRED_READ_SIZE 				7
#endif

#ifndef WRED_WRITE_MAX_SIZE
#define WRED_WRITE_MAX_SIZE 		2
#endif

#ifndef WRED_WRITE_SIZE
#define WRED_WRITE_SIZE 			2
#endif

#else //here WRED_PACKET_COUNTER_FEATURE defined

#ifndef WRED_READ_MAX_SIZE
#define WRED_READ_MAX_SIZE 			16
#endif

#ifndef WRED_READ_SIZE
#define WRED_READ_SIZE 				13
#endif

#ifndef WRED_WRITE_MAX_SIZE
#define WRED_WRITE_MAX_SIZE 		8
#endif

#ifndef WRED_WRITE_SIZE
#define WRED_WRITE_SIZE 			8
#endif

#endif

//WRED SRAM entry layout - to be moved into custom wred_defs.h
//
//    32               16                 0
//    +-----------------+-----------------+
//  0 |   green max th  |  green min th   |
//    +-----------------+-----------------+
//  1 |   yellow max th |  yellow min th  |
//    +-----------------+-----------------+
//  2 |   red max th    |  red min th     |
//    +-----------------+--------+--------+
//  3 |   green const   |srv time| sweight|
//    +-----------------+--------+--------+
//  4 |   yellow const  |    red const    |
//    +-----------------+-----------------+
//  5 |   green count   |  average length |
//    +-----------------+-----------------+
//  6 |   yellow count  |    red count    |
//    +-----------------+-----------------+
//  7 |           green packets           |
//    +-----------------------------------+
//  8 |           yellow packets          |
//    +-----------------------------------+
//  9 |           red packets             |
//    +-----------------------------------+
// 10 |           green bytes             |
//    +-----------------------------------+
// 11 |           yellow bytes            |
//    +-----------------------------------+
// 12 |           red bytes               |
//    +-----------------------------------+

#define WRED_READ_XFER_G_COUNT			$wred_xfer5
#define WRED_READ_XFER_Y_COUNT			$wred_xfer6
#define WRED_READ_XFER_R_COUNT			$wred_xfer6
#define WRED_READ_XFER_G_CONST			$wred_xfer3
#define WRED_READ_XFER_Y_CONST			$wred_xfer4
#define WRED_READ_XFER_R_CONST			$wred_xfer4
#define WRED_READ_XFER_G_MAX_TH			$wred_xfer0
#define WRED_READ_XFER_Y_MAX_TH			$wred_xfer1
#define WRED_READ_XFER_R_MAX_TH			$wred_xfer2
#define WRED_READ_XFER_G_MIN_TH			$wred_xfer0
#define WRED_READ_XFER_Y_MIN_TH			$wred_xfer1
#define WRED_READ_XFER_R_MIN_TH			$wred_xfer2
#define WRED_READ_XFER_WEIGHT			$wred_xfer3
#define WRED_READ_XFER_Q_AVG_LEN		$wred_xfer5
#define WRED_READ_XFER_G_PACKETS_CNT	$wred_xfer7
#define WRED_READ_XFER_Y_PACKETS_CNT	$wred_xfer8
#define WRED_READ_XFER_R_PACKETS_CNT	$wred_xfer9
#define WRED_READ_XFER_G_BYTES_CNT		$wred_xfer10
#define WRED_READ_XFER_Y_BYTES_CNT		$wred_xfer11
#define WRED_READ_XFER_R_BYTES_CNT		$wred_xfer12

#define WRED_WRITE_XFER_G_COUNT_AVG		$wred_xfer0
#define WRED_WRITE_XFER_YR_COUNT		$wred_xfer1
#define WRED_WRITE_XFER_G_PACKETS_CNT	$wred_xfer2
#define WRED_WRITE_XFER_Y_PACKETS_CNT	$wred_xfer3
#define WRED_WRITE_XFER_R_PACKETS_CNT	$wred_xfer4
#define WRED_WRITE_XFER_G_BYTES_CNT		$wred_xfer5
#define WRED_WRITE_XFER_Y_BYTES_CNT		$wred_xfer6
#define WRED_WRITE_XFER_R_BYTES_CNT		$wred_xfer7

#endif
#endif		// WRED_DEFINES_H

