/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmitted,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/
/*
 *      pos_ethernet_memory.h
 *
 *      This file contains the system memory map that can be
 *      customised for a given hardware/application
 */
#ifndef __POS_ETHERNET_MEMORY__H__
#define __POS_ETHERNET_MEMORY__H__

#define IPV4_COUNTERS_SCRATCH

#ifdef NEXTHOP_INFO_SDRAM
#undef NEXTHOP_INFO_SDRAM
#endif

#define NEXTHOP_INFO_SRAM

/****************************************
 *     SRAM and SDRAM DEFINITIONS
 ***************************************/
#ifdef FAST_SIMULATION

/*
 * Amount of SRAM space available for (meta data of) packet buffers. 0,5 MB.
 *  In simulation we use only 10K bytes.
 */
#define     BUF_SRAM_SIZE       (1024 * 10)

/*
 *  Amount of SDRAM space available for Packet buffers. 64 MB.
 *  In simulation we use only 640K bytes.
 */
#define     BUF_SDRAM_SIZE      (64 * 1024 * 10)

#else

/*
 * Amount of SRAM space available for (meta data of) packet buffers. 0,5 MB.
 *  In hardware we use 200K bytes.
 */
#define     BUF_SRAM_SIZE       (1024 * 10 * 2 * 10)

/*
 *  Amount of SDRAM space available for Packet buffers. 64 MB.
 *  In hardware we use 12M bytes.
 */
#define     BUF_SDRAM_SIZE      (64 * 1024 * 10 * 2 * 10)

#endif

/*
 *  Size of Meta Data (Buffer Descriptors) in SRAM. 32 entries per buffer.
 *  Should be a power of 2.
 */
#define     META_DATA_SIZE      32

/*  Size of packet buffer. */
#define     BUFFER_SIZE         2048


#ifdef USE_IMPORT_VAR

.import_var BUF_SRAM_BASE
.import_var DL_REL_BASE
.import_var BUF_FREE_LIST0
.import_var SRAM_FLT_TABLE_BASE
.import_var SRAM_PORT_BIT_MAP_BASE
.import_var L2_TABLE_SRAM_BASE
.import_var PACKET_TX_AVAIL_TBUF_ELE_SCR_ADDR
//.import_var NEXTHOP_TABLE_SRAM_BASE

#else
/* not USE_IMPORT_VAR */

/**********************************
 *     SRAM and SDRAM DEFINITIONS
 *********************************/
/*---------------------------------------------------------------------
 * SDRAM memory map
 *-------------------------------------------------------------------*/

/*
 *  Base Address in DRAM for the packet buffers.
 * This is for the actual packet data.
 */
#define     BUF_SDRAM_BASE      0

/*---------------------------------------------------------------------
 * SRAM memory map for channel 0.
 *-------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * i) Base address for meta data (buffer descriptors) of packet 
 *    buffers.
 *-------------------------------------------------------------------*/
/*
 *  Base address in SRAM for the packet buffers.
 *  This is for the meta data (buffer descriptors).
 */
#define     BUF_SRAM_BASE       0

/* Assume allocating 1 MB of SRAM for meta-data. */

/*---------------------------------------------------------------------
 * ii) Base address L2 table
 *-------------------------------------------------------------------*/

/*
 * L2 Table (each entry has 32 bytes, bit 31 of first long wors is valid bit
 * requires 64K * 32B = 2MB
 */
#define     L2_TABLE_SRAM_BASE              0x00100000
#define     L2_TABLE_SRAM_SIZE              0x00200000

/*---------------------------------------------------------------------
 * iiii) Base address for scheduler initialization
 *-------------------------------------------------------------------*/
/*  Area of SRAM which is zero'ed out */
#define     SRAM_ZERO_BLOCK                 (L2_TABLE_SRAM_BASE + L2_TABLE_SRAM_SIZE)
#define     SRAM_ZERO_BLOCK_SIZE            0x1000/* enough is 16*4 = 16 longs words needed
	                                               * to initialize scheduler xfers to 0 */

/*---------------------------------------------------------------------
 * iiiii) Base address for queue descriptors.
 *-------------------------------------------------------------------*/

/*  Base Address for Queue Descriptors in SRAM channel 0 [31:30] */
#define     QD_SRAM_BASE        			(SRAM_ZERO_BLOCK + SRAM_ZERO_BLOCK_SIZE)
/*  Total number of Queue Descriptors */
#define     QD_TOTAL            			1025


/*---------------------------------------------------------------------
 * SRAM memory map for channel 1.
 *-------------------------------------------------------------------*/
/*---------------------------------------------------------------------
 * i) Base address for trie table.
 *-------------------------------------------------------------------*/
#define		TRIE_TABLE_SRAM_BASE			0x40000000
#define		TRIE_TABLE_SIZE					0x200000 /*	Trie table size. 2MB */


/*---------------------------------------------------------------------
 * ii) Base address for WRED TABLE
 *-------------------------------------------------------------------*/
#define 	WRED_TABLE_SRAM_BASE			(TRIE_TABLE_SRAM_BASE + TRIE_TABLE_SIZE)
#define 	WRED_TABLE_SRAM_SIZE			0x4000/* enough is 256*16 */


/*---------------------------------------------------------------------
 * iii) Base address for WRED statistics
 *-------------------------------------------------------------------*/

#define     WRED_64BIT_STAT_SRAM_BASE		(WRED_TABLE_SRAM_BASE + WRED_TABLE_SRAM_SIZE)
#define     WRED_64BIT_STAT_SIZE   			0x2000


/*---------------------------------------------------------------------
 * iiii) Optionally IPv4 counters are in SRAM
 *     Normally should be in scratch memory.
 *-------------------------------------------------------------------*/

#ifdef IPV4_COUNTERS_SCRATCH

    /* Base for IPv4 32bit counters. This counter table is placed in
     * Scratch @ 12KB to improve performance.
     */
    #define     IPV4_STATS_TABLE_BASE       12288 
    /* space allocated to maintain these counters */
    #define     IPV4_STATS_TABLE_SIZE       0x800

    /*
     * define the below consts to ease computing of next SRAM locations
     * base after IPv4 32bit counters
     */
    #define     STATS_TABLE_SRAM_BASE       (WRED_64BIT_STAT_SRAM_BASE + WRED_64BIT_STAT_SIZE)
    /* space allocated to maintain these counters */
    #define     STATS_TABLE_SRAM_SIZE	    0x0

#else

    /* base for IPv4 32bit counters */
    #define     STATS_TABLE_SRAM_BASE       (WRED_64BIT_STAT_SRAM_BASE + WRED_64BIT_STAT_SIZE)
    /* space allocated to maintain these counters */
    #define     STATS_TABLE_SRAM_SIZE       0x800

#endif  /* IPV4_COUNTERS_SCRATCH */

/*---------------------------------------------------------------------
 * iiiii) Optional RX packet counters
 *-------------------------------------------------------------------*/

/*  Base for Packet RX counters */
#define     PACKET_COUNTERS_SRAM_BASE       (STATS_TABLE_SRAM_BASE + STATS_TABLE_SRAM_SIZE)
#define     PACKET_COUNTERS_SIZE            0x800 /*this is probably too much*/

/*---------------------------------------------------------------------
 * iiiii) Optional Process Control Block
 *-------------------------------------------------------------------*/
#ifdef PROCESS_CONTROL_BLOCK
/* control block */
	#define     CONTROL_BLOCK_SRAM_BASE     (PACKET_COUNTERS_SRAM_BASE + PACKET_COUNTERS_SIZE)
	#define     CONTROL_BLOCK_SIZE		    0x4 /* one long word cope with 32 ports */
#else //define to ease next defines

	#define     CONTROL_BLOCK_SRAM_BASE     (PACKET_COUNTERS_SRAM_BASE + PACKET_COUNTERS_SIZE)
	#define     CONTROL_BLOCK_SIZE		    0x0

#endif

/*---------------------------------------------------------------------
 * SRAM memory map for channel 2.
 *-------------------------------------------------------------------*/
#ifdef	DL_SPLIT_META_DATA
/*
 * When spliting meta data reserve the 1MB for meta data
 * and start first partition from 1MB offset
 */

/*---------------------------------------------------------------------
 * i) Base address for directed broadcast table.
 *-------------------------------------------------------------------*/
#define		DBCAST_TABLE_SRAM_BASE		0x80100000 
#define		DBCAST_TABLE_SIZE			0x2000	

#else
//	DL_SPLIT_META_DATA
/*
 * When not spliting meta data start from the begining
 */

/*---------------------------------------------------------------------
 * i) Base address for directed broadcast table.
 *-------------------------------------------------------------------*/
#define		DBCAST_TABLE_SRAM_BASE		0x80000000 
#define		DBCAST_TABLE_SIZE			0x2000	

#endif
// DL_SPLIT_META_DATA

/*---------------------------------------------------------------------
 * ii) Base address for next hop information.
 *-------------------------------------------------------------------*/

/* For better DRAM bank utilization, we put the nexthop info in
 * SRAM at this address. We also have a segment in DRAM reserved
 * to keep this info, should we decide to move back to DRAM. This
 * would be a compile time option for IPV4 forwarder.
 */

#define		NEXTHOP_TABLE_SRAM_BASE		(DBCAST_TABLE_SRAM_BASE + DBCAST_TABLE_SIZE)
/*  Each Next Hop entry has 8 bytes. So 4K is sufficient */
#define		NEXTHOP_TABLE_SIZE			0x1000


/*---------------------------------------------------------------------
 * iii) Optional TX packet counters
 *-------------------------------------------------------------------*/
/*
 * Packet TX needs 16*3 counters and each counter is 1LW
 * this implies 48*4 bytes.
 */
#define     PACKET_TX_COUNTER_BASE          (NEXTHOP_TABLE_SRAM_BASE + NEXTHOP_TABLE_SIZE)
#define     PACKET_TX_COUNTER_SIZE          0x800 /* enough is 48*4 (see above)*/

/* rename to be compabile with POS TX counters */
#define 	POS_TX_COUNTER_BASE				PACKET_TX_COUNTER_BASE


#endif
/* #ifdef USE_IMPORT_VAR */


/*---------------------------------------------------------------------
 * Definitions about entry sizes and so on
 *-------------------------------------------------------------------*/

/*
 * Trie Set defines if not present
 */	
#ifndef		SIZE_OF_TRIE_SET
#define		SIZE_OF_TRIE_SET				16
#endif

#ifndef 	NUM_TRIE_SETS
#define		NUM_TRIE_SETS					1024
#endif

/*
 * DBCAST table defines if not present
 */	
#ifndef		DBCAST_NUM_BLOCKS
#define		DBCAST_NUM_BLOCKS				256
#endif

/* Size of dbcast table entry in long words */
#ifndef		DBCAST_TABLE_BLOCK_SIZE
#define		DBCAST_TABLE_BLOCK_SIZE			4
#endif

/*
 * Next hop table defines if not present
 */	
#ifndef		NUM_NEXTHOPS
#define		NUM_NEXTHOPS					512
#endif

#ifndef		SIZE_OF_NEXTHOP_LW
#define		SIZE_OF_NEXTHOP_LW				2
#endif


/*********************************
 *         SCRATCH DEFINITIONS
 ********************************/

/* These are all Scratch ring definitions */

/*  There are two exception ring to send pkts up */
#define         EXCEPTION_RING_OUT_0            0
#define         EXCEPTION_RING_OUT_0_BASE       0
#define         EXCEPTION_RING_OUT_0_SIZE       128

#define         EXCEPTION_RING_OUT_1            1
#define         EXCEPTION_RING_OUT_1_BASE       512
#define         EXCEPTION_RING_OUT_1_SIZE       128

/*  There are two exception ring to send pkts down */
#define         EXCEPTION_RING_IN_0             2
#define         EXCEPTION_RING_IN_0_BASE        1024
#define         EXCEPTION_RING_IN_0_SIZE        128

#define         EXCEPTION_RING_IN_1             3
#define         EXCEPTION_RING_IN_1_BASE        1536
#define         EXCEPTION_RING_IN_1_SIZE        128

/*
 *  Scratch for communicating between ETH RX and IPv4 Blocks
 *  Ring number 4, base 0, and size 1024 Long Words
 */
#define         ETH_RX_TO_IPV4_SCR_RING         4
#define         ETH_RX_TO_IPV4_SCR_RING_BASE    2048
#define         ETH_RX_TO_IPV4_SCR_RING_SIZE    1024

/*
 *  Scratch for communicating between IPV4 and Queue Manager Blocks
 *  Ring number 5, base 512, and size 128 Long Words
 */
#define         IPV4_TO_QM_SCR_RING             5
#define         IPV4_TO_QM_SCR_RING_BASE        6144
#define         IPV4_TO_QM_SCR_RING_SIZE        256

/*
 *  Scratch for communicating between Scheduler and Queue Manager Blocks
 *  Ring number 6, base 1024, and size 256 Long Words
 */
#define         SCHEDULER_TO_QM_SCR_RING        6
#define         SCHEDULER_TO_QM_SCR_RING_BASE   7168
#define         SCHEDULER_TO_QM_SCR_RING_SIZE   256

/*
 *  First scractch ring for communicating between Queue Manager and Packet TX
 *  microblocks
 *  Ring number 6, base 4096, and size 128 long words
 */
#define         QM_TO_PACKET_TX_SCR_RING_0      7
#define         QM_TO_PACKET_TX_SCR_RING_0_BASE 8192
#define         QM_TO_PACKET_TX_SCR_RING_0_SIZE 256

/*
 *  Second scractch ring for communicating between Queue Manager and Packet TX
 *  microblocks
 *  Ring number 7, base 5120, and size 128 long words
 */
#define         QM_TO_PACKET_TX_SCR_RING_1      8
#define         QM_TO_PACKET_TX_SCR_RING_1_BASE 10240
#define         QM_TO_PACKET_TX_SCR_RING_1_SIZE 256

#ifndef USE_IMPORT_VAR
/*
 * Scratch address required by Packet TX for updating available
 * Transmit Buffers.
 */
#define         PACKET_TX_AVAIL_TBUF_ELE_SCR_ADDR   14336/*ok*/

//#define         POS_TX_COUNTER_BASE                 0x2A00
#endif

#endif      /* __POS_ETHERNET_MEMORY__H__ */




