/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/
/**
 * ============================================================================
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      ethernet_rtm_routes.c
 *
 * = DESCRIPTION
 *      This file contains scripts to initialize the pos_ethernet_2801
 *      application on an IXPD2801 platform.
 *
 * = CHANGE HISTORY
 *      08/22/2003      Created
 * ============================================================================
 */

#include "../pos_ethernet_memory.h"
#include "xscale_init.h"

#define NH_WRITE SRAM_WRITE
#define NUM_OF_PORTS 12

int rtm_routes(void) {

int trie_base = SRAM_UC_CH1_TO_XS(TRIE_TABLE_SRAM_BASE);

int nh_base = SRAM_UC_CH2_TO_XS(NEXTHOP_TABLE_SRAM_BASE);

#if (NUM_OF_PORTS == 4)
 
NH_WRITE(nh_base + 0x0,0xffffffff);

/* #
   # This rtm_routes is used for testing IPv4 example application
   # on an IXP 2800
 */ 

 /* #-----------------------------------------------
    # Limited broadcast - exception 		
    #-----------------------------------------------
 */

/* add 255.255.255.255 255.255.255.255 0x7255 0x8 1500 0x02000000
 */
	SRAM_WRITE(trie_base + 0x4053c,0x10000);

/* writing nexthop entry with blade 0x1 and NH 0x7255 and flags 0x8 */ 
NH_WRITE(nh_base + 0x8,0x72550108);

/* MTU is 1500, outport is 0x0 */ 
NH_WRITE(nh_base + 0xc,0x05dc0000);

	SRAM_WRITE(trie_base + 0x404fc,0x4);
	SRAM_WRITE(trie_base + 0x404bc,0x3);
	SRAM_WRITE(trie_base + 0x4047c,0x2);
	SRAM_WRITE(trie_base + 0x3fffc,0x1);

/* #-----------------------------------------------
   #  Invalid destination addresses 	 
   #----------------------------------------------
 */

/* # dst==0 - drop
 */ 

/* add 0.0.0.0 255.255.255.255 0x7000 0x4 1500 0x02000000
 */
	SRAM_WRITE(trie_base + 0x40600,0x20000);

/* writing nexthop entry with blade 0x1 and NH 0x7000 and flags 0x4 */ 
NH_WRITE(nh_base + 0x10,0x70000104);

/* MTU is 1500, outport is 0x0 */ 
NH_WRITE(nh_base + 0x14,0x05dc0000);

	SRAM_WRITE(trie_base + 0x405c0,0x8);
	SRAM_WRITE(trie_base + 0x40580,0x7);
	SRAM_WRITE(trie_base + 0x40540,0x6);
	SRAM_WRITE(trie_base + 0x0,0x5);

/* # dst==127.*.*.* - drop
 */ 

/* add 127.0.0.0 255.0.0.0 0x7127 0x4 1500 0x03000000
 */
	SRAM_WRITE(trie_base + 0x401fc,0x30000);

/* writing nexthop entry with blade 0x1 and NH 0x7127 and flags 0x4 */ 
NH_WRITE(nh_base + 0x18,0x71270104);

/* MTU is 1500, outport is 0x0 */ 
NH_WRITE(nh_base + 0x1c,0x05dc0000);


/* # dst==classE - drop
 */ 
/* add 240.0.0.0 240.0.0.0 0x7740 0x4 1500 0x04000000
 */

	SRAM_WRITE(trie_base + 0x403c0,0x40000);
	SRAM_WRITE(trie_base + 0x403c4,0x40000);
	SRAM_WRITE(trie_base + 0x403c8,0x40000);
	SRAM_WRITE(trie_base + 0x403cc,0x40000);
	SRAM_WRITE(trie_base + 0x403d0,0x40000);
	SRAM_WRITE(trie_base + 0x403d4,0x40000);
	SRAM_WRITE(trie_base + 0x403d8,0x40000);
	SRAM_WRITE(trie_base + 0x403dc,0x40000);
	SRAM_WRITE(trie_base + 0x403e0,0x40000);
	SRAM_WRITE(trie_base + 0x403e4,0x40000);
	SRAM_WRITE(trie_base + 0x403e8,0x40000);
	SRAM_WRITE(trie_base + 0x403ec,0x40000);
	SRAM_WRITE(trie_base + 0x403f0,0x40000);
	SRAM_WRITE(trie_base + 0x403f4,0x40000);
	SRAM_WRITE(trie_base + 0x403f8,0x40000);
	SRAM_WRITE(trie_base + 0x403fc,0x40000);

/* writing nexthop entry 0x4 with blade 0x1 and NH 0x7240 and flags 0x4 */ 
NH_WRITE(nh_base + 0x20,0x72400104);

/* MTU is 1500, outport is 0x0 */ 
NH_WRITE(nh_base + 0x24,0x05dc0000);

/* # dest==multicast - exception
 */ 
/* add 224.0.0.0 224.0.0.0 0x7224 0x10 1500 0x05000000
 */

	SRAM_WRITE(trie_base + 0x40380,0x50000);
	SRAM_WRITE(trie_base + 0x40384,0x50000);
	SRAM_WRITE(trie_base + 0x40388,0x50000);
	SRAM_WRITE(trie_base + 0x4038c,0x50000);
	SRAM_WRITE(trie_base + 0x40390,0x50000);
	SRAM_WRITE(trie_base + 0x40394,0x50000);
	SRAM_WRITE(trie_base + 0x40398,0x50000);
	SRAM_WRITE(trie_base + 0x4039c,0x50000);
	SRAM_WRITE(trie_base + 0x403a0,0x50000);
	SRAM_WRITE(trie_base + 0x403a4,0x50000);
	SRAM_WRITE(trie_base + 0x403a8,0x50000);
	SRAM_WRITE(trie_base + 0x403ac,0x50000);
	SRAM_WRITE(trie_base + 0x403b0,0x50000);
	SRAM_WRITE(trie_base + 0x403b4,0x50000);
	SRAM_WRITE(trie_base + 0x403b8,0x50000);
	SRAM_WRITE(trie_base + 0x403bc,0x50000);

/* writing nexthop entry 0x5 with flags 0x10 blade 0x1 and NH 0x7224 */ 

NH_WRITE(nh_base + 0x28,0x72240110);

/* MTU is 1500, outport is 0x0 */ 

NH_WRITE(nh_base + 0x2c,0x05dc0000);


/* 
 *//* 
 *//* 
 *//* #-----------------------------------------------
 */ /* # 16-bit routes - Forward			 
 */ /* #-----------------------------------------------
 */ /* 
 *//*  add 16.1.0.0 255.255.0.0 0x0101 0xa 1500 0x00000000
 */

	SRAM_WRITE(trie_base + 0x40a04,0x4c0000);

/* writing nexthop entry with flags 0x0 blade 0x1 and NH 0x0101  */ 
NH_WRITE(nh_base + 0x260,0x01010100);

/* MTU is 1500, outport is 0xa */ 
NH_WRITE(nh_base + 0x264,0x05dc000a);


	SRAM_WRITE(trie_base + 0x409c0,0x18);
	SRAM_WRITE(trie_base + 0x40040,0x17);

/*  add 16.2.0.0 255.255.0.0 0x0102 0xb 1500 0x02000000
 */

	SRAM_WRITE(trie_base + 0x40a08,0x4d0000);

/* writing nexthop entry with flags 0x0 blade 0x1 and NH 0x0102 */ 
NH_WRITE(nh_base + 0x268,0x01020100);

/* MTU is 1500, outport is 0xb */ 
NH_WRITE(nh_base + 0x26c,0x05dc000b);


/*  add 16.3.0.0 255.255.0.0 0x0103 0x8 1500 0x03000000
 */
	SRAM_WRITE(trie_base + 0x40a0c,0x4e0000);

/* writing nexthop entry with flags 0x0  blade 0x1 and NH 0x0103 */ 
NH_WRITE(nh_base + 0x270,0x01030100);

/* MTU is 1500, outport is 0x8 */ 
NH_WRITE(nh_base + 0x274,0x05dc0008);


/*  add 16.4.0.0 255.255.0.0 0x0104 0x9 1500 0x04000000
 */
	SRAM_WRITE(trie_base + 0x40a10,0x4f0000);

/* writing nexthop entry with flags 0x0 blade 0x1 and NH 0x0104 */ 
NH_WRITE(nh_base + 0x278,0x01040100);

/* MTU is 1500, outport is 0x9 */ 
NH_WRITE(nh_base + 0x27c,0x05dc0009);

#else /* version with 12 ports */

NH_WRITE(nh_base + 0x0,0xffffffff);

/* #
   # This rtm_routes is used for testing IPv4
   #
 */ 

 /* #-----------------------------------------------
    # Limited broadcast - exception 		
    #-----------------------------------------------
 */

/* add 255.255.255.255 255.255.255.255 0x7255 0x8 1500 0x02000000
 */

	SRAM_WRITE(trie_base + 0x4053c,0x10000);

/* writing nexthop entry with blade 0x1 and NH 0x7255 and flags 0x8 */ 
NH_WRITE(nh_base + 0x8,0x72550108);

/* MTU is 1500, outport is 0x0 */ 
NH_WRITE(nh_base + 0xc,0x05dc0000);

	SRAM_WRITE(trie_base + 0x404fc,0x4);
	SRAM_WRITE(trie_base + 0x404bc,0x3);
	SRAM_WRITE(trie_base + 0x4047c,0x2);
	SRAM_WRITE(trie_base + 0x3fffc,0x1);

/* #-----------------------------------------------
   #  Invalid destination addresses 	 
   #----------------------------------------------
 */

/* # dst==0 - drop
 */ 

/* add 0.0.0.0 255.255.255.255 0x7000 0x4 1500 0x02000000
 */

	SRAM_WRITE(trie_base + 0x40600,0x20000);

/* writing nexthop entry with blade 0x1 and NH 0x7000 and flags 0x4 */ 
NH_WRITE(nh_base + 0x10,0x70000104);

/* MTU is 1500, outport is 0x0 */ 
NH_WRITE(nh_base + 0x14,0x05dc0000);

	SRAM_WRITE(trie_base + 0x405c0,0x8);
	SRAM_WRITE(trie_base + 0x40580,0x7);
	SRAM_WRITE(trie_base + 0x40540,0x6);
	SRAM_WRITE(trie_base + 0x0,0x5);

/* # dst==127.*.*.* - drop
 */ 

/* add 127.0.0.0 255.0.0.0 0x7127 0x4 1500 0x03000000
 */

	SRAM_WRITE(trie_base + 0x401fc,0x30000);

/* writing nexthop entry with blade 0x1 and NH 0x7127 and flags 0x4 */ 

NH_WRITE(nh_base + 0x18,0x71270104);

/* MTU is 1500, outport is 0x0 */ 

NH_WRITE(nh_base + 0x1c,0x05dc0000);


/* # dst==classE - drop
 */ 
/* add 240.0.0.0 240.0.0.0 0x7740 0x4 1500 0x04000000
 */

	SRAM_WRITE(trie_base + 0x403c0,0x40000);
	SRAM_WRITE(trie_base + 0x403c4,0x40000);
	SRAM_WRITE(trie_base + 0x403c8,0x40000);
	SRAM_WRITE(trie_base + 0x403cc,0x40000);
	SRAM_WRITE(trie_base + 0x403d0,0x40000);
	SRAM_WRITE(trie_base + 0x403d4,0x40000);
	SRAM_WRITE(trie_base + 0x403d8,0x40000);
	SRAM_WRITE(trie_base + 0x403dc,0x40000);
	SRAM_WRITE(trie_base + 0x403e0,0x40000);
	SRAM_WRITE(trie_base + 0x403e4,0x40000);
	SRAM_WRITE(trie_base + 0x403e8,0x40000);
	SRAM_WRITE(trie_base + 0x403ec,0x40000);
	SRAM_WRITE(trie_base + 0x403f0,0x40000);
	SRAM_WRITE(trie_base + 0x403f4,0x40000);
	SRAM_WRITE(trie_base + 0x403f8,0x40000);
	SRAM_WRITE(trie_base + 0x403fc,0x40000);

/* writing nexthop entry 0x4 with blade 0x1 and NH 0x7240 and flags 0x4 */ 

NH_WRITE(nh_base + 0x20,0x72400104);

/* MTU is 1500, outport is 0x0 */ 

NH_WRITE(nh_base + 0x24,0x05dc0000);

/* # dest==multicast - exception
 */ 
/* add 224.0.0.0 224.0.0.0 0x7224 0x10 1500 0x05000000
 */

	SRAM_WRITE(trie_base + 0x40380,0x50000);
	SRAM_WRITE(trie_base + 0x40384,0x50000);
	SRAM_WRITE(trie_base + 0x40388,0x50000);
	SRAM_WRITE(trie_base + 0x4038c,0x50000);
	SRAM_WRITE(trie_base + 0x40390,0x50000);
	SRAM_WRITE(trie_base + 0x40394,0x50000);
	SRAM_WRITE(trie_base + 0x40398,0x50000);
	SRAM_WRITE(trie_base + 0x4039c,0x50000);
	SRAM_WRITE(trie_base + 0x403a0,0x50000);
	SRAM_WRITE(trie_base + 0x403a4,0x50000);
	SRAM_WRITE(trie_base + 0x403a8,0x50000);
	SRAM_WRITE(trie_base + 0x403ac,0x50000);
	SRAM_WRITE(trie_base + 0x403b0,0x50000);
	SRAM_WRITE(trie_base + 0x403b4,0x50000);
	SRAM_WRITE(trie_base + 0x403b8,0x50000);
	SRAM_WRITE(trie_base + 0x403bc,0x50000);

/* writing nexthop entry 0x5 with flags 0x10 blade 0x1 and NH 0x7224 */ 
NH_WRITE(nh_base + 0x28,0x72240110);

/* MTU is 1500, outport is 0x0 */ 
NH_WRITE(nh_base + 0x2c,0x05dc0000);

/* 
 *//* 
 *//* 
 *//* #-----------------------------------------------
 */ /* # 16-bit routes - Forward             
 */ /* #-----------------------------------------------
 */ /* 
 */
/*  add 16.1.0.0 255.255.0.0 0xfffe 0x0 9216 POS port 0
 */
    SRAM_WRITE(trie_base + 0x40a04,0x4c0000);


/* writing nexthop entry with flags 0x0 blade 0x1 and NH 0xfffe  */ 
NH_WRITE(nh_base + 0x260,0xfffe0100);
/*NH_WRITE(nh_base + 0x260,0x01000100);--Eth version*/
/* MTU is 9216, outport is 0x0 */ 
NH_WRITE(nh_base + 0x264,0x24000000);


    SRAM_WRITE(trie_base + 0x409c0,0x18);
    SRAM_WRITE(trie_base + 0x40040,0x17);

/*  add 16.2.0.0 255.255.0.0 0xfffe 0x1 9216 POS port 1
 */
    SRAM_WRITE(trie_base + 0x40a08,0x4d0000);

/* writing nexthop entry with flags 0x0 blade 0x1 and NH 0xfffe  */ 
NH_WRITE(nh_base + 0x268,0xfffe0100);
/*NH_WRITE(nh_base + 0x268,0x01010100);--Eth version */

/* MTU is 9216, outport is 0x1 */ 
NH_WRITE(nh_base + 0x26c,0x24000001);


/*  add 16.3.0.0 255.255.0.0 0xfffe 0x2 9216 POS port 2
 */
    SRAM_WRITE(trie_base + 0x40a0c,0x4e0000);

/* writing nexthop entry with flags 0x0  blade 0x1 and NH 0xfffe */ 
NH_WRITE(nh_base + 0x270,0xfffe0100);
/*NH_WRITE(nh_base + 0x270,0x01020100);--Eth version */

/* MTU is 9216, outport is 0x2 */ 
NH_WRITE(nh_base + 0x274,0x24000002);


/*  add 16.4.0.0 255.255.0.0 0xfffe 0x3 9216 POS port 3
 */
    SRAM_WRITE(trie_base + 0x40a10,0x4f0000);


/* writing nexthop entry with flags 0x0  blade 0x1 and NH 0xfffe */ 
NH_WRITE(nh_base + 0x278,0xfffe0100);
/*NH_WRITE(nh_base + 0x278,0x01030100);--Eth version */

/* MTU is 1500, outport is 0x3 */ 
NH_WRITE(nh_base + 0x27c,0x24000003);


/*  add 16.5.0.0 255.255.0.0 0x0104 0x4 1500 ETHER port 4
 */
    SRAM_WRITE(trie_base + 0x40a14,0x500000);

/* writing nexthop entry with flags 0x0  blade 0x1 and NH 0x0104 */ 
NH_WRITE(nh_base + 0x280,0x01040100);

/* MTU is 1500, outport is 0x4 */ 
NH_WRITE(nh_base + 0x284,0x05dc0004);


/*  add 16.6.0.0 255.255.0.0 0x0105 0x5 1500 ETHER port 5
 */
    SRAM_WRITE(trie_base + 0x40a18,0x510000);

/* writing nexthop entry with flags 0x0  blade 0x1 and NH 0x0105 */ 
NH_WRITE(nh_base + 0x288,0x01050100);

/* MTU is 1500, outport is 0x5 */ 
NH_WRITE(nh_base + 0x28c,0x05dc0005);


/*  add 16.7.0.0 255.255.0.0 0x0106 0x6 1500 ETHER port 6
 */
    SRAM_WRITE(trie_base + 0x40a1c,0x520000);

/* writing nexthop entry with flags 0x0  blade 0x1 and NH 0x0106 */ 
NH_WRITE(nh_base + 0x290,0x01060100);

/* MTU is 1500, outport is 0x6 */ 
NH_WRITE(nh_base + 0x294,0x05dc0006);


/*  add 16.8.0.0 255.255.0.0 0x0107 0x7 1500 ETHER port 7
 */
    SRAM_WRITE(trie_base + 0x40a20,0x530000);

/* writing nexthop entry with flags 0x0  blade 0x1 and NH 0x0107 */ 
NH_WRITE(nh_base + 0x298,0x01070100);

/* MTU is 1500, outport is 0x7 */ 
NH_WRITE(nh_base + 0x29c,0x05dc0007);


/*  add 16.9.0.0 255.255.0.0 0x0108 0x8 1500 ETHER port 8
 */
    SRAM_WRITE(trie_base + 0x40a24,0x540000);

/* writing nexthop entry with flags 0x0  blade 0x1 and NH 0x0108 */ 
NH_WRITE(nh_base + 0x2a0,0x01080100);

/* MTU is 1500, outport is 0x8 */ 
NH_WRITE(nh_base + 0x2a4,0x05dc0008);


/*  add 16.10.0.0 255.255.0.0 0x0109 0x9 1500 ETHER port 9
 */
    SRAM_WRITE(trie_base + 0x40a28,0x550000);

/* writing nexthop entry with flags 0x0  blade 0x1 and NH 0x0109 */ 
NH_WRITE(nh_base + 0x2a8,0x01090100);

/* MTU is 1500, outport is 0x9 */ 
NH_WRITE(nh_base + 0x2ac,0x05dc0009);


/*  add 16.11.0.0 255.255.0.0 0x010a 0xa 1500 ETHER port 10
 */
    SRAM_WRITE(trie_base + 0x40a2c,0x560000);

/* writing nexthop entry with flags 0x0  blade 0x1 and NH 0x010a */ 
NH_WRITE(nh_base + 0x2b0,0x010a0100);

/* MTU is 1500, outport is 0xa */ 
NH_WRITE(nh_base + 0x2b4,0x05dc000a);


/*  add 16.12.0.0 255.255.0.0 0x010b 0xb 1500 ETHER port 11
 */
    SRAM_WRITE(trie_base + 0x40a30,0x570000);

/* writing nexthop entry with flags 0x0  blade 0x1 and NH 0x010b */ 
NH_WRITE(nh_base + 0x2b8,0x010b0100);

/* MTU is 1500, outport is 0xb */ 
NH_WRITE(nh_base + 0x2bc,0x05dc000b);


/*  add 16.13.0.0 255.255.0.0 0x010c 0xc 1500 ETHER port 12
 */
    SRAM_WRITE(trie_base + 0x40a34,0x580000);

/* writing nexthop entry with flags 0x0  blade 0x1 and NH 0x010c */ 
NH_WRITE(nh_base + 0x2c0,0x010c0100);

/* MTU is 1500, outport is 0xc */ 
NH_WRITE(nh_base + 0x2c4,0x05dc000c);


/*  add 16.14.0.0 255.255.0.0 0x010d 0xd 1500 ETHER port 13
 */
    SRAM_WRITE(trie_base + 0x40a38,0x590000);

/* writing nexthop entry with flags 0x0  blade 0x1 and NH 0x010c */ 
NH_WRITE(nh_base + 0x2c8,0x010c0100);

/* MTU is 1500, outport is 0xc */ 
NH_WRITE(nh_base + 0x2cc,0x05dc000c);


/*  add 16.15.0.0 255.255.0.0 0x010e 0x0 1500 ETHER port 14
 */
    SRAM_WRITE(trie_base + 0x40a3c,0x5a0000);

/* writing nexthop entry with flags 0x0  blade 0x1 and NH 0x010e */ 
NH_WRITE(nh_base + 0x2d0,0x010e0100);

/* MTU is 1500, outport is 0xe */ 
NH_WRITE(nh_base + 0x2d4,0x05dc000e);


/*  add 16.16.0.0 255.255.0.0 0x010f 0xf 1500 ETHER port 15
 */
    SRAM_WRITE(trie_base + 0x40a40,0x5b0000);

/* writing nexthop entry with flags 0x0  blade 0x1 and NH 0x010f */ 
NH_WRITE(nh_base + 0x2d8,0x010f0100);

/* MTU is 1500, outport is 0xf */ 
NH_WRITE(nh_base + 0x2dc,0x05dc000f);


    SRAM_WRITE(trie_base + 0x409c4,0x19);

#endif 
return 1;


}







