/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/
/**
 * ============================================================================
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      l2_encap_init.c
 *
 * = DESCRIPTION
 *      This file contains scripts to initialize the pos_ethernet_2801
 *      application on an IXPD2801 platform.
 *
 * = CHANGE HISTORY
 *      08/22/2003      Created
 * ============================================================================
 */

#include "../pos_ethernet_memory.h"
#include "xscale_init.h"

/* Defines for APP setup */

#define _L2_TABLE_SRAM_BASE SRAM_UC_CH0_TO_XS(L2_TABLE_SRAM_BASE)

/*#define L2_TABLE_NUM_OF_ENTRIES	65536*/
#define L2_TABLE_NUM_OF_ENTRIES	(L2_TABLE_SRAM_SIZE/16)


/**
 * NAME: l2_encap_init
 *
 * DESCRIPTION: This function initializes memory for L2 table
 */
int l2_encap_init(void) {
	
	int finaladdr;

	printf("l2_encap_init()	: Initializing L2 Table SRAM memory :=..\n");

	finaladdr = _L2_TABLE_SRAM_BASE + L2_TABLE_SRAM_SIZE;

	printf("Clearing SRAM ...");
	init_sram(0x0,_L2_TABLE_SRAM_BASE,finaladdr);
    
	printf("L2 table memory initialization COMPLETE.\n\n");

	return 1;
    
}

/**
 * NAME: l2_encap_init_addresses
 *
 * DESCRIPTION: This function configures L2 addresses in the L2 table
 */
int l2_encap_init_addresses(void)
{
    int i;
    
	printf("l2_encap_init_addresse(): Configuring L2 Table addresses...\n");

    /* we need a number of L2 static addreses (values of them are meaningless)*/
    for (i = 0x100; i < 0x120; i++)
    {
        /* setup L2_table entry */
        /* entry i */
        set_sram(_L2_TABLE_SRAM_BASE + 0x000 + (i<<4),
                 0x800090d7);
        set_sram(_L2_TABLE_SRAM_BASE + 0x004 + (i<<4),
                 0x00080002 + (i<<8));
        set_sram(_L2_TABLE_SRAM_BASE + 0x008 + (i<<4),
                 0x01020300);
        set_sram(_L2_TABLE_SRAM_BASE + 0x00c + (i<<4),
                 0x01080000 + ((i-0x100)<<24));
    }

	printf("L2 Table configuration COMPLETE.\n\n");

	return 1;
}







