/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/
/**
 * ============================================================================
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      eth_drv_cfg.c
 *
 * = DESCRIPTION
 *      This file contains functions to initialize drivers for
 *      the 4oc12_pos_6gb_ethernet_2801 application on an IXPD2801 platform.
 *
 * = CHANGE HISTORY
 *      08/24/2003      Created
 * ============================================================================
 */
#include <stdio.h>
#include "ixf1104ce_driver_api.h"
#include "ixf1104ce_ioctl.h"
#include <memory.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include "pos_atm_ioctl.h"


/* defines for driver modes (bit 2:0) */
#define IX_CC_ETH_TX_DRIVER_MODE_FIBER               0x0
#define IX_CC_ETH_TX_DRIVER_MODE_GIGA_HALF_DUPLEX    0x1
#define IX_CC_ETH_TX_DRIVER_MODE_GIGA_FULL_DUPLEX    0x2
#define IX_CC_ETH_TX_DRIVER_MODE_100_HALF_DUPLEX     0x3
#define IX_CC_ETH_TX_DRIVER_MODE_100_FULL_DUPLEX     0x4
#define IX_CC_ETH_TX_DRIVER_MODE_10_HALF_DUPLEX      0x5
#define IX_CC_ETH_TX_DRIVER_MODE_10_FULL_DUPLEX      0x6


/* defines for driver parity (bit 3) */
#define IX_CC_ETH_TX_DRIVER_MODE_PARITY_ODD          0x0
#define IX_CC_ETH_TX_DRIVER_MODE_PARITY_EVEN         0x8


/* defines for driver block modes (bit 4) */
#define IX_CC_ETH_TX_DRIVER_MODE_BLOCK_MPHY          0x00
#define IX_CC_ETH_TX_DRIVER_MODE_BLOCK_4_8           0x10

#define IX_CC_ETH_TX_FRAME_SIZE  0x23FC /* 9216 - 4 */

#define IX_CC_ETH_TX_DRIVER_PORT_MASK_01   0x3
#define IX_CC_ETH_TX_DRIVER_PORT_MASK_23   0xc

#define IX_CC_ETH_TX_DRIVER_PORT_MASK_2800 0x03F0

void vCb(int ctx, int port, int statusMask)
{
    printf("\nPort %u, Status Mask %X\n", port, statusMask);
}


#define POS_ATM_DB0_DEVICE_NAME  "/dev/pos_atm_drv0"
#define POS_ATM_DB1_DEVICE_NAME  "/dev/pos_atm_drv1"
int atmFd0=-1, atmFd1=-1;

int
framer_init(unsigned int modeDB0, unsigned int modeDB1)
{
    char *modeDB0Str, *modeDB1Str;
    char oc3ModeStr[] = "oc3,pos";
    char oc12ModeStr[] = "oc12,pos";

    if( modeDB0 == 3 )
    {
        modeDB0Str = oc3ModeStr;
    }
    else
    {
        modeDB0Str = oc12ModeStr;
    }

    if( modeDB1 == 3 )
    {
        modeDB1Str = oc3ModeStr;
    }
    else
    {
        modeDB1Str = oc12ModeStr;
    }

    /* driver files opening */
    if ((atmFd0 = open(POS_ATM_DB0_DEVICE_NAME, O_RDONLY, 0)) < 0)
    {
        printf ("ATM DB0 ixf6048 driver not found.\n");
        return -1;
    }

    if (modeDB1 != 0)
    {
        /* driver files opening */
        if ((atmFd1 = open(POS_ATM_DB1_DEVICE_NAME, O_RDONLY, 0)) < 0)
        {
            printf ("ATM DB1 ixf6048 driver not found.\n");
            return -1;
        }
    }

    if (ioctl(atmFd0, POS_ATM_IOCTL_SET_LINK_STATE,
              POS_ATM_CMD_PARAM(0, POS_ATM_STATE_UP)) < 0)
    {
        printf ("\nCannot set Link Up for POS(/ATM) DB0. Errno = %X\n",
                errno);
        return -1;
    }

    if (modeDB1 != 0)
    {
        if (ioctl(atmFd1, POS_ATM_IOCTL_SET_LINK_STATE,
                  POS_ATM_CMD_PARAM(0, POS_ATM_STATE_UP)) < 0)
        {
            printf ("\nCannot set Link Up for POS(/ATM) DB1. Errno = %X\n",
                    errno);
            return -1;
        }
    }

    printf("framer_init END\n");

    return 0;
}

/******************************************************************************/
//static char myArray[512];

void framer_stats(unsigned int devIdx)
{
    pos_atm_ixf6048_counters_t stats;
    unsigned i;

    for (i = 0; i < 4; i++)
    {
        stats.link = i;
        stats.channel = 0;

        if (devIdx == 0)
        {
            if (ioctl(atmFd0, POS_ATM_IOCTL_GET_LINK_STATS, &stats) < 0)
            {
                printf ("\nError reading POS(/ATM) DB0 statistics."
                        " Errno = %X\n", errno);
            }
            else
            {
                printf("POS counters for link %d:\n", i);
                printf(" rxFrames:          %d\n",
                       stats.counters.pos.rxFrames);
                printf(" txFrames:          %d\n",
                       stats.counters.pos.txFrames);
                printf(" rxBytes:           %d\n",
                       stats.counters.pos.rxBytes);
                printf(" txBytes:           %d\n",
                       stats.counters.pos.txBytes);
                printf(" rxAbortedFrames:   %d\n",
                   stats.counters.pos.rxAbortedFrames);
                printf(" txAbortedFrames:   %d\n",
                       stats.counters.pos.txAbortedFrames);
                printf(" rxFcsErrors:       %d\n",
                       stats.counters.pos.rxFcsErrors);
                printf(" rxMinLengthErrors: %d\n",
                       stats.counters.pos.rxMinLengthErrors);
                printf(" rxMaxLengthErrors: %d\n",
                       stats.counters.pos.rxMaxLengthErrors);
                printf(" rxFifoOverflows:   %d\n",
                       stats.counters.pos.rxFifoOverflows);
                printf(" txFifoUnderflows:  %d\n",
                       stats.counters.pos.txFifoUnderflows);
            }
        }
        else if (devIdx == 1)
        {
            
            if (ioctl(atmFd1, POS_ATM_IOCTL_GET_LINK_STATS, &stats) < 0)
            {
                printf ("\nError reading POS(/ATM) DB0 statistics."
                        " Errno = %X\n", errno);
            }
            else
            {
                printf("POS counters for link %d:\n", i);
                printf(" rxFrames:          %d\n",
                       stats.counters.pos.rxFrames);
                printf(" txFrames:          %d\n",
                       stats.counters.pos.txFrames);
                printf(" rxBytes:           %d\n",
                       stats.counters.pos.rxBytes);
                printf(" txBytes:           %d\n",
                       stats.counters.pos.txBytes);
                printf(" rxAbortedFrames:   %d\n",
                       stats.counters.pos.rxAbortedFrames);
                printf(" txAbortedFrames:   %d\n",
                       stats.counters.pos.txAbortedFrames);
                printf(" rxFcsErrors:       %d\n",
                       stats.counters.pos.rxFcsErrors);
                printf(" rxMinLengthErrors: %d\n",
                       stats.counters.pos.rxMinLengthErrors);
                printf(" rxMaxLengthErrors: %d\n",
                       stats.counters.pos.rxMaxLengthErrors);
                printf(" rxFifoOverflows:   %d\n",
                       stats.counters.pos.rxFifoOverflows);
                printf(" txFifoUnderflows:  %d\n",
                       stats.counters.pos.txFifoUnderflows);
            }
        }
    }/* end of for */
}



/*
 * Read Vallejo statistics
 *
 */
void vallejo_stats(unsigned int port, uint32 handle)
{
    gbe_mac_ioctl_ptr_64 stats;
    gbe_mac_ioctl_ptr param;

    if (((1 << port) & IX_CC_ETH_TX_DRIVER_PORT_MASK_2800) == 0)
        return;

    stats.portIndex = port;
    param.portIndex = port;

    printf("---------------------------------------------------------\n");
    printf("Reading Vallejo statistics for port %d\n", port);
    printf("---------------------------------------------------------\n");

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_OCTETS_OK,
                                (void *)(&stats)))
        return;
    printf("GET_RX_OCTETS_OK         = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_OCTETS_BAD,
                                (void *)(&stats)))
        return;
    printf("GET_RX_OCTETS_BAD        = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_UC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_RX_UC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_MC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_RX_MC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_BC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_RX_BC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_64,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_64           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_65_127,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_65_127       = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_128_255,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_128_255      = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_256_511,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_256_511      = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_512_1023,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_512_1023     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_1024_1518,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_1024_1518    = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);
    
    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_1519_MAX,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_1519_MAX     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FCS_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RX_FCS_ERR           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_DATA_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RX_DATA_ERR          = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_ALLIGN_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RX_ALLIGN_ERR        = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_LONG_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RX_LONG_ERR          = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_JABBER_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RX_JABBER_ERR        = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PAUSE_MAC_CTL,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PAUSE_MAC_CTL     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_UNKNOWN_CTL_FRAME,
                                (void *)(&stats)))
        return;
    printf("GET_RX_UNKNOWN_CTL_FRAME = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_VLONG_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_VLONG_ERR            = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RUNT_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RUNT_ERR             = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_SHORT_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_SHORT_ERR            = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_SEQ_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_SEQ_ERR              = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_SYMBOL_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_SYMBOL_ERR           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    /**
     * TX Statistics Register IOCTL Commands.
     */
    printf("---------------------------------------------------------\n");

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_OCTETS_OK,
                                (void *)(&stats)))
        return;
    printf("GET_TX_OCTETS_OK         = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_OCTETS_BAD,
                                (void *)(&stats)))
        return;
    printf("GET_TX_OCTETS_BAD        = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_UC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_TX_UC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_MC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_TX_MC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_BC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_TX_BC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_64,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_64           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_65_127,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_65_127       = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_128_255,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_128_255      = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_256_511,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_256_511      = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_512_1023,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_512_1023     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_1024_1518,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_1024_1518    = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_1519_MAX,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_1519_MAX     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PAUSE_FRAME,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PAUSE_FRAME       = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_DEFERRED_ERR,
                                (void *)(&stats)))
        return;

    printf("GET_TX_DEFERRED_ERR      = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_TOTAL_COLLISION,
                                (void *)(&stats)))
        return;
    printf("GET_TX_TOTAL_COLLISION   = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_SINGLE_COLLISION,
                                (void *)(&stats)))
        return;
    printf("GET_TX_SINGLE_COLLISION  = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_MUL_COLLISION,
                                (void *)(&stats)))
        return;
    printf("GET_TX_MUL_COLLISION     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_LATE_COLLISION,
                                (void *)(&stats)))
        return;
    printf("GET_LATE_COLLISION       = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_EXCV_COLLISION,
                                (void *)(&stats)))
        return;
    printf("GET_TX_EXCV_COLLISION    = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_EXCV_DEFERRED_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_TX_EXCV_DEFERRED_ERR = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_EXCV_LEN_DROP,
                                (void *)(&stats)))
        return;
    printf("GET_TX_EXCV_LEN_DROP     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_UNDERRUN,
                                (void *)(&stats)))
        return;
    printf("GET_TX_UNDERRUN          = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_CRC_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_TX_CRC_ERR           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_FC_COLLISION_SEND,
                                (void *)(&stats)))
        return;
    printf("GET_FC_COLLISION_SEND    = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    /**
     * RX FIFO Register IOCTL Commands
     */
    printf("---------------------------------------------------------\n");

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_STN_ADDR,
                                (void *)(&stats)))
        return;
    printf("GET_STN_ADDR             = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);
    

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FIFO_HIGH_WATERMARK,
                                (void *)(&param)))
        return;
    printf("GET_RX_FIFO_HIGH_WM      = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FIFO_LOW_WATERMARK,
                                (void *)(&param)))
        return;
    printf("GET_RX_FIFO_LOW_WM       = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FRAME_REMOVED,
                                (void *)(&param)))
        return;
    printf("GET_RX_FRAME_REMOVED     = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FIFO_PORT_RESET,
                                (void *)(&param)))
        return;
    printf("GET_RX_FIFO_PORT_RESET   = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FIFO_FRAME_DROP,
                                (void *)(&param)))
        return;
    printf("GET_RX_FIFO_FRAME_DROP   = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FIFO_OVERFLOW_STT,
                                (void *)(&param)))
        return;
    printf("GET_RX_FIFO_OVERFLOW_STT = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_OUT_SEQUENCE_INFO,
                                (void *)(&param)))
        return;
    printf("GET_OUT_SEQUENCE_INFO    = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_DROPPED_PKTS,
                                (void *)(&param)))
        return;
    printf("GET_DROPPED_PKTS         = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RW_PTR_RX_FIFO,
                                (void *)(&param)))
        return;
    printf("GET_RW_PTR_RX_FIFO       = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_OCCUPANCY_RX_FIFO,
                                (void *)(&param)))
        return;
    printf("GET_OCCUPANCY_RX_FIFO    = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_JUMBO_PKT_SIZE,
                                (void *)(&param)))
        return;
    printf("GET_JUMBO_PKT_SIZE       = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_PKT_DROP_CAP_FIFO,
                                (void *)(&param)))
        return;
    printf("GET_PKT_DROP_CAP_FIFO    = 0x%08x\n",
           param.value);

}

int main(
int argc,
char **argv)
{
    uint32 err, i;
    gbe_mac_ioctl_ptr macIoctlInfo;
    gbe_mac_ioctl_ptr_64 macIoctlInfo_64;
    uint32 handle;
    unsigned int status;
    
    printf("Initializing POS ports ...\n");

    if (framer_init(12,0) < 0)
    {
        printf("framer_init failed\n");
        return 1;
    }
    
    printf("Initializing POS ports complete\n");

    handle = 0;

    printf("Initializing Giga Ethernet copper ports ...\n");
    
    status = GbEMAC_DeviceStart(IX_CC_ETH_TX_DRIVER_PORT_MASK_2800,
                                &handle,
                                IX_CC_ETH_TX_DRIVER_MODE_GIGA_FULL_DUPLEX |
                                IX_CC_ETH_TX_DRIVER_MODE_PARITY_ODD |
                                IX_CC_ETH_TX_DRIVER_MODE_BLOCK_MPHY);

    printf("GbEMAC_DeviceStart() ports:4-9 status: %0X ***\n", status);

    /*
     * For all ports set:
     * - the JUMBO MTU size
     * - enable Flow Control capabilities
     * - set RX_FIFO_LOW_WATER_MARK
     * - set RX_FIFO_HIGH_WATER_MARK
     */
    for (i = 0; i < 12; i++)
    {
        if ((IX_CC_ETH_TX_DRIVER_PORT_MASK_2800 & (1 << i)) == 0)
            continue;
        
        macIoctlInfo_64.portIndex = i;
        macIoctlInfo_64.valueHigh = 0x00000201;
        macIoctlInfo_64.valueLow = 0x02030001 + i;
        err = GbEMAC_Ioctl(&handle,
                           SET_STN_ADDR,
                           (void *)(&macIoctlInfo_64));
        if (err != SUCCESS)
        {
            printf("GbEMAC_Ioctl (SET_STN_ADDR) returned error = %d\n",err);
            return 1;
        }

        printf("MAC ADDR for port %d set\n",i);
        fflush(stdout);

        macIoctlInfo.portIndex = i;
        macIoctlInfo.value = IX_CC_ETH_TX_FRAME_SIZE;
        err = GbEMAC_Ioctl(&handle,
                           SET_MAX_FRAME_SIZE,
                           (void *)(&macIoctlInfo));
        if (err != SUCCESS)
        {
            printf("GbEMAC_Ioctl (SET_MAX_FRAME_SIZE)"
                   " returned error = %d\n",err);
            return 1;
        }

        printf("Frame size for port %d set\n",i);
        fflush(stdout);
            
        /*
         * The approximate value of high water mark is calculated according to
         * the following algorithm:
         * (MAC_FIFO_SIZE - 2 * IX_CC_ETH_TX_FRAME_SIZE) / 8
         */
        macIoctlInfo.value = 0x700;/* high water mark 0x700 8 bytes locations*/
        err = GbEMAC_Ioctl(&handle,
                           SET_RFIFO_HIGH_WATERMARK,
                           (void *)(&macIoctlInfo));
        if (err != SUCCESS)
        {
            printf("GbEMAC_Ioctl (SET_RFIFO_HIGH_WATERMARK)"
                   " returned error = %d\n",err);
            return 1;
        }

        printf("High watermark for port %d set\n",i);
        fflush(stdout);

        macIoctlInfo.value = 0x300;/* low water mark 0x300 8 bytes locations */
        err = GbEMAC_Ioctl(&handle,
                           SET_RFIFO_LOW_WATERMARK,
                           (void *)(&macIoctlInfo));
        if (err != SUCCESS)
        {
            printf("GbEMAC_Ioctl (SET_RFIFO_LOW_WATERMARK)"
                   " returned error = %d\n",err);
            return 1;
        }

        printf("Low watermark for port %d set\n",i);
        fflush(stdout);
            
        macIoctlInfo.value = 0x7; /* enable flow control capabilities */
        err = GbEMAC_Ioctl(&handle,
                           SET_FC_MODE,
                           (void *)(&macIoctlInfo));
        if (err != SUCCESS)
        {
            printf("GbEMAC_Ioctl (SET_FC_MODE) returned error = %d\n",err);
            return 1;
        }

        printf("Flow control for port %d set\n",i);
        fflush(stdout);

    }/* end for */
    
    printf("Copper ports initialization COMPLETE. Handle = %d\n",handle);

    while (1)
    {
        char c;
        
        c = getchar();
        switch (c)
        {
        case '0':
            framer_stats(0);
            break;
        case '1':
            for (i = 0; i < 4; i++)
            {
                vallejo_stats(i, handle);
            }
            break;
        case '2':
            for (i = 4; i < 8; i++)
            {
                vallejo_stats(i, handle);
            }
            break;
        case '3':
            for (i = 8; i < 12; i++)
            {
                vallejo_stats(i, handle);
            }
            break;
        break;

        case 'q':
        case 'Q':
            return 0;
                
        }
    }
    return 0;
}












