/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/
/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILENAME
 *      linux_mmap_lib.c
 *
 * = DESCRIPTION
 *
 * = CHANGE HISTORY
 *      06/24/2003      Created
 * ============================================================================
 */
#include "linux_mmap_lib.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>

unsigned char *pVirtSramChan0Base = NULL;
unsigned char *pVirtSramChan1Base = NULL;
unsigned char *pVirtSramChan2Base = NULL;
unsigned char *pVirtSramChan3Base = NULL;
unsigned char *pVirtSDramBase = NULL;
unsigned char *pVirtScratchBase = NULL;
unsigned char *pVirtSlowPortBase = NULL;

int memFd;

/*
  #define SRAM_CH1_ME_BASE 0x40000000
  #define SRAM_CH2_ME_BASE 0x80000000
  #define SRAM_CH3_ME_BASE 0xc0000000
*/

/* We support all Channels */
volatile unsigned int *
me2core_sram(unsigned int addr)
{

    if ((addr > PHY_SRAM_CH0 + SRAM_CH0_REAL_SIZE) &&
        (addr < PHY_SRAM_CH1))
    {
        printf("** EXCEPTION: trying to access out of bounds 0x%.8x in "
               "SRAM channel 0 (0x0 - 0x%.8x).\n",
               addr, SRAM_CH0_REAL_SIZE + PHY_SRAM_CH0);
        exit(-1);
    }
    else if ((addr >= PHY_SRAM_CH0) &&
             (addr < PHY_SRAM_CH0 + SRAM_CH0_REAL_SIZE))
    {
        return
            (volatile unsigned int *)(pVirtSramChan0Base + addr - PHY_SRAM_CH0);
    }
    else if ((addr > PHY_SRAM_CH1 + SRAM_CH1_REAL_SIZE) &&
             (addr < PHY_SRAM_CH2))
    {
        printf("** EXCEPTION: trying to access out of bounds 0x%.8x in "
               "SRAM channel 1 (0x9000000 - 0x%.8x).\n",
               addr, SRAM_CH1_REAL_SIZE + PHY_SRAM_CH1);
        exit(-1);
    }
    else if ((addr >= PHY_SRAM_CH1) &&
             (addr < PHY_SRAM_CH1 + SRAM_CH1_REAL_SIZE))
    {
        return
            (volatile unsigned int *)(pVirtSramChan1Base + addr - PHY_SRAM_CH1);
    }
    else if ((addr > PHY_SRAM_CH2 + SRAM_CH2_REAL_SIZE) &&
             (addr < PHY_SRAM_CH3))
    {
        printf("** EXCEPTION: trying to access out of bounds 0x%.8x in "
               "SRAM channel 2 (0xa000000 - 0x%.8x).\n",
               addr, SRAM_CH2_REAL_SIZE + PHY_SRAM_CH2);
        exit(-1);
    }
    else if ((addr >= PHY_SRAM_CH2) &&
             (addr < PHY_SRAM_CH2 + SRAM_CH2_REAL_SIZE))
    {
        return
            (volatile unsigned int *)(pVirtSramChan2Base + addr - PHY_SRAM_CH2);
    }
    else if (addr > PHY_SRAM_CH3 + SRAM_CH3_REAL_SIZE)
    {
        printf("** EXCEPTION: trying to access out of bounds 0x%.8x in "
               "SRAM channel 3 (0xb000000 - 0x%.8x).\n",
               addr, SRAM_CH3_REAL_SIZE + PHY_SRAM_CH3);
        exit(-1);
    }
    else if ((addr >= PHY_SRAM_CH3) &&
             (addr < PHY_SRAM_CH3 + SRAM_CH3_REAL_SIZE))
    {
        return
            (volatile unsigned int *)(pVirtSramChan3Base + addr - PHY_SRAM_CH3);
    }
    printf("** EXCEPTION: trying to access out of bounds 0x%.8x in "
           "an unknown channel\n", addr);
    exit(-1);
    return (volatile unsigned int *)-1;/* likely exception then */
}

volatile unsigned int *
me2core_dram(unsigned int addr)
{
    if (addr >  SDRAM_REAL_SIZE)
    {
        printf("** EXCEPTION: trying to access out of bounds 0x%.8x in "
               "SDRAM (0x0-0x%.8x).\n", addr, SDRAM_REAL_SIZE);
        exit(-1);
    }
    else
    {
        return (volatile unsigned int *)(addr + pVirtSDramBase);
    }
}

void
internal_set_scratch(unsigned int addr, unsigned int value)
{
    if (addr > PHY_SCRATCH + SCRATCH_SIZE)
    {
        printf("** EXCEPTION: trying to access out of bounds 0x%.8x in "
               "SCRATCH (0x0-0x%.8x).\n", addr, SCRATCH_SIZE + PHY_SCRATCH);
        exit(-1);
    }
    else
    {
        *(volatile unsigned int *)(addr + pVirtScratchBase - PHY_SCRATCH) =
            (unsigned int)(value);
    }
}


void
internal_set_slow_port(unsigned int addr, unsigned int value)
{
    if (addr > SLOW_PORT_BASE + SLOW_PORT_SIZE)
    {
        printf("** EXCEPTION: trying to access out of bounds 0x%.8x in SLOW_PORT (0x0-0x%.8x).\n",
               addr, SLOW_PORT_SIZE + SLOW_PORT_BASE);
        exit(-1);
    }
    else
    {
        *(volatile unsigned int *)(addr + pVirtSlowPortBase - SLOW_PORT_BASE) = (unsigned int)(value);
    }
}


unsigned int
internal_get_slow_port(unsigned int addr)
{
    if (addr > SLOW_PORT_BASE + SLOW_PORT_SIZE)
    {
        printf("** EXCEPTION: trying to access out of bounds 0x%.8x in "
               "SLOW_PORT (0x0-0x%.8x).\n",
               addr, SLOW_PORT_SIZE + SLOW_PORT_BASE);
        exit(-1);
    }
    else
    {
        return *((volatile unsigned int *)
                 (addr + pVirtSlowPortBase - SLOW_PORT_BASE));
    }
}

int
ind_lib_start(void)
{  
    if ((memFd = open("/dev/mem", O_RDWR)) < 0)
    {
        printf("Error opening /dev/mem\n");
        return -1;
    }
  
    if ((pVirtSramChan0Base = mmap(NULL, SRAM_CH0_REAL_SIZE,
                                   PROT_READ | PROT_WRITE,
                                   MAP_SHARED, memFd, PHY_SRAM_CH0)
         ) == MAP_FAILED)
    {
        printf("Error mapping SRAM channel 0\n");
        return -1;
    }

    printf("SRAM CH0: 0x%.8x mapped to 0x%.8x\n", PHY_SRAM_CH0,
           (unsigned int) pVirtSramChan0Base);
  
#if (SRAM_CH1_REAL_SIZE > 0)

    if ((pVirtSramChan1Base = mmap(NULL, SRAM_CH1_REAL_SIZE,
                                   PROT_READ | PROT_WRITE,
                                   MAP_SHARED, memFd, PHY_SRAM_CH1)
         ) == MAP_FAILED)
    {
        printf("Error mapping SRAM channel 1\n");
        return -1;
    }

    printf("SRAM CH1: 0x%.8x mapped to 0x%.8x\n", PHY_SRAM_CH1,
           (unsigned int)pVirtSramChan1Base);

#else

    printf("SRAM CH1: empty, not mapped.\n");

#endif
#if (SRAM_CH2_REAL_SIZE > 0)

    if ((pVirtSramChan2Base = mmap(NULL, SRAM_CH2_REAL_SIZE,
                                   PROT_READ | PROT_WRITE,
                                   MAP_SHARED, memFd, PHY_SRAM_CH2)
         ) == MAP_FAILED)
    {
        printf("Error mapping SRAM channel 1\n");
        return -1;
    }

    printf("SRAM CH2: 0x%.8x mapped to 0x%.8x\n", PHY_SRAM_CH2,
           (unsigned int)pVirtSramChan2Base);

#else

    printf("SRAM CH2: empty, not mapped.\n");

#endif

#if (SRAM_CH3_REAL_SIZE > 0)

    if ((pVirtSramChan3Base = mmap(NULL, SRAM_CH3_REAL_SIZE,
                                   PROT_READ | PROT_WRITE,
                                   MAP_SHARED, memFd, PHY_SRAM_CH3)
         ) == MAP_FAILED)
    {
        printf("Error mapping SRAM channel 3\n");
        return -1;
    }

    printf("SRAM CH3: 0x%.8x mapped to 0x%.8x\n", PHY_SRAM_CH3,
           (unsigned int)pVirtSramChan3Base);

#else

    printf("SRAM CH3: empty, not mapped.\n");

#endif

    if ((pVirtSDramBase = mmap(NULL, SDRAM_REAL_SIZE, PROT_READ | PROT_WRITE,
                               MAP_SHARED, memFd, 0)
         ) == MAP_FAILED)
    {
        printf("Error mapping SDRAM.\n");
        return -1;
    }

    printf("SDRAM   : 0x%.8x mapped to 0x%.8x\n", PHYS_SDRAM_BASE,
           (unsigned int) pVirtSDramBase);

    if ((pVirtScratchBase = mmap(NULL, SCRATCH_SIZE, PROT_READ | PROT_WRITE,
                                 MAP_SHARED, memFd, PHY_SCRATCH)
         ) == MAP_FAILED)
    {
        printf("Error mapping SCRATCH.\n");
        return -1;
    }

    printf("SCRATCH : 0x%.8x mapped to 0x%.8x\n", PHY_SCRATCH,
           (unsigned int) pVirtScratchBase);


    if ((pVirtSlowPortBase = mmap(NULL, SLOW_PORT_SIZE, PROT_READ | PROT_WRITE,
                                  MAP_SHARED, memFd, SLOW_PORT_BASE)
         ) == MAP_FAILED)
    {
        printf("Error mapping SLOW_PORT_MEMORY.\n");
        return -1;
    }

    printf("SLOW_PORT_MEMORY : 0x%.8x mapped to 0x%.8x\n", SLOW_PORT_BASE,
           (unsigned int) pVirtSlowPortBase);

    return 0;
}

void
ind_lib_end(void)
{
    if (pVirtSramChan0Base)
    {
        munmap(pVirtSramChan0Base, SRAM_CH0_REAL_SIZE);
    }
    
    if (pVirtSramChan1Base)
    {
        munmap(pVirtSramChan1Base, SRAM_CH1_REAL_SIZE);
    }

    if (pVirtSramChan2Base)
    {
        munmap(pVirtSramChan2Base, SRAM_CH2_REAL_SIZE);
    }

    if (pVirtSramChan3Base)
    {
        munmap(pVirtSramChan3Base, SRAM_CH3_REAL_SIZE);
    }

    if (pVirtSDramBase)
    {
        munmap(pVirtSDramBase, SDRAM_REAL_SIZE);
    }

    if (pVirtScratchBase)
    {
        munmap(pVirtScratchBase, SCRATCH_SIZE);
    }

    if (pVirtSlowPortBase)
    {
        munmap(pVirtSlowPortBase, SLOW_PORT_BASE);
    }

    close(memFd);

}


int mapMem(
unsigned char **pAddr,
unsigned int addr,
unsigned int size)
{
    *pAddr =  mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_SHARED, memFd, addr);
    if (*pAddr == MAP_FAILED)
    {
        return 1;
    }

    return 0;
}

void unmapMem(
unsigned char *pAddr,
unsigned int size)
{
    munmap(pAddr,size); 
}

void mapMemWrite(
unsigned char *pAddr,
unsigned int baseAddr,
unsigned int addr,
unsigned int value)
{
    *(volatile unsigned int*)(addr + pAddr - baseAddr) = value;
}

unsigned int mapMemRead(
unsigned char *pAddr,
unsigned int baseAddr,
unsigned int addr)
{
    return *((volatile unsigned int*)(addr + pAddr - baseAddr));
}





