/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/
/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILENAME
 *      linux_mmap_lib.h
 *
 * = DESCRIPTION
 *
 * = CHANGE HISTORY
 *      06/24/2003      Created
 * ============================================================================
 */

#define __C_COMPILER__
#define IXP2400
#define CORE_CODE 1

#include <stdio.h>
#include <stdlib.h>
#include <asm-arm/arch-ixp2000/hardware.h>
#include "definitions.h"
#include <linux/config.h>

#define PHY_SRAM_CH0    0x80000000
#define SRAM_CH0_SIZE   0x00400000

#define PHY_SRAM_CH1    0x90000000
#define SRAM_CH1_SIZE   0x00400000

#define PHY_SRAM_CH2    0xa0000000
#define SRAM_CH2_SIZE   0x00400000

#define PHY_SRAM_CH3    0xb0000000
#define SRAM_CH3_SIZE   0x00400000


#define PHY_SCRATCH     0xca000000
/*#define SCRATCH_SIZE    0x02000000*/
#define SCRATCH_SIZE    0x4000 /* 16KB */ /* from xscale init for VxWorks */


#define SLOW_PORT_BASE  0xc4000000
#define SLOW_PORT_SIZE  0x03000000


#ifndef SRAM_CH0_REAL_SIZE
//#define SRAM_CH0_REAL_SIZE (4*1024*1024)
#define SRAM_CH0_REAL_SIZE SRAM_CH0_SIZE
#endif

#ifndef SRAM_CH1_REAL_SIZE
#define SRAM_CH1_REAL_SIZE SRAM_CH1_SIZE
#endif

#ifndef SRAM_CH2_REAL_SIZE
#define SRAM_CH2_REAL_SIZE SRAM_CH2_SIZE
#endif

#ifndef SRAM_CH3_REAL_SIZE
#define SRAM_CH3_REAL_SIZE (0)
#endif


#ifndef SDRAM_REAL_SIZE
#define SDRAM_REAL_SIZE    ((768-64)*1024*1024)
#endif

#ifndef PHYS_SDRAM_SIZE
#define PHYS_SDRAM_SIZE    ((768-64)*1024*1024)
#endif

volatile unsigned int *me2core_sram(unsigned int addr);

volatile unsigned int *me2core_dram(unsigned int addr);

int ind_lib_start(void);

void ind_lib_end(void);

int log2(int arg);

void internal_set_scratch(unsigned int addr, unsigned int value);
void internal_set_slow_port(unsigned int addr, unsigned int value);
unsigned int internal_get_slow_port(unsigned int addr);

#define ps_start_packet_receive() \
        printf("Now ready to receive packets.")

#define get_sram(addr) \
    	(*(volatile unsigned int *) me2core_sram(addr))
#define set_sram(addr, value) \
        (*(volatile unsigned int *) me2core_sram(addr) = (unsigned int)(value))

#define get_dram(addr) \
        (*(volatile unsigned int *) me2core_dram(addr))
#define set_dram(addr, value) \
        (*(volatile unsigned int *) me2core_dram(addr) = (unsigned int)(value))

#define set_scratch(addr, value) \
        internal_set_scratch((addr), (unsigned int) (value));

#define set_slow_port(addr, value) \
        internal_set_slow_port((addr), (unsigned int) (value));

#define get_slow_port(addr) \
        internal_get_slow_port((addr))

#define init_lmem(me, value) \
        printf("Warning: Init LM for ME#%d with %08x. Unsupported...\n", me, value);

#define init_sram(value, startAddr, stopAddr) \
    { \
	    int _ii; \
        printf("Filling SRAM memory from 0x%x to 0x%x, size 0x%x\n", (startAddr), (stopAddr), (stopAddr) - (startAddr)); \
 	    for (_ii = startAddr; _ii < stopAddr; _ii += 4) set_sram(_ii, value); \
    }

#define init_dram(value, startAddr, stopAddr) \
    { \
       int _ii; \
       printf("Filling DRAM memory from 0x%x to 0x%x, size 0x%x\n", (startAddr), (stopAddr), (stopAddr) - (startAddr)); \
       for (_ii = startAddr; _ii < stopAddr; _ii += 4) set_dram(_ii, value); \
    }

#define init_scratch(value, startAddr, stopAddr) \
    { \
       int _ii; \
       printf("Filling SCRATCH memory from 0x%x to 0x%x, size 0x%x\n", (startAddr), (stopAddr), (stopAddr) - (startAddr)); \
       for (_ii = startAddr; _ii < stopAddr; _ii += 4) set_scratch(_ii, value); \
    }









