/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/
/**
 * ============================================================================
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      vallejo.c
 *
 * = DESCRIPTION
 *      This file contains scripts to initialize the vallejo quad gbeth PHY
 *      on an IXPD2401 platform.
 *
 * = CHANGE HISTORY
 *      01/22/2003      Created
 * ============================================================================
 */

/* per port index */
#define INDEX_PORT0 			0x00
#define INDEX_PORT1 			0x80
#define INDEX_PORT2 			0x100
#define INDEX_PORT3 			0x180


/* defines for driver modes (bit 2:0) */
#define IX_CC_ETH_TX_DRIVER_MODE_FIBER               0x0
#define IX_CC_ETH_TX_DRIVER_MODE_GIGA_HALF_DUPLEX    0x1
#define IX_CC_ETH_TX_DRIVER_MODE_GIGA_FULL_DUPLEX    0x2
#define IX_CC_ETH_TX_DRIVER_MODE_100_HALF_DUPLEX     0x3
#define IX_CC_ETH_TX_DRIVER_MODE_100_FULL_DUPLEX     0x4
#define IX_CC_ETH_TX_DRIVER_MODE_10_HALF_DUPLEX      0x5
#define IX_CC_ETH_TX_DRIVER_MODE_10_FULL_DUPLEX      0x6



/* defines for driver parity (bit 3) */
#define IX_CC_ETH_TX_DRIVER_MODE_PARITY_ODD          0x0
#define IX_CC_ETH_TX_DRIVER_MODE_PARITY_EVEN         0x8



/* defines for driver block modes (bit 4) */
#define IX_CC_ETH_TX_DRIVER_MODE_BLOCK_MPHY          0x00
#define IX_CC_ETH_TX_DRIVER_MODE_BLOCK_4_8           0x10


#define IX_CC_ETH_TX_DRIVER_PORT_MASK   0xF
#define IX_CC_ETH_TX_FRAME_SIZE  0x23FC /* 9216 - 4 */

/* Platform specific - in ixdp2401 platform we need to split the port mask
 * onto ports working in fiber/copper mode (front pair) and ports working only
 * in copper mode (backplane pair).
 */
#define IX_CC_ETH_TX_DRIVER_PORT_MASK1   0x3
#define IX_CC_ETH_TX_DRIVER_PORT_MASK2   0xc


/**
 * User defined include files required.
 */

/* ethernet driver files */
#include "ixf1104ce_driver_api.h"
#include "ixf1104ce_ioctl.h"
#include "ixf1104ce_internal.h"


/**
 * Pre-processor symbol and macro definitions.
 * 
 */
#define hal_delay_us(x) taskDelay(1) /* VxWorks specific */

#define eprintf printf

typedef unsigned int	uint32;



uint32 handle1, handle2;


/*
 * Initializes Vallejo, Alaska and CPLD in the requested mode
 *
 */
void vinit(char mode)
{
    uint32 err, i;
    gbe_mac_ioctl_ptr macIoctlInfo;
    gbe_mac_ioctl_ptr_64 macIoctlInfo_64;

    printf("Initializing GbEMAC in mode %c\n", mode);

    switch (mode)
    {
    case 'f':
    case 'F':
        /* setup fiber in Alaska */
        /**(volatile int*)0xc6024018 = 0x00000500;*/
        err = GbEMAC_DeviceStart(IX_CC_ETH_TX_DRIVER_PORT_MASK1,
                                 &handle1,
                                 IX_CC_ETH_TX_DRIVER_MODE_FIBER |
                                 IX_CC_ETH_TX_DRIVER_MODE_PARITY_ODD |
                                 IX_CC_ETH_TX_DRIVER_MODE_BLOCK_MPHY);
        break;
    case 'c':
    case 'C':
        /* Enable Copper in Alaska*/
        /**(volatile int*)(0xc6024018) = 0x000F0003; */
        err = GbEMAC_DeviceStart(IX_CC_ETH_TX_DRIVER_PORT_MASK1,
                                 &handle1,
                                 IX_CC_ETH_TX_DRIVER_MODE_GIGA_FULL_DUPLEX |
                                 IX_CC_ETH_TX_DRIVER_MODE_PARITY_ODD |
                                 IX_CC_ETH_TX_DRIVER_MODE_BLOCK_MPHY);
        break;
    default:
        return;
    }

    if(err != SUCCESS)
    {
        return;
    }/* end if */
    printf("Enabled fiber ports\n");
    
    err = GbEMAC_DeviceStart(IX_CC_ETH_TX_DRIVER_PORT_MASK2,
                             &handle2,
                             IX_CC_ETH_TX_DRIVER_MODE_GIGA_FULL_DUPLEX |
                             IX_CC_ETH_TX_DRIVER_MODE_PARITY_ODD |
                             IX_CC_ETH_TX_DRIVER_MODE_BLOCK_MPHY);
                             
    if(err != SUCCESS)
    {
        return;
    }/* end if */
    printf("Enabled backplane copper ports\n");

    /* Set the JUMPO MTU size for all ports */
    for (i = 0; i < 12; i++)
    {
        if (
            ((IX_CC_ETH_TX_DRIVER_PORT_MASK2 & (1 << i)) != 0) ||
            ((IX_CC_ETH_TX_DRIVER_PORT_MASK1 & (1 << i)) != 0)
            )
        {
            macIoctlInfo_64.portIndex = i;
            macIoctlInfo_64.valueHigh = 0x00000201;
            macIoctlInfo_64.valueLow = 0x02030001 + i;
            
            macIoctlInfo.portIndex = i;
            macIoctlInfo.value = IX_CC_ETH_TX_FRAME_SIZE;
            if ((IX_CC_ETH_TX_DRIVER_PORT_MASK1 & (1 << i)) != 0)
            {
                err = GbEMAC_Ioctl(&handle1,
                                   SET_STN_ADDR,
                                   (void *)(&macIoctlInfo_64));
                if (err != SUCCESS) return;
                
                err = GbEMAC_Ioctl(&handle1,
                               SET_MAX_FRAME_SIZE, 
                                   (void *)(&macIoctlInfo));
            }
            else
            {
                err = GbEMAC_Ioctl(&handle2,
                                   SET_STN_ADDR,
                                   (void *)(&macIoctlInfo_64));
                if (err != SUCCESS) return;
                
                err = GbEMAC_Ioctl(&handle2,
                                   SET_MAX_FRAME_SIZE, 
                                   (void *)(&macIoctlInfo));
            }   
            if (err != SUCCESS)
            {
                return;
            }
        }/* end if */
    }/* end for */

    printf("GbEMAC initialization COMPLETE\n");

}


/*
 * Read Vallejo statistics
 *
 */
void vallejo_stats(unsigned int port)
{
    gbe_mac_ioctl_ptr_64 stats;
    gbe_mac_ioctl_ptr param;
    uint32 handle;

    if (port >= 12) return;

    if (!
        (((IX_CC_ETH_TX_DRIVER_PORT_MASK2 & (1 << port)) != 0) ||
         ((IX_CC_ETH_TX_DRIVER_PORT_MASK1 & (1 << port)) != 0))
        )
        return;
    
    if ((IX_CC_ETH_TX_DRIVER_PORT_MASK1 & (1 << i)) != 0)
        handle = handle1;
    else
        handle = handle2;

    stats.portIndex = port;
    param.portIndex = port;

    printf("---------------------------------------------------------\n");
    printf("Reading Vallejo statistics for port %d\n", port);
    printf("---------------------------------------------------------\n");

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_OCTETS_OK,
                                (void *)(&stats)))
        return;
    printf("GET_RX_OCTETS_OK         = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_OCTETS_BAD,
                                (void *)(&stats)))
        return;
    printf("GET_RX_OCTETS_BAD        = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_UC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_RX_UC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_MC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_RX_MC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_BC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_RX_BC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_64,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_64           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_65_127,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_65_127       = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_128_255,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_128_255      = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_256_511,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_256_511      = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_512_1023,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_512_1023     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_1024_1518,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_1024_1518    = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_1519_MAX,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_1519_MAX     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FCS_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RX_FCS_ERR           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_DATA_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RX_DATA_ERR          = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_ALLIGN_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RX_ALLIGN_ERR        = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_LONG_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RX_LONG_ERR          = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_JABBER_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RX_JABBER_ERR        = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PAUSE_MAC_CTL,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PAUSE_MAC_CTL     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_UNKNOWN_CTL_FRAME,
                                (void *)(&stats)))
        return;
    printf("GET_RX_UNKNOWN_CTL_FRAME = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_VLONG_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_VLONG_ERR            = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RUNT_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RUNT_ERR             = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_SHORT_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_SHORT_ERR            = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_SEQ_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_SEQ_ERR              = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_SYMBOL_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_SYMBOL_ERR           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    /**
     * TX Statistics Register IOCTL Commands.
     */
    printf("---------------------------------------------------------\n");

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_OCTETS_OK,
                                (void *)(&stats)))
        return;
    printf("GET_TX_OCTETS_OK         = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_OCTETS_BAD,
                                (void *)(&stats)))
        return;
    printf("GET_TX_OCTETS_BAD        = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_UC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_TX_UC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_MC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_TX_MC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_BC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_TX_BC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_64,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_64           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_65_127,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_65_127       = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_128_255,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_128_255      = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_256_511,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_256_511      = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_512_1023,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_512_1023     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_1024_1518,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_1024_1518    = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_1519_MAX,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_1519_MAX     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PAUSE_FRAME,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PAUSE_FRAME       = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_DEFERRED_ERR,
                                (void *)(&stats)))
        return;

    printf("GET_TX_DEFERRED_ERR      = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_TOTAL_COLLISION,
                                (void *)(&stats)))
        return;
    printf("GET_TX_TOTAL_COLLISION   = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_SINGLE_COLLISION,
                                (void *)(&stats)))
        return;
    printf("GET_TX_SINGLE_COLLISION  = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_MUL_COLLISION,
                                (void *)(&stats)))
        return;
    printf("GET_TX_MUL_COLLISION     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_LATE_COLLISION,
                                (void *)(&stats)))
        return;
    printf("GET_LATE_COLLISION       = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_EXCV_COLLISION,
                                (void *)(&stats)))
        return;
    printf("GET_TX_EXCV_COLLISION    = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_EXCV_DEFERRED_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_TX_EXCV_DEFERRED_ERR = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_EXCV_LEN_DROP,
                                (void *)(&stats)))
        return;
    printf("GET_TX_EXCV_LEN_DROP     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_UNDERRUN,
                                (void *)(&stats)))
        return;
    printf("GET_TX_UNDERRUN          = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_CRC_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_TX_CRC_ERR           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_FC_COLLISION_SEND,
                                (void *)(&stats)))
        return;
    printf("GET_FC_COLLISION_SEND    = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    /**
     * RX FIFO Register IOCTL Commands
     */
    printf("---------------------------------------------------------\n");

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_STN_ADDR,
                                (void *)(&stats)))
        return;
    printf("GET_STN_ADDR             = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FIFO_HIGH_WATERMARK,
                                (void *)(&param)))
        return;
    printf("GET_RX_FIFO_HIGH_WM      = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FIFO_LOW_WATERMARK,
                                (void *)(&param)))
        return;
    printf("GET_RX_FIFO_LOW_WM       = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FRAME_REMOVED,
                                (void *)(&param)))
        return;
    printf("GET_RX_FRAME_REMOVED     = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FIFO_PORT_RESET,
                                (void *)(&param)))
        return;
    printf("GET_RX_FIFO_PORT_RESET   = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FIFO_FRAME_DROP,
                                (void *)(&param)))
        return;
    printf("GET_RX_FIFO_FRAME_DROP   = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FIFO_OVERFLOW_STT,
                                (void *)(&param)))
        return;
    printf("GET_RX_FIFO_OVERFLOW_STT = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_OUT_SEQUENCE_INFO,
                                (void *)(&param)))
        return;
    printf("GET_OUT_SEQUENCE_INFO    = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_DROPPED_PKTS,
                                (void *)(&param)))
        return;
    printf("GET_DROPPED_PKTS         = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RW_PTR_RX_FIFO,
                                (void *)(&param)))
        return;
    printf("GET_RW_PTR_RX_FIFO       = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_OCCUPANCY_RX_FIFO,
                                (void *)(&param)))
        return;
    printf("GET_OCCUPANCY_RX_FIFO    = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_JUMBO_PKT_SIZE,
                                (void *)(&param)))
        return;
    printf("GET_JUMBO_PKT_SIZE       = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_PKT_DROP_CAP_FIFO,
                                (void *)(&param)))
        return;
    printf("GET_PKT_DROP_CAP_FIFO    = 0x%08x\n",
           param.value);

}












