/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/
/**
 * ============================================================================
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      xscale_init.h
 *
 * = DESCRIPTION
 *      This file contains definitions used by xscale_init files.
 *
 * = CHANGE HISTORY
 *      06/17/2003      Initially created
 * ============================================================================
 */

#ifndef XSCALE_INIT_H
#define XSCALE_INIT_H

#ifndef XSCALE_INIT_FOR_LINUX

/* Macro to write and read one longword from memory 
   the macro assumes that the memory being read or written to
   is mapped 
 */
#define GET32(address)  	(*(volatile unsigned int *)(address))
#define PUT32(address,val)  (*(volatile unsigned int *)(address) = (val))

#define set_sram(a,b) 		PUT32(a,b)
#define set_dram(a,b) 		PUT32(a,b)

#define SDRAM_BEGIN	 0x2000000

#else

#include <stdio.h>
#include "linux_mmap_lib.h"

#define SDRAM_BEGIN	 0x0

int rtm_init(void);

#endif  /* XSCALE_INIT_FOR_LINUX */

#define SRAM_BEGIN       0x80000000
#define CHANNEL_0_OFFSET 0x00000000
#define CHANNEL_1_OFFSET 0x10000000
#define CHANNEL_2_OFFSET 0x20000000
#define CHANNEL_3_OFFSET 0x30000000
#define SCRATCH_BEGIN    0xca000000

#define SRAM_BASE_CH0	(SRAM_BEGIN + CHANNEL_0_OFFSET)
#define SRAM_BASE_CH1	(SRAM_BEGIN + CHANNEL_1_OFFSET)
#define SRAM_BASE_CH2	(SRAM_BEGIN + CHANNEL_2_OFFSET)
#define SRAM_BASE_CH3	(SRAM_BEGIN + CHANNEL_3_OFFSET)
#define SDRAM_BASE      SDRAM_BEGIN
#define SCRATCH_BASE    SCRATCH_BEGIN

#define SCRATCH_SIZE    0x4000 /* 16KB */

#define UC_SRAM_BASE_CH0	0x00000000
#define UC_SRAM_BASE_CH1	0x40000000
#define UC_SRAM_BASE_CH2	0x80000000
#define UC_SRAM_BASE_CH3	0xc0000000

#define SRAM_UC_CH0_TO_XS(a) ((a)-UC_SRAM_BASE_CH0+SRAM_BASE_CH0)
#define SRAM_UC_CH1_TO_XS(a) ((a)-UC_SRAM_BASE_CH1+SRAM_BASE_CH1)
#define SRAM_UC_CH2_TO_XS(a) ((a)-UC_SRAM_BASE_CH2+SRAM_BASE_CH2)
#define SRAM_UC_CH3_TO_XS(a) ((a)-UC_SRAM_BASE_CH3+SRAM_BASE_CH3)


#define	SRAM_WRITE(a,b)		set_sram((a),(b))

#define	SDRAM_WRITE(a,b)	set_dram((a),(b))


#endif /* XSCALE_INIT_H */











