/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      drv_cfg.c
 *
 * = DESCRIPTION
 *      This file contains functions to initialize drivers for
 *      the oc12_pos_gbeth_2401 application on an IXPD2401 platform.
 *
 * = AUTHOR
 *      Tomasz Madajczak
 *      tomasz.madajczak@intel.com
 *
 * = CHANGE HISTORY
 *      06/24/2003      Created by Tomasz Madajczak
 * ============================================================================
 */
#include <stdio.h>
#include "ixf1104ce_driver_api.h"
#include <memory.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include "pos_atm_ioctl.h"
#include <pos_atm_media_drv.h>


/* defines for driver modes (bit 2:0) */
#define IX_CC_ETH_TX_DRIVER_MODE_FIBER               0x0
#define IX_CC_ETH_TX_DRIVER_MODE_GIGA_HALF_DUPLEX    0x1
#define IX_CC_ETH_TX_DRIVER_MODE_GIGA_FULL_DUPLEX    0x2
#define IX_CC_ETH_TX_DRIVER_MODE_100_HALF_DUPLEX     0x3
#define IX_CC_ETH_TX_DRIVER_MODE_100_FULL_DUPLEX     0x4
#define IX_CC_ETH_TX_DRIVER_MODE_10_HALF_DUPLEX      0x5
#define IX_CC_ETH_TX_DRIVER_MODE_10_FULL_DUPLEX      0x6

/* defines for driver parity (bit 3) */
#define IX_CC_ETH_TX_DRIVER_MODE_PARITY_ODD          0x0
#define IX_CC_ETH_TX_DRIVER_MODE_PARITY_EVEN         0x8

/* defines for driver block modes (bit 4) */
#define IX_CC_ETH_TX_DRIVER_MODE_BLOCK_MPHY          0x00
#define IX_CC_ETH_TX_DRIVER_MODE_BLOCK_4_8           0x10

#define IX_CC_ETH_TX_FRAME_SIZE  0x23FC /* 9216 - 4 */

#define IX_CC_ETH_TX_DRIVER_PORT_MASK_01   0x3
#define IX_CC_ETH_TX_DRIVER_PORT_MASK_23   0xc

/* handle for DB0 DB1 respectively */
int handle[2] = {POS_ATM_INV_HANDLE, POS_ATM_INV_HANDLE};

/*
 * Read Vallejo statistics
 *
 */
void vallejo_stats(unsigned int port, uint32 handle)
{
    gbe_mac_ioctl_ptr_64 stats;
    gbe_mac_ioctl_ptr param;

    if (port != 8) return;

    stats.portIndex = port;
    param.portIndex = port;

    printf("---------------------------------------------------------\n");
    printf("Reading Vallejo statistics for port %d\n", port);
    printf("---------------------------------------------------------\n");

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_OCTETS_OK,
                                (void *)(&stats)))
        return;
    printf("GET_RX_OCTETS_OK         = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_OCTETS_BAD,
                                (void *)(&stats)))
        return;
    printf("GET_RX_OCTETS_BAD        = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_UC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_RX_UC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_MC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_RX_MC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_BC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_RX_BC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_64,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_64           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_65_127,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_65_127       = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_128_255,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_128_255      = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_256_511,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_256_511      = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_512_1023,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_512_1023     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_1024_1518,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_1024_1518    = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_1519_MAX,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_1519_MAX     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FCS_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RX_FCS_ERR           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_DATA_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RX_DATA_ERR          = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_ALLIGN_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RX_ALLIGN_ERR        = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_LONG_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RX_LONG_ERR          = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_JABBER_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RX_JABBER_ERR        = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PAUSE_MAC_CTL,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PAUSE_MAC_CTL     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_UNKNOWN_CTL_FRAME,
                                (void *)(&stats)))
        return;
    printf("GET_RX_UNKNOWN_CTL_FRAME = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_VLONG_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_VLONG_ERR            = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RUNT_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RUNT_ERR             = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_SHORT_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_SHORT_ERR            = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_SEQ_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_SEQ_ERR              = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_SYMBOL_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_SYMBOL_ERR           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    /**
     * TX Statistics Register IOCTL Commands.
     */
    printf("---------------------------------------------------------\n");

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_OCTETS_OK,
                                (void *)(&stats)))
        return;
    printf("GET_TX_OCTETS_OK         = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_OCTETS_BAD,
                                (void *)(&stats)))
        return;
    printf("GET_TX_OCTETS_BAD        = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_UC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_TX_UC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_MC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_TX_MC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_BC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_TX_BC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_64,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_64           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_65_127,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_65_127       = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_128_255,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_128_255      = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_256_511,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_256_511      = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_512_1023,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_512_1023     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_1024_1518,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_1024_1518    = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_1519_MAX,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_1519_MAX     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PAUSE_FRAME,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PAUSE_FRAME       = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_DEFERRED_ERR,
                                (void *)(&stats)))
        return;

    printf("GET_TX_DEFERRED_ERR      = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_TOTAL_COLLISION,
                                (void *)(&stats)))
        return;
    printf("GET_TX_TOTAL_COLLISION   = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_SINGLE_COLLISION,
                                (void *)(&stats)))
        return;
    printf("GET_TX_SINGLE_COLLISION  = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_MUL_COLLISION,
                                (void *)(&stats)))
        return;
    printf("GET_TX_MUL_COLLISION     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_LATE_COLLISION,
                                (void *)(&stats)))
        return;
    printf("GET_LATE_COLLISION       = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_EXCV_COLLISION,
                                (void *)(&stats)))
        return;
    printf("GET_TX_EXCV_COLLISION    = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_EXCV_DEFERRED_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_TX_EXCV_DEFERRED_ERR = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_EXCV_LEN_DROP,
                                (void *)(&stats)))
        return;
    printf("GET_TX_EXCV_LEN_DROP     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_UNDERRUN,
                                (void *)(&stats)))
        return;
    printf("GET_TX_UNDERRUN          = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_CRC_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_TX_CRC_ERR           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_FC_COLLISION_SEND,
                                (void *)(&stats)))
        return;
    printf("GET_FC_COLLISION_SEND    = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    /**
     * RX FIFO Register IOCTL Commands
     */
    printf("---------------------------------------------------------\n");

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_STN_ADDR,
                                (void *)(&stats)))
        return;
    printf("GET_STN_ADDR             = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);
    

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FIFO_HIGH_WATERMARK,
                                (void *)(&param)))
        return;
    printf("GET_RX_FIFO_HIGH_WM      = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FIFO_LOW_WATERMARK,
                                (void *)(&param)))
        return;
    printf("GET_RX_FIFO_LOW_WM       = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FRAME_REMOVED,
                                (void *)(&param)))
        return;
    printf("GET_RX_FRAME_REMOVED     = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FIFO_PORT_RESET,
                                (void *)(&param)))
        return;
    printf("GET_RX_FIFO_PORT_RESET   = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FIFO_FRAME_DROP,
                                (void *)(&param)))
        return;
    printf("GET_RX_FIFO_FRAME_DROP   = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FIFO_OVERFLOW_STT,
                                (void *)(&param)))
        return;
    printf("GET_RX_FIFO_OVERFLOW_STT = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_OUT_SEQUENCE_INFO,
                                (void *)(&param)))
        return;
    printf("GET_OUT_SEQUENCE_INFO    = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_DROPPED_PKTS,
                                (void *)(&param)))
        return;
    printf("GET_DROPPED_PKTS         = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RW_PTR_RX_FIFO,
                                (void *)(&param)))
        return;
    printf("GET_RW_PTR_RX_FIFO       = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_OCCUPANCY_RX_FIFO,
                                (void *)(&param)))
        return;
    printf("GET_OCCUPANCY_RX_FIFO    = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_JUMBO_PKT_SIZE,
                                (void *)(&param)))
        return;
    printf("GET_JUMBO_PKT_SIZE       = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_PKT_DROP_CAP_FIFO,
                                (void *)(&param)))
        return;
    printf("GET_PKT_DROP_CAP_FIFO    = 0x%08x\n",
           param.value);

}

/*****************************************************************************
 * Abstract:
 *    pos_atm_media_drv initialization function.
 *****************************************************************************/
int ix_cc_atmdrv_init(void)
{
    #ifdef ATM_DB0
        if (PosAtm_DriverStart("oc12,atm", "") == POS_ATM_FAILURE)
        {
            printf("\n Error starting POS ATM Media Driver in DB0\n");
            return(POS_ATM_FAILURE);
        }
    #endif

    #ifdef ATM_DB1
        if (PosAtm_DriverStart("", "oc12,atm") == POS_ATM_FAILURE)
        {
            printf("\n Error starting POS ATM Media Driver in DB0\n");
            return(POS_ATM_FAILURE);
        }
    #endif

    #ifdef ATM_DB0_DB1
        if (PosAtm_DriverStart("oc12,atm", "oc12,atm") == POS_ATM_FAILURE)
        {
            printf("\n Error starting POS ATM Media Driver in DB0\n");
            return(POS_ATM_FAILURE);
        }
    #endif

    printf("\n POS ATM Media Driver initialized successfully \n");
    return(0);
}

int ix_cc_atmdrv_linkup(int db, int link)
{
    if ( db != 0 && db != 1) {
        printf("\n Invalid DB number \n");
    }
    else
    {
        if (link != 0 && link != 1) {
            printf("\n Invalid link number \n");
        }
        else {
            if ( handle[db] == POS_ATM_INV_HANDLE) {
                /* the driver has not been opened for this db */
                if ((handle[db] = PosAtm_Open(db)) == POS_ATM_INV_HANDLE) {
                    printf("\n Error opening POS ATM Media Driver for DB %d\n", db);
                    return(0);
                }
            }
            /* lets set line state up for specified link */
            if (PosAtm_SetLinkState(handle[db], link, POS_ATM_STATE_UP) == POS_ATM_SUCCESS) {
                printf("\n POS ATM Media Driver on DB%d Line %d is UP\n", db, link);
            }
            else {
               printf("\n Error setting up the POS ATM Media Driver on DB%d Line %d\n", db, link);
            }
        }
    }
    return(0);
}


int main(
int argc,
char **argv)
{
    uint32 err, i;
    gbe_mac_ioctl_ptr macIoctlInfo;
    gbe_mac_ioctl_ptr_64 macIoctlInfo_64;
    uint32 handle;
    uint32 handle_table[4] = {0,0,0,0};
    
    unsigned int status;
    
    /* driver initialization starts here */
    err = ix_cc_atmdrv_init();
    if (err == POS_ATM_FAILURE)
    {
        printf("drv_cfg: ix_cc_atmdrv_init failed\n");
        return(-1);
    }
#ifdef ATM_DB0
    err = ix_cc_atmdrv_linkup(0, 0);
    if (err == POS_ATM_FAILURE)
    {
        printf("drv_cfg: ix_cc_atmdrv_linkup failed: DB0\n");
        return(-1);
    }    
    err = ix_cc_atmdrv_linkup(0, 1);
    if (err == POS_ATM_FAILURE)
    {
        printf("drv_cfg: ix_cc_atmdrv_linkup failed: DB0\n");
        return(-1);
    }    
#endif

#ifdef ATM_DB1
    err = ix_cc_atmdrv_linkup(1, 0);
    if (err == POS_ATM_FAILURE)
    {
        printf("drv_cfg: ix_cc_atmdrv_linkup failed: DB1\n");
        return(-1);
    }    
    err = ix_cc_atmdrv_linkup(1, 1);
    if (err == POS_ATM_FAILURE)
    {
        printf("drv_cfg: ix_cc_atmdrv_linkup failed: DB1\n");
        return(-1);
    }    
#endif

#ifdef ATM_DB0_DB1
    err = ix_cc_atmdrv_linkup(0, 0);
    if (err == POS_ATM_FAILURE)
    {
        printf("drv_cfg: ix_cc_atmdrv_linkup failed: DB0\n");
        return(-1);
    }   
    err = ix_cc_atmdrv_linkup(0, 1);
    if (err == POS_ATM_FAILURE)
    {
        printf("drv_cfg: ix_cc_atmdrv_linkup failed: DB0\n");
        return(-1);
    }        
    err = ix_cc_atmdrv_linkup(1, 0);
    if (err == POS_ATM_FAILURE)
    {
        printf("drv_cfg: ix_cc_atmdrv_linkup failed: DB1\n");
        return(-1);
    }    

    err = ix_cc_atmdrv_linkup(1, 1);
    if (err == POS_ATM_FAILURE)
    {
        printf("drv_cfg: ix_cc_atmdrv_linkup failed: DB1\n");
        return(-1);
    }           
#endif
    
    printf("ATM interfaces are up and working...\n");
    
    printf("Initializing Giga Ethernet copper ports ...\n");
    
    handle = 0;
    
    status = GbEMAC_DeviceStart((1<<8),
                                &handle,
                                IX_CC_ETH_TX_DRIVER_MODE_GIGA_FULL_DUPLEX |
                                IX_CC_ETH_TX_DRIVER_MODE_PARITY_EVEN |
                                IX_CC_ETH_TX_DRIVER_MODE_BLOCK_4_8);

    printf("\n*** GbEMAC_DeviceStart() ports: status: %0X ***\n", status);

    /*
     * For all ports set:
     * - the JUMBO MTU size
     * - enable Flow Control capabilities
     * - set RX_FIFO_LOW_WATER_MARK
     * - set RX_FIFO_HIGH_WATER_MARK
     */

    printf("\n*** Configuration of copper ports ***\n");
    for (i = 8; i < 9; i++)
    {
        /* Set promiscous mode in driver */
        macIoctlInfo.portIndex = i;
        macIoctlInfo.value = 0x20;
        err = GbEMAC_Ioctl(&handle,
                           SET_PKT_FILTER_CTL,
                           (void *)(&macIoctlInfo));
        if(err != SUCCESS)
        {
            printf("GbEMAC_Ioctl (SET_PKT_FILTER_CTL) returned error = %d\n",err);
            return 1;
        }/* end if */
        printf("Promiscous mode enabled enabled...\n");

        macIoctlInfo_64.portIndex = i;
        macIoctlInfo_64.valueHigh = 0x00000201;
        macIoctlInfo_64.valueLow = 0x02030001 + i;
        err = GbEMAC_Ioctl(&handle,
                           SET_STN_ADDR,
                           (void *)(&macIoctlInfo_64));
        if (err != SUCCESS)
        {
            printf("GbEMAC_Ioctl (SET_STN_ADDR) returned error = %d\n",err);
            return 1;
        }

        printf("MAC ADDR for port %d set\n",i);
        fflush(stdout);

        macIoctlInfo.portIndex = i;
        macIoctlInfo.value = IX_CC_ETH_TX_FRAME_SIZE;
        err = GbEMAC_Ioctl(&handle,
                           SET_MAX_FRAME_SIZE,
                           (void *)(&macIoctlInfo));
        if (err != SUCCESS)
        {
            printf("GbEMAC_Ioctl (SET_MAX_FRAME_SIZE)"
                   " returned error = %d\n",err);
            return 1;
        }

        printf("Frame size for port %d set\n",i);
        fflush(stdout);
            
        /*
         * The approximate value of high water mark is calculated according to
         * the following algorithm:
         * (MAC_FIFO_SIZE - 2 * IX_CC_ETH_TX_FRAME_SIZE) / 8
         */
        macIoctlInfo.value = 0x700;/* high water mark 0x700 8 bytes locations */
        err = GbEMAC_Ioctl(&handle,
                           SET_RFIFO_HIGH_WATERMARK,
                           (void *)(&macIoctlInfo));
        if (err != SUCCESS)
        {
            printf("GbEMAC_Ioctl (SET_RFIFO_HIGH_WATERMARK)"
                   " returned error = %d\n",err);
            return 1;
        }

        printf("High watermark for port %d set\n",i);
        fflush(stdout);

        macIoctlInfo.value = 0x300;/* low water mark 0x300 8 bytes locations */
        err = GbEMAC_Ioctl(&handle,
                           SET_RFIFO_LOW_WATERMARK,
                           (void *)(&macIoctlInfo));
        if (err != SUCCESS)
        {
            printf("GbEMAC_Ioctl (SET_RFIFO_LOW_WATERMARK)"
                   " returned error = %d\n",err);
            return 1;
        }

        printf("Low watermark for port %d set\n",i);
        fflush(stdout);
            
        macIoctlInfo.value = 0x7; /* enable flow control capabilities */
        err = GbEMAC_Ioctl(&handle,
                           SET_FC_MODE,
                           (void *)(&macIoctlInfo));
        if (err != SUCCESS)
        {
            printf("GbEMAC_Ioctl (SET_FC_MODE) returned error = %d\n",err);
            return 1;
        }

        printf("Flow control for port %d set\n",i);
        fflush(stdout);

    }/* end for */
    
    printf("Copper ports initialization COMPLETE. Handle = %d\n",handle);

    handle_table[8] = handle;
 
    while (1)
    {
        char c;
        
        c = getchar();
        switch (c)
        {
        case 's':
        case 'S':
            vallejo_stats(8, handle_table[8]);
            break;

        case 'q':
        case 'Q':
            return 0;
                
        }
    }
    return 0;
}












