/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      ethernet_rtm_routes.c
 *
 * = DESCRIPTION
 *      This file contains scripts to initialize the oc12_pos_gbeth_2401
 *      application on an IXPD2401 platform.
 *
 * = AUTHOR
 *      Tomasz Madajczak
 *      tomasz.madajczak@intel.com
 *
 * = CHANGE HISTORY
 *      01/22/2003      Adopted from a POS file by Tomasz Madajczak
 * ============================================================================
 */

#include "../ucode/dl_mem_map.h"
#include "xscale_init.h"

int rtm_routes() {

int trie_base = SRAM_BASE_CH0 + TRIE_TABLE_SRAM_BASE;

int nh_base = SDRAM_BASE + NEXTHOP_TABLE_SDRAM_BASE;


SDRAM_WRITE(nh_base + 0x0,0xffffffff);

/* #
   # This rtm_routes is used for testing IPv4 PR-5 onwards.
   #
 */

 /* #-----------------------------------------------
    # Limited broadcast - exception
    #-----------------------------------------------
 */

/* add 255.255.255.255 255.255.255.255 0x7255 0x8 1500 0x02000000
 */
/* @@@ Adding dest=0xffffffff, mask = 0xffffffff, blade_nh_id=0x2007255,mtu_port=0x05dc0000,flags=0x8 */

	SRAM_WRITE(trie_base + 0x4053c,0x10000);

/* writing nexthop entry with flags 0x8 */

SDRAM_WRITE(nh_base + 0x10,0x8);

/* blade 0x2 and NH 0x7255 */

SDRAM_WRITE(nh_base + 0x14,0x2007255);

/* MTU is 1500, outport is 0x0 */

SDRAM_WRITE(nh_base + 0x18,0x05dc0000);


SDRAM_WRITE(nh_base + 0x1c,0x0);


	SRAM_WRITE(trie_base + 0x404fc,0x4);


	SRAM_WRITE(trie_base + 0x404bc,0x3);


	SRAM_WRITE(trie_base + 0x4047c,0x2);


	SRAM_WRITE(trie_base + 0x3fffc,0x1);

/* #-----------------------------------------------
   #  Invalid destination addresses
   #----------------------------------------------
 */

/* # dst==0 - drop
 */

/* add 0.0.0.0 255.255.255.255 0x7000 0x4 1500 0x02000000
 */
/* @@@ Adding dest=0x0, mask = 0xffffffff, blade_nh_id=0x2007000,mtu_port=0x05dc0000,flags=0x4 */

	SRAM_WRITE(trie_base + 0x40600,0x20000);

/* writing nexthop entry with flags 0x4 */

SDRAM_WRITE(nh_base + 0x20,0x4);

/* blade 0x2 and NH 0x7000 */

SDRAM_WRITE(nh_base + 0x24,0x2007000);

/* MTU is 1500, outport is 0x0 */

SDRAM_WRITE(nh_base + 0x28,0x05dc0000);


SDRAM_WRITE(nh_base + 0x2c,0x0);


	SRAM_WRITE(trie_base + 0x405c0,0x8);


	SRAM_WRITE(trie_base + 0x40580,0x7);


	SRAM_WRITE(trie_base + 0x40540,0x6);


	SRAM_WRITE(trie_base + 0x0,0x5);

/* # dst==127.*.*.* - drop
 */

/* add 127.0.0.0 255.0.0.0 0x7127 0x4 1500 0x03000000
 */
/* @@@ Adding dest=0x7f000000, mask = 0xff000000, blade_nh_id=0x3007127,mtu_port=0x05dc0000,flags=0x4 */

	SRAM_WRITE(trie_base + 0x401fc,0x30000);

/* writing nexthop entry with flags 0x4 */

SDRAM_WRITE(nh_base + 0x30,0x4);

/* blade 0x3 and NH 0x7127 */

SDRAM_WRITE(nh_base + 0x34,0x3007127);

/* MTU is 1500, outport is 0x0 */

SDRAM_WRITE(nh_base + 0x38,0x05dc0000);


SDRAM_WRITE(nh_base + 0x3c,0x0);

/* # dst==classE - drop
 */
/* add 240.0.0.0 240.0.0.0 0x7740 0x4 1500 0x04000000
 */
/* @@@ Adding dest=0xf0000000, mask = 0xf0000000, blade_nh_id=0x4007240,mtu_port=0x05dc0000,flags=0x4 */

	SRAM_WRITE(trie_base + 0x403c0,0x40000);


	SRAM_WRITE(trie_base + 0x403c4,0x40000);


	SRAM_WRITE(trie_base + 0x403c8,0x40000);


	SRAM_WRITE(trie_base + 0x403cc,0x40000);


	SRAM_WRITE(trie_base + 0x403d0,0x40000);


	SRAM_WRITE(trie_base + 0x403d4,0x40000);


	SRAM_WRITE(trie_base + 0x403d8,0x40000);


	SRAM_WRITE(trie_base + 0x403dc,0x40000);


	SRAM_WRITE(trie_base + 0x403e0,0x40000);


	SRAM_WRITE(trie_base + 0x403e4,0x40000);


	SRAM_WRITE(trie_base + 0x403e8,0x40000);


	SRAM_WRITE(trie_base + 0x403ec,0x40000);


	SRAM_WRITE(trie_base + 0x403f0,0x40000);


	SRAM_WRITE(trie_base + 0x403f4,0x40000);


	SRAM_WRITE(trie_base + 0x403f8,0x40000);


	SRAM_WRITE(trie_base + 0x403fc,0x40000);

/* writing nexthop entry with flags 0x4 */ 

SDRAM_WRITE(nh_base + 0x40,0x4);

/* blade 0x4 and NH 0x7240 */

SDRAM_WRITE(nh_base + 0x44,0x4007240);

/* MTU is 1500, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x48,0x05dc0000);


SDRAM_WRITE(nh_base + 0x4c,0x0);

/* # dest==multicast - exception
 */ 
/* add 224.0.0.0 224.0.0.0 0x7224 0x10 1500 0x05000000
 */
/* @@@ Adding dest=0xe0000000, mask = 0xe0000000, blade_nh_id=0x5007224,mtu_port=0x05dc0000,flags=0x10 */

	SRAM_WRITE(trie_base + 0x40380,0x50000);


	SRAM_WRITE(trie_base + 0x40384,0x50000);


	SRAM_WRITE(trie_base + 0x40388,0x50000);


	SRAM_WRITE(trie_base + 0x4038c,0x50000);


	SRAM_WRITE(trie_base + 0x40390,0x50000);


	SRAM_WRITE(trie_base + 0x40394,0x50000);


	SRAM_WRITE(trie_base + 0x40398,0x50000);


	SRAM_WRITE(trie_base + 0x4039c,0x50000);


	SRAM_WRITE(trie_base + 0x403a0,0x50000);


	SRAM_WRITE(trie_base + 0x403a4,0x50000);


	SRAM_WRITE(trie_base + 0x403a8,0x50000);


	SRAM_WRITE(trie_base + 0x403ac,0x50000);


	SRAM_WRITE(trie_base + 0x403b0,0x50000);


	SRAM_WRITE(trie_base + 0x403b4,0x50000);


	SRAM_WRITE(trie_base + 0x403b8,0x50000);


	SRAM_WRITE(trie_base + 0x403bc,0x50000);

/* writing nexthop entry with flags 0x10 */ 

SDRAM_WRITE(nh_base + 0x50,0x10);

/* blade 0x5 and NH 0x7224 */

SDRAM_WRITE(nh_base + 0x54,0x5007224);

/* MTU is 1500, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x58,0x05dc0000);


SDRAM_WRITE(nh_base + 0x5c,0x0);


/* 
 *//* 
 *//* 
 *//* #-----------------------------------------------
 */ /* # 16-bit routes - Forward			 
 */ /* #-----------------------------------------------
 */ /* 
 *//*  add 16.1.0.0 255.255.0.0 0xfffe 0x0 1500 0x00000000
 */
/* @@@ Adding dest=0x10010000, mask = 0xffff0000, blade_nh_id=0x0101,mtu_port=0x05dc0000,flags=0x0 */

	SRAM_WRITE(trie_base + 0x40a04,0x4c0000);

/* writing nexthop entry with flags 0x0 */ 

SDRAM_WRITE(nh_base + 0x4c0,0x0);

/* blade 0x0 and NH 0xfffe */

SDRAM_WRITE(nh_base + 0x4c4,0xfffe);

/* MTU is 1500, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x4c8,0x05dc0000);


SDRAM_WRITE(nh_base + 0x4cc,0x0);


	SRAM_WRITE(trie_base + 0x409c0,0x18);


	SRAM_WRITE(trie_base + 0x40040,0x17);

/*  add 16.2.0.0 255.255.0.0 0xfffe 0x1 1500 0x02000000
 */
/* @@@ Adding dest=0x10020000, mask = 0xffff0000, blade_nh_id=0x2000102,mtu_port=0x05dc0001,flags=0x0 */

	SRAM_WRITE(trie_base + 0x40a08,0x4d0000);

/* writing nexthop entry with flags 0x0 */ 

SDRAM_WRITE(nh_base + 0x4d0,0x0);

/* blade 0x2 and NH 0xfffe */

SDRAM_WRITE(nh_base + 0x4d4,0x200fffe);

/* MTU is 1500, outport is 0x1 */ 

SDRAM_WRITE(nh_base + 0x4d8,0x05dc0001);


SDRAM_WRITE(nh_base + 0x4dc,0x0);

/*  add 16.3.0.0 255.255.0.0 0x0103 0x2 1500 0x03000000
 */
/* @@@ Adding dest=0x10030000, mask = 0xffff0000, blade_nh_id=0x3000103,mtu_port=0x05dc0002,flags=0x0 */

	SRAM_WRITE(trie_base + 0x40a0c,0x4e0000);

/* writing nexthop entry with flags 0x0 */ 

SDRAM_WRITE(nh_base + 0x4e0,0x0);

/* blade 0x3 and NH 0x0103 */

SDRAM_WRITE(nh_base + 0x4e4,0x3000103);

/* MTU is 1500, outport is 0x2 */ 

SDRAM_WRITE(nh_base + 0x4e8,0x05dc0002);


SDRAM_WRITE(nh_base + 0x4ec,0x0);

/*  add 16.4.0.0 255.255.0.0 0x0104 0x3 1500 0x04000000
 */
/* @@@ Adding dest=0x10040000, mask = 0xffff0000, blade_nh_id=0x4000104,mtu_port=0x05dc0003,flags=0x0 */

	SRAM_WRITE(trie_base + 0x40a10,0x4f0000);

/* writing nexthop entry with flags 0x0 */ 

SDRAM_WRITE(nh_base + 0x4f0,0x0);

/* blade 0x4 and NH 0x0104 */

SDRAM_WRITE(nh_base + 0x4f4,0x4000104);

/* MTU is 1500, outport is 0x3 */ 

SDRAM_WRITE(nh_base + 0x4f8,0x05dc0003);


SDRAM_WRITE(nh_base + 0x4fc,0x0);



return 1;


}
