/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      l2_encap_init.c
 *
 * = DESCRIPTION
 *      This file contains scripts to initialize the quad_gbeth_jamestown
 *      application on an IXPD2401 platform.
 *
 * = AUTHOR
 *      Tomasz Madajczak
 *      tomasz.madajczak@intel.com
 *
 * = CHANGE HISTORY
 *      01/22/2003      Adopted from a POS file by Tomasz Madajczak
 * ============================================================================
 */

#include "../ucode/dl_system.h"
#include "xscale_init.h"

#define _L2_TABLE_SRAM_BASE (SRAM_BASE_CH0 + L2_TABLE_SRAM_BASE)

/*#define L2_TABLE_NUM_OF_ENTRIES	65536*/
#define L2_TABLE_NUM_OF_ENTRIES	(L2_TABLE_SRAM_SIZE/32)

/**
 * NAME: l2_encap_init_addresses
 *
 * DESCRIPTION: This function configures L2 addresses in the L2 table
 */
int l2_encap_init_addresses(void)
{

    int i;

	printf("l2_encap_init_addresse(): Configuring L2 Table addresses...\n");

    /* we need a lot of L2 static addreses (values of them are meaningless)*/
    for (i = 0; i < 0x100; i++)
    {

        /* setup L2_table entry */
        /* entry i */
        set_sram(_L2_TABLE_SRAM_BASE + 0x000 + (i<<4), 0x800090d7);
        set_sram(_L2_TABLE_SRAM_BASE + 0x004 + (i<<4), 0x00080000 + (i<<8));
        set_sram(_L2_TABLE_SRAM_BASE + 0x008 + (i<<4), 0x90d80008);
        set_sram(_L2_TABLE_SRAM_BASE + 0x00c + (i<<4), 0x00080000);

    }
    set_sram(_L2_TABLE_SRAM_BASE + 0x000 + (0x103<<4), 0x800090d7);
    set_sram(_L2_TABLE_SRAM_BASE + 0x004 + (0x103<<4), 0x00080302);
    set_sram(_L2_TABLE_SRAM_BASE + 0x008 + (0x103<<4), 0x01020300);
    set_sram(_L2_TABLE_SRAM_BASE + 0x00c + (0x103<<4), 0x03080000);

    set_sram(_L2_TABLE_SRAM_BASE + 0x000 + (0x104<<4), 0x800090d7);
    set_sram(_L2_TABLE_SRAM_BASE + 0x004 + (0x104<<4), 0x00080402);
    set_sram(_L2_TABLE_SRAM_BASE + 0x008 + (0x104<<4), 0x01020300);
    set_sram(_L2_TABLE_SRAM_BASE + 0x00c + (0x104<<4), 0x04080000);


	printf("L2 Table configuration COMPLETE.\n\n");

	return 1;
}




