/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      oc12_pos_gbeth_2401_linux_init.c
 *
 * = DESCRIPTION
 *      This file contains functions to initialize the oc12_pos_gbeth_2401
 *      application on an IXPD2401 platform.
 *
 * = AUTHOR
 *      Tomasz Madajczak
 *      tomasz.madajczak@intel.com
 *
 * = CHANGE HISTORY
 *      06/24/2003      Created by Tomasz Madajczak
 * ============================================================================
 */
#include <stdio.h>
#include <memory.h>
#include <string.h>

#include "xscale_init.h"
#include "linux_mmap_lib.h"
#include "dl_system.h"
#include "dl_mem_map.h"

int THmask, LR, TQoffset, i;
int global=0;

void set_hash_table(int vpi_vci, int ports, int rxc_no, int L2port);
int crc32(int data1, int data3);
int aal5_tx_init(void);
int rtmInit(void);
int l3CreateRoutes(void);
int llc_encap_init(void);
int dbcast_init(void);
void packetQmInit(void);
int l2_encap_script_init(void);
int cellQmInit(void);
int atm_tm_init(void);
int ipv4ClearCounters(void);
int aal5_rx_port_stat_counters_init_zero(void);
int aal5_rx_vc_info_init_zero(void);
int aal5_rx_pri_init_zero(void);
int aal5_rx_sec_init_zero(void);
int aal5_rx_hash_rxc_init(void);

#include "rtm_init.ind"
#include "rtm_routes.ind"
#include "llc_encap.ind"
#include "dbcast_init.ind"
#include "PacketQmInit.ind"
#include "l2_encap_init.ind"
#include "qm_cell_init.ind"
#include "atm_tm_init.ind"
#include "aal5_rx_init.ind"
#include "aal5_rx_hash_table_init_my.ind"
#include "aal5_tx_init.ind"

int log2(x)
{
    return (log(x)/log(10));
}

#define _CONTROL_BLOCK_SRAM_BASE (SRAM_BASE_CH0 + CONTROL_BLOCK_SRAM_BASE)

void oc12_atm_gbeth_2401_setup(void)
{

    printf("Clearing scratch ...\n");
    init_scratch(0x0, SCRATCH_BASE, SCRATCH_BASE + SCRATCH_SIZE);

    printf("Clearing LLC ENCAP table table ...\n");
    llc_encap_init();

    printf("Clearing route table ...\n");
    rtmInit();

    printf("Initializing with some routes ...\n");
    l3CreateRoutes();

    printf("Clearing counters in SRAM ...\n");
    ipv4ClearCounters();

    printf("Initializing the directed broadcast table ...\n");
    dbcast_init();

    //
    //  here add dbast entries using addresses from rtm_routes.ind
    //  file (take the destination IP address for which the 'LOCAL'
    //  flag is set, and then add corresponding network broadcast
    //  address for that network in the table).
    //

    dbcast_add(0x8001ffff);
    dbcast_add(0x8002ffff);
    dbcast_add(0x8003ffff);
    dbcast_add(0x8004ffff);
    dbcast_add(0x8005ffff);
    dbcast_add(0x8006ffff);

    // here, set up the control block to enable only 0-31 ports
    // for forwarding
    set_sram(CONTROL_BLOCK_SRAM_BASE,0xffffffff);
    set_scratch(CONTROL_BLOCK_SCRATCH_BASE,0xffffffff);

    //-----------------------------------------------------
    //	Upstream QM setup
    //-----------------------------------------------------
    packetQmInit();

    //-----------------------------------------------------
    //	L2 Encap stuff
    //-----------------------------------------------------
    // initialize memory required for L2 Table for L2_ENCAP
    l2_encap_script_init();

    //-----------------------------------------------------
    //	Downstream QM setup
    //-----------------------------------------------------

    cellQmInit();

    //----------------------
    //     ATM TM
    //----------------------

    atm_tm_init();

    THmask = 0;
    LR = 0;
    TQoffset = 0;

#ifdef NQL_CONFIGURATION

    printf("NQL Configuration...\n");

    /* ********** Port 0 (0) ********** */
    for(i=0;i<4;i++)
    {
        atm_tm_add_hbr_port(0,i);
    }
/*    atm_tm_add_port(0, 1, 1);*/

    /* HBR VC - LLC SNAP VC */
    atm_tm_add_hbr_vc(0x1,0);
    llc_encap_set_next_hop_entry(0x01, 0x01, (0x01 | (0x0020<<12)), 0xaaaa0300, 0x00000800);

    /* LBR VC - VC MUX VC */
/*    atm_tm_add_vc(193, 0, 0, 0, 0, 0, 0, 0); // UBR with priority
    llc_encap_set_next_hop_entry(193, 193, (0x02 | (0x0001<<12)), 0xFFFFFFFF, 0xFFFFFFFF);*/

    for(i=0; i<=HBR_TQ_SRAM_SIZE; i+=4)
    {
        set_sram(HBR_TQ_SRAM_BASE+i,      0x01010101);
    }

#endif

#ifdef CONFIGURATION1
    printf("Configuration 1 for virtual ports and VCs ...\n");
    /* ********** Port 0 (0) ********** */
    for(i=0;i<2048;i+=8)
    {
        atm_tm_add_hbr_port(0,i);
    }


    atm_tm_add_hbr_vc(0x20,0);
    llc_encap_set_next_hop_entry(0x20, 0x20, (0x02 | (0x0001<<12)), 0xFFFFFFFF, 0xFFFFFFFF);

    /* ********** Port 1 (1) ********** */
    for(i=1;i<2048;i+=8)
    {
        atm_tm_add_hbr_port(1,i);
    }

    atm_tm_add_hbr_vc(0x21,1);
    llc_encap_set_next_hop_entry(0x21, 0x21, (0x02 | (0x0002<<12)), 0xFFFFFFFF, 0xFFFFFFFF);

    /* ********** Port 2 (2) ********** */
    for(i=2;i<2048;i+=8)
    {
        atm_tm_add_hbr_port(2,i);
    }

    atm_tm_add_hbr_vc(0x22,2);
    llc_encap_set_next_hop_entry(0x22, 0x22, (0x02 | (0x0003<<12)), 0xFFFFFFFF, 0xFFFFFFFF);

    /* ********** Port 3 (3) ********** */
    for(i=3;i<2048;i+=8)
    {
        atm_tm_add_hbr_port(3,i);
    }
    
    atm_tm_add_hbr_vc(0x23,3);
    llc_encap_set_next_hop_entry(0x23, 0x23, (0x02 | (0x0004<<12)), 0xFFFFFFFF, 0xFFFFFFFF);
    
    /* ********** Port 4 (4) ********** */
    for(i=4;i<2048;i+=8)
    {
        atm_tm_add_hbr_port(4,i);
    }
    
    atm_tm_add_hbr_vc(0x24,4);
    llc_encap_set_next_hop_entry(0x24, 0x24, (0x02 | (0x0005<<12)), 0xFFFFFFFF, 0xFFFFFFFF);
    
    /* ********** Port 5 (5) ********** */
    for(i=5;i<2048;i+=8)
    {
        atm_tm_add_hbr_port(5,i);
    }
        
    atm_tm_add_hbr_vc(0x25,5);
    llc_encap_set_next_hop_entry(0x25, 0x25, (0x02 | (0x0006<<12)), 0xFFFFFFFF, 0xFFFFFFFF);
    
    /* ********** Port 6 (6) ********** */
    for(i=6;i<2048;i+=8)
    {
        atm_tm_add_hbr_port(6,i);
    }
    
    atm_tm_add_hbr_vc(0x26,6);
    llc_encap_set_next_hop_entry(0x26, 0x26, (0x02 | (0x0007<<12)), 0xFFFFFFFF, 0xFFFFFFFF);
    
    /* ********** Port 7 (7) ********** */
    for(i=7;i<2048;i+=8)
    {
        atm_tm_add_hbr_port(7,i);
    }
    
    atm_tm_add_hbr_vc(0x27,7);
    llc_encap_set_next_hop_entry(0x27, 0x27, (0x02 | (0x0008<<12)), 0xFFFFFFFF, 0xFFFFFFFF);
    
    for(i=0; i<HBR_TQ_SRAM_SIZE; i+=16)
    {
        set_sram(HBR_TQ_SRAM_BASE,        0x23222120);
        set_sram(HBR_TQ_SRAM_BASE+i+4,    0x27262524);
        set_sram(HBR_TQ_SRAM_BASE+i+8,    0x23222120);
        set_sram(HBR_TQ_SRAM_BASE+i+12,   0x27262524);
    }
    
    printf("... Configuration 1 for virtual ports and VCs\n");
    
#endif
    
#ifdef CONFIGURATION2
    printf("Configuration 2 for virtual ports and VCs ...\n");
    
    atm_tm_add_port(0, 2048, 256);
    atm_tm_add_vc(193, 0, 0, 0, 0, 0, 0, 0); // UBR with priority
    llc_encap_set_next_hop_entry(193, 193, (0x02 | (0x0001<<12)), 0xFFFFFFFF, 0xFFFFFFFF);
    
    atm_tm_add_port(1, 2048, 256);
    atm_tm_add_vc(194, 1, 4, 8*424, 0, 0, 0, 0); // UBR with PCR
    llc_encap_set_next_hop_entry(194, 194, (0x02 | (0x0002<<12)), 0xFFFFFFFF, 0xFFFFFFFF);
    
    atm_tm_add_port(2, 2048, 256);
    atm_tm_add_vc(195, 2, 1, 8*424, 0, 0, 0, 0); // CBR
    llc_encap_set_next_hop_entry(195, 195, (0x02 | (0x0003<<12)), 0xFFFFFFFF, 0xFFFFFFFF);
    
    atm_tm_add_port(3, 2048, 256);
    atm_tm_add_vc(196, 3, 5, 8*424, 0, 0, 0, 0); // UBR with MDCR
    llc_encap_set_next_hop_entry(196, 196, (0x02 | (0x0004<<12)), 0xFFFFFFFF, 0xFFFFFFFF);
    
    atm_tm_add_port(4, 2048, 256);
    atm_tm_add_vc(197, 4, 3, 8*424, 0, 10*424, 0, 0); // nrt-VBR
    llc_encap_set_next_hop_entry(197, 197, (0x02 | (0x0005<<12)), 0xFFFFFFFF, 0xFFFFFFFF);
    
    atm_tm_add_port(5, 2048, 256);
    atm_tm_add_vc(198, 5, 6, 8*424, 0, 0, 0, 0); // GFR
    llc_encap_set_next_hop_entry(198, 198, (0x02 | (0x0006<<12)), 0xFFFFFFFF, 0xFFFFFFFF);
    
    atm_tm_add_port(6, 2048, 256);
    atm_tm_add_vc(199, 6, 1, 32*424, 0, 0, 0, 0); // CBR
    llc_encap_set_next_hop_entry(199, 199, (0x02 | (0x0007<<12)), 0xFFFFFFFF, 0xFFFFFFFF);
    
    atm_tm_add_port(7, 2048, 256);
    atm_tm_add_vc(200, 7, 2, 32*424, 0, 32*424, 0, 0); // rt-VBR
    llc_encap_set_next_hop_entry(200, 200, (0x02 | (0x0008<<12)), 0xFFFFFFFF, 0xFFFFFFFF);
    
    printf("... Configuration 2 for virtual ports and VCs\n");
#endif
    
    printf("\nHBR_TQ_SRAM_BASE:0x%08X\n",HBR_TQ_SRAM_BASE);
    
#if 0
    //-----------------------------------------------------
    // ETH PHY Port to L2 Port mapping */
    //-----------------------------------------------------
    set_eth_l2_port(1);
#endif
    

    //-----------------------------------------------------
    //      AAL5 RX - VC INFO Init (has RXC and VC STAT counters)
    //-----------------------------------------------------
    
    printf("Initializing VC info table(RXC and Rx VC Stat counters) to zero ...\n");
    aal5_rx_vc_info_init_zero();

    //-----------------------------------------------------
    // AAL5 RX - Port STAT COUNTERS Init
    //-----------------------------------------------------
    
    printf("Initializing Rx Port Stat counters to zero ...\n");
    aal5_rx_port_stat_counters_init_zero();

    //-----------------------------------------------------
    // AAL5 RX - HASH TABLE and RXC Init
    //-----------------------------------------------------
    
    printf("Initializing Pri & Sec hash tables to zero ...\n");
    aal5_rx_pri_init_zero();
    aal5_rx_sec_init_zero();
    printf("Setting up RXC, Pri & Sec hash tables ...\n");
    aal5_rx_hash_rxc_init();

    //-----------------------------------------------------
    //	Freelist initialisation
    //-----------------------------------------------------
    freelist_init(UPSTREAM_TEMPORARY_FREELIST_QDESC, UPSTREAM_BUF_LW0_SRAM_BASE, META_DATA_SIZE, NUMBER_OF_BUFFERS, UPSTREAM_BUF_LW0_CHANNEL_NUMBER);
    
    freelist_init(DOWNSTREAM_TEMPORARY_FREELIST_QDESC, DOWNSTREAM_BUF_LW0_SRAM_BASE, META_DATA_SIZE, NUMBER_OF_BUFFERS, DOWNSTREAM_BUF_LW0_CHANNEL_NUMBER);

    //-----------------------------------------------------
    //	L2 Table initialisation
    //-----------------------------------------------------
    //        L2TableInit();
    
    //-----------------------------------------------------
    //      AAL5 TX - Init
    //-----------------------------------------------------
    aal5_tx_init();
    
    printf("\n**********************************************\n");
    printf("\n     ATM-IPV4-ETH System setup COMPLETE.      \n");
    printf("\n**********************************************\n");

    return;
}


int main(int argc, char **argv)
{

    printf("OC12 ATM Gbeth system Linux Setup\n");

    printf("Initializing IND library.\n");
    ind_lib_start();

    printf("Initializing system.\n");

    oc12_atm_gbeth_2401_setup();    

    printf("Finishing IND library.\n");
    
    ind_lib_end();

    return 0;
}
