#ifndef	DL_MEM_MAP_H
#define	DL_MEM_MAP_H

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 2003 By Intel Corporation.  All rights reserved.
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanpathical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

/* FILE  dl_mem_map.h
 *
 * This file contains memory map customised for IXP2400 hardware/application.
 */

/*******************************************************************************
 *                           INCLUDE SECTION
 *******************************************************************************/
#include <dl_system.h>
#include <atm_tm_defs.h>
#include <aal5_rx_defs.h>
#include <aal5_tx_defs.h>
#include <qm_atm_defs.h>
#include <qm_shaper_defs.h>

/*------------------------------------------------------------------------------
 * SRAM memory controlers base addresses
 */
#ifndef XSCALE_INIT_FOR_LINUX
#define	SRAM_MEMORY_CHANNEL_0_BASE		(0x00000000 + 16*4)
#define	SRAM_MEMORY_CHANNEL_1_BASE		(0x40000000 + 16*4)
#else
#define	SRAM_MEMORY_CHANNEL_0_BASE		(SRAM_BASE_CH0 + 16*4)
#define	SRAM_MEMORY_CHANNEL_1_BASE		(SRAM_BASE_CH1 + 16*4)
#endif

/*------------------------------------------------------------------------------
 * DRAM BUFEERS definitions
 */

/* Base address in DRAM for the the actual packet buffer, starts at the begining of the DRAM memory */
#define	UPSTREAM_BUF_SDRAM_BASE			0x00

/* It is next to one defined for upstream */
#define	DOWNSTREAM_BUF_SDRAM_BASE		(UPSTREAM_BUF_SDRAM_BASE + BUF_SDRAM_SIZE)


/* Port info used by atm tm 4.1 scheduler */
#define	PORTINFO_SDRAM_BASE				(DOWNSTREAM_BUF_SDRAM_BASE + BUF_SDRAM_SIZE)


/*==============================================================================
 * 							SRAM MEMORY CHANNEL 0
 *
 *------------------------------------------------------------------------------
 */


/*------------------------------------------------------------------------------
 * META DATA TABLE in SRAM - definitions
 */

/*
 *	Base address in SRAM for the meta data (buffer descriptors) in upstream direction.
 */

/* Two Buffer Descriptor lists are handled in system (distribute upstream and
   downstream traffic) */

#define	UPSTREAM_BUF_LW0_SRAM_BASE			SRAM_MEMORY_CHANNEL_0_BASE
#define	UPSTREAM_BUF_LW0_CHANNEL_NUMBER		0

/*------------------------------------------------------------------------------
 * IPv4 Forwarder definitions
 */

#define NEXTHOP_TABLE_SRAM_BASE			(UPSTREAM_BUF_LW0_SRAM_BASE + BUF_SRAM_SIZE)

#define	NEXTHOP_TABLE_SIZE 				(NEXTHOP_TABLE_NUMBER_OF_ENTRIES * NEXTHOP_TABLE_ENTRY_SIZE)

/*------------------------------------------------------------------------------
 * LLC ENCAP TABLE
 */
#define	NEXT_HOP_TABLE_SRAM_BASE 	(NEXTHOP_TABLE_SRAM_BASE + NEXTHOP_TABLE_SIZE)
#define	NEXT_HOP_TABLE_SRAM_SIZE 	(NEXT_HOP_ENTRY_SIZE * NUMBER_OF_NEXT_HOP_ENTRIES)

/*------------------------------------------------------------------------------
 * QUEUE definitions
 */

/*
 *	Base Address for Queue Descriptors.
 */
#define UPSTREAM_QD_SRAM_BASE 		(NEXT_HOP_TABLE_SRAM_BASE + NEXT_HOP_TABLE_SRAM_SIZE)
#define UPSTREAM_QD_SRAM_SIZE 		(UPSTREAM_QD_TOTAL * UPSTREAM_QD_SIZE)

/*
 *	SRAM Controler used by the Upstream Queue Manager
 */
#define UPSTREAM_QD_SRAM_CHANNEL_BASE_ADDRESS		UPSTREAM_BUF_LW0_SRAM_BASE

/*------------------------------------------------------------------------------
 * Packet RX  definitions
 *
 *	Base for POS RX counters
 */
#define	PACKET_COUNTERS_SRAM_BASE		(UPSTREAM_QD_SRAM_BASE + UPSTREAM_QD_SRAM_SIZE)

/*
 *	space allocated to maintain these counters
 */
#define STATS_TABLE_SRAM_SIZE			0x800

/*-----------------------------------------------------------------------------
 * AAL5 RX defines
 */

/*----------------------------------------------------------------------------*/
/* Port STAT Table (counters)												  */
/*----------------------------------------------------------------------------*/
#define AAL5_RX_PORT_STAT_SRAM_BASE		(PACKET_COUNTERS_SRAM_BASE + STATS_TABLE_SRAM_SIZE)
#define AAL5_RX_PORT_STAT_TABLE_SIZE	(UPSTREAM_RX_MAX_PORTS * AAL5_RX_PORT_STAT_ENTRY_SIZE)

/*-----------------------------------------------------------------------------*/
/* VC INFO TABLE,(RXC table & VC STAT table (counters)  					   */
/*-----------------------------------------------------------------------------*/
#define AAL5_RX_VC_INFO_SRAM_BASE		(AAL5_RX_PORT_STAT_SRAM_BASE + AAL5_RX_PORT_STAT_TABLE_SIZE)
#define AAL5_RX_VC_INFO_TABLE_SIZE  	(UPSTREAM_RX_MAX_VCS * AAL5_RX_VC_INFO_ENTRY_SIZE)

/*-----------------------------------------------------------------------------*/
/* Primary VC Hash TABLE,(RXC table)                      					   */
/*-----------------------------------------------------------------------------*/
#define AAL5_RX_VC_HASH_PRI_SRAM_BASE   (AAL5_RX_VC_INFO_SRAM_BASE + AAL5_RX_VC_INFO_TABLE_SIZE)
#define AAL5_RX_VC_HASH_PRI_TABLE_SIZE	(AAL5_RX_VC_HASH_PRI_BKT_NUMBER * AAL5_RX_VC_HASH_PRI_BKT_ENTRY_SIZE)

/*-----------------------------------------------------------------------------*/
/* Secondary VC Hash TABLE,(RXC table)                    					   */
/*-----------------------------------------------------------------------------*/
#define AAL5_RX_VC_HASH_SEC_SRAM_BASE	(AAL5_RX_VC_HASH_PRI_SRAM_BASE + AAL5_RX_VC_HASH_PRI_TABLE_SIZE)
#define AAL5_RX_VC_HASH_SEC_TABLE_SIZE	(AAL5_RX_VC_HASH_SEC_BKT_NUMBER * AAL5_RX_VC_HASH_SEC_BKT_ENTRY_SIZE)

/*-----------------------------------------------------------------------------
 * AAL5 TX defines part 1
 */

/*	Base address AAL5 Transmit Contexts in SRAM. */
#define	AAL5_TX_CTX_SRAM_BASE			(AAL5_RX_VC_HASH_SEC_SRAM_BASE + AAL5_RX_VC_HASH_SEC_TABLE_SIZE)
#define	AAL5_TX_CTX_TABLE_SIZE          (DOWNSTREAM_TX_MAX_VCS * AAL5_TX_CTX_SIZE)

#define SRAM_MEMORY_CHANNEL_0_LAST      (AAL5_TX_CTX_SRAM_BASE + AAL5_TX_CTX_TABLE_SIZE)

#define COUNTERS

#ifdef COUNTERS

/*
 * IMPORTANT: Because we have additional memory allocated only in COUNTERS case, we have to
 * change last occupied address definition
 */
#undef SRAM_MEMORY_CHANNEL_0_LAST

/* AAL5_TX counts #pkts tx per VCQ. */
/* There are AAL5_TX_CTX_TOTAL VCQ's. */
/* Each counter is 1 LW (4 bytes). */
#define	AAL5_TX_PKTS_PER_VCQ_COUNTER_SRAM_BASE	    (AAL5_TX_CTX_SRAM_BASE + AAL5_TX_CTX_TABLE_SIZE)
#define	AAL5_TX_PKTS_PER_VCQ_COUNTER_TABLE_SIZE     (AAL5_TX_PKTS_PER_VCQ_COUNTER_SIZE * DOWNSTREAM_TX_MAX_VCS)
/*
// AAL5_TX counts #cells tx per VCQ.
// There are AAL5_TX_CTX_TOTAL VCQ's.
// Each counter is 1 LW (4 bytes).
*/
#define	AAL5_TX_CELLS_PER_VCQ_COUNTER_SRAM_BASE	    (AAL5_TX_PKTS_PER_VCQ_COUNTER_SRAM_BASE + AAL5_TX_PKTS_PER_VCQ_COUNTER_TABLE_SIZE)
#define	AAL5_TX_CELLS_PER_VCQ_COUNTER_TABLE_SIZE    (AAL5_TX_CELLS_PER_VCQ_COUNTER_SIZE * DOWNSTREAM_TX_MAX_VCS)
/*
// AAL5_TX counts #pkts tx per Output port.
// There are MAX_ATM_PORTS ports.
// Each counter is 1 LW (4 bytes).
*/
#define	AAL5_TX_PKTS_PER_OUTPORT_COUNTER_SRAM_BASE	(AAL5_TX_CELLS_PER_VCQ_COUNTER_SRAM_BASE + AAL5_TX_CELLS_PER_VCQ_COUNTER_TABLE_SIZE)
#define	AAL5_TX_PKTS_PER_OUTPORT_COUNTER_TABLE_SIZE	(AAL5_TX_PKTS_PER_OUTPORT_COUNTER_SIZE * DOWNSTREAM_TX_MAX_PORTS)

/*
// AAL5_TX counts #cells tx per output port.
// There are MAX_ATM_PORTS ports.
// Each counter is 1 LW (4 bytes).
*/
#define	AAL5_TX_CELLS_PER_OUTPORT_COUNTER_SRAM_BASE	    (AAL5_TX_PKTS_PER_OUTPORT_COUNTER_SRAM_BASE + AAL5_TX_PKTS_PER_OUTPORT_COUNTER_TABLE_SIZE)
#define	AAL5_TX_CELLS_PER_OUTPORT_COUNTER_TABLE_SIZE	(AAL5_TX_CELLS_PER_OUTPORT_COUNTER_SIZE * DOWNSTREAM_TX_MAX_PORTS)

#define SRAM_MEMORY_CHANNEL_0_LAST                   (AAL5_TX_CELLS_PER_OUTPORT_COUNTER_SRAM_BASE + AAL5_TX_CELLS_PER_OUTPORT_COUNTER_TABLE_SIZE)

#endif /* COUNTERS */

/*
 * This is the place where .ind script places Qdescriptor - read by Rx to Qarray
 */
#define UPSTREAM_TEMPORARY_FREELIST_QDESC 			((SRAM_MEMORY_CHANNEL_0_LAST  + 16) & 0xfffffff0)


/* 
 * SRAM channel 0 used memory 
 */
#define SRAM_MEMORY_CHANNEL_0_USED					(UPSTREAM_TEMPORARY_FREELIST_QDESC + QDESC_SIZE - SRAM_MEMORY_CHANNEL_0_BASE)

/*==============================================================================
 * 							SRAM MEMORY CHANNEL 1
 *
 *------------------------------------------------------------------------------
 */

/*
 *	Base address in SRAM for the meta data (buffer descriptors) in downstream direction.
 */

/* Two Buffer Descriptor lists are handled in system (distribute upstream and
   downstream traffic) */

#define	DOWNSTREAM_BUF_LW0_SRAM_BASE		SRAM_MEMORY_CHANNEL_1_BASE
#define	DOWNSTREAM_BUF_LW0_CHANNEL_NUMBER	1

/*------------------------------------------------------------------------------
 * QUEUE definitions
 */

/*
 *	Base Address for Queue Descriptors.
 */
#define DOWNSTREAM_QD_SRAM_BASE			(DOWNSTREAM_BUF_LW0_SRAM_BASE + BUF_SRAM_SIZE)
#define DOWNSTREAM_QD_SRAM_SIZE 		(DOWNSTREAM_QD_TOTAL * DOWNSTREAM_QD_SIZE)

/*
 *	SRAM Controler used by the Downstream Queue Manager
 */
#define DOWNSTREAM_QD_SRAM_CHANNEL_BASE_ADDRESS		DOWNSTREAM_BUF_LW0_SRAM_BASE

/*-----------------------------------------------------------------------------
 *	Trie Table
 */
#define TRIE_TABLE_SRAM_BASE			(DOWNSTREAM_QD_SRAM_BASE + DOWNSTREAM_QD_SRAM_SIZE)

/*
 * Trie Index Tables (in long words)
 */
#define TRIE_INDEX_TABLE_256            256
#define TRIE_INDEX_TABLE_64K            65536

/*
 * Number of trie entries 
 */
#define NUM_TRIE_SETS                   8192

/* 
 * Number of entries in trie set (in long words) 
 */
#define SIZE_OF_TRIE_SET                16

#define TRIE_SETS_SIZE                  SIZE_OF_TRIE_SET * NUM_TRIE_SETS
/*
 *	Trie Table structures size. 
 */
#define	TRIE_TABLE_SIZE 				((TRIE_INDEX_TABLE_256 + TRIE_INDEX_TABLE_64K + TRIE_SETS_SIZE)*4)


/*
 * base for IPv4 32bit counters
 */
#define STATS_TABLE_SRAM_BASE			(TRIE_TABLE_SRAM_BASE + TRIE_TABLE_SIZE)
#define STATS_TABLE_SIZE				((UPSTREAM_RX_MAX_VCS + DOWNSTREAM_RX_MAX_PORTS)*64)

/*
 *  Directed broadcast table base in SRAM
 */
#define DBCAST_TABLE_SRAM_BASE			(STATS_TABLE_SRAM_BASE + STATS_TABLE_SIZE)
#define DBCAST_TABLE_SIZE				0x2000

/*
 * control block
 */
#define CONTROL_BLOCK_SRAM_BASE 		(DBCAST_TABLE_SRAM_BASE + DBCAST_TABLE_SIZE)
#define	CONTROL_BLOCK_SRAM_SIZE			0x1000

/*  Area of SRAM which is zero'ed out */

#define     SRAM_ZERO_BLOCK                 (CONTROL_BLOCK_SRAM_BASE + CONTROL_BLOCK_SRAM_SIZE)
#define     SRAM_ZERO_BLOCK_SIZE            16*4 /* 16 longs words needed
                                                  * to initialize scheduler
                                                  * xfers to 0 */
 
/*
 *Ethernet MAC filtering hash table base: should be modified later
 */
#define     SRAM_FLT_TABLE_BASE             (SRAM_ZERO_BLOCK + SRAM_ZERO_BLOCK_SIZE)
#define     SRAM_FLT_TABLE_SIZE             0x0 /* needs (?) 0x100000 if enabled*/

/*
 * Input port bit map base: should be be set and updated by core
 * Each port uses 1 bits to define the port type
 *  bit 0 for port 0, bit 1 for port 1,..., bit 15 for port 15
 *  Bit definition:
 *      0   --- the port is defined as unit cast mode
 *      1  --- The port is defined promisc mode
 */
#define     SRAM_PORT_BIT_MAP_BASE          (SRAM_FLT_TABLE_BASE + SRAM_FLT_TABLE_SIZE)
#define     SRAM_PORT_BIT_MAP_SIZE          0x00000100 /*!check this! if used,
                                                        * 256 bytes should
                                                        * be enough */

/*------------------------------------------------------------------------------
 * Packet TX  definitions
 */

/* Packet TX needs 16*3 counters and each counter is 1LW */

#define PACKET_TX_COUNTER_BASE 	 		(SRAM_PORT_BIT_MAP_BASE + SRAM_PORT_BIT_MAP_SIZE)
#define	PACKET_TX_COUNTER_SIZE			(UPSTREAM_TX_MAX_PORTS * 3 * 4)

/*-----------------------------------------------------------------------------
 * L2 Table (ARP)
 */
#define L2_TABLE_SRAM_BASE				(PACKET_TX_COUNTER_BASE + PACKET_TX_COUNTER_SIZE)

/*-----------------------------------------------------------------------------
 * AAL5 TX defines
 */
/* Port Shaping table base */
#define	PORTSHAPING_SRAM_BASE			(L2_TABLE_SRAM_BASE + (L2_TABLE_ENTRY_SIZE * MAX_L2_TABLE_ENTRIES)) 

#define	GCRA_PARAM_SRAM_BASE			(PORTSHAPING_SRAM_BASE + PORTSHAPING_ENTRY_SIZE * DOWNSTREAM_TX_MAX_PORTS)

/* portinfo configuration */
#define PORTINFO_SRAM_ENTRY_SIZE	PORTINFO_ENTRY_SIZE 
#define	PORTINFO_SRAM_BASE			(GCRA_PARAM_SRAM_BASE + (DOWNSTREAM_TX_MAX_VCS * GCRA_ENTRY_SIZE))
#define	PORTINFO_SRAM_SIZE			(PORTINFO_SRAM_ENTRY_SIZE * MAX_ATM_PORTS) 

/* base address for the HBR TQ in SRAM */
#define HBR_TQ_SRAM_BASE				(PORTINFO_SRAM_BASE + PORTINFO_SRAM_SIZE)

/* location in SRAM for indicating that a new HBR TQ needs to be downloaded into LM */
#define	HBR_TQ_SRAM_INDICATOR			(HBR_TQ_SRAM_BASE + HBR_TQ_SRAM_SIZE)
#define HBR_TQ_SRAM_INDICATOR_SIZE      HBR_TQ_INDICATOR_SIZE

/* RT/NRT/UBR time queue length indicators SRAM base */
#define	RTQLEN_SRAM_BASE         (HBR_TQ_SRAM_INDICATOR + HBR_TQ_SRAM_INDICATOR_SIZE)
#define RTQLEN_TABLE_SIZE        (TQ_TOT * TQLEN_ENTRY_SIZE)

#define	NRTQLEN_SRAM_BASE		 (RTQLEN_SRAM_BASE + RTQLEN_TABLE_SIZE)
#define	NRTQLEN_TABLE_SIZE       (TQ_TOT * TQLEN_ENTRY_SIZE)

#define	UBRTQLEN_SRAM_BASE		 (NRTQLEN_SRAM_BASE + NRTQLEN_TABLE_SIZE)
#define	UBRTQLEN_TABLE_SIZE      (TQ_TOT * TQLEN_ENTRY_SIZE)

/* Base addresses for RT/NRT/UBR time queues in SRAM */
#define	RTQ_SRAM_BASE			(UBRTQLEN_SRAM_BASE + UBRTQLEN_TABLE_SIZE)
#define	RTQ_TABLE_SIZE			(TQ_TOT * TQ_SIZE)

#define	NRTQ_SRAM_BASE			(RTQ_SRAM_BASE + RTQ_TABLE_SIZE)
#define	NRTQ_TABLE_SIZE			(TQ_TOT * TQ_SIZE)

#define	UBRTQ_SRAM_BASE			(NRTQ_SRAM_BASE + NRTQ_TABLE_SIZE)
#define	UBRTQ_TABLE_SIZE		(TQ_TOT * TQ_SIZE)

/*
 * This is the place where .ind script places Qdescriptor - read by Rx to Qarray
 */

#define DOWNSTREAM_TEMPORARY_FREELIST_QDESC (((UBRTQ_SRAM_BASE + UBRTQ_TABLE_SIZE) + 16) & 0xfffffff0)


/* 
 * SRAM channel 1 used memory 
 */
#define SRAM_MEMORY_CHANNEL_1_USED	(DOWNSTREAM_TEMPORARY_FREELIST_QDESC + QDESC_SIZE - SRAM_MEMORY_CHANNEL_1_BASE)
 
/*
#ifdef SIMULATION
    #ifdef NQL_CONFIG
        #define	SRAM_MEMORY_CHANNEL_0_SIZE	0x0800000
        #define	SRAM_MEMORY_CHANNEL_1_SIZE	0x1000000
    #else
        #define	SRAM_MEMORY_CHANNEL_0_SIZE	0x0400000
        #define	SRAM_MEMORY_CHANNEL_1_SIZE	0x1000000
    #endif 

    #if (SRAM_MEMORY_CHANNEL_0_USED > SRAM_MEMORY_CHANNEL_0_SIZE)
      #ifndef CORE_CODE
        #error "SRAM memory channel 0 size exceeded."
      #endif
    #endif

    #if (SRAM_MEMORY_CHANNEL_1_USED > SRAM_MEMORY_CHANNEL_1_SIZE)
      #ifndef CORE_CODE
	    #error "SRAM memory channel 1 size exceeded."
      #endif
    #endif
#endif
*/

/*==============================================================================
 *                                Scratch rings
 *
 *------------------------------------------------------------------------------
 */

/* Scratch rings number 0 and 1 are used for ME-CC communication */
/* This can change if more scratch rings will be used by the core components */
#ifndef XSCALE_INIT_FOR_LINUX
#define FIRST_UCODE_SCRATCH_BASE                    0x800
#else
#define FIRST_UCODE_SCRATCH_BASE                    (SCRATCH_BASE  + 0x800)
#endif

/*
 *	Scratch for communicating between FWD and QM Blocks
 */
#define	DOWNSTREAM_FWD_TO_QM_SCR_RING_BASE          FIRST_UCODE_SCRATCH_BASE
#define	UPSTREAM_FWD_TO_QM_SCR_RING_BASE		    (DOWNSTREAM_FWD_TO_QM_SCR_RING_BASE + 4 * FWD_TO_QM_SCR_RING_SIZE)

#ifdef	DOWNSTREAM
    #define	FWD_TO_QM_SCR_RING_BASE		            DOWNSTREAM_FWD_TO_QM_SCR_RING_BASE
#else
    #define	FWD_TO_QM_SCR_RING_BASE		            UPSTREAM_FWD_TO_QM_SCR_RING_BASE
#endif


/*
 *	Scratch for communicating between Scheduler and Queue Manager Blocks
 */
#define DOWNSTREAM_SCHEDULER_TO_QM_SCR_RING_BASE	(UPSTREAM_FWD_TO_QM_SCR_RING_BASE + 4 * FWD_TO_QM_SCR_RING_SIZE)
#define UPSTREAM_SCHEDULER_TO_QM_SCR_RING_BASE		(DOWNSTREAM_SCHEDULER_TO_QM_SCR_RING_BASE + 4 * SCHEDULER_TO_QM_SCR_RING_SIZE)

#ifdef DOWNSTREAM
    #define SCHEDULER_TO_QM_SCR_RING_BASE	        DOWNSTREAM_SCHEDULER_TO_QM_SCR_RING_BASE
#else
    #define SCHEDULER_TO_QM_SCR_RING_BASE	        UPSTREAM_SCHEDULER_TO_QM_SCR_RING_BASE
#endif


/*
 *	Scratch for communicating between Queue Manager and ETH TX Transmit Blocks
 *	Ring number 8, base 1536, and size 128 Long Words
 */
#define	DOWNSTREAM_QM_TO_TX_SCR_RING_BASE		    (UPSTREAM_SCHEDULER_TO_QM_SCR_RING_BASE + 4 * SCHEDULER_TO_QM_SCR_RING_SIZE)
#define	UPSTREAM_QM_TO_TX_0_SCR_RING_BASE	    	(DOWNSTREAM_QM_TO_TX_SCR_RING_BASE + 4 * QM_TO_TX_SCR_RING_SIZE)

#ifdef DOWNSTREAM
    #define	QM_TO_TX_SCR_RING_BASE		            DOWNSTREAM_QM_TO_TX_SCR_RING_BASE
#else
    #define	QM_TO_TX_0_SCR_RING_BASE	            UPSTREAM_QM_TO_TX_0_SCR_RING_BASE
#endif


/*
 *	Scratch for communicating between Schaper and Scheduler Blocks
 *	Ring number 9, base 1536, and size 128 Long Words
 */

#define	DOWNSTREAM_SHAPER_TO_SCHEDULER_SCR_RING0_BASE	(UPSTREAM_QM_TO_TX_0_SCR_RING_BASE + 4 * QM_TO_TX_SCR_RING_SIZE)
#define	DOWNSTREAM_SHAPER_TO_SCHEDULER_SCR_RING1_BASE	(DOWNSTREAM_SHAPER_TO_SCHEDULER_SCR_RING0_BASE + 4 * SHAPER_TO_SCHEDULER_SCR_RING_SIZE)


#ifdef DOWNSTREAM
    #define SHAPER_TO_SCHEDULER_SCR_RING0_BASE		DOWNSTREAM_SHAPER_TO_SCHEDULER_SCR_RING0_BASE
    #define SHAPER_TO_SCHEDULER_SCR_RING1_BASE		DOWNSTREAM_SHAPER_TO_SCHEDULER_SCR_RING1_BASE
#endif

/*
 *	Scratch for communicating between RX and Forwarding Blocks
 */
#define	DOWNSTREAM_RX_TO_FWD_SCR_RING_BASE		    (DOWNSTREAM_SHAPER_TO_SCHEDULER_SCR_RING1_BASE + 4 * SHAPER_TO_SCHEDULER_SCR_RING_SIZE)
#define	UPSTREAM_RX_TO_FWD_SCR_RING_BASE		    (DOWNSTREAM_RX_TO_FWD_SCR_RING_BASE + 4 * RX_TO_FWD_SCR_RING_SIZE)

#define	ETH_RX_TO_FWD_SCR_RING_BASE		            DOWNSTREAM_RX_TO_FWD_SCR_RING_BASE
#define	ATM_RX_TO_FWD_SCR_RING_BASE		            UPSTREAM_RX_TO_FWD_SCR_RING_BASE


#define CONTROL_BLOCK_SCRATCH_BASE 		            (UPSTREAM_RX_TO_FWD_SCR_RING_BASE + 4 * RX_TO_FWD_SCR_RING_SIZE)
#define CONTROL_BLOCK_SCRATCH_SIZE			        4

/* Stores next available TBUF element 
 * Up to 4 partitions available 4B each*/
#define	AAL5_TX_AVAIL_TBUF_ELEMENT_SCRATCH_BASE	    (CONTROL_BLOCK_SCRATCH_BASE + CONTROL_BLOCK_SCRATCH_SIZE) 
#define	AAL5_TX_AVAIL_TBUF_TABLE_SIZE               (AAL5_TX_PARTITIONS_NUMBER*AAL5_TX_AVAIL_TBUF_ELEMENT_SIZE)


/* Stores cells transmitted */
#define	AAL5_TX_CELLS_TX_SCRATCH_BASE	            (AAL5_TX_AVAIL_TBUF_ELEMENT_SCRATCH_BASE + AAL5_TX_AVAIL_TBUF_TABLE_SIZE)
#define	AAL5_TX_CELLS_TX_TABLE_SIZE                 (AAL5_TX_PARTITIONS_NUMBER*AAL5_TX_CELLS_TX_ELEMENT_SIZE)


/*
#if ((AAL5_TX_CELLS_TX_SCRATCH_BASE + AAL5_TX_CELLS_TX_TABLE_SIZE) > 16384)
  #ifndef CORE_CODE
 	#error "Scratch size exceeded !"
  #endif
#endif
*/

/*
 *  These defines are used by the individual microcode to identify their
 *  scratch rings. The reason for these defines is to improve the portability
 *  of microblocks. User just has to make changes here and the microblocks will
 *  remain untouched. For the naming convention, the microblock name goes first,
 *  then the _RING_ specifier. Following this is the direction indicator. '_OUT'
 *  would imply the microblock is writing to this scratch ring and '_IN' would
 *  imply the microblock is reading from this scratch ring. If there is more
 *  than one 'IN' or 'OUT' ring the name will have '_<digit>' to
 *  differentiate the rings.
 *
 *  NOTE!!! The second set of SCR defines is located in dl_mem_map.h
 *
 */


/*
 * The input ring on which QM receives msgs from prev. block
 */

#ifdef  DOWNSTREAM

    #define QM_RING_IN_BASE					FWD_TO_QM_SCR_RING_BASE
    #define QM_RING_IN_0_BASE				FWD_TO_QM_SCR_RING_BASE
    #define QM_RING_IN_1_BASE				SCHEDULER_TO_QM_SCR_RING_BASE

#else

    #define QM_RING_IN_0_BASE				FWD_TO_QM_SCR_RING_BASE
    #define QM_RING_IN_1_BASE				SCHEDULER_TO_QM_SCR_RING_BASE

#endif

/*
 * The output ring in which QM Sends msgs to next block
 */

#ifdef  DOWNSTREAM

    #define QM_RING_OUT_0_BASE				QM_TO_TX_SCR_RING_BASE

    /* This QM_RING_OUT is used in SPHY mode only, but should be defined
     * to avoid compilation errors. 
     */

    #define QM_RING_OUT_1_BASE				QM_TO_TX_SCR_RING_BASE
    #define	QM_RING_OUT_2_BASE				QM_TO_TX_SCR_RING_BASE
    #define	QM_RING_OUT_3_BASE				QM_TO_TX_SCR_RING_BASE
    #define	QM_RING_OUT_BASE				QM_TO_TX_SCR_RING_BASE

#else

    #define	QM_RING_OUT_0_BASE				QM_TO_TX_0_SCR_RING_BASE

    /* 
     * This QM_RING_OUT is used in SPHY mode only, but should be defined
     * to avoid compilation errors. 
     */
    #define	QM_RING_OUT_1_BASE				QM_TO_TX_0_SCR_RING_BASE
    #define	QM_RING_OUT_2_BASE				QM_TO_TX_0_SCR_RING_BASE
    #define	QM_RING_OUT_3_BASE				QM_TO_TX_0_SCR_RING_BASE

#endif

/*
 * The output ring in which Sheduler Sends msgs to next block
 * (QM in this case)
 */

#ifdef DOWNSTREAM

    #define SCHEDULER_RING_OUT_0_BASE		SCHEDULER_TO_QM_SCR_RING_BASE

#else

    #define SCHEDULER_RING_OUT_BASE			SCHEDULER_TO_QM_SCR_RING_BASE
    
#endif


#ifdef DOWNSTREAM

    #define AAL5_TX_IN_0_BASE				QM_TO_TX_SCR_RING_BASE

#else
    /*
     * The input ring on which ETH TX receives msgs from prev. block
     */
    #define PACKET_TX_IN_0_BASE				QM_TO_TX_0_SCR_RING_BASE
    #define PACKET_TX_IN_1_BASE				QM_TO_TX_1_SCR_RING_BASE

#endif


#endif		 /*  DL_MEM_MAP_H  */


/*******************************  FILE HISTORY  *******************************
 * \main\1 Wed Aug 13 14:03:10 2003 adziecia
 * IGK0100025024: ATM-IPV4-ETH: simulation project check-in 
 *  
 *****************************************************************************/
