#ifndef __DL_SOURCE_UC__
#define __DL_SOURCE_UC__

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 2003 By Intel Corporation.  All rights reserved.
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

#include <stdmac.uc>
#include <sig_macros.uc>

/////////////////////////////
//	Some local defines
/////////////////////////////

/*	specify if next neighbor ring is used. If not, default is scratch rings.
 *	NOTE: DL_RING_* is now obsolete. Will be changed to use POS_RX_RING_IN,
 *	which is defined somwehere above.
 */
#define_eval DOWNSTREAM_FWD_RING_IN_EVAL	DOWNSTREAM_FWD_RING_IN
#define_eval UPSTREAM_FWD_RING_IN_EVAL		UPSTREAM_FWD_RING_IN


#if (DOWNSTREAM_FWD_RING_IN_EVAL > 11)
#error	"For Ring Number Greater than 11, we cannot use the SCR_Ring#_Status "
#error	"input state. Check RING_NUMBER"
#else
#define_eval	DOWNSTREAM_RING_FULL			SCR_Ring/**/DOWNSTREAM_FWD_RING_IN_EVAL/**/_Status
#endif

#if (UPSTREAM_FWD_RING_IN_EVAL > 11)
#error	"For Ring Number Greater than 11, we cannot use the SCR_Ring#_Status "
#error	"input state. Check RING_NUMBER"
#else
#define_eval	UPSTREAM_RING_FULL			SCR_Ring/**/UPSTREAM_FWD_RING_IN_EVAL/**/_Status
#endif

#if (FWD_TO_QM_SCR_RING > 11)
#error	"For Ring Number Greater than 11, we cannot use the SCR_Ring#_Status "
#error	"input state"
#else
#define_eval	FWD_TO_QM_SCR_RING_EVAL	FWD_TO_QM_SCR_RING
#define_eval DL_FWD_QM_RING_FULL SCR_Ring/**/FWD_TO_QM_SCR_RING_EVAL/**/_Status
#endif

/////////////////////////////
// Global variables/Registers
/////////////////////////////


.reg 	global	dl_ring				// scratch ring number for eth rx==>fwd.
.reg 	global	dl_buf_handle		// The current Buffer handle.
.reg	global	dl_eop_buf_handle	// For large packets, this is the last
									// buffer in the chain
.reg	global	dl_next_block		// Next block that should process
									// the buffer/packet
.reg	global	dl_cache_shift		// no of bytes that IP header was shifted

#define		PACKET_ROUTED	0x1
#define		PACKET_BRIDGED	0x2

///////////////////////////////////////////////////////////////////////////////
//
// _packet_get_l2_port
//
//	 	Description:
//
//			Get the L2 port associated with this physical port.
//
//	 	Outputs:
//
//			out_l2_port		:
//
//		Inputs:
//			none
//
//		Size:
//
//
//
///////////////////////////////////////////////////////////////////////////////

#macro _packet_get_l2_port[out_l2_port]
.begin
	.reg tmp

	immed[tmp, L2PORT_MAP_BASE]
	local_csr_wr[active_lm_addr_0, tmp]		; setup local memory for this port

	// Used alu[] instead of nop's to avoid removal during optimization
	alu[--, --, b , tmp]
	alu[--, --, b , tmp]
	alu[--, --, b , tmp]

	alu[out_l2_port, --, B, *l$index0]

.end
#endm



//------------------------------------------------------------------------------
// dl_sink_init:
//	 	Description:
//			Initialisation for ETH dl_sink.
//			Should be called before using dl_sink.
//			It simply sets a global variable dl_ring to the corresponding
//			ring number. This will later save one instruction in dl_sink
//			(for POS).
//
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//			1 instruction
//
#define		_dl_pos_sink_init	dl_sink_init

#macro dl_sink_init()
.begin

	// L2 Port initialization moved to EthRXInit.ind

.end
#endm

//------------------------------------------------------------------------------
// _dl_aal5_sink_init:
//	 	Description:
//			Initialisation for AAL5 Rx dl_sink.
//			Should be called before using dl_sink.
//			It simply sets a global variable dl_ring to the corresponding
//			ring number.
//
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//			1 instruction
//
#macro _dl_aal5_sink_init()
.begin
	alu_shf[dl_ring, --, B, DL_RING_NUMBER, <<2]
.end
#endm

//------------------------------------------------------------------------------
// dl_source_init:
//	 	Description:
//
//			Initialisation for dl_source. Should be called before using dl_source.
//			Currently not used/required. Place holder.
//
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//
//
#macro dl_source_init[]


#endm


//------------------------------------------------------------------------------
// _dl_eop_buf_handle_init:
//	 	Description:
//			THIS IS NOT AN EXPORTED MACRO. PLS DO NOT USE IT. THIS WILL CHANGE.
//			THIS IS ONLY TEMPORARY.
//
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//
//
#macro _dl_eop_buf_handle_init[]
.begin

	immed32[dl_eop_buf_handle, IX_NULL]

.end
#endm

//------------------------------------------------------------------------------
// dl_source:
//	 	Description: 
//
//			This macro is used by Forwarder Block to get packets and other info
//			from the ETH RX block. It receives them thro a scratch ring
//			(ETH_RX_TO_FWD_SCR_RING)
//			The following info is available from the ring.
//			dl_buf_handle		// The current buffer containing SOP 
//			dl_eop_buf_handle	// for large packets, the buffer containing EOP.
//								// Otherwise NULL
//			meta2				// Long Word 2 from the Meta Data
//			meta1				// Long Word 1 from the Meta data 
//			meta3				// Long Word 3 from the Meta Data
//			meta4				// Long Word 4 from the Meta Data
//			meta6				// Long Word 6 from the Meta Data
//			
//			THREAD_ORDER means if the execution of the threads need to be in
//			order. i.e wait for a signal from prev. thread, do some work and 
//			signal next thread. (This thread ordering is required, if the 
//			threads access some shared data, in order to get exclusive access 
//			to that shared data).
//
//	 	Outputs:
//
//		Inputs:
//			THREAD_ORDER	:	Specifies if the execution of the threads need
//								to be in order. Possible (constant) values are 
//								DL_THREAD_ORDER
//								DL_THREAD_NO_ORDER.
//			
//			sig_mask		:	Signal mask to wait on. This is the bitmask to 
//								be used in a ctx_arb[--] instrcution. (The
//								bitmask contains a set of signals used in
//								previous I/O on which we want to wait here)
//		Size:   			:
//			N/A	
//		
// 		Notes:
//		dl_source is meant to be used along with dl_wred_sink in the following
//		way (only).
//
//		dl_source[DL_THREAD_ORDER, 0]
//		.while (1)
//			do_what_ever[]
//			dl_wred_sink[...]
//			dl_source[DL_THREAD_NO_ORDER, ...]
//		.endw
//
//		dl_wred_sink is always ordered, meaning, it will always wait for a signal
//		from a previous thread, do its job and then signal the next thread.
//		Because dl_wred_sink is always ordered, we don't need the dl_source just
//		following it to be ordered. (but the first time we call
//		dl_source it needs to be ordered).
//
//		Performance optimisation: 
//		-------------------------
//		original code:
//		dl_wred_sink signals the next thread after producing on to the scratch
//		ring, which was at the end of that macro. Then dl_source is called
//		(no thread order). At the very top we had scratch[get...] ctx_swap[..].
//		This ctx_swap happened too soon (because the signal we sent to the next
//		thread takes about 10 cycles to reach) so the next thread is still
//		not ready to execute. This lead to large "Idle" time in the pipeline
//		affecting throughput.
//		
//		The following changes were made to improve performance.
//		1. In dl_wred_sink we signal the next thread at the first available
//		oppurtunity, even before the scratch[put...]. This is ok because the
//		next thread (in the same ME) cannot run until we sleep. So make sure
//		there is no ctx_swap between this signal and scratch_put.
//
//		2. If the next thread is another ME, then 1 is a wrong solution. So if
//		we have to signal another ME (because we are in thread 7), we postpone
//		the signalling until later (in dl_source macro). Thus, for thread 7,
//		dl_source will always signal the next ME.
//
//		3. soon after we make a scratch[put...] in dl_wred_sink, we make
//		scratch[get...] in dl_source. So we wait for the combined signals
//		(in dl_source macro) rather wait for wait for each of them separately.
//		(This is the reason for having 3 paramters to dl_wred_sink, and the second
//		paramter to dl_source).
//
//		For ease of understanding, look at dl_wred_sink and dl_source as one BIG
//		combined macro in the while loop.


#macro dl_source[THREAD_ORDER, sig_mask, ___PKTHDR]
.begin
.sig scr_get									; signal for scratch get
.reg ring

	//allocate xfer register. No Code generated.
	xbuf_alloc[$rdata, RX_TO_FWD_MSG_SIZE, read]

#if	(THREAD_ORDER == DL_THREAD_ORDER)

	// Wait for signal from previous thread to ensure thread ordering

	ctx_arb[sig_prev]

#endif
	
	//	Get Data from Scratch Ring.
	alu_shf[ring, --, B, FWD_RING_IN, <<2]
	scratch[get, $rdata0, 0, ring, FWD_RING_IN_MSG_SIZE], sig_done[scr_get]

	// 	Signal Next Thread. See Notes as why this code is like the way
	// it is shown here.
	br=ctx[7, thread_7#]						; Special case for thread 7.

#if	(THREAD_ORDER == DL_THREAD_ORDER)
	signal_next_ctx[&sig_prev]				; !7, so signal the next thread
#endif

	br[wait_for_signals#]

thread_7#:
	signal_me[DL_NEXT_ME, &sig_prev]			; Thread 7, So Signal Next ME


	//	We wait for the above scratch[get...] and the scratch[put...]
	//	in dl_wred_sink.
	//	See Notes above.

wait_for_signals#:

	//	Add the signal to the signal mask on which to wait
	alu_shf[sig_mask, sig_mask, OR, 1, <<&scr_get]

	// Wait for the signals
	local_csr_wr[active_ctx_wakeup_events, sig_mask]
	ctx_arb[--]

	//	Assembler doesn't know that we actually wait on the I/O (scratch[get..])
.io_completed scr_get

check_ring_empty#:

	//	Check if Ring is empty. ==> it would have returned zero
	alu[dl_buf_handle, $rdata0, -, 0]				; Check for 0. i.e check if ring empty
	beq[ring_empty#]								

	//	$rdata0...$rdata4 will have valid data now.
	dl_meta_init_cache[0, $rdata2, $rdata3, $rdata4, 0, 0, 0, 0]
	//dl_meta_init_cache[0, $rdata2, $rdata3, $rdata4, 0, 0, 0, 0]
    dl_meta_set_nexthop_id[0xffff]	// set nexthop ID to -1

	alu[dl_eop_buf_handle, --, B, $rdata1]

	br[load_hdr_cache#]
	// 	program flow continues in load_hdr_cache#:

	//	Handle Ring empty Condition.
ring_empty#:

	//	If ring is empty, just set dl_buf_handle and dl_next_block to IX_NULL;
	// 	Is this the right thing to do ? TBD ???

	move[dl_buf_handle, IX_NULL]				; Note IX_NULL is NOT ZERO.
	move[dl_next_block, IX_NULL]

	br[nothing_to_do#]

load_hdr_cache#:

#ifdef	DOWNSTREAM
	move[dl_next_block, BID_EDECAP]
#else
	move[dl_next_block, BID_ADECAP]
#endif

	// Read the IP header of the packet and cache it in some place
#define_eval TRANSFER_REGISTER ___PKTHDR/**/0/**/
.sig sig_ip

	dl_load_hdr_cache[TRANSFER_REGISTER, dl_buf_handle, sig_ip, sig_ip] 


nothing_to_do#:

	xbuf_free[$rdata]				; Free xfer registers. No Code generated.

   .set $$iphdr0, $$iphdr1, $$iphdr2, $$iphdr3, $$iphdr4
   .set $$iphdr5, $$iphdr6, $$iphdr7, $$iphdr8, $$iphdr9 

.end
#endm


//------------------------------------------------------------------------------
// dl_sink:
//	 	Description: 
//			Called by ETH RX to handover the packet to the next block (IPv4) in
//			the ingress pipeline. We do not do thread ordering in this macro.
//			(It is done by ETH RX itself because of tight budget)
//			
//			This macro will produce the following on to the scratch ring.
//			dl_buf_handle		// The current buffer containing SOP 
//			dl_eop_buf_handle	// for large packets, the buffer containing EOP.
//			meta2				// Long Word 2 from the Meta Data
//			meta1				// Long Word 1 from the Meta data 
//			meta3				// Long Word 3 from the Meta Data
//			meta4				// Long Word 4 from the Meta Data
//			meta6				// Long Word 6 from the Meta Data
//
//			This info will be used by the FWD block (by calling dl_source)
//
//			Note:
//			sig_mask is a bit mask of signals to wait on. This is passed in
//			a regiter. If there is no need to wait for any signal,
//			pass SIG_NONE.
//
//	 	Outputs:
//
//		Inputs:
//			wxfer_prefix:		SRAM Write Transfer Register name prefix
//								(using xbuf_alloc).
//			req_sig			:	signal to be used in the I/O (scratch put)
//			sig_mask		:	What to do with I/O operation - 
//								- wait for signal(s) as specified by sig_mask
//									(register)
//								- do not wait for signal, just return
//									(SIG_NONE - constant)
//			The following Global variables will also act as inputs.
//			dl_next_block		- IX_EXCEPTION, for exception packets
//								- BID_NEXT_BLOCK, block ID of the next block to
//									handle this packet.
//			dl_buf_handle		: Buffer handle of the buffer in the start of
//									chain (contianinf SOP)
//			dl_buf_eop_handle	: Buffer handle of the last buffer in the chain
//		Size:   				:
//			N/A
//
#macro dl_sink[wxfer_prefix, req_sig, sig_mask]
.begin
	.reg ring

	// Check if the buffer is a NULL buffer.
	alu_shf[--, --, B, dl_buf_handle, <<8]		; 24 LSBits contain the address
	beq[nothing_to_do#]

	//	Move required data to xfer registers
	move[wxfer_prefix/**/0, dl_buf_handle]
	move[wxfer_prefix/**/1, dl_eop_buf_handle]
	move[wxfer_prefix/**/3, dl_meta[2]]

	// We need to act only on EOP. otherwise just return.	

	br=byte[dl_next_block, 0, BID_EDECAP, sink_it_edecap#]
	br=byte[dl_next_block, 0, BID_ADECAP, sink_it_adecap#]

	//	In the case of an exception packet, it needs to be sent to the core
	//	through a different ring.

check_exception#:

	br!=byte[dl_next_block, 0, IX_EXCEPTION, nothing_to_do#]
	
	// Do Exception processing here.
	
	// For now just drop the packet.

drop_pkt#:

	// 	Drop the packet. Two cases to handle. Single buffer or a bufffer chain.
	//	It is determined by looking at the dl_eop_buf_handle

	alu[--, dl_eop_buf_handle, -, IX_NULL]	; if dl_eop_buf_handle == IX_NULL
	beq[drop_single_buffer#]				; then it is single buffer.

	// otherwise, buffer chain.
	dl_drop_buffer_chain[dl_buf_handle, dl_eop_buf_handle]
	br[nothing_to_do#]

drop_single_buffer#:
	dl_drop_buffer[dl_buf_handle]
	br[nothing_to_do#]

sink_it_edecap#:

	// Change value of meta data input_port field from msf port to L2 port.	
	.begin
		.reg l2_port

		_packet_get_l2_port[l2_port]
		dl_meta_set_input_port(l2_port)
	.end

	// Ok. We can sink. Check if the ring is full and then produce the data 
	// onto the ring. If ring full, drop the packet and continue.

 	br_inp_state[DOWNSTREAM_RING_FULL, drop_pkt#]			; Check if ring is full.

	// Ring Not Full. Put the packet on the ring.

	move[wxfer_prefix/**/2, dl_meta[1]]				; buffersize:offset -> xfer2
	move[wxfer_prefix/**/4, dl_meta[3]]				; inport | 0x0

	alu_shf[ring, --, B, DOWNSTREAM_FWD_RING_IN_EVAL, <<2]
	scratch[put, wxfer_prefix/**/0, 0, ring, RX_RING_OUT_MSG_SIZE],
		sig_done[req_sig]

	br[scratch_put_done#]

sink_it_adecap#:
			
	// Ok. We can sink. Check if the ring is full and then produce the data 
	// onto the ring. If ring full, drop the packet and continue.

 	br_inp_state[UPSTREAM_RING_FULL, drop_pkt#]			; Check if ring is full.

	// Ring Not Full. Put the packet on the ring.

	move[wxfer_prefix/**/2, dl_meta[1]]				; buffersize:offset -> xfer2
	move[wxfer_prefix/**/4, dl_meta[3]]				; inport | 0x0

	alu_shf[ring, --, B, UPSTREAM_FWD_RING_IN_EVAL, <<2]
	scratch[put, wxfer_prefix/**/0, 0, ring, RX_RING_OUT_MSG_SIZE],
		sig_done[req_sig]

scratch_put_done#:

#if (!streq('sig_mask', 'SIG_NONE'))
	//	Add the signal to the signal mask on which to wait
	alu_shf[sig_mask, sig_mask, OR, 1, <<&req_sig]
#endif

nothing_to_do#:

#if (!streq('sig_mask', 'SIG_NONE'))
	local_csr_wr[active_ctx_wakeup_events, sig_mask]
	ctx_arb[--]
#endif
	
.end
#endm

//------------------------------------------------------------------------------
// dl_qm_sink:
//	 	Description: 
//			Called by IPv4 Block to send the packet to Queue Manager thro'
//			a scratch ring. This macro will do thread ordering always. 
//		
//			This macro will produce the following on to the scratch ring.
//			dl_buf_handle		// The current buffer containing SOP 
//			dl_eop_buf_handle	// for large packets, the buffer containing EOP.
//								// Otherwise NULL
//			Queue Number		// Queue number into which this packet is to
//								//be queued.
//
//	 	Outputs:
//			sig_mask			- if the I/O operation is made (scratch put)
//								  then the sig_mask will have the bit
//								  corresponding to sig_scr set. (sig_scr is an
//								  input parameter)
//
//		Inputs:
//			sig_scr				- Signal to use in the I/O (scratch) operation
//			sig_meta			- Signal to use in the I/O meta-data operation
//			wxfer				- SRAM write xfer regsiter to be used in I/O.
//								  Should have been allocated using xbuf_alloc. 
//								  Atleast 3 regsiters should be allocated.
//			buf_size_incr		- SRAM write register used to store buffer size
//								  in metadata		
//
//
//			The following Global variables will also act as inputs.
//			dl_next_block		- IX_EXCEPTION, for exception packets
//								- IX_NULL, for null packets.
//								- BID_NEXT_BLOCK, block ID of the next block
//								  to handle this packet.
//			dl_buf_handle		: Buffer handle of the buffer in the start of
//								  chain (contianins SOP)
//			dl_buf_eop_handle	: Buffer handle of the last buffer in the chain
//		
//		Size:   				:
//			N/A
//
//		Notes:
//
//		See notes in dl_source macro

#macro dl_qm_sink[sig_mask, sig_scr, sig_meta, wxfer, $buf_size_incr, ___PKTHDR]
.begin
	.sig sig_meta_flush

	//	This macro can be seen in two parts. The normal packet case
	// (which is the critical path) and other(drop, null pktetc) cases

	//	First the normal case.

	// check for next block
	br!=byte[dl_next_block, 0, BID_NEXT_BLOCK, check_exception#]  
	
	xbuf_alloc[$meta_cache, 8, write]


	//	The 1st LW of the meta data is next buffer pointer and should not be
	// updated when doing dl_meta_flush_cache
	#define_eval META_SIZE (META_CACHE_SIZE -1)

	// Write back Meta data to SRAM
	dl_meta_flush_cache[$meta_cache, dl_buf_handle, sig_meta_flush, SIG_NONE, 1, META_SIZE]

	//	Wait for the prev. I/O as well as signal from previous thread.
	ctx_arb[sig_meta_flush, sig_prev]

	xbuf_free[$meta_cache]
	

	// 	Signal Next Thread. Signaling next thread has a latency of 10 cycles.
	//  So we signal at the first available oppurtunity. (see notes in
	//	dl_source macro). It is IMPORTANT that there should be not ctx_swap/arb
	//	from here to scratch[put...] below.

	br=ctx[7, thread_7#]						; Special case for thread 7.
	signal_next_ctx[&sig_prev]				    ; !7, so signal the next thread

thread_7#:

	// 	If thread 7 we have to signal next ME. But we cannot do that until we
	//	have issued the scratch[put..] command. We'll do it in dl_source macro.
	//	(see notes in dl_source).

	// Check if Ring is Full. If Full, just stall the pipeline.(ie keep looping)
check_ring_full#:
 	br_inp_state[DL_FWD_QM_RING_FULL, ring_is_full#]

	// At this point, ring has enough space. Sink the data.

#ifdef DOWNSTREAM

	// NOTE: multi-buffer packets are not supported.

	.reg packet_size 
	.reg no_addn_cell_packet_cell_count
	.reg correct_packet_cell_count

	dl_meta_get_packet_size(packet_size)
	
	; Compute packet cell count without taking ADDN_CELL into account.
	_dl_compute_packet_cell_count(no_addn_cell_packet_cell_count, 
									packet_size, --)
	
	; Compute correct packet cell count.
	_dl_compute_packet_cell_count(correct_packet_cell_count,
									packet_size, ADDN_CELL)

	; If (correct_packet_cell_count - no_addn_cell_packet_cell_count) == 1
	; increment eop buffer size by 1. This implies an additional cell needs
	; to be scheduled.
	; We increment eop buffer size so that the AAL5 transmit block does not
	; free this buffer until the additional cell dequeue request comes in.
	alu[--, correct_packet_cell_count, -, no_addn_cell_packet_cell_count]
	beq[compute_queue#]

	alu[$buf_size_incr, --, B, 1, <<16]
	alu[sig_mask, sig_mask, OR, 1, <<(&sig_meta)]

	alu[--, dl_eop_buf_handle, -, IX_NULL]
	beq[update_sop#]

	; Update eop buffer size
	.begin
		.reg channel_number, eop_buf_hdl

		ld_field_w_clr[eop_buf_hdl, 0111, dl_eop_buf_handle]

		alu[channel_number, --, B, BUF_SRAM_CHANNEL_REST, <<30] ; set channel number
		alu[eop_buf_hdl, channel_number, OR, eop_buf_hdl, <<2]	; shift to get SRAM address of the buffer descriptor

		sram[add, $buf_size_incr, eop_buf_hdl, 4], sig_done[sig_meta]

	.end 

	br[compute_queue#]

update_sop#:

	; Update sop buffer size.
	.begin
		.reg channel_number, sop_buf_hdl

		ld_field_w_clr[sop_buf_hdl, 0111, dl_buf_handle]

		alu[channel_number, --, B, BUF_SRAM_CHANNEL_REST, <<30] ; set channel number
		alu[sop_buf_hdl, channel_number, OR, sop_buf_hdl, <<2]	; shift to get SRAM address of the buffer descriptor

		sram[add, $buf_size_incr, sop_buf_hdl, 4], sig_done[sig_meta]

	.end 

compute_queue#:

	.reg queue_id

    ; get the VCW from LLC SNAP encap
	ld_field_w_clr[queue_id, 0011, src_sink_data[2]]	

	.reg scr_ring_lw2

	move(scr_ring_lw2, 0x0) ; CLP bit is cleared.

	alu_shf[scr_ring_lw2, queue_id, OR, 1, <<31] ; Set queue id and valid bit.

	; Store packet cell count
	alu_shf[scr_ring_lw2, scr_ring_lw2, OR, correct_packet_cell_count, <<18]

	move[wxfer/**/2, scr_ring_lw2]
	
	; Update sop buffer cell count.
	alu[correct_packet_cell_count, correct_packet_cell_count, -, 1]
	; Remove old Packet Length field before setting Cell Count  
	alu[dl_buf_handle, dl_buf_handle, AND~, 0x3F, <<24]
	alu_shf[dl_buf_handle, dl_buf_handle, OR, correct_packet_cell_count, <<24]
	move[wxfer/**/0, dl_buf_handle]

	move[wxfer/**/1, dl_eop_buf_handle]


#else // UPSTREAM

	move[wxfer/**/0, dl_buf_handle]
	move[wxfer/**/1, dl_eop_buf_handle]

	.reg queue_id port_id class_id

	dl_meta_get_output_port(port_id) ; MSF physical port.

	dl_meta_get_class_id(class_id)

	// queue_id = port_id*max_queue + class_id
	alu[queue_id, --, B, port_id, <<WRED_MAX_QUEUE]
	alu[queue_id, class_id, +, queue_id]

	alu_shf[wxfer/**/2, queue_id, OR, 1, <<31] ; Set valid bit.

#endif

.reg ring

	// ring nos in get/put is always multiple of 4.
	alu_shf[ring, --, b, FWD_RING_OUT, <<2]

	scratch[put, wxfer/**/0, 0, ring, FWD_RING_OUT_MSG_SIZE],
		sig_done[sig_scr]	; Produce 3 Long words on the ring

	//	To improve performance, we do not ctx_swap on scratch[put..]
	//	above. We'll wait on this signal in dl_source macro. (see notes in
	//	dl_source macro). However this(dl_qm_sink) macro has multiple paths in
	//	which we do not issue this scratch[put..] at all. e.g exception packets,
	//	or NULL packets etc. So if we issue scratch[put..], we set the sig_mask
	//	to indicate that.

	//	Add the signal to the signal mask on which to wait
	alu_shf[sig_mask, sig_mask, OR, 1, <<&sig_scr]

	br[end#]

ring_is_full#:
	nop									; to set break point
	br[check_ring_full#]				; go back and check again

	// 	Check for exception Packets. Exception packets are sent to the core
	//	on a separate ring. 

check_exception#:

	// 	Wait for signal from prev thread and then signal next context.
	
	ctx_arb[sig_prev]

	// Signal next context.
	signal_next_ctx[&sig_prev]

	br!=byte[dl_next_block, 0, IX_EXCEPTION, check_drop#]

	// 	Drop the packet. Two cases to handle. Single buffer or a bufffer chain.
	//	It is determined by looking at the dl_eop_buf_handle

	alu[--, dl_eop_buf_handle, -, IX_NULL]	; if dl_eop_buf_handle == IX_NULL
	beq[drop_single_buffer#]				; then it is single buffer.

	// otherwise, buffer chain.
	dl_drop_buffer_chain[dl_buf_handle, dl_eop_buf_handle]
	br[end#]

drop_single_buffer#:
	dl_drop_buffer[dl_buf_handle]
	br[end#]

check_drop#:

	br!=byte[dl_next_block, 0, IX_DROP, check_ix_null#]

	// 	Drop the packet. Two cases to handle. Single buffer or a bufffer chain.
	//	It is determined by looking at the dl_eop_buf_handle

	alu[--, dl_eop_buf_handle, -, IX_NULL]	; if dl_eop_buf_handle == IX_NULL
	beq[drop_single_buffer_1#]				; then it is single buffer.

	// otherwise, buffer chain.
	dl_drop_buffer_chain[dl_buf_handle, dl_eop_buf_handle]
	br[end#]

drop_single_buffer_1#:
	dl_drop_buffer[dl_buf_handle]
	br[end#]

check_ix_null#:

	// Do something to handle null packet. TBD
	nop

end#:

.end
#endm

//------------------------------------------------------------------------------
// dl_wred_sink:
//	 	Description: 
//			Called by FWD Block to send the packet to WRED thro' a scratch ring.
//			This macro will do thread ordering always. 
//		
//			This macro will produce the following on to the scratch ring.
//			dl_buf_handle		// The current buffer containing SOP 
//			dl_eop_buf_handle	// for large packets, the buffer containing EOP.
//			meta2				// Long Word 2 from the Meta Data
//			meta4				// Long Word 4 from the Meta Data
//
//			Note:
//			sig_mask is a bit mask of signals to wait on. This is passed in
//			a regiter. If there is no need to wait for any signal,
//			pass SIG_NONE.
//
//	 	Outputs:
//
//		Inputs:
//			wxfer_prefix:		SRAM Write Transfer Register name prefix
//								(using xbuf_alloc).
//			req_sig			:	signal to be used in the I/O (scratch put)
//			sig_mask		:	What to do with I/O operation - 
//								- wait for signal(s) as specified by sig_mask
//									(register)
//								- do not wait for signal, just return
//									(SIG_NONE - constant)
//			The following Global variables will also act as inputs.
//			dl_next_block		- IX_EXCEPTION, for exception packets
//								- BID_NEXT_BLOCK, block ID of the next block to
//									handle this packet.
//			dl_buf_handle		: Buffer handle of the buffer in the start of
//									chain (contianinf SOP)
//			dl_buf_eop_handle	: Buffer handle of the last buffer in the chain
//		Size:   				:
//			N/A
//
//		Notes:
//
//		See notes in dl_source macro


#macro dl_wred_sink[sig_mask, sig_scr, wxfer]
.begin
.sig	sig_meta_rest

	//	This macro can be seen in two parts. The normal packet case (which is
	//	the critical path) and other(drop, null pktetc) cases

	//	First the normal case.

	// now check for next block

	br!=byte[dl_next_block, 0, BID_NEXT_BLOCK, check_exception#]
	
	xbuf_alloc[$meta_cache, 8, write]


	//	The 1st LW of the meta data is next buffer pointer and should not be
	// updated when doing dl_meta_flush_cache
	#define_eval META_SIZE (META_CACHE_SIZE -1)
	// Write back Meta data to SRAM
	dl_meta_flush_cache[$meta_cache, dl_buf_handle, --, sig_meta_rest, SIG_NONE, 1, META_SIZE]

	//	Wait for the prev. I/O as well as signal from previous thread.
	ctx_arb[sig_meta_rest, sig_prev]

	xbuf_free[$meta_cache]

	// 	Signal Next Thread. Signaling next thread has a latency of 10 cycles.
	//  So we signal at the first available oppurtunity. (see notes in
	//	dl_source macro). It is IMPORTANT that there should be not ctx_swap/arb
	//	from here to scratch[put...] below.

	br=ctx[7, thread_7#]					; Special case for thread 7.
	signal_next_ctx[&sig_prev]				; !7, so signal the next thread

thread_7#:

	// 	If thread 7 we have to signal next ME. But we cannot do that until we
	//	have issued the scratch[put..] command. We'll do it in dl_source macro.
	//	(see notes in dl_source).

	// Check if Ring is FUll. If Full, just stall the pipeline. (i.e keep looping)
check_ring_full#:
 	br_inp_state[DL_FWD_WRED_RING_FULL, ring_is_full#]

	// At this point, ring has enough space. Sink the data.

	move[wxfer/**/0, dl_buf_handle]
	move[wxfer/**/1, dl_eop_buf_handle]

	move[wxfer/**/2, dl_meta[2]]					; packet_size
#ifdef WRED_INGRESS_MODE
	move[wxfer/**/3, dl_meta[4]]					; fabric_port
#else
	move[wxfer/**/3, dl_meta[3]]					; output_port
#endif
	move[wxfer/**/4, dl_meta[6]]					; class_id, color_id

.reg ring

	// ring nos in get/put is always multiple of 4.
	alu_shf[ring, --, b, FWD_RING_OUT, <<2]

	scratch[put, wxfer/**/0, 0, ring, FWD_RING_OUT_MSG_SIZE], sig_done[sig_scr]

	//	To improve performance, we do not ctx_swap on scratch[put..] above.
	//	We'll wait on this signal in dl_source macro. (see notes in dl_source
	//	macro). However this(dl_qm_sink) macro has multiple paths in which we
	//	do not issue this scratch[put..] at all. e.g exception packets, or NULL
	//	packets etc. So if we issue scratch[put..], we set the sig_mask to
	//	indicate that.

	//	Add the signal to the signal mask on which to wait

	alu_shf[sig_mask, sig_mask, OR, 1, &sig_scr]
	br[end#]

ring_is_full#:
	nop												; to set break point
	br[check_ring_full#]							; go back and check again

	// 	Check for exception Packets. Exception packets are sent to the core
	//	on a separate ring. 

check_exception#:

	ctx_arb[sig_prev]

	// Signal next context.

	br=ctx[7, thread_7_2#]					; Special case for thread 7.
	signal_next_ctx[&sig_prev]				; !7, so signal the next thread

thread_7_2#:

	br!=byte[dl_next_block, 0, IX_EXCEPTION, check_drop#]

	// 	Drop the packet. Two cases to handle. Single buffer or a bufffer chain.
	//	It is determined by looking at the dl_eop_buf_handle

	alu[--, dl_eop_buf_handle, -, IX_NULL]		; if dl_eop_buf_handle==IX_NULL
	beq[drop_single_buffer#]					; then it is single buffer.

	// otherwise, buffer chain.
	dl_drop_buffer_chain[dl_buf_handle, dl_eop_buf_handle]
	br[end#]

drop_single_buffer#:
	dl_drop_buffer[dl_buf_handle]
	br[end#]

check_drop#:

	br!=byte[dl_next_block, 0, IX_DROP, check_ix_null#]

	// 	Drop the packet. Two cases to handle. Single buffer or a bufffer chain.
	//	It is determined by looking at the dl_eop_buf_handle

	alu[--, dl_eop_buf_handle, -, IX_NULL]		; if dl_eop_buf_handle==IX_NULL
	beq[drop_single_buffer_1#]					; then it is single buffer.

	// otherwise, buffer chain.
	dl_drop_buffer_chain[dl_buf_handle, dl_eop_buf_handle]
	br[end#]

drop_single_buffer_1#:
	dl_drop_buffer[dl_buf_handle]
	br[end#]

check_ix_null#:

	// Do something to handle null packet. TBD
	nop

end#:

.end
#endm



//------------------------------------------------------------------------------
// dl_wred_source:
//	 	Description: 
//
//			This macro is used by WRED Block to get packets and other info
//			from the FWD block. It receives them thro a scratch ring
//			(FWD_TO_WRED_SCR_RING)
//			The following info is available from the ring.
//			dl_buf_handle		// The current buffer containing SOP 
//			dl_eop_buf_handle	// for large packets, the buffer containing EOP.
//								// Otherwise NULL
//			meta2				// Long Word 2 from the Meta Data
//			meta4				// Long Word 4 from the Meta Data
//			
//			THREAD_ORDER means if the execution of the threads need to be in
//			order. i.e wait for a signal from prev. thread, do some work and 
//			signal next thread. (This thread ordering is required, if the 
//			threads access some shared data, in order to get exclusive access 
//			to that shared data).
//
//	 	Outputs:
//
//		Inputs:
//			THREAD_ORDER	:	Specifies if the execution of the threads need
//								to be in order. Possible (constant) values are 
//								DL_THREAD_ORDER
//								DL_THREAD_NO_ORDER.
//			
//			sig_mask		:	Signal mask to wait on. This is the bitmask to 
//								be used in a ctx_arb[--] instrcution. (The
//								bitmask contains a set of signals used in
//								previous I/O on which we want to wait here)
//		Size:   			:
//			N/A	
//		
// 		Notes:
//		dl_source is meant to be used along with dl_qm_sink in the following
//		way (only).
//
//		dl_source[DL_THREAD_ORDER, 0]
//		.while (1)
//			do_what_ever[]
//			dl_qm_sink[...]
//			dl_source[DL_THREAD_NO_ORDER, ...]
//		.endw
//
//		dl_qm_sink is always ordered, meaning, it will always wait for a signal
//		from a previous thread, do its job and then signal the next thread.
//		Because dl_qm_sink is always ordered, we don't need the dl_source just
//		following it to be ordered. (but the first time we call
//		dl_source it needs to be ordered).
//
//		Performance optimisation: 
//		-------------------------
//		see comments in dl_source macro


#macro dl_wred_source[THREAD_ORDER, sig_mask]
.begin
.sig scr_get					; signal for scratch get
.reg ring

	xbuf_alloc[$rdata, FWD_TO_WRED_MSG_SIZE, read]

#if	(THREAD_ORDER == DL_THREAD_ORDER)

	// Wait for signal from previous thread to ensure thread ordering

	ctx_arb[sig_prev]

#endif
	
	//	Get Data from Scratch Ring.
	alu_shf[ring, --, B, WRED_RING_IN, <<2]	; Ring from which to consume
	scratch[get, $rdata0, 0, ring, WRED_RING_IN_MSG_SIZE], sig_done[scr_get]

	// 	Signal Next Thread. See Notes as why this code is like the way it is
	// shown here.
#if	(THREAD_ORDER == DL_THREAD_ORDER)
	signal_next_ctx[&sig_prev]
#endif

	//	We wait for the above scratch[get...] and the scratch[put...] in
	//	dl_qm_sink.
	//	See Notes above.

	//	Add the signal to the signal mask on which to wait
	alu_shf[sig_mask, sig_mask, OR, 1, <<&scr_get]

	local_csr_wr[active_ctx_wakeup_events, sig_mask]
	ctx_arb[--]

	//	Assembler doesn't know that we actually wait on the I/O (scratch[get..])
.io_completed scr_get

check_ring_empty#:

	//	Check if Ring is empty. ==> it would have returned zero

	alu[dl_buf_handle, $rdata0, -, 0]				; Check for 0. i.e check if ring empty
	beq[ring_empty#]								

	//	$rdata0...$rdata4 will have valid data now.
	//	...

	#ifdef WRED_INGRESS_MODE
		dl_meta_init_cache[0, 0, $rdata2, 0, $rdata3, 0, $rdata4, 0]
	#else
		dl_meta_init_cache[0, 0, $rdata2, $rdata3, 0, 0, $rdata4, 0]
	#endif

	move[dl_eop_buf_handle, $rdata1]

	br[set_next_block#]
	// 	program flow continues in set_next_block#:

	//	Handle Ring empty Condition.
ring_empty#:

	//	If ring is empty, just set dl_buf_handle and dl_next_block to IX_NULL
	// 	Is this the right thing to do ? TBD ???

	move[dl_buf_handle, IX_NULL]				; Note IX_NULL is NOT ZERO.
	move[dl_next_block, IX_NULL]
	br[nothing_to_do#]

set_next_block#:

	move[dl_next_block, BID_WRED]

nothing_to_do#:

	xbuf_free[$rdata]

.end
#endm

//------------------------------------------------------------------------------
// dl_set_exception:
//	 	Description: 
//			Set exception code. Not used for Now. Place Holder.
//
//	 	Outputs:
//
//		Inputs:
//		
//		Size:   				:
//	
//
#macro	dl_set_exception[block_id, exception_code]
.begin

	nop
    nop

.end
#endm

//------------------------------------------------------------------------------
// nn_prod_init:
//	 	Description: 
//			Initialise the Next Neighbour Mode (NN mode) for the producer. 
// 			For the producer, NN Registers are wrriten:from This ME.[bit 20 = 1]
//
//
//	 	Outputs:
//
//		Inputs:
//		
//		Size:   				:
//	
//

#macro nn_prod_init[]

.begin
.reg csr_data

	local_csr_rd[CTX_ENABLES]
	immed[csr_data,0]

	// For the source, NN Registers are wrriten: from This ME. [bit 20 = 1]

	bits_set[csr_data, 20, 1]						; [20] = 1

	local_csr_wr[CTX_ENABLES, csr_data]

.end
#endm

//------------------------------------------------------------------------------
// nn_cons_init:
//	 	Description: 
//			Initialise the Next Neighbour Mode (NN mode) for the consumer.
// 			For the consumer, NN Registers are wrriten: from previous ME.
//			[bit 20 = 0]
//			We do some one time initilaisation in consumer???
//			(so producer need not initialise them) Should it be porducer???? TBD
//
//	 	Outputs:
//
//		Inputs:
//		
//		Size:   				:
//	
//
#macro nn_cons_init[]
.begin
.reg csr_data

	local_csr_rd[CTX_ENABLES]
	immed[csr_data, 0]

// 	For the sink, NN Registers are wrriten: from previous ME. [bit 20 = 0]

	bits_clr[csr_data, 20, 1]						; [20] = 0

	// Tell when to trigger ring empty. i.e when nn_get == nn_put
	// (0 entires valid)
	// We have kept this and above instruction (alu_shf) separate for clarity. 
	// we can always merge them when the need arises.

	bits_clr[csr_data, 18, 3]						; [19:18] = 0
	local_csr_wr[CTX_ENABLES, csr_data]

	local_csr_wr[NN_GET, 0]
	local_csr_wr[NN_PUT, 0]
	
.end
#endm

///////////////////////////////////////////////////////////////////////////////
// dl_set_exception_priority:
//              Description: 
//                      Set exception code priority to Low (0) or High (1)
//
//              Outputs:
//
//              Inputs:
//              
//              Size:                                   :
//      
//
#macro  dl_set_exception_priority[priority]
        nop
#endm

//------------------------------------------------------------------
// _dl_compute_packet_cell_count()
//
//    Description: Computes number of 48-byte cells in the packet.
//    This algorithm was devised by Adrian Georgescu and Roland Arajs.
//
//    Parameters:
//        Outputs:
//        out_packet_cell_count :
//            Number of 48-byte cells in the packet.
//        Inputs:
//        in_packet_size:
//            Packet size in bytes.
//        IN_ARG :
//            When set to 'ADDN_CELL' the macro computes cell count
//            taking into consideration the need for an additional
//            cell for the packet if the payload of the last cell
//            of the packet is >40. The additional cell is needed
//            in this case to store the 8-byte CPCS trailer.
//
//     Example :
//                packet size  IN_ARG      packet cell count
//                (bytes)
//                 40           --          1
//                 47           --          1
//                 48           --          1
//                 49           --          2
//                 40           ADDN_CELL   1
//                 47           ADDN_CELL   2
//                 48           ADDN_CELL   2
//                 49           ADDN_CELL   2
//
//------------------------------------------------------------------
#macro _dl_compute_packet_cell_count(out_packet_cell_count, \
			in_packet_size, IN_ARG)
.begin

	//    Shown below is the C implementation of the algorithm.
	//
	//    int compute_packet_cell_count(int in_packet_size)
	//    {
	//        int p, c;
	//
	//        p = in_packet_size + (IN_ARG == 'ADDN_CELL')?56:48;
	//
	//        c = ((((p << 2) + p) << 2) + p);
	//
	//        c = (c + (c << 6));
	//
	//        out_packet_cell_count = \
	//            ((c >> 16) + ((((c & 0xffff) << 12) + c) >> 28));
	//
	//        return out_packet_cell_count;
	//    }

	.reg p, c

	#if (streq('IN_ARG', 'ADDN_CELL'))
	alu[p, in_packet_size, +, 56]
	#else
	alu[p, in_packet_size, +, 48]
	#endif

	.reg tmp OxFFFF

	move(OxFFFF, 0xFFFF)

	alu[tmp, --, B, p, <<2]
	alu[tmp, tmp, +, p]
	alu[tmp, --, B, tmp, <<2]
	alu[c, tmp, +, p]

	alu[tmp, --, B, c, <<6]
	alu[c, c, +, tmp]

	alu[tmp, c, AND, OxFFFF]
	alu[tmp, --, B, tmp, <<12]
	alu[tmp, tmp, +, c]
	alu[tmp, --, B, tmp, >>28]
	alu[c, --, B, c, >>16]

	alu[out_packet_cell_count, c, +, tmp]

.end
#endm // _dl_compute_packet_cell_count


#endif	// __DL_SOURCE_UC__
/*******************************  FILE HISTORY  *******************************
 * \main\1 Wed Aug 13 14:03:11 2003 adziecia 
 * IGK0100025024: ATM-IPV4-ETH: simulation project check-in 
 *  
 * \main\13 Tue Jun 24 14:19:52 2003 jpwysocz 
 * Defect #IGK0100023688. Corrected system initialization (signals) and 
 * forwarding path organization. 
 *  
 * \main\12 Tue May 27 17:37:23 2003 jpwysocz 
 * Defect #IGK0100022254. Added changes needed to stabilize build after a merge 
 * of the uCode with CC. 
 *  
 * \main\11 Tue Apr 29 15:18:41 2003 jpwysocz 
 * Defect #IGK0100021377. Freelist initialization scheme improved. Now init is 
 * moved to scripts (ind) what gives a huge acceleration. 
 *  
 * \main\10 Fri Apr  4 10:33:50 2003 jpwysocz 
 * Defect #IGK0100020839. This is a configuration tuning after the preparation 
 * of the code for the CC, now LP OC-12 transmits traffic in both directions. 
 *  
 * \main\9 Tue Apr  1 17:26:30 2003 jpwysocz 
 * Defect #IGK0100020689. Changed system configuration scheme to prepare it for 
 * Core Components addition. 
 *  
 * \main\8 Thu Mar 27 18:43:58 2003 jstelter 
 * Bug: #IGK0100020689
 * Reorganization of header structure. 
 *  
 * \main\7 Mon Feb 24 17:53:36 2003 jpwysocz 
 * Defect #FLOS100019957. Added SRAM channel handling and corrected SRAM 
 * address calculation (buffer handle) in dl_qm_sink. 
 *  
 * \main\6 Fri Feb 21 10:30:12 2003 jstelter 
 * Bug: #FLOS100019950
 * Warnings during compilation of oc12_dslam_eth 
 *  
 * \main\5 Wed Feb 19 14:43:18 2003 gnawrot 
 * FLOS100019898: order of received packets can be changed in sink to qm 
 *  
 * \main\4 Tue Feb 18 10:46:08 2003 cwierucx 
 * FLOS100019815: .io_completed operation in dl_qm_sink() macro could couse 
 * wrong buf_size write to SRAM 
 *  
 * \main\3 Fri Jan 31 15:50:09 2003 cwierucx 
 * FLOS100019406: Eth Phy to L2 Port mapping moved to script file. 
 *  
 * \main\2 Wed Jan  8 16:20:06 2003 gnawrot 
 * FLOS100018773: DSLAM OC-12 ETH (project setup) 
 *  
 * \main\1 Mon Jan  6 10:51:55 2003 cwierucx 
 * Initial checkin for Lime Point OC12 DSLAM ETH 
 *  
 * \main\23 Thu Jan  2 10:57:37 2003 gnawrot 
 * FLOS100018411: signals next/prev moved to forwarding pipeline 
 *  
 * \main\22 Tue Dec 31 11:12:39 2002 szawadzx 
 * FLOS100018654: Packet RX and TX integration with PR6 
 *  
 * \main\20 Sat Dec 14 19:28:58 2002 jpwysocz 
 * Corrected channel handling procedure (now functionality works on all memory 
 * banks - not only on ZERO). 
 *  
 * \main\19 Sat Dec 14 16:44:53 2002 jpwysocz 
 * Corrected meta data library usag, due to changes in library. 
 *  
 * \main\18 Sat Dec 14 16:15:59 2002 gnawrot 
 * FLOS100018434: meta data is not flushed back to SRAM in forwarding pipeline 
 *  
 * \main\17 Fri Dec 13 11:58:58 2002 gnawrot 
 * FLOS100018422: WRED moved to forwarding pipeline 
 *  
 * \main\16 Wed Dec 11 17:32:31 2002 jstelter 
 * Bug: #FLOS100018405
 * L2 Decap microblock should be moved to forwarding pipeline. 
 *  
 * \main\15 Fri Dec  6 18:57:55 2002 jstelter 
 * Bug: #FLOS100018345
 * Problem with sending ATM cells - there is no ATM cell ending AAL5 PDU 
 *  
 * \main\14 Thu Dec  5 18:14:53 2002 jstelter 
 * Bug: #FLOS100018345
 * To early the transfer register was freed. 
 *  
 * \main\13 Thu Dec  5 10:19:45 2002 szawadzx 
 * FLOS100018341: Removed dl_packet_path register 
 *  
 * \main\12 Tue Nov 26 14:39:22 2002 jstelter 
 * L2 Decap moved back to RX Microblock 
 *  
 * \main\11 Fri Nov 22 18:09:18 2002 jstelter 
 * removed unnecessary checking in dl_source() 
 *  
 * \main\10 Wed Nov 20 16:58:14 2002 jstelter 
 * Corrected dl_next_block settings. 
 *  
 * \main\9 Fri Nov  8 12:13:14 2002 gnawrot 
 * new parameter in dl_qm_sink() 
 *  
 * \main\8 Tue Nov  5 16:52:54 2002 gnawrot 
 * modified dl_qm_sink() 
 *  
 * \main\7 Fri Oct 25 14:49:17 2002 gnawrot 
 * correct next block set in dl_wred_source() 
 *  
 * \main\6 Wed Oct 23 14:59:02 2002 szawadzx 
 * Modified dl_sink 
 *  
 * \main\5 Mon Oct 21 16:38:19 2002 gnawrot 
 * dl_source() modified 
 *  
 * \main\4 Mon Oct 21 10:41:43 2002 szawadzx 
 * Added _dl_pos_sink_init definition 
 *  
 * \main\3 Fri Oct 18 16:40:40 2002 gnawrot 
 * added dl_set_exception_priority() 
 *  
 * \main\2 Mon Oct 14 16:36:59 2002 szawadzx 
 * Made some modifications required after memory mam has been defined 
 *  
 * \main\1 Tue Oct  8 16:42:13 2002 szawadzx
 * Initial version 
 *  
 *****************************************************************************/
