#ifndef	DL_SYSTEM_H
#define	DL_SYSTEM_H

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 2003 By Intel Corporation.  All rights reserved.
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

/* FILE  dl_system.h
 *
 * This file contains the system wide parameters that can be
 * customised for a given hardware/application
 */

#ifndef IXP2400
    #ifdef IXP2800
	    #error "Define only IXP2400."
    #endif
#endif

/*******************************************************************************
 *                           INCLUDE SECTION
 *******************************************************************************/

#include <dl_system_config.h>
#include <definitions.h>

#ifdef MICROCODE
#ifndef XSCALE_INIT_FOR_LINUX
#pragma warning(disable: 5145)
#endif
#endif

/*******************************************************************************
 *                           SRAM/DRAM SECTION
 *******************************************************************************/

/*------------------------------------------------------------------------------
 * BUFFER related definitions
 */

/*	Size of packet buffer. */
#define	BUFFER_SIZE						2048

/*	Amount of SDRAM space available for packet buffers.*/
#define BUF_SDRAM_SIZE					(BUFFER_SIZE * NUMBER_OF_BUFFERS)

/*
 *	Size of Meta Data (Buffer Descriptors) in SRAM. 32 entries per buffer.
 * 	Should be a power of 2.
 */
#define	META_DATA_SIZE					32

/*
 *	Amount of SRAM space available for (meta data of) packet buffers.
 */
#define BUF_SRAM_SIZE					(META_DATA_SIZE * NUMBER_OF_BUFFERS)

#ifdef USE_IMPORT_VAR

 /* This is workaround for IPV4 code in SDK 3.1 */
 #define STATS_TABLE_SRAM_BASE IPV4_STATS_TABLE_BASE

.import_var NEXT_HOP_TABLE_SRAM_BASE
.import_var UPSTREAM_BUF_SDRAM_BASE
.import_var DOWNSTREAM_BUF_SDRAM_BASE

.import_var	UPSTREAM_BUF_LW0_SRAM_BASE
.import_var	UPSTREAM_BUF_LW0_CHANNEL_NUMBER
.import_var	DOWNSTREAM_BUF_LW0_SRAM_BASE
.import_var	DOWNSTREAM_BUF_LW0_CHANNEL_NUMBER

.import_var UPSTREAM_DL_REL_BASE
.import_var DOWNSTREAM_DL_REL_BASE

.import_var BUF_FREE_LIST0

.import_var SRAM_PORT_BIT_MAP_BASE
.import_var SRAM_FLT_TABLE_BASE


#endif

/*******************************************************************************
 *                           DIFFSERV SECTION
 *******************************************************************************/

/*------------------------------------------------------------------------------
 * L2 Encap definitions
 */
#ifdef USE_IMPORT_VAR
.import_var L2_TABLE_SRAM_BASE
#endif

/*------------------------------------------------------------------------------
 * LLC ENACP definitions
 */

/* 2 ^ PO2_NEXT_HOP_ENTRY_SIZE = NEXT_HOP_ENTRY_SIZE */
#define		PO2_NEXT_HOP_ENTRY_SIZE					4


/* Size of each next hop entry in bytes. */
#define		NEXT_HOP_ENTRY_SIZE						(1 << PO2_NEXT_HOP_ENTRY_SIZE)

/* Number of next hop entries */
#define		NUMBER_OF_NEXT_HOP_ENTRIES				DOWNSTREAM_TX_MAX_VCS

/*------------------------------------------------------------------------------
 * CLASSIFIER definitions
 */
#ifndef USE_IMPORT_VAR
    #define	CLASSIFIER_6T_DEFAULT_FLOW_ID			0x1
    #define	CLASSIFIER_6T_DEFAULT_COLOR_ID			0x1
    #define	CLASSIFIER_6T_DEFAULT_CLASS_ID			0x00
    #define	CLASSIFIER_6T_DEFAULT_NEXTHOP_ID		0xFFFF
    #define CLASSIFIER_6T_DEFAULT_NEXTHOP_ID_TYPE	0x00
#endif /* USE_IMPORT_VAR */

/*------------------------------------------------------------------------------
 * SRTCM METER definitions
 */

/* Signal used to synchronize meter threads on particular microengines */
#define	SRTCM_NEXT_THREAD_SIG         	7

/*
 * Signal used to synchronize meter crtitical sections
 * on different microengines
 */
#define SRTCM_NEXT_ME_SIG             	11

#ifdef DOWNSTREAM
	#define FIRST_SRTCM_ME 0x01
#else
	#define FIRST_SRTCM_ME 0x10
#endif

/*
 * signal used to initialize timestamp on every timestamp
 */
#define   SRTCM_TIMESTAMP_SIG           9

/*------------------------------------------------------------------------------
 * L2QOS definitions
 */
/* Base LM address used by the L2QOS is settled just after the WRED */
#define L2QOS_MAP_LM_BASE		        512

/*------------------------------------------------------------------------------
 * DSCP Marker definitions
 */

#define DSCP_MARKER_IPV4_PACKET 		PKT_PROTO_IPv4	/* PPP_IPV4_TYPE */
#define DSCP_MARKER_IPV6_PACKET 		PKT_PROTO_IPv6	/* PPP_IPV6_TYPE */
#define DSCP_MARKER_MPLS_PACKET 		PKT_PROTO_NULL

/*
 * Scheduler Diffserv definitions
 */

/*
 * Number of ports
 * In the current implementation this number can't be greater than 16
 * It is restricted by the algorithm of chosing port to schedule.
 */
#define NUMBER_OF_BITS_FOR_PORT     2
#define NUMBER_OF_PORTS             (1 << NUMBER_OF_BITS_FOR_PORT)

/*
 * Local memory offset for port data structures
 */
#define PORT_LM_BASE_OFFSET		    1152


/*
 * define ATM_DB0 when ATM Daughtercard is placed in DB0
 * define ATM_DB1 when ATM Daughtercard is placed in DB1  (default configuration !!!)
 */
/* #define ATM_DB0 */
#ifndef ATM_DB0
#define ATM_DB0
#endif

#ifdef ATM_DB0
/*
 *	For Jamestown with SPI 0 utilized by ATM 0:0 and SPI 1 utilized by ETH Fiber 1
 */
#define RXTX_ATM_LINE					0			/* Physical MSF channel (SPI 0) */
#define RXTX_ETH_LINE					1			/* Physical MSF channel (SPI 1) */

#else
    #ifdef ATM_DB1
    /*
     *	For Jamestown with SPI 1 utilized by ATM 1:0 and SPI 0 utilized by ETH Fiber 0
     */
    #define RXTX_ATM_LINE				1			/* Physical MSF channel (SPI 1) */
    #define RXTX_ETH_LINE		    	0			/* Physical MSF channel (SPI 0) */

    #else
        /*
         *	For Simulation purposes with SPI 1 utilized by ATM and SPI 2 utilized by ETH
         */
        #define RXTX_ATM_LINE			1			/* Physical MSF channel (SPI 1) */
        #define RXTX_ETH_LINE			2			/* Physical MSF channel (SPI 2) */

    #endif /* ATM_DB1 */
#endif /* ATM DB0 */

#define AAL5_TX_LINE					RXTX_ATM_LINE	/* Needed by aal5_tx */

/*------------------------------------------------------------------------------
 * Common RX/TX  definitions
 */
#define MAX_BUFFER_OFFSET		        (DDR_BANK_SIZE * NO_OF_DDR_BANKS)

/* Define place where Context Descriptor starts in LM */
#define COMMON_RX_CTX_DESC_LM_BASE      0x400
#define CONTEXT_DESC_BASE               COMMON_RX_CTX_DESC_LM_BASE

/* Number of bytes occupied by the Context structure in LM */
#define COMMON_RX_CTX_SIZE		        (1 << 6)		/* in bytes. Must be power of 2. */
/*#define CONTEXT_SIZE			        COMMON_RX_CTX_SIZE*/

/* in LP_OC12 we have only one instance of the ETH port and associated L2 port
 * This association is kept in LM at the location specified below. The entry
 * will be initialized from EthRXInit.ind file. Make sure that L2PORT_MAP_BASE
 * defined there has the same value as defined below
 */
#define	L2PORT_MAP_BASE	(CONTEXT_DESC_BASE + (16 * COMMON_RX_CTX_SIZE)) /* 0x800 */
#define	L2PORT_MAP_ENTRY_SIZE	1

#ifdef USE_IMPORT_VAR
    .import_var ETH_L2_PORT_NUM
#else
    #define ETH_L2_PORT_NUM              1
#endif /* USE_IMPORT_VAR */

/*------------------------------------------------------------------------------
 * Packet RX  definitions
 */

/*
 *  free list id that is stored in meta data
 */
#ifndef USE_IMPORT_VAR
    #define	FREE_LIST_ID                 0x0
#endif

/*
 *	Receive Phy mode
 */
#define	RX_PHY_MODE    					SPHY_4_8

/*
 *	Base address POS Receive Contexts in Local Memory
 */
#define	POS_RX_CTX_LM_BASE    			0x0

/*
 *	Size of each POS Receive Context (in Bytes)
 */
#define	POS_RX_CTX_SIZE       			64

/*
 *	Total number of POS Receive Contexts.
 */
#define	POS_RX_CTX_TOTAL      			16

/* ports controlled by TX ME */
#define	PORTS_CTRLED_BY_FIRST_PACKET_TX_ME		0xFF00


/*-----------------------------------------------------------------------------
 * ALL5 RX defines
 */

/* Default offset is 7 */
#define CLP_BIT_OFFSET_IN_DL_META4	3

/*
 * For UTOPIA, there is only one partition
 * Total RBUF SIZE is 8 KB (8192 B), RBUF_ELE_SIZE = 64
 * no of rbuf elements = 8192 / 64 = 128
 */
#define	TOTAL_RBUF_SIZE         8192

/*
 * RBUF size should be the set to 128 for packet rx and aal5 rx
 */
#define RBUF_ELE_SIZE_LOG2	    7
#define RBUF_ELE_SIZE	        (1<<RBUF_ELE_SIZE_LOG2)

/* This is a pattern used to fill the VCs */
#define AAL5_RX_VC_HASH_FILLING 0x00000000

#ifdef DOWNSTREAM

#define VC_INFO_TOTAL 			DOWNSTREAM_TX_MAX_VCS
#define NO_OF_PORTS				DOWNSTREAM_TX_MAX_PORTS

#else /* DOWNSTREAM */

#define VC_INFO_TOTAL 			UPSTREAM_RX_MAX_VCS
#define NO_OF_PORTS				UPSTREAM_RX_MAX_PORTS

#endif /* DOWNSTREAM */

/*-----------------------------------------------------------------------------
 * Common TX defines
 */

#define TX_PHY_MODE				SPHY_4_8_SHARED

/* Number of Partitions for AAL5 TX - this is associated with TX_PHY_MODE*/
#define AAL5_TX_PARTITIONS_NUMBER           4

/* Size of the TBUF elements is common for both TXes (ATM and Packet) */
#define TBUF_ELE_SIZE_LOG2	    7
#define TBUF_ELE_SIZE	        (1<<TBUF_ELE_SIZE_LOG2)

/* Physical port to L2 port mapping has been moved into EthRXInit.ind */
#define TOTAL_TBUF_SIZE 		8192

/* Size of TCW in bytes. */
#define TCW_SIZE_LOG2			3

/* Define size of the Meta Data structure in SRAM (number of LWs) */
#define	  META_CACHE_SIZE		8

/*-----------------------------------------------------------------------------
 * Packet TX defines
 */

/*-----------------------------------------------------------------------------
 * AAL5 TX defines
 */
/* Define number AAL5 TX ME's in a functional pipeline */
/* If this design does not have an AAL5 TX functional pipeline, set to 1 */
#define			AAL5_TX_NUMBER_OF_FP_MEs	1

/* Set number of VCs in downstream direction */
#define AAL5_TX_NUMBER_OF_VC            DOWNSTREAM_TX_MAX_VCS

/*
 * Used to flow control cell transmission
 * set tbuf threshold to the total of available tx buffer pool per single
 * channel minus number of threads serving single channel
 */
#define 		NUM_OF_THREADS_PER_CHANNEL	8

/* Definition of the number of ATM ports used by the ATM TX */
#define	AAL5_TX_ME01					0x07

/*-----------------------------------------------------------------------------
 * ATM TM 4.1 defines
 */

/*
 *   QM and TX defines
 */
#ifdef DOWNSTREAM
    #define NUM_CLASS_BITS 					0
#else
#ifndef NUM_CLASS_DEFINED
    #define NUM_CLASS_BITS 					4
#endif
#endif /* DOWNSTREAM */


/* port number is coded on two bits: ports 0, 1, 2 ,3 */
/* the most significant three bits of 11-bits long port number */
#define NUM_PORT_BITS 					    3

/* Traffic Classes definition DO NOT CHANGE it here */
#define			RTQUEUE							rt_write#
#define			NRTQUEUE						nrt_write#
#define			UBRQUEUE						ubr_write#

/* Traffic Classes to SAR queue map */
#define			CLASS_CBR						RTQUEUE
#define			CLASS_RTVBR						RTQUEUE
#define			CLASS_NRTVBR					NRTQUEUE
#define			CLASS_UBR_PCR					UBRQUEUE
#define			CLASS_UBR_MDCR					UBRQUEUE
#define			CLASS_GFR						NRTQUEUE

/* Total number of Time Queues for each traffic class */
#define TQ_TOT 4096

/* AGGREGATION given as 2^AGGREGATION_LOG2 */
/* This is a timeslot aggregation factor   */
#define AGGREGATION_LOG2 4

/* Aggegation level in a time queue*/
#define AGGREGATION 				(1 << AGGREGATION_LOG2)
#define	AGGREGATION_MASK 			(AGGREGATION - 1)


/*------------------------------------------------------------------------------
 * QUEUE definitions
 */

/*
 *	One queue (per direction) is used as a DROP QUEUE. We have to specify
 *  the number unique number that is not used by the modules to perform
 *  packets forwarding. It have to exceed number of queues by one (number of
 *  VCs, etc.)
 */
/* ETH-2-ATM - we have to assign number according to VCs */
#define DOWNSTREAM_QM_DROP_QUEUE	    DOWNSTREAM_TX_MAX_VCS

/* ATM-2-ETH - we have to assign number according to VCs */
#define UPSTREAM_QM_DROP_QUEUE	        UPSTREAM_RX_MAX_VCS

/*
 * First 16 (0-15) SRAM Q-Array entries are allocated for the Queue Manager
 */
#define			QM_QARRAY_BASE			0
#define			QM_QARRAY_SIZE			16

/*
 *	Next 4 (16-19) SRAM Q-Array entries are allocated for buffer free list
 */
#define			BUF_QARRAY_BASE			16
#define			BUF_QARRAY_SIZE			4

/* This is a POOL_ID needed to create a second free list */
#define         BUF_FREE_LIST1			(BUF_FREE_LIST0 + 1)

/*
 *  Next 1 (20) SRAM Q-Array entry is used for Drop Queue in Queue Manager.
 */
#ifndef USE_IMPORT_VAR
#define		    QM_DROP_QUEUE_ENTRY		20
#endif

/* QM_SHAPER is located in ME 0:2 */
#define QM_SHAPER_ME					0x02

/*
 *	Total number of Queue Descriptors
 */
#define UPSTREAM_QD_TOTAL				(UPSTREAM_TX_MAX_PORTS*16)
/*
 *	Size of each Queue Descriptor Entry (in Bytes)
 */
#define	UPSTREAM_QD_SIZE				(1<<QD_LOG_SRAM_SIZE)
/*
 *	Total number of Queue Descriptors
 */
#define	DOWNSTREAM_QD_TOTAL				DOWNSTREAM_TX_MAX_VCS

/*
 *	Size of each Queue Descriptor Entry (in Bytes)
 */
#define DOWNSTREAM_QD_SIZE				(1<<QD_LOG_SRAM_SIZE)

/*------------------------------------------------------------------------------
 * QUEUE definitions
 */

#ifdef	DOWNSTREAM


#ifndef USE_IMPORT_VAR
   	#define QD_SRAM_BASE	DOWNSTREAM_QD_SRAM_BASE
    /* 1 queue per each VC - in simulation max number of VCs is 1024 */
   	#define QD_TOTAL		DOWNSTREAM_QD_TOTAL
   	/* Base address of the SRAM Channel used by QM */
	#define QD_SRAM_CHANNEL_BASE_ADDRESS		DOWNSTREAM_QD_SRAM_CHANNEL_BASE_ADDRESS

#endif

	#define QD_SIZE			                    DOWNSTREAM_QD_SIZE

	/* Channel number of the Queue Manager */
	#define SRAM_LW0_CHANNEL_NUMBER				DOWNSTREAM_BUF_LW0_CHANNEL_NUMBER
	#define BUF_SRAM_CHANNEL_LW0   				SRAM_LW0_CHANNEL_NUMBER

	#define SRAM_REST_CHANNEL_NUMBER			DOWNSTREAM_BUF_LW0_CHANNEL_NUMBER
	#define BUF_SRAM_CHANNEL_REST  				SRAM_LW0_CHANNEL_NUMBER

	#define	BUF_SRAM_BASE		   				DOWNSTREAM_BUF_LW0_SRAM_BASE

    /* DRAM base address for buffers */
	#define BUF_SDRAM_BASE         				DOWNSTREAM_BUF_SDRAM_BASE

    #define QM_DROP_QUEUE                       DOWNSTREAM_QM_DROP_QUEUE

#else /* DOWNSTREAM */

#ifndef USE_IMPORT_VAR
	#define QD_SRAM_BASE	UPSTREAM_QD_SRAM_BASE
	/* 16 queues on each of 4 ports */
	#define QD_TOTAL		UPSTREAM_QD_TOTAL
	/* Base address of the SRAM Channel */
	#define QD_SRAM_CHANNEL_BASE_ADDRESS		UPSTREAM_QD_SRAM_CHANNEL_BASE_ADDRESS
#endif

	#define QD_SIZE			UPSTREAM_QD_SIZE

	/* Channel number of the Queue Manager */
	#define SRAM_LW0_CHANNEL_NUMBER				UPSTREAM_BUF_LW0_CHANNEL_NUMBER
	#define BUF_SRAM_CHANNEL_LW0   				SRAM_LW0_CHANNEL_NUMBER

	#define SRAM_REST_CHANNEL_NUMBER			UPSTREAM_BUF_LW0_CHANNEL_NUMBER
	#define BUF_SRAM_CHANNEL_REST  				SRAM_LW0_CHANNEL_NUMBER

	#define	BUF_SRAM_BASE		   				UPSTREAM_BUF_LW0_SRAM_BASE

    /* DRAM base address for buffers */
	#define BUF_SDRAM_BASE         				UPSTREAM_BUF_SDRAM_BASE

    #define QM_DROP_QUEUE                       UPSTREAM_QM_DROP_QUEUE

#endif

/*------------------------------------------------------------------------------
 * IPv4 Forwarder definitions
 */

/*
 *	Each Next Hop entry is 16 bytes.
 */
#define NEXTHOP_TABLE_ENTRY_SIZE		16
#define NEXTHOP_TABLE_NUMBER_OF_ENTRIES	4096  /* 256 */


/*------------------------------------------------------------------------------
 * 6T CLASSIFIER definitions
 */
#define HASH_KEY_SIZE					15						/* CMW reduced by half */
#define HASH_TABLE_NUMBER_OF_ENTRIES 	(1 << HASH_KEY_SIZE)	/* depends on HASH_KEY_SIZE */
#define HASH_TABLE_ENTRY_SIZE 			32

/*------------------------------------------------------------------------------
 * SRTCM METER definitions
 */
#define	SRTCM_ENTRY_SRAM_SIZE			64
#define	SRTCM_NUMBER_OF_ENTRIES			0xFF



/*------------------------------------------------------------------------------
 * WRED definitions
 */
#define DOWNSTREAM_WRED_ENTRY_SRAM_SIZE	            64
#define DOWNSTREAM_WRED_64BIT_STAT_ENTRY_SRAM_SIZE	16
#define UPSTREAM_WRED_ENTRY_SRAM_SIZE	            64
#define UPSTREAM_WRED_64BIT_STAT_ENTRY_SRAM_SIZE	16

/*
 * signal used to synchronize wred threads on particular microengines
 */
#define	WRED_NEXT_THREAD_SIG     	    5

/*
 * signal used to synchronize wred crtitical sections on different microengines
 */

#define WRED_NEXT_ME_SIG				6

#ifdef	DOWNSTREAM
	#define FIRST_WRED_ME 0x01
#else
	#define FIRST_WRED_ME 0x10
#endif

/*
 * Base address in sram for the wred table
 */
#ifndef USE_IMPORT_VAR

    #ifdef	DOWNSTREAM
	    #define WRED_TABLE_SRAM_BASE		DOWNSTREAM_WRED_TABLE_SRAM_BASE
    	#define WRED_ENTRY_SRAM_SIZE		DOWNSTREAM_WRED_ENTRY_SRAM_SIZE
	    #define WRED_64BIT_STAT_SRAM_BASE	DOWNSTREAM_WRED_64BIT_STAT_SRAM_BASE
    	#define WRED_64BIT_STAT_SRAM_SIZE	DOWNSTREAM_WRED_64BIT_STAT_SRAM_SIZE
    #else
	    #define WRED_TABLE_SRAM_BASE		UPSTREAM_WRED_TABLE_SRAM_BASE
    	#define WRED_ENTRY_SRAM_SIZE		UPSTREAM_WRED_ENTRY_SRAM_SIZE
    	#define WRED_64BIT_STAT_SRAM_BASE	UPSTREAM_WRED_64BIT_STAT_SRAM_BASE
	    #define WRED_64BIT_STAT_SRAM_SIZE	UPSTREAM_WRED_64BIT_STAT_SRAM_SIZE
    #endif

#endif /* USE_IMPORT_VAR */

#ifdef EGRESS

#ifdef SCHEDULER
/*-------------------------------------------------------------------------------
 * DiffServ SCHEDULER definitions
 */

/* MTU for the media */
#define   MTU							    (9 * 1024)

/*
 * Mask defining high priority queues on ports
 */
#define   HIGH_PRIORITY_MASK			    0x000000FF

#define   SCHEDULER_MAX_NO_PENDING_REQUESTS	12

#endif /* SCHEDULER */

#endif /* EGRESS */

/*
 *	These tell if a the executing threads need to work in order. i.e wait for
 *	a signal from prev thread, do some work and signal next thread.
 */
#define			DL_THREAD_NO_ORDER				0x0
#define			DL_THREAD_ORDER					0x1

/*
 *	All the IX_* constants should be between 0 and 255 (i.e 1 byte) to
 *	enable using them as immed operands in instructions.
 *
 *	IX_NULL is used to denote a NULL (or empty) packet handle, It should not
 *  be zero, as it would interfere with ring empty (0) condition.
 *	Note: IX_NULL is too generic a name and may clash with core components code.
 *	So it's marked for deletion.  Start using UC_NULL
 */
#define			IX_NULL				0xFF
#define			IX_EXCEPTION		0x1
#define			IX_DROP				0x2
#define			UC_NULL				0xFF	/* NULL for microcode */
#define			DL_BUF_NULL			UC_NULL /* NULL Buffer representation */

/*
 *	Block IDs for various blocks. Block IDs Start from 0x20.
 *  (0 to 0x1f and 0xFF are reserved for IX_* )
 */

/*	Block IDs for various blocks. Block IDs Start from 0x20. (0 to 0x1f and 0xFF are 
	reserved for IX_* ) */

#define 		BID_LLC_SNAP_DECAP				0x21
#define			BID_IPV4						0x22
#define			BID_QM							0x23
#define			BID_L2BR						0x26 
#define			BID_ETHER_PPP_DISCOVERY			0x27
#define			BID_ETHER_PPP_SESSION			0x28
#define         BID_LLC_ENCAP                   0x29
#define			BID_IPV6						BID_DROP
#define			BID_ARP							BID_DROP
#define			BID_PPP_DISCOVERY				BID_DROP
#define			BID_PPP_SESSION					BID_DROP
#define			BID_EDECAP		                0x30 /* L2 Eth Classifier - Decapsulation */
#define			BID_ADECAP						BID_LLC_SNAP_DECAP /* L2 ATM Classifier - Decapsulation */
#define			BID_ETHER			            BID_EDECAP       /* required by packet rx */
#define			BID_NULL		                UC_NULL		     /* NULL Block ID      */
#define			BID_EXCEPTION	                IX_EXCEPTION     /* Exception Block ID */
#define			BID_DROP		                IX_DROP		     /* Drop Block ID      */

/* 	Static Bindings (a.k.a targets) for LLCSNAP microblock. 
 	Should this go into project/application specific global header file? */ 
#define 		TARGET_LLCSNAP_IPV4				BID_IPV4
#define 		TARGET_LLCSNAP_IPV6				BID_IPV6
#define 		TARGET_LLCSNAP_ARP				BID_ARP
#define 		TARGET_LLCSNAP_PPP_DISCOVERY	BID_PPP_DISCOVERY
#define 		TARGET_LLCSNAP_PPP_SESSION		BID_PPP_SESSION
#define 		TARGET_LLCSNAP_ETHER			BID_ETHER
#define			TARGET_LLCSNAP_MPLS_UNICAST		BID_DROP 

/*
 * Define Default Targets for Ethernet microblock.
 */
#ifndef		TARGET_ETHER_DECAP_IPV4
#define TARGET_ETHER_DECAP_IPV4		BID_IPV4
#endif

#ifndef		TARGET_ETHER_DECAP_IPV6
#define TARGET_ETHER_DECAP_IPV6		BID_IPV6
#endif

#ifndef		TARGET_ETHER_DECAP_ARP
#define TARGET_ETHER_DECAP_ARP		BID_ARP
#endif

#ifndef		TARGET_ETHER_DECAP_MPLS
#define TARGET_ETHER_DECAP_MPLS		BID_ILM_CLASSIFIER
#endif

#ifndef		TARGET_ETHER_DECAP_PPP_DISCOVERY
#define TARGET_ETHER_DECAP_PPP_DISCOVERY	BID_ETHER_PPP_DISCOVERY
#endif

#ifndef		TARGET_ETHER_DECAP_PPP_SESSION
#define TARGET_ETHER_DECAP_PPP_SESSION		BID_ETHER_PPP_SESSION
#endif
 

#ifdef DOWNSTREAM
#define			IPV4_NEXT1			BID_LLC_ENCAP 
#else
#define			IPV4_NEXT1			BID_QM
#endif

#ifndef			IPV4_LOCAL_NEXT1
#define			IPV4_LOCAL_NEXT1				IX_EXCEPTION
#endif 

#define         BID_LLC_ENCAP_NEXT1 BID_QM

/* Changes for PR6 compatibility */
#define		BID_ETHER_DECAP_NEXT1	(ETHER_IPV4_TYPE)
#define		BID_ETHER_DECAP_NEXT2	(ETHER_IPV6_TYPE)
#define		BID_ETHER_DECAP_NEXT3	(ETHER_ARP_TYPE)

/* these defines are in a diffrent way !*/
#define		BID_PPP_DECAP_NEXT1	    (BID_IPV4)
#define		BID_PPP_DECAP_NEXT2	    (IX_DROP)
#define		BID_PPP_DECAP_NEXT3	    (BID_IPV6) 

/* 	Definitions for ATM AAL5 RX */
#define			AAL5_RX_NEXT1					BID_ADECAP /* BID_LLC_SNAP_DECAP */
#define         AAL5_RX_FIRST_FREELIST			0		/* It uses Freelist_0 */
#define         AAL5_RX_THREAD_FREELIST			RX_THREAD_FREELIST_0

#ifndef USE_IMPORT_VAR
        /* mask is corelated to the size of the table */
        #define AAL5_RX_VC_HASH_PRI_INDEX_MASK      	(AAL5_RX_VC_HASH_PRI_BKT_NUMBER - 1)
#endif

/* is a fourth part of the PRI */
#define AAL5_RX_VC_HASH_SEC_BKT_NUMBER			(AAL5_RX_VC_HASH_PRI_BKT_NUMBER >> 2)

/* mask is corelated to the size of the table */
#define AAL5_RX_VC_HASH_SEC_INDEX_MASK          (AAL5_RX_VC_HASH_SEC_BKT_NUMBER - 1)

/*	BLADE ID  */
#ifndef USE_IMPORT_VAR
#define	THIS_BLADE_ID					        1
#endif

#ifdef DOWNSTREAM

#define MAX_ATM_PORTS					DOWNSTREAM_TX_MAX_PORTS

#else /* DOWNSTREAM */

#define MAX_ATM_PORTS					UPSTREAM_RX_MAX_PORTS

#endif /* DOWNSTREAM */

//	Signals being a precious resource, we use a single signal for 
//	inter-thread as well as any other inter-me signalling (init over signal)

#define			COMMON_INTER_THD_ME_SIGNAL		10

//	We are not able to use a single signal before of limitations in assembler.
//	until then, we need a separate signal for inter-thread signalling.

#define			INTER_THD_SIGNAL				12

//	ME_NIT_SIGNAL signal is used to let everyone know that initialisation is over
//	and all blocks in all MEs can start running. This signal will be typically issued
//	by the system initialisation code and all other MEs should wait for this signal
//	before starting to run.

#define			ME_INIT_SIGNAL					COMMON_INTER_THD_ME_SIGNAL
 
// 	Signal number to use in inter thread signalling. 

#define 		DL_SIG_WAKE				INTER_THD_SIGNAL 

/*==============================================================================
 *                                Scratch rings
 *
 *------------------------------------------------------------------------------
 */



/* Scratch rings number 0 and 1 are used for ME-CC communication */

/*
 *	Scratch for communicating between FWD and QM Blocks
 */
#define	FWD_TO_QM_SCR_RING_SIZE		            512
#define	DOWNSTREAM_FWD_TO_QM_SCR_RING			2
#define	UPSTREAM_FWD_TO_QM_SCR_RING				3

#ifdef	DOWNSTREAM
    #define	FWD_TO_QM_SCR_RING			        DOWNSTREAM_FWD_TO_QM_SCR_RING
#else
    #define	FWD_TO_QM_SCR_RING			        UPSTREAM_FWD_TO_QM_SCR_RING
#endif

#define L2_ENCAP_RING_OUT_0         FWD_TO_QM_SCR_RING
#ifndef CORE_CODE

#ifndef XSCALE_INIT_FOR_LINUX
#define_eval DL_DROP_RING	        FWD_TO_QM_SCR_RING
#else
#define DL_DROP_RING			    FWD_TO_QM_SCR_RING
#endif

#endif
#define	FWD_TO_QM_MSG_SIZE			            3		/* 3 longwords */


/*
 *	Scratch for communicating between Scheduler and Queue Manager Blocks
 */
#define SCHEDULER_TO_QM_SCR_RING_SIZE			256

#define DOWNSTREAM_SCHEDULER_TO_QM_SCR_RING		4
#define UPSTREAM_SCHEDULER_TO_QM_SCR_RING		5

#ifdef DOWNSTREAM
    #define SCHEDULER_TO_QM_SCR_RING		    DOWNSTREAM_SCHEDULER_TO_QM_SCR_RING
#else
    #define SCHEDULER_TO_QM_SCR_RING		    UPSTREAM_SCHEDULER_TO_QM_SCR_RING
#endif

/*
 *	Scratch for communicating between Queue Manager and ETH TX Transmit Blocks
 *	Ring number 8, base 1536, and size 128 Long Words
 */
#define	QM_TO_TX_SCR_RING_SIZE		            128 /* was 256 */
#define	DOWNSTREAM_QM_TO_TX_SCR_RING			6
#define	UPSTREAM_QM_TO_TX_0_SCR_RING			7


#ifdef DOWNSTREAM
    #define	QM_TO_TX_SCR_RING			        DOWNSTREAM_QM_TO_TX_SCR_RING
#else
    #define	QM_TO_TX_0_SCR_RING			        UPSTREAM_QM_TO_TX_0_SCR_RING
#endif

/*
 *	Scratch for communicating between Schaper and Scheduler Blocks
 *	Ring number 9, base 1536, and size 128 Long Words
 */
#define	SHAPER_TO_SCHEDULER_SCR_RING_SIZE	    128

#define	DOWNSTREAM_SHAPER_TO_SCHEDULER_SCR_RING0		8
#define	DOWNSTREAM_SHAPER_TO_SCHEDULER_SCR_RING1		9


#ifdef DOWNSTREAM
    #define SHAPER_TO_SCHEDULER_SCR_RING0		DOWNSTREAM_SHAPER_TO_SCHEDULER_SCR_RING0
    #define SHAPER_TO_SCHEDULER_SCR_RING1		DOWNSTREAM_SHAPER_TO_SCHEDULER_SCR_RING1
#endif


/*
 *	Scratch for communicating between RX and Forwarding Blocks
 */
#define	RX_TO_FWD_SCR_RING_SIZE					512

#define	DOWNSTREAM_RX_TO_FWD_SCR_RING			10
#define	UPSTREAM_RX_TO_FWD_SCR_RING				11

#define	ETH_RX_TO_FWD_SCR_RING			        DOWNSTREAM_RX_TO_FWD_SCR_RING
#define	ATM_RX_TO_FWD_SCR_RING			        UPSTREAM_RX_TO_FWD_SCR_RING

#define L2_ENCAP_RING_IN_0                      ETH_RX_TO_FWD_SCR_RING

#define	RX_TO_FWD_MSG_SIZE			           	5		/* 5 longwords */


/*
 *  These defines are used by the individual microcode to identify their
 *  scratch rings. The reason for these defines is to improve the portability
 *  of microblocks. User just has to make changes here and the microblocks will
 *  remain untouched. For the naming convention, the microblock name goes first,
 *  then the _RING_ specifier. Following this is the direction indicator. '_OUT'
 *  would imply the microblock is writing to this scratch ring and '_IN' would
 *  imply the microblock is reading from this scratch ring. If there is more
 *  than one 'IN' or 'OUT' ring the name will have '_<digit>' to
 *  differentiate the rings.
 *
 *  NOTE!!! The second set of SCR defines is located in dl_mem_map.h
 */

/*
 *	The Ring on which RX sends out msg to next block
 */
#define	RX_RING_OUT_MSG_SIZE		    RX_TO_FWD_MSG_SIZE

#define PACKET_RX_PORT_MAP			    0x07
#define	PACKET_RX_FIRST_FREELIST	    1		/* It uses Freelist_1 */
#define PACKET_RX_THREAD_FREELIST	    RX_THREAD_FREELIST_1

/*
 *	The input ring on which forwarding receives msgs from prev block
 */
#define DOWNSTREAM_FWD_RING_IN          ETH_RX_TO_FWD_SCR_RING
#define UPSTREAM_FWD_RING_IN            ATM_RX_TO_FWD_SCR_RING

#ifdef DOWNSTREAM
    #define	FWD_RING_IN					DOWNSTREAM_FWD_RING_IN
#else
    #define	FWD_RING_IN					UPSTREAM_FWD_RING_IN
#endif

#define	FWD_RING_IN_MSG_SIZE			RX_TO_FWD_MSG_SIZE

/*
 * The ring on which forwarding sends out msgs to next block
 * (Queue Manager in this case)
 */
#define	FWD_RING_OUT					FWD_TO_QM_SCR_RING
#define	FWD_RING_OUT_MSG_SIZE			FWD_TO_QM_MSG_SIZE

/*
 * The input ring on which QM receives msgs from prev. block
 */

#ifdef  DOWNSTREAM

    #define QM_RING_IN					FWD_TO_QM_SCR_RING
    #define QM_RING_IN_SIZE				FWD_TO_QM_SCR_RING_SIZE

    #define QM_RING_IN_0				FWD_TO_QM_SCR_RING
    #define QM_RING_IN_0_SIZE			FWD_TO_QM_SCR_RING_SIZE

    #define QM_RING_IN_1				SCHEDULER_TO_QM_SCR_RING
    #define QM_RING_IN_1_SIZE			SCHEDULER_TO_QM_SCR_RING_SIZE

#else

    #define QM_RING_IN_0				FWD_TO_QM_SCR_RING
    #define QM_RING_IN_0_SIZE			FWD_TO_QM_SCR_RING_SIZE

    #define QM_RING_IN_1				SCHEDULER_TO_QM_SCR_RING
    #define QM_RING_IN_1_SIZE			SCHEDULER_TO_QM_SCR_RING_SIZE

#endif

/*
 * The output ring in which QM Sends msgs to next block
 */

#ifdef  DOWNSTREAM

    #define QM_RING_OUT_0				QM_TO_TX_SCR_RING
    #define QM_RING_OUT_0_SIZE			QM_TO_TX_SCR_RING_SIZE

    /* This QM_RING_OUT is used in SPHY mode only, but should be defined
       to avoid compilation errors. */
    #define QM_RING_OUT_1				QM_TO_TX_SCR_RING
    #define QM_RING_OUT_1_SIZE			QM_TO_TX_SCR_RING_SIZE

    #define	QM_RING_OUT_2				QM_TO_TX_SCR_RING
    #define	QM_RING_OUT_2_SIZE			QM_TO_TX_SCR_RING_SIZE

    #define	QM_RING_OUT_3				QM_TO_TX_SCR_RING
    #define	QM_RING_OUT_3_SIZE			QM_TO_TX_SCR_RING_SIZE

    #define	QM_RING_OUT					QM_TO_TX_SCR_RING
    #define	QM_RING_OUT_SIZE			QM_TO_TX_SCR_RING_SIZE

#else

    #define	QM_RING_OUT_0				QM_TO_TX_0_SCR_RING
    #define	QM_RING_OUT_0_SIZE			QM_TO_TX_SCR_RING_SIZE

    /*
     * This QM_RING_OUT is used in SPHY mode only, but should be defined
     * to avoid compilation errors.
     */
    #define	QM_RING_OUT_1				QM_TO_TX_0_SCR_RING
    #define	QM_RING_OUT_1_SIZE			QM_TO_TX_SCR_RING_SIZE

    #define	QM_RING_OUT_2				QM_TO_TX_0_SCR_RING
    #define	QM_RING_OUT_2_SIZE			QM_TO_TX_SCR_RING_SIZE

    #define	QM_RING_OUT_3				QM_TO_TX_0_SCR_RING
    #define	QM_RING_OUT_3_SIZE			QM_TO_TX_SCR_RING_SIZE

#endif

/*
 * The output ring in which Sheduler Sends msgs to next block
 * (QM in this case)
 */

#ifdef DOWNSTREAM

    #define SCHEDULER_RING_OUT_0		SCHEDULER_TO_QM_SCR_RING
    #define SCHEDULER_RING_OUT_0_SIZE	SCHEDULER_TO_QM_SCR_RING_SIZE

#else

    #define SCHEDULER_RING_OUT			SCHEDULER_TO_QM_SCR_RING
    #define SCHEDULER_RING_OUT_SIZE		SCHEDULER_TO_QM_SCR_RING_SIZE

#endif

#ifdef DOWNSTREAM

    #define AAL5_TX_IN_0				QM_TO_TX_SCR_RING
    #define AAL5_TX_IN_0_SIZE			QM_TO_TX_SCR_RING_SIZE

#else
    /*
     * The input ring on which ETH TX receives msgs from prev. block
     */
    #define PACKET_TX_IN_0				QM_TO_TX_0_SCR_RING
    #define PACKET_TX_IN_0_SIZE			QM_TO_TX_SCR_RING_SIZE

    #define PACKET_TX_IN_1				QM_TO_TX_1_SCR_RING
    #define PACKET_TX_IN_1_SIZE			QM_TO_TX_SCR_RING_SIZE

#endif

#ifndef CORE_CODE
#ifndef CC_TEST_CONSOLE
#ifndef USE_IMPORT_VAR
	#include "dl_mem_map.h"
#endif /* USE_IMPORT_VAR */
#endif /* CC_TEST_CONSOLE */
#endif /* CORE_CODE */

#ifdef USE_IMPORT_VAR
	#ifdef DOWNSTREAM
		#define DL_REL_BASE		DOWNSTREAM_DL_REL_BASE
	#else
		#define DL_REL_BASE		UPSTREAM_DL_REL_BASE
	#endif

    #define BUF_SRAM_CHANNEL_NOT_ZERO

#else
	#ifdef DOWNSTREAM
        #ifndef CORE_CODE

        #ifndef XSCALE_INIT_FOR_LINUX
            #define_eval BUF_SRAM_BASE		DOWNSTREAM_BUF_LW0_SRAM_BASE
		    #define_eval BUF_SDRAM_BASE		DOWNSTREAM_BUF_SDRAM_BASE
        #else
            #define BUF_SRAM_BASE           DOWNSTREAM_BUF_LW0_SRAM_BASE
		    #define BUF_SDRAM_BASE          DOWNSTREAM_BUF_SDRAM_BASE
        #endif

        #else
            #define BUF_SRAM_BASE           DOWNSTREAM_BUF_LW0_SRAM_BASE
		    #define BUF_SDRAM_BASE          DOWNSTREAM_BUF_SDRAM_BASE
        #endif
	#else
        #ifndef CORE_CODE
        #ifndef XSCALE_INIT_FOR_LINUX
            #define_eval BUF_SRAM_BASE		UPSTREAM_BUF_LW0_SRAM_BASE
            #define_eval BUF_SDRAM_BASE		UPSTREAM_BUF_SDRAM_BASE
        #else
            #define BUF_SRAM_BASE           UPSTREAM_BUF_LW0_SRAM_BASE
            #define BUF_SDRAM_BASE          UPSTREAM_BUF_SDRAM_BASE
        #endif
        #else
            #define BUF_SRAM_BASE           UPSTREAM_BUF_LW0_SRAM_BASE
            #define BUF_SDRAM_BASE          UPSTREAM_BUF_SDRAM_BASE
        #endif
	#endif

	#define BUF_SRAM_BASE_LW			(BUF_SRAM_BASE + META_DATA_SIZE)
	#define BUF_SDRAM_BASE_LW			(BUF_SDRAM_BASE + BUFFER_SIZE)

	#define DL_REL_BASE		    		(BUF_SDRAM_BASE - ((BUF_SRAM_BASE & 0x3fffffff) * DL_DS_RATIO))
#endif

#endif /* DL_SYSTEM_H */

