#ifndef PKTHDR_CACHE_UC
#define PKTHDR_CACHE_UC
/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 2003 By Intel Corporation.  All rights reserved.
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

// IP header definitions
#ifndef	PKT_HDR_CACHE_SIZE
#define	PKT_HDR_CACHE_SIZE		10	// 16 longwords
#endif

#ifndef			PKT_HDR_DRAM_RD_SIZE
#define_eval	PKT_HDR_DRAM_RD_SIZE	(PKT_HDR_CACHE_SIZE >> 1)	// read 4 quadwords
#endif


#define HDR_CACHE_LOG_SIZE		5



//	Define Reassembly Context in Local memory.
//	Local memory base should be at a multiple of context_size

#define DL_CONTEXT_DESC_BASE		1024
#define	DL_CONTEXT_SIZE				4    // in LW

///////////////////////////////////////////////////////////////////////////////
//
//	Index into Local memory context cache (see data structure above)
//
///////////////////////////////////////////////////////////////////////////////

#define	P_FWD_DBASE				0
#define IN_PORT_ST_STATE		1
#define OUT_VLAN_PORT_INDEX		2
#define OUT_PORT_STATUS			3

///////////////////////////////////////////////////////////////////////////////
//
//	Short cuts for Context Cache in local memory
//
///////////////////////////////////////////////////////////////////////////////

#define	lm_p_fwd_dbase			*l$index1[P_FWD_DBASE]
#define	lm_in_port_st_state		*l$index1[IN_PORT_ST_STATE]
#define lm_out_vlan_port_index	*l$index1[OUT_VLAN_PORT_INDEX]
#define	lm_out_port_status		*l$index1[OUT_PORT_STATUS]


#macro  thread_num(out_th_num)

	local_csr_rd[ACTIVE_CTX_STS]
	immed[out_th_num, 0]
	alu[out_th_num, 0x7, and, out_th_num]

#endm


#define _PKT_HDR_IP_HEADER_SIZE		28
#define _PKT_HDR_IP_HEADER_SIZE_LW	(_PKT_HDR_IP_HEADER_SIZE >> 2)

#ifndef DOWNSTREAM


//------------------------------------------------------------------------------
// dl_shift_iphdr:
//	 	Description:
//			Moves data from in_buffer to out_buffer and shifts it if the offset
//			value in metadata modulo 8 equals 0 or 6
//
//	 	Outputs:
//			out_buffer	:	GPRs or write transfer regs buffer prefix
//
//		Inputs:
//			in_buffer	:	GPRs or read transfer regs buffer prefix
//
//		Size:   				:
//

#macro dl_shift_iphdr(out_buffer, in_buffer)
.begin
//.reg	buf_offset

	local_csr_wr[byte_index, 2]
	dl_meta_get_offset[dl_cache_shift]

	alu[dl_cache_shift, 0x7, AND, dl_cache_shift]


	br!=byte[dl_cache_shift, 0, 6, check_shift2#]
//	.if (dl_cache_shift == 6)

		#define	OFFSET	1
		#while (OFFSET < PKT_HDR_CACHE_SIZE)
			#define_eval	OFFSET_M	(OFFSET - 1)
			alu[out_buffer/**/OFFSET_M, --, B, in_buffer/**/OFFSET]
			#define_eval	OFFSET		(OFFSET + 1)
		#endloop
		#undef	OFFSET
		#undef	OFFSET_M
	br[shift_end#]

check_shift2#:

#if (!streq('out_buffer', 'in_buffer'))

	br!=byte[dl_cache_shift, 0, 2, check_shift0#]
//	.elif (dl_cache_shift == 2)

		#define	OFFSET	0
		#while (OFFSET < PKT_HDR_CACHE_SIZE)
			alu[out_buffer/**/OFFSET, --, B, in_buffer/**/OFFSET]
			#define_eval	OFFSET		(OFFSET + 1)
		#endloop
		#undef	OFFSET
	br[shift_end#]


#endif

check_shift0#:
	br!=byte[dl_cache_shift, 0, 0x0, shift_end#]
//	.elif (dl_cache_shift == 0)

		byte_align_be[--, in_buffer/**/0]
		#define	OFFSET	0
		#while (OFFSET <= PKT_HDR_CACHE_SIZE)
			byte_align_be[out_buffer/**/OFFSET, in_buffer/**/OFFSET]
			#define_eval OFFSET (OFFSET + 1)
		#endloop
		#undef	OFFSET

//	.endif

shift_end#:

.end
#endm



#else  // For DOWNSTREAM


//------------------------------------------------------------------------------
// dl_shift_iphdr:
//	 	Description:
//			Moves data from in_buffer to out_buffer and shifts it if the offset
//			value in metadata modulo 8 equals 0 or 6
//
//	 	Outputs:
//			out_buffer	:	GPRs or write transfer regs buffer prefix
//
//		Inputs:
//			in_buffer	:	GPRs or read transfer regs buffer prefix
//
//			in_inbuffer_tbyte_base	:	GPR containing base adress that should
//										be put in t_index_byte_index register
//
//		Size:   				:
//


#macro dl_shift_iphdr(out_buffer, in_buffer)
.begin
.reg	buf_offset reg_offset

	dl_meta_get_offset[buf_offset]

	br_bclr[buf_offset, 2, shift_18#], defer[1]
		alu[dl_cache_shift, 0x7, AND, buf_offset]

shift_14#:

		alu[out_buffer[0], --, B, in_buffer[3]]
		alu[out_buffer[1], --, B, in_buffer[4]]
		alu[out_buffer[2], --, B, in_buffer[5]]
		alu[out_buffer[3], --, B, in_buffer[6]]
		alu[out_buffer[4], --, B, in_buffer[7]]
		br[endshift#], defer[2]
		alu[out_buffer[5], --, B, in_buffer[8]]
		alu[out_buffer[6], --, B, in_buffer[9]]

shift_18#:

		alu[out_buffer[0], --, B, in_buffer[4]]
		alu[out_buffer[1], --, B, in_buffer[5]]
		alu[out_buffer[2], --, B, in_buffer[6]]
		alu[out_buffer[3], --, B, in_buffer[7]]
		alu[out_buffer[4], --, B, in_buffer[8]]
		alu[out_buffer[5], --, B, in_buffer[9]]
		alu[out_buffer[6], --, B, in_buffer[10]]
		alu[out_buffer[7], --, B, in_buffer[11]]

endshift#:

.end
#endm

#endif



//------------------------------------------------------------------------------
// dl_shift_iphdr_back
//	 	Description: 
//			Copies data from in_buffer to out_buffer, shifting it in_shift - 2
//			bytes right (big endian format)
//
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//
#macro dl_shift_iphdr_back(out_buffer, in_buffer, in_shift)
.begin

	local_csr_wr[byte_index, 2]

	br!=byte[in_shift, 0, 6, check_shift2#]
//	.if (in_shift == 6)

		#define_eval	OFFSET	(_PKT_HDR_IP_HEADER_SIZE_LW - 1)
		#while (OFFSET >= 0)
			#define_eval OFFSET_OUT (OFFSET + 1)
			alu[out_buffer/**/OFFSET_OUT, --, B, in_buffer/**/OFFSET]
			#define_eval OFFSET (OFFSET - 1)
		#endloop
		#undef	OFFSET
		#undef	OFFSET_OUT
	br[shift_back_end#]

check_shift2#:
#if (!streq('out_buffer', 'in_buffer'))
	br!=byte[in_shift, 0, 2, check_shift0#]
//	.elif (dl_cache_shift == 2)

		#define	OFFSET	0
		#while (OFFSET <= PKT_HDR_CACHE_SIZE)
			alu[out_buffer/**/OFFSET, --, B, in_buffer/**/OFFSET]
			#define_eval	OFFSET		(OFFSET + 1)
		#endloop
		#undef	OFFSET
	br[shift_back_end#]
#endif

check_shift0#:
	br!=byte[in_shift, 0, 0x0, shift_back_end#]
//	.elif (in_shift == 0)

		byte_align_be[--, in_buffer/**/0]

		#define	OFFSET	1
		#while (OFFSET <= (_PKT_HDR_IP_HEADER_SIZE_LW + 1))
			#define_eval OFFSET_OUT (OFFSET - 1)
			byte_align_be[out_buffer/**/OFFSET_OUT, in_buffer/**/OFFSET]
			#define_eval OFFSET (OFFSET + 1)
		#endloop
		#undef	OFFSET
		#undef	OFFSET_OUT

//	.endif

shift_back_end#:
.end
#endm


//------------------------------------------------------------------------------
// dl_load_hdr_cache:
//	 	Description: 
//			Reads the L2 header (xx bytes) of the packet from SDRAM and keeps 
//			it in a cache (DRAM transfer registers). 
//
//	 	Outputs:
//			out_header dram transfer registers will be set with appropriate
//			header data.
//
//		Inputs:
//			buf_handle	:			Handle of buffer.
//			req_sig		:			Signal to use in the I/O (DRAM read) operation.
//			sig_action	:			What to do with I/O operation - 
//									- wait for signal(s) as specified by sig_action
//									- do not wait for signal, just return (SIG_NONE)
//		Size:   				:
//
#macro dl_load_hdr_cache[out_header, in_buf_handle, in_req_sig, in_sig_action]
.begin
.reg 	sdram_offset, buf_offset

	//	Read 32 bytes of L2 Header from SDRAM

	dl_buf_get_data[sdram_offset, in_buf_handle]
	dl_meta_get_offset[buf_offset]
	dram[read, out_header, sdram_offset, buf_offset, PKT_HDR_DRAM_RD_SIZE],
		sig_done[in_req_sig]	; DRAM reads are in QWORD (8 bytes)

#if (!streq('in_sig_action', 'SIG_NONE'))
	ctx_arb[in_sig_action]
#endif

.end
#endm


//------------------------------------------------------------------------------
// dl_write_hdr_cache:
//	 	Description: 
//			Writes the IP header (xx bytes) of the packet to SDRAM 
//
//	 	Outputs:
//
//		Inputs:
//			in_header dram transfer registers
//			buf_handle	:			Handle of buffer.
//			req_sig		:			Signal to use in the I/O (DRAM write) operation.
//			sig_action	:			What to do with I/O operation - 
//									- wait for signal(s) as specified by sig_action
//									- do not wait for signal, just return (SIG_NONE)
//		Size:   				:
//
#macro dl_write_hdr_cache[in_header, in_buf_handle, in_req_sig, in_sig_action]
.begin
.reg 	sdram_offset, buf_offset

	//	write 32 bytes of IP Header to SDRAM

	dl_buf_get_data[sdram_offset, in_buf_handle]
	dl_meta_get_offset[buf_offset]
   	alu[buf_offset,buf_offset,+,2] 
	dram[write, in_header, sdram_offset, buf_offset, 2],
		sig_done[in_req_sig]	; DRAM writes are in QWORD (8 bytes)

#if (!streq('in_sig_action', 'SIG_NONE'))
	ctx_arb[in_sig_action]
#endif

.end
#endm

//------------------------------------------------------------------------------
// dl_iphdr_cache_init:
//	 	Description: 
//			This macro needs to be called before using iphdr registers
//			It allocates a set of GPRs to be used for caching.
//			This macro doesn't generate any code (instructions). It only
//			produces some assembler directives.
//
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//			0 Instructions
#macro dl_iphdr_cache_init[]

	xbuf_alloc($$iphdr, (PKT_HDR_CACHE_SIZE + 1), read_write)
#endm

//------------------------------------------------------------------------------
// dl_iphdr_cache_fini:
//	 	Description: 
//			This macro needs to be called at the end when iphdr registers are no 
//			longer required. It frees a set of DRAM xfer registers that were
//			allocated using dl_iphdr_cache_init. This macro	doesn't generate any
//			code (instructions). It only produces some assembler directives.
//
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//			0 Instructions

#macro dl_iphdr_cache_fini[]

	xbuf_free[$$iphdr]

#endm


#endif	//PKTHDR_CACHE_UC
/*******************************  FILE HISTORY  *******************************
 * \main\1 Wed Aug 13 14:03:18 2003 adziecia
 * IGK0100025024: ATM-IPV4-ETH: simulation project check-in 
 *  
 *****************************************************************************/
