#ifndef RM_QARRAY_UC
#define RM_QARRAY_UC

/*******************************************************************************
    INTEL CONFIDENTIAL
  
    Copyright 2000 - 2003 Intel Corporation All Rights Reserved.
  
    The source code contained or described herein and all documents related to
    the source code ("Material") are owned by Intel Corporation or its 
    suppliers or licensors.
  
    Title to the Material remains with Intel Corporation or its suppliers and
    licensors. The Material contains trade secrets and proprietary and
    confidential information of Intel or its suppliers and licensors.
    The Material is protected by worldwide copyright and trade secret laws and
    treaty provisions. No part of the Material may be used, copied, reproduced,
    modified, published, uploaded, posted, transmitted, distributed,
    or disclosed in any way without Intel's prior express written permission.
  
    No license under any patent, copyright, trade secret or other intellectual
    property right is granted to or conferred upon you by disclosure
    or delivery of the Materials, either expressly, by implication, inducement,
    estoppel or otherwise. Any license under such intellectual property rights
    must be express and approved by Intel in writing.
  
  
*******************************************************************************/

//  rm_qarray.uc
//
//  Initalization routines for SRAM Q-Array hardware.

// Exported APIs:
//
//  rm_qarray_init()
//  rm_qarray_init_chn(QARRAY_INIT_SRAM_BASE, QARRAY_CHANNEL)
//


//------------------------------------------------------------------------------
// Constants
#define RM_QARRAY_CHANNEL_NOT_PRESENT 0xffffffff

//Dfault number of SRAM channels
#ifndef IX_SRAM_CHANNELS_NUMBER
	#if (__IXPTYPE & __IXP2800)
    	#define IX_SRAM_CHANNELS_NUMBER 4
	#else
    	#define IX_SRAM_CHANNELS_NUMBER 2
    #endif
#endif


//------------------------------------------------------------------------------
// Patched symbols
#ifdef USE_IMPORT_VAR

    .import_var QARRAY_INIT_SRAM_BASE_CH0
    .import_var QARRAY_INIT_SRAM_BASE_CH1

    #if (__IXPTYPE == __IXP2800)
        .import_var QARRAY_INIT_SRAM_BASE_CH2
        .import_var QARRAY_INIT_SRAM_BASE_CH3
    #endif
#endif



//------------------------------------------------------------------------------
// rm_qarray_init()
//
//  Description:
//      This macro will initialize Q-array hardware for all channels for current
//      processor type.
//      Macro assumes that either Resource Manager running on XScale
//      or ind scripts will initialize sram memory with queeue descriptors
//      for eatch q-array entry on eatch channel.
//
//      Macro will expect that following
//      base addressed will declared in headed defining memory map:
//          QARRAY_INIT_SRAM_BASE_CH0
//          QARRAY_INIT_SRAM_BASE_CH1
//          QARRAY_INIT_SRAM_BASE_CH0 (only for IXP 2800)
//          QARRAY_INIT_SRAM_BASE_CH1 (only for IXP 2800)
//
//  Inputs: NONE
//
//  See also:
//      rm_qarray_init(QARRAY_INIT_SRAM_BASE)
//
//------------------------------------------------------------------------------
#macro rm_qarray_init()
.begin

    rm_qarray_init_chn(QARRAY_INIT_SRAM_BASE_CH0, 0)

    #if (IX_SRAM_CHANNELS_NUMBER > 1)
        rm_qarray_init_chn(QARRAY_INIT_SRAM_BASE_CH1, 1)
    #endif

    #if (IX_SRAM_CHANNELS_NUMBER > 2)
        rm_qarray_init_chn(QARRAY_INIT_SRAM_BASE_CH2, 2)
    #endif

    #if (IX_SRAM_CHANNELS_NUMBER > 3)
        rm_qarray_init_chn(QARRAY_INIT_SRAM_BASE_CH3, 3)
    #endif
.end
#endm

//------------------------------------------------------------------------------
// rm_qarray_init()
//
//  Description:
//      Macro initializaes all available qarray entries for SRAM channel
//      specified in given in_qarray_sram_base.
//      Macro should be used when qarray is not initialized by microcode
//      but rather by core component or scripts whitch prepares
//      queue descriptors and writes them to sram at given location.
//      This macro wil simly read those descriptors from sram into
//      Qarray hardware.
//
//  Inputs:
//      QARRAY_INIT_SRAM_BASE   Base LW address in sram that contains
//                              queue descriptors, either constant or GPR
//      QARRAY_CHANNEL          SRAM channel
//
//
//    Inputs:  NONE
//
//------------------------------------------------------------------------------
#macro rm_qarray_init_chn(QARRAY_INIT_SRAM_BASE, QARRAY_CHANNEL)
.begin
    // Following constants define range of Qarray entries to be read from sram
    // for each channel
    // Some applications may not use all qarray entries, in sach case they may
    // provide this definition to speed up initialization process
    #ifndef QARRAY_INIT_START_INDEX
        #define QARRAY_INIT_START_INDEX       0
    #endif
    #ifndef QARRAY_INIT_END_INDEX
        #define QARRAY_INIT_END_INDEX         63
    #endif

    #if (QARRAY_INIT_START_INDEX > QARRAY_INIT_END_INDEX)
        #error "QARRAY_INIT_START_INDEX must not be greater that QARRAY_INIT_END_INDEX"
    #endif

    .reg sram_base_reg
    .reg channel_num
    .reg count
    .reg sram_base_lw
	.reg qarray_entry

    ;This is eather pathed symbol or constant, in ether case it will work
    immed[sram_base_reg, (QARRAY_INIT_SRAM_BASE & 0xFFFF)]
    immed_w1[sram_base_reg, ((QARRAY_INIT_SRAM_BASE >> 16) & 0xFFFF)]

    ;Check special value indicating, that channel should not be initialized
    ;Checking byte 3 is enought - not possible that it will represent real
    ;address (since cahnnel is nod patched)
    br=byte[sram_base_reg, 3, ((RM_QARRAY_CHANNEL_NOT_PRESENT >>24) & 0xFF), no_channel#]

    ;Extract channel number from inout address (2 MSB bits)
    alu[channel_num, --, B, (QARRAY_CHANNEL), <<30]

    ;Number of channels to initialize
    immed[count, QARRAY_INIT_END_INDEX]

    ;Read queue descriptor for all 64 qarray entries
    .repeat
        ;Combine qarray entry index and sram channel
        alu[qarray_entry, channel_num, OR, count, <<24]

        ;Add sram offset into qarray_entry (index * 4 ), we can safely use
        ;OR since Qarray index is stored on bits 29:24, and sram LW offset
        ;is (63*4) at most. Adding qarray_entry and sram_base_lw will
        ;give us correct address.
        alu[qarray_entry, qarray_entry, OR, count, <<2]

        ;Read descriptor into Q-array hardware
        .reg $qd_return
        .sig qarray_signal

        sram[rd_qdesc_head, $qd_return, qarray_entry,sram_base_reg,2], sig_done[qarray_signal]
        sram[rd_qdesc_other, --, qarray_entry, sram_base_reg]

        ctx_arb[qarray_signal]

        alu[count, count, -, 1]
    .until (count < QARRAY_INIT_START_INDEX)

    //Channel is not present or not used
    no_channel#:

.end
#endm



#endif  //  RM_QARRAY_UC
