#ifndef SYSTEM_INIT_UC
#define SYSTEM_INIT_UC

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 2003 By Intel Corporation.  All rights reserved.
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

// FILE  system_init.uc
//

#include <dispatch_loop.uc>

#ifdef USE_IMPORT_VAR
    #include <rm_qarray.uc>
#endif

// define AAL5 RX parity - parity selection should be moved to aal5_rx.h - TBD
#ifdef UP_CTRL_PARITY
#undef UP_CTRL_PARITY
#define UP_CTRL_PARITY  		(RX_PARITY_EVEN << RX_PARITY_BITS)
#endif


//------------------------------------------------------------------------------
// init_meta_cache_to_zero:
//	 	Description:
//			Just sets the registers used for meta data caching to zero. [This
//			helps a lot when debugging, because all the unused meta data regsiters
//			will be zero instead of something like 0x5a5a5a).
//
//		Outputs:
//
//	 	Inputs:
//
//		Size:
//
#macro	init_meta_cache_to_zero[]
.begin
.reg zero
	immed32[zero, 0]
	dl_meta_init_cache[zero, zero, zero, zero, zero, zero, zero, zero]
.end
#endm

//------------------------------------------------------------------------------
// system_init:
//	 	Description:
//			Does all the initialisation to get the system up and running.
//			Typically it initialises all the various rings, creates the buffer
//			free list, initialises the various cache (meta, ip header etc).
//			Once initialisation is done, a signal is sent to all other MEs
//			so that rest of the code, that depends on this initialisation,
//			to run.
//
//		Outputs:
//
//	 	Inputs:
//
//		Size:
//
//
#macro	system_init[]
.begin

#ifdef USE_IMPORT_VAR

    ;Initiazlize q-arays with descriptors prepared by core components
    rm_qarray_init()

#else
	// Initialise the various scratch rings used in the ingress pipeline
	init_scratch_ring[DOWNSTREAM_RX_TO_FWD_SCR_RING_BASE,
						RX_TO_FWD_SCR_RING_SIZE,
						DOWNSTREAM_RX_TO_FWD_SCR_RING]
	init_scratch_ring[DOWNSTREAM_FWD_TO_QM_SCR_RING_BASE,
						FWD_TO_QM_SCR_RING_SIZE,
						DOWNSTREAM_FWD_TO_QM_SCR_RING]
	init_scratch_ring[DOWNSTREAM_SCHEDULER_TO_QM_SCR_RING_BASE,
						SCHEDULER_TO_QM_SCR_RING_SIZE,
						DOWNSTREAM_SCHEDULER_TO_QM_SCR_RING]
	init_scratch_ring[DOWNSTREAM_QM_TO_TX_SCR_RING_BASE,
						QM_TO_TX_SCR_RING_SIZE,
						DOWNSTREAM_QM_TO_TX_SCR_RING]

	init_scratch_ring[UPSTREAM_RX_TO_FWD_SCR_RING_BASE,
						RX_TO_FWD_SCR_RING_SIZE,
						UPSTREAM_RX_TO_FWD_SCR_RING]
	init_scratch_ring[UPSTREAM_FWD_TO_QM_SCR_RING_BASE,
						FWD_TO_QM_SCR_RING_SIZE,
						UPSTREAM_FWD_TO_QM_SCR_RING]
	init_scratch_ring[UPSTREAM_SCHEDULER_TO_QM_SCR_RING_BASE,
						SCHEDULER_TO_QM_SCR_RING_SIZE,
						UPSTREAM_SCHEDULER_TO_QM_SCR_RING]
	init_scratch_ring[UPSTREAM_QM_TO_TX_0_SCR_RING_BASE,
						QM_TO_TX_SCR_RING_SIZE,
						UPSTREAM_QM_TO_TX_0_SCR_RING]

#endif  /* USE_IMPORT_VAR */

	.sig cap_write
	.reg $enable_timestamp

	immed[$enable_timestamp, 0x300]
   	cap[write, $enable_timestamp, MISC_CONTROL], ctx_swap[cap_write] ; set 7th bit(Timestamp Enable)

	// To ease debugging. Not absoultely essential for proper execution.
	init_meta_cache_to_zero[]		; all the dl_meta* gprs will be set to 0.

	.sig remote sys_init_signal
	signal_me[0x01, &remote(sys_init_signal, 0x01)]	; ME 1
	signal_me[0x02, &remote(sys_init_signal, 0x02)]	; ME 2
	signal_me[0x03, &remote(sys_init_signal, 0x03)]	; ME 3

	// The Next 4 MEs are in Cluster 1.
	signal_me[0x10, &remote(sys_init_signal, 0x10)]	; ME 4
	signal_me[0x11, &remote(sys_init_signal, 0x11)]	; ME 5
	signal_me[0x12, &remote(sys_init_signal, 0x12)]	; ME 6
	signal_me[0x13, &remote(sys_init_signal, 0x13)]	; ME 7

.end
#endm


#endif	//  SYSTEM_INIT_UC
