/*===========================================================================
 * = COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 * ===========================================================================
 */

/* =========================================================================*/
/**
 * @file   ix_sa_cc_list.c
 *
 * Definition of CC_INITIALIZATION funciton.
 *
 **/
/*=========================================================================*/
/* $Id: ix_sa_cc_list.c,v 1.1.2.1 2004/03/11 17:43:24 ksandowi Exp $ */
#include "ix_cc_error.h"
#include "sa/ix_sa_cc_list.h"

#include "cc/ix_cc_qm_frmwrk.h"
#include "cc/ix_cc_ipv4.h"
#include "cc/ix_cc_eth_rx.h"
#include "cc/ix_cc_eth_tx.h"
#include "cc/ix_cc_sched.h"

#include "cc/ix_cc_atmsar_main_cci.h"
#include "cc/ix_cc_atmsar_agent_cci.h"
#include "cc/ix_cc_atmsartest_cci.h"

#include "cc/ix_cc_atmrx_cci.h"
#include "cc/ix_cc_atmtx_cci.h"
#include "cc/ix_cc_atmtm41_cci.h"
#include "ix_cc_llc_snap_encap.h"
#include "pos_atm_media_drv_cci.h"

char g_sa_uofFile[] = "/ixa/oc12_atm_ipv4_eth.uof";

IX_CC_QM_INSTANCE_TABLE_DEFINE()
IX_CC_QM_INSTANCE_ENTRY_DEFINE(IX_CC_QM_0_MSG_INPUT,IX_CC_QM_0_COMMON_PKT_INPUT,IX_CC_QM_0_EGRESS_PKT_INPUT,IX_CC_QM_0_EGRESS_PKT_OUTPUT,IX_CC_QM_0_UBLOCK_PKT_OUTPUT)
IX_CC_QM_INSTANCE_ENTRY_DEFINE(IX_CC_QM_1_MSG_INPUT,IX_CC_QM_1_COMMON_PKT_INPUT,IX_CC_QM_1_EGRESS_PKT_INPUT,IX_CC_QM_1_EGRESS_PKT_OUTPUT,IX_CC_QM_1_UBLOCK_PKT_OUTPUT)
IX_CC_QM_INSTANCE_TABLE_END


#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
/*  _ix_sa_set_cc */
/*  ---------------------------------------- */
/**
 * This function is used to set a init and fini function for a CC
 *
 * @return IX_SUCCESS upon success or an ix_error with one of the following values.
 * @retval NONE
**/
ix_error
_ix_sa_set_cc(ix_sa_ccfunc *arg_pCC_List, ix_uint32 arg_idx,
              ix_cc_init arg_Init, ix_cc_fini arg_Fini)
{
    ix_uint16 i = 0;

    IX_ERROR_CHECK_ARG_NULL(1, arg_pCC_List);
    arg_pCC_List[arg_idx].init = arg_Init;
    arg_pCC_List[arg_idx].fini = arg_Fini;
    for (i = 0; i < IX_CC_INSTANCE_MAX; i++)
    {
        arg_pCC_List[arg_idx].cc_handle[i] = 0;
    }
    return IX_SUCCESS;
}
#endif


/**
 * NAME: _ix_cc_application_get_instance
 *
 * DESCRIPTION: This function will retrieve the default instance of a CC.
 *
 * @Param:  - IN arg_pContext - context passed to the CC creation function
 *
 * @Return: This function returns the CC instance.
 */
ix_uint32
_ix_cc_application_get_instance(
void* arg_pContext)
{
    return (((ix_cc_init_context*)arg_pContext)->instanceCC );
}

/**
 * Application specific function pointer for CC instance retrieval function
 */
ix_cc_get_instance g_CCApplicationGetInstanceFPtr = _ix_cc_application_get_instance;

ix_uint32
ix_sa_init_hook_first(void)
{
    return (IX_SUCCESS);
}

ix_uint32
ix_sa_init_hook_pre_ee(void)
{
    ix_uint32 ret;

    /* register application specific multiinstance support function */
    ret = IX_CCI_CC_GET_INSTANCE_REGISTER(g_CCApplicationGetInstanceFPtr);
    if (!ret)
    {
        ix_ossl_message_log("Cannot register application specific multiinstance support function.");
        return(ret);
    }

    return (IX_SUCCESS);
}

ix_uint32
ix_sa_init_hook_pre_me(void)
{
    return (IX_SUCCESS);
}

ix_uint32
ix_sa_init_hook_last(void)
{
    ix_error err;

    /* driver initialization starts here */
    IX_ERROR_CG(ix_cc_atmdrv_init(), err, e_eng);
#ifdef ATM_DB0
    IX_ERROR_CG(ix_cc_atmdrv_linkup(0, 0), err, e_eng);
    IX_ERROR_CG(ix_cc_atmdrv_linkup(0, 1), err, e_eng);
#endif

#ifdef ATM_DB1
    IX_ERROR_CG(ix_cc_atmdrv_linkup(1, 0), err, e_eng);
    IX_ERROR_CG(ix_cc_atmdrv_linkup(1, 1), err, e_eng);
#endif

#ifdef ATM_DB0_DB1
    IX_ERROR_CG(ix_cc_atmdrv_linkup(0, 0), err, e_eng);
    IX_ERROR_CG(ix_cc_atmdrv_linkup(0, 1), err, e_eng);
    IX_ERROR_CG(ix_cc_atmdrv_linkup(1, 0), err, e_eng);
    IX_ERROR_CG(ix_cc_atmdrv_linkup(1, 1), err, e_eng);
#endif

e_eng:
    return (err);
}

ix_uint32
ix_sa_shutdown_hook_first(void)
{
    return (IX_SUCCESS);
}

ix_uint32
ix_sa_shutdown_hook_post_me(void)
{
    return (IX_SUCCESS);
}

ix_uint32
ix_sa_shutdown_hook_post_ee(void)
{
    return (IX_SUCCESS);
}

ix_uint32
ix_sa_shutdown_hook_last(void)
{
    return (IX_SUCCESS);
}

/*  _ix_sa_cc_list */
/*  ---------------------------------------- */
/**
 * This function generates and returns the ix_sa_ccfunc array.
 *
 * An array containing pointers to various cc functions is returned
 * to be used during cc initialization.
 *
 * @return IX_SUCCESS, a pass through ix_error, or an ix_error with one
 *         of the following values.
 * @retval IX_CC_ERROR_NULL arg_pCC_List is NULL
**/
ix_error
_ix_sa_cc_list(ix_sa_ccfunc *arg_pCC_List)
{
    IX_ERROR_CHECK_ARG_NULL(1, arg_pCC_List);

    /* add one entry for each CC in this configuration.  example below.
     *   _ix_sa_set_cc
     *          Param #1: the cclist passed to _ix_sa_cc_list (arg_pCC_List)
     *          Param #2: Identifier of CC as defined in bindings.h
     *          Param #3: The CCs init function (include header in this file)
     *          Param #4: The CCs fini function
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_DUMMY, _ix_sa_dummy_cc_init, _ix_sa_dummy_cc_fini);
    */

    /* ATM RX */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_ATMRX, ix_cc_atmrx_init, ix_cc_atmrx_fini);

    /* ATM TX */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_ATMTX, ix_cc_atmtx_init, ix_cc_atmtx_fini);

    /* ATM TM 4.1 */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_ATMTM41, ix_cc_atmtm41_init, ix_cc_atmtm41_fini);

    /* ATM SAR MAIN */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_ATMSAR_MAIN, ix_cc_atmsar_init, ix_cc_atmsar_fini);

    /* Queue Manager */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_QM, ix_cc_qm_init, ix_cc_qm_fini);

    /* IP v.4 */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_IPV4, ix_cc_ipv4_init, ix_cc_ipv4_fini);
   
    /* Ethernet RX */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_ETH_RX, ix_cc_eth_rx_init, ix_cc_eth_rx_fini);

    /* Ethernet TX */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_ETH_TX, ix_cc_eth_tx_init, ix_cc_eth_tx_fini);

    /* Scheduler */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_SCHEDULER, ix_cc_sched_init, ix_cc_sched_fini);

    /* LLC SNAP encap */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_LLC_SNAP_ENCAP, ix_cc_llc_snap_encap_init, ix_cc_llc_snap_encap_fini);
    
    return IX_SUCCESS;
}

