/*===========================================================================
 * = COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 * ===========================================================================
 */

/* =========================================================================*/
/**
 * @file   ix_sa_symbols.c
 *
 * Patches misc symbols for this configuration.
 **/
/*=========================================================================*/
#define IX_ERROR_FILE_IDENT "$Id: ix_sa_symbols.c,v 1.2.2.1 2004/03/11 17:44:12 ksandowi Exp $"


#include "ix_cc_error.h"
#include "sa/internal/internal_sa.h"
#include "sa/ix_sa_cc_list.h"


/*  _ix_sa_patch_symbols */
/*  ---------------------------------------- */
/**
 * Patch miscelanious symbols.
 *
 * This function patches a variety of symbols not specifially associated
 * with a given Core Component.
 *
 *
 * @param arg_sa_ctrl System Application Control Structure
 * @return IX_SUCCESS, a pass through ix_error, or an ix_error with one
 *         of the following values.
 * @retval NONE
**/
ix_error
_ix_sa_patch_symbols(ix_sa_ctrl *arg_sa_ctrl)
{
#if !defined(IX_TARGET_SIMNTgnu_Debug) && !defined(IX_TARGET_SIMNTgnu)
    ix_imported_symbol aImportSymbols[7];
    ix_uint32 relBase = 0;
    ix_buffer_free_list_info hwFreeListInfo;
    void *pZeroBlock;
    ix_uint32 phys_meta_address = 0;
    ix_uint32 phys_data_address = 0;
    ix_memory_type memType;
    ix_uint32 channel = 0;
    ix_uint32 zboffset = 0;
    ix_uint32 ethL2PortNum = 0;
    void *pPhysicalAddress = NULL;
    ix_configuration_property_handle hCoreRoot = IX_CP_CORE_PROPERTY_ROOT;
    ix_configuration_property_handle hProperty;
    ix_uint32 ingressBladeId;


    IX_ERROR_CHECK_ARG_NULL(1, arg_sa_ctrl);

    /* allocate a long work in sram to patch to the zeroblock symbol */
    IX_ERROR_CR(ix_rm_mem_alloc(IX_MEMORY_TYPE_SRAM, 0,
                                16*sizeof(ix_uint32), &pZeroBlock));

    ix_ossl_memset(pZeroBlock, 0, 16*sizeof(ix_uint32));

    /* get FL_01 data (upstream) */
    IX_ERROR_CR(ix_rm_buffer_free_list_get_info(arg_sa_ctrl->freelist[1],
                                                &hwFreeListInfo));

    IX_ERROR_CR(ix_rm_get_phys_offset((void*)hwFreeListInfo.m_pMetaBaseAddress,
                                      &memType, &channel, &phys_meta_address,
                                      &pPhysicalAddress));

    IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(channel, phys_meta_address);

    IX_ERROR_CR(ix_rm_get_phys_offset((void*)hwFreeListInfo.m_pDataBaseAddress,
                                      &memType, &channel, &phys_data_address,
                                      &pPhysicalAddress));

    IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(channel, phys_data_address);


    relBase = phys_data_address - ( phys_meta_address * (hwFreeListInfo.m_DataElementSize/hwFreeListInfo.m_MetaElementSize));

    ix_ossl_message_log("DL_REL_BASE UPSTREAM: %lu\n"
                        "  phys_data_address: 0x%lx\n"
                        "  phys_meta_address: 0x%lx\n"
                        "  m_DataElementSize: %lu\n"
                        "  m_MetaElementSize: %lu\n",
                        relBase,
                        phys_data_address,
                        phys_meta_address,
                        hwFreeListInfo.m_DataElementSize,
                        hwFreeListInfo.m_MetaElementSize);

    aImportSymbols[0].m_Value = relBase;
    aImportSymbols[0].m_Name  = "UPSTREAM_DL_REL_BASE" ;

    /* patch these symbols on all ME's */
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[0],1,aImportSymbols));
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[4],1,aImportSymbols));
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[6],1,aImportSymbols));

    aImportSymbols[0].m_Value = phys_meta_address;
    /* (ix_uint32)((ix_uint32)hwFreeListInfo.m_pMetaBaseAddress & 0x3fffffff); */
    aImportSymbols[0].m_Name  = "UPSTREAM_BUF_LW0_SRAM_BASE" ;

    /* patch the symbols onto relevant microengines */
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[0],1,aImportSymbols));
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[4],1,aImportSymbols));
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[5],1,aImportSymbols));
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[6],1,aImportSymbols));

    aImportSymbols[0].m_Value = (ix_uint32)hwFreeListInfo.m_FreeListInfo1;
    aImportSymbols[0].m_Name  = "FREE_LIST_ID";
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[0],1,aImportSymbols));


    aImportSymbols[0].m_Value = (ix_uint32) hwFreeListInfo.m_FreeListInfo;
    aImportSymbols[0].m_Name  = "BUF_FREE_LIST0";

    /* patch these symbols on all ME's */
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[0],1,aImportSymbols));
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[1],1,aImportSymbols));
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[2],1,aImportSymbols));
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[4],1,aImportSymbols));
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[5],1,aImportSymbols));
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[6],1,aImportSymbols));
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[7],1,aImportSymbols));


    /* get FL_02 data (downstream) */
    IX_ERROR_CR(ix_rm_buffer_free_list_get_info(arg_sa_ctrl->freelist[2],
                                                &hwFreeListInfo));

    IX_ERROR_CR(ix_rm_get_phys_offset((void*)hwFreeListInfo.m_pMetaBaseAddress,
                                      &memType, &channel, &phys_meta_address,
                                      &pPhysicalAddress));

    IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(channel, phys_meta_address);

    IX_ERROR_CR(ix_rm_get_phys_offset((void*)hwFreeListInfo.m_pDataBaseAddress,
                                      &memType, &channel, &phys_data_address,
                                      &pPhysicalAddress));

    IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(channel, phys_data_address);

    relBase = phys_data_address - ( phys_meta_address * (hwFreeListInfo.m_DataElementSize/hwFreeListInfo.m_MetaElementSize));

    ix_ossl_message_log("DL_REL_BASE DOWNSTREAM: %lu\n"
                        "  phys_data_address: 0x%lx\n"
                        "  phys_meta_address: 0x%lx\n"
                        "  m_DataElementSize: %lu\n"
                        "  m_MetaElementSize: %lu\n",
                        relBase,
                        phys_data_address,
                        phys_meta_address,
                        hwFreeListInfo.m_DataElementSize,
                        hwFreeListInfo.m_MetaElementSize);

    aImportSymbols[0].m_Value = relBase;
    aImportSymbols[0].m_Name  = "DOWNSTREAM_DL_REL_BASE" ;

    /* patch these symbols on all ME's */
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[0],1,aImportSymbols));
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[1],1,aImportSymbols));
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[7],1,aImportSymbols));


    aImportSymbols[0].m_Value = phys_meta_address;
    /*    (ix_uint32)((ix_uint32)hwFreeListInfo.m_pMetaBaseAddress & 0x3fffffff);*/

    aImportSymbols[0].m_Name  = "DOWNSTREAM_BUF_LW0_SRAM_BASE" ;

    aImportSymbols[1].m_Value = 1;
    aImportSymbols[1].m_Name  = "DOWNSTREAM_BUF_LW0_CHANNEL_NUMBER" ;

    /* patch the symbols onto relevant microengines */
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[0],1,aImportSymbols));
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[1],2,aImportSymbols));
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[2],1,aImportSymbols));
    /*IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[3],2,aImportSymbols));*/
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[7],2,aImportSymbols));


    /*
    relBase = phys_data_address - ( phys_meta_address *
                                     (hwFreeListInfo.m_DataElementSize/hwFreeListInfo.m_MetaElementSize));


    aImportSymbols[0].m_Value =
        (ix_uint32)((ix_uint32)hwFreeListInfo.m_pMetaBaseAddress & 0x3fffffff);
    aImportSymbols[0].m_Name  = "BUF_SRAM_BASE" ;

    aImportSymbols[1].m_Value =  relBase ;
    aImportSymbols[1].m_Name  = "DL_REL_BASE" ;
    */

    /* Prepare the symbol array */
    IX_ERROR_CR(ix_rm_get_phys_offset((void*)pZeroBlock,
                                      &memType, &channel, &zboffset,
                                      &pPhysicalAddress));

    aImportSymbols[0].m_Value = (ix_uint32)zboffset;
    aImportSymbols[0].m_Name  = "SRAM_ZERO_BLOCK";
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[5],1,aImportSymbols));


#ifdef IX_INCLUDE_REGISTRY

    /* open the property */
    IX_ERROR_CRT(ix_rm_cp_property_open(hCoreRoot,
                                        "System_Properties/BLADE_ID",
                                        &hProperty),
                 IX_CC_ERROR_ENTRY_NOT_FOUND,
                 IX_ERROR_LEVEL_REMOTE);

    /* read the property */
    IX_ERROR_CRT(ix_rm_cp_property_get_value_uint32(hProperty,
                                                    &ingressBladeId),
                 IX_CC_ERROR_ENTRY_NOT_FOUND,
                 IX_ERROR_LEVEL_REMOTE);

    IX_ERROR_CRT(ix_rm_cp_property_close(hProperty),
                 IX_CC_ERROR_ENTRY_NOT_FOUND,
                 IX_ERROR_LEVEL_WARNING);


    /* open the property */
    IX_ERROR_CRT(ix_rm_cp_property_open(hCoreRoot,
                                        "System_Properties/ETH_L2_PORT_NUM",
                                        &hProperty),
                 IX_CC_ERROR_ENTRY_NOT_FOUND,
                 IX_ERROR_LEVEL_REMOTE);

    /* read the property */
    IX_ERROR_CRT(ix_rm_cp_property_get_value_uint32(hProperty,
                                                    &ethL2PortNum),
                 IX_CC_ERROR_ENTRY_NOT_FOUND,
                 IX_ERROR_LEVEL_REMOTE);

    IX_ERROR_CRT(ix_rm_cp_property_close(hProperty),
                 IX_CC_ERROR_ENTRY_NOT_FOUND,
                 IX_ERROR_LEVEL_WARNING);


#else /* IX_INCLUDE_REGISTRY */

    ingressBladeId = 0x1;
    ethL2PortNum = 0x1;

#endif /* IX_INCLUDE_REGISTRY */

    /* Prepare the symbol array */
    aImportSymbols[0].m_Value = ethL2PortNum;
    aImportSymbols[0].m_Name  = "ETH_L2_PORT_NUM" ;

    /* patch the symbols onto relevant microengines */
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[0],1,aImportSymbols));

#endif
    return IX_SUCCESS;
}


int isNPUMaster(void)
{
    return 0;
}

