/*****************************************************************************
 *
 *                INTEL CORPORATION PROPRIETARY INFORMATION
 * This software was created by Intel Technology Poland and is supplied under
 *     the terms of a license agreement or nondisclosure agreement with
 *      Intel Corporation and may not be copied or disclosed except in
 *              accordance with the terms of that agreement.
 *        Copyright(c) 2003 Intel Corporation. All Rights Reserved.
 *
 * REDISTRIBUTION OF THIS SOURCE CODE MUST RETAIN THE ABOVE COPYRIGHT NOTICE
 *
 *****************************************************************************/
/*****************************************************************************
 *
 * File:  pos_atm_media_drv_cci.c
 *
 * Abstract:
 *      This file contains a calls to pos_atm_media_drv wrapper functions
 * which allow open, configure and get current state and statistics from controlled
 * device
 *
 *****************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>

/* avoid including memory map and other associated files from application code */
#define CC_TEST_CONSOLE
#include "dl_system.h"
#undef CC_TEST_CONSOLE
#include "pos_atm_media_drv.h"
#include "pos_atm_media_drv_cci.h"

/* handle for DB0 DB1 respectively */
 int handle[2] = {POS_ATM_INV_HANDLE, POS_ATM_INV_HANDLE};


#define MCCR_MSK1  0x0000F0F0
#define MCCR_MSK2  0x000000F0
#define MCCR_MSK3  0x00000000

#define FREQ_25MHz      0x300
#define FREQ_50MHz      0x355
#define FREQ_104MHz     0x3AA
#define FREQ_125MHz     0x3FF
#define FREQ_104_125MHz 0x0EE

int msf_pll_1(unsigned int freq)
{
    volatile unsigned int temp_val;

    if (freq == 0)
    {
            freq = FREQ_104_125MHz;  /* default */
    }

    temp_val = ((unsigned int)MCCR_MSK1 | (freq << 16));
    *((volatile unsigned int *) 0xC0004A08) = temp_val;

    temp_val = ((unsigned int)MCCR_MSK2 | (freq << 16));
    *((volatile unsigned int *) 0xC0004A08) = temp_val;

    temp_val = ((unsigned int)MCCR_MSK3 | (freq << 16));
    *((volatile unsigned int *) 0xC0004A08) = temp_val;

    return 0;
}


/*****************************************************************************
 * Abstract:
 *    pos_atm_media_drv initialization function.
 *****************************************************************************/
int ix_cc_atmdrv_init(void)
{
    #ifdef ATM_DB0
        if (PosAtm_DriverStart("oc12,atm", "") == POS_ATM_FAILURE)
        {
            return(POS_ATM_FAILURE);
        }
    #endif

    #ifdef ATM_DB1
        if (PosAtm_DriverStart("", "oc12,atm") == POS_ATM_FAILURE)
        {
            return(POS_ATM_FAILURE);
        }
    #endif

    #ifdef ATM_DB0_DB1
        if (PosAtm_DriverStart("oc12,atm", "oc12,atm") == POS_ATM_FAILURE)
        {
            return(POS_ATM_FAILURE);
        }
    #endif

    return(0);
}


int ix_cc_atmdrv_fini(void)
{
    int i;

    for ( i = 0; i < 4; i++) {
        if (handle[i] != POS_ATM_INV_HANDLE) {
            PosAtm_Close(handle[i]);
        }
    }
    PosAtm_DriverStop();
    return(0);
}


int ix_cc_atmdrv_linkup(int db, int link)
{
    if ( db != 0 && db != 1) {
    }
    else
    {
        if (link != 0 && link != 1) {
        }
        else {
            if ( handle[db] == POS_ATM_INV_HANDLE) {
                /* the driver has not been opened for this db */
                if ((handle[db] = PosAtm_Open(db)) == POS_ATM_INV_HANDLE) {
                    return(0);
                }
            }
            /* lets set line state up for specified link */
            PosAtm_SetLinkState(handle[db], link, POS_ATM_STATE_UP);
        }
    }
    return(0);
}



int framer_stats(int db, int link, int channel)
{

    return 0;
} /* framer_stats */













