# ============================================================================
#  INTEL CONFIDENTIAL
#
#  Copyright 2003 Intel Corporation All Rights Reserved.
#
#  The source code contained or described herein and all documents related to
#  the source code ("Material") are owned by Intel Corporation or its 
#  suppliers or licensors.
#
#  Title to the Material remains with Intel Corporation or its suppliers and
#  licensors. The Material contains trade secrets and proprietary and
#  confidential information of Intel or its suppliers and licensors.
#  The Material is protected by worldwide copyright and trade secret laws and
#  treaty provisions. No part of the Material may be used, copied, reproduced,
#  modified, published, uploaded, posted, transmitted, distributed,
#  or disclosed in any way without Intel's prior express written permission.
#
#  No license under any patent, copyright, trade secret or other intellectual
#  property right is granted to or conferred upon you by disclosure
#  or delivery of the Materials, either expressly, by implication, inducement,
#  estoppel or otherwise. Any license under such intellectual property rights
#  must be express and approved by Intel in writing.
#
# ============================================================================
# = PRODUCT
#      Intel(r) IXA SDK for the IXP2X00 Network Processor
#
# = FILENAME
#      oc12_pos_gbeth_2401_run.sh
#
# = DESCRIPTION
#      This file contains scripts to run the oc12_pos_gbeth_2401
#      application on an IXPD2401 platform.
#
# = CHANGE HISTORY
#      02/26/2003      Created
# ============================================================================
#
#!/bin/sh

if [ -z "$1" ] || [ -z "$2" ] || [ "$3" = "" ]; then
echo
echo Syntax for $0.
echo $0 [spin1a/spin2] [db0/db1] [bb/db].
echo
echo ie.1. Mezz spin2. POS in db1. Gb on mezz in db0.
echo $0 spin2 db1 db
echo
echo ie.2. Mezz spin2. POS in db1. Gb on baseboard [MIC or backplane].
echo $0 spin2 db1 bb
echo
exit
fi

if [ -z "$1" ]; then
    echo As a first parameter for this script you must provide mezzanine spin. The proper spins are: spin1a and spin2.
    echo Usage:
    echo $0 spin1a
    echo $0 spin2
    exit
fi
SPIN=
if [ "$1" = "spin1a" ] || [ "$1" = "spin2" ]; then
    SPIN=$1
else
    echo Undefined mezzanine spin.
    exit
fi

if [ "$2" = "db0" ]; then
    DB_CONF=0
elif [ "$2" = "db1" ]; then
    DB_CONF=1
else
    echo Specify which POS mezzanine to initialize - db0 or db1.
   exit
fi

if [ "$3" = "bb" ]; then
    GB_CONF=0
elif [ "$3" = "db" ]; then
    GB_CONF=1
else
    echo Specify where giga bit ports are positioned
    echo On baseboard [baseboard MIC or backplane] - \"bb\" or
    echo on mezzanine in db0 or db1 - \"db\".
   exit
fi


# enable source routing in the stack
echo "1" > /proc/sys/net/ipv4/conf/all/accept_source_route
echo "1" > /proc/sys/net/ipv4/conf/all/forwarding

insmod ./libossl.o
insmod ./halMeDrv.o
insmod ./halMev2_lib.o
insmod ./rm_lib.o
insmod ./cci_lib.o
insmod ./utilities_lib.o
insmod ./msg_support_lib.o
insmod ./properties_lib.o
insmod ./fragmentation_lib.o
insmod ./lkup_lib.o
insmod ./rtmv4_lib.o
insmod ./l2tm_lib.o
insmod ./l2_config_util.o
insmod ./ether_arp_lib.o
insmod ./ipv4_cc.o
insmod ./route_config_util.o

if [ $DB_CONF = 0 ]; then
if [ $GB_CONF = 0 ]; then
echo Configure with POS in DB0 and GB on backplane
    insmod ./bb_linux_drv val_conf="spi2;spi3;sphy;104mhz" db0_conf="spi0;spi1;sphy;104mhz"
elif [ $GB_CONF = 1 ]; then
echo Configure with POS in DB0 and GB on mezzanine
    insmod ./bb_linux_drv db1_conf="spi2;spi3;sphy;104mhz" db0_conf="spi0;spi1;sphy;104mhz"
fi
else
if [ $GB_CONF = 0 ]; then
echo Configure with POS in DB1 and GB on backplane
    insmod ./bb_linux_drv val_conf="spi2;spi3;sphy;104mhz" db1_conf="spi0;spi1;sphy;104mhz"
elif [ $GB_CONF = 1 ]; then
echo Configure with POS in DB1 and GB on mezzanine
    insmod ./bb_linux_drv db0_conf="spi2;spi3;sphy;104mhz" db1_conf="spi0;spi1;sphy;104mhz"
fi
fi

insmod ./ixf1104drv_kernel.o
insmod ./ethernet_tx_cc.o
insmod ./qm_cc.o
insmod ./sched_cc.o
insmod ./stkdrv_cc.o

if [ $DB_CONF = 0 ]; then
    insmod ./pos_atm_drv atmdb0="pos;oc12;$SPIN"
else
    insmod ./pos_atm_drv atmdb1="pos;oc12;$SPIN"
fi

insmod ./pos_rx_cc.o
insmod ./pos_aal5_tx.o
insmod ./oc12_pos_gbeth_2401_config.o
insmod ./sysapp_common_egress.o

./sa start 1 &


