/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/
/**
 * ============================================================================
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      ix_cc_pos_ppp_stub.c
 *
 * = DESCRIPTION
 *      This file contains PPP stub that allows for tests with Cisco routers.
 * 
 * = CHANGE HISTORY
 *      6/23/2003 - Created.
 *
 * ============================================================================
 * $Id: ix_cc_pos_ppp_stub.c,v 1.2.2.1 2004/03/11 10:55:34 tmadajcz Exp $
 */
#include <ix_ossl.h>
#include <ix_rm.h>
#include <ix_cci.h>

#include <ix_cc.h>
#include <ix_cc_error.h>
#include <bindings.h>
#include <ix_cc_properties.h>

#include "ix_cc_pos_ppp_stub.h"


/*****************************************************************************
 * Definitions
 */
/*
 * TBD: Redefine it in local bindings.h when bindings.h become local,
 * application specific file.
 */
#undef IX_CC_POS_RX_PKT_OUTPUT
#define IX_CC_POS_RX_PKT_OUTPUT	IX_CC_QM_COMMON_PKT_INPUT

#ifdef IX_DEBUG

/*
 * Trace levels shall be used in the following way:
 * 0 - stay quite
 * 1 - trace primary operations and errors
 * 2 - trace even minor operations
 * 3 - trace all function calls except for periodic ones
 * 4 - trace all functions, even periodic
 */
#ifdef __KERNEL__

#define PppTrace(level, format, args...)  \
        (level > pppTraceLevel) ? : printk("PPP: " format, ##args)
#define PppAssert(x) \
        (x) ? : printk(KERN_CRIT "PPP: assertion (" #x ") failed at " \
                       "%s[%d]: %s()\n", __FILE__, __LINE__, __FUNCTION__)
#else /* __KERNEL__ */

#define PppTrace(level, format, args...)  \
        (level > pppTraceLevel) ? : printf("PPP: " format, ##args)
#define PppAssert(x) \
        (x) ? : printf("PPP: assertion (" #x ") failed at " \
                       "%s[%d]: %s()\n", __FILE__, __LINE__, __FUNCTION__)
#endif /* __KERNEL__ */

#else /* IX_DEBUG */

#define PppTrace(level, format, args...)
#define PppAssert(x)

#endif /* IX_DEBUG */

#define PPP_HDR_LCP   		0xc021
#define PPP_HDR_IPCP  		0x8021

#define LCP_CODE_CLOSE 		0x00
#define LCP_CODE_CONFREQ 	0x01
#define LCP_CODE_CONFACK 	0x02
#define LCP_CODE_CONFNCK 	0x03
#define LCP_CODE_ECHOREQ 	0x09
#define LCP_CODE_ECHOREP 	0x0a

#define IPCP_CODE_CONFREQ 	0x01
#define IPCP_CODE_CONFACK 	0x02

typedef struct ix_s_pos_ppp_ctrl {
    ix_cc_init_context	context;
} ix_cc_pos_ppp_ctrl;

typedef struct ix_s_ppp_hdr{
	ix_uint16   hdr;           /* LCP header */
	ix_uint8    code;          /* LCP code */
        ix_uint8    id;            /* session id */
	ix_uint16   len;           /* session id */
} ix_cc_ppp_hdr __attribute__ ((packed));

typedef struct ix_s_lcp_confreq{
	ix_cc_ppp_hdr	pppHdr;    /* lcp header */
	ix_uint8   	mru[4];       /* mru */
	ix_uint8   	opt;       /* reserved */
	ix_uint8   	optLen;    /* reserved */
	ix_uint32   	magic;     /* magic */
} ix_cc_lcp_confreq __attribute__ ((packed));

typedef struct ix_s_lcp_nack{
	ix_cc_ppp_hdr	pppHdr;	   /* lcp header */
	ix_uint8   	opt;       /* reserved */
	ix_uint8   	optLen;    /* reserved */
	ix_uint32  	magic;     /* magic */
} ix_cc_lcp_nack __attribute__ ((packed));

typedef struct ix_s_lcp_echo{
	ix_cc_ppp_hdr	pppHdr;	   /* lcp header */
	ix_uint32  	magic;     /* magic */
} ix_cc_lcp_echo __attribute__ ((packed));


typedef struct ix_s_ipcp{
	ix_cc_ppp_hdr	pppHdr;	   /* lcp header */
	ix_uint8   	opt;       /* reserved */
	ix_uint8   	optLen;    /* reserved */
	ix_uint32  	ipAddr;     /* magic */
} ix_cc_ipcp_confreq __attribute__ ((packed));


/*****************************************************************************
 * Local functions prototypes
 */
ix_error ix_duplicate_packet(ix_buffer_handle, ix_buffer_handle*,
                             ix_buffer_free_list_handle);
ix_error pos_ppp_pkt_handler(ix_buffer_handle, ix_uint32, void*);


/*****************************************************************************
 * Variables
 */
int pppTraceLevel = 3;

#ifdef IX_DEBUG
static char *lcpCode2Name[] = {
  "CLOSE",
  "CONFREQ",
  "CONFACK",
  "CONFNCK",
  "unknown",
  "unknown",
  "unknown",
  "unknown",
  "unknown",
  "ECHOREQ",
  "ECHOREP"
  "unknown",
};

static char *ipcpCode2Name[] = {
  "unknown",
  "CONFREQ",
  "CONFACK",
  "unknown",
  "unknown"
};
#endif /* IX_DEBUG */


/*****************************************************************************
 * Global functions definition
 */

/**
 * NAME: ix_cc_pos_ppp_init
 * 
 * DESCRIPTION:
 * @Param: arg_hCc       - IN execution engine handle given by CCI, used when
 *                            registering packet and message handlers
 * @Param: arg_ppContext - I/O
 * 
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */
ix_error ix_cc_pos_ppp_init(
    ix_cc_handle	arg_hCc, 
    void	**arg_ppContext)
{
    ix_error	err;
    ix_cc_pos_ppp_ctrl	*pCtrl;
    
    PppTrace(2, "%s()\n", __FUNCTION__);
    
    /* Check whether arg_ppContext is invalid */
    if (arg_ppContext == NULL || *arg_ppContext == NULL)
    {
        return IX_ERROR_REMOTE(IX_CC_ERROR_NULL, ("NULL argument"));
    }
    pCtrl = ix_ossl_malloc(sizeof(*pCtrl));
    if (pCtrl == NULL)
    {
	return IX_ERROR_REMOTE(IX_CC_ERROR_OOM_SYSTEM, ("Out of memory"));
    }
    memset(pCtrl, 0, sizeof(*pCtrl));
    
    pCtrl->context = *(ix_cc_init_context*)*arg_ppContext;
    ix_rm_packet_set_receive_mode(IX_CC_POS_RX_PKT_INPUT, IX_COMM_ID_MODE_CALLBACK);
    IX_ERROR_CGT(ix_rm_packet_handler_register(IX_CC_POS_RX_PKT_INPUT,
                                               pos_ppp_pkt_handler, pCtrl),
                 err, IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_REMOTE, cleanup);
    *arg_ppContext = (void*)pCtrl;
    return IX_SUCCESS;
    
cleanup:
    ix_cc_pos_ppp_fini(arg_hCc, pCtrl);
    return err;
} /* ix_cc_pos_ppp_init() */


/**
 * NAME: ix_cc_pos_ppp_fini
 * 
 * DESCRIPTION: This function terminates the PPP stub core component.
 * 
 * @Param: arg_hCc	- IN
 * @Param: arg_pContext	- IN
 * 
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */
ix_error ix_cc_pos_ppp_fini(
    ix_cc_handle	arg_hCc,
    void	*arg_pContext)
{
    PppTrace(2, "%s()\n", __FUNCTION__);
    
    if (arg_pContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("NULL argument"));
    }
    ix_rm_packet_handler_unregister(IX_CC_POS_RX_PKT_INPUT);
    ix_ossl_free(arg_pContext);
    
    return IX_SUCCESS;
} /* ix_cc_pos_rx_fini */


/*****************************************************************************
 * Local functions definition
 */
ix_error ix_duplicate_packet(
    ix_buffer_handle	arg_hBuffer,
    ix_buffer_handle	*arg_hOutBuffer,
    ix_buffer_free_list_handle	arg_hFl)
{
    ix_buffer_handle	hDupBuffer, hDupPrevBuffer, hNextBuffer;
    ix_hw_buffer_meta	*pDupMeta, *pOrgMeta;
    ix_uint8	*pDupData, *pOrgData;
    ix_error	err = IX_SUCCESS;
    
    PppTrace(4, "%s(0x%x,..., 0x%x)\n", __FUNCTION__,
             (int)arg_hBuffer, (int)arg_hFl);
    
    *arg_hOutBuffer = IX_NULL_BUFFER_HANDLE;
    hNextBuffer = arg_hBuffer;
    hDupPrevBuffer = IX_NULL_BUFFER_HANDLE;
    do
    {
        if (ix_rm_buffer_alloc(arg_hFl, &hDupBuffer) != IX_SUCCESS)
        {
            return IX_ERROR_NEW(IX_CC_ERROR_OOR, IX_ERROR_LEVEL_WARNING);
        }
        IX_ERROR_CGT(ix_rm_buffer_get_meta(hNextBuffer, (void**)&pOrgMeta),
                   err, IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_WARNING, label_1);
        IX_ERROR_CGT(ix_rm_buffer_get_meta(hDupBuffer, (void**)&pDupMeta),
                   err, IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_WARNING, label_1);
        ix_ossl_memcpy(pDupMeta, pOrgMeta, sizeof(ix_hw_buffer_meta));
        
        IX_ERROR_CGT(ix_rm_buffer_get_data(hNextBuffer, (void**)&pOrgData),
                   err, IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_WARNING, label_1);
        IX_ERROR_CGT(ix_rm_buffer_get_data(hDupBuffer, (void**)&pDupData),
                   err, IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_WARNING, label_1);
        ix_ossl_memcpy(pDupData, pOrgData,
                       pOrgMeta->m_BufferSize + pOrgMeta->m_Offset);
        
        if (hDupPrevBuffer == IX_NULL_BUFFER_HANDLE)
        {
            *arg_hOutBuffer = hDupBuffer;
        }
        else
        {
            IX_ERROR_CGT(ix_rm_buffer_link(&hDupPrevBuffer, &hDupBuffer), err,
                        IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_WARNING, label_1);
        }
        hDupPrevBuffer = hDupBuffer;
        
        IX_ERROR_CGT(ix_rm_buffer_get_next(hNextBuffer, &hNextBuffer), err,
                     IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_WARNING, label_1);
    }
    while (hNextBuffer != IX_NULL_BUFFER_HANDLE);
    
    return IX_SUCCESS;
    
 label_1:  
    if (*arg_hOutBuffer != IX_NULL_BUFFER_HANDLE)
    {
        ix_rm_buffer_free_chain(*arg_hOutBuffer);
        *arg_hOutBuffer = IX_NULL_BUFFER_HANDLE;
    }
    return err;
} /* ix_duplicate_packet() */


/**
 * NAME: pos_ppp_pkt_handler
 * 
 * DESCRIPTION: This function is the common packet handler routine for
 *              POS Rx CC. TODO-Check for host/byte order if we inspect packet
 * 
 * @Param: arg_hBuffer - IN
 * @Param: arg_ExceptionCode - IN
 * @Param: arg_pContext - IN
 * 
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */  
ix_error pos_ppp_pkt_handler(
    ix_buffer_handle	arg_hBuffer,  
    ix_uint32	arg_ExceptionCode,
    void	*arg_pContext)
{
    ix_error	err = IX_SUCCESS;
    ix_buffer_handle	hDupBuffer;
    
    ix_hw_buffer_meta	*pMeta;
    void	*pData;
    ix_cc_ppp_hdr	*pppHdr;
    ix_cc_lcp_confreq	*lcpConfreq;
    ix_cc_ipcp_confreq	*pIpcpConfreq;
    ix_cc_lcp_echo	*pLcpEcho;
    ix_cc_pos_ppp_ctrl	*pCtrl = arg_pContext;
    ix_cc_properties_node	*pProp;
    
    PppTrace(4, "%s(0x%x, 0x%x,...)\n", __FUNCTION__,
             (int)arg_hBuffer, (int)arg_ExceptionCode);
    
    if (ix_cc_hw_get_packet_data(arg_hBuffer, &pData) != IX_SUCCESS ||
        ix_rm_buffer_get_meta(arg_hBuffer, (void**)&pMeta) != IX_SUCCESS)
    {
        PppTrace(2, "Cannot locate data for buffer 0x%x, trying to free...\n",
                 (int)arg_hBuffer);
        ix_rm_buffer_free_chain(arg_hBuffer);
        return IX_ERROR_NEW(IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_WARNING);
    }
    ((ix_uint8*)pData) -= 2;	/* get back to the L2 header */
    pppHdr = (ix_cc_ppp_hdr*)pData;
    switch (pppHdr->hdr)
    {
      case PPP_HDR_LCP:
        PppTrace(3, "Got packet LCP(0x%x) %s(0x%x)\n", pppHdr->hdr,
                 (pppHdr->code < sizeof(lcpCode2Name)/sizeof(lcpCode2Name[0]))?
                 lcpCode2Name[pppHdr->code] : "unknown", pppHdr->code);
        switch (pppHdr->code)
        {
          case LCP_CODE_CLOSE:
          case LCP_CODE_CONFREQ:
            pMeta->m_Offset -= 2;
            pMeta->m_BufferSize += 2;
            pMeta->m_PacketSize += 2;
            pMeta->m_OutputPort = pMeta->m_InputPort;
            err = ix_duplicate_packet(arg_hBuffer, &hDupBuffer,
                                      pCtrl->context.hFreeList);
            if (err != IX_SUCCESS)
            {
                ix_rm_buffer_free_chain(arg_hBuffer);
                return err;
            }
            /* Send a Confreq to the peer */
            lcpConfreq = (ix_cc_lcp_confreq*)pData;
            lcpConfreq->pppHdr.code = LCP_CODE_CONFACK;
            
            PppTrace(3, "Sending LCP(0x%x) %s(0x%x)\n", lcpConfreq->pppHdr.hdr,
                     lcpCode2Name[lcpConfreq->pppHdr.code],
                     lcpConfreq->pppHdr.code);
            
            err = ix_rm_packet_send(IX_CC_POS_RX_PKT_OUTPUT, arg_hBuffer, 0);
            if (err != IX_SUCCESS)
            {
                PppTrace(2, "LCP %s send failed (error 0x%x)\n",
                         lcpCode2Name[lcpConfreq->pppHdr.code], (int)err);
                ix_rm_buffer_free_chain(hDupBuffer);
                ix_rm_buffer_free_chain(arg_hBuffer);
                return IX_ERROR_NEW(IX_CC_ERROR_SEND_FAIL,
                                    IX_ERROR_LEVEL_WARNING);
            }
            err = ix_cc_hw_get_packet_data(hDupBuffer, &pData);
            PppAssert(err == IX_SUCCESS);
            lcpConfreq = (ix_cc_lcp_confreq*)pData;
            lcpConfreq->magic = lcpConfreq->magic * 2;
            
            PppTrace(3, "Sending LCP(0x%x) %s(0x%x)\n", lcpConfreq->pppHdr.hdr,
                     lcpCode2Name[lcpConfreq->pppHdr.code],
                     lcpConfreq->pppHdr.code);
            
            err = ix_rm_packet_send(IX_CC_POS_RX_PKT_OUTPUT, hDupBuffer, 0);
            if (err != IX_SUCCESS)
            {
                PppTrace(2, "LCP %s send failed (error 0x%x)\n",
                         lcpCode2Name[lcpConfreq->pppHdr.code], (int)err);
                ix_rm_buffer_free_chain(hDupBuffer);
                return IX_ERROR_NEW(IX_CC_ERROR_SEND_FAIL,
                                    IX_ERROR_LEVEL_WARNING);
            }
            break;
            
          case LCP_CODE_ECHOREQ:
            pLcpEcho = (ix_cc_lcp_echo *)pData;
            pLcpEcho->pppHdr.code = LCP_CODE_ECHOREP;
            
            pMeta->m_Offset -= 2;
            pMeta->m_BufferSize += 2;
            pMeta->m_PacketSize += 2;    
            pMeta->m_OutputPort = pMeta->m_InputPort;
            
            PppTrace(3, "Sending LCP(0x%x) %s(0x%x)\n", pLcpEcho->pppHdr.hdr,
                     lcpCode2Name[pLcpEcho->pppHdr.code],
                     pLcpEcho->pppHdr.code);
            
            err = ix_rm_packet_send(IX_CC_POS_RX_PKT_OUTPUT, arg_hBuffer, 0);
            if (err != IX_SUCCESS)
            {
                PppTrace(2, "LCP %s send failed (error 0x%x)\n",
                         lcpCode2Name[pLcpEcho->pppHdr.code], (int)err);
                ix_rm_buffer_free_chain(arg_hBuffer);
                return IX_ERROR_NEW(IX_CC_ERROR_SEND_FAIL,
                                    IX_ERROR_LEVEL_WARNING);
            }
            break;
            
          case LCP_CODE_CONFACK:
          case LCP_CODE_CONFNCK:
          default:
            ix_rm_buffer_free_chain(arg_hBuffer);
	}
	break;
        
      case PPP_HDR_IPCP:
        PppTrace(3, "Got packet IPCP(0x%x) %s(0x%x)\n", pppHdr->hdr,
               (pppHdr->code < sizeof(ipcpCode2Name)/sizeof(ipcpCode2Name[0]))?
                 ipcpCode2Name[pppHdr->code] : "unknown", pppHdr->code);
        switch (pppHdr->code)
        {
          case IPCP_CODE_CONFREQ:
            pMeta->m_Offset -= 2;
            pMeta->m_BufferSize += 2;
            pMeta->m_PacketSize += 2;    
            pMeta->m_OutputPort = pMeta->m_InputPort;
            err = ix_duplicate_packet(arg_hBuffer, &hDupBuffer,
                                      pCtrl->context.hFreeList);
            if (err != IX_SUCCESS)
            {
                ix_rm_buffer_free_chain(arg_hBuffer);
                return err;
            }
            /* Send a Confreq to the peer */
            pIpcpConfreq = (ix_cc_ipcp_confreq*)pData;
            pIpcpConfreq->pppHdr.code = IPCP_CODE_CONFACK;
            
            PppTrace(3, "Sending IPCP(0x%x) %s(0x%x)\n",
                     pIpcpConfreq->pppHdr.hdr,
                     ipcpCode2Name[pIpcpConfreq->pppHdr.code],
                     pIpcpConfreq->pppHdr.code);
            
            err = ix_rm_packet_send(IX_CC_POS_RX_PKT_OUTPUT, arg_hBuffer, 0);
            if (err != IX_SUCCESS)
            {
                PppTrace(2, "IPCP %s send failed (error 0x%x)\n",
                         ipcpCode2Name[pIpcpConfreq->pppHdr.code], (int)err);
                ix_rm_buffer_free_chain(hDupBuffer);
                ix_rm_buffer_free_chain(arg_hBuffer);
                return IX_ERROR_NEW(IX_CC_ERROR_SEND_FAIL,
                                    IX_ERROR_LEVEL_WARNING);        
            }
            
            err = ix_cc_hw_get_packet_data(hDupBuffer, &pData);
            PppAssert(err == IX_SUCCESS);
            ix_rm_buffer_get_meta(hDupBuffer, (void**)&pMeta);
            pIpcpConfreq = (ix_cc_ipcp_confreq*)pData;
            for (pProp = pCtrl->context.pPropertyList;
                 pProp != NULL; pProp = pProp->pNextPropNode)
            {
                if (pProp->pPropertyInfo != NULL &&
                    pProp->pPropertyInfo->port_id == pMeta->m_InputPort)
                {
                    pIpcpConfreq->ipAddr = pProp->pPropertyInfo->
                                               ip_prop[0].protocol.ipv4_prop.ipv4_address;
                    break;
                }
            }
            PppAssert(pProp != NULL);
            PppTrace(3, "Sending IPCP(0x%x) %s(0x%x) IP %d.%d.%d.%d\n",
                     pIpcpConfreq->pppHdr.hdr,
                     ipcpCode2Name[pIpcpConfreq->pppHdr.code],
                     pIpcpConfreq->pppHdr.code,
                     (int)pIpcpConfreq->ipAddr >> 24 & 0xff,
                     (int)pIpcpConfreq->ipAddr >> 16 & 0xff,
                     (int)pIpcpConfreq->ipAddr >>  8 & 0xff,
                     (int)pIpcpConfreq->ipAddr >>  0 & 0xff);
            
            err = ix_rm_packet_send(IX_CC_POS_RX_PKT_OUTPUT, hDupBuffer, 0);
            if (err != IX_SUCCESS)
            {
                PppTrace(2, "IPCP %s send failed (error 0x%x)\n",
                         ipcpCode2Name[pIpcpConfreq->pppHdr.code], (int)err);
                ix_rm_buffer_free_chain(hDupBuffer);
                return IX_ERROR_NEW(IX_CC_ERROR_SEND_FAIL,
                                    IX_ERROR_LEVEL_WARNING);        
            }
            break;
            
          default:
            ix_rm_buffer_free_chain(arg_hBuffer);
	}
	break;
        
      default:
        PppTrace(3, "Got packet unknown(0x%x) unknown(0x%x)\n",
                 pppHdr->hdr, pppHdr->code);
        ix_rm_buffer_free_chain(arg_hBuffer);
        break;
    }
    return err;
} /* pos_ppp_pkt_handler() */

