/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmitted,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/
/*
 * pkthdr_cache.uc
 * Contains IP Header Caching Macros. The macros supports PPP (POS)
 * and Ethernet encapsulations.
 */

#ifndef __PKTHDR_CACHE_UC__
#define __PKTHDR_CACHE_UC__

#ifndef	PKT_HDR_CACHE_SIZE_ETH
#define	PKT_HDR_CACHE_SIZE_ETH		10	// 10 longwords
#endif

#ifndef	PKT_HDR_CACHE_SIZE_POS
#define	PKT_HDR_CACHE_SIZE_POS		6	// 6 longwords
#endif


#ifndef			PKT_HDR_DRAM_RD_SIZE_ETH
#define_eval	PKT_HDR_DRAM_RD_SIZE_ETH	(PKT_HDR_CACHE_SIZE_ETH >> 1)// read 5 quadwords for Ethernet
#endif

#ifndef			PKT_HDR_DRAM_RD_SIZE_POS
#define_eval	PKT_HDR_DRAM_RD_SIZE_POS	(PKT_HDR_CACHE_SIZE_POS >> 1)// read 3 quadwords for POS
#endif

#ifdef			PKT_HDR_CACHE_SIZE
#undef			PKT_HDR_CACHE_SIZE
#endif
#define_eval	PKT_HDR_CACHE_SIZE          PKT_HDR_CACHE_SIZE_ETH// get the greater value


///////////////////////////////////////////////////////////////////////////////
// dl_load_iphdr_cache_eth:
//	 Description: 
//		Reads the IP header (32 bytes) of the Ethernet packet from SDRAM and keeps 
//		it in a cache (DRAM transfer registers). 
//
//	 Outputs:
//		The $$dl_iphdr# global dram transfer registers will be set with appropriate
//		header data.
//
//	Inputs:
//		buf_handle	:			Handle of buffer.
//		req_sig		:			Signal to use in the I/O (DRAM read) operation.
//		sig_action	:			What to do with I/O operation - 
//									- wait for signal(s) as specified by sig_action
//									- do not wait for signal, just return (SIG_NONE)
//	Size:   				:
//
#define	dl_load_iphdr_cache_eth		dl_iphdr_load_cache_eth

#macro dl_iphdr_load_cache_eth[buf_handle, req_sig, sig_action]
.begin
.reg 	sdram_offset, buf_offset

	//	Read 32 bytes of IP Header from SDRAM

	dl_buf_get_data[sdram_offset, buf_handle]
	dl_meta_get_offset[buf_offset]
#ifndef DISABLE_MAC_FILTERING
	dram[read, $$iphdr0, sdram_offset, buf_offset, PKT_HDR_DRAM_RD_SIZE_ETH],\
        sig_done[req_sig]
#else // DISABLE_MAC_FILTERING
	alu[buf_offset, buf_offset, +, 8]
	dram[read, $$iphdr2, sdram_offset, buf_offset, 4], sig_done[req_sig]
#endif // DISABLE_MAC_FILTERING

#if (!streq('sig_action', 'SIG_NONE'))
	ctx_arb[sig_action]
#endif

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_load_iphdr_cache_pos:
//	 Description: 
//		Reads the IP header (32 bytes) of the POS packet from SDRAM and keeps 
//		it in a cache (DRAM transfer registers). 
//
//	 Outputs:
//		The $$dl_iphdr# global dram transfer registers will be set with appropriate
//		header data.
//
//	 Inputs:
//		buf_handle	:			Handle of buffer.
//		req_sig		:			Signal to use in the I/O (DRAM read) operation.
//		sig_action	:			What to do with I/O operation - 
//								- wait for signal(s) as specified by sig_action
//								- do not wait for signal, just return (SIG_NONE)
//	 Size:   				:
//
#define	dl_load_iphdr_cache_pos		dl_iphdr_load_cache_pos

#macro dl_iphdr_load_cache_pos[buf_handle, req_sig, sig_action]
.begin
.reg 	sdram_offset, buf_offset

	//	Read 32 bytes of IP Header from SDRAM

	dl_buf_get_data[sdram_offset, buf_handle]
	dl_meta_get_offset[buf_offset]
	dram[read, $$iphdr0, sdram_offset, buf_offset, PKT_HDR_DRAM_RD_SIZE_POS],\
        sig_done[req_sig]

#if (!streq('sig_action', 'SIG_NONE'))
	ctx_arb[sig_action]
#endif

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_flush_iphdr_cache_eth_pos:
//	 Description: 
//		Writes back the IP header (32 bytes) from the cache to the packet in SDRAM. 
//      The macro supports the both encapsulations such as Ethernet and POS.
//
//	 Outputs:
//
//	 Inputs:
//		buf_handle	:			Handle of buffer.
//		req_sig		:			Signal to use in the I/O (DRAM read) operation.
//		sig_action	:			What to do with I/O operation - 
//								- wait for signal(s) as specified by sig_action
//								- do not wait for signal, just return (SIG_NONE)
//		The $$dl_iphdr# global dram transfer registers will be the set of registers 
//		from which data will be written back to SDRAM.
//
//	 Size:   				:
//
#define	dl_flush_iphdr_cache_eth_pos		dl_iphdr_flush_cache_eth_pos

#macro dl_iphdr_flush_cache_eth_pos[buf_handle, req_sig, sig_action]
.begin
.reg 	sdram_offset, buf_offset, input_port

	//	Write 32 bytes of IP Header from SDRAM
    dl_buf_get_data[sdram_offset, buf_handle]

    dl_meta_get_input_port[input_port]
    alu[--, input_port, -, MIN_ETH_PORT_NUM]
    br<0[dl_iphdr_flush_cache_pos#], defer[1]
	    dl_meta_get_offset[buf_offset]

   	alu[buf_offset,buf_offset,+,2]
	dram[write, $$iphdr4, sdram_offset, buf_offset, 2], sig_done[req_sig]

#if (!streq('sig_action', 'SIG_NONE'))
	ctx_arb[sig_action], and, br[end_of_macro#]
#else
    br[end_of_macro#]
#endif
  

dl_iphdr_flush_cache_pos#:
	alu[buf_offset,buf_offset,-,2]	
	dram[write, $$iphdr0, sdram_offset, buf_offset, PKT_HDR_DRAM_RD_SIZE_POS],\
        sig_done[req_sig]

#if (!streq('sig_action', 'SIG_NONE'))
	ctx_arb[sig_action]
#endif

end_of_macro#:
.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_iphdr_cache_init_eth_pos:
//	 Description: 
//		This macro needs to be called before any of the dl_iphdr_* can be called.
//		It allocates a set of DRAM xfer registers to be used for caching. This macro
//		doesn't generate any code (instructions). It only produces some assembler
//		directives.
//
//	 Outputs:
//
//	 Inputs:
//
//	 Size:   				:
//		0 Instructions

#macro dl_iphdr_cache_init_eth_pos[]

	xbuf_alloc[$$iphdr, PKT_HDR_CACHE_SIZE, read_write]

#endm

///////////////////////////////////////////////////////////////////////////////
// dl_iphdr_cache_fini_eth_pos:
//	 Description: 
//		This macro needs to be called at the end when dl_iphdr_macros are no longer
//		required. It frees a set of DRAM xfer registers that were allocated using
//		dl_iphdr_cache_init. This macro	doesn't generate any code (instructions). 
//		It only produces some assembler	directives.
//
//	 Outputs:
//
//	 Inputs:
//
//	 Size:   				:
//		0 Instructions

#macro dl_iphdr_cache_fini_eth_pos[]

	xbuf_free[$$iphdr]

#endm

#endif	//__PKTHDR_CACHE_UC__