/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/
/**
 * ============================================================================
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      ixf6048.c
 *
 * = DESCRIPTION
 *      This file contains functions to initialize the oc12_pos_gbeth_2401
 *      application on an IXPD2401 platform.
 *
 * = CHANGE HISTORY
 *      04/16/2003      Created
 * ============================================================================
 */

#ifdef VXWORKS
#include <vxWorks.h>
#else
#include <memory.h>
#include <string.h>
#endif

#include <pos_atm_media_drv.h>

#if (defined IX_PLATFORM_2401)
#ifdef USE_BB_DRV_CONFIG
#include "baseboard_media_drv.h"
unsigned int hwConfigPosPortMask;  /* POS Port Mask */
#endif /* USE_BB_DRV_CONFIG */
#endif /* IX_PLATFORM_2x01 */

int posFd = -1;

#ifdef USE_BB_DRV_CONFIG
int _set_FramerConfig()
{
	int		i;
	int		bb_handle;
	char	mezzanine_pos_atm = 0;

	hwConfigPosPortMask = 0;
	bb_handle = Bb_Open();

	/* Mezzanine in DB0 */
	switch(Bb_GetDevType(bb_handle, MEDIA_CARD1_ID))
	{
		case BB_FAILURE				:	break;
		case POS_ATM_OC312_ID		:   printf("Mezzanine %s ", POS_ATM_OC312_STR_ID);
										mezzanine_pos_atm |= 0x1;
										break;
		case POS_ATM_OC48_ID		:   printf("Mezzanine %s ", POS_ATM_OC48_STR_ID);
										mezzanine_pos_atm |= 0x1;
										break;
		case GB_ETH_ID				:   printf("Mezzanine %s ", GB_ETH_STR_ID);
										break;
		default						:	printf("ERROR!!! Device in DB0 unknown!!!\n");
										return -1;
	}

	/* MIC1 */
	switch(Bb_GetDevType(bb_handle, MIC1_ID))
	{
		case BB_FAILURE				:	if((mezzanine_pos_atm & 1) != 0)
										{
										 	printf("in DB0\n");
											hwConfigPosPortMask |= 1;
										}
										break;
		case MIC_F4xOC12_MEZZ_ID	:   printf("with %s in DB0\n", MIC_F4xOC12_MEZZ_STR_ID);
										hwConfigPosPortMask |= 1;
										break;
		case MIC_F1xOC48_MEZZ_ID	:   printf("with %s in DB0\n", MIC_F1xOC48_MEZZ_STR_ID);
										hwConfigPosPortMask |= 1;
										break;
		case MIC_F2xOC12_MEZZ_ID	:   printf("with %s in DB0\n", MIC_F2xOC12_MEZZ_STR_ID);
										hwConfigPosPortMask |= 1;
										break;
		default						:	break;
	}

	/* Mezzanine in DB1 */
	switch(Bb_GetDevType(bb_handle, MEDIA_CARD2_ID))
	{
		case BB_FAILURE				:	break;
		case POS_ATM_OC312_ID		:   printf("Mezzanine %s ", POS_ATM_OC312_STR_ID);
										mezzanine_pos_atm |= 0x2;
										break;
		case POS_ATM_OC48_ID		:   printf("Mezzanine %s ", POS_ATM_OC48_STR_ID);
										mezzanine_pos_atm |= 0x2;
										break;
		case GB_ETH_ID				:   printf("Mezzanine %s ", GB_ETH_STR_ID);
										break;
		default						:	printf("ERROR!!! Device in DB1 unknown!!!\n");
										return -1;
	}


	/* MIC2 */
	switch(Bb_GetDevType(bb_handle, MIC2_ID))
	{
		case BB_FAILURE				:	if((mezzanine_pos_atm & 2) != 0)
										{
										 	printf("in DB1\n");
											hwConfigPosPortMask |= 2;
										}
										break;
		case MIC_F4xOC12_MEZZ_ID	:   printf("with %s in DB1\n", MIC_F4xOC12_MEZZ_STR_ID);
										hwConfigPosPortMask |= 2;
										break;
		case MIC_F1xOC48_MEZZ_ID	:   printf("with %s in DB1\n", MIC_F1xOC48_MEZZ_STR_ID);
										hwConfigPosPortMask |= 2;
										break;
		case MIC_F2xOC12_MEZZ_ID	:   printf("with %s in DB1\n", MIC_F2xOC12_MEZZ_STR_ID);
										hwConfigPosPortMask |= 2;
										break;
		default						:	break;
	}

	Bb_Close(bb_handle);
	return 0;
}
#endif /* USE_BB_DRV_CONFIG */


void
framer_init(unsigned int modeDB0, unsigned int modeDB1)
{
    char *modeDB0Str, *modeDB1Str;
    char oc3ModeStr[] = "oc3,pos";
    char oc12ModeStr[] = "oc12,pos";

#ifdef USE_BB_DRV_CONFIG
	_set_FramerConfig();
#endif /* USE_BB_DRV_CONFIG */

    if ((modeDB0 != 3) && (modeDB0 != 12))
    {
        printf("Error! Wrong POS mode for device 0: %d\n", modeDB0);
        return;
    }
    if ((modeDB1 != 3) && (modeDB1 != 12))
    {
	printf("Error! Wrong POS mode for device 1: %d\n", modeDB1);
	return;
    }
    if (modeDB0 == 3)
    {
        modeDB0Str = oc3ModeStr;
    }
    else
    {
        modeDB0Str = oc12ModeStr;
    }
    if (modeDB1 == 3)
    {
        modeDB1Str = oc3ModeStr;
    }
    else
    {
        modeDB1Str = oc12ModeStr;
    }
    if (PosAtm_DriverStart(modeDB0Str, modeDB1Str) < 0)
    {
        printf("\nError initializing OC12 POS(/ATM) DB card. Test aborted.\n");
        return;
    }

#ifdef USE_BB_DRV_CONFIG
	switch(hwConfigPosPortMask)
	{
    	case 0	:	posFd = POS_ATM_INV_HANDLE;
        			printf ("There is no POS card in DB0 or DB1.\n");
        			return;
    	case 1	:	posFd = PosAtm_Open(0);
					break;
		case 2	:	posFd = PosAtm_Open(1);
					break;
		case 3	:	posFd = PosAtm_Open(0);
					break;
		default	:	posFd = POS_ATM_INV_HANDLE;
        			printf ("There is no POS card in DB0 or DB1.\n");
        			return;
	}
#else /* USE_BB_DRV_CONFIG */
#if defined IX_CONFIGURATION_atmdb0
    posFd = PosAtm_Open(0);
#else
    posFd = PosAtm_Open(1);
#endif /* IX_CONFIGURATION_atmdb0 */
#endif /* USE_BB_DRV_CONFIG */

    if (posFd < 0)
    {
        printf("Warning: Cannot open POS ATM daughter card 0\n");
    }
    else
    {
        if (PosAtm_SetLinkState(posFd, 0, POS_ATM_STATE_UP) < 0)
        {
            printf("\nERROR: Cannot enable port 0 on daughter card 0\n");
        }

        if (PosAtm_SetLinkState(posFd, 1, POS_ATM_STATE_UP) < 0)
        {
            printf("\nERROR: Cannot enable port 1 on daughter card 0\n");
        }
    }

    /* checking register */
    printf("<1> 0xca000100 = 0x%08x (required 0x01000011)\n",
           *((volatile unsigned int *)0xca000100));
    *((volatile unsigned int *)0xca000100) = 0x01000011;
    printf("<2> 0xca000100 = 0x%08x\n",
           *((volatile unsigned int *)0xca000100));
    printf("framer_init END\n");
}


/******************************************************************************/
//static char myArray[512];

void framer_stats(void)
{
    pos_atm_ixf6048_counters_t stats;

    stats.link = 0;
    stats.channel = 0;

    if (posFd < 0 || PosAtm_GetLinkStats(posFd, &stats) < 0)
    {
        printf("\nCannot read POS(/ATM) DB link 0 statistics\n");
    }
    else
    {
        printf("POS counters:\n");
	printf(" rxFrames:          %d\n", stats.counters.pos.rxFrames);
	printf(" txFrames:          %d\n", stats.counters.pos.txFrames);
	printf(" rxBytes:           %d\n", stats.counters.pos.rxBytes);
	printf(" txBytes:           %d\n", stats.counters.pos.txBytes);
	printf(" rxAbortedFrames:   %d\n", stats.counters.pos.rxAbortedFrames);
	printf(" txAbortedFrames:   %d\n", stats.counters.pos.txAbortedFrames);
 	printf(" rxFcsErrors:       %d\n", stats.counters.pos.rxFcsErrors);
  	printf(" rxMinLengthErrors: %d\n", stats.counters.pos.rxMinLengthErrors);
   	printf(" rxMaxLengthErrors: %d\n", stats.counters.pos.rxMaxLengthErrors);
   	printf(" rxFifoOverflows:   %d\n", stats.counters.pos.rxFifoOverflows);
   	printf(" txFifoUnderflows:  %d\n", stats.counters.pos.txFifoUnderflows);
    }

    stats.link = 1;
    if (posFd < 0 || PosAtm_GetLinkStats(posFd, &stats) < 0)
    {
        printf("\nCannot read POS(/ATM) DB link 1 statistics\n");
    }
    else
    {
        printf("POS counters:\n");
	printf(" rxFrames:          %d\n", stats.counters.pos.rxFrames);
	printf(" txFrames:          %d\n", stats.counters.pos.txFrames);
	printf(" rxBytes:           %d\n", stats.counters.pos.rxBytes);
	printf(" txBytes:           %d\n", stats.counters.pos.txBytes);
	printf(" rxAbortedFrames:   %d\n", stats.counters.pos.rxAbortedFrames);
	printf(" txAbortedFrames:   %d\n", stats.counters.pos.txAbortedFrames);
 	printf(" rxFcsErrors:       %d\n", stats.counters.pos.rxFcsErrors);
  	printf(" rxMinLengthErrors: %d\n", stats.counters.pos.rxMinLengthErrors);
   	printf(" rxMaxLengthErrors: %d\n", stats.counters.pos.rxMaxLengthErrors);
   	printf(" rxFifoOverflows:   %d\n", stats.counters.pos.rxFifoOverflows);
   	printf(" txFifoUnderflows:  %d\n", stats.counters.pos.txFifoUnderflows);
    }

}





