/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/
/**
 * ============================================================================
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      l2_encap_init.c
 *
 * = DESCRIPTION
 *      This file contains scripts to initialize the oc12_pos_gbeth_2401
 *      application on an IXPD2401 platform.
 *
 * = CHANGE HISTORY
 *      01/22/2003      Created
 * ============================================================================
 */

#include "../oc12_pos_gbeth_memory.h"
#include "xscale_init.h"

#define _L2_TABLE_SRAM_BASE (SRAM_BASE_CH0 + L2_TABLE_SRAM_BASE)

/*#define L2_TABLE_NUM_OF_ENTRIES	65536*/
#define L2_TABLE_NUM_OF_ENTRIES	(L2_TABLE_SRAM_SIZE/32)

/**
 * NAME: l2_encap_init_addresses
 *
 * DESCRIPTION: This function configures L2 addresses in the L2 table
 */
int l2_encap_init_addresses(void)
{

    int i;
    
	printf("l2_encap_init_addresse(): Configuring L2 Table addresses...\n");

    /* we need a lot of L2 static addreses (values of them are meaningless)*/
    for (i = 0; i < 0x100; i++)
    {

        /* setup L2_table entry */
        /* entry i */
        set_sram(_L2_TABLE_SRAM_BASE + 0x000 + (i<<4), 0x800090d7);
        set_sram(_L2_TABLE_SRAM_BASE + 0x004 + (i<<4), 0x00080000 + (i<<8));
        set_sram(_L2_TABLE_SRAM_BASE + 0x008 + (i<<4), 0x90d80008);
        set_sram(_L2_TABLE_SRAM_BASE + 0x00c + (i<<4), 0x00080000);

    }
    set_sram(_L2_TABLE_SRAM_BASE + 0x000 + (0x103<<4), 0x800090d7);
    set_sram(_L2_TABLE_SRAM_BASE + 0x004 + (0x103<<4), 0x00080302);
    set_sram(_L2_TABLE_SRAM_BASE + 0x008 + (0x103<<4), 0x01020300);
    set_sram(_L2_TABLE_SRAM_BASE + 0x00c + (0x103<<4), 0x03080000);
    
    set_sram(_L2_TABLE_SRAM_BASE + 0x000 + (0x104<<4), 0x800090d7);
    set_sram(_L2_TABLE_SRAM_BASE + 0x004 + (0x104<<4), 0x00080402);
    set_sram(_L2_TABLE_SRAM_BASE + 0x008 + (0x104<<4), 0x01020300);
    set_sram(_L2_TABLE_SRAM_BASE + 0x00c + (0x104<<4), 0x04080000);

    
	printf("L2 Table configuration COMPLETE.\n\n");

	return 1;
}




