/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/
/**
 * ============================================================================
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      drv_cfg.c
 *
 * = DESCRIPTION
 *      This file contains functions to initialize drivers for
 *      the oc12_pos_gbeth_2401 application on an IXPD2401 platform.
 *
 * = CHANGE HISTORY
 *      06/24/2003      Created
 * ============================================================================
 */
#include <stdio.h>
#include "ixf1104ce_driver_api.h"
#include <memory.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include "pos_atm_ioctl.h"
#include <pos_atm_media_drv.h>


/* defines for driver modes (bit 2:0) */
#define IX_CC_ETH_TX_DRIVER_MODE_FIBER               0x0
#define IX_CC_ETH_TX_DRIVER_MODE_GIGA_HALF_DUPLEX    0x1
#define IX_CC_ETH_TX_DRIVER_MODE_GIGA_FULL_DUPLEX    0x2
#define IX_CC_ETH_TX_DRIVER_MODE_100_HALF_DUPLEX     0x3
#define IX_CC_ETH_TX_DRIVER_MODE_100_FULL_DUPLEX     0x4
#define IX_CC_ETH_TX_DRIVER_MODE_10_HALF_DUPLEX      0x5
#define IX_CC_ETH_TX_DRIVER_MODE_10_FULL_DUPLEX      0x6



/* defines for driver parity (bit 3) */
#define IX_CC_ETH_TX_DRIVER_MODE_PARITY_ODD          0x0
#define IX_CC_ETH_TX_DRIVER_MODE_PARITY_EVEN         0x8



/* defines for driver block modes (bit 4) */
#define IX_CC_ETH_TX_DRIVER_MODE_BLOCK_MPHY          0x00
#define IX_CC_ETH_TX_DRIVER_MODE_BLOCK_4_8           0x10

#define IX_CC_ETH_TX_FRAME_SIZE  0x23FC /* 9216 - 4 */

#define IX_CC_ETH_TX_DRIVER_PORT_MASK_01   0x3
#define IX_CC_ETH_TX_DRIVER_PORT_MASK_23   0xc

#ifdef USE_BB_DRV_CONFIG
#define IX_CC_ETH_TX_MAX_NUM_PORTS         12
#else /* USE_BB_DRV_CONFIG */
#define IX_CC_ETH_TX_MAX_NUM_PORTS         4
#endif /* USE_BB_DRV_CONFIG */

#ifdef USE_BB_DRV_CONFIG
#include "baseboard_media_drv.h"
uint32 hwConfigEthPortMask; /* ETH Port Mask */
uint32 hwConfigEthMode[IX_CC_ETH_TX_MAX_NUM_PORTS]; /* ETH Port configuration mode value */
uint32 hwConfigPosPortMask;  /* POS Port Mask */
uint32 hw_port[4];
#endif /* USE_BB_DRV_CONFIG */


#ifdef USE_BB_DRV_CONFIG
int _ix_set_hwConfig()
{
	int		i;
	int		bb_handle;
	char	mezzanine_pos_atm = 0;

	hwConfigEthPortMask = 0;
	hwConfigPosPortMask = 0;
	bb_handle = Bb_Open();

	/* Mezzanine in DB0 */
	switch(Bb_GetDevType(bb_handle, MEDIA_CARD1_ID))
	{
		case BB_FAILURE				:	break;
		case POS_ATM_OC312_ID		:   printf("Mezzanine %s ", POS_ATM_OC312_STR_ID);
										mezzanine_pos_atm |= 0x1;
										break;
		case POS_ATM_OC48_ID		:   printf("Mezzanine %s ", POS_ATM_OC48_STR_ID);
										mezzanine_pos_atm |= 0x1;
										break;
		case GB_ETH_ID				:   printf("Mezzanine %s ", GB_ETH_STR_ID);
										break;
		default						:	printf("ERROR!!! Device in DB0 unknown!!!\n");
										return -1;
	}

	/* MIC1 */
	switch(Bb_GetDevType(bb_handle, MIC1_ID))
	{
		case BB_FAILURE				:	if((mezzanine_pos_atm & 1) != 0)
										{
										 	printf("in DB0\n");
											hwConfigPosPortMask |= 1;
										}
										else
											printf("No device in DB0\n");
										break;
		case MIC_C4xGB_MEZZ_ID 		:   printf("with %s in DB0\n", MIC_C4xGB_MEZZ_STR_ID);
										hwConfigEthPortMask |= 0xc0;
										for(i = 6; i < 8; i++)
										   hwConfigEthMode[i] = IX_CC_ETH_TX_DRIVER_MODE_GIGA_FULL_DUPLEX |
                             									IX_CC_ETH_TX_DRIVER_MODE_PARITY_EVEN |
                             									IX_CC_ETH_TX_DRIVER_MODE_BLOCK_MPHY;
										break;
		case MIC_F4xGB_MEZZ_ID		:   printf("with %s in DB0\n", MIC_F4xGB_MEZZ_STR_ID);
										hwConfigEthPortMask |= 0xc0;
										for(i = 6; i < 8; i++)
										   hwConfigEthMode[i] = IX_CC_ETH_TX_DRIVER_MODE_FIBER |
                             									IX_CC_ETH_TX_DRIVER_MODE_PARITY_EVEN |
                             									IX_CC_ETH_TX_DRIVER_MODE_BLOCK_MPHY;
										break;
		case MIC_C2xGB_BRD_ID		:   printf("%s on baseboard in slot 1\n", MIC_C2xGB_BRD_STR_ID);
										hwConfigEthPortMask |= 0xc;
										for(i = 2; i < 4; i++)
										   hwConfigEthMode[i] = IX_CC_ETH_TX_DRIVER_MODE_GIGA_FULL_DUPLEX |
                             									IX_CC_ETH_TX_DRIVER_MODE_PARITY_EVEN |
                             									IX_CC_ETH_TX_DRIVER_MODE_BLOCK_4_8;
										printf("Spi switch to front\n");
										Bb_SwitchSpiMic(bb_handle);
										break;
		case MIC_F4xOC12_MEZZ_ID	:   printf("with %s in DB0\n", MIC_F4xOC12_MEZZ_STR_ID);
										hwConfigPosPortMask |= 1;
										break;
		case MIC_F1xOC48_MEZZ_ID	:   printf("with %s in DB0\n", MIC_F1xOC48_MEZZ_STR_ID);
										hwConfigPosPortMask |= 1;
										break;
		case MIC_F2xOC12_MEZZ_ID	:   printf("with %s in DB0\n", MIC_F2xOC12_MEZZ_STR_ID);
										hwConfigPosPortMask |= 1;
										break;
		default						:	printf("ERROR!!! MIC1 unknown!!!\n");
										return -1;
	}

	/* Mezzanine in DB1 */
	switch(Bb_GetDevType(bb_handle, MEDIA_CARD2_ID))
	{
		case BB_FAILURE				:	break;
		case POS_ATM_OC312_ID		:   printf("Mezzanine %s ", POS_ATM_OC312_STR_ID);
										mezzanine_pos_atm |= 0x2;
										break;
		case POS_ATM_OC48_ID		:   printf("Mezzanine %s ", POS_ATM_OC48_STR_ID);
										mezzanine_pos_atm |= 0x2;
										break;
		case GB_ETH_ID				:   printf("Mezzanine %s ", GB_ETH_STR_ID);
										break;
		default						:	printf("ERROR!!! Device in DB1 unknown!!!\n");
										return -1;
	}

	/* MIC2 */
	switch(Bb_GetDevType(bb_handle, MIC2_ID))
	{
		case BB_FAILURE				:	if((mezzanine_pos_atm & 2) != 0)
										{
										 	printf("in DB1\n");
											hwConfigPosPortMask |= 2;
										}
										else
											printf("No device in DB1\n");
										break;
		case MIC_C4xGB_MEZZ_ID 		:   printf("with %s in DB1\n", MIC_C4xGB_MEZZ_STR_ID);
										hwConfigEthPortMask |= 0xc00;
										for(i = 10; i < 12; i++)
										   hwConfigEthMode[i] = IX_CC_ETH_TX_DRIVER_MODE_GIGA_FULL_DUPLEX |
                             									IX_CC_ETH_TX_DRIVER_MODE_PARITY_EVEN |
                             									IX_CC_ETH_TX_DRIVER_MODE_BLOCK_MPHY;
										break;
		case MIC_F4xGB_MEZZ_ID		:   printf("with %s in DB1\n", MIC_F4xGB_MEZZ_STR_ID);
										hwConfigEthPortMask |= 0xc00;
										for(i = 10; i < 12; i++)
										   hwConfigEthMode[i] = IX_CC_ETH_TX_DRIVER_MODE_FIBER |
                             									IX_CC_ETH_TX_DRIVER_MODE_PARITY_EVEN |
                             									IX_CC_ETH_TX_DRIVER_MODE_BLOCK_MPHY;
										break;
		case MIC_F2xGB_BRD_ID		:   printf("%s on baseboard in slot 2\n", MIC_F2xGB_BRD_STR_ID);
										hwConfigEthPortMask |= 0xc;
										for(i = 2; i < 4; i++)
										   hwConfigEthMode[i] = IX_CC_ETH_TX_DRIVER_MODE_FIBER |
                             									IX_CC_ETH_TX_DRIVER_MODE_PARITY_EVEN |
                             									IX_CC_ETH_TX_DRIVER_MODE_BLOCK_4_8;
										printf("Spi switch to front\n");
										Bb_SwitchSpiMic(bb_handle);
										break;
		case MIC_F4xOC12_MEZZ_ID	:   printf("with %s in DB1\n", MIC_F4xOC12_MEZZ_STR_ID);
										hwConfigPosPortMask |= 2;
										break;
		case MIC_F1xOC48_MEZZ_ID	:   printf("with %s in DB1\n", MIC_F1xOC48_MEZZ_STR_ID);
										hwConfigPosPortMask |= 2;
										break;
		case MIC_F2xOC12_MEZZ_ID	:   printf("with %s in DB1\n", MIC_F2xOC12_MEZZ_STR_ID);
										hwConfigPosPortMask |= 2;
										break;
		default						:	printf("ERROR!!! MIC2 unknown!!!\n");
										return -1;
	}

	if(hwConfigEthPortMask == 0)	/* if there are not MIC_C2xGB_BRD_ID,
									 * MIC_F2xGB_BRD_ID or simple GB_ETH_ID
									 */
	{
		hwConfigEthPortMask |= 0xc;	/* Add backplane ports to the port's mask */
		for(i = 2; i < 4; i++)
			hwConfigEthMode[i] = IX_CC_ETH_TX_DRIVER_MODE_GIGA_FULL_DUPLEX |
        					 	 IX_CC_ETH_TX_DRIVER_MODE_PARITY_EVEN |
                             	 IX_CC_ETH_TX_DRIVER_MODE_BLOCK_4_8;
	}

	Bb_Close(bb_handle);
	return 0;
}
#endif /* USE_BB_DRV_CONFIG */

void vCb(int ctx, int port, int statusMask)
{
    printf("\nPort %u, Status Mask %X\n", port, statusMask);
}

/*
 * Read Vallejo statistics
 *
 */
void vallejo_stats(unsigned int port, uint32 handle)
{
    gbe_mac_ioctl_ptr_64 stats;
    gbe_mac_ioctl_ptr param;

#ifdef	USE_BB_DRV_CONFIG
    if (port > 3)
		return;
	else
		port=hw_port[port];
#else /* USE_BB_DRV_CONFIG */
    if (port > 3) return;
#endif /* USE_BB_DRV_CONFIG */

    stats.portIndex = port;
    param.portIndex = port;

    printf("---------------------------------------------------------\n");
    printf("Reading Vallejo statistics for port %d\n", port);
    printf("---------------------------------------------------------\n");

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_OCTETS_OK,
                                (void *)(&stats)))
        return;
    printf("GET_RX_OCTETS_OK         = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_OCTETS_BAD,
                                (void *)(&stats)))
        return;
    printf("GET_RX_OCTETS_BAD        = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_UC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_RX_UC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_MC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_RX_MC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_BC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_RX_BC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_64,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_64           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_65_127,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_65_127       = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_128_255,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_128_255      = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_256_511,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_256_511      = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_512_1023,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_512_1023     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_1024_1518,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_1024_1518    = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PKTS_1519_MAX,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PKTS_1519_MAX     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FCS_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RX_FCS_ERR           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_DATA_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RX_DATA_ERR          = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_ALLIGN_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RX_ALLIGN_ERR        = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_LONG_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RX_LONG_ERR          = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_JABBER_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RX_JABBER_ERR        = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_PAUSE_MAC_CTL,
                                (void *)(&stats)))
        return;
    printf("GET_RX_PAUSE_MAC_CTL     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_UNKNOWN_CTL_FRAME,
                                (void *)(&stats)))
        return;
    printf("GET_RX_UNKNOWN_CTL_FRAME = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_VLONG_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_VLONG_ERR            = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RUNT_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_RUNT_ERR             = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_SHORT_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_SHORT_ERR            = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_SEQ_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_SEQ_ERR              = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_SYMBOL_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_SYMBOL_ERR           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    /**
     * TX Statistics Register IOCTL Commands.
     */
    printf("---------------------------------------------------------\n");

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_OCTETS_OK,
                                (void *)(&stats)))
        return;
    printf("GET_TX_OCTETS_OK         = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_OCTETS_BAD,
                                (void *)(&stats)))
        return;
    printf("GET_TX_OCTETS_BAD        = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_UC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_TX_UC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_MC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_TX_MC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_BC_PKTS,
                                (void *)(&stats)))
        return;
    printf("GET_TX_BC_PKTS           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_64,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_64           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_65_127,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_65_127       = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_128_255,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_128_255      = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_256_511,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_256_511      = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_512_1023,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_512_1023     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_1024_1518,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_1024_1518    = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PKTS_1519_MAX,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PKTS_1519_MAX     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_PAUSE_FRAME,
                                (void *)(&stats)))
        return;
    printf("GET_TX_PAUSE_FRAME       = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_DEFERRED_ERR,
                                (void *)(&stats)))
        return;

    printf("GET_TX_DEFERRED_ERR      = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_TOTAL_COLLISION,
                                (void *)(&stats)))
        return;
    printf("GET_TX_TOTAL_COLLISION   = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_SINGLE_COLLISION,
                                (void *)(&stats)))
        return;
    printf("GET_TX_SINGLE_COLLISION  = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_MUL_COLLISION,
                                (void *)(&stats)))
        return;
    printf("GET_TX_MUL_COLLISION     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_LATE_COLLISION,
                                (void *)(&stats)))
        return;
    printf("GET_LATE_COLLISION       = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_EXCV_COLLISION,
                                (void *)(&stats)))
        return;
    printf("GET_TX_EXCV_COLLISION    = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_EXCV_DEFERRED_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_TX_EXCV_DEFERRED_ERR = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_EXCV_LEN_DROP,
                                (void *)(&stats)))
        return;
    printf("GET_TX_EXCV_LEN_DROP     = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_UNDERRUN,
                                (void *)(&stats)))
        return;
    printf("GET_TX_UNDERRUN          = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_TX_CRC_ERR,
                                (void *)(&stats)))
        return;
    printf("GET_TX_CRC_ERR           = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_FC_COLLISION_SEND,
                                (void *)(&stats)))
        return;
    printf("GET_FC_COLLISION_SEND    = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    /**
     * RX FIFO Register IOCTL Commands
     */
    printf("---------------------------------------------------------\n");

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_STN_ADDR,
                                (void *)(&stats)))
        return;
    printf("GET_STN_ADDR             = 0x%08x-%08x\n",
           stats.valueHigh, stats.valueLow);


    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FIFO_HIGH_WATERMARK,
                                (void *)(&param)))
        return;
    printf("GET_RX_FIFO_HIGH_WM      = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FIFO_LOW_WATERMARK,
                                (void *)(&param)))
        return;
    printf("GET_RX_FIFO_LOW_WM       = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FRAME_REMOVED,
                                (void *)(&param)))
        return;
    printf("GET_RX_FRAME_REMOVED     = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FIFO_PORT_RESET,
                                (void *)(&param)))
        return;
    printf("GET_RX_FIFO_PORT_RESET   = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FIFO_FRAME_DROP,
                                (void *)(&param)))
        return;
    printf("GET_RX_FIFO_FRAME_DROP   = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RX_FIFO_OVERFLOW_STT,
                                (void *)(&param)))
        return;
    printf("GET_RX_FIFO_OVERFLOW_STT = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_OUT_SEQUENCE_INFO,
                                (void *)(&param)))
        return;
    printf("GET_OUT_SEQUENCE_INFO    = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_DROPPED_PKTS,
                                (void *)(&param)))
        return;
    printf("GET_DROPPED_PKTS         = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_RW_PTR_RX_FIFO,
                                (void *)(&param)))
        return;
    printf("GET_RW_PTR_RX_FIFO       = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_OCCUPANCY_RX_FIFO,
                                (void *)(&param)))
        return;
    printf("GET_OCCUPANCY_RX_FIFO    = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_JUMBO_PKT_SIZE,
                                (void *)(&param)))
        return;
    printf("GET_JUMBO_PKT_SIZE       = 0x%08x\n",
           param.value);

    if (SUCCESS != GbEMAC_Ioctl(&handle,
                                GET_PKT_DROP_CAP_FIFO,
                                (void *)(&param)))
        return;
    printf("GET_PKT_DROP_CAP_FIFO    = 0x%08x\n",
           param.value);

}


#define POS_ATM_DB0_DEVICE_NAME  "/dev/pos_atm_drv0"
#define POS_ATM_DB1_DEVICE_NAME  "/dev/pos_atm_drv1"
int atmFd=-1;

int
framer_init(void)
{

#ifdef USE_BB_DRV_CONFIG
	switch(hwConfigPosPortMask)
	{
    	case 0	:	atmFd = POS_ATM_INV_HANDLE;
        			printf ("There is no POS card in DB0 or DB1.\n");
        			return -1;
    	case 1	:	atmFd = PosAtm_Open(0);
					break;
		case 2	:	atmFd = PosAtm_Open(1);
					break;
		case 3	:	atmFd = PosAtm_Open(0);
					break;
		default	:	atmFd = POS_ATM_INV_HANDLE;
        			printf ("There is no POS card in DB0 or DB1.\n");
        			return -1;
	}
#else /* USE_BB_DRV_CONFIG */
#if defined IX_CONFIGURATION_atmdb0
    /* Open mezzanine DB0  */
    if ((atmFd = PosAtm_Open(0)) == POS_ATM_INV_HANDLE)
    {
        printf ("Cannot open DB0.\n");
        return -1;
    }
#else
    /* Open mezzanine DB1  */
    if ((atmFd = PosAtm_Open(1)) == POS_ATM_INV_HANDLE)
    {
        printf ("Cannot open DB1.\n");
        return -1;
    }
#endif /* IX_CONFIGURATION_atmdb0 */
#endif /* USE_BB_DRV_CONFIG */

    if(PosAtm_SetLinkState(atmFd, 0, POS_ATM_STATE_UP) < 0)
    {
        printf ("\nCannot set Link Up (0) for POS/ATM daughter card. Errno = %X\n", errno);
        return -1;
    }


    if(PosAtm_SetLinkState(atmFd, 1, POS_ATM_STATE_UP) < 0)
    {
        printf ("\nCannot set Link Up (1) for POS/ATM daughter card . Errno = %X\n", errno);
        return -1;
    }

    printf("framer_init END\n");

    return 0;
}

/******************************************************************************/
//static char myArray[512];

void framer_stats(unsigned int devIdx)
{
    pos_atm_ixf6048_counters_t stats;

    stats.link = 0;
    stats.channel = 0;

    if (devIdx == 0)
    {
        if (ioctl(atmFd, POS_ATM_IOCTL_GET_LINK_STATS, &stats) < 0)
        {
            printf ("\nError reading POS/ATM daughter card (link 0) statistics."
                    " Errno = %X\n", errno);
        }
        else
        {
            printf("POS counters:\n");
            printf(" rxFrames:          %d\n",
                   stats.counters.pos.rxFrames);
            printf(" txFrames:          %d\n",
                   stats.counters.pos.txFrames);
            printf(" rxBytes:           %d\n",
                   stats.counters.pos.rxBytes);
            printf(" txBytes:           %d\n",
                   stats.counters.pos.txBytes);
            printf(" rxAbortedFrames:   %d\n",
                   stats.counters.pos.rxAbortedFrames);
            printf(" txAbortedFrames:   %d\n",
                   stats.counters.pos.txAbortedFrames);
            printf(" rxFcsErrors:       %d\n",
                   stats.counters.pos.rxFcsErrors);
            printf(" rxMinLengthErrors: %d\n",
                   stats.counters.pos.rxMinLengthErrors);
            printf(" rxMaxLengthErrors: %d\n",
                   stats.counters.pos.rxMaxLengthErrors);
            printf(" rxFifoOverflows:   %d\n",
                   stats.counters.pos.rxFifoOverflows);
            printf(" txFifoUnderflows:  %d\n",
                   stats.counters.pos.txFifoUnderflows);
        }
    }
    else if (devIdx == 1)
    {
        stats.link = 1;
        if (ioctl(atmFd, POS_ATM_IOCTL_GET_LINK_STATS, &stats) < 0)
        {
            printf ("\nError reading POS/ATM daughter card (link 1) statistics."
                    " Errno = %X\n", errno);
        }
        else
        {
            printf("POS counters:\n");
            printf(" rxFrames:          %d\n",
                   stats.counters.pos.rxFrames);
            printf(" txFrames:          %d\n",
                   stats.counters.pos.txFrames);
            printf(" rxBytes:           %d\n",
                   stats.counters.pos.rxBytes);
            printf(" txBytes:           %d\n",
                   stats.counters.pos.txBytes);
            printf(" rxAbortedFrames:   %d\n",
                   stats.counters.pos.rxAbortedFrames);
            printf(" txAbortedFrames:   %d\n",
                   stats.counters.pos.txAbortedFrames);
            printf(" rxFcsErrors:       %d\n",
                   stats.counters.pos.rxFcsErrors);
            printf(" rxMinLengthErrors: %d\n",
                   stats.counters.pos.rxMinLengthErrors);
            printf(" rxMaxLengthErrors: %d\n",
                   stats.counters.pos.rxMaxLengthErrors);
            printf(" rxFifoOverflows:   %d\n",
                   stats.counters.pos.rxFifoOverflows);
            printf(" txFifoUnderflows:  %d\n",
                   stats.counters.pos.txFifoUnderflows);
        }
    }
}




int main(
int argc,
char **argv)
{
    uint32 err, i, j;
    gbe_mac_ioctl_ptr macIoctlInfo;
    gbe_mac_ioctl_ptr_64 macIoctlInfo_64;
    uint32 handle;
    uint32 handle_table[4] = {0,0,0,0};
    unsigned int status;
#ifdef USE_BB_DRV_CONFIG
	int bb_handle;

    printf("Checking configuration ...\n");
	_ix_set_hwConfig();
#endif /* USE_BB_DRV_CONFIG */

    printf("Initializing POS ports ...\n");

    if (framer_init() < 0)
    {
        printf("framer_init failed\n");
        return 1;
    }

    printf("Initializing POS ports COMPLETE\n");

	handle = 0;

#ifdef USE_BB_DRV_CONFIG
    printf("Initializing Giga Ethernet ports\n");
    for (i = 0; i < IX_CC_ETH_TX_MAX_NUM_PORTS; i++)
    {
    	if (hwConfigEthPortMask & (1 << i))
        {
        	err = GbEMAC_DeviceStart((1 << i), &handle, hwConfigEthMode[i]);
            if (err != SUCCESS)
			{
                printf("Media driver start failed!");
				return -1;
			}
        }
    }

#else /* USE_BB_DRV_CONFIG */
    printf("Initializing Giga Ethernet backplane copper ports ...\n");

    status = GbEMAC_DeviceStart(IX_CC_ETH_TX_DRIVER_PORT_MASK_23,
                                &handle,
                                IX_CC_ETH_TX_DRIVER_MODE_GIGA_FULL_DUPLEX |
                                IX_CC_ETH_TX_DRIVER_MODE_PARITY_EVEN |
                                IX_CC_ETH_TX_DRIVER_MODE_BLOCK_4_8);

    printf("\n*** GbEMAC_DeviceStart() ports: 2,3 status: %0X ***\n",status);

#ifdef SPI23_TO_FRONT

#ifdef GIGE_FRONT_FIBER

    status = GbEMAC_DeviceStart(IX_CC_ETH_TX_DRIVER_PORT_MASK_01,
                                &handle,
                                IX_CC_ETH_TX_DRIVER_MODE_FIBER |
                                IX_CC_ETH_TX_DRIVER_MODE_PARITY_EVEN |
                                IX_CC_ETH_TX_DRIVER_MODE_BLOCK_4_8);

#else /* gigeth front are copper ports */

    status = GbEMAC_DeviceStart(IX_CC_ETH_TX_DRIVER_PORT_MASK_01,
                                &handle,
                                IX_CC_ETH_TX_DRIVER_MODE_GIGA_FULL_DUPLEX |
                                IX_CC_ETH_TX_DRIVER_MODE_PARITY_EVEN |
                                IX_CC_ETH_TX_DRIVER_MODE_BLOCK_4_8);

#endif /* GIGE_FRONT_FIBER */

    printf("\n*** GbEMAC_DeviceStart() ports: 0,1 status: %0X ***\n",status);

#endif /* SPI23_TO_FRONT */
#endif /* USE_BB_DRV_CONFIG */


    /*
     * For all ports set:
     * - the JUMBO MTU size
     * - enable Flow Control capabilities
     * - set RX_FIFO_LOW_WATER_MARK
     * - set RX_FIFO_HIGH_WATER_MARK
     */

    printf("\n*** Configuration of Gb ports ***\n");
    /* GbEMAC_Callback(&handle, vCb, 0); */

#ifdef USE_BB_DRV_CONFIG
	i=1;
	for(j = 0; j < IX_CC_ETH_TX_MAX_NUM_PORTS; j++)
		if(hwConfigEthPortMask & (1 << j))
    	{
			i++;
			hw_port[i]=j;
        	macIoctlInfo_64.portIndex = j;
        	macIoctlInfo.portIndex = j;
#else /* USE_BB_DRV_CONFIG */
    for (i = 2; i < 4; i++)
    {
        macIoctlInfo_64.portIndex = i;
        macIoctlInfo.portIndex = i;
#endif /* USE_BB_DRV_CONFIG */
        macIoctlInfo_64.valueHigh = 0x00000201;
        macIoctlInfo_64.valueLow = 0x02030001 + i;
        err = GbEMAC_Ioctl(&handle,
                           SET_STN_ADDR,
                           (void *)(&macIoctlInfo_64));
        if (err != SUCCESS)
        {
            printf("GbEMAC_Ioctl (SET_STN_ADDR) returned error = %d\n",err);
            return 1;
        }

        printf("MAC ADDR for port %d set\n",i);
        fflush(stdout);

        macIoctlInfo.value = IX_CC_ETH_TX_FRAME_SIZE;
        err = GbEMAC_Ioctl(&handle,
                           SET_MAX_FRAME_SIZE,
                           (void *)(&macIoctlInfo));
        if (err != SUCCESS)
        {
            printf("GbEMAC_Ioctl (SET_MAX_FRAME_SIZE)"
                   " returned error = %d\n",err);
            return 1;
        }

        printf("Frame size for port %d set\n",i);
        fflush(stdout);

        /*
         * The approximate value of high water mark is calculated according to
         * the following algorithm:
         * (MAC_FIFO_SIZE - 2 * IX_CC_ETH_TX_FRAME_SIZE) / 8
         */
        macIoctlInfo.value = 0x700;/* high water mark 0x700 8 bytes locations */
        err = GbEMAC_Ioctl(&handle,
                           SET_RFIFO_HIGH_WATERMARK,
                           (void *)(&macIoctlInfo));
        if (err != SUCCESS)
        {
            printf("GbEMAC_Ioctl (SET_RFIFO_HIGH_WATERMARK)"
                   " returned error = %d\n",err);
            return 1;
        }

        printf("High watermark for port %d set\n",i);
        fflush(stdout);

        macIoctlInfo.value = 0x300;/* low water mark 0x300 8 bytes locations */
        err = GbEMAC_Ioctl(&handle,
                           SET_RFIFO_LOW_WATERMARK,
                           (void *)(&macIoctlInfo));
        if (err != SUCCESS)
        {
            printf("GbEMAC_Ioctl (SET_RFIFO_LOW_WATERMARK)"
                   " returned error = %d\n",err);
            return 1;
        }

        printf("Low watermark for port %d set\n",i);
        fflush(stdout);

        macIoctlInfo.value = 0x7; /* enable flow control capabilities */
        err = GbEMAC_Ioctl(&handle,
                           SET_FC_MODE,
                           (void *)(&macIoctlInfo));
        if (err != SUCCESS)
        {
            printf("GbEMAC_Ioctl (SET_FC_MODE) returned error = %d\n",err);
            return 1;
        }

        printf("Flow control for port %d set\n",i);
        fflush(stdout);

    }/* end for */

    printf("Copper ports initialization COMPLETE. Handle = %d\n",handle);

#ifdef	USE_BB_DRV_CONFIG
	if(hwConfigEthPortMask == IX_CC_ETH_TX_DRIVER_PORT_MASK_01)
	{
		printf("Changing output of SPI2 and SPI3 to front interfaces.\n");
		/*ioctl(handle, BB_IOCTL_SWITCH_SPI_MIC, arg3);*/
	}
#endif /* USE_BB_DRV_CONFIG */

    handle_table[0] = handle_table[1] = handle_table[2] = handle_table[3] = handle;

    while (1)
    {
        char c;
        int  port;

        c = getchar();
        switch (c)
        {
        case '0':
        case '1':
            port = (unsigned int)c - 0x30;
            framer_stats(port);
            break;

        case '2':
        case '3':
            port = (unsigned int)c - 0x30;
#ifdef USE_BB_DRV_CONFIG
            vallejo_stats(port, handle);
#else /* USE_BB_DRV_CONFIG */
            vallejo_stats(port, handle_table[port]);
#endif /* USE_BB_DRV_CONFIG */
            break;

        case 'q':
        case 'Q':
            return 0;

        }
    }
    return 0;
}

