//------------------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//------------------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: 	dl_source.uc
//
//      Purpose: 	Contains macros for dl_sink and dl_source for POS and IPv4 Blocks.
//
//      History:
//
//
//      Date            Comment                         By
//      ---------------------------------------------------------------------
//
//      11/19/2001      Created                         Senthil Nathan
//
//
/////////////////////////////////////////////////////////////////////////////

#ifndef __DL_SOURCE_UC__
#define __DL_SOURCE_UC__

#ifndef	DISABLE_PR5_REWORK

#include	<dl_msg_if_default.uc>

#macro dl_qm_sink[s_hl, e_e_hl, cls_n, o_n, p_size, flow_color_id, rx_sig, scr_sig,\
					 rt_lbl, qm_ring_num]

	dl_csix_rx_to_qm_sink[s_hl, e_e_hl, cls_n, o_n, p_size, flow_color_id, rx_sig, scr_sig,\
					 rt_lbl, qm_ring_num]
#endm

#else
	/*
	 * Original stuff
	 */

#include <stdmac.uc>
#include <sig_macros.uc>
#include <dispatch_loop.h>


/////////////////////////////
//	Some local defines
/////////////////////////////

#if (DL_RING_NUMBER > 11)
#error			"For Ring Number Greater than 11, we cannot use the SCR_Ring#_Full input state. Check RING_NUMBER"
#else
#define_eval	DL_RING_FULL			SCR_Ring/**/DL_RING_NUMBER/**/_Full	
#endif


/////////////////////////////
// Global variables/Registers
/////////////////////////////


.reg 	global	dl_ring					// scratch ring number for pos rx ==>ipv4. This can be made an absolut register
.reg 	global	dl_buf_handle			// The current Buffer handle.
.reg	global	dl_eop_buf_handle		// For large packets, this is the last buffer in the chain
.reg	global	dl_next_block			// Next block that should process the buffer/packet

///////////////////////////////////////////////////////////////////////////////
// _dl_pos_sink_init:
//	 	Description: 
//			Initialisation for POS dl_sink. Should be called before using dl_sink.
//			It simply sets a global variable dl_ring to the corresponding ring number
//			This will later save one instruction in dl_sink (for POS).
//
//			[Note: This is an internal macro not meant to be used by the user.
//			This macro will undergo some more changes.]
//
//	 	Outputs:
//			
//		Inputs:
//		
//		Size:   				:
//			1 instruction
//

#macro _dl_pos_sink_init[]
.begin

	alu_shf[dl_ring, --, B, DL_RING_NUMBER, <<2]	; ring number to be used in get/put 

.end
#endm

/////////////////////////////////////////////////////////////////////////////
// dl_qm_sink
//	
//	Description:
//		Send received packet to the qm block by writing scratch ring
//
//	Outputs:	
//		None
//		
//	Inputs:
//		s_hl:			SOP buffer handle register, this register value will be changed
//						after macroc is excuted.  If the caller want to keep this value  
//						the value should be stored some where 
//		e_e_hl:			Encoded EOP buffer handle: NULL for one buffer cframe
//						otherwise will be real EOP handle  
//		cls_n:			Class number
//		o_n:			Output port number
//		p_size:			packet size
// 		flow_color_id	flow and color encoded as flow_id[0..23] and color[24..31]
//		rx_sig:			Signal used to wait receving logid wake up  
//		scr_sig:		Scratch ring write signal
//		rt_lbl:			return label after waked up by receiving logic
//		qm_ring_num:	QM receiving ring number
//
//	CONSTANTS:	
//		None
//	
//
//	Size: 8 instructions
/////////////////////////////////////////////////////////////////////////////

#macro dl_qm_sink[s_hl, e_e_hl, cls_n, o_n, p_size, flow_color_id, rx_sig, scr_sig,\
					 rt_lbl, qm_ring_num]
.begin
/////////////////////////////////////////////////////////////////////////////
// Local register declaration

.reg tmp_0													; temporary gpr 0

	xbuf_alloc[$sink, 3, write]								; alloc $sink
	alu[$sink0, --, B, s_hl]								; set SOP hndl to $sink0
	alu[$sink1, --, B, e_e_hl]								; set EOP hndl to $sink1
#ifndef ATM_TX
	alu[tmp_0, cls_n, or, o_n, <<4 ]						; set output port/class num
#else
	alu[tmp_0, --, B, cls_n ]								; Do not set oput port num
#endif
	alu[$sink2, tmp_0, or,1, <<31 ]							; set MSB in $sink1
	scratch[put, $sink0, 0, qm_ring_num, 3],\
			sig_done[scr_sig]								; put to SCR ring
	ctx_arb[rx_sig, scr_sig],and,br[rt_lbl]					; ctx swap
	xbuf_free[$sink]										; free $sink
.end
#endm

#endif	// DISABLE_PR5_REWORK
#endif	// __DL_SOURCE_UC__