/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      xscale_me_init.c
 *
 * = DESCRIPTION
 *      This file contains functions to perform the Xscale initialization 
 *      for OC48 POS application. This application doesn't use the
 *      Resource Manager functions to download the code and relies on the
 *      workbench to do it.
 *
 * = AUTHOR
 *      Eswar Eduri
 *      eswar.m.eduri@intel.com
 *
 * = CHANGE HISTORY
 *      04/13/04 	Eswar Eduri	Created.
 * ============================================================================
 * $Id: xscale_me_init.c,v 1.22 2003/10/28 21:20:02 ttschlue Exp $
 */

/*
 *  Include files
 *	-------------
 */

#include	"xscale_me_init.h"
#include	"xscale_me_rm_app.h"

/*
 *  some global macro definitions
 */
#ifdef	__linux

MODULE_LICENSE("Proprietary");

EXPORT_NO_SYMBOLS;
static char *chip;	/* ingress or egress */
static struct proc_dir_entry *p_proc_file;
MODULE_PARM(chip,"s");

#endif	/* __linux */

int media_egress_reset(void);
int media_reset(void);
int rtm_routes(void);
int ingress_system_setup(void);
int egress_system_setup(void);
int clear_ipv4_counters(void);

void *VIRT_SRAM_CH0_BASE;
void *VIRT_SRAM_CH1_BASE;
void *VIRT_DRAM_BASE;
void *VIRT_SCRATCH_BASE;
void *VIRT_MSF_CSR_BASE;
void *VIRT_MSF_RBUF_BASE;
void *VIRT_CAP_CSR_GLOBAL_BASE;
void *VIRT_CAP_CSR_SCRATCH_CONFIG;
void *VIRT_XSCALE_LOCAL_CSR_BASE;
void *VIRT_SLOWPORT_CSR_BASE;
void *VIRT_MEDIA_CARD_BASE0;

/*
 * Include routes file
 */
#include "xscale_me_routes.c"


/**
 * NAME: init_scratch
 *
 * DESCRIPTION: This function initializes SRAM memory to a value
 *              in a given range 
 */
void init_scratch(int data, void *start_addr, void *end_addr)
{
#ifdef	__linux
	memset((char *)start_addr, 
			((unsigned int)end_addr-(unsigned int)start_addr), data);
#else
	memset((char *)start_addr,data,
			((unsigned int)end_addr-(unsigned int)start_addr));
#endif
}

/**
 * NAME: init_sram
 *
 * DESCRIPTION: This function initializes SRAM memory to a value
 *              in a given range 
 */
void init_sram(int data, void *start_addr, void *end_addr)
{

#ifdef	__linux
	memset((char *)start_addr, 
			((unsigned int)end_addr-(unsigned int)start_addr), data);
#else
	memset((char *)start_addr,data,
			((unsigned int)end_addr-(unsigned int)start_addr));
#endif
}


/**
 * NAME: csix_rx_init
 *
 * DESCRIPTION: This function initializes memory for CSIX RX microblock
 */
int csix_rx_init(void) {
	
	unsigned char* finaladdr;

	DBG_MSG("csix_rx_init()	: Initializing CRX SRAM memory :=..\n");

	finaladdr = CSIX_RX_SRAM_BASE + CSIX_RX_NUM_OF_ENTRIES*32 ;

	DBG_MSG("Clearing SRAM ...");
	init_sram(0x0,CSIX_RX_SRAM_BASE,finaladdr);

	DBG_MSG("QM memory initialization COMPLETE.\n\n");
	DBG_MSG("csix_rx_init()	: Initializing COUNTER SRAM memory :=..\n");

	finaladdr = CSIX_RX_COUNTER_BASE + CSIX_RX_NUM_OF_ENTRIES*16 ;

	DBG_MSG("Clearing SRAM ...");
	init_sram(0x0,CSIX_RX_COUNTER_BASE,finaladdr);

	DBG_MSG("COUNTER memory initialization COMPLETE.\n\n");

	return 0;
}


/**
 * NAME: qm_init
 *
 * DESCRIPTION: This routine initializes memory for QM Microblock
 *
 * This function is called both in Ingress and Egress and the base
 * should be set accordingly. TODO
 */
int qm_init(void)
{
	
	unsigned char* finaladdr;
	unsigned char* baseaddr;

	DBG_MSG("qm_init()	: Initializing QM SRAM memory :=..\n");

	baseaddr = QD_SRAM_BASE;
	finaladdr = baseaddr + NUM_OF_Q*16 ;

	DBG_MSG("Clearing SRAM ...");
	init_sram(0x0,QD_SRAM_BASE,finaladdr);

	DBG_MSG("QM memory initialization COMPLETE.\n\n");

	return 0;
}

/**
 * NAME: l2_encap_init
 *
 * DESCRIPTION: This routine initializes memory for ethernet encap
 */
int l2_encap_init(void) {
	
	unsigned char* finaladdr;
	unsigned char* base;
	unsigned char *l2_entry;

	int i;

	DBG_MSG("l2_encap_init()	: Initializing L2 Table SRAM memory..\n");

	base = L2_TABLE_SRAM_BASE;
	finaladdr = L2_TABLE_SRAM_BASE + L2_TABLE_NUM_OF_ENTRIES*32 ;

	DBG_MSG("Clearing SRAM ...");
	init_sram(0x0,L2_TABLE_SRAM_BASE,finaladdr);
	
	/*
	 * we will initialize only 256 L2 entries. Hence the NH entry should
	 * be within this range to be valid.
	 */
	for(i=0;i<=256;i++)
	{
		l2_entry = base + i * 16; 
	
		set_sram(l2_entry+ 0x000,0x800090d7);
		set_sram(l2_entry+ 0x004,0x00080000);
		set_sram(l2_entry+ 0x008,0x90d80008);
		set_sram(l2_entry+ 0x00c,0x00080000);

	}

	
	DBG_MSG("L2 Table memory initialization COMPLETE.\n\n");

	return 1;
}


/**
 * NAME: egress_system_setup
 *
 * DESCRIPTION: This routine will setup memory required to be initialized
 *              for microblocks and also initialize the 6048 media card.
 */
int egress_system_setup(void)
{
  
   /* Initialize the media */

   media_egress_reset() ;

   DBG_MSG("\n****************************************************\n");
   DBG_MSG("\n  OC48 POS Egress System setup begin ...   \n");
   DBG_MSG("\n****************************************************\n");

   DBG_MSG("Initialize the meta data memory to 0x0 ...\n");   	
#ifndef	_XSCALE_APP_WITH_RM_
   init_sram(0, SRAM_CH0_BASE, 
				   SRAM_CH0_BASE + EGRESS_BUF_SRAM_SIZE);
#endif

#ifndef	_XSCALE_APP_WITH_RM_
   /* Initlialize Scratch Rings used by Egress to 0x0 */
   DBG_MSG("Initialize the scratch rings used by Egress to 0x0 ...\n");   	
   init_scratch(0 ,SCRATCH_BASE, SCRATCH_BASE + 10252);
 #endif

   /* Initialize QM Microblock memory */
   qm_init();

   /* Initialize ethernet encap memory */
   l2_encap_init() ;

   /* Initialize CSIX Rx microblock memory */
   csix_rx_init();


   DBG_MSG("\n****************************************************\n");
   DBG_MSG("\n  OC48 POS Egress System setup COMPLETE.   \n");
   DBG_MSG("\n****************************************************\n");

   return 0 ;
}

/**
 * NAME: msf_pll_egress
 *
 * DESCRIPTION: This function initializes MSF Clock Control Register on Egress.
 */
int msf_pll_egress(void)
{
	unsigned int temp_val;
	unsigned int freq;


	freq = MSF_CLKCFG_EG;
	temp_val = ((unsigned int) MCCR_MASK1 | (freq << 16));
	DBG_MSG("Writing 0x%x with 0x%X\n", (unsigned int)MSF_CLK_CTRL_REG,
					(unsigned int)temp_val);
	PUT32(MSF_CLK_CTRL_REG, temp_val);

	temp_val = ((unsigned int)MCCR_MASK2 | (freq << 16));
	DBG_MSG("Writing 0x%x with 0x%X\n", 
					(unsigned int)MSF_CLK_CTRL_REG,
					(unsigned int)temp_val);
	PUT32(MSF_CLK_CTRL_REG, temp_val);

	temp_val = ((unsigned int)MCCR_MASK3 | (freq << 16));
	DBG_MSG("Writing 0x%x with 0x%X\n", 
					(unsigned int)MSF_CLK_CTRL_REG,
					(unsigned int)temp_val);
	PUT32(MSF_CLK_CTRL_REG, temp_val);

	/* now issue a read to flush write buffer */
	DBG_MSG("Value of 0x%x is 0x%x\n",
					(unsigned int)MSF_CLK_CTRL_REG,
					(unsigned int)GET32(MSF_CLK_CTRL_REG));
	return 0 ;
}


/**
 * NAME: media_egress__reset
 *
 * DESCRIPTION: This routine resets  MSF on the egress processor
 */
int media_egress_reset(void)
{


	/* Setting RESET_0 CAP CSR. */
	PUT32(IXP_RESET_0,0x208080) ;
	PUT32(IXP_RESET_0,0x208000) ;

	/* Set MSF Clock PLL 125MHz */


#ifdef	__linux	


#else
        taskDelay(1) ;
#endif

	return 0;

}


/**
 * NAME: rtm_init
 *
 * DESCRIPTION: This function initializes memory for RTM table
 */
int rtm_init(void) {
	
	int cnt;
	unsigned char *baseaddr, *addr;

	DBG_MSG("rtm_init()	: Initializing RTM memory :=..\n");

	baseaddr = SRAM_LOOKUP_BASE;
	cnt = 64 * 1024 + 256 + SIZE_OF_TRIE_SET * NUM_TRIE_SETS;
	addr = baseaddr + (cnt * 4);

	DBG_MSG("Clearing SRAM ...");
	init_sram(0x0,baseaddr,addr);
	
	baseaddr = SRAM_NEXTHOP_BASE;
	cnt = NUM_NEXTHOPS * SIZE_OF_NEXTHOP * 4;
	addr = baseaddr + cnt;

	DBG_MSG("Clearing SRAM nexthop info at 0x%x...",(unsigned int)baseaddr);
	init_sram(0x0,baseaddr,addr);

	DBG_MSG("RTM memory initialization COMPLETE.\n\n");

	return 1;
}

/**
 * NAME: ipv4_clear_counters
 *
 * DESCRIPTION: This function initializes  memory for IPV4 counters
 */
int ipv4_clear_counters(void){

	unsigned char *addr_lo,*addr_hi;

	addr_lo = IPV4_COUNTERS_BASE;
	addr_hi = addr_lo + IPV4_COUNTERS_SIZE;

#ifdef IPV4_COUNTERS_SCRATCH

    init_scratch(0x0,addr_lo,addr_hi);

#else

	init_sram(0x0,addr_lo,addr_hi);
#endif

	return 1;
}


/**
 * NAME: dbcast_init
 *
 * DESCRIPTION: This function initializes  memory for directed broadcast
 */
int dbcast_init(void){

	int size; 
	unsigned char *low, *high;

	size = DBCAST_NUM_BLOCKS * SRAM_READ_BLOCK_COUNT * 4;

	low = DBCAST_SRAM_BASE;
	high = low+size;

	init_sram(0x0,low,high);

	return 1;
}

/**
 * NAME: dbcast_add
 *
 * DESCRIPTION: This function adds entries for directed broadcast
 */
int	dbcast_add(int ipAddr){

	int hash_result, index, value;
	unsigned char *base;


	hash_result = (ipAddr) ^ (ipAddr >> 8) ^ (ipAddr >> 16) ^ (ipAddr>>24);

	hash_result = hash_result & 0xff;


	base = DBCAST_SRAM_BASE + (hash_result << SRAM_READ_SHIFT) * 4;

	for (index=0;index<SRAM_READ_BLOCK_COUNT-1;index++)
	{
		value = GET32(base+(index*4)); 
		if (value)
			continue;
		else
		{
			PUT32(base+(index*4),ipAddr);  
			DBG_MSG("Added 0x%x to DBCAST table at 0x%x\n",
				ipAddr,(unsigned int)(base+(index*4)));
			break;
		}

	}

	if (index == (SRAM_READ_BLOCK_COUNT-1))
		DBG_MSG("DBCAST table is full. Can't add 0x%x\n",ipAddr);

	return 1;
}

/**
 * NAME: ether_mac_filter
 *
 * DESCRIPTION: This function initializes the mac filter table memory
 */
int ether_mac_filter(void) {
	int offset;
	unsigned char* addr;

	DBG_MSG("ether_mac_filter()	: Initializing EThernet filter \
	table memory\n");

	DBG_MSG("Clean up all filter table entries\n");
	addr=SRAM_FLT_TABLE_BASE;
	init_sram(0x0,addr,(addr+FLT_TABLE_SIZE+FLT_ENTRY_SIZE));

	offset = 1;
	addr = SRAM_FLT_TABLE_BASE + offset*FLT_ENTRY_SIZE ;
	DBG_MSG("set up entry[0x%x]\n", (unsigned int)addr);
	PUT32(addr, 0x01010101);
	addr=addr+4;
	PUT32(addr, 0x02030001);

	offset = 3;										
	addr = SRAM_FLT_TABLE_BASE + offset*FLT_ENTRY_SIZE ;
	DBG_MSG("set up entry[0x%x]\n", (unsigned int)addr);
	PUT32(addr, 0x11111111);
	addr=addr+4;
	PUT32(addr, 0x01020003);

	offset = 2;
	addr = SRAM_FLT_TABLE_BASE + offset*FLT_ENTRY_SIZE ;
	DBG_MSG("set up entry[0x%x]\n", (unsigned int)addr);
	PUT32(addr, 0x00010203);
	addr=addr+4;
	PUT32(addr, 0x01030002);


	offset = 0;
	addr = SRAM_PORT_BIT_BASE + offset*SRAM_PORT_BIT_ENTRY;
	DBG_MSG("set up entry[0x%x]\n", (unsigned int)addr);
	PUT32(addr, 0xf); /*simulation value = 0xa*/	

	DBG_MSG("Ether filter table initialization COMPLETE.\n\n");


	return 1;
}


/**
 * NAME: ingress_system_setup
 *
 * DESCRIPTION: This function sets up the memory for ingress processor
 *              for the oc48_pos_ingress application
 */
int ingress_system_setup(void)
{

    DBG_MSG("\n****************************************************\n");
    DBG_MSG("\n  OC48 POS Ingress System setup begin ...  \n");
    DBG_MSG("\n****************************************************\n");

   DBG_MSG("Initialize the meta data memory to 0x0 ...\n");   	
#ifndef	_XSCALE_APP_WITH_RM_
   init_sram(0, SRAM_CH0_BASE, 
				   SRAM_CH0_BASE + INGRESS_BUF_SRAM_SIZE);
#endif

    /* Resetting the media everytime the app is restarted */

    media_reset() ;

    /* Initializing the route tables */

    DBG_MSG("Clearing route table ...\n");

    rtm_init();
   
    DBG_MSG("Initializing with some routes ...\n");

    rtm_routes();

    /*  clearing counters  */

    DBG_MSG("Clearing IPv4 counters  ...\n");

    clear_ipv4_counters();

    DBG_MSG("Initializing the directed broadcast table ...\n");

    dbcast_init();

    PUT32(CONTROL_BLOCK_SRAM_BASE,0xffffffff);

    /* This memory block should always be zero */

    PUT32(SRAM_ZERO_BLOCK, 0) ;

    /* mac filter setup */

    ether_mac_filter() ;

    /* qm initialization */

    qm_init(); 

    DBG_MSG("\n****************************************************\n");
    DBG_MSG("\n  OC48 POS Ingress System setup COMPLETE.  \n");
    DBG_MSG("\n****************************************************\n");
  
    return 0 ;

}


/**
 * NAME: msf_init
 *
 * DESCRIPTION: This function will initialize the the MSF RBUF and enable it 
 */
int msf_init(void)
{

	/* Setting RESET_0 CAP CSR. */
	PUT32(IXP_RESET_0,0x208080);
	PUT32(IXP_RESET_0,0x208000);

	/* disable MSF_Rx_Control MSF CSR */

	PUT32(MSF_RX_CTRL, 0x00000000);

	/* issue a read to flush write buffer */
	DBG_MSG("MSF_RX_CTRL is 0x%x\n",
			(unsigned int)GET32(MSF_RX_CTRL));

	return 0;
}


/**
 * NAME: msf_pll
 *
 * DESCRIPTION: This function initializes MSF Clock Control Register on Egress.
 */
int msf_pll(void)
{
	unsigned int temp_val;
	unsigned int freq;


	freq = MSF_CLKCFG_EG;
	temp_val = ((unsigned int) MCCR_MASK1 | (freq << 16));
	DBG_MSG("Writing 0x%x with 0x%X\n", (unsigned int)MSF_CLK_CTRL_REG,
					(unsigned int)temp_val);
	PUT32(MSF_CLK_CTRL_REG, temp_val);

	temp_val = ((unsigned int)MCCR_MASK2 | (freq << 16));
	DBG_MSG("Writing 0x%x with 0x%X\n", 
					(unsigned int)MSF_CLK_CTRL_REG,
					(unsigned int)temp_val);
	PUT32(MSF_CLK_CTRL_REG, temp_val);

	temp_val = ((unsigned int)MCCR_MASK3 | (freq << 16));
	DBG_MSG("Writing 0x%x with 0x%X\n", 
					(unsigned int)MSF_CLK_CTRL_REG,
					(unsigned int)temp_val);
	PUT32(MSF_CLK_CTRL_REG, temp_val);

		/* now issue a read to flush write buffer */
	DBG_MSG("Value of 0x%x is 0x%x\n",
					(unsigned int)MSF_CLK_CTRL_REG,
					(unsigned int)GET32(MSF_CLK_CTRL_REG));
	return 0 ;
}

/**
 * NAME: media_reset
 *
 * DESCRIPTION: This routine resets media settings and MSF on the ingress
 */
int media_reset(void)
{
	int tries, i;

	/* Initialize MSF */
	msf_init();

	return 0;
}


/**
 * NAME: read_ipv4_counters
 *
 * DESCRIPTION: This function reads IPv4 counters 
 */
int read_ipv4_counters(void)
{

	unsigned char *base = IPV4_COUNTERS_BASE;

	DBG_MSG("\n=====================  IPv4 Counters for all 4 ports ==================\n");

	
	DBG_MSG("RX       =%12d %12d %12d %12d \n",
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_RX  ) ),
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_RX  + 64  ) ), 
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_RX  + 128  ) ),
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_RX  + 192  ) ) ) ;


	DBG_MSG("FWD      =%12d %12d %12d %12d \n",
			( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_FWD  ) ),
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_FWD  + 64  ) ), 
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_FWD  + 128  ) ),
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_FWD  + 192  ) ) ) ;


	DBG_MSG("DROP     =%12d %12d %12d %12d \n",
			( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_DROP  ) ),
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_DROP  + 64  ) ), 
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_DROP  + 128  ) ),
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_DROP  + 192  ) ) ) ;



	DBG_MSG("EXCP     =%12d %12d %12d %12d \n",
			( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_EXCP  ) ),
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_EXCP  + 64  ) ), 
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_EXCP  + 128  ) ),
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_EXCP  + 192  ) ) ) ;


	DBG_MSG("BAD_HDR  =%12d %12d %12d %12d \n",
			( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_BAD_HDR  ) ),
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_BAD_HDR  + 64  ) ), 
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_BAD_HDR  + 128  ) ),
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_BAD_HDR  + 192  ) ) ) ;


	DBG_MSG("BAD_LEN  =%12d %12d %12d %12d \n",
			( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_BAD_LEN  ) ),
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_BAD_LEN  + 64  ) ), 
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_BAD_LEN  + 128  ) ),
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_BAD_LEN  + 192  ) ) ) ;


	DBG_MSG("BAD_TTL  =%12d %12d %12d %12d \n",
			( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_BAD_TTL  ) ),
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_BAD_TTL  + 64  ) ), 
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_BAD_TTL  + 128  ) ),
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_BAD_TTL  + 192  ) ) ) ;


	DBG_MSG("NO_ROUTE =%12d %12d %12d %12d \n",
			( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_NO_ROUTE  ) ),
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_NO_ROUTE  + 64  ) ), 
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_NO_ROUTE  + 128  ) ),
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_NO_ROUTE  + 192  ) ) ) ;


	DBG_MSG("TOO_SMALL=%12d %12d %12d %12d \n",
			( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_TOO_SMALL  ) ),
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_TOO_SMALL  + 64  ) ), 
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_TOO_SMALL  + 128  ) ),
		   ( unsigned int) *(( unsigned int *) (base + OFFSET_PKTS_TOO_SMALL  + 192  ) ) ) ;

	return 0;
}


/**
 * NAME: read_rbuf
 *
 * DESCRIPTION: This function reads rbuf given the element size and the element number
 *              By default it will read element 0
 */
int read_rbuf( unsigned int *element_num, unsigned int rbuf_size)
{

	int i ;

	unsigned int *rbuf_base, *rbuf_add ;

	if( rbuf_size  <= 64  ) 
	{
		DBG_MSG("Assuming rbuf_size to be 128\n") ;
		rbuf_size = 128 ;
	}

	rbuf_base = (unsigned int *) MSF_RBUF_BASE ;

	rbuf_add = ( unsigned int * ) ( ( unsigned int ) rbuf_base + ( ((unsigned int ) element_num ) * rbuf_size));

	DBG_MSG("Printing the Rbuf data at address 0x%x......\n", (unsigned int) rbuf_add);

	for(i=0; i<32; i++ )
	{
		DBG_MSG("0x%x\n", *(rbuf_add) );
		rbuf_add += 1 ;
	} 

	return 0 ;
}

/**
 * NAME: read_scratch_full_reg
 *
 * DESCRIPTION: This function reads the scratch full CSR register
 */
int read_scratch_full_reg(void)
{ 
	int i;
	unsigned int *p = (unsigned int*)CAP_CSR_SCRATCH;
	
	DBG_MSG("Scratch ring full register value is = 0x%x\n", GET32(SCRATCH_RING_STATUS)) ;

	/*
	 * also dump, for each ring, the base address, size, head ptr, tail ptr
	 * - all in byte address
	 */
	DBG_MSG("\t--- All are byte addresses. Size in bytes ---\n") ;
	for(i=0;i<16;i++)
	{
		unsigned int base,head,tail;
		int size;
		/* for all 16 scratch rings */
		switch(p[0]>>30)
		{
			case 0:
				size = 128; break;
			case 1:
				size = 256; break;
			case 2:
				size = 512; break;
			case 3:
				size = 1024; break;
		}

		base = (unsigned int)SCRATCH_BASE + (p[0] & 0x3fff);
		head = base + (p[1] & 0xfff);
		tail = base + (p[2] & 0xfff);

		DBG_MSG("Ring= %2d Size= %4d Base= 0x%.8x Hptr= 0x%.8x Tptr= 0x%.8x\n",
						i,size*(sizeof(unsigned int)),base,head,tail);
		p += 4;	/* ring control reg */
	}
	return 0 ;

}

/**
 * NAME: clear_ipv4_counters
 *
 * DESCRIPTION: This function clears the ipv4 counters
 */
int clear_ipv4_counters(void)
{
		unsigned char* base = IPV4_COUNTERS_BASE;

        *(( unsigned int *) (base + OFFSET_PKTS_RX      ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_FWD     ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_DROP    ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_EXCP    ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_BAD_HDR ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_BAD_LEN ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_BAD_TTL ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_NO_ROUTE) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_TOO_SMALL) ) = 0  ;



        *(( unsigned int *) (base + OFFSET_PKTS_RX        + 64 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_FWD       + 64 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_DROP      + 64 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_EXCP      + 64 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_BAD_HDR   + 64 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_BAD_LEN   + 64 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_BAD_TTL   + 64 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_NO_ROUTE  + 64 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_TOO_SMALL + 64 ) )  = 0 ;


        *(( unsigned int *) (base + OFFSET_PKTS_RX        + 128 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_FWD       + 128 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_DROP      + 128 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_EXCP      + 128 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_BAD_HDR   + 128 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_BAD_LEN   + 128 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_BAD_TTL   + 128 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_NO_ROUTE  + 128 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_TOO_SMALL + 128 ) )  = 0 ;


 	*(( unsigned int *) (base + OFFSET_PKTS_RX        + 192 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_FWD       + 192 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_DROP      + 192 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_EXCP      + 192 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_BAD_HDR   + 192 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_BAD_LEN   + 192 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_BAD_TTL   + 192 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_NO_ROUTE  + 192 ) )  = 0 ;
        *(( unsigned int *) (base + OFFSET_PKTS_TOO_SMALL + 192 ) )  = 0 ;


        DBG_MSG("IPV4 Counters Reset for all 4 ports \n") ;
 
        return 0 ;

}



int rtm_routes(void) {

	l3_create_routes();

	return 0;
}

void	msf_print_status(void)
{
	unsigned int temp;

	temp = GET32(MSF_RX_CTRL);
	DBG_MSG("MSF : MSF RX CTRL is 0x%x\n",temp);
	
	temp = GET32(MSF_TX_CTRL);
	DBG_MSG("MSF : MSF TX CTRL is 0x%x\n",temp);

	temp = GET32(MSF_RX_UP_CTRL_0);
	DBG_MSG("MSF : Port '0' RX UP CTRL is 0x%x\n",temp);
	temp = GET32(MSF_RX_UP_CTRL_1);
	DBG_MSG("MSF : Port '1' RX UP CTRL is 0x%x\n",temp);
	temp = GET32(MSF_RX_UP_CTRL_2);
	DBG_MSG("MSF : Port '2' RX UP CTRL is 0x%x\n",temp);

	temp = GET32(MSF_RX_UP_CTRL_3);
	DBG_MSG("MSF : Port '3' RX UP CTRL is 0x%x\n",temp);
	
	temp = GET32(MSF_TX_UP_CTRL_0);
	DBG_MSG("MSF : Port '0' TX UP CTRL is 0x%x\n",temp);
	temp = GET32(MSF_TX_UP_CTRL_1);
	DBG_MSG("MSF : Port '1' TX UP CTRL is 0x%x\n",temp);
	temp = GET32(MSF_TX_UP_CTRL_2);
	DBG_MSG("MSF : Port '2' TX UP CTRL is 0x%x\n",temp);
	temp = GET32(MSF_TX_UP_CTRL_3);
	DBG_MSG("MSF : Port '3' TX UP CTRL is 0x%x\n",temp);


	temp = GET32(MSF_INTR_STATUS);
	DBG_MSG("MSF : %d mpkts lost 'cos of no RBUFs\n",(temp>>8)&0xf);
	if(temp & 0x10)
		DBG_MSG("MSF : TCW programming error.\n");
	if(temp & 0x2)
		DBG_MSG("MSF : Incorrect vertical parity on CFrame(CSIX mode only)\n");
	if(temp & 0x1)
		DBG_MSG("MSF : Incorrect horizontal parity(all modes)\n");

	temp = GET32(MSF_TX_SEQ_0);
	DBG_MSG("MSF : TBUF '0' partition seq num is %d \n", temp & 0xff);
	if (temp & 0x80000000)
		DBG_MSG("MSF : TBUF '0' partition is empty (no valid elements)\n");
	else
		DBG_MSG("MSF : TBUF '0' partition is not-empty (some valid elements)\n");

	temp = GET32(MSF_TX_SEQ_1);
	DBG_MSG("MSF : TBUF '1' partition seq num is %d \n", temp & 0xff);
	if (temp & 0x80000000)
		DBG_MSG("MSF : TBUF '1' partition is empty (no valid elements)\n");
	else
		DBG_MSG("MSF : TBUF '1' partition is not-empty (some valid elements)\n");

	temp = GET32(MSF_TX_SEQ_2);
	DBG_MSG("MSF : TBUF '2' partition seq num is %d \n", temp & 0xff);
	if (temp & 0x80000000)
		DBG_MSG("MSF : TBUF '2' partition is empty (no valid elements)\n");
	else
		DBG_MSG("MSF : TBUF '2' partition is not-empty (some valid elements)\n");

	temp = GET32(MSF_TX_SEQ_3);
	DBG_MSG("MSF : TBUF '3' partition seq num is %d \n", temp & 0xff);
	if (temp & 0x80000000)
		DBG_MSG("MSF : TBUF '3' partition is empty (no valid elements)\n");
	else
		DBG_MSG("MSF : TBUF '3' partition is not-empty (some valid elements)\n");

}

/*
 * Obtain the memory map
 */
void	xscale_me_mmap_init(void)
{

	DBG_MSG("\nGetting memory mapping ...\n");

	VIRT_SRAM_CH0_BASE = (void *)(ix_uint32)Hal_sram_ch0_virtAddr;
	VIRT_SRAM_CH1_BASE = (void *)(ix_uint32)Hal_sram_ch1_virtAddr;
	VIRT_DRAM_BASE = (void *)(ix_uint32)Hal_dram_ch0_virtAddr;
	VIRT_SCRATCH_BASE = (void *)(ix_uint32)Hal_scratch_rd_wr_swap_virtAddr;
	VIRT_MSF_CSR_BASE = (void *)(ix_uint32)Hal_msf_virtAddr;
	VIRT_MSF_RBUF_BASE = (void *)((unsigned char*)VIRT_MSF_CSR_BASE + 0x2000);
	VIRT_CAP_CSR_GLOBAL_BASE = (void *)
			(((unsigned char*)((ix_uint32)Hal_cap_global_ctl_csr_virtAddr)) + 0xa00);
	VIRT_CAP_CSR_SCRATCH_CONFIG = (void *)
			(((unsigned char*)((ix_uint32)Hal_cap_global_ctl_csr_virtAddr)) + 0x800);
	VIRT_XSCALE_LOCAL_CSR_BASE = (void *)((ix_uint32)Hal_xscale_local_intctl_virtAddr);

	VIRT_SLOWPORT_CSR_BASE = (void *) ((ix_uint32)Hal_cap_slowport_csr_virtAddr); 
	VIRT_MEDIA_CARD_BASE0 = (void *) ((ix_uint32)Hal_slowport_dev1_virtAddr);

}


static unsigned char dump_page[4096];

void dump_memory(void *begin, int nbytes)
{
	int nl_words;
	int cnt;
	unsigned int *p = (unsigned int*)begin;
	unsigned char *buf=dump_page;

	/* somehow, I couldn't dump more than 256 bytes */
	if (nbytes > 256)
		nbytes = 256;

    nl_words = (nbytes + 0x3)/sizeof(long);
	DBG_MSG("\nDumping %d bytes from 0x%x.\n",
			(nl_words)*sizeof(long),(unsigned int)begin);

	for(cnt=0;cnt<nl_words;cnt++)
	{
		if((cnt%4) == 0)
			buf += sprintf(buf,"\n0x%8x : \t",(unsigned int)p);

		buf += sprintf(buf,"%.8x ",*p++);
	}

	DBG_MSG("%s\n\n",dump_page);

}


/*
 *========================================================
 * Below we have some debugging utiliies under Linux
 * These utilities typically dump/modify memory.
 *
 * One way to call invokes these commands is:
 *
 * $ echo <command> > /proc/xscme_init
 *
 */

#ifdef  __linux
/*
 * Code to dump some stuff under Linux and some debugging
 * utilities.
 */

/*
 * Code to dump some stuff under Linux and some debugging
 * utilities.
 */

static char my_name[] = "xscme";

int xscale_me_status(
	char *buf,char **start, off_t offset,int len,int *eof,void *data)
{

	char *addr = CAP_CSR_SCRATCH ;
	len = 0;

	if (!strcmp(chip,"ingress"))
	{
		read_ipv4_counters();
		DBG_MSG("\n");
	}

	read_scratch_full_reg();

	dump_memory(addr,16*16);

#ifdef	_XSCALE_APP_WITH_RM_

	DBG_MSG("\nNumber of Exception packets = %d.\n\n",numExcpPkts);
#endif

	return 0;
}

int xscale_me_cmd(
	struct file *file, const char *cmd, unsigned long count, void *data)
{
	char *str, *sp;
	char *token;
	char *delim="\t\n\b ";
	/*
	 * need to parse the command
	 */

	str = sp = vmalloc(count + 1);
	if (!str)
	{
		DBG_MSG("ERROR : can't allocated memory in %s.\n",my_name);
		return -1;
	}

	memcpy(str,cmd,count);
	str[count-1] = 0;	/* includes \n */

	/* DBG_MSG("\nExecuting command \"%s\"\n\n",str);*/

	while ((token = strsep(&sp,delim)))
	{
		/* ignore delimiters */
		if (strstr(delim,token) != NULL)
			continue;

		/*
		 * check for commands issued
		 */
		if (strcmp(token,"dump_sram")==0)
		{
			/*
			 * need to dump sram
			 */
			char *addr;
			unsigned long temp;
			token = strsep(&sp,delim);
			if (!token)
			{
				DBG_MSG("ERROR : Expected 'dump_sram offset numBytes'\n");
				return -1;
			}
			temp = simple_strtoul(token,NULL,0);
			if((temp >> 30) != 0)
				addr = VIRT_SRAM_CH1_BASE;
			else
				addr = VIRT_SRAM_CH0_BASE;
			addr += (temp & 0x3fffffff);
			token = strsep(&sp,delim);
			if (!token)
			{
				DBG_MSG("ERROR : Expected 'dump_sram offset numBytes'\n");
				return -1;
			}
			temp = simple_strtoul(token,NULL,0);
			dump_memory(addr,(int)temp);
	
		}
		else if(strcmp(token,"dump_dram")==0)
		{
			/*
			 * dump dram
			 */
			char *addr;
			unsigned long temp;
			token = strsep(&sp,delim);
			if (!token)
			{
				DBG_MSG("ERROR : Expected 'dump_dram offset numBytes'\n");
				return -1;
			}
			temp = simple_strtoul(token,NULL,0);
			addr = VIRT_DRAM_BASE;
			addr += (temp & 0x3fffffff);
			token = strsep(&sp,delim);
			if (!token)
			{
				DBG_MSG("ERROR : Expected 'dump_dram offset numBytes'\n");
				return -1;
			}
			temp = simple_strtoul(token,NULL,0);
			dump_memory(addr,(int)temp);
		}
		else if(strcmp(token,"dump_scratch")==0)
		{
			/*
			 * dump scratch
			 */
			char *addr;
			unsigned long temp;
			token = strsep(&sp,delim);
			if (!token)
			{
				DBG_MSG("ERROR : Expected 'dump_scratch offset numBytes'\n");
				return -1;
			}
			temp = simple_strtoul(token,NULL,0);
			addr = VIRT_SCRATCH_BASE;
			addr += (temp & 0x3fffffff);
			token = strsep(&sp,delim);
			if (!token)
			{
				DBG_MSG("ERROR : Expected 'dump_scratch offset numBytes'\n");
				return -1;
			}
			temp = simple_strtoul(token,NULL,0);
			dump_memory(addr,(int)temp);
		}
		else if(strcmp(token,"dump_rbuf")==0)
		{
			/*
			 * dump RBUF
			 */
			char *addr;
			unsigned long elem_num;
			unsigned long elem_size = 128;
			token = strsep(&sp,delim);
			if (!token)
			{
				DBG_MSG("ERROR : Expected 'dump_rbuf elemNum [elemSize]'\n");
				return -1;
			}
			elem_num = simple_strtoul(token,NULL,0);

			if (count >= (sp-str))
			{
				token = strsep(&sp,delim);
				if (!token)
				{
					elem_size = simple_strtoul(token,NULL,0) & 0x3ff;
				}
			}
			addr = VIRT_MSF_RBUF_BASE + (elem_num * elem_size);
			dump_memory(addr,(int)elem_size);
		}
		else if(strcmp(token,"d")==0)
		{
			/*
			 * simply dump 
			 */
			char *addr;
			unsigned long temp;
			token = strsep(&sp,delim);
			if (!token)
			{
				DBG_MSG("ERROR : Expected 'd virtAddr numBytes'\n");
				return -1;
			}
			addr = (unsigned char*) simple_strtoul(token,NULL,0);
			token = strsep(&sp,delim);
			if (!token)
			{
				DBG_MSG("ERROR : Expected 'd virtAddr numBytes'\n");
				return -1;
			}
			temp = simple_strtoul(token,NULL,0);
			dump_memory(addr,(int)temp);
		}
		else if(strcmp(token,"m")==0)
		{
			/*
			 * modify 
			 */
			unsigned long *addr;
			unsigned long temp;
			token = strsep(&sp,delim);
			if (!token)
			{
				DBG_MSG("ERROR : Expected 'm virtAddr value'\n");
				return -1;
			}
			addr = (unsigned long*) simple_strtoul(token,NULL,0);
			token = strsep(&sp,delim);
			if (!token)
			{
				DBG_MSG("ERROR : Expected 'm virtAddr value'\n");
				return -1;
			}
			temp = simple_strtoul(token,NULL,0);
			*addr = temp;
		}
		else if(strcmp(token,"clear_ipv4_counters")==0)
		{
			/*
			 * clear IPV4 counters
			 */
			clear_ipv4_counters();
		}
        else if(strcmp(token,"dump_scratch_rings")==0)
	    {
		    /*
			 * dump scratch rings
			 */
			char *addr = CAP_CSR_SCRATCH ;
			dump_memory(addr,16*16);
		}
        else if(strcmp(token,"msf_status")==0)
	    {
			msf_print_status();

		}
#ifdef	_XSCALE_APP_WITH_RM_
        else if(strcmp(token,"dump_buffer_stats")==0)
		{
		    /*
			 * dump buffer stats
			 */
			xscale_me_buffer_stats(chip);
		}
#endif
		else 
		{
			DBG_MSG("ERROR : Unknown command '%s'\n",str);
			return -1;
		}

	}

	/*
	 * free memory allocated
	 */
	vfree(str);
	return count;
}
#endif	/* __linux */



/*
 * init_module for the Xscale initialization code
 * ----------------------------------------------
 */


#ifdef	__linux
int system_setup(void)
#else
int	system_setup(char *chip)
#endif
{
	
	ix_error err;

	if (!chip)
	{
#ifdef	__linux
		DBG_MSG("ERROR: USAGE 'insmod xscale_init.o chip=<ingress/egress>.\n");
#else
	/* vxworks */
		DBG_MSG("ERROR: USAGE 'system_setup(\"ingress/egress\").\n");
#endif
		return -1;
	}

#ifdef	_XSCALE_APP_WITH_RM_
	/*
	 * Init RM anyway
	 */
	DBG_MSG("%s : Initializing RM ...\n",chip);
    err = ix_rm_init(0);
    if(err != IX_SUCCESS)
	{
		DBG_MSG("\n%s : RM Initialization FAILED.\n", chip);
        return err;
	}

	xscale_me_mmap_init();

#endif



	if(!strcmp(chip,"ingress"))
	{
		/*
		 * perform Ingress initialization
		 */
#ifdef	_XSCALE_APP_WITH_RM_
		if(xscale_me_rm_app_init("ingress"))
			return -1;
#endif
		ingress_system_setup();
#ifdef	_XSCALE_APP_WITH_RM_
		rm_app_start_ingress_me();
#endif
	}
	else if(!strcmp(chip,"egress"))
	{
		/*
		 * perform Egress initialization
		 */

		/* init media */
		media_init();

#ifdef	_XSCALE_APP_WITH_RM_
		if(xscale_me_rm_app_init("egress"))
			return -1;
#endif


		media_egress_reset();
		egress_system_setup();
#ifdef	_XSCALE_APP_WITH_RM_
		rm_app_start_egress_me();
#endif
	}
	else
	{
#ifdef	__linux
		DBG_MSG("ERROR: USAGE 'insmod xscale_init.o chip=<ingress/egress>.\n");
#else
	/* vxworks */
		DBG_MSG("ERROR: USAGE 'system_setup(\"ingress/egress\").\n");
#endif
		return -1;
	}

#ifdef	__linux
	p_proc_file = create_proc_entry(my_name,S_IFREG | S_IWUSR,&proc_root);

	if (p_proc_file)
	{
		p_proc_file->read_proc = xscale_me_status;
		p_proc_file->write_proc = xscale_me_cmd; 
	}

	else
	{
		DBG_MSG("WARNING : Can't regsiter stats under /proc \n");
		return 0;
	}
#endif

	DBG_MSG("\n@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n");
	DBG_MSG("        %s Initialization complete.         \n",chip);
	DBG_MSG("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n\n");
	return 0;

}

#ifdef	__linux
void system_clean(void)
#else
void system_clean(char *chip)
#endif
{

#ifdef	__linux
	if (p_proc_file)
		remove_proc_entry(my_name,&proc_root);
#endif


	DBG_MSG("\n@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n");
	DBG_MSG("        %s BYE BYE !!!         \n",chip);
	DBG_MSG("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n\n");
	/*
	 * nothing to do
	 */
	return;
}

#ifdef	__linux
module_init(system_setup);
module_exit(system_clean);
#endif	/* __linux */


