
/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      xscale_me_init.h
 *
 * = DESCRIPTION
 *      This file contains global definitions and macros used by the 
 *      source files of OC48 POS application.
 *
 * = AUTHOR
 *      Eswar Eduri
 *      eswar.m.eduri@intel.com
 *
 * = CHANGE HISTORY
 *      04/13/04 
 * ============================================================================
 * $Id: xscale_me_init.h,v 1.6 2003/06/24 18:00:59 emeduri Exp $
 */
#ifndef	__XSCALE_ME_INIT_H__
#define	__XSCALE_ME_INIT_H__

/*
 * Bunch of include files
 * ======================
 */

#ifdef	__linux
#include	<linux/module.h>
#include	<linux/version.h>
#include	<linux/kernel.h>
#include	<linux/init.h>
#include	<linux/types.h>
#include	<linux/slab.h>
#include	<linux/proc_fs.h>
#include	<linux/vmalloc.h>
#else	/* __vxworks */
	/*
	 * vxworks files here
	 */
#endif

#include	"halMmap.h"

/* Macro to write and read one longword from memory 
   the macro assumes that the memory being read or written to
   is mapped 
 */

#define GET32(address) 			(*(volatile unsigned int *)(address))
#define PUT32(address,val)		(*(volatile unsigned int *)(address) = (val))
#define set_sram(a, b) 			PUT32((a), (b))
#define set_dram(a, b) 			set_sram(a,b)
#define	SRAM_WRITE(a,b)			set_sram(a,b)
#define	SDRAM_WRITE(a,b)		set_dram(a,b)

/*
 * Memory mapped base addresses.
 * ==============================
 */

extern void *VIRT_SRAM_CH0_BASE;
extern void *VIRT_SRAM_CH1_BASE;
extern void *VIRT_DRAM_BASE;
extern void *VIRT_SCRATCH_BASE;
extern void *VIRT_MSF_CSR_BASE;
extern void *VIRT_MSF_RBUF_BASE;
extern void *VIRT_CAP_CSR_GLOBAL_BASE;
extern void *VIRT_CAP_CSR_SCRATCH_CONFIG;
extern void *VIRT_XSCALE_LOCAL_CSR_BASE;
extern void *VIRT_SLOWPORT_CSR_BASE;
extern void *VIRT_MEDIA_CARD_BASE0;

/* memory */
#define SRAM_CH0_BASE			VIRT_SRAM_CH0_BASE
#define SRAM_CH1_BASE			VIRT_SRAM_CH1_BASE
#define SDRAM_BASE	        	VIRT_DRAM_BASE
#define SCRATCH_BASE       		VIRT_SCRATCH_BASE

/* msf */
#define	MSF_CSR_BASE			VIRT_MSF_CSR_BASE
#define	MSF_RBUF_BASE			VIRT_MSF_RBUF_BASE

#define	MSF_TBUF_BASE			MSF_RBUF_BASE

/* csr */
#define	CAP_CSR_GLOBAL_BASE		VIRT_CAP_CSR_GLOBAL_BASE
#define	CAP_CSR_SCRATCH			VIRT_CAP_CSR_SCRATCH_CONFIG
#define	XSCALE_LOCAL_CSR_BASE	VIRT_XSCALE_LOCAL_CSR_BASE 

#define	SLOWPORT_CSR_BASE		VIRT_SLOWPORT_CSR_BASE	
#define	POS_MEDIA_CARD_BASE		VIRT_MEDIA_CARD_BASE0 

/*
 * specific CSRs addresses
 * -----------------------
 */
#define	IXP_RESET_0				CAP_CSR_GLOBAL_BASE + 0x0c
#define	MSF_CLK_CTRL_REG		CAP_CSR_GLOBAL_BASE + 0x08
#define	MSF_RX_CTRL				MSF_CSR_BASE + 0x00
#define	MSF_RX_UP_CTRL_0		MSF_CSR_BASE + 0x80
#define	MSF_RX_UP_CTRL_1		MSF_CSR_BASE + 0x84
#define	MSF_RX_UP_CTRL_2		MSF_CSR_BASE + 0x88
#define	MSF_RX_UP_CTRL_3		MSF_CSR_BASE + 0x8c
#define	MSF_TX_CTRL				MSF_CSR_BASE + 0x04
#define	MSF_TX_UP_CTRL_0		MSF_CSR_BASE + 0x90
#define	MSF_TX_UP_CTRL_1		MSF_CSR_BASE + 0x94
#define	MSF_TX_UP_CTRL_2		MSF_CSR_BASE + 0x98
#define	MSF_TX_UP_CTRL_3		MSF_CSR_BASE + 0x9c
#define	MSF_RBUF_ELE_DONE		MSF_CSR_BASE + 0x44
#define	SCRATCH_RING_STATUS		XSCALE_LOCAL_CSR_BASE + 0x58
#define	MSF_INTR_STATUS			MSF_CSR_BASE + 0x08
#define	MSF_TX_SEQ_0			MSF_CSR_BASE + 0x60
#define	MSF_TX_SEQ_1			MSF_CSR_BASE + 0x64
#define	MSF_TX_SEQ_2			MSF_CSR_BASE + 0x68
#define	MSF_TX_SEQ_3			MSF_CSR_BASE + 0x6c

/*
 * Now, the microblock specific tables. The pointers have some
 * default values equal to those specified in dl_system.h file.
 * However, we will overwrite these values when we use RM to
 * obtain this memory.
 * ============================================================
 */

#define CSIX_RX_NUM_OF_ENTRIES	1024
#define	L2_TABLE_NUM_OF_ENTRIES	65536
#define	FLT_TABLE_SIZE			1024
#define	FLT_ENTRY_SIZE			8
#define	SRAM_PORT_BIT_ENTRY		0x1
#define	SIZE_OF_TRIE_SET		16
#define	NUM_TRIE_SETS			1024
#define	NUM_NEXTHOPS			512
#define	SIZE_OF_NEXTHOP			4	
#define	SRAM_READ_BLOCK_COUNT	8	/* in number of long words */
#define	DBCAST_BLOCK_SIZE		SRAM_READ_BLOCK_COUNT
#define	SRAM_READ_SHIFT			3	/* block count is pow of 2 */
#define	DBCAST_NUM_BLOCKS		256
#define NUM_OF_Q				1025
#define IPV4_COUNTERS_SCRATCH   1	/* for counters in scratch */
#define	IPV4_COUNTERS_SIZE		0x1000
#define BUFFER_SIZE             2048
#define META_SIZE               32
#define	INGRESS_BUF_SRAM_SIZE	BUFFER_SIZE * META_SIZE
#define	EGRESS_BUF_SRAM_SIZE	BUFFER_SIZE * META_SIZE

/* offsets of counters in big_endian format.*/
#define	OFFSET_PKTS_RX			0 	/* Packets Rxed */
#define	OFFSET_PKTS_FWD			4 	/* Packets Fwd */
#define	OFFSET_PKTS_DROP		8 	/* Packets dropped */
#define	OFFSET_PKTS_EXCP		12 	/* Packets excp */
#define	OFFSET_PKTS_BAD_HDR		16 	/* pkts bad cksum */
#define	OFFSET_PKTS_BAD_LEN		20 	/* pkts bad len */
#define	OFFSET_PKTS_BAD_TTL		24 	/* pkts with bad TTL */
#define	OFFSET_PKTS_NO_ROUTE   	28 	/* packets with no route */
#define	OFFSET_PKTS_TOO_SMALL  	32 	/* packets too small */


/* Defines for MCCR register */
#define PLL_25MHZ				0x0
#define PLL_50MHZ				0x5
#define PLL_125MHZ				0xF

#define MSF_CLKCFG_MEDIA		PLL_125MHZ
#define MSF_CLKCFG_SF			PLL_125MHZ

#define MSF_CLKCFG_IN			(MSF_CLKCFG_MEDIA | (MSF_CLKCFG_SF << 4))
#define MSF_CLKCFG_EG			(MSF_CLKCFG_SF | (MSF_CLKCFG_MEDIA << 4))

#define MCCR_MASK1				0x0000F0F0 /* bypass and powerdown */
#define MCCR_MASK2				0x000000F0 /* powerdown */
#define MCCR_MASK3				0x00000000


#ifndef	_XSCALE_APP_WITH_RM_
	/****************************************************
	 * In this case, we will simply hardcode the values
	 * This is to be in sync with dl_system.h file
	 ****************************************************
	 */
/* defines for CSIX RX */
#define	CSIX_RX_SRAM_BASE		SRAM_CH0_BASE+0x200000
#define	CSIX_RX_COUNTER_BASE	SRAM_CH0_BASE+0x300000

/*
 * L2 table entries on Egress
 */
#define	L2_TABLE_SRAM_BASE 		SRAM_CH0_BASE + 0x320000 

/* ethernet mac filtering */

#define	SRAM_FLT_TABLE_BASE		SRAM_CH1_BASE + 0x302000
#define	SRAM_PORT_BIT_BASE		SRAM_CH1_BASE + 0x301000 

/* define for rtmInit */

#define	SRAM_LOOKUP_BASE		SRAM_CH1_BASE
#define	SRAM_NEXTHOP_BASE		SRAM_CH0_BASE + 0x300000 

/* define for directed dbcast_init */

#define	DBCAST_SRAM_BASE		SRAM_CH1_BASE + 0x201000 

/* IPV4 control block */ 
#define	CONTROL_BLOCK_SRAM_BASE	SRAM_CH1_BASE + 0x203000

#define	SRAM_ZERO_BLOCK			SRAM_CH0_BASE 

/* defines for QM */

#define	QD_SRAM_BASE			SRAM_CH0_BASE + 0x100000

/* define for ipv4clear counters */
#ifdef IPV4_COUNTERS_SCRATCH
#define IPV4_COUNTERS_BASE      SCRATCH_BASE + 11264
#else
#define IPV4_COUNTERS_BASE      SRAM_CH1_BASE + 0x200000
#endif

/*
 * Buffer related
 */
#define BUF_SDRAM_BASE          SDRAM_BASE

#else	/* _XSCALE_APP_WITH_RM_ */

	/*
	 * in this case, the application has to initialize
	 * these variables with proper values. This is ugly, but
	 * we can reuse the code from .ind script file. 
	 *
	 * This code is common to both Vxworks and Linux.
	 */
extern char* CSIX_RX_SRAM_BASE;
extern char* CSIX_RX_COUNTER_BASE;	
extern char* L2_TABLE_SRAM_BASE; 		
extern char* SRAM_FLT_TABLE_BASE;	
extern char* SRAM_PORT_BIT_BASE;		
extern char* SRAM_LOOKUP_BASE;	
extern char* SRAM_NEXTHOP_BASE;		
extern char* DBCAST_SRAM_BASE;		
extern char* CONTROL_BLOCK_SRAM_BASE;	
extern char* SRAM_ZERO_BLOCK;		
extern char* QD_SRAM_BASE;	
extern char* IPV4_COUNTERS_BASE;      
extern char* BUF_SDRAM_BASE;        
extern char* INGRESS_BUF_SRAM_BASE;
extern char* EGRESS_BUF_SRAM_BASE;

#endif	/* end of _XSCALE_APP_WITH_RM_ */

/*
 * DEBUGGING UTILITIES 
 * -------------------
 */

#ifdef	__linux
	/*
	 * Linux stuff
	 */
#define	DBG_ERR		KERN_ERR
#define	DBG_WARN	KERN_WARNING
#define	DBG_INFO	KERN_INFO
#define	DEFAULT_LVL	DBG_WARN	

#define	DBG_LOG(lvl,str,args...)	printk(lvl""str, ##args)
#define	DBG_MSG(str,args...)		DBG_LOG(DEFAULT_LVL, str, ##args)

#else	/* __linux */
	/*
	 * vxworks stuff 
	 */
#define	DBG_MSG(str,args...)		printf(str,##args)

#endif	/* __linux */

/*
 * function prototypes
 */
extern void xscale_me_mmap_init(void);
extern void media_init(void);
#ifdef	_XSCALE_APP_WITH_RM_
extern void xscale_me_buffer_stats(char *);
#endif
#endif	/* __XSCALE_ME_INIT_H__ */


