/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      xscale_me_rm_app.c
 *
 * = DESCRIPTION
 *      This is a sample application that will exercise Resource Manager 
 *      functions to download the microcode image after paching symbols, 
 *      start the MEs and handle RM->ME and ME->RM 
 *
 * = AUTHOR
 *      Eswar Eduri
 *      eswar.m.eduri@intel.com
 *
 * = CHANGE HISTORY
 * 	04/17/04	Eswar Eduri			Created
 * ============================================================================
 * $Id: xscale_me_rm_app.c,v 1.19 2003/11/20 23:12:57 ddtran Exp $
 */


/*
 * Include files
 */
#ifdef	__linux
#define	__NO_VERSION__
#endif

#include	"xscale_me_rm_app.h"


/*
 * variables used by functions borrowed from
 * script files
 */
char* CSIX_RX_SRAM_BASE;
char* CSIX_RX_COUNTER_BASE;	
char* L2_TABLE_SRAM_BASE; 		
char* SRAM_FLT_TABLE_BASE;	
char* SRAM_PORT_BIT_BASE;		
char* SRAM_LOOKUP_BASE;	
char* SRAM_NEXTHOP_BASE;		
char* DBCAST_SRAM_BASE;		
char* CONTROL_BLOCK_SRAM_BASE;	
char* SRAM_ZERO_BLOCK;		
char* QD_SRAM_BASE;	
char* IPV4_COUNTERS_BASE;      
char* BUF_SDRAM_BASE;        
char* INGRESS_BUF_SRAM_BASE;
char* EGRESS_BUF_SRAM_BASE;

/*
 * Globals for Ingress 
 */
#ifdef	__linux
char* s_IngressUOFFile = 
	"/mnt/uof/oc48_pos_ingress.uof";
#else
	/* windows */
char* s_IngressUOFFile = 
	"c:\\ixa_sdk_3.5\\src\\applications\\ipv4_forwarder\\oc48_pos\\ingress\\wbench_project\\oc48_pos_ingress.uof";
#endif
	/*
	 * buffer free list
	 */
ix_buffer_free_list_handle ingressHwFreeList = 0;
ix_buffer_free_list_info   ingressHwFreeListInfo ;
	/*
	 * Xscale - ME comm rings
	 */
ix_hw_ring_handle ingressCoreToMeDataRing = 0;
ix_hw_ring_handle ingressCoreToMeCtrlRing = 0;

t_app_mem_desc	ingress_mem[] = {
	/*
	 * TODO : add your memory map here. Name should be same as symbol
	 * to ease lookup later
	 */
	/* memType			memChan	size  value	name		varName*/
	{IX_MEMORY_TYPE_SRAM,1,2*1024*1024,	NULL,"TRIE_TABLE_SRAM_BASE",&SRAM_LOOKUP_BASE},
	{IX_MEMORY_TYPE_SRAM,1,1024,		NULL,"SRAM_FLT_TABLE_BASE",&SRAM_FLT_TABLE_BASE},
	{IX_MEMORY_TYPE_SRAM,1,128,			NULL,"SRAM_PORT_BIT_MAP_BASE",&SRAM_PORT_BIT_BASE},
	{IX_MEMORY_TYPE_SRAM,1,1024,		NULL,"PACKET_COUNTERS_SRAM_BASE",NULL},
	{IX_MEMORY_TYPE_SRAM,0,32768,		NULL,"QD_SRAM_BASE",&QD_SRAM_BASE},
	{IX_MEMORY_TYPE_SRAM,0,2048,		NULL,"SCHED_VOQ_CREDIT_BASE",NULL},
	{IX_MEMORY_TYPE_SRAM,0,64*1024,		NULL,"CSIX_TX_CTX_SRAM_BASE",NULL},
	{IX_MEMORY_TYPE_SRAM,0,1024,		NULL,"CSIX_TX_COUNTERS_SRAM_BASE",NULL},
	{IX_MEMORY_TYPE_SRAM,0,128,			NULL,"SRAM_ZERO_BLOCK",&SRAM_ZERO_BLOCK},
#ifdef	IPV4_COUNTERS_SCRATCH
	{IX_MEMORY_TYPE_SCRATCH,0,2048,		NULL,"IPV4_STATS_TABLE_BASE",&IPV4_COUNTERS_BASE},
#else
	{IX_MEMORY_TYPE_SRAM,1,4096,		NULL,"STATS_TABLE_SRAM_BASE",&IPV4_COUNTERS_BASE},
#endif
	{IX_MEMORY_TYPE_SRAM,1,8192,		NULL,"DBCAST_TABLE_SRAM_BASE",&DBCAST_SRAM_BASE},
	{IX_MEMORY_TYPE_SRAM,1,128,			NULL,"CONTROL_BLOCK_SRAM_BASE",&CONTROL_BLOCK_SRAM_BASE},
	{IX_MEMORY_TYPE_SRAM,0,64*1024,		NULL,"NEXTHOP_TABLE_SRAM_BASE",&SRAM_NEXTHOP_BASE}
};
#define	INGRESS_MEM_NUM_ENTRIES	sizeof(ingress_mem)/sizeof(ingress_mem[0])


/*
 * Globals for Egress 
 */
#ifdef	__linux
char* s_EgressUOFFile = 
	"/mnt/uof/oc48_sphy_pos_egress.uof";
#else	/* windows */
char* s_EgressUOFFile = 
	"C:\\IXA_SDK_3.5\\src\\applications\\ipv4_forwarder\\oc48_pos\\egress_sphy_mphy4\\wbench_project\\oc48_sphy_pos_egress.uof";
#endif
	/*
	 * buffer free list
	 */
ix_buffer_free_list_handle egressHwFreeList = 0;
ix_buffer_free_list_info   egressHwFreeListInfo ;

	/*
	 * Xscale - ME comm rings
	 */
ix_hw_ring_handle egressCoreToMeDataRing = 0;
ix_hw_ring_handle egressCoreToMeCtrlRing = 0;

t_app_mem_desc	egress_mem[] = {
	/*
	 * TODO : add your memory map here. Name should be same as symbol
	 * to ease lookup later
	 */
	/* memType			memChan	size  value	name		*/
	{IX_MEMORY_TYPE_SRAM,0,4*1024,	NULL,"L2_TABLE_SRAM_BASE",&L2_TABLE_SRAM_BASE},
	{IX_MEMORY_TYPE_SRAM,0,64*1024,	NULL,"CSIX_RX_CTX_SRAM_BASE",&CSIX_RX_SRAM_BASE},
	{IX_MEMORY_TYPE_SRAM,0,1024,	NULL,"CSIX_RX_COUNTERS_SRAM_BASE",&CSIX_RX_COUNTER_BASE},
	{IX_MEMORY_TYPE_SRAM,0,32768,	NULL,"QD_SRAM_BASE",&QD_SRAM_BASE},
	{IX_MEMORY_TYPE_SRAM,0,128,		NULL,"SRAM_ZERO_BLOCK",&SRAM_ZERO_BLOCK},
	{IX_MEMORY_TYPE_SRAM,1,8*1024,	NULL,"SCHED_WEIGHT_CREDIT_BASE",NULL}
};

#define	EGRESS_MEM_NUM_ENTRIES	sizeof(egress_mem)/sizeof(egress_mem[0])

unsigned int	numExcpPkts;
int start_micro_engines = 1;

/*
 * Sample packet handler
 *
 * NOTE : This function cann't handle packets at high rates with the 
 * DBG_MSGs turned on.
 */

/*
 * Sample packet handler
 *
 * NOTE : This function cann't handle packets at high rates with the 
 * DBG_MSGs turned on.
 */
ix_error xscale_me_pkt_handler(
	ix_buffer_handle arg_hBuffer, 
   	ix_uint32 arg_UserData,		/* exception code */
   	void* arg_pContext 
   	)
{
    const char me[] = "xscale_me_pkt_handler";

    ix_error err = IX_SUCCESS;
    ix_hw_buffer_meta* pMeta;
    unsigned char* pData ;

	ix_memory_type memType;
	ix_uint32	memChan;
	ix_uint32	offset1, offset2;
	void		*pAddr;

	numExcpPkts++;

	DBG_MSG("\n%s : Called for buffer 0x%x.\n",me,(unsigned int)arg_hBuffer);
			
    if(arg_hBuffer == 0xff)
		return err;


    err = ix_rm_buffer_get_meta(arg_hBuffer,(void**)&pMeta);
	if(err != IX_SUCCESS)
   		return err;

	/*
	 *  add meta data processing here
	 */

    err = ix_rm_buffer_get_data(arg_hBuffer, (void**)&pData);
	if(err != IX_SUCCESS)
   		return err;

	pData = pData + pMeta->m_Offset;

	err = ix_rm_get_phys_offset(
			(const void*) pMeta, &memType,&memChan,&offset1,&pAddr);
	if(err != IX_SUCCESS)
		return err;

	err = ix_rm_get_phys_offset(
			(const void*) pData, &memType,&memChan,&offset2,&pAddr);
	if(err != IX_SUCCESS)
		return err;

	
	DBG_MSG("Meta data at 0x%x(offset 0x%x)  pkt data starts at 0x%x(offset 0x%x)\n",
					(unsigned int)pMeta, (unsigned int)offset1,
					(unsigned int)pData, (unsigned int)offset2);
	
	/*
	 * add pkt data processing here
	 */
       
	/*
	 * Now, produce messages to microengines
	 *
	 */
	if ((int)arg_pContext == INGRESS_HANDLER)
	{

		switch(arg_UserData)
		{
			case 0x11:
				DBG_MSG("\t... Options ...\n");break;
			case 0x12:
				DBG_MSG("\t... length mismatch ...\n");break;
			case 0x13:
				DBG_MSG("\t... bad ttl ...\n");break;
			case 0x14:
				DBG_MSG("\t... multicast ...\n");break;
			case 0x15:
				DBG_MSG("\t... frag required ...\n");break;
			case 0x16:
				DBG_MSG("\t... redirect ...\n");break;
			case 0x17:
				DBG_MSG("\t... down ...\n");break;
			case 0x18:
				DBG_MSG("\t... no route ...\n");break;
			case 0x19:
				DBG_MSG("\t... local delivery ...\n");break;
			case 0x1a:
				DBG_MSG("\t... limited broadcast ...\n");break;
			default	:
				DBG_MSG("\t... UNKNOWN exception 0x%x ...\n",(int)arg_UserData);break;
		}

		/*
		 * ingress processing
		 */
		DBG_MSG("%s : Dropping packet ...\n",me);
   		return ix_rm_buffer_free_chain(arg_hBuffer);
#if 0	
		/* code here is only to show how packet can be sent back to MEs. This is just
		an example code snippet */
		msg[0] = arg_hBuffer;
		msg[1] = eop_handle;	/* need to travel the buffer chain to find this */
		msg[2] = BID_QM;

  		size = 3;
   		err = ix_rm_hw_ring_put(ingressCoreToMeCtrlRing, &size, msg);
   		if(err != IX_SUCCESS)
   		{
   			DBG_MSG("%s : possible control ring full condition \n",me) ;
   			return ix_rm_buffer_free_chain(arg_hBuffer);
   		}
#endif
		
	}
	else
		if ((int)arg_pContext == EGRESS_HANDLER)
		{
			/*
			 * egress processing
			 */
		DBG_MSG("%s : Dropping packet ...\n",me);
   		return ix_rm_buffer_free_chain(arg_hBuffer);

		}

   	return IX_SUCCESS;
}


/*
 * Symbol lookup. If match is found, converts the virtual address
 * into memory offset
 */
unsigned int	_sym_to_offset(char *sym,int numEntries, t_app_mem_desc *p)
{
	const char me[]="_sym_to_offset";
	int cnt;
	ix_memory_type memType;
	ix_uint32	memChan;
	ix_uint32	offset;
	void		*pAddr;
	ix_error 	err;

	for(cnt=0;cnt<numEntries;cnt++)
	{
		if (!strcmp(p[cnt].name,sym))
		{

			err = ix_rm_get_phys_offset(
				(const void*) p[cnt].value,
				&memType,&memChan,&offset,&pAddr);
    		if(err != IX_SUCCESS)
			{
				DBG_MSG("SYM_TO_VAL problem Virt addr 0x%x for entry %d.\n",
					(unsigned int)p[cnt].value,cnt);
				return -1;
			}
			else
			{
#if 0
				DBG_MSG("symbol %s, value 0x%x.\n",
						p[cnt].name, (unsigned int)p[cnt].value);
#endif
				return ((offset & 0x3fffffff) | (memChan << 30));
			}
		}
	}

	DBG_MSG("%s : Symbol %s not found.\n",me,sym);

	return -1; /* meaning no match */
}

void*	_sym_to_virt(char *sym,int numEntries, t_app_mem_desc *p)
{
	int cnt;

	for(cnt=0;cnt<numEntries;cnt++)
		if (!strcmp(p[cnt].name,sym))
			return p[cnt].value;

	return (void *)0x0; /* meaning no match */
}

/*
 * function to print list of import variables per ME
 * from UOF file. To be called after the UOF file is set
 */
void	_list_import_vars(char *filename)
{
	const char me[]="_list_import_vars";
	int status;
	uclo_objHandle_T *objHandle;
	uof_Image_T         *image;
	uof_encapPage_T     *pages;
	int meNum;

	if (!filename)
	{
		DBG_MSG("ERROR : Specify file name.\n");
		return;
	}
	/*
	 * load the obj file
	 */
	status = UcLo_LoadObjFile((void **)&objHandle,filename);
	if(status != UCLO_SUCCESS)
	{
		DBG_MSG("%s : LoadObjFile %s FAILED with status 0x%x.\n",me,filename,status);
		return;
	}

	DBG_MSG("\n============== %s : START ===============\n\n",me);
	/*
	 * for each ME
	 */
	for(meNum=0;meNum<IX_MAX_MICROENGINE_NUMBER;meNum++)
	{
		int p;

		image = objHandle->encapMeImage[meNum].imagePtr;
		pages = objHandle->encapMeImage[meNum].pages;

		DBG_MSG("ME NUMBER = %d\n",meNum);

		if(!image || !pages)
		{
			DBG_MSG("\t=== NO image loaded ===\n");
			continue;
		}

		/* traverse the pages */
		for(p = 0; p < image->numOfPage; p++)
		{
			unsigned int v;

			/* update all instance of the variable */
			for(v = 0; v < pages[p].numImpVar; v++)
			{ 
				DBG_MSG("\t%s\n",
					UcLo_getString(&objHandle->strTable,pages[p].impVar[v].name));
			}/* for */
		} /* for pages */
	}/* for ME */

	/*
	 * delete the object
	 */
	UcLo_DeleObj(objHandle);

	DBG_MSG("\n============== %s : DONE ===============\n\n",me);
	return;
}

	
/*
 * Ingress Xscale application
 * ==========================
 */

ix_error rm_app_init_ingress(void)
{
	ix_error err = IX_SUCCESS;

    const char me[] = "rm_app_init_ingress";


	/*
	 * commnunication between blocks
	 */
    ix_hw_ring_handle pktrxToIpv4Ring = 0;
    ix_hw_ring_handle ipv4ToQmRing    = 0;
    ix_hw_ring_handle schedToQmRing   = 0;
    ix_hw_ring_handle qmToCsistxRing  = 0;


	/*
	 * misc variables
	 */
#define	_INGRESS_NUM_SYMBOLS		8
    ix_imported_symbol aImportSymbols[_INGRESS_NUM_SYMBOLS];

    ix_uint32 dl_rel_base = 0;
    ix_uint32 i;

    ix_uint32 temp_meta_address = 0;
    ix_uint32 temp_data_address = 0;

	/*
	 * for memory conversion calls
	 */
	ix_memory_type memType;
	ix_uint32	memChan;
	ix_uint32	offset;
	void		*pAddr;
	unsigned int *pTmp;

	/**********************************************/

	DBG_MSG("%s : CALLED\n",me);

	DBG_MSG("%s : Setting packet receive mode ...\n",me);
    err = ix_rm_packet_set_receive_mode(IX_CC_IPV4_PKT_MICROBLOCK_ID, IX_COMM_ID_MODE_CALLBACK);
    if(err != IX_SUCCESS)
        return err;

	DBG_MSG("%s : Setting message receive mode ...\n",me);
    err = ix_rm_message_set_receive_mode(IX_CC_IPV4_PKT_MICROBLOCK_ID, IX_COMM_ID_MODE_CALLBACK);
    if(err != IX_SUCCESS)
        return err;

	DBG_MSG("%s : Registering packet handler ...\n",me);
    err = ix_rm_packet_handler_register(IX_CC_IPV4_PKT_MICROBLOCK_ID, xscale_me_pkt_handler, 
			(void*)INGRESS_HANDLER);
    if(err != IX_SUCCESS)
        goto error;

	DBG_MSG("%s : Registering message handler ...\n",me);
   	err = ix_rm_message_handler_register(IX_CC_IPV4_PKT_MICROBLOCK_ID, xscale_me_pkt_handler, 
			(void*)INGRESS_HANDLER);
    if(err != IX_SUCCESS)
        goto error;

	/*
	 * Create Xscale to ME scratch rings
	 */
    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_128, 
			IX_ME_COMM_CONTROL_RING_ID, &ingressCoreToMeCtrlRing);
    if(err != IX_SUCCESS)
         goto error;
	DBG_MSG("%s : Created Xscale to ME control ring 0x%.8x\n",me,(unsigned int)ingressCoreToMeCtrlRing);

    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_256, 
			IX_ME_COMM_DATA_RING_ID, &ingressCoreToMeDataRing);
    if(err != IX_SUCCESS)
        goto error;
	DBG_MSG("%s : Created Xscale to ME data ring 0x%.8x \n",me,(unsigned int)ingressCoreToMeDataRing);

	/*
	 * Create message scratch rings
	 */
    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_1K, 
			IX_PKTRX_TO_IPV4_COMM_RING_ID, &pktrxToIpv4Ring) ;
    if(err != IX_SUCCESS)
        goto error;
	DBG_MSG("%s : Creating Pkt Rx to IPV4 ring 0x%.8x\n",me,(unsigned int)pktrxToIpv4Ring);

    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_512, 
			IX_IPV4_TO_QM_COMM_RING_ID, &ipv4ToQmRing) ;
    if(err != IX_SUCCESS)
        goto error;
	DBG_MSG("%s : Created IPV4 to QM ring 0x%.8x\n",me,(unsigned int)ipv4ToQmRing);

    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_512, 
			IX_SCHED_TO_QM_COMM_RING_ID, &schedToQmRing) ;
    if(err != IX_SUCCESS)
        goto error;
	DBG_MSG("%s : Created Sched to QM ring 0x%.8x\n",me,(unsigned int)schedToQmRing);

    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_512, 
			IX_QM_TO_CSIXTX_COMM_RING_ID, &qmToCsistxRing) ;
    if(err != IX_SUCCESS)
        goto error;
	DBG_MSG("%s : Created QM to CSIX Tx ring 0x%.8x\n",me,(unsigned int)qmToCsistxRing);


   /* 
	* Create  a free list 
	*/
	DBG_MSG("%s : Creating buffer free list...\n",me);
    err = ix_rm_hw_buffer_free_list_create(INGRESS_NUM_BUFFERS, 
		sizeof(ix_hw_buffer_meta), INGRESS_BUF_SIZE, 
		INGRESS_BUF_SRAM_CHAN, INGRESS_BUF_DRAM_CHAN, 
        &ingressHwFreeList) ;
    if( err != IX_SUCCESS)
		return err ;

    /* 
	 * get information on the free list to get the values of variables to 
	 * be patched 
	 */
	DBG_MSG("%s : Reading buffer free list info...\n",me);
    ix_rm_buffer_free_list_get_info(
			ingressHwFreeList, &ingressHwFreeListInfo) ;

	/*
	 * Create the memory segments
	 */
	for(i=0;i<INGRESS_MEM_NUM_ENTRIES;i++)
	{
		err = ix_rm_mem_alloc(ingress_mem[i].memType,
				ingress_mem[i].memChan,
				ingress_mem[i].size,
				&(ingress_mem[i].value));
		if (err != IX_SUCCESS)
			goto error;
		if (ingress_mem[i].pVar)
				*(ingress_mem[i].pVar) = (unsigned char*)ingress_mem[i].value;
		DBG_MSG("Allocated %d bytes at 0x%x using RM for %s.\n",
						(int)ingress_mem[i].size,(unsigned int)ingress_mem[i].value,
						ingress_mem[i].name);
	}

	/*
	 * Simply print all symbols
	 */
	/*_list_import_vars(s_IngressUOFFile);*/

	/*
	 * Now we are ready to download the microcode
	 */
	DBG_MSG("%s : Setting microcode file ...\n",me); 
    err =  ix_rm_ueng_set_ucode(s_IngressUOFFile);
    if(err != IX_SUCCESS)
        goto error;

    /* 
	 * Set the variables to be patched 
	 *
	 */
 
   	DBG_MSG("%s : Patching microcode symbols ...\n",me); 
    aImportSymbols[0].m_Name  = "BUF_SRAM_BASE" ;
	err = ix_rm_get_phys_offset(
			(const void*) ingressHwFreeListInfo.m_pMetaBaseAddress,
			&memType,&memChan,&offset,&pAddr);
    if(err != IX_SUCCESS)
	{
		DBG_MSG("%s : Can't convert Virt addr 0x%x.\n",
			me,(unsigned int)ingressHwFreeListInfo.m_pMetaBaseAddress);
        goto error;
	}
	else
	{
		DBG_MSG("%s : SRAM Meta base offset is 0x%x, Phy addr is 0x%x\n",
				me,(unsigned int)offset, (unsigned int)(*(unsigned int**)pAddr));
	}
    aImportSymbols[0].m_Value = ( ix_uint32) (offset & 0x3fffffff);
	_PRINT_SYMBOL(0);
	INGRESS_BUF_SRAM_BASE = (unsigned char*)ingressHwFreeListInfo.m_pMetaBaseAddress;


	temp_meta_address = (ix_uint32) (offset & 0x3fffffff);
	err = ix_rm_get_phys_offset(
			(const void*) ingressHwFreeListInfo.m_pDataBaseAddress,
			&memType,&memChan,&offset,&pAddr);
	if(err != IX_SUCCESS)
		goto error;
    temp_data_address = ( ix_uint32) (offset & 0xffffffc0);
	BUF_SDRAM_BASE = (unsigned char*)ingressHwFreeListInfo.m_pDataBaseAddress;

    dl_rel_base = temp_data_address - 
		( temp_meta_address * 
           (ingressHwFreeListInfo.m_DataElementSize / 
			ingressHwFreeListInfo.m_MetaElementSize)) ;

    aImportSymbols[1].m_Value =  ( ix_uint32) ingressHwFreeListInfo.m_FreeListInfo;
    aImportSymbols[1].m_Name  = "BUF_FREE_LIST0" ;
	_PRINT_SYMBOL(1);

	/* patch on all MEs, except on 4 */
    for(i = 0; i < IX_MAX_MICROENGINE_NUMBER; i++)
    {
        if( i == 4) 
           continue ;
        err = ix_rm_ueng_patch_symbols(
				g_aMicroengineNumberMapping[i],2,aImportSymbols);
        if(err != IX_SUCCESS)
		{
			DBG_MSG("%s : Symbol BUF_SRAM_BASE/BUF_FREE_LIST0 failed for ME %d.\n",me,(int)i);
            goto error;
		}
    }

	/* patch on all, except 3 and 4 */
    aImportSymbols[0].m_Value =  dl_rel_base ;
    aImportSymbols[0].m_Name  = "DL_REL_BASE" ;
	_PRINT_SYMBOL(0);
    for(i = 0; i < IX_MAX_MICROENGINE_NUMBER; i++)
    {
        if( (i==3) || (i == 4)) 
           continue ;
        err = ix_rm_ueng_patch_symbols(
				g_aMicroengineNumberMapping[i],1,aImportSymbols);
        if(err != IX_SUCCESS)
            goto error;
    }

    /* 
	 * Patching for POS Rx 
	 */
     
    aImportSymbols[0].m_Value =  ( ix_uint32) ingressHwFreeListInfo.m_FreeListInfo1;
    aImportSymbols[0].m_Name  = "FREE_LIST_ID" ;
	_PRINT_SYMBOL(0);

    err = ix_rm_ueng_patch_symbols(0,1,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;


    /* 
	 * Patching for QM 
	 */

    aImportSymbols[0].m_Value = 
		_sym_to_offset("QD_SRAM_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[0].m_Value == -1)
		goto error;
    aImportSymbols[0].m_Name  = "QD_SRAM_BASE" ;
	_PRINT_SYMBOL(0);

    aImportSymbols[1].m_Value = 0x400;
    aImportSymbols[1].m_Name  = "QD_TOTAL" ;
	_PRINT_SYMBOL(1);

    aImportSymbols[2].m_Value = 20;
    aImportSymbols[2].m_Name  = "QM_DROP_QUEUE_ENTRY" ;
	_PRINT_SYMBOL(2);

     err = ix_rm_ueng_patch_symbols(3,3,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;

    /* 
	 * Patching for scheduler 
	 */

	pTmp = (unsigned int*)
		_sym_to_virt("SCHED_VOQ_CREDIT_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
	if (pTmp == NULL)
	{
		DBG_MSG("%s : Can't convirt SCHED_VOQ_CREDIT_BASE to virt address.\n",me);
		goto error;
	}
	*pTmp = 0x1;
	
    aImportSymbols[0].m_Value = 
		_sym_to_offset("SCHED_VOQ_CREDIT_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[0].m_Value == -1)
		goto error;
    aImportSymbols[0].m_Name =  "SCHED_VOQ_CREDIT_BASE" ;
	_PRINT_SYMBOL(0);

    err = ix_rm_ueng_patch_symbols(0x10,1,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;

    /* 
	 * Patching for CSIX TX 
	 */

    aImportSymbols[0].m_Value = 
		_sym_to_offset("CSIX_TX_CTX_SRAM_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[0].m_Value == -1)
		goto error;
    aImportSymbols[0].m_Name  = "CSIX_TX_CTX_SRAM_BASE" ;
	_PRINT_SYMBOL(0);

    aImportSymbols[1].m_Value = 
		_sym_to_offset("CSIX_TX_COUNTERS_SRAM_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[1].m_Value == -1)
		goto error;
    aImportSymbols[1].m_Name  = "CSIX_TX_COUNTERS_SRAM_BASE" ;
	_PRINT_SYMBOL(1);

    aImportSymbols[2].m_Value = THIS_BLADE_ID;
    aImportSymbols[2].m_Name  = "THIS_BLADE_ID" ;
	_PRINT_SYMBOL(2);

    aImportSymbols[3].m_Value = 
		_sym_to_offset("SRAM_ZERO_BLOCK",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[3].m_Value == -1)
		goto error;
    aImportSymbols[3].m_Name  = "SRAM_ZERO_BLOCK" ;
	_PRINT_SYMBOL(3);

    err = ix_rm_ueng_patch_symbols(0x13,4,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;

    /* 
	 * Patching for IPV4 
	 */

    aImportSymbols[0].m_Value = 
		_sym_to_offset("TRIE_TABLE_SRAM_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[0].m_Value == -1)
		goto error;
    aImportSymbols[0].m_Name  = "TRIE_TABLE_SRAM_BASE";
	_PRINT_SYMBOL(0);

#ifdef IPV4_COUNTERS_SCRATCH
    aImportSymbols[1].m_Value = 
		_sym_to_offset("IPV4_STATS_TABLE_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[1].m_Value == -1)
		goto error;
    aImportSymbols[1].m_Name  = "IPV4_STATS_TABLE_BASE";
	_PRINT_SYMBOL(1);
#else
    aImportSymbols[1].m_Value = 
		_sym_to_offset("STATS_TABLE_SRAM_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[1].m_Value == -1)
		goto error;
    aImportSymbols[1].m_Name  = "STATS_TABLE_SRAM_BASE";
	_PRINT_SYMBOL(1);
#endif
   
    aImportSymbols[2].m_Value = 
		_sym_to_offset("DBCAST_TABLE_SRAM_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[2].m_Value == -1)
		goto error;
    aImportSymbols[2].m_Name  = "DBCAST_TABLE_SRAM_BASE";
	_PRINT_SYMBOL(2);

    aImportSymbols[3].m_Value = 
		_sym_to_offset("CONTROL_BLOCK_SRAM_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[3].m_Value == -1)
		goto error;
    aImportSymbols[3].m_Name  = "CONTROL_BLOCK_SRAM_BASE";
	_PRINT_SYMBOL(3);
	/*
	 * initialize control block
	 */
	pTmp = (unsigned int*)
		_sym_to_virt("CONTROL_BLOCK_SRAM_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
	if (pTmp == NULL)
	{
		DBG_MSG("%s : Can't convirt CONTROL_BLOCK_SRAM_BASE to virt address.\n",me);
		goto error;
	}
	*(unsigned int*)pTmp = 0xffffffff;

    aImportSymbols[4].m_Value = THIS_BLADE_ID;
    aImportSymbols[4].m_Name  = "THIS_BLADE_ID";
	_PRINT_SYMBOL(4);

    aImportSymbols[5].m_Value = 
		_sym_to_offset("NEXTHOP_TABLE_SRAM_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[5].m_Value == -1)
		goto error;
    aImportSymbols[5].m_Name  = "NEXTHOP_TABLE_SRAM_BASE";
	_PRINT_SYMBOL(5);

#if 0
    aImportSymbols[6].m_Value = 
		_sym_to_offset("SRAM_FLT_TABLE_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[6].m_Value == -1)
		goto error;
    aImportSymbols[6].m_Name  = "SRAM_FLT_TABLE_BASE" ;
	_PRINT_SYMBOL(6);

    aImportSymbols[7].m_Value = 
		_sym_to_offset("SRAM_PORT_BIT_MAP_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[7].m_Value == -1)
		goto error;
    aImportSymbols[7].m_Name  = "SRAM_PORT_BIT_MAP_BASE" ;
	_PRINT_SYMBOL(7);
#endif

    err = ix_rm_ueng_patch_symbols(0x1,6,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;

    err = ix_rm_ueng_patch_symbols(0x2,6,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;
 
    err = ix_rm_ueng_patch_symbols(0x11,6,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;

    err = ix_rm_ueng_patch_symbols(0x12,6,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;


	return err;

	/*
	 * error processing
	 */
error:

	DBG_MSG("%s : ERROR cleaning up mess for INGRESS...\n",me);

    return err;

}

ix_error rm_app_start_ingress_me(void)
{
	/*
	 * starts ingress MEs
	 */
	ix_error err;
	const char me[]="rm_app_start_ingress_me";
    /* start running the microcode */

    DBG_MSG("%s : Loading the microcode... \n",me) ;
    err = ix_rm_ueng_load();
    if(err != IX_SUCCESS)
        goto error;

#ifdef	__vxworks
	taskDelay(2*sysClkRateGet());
#endif

	
	if (!start_micro_engines)
	{
		DBG_MSG("%s: Microengines NOT STARTED as per user selection\n");
		return err;
	}

    DBG_MSG("%s : Starting the microengine 0x%x... \n",me,0x13) ;
    err = ix_rm_ueng_start(0x13,0xff);
    if(err != IX_SUCCESS)
        goto error;

    DBG_MSG("%s : Starting the microengine 0x%x... \n",me,0x12) ;
    err = ix_rm_ueng_start(0x12,0xff);
    if(err != IX_SUCCESS)
        goto error;

    DBG_MSG("%s : Starting the microengine 0x%x... \n",me,0x11) ;
    err = ix_rm_ueng_start(0x11,0xff);
    if(err != IX_SUCCESS)
        goto error;

    DBG_MSG("%s : Starting the microengine 0x%x... \n",me,0x10) ;
    err = ix_rm_ueng_start(0x10,0xff);
    if(err != IX_SUCCESS)
        goto error;

    DBG_MSG("%s : Starting the microengine 0x%x... \n",me,0x3) ;
    err = ix_rm_ueng_start(3,0xff);
    if(err != IX_SUCCESS)
        goto error;

    DBG_MSG("%s : Starting the microengine 0x%x... \n",me,0x2) ;
    err = ix_rm_ueng_start(2,0xff);
    if(err != IX_SUCCESS)
        goto error;

    DBG_MSG("%s : Starting the microengine 0x%x... \n",me,0x1) ;
    err = ix_rm_ueng_start(1,0xff);
    if(err != IX_SUCCESS)
        goto error;

    DBG_MSG("%s : Starting the microengine 0x%x... \n",me,0x0) ;
    err = ix_rm_ueng_start(0,0xff);
    if(err != IX_SUCCESS)
        goto error;

    DBG_MSG("%s : MICROENGINES are RUNNING !!!\n",me) ;

error:
	return err;

}

/*
 * Egress application initialization code
 */
ix_error rm_app_init_egress(void)
{
	ix_error err = IX_SUCCESS;

    const char me[] = "rm_app_init_egress";

	/*
	 * scratch rings for message comminication
	 */
    ix_hw_ring_handle csixRxToQMRing = 0;
    ix_hw_ring_handle schedToQmRing  = 0;
    ix_hw_ring_handle qmToPosTxRing_0  = 0;
    ix_hw_ring_handle qmToPosTxRing_1  = 0;
    ix_hw_ring_handle qmToPosTxRing_2  = 0;
    ix_hw_ring_handle qmToPosTxRing_3  = 0;
    ix_hw_ring_handle l2ToQMTxRing   = 0;

	/*
	 * misc variables
	 */
#define	_EGRESS_NUM_SYMBOLS		7
    ix_imported_symbol aImportSymbols[_EGRESS_NUM_SYMBOLS];

    ix_uint32 dl_rel_base = 0;
    ix_uint32 i;

    ix_uint32 temp_meta_address = 0;
    ix_uint32 temp_data_address = 0;

	/*
	 * for memory conversion calls
	 */
	ix_memory_type memType;
	ix_uint32	memChan;
	ix_uint32	offset;
	void		*pAddr;
	unsigned int *pTmp;

	/**********************************************/

	DBG_MSG("%s : CALLED\n",me);

	DBG_MSG("%s : Setting packet receive mode ...\n",me);
    err = ix_rm_packet_set_receive_mode(IX_CC_ETH_TX_PKT_ID, IX_COMM_ID_MODE_CALLBACK);
    if(err != IX_SUCCESS)
        return err;

	DBG_MSG("%s : Setting message receive mode ...\n",me);
    err = ix_rm_message_set_receive_mode(IX_CC_ETH_TX_PKT_ID, IX_COMM_ID_MODE_CALLBACK);
    if(err != IX_SUCCESS)
        return err;


	DBG_MSG("%s : Registering packet handler ...\n",me);
    err = ix_rm_packet_handler_register(IX_CC_ETH_TX_PKT_ID, 
			xscale_me_pkt_handler, (void*)EGRESS_HANDLER);
    if(err != IX_SUCCESS)
        goto error;

	DBG_MSG("%s : Registering message handler ...\n",me);
    err = ix_rm_message_handler_register(IX_CC_ETH_TX_PKT_ID, 
			xscale_me_pkt_handler, (void*)EGRESS_HANDLER);
    if(err != IX_SUCCESS)
        goto error;


	/*
	 * Create Xscale to ME scratch rings
	 */
    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_128, 
			IX_ME_COMM_CONTROL_RING_ID, &egressCoreToMeCtrlRing);
    if(err != IX_SUCCESS)
         goto error;
	DBG_MSG("%s : Created Xscale to ME control ring 0x%.8x\n",me,(unsigned int)egressCoreToMeCtrlRing);

    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_256, 
			IX_ME_COMM_DATA_RING_ID, &egressCoreToMeDataRing);
    if(err != IX_SUCCESS)
        goto error;
	DBG_MSG("%s : Created Xscale to ME data ring 0x%.8x\n",me,(unsigned int)egressCoreToMeDataRing);

	/*
	 * Create message scratch rings
	 */
    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_512, 
			IX_CSIXRX_TO_QM_COMM_RING_ID, &csixRxToQMRing);
    if(err != IX_SUCCESS)
         goto error;
	DBG_MSG("%s : Creating CSIX RX to L2encap ring 0x%.8x\n",me,(unsigned int)csixRxToQMRing);

    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_512, 
			IX_L2_ENCAP_TO_QM_COMM_RING_ID, &l2ToQMTxRing);
    if(err != IX_SUCCESS)
         goto error;
 	DBG_MSG("%s : Creating L2encap to QM ring 0x%.8x\n",me,(unsigned int)l2ToQMTxRing);
 
    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_512, 
			IX_SCHED_TO_QM_EGRESS_COMM_RING_ID, &schedToQmRing);
    if(err != IX_SUCCESS)
         goto error;
	DBG_MSG("%s : Creating Sched to QM ring 0x%.8x\n",me,(unsigned int)schedToQmRing);

    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_256, 
			IX_QM_TO_POSTX_COMM_RING_ID_0, &qmToPosTxRing_0);
    if(err != IX_SUCCESS)
         goto error;
	DBG_MSG("%s : Created QM to Packet Tx ring (first) 0x%.8x\n",me,(unsigned int)qmToPosTxRing_0);

    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_256, 
			IX_QM_TO_POSTX_COMM_RING_ID_1, &qmToPosTxRing_1);
    if(err != IX_SUCCESS)
         goto error;
	DBG_MSG("%s : Created QM to Packet Tx ring (second) 0x%.8x\n",me,(unsigned int)qmToPosTxRing_1);

    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_256, 
			IX_QM_TO_POSTX_COMM_RING_ID_2, &qmToPosTxRing_2);
    if(err != IX_SUCCESS)
         goto error;
	DBG_MSG("%s : Created QM to Packet Tx ring (third) 0x%.8x\n",me,(unsigned int)qmToPosTxRing_2);

    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_256, 
			IX_QM_TO_POSTX_COMM_RING_ID_3, &qmToPosTxRing_3);
    if(err != IX_SUCCESS)
         goto error;
	DBG_MSG("%s : Created QM to Packet Tx ring (fourth) 0x%.8x\n",me,(unsigned int)qmToPosTxRing_3);


   /* 
	* Create  a free list 
	*/
	DBG_MSG("%s : Creating buffer free list...\n",me);
    err = ix_rm_hw_buffer_free_list_create(EGRESS_NUM_BUFFERS, 
		sizeof(ix_hw_buffer_meta), EGRESS_BUF_SIZE, 
		EGRESS_BUF_SRAM_CHAN, EGRESS_BUF_DRAM_CHAN, 
        &egressHwFreeList) ;
    if( err != IX_SUCCESS)
		return err ;

    /* 
	 * get information on the free list to get the values of variables to 
	 * be patched 
	 */
	DBG_MSG("%s : Reading buffer free list info...\n",me);
    ix_rm_buffer_free_list_get_info(
			egressHwFreeList, &egressHwFreeListInfo) ;

	/*
	 * Create the memory segments
	 */
	for(i=0;i<EGRESS_MEM_NUM_ENTRIES;i++)
	{
		err = ix_rm_mem_alloc(egress_mem[i].memType,
				egress_mem[i].memChan,
				egress_mem[i].size,
				&(egress_mem[i].value));
		if (err != IX_SUCCESS)
			goto error;
		if (egress_mem[i].pVar)
				*(egress_mem[i].pVar) = (unsigned char*)egress_mem[i].value;
		DBG_MSG("Allocated %d bytes at 0x%x using RM for %s.\n",
						(int)egress_mem[i].size,(unsigned int)egress_mem[i].value,
						egress_mem[i].name);
	}

	/*
	 * Simply print all symbols
	 */
	/*_list_import_vars(s_EgressUOFFile);*/

	/*
	 * Now we are ready to download the microcode
	 */
	DBG_MSG("%s : Setting microcode file ...\n",me);
    err =  ix_rm_ueng_set_ucode(s_EgressUOFFile);
    if(err != IX_SUCCESS)
        goto error;

    /* 
	 * Set the variables to be patched 
	 *
	 */
	DBG_MSG("%s : Patching microcode symbols ...\n",me);
    aImportSymbols[0].m_Name  = "BUF_SRAM_BASE" ;
	err = ix_rm_get_phys_offset(
			(const void*) egressHwFreeListInfo.m_pMetaBaseAddress,
			&memType,&memChan,&offset,&pAddr);
    if(err != IX_SUCCESS)
	{
		DBG_MSG("%s : Can't convert Virt addr 0x%x.\n",
			me,(unsigned int)egressHwFreeListInfo.m_pMetaBaseAddress);
        goto error;
	}
	else
	{
		DBG_MSG("%s : SRAM Meta base offset is 0x%x, Phy addr is 0x%x\n",
				me,(unsigned int)offset, (unsigned int)(*(unsigned int**)pAddr));
	}

    aImportSymbols[0].m_Value = ( ix_uint32) (offset & 0x3fffffff);
	_PRINT_SYMBOL(0);
	EGRESS_BUF_SRAM_BASE = (unsigned char*)egressHwFreeListInfo.m_pMetaBaseAddress;

	temp_meta_address = (ix_uint32) (offset & 0x3fffffff);
	err = ix_rm_get_phys_offset(
			(const void*) egressHwFreeListInfo.m_pDataBaseAddress,
			&memType,&memChan,&offset,&pAddr);
	if(err != IX_SUCCESS)
		goto error;
    temp_data_address = ( ix_uint32) (offset & 0xffffffc0);
	BUF_SDRAM_BASE = (unsigned char*)egressHwFreeListInfo.m_pDataBaseAddress;

    dl_rel_base = temp_data_address - 
		( temp_meta_address * 
           (egressHwFreeListInfo.m_DataElementSize / 
			egressHwFreeListInfo.m_MetaElementSize)) ;

    aImportSymbols[1].m_Value =  ( ix_uint32) egressHwFreeListInfo.m_FreeListInfo;
    aImportSymbols[1].m_Name  = "BUF_FREE_LIST0" ;
	_PRINT_SYMBOL(1);

	/*
	 * patch on 0,1,4 */
    for(i = 0; i < IX_MAX_MICROENGINE_NUMBER; i++)
    {
        if( (i==0) || (i == 1) || (i==4)) 
		{
        	err = ix_rm_ueng_patch_symbols(
				g_aMicroengineNumberMapping[i],2,aImportSymbols);
        	if(err != IX_SUCCESS)
			{
				DBG_MSG("%s : Symbol BUF_SRAM_BASE/BUF_FREE_LIST0 failed for ME %d.\n",me,(int)i);
            	goto error;
			}
		}
		else
           continue ;
    }

    aImportSymbols[0].m_Value =  dl_rel_base ;
    aImportSymbols[0].m_Name  = "DL_REL_BASE" ;
	_PRINT_SYMBOL(0);
	/*
	 * patch on  0,5,6 */
    for(i = 0; i < IX_MAX_MICROENGINE_NUMBER; i++)
    {
        if( (i==0) || (i == 4) || (i==5)) 
		{
        	err = ix_rm_ueng_patch_symbols(
				g_aMicroengineNumberMapping[i],1,aImportSymbols);
        	if(err != IX_SUCCESS)
            	goto error;
		}
    }


    /* 
	 * Patching for CSIX Rx 
	 */
               
    aImportSymbols[0].m_Value = 
		_sym_to_offset("CSIX_RX_CTX_SRAM_BASE",EGRESS_MEM_NUM_ENTRIES,
				egress_mem);
    if(aImportSymbols[0].m_Value == -1)
		goto error;
    aImportSymbols[0].m_Name  = "CSIX_RX_CTX_SRAM_BASE" ;
	_PRINT_SYMBOL(0);

    aImportSymbols[1].m_Value =  ( ix_uint32) egressHwFreeListInfo.m_FreeListInfo1;
    aImportSymbols[1].m_Name  = "FREE_LIST_ID" ;
	_PRINT_SYMBOL(1);

    err = ix_rm_ueng_patch_symbols(0,2,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;


    /* 
	 * Patching for Egress QM 
	 */

    aImportSymbols[0].m_Value = 20;
    aImportSymbols[0].m_Name  = "QM_DROP_QUEUE_ENTRY" ;
	_PRINT_SYMBOL(0);

    aImportSymbols[1].m_Value = 
		_sym_to_offset("QD_SRAM_BASE",EGRESS_MEM_NUM_ENTRIES,
				egress_mem);
    if(aImportSymbols[1].m_Value == -1)
		goto error;
    aImportSymbols[1].m_Name  = "QD_SRAM_BASE" ;
	_PRINT_SYMBOL(1);

    aImportSymbols[2].m_Value = 1024;
    aImportSymbols[2].m_Name  = "QD_TOTAL" ;
	_PRINT_SYMBOL(2);
    
    err = ix_rm_ueng_patch_symbols(1,3,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;

   /* 
	* Patching for Egress Scheduler 
	*/

	pTmp = (unsigned int*)
		_sym_to_virt("SCHED_WEIGHT_CREDIT_BASE",EGRESS_MEM_NUM_ENTRIES,
				egress_mem);

	/* Assign credit to all Ports (16 ports) */
	for(i=0;i<16;i++)
	{
		pTmp[i] = 0x1;
	}

	/* Assign credit to all queues (16 ports * 16 queues) */
	for(i=0;i<16*16;i++)
	{	
		/* Following credit for queue is taken from the Egress Scheduler Microblock
		 * MIN_CREDIT_INCREMENT  ((MTU * 8) / (1 << BITS_FOR_PACKET_LENGTH)) */
		pTmp[16+i] =  (((9*1024) * 8) / (1 << 7));

	}
   
    aImportSymbols[0].m_Value = 
		_sym_to_offset("SCHED_WEIGHT_CREDIT_BASE",EGRESS_MEM_NUM_ENTRIES,
				egress_mem);
    if(aImportSymbols[0].m_Value == -1)
		goto error;
    aImportSymbols[0].m_Name  = "SCHED_WEIGHT_CREDIT_BASE" ;

    aImportSymbols[1].m_Name  = "SRAM_ZERO_BLOCK" ;
    aImportSymbols[1].m_Value = 
		_sym_to_offset("SRAM_ZERO_BLOCK",EGRESS_MEM_NUM_ENTRIES,
				egress_mem);
    if(aImportSymbols[1].m_Value == -1)
		goto error;

    err = ix_rm_ueng_patch_symbols(2,2,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;

	return err;
	/*
	 * to reset microengine, use
	 * ix_rm_ueng_reset(me_clus_num, 1)
	 */

	/*
	 * error processing
	 *
	 */
error:

	DBG_MSG("%s : ERROR cleaning up mess for EGRESS...\n",me);


    return err ;

}

ix_error rm_app_start_egress_me(void)
{
	/*
	 * start the microengines
	 */
	ix_error err;
	const char me[]="rm_app_start_egress_me";

	DBG_MSG("%s : Loading microcode...\n",me);
    err = ix_rm_ueng_load();
    if(err != IX_SUCCESS)
        goto error;

#ifdef	__vxworks
	taskDelay(2*sysClkRateGet());
#endif


	if (!start_micro_engines)
	{
		DBG_MSG("%s: Microengines NOT STARTED as per user selection\n");
		return err;
	}

	DBG_MSG("%s : Starting microengine 0x%x...\n",me,0x11);
    err = ix_rm_ueng_start(0x11,0xff);
    if(err != IX_SUCCESS)
        goto error;

	DBG_MSG("%s : Starting microengine 0x%x...\n",me,0x10);
    err = ix_rm_ueng_start(0x10,0xff);
    if(err != IX_SUCCESS)
        goto error;

	DBG_MSG("%s : Starting microengine 0x%x...\n",me,0x2);
    err = ix_rm_ueng_start(0x2,0xff);
    if(err != IX_SUCCESS)
        goto error;

	DBG_MSG("%s : Starting microengine 0x%x...\n",me,0x1);
    err = ix_rm_ueng_start(0x1,0xff);
    if(err != IX_SUCCESS)
        goto error;

	DBG_MSG("%s : Starting microengine 0x%x...\n",me,0x0);
    err = ix_rm_ueng_start(0x0,0xff);
    if(err != IX_SUCCESS)
        goto error;

	DBG_MSG("%s : MICROENGINES are RUNNING !!!\n",me);

error:
	return err;
}

void	xscale_me_buffer_stats(char *chip)
{
	/* we have more buffers on Egress */
	ix_buffer_handle	a_Buf[EGRESS_NUM_BUFFERS], tmp;	
	ix_error	err;
	ix_buffer_handle low=0,high=0;
	int cnt;
	int num_buffers;
	ix_buffer_free_list_handle	hwFreeList;


	DBG_MSG("CAUTION : This function has to be called only after starting microengines\n");

	if(INGRESS(chip))
	{
		num_buffers = INGRESS_NUM_BUFFERS;
		hwFreeList = ingressHwFreeList;
	}
	else if(EGRESS(chip))
	{
		num_buffers = EGRESS_NUM_BUFFERS;
		hwFreeList = egressHwFreeList;
	}
	else
	{
		DBG_MSG("\nSpecify ingress/egress.\n\n");
		return;
	}

	/*
	 * first allocate all buffers and count
	 */
	for(cnt=0;cnt<num_buffers;cnt++)
	{
		err = ix_rm_buffer_alloc(hwFreeList,&tmp);
		if (err != IX_SUCCESS)
		{
			/* ix_error_dump(stderr,err);*/
			break;
		}
		else
		{
			a_Buf[cnt] = tmp;
			if (cnt==0)
			{
				low = tmp;high=tmp;
			}
			else
			{
				if (tmp > high)
					high = tmp;
				else if (tmp < low)
					low = tmp;
			}
		}
	}

	DBG_MSG("\nNumber of free buffers in Q-array is %d. Range is from 0x%x to 0x%x\n",
					cnt,(unsigned int)low,(unsigned int)high);

	/*
	 * then free all buffers
	 */
	for(cnt=cnt-1;cnt>=0;cnt--)
	{
		err = ix_rm_buffer_free(a_Buf[cnt]);
		if (err != IX_SUCCESS)
		{
			DBG_MSG("FAILED to free buffer 0x%x.\n",(unsigned int)a_Buf[cnt]);
		}
	}

	return;
}


/*
 * XScale-ME Application Initialization 
 *
 * Function returns IX_SUCCESS if everything OK or
 * an ix_error error code.
 */
int xscale_me_rm_app_init(char *chip)
{
	const char me[]="xscale_me_rm_app_init";
    ix_error err = IX_SUCCESS;

	DBG_MSG("%s : called.\n",me);

	/*
	 * Reset all
	 */
	DBG_MSG("%s : Resetting all microengines ...\n",me);
	ix_rm_ueng_reset_all(); 

    /**
     * Add processor specific extra required initialization
     */

	if (INGRESS(chip))
	{
		/*
		 * Ingress initialization 
		 */
		err = rm_app_init_ingress();
		if (err != IX_SUCCESS)
		{
			DBG_MSG("%s : Ingress App Init FAILED!.\n",me);
			return err;
		}

	}
	else if(EGRESS(chip))
	{
		/*
		 * Egress initialization
		 */
		err = rm_app_init_egress();
		if (err != IX_SUCCESS)
		{
			DBG_MSG("Egress App Init FAILED!.\n");
			return err;
		}

	}
	DBG_MSG("%s : %s App Init SUCCESSFUL.\n",me,chip);

    /**
     * Add extra required finalization
     */

	return err;

}

/*
 * Xscale-ME application cleanup 
 */
int xscale_me_rm_app_fini(char *chip)
{
	ix_error err = IX_SUCCESS;

	/*
	 * terminate resource manager
	 */
	err = ix_rm_term();

	if (INGRESS(chip))
	{
		/*
		 * Ingess cleanup
		 */


	}
	else if (EGRESS(chip))
	{
		/*
		 * Egress cleanup
		 */
	}

	return err;
}


