/*===========================================================================
 * = COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 * ===========================================================================
 */
/* $Id: egress_internal_registry_data_linux.h,v 1.7 2003/10/01 17:46:07 ktseng Exp $ */

#if !defined(_INTERNAL_REGISTRY_DATA_LINUX_H__)
#define __INTERNAL_REGISTRY_DATA_LINUX_H__

#include "ix_rm.h"
#include "bindings.h"
#include "ix_cc_microengines_bindings.h"
#include "dl_system.h"
#include "hardware.h"

/**
 * The following array contains core components properties 
 * that will be populated into the system registry by 
 * system application during initialization time.
 *
 * Each field in the array entry is defined as follows:
 *  1. "property path" - specify property path
 *  2. "property name" - specify property name 
 *  3. property attribute - 0: no property value required
 *                          1: property value required
 *  4. property type - property type
 *  5. property value - in the format of number
 *  6. property value - in the format of string
 */
ix_sa_registry_property gSAPropList[] = \
   { 
    /* Configuration properties */
    {"", "Target_Config", 1, 2, 0, "oc192_ethernet_egress"}, 
    {"", "INGRESS_EGRESS", 1, 1, 1, ""}, 

    /* System properties */
    {"", "System_Properties", 0, 0, 0, ""}, 
    {"/System_Properties", "AUTO_RESTART", 1, 1, 0, ""}, 
    {"/System_Properties", "BLADE_ID", 1, 1, 2, ""}, 
    {"/System_Properties", "SWITCH_FABRIC_LOOPBACK_FLAG", 1, 1, 1, ""}, 
    {"/System_Properties", "SRAM_CHANNEL_NUM", 1, 1, 0, ""}, 

    /* Port 0 configuration */
    {"/System_Properties", "PORT00", 1, 1, 0, ""}, 
    {"/System_Properties/PORT00", "IP_VER", 1, 2, 0, "IPV4"}, 
    {"/System_Properties/PORT00/IP_VER", "IP_ADDR", 1, 2, 0, "10.0.0.1"}, 
    {"/System_Properties/PORT00/IP_VER", "IP_MASK", 1, 2, 0, "255.255.255.0"}, 
    {"/System_Properties/PORT00/IP_VER", "IP_BCAST", 1, 2, 0, "10.0.0.255"}, 
    {"/System_Properties/PORT00/IP_VER", "IP_GATE", 1, 2, 0, "10.0.0.254"}, 
    {"/System_Properties/PORT00", "MTU", 1, 1, 9216, ""}, 
    {"/System_Properties/PORT00", "LINK_SPEED", 1, 1, 1000, ""}, 
    {"/System_Properties/PORT00", "MAC_ADDR", 1, 2, 0, "0a0b0c0d0e00"}, 
    {"/System_Properties/PORT00", "MEDIA_TYPE", 1, 2, 0, "1GB_ETHERNET"}, 
    /* Port 1 configuration */
    {"/System_Properties", "PORT01", 1, 1, 1, ""}, 
    {"/System_Properties/PORT01", "IP_VER", 1, 2, 0, "IPV4"}, 
    {"/System_Properties/PORT01/IP_VER", "IP_ADDR", 1, 2, 0, "11.0.0.1"}, 
    {"/System_Properties/PORT01/IP_VER", "IP_MASK", 1, 2, 0, "255.255.255.0"}, 
    {"/System_Properties/PORT01/IP_VER", "IP_BCAST", 1, 2, 0, "11.0.0.255"}, 
    {"/System_Properties/PORT01/IP_VER", "IP_GATE", 1, 2, 0, "11.0.0.254"}, 
    {"/System_Properties/PORT01", "MTU", 1, 1, 9216, ""}, 
    {"/System_Properties/PORT01", "LINK_SPEED", 1, 1, 1000, ""}, 
    {"/System_Properties/PORT01", "MAC_ADDR", 1, 2, 0, "0a0b0c0d0e01"}, 
    {"/System_Properties/PORT01", "MEDIA_TYPE", 1, 2, 0, "1GB_ETHERNET"}, 
    /* Port 2 configuration */
    {"/System_Properties", "PORT02", 1, 1, 2, ""}, 
    {"/System_Properties/PORT02", "IP_VER", 1, 2, 0, "IPV4"}, 
    {"/System_Properties/PORT02/IP_VER", "IP_ADDR", 1, 2, 0, "12.0.0.1"}, 
    {"/System_Properties/PORT02/IP_VER", "IP_MASK", 1, 2, 0, "255.255.255.0"}, 
    {"/System_Properties/PORT02/IP_VER", "IP_BCAST", 1, 2, 0, "12.0.0.255"}, 
    {"/System_Properties/PORT02/IP_VER", "IP_GATE", 1, 2, 0, "12.0.0.254"}, 
    {"/System_Properties/PORT02", "MTU", 1, 1, 9216, ""}, 
    {"/System_Properties/PORT02", "LINK_SPEED", 1, 1, 1000, ""}, 
    {"/System_Properties/PORT02", "MAC_ADDR", 1, 2, 0, "0a0b0c0d0e02"}, 
    {"/System_Properties/PORT02", "MEDIA_TYPE", 1, 2, 0, "1GB_ETHERNET"}, 
    /* Port 3 configuration */
    {"/System_Properties", "PORT03", 1, 1, 3, ""}, 
    {"/System_Properties/PORT03", "IP_VER", 1, 2, 0, "IPV4"}, 
    {"/System_Properties/PORT03/IP_VER", "IP_ADDR", 1, 2, 0, "13.0.0.1"}, 
    {"/System_Properties/PORT03/IP_VER", "IP_MASK", 1, 2, 0, "255.255.255.0"}, 
    {"/System_Properties/PORT03/IP_VER", "IP_BCAST", 1, 2, 0, "13.0.0.255"}, 
    {"/System_Properties/PORT03/IP_VER", "IP_GATE", 1, 2, 0, "13.0.0.254"}, 
    {"/System_Properties/PORT03", "MTU", 1, 1, 9216, ""}, 
    {"/System_Properties/PORT03", "LINK_SPEED", 1, 1, 1000, ""}, 
    {"/System_Properties/PORT03", "MAC_ADDR", 1, 2, 0, "0a0b0c0d0e03"}, 
    {"/System_Properties/PORT03", "MEDIA_TYPE", 1, 2, 0, "1GB_ETHERNET"}, 
    /* Port 4 configuration */
    {"/System_Properties", "PORT04", 1, 1, 4, ""}, 
    {"/System_Properties/PORT04", "IP_VER", 1, 2, 0, "IPV4"}, 
    {"/System_Properties/PORT04/IP_VER", "IP_ADDR", 1, 2, 0, "14.0.0.1"}, 
    {"/System_Properties/PORT04/IP_VER", "IP_MASK", 1, 2, 0, "255.255.255.0"}, 
    {"/System_Properties/PORT04/IP_VER", "IP_BCAST", 1, 2, 0, "14.0.0.255"}, 
    {"/System_Properties/PORT04/IP_VER", "IP_GATE", 1, 2, 0, "14.0.0.254"}, 
    {"/System_Properties/PORT04", "MTU", 1, 1, 9216, ""}, 
    {"/System_Properties/PORT04", "LINK_SPEED", 1, 1, 1000, ""}, 
    {"/System_Properties/PORT04", "MAC_ADDR", 1, 2, 0, "0a0b0c0d0e04"}, 
    {"/System_Properties/PORT04", "MEDIA_TYPE", 1, 2, 0, "1GB_ETHERNET"}, 
    /* Port 5 configuration */
    {"/System_Properties", "PORT05", 1, 1, 5, ""}, 
    {"/System_Properties/PORT05", "IP_VER", 1, 2, 0, "IPV4"}, 
    {"/System_Properties/PORT05/IP_VER", "IP_ADDR", 1, 2, 0, "15.0.0.1"}, 
    {"/System_Properties/PORT05/IP_VER", "IP_MASK", 1, 2, 0, "255.255.255.0"}, 
    {"/System_Properties/PORT05/IP_VER", "IP_BCAST", 1, 2, 0, "15.0.0.255"}, 
    {"/System_Properties/PORT05/IP_VER", "IP_GATE", 1, 2, 0, "15.0.0.254"}, 
    {"/System_Properties/PORT05", "MTU", 1, 1, 9216, ""}, 
    {"/System_Properties/PORT05", "LINK_SPEED", 1, 1, 1000, ""}, 
    {"/System_Properties/PORT05", "MAC_ADDR", 1, 2, 0, "0a0b0c0d0e05"}, 
    {"/System_Properties/PORT05", "MEDIA_TYPE", 1, 2, 0, "1GB_ETHERNET"}, 
    /* Port 6 configuration */
    {"/System_Properties", "PORT06", 1, 1, 6, ""}, 
    {"/System_Properties/PORT06", "IP_VER", 1, 2, 0, "IPV4"}, 
    {"/System_Properties/PORT06/IP_VER", "IP_ADDR", 1, 2, 0, "16.0.0.1"}, 
    {"/System_Properties/PORT06/IP_VER", "IP_MASK", 1, 2, 0, "255.255.255.0"}, 
    {"/System_Properties/PORT06/IP_VER", "IP_BCAST", 1, 2, 0, "16.0.0.255"}, 
    {"/System_Properties/PORT06/IP_VER", "IP_GATE", 1, 2, 0, "16.0.0.254"}, 
    {"/System_Properties/PORT06", "MTU", 1, 1, 9216, ""}, 
    {"/System_Properties/PORT06", "LINK_SPEED", 1, 1, 1000, ""}, 
    {"/System_Properties/PORT06", "MAC_ADDR", 1, 2, 0, "0a0b0c0d0e06"}, 
    {"/System_Properties/PORT06", "MEDIA_TYPE", 1, 2, 0, "1GB_ETHERNET"}, 
    /* Port 7 configuration */
    {"/System_Properties", "PORT07", 1, 1, 7, ""}, 
    {"/System_Properties/PORT07", "IP_VER", 1, 2, 0, "IPV4"}, 
    {"/System_Properties/PORT07/IP_VER", "IP_ADDR", 1, 2, 0, "17.0.0.1"}, 
    {"/System_Properties/PORT07/IP_VER", "IP_MASK", 1, 2, 0, "255.255.255.0"}, 
    {"/System_Properties/PORT07/IP_VER", "IP_BCAST", 1, 2, 0, "17.0.0.255"}, 
    {"/System_Properties/PORT07/IP_VER", "IP_GATE", 1, 2, 0, "17.0.0.254"}, 
    {"/System_Properties/PORT07", "MTU", 1, 1, 9216, ""}, 
    {"/System_Properties/PORT07", "LINK_SPEED", 1, 1, 1000, ""}, 
    {"/System_Properties/PORT07", "MAC_ADDR", 1, 2, 0, "0a0b0c0d0e07"}, 
    {"/System_Properties/PORT07", "MEDIA_TYPE", 1, 2, 0, "1GB_ETHERNET"}, 
    /* Port 8 configuration */
    {"/System_Properties", "PORT08", 1, 1, 8, ""}, 
    {"/System_Properties/PORT08", "IP_VER", 1, 2, 0, "IPV4"}, 
    {"/System_Properties/PORT08/IP_VER", "IP_ADDR", 1, 2, 0, "18.0.0.1"}, 
    {"/System_Properties/PORT08/IP_VER", "IP_MASK", 1, 2, 0, "255.255.255.0"}, 
    {"/System_Properties/PORT08/IP_VER", "IP_BCAST", 1, 2, 0, "18.0.0.255"}, 
    {"/System_Properties/PORT08/IP_VER", "IP_GATE", 1, 2, 0, "18.0.0.254"}, 
    {"/System_Properties/PORT08", "MTU", 1, 1, 9216, ""}, 
    {"/System_Properties/PORT08", "LINK_SPEED", 1, 1, 1000, ""}, 
    {"/System_Properties/PORT08", "MAC_ADDR", 1, 2, 0, "0a0b0c0d0e08"}, 
    {"/System_Properties/PORT08", "MEDIA_TYPE", 1, 2, 0, "1GB_ETHERNET"}, 
    /* Port 9 configuration */
    {"/System_Properties", "PORT09", 1, 1, 9, ""}, 
    {"/System_Properties/PORT09", "IP_VER", 1, 2, 0, "IPV4"}, 
    {"/System_Properties/PORT09/IP_VER", "IP_ADDR", 1, 2, 0, "19.0.0.1"}, 
    {"/System_Properties/PORT09/IP_VER", "IP_MASK", 1, 2, 0, "255.255.255.0"}, 
    {"/System_Properties/PORT09/IP_VER", "IP_BCAST", 1, 2, 0, "19.0.0.255"}, 
    {"/System_Properties/PORT09/IP_VER", "IP_GATE", 1, 2, 0, "19.0.0.254"}, 
    {"/System_Properties/PORT09", "MTU", 1, 1, 9216, ""}, 
    {"/System_Properties/PORT09", "LINK_SPEED", 1, 1, 1000, ""}, 
    {"/System_Properties/PORT09", "MAC_ADDR", 1, 2, 0, "0a0b0c0d0e09"}, 
    {"/System_Properties/PORT09", "MEDIA_TYPE", 1, 2, 0, "1GB_ETHERNET"}, 

    /* System app properties */
    {"", "SystemApp", 0, 0, 0, ""}, 
    {"/SystemApp", "FREELISTS", 0, 0, 0, ""}, 
    {"/SystemApp/FREELISTS", "FL_MSG", 0, 0, 0, ""}, 
    {"/SystemApp/FREELISTS/FL_MSG", "ELEMENT_COUNT", 1, 1, 256, ""}, 
    {"/SystemApp/FREELISTS/FL_MSG", "META_SIZE", 1, 1, 16, ""}, 
    {"/SystemApp/FREELISTS/FL_MSG", "DATA_SIZE", 1, 1, 128, ""}, 

    /* Hardware freelist 1 */
    {"/SystemApp/FREELISTS", "FL_01", 0, 0, 0, ""}, 
    {"/SystemApp/FREELISTS/FL_01", "ELEMENT_COUNT", 1, 1, 16384, ""}, 
    {"/SystemApp/FREELISTS/FL_01", "SRAM_SIZE", 1, 1, sizeof(ix_hw_buffer_meta), ""}, 
    {"/SystemApp/FREELISTS/FL_01", "SRAM_CHAN", 1, 1, 0, ""}, 
    {"/SystemApp/FREELISTS/FL_01", "DRAM_SIZE", 1, 1, 2048, ""}, 
    {"/SystemApp/FREELISTS/FL_01", "DRAM_CHAN", 1, 1, 0, ""}, 

    /* Scratch rings */
    {"/SystemApp", "SCRATCH_RINGS", 0, 0, 0, ""}, 

    /* CSIX_RX_TO_STATS_SCR_RING scratch ring */
    {"/SystemApp/SCRATCH_RINGS", "SR_01", 0, 0, 0, ""}, 
    {"/SystemApp/SCRATCH_RINGS/SR_01", "ID", 1, 1, CSIX_RX_TO_STATS_SCR_RING, ""}, 
    {"/SystemApp/SCRATCH_RINGS/SR_01", "CHAN", 1, 1, 0, ""}, 
    {"/SystemApp/SCRATCH_RINGS/SR_01", "ELEMENT_SIZE", 1, 1, 2, ""},

    /* ETHERNET_ARP_TO_STATS_SCR_RING scratch ring */
    {"/SystemApp/SCRATCH_RINGS", "SR_02", 0, 0, 0, ""}, 
    {"/SystemApp/SCRATCH_RINGS/SR_02", "ID", 1, 1, ETHERNET_ARP_TO_STATS_SCR_RING, ""}, 
    {"/SystemApp/SCRATCH_RINGS/SR_02", "CHAN", 1, 1, 0, ""}, 
    {"/SystemApp/SCRATCH_RINGS/SR_02", "ELEMENT_SIZE", 1, 1, 2, ""},

    /* CLASS_COUNT_PORT_SCHEDULER_SCR_RING scratch ring */
    {"/SystemApp/SCRATCH_RINGS", "SR_03", 0, 0, 0, ""}, 
    {"/SystemApp/SCRATCH_RINGS/SR_03", "ID", 1, 1, CLASS_COUNT_PORT_SCHEDULER_SCR_RING, ""}, 
    {"/SystemApp/SCRATCH_RINGS/SR_03", "CHAN", 1, 1, 0, ""}, 
    {"/SystemApp/SCRATCH_RINGS/SR_03", "ELEMENT_SIZE", 1, 1, 2, ""},

    /* QM_TO_PACKET_TX_SCR_RING_0 scratch ring */
    {"/SystemApp/SCRATCH_RINGS", "SR_04", 0, 0, 0, ""}, 
    {"/SystemApp/SCRATCH_RINGS/SR_04", "ID", 1, 1, QM_TO_PACKET_TX_SCR_RING_0, ""}, 
    {"/SystemApp/SCRATCH_RINGS/SR_04", "CHAN", 1, 1, 0, ""}, 
    {"/SystemApp/SCRATCH_RINGS/SR_04", "ELEMENT_SIZE", 1, 1, 1, ""}, 

    /* QM_TO_PACKET_TX_SCR_RING_1 scratch ring */
    {"/SystemApp/SCRATCH_RINGS", "SR_05", 0, 0, 0, ""}, 
    {"/SystemApp/SCRATCH_RINGS/SR_05", "ID", 1, 1, QM_TO_PACKET_TX_SCR_RING_1, ""}, 
    {"/SystemApp/SCRATCH_RINGS/SR_05", "CHAN", 1, 1, 0, ""}, 
    {"/SystemApp/SCRATCH_RINGS/SR_05", "ELEMENT_SIZE", 1, 1, 1, ""}, 

    /* IX_CORE_TO_UBLOCK_PACKET_CTRL_RING_ID scratch ring */
    {"/SystemApp/SCRATCH_RINGS", "SR_06", 0, 0, 0, ""}, 
    {"/SystemApp/SCRATCH_RINGS/SR_06", "ID", 1, 1, IX_CORE_TO_UBLOCK_PACKET_CTRL_RING_ID, ""}, 
    {"/SystemApp/SCRATCH_RINGS/SR_06", "CHAN", 1, 1, 0, ""}, 
    {"/SystemApp/SCRATCH_RINGS/SR_06", "ELEMENT_SIZE", 1, 1, 0, ""}, 

    /* Execution engines properties */
    {"/SystemApp", "SA_EXEC_ENGINES", 0, 0, 0, ""}, 
    {"/SystemApp/SA_EXEC_ENGINES", "SA_EE_01", 1, 1, 1, ""}, 

    /* List all CC IDs defined in bindings.h for being included in this application (should be using the defined values) */ 
    {"/SystemApp/SA_EXEC_ENGINES/SA_EE_01", "CC_LIST", 1, 2, 0, "6  7  8  4"},

    /* List all microengines to be started */
    {"/SystemApp", "MICROENGINES", 0, 0, 0, ""}, 
    {"/SystemApp/MICROENGINES", "ME_11", 1, 1, 11, ""}, 
    {"/SystemApp/MICROENGINES/ME_11", "CONTEXT_MASK", 1, 1, 255, ""}, 
    {"/SystemApp/MICROENGINES", "ME_10", 1, 1, 10, ""}, 
    {"/SystemApp/MICROENGINES/ME_10", "CONTEXT_MASK", 1, 1, 255, ""}, 
    {"/SystemApp/MICROENGINES", "ME_09", 1, 1, 9, ""}, 
    {"/SystemApp/MICROENGINES/ME_09", "CONTEXT_MASK", 1, 1, 255, ""}, 
    {"/SystemApp/MICROENGINES", "ME_08", 1, 1, 8, ""}, 
    {"/SystemApp/MICROENGINES/ME_08", "CONTEXT_MASK", 1, 1, 255, ""}, 
    {"/SystemApp/MICROENGINES", "ME_07", 1, 1, 7, ""}, 
    {"/SystemApp/MICROENGINES/ME_07", "CONTEXT_MASK", 1, 1, 255, ""}, 
    {"/SystemApp/MICROENGINES", "ME_06", 1, 1, 6, ""}, 
    {"/SystemApp/MICROENGINES/ME_06", "CONTEXT_MASK", 1, 1, 255, ""}, 
    {"/SystemApp/MICROENGINES", "ME_05", 1, 1, 5, ""}, 
    {"/SystemApp/MICROENGINES/ME_05", "CONTEXT_MASK", 1, 1, 255, ""}, 
    {"/SystemApp/MICROENGINES", "ME_04", 1, 1, 4, ""}, 
    {"/SystemApp/MICROENGINES/ME_04", "CONTEXT_MASK", 1, 1, 255, ""}, 
    {"/SystemApp/MICROENGINES", "ME_03", 1, 1, 3, ""}, 
    {"/SystemApp/MICROENGINES/ME_03", "CONTEXT_MASK", 1, 1, 255, ""}, 

    /* Ethernet Tx properties */
    {"", "ETH_TX", 0, 0, 0, ""}, 
    {"/ETH_TX", "ETH_TX_MEV2_NUM_MASK", 1, 1, 776, ""}, /* ME3, ME8, ME9 */

    /* CSIX RX properties */
    {"", "CSIX_RX", 0, 0, 0, ""}, 
    {"/CSIX_RX", "CSIX_RX_MEV2_NUM_MASK", 1, 1, 3072, ""}, /* ME 10 and 11 */

    /* QM properties */
    {"", "QM", 0, 0, 0, ""}, 
    {"/QM", "QM_UENG_NUM", 1, 1, 1, ""}, 
    {"/QM", "QUEUE_DESC_TOTAL", 1, 1, 1024, ""}, 
    {"/QM", "QM_UENG_MASK", 1, 1, 64, ""}, /* ME6 */

    /* Scheduler properties */
    {"", "SCHED", 0, 0, 0, ""}, 
    {"/SCHED", "SCHEDULER_TYPE", 1, 1, 1, ""}, 
    {"/SCHED", "SCHED_UENG_NUM", 1, 1, 2, ""}, 
    {"/SCHED", "NUMBER_OF_PORTS", 1, 1, 16, ""}, 
    {"/SCHED", "VOQ_WEIGHTS_PER_PORT", 1, 1, 1, ""}, 
    {"/SCHED", "VOQ_WEIGHT_VALUE", 1, 1, 1, ""}, 
    {"/SCHED", "PKT_WEIGHT_VALUE", 1, 1, 1, ""}, 
    {"/SCHED", "QUEUE_PORT_SIZE", 1, 1, 256, ""}, 

    /* L2 table manager properties */
    {"", "L2_TABLE_MANAGER", 0, 0, 0, ""}, 
    {"/L2_TABLE_MANAGER", "DEFAULT", 0, 0, 0, ""}, 
    {"/L2_TABLE_MANAGER/DEFAULT", "NUM_ENTRIES", 1, 1, 1024, ""}, 
    {"/L2_TABLE_MANAGER/DEFAULT", "MEMORY_TYPE", 1, 1, 0, ""}, 
    {"/L2_TABLE_MANAGER/DEFAULT", "MEMORY_CHANNEL", 1, 1, 1, ""}, 
    {"/L2_TABLE_MANAGER/DEFAULT", "HANDLE", 1, 1, 0, ""}, 
    {"/L2_TABLE_MANAGER/DEFAULT", "REFCOUNT", 1, 1, 0, ""},

    /* Array is terminated with the follwoing entry */
    {"", "", 0, 0, 0, ""}};

#endif
