/*===========================================================================
 * = COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 * ===========================================================================
 */

/* =========================================================================*/
/**
 * @file   ix_sa_cc_list.c
 *
 * Definition of CC_INITIALIZATION funciton.
 *
 **/
/*=========================================================================*/
/* $Id: ix_sa_cc_list.c,v 1.3 2003/09/23 20:15:13 ktseng Exp $ */
#include "ix_cc_error.h"
#include "sa/ix_sa_cc_list.h"

#include "cc/ix_cc_qm_frmwrk.h"
#include "cc/ix_cc_csix_rx.h"
#include "cc/ix_cc_sched.h"
#include "cc/ix_cc_eth_tx.h"
#ifdef INCLUDE_PDK
#include "cc/ix_cc_fpm_proxy.h"
#endif

#if !defined(IX_CONFIGURATION_oc192_ethernet_egress)
#error IX_CONFIGURATION_oc192_ethernet_egress not defined! Wrong configuration header.
#endif

/** 
 * Define the directory path where the uof file is located.
 * The base directoy specified by the environmental variable
 * IXA_SDK_DEV will be inserted by the build script.
 */
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
char g_sa_uofFile[] = "10x1gb_ethernet_egress.uof";
#else
char g_sa_uofFile[] = "src\\applications\\ipv4_v6_forwarder\\10gb_ethernet\\10x1GbE_egress\\wbench_project\\10x1gb_ethernet_egress.uof";
#endif
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
/*  _ix_sa_set_cc */
/*  ---------------------------------------- */
/**
 * This function is used to set a init and fini function for a CC
 *
 * @return IX_SUCCESS upon success or an ix_error with one of the following values.
 * @retval NONE
**/
ix_error
_ix_sa_set_cc(ix_sa_ccfunc *arg_pCC_List, ix_uint32 arg_idx,
              ix_cc_init arg_Init, ix_cc_fini arg_Fini)
{
    ix_uint32 i;

    IX_ERROR_CHECK_ARG_NULL(1, arg_pCC_List);
    arg_pCC_List[arg_idx].init = arg_Init;
    arg_pCC_List[arg_idx].fini = arg_Fini;
    for (i = 0; i < IX_CC_INSTANCE_MAX; i++)
    {
        arg_pCC_List[arg_idx].cc_handle[i] = 0;
    }
    return IX_SUCCESS;
}
#endif

/*  _ix_sa_cc_list */
/*  ---------------------------------------- */
/**
 * This function generates and returns the ix_sa_ccfunc array.
 *
 * An array containing pointers to various cc functions is returned
 * to be used during cc initialization.  
 *
 * @return IX_SUCCESS, a pass through ix_error, or an ix_error with one
 *         of the following values.
 * @retval IX_CC_ERROR_NULL arg_pCC_List is NULL
**/
ix_error
_ix_sa_cc_list(ix_sa_ccfunc *arg_pCC_List)
{
    IX_ERROR_CHECK_ARG_NULL(1, arg_pCC_List);

    /* add one entry for each CC in this configuration.  example below.
     *   _ix_sa_set_cc
     *          Param #1: the cclist passed to _ix_sa_cc_list (arg_pCC_List)
     *          Param #2: Identifier of CC as defined in bindings.h
     *          Param #3: The CCs init function (include header in this file)
     *          Param #4: The CCs fini function
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_DUMMY, _ix_sa_dummy_cc_init, _ix_sa_dummy_cc_fini);
    */

    /* add Queue Manager core component */
    _ix_sa_set_cc( arg_pCC_List,
        IX_CC_QM, ix_cc_qm_init, ix_cc_qm_fini );

    /* add Scheduler core component */
    _ix_sa_set_cc( arg_pCC_List,
        IX_CC_SCHEDULER, ix_cc_sched_init, ix_cc_sched_fini );

    /* add CSIX_RX core component */
    _ix_sa_set_cc( arg_pCC_List,
        IX_CC_CSIX_RX, ix_cc_csix_rx_init, ix_cc_csix_rx_fini );

    /* add ETH_TX core component */
    _ix_sa_set_cc( arg_pCC_List,
        IX_CC_ETH_TX, ix_cc_eth_tx_init, ix_cc_eth_tx_fini );

#ifdef INCLUDE_PDK
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_FPM_PROXY, ix_cc_fpm_proxy_init, ix_cc_fpm_proxy_fini);
#endif

    return IX_SUCCESS;
}
