/*===========================================================================
 * = COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 * ===========================================================================
 */

/* =========================================================================*/
/**
 * @file   ix_sa_symbols.c
 *
 * Patches misc symbols for this configuration.
 **/
/*=========================================================================*/
#define IX_ERROR_FILE_IDENT "$Id: ix_sa_symbols.c,v 1.11 2003/12/08 22:18:06 ttschlue Exp $"


#include "ix_cc_error.h"
#include "sa/internal/internal_sa.h"
#include "sa/ix_sa_cc_list.h"

#if !defined(IX_CONFIGURATION_oc192_ethernet_egress)
#error IX_CONFIGURATION_oc192_ethernet_egress not defined! Wrong configuration header.
#endif


/*  _ix_sa_patch_symbols */
/*  ---------------------------------------- */
/**
 * Patch miscelanious symbols.
 *
 * This function patches a variety of symbols not specifially associated
 * with a given Core Component.
 *
 * 
 * @param arg_sa_ctrl System Application Control Structure
 * @return IX_SUCCESS, a pass through ix_error, or an ix_error with one
 *         of the following values.
 * @retval NONE
**/
ix_error
_ix_sa_patch_symbols(ix_sa_ctrl *arg_sa_ctrl)
{
    ix_imported_symbol aImportSymbols[7]; 
    ix_uint32 relBase = 0;
    ix_buffer_free_list_info hwFreeListInfo;
    void *pZeroBlock;
    ix_uint32 phys_meta_address = 0;
    ix_uint32 phys_data_address = 0;
    ix_memory_type memType;
    ix_uint32 channel = 0;
    ix_uint32 zboffset = 0;
    ix_uint32 memOffset = 0;
    void *pPhysicalAddress = NULL;
    void *pSchedQueueBase;
    ix_uint32 i;

    IX_ERROR_CHECK_ARG_NULL(1, arg_sa_ctrl);

    /* allocate 128 bytes for patching the zeroblock symbol */
    IX_ERROR_CR(ix_rm_mem_alloc(IX_MEMORY_TYPE_SRAM, 0,
                                128, &pZeroBlock));

    /* Initialize the memory to all 0s */     
    ix_ossl_memset(pZeroBlock, 0, 128);

    IX_ERROR_CR(ix_rm_buffer_free_list_get_info(arg_sa_ctrl->freelist[1],
                                                &hwFreeListInfo));

    IX_ERROR_CR(ix_rm_get_phys_offset((void*)hwFreeListInfo.m_pMetaBaseAddress, 
                                      &memType, &channel, &phys_meta_address, 
                                      &pPhysicalAddress)); 

    IX_ERROR_CR(ix_rm_get_phys_offset((void*)hwFreeListInfo.m_pDataBaseAddress, 
                                      &memType, &channel, &phys_data_address, 
                                      &pPhysicalAddress)); 

    IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(0, phys_meta_address);
    IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(0, phys_data_address);

    relBase = phys_data_address - ( phys_meta_address * 
                                     (hwFreeListInfo.m_DataElementSize/hwFreeListInfo.m_MetaElementSize));

    aImportSymbols[0].m_Value = phys_meta_address;
    aImportSymbols[0].m_Name  = "BUF_SRAM_BASE" ;

    /* Patch to ME# 11, 10, 9, 8, 5, 4, 3 (0x0F38) */
    for (i = 0; i < IX_MAX_MICROENGINE_NUMBER; i++)
    {
         /* if the bit is set, then patch the symbols */
        if((0x0F38 & (1 << i)) != 0)
        {
            IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[i],
	                                         1,
					         aImportSymbols));
        }
    }

    aImportSymbols[0].m_Value = (ix_uint32) hwFreeListInfo.m_FreeListInfo;/* hwFreeList ; */
    aImportSymbols[0].m_Name  = "BUF_FREE_LIST0" ;

    /* Patch to ME# 11, 10, 9, 8, 5, 4, 3 (0x0F38) */
    for (i = 0; i < IX_MAX_MICROENGINE_NUMBER; i++)
    {
         /* if the bit is set, then patch the symbols */
        if((0x0F38 & (1 << i)) != 0)
        {
            IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[i],
	                                         1,
					         aImportSymbols));
        }
    }

    aImportSymbols[0].m_Value =  relBase ;
    aImportSymbols[0].m_Name  = "DL_REL_BASE" ;
    /* Patch to ME# 11, 9, 8 (0x0B00) */
    for (i = 0; i < IX_MAX_MICROENGINE_NUMBER; i++)
    {
         /* if the bit is set, then patch the symbols */
        if((0x0B00 & (1 << i)) != 0)
        {
            IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[i],
	                                         1,
					         aImportSymbols));
        }
    }

    /* Patch SRAM_ZERO_BLOCK to PACKET_TX microblock (ME #5) */
    IX_ERROR_CR(ix_rm_get_phys_offset((void*)pZeroBlock, 
                                      &memType, &channel, &zboffset, 
                                      &pPhysicalAddress)); 

    aImportSymbols[0].m_Value = (ix_uint32)zboffset;
    aImportSymbols[0].m_Name  = "SRAM_ZERO_BLOCK";
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[5], 
                                         1,
                                         aImportSymbols));

    /* Patch FREE_LIST_ID for CSIX_RX microblock */
    aImportSymbols[0].m_Value = (ix_uint32)hwFreeListInfo.m_FreeListInfo1;
    aImportSymbols[0].m_Name  = "FREE_LIST_ID";
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(0x13, 1, aImportSymbols));

    /* Allocate memory for SCHED_QUEUE_STRUCTURES_BASE for tx_helper microblock */
    IX_ERROR_CR(ix_rm_mem_alloc(IX_MEMORY_TYPE_SRAM,
                                2, /* Use SRAM channel 2 */
                                (1024 * 8), 
                                (void **)&pSchedQueueBase));

    /* First obtain the base offsets */
    IX_ERROR_CR(ix_rm_get_phys_offset(pSchedQueueBase,
                                      &memType,
                                      &channel,
                                      &memOffset,
                                      &pPhysicalAddress));

    /* Pack the SRAM channel number into the offset */
    IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(channel, memOffset);

    /* Address of the base of the trie table 8 */
    aImportSymbols[0].m_Value = memOffset;
    aImportSymbols[0].m_Name = "SCHED_QUEUE_STRUCTURES_BASE";

    IX_ERROR_CR(ix_rm_ueng_patch_symbols(0x7, 1, aImportSymbols));

    return IX_SUCCESS;
}

