//---------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//---------------------------------------------------------------------
///////////////////////////////////////////////////////////////////////
//
//
//      File Name: 	dl_source.uc
//
//      Purpose: 	Contains macros for dl_sink and dl_source for
//                  Ingress microblocks.
//
//      History:
//
//
//      Date            Comment                         By
//      ---------------------------------------------------------------
//
//      11/11/2002      Created                         asv
//
///////////////////////////////////////////////////////////////////////

#ifndef __DL_SOURCE_UC__
#define __DL_SOURCE_UC__

#include <stdmac.uc>
#include <sig_macros.uc>
#include <dispatch_loop.h>


/////////////////////////////
//	Some local defines
/////////////////////////////

#if (DL_RING_NUMBER > 11)
#error			"For Ring Number Greater than 11, we cannot use the SCR_Ring#_Full input state. Check RING_NUMBER"
#else
#define_eval	DL_RING_FULL			SCR_Ring/**/DL_RING_NUMBER/**/_Full	
#endif

/////////////////////////////
// Global variables/Registers
/////////////////////////////

.reg global	dl_ring					// scratch ring number for pos rx ==>ipv4. This can be made an absolut register
.reg global	dl_buf_handle			// The current Buffer handle.
.reg global	dl_eop_buf_handle		// For large packets, this is the last buffer in the chain
.reg global	dl_next_block			// Next block that should process the buffer/packet

.reg    global  dl_exception_reg
.set            dl_exception_reg

; Global variable for next thread signaling. 
.sig volatile 	sig_prev				
.addr			sig_prev				DL_SIG_WAKE


///////////////////////////////////////////////////////////////////////
// Macro Name  : dl_get_egress_exception_pkt
// Description : Macro to write the two longwords of exception
//				 and trigger interrupt to activate RM ISR for 
//				 exception packets. In the current implementation
//               all packets go to queue manager and it is assumed 
//               that all packet processing has been completed
//               at the core.
// Input       : in_buffer_handle, in_exception_code
// Constant    : nil
// Size        : 4
// Branches    : 0
////////////////////////////////////////////////////////////////////////
#macro dl_get_egress_exception_pkt()


.begin
   .reg ring, temp
   .reg queue_number
   .reg pkt_size
   .reg buf_size
   .reg $excep_ctrl_lw0  $excep_ctrl_lw1 $excep_ctrl_lw2
   .xfer_order $excep_ctrl_lw0  $excep_ctrl_lw1 $excep_ctrl_lw2
   .reg $excep_data_lw0, $excep_data_lw1, $excep_data_lw2
   .xfer_order $excep_data_lw0, $excep_data_lw1, $excep_data_lw2
   .sig scratch_rd_done, meta_read_signal         

   // set the ring number

   alu[ring, --,b, EXCEPTION_RING_IN_0, <<2]

   // read one longword from exception ring 1

   scratch[get, $excep_ctrl_lw0, ring, 0, 3], sig_done[scratch_rd_done]
   ctx_arb[scratch_rd_done]

   .if( $excep_ctrl_lw0 != 0 )

	 move[dl_buf_handle, $excep_ctrl_lw0]				; LW0 from ring is the current buffer handle (containign SOP)

	 move[dl_eop_buf_handle, $excep_ctrl_lw1]

	 // allocate 6 read xfer registers for reading in the meta data 

	 xbuf_alloc[$dl_meta, 7, read]

	 //read meta data

     dl_meta_load_cache[$excep_ctrl_lw0, $dl_meta, meta_read_signal, 1, 6]

	 // wait for meta data read to complete

	 ctx_arb[meta_read_signal]


	 //Write three long words on scratch ring

#ifdef	NEW_MSG_FORMAT
    dl_meta_get_output_port[queue_number]				; 
	dl_meta_get_packet_size[pkt_size]					;
	alu_shf[queue_number, --, B, queue_number, <<12]	; outport is in bits 15:12, 12:0 is qos class. XXX 0for now.

	alu_shf[$excep_data_lw0, queue_number, OR, pkt_size, <<16]	

	move[$excep_data_lw1, dl_buf_handle]
	move[$excep_data_lw2, dl_eop_buf_handle]

	alu_shf[ring, --, b, ETHERNET_ARP_RING_OUT, <<2]

#else	// #ifdef NEW_MSG_FORMAT

	// old code.

	move[$excep_data_lw0, dl_buf_handle]
	move[$excep_data_lw1, dl_eop_buf_handle]
	dl_meta_get_output_port[queue_number]				; LW2 on the ring is Queue number for Queue Manager
	alu_shf[queue_number, --, B, queue_number, <<4]		; fabric * 16 is the actual queue number
	alu_shf[$excep_data_lw2, queue_number, OR, 1, <<31]		; The msb is always set to 1, to prevent producing a value of 0 on to the ring.

	alu_shf[ring, --, b, QM_RING_IN_0, <<2]

#endif

    // check for ring full condition. if full then keep trying

check_ring#:
	br_inp_state[ETHER_ARP_SINK_RING_FULL, check_ring#]
	scratch[put, $excep_data_lw0, 0, ring, 3], sig_done[scratch_rd_done]
	ctx_arb[scratch_rd_done]

	br[end#]

    // if the ring is full go back and retry to write to scratch ring
ring_full#:
    br[check_ring#]      

end#:

   xbuf_free[$dl_meta]
  .endif


.end
read_again#:


#endm /* dl_get_egress_exception_pkt() */
 

 
#endif	// __DL_SOURCE_UC__