/*===========================================================================
 * = COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 * ===========================================================================
 */
/* $Id: ingress_internal_registry_data_linux.h,v 1.29 2003/12/10 17:51:04 ktseng Exp $ */

#if !defined(__IX_SA_INTERNAL_REGISTRY_DATA_H__)
#define __IX_SA_INTERNAL_REGISTRY_DATA_H__

#include "ix_rm.h"
#include "bindings.h"
#include "cc/ix_cc_rtmv4.h"
#include "ix_cc_microengines_bindings.h"
#include "dl_system.h"
#include "hardware.h"

/**
 * The following array contains core components properties
 * that will be populated into the system registry by
 * system application during initialization time.
 *
 * Each field in the array entry is defined as follows:
 *  1. "property path" - specify property path
 *  2. "property name" - specify property name
 *  3. property attribute - 0: no property value required
 *                          1: property value required
 *  4. property type - property type
 *  5. property value - in the format of number
 *  6. property value - in the format of string
 */

ix_sa_registry_property gSAPropList[] = \
   {
    /* Configuration properties */
    {"", "Target_Config", 1, 2, 0, "oc192_ethernet_ipv6_ingress"},
    {"", "INGRESS_EGRESS", 1, 1, 0, ""},

    /* System Properties */
    {"", "System_Properties", 0, 0, 0, ""},
    {"/System_Properties", "AUTO_RESTART", 1, 1, 0, ""},
    {"/System_Properties", "BLADE_ID", 1, 1, 2, ""},
    {"/System_Properties", "SWITCH_FABRIC_LOOPBACK_FLAG", 1, 1, 1, ""},
    {"/System_Properties", "SRAM_CHANNEL_NUM", 1, 1, 0, ""},
    {"/System_Properties", "REMOTE_STACK_FLAG", 1, 1, 0, ""},

    /* Port 0 configuration */
    {"/System_Properties", "PORT00", 1, 1, 0, ""},
    {"/System_Properties/PORT00", "IP_VER", 1, 2, 0, "IPV4V6"},
    {"/System_Properties/PORT00/IP_VER", "NUM_VIRTUAL_IF_IPV4", 1, 1, 1, ""},
    {"/System_Properties/PORT00/IP_VER", "IPV4_ADDR_0", 1, 2, 0, "10.0.0.1"},
    {"/System_Properties/PORT00/IP_VER", "IPV4_MASK_0", 1, 2, 0, "255.255.255.0"},
    {"/System_Properties/PORT00/IP_VER", "IPV4_BCAST_0", 1, 2, 0, "10.0.0.255"},
    {"/System_Properties/PORT00/IP_VER", "IPV4_GATE_0", 1, 2, 0, "10.0.0.254"},
    {"/System_Properties/PORT00/IP_VER", "NUM_VIRTUAL_IF_IPV6", 1, 1, 1, ""},
    {"/System_Properties/PORT00/IP_VER", "IPV6_ADDR_0", 1, 2, 0, "3ffe:0:200:f100::fe0d:0e0a"},
    {"/System_Properties/PORT00/IP_VER", "IPV6_PREFIX_LEN_0", 1, 1, 0, ""},
    {"/System_Properties/PORT00/IP_VER", "IPV6_GATE_0", 1, 2, 0, "3ffe:0:200:f100::fe0d:0e0d"},
    {"/System_Properties/PORT00", "MTU", 1, 1, 9216, ""},
    {"/System_Properties/PORT00", "LINK_SPEED", 1, 1, 1000, ""},
    {"/System_Properties/PORT00", "MAC_ADDR", 1, 2, 0, "0a0b0c0d0e00"},
    {"/System_Properties/PORT00", "MEDIA_TYPE", 1, 2, 0, "1GB_ETHERNET"},

    /* Port 1 configuration */
    {"/System_Properties", "PORT01", 1, 1, 1, ""},
    {"/System_Properties/PORT01", "IP_VER", 1, 2, 0, "IPV4V6"},
    {"/System_Properties/PORT01/IP_VER", "NUM_VIRTUAL_IF_IPV4", 1, 1, 1, ""},
    {"/System_Properties/PORT01/IP_VER", "IPV4_ADDR_0", 1, 2, 0, "11.0.0.1"},
    {"/System_Properties/PORT01/IP_VER", "IPV4_MASK_0", 1, 2, 0, "255.255.255.0"},
    {"/System_Properties/PORT01/IP_VER", "IPV4_BCAST_0", 1, 2, 0, "11.0.0.255"},
    {"/System_Properties/PORT01/IP_VER", "IPV4_GATE_0", 1, 2, 0, "11.0.0.254"},
    {"/System_Properties/PORT01/IP_VER", "NUM_VIRTUAL_IF_IPV6", 1, 1, 1, ""},
    {"/System_Properties/PORT01/IP_VER", "IPV6_ADDR_0", 1, 2, 0, "3ffe:0:200:f101::fe0d:0e0a"},
    {"/System_Properties/PORT01/IP_VER", "IPV6_PREFIX_LEN_0", 1, 1, 0, ""},
    {"/System_Properties/PORT01/IP_VER", "IPV6_GATE_0", 1, 2, 0, "3ffe:0:200:f101::fe0d:0e0d"},
    {"/System_Properties/PORT01", "MTU", 1, 1, 9216, ""},
    {"/System_Properties/PORT01", "LINK_SPEED", 1, 1, 1000, ""},
    {"/System_Properties/PORT01", "MAC_ADDR", 1, 2, 0, "0a0b0c0d0e01"},
    {"/System_Properties/PORT01", "MEDIA_TYPE", 1, 2, 0, "1GB_ETHERNET"},

    /* Port 2 configuration */
    {"/System_Properties", "PORT02", 1, 1, 2, ""},
    {"/System_Properties/PORT02", "IP_VER", 1, 2, 0, "IPV4V6"},
    {"/System_Properties/PORT02/IP_VER", "NUM_VIRTUAL_IF_IPV4", 1, 1, 1, ""},
    {"/System_Properties/PORT02/IP_VER", "IPV4_ADDR_0", 1, 2, 0, "12.0.0.1"},
    {"/System_Properties/PORT02/IP_VER", "IPV4_MASK_0", 1, 2, 0, "255.255.255.0"},
    {"/System_Properties/PORT02/IP_VER", "IPV4_BCAST_0", 1, 2, 0, "12.0.0.255"},
    {"/System_Properties/PORT02/IP_VER", "IPV4_GATE_0", 1, 2, 0, "12.0.0.254"},
    {"/System_Properties/PORT02/IP_VER", "NUM_VIRTUAL_IF_IPV6", 1, 1, 1, ""},
    {"/System_Properties/PORT02/IP_VER", "IPV6_ADDR_0", 1, 2, 0, "3ffe:0:200:f102::fe0d:0e0a"},
    {"/System_Properties/PORT02/IP_VER", "IPV6_PREFIX_LEN_0", 1, 1, 0, ""},
    {"/System_Properties/PORT02/IP_VER", "IPV6_GATE_0", 1, 2, 0, "3ffe:0:200:f102::fe0d:0e0d"},
    {"/System_Properties/PORT02", "MTU", 1, 1, 9216, ""},
    {"/System_Properties/PORT02", "LINK_SPEED", 1, 1, 1000, ""},
    {"/System_Properties/PORT02", "MAC_ADDR", 1, 2, 0, "0a0b0c0d0e02"},
    {"/System_Properties/PORT02", "MEDIA_TYPE", 1, 2, 0, "1GB_ETHERNET"},

    /* Port 3 configuration */
    {"/System_Properties", "PORT03", 1, 1, 3, ""},
    {"/System_Properties/PORT03", "IP_VER", 1, 2, 0, "IPV4V6"},
    {"/System_Properties/PORT03/IP_VER", "NUM_VIRTUAL_IF_IPV4", 1, 1, 1, ""},
    {"/System_Properties/PORT03/IP_VER", "IPV4_ADDR_0", 1, 2, 0, "13.0.0.1"},
    {"/System_Properties/PORT03/IP_VER", "IPV4_MASK_0", 1, 2, 0, "255.255.255.0"},
    {"/System_Properties/PORT03/IP_VER", "IPV4_BCAST_0", 1, 2, 0, "13.0.0.255"},
    {"/System_Properties/PORT03/IP_VER", "IPV4_GATE_0", 1, 2, 0, "13.0.0.254"},
    {"/System_Properties/PORT03/IP_VER", "NUM_VIRTUAL_IF_IPV6", 1, 1, 1, ""},
    {"/System_Properties/PORT03/IP_VER", "IPV6_ADDR_0", 1, 2, 0, "3ffe:0:200:f103::fe0d:0e0a"},
    {"/System_Properties/PORT03/IP_VER", "IPV6_PREFIX_LEN_0", 1, 1, 0, ""},
    {"/System_Properties/PORT03/IP_VER", "IPV6_GATE_0", 1, 2, 0, "3ffe:0:200:f103::fe0d:0e0d"},
    {"/System_Properties/PORT03", "MTU", 1, 1, 9216, ""},
    {"/System_Properties/PORT03", "LINK_SPEED", 1, 1, 1000, ""},
    {"/System_Properties/PORT03", "MAC_ADDR", 1, 2, 0, "0a0b0c0d0e03"}, 
    {"/System_Properties/PORT03", "MEDIA_TYPE", 1, 2, 0, "1GB_ETHERNET"},

    /* Port 4 configuration */
    {"/System_Properties", "PORT04", 1, 1, 4, ""}, 
    {"/System_Properties/PORT04", "IP_VER", 1, 2, 0, "IPV4V6"}, 
    {"/System_Properties/PORT04/IP_VER", "NUM_VIRTUAL_IF_IPV4", 1, 1, 1, ""},
    {"/System_Properties/PORT04/IP_VER", "IPV4_ADDR_0", 1, 2, 0, "14.0.0.1"}, 
    {"/System_Properties/PORT04/IP_VER", "IPV4_MASK_0", 1, 2, 0, "255.255.255.0"}, 
    {"/System_Properties/PORT04/IP_VER", "IPV4_BCAST_0", 1, 2, 0, "14.0.0.255"}, 
    {"/System_Properties/PORT04/IP_VER", "IPV4_GATE_0", 1, 2, 0, "14.0.0.254"}, 
    {"/System_Properties/PORT04/IP_VER", "NUM_VIRTUAL_IF_IPV6", 1, 1, 1, ""},
    {"/System_Properties/PORT04/IP_VER", "IPV6_ADDR_0", 1, 2, 0, "3ffe:0:200:f104::fe0d:0e0a"},
    {"/System_Properties/PORT04/IP_VER", "IPV6_PREFIX_LEN_0", 1, 1, 0, ""},
    {"/System_Properties/PORT04/IP_VER", "IPV6_GATE_0", 1, 2, 0, "3ffe:0:200:f104::fe0d:0e0d"},
    {"/System_Properties/PORT04", "MTU", 1, 1, 9216, ""}, 
    {"/System_Properties/PORT04", "LINK_SPEED", 1, 1, 1000, ""}, 
    {"/System_Properties/PORT04", "MAC_ADDR", 1, 2, 0, "0a0b0c0d0e04"}, 
    {"/System_Properties/PORT04", "MEDIA_TYPE", 1, 2, 0, "1GB_ETHERNET"}, 

    /* Port 5 configuration */
    {"/System_Properties", "PORT05", 1, 1, 5, ""}, 
    {"/System_Properties/PORT05", "IP_VER", 1, 2, 0, "IPV4V6"}, 
    {"/System_Properties/PORT05/IP_VER", "NUM_VIRTUAL_IF_IPV4", 1, 1, 1, ""},
    {"/System_Properties/PORT05/IP_VER", "IPV4_ADDR_0", 1, 2, 0, "15.0.0.1"}, 
    {"/System_Properties/PORT05/IP_VER", "IPV4_MASK_0", 1, 2, 0, "255.255.255.0"}, 
    {"/System_Properties/PORT05/IP_VER", "IPV4_BCAST_0", 1, 2, 0, "15.0.0.255"}, 
    {"/System_Properties/PORT05/IP_VER", "IPV4_GATE_0", 1, 2, 0, "15.0.0.254"}, 
    {"/System_Properties/PORT05/IP_VER", "NUM_VIRTUAL_IF_IPV6", 1, 1, 1, ""},
    {"/System_Properties/PORT05/IP_VER", "IPV6_ADDR_0", 1, 2, 0, "3ffe:0:200:f105::fe0d:0e0a"},
    {"/System_Properties/PORT05/IP_VER", "IPV6_PREFIX_LEN_0", 1, 1, 0, ""},
    {"/System_Properties/PORT05/IP_VER", "IPV6_GATE_0", 1, 2, 0, "3ffe:0:200:f105::fe0d:0e0d"},
    {"/System_Properties/PORT05", "MTU", 1, 1, 9216, ""}, 
    {"/System_Properties/PORT05", "LINK_SPEED", 1, 1, 1000, ""}, 
    {"/System_Properties/PORT05", "MAC_ADDR", 1, 2, 0, "0a0b0c0d0e05"}, 
    {"/System_Properties/PORT05", "MEDIA_TYPE", 1, 2, 0, "1GB_ETHERNET"}, 

    /* Port 6 configuration */
    {"/System_Properties", "PORT06", 1, 1, 6, ""}, 
    {"/System_Properties/PORT06", "IP_VER", 1, 2, 0, "IPV4V6"}, 
    {"/System_Properties/PORT06/IP_VER", "NUM_VIRTUAL_IF_IPV4", 1, 1, 1, ""},
    {"/System_Properties/PORT06/IP_VER", "IPV4_ADDR_0", 1, 2, 0, "16.0.0.1"}, 
    {"/System_Properties/PORT06/IP_VER", "IPV4_MASK_0", 1, 2, 0, "255.255.255.0"}, 
    {"/System_Properties/PORT06/IP_VER", "IPV4_BCAST_0", 1, 2, 0, "16.0.0.255"}, 
    {"/System_Properties/PORT06/IP_VER", "IPV4_GATE_0", 1, 2, 0, "16.0.0.254"}, 
    {"/System_Properties/PORT06/IP_VER", "NUM_VIRTUAL_IF_IPV6", 1, 1, 1, ""},
    {"/System_Properties/PORT06/IP_VER", "IPV6_ADDR_0", 1, 2, 0, "3ffe:0:200:f106::fe0d:0e0a"},
    {"/System_Properties/PORT06/IP_VER", "IPV6_PREFIX_LEN_0", 1, 1, 0, ""},
    {"/System_Properties/PORT06/IP_VER", "IPV6_GATE_0", 1, 2, 0, "3ffe:0:200:f106::fe0d:0e0d"},
    {"/System_Properties/PORT06", "MTU", 1, 1, 9216, ""}, 
    {"/System_Properties/PORT06", "LINK_SPEED", 1, 1, 1000, ""}, 
    {"/System_Properties/PORT06", "MAC_ADDR", 1, 2, 0, "0a0b0c0d0e06"}, 
    {"/System_Properties/PORT06", "MEDIA_TYPE", 1, 2, 0, "1GB_ETHERNET"}, 

    /* Port 7 configuration */
    {"/System_Properties", "PORT07", 1, 1, 7, ""}, 
    {"/System_Properties/PORT07", "IP_VER", 1, 2, 0, "IPV4V6"}, 
    {"/System_Properties/PORT07/IP_VER", "NUM_VIRTUAL_IF_IPV4", 1, 1, 1, ""},
    {"/System_Properties/PORT07/IP_VER", "IPV4_ADDR_0", 1, 2, 0, "17.0.0.1"}, 
    {"/System_Properties/PORT07/IP_VER", "IPV4_MASK_0", 1, 2, 0, "255.255.255.0"}, 
    {"/System_Properties/PORT07/IP_VER", "IPV4_BCAST_0", 1, 2, 0, "17.0.0.255"}, 
    {"/System_Properties/PORT07/IP_VER", "IPV4_GATE_0", 1, 2, 0, "17.0.0.254"}, 
    {"/System_Properties/PORT07/IP_VER", "NUM_VIRTUAL_IF_IPV6", 1, 1, 1, ""},
    {"/System_Properties/PORT07/IP_VER", "IPV6_ADDR_0", 1, 2, 0, "3ffe:0:200:f107::fe0d:0e0a"},
    {"/System_Properties/PORT07/IP_VER", "IPV6_PREFIX_LEN_0", 1, 1, 0, ""},
    {"/System_Properties/PORT07/IP_VER", "IPV6_GATE_0", 1, 2, 0, "3ffe:0:200:f107::fe0d:0e0d"},
    {"/System_Properties/PORT07", "MTU", 1, 1, 9216, ""}, 
    {"/System_Properties/PORT07", "LINK_SPEED", 1, 1, 1000, ""}, 
    {"/System_Properties/PORT07", "MAC_ADDR", 1, 2, 0, "0a0b0c0d0e07"}, 
    {"/System_Properties/PORT07", "MEDIA_TYPE", 1, 2, 0, "1GB_ETHERNET"}, 

    /* Port 8 configuration */
    {"/System_Properties", "PORT08", 1, 1, 8, ""}, 
    {"/System_Properties/PORT08", "IP_VER", 1, 2, 0, "IPV4V6"}, 
    {"/System_Properties/PORT08/IP_VER", "NUM_VIRTUAL_IF_IPV4", 1, 1, 1, ""},
    {"/System_Properties/PORT08/IP_VER", "IPV4_ADDR_0", 1, 2, 0, "18.0.0.1"}, 
    {"/System_Properties/PORT08/IP_VER", "IPV4_MASK_0", 1, 2, 0, "255.255.255.0"}, 
    {"/System_Properties/PORT08/IP_VER", "IPV4_BCAST_0", 1, 2, 0, "18.0.0.255"}, 
    {"/System_Properties/PORT08/IP_VER", "IPV4_GATE_0", 1, 2, 0, "18.0.0.254"}, 
    {"/System_Properties/PORT08/IP_VER", "NUM_VIRTUAL_IF_IPV6", 1, 1, 1, ""},
    {"/System_Properties/PORT08/IP_VER", "IPV6_ADDR_0", 1, 2, 0, "3ffe:0:200:f108::fe0d:0e0a"},
    {"/System_Properties/PORT08/IP_VER", "IPV6_PREFIX_LEN_0", 1, 1, 0, ""},
    {"/System_Properties/PORT08/IP_VER", "IPV6_GATE_0", 1, 2, 0, "3ffe:0:200:f108::fe0d:0e0d"},
    {"/System_Properties/PORT08", "MTU", 1, 1, 9216, ""}, 
    {"/System_Properties/PORT08", "LINK_SPEED", 1, 1, 1000, ""}, 
    {"/System_Properties/PORT08", "MAC_ADDR", 1, 2, 0, "0a0b0c0d0e08"}, 
    {"/System_Properties/PORT08", "MEDIA_TYPE", 1, 2, 0, "1GB_ETHERNET"}, 

    /* Port 9 configuration */
    {"/System_Properties", "PORT09", 1, 1, 9, ""},
    {"/System_Properties/PORT09", "IP_VER", 1, 2, 3, "IPV4V6"},
    {"/System_Properties/PORT09/IP_VER", "NUM_VIRTUAL_IF_IPV4", 1, 1, 1, ""},
    {"/System_Properties/PORT09/IP_VER", "IPV4_ADDR_0", 1, 2, 0, "19.0.0.1"}, 
    {"/System_Properties/PORT09/IP_VER", "IPV4_MASK_0", 1, 2, 0, "255.255.255.0"}, 
    {"/System_Properties/PORT09/IP_VER", "IPV4_BCAST_0", 1, 2, 0, "19.0.0.255"}, 
    {"/System_Properties/PORT09/IP_VER", "IPV4_GATE_0", 1, 2, 0, "19.0.0.254"}, 
    {"/System_Properties/PORT09/IP_VER", "NUM_VIRTUAL_IF_IPV6", 1, 1, 1, ""},
    {"/System_Properties/PORT09/IP_VER", "IPV6_ADDR_0", 1, 2, 0, "3ffe:0:200:f109::fe0d:0e0a"},
    {"/System_Properties/PORT09/IP_VER", "IPV6_PREFIX_LEN_0", 1, 1, 0, ""},
    {"/System_Properties/PORT09/IP_VER", "IPV6_GATE_0", 1, 2, 0, "3ffe:0:100:f102::fe0d:0e0f"},
    {"/System_Properties/PORT09", "MTU", 1, 1, 9216, ""},
    {"/System_Properties/PORT09", "LINK_SPEED", 1, 1, 1000, ""},
    {"/System_Properties/PORT09", "MAC_ADDR", 1, 2, 0, "0d0e0f0a0b0c"},
    {"/System_Properties/PORT09", "MEDIA_TYPE", 1, 2, 0, "1GB_ETHERNET"},

    /* System Application Properties */
    {"", "SystemApp", 0, 0, 0, ""},
    {"/SystemApp", "FREELISTS", 0, 0, 0, ""},
    {"/SystemApp/FREELISTS", "FL_MSG", 0, 0, 0, ""},
    {"/SystemApp/FREELISTS/FL_MSG", "ELEMENT_COUNT", 1, 1, 256, ""},
    {"/SystemApp/FREELISTS/FL_MSG", "META_SIZE", 1, 1, 64, ""},
    {"/SystemApp/FREELISTS/FL_MSG", "DATA_SIZE", 1, 1, 128, ""},

    /* Hardware freelist 1 */
    {"/SystemApp/FREELISTS", "FL_01", 0, 0, 0, ""},
    {"/SystemApp/FREELISTS/FL_01", "ELEMENT_COUNT", 1, 1, 16384, ""},
    {"/SystemApp/FREELISTS/FL_01", "SRAM_SIZE", 1, 1, sizeof(ix_hw_buffer_meta), ""},
    {"/SystemApp/FREELISTS/FL_01", "SRAM_CHAN", 1, 1, 0, ""},
    {"/SystemApp/FREELISTS/FL_01", "DRAM_SIZE", 1, 1, 2048, ""},
    {"/SystemApp/FREELISTS/FL_01", "DRAM_CHAN", 1, 1, 0, ""},

    /* Scratch rings */
    {"/SystemApp", "SCRATCH_RINGS", 0, 0, 0, ""},

    /* SCR ring for POS_RX_TO_IPV4_SCR_RING */
    {"/SystemApp/SCRATCH_RINGS", "SR_01", 0, 0, 0, ""},
    {"/SystemApp/SCRATCH_RINGS/SR_01", "ID", 1, 1, POS_RX_TO_IPV4_SCR_RING, ""},
    {"/SystemApp/SCRATCH_RINGS/SR_01", "CHAN", 1, 1, 0, ""},
    {"/SystemApp/SCRATCH_RINGS/SR_01", "ELEMENT_SIZE", 1, 1, 2, ""},

    /* SCR ring for IPV4_TO_QM_SCR_RING */
    {"/SystemApp/SCRATCH_RINGS", "SR_02", 0, 0, 0, ""},
    {"/SystemApp/SCRATCH_RINGS/SR_02", "ID", 1, 1, IPV4_TO_QM_SCR_RING, ""},
    {"/SystemApp/SCRATCH_RINGS/SR_02", "CHAN", 1, 1, 0, ""},
    {"/SystemApp/SCRATCH_RINGS/SR_02", "ELEMENT_SIZE", 1, 1, 2, ""},

    /* SCR ring for core-to-microblock */
    {"/SystemApp/SCRATCH_RINGS", "SR_03", 0, 0, 0, ""}, 
    {"/SystemApp/SCRATCH_RINGS/SR_03", "ID", 1, 1, IX_CORE_TO_UBLOCK_PACKET_CTRL_RING_ID, ""}, 
    {"/SystemApp/SCRATCH_RINGS/SR_03", "CHAN", 1, 1, 0, ""}, 
    {"/SystemApp/SCRATCH_RINGS/SR_03", "ELEMENT_SIZE", 1, 1, 0, ""}, 

    /* Execution engines properties */
    {"/SystemApp", "SA_EXEC_ENGINES", 0, 0, 0, ""},
    {"/SystemApp/SA_EXEC_ENGINES", "SA_EE_01", 1, 1, 1, ""},

    /* List all CC IDs defined in bindings.h for being included in this application (should be using the defined values) */ 
    {"/SystemApp/SA_EXEC_ENGINES/SA_EE_01", "CC_LIST", 1, 2, 0, "1 0 5 6 7 9 14 15"},

    /* List all microengines to be started for this application */
    {"/SystemApp", "MICROENGINES", 0, 0, 0, ""}, 
    {"/SystemApp/MICROENGINES", "ME_13", 1, 1, 13, ""}, 
    {"/SystemApp/MICROENGINES/ME_13", "CONTEXT_MASK", 1, 1, 255, ""}, 
    {"/SystemApp/MICROENGINES", "ME_12", 1, 1, 12, ""}, 
    {"/SystemApp/MICROENGINES/ME_12", "CONTEXT_MASK", 1, 1, 255, ""}, 
    {"/SystemApp/MICROENGINES", "ME_11", 1, 1, 11, ""}, 
    {"/SystemApp/MICROENGINES/ME_11", "CONTEXT_MASK", 1, 1, 255, ""}, 
    {"/SystemApp/MICROENGINES", "ME_10", 1, 1, 10, ""}, 
    {"/SystemApp/MICROENGINES/ME_10", "CONTEXT_MASK", 1, 1, 255, ""}, 
    {"/SystemApp/MICROENGINES", "ME_09", 1, 1, 9, ""}, 
    {"/SystemApp/MICROENGINES/ME_09", "CONTEXT_MASK", 1, 1, 255, ""},
    {"/SystemApp/MICROENGINES", "ME_08", 1, 1, 8, ""}, 
    {"/SystemApp/MICROENGINES/ME_08", "CONTEXT_MASK", 1, 1, 255, ""},
    {"/SystemApp/MICROENGINES", "ME_07", 1, 1, 7, ""}, 
    {"/SystemApp/MICROENGINES/ME_07", "CONTEXT_MASK", 1, 1, 255, ""}, 
    {"/SystemApp/MICROENGINES", "ME_06", 1, 1, 6, ""}, 
    {"/SystemApp/MICROENGINES/ME_06", "CONTEXT_MASK", 1, 1, 255, ""}, 
    {"/SystemApp/MICROENGINES", "ME_05", 1, 1, 5, ""}, 
    {"/SystemApp/MICROENGINES/ME_05", "CONTEXT_MASK", 1, 1, 255, ""}, 
    {"/SystemApp/MICROENGINES", "ME_02", 1, 1, 2, ""}, 
    {"/SystemApp/MICROENGINES/ME_02", "CONTEXT_MASK", 1, 1, 255, ""}, 
    {"/SystemApp/MICROENGINES", "ME_01", 1, 1, 1, ""}, 
    {"/SystemApp/MICROENGINES/ME_01", "CONTEXT_MASK", 1, 1, 255, ""}, 
    {"/SystemApp/MICROENGINES", "ME_00", 1, 1, 0, ""}, 
    {"/SystemApp/MICROENGINES/ME_00", "CONTEXT_MASK", 1, 1, 255, ""}, 

    /* IPv4 properties */
    {"", "IPV4", 0, 0, 0, ""},
    {"/IPV4", "ROUTE_TABLE_SIZE_HINT", 1, 1, 2048, ""},
    {"/IPV4", "ROUTE_TABLE_TYPE", 1, 1, 0, ""},
    {"/IPV4", "RTM_LOOKUP_MEMORY_TYPE", 1, 1, 1, ""},
    {"/IPV4", "RTM_LOOKUP_CHANNEL", 1, 1, 1, ""},
    {"/IPV4", "NEXT_HOP_TABLE_SIZE_HINT", 1, 1, 256, ""},
    {"/IPV4", "NEXT_HOP_MEMORY_TYPE", 1, 1, 1, ""},
    {"/IPV4", "NEXT_HOP_DATABASE_CHANNEL", 1, 1, 2, ""},
    {"/IPV4", "ME_FOR_IPV4", 1, 1, 12295, ""},

    /* IPv6 properties */
    {"", "IPV6", 0, 0, 0, ""},
    {"/IPV6", "ROUTE_TABLE_SIZE_HINT", 1, 1, 2048, ""},
    {"/IPV6", "ROUTE_TABLE_TYPE", 1, 1, 0, ""},
    {"/IPV6", "RTM_LOOKUP_MEMORY_TYPE", 1, 1, 1, ""},
    {"/IPV6", "RTM_LOOKUP_CHANNEL", 1, 1, 2, ""},
    {"/IPV6", "RTM_LOOKUP_CHANNEL_16BIT_TRIE", 1, 1, 2, ""},
    {"/IPV6", "NEXT_HOP_TABLE_SIZE_HINT", 1, 1, 256, ""},
    {"/IPV6", "NEXT_HOP_MEMORY_TYPE", 1, 1, 1, ""},
    {"/IPV6", "NEXT_HOP_DATABASE_CHANNEL", 1, 1, 2, ""},
    {"/IPV6", "ME_FOR_IPV6", 1, 1, 12295, ""}, /* ME# 0, 1, 2, 12, 13 */
    {"/IPV6", "NUM_DAD_XMIT", 1, 1, 2, ""},

    /* V6V4 Tunnel properties */
    {"", "V6V4", 0, 0, 0, ""},
    {"/V6V4", "END_TUNNEL_SIZE", 1, 1, 128, ""},
    {"/V6V4", "START_TUNNEL_SIZE", 1, 1, 256, ""},
    {"/V6V4", "INGRESS_LIST_TYPE", 1, 1, 12, ""},
    {"/V6V4", "INGRESS_LIST_SIZE_HINT", 1, 1, 256, ""},
    {"/V6V4", "END_TUNNEL_MEMORY_TYPE", 1, 1, 1, ""},
    {"/V6V4", "START_TUNNEL_MEMORY_TYPE", 1, 1, 1, ""},
    {"/V6V4", "INGRESS_LIST_MEMORY_TYPE", 1, 1, 1, ""},
    {"/V6V4", "END_TUNNEL_CHANNEL", 1, 1, 2, ""},
    {"/V6V4", "START_TUNNEL_CHANNEL", 1, 1, 2, ""},
    {"/V6V4", "INGRESS_LIST_CHANNEL", 1, 1, 1, ""},
    {"/V6V4", "ME_FOR_V6V4", 1, 1, 12295, ""}, /* ME# 0, 1, 2, 12, 13 */

    /* NATPT properties */
    {"", "NATPT", 0, 0, 0, ""},
    {"/NATPT", "NAT_TABLE_SIZE", 1, 1, 256, ""},
    {"/NATPT", "NAT_HASH_INDEX_TABLE_SIZE", 1, 1, 128, ""},
    {"/NATPT", "NAT_COLLISION_TABLE_SIZE", 1, 1, 128, ""},
    {"/NATPT", "ME_FOR_NATPT", 1, 1, 102, ""},
    {"/NATPT", "NATPT_PREFIX_LW0", 1, 1, 0, ""},
    {"/NATPT", "NATPT_PREFIX_LW1", 1, 1, 0, ""},
    {"/NATPT", "NATPT_PREFIX_LW2", 1, 1, 65535, ""},

    /* CSIX Tx properties */
    {"", "CSIX_TX", 0, 0, 0, ""},
    {"/CSIX_TX", "CSIX_TX_INGRESS_BLADE_ID", 1, 1, 1, ""},
    {"/CSIX_TX", "CSIX_TX_MEV2_NUM_MASK", 1, 1, 256, ""}, /* symbol patching for ME# 8 */

    /* Ethernet Rx properties */
    {"", "ETH_RX", 0, 0, 0, ""},
    {"/ETH_RX", "ETH_RX_MEV2_NUM_MASK", 1, 1, 3072, ""},
    {"/ETH_RX", "ETH_RX_FLT_TABLE_SIZE", 1, 1, 4096, ""},
    {"/ETH_RX", "L2_DECAP_MEV2_NUM_MASK", 1, 1, 12295, ""},
    {"/ETH_RX", "L2_DECAP_PORT_MODE_BIT_MAP", 1, 1, -1, ""},

    /* QM properties */
    {"", "QM", 0, 0, 0, ""},
    {"/QM", "QM_UENG_NUM", 1, 1, 7, ""},
    {"/QM", "QUEUE_DESC_TOTAL", 1, 1, 1024, ""},
    {"/QM", "QM_UENG_MASK", 1, 1, 128, ""}, /* ME 7 */

    /* Scheduler properties */
    {"", "SCHED", 0, 0, 0, ""},
    {"/SCHED", "SCHEDULER_TYPE", 1, 1, 0, ""},
    {"/SCHED", "SCHED_UENG_NUM", 1, 1, 6, ""},
    {"/SCHED", "NUMBER_OF_PORTS", 1, 1, 64, ""},
    {"/SCHED", "VOQ_WEIGHTS_PER_PORT", 1, 1, 1, ""},
    {"/SCHED", "VOQ_WEIGHT_VALUE", 1, 1, 1, ""},
    {"/SCHED", "PKT_WEIGHT_VALUE", 1, 1, 1, ""},
    {"/SCHED", "QUEUE_PORT_SIZE", 1, 1, 256, ""},

    /* Array is terminated with the follwoing entry */
    {"", "", 0, 0, 0, ""}};

#endif /* !defined(__IX_SA_INTERNAL_REGISTRY_DATA_H__) */
