/*===========================================================================
 * = COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 * ===========================================================================
 */

/* =========================================================================*/
/**
 * @file   ix_sa_cc_list.c
 *
 * Definition of CC_INITIALIZATION funciton.
 *
 **/
/*=========================================================================*/
/* $Id: ix_sa_cc_list.c,v 1.10 2003/10/27 23:06:13 ktseng Exp $ */

#include "ix_cc_error.h"
#include "sa/ix_sa_cc_list.h"

/* All CC includes here */
#include "cc/ix_cc_stkdrv.h"
#include "cc/ix_cc_ipv4.h"
#include "cc/ix_cc_ipv6.h"

#include "cc/ix_cc_ipv6.h"
#include "cc/ix_cc_v6v4.h"
#include "cc/ix_cc_natpt.h"

#include "cc/ix_cc_qm_frmwrk.h"
#include "cc/ix_cc_csix_tx.h"
#include "cc/ix_cc_sched.h"
#include "cc/ix_cc_eth_rx.h"

#ifdef INCLUDE_PDK
#include "cc/ix_cc_fpm_core.h"
#endif

#if !defined(IX_CONFIGURATION_oc192_ethernet_ipv6_ingress)

#error IX_CONFIGURATION_oc192_ethernet_ipv6_ingress not defined! Wrong configuration header.

#endif

/** 
 * Define the directory path where the uof file is located.
 * The base directoy specified by the environmental variable
 * IXA_SDK_DEV will be inserted by the build script.
 */

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
char g_sa_uofFile[] = "10gb_ethernet_ixp2800.uof";
#else
char g_sa_uofFile[] = "src\\applications\\ipv4_v6_forwarder\\10gb_ethernet\\10x1GbE_ingress\\wbench_project\\10gb_ethernet_ixp2800.uof";
#endif

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
/*  _ix_sa_set_cc */
/*  ---------------------------------------- */
/**
 * This function is used to set a init and fini function for a CC
 *
 * @return IX_SUCCESS upon success or an ix_error with one of the
 *         following values.
 * @retval NONE
**/
ix_error
_ix_sa_set_cc(ix_sa_ccfunc *arg_pCC_List, ix_uint32 arg_idx,
              ix_cc_init arg_Init, ix_cc_fini arg_Fini)
{
    ix_uint32 i;

    IX_ERROR_CHECK_ARG_NULL(1, arg_pCC_List);
    arg_pCC_List[arg_idx].init = arg_Init;
    arg_pCC_List[arg_idx].fini = arg_Fini;
    for (i = 0; i < IX_CC_INSTANCE_MAX; i++)
    {
        arg_pCC_List[arg_idx].cc_handle[i] = 0;
    }
    return IX_SUCCESS;
}
#endif /* (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_) */

/*  NAME: _ix_sa_cc_list */
/**
 * This function generates and returns the ix_sa_ccfunc array.
 *
 * An array containing pointers to various cc functions is returned
 * to be used during cc initialization.  
 *
 * @return IX_SUCCESS, a pass through ix_error, or an ix_error with one
 *         of the following values.
 * @retval IX_CC_ERROR_NULL arg_pCC_List is NULL
**/
ix_error
_ix_sa_cc_list(ix_sa_ccfunc *arg_pCC_List)
{
    IX_ERROR_CHECK_ARG_NULL(1, arg_pCC_List);

    /* Add one entry for each CC in this configuration.  
     * The format of the _ix_sa_set_cc function is as follows:
     *   Param #1: The cclist passed into this function.
     *   Param #2: Identifier of the core component as defined in bindings.h
     *   Param #3: Init function for the core component.
     *   Param #4: Finish function for the core component.
     *
     * _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_DUMMY, _ix_sa_dummy_cc_init, _ix_sa_dummy_cc_fini);
    */


    /* Stack Driver */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_STKDRV, ix_cc_stkdrv_init, ix_cc_stkdrv_fini);

    /* Queue Manager*/
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_QM, ix_cc_qm_init, ix_cc_qm_fini);

    /* IPV4 Forwarder */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_IPV4, ix_cc_ipv4_init, ix_cc_ipv4_fini);

    /* CSIX Tx */
   _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_CSIX_TX, ix_cc_csix_tx_init, ix_cc_csix_tx_fini);

    /* Scheduler */	
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_SCHEDULER, ix_cc_sched_init, ix_cc_sched_fini);

    /* Ethernet Rx */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_ETH_RX, ix_cc_eth_rx_init, ix_cc_eth_rx_fini);

    /* IPV6 Forwarder */
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_IPV6, ix_cc_ipv6_init, ix_cc_ipv6_fini);

    /* IPv4-IPv6 Tunneling Core Component */
    _ix_sa_set_cc(arg_pCC_List, 
                  IX_CC_V6V4, ix_cc_v6v4_init, ix_cc_v6v4_fini);
#if 0
    /* IPv4-IPv6 Translation Core Component */
    _ix_sa_set_cc(arg_pCC_List, 
                  IX_CC_NATPT, ix_cc_natpt_init, ix_cc_natpt_fini);
#endif
    /* FPM CC */
#ifdef INCLUDE_PDK
    _ix_sa_set_cc(arg_pCC_List,
                  IX_CC_FPM, ix_cc_fpm_init, ix_cc_fpm_fini);
#endif

    /* Additional Core Components here ... */

    return IX_SUCCESS;


}
