//------------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//------------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: 	pkthdr_cache.uc
//
//      Purpose: 
//					Contains IP Header Caching Macros.
//
//      History:
//
//
//      Date            Comment                         By
//      ---------------------------------------------------------------------
//
//      11/19/2001      Created                         Senthil Nathan
//
//
/////////////////////////////////////////////////////////////////////////////

#ifndef __PKTHDR_CACHE_UC__
#define __PKTHDR_CACHE_UC__

#include "definitions.h"
#include "ether.h"

#define_eval PKT_HDR_CACHE_BYTES  (PKT_HDR_CACHE_SIZE * 4)  // 64 bytes
#define_eval PKT_HDR_DRAM_RD_SIZE (PKT_HDR_CACHE_SIZE >> 1) // read 8 quadwords

// Base address of local memory buffers used for caching the IP header.

#define LM_PKTHDR_BASE             0

// Context relative offsets (in bytes) of protocol header buffers.

#define LM_BUF_OFFSET_IPV4_HDR     0
#define LM_BUF_OFFSET_IPV6_HDR     64
#define LM_BUF_OFFSET_IPV6_HDR_1   64
#define LM_BUF_OFFSET_IPV6_HDR_2   96

// Buffer sizes of header buffers in 32-bit words

#define LM_BUF_LWSIZE_IPV4_HDR	   8
#define LM_BUF_LWSIZE_IPV6_HDR     16
#define LM_BUF_LWSIZE_IPV6_HDR_2   8
#define LM_BUF_LWSIZE_IPV6_HDR_1   8

// Buffer sizes of header buffers in bytes

#define_eval LM_BUF_SIZE_IPV4_HDR      (LM_BUF_LWSIZE_IPV4_HDR * 4)
#define_eval LM_BUF_SIZE_IPV6_HDR      (LM_BUF_LWSIZE_IPV6_HDR * 4)
#define_eval LM_BUF_SIZE_IPV6_HDR_1    (LM_BUF_LWSIZE_IPV6_HDR_2 * 4)
#define_eval LM_BUF_SIZE_IPV6_HDR_2    (LM_BUF_LWSIZE_IPV6_HDR_1 * 4)

// Total pool size in bytes

#define_eval LM_POOL_SIZE (LM_BUF_OFFSET_IPV6_HDR + LM_BUF_SIZE_IPV6_HDR)

// Total number of bytes to cache for each header type.  We always cache at
// least 6 extra bytes to allow for alignment when writing back to DRAM.

#define LM_XFER_BYTES_IPV4          26
#define LM_XFER_BYTES_IPV6          46
#define LM_XFER_BYTES_IPV6_1        32
#define LM_XFER_BYTES_IPV6_2        14

// Actual header sizes in and in bytes

#define PKT_HDR_BYTES_ETHER         14
#define PKT_HDR_BYTES_IPV4          20
#define PKT_HDR_BYTES_IPV6          40
#define PKT_HDR_BYTES_IPV6_1        32
#define PKT_HDR_BYTES_IPV6_2        8

// Names of local memory buffers.

#define LM_IPV4_HDR            lmem_buf0
#define LM_IPV6_HDR            lmem_buf1
#define LM_IPV6_HDR_1          lmem_buf2
#define LM_IPV6_HDR_2          lmem_buf3

//NAT-PT requires an area in Local memory to be defined. This is used by nat-pt
//as a work buffer
#define LM_NATPT_WORK_BUF lmem_buf4
#define LM_NATPT_WORK_BUF_BASE 1024
#define LM_NATPT_WORK_BUF_SIZE 64
#define_eval LM_BUF_LWSIZE_NATPT_WORK (LM_NATPT_WORK_BUF_SIZE>>2)
#define LM_BUF_OFFSET_NATPT_WORK_BUF 0


// Tunnel header type bindings.

#define      V6V4_HDR_TYPE_DECAP     ETHER_IPV6_TYPE
#define      V6V4_HDR_TYPE_ENCAP     TUNNEL_ENCAP_FLAG

#define_eval ETHER_IPV6_IN_IPV4_TYPE (ETHER_IPV6_TYPE | V6V4_HDR_TYPE_ENCAP)

//Stores the offset to the very start of the packet. i.e the buffer headroom
//set aside when the packet was loaded into DRAM
//Note that the buffer headroom varies per packet (for better DRAM bank
//utilization) so we cannot have a hardcoded value for it
.reg volatile buffer_headroom

//We are able to avoid some trips to the DRAM when the buffer is loaded for
//the first time. This flag is used to indicate whether the buffer has been
//reloaded or not.
.reg volatile buffer_first_load

///////////////////////////////////////////////////////////////////////////////
// dl_load_iphdr_cache:
//	 	Description: 
//			Reads the initial portion of the packet header from DRAM inot 
//			DRAM transfer registers. 
//
//	 	Outputs:
//			The $$iphdr# global dram transfer registers will be set with appropriate
//			header data.
//
//		Inputs:
//			buf_handle	:			Handle of buffer.
//			req_sig		:			Signal to use in the I/O (DRAM read) operation.
//			sig_action	:			What to do with I/O operation - 
//									- wait for signal(s) as specified by sig_action
//									- do not wait for signal, just return (SIG_NONE)
//		Size:   				:
//
///////////////////////////////////////////////////////////////////////////////
#define	dl_load_iphdr_cache		dl_iphdr_load_cache

#macro dl_iphdr_load_cache[buf_handle, req_sig, sig_action]
.begin
.reg 	sdram_offset, buf_offset

	//	Read initial portion of packet header from DRAM

	dl_buf_get_data[sdram_offset, buf_handle]
	dl_meta_get_offset[buf_offset]

	alu[buffer_headroom, --, B, buf_offset] ;store the headroom in buffer for later
	alu[buffer_first_load, --, B, 1] ;flag used by dl_get_bytes
	dram[read, $$iphdr[0], sdram_offset, buf_offset, PKT_HDR_DRAM_RD_SIZE], sig_done[req_sig]	; DRAM reads are in QWORD (8 bytes)

#if (!streq('sig_action', 'SIG_NONE'))
	ctx_arb[sig_action]
#endif

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_iphdr_cache_init:
//	 	Description: 
//			This macro needs to be called before any of the dl_iphdr_* can be called.
//			It allocates a set of DRAM xfer registers to be used for caching. This macro
//			doesn't generate any code (instructions). It only produces some assembler
//			directives.
//
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//			0 Instructions
///////////////////////////////////////////////////////////////////////////////
#macro dl_iphdr_cache_init[]

	xbuf_alloc[$$iphdr, PKT_HDR_CACHE_SIZE, read_write]

    // Suppres assembler warnings about transfer registers used without being
	// set.
    .set $$iphdr[0], $$iphdr[1], $$iphdr[2], $$iphdr[3], $$iphdr[4], $$iphdr[5], $$iphdr[6]
	.set $$iphdr[7], $$iphdr[8], $$iphdr[9], $$iphdr[10], $$iphdr[11], $$iphdr[12]
	.set $$iphdr[13], $$iphdr[14], $$iphdr[15]

    // Set up local memory cache buffers.

	xbuf_alloc(LM_IPV4_HDR, LM_BUF_LWSIZE_IPV4_HDR, read_write)
	xbuf_alloc(LM_IPV6_HDR, LM_BUF_LWSIZE_IPV6_HDR, read_write)
	xbuf_alloc(LM_IPV6_HDR_1, LM_BUF_LWSIZE_IPV6_HDR_1, read_write)
	xbuf_alloc(LM_IPV6_HDR_2, LM_BUF_LWSIZE_IPV6_HDR_2, read_write)

	xbuf_bind_address(LM_IPV4_HDR, LM_PKTHDR_BASE,
	                  LM_POOL_SIZE, LM_BUF_OFFSET_IPV4_HDR)

	xbuf_bind_address(LM_IPV6_HDR, LM_PKTHDR_BASE,
	                  LM_POOL_SIZE, LM_BUF_OFFSET_IPV6_HDR)

	xbuf_bind_address(LM_IPV6_HDR_1, LM_PKTHDR_BASE,
	                  LM_POOL_SIZE, LM_BUF_OFFSET_IPV6_HDR_1)

	xbuf_bind_address(LM_IPV6_HDR_2, LM_PKTHDR_BASE,
	                  LM_POOL_SIZE, LM_BUF_OFFSET_IPV6_HDR_2)

    // Set thread ID in meta-data for use in xbuf_activate.

    .begin
	    .reg context_id

	    local_csr_rd[ACTIVE_CTX_STS]
	    immed[context_id, 0]
	    alu[context_id, context_id, AND, 0x7]

	    dl_set_thread_id(context_id)
    .end


#endm

///////////////////////////////////////////////////////////////////////////////
// dl_iphdr_cache_fini:
//	 	Description: 
//			This macro needs to be called at the end when dl_iphdr_macros are no longer
//			required. It frees a set of DRAM xfer registers that were allocated using
//			dl_iphdr_cache_init. This macro	doesn't generate any code (instructions). 
//			It only produces some assembler	directives.
//
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//			0 Instructions

#macro dl_iphdr_cache_fini[]

	xbuf_free[$$iphdr]

	xbuf_free(LM_IPV4_HDR)
	xbuf_free(LM_IPV6_HDR)
	xbuf_free(LM_IPV6_HDR_1)
	xbuf_free(LM_IPV6_HDR_2)

#endm

////////////////////////////////////////////////////////////////////////////////
// dl_copy_iphdr_to_lmem:
//
//      description:   
//
//			This macro copies the IP header to an aligned location in local
//          memory.  The IP header can be either an IPv4 or IPv6 header.
//
//	 	Outputs:
//          - IP header is copied to local memory cache buffer.
//
//		Inputs:
//			Header is currently in transfer registers.
//
////////////////////////////////////////////////////////////////////////////////
#macro dl_copy_iphdr_to_lmem()
.begin

    .reg hdr_type
	.reg context_id

    // Get thread ID for xbuf_activate.

	dl_get_thread_id(context_id)

	dl_meta_get_header_type(hdr_type)

    // Copy the IP header into the appropriate local memory buffer.  We
    // copy some additional bytes to allow for alignment when writing the
	// header back to DRAM.

    .if (hdr_type == ETHER_IPV6_TYPE)

        // Activate the IPv6 buffer and then copy the IPv6 header
        // into the local memory buffer.  We are limited to 32 bytes
		// per local memory buffer copy, so we must copy in two phases.

        xbuf_activate(LM_IPV6_HDR_2, 1, context_id, 0)
        xbuf_activate(LM_IPV6_HDR_1, 0, context_id, 1)

        xbuf_copy(LM_IPV6_HDR_1, 0, 0, $$iphdr, PKT_HDR_BYTES_ETHER,
                  0, LM_XFER_BYTES_IPV6_1, 0)

        #define_eval PART2_START (PKT_HDR_BYTES_ETHER+LM_XFER_BYTES_IPV6_1)

        xbuf_copy(LM_IPV6_HDR_2, LM_XFER_BYTES_IPV6_1, 0, $$iphdr,
                  PART2_START, 0, LM_XFER_BYTES_IPV6_2, 0)

	    xbuf_deactivate(LM_IPV6_HDR_1)
	    xbuf_deactivate(LM_IPV6_HDR_2)

        #undef PART2_START

    .elif (hdr_type == ETHER_IPV4_TYPE)

        // Activate the IPv4 buffer and then copy the IPv4 header
        // into the local memory buffer.

        xbuf_activate(LM_IPV4_HDR, 0, context_id, 1)

        xbuf_copy(LM_IPV4_HDR, 0, 0, $$iphdr, PKT_HDR_BYTES_ETHER,
                  0, LM_XFER_BYTES_IPV4, 0)

	    xbuf_deactivate(LM_IPV4_HDR)

        // Set next hop ID to -1 for the IPv4 forwarder.
        .begin
            .reg nh_id
            immed[nh_id, 0xffff]
            dl_meta_set_nexthop_id(nh_id)
        .end

    .endif

    // If the packet is neither IPv6 nor IPv4, it will be dropped so
    // we don't need to do anything.

copy_cache_done#:

.end

#endm


//////////////////////////////////////////////////////////////////////////////
// dl_load_ipv6_hdr
//     Description: Loads the IPv6 header.  This macro is executed by the
//                  tunnel decapsulation block when it needs to decapsulate a
//                  packet.
//                     
// Output:
//     xbuf_name      : Name of local memory buffer to receive the header.
//     xbuf_offset    : Offset in xbuf_name at which header should begin.
//     HDR_BYTE_COUNT : Number of bytes in the header.
//
// Inputs:
//
//
////////////////////////////////////////////////////////////////////////////////
#macro dl_load_ipv6_hdr(xbuf_name, xbuf_offset, HDR_BYTE_COUNT)
.begin
    .reg dram_offset
	.reg buf_offset
	.reg context_id
    .sig read_sig

#if ( !streq(xbuf_name, 'lmem_buf1') )
    #error "Memory buffer name for IPv6 is incorrect"
#endif

#if ( !isnum(xbuf_offset) )
    #error "Memory offset for IPv6 is incorrect"
#elif (xbuf_offset != 0)
    #error "xbuf_offset must be zero"
#endif

#if ( !isnum(HDR_BYTE_COUNT) )
    #error "Byte count for IPv6 header is incorrect"
#elif (HDR_BYTE_COUNT != PKT_HDR_BYTES_IPV6)
    #error "Byte count for IPv6 header is incorrect"
#endif

    // xbuf_copy allows only 32 bytes per local memory copy, so we must
	// do the IPv6 copy in 2 pieces.

	dl_get_thread_id(context_id)

    xbuf_activate(LM_IPV6_HDR_1, 0, context_id, 0)
    xbuf_activate(LM_IPV6_HDR_2, 1, context_id, 0)

	// Get the offset to the start of the current packet buffer from
	// the meta-data.

	dl_buf_get_data[dram_offset, dl_buf_handle]

	// Get offset from the start of the packet buffer to the
	// place where the actual packet has been placed. It should point just
	// beyond the ethernet header, which was removed by _ether_decap.
	// The first 30 bytes of the IPv6 header are already in the transfer
	// registers, so we need to read 16 more bytes at offset
	// 64 - PKT_HDR_BYTES_ETHER.

	#define_eval READ_OFFSET (PKT_HDR_CACHE_BYTES - PKT_HDR_BYTES_ETHER)

	dl_meta_get_offset[buf_offset]
	alu[buf_offset, buf_offset, +, READ_OFFSET]

	#undef READ_OFFSET

	// Copy the first 30 bytes of the IPv6 header to the local memory byffer.

	#define_eval IPV6_START_BYTE  (PKT_HDR_BYTES_ETHER + PKT_HDR_BYTES_IPV4)
	#define_eval BYTES_TO_COPY    (PKT_HDR_CACHE_BYTES - IPV6_START_BYTE)
	#define_eval BYTES_REMAINING  (LM_XFER_BYTES_IPV6 - BYTES_TO_COPY)

	xbuf_copy(LM_IPV6_HDR_1, 0, 0, $$iphdr, IPV6_START_BYTE, 0,
	          BYTES_TO_COPY, 0)

	// QW_COUNT is the maximum number of quad-words to read to make sure
	// we have all the header bytes required.

    #define_eval QW_COUNT ((BYTES_REMAINING + 7) / 8)

	dram[read, $$iphdr[0], dram_offset, buf_offset, QW_COUNT],
	    sig_done[read_sig]	

	ctx_arb[read_sig]

	// Append the final two bytes to the first 32-byte portion of the IPv6
	// header.

	ld_field[LM_IPV6_HDR_1[7], 0011, $$iphdr[0], >>16]

    // Copy the remaining bytes to the local memory buffer.

	#define_eval BYTES_REMAINING    (BYTES_REMAINING - 2)

    xbuf_copy(LM_IPV6_HDR_2, 0, 0, $$iphdr, 2, 0, BYTES_REMAINING, 0)

	xbuf_deactivate(LM_IPV6_HDR_1)
	xbuf_deactivate(LM_IPV6_HDR_1)

	#undef QW_COUNT
    #undef BYTES_REMAINING
	#undef BYTES_TO_COPY
	#undef IPV6_START_BYTE
.end
#endm

#macro dl_get_bytes(out_buf,\
					in_start_offset_in_header,\
					IN_BYTES_TO_READ,\
					in_req_sig,\
					in_sig_action)
.begin
	.reg rem_bytes
	.reg begin_offset
	.reg dest_offset
	.reg sdram_base //Address in DRAM where packet begins
	.reg qw_to_read
    .reg actual_dest_offset
    .reg actual_qw_to_read

	//First we check if the requested bytes are in transfer registers
	//already. If not, issue a DRAM read.
	;find the max requested offset
	alu[rem_bytes, --, B, IN_BYTES_TO_READ]
	alu[begin_offset, --, B, in_start_offset_in_header]
	alu[dest_offset, --, B, 0]

    //If the buffer $$iphdr was reloaded by someone, we do not know the
    //offset of the first byte loaded into the buffer(w.r.t the packet)
    //So we cannot use the below optimization. Simply read directly from DRAM
    .if (buffer_first_load == 1)
    	.if (in_start_offset_in_header < PKT_HDR_CACHE_BYTES)
    	  .begin
    		.reg end_offset

    		 alu[end_offset, in_start_offset_in_header, +, IN_BYTES_TO_READ]
    		alu[--, end_offset, -, PKT_HDR_CACHE_BYTES]
    		bge[LBL_PARTIAL_BUF#]
    			//We have already read the requested bytes
    			//Simply copy off into buffer passed in
    			alu[rem_bytes, --, B, IN_BYTES_TO_READ]
    			br[LBL_PAST_PARTIAL_BUF#]

    LBL_PARTIAL_BUF#:
    			//We have already read *some* of the requested bytes
    			//Copy these off into the buffer passed in
    			alu[rem_bytes, PKT_HDR_CACHE_BYTES, -, in_start_offset_in_header]

    LBL_PAST_PARTIAL_BUF#:
    		xbuf_copy(out_buf, 0, 0, $$iphdr, in_start_offset_in_header,
    				  0, rem_bytes, 0)

    		alu[--, end_offset, -, PKT_HDR_CACHE_BYTES]
    		ble[LBL_PAST_DRAM_READ#]

    			alu[begin_offset, --, B, PKT_HDR_CACHE_BYTES]
    			alu[dest_offset, --, B, rem_bytes]
    			alu[rem_bytes, IN_BYTES_TO_READ, -, rem_bytes]

    	  .end
    	.endif
    .endif

	//Need to read in more bytes from DRAM. Get the DRAM base
	dl_buf_get_data[sdram_base, dl_buf_handle]


    //We are about to reload the buffer, so reset buffer_first_load
    alu[buffer_first_load, --, B, 0]

	//Account for the buffer headroom
	alu[begin_offset, buffer_headroom, +, begin_offset]

	//DRAM accesses are done in quad word units. So first we find number
	//of quad words to read.
	//Also, the DRAM reads have to be done from an offset aligned on a
	//Quad word boundary. Since the DRAM instruction ignores the lowest
	//3 bits in the address, we don't need to do anything to ensure this.
	//What this means though is that the value read from DRAM can contain
	//some extra bytes before (if the address is not quadword aligned)
	//and/or after (if IN_BYTES_TO_READ is not a multiple of 8) 
	//the chunk of data we need to read.
	//Hence we read from DRAM into a temporary buffer & then we copy off
	//the right number of bytes from the correct offset in this buffer into
	//the final buffer to return to the caller.

	.begin
		.reg offset_in_temp_buf
        .reg unaligned_bytes
        .reg in_prepend

		//Based on the initial offset we might have to read in one more
		//Quadword. For eg. if initial offset=3, in_bytes_to_read=8
		//we will need to read in 2 Quadwords, as the 8 bytes we need
		//straddles a quadword boundary
		alu[offset_in_temp_buf, begin_offset, AND, 7]
		alu[qw_to_read, rem_bytes, +, 7]
		alu[qw_to_read, offset_in_temp_buf, +, qw_to_read]
		alu_shf[qw_to_read, --, B, qw_to_read, >>3]


		#define_eval MAX_REFCOUNT 8

        alu[actual_dest_offset, --, B, dest_offset]
        alu[in_prepend, --, B, 0]

LBL_READ_SOME_MORE#:
        .if (qw_to_read > 8)
            alu[actual_qw_to_read, --, B, 8]
        .else
            alu[actual_qw_to_read, --, B, qw_to_read]
        .endif

		alu[actual_qw_to_read,actual_qw_to_read,OR,0x10] ;Set override bit for ref count
		alu_shf[--, --, B, actual_qw_to_read, <<21]

		dram[read, $$iphdr[0], begin_offset, sdram_base, max_/**/MAX_REFCOUNT],
			 indirect_ref, sig_done[in_req_sig]

		#if (!streq(in_sig_action, 'SIG_NONE'))
			ctx_arb[in_sig_action]
		#endif

        alu[unaligned_bytes, actual_dest_offset, AND, 0x11]
        .if (unaligned_bytes != 0)
            ;dest offset is not aligned, so we need to pass in in_prepend
            xbuf_extract(in_prepend, $$iphdr, 0, 0, unaligned_bytes)
        .else
            alu[in_prepend, --, B, 0]
        .endif

		xbuf_copy(out_buf, 0, actual_dest_offset, $$iphdr, offset_in_temp_buf,
                  in_prepend, rem_bytes, 0)

        .if (qw_to_read > 8)
            alu[qw_to_read, qw_to_read, -, 8]
            alu[rem_bytes, rem_bytes, -, 64]
            alu[actual_dest_offset, actual_dest_offset, +, 64]
            alu[begin_offset, begin_offset, +, 64]
            br[LBL_READ_SOME_MORE]
        .endif

		#undef MAX_REFCOUNT

LBL_PAST_DRAM_READ#:
	.end
.end
#endm

//Transfers from GPR/LM to DRAM xfer reg & then calls dl_put_quad_words
#macro dl_put_quad_words_from_gpr_lm(src_buf,\
					start_offset_in_buffer,\
					QW_TO_WRITE,\
					req_sig,\
					sig_action)
.begin
	#define_eval NUM_REG QW_TO_WRITE<<1
	#define_eval NUM_BYTES QW_TO_WRITE<<3

	xbuf_alloc($$xfer_buf, NUM_REG, write)
	xbuf_copy($$xfer_buf, 0, 0, src_buf, 0, 0, NUM_BYTES, 0)

	dl_put_quad_words($$xfer_buf0, start_offset_in_buffer, QW_TO_WRITE,
					req_sig, sig_action)
	#undef NUM_BYTES
	#undef NUM_REG

	xbuf_free($$xfer_buf)
.end
#endm

#macro dl_put_quad_words(src_buf,\
					in_start_offset_in_buffer,\
					QW_TO_WRITE,\
					req_sig,\
					sig_action)
.begin
	.reg buf_offset
	.reg sdram_offset

	//This macro directly writes to DRAM
	//Note that if the offset being written is within the Packet header
	//cache (setup by the initial read in dl_source), this macro will
	//leave it in an inconsistent state; since the DRAM where the actual packet
	//lies is updated by the macro, but the header cache is not refreshed.
	//This is considered ok, the assumption being that other ublocks in this
	//application do not need to use this same field.
	//If there were other ublocks that needed to read bytes at the same offset,
	//the application would have ensured that it would be
	//in the header cache in local memory. OR This macro could be modified to
	//refresh the cache in transfer registers by issuing a read after the
	//write.
	dl_buf_get_data[sdram_offset, dl_buf_handle]

	alu[buf_offset, buffer_headroom, +, in_start_offset_in_buffer]

	dram[write, src_buf, sdram_offset, buf_offset, QW_TO_WRITE],
		 sig_done[req_sig]

	#if (!streq('sig_action', 'SIG_NONE'))
		ctx_arb[sig_action]
	#endif
.end
#endm

////////////////////////////////////////////////////////////////////////////////
//
// _flush_headers[]
//
//	 	Description: 
//
//		Write the ethernet and IP header to DRAM, given a constant alignment
//      for the start of the ethernet header.
//
//	 	Outputs:
//
//			The ethernet and IP headers are written into DRAM
//
//		Inputs
//			buf_handle	: Handle of buffer.
//          hdr_offset  : Offset of layer 2 header in packet buffer.
//          ALIGN_OFFSET: Number of extra bytes that must precede the
//                        first header byte to achieve quad-word
//                        alignment (0 - 7).
//          req_sig     : Signal to use for write operation(s).
//          sig_action  : What to do with FINAL write operation -
//                        - wait for signal(s) as specified by sig_action
//                        - do not wait, just return (SIG_NONE)
//
////////////////////////////////////////////////////////////////////////////////
#macro	_flush_headers(buf_handle, hdr_offset, ALIGN_OFFSET, req_sig, sig_action)
.begin
	.reg last_ipv4     // used with xbuf copy for merging IPv4 & IPv6 hdrs
	.reg last_ipv6     // used with xbuf copy for merging portions of IPv6 hdr
	.reg context_id    // thread ID for use in xbuf_activate
	.reg hdr_type      // header type from meta-data
	.reg dram_offset   // DRAM offset of packet buffer

    dl_get_thread_id(context_id)
	dl_buf_get_data(dram_offset, buf_handle)
	dl_meta_get_header_type(hdr_type)


    // Flush the IP header.

    .if (hdr_type == ETHER_IPV4_TYPE)

        // The IPv4 header needs to be flushed.

        xbuf_activate(LM_IPV4_HDR, 0, context_id, 1)

        xbuf_copy($$iphdr, 0, ALIGN_OFFSET, LM_IPV4_HDR, 0, 0,
                  LM_XFER_BYTES_IPV4, 0)

        #define_eval QW_COUNT  ((ALIGN_OFFSET + PKT_HDR_BYTES_IPV4 + 7) / 8)
	    dram[write, $$iphdr[0], dram_offset, hdr_offset, QW_COUNT],
	        sig_done[req_sig]

        xbuf_deactivate(LM_IPV4_HDR)

    .elif (hdr_type == ETHER_IPV6_IN_IPV4_TYPE)

        xbuf_activate(LM_IPV6_HDR_1, 1, context_id, 0)
        xbuf_activate(LM_IPV4_HDR, 0, context_id, 1)

        // Tunneled packet needs IPv4 and IPv6 header to be flushed.
        // Copy the IPv4 header to the transfer registers.

        xbuf_copy($$iphdr, last_ipv4, ALIGN_OFFSET, LM_IPV4_HDR, 0, 0,
                  PKT_HDR_BYTES_IPV4, 0)

        // Copy as much as possible of the IPv6 header after the IPv4
	    // header.  The first 32 bytes are copies from the first part
		// of the IPv6 buffer, followed by as much as will fit from
		// the second.

        xbuf_activate(LM_IPV6_HDR_2, 0, context_id, 0)

        #define_eval IPV6_START_BYTE (ALIGN_OFFSET + PKT_HDR_BYTES_IPV4)
		#define_eval BYTES_TO_COPY   PKT_HDR_BYTES_IPV6_1

        xbuf_copy($$iphdr, last_ipv6, IPV6_START_BYTE, LM_IPV6_HDR_1,
		          0, last_ipv4, BYTES_TO_COPY, 0)

        #define_eval IPV6_START_BYTE (IPV6_START_BYTE + BYTES_TO_COPY)
	    #define_eval BYTES_TO_COPY (PKT_HDR_CACHE_BYTES - IPV6_START_BYTE)

        xbuf_copy($$iphdr, 0, IPV6_START_BYTE, LM_IPV6_HDR_2, 0,
		          last_ipv6, BYTES_TO_COPY, 0)

        #define_eval QW_COUNT  PKT_HDR_DRAM_RD_SIZE
	    dram[write, $$iphdr[0], dram_offset, hdr_offset, QW_COUNT],
	        sig_done[req_sig]

        #if (BYTES_TO_COPY < PKT_HDR_BYTES_IPV6_2)

            // Must wait here so we can re-use the transfer registers.

            ctx_arb[req_sig]

		    // If we didn't get all the bytes written, we need to write the
			// last portion of the IPv6 header.

            #define_eval IPV6_START_BYTE  BYTES_TO_COPY
	        #define_eval BYTES_TO_COPY (LM_XFER_BYTES_IPV6_2 - IPV6_START_BYTE)

            xbuf_copy($$iphdr, 0, 0, LM_IPV6_HDR_2, IPV6_START_BYTE, 0,
		              BYTES_TO_COPY, 0)

            alu[hdr_offset, hdr_offset, +, PKT_HDR_CACHE_BYTES]

            #define_eval QW_COUNT (((PKT_HDR_BYTES_IPV6_2-IPV6_START_BYTE)+7)/8)
	        dram[write, $$iphdr[0], dram_offset, hdr_offset, QW_COUNT],
	            sig_done[req_sig]

        #endif

        xbuf_deactivate(LM_IPV6_HDR_1)
        xbuf_deactivate(LM_IPV6_HDR_2)

    .else
        // The IPv6 header needs to be flushed.

        xbuf_activate(LM_IPV6_HDR_2, 1, context_id, 0)
        xbuf_activate(LM_IPV6_HDR_1, 0, context_id, 1)

        xbuf_copy($$iphdr, last_ipv6, ALIGN_OFFSET, LM_IPV6_HDR_1, 0, 0,
                  PKT_HDR_BYTES_IPV6_1, 0)

        #define_eval IPV6_START_BYTE (ALIGN_OFFSET + PKT_HDR_BYTES_IPV6_1)
	    #define_eval BYTES_TO_COPY   LM_XFER_BYTES_IPV6_2

        xbuf_copy($$iphdr, 0, IPV6_START_BYTE,
		          LM_IPV6_HDR_2, 0, last_ipv6, BYTES_TO_COPY, 0)

        #define_eval QW_COUNT  ((ALIGN_OFFSET + PKT_HDR_BYTES_IPV6 + 7) / 8)
	    dram[write, $$iphdr[0], dram_offset, hdr_offset, QW_COUNT],
	        sig_done[req_sig]

        xbuf_deactivate(LM_IPV6_HDR_1)
        xbuf_deactivate(LM_IPV6_HDR_2)

    .endif

#if (!streq('sig_action', 'SIG_NONE'))
    ctx_arb[sig_action]
#endif


_flush_headers_done#:
    #undef IPV6_START_BYTE
    #undef BYTES_TO_COPY
    #undef QW_COUNT

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_flush_iphdr_cache:
//	 	Description: 
//			Writes back the IP header(s) from the local memory cache to the
//          packet in SDRAM. 
//
//	 	Outputs:
//
//		Inputs:
//			buf_handle	:			Handle of buffer.
//			req_sig		:			Signal to use in the I/O (DRAM read) operation.
//			sig_action	:			What to do with I/O operation - 
//									- wait for signal(s) as specified by sig_action
//									- do not wait for signal, just return (SIG_NONE)
//			The IP header (or headers, if the packet is tunneled) is in the
//          appropriate local memory buffer.  The header_type in the meta-data
//          is used to determine which headers to flush.
//
//		Size:   				:
//
///////////////////////////////////////////////////////////////////////////////
#define	dl_flush_iphdr_cache		dl_iphdr_flush_cache

#macro dl_iphdr_flush_cache[buf_handle, req_sig, sig_action]
.begin
	.reg hdr_offset

	dl_meta_get_offset(hdr_offset)

	// We need to determine the offset (0 - 7) at which the IP header begins
	// relative to the start of the quad-word aligned address at which the
	// write begins.  There are two possibilities:
	// 1: If no tunneling (encapsulation or decapsulation) was performed on
	// the packet or if a packet was decapsulated and re-encapsulated, then
	// the IP header will begin at offset 6.
	// 2: If a packet was encapsulated or decapsulated, the IP header will
	// begin at offset 2. 

    // 1: The IP header will begin at offset 6. This happens if,
    //    - no tunneling (encapsulation or decapsulation) was performed on
	//      the packet or if a packet was decapsulated and re-encapsulated
    // 2: The IP header will begin at offset 2. This happens if,
    //    - If a packet was encapsulated or decapsulated
    //    - If a packet is translated from v6 to v4 (The start address is
    //      moved ahead by 20. So the start offset changes from 6 to 2)
    //    - If a packet is translated from v4 to v6 (the start address is
    //      moved back by 20 since the v6 header is larger. Again the start
    //      offset changes from 6 to 2. 
    //      Note: Based on the v4 packet contents a fragment extension header
    //      might be added to the translated v6 header. This means the start 
    //      address must be moved back some more (by the size of the fragment
    //      header) Since the fragment header is a Quad-word in length, this
    //      does not change the offset.

	// Flush the header cache, given the alignment of the IP header relative
	// to the quad-word aligned start write address.

	alu[--, hdr_offset, AND, 4]
	bne[flush_hdr_align6#]

    _flush_headers(buf_handle, hdr_offset, 2, req_sig, sig_action)
    br[flush_hdr_end#]

flush_hdr_align6#:
    _flush_headers(buf_handle, hdr_offset, 6, req_sig, sig_action)

flush_hdr_end#:
.end
#endm

#endif	//__PKTHDR_CACHE_UC__

