/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      ingress_init.c
 *
 * = DESCRIPTION
 *      This file contains scripts to initialize the IXP2800 Ingress processor.
 *
 * = AUTHOR
 *      Ameya Varde
 *      ameya.s.varde@intel.com
 *
 * = CHANGE HISTORY
 *      03/11/2003 - Created 
 * ============================================================================
 */

#define	IXP_SCRIPTS
#define	IXP_XSCALE

#include "xscale_common.h"
#ifdef __linux
#include "../10x1GbE_ingress/wbench_project/dispatch_loop/dl_system.h"
#include "../../../../library/microblocks_library/include/dl_ingress_system_default_2800.h"
#else
#include "..\10x1GbE_ingress\wbench_project\dispatch_loop\dl_system.h"
#include "..\..\..\..\library\microblocks_library\include\dl_ingress_system_default_2800.h"
#endif

#ifdef IXP_XSCALE_RM
void init_sram(int data, void *start_addr, void *end_addr)
{
	memset((char *)start_addr,data,
			((unsigned int)end_addr-(unsigned int)start_addr));
}

void init_scratch(int data, void *start_addr, void *end_addr)
{
	memset((char *)start_addr,data,
			((unsigned int)end_addr-(unsigned int)start_addr));
}

#else	/* not IXP_XSCALE_RM */

void init_sram(int init_value, int start_addr, int end_addr)
{
	int addr;
#ifdef __linux
	int size;
	void *va;

	size = end_addr - start_addr;

	va = ioremap(start_addr, size);
	if (va != NULL) {
		for(addr=(int)va; addr<(int)va+size; addr+=4)
			*(volatile unsigned int *)(addr) = init_value;
		iounmap(va);
	} else
		printf("ioremap failed: address %x\n", start_addr);
#else
	for(addr=start_addr; addr<end_addr; addr+=4)
		set_sram(addr, init_value);
#endif
}

void init_scratch(int init_value, int start_addr, int end_addr)
{
	int addr;
#ifdef __linux
	int size;
	void *va;

	size = end_addr - start_addr;

	va = ioremap(start_addr, size);
	if (va != NULL) {
		for(addr=(int)va; addr<(int)va+size; addr+=4)
			*(volatile unsigned int *)(addr) = init_value;
		iounmap(va);
	} else
		printf("ioremap failed: address %x\n", start_addr);
#else
	for(addr=start_addr; addr<end_addr; addr+=4)
		set_scratch(addr, init_value);
#endif
}

#endif	/* IXP_XSCALE_RM */


/*----------------------
 *	IPv4
 *----------------------
 */

int rtm_init(void)
{	
	int cnt;
	#ifdef IXP_XSCALE_RM
		unsigned char *finaladdr;
	#else
		int finaladdr;
	#endif

	printf("Initializing SRAM trie table area...\n");

	cnt = 64 * 1024 + 256 + SIZE_OF_TRIE_SET * NUM_TRIE_SETS;
	finaladdr = TRIE_TABLE_SRAM_BASE + (cnt * 4);
	init_sram(0x0, TRIE_TABLE_SRAM_BASE, finaladdr);

	printf("Initialization complete.\n");

	printf("Initializing SRAM next hop table area...\n");

	cnt = NUM_NEXTHOPS * SIZE_OF_NEXTHOP_LW * 4;
	finaladdr = NEXTHOP_TABLE_SRAM_BASE + cnt;
	init_sram(0x0,NEXTHOP_TABLE_SRAM_BASE, finaladdr);

	printf("Initialization complete.\n");

	return 0;
}


int clear_ipv4_scratch_counters(void) {

	#ifdef IXP_XSCALE_RM
		unsigned char *finaladdr;
	#else
		int finaladdr;
	#endif

	printf("Initializing scratch IPv4 counters area...\n");

	finaladdr = IPV4_STATS_TABLE_BASE + IPV4_STATS_TABLE_SIZE;
	init_scratch(0x0, IPV4_STATS_TABLE_BASE, finaladdr);

	printf("Initialization complete.\n");

	return 0;
}


int dbcast_init(void) {

	int size;
	#ifdef IXP_XSCALE_RM
		unsigned char *finaladdr;
	#else
		int finaladdr;
	#endif

	printf("Initializing SRAM directed broadcast table area...\n");

	size = DBCAST_NUM_BLOCKS * DBCAST_TABLE_BLOCK_SIZE * 4;
	finaladdr = DBCAST_TABLE_SRAM_BASE + size;
	init_sram(0x0, DBCAST_TABLE_SRAM_BASE, finaladdr);

	printf("Initialization complete.\n");

	return 0;
}


/*	This function adds an address to dbcast table. 
//	NOTE : Add utmost DBCAST_TABLE_BLOCK_SIZE-1 entries
//  per block. Each block looks as follows.
//
//	 ---------------  BASE
//	|				|
//	|---------------
//	|	x			|
//	 ---------------
//	|	 			|
//	|---------------
//	|<Next addr>	|-> contains the full address of the next block
//	 ---------------
//	|				|
//	|---------------
//	|				|
//	 ---------------
//
*/
int dbcast_add(int ip_addr) {

	int hash_result, index, value;
	#ifdef IXP_XSCALE_RM
		unsigned char *base;
	#else
		int base;
	#endif

	/* Given an IP address of form A.B.C.D produce
	 * a hash_result of form A.A^B.A^B^C.A^B^C^D	
	 */

	hash_result = (ip_addr) ^ (ip_addr >> 8) ^ (ip_addr >> 16) ^ (ip_addr>>24);

	/* Get the key which is 8 bits */
	hash_result = hash_result & 0xff;

	/* Calculate the base */
	/* Assuming log2(DBCAST_TABLE_BLOCK_SIZE) == 2 */
	base = DBCAST_TABLE_SRAM_BASE + (hash_result << 2) * 4;

	for (index=0;index<DBCAST_TABLE_BLOCK_SIZE-1;index++)
	{
		value = get_sram(base+(index*4));
		if (value)
			continue;
		else
		{
			set_sram(base+(index*4),ip_addr);
			printf("Added 0x%x to DBCAST table at 0x%x\n",
				ip_addr,base+(index*4));
			break;
		}
	}

	if (index == (DBCAST_TABLE_BLOCK_SIZE-1))
		printf("Dbcast table is full. Can't add 0x%x\n",ip_addr);

	return 0;
}


/*----------------------
 *	QM
 *----------------------
 */
int ingress_qm_init(void) {
	
	#ifdef IXP_XSCALE_RM
		unsigned char *finaladdr;
	#else
		int finaladdr;
	#endif

	printf("\nInitializing SRAM queue descriptor area...\n");

	finaladdr = QD_SRAM_BASE + ((QD_TOTAL+2)*QD_SIZE);
	init_sram(0x0, QD_SRAM_BASE, finaladdr);

	printf("Initialization complete.\n");

	return 0;
}


void ingress_system_setup(void)
{
	#ifdef IXP_XSCALE_RM
	unsigned char *finaladdr;
	#else
	int finaladdr;
	#endif

	printf("\n******************************************************\n");
	printf("\n     System setup begin ...                           \n");
	printf("\n******************************************************\n");

	/*----------------------
	 *	Packet RX
	 *----------------------
	 */

	/*----------------------
	 *	IPv4 Forwarder
	 *----------------------
	 */

	/* Clear trie table and next hop area in SRAM. */
	rtm_init();

	/* Initialize the directed broadcast table. */
	dbcast_init();

	/* Clear IPv4 counters. */
	clear_ipv4_scratch_counters();

	printf("Adding some IPv4 routes...\n");
	l3_create_routes();

	/* Add dbcast entries using addresses from rtm_routes.ind file
	 * (take the destination IP address for which the 'LOCAL' flag is set,
	 * and then add corresponding network broadcast address for that
	 * network in the table).
	 */

	printf("Adding some dbcast entries...\n");
	dbcast_add(0x1affffff);
	dbcast_add(0x1a86ffff);
	dbcast_add(0x1a877fff);
	dbcast_add(0x1887ffff);
	dbcast_add(0x18870aff);
	dbcast_add(0x18870a00);

#ifndef IXP_XSCALE_RM

	/*
	 *-----------------------
	 *	IPV6 Forwarder
	 *-----------------------
         */

	printf("Initializing IPv6 route tables in SRAM ...\n");
	ipv6_init();

	printf("Adding some IPv6 routes...\n");
	route_add_16_64();

	/*
	 *------------------------
	 *	IPV6/V4 Tunneling
	 *------------------------
         */

	printf("Initializing Tunneling tables SRAM ...\n");
	v6v4_tunnel_init();

	printf("Setting up IPv6 decapsulation tunneling ...\n");
	setup_end_tunnels();

	printf("Setting up IPv6 encapsulation tunneling ...\n");
	setup_start_tunnels();

#endif	/* IXP_XSCALE_RM */

	/*----------------------
	 *	QM
	 *----------------------
	 */

	/* Initialize queue descriptor area in SRAM. */
	ingress_qm_init();

	/*----------------------
	 *	CSIX TX
	 *----------------------
	 */

	/*----------------------
	 *	COMMON
	 *----------------------
	 */

	/* If needed, set CSR to select DRAM address bits that
	 * decide DRAM bank scheduling.
	 * dc.d_0.DPL_DRAM_CTRL_RDH = 0x00021100;
	 * dc.d_1.DPL_DRAM_CTRL_RDH = 0x00021100;
	 * dc.d_2.DPL_DRAM_CTRL_RDH = 0x00021100;
	 */

#ifndef IXP_XSCALE_RM

	/* RM will initialize this area instead */

	/* Initialize SRAM meta-data area. */
	finaladdr = BUF_SRAM_BASE+BUF_SRAM_SIZE;
	init_sram(0x0, BUF_SRAM_BASE, finaladdr);

#endif

	printf("\n*********************************************************\n");
	printf("\n     System setup complete.                              \n");
	printf("\n*********************************************************\n");

}

#ifndef IXP_XSCALE_RM
/* With RM, iss includes mapping memory and patching symbols in addition to
 * setting up system. It's defined in xscale_me_init.c
 */

/* Shorter function names */
void iss(void)
{	
    ingress_system_setup();
}

#endif
