/*
 *******************************************************************************
 *                             Intel Proprietary
 *
 * Copyright (c) 2004 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmited,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                         Intel Corporation
 *                         2200 Mission College Blvd.
 *                         Santa Clara, CA  95052-8119
 *******************************************************************************
 */

#define	IXP_SCRIPTS
#define	IXP_XSCALE

#include "xscale_common.h"

#ifdef __linux
#include "../10x1GbE_ingress/wbench_project/dispatch_loop/dl_system.h"
#include "../../../../library/microblocks_library/include/dl_ingress_system_default_2800.h"
#include "../10x1GbE_egress/wbench_project/dispatch_loop/dl_system.h"
#include "../../../../library/microblocks_library/include/dl_egress_system_default_2800.h"
#else
#include "..\10x1GbE_ingress\wbench_project\dispatch_loop\dl_system.h"
#include "..\..\..\..\library\microblocks_library\include\dl_ingress_system_default_2800.h"
#include "..\10x1GbE_egress\wbench_project\dispatch_loop\dl_system.h"
#include "..\..\..\..\library\microblocks_library\include\dl_egress_system_default_2800.h"
#endif

#define	SRAM_WRITE(a,b)             set_sram((a),(b))

/* Definitions used by setup_start_tunnels() and setup_end_tunnels() to setup tunneling. */

#define V6V4_DECAP_FLAGS_VALID      0x8000
#define V6V4_ENCAP_FLAGS_VALID      0x8000

#define V6V4_DECAP_AUTO             0x0001
#define V6V4_SRC_VAL                0x0002
#define V6V4_DECAP_6TO4             0x0004
#define V6V4_DECAP_TOS_V6           0x0008

#define V6V4_ENCAP_AUTO             0x0001
#define V6V4_ENCAP_6TO4             0x0002
#define V6V4_ENCAP_TOS_V6           0x0004
#define V6V4_ENCAP_NO_DF            0x0008

/* Trie Types */
#define _4BIT_TRIE                  1
#define _8BIT_TRIE                  2
#define _16BIT_TRIE                 4


int ipv6_init() {
    
    int addr, end_addr;
    int cnt;

    addr = IPV6_TRIE_TABLE_SRAM_BASE_8;
    cnt = 64 * 1024 + 256 + SIZE_OF_IPV6_TRIE_SET * NUM_IPV6_TRIE_SETS;
    end_addr = addr + (cnt * 4);

    printf("Clearing IPv6 SRAM 8 bit trie table...from %x to %x \n", addr, end_addr);
    init_sram(0x0,addr,end_addr);

    addr = IPV6_L3_NEXTHOP_TABLE_SRAM_BASE;
    cnt = IPV6_NUM_L3_NEXTHOPS * IPV6_SIZE_OF_L3_NEXTHOP_LW * 4;
    end_addr = addr + cnt;

    printf("Clearing SRAM L3 nexthop info...from %x to %x \n", addr, end_addr);
    init_sram(0x0,addr,end_addr);

    #ifdef IPV6_STATS

    /*
     * Init counters
     */

    printf("\n\n IPv6: Reset Packet Counters ");

    addr = IPV6_STATS_TABLE_SRAM_BASE ;
    end_addr = addr + IPV6_STATS_TABLE_SRAM_SIZE;

    init_sram(0x0, addr, end_addr);

    #endif

    addr = IPV6_ROUTER_PREFIX_SRAM_BASE; 

    printf("\n\n IPv6: Set Router Prefix 0x3FFE:1234 /32 \n");

    /* 
     * Assume a 32-bit prefix 0x3FFE0000 /32 
     * To avoid matching the local memory trie cache, we set this to a value we will never use; ex 0x33FE0000
     */
    set_sram(addr+0x00, 0x33FE0000);
    set_sram(addr+0x04, 0x00000000);  

    /*
     * Set the Trie-entries 
     */
    set_sram(addr+0x08, 0x00010001);
    set_sram(addr+0x0C, 0x00020002);
    set_sram(addr+0x10, 0x00030003);
    set_sram(addr+0x14, 0x00000000);
    set_sram(addr+0x18, 0x00000000);
    set_sram(addr+0x1c, 0x00000000);
    set_sram(addr+0x20, 0x00000000);

    return 1;
}


int v6v4_tunnel_init(void) {
    
    int cnt, addr;
    int baseaddr;

    printf("v6v4_tunnel_init(): Initializing ipv6-ipv4 tunneling memory :=..\n");

    baseaddr = TUNNEL_ENCAP_NEXTHOP_SRAM_BASE;
    cnt = NUM_START_TUNNEL_NEXTHOPS * START_TUNNEL_NEXTHOP_LW;
    addr = baseaddr + (cnt * 4);

    printf("Clearing SRAM start tunnel next hop info...\n");
    init_sram(0x0,baseaddr,addr);
    

    baseaddr = TUNNEL_DECAP_NEXTHOP_SRAM_BASE;
    cnt = NUM_END_TUNNEL_NEXTHOPS * END_TUNNEL_NEXTHOP_LW;
    addr = baseaddr + (cnt * 4);

    printf("Clearing SRAM end tunnel next hop info...\n");
    init_sram(0x0,baseaddr,addr);
    
    /*
     Clear ingress source list.
     */

    baseaddr = TUNNEL_INGRESS_LIST_SRAM_BASE;

    #ifdef V6V4_INGRESS_LIST_TRIE

    cnt = 256 + (INGRESS_LIST_NUM_TRIE_BLOCKS * 16);
    cnt += V6V4_INGRESS_LIST_NUM_INDEX_BLOCKS * (V6V4_INGRESS_LIST_INDEX_BLOCK_SIZE / 2);
    printf("Clearing SRAM ingress source list trie blocks...\n");
    printf("1 cnt %x\n", cnt);

    #else

    cnt = V6V4_NUM_INGRESS_LIST_BLOCKS * V6V4_INGRESS_LIST_LW;
    printf("Clearing SRAM ingress source list blocks...\n");
    printf("2 cnt %x\n", cnt);

    #endif

    addr = baseaddr + (cnt * 4);
    init_sram(0x0,baseaddr,addr);

    #ifdef V6V4_DECAP_COUNTERS

    baseaddr = V6V4_DECAP_COUNTER_BASE;
    cnt = V6V4_NUM_DECAP_COUNTERS;
    addr = baseaddr + (cnt * 4);

    printf("Clearing tunnel decap packet counters...\n");
    init_sram(0x0,baseaddr,addr);

    #endif

    #ifdef V6V4_ENCAP_COUNTERS

    baseaddr = V6V4_ENCAP_COUNTER_BASE;
    cnt = V6V4_NUM_ENCAP_COUNTERS;
    addr = baseaddr + (cnt * 4);

    printf("Clearing tunnel encap packet counters...\n");
    init_sram(0x0,baseaddr,addr);

    #endif

    printf("V6V4 tunneling memory initialization COMPLETE.\n\n");

    return 1;
}


/*
 Creates a L3 next hop SRAM block at the index specified
 The next hop block is 16 bytes in size. 
     nh_index     : index at which to setup the L3 structure in the L3-next-hop table
     flags        : 0x1 - local, 0x2 - down, 0x4 - drop
     blade_id     : egress blade id
     nh_id_type   : type of next-hop-id ( 0x0 - L2 index, 0x2 - Tunneled, 0x4 - Compressed )
     nh_id        : contains l2 or tunneling table index
     port         : ouput port
     mtu          : mtu size 
*/
int ipv6_create_l3_next_hop(int index, int flags, int blade_id, int nh_type, int nh_id, int mtu, int port) {

    int addr;
    int blade_index;
    int mtu_port;

    /*
     Merge the blade id, l2 type and l2 index to form the second longword
     of the structure.
     */
    blade_index    = (nh_id & 0xFFFF) | ((nh_type & 0xF) <<20) | ((blade_id & 0xFF) <<24);

    /*
     Merge the mtu and port to form the third longword of the structure.
     */
    mtu_port = ((mtu & 0xFFFF) <<16) | (port & 0xFFFF);

    /*
     Calculate the addr of the next hop pointer based on the index
     Each block is 16 bytes
     */
    addr = IPV6_L3_NEXTHOP_TABLE_SRAM_BASE + (index * 16);

    /*
     Fill up the next hop information
     */
    set_sram(addr + 0x00,flags);
    set_sram(addr + 0x04,blade_index);
    set_sram(addr + 0x08,mtu_port);

    return 1;
}


/* set_8_bit_trie_entry

 Internal function to set a trie-entry for an 8-bit stride

 Parameters:

     block_id       : index of the 8-bit tire-block in to which the trie-entry is to be set
     index          : 8-bit portion of route-entry ip that is used as an index to access trie-entry
     next_hop       : value that will be set in the higher 16 bits of the trie entry as next-hop-pointer
     next_trie      : value that will be set in the lower 16 bits of the trie entry as next-trie-pointer
     bits_remaining : how many bits of the route-entry remains to be processed
                      used to determine if we have reached the end to set the next-hop
*/
int set_8_bit_trie_entry(int block_id, int index, int next_hop, int next_trie, int bits_remaining) {

    int location;
    int trie_entry;

    location = IPV6_TRIE_TABLE_SRAM_BASE_8 + (1024 * block_id);
    location += index * 4;
    trie_entry = get_sram(location);
    if ( !bits_remaining ) {
        printf("\n Setting next-hop  pointer in trie-block%d[%d](%d) <- 0x%08x ",  block_id, location, index, ((next_hop << 16) | (trie_entry & 0x0000FFFF)));
        /* Only clear and reset next-hop portion */
        set_sram(location, ((next_hop << 16) | (trie_entry & 0x0000FFFF)));
        return 0;
    }

    printf("\n Setting next-trie pointer in trie-block%d[%d](%d) <- 0x%08x", block_id, location,index, ((trie_entry & 0xFFFF0000 ) | next_trie));
    /* Clear and reset next-trie portion */
    set_sram(location, ((trie_entry & 0xFFFF0000 ) | next_trie));
    return 1;
}


/* ipv6_route_add

 Add a IPv6 route-entry to the route table


 Parameters:
     dst_127_96, int dst_95_64, int dst_63_32, int dst_31_0 : Four long contain the route-entry
     prefix_len : Length of the route-entry (in bits) that is being added
     next_hop_id : Index of the next-hop-structure that will be associated with the route-entry
     first_8_bit_block : Index of the trie-block on to which the first 8-bit stried will be mapped
                         Subsequent strides will be mapped to contiguous trie-blocks starting from the 
                         first block
 Examples :

     ipv6_route_add(0x3F21AABB, 0, 0, 0, 32, 4, 1)

     It will be mapped as follows
         0x3f21 - Root 16 bit trie-block
         0xAA   - trie-block 1
         0xBB   - trie-block 2

     Add some more routes

     ipv6_route_add(0x3F21AACC, 0, 0, 0, 32, 5, 1)     overlapping route

     It will be mapped as follows
         0x3f21 - Root 16 bit trie-block
         0xAA   - trie-block 1
         0xCC   - trie-block 2

     ipv6_route_add(0x3F22AABB, 0xCC, 0, 0, 40, 6, 7)  distinct route

     It will be mapped as follows
         0x3f22 - Root 16 bit trie-block
         0xAA   - trie-block 7
         0xBB   - trie-block 8
         0xCC   - trie-block 9
*/
int ipv6_route_add(int dst_127_96, int dst_95_64, int dst_63_32, int dst_31_0, int prefix_len, int next_hop_id, int first_8_bit_block) {

    int index;
    int location;
    int trie_entry;
    int this_block;
    int bits_remaining;

    if ( prefix_len < 16 ) {
        printf("\n Error: ipv6_route_add  - Can not add routes shorter than 16 bits ");
        return 0;
    }
    if ( prefix_len % 8 ) {
        printf("\n Error: ipv6_route_add  - Can not add routes with lenghts not multiple of 8 ");
        return 0;
    }

    /* We are dealing with the first 16-bit Root trie block */

    location = IPV6_TRIE_TABLE_SRAM_BASE_16 ;
    index = ((dst_127_96 >> 16) & 0xFFFF);
    location = location + (index * 4);
    bits_remaining = prefix_len - 16;

    /* If the prefix match is /16, then we add a valid next hop id
       Else, next hop idx is left alone */
    trie_entry = get_sram(location);
    if ( !bits_remaining ) {
        /* Clear and reset only next-hop portion */
        set_sram(location, ((next_hop_id << 16) | (trie_entry & 0x0000FFFF)));
        return 0;
    }
    /* Clear and reset next-trie portion */
    set_sram(location,((trie_entry & 0xFFFF0000 ) | first_8_bit_block));

    this_block = first_8_bit_block;
    bits_remaining -= 8;
    index = ((dst_127_96 >> 8) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_127_96 >> 0) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    /* 2nd Long-Word of DestIP */   

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_95_64 >> 24) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_95_64 >> 16) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_95_64 >> 8) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_95_64 >> 0) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    /* 3rd Long-Word of DestIP */   

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_63_32 >> 24) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_63_32 >> 16) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_63_32 >> 8) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_63_32 >> 0) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    /* 4th Long-Word of DestIP */   

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_31_0 >> 24) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_31_0 >> 16) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_31_0 >> 8) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_31_0 >> 0) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);

    return 1;
}


/* 
 * Creates an ending tunnel endpoint next hop SRAM block at the index
 * specified. The tunnel next hop block is 16 bytes in size, and contains
 * the following data:
 * Flags        (16 bits)	= flags
 * Ingress List (16 bits)	= ingress_list
 */
int create_end_tunnel_next_hop(int index, int flags, int ingress_list) {

	int addr;

	/* Merge the two 16-bit values into one longword, and set the ingress flag.*/

	int flags_list = ((flags | V6V4_DECAP_FLAGS_VALID) << 16) |
       (ingress_list & 0xFFFF);

	/* Calculate the addr of the tunnel next hop pointer based on the index
	   Each block is 16 bytes i.e, 0x10 bytes */
	addr = TUNNEL_DECAP_NEXTHOP_SRAM_BASE + (index * 0x04);

	/* Set the information.*/
	set_sram(addr + 0x00,flags_list);

	return 1;
}


/* 
 * Creates a starting tunnel endpoint next hop SRAM block at the index
 * specified. The tunnel next hop block is 16 bytes in size, and contains
 * the following data:
 * Flags        (16 bits)	= flags
 * Tunnel MTU   (16 bits)	= mtu
 * Local Addr   (32 bits)   = local_addr
 * Remote Addr  (32 bits)   = remote_addr
 * TTL          (8 bits)    = ttl
 */
int create_start_tunnel_next_hop(int index, int flags, int mtu, int local_addr, int remote_addr, int ttl, int tos, int sn_bcast) {

    int addr;

    /* Merge the flags and mtu into one longword, and set the egress flag. */

    int flags_mtu = ((flags | V6V4_ENCAP_FLAGS_VALID) << 16) | (mtu & 0xFFFF);

    /* Merge the TTL and TOS fields into one longword. */
    int ttl_tos = (ttl << 24) | ((tos & 0xFF) << 16);

    /* Calculate the addr of the tunnel next hop pointer based on the index
       Each block is 32 bytes i.e, 0x20 bytes */

    addr = TUNNEL_ENCAP_NEXTHOP_SRAM_BASE + (index * 0x20);

    /* Set the information. */

    set_sram(addr + 0x00,flags_mtu);
    set_sram(addr + 0x04,local_addr);
    set_sram(addr + 0x08,remote_addr);
    set_sram(addr + 0x0c,ttl_tos);
    set_sram(addr + 0x10,sn_bcast);

    return 1;
}


/* 
 * Add an address to an ingress list block.  The ingress list block is
 * 64 bytes in size and contains up to 12 address prefixes.  The format
 * is as follows:
 * NextBlock              (16 bits)
 * ValidityFlags          (16 bits)
 * MaskInfo_1_4           (32 bits)
 * Address_1 - Address_4  (4 x 32 bits)
 * MaskInfo_5_8           (32 bits)
 * Address_5 - Address_8  (4 x 32 bits)
 * MaskInfo_9_19          (32 bits)
 * Address_9 - Address_12 (4 x 32 bits)
 * Parameters passed to this routine are:
 * index : index of ingress list block
 * address : address value
 * prefix_length : prefix length (1 - 32)
 */
int ingress_tunnel_add_source(int index, int address, int prefix_length) {

    int addr;
    int flags;
    int addr_num = 0;
    int offset;
    int prefix_offset;
    int prefix_index;
    int prefix_specs;
    int mask;

    /* Calculate the addr of the tunnel next hop pointer based on the index
       Each block is 64 bytes i.e, 0x40 bytes */
	addr = TUNNEL_INGRESS_LIST_SRAM_BASE + (index * 0x40);

    /* Get the validity flags*/
    flags = get_sram(addr + 0x00);

    while (addr_num < 12) {
       if ((flags & (1 << addr_num)) == 0) {
           if (addr_num < 4) {
               offset = (addr_num + 2) * 4;
               prefix_offset = 0x04;
           }
           else if (addr_num < 8) {
               offset = (addr_num + 3) * 4;
               prefix_offset = 0x18;
           }
           else {
               offset = (addr_num + 4) * 4;
               prefix_offset = 0x2c;
           }

           prefix_index = addr_num % 4;

           prefix_specs = get_sram(addr + prefix_offset);
           mask = 0xff << (8 * prefix_index);
           prefix_specs &= ~mask;
           mask = (32 - prefix_length) << (8 * prefix_index);
           prefix_specs |= mask;

           set_sram(addr + prefix_offset, prefix_specs);

           mask = 0xffffffff << (32 - prefix_length);
           set_sram(addr + offset, address & mask);
           flags |= (1 << addr_num);
           set_sram(addr + 0x00, flags);

           return 1;
        }
        addr_num++;
    }

    return 1;
}


/*  
 * Simple route add routine. Adds 16 routes /64 routes
 */
int route_add_16_64(void) {

    /* Adding 16 IPv6 routes */

    ipv6_route_add(0x3FFE0000, 0xC0A80601, 0x00000000, 0x00000000, 64,  1,  1);
    ipv6_route_add(0x3FFE0000, 0xC0A80602, 0x00000000, 0x00000000, 64,  2,  1);
    ipv6_route_add(0x3FFE0000, 0xC0A80603, 0x00000000, 0x00000000, 64,  3,  1);
    ipv6_route_add(0x3FFE0000, 0xC0A80604, 0x00000000, 0x00000000, 64,  4,  1);
    ipv6_route_add(0x3FFE0000, 0xC0A80605, 0x00000000, 0x00000000, 64,  5,  1);
    ipv6_route_add(0x3FFE0000, 0xC0A80606, 0x00000000, 0x00000000, 64,  6,  1);
    ipv6_route_add(0x3FFE0000, 0xC0A80607, 0x00000000, 0x00000000, 64,  7,  1);
    ipv6_route_add(0x3FFE0000, 0xC0A80608, 0x00000000, 0x00000000, 64,  8,  1);
    ipv6_route_add(0x3FFE0000, 0xC0A80609, 0x00000000, 0x00000000, 64,  9,  1);
    ipv6_route_add(0x3FFE0000, 0xC0A8060A, 0x00000000, 0x00000000, 64, 10,  1);
    ipv6_route_add(0x3FFE0000, 0xC0A8060B, 0x00000000, 0x00000000, 64, 11,  1);
    ipv6_route_add(0x3FFE0000, 0xC0A8060C, 0x00000000, 0x00000000, 64, 12,  1);
    ipv6_route_add(0x3FFE0000, 0xC0A8060D, 0x00000000, 0x00000000, 64, 13,  1);
    ipv6_route_add(0x3FFE0000, 0xC0A8060E, 0x00000000, 0x00000000, 64, 14,  1);
    ipv6_route_add(0x3FFE0000, 0xC0A8060F, 0x00000000, 0x00000000, 64, 15,  1);
    ipv6_route_add(0x3FFE0000, 0xC0A80610, 0x00000000, 0x00000000, 64, 16,  1);

    /* Adding corresponding L3 next hop information.
     * The L2 index starts from an offset of L2_TABLE_NUM_IPV4_ENTRIES to avoid
     * overlapping with IPv4 L2 table entries on the Egress side. Applicable only when
     * next hop id type is NEXTHOP_TYPE_FORWARD.
     */

    ipv6_create_l3_next_hop( 0, 0,  0, 0, 0, 0, 0xffff);
    ipv6_create_l3_next_hop( 1, 0,  1, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES +  1), 9216, 0);
    ipv6_create_l3_next_hop( 2, 0,  2, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES +  2), 9216, 1);
    ipv6_create_l3_next_hop( 3, 0,  3, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES +  3), 9216, 2);
    ipv6_create_l3_next_hop( 4, 0,  4, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES +  4), 9216, 3);
    ipv6_create_l3_next_hop( 5, 0,  5, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES +  5), 9216, 4);
    ipv6_create_l3_next_hop( 6, 0,  6, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES +  6), 9216, 5);
    ipv6_create_l3_next_hop( 7, 0,  7, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES +  7), 9216, 6);
    ipv6_create_l3_next_hop( 8, 0,  8, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES +  8), 9216, 7);
    ipv6_create_l3_next_hop( 9, 0,  9, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES +  9), 9216, 8);
    ipv6_create_l3_next_hop(10, 0, 10, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES + 10), 9216, 9);
    ipv6_create_l3_next_hop(11, 0, 11, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES + 11), 9216, 0);
    ipv6_create_l3_next_hop(12, 0, 12, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES + 12), 9216, 1);
    ipv6_create_l3_next_hop(13, 0, 13, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES + 13), 9216, 2);
    ipv6_create_l3_next_hop(14, 0, 14, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES + 14), 9216, 3);
    ipv6_create_l3_next_hop(15, 0,  1, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES + 15), 9216, 4);
    ipv6_create_l3_next_hop(16, 0,  2, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES + 16), 9216, 5);

    return 1;
}


/*
 * Setup for decapsulation
 * This sets up the end tunnel next hop, L3 & L2 next hop, and ingress source list in a known configuration.
 */
int setup_end_tunnels(void) {

    printf("Setup_end_tunnels \n");

    printf("Creating end tunnel next hops \n"); 
    
    /* Set up the tunnel next hop entries. */
    /* NH id's 0x14 - 0x05 are returned by IPv4 dst addresses 124.0.20.0 - 124.0.29.0 */

    create_end_tunnel_next_hop(0x14, V6V4_DECAP_AUTO, 0);   /* IPv4 dst addr 124.0.20.0 */
    create_end_tunnel_next_hop(0x15, V6V4_SRC_VAL, 1);      /* IPv4 dst addr 124.0.21.0 */
    create_end_tunnel_next_hop(0x16, V6V4_DECAP_6TO4, 0);   /* IPv4 dst addr 124.0.22.0 */
    create_end_tunnel_next_hop(0x17, V6V4_DECAP_AUTO, 0);   /* IPv4 dst addr 124.0.23.0 */
    create_end_tunnel_next_hop(0x00, V6V4_SRC_VAL, 1);      /* IPv4 dst addr 124.0.24.0 */
    create_end_tunnel_next_hop(0x01, V6V4_DECAP_6TO4, 0);   /* IPv4 dst addr 124.0.25.0 */
    create_end_tunnel_next_hop(0x02, V6V4_DECAP_AUTO, 0);   /* IPv4 dst addr 124.0.26.0 */
    create_end_tunnel_next_hop(0x03, V6V4_SRC_VAL, 1);      /* IPv4 dst addr 124.0.27.0 */
    create_end_tunnel_next_hop(0x04, V6V4_DECAP_6TO4, 0);   /* IPv4 dst addr 124.0.28.0 */
    create_end_tunnel_next_hop(0x05, V6V4_DECAP_AUTO, 0);   /* IPv4 dst addr 124.0.29.0 */

    printf("Creating Ingress source list\n");

    /* Set up the ingress source list. */

    #ifdef V6V4_INGRESS_LIST_TRIE

    /* First set up index block.  Tunnel #2 is the configured tunnel, so
       add ID 2 to index block #1. */

    end_tunnel_add_index(1, 2);

    /* Now add the addresses to the trie tables.  Point all of them to index
       block 1, since there is only one configured tunnel in this test. */

    int next_block = 1;
    int num_blocks = 0;

    next_block += num_blocks;
    num_blocks = ingress_tunnel_add_source_trie(0x0A000000, 24, 1, next_block);

    printf("\n Total trie blocks used: %d \n",  next_block + num_blocks - 1);

    #else

    printf("ingress_tunnel_add_source\n");

    ingress_tunnel_add_source(1, 0x0A000000, 24);

    #endif

    /* Add IPv6 routes. */

    printf ("Setting up IPv6 route table \n");

    ipv6_route_add(0x3FFE0000, 0x00000000, 0x00000000, 0xC0A80605, 128, 17,  1);
    ipv6_route_add(0x3FFE0000, 0x00000000, 0x00000000, 0xC0A80606, 128, 18,  1);
    ipv6_route_add(0x3FFE0000, 0x00000000, 0x00000000, 0xC0A80607, 128, 19,  1);
    ipv6_route_add(0x3FFE0000, 0x00000000, 0x00000000, 0xC0A80608, 128, 20,  1);
    ipv6_route_add(0x3FFE0000, 0x00000000, 0x00000000, 0xC0A80609, 128, 21,  1);

    ipv6_route_add(0x20028600, 0x00030000, 0x00000000, 0x00000000,  48, 22, 31);
    ipv6_route_add(0x20028600, 0x00040000, 0x00000000, 0x00000000,  48, 23, 31);
    ipv6_route_add(0x20028600, 0x00050000, 0x00000000, 0x00000000,  48, 24, 31);
    ipv6_route_add(0x20028600, 0x00060000, 0x00000000, 0x00000000,  48, 25, 31);
    ipv6_route_add(0x20028600, 0x00070000, 0x00000000, 0x00000000,  48, 26, 31);

    printf("Creating L3 next hops \n");

    /* Set up the next hop entries. */
    /* Entries 0 - 16 are used by route_add_16_64() */

    ipv6_create_l3_next_hop(17, 0,  3, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES + 17), 9216, 0);
    ipv6_create_l3_next_hop(18, 0,  4, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES + 18), 9216, 1);
    ipv6_create_l3_next_hop(19, 0,  5, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES + 19), 9216, 2);
    ipv6_create_l3_next_hop(20, 0,  6, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES + 20), 9216, 3);
    ipv6_create_l3_next_hop(21, 0,  7, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES + 21), 9216, 4);
    ipv6_create_l3_next_hop(22, 0,  8, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES + 22), 9216, 5);
    ipv6_create_l3_next_hop(23, 0,  9, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES + 23), 9216, 6);
    ipv6_create_l3_next_hop(24, 0, 10, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES + 24), 9216, 7);
    ipv6_create_l3_next_hop(25, 0, 11, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES + 25), 9216, 8);
    ipv6_create_l3_next_hop(26, 0, 12, NEXTHOP_TYPE_FORWARD, (L2_TABLE_NUM_IPV4_ENTRIES + 26), 9216, 9);

    return 1;
}


/*
 * Setup for encapsulation
 * This sets up the start tunnel next hop, L3 & L2 next hop in a known configuration.
 */
int setup_start_tunnels(void) {

    printf ("Setting up IPv6 route table \n");

    /* ipv6_route_add(int dst_127_96, int dst_95_64, int dst_63_32, int dst_31_0,
                      int prefix_len, int next_hop_id, int first_8_bit_block)
     */

    ipv6_route_add(0x3FFE0000, 0x00000000, 0x00000000, 0xC0A80601, 128, 27,  1); /* Using IPv4 dst addr 124.0.0.0 */
    ipv6_route_add(0x00000000, 0x00000000, 0x00000000, 0x7c000100, 128, 28, 16); /* Using IPv4 dst addr 124.0.1.0 */
    ipv6_route_add(0x20027c00, 0x02000000, 0x00000000, 0x00000000,  48, 29, 31); /* Using IPv4 dst addr 124.0.2.0 */

    ipv6_route_add(0x3FFE0000, 0x00000000, 0x00000000, 0xC0A80602, 128, 30, 1);  /* Using IPv4 dst addr 124.0.3.0 */
    ipv6_route_add(0x00000000, 0x00000000, 0x00000000, 0x7c000401, 128, 31, 16); /* Using IPv4 dst addr 124.0.4.0 */
    ipv6_route_add(0x20027c00, 0x05010000, 0x00000000, 0x00000000,  48, 32, 31); /* Using IPv4 dst addr 124.0.5.0 */

    ipv6_route_add(0x3FFE0000, 0x00000000, 0x00000000, 0xC0A80603, 128, 33,  1); /* Using IPv4 dst addr 124.0.6.0 */
    ipv6_route_add(0x00000000, 0x00000000, 0x00000000, 0x7c000702, 128, 34, 16); /* Using IPv4 dst addr 124.0.7.0 */
    ipv6_route_add(0x20027c00, 0x08020000, 0x00000000, 0x00000000,  48, 35, 31); /* Using IPv4 dst addr 124.0.8.0 */

    ipv6_route_add(0x3FFE0000, 0x00000000, 0x00000000, 0xC0A80604, 128, 36,  1); /* Using IPv4 dst addr 124.0.9.0 */

    /* Set up the next hop entries. */
    /* Entries 0 - 16 are used by route_add_16_64() */
    /* Entries 17 - 26 are used by setup_end_tunnels() */

    ipv6_create_l3_next_hop(27, 0, 13, NEXTHOP_TYPE_TUNNEL, 1, 9216, 0);
    ipv6_create_l3_next_hop(28, 0, 14, NEXTHOP_TYPE_TUNNEL, 2, 9216, 1);
    ipv6_create_l3_next_hop(29, 0,  1, NEXTHOP_TYPE_TUNNEL, 3, 9216, 2);
    ipv6_create_l3_next_hop(30, 0,  2, NEXTHOP_TYPE_TUNNEL, 4, 9216, 3);
    ipv6_create_l3_next_hop(31, 0,  3, NEXTHOP_TYPE_TUNNEL, 2, 9216, 4);
    ipv6_create_l3_next_hop(32, 0,  4, NEXTHOP_TYPE_TUNNEL, 3, 9216, 5);
    ipv6_create_l3_next_hop(33, 0,  5, NEXTHOP_TYPE_TUNNEL, 5, 9216, 6);
    ipv6_create_l3_next_hop(34, 0,  6, NEXTHOP_TYPE_TUNNEL, 2, 9216, 7);
    ipv6_create_l3_next_hop(35, 0,  7, NEXTHOP_TYPE_TUNNEL, 3, 9216, 8);
    ipv6_create_l3_next_hop(36, 0,  8, NEXTHOP_TYPE_TUNNEL, 6, 9216, 9);

    /* Set up the tunnel next hop entries. */

    create_start_tunnel_next_hop(1, 0, 1200, 0x0a000000, 0x7c000000, 24, 0, 0);
    create_start_tunnel_next_hop(2, V6V4_ENCAP_AUTO | V6V4_ENCAP_NO_DF, 1500, 0x0a000000, 0, 16, 0, 0);
    create_start_tunnel_next_hop(3, V6V4_ENCAP_AUTO | V6V4_ENCAP_6TO4 | V6V4_ENCAP_NO_DF, 1500, 0x0a000000, 0, 32, 0, 0x7c00ffff);
    create_start_tunnel_next_hop(4, 0, 1200, 0x0a000000, 0x7c000300, 24, 0, 0);
    create_start_tunnel_next_hop(5, 0, 1200, 0x0a000000, 0x7c000600, 24, 0, 0);
    create_start_tunnel_next_hop(6, 0, 1200, 0x0a000000, 0x7c000900, 24, 0, 0);

    return 1;
}
