/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      linux_init.c
 *
 * = DESCRIPTION
 *      This file contains the linux kernel driver to initialize the 
 *	IXP2800 processor.
 *
 * = AUTHOR
 *
 * = CHANGE HISTORY
 * ============================================================================
 */

#define	IXP_SCRIPTS
#define	IXP_XSCALE

#include "xscale_common.h"

/*
 *  some global macro definitions
 */

MODULE_LICENSE("Proprietary");

EXPORT_NO_SYMBOLS;
static struct proc_dir_entry *p_proc_file;

void imi1(void);
void imi2(void);
void imi3(void);
void imi4(void);
void iss(void);

void emi1(void);
void emi2(void);
void emi3(void);
void emi4(void);
void ess(void);


unsigned int get32(unsigned int address)
{
	void *va;
	unsigned int val;

	va = ioremap(address, 4);
	if (va != NULL) {
		val = *(volatile unsigned int *)(va);
		iounmap(va);
		return val;
	} else
		printk("ioremap failed: address %x\n", address);
	return 0;
}


void put32(unsigned int address, unsigned int val)
{
	void *va;

	va = ioremap(address, 4);
	if (va != NULL) {
		*(volatile unsigned int *)(va) = val;
		iounmap(va);
	} else
		printk("ioremap failed: address %x\n", address);
}


/*
 * Code to dump some stuff under Linux and some debugging
 * utilities.
 */

static char my_name[] = "xscme";

int xscale_me_status(
	char *buf,char **start, off_t offset,int len,int *eof,void *data)
{
	return 0;
}


int xscale_me_cmd(
	struct file *file, const char *cmd, unsigned long count, void *data)
{
	char *str, *sp;
	char *token;
	char *delim="\t\n\b ";

	/*
	 * need to parse the command
	 */

	str = sp = vmalloc(count + 1);
	if (!str)
	{
		printk("ERROR : can't allocated memory in %s.\n",my_name);
		return -1;
	}

	memcpy(str,cmd,count);
	str[count-1] = 0;	/* includes \n */

	/* printk("\nExecuting command \"%s\"\n\n",str);*/

	while ((token = strsep(&sp,delim)))
	{
		/* ignore delimiters */
		if (strstr(delim,token) != NULL)
			continue;

		/*
		 * check for commands issued
		 */
		if (strcmp(token,"imi1")==0)
		{
			imi1();
		}
		else if (strcmp(token,"imi2")==0)
		{
			imi2();
		}
		else if (strcmp(token,"imi3")==0)
		{
			imi3();
		}
		else if (strcmp(token,"imi4")==0)
		{
			imi4();
		}
		else if (strcmp(token,"iss")==0)
		{
			iss();
		}
		else if (strcmp(token,"emi1")==0)
		{
			emi1();
		}
		else if (strcmp(token,"emi2")==0)
		{
			emi2();
		}
		else if (strcmp(token,"emi3")==0)
		{
			emi3();
		}
		else if (strcmp(token,"emi4")==0)
		{
			emi4();
		}
		else if (strcmp(token,"ess")==0)
		{
			ess();
		}
		else 
		{
			printk("ERROR : Unknown command '%s'\n",str);
			return -1;
		}

	}

	/*
	 * free memory allocated
	 */
	vfree(str);
	return count;
}


/*
 * init_module for the Xscale initialization code
 * ----------------------------------------------
 */


int system_setup(void)
{
	p_proc_file = create_proc_entry(my_name,S_IFREG | S_IWUSR,&proc_root);

	if (p_proc_file)
	{
		p_proc_file->read_proc = xscale_me_status;
		p_proc_file->write_proc = xscale_me_cmd; 
	}
	else
	{
		printk("WARNING : Can't regsiter stats under /proc \n");
		return 0;
	}

	printk("\n@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n");
	printk("           Initialization complete.\n");
	printk("@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@\n\n");
	return 0;
}


void system_clean(void)
{
	if (p_proc_file)
		remove_proc_entry(my_name,&proc_root);

	return;
}


module_init(system_setup);
module_exit(system_clean);

