/**   09/12/2003   
 *This version: is the same as the original except that now we deleted MSF_CLK_CONTROL writes
 *in IMI4 and turn on the rx and tx section enables at the very end
 * ============================================================================
 * = COPYRIGHT 
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 * 
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 * 
 * = FILENAME
 *      media.c
 *
 * = DESCRIPTION
 *      This file contains functions to initialize the media used by the 
 *      IXP2800 Ingress and Egress processor.
 *
 * = AUTHOR
 *      Ameya Varde
 *      ameya.s.varde@intel.com
 *
 * = CHANGE HISTORY
 *      03/11/2003 - Created 
 * ============================================================================
 */

#include "xscale_common.h"

/* Macro to write and read one longword from memory.
   The macro assumes that the memory being read or
   written to is mapped.
 */

#define get_slowport(address)           GET32(0xC6000000+(address*4))
#define set_slowport(address,val)       PUT32((0xC6000000+(address*4)),val)

/* Definitions */

#define FALSE                           0
#define	TRUE                            1
#define	DONT_CARE                       2

#define	POS                             1
#define ETH_1x10                        2
#define ETH_10x1                        3

#define	CROSS_ISLAND                    1
#define	GEORGES_ISLAND                  2

/* Select either CROSS_ISLAND or GEORGES_ISLAND*/
#define	MEDIA_CARD                      GEORGES_ISLAND


/* If MEDIA_CARD == CROSS_ISLAND, select either POS or ETH_1x10
 * If MEDIA_CARD == GEORGES_ISLAND, select ETH_10x1
 */
#define MEDIA_TYPE                      ETH_10x1

/* If MEDIA_CARD == CROSS_ISLAND, select either TRUE or FALSE
 * Select TRUE to enable SPI-4 media loopback from Egress to Ingress
 * Select FALSE to disable SPI-4 media loopback from Egress to Ingress
 * (i.e. when you want to receive and transmit using a packet generator)
 *
 * If MEDIA_CARD == GEORGES_ISLAND, select DONT_CARE
 * It does not need any special setting to do loopback
 */
#define MEDIA_LOOPBACK                  FALSE

void georges_island_init(void)
{

	int gi_stat_reg, gi_train_stat;

	/* Set display clock divider */
	PUT32(0xc0080000, 0x2);

	/*
	Enable LEDs
	LED Control Register (Addr: 0x509)
	This is a Ben Nevis register.
	Turns all the LEDs off
	*/
	PUT32(0xc6001424, 0x2);

	/*
	Turn all Green RX LEDs (middle row) on 
	This is a Ben Nevis register.
	Link LED Enable Register (Addr: 0x502)
	*/
	PUT32(0xc6001408, 0x3ff);

	/*
	Set bit 5 to start auto-negotiation one for each port
	This is Ben Nevis register: 0x18
	Diverse Config Register - 0x18
	*/
	PUT32(0xc6000060, 0x116d);
	PUT32(0xc6000260, 0x116d);
	PUT32(0xc6000460, 0x116d);
	PUT32(0xc6000660, 0x116d);
	PUT32(0xc6000860, 0x116d);
	PUT32(0xc6000a60, 0x116d);
	PUT32(0xc6000c60, 0x116d);
	PUT32(0xc6000e60, 0x116d);
	PUT32(0xc6001060, 0x116d);
	PUT32(0xc6001260, 0x116d);

	/*
	Set MAX frame size for each port as 9k
	This is Ben Nevis register: 0xf
	Max Frame Size Register - 0xof
	*/
	PUT32(0xc600003c, 0x240a);
	PUT32(0xc600023c, 0x240a);
	PUT32(0xc600043c, 0x240a);
	PUT32(0xc600063c, 0x240a);
	PUT32(0xc600083c, 0x240a);
	PUT32(0xc6000a3c, 0x240a);
	PUT32(0xc6000c3c, 0x240a);
	PUT32(0xc6000e3c, 0x240a);
	PUT32(0xc600103c, 0x240a);
	PUT32(0xc600123c, 0x240a);

	/*
         * Rx High watermark : 
         *   When this watermark is exceeded the MAC
         *   will start sending PAUSE frames
 	 */
	PUT32(0xc6001600, 0x2710);
	PUT32(0xc6001604, 0x2710);
	PUT32(0xc6001608, 0x2710);
	PUT32(0xc600160c, 0x2710);
	PUT32(0xc6001610, 0x2710);
	PUT32(0xc6001614, 0x2710);
	PUT32(0xc6001618, 0x2710);
	PUT32(0xc600161c, 0x2710);
	PUT32(0xc6001620, 0x2710);
	PUT32(0xc6001624, 0x2710);

	/*
         * Rx Low watermark :
         *   When this watermark is crossed the MAC
         *   will start sending PAUSE/Resume frames
	 */
	PUT32(0xc6001628, 0x1f40);
	PUT32(0xc600162c, 0x1f40);
	PUT32(0xc6001630, 0x1f40);
	PUT32(0xc6001634, 0x1f40);
	PUT32(0xc6001638, 0x1f40);
	PUT32(0xc600163c, 0x1f40);
	PUT32(0xc6001640, 0x1f40);
	PUT32(0xc6001644, 0x1f40);
	PUT32(0xc6001648, 0x1f40);
	PUT32(0xc600164c, 0x1f40);


	#if 0
	/*
         * Disable sending of flow control frames
         * FC Enable Register	
         * Port Index + 0x12
	*/
	PUT32(0xc6000048, 0x0);
	PUT32(0xc6000248, 0x0);
	PUT32(0xc6000448, 0x0);
	PUT32(0xc6000648, 0x0);
	PUT32(0xc6000848, 0x0);
	PUT32(0xc6000a48, 0x0);
	PUT32(0xc6000c48, 0x0);
	PUT32(0xc6000e48, 0x0);
	PUT32(0xc6001048, 0x0);
	PUT32(0xc6001248, 0x0);
	#endif


	/* Set periodic training intervals*/
	/*PUT32(0xc6001c04, 0x01ff);*/

	/*
	 Set MaxBurst1 value to 8 in order to allow RX receiving 128 bytes
	 keep the MaxBurst2 4 to avoid length error for 10gx1 pipeline.
	*/
	PUT32(0xc6001c00, 0x00080004);

	/*
	Bits 21 & 20 should be set for valid link: one for each port
	This is a Ben Nevis register.
	RX Config Word Register - 0x16
	*/

	/* Send training to Georges Island */
	printf("Sending training to Georges Island.\n");
	set_msf(0xA0, 0x00000100);
	taskDelay(5);
	
	gi_stat_reg = GET32(0xc6001c08);
	gi_train_stat = (gi_stat_reg >> 12) & 0x1;
	
	if (gi_train_stat == 1)
	{
			
		printf("Georges Island is sending FIFO status to Egress NPU.\n");
		set_msf(0xA0, 0x00000000);
		printf("Egress NPU stopped sending training to Georges Island.\n");
		taskDelay(5);
	}
	else
	{
		printf("Georges Island failed to train.\n");
		printf("Please verify that GI has a B0 IXF1110 MAC.\n");	
	}
}


void cross_island_init(void)
{
	int calypso_stat;

	#if (MEDIA_TYPE == POS)

	int pcs_stat1;

	/* Write to the Cross Island PLD - sets all the LAN mode strap options */
	#if (MEDIA_LOOPBACK == TRUE)
	PUT32(0xC6200000, 0x0000C0E8);
	#else
	PUT32(0xC6200000, 0x0000C0F8);
	#endif
	taskDelay(5);

	/* ------------------------------------------------------------
	 * Calypso global register configuration
	 * ------------------------------------------------------------
	 */

	/* GLB_RST - Global reset register (Calypso address: 0003H) */
	set_slowport(0x0003, 0x000F);

	/* GLB_CFG1 - Global configuration 1 register (Calypso address: 0000H) */
	set_slowport(0x0000, 0x0002);

	/* GLB_CLKCFG1 - Clock configuration 1 register (Calypso address: 0006H) */
	set_slowport(0x0006, 0xA000);

	/* GLB_CLKCFG2 - Clock configuration 2 register (Calypso address: 0008H) */
	set_slowport(0x0008, 0x0003);

	/* GLB_LINE - Global line interface register (000AH) */
	#if (MEDIA_LOOPBACK == TRUE)
	set_slowport(0x000A, 0x0404);
	#else
	set_slowport(0x000A, 0x0004);
	#endif

	/* GLB_SPI4 - Global SPI-4 register (0010H) */
	set_slowport(0x0010, 0x0001);

	/* GBL_RST - Global reset register (0003H) */
	set_slowport(0x0003, 0x0000);


	/* ------------------------------------------------------------
	 * SPI configuration (SIF)
	 * ------------------------------------------------------------
	 */

	/* SIF_INTEN - System interface interrupt enable register (300CH) */
	set_slowport(0x300C, 0x00FF);

	/* SIF_TEST - SPI-4 test register (300DH) */
	set_slowport(0x300D, 0x0000);

	/* SIF_MAC_THRESHOLD_1 - SPI-4 MAC pause upper threshold value register (300EH) */
	set_slowport(0x300E, 0x000A);

	/* SIF_MAC_THRESHOLD_1 - SPI-4 MAC pause lower threshold value register (300FH) */
	set_slowport(0x300F, 0x0400);


	/* ------------------------------------------------------------
	 * XGMAC configuration
	 * ------------------------------------------------------------
	 */

	/* XGMAC_CNTL1 - XGMAC control register (3100H-3101H) */
	set_slowport(0x3100, 0x7FFB);
	set_slowport(0x3101, 0x3804);


	/* ------------------------------------------------------------
	 * PCS configuration
	 * ------------------------------------------------------------
	 */

	/* PCS_CNTL1 - PCS control 1 register (3200H) */
	set_slowport(0x3200, 0x2840);

	/* PCS_10GCNTL2 - PCS control 2 register (3207H) */
	set_slowport(0x3207, 0x0002);

	/* PCS_CALY1 - IXF18101 specific PCS register 1 - misc. control signals (3231H) */
	set_slowport(0x3231, 0x000D);

	/* PCS_CALY3 - IXF18101 specific PCS register 3 - receive FIFO control (3233H) */
	set_slowport(0x3233, 0x0120);

	/* PCS_CALY4 - IXF18101 specific PCS register 4 - transmit FIFO control (3234H) */
	set_slowport(0x3234, 0x503C);

	/* PCS_CNTL1 - PCS control 1 register (3200H) */
	set_slowport(0x3200, 0x2040);

	pcs_stat1 = get_slowport(0x3201);
	printf("PCS_STAT1 register : 0x%x\n", pcs_stat1);


	/* ------------------------------------------------------------
	 * SONET configuration
	 * ------------------------------------------------------------
	 */

	/* TL_POH_SC - Transmit line POH source control buffer (1840H-1843H) */
	set_slowport(0x1840, 0x0000);
	set_slowport(0x1841, 0x0003);
	set_slowport(0x1842, 0x0002);
	set_slowport(0x1843, 0x0002);

	/* TL_POH_CTL - Transmit line POH control (1821H) */
	set_slowport(0x1821, 0x0000);

	/* TL_SC1_TOH_0 - Transmit line source configuration 1 for TOH (1882H) */
	set_slowport(0x1882, 0x0000);

	/* RL_POH_CFG2 - Receive path OH configuration register 2 (11C1H) */
	set_slowport(0x11C1, 0x0000);

	/* Allows jumbo packets (>1518 bytes) */
	set_slowport(0x3302, 0x7FFF);

#if 1

/* POS receive controller configuration (3300h) */
PUT32(0xC600cc00, 0x047d);

/* POS transmit controller configuration (3330h) */
PUT32(0xC600ccc0, 0x004d);

/* C2 byte value (1832h) */
PUT32(0xC60060c8, 0x0001);

/* G1 byte value (1833h) */
PUT32(0xC60060cc, 0x0000);

#endif

	#endif

	#if (MEDIA_TYPE == ETH_1x10)

	/* Write to the Cross Island PLD - sets all the LAN mode strap options */
	#if (MEDIA_LOOPBACK == TRUE)
	PUT32(0xC6200000, 0x0000C688);
	#else
	PUT32(0xC6200000, 0x0000C698);
	#endif
	taskDelay(5);

	/* Global enable reset (Calypso address: 0x0003) */
	printf("Global reset: 0x0003 = 0x%x (should be 0xf).\n", get_slowport(0x0003));
	set_slowport(0x0003, 0x000F);
	taskDelay(5);

	/* GLB_CFG1 - Disable SONET framer (Calypso address: 0x0000) */
	set_slowport(0x0000, 0x0005);
	taskDelay(5);

	/* GLB_CLKCFG1 - Clock configuration 1 register (Calypso address: 0x0006) */
	set_slowport(0x0006, 0xA000);
	taskDelay(5);

	/* GLB_CLKCFG1 - Clock configuration 1 register (Calypso address: 0x0008) */
	set_slowport(0x0008, 0x0003);
	taskDelay(5);

	/* Global line interface (Calypso address: 0x000A) */
	printf("Global line interface: 0x000a = 0x%x (should be 0x0).\n", get_slowport(0x000A));

	/* Original code */
	set_slowport(0x000A, 0x0004);

	/* Remote loopback testing */
	/* set_slowport(0x000A, 0x00000804); */

	taskDelay(5);
	printf("Global line interface: 0x000A = 0x%x (new vlaue).\n", get_slowport(0x000A));

	/* GLB_CLKCFG1 - SPI-4 receiving clock (Calypso address: 0x0010) */
	set_slowport(0x0010, 0x0000);
	taskDelay(5);

	/* Global disable reset (Calypso address: 0x0003) */
	set_slowport(0x0003, 0x0000);
	taskDelay(5);

	/* Set Upper (actually Lower) Threshold for MAC Pause to 10 (Calypso address: 0x300E) */
	set_slowport(0x300E, 0x000A);
	taskDelay(5);

	/* Set Lower (actually Upper) Threshold for MAC Pause to 1140 (Calypso address: 0x300F) */
	set_slowport(0x300F, 0x0474);
	taskDelay(5);

	/* Enable PCS power down mode (Calypso address: 0x3200) */
	set_slowport(0x3200, 0x2840);
	taskDelay(5);

	/* Disable PCS power down mode (Calypso address: 0x3200) */
	set_slowport(0x3200, 0x2040);
	taskDelay(5);

	/* Reset PCS registers (Calypso address: 0x3200) */
	set_slowport(0x3200, 0xA040);
	taskDelay(5);

	/*Configure MAC Rx to enable busy/idle decoding (Calypso address: 0x3116 - 0x3117) */
	set_slowport(0x3116, 0x0000);
	taskDelay(5);

	/*Configure MAC Rx to enable busy/idle decoding (Calypso address: 0x3116 - 0x3117) */
	set_slowport(0x3117, 0x0084);
	taskDelay(5);

	/*Enable Reception of Packets (Calypso address: 0x3100 - 0x3101) */
	set_slowport(0x3100, 0x7FFF);
	taskDelay(5);

	/*Enable Reception of Packets (Calypso address: 0x3100 - 0x3101) */
	set_slowport(0x3101, 0x3804);
	taskDelay(5);

	/*Configure MAC addrees up 16 bits (Calypso address: 0x3114 - 0x3115) */
	set_slowport(0x3114, 0x0000);
	taskDelay(5);

	/*Configure Max Frame Size > 9k(Calypso address: 0x3114 - 0x3115) */
	set_slowport(0x3115, 0x2600);
	taskDelay(5);

	/*Set up look back (Calypso address: 0x300d) */
	/*set_slowport(0x300d, 0x0002);*/
	/*taskDelay(5);*/

	#endif

	/* Send training to Cross Island */
	printf("Sending training to Cross Island.\n");
	set_msf(0xA0, 0x00000100);
	taskDelay(5);

	calypso_stat = GET32(0xC600C024);
	taskDelay(5);

	if (calypso_stat == 0x6)
		printf("Cross Island received training from Egress! (status = 0x%x)\n", calypso_stat);
	else
		printf("Cross Island failed to receive training from Egress. (status = 0x%x)\n", calypso_stat);

	printf("Stopped sending training to Cross Island.\n");
	set_msf(0xA0, 0x00000000);
	taskDelay(5);

	calypso_stat = GET32(0xC600C024);
	taskDelay(5);

	if (calypso_stat == 0x4)
		printf("Cross Island stopped receiving training from Egress! (status = 0x%x)\n", calypso_stat);
	else
		printf("Cross Island failed to stop receiving training from Egress. (status = 0x%x)\n", calypso_stat);

	printf("\n");

	#if (MEDIA_LOOPBACK == TRUE)
	/* Set Cross Island in FIFO loopback */
	PUT32(0xC600C034, 0x00000001);
	taskDelay(5);
	#endif
}


void ingress_media_init_1(void)
{
	int clock_control;
	int ixp_reset0;

	/* Slowport fix, CSR SP_WTC2 */
	PUT32(0xC0080008, 0x00000048);
	taskDelay(5);

	/* Setting CSR CLOCK_CONTROL */
	clock_control = GET32(0xC0004A14);
	clock_control &= 0xFF0FFFFF;
	clock_control |= (0x3 << 20);
	PUT32(0xC0004A14, clock_control);
	taskDelay(5);

	/* Setting CSR IXP_RESET0 */
	ixp_reset0 = GET32(0xC0004A0C);
#ifndef IXP_XSCALE_RM
	ixp_reset0 |= 0x00400080;
	ixp_reset0 &= 0xFFFF7FFF;	
#else
	ixp_reset0 |= (1 << 7);
#endif
	PUT32(0xC0004A0C, ixp_reset0);
	taskDelay(5);

	ixp_reset0 = GET32(0xC0004A0C);
#ifndef IXP_XSCALE_RM
	ixp_reset0 |= 0x00408000;
#endif
	ixp_reset0 &= 0xFFFFFF7F;
	PUT32(0xC0004A0C, ixp_reset0);
	taskDelay(5);

	/* Setting CSR IXP_RESET1 */
	PUT32(0xC0004A10, 0x00000000);
	taskDelay(5);

	/* Setting CSR MSF_IO_BUF_CNTL */
	/*set_msf(0x8008, 0x000014AF);*********************************/
	set_msf(0x8008, 0x000016A3);
	taskDelay(5);

	/* Setting CSR FC_IO_BUF_CNTL */
	set_msf(0x800C, 0x000014AF);
	taskDelay(5);

	/* Setting CSR MSF_Tx_Control (select transmit clock) */
	/*set_msf(0x04, 0x00100905);*/
	set_msf(0x04, 0x00100805);
	taskDelay(5);

	/* Setting CSR MSF_Clock_Control (enable transmit clocks) */
	set_msf(0x2C, 0x00000050);
	taskDelay(5);

	/* Setting CSR MSF_Clock_Control (enable transmit sections) */
/*	set_msf(0x2C, 0x00000154);
	taskDelay(5);
*/
}


void egress_media_init_1(void)
{
	int clock_control;
	int ixp_reset0;

	/* Slowport fix, CSR SP_WTC2 */
	/*PUT32(0xC0080008, 0x00000048);
	taskDelay(5);*/

	/* Setting CSR CLOCK_CONTROL */
	clock_control = GET32(0xC0004A14);
	clock_control &= 0xFF0FFFFF;
	clock_control |= (0x4 << 20);
	PUT32(0xC0004A14, clock_control);
	taskDelay(5);

	/* Setting CSR IXP_RESET0 */
	ixp_reset0 = GET32(0xC0004A0C);
	ixp_reset0 |= 0x00400080;
	ixp_reset0 &= 0xFFFF7FFF;
	PUT32(0xC0004A0C, ixp_reset0);
	taskDelay(5);

	ixp_reset0 = GET32(0xC0004A0C);
	ixp_reset0 |= 0x00408000;
	ixp_reset0 &= 0xFFFFFF7F;
	PUT32(0xC0004A0C, ixp_reset0);
	taskDelay(5);

	/* Setting CSR IXP_RESET1 */
	PUT32(0xC0004A10, 0x00000000);
	taskDelay(5);

	/* Setting CSR MSF_IO_BUF_CNTL */
	set_msf(0x8008, 0x000014AF);
	taskDelay(5);

	/* Setting CSR FC_IO_BUF_CNTL */
	set_msf(0x800C, 0x000014AF);
	taskDelay(5);

	/* Setting CSR MSF_Rx_Control (configure receive section for 2 partiton mode/duplex mode) */
	set_msf(0x0, 0x00000005);
	taskDelay(5);

	/* Setting CSR MSF_Clock_Control (enable receive clock select) */
	set_msf(0x2C, 0x00003000);
	taskDelay(5);

	/* Setting CSR MSF_Clock_Control (enable receive clock dll) */
	set_msf(0x2C, 0x00003C00);
	taskDelay(5);

	/* Setting CSR MSF_Clock_Control (enable receive sections) */
	set_msf(0x2C, 0x00003C0A);
	taskDelay(5);

	/* Setting CSR MSF_Tx_Control (select transmit clock) */
	set_msf(0x04, 0x00000904);
	taskDelay(5);

	/* Enabling CSR MSF_Tx_Control */
	set_msf(0x04, 0x20000D04);
	taskDelay(5);

	/* Setting CSR MSF_Clock_Control (enable transmit clocks) */
	set_msf(0x2C, 0x00003C5A);
	taskDelay(5);

	/* Setting CSR MSF_Clock_Control (enable transmit sections) */
	set_msf(0x2C, 0x00003D5F);
	taskDelay(5);


	#if (MEDIA_CARD == CROSS_ISLAND)

	/* Setting CSR TX_CALENDAR_LENGTH */
	set_msf(0x70, 0x00000010);
	taskDelay(5);

	/* Setting CSR TX_CALENDAR_0 */
	set_msf(0x1000, 0x00000000);
	taskDelay(5);

	#endif

	#if (MEDIA_CARD == GEORGES_ISLAND)

	/* Setting CSR TX_CALENDAR_LENGTH */
	set_msf(0x70, 0x0000000A);
	taskDelay(5);

	/* Setting CSR TX_CALENDAR_0 */
	set_msf(0x1000, 0x00000000);
	taskDelay(5);

	/* Setting CSR TX_CALENDAR_1 */
	set_msf(0x1004, 0x00000001);
	taskDelay(5);

	/* Setting CSR TX_CALENDAR_2 */
	set_msf(0x1008, 0x00000002);
	taskDelay(5);

	/* Setting CSR TX_CALENDAR_3 */
	set_msf(0x100C, 0x00000003);
	taskDelay(5);

	/* Setting CSR TX_CALENDAR_4 */
	set_msf(0x1010, 0x00000004);
	taskDelay(5);

	/* Setting CSR TX_CALENDAR_5 */
	set_msf(0x1014, 0x00000005);
	taskDelay(5);

	/* Setting CSR TX_CALENDAR_6 */
	set_msf(0x1018, 0x00000006);
	taskDelay(5);

	/* Setting CSR TX_CALENDAR_7 */
	set_msf(0x101C, 0x00000007);
	taskDelay(5);

	/* Setting CSR TX_CALENDAR_8 */
	set_msf(0x1020, 0x00000008);
	taskDelay(5);

	/* Setting CSR TX_CALENDAR_9 */
	set_msf(0x1024, 0x00000009);
	taskDelay(5);

	#endif

#if 0
	/* Setting CSR MSF_Clock_Control (enable receive clock) */
	set_msf(0x2C, 0x00003D7F);
	taskDelay(5);

	/* Setting CSR MSF_Clock_Control (enable receive calender section) */
	set_msf(0x2C, 0x00003F7F);
	taskDelay(5);
#endif

}


void ingress_media_init_2(void)
{
	/* Setting CSR MSF_Rx_Control (configure receive section for 2 partiton mode/duplex mode) */
	set_msf(0x0, 0x00000004);
	taskDelay(5);

	/* Setting CSR MSF_Clock_Control (enable receive clock select) */
	set_msf(0x2C, 0x00003150);
	taskDelay(5);

	/* Setting CSR MSF_Clock_Control (enable receive clock dll) */
	set_msf(0x2C, 0x00003D50);
	taskDelay(5);

	/* Setting CSR MSF_Clock_Control (enable receive sections) */
	set_msf(0x2C, 0x00003D50);
	taskDelay(5);

	/* Setting CSR MSF_Clock_Control (enable receive clock) */
	set_msf(0x2C, 0x00003D70);
	taskDelay(5);

	/* Setting CSR MSF_Clock_Control (enable receive calender section) */
	set_msf(0x2C, 0x00003F7D);
	taskDelay(5);

	/* Enabling CSR MSF_Tx_Control */
	/*set_msf(0x04, 0x80100D05);*/
	set_msf(0x04, 0x80100C05);
	taskDelay(5);

	/* Send data training to Egress */
	printf("\nSending continuous data training to Egress.\n");
	/*set_msf(0xa0, 0x00000100); *****************************************************/	
	set_msf(0xa0, 0x00008100);  
	taskDelay(5);

	printf("\n");

}


void egress_media_init_2(void)
{
	int msf_interrupt_status, dip4_err, rec_data_train;

	int attempt = 1;
	int MAX_ATTEMPT = 10;

	/* Wait for received data training MSF_INTERRUPT_STATUS (bit 7 should be set) */

	while(attempt <= MAX_ATTEMPT)
	{

	printf("\nAttempt %d to receive data training from Ingress.\n", attempt);

	msf_interrupt_status = get_msf(0x08);
	taskDelay(5);

	printf("\tMSF_INTERRUPT_STATUS = 0x%x.\n", msf_interrupt_status);	

	dip4_err = (msf_interrupt_status >> 2) & 0x1;
	rec_data_train = (msf_interrupt_status >> 7) & 0x1;

	if (rec_data_train==0)
	{
		printf("\tNo data training received.\n");
	}

	if (dip4_err==1)
	{
		printf("\tDPI4 parity failed.\n");
		set_msf(0x08, 0x00000004);
		taskDelay(5);
	}

	if (rec_data_train==1 && dip4_err==0)
	{
		printf("\tData training received successfully!\n");
		break;
	}
	
	attempt++;

	}

	if (rec_data_train==0) printf("\nFailed to receive data training from Ingress after %d attempts.\n", MAX_ATTEMPT);

	/* Enabling CSR MSF_Rx_Control */
	set_msf(0x00, 0x80000005);
	taskDelay(5);

	/* Setting CSR CSIX_TYPE_MAP */
	set_msf(0x0010, 0x08);

	/* Send flow control training to Ingress */
	printf("\nSending flow control training to Ingress.\n");
	set_msf(0xa8, 0x00000004);
	taskDelay(5);

	printf("\n");

}


void ingress_media_init_3(void)
{
	int train_flow_control;
	int msf_interrupt_status, rec_flw_ctl_train, parity_err;

	int attempt = 1;
	int MAX_ATTEMPT = 10;

	/* Stop training Egress */
	printf("\nStopped sending continuous data training to Egress.\n");
        /*set_msf(0xa0, 0x00000000);******************************************/
	set_msf(0xa0, 0x00008000);
	taskDelay(5);

	/* Start receiving flow control training from Egress */
	train_flow_control = get_msf(0xa8);
	train_flow_control &= 0xFFFFFFFD;
	set_msf(0xa8, train_flow_control);
	taskDelay(5);

	/* Wait for flow control training MSF_INTERRUPT_STATUS (bit 9 should be set) */

	while(attempt <= MAX_ATTEMPT)
	{

	printf("\nAttempt %d to receive flow control training from Egress.\n", attempt);

	msf_interrupt_status = get_msf(0x08);
	taskDelay(5);

	printf("\tMSF_INTERRUPT_STATUS = 0x%x.\n", msf_interrupt_status);	

	parity_err = ((msf_interrupt_status >> 6) & 0x1) | ((msf_interrupt_status >> 15) & 0x1);
	rec_flw_ctl_train = (msf_interrupt_status >> 9) & 0x1;

	if (rec_flw_ctl_train==0)
	{
		printf("\tNo flow control training received.\n");
	}

	if (parity_err==1)
	{
		printf("\tParity failed.\n");
		set_msf(0x08, 0x00008040);
		taskDelay(5);
	}

	if (rec_flw_ctl_train==1 && parity_err==0)
	{
		printf("\tFlow control training received successfully!\n");
		break;
	}
	
	attempt++;

	}

	if (rec_flw_ctl_train==0) printf("\nFailed to receive flow control training from Egress after %d attempts.\n", MAX_ATTEMPT);

	/* Stop receiving flow control training */
	set_msf(0xa8, 0x00000002);
	taskDelay(5);

	/* Setting CSR FC_STATUS_OVERRIDE */
	set_msf(0x28, 0x00000000);
	taskDelay(5);

	printf("\n");
}


void egress_media_init_3(void)
{
	int msf_interrupt_status, rec_data_train, data_train_stopped;

	int attempt = 1;
	int MAX_ATTEMPT = 10;

	/* Clear REC_DATA_TRAIN */
	set_msf(0x08, 0x00000080);
	taskDelay(5);

	/* Wait for end of data training MSF_INTERRUPT_STATUS (bit 7 should be reset) */

	while(attempt <= MAX_ATTEMPT)
	{

	printf("\nAttempt %d to stop receiving data training from Ingress.\n", attempt);

	msf_interrupt_status = get_msf(0x08);
	taskDelay(5);

	printf("\tMSF_INTERRUPT_STATUS = 0x%x.\n", msf_interrupt_status);	

	rec_data_train = (msf_interrupt_status >> 7) & 0x1;
	data_train_stopped = (msf_interrupt_status >> 12) & 0x1;

	if (rec_data_train==1 || data_train_stopped==0)
	{
		printf("\tStill receiving data training.\n");
		set_msf(0x08, 0x00000080);
		taskDelay(5);
	}

	if (rec_data_train==0 && data_train_stopped==1)
	{
		printf("\tStopped receiving data training!\n");
		break;
	}
	
	attempt++;

	}

	if (rec_data_train==1 || data_train_stopped==0)
		printf("\nFailed to stop receiving data training from Ingress after %d attempts.\n", MAX_ATTEMPT);

	printf("\n");

	#if (MEDIA_CARD == CROSS_ISLAND)
	/* Initialize Cross Island */
	cross_island_init();
	#endif

	#if (MEDIA_CARD == GEORGES_ISLAND)
	/* Initialize Georges Island */
	georges_island_init();
	#endif

	/* Turns training of to Georges */
	set_msf(0xa0, 0x00000000);
	taskDelay(5);

	/* Setting CSR FC_STATUS_OVERRIDE */
	set_msf(0x28, 0x00000000);
	taskDelay(5);

}


void ingress_media_init_4(void)
{

	int msf_interrupt_status, dip4_err, rec_data_train, data_train_stopped, train_done_bit;

	int attempt = 1;
	int MAX_ATTEMPT = 10;

	/* Setting CSR MSF_Clock_Control (enable receive clock select) */
/*	set_msf(0x2C, 0x00003155);
	taskDelay(5);*/

	/* Setting CSR MSF_Clock_Control (enable receive clock dll) */
/*	set_msf(0x2C, 0x00003D55);
	taskDelay(5);*/

	/* Setting CSR MSF_Clock_Control (enable receive sections) */
/*	set_msf(0x2C, 0x00003D5F);
	taskDelay(5);*/

	/* Setting CSR MSF_Clock_Control (enable receive clock) */
/*	set_msf(0x2C, 0x00003D7F);
	taskDelay(5);*/

	/* Setting CSR MSF_Clock_Control (enable receive calender section) */
	set_msf(0x2C, 0x00003F7F);
	taskDelay(5);

	/* Enabling CSR MSF_Rx_Control */
	set_msf(0x0, 0x40000004);
	taskDelay(5);

	#if (MEDIA_CARD == CROSS_ISLAND)
	/* Setting CSR RX_CALENDAR_LENGTH to a value that Cross Island expects */
	set_msf(0x48, 0x00000010);
	taskDelay(5);
	#endif

	#if (MEDIA_CARD == GEORGES_ISLAND)
	/* Setting CSR RX_CALENDAR_LENGTH to a value that Georges Island expects */
	set_msf(0x48, 0x0000000A);
	taskDelay(5);
	#endif


	/* Wait for received data training MSF_INTERRUPT_STATUS (bit 7 should be set) */
	/* clear dip4 error bit*/
	set_msf(0x08, 0x0000000f);

	while(attempt <= MAX_ATTEMPT)
	{

	printf("\nAttempt %d to receive data training and deskew the interface.\n", attempt);

	msf_interrupt_status = get_msf(0x08);
	taskDelay(5);

	printf("\tMSF_INTERRUPT_STATUS = 0x%x.\n", msf_interrupt_status);	

	dip4_err = (msf_interrupt_status >> 2) & 0x1;
	rec_data_train = (msf_interrupt_status >> 7) & 0x1;

	if (rec_data_train==0)
	{
		printf("\tNo data training received.\n");
	}

	if (dip4_err==1)
	{
		printf("\tDPI4 parity failed.\n");
		set_msf(0x08, 0x00000004);
		taskDelay(5);
	}

	if (rec_data_train==1 && dip4_err==0)
	{
		printf("\tData training received and interface deskewed successfully!\n");
		break;
	}
	
	attempt++;

	}

	if (rec_data_train==0)
		printf("\nFailed to receive data training after %d attempts.\n", MAX_ATTEMPT);
	

	/* Setting RSTAT_EN to tell the media card that Ingress received training */
	printf("\n");
	printf("\tMSF_INTERRUPT_STATUS before sending status = 0x%x.\n", msf_interrupt_status);
	set_msf(0xa0, 0x00008002);		
	taskDelay(5);

	attempt = 1;
	while (attempt <= MAX_ATTEMPT)
	{
	printf("\nAttempt %d to check Ingress NPU train stop bit.\n", attempt);

	msf_interrupt_status = get_msf(0x08);
	taskDelay(5);
	printf("\tMSF_INTERRUPT_STATUS = 0x%x.\n", msf_interrupt_status);

	train_done_bit = (msf_interrupt_status >>12) & 0x1;
	if (train_done_bit == 1)
	{
		printf("\tIngress NPU detected train stop from IXF1110!\n");
		break;
	}
	if (train_done_bit == 0)
	{
		printf("\tIngress NPU never detected train stop from IXF1110!\n");
		taskDelay(5);	
	}
	

	attempt++;
	}	
	if (train_done_bit ==0)
		printf("\nIngress NPU failed to deskew SPI-4 interface after %d attempts.\n", MAX_ATTEMPT);

	printf("\n");

	/* Wait for end of data training MSF_INTERRUPT_STATUS (bit 12 should be set) */

	/* Clear REC_DATA_TRAIN */
	msf_interrupt_status = get_msf(0x08);
	printf("\n");
	printf("\tMSF_INTERRUPT_STATUS before clearing data train bit = 0x%x.\n", msf_interrupt_status);	
	set_msf(0x08, 0x00000080);
	msf_interrupt_status = get_msf(0x08);
	printf("\tMSF_INTERRUPT_STATUS after clearing data train bit = 0x%x.\n", msf_interrupt_status);	
	printf("\n");
	taskDelay(5);

	attempt = 1;

	while(attempt <= MAX_ATTEMPT)
	{

	printf("\nAttempt %d to stop receiving data training.\n", attempt);

	msf_interrupt_status = get_msf(0x08);
	taskDelay(5);

	printf("\tMSF_INTERRUPT_STATUS = 0x%x.\n", msf_interrupt_status);	

	data_train_stopped = (msf_interrupt_status >> 7) & 0x1; /*change this bit from 12 to 7*/

	if (data_train_stopped==1)
	{
		printf("\tStill receiving data training.\n");
		set_msf(0x08, 0x00000080);	/*attempt to clear data train bit*/
	}

	if (data_train_stopped==0)
	{
		printf("\tStopped receiving data training!\n");
		break;
	}
	
	attempt++;

	}		

	if (data_train_stopped==1)
		printf("\nFailed to stop receiving data training after %d attempts.\n", MAX_ATTEMPT);

	printf("\n");

}


void egress_media_init_4(void)
{
	int media_stat, media_rec_FIFO_stat;
	int attempt = 1;
	int MAX_ATTEMPT = 10;

	if (MEDIA_CARD == CROSS_ISLAND)
	{
		while(attempt <= MAX_ATTEMPT)
		{

			printf("\nAttempt %d by Cross Island to receive FIFO status from Ingress.\n", attempt);

			media_stat = GET32(0xC600C024);
			taskDelay(5);

			if (media_stat==0x5)
			{
				printf("\tReceived FIFO status from Ingress! (status = 0x%x)\n", media_stat);
				break;
			}
			else
			{
				printf("\tDid not receive FIFO status from Ingress. (status = 0x%x)\n", media_stat);
			}	

			attempt++;
		}

		if (media_stat!=0x5)
		printf("\nFailed to receive FIFO status from Ingress after %d attempts.\n", MAX_ATTEMPT);

	}
	
	if (MEDIA_CARD == GEORGES_ISLAND)
	{
		while(attempt <= MAX_ATTEMPT)
		{

			printf("\nAttempt %d by Georges Island to receive FIFO status from Ingress.\n", attempt);
			media_stat = GET32(0xC6001c08);
			taskDelay(5);
			media_rec_FIFO_stat = (media_stat >> 13) & 0x1;

			if (media_rec_FIFO_stat ==0x1)
			{
				printf("\tReceived FIFO status from Ingress! (status = 0x%x)\n", media_stat);
				break;
			}
			else
			{
				printf("\tDid not receive FIFO status from Ingress. (status = 0x%x)\n", media_stat);
			}

			attempt++;

		}

		if (media_rec_FIFO_stat !=0x1)
		{
			printf("\nFailed to receive FIFO status from Ingress after %d attempts.\n", MAX_ATTEMPT);
			printf("Please verify that GI has a B0 IXF1110 MAC.\n");
		}

	}

	printf("\n");
	/* disable Ben Nevis Periodic training*/
	PUT32(0xc6001c04, 0x0000);

}

void write_ingress_phasemon_reg(void)
{
	set_msf(0x0300, 0x0000000e);
	set_msf(0x0304, 0x0000000f);
	set_msf(0x0308, 0x0000000e);
	set_msf(0x030c, 0x0000000e);
	set_msf(0x0310, 0x0000000e);
	set_msf(0x0314, 0x0000000e);
	set_msf(0x0318, 0x0000000e);
	set_msf(0x031c, 0x0000000d);
	set_msf(0x0320, 0x0000000f);
	set_msf(0x0324, 0x0000000e);
	set_msf(0x0328, 0x0000000e);
	set_msf(0x032c, 0x0000000e);
	set_msf(0x0330, 0x0000000d);
	set_msf(0x0334, 0x0000000d);
	set_msf(0x0338, 0x0000000c);
	set_msf(0x033c, 0x0000000e);
}


void ingress_test_packet(void)
{
	set_msf(0x2000, 0xc43a0000);
	set_msf(0x2004, 0x00050200);
	set_msf(0x2008, 0x00ff0024);
	set_msf(0x200C, 0x00000000);
	set_msf(0x2010, 0x00050000);
	set_msf(0x2014, 0x00000000);
	set_msf(0x2018, 0x45000028);
	set_msf(0x201C, 0x00000000);
	set_msf(0x2020, 0x0306a90f);
	set_msf(0x2024, 0x18877638);
	set_msf(0x2028, 0xc001c000);
	set_msf(0x202C, 0x00010203);
	set_msf(0x2030, 0x04050607);
	set_msf(0x2034, 0x08090a0b);
	set_msf(0x2038, 0x0c0d0e0f);
	set_msf(0x203C, 0x10111213);

	set_msf(0x1800, 0x40000001);
	set_msf(0x1804, 0x87654321);

}


void egress_test_packet(void)
{
	int base_addr;

	#if (MEDIA_TYPE == POS)

	base_addr = 0x2000;

	set_msf(base_addr, 0x00214500);
	set_msf(base_addr+4, 0x00280000);
	set_msf(base_addr+8, 0x00000406);
	set_msf(base_addr+12, 0xa80f1887);
	set_msf(base_addr+16, 0x7638c001);
	set_msf(base_addr+20, 0xc0000001);
	set_msf(base_addr+24, 0x02030405);
	set_msf(base_addr+28, 0x06070809);
	set_msf(base_addr+32, 0x0a0b0c0d);
	set_msf(base_addr+36, 0x0e0f1011);
	set_msf(base_addr+40, 0x12130000);

	base_addr = 0x1800;

	set_msf(base_addr, 0x30000300);
	set_msf(base_addr+4, 0x00000000);

	#endif

	#if (MEDIA_TYPE == ETH_1x10)

	base_addr = 0x2000;

	set_msf(base_addr, 0x01010101);
	set_msf(base_addr+4, 0x02030506);
	set_msf(base_addr+8, 0x0708090a);
	set_msf(base_addr+12, 0x08004500);
	set_msf(base_addr+16, 0x00320000);
	set_msf(base_addr+20, 0x00000406);
	set_msf(base_addr+24, 0x2cc40a00);
	set_msf(base_addr+28, 0x0001c001);
	set_msf(base_addr+32, 0xc0000000);
	set_msf(base_addr+36, 0x00010002);
	set_msf(base_addr+40, 0x00030004);
	set_msf(base_addr+44, 0x00050006);
	set_msf(base_addr+48, 0x00070008);
	set_msf(base_addr+52, 0x0009000a);
	set_msf(base_addr+56, 0x000b000c);
	set_msf(base_addr+60, 0x000d000e);

	base_addr = 0x1800;

	set_msf(base_addr, 0x40000300);
	set_msf(base_addr+4, 0x00000000);

	#endif

	#if (MEDIA_TYPE == ETH_10x1)

	int i;

	for(i=0; i<10; i++)
	{
		base_addr = 0x2000 + i*128;

		set_msf(base_addr, 0x01010101);
		set_msf(base_addr+4, 0x02030506);
		set_msf(base_addr+8, 0x0708090a);
		set_msf(base_addr+12, 0x08004500);
		set_msf(base_addr+16, 0x00320000);
		set_msf(base_addr+20, 0x00000406);
		set_msf(base_addr+24, 0x2cc40a00);
		set_msf(base_addr+28, 0x0001c001);
		set_msf(base_addr+32, 0xc0000000);
		set_msf(base_addr+36, 0x00010002);
		set_msf(base_addr+40, 0x00030004);
		set_msf(base_addr+44, 0x00050006);
		set_msf(base_addr+48, 0x00070008);
		set_msf(base_addr+52, 0x0009000a);
		set_msf(base_addr+56, 0x000b000c);
		set_msf(base_addr+60, 0x000d000e);

		base_addr = 0x1800 + i*16;

		set_msf(base_addr, (0x40000300+i));
		set_msf(base_addr+4, 0x00000000);
	}

	#endif
}


/* Shorter function names */
void imi1(void)
{	
    ingress_media_init_1();
}

void emi1(void)
{	
    egress_media_init_1();
}

void imi2(void)
{	
    ingress_media_init_2();
}

void emi2(void)
{	
    egress_media_init_2();
}

void imi3(void)
{	
    ingress_media_init_3();
}

void emi3(void)
{	
    egress_media_init_3();
	
}

void imi4(void)
{	
    ingress_media_init_4();
}

void emi4(void)
{	
    egress_media_init_4();
	
}

void itp(void)
{
	ingress_test_packet();
}

void etp(void)
{
	egress_test_packet();
}

void wpm(void)
{
	write_ingress_phasemon_reg();
}
