
/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      xscale_me_init.h
 *
 * = DESCRIPTION
 *      This file contains global definitions and macros used by the 
 *      source files of 10Gb Ethernet application.
 *
 * = AUTHOR
 *	ddt
 *
 * = CHANGE HISTORY
 *      04/13/03	Created for 4Gb Ethernet (IXP2400)
 *	09/09/03	Updated for 10Gb Ethernet (IXP2800)
 * ============================================================================
 */
#ifndef	__XSCALE_ME_INIT_H__
#define	__XSCALE_ME_INIT_H__

/*
 * Bunch of include files
 * ======================
 */

#include	"halMmap.h"

/* Macro to write and read one longword from memory 
   the macro assumes that the memory being read or written to
   is mapped 
 */

/*
 * Memory mapped base addresses.
 * ==============================
 */

extern void *VIRT_SRAM_CH0_BASE;
extern void *VIRT_SRAM_CH1_BASE;
extern void *VIRT_SRAM_CH2_BASE;
extern void *VIRT_SRAM_CH3_BASE;

extern void *VIRT_DRAM_BASE;
extern void *VIRT_SCRATCH_BASE;
extern void *VIRT_MSF_CSR_BASE;
extern void *VIRT_MSF_RBUF_BASE;
extern void *VIRT_CAP_CSR_GLOBAL_BASE;
extern void *VIRT_CAP_CSR_SCRATCH_CONFIG;
extern void *VIRT_XSCALE_LOCAL_CSR_BASE;

/* memory */
#define SRAM_CH0_BASE			VIRT_SRAM_CH0_BASE
#define SRAM_CH1_BASE			VIRT_SRAM_CH1_BASE
#define SRAM_CH2_BASE			VIRT_SRAM_CH2_BASE
#define SRAM_CH3_BASE			VIRT_SRAM_CH3_BASE

#define SDRAM_BASE	        	VIRT_DRAM_BASE
#define SCRATCH_BASE       		VIRT_SCRATCH_BASE

/* msf */
#define	MSF_CSR_BASE			VIRT_MSF_CSR_BASE
#define	MSF_RBUF_BASE			VIRT_MSF_RBUF_BASE

#define	MSF_TBUF_BASE			MSF_RBUF_BASE

/* csr */
#define	CAP_CSR_GLOBAL_BASE		VIRT_CAP_CSR_GLOBAL_BASE
#define	CAP_CSR_SCRATCH			VIRT_CAP_CSR_SCRATCH_CONFIG
#define	XSCALE_LOCAL_CSR_BASE	VIRT_XSCALE_LOCAL_CSR_BASE 



/*
 * specific CSRs addresses
 * -----------------------
 */
#define	IXP_RESET_0				CAP_CSR_GLOBAL_BASE + 0x0c
#define	MSF_CLK_CTRL_REG		CAP_CSR_GLOBAL_BASE + 0x08
#define	MSF_RX_CTRL				MSF_CSR_BASE + 0x00
#define	MSF_RX_UP_CTRL_0		MSF_CSR_BASE + 0x80
#define	MSF_RX_UP_CTRL_1		MSF_CSR_BASE + 0x84
#define	MSF_RX_UP_CTRL_2		MSF_CSR_BASE + 0x88
#define	MSF_RX_UP_CTRL_3		MSF_CSR_BASE + 0x8c
#define	MSF_TX_CTRL				MSF_CSR_BASE + 0x04
#define	MSF_TX_UP_CTRL_0		MSF_CSR_BASE + 0x90
#define	MSF_TX_UP_CTRL_1		MSF_CSR_BASE + 0x94
#define	MSF_TX_UP_CTRL_2		MSF_CSR_BASE + 0x98
#define	MSF_TX_UP_CTRL_3		MSF_CSR_BASE + 0x9c
#define	MSF_RBUF_ELE_DONE		MSF_CSR_BASE + 0x44
#define	SCRATCH_RING_STATUS		XSCALE_LOCAL_CSR_BASE + 0x58
#define	MSF_INTR_STATUS			MSF_CSR_BASE + 0x08
#define	MSF_TX_SEQ_0			MSF_CSR_BASE + 0x60
#define	MSF_TX_SEQ_1			MSF_CSR_BASE + 0x64
#define	MSF_TX_SEQ_2			MSF_CSR_BASE + 0x68
#define	MSF_TX_SEQ_3			MSF_CSR_BASE + 0x6c

#define META_SIZE               32
#define	INGRESS_BUF_SRAM_SIZE	BUFFER_SIZE * META_SIZE
#define	EGRESS_BUF_SRAM_SIZE	BUFFER_SIZE * META_SIZE

/* offsets of counters in big_endian format.*/
#define	OFFSET_PKTS_RX			0 	/* Packets Rxed */
#define	OFFSET_PKTS_FWD			4 	/* Packets Fwd */
#define	OFFSET_PKTS_DROP		8 	/* Packets dropped */
#define	OFFSET_PKTS_EXCP		12 	/* Packets excp */
#define	OFFSET_PKTS_BAD_HDR		16 	/* pkts bad cksum */
#define	OFFSET_PKTS_BAD_LEN		20 	/* pkts bad len */
#define	OFFSET_PKTS_BAD_TTL		24 	/* pkts with bad TTL */
#define	OFFSET_PKTS_NO_ROUTE   	28 	/* packets with no route */
#define	OFFSET_PKTS_TOO_SMALL  	32 	/* packets too small */


extern char* PACKET_COUNTERS_SRAM_BASE;	
extern char* CSIX_TX_CTX_SRAM_BASE;
extern char* CSIX_TX_COUNTERS_SRAM_BASE;
extern char* CSIX_RX_COUNTERS_SRAM_BASE;
extern char* SCHED_QUEUE_STRUCTURES_BASE;
extern char* SCHED_ROUND_COUNTERS_BASE;
extern char* IPV6_TRIE_TABLE_SRAM_BASE_8;
extern char* IPV6_TRIE_TABLE_SRAM_BASE_16;
extern char* IPV6_L3_NEXTHOP_TABLE_SRAM_BASE;
extern char* IPV6_ROUTER_PREFIX_SRAM_BASE;
extern char* TUNNEL_ENCAP_NEXTHOP_SRAM_BASE;
extern char* TUNNEL_DECAP_NEXTHOP_SRAM_BASE;

extern char* CSIX_RX_CTX_SRAM_BASE;
extern char* L2_TABLE_SRAM_BASE; 		
extern char* SRAM_FLT_TABLE_BASE;	
extern char* SRAM_PORT_BIT_MAP_BASE;
extern char* TRIE_TABLE_SRAM_BASE;
extern char* NEXTHOP_TABLE_SRAM_BASE;
extern char* DBCAST_TABLE_SRAM_BASE;
extern char* SRAM_ZERO_BLOCK;		
extern char* QD_SRAM_BASE;	
extern char* IPV4_STATS_TABLE_BASE;
extern char* BUF_SDRAM_BASE;        
extern char* INGRESS_BUF_SRAM_BASE;
extern char* EGRESS_BUF_SRAM_BASE;

/*
 * DEBUGGING UTILITIES 
 * -------------------
 */

#define	DBG_MSG(str,args...)		printf(str,##args)


/*
 * function prototypes
 */
extern void xscale_me_mmap_init(void);
extern void xscale_me_buffer_stats(char *);

#endif	/* __XSCALE_ME_INIT_H__ */

