/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      xscale_me_rm_app.c
 *
 * = DESCRIPTION
 *      This is a sample application that will exercise Resource Manager 
 *      functions to download the microcode image after paching symbols, 
 *      start the MEs and handle RM->ME and ME->RM 
 *
 * = AUTHOR
 *	ddt
 *
 * = CHANGE HISTORY
 * 	04/17/03	Created for 4Gb Ethernet (IXP2400)
 * 	09/09/03	Updated for 10Gb Ethernet (IXP2800)
 * ============================================================================
 */


/*
 * Include files
 */
#include	"xscale_me_rm_app.h"


/*
 * variables used by functions borrowed from
 * script files
 */
char* SRAM_FLT_TABLE_BASE;	
char* SRAM_PORT_BIT_MAP_BASE;		
char* TRIE_TABLE_SRAM_BASE;	
char* NEXTHOP_TABLE_SRAM_BASE;
char* DBCAST_TABLE_SRAM_BASE;
char* QD_SRAM_BASE;	
char* IPV4_STATS_TABLE_BASE;
char* SCHED_QUEUE_STRUCTURES_BASE;
char* SCHED_ROUND_COUNTERS_BASE;
char* IPV6_TRIE_TABLE_SRAM_BASE_8;
char* IPV6_TRIE_TABLE_SRAM_BASE_16;
char* IPV6_L3_NEXTHOP_TABLE_SRAM_BASE;
char* IPV6_ROUTER_PREFIX_SRAM_BASE;
char* TUNNEL_ENCAP_NEXTHOP_SRAM_BASE;
char* TUNNEL_DECAP_NEXTHOP_SRAM_BASE;
char* TUNNEL_INGRESS_LIST_SRAM_BASE;
char* BUF_SDRAM_BASE;        
char* PACKET_COUNTERS_SRAM_BASE;
char* CSIX_TX_CTX_SRAM_BASE;
char* CSIX_TX_COUNTERS_SRAM_BASE;
char* CSIX_RX_COUNTERS_SRAM_BASE;
char* CSIX_RX_CTX_SRAM_BASE;	
char* L2_TABLE_SRAM_BASE; 		
char* SRAM_ZERO_BLOCK;
char* INGRESS_BUF_SRAM_BASE;
char* EGRESS_BUF_SRAM_BASE;

/*
 * Globals for Ingress 
 */
static char* s_IngressUOFFile = 
	"c:\\ixa_sdk_3.5\\src\\applications\\ipv4_v6_forwarder\\10gb_ethernet\\10x1GbE_ingress\\wbench_project\\10gb_ethernet_ixp2800.uof";

/*
 * buffer free list
 */
ix_buffer_free_list_handle ingressHwFreeList = 0;
ix_buffer_free_list_info   ingressHwFreeListInfo ;

/*
 * Xscale - ME comm rings
 */
ix_hw_ring_handle ingressCoreToMeCtrlRing = 0;

t_app_mem_desc	ingress_mem[] = {
	/*
	 * TODO : add your memory map here. Name should be same as symbol
	 * to ease lookup later
	 * 	ddt: Move TRIE table to channel 1 (from 3) for TCAM workaround
	 */
	/* memType			memChan	size  value	name		varName*/
	{IX_MEMORY_TYPE_SRAM,1,2*1024*1024,	NULL,"TRIE_TABLE_SRAM_BASE",&TRIE_TABLE_SRAM_BASE},
	{IX_MEMORY_TYPE_SRAM,1,1024,		NULL,"SRAM_FLT_TABLE_BASE",&SRAM_FLT_TABLE_BASE},
	{IX_MEMORY_TYPE_SRAM,1,128,		NULL,"SRAM_PORT_BIT_MAP_BASE",&SRAM_PORT_BIT_MAP_BASE},
	{IX_MEMORY_TYPE_SRAM,2,1024,		NULL,"PACKET_COUNTERS_SRAM_BASE",&PACKET_COUNTERS_SRAM_BASE},
	{IX_MEMORY_TYPE_SRAM,0,32768,		NULL,"QD_SRAM_BASE",&QD_SRAM_BASE},
/*	{IX_MEMORY_TYPE_SRAM,0,2048,		NULL,"SCHED_VOQ_CREDIT_BASE",NULL}, */
	{IX_MEMORY_TYPE_SRAM,0,1*1024*1024,	NULL,"CSIX_TX_CTX_SRAM_BASE",&CSIX_TX_CTX_SRAM_BASE},
	{IX_MEMORY_TYPE_SRAM,1,1024,		NULL,"CSIX_TX_COUNTERS_SRAM_BASE",&CSIX_TX_COUNTERS_SRAM_BASE},
	{IX_MEMORY_TYPE_SRAM,0,128,		NULL,"SRAM_ZERO_BLOCK",&SRAM_ZERO_BLOCK},
	{IX_MEMORY_TYPE_SCRATCH,0,2048,		NULL,"IPV4_STATS_TABLE_BASE",&IPV4_STATS_TABLE_BASE},
	{IX_MEMORY_TYPE_SRAM,2,8192,		NULL,"DBCAST_TABLE_SRAM_BASE",&DBCAST_TABLE_SRAM_BASE},
/*	{IX_MEMORY_TYPE_SCRATCH,0,128,		NULL,"CONTROL_BLOCK_SCRATCH_BASE",NULL}, */
	{IX_MEMORY_TYPE_SRAM,2,4*1024,		NULL,"NEXTHOP_TABLE_SRAM_BASE",&NEXTHOP_TABLE_SRAM_BASE},
	{IX_MEMORY_TYPE_SRAM,3,1024,		NULL,"IPV6_TRIE_TABLE_SRAM_BASE_8",&IPV6_TRIE_TABLE_SRAM_BASE_8},
	{IX_MEMORY_TYPE_SRAM,3,1024,		NULL,"IPV6_TRIE_TABLE_SRAM_BASE_16",&IPV6_TRIE_TABLE_SRAM_BASE_16},
	{IX_MEMORY_TYPE_SRAM,3,1024,		NULL,"IPV6_L3_NEXTHOP_TABLE_SRAM_BASE",&IPV6_L3_NEXTHOP_TABLE_SRAM_BASE},
	{IX_MEMORY_TYPE_SRAM,0,128,		NULL,"IPV6_ROUTER_PREFIX_SRAM_BASE",&IPV6_ROUTER_PREFIX_SRAM_BASE},
	{IX_MEMORY_TYPE_SRAM,2,32*256,		NULL,"TUNNEL_ENCAP_NEXTHOP_SRAM_BASE",&TUNNEL_ENCAP_NEXTHOP_SRAM_BASE},	/* 256 entries */
	{IX_MEMORY_TYPE_SRAM,2,4*128,		NULL,"TUNNEL_DECAP_NEXTHOP_SRAM_BASE",&TUNNEL_DECAP_NEXTHOP_SRAM_BASE},	/* 128 entries */
	{IX_MEMORY_TYPE_SRAM,1,16384,		NULL,"TUNNEL_INGRESS_LIST_SRAM_BASE",&TUNNEL_INGRESS_LIST_SRAM_BASE}
};
#define	INGRESS_MEM_NUM_ENTRIES	sizeof(ingress_mem)/sizeof(ingress_mem[0])


/*
 * Globals for Egress 
 */

static char* s_EgressUOFFile = 
	"C:\\IXA_SDK_3.5\\src\\applications\\ipv4_v6_forwarder\\10gb_ethernet\\10x1GbE_egress\\wbench_project\\10x1gb_ethernet_egress.uof";

	/*
	 * buffer free list
	 */
ix_buffer_free_list_handle egressHwFreeList = 0;
ix_buffer_free_list_info   egressHwFreeListInfo ;

	/*
	 * Xscale - ME comm rings
	 */
ix_hw_ring_handle egressCoreToMeCtrlRing = 0;

t_app_mem_desc	egress_mem[] = {
	/*
	 * TODO : add your memory map here. Name should be same as symbol
	 * to ease lookup later
	 */
	/* memType	memChan	size  		value	name		varName */
	{IX_MEMORY_TYPE_SRAM,1,2*1024*1024,	NULL,"L2_TABLE_SRAM_BASE",&L2_TABLE_SRAM_BASE},
	{IX_MEMORY_TYPE_SRAM,1,32*1024,		NULL,"CSIX_RX_CTX_SRAM_BASE",&CSIX_RX_CTX_SRAM_BASE},
	{IX_MEMORY_TYPE_SRAM,2,1024,		NULL,"CSIX_RX_COUNTERS_SRAM_BASE",&CSIX_RX_COUNTERS_SRAM_BASE},
	{IX_MEMORY_TYPE_SRAM,0,32768,		NULL,"QD_SRAM_BASE",&QD_SRAM_BASE},
	{IX_MEMORY_TYPE_SRAM,0,128,		NULL,"SRAM_ZERO_BLOCK",&SRAM_ZERO_BLOCK},
	{IX_MEMORY_TYPE_SRAM,2,8*1024,		NULL,"SCHED_QUEUE_STRUCTURES_BASE",&SCHED_QUEUE_STRUCTURES_BASE},
	{IX_MEMORY_TYPE_SRAM,2,131072,		NULL,"SCHED_ROUND_COUNTERS_BASE", &SCHED_ROUND_COUNTERS_BASE}
};

#define	EGRESS_MEM_NUM_ENTRIES	sizeof(egress_mem)/sizeof(egress_mem[0])

unsigned int	numExcpPkts;
int start_micro_engines = 1;
ix_uint32 msg[3]; 
ix_uint32 size;

/*
 * Sample packet handler
 *
 * NOTE : This function cann't handle packets at high rates with the 
 * DBG_MSGs turned on.
 */
ix_error xscale_me_pkt_handler(
	ix_buffer_handle arg_hBuffer, 
   	ix_uint32 arg_UserData,		/* exception code */
   	void* arg_pContext 
   	)
{
    const char me[] = "xscale_me_pkt_handler";

    ix_error err = IX_SUCCESS;
    ix_hw_buffer_meta* pMeta;
    unsigned char* pData ;


	ix_memory_type memType;
	ix_uint32	memChan;
	ix_uint32	offset1, offset2;
	void		*pAddr;

	numExcpPkts++;

	DBG_MSG("\n%s : Called for buffer 0x%x.\n",me,(unsigned int)arg_hBuffer);
			
    if(arg_hBuffer == 0xff)
		return err;


    err = ix_rm_buffer_get_meta(arg_hBuffer,(void**)&pMeta);
	if(err != IX_SUCCESS)
   		return err;

	/*
	 *  add meta data processing here
	 */

    err = ix_rm_buffer_get_data(arg_hBuffer, (void**)&pData);
	if(err != IX_SUCCESS)
   		return err;

	pData = pData + pMeta->m_Offset;

	err = ix_rm_get_phys_offset(
			(const void*) pMeta, &memType,&memChan,&offset1,&pAddr);
	if(err != IX_SUCCESS)
		return err;

	err = ix_rm_get_phys_offset(
			(const void*) pData, &memType,&memChan,&offset2,&pAddr);
	if(err != IX_SUCCESS)
		return err;

	
	DBG_MSG("Meta data at 0x%x(offset 0x%x)  pkt data starts at 0x%x(offset 0x%x)\n",
					(unsigned int)pMeta, (unsigned int)offset1,
					(unsigned int)pData, (unsigned int)offset2);
	
	/*
	 * add pkt data processing here
	 */
       
	/*
	 * Now, produce messages to microengines
	 *
	 */
	if ((int)arg_pContext == INGRESS_HANDLER)
	{

		switch(arg_UserData)
		{
			case 0x11:
				DBG_MSG("\t... Options ...\n");break;
			case 0x12:
				DBG_MSG("\t... length mismatch ...\n");break;
			case 0x13:
				DBG_MSG("\t... bad ttl ...\n");break;
			case 0x14:
				DBG_MSG("\t... multicast ...\n");break;
			case 0x15:
				DBG_MSG("\t... frag required ...\n");break;
			case 0x16:
				DBG_MSG("\t... redirect ...\n");break;
			case 0x17:
				DBG_MSG("\t... down ...\n");break;
			case 0x18:
				DBG_MSG("\t... no route ...\n");break;
			case 0x19:
				DBG_MSG("\t... local delivery ...\n");break;
			case 0x1a:
				DBG_MSG("\t... limited broadcast ...\n");break;
			default	:
				DBG_MSG("\t... UNKNOWN exception 0x%x ...\n",(int)arg_UserData);break;
		}

		/*
		 * ingress processing
		 */
		
		DBG_MSG("%s : Dropping packet ...\n",me);
   		return ix_rm_buffer_free_chain(arg_hBuffer);
#if 0	
		/* code here is only to show how packet can be sent back to MEs. This is just
		an example code snippet */
		msg[0] = arg_hBuffer;
		msg[1] = eop_handle;	/* need to travel the buffer chain to find this */
		msg[2] = BID_QM;

  		size = 3;
   		err = ix_rm_hw_ring_put(ingressCoreToMeCtrlRing, &size, msg);
   		if(err != IX_SUCCESS)
   		{
   			DBG_MSG("%s : possible control ring full condition \n",me) ;
   			return ix_rm_buffer_free_chain(arg_hBuffer);
   		}
#endif
		
	}
	else
		if ((int)arg_pContext == EGRESS_HANDLER)
		{
			/*
			 * egress processing
			 */
		DBG_MSG("%s : Dropping packet ...\n",me);
   		return ix_rm_buffer_free_chain(arg_hBuffer);

		}

   	return IX_SUCCESS;
}


/*
 * Symbol lookup. If match is found, converts the virtual address
 * into memory offset
 */
unsigned int	_sym_to_offset(char *sym,int numEntries, t_app_mem_desc *p)
{
	const char me[]="_sym_to_offset";
	int cnt;
	ix_memory_type memType;
	ix_uint32	memChan;
	ix_uint32	offset;
	void		*pAddr;
	ix_error 	err;

	for(cnt=0;cnt<numEntries;cnt++)
	{
		if (!strcmp(p[cnt].name,sym))
		{

			err = ix_rm_get_phys_offset(
				(const void*) p[cnt].value,
				&memType,&memChan,&offset,&pAddr);
    		if(err != IX_SUCCESS)
			{
				DBG_MSG("SYM_TO_VAL problem Virt addr 0x%x for entry %d.\n",
					(unsigned int)p[cnt].value,cnt);
				return -1;
			}
			else
			{
#if 0
				DBG_MSG("symbol %s, value 0x%x.\n",
						p[cnt].name, (unsigned int)p[cnt].value);
#endif
				return ((offset & 0x3fffffff) | (memChan << 30));
			}
		}
	}

	DBG_MSG("%s : Symbol %s not found.\n",me,sym);

	return -1; /* meaning no match */
}

void*	_sym_to_virt(char *sym,int numEntries, t_app_mem_desc *p)
{
	int cnt;

	for(cnt=0;cnt<numEntries;cnt++)
		if (!strcmp(p[cnt].name,sym))
			return p[cnt].value;

	return (void *)0x0; /* meaning no match */
}

/*
 * function to print list of import variables per ME
 * from UOF file. To be called after the UOF file is set
 */
void	_list_import_vars(char *filename)
{
	const char me[]="_list_import_vars";
	int status;
	uclo_objHandle_T *objHandle;
	uof_Image_T         *image;
	uof_encapPage_T     *pages;
	int meNum;

	if (!filename)
	{
		DBG_MSG("ERROR : Specify file name.\n");
		return;
	}
	/*
	 * load the obj file
	 */
	status = UcLo_LoadObjFile((void **)&objHandle,filename);
	if(status != UCLO_SUCCESS)
	{
		DBG_MSG("\n%s : LoadObjFile %s FAILED with status 0x%x.\n",me,filename,status);
		return;
	}

	DBG_MSG("\n============== %s : START ===============\n\n",me);
	/*
	 * for each ME
	 */
	for(meNum=0;meNum<IX_MAX_MICROENGINE_NUMBER;meNum++)
	{
		int p;

		image = objHandle->encapMeImage[meNum].imagePtr;
		pages = objHandle->encapMeImage[meNum].pages;

		DBG_MSG("ME NUMBER = %d\n",meNum);

		if(!image || !pages)
		{
			DBG_MSG("\t=== NO image loaded ===\n");
			continue;
		}

		/* traverse the pages */
		for(p = 0; p < image->numOfPage; p++)
		{
			unsigned int v;

			/* update all instance of the variable */
			for(v = 0; v < pages[p].numImpVar; v++)
			{ 
				DBG_MSG("\t%s\n",
					UcLo_getString(&objHandle->strTable,pages[p].impVar[v].name));
			}/* for */
		} /* for pages */
	}/* for ME */

	/*
	 * delete the object
	 */
	UcLo_DeleObj(objHandle);

	DBG_MSG("\n============== %s : DONE ===============\n\n",me);
	return;
}

	
/*
 * Ingress Xscale application
 * ==========================
 */

ix_error rm_app_init_ingress(void)
{
	ix_error err = IX_SUCCESS;

    	const char me[] = "rm_app_init_ingress";


	/*
	 * commnunication between blocks
	 */
    	ix_hw_ring_handle pktrxToIpv4Ring = 0;
	ix_hw_ring_handle ipv4ToStatsRing  = 0;	 	

	/*
	 * misc variables
	 */

#define	_INGRESS_NUM_SYMBOLS		32
    ix_imported_symbol aImportSymbols[_INGRESS_NUM_SYMBOLS];

    ix_uint32 dl_rel_base = 0;
    ix_uint32 i;

    ix_uint32 temp_meta_address = 0;
    ix_uint32 temp_data_address = 0;

	/*
	 * for memory conversion calls
	 */
	ix_memory_type	memType;
	ix_uint32	memChan;
	ix_uint32	offset;
	void		*pAddr;
	unsigned int 	*pTmp;

	/**********************************************/

	DBG_MSG("%s : CALLED\n",me);

	DBG_MSG("%s : Setting packet receive mode ...\n",me);
    err = ix_rm_packet_set_receive_mode(IX_CC_IPV4_PKT_MICROBLOCK_ID, IX_COMM_ID_MODE_CALLBACK);
    if(err != IX_SUCCESS)
        return err;

	DBG_MSG("%s : Setting message receive mode ...\n",me);
    err = ix_rm_message_set_receive_mode(IX_CC_IPV4_PKT_MICROBLOCK_ID, IX_COMM_ID_MODE_CALLBACK);
    if(err != IX_SUCCESS)
        return err;

	DBG_MSG("%s : Registering packet handler ...\n",me);
    err = ix_rm_packet_handler_register(IX_CC_IPV4_PKT_MICROBLOCK_ID, xscale_me_pkt_handler, 
			(void*)INGRESS_HANDLER);
    if(err != IX_SUCCESS)
        goto error;

	DBG_MSG("%s : Registering message handler ...\n",me);
   	err = ix_rm_message_handler_register(IX_CC_IPV4_PKT_MICROBLOCK_ID, xscale_me_pkt_handler, 
			(void*)INGRESS_HANDLER);
    if(err != IX_SUCCESS)
        goto error;

	/*
	 * Create Xscale to ME scratch rings
	 */
    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_128, 
			IX_ME_COMM_CONTROL_RING_ID, &ingressCoreToMeCtrlRing);
    if(err != IX_SUCCESS)
         goto error;
	DBG_MSG("%s : Created Xscale to ME control ring 0x%.8x\n",me,(unsigned int)ingressCoreToMeCtrlRing);

	/*
	 * Create message scratch rings
	 */

    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_512, 
			IX_PKTRX_TO_IPV4_COMM_RING_ID, &pktrxToIpv4Ring) ;
    if(err != IX_SUCCESS)
        goto error;
	DBG_MSG("%s : Creating Pkt Rx to IPV4 ring 0x%.8x\n",me,(unsigned int)pktrxToIpv4Ring);

    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_512, 
			IX_IPV4_TO_STATS_COMM_RING_ID, &ipv4ToStatsRing) ;
    if(err != IX_SUCCESS)
        goto error;
	DBG_MSG("%s : Created IPV4 to Stats ring 0x%.8x\n",me,(unsigned int)ipv4ToStatsRing);

   /* 
	* Create  a free list
	*/
	DBG_MSG("%s : Creating buffer free list...\n",me);
    err = ix_rm_hw_buffer_free_list_create(INGRESS_NUM_BUFFERS, 
		sizeof(ix_hw_buffer_meta), INGRESS_BUF_SIZE, 
		INGRESS_BUF_SRAM_CHAN, INGRESS_BUF_DRAM_CHAN, 
        &ingressHwFreeList) ;
    if( err != IX_SUCCESS)
    {
     	DBG_MSG("%s : Creating buffer free list FAILED with err = 0x%x\n",me, err);
	return err ;
    }

    /* 
	 * get information on the free list to get the values of variables to 
	 * be patched 
	 */
	DBG_MSG("%s : Reading buffer free list info...\n",me);
    ix_rm_buffer_free_list_get_info(
			ingressHwFreeList, &ingressHwFreeListInfo) ;

	/*
	 * Create the memory segments
	 */
	for(i=0;i<INGRESS_MEM_NUM_ENTRIES;i++)
	{
		err = ix_rm_mem_alloc(ingress_mem[i].memType,
				ingress_mem[i].memChan,
				ingress_mem[i].size,
				&(ingress_mem[i].value));
		if (err != IX_SUCCESS)
		{
		    DBG_MSG("%s : ix_rm_mem_alloc FAILED with err = 0x%x\n",me, err);
		    goto error;
		}

		if (ingress_mem[i].pVar)
				*(ingress_mem[i].pVar) = (unsigned char*)ingress_mem[i].value;
		
		DBG_MSG("Allocated %d bytes at 0x%x using RM for %s.\n",
						(int)ingress_mem[i].size,(unsigned int)ingress_mem[i].value,
						ingress_mem[i].name);
	}

	/*
	 * Simply print all symbols
	 */
	/*_list_import_vars(s_IngressUOFFile); */

	/*
	 * Now we are ready to download the microcode
	 */
	DBG_MSG("%s : Setting microcode file ...\n",me); 
    err =  ix_rm_ueng_set_ucode(s_IngressUOFFile);
    if(err != IX_SUCCESS)
    {
	DBG_MSG("\n\nix_rm_ueng_set_ucode failed with err code = 0x%x\n\n.", err);
        goto error;
    }

    /* 
	 * Set the variables to be patched 
	 *
	 */
 
   	DBG_MSG("%s : Patching microcode symbols ...\n",me); 
    aImportSymbols[0].m_Name  = "BUF_SRAM_BASE" ;
	err = ix_rm_get_phys_offset(
			(const void*) ingressHwFreeListInfo.m_pMetaBaseAddress,
			&memType,&memChan,&offset,&pAddr);
    if(err != IX_SUCCESS)
	{
		DBG_MSG("%s : Can't convert Virt addr 0x%x.\n",
			me,(unsigned int)ingressHwFreeListInfo.m_pMetaBaseAddress);
        goto error;
	}
	else
	{
		DBG_MSG("%s : SRAM Meta base offset is 0x%x, Phy addr is 0x%x\n",
				me,(unsigned int)offset, (unsigned int)(*(unsigned int**)pAddr));
	}
    aImportSymbols[0].m_Value = ( ix_uint32) (offset & 0x3fffffff);
	_PRINT_SYMBOL(0);
	INGRESS_BUF_SRAM_BASE = (unsigned char*)ingressHwFreeListInfo.m_pMetaBaseAddress;


	temp_meta_address = (ix_uint32) (offset & 0x3fffffff);
	err = ix_rm_get_phys_offset(
			(const void*) ingressHwFreeListInfo.m_pDataBaseAddress,
			&memType,&memChan,&offset,&pAddr);
	if(err != IX_SUCCESS)
		goto error;
    temp_data_address = ( ix_uint32) (offset & 0xffffffc0);
	DBG_MSG("%s : Buf DRAM base offset = 0x%x\n",me,(unsigned int)temp_data_address);
	BUF_SDRAM_BASE = (unsigned char*)ingressHwFreeListInfo.m_pDataBaseAddress;

    dl_rel_base = temp_data_address - 
		( temp_meta_address * 
           (ingressHwFreeListInfo.m_DataElementSize / 
			ingressHwFreeListInfo.m_MetaElementSize)) ;

    aImportSymbols[1].m_Value =  ( ix_uint32) ingressHwFreeListInfo.m_FreeListInfo;
    aImportSymbols[1].m_Name  = "BUF_FREE_LIST0" ;
	_PRINT_SYMBOL(1);

    /* patch BUF_SRAM_BASE and BUF_FREE_LIST0 on all MEs */
    for(i = 0; i < (IX_MAX_MICROENGINE_NUMBER - 2); i++)
    {
	if ( (i == 3) || (i == 4) || (i == 6) || (i == 7) || (i == 8) )
	    continue;

        err = ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[i],2,aImportSymbols);

        if(err != IX_SUCCESS)
	{
		DBG_MSG("%s : Symbol BUF_SRAM_BASE/BUF_FREE_LIST0 failed for ME %d.\n",me,(int)i);
            	goto error;
	}
    }

    /* patch DL_REL_BASE */

    aImportSymbols[0].m_Value =  dl_rel_base ;
    aImportSymbols[0].m_Name  = "DL_REL_BASE" ;
	_PRINT_SYMBOL(0);

    for(i = 0; i < (IX_MAX_MICROENGINE_NUMBER - 2); i++)
    {
	if ( (i == 3) || (i == 4) || (i == 5) || (i == 6) || (i == 7) || (i == 9) || (i == 11) )
	    continue;

        err = ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[i],1,aImportSymbols);

        if(err != IX_SUCCESS)
	{
		DBG_MSG("%s : Symbol DL_REL_BASE failed for ME %d.\n",me,(int)i);
            	goto error;
	}
    }


    /* 
	 * Patching for Packet Rx 
	 */
     
    aImportSymbols[0].m_Value =  ( ix_uint32) ingressHwFreeListInfo.m_FreeListInfo1;
    aImportSymbols[0].m_Name  = "FREE_LIST_ID" ;
	_PRINT_SYMBOL(0);

/*
 * Clean up warning messages

    aImportSymbols[1].m_Value = 
		_sym_to_offset("PACKET_COUNTERS_SRAM_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[1].m_Value == -1)
		goto error;
    aImportSymbols[1].m_Name  = "PACKET_COUNTERS_SRAM_BASE" ;
	_PRINT_SYMBOL(1);

    err = ix_rm_ueng_patch_symbols(0x12,2,aImportSymbols); 
*/
    err = ix_rm_ueng_patch_symbols(0x12,1,aImportSymbols); 
    if(err != IX_SUCCESS)
    {
	DBG_MSG("%s : Patching failed for ME 1:2.\n",me);
        goto error;
    }
    err = ix_rm_ueng_patch_symbols(0x13,1,aImportSymbols); 
    if(err != IX_SUCCESS)
    {
	DBG_MSG("%s : Patching failed for ME 1:3.\n",me);
        goto error;
    }

    /* 
	 * Patching for QM 
	 */

    aImportSymbols[0].m_Value = 
		_sym_to_offset("QD_SRAM_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[0].m_Value == -1)
		goto error;
    aImportSymbols[0].m_Name  = "QD_SRAM_BASE" ;
	_PRINT_SYMBOL(0);

     err = ix_rm_ueng_patch_symbols(7,1,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;

    /* 
	 * Patching for Statistics
	 */

     err = ix_rm_ueng_patch_symbols(5,1,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;


    /* 
	 * Patching for CSIX TX 
	 */

    aImportSymbols[0].m_Value = 
		_sym_to_offset("CSIX_TX_CTX_SRAM_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[0].m_Value == -1)
		goto error;
    aImportSymbols[0].m_Name  = "CSIX_TX_CTX_SRAM_BASE" ;
	_PRINT_SYMBOL(0);

    aImportSymbols[1].m_Value = THIS_BLADE_ID;
    aImportSymbols[1].m_Name  = "THIS_BLADE_ID" ;
	_PRINT_SYMBOL(1);

/*
 * Clean up warning messages

    aImportSymbols[2].m_Value = 
		_sym_to_offset("SRAM_ZERO_BLOCK",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[2].m_Value == -1)
		goto error;
    aImportSymbols[2].m_Name  = "SRAM_ZERO_BLOCK" ;
	_PRINT_SYMBOL(2);


    aImportSymbols[3].m_Value = 
		_sym_to_offset("CSIX_TX_COUNTERS_SRAM_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[3].m_Value == -1)
		goto error;
    aImportSymbols[3].m_Name  = "CSIX_TX_COUNTERS_SRAM_BASE" ;
	_PRINT_SYMBOL(3);

    err = ix_rm_ueng_patch_symbols(0x10,4,aImportSymbols); 
*/
    err = ix_rm_ueng_patch_symbols(0x10,2,aImportSymbols); 
    if(err != IX_SUCCESS)
    {
	DBG_MSG("%s : Patching failed for ME 1:0.\n",me);
        goto error;
    }

/*
 * Clean up warning messages

    err = ix_rm_ueng_patch_symbols(0x11,3,aImportSymbols); 
    if(err != IX_SUCCESS)
    {
	DBG_MSG("%s : Patching failed for ME 1:1.\n",me);
        goto error;
    }
*/

    /* 
	 * Patching for IPV4 
	 */

    aImportSymbols[0].m_Value = 
		_sym_to_offset("TRIE_TABLE_SRAM_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[0].m_Value == -1)
		goto error;
    
    aImportSymbols[0].m_Name  = "TRIE_TABLE_SRAM_BASE";
	_PRINT_SYMBOL(0);

    aImportSymbols[1].m_Value = 
		_sym_to_offset("IPV4_STATS_TABLE_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[1].m_Value == -1)
		goto error;
    aImportSymbols[1].m_Name  = "IPV4_STATS_TABLE_BASE";
	_PRINT_SYMBOL(1);
   
    aImportSymbols[2].m_Value = 
		_sym_to_offset("DBCAST_TABLE_SRAM_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[2].m_Value == -1)
		goto error;
    aImportSymbols[2].m_Name  = "DBCAST_TABLE_SRAM_BASE";
	_PRINT_SYMBOL(2);

    aImportSymbols[3].m_Value = THIS_BLADE_ID;
    aImportSymbols[3].m_Name  = "THIS_BLADE_ID";
	_PRINT_SYMBOL(3);

    aImportSymbols[4].m_Value = 
		_sym_to_offset("NEXTHOP_TABLE_SRAM_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[4].m_Value == -1)
		goto error;
    aImportSymbols[4].m_Name  = "NEXTHOP_TABLE_SRAM_BASE";
	_PRINT_SYMBOL(4);

    aImportSymbols[5].m_Value = 
		_sym_to_offset("SRAM_FLT_TABLE_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[5].m_Value == -1)
		goto error;
    aImportSymbols[5].m_Name  = "SRAM_FLT_TABLE_BASE" ;
	_PRINT_SYMBOL(5);

    aImportSymbols[6].m_Value = 
		_sym_to_offset("SRAM_PORT_BIT_MAP_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[6].m_Value == -1)
		goto error;
    aImportSymbols[6].m_Name  = "SRAM_PORT_BIT_MAP_BASE" ;
	_PRINT_SYMBOL(6);

    aImportSymbols[7].m_Value = 
		_sym_to_offset("IPV6_TRIE_TABLE_SRAM_BASE_8",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[7].m_Value == -1)
		goto error;
    aImportSymbols[7].m_Name  = "IPV6_TRIE_TABLE_SRAM_BASE_8";
	_PRINT_SYMBOL(7);

    aImportSymbols[8].m_Value = 
		_sym_to_offset("IPV6_TRIE_TABLE_SRAM_BASE_16",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[8].m_Value == -1)
		goto error;
    aImportSymbols[8].m_Name  = "IPV6_TRIE_TABLE_SRAM_BASE_16";
	_PRINT_SYMBOL(8);

    aImportSymbols[9].m_Value = 
		_sym_to_offset("IPV6_L3_NEXTHOP_TABLE_SRAM_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[9].m_Value == -1)
		goto error;
    aImportSymbols[9].m_Name  = "IPV6_L3_NEXTHOP_TABLE_SRAM_BASE";
	_PRINT_SYMBOL(9);


    aImportSymbols[10].m_Value = 
		_sym_to_offset("IPV6_ROUTER_PREFIX_SRAM_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[10].m_Value == -1)
		goto error;
    aImportSymbols[10].m_Name  = "IPV6_ROUTER_PREFIX_SRAM_BASE";
	_PRINT_SYMBOL(10);

    aImportSymbols[11].m_Value = 
		_sym_to_offset("TUNNEL_ENCAP_NEXTHOP_SRAM_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[11].m_Value == -1)
		goto error;
    aImportSymbols[11].m_Name  = "TUNNEL_ENCAP_NEXTHOP_SRAM_BASE";
	_PRINT_SYMBOL(11);

    aImportSymbols[12].m_Value = 
		_sym_to_offset("TUNNEL_DECAP_NEXTHOP_SRAM_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[12].m_Value == -1)
		goto error;
    aImportSymbols[12].m_Name  = "TUNNEL_DECAP_NEXTHOP_SRAM_BASE";
	_PRINT_SYMBOL(12);

    aImportSymbols[13].m_Value = 
		_sym_to_offset("TUNNEL_INGRESS_LIST_SRAM_BASE",INGRESS_MEM_NUM_ENTRIES,
				ingress_mem);
    if(aImportSymbols[13].m_Value == -1)
		goto error;
    aImportSymbols[13].m_Name  = "TUNNEL_INGRESS_LIST_SRAM_BASE";
	_PRINT_SYMBOL(13);



	/*
	 * initialize control block
	 */
    err = ix_rm_ueng_patch_symbols(0x0,14,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;

    err = ix_rm_ueng_patch_symbols(0x1,14,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;
 
    err = ix_rm_ueng_patch_symbols(0x2,14,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;

    err = ix_rm_ueng_patch_symbols(0x14,14,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;

    err = ix_rm_ueng_patch_symbols(0x15,14,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;


	return err;

	/*
	 * error processing
	 */
error:

	DBG_MSG("%s : ERROR cleaning up mess for INGRESS...\n",me);
    return err;

}

ix_error rm_app_start_ingress_me(void)
{
	/*
	 * starts ingress MEs
	 */
	ix_error err;
	const char me[]="rm_app_start_ingress_me";
    /* start running the microcode */

    DBG_MSG("%s : Loading the microcode... \n",me) ;
    err = ix_rm_ueng_load();
    if(err != IX_SUCCESS)
        goto error;

	if (!start_micro_engines)
	{
		DBG_MSG("%s: Microengines NOT STARTED as per user selection\n");
		return err;
	}


    DBG_MSG("%s : Starting the microengine 0x%x... \n",me,0x15) ;
    err = ix_rm_ueng_start(0x15,0xff);
    if(err != IX_SUCCESS)
        goto error;

    DBG_MSG("%s : Starting the microengine 0x%x... \n",me,0x14) ;
    err = ix_rm_ueng_start(0x14,0xff);
    if(err != IX_SUCCESS)
        goto error;

    DBG_MSG("%s : Starting the microengine 0x%x... \n",me,0x13) ;
    err = ix_rm_ueng_start(0x13,0xff);
    if(err != IX_SUCCESS)
        goto error;

    DBG_MSG("%s : Starting the microengine 0x%x... \n",me,0x12) ;
    err = ix_rm_ueng_start(0x12,0xff);
    if(err != IX_SUCCESS)
        goto error;

    DBG_MSG("%s : Starting the microengine 0x%x... \n",me,0x11) ;
    err = ix_rm_ueng_start(0x11,0xff);
    if(err != IX_SUCCESS)
        goto error;

    DBG_MSG("%s : Starting the microengine 0x%x... \n",me,0x10) ;
    err = ix_rm_ueng_start(0x10,0xff);
    if(err != IX_SUCCESS)
        goto error;

    DBG_MSG("%s : Starting the microengine 0x%x... \n",me,0x7) ;
    err = ix_rm_ueng_start(7,0xff);
    if(err != IX_SUCCESS)
        goto error;

    DBG_MSG("%s : Starting the microengine 0x%x... \n",me,0x6) ;
    err = ix_rm_ueng_start(6,0xff);
    if(err != IX_SUCCESS)
        goto error;

    DBG_MSG("%s : Starting the microengine 0x%x... \n",me,0x5) ;
    err = ix_rm_ueng_start(5,0xff);
    if(err != IX_SUCCESS)
        goto error;

    DBG_MSG("%s : Starting the microengine 0x%x... \n",me,0x2) ;
    err = ix_rm_ueng_start(2,0xff);
    if(err != IX_SUCCESS)
        goto error;

    DBG_MSG("%s : Starting the microengine 0x%x... \n",me,0x1) ;
    err = ix_rm_ueng_start(1,0xff);
    if(err != IX_SUCCESS)
        goto error;

    DBG_MSG("%s : Starting the microengine 0x%x... \n",me,0x0) ;
    err = ix_rm_ueng_start(0,0xff);
    if(err != IX_SUCCESS)
        goto error;



    DBG_MSG("%s : MICROENGINES are RUNNING !!!\n",me) ;

error:
	return err;

}

/*
 * Egress application initialization code
 */
ix_error rm_app_init_egress(void)
{
	ix_error err = IX_SUCCESS;

    const char me[] = "rm_app_init_egress";

	/*
	 * scratch rings for message comminication
	 */
    ix_hw_ring_handle csixRxToStatsRing = 		0;
	ix_hw_ring_handle etherARPToStatsRing = 	0;
	ix_hw_ring_handle classCountPortSchedRing = 0;
    ix_hw_ring_handle qmToPktTxRing_0  = 		0;
    ix_hw_ring_handle qmToPktTxRing_1  = 		0;

	/*
	 * misc variables
	 */
#define	_EGRESS_NUM_SYMBOLS		32
    ix_imported_symbol aImportSymbols[_EGRESS_NUM_SYMBOLS];

    ix_uint32 dl_rel_base = 0;
    ix_uint32 i;

    ix_uint32 temp_meta_address = 0;
    ix_uint32 temp_data_address = 0;

	/*
	 * for memory conversion calls
	 */
	ix_memory_type memType;
	ix_uint32	memChan;
	ix_uint32	offset;
	void		*pAddr;
	unsigned int *pTmp;

	/**********************************************/

	DBG_MSG("%s : CALLED\n",me);

	DBG_MSG("%s : Setting packet receive mode ...\n",me);
    err = ix_rm_packet_set_receive_mode(IX_CC_ETH_TX_PKT_ID, IX_COMM_ID_MODE_CALLBACK);
    if(err != IX_SUCCESS)
        return err;

	DBG_MSG("%s : Setting message receive mode ...\n",me);
    err = ix_rm_message_set_receive_mode(IX_CC_ETH_TX_PKT_ID, IX_COMM_ID_MODE_CALLBACK);
    if(err != IX_SUCCESS)
        return err;


	DBG_MSG("%s : Registering packet handler ...\n",me);
    err = ix_rm_packet_handler_register(IX_CC_ETH_TX_PKT_ID, 
			xscale_me_pkt_handler, (void*)EGRESS_HANDLER);
    if(err != IX_SUCCESS)
        goto error;

	DBG_MSG("%s : Registering message handler ...\n",me);
    err = ix_rm_message_handler_register(IX_CC_ETH_TX_PKT_ID, 
			xscale_me_pkt_handler, (void*)EGRESS_HANDLER);
    if(err != IX_SUCCESS)
        goto error;


	/*
	 * Create Xscale to ME scratch rings
	 */
    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_128, 
			IX_ME_COMM_CONTROL_RING_ID, &egressCoreToMeCtrlRing);
    if(err != IX_SUCCESS)
         goto error;
	DBG_MSG("%s : Created Xscale to ME control ring 0x%.8x\n",me,(unsigned int)egressCoreToMeCtrlRing);

	/*
	 * Create message scratch rings
	 */
    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_512, 
			IX_CSIXRX_TO_STATS_COMM_RING_ID, &csixRxToStatsRing);
    if(err != IX_SUCCESS)
         goto error;
	DBG_MSG("%s : Creating CSIX RX to Stats ring 0x%.8x\n",me,(unsigned int)csixRxToStatsRing);

    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_512, 
			IX_ETHER_ARP_TO_STATS_COMM_RING_ID, &etherARPToStatsRing);
    if(err != IX_SUCCESS)
         goto error;
 	DBG_MSG("%s : Creating Ethernet_ARP to Stats ring 0x%.8x\n",me,(unsigned int)etherARPToStatsRing);

    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_512, 
			IX_CLASS_COUNT_PORT_SCHED_COMM_RING_ID, &classCountPortSchedRing);
    if(err != IX_SUCCESS)
         goto error;
	DBG_MSG("%s : Creating Class-Count-Port internal Sched ring 0x%.8x\n",me,(unsigned int)classCountPortSchedRing);

    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_256, 
			IX_QM_TO_PKTTX_COMM_RING_ID_0, &qmToPktTxRing_0);
    if(err != IX_SUCCESS)
         goto error;
	DBG_MSG("%s : Created QM to Packet Tx ring (first) 0x%.8x\n",me,(unsigned int)qmToPktTxRing_0);

    err = ix_rm_hw_scratch_ring_create(0, IX_SCRATCH_RING_SIZE_256, 
			IX_QM_TO_PKTTX_COMM_RING_ID_1, &qmToPktTxRing_1);
    if(err != IX_SUCCESS)
         goto error;
	DBG_MSG("%s : Created QM to Packet Tx ring (second) 0x%.8x\n",me,(unsigned int)qmToPktTxRing_1);

   /*
	* Create  a free list
	*/
	DBG_MSG("%s : Creating buffer free list...\n",me);
    err = ix_rm_hw_buffer_free_list_create(EGRESS_NUM_BUFFERS, 
		sizeof(ix_hw_buffer_meta), EGRESS_BUF_SIZE, 
		EGRESS_BUF_SRAM_CHAN, EGRESS_BUF_DRAM_CHAN, 
        &egressHwFreeList) ;
    if( err != IX_SUCCESS)
    {
     	DBG_MSG("%s : Creating buffer free list FAILED with err = 0x%x\n",me, err);
	return err ;
    }

    /* 
	 * get information on the free list to get the values of variables to 
	 * be patched 
	 */
	DBG_MSG("%s : Reading buffer free list info...\n",me);
    ix_rm_buffer_free_list_get_info(
			egressHwFreeList, &egressHwFreeListInfo) ;

	/*
	 * Create the memory segments
	 */
	for(i=0;i<EGRESS_MEM_NUM_ENTRIES;i++)
	{
		err = ix_rm_mem_alloc(egress_mem[i].memType,
				egress_mem[i].memChan,
				egress_mem[i].size,
				&(egress_mem[i].value));
		if (err != IX_SUCCESS)
			goto error;
		if (egress_mem[i].pVar)
				*(egress_mem[i].pVar) = (unsigned char*)egress_mem[i].value;
		DBG_MSG("Allocated %d bytes at 0x%x using RM for %s.\n",
						(int)egress_mem[i].size,(unsigned int)egress_mem[i].value,
						egress_mem[i].name);
	}

	/*
	 * Simply print all symbols
	 */
	/*_list_import_vars(s_EgressUOFFile);*/

	/*
	 * Now we are ready to download the microcode
	 */
	DBG_MSG("%s : Setting microcode file ...\n",me);
    err =  ix_rm_ueng_set_ucode(s_EgressUOFFile);
    if(err != IX_SUCCESS)
        goto error;

    /* 
	 * Set the variables to be patched 
	 *
	 */
	DBG_MSG("%s : Patching microcode symbols ...\n",me);
    aImportSymbols[0].m_Name  = "BUF_SRAM_BASE" ;
	err = ix_rm_get_phys_offset(
			(const void*) egressHwFreeListInfo.m_pMetaBaseAddress,
			&memType,&memChan,&offset,&pAddr);
    if(err != IX_SUCCESS)
	{
		DBG_MSG("%s : Can't convert Virt addr 0x%x.\n",
			me,(unsigned int)egressHwFreeListInfo.m_pMetaBaseAddress);
        goto error;
	}
	else
	{
		DBG_MSG("%s : SRAM Meta base offset is 0x%x, Phy addr is 0x%x\n",
				me,(unsigned int)offset, (unsigned int)(*(unsigned int**)pAddr));
	}

    aImportSymbols[0].m_Value = ( ix_uint32) (offset & 0x3fffffff);
	_PRINT_SYMBOL(0);
	EGRESS_BUF_SRAM_BASE = (unsigned char*)egressHwFreeListInfo.m_pMetaBaseAddress;

	temp_meta_address = (ix_uint32) (offset & 0x3fffffff);
	err = ix_rm_get_phys_offset(
			(const void*) egressHwFreeListInfo.m_pDataBaseAddress,
			&memType,&memChan,&offset,&pAddr);
	if(err != IX_SUCCESS)
		goto error;
    temp_data_address = ( ix_uint32) (offset & 0xffffffc0);
	DBG_MSG("%s : Buf DRAM base offset = 0x%x\n",me,(unsigned int)temp_data_address);
	BUF_SDRAM_BASE = (unsigned char*)egressHwFreeListInfo.m_pDataBaseAddress;

    dl_rel_base = temp_data_address - 
		( temp_meta_address * 
           (egressHwFreeListInfo.m_DataElementSize / 
			egressHwFreeListInfo.m_MetaElementSize)) ;

    aImportSymbols[1].m_Value =  ( ix_uint32) egressHwFreeListInfo.m_FreeListInfo;
    aImportSymbols[1].m_Name  = "BUF_FREE_LIST0" ;
	_PRINT_SYMBOL(1);

    /* Patching BUF_SRAM_BASE and BUF_FREE_LIST0 */
    for(i = 3; i < (IX_MAX_MICROENGINE_NUMBER - 4); i++)
    {

	/* patch for all microengines for now
	if ( (i == 5) || (i == 6) || (i == 7) )
	    continue;
	*/

        err = ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[i],2,aImportSymbols);
        if(err != IX_SUCCESS)
		{
			DBG_MSG("%s : Symbol BUF_SRAM_BASE/BUF_FREE_LIST0 failed for ME %d.\n",me,(int)i);
            goto error;
		}
    }

    /* Patching DL_REL_BASE */
    aImportSymbols[0].m_Value =  dl_rel_base ;
    aImportSymbols[0].m_Name  = "DL_REL_BASE" ;
	_PRINT_SYMBOL(0);

    for(i = 8; i < (IX_MAX_MICROENGINE_NUMBER - 4); i++)
    {
	/* patch for all microengines for now
	if (i == 10)
	    continue;
	*/

        err = ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[i],1,aImportSymbols);
        if(err != IX_SUCCESS)
		{
			DBG_MSG("%s : Symbol DL_REL_BASE failed for ME %d.\n",me,(int)i);
            goto error;
		}
    }

    /* Patching L2_TABLE_SRAM_BASE */
    aImportSymbols[0].m_Value = 
		_sym_to_offset("L2_TABLE_SRAM_BASE",EGRESS_MEM_NUM_ENTRIES,
				egress_mem);
    if(aImportSymbols[0].m_Value == -1)
		goto error;
    aImportSymbols[0].m_Name  = "L2_TABLE_SRAM_BASE" ;
	_PRINT_SYMBOL(0);    

    for(i = 3; i < (IX_MAX_MICROENGINE_NUMBER - 6); i++)
    {
	/* patch for all microengines for now
	if ( (i == 4) || (i == 5) || (i == 6) || (i == 7) )
	    continue;
	*/

        err = ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[i],1,aImportSymbols);
        if(err != IX_SUCCESS)
		{
			DBG_MSG("%s : Symbol L2_TABLE_SRAM_BASE failed for ME %d.\n",me,(int)i);
            goto error;
		}
    }

    /* 
	 * Patching for CSIX Rx 
	 */
    aImportSymbols[0].m_Value =  ( ix_uint32) egressHwFreeListInfo.m_FreeListInfo1;
    aImportSymbols[0].m_Name  = "FREE_LIST_ID" ;
	_PRINT_SYMBOL(0);

    err = ix_rm_ueng_patch_symbols(0x12,1,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;
    err = ix_rm_ueng_patch_symbols(0x13,1,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;

/*
 * Clean up warning messages

    aImportSymbols[0].m_Value = 
		_sym_to_offset("CSIX_RX_CTX_SRAM_BASE",EGRESS_MEM_NUM_ENTRIES,
				egress_mem);
    if(aImportSymbols[0].m_Value == -1)
		goto error;
    aImportSymbols[0].m_Name  = "CSIX_RX_CTX_SRAM_BASE" ;
	_PRINT_SYMBOL(0);
     
    aImportSymbols[1].m_Value =  ( ix_uint32) egressHwFreeListInfo.m_FreeListInfo1;
    aImportSymbols[1].m_Name  = "FREE_LIST_ID" ;
	_PRINT_SYMBOL(1);

    aImportSymbols[2].m_Value = 
		_sym_to_offset("CSIX_RX_COUNTERS_SRAM_BASE",EGRESS_MEM_NUM_ENTRIES,
				egress_mem);
    if(aImportSymbols[2].m_Value == -1)
		goto error;
    aImportSymbols[2].m_Name  = "CSIX_RX_COUNTERS_SRAM_BASE" ;
	_PRINT_SYMBOL(2);

    err = ix_rm_ueng_patch_symbols(0x12,3,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;
    err = ix_rm_ueng_patch_symbols(0x13,3,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;
*/

    /* 
	 * Patching for Tx_Helper
	 */

    aImportSymbols[0].m_Value = 
		_sym_to_offset("SCHED_QUEUE_STRUCTURES_BASE",EGRESS_MEM_NUM_ENTRIES,
				egress_mem);
    if(aImportSymbols[0].m_Value == -1)
		goto error;
    aImportSymbols[0].m_Name  = "SCHED_QUEUE_STRUCTURES_BASE" ;
	_PRINT_SYMBOL(0);

    err = ix_rm_ueng_patch_symbols(7,1,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;


    /* 
	 * Patching for Egress QM 
	 */

    aImportSymbols[0].m_Value = 
		_sym_to_offset("QD_SRAM_BASE",EGRESS_MEM_NUM_ENTRIES,
				egress_mem);
    if(aImportSymbols[0].m_Value == -1)
		goto error;
    aImportSymbols[0].m_Name  = "QD_SRAM_BASE" ;
	_PRINT_SYMBOL(0);

    err = ix_rm_ueng_patch_symbols(6,1,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;

    /* 
	 * Patching for Statistics
	 */

    err = ix_rm_ueng_patch_symbols(4,1,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;


   /* 
	* Patching for Egress Scheduler 
	*/

    aImportSymbols[0].m_Name  = "SRAM_ZERO_BLOCK" ;
    aImportSymbols[0].m_Value = 
		_sym_to_offset("SRAM_ZERO_BLOCK",EGRESS_MEM_NUM_ENTRIES,
				egress_mem);
    if(aImportSymbols[0].m_Value == -1)
		goto error;
		_PRINT_SYMBOL(0);

/*
 * Clean up warning messages

    aImportSymbols[1].m_Name  = "SCHED_ROUND_COUNTERS_BASE" ;
    aImportSymbols[1].m_Value = 
		_sym_to_offset("SCHED_ROUND_COUNTERS_BASE",EGRESS_MEM_NUM_ENTRIES,
				egress_mem);
    if(aImportSymbols[1].m_Value == -1)
		goto error;
		_PRINT_SYMBOL(1);

    err = ix_rm_ueng_patch_symbols(5,2,aImportSymbols); 
*/
    err = ix_rm_ueng_patch_symbols(5,1,aImportSymbols); 
    if(err != IX_SUCCESS)
        goto error;

	return err;
	/*
	 * to reset microengine, use
	 * ix_rm_ueng_reset(me_clus_num, 1)
	 */

	/*
	 * error processing
	 *
	 */
error:

	DBG_MSG("%s : ERROR cleaning up mess for EGRESS...\n",me);


    return err ;
}


ix_error rm_app_start_egress_me(void)
{
	/*
	 * start the microengines
	 */
	ix_error err;
	const char me[]="rm_app_start_egress_me";

	DBG_MSG("%s : Loading microcode...\n",me);
    err = ix_rm_ueng_load();
    if(err != IX_SUCCESS)
        goto error;
	
	if (!start_micro_engines)
	{
		DBG_MSG("%s: Microengines NOT STARTED as per user selection\n");
		return err;
	}
	
	DBG_MSG("%s : Starting microengine 0x%x...\n",me,0x13);
    err = ix_rm_ueng_start(0x13,0xff);
    if(err != IX_SUCCESS)
        goto error;

	DBG_MSG("%s : Starting microengine 0x%x...\n",me,0x12);
    err = ix_rm_ueng_start(0x12,0xff);
    if(err != IX_SUCCESS)
        goto error;

	DBG_MSG("%s : Starting microengine 0x%x...\n",me,0x11);
    err = ix_rm_ueng_start(0x11,0xff);
    if(err != IX_SUCCESS)
        goto error;

	DBG_MSG("%s : Starting microengine 0x%x...\n",me,0x10);
    err = ix_rm_ueng_start(0x10,0xff);
    if(err != IX_SUCCESS)
        goto error;

	DBG_MSG("%s : Starting microengine 0x%x...\n",me,0x7);
    err = ix_rm_ueng_start(0x7,0xff);
    if(err != IX_SUCCESS)
        goto error;

	DBG_MSG("%s : Starting microengine 0x%x...\n",me,0x6);
    err = ix_rm_ueng_start(0x6,0xff);
    if(err != IX_SUCCESS)
        goto error;

	DBG_MSG("%s : Starting microengine 0x%x...\n",me,0x5);
    err = ix_rm_ueng_start(0x5,0xff);
    if(err != IX_SUCCESS)
        goto error;

	DBG_MSG("%s : Starting microengine 0x%x...\n",me,0x4);
    err = ix_rm_ueng_start(0x4,0xff);
    if(err != IX_SUCCESS)
        goto error;

	DBG_MSG("%s : Starting microengine 0x%x...\n",me,0x3);
    err = ix_rm_ueng_start(0x3,0xff);
    if(err != IX_SUCCESS)
        goto error;

	DBG_MSG("%s : MICROENGINES are RUNNING !!!\n",me);

error:
	return err;
}

/*
 * XScale-ME Application Initialization 
 *
 * Function returns IX_SUCCESS if everything OK or
 * an ix_error error code.
 */
int xscale_me_rm_app_init(char *chip)
{
	const char me[]="xscale_me_rm_app_init";
    	ix_error err = IX_SUCCESS;

	DBG_MSG("%s : called.\n",me);

	/*
	 * Reset all
	 */
	DBG_MSG("%s : Resetting all microengines ...\n",me);
	ix_rm_ueng_reset_all(); 

    	/*
     	 * Add processor specific extra required initialization
     	 */

	if (INGRESS(chip))
	{
		/*
		 * Ingress initialization 
		 */
		err = rm_app_init_ingress();
		if (err != IX_SUCCESS)
		{
			DBG_MSG("%s : Ingress App Init FAILED!.\n",me);
			return err;
		}

	}
	else if(EGRESS(chip))
	{
		/*
		 * Egress initialization
		 */
		err = rm_app_init_egress();
		if (err != IX_SUCCESS)
		{
			DBG_MSG("Egress App Init FAILED!.\n");
			return err;
		}

	}
	DBG_MSG("%s : %s App Init SUCCESSFUL.\n",me,chip);

    /**
     * Add extra required finalization
     */

	return err;

}

/*
 * Xscale-ME application cleanup 
 */
int xscale_me_rm_app_fini(char *chip)
{
	ix_error err = IX_SUCCESS;

	/*
	 * terminate resource manager
	 */
	err = ix_rm_term();

	if (INGRESS(chip))
	{
		/*
		 * Ingess cleanup
		 */


	}
	else if (EGRESS(chip))
	{
		/*
		 * Egress cleanup
		 */
	}

	return err;
}

void	xscale_me_buffer_stats(char *chip)
{
	/* we have more buffers on Egress */
	ix_buffer_handle	a_Buf[EGRESS_NUM_BUFFERS], tmp;	
	ix_error	err;
	ix_buffer_handle low=0,high=0;
	int cnt;
	int num_buffers;
	ix_buffer_free_list_handle	hwFreeList;


	DBG_MSG("CAUTION : This function has to be called only after starting microengines\n");

	if(INGRESS(chip))
	{
		num_buffers = INGRESS_NUM_BUFFERS;
		hwFreeList = ingressHwFreeList;
	}
	else if(EGRESS(chip))
	{
		num_buffers = EGRESS_NUM_BUFFERS;
		hwFreeList = egressHwFreeList;
	}
	else
	{
		DBG_MSG("\nSpecify ingress/egress.\n\n");
		return;
	}

	/*
	 * first allocate all buffers and count
	 */
	for(cnt=0;cnt<num_buffers;cnt++)
	{
		err = ix_rm_buffer_alloc(hwFreeList,&tmp);
		if (err != IX_SUCCESS)
		{
			/* ix_error_dump(stderr,err);*/
			break;
		}
		else
		{
			a_Buf[cnt] = tmp;
			if (cnt==0)
			{
				low = tmp;high=tmp;
			}
			else
			{
				if (tmp > high)
					high = tmp;
				else if (tmp < low)
					low = tmp;
			}
		}
	}

	DBG_MSG("\nNumber of free buffers in Q-array is %d. Range is from 0x%x to 0x%x\n",
					cnt,(unsigned int)low,(unsigned int)high);

	/*
	 * then free all buffers
	 */
	for(cnt=cnt-1;cnt>=0;cnt--)
	{
		err = ix_rm_buffer_free(a_Buf[cnt]);
		if (err != IX_SUCCESS)
		{
			DBG_MSG("FAILED to free buffer 0x%x.\n",(unsigned int)a_Buf[cnt]);
		}
	}

	return;
}
