/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      xscale_me_rm_app.h
 *
 * = DESCRIPTION
 *      This file provides data structures and prototypes for the Xscale-ME
 *      application based on Resource Manager
 *
 * = AUTHOR
 *	ddt
 *
 * = CHANGE HISTORY
 * 	04/21/03	Created for 4Gb Ethernet (IXP2400)
 *	09/09/03	Updated for 10Gb Ethernet (IXP2800)
 * ============================================================================
 */

#if !defined(__XSCALE_ME_RM_APP_H__)
#define __XSCALE_ME_RM_APP_H__

#include "xscale_me_init.h"
#include "ix_ossl.h"
#include <ix_rm.h>

/*
 * for UCLO utilities
 */
#include "uclo.h"
#include "uclo_dev.h"
#include "uclo_utl.h"


/*
 * bindings
 */
#include "ix_cc_microengines_bindings.h"


/**
 * DESCRIPTION: This is the SCRATCH ring ID to be used for core to ME
 *  control communication
 *
 */
#define IX_ME_COMM_CONTROL_RING_ID        		2


/*
 * Ingress scratch ring definitions
 */
#define IX_PKTRX_TO_IPV4_COMM_RING_ID			4
#define	IX_IPV4_TO_STATS_COMM_RING_ID			5

/*
 * Egress scratch ring definitions
 */
#define IX_CSIXRX_TO_STATS_COMM_RING_ID			4
#define	IX_ETHER_ARP_TO_STATS_COMM_RING_ID		5
#define	IX_CLASS_COUNT_PORT_SCHED_COMM_RING_ID		6
#define IX_QM_TO_PKTTX_COMM_RING_ID_0			7
#define IX_QM_TO_PKTTX_COMM_RING_ID_1			8

#ifdef	_DEBUG_PRINT_SYMBOL_
#define	_PRINT_SYMBOL(a)	\
			DBG_MSG("\t%s = 0x%x\n",	\
			aImportSymbols[a].m_Name,(unsigned int)aImportSymbols[a].m_Value)
#else
#define	_PRINT_SYMBOL(a)
#endif

#define	INGRESS(s)	(!strcmp((s),"ingress")? 1:0)
#define	EGRESS(s)	(!strcmp((s),"egress")? 1:0)
#define	INGRESS_HANDLER		1
#define	EGRESS_HANDLER		2

#define	INGRESS_NUM_BUFFERS	4096
#define	INGRESS_BUF_SIZE	2048
#define	INGRESS_BUF_SRAM_CHAN	0
#define	INGRESS_BUF_DRAM_CHAN	0

#define	EGRESS_NUM_BUFFERS	4096
#define	EGRESS_BUF_SIZE		2048
#define	EGRESS_BUF_SRAM_CHAN	0
#define	EGRESS_BUF_DRAM_CHAN	0

#define	THIS_BLADE_ID		0xf
#define	BID_QM			0x23

typedef	struct	app_mem_desc
{
	ix_memory_type	memType;
	ix_uint32		memChan;
	ix_uint32		size;		/* size in bytes */
	void			*value;
	char			*name;
	char			**pVar;
} t_app_mem_desc;

extern int xscale_me_rm_app_init(char *chip);
extern int xscale_me_rm_app_fini(char *chip);
extern ix_error rm_app_start_ingress_me(void);
extern ix_error rm_app_start_egress_me(void);
extern unsigned int numExcpPkts;

#endif /* end !defined(__XSCALE_ME_RM_APP_H__) */
