//---------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//---------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: system_init.uc
//
//      Purpose: 
//
//      History:
//
//
//      Date            Comment                         By
//      ---------------------------------------------------------------------
//
//      11/19/2001      Created                         Senthil Nathan
//
//
/////////////////////////////////////////////////////////////////////////////

#ifndef __SYSTEM_INIT_UC__
#define __SYSTEM_INIT_UC__

#include <dispatch_loop.uc>

///////////////////////////////////////////////////////////////////////////////
// init_meta_cache_to_zero:
//	 	Description: 
//			Just sets the registers used for meta data caching to zero. (This 
//			helps a lot when debugging, because all the unused meta data regsiters
//			will be zero instead of something like 0x5a5a5a).
//
//		Outputs:
//
//	 	Inputs:
//
//		Size:   				
//
///////////////////////////////////////////////////////////////////////////////
#macro	init_meta_cache_to_zero()
.begin
.reg zero

	immed32(zero, 0)
	dl_meta_init_cache(zero, zero, zero, zero, zero, zero, zero, zero)	

.end
#endm

///////////////////////////////////////////////////////////////////////////////
// system_init:
//	 	Description: 
//			Does all the initialisation to get the system up and running.
//			Typically it initialises all the various rings, creates the buffer
//			free list, initialises the various cache (meta, ip header etc).
//			Once initialisation is done, a signal is sent to all other MEs
//			so that rest of the code, that depends on this initialisation, 
//			to run.
//
//		Outputs:
//
//	 	Inputs:
//
//		Size:   				
//
///////////////////////////////////////////////////////////////////////////////
#macro	system_init()


	; Initialize the various scratch rings used in the ingress pipeline

	init_scratch_ring(POS_RX_TO_IPV4_SCR_RING_BASE,
					  POS_RX_TO_IPV4_SCR_RING_SIZE,
					  POS_RX_TO_IPV4_SCR_RING)

	init_scratch_ring(IPV4_TO_QM_SCR_RING_BASE,
					  IPV4_TO_QM_SCR_RING_SIZE,
					  IPV4_TO_QM_SCR_RING)

	init_scratch_ring(EXCEPTION_RING_IN_0_BASE,
					  EXCEPTION_RING_IN_0_SIZE,
					  EXCEPTION_RING_IN_0)

	init_scratch_ring(EXCEPTION_RING_IN_1_BASE,
					  EXCEPTION_RING_IN_1_SIZE,
					  EXCEPTION_RING_IN_1)

	; Initialize the free buffer list

	dl_buffer_init()

	; To ease debugging. Not absolutely essential for proper execution.
	; Set all the dl_meta* gpr's to 0.
	init_meta_cache_to_zero()

	; Signal completion of system initialization to all ME's in the chip.
	signal_me(0x00, ME_INIT_SIGNAL)
	signal_me(0x01, ME_INIT_SIGNAL)
	signal_me(0x02, ME_INIT_SIGNAL)
	signal_me(0x03, ME_INIT_SIGNAL)
	signal_me(0x04, ME_INIT_SIGNAL)
	signal_me(0x05, ME_INIT_SIGNAL)
	signal_me(0x06, ME_INIT_SIGNAL)
	signal_me(0x07, ME_INIT_SIGNAL)

	signal_me(0x10, ME_INIT_SIGNAL)
	signal_me(0x11, ME_INIT_SIGNAL)
	signal_me(0x12, ME_INIT_SIGNAL)
	signal_me(0x13, ME_INIT_SIGNAL)
	signal_me(0x14, ME_INIT_SIGNAL)
	signal_me(0x15, ME_INIT_SIGNAL)
	signal_me(0x16, ME_INIT_SIGNAL)
	signal_me(0x17, ME_INIT_SIGNAL)								

#endm

#endif	//__SYSTEM_INIT_UC__
