/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      egress_init.c
 *
 * = DESCRIPTION
 *      This file contains scripts to initialize the IXP2800 Egress processor.
 *
 * = AUTHOR
 *      Ameya Varde
 *      ameya.s.varde@intel.com
 *
 * = CHANGE HISTORY
 *      03/11/2003 - Created 
 * ============================================================================
 */

#define IXP_SCRIPTS
#define	IXP_XSCALE

#include "xscale_common.h"
#include "..\1x10GbE_egress\wbench_project\dispatch_loop\dl_system.h"
#include "..\..\..\..\library\microblocks_library\include\dl_egress_system_default_2800.h"

/*----------------------
 *	CSIX RX
 *----------------------
 */
int csix_rx_ctx_init() {
	
	int finaladdr;

	printf("Initializing SRAM CSIX RX context area...\n");

	finaladdr = CSIX_RX_CTX_SRAM_BASE + CSIX_RX_CTX_TOTAL*CSIX_RX_CTX_SIZE;
	init_sram(0x0, CSIX_RX_CTX_SRAM_BASE, finaladdr);

	printf("Initialization complete.\n");

	return 1;
}


/*----------------------------------
 *	DRR Scheduler init code
 *----------------------------------
*/
int scheduler_init() {
	
	int finaladdr;

	printf("Initializing SRAM area for Scheduler ...\n");

	init_queue_structures_in_sram();

	init_round_counters_in_sram();

	printf("Initialization complete.\n");

	return 1;
}

int init_queue_structures_in_sram() {

	/* current_credit_used: lword0		
       last_round_scheduled_and_quantum_shift: lword1 [23:0]last round [31:24] quantum shift 
	   enqueued_packets_count: lword2	
 	   dequeue_packets_count: lword3
	*/

 	int x, y;
	int finaladdr, startaddr;
	int credit;
	

	printf("Start Scheduler Queue Structures initialization.\n");

	/*initialize for 10 ports	*/

	for(y=0; y<10; y++)
	{
		credit = 0x0f000001 + (y <<16);

		/*last round schedule[23:0] is always 1 at start.
		//Credit quantum[31:24] can be changed 
		//where [31:24] is n with 2^n is credit quantum per round
		//initialize for 256 queues, each queue structure is 16 bytes
		*/

  		for(x=0; x<256; x++)
		{
			startaddr = SCHED_QUEUE_STRUCTURES_BASE + (y << 12) + (x*16);
			finaladdr = startaddr + 3;


			/*credit used is always 0 to start with*/

			init_sram(0x0, startaddr, finaladdr);

			startaddr = finaladdr + 1;
			finaladdr = startaddr + 3;

			init_sram(credit, startaddr, finaladdr);

			startaddr = finaladdr + 1;
			finaladdr = startaddr + 7;

			/*enqueued packets counter is always 0 at start
			//dequeued packets counter is always 0 at start
			 */
			init_sram(0x0, startaddr, finaladdr);
		}
	}
	
	printf("Scheduler Queue Structures initialization complete.\n");

	return 1;
}

int init_round_counters_in_sram() {

 	int port;
	int finaladdr, startaddr;


	/* initialize round counter for 16 PORT AND 4K ROUNDS per port
	// (16k bytes per port since each counter is 4 bytes)
	*/

	printf("Start Scheduler Counters initialization.\n");

  	for(port=0; port<16; port++)
	{
		startaddr = SCHED_ROUND_COUNTERS_BASE + (port << 14);
		finaladdr = startaddr + 0x3fff;
		init_sram(0x0, startaddr, finaladdr);
	}

	printf("Scheduler Counters initialization complete.\n");

	return 1;
}


/*----------------------
 *	QM
 *----------------------
 */
int egress_qm_init() {
	
	int finaladdr;

	printf("Initializing SRAM queue descriptor area...\n");

	finaladdr = QD_SRAM_BASE + ((QD_TOTAL+2)*QD_SIZE);
	init_sram(0x0, QD_SRAM_BASE, finaladdr);

	printf("Initialization complete.\n");

	return 0;
}


/*---------------------------------------------------------------------
// This file contains functions that set up L2 table 
// information when running in simulation mode.
// The symbols used in this file are defined in a header file
// (typically dl_system.h) that should be included
// (using Simulation -> Options -> Startup) before the functions in
// this file are called.
---------------------------------------------------------------------*/

int l2_table_init() {
	
	int finaladdr;
	int xscale_sram_base;

	printf("Initializing SRAM L2 table area...\n");

	xscale_sram_base = L2_TABLE_SRAM_BASE;

	finaladdr = L2_TABLE_SRAM_BASE + L2_TABLE_NUM_ENTRIES*L2_TABLE_ENTRY_SIZE;

	init_sram(0x0, L2_TABLE_SRAM_BASE, finaladdr);

	printf("Adding L2 table entries...\n");

	/* entry 0 */
	set_sram(xscale_sram_base + 0x0,0x800090d7);
	set_sram(xscale_sram_base + 0x4,0x00080000);
	set_sram(xscale_sram_base + 0x8,0x90d80008);
	set_sram(xscale_sram_base + 0xc,0x00080000);

	/* entry 1 */
	set_sram(xscale_sram_base + 0x10,0x800090d7);
	set_sram(xscale_sram_base + 0x14,0x00080100);
	set_sram(xscale_sram_base + 0x18,0x90d80008);
	set_sram(xscale_sram_base + 0x1c,0x01080000);

	/* entry 2 */
	set_sram(xscale_sram_base + 0x20,0x800090d7);
	set_sram(xscale_sram_base + 0x24,0x00080200);
	set_sram(xscale_sram_base + 0x28,0x90d80008);
	set_sram(xscale_sram_base + 0x2c,0x02080000);

	/* entry 3 */
	set_sram(xscale_sram_base + 0x30,0x800090d7);
	set_sram(xscale_sram_base + 0x34,0x00080300);
	set_sram(xscale_sram_base + 0x38,0x90d80008);
	set_sram(xscale_sram_base + 0x3c,0x03080000);

	/* entry 4 */
	set_sram(xscale_sram_base + 0x40,0x800090d7);
	set_sram(xscale_sram_base + 0x44,0x00080400);
	set_sram(xscale_sram_base + 0x48,0x90d80008);
	set_sram(xscale_sram_base + 0x4c,0x04080000);

	/* entry 5 */
	set_sram(xscale_sram_base + 0x50,0x800090d7);
	set_sram(xscale_sram_base + 0x54,0x00080500);
	set_sram(xscale_sram_base + 0x58,0x90d80008);
	set_sram(xscale_sram_base + 0x5c,0x05080000);

	/* entry 6 */
	set_sram(xscale_sram_base + 0x60,0x800090d7);
	set_sram(xscale_sram_base + 0x64,0x00080600);
	set_sram(xscale_sram_base + 0x68,0x90d80008);
	set_sram(xscale_sram_base + 0x6c,0x06080000);

	/* entry 7 */
	set_sram(xscale_sram_base + 0x70,0x800090d7);
	set_sram(xscale_sram_base + 0x74,0x00080700);
	set_sram(xscale_sram_base + 0x78,0x90d80008);
	set_sram(xscale_sram_base + 0x7c,0x07080000);

	/* entry 8 */
	set_sram(xscale_sram_base + 0x80,0x800090d7);
	set_sram(xscale_sram_base + 0x84,0x00080800);
	set_sram(xscale_sram_base + 0x88,0x90d80008);
	set_sram(xscale_sram_base + 0x8c,0x08080000);

	/* entry 9 */
	set_sram(xscale_sram_base + 0x90,0x800090d7);
	set_sram(xscale_sram_base + 0x94,0x00080900);
	set_sram(xscale_sram_base + 0x98,0x90d80008);
	set_sram(xscale_sram_base + 0x9c,0x09080000);

	/* entry 10 */
	set_sram(xscale_sram_base + 0xa0,0x800090d7);
	set_sram(xscale_sram_base + 0xa4,0x00080a00);
	set_sram(xscale_sram_base + 0xa8,0x90d80008);
	set_sram(xscale_sram_base + 0xac,0x0a080000);

	/* entry 11 */
	set_sram(xscale_sram_base + 0xb0,0x800090d7);
	set_sram(xscale_sram_base + 0xb4,0x00080b00);
	set_sram(xscale_sram_base + 0xb8,0x90d80008);
	set_sram(xscale_sram_base + 0xbc,0x0b080000);

	/* entry 12 */
	set_sram(xscale_sram_base + 0xc0,0x800090d7);
	set_sram(xscale_sram_base + 0xc4,0x00080c00);
	set_sram(xscale_sram_base + 0xc8,0x90d80008);
	set_sram(xscale_sram_base + 0xcc,0x0c080000);

	/* entry 13 */
	set_sram(xscale_sram_base + 0xd0,0x800090d7);
	set_sram(xscale_sram_base + 0xd4,0x00080d00);
	set_sram(xscale_sram_base + 0xd8,0x90d80008);
	set_sram(xscale_sram_base + 0xdc,0x0d080000);

	/* entry 14 */
	set_sram(xscale_sram_base + 0xe0,0x800090d7);
	set_sram(xscale_sram_base + 0xe4,0x00080e00);
	set_sram(xscale_sram_base + 0xe8,0x90d80008);
	set_sram(xscale_sram_base + 0xec,0x0e080000);

	/* entry 15 */
	set_sram(xscale_sram_base + 0xf0,0x800090d7);
	set_sram(xscale_sram_base + 0xf4,0x00080f00);
	set_sram(xscale_sram_base + 0xf8,0x90d80008);
	set_sram(xscale_sram_base + 0xfc,0x0f080000);

	/* entry 16 */
	set_sram(xscale_sram_base + 0x100,0x800090d7);
	set_sram(xscale_sram_base + 0x104,0x00081000);
	set_sram(xscale_sram_base + 0x108,0x90d80008);
	set_sram(xscale_sram_base + 0x10c,0x10080000);

	/* entry 17 */
	set_sram(xscale_sram_base + 0x110,0x800090d7);
	set_sram(xscale_sram_base + 0x114,0x00081100);
	set_sram(xscale_sram_base + 0x118,0x90d80008);
	set_sram(xscale_sram_base + 0x11c,0x11080000);

	/* entry 18 */
	set_sram(xscale_sram_base + 0x120,0x800090d7);
	set_sram(xscale_sram_base + 0x124,0x00081200);
	set_sram(xscale_sram_base + 0x128,0x90d80008);
	set_sram(xscale_sram_base + 0x12c,0x12080000);

	/* entry 19 */
	set_sram(xscale_sram_base + 0x130,0x800090d7);
	set_sram(xscale_sram_base + 0x134,0x00081300);
	set_sram(xscale_sram_base + 0x138,0x90d80008);
	set_sram(xscale_sram_base + 0x13c,0x13080000);

	/* entry 20 */
	set_sram(xscale_sram_base + 0x140,0x800090d7);
	set_sram(xscale_sram_base + 0x144,0x00081400);
	set_sram(xscale_sram_base + 0x148,0x90d80008);
	set_sram(xscale_sram_base + 0x14c,0x14080000);

	/* entry 21 */
	set_sram(xscale_sram_base + 0x150,0x800090d7);
	set_sram(xscale_sram_base + 0x154,0x00081500);
	set_sram(xscale_sram_base + 0x158,0x90d80008);
	set_sram(xscale_sram_base + 0x15c,0x15080000);

	/* entry 22 */
	set_sram(xscale_sram_base + 0x160,0x800090d7);
	set_sram(xscale_sram_base + 0x164,0x00081600);
	set_sram(xscale_sram_base + 0x168,0x90d80008);
	set_sram(xscale_sram_base + 0x16c,0x16080000);

	/* entry 23 */
	set_sram(xscale_sram_base + 0x170,0x800090d7);
	set_sram(xscale_sram_base + 0x174,0x00081700);
	set_sram(xscale_sram_base + 0x178,0x90d80008);
	set_sram(xscale_sram_base + 0x17c,0x17080000);

	printf("IPv4 L2 initialization complete.\n");

	/* 'L2_TABLE_NUM_IPV4_ENTRIES' are assigned for IPv4 routes. 
	 * Beyond this entries are used for IPv6 routes.
	 */

	xscale_sram_base = L2_TABLE_SRAM_BASE + L2_TABLE_NUM_IPV4_ENTRIES * 16;

	/* entry 0 */
	set_sram(xscale_sram_base + 0x0,0x800090d7);
	set_sram(xscale_sram_base + 0x4,0x00080000);
	set_sram(xscale_sram_base + 0x8,0x90d80008);
	set_sram(xscale_sram_base + 0xc,0x0086dd00);

	/* entry 1 */
	set_sram(xscale_sram_base + 0x10,0x800090d7);
	set_sram(xscale_sram_base + 0x14,0x00080100);
	set_sram(xscale_sram_base + 0x18,0x90d80008);
	set_sram(xscale_sram_base + 0x1c,0x0186dd00);

	/* entry 2 */
	set_sram(xscale_sram_base + 0x20,0x800090d7);
	set_sram(xscale_sram_base + 0x24,0x00080200);
	set_sram(xscale_sram_base + 0x28,0x90d80008);
	set_sram(xscale_sram_base + 0x2c,0x0286dd00);

	/* entry 3 */
	set_sram(xscale_sram_base + 0x30,0x800090d7);
	set_sram(xscale_sram_base + 0x34,0x00080300);
	set_sram(xscale_sram_base + 0x38,0x90d80008);
	set_sram(xscale_sram_base + 0x3c,0x0386dd00);

	/* entry 4 */
	set_sram(xscale_sram_base + 0x40,0x800090d7);
	set_sram(xscale_sram_base + 0x44,0x00080400);
	set_sram(xscale_sram_base + 0x48,0x90d80008);
	set_sram(xscale_sram_base + 0x4c,0x0486dd00);

	/* entry 5 */
	set_sram(xscale_sram_base + 0x50,0x800090d7);
	set_sram(xscale_sram_base + 0x54,0x00080500);
	set_sram(xscale_sram_base + 0x58,0x90d80008);
	set_sram(xscale_sram_base + 0x5c,0x0586dd00);

	/* entry 6 */
	set_sram(xscale_sram_base + 0x60,0x800090d7);
	set_sram(xscale_sram_base + 0x64,0x00080600);
	set_sram(xscale_sram_base + 0x68,0x90d80008);
	set_sram(xscale_sram_base + 0x6c,0x0686dd00);

	/* entry 7 */
	set_sram(xscale_sram_base + 0x70,0x800090d7);
	set_sram(xscale_sram_base + 0x74,0x00080700);
	set_sram(xscale_sram_base + 0x78,0x90d80008);
	set_sram(xscale_sram_base + 0x7c,0x0786dd00);

	/* entry 8 */
	set_sram(xscale_sram_base + 0x80,0x800090d7);
	set_sram(xscale_sram_base + 0x84,0x00080800);
	set_sram(xscale_sram_base + 0x88,0x90d80008);
	set_sram(xscale_sram_base + 0x8c,0x0886dd00);

	/* entry 9 */
	set_sram(xscale_sram_base + 0x90,0x800090d7);
	set_sram(xscale_sram_base + 0x94,0x00080900);
	set_sram(xscale_sram_base + 0x98,0x90d80008);
	set_sram(xscale_sram_base + 0x9c,0x0986dd00);

	/* entry 10 */
	set_sram(xscale_sram_base + 0xa0,0x800090d7);
	set_sram(xscale_sram_base + 0xa4,0x00080a00);
	set_sram(xscale_sram_base + 0xa8,0x90d80008);
	set_sram(xscale_sram_base + 0xac,0x0a86dd00);

	/* entry 11 */
	set_sram(xscale_sram_base + 0xb0,0x800090d7);
	set_sram(xscale_sram_base + 0xb4,0x00080b00);
	set_sram(xscale_sram_base + 0xb8,0x90d80008);
	set_sram(xscale_sram_base + 0xbc,0x0b86dd00);

	/* entry 12 */
	set_sram(xscale_sram_base + 0xc0,0x800090d7);
	set_sram(xscale_sram_base + 0xc4,0x00080c00);
	set_sram(xscale_sram_base + 0xc8,0x90d80008);
	set_sram(xscale_sram_base + 0xcc,0x0c86dd00);

	/* entry 13 */
	set_sram(xscale_sram_base + 0xd0,0x800090d7);
	set_sram(xscale_sram_base + 0xd4,0x00080d00);
	set_sram(xscale_sram_base + 0xd8,0x90d80008);
	set_sram(xscale_sram_base + 0xdc,0x0d86dd00);

	/* entry 14 */
	set_sram(xscale_sram_base + 0xe0,0x800090d7);
	set_sram(xscale_sram_base + 0xe4,0x00080e00);
	set_sram(xscale_sram_base + 0xe8,0x90d80008);
	set_sram(xscale_sram_base + 0xec,0x0e86dd00);

	/* entry 15 */
	set_sram(xscale_sram_base + 0xf0,0x800090d7);
	set_sram(xscale_sram_base + 0xf4,0x00080f00);
	set_sram(xscale_sram_base + 0xf8,0x90d80008);
	set_sram(xscale_sram_base + 0xfc,0x0f86dd00);

	/* entry 16 */
	set_sram(xscale_sram_base + 0x100,0x800090d7);
	set_sram(xscale_sram_base + 0x104,0x00081000);
	set_sram(xscale_sram_base + 0x108,0x90d80008);
	set_sram(xscale_sram_base + 0x10c,0x1086dd00);

	/* entry 17 */
	set_sram(xscale_sram_base + 0x110,0x800090d7);
	set_sram(xscale_sram_base + 0x114,0x00081100);
	set_sram(xscale_sram_base + 0x118,0x90d80008);
	set_sram(xscale_sram_base + 0x11c,0x1186dd00);

	/* entry 18 */
	set_sram(xscale_sram_base + 0x120,0x800090d7);
	set_sram(xscale_sram_base + 0x124,0x00081200);
	set_sram(xscale_sram_base + 0x128,0x90d80008);
	set_sram(xscale_sram_base + 0x12c,0x1286dd00);

	/* entry 19 */
	set_sram(xscale_sram_base + 0x130,0x800090d7);
	set_sram(xscale_sram_base + 0x134,0x00081300);
	set_sram(xscale_sram_base + 0x138,0x90d80008);
	set_sram(xscale_sram_base + 0x13c,0x1386dd00);

	/* entry 20 */
	set_sram(xscale_sram_base + 0x140,0x800090d7);
	set_sram(xscale_sram_base + 0x144,0x00081400);
	set_sram(xscale_sram_base + 0x148,0x90d80008);
	set_sram(xscale_sram_base + 0x14c,0x1486dd00);

	/* entry 21 */
	set_sram(xscale_sram_base + 0x150,0x800090d7);
	set_sram(xscale_sram_base + 0x154,0x00081500);
	set_sram(xscale_sram_base + 0x158,0x90d80008);
	set_sram(xscale_sram_base + 0x15c,0x1586dd00);

	/* entry 22 */
	set_sram(xscale_sram_base + 0x160,0x800090d7);
	set_sram(xscale_sram_base + 0x164,0x00081600);
	set_sram(xscale_sram_base + 0x168,0x90d80008);
	set_sram(xscale_sram_base + 0x16c,0x1686dd00);

	/* entry 23 */
	set_sram(xscale_sram_base + 0x170,0x800090d7);
	set_sram(xscale_sram_base + 0x174,0x00081700);
	set_sram(xscale_sram_base + 0x178,0x90d80008);
	set_sram(xscale_sram_base + 0x17c,0x1786dd00);

	/* entry 24 */
	set_sram(xscale_sram_base + 0x180,0x800090d7);
	set_sram(xscale_sram_base + 0x184,0x00081800);
	set_sram(xscale_sram_base + 0x188,0x90d80008);
	set_sram(xscale_sram_base + 0x18c,0x1786dd00);

	/* entry 25 */
	set_sram(xscale_sram_base + 0x190,0x800090d7);
	set_sram(xscale_sram_base + 0x194,0x00081900);
	set_sram(xscale_sram_base + 0x198,0x90d80008);
	set_sram(xscale_sram_base + 0x19c,0x1786dd00);

	/* entry 26 */
	set_sram(xscale_sram_base + 0x1a0,0x800090d7);
	set_sram(xscale_sram_base + 0x1a4,0x00081a00);
	set_sram(xscale_sram_base + 0x1a8,0x90d80008);
	set_sram(xscale_sram_base + 0x1ac,0x1786dd00);

	printf("IPv6 L2 initialization complete.\n");

	return 1;
}

void egress_system_setup()
{
	printf("\n******************************************************\n");
	printf("\n     System setup begin ...                           \n");
	printf("\n******************************************************\n");

	/*----------------------
	 *	CSIX RX
	 *----------------------
	 */

	/* Initialize the receive contexts in SRAM. */
	csix_rx_ctx_init();


	/*----------------------
	 *	L2 Encape
	 *----------------------
	 */

	/* Initialize l2 ecape table in SRAM. */

	l2_table_init();

	/*----------------------
	 *	DRR Scheduler Init
	 *----------------------
	 */

	scheduler_init();

	/*----------------------
	 *	QM
	 *----------------------
	 */

	/* Initialize queue descriptor area in SRAM. */
	egress_qm_init();

	/*----------------------
	 *	Packet TX
	 *----------------------
	 */

	/*----------------------
	 *	COMMON
	 *----------------------
	 */

	printf("\n*********************************************************\n");
	printf("\n     System setup complete.                              \n");
	printf("\n*********************************************************\n");

}


/* Shorter function names */
void ess()
{	
    egress_system_setup();
}
