/*---------------------------------------------------------------------
 *                                                                     
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                       
 *      COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *      THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                       
 *                         INTEL  CORPORATION                            
 *                                                                      
 *                      2200 MISSION COLLEGE BLVD                        
 *                                                                       
 *                SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                       
 *---------------------------------------------------------------------
 *
 *        File Name: dl_system.h
 *
 *        Purpose: This file contains system wide parameters
 *                 that are customized for a given application.
 *
 *-------------------------------------------------------------------*/
#ifndef	__DL_SYSTEM__H__
#define	__DL_SYSTEM__H__

#ifndef IXP_SCRIPTS
#include <definitions.h>
#include <ix_cc_microengines_bindings.h>
#ifndef IXP_MICROCODE
#include <ixp.h> /* for micro-c */
#endif
#endif

/* Define chip. */
#ifndef IXP2400
#define IXP2400
#endif

/* Put IPv4 counters in scratch. */
#define IPV4_COUNTERS_SCRATCH

/* Size of the pakcet header to be cached (in long words) */ 
#define PKT_HDR_CACHE_SIZE   16

/* Receive Phy mode */
#define RX_PHY_MODE        SPHY_4_8

/*	Scratch for communicating between scheduler and queue manager */
#define RING_SIZE_128 		0
#define RING_SIZE_256		1
#define RING_SIZE_512		2
#define RING_SIZE_1024		3

#define INGRESS_SCHEDULER_DEQUEUE_RING_ID		 	SCHEDULER_TO_QM_SCR_RING
#define INGRESS_SCHEDULER_DEQUEUE_RING_BASE_ID		csr_scratch_ring_base_6
#define INGRESS_SCHEDULER_DEQUEUE_RING_HEAD_ID		csr_scratch_ring_head_6	
#define INGRESS_SCHEDULER_DEQUEUE_RING_TAIL_ID		csr_scratch_ring_tail_6
#define INGRESS_SCHEDULER_DEQUEUE_RING_SIZE			RING_SIZE_512
#define INGRESS_SCHEDULER_DEQUEUE_RING_BASE			SCHEDULER_TO_QM_SCR_RING_BASE				
#define	INGRESS_SCHEDULER_DEQUEUE_RING_FULL_VALUE	inp_state_scr_ring6_full 

#define	QM_RING_NUMBER								IPV4_RING_OUT

/* 
 * IPv6 Definitions
 */
#ifndef USE_IMPORT_VAR
#define IPV6_TRIE_TABLE_SRAM_BASE             0x600000
#endif /* USE_IMPORT_VARS */

#define	SIZE_OF_IPV6_TRIE_SET                 16
#define	NUM_IPV6_TRIE_SETS	                  1024

/* Size of the IPv6 trie table */
#define IPV6_TRIE_TABLE_SIZE                  0x400000

/* Hash Table for lower 64-bits of destination IPv6 address */
#define IPV6_HASH_TABLE_SRAM_BASE             0xC00000
#define IPV6_HASH_TABLE_SIZE                  0x20000

#ifndef USE_IMPORT_VAR
#define IPV6_STATS_TABLE_SRAM_BASE            0xC30000
#endif /* USE_IMPORT_VARS */

#ifndef USE_IMPORT_VAR
#define IPV6_ROUTER_PREFIX_SRAM_BASE          0xC20000
#endif
#define IPV6_ROUTER_PREFIX_SRAM_SIZE          0x10

/* Maximum number of blades and ports per blade (used for IPv6 statistics) */
#define NUM_BLADES                            0x40
#define PORTS_PER_BLADE_SHIFT                 4

/**
 * IPv6/v4 Tunnel Definitions
 */

/* Tunnel Next-Hop Arrays */
#ifndef USE_IMPORT_VAR
#define TUNNEL_ENCAP_NEXTHOP_SRAM_BASE        0x400000
#endif /* USE_IMPORT_VAR */

#define NUM_START_TUNNEL_NEXTHOPS             256
#define START_TUNNEL_NEXTHOP_LW               8
#define TUNNEL_ENCAP_NEXTHOP_SRAM_SIZE        0x2000

#ifndef USE_IMPORT_VAR
#define TUNNEL_DECAP_NEXTHOP_SRAM_BASE        0x402000
#endif /* USE_IMPORT_VAR */

#define NUM_END_TUNNEL_NEXTHOPS               128
#define END_TUNNEL_NEXTHOP_LW                 1
#define TUNNEL_DECAP_NEXTHOP_SRAM_SIZE        0x200

/* The prefix table for validating source addresses on tunneled
 * packets requiring decapsulation can be a simple linear list
 * or a trie table.  There are performance tradeoffs for using
 * each option.  The linear list is designed to be used in cases
 * where each tunnel supports a small number of prefixes (about 12
 * or less).  The trie table can be used if each tunnel can have a
 * large number of source prefixes.  However, since the trie table
 * is global for all ending tunnel endpoints, if more than one
 * tunnel allows a single prefix, a separate list needs to be used
 * to hold the tunnels that allow a particular prefix.
 */

#ifndef USE_IMPORT_VAR
#define TUNNEL_INGRESS_LIST_SRAM_BASE         0x404000
#endif /* USE_IMPORT_VAR */


#ifdef V6V4_INGRESS_LIST_TRIE

/**
 * Defines for Tunnel Ingress Source List using Trie Table.
 */
#define TUNNEL_INGRESS_LIST_NUM_TRIE_BLOCKS   512

/* The Ingress List trie entry contains an index, which can be either
 * a index of the tunnel next hop entry that supports the prefix, or it
 * can be 0xffff to indicate the prefix is valid for all tunnels, or
 * it can be an index into an array containing indexes of tunnel next hop
 * entries that support the prefix.  If a single prefix is allowed for
 * only one tunnel, V6V4_INGRESS_LIST_INDEX_BLOCK_SIZE should be set to
 * zero to avoid using a list.
 */

#define V6V4_INGRESS_LIST_INDEX_BLOCK_SIZE      8
#define V6V4_INGRESS_LIST_NUM_INDEX_BLOCKS      256

#else

/**
 * Defines for Tunnel Ingress Source Block Array using Linear List (default).
 */
#define V6V4_NUM_INGRESS_LIST_BLOCKS            256
#define V6V4_INGRESS_LIST_LW                    16

/* The maximum number of addresses per tunnel can be set to a value
 * from 1-12. If V6V4_INGRESS_LIST_MAX_ADDR is not set, then multiple
 * blocks can be used to store additional prefixes for a single tunnel,
 * if more are needed.  If V6V4_INGRESS_LIST_MAX_ADDR is set, then
 * only a single block can be used to store the addresses, and only the
 * specified number of addresses will be checked when validating a
 * source address.
 */

#define V6V4_INGRESS_LIST_MAX_ADDR  12

#endif

/* Base for tunneling decapsulation counters */

#ifdef V6V4_DECAP_COUNTERS
#ifndef USE_IMPORT_VAR
#define V6V4_DECAP_COUNTER_BASE     0x420000 
#endif /* USE_IMPORT_VAR */

#define V6V4_NUM_DECAP_COUNTERS     2
#endif /* V6V4_DECAP_COUNTES */

/* Base for tunneling encapsulation counters */
#ifdef  V6V4_ENCAP_COUNTERS
#ifndef USE_IMPORT_VAR
#define V6V4_ENCAP_COUNTER_BASE     0x420008 
#endif /* USE_IMPORT_VAR */

#define V6V4_NUM_ENCAP_COUNTERS     2
#endif /* V6V4_ENCAP_COUNTERS */


/*
 * Block ID for IPv6 and tunneling blocks
 */

#define BID_IPV6                IX_CC_IPV6_PKT_MICROBLOCK_ID

#define BID_V6V4_TUNNEL_DECAP   IX_CC_V6V4_DECAP_PKT_MICROBLOCK_ID
#define BID_V6V4_TUNNEL_ENCAP   IX_CC_V6V4_ENCAP_PKT_MICROBLOCK_ID
#define BID_ETHER_PPP_SESSION   0x28

/*
 * Tunnel Encapsulation Flag for Header Type.  Must not conflict with
 * any of the header types defined in definitions.h
 */
#define TUNNEL_ENCAP_FLAG       0x10

/* Values possible for nexthop_id_type */
#define NEXTHOP_TYPE_FORWARD    0x0
#define NEXTHOP_TYPE_TUNNEL     0x2

/* Signal used by timer for refreshing IPv6 trie-cache */
#define IPV6_CACHE_REFRESH_SIGNAL  11

/* Both NEXT_HOP_TABLE_SRAM_BASE and SIZE are defined in the default
 * settings
 */
#define IPV6_L3_NEXTHOP_TABLE_SRAM_BASE       NEXTHOP_TABLE_SRAM_BASE
#define IPV6_NUM_L3_NEXTHOPS                  NUM_NEXTHOPS
#define	IPV6_SIZE_OF_L3_NEXTHOP_LW            SIZE_OF_NEXTHOP_LW
#define IPV6_L3_NEXTHOP_TABLE_SIZE            NEXTHOP_TABLE_SIZE

/*
 * Static bindings.
 */

#ifdef ETHER_WITHOUT_L2
#define DL_SOURCE_NEXT1   BID_ETHER
#else
#define DL_SOURCE_NEXT1   BID_IPV4
#endif

#define PACKET_RX_NEXT1        BID_ETHER

#define ETHER_NEXT_IPV4        BID_IPV4
#define ETHER_NEXT_IPV6        BID_IPV6
#define IPV4_NEXT1             BID_V6V4_TUNNEL_DECAP
#define IPV6_NEXT_BLOCK        BID_V6V4_TUNNEL_ENCAP
#define V6V4_NEXT_IPV4         BID_IPV4
#define V6V4_NEXT_IPV6         BID_IPV6
#define V6V4_NEXT_NO_ENCAP     BID_QM
#define V6V4_NEXT_NO_DECAP     BID_QM



/* Include default settings. */

/* This file is also used by script files that setup the system when run
 * in simulation mode. When symbol IXP_SCRIPTS is defined, it indicates
 * that scripts are using this file. Since script files cannot access
 * path definitions, they should not encounter '#include <file>'. Instead
 * they include the <file> using Simulation -> Options -> Startup.
 */
#ifndef IXP_SCRIPTS
#include <dl_ingress_system_default.h>
#endif


#endif /* __DL_SYSTEM__H__ */