/*
 *---------------------------------------------------------------------------
 *
 *                  I N T E L   P R O P R I E T A R Y
 *
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT
 *     THE PRIOR WRITTEN PERMISSION OF :
 *
 *                        INTEL  CORPORATION
 *
 *                     2200 MISSION COLLEGE BLVD
 *
 *               SANTA  CLARA,  CALIFORNIA  95052-8119
 *
 *---------------------------------------------------------------------------
 */

#ifndef __ETHERNET_IPV6_C__
#define __ETHERNET_IPV6_C__

/*
 * include files
 */

#include "dl_system.h"

#include "dl_source.c"
#include "ether.c"
#include "pkthdr_cache.c"

/*
 * Include microblocks
 */
#include "ipv4_fwder.c"
#include "ipv6_fwder.c"
#include "v6v4_tunnel_decap.c"
#include "v6v4_tunnel_encap.c"

/*
 * 	ether_ipv6_kickstart()
 *
 *	 Description: 
 *
 *	Kick starts IPV6/V4 Forwarder blocks across the 4 MEs
 *	(1) when the system initialization is complete.
 *	(2) signal thread 0 on beginning ME after which the
 *  thread order is maintained by DL functions.
 *
 *	All the MEs (and threads) running IPV6/V4 Forwader call 
 *	this function. Only thread0 of first ME would execute 
 *	this fully. Other threads of first ME and all the
 *	threads on other MEs would continue and call the
 *	dl_source(DL_THREAD_ORDER,0x0), and wait for signal
 *	from previous thread. This would result in thread0
 *	of first ME kickstarting the whole chain.
 *
 *
 * 	Outputs:
 *	
 *		None.
 *
 *	Inputs
 *
 *		None.
 *
 *
 *	CONSTANTS
 *
 *
 *	Size: 
 *		N/A
 */
INLINE	ether_ipv6_kickstart(void)
{

#ifdef	IPV6_START_ME

	SIGNAL	system_init_signal;

	if (ctx()==0)
	{
		int sig;
		// issue self signal
		sig = __signal_number(&dl_sig_prev) << 3;
		local_csr_write(local_csr_same_me_signal,sig);

		// now initialize and wait on system init signal
		__assign_relative_register(&system_init_signal,ME_INIT_SIGNAL);

		// wait for system init signal
		wait_for_all(&system_init_signal);
	}
#endif
}



/*------------------------------------------------------------------------
 * The main function of the dispatch loop
 *------------------------------------------------------------------------
 */
int main()
{

	/*
 	 * initialize the microblocks
 	 */

	dl_init();					//initialize the dispatch loop

    ipv6ForwarderInit();
    v6v4_tunnel_encap_init();


	/*
	 * kickstart the forwarder microengines.
	 * kickstart the ethernet IPv6/v4 functional pipeline.
	 * NOTE: when running on hardware, we need to modify this so that
	 * context 0 in the first ME handles packets from the core.
	 */
	ether_ipv6_kickstart();

	dl_source(DL_THREAD_ORDER,0x0);	// begin with receiving packets in order

	/*
 	 * Execute microblocks in infinite loop
 	 */
	while(1)
	{

		SIGNAL			scratch_put;		// signal in scratch write
		SIGNAL_MASK		sig_mask;		    // mask of signals to wait on


#ifdef ETHER_WITHOUT_L2
		ether_decap_classify(pkt_hdr, 0x0, DRAM_RD_REG);
#endif

		dl_copy_iphdr_to_lmem();

	    /* IPv4 Forwarder */
		Ipv4Fwder(ipv4_hdr, 0, ipv4_hdr, 0, LOCALMEM, LOCALMEM);

	    /* Tunnel decapsulation */
	    v6v4_tunnel_decap(ipv6_hdr, ipv6_hdr, ipv4_hdr, 0, 0, 0,
	                      LOCALMEM, LOCALMEM, LOCALMEM);

	    /* IPv6 Forwarder */
	    ipv6Forwarder(ipv6_hdr, 0, ipv6_hdr, 0, LOCALMEM, LOCALMEM);

	    /* Tunnel encapsulation */
	    v6v4_tunnel_encap(ipv4_hdr, ipv6_hdr, 0, 0, LOCALMEM, LOCALMEM);

	    /* IPv4 Forwarder */
		Ipv4Fwder(ipv4_hdr, 0, ipv4_hdr, 0, LOCALMEM, LOCALMEM);

	    if (dlNextBlock == IPV4_NEXT1)
		{
	        dlNextBlock = BID_NEXT_BLOCK;
	    }

		__implicit_read(&scratch_put);
        sig_mask = 0;
		dl_qm_sink(&sig_mask,&scratch_put);		// dispose the packet

		dl_source(DL_THREAD_NO_ORDER,sig_mask);	// source next packet


	} // end while

	return 0;
}



#endif