/*--------------------------------------------------------------------------
 *                                                                     
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 *--------------------------------------------------------------------------
 */

#ifndef	__PKTHDR_CACHE_C__
#define	__PKTHDR_CACHE_C__

#include <dl_system.h>
#include <dl_meta.c>
#include <ether.h>

/* TBD: All of the routines in this file use the compiler switch
 * USE_BYTE_ALIGN_INTRINSIC to determine whether or not to use the
 * intrinsic function byte_align_block_be() to align and copy bytes
 * to and from the header cache.  This is here because the byte_align_block_be()
 * function uses lots of registers and usually causes the code to spill to
 * SRAM, even when compiling just one microblock (due to the compiler issue
 * with register allocation).  When the compiler issue is resolved, then one
 * or the other of these methods will be used (whichever one yields the best
 * overall results), and the compiler switch will go away.
 */

//	How many QUAD words of packet header to read/write from DRAM initially.

#define	PKTHDR_CACHE_SIZE			    4

#define IPV4_HDR_LW                     5
#define IPV6_HDR_LW                     10

#define IPV4_SHIFT_COUNT                2
#define IPV6_SHIFT_COUNT                2

#define IPV4_ALIGN_COUNT                9
#define IPV6_ALIGN_COUNT                13
#define IPV6_ENCAP_ALIGN_COUNT_1        8
#define IPV6_ENCAP_ALIGN_COUNT_2        5

#define IP_HDR_LW_OFFSET                3
#define IPV6_ENCAP_HDR_LW_OFFSET_1      8
#define IPV6_ENCAP_HDR_LW_OFFSET_2      0

#define IPV6_ENCAP_REMAINING_OFFSET     (64 - ETHER_HEADER_LENGTH)
#define IPV6_ENCAP_REMAINING_SIZE       3

#define IPV6_ENCAP_WR_OFFSET            8

#define FLUSH_ENCAP_WRITE_SIZE_1        5

#define V6V4_HDR_TYPE_DECAP             ETHER_IPV6_TYPE
#define V6V4_HDR_TYPE_ENCAP             TUNNEL_ENCAP_FLAG

#define ETHER_IPV6_IN_IPV4_TYPE         (ETHER_IPV6_TYPE | V6V4_HDR_TYPE_ENCAP)

extern dl_buf_handle_t 	dlBufHandle;			// The current buffer handle.

__declspec(dram_read_reg) unsigned int pkt_hdr[16];	// cache of packet header in registers.
__declspec(local_mem) unsigned int ipv4_hdr[8];     // local memory cache for ipv4 hdr.
__declspec(local_mem) unsigned int ipv6_hdr[16];    // local memory cache for ipv6 hdr.

/**************************************************************************
 * dl_copy_iphdr_to_lmem
 *
 *		Descripton 	:
 *          Copy the IP header to the local memory cache.  The meta data
 *          header type is used to determine which local memory buffer to
 *          use.
 *		
 *		Outputs		:
 *          None
 *
 *		Inputs		:
 *          None
 * 
 *      Returns:
 *          None
 */
INLINE	void dl_copy_iphdr_to_lmem(void)
{
#ifdef USE_BYTE_ALIGN_INTRINSIC
    if (dlMeta.headerType == ETHER_IPV4_TYPE)
	{
	    byte_align_block_be(
		    IPV4_ALIGN_COUNT, (void *)ipv4_hdr,
            (void *)(pkt_hdr + IP_HDR_LW_OFFSET),
            IPV4_SHIFT_COUNT);
	}
	else if (dlMeta.headerType == ETHER_IPV6_TYPE)
	{
	    byte_align_block_be(
		    IPV6_ALIGN_COUNT, (void *)ipv6_hdr,
            (void *)(pkt_hdr + IP_HDR_LW_OFFSET),
            IPV6_SHIFT_COUNT);
	}
#else
    if (dlMeta.headerType == ETHER_IPV4_TYPE)
	{
        unsigned int aligned_value;

        aligned_value = pkt_hdr[IP_HDR_LW_OFFSET] << 16;
        ipv4_hdr[0] = aligned_value | (pkt_hdr[IP_HDR_LW_OFFSET + 1] >> 16);
        aligned_value = pkt_hdr[IP_HDR_LW_OFFSET + 1] << 16;
        ipv4_hdr[1] = aligned_value | (pkt_hdr[IP_HDR_LW_OFFSET + 2] >> 16);
        aligned_value = pkt_hdr[IP_HDR_LW_OFFSET + 2] << 16;
        ipv4_hdr[2] = aligned_value | (pkt_hdr[IP_HDR_LW_OFFSET + 3] >> 16);
        aligned_value = pkt_hdr[IP_HDR_LW_OFFSET + 3] << 16;
        ipv4_hdr[3] = aligned_value | (pkt_hdr[IP_HDR_LW_OFFSET + 4] >> 16);
        aligned_value = pkt_hdr[IP_HDR_LW_OFFSET + 4] << 16;
        ipv4_hdr[4] = aligned_value | (pkt_hdr[IP_HDR_LW_OFFSET + 5] >> 16);
        aligned_value = pkt_hdr[IP_HDR_LW_OFFSET + 5] << 16;
        ipv4_hdr[5] = aligned_value | (pkt_hdr[IP_HDR_LW_OFFSET + 6] >> 16);
        aligned_value = pkt_hdr[IP_HDR_LW_OFFSET + 6] << 16;
        ipv4_hdr[6] = aligned_value | (pkt_hdr[IP_HDR_LW_OFFSET + 7] >> 16);
        aligned_value = pkt_hdr[IP_HDR_LW_OFFSET + 7] << 16;
        ipv4_hdr[7] = aligned_value | (pkt_hdr[IP_HDR_LW_OFFSET + 8] >> 16);
    }
	else if (dlMeta.headerType == ETHER_IPV6_TYPE)
	{
        unsigned int aligned_value;

        aligned_value = pkt_hdr[IP_HDR_LW_OFFSET] << 16;
        ipv6_hdr[0] = aligned_value | (pkt_hdr[IP_HDR_LW_OFFSET + 1] >> 16);
        aligned_value = pkt_hdr[IP_HDR_LW_OFFSET + 1] << 16;
        ipv6_hdr[1] = aligned_value | (pkt_hdr[IP_HDR_LW_OFFSET + 2] >> 16);
        aligned_value = pkt_hdr[IP_HDR_LW_OFFSET + 2] << 16;
        ipv6_hdr[2] = aligned_value | (pkt_hdr[IP_HDR_LW_OFFSET + 3] >> 16);
        aligned_value = pkt_hdr[IP_HDR_LW_OFFSET + 3] << 16;
        ipv6_hdr[3] = aligned_value | (pkt_hdr[IP_HDR_LW_OFFSET + 4] >> 16);
        aligned_value = pkt_hdr[IP_HDR_LW_OFFSET + 4] << 16;
        ipv6_hdr[4] = aligned_value | (pkt_hdr[IP_HDR_LW_OFFSET + 5] >> 16);
        aligned_value = pkt_hdr[IP_HDR_LW_OFFSET + 5] << 16;
        ipv6_hdr[5] = aligned_value | (pkt_hdr[IP_HDR_LW_OFFSET + 6] >> 16);
        aligned_value = pkt_hdr[IP_HDR_LW_OFFSET + 6] << 16;
        ipv6_hdr[6] = aligned_value | (pkt_hdr[IP_HDR_LW_OFFSET + 7] >> 16);
        aligned_value = pkt_hdr[IP_HDR_LW_OFFSET + 7] << 16;
        ipv6_hdr[7] = aligned_value | (pkt_hdr[IP_HDR_LW_OFFSET + 8] >> 16);
        aligned_value = pkt_hdr[IP_HDR_LW_OFFSET + 8] << 16;
        ipv6_hdr[8] = aligned_value | (pkt_hdr[IP_HDR_LW_OFFSET + 9] >> 16);
        aligned_value = pkt_hdr[IP_HDR_LW_OFFSET + 9] << 16;
        ipv6_hdr[9] = aligned_value | (pkt_hdr[IP_HDR_LW_OFFSET + 10] >> 16);
        aligned_value = pkt_hdr[IP_HDR_LW_OFFSET + 10] << 16;
        ipv6_hdr[10] = aligned_value | (pkt_hdr[IP_HDR_LW_OFFSET + 11] >> 16);
        aligned_value = pkt_hdr[IP_HDR_LW_OFFSET + 11] << 16;
        ipv6_hdr[11] = aligned_value | (pkt_hdr[IP_HDR_LW_OFFSET + 12] >> 16);
	}
#endif
	return;
}

/**************************************************************************
 * dl_load_ipv6_hdr
 *
 *		Descripton 	:
 *          Copy the IPv6 header to the local memory cache.  This is
 *          invoked by the tunneling block when it encouters a packet
 *          requiring decapsulation.
 *		
 *		Outputs		:
 *          None
 *
 *		Inputs		:
 *          pIpv6Hdr: pointer to buffer to receive the ipv6 header.
 *          in_offset: offset in buffer where header should begin.
 *          memType : memory type of ipv6 buffer.
 *          hdr_size: number of bytes to load.
 * 
 *      Returns:
 *          None
 */
#ifdef USE_BYTE_ALIGN_INTRINSIC
INLINE	void dl_load_ipv6_hdr(
    void *pIpv6Hdr,
	unsigned int in_offset,
    mem_t memType,
	unsigned int hdr_size)
{
    __declspec(sdram) unsigned char *p_pkt_hdr; 
    SIGNAL_PAIR	dram_sig;
	unsigned int last_value;

	/* Copy the first 30 bytes of the IPv6 header (which are already in
	 * pkt_hdr) to the local memory byffer.
	 */

    byte_align_block_be(
        IPV6_ENCAP_ALIGN_COUNT_1, (void *)ipv6_hdr,
        (void *)(pkt_hdr + IPV6_ENCAP_HDR_LW_OFFSET_1),
        IPV6_SHIFT_COUNT);

    last_value = pkt_hdr[15] << 16;

    /* Read the remaining bytes of the IPv6 header */

    p_pkt_hdr = (__declspec(dram) unsigned char *)(Dl_BufGetData(dlBufHandle) +
	             dlMeta.offset + IPV6_ENCAP_REMAINING_OFFSET);

    dram_read(pkt_hdr, (volatile void __declspec(sdram) *)p_pkt_hdr,
	          IPV6_ENCAP_REMAINING_SIZE, sig_done, &dram_sig);

    wait_for_all(&dram_sig);

    /* Fill in the last longword from the previous copy */

	ipv6_hdr[IPV6_ENCAP_WR_OFFSET - 1] = (last_value | (pkt_hdr[0] >> 16));

	/* Transfer in the remaining bytes */

    byte_align_block_be(
        IPV6_ENCAP_ALIGN_COUNT_2, (void *)(ipv6_hdr + IPV6_ENCAP_WR_OFFSET),
        (void *)pkt_hdr, IPV6_SHIFT_COUNT);

	return;
}
#else
INLINE	void dl_load_ipv6_hdr(
    void *pIpv6Hdr,
	unsigned int in_offset,
    mem_t memType,
	unsigned int hdr_size)
{
    __declspec(sdram) unsigned char *p_pkt_hdr; 
    SIGNAL_PAIR	dram_sig;
    unsigned int value;

	/* Copy the first 30 bytes of the IPv6 header (which are already in
	 * pkt_hdr) to the local memory byffer.
	 */

    value = pkt_hdr[IPV6_ENCAP_HDR_LW_OFFSET_1] << 16;
    ipv6_hdr[0] = value | (pkt_hdr[IPV6_ENCAP_HDR_LW_OFFSET_1 + 1] >> 16);
    value = pkt_hdr[IPV6_ENCAP_HDR_LW_OFFSET_1 + 1] << 16;
    ipv6_hdr[1] = value | (pkt_hdr[IPV6_ENCAP_HDR_LW_OFFSET_1 + 2] >> 16);
    value = pkt_hdr[IPV6_ENCAP_HDR_LW_OFFSET_1 + 2] << 16;
    ipv6_hdr[2] = value | (pkt_hdr[IPV6_ENCAP_HDR_LW_OFFSET_1 + 3] >> 16);
    value = pkt_hdr[IPV6_ENCAP_HDR_LW_OFFSET_1 + 3] << 16;
    ipv6_hdr[3] = value | (pkt_hdr[IPV6_ENCAP_HDR_LW_OFFSET_1 + 4] >> 16);
    value = pkt_hdr[IPV6_ENCAP_HDR_LW_OFFSET_1 + 4] << 16;
    ipv6_hdr[4] = value | (pkt_hdr[IPV6_ENCAP_HDR_LW_OFFSET_1 + 5] >> 16);
    value = pkt_hdr[IPV6_ENCAP_HDR_LW_OFFSET_1 + 5] << 16;
    ipv6_hdr[5] = value | (pkt_hdr[IPV6_ENCAP_HDR_LW_OFFSET_1 + 6] >> 16);
    value = pkt_hdr[IPV6_ENCAP_HDR_LW_OFFSET_1 + 6] << 16;
    ipv6_hdr[6] = value | (pkt_hdr[IPV6_ENCAP_HDR_LW_OFFSET_1 + 7] >> 16);
    value = pkt_hdr[IPV6_ENCAP_HDR_LW_OFFSET_1 + 7] << 16;

    /* Read the remaining bytes of the IPv6 header */

    p_pkt_hdr = (__declspec(dram) unsigned char *)(Dl_BufGetData(dlBufHandle) +
	             dlMeta.offset + IPV6_ENCAP_REMAINING_OFFSET);

    dram_read(pkt_hdr, (volatile void __declspec(sdram) *)p_pkt_hdr,
	          IPV6_ENCAP_REMAINING_SIZE, sig_done, &dram_sig);

    wait_for_all(&dram_sig);

    /* Finish copying where we left off */

	ipv6_hdr[7] = value | (pkt_hdr[0] >> 16);

    value = pkt_hdr[0] << 16;
    ipv6_hdr[8] = value | (pkt_hdr[1] >> 16);
    value = pkt_hdr[1] << 16;
    ipv6_hdr[9] = value | (pkt_hdr[2] >> 16);
    value = pkt_hdr[2] << 16;
    ipv6_hdr[10] = value | (pkt_hdr[3] >> 16);
    value = pkt_hdr[3] << 16;
    ipv6_hdr[11] = value | (pkt_hdr[4] >> 16);

	return;
}
#endif


/**************************************************************************
 * _dl_flush_hdr_aligned
 *
 *		Descripton 	:
 *          Copy the IPv4 and/or IPv6 headers from the local memory cache
 *          back to the packet buffer.
 *		
 *		Outputs		:
 *          None
 *
 *		Inputs		:
 *          align_offset: offset of start of IP header from the nearest
 *          quad word boundary.
 *          pWrSig: signal to use in dram write.
 * 
 *      Returns:
 *          None
 */
INLINE	void _dl_flush_hdr_aligned(
    unsigned int align_offset,
    SIGNAL_PAIR *pWrSig)
{
    __declspec(dram_write_reg) unsigned int pkthdr_out[16];
    __declspec(sdram) unsigned char *p_pkt_hdr;

    /* reg where actual header starts */
    unsigned int xreg_start = align_offset >> 2;
    /* number of quad words to be written */
    unsigned int write_size;
    /* byte index */
    unsigned int byte_index = align_offset % 4;
    /* shift amount */
    unsigned int shf_count = byte_index * 8;

    p_pkt_hdr = (__declspec(sdram) unsigned char *)	
        (Dl_BufGetData(dlBufHandle) + (dlMeta.offset & ~3));

    if (dlMeta.headerType == ETHER_IPV4_TYPE)
	{
        unsigned int flush_count;

        write_size = ((IPV4_HDR_LW * 4) + align_offset + 7) >> 3;
        flush_count = (write_size << 1) - xreg_start;

        if (xreg_start == 1)
        {
            pkthdr_out[0] = 0;
        }

        if (byte_index == 0)
        {
            pkthdr_out[xreg_start] = ipv4_hdr[0];
            pkthdr_out[xreg_start + 1] = ipv4_hdr[1];
            pkthdr_out[xreg_start + 2] = ipv4_hdr[2];
            pkthdr_out[xreg_start + 3] = ipv4_hdr[3];
            pkthdr_out[xreg_start + 4] = ipv4_hdr[4];
            if (xreg_start == 0)
            {
                pkthdr_out[xreg_start + 5] = ipv4_hdr[5];
            }
        }
        else
        {
            pkthdr_out[xreg_start] = ipv4_hdr[0] >> shf_count;

#ifdef USE_BYTE_ALIGN_INTRINSIC
            byte_align_block_be(
                flush_count, (void *)(pkthdr_out + xreg_start + 1),
                (void *)ipv4_hdr, byte_index);
#else
            pkthdr_out[xreg_start + 1] =
                (ipv4_hdr[0] << (32 - shf_count)) | (ipv4_hdr[1] >> shf_count);
            pkthdr_out[xreg_start + 2] =
                (ipv4_hdr[1] << (32 - shf_count)) | (ipv4_hdr[2] >> shf_count);
            pkthdr_out[xreg_start + 3] =
                (ipv4_hdr[2] << (32 - shf_count)) | (ipv4_hdr[3] >> shf_count);
            pkthdr_out[xreg_start + 4] =
                (ipv4_hdr[3] << (32 - shf_count)) | (ipv4_hdr[4] >> shf_count);
            pkthdr_out[xreg_start + 5] =
                (ipv4_hdr[4] << (32 - shf_count)) | (ipv4_hdr[5] >> shf_count);

            if (flush_count > 5)
                pkthdr_out[xreg_start + 6] = (ipv4_hdr[5] << (32 - shf_count)) |
                                             (ipv4_hdr[6] >> shf_count);
#endif
        }

		dram_write(pkthdr_out, p_pkt_hdr, write_size, sig_done, pWrSig);
    }
    else if (dlMeta.headerType == ETHER_IPV6_IN_IPV4_TYPE)
    {
        /*
         * Flush the IPv4 and IPv6 headers in two writes. Even though in
         * the case where align_offset < 4 we should be able to do it in
         * one write, a compiler error prevents this -- so we do all cases
         * using 2 writes.  In the case where the header starts in the 
         * second xreg (4 <= align_offset <= 7), the first write is 48
         * bytes and the second is 24 bytes.  In the case where the header
         * starts in the first xreg (0 <= align_offset <= 3), the first
         * write is 40 bytes and the second is 24 bytes.
         */

        unsigned int ipv6_flush_count;
        unsigned int ipv6_offset;
        unsigned int ipv4_offset = 0;

        if (xreg_start == 1)
        {
            pkthdr_out[0] = 0;
            /* First write is 6 quadwords */
            write_size = FLUSH_ENCAP_WRITE_SIZE_1 + 1;
        }
        else
        {
            /* First write is 5 quadwords */
            write_size = FLUSH_ENCAP_WRITE_SIZE_1;
        }

        if (byte_index == 0)
        {
            pkthdr_out[xreg_start] = ipv4_hdr[0];
            pkthdr_out[xreg_start + 1] = ipv4_hdr[1];
            pkthdr_out[xreg_start + 2] = ipv4_hdr[2];
            pkthdr_out[xreg_start + 3] = ipv4_hdr[3];
            pkthdr_out[xreg_start + 4] = ipv4_hdr[4];
            pkthdr_out[xreg_start + 5] = ipv6_hdr[0];
            pkthdr_out[xreg_start + 6] = ipv6_hdr[1];
            pkthdr_out[xreg_start + 7] = ipv6_hdr[2];
            pkthdr_out[xreg_start + 8] = ipv6_hdr[3];
            pkthdr_out[xreg_start + 9] = ipv6_hdr[4];

            ipv6_offset = 5;

            if (xreg_start == 1)
            {
                pkthdr_out[xreg_start + 10] = ipv6_hdr[5];
                ipv6_offset = 6;
            }
        }
        else
        {
            pkthdr_out[xreg_start] = ipv4_hdr[0] >> shf_count;

#ifdef USE_BYTE_ALIGN_INTRINSIC
            byte_align_block_be(
                IPV4_HDR_LW, (void *)(pkthdr_out + xreg_start + 1),
                (void *)ipv4_hdr, byte_index);
#else
            pkthdr_out[xreg_start + 1] =
                (ipv4_hdr[0] << (32 - shf_count)) | (ipv4_hdr[1] >> shf_count);
            pkthdr_out[xreg_start + 2] =
                (ipv4_hdr[1] << (32 - shf_count)) | (ipv4_hdr[2] >> shf_count);
            pkthdr_out[xreg_start + 3] =
                (ipv4_hdr[2] << (32 - shf_count)) | (ipv4_hdr[3] >> shf_count);
            pkthdr_out[xreg_start + 4] =
                (ipv4_hdr[3] << (32 - shf_count)) | (ipv4_hdr[4] >> shf_count);
#endif

            /* Set xreg_start to the index of the longword that contains the
             * last byte of the ipv4 header (and the first byte of the ipv6
             * header).  Then set that LW to the correct value.
             */

            xreg_start += IPV4_HDR_LW;
            pkthdr_out[xreg_start] =
                ((ipv4_hdr[4] << (32 - shf_count)) | (ipv6_hdr[0] >> shf_count));

            /* Now set ipv6_offset to the index of the last (partial) longword
             * of the ipv6 header that gets written.
             */

            ipv6_offset = (write_size << 1) - xreg_start - 1;

#ifdef USE_BYTE_ALIGN_INTRINSIC
            byte_align_block_be(
                ipv6_offset + 1, (void *)(pkthdr_out + xreg_start + 1),
                (void *)ipv6_hdr, IPV6_SHIFT_COUNT);
#else
            pkthdr_out[xreg_start + 1] =
                (ipv6_hdr[0] << (32 - shf_count)) | (ipv6_hdr[1] >> shf_count);
            pkthdr_out[xreg_start + 2] =
                (ipv6_hdr[1] << (32 - shf_count)) | (ipv6_hdr[2] >> shf_count);
            pkthdr_out[xreg_start + 3] =
                (ipv6_hdr[2] << (32 - shf_count)) | (ipv6_hdr[3] >> shf_count);
            pkthdr_out[xreg_start + 4] =
                (ipv6_hdr[3] << (32 - shf_count)) | (ipv6_hdr[4] >> shf_count);
            if (ipv6_offset > 4)
                pkthdr_out[xreg_start + 5] = (ipv6_hdr[4] << (32 - shf_count)) |
                                             (ipv6_hdr[5] >> shf_count);
#endif

        }

		dram_write(pkthdr_out, p_pkt_hdr, write_size, sig_done, pWrSig);

        /* Wait for first write to complete, then set up the second one,
         * which is 3 quadwords in either case */

	    wait_for_all(pWrSig);

        /* Advance the packet pointer by the number of bytes just written */

        p_pkt_hdr += (write_size << 3);

        if (byte_index == 0)
        {
            pkthdr_out[0] = ipv6_hdr[ipv6_offset];
            pkthdr_out[1] = ipv6_hdr[ipv6_offset + 1];
            pkthdr_out[2] = ipv6_hdr[ipv6_offset + 2];
            pkthdr_out[3] = ipv6_hdr[ipv6_offset + 3];
            pkthdr_out[4] = ipv6_hdr[ipv6_offset + 4];
            pkthdr_out[5] = ipv6_hdr[ipv6_offset + 5];
        }
        else
        {
            /* ipv6_offset contains the number of lws of the ipv6
             * header that we wrote the first time.  So now calculate how
             * much is left to be written.
             */

            write_size = (IPV6_HDR_LW - ipv6_offset + 1) >> 1;
            ipv6_flush_count = (write_size << 1) + 1;

#ifdef USE_BYTE_ALIGN_INTRINSIC
            byte_align_block_be(
                ipv6_flush_count, (void *)pkthdr_out,
                (void *)(ipv6_hdr + ipv6_offset), byte_index);
#else
            pkthdr_out[0] =
                (ipv6_hdr[ipv6_offset] << (32 - shf_count)) |
                (ipv6_hdr[ipv6_offset + 1] >> shf_count);
            pkthdr_out[1] =
                (ipv6_hdr[ipv6_offset + 1] << (32 - shf_count)) |
                (ipv6_hdr[ipv6_offset + 2] >> shf_count);
            pkthdr_out[2] =
                (ipv6_hdr[ipv6_offset + 2] << (32 - shf_count)) |
                (ipv6_hdr[ipv6_offset + 3] >> shf_count);
            pkthdr_out[3] =
                (ipv6_hdr[ipv6_offset + 3] << (32 - shf_count)) |
                (ipv6_hdr[ipv6_offset + 4] >> shf_count);
            pkthdr_out[4] =
                (ipv6_hdr[ipv6_offset + 4] << (32 - shf_count)) |
                (ipv6_hdr[ipv6_offset + 5] >> shf_count);
            pkthdr_out[5] =
                (ipv6_hdr[ipv6_offset + 5] << (32 - shf_count)) |
                (ipv6_hdr[ipv6_offset + 6] >> shf_count);
#endif
        }

	    dram_write(pkthdr_out, p_pkt_hdr, write_size, sig_done, pWrSig);
    }
    else if (dlMeta.headerType == ETHER_IPV6_TYPE)
	{
        unsigned int flush_count;

        write_size = ((IPV6_HDR_LW * 4) + align_offset + 7) >> 3;
        flush_count = (write_size << 1) - xreg_start;

        if (xreg_start == 1)
        {
            pkthdr_out[0] = 0;
        }

        if (byte_index == 0)
        {
            pkthdr_out[xreg_start] = ipv6_hdr[0];
            pkthdr_out[xreg_start + 1] = ipv6_hdr[1];
            pkthdr_out[xreg_start + 2] = ipv6_hdr[2];
            pkthdr_out[xreg_start + 3] = ipv6_hdr[3];
            pkthdr_out[xreg_start + 4] = ipv6_hdr[4];
            pkthdr_out[xreg_start + 5] = ipv6_hdr[5];
            pkthdr_out[xreg_start + 6] = ipv6_hdr[6];
            pkthdr_out[xreg_start + 7] = ipv6_hdr[7];
            pkthdr_out[xreg_start + 8] = ipv6_hdr[8];
            pkthdr_out[xreg_start + 9] = ipv6_hdr[9];

            if (xreg_start == 1)
            {
                pkthdr_out[xreg_start + 10] = ipv6_hdr[10];
            }
        }
        else
        {
            pkthdr_out[xreg_start] = ipv6_hdr[0] >> shf_count;

#ifdef USE_BYTE_ALIGN_INTRINSIC
            byte_align_block_be(
                flush_count, (void *)(pkthdr_out + xreg_start + 1),
                (void *)ipv6_hdr, byte_index);
#else
            pkthdr_out[xreg_start + 1] =
                (ipv6_hdr[0] << (32 - shf_count)) | (ipv6_hdr[1] >> shf_count);
            pkthdr_out[xreg_start + 2] =
                (ipv6_hdr[1] << (32 - shf_count)) | (ipv6_hdr[2] >> shf_count);
            pkthdr_out[xreg_start + 3] =
                (ipv6_hdr[2] << (32 - shf_count)) | (ipv6_hdr[3] >> shf_count);
            pkthdr_out[xreg_start + 4] =
                (ipv6_hdr[3] << (32 - shf_count)) | (ipv6_hdr[4] >> shf_count);
            pkthdr_out[xreg_start + 5] =
                (ipv6_hdr[4] << (32 - shf_count)) | (ipv6_hdr[5] >> shf_count);
            pkthdr_out[xreg_start + 6] =
                (ipv6_hdr[5] << (32 - shf_count)) | (ipv6_hdr[6] >> shf_count);
            pkthdr_out[xreg_start + 7] =
                (ipv6_hdr[6] << (32 - shf_count)) | (ipv6_hdr[7] >> shf_count);
            pkthdr_out[xreg_start + 8] =
                (ipv6_hdr[7] << (32 - shf_count)) | (ipv6_hdr[8] >> shf_count);
            pkthdr_out[xreg_start + 9] =
                (ipv6_hdr[8] << (32 - shf_count)) | (ipv6_hdr[9] >> shf_count);
            pkthdr_out[xreg_start + 10] =
                (ipv6_hdr[9] << (32 - shf_count)) | (ipv6_hdr[10] >> shf_count);

            if (flush_count > 11)
                pkthdr_out[xreg_start + 11] = (ipv6_hdr[10] << (32 - shf_count))
                                            | (ipv6_hdr[11] >> shf_count);
#endif
        }

		dram_write(pkthdr_out, p_pkt_hdr, write_size, sig_done, pWrSig);
    }

	return;
}


/**************************************************************************
 * dl_flush_iphdr_cache
 *
 *		Descripton 	:
 *          Copy the IPv4 and/or IPv6 headers from the local memory cache
 *          back to the packet buffer.
 *		
 *		Outputs		:
 *          None
 *
 *		Inputs		:
 *          pWrSig: signal to use in dram write.
 * 
 *      Returns:
 *          None
 */
INLINE	void dl_flush_iphdr_cache(
    SIGNAL_PAIR *pWrSig)
{

    /**
	 * We need to determine the offset (0 - 7) at which the IP header begins
	 * relative to the start of the quad-word aligned address at which the
	 * write begins.  There are two possibilities:
	 * 1: If no tunneling (encapsulation or decapsulation) was performed on
	 * the packet or if a packet was decapsulated and re-encapsulated, then
	 * the IP header will begin at offset 6.
	 * 2: If a packet was encapsulated or decapsulated, the IP header will
	 * begin at offset 2. 
     *
	 * Flush the header cache, given the alignment of the IP header relative
	 * to the quad-word aligned start write address.
	 */

    if (dlMeta.offset & 4)
    {
		/* Packet begins at offset 6, so last 3 bits of offset are 110 */
        _dl_flush_hdr_aligned(6, pWrSig);
    }
    else
    {
		/* Packet begins at offset 2, so last 3 bits of offset are 010 */
        _dl_flush_hdr_aligned(2, pWrSig);
    }

	return;
}

#endif