//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//
//      File Name:     dl_system.h
//
//      Purpose:     This file contains the system wide parameters that can be 
//                   customised for a given hardware/application 
//
//------------------------------------------------------------------------------

#ifndef    __SYSTEM__H__
#define    __SYSTEM__H__

/* Define chip */
#ifndef    IXP2400
#define    IXP2400
#endif

/* Define project as microcode based */
#ifndef    IXP_SCRIPTS
#ifndef    IXP_MICROCODE
#define    IXP_MICROCODE
#endif
#endif

/* Put IPv4 counters in scratch */
#define IPV4_COUNTERS_SCRATCH

/*Size of the Layer 2 header
  Since this application is for ethernet, set to Ethernet header length*/
#define L2_HEADER_LENGTH 14

/* Size of the pakcet header to be cached (in long words) */ 
#define PKT_HDR_CACHE_SIZE   16

/* Receive Phy mode */
#define RX_PHY_MODE        SPHY_4_8

/*  
 *-------------------------------
 * SRAM Memory Map for Channel 0
 *-------------------------------
 */

/* 
 * Base Address for Next Hop Information - "NEXTHOP_TABLE_SRAM_BASE"
 */

#ifndef USE_IMPORT_VAR
#ifdef IXP_SIMULATION
#define NEXTHOP_TABLE_SRAM_BASE   0x400000
#else /* hardware mode */
#define NEXTHOP_TABLE_SRAM_BASE   0x300000
#endif /* IXP_SIMULATION */
#else
#endif /* USE_IMPORT_VAR */

/* Size of the Trie Table - 4MB */
#define TRIE_TABLE_SIZE  0x400000

/*  
 *-------------------------------
 * SRAM Memory Map for Channel 1
 *-------------------------------
 */

/* 
 * Base Address for IPv4 Counters 
 */

#ifndef USE_IMPORT_VAR
#ifdef IPV4_COUNTERS_SCRATCH      /* define IPv4 counters in Scratch */
#define IPV4_STATS_TABLE_BASE  11264
#else
#ifdef IXP_SIMULATION
#define STATS_TABLE_SRAM_BASE  0x40400000
#else
#define STATS_TABLE_SRAM_BASE  0x40200000
#endif /* IXP_SIMULATION */
#endif /* IPV4_COUNTERS_SCRATCH */
#endif /* USE_IMPORT_VARS */

/* 
 * Base Address for POS RX Counters.
 */

#ifndef USE_IMPORT_VAR
#ifdef IXP_SIMULATION
#define PACKET_COUNTERS_SRAM_BASE  0x40400800
#else
#define PACKET_COUNTERS_SRAM_BASE  0x40200800
#endif /* IXP_SIMULATION */
#else
#endif /* USE_IMPORT_VARS */

/*
 * Base Address for Direct Broadcast table 
 */

#ifndef USE_IMPORT_VAR
#ifdef IXP_SIMULATION
#define DBCAST_TABLE_SRAM_BASE  0x40401000
#else
#define DBCAST_TABLE_SRAM_BASE  0x40201000
#endif /* IXP_SIMULATION */
#else
#endif /* USE_IMPORT_VARS */

/* 
 * Base Address for Control Block Table.
 */

#ifndef USE_IMPORT_VAR
#ifdef IXP_SIMULATION
#define CONTROL_BLOCK_SRAM_BASE  0x40403000
#else
#define CONTROL_BLOCK_SRAM_BASE  0x40203000
#endif /* IXP_SIMULATION */
#else
#endif /* USE_IMPORT_VARS */

/*
 * Base Address for Ethernet MAC Filtering Hash Table.
 */
#ifndef USE_IMPORT_VAR
#ifdef IXP_SIMULATION
#define SRAM_FLT_TABLE_BASE  0x43000000
#else
#define SRAM_FLT_TABLE_BASE  0x40302000
#endif /* IXP_SIMULATION */
#else
.import_var   SRAM_FLT_TABLE_BASE
#endif /* USE_IMPORT_VARS */

/* Size of each entry in Bytes */
#define SRAM_FLT_ENTRY_SIZE  8

/* Size of the Filtering Table in Bytes */ 
#define SRAM_FLT_TABLE_SIZE  1024

/* 
 * Base Address for the Port Bit Map base
 * Input Port Bit Map: Should be updated by the Core. Each port used one
 * bit - bit 1 for port 1, ..., bit 15 for port 15.
 * Each bit is defined as follows:
 *   0 -- the port is set in unicast mode.
 *   1 -- the port is set in promiscous mode.
 */

#ifndef USE_IMPORT_VAR
#ifdef IXP_SIMULATION
#define SRAM_PORT_BIT_MAP_BASE  0x41100000
#else
#define SRAM_PORT_BIT_MAP_BASE  0x40301000
#endif /* IXP_SIMULATION */
#else
.import_var   SRAM_PORT_BIT_MAP_BASE
#endif /* USE_IMPORT_VARS */

#define SRAM_PORT_BIT_ENTRY 0x1

//Base address for the NAT-PT translation table
#ifndef USE_IMPORT_VAR
#ifdef IXP_SIMULATION
#define NATPT_NAT_TABLE_BASE 0x42000000
#define NATPT_NAT_V6V4_HASH_TABLE_BASE 0x40300000
#define NATPT_NAT_V4V6_HASH_TABLE_BASE 0x40400000
#define NATPT_NAT_COLL_HASH_TABLE_BASE 0x40500000

#define NATPT_HASH_MUL_48_0  0x43434343
#define NATPT_HASH_MUL_48_1  0x43434343
#define NATPT_HASH_MUL_128_0 0x43434343
#define NATPT_HASH_MUL_128_1 0x43434343
#define NATPT_HASH_MUL_128_2 0x43434343
#define NATPT_HASH_MUL_128_3 0x43434343

#define NATPT_PREFIX_0 0x0
#define NATPT_PREFIX_1 0x0
#define NATPT_PREFIX_2 0x0000ffff

#ifdef NATPT_DEBUG_COUNTERS
#define NATPT_SRAM_COUNTER_BASE  0x40100000
#endif /* NATPT_DEBUG_COUNTERS */

#else
#define NATPT_NAT_TABLE_BASE 0x42000000
#define NATPT_NAT_V6V4_HASH_TABLE_BASE 0x40300000
#define NATPT_NAT_V4V6_HASH_TABLE_BASE 0x40400000
#define NATPT_NAT_COLL_HASH_TABLE_BASE 0x40500000

#define NATPT_HASH_MUL_48_0  0x43434343
#define NATPT_HASH_MUL_48_1  0x43434343
#define NATPT_HASH_MUL_128_0 0x43434343
#define NATPT_HASH_MUL_128_1 0x43434343
#define NATPT_HASH_MUL_128_2 0x43434343
#define NATPT_HASH_MUL_128_3 0x43434343

#define NATPT_PREFIX_0 0x0
#define NATPT_PREFIX_1 0x0
#define NATPT_PREFIX_2 0x0000ffff

#ifdef NATPT_DEBUG_COUNTERS
#define NATPT_SRAM_COUNTER_BASE  0x40100000
#endif /* NATPT_DEBUG_COUNTERS */

#endif /* IXP_SIMULATION */
#else
.import_var   NATPT_NAT_TABLE_BASE
.import_var   NATPT_NAT_V6V4_HASH_TABLE_BASE
.import_var   NATPT_NAT_V4V6_HASH_TABLE_BASE
.import_var   NATPT_NAT_COLL_HASH_TABLE_BASE

.import_var   NATPT_HASH_MUL_48_0
.import_var   NATPT_HASH_MUL_48_1
.import_var   NATPT_HASH_MUL_128_0
.import_var   NATPT_HASH_MUL_128_1
.import_var   NATPT_HASH_MUL_128_2
.import_var   NATPT_HASH_MUL_128_3

.import_var NATPT_PREFIX_0
.import_var NATPT_PREFIX_1
.import_var NATPT_PREFIX_2

#ifdef NATPT_DEBUG_COUNTERS
.import_var NATPT_SRAM_COUNTER_BASE
#endif /* NATPT_DEBUG_COUNTERS */

#endif /* USE_IMPORT_VARS */

/* Hash Table size */
#define HASH_TABLE_SIZE  1024 * 8

/* 
 * IPv6 Definitions
 */
#ifndef USE_IMPORT_VAR
#ifdef IXP_SIMULATION
#define IPV6_TRIE_TABLE_SRAM_BASE             0x600000
#else
#define IPV6_TRIE_TABLE_SRAM_BASE             0x600000
#endif /* IXP_SIMULATION */
#else
.import_var IPV6_TRIE_TABLE_SRAM_BASE
#endif /* USE_IMPORT_VARS */

#define	SIZE_OF_IPV6_TRIE_SET                 16
#define	NUM_IPV6_TRIE_SETS	                  1024

/* Size of the IPv6 trie table */
#define IPV6_TRIE_TABLE_SIZE                  0x400000

/* Both NEXT_HOP_TABLE_SRAM_BASE and SIZE are defined in the default
 * settings
 */
#ifndef USE_IMPORT_VAR
#ifdef IXP_SIMULATION
#define IPV6_L3_NEXTHOP_TABLE_SRAM_BASE       NEXTHOP_TABLE_SRAM_BASE
#else
#define IPV6_L3_NEXTHOP_TABLE_SRAM_BASE       NEXTHOP_TABLE_SRAM_BASE
#endif /* IXP_SIMULATION */
#endif /* USE_IMPORT_VARS */

#define IPV6_L3_NEXTHOP_TABLE_SIZE            NEXTHOP_TABLE_SIZE
#define IPV6_NUM_L3_NEXTHOPS                  NUM_NEXTHOPS
#define	IPV6_SIZE_OF_L3_NEXTHOP_LW            SIZE_OF_NEXTHOP_LW

/* Hash Table for lower 64-bits of destination IPv6 address */
#define IPV6_HASH_TABLE_SRAM_BASE             0xC00000
#define IPV6_HASH_TABLE_SIZE                  0x20000

/* 
 * Static Bindings (aka. targets) for various microblocks.
 */
#ifdef ETHER_WITHOUT_L2 
#define DL_SOURCE_NEXT1   BID_ETHER
#else
#define DL_SOURCE_NEXT1   BID_IPV4
#endif

/* 
 * IPv6 Forwarder Stat Counters
 */

#define NUM_BLADES                            0x40
#define PORTS_PER_BLADE_SHIFT                 4

#ifdef IPV6_STATS

#ifndef USE_IMPORT_VAR
#ifdef IXP_SIMULATION
#define IPV6_STATS_TABLE_SRAM_BASE  0xC30000
#else
#define IPV6_STATS_TABLE_SRAM_BASE  0xC30000
#endif /* IXP_SIMULATION */
#else
.import_var IPV6_STATS_TABLE_SRAM_BASE
#endif /* USE_IMPORT_VARS */

#endif /* IPV6_STATS */

/* 
 * SRAM Location where router's common IPv6 prefix will be
 * stored by XScale core.
 */

#ifndef USE_IMPORT_VAR
#ifdef IXP_SIMULATION
#define IPV6_ROUTER_PREFIX_SRAM_BASE          0xC20000
#else
#define IPV6_ROUTER_PREFIX_SRAM_BASE          0xC20000
#endif /* IXP_SIMULATION */
#else
.import_var IPV6_ROUTER_PREFIX_SRAM_BASE
#endif /* USE_IMPORT_VAR */

#define IPV6_ROUTER_PREFIX_SRAM_SIZE          0x10

/* Local memory location where router's IPv6 prefix is cached
 * with trie-entries.
 */ 

#define IPV6_TRIE_CACHE_LM_BASE              0x418
#define IPV6_TRIE_CACHE_LM_SIZE              0x10


/* Tunnel Next-Hop Arrays */
#ifndef USE_IMPORT_VAR
#ifdef IXP_SIMULATION
#define TUNNEL_ENCAP_NEXTHOP_SRAM_BASE      0x408000
#else
#define TUNNEL_ENCAP_NEXTHOP_SRAM_BASE      0x400000
#endif /* IXP_SIMULATION */
#else
.import_var TUNNEL_ENCAP_NEXTHOP_SRAM_BASE
#endif /* USE_IMPORT_VAR */

#define NUM_START_TUNNEL_NEXTHOPS           256
#define START_TUNNEL_NEXTHOP_LW             8
#define TUNNEL_ENCAP_NEXTHOP_SRAM_SIZE      0x2000

#ifndef USE_IMPORT_VAR
#ifdef IXP_SIMULATION
#define TUNNEL_DECAP_NEXTHOP_SRAM_BASE      0x40A000
#else
#define TUNNEL_DECAP_NEXTHOP_SRAM_BASE      0x402000
#endif /* IXP_SIMULATION */
#else
.import_var TUNNEL_DECAP_NEXTHOP_SRAM_BASE
#endif /* USE_IMPORT_VAR */

#define NUM_END_TUNNEL_NEXTHOPS             128
#define END_TUNNEL_NEXTHOP_LW               1
#define TUNNEL_DECAP_NEXTHOP_SRAM_SIZE      0x200


/* The prefix table for validating source addresses on tunneled
 * packets requiring decapsulation can be a simple linear list
 * or a trie table.  There are performance tradeoffs for using
 * each option.  The linear list is designed to be used in cases
 * where each tunnel supports a small number of prefixes (about 12
 * or less).  The trie table can be used if each tunnel can have a
 * large number of source prefixes.  However, since the trie table
 * is global for all ending tunnel endpoints, if more than one
 * tunnel allows a single prefix, a separate list needs to be used
 * to hold the tunnels that allow a particular prefix.
 */

#ifndef USE_IMPORT_VAR
#ifdef IXP_SIMULATION
#define TUNNEL_INGRESS_LIST_SRAM_BASE         0x40C000
#else
#define TUNNEL_INGRESS_LIST_SRAM_BASE         0x404000
#endif /* IXP_SIMULATION */
#else
.import_var TUNNEL_INGRESS_LIST_SRAM_BASE
#endif /* USE_IMPORT_VAR */

#ifdef V6V4_INGRESS_LIST_TRIE

/* Tunnel Ingress Source List using Trie Table */

#define TUNNEL_INGRESS_LIST_NUM_TRIE_BLOCKS   512

/* The Ingress List trie entry contains an index, which can be either
 * a index of the tunnel next hop entry that supports the prefix, or it
 * can be 0xffff to indicate the prefix is valid for all tunnels, or
 * it can be an index into an array containing indexes of tunnel next hop
 * entries that support the prefix.  If a single prefix is allowed for
 * only one tunnel, V6V4_INGRESS_LIST_INDEX_BLOCK_SIZE should be set to
 * zero to avoid using a list.
 */

#define V6V4_INGRESS_LIST_INDEX_BLOCK_SIZE    8
#define V6V4_INGRESS_LIST_NUM_INDEX_BLOCKS    256

#else

/* Tunnel Ingress Source Block Array using Linear List */
#define V6V4_NUM_INGRESS_LIST_BLOCKS            256
#define V6V4_INGRESS_LIST_LW                    16

/* The maximum number of addresses per tunnel can be set to a value
 * from 1-12. If V6V4_INGRESS_LIST_MAX_ADDR is not set, then multiple
 * blocks can be used to store additional prefixes for a single tunnel,
 * if more are needed.  If V6V4_INGRESS_LIST_MAX_ADDR is set, then
 * only a single block can be used to store the addresses, and only the
 * specified number of addresses will be checked when validating a
 * source address.
 */

#define V6V4_INGRESS_LIST_MAX_ADDR  12


#endif   /* V6V4_INGRESS_LIST_TRIE */

/* Base for tunneling decapsulation counters */

#ifdef V6V4_DECAP_COUNTERS
#ifndef USE_IMPORT_VAR
#ifdef IXP_SIMULATION
#define V6V4_DECAP_COUNTER_BASE  0x420000
#else
#define V6V4_DECAP_COUNTER_BASE  0x420000 
#endif /* IXP_SIMULATION */
#else
.import_var V6V4_DECAP_COUNTER_BASE
#endif /* USE_IMPORT_VAR */

#define V6V4_NUM_DECAP_COUNTERS     2
#endif /* V6V4_DECAP_COUNTES */

/* Base for tunneling encapsulation counters */
#ifdef  V6V4_ENCAP_COUNTERS
#ifndef USE_IMPORT_VAR
#ifdef IXP_SIMULATION
#define V6V4_ENCAP_COUNTER_BASE  0x420008
#else
#define V6V4_ENCAP_COUNTER_BASE  0x420008 
#endif /* IXP_SIMULATION */
#else
.import_var V6V4_ENCAP_COUNTER_BASE
#endif /* USE_IMPORT_VAR */

#define V6V4_NUM_ENCAP_COUNTERS     2
#endif /* V6V4_ENCAP_COUNTERS */

#define BID_ETHER_PPP_SESSION  0x28
#define BID_V6V4_TUNNEL_DECAP  IX_CC_V6V4_DECAP_PKT_MICROBLOCK_ID
#define BID_V6V4_TUNNEL_ENCAP  IX_CC_V6V4_ENCAP_PKT_MICROBLOCK_ID

/* NOTE: ether.uc currently sets the header type to these values instead
 * of dlNextBlock... we will have to change these mappings to BID_XXX
 * when that gets fixed.
 */
#define BID_ETHER_DECAP_NEXT1  ETHER_IPV4_TYPE
#define BID_ETHER_DECAP_NEXT2  ETHER_IPV6_TYPE
#define BID_ETHER_DECAP_NEXT3  ETHER_ARP_TYPE

#define TARGET_ETHER_DECAP_IPV4  BID_NATPT
#define TARGET_ETHER_DECAP_IPV6  BID_NATPT

#define IPV6_NEXT_BLOCK        BID_V6V4_TUNNEL_ENCAP
#define V6V4_NEXT_IPV4         BID_IPV4
#define V6V4_NEXT_IPV6         BID_IPV6
#define V6V4_NEXT_NO_ENCAP     BID_QM
#define V6V4_NEXT_NO_DECAP     BID_QM

#define V6V4_NATPT_TRANSLATED_V6  BID_IPV6
#define V6V4_NATPT_TRANSLATED_V4  BID_IPV4
#define V6V4_NATPT_PASSTHROUGH_V6 BID_IPV6
#define V6V4_NATPT_PASSTHROUGH_V4 BID_IPV4

/*
 * Tunnel Encapsulation Flag for Header Type.  Must not conflict with
 * any of the header types defined in definitions.h
 */
#define TUNNEL_ENCAP_FLAG  0x10

/* Values possible for nexthop_id_type */
#define NEXTHOP_TYPE_FORWARD  0x0
#define NEXTHOP_TYPE_TUNNEL   0x2

/* Signal used by timer for refreshing IPv6 trie-cache */
#define IPV6_CACHE_REFRESH_SIGNAL  11

/*
 *
 *    Source and Sink related defines
 */


/* Finally, include the default settings. This file defines the default
 * values for a whole bunch of parameters.
 *
 * Note that this header file (dl_system.h) will be included in the test
 * scripts (.ind files) in simulation mode for testing. The script files
 * do not understand the #include <filename> statement. To get around
 * this problem, we include the file below in a IXP_SCRIPTS define.
 * Scripts that need dl_system.h will includes it by going to 
 * Simulations -> Options -> Startup.
 */
#ifndef IXP_SCRIPTS
#include "dl_ingress_system_default.h"
#endif

/* 
 * Some of these definitions depend on the definitons defined in the default
 * file included above 
 */

#ifdef IXP_MICROCODE

#define_eval POS_RX_TO_FWDER_SCR_RING        POS_RX_TO_IPV4_SCR_RING
#define_eval POS_RX_TO_FWDER_SCR_RING_BASE   POS_RX_TO_IPV4_SCR_RING_BASE
#define_eval POS_RX_TO_FWDER_SCR_RING_SIZE   POS_RX_TO_IPV4_SCR_RING_SIZE

#define_eval FWDER_TO_QM_SCR_RING            IPV4_TO_QM_SCR_RING
#define_eval FWDER_TO_QM_SCR_RING_BASE       IPV4_TO_QM_SCR_RING_BASE
#define_eval FWDER_TO_QM_SCR_RING_SIZE       IPV4_TO_QM_SCR_RING_SIZE

#define_eval FWDER_RING_IN        POS_RX_TO_FWDER_SCR_RING
#define_eval FWDER_RING_IN_BASE   POS_RX_TO_FWDER_SCR_RING_BASE
#define_eval FWDER_RING_IN_SIZE   POS_RX_TO_FWDER_SCR_RING_SIZE

#define_eval FWDER_RING_OUT       FWDER_TO_QM_SCR_RING 
#define_eval FWDER_RING_OUT_BASE  FWDER_TO_QM_SCR_RING_BASE
#define_eval FWDER_RING_OUT_SIZE  FWDER_TO_QM_SCR_RING_SIZE

#else  /* Core Components */

#define POS_RX_TO_FWDER_SCR_RING        POS_RX_TO_IPV4_SCR_RING
#define POS_RX_TO_FWDER_SCR_RING_BASE   POS_RX_TO_IPV4_SCR_RING_BASE
#define POS_RX_TO_FWDER_SCR_RING_SIZE   POS_RX_TO_IPV4_SCR_RING_SIZE

#define FWDER_TO_QM_SCR_RING            IPV4_TO_QM_SCR_RING
#define FWDER_TO_QM_SCR_RING_BASE       IPV4_TO_QM_SCR_RING_BASE
#define FWDER_TO_QM_SCR_RING_SIZE       IPV4_TO_QM_SCR_RING_SIZE

#define FWDER_RING_IN        POS_RX_TO_FWDER_SCR_RING
#define FWDER_RING_IN_BASE   POS_RX_TO_FWDER_SCR_RING_BASE
#define FWDER_RING_IN_SIZE   POS_RX_TO_FWDER_SCR_RING_SIZE

#define FWDER_RING_OUT       FWDER_TO_QM_SCR_RING
#define FWDER_RING_OUT_BASE  FWDER_TO_QM_SCR_RING_BASE
#define FWDER_RING_OUT_SIZE  FWDER_TO_QM_SCR_RING_SIZE
#endif

/* Ring number to use. 0-15. */
#define_eval DL_RING_NUMBER          POS_RX_TO_FWDER_SCR_RING
#define_eval DL_DROP_RING            FWDER_TO_QM_SCR_RING

#endif // __SYSTEM__H__
