#include "xscale_common.h"

int ipv6_init() {
    
    int addr, end_addr;
    int cnt;

#if 0
    /* 
     * Clear route tables. 
     */

    addr = IPV6_TRIE_TABLE_SRAM_BASE_16;
    cnt  = 65535;
    end_addr = addr + (cnt * 4);

    printf("Clearing IPv6 SRAM 16 bit trie table... from %x to %x \n", addr, end_addr);
    init_sram(0x0,addr,end_addr);

#endif

    addr = IPV6_TRIE_TABLE_SRAM_BASE_8;
    cnt = 64 * 1024 + 256 + SIZE_OF_IPV6_TRIE_SET * NUM_IPV6_TRIE_SETS;
    end_addr = addr + (cnt * 4);

    printf("Clearing IPv6 SRAM 8 bit trie table...from %x to %x \n", addr, end_addr);
    init_sram(0x0,addr,end_addr);

	return;
    

    addr = IPV6_L3_NEXTHOP_TABLE_SRAM_BASE;
    cnt = IPV6_NUM_L3_NEXTHOPS * IPV6_SIZE_OF_L3_NEXTHOP_LW * 4;
    end_addr = addr + cnt;

    printf("Clearing SRAM L3 nexthop info...from %x to %x \n", addr, end_addr);
    init_sram(0x0,addr,end_addr);

#ifdef IPV6_STATS
    /*
     * Init counters
     */

    printf("\n\n IPv6: Reset Packet Counters =============");

    addr = IPV6_STATS_TABLE_SRAM_BASE ;
    end_addr = addr + IPV6_STATS_TABLE_SRAM_SIZE;

    init_sram(0x0, addr, end_addr);
#endif

    addr = IPV6_ROUTER_PREFIX_SRAM_BASE; 

    printf("\n\n IPv6: Set Router Prefix 0x3FFE:1234 /32 =============\n");        

    /* 
     * Assume a 32-bit prefix 0x3ffe1234 /32 
     */
    set_sram(addr+0x00, 0x33fe1234);
    set_sram(addr+0x04, 0xFFFFFFFF);  

    /*
     * Set the Trie-entries 
     */
    set_sram(addr+0x08, 0x00010001);
    set_sram(addr+0x0C, 0x00020002);
    set_sram(addr+0x10, 0x00030003);
    set_sram(addr+0x14, 0x00000000);
    set_sram(addr+0x18, 0x00000000);
    set_sram(addr+0x1c, 0x00000000);
    set_sram(addr+0x20, 0x00000000);

    return 0;
}

/*
// Creates a next hop SRAM block at the index specified
// The next hop block is 64 bytes in size
// We currently update only the following information
// writing nexthop entry for a given index
// Output port (32 bits)	= port
// MTU         (16 bits)	= mtu
// Flags       (16 bits)	= flags
// Dst MAC     (32 bits)	= dst_mac_hi
//             (16 bits)	  dst_mac_lo
// Src MAC     (16 bits)	= 0x    5555
//             (32 bits)	= 0x55555555
// Frame Type  (32 bits)	= 0x00000000
*/
int create_next_hop(int index, int dst_mac_hi, int dst_mac_lo, int blade_id, int port, int mtu, int flags) {

    /*
    // Create L3 structures
	// ipv6_create_l3_next_hop(index, flags, blade_id, nh_type, nh_id, mtu, port)
     */

	ipv6_create_l3_next_hop(index, flags, blade_id, 0, index, mtu, port);

	return 0;
}



/*
// Creates a L3 next hop SRAM block at the index specified
// The next hop block is 16 bytes in size. 
//     nh_index     : index at which to setup the L3 structure in the L3-next-hop table
//     flags        : 0x1 - local, 0x2 - down, 0x4 - drop
//     blade_id        :
//     nh_id_type   : type of next-hop-id ( 0 - L2 index, 0x2 - Tunneled, 0x4 - Compressed )
//     nh_id           : contains l2 or tunneling table index
//     port         : ouput port
//     mtu          : mtu size 
*/

int ipv6_create_l3_next_hop(int index, int flags, int blade_id, int nh_type, int nh_id, int mtu, int port) {

    int addr;
    int blade_index;
    int mtu_port;

    /*
    // Merge the blade id, l2 type and l2 index to form the second longword
    // of the structure.
     */
    blade_index    = (nh_id & 0xFFFF) | ((nh_type & 0xF) <<20) | ((blade_id & 0xFF) <<24);

    /*
    // Merge the mtu and port to form the third longword of the structure.
     */
    mtu_port = ((mtu & 0xFFFF) <<16) | (port & 0xFFFF);

    /*
    // Calculate the addr of the next hop pointer based on the index
    // Each block is 16 bytes
     */
    addr = IPV6_L3_NEXTHOP_TABLE_SRAM_BASE + (index * 16);


    /*
    // Fill up the next hop information
     */
    set_sram(addr + 0x00,flags);
    set_sram(addr + 0x04,blade_index);
    set_sram(addr + 0x08,mtu_port);

    return 0;
}


/*
// ipv6_create_l3_intermediate_next_hop
//
// Creates a L3 next hop SRAM block at the index specified by nh_index
// The next hop block is 16 bytes in size. 
//     nh_index          : index at which to setup the L3 structure in the L3-next-hop table
//     next_hop_id1-4    : contains L3 indices of other "real" next hop entries
//       status             : status of the links (UP/DOWN)
//
 */
int ipv6_create_l3_intermediate_next_hop(int nh_index, int nhid0, int nhid1, int nhid2, int nhid3, int status) {

    int addr, base;
    int id0_n_id1, id2_n_id3;
    int flags_n_status;

    /*
    // Calculate the addr of the next hop pointer based on the index
    // Each block is 16 bytes.
    */
    base =  IPV6_L3_NEXTHOP_TABLE_SRAM_BASE;
    addr =  base + (nh_index * 16);

    flags_n_status = (status << 24) | 0x0008;

    id0_n_id1 = (nhid0 << 16) | nhid1;
    id2_n_id3 = (nhid2 << 16) | nhid3;

    /*
    // Fill up the next hop information
     */
    printf ("ipv6_create_l3_intermediate_next_hop1 : location (0x%x) value (0x%x) \n",(addr + 0x00), flags_n_status);

    set_sram(addr + 0x00, flags_n_status);

    printf ("ipv6_create_l3_intermediate_next_hop1 : location (0x%x) value (0x%x) \n",(addr + 0x04), id0_n_id1);
    set_sram(addr + 0x04, id0_n_id1);

    printf ("ipv6_create_l3_intermediate_next_hop1 : location (0x%x) value (0x%x) \n",(addr + 0x08), id2_n_id3);
    set_sram(addr + 0x08, id2_n_id3);

    return 0;
}

/*
// set_8_bit_trie_entry
//
// Internal function to set a trie-entry for an 8-bit stride
//
// Parameters:
//
//     block_id       : index of the 8-bit tire-block in to which the trie-entry is to be set
//     index          : 8-bit portion of route-entry ip that is used as an index to access trie-entry
//     next_hop       : value that will be set in the higher 16 bits of the trie entry as next-hop-pointer
//     next_trie      : value that will be set in the lower 16 bits of the trie entry as next-trie-pointer
//     bits_remaining : how many bits of the route-entry remains to be processed
//                      used to determine if we have reached the end to set the next-hop
//
//
*/
int set_8_bit_trie_entry(int block_id, int index, int next_hop, int next_trie, int bits_remaining) {

    int location;
    int trie_entry;

    location = IPV6_TRIE_TABLE_SRAM_BASE_8 + (1024 * block_id);
    location += index * 4;
    trie_entry = get_sram(location);
    if ( !bits_remaining ) {
        printf("\n Setting next-hop  pointer in trie-block%d[%d](%h) <- 0x%08x ",  block_id, location, index, ((next_hop << 16) | (trie_entry & 0x0000FFFF)));
        /*
        // Only clear and reset next-hop portion
         */
        set_sram(location, ((next_hop << 16) | (trie_entry & 0x0000FFFF)));
        return 0;
    }

    printf("\n Setting next-trie pointer in trie-block%d[%d](%h) <- 0x%08x", block_id, location,index, ((trie_entry & 0xFFFF0000 ) | next_trie));
    /*
    // Clear and reset next-trie portion
     */
    set_sram(location, ((trie_entry & 0xFFFF0000 ) | next_trie));
    return 1;
}


/*
// ipv6_route_add
//
// Add a IPv6 route-entry to the route table
//
//
// Parameters:
//     dst_127_96, int dst_95_64, int dst_63_32, int dst_31_0 : Four long contain the route-entry
//     prefix_len : Length of the route-entry (in bits) that is being added
//     next_hop_id : Index of the next-hop-structure that will be associated with the route-entry
//     first_8_bit_block : Index of the trie-block on to which the first 8-bit stried will be mapped
//                         Subsequent strides will be mapped to contiguous trie-blocks starting from the 
//                         first block
// Examples :
//
//     // First, create next-hop at index 4 with DMAC 0xD10203040506, port 1, and given mtu and flags
//     ipv6_create_next_hop(4, 0xD1020304, 0x0506, 1<<4, MTU, NH_FLAGS)
//
//     // Create next-hop at index 5 with DMAC 0xE10203040506, port 2, and given mtu and flags
//     ipv6_create_next_hop(4, 0xE1020304, 0x0506, 2<<4, MTU, NH_FLAGS)
//
//     // Create next-hop at index 6 with DMAC 0xF10203040506, port 3, and given mtu and flags
//     ipv6_create_next_hop(4, 0xF1020304, 0x0506, 3<<4, MTU, NH_FLAGS)
//
//     Add a 32-bit route  0x3F21AABB::0 /32 with next-hop 4 (going out to port PORT)
//     Use trie-block 1 as a first trie
//
//     ipv6_route_add(0x3F21AABB, 0, 0, 0, 32, 4, 1)
//
//     It will be mapped as follows
//         0x3f21 - Root 16 bit trie-block
//         0xAA   - trie-block 1
//         0xBB   - trie-block 2
//
//     Add some more routes
//
//     ipv6_route_add(0x3F21AACC, 0, 0, 0, 32, 5, 1)    // overlapping route
//
//     It will be mapped as follows
//         0x3f21 - Root 16 bit trie-block
//         0xAA   - trie-block 1
//         0xCC   - trie-block 2
//
//     ipv6_route_add(0x3F22AABB, 0xCC, 0, 0, 40, 6, 7) // distinct route
//
//     It will be mapped as follows
//         0x3f22 - Root 16 bit trie-block
//         0xAA   - trie-block 7
//         0xBB   - trie-block 8
//         0xCC   - trie-block 9
//
//        
//    
//
*/

int ipv6_route_add(int dst_127_96, int dst_95_64, int dst_63_32, int dst_31_0, int prefix_len, int next_hop_id, int first_8_bit_block) {

    int index;
    int location;
    int trie_entry;
    int this_block;
    int bits_remaining;

    if ( prefix_len < 16 ) {
        printf("\n Error: ipv6_route_add  - Can not add routes shorter than 16 bits ");
        return 0;
    }
    if ( prefix_len % 8 ) {
        printf("\n Error: ipv6_route_add  - Can not add routes with lenghts not multiple of 8 ");
        return 0;
    }

    /*
    //
    // We are dealing with the first 16-bit Root trie block
    //
     */
    location = IPV6_TRIE_TABLE_SRAM_BASE_16 ;
    index = ((dst_127_96 >> 16) & 0xFFFF);
    location = location + (index * 4);
    bits_remaining = prefix_len - 16;

    /*
    // If the prefix match is /16, then we add a valid next hop id
    // Else, next hop idx is left alone
    */
    trie_entry = get_sram(location);
    if ( !bits_remaining ) {
        /*
        // Clear and reset only next-hop portion
         */
        set_sram(location, ((next_hop_id << 16) | (trie_entry & 0x0000FFFF)));
        return 0;
    }
    /*
    // Clear and reset next-tricxzve portion
     */
    set_sram(location,((trie_entry & 0xFFFF0000 ) | first_8_bit_block));

    this_block = first_8_bit_block;
    bits_remaining -= 8;
    index = ((dst_127_96 >> 8) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;


    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_127_96 >> 0) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    /*
    //
    // 2nd Long-Word of DestIP
    //   
     */

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_95_64 >> 24) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_95_64 >> 16) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_95_64 >> 8) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_95_64 >> 0) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    /*
    //
    // 3rd Long-Word of DestIP
    //   
     */


    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_63_32 >> 24) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_63_32 >> 16) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_63_32 >> 8) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_63_32 >> 0) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    /*
    //
    // 4th Long-Word of DestIP
    //   
     */

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_31_0 >> 24) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_31_0 >> 16) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_31_0 >> 8) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);
    if ( !bits_remaining )
       return 0;

    this_block += 1;
    bits_remaining -= 8;
    index = ((dst_31_0 >> 0) & 0xFF);
    set_8_bit_trie_entry(this_block, index, next_hop_id, (this_block+1), bits_remaining);

    return 0;

}

/*
//    Simple route add routine
//  Given an 128 bit IP address aaaa:bbcc:ddee:ffgg:..
//  It will add 2 routes one a /16 and the other a /128
//  The next hop destination MAC address will be setup to
//  reflect the level of the prefix matched (for easier validation)
//  i.e.,
//    aaaa::/16            -> Next Hop index 1, Dst Mac 0x00DD0100016
//  aaaa:bbcc:.../128  -> Next Hop index 15, Dst Mac 0x00DD0100128
//
//    The route add routine will always allocate 8 bit SRAM trie blocks 1, 2, 3, 4 ... 
 */
int route_add_16_128(int dst_127_96, int dst_95_64, int dst_63_32, int dst_31_0, int port) {

    int location    = 0;
    int    one_k_trie    = 1024;
    int mac_hi_32    = 0x00DD0100;
    int mtu          = 1024 * 9;
    int flags        = 0x0000;
    int block_id;
    int    next_hop_id;
	int	blade_id	= 3;

    /*
    // Setup the pointer from the first 16 bit trie block to point to
    // first 8 bit trie block
    // The next hop is set to index 1
     */
    block_id    =    1;
    next_hop_id    =    1;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_16 + (((dst_127_96 >> 16) & 0xFFFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 


	printf("\nBase = %08x Location = %08x, Value = %08x\n", IPV6_TRIE_TABLE_SRAM_BASE_16, location, ((next_hop_id << 16) | block_id)); 


    /*
    // writing nexthop entry for index with the last 16 bits denoting the
    // length of prefix matched at this point
     */
    create_next_hop(next_hop_id, mac_hi_32, 0x16, blade_id, port, mtu, flags);

    /*
    // Setup the pointer from the first 8 bit trie block to point to
    // second 8 bit trie block and so on
     */
    block_id    =    2;
    next_hop_id    =    0;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_127_96 >> 8) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

    /*
    //create_next_hop(next_hop_id, mac_hi_32, 0x24, port, mtu, flags);
     */

    block_id    =    3;
    next_hop_id    =    0;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + ((dst_127_96 & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

    /*
    //create_next_hop(next_hop_id, mac_hi_32, 0x32, port, mtu, flags);
     */


    block_id    =    4;
    next_hop_id    =    0;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64 >> 24) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

    /*
    //create_next_hop(next_hop_id, mac_hi_32, 0x40, port, mtu, flags);
     */

    block_id    =    5;
    next_hop_id    =    0;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64 >> 16) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

    /*
    //create_next_hop(next_hop_id, mac_hi_32, 0x48, port, mtu, flags);
     */


    block_id    =    6;
    next_hop_id    =    0;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64 >> 8) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

    /*
    //create_next_hop(next_hop_id, mac_hi_32, 0x56, port, mtu, flags);
     */


    block_id    =    7;
    next_hop_id    =    0;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

    /*
    //create_next_hop(next_hop_id, mac_hi_32, 0x64, port, mtu, flags);
     */

    block_id    =    8;
    next_hop_id    =    0;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_63_32 >> 24) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

    /*
    //create_next_hop(next_hop_id, mac_hi_32, 0x72, port, mtu, flags);
     */

    block_id    =    9;
    next_hop_id    =    0;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_63_32 >> 16) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

    /*
    //create_next_hop(next_hop_id, mac_hi_32, 0x80, port, mtu, flags);
     */


    block_id    =    10;
    next_hop_id    =    0;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_63_32 >> 8) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

    /*
    //create_next_hop(next_hop_id, mac_hi_32, 0x88, port, mtu, flags);
     */

    block_id    =    11;
    next_hop_id    =    0;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_63_32) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

    /*
    //create_next_hop(next_hop_id, mac_hi_32, 0x96, port, mtu, flags);
     */

    block_id    =    12;
    next_hop_id    =    0;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_31_0 >> 24) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

    /*
    //create_next_hop(next_hop_id, mac_hi_32, 0x104, port, mtu, flags);
     */


    block_id    =    13;
    next_hop_id    =    0;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_31_0 >> 16) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

    /*
    //create_next_hop(next_hop_id, mac_hi_32, 0x112, port, mtu, flags);
     */

    block_id    =    14;
    next_hop_id    =    0;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_31_0 >> 8) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

    /*
    //create_next_hop(next_hop_id, mac_hi_32, 0x120, port, mtu, flags);
     */


    block_id    =    15;
    next_hop_id    =    15;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_31_0) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 
    create_next_hop(next_hop_id, mac_hi_32, 0x128, blade_id, port, mtu, flags);

    return 0;

}


/*
//    Simple route add routine
//    Adds 16 routes /64 routes
//			Input Route -> Blade Id 1
//			Input Route -1 -> Blade Id 2
//			...
//			Input Route -15 -> Blade Id 16
//
 */
int route_add_16_64(int dst_127_96, int dst_95_64, int port) {

    int location    = 0;
    int one_k_trie  = 1024;
    int mac_hi_32   = 0x00DD0100;
    int mtu         = 1024 * 9;
    int flags       = 0x0000;
	int	blade_id;
    int block_id;
    int next_hop_id;


    /*
    // Setup the pointer from the first 16 bit trie block to point to
    // first 8 bit trie block
    // The next hop is set to index 1
     */
    block_id    =    1;
    next_hop_id =    0;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_16 + (((dst_127_96 >> 16) & 0xFFFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 


	printf("\nBase = %08x Location = %08x, Value = %08x\n", IPV6_TRIE_TABLE_SRAM_BASE_16, location, ((next_hop_id << 16) | block_id)); 


    /*
    // Setup the pointer from the first 8 bit trie block to point to
    // second 8 bit trie block and so on
     */
    block_id    =    2;
    next_hop_id =    0;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_127_96 >> 8) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

    block_id    =    3;
    next_hop_id =    0;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + ((dst_127_96 & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

    block_id    =    4;
    next_hop_id =    0;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64 >> 24) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

    block_id    =    5;
    next_hop_id =    0;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64 >> 16) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

    block_id    =    6;
    next_hop_id =    0;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64 >> 8) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

	/* Route 1 9012 */
    block_id    =    7;
    next_hop_id =    1;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

	blade_id    =    1;
    create_next_hop(next_hop_id, mac_hi_32, 0x64, blade_id, port, mtu, flags);

	/* Route 2 9011 */
    block_id    =    7;
    next_hop_id =    2;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64 - next_hop_id + 1) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 


	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x64, blade_id, port, mtu, flags);


	/* Route 3 9010 */
    block_id    =    7;
    next_hop_id =    3;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64 - next_hop_id + 1) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 


	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x64, blade_id, port, mtu, flags);


	/* Route 4 900F */
    block_id    =    7;
    next_hop_id =    4;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64 - next_hop_id + 1) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 


	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x64, blade_id, port, mtu, flags);

	/* Route 5 900E */
    block_id    =    7;
    next_hop_id =    5;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64 - next_hop_id + 1) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 


	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x64, blade_id, port, mtu, flags);


	/* Route 6 900D */
    block_id    =    7;
    next_hop_id =    6;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64 - next_hop_id + 1) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 


	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x64, blade_id, port, mtu, flags);


	/* Route 7 900C */
    block_id    =    7;
    next_hop_id =    7;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64 - next_hop_id + 1) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 


	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x64, blade_id, port, mtu, flags);

	/* Route 8 900B */
    block_id    =    7;
    next_hop_id =    8;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64 - next_hop_id + 1) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 


	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x64, blade_id, port, mtu, flags);

	/* Route 9 900A */
    block_id    =    7;
    next_hop_id =    9;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64 - next_hop_id + 1) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x64, blade_id, port, mtu, flags);


	/* Route 10 9009 */
    block_id    =    7;
    next_hop_id =    10;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64 - next_hop_id + 1) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 


	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x64, blade_id, port, mtu, flags);


	/* Route 11 9008 */
    block_id    =    7;
    next_hop_id =    11;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64 - next_hop_id + 1) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 


	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x64, blade_id, port, mtu, flags);


	/* Route 12 9007 */
    block_id    =    7;
    next_hop_id =    12;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64 - next_hop_id + 1) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 


	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x64, blade_id, port, mtu, flags);

	/* Route 13 9006 */
    block_id    =    7;
    next_hop_id =    13;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64 - next_hop_id + 1) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 


	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x64, blade_id, port, mtu, flags);

	/* Route 14 9005 */
    block_id    =    7;
    next_hop_id =    14;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64 - next_hop_id + 1) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 


	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x64, blade_id, port, mtu, flags);

	/* Route 15 9004 */
    block_id    =    7;
    next_hop_id =    15;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64 - next_hop_id + 1) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 


	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x64, blade_id, port, mtu, flags);

	/* Route 16 9003 */
    block_id    =    7;
    next_hop_id =    16;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_95_64 - next_hop_id + 1) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 


	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x64, blade_id, port, mtu, flags);


    return 0;

}




/*
//    Simple route add routine
//    Adds 16 routes /24 routes
//			Input Route -> Blade Id 1
//			Input Route -1 -> Blade Id 2
//			...
//			Input Route -15 -> Blade Id 16
//
 */
int route_add_16_24(int dst_127_96, int port) {

    int location    = 0;
    int one_k_trie  = 1024;
    int mac_hi_32   = 0x00DD0100;
    int mtu         = 1024 * 9;
    int flags       = 0x0000;
	int	blade_id;
    int block_id;
    int next_hop_id;

    /*
    // Setup the pointer from the first 16 bit trie block to point to
    // first 8 bit trie block
    // The next hop is set to index 1
     */
    block_id    =    1;
    next_hop_id =    0;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_16 + (((dst_127_96 >> 16) & 0xFFFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 


    /*
    // Setup the pointer from the first 8 bit trie block to point to
    // second 8 bit trie block and so on
     */

	/* Route 1 3FFE:12 */

    block_id    =    2;
    next_hop_id =    1;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + (((dst_127_96 >> 8) & 0xFF) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x24, blade_id, port, mtu, flags);

	/* Route 2 3FFE:11 */

    block_id    =    2;
    next_hop_id =    2;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + ((((dst_127_96 >> 8) & 0xFF) - next_hop_id + 1) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x24, blade_id, port, mtu, flags);

	/* Route 3 3FFE:10 */

    block_id    =    2;
    next_hop_id =    3;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + ((((dst_127_96 >> 8) & 0xFF) - next_hop_id + 1) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x24, blade_id, port, mtu, flags);

	/* Route 4 3FFE:0F */

    block_id    =    2;
    next_hop_id =    4;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + ((((dst_127_96 >> 8) & 0xFF) - next_hop_id + 1) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x24, blade_id, port, mtu, flags);

	/* Route 5 3FFE:0E */

    block_id    =    2;
    next_hop_id =    5;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + ((((dst_127_96 >> 8) & 0xFF) - next_hop_id + 1) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x24, blade_id, port, mtu, flags);

	/* Route 6 3FFE:0D */

    block_id    =    2;
    next_hop_id =    6;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + ((((dst_127_96 >> 8) & 0xFF) - next_hop_id + 1) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x24, blade_id, port, mtu, flags);

	/* Route 7 3FFE:0C */

    block_id    =    2;
    next_hop_id =    7;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + ((((dst_127_96 >> 8) & 0xFF) - next_hop_id + 1) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x24, blade_id, port, mtu, flags);

	/* Route 8 3FFE:0B */

    block_id    =    2;
    next_hop_id =    8;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + ((((dst_127_96 >> 8) & 0xFF) - next_hop_id + 1) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x24, blade_id, port, mtu, flags);


	/* Route 9 3FFE:0A */

    block_id    =    2;
    next_hop_id =    9;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + ((((dst_127_96 >> 8) & 0xFF) - next_hop_id + 1) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x24, blade_id, port, mtu, flags);

	/* Route 10 3FFE:09 */

    block_id    =    2;
    next_hop_id =    10;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + ((((dst_127_96 >> 8) & 0xFF) - next_hop_id + 1) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x24, blade_id, port, mtu, flags);

	/* Route 11 3FFE:08 */

    block_id    =    2;
    next_hop_id =    11;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + ((((dst_127_96 >> 8) & 0xFF) - next_hop_id + 1) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x24, blade_id, port, mtu, flags);

	/* Route 12 3FFE:07 */

    block_id    =    2;
    next_hop_id =    12;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + ((((dst_127_96 >> 8) & 0xFF) - next_hop_id + 1) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x24, blade_id, port, mtu, flags);

	/* Route 13 3FFE:06 */

    block_id    =    2;
    next_hop_id =    13;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + ((((dst_127_96 >> 8) & 0xFF) - next_hop_id + 1) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x24, blade_id, port, mtu, flags);

	/* Route 14 3FFE:05 */

    block_id    =    2;
    next_hop_id =    14;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + ((((dst_127_96 >> 8) & 0xFF) - next_hop_id + 1) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x24, blade_id, port, mtu, flags);

	/* Route 15 3FFE:04 */

    block_id    =    2;
    next_hop_id =    15;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + ((((dst_127_96 >> 8) & 0xFF) - next_hop_id + 1) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x24, blade_id, port, mtu, flags);


	/* Route 16 3FFE:03 */

    block_id    =    2;
    next_hop_id =    16;
    location    =    IPV6_TRIE_TABLE_SRAM_BASE_8 + (one_k_trie * (block_id - 1));
    location    =    location + ((((dst_127_96 >> 8) & 0xFF) - next_hop_id + 1) * 4);
    set_sram(location, ((next_hop_id << 16) | block_id)); 

	blade_id    =    next_hop_id;
    create_next_hop(next_hop_id, mac_hi_32, 0x24, blade_id, port, mtu, flags);

    return 0;

}


int l3CreateRoutes_TCS_IPv6_0001() {
    /*
    // Setup trie table for the route such that
    //    3FFE:1234:5678:9012:3456:7890:1234:5678/128 -> Dest MAC 0x00DD010128
    //    3FFE:1234:5678:9012:3456:7890:1234:5678/16  -> Dest MAC 0x00DD010016
     */

    /* route_add_16_128(0x3FFE1234, 0x56789012, 0x34567890, 0x12345678, 0x05); */
	route_add_16_64(0x3FFE1234, 0x56789012, 0x05);
	/* route_add_16_24(0x3FFE1234, 0x05); */
    return 1;

}



int v6v4TunnelInit() {
    
    int cnt, addr;
    int baseaddr;

    printf("v6v4TunnelInit(): Initializing ipv6-ipv4 tunneling memory :=..\n");

    baseaddr = TUNNEL_ENCAP_NEXTHOP_SRAM_BASE;
    cnt = NUM_START_TUNNEL_NEXTHOPS * START_TUNNEL_NEXTHOP_LW;
    addr = baseaddr + (cnt * 4);

    printf("Clearing SRAM start tunnel next hop info...\n");
    init_sram(0x0,baseaddr,addr);
    

    baseaddr = TUNNEL_DECAP_NEXTHOP_SRAM_BASE;
    cnt = NUM_END_TUNNEL_NEXTHOPS * END_TUNNEL_NEXTHOP_LW;
    addr = baseaddr + (cnt * 4);

    printf("Clearing SRAM end tunnel next hop info...\n");
    init_sram(0x0,baseaddr,addr);
    
    /*
    // Clear ingress source list.
     */

    baseaddr = TUNNEL_INGRESS_LIST_SRAM_BASE;

#ifdef V6V4_INGRESS_LIST_TRIE
    cnt = 256 + (INGRESS_LIST_NUM_TRIE_BLOCKS * 16);
    cnt += V6V4_INGRESS_LIST_NUM_INDEX_BLOCKS * (V6V4_INGRESS_LIST_INDEX_BLOCK_SIZE / 2);
    printf("Clearing SRAM ingress source list trie blocks...\n");
	printf("1 cnt %x\n", cnt);
#else
    cnt = V6V4_NUM_INGRESS_LIST_BLOCKS * V6V4_INGRESS_LIST_LW;
    printf("Clearing SRAM ingress source list blocks...\n");
	printf("2 cnt %x\n", cnt);
#endif

    addr = baseaddr + (cnt * 4);
    init_sram(0x0,baseaddr,addr);


#ifdef V6V4_DECAP_COUNTERS
    baseaddr = V6V4_DECAP_COUNTER_BASE;
    cnt = V6V4_NUM_DECAP_COUNTERS;
    addr = baseaddr + (cnt * 4);

    printf("Clearing tunnel decap packet counters...\n");
    init_sram(0x0,baseaddr,addr);
#endif

#ifdef V6V4_ENCAP_COUNTERS
    baseaddr = V6V4_ENCAP_COUNTER_BASE;
    cnt = V6V4_NUM_ENCAP_COUNTERS;
    addr = baseaddr + (cnt * 4);

    printf("Clearing tunnel encap packet counters...\n");
    init_sram(0x0,baseaddr,addr);
#endif


    printf("V6V4 tunneling memory initialization COMPLETE.\n\n");

    return 1;
}

