/*------------------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//------------------------------------------------------------------------------------
*/

#ifndef __PKTHDR_CACHE_UC__
#define __PKTHDR_CACHE_UC__

#include "definitions.h"
#include "ppp.h"

#define_eval PKT_HDR_CACHE_BYTES  (PKT_HDR_CACHE_SIZE * 4)  // 64 bytes
#define_eval PKT_HDR_DRAM_RD_SIZE (PKT_HDR_CACHE_SIZE >> 1) // read 8 quadwords

// Base address of local memory buffers used for caching the IP header.

#define LM_PKTHDR_BASE             0

// Context relative offsets (in bytes) of protocol header buffers.

#define LM_BUF_OFFSET_IPV4_HDR     0
#define LM_BUF_OFFSET_IPV6_HDR     64
#define LM_BUF_OFFSET_IPV6_HDR_1   64
#define LM_BUF_OFFSET_IPV6_HDR_2   96

// Buffer sizes of header buffers in 32-bit words

#define LM_BUF_LWSIZE_IPV4_HDR	   8
#define LM_BUF_LWSIZE_IPV6_HDR     16
#define LM_BUF_LWSIZE_IPV6_HDR_2   8
#define LM_BUF_LWSIZE_IPV6_HDR_1   8

// Buffer sizes of header buffers in bytes

#define_eval LM_BUF_SIZE_IPV4_HDR      (LM_BUF_LWSIZE_IPV4_HDR * 4)
#define_eval LM_BUF_SIZE_IPV6_HDR      (LM_BUF_LWSIZE_IPV6_HDR * 4)
#define_eval LM_BUF_SIZE_IPV6_HDR_1    (LM_BUF_LWSIZE_IPV6_HDR_2 * 4)
#define_eval LM_BUF_SIZE_IPV6_HDR_2    (LM_BUF_LWSIZE_IPV6_HDR_1 * 4)

// Total pool size in bytes

#define_eval LM_POOL_SIZE (LM_BUF_OFFSET_IPV6_HDR + LM_BUF_SIZE_IPV6_HDR)

// Total number of bytes to cache for each header type.  We always cache at
// least 6 extra bytes to allow for alignment when writing back to DRAM.

#define LM_XFER_BYTES_IPV4          26
#define LM_XFER_BYTES_IPV6          46
#define LM_XFER_BYTES_IPV6_1        32
#define LM_XFER_BYTES_IPV6_2        14

// Actual header sizes in and in bytes

#define PKT_HDR_BYTES_IPV4          20
#define PKT_HDR_BYTES_IPV6          40
#define PKT_HDR_BYTES_IPV6_1        32
#define PKT_HDR_BYTES_IPV6_2        8

// Names of local memory buffers.

#define LM_IPV4_HDR            lmem_buf0
#define LM_IPV6_HDR            lmem_buf1
#define LM_IPV6_HDR_1          lmem_buf2
#define LM_IPV6_HDR_2          lmem_buf3

// Tunnel header type bindings.

#define      V6V4_HDR_TYPE_DECAP     PPP_IPV6_TYPE
#define      V6V4_HDR_TYPE_ENCAP     TUNNEL_ENCAP_FLAG

#define_eval PPP_IPV6_IN_IPV4_TYPE (PPP_IPV6_TYPE | V6V4_HDR_TYPE_ENCAP)


///////////////////////////////////////////////////////////////////////////////
// dl_load_iphdr_cache:
//	 	Description: 
//			Reads the initial portion of the packet header from DRAM into 
//			DRAM transfer registers. 
//
//	 	Outputs:
//			The $$iphdr# global dram transfer registers will be set with appropriate
//			header data.
//
//		Inputs:
//			buf_handle	:			Handle of buffer.
//			req_sig		:			Signal to use in the I/O (DRAM read) operation.
//			sig_action	:			What to do with I/O operation - 
//									- wait for signal(s) as specified by sig_action
//									- do not wait for signal, just return (SIG_NONE)
//		Size:   				:
//
#define	dl_load_iphdr_cache		dl_iphdr_load_cache

#macro dl_iphdr_load_cache[buf_handle, req_sig, sig_action]
.begin
.reg 	sdram_offset, buf_offset

	//	Read initial portion of packet header from DRAM

	dl_buf_get_data[sdram_offset, buf_handle]
	dl_meta_get_offset[buf_offset]
	dram[read, $$iphdr[0], sdram_offset, buf_offset, PKT_HDR_DRAM_RD_SIZE], sig_done[req_sig]	; DRAM reads are in QWORD (8 bytes)

#if (!streq('sig_action', 'SIG_NONE'))
	ctx_arb[sig_action]
#endif

.end
#endm

// Variant of the above macro which only loads the first 32 bytes of the header
#macro dl_iphdr_load_cache_32[buf_handle, req_sig, sig_action]
.begin
.reg 	sdram_offset, buf_offset, pkt_size

	dl_buf_get_data[sdram_offset, buf_handle]
	dl_meta_get_offset[buf_offset]

	//	Read initial portion of packet header from DRAM
	dram[read, $$iphdr[0], sdram_offset, buf_offset, 4], sig_done[req_sig]	; DRAM reads are in QWORD (8 bytes)

#if (!streq('sig_action', 'SIG_NONE'))
	ctx_arb[sig_action]
#endif

.end
#endm



///////////////////////////////////////////////////////////////////////////////
// dl_iphdr_cache_init:
//	 	Description: 
//			This macro needs to be called before any of the dl_iphdr_* can be called.
//			It allocates a set of DRAM xfer registers to be used for caching. This macro
//			doesn't generate any code (instructions). It only produces some assembler
//			directives.
//
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//			0 Instructions

#macro dl_iphdr_cache_init[]

	xbuf_alloc[$$iphdr, PKT_HDR_CACHE_SIZE, read_write]

    // Suppres assembler warnings about transfer registers used without being
	// set.
    .set $$iphdr[0], $$iphdr[1], $$iphdr[2], $$iphdr[3], $$iphdr[4], $$iphdr[5], $$iphdr[6]
	.set $$iphdr[7], $$iphdr[8], $$iphdr[9], $$iphdr[10], $$iphdr[11], $$iphdr[12]
	.set $$iphdr[13], $$iphdr[14], $$iphdr[15]

    // Set up local memory cache buffers.

	xbuf_alloc(LM_IPV4_HDR, LM_BUF_LWSIZE_IPV4_HDR, read_write)
	xbuf_alloc(LM_IPV6_HDR, LM_BUF_LWSIZE_IPV6_HDR, read_write)
	xbuf_alloc(LM_IPV6_HDR_1, LM_BUF_LWSIZE_IPV6_HDR_1, read_write)
	xbuf_alloc(LM_IPV6_HDR_2, LM_BUF_LWSIZE_IPV6_HDR_2, read_write)

	xbuf_bind_address(LM_IPV4_HDR, LM_PKTHDR_BASE,
	                  LM_POOL_SIZE, LM_BUF_OFFSET_IPV4_HDR)

	xbuf_bind_address(LM_IPV6_HDR, LM_PKTHDR_BASE,
	                  LM_POOL_SIZE, LM_BUF_OFFSET_IPV6_HDR)

	xbuf_bind_address(LM_IPV6_HDR_1, LM_PKTHDR_BASE,
	                  LM_POOL_SIZE, LM_BUF_OFFSET_IPV6_HDR_1)

	xbuf_bind_address(LM_IPV6_HDR_2, LM_PKTHDR_BASE,
	                  LM_POOL_SIZE, LM_BUF_OFFSET_IPV6_HDR_2)

    // Set thread ID in meta-data for use in xbuf_activate.

    .begin
	    .reg context_id

	    local_csr_rd[ACTIVE_CTX_STS]
	    immed[context_id, 0]
	    alu[context_id, context_id, AND, 0x7]

	    dl_set_thread_id(context_id)
    .end

#endm

///////////////////////////////////////////////////////////////////////////////
// dl_iphdr_cache_fini:
//	 	Description: 
//			This macro needs to be called at the end when dl_iphdr_macros are no longer
//			required. It frees a set of DRAM xfer registers that were allocated using
//			dl_iphdr_cache_init. This macro	doesn't generate any code (instructions). 
//			It only produces some assembler	directives.
//
//	 	Outputs:
//
//		Inputs:
//
//		Size:   				:
//			0 Instructions

#macro dl_iphdr_cache_fini[]

	xbuf_free[$$iphdr]

	xbuf_free(LM_IPV4_HDR)
	xbuf_free(LM_IPV6_HDR)
	xbuf_free(LM_IPV6_HDR_1)
	xbuf_free(LM_IPV6_HDR_2)

#endm

////////////////////////////////////////////////////////////////////////////////
// dl_copy_iphdr_to_lmem:
//
//      description:   
//
//			This macro copies the IP header to an aligned location in local
//          memory.  The IP header can be either an IPv4 or IPv6 header.
//
//	 	Outputs:
//          - IP header is copied to local memory cache buffer.
//
//		Inputs:
//			Header is currently in transfer registers.
//
////////////////////////////////////////////////////////////////////////////////
#macro dl_copy_iphdr_to_lmem()
.begin

    .reg hdr_type
	.reg context_id

    // Get thread ID for xbuf_activate.

	dl_get_thread_id(context_id)

	dl_meta_get_header_type(hdr_type)

    // Copy the IP header into the appropriate local memory buffer.  We
    // copy some additional bytes to allow for alignment when writing the
	// header back to DRAM.

    .if (hdr_type == PPP_IPV6_TYPE)

        // Activate the IPv6 buffer and then copy the IPv6 header
        // into the local memory buffer.  We are limited to 32 bytes
		// per local memory buffer copy, so we must copy in two phases.

        xbuf_activate(LM_IPV6_HDR_1, 0, context_id, 1)
        xbuf_activate(LM_IPV6_HDR_2, 1, context_id, 0)

        xbuf_copy(LM_IPV6_HDR_1, 0, 0, $$iphdr, 
                  PPP_HEADER_LENGTH, 0, LM_XFER_BYTES_IPV6_1, 0)

        #define_eval PART2_START (PPP_HEADER_LENGTH+LM_XFER_BYTES_IPV6_1)

        xbuf_copy(LM_IPV6_HDR_2, LM_XFER_BYTES_IPV6_1, 0, $$iphdr,
                  PART2_START, 0, LM_XFER_BYTES_IPV6_2, 0)

	    xbuf_deactivate(LM_IPV6_HDR_1)
	    xbuf_deactivate(LM_IPV6_HDR_2)

        #undef PART2_START

    .elif (hdr_type == PPP_IPV4_TYPE)

        // Activate the IPv4 buffer and then copy the IPv4 header
        // into the local memory buffer.

        xbuf_activate(LM_IPV4_HDR, 0, context_id, 1)

        // Set next hop ID to -1 for the IPv4 forwarder.
        .begin
            .reg nh_id
            immed[nh_id, 0xffff]
            dl_meta_set_nexthop_id(nh_id)
        .end

        xbuf_copy(LM_IPV4_HDR, 0, 0, $$iphdr, PPP_HEADER_LENGTH,
                  0, LM_XFER_BYTES_IPV4, 0)

	    xbuf_deactivate(LM_IPV4_HDR)

    .endif

    // If the packet is neither IPv6 nor IPv4, it will be dropped so
    // we don't need to do anything.

copy_cache_done#:

.end

#endm


//////////////////////////////////////////////////////////////////////////////
// dl_load_ipv6_hdr
//     Description: Loads the IPv6 header.  This macro is executed by the
//                  tunnel decapsulation block when it needs to decapsulate a
//                  packet.
//                     
// Output:
//     xbuf_name      : Name of local memory buffer to receive the header.
//     xbuf_offset    : Offset in xbuf_name at which header should begin.
//     HDR_BYTE_COUNT : Number of bytes in the header.
//
// Inputs:
//
//
////////////////////////////////////////////////////////////////////////////////
#macro dl_load_ipv6_hdr(xbuf_name, xbuf_offset, HDR_BYTE_COUNT)
.begin
	.reg context_id

#if ( !streq(xbuf_name, 'lmem_buf1') )
    #error "Memory buffer name for IPv6 is incorrect"
#endif

#if ( !isnum(xbuf_offset) )
    #error "Memory offset for IPv6 is incorrect"
#elif (xbuf_offset != 0)
    #error "xbuf_offset must be zero"
#endif

#if ( !isnum(HDR_BYTE_COUNT) )
    #error "Byte count for IPv6 header is incorrect"
#elif (HDR_BYTE_COUNT != PKT_HDR_BYTES_IPV6)
    #error "Byte count for IPv6 header is incorrect"
#endif

    // xbuf_copy allows only 32 bytes per local memory copy, so we must
	// do the IPv6 copy in 2 pieces.

	dl_get_thread_id(context_id)

    xbuf_activate(LM_IPV6_HDR_1, 0, context_id, 1)
    xbuf_activate(LM_IPV6_HDR_2, 1, context_id, 0)

	// Copy the first 32 bytes of the IPv6 header to the local memory byffer.

	#define_eval IPV6_START_BYTE	(PPP_HEADER_LENGTH + PKT_HDR_BYTES_IPV4)
	#define_eval BYTES_TO_COPY   	PKT_HDR_BYTES_IPV6_1

	xbuf_copy(LM_IPV6_HDR_1, 0, 0, $$iphdr, IPV6_START_BYTE, 0, BYTES_TO_COPY, 0)

    // Copy the remaining bytes to the local memory buffer.

    #define_eval IPV6_START_BYTE 	(IPV6_START_BYTE + BYTES_TO_COPY)
	#define_eval BYTES_TO_COPY 		(PKT_HDR_CACHE_BYTES - IPV6_START_BYTE)

    xbuf_copy(LM_IPV6_HDR_2, 0, 0, $$iphdr, IPV6_START_BYTE, 0, BYTES_TO_COPY, 0)

	xbuf_deactivate(LM_IPV6_HDR_1)
	xbuf_deactivate(LM_IPV6_HDR_2)

	#undef BYTES_TO_COPY
	#undef IPV6_START_BYTE
.end
#endm

///////////////////////////////////////////////////////////////////////////////
// _flush_v6_headers:
//	 	Description: 
//			Write the IP v6 header to DRAM, given a constant alignment
//
//	 	Outputs:
//			dram_offset : DRAM offset of packet buffer
//          hdr_offset  : Offset of layer 2 header in packet buffer.
//          ALIGN_OFFSET: Number of extra bytes that must precede the
//                        first header byte to achieve quad-word
//                        alignment (0 - 7).
//          req_sig     : Signal to use for write operation(s).
//
//		Inputs:
//
///////////////////////////////////////////////////////////////////////////////

#macro	_flush_v6_headers(dram_offset, hdr_offset, ALIGN_OFFSET, req_sig)
.begin
	.reg last_ipv6     // used with xbuf copy for merging portions of IPv6 hdr

        xbuf_copy($$iphdr, last_ipv6, ALIGN_OFFSET, LM_IPV6_HDR_1, 0, 0,
                  PKT_HDR_BYTES_IPV6_1, 0)

        #define_eval IPV6_START_BYTE (ALIGN_OFFSET + PKT_HDR_BYTES_IPV6_1)
	    #define_eval BYTES_TO_COPY   LM_XFER_BYTES_IPV6_2

        xbuf_copy($$iphdr, 0, IPV6_START_BYTE, LM_IPV6_HDR_2, 0, last_ipv6, 
				  BYTES_TO_COPY, 0)

        #define_eval QW_COUNT  ((ALIGN_OFFSET + PKT_HDR_BYTES_IPV6 + 7) / 8)
	    dram[write, $$iphdr[0], dram_offset, hdr_offset, QW_COUNT],
	        sig_done[req_sig]
.end
#endm

///////////////////////////////////////////////////////////////////////////////
// dl_flush_iphdr_cache:
//	 	Description: 
//			Writes back the IP header(s) from the local memory cache to the
//          packet in SDRAM. 
//
//	 	Outputs:
//
//		Inputs:
//			buf_handle	:			Handle of buffer.
//			req_sig		:			Signal to use in the I/O (DRAM read) operation.
//			sig_action	:			What to do with I/O operation - 
//									- wait for signal(s) as specified by sig_action
//									- do not wait for signal, just return (SIG_NONE)
//			The IP header (or headers, if the packet is tunneled) is in the
//          appropriate local memory buffer.  The header_type in the meta-data
//          is used to determine which headers to flush.
//
//		Size:   				:
//
///////////////////////////////////////////////////////////////////////////////
#define	dl_flush_iphdr_cache		dl_iphdr_flush_cache

#macro dl_iphdr_flush_cache[buf_handle, req_sig, sig_action]
.begin
	.reg hdr_offset

	.reg last_ipv4     // used with xbuf copy for merging IPv4 & IPv6 hdrs
	.reg last_ipv6     // used with xbuf copy for merging portions of IPv6 hdr
	.reg context_id    // thread ID for use in xbuf_activate
	.reg hdr_type      // header type from meta-data
	.reg dram_offset   // DRAM offset of packet buffer
	.reg tmp

	dl_meta_get_offset(hdr_offset)
    dl_get_thread_id(context_id)
;	dl_buf_get_data(dram_offset, buf_handle)
	dl_meta_get_header_type(hdr_type)


    // Flush the IP header.

    .if (hdr_type == PPP_IPV4_TYPE)

        // The IPv4 header needs to be flushed.

        xbuf_activate(LM_IPV4_HDR, 0, context_id, 1)

		dl_buf_get_data(dram_offset, buf_handle)

		#ifdef IPV6_IPV4_ONLY_PIPELINE

        xbuf_copy($$iphdr, 0, PPP_HEADER_LENGTH, LM_IPV4_HDR, 0, 0, 16, 0)
	    dram[write, $$iphdr[0], dram_offset, hdr_offset, 2], sig_done[req_sig]

		#else //IPV6_IPV4_ONLY_PIPELINE

        xbuf_copy($$iphdr, 0, PPP_HEADER_LENGTH, LM_IPV4_HDR, 0, 0,
                  LM_XFER_BYTES_IPV4, 0)

        #define_eval QW_COUNT  ((PPP_HEADER_LENGTH + PKT_HDR_BYTES_IPV4 + 7) / 8)
	    dram[write, $$iphdr[0], dram_offset, hdr_offset, QW_COUNT],
	        sig_done[req_sig]

		#endif //IPV6_IPV4_ONLY_PIPELINE

        xbuf_deactivate(LM_IPV4_HDR)

    .elif (hdr_type == PPP_IPV6_TYPE)

        xbuf_activate(LM_IPV6_HDR_1, 0, context_id, 0)
        xbuf_activate(LM_IPV6_HDR_2, 1, context_id, 0)

		dl_buf_get_data(dram_offset, buf_handle)

		alu[tmp, 0xf, AND, hdr_offset]
		alu[--, tmp, -, PPP_HEADER_LENGTH]
		beq[flush_v6_hdr#]

		// packet was decapsulated
#ifdef PPP_2_BYTE_HEADER
	    _flush_v6_headers(dram_offset, hdr_offset, 6, req_sig)
#else
	    _flush_v6_headers(dram_offset, hdr_offset, 0, req_sig)
#endif
    	br[flush_v6_hdr_end#]

flush_v6_hdr#:
		// no tunneling
	    _flush_v6_headers(dram_offset, hdr_offset, PPP_HEADER_LENGTH, req_sig)

flush_v6_hdr_end#:
	    xbuf_deactivate(LM_IPV6_HDR_1)
        xbuf_deactivate(LM_IPV6_HDR_2)

	// packet was encapsulated
    // hdr_type == PPP_IPV6_IN_IPV4_TYPE
	.else

//        xbuf_activate(LM_IPV4_HDR, 0, context_id, 1)
        xbuf_activate(LM_IPV4_HDR, 0, context_id, 0)
        xbuf_activate(LM_IPV6_HDR_1, 1, context_id, 0)

		dl_buf_get_data(dram_offset, buf_handle)

        // Tunneled packet needs IPv4 and IPv6 header to be flushed.
        // Copy the IPv4 header to the transfer registers.

#ifdef PPP_2_BYTE_HEADER
        xbuf_copy($$iphdr, last_ipv4, 6, LM_IPV4_HDR, 0, 0,
                  PKT_HDR_BYTES_IPV4, 0)
#else
        xbuf_copy($$iphdr, last_ipv4, 0, LM_IPV4_HDR, 0, 0,
                  PKT_HDR_BYTES_IPV4, 0)
#endif

        // Copy as much as possible of the IPv6 header after the IPv4
	    // header.  The first 32 bytes are copies from the first part
		// of the IPv6 buffer, followed by as much as will fit from
		// the second.

        xbuf_activate(LM_IPV6_HDR_2, 0, context_id, 0)

        #define_eval IPV6_START_BYTE (PKT_HDR_BYTES_IPV4)
		#define_eval BYTES_TO_COPY   PKT_HDR_BYTES_IPV6_1

        xbuf_copy($$iphdr, last_ipv6, IPV6_START_BYTE, LM_IPV6_HDR_1,
		          0, last_ipv4, BYTES_TO_COPY, 0)

        #define_eval IPV6_START_BYTE (IPV6_START_BYTE + BYTES_TO_COPY)
	    #define_eval BYTES_TO_COPY (PKT_HDR_CACHE_BYTES - IPV6_START_BYTE)

        xbuf_copy($$iphdr, 0, IPV6_START_BYTE, LM_IPV6_HDR_2, 0,
		          last_ipv6, BYTES_TO_COPY, 0)

        #define_eval QW_COUNT  PKT_HDR_DRAM_RD_SIZE
	    dram[write, $$iphdr[0], dram_offset, hdr_offset, QW_COUNT],
	        sig_done[req_sig]

        xbuf_deactivate(LM_IPV6_HDR_1)
        xbuf_deactivate(LM_IPV6_HDR_2)

    .endif

#if (!streq('sig_action', 'SIG_NONE'))
    ctx_arb[sig_action]
#endif


_flush_headers_done#:
    #undef IPV6_START_BYTE
    #undef BYTES_TO_COPY
    #undef QW_COUNT

.end
#endm

#endif	//__PKTHDR_CACHE_UC__