/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      egress_init.c
 *
 * = DESCRIPTION
 *      This file contains scripts to initialize the IXP2800 Egress processor.
 *
 * = AUTHOR
 *      Ameya Varde
 *      ameya.s.varde@intel.com
 *
 * = CHANGE HISTORY
 *      03/11/2003 - Created 
 * ============================================================================
 */

#define IXP_SCRIPTS
#define	IXP_XSCALE

#include "xscale_common.h"
#ifdef __linux
#include "../egress_sphy_mphy4/wbench_project/dispatch_loop/dl_system.h"
#include "../../../../library/microblocks_library/include/dl_egress_system_default_2800.h"
#else
#include "..\egress_sphy_mphy4\wbench_project\dispatch_loop\dl_system.h"
#include "..\..\..\..\library\microblocks_library\include\dl_egress_system_default_2800.h"
#endif

/* Using init_sram() defined in ingress_init.c" */

/*----------------------
 *	CSIX RX
 *----------------------
 */
int csix_rx_ctx_init(void) {
	
	int finaladdr;

	printf("Initializing SRAM CSIX RX context area...\n");

	finaladdr = CSIX_RX_CTX_SRAM_BASE + CSIX_RX_CTX_TOTAL*CSIX_RX_CTX_SIZE;
	init_sram(0x0, CSIX_RX_CTX_SRAM_BASE, finaladdr);

	printf("Initialization complete.\n");

	return 1;
}


/*----------------------
 *	SCHEDULER
 *----------------------
 */
int sched_queue_struct_init(void) {
	int port, queue;
	int finaladdr, startaddr;
	int lword1, size;

	printf("Initializing SRAM Scheduler queue structures area...\n");

	/* each queue structure is 16 bytes */
	size = NUMBER_OF_QUEUES_PER_PORT * 16;

	/* lword0: current_credit_used
	 * lword1: last_round_scheduled_and_quantum_shift
	 * lword2: enqueued_packets_count
	 * lword3: dequeue_packets_count
	 */

	for(port=0; port<NUMBER_OF_PORTS; port++)
	{
		/* [15:0]  round number which has initial value of 1.
		 * [16:23] port number
		 * [31:24] n where 2^n is credit quantum per round
		 */
		lword1 = 0x0d000001 + (port << 16);
		
  		for(queue=0; queue<NUMBER_OF_QUEUES_PER_PORT; queue++)
		{
			startaddr = SCHED_QUEUE_STRUCTURES_BASE + (port * size) + (queue * 16);
			finaladdr = startaddr + 3;

			/* current credit used is always 0 to start with */
			init_sram(0x0, startaddr, finaladdr);

			startaddr = finaladdr + 1;
			finaladdr = startaddr + 3;

			/* adjust credit quantum */
			/* lword1 = lword1 + 0x01000000; */
			
			init_sram(lword1, startaddr, finaladdr);

			startaddr = finaladdr + 1;
			finaladdr = startaddr + 7;
			/* enqueued packets counter is always 0 at start */
			/* dequeued packets counter is always 0 at start */
			init_sram(0x0, startaddr, finaladdr);
		}
	}
	
	printf("Initialization complete.\n");

	return 0;
}


int sched_round_counters_init(void) {

	int finaladdr, cnt;

	printf("Initializing Scheduler counters area...\n");

	/* initialize 4K round counter for 16 PORT
	 * each counter is 4 bytes
	 */

	cnt = NUMBER_OF_PORTS * NUMBER_OF_ROUNDS_PER_PORT;

	finaladdr = SCHED_ROUND_COUNTERS_BASE + (cnt * 4);
	init_sram(0x0, SCHED_ROUND_COUNTERS_BASE, finaladdr);

	printf("Initialization complete.\n");

	return 0;
}


/*----------------------
 *	QM
 *----------------------
 */
int egress_qm_init(void) {
	
	int finaladdr;

	printf("Initializing SRAM queue descriptor area...\n");

	finaladdr = QD_SRAM_BASE + ((QD_TOTAL+2)*QD_SIZE);
	init_sram(0x0, QD_SRAM_BASE, finaladdr);

	printf("Initialization complete.\n");

	return 0;
}


void egress_system_setup(void)
{
	printf("\n******************************************************\n");
	printf("\n     System setup begin ...                           \n");
	printf("\n******************************************************\n");

	/*----------------------
	 *	CSIX RX
	 *----------------------
	 */

	/* Initialize the receive contexts in SRAM. */
	csix_rx_ctx_init();

	/*----------------------
	 *	Scheduler
	 *----------------------
	 */

	/* Initialize scheduler area in SRAM. */
	sched_queue_struct_init();
	sched_round_counters_init();

	/*----------------------
	 *	QM
	 *----------------------
	 */

	/* Initialize queue descriptor area in SRAM. */
	egress_qm_init();

	/*----------------------
	 *	Packet TX
	 *----------------------
	 */

	/*----------------------
	 *	COMMON
	 *----------------------
	 */

	printf("\n*********************************************************\n");
	printf("\n     System setup complete.                              \n");
	printf("\n*********************************************************\n");

}

/* Shorter function names */
void ess(void)
{	
    egress_system_setup();
}
