/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      egress_init.c
 *
 * = DESCRIPTION
 *      This file contains scripts to initialize the IXP2800 Egress processor.
 *
 * = AUTHOR
 *      Ameya Varde
 *      ameya.s.varde@intel.com
 *
 * = CHANGE HISTORY
 *      03/11/2003 - Created 
 * ============================================================================
 */

#define IXP_SCRIPTS
#define	IXP_XSCALE

#include "xscale_common.h"
#include "..\egress_sphy_mphy4\wbench_project\dispatch_loop\dl_system.h"
#include "..\..\..\..\library\microblocks_library\include\dl_egress_system_default_2800.h"

/* Using init_sram() defined in ingress_init.c" */

/*----------------------
 *	CSIX RX
 *----------------------
 */
int csix_rx_ctx_init() {
	
	int finaladdr;

	printf("Initializing SRAM CSIX RX context area...\n");

	finaladdr = CSIX_RX_CTX_SRAM_BASE + CSIX_RX_CTX_TOTAL*CSIX_RX_CTX_SIZE;
	init_sram(0x0, CSIX_RX_CTX_SRAM_BASE, finaladdr);

	printf("Initialization complete.\n");

	return 1;
}


/*----------------------
 *	QM
 *----------------------
 */
int egress_qm_init() {
	
	int finaladdr;

	printf("Initializing SRAM queue descriptor area...\n");

	finaladdr = QD_SRAM_BASE + ((QD_TOTAL+2)*QD_SIZE);
	init_sram(0x0, QD_SRAM_BASE, finaladdr);

	printf("Initialization complete.\n");

	return 0;
}


void egress_system_setup()
{
	printf("\n******************************************************\n");
	printf("\n     System setup begin ...                           \n");
	printf("\n******************************************************\n");

	/*----------------------
	 *	CSIX RX
	 *----------------------
	 */

	/* Initialize the receive contexts in SRAM. */
	csix_rx_ctx_init();

	/*----------------------
	 *	QM
	 *----------------------
	 */

	/* Initialize queue descriptor area in SRAM. */
	egress_qm_init();

	/*----------------------
	 *	Packet TX
	 *----------------------
	 */

	/*----------------------
	 *	COMMON
	 *----------------------
	 */

	printf("\n*********************************************************\n");
	printf("\n     System setup complete.                              \n");
	printf("\n*********************************************************\n");

}

/* Shorter function names */
void ess()
{	
    egress_system_setup();
}
