/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2004 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      mpls_nhlfe.c
 *
 * = DESCRIPTION
 *      This file contains scripts to initialize MPLS NHLFE tables.
 *
 * ============================================================================
 */
 
#define IXP_SCRIPTS
#define	IXP_XSCALE
#include "xscale_common.h"
#include "..\ingress\wbench_project\dispatch_loop\dl_system.h"
#include "..\..\..\..\library\microblocks_library\include\dl_ingress_system_default_2800.h"

#define	SRAM_WRITE(a,b)		set_sram((a),(b))
#define	SDRAM_WRITE(a,b)	set_sram((a),(b))

/* Flag Definitions */
#define MPLS_L3_IP_V4			    0x1
#define MPLS_NHLFE_VALID                    0x80
#define MPLS_NHLFE_SET                      0x40
#define MPLS_NHLFE_PIPE_TUNNEL              0x00
#define MPLS_NHLFE_UNIFORM_TUNNEL           0x10
#define MPLS_NHLFE_SHORT_PIPE_TUNNEL        0x20
#define MPLS_NHLFE_SHORT_PIPE_TUNNEL_PHP    0x30
#define MPLS_NHLFE_E_LSP                    0x08
#define MPLS_NHLFE_L_LSP                    0x0C
#define MPLS_NHLFE_TC_ENABLED               0x02
#define MPLS_NHLFE_PRECONF_EXP_MARKING      0x01

/* MPLS Operations */
#define MPLS_LABELOP_SWAP                   0x0
#define MPLS_LABELOP_SWAP_PUSH              0x1
#define MPLS_LABELOP_POP                    0x2
#define MPLS_LABELOP_POP_FORWARD            0x3

/* Label Space Initialization */
#ifdef PER_PLATFORM_LABEL_SPACE
    #define MIN_LABEL    0x0
    #define MAX_LABEL    0xFFFF
#endif

#ifdef PER_INTERFACE_LABEL_SPACE
    #define MIN_LABEL_0      0x0
    #define MAX_LABEL_0      0x7F
    #define ILM_BASE_PORT_0  0x0

    #define MIN_LABEL_1      0x80
    #define MAX_LABEL_1      0xFF
    #define ILM_BASE_PORT_1  (ILM_BASE_PORT_0 + ((MAX_LABEL_0 - MIN_LABEL_0 + 1) * 0x20))

    #define MIN_LABEL_2      0x100
    #define MAX_LABEL_2      0x17F
    #define ILM_BASE_PORT_2  (ILM_BASE_PORT_1 + ((MAX_LABEL_1 - MIN_LABEL_1 + 1) * 0x20))

    #define MIN_LABEL_3      0x180
    #define MAX_LABEL_3      0x1FF
    #define ILM_BASE_PORT_3  (ILM_BASE_PORT_2 + ((MAX_LABEL_2 - MIN_LABEL_2 + 1) * 0x20))

    #define MIN_LABEL_4      0x200
    #define MAX_LABEL_4      0x27F
    #define ILM_BASE_PORT_4  (ILM_BASE_PORT_3 + ((MAX_LABEL_3 - MIN_LABEL_3 + 1) * 0x20))

    #define MIN_LABEL_5      0x280
    #define MAX_LABEL_5      0x2FF
    #define ILM_BASE_PORT_5  (ILM_BASE_PORT_4 + ((MAX_LABEL_4 - MIN_LABEL_4 + 1) * 0x20))

    #define MIN_LABEL_6      0x300
    #define MAX_LABEL_6      0x37F
    #define ILM_BASE_PORT_6  (ILM_BASE_PORT_5 + ((MAX_LABEL_5 - MIN_LABEL_5 + 1) * 0x20))

    #define MIN_LABEL_7      0x380
    #define MAX_LABEL_7      0x3FF
    #define ILM_BASE_PORT_7  (ILM_BASE_PORT_6 + ((MAX_LABEL_6 - MIN_LABEL_6 + 1) * 0x20))

    #define MIN_LABEL_8      0x400
    #define MAX_LABEL_8      0x47F
    #define ILM_BASE_PORT_8  (ILM_BASE_PORT_7 + ((MAX_LABEL_7 - MIN_LABEL_7 + 1) * 0x20))

    #define MIN_LABEL_9      0x480
    #define MAX_LABEL_9      0x4FF
    #define ILM_BASE_PORT_9  (ILM_BASE_PORT_8 + ((MAX_LABEL_8 - MIN_LABEL_8 + 1) * 0x20))

    #define MIN_LABEL_10     0x500
    #define MAX_LABEL_10     0x57F
    #define ILM_BASE_PORT_10 (ILM_BASE_PORT_9 + ((MAX_LABEL_9 - MIN_LABEL_9 + 1) * 0x20))

    #define MIN_LABEL_11     0x580
    #define MAX_LABEL_11     0x5FF
    #define ILM_BASE_PORT_11 (ILM_BASE_PORT_10 + ((MAX_LABEL_10 - MIN_LABEL_10 + 1) * 0x20))

    #define MIN_LABEL_12     0x600
    #define MAX_LABEL_12     0x67F
    #define ILM_BASE_PORT_12 (ILM_BASE_PORT_11 + ((MAX_LABEL_11 - MIN_LABEL_11 + 1) * 0x20))

    #define MIN_LABEL_13     0x680
    #define MAX_LABEL_13     0x6FF
    #define ILM_BASE_PORT_13 (ILM_BASE_PORT_12 + ((MAX_LABEL_12 - MIN_LABEL_12 + 1) * 0x20))

    #define MIN_LABEL_14     0x700
    #define MAX_LABEL_14     0x77F
    #define ILM_BASE_PORT_14 (ILM_BASE_PORT_13 + ((MAX_LABEL_13 - MIN_LABEL_13 + 1) * 0x20))

    #define MIN_LABEL_15     0x780
    #define MAX_LABEL_15     0x7FF
    #define ILM_BASE_PORT_15 (ILM_BASE_PORT_14 + ((MAX_LABEL_14 - MIN_LABEL_14 + 1) * 0x20))

#endif


/* 
 * ---------------------------------------------------------------------------
 * Function init_ftn
 * This function initializes FTN NHLFE table
 * ---------------------------------------------------------------------------
 */
int init_ftn(){

#define MPLS_FTN_ENTRIES_NO 16
#define MPLS_FTN_ENTRY_SIZE 0x20

int start = MPLS_FTN_TABLE_BASE;
int end;

    end = start + (MPLS_FTN_ENTRIES_NO) * (MPLS_FTN_ENTRY_SIZE);
    init_sram(0x0, start, end);

#undef MPLS_FTN_ENTRIES_NO
#undef MPLS_FTN_ENTRY_SIZE

    return 1;
}

/* 
 * ---------------------------------------------------------------------------
 * Function init_ilm
 * This function initializes ILM NHLFE table.
 * ---------------------------------------------------------------------------
 */
int init_ilm(){

#define MPLS_ILM_ENTRY_SIZE 0x20

int start = MPLS_ILM_TABLE_BASE;
int end;

#ifdef PER_PLATFORM_LABEL_SPACE
    end = start + (16 + MAX_LABEL - MIN_LABEL) * (MPLS_ILM_ENTRY_SIZE);
#endif

#ifdef PER_INTERFACE_LABEL_SPACE
    end = start + (16 + (MAX_LABEL_0 - MIN_LABEL_0 + 1) + \
                        (MAX_LABEL_1 - MIN_LABEL_1 + 1) + \
                        (MAX_LABEL_2 - MIN_LABEL_2 + 1) + \
                        (MAX_LABEL_3 - MIN_LABEL_3 + 1) + \
                        (MAX_LABEL_4 - MIN_LABEL_4 + 1) + \
                        (MAX_LABEL_5 - MIN_LABEL_5 + 1) + \
                        (MAX_LABEL_6 - MIN_LABEL_6 + 1) + \
                        (MAX_LABEL_7 - MIN_LABEL_7 + 1) + \
                        (MAX_LABEL_8 - MIN_LABEL_8 + 1) + \
                        (MAX_LABEL_9 - MIN_LABEL_9 + 1) + \
                        (MAX_LABEL_10 - MIN_LABEL_10 + 1) + \
                        (MAX_LABEL_11 - MIN_LABEL_11 + 1) + \
                        (MAX_LABEL_12 - MIN_LABEL_12 + 1) + \
                        (MAX_LABEL_13 - MIN_LABEL_13 + 1) + \
                        (MAX_LABEL_14 - MIN_LABEL_14 + 1) + \
                        (MAX_LABEL_15 - MIN_LABEL_15 + 1)) * (MPLS_ILM_ENTRY_SIZE);
#endif

    init_sram(0x0, start, end);

    return 1;
}

/* 
 * ---------------------------------------------------------------------------
 * Function set_mpls_param_table
 * This function initializes ILM NHLFE Parameters table.
 * ---------------------------------------------------------------------------
 */
int set_mpls_param_table(){
int offset;

#ifdef PER_PLATFORM_LABEL_SPACE
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + 0x0, MIN_LABEL);
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + 0x4, MAX_LABEL);
#endif

#ifdef PER_INTERFACE_LABEL_SPACE
offset = 0;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MIN_LABEL_0);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MAX_LABEL_0);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, ILM_BASE_PORT_0);

offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MIN_LABEL_1);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MAX_LABEL_1);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, ILM_BASE_PORT_1);

offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MIN_LABEL_2);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MAX_LABEL_2);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, ILM_BASE_PORT_2);

offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MIN_LABEL_3);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MAX_LABEL_3);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, ILM_BASE_PORT_3);

offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MIN_LABEL_4);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MAX_LABEL_4);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, ILM_BASE_PORT_4);

offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MIN_LABEL_5);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MAX_LABEL_5);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, ILM_BASE_PORT_5);

offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MIN_LABEL_6);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MAX_LABEL_6);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, ILM_BASE_PORT_6);

offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MIN_LABEL_7);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MAX_LABEL_7);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, ILM_BASE_PORT_7);

offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MIN_LABEL_8);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MAX_LABEL_8);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, ILM_BASE_PORT_8);

offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MIN_LABEL_9);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MAX_LABEL_9);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, ILM_BASE_PORT_9);

offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MIN_LABEL_10);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MAX_LABEL_10);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, ILM_BASE_PORT_10);

offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MIN_LABEL_11);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MAX_LABEL_11);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, ILM_BASE_PORT_11);

offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MIN_LABEL_12);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MAX_LABEL_12);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, ILM_BASE_PORT_12);

offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MIN_LABEL_13);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MAX_LABEL_13);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, ILM_BASE_PORT_13);

offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MIN_LABEL_14);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MAX_LABEL_14);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, ILM_BASE_PORT_14);

offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MIN_LABEL_15);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, MAX_LABEL_15);
offset += 4;
    set_sram(MPLS_ILM_PARAM_TABLE_BASE + offset, ILM_BASE_PORT_15);

#endif

    return 1;
}

/* 
 * ---------------------------------------------------------------------------
 * Function addNhlfEntry
 * This function prepares one NHLF entry.
 * ---------------------------------------------------------------------------
 */

int addNhlfEntry(int address, int offset, \
				 int flags, int operation, int pushCount, int l3Protocol, int nextHopID, \
				 int maxLabPktSize, int outPortID, \
				 int topLabel, int outPortType, int ttl, \
				 int label1, int bladeID, \
				 int label2, int flowIdHigh, \
				 int label3, int flowIdLow, \
				 int exp2phb, int phb2exp){


	set_sram(address + offset + 0x0, (flags << 24) | (operation << 22) | \
				(pushCount << 19) | (l3Protocol << 16) | nextHopID);
	set_sram(address + offset + 0x4, (maxLabPktSize << 16) | outPortID); 
	set_sram(address + offset + 0x8, (topLabel << 12) | (outPortType << 4) | ttl);
	set_sram(address + offset + 0xC, (label1 << 12) | bladeID);
	set_sram(address + offset + 0x10, (label2 << 12) | flowIdHigh);
	set_sram(address + offset + 0x14, (label3 << 12) | flowIdLow); 
	set_sram(address + offset + 0x18, (exp2phb) | phb2exp); 
	set_sram(address + offset + 0x1C, 0);

	return 1;
}

/* 
 * ---------------------------------------------------------------------------
 * Function addNhlfeSetEntry
 * This function prepares one NHLFE Set entry.
 * ---------------------------------------------------------------------------
 */

int addNhlfeSetEntry(int address, int offset, int exp2phb, int phbMask1, \
			int phbMask2, int phbMask3, int phbMask4, int outSegID1, \
			int outSegID2, int outSegID3, int outSegID4) {

int id2_high, id2_low, id4_high, id4_low;

	id2_high = (outSegID2 >> 8) & 0xFFF;
	id2_low	 = outSegID2 & 0xFF;
	id4_high = (outSegID4 >> 16) & 0xF;
	id4_low  = outSegID4 & 0xFFFF;

	set_sram(address + offset + 0x0, (3 << 30) | (exp2phb << 8));
	set_sram(address + offset + 0x4, phbMask1);
	set_sram(address + offset + 0x8, phbMask2);
	set_sram(address + offset + 0xC, phbMask3);
	set_sram(address + offset + 0x10, phbMask4);
	set_sram(address + offset + 0x14, (outSegID1 << 12) | id2_high);
	set_sram(address + offset + 0x18, (id2_low << 24) | (outSegID3 << 4) | id4_high);
	set_sram(address + offset + 0x1C, (id4_low << 16));

	return 1;

/*
 *
	Example of NHLFE set init:
	--------------------------

Use addNhlfEntry to set up entry <ENTRY> in MPLS_ILM_NHLFE_SET_TABLE_BASE

address 	= MPLS_ILM_TABLE_BASE;
offset  	= 0 * 0x20;
exp2phb 	= 0; (not used)
phbMask1	= 0; (not used)
phbMask2	= 0; (not used)
phbMask3	= 0; (not used)
phbMask4	= 0; (not used)
outSegID1	= <ENTRY>;
outSegID2	= 0; (not used)
outSegID3	= 0; (not used)
outSegID4	= 0; (not used)

addNhlfeSetEntry(int address, int offset, int exp2phb, int phbMask1, \
		int phbMask2, int phbMask3, int phbMask4, int outSegID1, \
		int outSegID2, int outSegID3, int outSegID4)
 *
 */

}


/* 
 * ---------------------------------------------------------------------------
 * Function mpls_init
 * This function prepares MPLS forwarding tables.
 * ---------------------------------------------------------------------------
 */

int mpls_init() {

/* Declare variables. */

int address, offset;

int flags, operation, pushCount, l3Protocol, nextHopID;
int maxLabPktSize, outPortID;
int topLabel, outPortType, ttl;
int label1, bladeID;
int label2;
int label3;


/*--------------------------------------------------------------------------------*/

/* Initialize FTN and ILM NHLFEs */

init_ftn();
init_ilm();

/* Set ILM NHLFE Parameters Table */
set_mpls_param_table();

/*--------------------------------------------------------------------------------*/

/* Set up entries for FTN tests */
/* ============================ */

/* FTN entry 0 */
address = MPLS_FTN_TABLE_BASE;
offset = 0 * 0x20;

flags = (MPLS_NHLFE_VALID);
operation = 0;
pushCount = 1;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x11;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0xA0A0A;
outPortType = 0;
ttl = 1;

label1 = 0xB0B0B;
bladeID = 1;

label2 = 0xC0C0C;

label3 = 0xD0D0D;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* FTN entry 1 */
address = MPLS_FTN_TABLE_BASE;
offset = 1 * 0x20;

flags = (MPLS_NHLFE_VALID);
operation = 0;
pushCount = 1;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x12;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0xA1A1A;
outPortType = 0;
ttl = 1;

label1 = 0xB1B1B;
bladeID = 2;

label2 = 0xC1C1C;

label3 = 0xD1D1D;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* FTN entry 2 */
address = MPLS_FTN_TABLE_BASE;
offset = 2 * 0x20;

flags = (MPLS_NHLFE_VALID);
operation = 0;
pushCount = 4;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x13;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0xA2A2A;
outPortType = 0;
ttl = 1;

label1 = 0xB2B2B;
bladeID = 3;

label2 = 0xC2C2C;

label3 = 0xD2D2D;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* FTN entry 3 */
address = MPLS_FTN_TABLE_BASE;
offset = 3 * 0x20;

flags = (MPLS_NHLFE_VALID);
operation = 0;
pushCount = 4;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x14;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0xA3A3A;
outPortType = 0;
ttl = 1;

label1 = 0xB3B3B;
bladeID = 4;

label2 = 0xC3C3C;

label3 = 0xD3D3D;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* FTN entry 4 */
address = MPLS_FTN_TABLE_BASE;
offset = 4 * 0x20;

flags = (MPLS_NHLFE_VALID);
operation = 0;
pushCount = 1;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x15;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0xA4A4A;
outPortType = 0;
ttl = 1;

label1 = 0xB4B4B;
bladeID = 5;

label2 = 0xC4C4C;

label3 = 0xD4D4D;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* FTN entry 5 */
address = MPLS_FTN_TABLE_BASE;
offset = 5 * 0x20;

flags = (MPLS_NHLFE_VALID);
operation = 0;
pushCount = 1;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x16;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0xA5A5A;
outPortType = 0;
ttl = 1;

label1 = 0xB5B5B;
bladeID = 6;

label2 = 0xC5C5C;

label3 = 0xD5D5D;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* FTN entry 6 */
address = MPLS_FTN_TABLE_BASE;
offset = 6 * 0x20;

flags = (MPLS_NHLFE_VALID);
operation = 0;
pushCount = 1;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x17;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0xA6A6A;
outPortType = 0;
ttl = 1;

label1 = 0xB6B6B;
bladeID = 7;

label2 = 0xC6C6C;

label3 = 0xD6D6D;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* FTN entry 7 */
address = MPLS_FTN_TABLE_BASE;
offset = 7 * 0x20;

flags = (MPLS_NHLFE_VALID);
operation = 0;
pushCount = 1;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x18;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0xA7A7A;
outPortType = 0;
ttl = 1;

label1 = 0xB7B7B;
bladeID = 8;

label2 = 0xC7C7C;

label3 = 0xD7D7D;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* FTN entry 8 */
address = MPLS_FTN_TABLE_BASE;
offset = 8 * 0x20;

flags = (MPLS_NHLFE_VALID);
operation = 0;
pushCount = 1;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x19;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0xA8A8A;
outPortType = 0;
ttl = 1;

label1 = 0xB8B8B;
bladeID = 9;

label2 = 0xC8C8C;

label3 = 0xD8D8D;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* FTN entry 9 */
address = MPLS_FTN_TABLE_BASE;
offset = 9 * 0x20;

flags = (MPLS_NHLFE_VALID);
operation = 0;
pushCount = 1;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x1a;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0xA9A9A;
outPortType = 0;
ttl = 1;

label1 = 0xB9B9B;
bladeID = 0xa;

label2 = 0xC9C9C;

label3 = 0xD9D9D;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* Set up entries for ILM tests */
/* ============================ */

/* ILM entry 0x14 */
address = MPLS_ILM_TABLE_BASE;
offset = 0x14 * 0x20;

flags = (MPLS_NHLFE_VALID | MPLS_NHLFE_SHORT_PIPE_TUNNEL_PHP);
operation = MPLS_LABELOP_POP;
pushCount = 1;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x0;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0x14aaa;
outPortType = 0;
ttl = 1;

label1 = 0x14bbb;
bladeID = 1;

label2 = 0x14ccc;

label3 = 0x14ddd;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* ILM entry 0x15 */
address = MPLS_ILM_TABLE_BASE;
offset = 0x15 * 0x20;

flags = (MPLS_NHLFE_VALID | MPLS_NHLFE_SHORT_PIPE_TUNNEL_PHP);
operation = MPLS_LABELOP_POP;
pushCount = 1;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x1;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0x15aaa;
outPortType = 0;
ttl = 1;

label1 = 0x15bbb;
bladeID = 2;

label2 = 0x15ccc;

label3 = 0x15ddd;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* ILM entry 0x16 */
address = MPLS_ILM_TABLE_BASE;
offset = 0x16 * 0x20;

flags = (MPLS_NHLFE_VALID | MPLS_NHLFE_SHORT_PIPE_TUNNEL_PHP);
operation = MPLS_LABELOP_POP_FORWARD;
pushCount = 0;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x2;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0x16aaa;
outPortType = 0;
ttl = 1;

label1 = 0x16bbb;
bladeID = 3;

label2 = 0x16ccc;

label3 = 0x16ddd;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* ILM entry 0x17 */
address = MPLS_ILM_TABLE_BASE;
offset = 0x17 * 0x20;

flags = (MPLS_NHLFE_VALID | MPLS_NHLFE_SHORT_PIPE_TUNNEL_PHP);
operation = MPLS_LABELOP_POP;
pushCount = 0;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x3;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0x17aaa;
outPortType = 0;
ttl = 1;

label1 = 0x17bbb;
bladeID = 4;

label2 = 0x17ccc;

label3 = 0x17ddd;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* ILM entry 0x18 */
address = MPLS_ILM_TABLE_BASE;
offset = 0x18 * 0x20;

flags = (MPLS_NHLFE_VALID | MPLS_NHLFE_SHORT_PIPE_TUNNEL_PHP);
operation = MPLS_LABELOP_POP;
pushCount = 1;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x4;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0x18aaa;
outPortType = 0;
ttl = 1;

label1 = 0x18bbb;
bladeID = 5;

label2 = 0x18ccc;

label3 = 0x18ddd;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* ILM entry 0x19 */
address = MPLS_ILM_TABLE_BASE;
offset = 0x19 * 0x20;

flags = (MPLS_NHLFE_VALID | MPLS_NHLFE_SHORT_PIPE_TUNNEL_PHP);
operation = MPLS_LABELOP_POP_FORWARD;
pushCount = 3;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x5;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0x19aaa;
outPortType = 0;
ttl = 1;

label1 = 0x19bbb;
bladeID = 6;

label2 = 0x19ccc;

label3 = 0x19ddd;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* ILM entry 0x1a */
address = MPLS_ILM_TABLE_BASE;
offset = 0x1a * 0x20;

flags = (MPLS_NHLFE_VALID | MPLS_NHLFE_SHORT_PIPE_TUNNEL_PHP);
operation = MPLS_LABELOP_POP;
pushCount = 3;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x5;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0x1aaaa;
outPortType = 0;
ttl = 1;

label1 = 0x1abbb;
bladeID = 6;

label2 = 0x1accc;

label3 = 0x1addd;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* ILM entry 0x1b */
address = MPLS_ILM_TABLE_BASE;
offset = 0x1b * 0x20;

flags = (MPLS_NHLFE_VALID | MPLS_NHLFE_SHORT_PIPE_TUNNEL_PHP);
operation = MPLS_LABELOP_POP_FORWARD;
pushCount = 3;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x6;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0x1baaa;
outPortType = 0;
ttl = 1;

label1 = 0x1bbbb;
bladeID = 7;

label2 = 0x1bccc;

label3 = 0x1bddd;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);


/* ILM entry 0x1c */
address = MPLS_ILM_TABLE_BASE;
offset = 0x1c * 0x20;

flags = (MPLS_NHLFE_VALID | MPLS_NHLFE_SHORT_PIPE_TUNNEL);
operation = MPLS_LABELOP_SWAP;
pushCount = 1;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x7;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0x1caaa;
outPortType = 0;
ttl = 1;

label1 = 0x1cbbb;
bladeID = 8;

label2 = 0x1cccc;

label3 = 0x1cddd;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* ILM entry 0x1d */
address = MPLS_ILM_TABLE_BASE;
offset = 0x1d * 0x20;

flags = (MPLS_NHLFE_VALID | MPLS_NHLFE_SHORT_PIPE_TUNNEL);
operation = MPLS_LABELOP_SWAP;
pushCount = 1;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x8;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0x1daaa;
outPortType = 0;
ttl = 1;

label1 = 0x1dbbb;
bladeID = 9;

label2 = 0x1dccc;

label3 = 0x1dddd;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* ILM entry 0x1e */
address = MPLS_ILM_TABLE_BASE;
offset = 0x1e * 0x20;

flags = (MPLS_NHLFE_VALID);
operation = MPLS_LABELOP_SWAP_PUSH;
pushCount = 1;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x7;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0x1eaaa;
outPortType = 0;
ttl = 1;

label1 = 0x1ebbb;
bladeID = 8;

label2 = 0x1eccc;

label3 = 0x1eddd;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* ILM entry 0x1f */
address = MPLS_ILM_TABLE_BASE;
offset = 0x1f * 0x20;

flags = (MPLS_NHLFE_VALID);
operation = MPLS_LABELOP_SWAP_PUSH;
pushCount = 1;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x8;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0x1faaa;
outPortType = 0;
ttl = 1;

label1 = 0x1fbbb;
bladeID = 9;

label2 = 0x1fccc;

label3 = 0x1fddd;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* ILM entry 0x20 */
address = MPLS_ILM_TABLE_BASE;
offset = 0x20 * 0x20;

flags = (MPLS_NHLFE_VALID);
operation = MPLS_LABELOP_SWAP_PUSH;
pushCount = 1;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0x9;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0x20aaa;
outPortType = 0;
ttl = 1;

label1 = 0x20bbb;
bladeID = 1;

label2 = 0x20ccc;

label3 = 0x20ddd;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

/* ILM entry 0x21 */
address = MPLS_ILM_TABLE_BASE;
offset = 0x21 * 0x20;

flags = (MPLS_NHLFE_VALID);
operation = MPLS_LABELOP_SWAP_PUSH;
pushCount = 1;
l3Protocol = MPLS_L3_IP_V4;
nextHopID = 0xa;

maxLabPktSize = 9216;
outPortID = 0;

topLabel = 0x21aaa;
outPortType = 0;
ttl = 1;

label1 = 0x21bbb;
bladeID = 2;

label2 = 0x21ccc;

label3 = 0x21ddd;

addNhlfEntry(address, offset, \
             flags, operation, pushCount, l3Protocol, nextHopID, \
             maxLabPktSize, outPortID, \
             topLabel, outPortType, ttl, \
             label1, bladeID, \
             label2, 0, \
             label3, 0, \
             0, 0);

	return 0;
}
