/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      rtm_routes.c
 *
 * = DESCRIPTION
 *      This file contains scripts to initialize the IPv4 route table.
 *
 * = AUTHOR
 *      Dennis D. Tran
 *      dennis.dat.tran@intel.com
 *
 * = CHANGE HISTORY
 *		11/21/2003 - Created
 * ============================================================================
 */

#define IXP_SCRIPTS
#define	IXP_XSCALE
#include "xscale_common.h"
#include "..\ingress\wbench_project\dispatch_loop\dl_system.h"
#include "..\..\..\..\library\microblocks_library\include\dl_ingress_system_default_2800.h"

#define	SRAM_WRITE(a,b)		set_sram((a),(b))
#define	SDRAM_WRITE(a,b)	set_sram((a),(b))

int l3_create_routes() {

int trie_base = TRIE_TABLE_SRAM_BASE;

int nh_base = NEXTHOP_TABLE_SRAM_BASE;

/* NH is 0xffff, blade is 0x0, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x0,0xffff0000);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x4,0x23e80000);

/* #
 */ /* #	This file contains the routes to be added for running the unit tests
 */ /* #	
 */ /* #	The syntax is understood by RTM executable which is used to generate
 */ /* #	the rtm_routes.ind file.
 */ /* #
 */ /* #	The syntax for the add command is as follows.
 */ /* #
 */ /* #	add ip-addr netmask nexthopid flags mtu outport
 */ /* #
 */ /* #	ip-addr 	: IP address in A.B.C.D format
 */ /* #	netmask		: Netmask in A.B.C.D format
 */ /* #	nexthopid	: Nexthop ID. Only least significant 16 bits taken
 */ /* #	flags		: Normally, ZERO. See IPV4 LLD for more options
 */ /* #	mtu		: Max tramission unit
 */ /* #	outport		: Encoded value as follows
 */ /* #				bladeid  	: bits (31:24)
 */ /* #				nexthopIdType	: bits (15:8)
 */ /* #				outport  	: bits (7:0)
 */ /* #
 */ /* #	How to run the rtm.exe
 */ /* #	----------------------
 */ /* #	At command prompt, run 'rtm' on the routes.txt file
 */ /* #
 */ /* #	prompt:> rtm routes.txt
 */ /* #
 */ /* #	Then, give a file name for the script to be generated, say 
 */ /* #	rtm_routes.ind. Then replace this file in the Ingress project.
 */ /* 
 *//* #-------------------------------------------------
 */ /* # normal testing - 16 bit routes
 */ /* #-------------------------------------------------
 */ /* 
 *//* add 16.0.0.0 255.255.0.0 0xdead0000 0x0 9192 0x01000000
 */
/* @@@ Adding dest=0x10000000, mask = 0xffff0000, blade_nh_id=0x1000000,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40480,0x10000);

/* NH is 0x0, blade is 0x1, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x8,0x100);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0xc,0x23e80000);


	SRAM_WRITE(trie_base + 0x40440,0x2);


	SRAM_WRITE(trie_base + 0x40040,0x1);

/* add 16.1.0.0 255.255.0.0 0xdead0001 0x0 9192 0x02000000
 */
/* @@@ Adding dest=0x10010000, mask = 0xffff0000, blade_nh_id=0x2000001,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40484,0x20000);

/* NH is 0x1, blade is 0x2, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x10,0x10200);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x14,0x23e80000);

/* add 16.2.0.0 255.255.0.0 0xdead0002 0x0 9192 0x03000000
 */
/* @@@ Adding dest=0x10020000, mask = 0xffff0000, blade_nh_id=0x3000002,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40488,0x30000);

/* NH is 0x2, blade is 0x3, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x18,0x20300);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1c,0x23e80000);

/* add 16.3.0.0 255.255.0.0 0xdead0003 0x0 9192 0x04000000
 */
/* @@@ Adding dest=0x10030000, mask = 0xffff0000, blade_nh_id=0x4000003,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x4048c,0x40000);

/* NH is 0x3, blade is 0x4, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x20,0x30400);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x24,0x23e80000);

/* 
 *//* #-------------------------------------------------
 */ /* # normal testing - 24 bit routes
 */ /* #-------------------------------------------------
 */ /* 
 *//* add 124.0.0.0 255.255.255.0 0xdead0000 0x0 9192 0x01000000
 */
/* @@@ Adding dest=0x7c000000, mask = 0xffffff00, blade_nh_id=0x1000000,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40500,0x50000);

/* NH is 0x0, blade is 0x1, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x28,0x100);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x2c,0x23e80000);


	SRAM_WRITE(trie_base + 0x404c0,0x4);


	SRAM_WRITE(trie_base + 0x1f000,0x3);

/* add 124.0.1.0 255.255.255.0 0xdead0001 0x0 9192 0x02000000
 */
/* @@@ Adding dest=0x7c000100, mask = 0xffffff00, blade_nh_id=0x2000001,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40504,0x60000);

/* NH is 0x1, blade is 0x2, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x30,0x10200);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x34,0x23e80000);

/* add 124.0.2.0 255.255.255.0 0xdead0002 0x0 9192 0x03000000
 */
/* @@@ Adding dest=0x7c000200, mask = 0xffffff00, blade_nh_id=0x3000002,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40508,0x70000);

/* NH is 0x2, blade is 0x3, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x38,0x20300);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x3c,0x23e80000);

/* add 124.0.3.0 255.255.255.0 0xdead0003 0x0 9192 0x04000000
 */
/* @@@ Adding dest=0x7c000300, mask = 0xffffff00, blade_nh_id=0x4000003,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x4050c,0x80000);

/* NH is 0x3, blade is 0x4, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x40,0x30400);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x44,0x23e80000);

/* add 124.0.4.0 255.255.255.0 0xdead0004 0x0 9192 0x05000000
 */
/* @@@ Adding dest=0x7c000400, mask = 0xffffff00, blade_nh_id=0x5000004,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40510,0x90000);

/* NH is 0x4, blade is 0x5, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x48,0x40500);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x4c,0x23e80000);

/* add 124.0.5.0 255.255.255.0 0xdead0005 0x0 9192 0x06000000
 */
/* @@@ Adding dest=0x7c000500, mask = 0xffffff00, blade_nh_id=0x6000005,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40514,0xa0000);

/* NH is 0x5, blade is 0x6, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x50,0x50600);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x54,0x23e80000);

/* add 124.0.6.0 255.255.255.0 0xdead0006 0x0 9192 0x07000000
 */
/* @@@ Adding dest=0x7c000600, mask = 0xffffff00, blade_nh_id=0x7000006,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40518,0xb0000);

/* NH is 0x6, blade is 0x7, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x58,0x60700);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x5c,0x23e80000);

/* add 124.0.7.0 255.255.255.0 0xdead0007 0x0 9192 0x08000000
 */
/* @@@ Adding dest=0x7c000700, mask = 0xffffff00, blade_nh_id=0x8000007,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x4051c,0xc0000);

/* NH is 0x7, blade is 0x8, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x60,0x70800);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x64,0x23e80000);

/* add 124.0.8.0 255.255.255.0 0xdead0008 0x0 9192 0x09000000
 */
/* @@@ Adding dest=0x7c000800, mask = 0xffffff00, blade_nh_id=0x9000008,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40520,0xd0000);

/* NH is 0x8, blade is 0x9, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x68,0x80900);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x6c,0x23e80000);

/* add 124.0.9.0 255.255.255.0 0xdead0009 0x0 9192 0x0a000000
 */
/* @@@ Adding dest=0x7c000900, mask = 0xffffff00, blade_nh_id=0xa000009,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40524,0xe0000);

/* NH is 0x9, blade is 0xa, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x70,0x90a00);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x74,0x23e80000);

/* add 124.0.10.0 255.255.255.0 0xdead000a 0x0 9192 0x0b000000
 */
/* @@@ Adding dest=0x7c000a00, mask = 0xffffff00, blade_nh_id=0xb00000a,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40528,0xf0000);

/* NH is 0xa, blade is 0xb, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x78,0xa0b00);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x7c,0x23e80000);

/* add 124.0.11.0 255.255.255.0 0xdead000b 0x0 9192 0x0c000000
 */
/* @@@ Adding dest=0x7c000b00, mask = 0xffffff00, blade_nh_id=0xc00000b,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x4052c,0x100000);

/* NH is 0xb, blade is 0xc, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x80,0xb0c00);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x84,0x23e80000);

/* add 124.0.12.0 255.255.255.0 0xdead000c 0x0 9192 0x0d000000
 */
/* @@@ Adding dest=0x7c000c00, mask = 0xffffff00, blade_nh_id=0xd00000c,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40530,0x110000);

/* NH is 0xc, blade is 0xd, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x88,0xc0d00);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x8c,0x23e80000);

/* add 124.0.13.0 255.255.255.0 0xdead000d 0x0 9192 0x0e000000
 */
/* @@@ Adding dest=0x7c000d00, mask = 0xffffff00, blade_nh_id=0xe00000d,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40534,0x120000);

/* NH is 0xd, blade is 0xe, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x90,0xd0e00);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x94,0x23e80000);

/* add 124.0.14.0 255.255.255.0 0xdead000e 0x0 9192 0x01000000
 */
/* @@@ Adding dest=0x7c000e00, mask = 0xffffff00, blade_nh_id=0x100000e,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40538,0x130000);

/* NH is 0xe, blade is 0x1, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x98,0xe0100);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x9c,0x23e80000);

/* add 124.0.15.0 255.255.255.0 0xdead000f 0x0 9192 0x02000000
 */
/* @@@ Adding dest=0x7c000f00, mask = 0xffffff00, blade_nh_id=0x200000f,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x4053c,0x140000);

/* NH is 0xf, blade is 0x2, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0xa0,0xf0200);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0xa4,0x23e80000);

/* 
 *//* #-------------------------------------------------
 */ /* # more entries: 16 - 31
 */ /* #-------------------------------------------------
 */ /* 
 *//* add 124.0.16.0 255.255.255.0 0xdead0010 0x0 9192 0x03000000
 */
/* @@@ Adding dest=0x7c001000, mask = 0xffffff00, blade_nh_id=0x3000010,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40540,0x150000);

/* NH is 0x10, blade is 0x3, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0xa8,0x100300);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0xac,0x23e80000);


	SRAM_WRITE(trie_base + 0x404c4,0x5);

/* add 124.0.17.0 255.255.255.0 0xdead0011 0x0 9192 0x04000000
 */
/* @@@ Adding dest=0x7c001100, mask = 0xffffff00, blade_nh_id=0x4000011,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40544,0x160000);

/* NH is 0x11, blade is 0x4, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0xb0,0x110400);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0xb4,0x23e80000);

/* add 124.0.18.0 255.255.255.0 0xdead0012 0x0 9192 0x05000000
 */
/* @@@ Adding dest=0x7c001200, mask = 0xffffff00, blade_nh_id=0x5000012,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40548,0x170000);

/* NH is 0x12, blade is 0x5, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0xb8,0x120500);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0xbc,0x23e80000);

/* add 124.0.19.0 255.255.255.0 0xdead0013 0x0 9192 0x06000000
 */
/* @@@ Adding dest=0x7c001300, mask = 0xffffff00, blade_nh_id=0x6000013,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x4054c,0x180000);

/* NH is 0x13, blade is 0x6, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0xc0,0x130600);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0xc4,0x23e80000);

/* add 124.0.20.0 255.255.255.0 0xdead0014 0x0 9192 0x07000200
 */
/* @@@ Adding dest=0x7c001400, mask = 0xffffff00, blade_nh_id=0x7000014,mtu_port=0x23e80200,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40550,0x190000);

/* NH is 0x14, blade is 0x7, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0xc8,0x140700);

/* MTU is 9192, NH ID type is 2, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0xcc,0x23e80200);

/* add 124.0.21.0 255.255.255.0 0xdead0015 0x0 9192 0x08000200
 */
/* @@@ Adding dest=0x7c001500, mask = 0xffffff00, blade_nh_id=0x8000015,mtu_port=0x23e80200,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40554,0x1a0000);

/* NH is 0x15, blade is 0x8, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0xd0,0x150800);

/* MTU is 9192, NH ID type is 2, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0xd4,0x23e80200);

/* add 124.0.22.0 255.255.255.0 0xdead0016 0x0 9192 0x09000200
 */
/* @@@ Adding dest=0x7c001600, mask = 0xffffff00, blade_nh_id=0x9000016,mtu_port=0x23e80200,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40558,0x1b0000);

/* NH is 0x16, blade is 0x9, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0xd8,0x160900);

/* MTU is 9192, NH ID type is 2, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0xdc,0x23e80200);

/* add 124.0.23.0 255.255.255.0 0xdead0017 0x0 9192 0x0a000200
 */
/* @@@ Adding dest=0x7c001700, mask = 0xffffff00, blade_nh_id=0xa000017,mtu_port=0x23e80200,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x4055c,0x1c0000);

/* NH is 0x17, blade is 0xa, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0xe0,0x170a00);

/* MTU is 9192, NH ID type is 2, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0xe4,0x23e80200);

/* add 124.0.24.0 255.255.255.0 0xdead0000 0x0 9192 0x0b000200
 */
/* @@@ Adding dest=0x7c001800, mask = 0xffffff00, blade_nh_id=0xb000000,mtu_port=0x23e80200,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40560,0x1d0000);

/* NH is 0x0, blade is 0xb, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0xe8,0xb00);

/* MTU is 9192, NH ID type is 2, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0xec,0x23e80200);

/* add 124.0.25.0 255.255.255.0 0xdead0001 0x0 9192 0x0c000200
 */
/* @@@ Adding dest=0x7c001900, mask = 0xffffff00, blade_nh_id=0xc000001,mtu_port=0x23e80200,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40564,0x1e0000);

/* NH is 0x1, blade is 0xc, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0xf0,0x10c00);

/* MTU is 9192, NH ID type is 2, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0xf4,0x23e80200);

/* add 124.0.26.0 255.255.255.0 0xdead0002 0x0 9192 0x0d000200
 */
/* @@@ Adding dest=0x7c001a00, mask = 0xffffff00, blade_nh_id=0xd000002,mtu_port=0x23e80200,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40568,0x1f0000);

/* NH is 0x2, blade is 0xd, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0xf8,0x20d00);

/* MTU is 9192, NH ID type is 2, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0xfc,0x23e80200);

/* add 124.0.27.0 255.255.255.0 0xdead0003 0x0 9192 0x0e000200
 */
/* @@@ Adding dest=0x7c001b00, mask = 0xffffff00, blade_nh_id=0xe000003,mtu_port=0x23e80200,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x4056c,0x200000);

/* NH is 0x3, blade is 0xe, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x100,0x30e00);

/* MTU is 9192, NH ID type is 2, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x104,0x23e80200);

/* add 124.0.28.0 255.255.255.0 0xdead0004 0x0 9192 0x01000200
 */
/* @@@ Adding dest=0x7c001c00, mask = 0xffffff00, blade_nh_id=0x1000004,mtu_port=0x23e80200,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40570,0x210000);

/* NH is 0x4, blade is 0x1, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x108,0x40100);

/* MTU is 9192, NH ID type is 2, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x10c,0x23e80200);

/* add 124.0.29.0 255.255.255.0 0xdead0005 0x0 9192 0x02000200
 */
/* @@@ Adding dest=0x7c001d00, mask = 0xffffff00, blade_nh_id=0x2000005,mtu_port=0x23e80200,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40574,0x220000);

/* NH is 0x5, blade is 0x2, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x110,0x50200);

/* MTU is 9192, NH ID type is 2, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x114,0x23e80200);

/* add 124.0.30.0 255.255.255.0 0xdead0006 0x0 9192 0x03000000
 */
/* @@@ Adding dest=0x7c001e00, mask = 0xffffff00, blade_nh_id=0x3000006,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40578,0x230000);

/* NH is 0x6, blade is 0x3, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x118,0x60300);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x11c,0x23e80000);

/* add 124.0.31.0 255.255.255.0 0xdead0007 0x0 9192 0x04000000
 */
/* @@@ Adding dest=0x7c001f00, mask = 0xffffff00, blade_nh_id=0x4000007,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x4057c,0x240000);

/* NH is 0x7, blade is 0x4, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x120,0x70400);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x124,0x23e80000);

/* 
 *//* #-------------------------------------------------
 */ /* # MPLS functional testing - 24 bit routes
 */ /* #-------------------------------------------------
 */ /* add 190.1.190.0 255.255.255.0 0xdead0000 0x0 9192 0x01000200
 */
/* @@@ Adding dest=0xbe01be00, mask = 0xffffff00, blade_nh_id=0x1000000,mtu_port=0x23e80200,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x405f8,0x250000);

/* NH is 0x0, blade is 0x1, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x128,0x100);

/* MTU is 9192, NH ID type is 2, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x12c,0x23e80200);


	SRAM_WRITE(trie_base + 0x405ac,0x7);


	SRAM_WRITE(trie_base + 0x2f804,0x6);

/* add 190.1.191.0 255.255.255.0 0xdead0001 0x0 9192 0x02000200
 */
/* @@@ Adding dest=0xbe01bf00, mask = 0xffffff00, blade_nh_id=0x2000001,mtu_port=0x23e80200,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x405fc,0x260000);

/* NH is 0x1, blade is 0x2, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x130,0x10200);

/* MTU is 9192, NH ID type is 2, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x134,0x23e80200);

/* add 190.1.192.0 255.255.255.0 0xdead0002 0x0 9192 0x03000200
 */
/* @@@ Adding dest=0xbe01c000, mask = 0xffffff00, blade_nh_id=0x3000002,mtu_port=0x23e80200,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40600,0x270000);

/* NH is 0x2, blade is 0x3, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x138,0x20300);

/* MTU is 9192, NH ID type is 2, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x13c,0x23e80200);


	SRAM_WRITE(trie_base + 0x405b0,0x8);

/* add 190.1.193.0 255.255.255.0 0xdead0003 0x0 9192 0x04000200
 */
/* @@@ Adding dest=0xbe01c100, mask = 0xffffff00, blade_nh_id=0x4000003,mtu_port=0x23e80200,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40604,0x280000);

/* NH is 0x3, blade is 0x4, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x140,0x30400);

/* MTU is 9192, NH ID type is 2, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x144,0x23e80200);

/* add 190.1.194.0 255.255.255.0 0xdead0004 0x0 9192 0x05000200
 */
/* @@@ Adding dest=0xbe01c200, mask = 0xffffff00, blade_nh_id=0x5000004,mtu_port=0x23e80200,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40608,0x290000);

/* NH is 0x4, blade is 0x5, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x148,0x40500);

/* MTU is 9192, NH ID type is 2, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x14c,0x23e80200);

/* add 190.1.195.0 255.255.255.0 0xdead0005 0x0 9192 0x06000200
 */
/* @@@ Adding dest=0xbe01c300, mask = 0xffffff00, blade_nh_id=0x6000005,mtu_port=0x23e80200,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x4060c,0x2a0000);

/* NH is 0x5, blade is 0x6, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x150,0x50600);

/* MTU is 9192, NH ID type is 2, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x154,0x23e80200);

/* add 190.1.196.0 255.255.255.0 0xdead0006 0x0 9192 0x07000200
 */
/* @@@ Adding dest=0xbe01c400, mask = 0xffffff00, blade_nh_id=0x7000006,mtu_port=0x23e80200,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40610,0x2b0000);

/* NH is 0x6, blade is 0x7, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x158,0x60700);

/* MTU is 9192, NH ID type is 2, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x15c,0x23e80200);

/* add 190.1.197.0 255.255.255.0 0xdead0007 0x0 9192 0x08000200
 */
/* @@@ Adding dest=0xbe01c500, mask = 0xffffff00, blade_nh_id=0x8000007,mtu_port=0x23e80200,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40614,0x2c0000);

/* NH is 0x7, blade is 0x8, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x160,0x70800);

/* MTU is 9192, NH ID type is 2, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x164,0x23e80200);

/* add 190.1.198.0 255.255.255.0 0xdead0008 0x0 9192 0x09000200
 */
/* @@@ Adding dest=0xbe01c600, mask = 0xffffff00, blade_nh_id=0x9000008,mtu_port=0x23e80200,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40618,0x2d0000);

/* NH is 0x8, blade is 0x9, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x168,0x80900);

/* MTU is 9192, NH ID type is 2, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x16c,0x23e80200);

/* add 190.1.199.0 255.255.255.0 0xdead0009 0x0 9192 0x0a000200
 */
/* @@@ Adding dest=0xbe01c700, mask = 0xffffff00, blade_nh_id=0xa000009,mtu_port=0x23e80200,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x4061c,0x2e0000);

/* NH is 0x9, blade is 0xa, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x170,0x90a00);

/* MTU is 9192, NH ID type is 2, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x174,0x23e80200);

/* 
 *//* #-------------------------------------------------
 */ /* # normal testing - 32 bit routes
 */ /* #-------------------------------------------------
 */ /* 
 *//* add 32.0.0.0 255.255.255.255 0xdead0000 0x0 9212 0x01000000
 */
/* @@@ Adding dest=0x20000000, mask = 0xffffffff, blade_nh_id=0x1000000,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40700,0x2f0000);

/* NH is 0x0, blade is 0x1, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x178,0x100);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x17c,0x23fc0000);


	SRAM_WRITE(trie_base + 0x406c0,0xc);


	SRAM_WRITE(trie_base + 0x40680,0xb);


	SRAM_WRITE(trie_base + 0x40640,0xa);


	SRAM_WRITE(trie_base + 0x8000,0x9);

/* add 32.0.0.1 255.255.255.255 0xdead0001 0x0 9212 0x02000000
 */
/* @@@ Adding dest=0x20000001, mask = 0xffffffff, blade_nh_id=0x2000001,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40704,0x300000);

/* NH is 0x1, blade is 0x2, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x180,0x10200);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x184,0x23fc0000);

/* add 32.0.0.2 255.255.255.255 0xdead0002 0x0 9212 0x03000000
 */
/* @@@ Adding dest=0x20000002, mask = 0xffffffff, blade_nh_id=0x3000002,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40708,0x310000);

/* NH is 0x2, blade is 0x3, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x188,0x20300);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x18c,0x23fc0000);

/* add 32.0.0.3 255.255.255.255 0xdead0003 0x0 9212 0x04000000
 */
/* @@@ Adding dest=0x20000003, mask = 0xffffffff, blade_nh_id=0x4000003,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x4070c,0x320000);

/* NH is 0x3, blade is 0x4, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x190,0x30400);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x194,0x23fc0000);

/* add 32.0.0.4 255.255.255.255 0xdead0004 0x0 9212 0x05000000
 */
/* @@@ Adding dest=0x20000004, mask = 0xffffffff, blade_nh_id=0x5000004,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40710,0x330000);

/* NH is 0x4, blade is 0x5, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x198,0x40500);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x19c,0x23fc0000);

/* add 32.0.0.5 255.255.255.255 0xdead0005 0x0 9212 0x06000000
 */
/* @@@ Adding dest=0x20000005, mask = 0xffffffff, blade_nh_id=0x6000005,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40714,0x340000);

/* NH is 0x5, blade is 0x6, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1a0,0x50600);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1a4,0x23fc0000);

/* add 32.0.0.6 255.255.255.255 0xdead0006 0x0 9212 0x07000000
 */
/* @@@ Adding dest=0x20000006, mask = 0xffffffff, blade_nh_id=0x7000006,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40718,0x350000);

/* NH is 0x6, blade is 0x7, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1a8,0x60700);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1ac,0x23fc0000);

/* add 32.0.0.7 255.255.255.255 0xdead0007 0x0 9212 0x08000000
 */
/* @@@ Adding dest=0x20000007, mask = 0xffffffff, blade_nh_id=0x8000007,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x4071c,0x360000);

/* NH is 0x7, blade is 0x8, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1b0,0x70800);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1b4,0x23fc0000);

/* add 32.0.0.8 255.255.255.255 0xdead0008 0x0 9212 0x09000000
 */
/* @@@ Adding dest=0x20000008, mask = 0xffffffff, blade_nh_id=0x9000008,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40720,0x370000);

/* NH is 0x8, blade is 0x9, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1b8,0x80900);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1bc,0x23fc0000);

/* add 32.0.0.9 255.255.255.255 0xdead0009 0x0 9212 0x0a000000
 */
/* @@@ Adding dest=0x20000009, mask = 0xffffffff, blade_nh_id=0xa000009,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40724,0x380000);

/* NH is 0x9, blade is 0xa, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1c0,0x90a00);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1c4,0x23fc0000);

/* add 32.0.0.10 255.255.255.255 0xdead000a 0x0 9212 0x0b000000
 */
/* @@@ Adding dest=0x2000000a, mask = 0xffffffff, blade_nh_id=0xb00000a,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40728,0x390000);

/* NH is 0xa, blade is 0xb, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1c8,0xa0b00);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1cc,0x23fc0000);

/* add 32.0.0.11 255.255.255.255 0xdead000b 0x0 9212 0x0c000000
 */
/* @@@ Adding dest=0x2000000b, mask = 0xffffffff, blade_nh_id=0xc00000b,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x4072c,0x3a0000);

/* NH is 0xb, blade is 0xc, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1d0,0xb0c00);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1d4,0x23fc0000);

/* add 32.0.0.12 255.255.255.255 0xdead000c 0x0 9212 0x0d000000
 */
/* @@@ Adding dest=0x2000000c, mask = 0xffffffff, blade_nh_id=0xd00000c,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40730,0x3b0000);

/* NH is 0xc, blade is 0xd, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1d8,0xc0d00);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1dc,0x23fc0000);

/* add 32.0.0.13 255.255.255.255 0xdead000d 0x0 9212 0x0e000000
 */
/* @@@ Adding dest=0x2000000d, mask = 0xffffffff, blade_nh_id=0xe00000d,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40734,0x3c0000);

/* NH is 0xd, blade is 0xe, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1e0,0xd0e00);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1e4,0x23fc0000);

/* add 32.0.0.14 255.255.255.255 0xdead000e 0x0 9212 0x01000000
 */
/* @@@ Adding dest=0x2000000e, mask = 0xffffffff, blade_nh_id=0x100000e,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40738,0x3d0000);

/* NH is 0xe, blade is 0x1, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1e8,0xe0100);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1ec,0x23fc0000);

/* add 32.0.0.15 255.255.255.255 0xdead000f 0x0 9212 0x02000000
 */
/* @@@ Adding dest=0x2000000f, mask = 0xffffffff, blade_nh_id=0x200000f,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x4073c,0x3e0000);

/* NH is 0xf, blade is 0x2, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1f0,0xf0200);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1f4,0x23fc0000);

/* 
 *//* #-------------------------------------------------
 */ /* # more entries: 16 - 31
 */ /* #-------------------------------------------------
 */ /* 
 *//* add 32.0.0.16 255.255.255.255 0xdead0010 0x0 9212 0x03000000
 */
/* @@@ Adding dest=0x20000010, mask = 0xffffffff, blade_nh_id=0x3000010,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40740,0x3f0000);

/* NH is 0x10, blade is 0x3, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1f8,0x100300);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x1fc,0x23fc0000);


	SRAM_WRITE(trie_base + 0x406c4,0xd);

/* add 32.0.0.17 255.255.255.255 0xdead0011 0x0 9212 0x04000000
 */
/* @@@ Adding dest=0x20000011, mask = 0xffffffff, blade_nh_id=0x4000011,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40744,0x400000);

/* NH is 0x11, blade is 0x4, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x200,0x110400);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x204,0x23fc0000);

/* add 32.0.0.18 255.255.255.255 0xdead0012 0x0 9212 0x05000000
 */
/* @@@ Adding dest=0x20000012, mask = 0xffffffff, blade_nh_id=0x5000012,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40748,0x410000);

/* NH is 0x12, blade is 0x5, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x208,0x120500);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x20c,0x23fc0000);

/* add 32.0.0.19 255.255.255.255 0xdead0013 0x0 9212 0x06000000
 */
/* @@@ Adding dest=0x20000013, mask = 0xffffffff, blade_nh_id=0x6000013,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x4074c,0x420000);

/* NH is 0x13, blade is 0x6, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x210,0x130600);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x214,0x23fc0000);

/* add 32.0.0.20 255.255.255.255 0xdead0014 0x0 9212 0x07000000
 */
/* @@@ Adding dest=0x20000014, mask = 0xffffffff, blade_nh_id=0x7000014,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40750,0x430000);

/* NH is 0x14, blade is 0x7, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x218,0x140700);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x21c,0x23fc0000);

/* add 32.0.0.21 255.255.255.255 0xdead0015 0x0 9212 0x08000000
 */
/* @@@ Adding dest=0x20000015, mask = 0xffffffff, blade_nh_id=0x8000015,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40754,0x440000);

/* NH is 0x15, blade is 0x8, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x220,0x150800);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x224,0x23fc0000);

/* add 32.0.0.22 255.255.255.255 0xdead0016 0x0 9212 0x09000000
 */
/* @@@ Adding dest=0x20000016, mask = 0xffffffff, blade_nh_id=0x9000016,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40758,0x450000);

/* NH is 0x16, blade is 0x9, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x228,0x160900);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x22c,0x23fc0000);

/* add 32.0.0.23 255.255.255.255 0xdead0017 0x0 9212 0x0a000000
 */
/* @@@ Adding dest=0x20000017, mask = 0xffffffff, blade_nh_id=0xa000017,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x4075c,0x460000);

/* NH is 0x17, blade is 0xa, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x230,0x170a00);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x234,0x23fc0000);

/* add 32.0.0.24 255.255.255.255 0xdead0000 0x0 9212 0x0b000000
 */
/* @@@ Adding dest=0x20000018, mask = 0xffffffff, blade_nh_id=0xb000000,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40760,0x470000);

/* NH is 0x0, blade is 0xb, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x238,0xb00);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x23c,0x23fc0000);

/* add 32.0.0.25 255.255.255.255 0xdead0001 0x0 9212 0x0c000000
 */
/* @@@ Adding dest=0x20000019, mask = 0xffffffff, blade_nh_id=0xc000001,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40764,0x480000);

/* NH is 0x1, blade is 0xc, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x240,0x10c00);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x244,0x23fc0000);

/* add 32.0.0.26 255.255.255.255 0xdead0002 0x0 9212 0x0d000000
 */
/* @@@ Adding dest=0x2000001a, mask = 0xffffffff, blade_nh_id=0xd000002,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40768,0x490000);

/* NH is 0x2, blade is 0xd, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x248,0x20d00);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x24c,0x23fc0000);

/* add 32.0.0.27 255.255.255.255 0xdead0003 0x0 9212 0x0e000000
 */
/* @@@ Adding dest=0x2000001b, mask = 0xffffffff, blade_nh_id=0xe000003,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x4076c,0x4a0000);

/* NH is 0x3, blade is 0xe, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x250,0x30e00);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x254,0x23fc0000);

/* add 32.0.0.28 255.255.255.255 0xdead0004 0x0 9212 0x01000000
 */
/* @@@ Adding dest=0x2000001c, mask = 0xffffffff, blade_nh_id=0x1000004,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40770,0x4b0000);

/* NH is 0x4, blade is 0x1, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x258,0x40100);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x25c,0x23fc0000);

/* add 32.0.0.29 255.255.255.255 0xdead0005 0x0 9212 0x02000000
 */
/* @@@ Adding dest=0x2000001d, mask = 0xffffffff, blade_nh_id=0x2000005,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40774,0x4c0000);

/* NH is 0x5, blade is 0x2, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x260,0x50200);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x264,0x23fc0000);

/* add 32.0.0.30 255.255.255.255 0xdead0006 0x0 9212 0x03000000
 */
/* @@@ Adding dest=0x2000001e, mask = 0xffffffff, blade_nh_id=0x3000006,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40778,0x4d0000);

/* NH is 0x6, blade is 0x3, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x268,0x60300);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x26c,0x23fc0000);

/* add 32.0.0.31 255.255.255.255 0xdead0007 0x0 9212 0x04000000
 */
/* @@@ Adding dest=0x2000001f, mask = 0xffffffff, blade_nh_id=0x4000007,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x4077c,0x4e0000);

/* NH is 0x7, blade is 0x4, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x270,0x70400);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x274,0x23fc0000);

/* add 32.0.0.32 255.255.255.255 0xdead0008 0x0 9212 0x05000000
 */
/* @@@ Adding dest=0x20000020, mask = 0xffffffff, blade_nh_id=0x5000008,mtu_port=0x23fc0000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40780,0x4f0000);

/* NH is 0x8, blade is 0x5, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x278,0x80500);

/* MTU is 9212, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x27c,0x23fc0000);


	SRAM_WRITE(trie_base + 0x406c8,0xe);

/* 
 *//* 
 *//* # routing functionality tests.
 */ /* 
 *//* # local delivery
 */ /* add 24.135.118.57 255.255.255.255 0xdead0000 0x1 9192 0x02000001
 */
/* @@@ Adding dest=0x18877639, mask = 0xffffffff, blade_nh_id=0x2000000,mtu_port=0x23e80001,flags=0x1,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x408a4,0x500000);

/* NH is 0x0, blade is 0x2, flag is 0x1 */ 

SDRAM_WRITE(nh_base + 0x280,0x201);

/* MTU is 9192, NH ID type is 0, outport is 0x1 */ 

SDRAM_WRITE(nh_base + 0x284,0x23e80001);


	SRAM_WRITE(trie_base + 0x4084c,0x12);


	SRAM_WRITE(trie_base + 0x40818,0x11);


	SRAM_WRITE(trie_base + 0x407dc,0x10);


	SRAM_WRITE(trie_base + 0x621c,0xf);

/* 
 *//* #if down
 */ /* add 24.135.118.58 255.255.255.255 0xdead0001 0x2 9192 0x02000001
 */
/* @@@ Adding dest=0x1887763a, mask = 0xffffffff, blade_nh_id=0x2000001,mtu_port=0x23e80001,flags=0x2,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x408a8,0x510000);

/* NH is 0x1, blade is 0x2, flag is 0x2 */ 

SDRAM_WRITE(nh_base + 0x288,0x10202);

/* MTU is 9192, NH ID type is 0, outport is 0x1 */ 

SDRAM_WRITE(nh_base + 0x28c,0x23e80001);

/* 
 *//* #redirect
 */ /* add 24.135.118.59 255.255.255.255 0xdead0002 0x0 9192 0x01000000
 */
/* @@@ Adding dest=0x1887763b, mask = 0xffffffff, blade_nh_id=0x1000002,mtu_port=0x23e80000,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x408ac,0x520000);

/* NH is 0x2, blade is 0x1, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x290,0x20100);

/* MTU is 9192, NH ID type is 0, outport is 0x0 */ 

SDRAM_WRITE(nh_base + 0x294,0x23e80000);

/* 
 *//* #fragment
 */ /* add 24.135.118.60 255.255.255.255 0xdead0003 0x0 5  0x02000001
 */
/* @@@ Adding dest=0x1887763c, mask = 0xffffffff, blade_nh_id=0x2000003,mtu_port=0x50001,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x408b0,0x530000);

/* NH is 0x3, blade is 0x2, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x298,0x30200);

/* MTU is 5, NH ID type is 0, outport is 0x1 */ 

SDRAM_WRITE(nh_base + 0x29c,0x50001);

/* 
 *//* #8bit mask. Send pkt with dst: 0x1ac17638
 */ /* add 26.0.0.0 255.0.0.0 0xdead0008 0x0 9192 0x02000001
 */
/* @@@ Adding dest=0x1a000000, mask = 0xff000000, blade_nh_id=0x2000008,mtu_port=0x23e80001,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40068,0x540000);

/* NH is 0x8, blade is 0x2, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x2a0,0x80200);

/* MTU is 9192, NH ID type is 0, outport is 0x1 */ 

SDRAM_WRITE(nh_base + 0x2a4,0x23e80001);

/* 
 *//* #15bit mask (with 8bit already existing). 
 */ /* #Send pkt with dst: 0x1a878001
 */ /* add 26.134.0.0 255.254.0.0 0xdead0015 0x0 9192 0x02000001
 */
/* @@@ Adding dest=0x1a860000, mask = 0xfffe0000, blade_nh_id=0x2000015,mtu_port=0x23e80001,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40918,0x550000);


	SRAM_WRITE(trie_base + 0x4091c,0x550000);

/* NH is 0x15, blade is 0x2, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x2a8,0x150200);

/* MTU is 9192, NH ID type is 0, outport is 0x1 */ 

SDRAM_WRITE(nh_base + 0x2ac,0x23e80001);


	SRAM_WRITE(trie_base + 0x408e0,0x14);


	SRAM_WRITE(trie_base + 0x40068,0x540013);

/* 
 *//* #20bit mask (with 8 and 15 existing) 
 */ /* # send pkt with dst:0x1a877838
 */ /* add 26.135.112.0 255.255.240.0 0xdead0020 0x0 9192 0x02000001
 */
/* @@@ Adding dest=0x1a877000, mask = 0xfffff000, blade_nh_id=0x2000020,mtu_port=0x23e80001,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x4095c,0x560000);

/* NH is 0x20, blade is 0x2, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x2b0,0x200200);

/* MTU is 9192, NH ID type is 0, outport is 0x1 */ 

SDRAM_WRITE(nh_base + 0x2b4,0x23e80001);


	SRAM_WRITE(trie_base + 0x6a1c,0x15);

/* 
 *//* #32bit
 */ /* # send pkt with dst:0x1a877638
 */ /* add 26.135.118.56 255.255.255.255 0xdead0032 0x0 9192 0x02000001
 */
/* @@@ Adding dest=0x1a877638, mask = 0xffffffff, blade_nh_id=0x2000032,mtu_port=0x23e80001,flags=0x0,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40a20,0x570000);

/* NH is 0x32, blade is 0x2, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x2b8,0x320200);

/* MTU is 9192, NH ID type is 0, outport is 0x1 */ 

SDRAM_WRITE(nh_base + 0x2bc,0x23e80001);


	SRAM_WRITE(trie_base + 0x409cc,0x18);


	SRAM_WRITE(trie_base + 0x40998,0x17);


	SRAM_WRITE(trie_base + 0x4095c,0x560016);

/* 
 *//* # for destination dbcast address = 0x1a877fff, drop these packets
 */ /* add 26.135.127.255 255.255.255.255 0xdeadbeef 0x4 9212 0x05000005
 */
/* @@@ Adding dest=0x1a877fff, mask = 0xffffffff, blade_nh_id=0x500beef,mtu_port=0x23fc0005,flags=0x4,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40abc,0x580000);

/* NH is 0xbeef, blade is 0x5, flag is 0x4 */ 

SDRAM_WRITE(nh_base + 0x2c0,0xbeef0504);

/* MTU is 9212, NH ID type is 0, outport is 0x5 */ 

SDRAM_WRITE(nh_base + 0x2c4,0x23fc0005);


	SRAM_WRITE(trie_base + 0x40a7c,0x1a);


	SRAM_WRITE(trie_base + 0x409bc,0x19);

/* 
 *//* # for destination dbcast address = 0x1a877f00, drop these packets
 */ /* add 26.135.127.00 255.255.255.255 0xdeadbeef 0x4 9212 0x05000005
 */
/* @@@ Adding dest=0x1a877f00, mask = 0xffffffff, blade_nh_id=0x500beef,mtu_port=0x23fc0005,flags=0x4,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40ac0,0x590000);

/* NH is 0xbeef, blade is 0x5, flag is 0x4 */ 

SDRAM_WRITE(nh_base + 0x2c8,0xbeef0504);

/* MTU is 9212, NH ID type is 0, outport is 0x5 */ 

SDRAM_WRITE(nh_base + 0x2cc,0x23fc0005);


	SRAM_WRITE(trie_base + 0x40a40,0x1b);

/* 
 *//* # for destination dbcast address = 0x00000081, drop these packets
 */ /* add 0.0.0.81 255.255.255.255 0xdeadbeef 0x4 9212 0x05000005
 */
/* @@@ Adding dest=0x51, mask = 0xffffffff, blade_nh_id=0x500beef,mtu_port=0x23fc0005,flags=0x4,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40bc4,0x5a0000);

/* NH is 0xbeef, blade is 0x5, flag is 0x4 */ 

SDRAM_WRITE(nh_base + 0x2d0,0xbeef0504);

/* MTU is 9212, NH ID type is 0, outport is 0x5 */ 

SDRAM_WRITE(nh_base + 0x2d4,0x23fc0005);


	SRAM_WRITE(trie_base + 0x40b94,0x1f);


	SRAM_WRITE(trie_base + 0x40b40,0x1e);


	SRAM_WRITE(trie_base + 0x40b00,0x1d);


	SRAM_WRITE(trie_base + 0x0,0x1c);

/* 
 *//* #-----------------------------------------------
 */ /* # bunch of invalid destination addresses
 */ /* #-----------------------------------------------
 */ /* 
 *//* # drop if dst==0
 */ /* add 0.0.0.0 255.0.0.0 0xdeadbeef 0x4 9212 0x05000005
 */
/* @@@ Adding dest=0x0, mask = 0xff000000, blade_nh_id=0x500beef,mtu_port=0x23fc0005,flags=0x4,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40000,0x5b0000);

/* NH is 0xbeef, blade is 0x5, flag is 0x4 */ 

SDRAM_WRITE(nh_base + 0x2d8,0xbeef0504);

/* MTU is 9212, NH ID type is 0, outport is 0x5 */ 

SDRAM_WRITE(nh_base + 0x2dc,0x23fc0005);

/* 
 *//* # drop if dst==127.*
 */ /* add 127.0.0.0 255.0.0.0 0xdeadbeef 0x4 9212 0x05000005
 */
/* @@@ Adding dest=0x7f000000, mask = 0xff000000, blade_nh_id=0x500beef,mtu_port=0x23fc0005,flags=0x4,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x401fc,0x5c0000);

/* NH is 0xbeef, blade is 0x5, flag is 0x4 */ 

SDRAM_WRITE(nh_base + 0x2e0,0xbeef0504);

/* MTU is 9212, NH ID type is 0, outport is 0x5 */ 

SDRAM_WRITE(nh_base + 0x2e4,0x23fc0005);

/* 
 *//* # drop if dst==classE
 */ /* add 240.0.0.0 240.0.0.0 0xdeadbeef 0x4 9212 0x05000005
 */
/* @@@ Adding dest=0xf0000000, mask = 0xf0000000, blade_nh_id=0x500beef,mtu_port=0x23fc0005,flags=0x4,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x403c0,0x5d0000);


	SRAM_WRITE(trie_base + 0x403c4,0x5d0000);


	SRAM_WRITE(trie_base + 0x403c8,0x5d0000);


	SRAM_WRITE(trie_base + 0x403cc,0x5d0000);


	SRAM_WRITE(trie_base + 0x403d0,0x5d0000);


	SRAM_WRITE(trie_base + 0x403d4,0x5d0000);


	SRAM_WRITE(trie_base + 0x403d8,0x5d0000);


	SRAM_WRITE(trie_base + 0x403dc,0x5d0000);


	SRAM_WRITE(trie_base + 0x403e0,0x5d0000);


	SRAM_WRITE(trie_base + 0x403e4,0x5d0000);


	SRAM_WRITE(trie_base + 0x403e8,0x5d0000);


	SRAM_WRITE(trie_base + 0x403ec,0x5d0000);


	SRAM_WRITE(trie_base + 0x403f0,0x5d0000);


	SRAM_WRITE(trie_base + 0x403f4,0x5d0000);


	SRAM_WRITE(trie_base + 0x403f8,0x5d0000);


	SRAM_WRITE(trie_base + 0x403fc,0x5d0000);

/* NH is 0xbeef, blade is 0x5, flag is 0x4 */ 

SDRAM_WRITE(nh_base + 0x2e8,0xbeef0504);

/* MTU is 9212, NH ID type is 0, outport is 0x5 */ 

SDRAM_WRITE(nh_base + 0x2ec,0x23fc0005);

/* 
 *//* #------------------------------------------------
 */ /* # bunch of local delivery destination addresses
 */ /* #-------------------------------------------------
 */ /* 
 *//* # multicast
 */ /* add 224.0.0.0 240.0.0.0 0xdead0000 0x10 9192 0x02000001
 */
/* @@@ Adding dest=0xe0000000, mask = 0xf0000000, blade_nh_id=0x2000000,mtu_port=0x23e80001,flags=0x10,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40380,0x5e0000);


	SRAM_WRITE(trie_base + 0x40384,0x5e0000);


	SRAM_WRITE(trie_base + 0x40388,0x5e0000);


	SRAM_WRITE(trie_base + 0x4038c,0x5e0000);


	SRAM_WRITE(trie_base + 0x40390,0x5e0000);


	SRAM_WRITE(trie_base + 0x40394,0x5e0000);


	SRAM_WRITE(trie_base + 0x40398,0x5e0000);


	SRAM_WRITE(trie_base + 0x4039c,0x5e0000);


	SRAM_WRITE(trie_base + 0x403a0,0x5e0000);


	SRAM_WRITE(trie_base + 0x403a4,0x5e0000);


	SRAM_WRITE(trie_base + 0x403a8,0x5e0000);


	SRAM_WRITE(trie_base + 0x403ac,0x5e0000);


	SRAM_WRITE(trie_base + 0x403b0,0x5e0000);


	SRAM_WRITE(trie_base + 0x403b4,0x5e0000);


	SRAM_WRITE(trie_base + 0x403b8,0x5e0000);


	SRAM_WRITE(trie_base + 0x403bc,0x5e0000);

/* NH is 0x0, blade is 0x2, flag is 0x10 */ 

SDRAM_WRITE(nh_base + 0x2f0,0x210);

/* MTU is 9192, NH ID type is 0, outport is 0x1 */ 

SDRAM_WRITE(nh_base + 0x2f4,0x23e80001);

/* 
 *//* # limited broadcast
 */ /* add 255.255.255.255 255.255.255.255 0xdead0000 0x8 9192 0x02000001
 */
/* @@@ Adding dest=0xffffffff, mask = 0xffffffff, blade_nh_id=0x2000000,mtu_port=0x23e80001,flags=0x8,nhid_type=0x0 */

	SRAM_WRITE(trie_base + 0x40cfc,0x5f0000);

/* NH is 0x0, blade is 0x2, flag is 0x8 */ 

SDRAM_WRITE(nh_base + 0x2f8,0x208);

/* MTU is 9192, NH ID type is 0, outport is 0x1 */ 

SDRAM_WRITE(nh_base + 0x2fc,0x23e80001);


	SRAM_WRITE(trie_base + 0x40cbc,0x23);


	SRAM_WRITE(trie_base + 0x40c7c,0x22);


	SRAM_WRITE(trie_base + 0x40c3c,0x21);


	SRAM_WRITE(trie_base + 0x3fffc,0x20);

/* 
 *//* #-------------------------------------------------
 */ /* # non-IPV4 stuff
 */ /* #-------------------------------------------------
 */ /* add 10.3.19.130	255.255.255.255 0x0a03 0x2 9192 0x02000001 1
 */
/* @@@ Adding dest=0xa031382, mask = 0xffffffff, blade_nh_id=0x2000a03,mtu_port=0x23e80001,flags=0x2,nhid_type=0x1 */

	SRAM_WRITE(trie_base + 0x40dc8,0x600000);

/* NH is 0xa03, blade is 0x2, flag is 0x2 */ 

SDRAM_WRITE(nh_base + 0x300,0xa030202);

/* MTU is 9192, NH ID type is 0, outport is 0x1 */ 

SDRAM_WRITE(nh_base + 0x304,0x23e80001);


	SRAM_WRITE(trie_base + 0x40da0,0x27);


	SRAM_WRITE(trie_base + 0x40d4c,0x26);


	SRAM_WRITE(trie_base + 0x40d04,0x25);


	SRAM_WRITE(trie_base + 0x280c,0x24);

/* 
 *//* # default route
 */ /* # send pkt with 0x1b000001
 */ /* default 0xabcddcba 0x0 9192 0x03000003 *//* NH is 0xdcba, blade is 0x3, flag is 0x0 */ 

SDRAM_WRITE(nh_base + 0x0,0xdcba0300);

/* MTU is 9192, NH ID type is 0, outport is 0x3 */ 

SDRAM_WRITE(nh_base + 0x4,0x23e80003);




return 0;


}
