//------------------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2001-2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//------------------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: 	system.h
//
//      Purpose:        This file contains the system wide parameters that can be 
//                      customised for a given hardware/application 
//
//
/////////////////////////////////////////////////////////////////////////////
#ifndef	__SYSTEM__H__
#define	__SYSTEM__H__


/*	Chip is IXP2400 */
#ifndef IXP2400
#define IXP2400
#endif	

/* Define project as microcode based. */
#ifndef IXP_MICROCODE
#define IXP_MICROCODE
#endif

/*---------------------------------------------------------------------
 * Include global definition and configuration files
 *-------------------------------------------------------------------*/

#include <definitions.h>
#include <ix_cc_microengines_bindings.h>


/*---------------------------------------------------------------------
 *                             C A U T I O N
 *
 * The code below may not be changed by the user. If any of the below
 * code has to be overwritten, redefine it in a separate file and then
 * include this file.
 *-------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * 1) Miscellaneous settings.
 *-------------------------------------------------------------------*/

/* PHY mode for TX device. */
#ifndef		TX_PHY_MODE
#define		TX_PHY_MODE			MPHY_16
#endif

#ifndef		USE_IMPORT_VAR

/* Freelist id that is stored in meta data */
#ifndef		FREE_LIST_ID
#define		FREE_LIST_ID		0x0
#endif

/* This blade's id */
#ifndef		THIS_BLADE_ID
#define		THIS_BLADE_ID		255
#endif

#endif /* USE_IMPORT_VAR */


#ifdef		USE_IMPORT_VAR

/* Patched by core */
.import_var DL_REL_BASE
.import_var BUF_FREE_LIST0
.import_var BUF_SRAM_BASE
.import_var L2_TABLE_SRAM_BASE

#ifdef COUNTERS

.import_var PACKET_TX_COUNTER_BASE
#define POS_TX_COUNTER_BASE         PACKET_TX_COUNTER_BASE

#endif /* COUNTERS */

#endif /* USE_IMPORT_VAR */


/*---------------------------------------------------------------------
 * 2) Microblock specific settings.
 *-------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * i) CSIX RX microblock.
 *-------------------------------------------------------------------*/

/*
 * Size of each CSIX Receive Context (in Bytes)
 */
#ifndef		CSIX_TX_CTX_SIZE
#define		CSIX_TX_CTX_SIZE	32
#endif

/*---------------------------------------------------------------------
 * ii) L2 encap microblock.
 *-------------------------------------------------------------------*/

/* skip L2 encap module for performance test */
//#ifndef		SKIP_L2_ENCAP	
//define		SKIP_L2_ENCAP		SWITCH_ENABLE
//#endif

/*
 * Following definition enable this microblock add PPP header before payload 
 * so PPP L2_ENCAP microblock can be removed from pipeline for performance 
 * optimization
 */
//#if 		(SKIP_L2_ENCAP == SWITCH_ENABLE)
//#ifndef		ADD_PPP_HEADER
//#define  	ADD_PPP_HEADER		SWITCH_ENABLE	
//#endif		/* ADD_PPP_HEADER */
//#endif		/* SKIP_L2_ENCAP == SWITCH_ENABLE */

/*---------------------------------------------------------------------
 * iii) Queue manager microblock.
 *-------------------------------------------------------------------*/

/* Size of each queue descriptor entry in bytes. */
#ifndef		QD_SIZE
#define		QD_SIZE				16
#endif

/* Queue number for drop queue. (0-1023 are the normal queues used
   in forwarding pkts) */
#ifndef		QM_DROP_QUEUE
#define		QM_DROP_QUEUE		1024
#endif


/*---------------------------------------------------------------------
 * 3) SRAM memory map for channel 0.
 *-------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

 /* Area of SRAM which is zero'ed out. */
#ifndef		SRAM_ZERO_BLOCK
#define		SRAM_ZERO_BLOCK 	0x0
#endif

#endif /* USE_IMPORT_VAR */

/*---------------------------------------------------------------------
 * i) Base address for meta data (buffer descriptors) of packet 
 *    buffers.
 *-------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

#ifndef		BUF_SRAM_BASE
#define		BUF_SRAM_BASE		0
#endif

#endif /* USE_IMPORT_VAR */

/* Total SRAM size allocated in bytes for meta data.
   In simulation we use only 20KB. */
#ifndef		BUF_SRAM_SIZE
#ifdef 		IXP_SIMULATION
#define		BUF_SRAM_SIZE		(1024 * 10 * 2)
#else
#define		BUF_SRAM_SIZE		(1024 * 10 * 2 * 20)
#endif
#endif

/* Size of meta-data (buffer descriptor) in bytes for each buffer.
   Should be a power of 2. */
#ifndef		META_DATA_SIZE
#define		META_DATA_SIZE		32
#endif

/* Allocating 1 MB of SRAM for meta-data. */

/*---------------------------------------------------------------------
 * ii) Base address for queue descriptors.
 *-------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

#ifndef		QD_SRAM_BASE
#define		QD_SRAM_BASE		0x100000
#endif

/* Total number of queue descriptors. */
#ifndef		QD_TOTAL
#define		QD_TOTAL			1024
#endif

/* Allocating 1 MB of SRAM for queue descriptors. */

#endif /* USE_IMPORT_VAR */


/*---------------------------------------------------------------------
 * iii) Base address for CSIX RX contexts.
 *-------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

#ifndef		CSIX_RX_CTX_SRAM_BASE
#define		CSIX_RX_CTX_SRAM_BASE	0x200000
#endif

/*
 * Total number of CSIX Receive Contexts.
 */
#ifndef		CSIX_RX_CTX_TOTAL
#define		CSIX_RX_CTX_TOTAL		1024
#endif

#endif /* USE_IMPORT_VAR */

/*---------------------------------------------------------------------
 * iv) Base address for counters.
 *-------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

/*
 * CSIX_RX needs 1024 * 4 counters and each counter is 1LW 
 * This implies 1024*4*4 bytes
 */
#ifndef		CSIX_RX_COUNTERS_SRAM_BASE
#define		CSIX_RX_COUNTERS_SRAM_BASE	0x300000
#endif

#ifndef		CSIX_RX_COUNTER_BASE
#define		CSIX_RX_COUNTER_BASE		0x300000
#endif

/*
 * Packet TX needs 16*3 counters and each counter is 1LW 
 * this implies 48*4 bytes. Though CSIX RX counter does not require 
 * 0x10000 bytes but we start Packet TX counters from 0x310000, 
 * to provide memory for any additional counters in future.
 */
#ifndef		PACKET_TX_COUNTER_BASE
#define		PACKET_TX_COUNTER_BASE		0x310000	
#endif

#endif /* USE_IMPORT_VAR */

/*---------------------------------------------------------------------
 * v) Base address for L2 table.
 *-------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

/*
 * L2 Table (each entry has 32 bytes, bit 31 of first long word is
 * valid bit requires 64K * 32 = 2M 
 */
#ifndef		L2_TABLE_SRAM_BASE
#define 	L2_TABLE_SRAM_BASE			0x320000
#endif

#endif /* USE_IMPORT_VAR */


/*---------------------------------------------------------------------
 * 4) SRAM memory map for channel 1.
 *-------------------------------------------------------------------*/


/*---------------------------------------------------------------------
 * 5) SRAM Q-array memory map.
 *-------------------------------------------------------------------*/

/* SRAM q-array entries 0-15 are used by the queue manager to cache queues */
#ifndef		QM_QARRAY_BASE
#define		QM_QARRAY_BASE			0
#endif

#ifndef		QM_QARRAY_SIZE
#define		QM_QARRAY_SIZE			16
#endif

#ifndef		USE_IMPORT_VAR
//asv

/* SRAM q-array entries allocated for buffer free list. */
#ifndef		BUF_QARRAY_BASE
#define		BUF_QARRAY_BASE			16
#endif

#ifndef		BUF_QARRAY_SIZE
#define		BUF_QARRAY_SIZE			4
#endif

/* SRAM q-array entry allocated for drop queue in queue manager. */
#ifndef		QM_DROP_QUEUE_ENTRY
#define		QM_DROP_QUEUE_ENTRY		20
#endif

#endif /* USE_IMPORT_VAR */


/*---------------------------------------------------------------------
 * 6) DRAM memory map.
 *-------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * i) Base address for packet buffers. This is the actual packet data.
 *-------------------------------------------------------------------*/
#ifndef		USE_IMPORT_VAR

#ifndef		BUF_SDRAM_BASE
#define		BUF_SDRAM_BASE		0
#endif

#endif /* USE_IMPORT_VAR */

/* Total DRAM size allocated in bytes for packet buffers.
   In simulation we use only 1280KB. */
#ifndef		BUF_SDRAM_SIZE
#ifdef 		IXP_SIMULATION
#define		BUF_SDRAM_SIZE		(64 * 1024 * 10 * 2)
#else
#define		BUF_SDRAM_SIZE		(64 * 1024 * 10 * 2 * 20)
#endif
#endif

/*	Size of one packet buffer in bytes.  */
#ifndef		BUFFER_SIZE
#define		BUFFER_SIZE			2048
#endif

/* Allocating 64 MB of DRAM for packet buffers. */


/*---------------------------------------------------------------------
 * 7) Scratch and scratch ring memory map.
 *-------------------------------------------------------------------*/

/*  There are two exception ring to send pkts up */
#ifndef		EXCEPTION_RING_OUT_0
#define		EXCEPTION_RING_OUT_0			0
#endif

#ifndef		EXCEPTION_RING_OUT_1
#define		EXCEPTION_RING_OUT_1			1
#endif

/*  There are two exception ring to send pkts down */
#ifndef		EXCEPTION_RING_IN_0
#define		EXCEPTION_RING_IN_0				2
#endif

#ifndef		EXCEPTION_RING_IN_1
#define		EXCEPTION_RING_IN_1				3
#endif

/*
 * Scratch ring for communicating between CSIX RX and L2 Encap microblocks 
 */
#ifndef		L2_ENCAP_SOURCE_SCR
#define		L2_ENCAP_SOURCE_SCR
#define		L2_ENCAP_SOURCE_SCR_RING		4
#define		L2_ENCAP_SOURCE_SCR_RING_BASE	0
#define		L2_ENCAP_SOURCE_SCR_RING_SIZE	512
#endif		/* L2_ENCAP_SOURCE_SCR */

#ifndef		CSIX_RX_TO_QM_SCR
#define		CSIX_RX_TO_QM_SCR_RING			L2_ENCAP_SOURCE_SCR_RING
#define		CSIX_RX_TO_QM_SCR_RING_BASE		L2_ENCAP_SOURCE_SCR_RING_BASE
#define		CSIX_RX_TO_QM_SCR_RING_SIZE		L2_ENCAP_SOURCE_SCR_RING_SIZE
#endif		/* CSIX_RX_TO_QM_SCR */

/*
 * Scratch ring for communicating between Scheduler and Queue Manager Blocks
 */
#ifndef		SCHEDULER_TO_QM_SCR
#define		SCHEDULER_TO_QM_SCR
#define		SCHEDULER_TO_QM_SCR_RING		5
#define		SCHEDULER_TO_QM_SCR_RING_BASE	2048
#define		SCHEDULER_TO_QM_SCR_RING_SIZE	512
#endif		/* SCHEDULER_TO_QM_SCR */

/*
 * First scratch ring for communicating between Queue Manager and Packet TX 
 * microblocks.
 */
#ifndef		QM_TO_PACKET_TX_SCR_0
#define		QM_TO_PACKET_TX_SCR_0
#define 	QM_TO_PACKET_TX_SCR_RING_0		6
#define 	QM_TO_PACKET_TX_SCR_RING_0_BASE	4096
#define 	QM_TO_PACKET_TX_SCR_RING_0_SIZE	256
#endif		/* QM_TO_PACKET_TX_SCR_0 */

/*
 * Second scratch ring for communicating between Queue Manager and Packet TX 
 * microblocks.
 */
#ifndef		QM_TO_PACKET_TX_SCR_1
#define		QM_TO_PACKET_TX_SCR_1
#define 	QM_TO_PACKET_TX_SCR_RING_1		7
#define 	QM_TO_PACKET_TX_SCR_RING_1_BASE	5120
#define 	QM_TO_PACKET_TX_SCR_RING_1_SIZE	256
#endif		/* QM_TO_PACKET_TX_SCR_1 */

/*
 * Third scratch ring for communicating between Queue Manager and Packet TX 
 * microblocks.
 */
#ifndef		QM_TO_PACKET_TX_SCR_2
#define		QM_TO_PACKET_TX_SCR_2
#define 	QM_TO_PACKET_TX_SCR_RING_2		8
#define 	QM_TO_PACKET_TX_SCR_RING_2_BASE	6144
#define 	QM_TO_PACKET_TX_SCR_RING_2_SIZE	256
#endif		/* QM_TO_PACKET_TX_SCR_2 */

/* 
 * Fourth scratch ring for communicating between Queue Manager and Packet TX 
 * microblocks.
 */
#ifndef		QM_TO_PACKET_TX_SCR_3
#define		QM_TO_PACKET_TX_SCR_3
#define 	QM_TO_PACKET_TX_SCR_RING_3		9
#define 	QM_TO_PACKET_TX_SCR_RING_3_BASE	7168
#define 	QM_TO_PACKET_TX_SCR_RING_3_SIZE	256
#endif		/* QM_TO_PACKET_TX_SCR_3 */

/*
 * Scratch ring for communicating between L2 Encap and Queue Manager 
 * microblocks.
 */
#ifndef		L2_ENCAP_SINK_SCR
#define		L2_ENCAP_SINK_SCR
#define		L2_ENCAP_SINK_SCR_RING			10
#define		L2_ENCAP_SINK_SCR_RING_BASE		8192
#define		L2_ENCAP_SINK_SCR_RING_SIZE		512
#endif		/* L2_ENCAP_SINK_SCR */

/*
 * Scratch address required by Packet TX for updating available Transmit Buffers.
 */
#ifndef		PACKET_TX_AVAIL_TBUF_ELE_SCR_ADDR
#define 	PACKET_TX_AVAIL_TBUF_ELE_SCR_ADDR	14336
#endif 

/* These defines are used by the individual microcode to identify their
 * scratch rings. The reason for these defines is to improve the portability
 * of microblocks. User just has to make changes here and the microblocks will
 * remain untouched. For the naming convention, the microblock name goes first,
 * then the _RING_ specifier. Following this is the direction indicator.
 * '_OUT' would imply the microblock is writing to this scratch ring and
 * '_IN' would imply the microblock is reading from this scratch ring.
 * If there is more than one 'IN' or 'OUT' ring the name will have '_<digit>'
 * to differentiate the rings.
 * An '_IND' at the end indicates the compilation switch.
 */

#ifndef		CSIX_RX_RING_OUT_IND
#define		CSIX_RX_RING_OUT_IND
#define 	CSIX_RX_RING_OUT			L2_ENCAP_SOURCE_SCR_RING
#define		CSIX_RX_RING_OUT_BASE		L2_ENCAP_SOURCE_SCR_RING_BASE
#define		CSIX_RX_RING_OUT_SIZE		L2_ENCAP_SOURCE_SCR_RING_SIZE
#endif		/* CSIX_RX_RING_OUT_IND */

#ifndef SKIP_L2_ENCAP
#ifndef		QM_RING_IN_0_IND
#define		QM_RING_IN_0_IND
#define		QM_RING_IN_0				L2_ENCAP_SINK_SCR_RING
#define		QM_RING_IN_0_BASE			L2_ENCAP_SINK_SCR_RING_BASE
#define		QM_RING_IN_0_SIZE			L2_ENCAP_SINK_SCR_RING_SIZE
#endif		/* QM_RING_IN_0_IND */

#else 	/* SKIP_L2_ENCAP */
#ifndef		QM_RING_IN_0_IND
#define		QM_RING_IN_0_IND
#define		QM_RING_IN_0				L2_ENCAP_SOURCE_SCR_RING
#define		QM_RING_IN_0_BASE			L2_ENCAP_SOURCE_SCR_RING_BASE
#define		QM_RING_IN_0_SIZE			L2_ENCAP_SOURCE_SCR_RING_SIZE
#endif		/* QM_RING_IN_0_IND */

#endif 	/* SKIP_L2_ENCAP */

#ifndef		QM_RING_IN_1_IND
#define		QM_RING_IN_1_IND
#define		QM_RING_IN_1				SCHEDULER_TO_QM_SCR_RING
#define		QM_RING_IN_1_BASE			SCHEDULER_TO_QM_SCR_RING_BASE
#define		QM_RING_IN_1_SIZE			SCHEDULER_TO_QM_SCR_RING_SIZE
#endif

#ifndef		QM_RING_OUT_0_IND
#define		QM_RING_OUT_0_IND
#define		QM_RING_OUT_0				QM_TO_PACKET_TX_SCR_RING_0
#define		QM_RING_OUT_0_BASE			QM_TO_PACKET_TX_SCR_RING_0_BASE
#define		QM_RING_OUT_0_SIZE			QM_TO_PACKET_TX_SCR_RING_0_SIZE
#endif		/* QM_RING_OUT_0_IND */

#ifndef		QM_RING_OUT_1_IND
#define		QM_RING_OUT_1_IND
#define		QM_RING_OUT_1				QM_TO_PACKET_TX_SCR_RING_1	
#define		QM_RING_OUT_1_BASE			QM_TO_PACKET_TX_SCR_RING_1_BASE
#define		QM_RING_OUT_1_SIZE			QM_TO_PACKET_TX_SCR_RING_1_SIZE
#endif		/* QM_RING_OUT_1_IND */

#ifndef		QM_RING_OUT_2_IND
#define		QM_RING_OUT_2_IND
#define		QM_RING_OUT_2				QM_TO_PACKET_TX_SCR_RING_2
#define		QM_RING_OUT_2_BASE			QM_TO_PACKET_TX_SCR_RING_2_BASE
#define		QM_RING_OUT_2_SIZE			QM_TO_PACKET_TX_SCR_RING_2_SIZE
#endif
	 
#ifndef		QM_RING_OUT_3_IND
#define		QM_RING_OUT_3_IND
#define		QM_RING_OUT_3				QM_TO_PACKET_TX_SCR_RING_3	
#define		QM_RING_OUT_3_BASE			QM_TO_PACKET_TX_SCR_RING_3_BASE
#define		QM_RING_OUT_3_SIZE			QM_TO_PACKET_TX_SCR_RING_3_SIZE
#endif

#ifndef		SCHEDULER_RING_OUT_IND
#define		SCHEDULER_RING_OUT_IND
#define		SCHEDULER_RING_OUT			SCHEDULER_TO_QM_SCR_RING
#define		SCHEDULER_RING_OUT_BASE		SCHEDULER_TO_QM_SCR_RING_BASE			
#define		SCHEDULER_RING_OUT_SIZE		SCHEDULER_TO_QM_SCR_RING_SIZE
#endif		/* SCHEDULER_RING_OUT_IND */

#ifndef		PACKET_TX_IN_0_IND
#define		PACKET_TX_IN_0_IND
#define		PACKET_TX_IN_0				QM_TO_PACKET_TX_SCR_RING_0
#define		PACKET_TX_IN_0_BASE			QM_TO_PACKET_TX_SCR_RING_0_BASE
#define		PACKET_TX_IN_0_SIZE			QM_TO_PACKET_TX_SCR_RING_0_SIZE
#endif		/* PACKET_TX_IN_0_IND */

#ifndef		PACKET_TX_IN_1_IND
#define		PACKET_TX_IN_1_IND
#define 	PACKET_TX_IN_1				QM_TO_PACKET_TX_SCR_RING_1
#define		PACKET_TX_IN_1_BASE			QM_TO_PACKET_TX_SCR_RING_1_BASE
#define		PACKET_TX_IN_1_SIZE			QM_TO_PACKET_TX_SCR_RING_1_SIZE
#endif

#ifndef		PACKET_TX_IN_2_IND
#define		PACKET_TX_IN_2_IND								
#define		PACKET_TX_IN_2				QM_TO_PACKET_TX_SCR_RING_2
#define		PACKET_TX_IN_2_BASE			QM_TO_PACKET_TX_SCR_RING_2_BASE
#define		PACKET_TX_IN_2_SIZE			QM_TO_PACKET_TX_SCR_RING_2_SIZE
#endif

#ifndef		PACKET_TX_IN_3_IND
#define		PACKET_TX_IN_3_IND
#define 	PACKET_TX_IN_3				QM_TO_PACKET_TX_SCR_RING_3
#define		PACKET_TX_IN_3_BASE			QM_TO_PACKET_TX_SCR_RING_3_BASE
#define		PACKET_TX_IN_3_SIZE			QM_TO_PACKET_TX_SCR_RING_3_SIZE
#endif


#ifdef IXP_MICROCODE

/* Ring number to use. 0-15. */
#ifndef			DL_RING_NUMBER
#define_eval 	DL_RING_NUMBER			CSIX_RX_TO_QM_SCR_RING
#endif

#ifndef			DL_DROP_RING
#define_eval	DL_DROP_RING			CSIX_RX_TO_QM_SCR_RING
#endif

#else	/* Used for micro-c */

/* Ring number to use. 0-15. */
#ifndef			DL_RING_NUMBER
#define		 	DL_RING_NUMBER			CSIX_RX_TO_QM_SCR_RING
#endif

#endif	/* IXP_MICROCODE */

/*
 * Valid values for RING_SIZE are 128, 256, 512 or 1024.
 */
#ifndef		DL_RING_SIZE
#define 	DL_RING_SIZE				CSIX_RX_TO_QM_SCR_RING_SIZE
#endif

/* Base Address for Scratch Ring is 0x1000. Should be aligned 
 * on a 4 x RING_SIZE byte boundary.
 */
#ifndef		DL_RING_BASE
#define 	DL_RING_BASE				CSIX_RX_TO_QM_SCR_RING_BASE
#endif

/*
 * Some definition to keep compatible with legacy code
 */
#ifndef		QM_TO_POS_TX_SCR_RING_0
#define		QM_TO_POS_TX_SCR_RING_0		QM_TO_PACKET_TX_SCR_RING_0
#endif

#ifndef		QM_TO_POS_TX_SCR_RING_1
#define		QM_TO_POS_TX_SCR_RING_1		QM_TO_PACKET_TX_SCR_RING_1
#endif


/*---------------------------------------------------------------------
 * 8) System-wide signals.
 *-------------------------------------------------------------------*/

/* Signals being a precious resource, we use a single signal for 
 * inter-thread as well as any other inter-me signalling
 * (init over signal).
 */
#ifndef		COMMON_INTER_THD_ME_SIGNAL
#define		COMMON_INTER_THD_ME_SIGNAL		10
#endif

/* We are not able to use a single signal before of limitations in
 * assembler. Until then, we need a separate signal for inter-thread
 * signalling.
 */
#ifndef		INTER_THD_SIGNAL
#define		INTER_THD_SIGNAL				12
#endif

/* ME_INIT_SIGNAL signal is used to let all ME's know that system
 * initialization is over and all blocks in all MEs can start running.
 * This signal will be typically issued by the system initialization code
 * and all other MEs should wait for this signal before starting to run. 
 */
#ifndef		ME_INIT_SIGNAL
#define		ME_INIT_SIGNAL					COMMON_INTER_THD_ME_SIGNAL
#endif

/*
 * Signal number to use in inter thread signalling. 
 */
#ifndef		DL_SIG_WAKE
#define 	DL_SIG_WAKE						INTER_THD_SIGNAL
#endif
								

/*---------------------------------------------------------------------
 * 9) System communication ID's and block ID's.
 *-------------------------------------------------------------------*/

/* These tell if the executing threads need to work in order.
 * i.e wait for	a signal from prev thread, do some work and signal
 * next thread. */
#ifndef		DL_THREAD_NO_ORDER
#define		DL_THREAD_NO_ORDER	0x0
#endif

#ifndef		DL_THREAD_ORDER
#define		DL_THREAD_ORDER		0x1
#endif

/* All the IX_* constants should be between 0 and 255 (i.e 1 byte)
 * to enable using them as immed operands in instructions.
 *
 * IX_NULL is used to denote a NULL (or empty) packet handle. 
 * It should not be zero, as it would interfere with ring empty (0)
 * condition.
 * Note: IX_NULL is too generic a name and may clash with core
 *  components code. So it's marked for deletion. Start using UC_NULL.
 */
#ifndef		IX_NULL
#define		IX_NULL				0xFF
#endif

#ifndef		IX_EXCEPTION
#define		IX_EXCEPTION		0x1
#endif

#ifndef		IX_DROP
#define		IX_DROP				0x2
#endif

#ifndef		UC_NULL
#define		UC_NULL				0xFF		/* NULL for microcode */
#endif

#ifndef		DL_BUF_NULL
#define		DL_BUF_NULL			UC_NULL 	/* NULL buffer */
#endif

#ifndef		BID_NULL
#define		BID_NULL			UC_NULL		/* NULL block ID */
#endif

#ifndef		BID_EXCEPTION
#define		BID_EXCEPTION		0x1			/* Exception block ID */
#endif

#ifndef		BID_DROP
#define		BID_DROP			0x2			/* Drop block ID */
#endif

/* Block IDs for various blocks. Block IDs Start from 0x20.
 * (0x0 to 0x1f and 0xFF are reserved for IX_*).
 */
#ifndef		BID_POS
#define		BID_POS				0x21
#endif

#ifndef		BID_IPV4
#define		BID_IPV4			0x22
#endif

#ifndef		BID_QM
#define		BID_QM				0x23
#endif


#endif		// __SYSTEM__H__
