/*
 * ============================================================================
 * COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2003 Intel Corporation. All rights reserved.
 *
 * PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *                                                                          */
/** \file      ix_sa_debug.c
 *
 *  \brief     functions to support debugging.
 *
 */
#define IX_ERROR_FILE_IDENT "$Id: ix_sa_debug.c,v 1.5.2.1 2004/03/12 10:46:33 wneubaue Exp $"


#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include "ix_cc_error.h"
#include "sa/internal/internal_sa.h"
#include "sa/ix_sa_cc_list.h"
#include <ix_ossl.h>
#include <hal_mev2.h>
#include <hal_scratch.h>
#include <dbgMe_def.h>


ix_error
ix_start(ix_int32 bladeID, char* UOF_Path) {
    #if !defined(IX_USE_ENV_VAR)
    strcpy(g_sa_uofFile, UOF_Path);
    #endif
    return ix_sa_create(bladeID);
}

#ifdef IX_DEBUG 

#define XSCALE_SRAM_BASE 	0x80000000
#define XSCALE_DRAM_BASE 	0x02000000


int get_sram_address(int address) {
  return XSCALE_SRAM_BASE + (address & 0x3FFFFFFF) + 0x10000000 * (address >> 30);
}

int get_dram_address(int address) {
  return XSCALE_DRAM_BASE + address;
}

int set_sram(int address, int value)  {
   *((int*) get_sram_address(address)) = value;
   return 1;
}

int set_sram_xscale(int address, int value)  {
   *((int*) address) = value;
   return 1;
}


int set_dram(int address, int value)  {
   *((char*) get_dram_address(address)) = value;
   return 1;
}

int set_dram_xscale(int address, int value)  {
   *((char*) address) = value;
   return 1;
}


int get_sram(int address)  {
   return    *((int*) get_sram_address(address));
}

int get_sram_xscale(int address)  {
   return    *((int*) address);
}


int get_dram(int address)  {
   return    *((int*) get_dram_address(address));
}


int get_dram_xscale(int address)  {
   return    *((int*) address);
}

int init_sram(int value, int add_begin, int add_end) {
   int address = add_begin;
   printf("init_sram(): value = %d,  start address = %d, stop address = %d \n",
          value, (int) get_sram_address(add_begin), (int) get_sram_address(add_end));
   for (;address <= add_end; address = address + 4)
	set_sram(address, value);
   return 1;
};



#define ME_NUM(index) ((index) + ((index) & 0x08))

void
show_scratches ()
{
  ix_uint8        ScratchNo;
  ix_uint32       meAttnMask;
  ix_uint8        MeNo;

  for (ScratchNo = 0; ScratchNo < 11; ScratchNo++)
  {
    printf ("Scratch no %d ", ScratchNo);
    if (SCRATCH_RING_GET_IS_FULL_STATUS (ScratchNo))
      printf (" FULL !!");
    else
      printf ("not full");

    printf (" HEAD %04d    TAIL %04d bytes    Bytes in ring %d \n",
	    GET_SCRATCH_CSR (RING_X_HEAD, ScratchNo),
	    GET_SCRATCH_CSR (RING_X_TAIL, ScratchNo),
	    abs (GET_SCRATCH_CSR (RING_X_TAIL, ScratchNo) -
		 GET_SCRATCH_CSR (RING_X_HEAD, ScratchNo)));
  }

  printf("\n ... and special thanks to RW who led to the creation of this feature ...\n");

  for ( MeNo = 0;  MeNo<MAX_ME; MeNo++) {
     ix_uint32 me;
     if (((MeNo > 3) && (MeNo < 8)) || (MeNo > 11))  continue;
     me = ME_NUM(MeNo);
     printf (" ME %02x  -  PUT = %04d,    GET = %04d,    Bytes in ring %d\n", me, 
             GET_ME_CSR(me, NN_PUT),
             GET_ME_CSR(me, NN_GET),
             abs(GET_ME_CSR(me, NN_PUT) -
                GET_ME_CSR(me, NN_GET)));
  }


}



void show_mem(void* arg_address, int byteNum)
{

  ix_uint8* newAddress;

  for(newAddress = arg_address; byteNum > 0; byteNum -= 16, newAddress += 16)
  {

    printf("0x%p  - ", newAddress);
    printf("%02x ",*(newAddress + 0));
    printf("%02x ",*(newAddress + 1));
    printf("%02x ",*(newAddress + 2));
    printf("%02x ",*(newAddress + 3));
    printf("%02x ",*(newAddress + 4));
    printf("%02x ",*(newAddress + 5));
    printf("%02x ",*(newAddress + 6));
    printf("%02x ",*(newAddress + 7));

    printf(" %02x ",*(newAddress + 8));
    printf("%02x ",*(newAddress + 9));
    printf("%02x ",*(newAddress + 10));
    printf("%02x ",*(newAddress + 11));
    printf("%02x ",*(newAddress + 12));
    printf("%02x ",*(newAddress + 13));
    printf("%02x ",*(newAddress + 14));
    printf("%02x ",*(newAddress + 15));

    printf("\n");
  };

  return;
}




void show_metadata (ix_hw_buffer_meta *pHwMeta)
{
  printf ("Printing the meta ...\n");
  printf ("pHwMeta->m_HwNext       = %X\n", pHwMeta->m_HwNext);
  printf ("pHwMeta->m_BufferSize   = %X\n", pHwMeta->m_BufferSize);
  printf ("pHwMeta->m_Offset       = %X\n", pHwMeta->m_Offset);
  printf ("pHwMeta->m_PacketSize   = %X\n", pHwMeta->m_PacketSize);
  printf ("pHwMeta->m_BufferInfo   = %X\n", pHwMeta->m_BufferInfo);
  printf ("pHwMeta->m_InputPort    = %X\n", pHwMeta->m_InputPort);
  printf ("pHwMeta->m_OutputPort   = %X\n", pHwMeta->m_OutputPort);
  printf ("pHwMeta->m_NextHopID    = %X\n", pHwMeta->m_NextHopID);
  printf ("pHwMeta->m_FabricPort   = %X\n", pHwMeta->m_FabricPort);
  printf ("pHwMeta->m_Reserved1    = %X\n", pHwMeta->m_Reserved1);
  printf ("pHwMeta->m_NextHopIDType= %X\n", pHwMeta->m_NextHopIDType);
  printf ("pHwMeta->m_ColorID      = %X\n", pHwMeta->m_ColorID);
  printf ("pHwMeta->m_FlowID       = %X\n", pHwMeta->m_FlowID);
  printf ("pHwMeta->m_ClassID      = %X\n", pHwMeta->m_ClassID);
  printf ("pHwMeta->m_Reserved2    = %X\n", pHwMeta->m_Reserved2);
  printf ("pHwMeta->m_PacketNext   = %X\n", pHwMeta->m_PacketNext);

  return;
}



#endif
