/*===========================================================================
 * = COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000 -2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 * ===========================================================================
 */

/* =========================================================================*/
/**
 * @file   ix_sa_symbols.c
 *
 * Patches misc symbols for this configuration.
 **/
/*=========================================================================*/
#define IX_ERROR_FILE_IDENT "$Id: ix_sa_symbols.c,v 1.7.2.1 2004/03/12 10:46:34 wneubaue Exp $"


#include "ix_cc_error.h"
#include "sa/internal/internal_sa.h"
#include "sa/ix_sa_cc_list.h"

#if !defined(IX_CONFIGURATION_oc48_pos_ingress)
#error IX_CONFIGURATION_oc48_pos_ingress not defined! Wrong configuration header.
#endif


/*  _ix_sa_patch_symbols */
/*  ---------------------------------------- */
/**
 * Patch miscelanious symbols.
 *
 * This function patches a variety of symbols not specifially associated
 * with a given Core Component.
 *
 * 
 * @param arg_sa_ctrl System Application Control Structure
 * @return IX_SUCCESS, a pass through ix_error, or an ix_error with one
 *         of the following values.
 * @retval NONE
**/
ix_error
_ix_sa_patch_symbols(ix_sa_ctrl *arg_sa_ctrl)
{
    ix_imported_symbol aImportSymbols[7]; 
    ix_uint32 relBase = 0;
    ix_buffer_free_list_info hwFreeListInfo;
    int i = 0;
    void *pZeroBlock;
    ix_uint32 phys_meta_address = 0;
    ix_uint32 phys_data_address = 0;
    ix_memory_type memType;
    ix_uint32 channel = 0;
    ix_uint32 offset = 0;
    ix_uint32 zboffset = 0;
    void *pPhysicalAddress = NULL;
    ix_uint32 temp_meta_address = 0;
    ix_uint32 temp_data_address = 0;

    IX_ERROR_CHECK_ARG_NULL(1, arg_sa_ctrl);

    /* allocate a long work in sram to patch to the zeroblock symbol */
    IX_ERROR_CR(ix_rm_mem_alloc(IX_MEMORY_TYPE_SRAM, 0,
                                sizeof(ix_uint32), &pZeroBlock));


    IX_ERROR_CR(ix_rm_buffer_free_list_get_info(arg_sa_ctrl->freelist[1],
                                                &hwFreeListInfo));

    IX_ERROR_CR(ix_rm_get_phys_offset((void*)hwFreeListInfo.m_pMetaBaseAddress, 
                                      &memType, &channel, &phys_meta_address, 
                                      &pPhysicalAddress)); 

    IX_ERROR_CR(ix_rm_get_phys_offset((void*)hwFreeListInfo.m_pDataBaseAddress, 
                                      &memType, &channel, &phys_data_address, 
                                      &pPhysicalAddress)); 

    IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(0, phys_meta_address);
    IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(0, phys_data_address);

    relBase = phys_data_address - ( phys_meta_address * 
                                     (hwFreeListInfo.m_DataElementSize/hwFreeListInfo.m_MetaElementSize));


    aImportSymbols[0].m_Value = (ix_uint32)((ix_uint32)hwFreeListInfo.m_pMetaBaseAddress & 0x3fffffff);
    aImportSymbols[0].m_Name  = "BUF_SRAM_BASE" ;

    aImportSymbols[1].m_Value = (ix_uint32)hwFreeListInfo.m_FreeListInfo;/* hwFreeList ; */
    aImportSymbols[1].m_Name  = "BUF_FREE_LIST0" ;

    aImportSymbols[2].m_Value =  relBase ;
    aImportSymbols[2].m_Name  = "DL_REL_BASE" ;

    aImportSymbols[3].m_Value = (ix_uint32)hwFreeListInfo.m_FreeListInfo1;
    aImportSymbols[3].m_Name  = "FREE_LIST_ID";

    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[0],4,aImportSymbols));
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[1],3,aImportSymbols));
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[2],3,aImportSymbols));
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[3],2,aImportSymbols));
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[5],3,aImportSymbols));
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[6],3,aImportSymbols));


    IX_ERROR_CR(ix_rm_get_phys_offset((void*)pZeroBlock,
                                      &memType, &channel, &zboffset,
                                      &pPhysicalAddress));

    aImportSymbols[3].m_Value = (ix_uint32)zboffset;
    aImportSymbols[3].m_Name  = "SRAM_ZERO_BLOCK";

    /* patch these symbols on all ME's */
    IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[7],4,aImportSymbols));

    return IX_SUCCESS;
}

