//---------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2001-2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//---------------------------------------------------------------------
///////////////////////////////////////////////////////////////////////
//
//
//      File Name: 	dl_mpls_ipv4.uc
//
//      Purpose: 	Root file for MPLS - IPv4 pipeline.
//
//
///////////////////////////////////////////////////////////////////////
#ifndef	__DL_MPLS_IPv4_UC__
#define __DL_MPLS_IPv4_UC__

//	Include Files
//  include system.h from dispatch loop
#include	"dl_system.h"

#include	"definitions.h"

// include system_init.h from dispatch loop for inter ME signal
#include	"system_init.uc"

// include dispatch_loop.uc from dispatch loop
#include	"dispatch_loop.uc"

#include	"kick_start.uc"

// include MPLS microblock
#include	"mpls_ilm.uc"
#include	"mpls_ftn.uc"

// include IPv4 microblock
#include	"ipv4_fwder.uc"

// include ppp.uc from rx
#include 	"ppp.uc"

// include macros used to cache packet header
#include    "pkthdr_cache.uc"


// define header cache parameters
#define_eval HDR_CACHE     lmem_buf0
#define      HDR_OFFSET    16




//---------------------------------------------------------------------------
// Global Registers
//---------------------------------------------------------------------------

.reg global volatile ctx_rel_lm_addr 		// dispatch loop cache LM address
.reg global volatile ctx_rel_xfer_addr 		// first per ctx xfer address
.reg global volatile base_indirect
.reg global volatile labels_in_cache
.reg global volatile labels_to_write

//-----------------------------------------------------------
// Initialize blocks
//-----------------------------------------------------------

init#:

	// allocate cache to store the packet header
	dl_iphdr_cache_init[]

#ifndef	__PKTHDR
#define	__PKTHDR	$$iphdr	; for convenience. DL defines this.
#else
#warning	"__PKTHDR is already defined."
#endif

	; init buffer meta data cache to Zero.
	init_meta_cache_to_zero[]

	; Temporary : Initialize EOP handle default.
	_dl_eop_buf_handle_init[]

	; header caching initialization
	_dl_hdr_caching_init(HDR_CACHE)

    dl_source_init

	; mpls initialization
	_mpls_init()

	// ipv4fwder initialization
	_ipv4_fwder_init()

	// Now kick start forwarders
	_mpls_ipv4_kick_start()


//-----------------------------------------------------------
// dispatch loop
//-----------------------------------------------------------


.begin
.reg sig_mask
	immed32[sig_mask, 0]

	// get packets in order.
	dl_source[DL_THREAD_ORDER, sig_mask, HDR_CACHE]

.end


main_loop#:

	; classify packet
	_ppp_decap_classify(__PKTHDR,PPP_HEADER_OFFSET)

	; process MPLS packets: LSR and Egress LER
	; HDR_CACHE points to cache buffer
	; HDR_OFFSET indicates where packet header begins in cache buffer
	mpls_ilm(HDR_CACHE, HDR_OFFSET)

	_ipv4_fwder(HDR_CACHE, HDR_CACHE, HDR_OFFSET, HDR_OFFSET)

	; process MPLS packets: Ingress LER
	mpls_ftn(HDR_CACHE, HDR_OFFSET)

	// Now call DL macros to send the current packet
	// and received the next packet. Depending on the dl_next_block, these
	// macros can drop/send exception/send to next block. Also, we need to
	// maintain packet order when we send/receive packets.

.reg sig_mask
.sig sig_scr_put

	// allocate xfer registers to be used by dl_qm_sink.
	xbuf_alloc[$wxfer, 3, write]
	immed32[sig_mask, 0]

	// send packet.
	dl_qm_sink[sig_mask, sig_scr_put, $wxfer, HDR_CACHE]

	// receive packet
	dl_source[DL_THREAD_NO_ORDER, sig_mask, HDR_CACHE]

.io_completed sig_scr_put

	// free the xfer registers allocated
	xbuf_free[$wxfer]

	br[main_loop#]

term#:
; We never execute the below code. This is here for completeness only.

	; frees the IP header cache
	dl_iphdr_cache_fini[]

kill#:
; Control should never come here.

	nop
	nop

#endif __DL_MPLS_IPv4_UC__

