//---------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2001-2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//---------------------------------------------------------------------
///////////////////////////////////////////////////////////////////////
//
//
//      File Name: 	dl_source_ingress_a.uc
//
//      Purpose:        Contains macros for dl_sink and dl_source for
//                      Ingress A microblocks.
//
///////////////////////////////////////////////////////////////////////

#ifndef __DL_SOURCE_UC__
#define __DL_SOURCE_UC__

#include <stdmac.uc>
#include <sig_macros.uc>
#include <dispatch_loop.h>
#include <dl_misc.uc>
#include <xbuf.uc>


/////////////////////////////
//	Some local defines
/////////////////////////////

#if (DL_RING_NUMBER > 11)
#error			"For Ring Number Greater than 11, we cannot use the SCR_Ring#_Full input state. Check RING_NUMBER"
#else
#define_eval	DL_RING_FULL			SCR_Ring/**/DL_RING_NUMBER/**/_Status
#endif

#if (IPV4_TO_QM_SCR_RING > 11)
#error			"For Ring Number Greater than 11, we cannot use the SCR_Ring#_Full input state. Check IPV4_TO_QM_SCR_RING"
#else
#define_eval	DL_IPV4_QM_RING_FULL	SCR_Ring/**/IPV4_TO_QM_SCR_RING /**/_Status
#endif

/////////////////////////////
// Global variables/Registers
/////////////////////////////

.reg 	global	dl_ring					// scratch ring number for pkt rx ==>ipv4. This can be made an absolut register
.reg 	global	dl_buf_handle			// The current Buffer handle.
.reg	global	dl_eop_buf_handle		// For large packets, this is the last buffer in the chain
.reg	global	dl_next_block			// Next block that should process the buffer/packet
.reg    global  dl_exception_reg 		// Exception register used to send pkts to CORE
.set            dl_exception_reg


#ifdef	EXCEPTION_HANDLING_ME
.sig   global   volatile exc_sig  ; signal used to read data from CC->ME exception ring
 xbuf_alloc[$$exc_data, 3, read]  ; data from CC->ME exception ring
#endif 

.reg    global  mpls_ring_in
#ifndef MPLS_COUNTER_FEATURE
.reg    global  mpls_ring_out
#endif
.reg    global  sig_prev_reg

; Global variable for next thread signaling. 
.sig volatile 	sig_prev				
.addr			sig_prev				DL_SIG_WAKE


#macro dl_source_init[]
    #ifdef	EXCEPTION_HANDLING_ME
    .begin
        .reg exc_ring ; CC->ME exception ring
        alu[exc_ring, --,b, EXCEPTION_RING_IN_0, <<2]
        // Get data from scratch ring.
    	// We will check data in the next loop
    	scratch[get, $$exc_data[0], 0, exc_ring, 3], sig_done[exc_sig]
	    // For now execute fast path forwarding
    .end
    #endif

	; Ring from which to consume.
	alu[mpls_ring_in,  --, B, IPV4_RING_IN,  <<2]	
    #ifndef MPLS_COUNTER_FEATURE
    ; ring nos in get/put is always multiple of 4.
	alu[mpls_ring_out, --, b, IPV4_RING_OUT, <<2]
    #endif

	alu[sig_prev_reg, --, B, 0x80]                      ; [7] = 1, NEXT_CONTEXT
	alu[sig_prev_reg, sig_prev_reg, OR, &sig_prev, <<3]	; [6:3] = SIGNAL_NUMBER

#endm



///////////////////////////////////////////////////////////////////////////////
// _dl_pos_sink_init:
//	 	Description: 
//			Initialisation for POS dl_sink. Should be called before using dl_sink.
//			It simply sets a global variable dl_ring to the corresponding ring number
//			This will later save one instruction in dl_sink (for POS).
//
//			[Note: This is an internal macro not meant to be used by the user.
//			This macro will undergo some more changes.]
//
//	 	Outputs:
//			
//		Inputs:
//		
//		Size:   				:
//			1 instruction
//
///////////////////////////////////////////////////////////////////////////////
#macro _dl_pos_sink_init()
.begin
	alu[dl_ring, --, B, DL_RING_NUMBER, <<2]	; ring number to be used in get/put 
.end
#endm


///////////////////////////////////////////////////////////////////////////////
// _dl_eop_buf_handle_init:
//	 	Description: 
//			THIS IS NOT AN EXPORTED MACRO. PLS DO NOT USE IT. THIS WILL CHANGE.
//			THIS IS ONLY TEMPORARY.
//
//	 	Outputs:
//
//		Inputs:
//		
//		Size:   				:
//	
///////////////////////////////////////////////////////////////////////////////
#macro _dl_eop_buf_handle_init[]
.begin
	immed32[dl_eop_buf_handle, IX_NULL]
.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_source:
//	 	Description: 
//
//			This macro is used by IPv4 Block (for now) to get packets and other info
//			from the Packet RX block. It receives them through a scratch ring
//			The following info is available from the ring.
//
//          If the packet fits entirely in the sop buffer the format is
//
//          LW 0 (31:0) : sop buffer handle
//          LW 1 (31:16): input port number
//          LW 1 (15:12): freelist id
//          LW 1 (11:8) : rx stat
//          LW 1 (7:0)  : header type
//          LW 2 (31:16): sop buffer size
//          LW 2 (15:0) : offset in bytes of start of data in sop buffer
//
//          If the packet fits exceeds the sop buffer the format is
//
//          LW 0 (31:0) : sop buffer handle
//          LW 1 (31:0) : eop buffer handle
//          LW 2 (31:16): packet size
//          LW 2 (15:0) : offset in bytes of start of data in sop buffer
//			
//	 	Outputs:
//
//		Inputs:
//			THREAD_ORDER	:	Specifies if the execution of the threads need to be in
//								order. i.e wait for a signal from prev. thread, do some work and 
//								signal next thread. (This thread ordering is required, if the 
//								threads access some shared data, in order to get exclusive access 
//								to that shared data).
//								order. Possible (constant) values are 
//								DL_THREAD_ORDER
//								DL_THREAD_NO_ORDER.
//			
//			sig_mask		:	Signal mask to wait on. This is the bitmask tobe used in a
//								ctx_arb[--] instrcution. (The bitmask contains a set of signals
//								used in previous I/O on which we want to wait here)
//		Size:   			:
//			N/A	
//		
// 		Notes:
//		dl_source is meant to be used along with dl_qm_sink in the following way (only).
//
//		dl_source(DL_THREAD_ORDER, 0)
//		.while (1)
//			do_what_ever()
//			dl_qm_sink(...)
//			dl_source(DL_THREAD_NO_ORDER, ...)
//		.endw
//
//		dl_qm_sink is always ordered, meaning, it will always wait for a signal from a previous
//		thread, do its job and then signal the next thread. Because dl_qm_sink is always ordered,
//		we don't need the dl_source just following it to be ordered. (but the first time we call
//		dl_source it needs to be ordered).
//
//		Performance optimisation: 
//		-------------------------
//		original code:
//		dl_qm_sink signals the next thread after producing on to the scratch ring, which was
//		at the end of that macro. Then dl_source is called (no thread order). At the very top
//		we had a scratch[get...] ctx_swap[..]. This ctx_swap happened too soon (because the signal
//		we sent to the next thread takes about 10 cycles to reach) so the next thread is still
//		not ready to execute. This lead to large "Idle" time in the pipeline affecting throughput.
//		
//		The following changes were made to improve performance.
//		1. In dl_qm_sink we signal the next thread at the first available oppurtunity, even
//		before the scratch[put...]. This is ok because the next thread (in the same ME) cannot
//		run until we sleep. So make sure there is no ctx_swap between this signal and scratch_put.
//
//		2. If the next thread is another ME, then 1 is a wrong solution. So if we have to
//		signal another ME (because we are in thread 7), we postpone the signalling until 
//		later (in dl_source macro). Thus, for thread 7, dl_source will always signal the 
//		next ME.
//
//		3. soon after we make a scratch[put...] in dl_qm_sink, we make scratch[get...] in 
//		dl_source. So we wait for the combined signals (in dl_source macro) rather wait
//		for wait for each of them separately. (This is the reason for having 3 paramters
//		to dl_qm_sink, and the second paramter to dl_source).
//
//		For ease of understanding, look at dl_qm_sink and dl_source as one BIG combined 
//		macro in the while loop.
///////////////////////////////////////////////////////////////////////////////
#macro dl_source(THREAD_ORDER, sig_mask, HDR_CACHE)
.begin
.sig  sig_ip

#ifdef EXCEPTION_HANDLING_ME
	.reg register_offset

	br!=ctx[7, standard_path#]	; Special case for thread 7.

    // Thread 7 checks exception ring from CORE
    .begin
        .reg  exc_ring ; CC->ME exception ring

        // Check if data from exception scratch ring has already been read.
        br_!signal[exc_sig, standard_path#]

        // We check transfer registers in the next loop, so .set must be used 
    	// because of warning (level 3) 4701.
        .set $$exc_data[0]
        .set $$exc_data[1]
        .set $$exc_data[2]

	    alu[--, $$exc_data[0], -, 0]
    	beq[exc_bypass#]

        // Thread 7, So Signal Next ME
    	signal_me[DL_NEXT_ME, DL_SIG_WAKE]

        // Read meta data
    	.sig signal_meta_exc
    	.reg $meta_exc0 $meta_exc1 $meta_exc2 $meta_exc3
    	.reg $meta_exc4 $meta_exc5 $meta_exc6 $meta_exc7
	    .xfer_order $meta_exc0 $meta_exc1 $meta_exc2 $meta_exc3 \
	                $meta_exc4 $meta_exc5 $meta_exc6 $meta_exc7


        dl_meta_load_cache($$exc_data[0], $meta_exc, signal_meta_exc, 0, 8)

    	move(dl_buf_handle,     $$exc_data[0])
    	move(dl_eop_buf_handle, $$exc_data[1])
	    move(dl_next_block,     $$exc_data[2])

        immed32(labels_in_cache, (PKT_HDR_CACHE_SIZE - 1))
        immed32(labels_to_write, 0)

    	ctx_arb[signal_meta_exc]

    	dl_meta_init_cache[$meta_exc[0], $meta_exc[1], $meta_exc[2], $meta_exc[3], \
	                       $meta_exc[4], $meta_exc[5], $meta_exc[6], $meta_exc[7]]

        .begin
            .reg  sdram_offset
            .reg  buf_offset

            // Read 32 bytes of IP Header from SDRAM
            dl_buf_get_data[sdram_offset, dl_buf_handle]
            dl_meta_get_offset[buf_offset]
        
	        dram[read, $$iphdr0, sdram_offset, buf_offset, PKT_HDR_DRAM_RD_SIZE],
                 sig_done[sig_ip]; DRAM reads are in QWORD (8 bytes)

            alu[register_offset, sdram_offset, +, buf_offset]
            alu[register_offset, 7, and, register_offset]

            alu[exc_ring, --,b, EXCEPTION_RING_IN_0, <<2]

            ; Get data from scratch ring.
            ; We will check data in the next loop
            scratch[get, $$exc_data[0], 0, exc_ring, 3], sig_done[exc_sig]

            ctx_arb[--], defer[2], br[copy_hdr_to_lm#]
            ; Add the signal to the signal mask on which to wait
            alu[sig_mask, sig_mask, OR, 1, <<(&sig_ip)]
            local_csr_wr[active_ctx_wakeup_events, sig_mask]

        .end

    exc_bypass#:

        alu[exc_ring, --,b, EXCEPTION_RING_IN_0, <<2]
        // Get data from scratch ring.
    	// We will check data in the next loop
    	scratch[get, $$exc_data[0], 0, exc_ring, 3], sig_done[exc_sig]
	    // For now execute fast path forwarding

    .end

standard_path#:

    alu[register_offset, --, B, PPP_HEADER_LENGTH]


#else

    #define_eval register_offset PPP_HEADER_LENGTH

#endif /* EXCEPTION_HANDLING_ME */



	.sig scr_get

	xbuf_alloc($rdata, 3, read)

	#if	(THREAD_ORDER == DL_THREAD_ORDER)
	; Wait for signal from previous thread to ensure thread ordering
	ctx_arb[sig_prev]
	#endif

	scratch[get, $rdata0, 0, mpls_ring_in, 3], sig_done[scr_get]

	; If thread 7, signal a thread in the next ME.
	br=ctx[7, thread_7#]

	; For threads 0-6, signal next thread in the same ME.
	local_csr_wr[SAME_ME_SIGNAL, sig_prev_reg]
	; We wait for the above scratch[get] and the scratch[put] in dl_qm_sink.
	; See Notes above.
	ctx_arb[--], defer[2], br[check_ring_empty#]
	; Add the signal to the signal mask on which to wait
	alu[sig_mask, sig_mask, OR, 1, <<(&scr_get)]
	local_csr_wr[active_ctx_wakeup_events, sig_mask]

thread_7#:

	// For all other next ME's.
	; Signal thread 0 of the next ME.
	signal_me(DL_NEXT_ME, DL_SIG_WAKE)
	ctx_arb[--], defer[2]
	; Add the signal to the signal mask on which to wait
	alu[sig_mask, sig_mask, OR, 1, <<(&scr_get)]
	local_csr_wr[active_ctx_wakeup_events, sig_mask]

check_ring_empty#:

	; Assembler doesn't know that we actually wait on the I/O (scratch[get])
	.io_completed scr_get

	; Check if ring is empty and set 'dl_buf_handle'
	alu[dl_buf_handle, --, b, $rdata0]							
	beq[ring_empty#]		

	; Set sop buffer offset in meta-data.
	; This field is used by dl_iphdr_load_cache().
	dl_meta_set_offset($rdata2)

	; Read the IP header of the packet and cache it.
	dl_iphdr_load_cache(dl_buf_handle, sig_ip, SIG_NONE)

	// these are default conditions (cache IPv4 header)
	immed32(labels_in_cache, (PKT_HDR_CACHE_SIZE - 1))
	immed32(labels_to_write, 3)

	; Check if the packet fits entirely in the sop buffer.
	; If it does, the eop bit will be set. Bit 31 is the eop bit.
	br_bset[dl_buf_handle, 31, only_sop_buf#]

multi_buf#:

	.begin 

	xbuf_alloc($meta, 4, read)

	.sig sig_sop_meta

	; Read required meta-data from SRAM.
	dl_meta_load_cache(dl_buf_handle, $meta, sig_sop_meta, 1, 3)

	; Set nexthop ID to -1.
	dl_meta_set_nexthop_id(0xffff)

	.reg packet_size
	ld_field_w_clr[packet_size, 0011, $rdata2, >>16]
	dl_meta_set_packet_size(packet_size)

	ctx_arb[sig_ip, sig_sop_meta]

	; Initialize meta-data fields.
	dl_meta_init_cache(0, $meta1, $meta2, $meta3, 0, 0, 0, 0)

	xbuf_free($meta)

	.end


	; Set 'dl_next_block'.
	move(dl_next_block, DL_SOURCE_NEXT1)

	; Set 'dl_eop_buf_handle'.
	alu[dl_eop_buf_handle, --, b, $rdata1]

	br[copy_hdr_to_lm#]



only_sop_buf#:

	; Set meta-data fields whose values will be used if the packet
	; fits entirely in the sop buffer.

    alu[dl_meta[1], --, b, $rdata2]

	; Set freelist_id, rx_stat and header_type.
	ld_field_w_clr[dl_meta[2], 0011, $rdata1]

	; Set packet_size as same as sop_buffer_size.
	ld_field      [dl_meta[2], 1100, $rdata2]

	; Set input_port.
	ld_field_w_clr[dl_meta[3], 1100, $rdata1]

	; Set nexthop ID to -1.
	dl_meta_set_nexthop_id(0xffff)

	; Set 'dl_next_block'.
	move(dl_next_block, DL_SOURCE_NEXT1)

	; Set 'dl_eop_buf_handle'.
	alu[dl_eop_buf_handle, --, b, IX_NULL]

	ctx_arb[sig_ip], br[copy_hdr_to_lm#]


ring_empty#:

	; If ring is empty, just set dl_buf_handle and dl_next_block to IX_NULL;
	move(dl_buf_handle, IX_NULL)
	move(dl_next_block, IX_NULL)
    br[end#]

copy_hdr_to_lm#:
    ; Assembler doesn't know that we actually wait on
    ; dram read completion
    .io_completed sig_ip

	; Clear exception register.
	immed[dl_exception_reg,0x0]

	dl_load_hdr_cache_lm(HDR_CACHE, register_offset)

end#:

	xbuf_free($rdata)

	.set $$iphdr0, $$iphdr1, $$iphdr2, $$iphdr3, $$iphdr4, $$iphdr5

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_sink:
//	 	Description: 
//			Called by Packet RX to handover the packet to the next block (IPv4) in the
//			ingress pipeline. We do not do thread ordering in this macro. (It is done
//			by Packet RX itself because of tight budget)
//			
//			This macro will produce the following on to the scratch ring.
//
//          If the packet fits entirely in the sop buffer the format is
//
//          LW 0 (31:0) : sop buffer handle
//          LW 1 (31:16): input port number
//          LW 1 (15:12): freelist id
//          LW 1 (11:8) : rx stat
//          LW 1 (7:0)  : header type
//          LW 2 (31:16): sop buffer size
//          LW 2 (15:0) : offset in bytes of start of data in sop buffer
//
//          If the packet fits exceeds the sop buffer the format is
//
//          LW 0 (31:0) : sop buffer handle
//          LW 1 (31:0) : eop buffer handle
//          LW 2 (31:16): packet size
//          LW 2 (15:0) : offset in bytes of start of data in sop buffer
//
//			This info will be used by the IPV4 block (by calling dl_source)
//
//			Note:
//			sig_mask is a bit mask of signals to wait on. This is passed in a register.
//			If there is no need to wait for any signal, pass SIG_NONE. 
//
//	 	Outputs:
//
//		Inputs:
//			wxfer_prefix:		SRAM Write Transfer Register name prefix (using xbuf_alloc).
//			req_sig			:	signal to be used in the I/O (scratch put)
//			sig_mask		:	What to do with I/O operation - 
//								- wait for signal(s) as specified by sig_mask (register)
//								- do not wait for signal, just return (SIG_NONE - constant)
//			The following global variables will also act as inputs.
//			dl_next_block		- IX_EXCEPTION, for exception packets
//								- BID_NEXT_BLOCK, block ID of the next block to handle this packet.
//			dl_buf_handle		: Buffer handle of the buffer in the start of chain (containing SOP)
//			dl_buf_eop_handle	: Buffer handle of the last buffer in the chain
//		Size:   				:
//			N/A
//
///////////////////////////////////////////////////////////////////////////////
#macro dl_sink(wxfer_prefix, req_sig, sig_mask)
.begin

	; We sink a packet only on EOP.
	; 'dl_next_block' is set to 'BID_NEXT_BLOCK' on EOP.
	br=byte[dl_next_block, 0, BID_NEXT_BLOCK, sink_it#], defer[2]

	; Set lw's 0 and 2 of the write transfer registers.
	alu[wxfer_prefix/**/0, --, b, dl_buf_handle]
	alu[wxfer_prefix/**/2, --, b, dl_meta[1]]

check_exception#:

	; In the case of an exception packet, it needs to be sent to
	; the core (xscale) through a different ring. Else simply
	; wait for all signals.

	br!=byte[dl_next_block, 0, IX_EXCEPTION, wait_for_signals#]
	
	; Do exception processing here. For now just drop the packet.

drop_pkt#:

	; Drop the packet. Two cases to handle. Single buffer or a bufffer chain.
	; It is determined by looking at the dl_eop_buf_handle

	; If dl_eop_buf_handle == IX_NULL then it is single buffer.
	alu[--, dl_eop_buf_handle, -, IX_NULL]
	beq[drop_single_buffer#]				

	; Otherwise, buffer chain.
	dl_drop_buffer_chain(dl_buf_handle, dl_eop_buf_handle)

	//	XXX - This is hardcoded for port 0 for now. 
	//	To be fixed later for other cases.
	_packet_rx_incr_counter(0, PACKET_PKTS_DROPPED)

	br[wait_for_signals#]

drop_single_buffer#:

	dl_drop_buffer(dl_buf_handle)

	br[wait_for_signals#]

sink_it#:

	; Sink the packet. Check if the ring is full and then produce the data 
	; onto the ring. If ring full, drop the packet and continue.

	; Check if ring is full.
 	br_inp_state[DL_RING_FULL, drop_pkt#]

	; Check if the packet fits entirely in the sop buffer.
	; If it does, the eop bit will be set. Bit 31 is the eop bit.
	br_bset[dl_buf_handle, 31, scr_put#], defer[3]

	; Set lw1 for the case when the packet fits entirely in the sop buffer.
	.begin 
	.reg tmp
	ld_field_w_clr[tmp, 0011, dl_meta[2]]
	ld_field[tmp, 1100, dl_meta[3]]
	alu[wxfer_prefix/**/1, --, b, tmp]
	.end

		; Set lw1 and lw2 for the case when the packet exceeds the sop buffer.

		; Store 'dl_eop_buf_handle' in lw1.
		alu[wxfer_prefix/**/1, --, b, dl_eop_buf_handle]
	
		; Store packet size and sop buffer offset in lw2.
		.begin
		.reg tmp
		ld_field_w_clr[tmp, 0011, dl_meta[1]]
		ld_field[tmp, 1100, dl_meta[2]]
		alu[wxfer_prefix/**/2, --, b, tmp]
		.end

scr_put#:

	; Ring not full. Put the packet on the ring.
	scratch[put, wxfer_prefix/**/0, 0, dl_ring, 3], sig_done[req_sig]

	#if (!streq('sig_mask', 'SIG_NONE'))

	ctx_arb[--], br[end#], defer[2]

	; Add the signal to the signal mask on which to wait.
	alu[sig_mask, sig_mask, OR, 1, <<&req_sig]

	; Wait on all signals within 'sig_maask'.
	local_csr_wr[active_ctx_wakeup_events, sig_mask]

	#endif

wait_for_signals#:

	#if (!streq('sig_mask', 'SIG_NONE'))

	ctx_arb[--], defer[1]
	; Wait on all signals within 'sig_maask'.
	local_csr_wr[active_ctx_wakeup_events, sig_mask]

	#endif

end#:
	
.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_qm_sink:
//	 	Description: 
//			Called by IPv4 microblock to send the packet to the statistics ME.
//			The name of the macro is still kept as dl_qm_sink so that we can
//          re-use the IPv4 code that calls this macro.
//			This macro will do thread ordering always.
//		
//			This macro will produce the following on to the scratch ring.
//
//          LW 0 (31:16): (packet size - sop buffer size) in bytes
//						  ie size of packet outside the sop buffer. 
//                        This data is used by the statistics ME to compute
//                        number of cells in all mop buffers of the packet.
//          LW 0 (15:0) : queue number
//          LW 1 (31:0) : sop buffer handle
//          LW 2 (31:0) : eop buffer handle
//
//	 	Outputs:
//			sig_mask			- if the I/O operation is made (scratch put) then the sig_mask
//								  will have the bit corresponding to sig_scr set. (sig_scr is an
//								  input parameter)
//		Inputs:
//			sig_scr				- Signal to use in the I/O (scracth put ) operation
//			wxfer				- SRAM write xfer regsiter to be used in I/O.
//								  Should have been allocated using xbuf_alloc. 
//								  Atleast 3 regsiters should be allocated.
//
//			The following global variables will also act as inputs.
//			dl_next_block		- IX_EXCEPTION, for exception packets
//								- IX_NULL, for null packets.
//								- BID_NEXT_BLOCK, block ID of the next block to handle this packet.
//			dl_buf_handle		: Buffer handle of the buffer in the start of chain (contianinf SOP)
//			dl_buf_eop_handle	: Buffer handle of the last buffer in the chain
//		
//		Size:   				:
//			N/A
//
//		Notes:
//
//		See notes in dl_source macro
///////////////////////////////////////////////////////////////////////////////
#macro dl_qm_sink(sig_mask, sig_scr, wxfer, HDR_CACHE)
.begin

	.sig sig_meta sig_cache

	xbuf_alloc($meta_cache, 8, write)
	xbuf_alloc($$header,   10, write)

	; Check where this packet should be sent next.
	br!=byte[dl_next_block, 0, BID_NEXT_BLOCK, check_exception#]

normal_packet#:
	
	; Compute number of bytes in the packet outside the sop buffer.
	; This value is passed over the outgoing scratch ring and is
	; used by the statistics ME to compute the number of cells in
	; all mop buffers of the packet.
	.reg packet_size
    .reg sop_buf_size
    .reg mop_eop_buf_size

    alu[--, --, B, labels_to_write]
	beq[skip_write1#]

	; Write back IP hdr to DRAM.
	dl_flush_hdr_cache_lm(sig_cache, HDR_CACHE)
    alu[sig_mask, sig_mask, OR, 1, <<(&sig_cache)]

skip_write1#:
    alu[sig_mask, sig_mask, OR, 1, <<(&sig_prev)]


	; Recompute the sop buffer cell count.
	compute_sop_buf_cframes(dl_buf_handle)

	dl_meta_get_buffer_size(sop_buf_size)
	dl_meta_get_packet_size(packet_size)
	alu[mop_eop_buf_size, packet_size, -, sop_buf_size]

	; The 0th LW of meta data is next buffer pointer and should
	; not be updated when doing dl_meta_flush_cache().
	#define_eval META_SIZE (META_CACHE_SIZE -1)

	; Write back meta-data to SRAM.
	dl_meta_flush_cache($meta_cache, dl_buf_handle, sig_meta, SIG_NONE, 1, META_SIZE)
    alu[sig_mask, sig_mask, OR, 1, <<(&sig_meta)]

	; Wait for the previous I/O as well as signal from previous thread.
    local_csr_wr[ACTIVE_CTX_WAKEUP_EVENTS, sig_mask]
    ctx_arb[--]

	.io_completed sig_cache
	.io_completed sig_meta
	.io_completed sig_prev


check_ring_full#:

 	br_inp_state[DL_IPV4_QM_RING_FULL, ring_is_full#]


	// At this point, ring has enough space. Sink the data.

	move[wxfer[0], dl_buf_handle]						; LW0 on the ring is current buffer hande
	move[wxfer[1], dl_eop_buf_handle]					; LW1 on the ring is the last buffer in the chain,


.reg queue_number

	dl_meta_get_fabric_port[queue_number]				; LW2 on the ring is Queue number for Queue Manager
	alu_shf[queue_number, --, B, queue_number, <<4]		; fabric * 16 is the actual queue number
	alu_shf[wxfer[2], queue_number, OR, 1, <<31]		; The msb is always set to 1, to prevent producing a value of 0 on to the ring.

#ifdef MPLS_COUNTER_FEATURE
    .reg mpls_ring_out
    ; ring nos in get/put is always multiple of 4.
	alu[mpls_ring_out, --, b, IPV4_RING_OUT, <<2]
#endif

	scratch[put, wxfer[0], 0, mpls_ring_out, 3], sig_done[sig_scr]	; Produce 3 Long words on the ring

	; To improve performance, we do not ctx_swap on scratch[put] above. We'll wait
	; on this signal in dl_source macro. (see notes in dl_source macro). However this(dl_qm_sink) 
	; macro has multiple paths in which we do not issue this scratch[put] at all. 
	; e.g exception packets, or NULL packets etc. So if we issue scratch[put], 
	; we set the sig_mask to indicate that.

	; Add the signal to the signal mask on which to wait
	alu[sig_mask, --, B, 1, <<(&sig_scr)]

	br[end#]


ring_is_full#:

	; To set break point
	nop
	br[check_ring_full#]

	; Check for exception packets. Exception packets are sent to the core (Xscale)
	; on a separate ring. 

check_exception#:

	; This macro can be seen in two parts. The normal packet case
	;  (which is the critical path) and other(drop, null pkt etc) cases.

	; Recompute the sop buffer cell count.
	compute_sop_buf_cframes(dl_buf_handle)

    .reg local_sig_mask
    alu[local_sig_mask, --, B, 1, <<(&sig_prev)]

	br!=byte[dl_next_block, 0, IX_EXCEPTION, check_drop#]

	; The 0th LW of meta data is next buffer pointer and should
	; not be updated when doing dl_meta_flush_cache().
	#define_eval META_SIZE (META_CACHE_SIZE -1)

	; Write back meta-data to SRAM.
	dl_meta_flush_cache[$meta_cache, dl_buf_handle, sig_meta, SIG_NONE, 1, META_SIZE]
    alu[local_sig_mask, local_sig_mask, OR, 1, <<(&sig_meta)]

    alu[--, --, B, labels_to_write]
	beq[skip_write2#]

	; Write back IP hdr to DRAM.
	dl_flush_hdr_cache_lm(sig_cache, HDR_CACHE)
    alu[local_sig_mask, local_sig_mask, OR, 1, <<(&sig_cache)]

skip_write2#:

	; Wait for the previous I/O as well as signal from previous thread.
    local_csr_wr[ACTIVE_CTX_WAKEUP_EVENTS, local_sig_mask]
    ctx_arb[--]

	.io_completed sig_cache
	.io_completed sig_meta
	.io_completed sig_prev


#ifdef USE_IMPORT_VAR
    ; This code will send pkts to the CORE on two rings based on priority.
    dl_exception_send(dl_buf_handle)
#endif


	;  Signal next context.
;	br=ctx[7, skip_signaling1#]	; Special case for thread 7.
;	local_csr_wr[SAME_ME_SIGNAL, sig_prev_reg]

;skip_signaling1#:


#ifndef USE_IMPORT_VAR

	; Drop the packet. Two cases to handle. Single buffer or a bufffer chain.
	; It is determined by looking at the dl_eop_buf_handle

	alu[--, dl_eop_buf_handle, -, IX_NULL]					; if dl_eop_buf_handle == IX_NULL
	beq[drop_single_buffer#]								; then it is single buffer.

	dl_drop_buffer_chain[dl_buf_handle, dl_eop_buf_handle]	; otherwise, buffer chain.

drop_single_buffer#:
	dl_drop_buffer[dl_buf_handle]

#endif

	br[end#]


check_drop#:

	; Wait for signal from prev thread and then signal next context.
	
	ctx_arb[sig_prev]

	; Signal next context.

;	br=ctx[7, skip_signaling2#]								; Special case for thread 7.
	; !7, so signal the next thread
;  	local_csr_wr[SAME_ME_SIGNAL, sig_prev_reg]

;skip_signaling2#:

	br!=byte[dl_next_block, 0, IX_DROP, end#]

	; Drop the packet. Two cases to handle. Single buffer or a bufffer chain.
	; It is determined by looking at the dl_eop_buf_handle

	alu[--, dl_eop_buf_handle, -, IX_NULL]					; if dl_eop_buf_handle == IX_NULL
	beq[drop_single_buffer_1#]								; then it is single buffer.

	dl_drop_buffer_chain[dl_buf_handle, dl_eop_buf_handle]	; otherwise, buffer chain.
	br[end#]

drop_single_buffer_1#:

	dl_drop_buffer[dl_buf_handle]

end#:

	xbuf_free($$header)
	xbuf_free($meta_cache)

.end
#endm

#macro copy_packet(buf_handle)
.begin
.reg dram_base
.reg dram_base_dest
.reg dram_offset
.reg dram_offset_dest
.reg counter

	dl_buf_get_data[dram_base, buf_handle]
	dl_meta_get_offset[dram_offset]

    immed32(dram_base_dest,  0x8000000);
    immed32(dram_offset_dest,0x0000000);

    immed32(counter,         0x0000005);

.reg $$packet0 $$packet1 $$packet2 $$packet3
.reg $$packet4 $$packet5
.xfer_order $$packet0 $$packet1 $$packet2 $$packet3 $$packet4 $$packet5
loop#:

    .sig read_sig
	dram[read, $$packet0,  dram_base, dram_offset, 3], sig_done[read_sig]
    ctx_arb[read_sig];

    alu[$$packet0, --, B, $$packet0]
    alu[$$packet1, --, B, $$packet1]
    alu[$$packet2, --, B, $$packet2]
    alu[$$packet3, --, B, $$packet3]
    alu[$$packet4, --, B, $$packet4]
    alu[$$packet5, --, B, $$packet5]

    .sig write_sig
	dram[write, $$packet0, dram_base_dest, dram_offset_dest, 3], sig_done[write_sig]
    ctx_arb[write_sig];

    alu[dram_offset, dram_offset, +, 32]
    alu[dram_offset_dest, dram_offset_dest, +, 32]

    alu[counter, counter, -, 1]
    bne[loop#]


.end
#endm


#endif	// __DL_SOURCE_UC__