/*------------------------------------------------------------------------------------
//
//                   I N T E L   P R O P R I E T A R Y
//
//      COPYRIGHT (c)  2001-2003 BY  INTEL  CORPORATION.  ALL RIGHTS
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT
//      THE PRIOR WRITTEN PERMISSION OF :
//
//                         INTEL  CORPORATION
//
//                      2200 MISSION COLLEGE BLVD
//
//                SANTA  CLARA,  CALIFORNIA  95052-8119
//
//------------------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: 	dl_system.h
//
//      Purpose:        This file contains the system wide parameters that can be
//                      customised for a given hardware/application
//
////////////////////////////////////////////////////////////////////////////*/
#ifndef	__SYSTEM__H__
#define	__SYSTEM__H__

#ifndef		USE_IMPORT_VAR
#define		SIMULATION
#endif

/*
 *****************************************************************************
 * Include global configuration files
 */
#include <system.h>

#ifndef IXP_SCRIPTS
   #include "ix_cc_microengines_bindings.h"
#endif

/*
 *****************************************************************************
 * C A U T I O N
 *
 * The code below may not be changed by the user. If any of the below stuff
 * has to be overwritten, redefine it in a separate file and then include this
 * file.
 *****************************************************************************
*/

/*
 *****************************************************************************
 * Default chip
 *****************************************************************************
 */
#ifndef IXP2400
#define IXP2400
#endif

/*
 *****************************************************************************
 * Default microcode based project
 *****************************************************************************
 */
#ifndef IXP_MICROCODE
#define IXP_MICROCODE
#endif

/*
 *****************************************************************************
 * Default RX mode
 *****************************************************************************
 */
#ifndef		RX_PHY_MODE
#define 	RX_PHY_MODE 		SPHY_1_32
#endif

/*
 *****************************************************************************
 * Microblock specific configuration
 *****************************************************************************
 */

/*
 *****************************************************************************
 * Queue Manager
 */

/*	Size of each Queue Descriptor Entry (in Bytes) */
#ifndef		QD_SIZE
#define		QD_SIZE									16
#endif

/*	First 16 (0-15) SRAM Q-Array entries are allocated for the Queue Manager */
#ifndef		QM_QARRAY_BASE
#define		QM_QARRAY_BASE							0
#endif

#ifndef		QM_QARRAY_SIZE
#define		QM_QARRAY_SIZE							16
#endif

/*	Queue Number for Drop Queue. (0-1023 are the normal queues used
 *	in forwarding pkts) */
#ifndef		QM_DROP_QUEUE
#define		QM_DROP_QUEUE							1024
#endif

/*
 *****************************************************************************
 * CSIX Tx microblock
 */
/*	Size of each CSIX Transmit Context (in Bytes)  */
#ifndef		CSIX_TX_CTX_SIZE
#define		CSIX_TX_CTX_SIZE						64
#endif

/*	Total number of CSIX Transmit Contexts. */
#ifndef		CSIX_TX_CTX_TOTAL
#define		CSIX_TX_CTX_TOTAL						1024
#endif

/*
 *****************************************************************************
 * Packet Rx microblock
 */

/*	Base address POS Receive Contexts in Local Memory */
#ifndef		POS_RX_CTX_LM_BASE
#define		POS_RX_CTX_LM_BASE						0
#endif

/*	Size of each POS Receive Context (in Bytes) */
#ifndef		POS_RX_CTX_SIZE
#define		POS_RX_CTX_SIZE							64
#endif

/*	Total number of POS Receive Contexts. */
#ifndef		POS_RX_CTX_TOTAL
#define		POS_RX_CTX_TOTAL						16
#endif

/*
 *****************************************************************************
 * MPLS microblock
 */

#ifdef	USE_IMPORT_VAR
	/*
	 * these values will be patched by core
	 */

.import_var MPLS_ILM_TABLE_BASE
.import_var MPLS_FTN_TABLE_BASE
.import_var MPLS_ILM_NHLFE_SET_TABLE_BASE
.import_var MPLS_ILM_PARAM_TABLE_BASE

#ifdef __COUNTERS__
.import_var MPLS_ILM_STAT_BASE
.import_var MPLS_ILM_NHLFE_SET_STAT_BASE
.import_var MPLS_INPUT_PORT_STAT_BASE
.import_var MPLS_FTN_STAT_BASE
#endif


#else /* no import vars */
	/*
	 * we will hardcode the values here
	 */
/*
 *	MPLS forwarding tables bases
 */
#ifndef MPLS_FTN_TABLE_BASE
#define MPLS_FTN_TABLE_BASE							0x40055A20
#endif

#ifndef MPLS_ILM_TABLE_BASE
#define MPLS_ILM_TABLE_BASE							0x000956BC
#endif

#ifndef MPLS_ILM_NHLFE_SET_TABLE_BASE
#define MPLS_ILM_NHLFE_SET_TABLE_BASE				0x40055A20
#endif

#ifndef MPLS_ILM_PARAM_TABLE_BASE
#define MPLS_ILM_PARAM_TABLE_BASE					0x00095578
#endif

/*
 *	MPLS counters bases
 */

#ifdef __COUNTERS__
#define 	MPLS_ILM_STAT_BASE				        0x4008D3BC
#define 	MPLS_ILM_NHLFE_SET_STAT_BASE   			0x400566A0
#define 	MPLS_INPUT_PORT_STAT_BASE   			0x40054920
#define 	MPLS_FTN_STAT_BASE				        0x400566A0
#endif

#endif

#ifdef __COUNTERS__
    #define MPLS_STATS_FULL
#endif


/*
 *****************************************************************************
 * Some System SRAM and SDRAM Memory Related Defines. This stuff is common
 * to both Microcode and MicroC implementations.
 *****************************************************************************
 */
/* Include default settings. */
#define HDR_CACHE_LM_BASE							512
#define HDR_CACHE_LM_SIZE							64

#define LM_LABEL_SPACE_ADDR     					1024
#define INTERFACE_MAX           					16

/*
 * Define Label Space Type
 * PER_PLATFORM_LABEL_SPACE (default) or PER_INTERFACE_LABEL_SPACE
 */
#ifndef PER_INTERFACE_LABEL_SPACE
    #ifndef PER_PLATFORM_LABEL_SPACE
	    #define PER_PLATFORM_LABEL_SPACE
    #endif
#endif

/*	Amount of SRAM space available for (meta data of) packet buffers. 1 MB.
 *	In simulation we use only 10K bytes.
 */
#ifndef		BUF_SRAM_SIZE
	#ifdef      MPLS_HARDWARE_CONFIGURATION
		#define		BUF_SRAM_SIZE					(1024 * 512)
	#else
		#define		BUF_SRAM_SIZE					(1024 * 10)
	#endif
#endif

/*	Amount of SDRAM space available for Packet buffers. 64 MB.
 *	In simulation we use only 640K bytes.
 */
#ifndef		BUF_SDRAM_SIZE
	#ifdef      MPLS_HARDWARE_CONFIGURATION
		#define		BUF_SDRAM_SIZE					(64 * 1024 * 512)
	#else
		#define		BUF_SDRAM_SIZE					(64 * 1024 * 10)
	#endif
#endif

/*	Size of Meta Data (Buffer Descriptors) in SRAM. 32 entries per buffer.
 * 	Should be a power of 2.
 */
#ifndef		META_DATA_SIZE
#define		META_DATA_SIZE							32
#endif

/*	Size of packet buffer.  */
#ifndef		BUFFER_SIZE
#define		BUFFER_SIZE								2048
#endif

/* This defines the number of queues/class per port */
#ifndef		NUM_CLASS_BITS
#define 	NUM_CLASS_BITS							4
#endif




#ifdef	USE_IMPORT_VAR
	/*
	 * these values will be patched by core
	 */

.import_var BUF_SRAM_BASE
.import_var DL_REL_BASE 
.import_var BUF_FREE_LIST0


#else /* no import vars */
	/*
	 * we will hardcode the values here
	 */

/*
 * Base address in SRAM for the packet buffers. This is for the meta data
 * (buffer descriptors). */
#ifndef		BUF_SRAM_BASE
#define		BUF_SRAM_BASE							0x00008520
#endif

/*
 * Base Address in DRAM for the packet buffers. This is for the actual
 * packet data. */
#ifndef		BUF_SDRAM_BASE
#define		BUF_SDRAM_BASE							0x0046E8A0
#endif

/*
 * Base Address for Queue Descriptors. (channel 0 [31:30]) */
#ifndef		QD_SRAM_BASE
#define		QD_SRAM_BASE							0x000B727C
#endif

/*	Total number of Queue Descriptors */
#ifndef		QD_TOTAL
#define		QD_TOTAL								1024
#endif

/*	Base address CSIX Transmit Contexts in SRAM (channel 0 [31:30]) */
#ifndef		CSIX_TX_CTX_SRAM_BASE
#define		CSIX_TX_CTX_SRAM_BASE					0x000A317C
#endif

#ifndef		NEXTHOP_TABLE_SRAM_BASE
#define		NEXTHOP_TABLE_SRAM_BASE					0x40000500
#endif

#ifndef		NUM_NEXTHOPS
#define		NUM_NEXTHOPS							512
#endif

#ifndef		SIZE_OF_NEXTHOP_LW
#define		SIZE_OF_NEXTHOP_LW						4
#endif

#ifndef		NEXTHOP_TABLE_SDRAM_BASE
#define		NEXTHOP_TABLE_SDRAM_BASE				0x4000000
#endif

/*	Each Next Hop entry is 16 bytes. So 4K is sufficient */
#ifndef		NEXTHOP_TABLE_SIZE
#define		NEXTHOP_TABLE_SIZE						0x1000
#endif

/*	Trie Table in SRAM (channel 1 [31:30]) */
#ifndef		TRIE_TABLE_SRAM_BASE
#define		TRIE_TABLE_SRAM_BASE					0x40001510
#endif

/*	Trie Table size. */
#ifndef		TRIE_TABLE_SIZE
#define		TRIE_TABLE_SIZE							0x50600
#endif

#ifndef		SIZE_OF_TRIE_SET
#define		SIZE_OF_TRIE_SET						16
#endif

#ifndef 	NUM_TRIE_SETS
#define		NUM_TRIE_SETS							1024
#endif

#ifdef IPV4_COUNTERS_SCRATCH

	/* Base for IPv4 32bit counters. This counter table is placed in
	*  Scratch @ 11KB to improve performance.
	*/
	#ifndef		IPV4_STATS_TABLE_BASE
	#define         IPV4_STATS_TABLE_BASE			11264
	#endif

	/* space allocated to maintain these counters */
	#ifndef		IPV4_STATS_TABLE_SIZE
	#define         IPV4_STATS_TABLE_SIZE			0x800
	#endif

#else
	/*
	 * counters in SRAM
	 */
	#ifndef		STATS_TABLE_SRAM_BASE
	#define         STATS_TABLE_SRAM_BASE			0x3800
	#endif

	/* space allocated to maintain these counters */
	#ifndef		STATS_TABLE_SRAM_SIZE
	#define         STATS_TABLE_SRAM_SIZE			0x800
	#endif


#endif  /* IPV4_COUNTERS_SCRATCH */

/*	Base for POS RX counters */
#ifndef		PACKET_COUNTERS_SRAM_BASE
#define		PACKET_COUNTERS_SRAM_BASE				0x000B717C
#endif

/*  Directed broadcast table base in SRAM */
#ifndef		DBCAST_TABLE_SRAM_BASE
#define     DBCAST_TABLE_SRAM_BASE					0x40052120
#endif

/* directed broadcast table size */
#ifndef		DBCAST_TABLE_SIZE
#define     DBCAST_TABLE_SIZE						0x2000
#endif

#ifndef		DBCAST_NUM_BLOCKS
#define		DBCAST_NUM_BLOCKS						256
#endif

/* Size of dbcast table entry in long words */
#ifndef		DBCAST_TABLE_BLOCK_SIZE
#define		DBCAST_TABLE_BLOCK_SIZE					8
#endif

/* control block */
#ifndef		CONTROL_BLOCK_SRAM_BASE
#define		CONTROL_BLOCK_SRAM_BASE					0x40052110
#endif

/*	Next 4 (16-19) SRAM Q-Array entries are allocated for buffer free list */
#ifndef		BUF_QARRAY_BASE
#define		BUF_QARRAY_BASE							0x11
#endif

#ifndef		BUF_QARRAY_SIZE
#define		BUF_QARRAY_SIZE							4
#endif

/*	Next 1 (20) SRAM Q-Array entry is used for Drop Queue in Queue Manager. */
#ifndef		QM_DROP_QUEUE_ENTRY
#define		QM_DROP_QUEUE_ENTRY						0x20
#endif

/* Counter base for CSIX TX */
#ifndef		CSIX_TX_COUNTERS_SRAM_BASE
#define   	CSIX_TX_COUNTERS_SRAM_BASE   			0x000B317C
#endif

/*  Area of SRAM which is zero'ed out */
#ifndef		SRAM_ZERO_BLOCK
#define     SRAM_ZERO_BLOCK 						0x00095520
#endif

/* free list id that is stored in meta data */
#ifndef		FREE_LIST_ID
#define     FREE_LIST_ID                    		1
#endif

/*	BLADE ID */
#ifndef		THIS_BLADE_ID
#define		THIS_BLADE_ID							1
#endif


#endif /* USE_IMPORT_VAR */



/*
 ****************************************************************
 *	Some System Scratch Ring Related Defines
 ****************************************************************
 */


/*  There are two exception ring to send pkts up */
#ifndef		EXCEPTION_RING_OUT_0
#define         EXCEPTION_RING_OUT_0		0
#endif

#ifndef		EXCEPTION_RING_OUT_1
#define         EXCEPTION_RING_OUT_1		1
#endif


/*  There are two exception ring to send pkts down */
#ifndef		EXCEPTION_RING_IN_0
#define         EXCEPTION_RING_IN_0			2
#endif

#ifndef		EXCEPTION_RING_IN_1
#define         EXCEPTION_RING_IN_1			3
#endif


/*	Scratch for communicating between POS RX and IPv4 Blocks */
#ifndef		POS_RX_TO_IPV4_SCR
#define		POS_RX_TO_IPV4_SCR
#define		POS_RX_TO_IPV4_SCR_RING			4
#define		POS_RX_TO_IPV4_SCR_RING_BASE	0
#define		POS_RX_TO_IPV4_SCR_RING_SIZE	1024
#endif

/*	Scratch for communicating between IPV4 and Queue Manager Blocks */
#ifndef		IPV4_TO_QM_SCR
#define		IPV4_TO_QM_SCR
#define		IPV4_TO_QM_SCR_RING				5
#define		IPV4_TO_QM_SCR_RING_BASE		4096
#define		IPV4_TO_QM_SCR_RING_SIZE		512
#endif

/*	Scratch for communicating between Scheduler and Queue Manager Blocks */
#ifndef		SCHEDULER_TO_QM_SCR
#define		SCHEDULER_TO_QM_SCR
#define		SCHEDULER_TO_QM_SCR_RING		6
#define		SCHEDULER_TO_QM_SCR_RING_BASE	6144
#define		SCHEDULER_TO_QM_SCR_RING_SIZE	512
#endif

/*	Scratch for communicating between Queue Manager and CSIX Transmit Blocks */
#ifndef		QM_TO_CSIX_TX_SCR
#define		QM_TO_CSIX_TX_SCR
#define		QM_TO_CSIX_TX_SCR_RING			7
#define		QM_TO_CSIX_TX_SCR_RING_BASE		8192
#define		QM_TO_CSIX_TX_SCR_RING_SIZE		512
#endif


/* These defines are used by the individual microcode to identify their scratch rings.
 * The reason for these defines is to improve the portability of microblocks. User just
 * has to make changes here and the microblocks will remain untouched.
 * For the naming convention, the microblock name goes first, then the _RING_ specifier.
 * Following this is the direction indicator. '_OUT' would imply the microblock is writing
 * to this scratch ring and '_IN' would imply the microblock is reading from this scratch
 * ring. If there is more than one 'IN' or 'OUT' ring the name will have '_<digit>' to
 * differentiate the rings.
 *
 * An '_IND' at the end indicates the compilation switch.
 */


/*	The Ring on which POS RX sends out msg to next block */
#ifndef		POS_RX_RING_OUT_IND
#define		POS_RX_RING_OUT_IND
#define 	POS_RX_RING_OUT				POS_RX_TO_IPV4_SCR_RING
#define		POS_RX_RING_OUT_BASE		POS_RX_TO_IPV4_SCR_RING_BASE
#define		POS_RX_RING_OUT_SIZE		POS_RX_TO_IPV4_SCR_RING_SIZE
#endif

/*	The input ring ring on which IPV4 receives msgs from prev block */
#ifndef		IPV4_RING_IN_IND
#define		IPV4_RING_IN_IND
#define		IPV4_RING_IN				POS_RX_TO_IPV4_SCR_RING
#define		IPV4_RING_IN_BASE			POS_RX_TO_IPV4_SCR_RING_BASE
#define		IPV4_RING_IN_SIZE			POS_RX_TO_IPV4_SCR_RING_SIZE
#endif

/* The ring on which IPV4 sends out msg to next block (Queue Manager in this case) */
#ifndef		IPV4_RING_OUT_IND
#define		IPV4_RING_OUT_IND
#define		IPV4_RING_OUT				IPV4_TO_QM_SCR_RING
#define		IPV4_RING_OUT_BASE			IPV4_TO_QM_SCR_RING_BASE
#define		IPV4_RING_OUT_SIZE			IPV4_TO_QM_SCR_RING_SIZE
#endif

/* The input ring on which QM receives msgs from prev. block */
#ifndef		QM_RING_IN_IND
#define		QM_RING_IN_IND
#define		QM_RING_IN					IPV4_TO_QM_SCR_RING
#define		QM_RING_IN_BASE				IPV4_TO_QM_SCR_RING_BASE
#define		QM_RING_IN_SIZE				IPV4_TO_QM_SCR_RING_SIZE
#endif

/*	The output ring in which QM Sends msgs to next block (CSIX TX in this case) */
#ifndef		QM_RING_OUT_IND
#define		QM_RING_OUT_IND
#define		QM_RING_OUT					QM_TO_CSIX_TX_SCR_RING
#define		QM_RING_OUT_BASE			QM_TO_CSIX_TX_SCR_RING_BASE
#define		QM_RING_OUT_SIZE			QM_TO_CSIX_TX_SCR_RING_SIZE
#endif

/*	The input ring in which CSIX TX receives msgs from QM */
#ifndef		CSIX_TX_RING_IN_IND
#define		CSIX_TX_RING_IN_IND
#define		CSIX_TX_RING_IN				QM_TO_CSIX_TX_SCR_RING
#define		CSIX_TX_RING_IN_BASE		QM_TO_CSIX_TX_SCR_RING_BASE
#define		CSIX_TX_RING_IN_SIZE		QM_TO_CSIX_TX_SCR_RING_SIZE
#endif



/****************************************************************
 *	Some System Communication ID's and BlockID's Related Defines
 ****************************************************************
 */


/*	All the IX_* constants should be between 0 and 255 (i.e 1 byte) to enable using
 *	them as immed operands in instructions.
 *
 *	IX_NULL is used to denote a NULL (or empty) packet handle, It should not be zero,
 *	as it would interfere with ring empty (0) condition.
 *	Note: IX_NULL is too generic a name and may clash with core components code.
 *	So it's marked for deletion.  Start using UC_NULL
 */

#define         IX_NULL                 255
#define         IX_EXCEPTION            1
#define         IX_DROP                 2

#define         UC_NULL                 255        /* NULL for microcode         */
#define         DL_BUF_NULL             UC_NULL    /* NULL Buffer representation */


#define         BID_NULL                UC_NULL	   /* NULL Block ID */
#define         BID_EXCEPTION           1          /* Exception Block ID */
#define         BID_DROP                2          /* Drop Block ID      */



/*
 *****************************************************************************
 * Microblock Block IDs and their Bindings
 *****************************************************************************
 */
/*	Block IDs for various blocks. Block IDs Start from 0x20. (0 to 0x1f and 0xFF are
 *	reserved for IX_* )
 */
#ifndef		BID_POS
#define		BID_POS			IX_CC_POS_RX_PKT_ID
#endif

#ifndef		BID_IPV4
#define		BID_IPV4		IX_CC_IPV4_PKT_MICROBLOCK_ID
#endif

#ifndef		BID_IPV6
#define		BID_IPV6        BID_DROP
#endif

#ifndef		BID_QM
#define		BID_QM			IX_CC_QM_CORE_TO_MICROBLOCK_ID
#endif


#ifndef		BID_MPLSILM         
#define		BID_MPLSILM     IX_CC_MPLS_ILM_PKT_MICROBLOCK_ID
#endif


#ifndef		BID_MPLSFTN
#define		BID_MPLSFTN     IX_CC_MPLS_FTN_PKT_MICROBLOCK_ID
#endif


/* 	Static Bindings (a.k.a targets) for various microblocks. */

#ifdef		ETHER_WITHOUT_L2
	#ifndef		DL_SOURCE_NEXT1
	#define		DL_SOURCE_NEXT1			BID_ETHER
	#endif
#else

	#ifdef		PPP_RECEIVE
		#ifndef		DL_SOURCE_NEXT1
		#define		DL_SOURCE_NEXT1			BID_POS
		#endif
	#else
		#ifndef		DL_SOURCE_NEXT1
		#define		DL_SOURCE_NEXT1			BID_IPV4
		#endif

	#endif
#endif

#ifndef		POS_RX_NEXT1
#define		POS_RX_NEXT1			BID_IPV4
#endif

#ifndef		IPV4_NEXT1
#define	    IPV4_NEXT1				BID_MPLSFTN
#endif

#ifndef     IPV4_LOCAL_NEXT1
#define     IPV4_LOCAL_NEXT1        IX_EXCEPTION
#endif

#ifndef		RX_NEXT1
#define		RX_NEXT1				BID_POS
#endif

#ifdef		BID_MPLSFTN_NEXT1
#undef 		BID_MPLSFTN_NEXT1
#endif
#define		BID_MPLSFTN_NEXT1		BID_QM

#ifdef		BID_MPLSILM_NEXT1
#undef		BID_MPLSILM_NEXT1
#endif
#define		BID_MPLSILM_NEXT1		BID_QM

#ifdef		BID_MPLSILM_NEXT2
#undef		BID_MPLSILM_NEXT2
#endif
#define		BID_MPLSILM_NEXT2		BID_IPV4


#ifdef		BID_MPLSILM_NEXT3
#undef		BID_MPLSILM_NEXT3
#endif
#define		BID_MPLSILM_NEXT3		BID_IPV6

/*	To be Redefined as an import variable. And its value should be one of the above
 *	Block IDs.
 */

#ifndef		BID_NEXT_BLOCK
#define		BID_NEXT_BLOCK			BID_QM
#endif

/* Define next block id's for PPP decap */
#define	BID_PPP_DECAP_NEXT1		BID_IPV4
#define	BID_PPP_DECAP_NEXT2		BID_MPLSILM
#define	BID_PPP_DECAP_NEXT3		BID_IPV6


/****************************************************************
 *	             Some System Signal Related Defines
 ****************************************************************
 */

/*	Signals being a precious resource, we use a single signal for
 *	inter-thread as well as any other inter-me signalling (init over signal)
 */
#ifndef		COMMON_INTER_THD_ME_SIGNAL
#define		COMMON_INTER_THD_ME_SIGNAL		10
#endif

/*	We are not able to use a single signal before of limitations in assembler.
 *	until then, we need a separate signal for inter-thread signalling.
 */
#ifndef		INTER_THD_SIGNAL
#define		INTER_THD_SIGNAL				12
#endif

/*	ME_NIT_SIGNAL signal is used to let everyone know that initialisation is over
 *	and all blocks in all MEs can start running. This signal will be typically issued
 *	by the system initialisation code and all other MEs should wait for this signal
 *	before starting to run.
 */
#ifndef		ME_INIT_SIGNAL
#define		ME_INIT_SIGNAL					COMMON_INTER_THD_ME_SIGNAL
#endif



/****************************************************************
 *	             Source and Sink Related Defines
 ****************************************************************
 */

/*	specify if next neighbor ring is used. If not, default is scratch rings.
 *	NOTE: DL_RING_* is now obsolete. Will be changed to use POS_RX_RING_IN,
 *	which is defined somwehere above.
 */

/* 	Valid values for RING_SIZE are 128, 256, 512 or 1024. */
#ifndef		DL_RING_SIZE
#define 	DL_RING_SIZE		POS_RX_TO_IPV4_SCR_RING_SIZE
#endif

/* 	Base Address for Scratch Ring is 0x1000. Should be aligned
 * on a 4 x RING_SIZE byte boundary.
 */
#ifndef		DL_RING_BASE
#define 	DL_RING_BASE		POS_RX_TO_IPV4_SCR_RING_BASE
#endif

/*
 * Default size for PKT HDR to be cached in long words.
 */

#ifndef	PKT_HDR_CACHE_SIZE
#define	PKT_HDR_CACHE_SIZE		6	// 6 longwords
#endif

/* define evals are only used in microcode */

/* 	Ring number to use. 0-15. */
#ifndef DL_RING_NUMBER
#ifndef IXP_SCRIPTS
#define_eval DL_RING_NUMBER			POS_RX_TO_IPV4_SCR_RING
#else
#define 	 DL_RING_NUMBER			POS_RX_TO_IPV4_SCR_RING
#endif
#endif

#ifndef DL_DROP_RING
#ifndef IXP_SCRIPTS
#define_eval DL_DROP_RING			IPV4_TO_QM_SCR_RING
#else 
#define	     DL_DROP_RING			IPV4_TO_QM_SCR_RING
#endif
#endif

/* 	Signal number to use in inter thread signalling. */
#ifndef		DL_SIG_WAKE
#define 	DL_SIG_WAKE			INTER_THD_SIGNAL
#endif

/*	These tell if a the executing threads need to work in order. i.e wait for
 *	a signal from prev thread, do some work and signal next thread.
 */

#define		DL_THREAD_NO_ORDER		0x0
#define		DL_THREAD_ORDER			0x1

/*	The DDR (Double Data Rate DRAM) bank size is used to randomize buffer offset. */

#ifndef		DDR_BANK_SIZE
#define		DDR_BANK_SIZE			128
#endif
/*	Number of DDR banks */

#ifndef		NO_OF_DDR_BANKS
#define		NO_OF_DDR_BANKS			4
#endif

#define		MAX_BUFFER_OFFSET	(DDR_BANK_SIZE * NO_OF_DDR_BANKS)


#endif		// __SYSTEM__H__
