//----------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2001-2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//----------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: 	kick_start.uc
//
//      Assumptions:    Thisfile may be included by others 
//                      (and thus uses header guards)
//
/////////////////////////////////////////////////////////////////////////////

#ifndef KICK_START_UC
#define KICK_START_UC 1

#include <dl_meta.uc>


/////////////////////////////////////////////////////////////////////////////////////////
//
// _mpls_ipv4_kick_start()
//
//	 	Description: 
//
//		Kick starts MPLS Forwarder and IPV4 Forwarder blocks across the 4 MEs
//	(1) when the system initialization is complete.
//	(2) signal thread 0 on beginning ME after which the
//		thread order is maintained by DL macros.
//
//	All the MEs (and threads) running Forwader call 
//	this macro. Only thread0 of first ME would execute 
//	this fully. Other threads of first ME and all the
//	threads on other MEs would continue and call the
//	dl_source_mpls[DL_THREAD_ORDER], and wait for signal
//	from previous thread. This would result in thread0
//	of first ME kickstarting the whole chain.
//
//
//	 	Outputs:
//			None.
//
//		Inputs:
//			None.
//
//		CONSTANTS
//			None.	
//
//		Size: 
//
//			?? instructions. (Worst case cycle count)
//
////////////////////////////////////////////////////////////////////////////////////////
#macro	_mpls_ipv4_kick_start()

	// The MPLS/IPV4 fwder code can run on single ME or multiple MEs. For
	// Point Reyes, we run this code on 4 MEs as a chain. These MEs
	// need to wait for the system initialization to complete which
	// is communicated by sending a signal. So, thread0 of the 
	// first ME waits for this signal. Once, this signal is received.
	// this thread kick starts the ME chain. (Actually any thread on
	// any ME can act as kick starter).

	// We need to know the ME number to get this right. If we pickup
	// different MEs, the following line has to be modified accordingly.
	// For Point Reyes, '0x1' is the first ME in the chain. 
#ifdef	MPLS_START_ME	


	// we need to manually allocate a signal on which we wait.
	.sig	sys_init_signal
	.addr	sys_init_signal	ME_INIT_SIGNAL
	.set_sig sys_init_signal

	br!=ctx[0, rx_firsttime#]

	// along with the system initialization signal, we assert a wakeup
	// signal so that very first time around, we don't need to wait. This
	// helps in kick starting the chain.
	signal_first_ctx[DL_SIG_WAKE]

	// we wait for the signal here
	ctx_arb[sys_init_signal]

	// by exiting this block, we free up the signal

#endif	// START_ME

	// All threads execute this code. Here we call DL macro to receive
	// packets. The packet order is maintained by the DL macro.
rx_firsttime#:

#endm



#endif  /* KICK_START_UC 1 */
