/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      ATM SAR UNIT TEST Core Component
 *
 * = FILENAME
 *      ix_cc_atmsartest_private.h
 *
 * = DESCRIPTION
 *      This file defines all data types and function prototypes required
 *      internaly by the SAR CTRL MaIN CC module.
 *
 * = AUTHOR
 *       Adam Dzieciatkowski
 *       adam.dzieciatkowski@@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * 
 */

#if !defined(__IX_CC_ATMSAR_CONFIG_PRIVATE_H__)
#define __IX_CC_ATMSAR_CONFIG_PRIVATE_H__


#if defined(__cplusplus)
extern "C" {
#endif /* end defined(__cplusplus) */

#include "dl_system.h"

/**
 * MACRO NAME: IX_ATMSARTEST_UNUSED_ARG
 *
 * DESCRIPTION: This macro is used to remove the warnings generated for unused arguments.
 *
 */
#define IX_ATMSARTEST_UNUSED_ARG(arg_Argument) (void)arg_Argument


/**
 * TYPENAME: atmsar_config_context_t
 * 
 * DESCRIPTION:
 *    This structure defines format for SAR CONTROL MAIN  core component
 *    context.
 */
typedef struct ix_s_cc_atmsartest_context
{
    ix_cc_handle hAtmSarTestHandle;      /* Handle of SAR control main core component */
    ix_ossl_mutex_t AtmSarTestMutex;     /* mutex id */
} atmsar_config_context_t;



/*****************************************************************************
 *       Type definitions whose scope is limited to this file
 *****************************************************************************/

/*****************************************************************************
 * Abstract 
 *    Context of asynchonous call. Contains all information for continue work
 *    in callback function after processing asynchronous call in SAR Control
 *    core component.
 *
 * Fields:
 *    testId - identier of test
 *    xId - transaction ID
 *    context - parameters specific for particular operation on SAR Control.
 *    
 *****************************************************************************/
typedef struct {
    ix_uint8 used;          /* 0 - unused, 1-used */
    ix_uint32 xId;
    ix_ossl_sem_t *pSem;
    
    union{
        struct {
            ix_cc_atmsar_vc_descr_t VcDescr;
        } vcCreate;

        struct {
            ix_cc_atmsar_update_data_t updateData;
        } vcUpdate;
        
        struct {
            ix_cc_atmsar_vc_handle_t hVc;
        } vcRemove;

        struct {
            ix_cc_atmsar_vc_handle_t hVc;
        } vcStats;

        struct {
            ix_cc_atmsar_port_descr_t portDescr;
        } portCreate;

        struct {
            ix_cc_atmsar_port_handle_t hPort;
        } portRemove;

        struct {
            ix_cc_atmsar_port_handle_t hPort;
        } portStats;

        struct {
            ix_cc_atmsar_cid_descr_t cidDescr;
        } cidCreate;

        struct {
            ix_cc_atmsar_update_data_t cidUpdate;
        } cidUpdate;

        struct {
            ix_cc_atmsar_cid_handle_t hCid;
        } cidRemove;

        struct {
            ix_cc_atmsar_cid_handle_t hCid;
        } cidStats;

    } context;
} atmsar_config_async_context_t;


/*****************************************************************************
 * Abstract 
 *    The list of defined traffic proffile identificators.
 *
 * Purpose/Intended Use:
 *    It is used for definition of VC description. The VC description is used
 *    to perform operations on SAR CONTROL module
 *
 *****************************************************************************/
typedef enum
{
    ATMSAR_CONFIG_TP_ID_CBR_100 = 0,
    ATMSAR_CONFIG_TP_ID_CBR_200,
    ATMSAR_CONFIG_TP_ID_CBR_200000,
    ATMSAR_CONFIG_TP_ID_UBR_NOPRIORITY,
    ATMSAR_CONFIG_TP_ID_UBR_PRIORITY,
    ATMSAR_CONFIG_TP_ID_RTVBR,
    ATMSAR_CONFIG_TP_ID_NRTVBR,
    ATMSAR_CONFIG_TP_ID_GFR,
    ATMSAR_CONFIG_TP_ID_NONE,
    ATMSAR_CONFIG_TP_ID_MAX = 128
}atmsar_config_tp_id_t;


#define ATMSAR_CONFIG_TIMEOUT 10000

/*****************************************************************************
 * Abstract 
 *    The structure defines descrption of traffic parameters. The description
 *    can be used for defining VCs durung tests
 *
 *****************************************************************************/
typedef struct{
    atmsar_config_tp_id_t tpId;
    ix_uint8       serviceClass;
    ix_uint32      txPcr;
    ix_uint32      txScr;
    ix_uint32      mbs;           
    ix_uint32      mfs;           
    ix_uint32      mcr;           
    ix_uint8       cdvt;          
    ix_uint8       tagging;       
    ix_uint8       frmDisc;       
    ix_uint8       etoeOamF5FlInd;
    ix_uint8       ubrBcs;     
    ix_uint8       ubrPriority;   
}atmsar_config_tp_t;


/*****************************************************************************
 * Abstract 
 *    The type describes states of FSM for created inctances by SarUt Manager.
 *    Sar UT Manager manages instanes of VCs and CIDs.
 *    States and what kind of action can be performed for the states:
 *    - UNDEF - configuration parameters for the instance is not set. Allowed
 *         operation DEFINING. Next state will be DEFINED, after user call
 *         'VC/CID define'.
 *    - DEFINED - configuration parameters is ready. User can call 'VC/CID
 *         create' operation. Next state will be CREATING, after user call
 *         'VC/CID create'..
 *    - CREATING - user calls 'VC/CID create'. SAR Control still don't reply
 *         for request. No user operation is allowed. Next state will be
 *         CREATED.
 *    - CREATED - VC/CID is created. User can call requests 'VC/STATS stats'
 *         and 'VC/CID remove' for the VC/CID.
 *    - STAT_READING - user calls 'VC/CID get stats'. SAR Control still don't
 *         reply for request. No user operation is allowed. Next state will be
 *         CREATED.
 *    - REMOVING - user calls 'VC/CID remove'. SAR Control still don't reply
 *         for request. No user operation is allowed. Next state will be
 *         DEFINED.
 *
 *****************************************************************************/
typedef enum
{
    ATMSAR_CONFIG_FSM_ST_UNDEF=0,
    ATMSAR_CONFIG_FSM_ST_DEFINED,
    ATMSAR_CONFIG_FSM_ST_CREATING,
    ATMSAR_CONFIG_FSM_ST_CREATED,
    ATMSAR_CONFIG_FSM_ST_UPDATING,
    ATMSAR_CONFIG_FSM_ST_UPDATED,    
    ATMSAR_CONFIG_FSM_ST_STAT_READING,
    ATMSAR_CONFIG_FSM_ST_REMOVING
}atmsar_config_fsm_state_t;


/*****************************************************************************
 * Abstract 
 *    The structure describes all parameter of VC in SAR UNIT TEST module.
 *    It configuration and run-time parameters.
 *
 *****************************************************************************/
typedef struct
{
    ix_cc_atmsar_vc_handle_t handle;  /* handle assigned in SAR Control */
    atmsar_config_fsm_state_t state;            /* current FSM state               */
} atmsar_config_vc_t;



/*****************************************************************************
 * Abstract 
 *    The structure contains all private data of SAR UNIT TEST VC MANAGER
 *    subsystem.
 *
 *****************************************************************************/
typedef struct
{
#define ATMSAR_CONFIG_MGR_MAX_VC NUMBER_OF_VC /* count of VC supported by VC manager */
    atmsar_config_vc_t vc[ATMSAR_CONFIG_MGR_MAX_VC];
} atmsar_config_mgr_t;



#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_ATMSAR_CONFIG_PRIVATE_H__) */

