/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2002 By Intel Corporation.  All rights reserved.
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/**
 *****************************************************************************
 * @ingroup SarUt
 * File:  ix_cc_atmsar_config.c
 * @description
 *      The file defines for core component infrestructure  interface and
 *      internal functions for initialization and shutdown.
 *****************************************************************************/

/*****************************************************************************
 *
 * Contents:
 *        Definitions of following exported data:
 *            none
 *        Definitions of following exported functions
 *        Definitions of following local functions:
 *
 *****************************************************************************/


/*****************************************************************************
 *                   System defined include files
 *****************************************************************************/
#include "ix_cc_error.h"
#include "ix_rm.h"
#include "ix_ossl.h"
#include "ix_cc.h"
#include "ix_cc_macros.h"
#include "ix_cc_properties.h"
#include "ix_config.h"
#include "bindings.h"


#include "cc/ix_cc_msup.h"


/*****************************************************************************
 *                    User defined include files
 *****************************************************************************/
#include "cc/ix_cc_atmsar.h"
#include "ix_cc_atmsar_config_private.h"
#include "cc/internal/ix_cc_atmsar_common.h"
#include "cc/ix_cc_atmrx_cci.h"
#include "dl_system.h"

#include <stdio.h>

/*****************************************************************************
 * Abstract
 *    Variable contains all data of VC Manager
 *
 *****************************************************************************/
atmsar_config_mgr_t sarUtMgr;

/*****************************************************************************
 * Abstract
 *    The table defines descrption of traffic parameters. The description
 *    can be used for defining VCs durung tests
 *
 *****************************************************************************/

static atmsar_config_tp_t sarUtTpTab[ATMSAR_CONFIG_TP_ID_MAX]=
{  /* TP id,                         service class,                    aal type,           txPcr,  txScr, mbs, mfs, mcr, cdvt, tagging, frmDisc, etoeOamF5FlInd, ubrBcs,                          ubrPriority   */
    { ATMSAR_CONFIG_TP_ID_CBR_100,          IX_CC_ATMSAR_ATM_SERVICE_CBR,    100,       100,   0,   0,   0,   0,    0,       0,       0,              0,                               0},
    { ATMSAR_CONFIG_TP_ID_CBR_200,          IX_CC_ATMSAR_ATM_SERVICE_CBR,    200,       200,   0,   0,   0,   0,    0,       0,       0,              0,                               0},
    { ATMSAR_CONFIG_TP_ID_UBR_NOPRIORITY,   IX_CC_ATMSAR_ATM_SERVICE_UBR,    200,       0,     0,   0,   0,   100,  0,       0,       0,              IX_CC_ATMSAR_UBR_BCS_PCR,       0},
    { ATMSAR_CONFIG_TP_ID_UBR_PRIORITY,     IX_CC_ATMSAR_ATM_SERVICE_UBR,    200,       0,     0,   0,   0,   100,  0,       0,       0,              IX_CC_ATMSAR_UBR_BCS_NO_PCR,    1},
    { ATMSAR_CONFIG_TP_ID_RTVBR,            IX_CC_ATMSAR_ATM_SERVICE_RT_VBR, 200,       100,   100, 0,   100, 100,  0,       0,       0,              0,                               0},
    { ATMSAR_CONFIG_TP_ID_GFR,              IX_CC_ATMSAR_ATM_SERVICE_GFR,    200,       0,     0,   0,   100, 100,  0,       0,       0,              0,                               0},
    { ATMSAR_CONFIG_TP_ID_CBR_200000,       IX_CC_ATMSAR_ATM_SERVICE_CBR,    1415094,     0,     0,   0,   0,   0,    0,       0,       0,              0,                               0},
    { ATMSAR_CONFIG_TP_ID_NRTVBR,           IX_CC_ATMSAR_ATM_SERVICE_NRT_VBR,200,       100,   100, 0,   100, 100,  0,       0,       0,              0,                               0},
    { ATMSAR_CONFIG_TP_ID_NONE,             IX_CC_ATMSAR_ATM_SERVICE_NONE,   0  ,       0,     0,   0,   0,   0,    0,       0,       0,              0,                               0},
    { ATMSAR_CONFIG_TP_ID_MAX,              IX_CC_ATMSAR_ATM_SERVICE_NONE,   0  ,       0,     0,   0,   0,   0,    0,       0,       0,              0,                               0},
};

/*****************************************************************************
 *                    Static function prototypes.
 *****************************************************************************/

static ix_error
atmsar_config_cb_vccreate(ix_error arg_Result,
                      ix_cc_atmsar_vc_handle_t arg_hVc, void* arg_pContext);

static ix_error
atmsar_config_cb_vcupdate(ix_error arg_Result,
                      ix_cc_atmsar_vc_handle_t arg_hVc, void* arg_pContext);

static ix_error
atmsar_config_cb_vcremove(ix_error arg_Result, void* arg_pContext);

static void
atmsar_config_show_cb_result(char *pOperName, ix_error result);


/*****************************************************************************
 * Abstract:
 *    Shows result of SAR operation.
 *
 * Params:
 *    IN operName - textural description of executed operation
 *    IN result - result of operation
 *
 * Return Values:
 *	  none
 *
 *****************************************************************************/
void
atmsar_config_show_cb_result(
char *pOperName,
ix_error result)
{
    ix_ossl_message_log("TEST : %s CALLBACK %s:\n",
                        pOperName, (result==IX_SUCCESS)?"OK":"ERROR");
    if (result!=IX_SUCCESS)
    {
        ix_error_dump(NULL, result);
    }
}


/*****************************************************************************
 * Abstract:
 *    The callback function is called by core component infrastructure (CCI)
 *    when SAR Control module processed a message "VC create" and sent reply.
 *
 * Params:
 *    IN result - result of SAR Control operation
 *    IN hVc    - VC handle of new created VC
 *    IN pCtx   - Pointer to context of operation.
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
atmsar_config_cb_vccreate(
ix_error result,
ix_cc_atmsar_vc_handle_t hVc,
void* pCtx)
{
    atmsar_config_async_context_t *pContext= (atmsar_config_async_context_t*) pCtx;
    atmsar_config_vc_t* pVc;

    if (pContext->pSem != NULL)
    {
        /*
         * If we're not able to give the semaphore, print the error.  Always
         * return success.
         */
        IX_ERROR_CP(ix_ossl_sem_give(*(pContext->pSem)));
    }
    
    if (pContext->xId>=ATMSAR_CONFIG_MGR_MAX_VC)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("Vc ID %d is out of range", pContext->xId));
    }

    pVc = &sarUtMgr.vc[pContext->xId];

    atmsar_config_show_cb_result("VC CREATE", result);

    if (result == IX_SUCCESS)
    {
        pVc->handle = hVc;
    }

    ix_ossl_free(pCtx);
    return IX_SUCCESS;
}


/*****************************************************************************
 * Abstract:
 *    The callback function is called by core component infrastructure (CCI)
 *    when SAR Control module processed a message "VC create" and sent reply.
 *
 * Params:
 *    IN result - result of SAR Control operation
 *    IN hVc    - VC handle of new created VC
 *    IN pCtx   - Pointer to context of operation.
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
atmsar_config_cb_vcupdate(
ix_error result,
ix_cc_atmsar_vc_handle_t hVc,
void* pCtx)
{
    atmsar_config_async_context_t *pContext= (atmsar_config_async_context_t*) pCtx;

    if (pContext->pSem != NULL)
    {
        /*
         * If we're not able to give the semaphore, print the error.  Always
         * return success.
         */
        IX_ERROR_CP(ix_ossl_sem_give(*(pContext->pSem)));
    }
    
    if (pContext->xId>=ATMSAR_CONFIG_MGR_MAX_VC)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("Vc ID %d is out of range", pContext->xId));
    }

    atmsar_config_show_cb_result("VC UPDATE", result);
    
    ix_ossl_free(pCtx);
    return IX_SUCCESS;
}


/*****************************************************************************
 * Abstract:
 *    The callback function. It is call by CCI, when SAR Control pelies for
 *    'VC remove' request.
 *
 * Params:
 *    IN result - result of SAR Control operation
 *    IN pCtx   - Pointer to context of operation.
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
atmsar_config_cb_vcremove(
ix_error result,
void* pCtx)
{
    atmsar_config_async_context_t *pContext= (atmsar_config_async_context_t*) pCtx;
    atmsar_config_vc_t* pVc;

    if (pContext->pSem != NULL)
    {
        /*
         * If we're not able to give the semaphore, print the error.  Always
         * return success.
         */
        IX_ERROR_CP(ix_ossl_sem_give(*(pContext->pSem)));
    }

    if (pContext->xId>=ATMSAR_CONFIG_MGR_MAX_VC)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("Vc ID %d is out of range", pContext->xId));
    }

    pVc = &sarUtMgr.vc[pContext->xId];

    atmsar_config_show_cb_result("VC REMOVE", result);

    if (result == IX_SUCCESS)
    {
        pVc->handle = IX_CC_ATMSAR_VCID_INVALID;
    }

    ix_ossl_free(pContext);
    return IX_SUCCESS;
}

/*****************************************************************************
 * Abstract:
 *    The callback function. It is call by CCI, when SAR Control pelies for
 *    'VC statistic' request.
 *
 * Params:
 *    IN result - result of SAR Control operation
 *    IN pLookupResult - pointer to structure with returned statistic data.
 *    IN pCtx   - Pointer to context of operation.
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
atmsar_config_cb_vcstats(
ix_error result,
ix_cc_atmsar_vc_stats_t *arg_pLookupResult,
void* pCtx)
{
    atmsar_config_async_context_t *pContext= (atmsar_config_async_context_t*) pCtx;
    atmsar_config_vc_t* pVc;

    if (pContext->pSem != NULL)
    {
        /*
         * If we're not able to give the semaphore, print the error.  Always
         * return success.
         */
        IX_ERROR_CP(ix_ossl_sem_give(*(pContext->pSem)));
    }

    if (pContext->xId>=ATMSAR_CONFIG_MGR_MAX_VC)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("Vc ID %d is out of range", pContext->xId));
    }

    pVc = &sarUtMgr.vc[pContext->xId];
    
    atmsar_config_show_cb_result("VC STATS", result);
    
    ix_ossl_message_log("  RX pkts=%-8ld, cells=%-8ld, discards=%-8ld, cellsErr=%-8ld\n",
                        (ix_uint32)arg_pLookupResult->rx.packets,
                        (ix_uint32)arg_pLookupResult->rx.cells,
                        (ix_uint32)arg_pLookupResult->rx.discards,
                        (ix_uint32)arg_pLookupResult->rx.cellErrors);
    ix_ossl_message_log("  TX pkts=%-8ld, cells=%-8ld, discards=%-8ld\n",
                        (ix_uint32)arg_pLookupResult->tx.packets,
                        (ix_uint32)arg_pLookupResult->tx.cells,
                        (ix_uint32)arg_pLookupResult->tx.discards);
    
    ix_ossl_free(pCtx);
    return IX_SUCCESS;
}

/*****************************************************************************
 * Abstract:
 *    The callback function. It is call by CCI, when SAR Control pelies for
 *    'port statistic' request.
 *
 * Params:
 *    IN result - result of SAR Control operation
 *    IN pLookupResult - pointer to structure with returned statistic data.
 *    IN pCtx   - Pointer to context of operation.
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
atmsar_config_cb_portstats(
ix_error result,
ix_cc_atmsar_port_stats_t *pLookupResult,
void* pCtx)
{
    atmsar_config_async_context_t *pContext= (atmsar_config_async_context_t*) pCtx;

    if (pContext->pSem != NULL)
    {
        /*
         * If we're not able to give the semaphore, print the error.  Always
         * return success.
         */
        IX_ERROR_CP(ix_ossl_sem_give(*(pContext->pSem)));
    }

    if (pContext->xId>=MAX_ATM_PORTS)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("Port ID %d is out of range", pContext->xId));
    }
    
    atmsar_config_show_cb_result("PORT STATS", result);
    
    ix_ossl_message_log("Port %d Statistics:\n", pContext->context.portStats.hPort);
    ix_ossl_message_log("  RX pkts=%-8ld, cells=%-8ld, discards=%-8ld, cellsErr=%-8ld\n",
                        (ix_uint32)pLookupResult->rx.packets,
                        (ix_uint32)pLookupResult->rx.cells,
                        (ix_uint32)pLookupResult->rx.discards,
                        (ix_uint32)pLookupResult->rx.cellErrors);
    ix_ossl_message_log("  TX pkts=%-8ld, cells=%-8ld, discards=%-8ld\n",
                        (ix_uint32)pLookupResult->tx.packets,
                        (ix_uint32)pLookupResult->tx.cells,
                        (ix_uint32)pLookupResult->tx.discards);
    
    
    ix_ossl_free(pCtx);
    return IX_SUCCESS;
}


int addVc(const char *argv)
{
    ix_uint32 direction;
    ix_uint32 vcid;
    ix_uint32 port;
    ix_uint32 vpi;
    ix_uint32 vci;
    ix_uint32 profile;
    atmsar_config_async_context_t *pContext;
    ix_cc_atmsar_vc_descr_t vcDescr;
    atmsar_config_tp_t *pTp;
    ix_error err;
    ix_ossl_sem_t sem;

    
    if (argv == NULL)
    {
        ix_ossl_message_log("addVc: parameters missing.");
        return 1;
    }

    pContext = ix_ossl_malloc(sizeof(atmsar_config_async_context_t));
    if (pContext == NULL)
    {
        ix_ossl_message_log("addVc: not enough memory.");
        return(1);
    }

    /*
     * Parse the command line and extract all required parameters.
     */
    {
        if (sscanf(argv, "%li %li %li %li %li %li",
                   &direction, &vcid, &port, &vpi, &vci, &profile) != 6)
        {
            ix_ossl_message_log("addVc: Command lins parse error.");
            return 1;
        } 
        
        ix_ossl_message_log("Adding VC:\n");
        ix_ossl_message_log("direction = %02ld\n"
                            "vcid = %04ld, port = %02ld, vpi = %04ld\n"
                            "vci = %04ld, profile = %02ld \n",
                            direction,vcid,port,vpi,vci,profile);
    }

    /*
     * clean up the VC description structure
     */
    memset(&vcDescr, 0, sizeof(vcDescr));
    
    /*
     * set up VPI/VCI/port fields
     */
    vcDescr.spec.portNo = port;
    vcDescr.spec.vpi    = vpi;
    vcDescr.spec.vci    = vci;
	vcDescr.hdrType     = 0;
	vcDescr.l2port      = 0;
    vcDescr.aalType     = IX_CC_ATMSAR_AAL5;

    /*
     * get the pointer to the Traffic profile structure
     */
    pTp=&(sarUtTpTab[profile]);
    
    /*
     * copy the traffic profile parameters
     */
    vcDescr.trafficParams.serviceClass         = pTp->serviceClass;
    vcDescr.trafficParams.fwd.pcr              = pTp->txPcr;
    vcDescr.trafficParams.fwd.scr              = pTp->txScr;
    vcDescr.trafficParams.fwd.mbs              = pTp->mbs;
    vcDescr.trafficParams.fwd.mfs              = pTp->mfs;
    vcDescr.trafficParams.fwd.mcr              = pTp->mcr;
    vcDescr.trafficParams.fwd.cdvt             = pTp->cdvt;
    vcDescr.trafficParams.fwd.tagging          = pTp->tagging;
    vcDescr.trafficParams.fwd.frmDisc          = pTp->frmDisc;
    vcDescr.trafficParams.fwd.etoeOamF5FlInd   = pTp->etoeOamF5FlInd;
    vcDescr.trafficParams.fwd.ubrBcs           = pTp->ubrBcs;
    vcDescr.trafficParams.fwd.ubrPriority      = pTp->ubrPriority;
    
    /* prepare the context structure */
    memcpy(&(pContext->context.vcCreate.VcDescr),
           &vcDescr,
           sizeof(ix_cc_atmsar_vc_descr_t));
    pContext->used = 1;
    pContext->xId  = vcid;

    /* Allocate the semaphore - it is released by calback functions */
    IX_ERROR_CR(ix_ossl_sem_init(IX_OSSL_SEM_UNAVAILABLE, &sem));
    pContext->pSem = &sem;

    /* send the vc create async message */
    IX_ERROR_CG(ix_cc_atmsar_async_vc_create(direction,
                                             &vcDescr,
                                             atmsar_config_cb_vccreate,
                                             (void*)pContext),
                err,clean_sem);
    IX_ERROR_CG(ix_ossl_sem_take(sem, ATMSAR_CONFIG_TIMEOUT), err, clean_sem);
    IX_ERROR_CR(ix_ossl_sem_fini(sem));
    
    return 0;

clean_sem:
    (void) ix_ossl_sem_fini(sem);

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    ix_error_dump(0, err);
#else
    ix_error_dump(stdout, err);
#endif
    
    return -1;
}


int updateVc(const char *argv)
{
    ix_uint32 direction;
    ix_uint32 vcid;
    ix_uint32 l2port;
    ix_uint32 hdrtype;
    ix_error err;
    ix_ossl_sem_t sem;

    ix_cc_atmsar_update_data_t sarUpdateData;
    ix_cc_atmrx_update_data_t rxUpdateData;    
    atmsar_config_async_context_t *pContext;

    if (argv == NULL)
    {
        ix_ossl_message_log("addVc: parameters missing.");
        return 1;
    }

    pContext = ix_ossl_malloc(sizeof(atmsar_config_async_context_t));
    if (pContext == NULL)
    {
        ix_ossl_message_log("addVc: not enough memory.");
        return(1);
    }

    /*
     * Parse the command line and extract all required parameters.
     */
    {
        
        if (sscanf(argv, "%li %li %li %li",
                   &direction, &vcid, &l2port, &hdrtype) != 4)
        {
            ix_ossl_message_log("addVc: Command line parse error.");
            return 1;
        } 
        
        ix_ossl_message_log("Updating VC:\n");
        ix_ossl_message_log("direction = %02ld\n"
                            "vcid = %04ld, l2port = %04ld, hdrtype = %04ld\n",
                            direction,vcid,l2port,hdrtype);
    }

    /*
     * clean up the VC update structures
     */
    memset(&rxUpdateData, 0, sizeof(rxUpdateData));
    
    /*
     * set up l2port and header type fields
     */
    rxUpdateData.lw0.l2port  = l2port;
    rxUpdateData.lw1.hdrType = hdrtype;
    memcpy(&sarUpdateData, &rxUpdateData, sizeof(ix_cc_atmsar_update_data_t));
    
    /* prepare the context structure */
    memcpy(&(pContext->context.vcUpdate.updateData),
           &sarUpdateData,
           sizeof(ix_cc_atmsar_update_data_t));
    pContext->used = 1;
    pContext->xId  = vcid;

    /* Allocate the semaphore - it is released by calback functions */
    IX_ERROR_CR(ix_ossl_sem_init(IX_OSSL_SEM_UNAVAILABLE, &sem));
    pContext->pSem = &sem;
    
    /* send the vc create async message */
    IX_ERROR_CG(ix_cc_atmsar_async_vc_update(direction,
                                             sarUtMgr.vc[vcid].handle,
                                             &sarUpdateData,
                                             atmsar_config_cb_vcupdate,
                                             (void*)pContext),
                err,clean_sem);    
    IX_ERROR_CG(ix_ossl_sem_take(sem, ATMSAR_CONFIG_TIMEOUT), err, clean_sem);
    IX_ERROR_CR(ix_ossl_sem_fini(sem));
    
    return 0;

clean_sem:
    (void) ix_ossl_sem_fini(sem);

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    ix_error_dump(0, err);
#else
    ix_error_dump(stdout, err);
#endif
    
    return -1;
}

int delVc(const char *argv)
{
    ix_uint32 direction;
    ix_uint32 vcid;
    atmsar_config_async_context_t *pContext;
    ix_error err;
    ix_ossl_sem_t sem;

    if (argv == NULL)
    {
        ix_ossl_message_log("delVc: parameters missing.");
        return 1;
    }

    pContext = ix_ossl_malloc(sizeof(atmsar_config_async_context_t));
    if (pContext == NULL)
    {
        ix_ossl_message_log("delVc: not enough memory.");
        return(1);
    }

    /*
     * Parse the command line and extract all required parameters.
     */
    {
        
        if (sscanf(argv, "%li %li",
                   &direction, &vcid) != 2)
        {
            ix_ossl_message_log("delVc: Command line parse error.");
            return 1;
        } 
        
        ix_ossl_message_log("Removing VC:\n");
        ix_ossl_message_log("direction = %02ld vcid = %04ld\n",
                            direction,vcid);
    }

    /* prepare the context structure */
    pContext->context.vcRemove.hVc = sarUtMgr.vc[vcid].handle;
    pContext->used                 = 1;
    pContext->xId                  = vcid;

    /* Allocate the semaphore - it is released by calback functions */
    IX_ERROR_CR(ix_ossl_sem_init(IX_OSSL_SEM_UNAVAILABLE, &sem));
    pContext->pSem = &sem;
    
    /* send the vc remove async message */
    IX_ERROR_CG(ix_cc_atmsar_async_vc_remove(direction,
                                             sarUtMgr.vc[vcid].handle,
                                             atmsar_config_cb_vcremove,
                                             (void*)pContext),
                err,clean_sem);                
    IX_ERROR_CG(ix_ossl_sem_take(sem, ATMSAR_CONFIG_TIMEOUT), err, clean_sem);
    IX_ERROR_CR(ix_ossl_sem_fini(sem));
    
    return 0;

clean_sem:
    (void) ix_ossl_sem_fini(sem);

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    ix_error_dump(0, err);
#else
    ix_error_dump(stdout, err);
#endif
    
    return -1;
}

int getstatsVc(const char *argv)
{
    ix_uint32 direction;
    ix_uint32 vcid;
    atmsar_config_async_context_t *pContext;
    ix_error err;
    ix_ossl_sem_t sem;

    if (argv == NULL)
    {
        ix_ossl_message_log("getstatsVc: parameters missing.");
        return 1;
    }

    pContext = ix_ossl_malloc(sizeof(atmsar_config_async_context_t));
    if (pContext == NULL)
    {
        ix_ossl_message_log("getstatsVc: not enough memory.");
        return(1);
    }

    /*
     * Parse the command line and extract all required parameters.
     */
    {
        
        if (sscanf(argv, "%li %li",
                   &direction, &vcid) != 2)
        {
            ix_ossl_message_log("getstatsVc: Command line parse error.");
            return 1;
        } 
        
        ix_ossl_message_log("Getting statistics for VC:\n");
        ix_ossl_message_log("direction = %02ld vcid = %04ld\n",
                            direction,vcid);
    }

    /* prepare the context structure */
    pContext->context.vcStats.hVc = sarUtMgr.vc[vcid].handle;
    pContext->used                 = 1;
    pContext->xId                  = vcid;

    /* Allocate the semaphore - it is released by calback functions */
    IX_ERROR_CR(ix_ossl_sem_init(IX_OSSL_SEM_UNAVAILABLE, &sem));
    pContext->pSem = &sem;

    /* send the vc remove async message */
    IX_ERROR_CG(ix_cc_atmsar_async_get_vc_stats(direction,
                                                sarUtMgr.vc[vcid].handle,
                                                atmsar_config_cb_vcstats,
                                                (void*)pContext),
                err,clean_sem);
    IX_ERROR_CG(ix_ossl_sem_take(sem, ATMSAR_CONFIG_TIMEOUT), err, clean_sem);
    IX_ERROR_CR(ix_ossl_sem_fini(sem));
    
    return 0;

clean_sem:
    (void) ix_ossl_sem_fini(sem);

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    ix_error_dump(0, err);
#else
    ix_error_dump(stdout, err);
#endif
    
    return -1;
}

int getstatsPort(const char *argv)
{
    ix_uint32 direction;
    ix_uint32 port;
    atmsar_config_async_context_t *pContext;
    ix_error err;
    ix_ossl_sem_t sem;
    

    if (argv == NULL)
    {
        ix_ossl_message_log("getstatsPort: parameters missing.");
        return 1;
    }

    pContext = ix_ossl_malloc(sizeof(atmsar_config_async_context_t));
    if (pContext == NULL)
    {
        ix_ossl_message_log("getstatsPort: not enough memory.");
        return(1);
    }

    /*
     * Parse the command line and extract all required parameters.
     */
    {
        
        if (sscanf(argv, "%li %li",
                   &direction, &port) != 2)
        {
            ix_ossl_message_log("getstatsPort: Command line parse error.");
            return 1;
        } 
        
        ix_ossl_message_log("Getting statistics for PORT:\n");
        ix_ossl_message_log("direction = %02ld port = %04ld\n",
                            direction,port);
    }

    /* prepare the context structure */
    pContext->context.portStats.hPort = port;
    pContext->used                    = 1;
    pContext->xId                     = port;
    
    /* Allocate the semaphore - it is released by calback functions */
    IX_ERROR_CR(ix_ossl_sem_init(IX_OSSL_SEM_UNAVAILABLE, &sem));
    pContext->pSem = &sem;

    /* send the vc remove async message */
    IX_ERROR_CG(ix_cc_atmsar_async_get_port_stats(direction,
                                                  port,
                                                  atmsar_config_cb_portstats,
                                                  (void*)pContext),
                err,clean_sem);
    IX_ERROR_CG(ix_ossl_sem_take(sem, ATMSAR_CONFIG_TIMEOUT), err, clean_sem);
    IX_ERROR_CR(ix_ossl_sem_fini(sem));
    
    return 0;

clean_sem:
    (void) ix_ossl_sem_fini(sem);

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    ix_error_dump(0, err);
#else
    ix_error_dump(stdout, err);
#endif
    
    return -1;
}




