/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2X00 Network Processor, Release 6
 *
 * = FILENAME
 *    ix_cc_atmsar_config_linux_user_mode_util.c
 *
 * = DESCRIPTION
 *    This file contains linux user mode application 'rconfig' that 
 *    can be used to in target shell for configuring the route table.
 *
 * = CHANGE HISTORY
 *      05/06/2003 - Created.
 *
 * ============================================================================
 * $Id: 
 */
#include <fcntl.h>
#include "ix_cc_atmsar_config_linux_kernel_module.h"

void atmsarconfig_usage(void);

int main(int argc, char *argv[])
{

    int fd;
    int cmd;
    char *cmdArgStr;

    if (argc == 1)
    {
        atmsarconfig_usage();
        exit(0);
    }

    if (strcmp(argv[1],"addVc") == 0)
    {
        cmd = ADD_VC;
        cmdArgStr = argv[2];
    }
    else if (strcmp(argv[1],"delVc") == 0)
    {
        cmd = DEL_VC;
        cmdArgStr = argv[2];
    }
    else if (strcmp(argv[1],"updateVc") == 0)
    {
        cmd = UPDATE_VC;
        cmdArgStr = argv[2];
    }
    else if (strcmp(argv[1],"statsVc") == 0)
    {
        cmd = GETSTATS_VC;
        cmdArgStr = argv[2];
    }
    else if (strcmp(argv[1],"statsPort") == 0)
    {
        cmd = GETSTATS_PORT;
        cmdArgStr = argv[2];
    }
    else
    {
        printf("Unrecognized command\n");
        exit(1);
    }

    fd = open("/dev/SarConfig", O_CREAT | O_RDWR);

    ioctl(fd, cmd, cmdArgStr);

    close(fd);

    return 0;

} /* main */



void atmsarconfig_usage(void)
{
    printf("\nUSAGE: atmsar_config <cmd> <\"args\">\n\n");
    printf(" where <cmd> <\"args\"> pair can be one of the followings: \n");
    printf("  addVc \"direction vcid port vpi vci profile\"\n");
    printf("  delVc \"direction vcid\"\n");
    printf("  updateVc \"direction vcid l2port header_type\"\n");
    printf("  statsVc \"direction vcid\"\n");
    printf("  statsPort \"direction port\"\n");
}







