/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      ATM SAR Main Core Component
 *
 * = FILENAME
 *      ix_cc_atmsar_main.h
 *
 * = DESCRIPTION
 *      This file defines all data types and function prototypes required
 *      internaly by the ATM SAR MAIN CC module.
 *
 * = AUTHOR
 *      Jaroslaw Kogut
 *      jaroslaw.kogut@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * 
 */

#if !defined(__IX_CC_ATMSAR_PRIVATE_H__)
#define __IX_CC_ATMSAR_PRIVATE_H__

#include "cc/ix_cc_atmsar_plugin.h"
#include "cc/internal/ix_cc_atmsar_common.h"

#if defined(__cplusplus)
extern "C" {
#endif /* end defined(__cplusplus) */

/******************************************************************************
 *                PREPROCESSOR DEFINITIONS: MACROS & DEFINES
 *****************************************************************************/

/******************************************************************************
 *                         TYPES DEFINITION
 *****************************************************************************/

/**
 * TYPENAME: _ix_cc_atmsar_main_context_t
 * 
 * DESCRIPTION:
 *    This structure defines format for SAR CONTROL MAIN  core component
 *    context.
 */
typedef struct ix_s_cc_atmsar_context
{
    ix_cc_handle hAtmsarHandle;   /* Handle of SAR control main core component */
    ix_uint16 mainInitialized;     /* the flag is set to 1 if all SoftSar core  */
    /* subcomponents are registered into SAR CONRTOL MAIN and AGENT. Otherwise  */
    /* it is equal 0.                                                           */
    ix_uint16 agentInitialized;  /* the copy of initialization flags from agent */
    ix_uint16 readyFlag;         /* if all sub-systems are initialized, the flag*/
    /* is equal FALSE, otherwise it is equal TRUE                               */

    _ix_cc_atmsar_engine_t engine;
} _ix_cc_atmsar_main_context_t;


/******************************************************************************
 *                         GLOBAL VARIABLES
 *****************************************************************************/


/******************************************************************************
 *                        FUNCTIONS DECLARATION
 *****************************************************************************/


/**
 * NAME: _ix_cc_atmsar_main_get_ctx
 *
 * DESCRIPTION:
 *    This function returns pointer to ATM SAR MAIN Core Component
 *    context.
 * 
 * @Param:  IN  arg_instance : Id of SAR Instance for that the action is
 *                  performed.
 * @Return: pointer to context of ATM SAR MAIN Core Component.
 */
_ix_cc_atmsar_main_context_t* _ix_cc_atmsar_main_get_ctx(
ix_cc_atmsar_instance_t arg_instance);
    
#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_ATMSAR_PRIVATE_H__) */

