/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      ATM SAR Main Core Component
 *
 * = FILENAME
 *      ix_cc_atmsar_msg.h
 *
 * = DESCRIPTION
 *      This file defines all data types required for internal message sending
 *
 * = AUTHOR
 *      Jaroslaw Kogut
 *      jaroslaw.kogut@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * 
 */

#if !defined(___IX_CC_ATMSAR_MSG_H__)
#define ___IX_CC_ATMSAR_MSG_H__

#include "cc/ix_cc_atmsar.h"

#if defined(__cplusplus)
extern "C" {
#endif /* end defined(__cplusplus) */



/**
 * TYPENAME: _ix_cc_atmsar_messages_t
 * 
 * DESCRIPTION: This structure defines message codes for ATM SAR messages
 *
 */
typedef enum ix_e_cc_atmsar_messages
{

    _IX_CC_ATMSAR_MSG_FIRST =IX_CC_COMMON_MSG_ID_LAST,

    _IX_CC_ATMSAR_MSG_VC_CREATE,
    _IX_CC_ATMSAR_MSG_VC_UPDATE,
    _IX_CC_ATMSAR_MSG_VC_REMOVE,
    _IX_CC_ATMSAR_MSG_VC_CLEAR,
    _IX_CC_ATMSAR_MSG_GET_VC_STATS,
    _IX_CC_ATMSAR_MSG_PORT_CREATE,
    _IX_CC_ATMSAR_MSG_PORT_REMOVE,
    _IX_CC_ATMSAR_MSG_GET_PORT_STATS,
    _IX_CC_ATMSAR_MSG_CID_CREATE,
    _IX_CC_ATMSAR_MSG_CID_UPDATE,
    _IX_CC_ATMSAR_MSG_CID_REMOVE,
    _IX_CC_ATMSAR_MSG_GET_CID_STATS,
    _IX_CC_ATMSAR_MSG_AGENT_INIT,     /* ATM SAR Agent initialized */

    /* Add new messages before this */
    _IX_CC_ATMSAR_MSG_LAST
} _ix_cc_atmsar_messages_t;


/******************************************************************************
 *     FORMAT OF ASYNCHRONOUS MESSAGES USED IN ATM SAR CORE COMPONENT
 *****************************************************************************/


/**
 * TYPENAME: _ix_cc_atmsar_async_vc_create_request_t
 * 
 * DESCRIPTION:
 *    The structure defines format of request for async msg 'VC create'
 */
typedef struct ix_s_cc_atmsar_async_vc_create_request
{
    ix_cc_atmsar_instance_t sarInstance;
    ix_cc_atmsar_vc_descr_t descr;
} _ix_cc_atmsar_async_vc_create_request_t;


/**
 * TYPENAME: _ix_cc_atmsar_async_vc_create_reply_t
 * 
 * DESCRIPTION:
 *    The structure defines format of reply for async msg 'VC create'
 */
typedef struct ix_s_cc_atmsar_async_vc_create_reply
{
    ix_cc_atmsar_vc_handle_t handle;
} _ix_cc_atmsar_async_vc_create_reply_t;


/**
 * TYPENAME: _ix_cc_atmsar_async_vc_update_request_t
 * 
 * DESCRIPTION:
 *    The structure defines format of request for async msg 'VC update'
 */
typedef struct ix_s_cc_atmsar_async_vc_update_request
{
    ix_cc_atmsar_instance_t sarInstance;
    ix_cc_atmsar_vc_handle_t handle; 
    ix_cc_atmsar_update_data_t updateData;
} _ix_cc_atmsar_async_vc_update_request_t;


/**
 * TYPENAME: _ix_cc_atmsar_async_vc_update_reply_t
 * 
 * DESCRIPTION:
 *    The structure defines format of reply for async msg 'VC update'
 */
typedef struct ix_s_cc_atmsar_async_vc_update_reply
{
    ix_cc_atmsar_vc_handle_t handle; 
} _ix_cc_atmsar_async_vc_update_reply_t;


/**
 * TYPENAME: _ix_cc_atmsar_async_vc_remove_request_t
 * 
 * DESCRIPTION:
 *    The structure defines format of request for async msg 'VC remove'
 */
typedef struct ix_s_cc_atmsar_async_vc_remove_request
{
    ix_cc_atmsar_instance_t sarInstance;
    ix_cc_atmsar_vc_handle_t handle;
} _ix_cc_atmsar_async_vc_remove_request_t;


/**
 * TYPENAME: _ix_cc_atmsar_async_vc_remove_reply_t
 * 
 * DESCRIPTION:
 *    The structure defines format of reply for async msg 'VC remove'
 */
typedef struct ix_s_cc_atmsar_async_vc_remove_reply
{
    ix_cc_atmsar_vc_handle_t handle;
} _ix_cc_atmsar_async_vc_remove_reply_t;


/**
 * TYPENAME: _ix_cc_atmsar_async_vc_stats_request_t
 * 
 * DESCRIPTION:
 *    The structure defines format of request for async msg 'VC stats'
 */
typedef struct ix_s_cc_atmsar_async_vc_stats_request
{
    ix_cc_atmsar_instance_t sarInstance;
    ix_cc_atmsar_vc_handle_t handle;
} _ix_cc_atmsar_async_vc_stats_request_t;


/**
 * TYPENAME: _ix_cc_atmsar_async_vc_stats_reply_t
 * 
 * DESCRIPTION:
 *    The structure defines format of reply for async msg 'VC stats'
 */
typedef struct ix_s_cc_atmsar_async_vc_stats_reply
{
    ix_cc_atmsar_vc_stats_t stats;
} _ix_cc_atmsar_async_vc_stats_reply_t;


/**
 * TYPENAME: _ix_cc_atmsar_async_port_create_request_t
 * 
 * DESCRIPTION:
 *    The structure defines format of request for async msg 'port create'
 */
typedef struct ix_s_cc_atmsar_async_port_create_request
{
    ix_cc_atmsar_instance_t sarInstance;
    ix_cc_atmsar_port_descr_t descr;
} _ix_cc_atmsar_async_port_create_request_t;


/**
 * TYPENAME: _ix_cc_atmsar_async_port_create_reply_t
 * 
 * DESCRIPTION: The structure define format of reply for async msg 'port create'
 */
typedef struct ix_s_cc_atmsar_async_port_create_reply
{
    ix_cc_atmsar_port_handle_t handle;
} _ix_cc_atmsar_async_port_create_reply_t;


/**
 * TYPENAME: _ix_cc_atmsar_async_port_remove_request_t
 * 
 * DESCRIPTION:
 *    The structure defines format of request for async msg 'port remove'
 */
typedef struct ix_s_cc_atmsar_async_port_remove_request
{
    ix_cc_atmsar_instance_t sarInstance;
    ix_cc_atmsar_port_handle_t handle;
} _ix_cc_atmsar_async_port_remove_request_t;


/**
 * TYPENAME: _ix_cc_atmsar_async_port_remove_reply_t
 * 
 * DESCRIPTION:
 *    The structure defines format of reply for async msg 'port remove'
 */
typedef struct ix_s_cc_atmsar_async_port_remove_reply
{
    ix_cc_atmsar_port_handle_t handle;
} _ix_cc_atmsar_async_port_remove_reply_t;


/**
 * TYPENAME: _ix_cc_atmsar_async_port_stats_request_t
 * 
 * DESCRIPTION:
 *    The structure define format of request for async msg 'port stats'
 */
typedef struct ix_s_cc_atmsar_async_port_stats_request
{
    ix_cc_atmsar_instance_t sarInstance;
    ix_cc_atmsar_port_handle_t handle;
} _ix_cc_atmsar_async_port_stats_request_t;

/**
 * TYPENAME: _ix_cc_atmsar_async_port_stats_reply_t
 * 
 * DESCRIPTION:
 *    The structure defines format of reply for async msg 'port stats'
 */
typedef struct ix_s_cc_atmsar_async_port_stats_reply
{
    ix_cc_atmsar_port_stats_t stats;
} _ix_cc_atmsar_async_port_stats_reply_t;


/**
 * TYPENAME: _ix_cc_atmsar_async_cid_create_request_t
 * 
 * DESCRIPTION:
 *    The structure defines format of request for async msg 'cid create'
 */
typedef struct ix_s_cc_atmsar_async_cid_create_request
{
    ix_cc_atmsar_instance_t sarInstance;
    ix_cc_atmsar_cid_descr_t descr;
} _ix_cc_atmsar_async_cid_create_request_t;


/**
 * TYPENAME: _ix_cc_atmsar_async_cid_create_reply_t
 * 
 * DESCRIPTION:
 *    The structure defines format of reply for async msg 'cid create'
 */
typedef struct ix_s_cc_atmsar_async_cid_create_reply
{
    ix_cc_atmsar_cid_handle_t handle;
} _ix_cc_atmsar_async_cid_create_reply_t;

/**
 * TYPENAME: _ix_cc_atmsar_async_cid_update_request_t
 * 
 * DESCRIPTION:
 *    The structure defines format of request for async msg 'CID update'
 */
typedef struct ix_s_cc_atmsar_async_cid_update_request
{
    ix_cc_atmsar_instance_t sarInstance;
    ix_cc_atmsar_cid_handle_t handle;
    ix_cc_atmsar_update_data_t updateData;
} _ix_cc_atmsar_async_cid_update_request_t;

/**
 * TYPENAME: _ix_cc_atmsar_async_cid_update_reply_t
 * 
 * DESCRIPTION:
 *    The structure defines format of reply for async msg 'cid update'
 */
typedef struct ix_s_cc_atmsar_async_cid_update_reply
{
    ix_cc_atmsar_cid_handle_t handle;
} _ix_cc_atmsar_async_cid_update_reply_t;


/**
 * TYPENAME: _ix_cc_atmsar_async_port_cid_request_t
 * 
 * DESCRIPTION:
 *    The structure defines format of request for async msg 'cid remove'
 */
typedef struct ix_s_cc_atmsar_async_cid_remove_request
{
    ix_cc_atmsar_instance_t sarInstance;
    ix_cc_atmsar_cid_handle_t handle;
} _ix_cc_atmsar_async_cid_remove_request_t;


/**
 * TYPENAME: _ix_cc_atmsar_async_cid_remove_reply_t
 * 
 * DESCRIPTION:
 *    The structure defines format of reply for async msg 'cid remove'
 */
typedef struct ix_s_cc_atmsar_async_cid_remove_reply
{
    ix_cc_atmsar_cid_handle_t handle;
} _ix_cc_atmsar_async_cid_remove_reply_t;


/**
 * TYPENAME: _ix_cc_atmsar_async_cid_stats_request_t
 * 
 * DESCRIPTION:
 *    The structure defines format of request for async msg 'cid stats'
 */
typedef struct ix_s_cc_atmsar_async_cid_stats_request
{
    ix_cc_atmsar_instance_t sarInstance;
    ix_cc_atmsar_cid_handle_t handle;
} _ix_cc_atmsar_async_cid_stats_request_t;


/**
 * TYPENAME: _ix_cc_atmsar_async_cid_stats_reply_t
 * 
 * DESCRIPTION:
 *    The structure defines format of reply for async msg 'cid stats'
 */
typedef struct ix_s_cc_atmsar_async_cid_stats_reply
{
    ix_cc_atmsar_cid_stats_t stats;
} _ix_cc_atmsar_async_cid_stats_reply_t;


/**
 * TYPENAME: _ix_cc_atmsar_async_reply_t
 * 
 * DESCRIPTION:
      The structure defines common struct for all async msg replies.
 */
typedef union ix_u_cc_atmsar_async_reply
{
    _ix_cc_atmsar_async_vc_create_reply_t vcCreate;
    _ix_cc_atmsar_async_vc_update_reply_t vcUpdate;    
    _ix_cc_atmsar_async_vc_remove_reply_t vcRemove;
    _ix_cc_atmsar_async_vc_stats_reply_t vcStats;
    _ix_cc_atmsar_async_port_create_reply_t portCreate;
    _ix_cc_atmsar_async_port_remove_reply_t portRemove;
    _ix_cc_atmsar_async_port_stats_reply_t portStats;
    _ix_cc_atmsar_async_cid_create_reply_t cidCreate;
    _ix_cc_atmsar_async_cid_update_reply_t cidUpdate;
    _ix_cc_atmsar_async_cid_remove_reply_t cidRemove;
    _ix_cc_atmsar_async_cid_stats_reply_t cidStats;
} _ix_cc_atmsar_async_reply_t;


/**
 * TYPENAME: _ix_cc_atmsar_async_request_t
 * 
 * DESCRIPTION:
 *    The structure defines common struct for all async msg requests.
 */
typedef union ix_u_cc_atmsar_async_request
{
    _ix_cc_atmsar_async_vc_create_request_t vcCreate;
    _ix_cc_atmsar_async_vc_update_request_t vcUpdate;    
    _ix_cc_atmsar_async_vc_remove_request_t vcRemove;
    _ix_cc_atmsar_async_vc_stats_request_t vcStats;
    _ix_cc_atmsar_async_port_create_request_t portCreate;
    _ix_cc_atmsar_async_port_remove_request_t portRemove;
    _ix_cc_atmsar_async_port_stats_request_t portStats;
    _ix_cc_atmsar_async_cid_create_request_t cidCreate;
    _ix_cc_atmsar_async_cid_remove_request_t cidRemove;
    _ix_cc_atmsar_async_cid_stats_request_t cidStats;
} _ix_cc_atmsar_async_request_t;


/******************************************************************************
 *    FORMAT OF MESSAGES USED BETWEEN MAIN AND AGENT ATMSAR SUBCOMPONENTS
 *****************************************************************************/

/**
 * TYPENAME: _ix_cc_atmsar_agent_vc_create_request_t
 * 
 * DESCRIPTION:
 *    The structure defines format of request msg 'VC create' sent by ATM SAR
 *    main component to ATM SAR agent
 */
typedef struct ix_s_cc_atmsar_agent_vc_create_request
{
    ix_cc_atmsar_instance_t sarInstance;
    ix_cc_atmsar_vc_handle_t handle;
    ix_cc_atmsar_vc_descr_t descr;
} _ix_cc_atmsar_agent_vc_create_request_t;

/**
 * TYPENAME: _ix_cc_atmsar_agent_vc_update_request_t
 * 
 * DESCRIPTION:
 *    The structure defines format of request msg 'VC create' sent by ATM SAR
 *    main component to ATM SAR agent
 */
typedef struct ix_s_cc_atmsar_agent_vc_update_request
{
    ix_cc_atmsar_instance_t sarInstance;
    ix_cc_atmsar_vc_handle_t handle;
    ix_cc_atmsar_update_data_t descr;
} _ix_cc_atmsar_agent_vc_update_request_t;

/**
 * TYPENAME: _ix_cc_atmsar_agent_port_create_request_t
 * 
 * DESCRIPTION:
 *    The structure defines format of request msg 'port create' sent by ATM SAR
 *    main component to ATM SAR agent
 */
typedef struct ix_s_cc_atmsar_agent_port_create_request
{
    ix_cc_atmsar_instance_t sarInstance;
    ix_cc_atmsar_port_handle_t handle;
    ix_cc_atmsar_port_descr_t descr;
} _ix_cc_atmsar_agent_port_create_request_t;


/**
 * TYPENAME: _ix_cc_atmsar_agent_cid_create_request_t
 * 
 * DESCRIPTION:
 *    The structure defines format of request msg 'cid create' sent by ATM SAR
 *    main component to ATM SAR agent
 */
typedef struct ix_s_cc_atmsar_agent_cid_create_request
{
    ix_cc_atmsar_instance_t sarInstance;
    ix_cc_atmsar_cid_handle_t handle;
    ix_cc_atmsar_cid_descr_t descr;
} _ix_cc_atmsar_agent_cid_create_request_t;


/**
 * TYPENAME: _ix_cc_atmsar_agent_cid_update_request_t
 * 
 * DESCRIPTION:
 *    The structure defines format of request msg 'cid update' sent by ATM SAR
 *    main component to ATM SAR agent
 */
typedef struct ix_s_cc_atmsar_agent_cid_update_request
{
    ix_cc_atmsar_instance_t sarInstance;
    ix_cc_atmsar_cid_handle_t handle;
    ix_cc_atmsar_update_data_t update;
} _ix_cc_atmsar_agent_cid_update_request_t;

/**
 * TYPENAME: _ix_cc_atmsar_agent_init_notify_t
 * 
 * DESCRIPTION:
 *    The structure defines format of notification about initialisation state
 *    sent by ATM SAR agent component to ATM SAR main
 */
typedef struct ix_s_cc_atmsar_agent_init_notify
{
    ix_cc_atmsar_instance_t sarInstance;

#define _IX_CC_ATMSAR_INIT_MASK_PLUGIN      0x001
#define _IX_CC_ATMSAR_INIT_MASK_PORT_MGMNT  0x002
#define _IX_CC_ATMSAR_INIT_MASK_VC_MGMNT    0x004
#define _IX_CC_ATMSAR_INIT_MASK_CID_MGMNT   0x008
#define _IX_CC_ATMSAR_INIT_MASK_ALL_MGMNT   (_IX_CC_ATMSAR_INIT_MASK_PORT_MGMNT | _IX_CC_ATMSAR_INIT_MASK_VC_MGMNT | _IX_CC_ATMSAR_INIT_MASK_CID_MGMNT)
    ix_uint8 flags;
} _ix_cc_atmsar_agent_init_notify_t;


/**
 * DESCRIPTION:
 *    The most of definition of message API between MAIN and AGENT is only
 *    redefined from base asynchronous helper messages.
 */
typedef _ix_cc_atmsar_async_vc_create_reply_t _ix_cc_atmsar_agent_vc_create_reply_t;
typedef _ix_cc_atmsar_async_vc_update_reply_t _ix_cc_atmsar_agent_vc_update_reply_t;
typedef _ix_cc_atmsar_async_vc_remove_request_t _ix_cc_atmsar_agent_vc_remove_request_t;
typedef _ix_cc_atmsar_async_vc_remove_reply_t _ix_cc_atmsar_agent_vc_remove_reply_t;
typedef _ix_cc_atmsar_async_vc_stats_request_t _ix_cc_atmsar_agent_vc_stats_request_t;
typedef _ix_cc_atmsar_async_vc_stats_reply_t _ix_cc_atmsar_agent_vc_stats_reply_t;

typedef _ix_cc_atmsar_async_port_create_reply_t _ix_cc_atmsar_agent_port_create_reply_t;
typedef _ix_cc_atmsar_async_port_remove_request_t _ix_cc_atmsar_agent_port_remove_request_t;
typedef _ix_cc_atmsar_async_port_remove_reply_t _ix_cc_atmsar_agent_port_remove_reply_t;
typedef _ix_cc_atmsar_async_port_stats_request_t _ix_cc_atmsar_agent_port_stats_request_t;
typedef _ix_cc_atmsar_async_port_stats_reply_t _ix_cc_atmsar_agent_port_stats_reply_t;

typedef _ix_cc_atmsar_async_cid_create_reply_t _ix_cc_atmsar_agent_cid_create_reply_t;
typedef _ix_cc_atmsar_async_cid_update_reply_t _ix_cc_atmsar_agent_cid_update_reply_t;
typedef _ix_cc_atmsar_async_cid_remove_request_t _ix_cc_atmsar_agent_cid_remove_request_t;
typedef _ix_cc_atmsar_async_cid_remove_reply_t _ix_cc_atmsar_agent_cid_remove_reply_t;
typedef _ix_cc_atmsar_async_cid_stats_request_t _ix_cc_atmsar_agent_cid_stats_request_t;
typedef _ix_cc_atmsar_async_cid_stats_reply_t _ix_cc_atmsar_agent_cid_stats_reply_t;


/**
 * TYPENAME: _ix_cc_atmsar_agent_request_t
 * 
 * DESCRIPTION:
 *    TBD
 */
typedef union ix_u_cc_atmsar_agent_request
{
    _ix_cc_atmsar_agent_vc_create_request_t vcCreate;
    _ix_cc_atmsar_async_vc_update_request_t vcUpdate;    
    _ix_cc_atmsar_agent_vc_remove_request_t vcRemove;
    _ix_cc_atmsar_agent_vc_stats_request_t vcStats;
    _ix_cc_atmsar_agent_port_create_request_t portCreate;
    _ix_cc_atmsar_agent_port_remove_request_t portRemove;
    _ix_cc_atmsar_agent_port_stats_request_t portStats;
    _ix_cc_atmsar_agent_cid_create_request_t cidCreate;
    _ix_cc_atmsar_agent_cid_update_request_t cidUpdate;    
    _ix_cc_atmsar_agent_cid_remove_request_t cidRemove;
    _ix_cc_atmsar_agent_cid_stats_request_t cidStats;
} _ix_cc_atmsar_agent_request_t;


/**
 * TYPENAME: _ix_cc_atmsar_agent_reply_general_t
 * 
 * DESCRIPTION:
 *    The structure defines common struct for all async msg replies.
 */
typedef union ix_u_cc_atmsar_agent_reply_general
{
    _ix_cc_atmsar_agent_vc_create_reply_t vcCreate;
    _ix_cc_atmsar_agent_vc_update_reply_t vcUpdate;        
    _ix_cc_atmsar_agent_vc_remove_reply_t vcRemove;
    _ix_cc_atmsar_agent_vc_stats_reply_t vcStats;
    _ix_cc_atmsar_agent_port_create_reply_t portCreate;
    _ix_cc_atmsar_agent_port_remove_reply_t portRemove;
    _ix_cc_atmsar_agent_port_stats_reply_t portStats;
    _ix_cc_atmsar_agent_cid_create_reply_t cidCreate;
    _ix_cc_atmsar_agent_cid_update_reply_t cidUpdate;
    _ix_cc_atmsar_agent_cid_remove_reply_t cidRemove;
    _ix_cc_atmsar_agent_cid_stats_reply_t cidStats;
} _ix_cc_atmsar_agent_reply_general_t;


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(___IX_CC_ATMSAR_MSG_H__) */

