/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor, Release 5
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      ATM SAR Core Component
 *
 * = FILENAME
 *      ix_cc_atmsar.h
 *
 * = DESCRIPTION
 *      This file defines all exported data types and function prototypes
 *
 * = AUTHOR
 *       Jaroslaw Kogut
 *       jaroslaw.kogut@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *     
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * 
 */

#if !defined(__IX_CC_ATMSAR_H__)
#define __IX_CC_ATMSAR_H__

#include "ix_rm.h"
#include "ix_ossl.h"
#include "ix_cc.h"
#include "ix_cc_properties.h"

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

/******************************************************************************
 * IMPORTANT NOTE: Compilation flags
 *
 * ATMSAR_AGENT_ON_EGRESS=1
 *    Component SAR Control Agent is executed on egress processor.
 *
 * ATMSAR_MAIN_WITH_AGENT_SUPPORT=1
 *    SAR COntrol has only MAIN part. Calling to Agent is not performed.
 *
 * ATMSAR_AGENT=1
 *    SAR COntrol AGENT compiled in application.
 *
 * ATMSAR_MAIN=1
 *    SAR COntrol MAIN compiled in build.
 *
 * ATMSAR_MAIN_REMOTE=1
 * ATMSAR_AGENT_REMOTE=1
 *    ATM SAR can be located in defferent IXP processor/process.
 *    The flag should be set or not for application that includes SAR Control
 *    Agent, Main  or core components that use asynchronous helper functions
 *    provided in the API, defined in the file.
 *
 *****************************************************************************/

/*******************************************************************************
 *                     SAR instances support
 ******************************************************************************/

/**
 * TYPENAME: ix_cc_atmsar_instance_t
 * 
 * DESCRIPTION:
 *    TBD
 */
typedef ix_uint16 ix_cc_atmsar_instance_t;

/*******************************************************************************
 *                     OAM Support API declaration
 ******************************************************************************/

/**
 * TYPENAME: ix_cc_atmsar_oam_request_t
 * 
 * DESCRIPTION:
 *    TBD
 */
typedef enum ix_e_cc_atmsar_oam_request {
    IX_CC_ATMSAR_OAM_REQ_GLOBAL_STATS, /**< global statistics (collected for
                                          the interface and all VCs) */
    IX_CC_ATMSAR_OAM_REQ_VC_STATS,     /**< per VC statistics        */
    IX_CC_ATMSAR_OAM_REQ_AAL5_STATS,   /**< per AAL5 interface (used for general
                                          statistics) */
} ix_cc_atmsar_oam_request_t;


/**
 * TYPENAME: ix_cc_atmsar_oam_get_params_req_t
 * 
 * DESCRIPTION:
 *   The IX_CC_ATMSAR module sends this message to return statistics collected by the
 *   OAM support module, as requested in Ix_Cc_Atmsar_OamGetReq.
 */
typedef struct ix_s_cc_atmsar_oam_params
{
    ix_uint16 type;         /**< Type of requested information. One of:
                               IX_CC_ATMSAR_GLOBAL_STATS - global statistics (collected for
                               the interface and all VCs)
                               IX_CC_ATMSAR_VC_STATS - per VC statistics */
    ix_uint16 vcId;         /**< VC index. Valid only when type= IX_CC_ATMSAR_VC_STATS */
    ix_uint32 rxOamCells;   /**< Number of received OAM cells */
    ix_uint32 txOamCells;   /**< Number of sent OAM cells */
    ix_uint32 rxOamF5Cells;   /**< Number of received OAM F5 cells */
    ix_uint32 txOamF5Cells;   /**< Number of sent OAM F5 cells */
#define IX_CC_ATMSAR_OAM_ENABLED 0   /**< @ingroup IX_CC_ATMSARApi - enabled */
#define IX_CC_ATMSAR_OAM_DISABLED 1   /**< @ingroup IX_CC_ATMSARApi - disabled */
    ix_uint16 lbState;        /**< OAM loopback administrative state. Defined per
                                 interface. One of:
                                 IX_CC_ATMSAR_OAM_ENABLED - - enabled
                                 IX_CC_ATMSAR_OAM_DISABLED - - disabled */
#define IX_CC_ATMSAR_OAM_NORMAL 0   /**< @ingroup IX_CC_ATMSARApi normal */
#define IX_CC_ATMSAR_OAM_AIS 1   /**< @ingroup IX_CC_ATMSARApi - VC in AIS state */
    ix_uint16 oamState;       /**< VC OAM state. Valid only when type IX_CC_ATMSAR_VC_STATS
                                 specified. One of:
                                 IX_CC_ATMSAR_OAM_NORMAL - normal
                                 IX_CC_ATMSAR_OAM_AIS - - VC in AIS state */
} ix_cc_atmsar_oam_params_t;


/**
 * TYPENAME: ix_cc_atmsar_oam_pm_size_t
 * 
 * DESCRIPTION:
 *    The data type defines supported by OAM Performance Monitoring sizes of
 *    cell blocks:
 */
typedef enum ix_e_cc_atmsar_perf_monitoring_size {
    IX_CC_ATMSAR_OAM_PM_128_cells, 
    IX_CC_ATMSAR_OAM_PM_256_cells, 
    IX_CC_ATMSAR_OAM_PM_512_cells, 
    IX_CC_ATMSAR_OAM_PM_1024_cells
} ix_cc_atmsar_oam_pm_size_t;


/**
 * TYPENAME: ix_cc_atmsar_oam_specific_t
 * 
 * DESCRIPTION:
 *    The data type defines supported OAM specific parameters.
 *    NOTE: The OAM parameters are not used in current release of microblocks.
 *    It should be set to 0.
 */
typedef struct ix_s_cc_atmsar_OAM_specific {
    ix_uint16  queueNum;
    ix_uint8   perfMonitoringAdminState;
    ix_cc_atmsar_oam_pm_size_t   perfMonitoringBlockSize;
} ix_cc_atmsar_oam_specific_t;


/*******************************************************************************
 *                  Traffic Profile support API declaration
 ******************************************************************************/

/**
 * TYPENAME: ix_cc_atmsar_service_class_t
 * 
 * DESCRIPTION:
 *    The data type defines supported service class for VC
 */
typedef enum ix_e_cc_atmsar_service_class {
    IX_CC_ATMSAR_ATM_SERVICE_NONE = 0,
    IX_CC_ATMSAR_ATM_SERVICE_CBR,
    IX_CC_ATMSAR_ATM_SERVICE_GFR,
    IX_CC_ATMSAR_ATM_SERVICE_RT_VBR,
    IX_CC_ATMSAR_ATM_SERVICE_NRT_VBR,
    IX_CC_ATMSAR_ATM_SERVICE_ABR,
    IX_CC_ATMSAR_ATM_SERVICE_UBR,
} ix_cc_atmsar_service_class_t;


/**
 * TYPENAME: ix_cc_atmsar_ubr_behavior_class_t
 * 
 * DESCRIPTION:
 *    The data type defines supported UBR behavior class for VC
 */
typedef enum ix_e_cc_atmsar_ubr_bsc {
    IX_CC_ATMSAR_UBR_BCS_NO_PCR=0,   /**< without PCR, it is with priority */
    IX_CC_ATMSAR_UBR_BCS_PCR,        /**< with PCR                         */
    IX_CC_ATMSAR_UBR_BCS_MDCR,       /**< TBD Toma schould explain         */
} ix_cc_atmsar_ubr_bsc_t;


/**
 * TYPENAME: ix_cc_atmsar_tp_direction2_t
 * 
 * DESCRIPTION:
 *   This structure describes traffic parameters for one of VC direction
 *   (forward or backward).
 */
typedef struct ix_s_cc_atmsar_traffic_direction
{
    ix_uint32   pcr;            /**< peak  cell rate,                      */
    ix_uint32   scr;            /**< sust. cell rate,                      */
    ix_uint32   mbs;            /**< mean burst size,                      */
    ix_uint32   mfs;            /**< maximum frame size.                   */
    ix_uint32   mcr;            /**< min. cell rate,(for ABR and GFR       */
    ix_uint8    cdvt;           /**< cell delay variation tolerance        */
    ix_uint8    tagging;        /**< tagging requested in the direction    */
    ix_uint8    frmDisc;        /**< frame discard in rx direction         */
    ix_uint8    etoeOamF5FlInd; /**< e-to-e OAM F5 flow indicator          */
    ix_uint8    ubrBcs;         /**< behavior class selector, valid only
                                      for UBR service class. Allowed value
                                      See ix_cc_atmsar_ubr_bsc_t type */
    ix_uint8    ubrPriority;    /**< pritity for VC with traffic type
                                   UBR with prioroty */
} ix_cc_atmsar_tp_direction_t;


/**
 * TYPENAME: ix_cc_atmsar_traffic_param_t
 * 
 * DESCRIPTION:
 *   The structure describes traffic parameters used for 'create VC' operation.
 */
typedef struct ix_cc_atmsar_traffic_param
{
    ix_uint16   serviceClass;      /**< Allowed value see:
                                      ix_cc_atmsar_service_class_t     */
    ix_uint8    bstEffortReq;      /**< best effort requested           */
    ix_cc_atmsar_tp_direction_t bwd; /**< backward direction */
    ix_cc_atmsar_tp_direction_t fwd; /**< forward direction  */
} ix_cc_atmsar_traffic_param_t;


/*******************************************************************************
 *                      VC API declaration
 ******************************************************************************/


#define IX_CC_ATMSAR_VCID_INVALID             0xFFFF

/**
 * TYPENAME: ix_cc_atmsar_vc_handler
 * 
 * DESCRIPTION: This structure defines format for statistics data.
 *
 */
typedef ix_uint16 ix_cc_atmsar_vc_handle_t;


/**
 * TYPENAME: ix_cc_atmsar_vc_spec_t
 * 
 * DESCRIPTION:
 *    The data type defines VC identification parameters
 */
typedef struct ix_s_cc_atmsar_vc_spec {
    ix_uint16 vpi;
    ix_uint16 vci;
    ix_uint16 portNo;
} ix_cc_atmsar_vc_spec_t;


/**
 * TYPENAME: ix_cc_atmsar_aal
 * 
 * DESCRIPTION:
 *    The data type defines supported AAL types for VC
 */
typedef enum ix_e_cc_atmsar_aal_type {
    IX_CC_ATMSAR_AAL0, 
    IX_CC_ATMSAR_AAL1, 
    IX_CC_ATMSAR_AAL2, 
    IX_CC_ATMSAR_AAL34,
    IX_CC_ATMSAR_AAL5, 
    IX_CC_ATMSAR_AALRAW
} ix_cc_atmsar_aal_t;


/**
 * TYPENAME: ix_cc_atmsar_vc_params_t
 * 
 * DESCRIPTION:
 *   This structure describes all parameters for new created VC The Soft SAR can
 *   allocate data structures required for a new VC and configure the SAR
 *   hardware.
 */
typedef struct ix_s_cc_atmsar_vc_params
{
    ix_cc_atmsar_vc_spec_t       spec;                /**< portNo/vpi/vci        */
    ix_cc_atmsar_traffic_param_t trafficParams;   /**< VC traffic parameters */
    ix_cc_atmsar_oam_specific_t  oam;
    ix_uint8  aalType;           /**< allowed values - see ix_cc_atmsar_aal_t */
    ix_uint8  hdrType;
    ix_uint16 freeListId;        /**< Id of free list from which RX can allocate buffer */
    ix_uint16 l2port;
    ix_uint16 outputQueueNum;
#define IX_CC_ATMSAR_PACKET_PATH_REGULAR   0
#define IX_CC_ATMSAR_PACKET_PATH_EXCEPTION 1   
    ix_uint8  packetPath;        /**< RX data packet destination. It can be
                                    'excaption' if data from VC has to be handled by
                                    XScale core or is has to be 'regular' if data
                                    from VC should be processed with regular path */
} ix_cc_atmsar_vc_descr_t;


/**
 * TYPENAME: ix_cc_atmsar_update_data_t
 * 
 * DESCRIPTION:
 *    This structure describes all parameters for binding external object to
 *    ATMSAR object.
 */
typedef struct ix_s_cc_atmsar_update_data
{
    ix_uint32 lw0;
    ix_uint32 lw1;
} ix_cc_atmsar_update_data_t;


/******************************************************************************
 *                      PORT RELATED DEFINITIONS
 *****************************************************************************/


#define IX_CC_ATMSAR_PORTID_INVALID            0xFFFF


/**
 * TYPENAME: ix_cc_atmsar_port_handle_t
 * 
 * DESCRIPTION:
 *    TBD
 */
typedef ix_uint16 ix_cc_atmsar_port_handle_t;


/**
 * TYPENAME: ix_cc_atmsar_port_bandwidth_id
 * 
 * DESCRIPTION:
 *    This structure defines format for statistics data.
 */
typedef enum ix_e_cc_atmsar_port_bandwidth
{
    IX_CC_ATMSAR_PORT_BANDWIDTH_ID1=0,
    IX_CC_ATMSAR_PORT_BANDWIDTH_ID2,
    IX_CC_ATMSAR_PORT_BANDWIDTH_ID3,
    IX_CC_ATMSAR_PORT_BANDWIDTH_ID4,
    IX_CC_ATMSAR_PORT_BANDWIDTH_ID5,
    IX_CC_ATMSAR_PORT_BANDWIDTH_ID6,
    IX_CC_ATMSAR_PORT_BANDWIDTH_ID7,
    IX_CC_ATMSAR_PORT_BANDWIDTH_ID8,
    IX_CC_ATMSAR_PORT_BANDWIDTH_ID9,
    IX_CC_ATMSAR_PORT_BANDWIDTH_ID10,
    IX_CC_ATMSAR_PORT_BANDWIDTH_ID11,
    IX_CC_ATMSAR_PORT_BANDWIDTH_ID12
} ix_cc_atmsar_port_bandwidth_id_t;

#define IX_CC_ATMSAR_PORT_BANDWIDTH_DS0  IX_CC_ATMSAR_PORT_BANDWIDTH_ID1
#define IX_CC_ATMSAR_PORT_BANDWIDTH_OC3  IX_CC_ATMSAR_PORT_BANDWIDTH_ID8
#define IX_CC_ATMSAR_PORT_BANDWIDTH_OC12 IX_CC_ATMSAR_PORT_BANDWIDTH_ID10
#define IX_CC_ATMSAR_PORT_BANDWIDTH_OC48 IX_CC_ATMSAR_PORT_BANDWIDTH_ID12

/**
 * TYPENAME: ix_cc_atmsar_port_descr_t
 * 
 * DESCRIPTION:
 *    The data type defines all port parameters that are required for port adding operation:
 */
typedef struct ix_s_cc_atmsar_port_descr {
    ix_uint16 portId;
    ix_cc_atmsar_port_bandwidth_id_t bandwidth;
} ix_cc_atmsar_port_descr_t;


/******************************************************************************
 *                      CID RELATED DEFINITIONS
 *****************************************************************************/

#define IX_CC_ATMSAR_CIDID_INVALID            0xFFFF


/**
 * TYPENAME: ix_cc_atmsar_cid_handle_t
 * 
 * DESCRIPTION:
 *    TBD
 */
typedef ix_uint16 ix_cc_atmsar_cid_handle_t;


/**
 * TYPENAME: ix_cc_atmsar_cid_descr_t
 * 
 * DESCRIPTION:
 *    The data type defines all cid parameters that are required for CID
 *    create operation.
 */
typedef struct ix_s_cc_atmsar_cid_descr {
    ix_cc_atmsar_vc_handle_t vc; /**< VC handler */
    ix_uint8 cid;                  /**< CID identifier */
} ix_cc_atmsar_cid_descr_t;


/******************************************************************************
 *                      STATISTIC RELATED DEFINITIONS
 *****************************************************************************/

/**
 * TYPENAME: ix_cc_atmsar_stats_t
 * 
 * DESCRIPTION:
 *    Statistical Counters definition - structure used for all types of
 *    statistics - global, per AAL5, per VC
 */
typedef struct ix_s_cc_atmsar_stats
{
    struct {
        ix_uint64 packets;   /**< Number of received packets */
        ix_uint64 cells;   /**< Number of received cells */
        ix_uint64 discards;   /**< Number of rx packets discarded */
        ix_uint64 cellErrors;   /**< Number of cells received with HEC errors */
    } rx;
    struct {
        ix_uint64 packets;   /**< Number of transmitted packets */
        ix_uint64 cells;   /**< Number of transmitted cells */
        ix_uint64 discards;   /**< Number of tx packets discarded */
    } tx;
} ix_cc_atmsar_stats_t;


/**
 * TYPENAME: ix_cc_atmsar_vc_stats_t
 * 
 * DESCRIPTION:
 */
typedef ix_cc_atmsar_stats_t ix_cc_atmsar_vc_stats_t;

/**
 * TYPENAME: ix_cc_atmsar_port_stats_t
 * 
 * DESCRIPTION:
 */
typedef ix_cc_atmsar_stats_t ix_cc_atmsar_port_stats_t;

/**
 * TYPENAME: ix_cc_atmsar_port_stats_t
 * 
 * DESCRIPTION:
 */
typedef ix_cc_atmsar_stats_t ix_cc_atmsar_cid_stats_t;

/******************************************************************************
 *                      CALLBACK FUNTION FORMAT DEFINITIONS
 *****************************************************************************/

/**
 * TYPENAME: ix_cc_atmrx_cb_xxxxx
 * 
 * DESCRIPTION:
 *    Type definition for general callback functions
 */
typedef ix_error (*ix_cc_atmsar_cb_vc_create_t) (
ix_error arg_Result,
ix_cc_atmsar_vc_handle_t arg_hVc,
void* arg_pContext);

typedef ix_error (*ix_cc_atmsar_cb_vc_update_t) (
ix_error arg_Result,
ix_cc_atmsar_vc_handle_t arg_hVc,
void* arg_pContext);

typedef ix_error (*ix_cc_atmsar_cb_vc_remove_t) (
ix_error arg_Result,
void* arg_pContext);

typedef ix_error (*ix_cc_atmsar_cb_get_vc_stats_t) (
ix_error arg_Result,
ix_cc_atmsar_vc_stats_t *arg_pLookupResult,
void* arg_pContext);

typedef ix_error (*ix_cc_atmsar_cb_port_create_t) (
ix_error arg_Result,
ix_cc_atmsar_port_handle_t port,
void* arg_pContext);

typedef ix_error (*ix_cc_atmsar_cb_port_remove_t) (
ix_error arg_Result,
void* arg_pContext);

typedef ix_error (*ix_cc_atmsar_cb_get_port_stats_t) (
ix_error arg_Result,
ix_cc_atmsar_port_stats_t *arg_pLookupResult,
void* arg_pContext);

typedef ix_error (*ix_cc_atmsar_cb_cid_create_t) (
ix_error arg_Result,
ix_cc_atmsar_cid_handle_t port,
void* arg_pContext);

typedef ix_error (*ix_cc_atmsar_cb_cid_update_t) (
ix_error arg_Result,
ix_cc_atmsar_cid_handle_t port,
void* arg_pContext);

typedef ix_error (*ix_cc_atmsar_cb_cid_remove_t) (
ix_error arg_Result,
void* arg_pContext);

typedef ix_error (*ix_cc_atmsar_cb_get_cid_stats_t) (
ix_error arg_Result,
ix_cc_atmsar_cid_stats_t *arg_pLookupResult,
void* arg_pContext);

/**
 * TYPENAME: ix_cc_atmsar_cb_t
 * 
 * DESCRIPTION: This union defines format for TBD
 *
 */
typedef union {
    ix_cc_atmsar_cb_vc_create_t vcCreate;
    ix_cc_atmsar_cb_vc_update_t vcUpdate;
    ix_cc_atmsar_cb_vc_remove_t vcRemove;
    ix_cc_atmsar_cb_get_vc_stats_t vcStats;
    ix_cc_atmsar_cb_port_create_t portCreate;
    ix_cc_atmsar_cb_port_remove_t portRemove;
    ix_cc_atmsar_cb_get_port_stats_t portStats;
    ix_cc_atmsar_cb_cid_create_t cidCreate;
    ix_cc_atmsar_cb_cid_update_t cidUpdate;    
    ix_cc_atmsar_cb_cid_remove_t cidRemove;
    ix_cc_atmsar_cb_get_cid_stats_t cidStats;
} ix_cc_atmsar_cb_t;



/**
 * TYPENAME: ix_cc_atmsar_error_code_t
 * 
 * DESCRIPTION: This enumerated type will describe atmsar error codes
 *
 */
typedef enum ix_e_cc_atmsar_error_code
{
    IX_CC_ATMSAR_ERROR_FIRST = IX_ERROR_MAKE_GROUP(CC_ATMSAR),
    IX_CC_ATMSAR_ERROR_SUCCESS = IX_CC_ATMSAR_ERROR_FIRST,

    IX_CC_ATMSAR_ERROR_FAILED_PATCHING,
    IX_CC_ATMSAR_ERROR_REGISTRY,
    IX_CC_ATMSAR_ERROR_BUFFER_FREE,

    IX_CC_ATMSAR_ERROR_CCI,
    IX_CC_ATMSAR_ERROR_INVALID_CONTEXT,
    IX_CC_ATMSAR_ERROR_INVALID_CALLBACK,
    IX_CC_ATMSAR_ERROR_INVALID_INPUT_PARAM,
    IX_CC_ATMSAR_ERROR_INVALID_BUFFER,
    IX_CC_ATMSAR_ERROR_MSG_LIBRARY,
    IX_CC_ATMSAR_ERROR_INVALID_PACKET,
    IX_CC_ATMSAR_ERROR_INVALID_MASK,

    IX_CC_ATMSAR_ERROR_INVALID_FLAGS,
    IX_CC_ATMSAR_ERROR_INVALID_PACKETS_TO_DRAIN,
    IX_CC_ATMSAR_ERROR_INVALID_PARAM,

    IX_CC_ATMSAR_ERROR_CALLBACK,

    IX_CC_ATMSAR_ERROR_BUSY,
    IX_CC_ATMSAR_ERROR_NO_RESOURCES,

    /* unknown error code */
    IX_CC_ATMSAR_ERROR_UNKNOWN,
    IX_CC_ATMSAR_ERROR_LAST
} ix_cc_atmsar_error_code_t;


/******************************************************************************
 *                    MESSAGE HELPER FUNCTIONS API
 *****************************************************************************/

/**
 * NAME: ix_cc_atmsar_async_vc_create
 *
 * DESCRIPTION:
 *    This helper function builds and sends an VC create message to the
 *    CC.
 * 
 * @Param:  IN  arg_sarInstance : SAR instance ID.
 * @Param:  IN  arg_pVcDescr : parameters describing requested VC.
 * @Param:  IN  arg_Callback : a client provided callback function 
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error ix_cc_atmsar_async_vc_create(
const ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmsar_vc_descr_t* arg_pVcDescr,
ix_cc_atmsar_cb_vc_create_t arg_Callback,
void *arg_pUserContext);

/**
 * NAME: ix_cc_atmsar_async_vc_update
 *
 * DESCRIPTION:
 *    This helper function builds and sends an VC update message to the
 *    CC.
 * 
 * @Param:  IN  arg_sarInstance : SAR instance ID.
 * @Param:  IN  arg_pVcDescr : parameters describing requested VC.
 * @Param:  IN  arg_Callback : a client provided callback function 
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error ix_cc_atmsar_async_vc_update(
const ix_cc_atmsar_instance_t arg_sarInstance,
ix_cc_atmsar_vc_handle_t arg_hVc,
const ix_cc_atmsar_update_data_t* arg_pUpdateData,
ix_cc_atmsar_cb_vc_update_t arg_Callback,
void *arg_pUserContext);

/**
 * NAME: ix_cc_atmsar_async_vc_remove
 *
 * DESCRIPTION:
 *    This helper function builds and sends an VC remove message to
 *    the CC. 
 * 
 * @Param:  IN  arg_sarInstance : SAR instance ID.
 * @Param:  IN  arg_hVc : handler to VC to be removed
 * @Param:  IN  arg_Callback : a client provided callback function 
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM- invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error ix_cc_atmsar_async_vc_remove(
const ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmsar_vc_handle_t arg_hVc,
ix_cc_atmsar_cb_vc_remove_t arg_Callback,
void *arg_pUserContext);


/**
 * NAME: ix_cc_atmsar_async_get_vc_stats
 *
 * DESCRIPTION:
 *    This helper function builds and sends an VC get stats message to
 *    the CC. 
 * 
 * @Param:  IN  arg_sarInstance : SAR instance ID.
 * @Param:  IN  arg_hVc : handler to VC to be removed
 * @Param:  IN  arg_Callback : a client provided callback function
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * @Param:  OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error ix_cc_atmsar_async_get_vc_stats(
const ix_cc_atmsar_instance_t arg_sarInstance,
ix_cc_atmsar_vc_handle_t arg_hVc,
ix_cc_atmsar_cb_get_vc_stats_t arg_Callback,
void *arg_pUserContext);


/**
 * NAME: ix_cc_atmsar_async_port_create
 *
 * DESCRIPTION:
 *    This helper function builds and sends an ATM port create message to the
 *    CC.
 * 
 * @Param:  IN  arg_sarInstance : SAR instance ID.
 * @Param:  IN  arg_pPortDescr : parameters describing requested port.
 * @Param:  IN  arg_Callback : a client provided callback function 
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error ix_cc_atmsar_async_port_create(
const ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmsar_port_descr_t* arg_pPortDescr,
ix_cc_atmsar_cb_port_create_t arg_Callback,
void *arg_pUserContext);


/**
 * NAME: ix_cc_atmsar_async_port_remove
 *
 * DESCRIPTION:
 *    This helper function builds and sends an ATM port remove message to
 *    the CC. 
 * 
 * @Param:  IN  arg_sarInstance : SAR instance ID.
 * @Param:  IN  arg_port : ATM port to be removed
 * @Param:  IN  arg_Callback : a client provided callback function 
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM- invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error ix_cc_atmsar_async_port_remove(
const ix_cc_atmsar_instance_t arg_sarInstance,
ix_cc_atmsar_port_handle_t arg_port,
ix_cc_atmsar_cb_port_remove_t arg_Callback,
void *arg_pUserContext);


/**
 * NAME: ix_cc_atmsar_async_get_port_stats
 *
 * DESCRIPTION:
 *    This helper function builds and sends an ATM port get stats message to
 *    the CC. 
 * 
 * @Param:  IN  arg_sarInstance : SAR instance ID.
 * @Param:  IN  arg_port : port ID
 * @Param:  IN  arg_pLookupResult : pointer to ix_cc_atmsar_port_stats
 *                  structure where the requested data will be stored
 * @Param:  IN  arg_Callback : a client provided callback function
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * @Param:  OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error ix_cc_atmsar_async_get_port_stats(
const ix_cc_atmsar_instance_t arg_sarInstance,
ix_cc_atmsar_port_handle_t arg_port,
ix_cc_atmsar_cb_get_port_stats_t arg_Callback,
void *arg_pUserContext);


/**
 * NAME: ix_cc_atmsar_async_cid_create
 *
 * DESCRIPTION:
 *    This helper function builds and sends an CID create message to the
 *    CC.
 * 
 * @Param:  IN  arg_sarInstance : SAR instance ID.
 * @Param:  IN  arg_pCidDescr : parameters describing requested CID.
 * @Param:  IN  arg_callback : a client provided callback function 
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error ix_cc_atmsar_async_cid_create(
const ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmsar_cid_descr_t* arg_pCidDescr,
ix_cc_atmsar_cb_cid_create_t arg_callback,
void *arg_pUserContext);

/**
 * NAME: ix_cc_atmsar_async_cid_update
 *
 * DESCRIPTION:
 *    This helper function builds and sends an CID update message to the
 *    CC.
 * 
 * @Param:  IN  arg_sarInstance : SAR instance ID.
 * @Param:  IN  arg_pUpdate : parameters describing requested CID.
 * @Param:  IN  arg_callback : a client provided callback function 
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error ix_cc_atmsar_async_cid_update(
const ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmsar_cid_handle_t arg_hCid,
const ix_cc_atmsar_update_data_t* arg_pUpdate,
ix_cc_atmsar_cb_cid_update_t arg_Callback,
void *arg_pUserContext);

/**
 * NAME: ix_cc_atmsar_async_cid_remove
 *
 * DESCRIPTION:
 *    This helper function builds and sends an CID remove message to
 *    the CC. 
 * 
 * @Param:  IN  arg_sarInstance : SAR instance ID.
 * @Param:  IN  arg_cid : CID to be removed
 * @Param:  IN  arg_Callback : a client provided callback function 
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM- invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error ix_cc_atmsar_async_cid_remove(
const ix_cc_atmsar_instance_t arg_sarInstance,
ix_cc_atmsar_cid_handle_t arg_cid,
ix_cc_atmsar_cb_cid_remove_t arg_callback,
void *arg_pUserContext);


/**
 * NAME: ix_cc_atmsar_async_get_cid_stats
 *
 * DESCRIPTION:
 *    This helper function builds and sends an CID get stats message to
 *    the CC. 
 * 
 * @Param:  IN  arg_sarInstance : SAR instance ID.
 * @Param:  IN  arg_cid : CID handler
 * @Param:  IN  arg_pLookupResult : pointer to ix_cc_atmsar_cid_stats_t
 *                  structure where the requested data will be stored
 * @Param:  IN  arg_callback : a client provided callback function
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * @Param:  OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error ix_cc_atmsar_async_get_cid_stats(
const ix_cc_atmsar_instance_t arg_sarInstance,
ix_cc_atmsar_cid_handle_t arg_cid,
ix_cc_atmsar_cb_get_cid_stats_t arg_callback,
void *arg_pUserContext);


/******************************************************************************
 *                         LIBRARY FUNCTIONS API
 *****************************************************************************/

/**
 * NAME: ix_cc_atmsar_vc_create
 *
 * DESCRIPTION:
 *    Creating VC in the ATM SAR.
 * 
 * @Param:  IN  arg_pContext : pointer to core component context.
 * @Param:  IN  arg_pVcDescr : parameters describing requested VC.
 * @Param:  OUT arg_pHVc : pointer where is places handle to created VC
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 */
ix_error ix_cc_atmsar_vc_create(
void* arg_pContext,
const ix_cc_atmsar_vc_descr_t* arg_pVcDescr,
ix_cc_atmsar_vc_handle_t* arg_pHVc);


/**
 * NAME: ix_cc_atmsar_vc_update
 *
 * DESCRIPTION:
 *    Update VC with bind parameters.
 * 
 * @Param:  IN  arg_pContext : pointer to core component context.
 * @Param:  IN  arg_hVc : handler to VC to be updated
 * @Param:  IN  arg_pUpdateData : parameters describing remote part of data stream.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error ix_cc_atmsar_vc_update(
void* arg_pContext,
ix_cc_atmsar_vc_handle_t arg_hVc,
const ix_cc_atmsar_update_data_t* arg_pUpdateData);


/**
 * NAME: ix_cc_atmsar_vc_remove
 *
 * DESCRIPTION:
 *    Remove VC from ATM SAR.
 * 
 * @Param:  IN  arg_sarInstance : SAR instance ID.
 * @Param:  IN  arg_hVc : handler to VC to be removed
 * @Param:  IN  arg_Callback : a client provided callback function 
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM- invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error ix_cc_atmsar_vc_remove(
void* arg_pContext,
const ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmsar_vc_handle_t arg_hVc);


/**
 * NAME: ix_cc_atmsar_get_vc_stats
 *
 * DESCRIPTION:
 *    Get statistic for requested VC. 
 * 
 * @Param:  IN  arg_pContext : pointer to core component context.
 * @Param:  IN  arg_hVc : handler to VC to be removed
 * @Param:  IN/OUT - pointer where statistic result is stored
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error ix_cc_atmsar_get_vc_stats(
void* arg_pContext,
ix_cc_atmsar_vc_handle_t arg_hVc,
ix_cc_atmsar_vc_stats_t *arg_pLookupResult);


/**
 * NAME: ix_cc_atmsar_port_create
 *
 * DESCRIPTION:
 *    Create port in the ATM SAR.
 * 
 * @Param:  IN  arg_pContext : pointer to core component context.
 * @Param:  IN  arg_pPortDescr : parameters describing requested port.
 * @Param:  OUT arg_pHPort : pointer where is places handle to created port
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error ix_cc_atmsar_port_create(
void* arg_pContext,
const ix_cc_atmsar_port_descr_t* arg_pPortDescr,
ix_cc_atmsar_port_handle_t *arg_pHPort);


/**
 * NAME: ix_cc_atmsar_port_remove
 *
 * DESCRIPTION:
 *    Remove port from the ATM SAR. 
 * 
 * @Param:  IN  arg_pContext : pointer to core component context.
 * @Param:  IN  arg_port : ATM port to be removed
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM- invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error ix_cc_atmsar_port_remove(
void* arg_pContext,
ix_cc_atmsar_port_handle_t arg_port);


/**
 * NAME: ix_cc_atmsar_get_port_stats
 *
 * DESCRIPTION:
 *    Get statistic for requested port. 
 * 
 * @Param:  IN  arg_pContext : pointer to core component context.
 * @Param:  IN  arg_port : port ID
 * @Param:  IN/OUT  arg_pLookupResult : pointer to ix_cc_atmsar_port_stats
 *                  structure where the requested data will be stored
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error ix_cc_atmsar_get_port_stats(
void* arg_pContext,
ix_cc_atmsar_port_handle_t arg_port,
ix_cc_atmsar_port_stats_t *arg_pLookupResult);


/**
 * NAME: ix_cc_atmsar_cid_create
 *
 * DESCRIPTION:
 *    Create CID in the ATM SAR.
 * 
 * @Param:  IN  arg_pContext : pointer to core component context.
 * @Param:  IN  arg_pCidDescr : parameters describing requested CID.
 * @Param:  IN/OUT arg_pHCid : pointer where is places handle to created port
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error ix_cc_atmsar_cid_create(
void* arg_pContext,
const ix_cc_atmsar_cid_descr_t* arg_pCidDescr,
ix_cc_atmsar_cid_handle_t* arg_pHCid);


/**
 * NAME: ix_cc_atmsar_vc_update
 *
 * DESCRIPTION:
 *    Update CID with bind parameters.
 * 
 * @Param:  IN  arg_pContext : pointer to core component context.
 * @Param:  IN  arg_hCid : handle for CID to be updated
 * @Param:  IN  arg_pUpdateData : parameters describing remote part of data stream.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error ix_cc_atmsar_cid_update(
void* arg_pContext,
ix_cc_atmsar_cid_handle_t arg_hCid,
const ix_cc_atmsar_update_data_t* arg_pUpdateData);



/**
 * NAME: ix_cc_atmsar_cid_remove
 *
 * DESCRIPTION:
 *    Remove Cid from the ATM SAR core component. 
 * 
 * @Param:  IN  arg_pContext : pointer to core component context.
 * @Param:  IN  arg_cid : CID to be removed
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM- invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error ix_cc_atmsar_cid_remove(
void* arg_pContext,
ix_cc_atmsar_cid_handle_t arg_cid);


/**
 * NAME: ix_cc_atmsar_get_cid_stats
 *
 * DESCRIPTION:
 *    Get statistic for requested CID. 
 * 
 * @Param:  IN  arg_pContext : pointer to core component context.
 * @Param:  IN  arg_cid : CID handler
 * @Param:  IN/OUT  arg_pLookupResult : pointer to ix_cc_atmsar_cid_stats_t
 *                  structure where the requested data will be stored
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error ix_cc_atmsar_get_cid_stats(
void* arg_pContext,
ix_cc_atmsar_cid_handle_t arg_cid,
ix_cc_atmsar_cid_stats_t *arg_pLookupResult);


/******************************************************************************
 *                SUPPORT FOR BITS PASSING INTO/FROM STRUCTURE
 *****************************************************************************/


#define ATMSAR_BITS_SUPPORT_FUNC_DEFINE(__funcPfx, __varName, __lw, __offset, __mask) \
   static void   __funcPfx ## _set_ ##__varName(ix_uint32 *var, ix_uint32 arg) \
                              { \
                                  var[__lw] &= (~(__mask << __offset)); \
                                  var[__lw] |= (__mask & arg) << __offset;\
                              } \
   static ix_uint32 __funcPfx## _get_ ##__varName(ix_uint32 *var) { return ((var[__lw] >> __offset) & __mask); } \
   void* __tmp_ ## __funcPfx ## _set_ ##__varName=__funcPfx ## _set_ ##__varName; \
   void* __tmp_ ## __funcPfx ## _get_ ##__varName=__funcPfx ## _get_ ##__varName; \


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_ATMSAR_H__) */

