/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor, Release 5
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      ATM SAR Core Component
 *
 * = FILENAME
 *      ix_cc_sarcrtl_list.h
 *
 * = DESCRIPTION
 *      This file defines all exported data types and function prototypes
 *      for list support accessible in ATM.
 *
 * = AUTHOR
 *       Jaroslaw Kogut
 *       jaroslaw.kogut@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *     
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * 
 */

#if !defined(__IX_CC_ATMSAR_LIST_H__)
#define __IX_CC_ATMSAR_LIST_H__

#include "ix_rm.h"
#include "ix_ossl.h"
#include "ix_cc.h"
#include "ix_cc_properties.h"

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */


/**
 * TYPENAME: ix_cc_atm_list_entry_t
 * 
 * DESCRIPTION:
 *    It is description of entry requested by list mechanism. The descriptio is
 *    included inside entry
 */
typedef struct {
    ix_uint16    next;           /* Handle to the next entry                 */
    ix_uint16    prev;           /* Handle to the previous entry             */
} ix_cc_atm_list_entry_t;


/**
 * TYPENAME: ix_cc_atm_list_compare_t
 * 
 * DESCRIPTION:
 *    It is callback function that is called if TBD
 */
typedef ix_uint8                 /* if one>two than 1, if two>one than 2     */
(*ix_cc_atm_list_compare_t)(     /* if equal  return 0                       */
void *pFirst,                    /* first entry to compare                   */
void *pSecond);                  /* second entry to compare                  */


/**
 * TYPENAME: ix_cc_atm_list_insert_t
 * 
 * DESCRIPTION:
 *    TBD
 */
typedef ix_uint8                 /* if true insert entry before pNextEntry   */
(*ix_cc_atm_list_insert_t)(      /* otherwise false                          */
void *pMyEntry,                  /* my entries to compare                    */
void *pNextEntry);               /* entries send by list mechanism           */


/**
 * TYPENAME: ix_cc_atm_list_insert_t
 * 
 * DESCRIPTION:
 *    The callback function returns pointer to stucture.
 *
 * @Param:  - IN pUserData - Pointer to storage of entries
 * @Param:  - IN arg_handler - ID of entry
 *
 * @Return:
 *     Pointer if success otherwise NULL.
 */
typedef void*              
(*ix_cc_atm_list_get_entry_t)(
void *pUserData,
ix_uint16 arg_handler);


#define IX_CC_ATM_LIST_CMP_EQUAL         0
#define IX_CC_ATM_LIST_CMP_SECOND_LOWER  1
#define IX_CC_ATM_LIST_CMP_FIRST_LOWER   2

#define IX_CC_ATM_LIST_INVALID_HANDLE    0x0FFFF

#define IX_CC_ATM_LIST_OK     0
#define IX_CC_ATM_LIST_ERROR  1


/**
 * TYPENAME: ix_cc_atm_list_key_descr_t
 * 
 * DESCRIPTION:
 *    TBD
 */
typedef struct {
    ix_uint16    pos;            /* position in struct of ix_cc_atm_list_entry_t */
    ix_cc_atm_list_compare_t cmp;    /* function to compare two entries          */
    ix_cc_atm_list_insert_t insert;  /* function to compare during entry insert  */
} ix_cc_atm_list_key_descr_t;


/**
 * TYPENAME: ix_cc_atm_list_descr_t
 * 
 * DESCRIPTION:
 *    The structure describes a list. It contains all data required for servicing
 *    'pUserData 
 */
typedef struct {
    ix_uint32    count;             /* count of entry                        */
    ix_cc_atm_list_get_entry_t getEntry;
    ix_cc_atm_list_key_descr_t* pKey;/* pointer to key description           */
    ix_uint8     keyCount;           /* count of key accessible for the list  */
    ix_uint16    freeCount;
    ix_uint16    freeHead;
    void         *pUserData; /**< pointer to user data.  */
} ix_cc_atm_list_descr_t;


#define IX_CC_ATM_LIST_INIT_LINK     0
#define IX_CC_ATM_LIST_INIT_NO_LINK  1

/**
 * NAME: ix_cc_atm_list_init
 *
 * DESCRIPTION:
 *    The function initialize list
 * 
 * @Param:  - IN arg_pDescr - pointer to list description
 * @Param:  - IN arg_initFlag - type of list initialisation
 *
 * @Return: IX_CC_ATM_LIST_OK if successful or a IX_CC_ATM_LIST_ERROR
 *
 * Error Codes:
 *    IX_CC_ATM_LIST_ERROR
 */
ix_uint8
ix_cc_atm_list_init(
ix_cc_atm_list_descr_t *arg_pDescr,
ix_uint8 arg_initFlag);


/**
 * NAME: ix_cc_atm_list_alloc
 *
 * DESCRIPTION:
 *    The function gets entry from free list.
 * 
 * @Param:  - IN arg_pDescr - pointer to list description
 *
 * @Return:
 *     Handler of entry to use. If hander equals IX_CC_ATM_LIST_INVALID_HANDLER,
 *     it returns no entry,
 */
ix_uint16
ix_cc_atm_list_alloc(
ix_cc_atm_list_descr_t *arg_pDescr);


/**
 * NAME: ix_cc_atm_list_free
 *
 * DESCRIPTION:
 *    Puts an entry into the freelist
 * 
 * @Param:  - IN arg_handle - Handle of entry to put
 * @Param:  - IN arg_pDescr - pointer to list description
 *
 * @Return:
 *     None
 */
void
ix_cc_atm_list_free(
ix_uint16              arg_handle,
ix_cc_atm_list_descr_t *arg_pDescr);


/**
 * NAME: ix_cc_atm_list_free
 *
 * DESCRIPTION:
 *    This function inserts a entry to list where pHandler is head of list,
 *    key said which of next/prev handler should be serviced
 * 
 * @Param:  - IN arg_handle - Handle of entry to put
 * @Param:  - IN arg_key    - key of next/prev handler
 * @Param:  - IN arg_pHandler - handler to head of list
 * @Param:  - IN arg_pDescr - pointer to list description
 *
 * @Return:
 *     IX_CC_ATM_LIST_OK if inserted, otherwise IX_CC_ATM_LIST_ERROR.
 */
ix_uint8
ix_cc_atm_list_insert(
ix_uint16              arg_handle,
ix_uint16              arg_key,
ix_uint16              *arg_pHandler,
ix_cc_atm_list_descr_t *arg_pDescr);


/**
 * NAME: ix_cc_atm_list_free
 *
 * DESCRIPTION:
 *    This function remove a entry from the list.
 * 
 * @Param:  - IN arg_handle - Handle of entry to put
 * @Param:  - IN arg_key    - key of next/prev handler
 * @Param:  - IN arg_pHandler - handler to head of list
 * @Param:  - IN arg_pDescr - pointer to list description
 *
 * @Return:
 *     none
 */

void
ix_cc_atm_list_remove(
ix_uint16              arg_handle,
ix_uint16              arg_key,
ix_uint16              *arg_pHandler,
ix_cc_atm_list_descr_t *arg_pDescr);


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_ATMSAR_LIST_H__) */

