/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor, Release 5
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      ATM SAR Core Component
 *
 * = FILENAME
 *      ix_cc_atmsar_plugin.h
 *
 * = DESCRIPTION
 *      This file defines all exported data types and function prototypes
 *
 * = AUTHOR
 *       Jaroslaw Kogut
 *       jaroslaw.kogut@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *     
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * 
 */

#if !defined(__IX_CC_ATMSAR_PLUGIN_H__)
#define __IX_CC_ATMSAR_PLUGIN_H__

#include "ix_rm.h"
#include "ix_ossl.h"
#include "ix_cc.h"
#include "ix_cc_properties.h"
#include "cc/ix_cc_atmsar.h"

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

/******************************************************************************
 *
 *****************************************************************************/

/**
 * TYPENAME: ix_cc_atmsar_plugin_vc_create_t
 *
 * DESCRIPTION:
 *    This is aformat for function in plug-in  core compoment. The ATM SAR
 *    calls the function if it services "VC create" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_hVC :  handle to created VC.
 * @Param:  IN  arg_pVcDescr : parameters describing requested VC.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_NO_RESOURCES - no resources to add the entry 
 */
typedef ix_error (*ix_cc_atmsar_plugin_vc_create_t)(
void* arg_pInstance,
const ix_cc_atmsar_vc_handle_t arg_phVC,
const ix_cc_atmsar_vc_descr_t* arg_pVcDescr);

/**
 * TYPENAME: ix_cc_atmsar_plugin_vc_update_t
 *
 * DESCRIPTION:
 *    This is aformat for function in plug-in  core compoment. The ATM SAR
 *    calls the function if it services "VC update" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_hVC :  handle to created VC.
 * @Param:  IN  arg_pVcDescr : parameters describing requested VC.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_NO_RESOURCES - no resources to add the entry 
 */
typedef ix_error (*ix_cc_atmsar_plugin_vc_update_t)(
void* arg_pInstance,
const ix_cc_atmsar_vc_handle_t arg_phVC,
const ix_cc_atmsar_update_data_t* arg_pUpdate);

/**
 * TYPENAME: ix_cc_atmsar_plugin_vc_remove_t
 *
 * DESCRIPTION:
 *    This is aformat for function in plug-in  core compoment. The ATM SAR
 *    calls the function if it services "VC remove" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_hVc : handle to VC to be removed
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - vc successfully removed
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 */
typedef ix_error (*ix_cc_atmsar_plugin_vc_remove_t)(
void* arg_pInstance,
ix_cc_atmsar_vc_handle_t arg_hVc);


/**
 * TYPENAME: ix_cc_atmsar_plugin_get_vc_stats_t
 *
 * DESCRIPTION:
 *    This is aformat for function in plug-in  core compoment. The ATM SAR
 *    calls the function if it services "GET VC STATS" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_hVc : handle to VC to be removed
 * @Param:  OUT arg_pLookupResult : pointer to statistic structure
 *                  where the requested data will be stored
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - statistics successfully retrieved
 *    IX_CC_INVALID_PORT - specified ATM port does not exist 
 */
typedef ix_error (*ix_cc_atmsar_plugin_get_vc_stats_t)(
void* arg_pInstance,
ix_cc_atmsar_vc_handle_t arg_hVc,
ix_cc_atmsar_vc_stats_t *arg_pLookupResult);


/**
 * TYPENAME: ix_cc_atmsar_plugin_port_create_t
 *
 * DESCRIPTION:
 *    This is aformat for function in plug-in  core compoment. The ATM SAR
 *    calls the function if it services "PORT CREATE" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_pPortDescr : pointer to data describing parameters
 *                  for requested port
 * @Param:  OUT arg_hPort : ATM port handle
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_NO_RESOURCES - no resources to add the entry 
 */
typedef ix_error (*ix_cc_atmsar_plugin_port_create_t)(
void* arg_pInstance,
ix_cc_atmsar_port_handle_t arg_hPort,
const ix_cc_atmsar_port_descr_t* arg_pPortDescr);


/**
 * TYPENAME: ix_cc_atmsar_plugin_port_remove_t
 *
 * DESCRIPTION:
 *    This is aformat for function in plug-in  core compoment. The ATM SAR
 *    calls the function if it services "PORT REMOVE" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_port : ATM port to be removed
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - port successfully removed
 *    IX_CC_INVALID_PORT - specified port does not exist
 */
typedef ix_error (*ix_cc_atmsar_plugin_port_remove_t)(
void* arg_pInstance,
ix_cc_atmsar_port_handle_t arg_port);


/**
 * TYPENAME: ix_cc_atmsar_plugin_get_cb_port_stats_t
 *
 * DESCRIPTION:
 *    This is aformat for function in plug-in  core compoment. The ATM SAR
 *    calls the function if it services "GET PORT STATS" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_port : ATM port ID
 * @Param:  OUT arg_pLookupResult : pointer to ix_cc_sar_port_stats structure
 *                  where the requested data will be stored
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - statistics successfully retrieved
 *    IX_CC_INVALID_PORT - specified ATM port does not exist 
 */
typedef ix_error (*ix_cc_atmsar_plugin_get_port_stats_t)(
void* arg_pInstance,
ix_cc_atmsar_port_handle_t arg_port,
ix_cc_atmsar_port_stats_t *arg_pLookupResult);


/**
 * TYPENAME: ix_cc_atmsar_plugin_cid_create_t
 *
 * DESCRIPTION:
 *    This is aformat for function in plug-in  core compoment. The ATM SAR
 *    calls the function if it services "CID CREATE" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_pCidDescr : pointer to data describing parameters
 *                  for requested CID
 * @Param:  OUT arg_hPort : CID handle
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - CID successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_NO_RESOURCES - no resources to add the entry 
 */
typedef ix_error (*ix_cc_atmsar_plugin_cid_create_t)(
void* arg_pInstance,
ix_cc_atmsar_cid_handle_t arg_hCid,
const ix_cc_atmsar_cid_descr_t* arg_pCidDescr);

/**
 * TYPENAME: ix_cc_atmsar_plugin_cid_update_t
 *
 * DESCRIPTION:
 *    This is aformat for function in plug-in  core compoment. The ATM SAR
 *    calls the function if it services "CID UPDATE" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_pCidDescr : pointer to data describing parameters
 *                  for requested CID
 * @Param:  OUT arg_hPort : CID handle
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - CID successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_NO_RESOURCES - no resources to add the entry 
 */
typedef ix_error (*ix_cc_atmsar_plugin_cid_update_t)(
void* arg_pInstance,
ix_cc_atmsar_cid_handle_t arg_hCid,
const ix_cc_atmsar_update_data_t* arg_pUpdate);

/**
 * TYPENAME: ix_cc_atmsar_plugin_cid_remove_t
 *
 * DESCRIPTION:
 *    This is aformat for function in plug-in  core compoment. The ATM SAR
 *    calls the function if it services "CID REMOVE" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_cid : CID handle of CID to be removed
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - port successfully removed
 *    IX_CC_INVALID_PORT - specified port does not exist
 */
typedef ix_error (*ix_cc_atmsar_plugin_cid_remove_t)(
void* arg_pInstance,
ix_cc_atmsar_cid_handle_t arg_cid);


/**
 * TYPENAME: ix_cc_atmsar_plugin_get_cb_cid_stats_t
 *
 * DESCRIPTION:
 *    This is aformat for function in plug-in  core compoment. The ATM SAR
 *    calls the function if it services "GET CID STATS" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_cid : CID handle
 * @Param:  OUT arg_pLookupResult : pointer to ix_cc_sar_cid_stats_t structure
 *                  where the requested data will be stored
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - statistics successfully retrieved
 */
typedef ix_error (*ix_cc_atmsar_plugin_get_cid_stats_t)(
void* arg_pInstance,
ix_cc_atmsar_cid_handle_t arg_cid,
ix_cc_atmsar_cid_stats_t *arg_pLookupResult);


/**
 * TYPENAME: ix_cc_atmsar_plugin_vc_alloc_t
 *
 * DESCRIPTION:
 *    This is a format for function in plug-in core compoment. The ATM SAR
 *    calls the function if it wants to allocate VC for VC creation.
 *    The called function should allocate and return new VC handle.
 *    The 'arg_pVcDescr' agrument is passed to the function because
 *    parameters of VC could influent on assigned handle
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_pVcDescr : pointer to data describing parameters
 *                  for requested VC
 * @Param:  OUT arg_pHVc : poiter to data where assigned handle is stored
 *                  VC handle
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_NO_RESOURCES - no resources to add the entry 
 */
typedef ix_error (*ix_cc_atmsar_plugin_vc_alloc_t)(
void* arg_pInstance,
ix_cc_atmsar_vc_handle_t *arg_pHVc,
const ix_cc_atmsar_vc_descr_t *arg_pVcDescr);

/**
 * TYPENAME: ix_cc_atmsar_plugin_vc_free_t
 *
 * DESCRIPTION:
 *    This is a format for function in plug-in core compoment. The ATM SAR
 *    calls the function if it wants to free VC for VC destroying.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN arg_hVc : handle of freeing VC.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 */
typedef ix_error (*ix_cc_atmsar_plugin_vc_free_t)(
void* arg_pInstance,
ix_cc_atmsar_vc_handle_t arg_hVc);


/**
 * TYPENAME: ix_cc_atmsar_plugin_port_alloc_t
 *
 * DESCRIPTION:
 *    This is a format for function in plug-in core compoment. The ATM SAR
 *    calls the function if it wants to allocate port for port creation.
 *    The called function should allocate and return new port handle.
 *    The 'arg_pPortDescr' agrument is passed to the function because
 *    parameters of port could influent on assigned handle
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_pPortDescr : pointer to data describing parameters
 *                  for requested port
 * @Param:  OUT arg_pHPort : poiter to data where assigned handle is stored
 *                  ATM port handle
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_NO_RESOURCES - no resources to add the entry 
 */
typedef ix_error (*ix_cc_atmsar_plugin_port_alloc_t)(
void* arg_pInstance,
ix_cc_atmsar_port_handle_t *arg_pHPort,
const ix_cc_atmsar_port_descr_t *arg_pPortDescr);

/**
 * TYPENAME: ix_cc_atmsar_plugin_port_free_t
 *
 * DESCRIPTION:
 *    This is a format for function in plug-in core compoment. The ATM SAR
 *    calls the function if it wants to free port for port destroying.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN arg_hPort : handle of freeing ATM port.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 */
typedef ix_error (*ix_cc_atmsar_plugin_port_free_t)(
void* arg_pInstance,
ix_cc_atmsar_port_handle_t arg_hPort);


/**
 * TYPENAME: ix_cc_atmsar_plugin_cid_alloc_t
 *
 * DESCRIPTION:
 *    This is a format for function in plug-in core compoment. The ATM SAR
 *    calls the function if it wants to allocate CID for CID creation.
 *    The called function should allocate and return new CID handle.
 *    The 'arg_pCidDescr' agrument is passed to the function because
 *    parameters of CID could influent on assigned handle
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_pCidDescr : pointer to data describing parameters
 *                  for requested CID
 * @Param:  OUT arg_pHCid : poiter to data where assigned handle is stored
 *                  CID handle
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_NO_RESOURCES - no resources to add the entry 
 */
typedef ix_error (*ix_cc_atmsar_plugin_cid_alloc_t)(
void* arg_pInstance,
ix_cc_atmsar_cid_handle_t *arg_pHCid,
const ix_cc_atmsar_cid_descr_t *arg_pCidDescr);

/**
 * TYPENAME: ix_cc_atmsar_plugin_cid_free_t
 *
 * DESCRIPTION:
 *    This is a format for function in plug-in core compoment. The ATM SAR
 *    calls the function if it wants to free cid for cid destroying.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_hCid : handle of freeing CID.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 */
typedef ix_error (*ix_cc_atmsar_plugin_cid_free_t)(
void* arg_pInstance,
ix_cc_atmsar_cid_handle_t arg_hCid);


/**
 * TYPENAME: ix_cc_atmsar_plugin_cfg_port_descr_t
 * 
 * DESCRIPTION:
 *    It contains description of port read from static configuration
 */
typedef struct ix_s_cc_atmsar_plugin_cfg_port_descr_t
{
#define IX_CC_ATMSAR_PLUGIN_LINK_NAME_MAX 20
    char linkName[IX_CC_ATMSAR_PLUGIN_LINK_NAME_MAX]; /**< textual name of link.
                                                         It is read from static
                                                         configuraton */
    ix_uint32 linkRate;            /**< Line Rate of port. It is counted in
                                      cells per second. If lineRate is equal 0,
                                      the port is not present.  */
}ix_cc_atmsar_plugin_cfg_port_descr_t;


/**
 * TYPENAME: ix_cc_atmsar_plugin_cfg_params_t
 * 
 * DESCRIPTION:
 *    TBD
 */
typedef struct ix_s_cc_atmsar_plugin_cfg_params
{
    ix_uint16 numberOfVcs;         /**< value 0 means 65536 VCs   */
    ix_uint16 numberOfPorts;       /**< value 0 means 65536 ports */
    ix_uint8  aal2Included;
#define IX_CC_ATMSAR_PLUGIN_AAL2_INLUDED_NO   0
#define IX_CC_ATMSAR_PLUGIN_AAL2_INLUDED_YES  1
    ix_uint16 numberOfCids;        /**< value 0 means 65536 CIDs - the parameter is optional if AAL2 is included */
}ix_cc_atmsar_plugin_cfg_params_t;


/**
 * NAME: ix_cc_atmsar_plugin_reg_vc_service
 *
 * DESCRIPTION:
 *    The function registers ATM SAR plug-in into ATM SAR core
 *    component. The input parameters - callback routines may be NULL,
 *    indicating that the plug-in is not interested in that operation servicing.
 *
 * @Param:  IN  arg_coreComponentId : Id of core component that register
 *                  service.
 * @Param:  IN  arg_sarInstance : Id of SAR Instance for that the action is
 *                  performed.
 * @Param:  IN  arg_pCbVcCreate : pointer to callback routine servicing
 *                  "VC CREATE" operation internal in plug-in.
 * @Param:  IN  arg_pCbVcCreate : pointer to callback routine servicing
 *                  "VC UPDATE" operation internal in plug-in.
 * @Param:  IN  arg_pCbVcRemove : pointer to callback routine servicing
 *                  "VC REMOVE" operation internal in plug-in.
 * @Param:  IN  arg_pCbVcStats : pointer to callback routine servicing
 *                  "GET VC STATS" operation internal in plug-in.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - registering successful
 *    IX_CC_ERROR_ENTRY_NOT_FOUND - The ATM SAR is not ready for
 *         registering the core component for the specific ID.
 *    IX_CC_ERROR_DUPLICATE_ENTRY -- The ATM SAR already registered core
 *         component for the specific ID.
 *    IX_CC_ERROR_INTERNAL - The plug-in that trries to register works in the
 *          wrong thread. Check description of ATM SAR configuration tables
 *          and XML config description.
 */
ix_error ix_cc_atmsar_plugin_reg_vc_service(
ix_uint16 arg_coreComponentId,
ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmsar_plugin_vc_create_t arg_pCbVcCreate,
const ix_cc_atmsar_plugin_vc_update_t arg_pCbVcUpdate,
const ix_cc_atmsar_plugin_vc_remove_t arg_pCbVcRemove,
const ix_cc_atmsar_plugin_get_vc_stats_t arg_pCbVcStats);


/**
 * NAME: ix_cc_atmsar_plugin_reg_port_service
 *
 * DESCRIPTION:
 *    The function registers ATM SAR plug-in into ATM SAR core
 *    component. The input parameters - callback routines may be NULL,
 *    indicating that the plug-in is not interested in that operation servicing.
 *
 * @Param:  IN  arg_coreComponentId : Id of core component that register
 *                  service.
 * @Param:  IN  arg_sarInstance : Id of SAR Instance for that the action is
 *                  performed.
 * @Param:  IN  arg_pCbPortCreate : pointer to callback routine servicing
 *                  "PORT CREATE" operation internal in plug-in.
 * @Param:  IN  arg_pCbPortRemove : pointer to callback routine servicing
 *                  "PORT REMOVE" operation internal in plug-in.
 * @Param:  IN  arg_pCbPortStats : pointer to callback routine servicing
 *                  "GET PORT STATS" operation internal in plug-in.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - registering successful
 *    IX_CC_ERROR_ENTRY_NOT_FOUND - The ATM SAR is not ready for
 *         registering the core component for the specific ID.
 *    IX_CC_ERROR_DUPLICATE_ENTRY -- The ATM SAR already registered core
 *         component for the specific ID.
 *    IX_CC_ERROR_INTERNAL - The plug-in that trries to register works in the
 *          wrong thread. Check description of ATM SAR configuration tables
 *          and XML config description.
 */
ix_error ix_cc_atmsar_plugin_reg_port_service(
ix_uint16 arg_coreComponentId,
ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmsar_plugin_port_create_t arg_pCbPortCreate,
const ix_cc_atmsar_plugin_port_remove_t arg_pCbPortRemove,
const ix_cc_atmsar_plugin_get_port_stats_t arg_pCbPortStats);


/**
 * NAME: ix_cc_atmsar_plugin_reg_cid_service
 *
 * DESCRIPTION:
 *    The function registers ATM SAR plug-in into ATM SAR core
 *    component. The input parameters - callback routines may be NULL,
 *    indicating that the plug-in is not interested in that operation servicing.
 *
 * @Param:  IN  arg_coreComponentId : Id of core component that register
 *                  service.
 * @Param:  IN  arg_sarInstance : Id of SAR Instance for that the action is
 *                  performed.
 * @Param:  IN  arg_pCbCidCreate : pointer to callback routine servicing
 *                  "CID CREATE" operation internal in plug-in.
 * @Param:  IN  arg_pCbCidRemove : pointer to callback routine servicing
 *                  "CID REMOVE" operation internal in plug-in.
 * @Param:  IN  arg_pCbPortStats : pointer to callback routine servicing
 *                  "GET CID STATS" operation internal in plug-in.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - registering successful
 *    IX_CC_ERROR_ENTRY_NOT_FOUND - The ATM SAR is not ready for
 *         registering the core component for the specific ID.
 *    IX_CC_ERROR_DUPLICATE_ENTRY -- The ATM SAR already registered core
 *         component for the specific ID.
 *    IX_CC_ERROR_INTERNAL - The plug-in that trries to register works in the
 *          wrong thread. Check description of ATM SAR configuration tables
 *          and XML config description.
 */
ix_error ix_cc_atmsar_plugin_reg_cid_service(
ix_uint16 arg_coreComponentId,
ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmsar_plugin_cid_create_t arg_pCbCidCreate,
const ix_cc_atmsar_plugin_cid_update_t arg_pCbCidUpdate,
const ix_cc_atmsar_plugin_cid_remove_t arg_pCbCidRemove,
const ix_cc_atmsar_plugin_get_cid_stats_t arg_pCbCidStats);


/**
 * NAME: ix_cc_atmsar_plugin_reg_done
 *
 * DESCRIPTION:
 *    The function notifies ATM SAR that the plug-in already register all
 *    services.
 *
 * @Param:  IN  arg_coreComponentId : Id of core component that register
 *                  service.
 * @Param:  IN  arg_sarInstance : Id of SAR Instance for that the action is
 *                  performed.
 * @Param:  IN  arg_pInstance :  pointer to structure describing local data
 *                  instance for the 'arg_sarInstance'.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - registering successful
 *    IX_CC_ERROR_ENTRY_NOT_FOUND - The ATM SAR is not ready for
 *         registering the core component for the specific ID.
 *    IX_CC_ERROR_DUPLICATE_ENTRY -- The ATM SAR already registered core
 *         component for the specific ID.
 *    IX_CC_ERROR_INTERNAL - The plug-in that trries to register works in the
 *          wrong thread. Check description of ATM SAR configuration tables
 *          and XML config description.
 */
ix_error ix_cc_atmsar_plugin_reg_done(
ix_uint16 arg_coreComponentId,
ix_cc_atmsar_instance_t arg_sarInstance,
void* arg_pInstance);


/**
 * NAME: ix_cc_atmsar_plugin_reg_port_handle_service
 *
 * DESCRIPTION:
 *    The function registers ATM SAR plug-in into ATM SAR core
 *    component. The registered functions are used for allocating
 *    and freeing of port handle.
 *
 * @Param:  IN  arg_sarInstance : Id of SAR Instance for that the action is
 *                  performed.
 * @Param:  IN  arg_pInstance :  pointer to structure describing local data
 *                  instance for the 'arg_sarInstance'.
 * @Param:  IN  arg_pCbPortAlloc : pointer to callback routine allocating port
 *                  handle.
 * @Param:  IN  arg_pCbPortFree : pointer to callback routine freeing port
 *                  handle.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - registering successful
 *    IX_CC_ERROR_INTERNAL - The plug-in that trries to register works in the
 *          wrong thread. Check description of ATM SAR configuration tables
 *          and XML config description.
 */
ix_error ix_cc_atmsar_plugin_reg_port_handle_service(
ix_cc_atmsar_instance_t arg_sarInstance,
void* arg_pInstance,
const ix_cc_atmsar_plugin_port_alloc_t arg_pCbPortAlloc,
const ix_cc_atmsar_plugin_port_free_t arg_pCbPortFree);


/**
 * NAME: ix_cc_atmsar_plugin_reg_vc_handle_service
 *
 * DESCRIPTION:
 *    The function registers ATM SAR plug-in into ATM SAR core
 *    component. The registered functions are used for allocating
 *    and freeing of vc handle.
 *
 * @Param:  IN  arg_sarInstance : Id of SAR Instance for that the action is
 *                  performed.
 * @Param:  IN  arg_pInstance :  pointer to structure describing local data
 *                  instance for the 'arg_sarInstance'.
 * @Param:  IN  arg_pCbVcAlloc : pointer to callback routine allocating VC
 *                  handle.
 * @Param:  IN  arg_pCbVcFree : pointer to callback routine freeing VC
 *                  handle.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - registering successful
 *    IX_CC_ERROR_INTERNAL - The plug-in that trries to register works in the
 *          wrong thread. Check description of ATM SAR configuration tables
 *          and XML config description.
 */
ix_error ix_cc_atmsar_plugin_reg_vc_handle_service(
ix_cc_atmsar_instance_t arg_sarInstance,
void* arg_pInstance,
const ix_cc_atmsar_plugin_vc_alloc_t arg_pCbVcAlloc,
const ix_cc_atmsar_plugin_vc_free_t arg_pCbVcFree);


/**
 * NAME: ix_cc_atmsar_plugin_reg_cid_handle_service
 *
 * DESCRIPTION:
 *    The function registers ATM SAR plug-in into ATM SAR core
 *    component. The registered functions are used for allocating
 *    and freeing of CID handle.
 *
 * @Param:  IN  arg_sarInstance : Id of SAR Instance for that the action is
 *                  performed.
 * @Param:  IN  arg_pInstance :  pointer to structure describing local data
 *                  instance for the 'arg_sarInstance'.
 * @Param:  IN  arg_pCbCidAlloc : pointer to callback routine allocating CID
 *                  handle.
 * @Param:  IN  arg_pCbCidFree : pointer to callback routine freeing CID
 *                  handle.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - registering successful
 *    IX_CC_ERROR_INTERNAL - The plug-in that trries to register works in the
 *          wrong thread. Check description of ATM SAR configuration tables
 *          and XML config description.
 */
ix_error ix_cc_atmsar_plugin_reg_cid_handle_service(
ix_cc_atmsar_instance_t arg_sarInstance,
void* arg_pInstance,
const ix_cc_atmsar_plugin_cid_alloc_t arg_pCbCidAlloc,
const ix_cc_atmsar_plugin_cid_free_t arg_pCbCidFree);


/**
 * NAME: ix_cc_atmsar_plugin_get_cfg_params
 *
 * DESCRIPTION:
 *    The function gets configuration parameters for SAR instance. The values
 *    are read from registry or from configuration header. The configuration
 *    parameters describe general behaviors and functionality of SAR.
 *
 * @Param:  IN  arg_sarInstance : Id of SAR Instance for that the action is
 *                  performed.
 * @Param:  OUT arg_pCfgParams : pointer to structure where configuration
 *                  parameters are stored.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - registering successful
 *    IX_CC_ERROR_INTERNAL - The plug-in that trries to register works in the
 *          wrong thread. Check description of ATM SAR configuration tables
 *          and XML config description.
 */
ix_error ix_cc_atmsar_plugin_get_cfg_params(
ix_cc_atmsar_instance_t arg_sarInstance,
ix_cc_atmsar_plugin_cfg_params_t *arg_pCfgParams);


/**
 * NAME: ix_cc_atmsar_plugin_get_cfg_ports_params
 *
 * DESCRIPTION:
 *    The function gets port configuration parameters for SAR instance. The
 *    configuration is read from registry or from configuration header.
 *
 * @Param:  IN  arg_sarInstance : Id of SAR Instance for that the action is
 *                  performed.
 * @Param:  IN  arg_portNo : number of ports of port for that the action is performed.
 * @Param:  OUT arg_pCfgParams : pointer to table with structures where configuration
 *                  parameters for all ports are stored.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - registering successful
 *    IX_CC_ERROR_INTERNAL - The plug-in that trries to register works in the
 *          wrong thread. Check description of ATM SAR configuration tables
 *          and XML config description.
 */
ix_error ix_cc_atmsar_plugin_get_cfg_ports_params(
ix_cc_atmsar_instance_t arg_sarInstance,
ix_uint16 arg_portNo,
ix_cc_atmsar_plugin_cfg_port_descr_t *arg_pCfgParams);


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_ATMSAR_PLUGIN_H__) */

