/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      ATM SAR Agent Core Component - CCI - init function
 *
 * = FILENAME
 *      ix_cc_atmsar_agent_api.c
 *
 * = DESCRIPTION
 *
 * = AUTHOR
 *       Jaroslaw Kogut
 *       jaroslaw.kogut@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */
#define IX_ERROR_FILE_IDENT "$Id: ix_cc_atmsar_agent_api.c,v 1.2 2003/09/10 07:37:10 rwielick Exp $";

/**
 * System defined include files
 */
#include "ix_cc_error.h"
#include "ix_rm.h"
#include "ix_ossl.h"
#include "ix_cc.h"
#include "ix_cc_macros.h"
#include "ix_cc_properties.h"
#include "ix_config.h"
#include "bindings.h"
#include "cc/ix_cc_msup.h"

/**
 * User defined include files
 */
#include "cc/ix_cc_atmsar_agent_cci.h"
#include "cc/internal/ix_cc_atmsar_agent.h"
#include "cc/internal/ix_cc_atmsar_common.h"


/**
 * Preprocessor symbols and Macros used in this file
 */

/**
 * Type definitions whose scope is limited to this file
 */


/** 
 * Extern function prototypes.
 */


/** 
 * Static function prototypes.
 */


/** 
 * Function definitions.
 */


/******************************************************************************
 *          INTERNAL FUNCTIONS - FUNCTION FOR ASYNC MSG PROCESSING
 *****************************************************************************/

/**
 * NAME: ix_cc_atmsar_agent_vc_create
 *
 * DESCRIPTION:
 *    The function adds VC into ATM SAR Agent plugins.
 * 
 * @Param:  IN  arg_pContext : pointer to ATM SAR Agent CC
 * @Param:  IN  arg_hVc : handle to VC to be created
 * @Param:  IN  arg_pVcDescr : parameters describing requested VC.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 */
ix_error ix_cc_atmsar_agent_vc_create(
void *arg_pContext,
const ix_cc_atmsar_vc_handle_t arg_hVc,
const ix_cc_atmsar_vc_descr_t* arg_pVcDescr)
{
    _ix_cc_atmsar_agent_context_t *pCtx=arg_pContext;
    _IX_CC_ATMSAR_DBG_AGENT(("AGENT: VC CREATE ...\n"));
    return _ix_cc_atmsar_engine_vc_create(&(pCtx->engine), arg_hVc, arg_pVcDescr);
} /* end ix_cc_atmsar_agent_vc_create */

/**
 * NAME: ix_cc_atmsar_agent_vc_update
 *
 * DESCRIPTION:
 *    The function updates VC in ATM SAR Agent plugins.
 * 
 * @Param:  IN  arg_pContext : pointer to ATM SAR Agent CC
 * @Param:  IN  arg_hVc : handle to VC to be created
 * @Param:  IN  arg_pVcDescr : parameters describing requested VC.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 */
ix_error ix_cc_atmsar_agent_vc_update(
void *arg_pContext,
const ix_cc_atmsar_vc_handle_t arg_hVc,
const ix_cc_atmsar_update_data_t* arg_pUpdate)
{
    _ix_cc_atmsar_agent_context_t *pCtx=arg_pContext;
    _IX_CC_ATMSAR_DBG_AGENT(("AGENT: VC UPDATE ...\n"));
    return _ix_cc_atmsar_engine_vc_update(&(pCtx->engine), arg_hVc, arg_pUpdate);
} /* end ix_cc_atmsar_agent_vc_update */

/**
 * NAME: ix_cc_atmsar_agent_vc_remove
 *
 * DESCRIPTION:
 *    The function removes VC from ATM SAR Agent plugins.
 * 
 * @Param:  IN  arg_pContext : pointer to ATM SAR Agent CC
 * @Param:  IN  arg_hVc : handle to VC to be removed
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ERROR_SEND_FAIL - failure on sending a message
 */
ix_error ix_cc_atmsar_agent_vc_remove(
void *arg_pContext,
const ix_cc_atmsar_vc_handle_t arg_hVc)
{
    _ix_cc_atmsar_agent_context_t *pCtx=arg_pContext;
    _IX_CC_ATMSAR_DBG_AGENT(("AGENT: VC REMOVE msg received...\n"));
    return _ix_cc_atmsar_engine_vc_remove(&(pCtx->engine), arg_hVc);
} /* end ix_cc_atmsar_agent_vc_remove */


/**
 * NAME: ix_cc_atmsar_agent_vc_stats
 *
 * DESCRIPTION:
 *    The function reads statistic for VC from ATM SAR Agent plugins.
 * 
 * @Param:  IN  arg_pContext : pointer to ATM SAR Agent CC
 * @Param:  IN  arg_hVc : handle to VC to be removed
 * @Param:  OUT arg_pLookupResult; pointer to buffer where VC statistic has to
 *                  be stored.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ERROR_SEND_FAIL - failure on sending a message
 */
ix_error ix_cc_atmsar_agent_vc_stats(
void *arg_pContext,
const ix_cc_atmsar_vc_handle_t arg_hVc,
ix_cc_atmsar_vc_stats_t* arg_pLookupResult)
{
    _ix_cc_atmsar_agent_context_t *pCtx=arg_pContext;
    _IX_CC_ATMSAR_DBG_AGENT(("AGENT: VC STATS ...\n"));
    ix_ossl_memset(arg_pLookupResult, 0, sizeof(*arg_pLookupResult));
    return _ix_cc_atmsar_engine_get_vc_stats(&(pCtx->engine), arg_hVc,
                                             arg_pLookupResult);
} /* end ix_cc_atmsar_agent_vc_stats */


/**
 * NAME: ix_cc_atmsar_agent_port_create
 *
 * DESCRIPTION:
 *    The function adds Port into ATM SAR Agent plugins.
 * 
 * @Param:  IN  arg_pContext : pointer to ATM SAR Agent CC
 * @Param:  IN  arg_hPort : handle to port to be created
 * @Param:  IN  arg_pPortDescr : parameters describing requested port.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ERROR_SEND_FAIL - failure on sending a message
 */
ix_error ix_cc_atmsar_agent_port_create(
void *arg_pContext,
const ix_cc_atmsar_port_handle_t arg_hPort,
const ix_cc_atmsar_port_descr_t* arg_pPortDescr)
{
    _ix_cc_atmsar_agent_context_t *pCtx=arg_pContext;
    _IX_CC_ATMSAR_DBG_AGENT(("AGENT: PORT CREATE...\n"));
    return _ix_cc_atmsar_engine_port_create(&(pCtx->engine), arg_hPort,
                                            arg_pPortDescr);
} /* end ix_cc_atmsar_agent_port_create */


/**
 * NAME: ix_cc_atmsar_agent_port_remove
 *
 * DESCRIPTION:
 *    The function removes port from ATM SAR Agent plugins.
 * 
 * @Param:  IN  arg_pContext : pointer to ATM SAR Agent CC
 * @Param:  IN  arg_hPort : handle to port to be removed
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ERROR_SEND_FAIL - failure on sending a message
 */
ix_error ix_cc_atmsar_agent_port_remove(
void *arg_pContext,
const ix_cc_atmsar_port_handle_t arg_hPort)
{
    _ix_cc_atmsar_agent_context_t *pCtx=arg_pContext;
    _IX_CC_ATMSAR_DBG_AGENT(("AGENT: PORT REMOVE...\n"));
    return _ix_cc_atmsar_engine_port_remove(&(pCtx->engine), arg_hPort);
} /* end ix_cc_atmsar_agent_port_remove */


/**
 * NAME: ix_cc_atmsar_agent_port_stats
 *
 * DESCRIPTION:
 *    The function reads statistic for port from ATM SAR Agent plugins.
 * 
 * @Param:  IN  arg_pContext : pointer to ATM SAR Agent CC
 * @Param:  IN  arg_hPort : handle to port for statistic reading
 * @Param:  OUT arg_pLookupResult; pointer to buffer where port statistic has to
 *                  be stored.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ERROR_SEND_FAIL - failure on sending a message
 */
ix_error ix_cc_atmsar_agent_port_stats(
void *arg_pContext,
const ix_cc_atmsar_port_handle_t arg_hPort,
ix_cc_atmsar_port_stats_t* arg_pLookupResult)
{
    _ix_cc_atmsar_agent_context_t *pCtx=arg_pContext;
    _IX_CC_ATMSAR_DBG_AGENT(("AGENT: PORT STATS...\n"));
    ix_ossl_memset(arg_pLookupResult, 0, sizeof(*arg_pLookupResult));
    return _ix_cc_atmsar_engine_get_port_stats(&(pCtx->engine), arg_hPort,
                                               arg_pLookupResult);
} /* end ix_cc_atmsar_agent_port_stats */


/**
 * NAME: ix_cc_atmsar_agent_cid_create
 *
 * DESCRIPTION:
 *    The function adds cid into ATM SAR Agent plugins.
 * 
 * @Param:  IN  arg_pContext : pointer to ATM SAR Agent CC
 * @Param:  IN  arg_hCid : handle to CID to be created
 * @Param:  IN  arg_pCidDescr : parameters describing requested CID.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ERROR_SEND_FAIL - failure on sending a message
 */
ix_error ix_cc_atmsar_agent_cid_create(
void *arg_pContext,
const ix_cc_atmsar_cid_handle_t arg_hCid,
const ix_cc_atmsar_cid_descr_t* arg_pCidDescr)
{
    _ix_cc_atmsar_agent_context_t *pCtx=arg_pContext;
    _IX_CC_ATMSAR_DBG_AGENT(("AGENT: CID CREATE...\n"));
    return _ix_cc_atmsar_engine_cid_create(&(pCtx->engine), arg_hCid,
                                           arg_pCidDescr);
} /* end ix_cc_atmsar_agent_cid_create */


/**
 * NAME: ix_cc_atmsar_agent_vc_update
 *
 * DESCRIPTION:
 *    The function updates CID in ATM SAR Agent plugins.
 * 
 * @Param:  IN  arg_pContext : pointer to ATM SAR Agent CC
 * @Param:  IN  arg_cid : cid to be updated
 * @Param:  IN  arg_pUpdate : parameters describing requested CID.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 */
ix_error ix_cc_atmsar_agent_cid_update(
void *arg_pContext,
const ix_cc_atmsar_cid_handle_t arg_cid,
const ix_cc_atmsar_update_data_t* arg_pUpdate)
{
    _ix_cc_atmsar_agent_context_t *pCtx=arg_pContext;
    _IX_CC_ATMSAR_DBG_AGENT(("AGENT: CID UPDATE ...\n"));
    return _ix_cc_atmsar_engine_cid_update(&(pCtx->engine), arg_cid, arg_pUpdate);
} /* end ix_cc_atmsar_agent_cid_update */

/**
 * NAME: ix_cc_atmsar_agent_cid_remove
 *
 * DESCRIPTION:
 *    The function removes CID from ATM SAR Agent plugins.
 * 
 * @Param:  IN  arg_pContext : pointer to ATM SAR Agent CC
 * @Param:  IN  arg_hCid : handle to CID to be removed
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ERROR_SEND_FAIL - failure on sending a message
 */
ix_error ix_cc_atmsar_agent_cid_remove(
void *arg_pContext,
const ix_cc_atmsar_cid_handle_t arg_hCid)
{
    _ix_cc_atmsar_agent_context_t *pCtx=arg_pContext;
    _IX_CC_ATMSAR_DBG_AGENT(("AGENT: CID REMOVE...\n"));
    return _ix_cc_atmsar_engine_cid_remove(&(pCtx->engine), arg_hCid);
} /* end ix_cc_atmsar_agent_rec_cid_remove */


/**
 * NAME: ix_cc_atmsar_agent_cid_stats
 *
 * DESCRIPTION:
 *    The function reads statistic for CID from ATM SAR Agent plugins.
 * 
 * @Param:  IN  arg_pContext : pointer to ATM SAR Agent CC
 * @Param:  IN  arg_hCid : handle to CID for that statistic is read
 * @Param:  OUT arg_pLookupResult; pointer to buffer where CID statistic has to
 *                  be stored.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ERROR_SEND_FAIL - failure on sending a message
 */
ix_error ix_cc_atmsar_agent_cid_stats(
void *arg_pContext,
const ix_cc_atmsar_cid_handle_t arg_hCid,
ix_cc_atmsar_cid_stats_t* arg_pLookupResult)
{
    _ix_cc_atmsar_agent_context_t *pCtx=arg_pContext;
    _IX_CC_ATMSAR_DBG_AGENT(("AGENT: CID STATS...\n"));
    ix_ossl_memset(arg_pLookupResult, 0, sizeof(*arg_pLookupResult));
    return _ix_cc_atmsar_engine_get_cid_stats(&(pCtx->engine), arg_hCid,
                                              arg_pLookupResult);
} /* end ix_cc_atmsar_agent_cid_stats */

