/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      ATM SAR Core Component
 *
 * = FILENAME
 *      ix_cc_atmsar_engine.c
 *
 * = DESCRIPTION
 *
 * = AUTHOR
 *       Jaroslaw Kogut
 *       jaroslaw.kogut@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */
#define IX_ERROR_FILE_IDENT "$Id: ix_cc_atmsar_engine.c,v 1.4 2003/12/12 18:24:15 adziecia Exp $";



/**
 * System defined include files
 */
#include "ix_cc_error.h"
#include "ix_rm.h"
#include "ix_ossl.h"
#include "ix_cc.h"
#include "ix_cc_macros.h"
#include "ix_cc_properties.h"
#include "ix_config.h"
#include "bindings.h"
#include "cc/ix_cc_msup.h"

/**
 * User defined include files
 */
#include "cc/internal/ix_cc_atmsar_common.h"
#include "cc/ix_cc_atmsar_plugin.h"

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
#include <linux/ctype.h>
#include "cc/ix_cc_reg_util.h"
/* TBD: compilation warnings removed (for Linux only) */
int atoi(const char *nptr);
#endif

/**
 * Preprocessor symbols and Macros used in this file
 */

#define _IX_CC_ATMSAR_ENGINE_SERVICE_PLUGIN_INSTANCE pTab[i].pCtx

#define _IX_CC_ATMSAR_ENGINE_SERVICE(__pEngine, __operDescr, __id, __pCb, __pCbArgs)           \
    int i;                                                                                     \
    ix_error err = IX_SUCCESS;                                                                 \
    _ix_cc_atmsar_plugin_entry_t *pTab=__pEngine->pluginTab;                                   \
    _IX_CC_ATMSAR_DBG_ENG(("%-5s: ENGINE:%s<%5d>\n", __pEngine->appName, __operDescr, __id));  \
    for(i=0; i < __pEngine->ccNumber; i++)                                                     \
    {                                                                                          \
        if(pTab[i].__pCb == NULL)                                                              \
        {                                                                                      \
            continue;                                                                          \
        }                                                                                      \
        if((err = pTab[i].__pCb __pCbArgs ) != IX_SUCCESS)                                     \
        {                                                                                      \
            return err;                                                                        \
        }                                                                                      \
    }                                                                                          \
    return IX_SUCCESS;


#define _IX_CC_ATMSAR_ENGINE_SERVICE_REVERSE(__pEngine, __operDescr, __id, __pCb, __pCbArgs)   \
    int i;                                                                                     \
    ix_error err = IX_SUCCESS;                                                                 \
    _ix_cc_atmsar_plugin_entry_t *pTab=__pEngine->pluginTab;                                   \
    _IX_CC_ATMSAR_DBG_ENG(("%-5s: ENGINE:%s<%5d>\n", __pEngine->appName, __operDescr, __id));  \
    for(i=__pEngine->ccNumber-1; i >= 0; i--)                                                  \
    {                                                                                          \
        if(pTab[i].__pCb == NULL)                                                              \
        {                                                                                      \
            continue;                                                                          \
        }                                                                                      \
        if((err = pTab[i].__pCb __pCbArgs ) != IX_SUCCESS)                                     \
        {                                                                                      \
            return err;                                                                        \
        }                                                                                      \
    }                                                                                          \
    return IX_SUCCESS;


/**
 * Type definitions whose scope is limited to this file
 */

/**
 * Variable declarations global to this file only. Externs are followed by 
 * static variables.
 */

/** 
 * Extern function prototypes.
 */


/** 
 * Static function prototypes.
 */


/** 
 * Function definitions.
 */


/**
 * NAME: _ix_cc_atmsar_engine_vc_create
 *
 * DESCRIPTION:
 *    This function creates an VC with specified value in 'arg_pVcDescr'.
 *    The function calls the core component managed by agent or main
 *    ATM SAR. The list of sub-rutines for calling is pessed in
 *    'arg_pEngine'.
 *
 * @Param:  IN  arg_pEngine : pointer to structure describing supported
 *                  core components.
 * @Param:  IN  arg_pVcDescr : parameters describing requested VC.
 * @Param:  OUT arg_hVC : handle to created VC.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_NO_RESOURCES - no resources to add the entry 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error
_ix_cc_atmsar_engine_vc_create(
const _ix_cc_atmsar_engine_t *arg_pEngine,
const ix_cc_atmsar_vc_handle_t arg_hVc,
const ix_cc_atmsar_vc_descr_t* arg_pVcDescr)
{
    _IX_CC_ATMSAR_ENGINE_SERVICE(arg_pEngine, "VC CREATE", arg_hVc,
                                 pCbVcCreate, (_IX_CC_ATMSAR_ENGINE_SERVICE_PLUGIN_INSTANCE,
                                               arg_hVc, arg_pVcDescr));
} /* end _ix_cc_atmsar_engine_vc_create */

/**
 * NAME: _ix_cc_atmsar_engine_vc_update
 *
 * DESCRIPTION:
 *    This function updates a VC with specified value in 'arg_pVcDescr'.
 *    The function calls the core component managed by agent or main
 *    ATM SAR. The list of sub-rutines for calling is pessed in
 *    'arg_pEngine'.
 *
 * @Param:  IN  arg_pEngine : pointer to structure describing supported
 *                  core components.
 * @Param:  IN  arg_pUpdate : parameters describing requested VC.
 * @Param:  OUT arg_hVC : handle to created VC.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_NO_RESOURCES - no resources to add the entry 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error
_ix_cc_atmsar_engine_vc_update(
const _ix_cc_atmsar_engine_t *arg_pEngine,
const ix_cc_atmsar_vc_handle_t arg_hVc,
const ix_cc_atmsar_update_data_t* arg_pUpdate)
{
    _IX_CC_ATMSAR_ENGINE_SERVICE(arg_pEngine, "VC UPDATE", arg_hVc,
                                 pCbVcUpdate, (_IX_CC_ATMSAR_ENGINE_SERVICE_PLUGIN_INSTANCE,
                                               arg_hVc, arg_pUpdate));
} /* end _ix_cc_atmsar_engine_vc_update */

/**
 * NAME: _ix_cc_atmsar_engine_vc_remove
 *
 * DESCRIPTION:
 *    This function removes a VC specified by 'arg_hVc'.
 *    The function calls the core component managed by agent or main
 *    ATM SAR. The list of sub-rutines for calling is pessed in
 *    'arg_pEngine'.
 * 
 * @Param:  IN  arg_pEngine : pointer to structure describing supported
 *                  core components.
 * @Param:  IN  arg_pEngine : pointer to structure describing supported
 *                  core components.
 * @Param:  IN  arg_hVc : handle to VC to be removed
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - vc successfully removed
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error
_ix_cc_atmsar_engine_vc_remove(
const _ix_cc_atmsar_engine_t *arg_pEngine,
ix_cc_atmsar_vc_handle_t arg_hVc)
{
    _IX_CC_ATMSAR_ENGINE_SERVICE_REVERSE(arg_pEngine, "VC REMOVE", arg_hVc,
                                         pCbVcRemove, (_IX_CC_ATMSAR_ENGINE_SERVICE_PLUGIN_INSTANCE, arg_hVc));
} /* end _ix_cc_atmsar_engine_vc_remove */


/**
 * NAME: _ix_cc_atmsar_engine_get_port_stats
 *
 * DESCRIPTION:
 *    This function retrieves an VC statistics. 
 * 
 * @Param:  IN  arg_pEngine : pointer to structure describing supported
 *                  core components.
 * @Param:  IN  arg_hVc : handle to VC to be removed
 * @Param:  OUT arg_pLookupResult : pointer to statistic structure
 *                  where the requested data will be stored
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - statistics successfully retrieved
 *    IX_CC_INVALID_PORT - specified ATM port does not exist 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error
_ix_cc_atmsar_engine_get_vc_stats(
const _ix_cc_atmsar_engine_t *arg_pEngine,
ix_cc_atmsar_vc_handle_t arg_hVc,
ix_cc_atmsar_vc_stats_t *arg_pLookupResult)
{
    _IX_CC_ATMSAR_ENGINE_SERVICE(arg_pEngine, "VC STATS", arg_hVc,
                                 pCbVcStats, (_IX_CC_ATMSAR_ENGINE_SERVICE_PLUGIN_INSTANCE,
                                              arg_hVc, arg_pLookupResult));
} /* end _ix_cc_atmsar_engine_get_vc_stats */


/**
 * NAME: _ix_cc_atmsar_engine_port_create
 *
 * DESCRIPTION:
 *    This function creates an ATM port with default values.
 * 
 * @Param:  IN  arg_pEngine : pointer to structure describing supported
 *                  core components.
 * @Param:  IN  arg_pPortDescr : pointer to data describing parameters
 *                  for requested port
 * @Param:  OUT arg_pPort : pointer to a variable storing the created
 *                  ATM port ID
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_NO_RESOURCES - no resources to add the entry 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error
_ix_cc_atmsar_engine_port_create(
const _ix_cc_atmsar_engine_t *arg_pEngine,
ix_cc_atmsar_port_handle_t arg_port,
const ix_cc_atmsar_port_descr_t* arg_pPortDescr)
{
    _IX_CC_ATMSAR_ENGINE_SERVICE(arg_pEngine, "PORT CREATE", arg_port,
                                 pCbPortCreate, (_IX_CC_ATMSAR_ENGINE_SERVICE_PLUGIN_INSTANCE,
                                                 arg_port, arg_pPortDescr));
    return IX_SUCCESS;
} /* end _ix_cc_atmsar_engine_port_create */


/**
 * NAME: _ix_cc_atmsar_engine_port_remove
 *
 * DESCRIPTION:
 *    This function removes an ATM port from all VLANs.
 * 
 * @Param:  IN  arg_pEngine : pointer to structure describing supported
 *                  core components.
 * @Param:  IN  arg_port : ATM port to be removed
 * @Param:  OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - port successfully removed
 *    IX_CC_INVALID_PORT - specified port does not exist
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error
_ix_cc_atmsar_engine_port_remove(
const _ix_cc_atmsar_engine_t *arg_pEngine,
ix_cc_atmsar_port_handle_t arg_port)
{
    _IX_CC_ATMSAR_ENGINE_SERVICE_REVERSE(arg_pEngine, "PORT REMOVE", arg_port,
                                         pCbPortRemove, (_IX_CC_ATMSAR_ENGINE_SERVICE_PLUGIN_INSTANCE,
                                                         arg_port));
} /* end _ix_cc_atmsar_engine_port_remove */


/**
 * NAME: _ix_cc_atmsar_engine_get_port_stats
 *
 * DESCRIPTION:
 *    This function retrieves an ATM port statistics. 
 * 
 * @Param:  IN  arg_pEngine : pointer to structure describing supported
 *                  core components.
 * @Param:  IN  arg_port : ATM port ID
 * @Param:  OUT arg_pLookupResult : pointer to ix_cc_sar_port_stats structure
 *                  where the requested data will be stored
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - statistics successfully retrieved
 *    IX_CC_INVALID_PORT - specified ATM port does not exist 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error _ix_cc_atmsar_engine_get_port_stats(
const _ix_cc_atmsar_engine_t *arg_pEngine,
ix_cc_atmsar_port_handle_t arg_port,
ix_cc_atmsar_port_stats_t *arg_pLookupResult)
{
    _IX_CC_ATMSAR_ENGINE_SERVICE(arg_pEngine, "PORT STATS", arg_port,
                                 pCbPortStats, (_IX_CC_ATMSAR_ENGINE_SERVICE_PLUGIN_INSTANCE,
                                                arg_port, arg_pLookupResult));
} /* end   */


/**
 * NAME: _ix_cc_atmsar_engine_cid_create
 *
 * DESCRIPTION:
 *    This function creates an CID instance.
 * 
 * @Param:  IN  arg_pEngine : pointer to structure describing supported
 *                  core components.
 * @Param:  IN  arg_pCidDescr : pointer to data describing parameters
 *                  for requested CID
 * @Param:  OUT arg_pCid : pointer to a variable storing the created CID.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - Instance successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_NO_RESOURCES - no resources to add the entry 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error
_ix_cc_atmsar_engine_cid_create(
const _ix_cc_atmsar_engine_t *arg_pEngine,
ix_cc_atmsar_cid_handle_t arg_cid,
const ix_cc_atmsar_cid_descr_t* arg_pCidDescr)
{
    _IX_CC_ATMSAR_ENGINE_SERVICE(arg_pEngine, "CID CREATE", arg_cid,
                                 pCbCidCreate, (_IX_CC_ATMSAR_ENGINE_SERVICE_PLUGIN_INSTANCE,
                                                arg_cid, arg_pCidDescr));
    return IX_SUCCESS;
} /* end _ix_cc_atmsar_engine_cid_create */


/**
 * NAME: _ix_cc_atmsar_engine_vc_update
 *
 * DESCRIPTION:
 *    This function updates a VC with specified value in 'arg_pVcDescr'.
 *    The function calls the core component managed by agent or main
 *    ATM SAR. The list of sub-rutines for calling is pessed in
 *    'arg_pEngine'.
 *
 * @Param:  IN  arg_pEngine : pointer to structure describing supported
 *                  core components.
 * @Param:  IN  arg_pUpdate : parameters describing requested CID.
 * @Param:  OUT arg_cid : handle to updated CID.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM CID  successfully updated
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_NO_RESOURCES - no resources to add the entry 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error
_ix_cc_atmsar_engine_cid_update(
const _ix_cc_atmsar_engine_t *arg_pEngine,
const ix_cc_atmsar_cid_handle_t arg_cid,
const ix_cc_atmsar_update_data_t* arg_pUpdate)
{
    _IX_CC_ATMSAR_ENGINE_SERVICE(arg_pEngine, "CID UPDATE", arg_cid,
                                 pCbCidUpdate, (_IX_CC_ATMSAR_ENGINE_SERVICE_PLUGIN_INSTANCE,
                                                arg_cid, arg_pUpdate));
    return IX_SUCCESS;
} /* end _ix_cc_atmsar_engine_vc_update */

/**
 * NAME: _ix_cc_atmsar_engine_cid_remove
 *
 * DESCRIPTION:
 *    This function removes an CID instance.
 * 
 * @Param:  IN  arg_pEngine : pointer to structure describing supported
 *                  core components.
 * @Param:  IN  arg_cid : handle for CID to be removed
 * @Param:  OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - instance successfully removed
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error
_ix_cc_atmsar_engine_cid_remove(
const _ix_cc_atmsar_engine_t *arg_pEngine,
ix_cc_atmsar_cid_handle_t arg_cid)
{
    _IX_CC_ATMSAR_ENGINE_SERVICE_REVERSE(arg_pEngine, "CID REMOVE", arg_cid,
                                         pCbCidRemove, (_IX_CC_ATMSAR_ENGINE_SERVICE_PLUGIN_INSTANCE,
                                                        arg_cid));
} /* end _ix_cc_atmsar_engine_cid_remove */


/**
 * NAME: _ix_cc_atmsar_engine_get_cid_stats
 *
 * DESCRIPTION:
 *    This function retrieves an CID statistics. 
 * 
 * @Param:  IN  arg_pEngine : pointer to structure describing supported
 *                  core components.
 * @Param:  IN  arg_cid : CID handle
 * @Param:  OUT arg_pLookupResult : pointer to ix_cc_sar_cid_stats structure
 *                  where the requested data will be stored
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - statistics successfully retrieved
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error
_ix_cc_atmsar_engine_get_cid_stats(
const _ix_cc_atmsar_engine_t *arg_pEngine,
ix_cc_atmsar_cid_handle_t arg_cid,
ix_cc_atmsar_cid_stats_t *arg_pLookupResult)
{
    _IX_CC_ATMSAR_ENGINE_SERVICE(arg_pEngine, "CID STATS", arg_cid,
                                 pCbCidStats, (_IX_CC_ATMSAR_ENGINE_SERVICE_PLUGIN_INSTANCE,
                                               arg_cid, arg_pLookupResult));
} /* end _ix_cc_atmsar_engine_get_cid_stats */


/**
 * NAME: _ix_cc_atmsar_enum_cc_from_list
 *
 * DESCRIPTION:
 *    Enumerates CC_LIST
 *    Supports _ix_cc_atmsar_engine_read_cfg by enumerating each of the CC id
 *    numbers stored in CC_LIST.  Supports any non-digit separator.  Each time
 *    this function is called, it returns the next number in the list or 
 *    (-1) when done.  arg_listout is updated with the new location in 
 *    the original list string and should be passed as the first parameter
 *    to each call except the first.  On the first call, the original string
 *    is passed into arg_list.
 *
 * @Param: arg_list pointer to current location in list.
 * @Param: arg_listout new pointer to current location, used in next call
 * @Return: int the next number in the list or (-1) when complete.
 *
 * @retval -1 enumeration complete
 */
ix_int16
_ix_cc_atmsar_enum_cc_from_list(
char *arg_list,
char **arg_listout)
{
	ix_int16 ret;

    /* Validate Args */
    if((arg_list == NULL) || (arg_listout == NULL)){
        return -1;
    }

    /* Handle any leading spaces tabs etc.. */
	while((*arg_list != 0) && !isdigit((int)*arg_list)){
		arg_list++;
	}

    /* No more to find */
	if(*arg_list == 0)
		return -1;

    /* Get the value */
	ret = atoi(arg_list);
    /* Zip past the current number */
	while((*arg_list != 0) && isdigit((int)*arg_list)){
		arg_list++;
	}

    /* set arg_listout to the current place in the string */
	*arg_listout = arg_list;
	return ret;
} /* end _ix_cc_atmsar_enum_cc_from_list */


/**
 * NAME: _ix_cc_atmsar_engine_property_name_set
 *
 * DESCRIPTION:
 *    This primitive builds full path to variable. It adds configuration
 *    section prefix and adds instance to prefix.
 *           .
 * @Param:  IN  arg_sarInstance : Id of SAR Instance for that the action is
 *                  performed.
 * @Param:  OUT arg_pBuf : destination buffer
 * @Param:  IN  arg_pBufSize : size of destination buffer
 * @Param:  IN  arg_pPropertyName : buffer with property Name
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMTX_ERROR_REGISTRY  - not able to access registry.
 */
static ix_error
_ix_cc_atmsar_engine_property_name_set(
ix_cc_atmsar_instance_t arg_sarInstance,
char *arg_pBuf,
ix_uint16 arg_pBufSize,
const char *arg_pProperty)
{
    char *pSection="SAR";

    IX_ERROR_DCHECK_ARG_NULL(2, arg_pBuf);
    IX_ERROR_DCHECK_ARG_NULL(4, arg_pProperty);
    IX_ERROR_DCHECK_ARG_MAX(4, (strlen(pSection) + strlen(arg_pProperty) + 6),
                            arg_pBufSize, "%d");

    sprintf(arg_pBuf, "%s_%d%c%s", pSection, arg_sarInstance,
            IX_CP_PATH_NAME_DELIMITER, arg_pProperty);

    return IX_SUCCESS;
} /* end _ix_cc_atmsar_engine_property_name_set */


/**
 * NAME: _ix_cc_atmsar_engine_read_cfg
 *
 * DESCRIPTION:
 *    This function prepares/fills configuration structure. It reads
 *    configuration file or it gets the configuration from header file.
 *    The source of configuration is switched by compilation switch.
 *    The read configuration is preapared for one instance of SAR and for AGENT
 *    or MAIN core component. It depends on input parameters.
 * 
 * @Param:  IN  arg_instance :  Identificator of instance.
 * @Param:  IN  arg_moduleId :  Identificator of Module.
 * @Param:  IN  arg_pEngine :  pointer to structure that will be felt.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully created
 */
ix_error
_ix_cc_atmsar_engine_read_cfg(
ix_cc_atmsar_instance_t arg_instance,
_ix_cc_atmsar_module_id_t arg_moduleId,
_ix_cc_atmsar_engine_t* arg_pEngine)
{
#ifdef IX_INCLUDE_REGISTRY

#define _IX_CC_ATMSAR_MAX_PLUGIN 30
    static char* fncName= "_ix_cc_atmsar_engine_read_cfg";
    char name[IX_CP_MAX_PROPERTY_NAME_LENGTH*2];
    ix_uint16 ccIdTab[_IX_CC_ATMSAR_MAX_PLUGIN];
    ix_configuration_property_handle hCoreRoot = IX_CP_CORE_PROPERTY_ROOT;
    ix_cp_property_info cclistInfo;
    ix_configuration_property_handle hProperty;
    ix_int16 ccId;
    char *cclistData = NULL;
    char *cclistCur;
    ix_uint32 tmpUint32;
    ix_error err = IX_SUCCESS;
        
    cclistCur=(arg_moduleId==_IX_CC_ATMSAR_MODULE_ID_MAIN)?"CC_LIST_MAIN":"CC_LIST_AGENT";
    IX_ERROR_CR(_ix_cc_atmsar_engine_property_name_set(arg_instance, name, sizeof(name), cclistCur));
    IX_ERROR_CR(ix_rm_cp_property_open(hCoreRoot, name, &hProperty));
    IX_ERROR_CR(ix_rm_cp_property_get_info(hProperty, &cclistInfo));

    /* allocate space for and fetch the CCLIST*/
    cclistData = (char *)ix_ossl_malloc(cclistInfo.m_DataSize);
    if (cclistData == NULL)
    {
        return IX_ERROR_LOCAL(IX_CC_ERROR_OOM, 
                              ("Unable to allocate %d bytes for cclist.",
                               cclistInfo.m_DataSize));
    }
    
    IX_ERROR_CG(ix_rm_cp_property_get_value(hProperty, 
                                            &cclistInfo.m_DataSize, 
                                            cclistData),
                err, e_post_alloc);

    /*
     * enumerate each entry in the CCLIST string and store the id into
     * configuration structure of SAR
     */
    arg_pEngine->ccNumber=0;
    ccId = _ix_cc_atmsar_enum_cc_from_list(cclistData, &cclistCur);
    while (ccId >= 0)
    {
        _IX_CC_ATMSAR_DBG_ENG(("%s(): Read CC ID[%d]=%d\n", fncName,
                               arg_pEngine->ccNumber, ccId));

        ccIdTab[arg_pEngine->ccNumber]=ccId;
        arg_pEngine->ccNumber++;
        if (arg_pEngine->ccNumber >= _IX_CC_ATMSAR_MAX_PLUGIN)
        {
            err= IX_ERROR_LOCAL(IX_CC_ERROR_OOM, 
                                ("Temporary table is not ready for the number of entries."));
            goto e_post_alloc;
        }
        ccId = _ix_cc_atmsar_enum_cc_from_list(cclistCur, &cclistCur);        
    }
    if (cclistData != NULL)
        ix_ossl_free(cclistData);

    /*arg_pEngine->pluginTab= (_ix_cc_atmsar_plugin_entry_t*)cclistData;*/

    /*
     * Prepare table of plug-ins serviced by core component that request the
     * configuration. The table is empty. Only core component Id for plugins
     * is set into the new allocated table;.
     */
    tmpUint32=sizeof(arg_pEngine->pluginTab[0])*arg_pEngine->ccNumber;
    arg_pEngine->pluginTab= ix_ossl_malloc(tmpUint32);
    if (arg_pEngine->pluginTab == NULL)
    {
        return IX_ERROR_LOCAL(IX_CC_ERROR_OOM, 
                              ("Unable to allocate %ld bytes for plug-in description.",
                               tmpUint32));
    }
    ix_ossl_memset(arg_pEngine->pluginTab, 0, tmpUint32);

    for(ccId=0; ccId < arg_pEngine->ccNumber; ccId++)
    {
        arg_pEngine->pluginTab[ccId].plugInId=ccIdTab[ccId];
    }

    /* read the rest of parameters */
    IX_ERROR_CR(ix_cc_atmsar_plugin_get_cfg_params(arg_instance, &(arg_pEngine->cfg)));

    return IX_SUCCESS;

  e_post_alloc:
    if(cclistData != NULL){
        ix_ossl_free(cclistData);
    }
    return err;

#undef _IX_CC_ATMSAR_MAX_PLUGIN
    
#else /* IX_INCLUDE_REGISTRY */
    return IX_ERROR_LOCAL(IX_CC_ERROR_OOM, 
                          ("Functionality current is not available"));

#endif /* IX_INCLUDE_REGISTRY */

} /* end _ix_cc_atmsar_engine_read_cfg */


/**
 * NAME: ix_cc_atmsar_plugin_get_cfg_params
 *
 * DESCRIPTION:
 *    The function gets configuration parameters for SAR instance. The values
 *    are read from registry or from configuration header. The configuration
 *    parameters describe general behaviors and functionality of SAR.
 *
 * @Param:  IN  arg_sarInstance : Id of SAR Instance for that the action is
 *                  performed.
 * @Param:  OUT arg_pCfgParams : pointer to structure where configuration
 *                  parameters are stored.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - registering successful
 *    IX_CC_ERROR_INTERNAL - The plug-in that trries to register works in the
 *          wrong thread. Check description of ATM SAR configuration tables
 *          and XML config description.
 */
ix_error
ix_cc_atmsar_plugin_get_cfg_params(
ix_cc_atmsar_instance_t arg_sarInstance,
ix_cc_atmsar_plugin_cfg_params_t *arg_pCfgParams)
{
#ifdef IX_INCLUDE_REGISTRY

    char name[IX_CP_MAX_PROPERTY_NAME_LENGTH*2];
    ix_uint32 tmpUint32;
    ix_configuration_property_handle hCoreRoot = IX_CP_CORE_PROPERTY_ROOT;
    ix_configuration_property_handle hProperty;

    IX_ERROR_DCHECK_ARG_NULL(2, arg_pCfgParams);

    ix_ossl_memset(arg_pCfgParams, 0, sizeof(*arg_pCfgParams));

    /* AAL2_INCLUDED property */
    IX_ERROR_CR(_ix_cc_atmsar_engine_property_name_set(arg_sarInstance, name, sizeof(name), "AAL2_INCLUDED"));
    IX_ERROR_CR(ix_rm_cp_property_open(hCoreRoot, name, &hProperty));
    IX_ERROR_CR(ix_rm_cp_property_get_value_uint32(hProperty, &tmpUint32));
    arg_pCfgParams->aal2Included=(tmpUint32==1)?IX_CC_ATMSAR_PLUGIN_AAL2_INLUDED_YES:IX_CC_ATMSAR_PLUGIN_AAL2_INLUDED_NO;

    /* NUMBER_OF_VCS property */
    IX_ERROR_CR(_ix_cc_atmsar_engine_property_name_set(arg_sarInstance, name, sizeof(name), "NUMBER_OF_VCS"));
    IX_ERROR_CR(ix_rm_cp_property_open(hCoreRoot, name, &hProperty));
    IX_ERROR_CR(ix_rm_cp_property_get_value_uint32(hProperty, &tmpUint32));
    arg_pCfgParams->numberOfVcs=tmpUint32;
    
    /* NUMBER_OF_PORTS property */
    IX_ERROR_CR(_ix_cc_atmsar_engine_property_name_set(arg_sarInstance, name, sizeof(name), "NUMBER_OF_PORTS"));
    IX_ERROR_CR(ix_rm_cp_property_open(hCoreRoot, name, &hProperty));
    IX_ERROR_CR(ix_rm_cp_property_get_value_uint32(hProperty, &tmpUint32));
    arg_pCfgParams->numberOfPorts=tmpUint32;
    
    /* NUMBER_OF_CIDS property */
    IX_ERROR_CR(_ix_cc_atmsar_engine_property_name_set(arg_sarInstance, name, sizeof(name), "NUMBER_OF_CIDS"));
    IX_ERROR_CR(ix_rm_cp_property_open(hCoreRoot, name, &hProperty));
    IX_ERROR_CR(ix_rm_cp_property_get_value_uint32(hProperty, &tmpUint32));
    arg_pCfgParams->numberOfCids=tmpUint32;
    
    return IX_SUCCESS;

#else /* IX_INCLUDE_REGISTRY */

    IX_ERROR_DCHECK_ARG_NULL(2, arg_pCfgParams);
    
    ix_ossl_memset(arg_pCfgParams, 0, sizeof(*arg_pCfgParams));
    arg_pCfgParams->aal2Included=IX_CC_ATMSAR_PLUGIN_AAL2_INLUDED_NO;
    arg_pCfgParams->numberOfVcs=0;
    arg_pCfgParams->numberOfPorts=0;
    arg_pCfgParams->numberOfCids=0;
    
    return IX_SUCCESS;

#endif /* IX_INCLUDE_REGISTRY */

} /* end ix_cc_atmsar_plugin_get_cfg_params */


/**
 * NAME: ix_cc_atmsar_plugin_get_cfg_ports_params
 *
 * DESCRIPTION:
 *    The function gets port configuration parameters for SAR instance. The
 *    configuration is read from registry or from configuration header.
 *
 * @Param:  IN  arg_sarInstance : Id of SAR Instance for that the action is
 *                  performed.
 * @Param:  IN  arg_portNo : number of ports of port for that the action is performed.
 * @Param:  OUT arg_pCfgParams : pointer to table with structures where configuration
 *                  parameters for all ports are stored.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - registering successful
 *    IX_CC_ERROR_INTERNAL - The plug-in that trries to register works in the
 *          wrong thread. Check description of ATM SAR configuration tables
 *          and XML config description.
 */
ix_error
ix_cc_atmsar_plugin_get_cfg_ports_params(
ix_cc_atmsar_instance_t arg_sarInstance,
ix_uint16 arg_portNo,
ix_cc_atmsar_plugin_cfg_port_descr_t *arg_pCfgParams)
{
    static char* fncName= "ix_cc_atmsar_plugin_get_cfg_ports_params";
#ifdef IX_INCLUDE_REGISTRY

    char name[IX_CP_MAX_PROPERTY_NAME_LENGTH*2];
    char tmpName[IX_CP_MAX_PROPERTY_NAME_LENGTH*2];
    ix_uint32 tmpUint32;
    ix_configuration_property_handle hSpProp = 0;
    ix_configuration_property_handle hPortProperty = 0;
    ix_uint32 idx;
    ix_cp_property_info propertyInfo;
    ix_error err = IX_SUCCESS;
    ix_cc_atmsar_plugin_cfg_port_descr_t *pCfgParams;
    
    ix_configuration_property_handle hCoreRoot = IX_CP_CORE_PROPERTY_ROOT;
    ix_configuration_property_handle hProperty;

    IX_ERROR_DCHECK_ARG_NULL(3, arg_pCfgParams);

    IX_ERROR_CR(_ix_cc_atmsar_engine_property_name_set(arg_sarInstance, name, sizeof(name), ""));

    name[strlen(name)-1]= '\0'; /* remove last character */
    _IX_CC_ATMSAR_DBG_ENG(("%s(): START path:<%s>, portNo:%d\n", fncName, name, arg_portNo));
    
    /* open base property */
    IX_ERROR_CR(ix_cc_reg_get_prop_from_path(name, &hSpProp));

    /* clean structure */
    ix_ossl_memset(arg_pCfgParams, 0, arg_portNo*sizeof(*arg_pCfgParams));
    
    _IX_CC_ATMSAR_DBG_ENG(("%s(): hSpProp: %ld\n", fncName, hSpProp));

    /* enumerate ports */
    for(idx=0, hPortProperty=0;
        ((err=ix_rm_cp_property_get_subproperty(hSpProp, idx, &hPortProperty)) == IX_SUCCESS) && (hPortProperty != 0);
        idx++)
    {
        IX_ERROR_CG(ix_rm_cp_property_get_info(hPortProperty, &propertyInfo),
                    err, post_prop);

        /* make sure it starts with PORT*/
        if(strncmp(propertyInfo.m_aName, "PORT", 4))
        {
            /* not a port, continue */
            continue;
        }
        _IX_CC_ATMSAR_DBG_ENG(("%s(): PORT <%s>\n", fncName, propertyInfo.m_aName));

        IX_ERROR_CG(ix_rm_cp_property_get_value_uint32(hPortProperty, &tmpUint32),
                    err, post_prop);

        if(tmpUint32>arg_portNo)
        {
            err= IX_ERROR_LOCAL(IX_CC_ERROR_INTERNAL, 
                                ("For port <%s> id = %ld is out of number of port (%d).",
                                 propertyInfo.m_aName, tmpUint32, arg_portNo));
            goto post_prop;
        }

        pCfgParams=&(arg_pCfgParams[tmpUint32]);
        
        /*--------------------------------------------------------*/
        /* get the LINK_RATE */
        sprintf(tmpName, "%s%cLINK_RATE", propertyInfo.m_aName, IX_CP_PATH_NAME_DELIMITER);
        IX_ERROR_CG(_ix_cc_atmsar_engine_property_name_set(arg_sarInstance, name, sizeof(name), tmpName),
                    err, post_prop);
        IX_ERROR_CG(ix_rm_cp_property_open(hCoreRoot, name, &hProperty), err, post_prop);
        IX_ERROR_CG(ix_rm_cp_property_get_value_uint32(hProperty, &tmpUint32), err, post_prop);
        
        strncpy(pCfgParams->linkName, &(propertyInfo.m_aName[4]), IX_CC_ATMSAR_PLUGIN_LINK_NAME_MAX-1);
        pCfgParams->linkName[IX_CC_ATMSAR_PLUGIN_LINK_NAME_MAX-1]='\0';
        pCfgParams->linkRate=tmpUint32;

        _IX_CC_ATMSAR_DBG_ENG(("%s(): PORT <%s> link_rate=%ld\n", fncName, propertyInfo.m_aName, tmpUint32));

        /*
          IX_ERROR_CG(ix_rm_cp_property_close(hProperty), err, post_prop);
          IX_ERROR_CG(ix_rm_cp_property_close(hPortProperty), err, post_prop);
        */
    }

 post_prop:
    /*
    if (hSpProp != 0)
    {
        IX_ERROR_CR(ix_rm_cp_property_close(hSpProp));
    }
    if (hPortProperty != 0)
    {
        IX_ERROR_CR(ix_rm_cp_property_close(hPortProperty));
    }
    */
    return err;

#else /* IX_INCLUDE_REGISTRY */

    IX_ERROR_DCHECK_ARG_NULL(2, arg_pCfgParams);
    
    ix_ossl_memset(arg_pCfgParams, 0, sizeof(*arg_pCfgParams));
    arg_pCfgParams->lineRate=0;
    
    return IX_SUCCESS;

#endif /* IX_INCLUDE_REGISTRY */
} /* end ix_cc_atmsar_plugin_get_cfg_ports_params */

