/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      TBD
 *
 * = FILENAME
 *      ix_cc_atmsar_list.c
 *
 * = DESCRIPTION
 *      The file defines for init interface and internal functions.
 *
 * = AUTHOR
 *       Jaroslaw Kogut
 *       jaroslaw.kogut@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */
#define IX_ERROR_FILE_IDENT "$Id: ix_cc_atmsar_list.c,v 1.2 2003/09/10 07:37:10 rwielick Exp $";

/**
 * System defined include files
 */

/**
 * User defined include files
 */
#include "ix_ossl.h"
#include "cc/ix_cc_atmsar_list.h"

/**
 * Preprocessor symbols and Macros used in this file
 */

/**
 * Type definitions whose scope is limited to this file
 */


/**
 * Variable declarations global to this file only. Externs are followed by 
 * static variables.
 */


/** 
 * Extern function prototypes.
 */


/** 
 * Static function prototypes.
 */


/** 
 * Function definitions.
 */


/**
 * NAME: ix_cc_atm_list_init
 *
 * DESCRIPTION:
 *    The function initialize list
 * 
 * @Param:  - IN arg_pDescr - pointer to list description
 * @Param:  - IN arg_initFlag - type of list initialisation
 *
 * @Return: IX_CC_ATM_LIST_OK if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATM_LIST_ERROR
 */
ix_uint8
ix_cc_atm_list_init(
ix_cc_atm_list_descr_t *arg_pDescr,
ix_uint8 arg_initFlag)
{
    ix_cc_atm_list_entry_t* pEntry;
    ix_uint16 i;

    if(arg_pDescr->getEntry == NULL)
    {
        return IX_CC_ATM_LIST_ERROR;
    }
    if(arg_pDescr->pUserData == NULL)
    {
        return IX_CC_ATM_LIST_ERROR;
    }

    if(arg_pDescr->count >= (IX_CC_ATM_LIST_INVALID_HANDLE+1))
    {
        return IX_CC_ATM_LIST_ERROR;
    }

    switch(arg_initFlag)
    {
    case IX_CC_ATM_LIST_INIT_LINK:
        for (i=0 ; i < arg_pDescr->count ; i++)
        {
            pEntry=arg_pDescr->getEntry(arg_pDescr->pUserData, i);
            pEntry->next = i+1;
            pEntry->prev = i-1;
        }
        pEntry=(ix_cc_atm_list_entry_t*) arg_pDescr->getEntry(arg_pDescr->pUserData, 0);
        pEntry->prev = IX_CC_ATM_LIST_INVALID_HANDLE;
    
        pEntry=(ix_cc_atm_list_entry_t*) arg_pDescr->getEntry(arg_pDescr->pUserData, arg_pDescr->count-1);
        pEntry->next = IX_CC_ATM_LIST_INVALID_HANDLE;
        arg_pDescr->freeCount=arg_pDescr->count;
        arg_pDescr->freeHead=0;
        return IX_CC_ATM_LIST_OK;
        
    case IX_CC_ATM_LIST_INIT_NO_LINK:
        for (i=0 ; i < arg_pDescr->count ; i++)
        {
            pEntry=arg_pDescr->getEntry(arg_pDescr->pUserData, i);
            pEntry->next = IX_CC_ATM_LIST_INVALID_HANDLE;
            pEntry->prev = IX_CC_ATM_LIST_INVALID_HANDLE;
        }
        arg_pDescr->freeCount=0;
        arg_pDescr->freeHead=IX_CC_ATM_LIST_INVALID_HANDLE;
        return IX_CC_ATM_LIST_OK;
        
    default:
        return IX_CC_ATM_LIST_ERROR;
    }
}


/**
 * NAME: ix_cc_atm_list_alloc
 *
 * DESCRIPTION:
 *    The function gets entry from free list.
 * 
 * @Param:  - IN arg_pDescr - pointer to list description
 *
 * @Return:
 *     Handler of entry to use. If hander equals IX_CC_ATM_LIST_INVALID_HANDLE,
 *     it returns no entry,
 */
ix_uint16
ix_cc_atm_list_alloc(
ix_cc_atm_list_descr_t *arg_pDescr)
{
	ix_uint16              handle;
    ix_cc_atm_list_entry_t *pFirstFree;
    ix_cc_atm_list_entry_t *pNextFree;
    
	if (arg_pDescr->freeHead != IX_CC_ATM_LIST_INVALID_HANDLE)
    {
        handle=arg_pDescr->freeHead;
        pFirstFree = (ix_cc_atm_list_entry_t *)arg_pDescr->getEntry(arg_pDescr->pUserData, arg_pDescr->freeHead);
		if (pFirstFree->next != IX_CC_ATM_LIST_INVALID_HANDLE)
        {
            pNextFree=(ix_cc_atm_list_entry_t*)(arg_pDescr->getEntry(arg_pDescr->pUserData, pFirstFree->next));
			pNextFree->prev = IX_CC_ATM_LIST_INVALID_HANDLE;
		}
        arg_pDescr->freeHead=pFirstFree->next;
        pFirstFree->next = IX_CC_ATM_LIST_INVALID_HANDLE;
        pFirstFree->prev = IX_CC_ATM_LIST_INVALID_HANDLE;
        arg_pDescr->freeCount --;
		return (handle);
	}
	/* The freelist is empty */
    return IX_CC_ATM_LIST_INVALID_HANDLE;
}


/**
 * NAME: ix_cc_atm_list_free
 *
 * DESCRIPTION:
 *    Puts an entry into the freelist
 * 
 * @Param:  - IN arg_handle - Handle of entry to put
 * @Param:  - IN arg_pDescr - pointer to list description
 *
 * @Return:
 *     None
 */
void
ix_cc_atm_list_free(
ix_uint16              arg_handle,
ix_cc_atm_list_descr_t *arg_pDescr)
{
    ix_cc_atm_list_entry_t *pFirstFree;
    ix_cc_atm_list_entry_t *pPutEntry;

    pPutEntry = (ix_cc_atm_list_entry_t*)(arg_pDescr->getEntry(arg_pDescr->pUserData, arg_handle));
	pPutEntry->next = arg_pDescr->freeHead;
    pPutEntry->prev = IX_CC_ATM_LIST_INVALID_HANDLE;
	if (arg_pDescr->freeHead != IX_CC_ATM_LIST_INVALID_HANDLE)
    {
        pFirstFree=(ix_cc_atm_list_entry_t*)(arg_pDescr->getEntry(arg_pDescr->pUserData, arg_pDescr->freeHead));
		pFirstFree->prev = arg_handle;
    }
    arg_pDescr->freeHead = arg_handle;
    arg_pDescr->freeCount ++;
}


/**
 * NAME: ix_cc_atm_list_insert
 *
 * DESCRIPTION:
 *    This function inserts a entry to list where pHandler is head of list,
 *    key said which of next/prev handler should be serviced
 * 
 * @Param:  - IN arg_handle - Handle of entry to put
 * @Param:  - IN arg_key    - key of next/prev handler
 * @Param:  - IN arg_pHandler - handler to head of list
 * @Param:  - IN arg_pDescr - pointer to list description
 *
 * @Return:
 *     IX_CC_ATM_LIST_OK if inserted, otherwise IX_CC_ATM_LIST_ERROR.
 */
ix_uint8
ix_cc_atm_list_insert(
ix_uint16              arg_handle,
ix_uint16              arg_key,
ix_uint16              *arg_pHandler,
ix_cc_atm_list_descr_t *arg_pDescr)
{
    ix_uint8 *pTmpPos;
    ix_cc_atm_list_key_descr_t* pKey;
    ix_uint16 hPrev;             /* handler to previous entry before the entry   */
    ix_uint16 hNext;             /* handler to next entry after the entry        */
    ix_uint8 *pThisData;
    ix_uint8 *pTryData;
    ix_cc_atm_list_entry_t *pListHandle;
    ix_uint16 keyPos;

    keyPos = 0;                              /* default key pos is equal 0                   */
    hPrev = IX_CC_ATM_LIST_INVALID_HANDLE;  /* may be it should be first entry in list      */
    hNext = *arg_pHandler;                   /* may be it should be last entry in the list   */

    /* search place in list to put the entry */
    if( ( arg_pDescr->pKey != NULL) && ( arg_key < arg_pDescr->keyCount ) )
    {
        pKey = &(arg_pDescr->pKey[arg_key]);
        keyPos = pKey->pos;
        while(hNext != 0)
        {
            pTryData=arg_pDescr->getEntry(arg_pDescr->pUserData, hNext);

            /* comparision of currect entry and incomming entry */
            if( pKey->insert(pThisData, pTryData) == IX_CC_ATM_LIST_OK )
                break; /* place is fount */

            pTryData+=keyPos;
            pListHandle = (ix_cc_atm_list_entry_t*)(pTryData);

            hPrev = hNext;
            hNext = pListHandle->next;
        }
    }

    /* modyfication prevoius entry */
    if( hPrev == IX_CC_ATM_LIST_INVALID_HANDLE )
    {
        *arg_pHandler = arg_handle; /* head of list should be modified */
    }
    else
    {
        pTmpPos = arg_pDescr->getEntry(arg_pDescr->pUserData, hPrev);
        pTmpPos += keyPos;
        pListHandle = (ix_cc_atm_list_entry_t*)(pTmpPos);
        pListHandle->next = arg_handle;
    }
    
    /* modyfication next entry */
    if( hNext != IX_CC_ATM_LIST_INVALID_HANDLE )
    {
        pTmpPos = arg_pDescr->getEntry(arg_pDescr->pUserData, hNext);
        pTmpPos += keyPos;
        pListHandle = (ix_cc_atm_list_entry_t*)(pTmpPos);
        pListHandle->prev = arg_handle;
    }
    
    pTmpPos = arg_pDescr->getEntry(arg_pDescr->pUserData, arg_handle);
    pTmpPos += keyPos;
    pListHandle = (ix_cc_atm_list_entry_t*)(pTmpPos);
    pListHandle->prev = hPrev;
    pListHandle->next = hNext;
	return (IX_CC_ATM_LIST_OK);
}


/**
 * NAME: ix_cc_atm_list_free
 *
 * DESCRIPTION:
 *    This function remove a entry from the list.
 * 
 * @Param:  - IN arg_handle - Handle of entry to put
 * @Param:  - IN arg_key    - key of next/prev handler
 * @Param:  - IN arg_pHandler - handler to head of list
 * @Param:  - IN arg_pDescr - pointer to list description
 *
 * @Return:
 *     none
 */
void
ix_cc_atm_list_remove(
ix_uint16              arg_handle,
ix_uint16              arg_key,
ix_uint16              *arg_pHandler,
ix_cc_atm_list_descr_t *arg_pDescr)
{
    ix_uint8 *pTmpPos;
    ix_cc_atm_list_entry_t *pThisHandle;
    ix_cc_atm_list_entry_t *pTmpHandle;
    ix_uint16 keyPos;

    keyPos = 0;             /* default key pos is equal 0 */

    if( ( arg_pDescr->pKey != NULL) && ( arg_key < arg_pDescr->keyCount ) )
    {
        keyPos=arg_pDescr->pKey[arg_key].pos;
    }

    pTmpPos = arg_pDescr->getEntry(arg_pDescr->pUserData, arg_handle);
    pTmpPos += keyPos;
    pThisHandle = (ix_cc_atm_list_entry_t*)(pTmpPos);

    /* modyfication of previous entry */
    if(pThisHandle->prev != IX_CC_ATM_LIST_INVALID_HANDLE)
    {
        pTmpPos = arg_pDescr->getEntry(arg_pDescr->pUserData, pThisHandle->prev);
        pTmpPos += keyPos;
        pTmpHandle = (ix_cc_atm_list_entry_t*)(pTmpPos);
        pTmpHandle->next = pThisHandle->next;
    }

    /* modyfication of previous entry */
    if(pThisHandle->next != IX_CC_ATM_LIST_INVALID_HANDLE)
    {
        pTmpPos = arg_pDescr->getEntry(arg_pDescr->pUserData, pThisHandle->next);
        pTmpPos += keyPos;
        pTmpHandle = (ix_cc_atm_list_entry_t*)(pTmpPos);
        pTmpHandle->prev = pThisHandle->prev;
    }

    /* modyfication of head of list */
    if ( (*arg_pHandler) == arg_handle )
    {    
        (*arg_pHandler) = pThisHandle->next;
    }
}

