/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      ATM SAR MAIN Core Component - CCI - init function
 *
 * = FILENAME
 *      ix_cc_atmsar_cci.c
 *
 * = DESCRIPTION
 *      The file defines for core component infrestructure  interface and
 *      internal functions for initialization and shutdown.
 *
 * = AUTHOR
 *       Jaroslaw Kogut
 *       jaroslaw.kogut@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */
#define IX_ERROR_FILE_IDENT "$Id: ix_cc_atmsar_main_cci.c,v 1.3 2003/12/12 18:26:46 adziecia Exp $";



/**
 * System defined include files
 */
#include "ix_cc_error.h"
#include "ix_rm.h"
#include "ix_ossl.h"
#include "ix_cc.h"
#include "ix_cc_macros.h"
#include "ix_cc_properties.h"
#include "ix_config.h"
#include "bindings.h"
#include "cc/ix_cc_msup.h"

/**
 * User defined include files
 */
#include "cc/ix_cc_atmsar.h"
#include "cc/ix_cc_atmsar_main_cci.h"
#include "cc/internal/ix_cc_atmsar_main.h"
#include "cc/internal/ix_cc_atmsar_common.h"
#include "cc/internal/ix_cc_atmsar_msg.h"

/**
 * Preprocessor symbols and Macros used in this file
 */


/**
 * Type definitions whose scope is limited to this file
 */


/**
 * Variable declarations global to this file only. Externs are followed by 
 * static variables.
 */

static _ix_cc_atmsar_main_context_t* s_pAtmSarMainCtxTab[_IX_CC_ATMSAR_INSTANCE_MAX];


/** 
 * Extern function prototypes.
 */


/** 
 * Static function prototypes.
 */


/** 
 * Function definitions.
 */

/**
 * NAME: _ix_cc_atmsar_main_get_ctx
 *
 * DESCRIPTION:
 *    This function returns pointer to ATM SAR MAIN Core Component
 *    context.
 * 
 * @Param:  IN  arg_instance : Id of SAR Instance for that the action is
 *                  performed.
 * @Return: pointer to context of ATM SAR MAIN Core Component.
 */
_ix_cc_atmsar_main_context_t*
_ix_cc_atmsar_main_get_ctx(
ix_cc_atmsar_instance_t arg_instance)
{
    return s_pAtmSarMainCtxTab[arg_instance];
} /* end _ix_cc_atmsar_main_get_ctx */


/******************************************************************************
 *                   DEINICJALIZATION/SHUTDOWN FUNCTIONS
 *****************************************************************************/


/**
 * NAME: _ix_cc_atmsar_unregister_message_handler
 *
 * DESCRIPTION:
 *    This primitive will be used to unregister message handler. It calls CCI
 *    services in level 1 system and resore manager services in level 0 system.
 *
 * @Param:  IN  arg_pContext : pointer of core component context.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMSAR_ERROR_CCI - error from CCI
 */
ix_error
_ix_cc_atmsar_unregister_message_handler(
#ifdef CCI_MULTIINSTANCE
ix_cc_atmsar_instance_t instance,
#endif /* CCI_MULTIINSTANCE */
_ix_cc_atmsar_main_context_t *arg_pContext)
{
    ix_communication_id commId;

    _IX_CC_ATMSAR_UNUSED_ARG(arg_pContext);

#ifdef CCI_MULTIINSTANCE
    if (instance == 0)
    {
        commId = IX_CC_ATMSAR_MAIN_0_MSG_INPUT;
    }
    else
    {
        commId = IX_CC_ATMSAR_MAIN_1_MSG_INPUT;
    }
#else
    commId = IX_CC_ATMSAR_MAIN_MSG_INPUT;
#endif /* CCI_MULTIINSTANCE */
    
    /* Call CCI if CCI is enabled */
#if defined(IX_EXCLUDE_CCI)
    IX_ERROR_CRT(ix_rm_message_handler_unregister(commId),
                 IX_CC_ATMSAR_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);
#else
    IX_ERROR_CRT(ix_cci_cc_remove_message_handler(commId),
                 IX_CC_ATMSAR_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);
#endif
    return IX_SUCCESS;
}/* end _ix_cc_atmsar_unregister_message_handler */


/**
 * NAME: _ix_cc_atmsar_unregister_cci_packet_handlers
 *
 * DESCRIPTION:
 *    This primitive will be used to unregister packet handler with CCI. 
 * 
 * @Param:  - IN none
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMSAR_ERROR_CCI - error from CCI
 */

ix_error
_ix_cc_atmsar_unregister_cci_packet_handlers(void)
{
    /*#if !defined(IX_EXCLUDE_CCI) */
#if 0  
    /* 
     * the core component does not cooperate with microengines 
     * and it does need to process packets. Threrefore, the handler is not
     * registered. The function is defined for general framework of core 
     * components.
     */
    IX_ERROR_CRT(ix_cci_cc_remove_message_handler(IX_CC_ATMSAR_PKT_INPUT),
                 IX_CC_ATMSAR_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);
#endif
    return IX_SUCCESS;
}/* end _ix_cc_atmsar_unregister_cci_packet_handlers */


/**
 * NAME: _ix_cc_atmsar_unregister_rm_packet_handlers
 *
 * DESCRIPTION:
 *    This primitive will be used to unregister packet handler with resource
 *    manager
 * 
 * @Param:  - IN none
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMSAR_ERROR_CCI - error from RM
 */
ix_error
_ix_cc_atmsar_unregister_rm_packet_handlers(void)
{
#if 0
    /* 
     * the core component does not cooperate with microengines 
     * and it does need to process packets. Threrefore, the handler is not
     * registered. The function is defined for general framework of core 
     * components.
     */
    IX_ERROR_CRT(ix_rm_message_handler_unregister(IX_CC_ATMSAR_PKT_INPUT),
                     IX_CC_ATMSAR_ERROR_CCI,
                     IX_ERROR_LEVEL_WARNING);
#endif
    return IX_SUCCESS;
}/* end _ix_cc_atmsar_unregister_rm_packet_handlers */


/**
 * NAME: _ix_cc_atmsar_unregister_packet_handlers
 *
 * DESCRIPTION:
 *    This primitive will be used to unregister packet handler  with
 *    CCI/Resource manager
 *
 * @Param:  IN  arg_pContext : pointer of core component context.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMSAR_ERROR_CCI - error from CCI/RM
 */
ix_error
_ix_cc_atmsar_unregister_packet_handlers(
_ix_cc_atmsar_main_context_t *pContext)
{
    _IX_CC_ATMSAR_UNUSED_ARG(pContext);
    /* Call CCI if CCI is enabled */
#if defined(IX_EXCLUDE_CCI)
    IX_ERROR_CR(_ix_cc_atmsar_unregister_rm_packet_handlers());
#else
    IX_ERROR_CR(_ix_cc_atmsar_unregister_cci_packet_handlers());
#endif
    return IX_SUCCESS;
}/* end _ix_cc_atmsar_unregister_packet_handlers */


/**
 * NAME: _ix_cc_atmsar_delete_event_handler
 *
 * DESCRIPTION:
 *    This primitive will be used to delete event handler. User needs to
 *    implement this function in level0 system.
 *
 * @Param:  - IN arg_pEventId - id of ICMP event hanlder
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ERROR_UNIMPL - not implemented.
 */
ix_error
_ix_cc_atmsar_delete_event_handler(ix_ossl_thread_t arg_pEventId)
{
    return IX_ERROR_WARNING(IX_CC_ERROR_UNIMPL,("\n"));
}/* end _ix_cc_atmsar_delete_event_handler */


/******************************************************************************
 *                         INITIALISATION FUNCTION
 *****************************************************************************/


/**
 * NAME: _ix_cc_atmsar_register_message_handler
 *
 * DESCRIPTION: This primitive will be used to register message handler. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMSAR_ERROR_CCI - Error from CCI
 */
ix_error
_ix_cc_atmsar_register_message_handler(
#ifdef CCI_MULTIINSTANCE
ix_cc_atmsar_instance_t instance,
#endif /* CCI_MULTIINSTANCE */
_ix_cc_atmsar_main_context_t *arg_pContext)
{
    ix_communication_id commId;

    _IX_CC_ATMSAR_UNUSED_ARG(arg_pContext);

#ifdef CCI_MULTIINSTANCE
    if (instance == 0)
    {
        commId = IX_CC_ATMSAR_MAIN_0_MSG_INPUT;
    }
    else
    {
        commId = IX_CC_ATMSAR_MAIN_1_MSG_INPUT;
    }
#else
    commId = IX_CC_ATMSAR_MAIN_MSG_INPUT;
#endif /* CCI_MULTIINSTANCE */
    
    /* Call CCI message handler if CCI is enabled */
#if defined(IX_EXCLUDE_CCI) 
    IX_ERROR_CRT(ix_rm_message_handler_register(commId,
                                                ix_cc_atmsar_msg_handler,
                                                arg_pContext),
                 IX_CC_ATMSAR_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);
#else
    IX_ERROR_CRT(ix_cci_cc_add_message_handler(arg_pContext->hAtmsarHandle,
                                               commId,
                                               ix_cc_atmsar_msg_handler,
                                               IX_INPUT_TYPE_MULTI_SRC),
                 IX_CC_ATMSAR_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);

#endif
    return IX_SUCCESS;
}/* end _ix_cc_atmsar_register_message_handler */



/**
 * NAME: _ix_cc_atmsar_register_cci_packet_handlers
 *
 * DESCRIPTION:
 *    This primitive will be used to register packet handler with CCI.
 *
 * @Param:  IN  arg_pContext : pointer of core component context.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * IX_CC_ATMSAR_ERROR_CCI - error from CCI
 */
ix_error
_ix_cc_atmsar_register_cci_packet_handlers(
_ix_cc_atmsar_main_context_t *arg_pContext)
{
    /*#if !defined(IX_EXCLUDE_CCI)*/
#if 0
    /* 
     * the core component does not cooperate with microengines 
     * and it does need to process packets. Threrefore, the handler is not
     * registered. The function is defined for general framework of core 
     * components.
     */
    /* High priority packet handler needs to be registered as a message handler */
    IX_ERROR_CRT(ix_cci_cc_add_message_handler(arg_pContext->hAtmsarHandle,
                                               IX_CC_ATMSAR_PKT_INPUT,
                                               ix_cc_atmsar_pkt_handler,
                                               IX_INPUT_TYPE_SINGLE_SRC),
                 IX_CC_ATMSAR_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);
#endif
    return IX_SUCCESS;
}/* end _ix_cc_atmsar_register_cci_packet_handlers */


/**
 * NAME: _ix_cc_atmsar_register_rm_packet_handlers
 *
 * DESCRIPTION:
 *    This primitive will be used to register packet handler with resource
 *    manager.
 *
 * @Param:  IN  arg_pContext : pointer of core component context.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMSAR_ERROR_CCI - error from resource manager
 */
ix_error _ix_cc_atmsar_register_rm_packet_handlers(
_ix_cc_atmsar_main_context_t *arg_pContext)
{
#if 0
    /* 
     * the core component does not cooperate with microengines 
     * and it does need to process packets. Threrefore, the handler is not
     * registered. The function is defined for general framework of core 
     * components.
     */
    IX_ERROR_CRT(ix_rm_message_handler_register(IX_CC_ATMSAR_PKT_INPUT,
                                                ix_cc_atmsar_pkt_handler,
                                                arg_pContext),
                 IX_CC_ATMSAR_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);
#endif
    return IX_SUCCESS;
}/* end _ix_cc_atmsar_register_rm_packet_handlers */


/**
 * NAME: _ix_cc_atmsar_register_packet_handlers
 *
 * DESCRIPTION:
 *    This primitive will be used to register packet handlers. It registers
 *    with CCI in level 1 system and register with RM in level 0 system.
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMSAR_ERROR_CCI - error from CCI/resource manager
 */
ix_error _ix_cc_atmsar_register_packet_handlers(
_ix_cc_atmsar_main_context_t *arg_pContext)
{
    /* Call CCI packet handler if CCI is enabled */
#if defined(IX_EXCLUDE_CCI)
    IX_ERROR_CR(_ix_cc_atmsar_register_rm_packet_handlers(arg_pContext));
#else
    IX_ERROR_CR(_ix_cc_atmsar_register_cci_packet_handlers(arg_pContext));
#endif
    return IX_SUCCESS;
}/* end _ix_cc_atmsar_register_packet_handlers */


/**
 * NAME: _ix_cc_atmsar_main_init_verify
 *
 * DESCRIPTION:
 *    This function check status of plug-in table for SAR Control Main
 *    subcomponent. It checks status of initialisation for SAR Control
 *    agent.
 * 
 * @Param:  IN  arg_pCtx : pointer to ATM SAR Main core component context.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 */
static ix_error
_ix_cc_atmsar_main_init_verify(
void *arg_pCtx)
{
    ix_uint16 i;
    _ix_cc_atmsar_plugin_entry_t *pTab;
    _ix_cc_atmsar_main_context_t* pCtx=(_ix_cc_atmsar_main_context_t*)arg_pCtx;
    ix_uint16 testFlag;

    if (pCtx==NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("Input parameter(arg_pCtx) is null"));
    }

    pTab=pCtx->engine.pluginTab;
    
    if (pCtx->readyFlag==TRUE)
    {
        return IX_SUCCESS;
    }
    /* find uninitialized plug-ins */
    for (i=0; i < pCtx->engine.ccNumber ; i++)
    {
        if ((pTab[i].initFlag & _IX_CC_ATMSAR_PLUGIN_INITFLAG_GENERAL) == 0)
        {
            /* any plug-in is uninitialized */
            _IX_CC_ATMSAR_DBG_INIT(( "_ix_cc_atmsar_main_init_verify() entry %d is not initialised\n", i ));
            return IX_SUCCESS;
        }
    }

    /* OK. all is initialized, we can verify/set that SAR Control MAIN is ready */
    pCtx->mainInitialized = _IX_CC_ATMSAR_INIT_MASK_PLUGIN;
    _IX_CC_ATMSAR_DBG_INIT(("   all plug-ins initialized\n"));

    /* find local handlers managers */
    i=0;
    if (pCtx->engine.pCbPortAlloc != NULL)
    {
        _IX_CC_ATMSAR_DBG_INIT(("   local port management installed\n"));
        i |= _IX_CC_ATMSAR_INIT_MASK_PORT_MGMNT;
    }
    if (pCtx->engine.pCbVcAlloc != NULL)
    {
        _IX_CC_ATMSAR_DBG_INIT(("   local vc management installed\n"));
        i |= _IX_CC_ATMSAR_INIT_MASK_VC_MGMNT;
    }
    if (pCtx->engine.pCbCidAlloc != NULL)
    {
        _IX_CC_ATMSAR_DBG_INIT(("   local cid management installed\n"));
        i |= _IX_CC_ATMSAR_INIT_MASK_CID_MGMNT;
    }

    pCtx->mainInitialized|=i;
        
    /*
     * check that in system exist handler managers for the same type
     * on local and remote side
     */
    if (((i & pCtx->agentInitialized) | _IX_CC_ATMSAR_INIT_MASK_PLUGIN) !=_IX_CC_ATMSAR_INIT_MASK_PLUGIN)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("ERROR - managers are dubbled\n"));
    }

    /*
     * Verify that all required handler managers already registered.
     * The s_pAtmsarMainCtx->agentInitialized contains too status of
     * initialisation for plug-ins on agent side
     */
    i|=pCtx->agentInitialized;
    testFlag = _IX_CC_ATMSAR_INIT_MASK_ALL_MGMNT;

    if (pCtx->engine.cfg.aal2Included == IX_CC_ATMSAR_PLUGIN_AAL2_INLUDED_NO)
    {
        testFlag &= ~(_IX_CC_ATMSAR_INIT_MASK_CID_MGMNT);
    }
    
    if (i == (testFlag | _IX_CC_ATMSAR_INIT_MASK_PLUGIN))
    {
        _IX_CC_ATMSAR_DBG_INIT(("SAR CONTROL READY FOR WORKING\n"));
        pCtx->readyFlag=TRUE;
    }
    return IX_SUCCESS;
        
} /* end _ix_cc_atmsar_main_init_verify */


/******************************************************************************
 *                CORE COMPONENT INFRASTRUCTURE FUNCTIONS
 *****************************************************************************/

/**
 * NAME: ix_cc_atmsar_init
 *
 * DESCRIPTION:
 *    This primitive shall be called and returned successfully before
 *    requesting any service from this core component. This primitive should be
 *    called only once to initialize this core component.This function performs
 *    allocation of memory for symbols to be patched,Creation of 64 bit
 *    counters, registration of packet and message handlers, allocation and
 *    initialization of internal data structures.
 * 
 * @Param:  - IN arg_hCcHandle - handle to this core component, created by core
 *            component infrastructure; This shall be used later to get other
 *            (to add event handler) services from CCI.
 * @Param:  - IN-OUT arg_ppContext - Buffer free list and dynamic property data
 *            will be given during by system APPs during initialization. Upon
 *            successfull completion, it represents the location where the
 *            pointer to the control block allocated by this core component
 *            will be stored. The control block is internal to the core
 *            component and it contains information about internal data
 *            structures, allocated memory and other relevant information.
 *            Later this pointer to be passed into core component fini()
 *            function for freeing memory and other house keeping operations
 *            when this component is being destroyed.  
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ERROR_NULL - input parameter arg_ppContext is null
 *    IX_CC_ERROR_OOM   - memory allocation failure      
 *    IX_CC_ATMSAR_ERROR_FAILED_PATCHING   - patching failures                      
 *    IX_CC_ERROR_OOM_64BIT_COUNTER - 64 bit counter creation failure
 *    IX_CC_ATMSAR_ERROR_REGISTRY  - registry access failures
 *    IX_CC_ATMSAR_ERROR_RTM - failure from RTM core component
 *    IX_CC_ATMSAR_ERROR_CCI - failure from CCI 
 */
ix_error ix_cc_atmsar_init(
ix_cc_handle arg_hCcHandle,
void** arg_ppContext)
{
    ix_error err = IX_SUCCESS;
    _ix_cc_atmsar_main_context_t* pContext;
    ix_cc_atmsar_instance_t instance;

    /* Variables used in retrieving static data from arg_ppContext */
    ix_cc_init_context *pInitCtx;

    _IX_CC_ATMSAR_DBG_INIT(( "ix_cc_atmsar_init() START\n"));

#ifdef CCI_MULTIINSTANCE
    /* use multiinstance CC ID if applicable */
    instance = IX_HANDLE_TO_INSTANCE(arg_hCcHandle);
#else
    /* if not, use default instance number */
    instance=_IX_CC_ATMSAR_INSTANCE_ID_TMP;
#endif /* CCI_MULTIINSTANCE */

    /* Check for argument validity */
    if (arg_ppContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("Input parameter(arg_ppContext) is null"));

    }/* end if (arg_ppContext == NULL) */

    /* veryfication of pointer to context for the instance */
    if (s_pAtmSarMainCtxTab[instance] != NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("Instance %d for SAR Control Main already initialized"));
    }
    
    /* Allocate memory for SAR Control Main Context */
    pContext = (_ix_cc_atmsar_main_context_t*)ix_ossl_malloc(sizeof(_ix_cc_atmsar_main_context_t));

    if (pContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,
                                ("Failed to allocate memory for SAR CTRL context"));
    }/* end if (pContext == NULL) */

    /* initialize the structure */
    ix_ossl_memset(pContext, 0, sizeof(_ix_cc_atmsar_main_context_t));

    /* store SAR Control MAIN component handle in _ix_cc_atmsar_main_context_t */
    pContext->hAtmsarHandle = arg_hCcHandle;

    if ((err=ix_ossl_thread_get_id(&pContext->engine.threadId)) != IX_SUCCESS)
    {
        goto cleanupContext;
    }

    /* The pointer has to be set now for veryfing purpose */
    s_pAtmSarMainCtxTab[instance]=pContext;
    
    pContext->engine.pInitVerify=_ix_cc_atmsar_main_init_verify;
    pContext->engine.pCtx=pContext;
    pContext->engine.appName="MAIN";

    /* Soft SAR core components is not registered into SAR Control */
#ifdef ATMSAR_MAIN_WITH_AGENT_SUPPORT
    pContext->agentInitialized=0;
#else
    pContext->agentInitialized=_IX_CC_ATMSAR_INIT_MASK_PLUGIN;
#endif
    pContext->readyFlag=FALSE;
    
    /* Get the generic init context */
    pInitCtx = (ix_cc_init_context *)*arg_ppContext;

    IX_ERROR_CG(_ix_cc_atmsar_engine_read_cfg(instance,
                                              _IX_CC_ATMSAR_MODULE_ID_MAIN,
                                              &(pContext->engine)),
                err, cleanupContext);

    if (pContext->engine.pInitVerify == NULL)
    {
        err=IX_ERROR_WARNING(IX_CC_ERROR_INTERNAL, ("pInitVerify is NULL"));
        goto cleanupContext;
    }
    
    IX_ERROR_CG(pContext->engine.pInitVerify(pContext->engine.pCtx),
                err, cleanupContext);
    
#ifdef CCI_MULTIINSTANCE
    /* Register Message Handler */
    IX_ERROR_CG(_ix_cc_atmsar_register_message_handler(
                instance,
                pContext),
                err, cleanupContext);
#else
    /* Register Message Handler */
    IX_ERROR_CG(_ix_cc_atmsar_register_message_handler(
                pContext),
                err, cleanupContext);
#endif /* CCI_MULTIINSTANCE */

    /* Register Packet Handler */
    IX_ERROR_CG(_ix_cc_atmsar_register_packet_handlers(pContext),
                err, cleanupMessageHandler);

    *arg_ppContext = (void *)pContext;
    _IX_CC_ATMSAR_DBG_INIT(( "ix_cc_atmsar_init() SUCCESS\n"));
    return IX_SUCCESS;
    
  cleanupMessageHandler:
    _ix_cc_atmsar_unregister_message_handler(
#ifdef CCI_MULTIINSTANCE
                instance,
#endif /* CCI_MULTIINSTANCE */
                pContext);
  cleanupContext:
    ix_ossl_free(pContext);
    s_pAtmSarMainCtxTab[instance]=NULL;
    return err;
} /* end ix_cc_atmsar_init */


/**
 * NAME: ix_cc_atmsar_fini
 *
 * DESCRIPTION:
 *    This primitive shall be called to terminate services from this core  
 *    component. This primitive frees memory allocated during initialization,
 *    free all the created resources: delete 64 bit counter. If client claims
 *    for any services from the core component after calling this function,
 *    then the behavior is undefined.
 * 
 * @Param:  - IN arg_hCcHandle - handle to the core component
 * @Param:  - IN arg_pContext - pointer to the control block memory allocated 
 *                   earlier in ix_cc_atmsar_init function. 
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMSAR_ERROR_INVALID_INPUT_PARAM - invalid input parameter(s)
 *    IX_CC_ERROR _OOM - failure of memory deletion                
 *    IX_CC_ERROR_OOM_64BIT_COUNTER - failure of  64 bit counter deletion 
 *    IX_CC_ATMSAR_ERROR _RTM - failure from RTM
 *    IX_CC_ATMSAR_ERROR_CCI - failure from CCI
 */
ix_error ix_cc_atmsar_fini(
ix_cc_handle arg_hCcHandle,
void* arg_pContext)
{
    ix_cc_atmsar_instance_t instance;
    _ix_cc_atmsar_main_context_t* pContext= (_ix_cc_atmsar_main_context_t*)arg_pContext;

#ifdef CCI_MULTIINSTANCE
    /* use multiinstance CC ID if applicable */
    instance = IX_HANDLE_TO_INSTANCE(arg_hCcHandle);
#else
    /* if not, use default instance number */
    instance=_IX_CC_ATMSAR_INSTANCE_ID_TMP;
#endif /* CCI_MULTIINSTANCE */

    /* Validate Parameters */
    if (arg_pContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("arg_pContext is null"));

    }/* if (arg_pContext == NULL)  */ 

    if (arg_pContext != s_pAtmSarMainCtxTab[instance])
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("arg_pContext is different than global test variable"));

    }/* if (arg_pContext != s_pAtmsarMainCtx) */ 


    if (arg_hCcHandle != pContext->hAtmsarHandle)
    {
        return IX_ERROR_WARNING(IX_CC_ATMSAR_ERROR_INVALID_INPUT_PARAM,
                                ("Invalid input Parameters"));
    }/* (arg_hCcHandle != pContext->hAtmsarHandle) */ 

#ifdef CCI_MULTIINSTANCE
    IX_ERROR_CR(_ix_cc_atmsar_unregister_message_handler(
                instance,
                pContext));
#else
    /* Unregister Message Handler */
    IX_ERROR_CR(_ix_cc_atmsar_unregister_message_handler(
                pContext));
#endif /* CCI_MULTIINSTANCE */
    
    /* Unregister Packet Handler */
    IX_ERROR_CR(_ix_cc_atmsar_unregister_packet_handlers(pContext));
    
    /* Free Atmsar Context */ 
    s_pAtmSarMainCtxTab[instance]=NULL;
    ix_ossl_free(pContext);
    return IX_SUCCESS;
} /* end ix_cc_atmsar_fini */


/**
 * NAME: ix_cc_atmsar_pkt_handler
 *
 * DESCRIPTION:
 *    This is the only registered function to receive exception packets from
 *    high priority queue of ATM RX microblock. This function sends the packet
 *    to stack driver (ie, recieves only local delivery packets). 
 * 
 * @Param:  - IN arg_hBuffer: handle to a buffer which contains exception packets
 *                   from ATM RX microblock
 * @Param:  - IN arg_exceptionCode: will be ignored.
 * @Param:  - IN arg_pContext: pointer to ATM RX core component context.
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ERROR_SEND_FAIL - error from CCI
 */
ix_error ix_cc_atmsar_pkt_handler(
ix_buffer_handle arg_hBuffer,  
ix_uint32 arg_exceptionCode,
void* arg_pContext)
{
    ix_error err = IX_SUCCESS;
  
    switch (arg_exceptionCode)
    {
    default:
        IX_ERROR_CRT(ix_rm_buffer_free_chain(arg_hBuffer), IX_CC_ERROR_SEND_FAIL, 0);
        return IX_ERROR_WARNING(IX_CC_ERROR_UNDEFINED_EXCEP,
                                ("undefined exception code"));
        break;
    }/* end switch */

    if (err != IX_SUCCESS)
    {
        IX_ERROR_CRT(ix_rm_buffer_free_chain(arg_hBuffer), IX_CC_ERROR_SEND_FAIL, 0);
        return IX_ERROR_WARNING(IX_CC_ERROR_SEND_FAIL,
                                ("packet send failed - high priority"));
    }/* end if (err != IX_SUCCESS) */     
    return IX_SUCCESS;
} /* end ix_cc_atmsar_pkt_handler */


