/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      ATM SAR Main Core Component
 *
 * = FILENAME
 *      ix_cc_atmsar_helper.c
 *
 * = DESCRIPTION
 *      The file defines message helper interface functions.
 *
 * = AUTHOR
 *       Jaroslaw Kogut
 *       jaroslaw.kogut@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */
#define IX_ERROR_FILE_IDENT "$Id: ix_cc_atmsar_main_helpers.c,v 1.7 2003/12/12 19:00:49 adziecia Exp $";


/**
 * System defined include files
 */
#include "ix_cc_error.h"
#include "ix_rm.h"
#include "ix_ossl.h"
#include "ix_cc.h"
#include "ix_cc_macros.h"
#include "ix_cc_properties.h"
#include "ix_config.h"
#include "bindings.h"
#include "cc/ix_cc_msup.h"

#if (_IX_OS_TYPE_ != _IX_OS_LINUX_KERNEL)
#include <assert.h>
#endif

/**
 * User defined include files
 */
#include "cc/ix_cc_atmsar.h"
#include "cc/internal/ix_cc_atmsar_main.h"
#include "cc/internal/ix_cc_atmsar_msg.h"

/**
 * Preprocessor symbols and Macros used in this file
 */


/**
 * Type definitions whose scope is limited to this file
 */

/**
 * TYPENAME: _ix_cc_atmsar_async_ctx_t
 * 
 * DESCRIPTION: TBD.
 *
 */
typedef struct{
    void* pUserContext;
    ix_cc_atmsar_cb_t callback;
    _ix_cc_atmsar_messages_t msgCode;
    union {
        ix_cc_atmsar_port_stats_t* pPortStats;
    } args;
}_ix_cc_atmsar_async_ctx_t;


/**
 * Variable declarations global to this file only. Externs are followed by 
 * static variables.
 */


/** 
 * Extern function prototypes.
 */


/** 
 * Static function prototypes.
 */


/** 
 * Function definitions.
 */


/**
 * NAME: _ix_cc_atmsar_put_async_ctx
 *
 * DESCRIPTION:
 *    The function free context after usage by asynchronous helper functions.
 *    The function services a pool of contextes.
 * 
 * @Param:  IN: arg_pContext - context for freeing
 * 
 * @Return: None
 */
static void
_ix_cc_atmsar_put_async_ctx(
_ix_cc_atmsar_async_ctx_t* arg_pContext)
{
    ix_ossl_free(arg_pContext);
} /* end _ix_cc_atmsar_put_async_ctx */

/**
 * NAME: _ix_cc_atmsar_get_async_ctx
 *
 * DESCRIPTION:
 *    The function allocates context for asynchronous helper function.
 *    The function services a pool of contextes.
 * 
 * @Param:  none
 * 
 * @Return: pointer to context for asychronous SAR CTRL operation.
 *          NULL if no context is available.
 */
static _ix_cc_atmsar_async_ctx_t*
_ix_cc_atmsar_get_async_ctx(void)
{
    return (_ix_cc_atmsar_async_ctx_t*)ix_ossl_malloc(sizeof(_ix_cc_atmsar_async_ctx_t));
} /* end _ix_cc_atmsar_get_async_ctx */


/**
 * NAME: _ix_cc_atmsar_async_cb
 *
 * DESCRIPTION:
 *    This general callback function for helper functions of ATM SAR Module.
 * 
 * @Param:  IN  arg_Result is the ix_error from the called function.
 * @Param:  IN  arg_pContext is the user specified context
 * @Param:  IN  arg_Msg is the message.
 * @Param:  IN  arg_Msg_Len is the length of the message
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent
 *    IX_ERROR
 */
static ix_error
_ix_cc_atmsar_async_cb(
ix_error arg_Result,
void *arg_pContext,
void *arg_pMsg,
ix_uint32 arg_Msg_Len)
{
    _ix_cc_atmsar_async_ctx_t *pContext=arg_pContext;

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
#define CHECK_MESSAGE_SIZE(__msgType) \
    if (arg_Result != IX_SUCCESS)\
    {\
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,\
                                ("arg_Result != IX_SUCCESS"));\
    }\
    if (arg_Msg_Len != sizeof(__msgType) )\
    {\
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,\
                                ("arg_Msg_Len == sizeof(__msgType)"));\
    }    
#else
#define CHECK_MESSAGE_SIZE(__msgType) \
    assert((arg_Result != IX_SUCCESS) || (arg_Msg_Len == sizeof(__msgType)));
#endif
   
    switch(pContext->msgCode)
    {
    case _IX_CC_ATMSAR_MSG_VC_CREATE:
        CHECK_MESSAGE_SIZE(_ix_cc_atmsar_async_vc_create_reply_t);
        pContext->callback.vcCreate(arg_Result,
                                    ((_ix_cc_atmsar_async_vc_create_reply_t *)arg_pMsg)->handle,
                                    pContext->pUserContext);
        break;

    case _IX_CC_ATMSAR_MSG_VC_UPDATE:
        CHECK_MESSAGE_SIZE(_ix_cc_atmsar_async_vc_update_reply_t);
        pContext->callback.vcUpdate(arg_Result,
                                    ((_ix_cc_atmsar_async_vc_update_reply_t *)arg_pMsg)->handle,
                                    pContext->pUserContext);
        break;
        
    case _IX_CC_ATMSAR_MSG_VC_REMOVE:
        CHECK_MESSAGE_SIZE(_ix_cc_atmsar_async_vc_remove_reply_t);
        pContext->callback.vcRemove(arg_Result,
                                    pContext->pUserContext);
        break;

    case _IX_CC_ATMSAR_MSG_GET_VC_STATS:
        CHECK_MESSAGE_SIZE(_ix_cc_atmsar_async_vc_stats_reply_t);
        pContext->callback.vcStats(arg_Result,
                                   &((_ix_cc_atmsar_async_vc_stats_reply_t *)arg_pMsg)->stats,
                                   pContext->pUserContext);
        break;
        
    case _IX_CC_ATMSAR_MSG_PORT_CREATE:
        CHECK_MESSAGE_SIZE(_ix_cc_atmsar_async_port_create_reply_t);
        pContext->callback.portCreate(arg_Result,
                                    ((_ix_cc_atmsar_async_port_create_reply_t *)arg_pMsg)->handle,
                                    pContext->pUserContext);
        break;

    case _IX_CC_ATMSAR_MSG_PORT_REMOVE:
        CHECK_MESSAGE_SIZE(_ix_cc_atmsar_async_port_remove_reply_t);
        pContext->callback.portRemove(arg_Result,
                                    pContext->pUserContext);
        break;

    case _IX_CC_ATMSAR_MSG_GET_PORT_STATS:
        CHECK_MESSAGE_SIZE(_ix_cc_atmsar_async_port_stats_reply_t);
        pContext->callback.portStats( arg_Result,
                                    &((_ix_cc_atmsar_async_port_stats_reply_t *)arg_pMsg)->stats,
                                    pContext->pUserContext);
        break;
        
    case _IX_CC_ATMSAR_MSG_CID_CREATE:
        CHECK_MESSAGE_SIZE(_ix_cc_atmsar_async_cid_create_reply_t);
        pContext->callback.cidCreate(arg_Result,
                                     ((_ix_cc_atmsar_async_cid_create_reply_t *)arg_pMsg)->handle,
                                     pContext->pUserContext);
        break;

    case _IX_CC_ATMSAR_MSG_CID_UPDATE:
        CHECK_MESSAGE_SIZE(_ix_cc_atmsar_async_cid_update_reply_t);
        pContext->callback.cidUpdate(arg_Result,
                                     ((_ix_cc_atmsar_async_cid_update_reply_t *)arg_pMsg)->handle,
                                     pContext->pUserContext);
        break;
        
    case _IX_CC_ATMSAR_MSG_CID_REMOVE:
        CHECK_MESSAGE_SIZE(_ix_cc_atmsar_async_cid_remove_reply_t);
        pContext->callback.cidRemove(arg_Result,
                                    pContext->pUserContext);
        break;

    case _IX_CC_ATMSAR_MSG_GET_CID_STATS:
        CHECK_MESSAGE_SIZE(_ix_cc_atmsar_async_cid_stats_reply_t);
        pContext->callback.cidStats( arg_Result,
                                    &((_ix_cc_atmsar_async_cid_stats_reply_t *)arg_pMsg)->stats,
                                    pContext->pUserContext);
        break;
        
    default:
        _ix_cc_atmsar_put_async_ctx(pContext);
        return IX_ERROR_WARNING(IX_CC_ERROR_INTERNAL,
                                ("Invalid pointer: arg_pCallback"));
    }
    _ix_cc_atmsar_put_async_ctx(pContext);
    return IX_SUCCESS;
    
#undef CHECK_MESSAGE_SIZE
} /* end _ix_cc_atmsar_async_cb */


/**
 * NAME: _IX_CC_ATMSAR_SEND_ASYNC
 *
 * DESCRIPTION:
 *    The macro is useful. It defince common solution for sending all asynchronous
 *    messages to SAR CTRL.
 *    NOTE#1: that it requires that few variables are accessible in the macro
 *    calling place: There are 'arg_Callback', 'arg_pUserContext'.
 *    NOTE#2: The macro has to be called as last operation in function, because
 *    it  exits from function, after its execution
 *
 * @Param:  IN  __msgCode : code of sending message
 * @Param:  IN  __dataBuf : buffer with data for receing function
 * @Param:  IN  __sizeBuf : size of data buf
 */
#ifdef CCI_MULTIINSTANCE
#define _IX_CC_ATMSAR_SEND_ASYNC(__instance,                                    \
        __msgCode, __cb, __pContext, __dataBuf, __sizeBuf, __err, __err_label)  \
{                                                                               \
    if(arg_Callback == NULL)                                                    \
    {                                                                           \
        __err = IX_ERROR_WARNING(IX_CC_ERROR_NULL,                              \
                                ("Invalid pointer: arg_pCallback"));            \
        goto __atmsar_async_send_end;                                           \
    }/* end if(arg_Callback == NULL) */                                         \
    if((__pContext=_ix_cc_atmsar_get_async_ctx()) == NULL)                      \
    {                                                                           \
        __err = IX_ERROR_WARNING(IX_CC_ERROR_NULL,                              \
                                ("No available context for async. oper"));      \
        goto __atmsar_async_send_end;                                           \
    }                                                                           \
    __pContext->pUserContext=arg_pUserContext;                                  \
    __pContext->callback.__cb=arg_Callback;                                     \
    __pContext->msgCode=__msgCode;                                              \
    IX_ERROR_CGT(ix_cc_msup_send_async_msg(                                     \
      (!__instance)?IX_CC_ATMSAR_MAIN_0_MSG_INPUT:IX_CC_ATMSAR_MAIN_1_MSG_INPUT,\
                                           _ix_cc_atmsar_async_cb,              \
                                           (void*)__pContext,                   \
                                           __pContext->msgCode,                 \
                                           (void *)(__dataBuf),                 \
                                           __sizeBuf),                          \
                 __err,                                                         \
                 IX_CC_ERROR_SEND_FAIL,                                         \
                 IX_ERROR_LEVEL_WARNING,                                        \
                 __atmsar_async_send_cleanup);                                  \
    goto __atmsar_async_send_end;                                               \
  __atmsar_async_send_cleanup:                                                  \
    _ix_cc_atmsar_put_async_ctx(__pContext);                                    \
    __pContext=NULL;                                                            \
    goto __err_label;                                                           \
  __atmsar_async_send_end:                                                      \
    pContext = pContext;                                                        \
}
#else /* no multiinstance SAR */
#define _IX_CC_ATMSAR_SEND_ASYNC(                                               \
        __msgCode, __cb, __pContext, __dataBuf, __sizeBuf, __err, __err_label)  \
{                                                                               \
    if(arg_Callback == NULL)                                                    \
    {                                                                           \
        __err = IX_ERROR_WARNING(IX_CC_ERROR_NULL,                              \
                                ("Invalid pointer: arg_pCallback"));            \
        goto __atmsar_async_send_end;                                           \
    }/* end if(arg_Callback == NULL) */                                         \
    if((__pContext=_ix_cc_atmsar_get_async_ctx()) == NULL)                      \
    {                                                                           \
        __err = IX_ERROR_WARNING(IX_CC_ERROR_NULL,                              \
                                ("No available context for async. oper"));      \
        goto __atmsar_async_send_end;                                           \
    }                                                                           \
    __pContext->pUserContext=arg_pUserContext;                                  \
    __pContext->callback.__cb=arg_Callback;                                     \
    __pContext->msgCode=__msgCode;                                              \
    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_ATMSAR_MAIN_MSG_INPUT,         \
                                           _ix_cc_atmsar_async_cb,              \
                                           (void*)__pContext,                   \
                                           __pContext->msgCode,                 \
                                           (void *)(__dataBuf),                 \
                                           __sizeBuf),                          \
                 __err,                                                         \
                 IX_CC_ERROR_SEND_FAIL,                                         \
                 IX_ERROR_LEVEL_WARNING,                                        \
                 __atmsar_async_send_cleanup);                                  \
    goto __atmsar_async_send_end;                                               \
  __atmsar_async_send_cleanup:                                                  \
    _ix_cc_atmsar_put_async_ctx(__pContext);                                    \
    __pContext=NULL;                                                            \
    goto __err_label;                                                           \
  __atmsar_async_send_end:                                                      \
    pContext = pContext;                                                        \
}
#endif /* CCI_MULTIINSTANCE */

/**
 * NAME: ix_cc_atmsar_async_vc_create
 *
 * DESCRIPTION:
 *    This helper function builds and sends an VC create message to the
 *    CC.
 * 
 * @Param:  IN  arg_pVcDescr : parameters describing requested VC.
 * @Param:  IN  arg_Callback : a client provided callback function 
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ERROR_SEND_FAIL - failure on sending a message
 *    IX_CC_ATMSAR_ERROR_BUSY - the SAR CRTL does not have resources for the
 *        operation performing.
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_NO_RESOURCES - no resources to add the entry 
 */
ix_error
ix_cc_atmsar_async_vc_create(
const ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmsar_vc_descr_t* arg_pVcDescr,
ix_cc_atmsar_cb_vc_create_t arg_Callback,
void *arg_pUserContext)
{
    _ix_cc_atmsar_async_vc_create_request_t req;
    ix_error err = IX_SUCCESS;
    _ix_cc_atmsar_async_ctx_t *pContext;

    ix_ossl_memcpy(&(req.descr), arg_pVcDescr, sizeof(req.descr));
    req.sarInstance=arg_sarInstance;

#ifdef CCI_MULTIINSTANCE
    _IX_CC_ATMSAR_SEND_ASYNC(
    arg_sarInstance,
    _IX_CC_ATMSAR_MSG_VC_CREATE, vcCreate, pContext,
    &req, sizeof(req), err, error_label);
#else
    _IX_CC_ATMSAR_SEND_ASYNC(
    _IX_CC_ATMSAR_MSG_VC_CREATE, vcCreate, pContext,
    &req, sizeof(req), err, error_label);
#endif /* CCI_MULTIINSTANCE */

    pContext = pContext;
    
  error_label:
    return err;
} /* end ix_cc_atmsar_async_vc_create */


/**
 * NAME: ix_cc_atmsar_async_vc_update
 *
 * DESCRIPTION:
 *    This helper function builds and sends an CID update message to the
 *    CC.
 * 
 * @Param:  IN  arg_pVcDescr : parameters describing requested VC.
 * @Param:  IN  arg_hVc      : handle to the VC
 * @Param:  IN  arg_Callback : a client provided callback function 
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ERROR_SEND_FAIL - failure on sending a message
 *    IX_CC_ATMSAR_ERROR_BUSY - the ATM SAR does not have resources for the
 *        operation performing.
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_NO_RESOURCES - no resources to add the entry 
 */
ix_error
ix_cc_atmsar_async_vc_update(
const ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmsar_vc_handle_t arg_hVc,
const ix_cc_atmsar_update_data_t* arg_pUpdateData,
ix_cc_atmsar_cb_vc_update_t arg_Callback,
void *arg_pUserContext)
{
    _ix_cc_atmsar_async_vc_update_request_t req;
    ix_error err = IX_SUCCESS;
    _ix_cc_atmsar_async_ctx_t *pContext;

    ix_ossl_memcpy(&(req.updateData), arg_pUpdateData, sizeof(req.updateData));
    req.sarInstance=arg_sarInstance;
    req.handle=arg_hVc;

#ifdef CCI_MULTIINSTANCE
    _IX_CC_ATMSAR_SEND_ASYNC(
    arg_sarInstance,
    _IX_CC_ATMSAR_MSG_VC_UPDATE, vcUpdate, pContext,
    &req, sizeof(req), err, error_label);
#else
    _IX_CC_ATMSAR_SEND_ASYNC(
    _IX_CC_ATMSAR_MSG_VC_UPDATE, vcUpdate, pContext,
    &req, sizeof(req), err, error_label);
#endif /* CCI_MULTIINSTANCE */

    pContext = pContext;
    
  error_label:
    return err;
} /* end ix_cc_atmsar_async_vc_update */

/**
 * NAME: ix_cc_atmsar_async_vc_remove
 *
 * DESCRIPTION:
 *    This helper function builds and sends an VC remove message to
 *    the CC. 
 * 
 * @Param:  IN  arg_hVc : handle to VC to be removed
 * @Param:  IN  arg_Callback : a client provided callback function 
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM- invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error
ix_cc_atmsar_async_vc_remove(
const ix_cc_atmsar_instance_t arg_sarInstance,
ix_cc_atmsar_vc_handle_t arg_hVc,
ix_cc_atmsar_cb_vc_remove_t arg_Callback,
void *arg_pUserContext)
{
    _ix_cc_atmsar_async_vc_remove_request_t req;
    ix_error err = IX_SUCCESS;
    _ix_cc_atmsar_async_ctx_t *pContext;

    req.handle=arg_hVc;
    req.sarInstance=arg_sarInstance;

#ifdef CCI_MULTIINSTANCE
    _IX_CC_ATMSAR_SEND_ASYNC(
    arg_sarInstance,
    _IX_CC_ATMSAR_MSG_VC_REMOVE, vcRemove, pContext,
    &req, sizeof(req), err, error_label);
#else
    _IX_CC_ATMSAR_SEND_ASYNC(
    _IX_CC_ATMSAR_MSG_VC_REMOVE, vcRemove, pContext,
    &req, sizeof(req), err, error_label);
#endif /* CCI_MULTIINSTANCE */

    pContext = pContext;
    
  error_label:
    return err;
} /* end ix_cc_atmsar_async_vc_remove */


/**
 * NAME: ix_cc_atmsar_async_get_vc_stats
 *
 * DESCRIPTION:
 *    This helper function builds and sends an VC get stats message to
 *    the CC. 
 * 
 * @Param:  IN  arg_hVc : handle to VC to be removed
 * @Param:  IN  arg_Callback : a client provided callback function
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * @Param:  OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error
ix_cc_atmsar_async_get_vc_stats(
const ix_cc_atmsar_instance_t arg_sarInstance,
ix_cc_atmsar_vc_handle_t arg_hVc,
ix_cc_atmsar_cb_get_vc_stats_t arg_Callback,
void *arg_pUserContext)
{
    ix_error err = IX_SUCCESS;
    _ix_cc_atmsar_async_vc_stats_request_t req;
    _ix_cc_atmsar_async_ctx_t *pContext;

    req.handle=arg_hVc;
    req.sarInstance=arg_sarInstance;

#ifdef CCI_MULTIINSTANCE
    _IX_CC_ATMSAR_SEND_ASYNC(
    arg_sarInstance,
    _IX_CC_ATMSAR_MSG_GET_VC_STATS, vcStats, pContext,
    &req, sizeof(req), err, error_label);
#else
    _IX_CC_ATMSAR_SEND_ASYNC(
    _IX_CC_ATMSAR_MSG_GET_VC_STATS, vcStats, pContext,
    &req, sizeof(req), err, error_label);
#endif /* CCI_MULTIINSTANCE */

    pContext = pContext;
    
  error_label:
    return err;
} /* end ix_cc_atmsar_async_get_vc_stats */


/**
 * NAME: ix_cc_atmsar_async_port_create
 *
 * DESCRIPTION:
 *    This helper function builds and sends an ATM port create message to the
 *    CC.
 * 
 * @Param:  IN  arg_pPortDescr : parameters describing requested port.
 * @Param:  IN  arg_Callback : a client provided callback function 
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_NO_RESOURCES - no resources to add the entry 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error
ix_cc_atmsar_async_port_create(
const ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmsar_port_descr_t* arg_pPortDescr,
ix_cc_atmsar_cb_port_create_t arg_Callback,
void *arg_pUserContext)
{
    _ix_cc_atmsar_async_port_create_request_t req;
    ix_error err = IX_SUCCESS;
    _ix_cc_atmsar_async_ctx_t *pContext;

    ix_ossl_memcpy(&(req.descr), arg_pPortDescr, sizeof(req.descr));
    req.sarInstance=arg_sarInstance;

#ifdef CCI_MULTIINSTANCE
    _IX_CC_ATMSAR_SEND_ASYNC(
    arg_sarInstance,
    _IX_CC_ATMSAR_MSG_PORT_CREATE, portCreate, pContext,
    &req, sizeof(req), err, error_label);
#else
    _IX_CC_ATMSAR_SEND_ASYNC(
    _IX_CC_ATMSAR_MSG_PORT_CREATE, portCreate, pContext,
    &req, sizeof(req), err, error_label);
#endif /* CCI_MULTIINSTANCE */

    pContext = pContext;
    
  error_label:
    return err;
} /* end ix_cc_atmsar_async_port_create */


/**
 * NAME: ix_cc_atmsar_async_port_remove
 *
 * DESCRIPTION:
 *    This helper function builds and sends an ATM port remove message to
 *    the CC. 
 * 
 * @Param:  IN  arg_port : ATM port to be removed
 * @Param:  IN  arg_Callback : a client provided callback function 
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * @Param:  OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM- invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error
ix_cc_atmsar_async_port_remove(
const ix_cc_atmsar_instance_t arg_sarInstance,
ix_cc_atmsar_port_handle_t arg_port,
ix_cc_atmsar_cb_port_remove_t arg_Callback,
void *arg_pUserContext)
{
    _ix_cc_atmsar_async_port_remove_request_t req;
    ix_error err = IX_SUCCESS;
    _ix_cc_atmsar_async_ctx_t *pContext;

    req.handle=arg_port;
    req.sarInstance=arg_sarInstance;

#ifdef CCI_MULTIINSTANCE
    _IX_CC_ATMSAR_SEND_ASYNC(
    arg_sarInstance,
    _IX_CC_ATMSAR_MSG_PORT_REMOVE, portRemove, pContext,
    &req, sizeof(req), err, error_label);
#else
    _IX_CC_ATMSAR_SEND_ASYNC(
    _IX_CC_ATMSAR_MSG_PORT_REMOVE, portRemove, pContext,
    &req, sizeof(req), err, error_label);
#endif /* CCI_MULTIINSTANCE */

    pContext = pContext;
    
  error_label:
    return err;
} /* end ix_cc_atmsar_async_port_remove */


/**
 * NAME: ix_cc_atmsar_async_get_port_stats
 *
 * DESCRIPTION:
 *    This helper function builds and sends an ATM port get stats message to
 *    the CC. 
 * 
 * @Param:  IN  arg_port : port ID
 * @Param:  IN  arg_pLookupResult : pointer to ix_cc_atmsar_port_stats
 *                  structure where the requested data will be stored
 * @Param:  IN  arg_Callback : a client provided callback function
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * @Param:  OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error
ix_cc_atmsar_async_get_port_stats(
const ix_cc_atmsar_instance_t arg_sarInstance,
ix_cc_atmsar_port_handle_t arg_port,
ix_cc_atmsar_cb_get_port_stats_t arg_Callback,
void *arg_pUserContext)
{
    _ix_cc_atmsar_async_port_stats_request_t req;
    ix_error err = IX_SUCCESS;
    _ix_cc_atmsar_async_ctx_t *pContext;

    req.handle=arg_port;
    req.sarInstance=arg_sarInstance;

#ifdef CCI_MULTIINSTANCE
    _IX_CC_ATMSAR_SEND_ASYNC(
    arg_sarInstance,
    _IX_CC_ATMSAR_MSG_GET_PORT_STATS, portStats, pContext,
    &req, sizeof(req), err, error_label);
#else
    _IX_CC_ATMSAR_SEND_ASYNC(
    _IX_CC_ATMSAR_MSG_GET_PORT_STATS, portStats, pContext,
    &req, sizeof(req), err, error_label);
#endif /* CCI_MULTIINSTANCE */

    pContext = pContext;
    
  error_label:
    return err;
} /* end ix_cc_atmsar_async_get_port_stats */


/**
 * NAME: ix_cc_atmsar_async_cid_create
 *
 * DESCRIPTION:
 *    This helper function builds and sends an CID create message to the
 *    CC.
 * 
 * @Param:  IN  arg_pCidDescr : parameters describing requested CID.
 * @Param:  IN  arg_Callback : a client provided callback function 
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_NO_RESOURCES - no resources to add the entry 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error
ix_cc_atmsar_async_cid_create(
const ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmsar_cid_descr_t* arg_pCidDescr,
ix_cc_atmsar_cb_cid_create_t arg_Callback,
void *arg_pUserContext)
{
    _ix_cc_atmsar_async_cid_create_request_t req;
    ix_error err = IX_SUCCESS;
    _ix_cc_atmsar_async_ctx_t *pContext;

    ix_ossl_memcpy(&(req.descr), arg_pCidDescr, sizeof(req.descr));
    req.sarInstance=arg_sarInstance;

#ifdef CCI_MULTIINSTANCE
    _IX_CC_ATMSAR_SEND_ASYNC(
    arg_sarInstance,
    _IX_CC_ATMSAR_MSG_CID_CREATE, cidCreate, pContext,
    &req, sizeof(req), err, error_label);
#else
    _IX_CC_ATMSAR_SEND_ASYNC(
    _IX_CC_ATMSAR_MSG_CID_CREATE, cidCreate, pContext,
    &req, sizeof(req), err, error_label);
#endif /* CCI_MULTIINSTANCE */

    pContext = pContext;
    
  error_label:
    return err;
} /* end ix_cc_atmsar_async_cid_create */


/**
 * NAME: ix_cc_atmsar_async_cid_update
 *
 * DESCRIPTION:
 *    This helper function builds and sends an CID udpate message to the
 *    CC.
 * 
 * @Param:  IN  arg_pUpdate : parameters describing requested CID.
 * @Param:  IN  arg_Callback : a client provided callback function 
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_NO_RESOURCES - no resources to add the entry 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error
ix_cc_atmsar_async_cid_update(
const ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmsar_cid_handle_t arg_hCid,
const ix_cc_atmsar_update_data_t* arg_pUpdate,
ix_cc_atmsar_cb_cid_update_t arg_Callback,
void *arg_pUserContext)
{
    _ix_cc_atmsar_async_cid_update_request_t req;
    ix_error err = IX_SUCCESS;
    _ix_cc_atmsar_async_ctx_t *pContext;

    ix_ossl_memcpy(&(req.updateData), arg_pUpdate, sizeof(req.updateData));
    req.sarInstance=arg_sarInstance;
    req.handle=arg_hCid;

#ifdef CCI_MULTIINSTANCE
    _IX_CC_ATMSAR_SEND_ASYNC(
    arg_sarInstance,
    _IX_CC_ATMSAR_MSG_CID_UPDATE, cidUpdate, pContext,
    &req, sizeof(req), err, error_label);
#else
    _IX_CC_ATMSAR_SEND_ASYNC(
    _IX_CC_ATMSAR_MSG_CID_UPDATE, cidUpdate, pContext,
    &req, sizeof(req), err, error_label);
#endif /* CCI_MULTIINSTANCE */

    pContext = pContext;
    
  error_label:
    return err;
} /* end ix_cc_atmsar_async_cid_update */


/**
 * NAME: ix_cc_atmsar_async_cid_remove
 *
 * DESCRIPTION:
 *    This helper function builds and sends an CID remove message to
 *    the CC. 
 * 
 * @Param:  IN  arg_cid : CID handle to be removed
 * @Param:  IN  arg_Callback : a client provided callback function 
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * @Param:  OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM- invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error
ix_cc_atmsar_async_cid_remove(
const ix_cc_atmsar_instance_t arg_sarInstance,
ix_cc_atmsar_cid_handle_t arg_cid,
ix_cc_atmsar_cb_cid_remove_t arg_Callback,
void *arg_pUserContext)
{
    _ix_cc_atmsar_async_port_remove_request_t req;
    ix_error err = IX_SUCCESS;
    _ix_cc_atmsar_async_ctx_t *pContext;

    req.handle=arg_cid;
    req.sarInstance=arg_sarInstance;

#ifdef CCI_MULTIINSTANCE
    _IX_CC_ATMSAR_SEND_ASYNC(
    arg_sarInstance,
    _IX_CC_ATMSAR_MSG_CID_REMOVE, cidRemove, pContext,
    &req, sizeof(req), err, error_label);
#else
    _IX_CC_ATMSAR_SEND_ASYNC(
    _IX_CC_ATMSAR_MSG_CID_REMOVE, cidRemove, pContext,
    &req, sizeof(req), err, error_label);
#endif /* CCI_MULTIINSTANCE */

    pContext = pContext;
    
  error_label:
    return err;
} /* end ix_cc_atmsar_async_cid_remove */


/**
 * NAME: ix_cc_atmsar_async_get_cid_stats
 *
 * DESCRIPTION:
 *    This helper function builds and sends an CID get stats message to
 *    the CC. 
 * 
 * @Param:  IN  arg_cid : CID handle
 * @Param:  IN  arg_Callback : a client provided callback function
 * @Param:  IN  arg_pUserContext : pointer to client defined context
 * @Param:  OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - message was successfully built and sent 
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid pointer in input parameters
 *    IX_CC_ERROR_MSG_LIBRARY - failure on sending a message 
 *    IX_CC_ERROR_UNINIT - This CC has not been initialized
 */
ix_error
ix_cc_atmsar_async_get_cid_stats(
const ix_cc_atmsar_instance_t arg_sarInstance,
ix_cc_atmsar_cid_handle_t arg_cid,
ix_cc_atmsar_cb_get_cid_stats_t arg_Callback,
void *arg_pUserContext)
{
    _ix_cc_atmsar_async_cid_stats_request_t req;
    ix_error err = IX_SUCCESS;
    _ix_cc_atmsar_async_ctx_t *pContext;

    req.handle=arg_cid;
    req.sarInstance=arg_sarInstance;

#ifdef CCI_MULTIINSTANCE
    _IX_CC_ATMSAR_SEND_ASYNC(
    arg_sarInstance,
    _IX_CC_ATMSAR_MSG_GET_CID_STATS, cidStats, pContext,
    &req, sizeof(req), err, error_label);
#else
    _IX_CC_ATMSAR_SEND_ASYNC(
    _IX_CC_ATMSAR_MSG_GET_CID_STATS, cidStats, pContext,
    &req, sizeof(req), err, error_label);
#endif /* CCI_MULTIINSTANCE */

    pContext = pContext;
    
  error_label:
    return err;
} /* end ix_cc_atmsar_async_get_cid_stats */


