/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      ATM SAR Core Component
 *
 * = FILENAME
 *      ix_cc_atmsar_plugin.c
 *
 * = DESCRIPTION
 *      TBD
 *
 * = AUTHOR
 *       Jaroslaw Kogut
 *       jaroslaw.kogut@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */
#define IX_ERROR_FILE_IDENT "$Id: ix_cc_atmsar_plugin.c,v 1.2 2003/09/10 07:37:11 rwielick Exp $";



/**
 * System defined include files
 */
#include "ix_cc_error.h"
#include "ix_rm.h"
#include "ix_ossl.h"
#include "ix_cc.h"
#include "ix_cc_macros.h"
#include "ix_cc_properties.h"
#include "ix_config.h"
#include "bindings.h"
#include "cc/ix_cc_msup.h"

/**
 * User defined include files
 */
#include "cc/ix_cc_atmsar.h"

#ifdef ATMSAR_MAIN
#include "cc/internal/ix_cc_atmsar_main.h"
#endif /* ATMSAR_MAIN */

#ifdef ATMSAR_AGENT
#include "cc/internal/ix_cc_atmsar_agent.h"
#endif /* ATMSAR_AGENT */


/**
 * Preprocessor symbols and Macros used in this file
 */

#define _IX_CC_ATMSAR_PLUGIN_UNKNOW_IDX (-1)

/**
 * Type definitions whose scope is limited to this file
 */

/**
 * Variable declarations global to this file only. Externs are followed by 
 * static variables.
 */


/**
 * VARNAME: g_atmsarDebugMask
 *
 * DESCRIPTION:
 *    The variable contains bits mask for debug and traces logs
 */
ix_uint32 g_atmsarDebugMask=0x0FFFFFFFF;


/** 
 * Extern function prototypes.
 */


/** 
 * Static function prototypes.
 */


/** 
 * Function definitions.
 */


/**
 * NAME: _ix_cc_atmsar_plugin_find_engine
 *
 * DESCRIPTION:
 *    The function find engine for requested instance. The functions verify
 *    current thread ID with thread ID in the engines AGENT and Main. Base
 *    assumption is that Agent and Main works in different threads.
 *    If thread Id is not match or any of the structures are NULL for the
 *    instance function returns NULL>
 *
 * @Param:  IN  arg_instance : Id of core component instance.
 * 
 * @Return: Pointer to structure describing engine configuration data
 */
static _ix_cc_atmsar_engine_t*
_ix_cc_atmsar_plugin_find_engine(
ix_cc_atmsar_instance_t arg_instance)
{
    ix_ossl_thread_t threadId;
#ifdef ATMSAR_MAIN
    _ix_cc_atmsar_main_context_t* pMainCtx;
#endif /* ATMSAR_MAIN */  
#ifdef ATMSAR_AGENT
    _ix_cc_atmsar_agent_context_t* pAgentCtx;
#endif /* ATMSAR_AGENT */  

    if (arg_instance >= _IX_CC_ATMSAR_INSTANCE_MAX)
    {
        return NULL;
    }

    if (ix_ossl_thread_get_id(&threadId) != IX_SUCCESS)
    {
        return NULL;
    }

#ifdef ATMSAR_MAIN
    pMainCtx=_ix_cc_atmsar_main_get_ctx(arg_instance);
    if ((pMainCtx != NULL) && (pMainCtx->engine.threadId == threadId))
    {
        return &(pMainCtx->engine);
    }
#endif /* ATMSAR_MAIN */  
#ifdef ATMSAR_AGENT
    pAgentCtx=_ix_cc_atmsar_agent_get_ctx(arg_instance);
    if ((pAgentCtx != NULL) && (pAgentCtx->engine.threadId == threadId))
    {
        return &(pAgentCtx->engine);
    }
#endif /* ATMSAR_AGENT */
    return NULL;
} /* end _ix_cc_atmsar_plugin_find_core_in_engine */


/**
 * NAME: _ix_cc_atmsar_plugin_find_core_in_engine
 *
 * DESCRIPTION:
 *    The function registers ATM SAR plug-in into ATM SAR core
 *    component. The input parameters - callback routines may be NULL,
 *    indicating that the plug-in is not interested in that operation servicing.
 *
 * @Param:  IN  arg_instance : Id of core component instance.
 * @Param:  IN  arg_coreComponentId : Id of core component that register
 *                  service.
 * @Param:  OUT arg_pEntryId : pointer to integer contains entry index for
 *                  requested 'arg_coreComponentId'.
 * 
 * @Return: Pointer to structure describing engine configuration data
 */
static _ix_cc_atmsar_engine_t*
_ix_cc_atmsar_plugin_find_core_in_engine(
ix_cc_atmsar_instance_t arg_instance,
ix_uint16 arg_coreComponentId,
ix_uint16 *arg_pEntryId)
{
    _ix_cc_atmsar_engine_t* pEngine=NULL;
    _ix_cc_atmsar_plugin_entry_t *pTab;
    ix_uint16 i;

    pEngine = _ix_cc_atmsar_plugin_find_engine(arg_instance);

    *arg_pEntryId = _IX_CC_ATMSAR_PLUGIN_UNKNOW_IDX;
    if (pEngine != NULL)
    {
        pTab=pEngine->pluginTab;
        for (i=0; i < pEngine->ccNumber; i++)
        {
            if (pTab[i].plugInId==arg_coreComponentId)
            {
                *arg_pEntryId=i;
                break;
            }
        }
    }
    return pEngine;
} /* end _ix_cc_atmsar_plugin_find_core_in_engine */


/**
 * NAME: ix_cc_atmsar_plugin_reg_vc_service
 *
 * DESCRIPTION:
 *    The function registers ATM SAR plug-in into ATM SAR core
 *    component. The input parameters - callback routines may be NULL,
 *    indicating that the plug-in is not interested in that operation servicing.
 *
 * @Param:  IN  arg_coreComponentId : Id of core component that register
 *                  service.
 * @Param:  IN  arg_sarInstance : Id of SAR Instance for that the action is
 *                  performed.
 * @Param:  IN  arg_pCbVcCreate : pointer to callback routine servicing
 *                  "VC CREATE" operation internal in plug-in.
 * @Param:  IN  arg_pCbVcRemove : pointer to callback routine servicing
 *                  "VC REMOVE" operation internal in plug-in.
 * @Param:  IN  arg_pCbVcStats : pointer to callback routine servicing
 *                  "GET VC STATS" operation internal in plug-in.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - registering successful
 *    IX_CC_ERROR_ENTRY_NOT_FOUND - The ATM SAR is not ready for
 *         registering the core component for the specific ID.
 *    IX_CC_ERROR_DUPLICATE_ENTRY -- The ATM SAR already registered core
 *         component for the specific ID.
 *    IX_CC_ERROR_INTERNAL - The plug-in that trries to register works in the
 *          wrong thread. Check description of ATM SAR configuration tables
 *          and XML config description.
 */
ix_error
ix_cc_atmsar_plugin_reg_vc_service(
ix_uint16 arg_coreComponentId,
ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmsar_plugin_vc_create_t arg_pCbVcCreate,
const ix_cc_atmsar_plugin_vc_update_t arg_pCbVcUpdate,
const ix_cc_atmsar_plugin_vc_remove_t arg_pCbVcRemove,
const ix_cc_atmsar_plugin_get_vc_stats_t arg_pCbVcStats)
{
    ix_uint16 i;
    _ix_cc_atmsar_engine_t* pEngine;
    
    _IX_CC_ATMSAR_DBG_INIT(("ix_cc_atmsar_plugin_register\n"));

    pEngine=_ix_cc_atmsar_plugin_find_core_in_engine(arg_sarInstance, arg_coreComponentId, &i);
    if (pEngine == NULL)
    {   /* ERROR - no entry found */
        return IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND,
                                ("The module is not defined as ATM SAR Plugin"));
    }

    if ((pEngine->pluginTab[i].initFlag & _IX_CC_ATMSAR_PLUGIN_INITFLAG_VC) != 0)
    {   /* ERROR - plug-in already registered */
        return IX_ERROR_WARNING(IX_CC_ERROR_DUPLICATE_ENTRY,
                                ("For the MODULE ID plug-in service for VC already registered in ATM SAR"));
    }
    pEngine->pluginTab[i].initFlag|=_IX_CC_ATMSAR_PLUGIN_INITFLAG_VC;
    pEngine->pluginTab[i].pCbVcCreate=arg_pCbVcCreate;
    pEngine->pluginTab[i].pCbVcUpdate=arg_pCbVcUpdate;
    pEngine->pluginTab[i].pCbVcRemove=arg_pCbVcRemove;
    pEngine->pluginTab[i].pCbVcStats=arg_pCbVcStats;
   
    return IX_SUCCESS;
} /* end ix_cc_atmsar_plugin_reg_vc_service */


/**
 * NAME: ix_cc_atmsar_plugin_reg_port_service
 *
 * DESCRIPTION:
 *    The function registers ATM SAR plug-in into ATM SAR core
 *    component. The input parameters - callback routines may be NULL,
 *    indicating that the plug-in is not interested in that operation servicing.
 *
 * @Param:  IN  arg_coreComponentId : Id of core component that register
 *                  service.
 * @Param:  IN  arg_sarInstance : Id of SAR Instance for that the action is
 *                  performed.
 * @Param:  IN  arg_pCbPortCreate : pointer to callback routine servicing
 *                  "PORT CREATE" operation internal in plug-in.
 * @Param:  IN  arg_pCbPortRemove : pointer to callback routine servicing
 *                  "PORT REMOVE" operation internal in plug-in.
 * @Param:  IN  arg_pCbPortStats : pointer to callback routine servicing
 *                  "GET PORT STATS" operation internal in plug-in.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - registering successful
 *    IX_CC_ERROR_ENTRY_NOT_FOUND - The ATM SAR is not ready for
 *         registering the core component for the specific ID.
 *    IX_CC_ERROR_DUPLICATE_ENTRY -- The ATM SAR already registered core
 *         component for the specific ID.
 *    IX_CC_ERROR_INTERNAL - The plug-in that trries to register works in the
 *          wrong thread. Check description of ATM SAR configuration tables
 *          and XML config description.
 */
ix_error
ix_cc_atmsar_plugin_reg_port_service(
ix_uint16 arg_coreComponentId,
ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmsar_plugin_port_create_t arg_pCbPortCreate,
const ix_cc_atmsar_plugin_port_remove_t arg_pCbPortRemove,
const ix_cc_atmsar_plugin_get_port_stats_t arg_pCbPortStats)
{
    ix_uint16 i;
    _ix_cc_atmsar_engine_t* pEngine;
    
    _IX_CC_ATMSAR_DBG_INIT(("ix_cc_atmsar_plugin_register\n"));

    pEngine=_ix_cc_atmsar_plugin_find_core_in_engine(arg_sarInstance, arg_coreComponentId, &i);
    if (pEngine == NULL)
    {   /* ERROR - no entry found */
        return IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND,
                                ("The module is not defined as ATM SAR Plugin"));
    }

    if ((pEngine->pluginTab[i].initFlag & _IX_CC_ATMSAR_PLUGIN_INITFLAG_PORT) != 0)
    {   /* ERROR - plug-in already registered */
        return IX_ERROR_WARNING(IX_CC_ERROR_DUPLICATE_ENTRY,
                                ("For the MODULE ID plug-in service for PORT already registered in ATM SAR"));
    }
    pEngine->pluginTab[i].initFlag|=_IX_CC_ATMSAR_PLUGIN_INITFLAG_PORT;
    pEngine->pluginTab[i].pCbPortCreate=arg_pCbPortCreate;
    pEngine->pluginTab[i].pCbPortRemove=arg_pCbPortRemove;
    pEngine->pluginTab[i].pCbPortStats=arg_pCbPortStats;
   
    return IX_SUCCESS;
} /* end ix_cc_atmsar_plugin_reg_port_service */


/**
 * NAME: ix_cc_atmsar_plugin_reg_cid_service
 *
 * DESCRIPTION:
 *    The function registers ATM SAR plug-in into ATM SAR core
 *    component. The input parameters - callback routines may be NULL,
 *    indicating that the plug-in is not interested in that operation servicing.
 *
 * @Param:  IN  arg_coreComponentId : Id of core component that register
 *                  service.
 * @Param:  IN  arg_sarInstance : Id of SAR Instance for that the action is
 *                  performed.
 * @Param:  IN  arg_pCbCidCreate : pointer to callback routine servicing
 *                  "CID CREATE" operation internal in plug-in.
 * @Param:  IN  arg_pCbCidCreate : pointer to callback routine servicing
 *                  "CID UPDATE" operation internal in plug-in.
 * @Param:  IN  arg_pCbCidRemove : pointer to callback routine servicing
 *                  "CID REMOVE" operation internal in plug-in.
 * @Param:  IN  arg_pCbPortStats : pointer to callback routine servicing
 *                  "GET CID STATS" operation internal in plug-in.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - registering successful
 *    IX_CC_ERROR_ENTRY_NOT_FOUND - The ATM SAR is not ready for
 *         registering the core component for the specific ID.
 *    IX_CC_ERROR_DUPLICATE_ENTRY -- The ATM SAR already registered core
 *         component for the specific ID.
 *    IX_CC_ERROR_INTERNAL - The plug-in that trries to register works in the
 *          wrong thread. Check description of ATM SAR configuration tables
 *          and XML config description.
 */
ix_error
ix_cc_atmsar_plugin_reg_cid_service(
ix_uint16 arg_coreComponentId,
ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmsar_plugin_cid_create_t arg_pCbCidCreate,
const ix_cc_atmsar_plugin_cid_update_t arg_pCbCidUpdate,
const ix_cc_atmsar_plugin_cid_remove_t arg_pCbCidRemove,
const ix_cc_atmsar_plugin_get_cid_stats_t arg_pCbCidStats)
{
    ix_uint16 i;
    _ix_cc_atmsar_engine_t* pEngine;
    
    _IX_CC_ATMSAR_DBG_INIT(("ix_cc_atmsar_plugin_register\n"));

    pEngine=_ix_cc_atmsar_plugin_find_core_in_engine(arg_sarInstance, arg_coreComponentId, &i);
    if (pEngine == NULL)
    {   /* ERROR - no entry found */
        return IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND,
                                ("The module is not defined as ATM SAR Plugin"));
    }

    if (pEngine->cfg.aal2Included == IX_CC_ATMSAR_PLUGIN_AAL2_INLUDED_NO)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_INTERNAL,
                                ("ATM SAR instance %d does not support AAL2",
                                 arg_sarInstance));
    }

    if ((pEngine->pluginTab[i].initFlag & _IX_CC_ATMSAR_PLUGIN_INITFLAG_CID) != 0)
    {   /* ERROR - plug-in already registered */
        return IX_ERROR_WARNING(IX_CC_ERROR_DUPLICATE_ENTRY,
                                ("For the MODULE ID plug-in service for CID already registered in ATM SAR"));
    }
    pEngine->pluginTab[i].initFlag|=_IX_CC_ATMSAR_PLUGIN_INITFLAG_CID;
    pEngine->pluginTab[i].pCbCidCreate=arg_pCbCidCreate;
    pEngine->pluginTab[i].pCbCidUpdate=arg_pCbCidUpdate;    
    pEngine->pluginTab[i].pCbCidRemove=arg_pCbCidRemove;
    pEngine->pluginTab[i].pCbCidStats=arg_pCbCidStats;
   
    return IX_SUCCESS;
} /* end ix_cc_atmsar_plugin_reg_cid_service */

/**
 * NAME: ix_cc_atmsar_plugin_reg_done
 *
 * DESCRIPTION:
 *    The function notifies ATM SAR that the plug-in already register all
 *    services.
 *
 * @Param:  IN  arg_coreComponentId : Id of core component that register
 *                  service.
 * @Param:  IN  arg_sarInstance : Id of SAR Instance for that the action is
 *                  performed.
 * @Param:  IN  arg_pInstance :  pointer to structure describing local data
 *                  instance for the 'arg_sarInstance'.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - registering successful
 *    IX_CC_ERROR_ENTRY_NOT_FOUND - The ATM SAR is not ready for
 *         registering the core component for the specific ID.
 *    IX_CC_ERROR_DUPLICATE_ENTRY -- The ATM SAR already registered core
 *         component for the specific ID.
 *    IX_CC_ERROR_INTERNAL - The plug-in that trries to register works in the
 *          wrong thread. Check description of ATM SAR configuration tables
 *          and XML config description.
 */
ix_error
ix_cc_atmsar_plugin_reg_done(
ix_uint16 arg_coreComponentId,
ix_cc_atmsar_instance_t arg_sarInstance,
void* arg_pInstance)
{
    ix_uint16 i;
    _ix_cc_atmsar_engine_t* pEngine;
    
    _IX_CC_ATMSAR_DBG_INIT(("ix_cc_atmsar_plugin_register\n"));

    pEngine=_ix_cc_atmsar_plugin_find_core_in_engine(arg_sarInstance, arg_coreComponentId, &i);
    if (pEngine == NULL)
    {   /* ERROR - no entry found */
        return IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND,
                                ("The module is not defined as ATM SAR Plugin or ATM SAR is not initialized yet"));
    }
    if (arg_pInstance == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("The module context is NULL"));
    }
    if ((pEngine->pluginTab[i].initFlag & _IX_CC_ATMSAR_PLUGIN_INITFLAG_GENERAL) != 0)
    {   /* ERROR - plug-in already registered */
        return IX_ERROR_WARNING(IX_CC_ERROR_DUPLICATE_ENTRY,
                                ("For the MODULE ID plug-in service for CID already registered in ATM SAR"));
    }
    pEngine->pluginTab[i].initFlag|=_IX_CC_ATMSAR_PLUGIN_INITFLAG_GENERAL;
    pEngine->pluginTab[i].pCtx=arg_pInstance;
    if (pEngine->pInitVerify == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_INTERNAL, ("pInitVerify is NULL"));
    }
    return pEngine->pInitVerify(pEngine->pCtx);
} /* end ix_cc_atmsar_plugin_reg_done */


/**
 * NAME: ix_cc_atmsar_plugin_reg_port_handle_service
 *
 * DESCRIPTION:
 *    The function registers ATM SAR plug-in into ATM SAR core
 *    component. The registered functions are used for allocating
 *    and freeing of port handle.
 *
 * @Param:  IN  arg_sarInstance : Id of SAR Instance for that the action is
 *                  performed.
 * @Param:  IN  arg_pInstance :  pointer to structure describing local data
 *                  instance for the 'arg_sarInstance'.
 * @Param:  IN  arg_pCbPortAlloc : pointer to callback routine allocating port
 *                  handle.
 * @Param:  IN  arg_pCbPortFree : pointer to callback routine freeing port
 *                  handle.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - registering successful
 *    IX_CC_ERROR_INTERNAL - The plug-in that trries to register works in the
 *          wrong thread. Check description of ATM SAR configuration tables
 *          and XML config description.
 */
ix_error
ix_cc_atmsar_plugin_reg_port_handle_service(
ix_cc_atmsar_instance_t arg_sarInstance,
void* arg_pInstance,
const ix_cc_atmsar_plugin_port_alloc_t arg_pCbPortAlloc,
const ix_cc_atmsar_plugin_port_free_t arg_pCbPortFree)
{
    _ix_cc_atmsar_engine_t* pEngine;
    
    _IX_CC_ATMSAR_DBG_INIT(("ix_cc_atmsar_plugin_reg_port_handle_service\n"));
    if ((arg_pCbPortAlloc == NULL) || (arg_pCbPortFree == NULL))
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_INTERNAL, ("Any pointer is NULL"));
    }
    if (arg_pInstance == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("The module context is NULL"));
    }
    
    pEngine=_ix_cc_atmsar_plugin_find_engine(arg_sarInstance);
    if (pEngine == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_INTERNAL,
                                ("Engine pointer is NULL for instance %d", arg_sarInstance));
    }
    if (pEngine->pCbPortAlloc != NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_INTERNAL, ("Already set"));
    }
    pEngine->pCbPortAlloc=arg_pCbPortAlloc;
    pEngine->pCbPortFree=arg_pCbPortFree;
    pEngine->pPortHandleMgrCtx=arg_pInstance;
    if (pEngine->pInitVerify == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_INTERNAL, ("pInitVerify is NULL"));
    }
    return pEngine->pInitVerify(pEngine->pCtx);
} /* end ix_cc_atmsar_plugin_reg_port_handle_service */


/**
 * NAME: ix_cc_atmsar_plugin_reg_vc_handle_service
 *
 * DESCRIPTION:
 *    The function registers ATM SAR plug-in into ATM SAR core
 *    component. The registered functions are used for allocating
 *    and freeing of vc handle.
 *
 * @Param:  IN  arg_sarInstance : Id of SAR Instance for that the action is
 *                  performed.
 * @Param:  IN  arg_pInstance :  pointer to structure describing local data
 *                  instance for the 'arg_sarInstance'.
 * @Param:  IN  arg_pCbVcAlloc : pointer to callback routine allocating VC
 *                  handle.
 * @Param:  IN  arg_pCbVcFree : pointer to callback routine freeing VC
 *                  handle.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - registering successful
 *    IX_CC_ERROR_INTERNAL - The plug-in that trries to register works in the
 *          wrong thread. Check description of ATM SAR configuration tables
 *          and XML config description.
 */
ix_error
ix_cc_atmsar_plugin_reg_vc_handle_service(
ix_cc_atmsar_instance_t arg_sarInstance,
void* arg_pInstance,
const ix_cc_atmsar_plugin_vc_alloc_t arg_pCbVcAlloc,
const ix_cc_atmsar_plugin_vc_free_t arg_pCbVcFree)
{
    _ix_cc_atmsar_engine_t* pEngine;
    
    _IX_CC_ATMSAR_DBG_INIT(("ix_cc_atmsar_plugin_reg_vc_handle_service\n"));
    if ((arg_pCbVcAlloc == NULL) || (arg_pCbVcFree == NULL))
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_INTERNAL, ("Any pointer is NULL"));
    }
    if (arg_pInstance == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("The module context is NULL"));
    }

    pEngine=_ix_cc_atmsar_plugin_find_engine(arg_sarInstance);
    if (pEngine == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_INTERNAL,
                                ("Engine pointer is NULL for instance %d", arg_sarInstance));
    }
    if (pEngine->pCbVcAlloc != NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_INTERNAL, ("Already set"));
    }
    pEngine->pCbVcAlloc=arg_pCbVcAlloc;
    pEngine->pCbVcFree=arg_pCbVcFree;
    pEngine->pVcHandleMgrCtx=arg_pInstance;
    if (pEngine->pInitVerify == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_INTERNAL, ("pInitVerify is NULL"));
    }
    return pEngine->pInitVerify(pEngine->pCtx);
} /* end ix_cc_atmsar_plugin_reg_vc_handle_service */


/**
 * NAME: ix_cc_atmsar_plugin_reg_cid_handle_service
 *
 * DESCRIPTION:
 *    The function registers ATM SAR plug-in into ATM SAR core
 *    component. The registered functions are used for allocating
 *    and freeing of CID handle.
 *
 * @Param:  IN  arg_sarInstance : Id of SAR Instance for that the action is
 *                  performed.
 * @Param:  IN  arg_pInstance :  pointer to structure describing local data
 *                  instance for the 'arg_sarInstance'.
 * @Param:  IN  arg_pCbCidAlloc : pointer to callback routine allocating CID
 *                  handle.
 * @Param:  IN  arg_pCbCidFree : pointer to callback routine freeing CID
 *                  handle.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - registering successful
 *    IX_CC_ERROR_INTERNAL - The plug-in that trries to register works in the
 *          wrong thread. Check description of ATM SAR configuration tables
 *          and XML config description.
 */
ix_error
ix_cc_atmsar_plugin_reg_cid_handle_service(
ix_cc_atmsar_instance_t arg_sarInstance,
void* arg_pInstance,
const ix_cc_atmsar_plugin_cid_alloc_t arg_pCbCidAlloc,
const ix_cc_atmsar_plugin_cid_free_t arg_pCbCidFree)
{
    _ix_cc_atmsar_engine_t* pEngine;

    _IX_CC_ATMSAR_DBG_INIT(("ix_cc_atmsar_plugin_reg_cid_handle_service\n"));
    if ((arg_pCbCidAlloc == NULL) || (arg_pCbCidFree == NULL))
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_INTERNAL, ("Any pointer is NULL"));
    }
    if (arg_pInstance == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("The module context is NULL"));
    }

    pEngine=_ix_cc_atmsar_plugin_find_engine(arg_sarInstance);

    if (pEngine == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_INTERNAL,
                                ("Engine pointer is NULL for instance %d", arg_sarInstance));
    }

    if (pEngine->cfg.aal2Included == IX_CC_ATMSAR_PLUGIN_AAL2_INLUDED_NO)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_INTERNAL,
                                ("SAR control instance %d does not support AAL2",
                                 arg_sarInstance));
    }

    if (pEngine->pCbCidAlloc != NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_INTERNAL, ("Already set"));
    }
    pEngine->pCbCidAlloc=arg_pCbCidAlloc;
    pEngine->pCbCidFree=arg_pCbCidFree;
    pEngine->pCidHandleMgrCtx=arg_pInstance;
    if (pEngine->pInitVerify == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_INTERNAL, ("pInitVerify is NULL"));
    }
    return pEngine->pInitVerify(pEngine->pCtx);
} /* end ix_cc_atmsar_plugin_reg_cid_handle_service */

