/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2002 By Intel Corporation.  All rights reserved.
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/**
 *****************************************************************************
 * @ingroup SarUt
 * File:  ix_cc_atmsartest_cci.c
 * @description
 *      The file defines for core component infrestructure  interface and
 *      internal functions for initialization and shutdown.
 *****************************************************************************/

/*****************************************************************************
 *
 * Contents:
 *        Definitions of following exported data:
 *            none
 *        Definitions of following exported functions
 *            SarUt_T1()
 *            SarUt_T2()
 *            ix_cc_atmsartest_init()
 *            ix_cc_atmsartest_fini()
 *            SarUt_ShowVc()
 *            SarUt_VcDef()
 *            SarUt_VcCreate()
 *            SarUt_VcGetStats()
 *            SarUt_VcRemove()
 *            SarUt_PortGetStats()
 *            SarUt_ShowCid()
 *            SarUt_CidDef()
 *            SarUt_CidCreate()
 *            SarUt_CidGetStats()
 *            SarUt_CidRemove()
 *
 *        Definitions of following local functions:
 *            SarUtShowCbResult()
 *            SarUtCallbackVcCreate()
 *            SarUtCallbackVcUpdate()
 *            SarUtVcCreate()
 *            SarUtVcStats()
 *            SarUtVcRemove()
 *            SarUtCallbackVcCreate()
 *            SarUtCallbackVcRemove()
 *            SarUtCallbackVcStats()
 *            SarUtPortCreate()
 *            SarUtPortRemove()
 *            SarUtPortStats()
 *            SarUtCallbackPortCreate()
 *            SarUtCallbackPortRemove()
 *            SarUtCallbackPortStats()
 *            SarUtCidCreate()
 *            SarUtCidRemove()
 *            SarUtCidStats()
 *            SarUtCallbackCidCreate()
 *            SarUtCallbackCidRemove()
 *            SarUtCallbackCidStats()
 *            SarUtPrepareVcDescr()
 *            SarUtInit()
 *            SarUtMgrInit()
 *            SarUtGetTxtStateDescr()
 *
 *****************************************************************************/
#define IX_ERROR_FILE_IDENT "$Id: ix_cc_atmsartest_cci.c,v 1.6 2003/12/12 19:43:03 rwielick Exp $";

/*****************************************************************************
 *                   System defined include files
 *****************************************************************************/
#include "ix_cc_error.h"
#include "ix_rm.h"
#include "ix_ossl.h"
#include "ix_cc.h"
#include "ix_cc_macros.h"
#include "ix_cc_properties.h"
#include "ix_config.h"
#include "bindings.h"


#include "cc/ix_cc_msup.h"


/*****************************************************************************
 *                    User defined include files
 *****************************************************************************/
#include "cc/ix_cc_atmsar.h"
#include "cc/ix_cc_atmsartest_cci.h"
#include "cc/internal/ix_cc_atmsar_common.h"
#include "ix_cc_atmsartest_private.h"
#include "cc/ix_cc_atmrx_cci.h"
#include "dl_system.h"


/*****************************************************************************
 *        Preprocessor symbols and Macros used in this file
 *****************************************************************************/
#define SAR_UT_INST_0  0

/*****************************************************************************
 * Abstract
 *    Variable contains all data of VC Manager
 *
 *****************************************************************************/
static ix_uint8 staticLastProfile = SAR_UT_TP_ID_NONE;

/*****************************************************************************
 * Abstract
 *    Variable contains all data of VC Manager
 *
 *****************************************************************************/
sar_ut_mgr_t sarUtMgr;

/*****************************************************************************
 * Abstract
 *    To perform any operation on VC Manager, SAR UNIT VC MANAGET sub module
 *    reqiures initialization of structures. If flag is 0 system performs
 *    initialization if it is 1, the initialization already is performed.
 *
 *****************************************************************************/
static ix_uint8 sarUtMgrInit=0;

/*****************************************************************************
 * Abstract
 *  Test no 3 structures and defines.
 *
 *****************************************************************************/
#define VC_BREAK_VALUE 30


ix_int32 test3VcCreateMsgSend = 0;
ix_int32 test3VcCreateMsgRecv = 0;
ix_ossl_sem_t  test3Sid;


/*****************************************************************************
 * Abstract
 *    The table defines descrption of traffic parameters. The description
 *    can be used for defining VCs durung tests
 *
 *****************************************************************************/

static sar_ut_tp_t sarUtTpTab[SAR_UT_TP_ID_MAX]=
{  /* TP id,                         service class,                    aal type,           txPcr,  txScr, mbs, mfs, mcr, cdvt, tagging, frmDisc, etoeOamF5FlInd, ubrBcs,                          ubrPriority   */
    { SAR_UT_TP_ID_CBR_100,          IX_CC_ATMSAR_ATM_SERVICE_CBR,    100,       100,   0,   0,   0,   0,    0,       0,       0,              0,                               0},
    { SAR_UT_TP_ID_CBR_200,          IX_CC_ATMSAR_ATM_SERVICE_CBR,    200,       200,   0,   0,   0,   0,    0,       0,       0,              0,                               0},
    { SAR_UT_TP_ID_UBR_NOPRIORITY,   IX_CC_ATMSAR_ATM_SERVICE_UBR,    200,       0,     0,   0,   0,   100,  0,       0,       0,              IX_CC_ATMSAR_UBR_BCS_PCR,       0},
    { SAR_UT_TP_ID_UBR_PRIORITY,     IX_CC_ATMSAR_ATM_SERVICE_UBR,    200,       0,     0,   0,   0,   100,  0,       0,       0,              IX_CC_ATMSAR_UBR_BCS_NO_PCR,    1},
    { SAR_UT_TP_ID_RTVBR,            IX_CC_ATMSAR_ATM_SERVICE_RT_VBR, 200,       100,   0,   0,   0,   100,  0,       0,       0,              0,                               0},
    { SAR_UT_TP_ID_GFR,              IX_CC_ATMSAR_ATM_SERVICE_GFR,    200,       0,     0,   0,   100, 100,  0,       0,       0,              0,                               0},
    { SAR_UT_TP_ID_CBR_200000,       IX_CC_ATMSAR_ATM_SERVICE_CBR,    200000,    0,     0,   0,   0,   0,    0,       0,       0,              0,                               0},
    { SAR_UT_TP_ID_NRTVBR,           IX_CC_ATMSAR_ATM_SERVICE_NRT_VBR,200,       100,   0,   0,   0,   100,  0,       0,       0,              0,                               0},
    { SAR_UT_TP_ID_NONE,             IX_CC_ATMSAR_ATM_SERVICE_NONE,   0  ,       0,     0,   0,   0,   0,    0,       0,       0,              0,                               0},
    { SAR_UT_TP_ID_MAX,              IX_CC_ATMSAR_ATM_SERVICE_NONE,   0  ,       0,     0,   0,   0,   0,    0,       0,       0,              0,                               0},
};


/*****************************************************************************
 * Abstract
 *    The table maps ID defined in 'sar_ut_tp_id_t' to rows defined in
 *    'sarUtTpTab'. Indexes of table are values from sar_ut_tp_id_t. Rows
 *    contains indexes to table 'sarUtTpTab'.
 *
 *****************************************************************************/
static ix_uint16 sarUtTpMap[SAR_UT_TP_ID_MAX];


/*****************************************************************************
 * Abstract
 *    To perform test. SAR UNIT TEST module reqiured initialization of
 *    structures (i.e. sarUtTpMap). If flag is 0 system performs initialization
 *    if it is 1, the initialization already is performed.
 *
 *****************************************************************************/
static ix_uint8  sarUtInitFlag=0;


/*****************************************************************************
 * Abstract
 *    The core component context.
 *
 *****************************************************************************/
static sar_ut_context_t* s_pAtmsartestContext;


/*****************************************************************************
 *                    Extern function prototypes.
 *****************************************************************************/


/*****************************************************************************
 *                    Static function prototypes.
 *****************************************************************************/

static ix_error
SarUtVcCreate(sar_ut_test_id_t arg_testId, ix_uint32 transId,
              const ix_cc_atmsar_instance_t arg_sarInstance,
              const ix_cc_atmsar_vc_descr_t *arg_pVcDescr);

static ix_error
SarUtVcUpdate(sar_ut_test_id_t arg_testId, ix_uint32 transId,
              const ix_cc_atmsar_instance_t arg_sarInstance,
              ix_cc_atmsar_vc_handle_t arg_hVc,
              const ix_cc_atmsar_update_data_t* arg_pUpdateData);

static ix_error
SarUtVcStats(sar_ut_test_id_t arg_testId, ix_uint32 transId,
             const ix_cc_atmsar_instance_t arg_sarInstance,
             ix_cc_atmsar_vc_handle_t arg_hVc);

static ix_error
SarUtVcRemove(sar_ut_test_id_t arg_testId, ix_uint32 transId,
              const ix_cc_atmsar_instance_t arg_sarInstance,
              ix_cc_atmsar_vc_handle_t arg_hVc);

static ix_error
SarUtCallbackVcCreate(ix_error arg_Result,
                      ix_cc_atmsar_vc_handle_t arg_hVc, void* arg_pContext);

static ix_error
SarUtCallbackVcUpdate(ix_error arg_Result,
                      ix_cc_atmsar_vc_handle_t arg_hVc, void* arg_pContext);

static ix_error
SarUtCallbackVcRemove(ix_error arg_Result, void* arg_pContext);

static ix_error
SarUtCallbackVcStats(ix_error arg_Result,
                     ix_cc_atmsar_vc_stats_t *arg_pLookupResult, void* arg_pContext);

static ix_error
SarUtPortCreate(sar_ut_test_id_t arg_testId, ix_uint32 transId,
                const ix_cc_atmsar_instance_t arg_sarInstance,
                const ix_cc_atmsar_port_descr_t *arg_pPortDescr);

static ix_error
SarUtPortRemove(sar_ut_test_id_t arg_testId, ix_uint32 transId,
                const ix_cc_atmsar_instance_t arg_sarInstance,
                ix_cc_atmsar_port_handle_t arg_hPort);

static ix_error
SarUtPortStats(sar_ut_test_id_t arg_testId, ix_uint32 transId,
               const ix_cc_atmsar_instance_t arg_sarInstance,
               ix_cc_atmsar_port_handle_t arg_hPort);

static ix_error
SarUtCallbackPortCreate(ix_error arg_Result,
                        ix_cc_atmsar_port_handle_t arg_hPort, void* arg_pContext);

static ix_error
SarUtCallbackPortRemove(ix_error arg_Result, void* arg_pContext);

static ix_error
SarUtCallbackPortStats(ix_error arg_Result,
                       ix_cc_atmsar_port_stats_t *arg_pLookupResult, void* arg_pContext);

static ix_error
SarUtCidCreate(sar_ut_test_id_t arg_testId, ix_uint32 transId,
               const ix_cc_atmsar_instance_t arg_sarInstance,
               const ix_cc_atmsar_cid_descr_t *arg_pCidDescr);

static ix_error
SarUtCidUpdate(sar_ut_test_id_t arg_testId,
               ix_uint32 transId,
               ix_cc_atmsar_cid_handle_t arg_hCid,
               const ix_cc_atmsar_instance_t arg_sarInstance,
               const ix_cc_atmrx_update_data_t *arg_pUpdate);

static ix_error
SarUtCidRemove(sar_ut_test_id_t arg_testId, ix_uint32 transId,
               const ix_cc_atmsar_instance_t arg_sarInstance,
               ix_cc_atmsar_cid_handle_t arg_hCid);

static ix_error
SarUtCidStats(sar_ut_test_id_t arg_testId, ix_uint32 transId,
              const ix_cc_atmsar_instance_t arg_sarInstance,
              ix_cc_atmsar_cid_handle_t arg_hCid);

static ix_error
SarUtCallbackCidCreate(ix_error arg_Result,
                       ix_cc_atmsar_cid_handle_t arg_hCid, void* arg_pContext);

static ix_error
SarUtCallbackCidUpdate(ix_error arg_Result,
                       ix_cc_atmsar_cid_handle_t arg_hCid, void* arg_pContext);

static ix_error
SarUtCallbackCidRemove(ix_error arg_Result, void* arg_pContext);

static ix_error
SarUtCallbackCidStats(ix_error arg_Result,
                      ix_cc_atmsar_cid_stats_t *arg_pLookupResult, void* arg_pContext);

static ix_error
SarUtInit();

static ix_error
SarUtPrepareVcDescr(sar_ut_vc_descr_t* pVcUt, ix_cc_atmsar_vc_descr_t* pVc);

static void
SarUtShowCbResult(char *pOperName, ix_error result);

static char *
SarUtGetTxtStateDescr(
ix_uint16 state);

void SarUt_VcDef(
#ifdef CLI_SUPPORT
ix_uint16 direction,
#endif
ix_uint16 vcId, ix_uint16 port, ix_uint16 vpi, ix_uint16 vci,
ix_uint16 tpId, ix_uint16 vctype);


/*****************************************************************************
 *                        Function definitions.
 *****************************************************************************/
void
SarUt_ShowVc(
#ifdef CLI_SUPPORT
ix_int16 direction,
#endif
ix_uint16 vcId);

void
SarUt_ShowCid(ix_uint16 cidId);


/*****************************************************************************
 * Abstract:
 *    Shows result of SAR operation.
 *
 * Params:
 *    IN state ID
 *
 * Return Values:
 *	  pointer to string describing the state
 *
 *****************************************************************************/
char *
SarUtGetTxtStateDescr(
ix_uint16 state)
{
    switch (state)
    {
    case SAR_UT_FSM_ST_UNDEF:
        return "UNDEF";
    case SAR_UT_FSM_ST_DEFINED:
        return "DEFINED";
    case SAR_UT_FSM_ST_CREATING:
        return "CREATING";
    case SAR_UT_FSM_ST_CREATED:
        return "CREATED";
    case SAR_UT_FSM_ST_STAT_READING:
        return "STAT_READING";
    case SAR_UT_FSM_ST_REMOVING:
        return "REMOVING";
    case SAR_UT_FSM_ST_UPDATING:
        return "UPDATING";
    case SAR_UT_FSM_ST_UPDATED:
        return "UPDATED";
    default:
        return "ERROR";
    }
}


/*****************************************************************************
 * Abstract:
 *    Shows result of SAR operation.
 *
 * Params:
 *    IN operName - textural description of executed operation
 *    IN result - result of operation
 *
 * Return Values:
 *	  none
 *
 *****************************************************************************/
void
SarUtShowCbResult(
char *pOperName,
ix_error result)
{
    _IX_CC_ATMSAR_DBG_TEST(("TEST : %s CALLBACK %s:\n",
                            pOperName, (result==IX_SUCCESS)?"OK":"ERROR"));
    if (result!=IX_SUCCESS)
    {
        ix_error_dump(stderr, result);
    }
}


/*****************************************************************************
 * Abstract:
 *    The callback function is called by core component infrastructure (CCI)
 *    when SAR Control module processed a message "VC create" and sent reply.
 *
 * Params:
 *    IN result - result of SAR Control operation
 *    IN hVc    - VC handle of new created VC
 *    IN pCtx   - Pointer to context of operation.
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtCallbackVcCreate(
ix_error result,
ix_cc_atmsar_vc_handle_t hVc,
void* pCtx)
{
    sar_ut_async_context_t *pContext= (sar_ut_async_context_t*) pCtx;
    sar_ut_vc_t* pVc;
    switch (pContext->testId)
    {
    case SAR_UT_TEST_NO_3:
        test3VcCreateMsgRecv++;
        if (test3VcCreateMsgRecv % VC_BREAK_VALUE)
        {
            ix_ossl_sem_give(test3Sid);
        }
        /* now follow the SAR_UT_TEST_NO_MGR execution path
         * break must not be placed here */
    case SAR_UT_TEST_NO_MGR:
        if (pContext->xId>=SAR_UT_MGR_MAX_VC)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                    ("Vc ID %d is out of range", pContext->xId));
        }
        pVc= &sarUtMgr.vc[pContext->xId];
        if (pVc->state != SAR_UT_FSM_ST_CREATING)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                    ("Wrong FSM State %d for VC %d",
                                     pVc->state, pContext->xId));
        }

        SarUtShowCbResult("VC CREATE", result);
        if (result == IX_SUCCESS)
        {
            pVc->state = SAR_UT_FSM_ST_CREATED;
            pVc->handle= hVc;
        }
        else
        {
            pVc->state = SAR_UT_FSM_ST_DEFINED;
        }
#ifndef CLI_SUPPORT
        SarUt_ShowVc(pContext->xId);
#else
        SarUt_ShowVc(pVc->direction,pContext->xId);
#endif
        break;
    default:
        SarUtShowCbResult("VC CREATE", result);
        SarUtVcStats(pContext->testId, pContext->xId, SAR_UT_INST_0, hVc);
        break;
    }

    ix_ossl_free(pCtx);
    if (pContext->pSem != NULL)
    {
        /*
         * If we're not able to give the semaphore, print the error.  Always
         * return success.
         */
        IX_ERROR_CP(ix_ossl_sem_give(*(pContext->pSem)));
    }

    return IX_SUCCESS;
}


/*****************************************************************************
 * Abstract:
 *    The callback function is called by core component infrastructure (CCI)
 *    when SAR Control module processed a message "VC create" and sent reply.
 *
 * Params:
 *    IN result - result of SAR Control operation
 *    IN hVc    - VC handle of new created VC
 *    IN pCtx   - Pointer to context of operation.
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtCallbackVcUpdate(
ix_error result,
ix_cc_atmsar_vc_handle_t hVc,
void* pCtx)
{
    sar_ut_async_context_t *pContext= (sar_ut_async_context_t*) pCtx;
    sar_ut_vc_t* pVc;
    switch (pContext->testId)
    {
    case SAR_UT_TEST_NO_3:
        test3VcCreateMsgRecv++;
        if (test3VcCreateMsgRecv % VC_BREAK_VALUE)
        {
            ix_ossl_sem_give(test3Sid);
        }
        /* now follow the SAR_UT_TEST_NO_MGR execution path
         * break must not be placed here */
    case SAR_UT_TEST_NO_MGR:
        if (pContext->xId>=SAR_UT_MGR_MAX_VC)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                    ("Vc ID %d is out of range", pContext->xId));
        }
        pVc= &sarUtMgr.vc[pContext->xId];
        if (pVc->state != SAR_UT_FSM_ST_UPDATING)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                    ("Wrong FSM State %d for VC %d",
                                     pVc->state, pContext->xId));
        }

        if (result == IX_SUCCESS)
        {
            pVc->state = SAR_UT_FSM_ST_UPDATED;
        }
        else
        {
            pVc->state = SAR_UT_FSM_ST_CREATED;
        }

        SarUtShowCbResult("VC UPDATE", result);
#ifndef CLI_SUPPORT
        SarUt_ShowVc(pContext->xId);
#else
        SarUt_ShowVc(pVc->direction,pContext->xId);
#endif
        break;
    default:
        SarUtShowCbResult("VC CREATE", result);
        SarUtVcStats(pContext->testId, pContext->xId, SAR_UT_INST_0, hVc);
        break;
    }

    ix_ossl_free(pCtx);
    if (pContext->pSem != NULL)
    {
        /*
         * If we're not able to give the semaphore, print the error.  Always
         * return success.
         */
        IX_ERROR_CP(ix_ossl_sem_give(*(pContext->pSem)));
    }

    return IX_SUCCESS;
}

/*****************************************************************************
 * Abstract:
 *    Send request to SAR Control for VC creation.
 *
 * Params:
 *    IN arg_testId: Identiefier of test
 *    IN arg_pVcDescr: pointer to structure describing requested VC
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtVcCreate(
sar_ut_test_id_t arg_testId,
ix_uint32 transId,
const ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmsar_vc_descr_t *arg_pVcDescr)
{
    ix_ossl_sem_t CmdFinishedSem;
    ix_error err=IX_SUCCESS;
    sar_ut_async_context_t *pContext;

    pContext = ix_ossl_malloc(sizeof(sar_ut_async_context_t));

    if (pContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("Context is NULL"));
    }

    memcpy(&(pContext->context.vcCreate.VcDescr),
           arg_pVcDescr,
           sizeof(ix_cc_atmsar_vc_descr_t));

    pContext->used=1;
    pContext->testId=arg_testId;
    pContext->xId=transId;
  
    /* Allocate the semaphore - it is released by calback functions */
    IX_ERROR_CR(ix_ossl_sem_init(IX_OSSL_SEM_UNAVAILABLE, &CmdFinishedSem));
    pContext->pSem = &CmdFinishedSem;
    _IX_CC_ATMSAR_DBG_TEST(("TEST : Send msg VC CREATE with PortNo=%d,"
                            " VPI=%d, VCI=%d\n",
                            arg_pVcDescr->spec.portNo, arg_pVcDescr->spec.vpi,
                            arg_pVcDescr->spec.vci));
    err=ix_cc_atmsar_async_vc_create(arg_sarInstance,
                                      arg_pVcDescr,
                                      SarUtCallbackVcCreate,
                                      (void*)pContext);
    /* wait until this command finishes */                                             
    ix_ossl_sem_take(CmdFinishedSem, IX_OSSL_WAIT_FOREVER);    
    IX_ERROR_CR(ix_ossl_sem_fini(CmdFinishedSem));
    
    if ( err != IX_SUCCESS)
    {
        err=IX_ERROR_WARNING(err, ("Send request message failed"));
        ix_ossl_free(pContext);
    }
    return err;
}

/*****************************************************************************
 * Abstract:
 *    Send request to SAR Control for VC creation.
 *
 * Params:
 *    IN arg_testId: Identiefier of test
 *    IN arg_pVcDescr: pointer to structure describing requested VC
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtVcUpdate(
sar_ut_test_id_t arg_testId,
ix_uint32 transId,
const ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmsar_vc_handle_t arg_hVc,
const ix_cc_atmsar_update_data_t* arg_pUpdateData)
{
    ix_ossl_sem_t CmdFinishedSem;
    ix_error err=IX_SUCCESS;
    sar_ut_async_context_t *pContext;
    ix_cc_atmrx_update_data_t *pRxUpdateData;

    pContext = ix_ossl_malloc(sizeof(sar_ut_async_context_t));

    if (pContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("Context is NULL"));
    }

    /* store l2port and header type in the "VC UPDATE" message */
    memcpy(&(pContext->context.vcUpdate.updateData),
           arg_pUpdateData,
           sizeof(ix_cc_atmsar_update_data_t));

    pContext->used=1;
    pContext->testId=arg_testId;
    pContext->xId=transId;
    /* Allocate the semaphore - it is released by calback functions */
    IX_ERROR_CR(ix_ossl_sem_init(IX_OSSL_SEM_UNAVAILABLE, &CmdFinishedSem));
    pContext->pSem = &CmdFinishedSem;
    pRxUpdateData = (ix_cc_atmrx_update_data_t*)arg_pUpdateData;
    _IX_CC_ATMSAR_DBG_TEST(("TEST : Send msg VC UPDATE with VC HANDLE: %d,"
                            " L2PORT: %d HDR TYPE: %d\n",
                            arg_hVc,
                            pRxUpdateData->lw0.l2port,
                            pRxUpdateData->lw1.hdrType));
    err=ix_cc_atmsar_async_vc_update(arg_sarInstance,
                                     arg_hVc,
                                     arg_pUpdateData,
                                     SarUtCallbackVcUpdate,
                                     (void*)pContext);
    /* wait until this command finishes */                                             
    ix_ossl_sem_take(CmdFinishedSem, IX_OSSL_WAIT_FOREVER);    
    IX_ERROR_CR(ix_ossl_sem_fini(CmdFinishedSem));

    if ( err != IX_SUCCESS)
    {
        err=IX_ERROR_WARNING(err, ("Send request message failed"));
        ix_ossl_free(pContext);
    }
    return err;
}

/*****************************************************************************
 * Abstract:
 *    The callback function. It is call by CCI, when SAR Control pelies for
 *    'VC remove' request.
 *
 * Params:
 *    IN result - result of SAR Control operation
 *    IN pCtx   - Pointer to context of operation.
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtCallbackVcRemove(
ix_error result,
void* pCtx)
{
    sar_ut_async_context_t *pContext= (sar_ut_async_context_t*) pCtx;
    sar_ut_vc_t* pVc;
    switch (pContext->testId)
    {
    case SAR_UT_TEST_NO_MGR:
        if (pContext->xId>=SAR_UT_MGR_MAX_VC)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                    ("Vc ID %d is out of range", pContext->xId));
        }
        pVc= &sarUtMgr.vc[pContext->xId];
        if (pVc->state != SAR_UT_FSM_ST_REMOVING)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                    ("Wrong FSM State %d for VC %d",
                                     pVc->state, pContext->xId));
        }

        SarUtShowCbResult("VC REMOVE", result);
        pVc->state = SAR_UT_FSM_ST_DEFINED;
        pVc->handle= IX_CC_ATMSAR_VCID_INVALID;
#ifndef CLI_SUPPORT
        SarUt_ShowVc(pContext->xId);
#else
        SarUt_ShowVc(pVc->direction,pContext->xId);
#endif
        break;
    default:
        SarUtShowCbResult("VC REMOVE", result);
        break;
    }
    ix_ossl_free(pContext);
    if (pContext->pSem != NULL)
    {
        /*
         * If we're not able to give the semaphore, print the error.  Always
         * return success.
         */
        IX_ERROR_CP(ix_ossl_sem_give(*(pContext->pSem)));
    }
        
    return IX_SUCCESS;
}


/*****************************************************************************
 * Abstract:
 *    Send request to SAR Control for VC remove.
 *
 * Params:
 *    IN
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtVcRemove(
sar_ut_test_id_t arg_testId,
ix_uint32 transId,
const ix_cc_atmsar_instance_t arg_sarInstance,
ix_cc_atmsar_vc_handle_t arg_hVc)
{
    ix_ossl_sem_t CmdFinishedSem;	
    ix_error err;
    sar_ut_async_context_t *pContext;

    pContext = ix_ossl_malloc(sizeof(sar_ut_async_context_t));

    if (pContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("Context is NULL"));
    }

    pContext->context.vcRemove.hVc=arg_hVc;
    pContext->used=1;
    pContext->testId=arg_testId;
    pContext->xId=transId;
    /* Allocate the semaphore - it is released by calback functions */
    IX_ERROR_CR(ix_ossl_sem_init(IX_OSSL_SEM_UNAVAILABLE, &CmdFinishedSem));
    pContext->pSem = &CmdFinishedSem;
    _IX_CC_ATMSAR_DBG_TEST(("TEST : Send msg VC REMOVE hVc=%d\n",
                            arg_hVc));
    err=ix_cc_atmsar_async_vc_remove(arg_sarInstance,
                                      arg_hVc,
                                      SarUtCallbackVcRemove,
                                      (void*)pContext);
    /* wait until this command finishes */                                             
    ix_ossl_sem_take(CmdFinishedSem, IX_OSSL_WAIT_FOREVER);    
    IX_ERROR_CR(ix_ossl_sem_fini(CmdFinishedSem));

    if ( err != IX_SUCCESS)
    {
        IX_ERROR_WARNING(err, ("Send reply message failed"));
        ix_ossl_free(pContext);
    }
    return IX_SUCCESS;
}


/*****************************************************************************
 * Abstract:
 *    The callback function. It is call by CCI, when SAR Control pelies for
 *    'VC statistic' request.
 *
 * Params:
 *    IN result - result of SAR Control operation
 *    IN pLookupResult - pointer to structure with returned statistic data.
 *    IN pCtx   - Pointer to context of operation.
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtCallbackVcStats(
ix_error result,
ix_cc_atmsar_vc_stats_t *arg_pLookupResult,
void* pCtx)
{
    sar_ut_async_context_t *pContext= (sar_ut_async_context_t*) pCtx;
    sar_ut_vc_t* pVc;
    switch (pContext->testId)
    {
    case SAR_UT_TEST_NO_MGR:
        if (pContext->xId>=SAR_UT_MGR_MAX_VC)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                    ("Vc ID %d is out of range", pContext->xId));
        }
        pVc= &sarUtMgr.vc[pContext->xId];
        if (pVc->state != SAR_UT_FSM_ST_STAT_READING)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                    ("Wrong FSM State %d for VC %d",
                                     pVc->state, pContext->xId));
        }

        SarUtShowCbResult("VC STATS", result);
        pVc->state = SAR_UT_FSM_ST_UPDATED;
#ifndef CLI_SUPPORT
        SarUt_ShowVc(pContext->xId);
#else
        SarUt_ShowVc(pVc->direction,pContext->xId);
#endif
        _IX_CC_ATMSAR_DBG_TEST(("  RX pkts=%-8ld, cells=%-8ld, discards=%-8ld, cellsErr=%-8ld\n",
                                (ix_uint32)arg_pLookupResult->rx.packets,
                                (ix_uint32)arg_pLookupResult->rx.cells,
                                (ix_uint32)arg_pLookupResult->rx.discards,
                                (ix_uint32)arg_pLookupResult->rx.cellErrors));
        _IX_CC_ATMSAR_DBG_TEST(("  TX pkts=%-8ld, cells=%-8ld, discards=%-8ld\n",
                                (ix_uint32)arg_pLookupResult->tx.packets,
                                (ix_uint32)arg_pLookupResult->tx.cells,
                                (ix_uint32)arg_pLookupResult->tx.discards));

        break;
    default:
        SarUtShowCbResult("VC STATS", result);
        SarUtVcRemove(pContext->testId, pContext->xId,
                      SAR_UT_INST_0, pContext->context.vcStats.hVc);
        break;
    }
    ix_ossl_free(pCtx);
    if (pContext->pSem != NULL)
    {
        /*
         * If we're not able to give the semaphore, print the error.  Always
         * return success.
         */
        IX_ERROR_CP(ix_ossl_sem_give(*(pContext->pSem)));
    }

    return IX_SUCCESS;
}


/*****************************************************************************
 * Abstract:
 *
 * Params:
 *    IN
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtVcStats(
sar_ut_test_id_t arg_testId,
ix_uint32 transId,
const ix_cc_atmsar_instance_t arg_sarInstance,
ix_cc_atmsar_vc_handle_t arg_hVc)
{
    ix_ossl_sem_t CmdFinishedSem;
    ix_error err;
    sar_ut_async_context_t *pContext;

    pContext = ix_ossl_malloc(sizeof(sar_ut_async_context_t));
    if (pContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("Context is NULL"));
    }

    pContext->context.vcStats.hVc=arg_hVc;
    pContext->used=1;
    pContext->testId=arg_testId;
    pContext->xId=transId;
    /* Allocate the semaphore - it is released by calback functions */
    IX_ERROR_CR(ix_ossl_sem_init(IX_OSSL_SEM_UNAVAILABLE, &CmdFinishedSem));
    pContext->pSem = &CmdFinishedSem;
    _IX_CC_ATMSAR_DBG_TEST(("TEST : Send msg VC STATS hVc=%d\n", arg_hVc));

    err=ix_cc_atmsar_async_get_vc_stats(arg_sarInstance,
                                        arg_hVc,
                                        SarUtCallbackVcStats,
                                        (void*)pContext);
    /* wait until this command finishes */                                             
    ix_ossl_sem_take(CmdFinishedSem, IX_OSSL_WAIT_FOREVER);    
    IX_ERROR_CR(ix_ossl_sem_fini(CmdFinishedSem));

    if ( err != IX_SUCCESS)
    {
        IX_ERROR_WARNING(err, ("Send reply message failed"));
        ix_ossl_free(pContext);
    }
    return IX_SUCCESS;
}


/*****************************************************************************
 * Abstract:
 *    The callback function. It is call by CCI, when SAR Control pelies for
 *    'port create' request.
 *
 * Params:
 *    IN
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtCallbackPortCreate(
ix_error arg_Result,
ix_cc_atmsar_port_handle_t arg_hPort,
void* arg_pContext)
{
    sar_ut_async_context_t *pContext= (sar_ut_async_context_t*) arg_pContext;
    SarUtShowCbResult("PORT CREATE", arg_Result);
    SarUtPortStats(pContext->testId, pContext->xId, SAR_UT_INST_0,
                   pContext->context.portStats.hPort);
    ix_ossl_free(pContext);
    if (pContext->pSem != NULL)
    {
        /*
         * If we're not able to give the semaphore, print the error.  Always
         * return success.
         */
        IX_ERROR_CP(ix_ossl_sem_give(*(pContext->pSem)));
    }

    return IX_SUCCESS;
}


/*****************************************************************************
 * Abstract:
 *
 * Params:
 *    IN
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtPortCreate(
sar_ut_test_id_t arg_testId,
ix_uint32 transId,
const ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmsar_port_descr_t *arg_pPortDescr)
{
    ix_ossl_sem_t CmdFinishedSem;
    ix_error err;
    sar_ut_async_context_t *pContext;

    pContext = ix_ossl_malloc(sizeof(sar_ut_async_context_t));
    if (pContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("Context is NULL"));
    }

    memcpy(&(pContext->context.portCreate.portDescr),
           arg_pPortDescr,
           sizeof(ix_cc_atmsar_port_descr_t));

    pContext->used=1;
    pContext->testId=arg_testId;
    pContext->xId=transId;

    /* Allocate the semaphore - it is released by calback functions */
    IX_ERROR_CR(ix_ossl_sem_init(IX_OSSL_SEM_UNAVAILABLE, &CmdFinishedSem));
    pContext->pSem = &CmdFinishedSem;

    _IX_CC_ATMSAR_DBG_TEST(("TEST : Send msg VC CREATE with PortNo=%d,"
                            " BandwidthId=%d\n",
                            arg_pPortDescr->portId, arg_pPortDescr->bandwidth));
    err=ix_cc_atmsar_async_port_create(arg_sarInstance,
                                       arg_pPortDescr,
                                       SarUtCallbackPortCreate,
                                       (void*)pContext);
    /* wait until this command finishes */                                             
    ix_ossl_sem_take(CmdFinishedSem, IX_OSSL_WAIT_FOREVER);    
    IX_ERROR_CR(ix_ossl_sem_fini(CmdFinishedSem));

    if ( err != IX_SUCCESS)
    {
        IX_ERROR_WARNING(err, ("Send reply message failed"));
        ix_ossl_free(pContext);
    }
    return IX_SUCCESS;
}


/*****************************************************************************
 * Abstract:
 *
 * Params:
 *    IN
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtCallbackPortRemove(
ix_error arg_Result,
void* arg_pContext)
{
    sar_ut_async_context_t *pContext= (sar_ut_async_context_t*) arg_pContext;
    SarUtShowCbResult("PORT REMOVE", arg_Result);
    ix_ossl_free(pContext);
    if (pContext->pSem != NULL)
    {
        /*
         * If we're not able to give the semaphore, print the error.  Always
         * return success.
         */
        IX_ERROR_CP(ix_ossl_sem_give(*(pContext->pSem)));
    }
        
    return IX_SUCCESS;
}


/*****************************************************************************
 * Abstract:
 *
 * Params:
 *    IN arg_testId: Identiefier of test
 *    IN
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtPortRemove(
sar_ut_test_id_t arg_testId,
ix_uint32 transId,
const ix_cc_atmsar_instance_t arg_sarInstance,
ix_cc_atmsar_port_handle_t arg_hPort)
{
    ix_ossl_sem_t CmdFinishedSem;	
    ix_error err;
    sar_ut_async_context_t *pContext;

    pContext = ix_ossl_malloc(sizeof(sar_ut_async_context_t));
    if (pContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("Context is NULL"));
    }

    pContext->context.portRemove.hPort=arg_hPort;
    pContext->used=1;
    pContext->testId=arg_testId;
    pContext->xId=transId;

    /* Allocate the semaphore - it is released by calback functions */
    IX_ERROR_CR(ix_ossl_sem_init(IX_OSSL_SEM_UNAVAILABLE, &CmdFinishedSem));
    pContext->pSem = &CmdFinishedSem;
    _IX_CC_ATMSAR_DBG_TEST(("TEST : Send msg PORT REMOVE hPort=%d\n",
                            arg_hPort));
    err=ix_cc_atmsar_async_port_remove(arg_sarInstance,
                                        arg_hPort,
                                        SarUtCallbackPortRemove,
                                        (void*)pContext);
    /* wait until this command finishes */                                             
    ix_ossl_sem_take(CmdFinishedSem, IX_OSSL_WAIT_FOREVER);    
    IX_ERROR_CR(ix_ossl_sem_fini(CmdFinishedSem));

    if ( err != IX_SUCCESS)
    {
        IX_ERROR_WARNING(err, ("Send reply message failed"));
        ix_ossl_free(pContext);
    }
    return IX_SUCCESS;
}


/*****************************************************************************
 * Abstract:
 *    The callback function. It is call by CCI, when SAR Control pelies for
 *    'port statistic' request.
 *
 * Params:
 *    IN result - result of SAR Control operation
 *    IN pLookupResult - pointer to structure with returned statistic data.
 *    IN pCtx   - Pointer to context of operation.
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtCallbackPortStats(
ix_error result,
ix_cc_atmsar_port_stats_t *pLookupResult,
void* pCtx)
{
    sar_ut_async_context_t *pContext= (sar_ut_async_context_t*) pCtx;
    switch (pContext->testId)
    {
    case SAR_UT_TEST_NO_MGR:
        if (pContext->xId>=MAX_ATM_PORTS)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                    ("Port ID %d is out of range", pContext->xId));
        }

        SarUtShowCbResult("PORT STATS", result);
        _IX_CC_ATMSAR_DBG_TEST(("Port %d Statistics:\n", pContext->context.portStats.hPort));
        _IX_CC_ATMSAR_DBG_TEST(("  RX pkts=%-8ld, cells=%-8ld, discards=%-8ld, cellsErr=%-8ld\n",
                                (ix_uint32)pLookupResult->rx.packets,
                                (ix_uint32)pLookupResult->rx.cells,
                                (ix_uint32)pLookupResult->rx.discards,
                                (ix_uint32)pLookupResult->rx.cellErrors));
        _IX_CC_ATMSAR_DBG_TEST(("  TX pkts=%-8ld, cells=%-8ld, discards=%-8ld\n",
                                (ix_uint32)pLookupResult->tx.packets,
                                (ix_uint32)pLookupResult->tx.cells,
                                (ix_uint32)pLookupResult->tx.discards));

        break;
    default:
        SarUtShowCbResult("PORT STATS", result);
        SarUtPortRemove(pContext->testId, pContext->xId, SAR_UT_INST_0,
                        pContext->context.portStats.hPort);
        break;
    }
    ix_ossl_free(pCtx);
    if (pContext->pSem != NULL)
    {
        /*
         * If we're not able to give the semaphore, print the error.  Always
         * return success.
         */
        IX_ERROR_CP(ix_ossl_sem_give(*(pContext->pSem)));
    }
       
    return IX_SUCCESS;
}

/*****************************************************************************
 * Abstract:
 *
 * Params:
 *    IN arg_testId: Identiefier of test
 *    IN arg_hPort:
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtPortStats(
sar_ut_test_id_t arg_testId,
ix_uint32 transId,
const ix_cc_atmsar_instance_t arg_sarInstance,
ix_cc_atmsar_port_handle_t arg_hPort)
{
    ix_ossl_sem_t CmdFinishedSem;
    ix_error err;
    sar_ut_async_context_t *pContext;

    pContext = ix_ossl_malloc(sizeof(sar_ut_async_context_t));
    if (pContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("Context is NULL"));
    }

    pContext->context.portStats.hPort=arg_hPort;
    pContext->used=1;
    pContext->testId=arg_testId;
    pContext->xId=transId;

    /* Allocate the semaphore - it is released by calback functions */
    IX_ERROR_CR(ix_ossl_sem_init(IX_OSSL_SEM_UNAVAILABLE, &CmdFinishedSem));
    pContext->pSem = &CmdFinishedSem;
    
    _IX_CC_ATMSAR_DBG_TEST(("TEST : Send msg PORT STATS hPort=%d\n", arg_hPort));
    err=ix_cc_atmsar_async_get_port_stats(arg_sarInstance,
                                          arg_hPort,
                                          SarUtCallbackPortStats,
                                          (void*)pContext);
    /* wait until this command finishes */                                             
    ix_ossl_sem_take(CmdFinishedSem, IX_OSSL_WAIT_FOREVER);    
    IX_ERROR_CR(ix_ossl_sem_fini(CmdFinishedSem));

    if ( err != IX_SUCCESS)
    {
        IX_ERROR_WARNING(err, ("Send reply message failed"));
        ix_ossl_free(pContext);
    }
    return IX_SUCCESS;
}


/*****************************************************************************
 * Abstract:
 *    The callback function is called by core component infrastructure (CCI)
 *    when SAR Control module processed a message "CID create" and sent reply.
 *
 * Params:
 *    IN result - result of SAR Control operation
 *    IN hVc    - VC handle of new created VC
 *    IN pCtx   - Pointer to context of operation.
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtCallbackCidCreate(
ix_error result,
ix_cc_atmsar_cid_handle_t hCid,
void* pCtx)
{
    sar_ut_async_context_t *pContext= (sar_ut_async_context_t*) pCtx;
    sar_ut_cid_t* pCid;
    switch (pContext->testId)
    {
    case SAR_UT_TEST_NO_MGR:
        if (pContext->xId>=SAR_UT_MGR_MAX_CID)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                    ("Cid ID %d is out of range", pContext->xId));
        }
        pCid= &sarUtMgr.cid[pContext->xId];
        if (pCid->state != SAR_UT_FSM_ST_CREATING)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                    ("Wrong FSM State %d for CID %d",
                                     pCid->state, pContext->xId));
        }

        SarUtShowCbResult("CID CREATE", result);
        if (result == IX_SUCCESS)
        {
            pCid->state = SAR_UT_FSM_ST_CREATED;
            pCid->handle= hCid;
        }
        else
        {
            pCid->state = SAR_UT_FSM_ST_DEFINED;
        }
        SarUt_ShowCid(pContext->xId);
        break;
    default:
        SarUtShowCbResult("CID CREATE", result);
        SarUtCidStats(pContext->testId, pContext->xId, SAR_UT_INST_0, hCid);
        break;
    }

    ix_ossl_free(pCtx);
      if (pContext->pSem != NULL)
    {
        /*
         * If we're not able to give the semaphore, print the error.  Always
         * return success.
         */
        IX_ERROR_CP(ix_ossl_sem_give(*(pContext->pSem)));
    }

    return IX_SUCCESS;
}

/*****************************************************************************
 * Abstract:
 *    The callback function is called by core component infrastructure (CCI)
 *    when SAR Control module processed a message "CID update" and sent reply.
 *
 * Params:
 *    IN result - result of SAR Control operation
 *    IN hVc    - VC handle of new created VC
 *    IN pCtx   - Pointer to context of operation.
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtCallbackCidUpdate(
ix_error result,
ix_cc_atmsar_cid_handle_t hCid,
void* pCtx)
{
    sar_ut_async_context_t *pContext= (sar_ut_async_context_t*) pCtx;
    sar_ut_cid_t* pCid;
    switch (pContext->testId)
    {
    case SAR_UT_TEST_NO_MGR:
        if (pContext->xId>=SAR_UT_MGR_MAX_CID)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                    ("Cid ID %d is out of range", pContext->xId));
        }
        pCid= &sarUtMgr.cid[pContext->xId];
        if (pCid->state != SAR_UT_FSM_ST_UPDATING)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                    ("Wrong FSM State %d for CID %d",
                                     pCid->state, pContext->xId));
        }

        SarUtShowCbResult("CID UPDATE", result);
        if (result == IX_SUCCESS)
        {
            pCid->state = SAR_UT_FSM_ST_UPDATED;
            pCid->handle= hCid;
        }
        else
        {
            pCid->state = SAR_UT_FSM_ST_CREATED;
        }
        SarUt_ShowCid(pContext->xId);
        break;
    default:
        SarUtShowCbResult("CID UPDATE", result);
        SarUtCidStats(pContext->testId, pContext->xId, SAR_UT_INST_0, hCid);
        break;
    }

    ix_ossl_free(pCtx);
    if (pContext->pSem != NULL)
    {
        /*
         * If we're not able to give the semaphore, print the error.  Always
         * return success.
         */
        IX_ERROR_CP(ix_ossl_sem_give(*(pContext->pSem)));
    }

    return IX_SUCCESS;
}

/*****************************************************************************
 * Abstract:
 *    Send request to SAR Control for CID creation.
 *
 * Params:
 *    IN arg_testId: Identiefier of test
 *    IN arg_pVcDescr: pointer to structure describing requested CID
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtCidCreate(
sar_ut_test_id_t arg_testId,
ix_uint32 transId,
const ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmsar_cid_descr_t *arg_pCidDescr)
{
    ix_ossl_sem_t CmdFinishedSem;
    ix_error err=IX_SUCCESS;
    sar_ut_async_context_t *pContext;

    pContext = ix_ossl_malloc(sizeof(sar_ut_async_context_t));

    if (pContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("Context is NULL"));
    }

    memcpy(&(pContext->context.cidCreate.cidDescr),
           arg_pCidDescr,
           sizeof(ix_cc_atmsar_cid_descr_t));

    pContext->used=1;
    pContext->testId=arg_testId;
    pContext->xId=transId;

    /* Allocate the semaphore - it is released by calback functions */
    IX_ERROR_CR(ix_ossl_sem_init(IX_OSSL_SEM_UNAVAILABLE, &CmdFinishedSem));
    pContext->pSem = &CmdFinishedSem;
    _IX_CC_ATMSAR_DBG_TEST(("TEST : Send msg CID CREATE with CidNo=%d,"
                            " on VC with handle %d\n",
                            arg_pCidDescr->cid, arg_pCidDescr->vc));
    err=ix_cc_atmsar_async_cid_create(arg_sarInstance,
                                      arg_pCidDescr,
                                      SarUtCallbackCidCreate,
                                      (void*)pContext);
    /* wait until this command finishes */                                             
    ix_ossl_sem_take(CmdFinishedSem, IX_OSSL_WAIT_FOREVER);    
    IX_ERROR_CR(ix_ossl_sem_fini(CmdFinishedSem));

    if ( err != IX_SUCCESS)
    {
        err=IX_ERROR_WARNING(err, ("Send request message failed"));
        ix_ossl_free(pContext);
    }
    return err;
} /* SarUtCidCreate */


/*****************************************************************************
 * Abstract:
 *    Send request to SAR Control for CID update.
 *
 * Params:
 *    IN arg_testId: Identiefier of test
 *    IN arg_pVcDescr: pointer to structure describing requested CID
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtCidUpdate(
sar_ut_test_id_t arg_testId,
ix_uint32 transId,
ix_cc_atmsar_cid_handle_t arg_hCid,
const ix_cc_atmsar_instance_t arg_sarInstance,
const ix_cc_atmrx_update_data_t *arg_pUpdate)
{
    ix_ossl_sem_t CmdFinishedSem;
    ix_error err=IX_SUCCESS;
    sar_ut_async_context_t *pContext;

    pContext = ix_ossl_malloc(sizeof(sar_ut_async_context_t));

    if (pContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("Context is NULL"));
    }

    memcpy(&(pContext->context.cidUpdate.cidUpdate),
           arg_pUpdate,
           sizeof(ix_cc_atmrx_update_data_t));

    pContext->used=1;
    pContext->testId=arg_testId;
    pContext->xId=transId;

    /* Allocate the semaphore - it is released by calback functions */
    IX_ERROR_CR(ix_ossl_sem_init(IX_OSSL_SEM_UNAVAILABLE, &CmdFinishedSem));
    pContext->pSem = &CmdFinishedSem;
    _IX_CC_ATMSAR_DBG_TEST(("TEST : Send msg CID UPDATE with suId=%d",
                            arg_pUpdate->lw0.suId));
    err=ix_cc_atmsar_async_cid_update(arg_sarInstance,
                                      arg_hCid,
                                      (ix_cc_atmsar_update_data_t *)arg_pUpdate,
                                      SarUtCallbackCidUpdate,
                                      (void*)pContext);
    /* wait until this command finishes */
    ix_ossl_sem_take(CmdFinishedSem, IX_OSSL_WAIT_FOREVER);
    IX_ERROR_CR(ix_ossl_sem_fini(CmdFinishedSem));

    if ( err != IX_SUCCESS)
    {
        err=IX_ERROR_WARNING(err, ("Send request message failed"));
        ix_ossl_free(pContext);
    }
    return err;
} /* SarUtCidUpdate */

/*****************************************************************************
 * Abstract:
 *    The callback function. It is call by CCI, when SAR Control pelies for
 *    'CID remove' request.
 *
 * Params:
 *    IN result - result of SAR Control operation
 *    IN pCtx   - Pointer to context of operation.
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtCallbackCidRemove(
ix_error result,
void* pCtx)
{
    sar_ut_async_context_t *pContext= (sar_ut_async_context_t*) pCtx;
    sar_ut_cid_t* pCid;
    sar_ut_vc_t* pVc;
    switch (pContext->testId)
    {
    case SAR_UT_TEST_NO_MGR:
        if (pContext->xId>=SAR_UT_MGR_MAX_CID)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                    ("CID ID %d is out of range", pContext->xId));
        }
        pCid= &sarUtMgr.cid[pContext->xId];
        if (pCid->state != SAR_UT_FSM_ST_REMOVING)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                    ("Wrong FSM State %d for CID %d",
                                     pVc->state, pContext->xId));
        }

        pVc=&sarUtMgr.vc[pCid->cfg.vcId];
        pVc->cidNo--;
        SarUtShowCbResult("CID REMOVE", result);
        pCid->state = SAR_UT_FSM_ST_DEFINED;
        pCid->handle= IX_CC_ATMSAR_CIDID_INVALID;
        SarUt_ShowCid(pContext->xId);
        break;
    default:
        SarUtShowCbResult("CID REMOVE", result);
        break;
    }
    ix_ossl_free(pContext);
  if (pContext->pSem != NULL)
    {
        /*
         * If we're not able to give the semaphore, print the error.  Always
         * return success.
         */
        IX_ERROR_CP(ix_ossl_sem_give(*(pContext->pSem)));
    }

    return IX_SUCCESS;
} /* SarUtCallbackCidRemove */


/*****************************************************************************
 * Abstract:
 *    Send request to SAR Control for CID remove.
 *
 * Params:
 *    IN
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtCidRemove(
sar_ut_test_id_t arg_testId,
ix_uint32 transId,
const ix_cc_atmsar_instance_t arg_sarInstance,
ix_cc_atmsar_cid_handle_t arg_hCid)
{
    ix_ossl_sem_t CmdFinishedSem;
    ix_error err;
    sar_ut_async_context_t *pContext;

    pContext = ix_ossl_malloc(sizeof(sar_ut_async_context_t));

    if (pContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("Context is NULL"));
    }

    pContext->context.cidRemove.hCid=arg_hCid;
    pContext->used=1;
    pContext->testId=arg_testId;
    pContext->xId=transId;

    /* Allocate the semaphore - it is released by calback functions */
    IX_ERROR_CR(ix_ossl_sem_init(IX_OSSL_SEM_UNAVAILABLE, &CmdFinishedSem));
    pContext->pSem = &CmdFinishedSem;
    _IX_CC_ATMSAR_DBG_TEST(("TEST : Send msg CID REMOVE hCid=%d\n",
                            arg_hCid));
    err=ix_cc_atmsar_async_cid_remove(arg_sarInstance,
                                       arg_hCid,
                                       SarUtCallbackCidRemove,
                                       (void*)pContext);
    /* wait until this command finishes */                                             
    ix_ossl_sem_take(CmdFinishedSem, IX_OSSL_WAIT_FOREVER);    
    IX_ERROR_CR(ix_ossl_sem_fini(CmdFinishedSem));

    if ( err != IX_SUCCESS)
    {
        IX_ERROR_WARNING(err, ("Send reply message failed"));
        ix_ossl_free(pContext);
    }
    return IX_SUCCESS;
} /* SarUtCidRemove */


/*****************************************************************************
 * Abstract:
 *    The callback function. It is call by CCI, when SAR Control pelies for
 *    'CID statistic' request.
 *
 * Params:
 *    IN result - result of SAR Control operation
 *    IN pLookupResult - pointer to structure with returned statistic data.
 *    IN pCtx   - Pointer to context of operation.
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtCallbackCidStats(
ix_error result,
ix_cc_atmsar_cid_stats_t *arg_pLookupResult,
void* pCtx)
{
    sar_ut_async_context_t *pContext= (sar_ut_async_context_t*) pCtx;
    sar_ut_cid_t* pCid;
    switch (pContext->testId)
    {
    case SAR_UT_TEST_NO_MGR:
        if (pContext->xId>=SAR_UT_MGR_MAX_CID)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                    ("CID ID %d is out of range", pContext->xId));
        }
        pCid= &sarUtMgr.cid[pContext->xId];
        if (pCid->state != SAR_UT_FSM_ST_STAT_READING)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                    ("Wrong FSM State %d for CID %d",
                                     pCid->state, pContext->xId));
        }

        SarUtShowCbResult("CID STATS", result);
        pCid->state = SAR_UT_FSM_ST_UPDATED;
        SarUt_ShowCid(pContext->xId);
        _IX_CC_ATMSAR_DBG_TEST(("  RX pkts=%-8ld, cells=%-8ld, discards=%-8ld, cellsErr=%-8ld\n",
                                (ix_uint32)arg_pLookupResult->rx.packets,
                                (ix_uint32)arg_pLookupResult->rx.cells,
                                (ix_uint32)arg_pLookupResult->rx.discards,
                                (ix_uint32)arg_pLookupResult->rx.cellErrors));
        _IX_CC_ATMSAR_DBG_TEST(("  TX pkts=%-8ld, cells=%-8ld, discards=%-8ld\n",
                                (ix_uint32)arg_pLookupResult->tx.packets,
                                (ix_uint32)arg_pLookupResult->tx.cells,
                                (ix_uint32)arg_pLookupResult->tx.discards));

        break;
    default:
        SarUtShowCbResult("CID STATS", result);
        SarUtCidRemove(pContext->testId, pContext->xId, SAR_UT_INST_0,
                       pContext->context.cidStats.hCid);
        break;
    }
    ix_ossl_free(pCtx);
    if (pContext->pSem != NULL)
    {
        /*
         * If we're not able to give the semaphore, print the error.  Always
         * return success.
         */
        IX_ERROR_CP(ix_ossl_sem_give(*(pContext->pSem)));
    }
   
    return IX_SUCCESS;
} /* SarUtCallbackCidStats */


/*****************************************************************************
 * Abstract:
 *
 * Params:
 *    IN
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtCidStats(
sar_ut_test_id_t arg_testId,
ix_uint32 transId,
const ix_cc_atmsar_instance_t arg_sarInstance,
ix_cc_atmsar_cid_handle_t arg_hCid)
{
    ix_ossl_sem_t CmdFinishedSem;    
    ix_error err;
    sar_ut_async_context_t *pContext;

    pContext = ix_ossl_malloc(sizeof(sar_ut_async_context_t));
    if (pContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("Context is NULL"));
    }

    pContext->context.cidStats.hCid=arg_hCid;
    pContext->used=1;
    pContext->testId=arg_testId;
    pContext->xId=transId;

    /* Allocate the semaphore - it is released by calback functions */
    IX_ERROR_CR(ix_ossl_sem_init(IX_OSSL_SEM_UNAVAILABLE, &CmdFinishedSem));
    pContext->pSem = &CmdFinishedSem;
    _IX_CC_ATMSAR_DBG_TEST(("TEST : Send msg CID STATS hCid=%d\n", arg_hCid));
    err=ix_cc_atmsar_async_get_cid_stats(arg_sarInstance,
                                         arg_hCid,
                                         SarUtCallbackCidStats,
                                         (void*)pContext);
    /* wait until this command finishes */                                             
    ix_ossl_sem_take(CmdFinishedSem, IX_OSSL_WAIT_FOREVER);    
    IX_ERROR_CR(ix_ossl_sem_fini(CmdFinishedSem));
                                         
    if ( err != IX_SUCCESS)
    {
        IX_ERROR_WARNING(err, ("Send reply message failed"));
        ix_ossl_free(pContext);
    }
    return IX_SUCCESS;
} /* SarUtCidStats */


/*****************************************************************************
 * Abstract:
 *    Fills VC structure pointed by 'pVc' with parameters from 'pVcUt'
 *
 * Side Effects:
 *    None
 *
 * Assumptions:
 *    None
 *
 * Return Values:
 *	  IX_SUCCESS - State was changed.
 *	  IX_CC_ERROR_NULL - Internal error occured. State isn't changed.
 *
 *****************************************************************************/
ix_error
SarUtPrepareVcDescr(
sar_ut_vc_descr_t* pVcUt,      /**< description of VC */
ix_cc_atmsar_vc_descr_t* pVc) /**< structure that need to fill */
{
    sar_ut_tp_t* pTp;
    pVc->spec.portNo=pVcUt->port;
    pVc->spec.vpi=pVcUt->vpi;
    pVc->spec.vci=pVcUt->vci;
	pVc->hdrType=pVcUt->hdrType;
	pVc->l2port=pVcUt->l2port;

    if (pVcUt->tpId>=SAR_UT_TP_ID_MAX)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("Traffic Profile wrong ID"));
    }
    pTp=&(sarUtTpTab[pVcUt->tpId]);
    pVc->aalType=pVcUt->aalType;
    pVc->trafficParams.serviceClass=pTp->serviceClass;
    pVc->trafficParams.fwd.pcr=pTp->txPcr;
    pVc->trafficParams.fwd.scr=pTp->txScr;
    pVc->trafficParams.fwd.mbs = pTp->mbs;
    pVc->trafficParams.fwd.mfs = pTp->mfs;
    pVc->trafficParams.fwd.mcr = pTp->mcr;
    pVc->trafficParams.fwd.cdvt = pTp->cdvt;
    pVc->trafficParams.fwd.tagging = pTp->tagging;
    pVc->trafficParams.fwd.frmDisc = pTp->frmDisc;
    pVc->trafficParams.fwd.etoeOamF5FlInd = pTp->etoeOamF5FlInd;
    pVc->trafficParams.fwd.ubrBcs = pTp->ubrBcs;
    pVc->trafficParams.fwd.ubrPriority = pTp->ubrPriority;
    return IX_SUCCESS;
}


/*****************************************************************************
 * Abstract:
 *    Initialize SAR UNIT TEST module structures
 *
 * Params:
 *    IN
 *
 * Return Values:
 *	  IX_SUCCESS -
 *
 *****************************************************************************/
ix_error
SarUtInit()
{
    ix_uint16 i;
    if (sarUtInitFlag==1)
    {
        return IX_SUCCESS;
    }
    sarUtInitFlag=1;

    for (i=0; sarUtTpTab[i].tpId!=SAR_UT_TP_ID_MAX; i++)
    {
        sarUtTpMap[sarUtTpTab[i].tpId]=i;
    }

    return IX_SUCCESS;
}


/******************************************************************************
 ******************************************************************************
 *
 *                          SAR TEST 1
 *
 ******************************************************************************
 *****************************************************************************/

sar_ut_vc_descr_t sarUtTest1VcTab[]=
{  /*port, vpi, vci, traffic proffile id */
    {0,    0,   100, SAR_UT_TP_ID_CBR_100, 0, 200},
    {0,    0,   101, SAR_UT_TP_ID_CBR_100, 0, 201},
    {0,    0,   102, SAR_UT_TP_ID_CBR_100, 0, 202},
    {0,    0,   103, SAR_UT_TP_ID_CBR_100, 0, 203},
    {0,    0,   104, SAR_UT_TP_ID_CBR_100, 0, 204},
    {0,    0,   105, SAR_UT_TP_ID_CBR_100, 0, 205},
    {0,    0,   106, SAR_UT_TP_ID_CBR_100, 0, 206},
    {0,    0,   107, SAR_UT_TP_ID_CBR_100, 0, 207},
    {0,    0,   108, SAR_UT_TP_ID_CBR_100, 0, 208},
    {0,    0,   109, SAR_UT_TP_ID_CBR_100, 0, 209},
    {0,    0,   110, SAR_UT_TP_ID_CBR_100, 0, 210},
    {0,    0,   110, SAR_UT_TP_ID_NONE,    0, 211},
};


/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     Run test 1
 *
 * @retval IX_SUCCESS module is operating correct
 *
 * @note
 *     This function is used for unit test purpose
 *
 *****************************************************************************/
ix_error
SarUt_T1()
{
    ix_cc_atmsar_vc_descr_t vcDescr;
    ix_cc_atmsar_port_descr_t portDescr;
    ix_uint16 i;

    SarUtInit();

    portDescr.bandwidth=IX_CC_ATMSAR_PORT_BANDWIDTH_DS0;
    portDescr.portId=12345;

    for (i=0; sarUtTest1VcTab[i].tpId!=SAR_UT_TP_ID_NONE; i++)
    {
        memset(&vcDescr, 0, sizeof(vcDescr));
        SarUtPrepareVcDescr(&(sarUtTest1VcTab[i]),&vcDescr);
        SarUtVcCreate(SAR_UT_TEST_NO_1, i, SAR_UT_INST_0, &vcDescr);
    }

    for (i=0; i<2; i++)
    {
        portDescr.bandwidth=IX_CC_ATMSAR_PORT_BANDWIDTH_DS0;
        portDescr.portId=12345+i;
        SarUtPortCreate(SAR_UT_TEST_NO_1, i, SAR_UT_INST_0, &portDescr);
    }
    return IX_SUCCESS;
}

/******************************************************************************
 ******************************************************************************
 *
 *                          SAR TEST 2
 *
 ******************************************************************************
 *****************************************************************************/

sar_ut_vc_descr_t sarUtTest2VcTab[]=
{  /*port, vpi, vci, traffic proffile id */
    {0,    0,   110, SAR_UT_TP_ID_CBR_100, 0, 250},
    {0,    0,   110, SAR_UT_TP_ID_NONE, 0, 251},
};

/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     Run test 2
 *
 * @retval IX_SUCCESS module is operating correct
 *
 * @note
 *     This function is used for unit test purpose
 *
 *****************************************************************************/
ix_error
SarUt_T2()
{
    ix_cc_atmsar_vc_descr_t vcDescr;
    ix_uint16 i;

    SarUtInit();
    _IX_CC_ATMSAR_DBG_TEST(("TEST 2: Create, read stats and destroy VC\n"));

    for (i=0; sarUtTest2VcTab[i].tpId!=SAR_UT_TP_ID_NONE; i++)
    {
        memset(&vcDescr, 0, sizeof(vcDescr));
        SarUtPrepareVcDescr(&(sarUtTest2VcTab[i]),&vcDescr);
        SarUtVcCreate(SAR_UT_TEST_NO_2, i, SAR_UT_INST_0, &vcDescr);
    }

    return IX_SUCCESS;
}

/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     Run test 3
 *
 * @retval IX_SUCCESS module is operating correct
 *
 * @note
 *     This function is used for unit test purpose
 *
 *****************************************************************************/
ix_error
SarUt_T3()
{
    ix_cc_atmsar_vc_descr_t vcDescr;
    ix_uint16 i;
    ix_error result;
    sar_ut_vc_t *pVc;

    SarUtInit();

    result = ix_ossl_sem_init(IX_OSSL_SEM_AVAILABLE, &test3Sid);
    if (result != IX_SUCCESS)
    {
        return result;
    }

    for (i=1;i<(NUMBER_OF_VC-128);i++)   /* !!!mru */
    {
#ifndef CLI_SUPPORT
        SarUt_VcDef(i, 0, 0, i, 0, IX_CC_ATMSAR_AAL5);
#else
        SarUt_VcDef(SAR_UT_INST_0, i, 0, 0, i, 0, IX_CC_ATMSAR_AAL5);
#endif
        pVc=&sarUtMgr.vc[i];
        memset(&vcDescr, 0, sizeof(vcDescr));
        pVc->cidNo=0;
        pVc->state=SAR_UT_FSM_ST_CREATING;
        SarUtPrepareVcDescr(&(pVc->cfg),&vcDescr);
        SarUtVcCreate(SAR_UT_TEST_NO_3, i, SAR_UT_INST_0, &vcDescr);
#ifndef CLI_SUPPORT
        SarUt_ShowVc(i);
#else
        SarUt_ShowVc(SAR_UT_INST_0,i);
#endif

        test3VcCreateMsgSend++;

        if (test3VcCreateMsgSend % VC_BREAK_VALUE)
        {
            ix_ossl_sem_take(test3Sid, IX_OSSL_WAIT_FOREVER);
        }
    }

    return IX_SUCCESS;
}


/******************************************************************************
 ******************************************************************************
 *
 *                          SAR UT VC MANAGER
 *
 ******************************************************************************
 *****************************************************************************/

/*****************************************************************************
 * Abstract:
 *    Initialize SAR UNIT VC MANAGER sub module
 *
 * Return Values:
 *	  none
 *
 *****************************************************************************/
void
SarUtMgrInit()
{
    ix_uint16 i;
    if (sarUtMgrInit!=0)
    {
        return;
    }
    sarUtMgrInit=1;
    memset(&sarUtMgr, 0, sizeof(sarUtMgr));

    for (i=0; i<SAR_UT_MGR_MAX_VC; i++)
    {
        sarUtMgr.vc[i].state=SAR_UT_FSM_ST_UNDEF;
        sarUtMgr.vc[i].handle=IX_CC_ATMSAR_VCID_INVALID;
    }
    for (i=0; i<SAR_UT_MGR_MAX_CID; i++)
    {
        sarUtMgr.cid[i].state=SAR_UT_FSM_ST_UNDEF;
        sarUtMgr.cid[i].handle=IX_CC_ATMSAR_CIDID_INVALID;
    }
}


/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     The function shows all parameters of VC defined in SAR UT VC MANAGER
 *
 * @param IN vcId - user identifier of VC that user wants to show
 *
 *****************************************************************************/
void
SarUt_ShowVc(
#ifdef CLI_SUPPORT
ix_int16 direction,
#endif
ix_uint16 vcId)
{
    sar_ut_vc_t *pVc;
    if (vcId>=SAR_UT_MGR_MAX_VC)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: Wrong VC ID %d\n", vcId));
        return;
    }
    pVc=&sarUtMgr.vc[vcId];

    _IX_CC_ATMSAR_DBG_TEST(("VC[%-4d] CFG: port:%-4d vpi:%-4d vci:%-4d, tpId:%d\n",
                            vcId, pVc->cfg.port, pVc->cfg.vpi, pVc->cfg.vci,
                            pVc->cfg.tpId));
    _IX_CC_ATMSAR_DBG_TEST(("         handle: %d, State:%s\n",
                            pVc->handle, SarUtGetTxtStateDescr(pVc->state)));
}


/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     The function prepares a new VC description in
 *
 * @retval IX_SUCCESS module is operating correct
 *
 * @note
 *     This function is used for unit test purpose
 *
 *****************************************************************************/
void
SarUt_VcDef(
#ifdef CLI_SUPPORT
ix_uint16 direction,
#endif
ix_uint16  vcId,
ix_uint16  port,
ix_uint16  vpi,
ix_uint16  vci,
ix_uint16  tpId,
ix_uint16 vctype)
{
    sar_ut_vc_t *pVc;
    SarUtMgrInit();

    if (vcId>=SAR_UT_MGR_MAX_VC)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: Wrong VC ID %d\n", vcId));
        return;
    }
    if (tpId>=SAR_UT_TP_ID_MAX)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: Wrong Traffic proffile ID %d\n", tpId));
        return;
    }
    pVc=&sarUtMgr.vc[vcId];
    if (pVc->state==SAR_UT_FSM_ST_CREATING)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: VC %d during CREATE action\n", vcId));
        return;
    }
    if (pVc->state==SAR_UT_FSM_ST_STAT_READING)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: VC %d during GET STATS action\n", vcId));
        return;
    }
    if (pVc->state==SAR_UT_FSM_ST_REMOVING)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: VC %d during CREATE action\n", vcId));
        return;
    }
    pVc->state=SAR_UT_FSM_ST_DEFINED;
    pVc->cfg.port=port;
    pVc->cfg.vpi=vpi;
    pVc->cfg.vci=vci;
    pVc->cfg.tpId=tpId;
    pVc->cfg.hdrType=0;
    pVc->cfg.l2port=0;
    pVc->cfg.aalType=vctype;
#ifdef CLI_SUPPORT
    pVc->direction = direction;
    SarUt_ShowVc(direction,vcId);
#else
    SarUt_ShowVc(vcId);
#endif
}


/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     The function calls create VC for defined VC
 *
 * @retval IX_SUCCESS module is operating correct
 *
 * @note
 *     This function is used for unit test purpose
 *
 *****************************************************************************/
void
SarUt_VcCreate(
ix_uint16  vcId)
{
    ix_cc_atmsar_vc_descr_t vcDescr;
    sar_ut_vc_t *pVc;
    SarUtMgrInit();

    if (vcId>=SAR_UT_MGR_MAX_VC)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: Wrong VC ID %d\n", vcId));
        return;
    }

    pVc=&sarUtMgr.vc[vcId];
    if (pVc->state==SAR_UT_FSM_ST_UNDEF)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: Undefined parameters for VC %d\n", vcId));
        return;
    }

    if (pVc->state==SAR_UT_FSM_ST_CREATING)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: VC %d during CREATE action\n", vcId));
        return;
    }
    if (pVc->state==SAR_UT_FSM_ST_STAT_READING)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: VC %d during GET STATS action\n", vcId));
        return;
    }
    if (pVc->state==SAR_UT_FSM_ST_REMOVING)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: VC %d during CREATE action\n", vcId));
        return;
    }
    memset(&vcDescr, 0, sizeof(vcDescr));
    pVc->cidNo=0;
    pVc->state=SAR_UT_FSM_ST_CREATING;
    SarUtPrepareVcDescr(&(pVc->cfg),&vcDescr);
#ifndef CLI_SUPPORT
    SarUtVcCreate(SAR_UT_TEST_NO_MGR, vcId, SAR_UT_INST_0, &vcDescr);
    SarUt_ShowVc(vcId);
#else
    SarUtVcCreate(SAR_UT_TEST_NO_MGR, vcId, pVc->direction, &vcDescr);
    SarUt_ShowVc(pVc->direction, vcId);
#endif
}

/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     The function calls create VC for defined VC
 *
 * @retval IX_SUCCESS module is operating correct
 *
 * @note
 *     This function is used for unit test purpose
 *
 *****************************************************************************/
void
SarUt_VcUpdate(
ix_uint16  vcId,
ix_uint8  hdrType,
ix_uint16 l2port)
{
    ix_cc_atmsar_update_data_t sarUpdateData;
    ix_cc_atmsar_vc_descr_t vcDescr;
    ix_cc_atmrx_update_data_t rxUpdateData;
    sar_ut_vc_t *pVc;
    SarUtMgrInit();

    if (vcId>=SAR_UT_MGR_MAX_VC)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: Wrong VC ID %d\n", vcId));
        return;
    }

    pVc=&sarUtMgr.vc[vcId];
    if (pVc->state==SAR_UT_FSM_ST_UNDEF)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: Undefined parameters for VC %d\n", vcId));
        return;
    }

    if (pVc->state==SAR_UT_FSM_ST_CREATING)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: VC %d during CREATE action\n", vcId));
        return;
    }
    if (pVc->state==SAR_UT_FSM_ST_STAT_READING)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: VC %d during GET STATS action\n", vcId));
        return;
    }
    if (pVc->state==SAR_UT_FSM_ST_REMOVING)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: VC %d during CREATE action\n", vcId));
        return;
    }

    pVc->cfg.hdrType=hdrType;
    pVc->cfg.l2port=l2port;
    memset(&vcDescr, 0, sizeof(vcDescr));
    pVc->state=SAR_UT_FSM_ST_UPDATING;
    SarUtPrepareVcDescr(&(pVc->cfg),&vcDescr);

    /* store l2port and header type in the "VC UPDATE" message */
    rxUpdateData.lw0.l2port = l2port;
    rxUpdateData.lw1.hdrType = hdrType;
    memcpy(&sarUpdateData,&rxUpdateData,sizeof(ix_cc_atmsar_update_data_t));

#ifndef CLI_SUPPORT
    SarUtVcUpdate(SAR_UT_TEST_NO_MGR, vcId, SAR_UT_INST_0, pVc->handle, &sarUpdateData);
    SarUt_ShowVc(vcId);
#else
    SarUtVcUpdate(SAR_UT_TEST_NO_MGR, vcId, pVc->direction, pVc->handle, &sarUpdateData);
    SarUt_ShowVc(pVc->direction, vcId);
#endif
}

/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     The function calls get stats  VC for created VC
 *
 *****************************************************************************/
void
SarUt_VcGetStats(
#ifdef CLI_SUPPORT
ix_int16 direction,
#endif
ix_uint16  vcId)
{
    sar_ut_vc_t *pVc;
    SarUtMgrInit();

    if (vcId>=SAR_UT_MGR_MAX_VC)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: Wrong VC ID %d\n", vcId));
        return;
    }

    pVc=&sarUtMgr.vc[vcId];

    if (pVc->state!=SAR_UT_FSM_ST_UPDATED)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: Wrong state VC %d\n", vcId));
        return;
    }
    pVc->state=SAR_UT_FSM_ST_STAT_READING;
#ifndef CLI_SUPPORT
    SarUtVcStats(SAR_UT_TEST_NO_MGR, vcId, SAR_UT_INST_0, pVc->handle);
    SarUt_ShowVc(vcId);
#else
    SarUtVcStats(SAR_UT_TEST_NO_MGR, vcId, direction, pVc->handle);
    SarUt_ShowVc(direction,vcId);
#endif
}


/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     The function calls get stats  VC for created VC
 *
 *****************************************************************************/
void
SarUt_VcRemove(
#ifdef CLI_SUPPORT
ix_int16 direction,
#endif
ix_uint16 vcId)
{
    sar_ut_vc_t *pVc;
    SarUtMgrInit();

    if (vcId>=SAR_UT_MGR_MAX_VC)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: Wrong VC ID %d\n", vcId));
        return;
    }

    pVc=&sarUtMgr.vc[vcId];

    if (!((pVc->state==SAR_UT_FSM_ST_CREATED) || (pVc->state==SAR_UT_FSM_ST_UPDATED)))
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: Wrong state VC %d\n", vcId));
        return;
    }
    if (pVc->cidNo != 0)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: Cannot remove VC because CIDs are assigned to the VC\n"));
        return;
    }

    pVc->state=SAR_UT_FSM_ST_REMOVING;
#ifndef CLI_SUPPORT
    SarUtVcRemove(SAR_UT_TEST_NO_MGR, vcId, SAR_UT_INST_0, pVc->handle);
    SarUt_ShowVc(vcId);
#else
    SarUtVcRemove(SAR_UT_TEST_NO_MGR, vcId, direction, pVc->handle);
    SarUt_ShowVc(direction,vcId);
#endif
}

/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     The function creates a new CBR traffic profile.
 *
 *****************************************************************************/
ix_error
SarUt_TPCBRAdd(
ix_uint8 tpId,
ix_uint32 pcr,
ix_uint32 cdvt)
{
    if ( (sarUtTpTab[tpId].serviceClass != IX_CC_ATMSAR_ATM_SERVICE_NONE) ||
         (tpId < staticLastProfile) || (tpId > SAR_UT_TP_ID_MAX) )
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR UT: Traffic profile identifier (%d)  is already used.\n", tpId));

        return (-1);
    }

    memset((void*)&sarUtTpTab[tpId],0,sizeof(sar_ut_tp_t));

    sarUtTpTab[tpId].tpId = tpId;
    sarUtTpTab[tpId].serviceClass = IX_CC_ATMSAR_ATM_SERVICE_CBR;
    sarUtTpTab[tpId].txPcr = pcr;
    sarUtTpTab[tpId].cdvt = cdvt;

    _IX_CC_ATMSAR_DBG_TEST(("SAR UT: CBR Traffic Profile added.\n Traffic Profile Id: %d\n"
                            "     PCR  = 0x%x\n"
                            "     CDVT = 0x%x\n\n",
                            tpId,
                            pcr,
                            cdvt));

    return (IX_SUCCESS);
}

/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     The function creates a new rtVBR traffic profile.
 *
 *****************************************************************************/
ix_error SarUt_TPRTVBRAdd(
ix_uint8 tpId,
ix_uint32 pcr,
ix_uint32 scr,
ix_uint32 cdvt,
ix_uint32 mbs)
{
    if ( (sarUtTpTab[tpId].serviceClass != IX_CC_ATMSAR_ATM_SERVICE_NONE) ||
         (tpId < staticLastProfile) || (tpId > SAR_UT_TP_ID_MAX) )
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR UT: Traffic profile identifier (%d)  is already used...\n", tpId));

        return (-1);
    }

    memset((void*)&sarUtTpTab[tpId],0,sizeof(sar_ut_tp_t));

    sarUtTpTab[tpId].tpId = tpId;
    sarUtTpTab[tpId].serviceClass = IX_CC_ATMSAR_ATM_SERVICE_RT_VBR;
    sarUtTpTab[tpId].txPcr = pcr;
    sarUtTpTab[tpId].txScr = scr;
    sarUtTpTab[tpId].cdvt = cdvt;
    sarUtTpTab[tpId].mbs = mbs;

    _IX_CC_ATMSAR_DBG_TEST(("SAR UT: rtVBR Traffic Profile added.\n Traffic Profile Id: %d\n"
                            "    pcr   = 0x%x\n"
                            "    scr   = 0x%x\n"
                            "    cdvt  = 0x%x\n"
                            "    mbs   = 0x%x\n",
                            tpId,
                            pcr,
                            scr,
                            cdvt,
                            mbs));

    return (IX_SUCCESS);
}

/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     The function creates a new rtVBR traffic profile.
 *
 *****************************************************************************/
ix_error SarUt_TPNRTVBRAdd(
ix_uint8 tpId,
ix_uint32 pcr,
ix_uint32 scr,
ix_uint32 cdvt,
ix_uint32 mbs)
{
    if ( (sarUtTpTab[tpId].serviceClass != IX_CC_ATMSAR_ATM_SERVICE_NONE) ||
         (tpId < staticLastProfile) || (tpId > SAR_UT_TP_ID_MAX) )
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR UT: Traffic profile identifier (%d)  is already used...\n", tpId));

        return (-1);
    }

    memset((void*)&sarUtTpTab[tpId],0,sizeof(sar_ut_tp_t));

    sarUtTpTab[tpId].tpId = tpId;
    sarUtTpTab[tpId].serviceClass = IX_CC_ATMSAR_ATM_SERVICE_NRT_VBR;
    sarUtTpTab[tpId].txPcr = pcr;
    sarUtTpTab[tpId].txScr = scr;
    sarUtTpTab[tpId].cdvt = cdvt;
    sarUtTpTab[tpId].mbs = mbs;

    _IX_CC_ATMSAR_DBG_TEST(("SAR UT: nrtVBR Traffic Profile added.\n Traffic Profile Id: %d\n"
                            "    pcr   = 0x%x\n"
                            "    scr   = 0x%x\n"
                            "    cdvt  = 0x%x\n"
                            "    mbs   = 0x%x\n",
                            tpId,
                            pcr,
                            scr,
                            cdvt,
                            mbs));

    return (IX_SUCCESS);
}

/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     The function creates a new GFR  traffic profile.
 *
 *****************************************************************************/
ix_error SarUt_TPGFRAdd(
ix_uint8 tpId,
ix_uint32 pcr,
ix_uint32 mcr,
ix_uint32 cdvt,
ix_uint32 mbs)
{
    if ( (sarUtTpTab[tpId].serviceClass != IX_CC_ATMSAR_ATM_SERVICE_NONE) ||
         (tpId < staticLastProfile) || (tpId > SAR_UT_TP_ID_MAX) )
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR UT: Traffic profile identifier (%d)  is already used.\n", tpId));

        return (-1);
    }

    memset((void*)&sarUtTpTab[tpId],0,sizeof(sar_ut_tp_t));

    sarUtTpTab[tpId].tpId = tpId;
    sarUtTpTab[tpId].serviceClass = IX_CC_ATMSAR_ATM_SERVICE_GFR;
    sarUtTpTab[tpId].txPcr = pcr;
    sarUtTpTab[tpId].mcr = mcr;
    sarUtTpTab[tpId].cdvt = cdvt;
    sarUtTpTab[tpId].mbs = mbs;

    _IX_CC_ATMSAR_DBG_TEST(("SAR UT: GFR Traffic Profile added.\n Traffic Profile Id: %d\n"
                            "    pcr   = 0x%x\n"
                            "    mcr   = 0x%x\n"
                            "    cdvt  = 0x%x\n"
                            "    mbs   = 0x%x\n",
                            tpId,
                            pcr,
                            mcr,
                            cdvt,
                            mbs));

    return (IX_SUCCESS);
}


/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     The function creates a new UBR traffic profile.
 *
 *****************************************************************************/
ix_error SarUt_TPUBRAdd(
ix_uint8 tpId,
ix_uint32 ubrBcs,
ix_uint32 ubrPriority,
ix_uint32 pcr,
ix_uint32 cdvt)
{
    if ( (sarUtTpTab[tpId].serviceClass != IX_CC_ATMSAR_ATM_SERVICE_NONE) ||
         (tpId < staticLastProfile) || (tpId > SAR_UT_TP_ID_MAX) )
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR UT: Traffic profile identifier (%d)  is already used...\n", tpId));

        return (-1);
    }

    memset((void*)&sarUtTpTab[tpId],0,sizeof(sar_ut_tp_t));

    sarUtTpTab[tpId].tpId = tpId;
    sarUtTpTab[tpId].serviceClass = IX_CC_ATMSAR_ATM_SERVICE_UBR;
    sarUtTpTab[tpId].ubrBcs = ubrBcs;
    sarUtTpTab[tpId].ubrPriority = ubrPriority;
    sarUtTpTab[tpId].txPcr = pcr;
    sarUtTpTab[tpId].cdvt = cdvt;

    _IX_CC_ATMSAR_DBG_TEST(("SAR UT: UBR Traffic Profile added.\n Traffic Profile Id: %d\n"
                            "    ubrBsc      = 0x%x\n"
                            "    ubrPriority = 0x%x\n"
                            "    pcr         = 0x%x\n"
                        "    cdvt        = 0x%x\n",
                            tpId,
                            ubrBcs,
                            ubrPriority,
                            pcr,
                            cdvt));

    return (IX_SUCCESS);
}

/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     The function removes a defined traffic profile.
 *
 *****************************************************************************/
ix_error SarUt_RemoveTP(
ix_uint8 tpId)
{
    if ((tpId < staticLastProfile) || (tpId > SAR_UT_TP_ID_MAX))
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR UT: Traffic profile identifier (%d)  is out of range.\n", tpId));

        return (-1);
    }

    memset((void*)&sarUtTpTab[tpId],0,sizeof(sar_ut_tp_t));

    _IX_CC_ATMSAR_DBG_TEST(("SAR UT: Traffic profile identifier (%d) removed.\n",
                            tpId));

    return (IX_SUCCESS);
}



/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     The function calls get statistic for port
 *
 *****************************************************************************/
void
SarUt_PortGetStats(
#ifdef CLI_SUPPORT
ix_int16 direction,
#endif
ix_uint16  portId)
{
    SarUtMgrInit();

    if (portId>=MAX_ATM_PORTS)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: Wrong Port ID %d\n", portId));
        return;
    }
#ifndef CLI_SUPPORT
    SarUtPortStats(SAR_UT_TEST_NO_MGR, portId, SAR_UT_INST_0, portId);
#else
    SarUtPortStats(SAR_UT_TEST_NO_MGR, portId, direction, portId);
#endif
}



/*****************************************************************************
 * Abstract:
 *    Fills CID structure pointed by 'pCid' with parameters from 'pCidUt'
 *
 * Side Effects:
 *    None
 *
 * Assumptions:
 *    None
 *
 * Return Values:
 *	  IX_SUCCESS - State was changed.
 *	  IX_CC_ERROR_NULL - Internal error occured. State isn't changed.
 *
 *****************************************************************************/
ix_error
SarUtPrepareCidDescr(
sar_ut_cid_descr_t* pCidUt,      /**< description of CID */
ix_cc_atmsar_cid_descr_t* pCid) /**< structure that need to fill */
{
    sar_ut_vc_t *pVc;

    pVc=&sarUtMgr.vc[pCidUt->vcId];
    switch (pVc->state)
    {
    case SAR_UT_FSM_ST_UNDEF:
    case SAR_UT_FSM_ST_DEFINED:
    case SAR_UT_FSM_ST_CREATING:
    case SAR_UT_FSM_ST_REMOVING:
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("VC is not exist, current state of VC is %s",
                                                   SarUtGetTxtStateDescr(pVc->state)));

    case SAR_UT_FSM_ST_CREATED:
    case SAR_UT_FSM_ST_STAT_READING:
    default:
        break;
    }

    pCid->vc=pVc->handle;
    pCid->cid=pCidUt->cidNo;
    return IX_SUCCESS;
}

/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     The function shows all parameters of CID defined in SAR UT MANAGER
 *
 * @param IN cidId - user identifier of CID that user wants to show
 *
 *****************************************************************************/
void
SarUt_ShowCid(
ix_uint16 cidId)
{
    sar_ut_cid_t *pCid;
    if (cidId>=SAR_UT_MGR_MAX_CID)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: Wrong CID ID %d\n", cidId));
        return;
    }
    pCid=&sarUtMgr.cid[cidId];

    _IX_CC_ATMSAR_DBG_TEST(("VC[%-4d] CFG: cidNo:%-4d vcID:%-4d, vcHandle:%-4d\n",
                            cidId, pCid->cfg.cidNo, pCid->cfg.vcId,
                            sarUtMgr.vc[pCid->cfg.vcId].handle));
    _IX_CC_ATMSAR_DBG_TEST(("         handle: %d, State:%s\n",
                            pCid->handle, SarUtGetTxtStateDescr(pCid->state)));
}


/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     The function prepares a new CID description in SAR UT Manager
 *
 * @note
 *     This function is used for unit test purpose
 *
 *****************************************************************************/
void
SarUt_CidDef(
ix_uint16 cidId,   /* CID ID used in SAR UT Manager  */
ix_uint16 vcId,    /* VC ID in SAR UT manager        */
ix_uint8 cidNo)    /* CID in VC pointed by vcId      */
{
    sar_ut_cid_t *pCid;
    SarUtMgrInit();

    if (cidId>=SAR_UT_MGR_MAX_CID)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: Wrong CID ID %d\n", cidId));
        return;
    }
    pCid=&sarUtMgr.cid[cidId];
    switch (pCid->state)
    {
    case SAR_UT_FSM_ST_CREATING:
    case SAR_UT_FSM_ST_STAT_READING:
    case SAR_UT_FSM_ST_REMOVING:
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: CID %d in state %s\n", cidId,
                                SarUtGetTxtStateDescr(pCid->state)));
        return;
    default:
        break;
    }
    pCid->state=SAR_UT_FSM_ST_DEFINED;
    pCid->cfg.vcId=vcId;
    pCid->cfg.cidNo=cidNo;
    SarUt_ShowCid(cidId);
}


/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     The function calls create CID for defined CID
 *
 * @retval IX_SUCCESS module is operating correct
 *
 * @note
 *     This function is used for unit test purpose
 *
 *****************************************************************************/
void
SarUt_CidCreate(
ix_uint16 cidId)
{
    ix_cc_atmsar_cid_descr_t cidDescr;
    sar_ut_cid_t *pCid;
    sar_ut_vc_t *pVc;
    ix_error err;

    SarUtMgrInit();

    if (cidId>=SAR_UT_MGR_MAX_CID)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: Wrong CID ID %d\n", cidId));
        return;
    }

    pCid=&sarUtMgr.cid[cidId];
    switch (pCid->state)
    {
    case SAR_UT_FSM_ST_UNDEF:
    case SAR_UT_FSM_ST_CREATING:
    case SAR_UT_FSM_ST_STAT_READING:
    case SAR_UT_FSM_ST_REMOVING:
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: CID %d in state %s\n", cidId,
                            SarUtGetTxtStateDescr(pCid->state)));
        return;
    default:
        break;
    }
    memset(&cidDescr, 0, sizeof(cidDescr));
    err=SarUtPrepareCidDescr(&(pCid->cfg),&cidDescr);
    if (err != IX_SUCCESS)
    {
        ix_error_dump(stderr, err);
        return;
    }
    pCid->state=SAR_UT_FSM_ST_CREATING;
    pVc=&sarUtMgr.vc[pCid->cfg.vcId];
    pVc->cidNo++;
    err=SarUtCidCreate(SAR_UT_TEST_NO_MGR, cidId, SAR_UT_INST_0, &cidDescr);
    if (err != IX_SUCCESS)
    {
        ix_error_dump(stderr, err);
        return;
    }
    SarUt_ShowCid(cidId);
}

/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     The function calls update CID for defined CID
 *
 * @retval IX_SUCCESS module is operating correct
 *
 * @note
 *     This function is used for unit test purpose
 *
 *****************************************************************************/
void
SarUt_CidUpdate(
ix_uint16 cidId,
ix_uint32 suId)
{
    ix_cc_atmrx_update_data_t rxUpdateData;
    sar_ut_cid_t *pCid;
    sar_ut_vc_t *pVc;
    ix_error err;

    if (cidId>=SAR_UT_MGR_MAX_CID)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: Wrong CID ID %d\n", cidId));
        return;
    }

    pCid=&sarUtMgr.cid[cidId];
    switch (pCid->state)
    {
    case SAR_UT_FSM_ST_UNDEF:
    case SAR_UT_FSM_ST_CREATING:
    case SAR_UT_FSM_ST_STAT_READING:
    case SAR_UT_FSM_ST_REMOVING:
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: CID %d in state %s\n", cidId,
                            SarUtGetTxtStateDescr(pCid->state)));
        return;
    default:
        break;
    }

    pCid->state=SAR_UT_FSM_ST_UPDATING;
    pVc=&sarUtMgr.vc[pCid->cfg.vcId];

    rxUpdateData.lw0.suId = suId;
    rxUpdateData.lw1.optionalInfo = 0;

    err=SarUtCidUpdate(SAR_UT_TEST_NO_MGR,
                       cidId,
                       pCid->handle,
                       SAR_UT_INST_0,
                       &rxUpdateData);
    if (err != IX_SUCCESS)
    {
        ix_error_dump(stderr, err);
        return;
    }

    SarUt_ShowCid(cidId);
}

/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     The function calls get stats CID for created CID
 *
 *****************************************************************************/
void
SarUt_CidGetStats(
ix_uint16 cidId)
{
    sar_ut_cid_t *pCid;
    SarUtMgrInit();

    if (cidId>=SAR_UT_MGR_MAX_CID)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: Wrong CID ID %d\n", cidId));
        return;
    }

    pCid=&sarUtMgr.cid[cidId];

    if (pCid->state!=SAR_UT_FSM_ST_UPDATED)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: Wrong state CID %d\n", cidId));
        return;
    }
    pCid->state=SAR_UT_FSM_ST_STAT_READING;
    SarUtCidStats(SAR_UT_TEST_NO_MGR, cidId, SAR_UT_INST_0, pCid->handle);
    SarUt_ShowCid(cidId);
}


/** @ingroup SarUt ***********************************************************
 *
 * @description
 *     The function calls remove CID for created CID
 *
 *****************************************************************************/
void
SarUt_CidRemove(
ix_uint16 cidId)
{
    sar_ut_cid_t *pCid;
    SarUtMgrInit();

    if (cidId>=SAR_UT_MGR_MAX_CID)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: Wrong CID ID %d\n", cidId));
        return;
    }

    pCid=&sarUtMgr.cid[cidId];

    if (pCid->state!=SAR_UT_FSM_ST_CREATED)
    {
        _IX_CC_ATMSAR_DBG_TEST(("SAR_UT: CID %d has wrong state: %s\n",
                                cidId, SarUtGetTxtStateDescr(pCid->state)));
        return;
    }
    pCid->state=SAR_UT_FSM_ST_REMOVING;
    SarUtCidRemove(SAR_UT_TEST_NO_MGR, cidId, SAR_UT_INST_0, pCid->handle);
    SarUt_ShowCid(cidId);
}

/******************************************************************************
 *                   Function for API testing
 *****************************************************************************/


/******************************************************************************
 *                   DEINICJALIZATION/SHUTDOWN FUNCTIONS
 *****************************************************************************/


/******************************************************************************
 *                         INITIALISATION FUNCTION
 *****************************************************************************/


/******************************************************************************
 *                CORE COMPONENT INFRASTRUCTURE FUNCTIONS
 *****************************************************************************/

/** @ingroup SarUt ***********************************************************
 *
 * @description
 *    This primitive shall be called and returned successfully before
 *    requesting any service from this core component. This primitive should be
 *    called only once to initialize this core component.This function performs
 *    allocation of memory for symbols to be patched,Creation of 64 bit
 *    counters, registration of packet and message handles, allocation and
 *    initialization of internal data structures.
 *
 * @Param:  - IN arg_hCcHandle - handle to this core component, created by core
 *            component infrastructure; This shall be used later to get other
 *            (to add event handle) services from CCI.
 * @Param:  - IN-OUT arg_ppContext - Buffer free list and dynamic property data
 *            will be given during by system APPs during initialization. Upon
 *            successfull completion, it represents the location where the
 *            pointer to the control block allocated by this core component
 *            will be stored. The control block is internal to the core
 *            component and it contains information about internal data
 *            structures, allocated memory and other relevant information.
 *            Later this pointer to be passed into core component fini()
 *            function for freeing memory and other house keeping operations
 *            when this component is being destroyed.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 * @retval IX_CC_ERROR_NULL - input parameter arg_ppContext is null
 * @retval IX_CC_ERROR_OOM   - memory allocation failure
 * @retval IX_CC_ATMSARTEST_ERROR_FAILED_PATCHING   - patching failures
 * @retval IX_CC_ERROR_OOM_64BIT_COUNTER - 64 bit counter creation failure
 * @retval IX_CC_ATMSARTEST_ERROR_REGISTRY  - registry access failures
 * @retval IX_CC_ATMSARTEST_ERROR_RTM - failure from RTM core component
 * @retval IX_CC_ATMSARTEST_ERROR_CCI - failure from CCI
 *
 *****************************************************************************/
ix_error
ix_cc_atmsartest_init(
ix_cc_handle arg_hCcHandle,
void** arg_ppContext)
{
    ix_error err = IX_SUCCESS;
    sar_ut_context_t* pContext;

    /* Variables used in retrieving static data from arg_ppContext */
    ix_cc_init_context *pInitCtx;

    /* Check for argument validity */
    _IX_CC_ATMSAR_DBG_TEST(("ix_cc_atmsartest_init() START\n"));
    if (arg_ppContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("Input parameter(arg_ppContext) is null"));
    }/* end if (arg_ppContext == NULL) */

    /* Allocate memory for core component context */
    pContext = (sar_ut_context_t*)ix_ossl_malloc(sizeof(sar_ut_context_t));
    if (pContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,
                                ("Failed to allocate memory for L2 Bridge context"));
    }/* end if (pContext == NULL) */

    /* initialize the structure */
    ix_ossl_memset(pContext, 0, sizeof(sar_ut_context_t));

    /* store core component handle in sar_ut_context_t */
    pContext->hAtmSarTestHandle = arg_hCcHandle;

    /* Get the generic init context */
    pInitCtx = (ix_cc_init_context *)*arg_ppContext;

    /* Get mutex from OSSL */
    err = ix_ossl_mutex_init(IX_OSSL_MUTEX_UNLOCK,&(pContext->AtmSarTestMutex));
    if (err != IX_SUCCESS)
    {
        err = IX_ERROR_WARNING(IX_CC_ERROR_OOR, ("Failed to create mutext"));
        goto cleanupMutex;
    }/* end if (err != IX_SUCCESS) */

    *arg_ppContext = (void *)pContext;
    /* for testing */
    s_pAtmsartestContext=pContext;
    _IX_CC_ATMSAR_DBG_TEST(("ix_cc_atmsartest_init() SUCCESS\n"));
    return IX_SUCCESS;

  cleanupMutex:
    IX_ERROR_CR(ix_ossl_mutex_fini(pContext->AtmSarTestMutex));

    ix_ossl_free(pContext);
    return err;
} /* end ix_cc_atmsartest_init */


/** @ingroup SarUt ***********************************************************
 *
 * @description
 *    This primitive shall be called to terminate services from this core
 *    component. This primitive frees memory allocated during initialization,
 *    free all the created resources: delete 64 bit counter. If client claims
 *    for any services from the core component after calling this function,
 *    then the behavior is undefined.
 *
 * @Param:  - IN arg_hCcHandle - handle to the core component
 * @Param:  - IN arg_pContext - pointer to the control block memory allocated
 *                   earlier in ix_cc_atmsartest_init function.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 * @retval IX_CC_ATMSARTEST_ERROR_INVALID_INPUT_PARAM - invalid input parameter(s)
 * @retval IX_CC_ERROR _OOM - failure of memory deletion
 * @retval IX_CC_ERROR_OOM_64BIT_COUNTER - failure of  64 bit counter deletion
 * @retval IX_CC_ATMSARTEST_ERROR _RTM - failure from RTM
 * @retval IX_CC_ATMSARTEST_ERROR_CCI - failure from CCI
 *
 *****************************************************************************/
ix_error
ix_cc_atmsartest_fini(
ix_cc_handle arg_hCcHandle,
void* arg_pContext)
{
    sar_ut_context_t* pContext= (sar_ut_context_t*)arg_pContext;
    /* Validate Parameters */
    if (arg_pContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("arg_pContext is null"));
    }/* if (arg_pContext == NULL)  */

    if (arg_pContext != s_pAtmsartestContext)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("arg_pContext is different than global test variable"));

    }/* if (arg_pContext != s_pAtmsartestContext) */

    if (arg_hCcHandle != pContext->hAtmSarTestHandle)
    {
        return IX_ERROR_WARNING(IX_CC_ATMSAR_ERROR_INVALID_INPUT_PARAM,
                                ("Invalid input Parameters"));
    }/* (arg_hCcHandle != pContext->hAtmsartestHandle) */

    /* Free mutex */
    IX_ERROR_CR(ix_ossl_mutex_fini(pContext->AtmSarTestMutex));

    /* Free Atmsartest Context */
    ix_ossl_free(pContext);
    s_pAtmsartestContext=NULL;
    return IX_SUCCESS;
} /* end ix_cc_atmsartest_fini */

