//////////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: ipv6.h
//
//      Purpose: This file defines all the generic IPv6 definitions.
//
//      History:
//      Date            Comment                                     By
// -----------------------------------------------------------------------------
//      04/12/2002      Created
// -----------------------------------------------------------------------------

#ifndef    _IPV6_H_
#define    _IPV6_H_

//----------------------------------------------
//    byte offset,size of IPv6 header fields. 
//----------------------------------------------
// From RFC 2460
//
//   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
//   |Version| Traffic Class |           Flow Label                  |
//   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
//   |         Payload Length        |  Next Header  |   Hop Limit   |
//   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
//   |                                                               |
//   +                                                               +
//   |                                                               |
//   +                         Source Address                        +
//   |                                                               |
//   +                                                               +
//   |                                                               |
//   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
//   |                                                               |
//   +                                                               +
//   |                                                               |
//   +                      Destination Address                      +
//   |                                                               |
//   +                                                               +
//   |                                                               |
//   +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
//
//   Version              4-bit Internet Protocol version number = 6.
//
//   Traffic Class        8-bit traffic class field.  See section 7.
//
//   Flow Label           20-bit flow label.  See section 6.
//
//   Payload Length       16-bit unsigned integer.  Length of the IPv6
//                        payload, i.e., the rest of the packet following
//                        this IPv6 header, in octets.  (Note that any
//
//                        extension headers [section 4] present are
//                        considered part of the payload, i.e., included
//                        in the length count.)
//
//   Next Header          8-bit selector.  Identifies the type of header
//                        immediately following the IPv6 header.  Uses the
//                        same values as the IPv4 Protocol field [RFC-1700
//                        et seq.].
//
//   Hop Limit            8-bit unsigned integer.  Decremented by 1 by
//                        each node that forwards the packet. The packet
//                        is discarded if Hop Limit is decremented to
//                        zero.
//
//   Source Address       128-bit address of the originator of the packet.
//                        See [ADDRARCH].
//
//   Destination Address  128-bit address of the intended recipient of the
//                        packet (possibly not the ultimate recipient, if
//                        a Routing header is present).  See [ADDRARCH]
//                        and section 4.4.
//
//
//  IXP2XXX Mapping 
//
//  Byte 0       1        2        3        4       5         6       7
// |--------|--------|--------|--------|--------|--------|--------|--------|
//
//  Long Word        0                                   1
// |-----------------------------------|-----------------------------------|
//
//  Quad Word
// |-----------------------------------------------------------------------|
//
//
//  -----------------------------------------------------------------------
// |Ver| Tfc Cls|     Flow  Label      | Payload Length  | NextHdr| HopLmt |
// |-----------------------------------------------------------------------|
// |                          Source IPv6 Addr                             |
// |                                                                       |
// |-----------------------------------------------------------------------|
// |                       Destination IPv6 Addr                           |
// |                                                                       |
// |-----------------------------------------------------------------------|
// |                                                                       |
// |                                 DATA                                  |
// |                                .......                                |
//
//


//
// Starting byte and count for IPv6 header fields.  These are used primarily
// by microC.

#define IPV6_VERSION_TRAFFIC_CLASS_FLOW_LABEL_START     0
#define IPV6_VERSION_TRAFFIC_CLASS_FLOW_LABEL_COUNT     4

#define IPV6_VERSION_TRAFFIC_CLASS_HI4_START            0
#define IPV6_VERSION_TRAFFIC_CLASS_HI4_COUNT            1

#define IPV6_TRAFFIC_CLASS_LO4_FLOW_LABEL_HI4_START     1
#define IPV6_TRAFFIC_CLASS_LO4_FLOW_LABEL_HI4_COUNT     1

#define IPV6_FLOW_LABEL_LO16_START                      2
#define IPV6_FLOW_LABEL_LO16_COUNT                      2

#define IPV6_BYTES_4_7_START                            4
#define IPV6_BYTES_4_7_COUNT                            4

#define IPV6_PAYLOAD_LENGTH_START                       4
#define IPV6_PAYLOAD_LENGTH_COUNT                       2

#define IPV6_NEXT_HEADER_START                          6
#define IPV6_NEXT_HEADER_COUNT                          1

#define IPV6_HOP_LIMIT_OFFSET                           7
#define IPV6_HOP_LIMIT_LENGTH                           1

#define IPV6_SRC_ADDR_START                             8
#define IPV6_SRC_ADDR_COUNT                             16

#define IPV6_SRC_ADDR_127_96_START                      8
#define IPV6_SRC_ADDR_127_96_COUNT                      4

#define IPV6_SRC_ADDR_95_64_START                       12
#define IPV6_SRC_ADDR_95_64_COUNT                       4

#define IPV6_SRC_ADDR_63_32_START                       16
#define IPV6_SRC_ADDR_63_32_COUNT                       4

#define IPV6_SRC_ADDR_31_0_START                        20
#define IPV6_SRC_ADDR_31_0_COUNT                        4

#define IPV6_DST_ADDR_START                             24
#define IPV6_DST_ADDR_COUNT                             16

#define IPV6_DST_ADDR_127_96_START                      24
#define IPV6_DST_ADDR_127_96_COUNT                      4

#define IPV6_DST_ADDR_95_64_START                       28
#define IPV6_DST_ADDR_95_64_COUNT                       4

#define IPV6_DST_ADDR_63_32_START                       32
#define IPV6_DST_ADDR_63_32_COUNT                       4

#define IPV6_DST_ADDR_31_0_START                        36
#define IPV6_DST_ADDR_31_0_COUNT                        4


#define IPV6_VERSION_TRAFFIC_CLASS_HI4         0, 1      // 4 bit version and
                                                         // MSB 4 bits of 
                                                         // traffic class
                                                         // offset 0, 1 byte
#define IPV6_TRAFFIC_CLASS_LO4_FLOW_LABEL_HI4  1, 1      // 4 LSB bits of
                                                         // traffic class and 
                                                         // 4 MSB bits of flow
                                                         // label
                                                         // offset 1, 1 byte
#define IPV6_FLOW_LABEL_LO16                   2, 2      // 16 LSB bits of bits
                                                         // of flow label
                                                         // offset 2, 2 bytes
#define IPV6_BYTES_4_7                         4, 4                                                                 

#define IPV6_PAYLOAD_LENGTH                    4, 2      // Payload length
                                                         // offset 4, 2 bytes
#define IPV6_NEXT_HEADER                       6, 1      // Next Header
                                                         // offset 6, 1 bytes
#define IPV6_HOP_LIMIT                         7, 1      // Hop Limit
                                                         // offset 7, 1 bytes
#define IPV6_SRC_ADDR                          8, 16     // Source Address
                                                         // offset 8, 16 bytes
#define IPV6_DST_ADDR                         24, 16     // Destination Address
                                                         // offset 24, 16 bytes
#define IPV6_SRC_ADDR_127_120                  8, 1      // source Address
                                                         // bits [127..112]
                                                         // offset  8, 1 byte
#define IPV6_SRC_ADDR_127_96                   8, 4      // Source Address
                                                         // bits [127..96]
                                                         // offset  8, 4 bytes
#define IPV6_SRC_ADDR_95_64                   12, 4      // Source Address
                                                         // bits [ 95..64]
                                                         // offset 12, 4 bytes
#define IPV6_SRC_ADDR_63_32                   16, 4      // Source Address
                                                         // bits [ 63..32]
                                                         // offset 16, 4 bytes
#define IPV6_SRC_ADDR_31_0                    20, 4      // Source Address
                                                         // bits [ 31.. 0]
                                                         // offset 20, 4 bytes

#define IPV6_DST_ADDR_127_96                  24, 4      // Dst Address
                                                         // bits [127..96]
                                                         // offset 24, 4 bytes
#define IPV6_DST_ADDR_95_64                   28, 4      // Dst Address
                                                         // bits [ 95..64]
                                                         // offset 28, 4 bytes
#define IPV6_DST_ADDR_63_32                   32, 4      // Dst Address
                                                         // bits [ 63..32]
                                                         // offset 32, 4 bytes
#define IPV6_DST_ADDR_31_0                    36, 4      // Dst Address
                                                         // bits [ 31.. 0]
                                                         // offset 36, 4 bytes

//IPV6 extension header values
#define IPV6_EXTN_HOP_BY_HOP 0
#define IPV6_EXTN_ROUTING 43
#define IPV6_EXTN_FRAGMENT 44
#define IPV6_EXTN_DESTINATION 60
#define IPV6_EXTN_AUTHENTICATION 51
#define IPV6_EXTN_ENCAPSULATION 50

#endif    //_IPV6_H_
