//--------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2002-2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//---------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
//
//      File Name: 	v6v4_natpt_funcs.uc
//
//      Purpose: 
//				Contains subroutines called by NATPT microblock.
//
//      History:
//
//      Date            Comment
//      ---------------------------------------------------------------------
//      3/25/2003      Created
/////////////////////////////////////////////////////////////////////////////

#ifndef __V6V4_NATPT_FUNCS_UC
#define __V6V4_NATPT_FUNCS_UC

br[LBL_PAST_DL_FUNCS#]

//This macro does the same tasks as the generic xbuf_copy macro.
//It handles the specific case of,
// [Local memory buffers, variable source offset, variable destination offset].
//The macro assumes that the parameters are in registers. And that these
//registers can be modified.
//Parameters have the same meaning as the original xbuf_copy macro
#macro _xbuf_copy_lmem_var_src_var_dest(out_dest_xbuf, out_last_element,\
                                        dest_start_byte, in_src_xbuf,\
					                    src_start_byte, in_prepend,\
                                        total_bytes_to_copy, DATA_SPREAD)
.begin
	.reg handoff
	.reg cur_bytes_to_copy

lbl_copy_more#:
	.if (total_bytes_to_copy >= 4)
		alu[cur_bytes_to_copy, --, B, 4]
	.else
		alu[cur_bytes_to_copy, --, B, total_bytes_to_copy]
	.endif

	xbuf_extract(handoff, in_src_xbuf, 0, src_start_byte, cur_bytes_to_copy)
	xbuf_insert(out_dest_xbuf, handoff, 0, dest_start_byte, cur_bytes_to_copy)

	alu[total_bytes_to_copy, total_bytes_to_copy, -, cur_bytes_to_copy]
	bgt[lbl_copy_more#], defer[2]
	alu[src_start_byte, src_start_byte, +, cur_bytes_to_copy]
	alu[dest_start_byte, dest_start_byte, +, cur_bytes_to_copy]
.end
#endm

//The 'function' version of the macro dl_get_bytes. This has been created to
//work around code-store overflow issues faced while integrating NATPT into
//the ethernet ipv6 functional pipeline.
//
//Assumes the existence of buffers LM_NATPT_WORK_BUF and $$iphdr
//So these buffers need to be alloced *before* this code is *included*

//Func call, to use when reading into LM_NATPT_WORK_BUF.
//Also, does not allow a signal to be specified.
//Assumes start offset & bytes to read are passed in, in the registers defined
//below.
//Note: Currently hardcoded to work with a local memory buffer indexed by 
//		LM index 1. Can extend by adding another parameter which indicates
//		the index to use (0/1)

//The following will never be called. BUT we need it here for expanding the
//xbuf macros used in the function below properly.
//By calling this we basically ensure that the correct LM index is set
//NOTE it is the responsibility of the caller to make the xbuf_activate call
//before making the function call, so that the buffer address is loaded and the
//correct context id is used

dl_get_bytes#:
.reg in_start_offset_in_header
.reg in_bytes_to_read

.begin
	.sig sig_dl_get_bytes

	.reg rem_bytes
	.reg begin_offset
	.reg dest_offset
	.reg sdram_base //Address in DRAM where packet begins
	.reg qw_to_read

	xbuf_activate(LM_NATPT_WORK_BUF, 1, 0,0)

	//First we check if the requested bytes are in transfer registers
	//already. If not, issue a DRAM read.
	;find the max requested offset
	alu[rem_bytes, --, B, IN_BYTES_TO_READ]
	alu[begin_offset, --, B, in_start_offset_in_header]
	alu[dest_offset, --, B, 0]

    //If the buffer $$iphdr was reloaded by someone, we do not know the
    //offset of the first byte loaded into the buffer(w.r.t the packet)
    //So we cannot use the below optimization. Simply read directly from DRAM
    .if (buffer_first_load == 1)
    	.if (in_start_offset_in_header < PKT_HDR_CACHE_BYTES)
    	  .begin
    		.reg end_offset

    	    alu[end_offset, in_start_offset_in_header, +, IN_BYTES_TO_READ]
    		alu[--, end_offset, -, PKT_HDR_CACHE_BYTES]
    		bge[LBL_PARTIAL_BUF#]
    			//We have already read the requested bytes
    			//Simply copy off into buffer passed in
    			alu[rem_bytes, --, B, IN_BYTES_TO_READ]
    			br[LBL_PAST_PARTIAL_BUF#]

    LBL_PARTIAL_BUF#:
    			//We have already read *some* of the requested bytes
    			//Copy these off into the buffer passed in
    			alu[rem_bytes, PKT_HDR_CACHE_BYTES, -, in_start_offset_in_header]

    LBL_PAST_PARTIAL_BUF#:
    		xbuf_copy(LM_NATPT_WORK_BUF, 0, 0, $$iphdr, in_start_offset_in_header,
    				  0, rem_bytes, 0)

    		alu[--, end_offset, -, PKT_HDR_CACHE_BYTES]
    		ble[LBL_PAST_DRAM_READ#]

    		alu[begin_offset, --, B, PKT_HDR_CACHE_BYTES]
    		alu[dest_offset, --, B, rem_bytes]
    		alu[rem_bytes, IN_BYTES_TO_READ, -, rem_bytes]

          .end
    	.endif
    .endif

	//Need to read in more bytes from DRAM. Get the DRAM base
	dl_buf_get_data[sdram_base, dl_buf_handle]

    //We are about to reload the buffer, so reset buffer_first_load
    alu[buffer_first_load, --, B, 0]

	//Account for the buffer headroom

//buf_offset keeps moving. once a ublock is done working on a header it moves
//buf_offset past the point it worked on. We want the offset to the very start
//of the buffer. Since we do not know here which ublock in the pipeline will
//call this macro we do not use the meta data. 
//Note: Account for the buffer headroom
//		dl_meta_get_offset[buf_offset] ;get the buf offset
//		alu[buf_offset, buf_offset, +, in_start_offset_in_header]

	alu[begin_offset, buffer_headroom, +, begin_offset]

	//DRAM accesses are done in quad word units. So first we find number
	//of quad words to read.
	//Also, the DRAM reads have to be done from an offset aligned on a
	//Quad word boundary. Since the DRAM instruction ignores the lowest
	//3 bits in the address, we don't need to do anything to ensure this.
	//What this means though is that the value read from DRAM can contain
	//some extra bytes before (if the address is not quadword aligned)
	//and/or after (if IN_BYTES_TO_READ is not a multiple of 8) 
	//the chunk of data we need to read.
	//Hence we read from DRAM into a temporary buffer & then we copy off
	//the right number of bytes from the correct offset in this buffer into
	//the final buffer to return to the caller.

	.begin
		.reg offset_in_temp_buf
        .reg actual_dest_offset
        .reg actual_qw_to_read

		//Based on the initial offset we might have to read in one more
		//Quadword. For eg. if initial offset=3, in_bytes_to_read=8
		//we will need to read in 2 Quadwords, as the 8 bytes we need
		//straddles a quadword boundary
		alu[offset_in_temp_buf, begin_offset, AND, 7]
		alu[qw_to_read, rem_bytes, +, 7]
		alu[qw_to_read, offset_in_temp_buf, +, qw_to_read]
		alu_shf[qw_to_read, --, B, qw_to_read, >>3]

		#define_eval MAX_REFCOUNT 8

        alu[actual_dest_offset, --, B, dest_offset]

//A generic dl_get_bytes macro would need these checks. But since we lack
//codespace and more importantly since we ensure that we never call this
//macro requesting more than 8 quadwords, this code is ifdef'd out
#ifdef NO_CODESTORE_OVERFLOW
LBL_READ_SOME_MORE#:
        .if (qw_to_read > 8)
            alu[actual_qw_to_read, --, B, 8]
        .else
            alu[actual_qw_to_read, --, B, qw_to_read]
        .endif
#else
		alu[actual_qw_to_read, --, B, qw_to_read]
#endif //NO_CODESTORE_OVERFLOW

		alu[actual_qw_to_read,actual_qw_to_read,OR,0x10] ;Set override bit for ref count
		alu_shf[--, --, B, actual_qw_to_read, <<21]

		dram[read, $$iphdr0, begin_offset, sdram_base, max_/**/MAX_REFCOUNT],
			 indirect_ref, sig_done[sig_dl_get_bytes]
		ctx_arb[sig_dl_get_bytes]

		//The xbuf code does not handle our specific case. So we have an internal
		//macro for our specific case.
//		xbuf_copy(LM_NATPT_WORK_BUF, 0, dest_offset, $$iphdr, offset_in_temp_buf, 0,
//				  rem_bytes, 0)
		_xbuf_copy_lmem_var_src_var_dest(LM_NATPT_WORK_BUF, 0, actual_dest_offset,
                                         $$iphdr, offset_in_temp_buf, 0,
                                         rem_bytes, 0)

#ifdef NO_CODESTORE_OVERFLOW
        .if (qw_to_read > 8)
            alu[qw_to_read, qw_to_read, -, 8]
            alu[rem_bytes, rem_bytes, -, 64]
            alu[actual_dest_offset, actual_dest_offset, +, 64]
            alu[begin_offset, begin_offset, +, 64]
            br[LBL_READ_SOME_MORE#]
        .endif
#endif //NO_CODESTORE_OVERFLOW

		#undef MAX_REFCOUNT

LBL_PAST_DRAM_READ#:
	.end

	rtn[ret_reg]
.end

LBL_PAST_DL_FUNCS#:
#endif //__V6V4_NATPT_FUNCS_UC

