//---------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2002-2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//---------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: 	v6v4_natpt_util.uc
//
//      Purpose: 
//					Contains helper routines used by the NATPT microblock.
//
//      History:
//
//
//      Date            Comment
//      ---------------------------------------------------------------------
//
//      3/19/2003      Created
//
//
/////////////////////////////////////////////////////////////////////////////

//Value indicating that there is no entry in this row in the table
//When initialized, all rows in the tables are set to this value.
#define NO_ENTRY 0xFFFFFFFF

#define HASH_ENTRY_SIZE_LW 2

//Each NAT entry is 32 bytes.
#define NAT_ENTRY_SIZE_LW 8
#define NAT_ENTRY_SIZE_SHIFT 5

//We only need to read 24 bytes
#define NAT_ENTRY_READ_SIZE_LW 6

#define HE_ENTRY_POINTER_OFFSET 0
#define HE_NEXT_OFFSET 1

#define NE_V4_ADDR_OFFSET 0
#define NE_V6_ADDR_127_96_OFFSET 1
#define NE_V6_ADDR_95_64_OFFSET 2
#define NE_V6_ADDR_63_32_OFFSET 3
#define NE_V6_ADDR_31_0_OFFSET 4
#define NE_ENTRY_USED_V6NHID_OFFSET 21

////////////////////////////////////////////////////////////////////////////////
// call
//
//    Description:
//        Utility macro that can be used to make function calls
//
//    Parameters:
//        Outputs:  None
//          
//        Inputs:
//          - the_function: Label to jump to.
////////////////////////////////////////////////////////////////////////////////

#macro call(the_function)
	load_addr[ret_reg, return_here#]
	br[the_function]
	return_here#:
#endm


#macro _natpt_hash_init()
.begin
    .sig hash_init_sig1, hash_init_sig2, hash_init_sig3, hash_init_sig4, hash_init_sig5, hash_init_sig6
    .reg $reg_mul_48_0, $reg_mul_48_1
    .reg $reg_mul_128_0, $reg_mul_128_1, $reg_mul_128_2, $reg_mul_128_3

    immed32[$reg_mul_48_0, NATPT_HASH_MUL_48_0]
    immed32[$reg_mul_48_1, NATPT_HASH_MUL_48_1]

    immed32[$reg_mul_128_0, NATPT_HASH_MUL_128_0]
    immed32[$reg_mul_128_1, NATPT_HASH_MUL_128_1]
    immed32[$reg_mul_128_2, NATPT_HASH_MUL_128_2]
    immed32[$reg_mul_128_3, NATPT_HASH_MUL_128_3]

    cap[write, $reg_mul_48_0, HASH_MULTIPLIER_48_0], sig_done[hash_init_sig1]
    cap[write, $reg_mul_48_1, HASH_MULTIPLIER_48_1], sig_done[hash_init_sig2]

    cap[write, $reg_mul_128_0, HASH_MULTIPLIER_128_0], sig_done[hash_init_sig3]
    cap[write, $reg_mul_128_1, HASH_MULTIPLIER_128_1], sig_done[hash_init_sig4]
    cap[write, $reg_mul_128_2, HASH_MULTIPLIER_128_2], sig_done[hash_init_sig5]
    cap[write, $reg_mul_128_3, HASH_MULTIPLIER_128_3], sig_done[hash_init_sig6]

    ctx_arb[hash_init_sig1, hash_init_sig2, hash_init_sig3, hash_init_sig4, hash_init_sig5, hash_init_sig6]
.end
#endm

////////////////////////////////////////////////////////////////////////////////
// _get_hash_v4addr
//
//    Description:
//        Hashing routine for a v4 address (i.e a 32 bit value)
//
//    Parameters:
//        Outputs:
//          - hash_value: The hash value generated
//        Inputs:
//          - dest_addr: The 32 bit value on which to hash
////////////////////////////////////////////////////////////////////////////////
#macro _get_hash_v4addr(hash_value, dest_addr)
.begin
    .reg $hash_reg0, $hash_reg1
    .xfer_order $hash_reg0, $hash_reg1
    .sig sig_hash

    alu[$hash_reg0, --, B, dest_addr]
    alu[$hash_reg1, --, B, 0]

    hash_48[$hash_reg0, 1], sig_done[sig_hash]
    ctx_arb[sig_hash]

    //We only pick out the lower seven bits
    alu[hash_value, $hash_reg0, AND, 0x7f]
.end
#endm

////////////////////////////////////////////////////////////////////////////////
// _get_hash_v6addr
//
//    Description:
//        Hashing routine for a v6 address (i.e a 128 bit value)
//
//    Parameters:
//        Outputs:
//          - hash_value: The hash value generated
//        Inputs:
//          - in_sa_127_96:
//          - in_sa_95_64:
//          - in_sa_63_32:
//          - in_sa_31_0: 
//              The 128 bit value on which to hash
////////////////////////////////////////////////////////////////////////////////
#macro 	_get_hash_v6addr(hash_value, in_sa_127_96, in_sa_95_64, in_sa_63_32, in_sa_31_0)
.begin
    .reg $hash_reg0, $hash_reg1, $hash_reg2, $hash_reg3
    .xfer_order $hash_reg0, $hash_reg1, $hash_reg2, $hash_reg3
    .sig sig_hash

    alu[$hash_reg3, --, B, in_sa_127_96]
    alu[$hash_reg2, --, B, in_sa_95_64]
    alu[$hash_reg1, --, B, in_sa_63_32]
    alu[$hash_reg0, --, B, in_sa_31_0]

    hash_128[$hash_reg0, 1], sig_done[sig_hash]
    ctx_arb[sig_hash]

    //We only pick out the lower seven bits
    alu[hash_value, $hash_reg0, AND, 0x7f]
.end
#endm

////////////////////////////////////////////////////////////////////////////////
// _natpt_v4_v6_hash
//
//    Description:
//        Hashes into the given hash table using the destination address 
//        (32 bit hash key) passed,
//        and returns the binding information created for the address.
//        If needed, this function walks the hash collision chain in the hash
//        table till it finds a 'hit'
//
//    Parameters:
//        Outputs:
//          - out_hit: set to 1 if a match found, 0 otherwise
//          - out_nat_entry: Holds the binding entry if there is a hit
//                           This parameter should be ignored if there was a
//                           miss
//        Inputs:
//          - in_dest_addr : The 32 bit hash key
//          - IN_HASH_TABLE: Start address of table (in SRAM) in which to hash
////////////////////////////////////////////////////////////////////////////////
#macro _natpt_v4_v6_hash(out_hit, out_nat_entry, in_dest_addr,\
						 IN_HASH_TABLE_BASE, IN_HASH_COLL_TABLE_BASE,\
						 IN_BINDINGS_TABLE_BASE)
.begin
	.reg read $hash_entry0, $hash_entry1
	.xfer_order $hash_entry0, $hash_entry1
	.reg hash_value, offset
	.reg hash_table_base, bindings_table_base, coll_table_base
	.reg write $entry_used ;xfer register used to update a flag present in a
						   ;NAT record indicating that the record was accessed
						   ;recently

	.sig sig_nat_hash

	//The following to get rid of warnings related to uninitialized xfer
	//register access. Basically we know that whenever hit = 1, the xfer reg
	//are appropriately initialized. This is the only case when the caller
	//should try to access the xfer registers
	#define LOOP 0
	#while (LOOP < NAT_ENTRY_READ_SIZE_LW)
		.set out_nat_entry/**/LOOP
		#define_eval LOOP (LOOP+1)
	#endloop

	_get_hash_v4addr(hash_value, in_dest_addr)
	alu_shf[hash_value, --, B, hash_value, <<3] ;each hash entry is 8 bytes 

	immed32[hash_table_base, IN_HASH_TABLE_BASE]
	sram[read, $hash_entry0, hash_table_base, hash_value, 2],
			 ctx_swap[sig_nat_hash]

	.if ($hash_entry[HE_ENTRY_POINTER_OFFSET] == NO_ENTRY)
	 	alu[out_hit, --, B, 0] ;No entry present.
	.else
		//Lets see if the entry matches the key passed in
		immed32(coll_table_base, IN_HASH_COLL_TABLE_BASE)
		immed32(bindings_table_base, IN_BINDINGS_TABLE_BASE)
	  LBL_CHECK_ENTRY#:
	  	alu_shf[offset, --, B, $hash_entry[HE_ENTRY_POINTER_OFFSET], <<NAT_ENTRY_SIZE_SHIFT]
		sram[read, out_nat_entry/**/0, bindings_table_base, offset, NAT_ENTRY_READ_SIZE_LW],
				 ctx_swap[sig_nat_hash]

		.if (out_nat_entry[NE_V4_ADDR_OFFSET] == in_dest_addr)
			alu[out_hit, --, B, 1]

			//Update the entryUsed flag here
			//Optimization: Need not wait for this operation to complete.
			//   Will need to ensure that the xfer register is available
			//   till the value is read out from it.
			immed32($entry_used, 0x80000000)

			//We will be exiting now, so we can change the value of
			//bindings_table_base here. This to avoid using another register
			alu[bindings_table_base, bindings_table_base, +, offset]
			sram[set, $entry_used, bindings_table_base, NE_ENTRY_USED_V6NHID_OFFSET], 
				ctx_swap[sig_nat_hash]
		.else
			.if ($hash_entry[HE_NEXT_OFFSET] == NO_ENTRY)
				//There are no more entries for this hash index. And none of
				//the entries matched the key passed in.
				alu[out_hit, --, B, 0]
			.else
				//More than one entry hashed to the same location. There exists a
				//collision chain
				alu_shf[hash_value, --, B, $hash_entry[HE_NEXT_OFFSET], <<3] ;each hash entry is 8 bytes 
				sram[read, $hash_entry0, coll_table_base, hash_value, HASH_ENTRY_SIZE_LW],
					 ctx_swap[sig_nat_hash]
				br[LBL_CHECK_ENTRY#]
			.endif
		.endif
	.endif
.end
#endm

////////////////////////////////////////////////////////////////////////////////
// _natpt_v6_v4_hash
//
//    Description:
//        Hashes into the given hash table using the 128 bit key passed,
//        and returns the binding information created for the address.
//        If needed, this function walks the hash collision chain in the hash
//        table till it finds a 'hit'
//
//    Parameters:
//        Outputs:
//          - out_hit: set to 1 if a match found, 0 otherwise
//          - out_nat_entry: Holds the binding entry if there is a hit
//                           This parameter should be ignored if there was a
//                           miss
//        Inputs:
//          - IN_NAT_ENTRY_SIZE:The number of LW to read from the binding entry
//          - in_sa_127_96:
//          - in_sa_95_64:
//          - in_sa_63_32:
//          - in_sa_31_0: 
//              The 128 bit hash key
//          - IN_HASH_TABLE: Start address of table (in SRAM) in which to hash
////////////////////////////////////////////////////////////////////////////////
#macro _natpt_v6_v4_hash(out_hit, out_nat_entry, IN_NAT_ENTRY_SIZE,\
						 in_sa_127_96, in_sa_95_64, in_sa_63_32, in_sa_31_0,\
						 IN_HASH_TABLE_BASE, IN_HASH_COLL_TABLE_BASE,\
						 IN_BINDINGS_TABLE_BASE)
.begin
	.reg $hash_entry0, $hash_entry1
	.xfer_order $hash_entry0, $hash_entry1
	.reg hash_value, offset
	.reg hash_table_base, bindings_table_base, coll_table_base
	.reg write $entry_used ;xfer register used to update a flag present in a
						   ;NAT record indicating that the record was accessed
						   ;recently

	.sig sig_nat_hash

	//The following to get rid of warnings related to uninitialized xfer
	//register access. Basically we know that whenever hit = 1, the xfer reg
	//are appropriately initialized. This is the only case when the caller
	//should try to access the xfer registers
	#define LOOP 0
	#while (LOOP < IN_NAT_ENTRY_SIZE)
		.set out_nat_entry/**/LOOP
		#define_eval LOOP (LOOP+1)
	#endloop

	_get_hash_v6addr(hash_value, in_sa_127_96, in_sa_95_64, in_sa_63_32, in_sa_31_0)
	alu_shf[hash_value, --, B, hash_value, <<3] ;each hash entry is 8 bytes 

	immed32[hash_table_base, IN_HASH_TABLE_BASE]
	sram[read, $hash_entry0, hash_table_base, hash_value, 2],
			 ctx_swap[sig_nat_hash]

	.if ($hash_entry[HE_ENTRY_POINTER_OFFSET] == NO_ENTRY)
	 	alu[out_hit, --, B, 0] ;No entry present.
	.else
		//Lets see if the entry matches the key passed in
		immed32[coll_table_base, IN_HASH_COLL_TABLE_BASE]
		immed32[bindings_table_base, IN_BINDINGS_TABLE_BASE]
	  LBL_CHECK_ENTRY#:
	  	alu_shf[offset, --, B, $hash_entry[HE_ENTRY_POINTER_OFFSET], <<NAT_ENTRY_SIZE_SHIFT]
		sram[read, out_nat_entry/**/0, bindings_table_base, offset, IN_NAT_ENTRY_SIZE],
				 ctx_swap[sig_nat_hash]

		.if ((out_nat_entry[NE_V6_ADDR_127_96_OFFSET] == in_sa_127_96) && \
			 (out_nat_entry[NE_V6_ADDR_95_64_OFFSET] == in_sa_95_64) && \
			 (out_nat_entry[NE_V6_ADDR_63_32_OFFSET] == in_sa_63_32) && \
			 (out_nat_entry[NE_V6_ADDR_31_0_OFFSET] == in_sa_31_0))
				alu[out_hit, --, B, 1]

				//Update the entryUsed flag here
				//Optimization: Need not wait for this operation to complete.
				//   Will need to ensure that the xfer register is available
				//   till the value is read out from it.

				immed32($entry_used, 0x80000000)

				//We will be exiting now, so we can change the value of
				//bindings_table_base here. This to avoid using another register
				alu[bindings_table_base, bindings_table_base, +, offset]
				sram[set, $entry_used, bindings_table_base, NE_ENTRY_USED_V6NHID_OFFSET],
					ctx_swap[sig_nat_hash]
		.else
			//The NAT record pointed to by the current hash entry did not match.
			//Are there any more hash entries?
			.if ($hash_entry[HE_NEXT_OFFSET] == NO_ENTRY)
				//There are no more entries for this hash index. And none of
				//the NAT records matched the key passed in.
				alu[out_hit, --, B, 0]
			.else
				//More than one entry hashed to the same location. There exists a
				//collision chain
				alu_shf[hash_value, --, B, $hash_entry[HE_NEXT_OFFSET], <<3] ;each hash entry is 8 bytes 
				sram[read, $hash_entry0, coll_table_base, hash_value, 2],
					 ctx_swap[sig_nat_hash]
				br[LBL_CHECK_ENTRY#]
			.endif
		.endif
	.endif
.end
#endm

#if 0
#macro _napt_hash(index,\
				   transport_protocol,\
				   transport_id,\
				   NAPTPT_HASH_TABLE_BASE)
	#warning "Placeholder for _napt_hash"
	alu[index, --, B, 0]
#endm

#macro _napt_translate_to_v6(index,\
							  transport_protocol,\
							  transport_hdr)
	#warning "Placeholder for _napt_translate_to_v6"
	alu[index, --, B, 0]
#endm

#macro _napt_translate_to_v4( index, in_ipv6_hdr, da_31_0, out_ipv4_hdr)
	#warning "Placeholder for _napt_translate_to_v4"
	alu[index, --, B, 0]
#endm
#endif

////////////////////////////////////////////////////////////////////////////////
// _repeat_per_elem_in_list
//
//    Description:
//        Utility macro that calls 'func_to_call' for each element in a comma
//        separated string.
//
//    Parameters:
//        Outputs: None
//        Inputs:
//           - func_to_call: The macro to call on each element
//           - va_list     : String containing comma separated list of elements
//                           for which the macro needs to be called.
//
//    Side effects:
//        Generates code.
////////////////////////////////////////////////////////////////////////////////
#macro _repeat_per_elem_in_list(func_to_call, va_list)
	#define_eval LOOP 1
	#define_eval orig_string va_list

	#while (LOOP > 0)
		#define_eval index strstr('orig_string', ",")
		#define_eval length strlen('orig_string')

		#if (index != 0)
			#define_eval cur_tok strleft('orig_string', (index-1))
			//Remove leading/trailing white space
			#define_eval cur_tok 'cur_tok'

			func_to_call(cur_tok)

			#define_eval orig_string strright('orig_string', (length-index))

			#undef cur_tok
		#else
			func_to_call(orig_string)

			#define_eval LOOP 0
		#endif

		#undef length
		#undef index
	#endloop

	#undef orig_string
	#undef LOOP
#endm

////////////////////////////////////////////////////////////////////////////////
// add_wrapper
//
//    Description:
//        Wrapper routine to setup input params.
//
//    Parameters:
//        Outputs:
//        Inputs:
//           - param: The parameter to setup prior to making a function call
////////////////////////////////////////////////////////////////////////////////
#macro add_wrapper(param)
    move(in_operand, param)

#ifdef NO_CODESTORE_OVERFLOW
	_modify_checksum(checksum, '+', in_operand)
#else
	call(_modify_checksum_ADD#)
#endif
#endm

////////////////////////////////////////////////////////////////////////////////
// del_wrapper
//
//    Description:
//        Wrapper routine to setup input params.
//
//    Parameters:
//        Outputs:
//        Inputs:
//           - param: The parameter to setup prior to making a function call
////////////////////////////////////////////////////////////////////////////////
#macro del_wrapper(param)
    move(in_operand, param)

#ifdef NO_CODESTORE_OVERFLOW
	_modify_checksum(checksum, '-', in_operand)
#else
	call(_modify_checksum_SUB#)
#endif
#endm

////////////////////////////////////////////////////////////////////////////////
// _natpt_increment_counter
//
//    Description:
//        Increments counter in SRAM
//
//    Parameters:
//        Outputs:
//        Inputs:
//           - in_counter_base: The base of the counters in SRAM
//           - OFFSET: Offset from base where the data is present
//    Side effects:
//        The value of the data at the specified location is incremented by 1.
////////////////////////////////////////////////////////////////////////////////
#macro _natpt_increment_counter(in_counter_base, OFFSET)
    sram[incr, --, OFFSET, in_counter_base]
#endm
