/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.x for the IXP2xxx Network Processors
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      IPv6 Network Address Translation - Protocol Translation Module.
 *
 * = FILENAME
 *      ix_cc_natpt.h
 *
 * = DESCRIPTION
 *      This file defines all exported data types and function prototypes
 *
 * = AUTHOR
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *   03/03/03
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: ix_cc_natpt.h,v 1.5 2003/04/30 01:06:23 sshegde Exp $
 */

#if !defined(__IX_CC_NATPT_H__)
#define __IX_CC_NATPT_H__

#include "ix_rm.h"
#include "ix_ossl.h"
#include "ix_cc.h"

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */


/**
 * TYPENAME: ix_cc_natpt_messages
 * 
 * DESCRIPTION: This structure defines format for NAT-PT messages
 *
 */
typedef enum ix_e_cc_natpt_messages
{
    /* NAT-PT Messages */

    IX_CC_NATPT_MSG_SET_CONFIG_PARAM = IX_CC_COMMON_MSG_ID_LAST,
    IX_CC_NATPT_MSG_GET_CONFIG_PARAM,
    IX_CC_NATPT_MSG_ADD_STATIC_MAP,
    IX_CC_NATPT_MSG_REM_STATIC_MAP,
    IX_CC_NATPT_MSG_GET_STATIC_MAP,
    IX_CC_NATPT_MSG_ADD_V4ADDR_POOL,
    IX_CC_NATPT_MSG_REM_V4ADDR_POOL,
    IX_CC_NATPT_MSG_GET_V4ADDR_POOL,
    IX_CC_NATPT_MSG_SET_NAPT_MODE,
    IX_CC_NATPT_MSG_ADD_V4V6_PORT_MAP,
    IX_CC_NATPT_MSG_REM_V4V6_PORT_MAP,
    IX_CC_NATPT_MSG_GET_V4V6_PORT_MAP,
    IX_CC_NATPT_MSG_GET_ACTIVE_SESSIONS,

    /* Add new messages before this */
    IX_CC_NATPT_MSG_LAST     

} ix_cc_natpt_messages;

/**
 * TYPENAME: ix_cc_natpt_error_code
 * 
 * DESCRIPTION: This enumerated type will describe NAT-PT error codes
 *
 */
typedef enum ix_e_cc_natpt_error_code
{
    IX_CC_NATPT_ERROR_FIRST = IX_ERROR_MAKE_GROUP(CC_NATPT),
    IX_CC_NATPT_ERROR_SUCCESS = IX_CC_NATPT_ERROR_FIRST,

    IX_CC_NATPT_ERROR_FAILED_PATCHING,
    IX_CC_NATPT_ERROR_REGISTRY,
    IX_CC_NATPT_ERROR_BUFFER_FREE,

    IX_CC_NATPT_ERROR_CCI,
    IX_CC_NATPT_ERROR_INVALID_CONTEXT,
    IX_CC_NATPT_ERROR_INVALID_INPUT_PARAM,

    IX_CC_NATPT_ERROR_INVALID_BUFFER,
    IX_CC_NATPT_ERROR_MSG_LIBRARY,
    IX_CC_NATPT_ERROR_INVALID_PACKET,

    IX_CC_NATPT_ERROR_INVALID_PROP_ID,
    IX_CC_NATPT_ERROR_FAILED_GET_PORT_STATUS, 
    IX_CC_NATPT_ERROR_CALLBACK,

    IX_CC_NATPT_ERROR_NOT_INITIALIZED,
    IX_CC_NATPT_ERROR_IGNORE_PACKET,

    IX_CC_NATPT_ERROR_BUFFER_TOO_SMALL,

    IX_CC_NATPT_ERROR_NOT_IMPLEMENTED,

    IX_CC_NATPT_ERROR_ICMP_TRANSLATE,

    /* unknown error code */
    IX_CC_NATPT_ERROR_UNKNOWN,

    IX_CC_NATPT_ERROR_LAST

} ix_cc_natpt_error_code;


/* 
 *****************************************************************************
 * This section defines all the data strcutures for natpt API
 *****************************************************************************
 */

/**
 * TYPENAME: ix_cc_natpt_config_params
 * 
 * DESCRIPTION: This structure defines configuration paramters that are
 * used by the core component.
 * idleCheckInterval: This value is given in seconds. This is used by the 
 * core component to check at regular intervals whether any packet 
 * activity for a NAT-PT session has taken place since the last check. 
 * After detection of no activity, the session is removed. 
 * Default value is 300*
 */
typedef struct ix_s_cc_natpt_config_params
{
    ix_uint32 idleCheckInterval;

} ix_cc_natpt_config_params;


/**
 * TYPENAME: ix_cc_natpt_v6addr
 * 
 * DESCRIPTION: This data type specifies an IPv6 address. 
 */
typedef ix_uint128 ix_cc_natpt_v6addr;


/**
 * TYPENAME: ix_cc_natpt_v4addr
 * 
 * DESCRIPTION: This data type specifies an IPv4 address. 
 */
typedef ix_uint32 ix_cc_natpt_v4addr;


/**
 * TYPENAME: ix_cc_natpt_static_v6v4map
 * 
 * DESCRIPTION: This structure is used to specify a static IPv6 address to 
 * IPv4 address mapping.
 */
typedef struct ix_s_cc_natpt_static_v6v4map
{
    ix_cc_natpt_v6addr v6addr;
    ix_cc_natpt_v4addr v4addr;

} ix_cc_natpt_static_v6v4map;


/**
 * TYPENAME: ix_cc_natpt_v4v6portmap
 * 
 * DESCRIPTION: This structure is used to specify a static mapping of 
 * IPv6 address, IPv4 address and TCP/UDP Port number. However, IPv4 
 * address is not given since it is automatically chosen.
 */
typedef struct ix_s_cc_natpt_v4v6portmap 
{
    ix_cc_natpt_v6addr v6addr;
    ix_uint16 protocol;
    ix_uint16 portNumber;

} ix_cc_natpt_v4v6portmap;


/**
 * TYPENAME: ix_cc_natpt_naptmode
 * 
 * DESCRIPTION: This data type is used to turn ON/OFF NAPT-PT mode of 
 * operation.
 */
typedef enum ix_e_cc_natpt_naptmode 
{
    IX_CC_NATPT_NAPT_OFF,
    IX_CC_NATPT_NAPT_ON

} ix_cc_natpt_naptmode;


/**
 * TYPENAME: ix_cc_natpt_naptmode
 * 
 * DESCRIPTION: This structure contains information of a NAT-PT session.
 */

typedef struct ix_s_cc_natpt_session 
{
    ix_cc_natpt_v6addr v6addr;
    ix_cc_natpt_v4addr v4addr;
    ix_uint16 protocol;
    ix_uint16 v6PortNumber;
    ix_uint16 v4PortNumber;

} ix_cc_natpt_session;



/* 
 *****************************************************************************
 * This section defines all the asynchronous callback types 
 *****************************************************************************
 */


/**
 * TYPENAME: ix_cc_natpt_cb
 * 
 * DESCRIPTION: Type definition for general NATPT Message callbacks.
 */
typedef ix_error (*ix_cc_natpt_cb)(
                ix_error arg_Result,
                void* arg_pContext);


/**
 * TYPENAME: ix_cc_natpt_cb_get_config_parameters
 * 
 * DESCRIPTION: Type definition for callback function to get current
 * configuration paramters
 */
typedef ix_error (*ix_cc_natpt_cb_get_config_parameters) (
                ix_error arg_Result,
                void * arg_pClientContext,
                ix_cc_natpt_config_params * arg_pConfig);
                

/**
 * TYPENAME: ix_cc_natpt_cb_get_static_mapping
 * 
 * DESCRIPTION: Type definition for callback function to get current
 * static mapping
 */
typedef ix_error (*ix_cc_natpt_cb_get_static_mapping) (
                ix_error arg_Result,
                void * arg_pClientContext,
                ix_uint32 arg_NumItems,
                ix_cc_natpt_static_v6v4map * arg_pv6v4MapArray);


/**
 * TYPENAME: ix_cc_natpt_cb_get_v4addr_pool
 * 
 * DESCRIPTION: Type definition for callback function to get current
 * IPv4 address pool
 */
typedef ix_error (*ix_cc_natpt_cb_get_v4addr_pool) (
                ix_error arg_Result,
                void * arg_pClientContext,
                ix_uint32 arg_NumItems,
                ix_cc_natpt_v4addr * arg_pv4addrArray);


/**
 * TYPENAME: ix_cc_natpt_cb_get_v4v6port_mapping
 * 
 * DESCRIPTION: Type definition for callback function to get current
 * static IPv4/IPv6/PortNumber mapping.
 */
typedef ix_error (*ix_cc_natpt_cb_get_v4v6port_mapping) (
                ix_error arg_Result,
                void * arg_pClientContext,
                ix_uint32 arg_NumItems,
                ix_cc_natpt_v4v6portmap * arg_pv4v6portMapArray);


/**
 * TYPENAME: ix_cc_natpt_cb_get_active_sessions
 * 
 * DESCRIPTION: Type definition for callback function to get current
 * active NATPT sessions.
 */
typedef ix_error (*ix_cc_natpt_cb_get_active_sessions) (
                ix_error arg_Result,
                void * arg_pClientContext,
                ix_uint32 arg_NumItems,
                ix_cc_natpt_session * arg_pSessionArray);


/* 
 *****************************************************************************
 * This section defines the prototypes for all the asynchronous functions 
 *****************************************************************************
 */


/**
 * NAME: ix_cc_natpt_async_set_config_parameters
 *
 * DESCRIPTION:This message helper primitive helps client to set global
 * configuration parameters
 * 
 * @Param:  - IN arg_pConfig: pointer to config parameters structure.
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_set_config_parameters(
                ix_cc_natpt_config_params * arg_pConfig,
                ix_cc_natpt_cb arg_Callback,
                void * arg_pClientContext);


/**
 * NAME: ix_cc_natpt_async_get_config_parameters
 *
 * DESCRIPTION:This message helper primitive helps client to get global
 * configuration parameters. The data is returned via the callback.
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_get_config_parameters(
                ix_cc_natpt_cb_get_config_parameters arg_Callback,
                void * arg_pClientContext);


/**
 * NAME: ix_cc_natpt_async_add_static_mapping
 *
 * DESCRIPTION:This message helper primitive add static ipv6/ipv4 address
 * mapping.
 * 
 * @Param:  - IN arg_pv6v4Map: Pointer to a v6 v4 address mapping structure.
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_add_static_mapping(
                ix_cc_natpt_static_v6v4map * arg_pv6v4Map,
                ix_cc_natpt_cb arg_Callback,
                void * arg_pClientContext);


/**
 * NAME: ix_cc_natpt_async_remove_static_mapping
 *
 * DESCRIPTION:This message helper primitive remove static ipv6/ipv4 address
 * mapping.
 * 
 * @Param:  - IN arg_pv6v4Map: Pointer to a v6 v4 address mapping structure.
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_remove_static_mapping(
                ix_cc_natpt_static_v6v4map * arg_pv6v4Map,
                ix_cc_natpt_cb arg_Callback,
                void * arg_pClientContext);


/**
 * NAME: ix_cc_natpt_async_get_static_mapping
 *
 * DESCRIPTION:This message helper primitive get static ipv6/ipv4 address
 * mapping.
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_get_static_mapping(
                ix_cc_natpt_cb_get_static_mapping arg_Callback,
                void * arg_pClientContext);


/**
 * NAME: ix_cc_natpt_async_add_v4addr_pool
 *
 * DESCRIPTION:This message helper primitive helps client to add IPv4
 * addresses to pool.
 * 
 * @Param:  - IN arg_NumItems: Number of v4 addrs in the array pointed 
 *                             by the next argument.
 * @Param:  - IN arg_pv4addrArray: Pointer to a v4 address array.
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_add_v4addr_pool(
                ix_uint32 arg_NumItems,
                ix_cc_natpt_v4addr * arg_pv4addrArray,
                ix_cc_natpt_cb arg_Callback,
                void * arg_pClientContext);


/**
 * NAME: ix_cc_natpt_async_remove_v4addr_pool
 *
 * DESCRIPTION:This message helper primitive helps client to remove IPv4
 * addresses from pool.
 * 
 * @Param:  - IN arg_NumItems: Number of v4 addrs in the array pointed 
 *                             by the next argument.
 * @Param:  - IN arg_pv4addrArray: Pointer to a v4 address array.
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_remove_v4addr_pool(
                ix_uint32 arg_NumItems,
                ix_cc_natpt_v4addr * arg_pv4addrArray,
                ix_cc_natpt_cb arg_Callback,
                void * arg_pClientContext);


/**
 * NAME: ix_cc_natpt_async_get_v4addr_pool
 *
 * DESCRIPTION:This message helper primitive helps client to get current
 * list of ipv4 address in pool.
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_get_v4addr_pool(
                ix_cc_natpt_cb_get_v4addr_pool arg_Callback,
                void * arg_pClientContext);


/**
 * NAME: ix_cc_natpt_async_set_napt_mode
 *
 * DESCRIPTION:This message helper primitive helps client to turn NAPT
 * mode ON/OFF.
 * 
 * @Param:  - IN arg_NaptMode: NAPT mode flag.
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_set_napt_mode(
                ix_cc_natpt_naptmode arg_NaptMode,
                ix_cc_natpt_cb arg_Callback,
                void * arg_pClientContext);


/**
 * NAME: ix_cc_natpt_async_add_v4v6port_mapping
 *
 * DESCRIPTION:This message helper primitive helps client to add a 
 * IPv4addr/IPv6Addr/PortNubmer mapping.
 * 
 * @Param:  - IN arg_pv4v6portMap: Pointer to a v4 to v6 address with port 
 *                                 number mapping structure.
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_add_v4v6port_mapping(
                ix_cc_natpt_v4v6portmap * arg_pv4v6portMap,
                ix_cc_natpt_cb arg_Callback,
                void * arg_pClientContext);


/**
 * NAME: ix_cc_natpt_async_remove_v4v6port_mapping
 *
 * DESCRIPTION:This message helper primitive helps client to remove a 
 * IPv4addr/IPv6Addr/PortNubmer mapping.
 * 
 * @Param:  - IN arg_pv4v6portMap: Pointer to a v4 to v6 address with port 
 *                                 number mapping structure.
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_remove_v4v6port_mapping(
                ix_cc_natpt_v4v6portmap * arg_pv4v6portMap,
                ix_cc_natpt_cb arg_Callback,
                void * arg_pClientContext);


/**
 * NAME: ix_cc_natpt_async_get_v4v6port_mapping
 *
 * DESCRIPTION:This message helper primitive helps client to get current
 * list of IPv4addr/IPv6Addr/PortNubmer mappings.
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_get_v4v6port_mapping(
                ix_cc_natpt_cb_get_v4v6port_mapping arg_Callback,
                void * arg_pClientContext);


/**
 * NAME: ix_cc_natpt_async_get_active_sessions
 *
 * DESCRIPTION:This message helper primitive helps client to get current
 * list of active NATPT sessions.
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_get_active_sessions (
                ix_cc_natpt_cb_get_active_sessions arg_Callback,
                void * arg_pClientContext);
    


/* 
 *****************************************************************************
 * This section defines the prototypes for the init/fini and handler funcs
 *****************************************************************************
 */


/**
 * NAME: ix_cc_natpt_init
 *
 * DESCRIPTION: This primitive shall be called and returned successfully 
 * before requesting any service from the NAT-PT core component. This 
 * primitive should be called only once to initialize NAT-PT component.
 * This function performs allocation of memory for symbols to be patched,
 * Creation of 64 bit counters, registration of packet and message handlers,
 * initialization and configuration of tables needed by NAT-PT and allocation 
 * and initialization of internal data structures.
 * 
 * @Param:  - IN arg_hCcHandle - handle to NAT-PT core component, created 
 * by core component infrastructure; this shall be used later to get other 
 * (to add event handler) services from core component infrastructure.
 *
 * @Param:  - OUT arg_ppContext - location where the pointer to the control 
 * block allocated by NAT-PT core component will be stored. The control block 
 * is internal to NAT-PT core component and it contains information about 
 * NAT-PT internal data structures, allocated memory and other relevant 
 * information. Later this pointer to be passed into ix_cc_natpt_fini 
 * function for freeing memory and other house keeping operations when 
 * IPv6 component is being destroyed.  
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOM   - memory allocation failure      
 * IX_CC_NATPT_ERROR_FAILED_PATCHING   - patching failures                      
 * IX_CC_ERROR_OOM_64BIT_COUNTER - 64 bit counter creation failure
 * IX_CC_NATPT_ERROR_REGISTRY  - registry access failures
 * IX_CC_NATPT_ERROR_CCI - failure from CCI 
 * 
 */

ix_error ix_cc_natpt_init(
                         ix_cc_handle arg_hCcHandle,
                         void** arg_ppContext
                        );



/**
 * NAME: ix_cc_natpt_fini
 *
 * DESCRIPTION: This primitive shall be called to terminate services from 
 * NAT-PT forwarder component. This primitive frees memory allocated during 
 * initialization, free all the created resources: 
 * delete 64 bit counter. If client claims for any services from NAT_PT 
 * after calling this function, then the behavior is undefined.
 * 
 * @Param:  - IN arg_hCcHandle - handle to NAT-PT core component
 *
 * @Param:  - IN arg_pContext - pointer to the control block memory allocated 
 * earlier in ix_cc_natpt_init function. 
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_NATPT_ERROR_INVALID_INPUT_PARAM - invalid input parameter(s)
 * IX_CC_ERROR_OOM   - failure of memory deletion                
 * IX_CC_ERROR_OOM_64BIT_COUNTER - failure of  64 bit counter deletion 
 * IX_CC_NATPT_ERROR_CCI - failure from CCI
 */

ix_error ix_cc_natpt_fini(ix_cc_handle arg_hCcHandle,
                          void* arg_pContext);


/**
 * NAME: ix_cc_natpt_msg_handler
 *
 * DESCRIPTION: This primitive is the message handler function for NAT-PT core 
 * component. NAT-PT core component receives messages from other core 
 * components through this message handler function and internally it 
 * calls appropriate library function to process the message. This message 
 * handler shall be used to update dynamic properties.
 * 
 * @Param:  - IN arg_hDataToken - buffer handle embedding information for 
 *                                the message passed in arg_UserData.
 *
 * arg_UserData - message type
 *
 * @Param:  - OUT - arg_pComponentContext: pointer to NAT-PT core component 
 *                  context that will be passed to the core component when 
 *                  a message arrives. This context was defined by the core 
 *                  component and passed to core components infrastructure 
 *                  through ix_cc_natpt_init function.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - null argument
 * IX_CC_NATPT_ERROR_INVALID_CONTEXT - context is unusable
 * IX_CC_ERROR_UNDEFINED_MSG - un supported message
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - error from message support library
 * 
 */
ix_error ix_cc_natpt_msg_handler(
                                ix_buffer_handle arg_hDataToken,
                                ix_uint32 arg_UserData,
                                void* arg_pContext
                                );



/**
 * NAME: ix_cc_natpt_microblock_pkt_handler
 *
 * DESCRIPTION: This is the only registered function to receive exception 
 * packets from NAT-PT microblock.
 * 
 * @Param:  - IN arg_hDataToken: handle to a buffer which contains exception 
 * packets from NAT-PT microblock
 * @Param: - IN arg_ExceptionCode: will be ignored.
 * @Param: - IN arg_pContext: pointer to NAT-PT core component context that 
 * will be passed to the core component when a packet arrives. This context 
 * was defined by the core component and passed to core components 
 * infrastructure through the ix_cc_natpt_init function.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_NATPT_ERROR_CCI - error from CCI
 */

ix_error ix_cc_natpt_microblock_pkt_handler(
                                ix_buffer_handle arg_hDataToken,  
                                ix_uint32 arg_ExceptionCode,
                                void* arg_pContext);



/* 
 *****************************************************************************
 * This section defines the prototypes for all the NATPT library functions.
 *****************************************************************************
 */


/**
 * NAME: ix_cc_natpt_set_config_parameters
 *
 * DESCRIPTION:This library primitive helps client to set global
 * configuration parameters
 * 
 * @Param:  - IN arg_pConfig: pointer to config parameters structure.
 * @Param:  - IN arg_pContext: pointer to NATPT context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_INVALID_INPUT_PARAM - the information pointed to by 
 *              arg_pConfig is invalid.
 */
ix_error ix_cc_natpt_set_config_parameters(
                ix_cc_natpt_config_params * arg_pConfig,
                void * arg_pContext);


/**
 * NAME: ix_cc_natpt_get_config_parameters
 *
 * DESCRIPTION:This library primitive helps client to get global
 * configuration parameters.
 * 
 * @Param:  - IN arg_pContext: pointer to NATPT context. 
 *
 * @Param:  - OUT arg_pConfig: Pointer to configuration parameters structure 
 *                             that will be filled.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 */
ix_error ix_cc_natpt_get_config_parameters(
                ix_cc_natpt_config_params * arg_pConfig,
                void * arg_pContext);


/**
 * NAME: ix_cc_natpt_add_static_mapping
 *
 * DESCRIPTION:This library primitive add static ipv6/ipv4 address
 * mapping.
 * 
 * @Param:  - IN arg_pv6v4Map: Pointer to a v6 v4 address mapping structure.
 * @Param:  - IN arg_pContext: pointer to NATPT context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_INVALID_INPUT_PARAM - the information pointed to by 
 *              arg_pv6v4Map is invalid.
 */
ix_error ix_cc_natpt_add_static_mapping(
                ix_cc_natpt_static_v6v4map * arg_pv6v4Map,
                void * arg_pContext);


/**
 * NAME: ix_cc_natpt_remove_static_mapping
 *
 * DESCRIPTION:This library primitive remove static ipv6/ipv4 address
 * mapping.
 * 
 * @Param:  - IN arg_pv6v4Map: Pointer to a v6 v4 address mapping structure.
 * @Param:  - IN arg_pContext: pointer to NATPT context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_INVALID_INPUT_PARAM - the information pointed to by 
 *              arg_pv6v4Map is invalid.
 */
ix_error ix_cc_natpt_remove_static_mapping(
                ix_cc_natpt_static_v6v4map * arg_pv6v4Map,
                void * arg_pContext);


/**
 * NAME: ix_cc_natpt_get_static_mapping
 *
 * DESCRIPTION:This library primitive get static ipv6/ipv4 address
 * mapping.
 * 
 * @Param:  - IN arg_pContext: pointer to NATPT context. 
 *
 * @Param:  - IN/OUT arg_NumItems: On input, the number of entries that the 
 *                   buffer pointed to by arg_pv6v4MapArray can accommodate.  
 *                   On output, the actual number of entries written to the 
 *                   buffer.  If IX_CC_NATPT_ERROR_BUFFER_TOO_SMALL is 
 *                   returned, then this number will be the total number 
 *                   of entries that would have been written to the buffer 
 *                   if it had been large enough.
 *
 * @Param:  - OUT arg_pv6v4MapArray : Pointer to buffer into which the array 
 *                of v6-v4 address mappings will be returned.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_INVALID_INPUT_PARAM - arg_NumItems is zero.
 * IX_CC_NATPT_ERROR_BUFFER_TOO_SMALL - more items exist than could fit 
 *                  into the supplied buffer (non-fatal error)
 */
ix_error ix_cc_natpt_get_static_mapping(
                ix_uint32 * arg_NumItems,
                ix_cc_natpt_static_v6v4map * arg_pv6v4MapArray,
                void * arg_pContext);


/**
 * NAME: ix_cc_natpt_add_v4addr_pool
 *
 * DESCRIPTION:This library primitive helps client to add IPv4
 * addresses to pool.
 * 
 * @Param:  - IN arg_NumItems: Number of v4 addrs in the array pointed 
 *                             by the next argument.
 * @Param:  - IN arg_pv4addrArray: Pointer to a v4 address array.
 * @Param:  - IN arg_pContext: pointer to NATPT context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_INVALID_INPUT_PARAM - the information pointed to 
 *              by arg_pv4addrArray is invalid.
 */
ix_error ix_cc_natpt_add_v4addr_pool(
                ix_uint32 arg_NumItems,
                ix_cc_natpt_v4addr * arg_pv4addrArray,
                void * arg_pContext);


/**
 * NAME: ix_cc_natpt_remove_v4addr_pool
 *
 * DESCRIPTION:This library primitive helps client to remove IPv4
 * addresses from pool.
 * 
 * @Param:  - IN arg_NumItems: Number of v4 addrs in the array pointed 
 *                             by the next argument.
 * @Param:  - IN arg_pv4addrArray: Pointer to a v4 address array.
 * @Param:  - IN arg_pContext: pointer to NATPT context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_INVALID_INPUT_PARAM - the information pointed to 
 *              by arg_pv4addrArray is invalid.
 */
ix_error ix_cc_natpt_remove_v4addr_pool(
                ix_uint32 arg_NumItems,
                ix_cc_natpt_v4addr * arg_pv4addrArray,
                void * arg_pContext);


/**
 * NAME: ix_cc_natpt_get_v4addr_pool
 *
 * DESCRIPTION:This library primitive helps client to get current
 * list of ipv4 address in pool.
 * 
 * @Param:  - IN arg_pContext: pointer to NATPT context. 
 * 
 * @Param:  - IN/OUT arg_NumItems: On input, the number of entries that the 
 *                   buffer pointed to by arg_pv4addrArray can accommodate.
 *                   On output, the actual number of entries written to the 
 *                   buffer.  If IX_CC_NATPT_ERROR_BUFFER_TOO_SMALL is 
 *                   returned, then this number will be the total number of 
 *                   entries that would have been written to the buffer if 
 *                   it had been large enough.
 *
 * @Param:  - OUT arg_pv4addrArray : Pointer to buffer into which the array 
 *                of IPv4 addresses will be returned.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_INVALID_INPUT_PARAM - arg_NumItems is zero.
 * IX_CC_NATPT_ERROR_BUFFER_TOO_SMALL - more items exist than could fit 
 *              into the supplied buffer (non-fatal error)
 */
ix_error ix_cc_natpt_get_v4addr_pool(
                ix_uint32 * arg_NumItems,
                ix_cc_natpt_v4addr * arg_pv4addrArray,
                void * arg_pContext);


/**
 * NAME: ix_cc_natpt_set_napt_mode
 *
 * DESCRIPTION:This library primitive helps client to turn NAPT
 * mode ON/OFF.
 * 
 * @Param:  - IN arg_NaptMode: NAPT mode flag.
 * @Param:  - IN arg_pContext: pointer to NATPT context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 */
ix_error ix_cc_natpt_set_napt_mode(
                ix_cc_natpt_naptmode arg_NaptMode,
                void * arg_pContext);


/**
 * NAME: ix_cc_natpt_add_v4v6port_mapping
 *
 * DESCRIPTION:This library primitive helps client to add a 
 * IPv4addr/IPv6Addr/PortNubmer mapping.
 * 
 * @Param:  - IN arg_pv4v6portMap: Pointer to a v4 to v6 address with port 
 *                                 number mapping structure.
 * @Param:  - IN arg_pContext: pointer to NATPT context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_INVALID_INPUT_PARAM - the information pointed to by 
 *              arg_pv4v6portMap is invalid.
 */
ix_error ix_cc_natpt_add_v4v6port_mapping(
                ix_cc_natpt_v4v6portmap * arg_pv4v6portMap,
                void * arg_pContext);


/**
 * NAME: ix_cc_natpt_remove_v4v6port_mapping
 *
 * DESCRIPTION:This library primitive helps client to remove a 
 * IPv4addr/IPv6Addr/PortNubmer mapping.
 * 
 * @Param:  - IN arg_pv4v6portMap: Pointer to a v4 to v6 address with port 
 *                                 number mapping structure.
 * @Param:  - IN arg_pContext: pointer to NATPT context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_INVALID_INPUT_PARAM - the information pointed to by 
 *              arg_pv4v6portMap is invalid.
 */
ix_error ix_cc_natpt_remove_v4v6port_mapping(
                ix_cc_natpt_v4v6portmap * arg_pv4v6portMap,
                void * arg_pContext);


/**
 * NAME: ix_cc_natpt_get_v4v6port_mapping
 *
 * DESCRIPTION:This library primitive helps client to get current
 * list of IPv4addr/IPv6Addr/PortNubmer mappings.
 * 
 * @Param:  - IN arg_pContext: pointer to NATPT context. 
 * 
 * @Param:  - IN/OUT arg_NumItems: On input, the number of entries that the 
 *                   buffer pointed to by arg_pv4v6portMapArray can 
 *                   accommodate.
 *                   On output, the actual number of entries written to the 
 *                   buffer.  If IX_CC_NATPT_ERROR_BUFFER_TOO_SMALL is 
 *                   returned, then this number will be the total number of 
 *                   entries that would have been written to the buffer if 
 *                   it had been large enough.
 *
 * @Param:  - OUT arg_pv4v6portMapArray : Pointer to buffer into which the 
 *                array of IPv4/IPv6/Port mappings will be returned.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_INVALID_INPUT_PARAM - arg_NumItems is zero.
 * IX_CC_NATPT_ERROR_BUFFER_TOO_SMALL - more items exist than could fit 
 *              into the supplied buffer (non-fatal error)
 */
ix_error ix_cc_natpt_get_v4v6port_mapping(
                ix_uint32 * arg_NumItems,
                ix_cc_natpt_v4v6portmap * arg_pv4v6portMapArray,
                void * arg_pContext);


/**
 * NAME: ix_cc_natpt_get_active_sessions
 *
 * DESCRIPTION:This library primitive helps client to get current
 * list of active NATPT sessions.
 * 
 * @Param:  - IN arg_pContext: pointer to NATPT context. 
 * 
 * @Param:  - IN/OUT arg_NumItems: On input, the number of entries that the 
 *                   buffer pointed to by arg_pSessionArray can 
 *                   accommodate.
 *                   On output, the actual number of entries written to the 
 *                   buffer.  If IX_CC_NATPT_ERROR_BUFFER_TOO_SMALL is 
 *                   returned, then this number will be the total number of 
 *                   entries that would have been written to the buffer if 
 *                   it had been large enough.
 *
 * @Param:  - OUT arg_pSessionArray : Pointer to buffer into which the 
 *                array of active NATPT sessions will be returned.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_INVALID_INPUT_PARAM - arg_NumItems is zero.
 * IX_CC_NATPT_ERROR_BUFFER_TOO_SMALL - more items exist than could fit 
 */
ix_error ix_cc_natpt_get_active_sessions (
                ix_uint32 * arg_NumItems,
                ix_cc_natpt_session * arg_pSessionArray,
                void * arg_pContext);
    

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_NATPT_H__) */
