/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.x for the IXP2xxx Network Processors
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      Network Address Translation/Protocol Translation Core Component.
 *
 * = FILENAME
 *      ix_cc_natpt_fini.h
 *
 * = DESCRIPTION
 *      This file defines function prototypes required for De-init of 
 *      NATPT CC module.
 *
 * = AUTHOR
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      03/03/03
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: ix_cc_natpt_fini.h,v 1.5 2003/10/31 19:31:21 ktseng Exp $
 */


#if !defined(__IX_CC_NATPT_FINI_H__)
#define __IX_CC_NATPT_FINI_H__


#if defined(__cplusplus)
extern "C" {
#endif /* end defined(__cplusplus) */


/**
 * NAME: _ix_cc_natpt_unregister_message_handler
 *
 * DESCRIPTION: This primitive will be used to unregister message handler. 
 * It calls CCI services in level 1 systema and resore manager services in
 * level 0 system.
 *
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_NATPT_ERROR_CCI - error from CCI
 * 
 */

ix_error _ix_cc_natpt_unregister_message_handler(void);


/**
 * NAME: _ix_cc_natpt_unregister_cci_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to unregister packet 
 * handler with CCI. 
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_NATPT_ERROR_CCI - error from CCI
 * 
 */

ix_error _ix_cc_natpt_unregister_cci_packet_handlers(void);

/**
 * NAME: _ix_cc_natpt_unregister_rm_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to unregister packet handler with 
 * resource manager
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_NATPT_ERROR_CCI - error from RM
 * 
 */

ix_error _ix_cc_natpt_unregister_rm_packet_handlers(void);


/**
 * NAME: _ix_cc_natpt_unregister_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to unregister packet handler 
 * with CCI/Resource manager
 *
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_NATPT_ERROR_CCI - error from CCI/RM
 * 
 */

ix_error _ix_cc_natpt_unregister_packet_handlers(void);


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_NATPT_FINI_H__) */
