/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.x for the IXP2xxx Network Processors
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      IPv4/v6 Translation Core Component
 *
 * = FILENAME
 *      ix_cc_natpt_header_defs.h
 *
 * = DESCRIPTION
 *   This file contains definition of the IPv4 and IPv6 header structures
 *   needed for translation.   
 *
 *
 * = CREATION TIME
 *      03/03/03
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: ix_cc_natpt_header_defs.h,v 1.10 2003/09/03 23:31:27 ktseng Exp $
 */

#if !defined(__IX_CC_NATPT_HEADER_DEFS_H__)
#define __IX_CC_NATPT_HEADER_DEFS_H__

/**
 * Preprocessor symbol and macro definitions.
 */
#if !(_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
#include <stdio.h>
#endif

#include <ix_ossl.h>
#include <ix_rm.h>
#include <ix_cc.h>
#include <ix_netmacros.h>
#include "ix_cc_error.h"
#include "bindings.h"


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */


/** 
 * General defines
 */

#define IX_CC_NATPT_MIN_IPV4_LEN         20
#define IX_CC_NATPT_MAX_IPV4_LEN         60
#define IX_CC_NATPT_IPV6_HDR_LEN         40
#define IX_CC_NATPT_IP_PROTO_ICMP        1
#define IX_CC_NATPT_IP_PROTO_IPV6        41
#define IX_CC_NATPT_ICMP_HDR_LEN         8
#define IX_CC_NATPT_IPV6ADDRLEN_BYTES    16

/*Next header values*/
#define IX_CC_NATPT_NEXT_HDR_HOPBYHOP           0
#define IX_CC_NATPT_NEXT_HDR_ROUTING            43
#define IX_CC_NATPT_NEXT_HDR_FRAGMENT           44
#define IX_CC_NATPT_NEXT_HDR_ENCAPSULATION      50
#define IX_CC_NATPT_NEXT_HDR_AUTHENTICATION     51
#define IX_CC_NATPT_NEXT_HDR_NONE               59
#define IX_CC_NATPT_NEXT_HDR_DSTOPT             60
#define IX_CC_NATPT_NEXT_HDR_TCP                6
#define IX_CC_NATPT_NEXT_HDR_UDP                17
#define IX_CC_NATPT_NEXT_HDR_ICMP               58

#define IX_CC_NATPT_IPPROTO_ICMP                1

#define IX_CC_NATPT_ICMPV4_MAX_PAYLOAD          576
#define IX_CC_NATPT_ICMPV6_MAX_PAYLOAD          1214

/*ICMPv6 Message types*/
#define IX_CC_ICMPV6_TYPE_DEST_UNREACHABLE      1
#define IX_CC_ICMPV6_TYPE_PACKET_TOO_BIG        2
#define IX_CC_ICMPV6_TYPE_TIME_EXCEEDED         3
#define IX_CC_ICMPV6_TYPE_PARAMETER_PROBLEM     4
#define IX_CC_ICMPV6_TYPE_ECHO_REQUEST          128
#define IX_CC_ICMPV6_TYPE_ECHO_REPLY            129

#define IX_CC_ICMPV6_CODE_ZERO              0

/*Codes for the Destination Unreachable Message type*/
#define IX_CC_ICMPV6_DU_CODE_NOROUTE            0
#define IX_CC_ICMPV6_DU_CODE_COMM_PROHIBITED    1
#define IX_CC_ICMPV6_DU_CODE_NOT_ASSIGNED       2
#define IX_CC_ICMPV6_DU_CODE_ADDR_UNREACHABLE   3
#define IX_CC_ICMPV6_DU_CODE_PORT_UNREACHABLE   4
    
/*Codes for the Parameter  Problem Message type*/
#define IX_CC_ICMPV6_PP_CODE_WRONG_HDR_FIELD     0
#define IX_CC_ICMPV6_PP_CODE_NEXT_HDR_UNKNOWN    1
#define IX_CC_ICMPV6_PP_CODE_OPTION_UNKNOWN      2


/*ICMPv4 Message types*/
#define IX_CC_ICMPV4_TYPE_ECHO_REQUEST          8
#define IX_CC_ICMPV4_TYPE_ECHO_REPLY            0
#define IX_CC_ICMPV4_TYPE_DEST_UNREACHABLE      3
#define IX_CC_ICMPV4_TYPE_TIME_EXCEEDED         11
#define IX_CC_ICMPV4_TYPE_PARAMETER_PROBLEM     12

#define IX_CC_ICMPV4_CODE_ZERO              0

/*Codes for the Destination Unreachable Message type*/
#define IX_CC_ICMPV4_DU_CODE_NET_UNREACHABLE        0
#define IX_CC_ICMPV4_DU_CODE_HOST_UNREACHABLE       1
#define IX_CC_ICMPV4_DU_CODE_PROTOCOL_UNREACHABLE   2
#define IX_CC_ICMPV4_DU_CODE_PORT_UNREACHABLE       3
#define IX_CC_ICMPV4_DU_CODE_FRAGMENTATION          4
#define IX_CC_ICMPV4_DU_CODE_SOURCE_ROUTE_FAIL      5
#define IX_CC_ICMPV4_DU_CODE_SIX                    6
#define IX_CC_ICMPV4_DU_CODE_SEVEN                  7
#define IX_CC_ICMPV4_DU_CODE_EIGHT                  8
#define IX_CC_ICMPV4_DU_CODE_NINE                   9
#define IX_CC_ICMPV4_DU_CODE_TEN                    10
#define IX_CC_ICMPV4_DU_CODE_ELEVEN                 11
#define IX_CC_ICMPV4_DU_CODE_TWELVE                 12
    
/*Codes for the Time Exceeded Message type*/
#define IX_CC_ICMPV4_TE_CODE_TTL_EXCEEDED           0
#define IX_CC_ICMPV4_TE_CODE_REASSEMBLY_EXCEEDED    1

/*Codes for the Parameter  Problem Message type*/
#define IX_CC_ICMPV4_PP_CODE_POINTER_INDICATES      0


/**
 * Type definitions.
 */

/**
 * TYPENAME: ix_cc_natpt_ipv4_header
 * 
 * DESCRIPTION: This struct defines the format of IP header 
 *
 */

typedef struct ix_s_cc_natpt_ipv4_header
{
    ix_uint8    vers_hdrLength;
    ix_uint8    tos;
    ix_uint16   totalLength;
    ix_uint16   id;
    ix_uint16   flags_fragOffset;
    ix_uint8    ttl;
    ix_uint8    protocol;
    ix_uint16   checksum;
    ix_uint32   sourceAddress;
    ix_uint32   destAddress;
    ix_uint32   options[10];
} __attribute__ ((packed)) ix_cc_natpt_ipv4_header;

/**
 * TYPENAME: ix_cc_natpt_icmp_header
 * 
 * DESCRIPTION: This struct defines the format of ICMP header 
 *
 */
typedef struct ix_s_cc_natpt_icmp_header
{
    ix_uint8    type;
    ix_uint8    code;
    ix_uint16   checksum;
    ix_uint32   extra;
} __attribute__ ((packed)) ix_cc_natpt_icmp_header;


/**
 * TYPENAME: ix_cc_natpt_ipv6_header
 * 
 * DESCRIPTION: This struct defines the format of IPv6 header 
 *
 */

typedef struct ix_s_cc_natpt_ipv6_header
{
#if IX_BYTE_ORDER == IX_LITTLE_ENDIAN 
    ix_uint32 flow_label:20;
    ix_uint32 traffic_class:8;
    ix_uint32 version:4;
#elif IX_BYTE_ORDER == IX_BIG_ENDIAN
    ix_uint32 version:4;
    ix_uint32 traffic_class:8;
    ix_uint32 flow_label:20;
#endif
    ix_uint16 payloadLength;
    ix_uint8  nextHeader;
    ix_uint8  hopLimit;
    ix_uint128 sourceAddress;
    ix_uint128 destAddress;
} __attribute__ ((packed)) ix_cc_natpt_ipv6_header;


/**
 * TYPENAME: ix_cc_natpt_ipv6_extn_hdr_common
 *
 * DESCRIPTION: Defines the common part of the IP extension headers.
 *
 */
typedef struct ix_s_cc_natpt_ipv6_extn_hdr_common
{
    ix_uint8    nextHeader;
    ix_uint8    hdrLength;
}__attribute__ ((packed)) ix_cc_natpt_ipv6_extn_hdr_common;


/**
 * TYPENAME: ix_cc_natpt_ipv6_pseudo_header
 *
 * DESCRIPTION: The NATPT pseudo header required for checksum calculations
 *
 */
typedef struct ix_s_cc_natpt_ipv6_pseudo_header
{
    ix_uint128     srcAddress;
    ix_uint128     dstAddress;
    ix_uint32      pktLength;
    ix_uint32      unused:24, nextHdr:8;
} __attribute__ ((packed)) ix_cc_natpt_ipv6_pseudo_header;


/**
 * TYPENAME: ix_cc_natpt_ipv6_fragment_hdr
 *
 * DESCRIPTION: Defines the ipv6 fragment hdr
 *
 */
typedef struct ix_s_cc_natpt_ipv6_fragment_hdr
{
    ix_uint8    nextHeader;
    ix_uint8    reservd;
    ix_uint16   fragOffset:13;
    ix_uint16   reserved:2;
    ix_uint16   mFlag:1;
    ix_uint32   ident;
}__attribute__ ((packed)) ix_cc_natpt_ipv6_fragment_hdr;



/**
 * TYPENAME: ix_cc_natpt_udp_hdr
 *
 * DESCRIPTION: Defines the UDP hdr
 *
 */
typedef struct ix_s_cc_natpt_udp_hdr
{
    ix_uint16   srcPort;
    ix_uint16   dstPort;
    ix_uint16   length;
    ix_uint16   checksum;
}__attribute__ ((packed)) ix_cc_natpt_udp_hdr;






/*
 * Prototypes for interface functions.
 */


/**
 * Exported variables.
 */

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_NATPT_HEADER_DEFS_H__) */
