/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.x for the IXP2xxx Network Processors
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      IPv6 NATPT Core Component - Definitions for translation of ICMP
 *      packets
 *
 * = FILENAME
 *      ix_cc_natpt_icmp.h
 *
 * = DESCRIPTION
 *      This file defines all data types, macros and function prototypes
 *      that support handling of ICMP messages that require translation.
 *
 * = CREATION TIME
 *      03/03/03
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: ix_cc_natpt_icmp.h,v 1.4 2003/04/30 01:06:33 sshegde Exp $
 */

#if !defined(__IX_CC_NATPT_ICMP_H__)
#define __IX_CC_NATPT_ICMP_H__

#include "cc/ix_cc_ipv6.h"

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */


/**
 * Defines for ICMP types that are checked for by tunneling.
 */

#define IX_CC_V6V4_ICMP_DEST_UNREACH    3
#define IX_CC_V6V4_ICMP_TIME_EXCEEDED   11
#define IX_CC_V6V4_ICMP_PARAM_PROBLEM   12

/**
 * Codes for Destination Unreachable types that are mapped to ICMPv6
 * equivalents.
 */

#define IX_CC_V6V4_ICMP_DU_NETWORK      0
#define IX_CC_V6V4_ICMP_DU_HOST         1
#define IX_CC_V6V4_ICMP_DU_PROTOCOL     2
#define IX_CC_V6V4_ICMP_DU_PORT         3
#define IX_CC_V6V4_ICMP_DU_FRAG         4
#define IX_CC_V6V4_ICMP_DU_COMM_PROHIB  10

/**
 * Codes for Time Exceeded types that are mapped to ICMPv6
 * equivalents.
 */

#define IX_CC_V6V4_ICMP_TE_TTL          0
#define IX_CC_V6V4_ICMP_TE_FRAG_TIMER   1


/** 
 * General defines for ICMP
 */

#define IX_CC_V6V4_ICMP_MIN_DATA        8
#define IX_CC_V6V4_ICMP_HDR_LEN         8
#define IX_CC_V6V4_MIN_IPV4_LEN         20
#define IX_CC_V6V4_MIN_IPV6_LEN         40
#define IX_CC_V6V4_IP_PROTO_ICMP        1
#define IX_CC_V6V4_IP_PROTO_IPV6        41


/**
 * NAME: _ix_cc_natpt_icmp_v6_packet
 *
 * DESCRIPTION: This primitive is used to translate an ICMPv6 packet to ICMPv4
 * packet. Then the packet is given to IPv4 Forwarder.
 *           
 * @Param:  - IN arg_hPacket - handle to buffer containing the packet.
 *
 * @Param:  - IN arg_pContext - pointer to tunneling core component
 * context buffer.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_INTERNAL - internal error 
 * IX_CC_NATPT_ERROR_ICMP_TRANSLATE - could not translate
 * 
 */
ix_error _ix_cc_natpt_icmp_v6_packet(
    ix_buffer_handle arg_hPacket,  
    void* arg_pContext
    );

/**
 * NAME: _ix_cc_natpt_icmp_v4_packet
 *
 * DESCRIPTION: This primitive is used to translate an ICMPv4 packet to ICMPv6
 * packet. Then the packet is given to IPv6 Forwarder.
 *           
 * @Param:  - IN arg_hPacket - handle to buffer containing the packet.
 *
 * @Param:  - IN arg_pContext - pointer to tunneling core component
 * context buffer.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_INTERNAL - internal error 
 * IX_CC_NATPT_ERROR_ICMP_TRANSLATE - could not translate
 * 
 */
ix_error _ix_cc_natpt_icmp_v4_packet(
    ix_buffer_handle arg_hPacket,  
    void* arg_pContext
    );



#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_NATPT_ICMP_H__) */
