/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.x for the IXP2xxx Network Processors
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      Network Address Translation/Protocol Translation Core Component
 *
 * = FILENAME
 *      ix_cc_natpt_init.h
 *
 * = DESCRIPTION
 *      This file defines all data types and function prototypes required 
 *      internaly by the NAT-PT CC module.
 *
 * = AUTHOR
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      03/03/03
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: ix_cc_natpt_init.h,v 1.16 2003/09/03 23:25:40 ktseng Exp $
 */

#if !defined(__IX_CC_NATPT_INIT_H__)
#define __IX_CC_NATPT_INIT_H__

#include "ix_rm.h"
#include "ix_ossl.h"
#include "ix_cc.h"
#include "cc/ix_cc_index_pool.h"
#include "cc/ix_cc_natpt.h"

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
#include "cc/natpt/internal/ix_cc_natpt_slist.h"
#else
#include "cc/ix_cc_slist.h"
#endif

#if defined(__cplusplus)
extern "C" {
#endif /* end defined(__cplusplus) */


/* Registry Values */
#define IX_CC_NATPT_REGISTRY_KEY_NAT_TABLE_SIZE  "NATPT/NAT_TABLE_SIZE"
#define IX_CC_NATPT_REGISTRY_KEY_NAT_HASH_INDEX_TABLE_SIZE  "NATPT/NAT_HASH_INDEX_TABLE_SIZE"
#define IX_CC_NATPT_REGISTRY_KEY_NAT_COLLISION_TABLE_SIZE  "NATPT/NAT_COLLISION_TABLE_SIZE"
#define IX_CC_NATPT_REGISTRY_KEY_ME_FOR_NATPT  "NATPT/ME_FOR_NATPT"
#define IX_CC_NATPT_REGISTRY_KEY_NATPT_PREFIX_LW0 "NATPT/NATPT_PREFIX_LW0"
#define IX_CC_NATPT_REGISTRY_KEY_NATPT_PREFIX_LW1 "NATPT/NATPT_PREFIX_LW1"
#define IX_CC_NATPT_REGISTRY_KEY_NATPT_PREFIX_LW2 "NATPT/NATPT_PREFIX_LW2"

/* 
 * Default Values for Registry 
 */

/* NAT table size hint */
#define IX_CC_NATPT_NAT_TABLE_SIZE_HINT 256 

/* NAT hash table size hint */
#define IX_CC_NATPT_NAT_HASH_TABLE_SIZE_HINT 128 

/* NAT hash collision table size hint */
#define IX_CC_NATPT_NAT_COLLISION_TABLE_SIZE_HINT 128 


/* V4 address pool list size hint */
#define IX_CC_NATPT_V4ADDRPOOL_SIZE_HINT 32 

/* V4/V6 Static Map list size hint */
#define IX_CC_NATPT_STATICMAP_SIZE_HINT 8 

/* Active Sessions list size hint */
#define IX_CC_NATPT_ACTIVE_SESSIONS_SIZE_HINT 128 

/* NAT TABLE memory type: SRAM or DRAM */
#define IX_CC_NATPT_NAT_TABLE_MEMORY_TYPE  IX_MEMORY_TYPE_SRAM

/* NAT TABLE SRAM/DRAM Channel number */
#define IX_CC_NATPT_NAT_TABLE_CHANNEL 0


/* number of microengines: 8 for IXP2400 */
#define IX_CC_NATPT_NUMBER_OF_MICROENGINES 8 

/* Micro engine allocated for IPv6 are 0x01,0x02,0x11,0x12 */
#define IX_CC_NATPT_ME 0x66 

/* Maximum size of symbol name */
#define IX_CC_NATPT_SYMBOL_SIZE 128 

/* 
 * Number of Symbols to be patched
 */
#define IX_CC_NATPT_NUMBER_OF_SYMBOLS 14

/* 
 * Symbol names (strings) to be patched. These define the values
 * of various symbols used by the core and the microblock
 */

#define IX_CC_NATPT_PATCHNAME_NAT_TABLE_BASE  "NATPT_NAT_TABLE_BASE" 
#define IX_CC_NATPT_PATCHNAME_NAT_V6V4_HASH_TABLE_BASE  "NATPT_NAT_V6V4_HASH_TABLE_BASE" 
#define IX_CC_NATPT_PATCHNAME_NAT_V4V6_HASH_TABLE_BASE  "NATPT_NAT_V4V6_HASH_TABLE_BASE" 
#define IX_CC_NATPT_PATCHNAME_NAT_COLL_HASH_TABLE_BASE  "NATPT_NAT_COLL_HASH_TABLE_BASE" 
#define IX_CC_NATPT_PATCHNAME_ID   "THIS_BLADE_ID" 

#define IX_CC_NATPT_PATCHNAME_H48_MULT_0 "NATPT_HASH_MUL_48_0" 
#define IX_CC_NATPT_PATCHNAME_H48_MULT_1 "NATPT_HASH_MUL_48_1" 
  
#define IX_CC_NATPT_PATCHNAME_H128_MULT_0 "NATPT_HASH_MUL_128_0" 
#define IX_CC_NATPT_PATCHNAME_H128_MULT_1 "NATPT_HASH_MUL_128_1" 
#define IX_CC_NATPT_PATCHNAME_H128_MULT_2 "NATPT_HASH_MUL_128_2" 
#define IX_CC_NATPT_PATCHNAME_H128_MULT_3 "NATPT_HASH_MUL_128_3" 

#define IX_CC_NATPT_PATCHNAME_PREFIX_0 "NATPT_PREFIX_0" 
#define IX_CC_NATPT_PATCHNAME_PREFIX_1 "NATPT_PREFIX_1" 
#define IX_CC_NATPT_PATCHNAME_PREFIX_2 "NATPT_PREFIX_2" 

#define IX_CC_NATPT_PATCHNAME_SRAM_COUNTER_BASE  "NATPT_SRAM_COUNTER_BASE" 


#define IX_CC_NATPT_HASH48_MULTIPLIER_LW0       0x43434343
#define IX_CC_NATPT_HASH48_MULTIPLIER_LW1       0x43434343

#define IX_CC_NATPT_HASH128_MULTIPLIER_LW0       0x43434343
#define IX_CC_NATPT_HASH128_MULTIPLIER_LW1       0x43434343
#define IX_CC_NATPT_HASH128_MULTIPLIER_LW2       0x43434343
#define IX_CC_NATPT_HASH128_MULTIPLIER_LW3       0x43434343

#define IX_CC_NATPT_PREFIX_LW0      0x0
#define IX_CC_NATPT_PREFIX_LW1      0x0
#define IX_CC_NATPT_PREFIX_LW2      0x0000ffff

#define IX_CC_NATPT_MILLISECONDS_PER_SECOND 1000 /* millseconds */

#define IX_CC_NATPT_DEFAULT_IDLE_CHECK_INTERVAL 300

/**
 * TYPENAME: ix_cc_natpt_prop_data
 * 
 * DESCRIPTION: This structure defines format for dynamic property data.
 *
 */

typedef struct ix_s_cc_natpt_prop_data
{
    ix_uint32 ununsed;

} ix_cc_natpt_prop_data;


/*
 * TYPENAME: ix_cc_natpt_context
 * 
 * DESCRIPTION: This structure defines format for NAT-PT core component context
 *
 */
typedef struct ix_s_cc_natpt_context
{
    ix_cc_handle hNatPtHandle;   /* Handle of NAT-PT core component */
    ix_buffer_free_list_handle hBufferFreeList; /* handle of free list buffer */

    ix_ossl_mutex_t NatPtMutex; /* mutex id */

    ix_uint32 idleCheckInterval; /* interval for timing out entries */
#if !defined(IX_EXCLUDE_CCI)
    ix_event_handle hEventHandle; /* Event handler handle  */
#endif

    ix_cc_natpt_naptmode naptMode; /* NAPT ON or OFF */

    LPLIST staticMapList;
    LPLIST v4PoolList;

    /* base address of NAT table for microblock patching */
    ix_uint32* pMbNatTableBase;
    /* index pool for NAT table */
    ix_cc_index_pool natTableIndexPool;

    /* base address of NAT v6-v4 hash table for microblock patching */
    ix_uint32* pMbNatV6V4HashTableBase;
    /* index pool for NAT v6-v4 hash table */
    ix_cc_index_pool natV6V4HashTableIndexPool;

    /* base address of NAT v4-v6 hash table for microblock patching */
    ix_uint32* pMbNatV4V6HashTableBase;
    /* index pool for NAT v4-v6 hash table */
    ix_cc_index_pool natV4V6HashTableIndexPool;

    /* base address of NAT hash collision table for microblock patching */
    ix_uint32* pMbNatCollisionTableBase;
    /* index pool for NAT hash collision table */
    ix_cc_index_pool natCollisionTableIndexPool;

} ix_cc_natpt_context;

/**
 * TYPENAME: ix_cc_natpt_registry_data
 *
 * DESCRIPTION: This structure defines format for registry data.
 *
 */

typedef struct ix_s_cc_natpt_registry_data
{
    ix_uint32 localBladeId;          /* id of our blade */
    ix_uint32 meNumbers;             /* Me numbers for NATPT CC */
    ix_uint32 natTableSize;
    ix_uint32 natHashTableSize;
    ix_uint32 natCollisionTableSize;
    ix_uint32 natptPrefix0;
    ix_uint32 natptPrefix1;
    ix_uint32 natptPrefix2;
} ix_cc_natpt_registry_data;


/* Global NAT-PT context */
extern ix_cc_natpt_context* g_pNatPtContext;


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_NATPT_INIT_H__) */
