/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.x for the IXP2xxx Network Processors
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      NATPT Core Component
 *
 * = FILENAME
 *      ix_cc_natpt_lists.h
 *
 * = DESCRIPTION
 *      This file defines all data types, macros and function prototypes
 *      that support management of the IPv4 Address Pool and Static
 *      Mapping.
 *
 * = CREATION TIME
 *      03/03/03
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: ix_cc_natpt_lists.h,v 1.7 2003/04/30 01:06:33 sshegde Exp $
 */

#if !defined(__IX_CC_NATPT_LISTS_H__)
#define __IX_CC_NATPT_LISTS_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */



/**
 * TYPENAME: ix_cc_natpt_v4PoolEntry
 *
 * DESCRIPTION: This structure defines format for registry data.
 *
 */

typedef struct ix_s_cc_natpt_v4PoolEntry
{
    ix_uint32 v4addr;
    ix_uint8 isUsed;
} ix_cc_natpt_v4PoolEntry;


/**
 * TYPENAME: ix_cc_natpt_stV4V6MapEntry
 *
 * DESCRIPTION: This structure defines format for registry data.
 *
 */

typedef struct ix_s_cc_natpt_stV4V6MapEntry
{
    ix_cc_natpt_static_v6v4map v6v4Map;
    ix_uint32 isUsed;
} ix_cc_natpt_stV4V6MapEntry;


extern ix_error _ix_cc_natpt_create_lists(ix_cc_natpt_context* natptContext);
extern ix_error _ix_cc_natpt_cleanup_lists(ix_cc_natpt_context* natptContext);

extern ix_error _ix_cc_natpt_add_v4v6map(
                LPLIST arg_stMapList,
                ix_cc_natpt_static_v6v4map* arg_v6v4Map);
extern ix_error _ix_cc_natpt_remove_v4v6map(
                LPLIST arg_stMapList,
                ix_cc_natpt_static_v6v4map* arg_v6v4Map);
extern ix_error _ix_cc_natpt_get_v4v6map(
                LPLIST arg_stMapList,
                ix_uint32* arg_NumItems,
                ix_cc_natpt_static_v6v4map* arg_v6v4Map);

extern ix_error _ix_cc_natpt_add_v4pool(
                LPLIST arg_poolList,
                ix_uint32 arg_NumItems,
                ix_cc_natpt_v4addr* arg_pv4addrArray);
extern ix_error _ix_cc_natpt_remove_v4pool(
                LPLIST arg_poolList,
                ix_uint32 arg_NumItems,
                ix_cc_natpt_v4addr* arg_pv4addrArray);
extern ix_error _ix_cc_natpt_get_v4pool(
                LPLIST arg_poolList,
                ix_uint32* arg_NumItems,
                ix_cc_natpt_v4addr* arg_pv4addrArray);

extern ix_error _ix_cc_natpt_alloc_ipv4addr(
                LPLIST arg_poolList,
                ix_uint32* arg_ipv4Addr);

extern ix_error _ix_cc_natpt_free_ipv4addr(
                LPLIST arg_poolList,
                ix_uint32 arg_ipv4Addr);

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_NATPT_LISTS_H__) */
