/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.x for the IXP2xxx Network Processors
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      Core Component Infrastructure - message handler and message callbacks
 *
 * = FILENAME
 *      ix_cc_natpt_msg_handler.h
 *
 * = DESCRIPTION
 *      This file defines all data types and function prototypes 
 *      required internaly by the NATPT CC for message handling .
 *
 * = AUTHOR
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      03/03/03
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: ix_cc_natpt_msg_handler.h,v 1.9 2003/04/30 01:06:33 sshegde Exp $
 */

#if !defined(__IX_CC_NATPT_MSG_HANDLER_H__)
#define __IX_CC_NATPT_MSG_HANDLER_H__


#if defined(__cplusplus)
extern "C" {
#endif /* end defined(__cplusplus) */



/**
 * TYPENAME: ix_cc_natpt_v4addr_pool_data
 * 
 * DESCRIPTION: This structure defines the format for the add IPv4 address to
 * IPv4 pool message helper.
 *
 */
typedef struct ix_s_cc_natpt_v4addr_pool_data
{
    ix_uint32 numItems;
    ix_cc_natpt_v4addr* pv4addrArray;
} ix_cc_natpt_v4addr_pool_data;


/**
 * TYPENAME: ix_cc_natpt_general_callback_data
 * 
 * DESCRIPTION: This structure defines the format for general callback data.
 *
 */
typedef struct ix_s_cc_natpt_general_callback_data
{
    ix_cc_natpt_cb pUserCallback;
    void* pUserContext;
} ix_cc_natpt_general_callback_data;


/**
 * TYPENAME: ix_cc_natpt_get_config_parameters_callback_data
 * 
 * DESCRIPTION: This structure defines format for get_config_parameters 
 * callback data.
 *
 */
typedef struct ix_s_cc_natpt_get_config_parameters_callback_data
{
    ix_cc_natpt_cb_get_config_parameters pUserCallback;
    void* pUserContext;
} ix_cc_natpt_get_config_parameters_callback_data;


/**
 * TYPENAME: ix_cc_natpt_get_static_mapping_callback_data
 * 
 * DESCRIPTION: This structure defines format for get_static_mapping 
 * callback data.
 *
 */
typedef struct ix_s_cc_natpt_get_static_mapping_callback_data
{
    ix_cc_natpt_cb_get_static_mapping pUserCallback;
    void* pUserContext;
} ix_cc_natpt_get_static_mapping_callback_data;


/**
 * TYPENAME: ix_cc_natpt_get_v4addr_pool_callback_data
 * 
 * DESCRIPTION: This structure defines format for get_v4addr_pool 
 * callback data.
 *
 */
typedef struct ix_s_cc_natpt_get_v4addr_pool_callback_data
{
    ix_cc_natpt_cb_get_v4addr_pool pUserCallback;
    void* pUserContext;
} ix_cc_natpt_get_v4addr_pool_callback_data;


/**
 * TYPENAME: ix_cc_natpt_get_active_sessions_callback_data
 * 
 * DESCRIPTION: This structure defines format for get_active_sessions
 * callback data.
 *
 */
typedef struct ix_s_cc_natpt_get_active_sessions_callback_data
{
    ix_cc_natpt_cb_get_active_sessions pUserCallback;
    void* pUserContext;
} ix_cc_natpt_get_active_sessions_callback_data;



/**
 * NAME: _ix_cc_natpt_cb_genaral
 *
 * DESCRIPTION: This is a general callback function. The message support
 * library  calls this function to return the status of the requested
 * asynchronous operation.
 * @Param:  - IN  arg_Result: result of the asynchronous operation
 * @Param:  - IN  arg_pContext: context provided by caller
 * @Param:  - IN  arg_pMsg: return message
 * @Param:  - IN  arg_MsgLength: length of the message
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_NATPT_ERROR_CALLBACK - user callback returns error message
 * 
 */

ix_error _ix_cc_natpt_cb_general(
                ix_error arg_Result,
                void* arg_pContext,
                void* arg_pMsg,
                ix_uint32 arg_MsgLength);


/**
 * NAME: _ix_cc_natpt_cb_get_config_parameters
 *
 * DESCRIPTION: This is a callback function for getting configuration
 * parameters. The message support library  calls this function to return 
 * the status of the requested asynchronous operation.
 *
 * @Param:  - IN  arg_Result: result of the asynchronous operation
 * @Param:  - IN  arg_pContext: context provided by caller
 * @Param:  - IN  arg_pMsg: return message
 * @Param:  - IN  arg_MsgLength: length of the message
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_NATPT_ERROR_CALLBACK - user callback returns error message
 * 
 */

ix_error _ix_cc_natpt_cb_get_config_parameters(
                ix_error arg_Result,
                void* arg_pContext,
                void* arg_pMsg,
                ix_uint32 arg_MsgLength);


/**
 * NAME: _ix_cc_natpt_cb_get_static_mapping
 *
 * DESCRIPTION: This is a callback function for getting current static
 * mapping data. The message support library  calls this function to 
 * return the status of the requested asynchronous operation.
 *
 * @Param:  - IN  arg_Result: result of the asynchronous operation
 * @Param:  - IN  arg_pContext: context provided by caller
 * @Param:  - IN  arg_pMsg: return message
 * @Param:  - IN  arg_MsgLength: length of the message
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_NATPT_ERROR_CALLBACK - user callback returns error message
 * 
 */

ix_error _ix_cc_natpt_cb_get_static_mapping(
                ix_error arg_Result,
                void* arg_pContext,
                void* arg_pMsg,
                ix_uint32 arg_MsgLength);


/**
 * NAME: _ix_cc_natpt_cb_get_v4addr_pool
 *
 * DESCRIPTION: This is a callback function for getting current IPv4
 * address pool. The message support library  calls this function to return 
 * the status of the requested asynchronous operation.
 *
 * @Param:  - IN  arg_Result: result of the asynchronous operation
 * @Param:  - IN  arg_pContext: context provided by caller
 * @Param:  - IN  arg_pMsg: return message
 * @Param:  - IN  arg_MsgLength: length of the message
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_NATPT_ERROR_CALLBACK - user callback returns error message
 * 
 */

ix_error _ix_cc_natpt_cb_get_v4addr_pool(
                ix_error arg_Result,
                void* arg_pContext,
                void* arg_pMsg,
                ix_uint32 arg_MsgLength);


/**
 * NAME: _ix_cc_natpt_cb_get_active_sessions
 *
 * DESCRIPTION: This is a callback function for getting active NATPT 
 * sessions. The message support library  calls this function to return 
 * the status of the requested asynchronous operation.
 *
 * @Param:  - IN  arg_Result: result of the asynchronous operation
 * @Param:  - IN  arg_pContext: context provided by caller
 * @Param:  - IN  arg_pMsg: return message
 * @Param:  - IN  arg_MsgLength: length of the message
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_NATPT_ERROR_CALLBACK - user callback returns error message
 * 
 */

ix_error _ix_cc_natpt_cb_get_active_sessions(
                ix_error arg_Result,
                void* arg_pContext,
                void* arg_pMsg,
                ix_uint32 arg_MsgLength);


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_NATPT_MSG_HANDLER_H__) */
