/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.x for the IXP2xxx Network Processors
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      NATPT Core Component
 *
 * = FILENAME
 *      ix_cc_natpt_utils.h
 *
 * = DESCRIPTION
 *      Header file for utility functions for NATPT core component. 
 *
 * = CREATION TIME
 *      03/03/03
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: ix_cc_natpt_utils.h,v 1.8 2003/04/30 01:06:33 sshegde Exp $
 */

#if !defined(__IX_CC_NATPT_UTILS_H__)
#define __IX_CC_NATPT_UTILS_H__

#include <assert.h>

#include "ix_cc_error.h"
#include "ix_cc_natpt_header_defs.h"
#include "cc/ix_cc_ipv6.h"
#include "ix_netmacros.h"


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

/**
 * Prototypes for interface functions.
 */

/**
 * NAME: _ix_cc_natpt_print_ipv6_address
 *
 * DESCRIPTION: This function prints an IPv6 address.
 * 
 * @Param:  - IN pAddr - pointer to the IPv6 address.
 *
 * return:
 * none.
 *
 */
void _ix_cc_natpt_print_ipv6_address(ix_uint128 *pAddr);


/* 
 * The following macro's expect the IPv6 address to be in 
 * Host Byte Order Only.
 */

/* NAME: NATPT_IS_IPV6_ADDRESS_ZERO
 *
 * DESCRIPTION: Check to see if an IPv6 address is zero.
 *
 * @Param: IN arg_src - Pointer to the IPv6 address.
 *
 * return: 1 if it is.
 *         0 Otherwise
 * 
 * NOTE: THIS MACRO EXPECTS THE IPV6 ADDRESS IN HOST BYTE ORDER
 */
ix_uint8
NATPT_IS_IPV6_ADDRESS_ZERO(ix_uint128 *arg_ipv6Addr);


/* NAME: MAKE_IPV4_MAPPED_IPV6_ADDRESS
 *
 * DESCRIPTION: Create IPv4 mapped IPv6 Address
 *
 * @Param: IN arg_ipv4Addr - IPv4 address.
 *
 * @Param: IN arg_ipv6Addr - Pointer to the IPv6 address.
 *
 * return: none.
 *
 * NOTE: THIS MACRO EXPECTS THE ADDRESSES BE IN HOST BYTE ORDER
 */
void
MAKE_IPV4_MAPPED_IPV6_ADDRESS(ix_uint32 arg_ipv4Addr, 
                              ix_uint128 *arg_ipv6Addr);


/* NAME: NATPT_COPY_IPV6_ADDRESS
 *
 * DESCRIPTION: Copies the source IPv6 address into the destination.
 *
 * @Param: IN arg_dst - Pointer to the destination IPv6 address.
 * @Param: IN arg_src - Pointer to the source IPv6 address.
 *
 * return: none.
 *
 * NOTE: THIS MACRO EXPECTS THE IPV6 ADDRESS IN HOST BYTE ORDER
 */

void
NATPT_COPY_IPV6_ADDRESS(ix_uint128 *arg_dst, ix_uint128 *arg_src);

/* 
 * NAME: NATPT_IS_IPV6_ADDRESS_EQUAL
 *
 * DESCRIPTION: Compare two IPv6 addresses. 
 *
 * @Param: IN arg_src - Pointer to the IPv6 address.
 * @Param: OUT arg_dst - Pointer to where the result of the operation 
 *                        should be stored.
 * return: 1 if they are equal
 *         0 Otherwise
 *
 * NOTE: THIS MACRO EXPECTS THE IPV6 ADDRESS IN HOST BYTE ORDER
 */
ix_uint8
NATPT_IS_IPV6_ADDRESS_EQUAL(ix_uint128 *arg_src, ix_uint128 *arg_dst);


/**
 * Exported variables.
 */

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_NATPT_UTILS_H__) */
