/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.x for the IXP2xxx Network Processors
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      NATPT Core Component - fini
 *
 * = FILENAME
 *      ix_cc_natpt_fini.c
 *
 * = DESCRIPTION
 *      The file defines fini interface and internal functions.
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      03/03/03
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */
#define IX_ERROR_FILE_IDENT "$Id: ix_cc_natpt_fini.c,v 1.9 2003/10/31 19:28:36 ktseng Exp $";


/**
 * System defined include files
 */

#include "ix_cc_error.h"
#include "ix_rm.h"
#include "ix_ossl.h"
#include "ix_cc.h"
#include "bindings.h"

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
#include "cc/natpt/internal/ix_cc_natpt_slist.h"
#else
#include "cc/ix_cc_slist.h"
#endif

#include "cc/ix_cc_natpt.h"


/**
 * User defined include files
 */
#include "cc/natpt/internal/ix_cc_natpt_init.h"
#include "cc/natpt/internal/ix_cc_natpt_fini.h"
#include "cc/natpt/internal/ix_cc_natpt_lists.h"


/**
 * Preprocessor symbols and Macros used in this file
 */



/**
 * Type definitions whose scope is limited to this file
 */


/**
 * Variable declarations global to this file only. Externs are followed by 
 * static variables.
 */

extern ix_cc_natpt_context* g_pNatPtContext;


/** 
 * Extern function prototypes.
 */


/** 
 * Static function prototypes.
 */


/** 
 * Function definitions.
 */



/**
 * NAME: _ix_cc_natpt_unregister_message_handler
 *
 * DESCRIPTION: This primitive will be used to unregister message handler. 
 * It calls CCI services in level 1 system and resore manager services in
 * level 0 system.
 *
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_NATPT_ERROR_CCI - error from CCI
 */

ix_error _ix_cc_natpt_unregister_message_handler(void)
{
    /* Call CCI if CCI is enabled */

#if defined(IX_EXCLUDE_CCI)
        IX_ERROR_CRT(ix_rm_message_handler_unregister(IX_CC_NATPT_MSG_INPUT),
                     IX_CC_NATPT_ERROR_CCI,
                     IX_ERROR_LEVEL_WARNING);
#else
        IX_ERROR_CRT(ix_cci_cc_remove_message_handler(IX_CC_NATPT_MSG_INPUT),
                     IX_CC_NATPT_ERROR_CCI,
                     IX_ERROR_LEVEL_WARNING);
#endif

    return IX_SUCCESS;
}/* end _ix_cc_natpt_unregister_message_handler */



/**
 * NAME: _ix_cc_natpt_unregister_cci_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to unregister packet handler with 
 * CCI. 
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_CCI - error from CCI
 * 
 */

ix_error _ix_cc_natpt_unregister_cci_packet_handlers(void)
{

#if !defined(IX_EXCLUDE_CCI)
    IX_ERROR_CRT(ix_cci_cc_remove_packet_handler(
                     IX_CC_NATPT_PKT_MICROBLOCK_INPUT),
                 IX_CC_NATPT_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);
#endif

    return IX_SUCCESS;

}/* end _ix_cc_natpt_unregister_cci_packet_handlers */


/**
 * NAME: _ix_cc_natpt_unregister_rm_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to unregister packet handler with 
 * resource manager
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_NAPT_ERROR_CCI - error from RM
 * 
 */

ix_error _ix_cc_natpt_unregister_rm_packet_handlers(void)
{

    IX_ERROR_CRT(ix_rm_message_handler_unregister(
                               IX_CC_NATPT_PKT_MICROBLOCK_INPUT),
                     IX_CC_NATPT_ERROR_CCI,
                     IX_ERROR_LEVEL_WARNING);
    return IX_SUCCESS;

}/* end _ix_cc_natpt_unregister_rm_packet_handlers */



/**
 * NAME: _ix_cc_natpt_unregister_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to unregister packet handler 
 * with CCI/Resource manager
 *
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_NATPT_ERROR_CCI - error from CCI/RM
 * 
 */

ix_error _ix_cc_natpt_unregister_packet_handlers(void)
{
 
    /* Call CCI if CCI is enabled */
#if defined(IX_EXCLUDE_CCI)
        IX_ERROR_CR(_ix_cc_natpt_unregister_rm_packet_handlers());
#else
        IX_ERROR_CR(_ix_cc_natpt_unregister_cci_packet_handlers());
#endif

    return IX_SUCCESS;

}/* end _ix_cc_natpt_unregister_packet_handlers */


/**
 * NAME: ix_cc_natpt_fini
 *
 * DESCRIPTION: This primitive shall be called to terminate services from 
 * NATPT component. This primitive frees memory allocated during 
 * initialization, free all the created resources: delete 
 * 64 bit counter. If client claims for any services from NATPT after calling 
 * this function, then the behavior is undefined.
 * 
 * @Param:  - IN arg_hCcHandle - handle to NATPT Forwader core component
 *
 * @Param:  - IN arg_pContext - pointer to the control block memory allocated 
 * earlier in ix_cc_natpt_init function. 
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_NATPT_ERROR_INVALID_INPUT_PARAM - invalid input parameter(s)
 * IX_CC_ERROR_OOM - failure of memory deletion                
 * IX_CC_ERROR_OOM_64BIT_COUNTER - failure of  64 bit counter deletion 
 * IX_CC_NATPT_ERROR_CCI - failure from CCI
 * 
 */

ix_error ix_cc_natpt_fini(ix_cc_handle arg_hCcHandle,
                          void* arg_pContext)
{

    /* Validate Parameters */
    if(arg_pContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                            ("arg_pContext is null"));

    }/* if(arg_pContext == NULL)  */ 


    if((arg_hCcHandle != g_pNatPtContext->hNatPtHandle) ||
       (ix_cc_natpt_context *)arg_pContext != g_pNatPtContext)
    {
        return IX_ERROR_WARNING(IX_CC_NATPT_ERROR_INVALID_INPUT_PARAM,
                            ("Invalid input Parameters"));
    }
  
    /* Unregister Message Handler */
    IX_ERROR_CR(_ix_cc_natpt_unregister_message_handler());
  
    /* Unregister Packet Handler */
    IX_ERROR_CR(_ix_cc_natpt_unregister_packet_handlers());

    /* Remove Event Handler */
#if !defined(IX_EXCLUDE_CCI)
    if (g_pNatPtContext->hEventHandle)
    {
        IX_ERROR_CRT(ix_cci_cc_remove_event_handler(
                        g_pNatPtContext->hNatPtHandle, 
                        g_pNatPtContext->hEventHandle),
                IX_CC_NATPT_ERROR_CCI, IX_ERROR_LEVEL_WARNING);
    }
#endif /* ! IX_EXCLUDE_CCI */

    /* Free mutex */
    IX_ERROR_CR(ix_ossl_mutex_fini(g_pNatPtContext->NatPtMutex));

    /* delete lists */
    IX_ERROR_CR(_ix_cc_natpt_cleanup_lists(g_pNatPtContext));

    /* Free NATPT Context */ 
    ix_ossl_free(g_pNatPtContext);
        
    return IX_SUCCESS;

} /* end ix_cc_natpt_fini */
