/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.x for the IXP2xxx Network Processors
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      NATPT Core Component
 *
 * = FILENAME
 *      ix_cc_natpt_init.c
 *
 * = DESCRIPTION
 *      The file  contains initialization routines for the Core Component.
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      03/03/03
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */
#define IX_ERROR_FILE_IDENT "$Id: ix_cc_natpt_init.c,v 1.18 2003/10/31 19:27:58 ktseng Exp $";


/**
 * System defined include files
 */
#include "ix_cc_error.h"
#include "ix_rm.h"
#include "ix_ossl.h"
#include "ix_cc.h"
#include "ix_cc_macros.h"
#include "ix_config.h"
#include "bindings.h"

/**
 * User defined include files
 */
#include "cc/natpt/internal/ix_cc_natpt_init.h"
#include "cc/natpt/internal/ix_cc_natpt_fini.h" 
#include "cc/natpt/internal/ix_cc_natpt_prop.h"
#include "cc/natpt/internal/ix_cc_natpt_lists.h"
#include "cc/natpt/internal/ix_cc_natpt_nat_table.h"

/**
 * Preprocessor symbols and Macros used in this file
 */

/**
 * Type definitions whose scope is limited to this file
 */

/**
 * Variable declarations global to this file only. Externs are followed by 
 * static variables.
 */

ix_cc_natpt_context *g_pNatPtContext;
ix_cc_natpt_registry_data g_NatPtRegistryData;


/** 
 * Extern function prototypes.
 */


/** 
 * Static function prototypes.
 */


/** 
 * Function definitions.
 */


/**
 * NAME: _ix_cc_natpt_retrieve_registry_data
 *
 * DESCRIPTION: This primitive will be used to retrieve data from registry 
 * for a given property type.
 * 
 * @Param:  - IN arg_pPropertyName - pointer to property name string 
 *
 * @Param:  - OUT arg_pPropertyValue - pointer to property value 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_NATPT_ERROR_REGISTRY  - failed to access registry
 * 
 */

ix_error _ix_cc_natpt_retrieve_registry_data(const char* arg_pPropertyName,
                                            ix_uint32* arg_pPropertyValue)
{
    ix_configuration_property_handle hNatPtPropertyHandle; 

    /* Get static data from registry */    
    IX_ERROR_CRT(ix_rm_cp_property_open(IX_CP_CORE_PROPERTY_ROOT,
                                        arg_pPropertyName,
                                        &hNatPtPropertyHandle),
                 IX_CC_NATPT_ERROR_REGISTRY,
                 IX_ERROR_LEVEL_WARNING);

    IX_ERROR_CRT(ix_rm_cp_property_get_value_uint32(hNatPtPropertyHandle,
                                                   arg_pPropertyValue),
                 IX_CC_NATPT_ERROR_REGISTRY,
                 IX_ERROR_LEVEL_WARNING);

    IX_ERROR_CRT(ix_rm_cp_property_close(hNatPtPropertyHandle),
                 IX_CC_NATPT_ERROR_REGISTRY,
                 IX_ERROR_LEVEL_WARNING);

    return IX_SUCCESS;

}/* end _ix_cc_natpt_retrieve_registry_data */


/**
 * NAME: _ix_cc_natpt_registry
 *
 * DESCRIPTION: This primitive will be used to retreive data from registry for
 * all NATPT static data and stored in g_NatPtRegistryData 
 *           .
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT  none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_NATPT_ERROR_REGISTRY  - not able to access registry.
 * 
 */

ix_error _ix_cc_natpt_registry(void)
{
       
    /* Get micro engine numbers from registry */
    IX_ERROR_CR(_ix_cc_natpt_retrieve_registry_data(
                          IX_CC_NATPT_REGISTRY_KEY_ME_FOR_NATPT,
                          &g_NatPtRegistryData.meNumbers));
   
    IX_ERROR_CR(_ix_cc_natpt_retrieve_registry_data(
                       IX_CC_NATPT_REGISTRY_KEY_NAT_TABLE_SIZE,
                      &g_NatPtRegistryData.natTableSize));
   
    IX_ERROR_CR(_ix_cc_natpt_retrieve_registry_data(
                    IX_CC_NATPT_REGISTRY_KEY_NAT_HASH_INDEX_TABLE_SIZE,
                    &g_NatPtRegistryData.natHashTableSize));

    IX_ERROR_CR(_ix_cc_natpt_retrieve_registry_data(
                    IX_CC_NATPT_REGISTRY_KEY_NAT_COLLISION_TABLE_SIZE,
                    &g_NatPtRegistryData.natCollisionTableSize));
   
    IX_ERROR_CR(_ix_cc_natpt_retrieve_registry_data(
                    IX_CC_NATPT_REGISTRY_KEY_NATPT_PREFIX_LW0,
                    &g_NatPtRegistryData.natptPrefix0));
   
    IX_ERROR_CR(_ix_cc_natpt_retrieve_registry_data(
                    IX_CC_NATPT_REGISTRY_KEY_NATPT_PREFIX_LW1,
                    &g_NatPtRegistryData.natptPrefix1));
   
    IX_ERROR_CR(_ix_cc_natpt_retrieve_registry_data(
                    IX_CC_NATPT_REGISTRY_KEY_NATPT_PREFIX_LW2,
                    &g_NatPtRegistryData.natptPrefix2));
   
    return IX_SUCCESS;
}/* end _ix_cc_natpt_registry */


/**
 * NAME: _ix_cc_natpt_alloc_shared_memory
 *
 * DESCRIPTION: This primitive is used to allocate a region of shared memory.
 * 
 * @Param:  - IN arg_MemSize: Total size of the table in bytes.
 * @Param:  - IN arg_MemType: Type of memory.
 * @Param:  - IN arg_MemChannel: Memory channel.
 *
 * @Param:  - OUT arg_ppTableBase: Location into which to return table
 * address.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOM  - not able to allocate memory
 * 
 */

ix_error _ix_cc_natpt_alloc_shared_memory(
    ix_uint32 arg_MemSize,
    ix_memory_type arg_MemType,
    ix_uint32 arg_MemChannel,
    void** arg_ppTableBase
    )
{
    ix_error err = IX_SUCCESS;

    /*
     * Allocate the shared memory (either SRAM or DRAM) from the Resource
     * Manager.
     */
    err = ix_rm_mem_alloc_aligned(
        arg_MemType,
        arg_MemChannel,
        arg_MemSize, IX_MEMORY_ALIGNMENT_TYPE_VIRTUAL, 3,
        arg_ppTableBase);

    if (err != IX_SUCCESS)
    {
        switch ((int) IX_ERROR_GET_CODE(err))
        {
            case IX_RM_ERROR_ASSERTION:
                err = IX_ERROR_CHAIN(err, IX_CC_ERROR_RANGE, 0,
                    ("Invalid channel: %d", arg_MemChannel));
                break;

            case IX_RM_ERROR_CANNOT_ALLOCATE_SIZE:
                err = IX_ERROR_CHAIN(err, IX_CC_ERROR_OOM, 0,
                    ("Out of SRAM memory\n"
                     "\tCould not allocate %d bytes from the RM",
                     arg_MemSize));
                break;

            default:
                err = IX_ERROR_CHAIN(err, IX_CC_ERROR_INTERNAL, 0,
                    ("Could not allocate %d bytes from the RM", arg_MemSize));
                break;
        }
        return err;
    }

    if (*arg_ppTableBase == NULL)
    {
        return IX_ERROR_LOCAL(IX_CC_ERROR_INTERNAL,
            ("Could not allocate %d bytes from the RM", arg_MemSize));
    }

    return err;

}/* end _ix_cc_natpt_alloc_shared_memory */


ix_error _ix_cc_natpt_create_tables(void)
{
    ix_error err = IX_SUCCESS;
    ix_uint32 memSize;

    /* IX_ERROR_CHECK_ARG_RANGE(1, arg_NumEntries, 2UL, 65535UL, "%ld"); */

    /* NAT Table */
    memSize =  g_NatPtRegistryData.natTableSize * 
                    sizeof(ix_cc_natpt_nat_entry);

    IX_ERROR_CR(_ix_cc_natpt_alloc_shared_memory(memSize, 
                    IX_CC_NATPT_NAT_TABLE_MEMORY_TYPE, 
                    IX_CC_NATPT_NAT_TABLE_CHANNEL,
                    (void **)&g_pNatPtContext->pMbNatTableBase));

    ix_ossl_memset(g_pNatPtContext->pMbNatTableBase, 0x0, memSize);

    /* Allocate index pool for NAT Table */  
    IX_ERROR_CG(
            ix_cc_index_pool_init(
                    g_NatPtRegistryData.natTableSize, 0,
                    &g_pNatPtContext->natTableIndexPool),
            err,
            cleanupMem1);

    /* NAT V6 to V4 HashTable */
    memSize =  g_NatPtRegistryData.natHashTableSize * 
                    sizeof(ix_cc_natpt_nat_hash_entry);
    IX_ERROR_CG(
            _ix_cc_natpt_alloc_shared_memory(memSize, 
                    IX_CC_NATPT_NAT_TABLE_MEMORY_TYPE, 
                    IX_CC_NATPT_NAT_TABLE_CHANNEL,
                    (void **)&g_pNatPtContext->pMbNatV6V4HashTableBase),
            err,
            cleanupMem2);

    ix_ossl_memset(g_pNatPtContext->pMbNatV6V4HashTableBase, 0xff, memSize);

    /* Allocate index pool for V6 to V4 HashTable */  
    IX_ERROR_CG(
            ix_cc_index_pool_init(
                    g_NatPtRegistryData.natHashTableSize, 0,
                    &g_pNatPtContext->natV6V4HashTableIndexPool),
            err,
            cleanupMem3);

    /* NAT V4 to V6 HashTable */
    memSize =  g_NatPtRegistryData.natHashTableSize * 
                    sizeof(ix_cc_natpt_nat_hash_entry);
    IX_ERROR_CG(_ix_cc_natpt_alloc_shared_memory(memSize, 
                    IX_CC_NATPT_NAT_TABLE_MEMORY_TYPE, 
                    IX_CC_NATPT_NAT_TABLE_CHANNEL,
                    (void **)&g_pNatPtContext->pMbNatV4V6HashTableBase),
            err,
            cleanupMem4);

    ix_ossl_memset(g_pNatPtContext->pMbNatV4V6HashTableBase, 0xff, memSize);

    /* Allocate index pool for V6 to V4 HashTable */  
    IX_ERROR_CG(
            ix_cc_index_pool_init(
                    g_NatPtRegistryData.natHashTableSize, 0,
                    &g_pNatPtContext->natV4V6HashTableIndexPool),
            err,
            cleanupMem5);

    /* NAT Hash Collision Table */
    memSize =  g_NatPtRegistryData.natCollisionTableSize * 
                    sizeof(ix_cc_natpt_nat_hash_entry);
    IX_ERROR_CG(_ix_cc_natpt_alloc_shared_memory(memSize, 
                    IX_CC_NATPT_NAT_TABLE_MEMORY_TYPE, 
                    IX_CC_NATPT_NAT_TABLE_CHANNEL,
                    (void **)&g_pNatPtContext->pMbNatCollisionTableBase),
            err,
            cleanupMem6);

    ix_ossl_memset(g_pNatPtContext->pMbNatCollisionTableBase, 0xff, memSize);

    /* Allocate index pool for Collision Table */  
    IX_ERROR_CG(
            ix_cc_index_pool_init(
                    g_NatPtRegistryData.natCollisionTableSize, 0,
                    &g_pNatPtContext->natCollisionTableIndexPool),
            err,
            cleanupMem7);

    return IX_SUCCESS;

cleanupMem7:
    ix_rm_mem_free(g_pNatPtContext->pMbNatCollisionTableBase);
cleanupMem6:
    ix_cc_index_pool_fini(g_pNatPtContext->natV4V6HashTableIndexPool);
cleanupMem5:
    ix_rm_mem_free(g_pNatPtContext->pMbNatV4V6HashTableBase);
cleanupMem4:
    ix_cc_index_pool_fini(g_pNatPtContext->natV6V4HashTableIndexPool);
cleanupMem3:
    ix_rm_mem_free(g_pNatPtContext->pMbNatV6V4HashTableBase);
cleanupMem2:
    ix_cc_index_pool_fini(g_pNatPtContext->natTableIndexPool);
cleanupMem1:
    ix_rm_mem_free(g_pNatPtContext->pMbNatTableBase);

    return err;
}

ix_error _ix_cc_natpt_cleanup_tables(void)
{
    ix_cc_index_pool_fini(g_pNatPtContext->natCollisionTableIndexPool);
    ix_rm_mem_free(g_pNatPtContext->pMbNatCollisionTableBase);
    ix_cc_index_pool_fini(g_pNatPtContext->natV4V6HashTableIndexPool);
    ix_rm_mem_free(g_pNatPtContext->pMbNatV4V6HashTableBase);
    ix_cc_index_pool_fini(g_pNatPtContext->natV6V4HashTableIndexPool);
    ix_rm_mem_free(g_pNatPtContext->pMbNatV6V4HashTableBase);
    ix_cc_index_pool_fini(g_pNatPtContext->natTableIndexPool);
    ix_rm_mem_free(g_pNatPtContext->pMbNatTableBase);
    return IX_SUCCESS;
}


/**
 * NAME: _ix_cc_natpt_patch_symbols
 *
 * DESCRIPTION: This primitive will be used to patch symbols to natpt 
 * microblocks. 
 * 
 * @Param:  - IN  none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * IX_CC_NATPT_ERROR_FAILED_PATCHING - failed patching by calling resource 
 * manager function
 */

ix_error _ix_cc_natpt_patch_symbols(void)
{
    ix_uint8 natptIndex;
    ix_memory_type memType;
    ix_uint32 memChannel;
    ix_uint32 memOffset;
    void* pPhysAddr;
    ix_uint32* pDebugCounters;
    ix_imported_symbol natPtSymbols[IX_CC_NATPT_NUMBER_OF_SYMBOLS];

    /* Patch base address for NAT Table */
    IX_ERROR_CRT(
        ix_rm_get_phys_offset(g_pNatPtContext->pMbNatTableBase,
                              &memType,
                              &memChannel,
                              &memOffset,
                              &pPhysAddr),
        IX_CC_NATPT_ERROR_FAILED_PATCHING,
        IX_ERROR_LEVEL_WARNING);

    if (memType == IX_MEMORY_TYPE_SRAM)
    {
        IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(memChannel,memOffset);
    }

    natPtSymbols[0].m_Value = memOffset;
    natPtSymbols[0].m_Name = IX_CC_NATPT_PATCHNAME_NAT_TABLE_BASE;

    /* Patch base address for NAT V6 to V4 Hash Table */
    IX_ERROR_CRT(
        ix_rm_get_phys_offset(g_pNatPtContext->pMbNatV6V4HashTableBase,
                              &memType,
                              &memChannel,
                              &memOffset,
                              &pPhysAddr),
        IX_CC_NATPT_ERROR_FAILED_PATCHING,
        IX_ERROR_LEVEL_WARNING);

    if (memType == IX_MEMORY_TYPE_SRAM)
    {
        IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(memChannel,memOffset);
    }

    natPtSymbols[1].m_Value = memOffset;
    natPtSymbols[1].m_Name = IX_CC_NATPT_PATCHNAME_NAT_V6V4_HASH_TABLE_BASE;

    /* Patch base address for NAT V4 to V6 Hash Table */
    IX_ERROR_CRT(
        ix_rm_get_phys_offset(g_pNatPtContext->pMbNatV4V6HashTableBase,
                              &memType,
                              &memChannel,
                              &memOffset,
                              &pPhysAddr),
        IX_CC_NATPT_ERROR_FAILED_PATCHING,
        IX_ERROR_LEVEL_WARNING);

    if (memType == IX_MEMORY_TYPE_SRAM)
    {
        IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(memChannel,memOffset);
    }

    natPtSymbols[2].m_Value = memOffset;
    natPtSymbols[2].m_Name = IX_CC_NATPT_PATCHNAME_NAT_V4V6_HASH_TABLE_BASE;

    /* Patch base address for NAT Collision Hash Table */
    IX_ERROR_CRT(
        ix_rm_get_phys_offset(g_pNatPtContext->pMbNatCollisionTableBase,
                              &memType,
                              &memChannel,
                              &memOffset,
                              &pPhysAddr),
        IX_CC_NATPT_ERROR_FAILED_PATCHING,
        IX_ERROR_LEVEL_WARNING);

    if (memType == IX_MEMORY_TYPE_SRAM)
    {
        IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(memChannel,memOffset);
    }

    natPtSymbols[3].m_Value = memOffset;
    natPtSymbols[3].m_Name = IX_CC_NATPT_PATCHNAME_NAT_COLL_HASH_TABLE_BASE;

    natPtSymbols[4].m_Value = IX_CC_NATPT_HASH48_MULTIPLIER_LW0;
    natPtSymbols[4].m_Name = IX_CC_NATPT_PATCHNAME_H48_MULT_0;
    natPtSymbols[5].m_Value = IX_CC_NATPT_HASH48_MULTIPLIER_LW1;
    natPtSymbols[5].m_Name = IX_CC_NATPT_PATCHNAME_H48_MULT_1;

    natPtSymbols[6].m_Value = IX_CC_NATPT_HASH128_MULTIPLIER_LW0;
    natPtSymbols[6].m_Name = IX_CC_NATPT_PATCHNAME_H128_MULT_0;

    natPtSymbols[7].m_Value = IX_CC_NATPT_HASH128_MULTIPLIER_LW1;
    natPtSymbols[7].m_Name = IX_CC_NATPT_PATCHNAME_H128_MULT_1;

    natPtSymbols[8].m_Value = IX_CC_NATPT_HASH128_MULTIPLIER_LW2;
    natPtSymbols[8].m_Name = IX_CC_NATPT_PATCHNAME_H128_MULT_2;

    natPtSymbols[9].m_Value = IX_CC_NATPT_HASH128_MULTIPLIER_LW3;
    natPtSymbols[9].m_Name = IX_CC_NATPT_PATCHNAME_H128_MULT_3;

    natPtSymbols[10].m_Value = g_NatPtRegistryData.natptPrefix0;
    natPtSymbols[10].m_Name = IX_CC_NATPT_PATCHNAME_PREFIX_0;

    natPtSymbols[11].m_Value = g_NatPtRegistryData.natptPrefix1;
    natPtSymbols[11].m_Name = IX_CC_NATPT_PATCHNAME_PREFIX_1;

    natPtSymbols[12].m_Value = g_NatPtRegistryData.natptPrefix2;
    natPtSymbols[12].m_Name = IX_CC_NATPT_PATCHNAME_PREFIX_2;

    /* NAT Debug Counters just 20 bytes */
    _ix_cc_natpt_alloc_shared_memory(20, IX_MEMORY_TYPE_SRAM,  
                    IX_CC_NATPT_NAT_TABLE_CHANNEL,
                    (void **)&pDebugCounters);

    ix_rm_get_phys_offset(pDebugCounters,
                          &memType,
                          &memChannel,
                          &memOffset,
                          &pPhysAddr);

    if (memType == IX_MEMORY_TYPE_SRAM)
    {
        IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(memChannel,memOffset);
    }

    natPtSymbols[13].m_Value = memOffset;
    natPtSymbols[13].m_Name = IX_CC_NATPT_PATCHNAME_SRAM_COUNTER_BASE;

    /* 
     * This loop will check to see if a microengine is being used
     * by our microblock (the NATPT microblock) and if it is, it'll
     * patch the microblock symbols appropriately
     */

    for (natptIndex= 0; 
         natptIndex< IX_CC_NATPT_NUMBER_OF_MICROENGINES; 
         natptIndex++)
    {
         /* if the bit is set, then patch the symbols */
        if(((g_NatPtRegistryData.meNumbers) & (1 << natptIndex)) != 0)
        {
            IX_ERROR_CRT(ix_rm_ueng_patch_symbols(
                            g_aMicroengineNumberMapping[natptIndex],
                                IX_CC_NATPT_NUMBER_OF_SYMBOLS, natPtSymbols),
                         IX_CC_NATPT_ERROR_FAILED_PATCHING,
                         IX_ERROR_LEVEL_WARNING);
        }/* end if */
    }/*end for (index = 0; index<IX_CC_NATPT_NUMBER_OF_MICROENGINES; index++) */


    return IX_SUCCESS;

}/* end _ix_cc_natpt_patch_symbols */


/**
 * NAME: _ix_cc_natpt_create_64bit_counters
 *
 * DESCRIPTION: This primitive will be used to create 64 bit counters.
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * IX_CC_ERROR_OOM_64BIT_COUNTER - error in creating 64 bit counter
 * 
 * 
 */

ix_error _ix_cc_natpt_create_64bit_counters(void)
{
    /* Create all 64 bit counters here.. */

    return IX_SUCCESS;

}/* end _ix_cc_natpt_create_64bit_counters */


/**
 * NAME: _ix_cc_natpt_register_message_handler
 *
 * DESCRIPTION: This primitive will be used to register message handler. 
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT  none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_NATPT_ERROR_CCI - Error from CCI
 * 
 */

ix_error _ix_cc_natpt_register_message_handler(void)
{
    
    /* Call CCI message handler if CCI is enabled */
#if defined(IX_EXCLUDE_CCI)
        IX_ERROR_CRT(ix_rm_message_handler_register(IX_CC_NATPT_MSG_INPUT,
                                            ix_cc_natpt_msg_handler,
                                            g_pNatPtContext),
                 IX_CC_NATPT_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);
#else
        IX_ERROR_CRT(ix_cci_cc_add_message_handler(
                                        g_pNatPtContext->hNatPtHandle,
                                            IX_CC_NATPT_MSG_INPUT,
                                            ix_cc_natpt_msg_handler,
                                            IX_INPUT_TYPE_MULTI_SRC),
                 IX_CC_NATPT_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);

#endif

    return IX_SUCCESS;

}/* end _ix_cc_natpt_register_message_handler */



/**
 * NAME: _ix_cc_natpt_register_cci_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to register packet handler with CCI. 
 * @Param:  - IN  none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_NATPT_ERROR_CCI - error from CCI
 * 
 */

ix_error _ix_cc_natpt_register_cci_packet_handlers(void)
{

#if !defined(IX_EXCLUDE_CCI)

    IX_ERROR_CRT(ix_cci_cc_add_packet_handler(g_pNatPtContext->hNatPtHandle,
                                IX_CC_NATPT_PKT_MICROBLOCK_INPUT,
                                ix_cc_natpt_microblock_pkt_handler,
                                IX_INPUT_TYPE_SINGLE_SRC),
                 IX_CC_NATPT_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);

#endif

    return IX_SUCCESS;

}/* end _ix_cc_natpt_register_cci_packet_handlers */


/**
 * NAME: _ix_cc_natpt_register_rm_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to register packet handler 
 * with resource manager
 *
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_NATPT_ERROR_CCI - error from resource manager
 * 
 */

ix_error _ix_cc_natpt_register_rm_packet_handlers(void)
{

    IX_ERROR_CRT(ix_rm_packet_handler_register(
                      IX_CC_NATPT_PKT_MICROBLOCK_INPUT,
                      ix_cc_natpt_microblock_pkt_handler,
                      g_pNatPtContext),
                 IX_CC_NATPT_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);

    return IX_SUCCESS;

}/* end _ix_cc_natpt_register_rm_packet_handlers */

/**
 * NAME: _ix_cc__natpt_register_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to register packet handlers. 
 * It registers with CCI in level 1 system and register with RM in
 * level 0 system.
 * 
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_NATPT_ERROR_CCI - error from CCI/resource manager
 * 
 */

ix_error _ix_cc_natpt_register_packet_handlers(void)
{
    /* Call CCI packet handler if CCI is enabled */
#if defined(IX_EXCLUDE_CCI)

        IX_ERROR_CR(_ix_cc_natpt_register_rm_packet_handlers());

#else

        IX_ERROR_CR(_ix_cc_natpt_register_cci_packet_handlers());

#endif

    return IX_SUCCESS;
}/* end _ix_cc_natpt_register_packet_handlers */


/**
 * NAME: _ix_cc_natpt_initialize_hash_multipliers
 *
 * DESCRIPTION: This primitive initializes hash multipliers.
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: - none
 */
void _ix_cc_natpt_initialize_hash_multipliers(void)
{
    ix_error err;
    ix_hash_multiplier_48 mult48;
    ix_hash_multiplier_128 mult128;


    mult48.m_LW0 = IX_CC_NATPT_HASH48_MULTIPLIER_LW0;
    mult48.m_LW1 = IX_CC_NATPT_HASH48_MULTIPLIER_LW1;
    
    err = ix_rm_hash_48_multiplier_set(&mult48);

    if (err != IX_SUCCESS)
    {
#if defined(NATPT_DEBUG)
        ix_ossl_message_log("ix_rm_hash_48_multiplier_set "
                "returned error (0x%lx)\n", err);
#endif /* NATPT_DEBUG */
    }

    mult128.m_LW0 = IX_CC_NATPT_HASH128_MULTIPLIER_LW0;
    mult128.m_LW1 = IX_CC_NATPT_HASH128_MULTIPLIER_LW1;
    mult128.m_LW2 = IX_CC_NATPT_HASH128_MULTIPLIER_LW2;
    mult128.m_LW3 = IX_CC_NATPT_HASH128_MULTIPLIER_LW3;

    err = ix_rm_hash_128_multiplier_set(&mult128);

    if (err != IX_SUCCESS)
    {
#if defined(NATPT_DEBUG)
        ix_ossl_message_log("ix_rm_hash_128_multiplier_set "
                "returned error (0x%lx)\n", err);
#endif /* NATPT_DEBUG */
    }

}


/**
 * NAME: ix_cc_natpt_init
 *
 * DESCRIPTION: This primitive shall be called and returned successfully 
 * before requesting any service from NATPT Forwarder core component. 
 * This primitive should be called only once to initialize NATPT core component.
 * This function performs allocation of memory for symbols to be patched,
 * Creation of 64 bit counters, registration of packet and message 
 * handlers.
 * 
 * @Param:  - IN arg_hCcHandle - handle to NAT-PT core component, 
 *            created by core component infrastructure; 
 *            this shall be used later to get other (to add event handler) 
 *            services from core component infrastructure.
 *
 * @Param:  - IN-OUT arg_ppContext - Buffer free list and dynamic property 
 *            data will be given during by system APPs during initialization. 
 *            Upon successfull completion, it represents the location where 
 *            the pointer to the control block allocated by NATPT 
 *            core component will be stored. The control block is internal 
 *            to NATPT core component and it contains information 
 *            about NATPT internal data structures, allocated memory and other 
 *            relevant information. Later this pointer to be passed into  
 *            ix_cc_natpt_fini function for freeing memory and other house 
 *            keeping operations when NATPT component is being destroyed.  
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - input parameter arg_ppContext is null
 * IX_CC_ERROR_OOM   - memory allocation failure      
 * IX_CC_NATPT_ERROR_FAILED_PATCHING   - patching failures                      
 * IX_CC_ERROR_OOM_64BIT_COUNTER - 64 bit counter creation failure
 * IX_CC_NATPT_ERROR_REGISTRY  - registry access failures
 * IX_CC_NATPT_ERROR_CCI - failure from CCI 
 * 
 */

ix_error ix_cc_natpt_init(ix_cc_handle arg_hCcHandle,
                          void** arg_ppContext)
{
    ix_error err = IX_SUCCESS;

    /* Variables used in retrieving static data from arg_ppContext */
    ix_cc_init_context *pInitCtx;
    ix_cc_properties_node *pPropNode;

    /* Check for argument validity */
    if(arg_ppContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("Input parameter(arg_ppContext) is null"));

    }/* end if (arg_ppContext == NULL) */

    /* Allocate memory for NAT-PT Context */
    g_pNatPtContext = (ix_cc_natpt_context *)
                                   ix_ossl_malloc(sizeof(ix_cc_natpt_context));
    if(g_pNatPtContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,
                               ("Failed to allocate memory for natpt context"));
     }/* end if (g_pNatPtContext == NULL) */

    /* initialize the structure */
    ix_ossl_memset(g_pNatPtContext, 0, sizeof(ix_cc_natpt_context));

    /* store natpt component handle in ix_cc_natpt_context */
    g_pNatPtContext->hNatPtHandle = arg_hCcHandle;

    /* Get the generic init context */
    pInitCtx = (ix_cc_init_context *)*arg_ppContext;

    /* Store free list handle in g_pNatPtContext */
    g_pNatPtContext->hBufferFreeList = pInitCtx->hFreeList;

    g_pNatPtContext->idleCheckInterval = 
            IX_CC_NATPT_DEFAULT_IDLE_CHECK_INTERVAL;

    /* 
     * Retrieve static configuration from registry. If there is no registry, 
     * then the data structure will be filled with data from header file. 
     */
#if defined (IX_INCLUDE_REGISTRY)

    IX_ERROR_CG(_ix_cc_natpt_registry(), err, cleanupContext);

#else
    
    /* 
     * Default registry options 
     */
       
    g_NatPtRegistryData.meNumbers = IX_CC_NATPT_ME;   

    g_NatPtRegistryData.natTableSize = IX_CC_NATPT_NAT_TABLE_SIZE_HINT;
    g_NatPtRegistryData.natHashTableSize = 
                IX_CC_NATPT_NAT_HASH_TABLE_SIZE_HINT;
    g_NatPtRegistryData.natCollisionTableSize = 
                IX_CC_NATPT_NAT_COLLISION_TABLE_SIZE_HINT;

#endif

    /* Initialize Lists */
    IX_ERROR_CG(_ix_cc_natpt_create_lists(g_pNatPtContext), err, 
                            cleanupContext);

    /* Create CC/Microblock shared Tables */
    IX_ERROR_CG(_ix_cc_natpt_create_tables(), err, cleanupLists);

    /*
     * Retrieve local blade property information from init context 
     * (This will work for both level-1 and level-0 modes)
     */
    pPropNode = pInitCtx->pPropertyList;

    /* Get blade Id from first node */
    if(pPropNode != NULL)
    {
        g_NatPtRegistryData.localBladeId  = pPropNode->pPropertyInfo->blade_id;
    }/* end if */

    /* Patch symbols */
    IX_ERROR_CG(_ix_cc_natpt_patch_symbols(), err, cleanupTables);

    /* Register Message Handler */
    IX_ERROR_CG(_ix_cc_natpt_register_message_handler(), err, cleanupTables);

    /* Register Packet Handler */
    IX_ERROR_CG(_ix_cc_natpt_register_packet_handlers(), err, cleanupMessageHandler);

    /* Get mutex from OSSL */
    err = ix_ossl_mutex_init(IX_OSSL_MUTEX_UNLOCK, &g_pNatPtContext->NatPtMutex);
    if (err != IX_SUCCESS)
    {
        err = IX_ERROR_WARNING(IX_CC_ERROR_OOR, ("Failed to create mutext"));

        goto cleanupPacketHandlers;

    }/* end if (g_pNatPtContext == NULL) */

    while (pPropNode != NULL)
    {
        /* Do something appropriate for NAT-PT with the property node.. */

    /* next node */
        pPropNode = pPropNode->pNextPropNode;

    }/* end while */

    /* Initialize Hash multipliers */
    _ix_cc_natpt_initialize_hash_multipliers();

#if !defined(IX_EXCLUDE_CCI)
    g_pNatPtContext->hEventHandle = 0;
#endif /* ! IX_EXCLUDE_CCI */

    IX_ERROR_CG(_ix_cc_natpt_set_entry_timeout_event_handler(g_pNatPtContext),
                         err, cleanupPacketHandlers);

    *arg_ppContext = (void *)g_pNatPtContext;

    return IX_SUCCESS;

cleanupPacketHandlers:
    _ix_cc_natpt_unregister_packet_handlers();
cleanupMessageHandler:
    _ix_cc_natpt_unregister_message_handler();
cleanupTables:
    _ix_cc_natpt_cleanup_tables();
cleanupLists:
    _ix_cc_natpt_cleanup_lists(g_pNatPtContext);
cleanupContext:
    ix_ossl_free(g_pNatPtContext);

    return err;

} /* end ix_cc_ipv6_init */
