/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.x for the IXP2xxx Network Processors
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      NAT-PT Core Component - Message Helpers.
 *
 * = FILENAME
 *      ix_cc_natpt_msg_helpers.c
 *
 * = DESCRIPTION
 *      The file defines message helper interface functions.
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      03/03/03
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */
#define IX_ERROR_FILE_IDENT "$Id: ix_cc_natpt_msg_helpers.c,v 1.11 2003/07/11 17:51:29 sshegde Exp $";


/**
 * System defined include files
 */
#include "ix_cc_error.h"
#include "ix_ossl.h"
#include "ix_rm.h"
#include "ix_cc.h"
#include "bindings.h"
#include "cc/ix_cc_msup.h"
#include "cc/ix_cc_natpt.h"

/**
 * User defined include files
 */

#include "cc/natpt/internal/ix_cc_natpt_msg_handler.h"


/**
 * Preprocessor symbols and Macros used in this file
 */

/**
 * Type definitions whose scope is limited to this file
 */


/**
 * Variable declarations global to this file only. Externs are followed by 
 * static variables.
 */


/** 
 * Extern function prototypes.
 */

/** 
 * Static function prototypes.
 */




/** 
 * Function definitions.
 */


/**
 * NAME: ix_cc_natpt_async_set_config_parameters
 *
 * DESCRIPTION:This message helper primitive helps client to set global
 * configuration parameters
 * 
 * @Param:  - IN arg_pConfig: pointer to config parameters structure.
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_set_config_parameters(
                ix_cc_natpt_config_params * arg_pConfig,
                ix_cc_natpt_cb arg_Callback,
                void * arg_pClientContext)
{
    ix_cc_natpt_config_params msgData;
    ix_cc_natpt_general_callback_data * cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_pConfig == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pConfig"));
    }/* end if(arg_pConfig == NULL) */

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_natpt_general_callback_data *)
        ix_ossl_malloc(sizeof(ix_cc_natpt_general_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData.idleCheckInterval = arg_pConfig->idleCheckInterval;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_NATPT_MSG_INPUT,
                                    _ix_cc_natpt_cb_general,
                                    (void*)cbContext,
                                    IX_CC_NATPT_MSG_SET_CONFIG_PARAM,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_NATPT_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_natpt_async_get_config_parameters
 *
 * DESCRIPTION:This message helper primitive helps client to get global
 * configuration parameters. The data is returned via the callback.
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_get_config_parameters(
                ix_cc_natpt_cb_get_config_parameters arg_Callback,
                void * arg_pClientContext)
{
    ix_cc_natpt_get_config_parameters_callback_data * cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_natpt_get_config_parameters_callback_data *)
        ix_ossl_malloc(sizeof(ix_cc_natpt_get_config_parameters_callback_data));
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_NATPT_MSG_INPUT,
                                    _ix_cc_natpt_cb_get_config_parameters,
                                    (void*)cbContext,
                                    IX_CC_NATPT_MSG_GET_CONFIG_PARAM,
                                    NULL,
                                    0),
                 err,
                 IX_CC_NATPT_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_natpt_async_add_static_mapping
 *
 * DESCRIPTION:This message helper primitive add static ipv6/ipv4 address
 * mapping.
 * 
 * @Param:  - IN arg_pv6v4Map: Pointer to a v6 v4 address mapping structure.
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_add_static_mapping(
                ix_cc_natpt_static_v6v4map * arg_pv6v4Map,
                ix_cc_natpt_cb arg_Callback,
                void * arg_pClientContext)
{
    ix_cc_natpt_static_v6v4map msgData;
    ix_cc_natpt_general_callback_data * cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_pv6v4Map == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pv6v4Map"));
    }/* end if(arg_pv6v4Map == NULL) */

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_natpt_general_callback_data *)
        ix_ossl_malloc(sizeof(ix_cc_natpt_general_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData.v6addr = arg_pv6v4Map->v6addr;
    msgData.v4addr = arg_pv6v4Map->v4addr;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_NATPT_MSG_INPUT,
                                    _ix_cc_natpt_cb_general,
                                    (void*)cbContext,
                                    IX_CC_NATPT_MSG_ADD_STATIC_MAP,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_NATPT_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_natpt_async_remove_static_mapping
 *
 * DESCRIPTION:This message helper primitive remove static ipv6/ipv4 address
 * mapping.
 * 
 * @Param:  - IN arg_pv6v4Map: Pointer to a v6 v4 address mapping structure.
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_remove_static_mapping(
                ix_cc_natpt_static_v6v4map * arg_pv6v4Map,
                ix_cc_natpt_cb arg_Callback,
                void * arg_pClientContext)
{
    ix_cc_natpt_static_v6v4map msgData;
    ix_cc_natpt_general_callback_data * cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_pv6v4Map == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pv6v4Map"));
    }/* end if(arg_pv6v4Map == NULL) */

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_natpt_general_callback_data *)
        ix_ossl_malloc(sizeof(ix_cc_natpt_general_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData.v6addr = arg_pv6v4Map->v6addr;
    msgData.v4addr = arg_pv6v4Map->v4addr;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_NATPT_MSG_INPUT,
                                    _ix_cc_natpt_cb_general,
                                    (void*)cbContext,
                                    IX_CC_NATPT_MSG_REM_STATIC_MAP,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_NATPT_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_natpt_async_get_static_mapping
 *
 * DESCRIPTION:This message helper primitive get static ipv6/ipv4 address
 * mapping.
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_get_static_mapping(
                ix_cc_natpt_cb_get_static_mapping arg_Callback,
                void * arg_pClientContext)
{
    ix_cc_natpt_get_static_mapping_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_natpt_get_static_mapping_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_natpt_get_static_mapping_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_NATPT_MSG_INPUT,
                                    _ix_cc_natpt_cb_get_static_mapping,
                                    (void*)cbContext,
                                    IX_CC_NATPT_MSG_GET_STATIC_MAP,
                                    NULL,
                                    0),
                 err,
                 IX_CC_NATPT_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_natpt_async_add_v4addr_pool
 *
 * DESCRIPTION:This message helper primitive helps client to add IPv4
 * addresses to pool.
 * 
 * @Param:  - IN arg_NumItems: Number of v4 addrs in the array pointed 
 *                             by the next argument.
 * @Param:  - IN arg_pv4addrArray: Pointer to a v4 address array.
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_add_v4addr_pool(
                ix_uint32 arg_NumItems,
                ix_cc_natpt_v4addr * arg_pv4addrArray,
                ix_cc_natpt_cb arg_Callback,
                void * arg_pClientContext)
{
    ix_cc_natpt_v4addr_pool_data msgData;
    ix_cc_natpt_general_callback_data * cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_pv4addrArray == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pv4addrArray"));
    }/* end if(arg_pv4addrArray == NULL) */

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_natpt_general_callback_data *)
        ix_ossl_malloc(sizeof(ix_cc_natpt_general_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData.numItems = arg_NumItems;

    msgData.pv4addrArray = ix_ossl_malloc(sizeof(ix_cc_natpt_v4addr) * 
                msgData.numItems);
    if(msgData.pv4addrArray == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,
                    ("No memory for v4addrArray"));
    }

    ix_ossl_memcpy(msgData.pv4addrArray, arg_pv4addrArray, 
                sizeof(ix_cc_natpt_v4addr) * msgData.numItems);

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_NATPT_MSG_INPUT,
                                    _ix_cc_natpt_cb_general,
                                    (void*)cbContext,
                                    IX_CC_NATPT_MSG_ADD_V4ADDR_POOL,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_NATPT_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
        ix_ossl_free(msgData.pv4addrArray);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_natpt_async_remove_v4addr_pool
 *
 * DESCRIPTION:This message helper primitive helps client to remove IPv4
 * addresses from pool.
 * 
 * @Param:  - IN arg_NumItems: Number of v4 addrs in the array pointed 
 *                             by the next argument.
 * @Param:  - IN arg_pv4addrArray: Pointer to a v4 address array.
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_remove_v4addr_pool(
                ix_uint32 arg_NumItems,
                ix_cc_natpt_v4addr * arg_pv4addrArray,
                ix_cc_natpt_cb arg_Callback,
                void * arg_pClientContext)
{
    ix_cc_natpt_v4addr_pool_data msgData;
    ix_cc_natpt_general_callback_data * cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_pv4addrArray == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pv4addrArray"));
    }/* end if(arg_pv4addrArray == NULL) */

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_natpt_general_callback_data *)
        ix_ossl_malloc(sizeof(ix_cc_natpt_general_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData.numItems = arg_NumItems;

    msgData.pv4addrArray = ix_ossl_malloc(sizeof(ix_cc_natpt_v4addr) * 
                msgData.numItems);
    if(msgData.pv4addrArray == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,
                    ("No memory for v4addrArray"));
    }

    ix_ossl_memcpy(msgData.pv4addrArray, arg_pv4addrArray, 
                sizeof(ix_cc_natpt_v4addr) * msgData.numItems);

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_NATPT_MSG_INPUT,
                                    _ix_cc_natpt_cb_general,
                                    (void*)cbContext,
                                    IX_CC_NATPT_MSG_REM_V4ADDR_POOL,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_NATPT_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
        ix_ossl_free(msgData.pv4addrArray);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_natpt_async_get_v4addr_pool
 *
 * DESCRIPTION:This message helper primitive helps client to get current
 * list of ipv4 address in pool.
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_get_v4addr_pool(
                ix_cc_natpt_cb_get_v4addr_pool arg_Callback,
                void * arg_pClientContext)
{
    ix_cc_natpt_get_v4addr_pool_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_natpt_get_v4addr_pool_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_natpt_get_v4addr_pool_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_NATPT_MSG_INPUT,
                                    _ix_cc_natpt_cb_get_v4addr_pool,
                                    (void*)cbContext,
                                    IX_CC_NATPT_MSG_GET_V4ADDR_POOL,
                                    NULL,
                                    0),
                 err,
                 IX_CC_NATPT_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_natpt_async_set_napt_mode
 *
 * DESCRIPTION:This message helper primitive helps client to turn NAPT
 * mode ON/OFF.
 * 
 * @Param:  - IN arg_NaptMode: NAPT mode flag.
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_set_napt_mode(
                ix_cc_natpt_naptmode arg_NaptMode,
                ix_cc_natpt_cb arg_Callback,
                void * arg_pClientContext)
{
    return IX_CC_NATPT_ERROR_NOT_IMPLEMENTED;
}


/**
 * NAME: ix_cc_natpt_async_add_v4v6port_mapping
 *
 * DESCRIPTION:This message helper primitive helps client to add a 
 * IPv4addr/IPv6Addr/PortNubmer mapping.
 * 
 * @Param:  - IN arg_pv4v6portMap: Pointer to a v4 to v6 address with port 
 *                                 number mapping structure.
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_add_v4v6port_mapping(
                ix_cc_natpt_v4v6portmap * arg_pv4v6portMap,
                ix_cc_natpt_cb arg_Callback,
                void * arg_pClientContext)
{
    return IX_CC_NATPT_ERROR_NOT_IMPLEMENTED;
}


/**
 * NAME: ix_cc_natpt_async_remove_v4v6port_mapping
 *
 * DESCRIPTION:This message helper primitive helps client to remove a 
 * IPv4addr/IPv6Addr/PortNubmer mapping.
 * 
 * @Param:  - IN arg_pv4v6portMap: Pointer to a v4 to v6 address with port 
 *                                 number mapping structure.
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_remove_v4v6port_mapping(
                ix_cc_natpt_v4v6portmap * arg_pv4v6portMap,
                ix_cc_natpt_cb arg_Callback,
                void * arg_pClientContext)
{
    return IX_CC_NATPT_ERROR_NOT_IMPLEMENTED;
}


/**
 * NAME: ix_cc_natpt_async_get_v4v6port_mapping
 *
 * DESCRIPTION:This message helper primitive helps client to get current
 * list of IPv4addr/IPv6Addr/PortNubmer mappings.
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_get_v4v6port_mapping(
                ix_cc_natpt_cb_get_v4v6port_mapping arg_Callback,
                void * arg_pClientContext)
{
    return IX_CC_NATPT_ERROR_NOT_IMPLEMENTED;
}


/**
 * NAME: ix_cc_natpt_async_get_active_sessions
 *
 * DESCRIPTION:This message helper primitive helps client to get current
 * list of active NATPT sessions.
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pClientContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_NATPT_ERROR_MSG_LIBRARY - failure from Message Support Library 
 */
ix_error ix_cc_natpt_async_get_active_sessions (
                ix_cc_natpt_cb_get_active_sessions arg_Callback,
                void * arg_pClientContext)
{
    ix_cc_natpt_get_active_sessions_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_natpt_get_active_sessions_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_natpt_get_active_sessions_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_NATPT_MSG_INPUT,
                                    _ix_cc_natpt_cb_get_active_sessions,
                                    (void*)cbContext,
                                    IX_CC_NATPT_MSG_GET_ACTIVE_SESSIONS,
                                    NULL,
                                    0),
                 err,
                 IX_CC_NATPT_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}
    
