/****************************************************************************
//             INTEL CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Intel Corporation and may not be copied
//  or disclosed except in accordance with the terms of that agreement.
//  Copyright (c) 2000-2003 Intel Corporation. All Rights Reserved.
//
//      $Workfile:   ix_cc_ipv6_slist.c  $
//      $Revision: 1.4 $
//      $Date: 2003/12/12 16:42:10 $
//
//  Purpose:    Provides methods to create, add, delete objects from
//              a linked list.
//
****************************************************************************/

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
#include "ix_cc_error.h"
#include "ix_rm.h"
#include "ix_ossl.h"
#else
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#endif

#include "cc/natpt/internal/ix_cc_natpt_slist.h"
/*#include <csportability.h>*/
/* #defines used in this file only */

#define     FIRST_TIME      0xFFFFFFFF


/***************************************************************************/
/* List operations                                                         */

/****************************************************************************
//
// PROCEDURE:   ListCreate()
//
// PARAMETERS:
//
// DESCRIPTION:
//              Creates a linked list with no items within.
//
// RETURNS:
//              Pointer to the list created. NULL if calloc fails.
//
****************************************************************************/

LPLIST ListCreate(void)
{
    LPLIST list;

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    list = (LPLIST) ix_ossl_malloc(sizeof(SLIST));

    if (list)
    {
       ix_ossl_memset(list, 0, sizeof(SLIST));       
    }
#else
    list = (LPLIST) calloc (1, sizeof(SLIST));
#endif

#ifdef _LISTDEBUG
    CSTRACE( (CSTRACE_INFO, "ListCreate %X\n",list) );
#endif // _LISTDEBUG

    if (list)
    {
        /* initialize data members */
        list->head = NULL;
        list->tail = NULL;
        list->count = 0;
    }
    
    return list;

} /* ListCreate */


/****************************************************************************
//
// PROCEDURE:   ListAddItem()
//
// PARAMETERS:
//  list        Pointer to a LIST
//
//  data        Pointer to data item to be added
//
// DESCRIPTION:
//              Adds a data item at the end of list.
//
// RETURNS:
//              TRUE if success, FALSE if calloc fails.
//
****************************************************************************/

ix_uint8 ListAddItem(LPLIST list, void * data)
{
    LPLISTITEM newListItem;

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    if (list == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,("list is Null"));
    if (data == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,("data is Null"));

    newListItem = (LPLISTITEM) ix_ossl_malloc(sizeof(LISTITEM));

    if (!newListItem)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM, 
	          ("Memory allocation for ipv6 linked list failed \n"));
    }
    ix_ossl_memset(newListItem, 0, sizeof(LISTITEM));
#else
    assert(list);
    assert(data);
    newListItem = (LPLISTITEM) calloc (1, sizeof(LISTITEM));
#endif

    if (newListItem)
    {
        newListItem->data = data;
        newListItem->next = NULL;

        if (list->head == NULL)
        {
            /* no list items right now */
            list->head = list->tail = newListItem;
            newListItem->prev = NULL;
        }
        else
        {
            /* there are one or more item */
            newListItem->prev = list->tail;
            list->tail->next = newListItem;
            list->tail = newListItem;
        }

        /* increment number of items */
        list->count ++;
        return TRUE;
    }
    else
    {
        return FALSE;
    } /* if listItem */

} /* ListAddItem */


/****************************************************************************
//
// PROCEDURE:   ListRemoveItem()
//
// PARAMETERS:
//  list        Pointer to a LIST
//
//  data        Pointer to data item to be added
//
// DESCRIPTION:
//              Removes a perticular data item from the list
//
// RETURNS:
//              TRUE if that data item is found, else FALSE is returned.
//
****************************************************************************/

ix_uint8 ListRemoveItem(LPLIST list, void * data)
{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    LPLISTITEM  current_linux;
#else
    LPLISTITEM  current;
#endif
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    if (list == NULL)
       return IX_ERROR_WARNING(IX_CC_ERROR_NULL,("list is Null"));
    if (data == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,("data is Null"));
#else
    assert(list);
    assert(data);
#endif
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    current_linux = list->head;
#else
    current = list->head;
#endif

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    while(current_linux)
#else
    while(current)
#endif
    {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        if (current_linux->data == data)
#else
        if (current->data == data)
#endif
        {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
            if (current_linux->next == NULL && current_linux->prev == NULL)
#else
            if (current->next == NULL && current->prev == NULL)
#endif
            {
                /* this is the only item */
                list->head = list->tail = NULL;
            }
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
            else if (current_linux->next == NULL)
#else
            else if (current->next == NULL)
#endif
            {
                /* this is the last item */
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
                list->tail = current_linux->prev;
#else
                list->tail = current->prev;
#endif
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
                current_linux->prev->next = NULL;
#else
                current->prev->next = NULL;
#endif
            }
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
            else if (current_linux->prev == NULL)
#else
            else if (current->prev == NULL)
#endif
            {
                /* this is the first item */
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
                list->head = current_linux->next;
#else
                list->head = current->next;
#endif
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
                current_linux->next->prev = NULL;
#else
                current->next->prev = NULL;
#endif
            }
            else
            {
                /* somewhere in the middle */
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
                current_linux->next->prev = current_linux->prev;
#else
                current->next->prev = current->prev;
#endif
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
                current_linux->prev->next = current_linux->next;
#else
                current->prev->next = current->next;
#endif
            } /* if */
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
            ix_ossl_free(current_linux);
#else
            free(current);
#endif
            list->count --;

            return TRUE;
        } /* if */

        /* traverse further */
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        current_linux = current_linux->next;
#else
        current = current->next;
#endif
    } /* while */

    return FALSE;

} /* ListRemoveItem */


/****************************************************************************
//
// PROCEDURE:   ListFindItem()
//
// PARAMETERS:
//  list        Pointer to a LIST
//
//  data        Pointer to data item to be added
//
// DESCRIPTION:
//              Searches for the data item in the list.
//
// RETURNS:
//              TRUE if that data item is found, else FALSE is returned.
//
****************************************************************************/

ix_uint8 ListFindItem(LPLIST list, void * data)
{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    LPLISTITEM  current_linux;
#else
    LPLISTITEM  current;
#endif
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    if (list == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,("list is Null"));
    if (data == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,("data is Null"));
#else
    assert(list);
    assert(data);
#endif

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    current_linux = list->head;
#else
    current = list->head;
#endif

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    while(current_linux)
#else
    while(current)
#endif
    {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        if (current_linux->data == data)
#else
        if (current->data == data)
#endif
        {
            return TRUE;
        } /* if */

        /* traverse further */
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        current_linux = current_linux->next;
#else
        current = current->next;
#endif
    } /* while */

    return FALSE;

} /* ListFindItem */


/****************************************************************************
//
// PROCEDURE:   ListGetFirst()
//
// PARAMETERS:
//  list        Pointer to a LIST
//
// DESCRIPTION:
//              Gets number of items in the list
//
// RETURNS:
//              Number of items in the list
//
****************************************************************************/

void * ListGetFirst(LPLIST list)
{
    if (list->head)
    {
        return list->head->data;
    } /* if */
    else
    {
        return NULL;
    } /* else */
} /* ListGetFirst */


/****************************************************************************
//
// PROCEDURE:   ListGetLast()
//
// PARAMETERS:
//  list        Pointer to a LIST
//
// DESCRIPTION:
//              Gets number of items in the list
//
// RETURNS:
//              Number of items in the list
//
****************************************************************************/

void * ListGetLast(LPLIST list)
{
    if (list->tail)
    {
        return list->tail->data;
    } /* if */
    else
    {
        return NULL;
    } /* else */
} /* ListGetLast */



/****************************************************************************
//
// PROCEDURE:   ListGetCount()
//
// PARAMETERS:
//  list        Pointer to a LIST
//
// DESCRIPTION:
//              Gets number of items in the list
//
// RETURNS:
//              Number of items in the list
//
****************************************************************************/

int ListGetCount(LPLIST list)
{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    if (list == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,("list is Null"));
#else
    assert(list);
#endif

    return (list->count);

} /* ListGetCount */


/****************************************************************************
//
// PROCEDURE:   ListDelete()
//
// PARAMETERS:
//  list        Pointer to a LIST
//
// DESCRIPTION:
//              Deletes all the items in the list and the list itself
//
// RETURNS:
//              None
//
****************************************************************************/

void ListDelete(LPLIST list)
{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    LPLISTITEM  current_linux;
#else
    LPLISTITEM  current;
#endif
    LPLISTITEM tmpItem;
    if (list == NULL)
    {
#ifdef _LISTDEBUG
        ix_ossl_message_log("list is Null\n");
#endif
        return;
    }

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    current_linux = list->head;
#else
    current = list->head;
#endif

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    while(current_linux)
#else
    while(current)
#endif
    {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        tmpItem = current_linux->next;
#else
        tmpItem = current->next;
#endif

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
            ix_ossl_free(current_linux);
#else
            free(current);
#endif
        list->count --;

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        current_linux = tmpItem;
#else
        current = tmpItem;
#endif
    } /* while */

#ifdef _LISTDEBUG
    CSTRACE( (CSTRACE_INFO, "ListDelete %X\n",list) );
#endif // _LISTDEBUG

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
	ix_ossl_free(list);
#else
    free(list);
#endif

} /* ListDelete */

/****************************************************************************
//
// PROCEDURE:   ListFreeDelete()
//
// PARAMETERS:
//  list        Pointer to a LIST
//
// DESCRIPTION:
//              Deletes all the items in the list, list's user data
//              and the list itself
//
// RETURNS:
//              None
//
****************************************************************************/

void ListFreeDelete(LPLIST list)
{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    LPLISTITEM  current_linux;
#else
    LPLISTITEM  current;
#endif
    LPLISTITEM tmpItem;
    if (list == NULL)
    {
#ifdef _LISTDEBUG
        ix_ossl_message_log("list is Null\n");
#endif
        return;
    }

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    current_linux = list->head;
#else
    current = list->head;
#endif

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    while(current_linux)
#else
    while(current)
#endif
    {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        tmpItem = current_linux->next;
#else
        tmpItem = current->next;
#endif

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
            ix_ossl_free(current_linux->data);
#else
            free(current->data);
#endif
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
            ix_ossl_free(current_linux);
#else
            free(current);
#endif
        list->count --;
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        current_linux = tmpItem;
#else
        current = tmpItem;
#endif
    } /* while */

#ifdef _LISTDEBUG
    CSTRACE( (CSTRACE_INFO, "ListFreeDelete %X\n",list) );
#endif // _LISTDEBUG

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
	ix_ossl_free(list);
#else
    free(list);
#endif
}



/**************************************************************************/
/* List Iterator operations                                               */ 


/****************************************************************************
//
// PROCEDURE:   ListItrCreate()
//
// PARAMETERS:
//  list        Pointer to a LIST
//
// DESCRIPTION:
//              Creates an iterator for the list
//
// RETURNS:
//              Pointer to List Iterator or NULL if calloc fails.
//
****************************************************************************/

LPLISTITR ListItrCreate(LPLIST list)
{
    LPLISTITR listItr;

    if (list == NULL)
    {
#ifdef _LISTDEBUG
        ix_ossl_message_log("list is NULL.\n");
#endif
        return(NULL);
    }

    listItr = (LPLISTITR)ix_ossl_malloc(sizeof(LISTITR));

    if (!listItr)
    {
#ifdef _LISTDEBUG
        ix_ossl_message_log("Memory allocation for ipv6 linked list failed \n");
#endif
        return(NULL);
    }
    ix_ossl_memset(listItr, 0, sizeof(LISTITR));

#ifdef _LISTDEBUG
    CSTRACE( (CSTRACE_INFO, "ListItrCreate %X\n",listItr) );
#endif // _LISTDEBUG

    if (listItr)
    {
        /* initialize data members */
        listItr->list = list;
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        listItr->current_linux = (LPLISTITEM) FIRST_TIME;
#else
        listItr->current = (LPLISTITEM) FIRST_TIME;
#endif
    }

    return listItr;

} /* ListItrCreate */

/****************************************************************************
//
// PROCEDURE:   ListItrFirst()
//
// PARAMETERS:
//  list        Pointer to a LIST
//
// DESCRIPTION:
//              iterator for the first item in the list
//
// RETURNS:
//              Pointer to List Iterator or NULL if calloc fails.
//
****************************************************************************/

LPLISTITR ListItrFirst(LPLIST list, LPLISTITR listItr)
{
    if (list == NULL)
    {
#ifdef _LISTDEBUG
        ix_ossl_message_log("list is NULL.\n");
#endif
        return(NULL);
    }
    
    if (listItr)
    {
        /* initialize data members */
        listItr->list = list;
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        listItr->current_linux = (LPLISTITEM) FIRST_TIME;
#else
        listItr->current = (LPLISTITEM) FIRST_TIME;
#endif
    }

    return listItr;

} /* ListItrFirst */



/****************************************************************************
//
// PROCEDURE:   ListItrGetNext()
//
// PARAMETERS:
//  listItr     Pointer to a LISTITR
//
// DESCRIPTION:
//              Gets the data pointed by the "current" pointer in the
//              list iterator. "current" is incremented before.
//
// RETURNS:
//              Pointer to data. NULL if end of list is reached
//
****************************************************************************/

void * ListItrGetNext(LPLISTITR listItr)
{
    if (listItr == NULL)
    {
#ifdef _LISTDEBUG
        ix_ossl_message_log("list is NULL.\n");
#endif
        return(NULL);
    }

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    if (listItr->current_linux == (LPLISTITEM)FIRST_TIME)
#else
    if (listItr->current == (LPLISTITEM)FIRST_TIME)
#endif
    {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        listItr->current_linux = listItr->list->head;
#else
        listItr->current = listItr->list->head;
#endif
    }
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    else if (listItr->current_linux == NULL)
#else
    else if (listItr->current == NULL)
#endif
    {
        return NULL;
    }
    else
    {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        listItr->current_linux = listItr->current_linux->next;
#else
        listItr->current = listItr->current->next;
#endif
    }

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    if (listItr->current_linux)
#else
    if (listItr->current)
#endif
    {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        return listItr->current_linux->data;
#else
        return listItr->current->data;
#endif
    }
    else
    {
        return NULL;
    }
} /* ListItrGetNext */

/****************************************************************************
//
// PROCEDURE:   ListItrInsertAfter()
//
// PARAMETERS:
//  listItr     Pointer to a LISTITR
//
//  data        Pointer to data item to be iserted
//
// DESCRIPTION:
//              Inserts the data item after the data item pointed by
//              "current" pointer.
//
// RETURNS:
//              TRUE on success. FALSE if calloc fails
//
****************************************************************************/

ix_uint8 ListItrInsertAfter(LPLISTITR listItr, void * data)
{
    LPLISTITEM newListItem;

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    if (listItr == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,("listItr is Null"));
    if (data == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,("data is Null"));
    if (((unsigned int)listItr->current_linux) != FIRST_TIME)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
		("listItr->current_linux != FIRST_TIME"));
    newListItem = (LPLISTITEM) ix_ossl_malloc(sizeof(LISTITEM));

    if (!newListItem)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM, 
	          ("Memory allocation for ipv6 linked list failed \n"));
    }
    ix_ossl_memset(newListItem, 0, sizeof(LISTITEM)); 

#else
    assert(listItr);
    assert(data);
    assert(((unsigned int)listItr->current) != FIRST_TIME);
    newListItem = (LPLISTITEM) calloc (1, sizeof(LISTITEM));
#endif

    if (newListItem)
    {
        newListItem->data = data;

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        if (listItr->current_linux == NULL)
#else
        if (listItr->current == NULL)
#endif
        {
            if (listItr->list->count == 0)
            {
                /* there are no items right now */
                listItr->list->head = listItr->list->tail = newListItem;
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
                listItr->current_linux = listItr->list->head;
#else
                listItr->current = listItr->list->head;
#endif
            } /* if */
            else
            {
                /* there are some items, current points to null */
                newListItem->prev = listItr->list->tail;
                newListItem->next = NULL;
                listItr->list->tail->next = newListItem;
                listItr->list->tail = newListItem;
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
                listItr->current_linux = newListItem;
#else
                listItr->current = newListItem;
#endif
            } /* else */
        } /* if */
        else
        {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
            newListItem->prev = listItr->current_linux;
#else
            newListItem->prev = listItr->current;
#endif
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
            newListItem->next = listItr->current_linux->next;
#else
            newListItem->next = listItr->current->next;
#endif

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
            if (listItr->current_linux->next)
#else            
            if (listItr->current->next)
#endif
            {
                /* there are items after */
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
                listItr->current_linux->next->prev = newListItem;
#else
                listItr->current->next->prev = newListItem;
#endif
            } /* if */
            else
            {
                /* last item so move the tail */
                listItr->list->tail = newListItem;
            } /* else */
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
            listItr->current_linux->next = newListItem;
#else
            listItr->current->next = newListItem;
#endif
        } /* else */

        /* increment number of items */
        listItr->list->count ++;
        return TRUE;
    } /* if */
    else
    {
        return FALSE;
    } /* else */

} /* ListItrInsertAfter */


/****************************************************************************
//
// PROCEDURE:   ListItrInsertBefore()
//
// PARAMETERS:
//  listItr     Pointer to a LISTITR
//
//  data        Pointer to data item to be iserted
//
// DESCRIPTION:
//              Inserts the data item before the data item pointed by
//              "current" pointer.
//
// RETURNS:
//              TRUE on success. FALSE if calloc fails
//
****************************************************************************/

ix_uint8 ListItrInsertBefore(LPLISTITR listItr, void * data)
{
    LPLISTITEM newListItem;

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    if (listItr == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,("listItr is Null"));
    if (data == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,("data is Null"));
    if (((unsigned int)listItr->current_linux) != FIRST_TIME)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
		("listItr->current_linux != FIRST_TIME"));
    newListItem = (LPLISTITEM) ix_ossl_malloc(sizeof(LISTITEM));

    if (!newListItem)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM, 
	          ("Memory allocation for ipv6 linked list failed \n"));
    }
    ix_ossl_memset(newListItem, 0, sizeof(LISTITEM));

#else
    assert(listItr);
    assert(data);
    assert(((unsigned int)listItr->current) != FIRST_TIME);
    newListItem = (LPLISTITEM) calloc (1, sizeof(LISTITEM));
#endif

    
    if (newListItem)
    {
        newListItem->data = data;

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        if (listItr->current_linux == NULL)
#else
        if (listItr->current == NULL)
#endif
        {
            if (listItr->list->count == 0)
            {
                /* there are no items right now */
                listItr->list->head = listItr->list->tail = newListItem;
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
                listItr->current_linux = listItr->list->head;
#else
                listItr->current = listItr->list->head;
#endif
            } /* if */
            else
            {
                /* there are some items, current points to null */
                newListItem->prev = listItr->list->tail;
                newListItem->next = NULL;
                listItr->list->tail->next = newListItem;
                listItr->list->tail = newListItem;
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
                listItr->current_linux = newListItem;
#else
                listItr->current = newListItem;
#endif
            } /* else */
        } /* if */
        else
        {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
            newListItem->next = listItr->current_linux;
#else
            newListItem->next = listItr->current;
#endif
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
            newListItem->prev = listItr->current_linux->prev;
#else
            newListItem->prev = listItr->current->prev;
#endif

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
            if (listItr->current_linux->prev)
#else            
            if (listItr->current->prev)
#endif
            {
                /* there are items before */
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
                listItr->current_linux->prev->next = newListItem;
#else
                listItr->current->prev->next = newListItem;
#endif
            } /* if */
            else
            {
                /* first item so move the head */
                listItr->list->head = newListItem;
            } /* else */

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
            listItr->current_linux->prev = newListItem;
#else
            listItr->current->prev = newListItem;
#endif
        } /* else */

        /* increment number of items */
        listItr->list->count ++;
        return TRUE;
    } /* if */
    else
    {
        return FALSE;
    } /* else */

} /* ListItrInsertBefore */


/****************************************************************************
//
// PROCEDURE:   ListItrRemoveItem()
//
// PARAMETERS:
//  listItr     Pointer to a LISTITR
//
// DESCRIPTION:
//              Removes the data item pointed by "current" pointer.
//
// RETURNS:
//              TRUE on success. FLASE if current is invalid.
//
****************************************************************************/

ix_uint8 ListItrRemoveItem(LPLISTITR listItr)
{
    LPLISTITEM tmpItem;
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
	if (listItr == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
		("listItr is Null"));
	if (listItr->current_linux == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
		("listItr->current_linux is Null"));
#else
    assert(listItr);
    assert(listItr->current);
#endif

    /* not yet iterated */
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    if (listItr->current_linux == (LPLISTITEM) FIRST_TIME)
#else
    if (listItr->current == (LPLISTITEM) FIRST_TIME)
#endif
    {
        return FALSE;
    }

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    if (listItr->current_linux->next == NULL && listItr->current_linux->prev == NULL)
#else
    if (listItr->current->next == NULL && listItr->current->prev == NULL)
#endif
    {
        /* this is the only item */
        listItr->list->head = listItr->list->tail = NULL;
        tmpItem = NULL;
    }
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    else if (listItr->current_linux->next == NULL)
#else
    else if (listItr->current->next == NULL)
#endif
    {
        /* this is the last item */
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        listItr->list->tail = listItr->current_linux->prev;
#else
        listItr->list->tail = listItr->current->prev;
#endif
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        listItr->current_linux->prev->next = NULL;
#else
        listItr->current->prev->next = NULL;
#endif
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        tmpItem = listItr->current_linux->prev;
#else
        tmpItem = listItr->current->prev;
#endif
    }
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    else if (listItr->current_linux->prev == NULL)
#else
    else if (listItr->current->prev == NULL)
#endif
    {
        /* this is the first item */
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        listItr->list->head = listItr->current_linux->next;
#else
        listItr->list->head = listItr->current->next;
#endif
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        listItr->current_linux->next->prev = NULL;
#else
        listItr->current->next->prev = NULL;
#endif
        tmpItem = (LPLISTITEM) FIRST_TIME;
    }
    else
    {
        /* somewhere in the middle */
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        listItr->current_linux->next->prev = listItr->current_linux->prev;
#else
        listItr->current->next->prev = listItr->current->prev;
#endif
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        listItr->current_linux->prev->next = listItr->current_linux->next;
#else
        listItr->current->prev->next = listItr->current->next;
#endif

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        tmpItem = listItr->current_linux->prev;
#else
        tmpItem = listItr->current->prev;
#endif
    } /* if */

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    ix_ossl_free(listItr->current_linux);
#else
    free(listItr->current);
#endif
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    listItr->current_linux = tmpItem;
#else
    listItr->current = tmpItem;
#endif
    listItr->list->count --;

    return TRUE;

} /* ListItrRemove */


/****************************************************************************
//
// PROCEDURE:   ListItrDelete()
//
// PARAMETERS:
//  listItr     Pointer to a LISTITR
//
// DESCRIPTION:
//              Frees the LISTITR
//
// RETURNS:
//              None
//
****************************************************************************/

void ListItrDelete(LPLISTITR listItr)
{
    if (listItr == NULL)
    {
#ifdef _LISTDEBUG
        ix_ossl_message_log("list is NULL.\n");
#endif
        return;
    }

#ifdef _LISTDEBUG
    CSTRACE( (CSTRACE_INFO, "ListItrDelete %X\n",listItr) );
#endif // _LISTDEBUG
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    ix_ossl_free(listItr);
#else
    free(listItr);
#endif
} /* ListItrDelete */
