/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 * 
 * = PRODUCT
 *      Intel(r) IXA SDK 3.x for the IXP2xxx Network Processors
 *
 * = MODULE
 *      NATPT Core Component
 *
 * = FILENAME
 *      ix_cc_natpt_utils.c
 *
 * = DESCRIPTION
 *      The file  contains some utility functions to assist NATPT core
 *      component. 
 *
 * = CREATION TIME
 *      03/03/2003
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */

#define IX_ERROR_FILE_IDENT "$Id: ix_cc_natpt_utils.c,v 1.10 2003/10/31 19:24:02 ktseng Exp $";

/**
 * System defined include files required.
 */

#include "ix_cc_error.h"

/**
 * User defined include files required.
 */

#include "cc/natpt/internal/ix_cc_natpt_utils.h"

/**
 * Preprocessor symbols and macros used in this file.
 */

/**
 * Types definitions whose scope is limited to this file.
 */

/**
 * Variable declarations global to this file only.  Externs are followed by
 * static variables.
 */

/* This function prints an IPv6 address */
void _ix_cc_natpt_print_ipv6_address(ix_uint128 *pAddr)
{
    ix_uint8 i;

    for (i = 0; i < 4; i++)
    {
        ix_ossl_message_log("IPv6[%d] %lx \n", i, pAddr->aUint32[i]);
    }

    return;
}

/* NAME: NATPT_IS_IPV6_ADDRESS_ZERO
 *
 * DESCRIPTION: Check to see if an IPv6 address is zero.
 *
 * @Param: IN arg_src - Pointer to the IPv6 address.
 *
 * return: 1 if it is.
 *         0 Otherwise
 * 
 */
ix_uint8
NATPT_IS_IPV6_ADDRESS_ZERO(ix_uint128 *arg_ipv6Addr)
{
    ix_uint8 i;
    ix_uint32 value;
    
    for (i = 0; i < 4; i++)
    {
        value = arg_ipv6Addr->aUint32[i];
        if (value != 0x0)
        {
            return 0;   /* FALSE */
        }
    }

    return 1;   /* TRUE */
}


/* NAME: MAKE_IPV4_MAPPED_IPV6_ADDRESS
 *
 * DESCRIPTION: Create IPv4 mapped IPv6 Address
 *
 * @Param: IN arg_ipv4Addr - IPv4 address.
 *
 * @Param: IN arg_ipv6Addr - Pointer to the IPv6 address.
 *
 * return: none.
 *
 */
void
MAKE_IPV4_MAPPED_IPV6_ADDRESS(ix_uint32 arg_ipv4Addr, 
                              ix_uint128 *arg_ipv6Addr)
{
    arg_ipv6Addr->aUint32[0] = 0;
    arg_ipv6Addr->aUint32[1] = 0;
    arg_ipv6Addr->aUint32[2] = 0x0000ffff;
    arg_ipv6Addr->aUint32[3] = arg_ipv4Addr;
}


/* NAME: NATPT_COPY_IPV6_ADDRESS
 *
 * DESCRIPTION: Copies the source IPv6 address into the destination.
 *
 * @Param: IN arg_dst - Pointer to the destination IPv6 address.
 * @Param: IN arg_src - Pointer to the source IPv6 address.
 *
 * return: none.
 *
 */
void
NATPT_COPY_IPV6_ADDRESS(ix_uint128 *arg_dst, ix_uint128 *arg_src)
{
    ix_uint8 i;

    for (i = 0; i < 4; i++)
    {
        arg_dst->aUint32[i] = arg_src->aUint32[i];
    }

    return;
}

/* 
 * NAME: NATPT_IS_IPV6_ADDRESS_EQUAL
 *
 * DESCRIPTION: Compare two IPv6 addresses. 
 *
 * @Param: IN arg_src - Pointer to the IPv6 address.
 * @Param: OUT arg_dst - Pointer to where the result of the operation 
 *                        should be stored.
 * return: 1 if they are equal
 *         0 Otherwise
 *
 */
ix_uint8
NATPT_IS_IPV6_ADDRESS_EQUAL(ix_uint128 *arg_src, ix_uint128 *arg_dst)
{
    ix_uint8 i;
    ix_uint32 value1;
    ix_uint32 value2;

    for (i = 0; i < 4; i++)
    {
        value1 = arg_src->aUint32[i];
        value2 = arg_dst->aUint32[i];

        if (value1 != value2)
        {
            return 0;    /* FALSE */
        }
    }

    return 1;
}

