/*/////////////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
///////////////////////////////////////////////////////////////////////////////
//
//      File Name: ipv6_fwder_uc.h
//
//      Purpose: This file contains defines that are used by the microcode,
//               microC and the core components (might need appropriate 
//               switches to be turned on).
//
//      History:
//
//      Date            Comment                         By
//      --------------------------------------------------------------------	
//
//      12/10/2002      Created                         Ravi Murty
//
//
///////////////////////////////////////////////////////////////////////////////
*/

#ifndef	__IPV6_FWDER_UC_H__
#define	__IPV6_FWDER_UC_H__

#ifdef	MICROENGINE
#ifndef	IP_HDR_CACHE_LM
#ifndef	IP_HDR_OFFSET
#error	"IP_HDR_OFFSET not defined"
#endif
#endif	
#endif	/* MICROENGINE */

#ifdef USE_IMPORT_VAR

.import_var IPV6_TRIE_TABLE_SRAM_BASE_16
.import_var IPV6_TRIE_TABLE_SRAM_BASE_8
.import_var IPV6_L3_NEXTHOP_TABLE_SRAM_BASE

#endif

/* 
 * Exception codes: Used both by the microcode and the core component 
 * Should not exceed 16 bits 
 */
#define    IPV6_EXCP_VERSION             0x40
#define    IPV6_EXCP_BAD_TOTAL_LENGTH    0x41
#define    IPV6_EXCP_BAD_HOPLIMIT        0x42
#define    IPV6_EXCP_MULTICAST           0x43
#define    IPV6_EXCP_LINKLOCAL           0x44
#define    IPV6_EXCP_OPTIONS             0x45

#define    IPV6_EXCP_LKUPFAILED          0x46
#define    IPV6_EXCP_LOCAL               0x47
#define    IPV6_EXCP_MTU                 0x48
#define    IPV6_EXCP_REDIRECT            0x49

#ifdef IPV6_STATS

/* 
 * Offset of various Microblock maintained counters in 
 * Big-Endian format.
 *
 * Listed here are the offsets of various counters maintained by 
 * the microblocks. 
 */

/* 
 * Incoming Counters 
 */

#define	IPV6_IN_RECV           0   /* total recd pkts */
#define	IPV6_IN_OCTETS         4   /* total bytes recd in IPv6 pkts */
#define IPV6_IN_FORW           8   /* count of non-local pkts for which a route
                                    * was found
				    */
#define IPV6_IN_ADDR_ERRORS    12  /* count of pkts discarded because dest
                                    * address was in error
				    */
#define IPV6_IN_TRUNCATED      16  /* count of pkts discarded because the 
                                    * packet did not carry enough data
				    */
#define IPV6_IN_HDR_ERRORS     20  /* number of packets in error 
                                    */
/* 
 * Outgoing counters 
 */

#define IPV6_OUT_FORW          0   /* count of pkts for which route 
                                    * was found 
				    */
#define IPV6_OUT_TRANSMITS     4   /* count of pkts forwarder to lower layer */
#define IPV6_OUT_OCTETS        8   /* number of bytes in IPv6 pkts delivered 
                                    * to lower layers for trasmission 
				    */
/* Closest power of 2 to include all the IN counter mentioned above. */
#define	IPV6_IN_STATS_PER_PORT_SHIFT      5
/* Closest power of 2 to include all the OUT counters mentioned above */
#define IPV6_OUT_STATS_PER_PORT_SHIFT     4



#ifdef IXP_MICROCODE  /* for microcode */

#define_eval IPV6_IN_STATS_PER_PORT  (1 << IPV6_IN_STATS_PER_PORT_SHIFT)
#define_eval IPV6_OUT_STATS_PER_PORT (1 << IPV6_OUT_STATS_PER_PORT_SHIFT)
#define_eval IPV6_OUT_STATS_PER_BLADE_SHIFT \
             (IPV6_OUT_STATS_PER_PORT_SHIFT + PORTS_PER_BLADE_SHIFT)

#else /* for microC + Core component */

#define IPV6_IN_STATS_PER_PORT  (1 << IPV6_IN_STATS_PER_PORT_SHIFT)
#define IPV6_OUT_STATS_PER_PORT (1 << IPV6_OUT_STATS_PER_PORT_SHIFT)
#define IPV6_OUT_STATS_PER_BLADE_SHIFT \
        (IPV6_OUT_STATS_PER_PORT_SHIFT + PORTS_PER_BLADE_SHIFT)

#endif /* IXP_MICROCODE */

#endif /* IPV6_STATS */

#ifdef IXP_MICROCODE /* for microcode */

#define_eval PORTS_PER_BLADE   (1 << PORTS_PER_BLADE_SHIFT)

#else  /* for microC + core components */

#define PORTS_PER_BLADE (1 << PORTS_PER_BLADE_SHIFT)

#endif /* IXP_MICROCODE */

#define IPV6_NUM_HASH_BITS      14

/*
 * Next Hop Information.
 */

/*
 * Structure of the Next-Hop Entry.
 *
 *  Structure of a "real" next hop entry. Each entry is 4 long words
 *  (16 bytes) long.
 *
 *  Byte 3                  2                  1               0
 * |----------------|----------------|----------------|----------------|
 * +-------------------------------------------------------------------+
 * |                |     Valid      |       Generic Flags             |
 * |----------------------------------------------------  -------------|
 * | Blade ID       |    Type |  Res |       Next Hop ID               |
 * |-------------------------------------------------------------------|
 * |          MTU                    |          Output Port            |
 * |-------------------------------------------------------------------|
 * |                             Reserved                              |
 * |-------------------------------------------------------------------|
 *
 *
 *  Structure of an "intermediate" nexthop entry. Each entry is 4 long
 *  words (16 bytes) long.
 *
 *  Byte 3                  2                  1               0
 * |----------------|----------------|----------------|----------------|
 * +-------------------------------------------------------------------+
 * | Status | Res   |     Valid      |       Generic Flags             |
 * |----------------------------------------------------  -------------|
 * |         Next Hop ID0            |        Next Hop ID1             |
 * |-------------------------------------------------------------------|
 * |         Next Hop ID2            |        Next Hop ID3             |
 * |-------------------------------------------------------------------|
 * |                                Reserved                           |
 * |-------------------------------------------------------------------|
 *
 */

/* Next Hop Type definitions */
#define IPV6_NH_ID_TYPE_FWRD      0x0
#define IPV6_NH_ID_TYPE_TUNNEL    0x2
#define IPV6_NH_ID_TYPE_COMPRESS  0x4


#define IPV6_NH_FLAGS_START_BIT    0

#ifdef  IXP_MICROCODE

#define_eval IPV6_NH_FLAGS_LOCAL_BIT    (IPV6_NH_FLAGS_START_BIT+0)
#define_eval IPV6_NH_FLAGS_DOWN_BIT     (IPV6_NH_FLAGS_START_BIT+1)
#define_eval IPV6_NH_FLAGS_DROP_BIT     (IPV6_NH_FLAGS_START_BIT+2)
#define_eval IPV6_NH_FLAGS_MULTIPLE_BIT (IPV6_NH_FLAGS_START_BIT+3)

#define_eval IPV6_NH_FLAGS_LOCAL        (1<<IPV6_NH_FLAGS_LOCAL_BIT)
#define_eval IPV6_NH_FLAGS_DOWN         (1<<IPV6_NH_FLAGS_DOWN_BIT)
#define_eval IPV6_NH_FLAGS_DROP         (1<<IPV6_NH_FLAGS_DROP_BIT)
#define_eval IPV6_NH_FLAGS_MULTIPLE     (1<<IPV6_NH_FLAGS_MULTIPLE_BIT)

#else

#define IPV6_NH_FLAGS_LOCAL_BIT     (IPV6_NH_FLAGS_START_BIT+0)
#define IPV6_NH_FLAGS_DOWN_BIT      (IPV6_NH_FLAGS_START_BIT+1)
#define IPV6_NH_FLAGS_DROP_BIT      (IPV6_NH_FLAGS_START_BIT+2)
#define IPV6_NH_FLAGS_MULTIPLE_BIT  (IPV6_NH_FLAGS_START_BIT+3)

#define IPV6_NH_FLAGS_LOCAL         (1<<IPV6_NH_FLAGS_LOCAL_BIT)
#define IPV6_NH_FLAGS_DOWN          (1<<IPV6_NH_FLAGS_DOWN_BIT)
#define IPV6_NH_FLAGS_DROP          (1<<IPV6_NH_FLAGS_DROP_BIT)
#define IPV6_NH_FLAGS_MULTIPLE      (1<<IPV6_NH_FLAGS_MULTIPLE_BIT)

#endif /* IXP_MICROCODE */


/* 
 * The following defines show the (size, offset) pairs for each 
 * field in the next hop structure.
 */

/* Each next hop structures is 16 bytes long */
#define IPV6_NH_SIZE_LW         4
#define IPV6_NH_SIZE_SHIFT      4

/* Long word definitions of each field */
#define IPV6_NH_FLAGS_LW        0
#define IPV6_NH_NHID_LW         1
#define IPV6_NH_BLADE_LW        1
#define IPV6_NH_MTU_LW          2
#define IPV6_NH_OUTPORT_LW      2

/* L3 Next Hop Structure */
#define IPV6_NH_FLAGS           2, 2
#define IPV6_NH_BLADE_ID        4, 1
#define IPV6_NH_ID_TYPE         5, 1
#define IPV6_NH_ID              6, 2
#define IPV6_NH_MTU             8, 2
#define IPV6_NH_OUTPORT         10, 2

#ifdef IPV6_LOADBALANCE

/* Additional fields for supporting "intermediate" 
 * Next-Hop entries. 
 */

#define IPV6_NH_STATUS          0, 1
#define IPV6_NH_NHID0           4, 2
#define IPV6_NH_NHID1           6, 2
#define IPV6_NH_NHID2           8, 2
#define IPV6_NH_NHID3           10, 2

#define IPV6_NH_NHID_TOTAL      4

#endif /* IPV6_LOADBALANCE */

/* Miscellaneous definitions */
#define LM_INDEX0 0
#define LM_INDEX1 1

#define WAIT      0
#define N0_WAIT   1

#endif	/*__IPV6_FWDER_UC_H__ */
