/*******************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 2003 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *******************************************************************************/

/******************************************************************************
 *
 * File:  ipv6_fwder_init.c
 *
 * Abstract:
 *      IPv6 Forwarder Initialization Code.
 *
 * Exported APIs: 
 *   ipv6ForwarderInit()
 *
 *****************************************************************************/


#ifndef	__IPV6_FWDER_INIT_C__
#define	__IPV6_FWDER_INIT_C__


#define FUTURE_COUNT        0x50000
#define IPV6_TRIE_CACHE_LW  10

SIGNAL ipv6CacheRefreshSignal;
__declspec(local_mem) unsigned int trieCache[IPV6_TRIE_CACHE_LW];

/******************************************************************************
 * ipv6ForwarderInit
 *
 *		Descripton 	:
 *        Perform initialization for the IPv6 Forwarder.
 *		
 *		Outputs		:
 *			None.
 *
 *		Inputs		:
 *          None.
 * 
 *      Returns:
 *          None.
 */
INLINE	void ipv6ForwarderInit(void)
{
    SIGNAL capSig;
    SIGNAL sramSig;
    __declspec(sram_read_reg) unsigned int capRead;
    __declspec(sram_write_reg) unsigned int capWrite;
    __declspec(sram_read_reg) unsigned int sramTrieCache[IPV6_TRIE_CACHE_LW];
    unsigned int futureCount;

	__implicit_read(&ipv6CacheRefreshSignal);
    __assign_relative_register(
        &ipv6CacheRefreshSignal, IPV6_CACHE_REFRESH_SIGNAL);

    if (__ctx() == 0)
    {
        /* Enable timestamp */

        cap_csr_read(&capRead, csr_misc_control, ctx_swap, &capSig);
        capWrite = capRead | 0x80;
        cap_csr_write(&capWrite, csr_misc_control, ctx_swap, &capSig);

        /* Write future count */

        local_csr_write(local_csr_csr_ctx_pointer, 0);
        local_csr_write(
            local_csr_indirect_future_count_signal, IPV6_CACHE_REFRESH_SIGNAL);

        futureCount = local_csr_read(local_csr_timestamp_low);
        futureCount += FUTURE_COUNT;
        local_csr_write(
            local_csr_indirect_ctx_future_count, futureCount);

        /* Set up the trie cache.  The first 2 LW (64 bits) are the
         * router prefix, and the last 8 LW are the route information
         * (nexthop and next trie pointers) for each 8 bits of the
         * prefix.
         */

    	sram_read((__declspec(sram_read_reg) void *)sramTrieCache,
    			(volatile __declspec(sram) void *)IPV6_ROUTER_PREFIX_SRAM_BASE,
                IPV6_TRIE_CACHE_LW, ctx_swap, &sramSig);

        trieCache[0] = sramTrieCache[0];
        trieCache[1] = sramTrieCache[1];
        trieCache[2] = sramTrieCache[2];
        trieCache[3] = sramTrieCache[3];
        trieCache[4] = sramTrieCache[4];
        trieCache[5] = sramTrieCache[5];
        trieCache[6] = sramTrieCache[6];
        trieCache[7] = sramTrieCache[7];
        trieCache[8] = sramTrieCache[8];
        trieCache[9] = sramTrieCache[9];

    }

    return;
}

#endif	/* __IPV6_FWDER_INIT_C__ */