////////////////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
////////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: ipv6_fwder_init.uc
//
//      Purpose: Contains supporting macro's that performs the IPv6 Forwarding 
//                 micro block initialzation
//
//      History:
//
//
//      Date            Comment                                     By
//      ------------------------------------------------------------------------
//
//      03/19/2002      Created                                     
//
//
////////////////////////////////////////////////////////////////////////////////

#ifndef    __IPV6_FWDER_INIT_UC__
#define    __IPV6_FWDER_INIT_UC__

#ifdef IPV6_STATS
#include "ipv6.h"
#endif

#define FUTURE_COUNT 0x50000

//
// Global Future count signal
//
.sig  volatile ipv6_cache_refresh_sig
.addr ipv6_cache_refresh_sig IPV6_CACHE_REFRESH_SIGNAL


////////////////////////////////////////////////////////////////////////////////
//
// ipv6_forwarder_init[]
//
// Description: 
//
//     Initializes the IPV6 Forwarder microblock
//
// Outputs:
//     None.
//
// Inputs:
//     None.
//
// CONSTANTS
//     None.
//
// Global variables
//
// Size: 
//
//            ?? instructions. (Worst case cycle count)
//
////////////////////////////////////////////////////////////////////////////////
#macro ipv6_forwarder_init()
#ifdef IPV6_STATS
	//	Global Registers
	.reg @ipv6_sram_counter_base
	.reg @ipv6_out_sram_counter_base
#endif // IPV6_STATS

    // Initialize only in the first context of each microengine

    br!=ctx[0,init_done#]

#ifdef IPV6_STATS
	.begin
		.reg temp

		; initialize SRAM counter base
		#define_eval INCOMING_STATS_SIZE (IPV6_IN_STATS_PER_PORT)*(PORTS_PER_BLADE)

		immed32(@ipv6_sram_counter_base, IPV6_STATS_TABLE_SRAM_BASE)
		immed32(temp, INCOMING_STATS_SIZE)
		alu[@ipv6_out_sram_counter_base, @ipv6_sram_counter_base, +, temp]
	.end
#endif //IPV6_STATS

    //
    // Set up timer interrupt for Local Memory trie-cache refresh
    //
    
    // Enable timestamp. 
    .begin
        .sig cap_read_sig
        .reg $cap_read
        cap[read, $cap_read, MISC_CONTROL], ctx_swap[cap_read_sig]
        alu[$cap_read, 0x80, OR, $cap_read]
        cap[write, $cap_read, MISC_CONTROL], ctx_swap[cap_read_sig]
    .end

    // Write future count

    ;; which context to use
    local_csr_wr[CSR_CTX_Pointer, 0]
    nop
    nop

    local_csr_wr[INDIRECT_FUTURE_COUNT_SIGNAL, IPV6_CACHE_REFRESH_SIGNAL]
    .begin
        .reg temp future_count_reg
        local_csr_rd[timestamp_low]
        immed[temp, 0]
        immed32[future_count_reg, FUTURE_COUNT]
        alu[temp, temp, +, future_count_reg]
        local_csr_wr[INDIRECT_CTX_FUTURE_COUNT, temp]
    .end
    nop
    nop

    //
    // Set up trie-cache
    //

    // Copy routers IPv6 prefix from SRAM 
    // Prepare the whole cache of 2 + 8 words
    // 2 long-worads for prefix value, 8 long-words for trie-entries
    .begin
        .reg sram_base
        .sig sram_swap_signal
        .reg tmp

        xbuf_alloc($t, 10, read)

        ;; set LM base pointer 
        immed[tmp, IPV6_TRIE_CACHE_LM_BASE]
        local_csr_wr[active_lm_addr_1, tmp]
        immed32[sram_base, IPV6_ROUTER_PREFIX_SRAM_BASE]

        ;; read from SRAM
        alu[--, --, B, 0x1A, <<21]
        sram[read, $t[0], sram_base, 0, max_10], 
                indirect_ref, ctx_swap[sram_swap_signal]

        ;; move to local memory
        alu[*l$index1++, --, B, $t[0]]
        alu[*l$index1++, --, B, $t[1]]
        alu[*l$index1++, --, B, $t[2]]
        alu[*l$index1++, --, B, $t[3]]
        alu[*l$index1++, --, B, $t[4]]
        alu[*l$index1++, --, B, $t[5]]
        alu[*l$index1++, --, B, $t[6]]
        alu[*l$index1++, --, B, $t[7]]
        alu[*l$index1++, --, B, $t[8]]
        alu[*l$index1++, --, B, $t[9]]

    .end

init_done#:

#endm



#endif    // __IPV6_FWDER_INIT_UC__

