/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 5
 *
 * = MODULE
 *      IP Options Handler module
 *
 * = FILENAME
 *      ip_options.h
 *
 * = DESCRIPTION
 *      This file contains definition of IP options handling
 *      functions
 *
 * = CREATION TIME
 *      12/12/2002
 *
 * = CHANGE HISTORY
 *
 *
 * ============================================================================
 * $Id: ip_options.h,v 1.2 2003/01/31 01:41:12 rmurty Exp $
 */

#if !defined(__IP_OPTIONS_H__)
#define __IP_OPTIONS_H__

#include "ix_cc_error.h"
#include "ix_netmacros.h"
#include "icmp6.h"

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

/**
 * Preprocessor symbol and macro definitions.
 */

/* Codes for different options that might be present in the 
 * Next Header field in the IPv6 Header and other extension headers.
 */

/**
 * Prototypes for interface functions.
 */

/**
 * NAME: _ix_cc_ipv6_parse_ext_headers
 *
 * DESCRIPTION: This function is called when the packet has one or more
 * extension headers (indicated by the presence of an options header
 * value in the "next header" field in the IPv6 header.
 * It appropriately modifies the header of the packet and decides if the 
 * packet is to be dropped or forwarded. The actual dropping and forwarding 
 * of the packet is caller's responsibility 
 * 
 * @Param:  - IN arg_hBuffer - the actual IPv6 packet 
 *            IN arg_pIphdr - Pointer to IP header
 *
 * @Param:  - INOUT  - None
 * @Param:  - OUT - arg_pDrop - On return set to 1 if the packet should 
 *                              be dropped
 *                              On return set to 0 if the packet should 
 *                              be forwarded  
 *
 * @Return: IX_SUCCESS if successful 
 *          valid ix_error token for failure.
 *
 *          arg_pDrop should be set accordingly on return
 */
ix_error _ix_cc_ipv6_parse_ext_headers(ix_cc_ipv6_header *arg_pIphdr, 
                                       ix_buffer_handle arg_hBuffer,
                                       ix_uint32 *arg_pDrop);


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */


#endif /* end !defined(__IP_OPTIONS_H__) */
