/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = FILENAME
 *      ix_cc_ipv6_icmp_nd.h
 *
 * = DESCRIPTION
 *      
 *
 * = AUTHOR
 *      Amol Kulkarni
 *
 * = CHANGE HISTORY
 *       11/28/2002 1:53:42 PM - creation time
 *
 * ============================================================================
 * $Id: ix_cc_ipv6_icmp_nd.h,v 1.7 2003/05/12 04:56:29 akulkar1 Exp $
 */

#if !defined(__ICMP_ND_H__)
#define __ICMP_ND_H__

#include "ix_cc_ipv6_nd.h"

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

#define IX_CC_IPV6_ICMP_TYPE_RTR_SOL    133
#define IX_CC_IPV6_ICMP_TYPE_RTR_ADV    134
#define IX_CC_IPV6_ICMP_TYPE_NBR_SOL    135
#define IX_CC_IPV6_ICMP_TYPE_NBR_ADV    136
#define IX_CC_IPV6_ICMP_TYPE_REDIRECT   137
    
#define IX_CC_IPV6_ND_OPT_SRC_L2_ADDR 1
#define IX_CC_IPV6_ND_OPT_DST_L2_ADDR 2

#define IX_CC_IPV6_ND_NBR_ADV_MSG_MIN_LEN  24 /*min length of this ICMP pkt*/
#define IX_CC_IPV6_ND_NBR_SOL_MSG_MIN_LEN  24

/********Message definitions*******/
    
/**
 * TYPENAME: ix_ipv6_nd_rtr_adv_msg
 *
 * DESCRIPTION:
 *      This structure defines the message format for the router advertisement 
 *      message. Note that this structure only defines the mandatory fields.
 *      Optional fields are defined elsewhere.
 */
typedef struct ix_s_ipv6_icmp_rtr_adv_msg
{
    #if ix_byte_order == ix_little_endian
        ix_uint32   rtrlifetime:16;
        ix_uint32   reserved:6;
        ix_uint32   otherconfig:1;
        ix_uint32   managed:1;
        ix_uint32   curhoplimit:8;
    #else
        ix_uint32   curhoplimit:8;
        ix_uint32   managed:1;
        ix_uint32   otherconfig:1;
        ix_uint32   reserved:6;
        ix_uint32   rtrlifetime:16;
    #endif
    ix_uint32   reachabletime;
    ix_uint32   retranstimer;
}__attribute__ ((packed)) ix_ipv6_nd_rtr_adv_msg;

/**
 * TYPENAME: ix_ipv6_icmp_nbr_adv_msg
 *
 * DESCRIPTION:
 *        This structure defines the message format for the neighbor advertisement
 *        message. Note that this structure only defines the mandatory fields. 
 *        Optional fields are defined elsewhere.
 */
typedef struct ix_s_ipv6_icmp_nbr_adv_msg
{
    #if IX_BYTE_ORDER == IX_LITTLE_ENDIAN
        ix_uint16   reserverd1;
        ix_uint8    reserverd2;
        ix_uint8    reserverd3:5;
        ix_uint8    override:1;
        ix_uint8    solicited:1;
        ix_uint8    router:1;
    #else
        ix_uint32    router:1;
        ix_uint32    solicited:1;
        ix_uint32    override:1;
        ix_uint32    reserverd:29;
    #endif
    ix_ipv6_addr    targetAddr;
}__attribute__ ((packed)) ix_ipv6_icmp_nbr_adv_msg;

/**
 * TYPENAME: ix_ipv6_nd_nbr_sol_msg
 *
 * DESCRIPTION:
 *        This structure defines the message format for the neighbor 
 *        solicitation message. Note that this structure only defines the 
 *        mandatory fields.  Optional fields are defined elsewhere.
 */
typedef struct ix_s_ipv6_icmp_nbr_sol_msg
{
    ix_uint32       reserved;
    ix_ipv6_addr    targetAddr;
}__attribute__ ((packed)) ix_ipv6_icmp_nbr_sol_msg;

/**
 * TYPENAME: ix_ipv6_nd_options_l2_addr
 *
 * DESCRIPTION:
 *      This structure defines the format for the source/target
 *      link layer address options.
 */
typedef struct ix_s_ipv6_icmp_options_l2_addr
{
    ix_uint8    type;
    ix_uint8    length;         /*Length in number of 8 octets. The length is 
                                  almost always 1*/
    ix_uint8    linkLayerAddr[IX_CC_IPV6_MAC_ADDR_LEN];   
}__attribute__ ((packed)) ix_ipv6_nd_options_l2_addr;

/*************General Structure definitions**************/

/**
 * TYPENAME: ix_ipv6_nd_nbr_adv_params
 *
 * DESCRIPTION:
 *      This structure contains all the parameters required to construct
 *      a neighbor advertisement message.
 *      This structure will be populated and passed in to a function to 
 *      construct the neighbor advertisement message. The resultant message
 *      will be in the format specified in the ix_ipv6_icmp_nbr_adv_msg 
 *      structure.
 */
typedef struct ix_s_ipv6_icmp_nbr_adv_params
{
    ix_uint8    isRouter;
    ix_uint8    solicited;
    ix_uint8    override;
    ix_ipv6_addr    targetAddr;

    ix_uint8    l2AddrLength;
    ix_uint8    l2Addr[IX_CC_IPV6_MAC_ADDR_LEN];

    ix_uint8    macHdrSpecified;
    ix_ipv6_eth_hdr    macHdr;
} ix_ipv6_nd_nbr_adv_params;

/**
 * TYPENAME: ix_ipv6_nd_rtr_adv_params
 *
 * DESCRIPTION:
 *      This structure contains all the parameters required to construct
 *      a neighbor advertisement message.
 *      This structure will be populated and passed in to a function to 
 *      construct the neighbor advertisement message. The resultant message
 *      will be in the format specified in the ix_ipv6_icmp_nbr_adv_msg 
 *      structure.
 *
 */
/*typedef struct ix_s_ipv6_icmp_rtr_adv_params
{
    ix_uint32   nextHopLimit;
    ix_uint8    managed;
    ix_uint8    otherConfig;
    ix_uint32   rtrLifetime;
    ix_uint32   reachableTime;
    ix_uint32   retransTimer;
} ix_ipv6_nd_rtr_adv_params;*/


/****Function prototypes*/
/**
 * NAME: ix_cc_ipv6_icmp_send_rtr_adv
 *
 * DESCRIPTION: This function will create the payload for the router 
 *              advertisement message. It will then call the appropriate 
 *              function to build the ICMP message and send it out of the
 *              proper interface.
 * 
 * @Param:  - IN arg_SrcAddr: The source address
 * @Param:  - IN arg_DstAddr: The destination address.
 * @Param:  - IN arg_Params: Pointer to the rtr_adv_params structure. 
 * @Param:  - IN arg_OutPutPort: The Port from which this NBR sol is going 
 *            from.
 * @Param:  - IN arg_bladeId: The blade ID corresponding to the output port.
 *
 * @Return: 0 if successful or a valid ix_error token for failure.
 */
ix_error ix_cc_ipv6_icmp_send_rtr_adv(
                ix_ipv6_addr    arg_SrcAddr,
                ix_ipv6_addr    arg_DstAddr,
                ix_ipv6_nd_rtr_adv_params *arg_Params,
                ix_uint16       arg_OutPutPort,
                ix_uint32       arg_bladeId);

/**
 * NAME: ix_cc_ipv6_icmp_send_nbr_adv
 *
 * DESCRIPTION: This function will create the payload for the neighbor 
 *              advertisement message. It will then call the appropriate 
 *              function to build the ICMP message and send it out of the
 *              proper interface.
 * 
 * @Param:  - IN arg_SrcAddr: The source address
 * @Param:  - IN arg_DstAddr: The destination address.
 * @Param:  - IN arg_Params: Pointer to the nbr_adv_params structure. 
 * @Param:  - IN arg_OutPutPort: The Port from which this NBR sol is going 
 *            from.
 * @Param:  - IN arg_bladeId: The blade ID corresponding to the output port.
 *
 * @Return: 0 if successful or a valid ix_error token for failure.
 */
ix_error ix_cc_ipv6_icmp_send_nbr_adv(
                ix_ipv6_addr    arg_SrcAddr,
                ix_ipv6_addr    arg_DstAddr,
                ix_ipv6_nd_nbr_adv_params *arg_Params,
                ix_uint16       arg_OutPutPort,
                ix_uint32       arg_bladeId);

/**
 * NAME: ix_cc_ipv6_icmp_send_nbr_sol
 *
 * DESCRIPTION: This function will create the payload for the neighbor 
 *              solicitation message. It will then call the appropriate 
 *              function to build the ICMP message and send it out of the
 *              proper interface.
 * 
 * @Param:  - IN arg_SrcAddr: The source address
 * @Param:  - IN arg_DstAddr: The destination address.
 * @Param:  - IN arg_TargetAddrParam: The target address.
 * @Param:  - IN arg_pL2OptParams: Pointer to an array containing the L2 
 *                                 address. If the option is unspecified, 
 *                                 this argument should be NULL.
 * @Param:  - IN arg_OutPutPort: The Port from which this NBR sol is going 
 *            from.
 * @Param:  - IN arg_bladeId: The blade ID corresponding to the output port.
 *
 * @Return: 0 if successful or a valid ix_error token for failure.
 */
ix_error ix_cc_ipv6_icmp_send_nbr_sol(
                ix_ipv6_addr    arg_SrcAddr,
                ix_ipv6_addr    arg_DstAddr,
                ix_ipv6_addr    arg_TargetAddrParam,
                ix_uint8        *arg_pL2OptParam,
                ix_uint16       arg_OutPutPort,
                ix_uint32       arg_bladeId);

                                  
#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /*#if !defined(__ICMP_ND_H__)*/ 
