/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      IPv6 Forwarder Core Component
 *
 * = FILENAME
 *      ix_cc_ipv6_init.h
 *
 * = DESCRIPTION
 *      This file defines all data types and function prototypes required 
 *      internaly by the IPv6 CCI module.
 *
 * = AUTHOR
 *       Ravi Murty
 *       ravi.murty@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      11/11/2002 7:45AM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */

#if !defined(__IX_CC_IPV6_INIT_H__)

#define __IX_CC_IPV6_INIT_H__
#include "ix_rm.h"
#include "ix_ossl.h"
#include "ix_cc.h"
#include "cc/ix_cc_rtmv6.h"
#include "cc/ix_cc_ipv6.h"

#if defined(__cplusplus)
extern "C" {
#endif /* end defined(__cplusplus) */


/* Port Forwarding - Enable / Disable */
#define IX_CC_IPV6_DISABLE_PORT_STATUS IX_CC_PHYSICAL_IF_STATUS_DOWN
#define IX_CC_IPV6_ENABLE_PORT_STATUS  IX_CC_PHYSICAL_IF_STATUS_UP

/* Registry Values */
#define IX_CC_IPV6_REGISTRY_KEY_ROUTE_TABLE_SIZE  \
                                            "IPV6/ROUTE_TABLE_SIZE_HINT"
#define IX_CC_IPV6_REGISTRY_KEY_ROUTE_TABLE_TYPE  \
                                            "IPV6/ROUTE_TABLE_TYPE"
#define IX_CC_IPV6_REGISTRY_KEY_RTM_LOOKUP_MEMORY_TYPE  \
                                            "IPV6/RTM_LOOKUP_MEMORY_TYPE"
#define IX_CC_IPV6_REGISTRY_KEY_RTM_LOOKUP_CHANNEL  \
                                            "IPV6/RTM_LOOKUP_CHANNEL"
#define IX_CC_IPV6_REGISTRY_KEY_NEXT_HOP_TABLE_SIZE  \
                                            "IPV6/NEXT_HOP_TABLE_SIZE_HINT"
#define IX_CC_IPV6_REGISTRY_KEY_NEXT_HOP_DATABASE_MEMORY_TYPE  \
                                            "IPV6/NEXT_HOP_MEMORY_TYPE"
#define IX_CC_IPV6_REGISTRY_KEY_NEXT_HOP_DATABASE_CHANNEL  \
                                            "IPV6/NEXT_HOP_DATABASE_CHANNEL"
#define IX_CC_IPV6_REGISTRY_KEY_LOCAL_BLADE_ID  "IPV6/LOCAL_BLADE_ID"
#define IX_CC_IPV6_REGISTRY_KEY_ME_FOR_IPV6  "IPV6/ME_FOR_IPV6"


#ifdef SPLIT_V6_TRIE_TABLE
#define IX_CC_IPV6_REGISTRY_KEY_RTM_LOOKUP_CHANNEL_16BIT_TRIE  \
                                            "IPV6/RTM_LOOKUP_CHANNEL_16BIT_TRIE"
#endif /* SPLIT_V6_TRIE_TABLE */

/* Default Values for Registry */
/* Route table size hint */
#define IX_CC_IPV6_ROUTE_TABLE_SIZE_HINT 256 

/* Route table type: TCAM or Software */
#define IX_CC_IPV6_RTM_LOOKUP_TYPE IX_CC_RTMV6_LKUP_SOFTWARE 

/* Look up memory type SRAM or SDRAM */
#define IX_CC_IPV6_RTM_LOOKUP_MEMORY_TYPE IX_CC_RTMV6_MEM_SRAM  

#ifdef SPLIT_V6_TRIE_TABLE
/* RTM look up SRAM/DRAM Channel number for 16-bit trie table if it is split */
#define IX_CC_IPV6_RTM_LOOKUP_CHANNEL_16BIT_TRIE 2
#endif /* SPLIT_V6_TRIE_TABLE */

/* Next hop data base table size hint */
#define IX_CC_IPV6_NEXT_HOP_TABLE_SIZE_HINT 256 

/* next hop data base memory type SRAM or SDRAM */
#define IX_CC_IPV6_NEXT_HOP_MEMORY_TYPE IX_CC_RTMV6_MEM_SRAM 

#if (_IX_HARDWARE_TYPE_ == _IX_HW_2800_)

#define IX_CC_IPV6_NUMBER_OF_MICROENGINES 16 
#define IX_CC_IPV6_ME 0x3007  /* ME# 0, 1, 2, 12, 13 */
/* RTM look up SRAM/DRAM Channel number */
#define IX_CC_IPV6_RTM_LOOKUP_CHANNEL 2 
/* Next Hop Data base SRAM/DRAM Channel number */
#define IX_CC_IPV6_NEXT_HOP_CHANNEL 2  

#else /* (_IX_HARDWARE_TYPE_ != _IX_HW_2800_) */

/* number of microengines: 8 for IXP2400 */
#define IX_CC_IPV6_NUMBER_OF_MICROENGINES 8 
/* Micro engine allocated for IPv6 are 0x01,0x02,0x11,0x12 */
#define IX_CC_IPV6_ME 0x66 
/* RTM look up SRAM/DRAM Channel number */
#define IX_CC_IPV6_RTM_LOOKUP_CHANNEL 1 
/* Next Hop Data base SRAM/DRAM Channel number */
#define IX_CC_IPV6_NEXT_HOP_CHANNEL 0  

#endif /* (_IX_HARDWARE_TYPE_ == _IX_HW_2800_) */


/* Maximum size of symbol name */
#define IX_CC_IPV6_SYMBOL_SIZE 128 

/* Other defines */
#define IX_CC_IPV6_SRAM_CHANNEL 1

/* incoming counters maintained by the microblock for which we need
 * to maintain 64-bit "shadow" counters in the core components
 */
#define IX_CC_IPV6_NUMBER_OF_IN_MB_COUNTERS   2
#define IX_CC_IPV6_TOTAL_NUMBER_OF_IN_MB_COUNTERS  6

/* outgoing counters maintained by the microblock for which we need
 * to maintain 64-bit "shadow" counters in the core components
 */
#define IX_CC_IPV6_NUMBER_OF_OUT_MB_COUNTERS  2
#define IX_CC_IPV6_TOTAL_NUMBER_OF_OUT_MB_COUNTERS  3


/* 
 * Number of Symbols to be patched
 */
#define IX_CC_IPV6_MAX_NUMBER_OF_SYMBOLS 10   

#define IX_CC_IPV6_COUNTER_OVERFLOW_TIME 13 
                         /* counter overflow time for 64 bit counter create */

#define IX_CC_IPV6_MILLISECONDS_PER_SECOND 1000 /* millseconds */

#define IX_CC_IPV6_ICMP_EVENT_PRIORITY  1


/* 
 * Symbol names (strings) to be patched. These define the values
 * of various symbols used by the core and the microblock
 */

/* symbol name for statistics table */
#define IX_CC_IPV6_PATCHNAME_STATS_TABLE  "IPV6_STATS_TABLE_SRAM_BASE" 

/* symbol name for RTMv6 Trie table base */
#define IX_CC_IPV6_PATCHNAME_TRIE_TABLE  "IPV6_TRIE_TABLE_SRAM_BASE" 

/* symbol name for RTM next hop table */
#define IX_CC_IPV6_PATCHNAME_NEXTHOP_TABLE "IPV6_L3_NEXTHOP_TABLE_SRAM_BASE" 

/* symbol name for the router prefix sram base */
#define IX_CC_IPV6_PATCHNAME_ROUTER_PREFIX "IPV6_ROUTER_PREFIX_SRAM_BASE"

/* symbol name for IPv6 Forwarder blade ID */
#define IX_CC_IPV6_PATCHNAME_ID   "THIS_BLADE_ID" 

#if (_IX_HARDWARE_TYPE_ == _IX_HW_2800_)
#define IX_CC_IPV6_PATCHNAME_TRIE_TABLE_8  "IPV6_TRIE_TABLE_SRAM_BASE_8" 
#define IX_CC_IPV6_PATCHNAME_TRIE_TABLE_16 "IPV6_TRIE_TABLE_SRAM_BASE_16" 
#endif /* (_IX_HARDWARE_TYPE_ == _IX_HW_2800_) */


/* Macros to read and write memory */
#define IX_IPV6_MEM_UINT32_READ(pAddr) (*pAddr);

#define IX_IPV6_MEM_UINT32_WRITE(pAddr, data) (*pAddr = data);


/**
 * TYPENAME: ix_cc_ipv6_prop_data
 * 
 * DESCRIPTION: This structure defines format for dynamic property data.
 *
 */

typedef struct ix_s_cc_ipv6_prop_data
{
    ix_uint8   ipv6_num_elements;
    ix_uint128 ipv6_address[PROP_MAX_IPV6_ADDRESSES];  
                                               /* In Network Byte Order */
} ix_cc_ipv6_prop_data;

/**
 * TYPENAME: ix_cc_ipv6_mb_ctrl
 * 
 * DESCRIPTION: This structure defines format for microblock control data 
 * structure
 */

typedef struct ix_s_cc_ipv6_mb_ctrl
{
    ix_uint32 portForward;
    ix_uint32 reserved1;
    ix_uint32 reserved2;
    ix_uint32 reserved3;
} ix_cc_ipv6_mb_ctrl;

/* size of microblock control block */
#define IX_CC_IPV6_SIZE_OF_CONTROL_BLOCK  sizeof(ix_cc_ipv6_mb_ctrl)

#define IX_CC_IPV6_SIZE_OF_ROUTER_PREFIX  10 * sizeof(ix_uint32)

/**
 * TYPENAME: ix_cc_in_ipv6_fwd_stats_per_port
 * 
 * DESCRIPTION: This structure defines the format of the IPv6 statistics
 * maintained by the core component. The incoming statistics are maintained
 * for every port on THIS_BLADE_ID.
 *
 * NOTE: Some counters are maintained by both the IPv6 core and the microblock
 * and are listed here.
 *
 */
typedef struct ix_s_cc_in_ipv6_fwd_stats_per_port
{
    /* 
     * The following counters are maintained both by the core and 
     * the IPv6 microblock 
     */
    ix_uint32 ipv6CoreInHdrErrors;          /* number of packets with bad 
                                             * headers 
					     */
    ix_uint32 ipv6CoreInAddrErrors;         /* number of packets with bad 
                                             * destination address 
					     */
    ix_uint32 ipv6CoreInTruncatedPkts;      /* number of truncated packets */

    ix_uint32 ipv6CoreInForwDgrms;          /* number of packet for which 
                                             * forwarding was attempted 
					     */
    ix_uint64 ipv6CoreInRecv;               /* number of packets recd */ 
    ix_uint64 ipv6CoreInOctets;             /* number of octets recd */ 

    /* The following counters are maintained by the core only */
    ix_uint32 ipv6CoreInNoRoutes;	/* packets with no route to dest */
    ix_uint32 ipv6CoreInUnknownProtos;  /* packets with unknown protocol */
    ix_uint32 ipv6CoreInDiscards;	/* packets discarded due to 
                                         * other problems 
					 */
    ix_uint32 ipv6CoreInDelivers;	 /* packets delivered successfully */
    ix_uint64 ipv6CoreInMulticast;	 /* packet of multicast IP recd */
    ix_uint64 ipv6CoreInMulticastOctets; /* number of bytes recd in multicast
                                          * packets 
					  */
} ix_cc_in_ipv6_fwd_stats_per_port;

/**
 * TYPENAME: ix_cc_out_ipv6_fwd_stats_per_port
 * 
 * DESCRIPTION: This structure defines the format for IPV6 statistics 
 * maintained by the core only.
 *
 * NOTE: Some counters are maintained by both the IPv6 core and the microblock
 * and are listed here.
 *
 */
typedef struct ix_s_cc_out_ipv6_fwd_stats_per_port
{
    /* 
     * Out going interface statistics, maintained for all ports on 
     * all blades
     */

    ix_uint32 ipv6CoreOutForwDgrms;     /* Number of pkts for which a route 
                                         * was found 
					 */
    ix_uint32 ipv6CoreOutDiscards;      /* packets dropped due to other 
                                         * other problems
					 */
    ix_uint64 ipv6CoreOutTransmits;     /* packets sent to the lower layers
                                         * for transmission.
					 */
    ix_uint64 ipv6CoreOutOctets;        /* Octetes sent to the lower layers
                                         * for transmission.
					 */
    ix_uint64 ipv6CoreOutMulticast;       /* number of multicast IP tx'ed */
    ix_uint64 ipv6CoreOutMulticastOctets; /* number of bytes tx'ed in 
                                           * multicast packets 
					   */
} ix_cc_out_ipv6_fwd_stats_per_port;


/**
 * TYPENAME: ix_cc_ipv6_fwd_stats
 * 
 * DESCRIPTION: This structure defines the format for IPV6 statistics 
 * maintained by the core only.
 *
 * NOTE: Some counters are maintained by both the IPv6 core and the microblock
 * and are listed here.
 *
 */
typedef struct ix_s_cc_ipv6_fwd_stats
{
    /* Incoming stats are maintained for all ports on THIS_BLADE */
    ix_cc_in_ipv6_fwd_stats_per_port ipv6InCore[MAX_NUMBER_OF_PORTS_PER_BLADE];

    /* Outgoing stats are maintained for all ports on every blade.
     * To access the stats for port "p" on blade id "b", the index
     * in this array is:
     * index = (MAX_NUMBER_OF_PORTS_PER_BLADE * b) + p
     */
    ix_cc_out_ipv6_fwd_stats_per_port ipv6OutCore[IPV6_TOTAL_NUMBER_OF_PORTS];

    /* These stats are not interface specific */
    ix_uint32 ipv6CoreOutRequests;      /* packets sent down from protocols
                                         * running on this machine
					 */
    
    ix_uint32 ipv6CoreOutNoRoutes;      /* packets dropped- no routes no dest */
} ix_cc_ipv6_fwd_stats;

/**
 * TYPENAME: ix_cc_in_64bit_handles
 * 
 * DESCRIPTION: This structure defines the array of handles for 64-bit
 * incoming counters maintained by the core component.
 * 
 * NOTE: This is a count of handles per port on a blade.
 *
 */
typedef struct ix_s_cc_in_64bit_handles
{
    /* 64-bit counter handles */
    ix_counter_64bit_handle handles[IX_CC_IPV6_NUMBER_OF_IN_MB_COUNTERS]; 

} ix_cc_in_64bit_handles;

/**
 * TYPENAME: ix_cc_out_64bit_handles
 * 
 * DESCRIPTION: This structure defines the array of handles for 64-bit
 * outgoing counters maintained by the core component.
 * 
 * NOTE: This is a count of handles per port on a blade.
 *
 */
typedef struct ix_s_cc_out_64bit_handles
{
    /* 64-bit counter handles */
    ix_counter_64bit_handle handles[IX_CC_IPV6_NUMBER_OF_OUT_MB_COUNTERS]; 

} ix_cc_out_64bit_handles;

/**
 * TYPENAME: ix_cc_ipv6_context
 * 
 * DESCRIPTION: This structure defines format for IPv6 Forwarder 
 * core component context
 *
 */
typedef struct ix_s_cc_ipv6_context
{
    ix_cc_handle hIpv6Handle;   /* Handle of IPv6 Forwarder core component */
    ix_cc_rtmv6 hRtm;           /* RTMv6 handle  */

    /* 64-bit counter handles */

    /* Incoming counter handles */
    ix_cc_in_64bit_handles 
             inCounter64BitHandle[MAX_NUMBER_OF_PORTS_PER_BLADE];

    /* Outgoing handles */
    ix_cc_out_64bit_handles
             outCounter64BitHandle[IPV6_TOTAL_NUMBER_OF_PORTS];

    ix_cc_ipv6_mb_ctrl *pMbCbBase; /* Base address of IPv6 control block */

    ix_uint32 *pRouterPrefix;   /* Base address of the IPv6 Router Prefix */

#if IX_EXCLUDE_CCI
    ix_ossl_thread_t icmpThreadId; /* ICMP thread id - this is for level0 support */
#else
    ix_event_handle hEventHandle; /* Event handler handle  */
#endif

    ix_ossl_mutex_t ipv6Mutex; /* mutex id */
    ix_uint32* pMbBase;        /* base address of micro block and IPv6 
                                * shared data 
				*/
    ix_uint32* pMbStatsBase;   /* statistics base address created in SRAM 
                                * for microblock patching 
				*/

    ix_buffer_free_list_handle hBufferFreeList; /* handle of free list buffer */

    ix_cc_ipv6_prop_data ipv6PropTable[IPV6_TOTAL_NUMBER_OF_PORTS];

    ix_cc_ipv6_fwd_stats ipv6FwdStats; /* IPv6 core statistics */

} ix_cc_ipv6_context;


/**
 * TYPENAME: ix_cc_ipv6_registry_data
 * 
 * DESCRIPTION: This structure defines format for registry data.
 *
 */

typedef struct ix_s_cc_ipv6_registry_data
{
    ix_cc_rtmv6_config rtmConfig;    /* RTMv6 configuration information */
    ix_uint32 localBladeId;          /* id of our blade */
    ix_uint32 meNumbers;             /* Me numbers for IPv6 forwarder */
} ix_cc_ipv6_registry_data;


extern ix_cc_ipv6_context* g_pIpv6Context;

/**
 * NAME: _ix_cc_ipv6_retrieve_registry_data
 *
 * DESCRIPTION: This primitive will be used to retreive data from registry 
 * for a given property type. If there is no registry, then the data structure 
 * will be filled with data from header file.
 * 
 * @Param:  - IN arg_pPropertyName - pointer to property name string 
 *
 * @Param:  - OUT arg_pPropertyValue - pointer to property value 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_REGISTRY  - failed to access registry
 * 
 */

ix_error _ix_cc_ipv6_retrieve_registry_data(const char* arg_pPropertyName,
                                            ix_uint32* arg_pPropertyValue);


/**
 * NAME: _ix_cc_ipv6_registry
 *
 * DESCRIPTION: This primitive will be used to retreive data from registry for
 * all IPV6 static data and stored in ipv4RegistyData 
 *           .
 * 
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_REGISTRY  - not able to access registry.
 * 
 */

ix_error _ix_cc_ipv6_registry(void);

/**
 * NAME: _ix_cc_ipv6_alloc_symbols
 *
 * DESCRIPTION: This primitive will be used to alloc memory for symbols 
 * to be patched. 
 * 
 *           .
 * 
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOM  - not able to allocate memory
 * 
 */

ix_error _ix_cc_ipv6_alloc_symbols(void);


/**
 * NAME: _ix_cc_ipv6_patch_symbols
 *
 * DESCRIPTION: This primitive will be used to patch symbols to ipv6 
 * microblocks. 
 * 
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * IX_CC_IPV6_ERROR_RTM - error from RTM module
 * IX_CC_IPV6_ERROR_FAILED_PATCHING - failed patching by calling resource 
 * manager function
 * 
 */
ix_error _ix_cc_ipv6_patch_symbols(void);


/**
 * NAME: _ix_cc_ipv6_create_64bit_counters
 *
 * DESCRIPTION: This primitive will be used to create 64 bit counters.
 * 
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * IX_CC_ERROR_OOM_64BIT_COUNTER - error in creating 64 bit counter
 * 
 * 
 */

ix_error _ix_cc_ipv6_create_64bit_counters(void);


/**
 * NAME: _ix_cc_ipv6_add_reserved_next_hops
 *
 * DESCRIPTION: This primitive will be used to add reserved next hops into RTM.
 * 
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * IX_CC_IPV6_ERROR_RTM - error from RTM
 * 
 */

ix_error _ix_cc_ipv6_add_reserved_next_hops(void);

/**
 * NAME: _ix_cc_ipv6_add_reserved_prefixes
 *
 * DESCRIPTION: This primitive will be used to add reserved prefixes to the RTM.
 * 
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * IX_CC_IPV6_ERROR_RTM - error from RTM
 * 
 */

ix_error _ix_cc_ipv6_add_reserved_prefixes(void);

/**
 * NAME: _ix_cc_ipv6_add_invalid_routes
 *
 * DESCRIPTION: This primitive will be used to add martian addresses to RTM.
 * 
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * IX_CC_IPV6_ERROR_RTM - error from RTM function call
 * 
 * 
 */

ix_error _ix_cc_ipv6_add_invalid_routes(void);



/**
 * NAME: _ix_cc_ipv6_configure_rtm
 *
 * DESCRIPTION: This primitive will be used to configure RTM.
 * 
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_RTM - error from RTM function
 * 
 */

ix_error _ix_cc_ipv6_configure_rtm(void);


/**
 * NAME: _ix_cc_ipv6_register_message_handler
 *
 * DESCRIPTION: This primitive will be used to register message handler. 
 *
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_CCI - Error from CCI
 * 
 */

ix_error _ix_cc_ipv6_register_message_handler(void);


/**
 * NAME: _ix_cc_ipv6_register_cci_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to register packet handler with CCI.
 *
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_CCI - error from CCI
 * 
 */

ix_error _ix_cc_ipv6_register_cci_packet_handlers(void);


/**
 * NAME: _ix_cc_ipv6_register_rm_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to register packet handler with 
 * resource manager. This will be used in level 0 system
 *
 *
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_CCI - error from resource manager
 * 
 */

ix_error _ix_cc_ipv6_register_rm_packet_handlers(void);


/**
 * NAME: _ix_cc_ipv6_register_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to register packet handlers. 
 * 
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_CCI - error from CCI
 * 
 */

ix_error _ix_cc_ipv6_register_packet_handlers(void);


/**
 * NAME: _ix_cc_ipv6_add_icmp_event_handler
 *
 * DESCRIPTION: This primitive will be used to add icmp event handler. 
 * This is useful to drain ICMP packets priodically from queue.
 * 
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_CCI - error from event handler
 * 
 */

ix_error _ix_cc_ipv6_add_icmp_event_handler(void);


/**
 * NAME: _ix_cc_ipv6_create_event_handler
 *
 * DESCRIPTION: This primitive is used to create event handler if there is no
 * CCI. Users need to call _ix_cc_ipv4_icmp_send_message function periodically
 * to drain ICMP packets from queue.
 * 
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *  
 * IX_CC_IPV6_ERROR_CCI - error to create event handler
 * 
 */

ix_error _ix_cc_ipv6_create_event_handler(void* arg_pContext,
                                          ix_ossl_thread_t* arg_pEventId);


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_IPV6_INIT_H__) */
