/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = FILENAME
 *      ix_cc_ipv6_nd.h
 *
 * = DESCRIPTION
 *      This file defines default values for configuration variables required
 *      for neighbor discovery. It also contains data definitions and function
 *      prototypes for neighbor discovery.
 *
 * = CHANGE HISTORY
 *       11/28/2002 1:53:42 PM - creation time
 *
 * ============================================================================
 * $Id: ix_cc_ipv6_nd.h,v 1.8 2003/11/01 02:29:56 rranjeet Exp $
 */
#if !defined(__NBR_DISCOVERY_H)
#define __NBR_DISCOVERY_H

#include "ix_types.h"
#include "ix_cc_ipv6_init.h"
#include "v6_header_defs.h"

/* SSD_TBC: Remove this once slist change is done */
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
#include "ix_cc_ipv6_slist.h"
#else
#include "cc/ix_cc_slist.h"
#endif
#include "ix_ossl.h"

#if 0
#if !defined(IX_EXCLUDE_CCI)
#include "cci/cci_types.h" 
#endif /* IX_EXCLUDE_CCI */
#endif

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

/****Router constants****/
#define IX_CC_ND_MAX_INITIAL_RTR_ADVERT_INTERVAL    16 /*seconds*/
#define IX_CC_ND_MAX_INITIAL_RTR_ADVERTISEMENTS     3  /*transmissions*/
#define IX_CC_ND_MAX_FINAL_RTR_ADVERTISEMENTS       3  /*transmissions*/
#define IX_CC_ND_MIN_DELAY_BETWEEN_RAS              3  /*seconds*/
#define IX_CC_ND_MAX_RA_DELAY_TIME                  500 /*millisecs*/

/*****Host Constants*****/
#define IX_CC_ND_MAX_RTR_SOL_DELAY                  1 /*second*/

/****Node Constants****/
#define IX_CC_ND_MAX_MULTICAST_SOLICIT              3   /*transmissions*/
#define IX_CC_ND_MAX_UNICAST_SOLICIT                3   /*transmissions*/
#define IX_CC_ND_MAX_ANYCAST_DELAY_TIME             1   /*second*/
#define IX_CC_ND_MAX_NEIGHBOR_ADVERTISEMENT         3   /*transmissions*/
#define IX_CC_ND_REACHABLE_TIME                     30000 /*millisecs*/
#define IX_CC_ND_RETRANS_TIMER                      1000  /*millisecs*/
#define IX_CC_ND_DELAY_FIRST_PROBE_TIME             5   /*seconds*/
#define IX_CC_ND_MIN_RANDOM_FACTOR                  0.5
#define IX_CC_ND_MAX_RANDOM_FACTOR                  1.5

/***Default values for the Router configuration variables**/
#define IX_CC_IPV6_ND_SEND_ADVS         0
#define IX_CC_IPV6_ND_MAX_RTR_INTERVAL  600
#define IX_CC_IPV6_ND_MIN_RTR_INTERVAL  198 /*0.33 * MAX_RTR_INTERVAL*/
#define IX_CC_IPV6_ND_MANAGED           0
#define IX_CC_IPV6_ND_OTHER_CONFIG      0
#define IX_CC_IPV6_ND_LINK_MTU          0
#define IX_CC_IPV6_ND_REACHABLE_TIME    0
#define IX_CC_IPV6_ND_RETRANS_TIMER     0
#define IX_CC_IPV6_ND_HOP_LIMIT         255 /*Couldn't find IANA value*/
#define IX_CC_IPV6_ND_DEFAULT_LIFETIME  600 /* either 0 or between 
                                             * MAX_RTR_INTERVAL and 9000*/

#define IX_CC_IPV6_ND_ALL_NODES_ADDR "ff02:0000:0000:0000:0000:0000:0000:0001"
#define IX_CC_IPV6_ND_ALL_RTRS_ADDR "ff02:0000:0000:0000:0000:0000:0000:0002"

#define IX_CC_IPV6_ETH_HDR_LEN 14
#define IX_CC_IPV6_ETH_HDR_PROTO_V6 0x86DD

/**
 * TYPENAME: ix_ipv6_nd_reachability_state
 *
 * DESCRIPTION:
 *          This enumeration contains the various reachability states
 *          for a neighbor.
 *
 */
typedef enum ix_e_ipv6_nd_reachability_state
{
    IX_REACH_TYPE_FIRST = 0,
    IX_REACH_TYPE_INCOMPLETE = IX_REACH_TYPE_FIRST,
    IX_REACH_TYPE_REACHABLE,
    IX_REACH_TYPE_STALE,
    IX_REACH_TYPE_DELAY,
    IX_REACH_TYPE_PROBE,
    IX_REACH_TYPE_LAST
}ix_ipv6_nd_reachability_state;

/**
 * TYPENAME: ix_ipv6_nd_nbr_info
 * 
 * DESCRIPTION: 
 *          This structure is used to store information about a neighbor.
 *          This structure is in turn stored in the neighbor cache.
 *
 */
typedef struct ix_s_ipv6_nd_nbr_info
{
    ix_uint128      ipAddr;
    ix_uint16       outputPort;
    ix_uint8        isValidL2Addr;
    ix_uint8        l2Addr[IX_CC_IPV6_MAC_ADDR_LEN];
    ix_uint8        isRouter;
    ix_ipv6_nd_reachability_state   state;
    ix_uint32       numUnansProbes; /*No of unanswered probes*/
    ix_ossl_time_t  nextNUDEvent;   /*Next unreachability detection event*/

    LPLIST          packetQueue;  /*list of ix_ipv6_nd_pkt_queue_struct*/
    ix_uint8        nbrSolEventScheduled; /*flag.*/

#if !defined(IX_EXCLUDE_CCI)
    ix_event_handle hNbrSolEventHdl;
#endif /* IX_EXCLUDE_CCI */

    ix_uint32       l2Id;
}ix_ipv6_nd_nbr_info;
                  
/**
 * TYPENAME: ix_ipv6_nd_pkt_queue_struct
 *
 * DESCRIPTION:
 *      This structure contains a buffer handle and a context to store
 *      the handle of a packet that needs address resolution. It also stores
 *      the context of the client request the address resolution.
 *      Note that in case the caller is not the ethernet TX cc (which sends
 *      packets to us), the buffer handle will be set to IX_NULL_BUFFER_HANDLE
 *      and only the context will be valid. This means that after address 
 *      resolution, we only need to send a message to the caller, and there's
 *      no need to forward a packet
 */
typedef struct ix_s_ipv6_nd_pkt_queue_struct
{
    ix_buffer_handle    hBuffer;
    void                *pContext;
}ix_ipv6_nd_pkt_queue_struct;

/**
 * TYPENAME: ix_ipv6_nd_rtr_adv_params
 *
 * DESCRIPTION:
 *      This structure contains all the parameters required to construct
 *      a router advertisement message.
 *      This structure will be populated and passed in to a function to 
 *      construct the router advertisement message. The resultant message
 *      will be in the format specified in the ix_ipv6_nd_rtr_adv_msg 
 *      structure.
 */
typedef struct ix_s_ipv6_nd_rtr_adv_params
{
    ix_uint16   maxRtrAdvInterval;
    ix_uint16   minRtrAdvInterval;
    
    ix_uint8    nextHopLmit;
    ix_uint8    managed;
    ix_uint8    otherConfig;
    ix_uint16   rtrLifetime;
    ix_uint32   reachableTime;
    ix_uint32   retransmitTimer;

    ix_uint8    l2AddrLength; /*the length of the l2 address in bytes*/
    ix_uint8    l2Addr[IX_CC_IPV6_MAC_ADDR_LEN];

    ix_uint16   mtu;
    /*Placeholder for prefix info*/    

} ix_ipv6_nd_rtr_adv_params;

/**
 * TYPENAME: ix_ipv6_nd_port
 *
 * DESCRIPTION:
 *          Contains all parameters related to a particular port on the FE.
 *
 */
typedef struct ix_s_ipv6_nd_port
{
    ix_uint32   portId;
    ix_cc_properties *pPortProperties;

    ix_uint8        sendRtrAdv;
    ix_uint8        prevSendAdv;    /*Used for sending the last ADV if rtr stops
                                  advertising*/
 
    ix_ipv6_nd_rtr_adv_params rtrAdv;

#if !defined(IX_EXCLUDE_CCI)
    ix_event_handle hRtrAdvHandle;
#endif /* IX_EXCLUDE_CCI */

    ix_ossl_time_t  lastAdvTime;
    ix_ossl_time_t  nextAdvTime;

    LPLIST          ipAddrList; /*List of ix_ipv6_addr_conf_ip_struct*/
}ix_ipv6_nd_port;

/**
 * TYPENAME: ix_ipv6_addr_conf_addr_state
 *
 * DESCRIPTION:
 *      This enumeration describes the various states that an IP address
 *      assigned to an interface can be in.
 */
typedef enum ix_e_ipv6_addr_conf_addr_state
{
    IX_ADDR_RES_STATE_FIRST = 0,
    IX_ADDR_RES_STATE_TENTATIVE = IX_ADDR_RES_STATE_FIRST,
    IX_ADDR_RES_STATE_PREFERRED,
    IX_ADDR_RES_STATE_DEPRECATED,
    IX_ADDR_RES_STATE_VALID,
    IX_ADDR_RES_STATE_INVALID,
    IX_ADDR_RES_STATE_LAST
}ix_ipv6_addr_conf_addr_state;

/**
 * TYPENAME: ix_ipv6_addr_conf_ip_struct
 *
 * DESCRIPTION:
 *      This structure stores an IP address assigned to an interface along 
 *      with related information like its state. A list of such structures,
 *      one for each IP address, is maintained for each port
 *
 */
typedef struct ix_s_ipv6_addr_conf_ip_struct
{
    ix_uint128  ipAddr;
    ix_uint8    linkLayerAddr; /*flag identifies link layer address*/
    ix_ipv6_addr_conf_addr_state state;
    ix_uint32   numDadTransmits;    /*Number of Nbr Sol transmits remaining*/   

#if !defined(IX_EXCLUDE_CCI)
    ix_event_handle hDadHandle;
#endif /* IX_EXCLUDE_CCI */

}ix_ipv6_addr_conf_ip_struct;

/**
 * TYPENAME: ix_ipv6_addr_conf_context
 *
 * DESCRIPTION:
 *      This structure contains the port ID and pointer to the IP address 
 *      structure stored in the ipAddrList. It is used as the context
 *      while setting event timers for Duplicate Address Detection.
 */
typedef struct ix_s_ipv6_addr_conf_context
{
    ix_uint32   portId;
    ix_ipv6_addr_conf_ip_struct *pIpStruct;
}ix_ipv6_addr_conf_context;

/**
 * TYPENAME: ix_ipv6_eth_hdr
 * DESCRIPTION: The ethernet header
 */
typedef struct ix_s_ipv6_eth_hdr
{
    ix_uint8    dstAddr[IX_CC_IPV6_MAC_ADDR_LEN];      /* Ethernet destination address */
    ix_uint8    srcAddr[IX_CC_IPV6_MAC_ADDR_LEN];      /* Ethernet source address */
    ix_uint16   protocol;      /* packet type ID field */    
}__attribute__ ((packed)) ix_ipv6_eth_hdr;

/***********
 * EXTERNAL API
 */
ix_error ix_cc_nd_init(ix_cc_properties_node *properties,
                       ix_cc_ipv6_context *arg_pIpv6Context);
ix_error ix_cc_nd_fini(void);
ix_error ix_cc_nd_process_incoming_packet(ix_buffer_handle arg_hBuffer);
ix_error ix_cc_nd_perform_addr_resolution(ix_buffer_handle arg_hBuffer,
                                          void *arg_pContext);
ix_error ix_cc_nd_add_neighbor(
                            ix_uint16 arg_L2Index,
                            ix_uint128 arg_NbrIp,
                            ix_uint16 arg_OutputPort,
                            ix_uint8* arg_L2Addr,
                            void* arg_pUserContext);
ix_error ix_cc_nd_del_neighbor(
                            ix_uint16 arg_L2Index,
                            void* arg_pUserContext);

ix_error ix_cc_addr_conf_handle_dad_event(void *pContext);

ix_error testNbrAdd(void);

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__NBR_DISCOVERY_H) */

