/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor, Release 5
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      Core Component Infrastructure - Dynamic Property Table management
 *
 * = FILENAME
 *      ix_cc_ipv4_prop.h
 *
 * = DESCRIPTION
 *      This file defines all data types and function prototypes required 
 *      by the IPv6 dynamic property table management module.
 *
 * = AUTHOR
 *       Ravi Murty
 *       ravi.murty@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      12/03/2002 10:30 AM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */

#if !defined(__IX_CC_IPV6_PROP_H__)
#define __IX_CC_IPV6_PROP_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */


/**
 * NAME: _ix_cc_ipv6_get_port_ip_address
 *
 * DESCRIPTION: This primitive will be used to get an array of IPv6 address 
 *              of a port. The maximum number of addresses returned
 *              is defined by PROP_MAX_IPV6_ADDRESSES
 * 
 * @Param:  - IN arg_BladeId - blade number of ip address to be retrieved
 * @Param:  - IN arg_PortId - port number of ip address to be retrieved
 *
 * @Param:  - OUT arg_pIpAddress - Array of IPv6 address of given blade 
 *                                and port
 * @Param:  - OUT arg_pNumElements - Number of elements in the array
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_INVALID_INPUT_PARAM - invalid blade/port number
 * IX_CC_ERROR_ENTRY_NOT_FOUND - no data in location
 */
ix_error _ix_cc_ipv6_get_port_ip_address(ix_uint16 arg_BladeId,
                                         ix_uint16 arg_PortId, 
					 ix_uint8 *arg_pNumElements,
                                         ix_uint128 *arg_pIpAddr);

/**
 * NAME: _ix_cc_ipv6_prop_get_port_status
 *
 * DESCRIPTION: This primitive will be used to get status of local port
 * 
 * @Param:  - IN arg_PortId - port number of status to be retrieved
 *
 * @Param:  - OUT arg_pPortStatus - status value of given port
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_INVALID_INPUT_PARAM - invalid port number
 * 
 */
ix_error _ix_cc_ipv6_get_port_status(ix_uint16 arg_PortId, 
                                     ix_uint32* arg_pPortStatus);


/**
 * NAME: _ix_cc_ipv6_property_update_port_status
 *
 * DESCRIPTION: This primitive will be used to set status of local port
 * 
 * @Param:  - IN arg_PortId - port number 
 *
 * @Param:  - OUT arg_pPortStatus - status value of given port
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_INVALID_INPUT_PARAM - invalid port number
 * 
 */

ix_error _ix_cc_ipv6_property_update_port_status(ix_uint16 arg_PortId, 
                                                 ix_uint32 arg_PortStatus);



/**
 * NAME: _ix_cc_ipv6_property_update_ip_address
 *
 * DESCRIPTION: This primitive will be used to add/delete ip address and 
 * net mask in dynamic property table and directed broadcast table
 * 
 * @Param:  - IN arg_PropId - property id : add/delete ip address
 * @Param:  - IN arg_BladeId - blade number
 * @Param:  - IN arg_PordId - port number
 * @Param:  - IN arg_Ipv6Addr - IPv6 address of port
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_INVALID_INPUT_PARAM - invalid blade/port number or invalid
 *                                        ipv6 address if it is a delete 
 *                                        operation.
 */
ix_error _ix_cc_ipv6_property_update_ip_address(ix_uint16 arg_PropId,
                                                ix_uint16 arg_BladeId,
                                                ix_uint16 arg_PortId,
                                                ix_uint128 arg_Ipv6Addr);


/**
 * NAME: _ix_cc_ipv6_prop_table_lookup_ip_address
 *
 * DESCRIPTION: This primitive will be used to lookup ipv6 address
 * in dynamic property table
 *           
 * @Param:  - IN arg_IpAddr - ipv6 address(non zero) to be searched in table.
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_ENTRY_NOT_FOUND - no entry in table 
 * 
 */

ix_error _ix_cc_ipv6_prop_table_lookup_ip_address(ix_uint128 arg_IpAddr);


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_IPV6_PROP_H__) */
