/**************************************************************************** 
//             INTEL CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or 
//  nondisclosure agreement with Intel Corporation and may not be copied 
//  or disclosed except in accordance with the terms of that agreement. 
//  Copyright (c) 2000-2001 Intel Corporation. All Rights Reserved.
//
//
//  $Workfile:   slist.h  $
//  $Revision: 1.2 $
//  $Date: 2003/09/10 01:38:35 $
//
//  Purpose:    Provides prototypes for all the methods used in slist.c
//
****************************************************************************/

/* SSD_TBC: Remove this file after SLIST fix is done */

#if !defined SLIST_HDR
#define SLIST_HDR

#include "ix_types.h"

#ifdef __cplusplus
extern "C" {
#endif

#ifndef TRUE
#define TRUE    1
#endif

#ifndef FALSE
#define FALSE   0
#endif

/* this is an item in the list and stores pointers to data, next and
   previous list items */
typedef struct tag_LISTITEM
{
    void *      data;
    struct tag_LISTITEM *  next;      
    struct tag_LISTITEM *  prev;      
} LISTITEM, * LPLISTITEM;

typedef const LISTITEM * LPCLISTITEM;
    

/* this is created when Createlist is called and stores the head and tail
   of the list. */
typedef struct tag_LIST
{
    LPLISTITEM  head;
    LPLISTITEM  tail;
    short       count;
} SLIST, * LPLIST;

typedef const SLIST * LPCLIST;
    

/* this is created whenever an iterator is needed for iterating the list */
typedef struct tag_LISTITR
{
    LPLIST      list;
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    LPLISTITEM  current_linux;
#else
    LPLISTITEM  current;
#endif
} LISTITR, * LPLISTITR;

typedef const LISTITR * LPCLISTITR;
    

/* Function prototypes */

/* List operations */
extern LPLIST ListCreate(void);
extern ix_uint8 ListAddItem(LPLIST list, void * data);
extern ix_uint8 ListRemoveItem(LPLIST list, void * data);
extern ix_uint8 ListFindItem(LPLIST list, void * data);
extern void * ListGetFirst(LPLIST list);
extern void * ListGetLast(LPLIST list);
extern int ListGetCount(LPLIST list);
extern void ListDelete(LPLIST list);
extern void ListFreeDelete(LPLIST list);


/* List Iterator operations */
extern LPLISTITR ListItrCreate(LPLIST list);
extern LPLISTITR ListItrFirst(LPLIST list, LPLISTITR listItr);
extern void * ListItrGetNext(LPLISTITR listItr);
extern ix_uint8 ListItrInsertAfter(LPLISTITR listItr, void * data);
extern ix_uint8 ListItrInsertBefore(LPLISTITR listItr, void * data);
extern ix_uint8 ListItrRemoveItem(LPLISTITR listItr);
extern void ListItrDelete(LPLISTITR listItr);

#ifdef __cplusplus
}
#endif


#endif /* SLIST_HDR */

