/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      IPv6 Forwarder Core Component
 *
 * = FILENAME
 *      v6_header_defs.h
 *
 * = DESCRIPTION
 *   This file contains definition of IPv6 header structure   
 *
 *
 * = CREATION TIME
 *      12/13/2002
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */

#if !defined(__V6HEADER_DEFS_H__)
#define __V6HEADER_DEFS_H__

/**
 * Preprocessor symbol and macro definitions.
 */
#include "ix_types.h"
#include <stdio.h>
#include <ix_ossl.h>
#include <ix_rm.h>
#include <ix_cc.h>
#include <ix_netmacros.h>
#include "ix_cc_error.h"
#include "ix_cc_properties.h"
#include "bindings.h"


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

#define IX_CC_IPV6_ARRAYSIZE    256

#define IX_CC_IPV6_SUCCESS 1
#define IX_CC_IPV6_FAIL 0

/**
 * TYPENAME: ix_ipv6_header
 * 
 * DESCRIPTION: This struct defines the format of IPv6 header 
 *
 */

#define IX_CC_IPV6_HDR_LEN              40
#define IX_CC_IPV6_HOP_LIMIT_MAX        255
#define IX_CC_IPV6_ADDRESS_LENGTH_BYTES 16
#define IX_CC_IPV6_ADDRESS_LENGTH_BITS  128

#define IX_CC_IPV6_MAC_ADDR_LEN IX_CC_MAC_ADDR_LEN

/*Next header values*/
#define IX_CC_IPV6_NEXT_HDR_HOPBYHOP    0
#define IX_CC_IPV6_NEXT_HDR_ROUTING     43
#define IX_CC_IPV6_NEXT_HDR_FRAGMENT    44
#define IX_CC_IPV6_NEXT_HDR_ICMP        58 
#define IX_CC_IPV6_NEXT_HDR_NONE        59
#define IX_CC_IPV6_NEXT_HDR_DSTOPT      60

#define IX_CC_IPV6_UNSPECIFIED_ADDR "0000:0000:0000:0000:0000:0000:0000:0000"

typedef ix_uint128   ix_ipv6_addr;

/* 
 * IPv6 Header structure as per RFC 2460 
 */

typedef struct ix_s_ipv6_header {

#if IX_BYTE_ORDER == IX_LITTLE_ENDIAN 
              
    ix_uint32 flow_label:20;       /* flow label */
    ix_uint32 traffic_class:8;     /* traffic class */
    ix_uint32 version:4;           /* version, must be 6 */

#elif IX_BYTE_ORDER == IX_BIG_ENDIAN

    ix_uint32 version:4;
    ix_uint32 traffic_class:8;
    ix_uint32 flow_label:20;

#endif

    ix_uint16   payloadLength;     /* The length of the payload in bytes*/
    ix_uint8    nextHeader;        /* Identifies the type of the header
                                    * following this header
				    */
    ix_uint8    hopLimit;
    
    ix_uint128  srcAddress;      /*The 128 bit source address*/
    ix_uint128  dstAddress;      /*The 128 bit destination address*/   
} ix_cc_ipv6_header;

/**
 * TYPENAME: ix_cc_ipv6_options_hdr_common
 *
 * DESCRIPTION: Defines the common part of the IP options headers.
 *
 */
typedef struct ix_s_cc_ipv6_options_hdr_common
{
    ix_uint8    nextHeader;
    ix_uint8    hdrLength;
}ix_cc_ipv6_options_hdr_common;

/**
 * TYPENAME: ix_icmp_header
 * 
 * DESCRIPTION: This struct defines the format of ICMP header 
 *
 */
typedef struct ix_s_cc_ipv6_icmp_header
{
    ix_uint8    type;
    ix_uint8    code;
    ix_uint16   checksum;
} ix_cc_ipv6_icmp_header;

#define IX_CC_ICMPV6_HDR_LEN 4

/**
 * TYPENAME: ix_cc_ipv6_pseudo_header
 *
 * DESCRIPTION: The IPV6 pseudo header required for checksum calculations
 *
 */
typedef struct ix_s_cc_ipv6_pseudo_header
{
    ix_ipv6_addr   srcAddress;
    ix_ipv6_addr   dstAddress;
    ix_uint32      pktLength;
    ix_uint32      unused:24, nextHdr:8;
} ix_cc_ipv6_pseudo_header;

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__V6HEADER_DEFS_H__) */
