/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor, Release 5
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      IPv6 Forwarder Core Component
 *
 * = FILENAME
 *      ix_cc_ipv6.h
 *
 * = DESCRIPTION
 *      This file defines all exported data types and function prototypes
 *
 * = AUTHOR
 *       Ravi Murty
 *       ravi.murty@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      12/09/2002 
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */

#if !defined(__IX_CC_IPV6_H__)
#define __IX_CC_IPV6_H__

#include "ix_rm.h"
#include "ix_ossl.h"
#include "ix_cc.h"
#include "cc/ix_cc_rtmv6.h"
#include "ipv6_fwder_uc.h"

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

#define IX_CC_IPV6_NH_LOCAL 0xFFFFFFFE
#define IX_CC_IPV6_NH_DROP 0xFFFFFFFD
#define IX_CC_IPV6_NH_NBR_DISC 0xFFFFFFFC

#define THIS_BLADE_ID 1

/* Total number of port in the system */
#define IPV6_TOTAL_NUMBER_OF_PORTS \
                    MAX_NUMBER_OF_BLADES * MAX_NUMBER_OF_PORTS_PER_BLADE
/**
 * TYPENAME: ix_cc_in_ipv6_stats_data
 * 
 * DESCRIPTION: This structure defines the format of the statistics
 * data maintained for the incoming interfaces. Some of the counter are
 * maintained by the core and some by the microblock, the names
 * of the variables should make it easy to identify where the counters 
 * are maintained.
 *
 * Note that incoming stats are maintained for THIS_BLADE_ID only (for all
 * ports on this blade).
 */

typedef struct ix_s_cc_in_ipv6_stats_data
{
    /* Mircoblock only */
    ix_uint64  ipv6MbInReceives;
    ix_uint64  ipv6MbInOctets;
    ix_uint32  ipv6MbInForwDatagrams;
    ix_uint32  ipv6MbInAddrErrors;
    ix_uint32  ipv6MbInTruncated;

    /* Core and Microblock */
    ix_uint32  ipv6CoreMbInHdrErrors;

    /* Core only */
    ix_uint32  ipv6CoreInNoRoutes;
    ix_uint32  ipv6CoreInUnknownProtos;
    ix_uint32  ipv6CoreInDiscards;
    ix_uint32  ipv6CoreInDelivers;
    ix_uint64  ipv6CoreInMulticast;
    ix_uint64  ipv6CoreInMulticastOctets;

} ix_cc_in_ipv6_stats_data;

/**
 * TYPENAME: ix_cc_out_ipv6_stats_data
 * 
 * DESCRIPTION: This structure defines format for statistics data. These
 * stats are maintained both in the core and the microblock and the names
 * of the variables should make it easy to identify where the counters 
 * are maintained.
 *
 * Outgoing stats are maintained for outgoing interfaces on all blades.
 *
 */

typedef struct ix_s_cc_out_ipv6_stats_data
{
   /* Microblock only */
   ix_uint32  ipv6MbOutForwDatagrams;
   ix_uint64  ipv6MbOutTransmits;
   ix_uint64  ipv6MbOutOctets;

   /* Core only */
   ix_uint32  ipv6CoreOutRequests;
   ix_uint32  ipv6CoreOutNoRoutes;
   ix_uint32  ipv6CoreOutDiscards;
   ix_uint64  ipv6CoreOutMulticast;
   ix_uint64  ipv6CoreOutMulticastOctets;

} ix_cc_out_ipv6_stats_data;

/**
 * TYPENAME: ix_cc_ipv6_stats_data
 * 
 * DESCRIPTION: This structure defines format for statistics data. These
 * stats are maintained both in the core and the microblock and the names
 * of the variables should make it easy to identify where the counters 
 * are maintained.
 *
 */
typedef struct ix_s_cc_ipv6_stats_data
{
    /* Incoming stats */
    ix_cc_in_ipv6_stats_data  ipv6InStats[MAX_NUMBER_OF_PORTS_PER_BLADE];

    /* Outgoing stats.
     * IPV6_TOTAL_NUMBER_OF_PORTS =  
     *     (MAX_NUMBER_OF_PORT_PER_BLADE * MAX_NUMBER_OF_BLADES)
     */
    ix_cc_out_ipv6_stats_data ipv6OutStats[IPV6_TOTAL_NUMBER_OF_PORTS];

    /* RTMv6 Stats */
    ix_uint32  ipv6NumberOfRoutes;
    ix_uint32  ipv6NumberOfNextHops;

} ix_cc_ipv6_stats_data;


/**
 * TYPENAME: ix_cc_ipv6_messages
 * 
 * DESCRIPTION: This structure defines format for IPv6 messages
 *
 */
typedef enum ix_e_cc_ipv6_messages
{

    /* Route Messages */
    IX_CC_IPV6_MSG_ADD_PREFIX = IX_CC_COMMON_MSG_ID_LAST,

    IX_CC_IPV6_MSG_DELETE_PREFIX,
    IX_CC_IPV6_MSG_LOOKUP_PREFIX,
    IX_CC_IPV6_MSG_UPDATE_PREFIX,
    IX_CC_IPV6_MSG_PURGE_ROUTES,
    IX_CC_IPV6_MSG_DUMP_ROUTES,


    /* Next Hop Messages */
    IX_CC_IPV6_MSG_ADD_NEXT_HOP,
    IX_CC_IPV6_MSG_DELETE_NEXT_HOP,
    IX_CC_IPV6_MSG_UPDATE_NEXT_HOP,
    IX_CC_IPV6_MSG_GET_NEXT_HOP,
    IX_CC_IPV6_MSG_PURGE_RTM,
    IX_CC_IPV6_MSG_DUMP_NEXT_HOPS,
    IX_CC_IPV6_MSG_SET_MTU,
    IX_CC_IPV6_MSG_SET_FLAGS,


    /* ICMP Messages */
    IX_CC_IPV6_MSG_GET_RATE_LIMIT_TIME,
    IX_CC_IPV6_MSG_SET_RATE_LIMIT_TIME,
    IX_CC_IPV6_MSG_GET_QUEUE_DEPTH,
    IX_CC_IPV6_MSG_GET_PACKETS_TO_DRAIN,
    IX_CC_IPV6_MSG_SET_PACKETS_TO_DRAIN,
    IX_CC_IPV6_MSG_SEND_ICMP_ERROR,
    IX_CC_IPV6_MSG_SEND_ICMP_INFO,
   
    /* Statistics message */
    IX_CC_IPV6_MSG_GET_STATISTICS,

    /*Neighbor discovery messages*/
    IX_CC_IPV6_MSG_ADD_NBR,
    IX_CC_IPV6_MSG_DEL_NBR,

    /* Prefix change (update/delete) notification message */
    IX_CC_IPV6_MSG_PREFIX_CHANGE_NOTIFY,

    /* Add new messages before this */
    IX_CC_IPV6_MSG_LAST     
} ix_cc_ipv6_messages;

/**
 * TYPENAME: ix_cc_ipv6_error_code
 * 
 * DESCRIPTION: This enumerated type will describe ipv6 error codes
 *
 */
typedef enum ix_e_cc_ipv6_error_code
{
    IX_CC_IPV6_ERROR_FIRST = IX_ERROR_MAKE_GROUP(CC_IPV6),
    IX_CC_IPV6_ERROR_SUCCESS = IX_CC_IPV6_ERROR_FIRST,

    IX_CC_IPV6_ERROR_FAILED_PATCHING,
    IX_CC_IPV6_ERROR_REGISTRY,
    IX_CC_IPV6_ERROR_BUFFER_FREE,

    IX_CC_IPV6_ERROR_RTM,
    IX_CC_IPV6_ERROR_CCI,
    IX_CC_IPV6_ERROR_NBR_DISC,
    IX_CC_IPV6_ERROR_INVALID_CONTEXT,
    IX_CC_IPV6_ERROR_INVALID_INPUT_PARAM,
    IX_CC_IPV6_ERROR_INVALID_BUFFER,
    IX_CC_IPV6_ERROR_MSG_LIBRARY,
    IX_CC_IPV6_ERROR_INVALID_PACKET,
    IX_CC_IPV6_ERROR_INVALID_PREFIX,
    IX_CC_IPV6_ERROR_INVALID_ADDRESS,
    IX_CC_IPV6_ERROR_DUPLICATE_ADDRESS_DETECTED,

    IX_CC_IPV6_ERROR_INVALID_NH_INFO,
    IX_CC_IPV6_ERROR_INVALID_FLAGS,
    IX_CC_IPV6_ERROR_INVALID_RATE_LIMIT_TIME,
    IX_CC_IPV6_ERROR_INVALID_PACKETS_TO_DRAIN,
    IX_CC_IPV6_ERROR_DUPLICATE_NEXT_HOP,
    IX_CC_IPV6_ERROR_NEXT_HOP_ID_IN_USE,

    IX_CC_IPV6_ERROR_INVALID_PROP_ID,
    IX_CC_IPV6_ERROR_FAILED_GET_PORT_STATUS, 
    IX_CC_IPV6_ERROR_CALLBACK,
    IX_CC_IPV6_ERROR_FAILED_ADD_SOURCE_BROADCAST,
    IX_CC_IPV6_ERROR_FAILED_DELETE_SOURCE_BROADCAST,
    IX_CC_IPV6_ERROR_FAILED_ADDRESS_RESOLUTION,

    IX_CC_IPV6_ERROR_NOT_INITIALIZED,
    IX_CC_IPV6_ERROR_IGNORE_PACKET,
    
    /* unknown error code */
    IX_CC_IPV6_ERROR_UNKNOWN,

    IX_CC_IPV6_ERROR_LAST
} ix_cc_ipv6_error_code;

/**
 * TYPENAME: ix_cc_ipv6_dump_data
 * 
 * DESCRIPTION: This structure defines format for memory dump data .
 *
 */
typedef struct ix_s_cc_ipv6_dump_data
{
    char* pBuffer;
    ix_uint32 size;
} ix_cc_ipv6_dump_data;

/**
 * TYPENAME: ix_cc_ipv6_prefix_change_notify
 * 
 * DESCRIPTION: This structure defines format of the structure that contains
 * information about a route update. It includes the prefix, prefix length 
 * and the new next hop ID that was updated.
 */
typedef struct ix_s_cc_ipv6_prefix_change_notify
{
    ix_uint128       prefix;
    ix_uint8         prefixLength;
    ix_cc_rtmv6_nhid nextHopId;

} ix_cc_ipv6_prefix_change_notify;

/**
 * TYPENAME: ix_cc_ipv6_icmp_err_type
 *
 * DESCRIPTION: This enumeration defines error types for ICMP error messages.
 *
 */
typedef enum ix_e_cc_ipv6_icmp_err_type{
    IX_CC_IPV6_ICMP_ERR_FIRST = 0,
    IX_CC_IPV6_ICMP_ERR_DEST_UNREACHABLE = IX_CC_IPV6_ICMP_ERR_FIRST ,
    IX_CC_IPV6_ICMP_ERR_PKT_TOO_BIG,
    IX_CC_IPV6_ICMP_ERR_TIME_EXCEEDED,
    IX_CC_IPV6_ICMP_ERR_PARAM_PROBLEM,
    IX_CC_IPV6_ICMP_ERR_LAST
}ix_cc_ipv6_icmp_err_type;

/**
 * TYPENAME: ix_cc_ipv6_icmp_err_code
 *
 * DESCRIPTION: This enumeration defines error codes for ICMP error messages.
 *
 */
typedef enum ix_e_cc_ipv6_icmp_err_code{
    IX_CC_IPV6_ICMP_CODE_FIRST = 0,
    IX_CC_IPV6_ICMP_CODE_DU_NO_ROUTE = IX_CC_IPV6_ICMP_CODE_FIRST,
    IX_CC_IPV6_ICMP_CODE_DU_COMM_PROHIBITED,
    IX_CC_IPV6_ICMP_CODE_DU_ADDR_UNREACHABLE,
    IX_CC_IPV6_ICMP_CODE_DU_PORT_UNREACHABLE,
    IX_CC_IPV6_ICMP_CODE_PKT_TOO_BIG,
    IX_CC_IPV6_ICMP_CODE_TE_HOP_LIMIT_EXCEEDED,
    IX_CC_IPV6_ICMP_CODE_TE_REASSEMBLY_EXCEEDED,
    IX_CC_IPV6_ICMP_CODE_PP_WRONG_HDR_FIELD,
    IX_CC_IPV6_ICMP_CODE_PP_NEXT_HDR_UNKNOWN,
    IX_CC_IPV6_ICMP_CODE_PP_OPTION_UNKNOWN,
    IX_CC_IPV6_ICMP_CODE_LAST
}ix_cc_ipv6_icmp_err_code;

/**
 * TYPENAME: ix_cc_ipv6_cb_route_op
 * 
 * DESCRIPTION: Type definition for general IPv6 Message callbacks.
 *
 */

typedef ix_error (*ix_cc_ipv6_cb_route_op)(ix_error arg_Result, 
                                           void* arg_pContext);



/**
 * TYPENAME: ix_cc_ipv6_cb_lookup_route
 * 
 * DESCRIPTION: Type definition for rtm route lookup callback
 *
 */

typedef ix_error (*ix_cc_ipv6_cb_lookup_route)(ix_error arg_Result,
                                  void* arg_pContext,
                                  ix_cc_rtmv6_next_hop_info* arg_pNextHopInfo);

/**
 * TYPENAME: ix_cc_ipv6_cb_dump_data
 * 
 * DESCRIPTION: Type definition for rtm route and next hop lookup callback
 *
 */

typedef ix_error (*ix_cc_ipv6_cb_dump_data)(ix_error arg_Result,
                                            void* arg_pContext,
                                            ix_cc_ipv6_dump_data* arg_pBuffer);



/**
 * TYPENAME: ix_cc_ipv6_cb_get_next_hop
 * 
 * DESCRIPTION: Type definition for next hop lookup callback.
 *
 */

typedef ix_error (*ix_cc_ipv6_cb_get_next_hop)(ix_error arg_Result,
                                 void* arg_pContext,
                                 ix_cc_rtmv6_next_hop_info* arg_pNextHopInfo);



/**
 * TYPENAME: ix_cc_ipv6_cb_get_rate_limit_time
 * 
 * DESCRIPTION: Type definition for ICMP get rate limit time callback.
 *
 */

typedef ix_error (*ix_cc_ipv6_cb_get_rate_limit_time)(ix_error arg_Result,
                                                      void* arg_pContext,
                                                      ix_uint16 arg_Data);




/**
 * TYPENAME: ix_cc_ipv6_cb_get_queue_depth
 * 
 * DESCRIPTION: Type definition for ICMP get queue depth callback.
 *
 */

typedef ix_error (*ix_cc_ipv6_cb_get_queue_depth)(ix_error arg_Result,
                                                  void* arg_pContext,
                                                  ix_uint16 arg_QueueDepth);





/**
 * TYPENAME: ix_cc_ipv6_cb_get_packets_to_drain
 * 
 * DESCRIPTION: Type definition for ICMP get packets to drain callback.
 *
 */

typedef ix_error (*ix_cc_ipv6_cb_get_packets_to_drain)(ix_error arg_Result,
                                                       void* arg_pContext,
                                                       ix_uint16 arg_PacketsToDrain);



/**
 * TYPENAME: ix_cc_ipv6_cb_get_statistics
 * 
 * DESCRIPTION: Type definition for rtm route and next hop lookup callback
 *
 */

typedef ix_error (*ix_cc_ipv6_cb_get_statistics)(ix_error arg_Result,
                                                 void* arg_pContext,
                                                 ix_cc_ipv6_stats_data* arg_pBuffer);


/**
 * TYPENAME: ix_cc_ipv6_cb_addr_res
 *
 * DESCRIPTION: Type definition for address resolution callback
 *
 */
typedef ix_error (*ix_cc_ipv6_cb_addr_res)(ix_error arg_Result,
                                           void* arg_pContext,
                                           ix_uint8 *arg_L2Addr);

/**
 * NAME: ix_cc_ipv6_async_add_prefix
 *
 * DESCRIPTION: This message helper primitive helps client to add a prefix 
 * in RTMv6. Client needs to successfully add nextHopId to RTMv6 by calling 
 * ix_cc_ipv6_async_add_next_hop messaging helper primitive before sending 
 * this message to IPv6 Forwarder core component. 
 * 
 * @Param:  - IN arg_IpAddr: network IP address identifying the prefix.
 * @Param:  - IN arg_PrefixLen: prefix length for the prefix.
 * @Param:  - IN arg_NextHopId: next hop identifier for prefix.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */


ix_error ix_cc_ipv6_async_add_prefix(ix_uint128 arg_IpAddr,
                                    ix_uint8 arg_PrefixLen,
                                    ix_cc_rtmv6_nhid arg_NextHopId,
                                    ix_cc_ipv6_cb_route_op arg_Callback,
                                    void* arg_pUserContext);

/**
 * NAME: ix_cc_ipv6_async_delete_prefix
 *
 * DESCRIPTION:This message helper primitive helps a client to delete 
 * a prefix in RTMv6. Client needs to delete all prefix associated with a 
 * next hop to delete the next hop by calling ix_cc_ipv6_async_delete_next_hop 
 * message helper primitive. 
 * 
 * @Param:  - IN arg_IpAddr: network IP address identifying the prefix.
 * @Param:  - IN arg_PrefixLen: prefix length for the prefix.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */


ix_error ix_cc_ipv6_async_delete_prefix(ix_uint128 arg_IpAddr,
                                       ix_uint8 arg_NetMask,
                                       ix_cc_ipv6_cb_route_op arg_Callback,
                                       void* arg_pUserContext);


/**
 * NAME: ix_cc_ipv6_async_update_prefix
 *
 * DESCRIPTION: This message helper primitive helps client to update a prefix in RTM. 
 * Client needs to successfully add nextHopId to RTM by calling ix_cc_ipv6_async_add_next_hop
 * messaging helper primitive before sending this message to IPv6 Forwarder core component. 
 * 
 * @Param:  - IN arg_IpAddr: network IP address identifying the prefix.
 * @Param:  - IN arg_NetMask: network mask for the prefix.
 * @Param:  - IN arg_NextHopId: next hop identifier for the prefix.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */

ix_error ix_cc_ipv6_async_update_prefix(ix_uint128 arg_IpAddr,
                                    ix_uint8 arg_NetMask,
                                    ix_cc_rtmv6_nhid arg_NextHopId,
                                    ix_cc_ipv6_cb_route_op arg_Callback,
                                    void* arg_pUserContext);

/**
 * NAME: ix_cc_ipv6_async_lookup_prefix
 *
 * DESCRIPTION: This primitive helps a client to look up routing information 
 * for a given IP address. If there is a successful match, then the information will be 
 * given to client are nextHopId (including the blade id), portId, mtu and flags. 
 * If there is no match, then IX_CC_RTMV6_NHID_NO_ROUTE will be assigned to nextHopId
 * field of pNextHopData and all other information are invalid.
 * 
 * @Param:  - IN arg_IpAddr: network IP address identifying the prefix.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_ipv6_async_lookup_prefix(ix_uint128 arg_IpAddr,
                                       ix_cc_ipv6_cb_lookup_route arg_Callback,
                                       void* arg_pUserContext);



/**
 * NAME: ix_cc_ipv6_async_purge_prefixes
 *
 * DESCRIPTION: This primitive helps a client to remove all routes from RTM.
 * 
 *
 * @Param:  - IN None
 * @Param:  - OUT None
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_ipv6_async_purge_prefixes(void);



/**
 * NAME: ix_cc_ipv6_async_dump_prefixes
 *
 * DESCRIPTION: This primitive helps to dump all routes of RTM in memory
 * 
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_ipv6_async_dump_prefixes(ix_cc_ipv6_cb_dump_data arg_Callback,
                                      void* arg_pUserContext
                                     );


/**
 * NAME: ix_cc_ipv6_async_add_next_hop
 *
 * DESCRIPTION: This primitive helps a client to add next hop information to RTM database. 
 * Client needs to add next hop to RTM in order to add routes referring to that next hop.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop to be added
 * @Param:  - IN arg_pNextHopData: pointer to next hop information
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined contex
 * 
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_ipv6_async_add_next_hop(
                                ix_cc_rtmv6_nhid arg_NextHopId,
                                ix_uint32 arg_NumNextHops,
                                ix_cc_rtmv6_next_hop_info* arg_pNextHopData,
                                ix_cc_ipv6_cb_route_op arg_Callback,
                                void* arg_pUserContext
                                );


/**
 * NAME: ix_cc_ipv6_async_delete_next_hop
 *
 * DESCRIPTION: This message helper primitive helps a client to delete next hop information 
 * from RTM database. Client needs to remove all routes associated with next hop before 
 * calling this primitive. If not, client gets an error.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop to be removed.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */


ix_error ix_cc_ipv6_async_delete_next_hop(ix_cc_rtmv6_nhid arg_NextHopId,
                                          ix_cc_ipv6_cb_route_op arg_Callback,
                                          void* arg_pUserContext
                                          );


/**
 * NAME: ix_cc_ipv6_async_update_next_hop
 *
 * DESCRIPTION: This primitive helps a client to update next hop information to RTM database. 
 * Client needs to add next hop to RTM in order to add routes referring to that next hop.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop to be updated
 * @Param:  - IN arg_pNextHopData: pointer to next hop information
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined contex
 * 
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_ipv6_async_update_next_hop(
                                ix_cc_rtmv6_nhid arg_NextHopId,
                                ix_uint32 arg_NumNextHops,
                                ix_cc_rtmv6_next_hop_info* arg_pNextHopData,
                                ix_cc_ipv6_cb_route_op arg_Callback,
                                void* arg_pUserContext
                                );


/**
 * NAME: ix_cc_ipv6_async_get_next_hop
 *
 * DESCRIPTION: This message helper primitive helps a client to retrieve 
 * next hop information from RTM database. 
 * 
 * @Param:  - IN arg_NextHopId: Identifier of the next hop information to be retrieved from RTM.
 * @Param:  - IN arg_Callback: pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_ipv6_async_get_next_hop(ix_cc_rtmv6_nhid arg_NextHopId,
                                       ix_cc_ipv6_cb_get_next_hop arg_Callback,
                                       void* arg_pUserContext
                                      );


/**
 * NAME: ix_cc_ipv6_async_purge_rtm
 *
 * DESCRIPTION: This primitive helps a client to removes all routes and next 
 *              hops from RTM database
 * 
 *
 * @Param:  - IN - none
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */


ix_error ix_cc_ipv6_async_purge_rtm(void);


/**
 * NAME: ix_cc_ipv6_async_dump_next_hops
 *
 * DESCRIPTION:   This primitive helps to dump all next hops of RTM in memory.
 * 
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_ipv6_async_dump_next_hops(ix_cc_ipv6_cb_dump_data arg_Callback,
                                         void* arg_pUserContext
                                        );


/**
 * NAME: ix_cc_ipv6_async_set_mtu
 *
 * DESCRIPTION:This primitive helps client to update MTU for a given next hop. 
 * Before calling this message helper, client needs to call ix_cc_ipv6_async_add_next_hop to 
 * add next hop identifier in RTM database. The validity of MTU shall be done by the client.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop.
 * @Param:  - IN arg_Mtu: new maximum transmission unit for the given next hop.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */


ix_error ix_cc_ipv6_async_set_mtu(ix_cc_rtmv6_nhid arg_NextHopId,
                                  ix_uint32 arg_Mtu,
                                  ix_cc_ipv6_cb_route_op arg_Callback,
                                  void* arg_pUserContext
                                 );


/**
 * NAME: ix_cc_ipv6_async_set_flags
 *
 * DESCRIPTION:This primitive helps client to update flags for a given next hop.
 * Before calling this message helper, client needs to call ix_cc_ipv6_async_add_next_hop to add next hop 
 * identifier in RTM database. The supported flags[BLDBLK] are defined by IPv6 Forwarder microblock.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop.
 * @Param:  - IN arg_Flags: new flags for the given next hop 
 * @Param:  - IN arg_pUserContext: pointer to client defined context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */
ix_error ix_cc_ipv6_async_set_flags(ix_cc_rtmv6_nhid arg_NextHopId,
                                    ix_uint32 arg_Flags,
                                    ix_cc_ipv6_cb_route_op arg_Callback,
                                    void* arg_pUserContext
                                   );



/**
 * NAME: ix_cc_ipv6_async_get_rate_limit_time
 *
 * DESCRIPTION:This message helper primitive helps client to retrieve the number of seconds 
 * between when the event will fire causing the ICMP messages to dequeue and send. 
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_ipv6_async_get_rate_limit_time(ix_cc_ipv6_cb_get_rate_limit_time 
                                                                       arg_Callback,
                                              void* arg_pUserContext);


/**
 * NAME: ix_cc_ipv6_async_set_rate_limit_time
 *
 * DESCRIPTION: This message helper primitive helps client to set the number of seconds between 
 * when the event will fire causing the ICMP messages to dequeue and send.  The lower the value of arg_SleepTime, 
 * the more frequent the ICMP messages will be dequeued and sent.
 * 
 * @Param:  - IN arg_SleepTime: minimum value is 1 second and maximum value is 32 seconds.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV6_ERROR_INVALID_RATE_LIMIT_TIME - invalid sleep time
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_ipv6_async_set_rate_limit_time(ix_uint16 arg_SleepTime);



/**
 * NAME: ix_cc_ipv6_async_get_queue_depth
 *
 * DESCRIPTION:This message helper primitive helps client to retrieves the depth of the ICMP error message queue. 
 * Client retrieves the maximum number of ICMP messages that can fit in the queue, not the number of messages
 * currently in the queue.
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */
ix_error ix_cc_ipv6_async_get_queue_depth(ix_cc_ipv6_cb_get_queue_depth arg_Callback,
                                          void* arg_pUserContext
                                         );

/**
 * NAME: ix_cc_ipv6_async_get_packets_to_drain
 *
 * DESCRIPTION:This message helper primitive helps client to retrieve the maximum number of ICMP error messages 
 * to send each time the queue processing event fires. The processing event will send up to this many ICMP error
 * packets from the queue before going back to sleep. 
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */
ix_error ix_cc_ipv6_async_get_packets_to_drain(ix_cc_ipv6_cb_get_packets_to_drain arg_Callback,
                                               void* arg_pUserContext
                                              );

/**
 * NAME: ix_cc_ipv6_async_set_packets_to_drain
 *
 * DESCRIPTION:This message helper primitive helps client to set the maximum number of ICMP error messages to 
 * send each time the queue processing event fires. The value to arg_PacketsToDrain must not exceed the depth of 
 * the queue.
 * 
 * @Param:  - IN arg_PacketsToDrain: The maximum number of packets to send each time the queue processing event fires.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV6_ERROR_INVALID_PACKETS_TO_DRAIN - invalid value for packets to drain
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */
ix_error ix_cc_ipv6_async_set_packets_to_drain(ix_uint16 arg_PacketsToDrain);




/**
 * NAME: ix_cc_ipv6_async_get_statistics
 *
 * DESCRIPTION:This message helper primitive helps client to retrieve statistics report from IPv6 Forwarder core component.
 * This primitive will give the statistics from IPv6 microblock, RTM and IPV6 Forwarder core component.  
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pUserContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_ipv6_async_get_statistics(ix_cc_ipv6_cb_get_statistics arg_Callback,
                                         void* arg_pUserContext
                                        );


/**
 * NAME: ix_cc_ipv6_async_send_icmp_error
 *
 * DESCRIPTION: This message helper primitive helps the client to send an ICMP
 * error message.
 *
 * @Param:  - IN arg_IcmpErrType: The ICMP error type to include in the message
 * @Param:  - IN arg_IcmpErrCode: The ICMP error code to include in the message
 * @Param:  - IN arg_hBuffer: The IPv6 packet causing the error.
 * @Param:  - IN arg_Param: Contains extra information if required by the ICMP
 *                          error message type.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 *
 */
ix_error ix_cc_ipv6_async_send_icmp_error(
                            ix_cc_ipv6_icmp_err_type    arg_IcmpErrType,
                            ix_cc_ipv6_icmp_err_code    arg_IcmpErrCode,
                            ix_buffer_handle            arg_hBuffer,
                            ix_uint32                   arg_Param,
                            ix_cc_ipv6_cb_route_op      arg_Callback,
                            void*                       arg_pUserContext);

/**
 * NAME: ix_cc_ipv6_async_send_icmp_info_message
 *
 * DESCRIPTION: This message helper primitive helps the client to send an ICMP
 *              informational message.
 *
 * @Param:  - IN arg_SrcIpAddr: The IPv6 address to be included in the source
 *                              address field of the final IPv6 packet.
 * @Param:  - IN arg_DstAddr: The destination IPv6 address.
 * @Param:  - IN arg_Type: The type of the ICMP message. This value will be
 *                          included in the ICMP header.
 * @Param:  - IN arg_Code: The ICMP message code corresponding to the message
 *                          type.
 * @Param:  - IN arg_PayloadLength: The length of the ICMP payload.
 * @Param:  - IN arg_pPayload: The payload to include in the ICMP packet.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 *
 */
ix_error ix_cc_ipv6_async_send_icmp_info_message(
                                        ix_uint128  arg_SrcIpAddr,
                                        ix_uint128  arg_DstIpAddr,
                                        ix_uint8    arg_Type,
                                        ix_uint8    arg_Code,
                                        ix_uint32   arg_PayloadLength,
                                        ix_uint8    *arg_pPayload,
                                        ix_cc_ipv6_cb_route_op arg_Callback,
                                        void        *arg_pUserContext);

/**
 * NAME: ix_cc_ipv6_async_perform_addr_resolution
 *
 * DESCRIPTION: This message helper primitive helps the cllient to request
 *              address resolution to be performed for a packet destined
 *              to a neighbor.
 *
 * @Param:  - IN arg_hBuffer: The hardware buffer containing the ethernet
 *                            packet to be sent to the neighbor
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 *
 */
ix_error ix_cc_ipv6_async_perform_addr_resolution(
                                         ix_buffer_handle arg_hBuffer,
                                         ix_cc_ipv6_cb_route_op  arg_Callback,
                                         void* arg_pUserContext);


/**
 * NAME: ix_cc_ipv6_async_add_neighbor
 *
 * DESCRIPTION: This message helper primitive helps the cllient to add a
 *              neighbor to the L2 table
 *
 * @Param:  - IN arg_L2Index: The index into the L2 table at which this
 *                            information should be added.
 * @Param   - IN arg_NbrIp:   The IPv6 address of the neighbor.
 * @Param   - IN arg_OutputPort: The egress port from which all traffic
 *                            destined for the neighbor should exit.
 * @Param   - IN arg_L2Addr: The MAC address of the neighbor.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 *
 */
ix_error ix_cc_ipv6_async_add_neighbor(
                                    ix_uint16 arg_L2Index,
                                    ix_uint128 arg_NbrIp,
                                    ix_uint16 arg_OutputPort,
                                    ix_uint8 *arg_L2Addr,
                                    ix_cc_ipv6_cb_route_op arg_Callback,
                                    void* arg_pUserContext);

/**
 * NAME: ix_cc_ipv6_async_del_neighbor
 *
 * DESCRIPTION: This message helper primitive helps the cllient to add a
 *              neighbor to the L2 table
 *
 * @Param:  - IN arg_L2Index: The index into the L2 table at which this
 *                            information should be added.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 *
 */
ix_error ix_cc_ipv6_async_del_neighbor(
                                 ix_uint16 arg_L2Index,
                                 ix_cc_ipv6_cb_route_op arg_Callback,
                                 void* arg_pUserContext);


/**
 * NAME: ix_cc_ipv6_sync_add_neighbor
 *
 * DESCRIPTION: This message helper primitive helps the cllient to add a
 *              neighbor to the L2 table synchronously. This call MUST NOT
 *              be invoked by core components.
 *
 * @Param:  - IN arg_L2Index: The index into the L2 table at which this
 *                            information should be added.
 * @Param   - IN arg_NbrIp:   The IPv6 address of the neighbor.
 * @Param   - IN arg_OutputPort: The egress port from which all traffic
 *                            destined for the neighbor should exit.
 * @Param   - IN arg_L2Addr: The MAC address of the neighbor.
 *
 * @Param:  - OUT - none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 *
 */
ix_error ix_cc_ipv6_sync_add_neighbor(
                                    ix_uint16 arg_L2Index,
                                    ix_uint128 arg_NbrIp,
                                    ix_uint16 arg_OutputPort,
                                    ix_uint8 *arg_L2Addr);

/**
 * NAME: ix_cc_ipv6_sync_del_neighbor
 *
 * DESCRIPTION: This message helper primitive helps the cllient to add a
 *              neighbor to the L2 table synchronously. This call MUST NOT
 *              be invoked by other core components.
 *
 * @Param:  - IN arg_L2Index: The index into the L2 table at which this
 *                            information should be added.
 *
 * @Param:  - OUT - none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 *
 */
ix_error ix_cc_ipv6_sync_del_neighbor(
                                 ix_uint16 arg_L2Index);

/**
 * NAME: ix_cc_ipv6_init
 *
 * DESCRIPTION: This primitive shall be called and returned successfully before requesting any service
 *              from IPv6 Forwarder component. This primitive should be called only once to initialize 
 *              IPv6 Forwarder component.This function performs allocation of memory for symbols to be patched,
 *              Creation of 64 bit counters, registration of packet and message handlers,
 *              initialization and configuration of RTM, allocation and initialization of internal data structures
 *              and creation of event handler for ICMP.
 * 
 * @Param:  - IN arg_hCcHandle - handle to IPV6 core component, created by core component infrastructure; 
 *            this shall be used later to get other (to add event handler) services from core component infrastructure.
 *
 * @Param:  - OUT arg_ppContext - location where the pointer to the control block allocated by 
 *            IPv6 Forwarder core component will be stored. The control block is internal to IPv6 
 *            Forwarder core component and it contains information about IPv4 internal data structures, 
 *            allocated memory and other relevant information. Later this pointer to be passed into  
 *            ix_cc_ipv6_fini function for freeing memory and other house keeping operations 
 *            when IPv6 component is being destroyed.  
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOM   - memory allocation failure      
 * IX_CC_IPV6_ERROR_FAILED_PATCHING   - patching failures                      
 * IX_CC_ERROR_OOM_64BIT_COUNTER - 64 bit counter creation failure
 * IX_CC_IPV6_ERROR_REGISTRY  - registry access failures
 * IX_CC_IPV6_ERROR_RTM - failure from RTM core component
 * IX_CC_IPV6_ERROR_CCI - failure from CCI 
 * 
 */

ix_error ix_cc_ipv6_init(
                         ix_cc_handle arg_hCcHandle,
                         void** arg_ppContext
                        );



/**
 * NAME: ix_cc_ipv6_fini
 *
 * DESCRIPTION: This primitive shall be called to terminate services from IPv6 forwarder 
 * component. This primitive frees memory allocated during initialization, shut down RTM, 
 * free all the created resources: delete 64 bit counter. If client claims for any services 
 * from IPv6 after scalling this function, then the behavior is undefined.
 * 
 * @Param:  - IN arg_hCcHandle - handle to IPv6 Forwader core component
 *
 * @Param:  - IN arg_pContext - ppointer to the control block memory allocated 
 * earlier in ix_cc_ipv6_init function. 
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_INVALID_INPUT_PARAM - invalid input parameter(s)
 * IX_CC_ERROR _OOM   - failure of memory deletion                
 * IX_CC_ERROR_OOM_64BIT_COUNTER - failure of  64 bit counter deletion 
 * IX_CC_IPV6_ERROR _RTM - failure from RTM
 * IX_CC_IPV6_ERROR_CCI - failure from CCI
 * 
 */
ix_error ix_cc_ipv6_fini(
                         ix_cc_handle arg_hCcHandle,
                         void* arg_pContext
                        );


/**
 * NAME: ix_cc_ipv6_msg_handler
 *
 * DESCRIPTION: This primitive is the message handler function for IPv6 Forwarder
 * core component. IPV6 Forwarder core component receives messages from other core component
 * through this message handler function and internally it calls appropriate library function
 * to process the message. This message handler shall be used to update dynamic properties.
 * 
 * @Param:  - IN arg_hDataToken - buffer handle embedding information for the message
 * passed in arg_UserData.
 *
 * arg_UserData - message type
 *
 * @Param:  - OUT - arg_pComponentContext: pointer to IPv6 Forwarder core component context 
 * that will be passed to the core component when a message arrives. This context was defined
 * by the core component and passed to core components infrastructure through 
 * ix_cc_ipv6_init function.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - null argument
 * IX_CC_IPV6_ERROR_INVALID_CONTEXT - context is unusable
 * IX_CC_ERROR_UNDEFINED_MSG - un supported message
 * IX_CC_IPV6_ERROR_MSG_LIBRARY - error from message support library
 * 
 */
ix_error ix_cc_ipv6_msg_handler(
                                ix_buffer_handle arg_hDataToken,
                                ix_uint32 arg_UserData,
                                void* arg_pContext
                                );



/**
 * NAME: ix_cc_ipv6_microblock_high_priority_pkt_handler
 *
 * DESCRIPTION: This is the only registered function to receive exception packets from high priority
 * queue of IPv6 microblock. This function sends the packet to stack driver
 * (ie, recieved only local delivery packets). 
 * 
 * @Param:  - IN arg_hDataToken: handle to a buffer which contains exception packets
 *  from IPv6 microblock
 * @Param: - IN arg_ExceptionCode: will be ignored.
 * @Param: - IN arg_pContext: pointer to IPV6 Forwarder core component context that 
 * will be passed to the core component when a packet arrives. This context was defined by
 * the core component and passed to core components infrastructure through 
 * the ix_cc_ipv6_init function.
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_CCI - error from CCI
 * 
 */

ix_error ix_cc_ipv6_microblock_high_priority_pkt_handler(
                                                         ix_buffer_handle arg_hDataToken,  
                                                         ix_uint32 arg_ExceptionCode,
                                                         void* arg_pContext
                                                        );




/**
 * NAME: ix_cc_ipv6_microblock_low_priority_pkt_handler
 *
 * DESCRIPTION: TThis is the registered function to receive exception packets from 
 * IPv6 microblock. This function internally calls/performs different functions/operations
 * based on exception code for the given packet.
 * 
 * @Param:  - IN arg_hDataToken: handle to a buffer which contains exception packets
 * from IPv6 microblock
 * @Param: - IN arg_ExceptionCode: Exception codes generated by IPv6 forwarder microblock.
 * @Param: - IN arg_pContext: pointer to IPV6 Forwarder core component context that 
 * will be passed to the core component when a packet arrives. This context was defined by
 * the core component and passed to core components infrastructure through 
 * the ix_cc_ipv6_init function.
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_CCI - error from CCI
 * IX_CC_ERROR_UNDEFINED_EXCEP - unsupported exception code
 * IX_CC_ERROR_INTERNAL - an internal error
 * IX_CC_ERROR_ALIGN - pointer not aligned properly
 * IX_CC_ERROR_NULL - null pointer
 * IX_CC_IPV6_ERROR_RTM - error from RTM
 *
 *
 */
ix_error ix_cc_ipv6_microblock_low_priority_pkt_handler(
                                                        ix_buffer_handle arg_hDataToken,  
                                                        ix_uint32 arg_ExceptionCode,
                                                        void* arg_pContext
                                                       );





/**
 * NAME: ix_cc_ipv6_stkdrv_pkt_handler
 *
 * DESCRIPTION: This is the registered function to receive packets from stack driver 
 * core component. Packet comes from stack driver need special processing by 
 * IPv6 core component: no TTL decrement, no IP header validation.
 * 
 * @Param:  - IN arg_hDataToken: handle to a buffer which contains packet
 * from stack driver core component. - 
 *
 * @Param:  - IN arg_ExceptionCode:  will be Ignored 
 *
 * @Param:  - IN arg_pComponnetContext: pointer to IPV6 Forwarder core component context
 * that will be passed to the core component when a packet arrives. This context was 
 * defined by the core component and passed to core components infrastructure through 
 * the ix_cc_ipv6_init function. 
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_CCI - error from CCI
 * IX_CC_ERROR_INTERNAL - an internal error
 * IX_CC_ERROR_ALIGN - pointer not aligned properly
 * IX_CC_ERROR_NULL - null pointer
 * 
 */
ix_error ix_cc_ipv6_stkdrv_pkt_handler(
                                           ix_buffer_handle arg_hDataToken,
                                           ix_uint32 arg_ExceptionCode,
                                           void* arg_pContext
                                           );




/**
 * NAME: ix_cc_ipv6_common_pkt_handler
 *
 * DESCRIPTION: This is the registered function to receive packets 
 * from any core component other than stack driver: interface Rx core component.
 * 
 * @Param:  - IN arg_hDataToken: handle to a buffer which contains packet from any core component other than stack driver.
 * @Param:  - INarg_ExceptionCode:  will be Ignored
 * @Param:  - INarg_pComponnetContext: pointer to IPV6 Forwarder core component context 
 * that will be passed to the core component when a packet arrives. 
 * This context was defined by the core component and passed to core components infrastructure
 * through the ix_cc_ipv6_init function.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_INTERNAL - an internal error
 * IX_CC_ERROR_ALIGN - pointer not aligned properly
 * IX_CC_ERROR_NULL - null pointer
 * 
 */
ix_error ix_cc_ipv6_common_pkt_handler(
                                           ix_buffer_handle arg_hDataToken,
                                           ix_uint32 arg_ExceptionCode,
                                           void* arg_pContext
                                           );



/**
 * NAME: ix_cc_ipv6_add_prefix
 *
 * DESCRIPTION:This library primitive helps to add a prefix in RTM. 
 * This API internally calls ix_cc_rtmv6_add_prefix API of RTM core component 
 * to add a prefix in RTM database.
 * 
 * @Param:  - IN arg_IpAddr: network IP address identifying the prefix.
 * @Param:  - IN arg_NetMask: network mask for the prefix.
 * @Param:  - IN arg_NextHopId: next hop identifier for prefix.
 * @Param:  - IN arg_pContext: pointer to IPv6 context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_ENTRY_NOT_FOUND - invalid next hop identifier
 * IX_CC_ERROR_DUPLICATE_ROUTE - prefix was already added to this NHID
 * IX_CC_ERROR_CONFLICTING_ROUTE - prefix was already added to another NHID
 * IX_CC_IPV6_ERROR_RTM - RTM internal failure
 * 
 */

ix_error ix_cc_ipv6_add_prefix(ix_uint128 arg_IpAddr,
                              ix_uint8 arg_NetMask,
                              ix_cc_rtmv6_nhid arg_NextHopId, 
                              void* arg_pContext);



/**
 * NAME: ix_cc_ipv6_delete_prefix
 *
 * DESCRIPTION: This library primitive helps to delete a prefix from RTM. 
 * This API internally calls ix_cc_rtmv6_del_prefix API of RTM core component to delete prefix from RTM database.
 * 
 * @Param:  - IN arg_IpAddr: network IP address identifying the prefix
 *
 * @Param:  - IN arg_NetMask: network mask for the prefix
 * @Param:  - IN arg_pContext: pointer to IPv6 context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_ENTRY_NOT_FOUND - entry not found in RTM
 * IX_CC_IPV6_ERROR_RTM - RTM internal failure
 * 
 */

ix_error ix_cc_ipv6_delete_prefix(ix_uint128 arg_IpAddr,
                                 ix_uint8 arg_NetMask, 
                                 void* arg_pContext);


/**
 * NAME: ix_cc_ipv6_update_prefix
 *
 * DESCRIPTION:This library primitive helps to add a prefix in RTM. 
 * This API internally calls ix_cc_rtmv6_update_prefix API of RTM core component to 
 * add prefix in RTM database. 
 * 
 * @Param:  - IN arg_IpAddr: network IP address the prefix
 * @Param:  - IN arg_NetMask: network mask for the prefix
 * @Param:  - IN arg_NextHopId: hop identifier for prefix to be updated
 * @Param:  - IN arg_pContext: pointer to IPv6 context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_ENTRY_NOT_FOUND - invalid next hop identifier
 * IX_CC_IPV6_ERROR_INVALID_MASK - invalid netmask
 * IX_CC_ERROR_CONFLICTING_ROUTE - prefix was already added to another NHID
 * IX_CC_IPV6_ERROR_RTM - RTM internal failure
 * 
 */

ix_error ix_cc_ipv6_update_prefix(ix_uint128 arg_IpAddr,
                              ix_uint8 arg_NetMask,
                              ix_cc_rtmv6_nhid arg_NextHopId, 
                              void* arg_pContext);

/**
 * NAME: ix_cc_ipv6_lookup_prefix
 *
 * DESCRIPTION: This primitive helps a client to look up routing information for a given IP address.
 * If there is a successful match, then the information will be given to client are nextHopId (including the blade id),
 * portId, mtu and flags. 
 * 
 * @Param:  - IN arg_IpAddr: network IP address identifying the prefix to be updated.
 * @Param:  - INarg_NetMask: network mask for the prefix to be updated.
 * @Param:  - INarg_NextHopId: next hop identifier for the prefix to be updated.
 * @Param:  - IN arg_pContext: pointer to IPv6 context 
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - null pinter
 * IX_CC_IPV6_ERROR_RTM - RTM internal error
 * 
 */

ix_error ix_cc_ipv6_lookup_prefix(ix_uint128 arg_IpAddr,
                                 ix_cc_rtmv6_next_hop_info *arg_pNextHopInfo, 
                                 void* arg_pContext);



/**
 * NAME: ix_cc_ipv6_purge_prefixes
 *
 * DESCRIPTION: This primitive helps a client to remove all routes from RTM.
 * 
 * @Param:  - IN arg_pContext: pointer to IPv6 context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - null pointer
 * IX_CC_ERROR_ALIGN - pointer not aligned properly
 * 
 */
ix_error ix_cc_ipv6_purge_prefixes(void* arg_pContext);


/**
 * NAME: ix_cc_ipv6_dump_prefixes
 *
 * DESCRIPTION: This primitive helps to dump all routes of RTM in memory
 * 
 * @Param:  - IN arg_pBuffer - pointer to a block of memory where routes are to be stored.
 * @Param:  - IN arg_Size - size of buffer in bytes.
 * @Param:  - IN arg_pContext: pointer to IPv6 context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - null pointer
 * IX_CC_ERROR_ALIGN - pointer not aligned properly
 * IX_CC_ERROR_FULL -  buffer is too small
 */
ix_error ix_cc_ipv6_dump_prefixes(char* arg_pBuffer,
                                ix_uint32 arg_Size, 
                                void* arg_pContext);


/**
 * NAME: ix_cc_ipv6_add_next_hop
 *
 * DESCRIPTION: This primitive helps a client to add next hop information to RTM database.
 * Client needs to add next hop to RTM in order to add routes referring to that next hop. 
 * 
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop to be added.
 * @Param:  - IN arg_pNextHopData: defined by RTM
 * @Param:  - IN arg_pContext: pointer to IPv6 context
 *
 * @Param:  - OUT  none 
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_DUPLICATE_ROUTE - route was already added to this NHID
 * IX_CC_ERROR_CONFLICTING_ROUTE - route was already added to another NHID
 * IX_CC_IPV6_ERROR_RTM - RTM internal failure
 * IX_CC_IPV6_ERROR_INVALID_NH_INFO - invalid next hop information
 */

ix_error ix_cc_ipv6_add_next_hop(ix_cc_rtmv6_nhid arg_NextHopId,
                                 ix_uint32 arg_NumNextHops,
                                 ix_cc_rtmv6_next_hop_info* arg_pNextHopInfo, 
                                 void* arg_pContext);


/**
 * NAME: ix_cc_ipv6_delete_next_hop
 *
 * DESCRIPTION: This primitive helps a client to delete next hop information from RTM database. 
 * Client needs to remove all routes associated with next hop before calling this primitive. 
 * If not, client gets an error.This API internally calls ix_cc_rtmv6_delete_next_hop API of RTM
 * to delete next hop information from RTM.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop to be removed.
 * @Param:  - IN arg_pContext: pointer to IPv6 context
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_RTM - RTM internal failure
 * IX_CC_ERROR_ENTRY_NOT_FOUND - invalid next hop id
 * IX_CC_IPV6_ERROR_NEXT_HOP_ID_IN_USE -next hop id is in use
 */

ix_error ix_cc_ipv6_delete_next_hop(ix_cc_rtmv6_nhid arg_pNextHopId, 
                                    void* arg_pContext);


/**
 * NAME: ix_cc_ipv6_update_next_hop
 *
 * DESCRIPTION: This primitive helps a client to add next hop information to RTM database.
 * Client needs to add next hop to RTM in order to add routes referring to that next hop. 
 * 
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop to be added.
 * @Param:  - IN arg_pNextHopData: defined by RTM
 * @Param:  - IN arg_pContext: pointer to IPv6 context
 *
 * @Param:  - OUT  none 
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_RTM - RTM internal failure
 * IX_CC_IPV6_ERROR_INVALID_NH_INFO - invalid next hop information
 * IX_CC_ERROR_ENTRY_NOT_FOUND - invalid next hop id
 */

ix_error ix_cc_ipv6_update_next_hop(
                               ix_cc_rtmv6_nhid arg_NextHopId,
			       ix_uint32 arg_numNextHops,
                               ix_cc_rtmv6_next_hop_info* arg_pNextHopInfo, 
                               void* arg_pContext);



/**
 * NAME: ix_cc_ipv6_get_next_hop
 *
 * DESCRIPTION:This library primitive helps to retrieve next hop information from RTM.
 * This API internally calls ix_cc_rtmv6_get_next_hop API of RTM core component to 
 * retrieve next hop information. 
 * 
 * @Param:  - IN arg_NextHopId: Identifier of the next hop information to be retrieved from RTM.
 * @Param:  - IN arg_pContext: pointer to IPv6 context
 *
 * @Param:  - OUT arg_pNextHopInfo: Result will be placed into this structure
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_RTM - RTM internal failure
 * IX_CC_ERROR_ENTRY_NOT_FOUND - invalid next hop id
 * 
 */
ix_error ix_cc_ipv6_get_next_hop(
                                 ix_cc_rtmv6_nhid arg_NextHopId,
                                 ix_cc_rtmv6_next_hop_info* arg_pNextHopInfo, 
                                 void* arg_pContext
                                );




/**
 * NAME: ix_cc_ipv6_dump_next_hops
 *
 * DESCRIPTION: This API internally calls ix_cc_rtmv6_dump_next_hop API of RTM core component. 
 * Client shall use IX_CC_RTMV6_DUMP_NEXT_HOP_SIZE to determine how many bytes to allocate.
 * 
 * @Param:  - IN arg_pBuffer - pointer to a block of memory where next hops are to be stored.
 * @Param:  - IN arg_Size - size of buffer in bytes
 * @Param:  - IN arg_pContext: pointer to IPv6 context
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_RTM - RTM failure
 * 
 */

ix_error ix_cc_ipv6_dump_next_hops(
                                char* arg_pBuffer,
                                ix_uint32 arg_Size, 
                                void* arg_pContext);


/**
 * NAME: ix_cc_ipv6_purge_rtm
 *
 * DESCRIPTION: This primitive helps a client to removes all routes and next hops from RTM database.
 * 
 * @Param:  - IN arg_pContext: pointer to IPv6 context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_RTM - RTM failure
 * 
 */
ix_error ix_cc_ipv6_purge_rtm(void* arg_pContext);



/**
 * NAME: ix_cc_ipv6_set_mtu
 *
 * DESCRIPTION: This primitive helps client to update MTU for a given next hop. 
 * Before calling this, client needs to call ix_cc_ipv6_add_next_hop to add next hop 
 * identifier in RTM database. The validity of MTU shall be done by the client.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop.
 * @Param:  - IN arg_Mtu: new maximum transmission unit for the given next hop.
 * @Param:  - IN arg_pContext: pointer to IPv6 context
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV6_ERROR_RTM - RTM failure
 * IX_CC_ERROR_ENTRY_NOT_FOUND - invalid next hop id 
 * IX_CC_IPV6_ERROR_INVALID_NH_INFO - invalid mtu
 */

ix_error ix_cc_ipv6_set_mtu(
                            ix_cc_rtmv6_nhid arg_NextHopId,
                            ix_uint32 arg_Mtu, 
                            void* arg_pContext);



/**
 * NAME: ix_cc_ipv6_set_flags
 *
 * DESCRIPTION:This primitive helps client to update flags for a given next hop.
 * Before calling this primitive, client needs to call ix_cc_ipv6_add_next_hop to 
 * add next hop identifier in RTM database. 
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop.
 * @Param:  - IN arg_Flags: new flags for the given next hop 
 * @Param:  - IN arg_pContext: pointer to IPv6 context
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_RTM - RTM failure
 * IX_CC_ERROR_ENTRY_NOT_FOUND - invalid next hop id 
 * IX_CC_IPV6_ERROR_INVALID_FLAGS - invalid flags 
 * 
 */
ix_error ix_cc_ipv6_set_flags(
                              ix_cc_rtmv6_nhid arg_NextHopId,
                              ix_uint32 arg_Flags, 
                              void* arg_pContext);



/**
 * NAME: ix_cc_ipv6_get_rate_limit_time
 *
 * DESCRIPTION: This library primitive helps to retrieve the number of seconds 
 * between when the event will fire causing the ICMP messages to be dequeued and sent. 
 * The lower the value of arg_pSleepTime, the more frequently the ICMP messages will be 
 * dequeued and sent.  
 * 
 * @Param:  - IN arg_pContext: pointer to IPv6 context
 *
 * @Param:  - OUT arg_pSleepTime: ICMP sleep period 
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - null pointer
 * 
 */
ix_error ix_cc_ipv6_get_rate_limit_time(ix_uint16* arg_pSleepTime, 
                                        void* arg_pContext);



/**
 * NAME: ix_cc_ipv6_set_rate_limit_time
 *
 * DESCRIPTION: This library primitive helps to set the period between events firing to process 
 * the ICMP error message queue. The lower the value of arg_SleepTime, the more frequently
 * the output queue will be processed. The minimum value of arg_SleepTime is 1 sec and 
 * maximum value can be 32sec.
 * 
 * @Param:  - IN arg_SleepTime: the number of seconds to wait between processing
 * the ICMP error message queue.
 * @Param:  - IN arg_pContext: pointer to IPv6 context
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_INVALID_RATE_LIMIT_TIME - invalid sleep time
 * 
 */

ix_error ix_cc_ipv6_set_rate_limit_time(ix_uint16 arg_pSleepTime, 
                                        void* arg_pContext);




/**
 * NAME: ix_cc_ipv6_get_queue_depth
 *
 * DESCRIPTION: This library primitive helps to retrieves the depth of the ICMP error message 
 * queue. That is, the maximum number of ICMP messages that can fit in the queue, not the
 * number of messages currently in the queue.
 * 
 * @Param:  - IN arg_pContext: pointer to IPv6 context
 *
 * @Param:  - OUT arg_pQueueDepth: the current number of entries that the ICMP error message queue can hold.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - null pointer
 * 
 */
ix_error ix_cc_ipv6_get_queue_depth(ix_uint16* arg_pQueueDepth, 
                                    void* arg_pContext);


/**
 * NAME: ix_cc_ipv6_get_packets_to_drain
 *
 * DESCRIPTION:This library primitive helps to retrieve the maximum number of ICMP error messages
 * to send (from the ICMP message queue ) each time the queue processing event fires. 
 * 
 * @Param:  - IN arg_pContext: pointer to IPv6 context
 *
 * @Param:  - OUT arg_pPacketsToDrain: the maximum number of packets to send each time the 
 * event fires for processing the queue.  
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * 
 */
ix_error ix_cc_ipv6_get_packets_to_drain(ix_uint16* arg_pPacketsToDrain, 
                                         void* arg_pContext);


/**
 * NAME: ix_cc_ipv6_set_packets_to_drain
 *
 * DESCRIPTION: This library primitive helps to set the maximum number of ICMP error messages 
 * to send each time the queue processing event fires. The value of arg_PacketsToDrain
 * must not exceed the depth of the queue.
 * 
 * @Param:  - IN arg_PacketsToDrain: The maximum number of packets to send each time the queue processing
 * event fires. 
 * @Param:  - IN arg_pContext: pointer to IPv6 context
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_INVALID_PACKETS_TO_DRAIN - invalid value
 * 
 */
ix_error ix_cc_ipv6_set_packets_to_drain(ix_uint16 arg_pPacketsToDrain, 
                                         void* arg_pContext);



/**
 * NAME: ix_cc_ipv6_get_statistics
 *
 * DESCRIPTION: This library primitive helps to retrieve statistics report from IPV6 Forwarder 
 * core component. This primitive will give the statistics of IPV6 microblock, RTM and 
 * IPV6 Forwarder core component. Client needs to allocate memory to hold data.
 * 
 * @Param:  - IN arg_pContext: pointer to IPv6 context
 *
 * @Param:  - OUT arg_pBuffer: pointer to memory to hold statistics data.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - null pointer
 * 
 */
ix_error ix_cc_ipv6_get_statistics(ix_cc_ipv6_stats_data* arg_pBuffer,
                                   void* arg_pContext);


/**
 * NAME: ix_cc_ipv6_set_property
 *
 * DESCRIPTION: This library primitive helps to set dynamic properties of 
 * IPv6 Forwarder core component. 
 * 
 * @Param:  - IN arg_PropId: Identifier of a property or properties(ORed) to be updated 
 * for a port
 * @Param:  - IN arg_pPropData: pointer to dynamic property data structure
 * @Param:  - IN arg_pContext: pointer to IPv6 context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * IX_CC_ERROR_NUL - invalid pointer
 * IX_CC_IPV6_ERROR_INVALID_INPUT_PARAM - invalid input param
 * IX_ERROR_FULL - table is full
 * IX_CC_ERROR_ENTRY_NOT_FOUND - entry not found in table
 *
 */
ix_error ix_cc_ipv6_set_property(ix_uint32 arg_PropId,
                                 ix_cc_properties* arg_pPropData, 
                                 void* arg_pContext);

/**
 * NAME: ix_cc_ipv6_send_icmp_error
 *
 * DESCRIPTION: This library primitive helps the client to send an ICMP
 * error message.
 *
 * @Param:  - IN arg_IcmpErrType: The ICMP error type to include in the message
 * @Param:  - IN arg_IcmpErrCode: The ICMP error code to include in the message
 * @Param:  - IN arg_hBuffer: The IPv6 packet causing the error.
 * @Param:  - IN arg_Param: Contains extra information if required by the ICMP
 *                          error message type.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 *
 */
ix_error ix_cc_ipv6_send_icmp_error(
                            ix_cc_ipv6_icmp_err_type    arg_IcmpErrType,
                            ix_cc_ipv6_icmp_err_code    arg_IcmpErrCode,
                            ix_buffer_handle            arg_hBuffer,
                            ix_uint32                   arg_Param,
                            void*                       arg_pUserContext);


/**
 * NAME: ix_cc_ipv6_send_icmp_info_message
 *
 * DESCRIPTION: This library primitive helps the client to send an ICMP
 *              informational message.
 *
 * @Param:  - IN arg_SrcIpAddr: The IPv6 address to be included in the source
 *                              address field of the final IPv6 packet.
 * @Param:  - IN arg_DstAddr: The destination IPv6 address.
 * @Param:  - IN arg_Type: The type of the ICMP message. This value will be
 *                          included in the ICMP header.
 * @Param:  - IN arg_Code: The ICMP message code corresponding to the message
 *                          type.
 * @Param:  - IN arg_PayloadLength: The length of the ICMP payload.
 * @Param:  - IN arg_pPayload: The payload to include in the ICMP packet.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 *
 */
ix_error ix_cc_ipv6_send_icmp_info_message(
                                        ix_uint128  arg_SrcIpAddr,
                                        ix_uint128  arg_DstIpAddr,
                                        ix_uint8    arg_Type,
                                        ix_uint8    arg_Code,
                                        ix_uint32   arg_PayloadLength,
                                        ix_uint8    *arg_pPayload,
                                        void        *arg_pUserContext);

/**
 * NAME: ix_cc_ipv6_perform_addr_resolution
 *
 * DESCRIPTION: This library primitive helps the cllient to request
 *              address resolution to be performed for a packet destined
 *              to a neighbor.
 *
 * @Param:  - IN arg_hBuffer: The hardware buffer containing the ethernet
 *                            packet to be sent to the neighbor
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 *
 */
ix_error ix_cc_ipv6_perform_addr_resolution(
                                         ix_buffer_handle arg_hBuffer,
                                         void* arg_pUserContext);



/**
 * NAME: ix_cc_ipv6_add_neighbor
 *
 * DESCRIPTION: This library primitive helps the cllient to add a
 *              neighbor to the L2 table
 *
 * @Param:  - IN arg_L2Index: The index into the L2 table at which this
 *                            information should be added.
 * @Param   - IN arg_NbrIp:   The IPv6 address of the neighbor.
 * @Param   - IN arg_OutputPort: The egress port from which all traffic
 *                            destined for the neighbor should exit.
 * @Param   - IN arg_L2Addr: The MAC address of the neighbor.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 *
 */
ix_error ix_cc_ipv6_add_neighbor(
                                    ix_uint16 arg_L2Index,
                                    ix_uint128 arg_NbrIp,
                                    ix_uint16 arg_OutputPort,
                                    ix_uint8 *arg_L2Addr,
                                    void* arg_pUserContext);

/**
 * NAME: ix_cc_ipv6_del_neighbor
 *
 * DESCRIPTION: This library primitive helps the cllient to add a
 *              neighbor to the L2 table
 *
 * @Param:  - IN arg_L2Index: The index into the L2 table at which this
 *                            information should be added.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 *
 */
ix_error ix_cc_ipv6_del_neighbor(
                                 ix_uint16 arg_L2Index,
                                 void* arg_pUserContext);


typedef ix_uint32 *route_update_handle;

/**
 * NAME: ix_cc_ipv6_prefix_update_register
 *
 * DESCRIPTION: This primitive will allow a client to register for prefix update
 * notifications in the future. The client will be returned a handle which needs
 * to be used to inform the IPv6 CC about particular prefixes that this client
 * is interested in hearing about (see ix_cc_prefix_update_notify and 
 * ix_cc_prefix_update_cancel).
 *
 * @Param:  - IN arg_msgID: User supplied message ID to which route updates 
 * notifications will be sent.
 *
 * @Param:  - OUT arg_pHandle: Handle to be used for notifications/un-registration.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 */
ix_error ix_cc_ipv6_prefix_update_register(ix_uint32 arg_msgID, 
                                          route_update_handle *arg_pHandle);

/**
 * NAME: ix_cc_ipv6_prefix_update_unregister
 *
 * DESCRIPTION: This primitive will allow a client to un-register with the IPv6 CC
 * After this call returns, the client will no longer receive any prefix update
 * notification messages.
 *
 * @Param:  - IN arg_handle: Handle returned when the client registered for 
 * route update notifications.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 */
ix_error ix_cc_ipv6_prefix_update_unregister(route_update_handle arg_handle);

/**
 * NAME: ix_cc_ipv6_prefix_update_notify
 *
 * DESCRIPTION: This primitive will allow a client to inform the IPv6 CC about
 * specific prefixes it cares about. When the prefix is updated, the client
 * will recieve a message giving the client the new next hop ID associated with that
 * prefix/prefix_length combination. (Also see ix_cc_route_udpate_cancel).
 *
 * @Param:  - IN arg_handle: Handle returned when the client registered for 
 * route update notifications.
 * @Param:  - IN arg_prefix: The prefix that the client is interested in.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 */
ix_error ix_cc_ipv6_prefix_update_notify(route_update_handle arg_handle,
                                         ix_uint128 arg_prefix);

/**
 * NAME: ix_cc_ipv6_prefix_update_cancel
 *
 * DESCRIPTION: This primitive will allow a client cancel any previous notification
 * request for prefix updateds (see ix_cc_route_update_notify).
 *
 * @Param:  - IN arg_handle: Handle returned when the client registered for 
 * route update notifications.
 * @Param:  - IN arg_prefix: The prefix that the client is interested in.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 */
ix_error ix_cc_ipv6_prefix_update_cancel(route_update_handle arg_handle,
                                         ix_uint128 arg_prefix);



#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_IPV6_H__) */
