/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 6
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      IPv6 Forwarder Core Component - fini
 *
 * = FILENAME
 *      ix_cc_ipv6_fini.c
 *
 * = DESCRIPTION
 *      The file defines fini interface and internal functions.
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      12/03/2002
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */
#define IX_ERROR_FILE_IDENT "$Id: ix_cc_ipv6_fini.c,v 1.9 2003/11/01 00:44:42 rranjeet Exp $";


/**
 * System defined include files
 */

/**
 * User defined include files
 */

#include "ix_cc_error.h"

#include "ix_rm.h"
#include "ix_ossl.h"
#include "cc/ix_cc_rtmv6.h"

#include "ix_cc.h"
#include "bindings.h"
#include "cc/ix_cc_ipv6.h"
#include "cc/ipv6/internal/ix_cc_ipv6_init.h"
#include "cc/ipv6/internal/ix_cc_ipv6_fini.h"
#include "cc/ipv6/internal/ix_cc_ipv6_nd.h"
#include "cc/ipv6/internal/icmp6.h"


/**
 * Preprocessor symbols and Macros used in this file
 */



/**
 * Type definitions whose scope is limited to this file
 */


/**
 * Variable declarations global to this file only. Externs are followed by 
 * static variables.
 */


/** 
 * Extern function prototypes.
 */

extern ix_cc_ipv6_context* g_pIpv6Context;


/** 
 * Static function prototypes.
 */


/** 
 * Function definitions.
 */



/**
 * NAME: _ix_cc_ipv6_free_symbols
 *
 * DESCRIPTION: This primitive will be used to free memory for symbols 
 * allocated for patching.
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOM - failed to free memory for symbols.
 * 
 */

ix_error _ix_cc_ipv6_free_symbols(void)
{
    /* Free memory allocated for Statistics block */
    IX_ERROR_CRT(ix_rm_mem_free(g_pIpv6Context->pMbStatsBase),
                                IX_CC_ERROR_OOM, IX_ERROR_LEVEL_LOCAL);           
    g_pIpv6Context->pMbStatsBase = NULL;

    /* Free memory allocated for Control block */
    IX_ERROR_CRT(ix_rm_mem_free(g_pIpv6Context->pMbCbBase),
                                IX_CC_ERROR_OOM, IX_ERROR_LEVEL_LOCAL);           
    g_pIpv6Context->pMbCbBase = NULL;

    return IX_SUCCESS;

}/* end _ix_cc_ipv6_free_symbols */


/**
 * NAME: _ix_cc_ipv6_delete_64bit_counters
 *
 * DESCRIPTION: This primitive will be used to delete 64 bit counters. 
 * 
 * @Param:  - IN  none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOM_64BIT_COUNTER - failed to delete 64 bit counter.
 * 
 */

ix_error _ix_cc_ipv6_delete_64bit_counters(void)
{
#ifdef IPV6_STATS
    ix_uint32 index, portIndex, bladeIndex, counter;

    /* Incoming counters */
    for (portIndex = 0; portIndex < IX_CC_IPV6_NUMBER_OF_IN_MB_COUNTERS;
	                                                          portIndex++)
    {
	for (counter = 0; counter < IX_CC_IPV6_NUMBER_OF_IN_MB_COUNTERS;
	                                                          counter++)
        {

            IX_ERROR_CRT(ix_rm_counter_64bit_delete(
              g_pIpv6Context->inCounter64BitHandle[portIndex].handles[counter]),
              IX_CC_ERROR_OOM_64BIT_COUNTER,
              IX_ERROR_LEVEL_WARNING);
        }

    } /* end for(portIndex = 0; portIndex < IX_CC_IPV6_NUMBER_OF_IN_MB_COUNTERS;
                                                             portIndex++) */
    /* Outgoing counters */
    for (bladeIndex = 0; bladeIndex < MAX_NUMBER_OF_BLADES; bladeIndex++)
    {
        for (portIndex = 0; portIndex < MAX_NUMBER_OF_PORTS_PER_BLADE; 
	                                                       portIndex++)
        {
	    index = (MAX_NUMBER_OF_PORTS_PER_BLADE * bladeIndex) + portIndex;

	    for (counter = 0; counter < IX_CC_IPV6_NUMBER_OF_OUT_MB_COUNTERS;
	                                                       counter++)
            {
                IX_ERROR_CRT(ix_rm_counter_64bit_delete(
                 g_pIpv6Context->outCounter64BitHandle[index].handles[counter]),
                    IX_CC_ERROR_OOM_64BIT_COUNTER,
		    IX_ERROR_LEVEL_WARNING);
	        }
	    }
    }
#endif

    return IX_SUCCESS;
}/* end _ix_cc_ipv6_create_64bit_counters */

/**
 * NAME: _ix_cc_ipv6_terminate_rtm_services
 *
 * DESCRIPTION: This primitive will be used to terminate RTM services: 
 * 1. remove all the next hops and routes from RTM
 * 3. Terminate RTM by calling fini function of RTM
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_RTM - error from RTM
 * 
 */

ix_error _ix_cc_ipv6_terminate_rtm_services(void)
{
    /* Call RTM purge to delete all routes and next hops from RTM */
    IX_ERROR_CRT( ix_cc_rtmv6_purge(g_pIpv6Context->hRtm),
                 IX_CC_IPV6_ERROR_RTM,IX_ERROR_LEVEL_WARNING);

    /* Call RTM fini */
    IX_ERROR_CRT(ix_cc_rtmv6_fini(g_pIpv6Context->hRtm),
                 IX_CC_IPV6_ERROR_RTM,IX_ERROR_LEVEL_WARNING);

    return IX_SUCCESS;
}/* end _ix_cc_ipv6_terminate_rtm_services */



/**
 * NAME: _ix_cc_ipv6_unregister_message_handler
 *
 * DESCRIPTION: This primitive will be used to unregister message handler. 
 * It calls CCI services in level 1 system and resore manager services in
 * level 0 system.
 *
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_CCI - error from CCI
 * 
 */

ix_error _ix_cc_ipv6_unregister_message_handler(void)
{
    /* Call CCI if CCI is enabled */

#if defined(IX_EXCLUDE_CCI)
        IX_ERROR_CRT(ix_rm_message_handler_unregister(IX_CC_IPV6_MSG_INPUT),
                     IX_CC_IPV6_ERROR_CCI,
                     IX_ERROR_LEVEL_WARNING);
#else
        IX_ERROR_CRT(ix_cci_cc_remove_message_handler(IX_CC_IPV6_MSG_INPUT),
                     IX_CC_IPV6_ERROR_CCI,
                     IX_ERROR_LEVEL_WARNING);
#endif

    return IX_SUCCESS;
}/* end _ix_cc_ipv6_unregister_message_handler */



/**
 * NAME: _ix_cc_ipv6_unregister_cci_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to unregister packet handler with CCI. 
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_CCI - error from CCI
 * 
 */

ix_error _ix_cc_ipv6_unregister_cci_packet_handlers(void)
{

#if !defined(IX_EXCLUDE_CCI)
    IX_ERROR_CRT(ix_cci_cc_remove_message_handler(
                                IX_CC_IPV6_PKT_MICROBLOCK_HIGH_PRIORITY_INPUT),
                     IX_CC_IPV6_ERROR_CCI,
                     IX_ERROR_LEVEL_WARNING);
    

    IX_ERROR_CRT(ix_cci_cc_remove_packet_handler(
                                IX_CC_IPV6_PKT_MICROBLOCK_LOW_PRIORITY_INPUT),
                     IX_CC_IPV6_ERROR_CCI,
                     IX_ERROR_LEVEL_WARNING);

    IX_ERROR_CRT(ix_cci_cc_remove_packet_handler(
                                IX_CC_IPV6_PKT_STKDRV_INPUT),
                     IX_CC_IPV6_ERROR_CCI,
                     IX_ERROR_LEVEL_WARNING);

    IX_ERROR_CRT(ix_cci_cc_remove_packet_handler(
                                IX_CC_IPV6_PKT_COMMON_INPUT),
                     IX_CC_IPV6_ERROR_CCI,
                     IX_ERROR_LEVEL_WARNING);
#endif

    return IX_SUCCESS;

}/* end _ix_cc_ipv6_unregister_cci_packet_handlers */


/**
 * NAME: _ix_cc_ipv6_unregister_rm_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to unregister packet handler with 
 * resource manager
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_CCI - error from RM
 * 
 */

ix_error _ix_cc_ipv6_unregister_rm_packet_handlers(void)
{

    IX_ERROR_CRT(ix_rm_message_handler_unregister(
                               IX_CC_IPV6_PKT_MICROBLOCK_HIGH_PRIORITY_INPUT),
                     IX_CC_IPV6_ERROR_CCI,
                     IX_ERROR_LEVEL_WARNING);

    IX_ERROR_CRT(ix_rm_packet_handler_unregister(
                               IX_CC_IPV6_PKT_MICROBLOCK_LOW_PRIORITY_INPUT),
                     IX_CC_IPV6_ERROR_CCI,
                     IX_ERROR_LEVEL_WARNING);

    IX_ERROR_CRT(ix_rm_packet_handler_unregister(
                               IX_CC_IPV6_PKT_STKDRV_INPUT),
                     IX_CC_IPV6_ERROR_CCI,
                     IX_ERROR_LEVEL_WARNING);


    IX_ERROR_CRT(ix_rm_packet_handler_unregister(
                               IX_CC_IPV6_PKT_COMMON_INPUT),
                     IX_CC_IPV6_ERROR_CCI,
                     IX_ERROR_LEVEL_WARNING);
    return IX_SUCCESS;

}/* end _ix_cc_ipv6_unregister_rm_packet_handlers */



/**
 * NAME: _ix_cc_ipv6_unregister_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to unregister packet handler 
 * with CCI/Resource manager
 *
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_CCI - error from CCI/RM
 * 
 */

ix_error _ix_cc_ipv6_unregister_packet_handlers(void)
{
 
    /* Call CCI if CCI is enabled */
#if defined(IX_EXCLUDE_CCI)
        IX_ERROR_CR(_ix_cc_ipv6_unregister_rm_packet_handlers());
#else
        IX_ERROR_CR(_ix_cc_ipv6_unregister_cci_packet_handlers());
#endif

    return IX_SUCCESS;

}/* end _ix_cc_ipv6_unregister_packet_handlers */


/**
 * NAME: _ix_cc_ipv6_delete_event_handler
 *
 * DESCRIPTION: This primitive will be used to delete event handler.
 * User needs to implement this function in level0 system.
 *
 * @Param:  - IN arg_pEventId - id of ICMP event hanlder
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_UNIMPL - not implemented.
 * 
 */

ix_error _ix_cc_ipv6_delete_event_handler(ix_ossl_thread_t arg_pEventId)
{


    return IX_ERROR_WARNING(IX_CC_ERROR_UNIMPL,("No Event Handler for Icmp\n"));
}/* end _ix_cc_ipv6_delete_event_handler */


/**
 * NAME: _ix_cc_ipv6_delete_icmp_event_handler
 *
 * DESCRIPTION: This primitive will be used to delete icmp event handler. 
 * It calls CCI delete event handler function to delete an event handler 
 * in level1 system and calls _ix_cc_ipv6_delete_event_handler in level 0 system
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_CCI - error from delete event hanler
 * 
 */

ix_error _ix_cc_ipv6_delete_icmp_event_handler(void)
{

#ifdef NOT_YET
        
#if defined(IX_EXCLUDE_CCI) /* compile time flag */
    IX_ERROR_CR(_ix_cc_ipv6_delete_event_handler(g_pIpv6Context->icmpThreadId));
#else 
    IX_ERROR_CRT(ix_cci_cc_remove_event_handler(g_pIpv6Context->hIpv6Handle,
                                             g_pIpv6Context->hEventHandle),
                      IX_CC_IPV6_ERROR_CCI,
                      IX_ERROR_LEVEL_WARNING);
#endif

#endif

    return IX_SUCCESS;
}/* end _ix_cc_ipv6_delete_icmp_event_handler */


/**
 * NAME: ix_cc_ipv6_fini
 *
 * DESCRIPTION: This primitive shall be called to terminate services from 
 * IPv6 forwarder component. This primitive frees memory allocated during 
 * initialization, shut down RTM, free all the created resources: delete 
 * 64 bit counter. If client claims for any services from IPv6 after calling 
 * this function, then the behavior is undefined.
 * 
 * @Param:  - IN arg_hCcHandle - handle to IPv6 Forwader core component
 *
 * @Param:  - IN arg_pContext - pointer to the control block memory allocated 
 * earlier in ix_cc_ipv6_init function. 
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_INVALID_INPUT_PARAM - invalid input parameter(s)
 * IX_CC_ERROR_OOM - failure of memory deletion                
 * IX_CC_ERROR_OOM_64BIT_COUNTER - failure of  64 bit counter deletion 
 * IX_CC_IPV6_ERROR _RTM - failure from RTM
 * IX_CC_IPV6_ERROR_CCI - failure from CCI
 * 
 */

ix_error ix_cc_ipv6_fini(ix_cc_handle arg_hCcHandle,
                         void* arg_pContext)
{

    /* Validate Parameters */
    if(arg_pContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                            ("arg_pContext is null"));

    }/* if(arg_pContext == NULL)  */ 


    if((arg_hCcHandle != g_pIpv6Context->hIpv6Handle) ||
       (ix_cc_ipv6_context *)arg_pContext != g_pIpv6Context)
    {
        return IX_ERROR_WARNING(IX_CC_IPV6_ERROR_INVALID_INPUT_PARAM,
                            ("Invalid input Parameters"));
    }/* if(arg_hCcHandle != g_pIpv6Context->hIpv6Handle ||
       (ix_cc_ipv6_context *)arg_pContext != g_pIpv6Context) */ 
  
    /* Terminate RTM services*/
    IX_ERROR_CR(_ix_cc_ipv6_terminate_rtm_services());  

    /* Free memory for symbols */
    IX_ERROR_CR(_ix_cc_ipv6_free_symbols());   

    /* Delete 64 bit counter */
    IX_ERROR_CR(_ix_cc_ipv6_delete_64bit_counters());
    
    /* Unregister Message Handler */
    IX_ERROR_CR(_ix_cc_ipv6_unregister_message_handler());
  
    /* Unregister Packet Handler */
    IX_ERROR_CR(_ix_cc_ipv6_unregister_packet_handlers());
   
    /* Remove ICMP event handler */
    /*IX_ERROR_CR(_ix_cc_ipv6_delete_icmp_event_handler());*/
    
    /*Terminate ND services*/
    ix_cc_nd_fini();
    
    /*Terminate ICMP services*/
    _ix_cc_ipv6_icmp_fini();

    /* Free mutex */
    IX_ERROR_CR(ix_ossl_mutex_fini(g_pIpv6Context->ipv6Mutex));

    /* Free IPv6 Context */ 
    ix_ossl_free(g_pIpv6Context);
        
    return IX_SUCCESS;

} /* end ix_cc_ipv6_fini */
