/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.

 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor
 *
 * 
 *
 * = MODULE
 *     IPv6 Core Component - ICMP module - Neighbor DIscovery Messages
 *
 * = FILENAME
 *      ix_cc_ipv6_icmp_nd.c
 *
 *
 * = DESCRIPTION
 *   This file contains the implementation of ICMP message building 
 *   functions required for neighbor discovery.
 *   The Neighbor Discovery ICMP messages supported are Router Advertisement,
 *   Neighbor Solicitation and Neighbor Advertisement.
 * 
 *
 *
 * = CREATION TIME
 *      6/15/2002 5:56:24 PM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */


static const char s_FileIdentifier[] = "$Id: ix_cc_ipv6_icmp_nd.c,v 1.12 2003/11/01 00:56:48 rranjeet Exp $";

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
#include "ix_ossl.h"
#include "cc/ipv6/internal/v6_header_defs.h"
#include "cc/ipv6/internal/icmp6.h"
#include "cc/ipv6/internal/ix_cc_ipv6_icmp_nd.h"
#include "ix_error.h"
#else
#include "cc/ipv6/internal/v6_header_defs.h"
#include "cc/ipv6/internal/icmp6.h"
#include "cc/ipv6/internal/ix_cc_ipv6_icmp_nd.h"
#include "ix_error.h"
#endif

/*
 * ix_cc_ipv6_icmp_send_rtr_adv
 */
ix_error ix_cc_ipv6_icmp_send_rtr_adv(
                        ix_ipv6_addr    arg_SrcAddr,
                        ix_ipv6_addr    arg_DstAddr,
                        ix_ipv6_nd_rtr_adv_params *arg_Params,
                        ix_uint16       arg_OutPutPort,
                        ix_uint32       arg_bladeId)
{
    ix_error    errVal;
    ix_uint8    rtrAdvBuffer[IX_CC_ICMPV6_PAYLOAD_LIMIT];
    ix_uint32   messageLength;
    ix_ipv6_nd_rtr_adv_msg      *pRtrAdvMsg;
    ix_ipv6_nd_options_l2_addr  *pL2Options;
    ix_buffer_handle hBuf = IX_NULL_BUFFER_HANDLE;
    ix_hw_buffer_meta *pHwMeta;

    if(!arg_Params)
        return IX_CC_ERROR_NULL; 

    /*Populate the main router advertisement message*/
    pRtrAdvMsg = (ix_ipv6_nd_rtr_adv_msg *)rtrAdvBuffer;
    pRtrAdvMsg->curhoplimit = arg_Params->nextHopLmit;
    pRtrAdvMsg->managed = arg_Params->managed;
    pRtrAdvMsg->otherconfig = arg_Params->otherConfig;
    pRtrAdvMsg->rtrlifetime = arg_Params->rtrLifetime;
    pRtrAdvMsg->reachabletime = arg_Params->reachableTime;
    pRtrAdvMsg->retranstimer = arg_Params->retransmitTimer;    
    
    messageLength = sizeof(ix_ipv6_nd_rtr_adv_msg);

    /*Populate the source address options if applicable*/
    if(arg_Params->l2AddrLength != 0)
    {
        pL2Options = (ix_ipv6_nd_options_l2_addr *) 
                                                (rtrAdvBuffer + messageLength);
        
        pL2Options->type = IX_CC_IPV6_ND_OPT_SRC_L2_ADDR;
        /*Calculate length in multiples of 8 octets*/
        pL2Options->length = arg_Params->l2AddrLength / 8;
        if(arg_Params->l2AddrLength % 8)
            pL2Options->length++;
	ix_ossl_memcpy(pL2Options->linkLayerAddr, arg_Params->l2Addr, 6);

        messageLength += sizeof(ix_ipv6_nd_options_l2_addr);
    }


    IX_ERROR_CR(
             _ix_cc_ipv6_icmp_build_info_message_ex(arg_SrcAddr, 
                                                 arg_DstAddr,
                                                 IX_CC_IPV6_ICMP_TYPE_RTR_ADV, 
                                                 IX_CC_ICMPV6_CODE_ZERO,
                                                 messageLength, 
                                                 rtrAdvBuffer, 
                                                 arg_OutPutPort, &hBuf));
    if(hBuf == IX_NULL_BUFFER_HANDLE)
    {
        return IX_CC_ERROR_NULL;
    }

    IX_IPV6_ERROR_CGT(ix_rm_buffer_get_meta(hBuf, (void **)&pHwMeta),
                                 errVal, LBL_ERR);
    pHwMeta->m_NextHopID = 0;
    pHwMeta->m_OutputPort = IX_HTON16(arg_OutPutPort);
    pHwMeta->m_FabricPort = (ix_uint8)arg_bladeId;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Sending Packet to Queue Manager\n");
#endif /*IPV6_DEBUG*/

   errVal = ix_rm_packet_send(IX_CC_IPV6_PKT_QM_OUTPUT, hBuf, 0);

#if defined(IPV6_DEBUG)
    if(errVal != IX_SUCCESS)
        ix_ossl_message_log("Queue Manager returned an error: %ld\n", errVal);
    else
        ix_ossl_message_log("Successfully sent the packet to Queue Manager\n");
#endif /*IPV6_DEBUG*/

    return errVal;

LBL_ERR:
    ix_rm_buffer_free_chain(hBuf);
    return errVal;
}

ix_error ix_cc_ipv6_icmp_send_nbr_adv(
                        ix_ipv6_addr    arg_SrcAddr,
                        ix_ipv6_addr    arg_DstAddr,
                        ix_ipv6_nd_nbr_adv_params *arg_Params,
                        ix_uint16       arg_OutPutPort,
                        ix_uint32       arg_bladeId)
{
    ix_uint8    nbrAdvBuffer[IX_CC_ICMPV6_PAYLOAD_LIMIT + IX_CC_IPV6_ETH_HDR_LEN];
    ix_uint32   messageLength;
    ix_ipv6_icmp_nbr_adv_msg      *pNbrAdvMsg;
    ix_ipv6_nd_options_l2_addr  *pL2Options;
    ix_buffer_handle hBuf = IX_NULL_BUFFER_HANDLE;
    ix_hw_buffer_meta *pHwMeta;
    void *pData;
    ix_error errVal;
    ix_uint32 pktSize;
    ix_uint32 cell_count=0;

    if(!arg_Params)
        return IX_CC_ERROR_NULL; 

    /*Populate the main nbr advertisement message*/
    pNbrAdvMsg = (ix_ipv6_icmp_nbr_adv_msg *)nbrAdvBuffer;
    pNbrAdvMsg->router = arg_Params->isRouter;
    pNbrAdvMsg->solicited = arg_Params->solicited;
    pNbrAdvMsg->override = arg_Params->override;
    ix_ossl_memcpy(&pNbrAdvMsg->targetAddr, &arg_Params->targetAddr,
            IX_CC_IPV6_ADDRESS_LENGTH_BYTES );
    
    messageLength = sizeof(ix_ipv6_icmp_nbr_adv_msg);

    /*Populate the source address options if applicable*/
    if(arg_Params->l2AddrLength != 0)
    {
        pL2Options = (ix_ipv6_nd_options_l2_addr *) 
                                                (nbrAdvBuffer + messageLength);
        
        pL2Options->type = IX_CC_IPV6_ND_OPT_DST_L2_ADDR;
        /*Calculate length in multiples of 8 octets*/
        pL2Options->length = arg_Params->l2AddrLength / 8;
        if(arg_Params->l2AddrLength % 8)
            pL2Options->length++;
	ix_ossl_memcpy(pL2Options->linkLayerAddr, arg_Params->l2Addr, 6);

        messageLength += sizeof(ix_ipv6_nd_options_l2_addr);
    }

    IX_ERROR_CR(
          _ix_cc_ipv6_icmp_build_info_message_ex(arg_SrcAddr, 
                                              arg_DstAddr,
                                              IX_CC_IPV6_ICMP_TYPE_NBR_ADV, 
                                              IX_CC_ICMPV6_CODE_ZERO,
                                              messageLength, 
                                              nbrAdvBuffer,
                                              arg_OutPutPort, &hBuf));
   if(hBuf == IX_NULL_BUFFER_HANDLE)
   {
       return IX_CC_ERROR_NULL;
   }

   IX_IPV6_ERROR_CGT(ix_rm_buffer_get_meta(hBuf, (void **)&pHwMeta),
                                 errVal, LBL_ERR);
   pHwMeta->m_OutputPort = IX_HTON16(arg_OutPutPort);
   pHwMeta->m_FabricPort = (ix_uint8)arg_bladeId;

   /*If the mac header is specified, append it to the packet*/
   if(arg_Params->macHdrSpecified)
   {
#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Appending Ethernet Header to the IPv6 packet\n");
#endif /*IPV6_DEBUG*/

       pHwMeta->m_Offset = IX_HTON16(IX_NTOH16(pHwMeta->m_Offset) - 
                                                                IX_CC_IPV6_ETH_HDR_LEN);     
       IX_IPV6_ERROR_CGT(ix_cc_hw_get_packet_data(hBuf, &pData), errVal, LBL_ERR);
       ix_ossl_memcpy(pData, &(arg_Params->macHdr), IX_CC_IPV6_ETH_HDR_LEN);

       pktSize = IX_NTOH16(pHwMeta->m_PacketSize);
       pHwMeta->m_PacketSize = IX_HTON16(pktSize + IX_CC_IPV6_ETH_HDR_LEN);
       pHwMeta->m_BufferSize = IX_HTON16(IX_NTOH16(pHwMeta->m_BufferSize) +
                                                                IX_CC_IPV6_ETH_HDR_LEN);
       /* Reset cell count */
       cell_count = ix_cc_calculate_cell_count(pktSize, 1);
       ix_rm_buffer_cell_count_set(&hBuf, cell_count);

       pHwMeta->m_NextHopID = -1;
   }
   else
   {
       pHwMeta->m_NextHopID = 0;
   }
#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Sending Packet to Queue Manager\n");
#endif /*IPV6_DEBUG*/

   errVal = ix_rm_packet_send(IX_CC_IPV6_PKT_QM_OUTPUT, hBuf, 0);

#if defined(IPV6_DEBUG)
    if(errVal != IX_SUCCESS)
        ix_ossl_message_log("Queue Manager returned an error: %ld\n", errVal);
    else
        ix_ossl_message_log("Successfully sent the packet to Queue Manager\n");
#endif /*IPV6_DEBUG*/

   return errVal;

LBL_ERR:
    ix_rm_buffer_free_chain(hBuf);
    return errVal;
}
          
/**
 * TYPENAME: ix_cc_ipv6_icmp_send_nbr_sol
 *
 */
ix_error ix_cc_ipv6_icmp_send_nbr_sol(ix_ipv6_addr  arg_SrcAddr,
                                    ix_ipv6_addr    arg_DstAddr,
                                    ix_ipv6_addr    arg_TargetAddrParam,
                                    ix_uint8        *arg_pL2OptParam, 
                                    ix_uint16       arg_OutPutPort,
                                    ix_uint32       arg_bladeId)

{
    ix_uint8    nbrSolBuffer[IX_CC_ICMPV6_PAYLOAD_LIMIT];
    ix_uint32   messageLength = 4;
    ix_ipv6_nd_options_l2_addr  *pL2Options;
    ix_ipv6_addr    tempAddr;
    ix_buffer_handle hBuf = IX_NULL_BUFFER_HANDLE;
    ix_hw_buffer_meta *pHwMeta;
    ix_error errVal;

    ix_ossl_memset(nbrSolBuffer, 0, messageLength);

#if defined(IPV6_DEBUG)
    ix_ossl_message_log ("The address being resolved is %lx %lx %lx %lx \n", 
                                          arg_TargetAddrParam.aUint32[0],
                                          arg_TargetAddrParam.aUint32[1],
                                          arg_TargetAddrParam.aUint32[2],
                                          arg_TargetAddrParam.aUint32[3]);
    ix_ossl_message_log ("Generating NBR sol for Port 0x%x  on blade 0x%lx\n", 
    			   arg_OutPutPort, arg_bladeId);          
#endif /* IPV6_DEBUG */

    /* Copy the address we're sending this NBR Sol for */
    ix_ossl_memcpy((void*)(nbrSolBuffer + messageLength), 
                   &arg_TargetAddrParam, 
                   IX_CC_IPV6_ADDRESS_LENGTH_BYTES);

    messageLength += IX_CC_IPV6_ADDRESS_LENGTH_BYTES;

    /*
     * The source address option can only be used if the 
     * source IPv6 address is NOT unspecified
     */
    ix_ossl_memset(&tempAddr, 0, IX_CC_IPV6_ADDRESS_LENGTH_BYTES);

    /* 
     * If the source IPv6 address is not "unspecified" and there is a target
     * option, copy the target optional parameter in the NBR 
     * Sol message 
     */
    if(memcmp(&arg_SrcAddr, &tempAddr, IX_CC_IPV6_ADDRESS_LENGTH_BYTES) && 
                                                             arg_pL2OptParam)
    {
        pL2Options = (ix_ipv6_nd_options_l2_addr *) 
                                               (nbrSolBuffer + messageLength);
        pL2Options->type = IX_CC_IPV6_ND_OPT_SRC_L2_ADDR;
        pL2Options->length = 1;
        ix_ossl_memcpy(pL2Options->linkLayerAddr, arg_pL2OptParam, 
                                                   IX_CC_IPV6_MAC_ADDR_LEN);
        messageLength += sizeof(ix_ipv6_nd_options_l2_addr);
    }

    /* Build a regular ICMP packet and send it out */
    IX_ERROR_CR(_ix_cc_ipv6_icmp_build_info_message_ex(arg_SrcAddr, arg_DstAddr,
                                                IX_CC_IPV6_ICMP_TYPE_NBR_SOL, 
                                                IX_CC_ICMPV6_CODE_ZERO, 
                                                messageLength, 
                                                nbrSolBuffer, 
                                                arg_OutPutPort, &hBuf));

   if(hBuf == IX_NULL_BUFFER_HANDLE)
   {
       return IX_CC_ERROR_NULL;
   }

   IX_IPV6_ERROR_CGT(ix_rm_buffer_get_meta(hBuf, (void **)&pHwMeta),
                                 errVal, LBL_ERR);
    pHwMeta->m_NextHopID = 0;
    pHwMeta->m_OutputPort = IX_HTON16(arg_OutPutPort);
    pHwMeta->m_FabricPort = (ix_uint8)arg_bladeId;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Sending NBR SOL Packet to Queue Manager\n");
#endif /*IPV6_DEBUG*/

    errVal = ix_rm_packet_send(IX_CC_IPV6_PKT_QM_OUTPUT, hBuf, 0);

#if defined(IPV6_DEBUG)
    if(errVal != IX_SUCCESS)
        ix_ossl_message_log("Queue Manager returned an error: %ld\n", errVal);
    else
        ix_ossl_message_log("Successfully sent the packet to Queue Manager\n");
#endif /*IPV6_DEBUG*/

    return errVal;

LBL_ERR:

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("ICMP ND: Failed to build Nbr Sol message\n");
#endif /* IPV6_DEBUG */

    ix_rm_buffer_free_chain(hBuf);
    return errVal;
}
