/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 6
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      IPv6 Forwarder Core Component Initialization Module
 *
 * = FILENAME
 *      ix_cc_ipv6_init.c
 *
 * = DESCRIPTION
 *      The file defines for init interface and internal functions.
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      12/03/2002
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */
#define IX_ERROR_FILE_IDENT "$Id: ix_cc_ipv6_init.c,v 1.28 2003/11/06 17:05:40 ktseng Exp $";

/**
 * System defined include files
 */

/**
 * User defined include files
 */
#include "ix_cc_error.h"

#include "ix_rm.h"
#include "ix_ossl.h"


#include "cc/ix_cc_rtmv6.h"

#include "ix_cc.h"
#include "ix_cc_macros.h"
#include "ix_config.h"
#include "bindings.h"
#include "ipv6_fwder_uc.h"
#include "cc/ix_cc_ipv6.h"
#include "cc/ipv6/internal/ix_cc_ipv6_init.h"
#include "cc/ipv6/internal/ix_cc_ipv6_fini.h" 
#include "cc/ipv6/internal/ix_cc_ipv6_prop.h"
#include "cc/ipv6/internal/icmp6.h"
#include "cc/ipv6/internal/ix_cc_ipv6_nd.h"

/**
 * Preprocessor symbols and Macros used in this file
 */

/**
 * Type definitions whose scope is limited to this file
 */

/**
 * Variable declarations global to this file only. Externs are followed by 
 * static variables.
 */

ix_cc_ipv6_context *g_pIpv6Context;
ix_cc_ipv6_registry_data g_Ipv6RegistryData;

extern ix_uint16 g_Icmpv6SleepTime; /* To set ICMP event handler sleep time */

/** 
 * Extern function prototypes.
 */


/** 
 * Static function prototypes.
 */


/** 
 * Function definitions.
 */


/**
 * NAME: _ix_cc_ipv6_retrieve_registry_data
 *
 * DESCRIPTION: This primitive will be used to retrieve data from registry 
 * for a given property type.
 * 
 * @Param:  - IN arg_pPropertyName - pointer to property name string 
 *
 * @Param:  - OUT arg_pPropertyValue - pointer to property value 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_REGISTRY  - failed to access registry
 * 
 */

ix_error _ix_cc_ipv6_retrieve_registry_data(const char* arg_pPropertyName,
                                            ix_uint32* arg_pPropertyValue)
{
    ix_configuration_property_handle hIpv6PropertyHandle; 

    /* Get static data from registry */    
    IX_ERROR_CRT(ix_rm_cp_property_open(IX_CP_CORE_PROPERTY_ROOT,
                                        arg_pPropertyName,
                                        &hIpv6PropertyHandle),
                 IX_CC_IPV6_ERROR_REGISTRY,
                 IX_ERROR_LEVEL_WARNING);

    IX_ERROR_CRT(ix_rm_cp_property_get_value_uint32(hIpv6PropertyHandle,
                                                   arg_pPropertyValue),
                 IX_CC_IPV6_ERROR_REGISTRY,
                 IX_ERROR_LEVEL_WARNING);

    IX_ERROR_CRT(ix_rm_cp_property_close(hIpv6PropertyHandle),
                 IX_CC_IPV6_ERROR_REGISTRY,
                 IX_ERROR_LEVEL_WARNING);

    return IX_SUCCESS;
}/* end _ix_cc_ipv6_retrieve_registry_data */


/**
 * NAME: _ix_cc_ipv6_registry
 *
 * DESCRIPTION: This primitive will be used to retreive data from registry for
 * all IPV6 static data and stored in ipv6RegistyData 
 *           .
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT  none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_REGISTRY  - not able to access registry.
 * 
 */

ix_error _ix_cc_ipv6_registry()
{
       
    /* Get route table size from registry */
    IX_ERROR_CR(_ix_cc_ipv6_retrieve_registry_data(
                         IX_CC_IPV6_REGISTRY_KEY_ROUTE_TABLE_SIZE,
                         &g_Ipv6RegistryData.rtmConfig.routeTableSizeHint));

   /* Get route table type from registry */
    IX_ERROR_CR(_ix_cc_ipv6_retrieve_registry_data(
                         IX_CC_IPV6_REGISTRY_KEY_ROUTE_TABLE_TYPE,
                         (ix_uint32 *)&g_Ipv6RegistryData.rtmConfig.lkupType));

    /* Get RTM lookup memory type from registry */
    IX_ERROR_CR(_ix_cc_ipv6_retrieve_registry_data(
                      IX_CC_IPV6_REGISTRY_KEY_RTM_LOOKUP_MEMORY_TYPE,
                      (ix_uint32 *)&g_Ipv6RegistryData.rtmConfig.lkupMemType));

     /* Get RTM lookup memory channel number from registry */
    IX_ERROR_CR(_ix_cc_ipv6_retrieve_registry_data(
                      IX_CC_IPV6_REGISTRY_KEY_RTM_LOOKUP_CHANNEL,
                      &g_Ipv6RegistryData.rtmConfig.lkupChannel));

#ifdef SPLIT_V6_TRIE_TABLE

     /* Get RTM lookup memory channel number from registry */
    IX_ERROR_CR(_ix_cc_ipv6_retrieve_registry_data(
                      IX_CC_IPV6_REGISTRY_KEY_RTM_LOOKUP_CHANNEL_16BIT_TRIE,
                      &g_Ipv6RegistryData.rtmConfig.lkupChannel16BitTrie));

#endif /* SPLIT_V6_TRIE_TABLE */

    /* Get next hop table size from registry */
    IX_ERROR_CR(_ix_cc_ipv6_retrieve_registry_data(
                      IX_CC_IPV6_REGISTRY_KEY_NEXT_HOP_TABLE_SIZE,
                      &g_Ipv6RegistryData.rtmConfig.nhdbSizeHint));

    /* Get next hop data base memory type from registry */
    IX_ERROR_CR(_ix_cc_ipv6_retrieve_registry_data(
                      IX_CC_IPV6_REGISTRY_KEY_NEXT_HOP_DATABASE_MEMORY_TYPE ,
                      (ix_uint32 *)&g_Ipv6RegistryData.rtmConfig.nhdbMemType));
 
    /* Get next hop data base memory channel from registry */
    IX_ERROR_CR(_ix_cc_ipv6_retrieve_registry_data(
                          IX_CC_IPV6_REGISTRY_KEY_NEXT_HOP_DATABASE_CHANNEL ,
                          &g_Ipv6RegistryData.rtmConfig.nhdbChannel));

    /* Get micro engine numbers from registry */
    IX_ERROR_CR(_ix_cc_ipv6_retrieve_registry_data(
                          IX_CC_IPV6_REGISTRY_KEY_ME_FOR_IPV6,
                          &g_Ipv6RegistryData.meNumbers));
   
    return IX_SUCCESS;
}/* end _ix_cc_ipv6_registry */


/**
 * NAME: _ix_cc_ipv6_alloc_symbols
 *
 * DESCRIPTION: This primitive will be used to allocate memory for symbols 
 * to be patched. 
 * 
 * @Param:  - IN - none
 *
 * @Param:  - OUT  - none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOM  - not able to allocate memory
 * 
 */

ix_error _ix_cc_ipv6_alloc_symbols()
{
    ix_error err = IX_SUCCESS;

    ix_uint32 inMemSize;
    ix_uint32 outMemSize;

    /* Memory required for incoming interfaces */
    inMemSize = (IX_CC_IPV6_TOTAL_NUMBER_OF_IN_MB_COUNTERS * 
                         sizeof(ix_uint32) * MAX_NUMBER_OF_PORTS_PER_BLADE);

    /* Memory required for outgoing interfaces */
    outMemSize = (IX_CC_IPV6_TOTAL_NUMBER_OF_OUT_MB_COUNTERS * 
                         sizeof(ix_uint32) * IPV6_TOTAL_NUMBER_OF_PORTS);

    /* Allocate memory in SRAM to maintain incoming/outgoing stat counters */
    err = ix_rm_mem_alloc(IX_MEMORY_TYPE_SRAM,
                          0,
                          (inMemSize + outMemSize),
                          (void**)&g_pIpv6Context->pMbStatsBase);

    if (err != IX_SUCCESS)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM, 
	          ("Memory allocation for ipv6 stats block failed \n"));
    }

    ix_ossl_memset(g_pIpv6Context->pMbStatsBase, 0, (inMemSize + outMemSize));       
    /* Allocate space for the IPv6 Control Block */
    err = ix_rm_mem_alloc(IX_MEMORY_TYPE_SRAM,
                          0,
			  IX_CC_IPV6_SIZE_OF_CONTROL_BLOCK,
			  (void **)&g_pIpv6Context->pMbCbBase);
    if (err != IX_SUCCESS)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM, 
	          ("Memory allocation for ipv6 control block failed \n"));
    }

    /* Allocate space to the IPv6 Router Prefix */
    err = ix_rm_mem_alloc(IX_MEMORY_TYPE_SRAM,
                          0,
			  IX_CC_IPV6_SIZE_OF_ROUTER_PREFIX,
			  (void **)&g_pIpv6Context->pRouterPrefix);
    if (err != IX_SUCCESS)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,
	          ("Memory allocation for ipv6 router prefix failed\n"));
    }

    return IX_SUCCESS;

} /* end _ix_cc_ipv6_alloc_symbols */


/**
 * NAME: _ix_cc_ipv6_patch_symbols
 *
 * DESCRIPTION: This primitive will be used to patch symbols to ipv6 
 * microblocks. 
 * 
 * @Param:  - IN  none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * IX_CC_IPV6_ERROR_RTM - error from RTM module
 * IX_CC_IPV6_ERROR_FAILED_PATCHING - failed patching by calling resource 
 * manager function
 */

ix_error _ix_cc_ipv6_patch_symbols()
{
    ix_cc_rtmv6_symbols rtmSymbols;
    ix_imported_symbol ipv6Symbols[IX_CC_IPV6_MAX_NUMBER_OF_SYMBOLS]; 
    ix_uint32 ipv6Index;

    ix_memory_type memType;
    ix_uint32 memChannel;
    ix_uint32 memOffset;
    void* pPhysAddr;
    ix_uint32 i = 0;
    
    /* Get RTM symbols */
    IX_ERROR_CRT(ix_cc_rtmv6_get_symbols(g_pIpv6Context->hRtm, 
                                         &rtmSymbols),
                 IX_CC_IPV6_ERROR_RTM,
                 IX_ERROR_LEVEL_WARNING);

#if (_IX_HARDWARE_TYPE_ == _IX_HW_2800_)

#ifdef SPLIT_V6_TRIE_TABLE
    /* Address of the base of the trie table 8 */
    ipv6Symbols[i].m_Value = rtmSymbols.lkupTableData1;
    ipv6Symbols[i].m_Name = IX_CC_IPV6_PATCHNAME_TRIE_TABLE_8;
    i++;

    /* Address of the base of the trie table 16 */
    ipv6Symbols[i].m_Value = rtmSymbols.lkupTableID;
    ipv6Symbols[i].m_Name = IX_CC_IPV6_PATCHNAME_TRIE_TABLE_16;
    i++;

    /* Address of the base of the next hop table */
    ipv6Symbols[i].m_Value = rtmSymbols.nextHopBase;
    ipv6Symbols[i].m_Name = IX_CC_IPV6_PATCHNAME_NEXTHOP_TABLE;
    i++;
#else /* SPLIT_V6_TRIE_TABLE */
    /* Address of the base of the trie table 16 */
    ipv6Symbols[i].m_Value = rtmSymbols.lkupTableID;
    ipv6Symbols[i].m_Name = IX_CC_IPV6_PATCHNAME_TRIE_TABLE;
    i++;

    /* Address of the base of the next hop table */
    ipv6Symbols[i].m_Value = rtmSymbols.nextHopBase;
    ipv6Symbols[i].m_Name = IX_CC_IPV6_PATCHNAME_NEXTHOP_TABLE;
    i++;
#endif /* SPLIT_V6_TRIE_TABLE */

#else /* (_IX_HARDWARE_TYPE_ != _IX_HW_2800_) */

    /* Address containing the Local Blade ID */
    ipv6Symbols[i].m_Value = g_Ipv6RegistryData.localBladeId;
    ipv6Symbols[i].m_Name = IX_CC_IPV6_PATCHNAME_ID;
    i++;

    /* Address of the base of the trie table */
    ipv6Symbols[i].m_Value = rtmSymbols.lkupTableID;
    ipv6Symbols[i].m_Name = IX_CC_IPV6_PATCHNAME_TRIE_TABLE;
    i++;

    /* Address of the base of the next hop table */
    ipv6Symbols[i].m_Value =rtmSymbols.nextHopBase;
    ipv6Symbols[i].m_Name = IX_CC_IPV6_PATCHNAME_NEXTHOP_TABLE;
    i++;

#endif /* (_IX_HARDWARE_TYPE_ != _IX_HW_2800_) */

    IX_ERROR_CRT(ix_rm_get_phys_offset(g_pIpv6Context->pRouterPrefix,
                                       &memType,
				       &memChannel,
				       &memOffset,
				       &pPhysAddr),
                  IX_CC_IPV6_ERROR_FAILED_PATCHING,
		  IX_ERROR_LEVEL_WARNING);

    if (memType == IX_MEMORY_TYPE_SRAM)
    {
        IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(memChannel, memOffset);
    } /* end if memory type */

    /* Address of the Stats Table */
    ipv6Symbols[i].m_Value = memOffset; 
    ipv6Symbols[i].m_Name = IX_CC_IPV6_PATCHNAME_ROUTER_PREFIX;
    i++;

#ifdef IPV6_STATS
   
    /* 
     * Patch base address for the statistics block - 
     * get the physical offset of memory
     */
    IX_ERROR_CRT(ix_rm_get_phys_offset(g_pIpv6Context->pMbStatsBase,
                                       &memType,
                                       &memChannel,
                                       &memOffset,
                                       &pPhysAddr),
                 IX_CC_IPV6_ERROR_FAILED_PATCHING,
                 IX_ERROR_LEVEL_WARNING); 

    if(memType == IX_MEMORY_TYPE_SRAM)
    {
        IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(memChannel, memOffset);
    }/* end if memory type */   

    /* Address of the Stats Table */
    ipv6Symbols[i].m_Value = memOffset; 
    ipv6Symbols[i].m_Name = IX_CC_IPV6_PATCHNAME_STATS_TABLE;
    i++;

#endif /* IPV6_STATS */

    /* 
     * This loop will check to see if a microengine is being used
     * by our microblock (the IPv6 microblock) and if it is, it'll
     * patch the microblock symbols appropriately
     */

    for (ipv6Index = 0; 
         ipv6Index < IX_CC_IPV6_NUMBER_OF_MICROENGINES; 
         ipv6Index++)
    {
         /* if the bit is set, then patch the symbols */
        if(((g_Ipv6RegistryData.meNumbers) & (1 << ipv6Index)) != 0)
        {
            IX_ERROR_CRT(ix_rm_ueng_patch_symbols(
	                        g_aMicroengineNumberMapping[ipv6Index],
                                i, ipv6Symbols),
                         IX_CC_IPV6_ERROR_FAILED_PATCHING,
                         IX_ERROR_LEVEL_WARNING);
        }/* end if */
    }/* end for (index = 0; index<IX_CC_IPV6_NUMBER_OF_MICROENGINES; index++) */


    return IX_SUCCESS;

}/* end _ix_cc_ipv6_patch_symbols */


/**
 * NAME: _ix_cc_ipv6_create_64bit_counters
 *
 * DESCRIPTION: This primitive will be used to create 64 bit counters.
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * IX_CC_ERROR_OOM_64BIT_COUNTER - error in creating 64 bit counter
 * 
 * 
 */

ix_error _ix_cc_ipv6_create_64bit_counters()
{
#ifdef IPV6_STATS

   ix_uint32 index, portIndex, bladeIndex;
   ix_uint32 baseAddress;
   ix_uint32 outBaseAddr;

   /* Create incoming 64-bit counters */
   for (portIndex = 0; portIndex < MAX_NUMBER_OF_PORTS_PER_BLADE; portIndex++)
   {
       baseAddress = (ix_uint32)((ix_uint32*)g_pIpv6Context->pMbStatsBase + 
                     ((IX_CC_IPV6_TOTAL_NUMBER_OF_IN_MB_COUNTERS * portIndex) +
                      IPV6_IN_RECV));

       /* Create an array of handles and save them */
       IX_ERROR_CRT(ix_rm_counter_64bit_new(
                                IX_CC_IPV6_NUMBER_OF_IN_MB_COUNTERS,
			        (ix_uint32 *)baseAddress,
                                IX_CC_IPV6_COUNTER_OVERFLOW_TIME,
                       g_pIpv6Context->inCounter64BitHandle[portIndex].handles),

                    IX_CC_ERROR_OOM_64BIT_COUNTER,
                    IX_ERROR_LEVEL_WARNING);
   }

   /* First skip over the incoming counters */
   outBaseAddr = (ix_uint32)((ix_uint32 *)g_pIpv6Context->pMbStatsBase + 
                    (MAX_NUMBER_OF_PORTS_PER_BLADE *
		             IX_CC_IPV6_TOTAL_NUMBER_OF_IN_MB_COUNTERS));
                                                     
   /* Create outgoing 64-bit counters. Remember, we maintain
    * outgoing counters for all ports on all blades. 
    */

   for (bladeIndex = 0; bladeIndex < MAX_NUMBER_OF_BLADES; bladeIndex++)
   {
       for (portIndex = 0; portIndex < MAX_NUMBER_OF_PORTS_PER_BLADE;
                                                                portIndex++)
       {
	   /* index indicates the number of ports we have skipped */
           index = (MAX_NUMBER_OF_PORTS_PER_BLADE * bladeIndex) + portIndex;

           /* base address of out stats for this {blade/port} index */
	   baseAddress = (ix_uint32)((ix_uint32 *)outBaseAddr + 
              ((IX_CC_IPV6_TOTAL_NUMBER_OF_OUT_MB_COUNTERS * index) + 1));
           
           /* 64-bit counters for incoming interfaces */
           IX_ERROR_CRT(ix_rm_counter_64bit_new(
	                               IX_CC_IPV6_NUMBER_OF_OUT_MB_COUNTERS,
                                       (ix_uint32*)baseAddress,
                                       IX_CC_IPV6_COUNTER_OVERFLOW_TIME,
                          g_pIpv6Context->outCounter64BitHandle[index].handles),
                        IX_CC_ERROR_OOM_64BIT_COUNTER,
                        IX_ERROR_LEVEL_WARNING);
        }
    }

#endif /* IPV6_STATS */

   return IX_SUCCESS;

}/* end _ix_cc_ipv6_create_64bit_counters */


/**
 * NAME: _ix_cc_ipv6_add_reserved_next_hops
 *
 * DESCRIPTION: This primitive will be used to add reserved next hops into RTM    .
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT  none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * IX_CC_IPV6_ERROR_RTM - error from RTM
 * 
 * 
 */

ix_error _ix_cc_ipv6_add_reserved_next_hops()
{

    ix_cc_rtmv6_next_hop_info nextHopInfo;
   
    /* Add reserved next hops to RTM - NH_DROP */
    nextHopInfo.bladeID = 0;
    nextHopInfo.l2Index = 0;
    nextHopInfo.portID = 0;
    nextHopInfo.mtu = 1500;
    nextHopInfo.flags = IPV6_NH_FLAGS_DROP;
    nextHopInfo.l2IndexType = 0;

    IX_ERROR_CRT(ix_cc_rtmv6_add_next_hop(g_pIpv6Context->hRtm,
                                          IX_CC_IPV6_NH_DROP,
					  1,
					  &nextHopInfo),
                 IX_CC_IPV6_ERROR_RTM,
                 IX_ERROR_LEVEL_WARNING);

    /* Add reserved next hops to RTM - NH_LOCAL */
    nextHopInfo.flags = IPV6_NH_FLAGS_LOCAL;
    IX_ERROR_CRT(ix_cc_rtmv6_add_next_hop(g_pIpv6Context->hRtm,
                                          IX_CC_IPV6_NH_LOCAL,
					  1,
					  &nextHopInfo),
                 IX_CC_IPV6_ERROR_RTM,
                 IX_ERROR_LEVEL_WARNING);

    /* What about DOWN flag? */

    return IX_SUCCESS;

}/* end _ix_cc_ipv6_add_reserved_next_hops */

/**
 * NAME: _ix_cc_ipv6_add_reserved_prefixes
 *
 * DESCRIPTION: This primitive will be used to add reserved prefixes to the 
 * RTMv6. 
 * 
 * @Param:  - IN none
 * @Param:  - OUT  none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * IX_CC_IPV6_ERROR_RTM - error from RTM
 */

ix_error _ix_cc_ipv6_add_reserved_prefixes()
{
    ix_cc_rtmv6_next_hop_info nextHopInfo;
    ix_uint128 NbrDiscResIpv6Addr;
   
    /* Add reserved next hops to RTM - NH_NBR_DISC */
    nextHopInfo.bladeID = 2;
    nextHopInfo.l2Index = 0x100;
    nextHopInfo.portID = 0x0;
    nextHopInfo.mtu = 2000;
    nextHopInfo.flags = 0x0;
    nextHopInfo.l2IndexType = 0x0;

    IX_ERROR_CRT(ix_cc_rtmv6_add_next_hop(g_pIpv6Context->hRtm,
                                          IX_CC_IPV6_NH_NBR_DISC,
					  1,
					  &nextHopInfo),
                 IX_CC_IPV6_ERROR_RTM,
                 IX_ERROR_LEVEL_WARNING);

    /* Multicast address */
    NbrDiscResIpv6Addr.aUint32[0] = 0xFF000000;
    NbrDiscResIpv6Addr.aUint32[1] = 0x0;
    NbrDiscResIpv6Addr.aUint32[2] = 0x0;
    NbrDiscResIpv6Addr.aUint32[3] = 0x00;

    IX_ERROR_CRT(ix_cc_rtmv6_add_prefix(g_pIpv6Context->hRtm,  
                                        NbrDiscResIpv6Addr,
                                        8,
                                        IX_CC_IPV6_NH_NBR_DISC), 
               IX_CC_IPV6_ERROR_RTM,
               IX_ERROR_LEVEL_WARNING); 

    return IX_SUCCESS;
}/* _ix_cc_ipv6_add_reserved_reserved_prefixes */


/**
 * NAME: _ix_cc_ipv6_add_invalid_routes
 *
 * DESCRIPTION: This primitive will be used to add martian addresses to RTMv6.
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * IX_CC_IPV6_ERROR_RTM - error from RTM function call
 * 
 * 
 */

ix_error _ix_cc_ipv6_add_invalid_routes()
{
    /* Most invalid routes - like destination address ::1 or 0 should
     * be "filtered" at header_validation time and the packets will
     * be dropped. So this function does nothing for now.
     */

    return IX_SUCCESS;

}/* end _ix_cc_ipv6_add_invalid_routes */

/**
 * NAME: _ix_cc_ipv6_configure_rtm
 *
 * DESCRIPTION: This primitive will be used to configure RTMv6. 
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT  none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_RTM - error from RTM function
 * 
 */

ix_error _ix_cc_ipv6_configure_rtm()
{

    /* 
     * Initalize RTMv6. The handle to RTMv6 is stored in the global
     * IPv6 context for all future operations.
     */

    IX_ERROR_CRT(ix_cc_rtmv6_init(&g_Ipv6RegistryData.rtmConfig,
                                  &g_pIpv6Context->hRtm),
                 IX_CC_IPV6_ERROR_RTM,
                 IX_ERROR_LEVEL_WARNING);
                             
    /* Add reserved next hops to RTMv6 */
    IX_ERROR_CR(_ix_cc_ipv6_add_reserved_next_hops());

    /* Add invalid IPv6 routes */
    IX_ERROR_CR(_ix_cc_ipv6_add_invalid_routes());

    return IX_SUCCESS;

}/* _ix_cc_ipv6_configure_rtm */
    

/**
 * NAME: _ix_cc_ipv6_register_message_handler
 *
 * DESCRIPTION: This primitive will be used to register message handler. 
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT  none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_CCI - Error from CCI
 * 
 */

ix_error _ix_cc_ipv6_register_message_handler()
{
    
    /* Call CCI message handler if CCI is enabled */
#if defined(IX_EXCLUDE_CCI)
        IX_ERROR_CRT(ix_rm_message_handler_register(IX_CC_IPV6_MSG_INPUT,
                                            ix_cc_ipv6_msg_handler,
                                            g_pIpv6Context),
                 IX_CC_IPV6_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);
#else
        IX_ERROR_CRT(ix_cci_cc_add_message_handler(g_pIpv6Context->hIpv6Handle,
                                            IX_CC_IPV6_MSG_INPUT,
                                            ix_cc_ipv6_msg_handler,
                                            IX_INPUT_TYPE_MULTI_SRC),
                 IX_CC_IPV6_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);

#endif

    return IX_SUCCESS;
}/* end _ix_cc_ipv6_register_message_handler */



/**
 * NAME: _ix_cc_ipv6_register_cci_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to register packet handler with CCI. 
 * @Param:  - IN  none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_CCI - error from CCI
 * 
 */

ix_error _ix_cc_ipv6_register_cci_packet_handlers()
{

#if !defined(IX_EXCLUDE_CCI)

    /* 
     * High priority packet handler needs to be registered as a message handler 
     * Note: Message handlers seem to be drained before packet handlers 
     */

    IX_ERROR_CRT(ix_cci_cc_add_message_handler(g_pIpv6Context->hIpv6Handle,
                                IX_CC_IPV6_PKT_MICROBLOCK_HIGH_PRIORITY_INPUT,
                                ix_cc_ipv6_microblock_high_priority_pkt_handler,
                                IX_INPUT_TYPE_SINGLE_SRC),
                 IX_CC_IPV6_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);


    IX_ERROR_CRT(ix_cci_cc_add_packet_handler(g_pIpv6Context->hIpv6Handle,
                                 IX_CC_IPV6_PKT_MICROBLOCK_LOW_PRIORITY_INPUT,
                                 ix_cc_ipv6_microblock_low_priority_pkt_handler,
                                 IX_INPUT_TYPE_SINGLE_SRC),
                 IX_CC_IPV6_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);


    IX_ERROR_CRT(ix_cci_cc_add_packet_handler(g_pIpv6Context->hIpv6Handle,
                                 IX_CC_IPV6_PKT_STKDRV_INPUT,
                                 ix_cc_ipv6_stkdrv_pkt_handler,
                                 IX_INPUT_TYPE_SINGLE_SRC),
                 IX_CC_IPV6_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);

    IX_ERROR_CRT(ix_cci_cc_add_packet_handler(g_pIpv6Context->hIpv6Handle,
                                 IX_CC_IPV6_PKT_COMMON_INPUT,
                                 ix_cc_ipv6_common_pkt_handler,
                                 IX_INPUT_TYPE_MULTI_SRC),
                 IX_CC_IPV6_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);
#endif

    return IX_SUCCESS;

}/* end _ix_cc_ipv6_register_cci_packet_handlers */


/**
 * NAME: _ix_cc_ipv6_register_rm_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to register packet handler 
 * with resource manager
 *
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_CCI - error from resource manager
 * 
 */

ix_error _ix_cc_ipv6_register_rm_packet_handlers()
{

    IX_ERROR_CRT(ix_rm_message_handler_register(
                      IX_CC_IPV6_PKT_MICROBLOCK_HIGH_PRIORITY_INPUT,
                      ix_cc_ipv6_microblock_high_priority_pkt_handler,
                      g_pIpv6Context),
                 IX_CC_IPV6_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);


    IX_ERROR_CRT(ix_rm_packet_handler_register(
                      IX_CC_IPV6_PKT_MICROBLOCK_LOW_PRIORITY_INPUT,
                      ix_cc_ipv6_microblock_low_priority_pkt_handler,
                      g_pIpv6Context),
                 IX_CC_IPV6_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);


    IX_ERROR_CRT(ix_rm_packet_handler_register( 
                      IX_CC_IPV6_PKT_STKDRV_INPUT,
                      ix_cc_ipv6_stkdrv_pkt_handler,
                      g_pIpv6Context),
                 IX_CC_IPV6_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);

    IX_ERROR_CRT(ix_rm_packet_handler_register(
                      IX_CC_IPV6_PKT_COMMON_INPUT,
                      ix_cc_ipv6_common_pkt_handler,
                      g_pIpv6Context),
                 IX_CC_IPV6_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);

    return IX_SUCCESS;

}/* end _ix_cc_ipv6_register_rm_packet_handlers */

/**
 * NAME: _ix_cc_ipv6_register_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to register packet handlers. 
 * It registers with CCI in level 1 system and register with RM in
 * level 0 system.
 * 
 *           .
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_CCI - error from CCI/resource manager
 * 
 */

ix_error _ix_cc_ipv6_register_packet_handlers()
{
    /* Call CCI packet handler if CCI is enabled */
#if defined(IX_EXCLUDE_CCI)

        IX_ERROR_CR(_ix_cc_ipv6_register_rm_packet_handlers());

#else

        IX_ERROR_CR(_ix_cc_ipv6_register_cci_packet_handlers());

#endif

    return IX_SUCCESS;
}/* end _ix_cc_ipv6_register_packet_handlers */


/**
 * NAME: _ix_cc_ipv6_create_event_handler
 *
 * DESCRIPTION: This primitive is used to create event handler if there is no
 * CCI. This API needs to be implemented by client in LEVEL0 syatem.
 * User needs to call _ix_cc_ipv6_icmp_send_message function periodically
 * to drain ICMP packets from queue. 
 * 
 * @Param:  - IN arg_pContext: IPv6 context
 *
 * @Param:  - OUT arg_pEventId:Event handler Id
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *  
 * IX_CC_ERROR_UNIMPL - not implemented
 * 
 */

ix_error _ix_cc_ipv6_create_event_handler(void* arg_pContext,
                                          ix_ossl_thread_t* arg_pEventId)
{ 

    return IX_ERROR_WARNING(IX_CC_ERROR_UNIMPL,("No Event Handler for Icmp\n"));
}/* end _ix_cc_ipv6_create_event_handler */


/**
 * NAME: _ix_cc_ipv6_add_icmp_event_handler
 *
 * DESCRIPTION: This primitive will be used to add icmp event handler.This is useful to
 * drain ICMP packets priodically from queue           .
 * 
 * @Param:  - IN  none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_CCI - error from event handler
 * 
 */

ix_error _ix_cc_ipv6_add_icmp_event_handler()
{

#ifdef NOT_YET

    ix_uint32 eventInterval;

    /* 
     * Calculate event interval in milliseconds, data in g_Icmpv6SleepTime 
     * is in seconds 
     */

     eventInterval = g_Icmpv6SleepTime * IX_CC_IPV6_MILLISECONDS_PER_SECOND; 

    
#if defined(IX_EXCLUDE_CCI) /* compile time flag */

        IX_ERROR_CR(_ix_cc_ipv6_create_event_handler(g_pIpv6Context,
	                                    &g_pIpv6Context->icmpThreadId));
#else 

        IX_ERROR_CRT(ix_cci_cc_add_event_handler(g_pIpv6Context->hIpv6Handle,
                                          eventInterval,
                                          _ix_cc_ipv6_icmp_send_err_message,
                                          IX_EVENT_TYPE_PERIODIC,
                                          IX_CC_IPV6_ICMP_EVENT_PRIORITY,
                                          &g_pIpv6Context->hEventHandle),
                 IX_CC_IPV6_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);

#endif
#endif
    return IX_SUCCESS;
}/* end _ix_cc_ipv6_add_icmp_event_handler */


/**
 * NAME: ix_cc_ipv6_init
 *
 * DESCRIPTION: This primitive shall be called and returned successfully 
 * before requesting any service from IPv6 Forwarder core component. 
 * This primitive should be called only once to initialize IPv6 Forwarder 
 * core component. This function performs allocation of memory for symbols to 
 * be patched, Creation of 64 bit counters, registration of packet and message 
 * handlers, initialization and configuration of RTM, allocation and 
 * initialization of internal data structures and creation of event handler 
 * for ICMP.
 * 
 * @Param:  - IN arg_hCcHandle - handle to IPV6 forwarder core component, 
 *            created by core component infrastructure; 
 *            this shall be used later to get other (to add event handler) 
 *            services from core component infrastructure.
 *
 * @Param:  - IN-OUT arg_ppContext - Buffer free list and dynamic property 
 *            data will be given during by system APPs during initialization. 
 *            Upon successfull completion, it represents the location where 
 *            the pointer to the control block allocated by IPv6 Forwarder 
 *            core component will be stored. The control block is internal 
 *            to IPv6 Forwarder core component and it contains information 
 *            about IPv6 internal data structures, allocated memory and other 
 *            relevant information. Later this pointer to be passed into  
 *            ix_cc_ipv6_fini function for freeing memory and other house 
 *            keeping operations when IPv6 component is being destroyed.  
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - input parameter arg_ppContext is null
 * IX_CC_ERROR_OOM   - memory allocation failure      
 * IX_CC_IPV6_ERROR_FAILED_PATCHING   - patching failures                      
 * IX_CC_ERROR_OOM_64BIT_COUNTER - 64 bit counter creation failure
 * IX_CC_IPV6_ERROR_REGISTRY  - registry access failures
 * IX_CC_IPV6_ERROR_RTM - failure from RTM core component
 * IX_CC_IPV6_ERROR_CCI - failure from CCI 
 * 
 */

ix_error ix_cc_ipv6_init(ix_cc_handle arg_hCcHandle,
                         void** arg_ppContext)
{
    ix_error err = IX_SUCCESS;
    ix_uint8 cnt;
    ix_uint8 flag = 0;

    /* Variables used in retrieving static data from arg_ppContext */
    ix_cc_init_context *pInitCtx;
    ix_cc_properties_node *pPropNode;

    /* Check for argument validity */
    if(arg_ppContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("Input parameter(arg_ppContext) is null"));

    }/* end if (arg_ppContext == NULL) */

    /* Allocate memory for IPv6 Context */
    g_pIpv6Context = (ix_cc_ipv6_context *)
                                    ix_ossl_malloc(sizeof(ix_cc_ipv6_context));
    if(g_pIpv6Context == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,
                               ("Failed to allocate memory for ipv6 context"));
     }/* end if (g_pIpv6Context == NULL) */

    /* initialize the structure */
    ix_ossl_memset(g_pIpv6Context, 0, sizeof(ix_cc_ipv6_context));

    /* store ipv6 component handle in ix_cc_ipv6_context */
    g_pIpv6Context->hIpv6Handle = arg_hCcHandle;

    /* Get the generic init context */
    pInitCtx = (ix_cc_init_context *)*arg_ppContext;

    /* Store free list handle in g_pIpv6Context */
    g_pIpv6Context->hBufferFreeList = pInitCtx->hFreeList;

    /* 
     * Retrieve static configuration from registry. If there is no registry, 
     * then the data structure will be filled with data from header file. 
     */
#if defined (IX_INCLUDE_REGISTRY)

    IX_ERROR_CG(_ix_cc_ipv6_registry(), err, cleanupContext);

#else
    
    /* 
     * Default registry options */
       
    g_Ipv6RegistryData.rtmConfig.routeTableSizeHint = 
                                            IX_CC_IPV6_ROUTE_TABLE_SIZE_HINT;
    g_Ipv6RegistryData.rtmConfig.lkupType = IX_CC_IPV6_RTM_LOOKUP_TYPE;
    g_Ipv6RegistryData.rtmConfig.lkupMemType = 
	                                    IX_CC_IPV6_RTM_LOOKUP_MEMORY_TYPE;
    g_Ipv6RegistryData.rtmConfig.lkupChannel = 
                                            IX_CC_IPV6_RTM_LOOKUP_CHANNEL;
#ifdef SPLIT_V6_TRIE_TABLE
    g_Ipv6RegistryData.rtmConfig.lkupChannel16BitTrie = 
                                     IX_CC_IPV6_RTM_LOOKUP_CHANNEL_16BIT_TRIE;
#endif /* SPLIT_V6_TRIE_TABLE */
    g_Ipv6RegistryData.rtmConfig.nhdbSizeHint = 
	                                  IX_CC_IPV6_NEXT_HOP_TABLE_SIZE_HINT;
    g_Ipv6RegistryData.rtmConfig.nhdbMemType = 
	                                      IX_CC_IPV6_NEXT_HOP_MEMORY_TYPE;
    g_Ipv6RegistryData.rtmConfig.nhdbChannel = 
	                                      IX_CC_IPV6_NEXT_HOP_CHANNEL;
    g_Ipv6RegistryData.meNumbers = IX_CC_IPV6_ME;   

#endif

    /* 
     * Initialization and configuration of RTMv6
     *
     * - Add reserved next hops to RTMv6
     * - Add martian routes to RTMv6
     */

    IX_ERROR_CG(_ix_cc_ipv6_configure_rtm(), err, cleanupContext);
   
    /* Allocate memory for symbols to be patched */
    IX_ERROR_CG(_ix_cc_ipv6_alloc_symbols(), err, cleanupRTM);

    /*
     * Retrieve local blade property information from init context 
     * (This will work for both level-1 and level-0 modes)
     */
    pPropNode = pInitCtx->pPropertyList;

    /* Get blade Id from first node */
    if(pPropNode != NULL)
    {
        g_Ipv6RegistryData.localBladeId  = pPropNode->pPropertyInfo->blade_id;
    }/* end if */

    /* Patch symbols */
    IX_ERROR_CG(_ix_cc_ipv6_patch_symbols(), err, cleanupRmMemory);

    /* 64 bit counter creation */
    IX_ERROR_CG(_ix_cc_ipv6_create_64bit_counters(), err, cleanupRmMemory);   
   
    /* Register Message Handler */
    IX_ERROR_CG(_ix_cc_ipv6_register_message_handler(), err, cleanupCounter);

    /* Register Packet Handler */
    IX_ERROR_CG(_ix_cc_ipv6_register_packet_handlers(), err, cleanupMessageHandler);

    /* Get mutex from OSSL */
    err = ix_ossl_mutex_init(IX_OSSL_MUTEX_UNLOCK, &g_pIpv6Context->ipv6Mutex);
    if (err != IX_SUCCESS)
    {
        err = IX_ERROR_WARNING(IX_CC_ERROR_OOR, ("Failed to create mutext"));

        goto cleanupPacketHandlers;

    }/* end if (g_pIpv6Context == NULL) */

    while (pPropNode != NULL)
    {
        flag = 0;
        for (cnt = 0;
             cnt < pPropNode->pPropertyInfo->no_virtual_if;
             cnt++)
        {
            if (pPropNode->pPropertyInfo->ip_prop[cnt].ip_version
                == IX_CC_IP_VERSION_IPV6)
            {
                /* Setup Port Forwarding Enable/Disable status only
                 * for the interface
                 */
                if (flag == 0)
                {
                    IX_ERROR_CG(_ix_cc_ipv6_property_update_port_status(
                                    pPropNode->pPropertyInfo->port_id,
                                    pPropNode->pPropertyInfo->physical_if_state),
                        err, cleanupMutex);
                    flag = 1; 
                }
                /* Setup the IPv6 addresses for this port */ 
                IX_ERROR_CG(_ix_cc_ipv6_property_update_ip_address(
   	                        IX_CC_SET_PROPERTY_ADD_INTERFACE_IPV6,
                                pPropNode->pPropertyInfo->blade_id,
                                pPropNode->pPropertyInfo->port_id,
                                pPropNode->pPropertyInfo->ip_prop[cnt].protocol.ipv6_prop.ipv6_address.addr),
                    err, cleanupMutex);
           }
        }
        pPropNode = pPropNode->pNextPropNode;
    }/* end while */

#if 1
    /* Add reserved NBR DISC next hop and prefix */
    _ix_cc_ipv6_add_reserved_prefixes();
#endif
    
    /* Initalize ICMP */
    IX_ERROR_CG(_ix_cc_ipv6_icmp_init(g_pIpv6Context),
                err, cleanupMutex);

    /* Initialize Neighbor Discovery and Address Auto-Configuration */
    IX_ERROR_CG(ix_cc_nd_init(pInitCtx->pPropertyList, g_pIpv6Context),
                err, cleanupIcmp);

    /* Register event handler */
    /*IX_ERROR_CG(_ix_cc_ipv6_add_icmp_event_handler(),err, cleanupIcmp);*/
   
    *arg_ppContext = (void *)g_pIpv6Context;

    return IX_SUCCESS;

cleanupIcmp:
    _ix_cc_ipv6_icmp_fini();
cleanupMutex:
    ix_ossl_mutex_fini(g_pIpv6Context->ipv6Mutex); 
cleanupPacketHandlers:
    _ix_cc_ipv6_unregister_packet_handlers();
cleanupMessageHandler:
    _ix_cc_ipv6_unregister_message_handler();
cleanupCounter:
    _ix_cc_ipv6_delete_64bit_counters();
cleanupRmMemory:
    _ix_cc_ipv6_free_symbols();
cleanupRTM:
    _ix_cc_ipv6_terminate_rtm_services();
cleanupContext:
    ix_ossl_free(g_pIpv6Context);

    return err;

} /* end ix_cc_ipv6_init */
