/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 6
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      IPv6 Forwarder Core Component - message helpers
 *
 * = FILENAME
 *      ix_cc_ipv6_msg_helpers.c
 *
 * = DESCRIPTION
 *      The file defines message helper interface functions.
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      12/09/2002
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */
#define IX_ERROR_FILE_IDENT "$Id: ix_cc_ipv6_msg_helpers.c,v 1.17 2003/11/01 00:57:27 rranjeet Exp $";


/**
 * System defined include files
 */

/**
 * User defined include files
 */

#include "ix_cc_error.h"
#include "ix_ossl.h"
#include "ix_rm.h"
#include "cc/ix_cc_rtmv6.h"
#include "ipv6_fwder_uc.h"
#include "ix_cc.h"
#include "bindings.h"
#include "cc/ix_cc_ipv6.h"
#include "cc/ix_cc_msup.h"


#include "cc/ipv6/internal/icmp6.h"
#include "cc/ipv6/internal/ix_cc_ipv6_msg_handler.h"


/**
 * Preprocessor symbols and Macros used in this file
 */

/**
 * Type definitions whose scope is limited to this file
 */


/**
 * Variable declarations global to this file only. Externs are followed by 
 * static variables.
 */


/** 
 * Extern function prototypes.
 */

/** 
 * Static function prototypes.
 */




/** 
 * Function definitions.
 */



/**
 * NAME: ix_cc_ipv6_async_add_prefix
 *
 * DESCRIPTION: This message helper primitive helps client to add a prefix in 
 * RTMv6. Client needs to successfully add nextHopId to RTM by calling 
 * ix_cc_ipv6_async_add_next_hop messaging helper primitive before sending 
 * this message to IPv6 Forwarder core component. 
 * 
 * @Param:  - IN arg_IpAddr: network IPv6 address identifying the prefix.
 * @Param:  - IN arg_prefixLength: prefix length for the prefix.
 * @Param:  - IN arg_NextHopId: next hop identifier for prefix.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */

ix_error ix_cc_ipv6_async_add_prefix(ix_uint128 arg_IpAddr,
                                    ix_uint8 arg_PrefixLength,
                                    ix_cc_rtmv6_nhid arg_NextHopId,
                                    ix_cc_ipv6_cb_route_op arg_Callback,
                                    void* arg_pUserContext)
{
    ix_cc_ipv6_add_prefix_data addPrefix;
    ix_cc_ipv6_general_callback_data *cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_ipv6_general_callback_data*) 
                     ix_ossl_malloc(sizeof(ix_cc_ipv6_general_callback_data));
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    addPrefix.ipv6Addr = arg_IpAddr;
    addPrefix.prefixLength = arg_PrefixLength;
    addPrefix.nextHopId = arg_NextHopId;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_IPV6_MSG_INPUT,
                                           _ix_cc_ipv6_cb_general,
                                           (void*)cbContext,
                                           IX_CC_IPV6_MSG_ADD_PREFIX,
                                           (void *)&addPrefix,
                                           sizeof(ix_cc_ipv6_add_prefix_data)),
                 err,
                 IX_CC_IPV6_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:

    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);

    }/* end if cbContext != NULL */

    return err;
} /* end ix_cc_ipv6_async_add_prefix */

/**
 * NAME: ix_cc_ipv6_async_delete_prefix
 *
 * DESCRIPTION:This message helper primitive helps a client to delete a 
 * prefix in RTMv6. Client needs to delete all prefix associate with a 
 * next hop to delete the next hop by calling ix_cc_ipv6_async_delete_next_hop 
 * message helper primitive. 
 * 
 * @Param:  - IN arg_IpAddr: network IPv6 address identifying the prefix.
 * @Param:  - IN arg_PrefixLength: prefix length for the prefix.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_ipv6_async_delete_prefix(ix_uint128 arg_IpAddr,
                                       ix_uint8 arg_PrefixLength,
                                       ix_cc_ipv6_cb_route_op arg_Callback,
                                       void* arg_pUserContext)
{
    ix_cc_ipv6_delete_prefix_data deletePrefix;
    ix_cc_ipv6_general_callback_data *cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_ipv6_general_callback_data*) 
                    ix_ossl_malloc(sizeof(ix_cc_ipv6_general_callback_data));

    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */
    
    deletePrefix.ipv6Addr = arg_IpAddr;
    deletePrefix.prefixLength = arg_PrefixLength;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_IPV6_MSG_INPUT,
                                          _ix_cc_ipv6_cb_general,
                                          (void *)cbContext,
                                          IX_CC_IPV6_MSG_DELETE_PREFIX,
                                          (void *)&deletePrefix,
                                         sizeof(ix_cc_ipv6_delete_prefix_data)),
                 err,
                 IX_CC_IPV6_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING, cleanup_context);

    return IX_SUCCESS;

cleanup_context:

    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */

    return err;

} /* end ix_cc_ipv6_async_delete_prefix */




/**
 * NAME: ix_cc_ipv6_async_update_prefix
 *
 * DESCRIPTION: This message helper primitive helps client to update a 
 * prefix in RTM. Client needs to successfully add nextHopId to RTM by calling 
 * ix_cc_ipv6_async_add_next_hop messaging helper primitive before sending 
 * this message to IPv6 Forwarder core component. 
 * 
 * @Param:  - IN arg_IpAddr: network IPv6 address identifying the prefix.
 * @Param:  - IN arg_PrefixLength: prefix length for the prefix.
 * @Param:  - IN arg_NextHopId: next hop identifier for the prefix.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */

ix_error ix_cc_ipv6_async_update_prefix(ix_uint128 arg_IpAddr,
                                        ix_uint8 arg_prefixLength,
                                        ix_cc_rtmv6_nhid arg_NextHopId,
                                        ix_cc_ipv6_cb_route_op arg_Callback,
                                        void* arg_pUserContext)
{
    ix_cc_ipv6_add_prefix_data addPrefix;
    ix_cc_ipv6_general_callback_data *cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_ipv6_general_callback_data*) 
                     ix_ossl_malloc(sizeof(ix_cc_ipv6_general_callback_data));

    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */
    
    addPrefix.ipv6Addr = arg_IpAddr;
    addPrefix.prefixLength = arg_prefixLength;
    addPrefix.nextHopId = arg_NextHopId;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_IPV6_MSG_INPUT,
                                          _ix_cc_ipv6_cb_general,
                                          (void *)cbContext,
                                          IX_CC_IPV6_MSG_UPDATE_PREFIX,
                                          (void *)&addPrefix,
                                          sizeof(ix_cc_ipv6_add_prefix_data)),
                 err,
                 IX_CC_IPV6_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING, cleanup_context);
   

    return IX_SUCCESS;

cleanup_context:

    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */

    return err;

} /* end ix_cc_ipv6_async_update_prefix */



/**
 * NAME: ix_cc_ipv6_async_lookup_prefix
 *
 * DESCRIPTION: This primitive helps a client to look up routing 
 * information for a given IPv6 address. If there is a successful match, 
 * the information will be given to client are nextHopId (including the 
 * blade id), portId, mtu and flags. If there is no match, then 
 * IX_CC_RTMV6_NHID_NO_ROUTE will be assigned to nextHopId field of 
 * pNextHopData and all other information are invalid.
 * 
 * @Param:  - IN arg_IpAddr: network IPv6 address identifying the prefix.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_ipv6_async_lookup_prefix(ix_uint128 arg_IpAddr,
                                       ix_cc_ipv6_cb_lookup_route arg_Callback,
                                       void *arg_pUserContext)
{
    ix_cc_ipv6_lookup_callback_data *cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL)  */

    cbContext = (ix_cc_ipv6_lookup_callback_data*) 
                      ix_ossl_malloc(sizeof(ix_cc_ipv6_lookup_callback_data));

    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_IPV6_MSG_INPUT,
                                           _ix_cc_ipv6_cb_lookup_rtm,
                                           (void *)cbContext,
                                           IX_CC_IPV6_MSG_LOOKUP_PREFIX,
                                           (void *)&arg_IpAddr,
                                           sizeof(ix_uint128)),
                 err,
                 IX_CC_IPV6_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING, cleanup_context);

    return IX_SUCCESS;

cleanup_context:

    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */

    return err;

} /* end ix_cc_ipv6_async_lookup_prefix */


/**
 * NAME: ix_cc_ipv6_async_purge_prefixes
 *
 * DESCRIPTION: This primitive helps a client to remove all routes from RTMv6.
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_ipv6_async_purge_prefixes()
{

    IX_ERROR_CRT(ix_cc_msup_send_msg(IX_CC_IPV6_MSG_INPUT,
                                     IX_CC_IPV6_MSG_PURGE_ROUTES,
                                     NULL,
				     0),
                 IX_CC_IPV6_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING);

    return IX_SUCCESS;

} /* end ix_cc_ipv6_async_purge_prefixes */


/**
 * NAME: ix_cc_ipv6_async_dump_prefixes
 *
 * DESCRIPTION: This primitive helps to dump all routes of RTMv6 in memory
 * 
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_ipv6_async_dump_prefixes(ix_cc_ipv6_cb_dump_data arg_Callback,
                                      void* arg_pUserContext)
{
    ix_cc_ipv6_dump_callback_data *cbContext;
    ix_error err = IX_SUCCESS;
    

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("Invalid pointer: arg_pCallback "));
    }/* end if(arg_Callback == NULL)  */

    cbContext = (ix_cc_ipv6_dump_callback_data *) 
                        ix_ossl_malloc(sizeof(ix_cc_ipv6_dump_callback_data));
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */


    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_IPV6_MSG_INPUT,
                                           _ix_cc_ipv6_cb_dump_rtm,
                                           (void *)cbContext,
                                           IX_CC_IPV6_MSG_DUMP_ROUTES,
                                           NULL,
                                           0),
                 err,
                 IX_CC_IPV6_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING, cleanup_context);
   
    return IX_SUCCESS;

cleanup_context:

    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */

    return err;

} /* end ix_cc_ipv6_async_dump_prefixes */



/**
 * NAME: ix_cc_ipv6_async_add_next_hop
 *
 * DESCRIPTION: This primitive helps a client to add next hop information 
 * to RTMv6 database. Client needs to add next hop to RTMv6 in order to 
 * add routes referring to that next hop.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop to be added
 * @Param:  - IN arg_NumNextHops: The number of next hop info structures 
 *                              contained in arg_pNextHopData.
 * @Param:  - IN arg_pNextHopData: pointer to next hop information
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined contex
 * 
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_ipv6_async_add_next_hop(ix_cc_rtmv6_nhid arg_NextHopId,
                                    ix_uint32 arg_NumNextHops,
                                    ix_cc_rtmv6_next_hop_info* arg_pNextHopData,
                                    ix_cc_ipv6_cb_route_op arg_Callback,
                                    void* arg_pUserContext)
{
    ix_cc_ipv6_general_callback_data *cbContext;
    ix_cc_ipv6_add_next_hop_data addNextHop;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_ipv6_general_callback_data*) 
                    ix_ossl_malloc(sizeof(ix_cc_ipv6_general_callback_data));
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */


    addNextHop.nextHopId = arg_NextHopId;
    addNextHop.numNextHops = arg_NumNextHops;

    /* Copy all the next hops */
    ix_ossl_memcpy(&addNextHop.nextHopInfo[0],
                   arg_pNextHopData,
                   arg_NumNextHops * sizeof(ix_cc_rtmv6_next_hop_info));

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_IPV6_MSG_INPUT,
                                           _ix_cc_ipv6_cb_general,
                                           (void *)cbContext,
                                           IX_CC_IPV6_MSG_ADD_NEXT_HOP,
                                           (void *)&addNextHop,
                                          sizeof(ix_cc_ipv6_add_next_hop_data)),
                 err,
                 IX_CC_IPV6_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING, cleanup_context);   

    return IX_SUCCESS;

cleanup_context:

    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */

    return err;

} /* end ix_cc_ipv6_async_add_next_hop */


/**
 * NAME: ix_cc_ipv6_async_delete_next_hop
 *
 * DESCRIPTION: This message helper primitive helps a client to delete next hop information 
 * from RTM database. Client needs to remove all routes associated with next hop before 
 * calling this primitive. If not, client gets an error.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop to be removed.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_ipv6_async_delete_next_hop(ix_cc_rtmv6_nhid arg_NextHopId,
                                          ix_cc_ipv6_cb_route_op arg_Callback,
                                          void* arg_pUserContext
                                          )
{
    ix_cc_ipv6_general_callback_data *cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */
    cbContext = (ix_cc_ipv6_general_callback_data*) ix_ossl_malloc(sizeof(ix_cc_ipv6_general_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */


    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_IPV6_MSG_INPUT,
                                    _ix_cc_ipv6_cb_general,
                                    (void *)cbContext,
                                    IX_CC_IPV6_MSG_DELETE_NEXT_HOP,
                                    (void *)&arg_NextHopId,
                                    sizeof(ix_cc_rtmv6_nhid)),
                 err,
                 IX_CC_IPV6_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING, cleanup_context);

    return IX_SUCCESS;
cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;

} /* end ix_cc_ipv6_async_delete_next_hop */



/**
 * NAME: ix_cc_ipv6_async_update_next_hop
 *
 * DESCRIPTION: This primitive helps a client to update next hop information 
 * to RTMv6 database. Client needs to add next hop to RTMv6 in order to 
 * add routes referring to that next hop.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop to be updated
 * @Param:  - IN arg_NumNextHops: The number of next hop info structures 
 *                              contained in arg_pNextHopData.
 * @Param:  - IN arg_pNextHopData: pointer to next hop information
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined contex
 * 
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_ipv6_async_update_next_hop(ix_cc_rtmv6_nhid arg_NextHopId,
                                    ix_uint32 arg_NumNextHops,
                                    ix_cc_rtmv6_next_hop_info* arg_pNextHopData,
                                    ix_cc_ipv6_cb_route_op arg_Callback,
                                    void* arg_pUserContext)
{
    ix_cc_ipv6_general_callback_data *cbContext;
    ix_cc_ipv6_add_next_hop_data addNextHop;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_ipv6_general_callback_data*) 
                      ix_ossl_malloc(sizeof(ix_cc_ipv6_general_callback_data));

    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */


    addNextHop.nextHopId = arg_NextHopId;
    ix_ossl_memcpy(&addNextHop.nextHopInfo, arg_pNextHopData, 
                                        sizeof(ix_cc_rtmv6_next_hop_info));

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_IPV6_MSG_INPUT,
                                           _ix_cc_ipv6_cb_general,
                                           (void *)cbContext,
                                           IX_CC_IPV6_MSG_UPDATE_NEXT_HOP,
                                           (void *)&addNextHop,
                                          sizeof(ix_cc_ipv6_add_next_hop_data)),
                  err,
                  IX_CC_IPV6_ERROR_MSG_LIBRARY,
                  IX_ERROR_LEVEL_WARNING, cleanup_context);   

    return IX_SUCCESS;

cleanup_context:

    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */

    return err;

} /* end ix_cc_ipv6_async_update_next_hop */

/**
 * NAME: ix_cc_ipv6_async_get_next_hop
 *
 * DESCRIPTION: This message helper primitive helps a client to retrieve 
 * next hop information from RTMv6 database. 
 * 
 * @Param:  - IN arg_NextHopId: Identifier of the next hop information to 
 *            be retrieved from RTM.
 * @Param:  - IN arg_Callback: pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_ipv6_async_get_next_hop(ix_cc_rtmv6_nhid arg_NextHopId,
                                       ix_cc_ipv6_cb_get_next_hop arg_Callback,
                                       void* arg_pUserContext)
{

    ix_cc_ipv6_lookup_callback_data *cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));

    }/* end if(arg_Callback == NULL)  */

    cbContext = (ix_cc_ipv6_lookup_callback_data*) 
                      ix_ossl_malloc(sizeof(ix_cc_ipv6_lookup_callback_data));

    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */
    
    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_IPV6_MSG_INPUT,
                                           _ix_cc_ipv6_cb_lookup_rtm,
                                           (void *)cbContext,
                                           IX_CC_IPV6_MSG_GET_NEXT_HOP,
                                           (void *)&arg_NextHopId,
                                           sizeof(ix_cc_rtmv6_nhid)),
                 err,
                 IX_CC_IPV6_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING, cleanup_context);

    return IX_SUCCESS;

cleanup_context:

    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */

    return err;

} /* end ix_cc_ipv6_async_get_next_hop */

/**
 * NAME: ix_cc_ipv6_async_purge_rtm
 *
 * DESCRIPTION: This primitive helps a client to removes all routes and next 
 * hops from RTMv6 database
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_ipv6_async_purge_rtm()
{
    IX_ERROR_CRT(ix_cc_msup_send_msg(IX_CC_IPV6_MSG_INPUT,
                              IX_CC_IPV6_MSG_PURGE_RTM,
                              NULL,
                              0),
                 IX_CC_IPV6_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING);

    return IX_SUCCESS;

} /* end ix_cc_ipv6_async_purge_rtm */

/**
 * NAME: ix_cc_ipv6_async_dump_next_hops
 *
 * DESCRIPTION:   This primitive helps to dump all next hops of RTMv6 in memory.
 * 
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_ipv6_async_dump_next_hops(ix_cc_ipv6_cb_dump_data arg_Callback,
                                         void* arg_pUserContext)
{
    ix_cc_ipv6_dump_callback_data *cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("Invalid pointer: arg_pCallback "));
    }/* end if(arg_Callback == NULL)  */

    cbContext = (ix_cc_ipv6_dump_callback_data *) 
                        ix_ossl_malloc(sizeof(ix_cc_ipv6_dump_callback_data));

    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_IPV6_MSG_INPUT,
                                           _ix_cc_ipv6_cb_dump_rtm,
                                           (void *)cbContext,
                                           IX_CC_IPV6_MSG_DUMP_NEXT_HOPS,
                                           NULL,
                                           0),
                  err,
                  IX_CC_IPV6_ERROR_MSG_LIBRARY,
                  IX_ERROR_LEVEL_WARNING, cleanup_context);
      
    return IX_SUCCESS;

cleanup_context:

    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */

    return err;

} /* end ix_cc_ipv6_async_dump_next_hops */

/**
 * NAME: ix_cc_ipv6_async_set_mtu
 *
 * DESCRIPTION:This primitive helps client to update MTU for a given next hop. 
 * Before calling this message helper, client needs to call 
 * ix_cc_ipv6_async_add_next_hop to add next hop identifier in RTMv6 database. 
 * The validity of MTU shall be done by the client.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop.
 * @Param:  - IN arg_Mtu: new maximum transmission unit for the given next hop.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_ipv6_async_set_mtu(ix_cc_rtmv6_nhid arg_NextHopId,
                                  ix_uint32 arg_Mtu,
                                  ix_cc_ipv6_cb_route_op arg_Callback,
                                  void* arg_pUserContext
                                 )
{
    ix_cc_ipv6_general_callback_data *cbContext;
    ix_cc_ipv6_set_mtu_data mtuData;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback "));
    }/* end if(arg_Callback == NULL)  */

    cbContext = (ix_cc_ipv6_general_callback_data *) 
                  ix_ossl_malloc(sizeof(ix_cc_ipv6_general_callback_data));

    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pUserContext;

    mtuData.nextHopId = arg_NextHopId;
    mtuData.mtu = arg_Mtu;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_IPV6_MSG_INPUT,
                                           _ix_cc_ipv6_cb_general,
                                           (void *)cbContext,
                                           IX_CC_IPV6_MSG_SET_MTU,
                                           (void*)&mtuData,
                                           sizeof(ix_cc_ipv6_set_mtu_data)),
                  err,
                  IX_CC_IPV6_ERROR_MSG_LIBRARY,
                  IX_ERROR_LEVEL_WARNING, cleanup_context);
    
    return IX_SUCCESS;

cleanup_context:

    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */

    return err;

} /* end ix_cc_ipv6_async_set_mtu  */





/**
 * NAME: ix_cc_ipv6_async_set_flags
 *
 * DESCRIPTION: This primitive helps client to update flags for a given next 
 * hop. Before calling this message helper, client needs to call 
 * ix_cc_ipv6_async_add_next_hop to add next hop identifier in RTM database. 
 * The supported flags are defined by IPv6 Forwarder microblock.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop.
 * @Param:  - IN arg_Flags: new flags[BLDBLK] for the given next hop 
 * @Param:  - IN arg_pUserContext: pointer to client defined context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * IX_CC_IPV6_ERROR_INVALID_FLAGS - invalid flags
 */

ix_error ix_cc_ipv6_async_set_flags(ix_cc_rtmv6_nhid arg_NextHopId,
                                    ix_uint32 arg_Flags,
                                    ix_cc_ipv6_cb_route_op arg_Callback,
                                    void* arg_pUserContext)
{
    ix_cc_ipv6_general_callback_data *cbContext;
    ix_cc_ipv6_set_flags_data flagsData;
    ix_error err = IX_SUCCESS;
    

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("Invalid pointer: arg_pCallback "));
    }/* end if(arg_Callback == NULL)  */

    cbContext = (ix_cc_ipv6_general_callback_data*) 
                   ix_ossl_malloc(sizeof(ix_cc_ipv6_general_callback_data));

    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    if((arg_Flags !=IPV6_NH_FLAGS_LOCAL) &&
       (arg_Flags !=IPV6_NH_FLAGS_DOWN) &&
       (arg_Flags !=IPV6_NH_FLAGS_DROP))
    {
        return IX_ERROR_WARNING(IX_CC_IPV6_ERROR_INVALID_FLAGS,
                                    ("Invalid flags "));
    }/* end if flag bit */

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pUserContext;

    flagsData.nextHopId = arg_NextHopId;
    flagsData.flags = arg_Flags;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_IPV6_MSG_INPUT,
                                           _ix_cc_ipv6_cb_general,
                                           (void *)cbContext,
                                           IX_CC_IPV6_MSG_SET_FLAGS,
                                           (void*)&flagsData,
                                           sizeof(ix_cc_ipv6_set_flags_data)),
                 err,
                 IX_CC_IPV6_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING, cleanup_context);
   
    return IX_SUCCESS;

cleanup_context:

    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */

    return err;

} /* end ix_cc_ipv6_async_set_flags */



/**
 * NAME: ix_cc_ipv6_async_get_rate_limit_time
 *
 * DESCRIPTION: This message helper primitive helps client to retrieve the 
 * number of seconds between when the event will fire causing the ICMP 
 * messages to dequeue and send. 
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_ipv6_async_get_rate_limit_time(
                                 ix_cc_ipv6_cb_get_rate_limit_time arg_Callback,
                                 void* arg_pUserContext)
{
    ix_cc_ipv6_icmp_callback_data *cbContext;
    ix_error err = IX_SUCCESS;
    

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("Invalid pointer: arg_pCallback "));
    }/* end if(arg_Callback == NULL)  */

    cbContext = (ix_cc_ipv6_icmp_callback_data*) 
                        ix_ossl_malloc(sizeof(ix_cc_ipv6_icmp_callback_data));

    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */


    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_IPV6_MSG_INPUT,
                                           _ix_cc_ipv6_cb_icmp,
                                           (void *)cbContext,
                                           IX_CC_IPV6_MSG_GET_RATE_LIMIT_TIME,
                                           NULL,
                                           0),
                 err,
                 IX_CC_IPV6_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING, cleanup_context);
   
    return IX_SUCCESS;

cleanup_context:

    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */

    return err;

} /* end ix_cc_ipv6_async_get_rate_limit_time  */



/**
 * NAME: ix_cc_ipv6_async_set_rate_limit_time
 *
 * DESCRIPTION: This message helper primitive helps client to set the 
 * number of seconds between when the event will fire causing the 
 * ICMP messages to dequeue and send.  The lower the value of 
 * arg_SleepTime, the more frequent the ICMP messages will be dequeued 
 * and sent.
 * 
 * @Param:  - IN arg_SleepTime: minimum value is 1 second and maximum value 
 *            is 32 seconds.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV6_ERROR_INVALID_RATE_LIMIT_TIME - invalid sleep time
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_ipv6_async_set_rate_limit_time(ix_uint16 arg_SleepTime)
{

    if(( arg_SleepTime < IX_CC_ICMPV6_MIN_RATE_LIMIT_TIME) ||
       ( arg_SleepTime > IX_CC_ICMPV6_MAX_RATE_LIMIT_TIME))
    {
        return IX_ERROR_WARNING(IX_CC_IPV6_ERROR_INVALID_RATE_LIMIT_TIME,
                                ("Invalid value: arg_SleepTime "));
    }/* end if(arg_Callback == NULL)  */


    IX_ERROR_CRT(ix_cc_msup_send_msg(IX_CC_IPV6_MSG_INPUT,
                                     IX_CC_IPV6_MSG_SET_RATE_LIMIT_TIME,
                                     (void*)&arg_SleepTime,
                                     sizeof(ix_uint16)),
                 IX_CC_IPV6_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING);

    return IX_SUCCESS;

} /* end ix_cc_ipv6_async_set_rate_limit_time */

/**
 * NAME: ix_cc_ipv6_async_get_queue_depth
 *
 * DESCRIPTION:This message helper primitive helps client to retrieves the 
 * depth of the ICMP error message queue. Client retrieves the maximum number 
 * of ICMP messages that can fit in the queue, not the number of messages
 * currently in the queue.
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_ipv6_async_get_queue_depth(ix_cc_ipv6_cb_get_queue_depth arg_Callback,
                                          void* arg_pUserContext
                                         )
{
    ix_cc_ipv6_icmp_callback_data *cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_Callback "));
    }/* end if(arg_Callback == NULL)  */
    cbContext = (ix_cc_ipv6_icmp_callback_data*) ix_ossl_malloc(sizeof(ix_cc_ipv6_icmp_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */


    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_IPV6_MSG_INPUT,
                                           _ix_cc_ipv6_cb_icmp,
                                           (void *)cbContext,
                                           IX_CC_IPV6_MSG_GET_QUEUE_DEPTH,
                                           NULL,
                                           0),
                  err,
                  IX_CC_IPV6_ERROR_MSG_LIBRARY,
                  IX_ERROR_LEVEL_WARNING, cleanup_context);
   
    return IX_SUCCESS;

cleanup_context:

    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */

    return err;

} /* end ix_cc_ipv6_async_get_queue_depth  */

/**
 * NAME: ix_cc_ipv6_async_get_packets_to_drain
 *
 * DESCRIPTION: This message helper primitive helps client to retrieve 
 * the maximum number of ICMP error messages to send each time the queue 
 * processing event fires. The processing event will send up to this many 
 * ICMP error packets from the queue before going back to sleep. 
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_ipv6_async_get_packets_to_drain(
                               ix_cc_ipv6_cb_get_packets_to_drain arg_Callback,
                               void* arg_pUserContext)
{
    ix_cc_ipv6_icmp_callback_data *cbContext;
    ix_error err = IX_SUCCESS;
    

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("Invalid pointer: arg_pCallback "));
    }/* end if(arg_Callback == NULL)  */

    cbContext = (ix_cc_ipv6_icmp_callback_data*) 
                     ix_ossl_malloc(sizeof(ix_cc_ipv6_icmp_callback_data));

    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_IPV6_MSG_INPUT,
                                           _ix_cc_ipv6_cb_icmp,
                                           (void *)cbContext,
                                           IX_CC_IPV6_MSG_GET_PACKETS_TO_DRAIN,
                                           NULL,
                                           0),
                 err,
                 IX_CC_IPV6_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING, cleanup_context);
   
    return IX_SUCCESS;

cleanup_context:

    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */

    return err;

} /* end ix_cc_ipv6_async_get_packets_to_drain  */




/**
 * NAME: ix_cc_ipv6_async_set_packets_to_drain
 *
 * DESCRIPTION:This message helper primitive helps client to set the maximum 
 * number of ICMP error messages to send each time the queue processing event 
 * fires. The value to arg_PacketsToDrain must not exceed the depth of 
 * the queue.
 * 
 * @Param:  - IN arg_PacketsToDrain: The maximum number of packets to send 
 *            each time the queue processing event fires.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV6_ERROR_INVALID_PACKETS_TO_DRAIN - invalid value for packets to drain
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_ipv6_async_set_packets_to_drain(ix_uint16 arg_PacketsToDrain)
{

    if(arg_PacketsToDrain > g_Icmpv6QueueDepth)
    {
        return IX_ERROR_WARNING(IX_CC_IPV6_ERROR_INVALID_PACKETS_TO_DRAIN,
                                   ("Invalid value: arg_PacketsToDrain"));
    }/* end if(arg_Callback == NULL)  */


    IX_ERROR_CRT(ix_cc_msup_send_msg(IX_CC_IPV6_MSG_INPUT,
                                     IX_CC_IPV6_MSG_SET_PACKETS_TO_DRAIN,
                                     (void*)&arg_PacketsToDrain,
                                     sizeof(ix_uint16)),
                  IX_CC_IPV6_ERROR_MSG_LIBRARY,
                  IX_ERROR_LEVEL_WARNING);

    return IX_SUCCESS;

} /* end ix_cc_ipv6_async_set_packets_to_drain */

/**
 * NAME: ix_cc_ipv6_async_get_statistics
 *
 * DESCRIPTION: This message helper primitive helps client to retrieve 
 * statistics report from IPv6 Forwarder core component. This primitive 
 * will give the statistics from IPv6 microblock, RTMv6 and IPV6 Forwarder 
 * core component.  
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pUserContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_ipv6_async_get_statistics(
                                     ix_cc_ipv6_cb_get_statistics arg_Callback,
                                     void* arg_pUserContext)
{
    ix_cc_ipv6_statistics_callback_data *cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback "));
    }/* end if(arg_Callback == NULL)  */

    cbContext = (ix_cc_ipv6_statistics_callback_data *) 
               ix_ossl_malloc(sizeof(ix_cc_ipv6_statistics_callback_data));

    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_IPV6_MSG_INPUT,
                                           _ix_cc_ipv6_cb_statistics,
                                           (void *)cbContext,
                                           IX_CC_IPV6_MSG_GET_STATISTICS,
                                           NULL,
                                           0),
                  err,
                  IX_CC_IPV6_ERROR_MSG_LIBRARY,
                  IX_ERROR_LEVEL_WARNING, cleanup_context);

    return IX_SUCCESS;

cleanup_context:

    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */

    return err;

} /* end ix_cc_ipv6_async_get_statistics  */

/**
 * NAME: ix_cc_ipv6_async_send_icmp_error
 *
 * DESCRIPTION: This message helper primitive helps the client to send an ICMP
 * error message.  
 * 
 * @Param:  - IN arg_IcmpErrType: The ICMP error type to include in the message
 * @Param:  - IN arg_IcmpErrCode: The ICMP error code to include in the message
 * @Param:  - IN arg_hBuffer: The IPv6 packet causing the error.
 * @Param:    - IN arg_Param: Contains extra information if required by the ICMP
 *                            error message type.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */

ix_error ix_cc_ipv6_async_send_icmp_error(
                        ix_cc_ipv6_icmp_err_type    arg_IcmpErrType,
                        ix_cc_ipv6_icmp_err_code    arg_IcmpErrCode,
                        ix_buffer_handle            arg_hBuffer,
                        ix_uint32                    arg_Param,
                        ix_cc_ipv6_cb_route_op        arg_Callback,
                        void*                        arg_pUserContext)
{
    ix_cc_ipv6_icmp_err_data icmpData;
    ix_cc_ipv6_general_callback_data *cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_ipv6_general_callback_data*) 
                     ix_ossl_malloc(sizeof(ix_cc_ipv6_general_callback_data));
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    icmpData.icmpErrType = arg_IcmpErrType;
    icmpData.icmpErrCode = arg_IcmpErrCode;
    icmpData.hBuffer = arg_hBuffer;
    icmpData.param = arg_Param;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_IPV6_MSG_INPUT,
                                           _ix_cc_ipv6_cb_general,
                                           (void*)cbContext,
                                           IX_CC_IPV6_MSG_SEND_ICMP_ERROR,
                                           (void *)&icmpData,
                                           sizeof(ix_cc_ipv6_icmp_err_data)),
                 err,
                 IX_CC_IPV6_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:

    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);

    }/* end if cbContext != NULL */

    return err;
} /* end ix_cc_ipv6_async_send_icmp_error */

/**
 * NAME: ix_cc_ipv6_async_send_icmp_info_message
 *
 * DESCRIPTION: This message helper primitive helps the client to send an ICMP
 *                informational message.  
 * 
 * @Param:  - IN arg_SrcIpAddr: The IPv6 address to be included in the source
 *                                address field of the final IPv6 packet.
 * @Param:  - IN arg_DstAddr: The destination IPv6 address.
 * @Param:  - IN arg_Type: The type of the ICMP message. This value will be 
 *                            included in the ICMP header.
 * @Param:    - IN arg_Code: The ICMP message code corresponding to the message
 *                            type.
 * @Param:    - IN arg_PayloadLength: The length of the ICMP payload.
 * @Param:    - IN arg_pPayload: The payload to include in the ICMP packet.
y * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */
ix_error ix_cc_ipv6_async_send_icmp_info_message(
                                ix_uint128    arg_SrcIpAddr,
                                ix_uint128    arg_DstIpAddr,
                                ix_uint8    arg_Type,
                                ix_uint8    arg_Code,
                                ix_uint32    arg_PayloadLength,
                                ix_uint8    *arg_pPayload,
                                ix_cc_ipv6_cb_route_op arg_Callback,
                                void        *arg_pUserContext)
{
    ix_cc_ipv6_icmp_info_data icmpData;
    ix_cc_ipv6_general_callback_data *cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    if(arg_PayloadLength && !arg_pPayload)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pPayload"));
    }

    cbContext = (ix_cc_ipv6_general_callback_data*) 
                     ix_ossl_malloc(sizeof(ix_cc_ipv6_general_callback_data));
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    ix_ossl_memcpy(&icmpData.srcIpAddr, &arg_SrcIpAddr, sizeof(ix_uint128));
    ix_ossl_memcpy(&icmpData.dstIpAddr, &arg_DstIpAddr, sizeof(ix_uint128));
    icmpData.type = arg_Type;
    icmpData.code = arg_Code;
    icmpData.payloadLength = arg_PayloadLength;
    
    icmpData.pPayload = (ix_uint8 *) ix_ossl_malloc(arg_PayloadLength);
    if(icmpData.pPayload == NULL)
    {
         return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for payload"));
    }

    ix_ossl_memcpy(icmpData.pPayload, arg_pPayload, arg_PayloadLength);
    
    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_IPV6_MSG_INPUT,
                                           _ix_cc_ipv6_cb_general,
                                           (void*)cbContext,
                                           IX_CC_IPV6_MSG_SEND_ICMP_INFO,
                                           (void *)&icmpData,
                                           sizeof(ix_cc_ipv6_icmp_info_data)),
                 err,
                 IX_CC_IPV6_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:

    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);

    }/* end if cbContext != NULL */

    if(icmpData.pPayload != NULL)
    {
        ix_ossl_free(icmpData.pPayload);
    }
    
    return err;
} /* end ix_cc_ipv6_async_send_icmp_info */

/**
 * NAME: ix_cc_ipv6_async_perform_addr_resolution
 *
 * DESCRIPTION: This message helper primitive helps the cllient to request
 *              address resolution to be performed for a packet destined
 *              to a neighbor.
 * 
 * @Param:  - IN arg_hBuffer: The hardware buffer containing the ethernet 
 *                            packet to be sent to the neighbor
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */
ix_error ix_cc_ipv6_async_perform_addr_resolution(
                                    ix_buffer_handle arg_hBuffer,
                                    ix_cc_ipv6_cb_route_op  arg_Callback,
                                    void* arg_pUserContext)
{
    ix_cc_ipv6_general_callback_data *cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_ipv6_general_callback_data*) 
                     ix_ossl_malloc(sizeof(ix_cc_ipv6_general_callback_data));
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_IPV6_MSG_INPUT,
                                           _ix_cc_ipv6_cb_general,
                                           (void*)cbContext,
                                         IX_CC_IPV6_MSG_PERFORM_ADDR_RESOLUTION,
                                           (void *)&arg_hBuffer,
                                           sizeof(ix_buffer_handle)),
                 err,
                 IX_CC_IPV6_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:

    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);

    }/* end if cbContext != NULL */

    return err;
}/*end of ix_cc_ipv6_async_perform_addr_resolution*/


/**
 * NAME: ix_cc_ipv6_async_add_neighbor
 *
 * DESCRIPTION: This message helper primitive helps the cllient to add a 
 *              neighbor to the L2 table
 * 
 * @Param:  - IN arg_L2Index: The index into the L2 table at which this 
 *                            information should be added.
 * @Param   - IN arg_NbrIp:   The IPv6 address of the neighbor.
 * @Param   - IN arg_OutputPort: The egress port from which all traffic 
 *                            destined for the neighbor should exit.
 * @Param   - IN arg_L2Addr: The MAC address of the neighbor.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */
ix_error ix_cc_ipv6_async_add_neighbor(
                                ix_uint16 arg_L2Index,
                                ix_uint128 arg_NbrIp,
                                ix_uint16 arg_OutputPort,
                                ix_uint8 *arg_L2Addr,
                                ix_cc_ipv6_cb_route_op arg_Callback,
                                void* arg_pUserContext)
{
    ix_cc_ipv6_add_nbr_data   nbrData;
    ix_cc_ipv6_general_callback_data *cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    if(arg_L2Addr == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_L2Addr"));
    }
    
    cbContext = (ix_cc_ipv6_general_callback_data*) 
                     ix_ossl_malloc(sizeof(ix_cc_ipv6_general_callback_data));
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */
    
    nbrData.l2Addr = (ix_uint8 *)ix_ossl_malloc(IX_CC_MAC_ADDR_LEN);
    if(!nbrData.l2Addr)
    {
        goto cleanup_context;
    }
    
    nbrData.l2Index = arg_L2Index;
    ix_ossl_memcpy(&nbrData.nbrIp, &arg_NbrIp, sizeof(ix_uint128));
    nbrData.outputPort = arg_OutputPort;
    ix_ossl_memcpy(nbrData.l2Addr, arg_L2Addr,IX_CC_MAC_ADDR_LEN);
    
    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_IPV6_MSG_INPUT,
                                           _ix_cc_ipv6_cb_general,
                                           (void*)cbContext,
                                           IX_CC_IPV6_MSG_ADD_NBR,
                                           (void *)&nbrData,
                                           sizeof(ix_cc_ipv6_add_nbr_data)),
                 err,
                 IX_CC_IPV6_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:

    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */

    if(nbrData.l2Addr)
    {
        ix_ossl_free(nbrData.l2Addr);
    }/*end of if(nbrData.l2Addr)*/

    return err;
}/*end of ix_cc_ipv6_async_add_neighbor*/

/**
 * NAME: ix_cc_ipv6_async_del_neighbor
 *
 * DESCRIPTION: This message helper primitive helps the cllient to add a 
 *              neighbor to the L2 table
 * 
 * @Param:  - IN arg_L2Index: The index into the L2 table at which this 
 *                            information should be added.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */
ix_error ix_cc_ipv6_async_del_neighbor(
                                    ix_uint16 arg_L2Index,
                                    ix_cc_ipv6_cb_route_op arg_Callback,
                                    void* arg_pUserContext)
{   
    ix_cc_ipv6_general_callback_data *cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_ipv6_general_callback_data*) 
                     ix_ossl_malloc(sizeof(ix_cc_ipv6_general_callback_data));
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_IPV6_MSG_INPUT,
                                           _ix_cc_ipv6_cb_general,
                                           (void*)cbContext,
                                           IX_CC_IPV6_MSG_DEL_NBR,
                                           (void *)&arg_L2Index,
                                           sizeof(ix_uint16)),
                 err,
                 IX_CC_IPV6_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:

    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */

    return err;
}/*end of ix_cc_ipv6_async_del_neighbor*/


/**
 * NAME: ix_cc_ipv6_sync_add_neighbor
 *
 * DESCRIPTION: This message helper primitive helps the cllient to add a
 *              neighbor to the L2 table synchronously. This call MUST NOT
 *              be invoked by core components.
 *
 * @Param:  - IN arg_L2Index: The index into the L2 table at which this
 *                            information should be added.
 * @Param   - IN arg_NbrIp:   The IPv6 address of the neighbor.
 * @Param   - IN arg_OutputPort: The egress port from which all traffic
 *                            destined for the neighbor should exit.
 * @Param   - IN arg_L2Addr: The MAC address of the neighbor.
 *
 * @Param:  - OUT - none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 *
 */
ix_error ix_cc_ipv6_sync_add_neighbor(
                                    ix_uint16 arg_L2Index,
                                    ix_uint128 arg_NbrIp,
                                    ix_uint16 arg_OutputPort,
                                    ix_uint8 *arg_L2Addr)
{
    ix_cc_ipv6_add_nbr_data nbrData;
    ix_error                err = IX_SUCCESS;
    ix_error                *retCode;
    ix_uint32               retMsgLen = 0;

    nbrData.l2Index = arg_L2Index;
    ix_ossl_memcpy(&nbrData.nbrIp, &arg_NbrIp, sizeof(ix_uint128));
    nbrData.outputPort = arg_OutputPort;
    if(arg_L2Addr)
    {
        nbrData.l2Addr = (ix_uint8 *) ix_ossl_malloc(IX_CC_MAC_ADDR_LEN);
        if(!nbrData.l2Addr)
            return IX_CC_ERROR_OOM;
        ix_ossl_memcpy(nbrData.l2Addr, arg_L2Addr,IX_CC_MAC_ADDR_LEN);
    }
    else
        nbrData.l2Addr = NULL;
    
    err = ix_cc_msup_send_sync_msg(IX_CC_IPV6_MSG_INPUT,
                                           IX_CC_IPV6_MSG_ADD_NBR,
                                           (void *)&nbrData,
                                           sizeof(ix_cc_ipv6_add_nbr_data),
                                           (void **)retCode,
                                           &retMsgLen);

    if(nbrData.l2Addr)
        ix_ossl_free(nbrData.l2Addr);

    /* if the call to message support library was successful, return whatever
     * return code we obtain, else return the err value indicating why
     * the call to msup failed*/
    if(err == IX_SUCCESS)
        return *retCode;
    else
        return err;

}

/**
 * NAME: ix_cc_ipv6_sync_del_neighbor
 *
 * DESCRIPTION: This message helper primitive helps the cllient to add a
 *              neighbor to the L2 table synchronously. This call MUST NOT
 *              be invoked by other core components.
 *
 * @Param:  - IN arg_L2Index: The index into the L2 table at which this
 *                            information should be added.
 *
 * @Param:  - OUT - none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 *
 */
ix_error ix_cc_ipv6_sync_del_neighbor(
                                 ix_uint16 arg_L2Index)
{

    ix_error    err = IX_SUCCESS;
    ix_error    *retCode;
    ix_uint32   retMsgLen = 0;
    
    err = ix_cc_msup_send_sync_msg(IX_CC_IPV6_MSG_INPUT,
                                   IX_CC_IPV6_MSG_DEL_NBR,
                                   (void *)&arg_L2Index,
                                   sizeof(ix_uint16),
                                   (void **)retCode,
                                   &retMsgLen);
    if(err == IX_SUCCESS)
        return *retCode;
    else
        return err;
}
