/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 6
 *
 * = FILENAME
 *      ix_cc_ipv6_nd.c
 *
 * = DESCRIPTION
 *      This file contains the basic Neighbor Discovery and Address 
 *      Autoconfiguration implementations for IPv6 
 *
 * = CHANGE HISTORY
 *       1/8/2002 1:53:42 PM - creation time
 */

static const char s_FileIdentifier[] = "$Id: ix_cc_ipv6_nd.c,v 1.32.2.1 2004/03/22 18:34:03 rranjeet Exp $";
/**
 * System defined include files required.
 */
#if !(_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
#include <stdlib.h>
#endif

/**
 * User defined include files required.
 */
#include "ix_cc_error.h"
#include "ix_ossl.h"
#include "ix_rm.h"
#include "ix_cc.h"
#include "ix_cc_macros.h"

#include "ix_cc_properties.h"
#include "cc/ix_cc_msup.h"

#include "cc/ipv6/internal/ix_cc_ipv6_nd.h"
#include "cc/ipv6/internal/v6_header_defs.h"
#include "cc/ipv6/internal/ipv6_fwdr.h"
#include "cc/ipv6/internal/icmp6.h"
#include "cc/ipv6/internal/ix_cc_ipv6_icmp_nd.h"
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
#include "cc/ipv6/internal/ix_cc_ipv6_slist.h"
#else
#include "cc/ix_cc_slist.h"
#endif
#include "cc/ipv6/internal/ix_cc_ipv6_msg_handler.h"
#include "cc/ipv6/internal/utils.h"

#ifdef USE_FPM_CC
#include "cc/ix_cc_fpm_core.h"
#endif

#ifdef MEAT_TEST
 LPLIST g_Context_Nd;
#endif

/**
 * Pre-processor symbols and macros used in this file.
 */
#define IX_CC_IPV6_ND_EVENT_PRIORITY 1
#define IX_CC_IPV6_ND_ETH_HDR_LEN    14
#define IX_CC_IPV6_ND_ETH_TRLR_LEN   4
#define ND_LOCK() ix_ossl_mutex_lock(g_nd_mutex, IX_OSSL_WAIT_FOREVER)
#define ND_UNLOCK() ix_ossl_mutex_unlock(g_nd_mutex)
#define CHK_INIT() \
    if(!g_Initialized) \
    {\
        return IX_CC_IPV6_ERROR_NOT_INITIALIZED;\
    }

       
#define MAKE_UNSPEC_ADDR(__addr) \
    {                            \
        ix_ossl_memset(&__addr, 0, sizeof(ix_uint128));\
    }

#define MAKE_SOL_NODE_MASK(__addr)  \
    {                               \
        ix_ossl_memset(&__addr, 0xFF, sizeof(ix_uint128));\
        __addr.aUint8[13] = __addr.aUint8[14] = __addr.aUint8[15] = 0; \
    }

/*__src is a pointer. __dest is a pointer to a pointer*/
#define MAKE_SOL_NODE_ADDR(__src, __dest)       \
    {                                           \
        ix_ossl_memset(__dest, 0, sizeof(ix_uint128));\
        (__dest)->aUint8[0] = 0xFF;                \
        (__dest)->aUint8[1] = 0x02;                \
        (__dest)->aUint8[11] = 0x01;               \
        (__dest)->aUint8[12] = 0xFF;               \
        (__dest)->aUint8[13] = __src.aUint8[13];   \
        (__dest)->aUint8[14] = __src.aUint8[14];   \
        (__dest)->aUint8[15] = __src.aUint8[15];   \
    }

#define MAKE_LINK_LOCAL_MASK(__addr)    \
    {                                   \
        ix_ossl_memset(&__addr, 0xFF, sizeof(ix_uint128)/2);\
        ix_ossl_memset(&__addr.aUint8[8], 0, sizeof(ix_uint128)/2);\
    }

#define MAKE_BLANK_LL_ADDR(__addr)  \
    {                               \
        ix_ossl_memset(&__addr, 0, sizeof(ix_uint128));\
        __addr.aUint8[0] = 0xFE;    \
        __addr.aUint8[1] = 0x80;    \
    }

#define MAKE_ALL_NODES_ADDR(__addr) \
    {                               \
        ix_ossl_memset(&__addr, 0, sizeof(ix_uint128));\
        __addr.aUint8[0] = 0xFF;    \
        __addr.aUint8[1] = 0x02;    \
        __addr.aUint8[15] = 0x01;   \
    }

#define MAKE_ALL_RTRS_ADDR(__addr)  \
    {                               \
        ix_ossl_memset(&__addr, 0, sizeof(ix_uint128));\
        __addr.aUint8[0] = 0xFF;    \
        __addr.aUint8[1] = 0x02;    \
        __addr.aUint8[15] = 0x02;   \
    }

#define IX_CC_ND_EVENT_EXTRA_DELAY  3000  /* in msec */

/**
 * Types definitions whose scope is limited to this file.
 */

#define ETH_IPV6_PROTO  0x86DD

/**
 * Variable declarations global to this file only.  Externs are followed by
 * static variables.
 */
static ix_ossl_mutex_t g_nd_mutex;
static ix_uint8        g_Initialized = FALSE;
static ix_uint32       g_DadTransmits;
static LPLIST          g_portList;
static LPLIST          g_nbrCache;
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
extern ix_cc_ipv6_context *g_pIpv6Context;
#else
static ix_cc_ipv6_context *g_pIpv6Context;
#endif

/**
 * Extern function prototypes.
 */

/**
 * Static function prototypes. Function descriptions are provided in respective
 * function headers.
 */

static ix_error ix_cc_nd_update_properties(ix_cc_properties_node *pProperties);
#ifdef SEND_RTR_ADVS
static ix_error ix_cc_nd_start_unsol_rtr_advs(void);
#endif
static ix_error ix_cc_nd_process_rtr_sol( ix_buffer_handle arg_hBuffer,
                                     ix_uint8   *arg_pIpv6Pkt,
                                     ix_uint8   *arg_pIcmpPkt);
static ix_error ix_cc_nd_process_nbr_sol(ix_buffer_handle arg_hBuffer,
                                  ix_uint8  *arg_pIpv6Pkt, 
                                  ix_uint8 arg_srcIpAddrSpecified,
                                  ix_ipv6_nd_options_l2_addr *arg_pL2Opt);
static ix_error ix_cc_nd_process_nbr_adv(ix_uint8  *arg_pIpv6Pkt, 
                                  ix_ipv6_nd_options_l2_addr *arg_pL2Opt,
                                  ix_ipv6_nd_port *pPort);
static ix_error ix_cc_nd_validate_rtr_sol(ix_uint8 *pPkt, 
                                   ix_uint8 *pIcmpPkt,
                                   ix_uint32 icmpOffset,
                                   ix_uint8 *arg_IpAddrIsSpecified,
                                  ix_ipv6_nd_options_l2_addr **arg_pL2Options);
static ix_error ix_cc_nd_validate_nbr_sol(ix_buffer_handle arg_hBuffer,
                                   ix_uint8 *pPkt, 
                                   ix_uint8 *pIcmpPkt,
                                   ix_uint8 *arg_IpAddrIsSpecified,
                                   ix_ipv6_nd_options_l2_addr **arg_pL2Options);
static ix_error ix_cc_nd_validate_nbr_adv(ix_buffer_handle arg_hBuffer,
                                   ix_uint8     *arg_pIpv6Pkt, 
                                   ix_uint8     *arg_pIcmpPkt,
                                   ix_ipv6_nd_options_l2_addr **arg_pL2Options);
static ix_error ix_cc_nd_validate_nbr_sol_target_address(
                                                ix_buffer_handle arg_hBuffer,
                                                ix_uint128 arg_ipAddr,
                                                ix_uint8 arg_srcAddrSpecified);
static ix_error ix_cc_nd_validate_nbr_adv_target_addr(
                                                ix_buffer_handle arg_hBuffer,
                                                ix_uint128   arg_ipAddr);
static ix_error ix_cc_nd_send_unsol_rtr_adv(void *arg_pContext);
static ix_error ix_cc_nd_send_sol_nbr_adv(ix_buffer_handle arg_hBuffer,
                                   ix_uint8     *pIpv6Pkt,
                                   ix_uint8     srcAddrSpecified,
                                   ix_ipv6_nd_options_l2_addr *arg_pL2Opt);
ix_error ix_cc_nd_send_queued_pkts(ix_ipv6_nd_nbr_info *pNbrEntry, ix_error result);
static ix_error ix_cc_nd_send_icmp_unreachable(LPLIST arg_PktList);
ix_error ix_cc_nd_handle_nbr_sol_event(void *arg_pContext);
static ix_error ix_cc_nd_extract_pkt_from_buffer(ix_buffer_handle arg_hBuffer,
                                             ix_uint8         **arg_pPkt);
static ix_uint8 ix_cc_nd_locate_icmp_pkt(ix_uint8  *arg_pIpv6Pkt,
                                  ix_uint8  **arg_pIcmpLocation,
                                  ix_uint32 *arg_Offset);
static ix_error ix_cc_nd_find_port(ix_uint16 arg_PortId,
                                  ix_ipv6_nd_port **arg_PortStruct);
static ix_error ix_cc_nd_find_nbrCache_entry(ix_uint128 ipAddr, 
                                   ix_ipv6_nd_nbr_info **pNbrEntry);
static ix_error ix_cc_nd_create_nbr_entry(ix_ipv6_nd_nbr_info **arg_pNbrEntry);
static ix_error ix_cc_nd_cleanup_nbr_cache(void);
static ix_error ix_cc_nd_cleanup_port_list(void);
static ix_error ix_cc_nd_free_buffer_list(LPLIST arg_PktList, 
                                    ix_error arg_Result,
                                    ix_uint8 atg_freeChain);
#if defined(XML_SYSAPP_SUPPORT)
static ix_uint8 ix_cc_nd_is_ll_ip_addr(ix_uint128 arg_ipAddr);
#endif
static ix_error ix_cc_nd_cb_del_nbr(ix_error arg_Result,
                                      void *arg_pContext);
static ix_error ix_cc_nd_update_l2_table(ix_ipv6_nd_nbr_info *pNbrEntry,
                                  ix_ipv6_nd_port *pPort);
static ix_error ix_cc_nd_cb_l2_update(ix_error arg_Result,
                               void *arg_pContext);
static ix_error ix_cc_nd_start_addr_resolution(ix_ipv6_nd_nbr_info *pNbrEntry,
                                               ix_ipv6_nd_port *pPort);
ix_error ix_cc_nd_add_eth_hdr(ix_uint8 *pMacAddr, ix_buffer_handle arg_hBuffer);
/*Addr autoconf functions*/
static ix_error ix_cc_addr_conf_init(void);
static ix_error ix_cc_addr_conf_generate_ll_addr(ix_ipv6_nd_port *arg_pPort, 
                                          ix_uint128 *ipAddr);
static ix_error ix_cc_addr_conf_set_dad_event(ix_ipv6_nd_port *arg_pPort,
                                    ix_ipv6_addr_conf_ip_struct *arg_pIpStruct);
static ix_uint8 ix_cc_addr_conf_is_ip_ptr_valid(ix_ipv6_nd_port *arg_pPort,
                                 ix_ipv6_addr_conf_ip_struct *arg_pIpStruct);
static ix_uint8 ix_cc_addr_conf_find_ip_addr(ix_ipv6_nd_port *arg_pPort,
                                  ix_uint128 ipAddr,
                                  ix_ipv6_addr_conf_ip_struct **arg_pIpStruct);
static ix_error ix_cc_addr_conf_find_ll_addr_struct(ix_ipv6_nd_port *pPort, 
                                            ix_ipv6_addr_conf_ip_struct **pIp);

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)  
extern ix_uint32 ix_cc_rand(void);
#endif

/**
 * Function definitions.
 */

/**
 * TYPENAME: ix_cc_nd_init
 *
 * DESCRIPTION: Initializes the Neighbor Discovery and Address 
 *              Autoconfiguration features.
 *
 * @Param: - IN properties - property descriptions for all interfaces.
 * @Param: - OUT - None
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 *
 */
ix_error ix_cc_nd_init(ix_cc_properties_node *arg_pProperties,
                       ix_cc_ipv6_context *arg_pIpv6Context)
{
    ix_error err;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */

    if(!arg_pProperties || !arg_pIpv6Context)
        return IX_CC_IPV6_ERROR_INVALID_INPUT_PARAM;

    if(g_Initialized)
        return IX_SUCCESS;

    if((g_nbrCache = ListCreate()) == NULL)
        return IX_CC_ERROR_OOM;

    err = ix_ossl_mutex_init(IX_OSSL_MUTEX_UNLOCK, &g_nd_mutex);
    if(err != IX_SUCCESS)
        return err;

    /*read the registry and initialize variables*/
    ND_LOCK();
    g_DadTransmits = 2; /*Default value. To be changed when this is read from 
                          the registry*/
    g_pIpv6Context = arg_pIpv6Context;

    IX_IPV6_ERROR_CGT(ix_cc_nd_update_properties(arg_pProperties),
                      err, LBL_ERR);
#ifdef MEAT_TEST
    g_Context_Nd = ListCreate();
    if(!g_Context_Nd)
    {
        err = IX_CC_ERROR_OOM;
        goto LBL_ERR;
    }
#endif

    IX_IPV6_ERROR_CGT(ix_cc_addr_conf_init(), err, LBL_ERR);

#ifdef SEND_RTR_ADVS
    /*Start the router advertisements*/
    IX_IPV6_ERROR_CGT(ix_cc_nd_start_unsol_rtr_advs(), err, LBL_ERR);
#endif

    g_Initialized = TRUE;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("IPV6 ND INitialized successfully.\n");
#endif /* IPV6_DEBUG */

LBL_ERR:
    if(err != IX_SUCCESS)
    {
        ix_ossl_message_log("@@@ IPV6 ND Initialization failed.\n");
    }
    ND_UNLOCK();
    return err;
}/*ix_cc_nd_init*/

/**
 * TYPENAME: ix_cc_nd_fini
 *
 * DESCRIPTION: Shuts down the ND component.
 * 
 * @Param: - None
 *
 * @Return
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_fini(void)
{

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */

    CHK_INIT();
    ND_LOCK();
    ix_cc_nd_cleanup_nbr_cache();
    ix_cc_nd_cleanup_port_list();
    g_Initialized = FALSE;

#ifdef MEAT_TEST
    if(g_Context_Nd)
        ListDelete(g_Context_Nd);
#endif
    ND_UNLOCK();

    ix_ossl_mutex_fini(g_nd_mutex);

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("IPv6 ND De-Initialized\n");
#endif /* IPV6_DEBUG */

    return IX_SUCCESS;
}/*ix_cc_nd_fini*/

/**
 * TYPENAME: ix_cc_nd_process_incoming_packet
 *
 * DESCRIPTION: The common function to call for processing any type
 *              of packet for ND.
 * 
 * @Param:  - IN arg_hBuffer - Handle to the buffer containing the packet.
 * @Param:  - OUT - None.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_process_incoming_packet(ix_buffer_handle arg_hBuffer)
{
    ix_error            errVal;
    ix_uint8            *pIpv6Pkt;
    ix_cc_ipv6_header   *pIpv6Header;
    ix_cc_ipv6_icmp_header *pIcmpHeader;
    ix_uint32           icmpOffset;    
    ix_uint32           pktLen;    
    ix_uint8                srcIpAddr;
    ix_ipv6_nd_options_l2_addr *pL2Opt;
    ix_hw_buffer_meta   *pHwMeta;
    ix_ipv6_nd_port     *pPort;

    CHK_INIT();

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */

    errVal = ix_cc_nd_extract_pkt_from_buffer(arg_hBuffer, &pIpv6Pkt);
    if(errVal != IX_SUCCESS)
        return errVal;

    pIpv6Header = (ix_cc_ipv6_header *) pIpv6Pkt;
        
    if(!ix_cc_nd_locate_icmp_pkt(pIpv6Pkt, (ix_uint8 **)&pIcmpHeader, 
                                 &icmpOffset))
    {
        /*ICMP header not found. This is probably not an ICMP packet*/
        errVal = IX_CC_IPV6_ERROR_INVALID_PACKET; 
        goto LBL_EXIT;
    }
    
    if(pIpv6Header->hopLimit != IX_CC_IPV6_HOP_LIMIT_MAX)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: IPv6 hop limit check failed\n");
#endif /* IPV6_DEBUG */
        errVal = IX_CC_ERROR_RANGE;
        goto LBL_EXIT;
    }

    /* Now calculate the ICMP packet length. Start with the total IPv6
     * pkt length, including the options headers*/
    pktLen = IX_NTOH16(pIpv6Header->payloadLength)+IX_CC_IPV6_HDR_LEN;
    /* to get ICMP pkt length, subtract the offset at which the ICMP pkt 
     * occurs*/
    pktLen -= icmpOffset;

    errVal = _ix_cc_ipv6_icmp_validate_checksum((ix_uint8 *)pIpv6Header,
                                         pktLen, (ix_uint8 *)pIcmpHeader);
    if(errVal != IX_SUCCESS)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Checksum calculation failed\n");
#endif /* IPV6_DEBUG */
        goto LBL_EXIT;
    }

    pL2Opt = NULL;
    switch(pIcmpHeader->type)
    {
        case IX_CC_IPV6_ICMP_TYPE_RTR_SOL:
#if defined(IPV6_DEBUG)
            ix_ossl_message_log("IPv6ND: Received RTR SOL message\n");
#endif /* IPV6_DEBUG */
            errVal = ix_cc_nd_process_rtr_sol(
                                        arg_hBuffer, pIpv6Pkt, 
                                        (ix_uint8 *) pIcmpHeader);
            break;
        case IX_CC_IPV6_ICMP_TYPE_NBR_SOL:
#if defined(IPV6_DEBUG)
            ix_ossl_message_log("Received NBR SOL message\n");
#endif /* IPV6_DEBUG */
            errVal =  ix_cc_nd_validate_nbr_sol(arg_hBuffer, pIpv6Pkt, 
                                                (ix_uint8 *)pIcmpHeader, 
                                                &srcIpAddr, &pL2Opt);
            if(errVal != IX_SUCCESS)
                goto LBL_EXIT;
            errVal = ix_cc_nd_process_nbr_sol(
                                        arg_hBuffer, pIpv6Pkt, 
                                        srcIpAddr, pL2Opt);
            break;
        case IX_CC_IPV6_ICMP_TYPE_NBR_ADV:
#if defined(IPV6_DEBUG)
            ix_ossl_message_log("Received NBR ADV message\n");
#endif /* IPV6_DEBUG */
            errVal =  ix_cc_nd_validate_nbr_adv(arg_hBuffer, pIpv6Pkt, 
                                                (ix_uint8 *)pIcmpHeader, 
                                                &pL2Opt);
            if(errVal != IX_SUCCESS)
                goto LBL_EXIT;
 
            /*Get the port structure of the input port*/
            errVal = ix_rm_buffer_get_meta(arg_hBuffer, (void **)&pHwMeta);

            if(errVal != IX_SUCCESS)
                goto LBL_EXIT;

            errVal = ix_cc_nd_find_port(pHwMeta->m_InputPort, &pPort);
            if( (errVal != IX_SUCCESS) || !pPort)
                goto LBL_EXIT;
 
            errVal = ix_cc_nd_process_nbr_adv((ix_uint8 *)pIcmpHeader, pL2Opt,
                                              pPort);
            break;
        default:
#if defined(IPV6_DEBUG)
            ix_ossl_message_log("IPv6ND: Unknown message received, Dropping...\n");
#endif /* IPV6_DEBUG */
            errVal = IX_CC_IPV6_ERROR_INVALID_PACKET;
    }
    
LBL_EXIT:
    if(errVal != IX_SUCCESS)
    {
        ix_rm_buffer_free_chain(arg_hBuffer);
    }

    ix_ossl_free(pIpv6Pkt);
    return IX_SUCCESS;
}/*ix_cc_nd_process_incoming_packet*/

/**
 * TYPENAME: ix_cc_nd_perform_addr_resolution
 *
 * DESCRIPTION:
 *          This function is called by the ethernet TX CC when it gets
 *          a packet with an invalid L2 ID. It first looks for the 
 *          neighbor's L2 address in the neighbor cache. If found, it
 *          updates the L2 table. If not found, it starts the Address
 *          Resolution process per RFC2461, section 7.2.
 *          
 * @Param:  - IN arg_hBuffer - Handle to the buffer containing the packet.
 * @Param:  - OUT - None.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_perform_addr_resolution(ix_buffer_handle arg_hBuffer,
                                          void *arg_pContext)
{
    ix_error            errVal = IX_SUCCESS;
    ix_hw_buffer_meta   *pHwMeta;
    void                *pData;
    ix_ipv6_nd_port     *pPort;
    ix_cc_ipv6_header   *pIpHdr;
    ix_ipv6_nd_nbr_info *pNbrEntry;
    ix_ipv6_nd_pkt_queue_struct *pktQueue = NULL;
    ix_uint16           outputPort;
    ix_uint128          ipv6Addr;
    
#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */

    IX_ERROR_CGT(ix_rm_buffer_get_meta(arg_hBuffer, 
                                      (void **)&pHwMeta),
                 errVal,
                 IX_CC_IPV6_ERROR_CCI, 
                 IX_ERROR_LEVEL_WARNING,
                 LBL_EXIT_NO_ND_UNLOCK);

    /* Sanity check: The output port should be on this blade*/
    outputPort = pHwMeta->m_OutputPort; /*This should remain in NBO*/
    ND_LOCK();
    errVal = ix_cc_nd_find_port(outputPort, &pPort);
    if( (errVal != IX_SUCCESS) || !pPort)
        goto LBL_EXIT;
    
    /* Allocate memory for the packet queue structure and store it in 
    * the packet queue*/
    pktQueue = (ix_ipv6_nd_pkt_queue_struct *)
                        ix_ossl_malloc(sizeof(ix_ipv6_nd_pkt_queue_struct));
    if(!pktQueue)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Out of memory: pktQueue\n");
#endif /* IPV6_DEBUG */
        errVal = IX_CC_ERROR_OOM;
        goto LBL_EXIT;
    }
            
    pktQueue->hBuffer = arg_hBuffer;
    pktQueue->pContext = arg_pContext;

    /* Now the packet queue buffer is ready. We'll need to add it to the 
     * packet queue in the nbr cache entry when we find it*/

    IX_IPV6_ERROR_CGT(ix_cc_hw_get_packet_data(arg_hBuffer, &pData),
                      errVal, LBL_ERR);

    pIpHdr = (ix_cc_ipv6_header *)pData;

    ix_ossl_memcpy(&ipv6Addr, &pIpHdr->dstAddress, sizeof(ix_uint128));
    
    IX_IPV6_ERROR_CGT(ix_cc_nd_find_nbrCache_entry(ipv6Addr, &pNbrEntry),
                      errVal, LBL_ERR);
    if(pNbrEntry)
    {
        pNbrEntry->l2Id = IX_NTOH16(pHwMeta->m_NextHopID);
        
        ListAddItem(pNbrEntry->packetQueue, pktQueue);

        if(pNbrEntry->isValidL2Addr && 
           (pNbrEntry->state == IX_REACH_TYPE_REACHABLE))
        {
#if defined(IPV6_DEBUG)
            ix_ossl_message_log("State is REACHABLE. Updating the L2 table\n");
#endif /* IPV6_DEBUG */

            /*found the entry. Update the L2 Table, and forward the packet*/
            errVal = ix_cc_nd_update_l2_table(pNbrEntry, pPort);
            if(errVal != IX_SUCCESS)
            {
                ix_ossl_message_log("ND updating the L2 table failed\n");
            }

            /*Send all queued packets*/
            /* The called function takes care of the buffer in case of error */
            errVal = ix_cc_nd_send_queued_pkts(pNbrEntry, IX_SUCCESS);

            ND_UNLOCK();
            return(errVal);
        }
    }/*if(pNbrEntry)*/ 
    else
    {
        /*Create a new nbr entry and set the Nbr sol timer*/
        IX_IPV6_ERROR_CGT(ix_cc_nd_create_nbr_entry(&pNbrEntry), 
                          errVal, LBL_ERR);
        ix_ossl_memcpy(&pNbrEntry->ipAddr, &pIpHdr->dstAddress, 
                       IX_CC_IPV6_ADDRESS_LENGTH_BYTES);

        pNbrEntry->outputPort = outputPort;
        pNbrEntry->state = IX_REACH_TYPE_INCOMPLETE;
        pNbrEntry->l2Id = IX_NTOH16(pHwMeta->m_NextHopID);

        /*add the packet queue biffer to the queue*/
        ListAddItem(pNbrEntry->packetQueue, pktQueue);

#if defined(IPV6_DEBUG)
        ix_ossl_message_log("Created a new Nbr Cache Entry and set the State to "
                            "INCOMPLETE\n");
#endif /* IPV6_DEBUG */
    }

    errVal = ix_cc_nd_start_addr_resolution(pNbrEntry, pPort);
    ND_UNLOCK();
    return errVal;

LBL_ERR:
    if (pktQueue != NULL)
    {
        ix_ossl_free(pktQueue);
    }
LBL_EXIT:
    ND_UNLOCK();
LBL_EXIT_NO_ND_UNLOCK:
    ix_rm_buffer_free_chain(arg_hBuffer);
    return errVal;
}/*ix_cc_nd_perform_addr_resolution*/


/**
 * TYPENAME: ix_cc_nd_add_neighbor
 *
 * DESCRIPTION:
 *          This is an internal function which is called when the client makes
 *          an add neighbor request. This function searches for the Neighbor
 *          cache entry corresponding to the IP address. If not found, it
 *          creates one and updates it. It also tries to verify the L2 addr
 *          by sending a Nbr Sol to the neighbor.
 *
 * @Param:  - IN arg_L2Index - The index into the L2 table where this info is 
 *                             to be added.
 * @Param:  - IN arg_NbrIp - The neighbor's IP address.
 * @Param:  - IN arg_OutputPort - The egress port.
 * @Param:  - IN arg_L2Addr - The neoghbor's MAC address.
 * @Param:  - IN arg_pUserContext - The client's context.
 * @Param:  - OUT None.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_add_neighbor(
                ix_uint16 arg_L2Index,
                ix_uint128 arg_NbrIp,
                ix_uint16 arg_OutputPort,
                ix_uint8* arg_L2Addr,
                void* arg_pUserContext)
{
    ix_error            errVal;
    ix_ipv6_nd_port     *pPort;
    ix_ipv6_nd_nbr_info *pNbrEntry;
    ix_uint128          tempIp;
    ix_ipv6_nd_pkt_queue_struct *pktQueue;
    ix_uint8            resolveAddress = TRUE;/*FLAG: indicates whether to 
                                                start address resolution*/

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */

    ND_LOCK();
    /* Get the port corresponding to the output port. We need that to get the
     * src MAC address*/
    errVal = ix_cc_nd_find_port(IX_HTON16(arg_OutputPort), &pPort);
    if((errVal != IX_SUCCESS) || (!pPort))
        goto LBL_MSG;
   
    /* Find the nbr cache entry. If not found, create a new one.*/
    IX_HTON128(&arg_NbrIp, &tempIp);
    errVal = ix_cc_nd_find_nbrCache_entry(tempIp, &pNbrEntry);
    if(errVal != IX_SUCCESS)
        goto LBL_MSG;

    if(!pNbrEntry)
    {
        /*Create a new neighbor entry*/
        IX_IPV6_ERROR_CGT(ix_cc_nd_create_nbr_entry(&pNbrEntry),
                          errVal, LBL_MSG);

        ix_ossl_memcpy(&pNbrEntry->ipAddr, &tempIp, sizeof(ix_uint128));
        pNbrEntry->state = IX_REACH_TYPE_INCOMPLETE; 
        pNbrEntry->outputPort = IX_HTON16(arg_OutputPort);

#if defined(IPV6_DEBUG)
        ix_ossl_message_log("Created a new Nbr Cache entry and set the state to "
                            "INCOMPLETE\n");
#endif /* IPV6_DEBUG */
    }                     
    
    pNbrEntry->l2Id = IX_HTON16(arg_L2Index);

    /* Add the calling function's context information into the nbrEntry. This
     * will be used while sending a message back to the caller*/
    pktQueue = (ix_ipv6_nd_pkt_queue_struct *)
                    ix_ossl_malloc(sizeof(ix_ipv6_nd_pkt_queue_struct));
    if(!pktQueue)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPV6ND: Out of memory: pktQueue\n");
#endif /* IPV6_DEBUG */
        errVal = IX_CC_ERROR_OOM;
        goto LBL_MSG;
    }
        
    pktQueue->hBuffer = IX_NULL_BUFFER_HANDLE;
    pktQueue->pContext = arg_pUserContext;

    ListAddItem(pNbrEntry->packetQueue, pktQueue);

    if(arg_L2Addr)
    {
        /* Now check if the specified L2 address matches the one stored in the
         * nbr entry, if it doesn't we need to start address resolution*/
        if(pNbrEntry->isValidL2Addr &&
           (!memcmp(arg_L2Addr, pNbrEntry->l2Addr, IX_CC_IPV6_MAC_ADDR_LEN)))
        {
            errVal = IX_SUCCESS;
            resolveAddress = FALSE;
        }
        else
        {
            /*Doesn't match. Copy the current address*/
            ix_ossl_memcpy(pNbrEntry->l2Addr, arg_L2Addr, 
                           IX_CC_IPV6_MAC_ADDR_LEN);
            pNbrEntry->isValidL2Addr = TRUE;
        }
        
        IX_IPV6_ERROR_CGT(ix_cc_nd_update_l2_table(pNbrEntry, pPort),
                          errVal, LBL_MSG);
           
    }/*if(arg_L2Addr)*/
        
    if(resolveAddress == TRUE)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("Setting State to INCOMPLTETE & starting addr resolution\n");
#endif /* IPV6_DEBUG */

        /*Start address resolution*/
        pNbrEntry->state = IX_REACH_TYPE_INCOMPLETE;
        errVal = ix_cc_nd_start_addr_resolution(pNbrEntry, pPort);
    }
    if(errVal == IX_SUCCESS)
        goto LBL_EXIT;

LBL_MSG:
#if defined(IPV6_DEBUG)
   ix_ossl_message_log("IPv6ND: Add Nbr failed. Sending message to client.\n");
#endif /* IPV6_DEBUG */

#ifndef MEAT_TEST
    ix_cc_msup_send_reply_msg(arg_pUserContext, NULL, 0, errVal);
#endif
LBL_EXIT:
    ND_UNLOCK();
    return errVal;
}/*ix_cc_nd_add_neighbor*/


/**
 * TYPENAME: ix_cc_nd_del_neighbor
 * 
 * DESCRIPTION:
 *          This is the internal function for handling delete neighbor
 *          calls. It makes a call to the egress FP Module.
 *
 * @Param:  - IN arg_L2Index: The index into the L2 table at which the entry 
 *                            is to be deleted.
 * @Param:  - IN arg_pUserContext: The user defined context.
 * @Param:  - OUT None.
 * 
 * @Result: IX_SUCCESS on success, relevant error code otherwise.
 */                            
ix_error ix_cc_nd_del_neighbor(
                ix_uint16 arg_L2Index,
                void* arg_pUserContext)
{
    ix_error            errVal;
    ix_cc_ipv6_general_callback_data *pL2Context;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */

     pL2Context = (ix_cc_ipv6_general_callback_data *)
                    ix_ossl_malloc(sizeof(ix_cc_ipv6_general_callback_data));
    if(!pL2Context)
    {
        errVal = IX_CC_ERROR_OOM;
        goto LBL_ERR;
    }
   
    /*call the FPM to add the neighbor*/
    /*pL2Context->pUserCallback = ix_cc_nd_cb_del_nbr;
    pL2Context->pUserContext = arg_pUserContext;*/
#ifdef USE_FPM_CC
    errVal = ix_cc_fpm_async_del_l2_entry(arg_L2Index,
                                          ix_cc_nd_cb_del_nbr,
                                          arg_pUserContext);
#else
    errVal = ix_cc_nd_cb_del_nbr(IX_SUCCESS, arg_pUserContext);
#endif
    
    if(errVal != IX_SUCCESS)
        goto LBL_ERR;

    return IX_SUCCESS;

LBL_ERR:
    if(pL2Context)
        ix_ossl_free(pL2Context);

#if defined(IPV6_DEBUG)
   ix_ossl_message_log("IPV6ND: Del Nbr failed. Sending message to client.\n");
#endif /* IPV6_DEBUG */

#ifndef MEAT_TEST
    ix_cc_msup_send_reply_msg(arg_pUserContext, NULL, 0, errVal);
#endif
    return errVal;
}/*ix_cc_nd_del_neighbor*/

/**
 * TYPENAME: ix_cc_nd_cb_del_nbr
 *
 * DESCRIPTION:
 *          Handles L2 callbacks.
 *
 * @Param:  - IN arg_Result: The result of the particular call.
 * @Param:  - IN arg_pContext: The user defined context which is passed back
 *                             to the user.
 * @Param:  - OUT None.
 *
 * @Result: IX_CC_ERROR_NULL if arg_pContext is NULL. IX_SUCCESS otherwise.
 */
ix_error ix_cc_nd_cb_del_nbr(ix_error arg_Result,
                             void *arg_pContext)
{
#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */

    if(!arg_pContext)
        return IX_CC_ERROR_NULL;

#ifndef MEAT_TEST
    ix_cc_msup_send_reply_msg(arg_pContext, NULL, 0, arg_Result);
#endif
    return IX_SUCCESS;
}/*ix_cc_nd_cb_del_nbr*/

/**
 * TYPENAME: ix_cc_nd_cb_l2_update
 *
 * Description: Callback handler for the L2 update function.
 *
 * @Param:  - IN arg_Result: Result of the corresponding call.
 * @Param:  - IN arg_pContext: The context identifying the call.
 * @Param:  - OUT None.
 *
 * @Result: IX_SUCCESS on success, appropriate error code otherwise.
 */
ix_error ix_cc_nd_cb_l2_update(ix_error arg_Result,
                               void *arg_pContext)
{
    ix_error errVal;
    ix_ipv6_nd_nbr_info *pNbrEntry;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */

    if(!arg_pContext)
    {
        errVal = IX_CC_ERROR_OOM;
        goto LBL_EXIT;
    }
        
    IX_IPV6_ERROR_CGT(ix_cc_nd_find_nbrCache_entry(
                                            *(ix_uint128 *)arg_pContext, 
                                            &pNbrEntry),
                      errVal, LBL_EXIT);

    if(!pNbrEntry)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Invalid context passed to L2 Update callback. Exiting...\n");
#endif /* IPV6_DEBUG */
        errVal = IX_CC_IPV6_ERROR_INVALID_CONTEXT;
        goto LBL_EXIT;
    }
    
    if(arg_Result == IX_SUCCESS)
    {
       errVal = ix_cc_nd_send_queued_pkts(pNbrEntry, arg_Result);
    }
    else
    {
        ix_cc_nd_send_icmp_unreachable(pNbrEntry->packetQueue);
        /*Remove buffers from the queue, but don't free them*/
        errVal = ix_cc_nd_free_buffer_list(pNbrEntry->packetQueue, 
                                           arg_Result, 0);
    }

LBL_EXIT:
    return errVal;
}/*ix_cc_nd_cb_l2_update*/

/*************************************************************************
 *
 *                      INTERNAL FUNCTIONS
 *
 ************************************************************************/

/************************************************************************
 *
 *                  NEIGHBOR DISCOVERY FUNCTIONS
 *                      
 ***********************************************************************/

/**
 * TYPENAME: ix_cc_nd_update_properties
 *
 * DESCRIPTION: Reads the supplied properties and generates internal data 
 *              structures.
 * 
 * @Param:  - IN pProperties - Pointer to the property structure obtained from
 *                             IPv6.
 * @Param:  - OUT - None.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_update_properties(ix_cc_properties_node *pProperties)
{
    ix_error    errVal;
    ix_cc_properties_node *pCurPropNode;
    ix_ipv6_nd_port *pCurPort;
#if defined(XML_SYSAPP_SUPPORT)
    ix_ipv6_addr_conf_ip_struct *pIpStruct;
    ix_uint128      tempIp;
#endif
    ix_uint32       count;
    
#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */

    if(!pProperties)
    {
        errVal = IX_CC_ERROR_NULL;
        goto LBL_EXIT;
    }

    if((g_portList = ListCreate()) == NULL)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Update Properties: List creation failed\n");
#endif /* IPV6_DEBUG */

        errVal = IX_CC_ERROR_OOM;
        goto LBL_EXIT;
    }

    pCurPropNode = pProperties;
    errVal = IX_CC_ERROR_OOM;
    while(pCurPropNode)
    {
#if defined(XML_SYSAPP_SUPPORT)
        if(pCurPropNode->pPropertyInfo->ip_version == IX_CC_IP_VERSION_IPV4)
        {
            pCurPropNode = pCurPropNode->pNextPropNode;
            continue;
        }
#endif /* XML_SYSAPP_SUPPORT */
        
        pCurPort = (ix_ipv6_nd_port *) ix_ossl_malloc(sizeof(ix_ipv6_nd_port));
        if(!pCurPort)
        {
#if defined(IPV6_DEBUG)
            ix_ossl_message_log("IPv6ND: Out of Memory: pCurPort\n");
#endif /* IPV6_DEBUG */
            goto LBL_ERR;
        }
        
        pCurPort->portId = IX_HTON16(pCurPropNode->pPropertyInfo->port_id);
        pCurPort->pPortProperties = pCurPropNode->pPropertyInfo;

#ifdef SEND_RTR_ADVS
        /*In the future, we may choose to get actual parameters from propnode*/
        pCurPort->sendRtrAdv = IX_CC_IPV6_ND_SEND_ADVS;
        pCurPort->rtrAdv.maxRtrAdvInterval = IX_CC_IPV6_ND_MAX_RTR_INTERVAL;
        pCurPort->rtrAdv.minRtrAdvInterval = IX_CC_IPV6_ND_MIN_RTR_INTERVAL;
        pCurPort->rtrAdv.nextHopLmit = IX_CC_IPV6_ND_HOP_LIMIT;
        pCurPort->rtrAdv.managed = IX_CC_IPV6_ND_MANAGED;
        pCurPort->rtrAdv.otherConfig = IX_CC_IPV6_ND_OTHER_CONFIG;
        pCurPort->rtrAdv.rtrLifetime = IX_CC_IPV6_ND_DEFAULT_LIFETIME;
        pCurPort->rtrAdv.reachableTime = IX_CC_IPV6_ND_REACHABLE_TIME;
        pCurPort->rtrAdv.retransmitTimer = IX_CC_IPV6_ND_RETRANS_TIMER;
        ix_ossl_memcpy(pCurPort.rtrAdv.l2Addr, 
                       pCurPropNode->pPropertyInfo->mac_addr, 
                       IX_CC_IPV6_MAC_ADDR_LEN);
        pCurPort.rtrAdv.l2AddrLength = IX_CC_IPV6_MAC_ADDR_LEN;
        pCurPort->rtrAdv.mtu = pCurPropNode->pPropertyInfo->mtu;
#endif

        
        if((pCurPort->ipAddrList = ListCreate()) == NULL)
        {
            errVal = IX_CC_ERROR_OOM;
            goto LBL_PORT_ERR;
        }

        errVal = IX_CC_ERROR_OOM;
        /* 
         * Iterate through the list of IP addresses and add each to the
         * port's IP address list
         */
        count = 0;
#if defined(XML_SYSAPP_SUPPORT)
        while(count < 
              pCurPropNode->pPropertyInfo->no_virtual_if)
        {
            pIpStruct = (ix_ipv6_addr_conf_ip_struct *)ix_ossl_malloc(
                                        sizeof(ix_ipv6_addr_conf_ip_struct));
            if(!pIpStruct)
                goto LBL_PORT_ERR;

            IX_HTON128(
            &(pCurPort->pPortProperties->ip_prop[count].protocol.ipv6_prop.ipv6_address.addr), &tempIp);

            ix_ossl_memcpy(&pIpStruct->ipAddr, &tempIp, sizeof(ix_uint128));
            pIpStruct->linkLayerAddr = 
                            ix_cc_nd_is_ll_ip_addr(pIpStruct->ipAddr);
            pIpStruct->state = IX_ADDR_RES_STATE_TENTATIVE;
            pIpStruct->numDadTransmits = g_DadTransmits;
            if(!ListAddItem(pCurPort->ipAddrList, (void *)pIpStruct))
                goto LBL_ADDR_ERR;

            count++;
        }
#endif /* XML_SYSAPP_SUPPORT */

        if(!ListAddItem(g_portList, (void *)pCurPort))
        {
            errVal = IX_CC_ERROR_OOM;
            goto LBL_PORT_ERR;
        }

        pCurPropNode = pCurPropNode->pNextPropNode;
    }/*while*/

    errVal = IX_SUCCESS;
    goto LBL_EXIT;

#if defined(XML_SYSAPP_SUPPORT)
LBL_ADDR_ERR:
    if(pIpStruct)
        ix_ossl_free(pIpStruct);
#endif /* XML_SYSAPP_SUPPORT */
LBL_PORT_ERR:
    if(pCurPort)
    {
        if(pCurPort->ipAddrList)
            ListFreeDelete(pCurPort->ipAddrList);

        ix_ossl_free(pCurPort);
    }
LBL_ERR:
    ix_cc_nd_cleanup_port_list();
#if defined(IPV6_DEBUG)
    ix_ossl_message_log("IPv6ND: Update Properties failed. Error: %ld\n", errVal);
#endif /* IPV6_DEBUG */

LBL_EXIT:
    return errVal;    
}/*ix_cc_nd_update_properties*/

#ifdef SEND_RTR_ADVS
/**
 * TYPENAME: ix_cc_nd_start_unsol_rtr_advs
 *
 * DESCRIPTION:
 *          Sets timers for all ports to send router advertisements.
 * 
 * @Param:  - None.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_start_unsol_rtr_advs(void)
{
    LPLISTITR   portItr;
    ix_ipv6_nd_port *pCurPort;
    ix_uint16   diff, interval;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */

    ND_LOCK();
    portItr = ListItrCreate(g_portList);
    if(!portItr)
    {
        ND_UNLOCK();
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,
			        ("List creation failed \n"));
    }

    while((pCurPort = (ix_ipv6_nd_port *)ListItrGetNext(portItr)))
    {
        if(!pCurPort->sendRtrAdv)
            continue;
        diff = pCurPort->rtrAdv.maxRtrAdvInterval - 
               pCurPort->rtrAdv.minRtrAdvInterval;
        /*Calculate a random interval between the min and max RtrAdvIntervals*/
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)  
        interval = pCurPort->rtrAdv.minRtrAdvInterval + (ix_cc_rand()%diff);
#else
        interval = pCurPort->rtrAdv.minRtrAdvInterval + (rand()%diff);
#endif

#if !defined(IX_EXCLUDE_CCI)
        if(IX_SUCCESS != ix_cci_cc_add_event_handler_ex(
                                            g_pIpv6Context->hIpv6Handle,
                                            interval*1000,
                                            ix_cc_nd_send_unsol_rtr_adv,
                                            IX_EVENT_TYPE_ONESHOT,
                                            IX_CC_IPV6_ND_EVENT_PRIORITY,
                                            (void *) &(pCurPort->portId),
                                            &(pCurPort->hRtrAdvHandle)))
        {
#if defined(IPV6_DEBUG)
            ix_ossl_message_log("IPv6ND: Failed to add event handler\n");
#endif /* IPV6_DEBUG */
            ND_UNLOCK();
            return IX_CC_IPV6_ERROR_CCI;
        }
#elif MEAT_TEST
        ListAddItem(g_Context_Nd, (void *) &(pCurPort->portId));
#endif /* IX_EXCLUDE_CCI */
 
        pCurPort->lastAdvTime.sec = pCurPort->lastAdvTime.nsec = 0;
        ix_ossl_time_get(&(pCurPort->nextAdvTime));
        pCurPort->nextAdvTime.sec+=interval; /*add interval to current time*/
    }

    ListItrDelete(portItr);
    ND_UNLOCK();

    return IX_SUCCESS;
}/*ix_cc_nd_start_unsol_rtr_advs*/
#endif

/**
 * TYPENAME: ix_cc_nd_process_rtr_sol
 *
 * DESCRIPTION: Processes a router solicitation request per RFC2461, section
 *              6.2.6.
 * 
 * @Param:  - IN arg_hBuffer - Handle to the buffer containing the packet.
 * @Param:  - IN arg_pIpv6Pkt - Pointer to the IPv6 packet extracted from the
 *                              buffer.
 * @Param:  - IN arg_pIcmpPkt - Pointer to the start of the ICMP packet 
 *                              within the IPv6 packet.
 * @Param:  - OUT - None.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_process_rtr_sol( ix_buffer_handle arg_hBuffer,
                                     ix_uint8   *arg_pIpv6Pkt,
                                     ix_uint8   *arg_pIcmpPkt)
{
    ix_error            errVal = IX_SUCCESS;
    ix_hw_buffer_meta   *pHwMeta;
    ix_cc_ipv6_header   *pIpv6Hdr;
    ix_ipv6_nd_port     *pPort;
    ix_uint32           randDelay;
    ix_ossl_time_t      sysTime = {0,0};
    ix_ossl_time_t      tempTime = {0,0};
    ix_uint8                srcIpAddrSpecified;
    ix_ipv6_nd_options_l2_addr *pL2OptHdr = NULL;
    ix_ipv6_nd_nbr_info *pNbrEntry = NULL;
    ix_uint32  icmpOffset;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */
    
    IX_ERROR_CRT(ix_rm_buffer_get_meta( arg_hBuffer, (void **)&pHwMeta),
                                 IX_CC_IPV6_ERROR_CCI, 0);
    
    icmpOffset = arg_pIcmpPkt - arg_pIpv6Pkt;
    errVal = ix_cc_nd_validate_rtr_sol(arg_pIpv6Pkt, arg_pIcmpPkt, 
                        icmpOffset, &srcIpAddrSpecified, &pL2OptHdr);
    if(errVal != IX_SUCCESS)
        goto LBL_ERR; 

    ND_LOCK();
    errVal = ix_cc_nd_find_port(pHwMeta->m_InputPort, &pPort);
    if( (errVal != IX_SUCCESS) || !pPort)
        goto LBL_ERR;

    /*now begin the nbr advertisement phase*/
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_) 
    randDelay = ix_cc_rand()%IX_CC_ND_MAX_RA_DELAY_TIME;
#else
    randDelay = rand()%IX_CC_ND_MAX_RA_DELAY_TIME;
#endif
    ix_ossl_time_get(&sysTime);
    
    tempTime.sec = sysTime.sec+randDelay;
    tempTime.nsec = sysTime.nsec;
    if(IX_OSSL_TIME_LT(tempTime, pPort->nextAdvTime))
    {
#if !defined(IX_EXCLUDE_CCI)
        IX_ERROR_CGT(ix_cci_cc_remove_event_handler(g_pIpv6Context->hIpv6Handle,
                                                    pPort->hRtrAdvHandle),
                     errVal, IX_CC_IPV6_ERROR_CCI, 0, LBL_ERR);
#endif /* IX_EXCLUDE_CCI */

        /*Was the previous adv sent within the last MIN_DELAY_BETN_RAS secs?*/
        tempTime.sec = sysTime.sec;
        IX_OSSL_TIME_SUB(tempTime, pPort->lastAdvTime);
        if(tempTime.sec < IX_CC_ND_MIN_DELAY_BETWEEN_RAS)
        {
            /*schedule the next adv after MIN_DELAY_BETN_RAS + random delay*/
            randDelay += (pPort->lastAdvTime.sec + 
                          IX_CC_ND_MIN_DELAY_BETWEEN_RAS - sysTime.sec);
        }
    
#if !defined(IX_EXCLUDE_CCI)
        if(IX_SUCCESS != ix_cci_cc_add_event_handler_ex(
                                            g_pIpv6Context->hIpv6Handle,
                                            randDelay*1000,
                                            ix_cc_nd_send_unsol_rtr_adv,
                                            IX_EVENT_TYPE_ONESHOT,
                                            IX_CC_IPV6_ND_EVENT_PRIORITY,
                                            (void *) &(pPort->portId),
                                            &(pPort->hRtrAdvHandle)))
        {
#if defined(IPV6_DEBUG)
            ix_ossl_message_log("IPv6ND: Failed to add event handler\n");
#endif /* IPV6_DEBUG */
            errVal = IX_CC_IPV6_ERROR_CCI;
            goto LBL_ERR;
        }
#elif MEAT_TEST
        ListAddItem(g_Context_Nd, (void *) &(pPort->portId));
#endif /* IX_EXCLUDE_CCI */
        pPort->nextAdvTime.sec = sysTime.sec + randDelay;
        pPort->nextAdvTime.nsec = 0;
    }/*if*/

    /*now update the nbr cache*/
    if(!srcIpAddrSpecified)
        goto LBL_ERR; /*Not an error. No need to touch nbr cache*/
        
    pIpv6Hdr = (ix_cc_ipv6_header *) arg_pIpv6Pkt;
    if(IX_SUCCESS != ix_cc_nd_find_nbrCache_entry(pIpv6Hdr->srcAddress, 
                                                    &pNbrEntry))
    {
        errVal = IX_CC_IPV6_ERROR_NBR_DISC;
        goto LBL_ERR;
    }

    errVal = IX_CC_ERROR_OOM;
    if(!pNbrEntry)
    {
        /*create a new nbr entry and add it to the cache*/
        if(IX_SUCCESS != ix_cc_nd_create_nbr_entry(&pNbrEntry))
        {
            errVal = IX_CC_IPV6_ERROR_NBR_DISC;
            goto LBL_ERR;
        }

        ix_ossl_memcpy(&pNbrEntry->ipAddr, &pIpv6Hdr->srcAddress, 
                       IX_CC_IPV6_ADDRESS_LENGTH_BYTES);
    }/*if*/

    if(pL2OptHdr && strncmp(pL2OptHdr->linkLayerAddr, pNbrEntry->l2Addr, 
                                                    IX_CC_IPV6_MAC_ADDR_LEN))
    {
        /*The src link layer option is specified, and the addresses don't
         * match*/
        ix_ossl_memcpy(pNbrEntry->l2Addr, pL2OptHdr->linkLayerAddr,
                        IX_CC_IPV6_MAC_ADDR_LEN);
        pNbrEntry->state = IX_REACH_TYPE_STALE;

#if defined(IPV6_DEBUG)
            ix_ossl_message_log("Setting State to STALE\n");
#endif /* IPV6_DEBUG */
    }

    pNbrEntry->isRouter = FALSE;

LBL_ERR:
    if(errVal != IX_SUCCESS)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Process RTR SOL failed. Error: %ld\n", errVal);
#endif /* IPV6_DEBUG */
   }

    ND_UNLOCK();
    return errVal;
}/*ix_cc_nd_process_rtr_sol*/

/**
 * TYPENAME: ix_cc_nd_process_nbr_sol
 *
 * DESCRIPTION:
 *          This function processes incoming nbr solicitation messages
 *          per RFC2461, section 7.2.3
 *
 * @Param:  - IN arg_hBuffer - Handle to the buffer containing the packet.
 * @Param:  - IN arg_pIpv6Pkt - Pointer to the IPv6 packet extracted from the
 *                              buffer.
 * @Param:  - IN arg_srcIpAddrSpecified - A flag indicating whether the source
 *                              IP address is the Unspecified Address. A value
 *                              of 1 indicates that the address is specified
 *                              and a value of 0 indicates that it is the 
 *                              Unspecified address.
 * @Param:  - IN arg_pL2Opt - Pointer to the L2 options field in the ICMP 
 *                              message. It may be NULL if no L2 options are
 *                              specified.
 * @Param:  - OUT - None.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_process_nbr_sol(ix_buffer_handle arg_hBuffer,
                                  ix_uint8  *arg_pIpv6Pkt, 
                                  ix_uint8 arg_srcIpAddrSpecified,
                                  ix_ipv6_nd_options_l2_addr *arg_pL2Opt)
{
    ix_error            errVal;
    ix_cc_ipv6_header   *pIpv6Hdr;
    ix_ipv6_nd_nbr_info *pNbrEntry;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */

    if(!arg_srcIpAddrSpecified || !arg_pL2Opt)
    {
        /* we don't have all the information needed to update the nbr cache. 
         * so just send the nbr adv without updating the cache*/
        errVal = ix_cc_nd_send_sol_nbr_adv(arg_hBuffer, arg_pIpv6Pkt, 
                                       arg_srcIpAddrSpecified, arg_pL2Opt);
        goto LBL_EXIT;
    }

    /*now find the Nbr Entry*/
    pIpv6Hdr = (ix_cc_ipv6_header *) arg_pIpv6Pkt;

    ND_LOCK();
    errVal = ix_cc_nd_find_nbrCache_entry(pIpv6Hdr->srcAddress, &pNbrEntry);
    if(errVal != IX_SUCCESS)
        goto LBL_ERR;

    if(!pNbrEntry)
    {
        /*create a new nbr entry and add it to the cache*/
        errVal = ix_cc_nd_create_nbr_entry(&pNbrEntry); 
        if(errVal != IX_SUCCESS)
            goto LBL_ERR;

        ix_ossl_memcpy(&pNbrEntry->ipAddr, &pIpv6Hdr->srcAddress, 
                       IX_CC_IPV6_ADDRESS_LENGTH_BYTES);

        pNbrEntry->isRouter = FALSE;
    }/*if*/

    if(strncmp(arg_pL2Opt->linkLayerAddr, pNbrEntry->l2Addr, 
                                                    IX_CC_IPV6_MAC_ADDR_LEN))
    {
        /*The src link layer option is specified, and the addresses don't
         * match*/
        ix_ossl_memcpy(pNbrEntry->l2Addr, arg_pL2Opt->linkLayerAddr,
                        IX_CC_IPV6_MAC_ADDR_LEN);
        pNbrEntry->isValidL2Addr = TRUE; 
        pNbrEntry->state = IX_REACH_TYPE_STALE;

#if defined(IPV6_DEBUG)
        ix_ossl_message_log("Setting State to STALE\n");
#endif /* IPV6_DEBUG */
    }

    errVal = ix_cc_nd_send_sol_nbr_adv(arg_hBuffer, arg_pIpv6Pkt, 
                                       arg_srcIpAddrSpecified, arg_pL2Opt);

LBL_ERR:
    ND_UNLOCK();
LBL_EXIT:
    if(errVal != IX_SUCCESS)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Process NBR SOL failed. Error: %ld\n", errVal);
#endif /* IPV6_DEBUG */
    }

    return errVal;
}/*ix_cc_nd_process_nbr_sol*/

/**
 * TYPENAME: ix_cc_nd_process_nbr_adv
 *
 * DESCRIPTION:
 *          This function processes incoming nbr adv messages as defined
 *          in RFC2461, section 7.2.5.
 *
 * @Param:  - IN arg_pIcmpPkt - Pointer to the ICMP packet extracted from the
 *                              buffer.
 * @Param:  - IN arg_pL2Opt - Pointer to the L2 options field in the ICMP 
 *                              message. It may be NULL if no L2 options are
 *                              specified.
 * @Param:  - OUT - None.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_process_nbr_adv(ix_uint8 *arg_pIcmpPkt,
                                  ix_ipv6_nd_options_l2_addr *arg_pL2Opt,
                                  ix_ipv6_nd_port *pPort)
{
    ix_error                    errVal;
    ix_uint8                    l2AddrsDiffer = TRUE;
    ix_ipv6_nd_nbr_info         *pNbrEntry;
    ix_ipv6_icmp_nbr_adv_msg    *pNbrAdv;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */
 
    pNbrAdv = (ix_ipv6_icmp_nbr_adv_msg *)(arg_pIcmpPkt+IX_CC_ICMPV6_HDR_LEN);
   
    ND_LOCK();
    errVal = ix_cc_nd_find_nbrCache_entry(pNbrAdv->targetAddr, &pNbrEntry);
    if(errVal != IX_SUCCESS)
    {
        /*discard the packet*/
        goto LBL_FREE_BUF;
    }
    else if(!pNbrEntry)
    {
        errVal = IX_CC_IPV6_ERROR_IGNORE_PACKET;
        goto LBL_FREE_BUF;
    }

    /*do some pre-processing. This will make life simpler later!*/
    if(arg_pL2Opt)
    {
        if(!memcmp(arg_pL2Opt->linkLayerAddr, pNbrEntry->l2Addr,
                   IX_CC_IPV6_MAC_ADDR_LEN))
        {
            l2AddrsDiffer = FALSE;
        }
    }

    /******Start processing the nbr adv according to the RFC****/
    errVal = IX_SUCCESS;
    if(pNbrEntry->state == IX_REACH_TYPE_INCOMPLETE)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("Process Nbr Adv: IX_REACH_TYPE_INCOMPLETE\n");
#endif /* IPV6_DEBUG */

        if(!arg_pL2Opt)
            goto LBL_FREE_BUF;
        
        ix_ossl_memcpy(pNbrEntry->l2Addr, arg_pL2Opt->linkLayerAddr, 
                       IX_CC_IPV6_MAC_ADDR_LEN);
        pNbrEntry->isValidL2Addr = TRUE;
        
        if(pNbrAdv->solicited)
        {
            pNbrEntry->state = IX_REACH_TYPE_REACHABLE;
#if defined(IPV6_DEBUG)
            ix_ossl_message_log("Setting State to REACHABLE\n");
            ix_ossl_message_log("#### Address Resolution Succeeded ###\n");
            _ix_cc_print_ipv6_address(&pNbrEntry->ipAddr);
            ix_ossl_message_log("The MAC address resolved: %x:%x:%x:%x:%x:%x\n",
                                 pNbrEntry->l2Addr[0], pNbrEntry->l2Addr[1],
                                 pNbrEntry->l2Addr[2], pNbrEntry->l2Addr[3],
                                 pNbrEntry->l2Addr[4], pNbrEntry->l2Addr[5]);
#endif /* IPV6_DEBUG */
        }
        else
        {
            pNbrEntry->state = IX_REACH_TYPE_STALE;
#if defined(IPV6_DEBUG)
            ix_ossl_message_log("Setting State to STALE\n");
#endif /* IPV6_DEBUG */
        }

        if(pNbrAdv->router)
            pNbrEntry->isRouter = TRUE;
        else
            pNbrEntry->isRouter = FALSE;

        IX_IPV6_ERROR_CGT(ix_cc_nd_update_l2_table(pNbrEntry, pPort),
                          errVal, LBL_FREE_BUF);
        /*IX_IPV6_ERROR_CGT(ix_cc_nd_send_queued_pkts(pNbrEntry, errVal),
                          errVal, LBL_FREE_BUF);*/
    }
    else
    { 
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("Process Nbr Adv: State is not IX_REACH_TYPE_INCOMPLETE\n");
#endif /* IPV6_DEBUG */
        if(!pNbrAdv->override)
        {
            if(arg_pL2Opt && l2AddrsDiffer)
            {
                if(pNbrEntry->state == IX_REACH_TYPE_REACHABLE)
                {
                    pNbrEntry->state = IX_REACH_TYPE_STALE;
#if defined(IPV6_DEBUG)
                    ix_ossl_message_log("Setting state to STALE\n");
#endif /* IPV6_DEBUG */
                }

                /*exit*/
                goto LBL_FREE_BUF;
            }/*if(2OptSpecified...*/
            /*else case: fall thru...*/
            
        }/*if(!pNbrAdv->...*/

        if(arg_pL2Opt && l2AddrsDiffer)
        {
            ix_ossl_memcpy(pNbrEntry->l2Addr, arg_pL2Opt->linkLayerAddr,
                                           IX_CC_IPV6_MAC_ADDR_LEN);
            pNbrEntry->isValidL2Addr = TRUE;
        }
        
        if(pNbrAdv->solicited)
        {
            pNbrEntry->state = IX_REACH_TYPE_REACHABLE;
#if defined(IPV6_DEBUG)
            ix_ossl_message_log("Setting State to REACHABLE\n");
#endif /* IPV6_DEBUG */
        }
        else
        {
            if(l2AddrsDiffer)
            {
                pNbrEntry->state = IX_REACH_TYPE_STALE;
#if defined(IPV6_DEBUG)
                ix_ossl_message_log("Setting State to STALE\n");
#endif /* IPV6_DEBUG */
            }
        }

        if(pNbrAdv->router)
            pNbrEntry->isRouter = TRUE;
        else
            pNbrEntry->isRouter = FALSE;
    }

LBL_FREE_BUF:
    ND_UNLOCK();
    if(errVal == IX_CC_IPV6_ERROR_IGNORE_PACKET)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Ignoring NBR ADV pkt. No Nbr Cache entry found\n");
#endif /* IPV6_DEBUG */
    }
    else if(errVal != IX_SUCCESS)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Process NBR ADV failed. Error: %ld\n", errVal);
#endif /* IPV6_DEBUG */
    }
    return errVal;
}/*ix_cc_nd_process_nbr_adv*/

/**
 * TYPENAME: ix_cc_nd_validate_rtr_sol
 *
 * DESCRIPTION: Validates router solicitation messages per RFC2461, 
 *              section 6.1.1
 * 
 * @Param:  - IN pPkt - Pointer to the IPv6 packet.
 * @Param:  - IN pIcmpPkt - Pointer to the ICMP packet within the IPv6 Packet.
 * @Param:  - IN icmpOffset - Offset from the start of the IPv6 pkt at which
 *                          the ICMP packet is located.
 * @Param:  - OUT arg_IpAddrIsSpecified - Flag indicating the type of source
 *                          IP address. 0 indicates the Unspecified address.
 * @Param:  - OUT arg_pL2Options: Pointer to the L2 options within the ICMP
 *                          packet. May be NULL if no options are specified.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_validate_rtr_sol(ix_uint8 *pPkt, 
                                   ix_uint8 *pIcmpPkt,
                                   ix_uint32 icmpOffset,
                                   ix_uint8 *arg_IpAddrIsSpecified,
                                   ix_ipv6_nd_options_l2_addr **arg_pL2Options)
{
    ix_cc_ipv6_header *pIpv6Header;
    ix_cc_ipv6_icmp_header *pIcmpHeader;
    ix_uint8    *pCurPosn;
    ix_uint32   pktLen;
    ix_uint128  tempAddr;
    ix_ipv6_nd_options_l2_addr *pL2OptionsHdr;
    
#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */

    pCurPosn = pPkt;
    pIpv6Header = (ix_cc_ipv6_header *) pCurPosn;
    pIcmpHeader = (ix_cc_ipv6_icmp_header *) pIcmpPkt;
    
    if(pIpv6Header->hopLimit != IX_CC_IPV6_HOP_LIMIT_MAX)
        return IX_CC_ERROR_RANGE;

    /* Now calculate the ICMP packet length. Start with the total IPv6
     * pkt length, and subtract the offset at which the ICMP pkt occurs*/
    pktLen = IX_NTOH16(pIpv6Header->payloadLength)+IX_CC_IPV6_HDR_LEN;
    pktLen -= icmpOffset;

    if(pIcmpHeader->type != IX_CC_IPV6_ICMP_TYPE_RTR_SOL)
    {
#if defined(IPV6_DEBUG)
       ix_ossl_message_log("IPv6ND: RTR SOL failed: Hdr type doesn't match\n");
#endif /* IPV6_DEBUG */
        return IX_CC_IPV6_ERROR_INVALID_PACKET; 
    }

    if(pIcmpHeader->code != 0)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: RTR SOL failed: Hdr code not 0\n");        
#endif /* IPV6_DEBUG */
        return IX_CC_IPV6_ERROR_INVALID_PACKET; 
    }
        
    /*check if there is a src link layer option. Subtract hdr len and 
     *mandatory field length from pktLen.*/
    pktLen-=(IX_CC_ICMPV6_HDR_LEN+4);
    if(pktLen == 0)
    {
        /*no options present*/
        return IX_SUCCESS;
    }
    
    pCurPosn+=IX_CC_ICMPV6_HDR_LEN+4; /*move beyond the reserved field*/
    pL2OptionsHdr = (ix_ipv6_nd_options_l2_addr *)pCurPosn;
    if(pL2OptionsHdr->type != IX_CC_IPV6_ND_OPT_SRC_L2_ADDR)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: RTR SOL failed: Invalid options hdr type\n");
#endif /* IPV6_DEBUG */

        /*illegal type*/
        return IX_CC_IPV6_ERROR_INVALID_PACKET;
    }

    if(pL2OptionsHdr->length == 0)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: RTR SOL failed: Invalid option length\n");
#endif /* IPV6_DEBUG */
        return IX_CC_IPV6_ERROR_INVALID_PACKET;
    }
    
    *arg_pL2Options = pL2OptionsHdr;
    /*since the src link layer address is present, the src IP addr 
     * shouldn't be unspecified
     */
    MAKE_UNSPEC_ADDR(tempAddr);
    if(!memcmp(&pIpv6Header->srcAddress, &tempAddr, sizeof(ix_uint128)))
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: RTR SOL failed: Src address unspecified\n");
#endif /* IPV6_DEBUG */
        return IX_CC_IPV6_ERROR_INVALID_ADDRESS;
    }
            
    *arg_IpAddrIsSpecified = TRUE;
    return IX_SUCCESS;
}/*ix_cc_nd_validate_rtr_sol*/

/**
 * TYPENAME: ix_cc_nd_validate_nbr_sol
 *
 * DESCRIPTION: Validates neighbor solicitation messages per RFC2461, 
 *              sections 7.1.1 and 7.2.3
 *
 * @Param:  - IN arg_hBuffer - Handle to the buffer containing the packet.
 * @Param:  - IN pPkt - Pointer to the IPv6 packet extracted from the
 *                              buffer.
 * @Param:  - IN pIcmpPkt - Pointer to the start of the ICMP packet 
 *                              within the IPv6 packet.
 * @Param:  - OUT arg_IpAddrIsSpecified - A flag indicating whether the source
 *                              IP address is the Unspecified Address. A value
 *                              of 1 indicates that the address is specified
 *                              and a value of 0 indicates that it is the 
 *                              Unspecified address.
 * @Param:  - OUT arg_pL2Options - Pointer to the L2 options field in the ICMP 
 *                              message. It may be NULL if no L2 options are
 *                              specified.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_validate_nbr_sol(ix_buffer_handle arg_hBuffer,
                                   ix_uint8 *pPkt, 
                                   ix_uint8 *pIcmpPkt,
                                   ix_uint8 *arg_IpAddrIsSpecified,
                                   ix_ipv6_nd_options_l2_addr **arg_pL2Options)
{
    ix_error    errVal;
    ix_cc_ipv6_header *pIpv6Header;
    ix_cc_ipv6_icmp_header *pIcmpHeader;
    ix_uint8    *pCurPosn;
    ix_uint32   pktLen;
    ix_ipv6_icmp_nbr_sol_msg *pNbrSolMsg;
    ix_ipv6_nd_options_l2_addr *pL2OptionsHdr;
    ix_uint128  tempAddr;
    ix_uint32   icmpOffset;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */

    pCurPosn = (ix_uint8 *)pPkt;
    pIpv6Header = (ix_cc_ipv6_header *) pCurPosn;
    pIcmpHeader = (ix_cc_ipv6_icmp_header *) pIcmpPkt;
    
    if(pIpv6Header->hopLimit != IX_CC_IPV6_HOP_LIMIT_MAX)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR SOL: Hop Limit not 255\n");
#endif /* IPV6_DEBUG */
        return IX_CC_ERROR_RANGE;
    }

    icmpOffset = pIcmpPkt - pPkt; 
    /* Now calculate the ICMP packet length. Start with the total IPv6
     * pkt length, and subtract the offset at which the ICMP pkt occurs*/
    pktLen = IX_NTOH16(pIpv6Header->payloadLength)+IX_CC_IPV6_HDR_LEN;
    pktLen -= icmpOffset;
    
    /*ICMP length must be at least 24 octets*/
    if(pktLen < IX_CC_IPV6_ND_NBR_SOL_MSG_MIN_LEN)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR SOL: pkt length < 24\n");
#endif /* IPV6_DEBUG */
        return IX_CC_ERROR_RANGE;
    }

    if(pIcmpHeader->type != IX_CC_IPV6_ICMP_TYPE_NBR_SOL)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR SOL: ICMP type invalid\n");
#endif /* IPV6_DEBUG */
        return IX_CC_IPV6_ERROR_INVALID_PACKET; 
    }

    if(pIcmpHeader->code != IX_CC_ICMPV6_CODE_ZERO)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR SOL: ICMP Code invalid\n");
#endif /* IPV6_DEBUG */
        return IX_CC_IPV6_ERROR_INVALID_PACKET;
    }

    /*Check if the IP source address is specified*/
    MAKE_UNSPEC_ADDR(tempAddr);
    if(!memcmp(&pIpv6Header->srcAddress, &tempAddr, sizeof(ix_uint128)))
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("Source Address is Unspecified\n");
#endif
        *arg_IpAddrIsSpecified = FALSE;
    }
    else
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("Source Address is Specified\n");
#endif
        *arg_IpAddrIsSpecified = TRUE;
    }
            
    /* if src address is the unspecified address, the dest address should
     * be the solicited node multicast address*/
    if(!*arg_IpAddrIsSpecified)
    {
        /* construct a Solicited node multicast address with trailing bits 
         * of the destination address.*/
        MAKE_SOL_NODE_ADDR(pIpv6Header->dstAddress, &tempAddr);
        /* The destination address should be a valid solicited node addr*/
        if(memcmp(&tempAddr, &pIpv6Header->dstAddress, sizeof(ix_uint128)))
        {
#if defined(IPV6_DEBUG)
            ix_ossl_message_log("IPv6ND: Validate NBR SOL: Addr is not"
                                " solicited node addr\n");
#endif /* IPV6_DEBUG */
            return IX_CC_IPV6_ERROR_INVALID_ADDRESS;
        }
    }    

    pCurPosn = pIcmpPkt+IX_CC_ICMPV6_HDR_LEN;
    pNbrSolMsg = (ix_ipv6_icmp_nbr_sol_msg *)pCurPosn;

    /*Perform Address Autoconfiguration checks*/
    errVal = ix_cc_nd_validate_nbr_sol_target_address(arg_hBuffer, 
                                    pNbrSolMsg->targetAddr,
                                    *arg_IpAddrIsSpecified);
    if(errVal != IX_SUCCESS)
        return errVal;
    
    /*check if there is a src link layer option. Subtract hdr len and 
     *mandatory field length from pktLen.*/
    pktLen-=(IX_CC_ICMPV6_HDR_LEN+sizeof(ix_ipv6_icmp_nbr_sol_msg));
    if(!pktLen)
        return IX_SUCCESS;

    /*pktLen is not 0. i.e. L2 options are present*/
    if(!*arg_IpAddrIsSpecified)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR SOL: L2 options present"
                            " for unspecified src addr\n");
#endif /* IPV6_DEBUG */
        /*L2 options cannot be present if src addr is unspecified*/
        return IX_CC_IPV6_ERROR_INVALID_ADDRESS;
    }
    
    /*move beyond the mandatory fields*/
    pCurPosn += sizeof(ix_ipv6_icmp_nbr_sol_msg); 
    pL2OptionsHdr = (ix_ipv6_nd_options_l2_addr *)pCurPosn;
    if(pL2OptionsHdr->type != IX_CC_IPV6_ND_OPT_SRC_L2_ADDR)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR SOL: invalid option type\n");
#endif /* IPV6_DEBUG */
        /*illegal type*/
        return IX_CC_IPV6_ERROR_INVALID_PACKET;
    }

    if(pL2OptionsHdr->length == 0)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR SOL: Invalid option length\n");
#endif /* IPV6_DEBUG */
        return IX_CC_IPV6_ERROR_INVALID_PACKET;
    }
    
    *arg_pL2Options = pL2OptionsHdr;
    return IX_SUCCESS;
}/*ix_cc_nd_validate_nbr_sol*/

/**
 * TYPENAME: ix_cc_nd_validate_nbr_adv
 *
 * DESCRIPTION:
 *          This function validates nbr adv messages per RFC2461, 
 *          section 7.1.2
 *
 * DESCRIPTION:
 *
 * @Param:  - IN arg_hBuffer - Handle to the buffer containing the packet.
 * @Param:  - IN arg_pIpv6Pkt - Pointer to the IPv6 packet extracted from the
 *                              buffer.
 * @Param:  - IN arg_pIcmpPkt - Pointer to the start of the ICMP packet 
 *                              within the IPv6 packet.
 * @Param:  - OUT arg_pL2Options - Pointer to the L2 options field in the ICMP 
 *                              message. It may be NULL if no L2 options are
 *                              specified.
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_validate_nbr_adv(ix_buffer_handle arg_hBuffer,
                                   ix_uint8     *arg_pIpv6Pkt, 
                                   ix_uint8     *arg_pIcmpPkt,
                                   ix_ipv6_nd_options_l2_addr **arg_pL2Options)
{
    ix_error    errVal;
    ix_cc_ipv6_header *pIpv6Header;
    ix_cc_ipv6_icmp_header *pIcmpHeader;
    ix_uint8    *pCurPosn;
    ix_uint32   pktLen;
    ix_ipv6_nd_options_l2_addr *pL2OptionsHdr;
    ix_ipv6_icmp_nbr_adv_msg  *pNbrAdvMsg;
    ix_uint32    icmpOffset;
    
#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */

    pIpv6Header = (ix_cc_ipv6_header *) arg_pIpv6Pkt;
    pIcmpHeader = (ix_cc_ipv6_icmp_header *) arg_pIcmpPkt;
    pCurPosn = arg_pIcmpPkt;
   
    if(pIpv6Header->hopLimit != IX_CC_IPV6_HOP_LIMIT_MAX)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR ADV: hop limit not 255\n"); 
#endif /* IPV6_DEBUG */
        return IX_CC_ERROR_RANGE;
    }

    icmpOffset = arg_pIcmpPkt - arg_pIpv6Pkt;
    /* Now calculate the ICMP packet length. Start with the total IPv6
     * pkt length, and subtract the offset at which the ICMP pkt occurs*/
    pktLen = IX_NTOH16(pIpv6Header->payloadLength)+IX_CC_IPV6_HDR_LEN;
    pktLen -= icmpOffset;
    
    /*ICMP length must be at least 24 octets*/
    if(pktLen < IX_CC_IPV6_ND_NBR_ADV_MSG_MIN_LEN)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR ADV: pkt length < 24\n");
#endif /* IPV6_DEBUG */
        return IX_CC_ERROR_RANGE;
    }

    if(pIcmpHeader->type != IX_CC_IPV6_ICMP_TYPE_NBR_ADV)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR ADV: invalid header type\n");
#endif /* IPV6_DEBUG */
        return IX_CC_IPV6_ERROR_INVALID_PACKET;
    }

    if(pIcmpHeader->code != IX_CC_ICMPV6_CODE_ZERO)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR ADV: invalid header code\n");
#endif /* IPV6_DEBUG */
        return IX_CC_IPV6_ERROR_INVALID_PACKET;
    }
        
    /*check if there is a src link layer option. Subtract hdr len and 
     *mandatory field length from pktLen.*/
    pktLen-=(IX_CC_ICMPV6_HDR_LEN+sizeof(ix_ipv6_icmp_nbr_adv_msg));
    if(pktLen == 0)
    {
        /*no options present*/
        return IX_SUCCESS;
    }

    /*Check if the Target IP address is multicast*/
    pCurPosn += IX_CC_ICMPV6_HDR_LEN;
    pNbrAdvMsg = (ix_ipv6_icmp_nbr_adv_msg *) pCurPosn;
    errVal = ix_cc_nd_validate_nbr_adv_target_addr(arg_hBuffer,
                                                   pNbrAdvMsg->targetAddr);
    if(errVal != IX_SUCCESS)
        return errVal;

    /*move beyond the mandatory fields*/
    pCurPosn+=sizeof(ix_ipv6_icmp_nbr_adv_msg); 
    pL2OptionsHdr = (ix_ipv6_nd_options_l2_addr *)pCurPosn;
    if(pL2OptionsHdr->type != IX_CC_IPV6_ND_OPT_DST_L2_ADDR)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR ADV: invalid option type\n");
#endif /* IPV6_DEBUG */

        /*illegal type*/
        return IX_CC_IPV6_ERROR_INVALID_PACKET;
    }

    if(pL2OptionsHdr->length == 0)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR ADV: Invalid option length\n");
#endif /* IPV6_DEBUG */
        return IX_CC_IPV6_ERROR_INVALID_PACKET;
    }
    
    *arg_pL2Options = pL2OptionsHdr;
    return IX_SUCCESS;
}/*ix_cc_nd_validate_nbr_adv*/

/**
 * TYPENAME: ix_cc_nd_validate_nbr_sol_target_address
 * 
 * DESCRIPTION: Checks if the target address supplied in the neighbor 
 *              solicitation message is valid as per the rules in 
 *              RFC2461, section 7.2.3. In addition, it checks the
 *              Duplicate Address Detection rules as specified in
 *              RFC2462, section 5.4.3
 *
 * @Param:  - IN arg_hBuffer - Handle to the buffer containing the packet.
 * @Param:  - IN arg_IpAddr - The IPv6 Address to be validated.
 * @Param:  - IN arg_srcAddrSpecified - A flag indicating whether the source
 *                              IP address is the Unspecified Address. 
 * @Param:  - OUT - None.
 * 
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_validate_nbr_sol_target_address(ix_buffer_handle arg_hBuffer,
                                                  ix_uint128 arg_ipAddr,
                                                  ix_uint8 arg_srcAddrSpecified)
{
    ix_error            errVal;
    ix_hw_buffer_meta   *pHwMeta;
    ix_ipv6_nd_port     *pPort;
    ix_ipv6_addr_conf_ip_struct *pAddrStruct;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */
    
    if(arg_ipAddr.aUint8[0] == 0xFF)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR SOL: Target Addr is multicast\n");
#endif /* IPV6_DEBUG */
        return IX_CC_IPV6_ERROR_INVALID_ADDRESS;
    }
    
    if(IX_SUCCESS != ix_rm_buffer_get_meta( arg_hBuffer, (void **)&pHwMeta))
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR SOL: unable to get meta data\n");
#endif /* IPV6_DEBUG */
         return IX_CC_IPV6_ERROR_CCI;
    }

    errVal = ix_cc_nd_find_port(pHwMeta->m_InputPort, &pPort);
    if(errVal != IX_SUCCESS)
        return errVal;

    if(!ix_cc_addr_conf_find_ip_addr(pPort, arg_ipAddr, &pAddrStruct))
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR SOL: unknown target address\n");
#endif /* IPV6_DEBUG */
        return IX_CC_IPV6_ERROR_INVALID_INPUT_PARAM;
    }
    
    if(pAddrStruct->state == IX_ADDR_RES_STATE_INVALID)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR SOL: Address state invalid\n");
#endif /* IPV6_DEBUG */
        return IX_CC_IPV6_ERROR_INVALID_ADDRESS;
    }
    
    /*For states other than tentative, there's nothing more to do*/
    if(pAddrStruct->state != IX_ADDR_RES_STATE_TENTATIVE)
        return IX_SUCCESS;
    
    if(arg_srcAddrSpecified)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR SOL: Addr state tentative,"
                            " ignoring packet...\n");
#endif /* IPV6_DEBUG */
        /* The src address is specified, but the addr state is tentative.
         * This is a pkt for some node performing addr resolution on the
         * tentative address. IGNORE THE PACKET*/
        return IX_CC_IPV6_ERROR_IGNORE_PACKET;
    }
    
    /* At this point, we know that src addr is unspecified and target addr is
     * tentative. This means that someone else is doing DAD on this address*/
    pAddrStruct->state = IX_ADDR_RES_STATE_INVALID;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("IPv6ND: Validate NBR SOL: Duplicate Address Detected\n");
#endif /* IPV6_DEBUG */

    return IX_CC_IPV6_ERROR_DUPLICATE_ADDRESS_DETECTED;

}/*ix_cc_nd_validate_nbr_sol_target_address*/

/**
 * TYPENAME: ix_cc_nd_validate_nbr_adv_target_addr
 *
 * DESCRIPTION: Checks the target address according to the Duplicate
 *              Address Detection rules specified in RFC2462, section 5.4.4
 * 
 * @Param:  - IN arg_hBuffer - Handle to the buffer containing the packet.
 * @Param:  - IN arg_IpAddr - The IPv6 Address to be validated.
 * @Param:  - OUT - None.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_validate_nbr_adv_target_addr(ix_buffer_handle arg_hBuffer,
                                               ix_uint128   arg_ipAddr)
{
    ix_error errVal;
    ix_hw_buffer_meta   *pHwMeta;
    ix_ipv6_nd_port     *pPort;
    ix_ipv6_addr_conf_ip_struct *pAddrStruct;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */

    if(arg_ipAddr.aUint8[0] == 0xFF)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR Adv: Addr is multicast\n"); 
#endif /* IPV6_DEBUG */
        return IX_CC_IPV6_ERROR_INVALID_ADDRESS;
    }
    
    if(IX_SUCCESS != ix_rm_buffer_get_meta( arg_hBuffer, (void **)&pHwMeta))
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR Adv: Unable to get meta data\n");
#endif /* IPV6_DEBUG */
         return IX_CC_IPV6_ERROR_CCI;
    }

    errVal = ix_cc_nd_find_port(pHwMeta->m_InputPort, &pPort);
    if(errVal != IX_SUCCESS)
        return errVal;

    if(!ix_cc_addr_conf_find_ip_addr(pPort, arg_ipAddr, &pAddrStruct))
    {
        /*this message is for some address which hasn't been assigned to this
         * port. It was probably sent by a nbr as a result of some Nbr Sol.
         * Stop DAD processing*/
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR Adv: Target IP addr doesn't"
                            " belong to this port. No DAD needed. Returning SUCCESS\n");
#endif /* IPV6_DEBUG */
        return IX_SUCCESS;
    }
    
    if(pAddrStruct->state == IX_ADDR_RES_STATE_INVALID)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR Adv: Addr state invalid\n");
#endif /* IPV6_DEBUG */
        return IX_CC_IPV6_ERROR_INVALID_ADDRESS;
    }
 
    if(pAddrStruct->state == IX_ADDR_RES_STATE_TENTATIVE)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Validate NBR Adv: Duplicate Address Detected\n");
#endif /* IPV6_DEBUG */
        pAddrStruct->state = IX_ADDR_RES_STATE_INVALID;
        return IX_CC_IPV6_ERROR_DUPLICATE_ADDRESS_DETECTED;
    }

    return IX_SUCCESS;
}/*ix_cc_nd_validate_nbr_adv_target_addr*/

/**
 * TYPENAME: ix_cc_nd_send_unsol_rtr_adv
 *
 * DESCRIPTION:
 *          Event handler. Called when the timer event expires. Send out
 *          the router advertisement message
 * 
 * @Param:  - IN arg_pContext - Context identifying the event. 
 * @Param:  - OUT - None.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.

 */
ix_error ix_cc_nd_send_unsol_rtr_adv(void *arg_pContext)
{
    ix_uint16   diff;
    ix_uint16   interval;
    LPLISTITR   portItr;
    ix_ipv6_nd_port *pCurPort;
    ix_uint128    tempAddr;
    ix_error        errVal;
    ix_ipv6_addr_conf_ip_struct *addrStruct;


#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */
    
    ND_LOCK();
    portItr = ListItrCreate(g_portList);
    if(!portItr)
    {
        ND_UNLOCK();
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Send RTR ADV: OOM\n");
#endif /* IPV6_DEBUG */
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,
			        ("List creation failed \n"));
    }

    errVal = IX_SUCCESS;
    while((pCurPort = (ix_ipv6_nd_port *)ListItrGetNext(portItr)))
    {
        /*The event context we stored was the port id. Find the correct ID*/
        if(pCurPort->portId != *(ix_uint32 *)arg_pContext)
           continue;

        /*Don't send if both flags have been cleared. Send if either flag
         *is set. 
         *if prevSendAdv == TRUE and sendRtrAdv == FALSE, the router has
         *been told to stop sending rtr advertisements. However, it needs to 
         *send one last adv to inform its neighbors of this change*/
       if( (pCurPort->sendRtrAdv == FALSE) && (pCurPort->prevSendAdv == FALSE))
            goto LBL_EXIT;

        pCurPort->prevSendAdv = pCurPort->sendRtrAdv;

        /*Send the router adv message*/
        addrStruct = ListGetFirst(pCurPort->ipAddrList);
        if(!addrStruct)
        {
            errVal = IX_CC_ERROR_NULL;
            goto LBL_EXIT;
        }
        
        MAKE_ALL_RTRS_ADDR(tempAddr);
        errVal = ix_cc_ipv6_icmp_send_rtr_adv(addrStruct->ipAddr, 
                                              tempAddr,
                                              &pCurPort->rtrAdv,
                                              pCurPort->portId,
                                          pCurPort->pPortProperties->blade_id);
        if(errVal != IX_SUCCESS)
            goto LBL_EXIT;

        diff = pCurPort->rtrAdv.maxRtrAdvInterval - 
               pCurPort->rtrAdv.minRtrAdvInterval;
        /*Calculate a random interval between the min and max RtrAdvIntervals*/
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_) 
        interval = pCurPort->rtrAdv.minRtrAdvInterval + (ix_cc_rand()%diff);
#else
        interval = pCurPort->rtrAdv.minRtrAdvInterval + (rand()%diff);
#endif
        
#if !defined(IX_EXCLUDE_CCI)
        if(IX_SUCCESS != ix_cci_cc_add_event_handler_ex(
                                            g_pIpv6Context->hIpv6Handle,
                                            interval*1000,
                                            ix_cc_nd_send_unsol_rtr_adv,
                                            IX_EVENT_TYPE_ONESHOT,
                                            IX_CC_IPV6_ND_EVENT_PRIORITY,
                                            (void *) &(pCurPort->portId),
                                            &pCurPort->hRtrAdvHandle))
        {
#if defined(IPV6_DEBUG)
            ix_ossl_message_log("IPv6ND: Send RTR ADV: Failed to add event handler\n");
#endif /* IPV6_DEBUG */
            errVal = IX_CC_IPV6_ERROR_CCI;
            goto LBL_EXIT;
        }
#elif MEAT_TEST
        ListAddItem(g_Context_Nd, (void *) &(pCurPort->portId));
#endif /* IX_EXCLUDE_CCI */

        /*Update the time values*/
        pCurPort->lastAdvTime.sec = pCurPort->nextAdvTime.sec;
        pCurPort->lastAdvTime.nsec = pCurPort->nextAdvTime.sec;
        ix_ossl_time_get(&pCurPort->nextAdvTime);
        pCurPort->nextAdvTime.sec += interval;
    }

LBL_EXIT:
    ListItrDelete(portItr);
    ND_UNLOCK();
    if(errVal != IX_SUCCESS)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Send RTR ADV failed. Error: %ld\n", errVal);
#endif /* IPV6_DEBUG */
    }
    return errVal;
}/*ix_cc_nd_send_unsol_rtr_adv*/

/**
 * TYPENAME: ix_cc_nd_send_sol_nbr_adv
 *
 * DESCRIPTION: Sends a solicited neighbor advertisement message. The 
 *              supplied hardware bufer handle must point to a packet
 *              containing a neighbor solicitation message.
 * 
 * @Param:  - IN arg_hBuffer - Handle to the buffer containing the packet.
 * @Param:  - IN pIpv6Pkt - Pointer to the IPv6 packet extracted from the
                              buffer.
 * @Param:  - IN srcAddrSpecified - A flag indicating whether the source
 *                              IP address is the Unspecified Address. A value
 *                              of 1 indicates that the address is specified
 *                              and a value of 0 indicates that it is the 
 *                              Unspecified address.
 * @Param:  - OUT - None.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_send_sol_nbr_adv(ix_buffer_handle arg_hBuffer,
                                   ix_uint8 *pIpv6Pkt,
                                   ix_uint8 srcAddrSpecified,
                                   ix_ipv6_nd_options_l2_addr *arg_pL2Opt)
{
    ix_uint32       icmpOffset;
    ix_uint8        *pIcmpPkt;
    ix_ipv6_nd_port *pPort;
    ix_cc_ipv6_header           *pIpv6Hdr;
    ix_ipv6_icmp_nbr_sol_msg    *pNbrSolMsg;
    ix_ipv6_nd_nbr_adv_params   nbrAdv;
    ix_hw_buffer_meta           *pHwMeta;
    ix_uint128                  tempAddr, tempHbAddr, tempHbTargAddr;
    
#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */
    
    nbrAdv.isRouter = TRUE;
    nbrAdv.solicited = TRUE;
    nbrAdv.override = TRUE;

    /*Now to get the target address*/
    if(!ix_cc_nd_locate_icmp_pkt(pIpv6Pkt, &pIcmpPkt, &icmpOffset))
        return IX_CC_IPV6_ERROR_INVALID_PACKET;

    pNbrSolMsg = (ix_ipv6_icmp_nbr_sol_msg *) 
                                        (pIcmpPkt + IX_CC_ICMPV6_HDR_LEN);
    ix_ossl_memcpy(&nbrAdv.targetAddr, &pNbrSolMsg->targetAddr, 
                   IX_CC_IPV6_ADDRESS_LENGTH_BYTES);

    /*now to get the L2 address*/
    IX_ERROR_CR(ix_rm_buffer_get_meta(arg_hBuffer, (void **)&pHwMeta));
    IX_ERROR_CR(ix_cc_nd_find_port(pHwMeta->m_InputPort, &pPort));
    ix_ossl_memcpy(nbrAdv.l2Addr, pPort->pPortProperties->mac_addr,
                   IX_CC_IPV6_MAC_ADDR_LEN);
    nbrAdv.l2AddrLength = IX_CC_IPV6_MAC_ADDR_LEN;

    /*Decide whether to include the ethernet header*/
    if(arg_pL2Opt)
    {
        nbrAdv.macHdrSpecified = TRUE;
        ix_ossl_memcpy(nbrAdv.macHdr.dstAddr, arg_pL2Opt->linkLayerAddr,
                       IX_CC_IPV6_MAC_ADDR_LEN);
        ix_ossl_memcpy(nbrAdv.macHdr.srcAddr, pPort->pPortProperties->mac_addr,
                       IX_CC_IPV6_MAC_ADDR_LEN);
        nbrAdv.macHdr.protocol = IX_CC_IPV6_ETH_HDR_PROTO_V6;
    }
    else
    {
        nbrAdv.macHdrSpecified = FALSE;
    }

    /*Now send the nbr adv message*/
    IX_NTOH128(&pNbrSolMsg->targetAddr, &tempHbTargAddr);
    if(!srcAddrSpecified)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("Sending a Nbr Adv to the All Nodes address\n");
#endif /*IPV6_DEBUG*/

        MAKE_ALL_NODES_ADDR(tempAddr);
        IX_NTOH128(&tempAddr, &tempHbAddr);
        IX_ERROR_CR(ix_cc_ipv6_icmp_send_nbr_adv(tempHbTargAddr,
                                           tempHbAddr,
                                           &nbrAdv,
                                           IX_NTOH16(pHwMeta->m_InputPort),
                                           pPort->pPortProperties->blade_id));
    }
    else
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("Sending a Nbr Adv\n");
#endif /*IPV6_DEBUG*/

        pIpv6Hdr = (ix_cc_ipv6_header *)pIpv6Pkt;
        IX_NTOH128(&pIpv6Hdr->srcAddress, &tempHbAddr);
        IX_ERROR_CR(ix_cc_ipv6_icmp_send_nbr_adv(tempHbTargAddr,
                                               tempHbAddr,
                                               &nbrAdv,
                                               IX_NTOH16(pHwMeta->m_InputPort),
                                             pPort->pPortProperties->blade_id));
    }

    return IX_SUCCESS;
}/*ix_cc_nd_send_sol_nbr_adv*/


/**
 * TYPENAME: ix_cc_nd_send_queued_pkts
 *
 * DESCRIPTION: Sends the packets in the list to the IPv6 forwarder in order
 *              to be transmitted to the next hop.
 * 
 * @Param:  - IN arg_PktList - List of packets.
 * @Param:  - IN result - Result to send to the caller
 * @Param:  - OUT - None.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_send_queued_pkts(ix_ipv6_nd_nbr_info *pNbrEntry, ix_error result)
{
    LPLIST      pktList;
    LPLISTITR   pktItr;
    ix_ipv6_nd_pkt_queue_struct *pktStruct;
 
#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */
    
    pktList = pNbrEntry->packetQueue;
    pktItr = ListItrCreate(pktList);
    if(!pktItr)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,
			        ("List creation failed \n"));
    }
    
    while((pktStruct = (ix_ipv6_nd_pkt_queue_struct *)ListItrGetNext(pktItr)))
    {
        ListItrRemoveItem(pktItr);
        
        /*if the packet structure is used for storing a packet, free the 
         *buffer chain. The other use of the packet structure is for
         *storing the context of a neighbor add call. In this case, only
         *send a message*/
        if(pktStruct->hBuffer != IX_NULL_BUFFER_HANDLE)
        {
            /*Send the packet. However, do it only if the state is reachable. */ 
            if(pNbrEntry->state != IX_REACH_TYPE_REACHABLE)
                continue;

            /* next, check if the L2 index is set to 0. This would indicate that
             * there was no lookup done on the packet before it reached the Ethernet
             * Tx core component. Therefore, there will probably be nothing stored
             * at that index in the L2 table. This means that when the Ethernet 
             * microblock inspects the L2 Index in the meta data, it will find it 0
             * and send it to the Ethernet Tx CC, which will call address resolution!
             * To avoid this viscious circle, we generate the ethernet header here 
             * itself and set the L2Index in the meta data to -1. This way, the 
             * microblock will pass the packet without doing a lookup.
             */
             if(pNbrEntry->l2Id == 0)
             {
                /*Generate an ethernet header and append it to the packet*/ 
                ix_cc_nd_add_eth_hdr(pNbrEntry->l2Addr, pktStruct->hBuffer);
             }
             if (ix_rm_packet_send(IX_CC_IPV6_PKT_QM_OUTPUT, pktStruct->hBuffer, 0) != IX_SUCCESS)
             {
                 ix_rm_buffer_free_chain(pktStruct->hBuffer);
             }
        }
        else
        {
#ifndef MEAT_TEST
            ix_cc_msup_send_reply_msg(pktStruct->pContext, NULL, 0, result);
#endif
        }
        ix_ossl_free(pktStruct);
    }

    ListItrDelete(pktItr);
    return IX_SUCCESS;
}/*ix_cc_nd_send_queued_pkts*/

/**
 * TYPENAME: ix_cc_nd_send_icmp_unreachable
 *
 * DESCRIPTION:
 *          Sends ICMP destination unreachable errors to all packets in the
 *          list. Note: This function doesn't free the list.
 * 
 * @Param:  - IN arg_PktList - List of packets.
 * @Param:  - OUT - None.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_send_icmp_unreachable(LPLIST arg_PktList)
{
    LPLISTITR   pktItr;
    ix_ipv6_nd_pkt_queue_struct *pktStruct;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */
    
    pktItr = ListItrCreate(arg_PktList);
    if(!pktItr)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,
			        ("List creation failed \n"));
    }

    while((pktStruct = (ix_ipv6_nd_pkt_queue_struct *) ListItrGetNext(pktItr)))
    {
        if(pktStruct->hBuffer != IX_NULL_BUFFER_HANDLE)
            _ix_cc_ipv6_icmp_dest_unreachable(pktStruct->hBuffer, 
                                        IX_CC_ICMPV6_DU_CODE_ADDR_UNREACHABLE);
    }

    ListItrDelete(pktItr);
    return IX_SUCCESS;
}/*ix_cc_nd_send_icmp_unreachable*/

/**
 * TYPENAME: ix_cc_nd_handle_nbr_sol_event
 *
 * DESCRIPTION:
 *          Handles periodic neighbor solicitation events per RFC2461 
 *          section 7.2.2.
 *          
 * @Param:  - IN arg_pContext - Context identifying the event. 
 * @Param:  - OUT - None.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_handle_nbr_sol_event(void *arg_pContext)
{
    ix_error            errVal = IX_SUCCESS;
    ix_uint128          *pIpv6Addr, solNodeAddr;
    ix_uint128          tempIpAddr1, tempIpAddr2, tempIpAddr3;
    ix_ipv6_nd_nbr_info *pNbrEntry;
    ix_ipv6_nd_port     *pPort;
    ix_ipv6_addr_conf_ip_struct *ipStruct;
    
#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */
    
    pIpv6Addr = (ix_uint128 *)arg_pContext;
    ND_LOCK();
    IX_IPV6_ERROR_CGT(ix_cc_nd_find_nbrCache_entry(*pIpv6Addr, &pNbrEntry),
                      errVal, LBL_ERR);

    if(!pNbrEntry)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: NBR SOL Event CB: Invalid context\n");
#endif /* IPV6_DEBUG */

        /*The nbr doesn't exist!*/
        errVal = IX_CC_ERROR_NULL; 
        goto LBL_ERR;
    }

    pNbrEntry->nbrSolEventScheduled = FALSE;

    if(pNbrEntry->state != IX_REACH_TYPE_INCOMPLETE)
    {
#if defined(IPV6_DEBUG)
                ix_ossl_message_log("State is not INCOMPLETE. Exiting...\n");
#endif /* IPV6_DEBUG */
        goto LBL_ERR;
    }

    if(pNbrEntry->numUnansProbes >= IX_CC_ND_MAX_MULTICAST_SOLICIT)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: NBR SOL Event CB: Max number of NBR Sols exceeded. Address Resolution failed\n");
#endif /* IPV6_DEBUG */

        /*Nbr Disc has failed*/
        IX_IPV6_ERROR_CGT(ix_cc_nd_send_icmp_unreachable(
                                                    pNbrEntry->packetQueue),
                          errVal, LBL_ERR);
        IX_IPV6_ERROR_CGT(ix_cc_nd_free_buffer_list(pNbrEntry->packetQueue,
                                IX_CC_IPV6_ERROR_FAILED_ADDRESS_RESOLUTION, 0),
                          errVal, LBL_ERR);
        pNbrEntry->numUnansProbes = 0;
        /*pNbrEntry->state = IX_ADDR_RES_STATE_INVALID;*/
        goto LBL_ERR;
    }

    IX_IPV6_ERROR_CGT(ix_cc_nd_find_port(pNbrEntry->outputPort, &pPort),
                      errVal, LBL_ERR);

    /*Now send a Nbr Sol message with the L2 address option*/
    /*First, search for the link local address*/
    IX_IPV6_ERROR_CGT(ix_cc_addr_conf_find_ll_addr_struct(pPort, &ipStruct),
                      errVal, LBL_ERR);
    if(!ipStruct)
    {
        /* No link local address. Use any address. if there is none, 
         * return error*/
        if(!(ipStruct = ListGetFirst(pPort->ipAddrList) ))
        {
            errVal = IX_CC_ERROR_OOM;
            goto LBL_ERR;
        }
    }

    MAKE_SOL_NODE_ADDR(pNbrEntry->ipAddr, &solNodeAddr);

    IX_NTOH128(&ipStruct->ipAddr, &tempIpAddr1);
    IX_NTOH128(&solNodeAddr, &tempIpAddr2);
    IX_NTOH128(&pNbrEntry->ipAddr, &tempIpAddr3);
    IX_IPV6_ERROR_CGT(ix_cc_ipv6_icmp_send_nbr_sol(tempIpAddr1, tempIpAddr2,
                                        tempIpAddr3,
                                        pPort->pPortProperties->mac_addr,
                                        IX_NTOH16(pPort->portId), 
                                        pPort->pPortProperties->blade_id),
                                        errVal, LBL_ERR);
    pNbrEntry->numUnansProbes++;

    /*Set timer for next Nbr Sol message*/
#if !defined(IX_EXCLUDE_CCI)
    errVal = ix_cci_cc_add_event_handler_ex(g_pIpv6Context->hIpv6Handle,
                                            IX_CC_ND_RETRANS_TIMER,
                                            ix_cc_nd_handle_nbr_sol_event,
                                            IX_EVENT_TYPE_ONESHOT,
                                            IX_CC_IPV6_ND_EVENT_PRIORITY,
                                            (void *) &(pNbrEntry->ipAddr),
                                            &(pNbrEntry->hNbrSolEventHdl));
    if(errVal == IX_SUCCESS)
        pNbrEntry->nbrSolEventScheduled = TRUE;
#elif MEAT_TEST
        ListAddItem(g_Context_Nd, (void *) &(pNbrEntry->ipAddr));
        pNbrEntry->nbrSolEventScheduled = TRUE;
#endif /* IX_EXCLUDE_CCI */

LBL_ERR:
    ND_UNLOCK();

    if(errVal != IX_SUCCESS)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: NBR SOL Event CB failed. Error: %ld\n", errVal);
#endif /* IPV6_DEBUG */
    }
    return errVal;
}/*ix_cc_nd_handle_nbr_sol_event*/


/**
 * TYPENAME: ix_cc_nd_extract_pkt_from_buffer
 *
 * DESCRIPTION: Extracts the IPv6 packet from the hardware buffer(s).
 * 
 * @Param:  - IN arg_hBuffer - Handle to the buffer containing the packet.
 * @Param:  - OUT arg_pPkt - Pointer to the IPv6 packet extracted from the
 *                          buffer.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_extract_pkt_from_buffer(ix_buffer_handle arg_hBuffer,
                                             ix_uint8         **arg_pPkt)
{
    ix_buffer_handle  curBufHdl, newBufHdl;
    ix_hw_buffer_meta *pHwMeta;
    void              *pData, *pCurPosn;
    ix_int32          pktLen, curDataLen;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */
    
    IX_ERROR_CRT(ix_rm_buffer_get_meta( arg_hBuffer, (void **)&pHwMeta),
                                             IX_CC_IPV6_ERROR_CCI, 0);

    pktLen = IX_NTOH16(pHwMeta->m_PacketSize);
    *arg_pPkt = (ix_uint8 *) ix_ossl_malloc(pktLen);
    if(!*arg_pPkt)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Extract Pkt: Malloc failed\n");
#endif /* IPV6_DEBUG */
        return IX_CC_ERROR_OOM;
    }

    pCurPosn = *arg_pPkt;
    curBufHdl = arg_hBuffer;
    while(pktLen > 0)
    {
        curDataLen = IX_NTOH16(pHwMeta->m_BufferSize);
        IX_ERROR_CRT(ix_cc_hw_get_packet_data(curBufHdl, &pData),
                                 IX_CC_ERROR_INTERNAL, 0);
        if(!ix_ossl_memcpy(pCurPosn, pData, curDataLen))
        {
#if defined(IPV6_DEBUG)
            ix_ossl_message_log("IPv6ND: Extract Pkt: Memcpy failed\n");
#endif /* IPV6_DEBUG */
            ix_ossl_free(arg_pPkt);
            return IX_CC_ERROR_OOM;
        }

        pCurPosn += curDataLen;
        pktLen -= curDataLen;

        IX_ERROR_CRT(ix_rm_buffer_get_next( curBufHdl, &newBufHdl),
                                 IX_CC_IPV6_ERROR_CCI, 0);
        if(newBufHdl == IX_NULL_BUFFER_HANDLE)
            break; /*no more buffers in the chain*/

        curBufHdl = newBufHdl;
        IX_ERROR_CRT(ix_rm_buffer_get_meta( curBufHdl, (void **)&pHwMeta),
                                 IX_CC_IPV6_ERROR_CCI, 0);

    }/*while*/

    return IX_SUCCESS;
}/*ix_cc_nd_extract_pkt_from_buffer*/

/**
 * TYPENAME: ix_cc_nd_locate_icmp_pkt
 *
 * DESCRIPTION: Locates an ICMP packet within the supplied IPv6 packet. It 
 *              skips the list of known IPv6 extension headers, until it
 *              either finds the ICMP header or an unknown header.
 * 
 * @Param:  - IN arg_pIpv6Pkt - Pointer to the IPv6 packet extracted from the
 *                              buffer.
 * @Param:  - OUT arg_pIcmpLocation - Pointer to the start of the ICMP packet 
 *                              within the IPv6 packet.
 * @Param:  - OUT arg_Offset - The offset from the start of the IPV6 packet
 *                              at which the ICMP packet starts.
 *
 * @Return: TRUE if ICMP packet is found, FALSE otherwise.
 */
ix_uint8 ix_cc_nd_locate_icmp_pkt(ix_uint8  *arg_pIpv6Pkt,
                                  ix_uint8  **arg_pIcmpLocation,
                                  ix_uint32 *arg_Offset)
{
    ix_cc_ipv6_header   *pIpv6Header;
    ix_cc_ipv6_options_hdr_common *pIpOptHdr;
    ix_uint8            *pCurPosn;
    ix_uint8            nextHeader;
    ix_uint32           offset, curLength;
    ix_uint8            found = FALSE;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */
    
    pCurPosn = arg_pIpv6Pkt;
    pIpv6Header = (ix_cc_ipv6_header *) pCurPosn;
    nextHeader = pIpv6Header->nextHeader;
    
    pCurPosn += IX_CC_IPV6_HDR_LEN;
    offset = IX_CC_IPV6_HDR_LEN;

    /* get the ICMP header.
     * IMPORTANT NOTE: It is assumed here that the calling function has
     * already processed any headers preceeding the ICMP header 
     * (RFC 2460, section 4, pg 6)*/
    while( (nextHeader != IX_CC_IPV6_NEXT_HDR_NONE) && !found)
    {
        switch(nextHeader)
        {
            case IX_CC_IPV6_NEXT_HDR_HOPBYHOP: /*fall thru*/
            case IX_CC_IPV6_NEXT_HDR_ROUTING: /*fall thru*/
            case IX_CC_IPV6_NEXT_HDR_FRAGMENT: /*fall thru*/
            case IX_CC_IPV6_NEXT_HDR_DSTOPT:
                pIpOptHdr = (ix_cc_ipv6_options_hdr_common *) pCurPosn;
                nextHeader = pIpOptHdr->nextHeader;
                
                /*Length of the header is in 'num of 8 octets', excluding
                 *the first 8 octets. So add 1 to length, before multiplying
                 *by 8*/
                curLength = (1+IX_NTOH16(pIpOptHdr->hdrLength))*8;
                pCurPosn += curLength;
                offset += curLength;

#if defined(IPV6_DEBUG)
                ix_ossl_message_log("ICMP Message located!\n");
#endif /* IPV6_DEBUG */

                break;
            case IX_CC_IPV6_NEXT_HDR_ICMP:
                *arg_pIcmpLocation = pCurPosn;
                *arg_Offset = offset;
                found = TRUE;
                break;
            default:
                /*don't care what other upper layer protocol headers there are.
                 * This isn't an ICMP packet, so we shouldn't process any 
                 * further*/
                nextHeader = IX_CC_IPV6_NEXT_HDR_NONE;
#if defined(IPV6_DEBUG)
                ix_ossl_message_log("IPv6ND: Locate ICMP: Unknown Next hdr\n");
#endif /* IPV6_DEBUG */
                break;
        }/*switch*/        
    }/*while*/
 
    if(!found)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Locate ICMP: ICMP message not found in pkt\n");
#endif /* IPV6_DEBUG */
    }
    return found;
}/*ix_cc_nd_locate_icmp_pkt*/

/**
 * TYPENAME: ix_cc_nd_find_port
 *
 * DESCRIPTION: Searches g_portList for the port structure containing
 *              the supplied port ID.
 * 
 * @Param:  - IN arg_PortId - The port number.
 * @Param:  - OUT arg_PortStruct - The structure containing information
 *                            about the port.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_find_port(ix_uint16 arg_PortId,
                                  ix_ipv6_nd_port **arg_PortStruct)
{
    ix_ipv6_nd_port *pCurPort;
    LPLISTITR   portItr;
    ix_uint8        found = FALSE;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */
    
    portItr = ListItrCreate(g_portList);
    if(!portItr)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Find Port: OOM\n");
#endif /* IPV6_DEBUG */
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,
			        ("List creation failed \n"));
    }

    while((pCurPort = (ix_ipv6_nd_port *)ListItrGetNext(portItr)))
    {
        if(pCurPort->portId == arg_PortId)
        {
            found = TRUE;
            *arg_PortStruct = pCurPort;
            break;
        }
    }

    ListItrDelete(portItr);

    if(!found)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Find Port failed\n");
#endif /* IPV6_DEBUG */
        return IX_CC_ERROR_INTERNAL;
    }

    return IX_SUCCESS;
}/*ix_cc_nd_find_port*/


/**
 * TYPENAME: ix_cc_nd_find_nbrCache_entry
 *
 * DESCRIPTION: Searches the g_nbrCache list for the neighbor cache entry
 *              containing the specified IP address.
 * 
 * @Param:  - IN ipAddr - The IPv6 address of the neighbor.
 * @Param:  - OUT pNbrEntry - The neighbor cache entry corresponding to the 
 *                        neighbor's IP address.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_find_nbrCache_entry(ix_uint128 ipAddr, 
                                   ix_ipv6_nd_nbr_info **pNbrEntry)
{
    LPLISTITR   nbrItr;
    ix_ipv6_nd_nbr_info *pCurNbrEntry;
    ix_uint8        found = FALSE;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */
    
    nbrItr = ListItrCreate(g_nbrCache);
    if(!nbrItr)
    {
        ix_ossl_message_log("IPv6ND: Find Nbr Cache: OOM\n");
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,
			        ("List creation failed \n"));
    }

#if defined(IPV6_DEBUG)
    ix_ossl_message_log ("The address being searched is %lx %lx %lx %lx\n",
                              ipAddr.aUint32[0], ipAddr.aUint32[1],
                              ipAddr.aUint32[2], ipAddr.aUint32[3]);
#endif /* IPV6_DEBUG */

    while((pCurNbrEntry = (ix_ipv6_nd_nbr_info *)ListItrGetNext(nbrItr)))
    {
#if defined(IPV6_DEBUG)
    ix_ossl_message_log ("The address being compared is %lx %lx %lx %lx \n",
               pCurNbrEntry->ipAddr.aUint32[0], pCurNbrEntry->ipAddr.aUint32[1],
               pCurNbrEntry->ipAddr.aUint32[2], pCurNbrEntry->ipAddr.aUint32[3]);
#endif /* IPV6_DEBUG */

        if(!memcmp(&ipAddr, &pCurNbrEntry->ipAddr, 
                    IX_CC_IPV6_ADDRESS_LENGTH_BYTES))
        {
            found = TRUE;
            break;
        }
    }

    ListItrDelete(nbrItr);
    if(found)
        *pNbrEntry = pCurNbrEntry;
    else
    {
        *pNbrEntry = NULL;
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Find Nbr Cache: Entry not found\n");
#endif /* IPV6_DEBUG */
    }
    
    return IX_SUCCESS;
}/*ix_cc_nd_find_nbrCache_entry*/

/**
 * TYPENAME: ix_cc_nd_create_nbr_entry
 *
 * DESCRIPTION:
 *          Creates a new nbr entry, initializes all fields to 0, and adds
 *          it to the nbr cache
 * 
 * @Param:  - IN - None.
 * @Param:  - OUT arg_pNbrEntry - Empty neighbor info structure.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_create_nbr_entry(ix_ipv6_nd_nbr_info **arg_pNbrEntry)
{
    ix_ipv6_nd_nbr_info *pNbrEntry;
    ix_error            errVal = IX_CC_ERROR_OOM;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */
    
    /*create a new nbr entry and add it to the cache*/
    pNbrEntry = (ix_ipv6_nd_nbr_info *) ix_ossl_malloc(
                                        sizeof(ix_ipv6_nd_nbr_info));
    if(!pNbrEntry)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Create Nbr Entry: Malloc failed\n");
#endif /* IPV6_DEBUG */
        goto LBL_EXIT;
    }

    ix_ossl_memset(pNbrEntry, 0, sizeof(ix_ipv6_nd_nbr_info));

    if((pNbrEntry->packetQueue = ListCreate()) == NULL)
        goto LBL_ERR;

    if(!ListAddItem(g_nbrCache, (void *)pNbrEntry))
    {
        ListDelete(pNbrEntry->packetQueue);
        goto LBL_ERR;
    }

    *arg_pNbrEntry = pNbrEntry;

    errVal = IX_SUCCESS;
    goto LBL_EXIT;

LBL_ERR:
#if defined(IPV6_DEBUG)
    ix_ossl_message_log("IPv6ND: Create Nbr Entry Failed\n");
#endif /* IPV6_DEBUG */

    ix_ossl_free(pNbrEntry);
LBL_EXIT:
    return errVal;
}/*ix_cc_nd_create_nbr_entry*/

/**
 * TYPENAME: ix_cc_nd_cleanup_nbr_cache
 *
 * DESCRIPTION: Deletes the contents of the neighbor cache. Deletes the 
 *              g_nbrCache list as well.
 * 
 * @Param:  - None.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_cleanup_nbr_cache(void)
{
    LPLISTITR   nbrItr;
    ix_ipv6_nd_nbr_info *nbrEntry;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */
    
    if(!g_nbrCache)
        return IX_CC_ERROR_NULL;

    nbrItr = ListItrCreate(g_nbrCache);
    if(!nbrItr)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,
			        ("List creation failed \n"));
    }

    while((nbrEntry = (ix_ipv6_nd_nbr_info *)ListItrGetNext(nbrItr)))
    {
        ListItrRemoveItem(nbrItr);
        ix_cc_nd_free_buffer_list(nbrEntry->packetQueue, IX_SUCCESS, 1);
        ListDelete(nbrEntry->packetQueue);
        ix_ossl_free(nbrEntry);
    }
    ListItrDelete(nbrItr);
    ListDelete(g_nbrCache);
    return IX_SUCCESS;
}/*ix_cc_nd_cleanup_nbr_cache*/

/**
 * TYPENAME: ix_cc_nd_cleanup_port_list
 *
 * DESCRIPTION: Deletes the contents of the port list. Deletes the g_portList
 *              list as well.
 * 
 * @Param:  - None.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_nd_cleanup_port_list(void)
{
    LPLISTITR   portItr;
    ix_ipv6_nd_port *portEntry;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */
    
    if(!g_portList)
        return IX_CC_ERROR_NULL;

    portItr = ListItrCreate(g_portList);
    if(!portItr)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,
			        ("List creation failed \n"));
    }

    while((portEntry = (ix_ipv6_nd_port *)ListItrGetNext(portItr)))
    {
        ListItrRemoveItem(portItr);

        if(portEntry->ipAddrList)
            ListFreeDelete(portEntry->ipAddrList);

        ix_ossl_free(portEntry);
    }
    ListItrDelete(portItr);
    ListDelete(g_portList);
    return IX_SUCCESS;
}/*ix_cc_nd_cleanup_port_list*/

/**
 * TYPENAME: ix_cc_nd_free_buffer_list
 *
 * DESCRIPTION: Frees the buffers contained in the list. Does NOT delete
 *              the list itself (i.e. buffers may later be added to the same
 *              list).
 * 
 * @Param:  - IN arg_PktList - List of packets to be freed. 
 * @Param:  - IN arg_Result - List of packets to be freed. 
 * @Param:  - IN arg_freeChain - if 1, indicates that the buffer chains should 
 *                      be freed. If 0, indicates that the buffers should 
 *                      only be removed from the list and not be freed (this
 *                      is useful when ICMP error messages have been sent for
 *                      the buffers).
 * @Param:  - OUT - None.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 * DESCRIPTION:
 *          Frees all hardware buffers in the list.
 */
ix_error ix_cc_nd_free_buffer_list(LPLIST arg_PktList, ix_error arg_Result,
                                   ix_uint8 arg_freeChain)
{
    LPLISTITR   pktItr;
    ix_ipv6_nd_pkt_queue_struct *pktStruct;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */
    
    pktItr = ListItrCreate(arg_PktList);
    if(!pktItr)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,
			        ("List creation failed \n"));
    }

    while((pktStruct = (ix_ipv6_nd_pkt_queue_struct *)
                                                    ListItrGetNext(pktItr)))
    {
        ListItrRemoveItem(pktItr);

        /*if the packet structure is used for storing a packet, free the 
         *buffer chain. The other use of the packet structure is for
         *storing the context of a neighbor add call. In this case, only
         *send a message*/
        if(pktStruct->hBuffer == IX_NULL_BUFFER_HANDLE)
        {
#ifndef MEAT_TEST
           ix_cc_msup_send_reply_msg(pktStruct->pContext, NULL, 0, arg_Result);
#endif
        }
        else
        {
            if(arg_freeChain)
                ix_rm_buffer_free_chain(pktStruct->hBuffer);
        }
        ix_ossl_free(pktStruct);
    }

    ListItrDelete(pktItr);
    return IX_SUCCESS;
}/*ix_cc_nd_free_buffer_list*/

#if defined(XML_SYSAPP_SUPPORT)
/**
 * TYPENAME: ix_cc_nd_is_ll_ip_addr
 *
 * DESCRIPTION: Checks whether the supplied IP address is a link local address
 * 
 * @Param:  - IN arg_IpAddr - The IPv6 Address to be checked.
 * @Param:  - OUT - None.
 *
 * @Return: TRUE if the address is link local, FALSE otherwise
 */
ix_uint8 ix_cc_nd_is_ll_ip_addr(ix_uint128 arg_ipAddr)
{
    ix_uint128 tempAddr;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */
    
    MAKE_BLANK_LL_ADDR(tempAddr);

    /*Return TRUE if the upper 64 bits match*/
   if(!memcmp(&tempAddr, &arg_ipAddr, sizeof(ix_uint128)/2))
       return TRUE;
   else
       return FALSE;
}/*ix_cc_nd_is_ll_ip_addr*/
#endif

/**
 * TYPENAME: ix_cc_nd_start_addr_resolution
 *
 * DESCRIPTION: 
 *          This function starts the process of address resolution
 *          by sending a neighbor sol or setting an event for sending
 *          one.
 *
 * @Param:  - IN pNbrEntry: Pointer to the Neighbor entry structure
 *                          on which the address resolution is to be
 *                          performed.
 * @Param:  - OUT NOne.
 *
 * @Result: IX_SUCCESS if successful, appropriate error otherwise.
 */
ix_error ix_cc_nd_start_addr_resolution(ix_ipv6_nd_nbr_info *pNbrEntry,
                                        ix_ipv6_nd_port *pPort)
{
    ix_error errVal;
    ix_ipv6_addr_conf_ip_struct *ipStruct;
    ix_uint128 solNodeAddr;
    ix_uint128 tempIpAddr1, tempIpAddr2, tempIpAddr3;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */
    
    /*Now send a Nbr Sol message with the L2 address option*/
    if(pNbrEntry->nbrSolEventScheduled == TRUE)
    {
        /*An event has already been scheduled for nbr sol. so don't do
         * anything at this point*/
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Addr Res: Nbr Sol event already scheduled"
                ". Exiting with IX_SUCCESS.\n");
#endif /* IPV6_DEBUG */
        errVal = IX_SUCCESS;
        goto LBL_ERR;
    }

    /* Now check if the max allowed probes have already been sent. If they 
     * have, exit.*/
    if(pNbrEntry->numUnansProbes >= IX_CC_ND_MAX_MULTICAST_SOLICIT)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Addr Res: Max probes sent.\n");
#endif /* IPV6_DEBUG */
        errVal = IX_CC_IPV6_ERROR_FAILED_ADDRESS_RESOLUTION;
        goto LBL_ERR;
    }

    IX_IPV6_ERROR_CGT(ix_cc_addr_conf_find_ll_addr_struct(pPort, &ipStruct),
                      errVal, LBL_ERR);

    if(!ipStruct)
    {
        /*if there's no link local address, use any other address*/
        if(!(ipStruct = ListGetFirst(pPort->ipAddrList)))
        {
            errVal = IX_CC_ERROR_OOM;
            goto LBL_ERR;
        }
    }

    MAKE_SOL_NODE_ADDR(pNbrEntry->ipAddr, &solNodeAddr);

    IX_NTOH128(&ipStruct->ipAddr, &tempIpAddr1);
    IX_NTOH128(&solNodeAddr, &tempIpAddr2);
    IX_NTOH128(&pNbrEntry->ipAddr, &tempIpAddr3);

    ix_cc_ipv6_icmp_send_nbr_sol(tempIpAddr1, tempIpAddr2,
                                 tempIpAddr3,
                                 pPort->pPortProperties->mac_addr,
                                 IX_NTOH16(pPort->portId),
                                 pPort->pPortProperties->blade_id);
    pNbrEntry->numUnansProbes++;

    /*Set timer for next Nbr Sol message*/
#if !defined(IX_EXCLUDE_CCI)
    errVal = ix_cci_cc_add_event_handler_ex(g_pIpv6Context->hIpv6Handle,
                                            IX_CC_ND_RETRANS_TIMER,
                                            ix_cc_nd_handle_nbr_sol_event,
                                            IX_EVENT_TYPE_ONESHOT,
                                            IX_CC_IPV6_ND_EVENT_PRIORITY,
                                            (void *) &(pNbrEntry->ipAddr),
                                            &(pNbrEntry->hNbrSolEventHdl));
    if(errVal == IX_SUCCESS)
    {
        pNbrEntry->nbrSolEventScheduled = TRUE;
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Added Event for sending Nbr Sol.\n");
#endif /* IPV6_DEBUG */
    }
    else
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Addr Res: Failed to add event handler\n");
#endif /* IPV6_DEBUG */
    }
#elif MEAT_TEST
        ListAddItem(g_Context_Nd, (void *) &(pNbrEntry->ipAddr));
        pNbrEntry->nbrSolEventScheduled = TRUE;
#endif /* IX_EXCLUDE_CCI */
    
LBL_ERR:
    return errVal;
}/*ix_cc_nd_start_addr_resolution*/

/**
 * TYPENAME: ix_cc_nd_update_l2_table
 *
 * DESCRIPTION:
 *      This function creates an L2 TM entry srtucture, populates it and
 *      makes a call to add it to the L2 table.
 *
 * @Param:  - IN pNbrEntry: Pointer to the neighbor entry.
 * @Param:  - IN pPort: Pointer to the outgoing port's structure.
 * @Param:  - OUT None.
 *
 * @Result: IX_SUCCESS if the call is successful. Appropriate error
 *          code otherwise.
 */
ix_error ix_cc_nd_update_l2_table(ix_ipv6_nd_nbr_info *pNbrEntry,
                                  ix_ipv6_nd_port *pPort)
{
    ix_error            errVal = IX_SUCCESS;
#ifdef USE_FPM_CC
    ix_cc_l2tm_entry    *l2Entry;
#endif
    ix_uint128          *pContext;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */
    
    /*We want our context to be the IP address of the neighbor*/
    pContext = (ix_uint128 *) ix_ossl_malloc(sizeof(ix_uint128));
    if(!pContext)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Update L2 table: Malloc failed\n");
#endif /* IPV6_DEBUG */
        errVal = IX_CC_ERROR_OOM;
        goto LBL_EXIT;
    }

    ix_ossl_memcpy(pContext, &pNbrEntry->ipAddr, sizeof(ix_uint128));

#ifdef USE_FPM_CC
    l2Entry = (ix_cc_l2tm_entry *) ix_ossl_malloc(sizeof(ix_cc_l2tm_entry));
    if(!l2Entry)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Update L2 table: Malloc Failed\n");
#endif /* IPV6_DEBUG */
        errVal = IX_CC_ERROR_OOM;
        goto LBL_FREE_CONTEXT;
    }

    /*populate the l2 structure*/
    l2Entry->l2AddrType =  IX_CC_L2TM_L2ADDR_ETHER;
    l2Entry->ipAddrType = IX_CC_L2TM_IPADDR_IPV6;
    ix_ossl_memcpy(&l2Entry->l3Address, &pNbrEntry->ipAddr,
                           sizeof(ix_uint128));
    ix_ossl_memcpy(l2Entry->header.ether.destMAC, pNbrEntry->l2Addr,
                           IX_CC_IPV6_MAC_ADDR_LEN);
    ix_ossl_memcpy(l2Entry->header.ether.sourceMAC, 
                           pPort->pPortProperties->mac_addr, 
                           IX_CC_IPV6_MAC_ADDR_LEN);
    l2Entry->header.ether.protocol = ETH_IPV6_PROTO;

    errVal = ix_cc_fpm_async_add_l2_entry(pNbrEntry->l2Id,
                                          pNbrEntry->outputPort,
                                          l2Entry, ix_cc_nd_cb_l2_update,
                                          pContext);
#if defined(IPV6_DEBUG)
    ix_ossl_message_log("IPv6ND: Called FPM\n");
#endif /* IPV6_DEBUG */

#else
    errVal = ix_cc_nd_cb_l2_update(IX_SUCCESS, pContext);
#if defined(IPV6_DEBUG)
    ix_ossl_message_log("IPv6ND: Directly calling the callback\n");
#endif /* IPV6_DEBUG */

#endif /* USE_FPM_CC */
    
    if(errVal == IX_SUCCESS)
        goto LBL_EXIT;
    else
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Update L2 table: FPM call failed\n");
#endif /* IPV6_DEBUG */
    }

#ifdef USE_FPM_CC
    if(l2Entry)
        ix_ossl_free(l2Entry);
LBL_FREE_CONTEXT:
#endif
    if(pContext)
        ix_ossl_free(pContext);
LBL_EXIT:
    return errVal;
}/*ix_cc_nd_update_l2_table*/

ix_error ix_cc_nd_add_eth_hdr(ix_uint8 *pMacAddr, ix_buffer_handle arg_hBuffer)
{
    ix_hw_buffer_meta *pHwMeta;
    void              *pData;
    ix_ipv6_nd_port   *pPort;
    ix_ipv6_eth_hdr   *pEthHdr;
    ix_uint32         pktSize;
    ix_uint32         cell_count=0;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */

    IX_ERROR_CRT(ix_rm_buffer_get_meta( arg_hBuffer, (void **)&pHwMeta),
                                             IX_CC_IPV6_ERROR_CCI, 0);

    IX_ERROR_CRT(ix_cc_nd_find_port(pHwMeta->m_OutputPort, &pPort),
                                        IX_CC_ERROR_INTERNAL, 0);

    pHwMeta->m_Offset = IX_HTON16(IX_NTOH16(pHwMeta->m_Offset) -
                                                               IX_CC_IPV6_ETH_HDR_LEN);

    IX_ERROR_CRT(ix_cc_hw_get_packet_data(arg_hBuffer, &pData), 
                                        IX_CC_ERROR_INTERNAL, 0); 
    pEthHdr = (ix_ipv6_eth_hdr *)pData;
    ix_ossl_memcpy(pEthHdr->dstAddr, pMacAddr, IX_CC_IPV6_MAC_ADDR_LEN); 
    ix_ossl_memcpy(pEthHdr->srcAddr, pPort->pPortProperties->mac_addr, 
                   IX_CC_IPV6_MAC_ADDR_LEN);
    pEthHdr->protocol = IX_CC_IPV6_ETH_HDR_PROTO_V6;
    pEthHdr->protocol = IX_HTON16(pEthHdr->protocol);
    
    pktSize = IX_NTOH16(pHwMeta->m_PacketSize);
    pHwMeta->m_PacketSize = IX_HTON16(pktSize + IX_CC_IPV6_ETH_HDR_LEN);
    pHwMeta->m_BufferSize = IX_HTON16(IX_NTOH16(pHwMeta->m_BufferSize) +
                                                            IX_CC_IPV6_ETH_HDR_LEN);
    /* Reset cell count */
    cell_count = ix_cc_calculate_cell_count(pktSize, 1);
    ix_rm_buffer_cell_count_set(&arg_hBuffer, cell_count);

    pHwMeta->m_NextHopID = -1;
    return IX_SUCCESS;
}
/***************************************************************************
 * 
 *              ADDRESS AUTOCONFIGURATION FUNCTIONS
 *              
 **************************************************************************/

/**
 * TYPENAME: ix_cc_addr_conf_init
 * 
 * DESCRIPTION:
 *      This function starts the address autoconfiguration component. It 
 *      initializes duplicate address detection for each IP address assigned 
 *      to every port. Also, if it doesn't find a link local address for a 
 *      port, it assigns a link local address as defined in RFC 2462,
 *      section 5.3
 * 
 * @Param:  - None.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_addr_conf_init(void)
{
    ix_error    errVal;
    LPLISTITR   portItr;
    ix_ipv6_nd_port *portEntry;
    ix_uint8    linkLocalAddrFound;
    LPLISTITR   addrItr;
    ix_ipv6_addr_conf_ip_struct *addrEntry;
    ix_uint128  tempIpAddr;
    
#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */
    
    if(!g_portList)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: AddrAutoconf Init: g_PortList not initialized\n");
#endif /* IPV6_DEBUG */
        return IX_CC_ERROR_NULL;
    }

    portItr = ListItrCreate(g_portList);
    if(!portItr)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: AddrAutoconf Init: OOM\n");
#endif /* IPV6_DEBUG */
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,
			        ("List creation failed \n"));
    }
    
    while((portEntry = (ix_ipv6_nd_port *)ListItrGetNext(portItr)))
    {
        linkLocalAddrFound = FALSE;
        if(!portEntry->ipAddrList)
        {
            /* Something's wrong. This list dhould have been created in the
             * Update Properties function*/
#if defined(IPV6_DEBUG)
            ix_ossl_message_log("IPv6ND: AddrAutoconf Init: "
                                "portEntry->ipAddrList uninitialized\n");
#endif /* IPV6_DEBUG */
            return IX_CC_ERROR_NULL;
        }

        addrItr = ListItrCreate(portEntry->ipAddrList);
        if(!addrItr)
        {
#if defined(IPV6_DEBUG)
            ix_ossl_message_log("IPv6ND: AddrAutoconf Init: OOM\n");
#endif /* IPV6_DEBUG */
            errVal = IX_ERROR_WARNING(IX_CC_ERROR_OOM,
			            ("List creation failed \n"));
            goto LBL_EXIT;
        }
        while((addrEntry = (ix_ipv6_addr_conf_ip_struct *) ListItrGetNext
                                                            (addrItr)))
        {
            if(addrEntry->state == IX_ADDR_RES_STATE_TENTATIVE)
            {
                if(addrEntry->numDadTransmits == 0)
                {
#if defined(IPV6_DEBUG)
                    ix_ossl_message_log("IPv6ND: AddrAutoconf Init: Max DAD"
                                        " transmits already sent\n");
#endif /* IPV6_DEBUG */
                    /*Generate an event?*/
                    addrEntry->state = IX_ADDR_RES_STATE_INVALID;
                    continue;
                }
                IX_IPV6_ERROR_CGT(ix_cc_addr_conf_set_dad_event(portEntry,
                                                                addrEntry),
                                  errVal, LBL_ERR);
                if(addrEntry->linkLayerAddr)
                    linkLocalAddrFound = TRUE;
            }
        }/*while*/
        
        ListItrDelete(addrItr);
        if(!linkLocalAddrFound)
        {
            /* The link local address was not manually specified. Hence we
             * need to assign it based on the MAC address of the link*/
            addrEntry = (ix_ipv6_addr_conf_ip_struct *)
                            ix_ossl_malloc(sizeof(ix_ipv6_addr_conf_ip_struct));    
            if(!addrEntry)
            {
#if defined(IPV6_DEBUG)
                ix_ossl_message_log("IPv6ND: AddrAutoconf Init: Malloc failed\n");
#endif /* IPV6_DEBUG */
                errVal = IX_CC_ERROR_OOM;
                goto LBL_ERR;
            }
            IX_IPV6_ERROR_CGT(ix_cc_addr_conf_generate_ll_addr(portEntry,
                                                         &addrEntry->ipAddr),
                              errVal, LBL_ENTRY_ERR);

           IX_NTOH128(&addrEntry->ipAddr, &tempIpAddr);
            _ix_cc_ipv6_property_update_ip_address(
                                IX_CC_SET_PROPERTY_ADD_INTERFACE_IPV6,
                                THIS_BLADE_ID,
                                IX_NTOH16(portEntry->portId),
                                tempIpAddr);

            addrEntry->state = IX_ADDR_RES_STATE_TENTATIVE;
            addrEntry->numDadTransmits = g_DadTransmits;        
            /* insert the new structure into the list and then start DAD*/
            if(!ListAddItem(portEntry->ipAddrList, (void *)addrEntry))
            {
#if defined(IPV6_DEBUG)
                ix_ossl_message_log("IPv6ND: AddrAutoconf Init: List add failed\n");
#endif /* IPV6_DEBUG */
                errVal = IX_CC_ERROR_OOM;
                goto LBL_ENTRY_ERR;
            }
            IX_IPV6_ERROR_CGT(ix_cc_addr_conf_set_dad_event(portEntry,
                                                            addrEntry),
                              errVal, LBL_ENTRY_ERR);
        }
    }/*while*/
    
    errVal = IX_SUCCESS;
#if defined(IPV6_DEBUG)
    ix_ossl_message_log("IPv6ND: AddrAutoconf initialized successfully\n");
#endif /* IPV6_DEBUG */
    goto LBL_EXIT;

LBL_ENTRY_ERR:
    if(addrEntry)
        ix_ossl_free(addrEntry);
LBL_ERR:
    if(addrItr)
        ListItrDelete(addrItr);
LBL_EXIT:
    if(portItr)
        ListItrDelete(portItr);
    return errVal;
}/*ix_cc_addr_conf_init*/

/**
 * TYPENAME: ix_cc_addr_conf_generate_ll_addr
 *
 * DESCRIPTION: Generates a link local address based on the interface
 *              identifier associated with the supplied port. See
 *              RFC2461, section 5.3 and RFC2464, sections 4 and 5
 * 
 * @Param:  - IN arg_pPort - Pointer to the port structure.
 * @Param:  - OUT ipAddr - The generated IPv6 addrss.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_addr_conf_generate_ll_addr(ix_ipv6_nd_port *arg_pPort, 
                                          ix_uint128 *ipAddr)
{
    ix_uint8 *macAddr;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */
    
    if(!arg_pPort || !ipAddr)
        return IX_CC_ERROR_NULL;

    macAddr = arg_pPort->pPortProperties->mac_addr;
    MAKE_BLANK_LL_ADDR((*ipAddr));
    /*Now copy the Interface Identifier*/
    (*ipAddr).aUint8[8] = macAddr[0] ^ 0x02; /*flip the Universal/Local bit*/
    (*ipAddr).aUint8[9] = macAddr[1];
    (*ipAddr).aUint8[10] = macAddr[2];
    (*ipAddr).aUint8[11] = 0xFF;
    (*ipAddr).aUint8[12] = 0xFE;
    (*ipAddr).aUint8[13] = macAddr[3];
    (*ipAddr).aUint8[14] = macAddr[4];
    (*ipAddr).aUint8[15] = macAddr[5];

    return IX_SUCCESS;
}/*ix_cc_addr_conf_generate_ll_addr*/

/**
 * TYPENAME: ix_cc_addr_conf_set_dad_event
 *
 * DESCRIPTION: Sets an event to send Neighbor Solicitation messages
 *              for Duplicate Address Detection.
 * 
 * @Param:  - IN arg_pPort - Pointer to the port structure.
 * @Param:  - IN arg_pIpStruct - The specific IP address structure stored 
 *                          within the port's IP address list.
 * @Param:  - OUT - None.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_error ix_cc_addr_conf_set_dad_event(ix_ipv6_nd_port *arg_pPort,
                                    ix_ipv6_addr_conf_ip_struct *arg_pIpStruct)
{
    ix_error    errVal;
    ix_ipv6_addr_conf_context *pContext;
    ix_uint32   randDelay;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */

    pContext = (ix_ipv6_addr_conf_context *)
                            ix_ossl_malloc(sizeof(ix_ipv6_addr_conf_context));
    if(!pContext)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Set DAD event: Malloc failed\n");
#endif /* IPV6_DEBUG */
        return IX_CC_ERROR_OOM;
    }

    pContext->portId = arg_pPort->portId;
    pContext->pIpStruct = arg_pIpStruct;

    /*Calculate a random delay between 0 and MAX_RTR_SOL_DELAY in milliSecs*/

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)  
    randDelay = ix_cc_rand()%(IX_CC_ND_MAX_RTR_SOL_DELAY * 1000);
#else
    randDelay = rand()%(IX_CC_ND_MAX_RTR_SOL_DELAY * 1000);
#endif

    /** 
     * If this is getting invoked as a part of initialization, make sure that the
     * randDelay is enough to ensure that eventHandler is invoked after the complete 
     * initilization is done. Increase the randDelay by IX_CC_ND_EVENT_EXTRA_DELAY 
     * to allow time for ix_cc_ipv6_init() to be completed first.
     * (This is required for both Linux build and vxWorks build)
     */
    if(!g_Initialized)
    {
       randDelay = randDelay + IX_CC_ND_EVENT_EXTRA_DELAY;
    }

#if !defined(IX_EXCLUDE_CCI)
    errVal = ix_cci_cc_add_event_handler_ex(
                                            g_pIpv6Context->hIpv6Handle,
                                            randDelay,
                                            ix_cc_addr_conf_handle_dad_event,
                                            IX_EVENT_TYPE_ONESHOT,
                                            IX_CC_IPV6_ND_EVENT_PRIORITY,
                                            (void *) pContext,
                                            &(arg_pIpStruct->hDadHandle));
    if(errVal != IX_SUCCESS)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: Set DAD event: Failed to add event\n");
#endif /* IPV6_DEBUG */
        goto LBL_ERR;
    }
#elif MEAT_TEST
    ListAddItem(g_Context_Nd,pContext);
#endif /* IX_EXCLUDE_CCI */

    return IX_SUCCESS;
LBL_ERR:
    ix_ossl_free(pContext);
    return errVal;
}/*ix_cc_addr_conf_set_dad_event*/

/**
 * TYPENAME: ix_cc_addr_conf_handle_dad_event
 *
 * DESCRIPTION: Handles DAD events set earlier. 
 *              If the max allowed Nbr Sol messages have already been sent,
 *              DAD has failed. Else, it sends a neighbor SOl message and 
 *              sets another event.
 * 
 * @Param:  - IN arg_pContext - Context identifying the event. 
 * @Param:  - OUT - None.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 *
 */
ix_error ix_cc_addr_conf_handle_dad_event(void *pContext)
{
    ix_error errVal;
    ix_ipv6_addr_conf_context *pAddrContext;
    ix_ipv6_nd_port *pPort;
    ix_uint128  srcAddr, allNodesAddr; 
    ix_uint128  tempIpAddr1, tempIpAddr2, tempIpAddr3;
    
    CHK_INIT();
    
#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */

    if(!pContext)
    {
        return IX_CC_ERROR_NULL;
    }

    pAddrContext = (ix_ipv6_addr_conf_context *)pContext;
    IX_IPV6_ERROR_CGT(ix_cc_nd_find_port(pAddrContext->portId, &pPort),
                                                       errVal, LBL_EXIT);

    if(!ix_cc_addr_conf_is_ip_ptr_valid(pPort, pAddrContext->pIpStruct))
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: DAD Event CB: Invalid Context\n");
#endif /* IPV6_DEBUG */
        errVal = IX_CC_IPV6_ERROR_INVALID_CONTEXT;
        goto LBL_EXIT;
    }

    errVal = IX_SUCCESS;
    if(pAddrContext->pIpStruct->state != IX_ADDR_RES_STATE_TENTATIVE)
    {
        goto LBL_EXIT;
    }

    if(pAddrContext->pIpStruct->numDadTransmits == 0)
    {
        /* No Nbr Advs received within this time. DAD succeeded! */
        /* Set the address state to Valid and exit */
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: DAD Event CB: DAD succeeded!\n");
#endif /* IPV6_DEBUG */
        pAddrContext->pIpStruct->state = IX_ADDR_RES_STATE_VALID;
        goto LBL_EXIT;
    }

    /* Send a Nbr sol: The address it is sent to is a AllNode Multicast */
    ix_ossl_memset(&srcAddr, 0, sizeof(ix_uint128));
    ix_ossl_memset(&allNodesAddr, 0, sizeof(ix_uint128));
    allNodesAddr.aUint8[0] = 0xFF;
    allNodesAddr.aUint8[1] = 0x02;
    allNodesAddr.aUint8[15] = 0x01;

    /* Send a NBR Sol out the port speicified in the parameter */
    IX_NTOH128(&srcAddr, &tempIpAddr1);
    IX_NTOH128(&allNodesAddr, &tempIpAddr2);
    IX_NTOH128(&pAddrContext->pIpStruct->ipAddr, &tempIpAddr3);

    IX_IPV6_ERROR_CGT(
                ix_cc_ipv6_icmp_send_nbr_sol(tempIpAddr1, 
                                             tempIpAddr2,
                                             tempIpAddr3,
                                             pPort->pPortProperties->mac_addr, 
                                             IX_NTOH16(pPort->portId),
                                             pPort->pPortProperties->blade_id),
                         errVal, LBL_EXIT);

    pAddrContext->pIpStruct->numDadTransmits--;
    IX_IPV6_ERROR_CGT(ix_cc_addr_conf_set_dad_event(pPort, 
                                                    pAddrContext->pIpStruct),
                                          errVal, LBL_EXIT);
    
LBL_EXIT:
    if(errVal != IX_SUCCESS)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: DAD Event CB failed. Error: %ld\n", errVal);
#endif /* IPV6_DEBUG */
    }

    /*
     * pContext is allocated in the function ix_cc_addr_conf_set_dad_event.
     * Hence this should be freed here in the event handler.
     */
    if (pContext != NULL)
    {
        ix_ossl_free(pContext);
    }

    return errVal;

}/*ix_cc_addr_conf_handle_dad_event*/

/**
 * TYPENAME: ix_cc_addr_conf_is_ip_ptr_valid
 * 
 * DESCRIPTION: Checks if the pointer to the address structure is valid.
 * 
 * @Param:  - IN arg_pPort - Pointer to the port structure.
 * @Param:  - IN arg_pIpStruct - The specific IP address structure stored 
 *                          within the port's IP address list.
 * @Param:  - OUT - None.
 *
 * @Return: TRUE if the pointer is valid, FALSE otherwise.
 */
ix_uint8 ix_cc_addr_conf_is_ip_ptr_valid(ix_ipv6_nd_port *arg_pPort,
                                 ix_ipv6_addr_conf_ip_struct *arg_pIpStruct)
{
    LPLISTITR   addrItr;
    ix_ipv6_addr_conf_ip_struct *tempAddr;
    ix_uint8    found = FALSE;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */

    addrItr = ListItrCreate(arg_pPort->ipAddrList);
    if(!addrItr)
        return FALSE;

    while((tempAddr = (ix_ipv6_addr_conf_ip_struct *) ListItrGetNext(addrItr)))
    {
        if(tempAddr == arg_pIpStruct)
        {
            found = TRUE;
            break;
        }
    }

    if(!found)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: AddrAutoconfig: Invalid IP Ptr\n");
#endif /* IPV6_DEBUG */
    }

    ListItrDelete(addrItr);
    return found;
}/*ix_cc_addr_conf_is_ip_ptr_valid*/

/**
 * TYPENAME: ix_cc_addr_conf_find_ip_addr
 *
 * DESCRIPTION: Finds the address structure containing the supplied IP
 *              address.
 * 
 * @Param:  - IN arg_pPort - Pointer to the port structure.
 * @Param:  - IN arg_ipAddr - The IP address to find.
 * @Param:  - OUT arg_pIpStruct - The specific IP address structure stored 
 *                          within the port's IP address list.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for
 *          failure.
 */
ix_uint8 ix_cc_addr_conf_find_ip_addr(ix_ipv6_nd_port *arg_pPort,
                                  ix_uint128 arg_ipAddr,
                                  ix_ipv6_addr_conf_ip_struct **arg_pIpStruct)
{
    LPLISTITR   addrItr;
    ix_ipv6_addr_conf_ip_struct *tempAddr;
    ix_uint8    found = FALSE;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */

    addrItr = ListItrCreate(arg_pPort->ipAddrList);
    if(!addrItr)
        return FALSE;

    while((tempAddr = (ix_ipv6_addr_conf_ip_struct *) ListItrGetNext(addrItr)))
    {
        if(!memcmp(&tempAddr->ipAddr, &arg_ipAddr, sizeof(ix_uint128)))
        {
            found = TRUE;
            *arg_pIpStruct = tempAddr;
            break;
        }
    }

    if(!found)
    {
#if defined(IPV6_DEBUG)
        ix_ossl_message_log("IPv6ND: AddrAutoconfig: IP struct not found\n");
#endif /* IPV6_DEBUG */
    }

    ListItrDelete(addrItr);
    return found;
}/*ix_cc_addr_conf_find_ip_addr*/


static ix_error ix_cc_addr_conf_find_ll_addr_struct(ix_ipv6_nd_port *pPort, 
                                             ix_ipv6_addr_conf_ip_struct **pIp)
{
    LPLISTITR   addrItr;
    ix_ipv6_addr_conf_ip_struct *ipStruct;
    ix_uint8    found;

#if defined(IPV6_DEBUG)
    ix_ossl_message_log("Entering %s\n", __FUNCTION__);
#endif /* IPV6_DEBUG */

    addrItr = ListItrCreate(pPort->ipAddrList);
    if(!addrItr)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,
			        ("List creation failed \n"));
    }

    while((ipStruct = (ix_ipv6_addr_conf_ip_struct *)ListItrGetNext(addrItr)))
    {
        if(ipStruct->linkLayerAddr == TRUE)
        {
            found = TRUE;
            break;
        }
    }

    ListItrDelete(addrItr);
   
    if(found)
        *pIp = ipStruct;
    else
        *pIp = NULL;

    return IX_SUCCESS;
}


