/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 6
 *
 * = MODULE
 *      IPv6 Forwarder Core Component
 *
 * = FILENAME
 *      ip_options.c
 *
 * = DESCRIPTION
 *   This file contains functions that handle IPv6 extension headers and
 *   options present in extension headers.
 *
 * = CREATION TIME
 *      12/16/2002
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */
#define IX_ERROR_FILE_IDENT "$Id: v6_ip_options.c,v 1.3 2003/12/12 00:49:54 ktseng Exp $";

/**
 * System defined include files required.
 */

/**
 * User defined include files required.
 */
#include "ix_cc_error.h"
#include "cc/ipv6/internal/ip_options.h"
#include "cc/ipv6/internal/v6_header_validation.h"

/**
 * Preprocessor symbols and macros used in this file.
 */

/**
 * Types definitions whose scope is limited to this file.
 */

/**
 * Variable declarations global to this file only.  Externs are followed by
 * static variables.
 */

/**
 * Function definitions.
 */

/**
 * NAME: _ix_cc_ipv6_parse_ext_headers
 *
 * DESCRIPTION: This function is called when the packet has extension headers
 * It appropriately modifies the header of the packet and decides if the 
 * packet needs to be forwarded or dropped.
 * It is the responsibilty of the caller of this function to implement the 
 * actual forwarding/dropping of the packet.
 * 
 * @Param:  - IN arg_pIphdr - Pointer to IPv6 header
 *            IN arg_pIphdr - pointer to the buffer containing the header.
 *
 * @Param:  - OUT - arg_pDrop - On return set to 1 if the packet should 
 *                              be dropped
 *                              On return set to 0 if the packet should 
 *                              be forwarded  
 *
 * @Return: IX_SUCCESS if successful 
 *          valid ix_error token for failure.
 *
 * Note: This function will free the buffer in cases where it returns an
 * error or the packet has been sent up to the stack driver (the arg_pDrop
 * flag is set). If not, the packet needs furthur processing.
 */
ix_error _ix_cc_ipv6_parse_ext_headers(ix_cc_ipv6_header *arg_pIpHdr,
                                       ix_buffer_handle arg_hBuffer,
                                       ix_uint32 *arg_pDrop)
{
    /* 
     * This argument should read "if this is set, the forwarder will continue
     * as though the packet is no longer in its control
     */

    *arg_pDrop = 0;

    /* Check for extension and ICMP headers */
    _ix_cc_ipv6_inspect_pkt_hdr(arg_hBuffer, (ix_uint8 *)arg_pDrop);

    return IX_SUCCESS;
}
