/******************************************************************************
 *                                                                     
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001-2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 ******************************************************************************
 *
 *
 *      File Name: v6v4_tunnel_decap_uc.h
 *
 *      Purpose: This file contains tunneling decapsulation definitions
 *      applicable to the microengines. 
 *
 *      History:
 *
 *      Date            Comment                         		
 *      -----------------------------------------------------------------------
 *
 *      02/12/2003      Created
 *
 *
 ******************************************************************************/

#ifndef	__V6V4_TUNNEL_DECAP_UC_H__
#define	__V6V4_TUNNEL_DECAP_UC_H__

/* Exception codes */

#define V6V4_DECAP_EXCEPTION_LOCAL      0x80
#define V6V4_DECAP_EXCEPTION_AUTO       0x81
#define V6V4_DECAP_EXCEPTION_FRAGMENT   0x82

/* Offsets for optional packet counters */

#ifdef V6V4_DECAP_COUNTERS
#define V6V4_DECAP_DROP_COUNT           0
#define V6V4_DECAP_EXCP_COUNT           4
#endif


/******************************************************************************
 *	Format of ending tunnel endpoint next hop information.
 *-----------------------------------------------------------------------------
 *
 *  Bits
 *  31                24                16                8                 0
 *  |--------|--------|--------|--------|--------|--------|--------|--------|
 *
 *  +-----------------------------------------------------------------------+
 *  |               Flags               |        Ingress Source List        |
 *  |-----------------------------------------------------------------------|
 *  |                                Unused                                 |
 *  |-----------------------------------------------------------------------|
 *  |                                Unused                                 |
 *  |-----------------------------------------------------------------------|
 *  |                                Unused                                 |
 *  +-----------------------------------------------------------------------+
 *
 *   Flags                End tunnel flags.  Indicates whether the entry
 *                        represents an ending tunnel endpoint, and if so,
 *                        how to validate a packet before decapsulating.
 *
 *   Ingress Source List  Index of list of valid source prefixes for an
 *                        incoming tunnel.
 *
 *****************************************************************************/


/* Bit definitions for tunnel flags */

#define V6V4_DECAP_FLAGS_VALID              31

#define V6V4_DECAP_FLAGS_AUTO               16
#define V6V4_DECAP_FLAGS_SRC_VALIDATE       17
#define V6V4_DECAP_FLAGS_6TO4               18
#define V6V4_DECAP_FLAGS_TOS_V6             19

/* Longword offsets of data in tunnel next hop entry. */

#define V6V4_DECAP_TNH_FLAGS_LW             0
#define V6V4_DECAP_TNH_INGRESS_LIST_LW      0

/* Byte offsets and lengths of relevant data in tunnel next hop entry. */

#define V6V4_DECAP_TNH_INGRESS_LIST         2, 2

/* Number of longwords in each entry. */

#define V6V4_DECAP_TNH_LW_COUNT             1

/* Shift amount for reading entry (total entry size = 4). */

#define V6V4_DECAP_TNH_SHF                  2


#ifndef V6V4_INGRESS_LIST_TRIE

/******************************************************************************
 * Tunnel Ingress Source Validation List structure (default format)
 * This structure is used to validate the source address for packets received
 * on configured tunnels.
 *
 *----------------------------------------------
 *	byte offset,size of ingress list fields
 *----------------------------------------------
 *
 *  Byte 0       1        2        3        4       5         6       7
 * |--------|--------|--------|--------|--------|--------|--------|--------|
 *
 * +-----------------------------------------------------------------------+
 * |    Next Block   | Validity Flags  |        Mask Length 1 - 4          |
 * |-----------------------------------------------------------------------|
 * |            Address 1              |           Address 2               |
 * |-----------------------------------------------------------------------|
 * |            Address 3              |           Address 4               |
 * |-----------------------------------------------------------------------|
 * |        Mask Length 5 - 8          |           Address 5               |
 * |-----------------------------------------------------------------------|
 * |            Address 6              |           Address 7               |
 * |-----------------------------------------------------------------------|
 * |            Address 8              |        Mask Length 9 - 12         |
 * |-----------------------------------------------------------------------|
 * |            Address 9              |           Address 10              |
 * |-----------------------------------------------------------------------|
 * |            Address 11             |           Address 12              |
 * +-----------------------------------------------------------------------+
 *
 * Size 8 Quad words = 64 bytes
 *
 * Next Block: Index of next block.
 *
 * Validity Flags: Indicates whether each address entry is valid (bits 11:0)
 *
 * Mask Length 1 - 4: Specifies number of "don't care" bits to mask
 *     from source address when comparing to addresses 1 - 4
 *    (8 bits per address).
 *
 * Mask Length 5 - 8: Specifies number of "don't care" bits to mask
 *     from source address when comparing to addresses 5 - 8
 *     (8 bits per address).
 *
 * Mask Length 9 - 12: Specifies number of "don't care" bits to mask
 *     from source address when comparing to addresses 9 - 12
 *     (8 bits per address).
 *
 *
 * Address N: Address to compare to masked source address.
 *
 *****************************************************************************/

#define V6V4_ISL_FLAGS           0
#define V6V4_ISL_NEXT_BLOCK      0
#define V6V4_ISL_PREFIX_1_4      1
#define V6V4_ISL_ADDRESS_1       2
#define V6V4_ISL_ADDRESS_2       3
#define V6V4_ISL_ADDRESS_3       4
#define V6V4_ISL_ADDRESS_4       5
#define V6V4_ISL_PREFIX_5_8      6
#define V6V4_ISL_ADDRESS_5       7
#define V6V4_ISL_ADDRESS_6       8
#define V6V4_ISL_ADDRESS_7       9
#define V6V4_ISL_ADDRESS_8       10
#define V6V4_ISL_PREFIX_9_12     11
#define V6V4_ISL_ADDRESS_9       12
#define V6V4_ISL_ADDRESS_10      13
#define V6V4_ISL_ADDRESS_11      14
#define V6V4_ISL_ADDRESS_12      15

#define V6V4_ISL_ADDR_PER_BLOCK  12
#define V6V4_ISL_LW_COUNT        16
#define V6V4_ISL_SHF             6

/* V6V4_INGRESS_LIST_MAX_ADDR can optionally be set outside the block to set
 * an absolute limit on the number of entries that can be in an ingress list.
 */

#ifndef V6V4_INGRESS_LIST_MAX_ADDR

/* No limit specified */

#define V6V4_MAX_ADDRESS                V6V4_ISL_ADDR_PER_BLOCK
#define V6V4_ISL_EXTRA_LW               4
#define V6V4_INGRESS_LIST_MULTI_BLOCKS

#else

/* Limit specified */

#define V6V4_MAX_ADDRESS   V6V4_INGRESS_LIST_MAX_ADDR
#ifdef  V6V4_INGRESS_LIST_MULTI_BLOCKS
#undef  V6V4_INGRESS_LIST_MULTI_BLOCKS
#endif

#if (V6V4_MAX_ADDRESS < 1)
	#error 2 "V6V4_INGRESS_LIST_MAX_ADDR must be at least 1"
#elif (V6V4_MAX_ADDRESS < 5)
    #define V6V4_ISL_EXTRA_LW 2
#elif (V6V4_MAX_ADDRESS < 9)
    #define V6V4_ISL_EXTRA_LW 3
#elif (V6V4_MAX_ADDRESS < 13)
    #define V6V4_ISL_EXTRA_LW 4
#else
	#error 2 "V6V4_INGRESS_LIST_MAX_ADDR must be between 1 and 12"
#endif

#ifdef IXP_MICROCODE
    #define_eval V6V4_INGRESS_LIST_MAX (V6V4_MAX_ADDRESS + V6V4_ISL_EXTRA_LW)
#else
    #define      V6V4_INGRESS_LIST_MAX (V6V4_MAX_ADDRESS + V6V4_ISL_EXTRA_LW)
#endif


#endif /* ifndef V6V4_INGRESS_LIST_MAX_ADDR */

#else

/*
 * Trie blocks used for ingress source validation.  The trie tables are
 * global, and the index value in the trie block refers to a block used to
 * store all tunnel indexes that allow a prefix.  Presumably, this should be
 * small, so the default size of these blocks is 8 (which allows 3 tunnel IDs).
 * This can be changed by setting V6V4_INGRESS_LIST_INDEX_BLOCK_SIZE to a value
 * greater than 8.  The value will be adjusted upward to the next power of 2,
 * with a maximum of 64.  Each block is an array of 16-bit indexes, with the
 * first one reserved as an index to the next block.
 */

#ifndef V6V4_INGRESS_LIST_INDEX_BLOCK_SIZE
#define V6V4_INGRESS_LIST_INDEX_BLOCK_SIZE 8
#endif

#if (V6V4_INGRESS_LIST_INDEX_BLOCK_SIZE > 32)
    #define V6V4_INGRESS_LIST_INDEX_LW 16
    #define V6V4_ISL_INDEX_SHF 6
#elif (V6V4_INGRESS_LIST_INDEX_BLOCK_SIZE > 16)
    #define V6V4_INGRESS_LIST_INDEX_LW 8
    #define V6V4_ISL_INDEX_SHF 5
#elif (V6V4_INGRESS_LIST_INDEX_BLOCK_SIZE > 8)
    #define V6V4_INGRESS_LIST_INDEX_LW 4
    #define V6V4_ISL_INDEX_SHF 4
#else
    #define V6V4_INGRESS_LIST_INDEX_LW 2
    #define V6V4_ISL_INDEX_SHF 3
#endif

/* Constants needed for processing trie-based ingress source list. */

#define V6V4_MAX_TRIES			    7
#define V6V4_ISL_TRIE_SHF           6
#define V6V4_ISL_TRIE_BASE_SHF      22

/* Calculate some convienient numbers based on the trie and index
 * configuration
 */

#ifdef IXP_MICROCODE
#define_eval V6V4_ISL_INDEX_PER_BLOCK ((V6V4_INGRESS_LIST_INDEX_LW * 2) - 1)

#define_eval V4V6_ISL_TRIE_BLOCKS_BASE \
             (TUNNEL_INGRESS_LIST_SRAM_BASE + 1024)

#define_eval V6V4_ISL_TRIE_BLOCKS_SIZE \
             (TUNNEL_INGRESS_LIST_NUM_TRIE_BLOCKS * 64)

#define_eval V4V6_ISL_INDEX_BLOCKS_BASE \
             (V4V6_ISL_TRIE_BLOCKS_BASE + V6V4_ISL_TRIE_BLOCKS_SIZE)
#else

#define V6V4_ISL_INDEX_PER_BLOCK ((V6V4_INGRESS_LIST_INDEX_LW * 2) - 1)

#define V4V6_ISL_TRIE_BLOCKS_BASE \
             (TUNNEL_INGRESS_LIST_SRAM_BASE + 1024)

#define V6V4_ISL_TRIE_BLOCKS_SIZE \
             (TUNNEL_INGRESS_LIST_NUM_TRIE_BLOCKS * 64)

#define V4V6_ISL_INDEX_BLOCKS_BASE \
             (V4V6_ISL_TRIE_BLOCKS_BASE + V6V4_ISL_TRIE_BLOCKS_SIZE)
#endif

#endif /* V6V4_INGRESS_LIST_TRIE */


/* Constants we need for processing header. */

#define V6V4_IPV4_FRAGMENT_BITS     14

#ifdef IXP_MICROCODE
#define_eval V6V4_FRAGMENT_SHF      (32 - V6V4_IPV4_FRAGMENT_BITS)
#else
#define      V6V4_FRAGMENT_SHF      (32 - V6V4_IPV4_FRAGMENT_BITS)
#endif


#endif	/* __V6V4_TUNNEL_DECAP_UC_H__ */
