/******************************************************************************
 *                                                                     
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                      
 *     COPYRIGHT (c)  2001-2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *     THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                      
 *                        INTEL  CORPORATION                            
 *                                                                     
 *                     2200 MISSION COLLEGE BLVD                        
 *                                                                      
 *               SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                      
 ******************************************************************************
 *
 *
 *      File Name: v6v4_tunnel_encap_uc.h
 *
 *      Purpose: This file contains tunneling encapsulation definitions
 *      applicable to the microengines. 
 *
 *      History:
 *
 *      Date            Comment                         		
 *      -----------------------------------------------------------------------
 *
 *      02/12/2003      Created
 *
 *
 ******************************************************************************/

#ifndef	__V6V4_TUNNEL_ENCAP_UC_H__
#define	__V6V4_TUNNEL_ENCAP_UC_H__


// Exception codes

#define V6V4_ENCAP_EXCEPTION_MTU         0x90


/******************************************************************************
 *	Format of starting tunnel endpoint next hop information.
 *-----------------------------------------------------------------------------
 *
 *  Bits
 *  31                24                16                8                 0
 *  |--------|--------|--------|--------|--------|--------|--------|--------|
 *
 *  +-----------------------------------------------------------------------+
 *  |               Flags               |             Path MTU              |
 *  |-----------------------------------------------------------------------|
 *  |                             Local Address                             |
 *  |-----------------------------------------------------------------------|
 *  |                            Remote Address                             |
 *  |-----------------------------------------------------------------------|
 *  |       TTL       |       TOS       |              Unused               |
 *  |-----------------------------------------------------------------------|
 *  |                        Subnet Broadcast Address                       |
 *  |-----------------------------------------------------------------------|
 *  |                                Unused                                 |
 *  |-----------------------------------------------------------------------|
 *  |                                Unused                                 |
 *  |-----------------------------------------------------------------------|
 *  |                                Unused                                 |
 *  +-----------------------------------------------------------------------+
 *
 *   Flags                Start tunnel flags.  Indicates whether the entry
 *                        represents a starting tunnel endpoint, and if so,
 *                        how to obtain the IPv4 destination address for the
 *                        encapsulated packet.
 *
 *   Local Address        IPv4 local address to use for the tunnel.
 *
 *   Remote Address       IPv4 remote address to use for the tunnel.
 *
 *   TTL                  TTL value to use for the tunnel.
 *
 *   TOS                  Type of service value to use for the tunnel.
 *
 *   Subnet Broadcast
 *   Address              Subnet broadcast address that cannot be used as
 *                        a destination for an automatic tunnel.
 *
 *****************************************************************************/


/* Bit definitions for tunnel flags. */

#define V6V4_ENCAP_FLAGS_VALID              31

#define V6V4_ENCAP_FLAGS_AUTO               16
#define V6V4_ENCAP_FLAGS_6TO4               17
#define V6V4_ENCAP_FLAGS_TOS_V6             18
#define V6V4_ENCAP_FLAGS_NO_DF              19

/* Longword offsets of data in tunnel next hop entry. */

#define V6V4_ENCAP_TNH_FLAGS_LW             0
#define V6V4_ENCAP_TNH_MTU_LW               0
#define V6V4_ENCAP_TNH_LOCAL_ADDRESS_LW     1
#define V6V4_ENCAP_TNH_REMOTE_ADDRESS_LW    2
#define V6V4_ENCAP_TNH_TTL_LW               3
#define V6V4_ENCAP_TNH_TOS_LW               3
#define V6V4_ENCAP_TNH_SUBNET_BC_LW         4

/* Byte offsets of data in tunnel next hop entry. */

#define V6V4_ENCAP_TNH_MTU_START                2
#define V6V4_ENCAP_TNH_LOCAL_ADDR_START         4
#define V6V4_ENCAP_TNH_LOCAL_ADDR_31_16_START   4
#define V6V4_ENCAP_TNH_LOCAL_ADDR_15_0_START    6
#define V6V4_ENCAP_TNH_REMOTE_ADDR_START        8
#define V6V4_ENCAP_TNH_REMOTE_ADDR_31_16_START  8
#define V6V4_ENCAP_TNH_REMOTE_ADDR_15_0_START   10
#define V6V4_ENCAP_TNH_TTL_START                12
#define V6V4_ENCAP_TNH_TOS_START                13
#define V6V4_ENCAP_TNH_SUBNET_BC_START          16

/* Length of data in tunnel next hop entry. */

#define V6V4_ENCAP_TNH_MTU_LENGTH               2
#define V6V4_ENCAP_TNH_LOCAL_ADDR_LENGTH        4
#define V6V4_ENCAP_TNH_LOCAL_ADDR_31_16_LENGTH  2
#define V6V4_ENCAP_TNH_LOCAL_ADDR_15_0_LENGTH   2
#define V6V4_ENCAP_TNH_REMOTE_ADDR_LENGTH       4
#define V6V4_ENCAP_TNH_REMOTE_ADDR_31_16_LENGTH 2
#define V6V4_ENCAP_TNH_REMOTE_ADDR_15_0_LENGTH  2
#define V6V4_ENCAP_TNH_TTL_LENGTH               1
#define V6V4_ENCAP_TNH_TOS_LENGTH               1
#define V6V4_ENCAP_TNH_SUBNET_BC_LENGTH         4

/* Byte offsets and lengths of relevant data in tunnel next hop entry.
 * in the format convenient for the microcode.
 */

#define V6V4_ENCAP_TNH_MTU                      2, 2
#define V6V4_ENCAP_TNH_LOCAL_ADDR               4, 4
#define V6V4_ENCAP_TNH_LOCAL_ADDR_31_16         4, 2
#define V6V4_ENCAP_TNH_LOCAL_ADDR_15_0          6, 2
#define V6V4_ENCAP_TNH_REMOTE_ADDR              8, 4
#define V6V4_ENCAP_TNH_REMOTE_ADDR_31_16        8, 2
#define V6V4_ENCAP_TNH_REMOTE_ADDR_15_0         10, 2
#define V6V4_ENCAP_TNH_TTL                      12, 1
#define V6V4_ENCAP_TNH_TOS                      13, 1
#define V6V4_ENCAP_TNH_SUBNET_BC                16, 4

/* Number of longwords in the entry. */

#define V6V4_ENCAP_TNH_LW_COUNT             8

/* Shift amount for reading entry (total entry size = 32). */

#define V6V4_ENCAP_TNH_SHF                  5

/* Offsets for the parts of the IPv6 header that we need. */

#define IPV6_6TO4_V4_DEST_ADDR            26, 4
#define IPV6_6TO4_V4_DEST_ADDR_HI         26, 2
#define IPV6_6TO4_V4_DEST_ADDR_LO         28, 2
#define IPV6_AUTO_V4_DEST_ADDR            36, 4

/* Constant values used in the encapsulating header. */

#define IPV4_VERSION                      4
#define IPV4_DONT_FRAGMENT                0x4000
#define IPV4_DONT_FRAGMENT_BIT            14
#define IPV4_FRAGMENTED                   0x3FFF

#define V6V4_HEADER_LW                    5
#define V6V4_TOS                          0


#ifdef IXP_MICROCODE
#define_eval V6V4_VERS_IHL_TOS  \
    ((IPV4_VERSION << 12) | (V6V4_HEADER_LW << 8) | V6V4_TOS)

#define_eval V6V4_FLAGS_FRAGMENT IPV4_DONT_FRAGMENT

#else

#define V6V4_VERS_IHL_TOS  \
    ((IPV4_VERSION << 12) | (V6V4_HEADER_LW << 8) | V6V4_TOS)

#define V6V4_FLAGS_FRAGMENT IPV4_DONT_FRAGMENT
#endif

/* Offsets for each of the longwords in the IPv4 header. */

#define IPV4_HDR_LW_0		0, 4
#define IPV4_HDR_LW_1		4, 4
#define IPV4_HDR_LW_2		8, 4
#define IPV4_HDR_LW_3		12, 4
#define IPV4_HDR_LW_4		16, 4
#define IPV4_HDR_WD_8		16, 2
#define IPV4_HDR_WD_9		18, 2

/* Offsets for optional packet counters */

#ifdef V6V4_ENCAP_COUNTERS
#define V6V4_ENCAP_DROP_COUNT       0
#define V6V4_ENCAP_EXCP_COUNT       4
#endif


#endif	/* __V6V4_TUNNEL_ENCAP_UC_H__ */
