/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor, Release 6
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      IPv6/v4 Tunneling Core Component
 *
 * = FILENAME
 *      ix_cc_v6v4.h
 *
 * = DESCRIPTION
 *      This file defines all exported data types and function prototypes
 *
 * = CREATION TIME
 *      11/11/2002 12:15 PM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: ix_cc_v6v4.h,v 1.3 2003/01/30 18:26:22 cabell Exp $
 */

#if !defined(__IX_CC_V6V4_H__)
#define __IX_CC_V6V4_H__

#include "ix_rm.h"
#include "ix_ossl.h"
#include "ix_cc.h"

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */


/**
 * These defines are used to select or deselect boolean options in
 * the tunneling API.
 */
#define IX_CC_V6V4_SELECT           1
#define IX_CC_V6V4_CLEAR            0


/**
 * TYPENAME: ix_cc_v6v4_statistics
 * 
 * DESCRIPTION: This structure defines format for statistics data.
 *
 */

typedef struct ix_s_cc_v6v4_statistics
{
    ix_uint32 decapMbDropPkts;
    ix_uint32 decapMbExcpPkts;
    ix_uint32 encapMbDropPkts;
    ix_uint32 encapMbExcpPkts;
    ix_uint32 localPkts;
    ix_uint32 autoTunnelPkts;
    ix_uint32 fragmentsRcvd;
    ix_uint32 reassembledPkts;
    ix_uint32 pktTooBigMsgsSent;
    ix_uint32 pktsEncapsulated;
    ix_uint32 pktsDecapsulated;
} ix_cc_v6v4_statistics;



/**
 * TYPENAME: ix_cc_v6v4_messages
 * 
 * DESCRIPTION: This structure defines format for IPv6/v4 tunnelling
 *              messages
 *
 */
typedef enum ix_e_cc_v6v4_messages
{
    /* End Tunnel Messages */

    IX_CC_V6V4_MSG_ADD_END_TUNNEL =IX_CC_COMMON_MSG_ID_LAST,

    IX_CC_V6V4_MSG_DELETE_END_TUNNEL,
    IX_CC_V6V4_MSG_SET_DECAP_TOS_OPTION,
    IX_CC_V6V4_MSG_SET_SRC_VALIDATION,
    IX_CC_V6V4_MSG_GET_END_TUNNEL,
    IX_CC_V6V4_MSG_DUMP_END_TUNNELS,


    /* Ingress Source List Messages */
    IX_CC_V6V4_MSG_ADD_ALLOWED_SOURCE,
    IX_CC_V6V4_MSG_DELETE_ALLOWED_SOURCE,
    IX_CC_V6V4_MSG_CLEAR_ALLOWED_SOURCES,
    IX_CC_V6V4_MSG_GET_ALLOWED_SOURCES,


    /* Start Tunnel Messages */

    IX_CC_V6V4_MSG_ADD_START_TUNNEL,
    IX_CC_V6V4_MSG_DELETE_START_TUNNEL,
    IX_CC_V6V4_MSG_SET_TTL,
    IX_CC_V6V4_MSG_SET_ENCAP_TOS_OPTION,
    IX_CC_V6V4_MSG_SET_TOS,
    IX_CC_V6V4_MSG_SET_MTU,
    IX_CC_V6V4_MSG_SET_SUBNET_BROADCAST,
    IX_CC_V6V4_MSG_GET_START_TUNNEL,
    IX_CC_V6V4_MSG_DUMP_START_TUNNELS,

    /* Statistics message */
    IX_CC_V6V4_MSG_GET_STATISTICS,


    /* Add new messages before this */
    IX_CC_V6V4_MSG_LAST     
} ix_cc_v6v4_messages;



/**
 * TYPENAME: ix_cc_v6v4_error_code
 * 
 * DESCRIPTION: This enumerated type will describe v6v4 error codes
 *
 */

#define IX_ERROR_GROUP_CC_V6V4 IX_ERROR_GROUP_CC_IPV6TM

typedef enum ix_e_cc_v6v4_error_code
{
    IX_CC_V6V4_ERROR_FIRST = IX_ERROR_MAKE_GROUP(CC_V6V4),
    IX_CC_V6V4_ERROR_SUCCESS = IX_CC_V6V4_ERROR_FIRST,

    IX_CC_V6V4_ERROR_FAILED_PATCHING,
    IX_CC_V6V4_ERROR_REGISTRY,
    IX_CC_V6V4_ERROR_BUFFER_FREE,

    IX_CC_V6V4_ERROR_RM,
    IX_CC_V6V4_ERROR_CCI,
    IX_CC_V6V4_ERROR_INVALID_CONTEXT,
    IX_CC_V6V4_ERROR_INVALID_INPUT_PARAM,
    IX_CC_V6V4_ERROR_INVALID_BUFFER,
    IX_CC_V6V4_ERROR_MSG_LIBRARY,
    IX_CC_V6V4_ERROR_INVALID_PACKET,
    IX_CC_V6V4_ERROR_INVALID_ADDRESS,
    IX_CC_V6V4_ERROR_INVALID_MASK,
    IX_CC_V6V4_ERROR_INVALID_POINTER,
    IX_CC_V6V4_ERROR_INVALID_TUNNEL_ADDRESS,
    IX_CC_V6V4_ERROR_INVALID_TUNNEL_ENTRY,

    IX_CC_V6V4_ERROR_DUPLICATE_ID,
    IX_CC_V6V4_ERROR_BUFFER_TOO_SMALL,

    IX_CC_V6V4_ERROR_INVALID_PROP_ID,
    IX_CC_V6V4_ERROR_CALLBACK,
    
    /* unknown error code */
    IX_CC_V6V4_ERROR_UNKNOWN,
    IX_CC_V6V4_ERROR_LAST
} ix_cc_v6v4_error_code;


/*
 * Enumeration of the supported memory types.
 */
typedef enum ix_e_cc_v6v4_mem_type
{
    IX_CC_V6V4_MEM_DRAM,
    IX_CC_V6V4_MEM_SRAM,
    IX_CC_V6V4_MEM_LAST
} ix_cc_v6v4_mem_type;                                                               


/*
 * Enumeration of the supported formats for the ingress source list.
 * The values in this enumerated type specify both the format of the
 * ingress source list and some information about how many prefixes
 * are supported for each tunnel or how many tunnels will allow a
 * given prefix.
 */
typedef enum ix_e_cc_v6v4_isl_type
{
    /* 
     * The DEFAULT formats use a per-tunnel array to store the ingress
     * source list.  The NO_MAX option stores the list in a chain of
     * fixed size blocks.  The MAX_x options restrict the number of
     * entries to x, and use a single fixed size block.
     */

    IX_CC_V6V4_ISL_DEFAULT_NO_MAX = 0,
    IX_CC_V6V4_ISL_DEFAULT_MAX_1 = 1,
    IX_CC_V6V4_ISL_DEFAULT_MAX_2 = 2,
    IX_CC_V6V4_ISL_DEFAULT_MAX_3 = 3,
    IX_CC_V6V4_ISL_DEFAULT_MAX_4 = 4,
    IX_CC_V6V4_ISL_DEFAULT_MAX_5 = 5,
    IX_CC_V6V4_ISL_DEFAULT_MAX_6 = 6,
    IX_CC_V6V4_ISL_DEFAULT_MAX_7 = 7,
    IX_CC_V6V4_ISL_DEFAULT_MAX_8 = 8,
    IX_CC_V6V4_ISL_DEFAULT_MAX_9 = 9,
    IX_CC_V6V4_ISL_DEFAULT_MAX_10 = 10,
    IX_CC_V6V4_ISL_DEFAULT_MAX_11 = 11,
    IX_CC_V6V4_ISL_DEFAULT_MAX_12 = 12,

    IX_CC_V6V4_ISL_DEFAULT_LAST = 13,

    /* The TRIE formats use a global trie table to store the prefixes of the
     * ingress source list.  The "SM, MED, LG, XLG" gives an indication of
     * about how many tunnels will allow the same prefix, and is used to
     * determine the size of the index blocks that store the tunnel IDs that
     * allow a given prefix.  The corresponding index block sizes in bytes
     * are 8 (3 tunnels/block), 16 (7 tunnels/block), 32 (15 tunnels/block),
     * and 64 (31 tunnels/block).
     */

    IX_CC_V6V4_ISL_TRIE_FIRST = 256,

    IX_CC_V6V4_ISL_TRIE_SM = IX_CC_V6V4_ISL_TRIE_FIRST,
    IX_CC_V6V4_ISL_TRIE_MED,
    IX_CC_V6V4_ISL_TRIE_LG,
    IX_CC_V6V4_ISL_TRIE_XLG,

    IX_CC_V6V4_ISL_LAST
} ix_e_cc_v6v4_isl_type;                                                              




/**
 * TYPENAME: ix_cc_v6v4_tunnel_handle
 * 
 * DESCRIPTION: Type definition for end tunnel endpoint identifier.
 *
 */
typedef ix_uint32 ix_cc_v6v4_tunnel_handle;


/**
 * TYPENAME: ix_cc_v6v4_end_tunnel_config
 * 
 * DESCRIPTION: This structure defines the configuration data for
 *              end tunnel endpoints.
 *
 */
typedef struct ix_s_cc_v6v4_end_tunnel_config
{
    ix_uint16 flags;
} ix_cc_v6v4_end_tunnel_config;

/**
 * Flags definitions for ix_cc_v6v4_end_tunnel_config.flags.
 */

#define IX_CC_V6V4_DECAP_AUTO       0x0001
#define IX_CC_V6V4_SRC_VALIDATE     0x0002
#define IX_CC_V6V4_DECAP_6TO4       0x0004
#define IX_CC_V6V4_DECAP_TOS_V6     0x0008


/**
 * TYPENAME: ix_cc_v6v4_end_tunnel_info
 * 
 * DESCRIPTION: This structure contains all information describing
 *              end tunnel endpoints.
 *
 */
typedef struct ix_s_cc_v6v4_end_tunnel_info
{
    ix_cc_v6v4_tunnel_handle     hTunnel;
    ix_cc_v6v4_end_tunnel_config config;
} ix_cc_v6v4_end_tunnel_info;


/**
 * TYPENAME: ix_cc_v6v4_ingress_source_entry
 * 
 * DESCRIPTION: This structure defines format for memory dump data .
 *
 */
typedef struct ix_s_cc_v6v4_ingress_source_entry
{
    ix_uint32 ipAddr;       /* Source IP address */
    ix_uint8  maskLength;   /* Number of meaningful bits in ipAddr */
} ix_cc_v6v4_ingress_source_entry;



/**
 * TYPENAME: ix_cc_v6v4_interface_id
 * 
 * DESCRIPTION: This structure identifies an interface.
 *
 */
typedef struct ix_s_cc_v6v4_interface_id
{
    ix_uint16 bladeId;
    ix_uint16 portId;
} ix_cc_v6v4_interface_id;



/**
 * TYPENAME: ix_cc_v6v4_start_tunnel_config
 * 
 * DESCRIPTION: This structure defines the configuration data for
 *              start tunnel endpoints.
 *
 */
typedef struct ix_s_cc_v6v4_start_tunnel_config
{
    ix_uint16 flags;
    ix_uint16 pathMtu;
    ix_cc_v6v4_interface_id localInterface;
    ix_uint32 remoteIpv4Address;
    ix_uint32 subnetBroadcast;
    ix_uint8  ttl;
    ix_uint8  tos;
} ix_cc_v6v4_start_tunnel_config;

/**
 * Flags definitions for ix_cc_v6v4_start_tunnel_config.flags.
 */

#define IX_CC_V6V4_ENCAP_AUTO       0x0001
#define IX_CC_V6V4_ENCAP_6TO4       0x0002
#define IX_CC_V6V4_ENCAP_TOS_V6     0x0004

/**
 * Special values for pathMtu and subnetBroadcast
 */

#define IX_CC_V6V4_PATH_MTU_INTERFACE   0
#define IX_CC_V6V4_BROADCAST_INTERFACE  0


/**
 * TYPENAME: ix_cc_v6v4_start_tunnel_info
 * 
 * DESCRIPTION: This structure contains all information describing
 *              start tunnel endpoints.
 *
 */
typedef struct ix_s_cc_v6v4_start_tunnel_info
{
    ix_cc_v6v4_tunnel_handle        hTunnel;
    ix_cc_v6v4_start_tunnel_config  config;
    ix_uint8                        trackMtu;
    ix_uint8                        trackBroadcast;
} ix_cc_v6v4_start_tunnel_info;


/**
 * Special value for setting encap TOS option to default in API
 */

#define IX_CC_V6V4_ENCAP_TOS_DEFAULT    0

/**
 * TYPENAME: ix_cc_v6v4_cb
 * 
 * DESCRIPTION: Type definition for general tunneling Message callbacks.
 *
 */

typedef ix_error (*ix_cc_v6v4_cb)(
            ix_error arg_Result,
            void* arg_pContext
        );



/**
 * TYPENAME: ix_cc_v6v4_cb_add_tunnel
 * 
 * DESCRIPTION: Type definition for add tunnel callbacks
 *
 */

typedef ix_error (*ix_cc_v6v4_cb_add_tunnel)(
            ix_error arg_Result,
            void* arg_pContext,
            ix_cc_v6v4_tunnel_handle arg_hTunnel
        );

/**
 * TYPENAME: ix_cc_v6v4_cb_end_tunnel
 * 
 * DESCRIPTION: Type definition for get end tunnel callback
 *
 */

typedef ix_error (*ix_cc_v6v4_cb_end_tunnel)(
            ix_error arg_Result,
            void* arg_pContext,
            ix_cc_v6v4_end_tunnel_info* arg_pTunnelInfo
        );

/**
 * TYPENAME: ix_cc_v6v4_cb_get_sources
 * 
 * DESCRIPTION: Type definition for ingress source list callback
 *
 */

typedef ix_error (*ix_cc_v6v4_cb_get_sources)(
            ix_error arg_Result,
            void* arg_pContext,
            ix_uint32 arg_NumEntries,
            ix_cc_v6v4_ingress_source_entry* arg_paEntries
        );



/**
 * TYPENAME: ix_cc_v6v4_cb_dump_end_tunnels
 * 
 * DESCRIPTION: Type definition for dump end tunnels callback
 *
 */

typedef ix_error (*ix_cc_v6v4_cb_dump_end_tunnels)(
            ix_error arg_Result,
            void* arg_pContext,
            ix_uint32 arg_NumEntries,
            ix_cc_v6v4_end_tunnel_info* arg_paInfo
        );



/**
 * TYPENAME: ix_cc_v6v4_cb_start_tunnel
 * 
 * DESCRIPTION: Type definition for get start tunnel callback
 *
 */

typedef ix_error (*ix_cc_v6v4_cb_start_tunnel)(
            ix_error arg_Result,
            void* arg_pContext,
            ix_cc_v6v4_start_tunnel_info* arg_pTunnelInfo
        );


/**
 * TYPENAME: ix_cc_v6v4_cb_dump_start_tunnels
 * 
 * DESCRIPTION: Type definition for dump start tunnels callback
 *
 */

typedef ix_error (*ix_cc_v6v4_cb_dump_start_tunnels)(
            ix_error arg_Result,
            void* arg_pContext,
            ix_uint32 arg_NumEntries,
            ix_cc_v6v4_start_tunnel_info* arg_paInfo
        );



/**
 * TYPENAME: ix_cc_v6v4_cb_statistics
 * 
 * DESCRIPTION: Type definition for get statistics callback
 *
 */

typedef ix_error (*ix_cc_v6v4_cb_statistics)(
            ix_error arg_Result,
            void* arg_pContext,
            ix_cc_v6v4_statistics* arg_pStats
        );




/**
 * NAME: ix_cc_v6v4_async_add_end_tunnel
 *
 * DESCRIPTION: This message helper primitive helps the client to add an ending
 * tunnel endpoint.
 * 
 * @Param:  - IN arg_pConfig: pointer to configuration information about the tunnel.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */


ix_error ix_cc_v6v4_async_add_end_tunnel(
            ix_cc_v6v4_end_tunnel_config* arg_pConfig,
            ix_cc_v6v4_cb_add_tunnel arg_Callback,
            void* arg_pClientContext
         );

/**
 * NAME: ix_cc_v6v4_async_delete_end_tunnel
 *
 * DESCRIPTION:This message helper primitive helps a client to delete an end
 * tunnel endpoint. 
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */


ix_error ix_cc_v6v4_async_delete_end_tunnel(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_cc_v6v4_cb arg_Callback,
            void* arg_pClientContext
         );


/**
 * NAME: ix_cc_v6v4_async_set_decap_tos_option
 *
 * DESCRIPTION: This message helper primitive helps the client to update the TOS
 * option for decapsulation. 
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Selection: option selection.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */

ix_error ix_cc_v6v4_async_set_decap_tos_option(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_uint8 arg_Selection,
            ix_cc_v6v4_cb arg_Callback,
            void* arg_pClientContext
         );

/**
 * NAME: ix_cc_v6v4_async_set_src_validation
 *
 * DESCRIPTION: This message helper primitive helps the client to update the
 * ingress source validation option for decapsulation. 
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Selection: option selection.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */

ix_error ix_cc_v6v4_async_set_src_validation(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_uint8 arg_Selection,
            ix_cc_v6v4_cb arg_Callback,
            void* arg_pClientContext
         );

/**
 * NAME: ix_cc_v6v4_async_get_end_tunnel
 *
 * DESCRIPTION: This primitive helps a client to retrieve the information 
 * for a given end tunnel endpoint.
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_v6v4_async_get_end_tunnel(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_cc_v6v4_cb_end_tunnel arg_Callback,
            void* arg_pClientContext
         );



/**
 * NAME: ix_cc_v6v4_async_add_allowed_source
 *
 * DESCRIPTION: This primitive helps a client to add a prefix to the ingress
 * source address validation list for a end tunnel endpoint.
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_pEntry: pointer to ip address/prefix length to be added.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_v6v4_async_add_allowed_source(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_cc_v6v4_ingress_source_entry* arg_pEntry,
            ix_cc_v6v4_cb arg_Callback,
            void* arg_pClientContext
         );



/**
 * NAME: ix_cc_v6v4_async_delete_allowed_source
 *
 * DESCRIPTION: This primitive helps a client to delete a prefix from the ingress
 * source address validation list for a end tunnel endpoint.
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_pEntry: pointer to ip address/prefix length to be deleted.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_v6v4_async_delete_allowed_source(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_cc_v6v4_ingress_source_entry* arg_pEntry,
            ix_cc_v6v4_cb arg_Callback,
            void* arg_pClientContext
         );



/**
 * NAME: ix_cc_v6v4_async_clear_allowed_sources
 *
 * DESCRIPTION: This primitive helps a client to delete all prefixes from the
 * ingress source address validation list for a end tunnel endpoint.
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_v6v4_async_clear_allowed_sources(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_cc_v6v4_cb arg_Callback,
            void* arg_pClientContext
         );



/**
 * NAME: ix_cc_v6v4_async_get_allowed_sources
 *
 * DESCRIPTION: This primitive helps a client to retrieve a list of all prefixes
 * in the ingress source address validation list for a end tunnel endpoint.
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_pEntry: pointer to ip address/prefix length to be deleted.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_v6v4_async_get_allowed_sources(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_cc_v6v4_cb_get_sources arg_Callback,
            void* arg_pClientContext
         );



/**
 * NAME: ix_cc_v6v4_async_dump_end_tunnels
 *
 * DESCRIPTION: This primitive helps a client to retrieve a list of all end
 * tunnel endpoints.
 * 
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_v6v4_async_dump_end_tunnels(
            ix_cc_v6v4_cb_dump_end_tunnels arg_Callback,
            void* arg_pClientContext
         );



/**
 * NAME: ix_cc_v6v4_async_add_start_tunnel
 *
 * DESCRIPTION: This message helper primitive helps the client to add a start
 * tunnel endpoint.
 * 
 * @Param:  - IN arg_pConfig: pointer to configuration information about the tunnel.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */


ix_error ix_cc_v6v4_async_add_start_tunnel(
            ix_cc_v6v4_start_tunnel_config* arg_pConfig,
            ix_cc_v6v4_cb_add_tunnel arg_Callback,
            void* arg_pClientContext
         );

/**
 * NAME: ix_cc_v6v4_async_delete_start_tunnel
 *
 * DESCRIPTION:This message helper primitive helps a client to delete a start
 * tunnel endpoint. 
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */


ix_error ix_cc_v6v4_async_delete_start_tunnel(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_cc_v6v4_cb arg_Callback,
            void* arg_pClientContext
         );


/**
 * NAME: ix_cc_v6v4_async_set_ttl
 *
 * DESCRIPTION:This message helper primitive helps a client to set the TTL
 * value for encapsulation. 
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Ttl: TTL value to set.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */


ix_error ix_cc_v6v4_async_set_ttl(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_uint8 arg_Ttl,
            ix_cc_v6v4_cb arg_Callback,
            void* arg_pClientContext
         );


/**
 * NAME: ix_cc_v6v4_async_set_encap_tos_option
 *
 * DESCRIPTION: This message helper primitive helps the client to update the TOS
 * option for encapsulation. 
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Selection: option selection.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */

ix_error ix_cc_v6v4_async_set_encap_tos_option(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_uint8 arg_Selection,
            ix_cc_v6v4_cb arg_Callback,
            void* arg_pClientContext
         );


/**
 * NAME: ix_cc_v6v4_async_set_tos
 *
 * DESCRIPTION: This message helper primitive helps the client to update the TOS
 * value to be used in encapsulation if the tunnel is not configured to encode
 * the TOS byte from the IPv6 header. 
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Tos: TOS value to set.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */

ix_error ix_cc_v6v4_async_set_tos(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_uint8 arg_Tos,
            ix_cc_v6v4_cb arg_Callback,
            void* arg_pClientContext
         );


/**
 * NAME: ix_cc_v6v4_async_set_mtu
 *
 * DESCRIPTION:This message helper primitive helps a client to set the path MTU
 * value for a start tunnel endpoint. 
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Mtu: path MTU value to set.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */


ix_error ix_cc_v6v4_async_set_mtu(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_uint8 arg_Mtu,
            ix_cc_v6v4_cb arg_Callback,
            void* arg_pClientContext
         );


/**
 * NAME: ix_cc_v6v4_async_set_subnet_broadcast
 *
 * DESCRIPTION:This message helper primitive helps a client to set the subnet
 * broadcast address for a start tunnel endpoint. 
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_SubnetBroadcast: subnet broadcast address to set.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */


ix_error ix_cc_v6v4_async_set_subnet_broadcast(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_uint32 arg_SubnetBroadcast,
            ix_cc_v6v4_cb arg_Callback,
            void* arg_pClientContext
         );


/**
 * NAME: ix_cc_v6v4_async_get_start_tunnel
 *
 * DESCRIPTION: This primitive helps a client to retrieve the information 
 * for a given start tunnel endpoint.
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_v6v4_async_get_start_tunnel(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_cc_v6v4_cb_start_tunnel arg_Callback,
            void* arg_pClientContext
         );



/**
 * NAME: ix_cc_v6v4_async_dump_start_tunnels
 *
 * DESCRIPTION: This primitive helps a client to retrieve a list of all start
 * tunnel endpoints.
 * 
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_v6v4_async_dump_start_tunnels(
            ix_cc_v6v4_cb_dump_start_tunnels arg_Callback,
            void* arg_pClientContext
         );



/**
 * NAME: ix_cc_v6v4_async_get_statistics
 *
 * DESCRIPTION: This primitive helps a client to retrieve the tunneling
 * statistics.
 * 
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_v6v4_async_get_statistics(
            ix_cc_v6v4_cb_statistics arg_Callback,
            void* arg_pClientContext
         );




/**
 * NAME: ix_cc_v6v4_init
 *
 * DESCRIPTION: This primitive shall be called and returned successfully before
 *              requesting any services from the tunneling core component.
 *              This primitive should be called only once to initialize the
 *              tunneling core component. This function performs allocation of
 *              memory for symbols to be patched, registration of packet and
 *              message handlers, initialization and configuration of shared
 *              memory used for tunneling and allocation and initialization of
 *              internal data structures.
 * 
 * @Param:  -   IN arg_hCcHandle - handle to tunneling core component, created
 *              by core component infrastructure; this shall be used later to
 *              get other services from core component infrastructure.
 *
 * @Param:  -   IN-OUT arg_ppContext - Upon input, points to a generic core
 *              component initialization context, from which general
 *              information that might be relevant to the core component can
 *              be obtained.  Upon successful completion of this function,
 *              the tunneling core component will update this location with
 *              a pointer to its context information.  This information is
 *              allocated by the tunneling core component and contains
 *              data that it uses internally. Later, when the core component
 *              is being destroyed, this pointer is passed to ix_cc_v6v4_fini()
 *              for freeing memory and performing other housekeeping
 *              operations.  
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - input parameter arg_ppContext is null
 * IX_CC_ERROR_OOM   - memory allocation failure      
 * IX_CC_V6V4_ERROR_FAILED_PATCHING   - patching failures                      
 * IX_CC_V6V4_ERROR_REGISTRY  - registry access failures
 * IX_CC_V6V4_ERROR_CCI - failure from CCI 
 * 
 */
ix_error ix_cc_v6v4_init(
            ix_cc_handle arg_hCcHandle,
            void** arg_ppContext
         );



/**
 * NAME: ix_cc_v6v4_fini
 *
 * DESCRIPTION: This primitive shall be called to terminate services from the
 * Tunneling component. This primitive frees memory allocated during
 * initialization and frees all the created resources. If client requests any
 * services from Tunneling after calling this function, then the behavior is
 * undefined.
 * 
 * @Param:  - IN arg_hCcHandle - handle to the Tunneling core component
 *
 * @Param:  - IN arg_pContext - pointer to the context information allocated 
 * earlier in ix_cc_v6v4_init(). 
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_INVALID_INPUT_PARAM - invalid input parameter(s)
 * IX_CC_ERROR_OOM   - failure of memory deletion                
 * IX_CC_V6V4_ERROR_CCI - failure from CCI
 * 
 */
ix_error ix_cc_v6v4_fini(
            ix_cc_handle arg_hCcHandle,
            void* arg_pContext
         );


/**
 * NAME: ix_cc_v6v4_msg_handler
 *
 * DESCRIPTION: This primitive is the message handler function for the
 * Tunneling core component. The Tunneling core component receives messages
 * from other core components through this message handler function and
 * internally it calls appropriate library function to process the message.
 * This message handler shall be used to update dynamic properties.
 * 
 * @Param: - IN arg_hMsgInfo - buffer handle embedding information for the message
 * passed in arg_UserData.
 *
 * @Param: - IN arg_MsgType - message type
 *
 * @Param: - IN arg_pContext: pointer to Tunneling core component context that 
 * was defined by the core component and passed to the core component
 * infrastructure through ix_cc_v6v4_init().
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - null argument
 * IX_CC_ERROR_UNDEFINED_MSG - unsupported message
 * IX_CC_V6V4_ERROR_MSG_LIBRARY - error from message support library
 * IX_CC_V6V4_ERROR_BUFFER_FREE - error from RM for freeing buffer
 * 
 */
ix_error ix_cc_v6v4_msg_handler(
            ix_buffer_handle arg_hMsgInfo,
            ix_uint32 arg_MsgType,
            void* arg_pContext
         );



/**
 * NAME: ix_cc_v6v4_microblock_pkt_handler
 *
 * DESCRIPTION: This is the registered function to receive exception packets from
 * the tunneling microblocks. 
 * 
 * @Param:  - IN arg_hPacket: handle to a buffer which contains exception packets
 * from tunneling microblock
 * @Param: - IN arg_ExceptionCode: exception code set by microblock.
 * @Param: - IN arg_pContext: pointer to Tunneling core component context that 
 * was defined by the core component and passed to the core component
 * infrastructure through ix_cc_v6v4_init().
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_CCI - error from CCI
 * 
 */

ix_error ix_cc_v6v4_microblock_pkt_handler(
            ix_buffer_handle arg_hPacket,  
            ix_uint32 arg_ExceptionCode,
            void* arg_pContext
         );




/**
 * NAME: ix_cc_v6v4_ipv6_pkt_handler
 *
 * DESCRIPTION: This is the registered function to receive IPv6 packets from
 * other core components. 
 * 
 * @Param:  - IN arg_hPacket: handle to a buffer which contains IPv6 packet.
 * @Param: - IN arg_ExceptionCode: not used.
 * @Param: - IN arg_pContext: pointer to Tunneling core component context that 
 * was defined by the core component and passed to the core component
 * infrastructure through ix_cc_v6v4_init().
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_CCI - error from CCI
 * 
 */

ix_error ix_cc_v6v4_ipv6_pkt_handler(
            ix_buffer_handle arg_hPacket,  
            ix_uint32 arg_ExceptionCode,
            void* arg_pContext
         );


/**
 * NAME: ix_cc_v6v4_ipv4_pkt_handler
 *
 * DESCRIPTION: This is the registered function to receive IPv4 packets from
 * other core components. 
 * 
 * @Param:  - IN arg_hPacket: handle to a buffer which contains IPv4 packet.
 * @Param: - IN arg_ExceptionCode: not used.
 * @Param: - IN arg_pContext: pointer to Tunneling core component context that 
 * was defined by the core component and passed to the core component
 * infrastructure through ix_cc_v6v4_init().
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_CCI - error from CCI
 * 
 */

ix_error ix_cc_v6v4_ipv4_pkt_handler(
            ix_buffer_handle arg_hPacket,  
            ix_uint32 arg_ExceptionCode,
            void* arg_pContext
         );


/**
 * NAME: ix_cc_v6v4_add_end_tunnel
 *
 * DESCRIPTION:This library primitive adds a new end tunnel endpoint. 
 * 
 * @Param:  - IN arg_pConfig: pointer to configuration information.
 * @Param:  - IN arg_pContext: pointer to tunneling context
 *
 * @Param:  - OUT arg_phTunnel: pointer to location in which to return
 * tunneling component's handle to tunnel.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_FULL - out of resources
 * IX_CC_ERROR_V6V4_INVALID_INPUT_PARAM - input parameter is invalid
 * 
 */

ix_error ix_cc_v6v4_add_end_tunnel(
            ix_cc_v6v4_end_tunnel_config* arg_pConfig,
            void* arg_pContext,
            ix_cc_v6v4_tunnel_handle* arg_phTunnel
         );



/**
 * NAME: ix_cc_v6v4_delete_end_tunnel
 *
 * DESCRIPTION:This library primitive deletes an end tunnel endpoint. 
 * 
 * @Param:  - IN arg_hTunnel: Handle to end tunnel.
 * @Param:  - IN arg_pContext: pointer to tunneling context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_V6V4_INVALID_INPUT_PARAM - input parameter is invalid
 * 
 */

ix_error ix_cc_v6v4_delete_end_tunnel(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            void* arg_pContext
         );



/**
 * NAME: ix_cc_v6v4_set_decap_tos_option
 *
 * DESCRIPTION:This library primitive sets the TOS option for decapsulation.
 * 
 * @Param:  - IN arg_hTunnel: Handle to end tunnel.
 * @Param:  - IN arg_Selection: IX_CC_V6V4_SELECT to select the option or
 * IX_CC_V6V4_CLEAR to deselect the option.
 * @Param:  - IN arg_pContext: pointer to tunneling context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_V6V4_INVALID_INPUT_PARAM - input parameter is invalid
 * 
 */

ix_error ix_cc_v6v4_set_decap_tos_option(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_uint8 arg_Selection,
            void* arg_pContext
         );



/**
 * NAME: ix_cc_v6v4_set_src_validation
 *
 * DESCRIPTION:This library primitive sets the ingress source validation
 * option for decapsulation.
 * 
 * @Param:  - IN arg_hTunnel: Handle to end tunnel.
 * @Param:  - IN arg_Selection: IX_CC_V6V4_SELECT to select the option or
 * IX_CC_V6V4_CLEAR to deselect the option.
 * @Param:  - IN arg_pContext: pointer to tunneling context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_V6V4_INVALID_INPUT_PARAM - input parameter is invalid
 * 
 */

ix_error ix_cc_v6v4_set_src_validation(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_uint8 arg_Selection,
            void* arg_pContext
         );



/**
 * NAME: ix_cc_v6v4_get_end_tunnel
 *
 * DESCRIPTION:This library primitive returns the configuration information
 * for a specific end tunnel endpoint.
 * 
 * @Param:  - IN arg_pContext: pointer to tunneling context
 *
 * @Param:  - IN/OUT arg_pTunnelInfo: Pointer to buffer to contain tunnel
 * information.  As input, the client initializes arg_pTunnelInfo->hTunnel
 * with the handle to the tunnel it is interested in.  As output, the core
 * component supplies the remaining information in the structure.
 * 
 * @Param:  - OUT - none.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_V6V4_INVALID_INPUT_PARAM - input parameter is invalid
 * 
 */

ix_error ix_cc_v6v4_get_end_tunnel(
            ix_cc_v6v4_end_tunnel_info* arg_pTunnelInfo,
            void* arg_pContext
         );



/**
 * NAME: ix_cc_v6v4_add_allowed_source
 *
 * DESCRIPTION:This library primitive adds an entry to the ingress source
 * validation list for an end tunnel endpoint.
 * 
 * @Param:  - IN arg_hTunnel: Handle to end tunnel.
 * @Param:  - IN arg_pEntry: pointer to buffer containing the address and
 * prefix length to add to the ingress list.
 * @Param:  - IN arg_pContext: pointer to tunneling context
 *
 * @Param:  - OUT - none.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_OOR - out of resources
 * IX_CC_ERROR_V6V4_INVALID_INPUT_PARAM - input parameter is invalid
 * 
 */

ix_error ix_cc_v6v4_add_allowed_source(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_cc_v6v4_ingress_source_entry* arg_pEntry,
            void* arg_pContext
         );



/**
 * NAME: ix_cc_v6v4_delete_allowed_source
 *
 * DESCRIPTION:This library primitive deletes an entry from the ingress source
 * validation list for an end tunnel endpoint.
 * 
 * @Param:  - IN arg_hTunnel: Handle to end tunnel.
 * @Param:  - IN arg_pEntry: pointer to buffer containing the address and
 * prefix length to delete from the ingress list.
 * @Param:  - IN arg_pContext: pointer to tunneling context
 *
 * @Param:  - OUT - none.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_V6V4_INVALID_INPUT_PARAM - input parameter is invalid
 * 
 */

ix_error ix_cc_v6v4_delete_allowed_source(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_cc_v6v4_ingress_source_entry* arg_pEntry,
            void* arg_pContext
         );



/**
 * NAME: ix_cc_v6v4_clear_allowed_sources
 *
 * DESCRIPTION:This library primitive deletes all entries from the ingress
 * source validation list for an end tunnel endpoint.
 * 
 * @Param:  - IN arg_hTunnel: Handle to end tunnel.
 * @Param:  - IN arg_pContext: pointer to tunneling context
 *
 * @Param:  - OUT - none.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_V6V4_INVALID_INPUT_PARAM - input parameter is invalid
 * 
 */

ix_error ix_cc_v6v4_clear_allowed_sources(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            void* arg_pContext
         );



/**
 * NAME: ix_cc_v6v4_get_allowed_sources
 *
 * DESCRIPTION:This library primitive returns a list of all entries in the
 * ingress source list for an end tunnel endpoint.
 * 
 * @Param:  - IN arg_hTunnel: Handle to end tunnel.
 * @Param:  - IN arg_pContext: pointer to tunneling context
 *
 * @Param:  - IN/OUT arg_pNumEntries: On input, points to the maximum number
 * of entries that can be written to the output buffer.  On output, points to
 * the number of actual entries in the ingress source list for the tunnel.
 *
 * @Param:  - OUT arg_paEntries: pointer to buffer into which to return the
 * array of ingress source list entries.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_V6V4_BUFFER_TOO_SMALL - buffer too small
 * IX_CC_ERROR_V6V4_INVALID_INPUT_PARAM - input parameter is invalid
 * 
 */

ix_error ix_cc_v6v4_get_allowed_sources(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_uint32* arg_pNumEntries,
            ix_cc_v6v4_ingress_source_entry* arg_paEntries,
            void* arg_pContext
         );



/**
 * NAME: ix_cc_v6v4_dump_end_tunnels
 *
 * DESCRIPTION:This library primitive returns a list of all end tunnel
 * endpoints.
 * 
 * @Param:  - IN arg_pContext: pointer to tunneling context
 *
 * @Param:  - IN/OUT arg_pNumEntries: On input, points to the maximum number
 * of entries that can be written to the output buffer.  On output, points to
 * the number of actual end tunnel endpoints.
 *
 * @Param:  - OUT arg_paInfo: pointer to buffer into which to return the array
 * of end tunnel endpoints.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_V6V4_BUFFER_TOO_SMALL - buffer too small
 * 
 */

ix_error ix_cc_v6v4_dump_end_tunnels(
            ix_uint32* arg_pNumEntries,
            ix_cc_v6v4_end_tunnel_info * arg_paInfo,
            void* arg_pContext
         );



/**
 * NAME: ix_cc_v6v4_add_start_tunnel
 *
 * DESCRIPTION:This library primitive adds a new start tunnel endpoint. 
 * 
 * @Param:  - IN arg_pConfig: pointer to configuration information.
 * @Param:  - IN arg_pContext: pointer to tunneling context
 *
 * @Param:  - OUT arg_phTunnel: pointer to location in which to return
 * tunneling component's handle to tunnel.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_OOR - out of resources
 * IX_CC_ERROR_V6V4_INVALID_INPUT_PARAM - input parameter is invalid
 * 
 */

ix_error ix_cc_v6v4_add_start_tunnel(
            ix_cc_v6v4_start_tunnel_config* arg_pConfig,
            void* arg_pContext,
            ix_cc_v6v4_tunnel_handle* arg_phTunnel
         );



/**
 * NAME: ix_cc_v6v4_delete_start_tunnel
 *
 * DESCRIPTION:This library primitive deletes a start tunnel endpoint. 
 * 
 * @Param:  - IN arg_hTunnel: Handle to start tunnel.
 * @Param:  - IN arg_pContext: pointer to tunneling context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_V6V4_INVALID_INPUT_PARAM - input parameter is invalid
 * 
 */

ix_error ix_cc_v6v4_delete_start_tunnel(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            void* arg_pContext
         );



/**
 * NAME: ix_cc_v6v4_set_ttl
 *
 * DESCRIPTION:This library primitive sets the TTL value to be used for
 * encapsulation on a given tunnel.
 * 
 * @Param:  - IN arg_hTunnel: Handle to start tunnel.
 * @Param:  - IN arg_Ttl: Time-to-live value.
 * @Param:  - IN arg_pContext: pointer to tunneling context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_V6V4_INVALID_INPUT_PARAM - input parameter is invalid
 * 
 */

ix_error ix_cc_v6v4_set_ttl(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_uint8 arg_Ttl,
            void* arg_pContext
         );



/**
 * NAME: ix_cc_v6v4_set_encap_tos_option
 *
 * DESCRIPTION:This library primitive specifies the manner in which the
 * TOS byte is encoded during encapsulation for a given tunnel.
 * 
 * @Param:  - IN arg_hTunnel: Handle to start tunnel.
 * @Param:  - IN arg_Selection: value indicating which behavior is to be used.
 * @Param:  - IN arg_pContext: pointer to tunneling context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_V6V4_INVALID_INPUT_PARAM - input parameter is invalid
 * 
 */

ix_error ix_cc_v6v4_set_encap_tos_option(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_uint8 arg_Selection,
            void* arg_pContext
         );



/**
 * NAME: ix_cc_v6v4_set_tos
 *
 * DESCRIPTION:This library primitive specifies the value to use to encode
 * the TOS byte during encapsulation, if the tunnel is not configured to 
 * encode the TOS byte from the IPv6 header.
 * 
 * @Param:  - IN arg_hTunnel: Handle to start tunnel.
 * @Param:  - IN arg_Tos: TOS value.
 * @Param:  - IN arg_pContext: pointer to tunneling context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_V6V4_INVALID_INPUT_PARAM - input parameter is invalid
 * 
 */

ix_error ix_cc_v6v4_set_tos(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_uint8 arg_Tos,
            void* arg_pContext
         );



/**
 * NAME: ix_cc_v6v4_set_mtu
 *
 * DESCRIPTION:This library primitive sets the path MTU value for a given
 * tunnel.
 * 
 * @Param:  - IN arg_hTunnel: Handle to start tunnel.
 * @Param:  - IN arg_Mtu: Path MTU value.
 * @Param:  - IN arg_pContext: pointer to tunneling context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_V6V4_INVALID_INPUT_PARAM - input parameter is invalid
 * 
 */

ix_error ix_cc_v6v4_set_mtu(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_uint16 arg_Mtu,
            void* arg_pContext
         );



/**
 * NAME: ix_cc_v6v4_set_subnet_broadcast
 *
 * DESCRIPTION:This library primitive sets the subnet broadcast address
 * for a given tunnel.
 * 
 * @Param:  - IN arg_hTunnel: Handle to start tunnel.
 * @Param:  - IN arg_SubnetBroadcast: Subnet broadcast address.
 * @Param:  - IN arg_pContext: pointer to tunneling context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_V6V4_INVALID_INPUT_PARAM - input parameter is invalid
 * 
 */

ix_error ix_cc_v6v4_set_subnet_broadcast(
            ix_cc_v6v4_tunnel_handle arg_hTunnel,
            ix_uint32 arg_SubnetBroadcast,
            void* arg_pContext
         );



/**
 * NAME: ix_cc_v6v4_get_start_tunnel
 *
 * DESCRIPTION:This library primitive returns the configuration information
 * for a specific start tunnel endpoint.
 * 
 * @Param:  - IN arg_pContext: pointer to tunneling context
 *
 * @Param:  - IN/OUT arg_pTunnelInfo: Pointer to buffer to contain tunnel
 * information.  As input, the client initializes arg_pTunnelInfo->hTunnel
 * with the handle to the tunnel it is interested in.  As output, the core
 * component supplies the remaining information in the structure.
 * 
 * @Param:  - OUT - none.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_V6V4_INVALID_INPUT_PARAM - input parameter is invalid
 * 
 */

ix_error ix_cc_v6v4_get_start_tunnel(
            ix_cc_v6v4_start_tunnel_info* arg_pTunnelInfo,
            void* arg_pContext
         );



/**
 * NAME: ix_cc_v6v4_dump_start_tunnels
 *
 * DESCRIPTION:This library primitive returns a list of all start tunnel
 * endpoints.
 * 
 * @Param:  - IN arg_pContext: pointer to tunneling context
 *
 * @Param:  - IN/OUT arg_pNumEntries: On input, points to the maximum number
 * of entries that can be written to the output buffer.  On output, points to
 * the number of actual start tunnel endpoints.
 *
 * @Param:  - OUT arg_paInfo: pointer to buffer into which to return the array
 * of tart tunnel endpoints.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_V6V4_BUFFER_TOO_SMALL - buffer too small
 * 
 */

ix_error ix_cc_v6v4_dump_start_tunnels(
            ix_uint32* arg_pNumEntries,
            ix_cc_v6v4_start_tunnel_info * arg_paInfo,
            void* arg_pContext
         );


/**
 * NAME: ix_cc_v6v4_get_statistics
 *
 * DESCRIPTION:This library primitive returns the tunneling statistics.
 * 
 * @Param:  - IN arg_pContext: pointer to tunneling context
 *
 * @Param:  - OUT arg_pStats: pointer to buffer into which to return
 * the statistics.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_V6V4_INVALID_INPUT_PARAM - input parameter is invalid
 * 
 */

ix_error ix_cc_v6v4_get_statistics(
            ix_cc_v6v4_statistics * arg_pStats,
            void* arg_pContext
         );




/**
 * NAME: ix_cc_v6v4_set_property
 *
 * DESCRIPTION: This library primitive helps to set dynamic properties of 
 * the tunneling core component. 
 * 
 * @Param:  - IN arg_PropId: Identifier of a property or properties(ORed)
 * to be updated for a port
 * @Param:  - IN arg_pPropData: pointer to dynamic property data structure
 * @Param:  - IN arg_pContext: pointer to tunneling context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * IX_CC_ERROR_NUL - invalid pointer
 * IX_CC_IPV4_ERROR_INVALID_INPUT_PARAM - invalid input param
 * IX_ERROR_FULL - table is full
 * IX_CC_ERROR_ENTRY_NOT_FOUND - entry not found in table
 *
 */
ix_error ix_cc_v6v4_set_property(
            ix_uint32 arg_PropId,
            ix_cc_properties* arg_pPropData, 
            void* arg_pContext
         );



#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_V6V4_H__) */
