/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor, Release 6
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      Core Component Infrastructure - Shared Memory Configuration 
 *      definitions
 *
 * = FILENAME
 *      ix_cc_v6v4_config.h
 *
 * = DESCRIPTION
 *      This file defines all data types, macros and function prototypes
 *      that support shared memory configuration for tunneling.
 *
 * = CREATION TIME
 *      11/19/2002 9:07 AM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: ix_cc_v6v4_config.h,v 1.5 2003/04/28 16:35:23 ktseng Exp $
 */

#if !defined(__IX_CC_V6V4_CONFIG_H__)
#define __IX_CC_V6V4_CONFIG_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */


/**
 * Validity flag definition.
 */
#define IX_CC_V6V4_FLAG_TUNNEL_VALID    0x8000

/**
 * Internal flags defined for start tunnel entry that are not exposed in the
 * API.
 */
#define IX_CC_V6V4_ENCAP_NO_DF          0x0008


/**
 * TYPENAME: ix_cc_v6v4_end_tunnel
 * 
 * DESCRIPTION: This structure defines the format for an entry in the end
 * tunnel table in shared memory.
 *
 */
typedef struct ix_s_cc_v6v4_end_tunnel
{
    ix_uint16 flags;
    ix_uint16 ingressList;
} ix_cc_v6v4_end_tunnel;


/**
 * TYPENAME: ix_cc_v6v4_start_tunnel
 * 
 * DESCRIPTION: This structure defines the format for an entry in the end
 * tunnel table in shared memory.
 *
 */
typedef struct ix_s_cc_v6v4_start_tunnel
{
    ix_uint16 flags;
    ix_uint16 pathMtu;
    ix_uint32 localAddress;
    ix_uint32 remoteAddress;
    ix_uint8  ttl;
    ix_uint8  tos;
    ix_uint16 reserved1;
    ix_uint32 subnetBroadcast;
    ix_uint32 reserved2[3];
} ix_cc_v6v4_start_tunnel;


/* Bits 28:31 of a tunnel handle indicate the type of tunnel endpoint */

#define IX_CC_V6V4_END_TUNNEL_HANDLE        0xE
#define IX_CC_V6V4_START_TUNNEL_HANDLE      0xC


/**
 * NAME: IX_CC_V6V4_MAKE_END_TUNNEL_HANDLE
 *
 * DESCRIPTION: This macro creates a handle to an end tunnel entry.
 *           .
 * @Param:  - IN __i - index of tunnel entry.
 *
 */

#define IX_CC_V6V4_MAKE_END_TUNNEL_HANDLE(__i) \
    ((__i) | (IX_CC_V6V4_END_TUNNEL_HANDLE << 28))

/**
 * NAME: IX_CC_V6V4_MAKE_START_TUNNEL_HANDLE
 *
 * DESCRIPTION: This macro creates a handle to an start tunnel entry.
 *           .
 * @Param:  - IN __i - index of tunnel entry.
 *
 */

#define IX_CC_V6V4_MAKE_START_TUNNEL_HANDLE(__i) \
    ((__i) | (IX_CC_V6V4_START_TUNNEL_HANDLE << 28))


/**
 * NAME: IX_CC_V6V4_HANDLE_TYPE_FROM_HANDLE
 *
 * DESCRIPTION: This macro extracts the handle type from a handle.
 *           .
 * @Param:  - IN __h - tunnel handle.
 *
 */

#define IX_CC_V6V4_HANDLE_TYPE_FROM_HANDLE(__h) ((__h) >> 28)


/**
 * NAME: IX_CC_V6V4_INDEX_FROM_HANDLE
 *
 * DESCRIPTION: This macro extracts the index from a handle.
 *           .
 * @Param:  - IN __h - tunnel handle.
 *
 */

#define IX_CC_V6V4_INDEX_FROM_HANDLE(__h) ((ix_uint16)((__h) & 0xffff))



/**
 * NAME: IX_CC_V6V4_END_TUNNEL_HANDLE_VALID
 *
 * DESCRIPTION: This macro validates an end tunnel handle.
 *           .
 * @Param:  - IN __h - tunnel handle.
 *
 */

#define IX_CC_V6V4_END_TUNNEL_HANDLE_VALID(__h) \
    (IX_CC_V6V4_HANDLE_TYPE_FROM_HANDLE(__h) == IX_CC_V6V4_END_TUNNEL_HANDLE)

/**
 * NAME: IX_CC_V6V4_START_TUNNEL_HANDLE_VALID
 *
 * DESCRIPTION: This macro validates an end tunnel handle.
 *           .
 * @Param:  - IN __h - tunnel handle.
 *
 */

#define IX_CC_V6V4_START_TUNNEL_HANDLE_VALID(__h) \
    (IX_CC_V6V4_HANDLE_TYPE_FROM_HANDLE(__h) == IX_CC_V6V4_START_TUNNEL_HANDLE)

/**
 * NAME: IX_CC_V6V4_END_TUNNEL_FLAGS_VALID
 *
 * DESCRIPTION: This macro validates a set of flags for an end tunnel
 * endpoint.
 *           .
 * @Param:  - IN __f - flags to be validated.
 *
 */

#define IX_CC_V6V4_END_TUNNEL_FLAGS_ALL \
    (IX_CC_V6V4_DECAP_AUTO | \
     IX_CC_V6V4_SRC_VALIDATE | \
     IX_CC_V6V4_DECAP_6TO4 | \
     IX_CC_V6V4_DECAP_TOS_V6)

#define IX_CC_V6V4_END_TUNNEL_FLAGS_VALID(__f) \
(((__f) & IX_CC_V6V4_END_TUNNEL_FLAGS_ALL) == (__f))


/**
 * NAME: IX_CC_V6V4_START_TUNNEL_FLAGS_VALID
 *
 * DESCRIPTION: This macro validates a set of flags for an start tunnel
 * endpoint.
 *           .
 * @Param:  - IN __f - flags to be validated.
 *
 */

#define IX_CC_V6V4_START_TUNNEL_FLAGS_ALL \
    (IX_CC_V6V4_ENCAP_AUTO | \
     IX_CC_V6V4_ENCAP_6TO4 | \
     IX_CC_V6V4_ENCAP_TOS_V6)

#define IX_CC_V6V4_START_TUNNEL_FLAGS_VALID(__f) \
(((__f) & IX_CC_V6V4_START_TUNNEL_FLAGS_ALL) == (__f))


/*
 * NAME: _READ_END_TUNNEL()
 *
 * DESCRIPTION: Reads an end tunnel endpoint from shared memory and
 * places it into local memory.
 *
 * @Param:      - IN    __base
 *                      Base address of end tunnel table.
 *
 * @Param:      - IN    __index
 *                      Index of tunnel entry to be read.
 *
 * @Param:      - OUT   __local
 *                      Local structure where the data will be placed.
 *                      
 */
#define _READ_END_TUNNEL(__base, __index, __local)                      \
    (__local).flags = IX_RM_MEM_UINT16_READ(                   		\
        &(((ix_cc_v6v4_end_tunnel*)(__base))[__index].flags));		\
    (__local).ingressList = IX_RM_MEM_UINT16_READ(                      \
        &(((ix_cc_v6v4_end_tunnel*)(__base))[__index].ingressList));


/*
 * NAME: _WRITE_END_TUNNEL()
 *
 * DESCRIPTION: Writes a local copy of an end tunnel endpoint to
 * shared memory.
 *
 * @Param:      - IN    __base
 *                      Base address of end tunnel table.
 *
 * @Param:      - IN    __index
 *                      Index of tunnel entry to be written.
 *
 * @Param:      - IN    __local
 *                      Local structure to be copied into shared memory.
 */
#define _WRITE_END_TUNNEL(__base, __index, __local)                     \
    (void) IX_RM_MEM_UINT32_WRITE(                                      \
        (ix_uint32*) &(((ix_cc_v6v4_end_tunnel*)(__base))[__index]),    \
        *(ix_uint32*) &(__local))

/*
 * NAME: _CLEAR_END_TUNNEL()
 *
 * DESCRIPTION: Clears an end tunnel endpoint in shared memory.
 *
 * @Param:      - IN    __base
 *                      Base address of end tunnel table.
 *
 * @Param:      - IN    __index
 *                      Index of tunnel entry to be written.
 */
#define _CLEAR_END_TUNNEL(__base, __index)                              \
    (void) IX_RM_MEM_UINT32_WRITE(                                      \
        (ix_uint32*) &(((ix_cc_v6v4_end_tunnel*)(__base))[__index]),    \
        0)

/*
 * NAME: _WRITE_DECAP_FLAGS()
 *
 * DESCRIPTION: Writes the flags field to an end tunnel endpoint
 * in shared memory.  Note that only 16 bits are written.
 *
 * @Param:      - IN    __base
 *                      Base address of end tunnel table.
 *
 * @Param:      - IN    __index
 *                      Index of tunnel entry to be written.
 *
 * @Param:      - IN    __flags
 *                      Flags value to be copied into shared memory.
 */
#define _WRITE_DECAP_FLAGS(__base, __index, __flags)                    \
    (void) IX_RM_MEM_UINT16_WRITE(                                      \
        &(((ix_cc_v6v4_end_tunnel*)(__base))[__index].flags), __flags)


/*
 * NAME: _WRITE_DECAP_INGRESS_LIST()
 *
 * DESCRIPTION: Writes the ingress list field to an end tunnel endpoint
 * in shared memory.  Note that only 16 bits are written.
 *
 * @Param:      - IN    __base
 *                      Base address of end tunnel table.
 *
 * @Param:      - IN    __index
 *                      Index of tunnel entry to be written.
 *
 * @Param:      - IN    __isl
 *                      Ingress list value to be copied into shared memory.
 */
#define _WRITE_DECAP_INGRESS_LIST(__base, __index, __isl)                   \
    (void) IX_RM_MEM_UINT16_WRITE(                                          \
        &(((ix_cc_v6v4_end_tunnel*)(__base))[__index].ingressList), __isl)



/*
 * NAME: _READ_START_TUNNEL()
 *
 * DESCRIPTION: Reads a start tunnel endpoint from shared memory and
 * places it into local memory.
 *
 * @Param:      - IN    __base
 *                      Base address of start tunnel table.
 *
 * @Param:      - IN    __index
 *                      Index of tunnel entry to be read.
 *
 * @Param:      - OUT   __local
 *                      Local structure where the data will be placed.
 *                      
 */
#define _READ_START_TUNNEL(__base, __index, __local)                        \
    *(ix_uint64*) &(__local) = IX_RM_MEM_UINT64_READ(                       \
        (ix_uint64*) &(((ix_cc_v6v4_start_tunnel*)(__base))[__index]));     \
    *(((ix_uint64*) &(__local))+1) = IX_RM_MEM_UINT64_READ(                 \
        (ix_uint64*) &(((ix_cc_v6v4_start_tunnel*)(__base))[__index])+1);  \
    *(((ix_uint64*) &(__local))+2) = IX_RM_MEM_UINT64_READ(                 \
        (ix_uint64*) &(((ix_cc_v6v4_start_tunnel*)(__base))[__index])+2);  \
    *(((ix_uint64*) &(__local))+3) = IX_RM_MEM_UINT64_READ(                 \
        (ix_uint64*) &(((ix_cc_v6v4_start_tunnel*)(__base))[__index])+3);


/*
 * NAME: _WRITE_START_TUNNEL()
 *
 * DESCRIPTION: Writes a local copy of a start tunnel endpoint to
 * shared memory.
 *
 * @Param:      - IN    __base
 *                      Base address of start tunnel table.
 *
 * @Param:      - IN    __index
 *                      Index of tunnel entry to be written.
 *
 * @Param:      - IN    __local
 *                      Local structure to be copied into shared memory.
 */
#define _WRITE_START_TUNNEL(__base, __index, __local)                       \
    (void) IX_RM_MEM_UINT64_WRITE(                                          \
        (ix_uint64*) &(((ix_cc_v6v4_start_tunnel*)(__base))[__index]),      \
        *(ix_uint64*) &(__local));                                          \
    (void) IX_RM_MEM_UINT64_WRITE(                                          \
        (ix_uint64*) &(((ix_cc_v6v4_start_tunnel*)(__base))[__index]) + 1,  \
        *((ix_uint64*) &(__local)+1));                                      \
    (void) IX_RM_MEM_UINT64_WRITE(                                          \
        (ix_uint64*) &(((ix_cc_v6v4_start_tunnel*)(__base))[__index]) + 2,  \
        *((ix_uint64*) &(__local)+2));                                      \
    (void) IX_RM_MEM_UINT64_WRITE(                                          \
        (ix_uint64*) &(((ix_cc_v6v4_start_tunnel*)(__base))[__index]) + 3,  \
        *((ix_uint64*) &(__local)+3));

/*
 * NAME: _CLEAR_START_TUNNEL()
 *
 * DESCRIPTION: Clears a start tunnel endpoint in shared memory.
 *
 * @Param:      - IN    __base
 *                      Base address of start tunnel table.
 *
 * @Param:      - IN    __index
 *                      Index of tunnel entry to be written.
 */
#define _CLEAR_START_TUNNEL(__base, __index)                                \
    (void) IX_RM_MEM_UINT64_WRITE(                                          \
        (ix_uint64*) &(((ix_cc_v6v4_start_tunnel*)(__base))[__index]),      \
        0);                                                                 \
    (void) IX_RM_MEM_UINT64_WRITE(                                          \
        (ix_uint64*) &(((ix_cc_v6v4_start_tunnel*)(__base))[__index]) + 1,  \
        0);                                                                 \
    (void) IX_RM_MEM_UINT64_WRITE(                                          \
        (ix_uint64*) &(((ix_cc_v6v4_start_tunnel*)(__base))[__index]) + 2,  \
        0);                                                                 \
    (void) IX_RM_MEM_UINT64_WRITE(                                          \
        (ix_uint64*) &(((ix_cc_v6v4_start_tunnel*)(__base))[__index]) + 3,  \
        0);

/*
 * NAME: _WRITE_ENCAP_FLAGS()
 *
 * DESCRIPTION: Writes the flags field to a start tunnel endpoint
 * in shared memory.  Note that only 16 bits are written.
 *
 * @Param:      - IN    __base
 *                      Base address of tart tunnel table.
 *
 * @Param:      - IN    __index
 *                      Index of tunnel entry to be written.
 *
 * @Param:      - IN    __flags
 *                      Flags value to be copied into shared memory.
 */
#define _WRITE_ENCAP_FLAGS(__base, __index, __flags)                    \
    (void) IX_RM_MEM_UINT16_WRITE(                                      \
        &(((ix_cc_v6v4_start_tunnel*)(__base))[__index].flags), __flags)

/*
 * NAME: _WRITE_ENCAP_TTL()
 *
 * DESCRIPTION: Writes the time-to-live field to a start tunnel
 * endpoint in shared memory.  Note that only 8 bits are written.
 *
 * @Param:      - IN    __base
 *                      Base address of start tunnel table.
 *
 * @Param:      - IN    __index
 *                      Index of tunnel entry to be written.
 *
 * @Param:      - IN    __ttl
 *                      TTL value to be written.
 */
#define _WRITE_ENCAP_TTL(__base, __index, __ttl)                        \
    (void) IX_RM_MEM_UINT8_WRITE(                                       \
        &(((ix_cc_v6v4_start_tunnel*)(__base))[__index].ttl), __ttl)

/*
 * NAME: _WRITE_ENCAP_TOS()
 *
 * DESCRIPTION: Writes the type of service field to a start tunnel
 * endpoint in shared memory.  Note that only 8 bits are written.
 *
 * @Param:      - IN    __base
 *                      Base address of start tunnel table.
 *
 * @Param:      - IN    __index
 *                      Index of tunnel entry to be written.
 *
 * @Param:      - IN    __tos
 *                      TOS value to be written.
 */
#define _WRITE_ENCAP_TOS(__base, __index, __tos)                        \
    (void) IX_RM_MEM_UINT8_WRITE(                                       \
        &(((ix_cc_v6v4_start_tunnel*)(__base))[__index].tos), __tos)

/*
 * NAME: _WRITE_ENCAP_MTU()
 *
 * DESCRIPTION: Writes the path MTU field to a start tunnel
 * endpoint in shared memory.  Note that only 16 bits are written.
 *
 * @Param:      - IN    __base
 *                      Base address of start tunnel table.
 *
 * @Param:      - IN    __index
 *                      Index of tunnel entry to be written.
 *
 * @Param:      - IN    __mtu
 *                      MTU value to be written.
 */
#define _WRITE_ENCAP_MTU(__base, __index, __mtu)                        \
    (void) IX_RM_MEM_UINT16_WRITE(                                      \
        &(((ix_cc_v6v4_start_tunnel*)(__base))[__index].pathMtu), __mtu)

/*
 * NAME: _READ_ENCAP_MTU()
 *
 * DESCRIPTION: Reads the path MTU field in a start tunnel
 * endpoint in shared memory.  Note that only 16 bits are read.
 *
 * @Param:      - IN    __base
 *                      Base address of start tunnel table.
 *
 * @Param:      - IN    __index
 *                      Index of tunnel entry to be written.
 *
 */
#define _READ_ENCAP_MTU(__base, __index)                                \
    IX_RM_MEM_UINT16_READ(                                              \
        &(((ix_cc_v6v4_start_tunnel*)(__base))[__index].pathMtu))

/*
 * NAME: _WRITE_ENCAP_SUBNET_BROADCAST()
 *
 * DESCRIPTION: Writes the subnet broadcast field to a start tunnel
 * endpoint in shared memory.
 *
 * @Param:      - IN    __base
 *                      Base address of start tunnel table.
 *
 * @Param:      - IN    __index
 *                      Index of tunnel entry to be written.
 *
 * @Param:      - IN    __adr
 *                      Subnet broadcast address to be written.
 */
#define _WRITE_ENCAP_SUBNET_BROADCAST(__base, __index, __adr)           \
    (void) IX_RM_MEM_UINT32_WRITE(                                      \
      &(((ix_cc_v6v4_start_tunnel*)(__base))[__index].subnetBroadcast), \
      __adr)


/*
 * NAME: _WRITE_ENCAP_LOCAL_ADDR()
 *
 * DESCRIPTION: Writes the local ipv4 address field to a start tunnel
 * endpoint in shared memory.
 *
 * @Param:      - IN    __base
 *                      Base address of start tunnel table.
 *
 * @Param:      - IN    __index
 *                      Index of tunnel entry to be written.
 *
 * @Param:      - IN    __adr
 *                      IPv4 address to be written.
 */ 
#define _WRITE_ENCAP_LOCAL_ADDR(__base, __index, __adr)                 \
    (void) IX_RM_MEM_UINT32_WRITE(                                      \
      &(((ix_cc_v6v4_start_tunnel*)(__base))[__index].localAddress),    \
      __adr)



#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_V6V4_CONFIG_H__) */
