/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor, Release 6
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      Core Component Infrastructure - Definitions for decapsulation.
 *
 * = FILENAME
 *      ix_cc_v6v4_decap.h
 *
 * = DESCRIPTION
 *      This file defines all data types, macros and function prototypes
 *      that support the handling of packets requiring decapsulation.
 *
 * = CREATION TIME
 *      12/31/2002 12:10 PM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: ix_cc_v6v4_decap.h,v 1.3 2003/01/30 18:31:53 cabell Exp $
 */

#if !defined(__IX_CC_V6V4_DECAP_H__)
#define __IX_CC_V6V4_DECAP_H__

#include "cc/v6v4_tunnel/internal/ix_cc_v6v4_header_defs.h"

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */



/**
 * NAME: _ix_cc_v6v4_handle_ipv4_packet
 *
 * DESCRIPTION: This primitive is used to process IPv4 packets that
 * require decapsulation.
 *           
 * @Param:  - IN arg_hPacket - handle to buffer containing the packet.
 *
 * @Param:  - IN arg_pContext - pointer to tunneling core component
 * context buffer.
 *
 * @Param:  - OUT None.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_INTERNAL - internal error 
 * 
 */

ix_error _ix_cc_v6v4_handle_ipv4_packet(
    ix_buffer_handle arg_hPacket,  
    void* arg_pContext
    );


/**
 * NAME: _ix_cc_v6v4_decapsulate
 *
 * DESCRIPTION: This primitive is used to adjust the meta data remove the
 * IPv4 header from a tunneled packet.
 *           
 * @Param:  - IN arg_hPacket - handle to buffer containing the packet.
 *
 * @Param:  - IN arg_pContext - pointer to tunneling core component
 * context buffer.
 *
 * @Param:  - OUT None.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_INTERNAL - internal error 
 * 
 */

ix_error _ix_cc_v6v4_decapsulate(
    ix_buffer_handle arg_hPacket,  
    void* arg_pContext
    );




#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_V6V4_DECAP_H__) */
