/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor, Release 6
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      IPv6/v4 Tunneling Core Component
 *
 * = FILENAME
 *      ix_cc_v6v4_fini.h
 *
 * = DESCRIPTION
 *      This file defines all data types and function prototypes required
 *      internally by the termination portion of IPv6/v4 Tunneling CCI module.
 *
 * = CREATION TIME
 *      11/14/2002 9:31 AM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: ix_cc_v6v4_fini.h,v 1.4 2003/12/12 00:56:02 ktseng Exp $
 */

#if !defined(__IX_CC_V6V4_FINI_H__)
#define __IX_CC_V6V4_FINI_H__


#if defined(__cplusplus)
extern "C" {
#endif /* end defined(__cplusplus) */



/**
 * NAME: _ix_cc_v6v4_cleanup_reassembly
 *
 * DESCRIPTION: This primitive will be used to release resources
 * allocated for reassembly.
 *
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_INTERNAL - error from CCI
 * 
 */

ix_error _ix_cc_v6v4_cleanup_reassembly(void);


/**
 * NAME: _ix_cc_v6v4_unregister_message_handler
 *
 * DESCRIPTION: This primitive will be used to unregister the message handler.
 * It calls CCI services in level 1 system and resource manager services in
 * level 0 system.
 *
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV4_ERROR_CCI - error from CCI
 * 
 */

ix_error _ix_cc_v6v4_unregister_message_handler(void);


/**
 * NAME: _ix_cc_v6v4_unregister_cci_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to unregister the CCI packet
 * handlers.
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV4_ERROR_CCI - error from CCI
 * 
 */

ix_error _ix_cc_v6v4_unregister_cci_packet_handlers(void);


/**
 * NAME: _ix_cc_v6v4_unregister_rm_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to unregister the packet handler
 * with the resource manager
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV4_ERROR_CCI - error from RM
 * 
 */

ix_error _ix_cc_v6v4_unregister_rm_packet_handlers(void);


/**
 * NAME: _ix_cc_v6v4_unregister_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to unregister the packet handlers.
 *
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV4_ERROR_CCI - error from CCI/RM
 * 
 */

ix_error _ix_cc_v6v4_unregister_packet_handlers(void);


/**
 * NAME: _ix_cc_v6v4_end_tunnel_cleanup
 *
 * DESCRIPTION: This primitive is used to release the resources that were
 * allocated to manage end tunnel configuration.
 *           
 * @Param:  - IN None.
 *
 * @Param:  - OUT None.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOR - Out of resources 
 * IX_CC_ERROR_INTERNAL - internal error occurred 
 * 
 */

ix_error _ix_cc_v6v4_end_tunnel_cleanup(void);


/**
 * NAME: _ix_cc_v6v4_start_tunnel_cleanup
 *
 * DESCRIPTION: This primitive is used to release the resources that were
 * allocated to manage start tunnel configuration.
 *           
 * @Param:  - IN None.
 *
 * @Param:  - OUT None.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_INTERNAL - internal error occurred 
 * 
 */

ix_error _ix_cc_v6v4_start_tunnel_cleanup(void);


/**
 * NAME: _ix_cc_v6v4_free_counters
 *
 * DESCRIPTION: This primitive will be used to free memory allocated for
 * microblock counters.         .
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOM - failed to free memory for symbols.
 * 
 */

ix_error _ix_cc_v6v4_free_counters(void);


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_V6V4_FINI_H__) */
