/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      IPv4/v6 Tunneling Core Component
 *
 * = FILENAME
 *      ix_cc_v6v4_header_defs.h
 *
 * = DESCRIPTION
 *   This file contains definition of the IPv4 and IPv6 header structures
 *   needed for tunneling.   
 *
 *
 * = CREATION TIME
 *      12/30/2002 2:59 PM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: ix_cc_v6v4_header_defs.h,v 1.2 2003/01/28 16:39:08 cabell Exp $
 */

#if !defined(__IX_CC_V6V4_HEADER_DEFS_H__)
#define __IX_CC_V6V4_HEADER_DEFS_H__

/**
 * Preprocessor symbol and macro definitions.
 */
#include <stdio.h>
#include <ix_ossl.h>
#include <ix_rm.h>
#include <ix_cc.h>
#include <ix_netmacros.h>
#include "ix_cc_error.h"
#include "bindings.h"


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */


/** 
 * General defines
 */

#define IX_CC_V6V4_MIN_IPV4_LEN         20
#define IX_CC_V6V4_MAX_IPV4_LEN         60
#define IX_CC_V6V4_MIN_IPV6_LEN         40
#define IX_CC_V6V4_IP_PROTO_ICMP        1
#define IX_CC_V6V4_IP_PROTO_IPV6        41

#define IX_CC_V6V4_6TO4_PREFIX          0x2002

/**
 * Type definitions.
 */

/**
 * TYPENAME: ix_cc_v6v4_ipv4_header
 * 
 * DESCRIPTION: This struct defines the format of IP header 
 *
 */

typedef struct ix_s_cc_v6v4_ipv4_header
{
    ix_uint8    vers_hdrLength;
    ix_uint8    tos;
    ix_uint16   totalLength;
    ix_uint16   id;
    ix_uint16   flags_fragOffset;
    ix_uint8    ttl;
    ix_uint8    protocol;
    ix_uint16   checksum;
    ix_uint32   sourceAddress;
    ix_uint32   destAddress;
    ix_uint32   options[10];
} ix_cc_v6v4_ipv4_header;

/**
 * TYPENAME: ix_cc_v6v4_icmp_header
 * 
 * DESCRIPTION: This struct defines the format of ICMP header 
 *
 */
typedef struct ix_s_cc_v6v4_icmp_header
{
    ix_uint8    type;
    ix_uint8    code;
    ix_uint16   checksum;
    ix_uint32   extra;
} ix_cc_v6v4_icmp_header;


/**
 * TYPENAME: ix_cc_v6v4_ipv6_header
 * 
 * DESCRIPTION: This struct defines the format of IPv6 header 
 *
 */

typedef struct ix_s_cc_v6v4_ipv6_header
{
    ix_uint32 vers_class_flow;
    ix_uint16 payloadLength;
    ix_uint8  nextHeader;
    ix_uint8  hopLimit;
    ix_uint128 sourceAddress;
    ix_uint128 destAddress;
} ix_cc_v6v4_ipv6_header;


/**
 * Prototypes for interface functions.
 */


/**
 * Exported variables.
 */

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__HEADER_DEFS_H__) */
