/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor, Release 6
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      Core Component Infrastructure - Definitions for tunnel ICMP error
 *      message detection.
 *
 * = FILENAME
 *      ix_cc_v6v4_icmp.h
 *
 * = DESCRIPTION
 *      This file defines all data types, macros and function prototypes
 *      that support the detection and handling of ICMP error messages
 *      for tunneled packets.
 *
 * = CREATION TIME
 *      12/30/2002 2:02 PM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: ix_cc_v6v4_icmp.h,v 1.3 2003/01/28 16:38:30 cabell Exp $
 */

#if !defined(__IX_CC_V6V4_ICMP_H__)
#define __IX_CC_V6V4_ICMP_H__

#include "cc/ix_cc_ipv6.h"

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */


/**
 * Defines for ICMP types that are checked for by tunneling.
 */

#define IX_CC_V6V4_ICMP_DEST_UNREACH    3
#define IX_CC_V6V4_ICMP_TIME_EXCEEDED   11
#define IX_CC_V6V4_ICMP_PARAM_PROBLEM   12

/**
 * Codes for Destination Unreachable types that are mapped to ICMPv6
 * equivalents.
 */

#define IX_CC_V6V4_ICMP_DU_NETWORK      0
#define IX_CC_V6V4_ICMP_DU_HOST         1
#define IX_CC_V6V4_ICMP_DU_PROTOCOL     2
#define IX_CC_V6V4_ICMP_DU_PORT         3
#define IX_CC_V6V4_ICMP_DU_FRAG         4
#define IX_CC_V6V4_ICMP_DU_COMM_PROHIB  10

/**
 * Codes for Time Exceeded types that are mapped to ICMPv6
 * equivalents.
 */

#define IX_CC_V6V4_ICMP_TE_TTL          0
#define IX_CC_V6V4_ICMP_TE_FRAG_TIMER   1


/** 
 * General defines for ICMP
 */

#define IX_CC_V6V4_ICMP_MIN_DATA        8
#define IX_CC_V6V4_ICMP_HDR_LEN         8
#define IX_CC_V6V4_MIN_IPV4_LEN         20
#define IX_CC_V6V4_MIN_IPV6_LEN         40
#define IX_CC_V6V4_IP_PROTO_ICMP        1
#define IX_CC_V6V4_IP_PROTO_IPV6        41


/**
 * NAME: _ix_cc_v6v4_icmp_check
 *
 * DESCRIPTION: This primitive is used to check a packet to determine if it is
 * an ICMP error message received for tunneled packet.
 *           
 * @Param:  - IN arg_hPacket - handle to buffer containing the packet.
 *
 * @Param:  - IN arg_pContext - pointer to tunneling core component
 * context buffer.
 *
 * @Param:  - OUT arg_pSend - pointer to location in which to return an
 * indicator of whether the packet is to be sent up to the stack driver or
 * whether it was handled internally.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_INTERNAL - internal error 
 * 
 */

ix_error _ix_cc_v6v4_icmp_check(
    ix_buffer_handle arg_hPacket,  
    void* arg_pContext,
    ix_uint8* arg_pSend
    );


/**
 * NAME: _ix_cc_v6v4_pkt_too_big
 *
 * DESCRIPTION: This primitive is used to send an ICMPv6 Packet too Big
 * message.
 *           
 * @Param:  - IN arg_hPacket - handle to buffer containing the IPv6 packet.
 *
 * @Param:  - IN arg_pContext - pointer to tunneling core component
 * context buffer.
 *
 * @Param:  - OUT None.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_INTERNAL - internal error 
 * 
 */

ix_error _ix_cc_v6v4_pkt_too_big(
    ix_buffer_handle arg_hPacket,  
    void* arg_pContext
    );


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_V6V4_ICMP_H__) */
