/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      IPv6/v4 Tunneling Forwarder Core Component
 *
 * = FILENAME
 *      ix_cc_ipv4_init.h
 *
 * = DESCRIPTION
 *      This file defines all data types and function prototypes required
 *      internally by the initialization portio of IPv6/v4 Tunneling CCI
 *      module.
 *
 * = CREATION TIME
 *      11/13/2002 2:58 PM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: ix_cc_v6v4_init.h,v 1.8 2003/12/10 17:52:41 ktseng Exp $
 */

#if !defined(__IX_CC_V6V4_INIT_H__)
#define __IX_CC_V6V4_INIT_H__

#include "cc/ix_cc_index_pool.h"
#include "cc/v6v4_tunnel/internal/ix_cc_v6v4_ingress_list.h"
#include "cc/v6v4_tunnel/internal/ix_cc_v6v4_reassembly.h"

#if defined(__cplusplus)
extern "C" {
#endif /* end defined(__cplusplus) */


#define IX_CC_V6V4_REGISTRY_KEY_END_TUNNEL_SIZE  "V6V4/END_TUNNEL_SIZE"
#define IX_CC_V6V4_REGISTRY_KEY_START_TUNNEL_SIZE  "V6V4/START_TUNNEL_SIZE"
#define IX_CC_V6V4_REGISTRY_KEY_INGRESS_LIST_TYPE  "V6V4/INGRESS_LIST_TYPE"
#define IX_CC_V6V4_REGISTRY_KEY_INGRESS_LIST_SIZE  "V6V4/INGRESS_LIST_SIZE_HINT"
#define IX_CC_V6V4_REGISTRY_KEY_END_TUNNEL_MEMORY_TYPE  "V6V4/END_TUNNEL_MEMORY_TYPE"
#define IX_CC_V6V4_REGISTRY_KEY_START_TUNNEL_MEMORY_TYPE  "V6V4/START_TUNNEL_MEMORY_TYPE"
#define IX_CC_V6V4_REGISTRY_KEY_INGRESS_LIST_MEMORY_TYPE  "V6V4/INGRESS_LIST_MEMORY_TYPE"
#define IX_CC_V6V4_REGISTRY_KEY_END_TUNNEL_CHANNEL  "V6V4/END_TUNNEL_CHANNEL"
#define IX_CC_V6V4_REGISTRY_KEY_START_TUNNEL_CHANNEL  "V6V4/START_TUNNEL_CHANNEL"
#define IX_CC_V6V4_REGISTRY_KEY_INGRESS_LIST_CHANNEL  "V6V4/INGRESS_LIST_CHANNEL"
#define IX_CC_V6V4_REGISTRY_KEY_ME_FOR_V6V4  "V6V4/ME_FOR_V6V4"


#define IX_CC_V6V4_SRAM_CHANNEL  0
#define IX_CC_V6V4_NUMBER_OF_MB_DECAP_COUNTERS 2  /* number of decap microblock counters */
#define IX_CC_V6V4_NUMBER_OF_MB_ENCAP_COUNTERS 2  /* number of encap microblock counters */
#define IX_CC_V6V4_NUMBER_OF_MB_COUNTERS (IX_CC_V6V4_NUMBER_OF_MB_DECAP_COUNTERS+IX_CC_V6V4_NUMBER_OF_MB_ENCAP_COUNTERS)

#if (_IX_HARDWARE_TYPE_ == _IX_HW_2800_)
#define IX_CC_V6V4_NUMBER_OF_SYMBOLS     3 /* number symbols to be patched */
#define IX_CC_V6V4_END_TUNNEL_CHANNEL    2
#define IX_CC_V6V4_START_TUNNEL_CHANNEL  2
#else /* (_IX_HARDWARE_TYPE_ == _IX_HW_2800_) */
#define IX_CC_V6V4_NUMBER_OF_SYMBOLS     5 /* number symbols to be patched */
#define IX_CC_V6V4_END_TUNNEL_CHANNEL    0 /* End Tunnel SRAM/DRAM Channel number */
#define IX_CC_V6V4_START_TUNNEL_CHANNEL  0 /* Start Tunnel SRAM/DRAM Channel number */
#endif /* (_IX_HARDWARE_TYPE_ == _IX_HW_2800_) */

#if !defined(IX_CC_V6V4_INGRESS_LIST_FORMAT)
#define IX_CC_V6V4_INGRESS_LIST_FORMAT IX_CC_V6V4_ISL_DEFAULT_MAX_12
#endif
#define IX_CC_V6V4_ALLOWED_SRCS_HINT 12

#define IX_CC_V6V4_END_TUNNEL_SIZE 128 /* Maximum end tunnels */
#define IX_CC_V6V4_START_TUNNEL_SIZE 256 /* Maximum start tunnels */
#define IX_CC_V6V4_INGRESS_LIST_SIZE_HINT 256 /* Size hint for ingress source list */
#define IX_CC_V6V4_END_TUNNEL_MEMORY_TYPE  IX_CC_V6V4_MEM_SRAM /* End tunnel memory type: SRAM or DRAM */
#define IX_CC_V6V4_START_TUNNEL_MEMORY_TYPE  IX_CC_V6V4_MEM_SRAM /* Start tunnel memory type: SRAM or DRAM */
#define IX_CC_V6V4_INGRESS_LIST_MEMORY_TYPE  IX_CC_V6V4_MEM_SRAM /* Ingress list memory type: SRAM or DRAM */

#if (_IX_HARDWARE_TYPE_ == _IX_HW_2800_)

#define IX_CC_V6V4_NUMBER_OF_MICROENGINES 16
#define IX_CC_V6V4_ME 0x3007
#define IX_CC_V6V4_INGRESS_LIST_CHANNEL 1 /* Ingress list SRAM/DRAM Channel number */

#else /* (_IX_HARDWARE_TYPE_ == _IX_HW_2800_) */

#define IX_CC_V6V4_NUMBER_OF_MICROENGINES 8 /* number of microengines: 8 for IXP2400 */
#define IX_CC_V6V4_ME 0x66 /* Micro engine allocated for IPv4 are 0x01,0x02,0x11,0x12 */
#define IX_CC_V6V4_INGRESS_LIST_CHANNEL 0 /* Ingress list SRAM/DRAM Channel number */

#endif /* (_IX_HARDWARE_TYPE_ == _IX_HW_2800_) */

#define IX_CC_V6V4_SYMBOL_SIZE 128 /* Maximum size of symbol name */
#define IX_CC_V6V4_PATCHNAME_END_TUNNEL_TABLE "TUNNEL_DECAP_NEXTHOP_SRAM_BASE" /* symbol name for end tunnel table */
#define IX_CC_V6V4_PATCHNAME_START_TUNNEL_TABLE "TUNNEL_ENCAP_NEXTHOP_SRAM_BASE" /* symbol name for end tunnel table */
#define IX_CC_V6V4_PATCHNAME_INGRESS_LIST "TUNNEL_INGRESS_LIST_SRAM_BASE" /*symbol name for ingress list block array */
#define IX_CC_V6V4_PATCHNAME_INGRESS_LIST_TRIE "TUNNEL_INGRESS_LIST_SRAM_TRIE_BASE" /*symbol name for ingress list trie blocks */
#define IX_CC_V6V4_PATCHNAME_DECAP_STATS "V6V4_DECAP_COUNTER_BASE" /* symbol name for decap counters */
#define IX_CC_V6V4_PATCHNAME_ENCAP_STATS "V6V4_ENCAP_COUNTER_BASE" /* symbol name for encap counters */

#define IX_CC_V6V4_INITIAL_MTU    1300

#define IX_CC_V6V4_HDRTYPE_DECAP  0x02
#define IX_CC_V6V4_HDRTYPE_ENCAP  0x10

#define TOTAL_NUMBER_OF_PORTS \
    (MAX_NUMBER_OF_BLADES * MAX_NUMBER_OF_PORTS_PER_BLADE)

/**
 * Defines the number of tunnels in an ix_cc_v6v4_tunnel_group, which maintains
 * which start tunnels are configured on an interface
 */
#define IX_CC_V6V4_TUNNEL_LIST_ENTRIES      8

/**
 * Reassembly parameters.
 *   IX_CC_V6V4_REASSEMBLY_TICK - defines the length (in ms) of one tick of
 *   the reassembly timer.
 *   IX_CC_V6V4_REASSEMBLY_TIMEOUT - defines the number of ticks for which
 *   reassembly resources will be maintained for a packet without receiving
 *   a fragment.
 *   IX_CC_V6V4_REASSEMBLY_PRIORITY - defines the priority to be passed to
 *   ix_cci_cc_add_event_handler.
 */
#define IX_CC_V6V4_REASSEMBLY_TICK      1000
#define IX_CC_V6V4_REASSEMBLY_TIMEOUT   15
#define IX_CC_V6V4_REASSEMBLY_PRIORITY  0


/**
 * TYPENAME: ix_cc_v6v4_prop_data
 * 
 * DESCRIPTION: This structure defines the format for dynamic property data.
 *
 */

typedef struct ix_s_cc_v6v4_prop_data
{
    ix_uint32 ipAddr;
    ix_uint32 netMask;
    ix_uint32 broadcast;
    ix_uint16 mtu;
} ix_cc_v6v4_prop_data;

/**
 * TYPENAME: ix_cc_v6v4_core_stats
 * 
 * DESCRIPTION: This structure defines the format for the tunneling
 * core statistics.
 *
 */
typedef struct ix_s_cc_v6v4_core_stats
{
    ix_uint32 localPkts;
    ix_uint32 autoTunnelPkts;
    ix_uint32 fragmentsRcvd;
    ix_uint32 reassembledPkts;
    ix_uint32 pktTooBigMsgsSent;
    ix_uint32 pktsEncapsulated;
    ix_uint32 pktsDecapsulated;
} ix_cc_v6v4_core_stats;

/**
 * TYPENAME: ix_cc_v6v4_tunnel_group
 * 
 * DESCRIPTION: This structure defines some information needed to track.
 * which tunnels are configured on each interface and whether or not we
 * are tracking certain interface properties for the tunnel.
 *
 */
typedef struct ix_s_cc_v6v4_tunnel_group
{
    struct ix_s_cc_v6v4_tunnel_group* pNext;
    ix_uint32                        numEntries;
    ix_uint8                         aFlags[IX_CC_V6V4_TUNNEL_LIST_ENTRIES];
    ix_uint16                        aTunnel[IX_CC_V6V4_TUNNEL_LIST_ENTRIES];
} ix_cc_v6v4_tunnel_group;


/**
 * TYPENAME: ix_cc_v6v4_context
 * 
 * DESCRIPTION: This structure defines the format for the Tunneling
 * core component context
 *
 */
typedef struct ix_s_cc_v6v4_context
{
    /* Handle of tunneling core component */
    ix_cc_handle hCcHandle;

    /* mutex id */
    ix_ossl_mutex_t v6v4Mutex;

    /* base address of end tunnel table for microblock patching */
    ix_uint32* pMbEndTunnelBase;

    /* base address of start tunnel table for microblock patching */
    ix_uint32* pMbStartTunnelBase;

    /* statistics base address for microblock patching */
    ix_uint32* pMbDecapStatsBase;
    ix_uint32* pMbEncapStatsBase;

    /* ingress source list info */
    ix_cc_v6v4_ingress_list_data islInfo;

     /* handle of free list buffer */
    ix_buffer_free_list_handle hBufferFreeList;

    /* interface property table */
    ix_cc_v6v4_prop_data aIfPropTable[TOTAL_NUMBER_OF_PORTS];

     /* tunneling core component statistics */
    ix_cc_v6v4_core_stats ccStats;

    /* pool of available indexes for end tunnel entries */
    ix_cc_index_pool endTunnelIndexPool;

    /* pool of available indexes for start tunnel entries */
    ix_cc_index_pool startTunnelIndexPool;

    /* tunnels per interface table */
    ix_cc_v6v4_tunnel_group* aIfTunnelTable[TOTAL_NUMBER_OF_PORTS];

    /* ingress source validation list handlers */
    ix_cc_v6v4_ingress_list_handlers ingressListHandlers;

    /* IPv4 reassembly resources for tunneled packets */
    ix_cc_v6v4_reassembly hReassembly;

#if !defined(IX_EXCLUDE_CCI)
    /* Reassembly timer event handle */
    ix_event_handle hTimerEvent;
#endif

} ix_cc_v6v4_context;




/**
 * TYPENAME: ix_cc_v6v4_registry_data
 * 
 * DESCRIPTION: This structure defines the format for registry data.
 *
 */

typedef struct ix_s_cc_v6v4_registry_data
{
    ix_uint32 meNumbers;
    ix_uint32 localBladeId;
    ix_uint32 endTunnelSize;
    ix_uint32 startTunnelSize;
    ix_uint32 ingressListSizeHint;
    ix_e_cc_v6v4_isl_type ingressListFormat;
    ix_cc_v6v4_mem_type endTunnelMemType;
    ix_cc_v6v4_mem_type startTunnelMemType;
    ix_cc_v6v4_mem_type ingressListMemType;
    ix_uint32 endTunnelChannel;
    ix_uint32 startTunnelChannel;
    ix_uint32 ingressListChannel;

} ix_cc_v6v4_registry_data;


extern ix_cc_v6v4_context* g_pV6v4Context;


/**
 * NAME: _ix_cc_v6v4_reassembly_event_handler
 *
 * DESCRIPTION: This primitive is used to handle the reassembly timer events.
 * 
 * @Param:  - IN arg_pContext - pointer to core component context 
 *
 * @Param:  - OUT None 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 */

ix_error _ix_cc_v6v4_reassembly_event_handler(
    void* arg_pContext
    );

/**
 * NAME: _ix_cc_v6v4_retrieve_registry_data
 *
 * DESCRIPTION: This primitive will be used to retreive data from the registry
 * for a given property type. If there is no registry, then the data structure 
 * will be filled with data from header file.
 *           .
 * 
 * @Param:  - IN arg_pPropertyName - pointer to property name string 
 *
 * @Param:  - OUT arg_pPropertyValue - pointer to property value 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_REGISTRY  - failed to access registry
 * 
 */


ix_error _ix_cc_v6v4_retrieve_registry_data(
    const char* arg_pPropertyName,
    ix_uint32* arg_pPropertyValue
    );




/**
 * NAME: _ix_cc_v6v4_registry
 *
 * DESCRIPTION: This primitive will be used to retreive data from the registry
 * for all tunneling static data and store it in v6v4RegistyData 
 *           .
 * 
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_REGISTRY  - not able to access registry.
 * 
 */

ix_error _ix_cc_v6v4_registry(void);



/**
 * NAME: _ix_cc_v6v4_alloc_shared_memory
 *
 * DESCRIPTION: This primitive is used to allocate a region of shared memory.
 * 
 * @Param:  - IN arg_MemSize: Total size of the table in bytes.
 * @Param:  - IN arg_MemType: Type of memory.
 * @Param:  - IN arg_MemChannel: Memory channel.
 *
 * @Param:  - OUT arg_ppTableBase: Location into which to return table
 * address.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOM  - not able to allocate memory
 * 
 */

ix_error _ix_cc_v6v4_alloc_shared_memory(
    ix_uint32 arg_MemSize,
    ix_cc_v6v4_mem_type arg_MemType,
    ix_uint32 arg_MemChannel,
    void** arg_ppTableBase
    );


/**
 * NAME: _ix_cc_v6v4_alloc_shared_table
 *
 * DESCRIPTION: This primitive is used to allocate a table in shared memory. 
 * 
 * @Param:  - IN arg_NumEntries: Number of entries in the table.
 * @Param:  - IN arg_EntrySize: Size of each entry in bytes.
 * @Param:  - IN arg_MemType: Type of memory.
 * @Param:  - IN arg_MemChannel: Memory channel.
 *
 * @Param:  - OUT arg_ppTableBase: Location into which to return table
 * address.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOM  - not able to allocate memory
 * 
 */

ix_error _ix_cc_v6v4_alloc_shared_table(
    ix_uint32 arg_NumEntries,
    ix_uint32 arg_EntrySize,
    ix_cc_v6v4_mem_type arg_MemType,
    ix_uint32 arg_MemChannel,
    void** arg_ppTableBase
    );


/**
 * NAME: _ix_cc_v6v4_patch_symbols
 *
 * DESCRIPTION: This primitive will be used to patch symbols to the tunneling
 * microblocks. 
 * 
 *           .
 * 
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * IX_CC_V6V4_ERROR_FAILED_PATCHING - failed patching by calling resource
 * manager function
 * 
 */
ix_error _ix_cc_v6v4_patch_symbols(void);


/**
 * NAME: _ix_cc_v6v4_init_reassembly
 *
 * DESCRIPTION: This primitive will be used to initialize the resources
 * for reassembly of tunneled packets. 
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT  none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOR - Out of resources
 * IX_CC_ERROR_OOM - Out of memory
 * IX_CC_ERROR_INTERNAL - Error from CCI
 * 
 */

ix_error _ix_cc_v6v4_init_reassembly(void);


/**
 * NAME: _ix_cc_v6v4_register_message_handler
 *
 * DESCRIPTION: This primitive will be used to register the message handler. 
 *
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_CCI - Error from CCI
 * 
 */

ix_error _ix_cc_v6v4_register_message_handler(void);


/**
 * NAME: _ix_cc_v6v4_register_cci_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to register the packet handlers
 * with CCI.
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_CCI - error from CCI
 * 
 */

ix_error _ix_cc_v6v4_register_cci_packet_handlers(void);


/**
 * NAME: _ix_cc_v6v4_register_rm_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to register the packet handler
 * with the resource manager. This will be used in level 0 system.
 *
 *
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_CCI - error from resource manager
 * 
 */

ix_error _ix_cc_v6v4_register_rm_packet_handlers(void);


/**
 * NAME: _ix_cc_v6v4_register_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to register the packet handlers.
 * 
 *           .
 * 
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_CCI - error from CCI
 * 
 */

ix_error _ix_cc_v6v4_register_packet_handlers(void);


/**
 * NAME: _ix_cc_v6v4_end_tunnel_setup
 *
 * DESCRIPTION: This primitive is used to allocate the resources needed
 * to manage end tunnel configuration.
 *           
 * @Param:  - IN None.
 *
 * @Param:  - OUT None.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOR - Out of resources 
 * IX_CC_ERROR_INTERNAL - internal error occurred 
 * 
 */

ix_error _ix_cc_v6v4_end_tunnel_setup(void);


/**
 * NAME: _ix_cc_v6v4_start_tunnel_setup
 *
 * DESCRIPTION: This primitive is used to allocate the resources needed
 * to manage start tunnel configuration.
 *           
 * @Param:  - IN None.
 *
 * @Param:  - OUT None.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_INTERNAL - internal error occurred 
 * 
 */

ix_error _ix_cc_v6v4_start_tunnel_setup(void);


/**
 * NAME: _ix_cc_v6v4_alloc_counters
 *
 * DESCRIPTION: This primitive is used to allocate shared memory for
 * counters.
 * 
 * @Param:  - IN - none
 *
 * @Param:  - OUT  - none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOM  - not able to allocate memory
 * 
 */

ix_error _ix_cc_v6v4_alloc_counters(void);





#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_V6V4_INIT_H__) */
