/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor, Release 6
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      Core Component Infrastructure - message handler and message callbacks
 *
 * = FILENAME
 *      ix_cc_v6v4_msg_handler.h
 *
 * = DESCRIPTION
 *      This file defines all data types and function prototypes required
 *      internally by the message handlers of the IPv6/v4 Tunneling CCI
 *      module.
 *
 * = CREATION TIME
 *      11/14/2002 9:39 AM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: ix_cc_v6v4_msg_handler.h,v 1.3 2003/01/30 18:33:31 cabell Exp $
 */

#if !defined(__IX_CC_V6V4_MSG_HANDLER_H__)
#define __IX_CC_V6V4_MSG_HANDLER_H__


#if defined(__cplusplus)
extern "C" {
#endif /* end defined(__cplusplus) */


/**
 * TYPENAME: ix_cc_v6v4_set_decap_tos_data
 * 
 * DESCRIPTION: This structure defines the format for the set decap TOS
 * message helper.
 *
 */
typedef struct ix_s_cc_v6v4_set_decap_tos_data
{
    ix_cc_v6v4_tunnel_handle hTunnel;
    ix_uint8 selection;
} ix_cc_v6v4_set_decap_tos_data;

/**
 * TYPENAME: ix_cc_v6v4_set_src_validation_data
 * 
 * DESCRIPTION: This structure defines the format for the set source
 * validation message helper.
 *
 */
typedef struct ix_s_cc_v6v4_set_src_validation_data
{
    ix_cc_v6v4_tunnel_handle hTunnel;
    ix_uint8 selection;
} ix_cc_v6v4_set_src_validation_data;

/**
 * TYPENAME: ix_cc_v6v4_add_allowed_source_data
 * 
 * DESCRIPTION: This structure defines the format for the add allowed source
 * message helper.
 *
 */
typedef struct ix_s_cc_v6v4_add_allowed_source_data
{
    ix_cc_v6v4_tunnel_handle hTunnel;
    ix_cc_v6v4_ingress_source_entry islEntry;
} ix_cc_v6v4_add_allowed_source_data;


/**
 * TYPENAME: ix_cc_v6v4_delete_allowed_source_data
 * 
 * DESCRIPTION: This structure defines the format for the delete allowed source
 * message helper.
 *
 */
typedef struct ix_s_cc_v6v4_delete_allowed_source_data
{
    ix_cc_v6v4_tunnel_handle hTunnel;
    ix_cc_v6v4_ingress_source_entry islEntry;
} ix_cc_v6v4_delete_allowed_source_data;


/**
 * TYPENAME: ix_cc_v6v4_set_ttl_data
 * 
 * DESCRIPTION: This structure defines the format for the set TTL message
 * helper.
 *
 */
typedef struct ix_s_cc_v6v4_set_ttl_data
{
    ix_cc_v6v4_tunnel_handle hTunnel;
    ix_uint8 ttl;
} ix_cc_v6v4_set_ttl_data;


/**
 * TYPENAME: ix_cc_v6v4_set_encap_tos_data
 * 
 * DESCRIPTION: This structure defines the format for the set encap TOS
 * option message helper.
 *
 */
typedef struct ix_s_cc_v6v4_set_encap_tos_data
{
    ix_cc_v6v4_tunnel_handle hTunnel;
    ix_uint8 selection;
} ix_cc_v6v4_set_encap_tos_data;

/**
 * TYPENAME: ix_cc_v6v4_set_tos_data
 * 
 * DESCRIPTION: This structure defines the format for the set TOS value
 * message helper.
 *
 */
typedef struct ix_s_cc_v6v4_set_tos_data
{
    ix_cc_v6v4_tunnel_handle hTunnel;
    ix_uint8 tos;
} ix_cc_v6v4_set_tos_data;

/**
 * TYPENAME: ix_cc_v6v4_set_mtu_data
 * 
 * DESCRIPTION: This structure defines the format for the set mtu message
 * helper.
 *
 */
typedef struct ix_s_cc_v6v4_set_mtu_data
{
    ix_cc_v6v4_tunnel_handle hTunnel;
    ix_uint16 mtu;
} ix_cc_v6v4_set_mtu_data;


/**
 * TYPENAME: ix_cc_v6v4_set_subnet_broadcast_data
 * 
 * DESCRIPTION: This structure defines the format for the set subnet broadcast
 * message helper.
 *
 */
typedef struct ix_s_cc_v6v4_set_subnet_broadcast_data
{
    ix_cc_v6v4_tunnel_handle hTunnel;
    ix_uint32 subnetBroadcast;
} ix_cc_v6v4_set_subnet_broadcast_data;


/**
 * TYPENAME: ix_cc_v6v4_general_callback_data
 * 
 * DESCRIPTION: This structure defines the format for general callback data.
 *
 */
typedef struct ix_s_cc_v6v4_general_callback_data
{
    ix_cc_v6v4_cb pUserCallback;
    void* pUserContext;
} ix_cc_v6v4_general_callback_data;



/**
 * TYPENAME: ix_cc_v6v4_add_tunnel_callback_data
 * 
 * DESCRIPTION: This structure defines the format for add tunnel callback data.
 *
 */
typedef struct ix_s_cc_v6v4_add_tunnel_callback_data
{
    ix_cc_v6v4_cb_add_tunnel pUserCallback;
    void* pUserContext;
} ix_cc_v6v4_add_tunnel_callback_data;


/**
 * TYPENAME: ix_cc_v6v4_end_tunnel_callback_data
 * 
 * DESCRIPTION: This structure defines the format for get end tunnel callback
 * data.
 *
 */
typedef struct ix_s_cc_v6v4_end_tunnel_callback_data
{
    ix_cc_v6v4_cb_end_tunnel pUserCallback;
    void* pUserContext;
} ix_cc_v6v4_end_tunnel_callback_data;


/**
 * TYPENAME: ix_cc_v6v4_get_sources_callback_data
 * 
 * DESCRIPTION: This structure defines the format for get sources callback
 * data.
 *
 */
typedef struct ix_s_cc_v6v4_get_sources_callback_data
{
    ix_cc_v6v4_cb_get_sources pUserCallback;
    void* pUserContext;
} ix_cc_v6v4_get_sources_callback_data;


/**
 * TYPENAME: ix_cc_v6v4_dump_end_tunnels_callback_data
 * 
 * DESCRIPTION: This structure defines the format for dump end tunnels
 * callback data.
 *
 */
typedef struct ix_s_cc_v6v4_dump_end_tunnels_callback_data
{
    ix_cc_v6v4_cb_dump_end_tunnels pUserCallback;
    void* pUserContext;
} ix_cc_v6v4_dump_end_tunnels_callback_data;


/**
 * TYPENAME: ix_cc_v6v4_start_tunnel_callback_data
 * 
 * DESCRIPTION: This structure defines the format for get start tunnel callback
 * data.
 *
 */
typedef struct ix_s_cc_v6v4_start_tunnel_callback_data
{
    ix_cc_v6v4_cb_start_tunnel pUserCallback;
    void* pUserContext;
} ix_cc_v6v4_start_tunnel_callback_data;


/**
 * TYPENAME: ix_cc_v6v4_dump_start_tunnels_callback_data
 * 
 * DESCRIPTION: This structure defines the format for dump start tunnels
 * callback data.
 *
 */
typedef struct ix_s_cc_v6v4_dump_start_tunnels_callback_data
{
    ix_cc_v6v4_cb_dump_start_tunnels pUserCallback;
    void* pUserContext;
} ix_cc_v6v4_dump_start_tunnels_callback_data;




/**
 * TYPENAME: ix_cc_v6v4_statistics_callback_data
 * 
 * DESCRIPTION: This structure defines the format for statistics callback data.
 *
 */
typedef struct ix_s_cc_v6v4_statistics_callback_data
{
    ix_cc_v6v4_cb_statistics pUserCallback;
    void* pUserContext;
} ix_cc_v6v4_statistics_callback_data;






/**
 * NAME: _ix_cc_v6v4_cb_genaral
 *
 * DESCRIPTION: This is a general callback function. The message support
 * library  calls this function to return the status of the requested
 * asynchronous operation.
 * @Param:  - IN  arg_Result: result of the asynchronous operation
 * @Param:  - IN  arg_pContext: context provided by caller
 * @Param:  - IN  arg_pMsg: return message
 * @Param:  - IN  arg_MsgLength: length of the message
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_CALLBACK - user callback returns error message
 * 
 */

ix_error _ix_cc_v6v4_cb_general(
    ix_error arg_Result,
    void* arg_pContext,
    void* arg_pMsg,
    ix_uint32 arg_MsgLength
    );

/**
 * NAME: _ix_cc_v6v4_cb_add_tunnel
 *
 * DESCRIPTION: This is a callback function for add tunnel message helpers.
 * The message support library calls this function to return the status of the
 * requested asynchrouns operation.
 * @Param:  - IN  arg_Result: result of the asynchronous operation
 * @Param:  - IN  arg_pContext: context provided by caller
 * @Param:  - IN  arg_pMsg: return message
 * @Param:  - IN  arg_MsgLength: length of the message
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error _ix_cc_v6v4_cb_add_tunnel(
    ix_error arg_Result,
    void* arg_pContext,
    void* arg_pMsg,
    ix_uint32 arg_MsgLength
    );

/**
 * NAME: _ix_cc_v6v4_cb_end_tunnel
 *
 * DESCRIPTION: This is a callback function for add tunnel message helpers.
 * The message support library calls this function to return the status of the
 * requested asynchrouns operation.
 * @Param:  - IN  arg_Result: result of the asynchronous operation
 * @Param:  - IN  arg_pContext: context provided by caller
 * @Param:  - IN  arg_pMsg: return message
 * @Param:  - IN  arg_MsgLength: length of the message
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error _ix_cc_v6v4_cb_end_tunnel(
    ix_error arg_Result,
    void* arg_pContext,
    void* arg_pMsg,
    ix_uint32 arg_MsgLength
    );

/**
 * NAME: _ix_cc_v6v4_cb_get_sources
 *
 * DESCRIPTION: This is a callback function for get allowed sources message
 * helpers. The message support library calls this function to return the
 * status of the requested asynchrouns operation.
 * @Param:  - IN  arg_Result: result of the asynchronous operation
 * @Param:  - IN  arg_pContext: context provided by caller
 * @Param:  - IN  arg_pMsg: return message
 * @Param:  - IN  arg_MsgLength: length of the message
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error _ix_cc_v6v4_cb_get_sources(
    ix_error arg_Result,
    void* arg_pContext,
    void* arg_pMsg,
    ix_uint32 arg_MsgLength
    );

/**
 * NAME: _ix_cc_v6v4_cb_dump_end_tunnels
 *
 * DESCRIPTION: This is a callback function for dump end tunnel message
 * helpers. The message support library calls this function to return the
 * status of the requested asynchrouns operation.
 * @Param:  - IN  arg_Result: result of the asynchronous operation
 * @Param:  - IN  arg_pContext: context provided by caller
 * @Param:  - IN  arg_pMsg: return message
 * @Param:  - IN  arg_MsgLength: length of the message
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error _ix_cc_v6v4_cb_dump_end_tunnels(
    ix_error arg_Result,
    void* arg_pContext,
    void* arg_pMsg,
    ix_uint32 arg_MsgLength
    );

/**
 * NAME: _ix_cc_v6v4_cb_start_tunnel
 *
 * DESCRIPTION: This is a callback function for get start tunnel message
 * helpers. The message support library calls this function to return the
 * status of the requested asynchrouns operation.
 * @Param:  - IN  arg_Result: result of the asynchronous operation
 * @Param:  - IN  arg_pContext: context provided by caller
 * @Param:  - IN  arg_pMsg: return message
 * @Param:  - IN  arg_MsgLength: length of the message
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error _ix_cc_v6v4_cb_start_tunnel(
    ix_error arg_Result,
    void* arg_pContext,
    void* arg_pMsg,
    ix_uint32 arg_MsgLength
    );


/**
 * NAME: _ix_cc_v6v4_cb_dump_start_tunnels
 *
 * DESCRIPTION: This is a callback function for dump start tunnel message
 * helpers. The message support library calls this function to return the
 * status of the requested asynchrouns operation.
 * @Param:  - IN  arg_Result: result of the asynchronous operation
 * @Param:  - IN  arg_pContext: context provided by caller
 * @Param:  - IN  arg_pMsg: return message
 * @Param:  - IN  arg_MsgLength: length of the message
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error _ix_cc_v6v4_cb_dump_start_tunnels(
    ix_error arg_Result,
    void* arg_pContext,
    void* arg_pMsg,
    ix_uint32 arg_MsgLength
    );

/**
 * NAME: _ix_cc_v6v4_cb_statistics
 *
 * DESCRIPTION: This is a callback function for get statistics message
 * helpers. The message support library calls this function to return the
 * status of the requested asynchrouns operation.
 * @Param:  - IN  arg_Result: result of the asynchronous operation
 * @Param:  - IN  arg_pContext: context provided by caller
 * @Param:  - IN  arg_pMsg: return message
 * @Param:  - IN  arg_MsgLength: length of the message
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error _ix_cc_v6v4_cb_statistics(
    ix_error arg_Result,
    void* arg_pContext,
    void* arg_pMsg,
    ix_uint32 arg_MsgLength
    );


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_V6V4_MSG_HANDLER_H__) */
