/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor, Release 6
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      Core Component Infrastructure - Dynamic Property Table management
 *
 * = FILENAME
 *      ix_cc_v6v4_prop.h
 *
 * = DESCRIPTION
 *      This file defines all data types and function prototypes required by
 *      the IPv6/v4 tunneling dynamic property table management module.
 *
 * = CREATION TIME
 *      11/15/2002 10:16 AM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: ix_cc_v6v4_prop.h,v 1.3 2003/01/30 18:33:31 cabell Exp $
 */

#if !defined(__IX_CC_V6V4_PROP_H__)
#define __IX_CC_V6V4_PROP_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */





/**
 * NAME: _ix_cc_v6v4_prop_get_port_ip_address
 *
 * DESCRIPTION: This primitive is used to get IP address of a port
 *           .
 * 
 * @Param:  - IN arg_BladeId - blade number of ip address to be retrieved
 * @Param:  - IN arg_PortId - port number of ip address to be retrieved
 *
 * @Param:  - OUT arg_pIpAddress - IP address of given blade and port
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_INVALID_INPUT_PARAM - invalid blade/port number
 * IX_CC_ERROR_ENTRY_NOT_FOUND - no data in location
 */
ix_error _ix_cc_v6v4_get_port_ip_address(
    ix_uint16 arg_BladeId,
    ix_uint16 arg_PortId, 
    ix_uint32 *arg_pIpAddr
    );


/**
 * NAME: _ix_cc_v6v4_prop_get_port_broadcast_address
 *
 * DESCRIPTION: This primitive is used to get the broadcast address of a
 * port
 *           .
 * 
 * @Param:  - IN arg_BladeId - blade number for interface
 * @Param:  - IN arg_PortId - port number for interface
 *
 * @Param:  - OUT arg_pBdcastAddr - Broadcast address of given blade and port
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_INVALID_INPUT_PARAM - invalid blade/port number
 * IX_CC_ERROR_ENTRY_NOT_FOUND - no data in location
 */
ix_error _ix_cc_v6v4_prop_get_port_broadcast_address(
    ix_uint16 arg_BladeId,
    ix_uint16 arg_PortId, 
    ix_uint32 *arg_pBdcastAddr
    );


/**
 * NAME: _ix_cc_v6v4_prop_get_port_mtu
 *
 * DESCRIPTION: This primitive is used to get the mtu for a port.
 *           .
 * 
 * @Param:  - IN arg_BladeId - blade number of interface
 * @Param:  - IN arg_PortId - port number of interface
 *
 * @Param:  - OUT arg_pMtu - Mtu for of given blade and port
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_INVALID_INPUT_PARAM - invalid blade/port number
 * IX_CC_ERROR_ENTRY_NOT_FOUND - no data in location
 */
ix_error _ix_cc_v6v4_prop_get_port_mtu(
    ix_uint16 arg_BladeId,
    ix_uint16 arg_PortId, 
    ix_uint16 *arg_pMtu
    );


/**
 * NAME: _ix_cc_v6v4_property_update_ip_address
 *
 * DESCRIPTION: This primitive is used to add/delete ip address and
 * net mask in the dynamic property table.
 *           .
 * 
 * @Param:  - IN arg_PropId - property id : add/delete IP address
 * @Param:  - IN arg_BladeId - blade number
 * @Param:  - IN arg_PordId - port number
 * @Param:  - IN arg_IpAddr - IP address
 * @Param:  - IN arg_NetMask - Subnet mask
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_INVALID_INPUT_PARAM - invalid blade/port number 
 * 
 */
ix_error _ix_cc_v6v4_property_update_ip_address(
    ix_uint16 arg_PropId,
    ix_uint32 arg_BladeId,
    ix_uint32 arg_PortId,
    ix_uint32 arg_IpAddr,
    ix_uint32 arg_NetMask
    );


/**
 * NAME: _ix_cc_v6v4_property_update_broadcast
 *
 * DESCRIPTION: This primitive is used to add/delete broadcast address
 * in the dynamic property table.
 *           .
 * 
 * @Param:  - IN arg_PropId - property id : add/delete broadcast
 * @Param:  - IN arg_BladeId - blade number
 * @Param:  - IN arg_PordId - port number
 * @Param:  - IN arg_BroadcastAddr - broadcast address
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_INVALID_INPUT_PARAM - invalid blade/port number 
 * 
 */
ix_error _ix_cc_v6v4_property_update_broadcast(
    ix_uint16 arg_PropId,
    ix_uint32 arg_BladeId,
    ix_uint32 arg_PortId,
    ix_uint32 arg_BroadcastAddr
    );


/**
 * NAME: _ix_cc_v6v4_property_update_mtu
 *
 * DESCRIPTION: This primitive is used to update the MTU of an interface
 * in the dynamic property table.
 *           .
 * 
 * @Param:  - IN arg_BladeId - blade number
 * @Param:  - IN arg_PordId - port number
 * @Param:  - IN arg_Mtu - MTU
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_INVALID_INPUT_PARAM - invalid blade/port number 
 * 
 */
ix_error _ix_cc_v6v4_property_update_mtu(
    ix_uint32 arg_BladeId,
    ix_uint32 arg_PortId,
    ix_uint16 arg_Mtu
    );


/**
 * NAME: _ix_cc_v6v4_prop_table_lookup_ip_address
 *
 * DESCRIPTION: This primitive is used to lookup an ip address in the
 * dynamic property table and return the associated blade ID and port ID.
 *           
 * @Param:  - IN arg_IpAddr - ip address(non zero) to be searched in table.
 *
 * @Param:  - OUT arg_pBladeId - pointer to location in which to return the
 * blade ID associated with the IP address.
 *
 * @Param:  - OUT arg_pPortId - pointer to location in which to return the
 * port ID associated with the IP address.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_ENTRY_NOT_FOUND - no entry in table 
 * 
 */

ix_error _ix_cc_v6v4_prop_table_lookup_ip_address(
    ix_uint32 arg_IpAddr,
    ix_uint16* arg_pBladeId,
    ix_uint16* arg_pPortId
    );


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_V6V4_PROP_H__) */
