/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor, Release 6
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      Core Component Infrastructure - Definitions for tunnel property
 *      management.
 *
 * = FILENAME
 *      ix_cc_v6v4_tunnel_prop.h
 *
 * = DESCRIPTION
 *      This file defines all data types, macros and function prototypes
 *      that support the management of property updates as they apply to
 *      start tunnel endpoint configuration.
 *
 * = CREATION TIME
 *      11/19/2002 9:07 AM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: ix_cc_v6v4_tunnel_prop.h,v 1.4 2003/10/31 17:27:48 ktseng Exp $
 */


#if !defined(__IX_CC_V6V4_TUNNEL_PROP_H__)
#define __IX_CC_V6V4_TUNNEL_PROP_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */



/**
 * NAME: _ix_cc_v6v4_add_tunnel_to_interface
 *
 * DESCRIPTION: This primitive is used to associate a tunnel with an
 * interface.
 *           
 * @Param:  - IN arg_BladeId - blade ID for the interface.
 *
 * @Param:  - IN arg_PortId - port ID for the interface.
 *
 * @Param:  - IN arg_TunnelIndex - index of the start tunnel.
 *
 * @Param:  - IN arg_TrackMtu - indicates whether to maintain the tunnel
 * MTU as the interface MTU.
 *
 * @Param:  - IN arg_TrackBroadcast - indicates whether to maintain the tunnel
 * subnet broadcast address as the interface broadcast address.
 *
 * @Param:  - OUT None.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_ENTRY_NOT_FOUND - no entry in table 
 * 
 */

ix_error _ix_cc_v6v4_add_tunnel_to_interface(
    ix_uint16 arg_BladeId,
    ix_uint16 arg_PortId,
    ix_uint16 arg_TunnelIndex,
    ix_uint8 arg_TrackMtu,
    ix_uint8 arg_TrackBroadcast
    );



/**
 * NAME: _ix_cc_v6v4_remove_tunnel_from_interface
 *
 * DESCRIPTION: This primitive is used to remove a tunnel from an interface.
 *           
 * @Param:  - IN arg_BladeId - blade ID for the interface.
 *
 * @Param:  - IN arg_PortId - port ID for the interface.
 *
 * @Param:  - IN arg_TunnelIndex - index of the start tunnel.
 *
 * @Param:  - OUT None.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_ENTRY_NOT_FOUND - no entry in table 
 * 
 */

ix_error _ix_cc_v6v4_remove_tunnel_from_interface(
    ix_uint16 arg_BladeId,
    ix_uint16 arg_PortId,
    ix_uint16 arg_TunnelIndex
    );



/**
 * NAME: _ix_cc_v6v4_set_track_mtu
 *
 * DESCRIPTION: This primitive is used to enable or disable MTU tracking
 * for a tunnel.
 *           
 * @Param:  - IN arg_BladeId - blade ID for the interface.
 *
 * @Param:  - IN arg_PortId - port ID for the interface.
 *
 * @Param:  - IN arg_TunnelIndex - index of the start tunnel.
 *
 * @Param:  - IN arg_Selection - index of the start tunnel.
 *
 * @Param:  - OUT None.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_ENTRY_NOT_FOUND - no entry in table 
 * 
 */

ix_error _ix_cc_v6v4_set_track_mtu(
    ix_uint16 arg_BladeId,
    ix_uint16 arg_PortId,
    ix_uint16 arg_TunnelIndex,
    ix_uint8 arg_Selection
    );


/**
 * NAME: _ix_cc_v6v4_set_track_broadcast
 *
 * DESCRIPTION: This primitive is used to enable or disable broadcast tracking
 * for a tunnel.
 *           
 * @Param:  - IN arg_BladeId - blade ID for the interface.
 *
 * @Param:  - IN arg_PortId - port ID for the interface.
 *
 * @Param:  - IN arg_TunnelIndex - index of the start tunnel.
 *
 * @Param:  - IN arg_Selection - index of the start tunnel.
 *
 * @Param:  - OUT None.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_ENTRY_NOT_FOUND - no entry in table 
 * 
 */

ix_error _ix_cc_v6v4_set_track_broadcast(
    ix_uint16 arg_BladeId,
    ix_uint16 arg_PortId,
    ix_uint16 arg_TunnelIndex,
    ix_uint8 arg_Selection
    );


/**
 * NAME: _ix_cc_v6v4_get_tunnel_track_info
 *
 * DESCRIPTION: This primitive is used to retrieve the current MTU and
 * broadcast tracking settings for a tunnel.
 *           
 * @Param:  - IN arg_BladeId - blade ID for the interface.
 *
 * @Param:  - IN arg_PortId - port ID for the interface.
 *
 * @Param:  - IN arg_TunnelIndex - index of the start tunnel.
 *
 * @Param:  - OUT arg_pTrackMtu - pointer to location in which to return
 * track MTU setting for the tunnel.
 *
 * @Param:  - OUT arg_pTrackMtu - pointer to location in which to return
 * track broadcast setting for the tunnel.
 *
 * @Param:  - OUT None.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_ENTRY_NOT_FOUND - no entry in table 
 * 
 */

ix_error _ix_cc_v6v4_get_tunnel_track_info(
    ix_uint16 arg_BladeId,
    ix_uint16 arg_PortId,
    ix_uint16 arg_TunnelIndex,
    ix_uint8* arg_pTrackMtu,
    ix_uint8* arg_pTrackBroadcast
    );


/**
 * NAME: _ix_cc_v6v4_update_tunnels_ip_addr
 *
 * DESCRIPTION: This primitive is used to update the local IP address
 * for all tunnels associated with an interface.
 *           
 * @Param:  - IN arg_BladeId - blade ID for the interface.
 *
 * @Param:  - IN arg_PortId - port ID for the interface.
 *
 * @Param:  - IN arg_IpAddr - IP address to be set.
 *
 * @Param:  - OUT None.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_ENTRY_NOT_FOUND - no entry in table 
 * 
 */

ix_error _ix_cc_v6v4_update_tunnels_ip_addr(
    ix_uint16 arg_BladeId,
    ix_uint16 arg_PortId,
    ix_uint32 arg_IpAddr
    );



/**
 * NAME: _ix_cc_v6v4_update_tunnels_mtu
 *
 * DESCRIPTION: This primitive is used to update the path MTU for all
 * tunnels associated with an interface that are configured for MTU tracking.
 *           
 * @Param:  - IN arg_BladeId - blade ID for the interface.
 *
 * @Param:  - IN arg_PortId - port ID for the interface.
 *
 * @Param:  - IN arg_Mtu - MTU to be set.
 *
 * @Param:  - OUT None.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_ENTRY_NOT_FOUND - no entry in table 
 * 
 */

ix_error _ix_cc_v6v4_update_tunnels_mtu(
    ix_uint16 arg_BladeId,
    ix_uint16 arg_PortId,
    ix_uint16 arg_Mtu
    );



/**
 * NAME: _ix_cc_v6v4_update_tunnels_broadcast
 *
 * DESCRIPTION: This primitive is used to update the subnet broadcast
 * address for all tunnels associated with an interface that are configured
 * for broadcast tracking.
 *           
 * @Param:  - IN arg_BladeId - blade ID for the interface.
 *
 * @Param:  - IN arg_PortId - port ID for the interface.
 *
 * @Param:  - IN arg_Broadcast - broadcast address to be set.
 *
 * @Param:  - OUT None.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_ENTRY_NOT_FOUND - no entry in table 
 * 
 */

ix_error _ix_cc_v6v4_update_tunnels_broadcast(
    ix_uint16 arg_BladeId,
    ix_uint16 arg_PortId,
    ix_uint32 arg_Broadcast
    );



/**
 * NAME: _ix_cc_v6v4_get_num_tunnels_for_interface
 *
 * DESCRIPTION: This primitive is used to check how many tunnels are
 * configured on an interface.
 *           
 * @Param:  - IN arg_BladeId - blade ID for the interface.
 *
 * @Param:  - IN arg_PortId - port ID for the interface.
 *
 * @Param:  - OUT arg_pNumTunnels - pointer to location in which to return
 * the number of tunnels.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_INVALID_INPUT_PARAM - blade or port invalid 
 * IX_CC_ERROR_NULL - arg_NumTunnels is NULL 
 * 
 */

ix_error _ix_cc_v6v4_get_num_tunnels_for_interface(
    ix_uint16 arg_BladeId,
    ix_uint16 arg_PortId,
    ix_uint16* arg_pNumTunnels
    );


/**
 * NAME: _ix_cc_v6v4_free_tunnel_interface_info
 *
 * DESCRIPTION: This primitive is used to free all resources allocated
 * to maintain the tunnel-interface associations.
 *           
 * @Param:  - IN None.
 *
 * @Param:  - OUT None.
 *
 * @Return: IX_SUCCESS.
 *
 * Error Codes:
 * 
 */

ix_error _ix_cc_v6v4_free_tunnel_interface_info(void);



#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_V6V4_TUNNEL_PROP_H__) */
