/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      IPv6/v4 Tunneling Core Component - fini
 *
 * = FILENAME
 *      ix_cc_v6v4_fini.c
 *
 * = DESCRIPTION
 *      The file defines fini interface and internal functions.
 *
 * = CREATION TIME
 *      11/15/2002 10:45 AM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */

#define IX_ERROR_FILE_IDENT "$Id: ix_cc_v6v4_fini.c,v 1.6 2003/11/12 02:49:48 rranjeet Exp $";



/**
 * System defined include files
 */


/**
 * User defined include files
 */

#include "ix_cc_error.h"

#include "ix_rm.h"
#include "ix_ossl.h"

#include "ix_cc.h"
#include "bindings.h"
#include "cc/ix_cc_v6v4.h"
#include "cc/v6v4_tunnel/internal/ix_cc_v6v4_init.h"
#include "cc/v6v4_tunnel/internal/ix_cc_v6v4_fini.h" 
#include "cc/v6v4_tunnel/internal/ix_cc_v6v4_tunnel_prop.h" 






/**
 * Preprocessor symbols and Macros used in this file
 */



/**
 * Type definitions whose scope is limited to this file
 */


/**
 * Variable declarations global to this file only. Externs are followed by 
 * static variables.
 */


/** 
 * Extern function prototypes.
 */

extern ix_cc_v6v4_context* g_pV6v4Context;


/** 
 * Static function prototypes.
 */


/** 
 * Function definitions.
 */



/**
 * NAME: _ix_cc_v6v4_cleanup_reassembly
 *
 * DESCRIPTION: This primitive will be used to release resources
 * allocated for reassembly.
 *
 * @Param:  - IN 
 *
 * @Param:  - OUT 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_INTERNAL - error from CCI
 * 
 */

ix_error _ix_cc_v6v4_cleanup_reassembly(void)
{

#if !defined(IX_EXCLUDE_CCI)
    IX_ERROR_CRT(ix_cci_cc_remove_event_handler(
                    g_pV6v4Context->hCcHandle, g_pV6v4Context->hTimerEvent),
                 IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_WARNING);
#endif

    IX_ERROR_CR(_ix_cc_v6v4_reasm_cleanup(g_pV6v4Context->hReassembly));

    return IX_SUCCESS;
} /* end - _ix_cc_v6v4_cleanup_reassembly */


/**
 * NAME: _ix_cc_v6v4_unregister_message_handler
 *
 * DESCRIPTION: This primitive is used to unregister the message handler. 
 * It calls CCI services in level 1 system and resource manager services in
 * level 0 system.
 *
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_CCI - error from CCI
 * 
 */

ix_error _ix_cc_v6v4_unregister_message_handler(void)
{
    /* Call CCI if CCI is enabled */

    #if defined(IX_EXCLUDE_CCI)
        IX_ERROR_CRT(ix_rm_message_handler_unregister(IX_CC_V6V4_MSG_INPUT),
                     IX_CC_V6V4_ERROR_CCI,
                     IX_ERROR_LEVEL_WARNING);
    #else
        IX_ERROR_CRT(ix_cci_cc_remove_message_handler(IX_CC_V6V4_MSG_INPUT),
                     IX_CC_V6V4_ERROR_CCI,
                     IX_ERROR_LEVEL_WARNING);
    #endif

    return IX_SUCCESS;
}/* end _ix_cc_v6v4_unregister_message_handler */



/**
 * NAME: _ix_cc_v6v4_unregister_cci_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to unregister packet handler with CCI. 
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_CCI - error from CCI
 * 
 */

ix_error _ix_cc_v6v4_unregister_cci_packet_handlers(void)
{

#if !defined(IX_EXCLUDE_CCI)

    IX_ERROR_CRT(
        ix_cci_cc_remove_packet_handler(IX_CC_V6V4_PKT_MICROBLOCK_INPUT),
        IX_CC_V6V4_ERROR_CCI,
        IX_ERROR_LEVEL_WARNING);

    IX_ERROR_CRT(
        ix_cci_cc_remove_packet_handler(IX_CC_V6V4_PKT_IPV6_INPUT),
        IX_CC_V6V4_ERROR_CCI,
        IX_ERROR_LEVEL_WARNING);

#endif

    return IX_SUCCESS;

}/* end _ix_cc_v6v4_unregister_cci_packet_handlers */


/**
 * NAME: _ix_cc_v6v4_unregister_rm_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to unregister packet handler with 
 * resource manager
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_RM - error from RM
 * 
 */

ix_error _ix_cc_v6v4_unregister_rm_packet_handlers(void)
{

    IX_ERROR_CRT(
        ix_rm_packet_handler_unregister(IX_CC_V6V4_PKT_MICROBLOCK_INPUT),
        IX_CC_V6V4_ERROR_RM,
        IX_ERROR_LEVEL_WARNING);

    IX_ERROR_CRT(
        ix_rm_packet_handler_unregister(IX_CC_V6V4_PKT_IPV6_INPUT),
        IX_CC_V6V4_ERROR_RM,
        IX_ERROR_LEVEL_WARNING);

    return IX_SUCCESS;

}/* end _ix_cc_v6v4_unregister_rm_packet_handlers */



/**
 * NAME: _ix_cc_v6v4_unregister_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to unregister packet handler 
 * with CCI/Resource manager
 *
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_CCI - error from CCI/RM
 * 
 */

ix_error _ix_cc_v6v4_unregister_packet_handlers(void)
{
 
    /* Call CCI if CCI is enabled */
    #if defined(IX_EXCLUDE_CCI)
        IX_ERROR_CR(_ix_cc_v6v4_unregister_rm_packet_handlers());
    #else
        IX_ERROR_CR(_ix_cc_v6v4_unregister_cci_packet_handlers());
    #endif

    return IX_SUCCESS;

}/* end _ix_cc_v6v4_unregister_packet_handlers */


/**
 * NAME: _ix_cc_v6v4_end_tunnel_cleanup
 *
 * DESCRIPTION: This primitive is used to release the resources that were
 * allocated to manage end tunnel configuration.
 *           
 * @Param:  - IN None.
 *
 * @Param:  - OUT None.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_INTERNAL - internal error occurred 
 * 
 */

ix_error _ix_cc_v6v4_end_tunnel_cleanup(void)
{
    /* Free shared memory */

    IX_ERROR_CRT(
        ix_rm_mem_free(g_pV6v4Context->pMbEndTunnelBase),
        IX_CC_ERROR_INTERNAL,
        IX_ERROR_LEVEL_LOCAL);

    g_pV6v4Context->pMbEndTunnelBase = NULL;

    IX_ERROR_CRT(
        ix_cc_index_pool_fini(g_pV6v4Context->endTunnelIndexPool),
        IX_CC_ERROR_INTERNAL,
        IX_ERROR_LEVEL_WARNING);

    g_pV6v4Context->endTunnelIndexPool = NULL;

    IX_ERROR_CR(
        (*g_pV6v4Context->ingressListHandlers.cleanup)(
            &g_pV6v4Context->islInfo)
    );

    g_pV6v4Context->islInfo.pMbIslBase = NULL;
    g_pV6v4Context->islInfo.pIslContext = NULL;

    return IX_SUCCESS;

}/* end _ix_cc_v6v4_end_tunnel_cleanup */


/**
 * NAME: _ix_cc_v6v4_start_tunnel_cleanup
 *
 * DESCRIPTION: This primitive is used to release the resources that were
 * allocated to manage start tunnel configuration.
 *           
 * @Param:  - IN None.
 *
 * @Param:  - OUT None.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_INTERNAL - internal error occurred 
 * 
 */

ix_error _ix_cc_v6v4_start_tunnel_cleanup(void)
{
    /* Free shared memory */

    IX_ERROR_CRT(
        ix_rm_mem_free(g_pV6v4Context->pMbStartTunnelBase),
        IX_CC_ERROR_INTERNAL,
        IX_ERROR_LEVEL_LOCAL);

    g_pV6v4Context->pMbStartTunnelBase = NULL;

    IX_ERROR_CRT(
        ix_cc_index_pool_fini(g_pV6v4Context->startTunnelIndexPool),
        IX_CC_ERROR_INTERNAL,
        IX_ERROR_LEVEL_WARNING);

    g_pV6v4Context->startTunnelIndexPool = NULL;

    /* Free interface tracking information */

    _ix_cc_v6v4_free_tunnel_interface_info();

    return IX_SUCCESS;

}/* end _ix_cc_v6v4_start_tunnel_cleanup */





/**
 * NAME: _ix_cc_v6v4_free_counters
 *
 * DESCRIPTION: This primitive is used to free memory allocated for
 * microblock counters.         .
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_INTERNAL - failed to free memory for symbols.
 * 
 */

ix_error _ix_cc_v6v4_free_counters(void)
{

    IX_ERROR_CRT(
        ix_rm_mem_free(g_pV6v4Context->pMbDecapStatsBase),
        IX_CC_ERROR_INTERNAL,
        IX_ERROR_LEVEL_LOCAL);

    g_pV6v4Context->pMbDecapStatsBase = NULL;
    g_pV6v4Context->pMbEncapStatsBase = NULL;

    return IX_SUCCESS;

}/* end _ix_cc_v6v4_free_counters */


/**
 * NAME: ix_cc_v6v4_fini
 *
 * DESCRIPTION: This primitive is called to terminate services from the
 * tunneling core component. This primitive frees all memory allocated during
 * initialization, frees all created resources and unregisters all message
 * and packet handlers. If the client requests any services from IPv6/v4
 * tunneling after calling this function, then the behavior is undefined.
 * 
 * @Param:  - IN arg_hCcHandle - handle to tunneling core component
 *
 * @Param:  - IN arg_pContext - pointer to the context information allocated 
 * earlier in ix_cc_v6v4_init(). 
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_INVALID_INPUT_PARAM - invalid input parameter(s)
 * IX_CC_ERROR_INTERNAL - failure of memory deletion                
 * IX_CC_V6V4_ERROR_CCI - failure from CCI
 * 
 */

ix_error ix_cc_v6v4_fini(
    ix_cc_handle arg_hCcHandle,
    void* arg_pContext
    )
{

    /* Validate Parameters */
    if(arg_pContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                            ("arg_pContext is null"));

    }/* if(arg_pContext == NULL)  */ 


    if((arg_hCcHandle != g_pV6v4Context->hCcHandle) ||
       (ix_cc_v6v4_context *)arg_pContext != g_pV6v4Context)
    {
        return IX_ERROR_WARNING(IX_CC_V6V4_ERROR_INVALID_INPUT_PARAM,
                            ("Invalid input Parameters"));
    }/* if(arg_hCcHandle != g_pV6v4Context->hV6v4Handle ||
       (ix_cc_v6v4_context *)arg_pContext != g_pV6v4Context) */ 

    /* Free resources for end tunnels */
    IX_ERROR_CR(_ix_cc_v6v4_end_tunnel_cleanup());

    /* Free resources for start tunnels */
    IX_ERROR_CR(_ix_cc_v6v4_start_tunnel_cleanup());

    /* Free memory for counters */
    IX_ERROR_CR(_ix_cc_v6v4_free_counters());

    /* Free resources for reassembly */
    IX_ERROR_CR(_ix_cc_v6v4_cleanup_reassembly());

    /* Unregister Message Handler */
    IX_ERROR_CR(_ix_cc_v6v4_unregister_message_handler());
  
    /* Unregister Packet Handlers */
    IX_ERROR_CR(_ix_cc_v6v4_unregister_packet_handlers());

    /* Free mutex */
    IX_ERROR_CR(ix_ossl_mutex_fini(g_pV6v4Context->v6v4Mutex));

    /* Free tunneling context */ 
    ix_ossl_free(g_pV6v4Context);
        
    return IX_SUCCESS;

} /* end ix_cc_v6v4_fini */
