/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      IPv6/v4 Tunneling Core Component - CCI - init function
 *
 * = FILENAME
 *      ix_cc_v6v4_init.c
 *
 * = DESCRIPTION
 *      The file contains the initialization functions for the tunneling
 *      core component.
 *
 * = CREATION TIME
 *      11/14/2002 10:34 AM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */
#define IX_ERROR_FILE_IDENT "$Id: ix_cc_v6v4_init.c,v 1.9 2003/11/12 02:50:51 rranjeet Exp $";



/**
 * System defined include files
 */

/**
 * User defined include files
 */
#include "ix_cc_error.h"

#include "ix_rm.h"
#include "ix_ossl.h"


#include "ix_cc.h"
#include "ix_cc_macros.h"
#include "ix_config.h"
#include "bindings.h"
#include "cc/ix_cc_v6v4.h"
#include "cc/v6v4_tunnel/internal/ix_cc_v6v4_init.h"
#include "cc/v6v4_tunnel/internal/ix_cc_v6v4_fini.h" 
#include "cc/v6v4_tunnel/internal/ix_cc_v6v4_config.h" 
#include "cc/v6v4_tunnel/internal/ix_cc_v6v4_ingress_list_def.h" 
#include "cc/v6v4_tunnel/internal/ix_cc_v6v4_ingress_list_trie.h" 
#include "cc/v6v4_tunnel/internal/ix_cc_v6v4_prop.h" 

/**
 * Preprocessor symbols and Macros used in this file
 */

/**
 * Type definitions whose scope is limited to this file
 */

/**
 * Variable declarations global to this file only. Externs are followed by 
 * static variables.
 */

ix_cc_v6v4_context* g_pV6v4Context;
ix_cc_v6v4_registry_data g_V6v4RegistryData;

/** 
 * Extern function prototypes.
 */


/** 
 * Static function prototypes.
 */


/** 
 * Function definitions.
 */


/**
 * NAME: _ix_cc_v6v4_reassembly_event_handler
 *
 * DESCRIPTION: This primitive is used to handle the reassembly timer events.
 * 
 * @Param:  - IN arg_pContext - pointer to core component context 
 *
 * @Param:  - OUT None 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 */

ix_error _ix_cc_v6v4_reassembly_event_handler(
    void* arg_pContext
    )
{
    ix_cc_v6v4_context* pContext = (ix_cc_v6v4_context*)arg_pContext;

    return _ix_cc_v6v4_reassembly_tick(pContext->hReassembly);
} /* end - _ix_cc_v6v4_reassembly_event_handler */


/**
 * NAME: _ix_cc_v6v4_retrieve_registry_data
 *
 * DESCRIPTION: This primitive will be used to retrieve data from registry 
 * for a given property type.
 *           .
 * 
 * @Param:  - IN arg_pPropertyName - pointer to property name string 
 *
 * @Param:  - OUT arg_pPropertyValue - pointer to property value 
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV4_ERROR_REGISTRY  - failed to access registry
 * 
 */

ix_error _ix_cc_v6v4_retrieve_registry_data(
    const char* arg_pPropertyName,
    ix_uint32* arg_pPropertyValue)
{
    ix_configuration_property_handle hV6v4PropertyHandle; 

    /* Get static data from registry */    
    IX_ERROR_CRT(
        ix_rm_cp_property_open(IX_CP_CORE_PROPERTY_ROOT,
                               arg_pPropertyName,
                               &hV6v4PropertyHandle),
        IX_CC_V6V4_ERROR_REGISTRY,
        IX_ERROR_LEVEL_WARNING);

    IX_ERROR_CRT(
        ix_rm_cp_property_get_value_uint32(hV6v4PropertyHandle,
                                           arg_pPropertyValue),
        IX_CC_V6V4_ERROR_REGISTRY,
        IX_ERROR_LEVEL_WARNING);

    IX_ERROR_CRT(
        ix_rm_cp_property_close(hV6v4PropertyHandle),
        IX_CC_V6V4_ERROR_REGISTRY,
        IX_ERROR_LEVEL_WARNING);

    return IX_SUCCESS;
}/* end _ix_cc_v6v4_retrieve_registry_data */



/**
 * NAME: _ix_cc_v6v4_registry
 *
 * DESCRIPTION: This primitive will be used to retreive data from registry for
 * all IPV4 static data and stored in g_V6v4RegistryData 
 *           .
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT  none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_REGISTRY  - not able to access registry.
 * 
 */

ix_error _ix_cc_v6v4_registry(void)
{
       
    /* Get end tunnel size from registry */
    IX_ERROR_CR(
        _ix_cc_v6v4_retrieve_registry_data(
            IX_CC_V6V4_REGISTRY_KEY_END_TUNNEL_SIZE,
            &g_V6v4RegistryData.endTunnelSize)
    );

    /* Get start tunnel size from registry */
    IX_ERROR_CR(
        _ix_cc_v6v4_retrieve_registry_data(
            IX_CC_V6V4_REGISTRY_KEY_START_TUNNEL_SIZE,
            &g_V6v4RegistryData.startTunnelSize)
    );

    /* Get ingress list type from registry */
    IX_ERROR_CR(
        _ix_cc_v6v4_retrieve_registry_data(
            IX_CC_V6V4_REGISTRY_KEY_INGRESS_LIST_TYPE,
            (ix_uint32 *)&g_V6v4RegistryData.ingressListFormat)
    );

    /* Get ingress list size from registry */
    IX_ERROR_CR(
        _ix_cc_v6v4_retrieve_registry_data(
            IX_CC_V6V4_REGISTRY_KEY_INGRESS_LIST_SIZE,
            &g_V6v4RegistryData.ingressListSizeHint)
    );

    /* Get end tunnel memory type from registry */
    IX_ERROR_CR(
        _ix_cc_v6v4_retrieve_registry_data(
            IX_CC_V6V4_REGISTRY_KEY_END_TUNNEL_MEMORY_TYPE,
            (ix_uint32 *)&g_V6v4RegistryData.endTunnelMemType)
    );

    /* Get start tunnel memory type from registry */
    IX_ERROR_CR(
        _ix_cc_v6v4_retrieve_registry_data(
            IX_CC_V6V4_REGISTRY_KEY_START_TUNNEL_MEMORY_TYPE,
            (ix_uint32 *)&g_V6v4RegistryData.startTunnelMemType)
    );

    /* Get ingress list memory type from registry */
    IX_ERROR_CR(
        _ix_cc_v6v4_retrieve_registry_data(
            IX_CC_V6V4_REGISTRY_KEY_INGRESS_LIST_MEMORY_TYPE,
            (ix_uint32 *)&g_V6v4RegistryData.ingressListMemType)
    );

    /* Get end tunnel memory channel from registry */
    IX_ERROR_CR(
        _ix_cc_v6v4_retrieve_registry_data(
            IX_CC_V6V4_REGISTRY_KEY_END_TUNNEL_CHANNEL,
            &g_V6v4RegistryData.endTunnelChannel)
    );

    /* Get start tunnel memory channel from registry */
    IX_ERROR_CR(
        _ix_cc_v6v4_retrieve_registry_data(
            IX_CC_V6V4_REGISTRY_KEY_START_TUNNEL_CHANNEL,
            &g_V6v4RegistryData.startTunnelChannel)
    );

    /* Get ingress list memory channel from registry */
    IX_ERROR_CR(
        _ix_cc_v6v4_retrieve_registry_data(
            IX_CC_V6V4_REGISTRY_KEY_INGRESS_LIST_CHANNEL,
            &g_V6v4RegistryData.ingressListChannel)
    );


    /* Get micro engine numbers from registry */
    IX_ERROR_CR(
        _ix_cc_v6v4_retrieve_registry_data(
            IX_CC_V6V4_REGISTRY_KEY_ME_FOR_V6V4,
            &g_V6v4RegistryData.meNumbers)
    );
   
    return IX_SUCCESS;
}/* end _ix_cc_v6v4_registry */


/**
 * NAME: _ix_cc_v6v4_patch_symbols
 *
 * DESCRIPTION: This primitive will be used to patch symbols to the tunneling
 * microblocks. 
 * 
 * @Param:  - IN  none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * IX_CC_V6V4_ERROR_FAILED_PATCHING - failed patching by calling resource
 * manager function
 * 
 */

ix_error _ix_cc_v6v4_patch_symbols(void)
{
    ix_imported_symbol v6v4Symbols[IX_CC_V6V4_NUMBER_OF_SYMBOLS]; 
    ix_uint32 v6v4Index;
    ix_memory_type memType;
    ix_uint32 memChannel;
    ix_uint32 memOffset;
    void* pPhysAddr;
    ix_uint32 count = 0;

#if defined(V6V4_DECAP_COUNTERS)
    /* Patch base address for decap statistics */
    IX_ERROR_CRT(
        ix_rm_get_phys_offset(g_pV6v4Context->pMbDecapStatsBase,
                              &memType,
                              &memChannel,
                              &memOffset,
                              &pPhysAddr),
        IX_CC_V6V4_ERROR_FAILED_PATCHING,
        IX_ERROR_LEVEL_WARNING);

    if (memType == IX_MEMORY_TYPE_SRAM)
    {
        IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(memChannel,memOffset);
    }/* end if memory type */

    v6v4Symbols[count].m_Value = memOffset; 
    v6v4Symbols[count].m_Name = IX_CC_V6V4_PATCHNAME_DECAP_STATS;
    count++;
#endif

#if defined(V6V4_ENCAP_COUNTERS)
    /* Patch base address for encap statistics */
    IX_ERROR_CRT(
        ix_rm_get_phys_offset(g_pV6v4Context->pMbEncapStatsBase,
                              &memType,
                              &memChannel,
                              &memOffset,
                              &pPhysAddr),
        IX_CC_V6V4_ERROR_FAILED_PATCHING,
        IX_ERROR_LEVEL_WARNING);

    if (memType == IX_MEMORY_TYPE_SRAM)
    {
        IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(memChannel,memOffset);
    }/* end if memory type */ 

    v6v4Symbols[count].m_Value = memOffset;
    v6v4Symbols[count].m_Name = IX_CC_V6V4_PATCHNAME_ENCAP_STATS;
    count++;
#endif

    /* Patch base address for end tunnel array */
    IX_ERROR_CRT(
        ix_rm_get_phys_offset(g_pV6v4Context->pMbEndTunnelBase,
                              &memType,
                              &memChannel,
                              &memOffset,
                              &pPhysAddr),
        IX_CC_V6V4_ERROR_FAILED_PATCHING,
        IX_ERROR_LEVEL_WARNING);

    if (memType == IX_MEMORY_TYPE_SRAM)
    {
        IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(memChannel,memOffset);
    }/* end if memory type */ 

    v6v4Symbols[count].m_Value = memOffset;
    v6v4Symbols[count].m_Name = IX_CC_V6V4_PATCHNAME_END_TUNNEL_TABLE;
    count++;

    /* Patch base address for start tunnel array */
    IX_ERROR_CRT(
        ix_rm_get_phys_offset(g_pV6v4Context->pMbStartTunnelBase,
                              &memType,
                              &memChannel,
                              &memOffset,
                              &pPhysAddr),
        IX_CC_V6V4_ERROR_FAILED_PATCHING,
        IX_ERROR_LEVEL_WARNING);

    if (memType == IX_MEMORY_TYPE_SRAM)
    {
        IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(memChannel,memOffset);
    }/* end if memory type */ 

    v6v4Symbols[count].m_Value = memOffset;
    v6v4Symbols[count].m_Name = IX_CC_V6V4_PATCHNAME_START_TUNNEL_TABLE;
    count++;

    /* Patch base address for ingress source list */
    IX_ERROR_CRT(
        ix_rm_get_phys_offset(g_pV6v4Context->islInfo.pMbIslBase,
                              &memType,
                              &memChannel,
                              &memOffset,
                              &pPhysAddr),
        IX_CC_V6V4_ERROR_FAILED_PATCHING,
        IX_ERROR_LEVEL_WARNING);

    if (memType == IX_MEMORY_TYPE_SRAM)
    {
        IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(memChannel,memOffset);
    }/* end if memory type */ 

    v6v4Symbols[count].m_Value = memOffset;
    v6v4Symbols[count].m_Name = IX_CC_V6V4_PATCHNAME_INGRESS_LIST;
    count++;

    for (v6v4Index = 0; v6v4Index < IX_CC_V6V4_NUMBER_OF_MICROENGINES;
         v6v4Index++)
    {
         /* if the bit is set, then patch the symbols */
        if(((g_V6v4RegistryData.meNumbers) & (1 << v6v4Index)) != 0)
        {
            IX_ERROR_CRT(
                ix_rm_ueng_patch_symbols(
                    g_aMicroengineNumberMapping[v6v4Index],
                    count, v6v4Symbols),
                IX_CC_V6V4_ERROR_FAILED_PATCHING,
                IX_ERROR_LEVEL_WARNING);
        }/* end if */
    }/* end for */


    return IX_SUCCESS;
}/* end _ix_cc_v6v4_patch_symbols */




/**
 * NAME: _ix_cc_v6v4_alloc_shared_memory
 *
 * DESCRIPTION: This primitive is used to allocate a region of shared memory.
 * 
 * @Param:  - IN arg_MemSize: Total size of the table in bytes.
 * @Param:  - IN arg_MemType: Type of memory.
 * @Param:  - IN arg_MemChannel: Memory channel.
 *
 * @Param:  - OUT arg_ppTableBase: Location into which to return table
 * address.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOM  - not able to allocate memory
 * 
 */

ix_error _ix_cc_v6v4_alloc_shared_memory(
    ix_uint32 arg_MemSize,
    ix_cc_v6v4_mem_type arg_MemType,
    ix_uint32 arg_MemChannel,
    void** arg_ppTableBase
    )
{
    ix_error err = IX_SUCCESS;
    ix_memory_type rmMemType;

    switch (arg_MemType)
    {
        case IX_CC_V6V4_MEM_SRAM:
            rmMemType   = IX_MEMORY_TYPE_SRAM;
            break;

        case IX_CC_V6V4_MEM_DRAM:
            rmMemType   = IX_MEMORY_TYPE_DRAM;
            break;

        default:
            return IX_ERROR_WARNING(IX_CC_ERROR_RANGE,
                ("Invalid memory type: %d", arg_MemType));
    }

    /*
     * Allocate the shared memory (either SRAM or DRAM) from the Resource
     * Manager.
     */
    err = ix_rm_mem_alloc(
        rmMemType,
        arg_MemChannel,
        arg_MemSize,
        arg_ppTableBase);

    if (err != IX_SUCCESS)
    {
        switch ((int) IX_ERROR_GET_CODE(err))
        {
            case IX_RM_ERROR_ASSERTION:
                err = IX_ERROR_CHAIN(err, IX_CC_ERROR_RANGE, 0,
                    ("Invalid channel: %d", arg_MemChannel));
                break;

            case IX_RM_ERROR_CANNOT_ALLOCATE_SIZE:
                err = IX_ERROR_CHAIN(err, IX_CC_ERROR_OOM, 0,
                    ("Out of %s memory\n"
                     "\tCould not allocate %d bytes from the RM",
                     ((rmMemType == IX_MEMORY_TYPE_SRAM) ? "SRAM" : "DRAM"),
                     arg_MemSize));
                break;

            default:
                err = IX_ERROR_CHAIN(err, IX_CC_ERROR_INTERNAL, 0,
                    ("Could not allocate %d bytes from the RM", arg_MemSize));
                break;
        }
        return err;
    }

    if (*arg_ppTableBase == NULL)
    {
        return IX_ERROR_LOCAL(IX_CC_ERROR_INTERNAL,
            ("Could not allocate %d bytes from the RM", arg_MemSize));
    }

    ix_ossl_memset(*arg_ppTableBase, 0, arg_MemSize);

    return err;

}/* end _ix_cc_v6v4_alloc_shared_memory */



/**
 * NAME: _ix_cc_v6v4_alloc_shared_table
 *
 * DESCRIPTION: This primitive is used to allocate a table in shared memory. 
 * 
 * @Param:  - IN arg_NumEntries: Number of entries in the table.
 * @Param:  - IN arg_EntrySize: Size of each entry in bytes.
 * @Param:  - IN arg_MemType: Type of memory.
 * @Param:  - IN arg_MemChannel: Memory channel.
 *
 * @Param:  - OUT arg_ppTableBase: Location into which to return table
 * address.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOM  - not able to allocate memory
 * 
 */

ix_error _ix_cc_v6v4_alloc_shared_table(
    ix_uint32 arg_NumEntries,
    ix_uint32 arg_EntrySize,
    ix_cc_v6v4_mem_type arg_MemType,
    ix_uint32 arg_MemChannel,
    void** arg_ppTableBase
    )
{
    ix_error err = IX_SUCCESS;
    ix_uint32 memSize;

    IX_ERROR_CHECK_ARG_RANGE(1, arg_NumEntries, 2UL, 65535UL, "%ld");

    memSize = arg_NumEntries * arg_EntrySize;

    IX_ERROR_CR(
        _ix_cc_v6v4_alloc_shared_memory(memSize, arg_MemType, arg_MemChannel,
                                        arg_ppTableBase));


    return err;

}/* end _ix_cc_v6v4_alloc_shared_table */



/**
 * NAME: _ix_cc_v6v4_init_reassembly
 *
 * DESCRIPTION: This primitive will be used to initialize the resources
 * for reassembly of tunneled packets. 
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT  none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOR - Out of resources
 * IX_CC_ERROR_OOM - Out of memory
 * IX_CC_ERROR_INTERNAL - Error from CCI
 * 
 */

ix_error _ix_cc_v6v4_init_reassembly(void)
{
    ix_error err;

    IX_ERROR_CR(_ix_cc_v6v4_reasm_init(
                    IX_CC_V6V4_REASSEMBLY_TIMEOUT,
                    g_pV6v4Context->hBufferFreeList,
                    &g_pV6v4Context->hReassembly));
    
    /* Register event handler if CCI is enabled */

#if !defined(IX_EXCLUDE_CCI)
    IX_ERROR_CGT(ix_cci_cc_add_event_handler_ex(
                    g_pV6v4Context->hCcHandle,
                    IX_CC_V6V4_REASSEMBLY_TICK,
                    _ix_cc_v6v4_reassembly_event_handler,
                    IX_EVENT_TYPE_PERIODIC,
                    IX_CC_V6V4_REASSEMBLY_PRIORITY,
                    g_pV6v4Context,
                    &g_pV6v4Context->hTimerEvent),
                err,
                IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_WARNING,
                cleanupReasm);
#endif

    return IX_SUCCESS;

cleanupReasm:
    _ix_cc_v6v4_reasm_cleanup(g_pV6v4Context->hReassembly);
    return err;

}/* end _ix_cc_v6v4_init_reassembly */



/**
 * NAME: _ix_cc_v6v4_register_message_handler
 *
 * DESCRIPTION: This primitive will be used to register message handler. 
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT  none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_CCI - Error from CCI
 * 
 */

ix_error _ix_cc_v6v4_register_message_handler(void)
{
    
    /* Call CCI message handler if CCI is enabled */
    #if defined(IX_EXCLUDE_CCI)
        IX_ERROR_CRT(
            ix_rm_message_handler_register(IX_CC_V6V4_MSG_INPUT,
                                           ix_cc_v6v4_msg_handler,
                                           g_pV6v4Context),
            IX_CC_V6V4_ERROR_CCI,
            IX_ERROR_LEVEL_WARNING);
    #else
        IX_ERROR_CRT(
            ix_cci_cc_add_message_handler(g_pV6v4Context->hCcHandle,
                                          IX_CC_V6V4_MSG_INPUT,
                                          ix_cc_v6v4_msg_handler,
                                          IX_INPUT_TYPE_MULTI_SRC),
            IX_CC_V6V4_ERROR_CCI,
            IX_ERROR_LEVEL_WARNING);

    #endif

    return IX_SUCCESS;
}/* end _ix_cc_v6v4_register_message_handler */



/**
 * NAME: _ix_cc_v6v4_register_cci_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to register packet handlers
 * with CCI. 
 * @Param:  - IN  none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_CCI - error from CCI
 * 
 */

ix_error _ix_cc_v6v4_register_cci_packet_handlers(void)
{
    ix_error err = IX_SUCCESS;

#if !defined(IX_EXCLUDE_CCI)


    IX_ERROR_CRT(
        ix_cci_cc_add_packet_handler(g_pV6v4Context->hCcHandle,
                                     IX_CC_V6V4_PKT_MICROBLOCK_INPUT,
                                     ix_cc_v6v4_microblock_pkt_handler,
                                     IX_INPUT_TYPE_SINGLE_SRC),
        IX_CC_V6V4_ERROR_CCI,
        IX_ERROR_LEVEL_WARNING);


    IX_ERROR_CGT(
        ix_cci_cc_add_packet_handler(g_pV6v4Context->hCcHandle,
                                     IX_CC_V6V4_PKT_IPV6_INPUT,
                                     ix_cc_v6v4_ipv6_pkt_handler,
                                     IX_INPUT_TYPE_MULTI_SRC),
        err,
        IX_CC_V6V4_ERROR_CCI,
        IX_ERROR_LEVEL_WARNING,
        cleanupMbHandler);


    IX_ERROR_CGT(
        ix_cci_cc_add_packet_handler(g_pV6v4Context->hCcHandle,
                                     IX_CC_V6V4_PKT_IPV4_INPUT,
                                     ix_cc_v6v4_ipv4_pkt_handler,
                                     IX_INPUT_TYPE_MULTI_SRC),
        err,
        IX_CC_V6V4_ERROR_CCI,
        IX_ERROR_LEVEL_WARNING,
        cleanupIpv6Handler);

    return IX_SUCCESS;

cleanupIpv6Handler:
    ix_cci_cc_remove_packet_handler(IX_CC_V6V4_PKT_IPV6_INPUT);

cleanupMbHandler:
    ix_cci_cc_remove_packet_handler(IX_CC_V6V4_PKT_MICROBLOCK_INPUT);

#endif

    return err;


}/* end _ix_cc_v6v4_register_cci_packet_handlers */


/**
 * NAME: _ix_cc_v6v4_register_rm_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to register packet handlers with
 * the resource manager.
 *
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_RM - error from resource manager
 * 
 */

ix_error _ix_cc_v6v4_register_rm_packet_handlers(void)
{
    ix_error err;

    IX_ERROR_CRT(
        ix_rm_message_handler_register(IX_CC_V6V4_PKT_MICROBLOCK_INPUT,
                                       ix_cc_v6v4_microblock_pkt_handler,
                                       g_pV6v4Context),
        IX_CC_V6V4_ERROR_RM,
        IX_ERROR_LEVEL_WARNING);


    IX_ERROR_CGT(
        ix_rm_packet_handler_register(IX_CC_V6V4_PKT_IPV6_INPUT,
                                      ix_cc_v6v4_ipv6_pkt_handler,
                                      g_pV6v4Context),
        err,
        IX_CC_V6V4_ERROR_RM,
        IX_ERROR_LEVEL_WARNING,
        cleanupMbHandler);


    IX_ERROR_CGT(
        ix_rm_packet_handler_register(IX_CC_V6V4_PKT_IPV4_INPUT,
                                      ix_cc_v6v4_ipv4_pkt_handler,
                                      g_pV6v4Context),
        err,
        IX_CC_V6V4_ERROR_RM,
        IX_ERROR_LEVEL_WARNING,
        cleanupIpv6Handler);


    return IX_SUCCESS;

cleanupIpv6Handler:
    ix_rm_packet_handler_unregister(IX_CC_V6V4_PKT_IPV6_INPUT);

cleanupMbHandler:
    ix_rm_packet_handler_unregister(IX_CC_V6V4_PKT_MICROBLOCK_INPUT);
    return err;

}/* end _ix_cc_v6v4_register_rm_packet_handlers */



/**
 * NAME: _ix_cc_v6v4_register_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to register packet handlers. 
 * It registers with CCI in level 1 system and register with RM in
 * level 0 system.
 * 
 *           .
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_V6V4_ERROR_CCI - error from CCI/resource manager
 * 
 */

ix_error _ix_cc_v6v4_register_packet_handlers(void)
{
    /* Call CCI packet handler if CCI is enabled */
    #if defined(IX_EXCLUDE_CCI)
        IX_ERROR_CR(_ix_cc_v6v4_register_rm_packet_handlers());
    #else
        IX_ERROR_CR(_ix_cc_v6v4_register_cci_packet_handlers());
    #endif

    return IX_SUCCESS;
}/* end _ix_cc_v6v4_register_packet_handlers */


/**
 * NAME: _ix_cc_v6v4_end_tunnel_setup
 *
 * DESCRIPTION: This primitive is used to allocate the resources needed
 * to manage end tunnel configuration.
 *           
 * @Param:  - IN None.
 *
 * @Param:  - OUT None.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOR - Out of resources 
 * IX_CC_ERROR_INTERNAL - internal error occurred 
 * 
 */

ix_error _ix_cc_v6v4_end_tunnel_setup(void)
{
    ix_error err;

    /* Allocate shared memory for end tunnel next hop table */
    IX_ERROR_CR(
        _ix_cc_v6v4_alloc_shared_table(
                g_V6v4RegistryData.endTunnelSize,
                sizeof(ix_cc_v6v4_end_tunnel),
                g_V6v4RegistryData.endTunnelMemType,
                g_V6v4RegistryData.endTunnelChannel,
                (void**)&g_pV6v4Context->pMbEndTunnelBase)
    );

    /* Allocate index pool for end tunnel next hop table */  
    IX_ERROR_CG(
        ix_cc_index_pool_init(
            g_V6v4RegistryData.endTunnelSize, 1,
            &g_pV6v4Context->endTunnelIndexPool),
        err,
        cleanupMem);

    /* Allocate resources for ingress source validation list */
    if (g_V6v4RegistryData.ingressListFormat < IX_CC_V6V4_ISL_DEFAULT_LAST)
    {
        IX_ERROR_CG(
            _ix_cc_v6v4_ingress_list_init_def_handlers(
                &g_pV6v4Context->ingressListHandlers),
            err,
            cleanupIndexPool
        );
    }
    else if ((g_V6v4RegistryData.ingressListFormat >= 
                IX_CC_V6V4_ISL_TRIE_FIRST) &&
             (g_V6v4RegistryData.ingressListFormat < IX_CC_V6V4_ISL_LAST))
    {
        IX_ERROR_CG(
            _ix_cc_v6v4_ingress_list_init_trie_handlers(
                &g_pV6v4Context->ingressListHandlers),
            err,
            cleanupIndexPool
        );
    }
    else
    {
        err = IX_ERROR_WARNING(IX_CC_ERROR_RANGE,
                              ("Ingress list format is invalid"));
        goto cleanupIndexPool;
    }

    IX_ERROR_CG(
        (*g_pV6v4Context->ingressListHandlers.setup)(
            &g_pV6v4Context->islInfo),
        err,
        cleanupIndexPool
    );

    return IX_SUCCESS;

cleanupIndexPool:
    ix_cc_index_pool_fini(g_pV6v4Context->endTunnelIndexPool);
cleanupMem:
    ix_rm_mem_free(g_pV6v4Context->pMbEndTunnelBase);
    return err;

} /* end - _ix_cc_v6v4_end_tunnel_setup */


/**
 * NAME: _ix_cc_v6v4_start_tunnel_setup
 *
 * DESCRIPTION: This primitive is used to allocate the resources needed
 * to manage start tunnel configuration.
 *           
 * @Param:  - IN None.
 *
 * @Param:  - OUT None.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_INTERNAL - internal error occurred 
 * 
 */

ix_error _ix_cc_v6v4_start_tunnel_setup(void)
{
    ix_error err;

    /* Allocate shared memory for end tunnel next hop table */
    IX_ERROR_CR(
        _ix_cc_v6v4_alloc_shared_table(
                g_V6v4RegistryData.startTunnelSize,
                sizeof(ix_cc_v6v4_start_tunnel),
                g_V6v4RegistryData.startTunnelMemType,
                g_V6v4RegistryData.startTunnelChannel,
                (void**)&g_pV6v4Context->pMbStartTunnelBase)
    );
  
    IX_ERROR_CG(
        ix_cc_index_pool_init(
            g_V6v4RegistryData.startTunnelSize, 1,
            &g_pV6v4Context->startTunnelIndexPool),
        err,
        cleanupMem);

    return IX_SUCCESS;

cleanupMem:
    ix_rm_mem_free(g_pV6v4Context->pMbStartTunnelBase);
    return err;

} /* end - _ix_cc_v6v4_start_tunnel_setup */


/**
 * NAME: _ix_cc_v6v4_alloc_counters
 *
 * DESCRIPTION: This primitive is used to allocate shared memory for
 * counters.
 * 
 * @Param:  - IN - none
 *
 * @Param:  - OUT  - none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOM  - not able to allocate memory
 * 
 */

ix_error _ix_cc_v6v4_alloc_counters(void)
{
    ix_error err = IX_SUCCESS;
    ix_uint32 memSize;
    ix_uint32 * pStatsBase;

    memSize = IX_CC_V6V4_NUMBER_OF_MB_COUNTERS * sizeof(ix_uint32);

    /* Allocation of memory for  microblock counters */
    err = ix_rm_mem_alloc(IX_MEMORY_TYPE_SRAM,
                          IX_CC_V6V4_SRAM_CHANNEL,
                          memSize,
                          (void**)&pStatsBase);
    if(err != IX_SUCCESS)
    {
        return IX_ERROR_WARNING(
                IX_CC_ERROR_OOM,
                ("Memory allocation failed for symbols to be patched"));
    }/* end if (err != IX_SUCCESS) */
     
    ix_ossl_memset(pStatsBase,0,memSize);

    g_pV6v4Context->pMbDecapStatsBase = pStatsBase;
    g_pV6v4Context->pMbEncapStatsBase = pStatsBase +
                    IX_CC_V6V4_NUMBER_OF_MB_DECAP_COUNTERS;

    return IX_SUCCESS;

}/* end _ix_cc_v6v4_alloc_counters */



/**
 * NAME: ix_cc_v6v4_init
 *
 * DESCRIPTION: This primitive shall be called and returned successfully before
 *              requesting any services from the tunneling core component.
 *              This primitive should be called only once to initialize the
 *              tunneling core component. This function performs allocation of
 *              memory for symbols to be patched, registration of packet and
 *              message handlers, initialization and configuration of shared
 *              memory used for tunneling and allocation and initialization of
 *              internal data structures.
 * 
 * @Param:  -   IN arg_hCcHandle - handle to tunneling core component, created
 *              by core component infrastructure; this shall be used later to
 *              get other services from core component infrastructure.
 *
 * @Param:  -   IN-OUT arg_ppContext - Upon input, points to a generic core
 *              component initialization context, from which general
 *              information that might be relevant to the core component can
 *              be obtained.  Upon successful completion of this function,
 *              the tunneling core component will update this location with
 *              a pointer to its context information.  This information is
 *              allocated by the tunneling core component and contains
 *              data that it uses internally. Later, when the core component
 *              is being destroyed, this pointer is passed to ix_cc_v6v4_fini()
 *              for freeing memory and performing other housekeeping
 *              operations.  
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - input parameter arg_ppContext is null
 * IX_CC_ERROR_OOM   - memory allocation failure
 * IX_CC_ERROR_RANGE - registry parameter value is out of range
 * IX_CC_V6V4_ERROR_FAILED_PATCHING   - patching failures                      
 * IX_CC_V6V4_ERROR_REGISTRY  - registry access failures
 * IX_CC_V6V4_ERROR_CCI - failure from CCI 
 * IX_CC_V6V4_ERROR_RM - failure from resource manager 
 * 
 */

ix_error ix_cc_v6v4_init(
                         ix_cc_handle arg_hCcHandle,
                         void** arg_ppContext
                        )
{
    ix_error err = IX_SUCCESS;

    /* Variables used in retrieving static data from arg_ppContext */
    ix_cc_init_context *pInitCtx;
    ix_cc_properties_node *pPropNode;

    /* Check for argument validity */
    if(arg_ppContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("Input parameter(arg_ppContext) is null"));

    }/* end if (arg_ppContext == NULL) */

    /* Allocate memory for tunneling Context */
    g_pV6v4Context = (ix_cc_v6v4_context*)
        ix_ossl_malloc(sizeof(ix_cc_v6v4_context));

    if (g_pV6v4Context == NULL)
    {
        return IX_ERROR_WARNING(
                    IX_CC_ERROR_OOM,
                    ("Failed to allocate memory for tunneling context"));
    }/* end if (g_pV6v4Context == NULL) */

    /* initialize the structure */
    ix_ossl_memset(g_pV6v4Context, 0, sizeof(ix_cc_v6v4_context));

    /* store tunneling component handle in ix_cc_v6v4_context */
    g_pV6v4Context->hCcHandle = arg_hCcHandle;

    /* Get the generic init context */
    pInitCtx = (ix_cc_init_context *)*arg_ppContext;
    /* Store free list handle in g_pIpv4Context */
    g_pV6v4Context->hBufferFreeList = pInitCtx->hFreeList;

    /* Retrieve static configuration from registry. If there is no registry, 
     * then the data structure 
     * will be filled with data from header file. */
    #if defined (IX_INCLUDE_REGISTRY)
        IX_ERROR_CG(_ix_cc_v6v4_registry(),err,cleanupContext);
    #else
        g_V6v4RegistryData.endTunnelSize = 
            IX_CC_V6V4_END_TUNNEL_SIZE;

        g_V6v4RegistryData.startTunnelSize = 
            IX_CC_V6V4_START_TUNNEL_SIZE;

        g_V6v4RegistryData.ingressListSizeHint =
            IX_CC_V6V4_INGRESS_LIST_SIZE_HINT;

        g_V6v4RegistryData.ingressListFormat =
            IX_CC_V6V4_INGRESS_LIST_FORMAT;

        g_V6v4RegistryData.endTunnelMemType =
            IX_CC_V6V4_END_TUNNEL_MEMORY_TYPE;

        g_V6v4RegistryData.startTunnelMemType  =
            IX_CC_V6V4_START_TUNNEL_MEMORY_TYPE;

        g_V6v4RegistryData.ingressListMemType  =
            IX_CC_V6V4_INGRESS_LIST_MEMORY_TYPE;

        g_V6v4RegistryData.endTunnelChannel =
            IX_CC_V6V4_END_TUNNEL_CHANNEL;

        g_V6v4RegistryData.startTunnelChannel  =
            IX_CC_V6V4_START_TUNNEL_CHANNEL;

        g_V6v4RegistryData.ingressListChannel =
             IX_CC_V6V4_INGRESS_LIST_CHANNEL;

        g_V6v4RegistryData.meNumbers  = IX_CC_V6V4_ME;   
    #endif

    /* 
     * Allocate shared memory and internal memory needed to help manage
     * the shared memory.
     */
    IX_ERROR_CG(_ix_cc_v6v4_alloc_counters(),err,cleanupContext);
    IX_ERROR_CG(_ix_cc_v6v4_end_tunnel_setup(), err, cleanupCounters);
    IX_ERROR_CG(_ix_cc_v6v4_start_tunnel_setup(), err, cleanupEndTunnels);

    /**
     * Retrieve local blade property information from init context 
     * (This will work for both level-1 and level-0 modes)
     */
    pPropNode = pInitCtx->pPropertyList;

    /* Get blade Id from first node */
    if(pPropNode != NULL)
    {
        g_V6v4RegistryData.localBladeId  = pPropNode->pPropertyInfo->blade_id;
    }/* end if */

    /* Patch symbols */
    IX_ERROR_CG(_ix_cc_v6v4_patch_symbols(),err,cleanupStartTunnels);

    /* Register Message Handler */
    IX_ERROR_CG(_ix_cc_v6v4_register_message_handler(),err,cleanupStartTunnels);

    /* Register Packet Handler */
    IX_ERROR_CG(
        _ix_cc_v6v4_register_packet_handlers(),
        err,cleanupMessageHandler);

    /* Allocate resources for Reassembly */
    IX_ERROR_CG(
        _ix_cc_v6v4_init_reassembly(), err,cleanupPacketHandlers);

    /* Get mutex from OSSL */
    err = ix_ossl_mutex_init(IX_OSSL_MUTEX_UNLOCK,&g_pV6v4Context->v6v4Mutex);
    if(err != IX_SUCCESS)
    {
        err = IX_ERROR_WARNING(IX_CC_ERROR_OOR,
                               ("Failed to create mutex"));
        goto cleanupEventHandler;

    }/* end if (g_pV6v4Context == NULL) */

    while(pPropNode != NULL)
    {
        ix_uint32 loop_var;

        for (loop_var = 0;
             loop_var < pPropNode->pPropertyInfo->no_virtual_if;
             loop_var++)
        {
            if (pPropNode->pPropertyInfo->ip_prop[loop_var].ip_version
                == IX_CC_IP_VERSION_IPV4)
            {
                IX_ERROR_CG(
                    _ix_cc_v6v4_property_update_ip_address(
                        IX_CC_SET_PROPERTY_ADD_INTERFACE_IPV4,
                        pPropNode->pPropertyInfo->blade_id,
                        pPropNode->pPropertyInfo->port_id,
                        pPropNode->pPropertyInfo->ip_prop[loop_var].protocol.ipv4_prop.ipv4_address,
                        pPropNode->pPropertyInfo->ip_prop[loop_var].protocol.ipv4_prop.ipv4_subnet_mask),
                    err, cleanupMutex);

                IX_ERROR_CG(
                    _ix_cc_v6v4_property_update_broadcast(
                        IX_CC_SET_PROPERTY_ADD_BROADCAST_IPV4,
                        pPropNode->pPropertyInfo->blade_id,
                        pPropNode->pPropertyInfo->port_id,
                        pPropNode->pPropertyInfo->ip_prop[loop_var].protocol.ipv4_prop.ipv4_broadcast),
                    err, cleanupMutex);
            }
        }

       IX_ERROR_CG(
          _ix_cc_v6v4_property_update_mtu(
             pPropNode->pPropertyInfo->blade_id,
             pPropNode->pPropertyInfo->port_id,
             pPropNode->pPropertyInfo->mtu),
       err, cleanupMutex);

       pPropNode = pPropNode->pNextPropNode;
    }/* end while */

   
    *arg_ppContext = (void *)g_pV6v4Context;
    return IX_SUCCESS;

    cleanupMutex:
    ix_ossl_mutex_fini(g_pV6v4Context->v6v4Mutex);
    cleanupEventHandler:
    _ix_cc_v6v4_cleanup_reassembly();
    cleanupPacketHandlers:
    _ix_cc_v6v4_unregister_packet_handlers();
    cleanupMessageHandler:
    _ix_cc_v6v4_unregister_message_handler();
    cleanupStartTunnels:
    _ix_cc_v6v4_start_tunnel_cleanup();
    cleanupEndTunnels:
    _ix_cc_v6v4_end_tunnel_cleanup();
    cleanupCounters:
    _ix_cc_v6v4_free_counters();
    cleanupContext:
    ix_ossl_free(g_pV6v4Context);

    return err;

} /* end ix_cc_v6v4_init */
