/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 6
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      IPv6/v4 Tunneling Core Component - message helpers
 *
 * = FILENAME
 *      ix_cc_v6v4_msg_helpers.c
 *
 * = DESCRIPTION
 *      The file defines message helper interface functions.
 *
 * = CREATION TIME
 *      11/18/2002 11:15 AM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */
#define IX_ERROR_FILE_IDENT "$Id: ix_cc_v6v4_msg_helpers.c,v 1.3 2003/01/30 19:08:47 cabell Exp $";



/**
 * System defined include files
 */

/**
 * User defined include files
 */

#include "ix_cc_error.h"
#include "ix_ossl.h"
#include "ix_rm.h"
#include "ix_cc.h"
#include "bindings.h"
#include "cc/ix_cc_v6v4.h"
#include "cc/ix_cc_msup.h"


#include "cc/v6v4_tunnel/internal/ix_cc_v6v4_msg_handler.h"




/**
 * Preprocessor symbols and Macros used in this file
 */

/**
 * Type definitions whose scope is limited to this file
 */


/**
 * Variable declarations global to this file only. Externs are followed by 
 * static variables.
 */


/** 
 * Extern function prototypes.
 */

/** 
 * Static function prototypes.
 */




/** 
 * Function definitions.
 */



/**
 * NAME: ix_cc_v6v4_async_add_end_tunnel
 *
 * DESCRIPTION: This message helper primitive helps the client to add an ending
 * tunnel endpoint.
 * 
 * @Param:  - IN arg_pConfig: pointer to configuration information about the tunnel.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */


ix_error ix_cc_v6v4_async_add_end_tunnel(
    ix_cc_v6v4_end_tunnel_config* arg_pConfig,
    ix_cc_v6v4_cb_add_tunnel arg_Callback,
    void* arg_pClientContext
    )
{
    ix_cc_v6v4_end_tunnel_config msgData;
    ix_cc_v6v4_add_tunnel_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if (arg_pConfig == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pConfig"));
    }/* end if(arg_pConfig == NULL) */

    if (arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_v6v4_add_tunnel_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_v6v4_add_tunnel_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData = *arg_pConfig;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_V6V4_MSG_INPUT,
                                    _ix_cc_v6v4_cb_add_tunnel,
                                    (void*)cbContext,
                                    IX_CC_V6V4_MSG_ADD_END_TUNNEL,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_V6V4_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_v6v4_async_delete_end_tunnel
 *
 * DESCRIPTION:This message helper primitive helps a client to delete an end
 * tunnel endpoint. 
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_v6v4_async_delete_end_tunnel(
    ix_cc_v6v4_tunnel_handle arg_hTunnel,
    ix_cc_v6v4_cb arg_Callback,
    void* arg_pClientContext
    )
{
    ix_cc_v6v4_tunnel_handle msgData;
    ix_cc_v6v4_general_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_v6v4_general_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_v6v4_general_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData = arg_hTunnel;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_V6V4_MSG_INPUT,
                                    _ix_cc_v6v4_cb_general,
                                    (void*)cbContext,
                                    IX_CC_V6V4_MSG_DELETE_END_TUNNEL,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_V6V4_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_v6v4_async_set_decap_tos_option
 *
 * DESCRIPTION: This message helper primitive helps the client to update the TOS
 * option for decapsulation. 
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Selection: option selection.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */

ix_error ix_cc_v6v4_async_set_decap_tos_option(
    ix_cc_v6v4_tunnel_handle arg_hTunnel,
    ix_uint8 arg_Selection,
    ix_cc_v6v4_cb arg_Callback,
    void* arg_pClientContext
    )
{
    ix_cc_v6v4_set_decap_tos_data msgData;
    ix_cc_v6v4_general_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_v6v4_general_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_v6v4_general_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData.hTunnel = arg_hTunnel;
    msgData.selection = arg_Selection;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_V6V4_MSG_INPUT,
                                    _ix_cc_v6v4_cb_general,
                                    (void*)cbContext,
                                    IX_CC_V6V4_MSG_SET_DECAP_TOS_OPTION,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_V6V4_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_v6v4_async_set_src_validation
 *
 * DESCRIPTION: This message helper primitive helps the client to update the
 * ingress source validation option for decapsulation. 
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Selection: option selection.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */

ix_error ix_cc_v6v4_async_set_src_validation(
    ix_cc_v6v4_tunnel_handle arg_hTunnel,
    ix_uint8 arg_Selection,
    ix_cc_v6v4_cb arg_Callback,
    void* arg_pClientContext
    )
{
    ix_cc_v6v4_set_src_validation_data msgData;
    ix_cc_v6v4_general_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_v6v4_general_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_v6v4_general_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData.hTunnel = arg_hTunnel;
    msgData.selection = arg_Selection;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_V6V4_MSG_INPUT,
                                    _ix_cc_v6v4_cb_general,
                                    (void*)cbContext,
                                    IX_CC_V6V4_MSG_SET_SRC_VALIDATION,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_V6V4_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_v6v4_async_get_end_tunnel
 *
 * DESCRIPTION: This primitive helps a client to retrieve the information 
 * for a given end tunnel endpoint.
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_v6v4_async_get_end_tunnel(
    ix_cc_v6v4_tunnel_handle arg_hTunnel,
    ix_cc_v6v4_cb_end_tunnel arg_Callback,
    void* arg_pClientContext
    )
{
    ix_cc_v6v4_tunnel_handle msgData;
    ix_cc_v6v4_end_tunnel_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_v6v4_end_tunnel_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_v6v4_end_tunnel_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData = arg_hTunnel;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_V6V4_MSG_INPUT,
                                    _ix_cc_v6v4_cb_end_tunnel,
                                    (void*)cbContext,
                                    IX_CC_V6V4_MSG_GET_END_TUNNEL,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_V6V4_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_v6v4_async_add_allowed_source
 *
 * DESCRIPTION: This primitive helps a client to add a prefix to the ingress
 * source address validation list for a end tunnel endpoint.
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_pEntry: pointer to ip address/prefix length to be added.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_v6v4_async_add_allowed_source(
    ix_cc_v6v4_tunnel_handle arg_hTunnel,
    ix_cc_v6v4_ingress_source_entry* arg_pEntry,
    ix_cc_v6v4_cb arg_Callback,
    void* arg_pClientContext
    )
{
    ix_cc_v6v4_add_allowed_source_data msgData;
    ix_cc_v6v4_general_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_pEntry == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pEntry"));
    }/* end if(arg_pEntry == NULL) */

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_v6v4_general_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_v6v4_general_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData.hTunnel = arg_hTunnel;
    msgData.islEntry = *arg_pEntry;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_V6V4_MSG_INPUT,
                                    _ix_cc_v6v4_cb_general,
                                    (void*)cbContext,
                                    IX_CC_V6V4_MSG_ADD_ALLOWED_SOURCE,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_V6V4_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_v6v4_async_delete_allowed_source
 *
 * DESCRIPTION: This primitive helps a client to delete a prefix from the ingress
 * source address validation list for a end tunnel endpoint.
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_pEntry: pointer to ip address/prefix length to be deleted.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_v6v4_async_delete_allowed_source(
    ix_cc_v6v4_tunnel_handle arg_hTunnel,
    ix_cc_v6v4_ingress_source_entry* arg_pEntry,
    ix_cc_v6v4_cb arg_Callback,
    void* arg_pClientContext
    )
{
    ix_cc_v6v4_delete_allowed_source_data msgData;
    ix_cc_v6v4_general_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_pEntry == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pEntry"));
    }/* end if(arg_pEntry == NULL) */

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_v6v4_general_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_v6v4_general_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData.hTunnel = arg_hTunnel;
    msgData.islEntry = *arg_pEntry;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_V6V4_MSG_INPUT,
                                    _ix_cc_v6v4_cb_general,
                                    (void*)cbContext,
                                    IX_CC_V6V4_MSG_DELETE_ALLOWED_SOURCE,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_V6V4_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_v6v4_async_clear_allowed_sources
 *
 * DESCRIPTION: This primitive helps a client to delete all prefixes from the
 * ingress source address validation list for a end tunnel endpoint.
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_v6v4_async_clear_allowed_sources(
    ix_cc_v6v4_tunnel_handle arg_hTunnel,
    ix_cc_v6v4_cb arg_Callback,
    void* arg_pClientContext
    )
{
    ix_cc_v6v4_tunnel_handle msgData;
    ix_cc_v6v4_general_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_v6v4_general_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_v6v4_general_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData = arg_hTunnel;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_V6V4_MSG_INPUT,
                                    _ix_cc_v6v4_cb_general,
                                    (void*)cbContext,
                                    IX_CC_V6V4_MSG_CLEAR_ALLOWED_SOURCES,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_V6V4_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_v6v4_async_get_allowed_sources
 *
 * DESCRIPTION: This primitive helps a client to retrieve a list of all prefixes
 * in the ingress source address validation list for a end tunnel endpoint.
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_pEntry: pointer to ip address/prefix length to be deleted.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_v6v4_async_get_allowed_sources(
    ix_cc_v6v4_tunnel_handle arg_hTunnel,
    ix_cc_v6v4_cb_get_sources arg_Callback,
    void* arg_pClientContext
    )
{
    ix_cc_v6v4_tunnel_handle msgData;
    ix_cc_v6v4_get_sources_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_v6v4_get_sources_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_v6v4_get_sources_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData = arg_hTunnel;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_V6V4_MSG_INPUT,
                                    _ix_cc_v6v4_cb_get_sources,
                                    (void*)cbContext,
                                    IX_CC_V6V4_MSG_GET_ALLOWED_SOURCES,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_V6V4_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_v6v4_async_dump_end_tunnels
 *
 * DESCRIPTION: This primitive helps a client to retrieve a list of all end
 * tunnel endpoints.
 * 
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_v6v4_async_dump_end_tunnels(
    ix_cc_v6v4_cb_dump_end_tunnels arg_Callback,
    void* arg_pClientContext
    )
{
    ix_cc_v6v4_dump_end_tunnels_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_v6v4_dump_end_tunnels_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_v6v4_dump_end_tunnels_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_V6V4_MSG_INPUT,
                                    _ix_cc_v6v4_cb_dump_end_tunnels,
                                    (void*)cbContext,
                                    IX_CC_V6V4_MSG_DUMP_END_TUNNELS,
                                    NULL,
                                    0),
                 err,
                 IX_CC_V6V4_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_v6v4_async_add_start_tunnel
 *
 * DESCRIPTION: This message helper primitive helps the client to add a start
 * tunnel endpoint.
 * 
 * @Param:  - IN arg_pConfig: pointer to configuration information about the tunnel.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */


ix_error ix_cc_v6v4_async_add_start_tunnel(
    ix_cc_v6v4_start_tunnel_config* arg_pConfig,
    ix_cc_v6v4_cb_add_tunnel arg_Callback,
    void* arg_pClientContext
    )
{
    ix_cc_v6v4_start_tunnel_config msgData;
    ix_cc_v6v4_add_tunnel_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_pConfig == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pConfig"));
    }/* end if(arg_pConfig == NULL) */

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_v6v4_add_tunnel_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_v6v4_add_tunnel_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData = *arg_pConfig;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_V6V4_MSG_INPUT,
                                    _ix_cc_v6v4_cb_add_tunnel,
                                    (void*)cbContext,
                                    IX_CC_V6V4_MSG_ADD_START_TUNNEL,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_V6V4_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_v6v4_async_delete_start_tunnel
 *
 * DESCRIPTION:This message helper primitive helps a client to delete a start
 * tunnel endpoint. 
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */


ix_error ix_cc_v6v4_async_delete_start_tunnel(
    ix_cc_v6v4_tunnel_handle arg_hTunnel,
    ix_cc_v6v4_cb arg_Callback,
    void* arg_pClientContext
    )
{
    ix_cc_v6v4_tunnel_handle msgData;
    ix_cc_v6v4_general_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_v6v4_general_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_v6v4_general_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData = arg_hTunnel;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_V6V4_MSG_INPUT,
                                    _ix_cc_v6v4_cb_general,
                                    (void*)cbContext,
                                    IX_CC_V6V4_MSG_DELETE_START_TUNNEL,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_V6V4_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_v6v4_async_set_ttl
 *
 * DESCRIPTION:This message helper primitive helps a client to set the TTL
 * value for encapsulation. 
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Ttl: TTL value to set.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */


ix_error ix_cc_v6v4_async_set_ttl(
    ix_cc_v6v4_tunnel_handle arg_hTunnel,
    ix_uint8 arg_Ttl,
    ix_cc_v6v4_cb arg_Callback,
    void* arg_pClientContext
    )
{
    ix_cc_v6v4_set_ttl_data msgData;
    ix_cc_v6v4_general_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_v6v4_general_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_v6v4_general_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData.hTunnel = arg_hTunnel;
    msgData.ttl = arg_Ttl;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_V6V4_MSG_INPUT,
                                    _ix_cc_v6v4_cb_general,
                                    (void*)cbContext,
                                    IX_CC_V6V4_MSG_SET_TTL,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_V6V4_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_v6v4_async_set_encap_tos_option
 *
 * DESCRIPTION: This message helper primitive helps the client to update the TOS
 * option for encapsulation. 
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Selection: option selection.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */

ix_error ix_cc_v6v4_async_set_encap_tos_option(
    ix_cc_v6v4_tunnel_handle arg_hTunnel,
    ix_uint8 arg_Selection,
    ix_cc_v6v4_cb arg_Callback,
    void* arg_pClientContext
    )
{
    ix_cc_v6v4_set_encap_tos_data msgData;
    ix_cc_v6v4_general_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_v6v4_general_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_v6v4_general_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData.hTunnel = arg_hTunnel;
    msgData.selection = arg_Selection;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_V6V4_MSG_INPUT,
                                    _ix_cc_v6v4_cb_general,
                                    (void*)cbContext,
                                    IX_CC_V6V4_MSG_SET_ENCAP_TOS_OPTION,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_V6V4_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_v6v4_async_set_tos
 *
 * DESCRIPTION: This message helper primitive helps the client to update the TOS
 * value to be used in encapsulation if the tunnel is not configured to encode
 * the TOS byte from the IPv6 header. 
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Tos: TOS value to set.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */

ix_error ix_cc_v6v4_async_set_tos(
    ix_cc_v6v4_tunnel_handle arg_hTunnel,
    ix_uint8 arg_Tos,
    ix_cc_v6v4_cb arg_Callback,
    void* arg_pClientContext
    )
{
    ix_cc_v6v4_set_tos_data msgData;
    ix_cc_v6v4_general_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_v6v4_general_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_v6v4_general_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData.hTunnel = arg_hTunnel;
    msgData.tos = arg_Tos;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_V6V4_MSG_INPUT,
                                    _ix_cc_v6v4_cb_general,
                                    (void*)cbContext,
                                    IX_CC_V6V4_MSG_SET_TOS,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_V6V4_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_v6v4_async_set_mtu
 *
 * DESCRIPTION:This message helper primitive helps a client to set the path MTU
 * value for a start tunnel endpoint. 
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Mtu: path MTU value to set.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */


ix_error ix_cc_v6v4_async_set_mtu(
    ix_cc_v6v4_tunnel_handle arg_hTunnel,
    ix_uint8 arg_Mtu,
    ix_cc_v6v4_cb arg_Callback,
    void* arg_pClientContext
    )
{
    ix_cc_v6v4_set_mtu_data msgData;
    ix_cc_v6v4_general_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_v6v4_general_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_v6v4_general_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData.hTunnel = arg_hTunnel;
    msgData.mtu = arg_Mtu;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_V6V4_MSG_INPUT,
                                    _ix_cc_v6v4_cb_general,
                                    (void*)cbContext,
                                    IX_CC_V6V4_MSG_SET_MTU,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_V6V4_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_v6v4_async_set_subnet_broadcast
 *
 * DESCRIPTION:This message helper primitive helps a client to set the subnet
 * broadcast address for a start tunnel endpoint. 
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_SubnetBroadcast: subnet broadcast address to set.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */


ix_error ix_cc_v6v4_async_set_subnet_broadcast(
    ix_cc_v6v4_tunnel_handle arg_hTunnel,
    ix_uint32 arg_SubnetBroadcast,
    ix_cc_v6v4_cb arg_Callback,
    void* arg_pClientContext
    )
{
    ix_cc_v6v4_set_subnet_broadcast_data msgData;
    ix_cc_v6v4_general_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_v6v4_general_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_v6v4_general_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData.hTunnel = arg_hTunnel;
    msgData.subnetBroadcast = arg_SubnetBroadcast;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_V6V4_MSG_INPUT,
                                    _ix_cc_v6v4_cb_general,
                                    (void*)cbContext,
                                    IX_CC_V6V4_MSG_SET_SUBNET_BROADCAST,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_V6V4_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_v6v4_async_get_start_tunnel
 *
 * DESCRIPTION: This primitive helps a client to retrieve the information 
 * for a given start tunnel endpoint.
 * 
 * @Param:  - IN arg_hTunnel: handle to tunnel.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_v6v4_async_get_start_tunnel(
    ix_cc_v6v4_tunnel_handle arg_hTunnel,
    ix_cc_v6v4_cb_start_tunnel arg_Callback,
    void* arg_pClientContext
    )
{
    ix_cc_v6v4_tunnel_handle msgData;
    ix_cc_v6v4_start_tunnel_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_v6v4_start_tunnel_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_v6v4_start_tunnel_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    msgData = arg_hTunnel;

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_V6V4_MSG_INPUT,
                                    _ix_cc_v6v4_cb_start_tunnel,
                                    (void*)cbContext,
                                    IX_CC_V6V4_MSG_GET_START_TUNNEL,
                                    (void *)&msgData,
                                    sizeof(msgData)),
                 err,
                 IX_CC_V6V4_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_v6v4_async_dump_start_tunnels
 *
 * DESCRIPTION: This primitive helps a client to retrieve a list of all start
 * tunnel endpoints.
 * 
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_v6v4_async_dump_start_tunnels(
    ix_cc_v6v4_cb_dump_start_tunnels arg_Callback,
    void* arg_pClientContext
    )
{
    ix_cc_v6v4_dump_start_tunnels_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_v6v4_dump_start_tunnels_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_v6v4_dump_start_tunnels_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_V6V4_MSG_INPUT,
                                    _ix_cc_v6v4_cb_dump_start_tunnels,
                                    (void*)cbContext,
                                    IX_CC_V6V4_MSG_DUMP_START_TUNNELS,
                                    NULL,
                                    0),
                 err,
                 IX_CC_V6V4_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}


/**
 * NAME: ix_cc_v6v4_async_get_statistics
 *
 * DESCRIPTION: This primitive helps a client to retrieve the tunneling
 * statistics.
 * 
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pClientContext: pointer to client defined context.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_v6v4_async_get_statistics(
    ix_cc_v6v4_cb_statistics arg_Callback,
    void* arg_pClientContext
    )
{
    ix_cc_v6v4_statistics_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    if(arg_Callback == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("Invalid pointer: arg_pCallback"));
    }/* end if(arg_Callback == NULL) */

    cbContext = (ix_cc_v6v4_statistics_callback_data*)
        ix_ossl_malloc(sizeof(ix_cc_v6v4_statistics_callback_data));    
    if(cbContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,("No memory for context"));
    }/* end if no memory for context */

    cbContext->pUserCallback = arg_Callback;   
    cbContext->pUserContext = arg_pClientContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_V6V4_MSG_INPUT,
                                    _ix_cc_v6v4_cb_statistics,
                                    (void*)cbContext,
                                    IX_CC_V6V4_MSG_GET_STATISTICS,
                                    NULL,
                                    0),
                 err,
                 IX_CC_V6V4_ERROR_MSG_LIBRARY,
                 IX_ERROR_LEVEL_WARNING,
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:
    if(cbContext != NULL)
    {
        ix_ossl_free(cbContext);
    }/* end if cbContext != NULL */
    return err;
}
