/******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
******************************************************************************/


/*
 * mpls.h
 *
 * Pre-processor symbols for MPLS forwarder.
 */



#ifndef MPLS_H
#define MPLS_H 1

#ifndef MPLS_HARDWARE_CONFIGURATION
    #include "dl_system.h"
#endif

/*
 ---------------------------------------------------------------------------
			 Define where to store forwaring tables
 ---------------------------------------------------------------------------
*/

/*
 * NHLFE table
 */
#ifdef FTN_SRAM
	#define FTN_CACHE								$ftn_nhlfe
#endif

#ifdef FTN_DRAM
	#define FTN_CACHE								$$ftn_nhlfe
#endif


/*
 * ILM table
 */
#ifdef ILM_SRAM
	#define ILM_CACHE								$ilm_nhlfe
#endif

#ifdef ILM_DRAM
	#define ILM_CACHE								$$ilm_nhlfe
#endif


/*
 * EXP2PHB table
 */
#ifdef EXP2PHB_SRAM
	#define EXP2PHB_CACHE							$exp2phb_entry
#endif

#ifdef EXP2PHB_DRAM
	#define EXP2PHB_CACHE							$$exp2phb_entry
#endif


/*
 * PHB2EXP table
 */
#ifdef PHB2EXP_SRAM
	#define PHB2EXP_CACHE							$phb2exp_entry
#endif

#ifdef PHB2EXP_DRAM
	#define PHB2EXP_CACHE							$$phb2exp_entry
#endif


/*
 ---------------------------------------------------------------------------
			 Define flag bits
 ---------------------------------------------------------------------------
*/

/*
 * FTN flags
 */
#define MPLS_FTN_NHLFE_VALID_BIT					31
#define MPLS_FTN_NHLFE_SET_BIT						30
#define MPLS_FTN_NHLFE_SHORT_PIPE_TUNNEL_MODEL		29
#define MPLS_FTN_NHLFE_TC_ENABLED_BIT				25


/*
 * ILM flags 
 */
#define MPLS_ILM_VALID_BIT							31
#define MPLS_ILM_NHLFE_SET_BIT						30
#define MPLS_ILM_SHORT_PIPE_TUNNEL_MODEL			29
#define MPLS_ILM_DIFFSERV_LSP_BIT					27
#define MPLS_ILM_TC_ENABLED_BIT						25


/*
 * Tunnel models
 */
#define MPLS_PIPE_TUNNEL_MODEL						0
#define MPLS_UNIFORM_TUNNEL_MODEL					1
#define MPLS_SHORT_PIPE_TUNNEL_MODEL				2
#define MPLS_SHORT_PIPE_WITH_PHB_TUNNEL_MODEL		3


/*
 * Label operations
 */
#define MPLS_LABELOP_SWAP                   		0x0
#define MPLS_LABELOP_SWAP_PUSH              		0x1
#define MPLS_LABELOP_POP                    		0x2
#define MPLS_LABELOP_POP_FORWARD            		0x3

/*
 ---------------------------------------------------------------------------
			 Define tables offset
 ---------------------------------------------------------------------------
*/

/*
 * MPLS Input Interface Counter offsets
 */
#define MPLS_IN_IFC_PKTS							0x0
#define MPLS_IN_IFC_FAILED_LOOKUP_PKTS				0x4
#define MPLS_IN_IFC_ERROR_PKTS						0x8
#define MPLS_IN_IFC_OCTETS							0xC


/*
 * MPLS Input Segment Counter offsets
 */
#define MPLS_IN_SEGMENT_OCTETS              		0x0
#define MPLS_IN_SEGMENT_PKTS                		0x4
#define MPLS_IN_SEGMENT_ERROR_OCTETS        		0x8
#define MPLS_IN_SEGMENT_ERROR_PKTS          		0xC
#define MPLS_IN_SEG_FRAGM_OCT_DELTA					0x10
#define MPLS_IN_SEG_FRAGM_PKT_DELTA					0x14


/*
 ---------------------------------------------------------------------------
 			Other
 ---------------------------------------------------------------------------
*/

#define MPLS_RES_LABEL_MAX                      15
#define MPLS_NOT_RES_LABEL_MIN                  16
#define MPLS_HDR_CACHE_OFFSET                   16
#define MPLS_STACK_BIT				8
#define MPLS_NHLFE_INDEX_SHIFT                  5
#define MPLS_PIPE_TTL                           0xFF
#define HDR_POINTER                            *l$index1
	

#endif /* end defined(MPLS_H) */
