//----------------------------------------------------------------------------
//
//                   I N T E L   P R O P R I E T A R Y
//
//      COPYRIGHT (c)  2001-2003 BY  INTEL  CORPORATION.  ALL RIGHTS
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT
//      THE PRIOR WRITTEN PERMISSION OF :
//
//                         INTEL  CORPORATION
//
//                      2200 MISSION COLLEGE BLVD
//
//                SANTA  CLARA,  CALIFORNIA  95052-8119
//
//----------------------------------------------------------------------------
/////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: 	mpls_util.uc
//
//      Purpose: 	Functions used by MPLS ILM and MPLS FTN.
//
// 		Assumptions:
// 				This file may be included by others
//				(and thus uses header guards)
//
//
//
/////////////////////////////////////////////////////////////////////////////

// Exported APIs:

// _mpls_init()

// _mpls_fetch_ilm_nhlfe(ilm_dst, ilm_address, ilm_offset, num_word)
// _mpls_fetch_ftn_nhlfe(ftn_address, ftn_offset)

// _mpls_push_labels_lm(push_count, TUNNEL_MODEL, DIFFSERV_SWITCH,
//					HDR_CACHE, HDR_WORD_OFFSET, NHLFE_CACHE, ttl)
// _mpls_swap_push_labels_lm(push_count, TUNNEL_MODEL, DIFFSERV_SWITCH,
//					HDR_POINTER, NHLFE_CACHE, ttl)
// _mpls_swap_labels_lm(HDR_POINTER, NHLFE_CACHE, ttl)

// _mpls_ilm_operation_extract(value)
// _mpls_ilm_tunnel_model_extract(value)
// _mpls_ilm_ttl_extract(value)
// _mpls_ilm_lsp_type_extract(value)
// _mpls_ilm_push_count_extract(value)
// _mpls_ilm_outport_extract(value)
// _mpls_ilm_max_lab_pkt_size_extract(value)
// _mpls_ilm_l3protocol_extract(value)

// _mpls_ftn_lsp_type_extract(value)
// _mpls_ftn_tunnel_model_extract(value)
// _mpls_ftn_operation_extract(value)
// _mpls_ftn_push_count_extract(value)
// _mpls_ftn_ttl_extract(value)
// _mpls_ftn_blade_id_extract(value)
// _mpls_ftn_outport_extract(value)
// _mpls_ftn_max_lab_pkt_size_extract(value)

// _mpls_cache_ip_header(labels_in_cache)
// _mpls_update_ipv4_header_lm(out_hdr, HDR_OFFSET, ttl)

// _mpls_update_meta_data(NHLFE_CACHE)
// _mpls_update_dl_size(number_of_labels)

// _mpls_select_nhlfe_from_set(in_cache, out_nhlfe_offset)
// _mpls_set_header_type(hdr_type)

// _mpls_get_in_port_stat_addr(in_port_stat_addr, input_port)
// _mpls_update_in_ifc_error_counter(input_port)
// _mpls_update_rx_port_counters(input_port, packet_size)
// _mpls_update_incoming_counters_ftn(in_val, packet_size)
// _mpls_update_incoming_counters_ilm(address, packet_size)
// _mpls_update_in_ILM_discard_counter(incoming_label)

// ---------------------------------------------------------------------
// The following APIs are not currently used in this release:
// ---------------------------------------------------------------------
// _mpls_fetch_phb2exp(phb2exp_entry, phb2exp_address, phb2exp_offset)
// _mpls_fetch_exp2phb(exp2phb_address, exp2phb_offset)
// _mpls_phb2exp_exp_field_extract(exp_field, phb2exp_entry, EXP_NUM)
// _mpls_exp2phb_exp_extract(value)

// _mpls_push_router_alert_label(HDR_POINTER, ttl)

// _mpls_set_diffserv_dl_variables(EXP2PHB_CACHE)

// _mpls_get_out_port_stat_addr(out_port_stat_addr, blade_id, output_port)

// _mpls_update_in_segment_counters(packet_size)
// _mpls_update_tx_counters(NHLFE_CACHE, packet_size)
// _mpls_update_failed_lookup_counter(input_port)

#ifndef MPLS_UTIL_UC
#define MPLS_UTIL_UC 1

//-----------------------------------------------------------------------------
// Included files:
//-----------------------------------------------------------------------------
#include <stdmac.uc>
#include <xbuf.uc>
#include <dram.uc>
#include <bytefield.uc>
#include "dl_system.h"
#include "mpls.h"

//-----------------------------------------------------------------------------
// Macros:
//-----------------------------------------------------------------------------

// ---------------------------------------------------------------------------
// _mpls_init()
//
//
//    Description: Initialization of global registers.
//					Should be run only by thread 0.
//
// ---------------------------------------------------------------------------
#macro _mpls_init()

 	br!=ctx[0, mpls_init_label_space#]

	.reg @ilm_nhlfe_addr
	immed32(@ilm_nhlfe_addr, MPLS_ILM_TABLE_BASE)

	.reg @ilm_nhlfe_set_addr
	immed32(@ilm_nhlfe_set_addr, MPLS_ILM_NHLFE_SET_TABLE_BASE)

	.reg @ftn_nhlfe_addr
	immed32(@ftn_nhlfe_addr, MPLS_FTN_TABLE_BASE)

#ifdef __COUNTERS__

    .reg @in_port_stat	// input port statistics base
    immed32(@in_port_stat, MPLS_INPUT_PORT_STAT_BASE)

#endif

mpls_init_label_space#:

/*
 * For PER_INTERFACE_LABEL_SPACE, populate per interface information
 * from SRAM to Local Memory
 */
#ifdef PER_INTERFACE_LABEL_SPACE
    _mpls_populate_label_space()
#endif

/*
 * For PER_PLATFORM_LABEL_SPACE, populate per platform information
 * from SRAM to Local Memory
 */
#ifdef PER_PLATFORM_LABEL_SPACE
    _mpls_set_min_max_labels()
#endif

#endm

#macro _mpls_set_min_max_labels()
.begin
.reg lm_base sram_base
.reg tmp
.sig label_space_sig
	immed32(lm_base, LM_LABEL_SPACE_ADDR)
	local_csr_wr[active_lm_addr_1, lm_base]

	xbuf_alloc($min_max_labels, 2, READ)
	immed32(sram_base, MPLS_ILM_PARAM_TABLE_BASE)
    sram[read, $min_max_labels[0], sram_base, 0, 2], \
        ctx_swap[label_space_sig]

	nop
    alu[*l$index1++, --, B, $min_max_labels[0]]
    alu[*l$index1, --, B, $min_max_labels[1]]

	xbuf_free($min_max_labels)
.end
#endm

#macro _mpls_populate_label_space()
.begin
.reg lm_base sram_base sram_offset
.reg tmp
.sig label_space_sig

	immed32(lm_base, LM_LABEL_SPACE_ADDR)
	local_csr_wr[active_lm_addr_1, lm_base]

	// calculate number of SRAM xfers needed for update to LM
#ifdef __COUNTERS__
	#define_eval 	_MPLS_TOTAL_NUM_XFER	(INTERFACE_MAX * 4)
#else
	#define_eval 	_MPLS_TOTAL_NUM_XFER	(INTERFACE_MAX * 3)
#endif
	#define_eval	_MPLS_NUM_LOOP			(_MPLS_TOTAL_NUM_XFER / 8)
	#define_eval	_MPLS_NUM_LOOP_COPY		(_MPLS_TOTAL_NUM_XFER / 8)
	#define_eval	_MPLS_REMAINDER			(_MPLS_TOTAL_NUM_XFER % 8)

	#if (_MPLS_TOTAL_NUM_XFER < 9)
		xbuf_alloc($label_space_xfer, _MPLS_TOTAL_NUM_XFER, READ)
	#else
		xbuf_alloc($label_space_xfer, 8, READ)
	#endif

	immed32(sram_base, MPLS_ILM_PARAM_TABLE_BASE)

	#define_eval _MPLS_OFFSET	0
	#while (_MPLS_NUM_LOOP > 0)
	    immed32(sram_offset, _MPLS_OFFSET)
	 	sram[read, $label_space_xfer0, sram_base, sram_offset, 8], \
				ctx_swap[label_space_sig]
		// nop to fill 3 slots required for local_csr_wr
		nop

		// copy from SRAM xfers to LM
		#define_eval __MPLS_INDEX	0
		#while (__MPLS_INDEX < 8)
		    #define_eval _MPLS_REG_NR (((_MPLS_NUM_LOOP_COPY - _MPLS_NUM_LOOP) * 8) + (__MPLS_INDEX) + 1)
            alu[*l$index1++, --, B, $label_space_xfer/**/__MPLS_INDEX]
			#define_eval __MPLS_INDEX	(__MPLS_INDEX + 1)
		#endloop

		#define_eval _MPLS_NUM_LOOP	(_MPLS_NUM_LOOP - 1)
		#define_eval _MPLS_OFFSET	(_MPLS_OFFSET + (8 * 4))
	#endloop

	#if (_MPLS_REMAINDER > 0)
	 	sram[read, $label_space_xfer0, sram_base, _MPLS_OFFSET, _MPLS_REMAINDER], \
				ctx_swap[label_space_sig]
		// nop to fill 3 slots required for local_csr_wr
		nop
		// copy from SRAM xfers to LM
		#define_eval __MPLS_INDEX	0
		#while (__MPLS_INDEX < _MPLS_REMAINDER)
		    #define_eval _MPLS_REG_NR ((_MPLS_NUM_LOOP_COPY * 8) + (__MPLS_INDEX) + 1)
	        alu[*l$index1++, --, B, $label_space_xfer/**/__MPLS_INDEX]
			#define_eval __MPLS_INDEX	(__MPLS_INDEX + 1)
		#endloop
	#endif

	// clean up name space
	#undef _MPLS_REG_NR
	#undef _MPLS_TOTAL_NUM_XFER
	#undef _MPLS_NUM_LOOP
	#undef _MPLS_NUM_LOOP_COPY
	#undef _MPLS_REMAINDER
	#undef _MPLS_OFFSET
	#undef __MPLS_INDEX

	xbuf_free($label_space_xfer)

.end
#endm

#macro _mpls_lm_in_out_port_populate()
.begin
.reg tmp_base port_base
.sig in_out_port_sig

	immed32(tmp_base, MPLS_PORTS_LM_BASE)
	local_csr_wr[active_lm_addr_1, tmp_base]

	xbuf_alloc($lm_port_xfer, 1, READ)

	immed32(port_base, MPLS_PORTS_SRAM_BASE)
 	sram[read, $lm_port_xfer0, port_base, 0, 1], ctx_swap[in_out_port_sig]
	nop
	alu[*l$index1, --, B, $lm_port_xfer0]
	xbuf_free($lm_port_xfer)

.end
#endm



// ----------------------------------------------------------------------------
// _mpls_fetch_ilm_nhlfe(ilm_dst, ilm_address, ilm_offset, num_word)
//
//
//    Description: Read ILM entries from DRAM or SRAM memory.
//
//
//  	Inputs:
//			ilm_dst - where to store the fetched values
//			ilm_address - base memory address
//			ilm_offset - memory offset
//			num_word - number of LWs to read
//
// ----------------------------------------------------------------------------
#macro _mpls_fetch_ilm_nhlfe(ilm_dst, ilm_address, ilm_offset, NUM_WORD)
.begin
.sig sig_fetch_ilm

	#ifdef ILM_SRAM

		sram[read, ilm_dst, ilm_address, ilm_offset, NUM_WORD], ctx_swap[sig_fetch_ilm]

	#else
		#ifdef ILM_DRAM

			#if (NUM_WORD == 1)
				#define_eval NUM_QW		1
			#else
				#define_eval NUM_QW	(NUM_WORD/2)
			#endif

			dram[read, ilm_dst, ilm_address, ilm_offset, NUM_QW], sig_done[sig_fetch_ilm]
			ctx_arb[sig_fetch_ilm]

		#else
			#error "ILM NHLFE table is not allocated."
		#endif
	#endif

.end
#endm


// ----------------------------------------------------------------------------
// _mpls_fetch_ftn_nhlfe(ftn_address, ftn_offset)
//
//
//    Description: Read FTN NHLF entry from DRAM or SRAM memory.
//
//
//  	Inputs:
//			ftn_address - base memory address
//			ftn_offset  - memory offset
//
// ----------------------------------------------------------------------------
#macro _mpls_fetch_ftn_nhlfe(ftn_address, ftn_offset)
.begin
.sig sig_fetch_nhlfe

	#ifdef FTN_SRAM
		#ifdef FTN_NON_DIFFSERV
			sram[read, FTN_CACHE[0], ftn_address, ftn_offset, 6], \
							ctx_swap[sig_fetch_nhlfe]
		#else
			sram[read, FTN_CACHE[0], ftn_address, ftn_offset, 8], \
							ctx_swap[sig_fetch_nhlfe]
		#endif
	#else
		#ifdef FTN_DRAM

			dram[read, FTN_CACHE[0], ftn_address, ftn_offset, 4], \
					sig_done[sig_fetch_nhlfe]
			ctx_arb[sig_fetch_nhlfe]

		#else
			#error "FTN NHLFE table is not allocated."
		#endif
	#endif

.end
#endm


// ----------------------------------------------------------------------------
// _mpls_phb2exp_exp_field_extract(exp_field, phb2exp_entry, EXP_NUM)
//
//    Description: Extracts EXP field from the PHB2EXP table entry.
//
//  	Outputs:
//			exp_field - extracted value
//
//		Inputs:
//			phb2exp_entry - PHB2EXP table entry
//
//		Constants:
//			EXP_NUM - specifies which entry should be extracted
//
// ----------------------------------------------------------------------------
#macro _mpls_phb2exp_exp_field_extract(exp_value, phb2exp_entry, EXP_NUM)

#if (streq(EXP_NUM, EXP0))

	alu_shf_right(exp_value, 0x7, AND, phb2exp_entry, 9)

#elif (streq(EXP_NUM, EXP1))

	alu_shf_right(exp_value, 0x7, AND, phb2exp_entry, 6)

#elif (streq(EXP_NUM, EXP2))

	alu_shf_right(exp_value, 0x7, AND, phb2exp_entry, 3)

#elif (streq(EXP_NUM, EXP3))

	alu_op(exp_value, 0x7, AND, phb2exp_entry)

#else

	#error 3 "in macro _mpls_phb2exp_exp_field_extract wrong argument EXP_NUM"

#endif

#endm


// ----------------------------------------------------------------------------
// _mpls_exp2phb_exp_extract(value)
//
//
//    Description: Extracts outEXP field from EXP2PHB table entry.
//
//
//  	Outputs:
//			value - extracted value
//
//
// ----------------------------------------------------------------------------
#macro _mpls_exp2phb_exp_extract(value)
	alu_op(value, 0x7, AND, EXP2PHB_CACHE[0])
#endm


// ----------------------------------------------------------------------------
// _mpls_push_labels_lm(push_count, TUNNEL_MODEL, DIFFSERV_SWITCH,
//					HDR_CACHE, HDR_WORD_OFFSET, FTN_CACHE, ttl)
//
//
//		Description: Appends labels to the IP datagram cache in LM.
//
//		Outputs:
//
//			push_count - number of labels appended
//
//		Inputs:
//
//			ttl - TTL value
//
//		Constants:
//
//			HDR_CACHE 	- packet header cache buffer
//			HDR_WORD_OFFSET - offset within the cache buffer
//							where packet data begins (in 32-bit words)
//			FTN_CACHE - FTN NHLF entry cache buffer
//			TUNNEL_MODEL - specifies tunnel model (UNIFORM or PIPE)
//			DIFFSERV_SWITCH - indicates DIFFSERV or NON_DIFFSERV LSP
//
// ----------------------------------------------------------------------------
#macro 	_mpls_push_labels_lm(push_count, labels_to_write, TUNNEL_MODEL, \
				DIFFSERV_SWITCH, HDR_CACHE, HDR_WORD_OFFSET, FTN_CACHE, ttl)
.begin

#define_eval HDR_WORD_OFFSET_1 HDR_WORD_OFFSET-1
#define_eval HDR_WORD_OFFSET_2 HDR_WORD_OFFSET-2
#define_eval HDR_WORD_OFFSET_3 HDR_WORD_OFFSET-3
#define_eval HDR_WORD_OFFSET_4 HDR_WORD_OFFSET-4

//TBD define MPLS_LABEL_MASK
.reg label_mask
    immed32(label_mask, 0xFFFFF000)
.reg top_label

	_mpls_ftn_push_count_extract(push_count)
	alu[top_label, FTN_CACHE[2], AND, label_mask]

	alu[--, push_count, - , 1]
	bne[two_labels#]

#if (streq(TUNNEL_MODEL, UNIFORM))
	alu[HDR_CACHE[HDR_WORD_OFFSET_1], top_label, OR, ttl]
#else
	alu[HDR_CACHE[HDR_WORD_OFFSET_1], top_label, OR, MPLS_PIPE_TTL]
#endif

	br[end_macro#]

two_labels#:
	alu[--, push_count, - , 2]
	bne[three_labels#]

#if (streq(TUNNEL_MODEL, UNIFORM))
	alu[HDR_CACHE[HDR_WORD_OFFSET_2], top_label, OR, ttl]
	alu[HDR_CACHE[HDR_WORD_OFFSET_1], FTN_CACHE[3], AND, label_mask]
	alu[HDR_CACHE[HDR_WORD_OFFSET_1], HDR_CACHE[HDR_WORD_OFFSET_1], OR, ttl]
#else
	alu[HDR_CACHE[HDR_WORD_OFFSET_2], top_label, OR, MPLS_PIPE_TTL]
	alu[HDR_CACHE[HDR_WORD_OFFSET_1], FTN_CACHE[3], AND, label_mask]
	alu[HDR_CACHE[HDR_WORD_OFFSET_1], HDR_CACHE[HDR_WORD_OFFSET_1], OR, MPLS_PIPE_TTL]
#endif

	br[end_macro#]

three_labels#:
		alu[--, push_count, - , 3]
		bne[four_labels#]

#if (streq(TUNNEL_MODEL, UNIFORM))
	alu[HDR_CACHE[HDR_WORD_OFFSET_3], top_label, OR, ttl]
	alu[HDR_CACHE[HDR_WORD_OFFSET_2], FTN_CACHE[3], AND, label_mask]
	alu[HDR_CACHE[HDR_WORD_OFFSET_2], HDR_CACHE[HDR_WORD_OFFSET_2], OR, ttl]
	alu[HDR_CACHE[HDR_WORD_OFFSET_1], FTN_CACHE[4], AND, label_mask]
	alu[HDR_CACHE[HDR_WORD_OFFSET_1], HDR_CACHE[HDR_WORD_OFFSET_1], OR, ttl]
#else
	alu[HDR_CACHE[HDR_WORD_OFFSET_3], top_label, OR, MPLS_PIPE_TTL]
	alu[HDR_CACHE[HDR_WORD_OFFSET_2], FTN_CACHE[3], AND, label_mask]
	alu[HDR_CACHE[HDR_WORD_OFFSET_2], HDR_CACHE[HDR_WORD_OFFSET_2], OR, MPLS_PIPE_TTL]
	alu[HDR_CACHE[HDR_WORD_OFFSET_1], FTN_CACHE[4], AND, label_mask]
	alu[HDR_CACHE[HDR_WORD_OFFSET_1], HDR_CACHE[HDR_WORD_OFFSET_1], OR, MPLS_PIPE_TTL]
#endif

	br[end_macro#]

four_labels#:

#if (streq(TUNNEL_MODEL, UNIFORM))
	alu[HDR_CACHE[HDR_WORD_OFFSET_4], top_label, OR, ttl]
	alu[HDR_CACHE[HDR_WORD_OFFSET_3], FTN_CACHE[3], AND, label_mask]
	alu[HDR_CACHE[HDR_WORD_OFFSET_3], HDR_CACHE[HDR_WORD_OFFSET_3], OR, ttl]
	alu[HDR_CACHE[HDR_WORD_OFFSET_2], FTN_CACHE[4], AND, label_mask]
	alu[HDR_CACHE[HDR_WORD_OFFSET_2], HDR_CACHE[HDR_WORD_OFFSET_2], OR, ttl]
	alu[HDR_CACHE[HDR_WORD_OFFSET_1], FTN_CACHE[5], AND, label_mask]
	alu[HDR_CACHE[HDR_WORD_OFFSET_1], HDR_CACHE[HDR_WORD_OFFSET_1], OR, ttl]
#else
	alu[HDR_CACHE[HDR_WORD_OFFSET_4], top_label, OR, MPLS_PIPE_TTL]
	alu[HDR_CACHE[HDR_WORD_OFFSET_3], FTN_CACHE[3], AND, label_mask]
	alu[HDR_CACHE[HDR_WORD_OFFSET_3], HDR_CACHE[HDR_WORD_OFFSET_3], OR, MPLS_PIPE_TTL]
	alu[HDR_CACHE[HDR_WORD_OFFSET_2], FTN_CACHE[4], AND, label_mask]
	alu[HDR_CACHE[HDR_WORD_OFFSET_2], HDR_CACHE[HDR_WORD_OFFSET_2], OR, MPLS_PIPE_TTL]
	alu[HDR_CACHE[HDR_WORD_OFFSET_1], FTN_CACHE[5], AND, label_mask]
	alu[HDR_CACHE[HDR_WORD_OFFSET_1], HDR_CACHE[HDR_WORD_OFFSET_1], OR, MPLS_PIPE_TTL]
#endif

end_macro#:
	
	// set S-bit for the bottom label
	alu_shf[HDR_CACHE[HDR_WORD_OFFSET_1], HDR_CACHE[HDR_WORD_OFFSET_1], OR, 1, <<MPLS_STACK_BIT]

	alu_op(labels_in_cache, labels_in_cache, +, push_count)

#if (streq(TUNNEL_MODEL, UNIFORM))
	move(labels_to_write, push_count)
#else
	alu_op(labels_to_write, labels_to_write, +, push_count)
#endif

#undef HDR_WORD_OFFSET_1
#undef HDR_WORD_OFFSET_2
#undef HDR_WORD_OFFSET_3
#undef HDR_WORD_OFFSET_4

.end
#endm


// ----------------------------------------------------------------------------
// _mpls_swap_push_labels_lm(push_count, TUNNEL_MODEL, DIFFSERV_SWITCH,
//					HDR_POINTER, ILM_CACHE, ttl)
//
//
//		Description: Swaps MPLS label on the top of the stack and appends
//					one to three new labels to the MPLS packet header
//					cached in LM.
//
//		Outputs:
//
//			push_count - number of labels appended
//
//		Inputs:
//
//			ttl - TTL value
//
//		Constants:
//
//			HDR_POINTER 	- points to the header cache buffer
//							where packet data begins
//			ILM_CACHE - NHLF entry cache buffer
//			TUNNEL_MODEL - specifies tunnel model (UNIFORM or PIPE)
//			DIFFSERV_SWITCH - indicates DIFFSERV or NON_DIFFSERV LSP
//
// ----------------------------------------------------------------------------
#macro _mpls_swap_push_labels_lm(push_count, TUNNEL_MODEL, DIFFSERV_SWITCH, \
						HDR_POINTER, ILM_CACHE, ttl)
.begin
.reg label_value

	_mpls_ilm_push_count_extract(push_count)

	alu[--, push_count, - , 1]
	bne[two_labels#]

	alu_shf[HDR_POINTER, HDR_POINTER, AND, 1, <<8]	// preserve label's S-bit

	ld_field_w_clr[label_value, 1110, ILM_CACHE[3]]	//extract label value
	alu[HDR_POINTER, HDR_POINTER, OR, label_value]

	alu[HDR_POINTER--, HDR_POINTER, OR, ttl]

	alu[HDR_POINTER, ILM_CACHE[2], AND~, 0xFF]

#if (streq(TUNNEL_MODEL, PIPE))
	move(ttl, MPLS_PIPE_TTL)
#endif
	alu[HDR_POINTER, HDR_POINTER, OR, ttl]

	br[end_macro#]

two_labels#:
	alu[--, push_count, - , 2]
	bne[three_labels#]

	alu_shf[HDR_POINTER, HDR_POINTER, AND, 1, <<8]	// preserve label's S-bit

	ld_field_w_clr[label_value, 1110, ILM_CACHE[4]]	//extract label value
	alu[HDR_POINTER, HDR_POINTER, OR, label_value]

	alu[HDR_POINTER--, HDR_POINTER, OR, ttl]

	alu[HDR_POINTER, ILM_CACHE[3], AND~, 0xFF]

#if (streq(TUNNEL_MODEL, PIPE))
	move(ttl, MPLS_PIPE_TTL)
#endif

	alu[HDR_POINTER--, HDR_POINTER, OR, ttl]

	alu[HDR_POINTER, ILM_CACHE[2], AND~, 0xFF]

	alu[HDR_POINTER, HDR_POINTER, OR, ttl]

	br[end_macro#]

three_labels#:

	alu_shf[HDR_POINTER, HDR_POINTER, AND, 1, <<8]	// preserve label's S-bit

	ld_field_w_clr[label_value, 1110, ILM_CACHE[5]]	//extract label value
	alu[HDR_POINTER, HDR_POINTER, OR, label_value]

	alu[HDR_POINTER--, HDR_POINTER, OR, ttl]

	alu[HDR_POINTER, ILM_CACHE[4], AND~, 0xFF]

#if (streq(TUNNEL_MODEL, PIPE))
	move(ttl, MPLS_PIPE_TTL)
#endif

	alu[HDR_POINTER--, HDR_POINTER, OR, ttl]

	alu[HDR_POINTER, ILM_CACHE[3], AND~, 0xFF]

	alu[HDR_POINTER--, HDR_POINTER, OR, ttl]

	alu[HDR_POINTER, ILM_CACHE[2], AND~, 0xFF]

	alu[HDR_POINTER, HDR_POINTER, OR, ttl]

end_macro#:

	alu_op(labels_in_cache, labels_in_cache, +, push_count)
	alu_op(labels_to_write, push_count, +, 1)

.end
#endm


// ----------------------------------------------------------------------------
// _mpls_swap_labels_lm(HDR_POINTER, ILM_CACHE, ttl)
//
//
//		Description: Swaps MPLS label on the top of the label stack.
//
//		Inputs:
//
//			ttl - TTL value
//
//		Constants:
//
//			HDR_POINTER - points to the packet header cache buffer
//							where packet data begins
//			ILM_CACHE - NHLF entry cache buffer
//
// ----------------------------------------------------------------------------
#macro _mpls_swap_labels_lm(DIFFSERV_SWITCH, HDR_POINTER, ILM_CACHE, ttl)
.begin

	alu_shf[HDR_POINTER, HDR_POINTER, AND, 1, <<8]	// preserve label's S-bit
.reg label_value
	ld_field_w_clr[label_value, 1110, ILM_CACHE[2]]	//extract label value
	alu[HDR_POINTER, HDR_POINTER, OR, label_value]
	alu[HDR_POINTER, HDR_POINTER, OR, ttl]

	immed32(labels_to_write, 1)
.end
#endm

// ----------------------------------------------------------------------------
// _mpls_fetch_exp2phb(exp2phb_address, exp2phb_offset)
//
//
//    Description: Read EXP2PHB table entry from DRAM or SRAM.
//
//  	Inputs:
//			exp2phb_address - base memory address
//			exp2phb_offset  - memory offset
//
// ----------------------------------------------------------------------------
#macro _mpls_fetch_exp2phb(exp2phb_address, exp2phb_offset)
.begin

.sig sig_fetch_exp2phb


	#ifdef EXP2PHB_SRAM

// Dennis March 31
//		sram[read, EXP2PHB_CACHE[0], exp2phb_address, exp2phb_offset, 2], \
//						ctx_swap[sig_fetch_exp2phb]
		sram[read, EXP2PHB_CACHE[0], exp2phb_address, exp2phb_offset, 1], \
						ctx_swap[sig_fetch_exp2phb]
	#else
		#ifdef EXP2PHB_DRAM

			dram[read, EXP2PHB_CACHE[0], exp2phb_address, exp2phb_offset, 1], \
					sig_done[sig_fetch_exp2phb]
			ctx_arb[sig_fetch_exp2phb]

		#else
			#error "EXP2PHB table is not allocated."
		#endif
	#endif

.end
#endm


// ----------------------------------------------------------------------------
// _mpls_set_diffserv_dl_variables(EXP2PHB_CACHE)
//
//
//    Description: Set flow_id, class_id and color dispatch loop variables.
//
//  	Inputs:
//			EXP2PHB_CACHE - EXP2PHB table entry cache
//
// ----------------------------------------------------------------------------
#macro _mpls_set_diffserv_dl_variables(EXP2PHB_CACHE)
.begin

// Dennis March 31
.reg val
	ld_field_w_clr[val, 0011, EXP2PHB_CACHE[0], >>16]
	dl_meta_set_flow_id(val)

	ld_field_w_clr[val, 0001, EXP2PHB_CACHE[0], >>8]
	dl_meta_set_class_id(val)

	alu_shf[val, 0xF, AND, EXP2PHB_CACHE[0], >>4]
	dl_meta_set_color(val)

.end
#endm


// ----------------------------------------------------------------------------
// _mpls_fetch_phb2exp(phb2exp_entry, phb2exp_address, phb2exp_offset)
//
//
//    Description: Read PHB2EXP table entry from DRAM or SRAM.
//
//		Outputs:
//			phb2exp_entry   - fetched phb2exp entry
//  	Inputs:
//			phb2exp_address - base memory address
//			phb2exp_offset  - memory offset
//
// ----------------------------------------------------------------------------
#macro _mpls_fetch_phb2exp(phb2exp_entry, phb2exp_address, phb2exp_offset)
.begin
.sig sig_fetch_phb2exp


	#ifdef PHB2EXP_SRAM

		sram[read, PHB2EXP_CACHE[0], phb2exp_address, phb2exp_offset, 1], \
						ctx_swap[sig_fetch_phb2exp]
		alu[phb2exp_entry, --, B, PHB2EXP_CACHE[0]]

	#else
		#ifdef PHB2EXP_DRAM

			dram[read, PHB2EXP_CACHE[0], phb2exp_address, phb2exp_offset, 1], \
					sig_done[sig_fetch_phb2exp]
			ctx_arb[sig_fetch_phb2exp]

			alu[--, phb2exp_offset, AND, 4]
			beq[entry_in_first_reg#]
			alu[phb2exp_entry, --, B, PHB2EXP_CACHE[1]]
			br[end_macro#]
entry_in_first_reg#:

			alu[phb2exp_entry, --, B, PHB2EXP_CACHE[0]]
end_macro#:
		#else
			#error "PHB2EXP table is not allocated."
		#endif
	#endif

.end
#endm


// ----------------------------------------------------------------------------
// _mpls_push_router_alert_label(HDR_POINTER, ttl)
//
//    Description: Appends ROUTER_ALERT label to the MPLS
//                 label stack cached in LM.
//				   This macro is used when position of ROUTER_ALERT label
//				   in the label stack is known at compile time.
//
//  	Inputs:
//			ttl - TTL value
//
//		Constants:
//			HDR_POINTER - points to the packet header cache buffer
//							where packet data begins
//
// ----------------------------------------------------------------------------
#macro _mpls_push_router_alert_label(HDR_POINTER, ttl)

	alu[--, --, B, HDR_POINTER--]
	alu_shf_left(HDR_POINTER, ttl, OR, 1, 12)

	alu_op(labels_in_cache, labels_in_cache, +, 1)

	alu_op(labels_to_write, labels_to_write, +, 1)

#endm


// ----------------------------------------------------------------------------
// _mpls_ilm_operation_extract(value)
//
//    Description: Extracts operation field from ILM table entry.
//
//  	Outputs:
//			value - extracted value
//
//
// ----------------------------------------------------------------------------
#macro _mpls_ilm_operation_extract(value)

	alu_shf_right(value, 0x3, AND, ILM_CACHE[0], 22)

#endm


// ----------------------------------------------------------------------------
// _mpls_ilm_tunnel_model_extract(value)
//
//    Description: Extracts MPLS tunnel model flag from
//                 flags field from ILM table entry.
//
//  	Outputs:
//			value - extracted value
//
//
// ----------------------------------------------------------------------------
#macro _mpls_ilm_tunnel_model_extract(value)

	alu_shf_right(value, 0x3, AND, ILM_CACHE[0], 28)

#endm

// ----------------------------------------------------------------------------
// _mpls_ilm_ttl_extract(value)
//
//
//    Description: Extracts TTL field from ILM NHLF entry.
//
//  	Outputs:
//			value - extracted value
//
//
// ----------------------------------------------------------------------------
#macro _mpls_ilm_ttl_extract(value)

	alu_op(value, 0xF, AND, ILM_CACHE[2])

#endm

// ----------------------------------------------------------------------------
// _mpls_ilm_lsp_type_extract(value)
//
//    Description: Extracts MPLS LSP type flag from
//                 flags field from NHLF entry.
//
//  	Outputs:
//			value - extracted value
//
//
// ----------------------------------------------------------------------------
#macro _mpls_ilm_lsp_type_extract(value)

	alu_shf_right(value, 0x3, AND, ILM_CACHE[0], 26)

#endm

// ----------------------------------------------------------------------------
// _mpls_ilm_push_count_extract(value)
//
//
//    Description: Extracts pushCount fields from NHLF entry.
//
//  	Outputs:
//			value - extracted value
//
//
//
// ----------------------------------------------------------------------------
#macro _mpls_ilm_push_count_extract(value)

	alu_shf_right(value, 0x7, AND, ILM_CACHE[0], 19)

#endm

// ----------------------------------------------------------------------------
// _mpls_ilm_outport_extract(value)
//
//
//    Description: Extracts output port field from NHLF entry.
//
//
//  	Outputs:
//			value - extracted value
//
//
// ----------------------------------------------------------------------------
#macro _mpls_ilm_outport_extract(value)

	alu_op(value, 0, +16, ILM_CACHE[0])

#endm

// ----------------------------------------------------------------------------
// _mpls_ilm_max_lab_pkt_size_extract(value)
//
//
//    Description: Extracts max_lab_pkt_size field from NHLF entry.
//
//
//  	Outputs:
//			value - extracted value
//
//
// ----------------------------------------------------------------------------
#macro _mpls_ilm_max_lab_pkt_size_extract(value)

	shf_right(value, ILM_CACHE[1], 16)

#endm

// ----------------------------------------------------------------------------
// _mpls_ilm_l3protocol_extract(value)
//
//
//    Description: Extracts max_lab_pkt_size field from NHLF entry.
//
//
//  	Outputs:
//			value - extracted value
//
//
// ----------------------------------------------------------------------------
#macro _mpls_ilm_l3protocol_extract(value)

	alu_shf_right(value, 0x7, AND, ILM_CACHE[0], 16)

#endm

// ----------------------------------------------------------------------------
// _mpls_ftn_lsp_type_extract(value)
//
//    Description: Extracts MPLS LSP type flag from
//                 flags field from NHLF entry.
//
//  	Outputs:
//			value - extracted value
//
//
// ----------------------------------------------------------------------------
#macro _mpls_ftn_lsp_type_extract(value)

	alu_shf_right(value, 0x3, AND, FTN_CACHE[0], 26)

#endm


// ----------------------------------------------------------------------------
// _mpls_ftn_tunnel_model_extract(value)
//
//    Description: Extracts MPLS tunnel model flag from
//                 flags field from NHLF entry.
//
//  	Outputs:
//			value - extracted value
//
//
// ----------------------------------------------------------------------------
#macro _mpls_ftn_tunnel_model_extract(value)

	alu_shf_right(value, 0x3, AND, FTN_CACHE[0], 28)

#endm


// ----------------------------------------------------------------------------
// _mpls_ftn_operation_extract(value)
//
//    Description: Extracts operation field from NHLF entry.
//
//  	Outputs:
//			value - extracted value
//
//
// ----------------------------------------------------------------------------
#macro _mpls_ftn_operation_extract(value)

	alu_shf_right(value, 0x3, AND, FTN_CACHE[0], 22)

#endm


// ----------------------------------------------------------------------------
// _mpls_ftn_push_count_extract(value)
//
//
//    Description: Extracts pushCount fields from NHLF entry.
//
//  	Outputs:
//			value - extracted value
//
//
//
// ----------------------------------------------------------------------------
#macro _mpls_ftn_push_count_extract(value)

	alu_shf_right(value, 0x7, AND, FTN_CACHE[0], 19)

#endm


// ----------------------------------------------------------------------------
// _mpls_ftn_ttl_extract(value)
//
//
//    Description: Extracts TTL field from NHLF entry.
//
//
//  	Outputs:
//			value - extracted value
//
//
// ----------------------------------------------------------------------------
#macro _mpls_ftn_ttl_extract(value)

	alu_op(value, 0xF, AND, FTN_CACHE[2])

#endm


// ----------------------------------------------------------------------------
// _mpls_ftn_blade_id_extract(value)
//
//
//    Description: Extracts blade ID field from NHLF entry.
//
//
//  	Outputs:
//			value - extracted value
//
//
// ----------------------------------------------------------------------------
#macro _mpls_ftn_blade_id_extract(value)

	alu_op(value, 0xFF, AND, FTN_CACHE[3])

#endm


// ----------------------------------------------------------------------------
// _mpls_ftn_outport_extract(value)
//
//
//    Description: Extracts output port field from NHLF entry.
//
//
//  	Outputs:
//			value - extracted value
//
//
// ----------------------------------------------------------------------------
#macro _mpls_ftn_outport_extract(value)

	alu_op(value, 0, +16, FTN_CACHE[0])

#endm


// ----------------------------------------------------------------------------
// _mpls_ftn_max_lab_pkt_size_extract(value)
//
//
//    Description: Extracts max_lab_pkt_size field from NHLF entry.
//
//
//  	Outputs:
//			value - extracted value
//
//
// ----------------------------------------------------------------------------
#macro _mpls_ftn_max_lab_pkt_size_extract(value)

	shf_right(value, FTN_CACHE[1], 16)

#endm


// ----------------------------------------------------------------------------
// _mpls_cache_ip_header(labels_in_cache)
//
//
//    Description: IP forwarder assumes that the header is cached at certain -
//					specified as a compilation constant - offset. This macro
//					prepares data in cache so that IP forwarder could
//					interpret it properly.
//
//  	Input:
//			labels_in_cache - dispatch loop support variable specifying how
//				many valid LWs (or MPLS labels) of data are already in cache.
//
// ----------------------------------------------------------------------------
#macro _mpls_cache_ip_header(labels_in_cache)
.begin

	.reg lm_address

    #if (PKT_HDR_CACHE_SIZE > 6)
	alu[--, 5, -, labels_in_cache]
	ble[no_dram_read#]
    #endif

    .sig sig_hdr_read

	xbuf_alloc($$header_read, 8, read)

   .reg dram_handle 
   .reg dl_packet_offset
   .reg qword_offset

	// Register T_INDEX_BYTE_INDEX holds:
	//	- transfer register address on bits 8:2
	//  - quadword offset on bits 7:0
	.reg t_idx_byte_idx

	dl_meta_get_offset(dl_packet_offset)

	dl_buf_get_data(dram_handle, dl_buf_handle)
	alu[qword_offset, dl_packet_offset, AND, 0x7]

	alu[--, 4, -, qword_offset]
	bge[small_offset#]

    // qword_offset greater than 4
	// read packet header and reload LM pointers
	dram_read($$header_read[0], dram_handle, dl_packet_offset,
			  4, sig_hdr_read, SIG_NONE, ___)

	// Read transfer register address and move it to bits 8:2.
	// ctx_rel_xfer_addr holds context-relative transfer register base addres.

	alu_shf[t_idx_byte_idx, ctx_rel_xfer_addr, OR , &$$header_read[0], <<2]

	// Add quadword offset on bits 2:0
	//
	//
	// Generally, if quadword offset is between 4-7 (and thus bit 2 is set),
	// we will add one to T_INDEX field and we will start reading data
	// from $$ip_header_read[1].

	alu[t_idx_byte_idx, t_idx_byte_idx, +, qword_offset]

	ctx_arb[sig_hdr_read], br[align_hdr#]

small_offset#:
    // qword_offset less than or equal 4
   .set $$header_read6
   .set $$header_read7

	// read packet header and reload LM pointers
	dram_read($$header_read[0], dram_handle, dl_packet_offset,
			  3, sig_hdr_read, SIG_NONE, ___)

	// Read transfer register address and move it to bits 8:2.
	// ctx_rel_xfer_addr holds context-relative transfer register base addres.

	alu_shf[t_idx_byte_idx, ctx_rel_xfer_addr, OR , &$$header_read[0], <<2]

	// Add quadword offset on bits 2:0
	//
	//
	// Generally, if quadword offset is between 4-7 (and thus bit 2 is set),
	// we will add one to T_INDEX field and we will start reading data
	// from $$ip_header_read[1].

	alu[t_idx_byte_idx, t_idx_byte_idx, +, qword_offset]

	ctx_arb[sig_hdr_read], br[align_hdr#]


#if (PKT_HDR_CACHE_SIZE > 6)
no_dram_read#:
	alu[HDR_CACHE[4], --, B, HDR_POINTER++]
	alu[HDR_CACHE[5], --, B, HDR_POINTER++]
	alu[HDR_CACHE[6], --, B, HDR_POINTER++]
	alu[HDR_CACHE[7], --, B, HDR_POINTER++]
	alu[HDR_CACHE[8], --, B, HDR_POINTER++]
	br[finish_cache#]
#endif


align_hdr#:

	local_csr_wr[T_INDEX_BYTE_INDEX, t_idx_byte_idx]

	// there must be three instructions between
	// local_csr_wr and byte_align_be

	immed32(labels_in_cache, (PKT_HDR_CACHE_SIZE - 1))
	nop
	nop


.begin
.reg tmp_cache

	// Here we align and load labeled packet or ipv4 packet header.
	byte_align_be[--, *$$index++]
	byte_align_be[HDR_CACHE[4], *$$index++]
	byte_align_be[HDR_CACHE[5], *$$index++]
	byte_align_be[HDR_CACHE[6], *$$index++]
	byte_align_be[HDR_CACHE[7], *$$index++]
	byte_align_be[tmp_cache, *$$index++]
	move(HDR_CACHE[8], tmp_cache)
.end  // .reg tmp_cache


finish_cache#:


	xbuf_free($$header_read)
.end
#endm


// ----------------------------------------------------------------------------
// _mpls_update_ipv4_header_lm(out_hdr, HDR_OFFSET, ttl)
//
//    Description: Update IPv4 header in the LM cache
//
//    Parameters:
//		In/Out:		out_hdr 	- header cache
//      Inputs: 	HDR_WORD_OFFSET - header cache offset (in 32-bit words)
//					ttl - TTL value to be put in mpls labels except for
//						the top label which may be different and
//						is set elsewhere
//
// ----------------------------------------------------------------------------
#macro _mpls_update_ipv4_header_lm(out_hdr, HDR_WORD_OFFSET, ttl)

#define_eval HDR_WORD_OFFSET_2 HDR_WORD_OFFSET + 2

.begin
.reg checksum
.reg old_ttl
.reg new_ttl

	ld_field_w_clr[checksum, 0011, out_hdr[HDR_WORD_OFFSET_2]]
	ld_field_w_clr[old_ttl, 0011, out_hdr[HDR_WORD_OFFSET_2], >>16]
	ld_field_w_clr[new_ttl, 0001, out_hdr[HDR_WORD_OFFSET_2], >>16]
	alu_shf[new_ttl, new_ttl, OR, ttl, <<8]


	// Incremental checksum computation per RFC 1624
	alu[old_ttl, --, ~B, old_ttl]
	alu[checksum, checksum, -, old_ttl]
	alu[checksum, checksum, -carry, new_ttl]
	alu[checksum, checksum, -carry, 0]

.reg updated_checksum

	// Rewrite checksum to IP header
	ld_field_w_clr[updated_checksum, 0011, checksum]
	alu_shf[out_hdr[HDR_WORD_OFFSET_2], updated_checksum, OR, new_ttl, <<16]

.end

#undef HDR_WORD_OFFSET_2

#endm


// ---------------------------------------------------------------------------
// _mpls_get_out_port_stat_addr(out_port_stat_addr, blade_id, output_port)
//
//		Description:
//
// 			Gets address to output port statistics from blade_id
//			and output port number.
//
//
//  	Inputs:
//
//			blade_id - the id of blade that hosts output port
//			output_port - output port number
//
//  	Outputs:
//			out_port_stat_addr - statistics base for particular port
//
//
// ---------------------------------------------------------------------------
#macro _mpls_get_out_port_stat_addr(out_port_stat_addr, blade_id, output_port)

	add_shf_left(out_port_stat_addr, @out_port_stat, blade_id, 9)
	add_shf_left(out_port_stat_addr, out_port_stat_addr, output_port, 4)

#endm


// ---------------------------------------------------------------------------
// _mpls_get_in_port_stat_addr(in_port_stat_addr, input_port)
//
//		Description:
//
// 			Gets address to input port statistics from input port number.
//
//
//  	Inputs:
//
//			input_port - input port number
//
//  	Outputs:
//			in_port_stat_addr - statistics base for particular port
//
// ---------------------------------------------------------------------------
#macro _mpls_get_in_port_stat_addr(in_port_stat_addr, input_port)

	add_shf_left(in_port_stat_addr, @in_port_stat, input_port, 4)

#endm


// ---------------------------------------------------------------------------
// _mpls_update_in_segment_counters(packet_size)
//
//		Description:
//
// 			Updates input segment statistics.
//
//
//  	Inputs:
//
//			packet_size - packet size in bytes
//
//
// ---------------------------------------------------------------------------
#macro _mpls_update_in_segment_counters(packet_size)
.begin

	// Update number of packets sent on this input segment
	sram_incr(ILM_CACHE[4], MPLS_IN_SEGMENT_PKTS)
	#ifdef MPLS_STATS_FULL

	.sig sig_stat
	xbuf_alloc($packet_size, 1, write)
	move($packet_size[0], packet_size)

		// Update number of octets sent on this input segment
		sram_add($packet_size[0], ILM_CACHE[4],
				MPLS_IN_SEGMENT_OCTETS, sig_stat, sig_stat, ___)

	xbuf_free($packet_size)
	#endif
.end
#endm

// ---------------------------------------------------------------------------
// _mpls_update_incoming_counters(in_val, packet_size)
//
//		Description:
//
// 			Updates input FTN statistics.
//
//
//  	Inputs:
//
//			in_val 			- incoming FEC value
//			packet_size 	- packet size in bytes
//
// ---------------------------------------------------------------------------
#macro _mpls_update_incoming_counters_ftn(in_val, packet_size)
.begin
.reg out_counter_addr

	immed32(out_counter_addr, MPLS_FTN_STAT_BASE)
	add_shf_left(out_counter_addr, out_counter_addr, in_val, 4)
	add_shf_left(out_counter_addr, out_counter_addr, in_val, 3)

	// Update number of packets sent on this input FTN
	sram_incr(out_counter_addr, MPLS_IN_SEGMENT_PKTS)
	#ifdef MPLS_STATS_FULL

	.sig sig_stat
	xbuf_alloc($packet_size, 1, write)
	move($packet_size[0], packet_size)

		// Update number of octets sent on this input FTN
		sram_add($packet_size[0], out_counter_addr,
				MPLS_IN_SEGMENT_OCTETS, sig_stat, sig_stat, ___)
	xbuf_free($packet_size)

	#endif
.end
#endm

// ---------------------------------------------------------------------------
// _mpls_update_incoming_counters_ilm(address, packet_size)
//
//		Description:
//
// 			Updates input ILM statistics.
//
//
//  	Inputs:
//
//			address			- address to the Input ILM Statistics
//			packet_size 	- packet size in bytes
//
// ---------------------------------------------------------------------------
#macro _mpls_update_incoming_counters_ilm(address, packet_size)
.begin

	sram_incr(address, MPLS_IN_SEGMENT_PKTS)
	#ifdef MPLS_STATS_FULL

	.sig sig_stat
	xbuf_alloc($packet_size, 1, write)
	move($packet_size[0], packet_size)

		// Update number of octets sent on this input ILM
		sram_add($packet_size[0], address,
				MPLS_IN_SEGMENT_OCTETS, sig_stat, sig_stat, ___)
	xbuf_free($packet_size)

	#endif
.end
#endm

// ---------------------------------------------------------------------------
// _mpls_update_in_segment_discard_counter()
//
//		Description:
//
// 			Updates discarded packet received by input segment. .
//
//
//  	Inputs:
//
//
// ---------------------------------------------------------------------------
#macro _mpls_update_in_segment_discard_counter()
.begin

	sram_incr(ILM_CACHE[4], MPLS_IN_SEGMENT_DISCARDS)

.end
#endm

// ---------------------------------------------------------------------------
// _mpls_update_in_ILM_discard_counter(incoming_label)
//
//		Description:
//
// 			Updates discarded packet received by input ILM.
//
//
//  	Inputs:
//
//			incoming_label	- incoming MPLS label
// ---------------------------------------------------------------------------
#macro _mpls_update_in_ILM_discard_counter(incoming_label)
.begin
.reg out_counter_addr

	immed32(out_counter_addr, MPLS_ILM_STAT_BASE)
	add_shf_left(out_counter_addr, out_counter_addr, incoming_label, 4)
	sram_incr(out_counter_addr, MPLS_IN_SEGMENT_DISCARDS)

.end
#endm


// ----------------------------------------------------------------------------
// _mpls_update_tx_counters(FTN_CACHE, packet_size)
//
//    Description:
//
//			Updates output port and output segment counters.
//
//
// ----------------------------------------------------------------------------
#macro _mpls_update_tx_counters(FTN_CACHE, packet_size)
.begin

.reg blade_id output_port

	_mpls_ftn_blade_id_extract(blade_id)
	_mpls_ftn_outport_extract(output_port)

.reg out_seg_stat_addr
.reg out_port_stat_addr

	// Update number of packets sent on out-segment
	sram_incr(FTN_CACHE[6], MPLS_OUT_SEGMENT_PKTS)

	_mpls_get_out_port_stat_addr(out_port_stat_addr, blade_id, output_port)

	// Update number of packets sent on this interface
	sram_incr(out_port_stat_addr, MPLS_OUT_IFC_PKTS)


	#ifdef MPLS_STATS_FULL

	.sig sig_stat_3
	.sig sig_stat_4

		xbuf_alloc($packet_size, 1, write)
		move($packet_size[0], packet_size)

		// Update number of octets sent on this out-segment
		sram_add($packet_size[0], FTN_CACHE[6],
				MPLS_OUT_SEGMENT_OCTETS, sig_stat_4, SIG_NONE, ___)

		// Update number of octets sent on this interface
		sram_add($packet_size[0], out_port_stat_addr, MPLS_OUT_IFC_OCTETS,
					sig_stat_3,	SIG_NONE, ___)

		ctx_arb[sig_stat_3, sig_stat_4]
		xbuf_free($packet_size)

	#endif

.end
#endm


// ----------------------------------------------------------------------------
// _mpls_update_meta_data(NHLFE_CACHE)
//
//    Description:
//			Sets meta data from ILM or FTN NHLFE.
//
//    Inputs:
//			NHLFE_CACHE	- ILM_CACHE or FTN_CACHE
//								
// ----------------------------------------------------------------------------
#macro _mpls_update_meta_data(NHLFE_CACHE)

#if (streq(NHLFE_CACHE, ILM_CACHE))

	dl_meta_set_nexthop_id(ILM_CACHE[0])
	dl_meta_set_fabric_port(ILM_CACHE[3])
	dl_meta_set_output_port(ILM_CACHE[1])

#elif (streq(NHLFE_CACHE, FTN_CACHE))

	dl_meta_set_nexthop_id(FTN_CACHE[0])
	dl_meta_set_fabric_port(FTN_CACHE[3])
	dl_meta_set_output_port(FTN_CACHE[1])

#else
	#error "_mpls_update_meta_data: Must be either ILM_CACHE or FTN_CACHE."
#endif

#endm


// ---------------------------------------------------------------------------
// _mpls_update_dl_size(number_of_labels)
//
//
//    Description:
//
// 			Update dl_packet_size, dl_packet_offset and dl_buf_size
//				meta data variables.
//
//    Inputs:
//
//			number_of_labels - number of MPLS labels pushed onto
//								 the label stack
//
// ---------------------------------------------------------------------------
#macro _mpls_update_dl_size(number_of_labels)
.begin
.reg dl_packet_offset dl_packet_size dl_buf_size

		dl_meta_get_offset(dl_packet_offset)
		alu_shf_left(dl_packet_offset, dl_packet_offset, - , number_of_labels, 2)
		dl_meta_set_offset(dl_packet_offset)

		dl_meta_get_packet_size(dl_packet_size)
		alu_shf_left(dl_packet_size, dl_packet_size, + , number_of_labels, 2)
		dl_meta_set_packet_size(dl_packet_size)

		dl_meta_get_buffer_size(dl_buf_size)
		alu_shf_left(dl_buf_size, dl_buf_size, + , number_of_labels, 2)
		dl_meta_set_buffer_size(dl_buf_size)

.end
#endm


// ---------------------------------------------------------------------------
// _mpls_update_rx_port_counters(input_port, packet_size)
//
//
//    Description:
//
// 			Update input port statistics.
//
// ---------------------------------------------------------------------------
#macro _mpls_update_rx_port_counters(input_port, packet_size)
.begin

.reg in_port_stat_addr		// Input port statistics address

	_mpls_get_in_port_stat_addr(in_port_stat_addr, input_port)
	sram_incr(in_port_stat_addr, MPLS_IN_IFC_PKTS)


	#ifdef MPLS_STATS_FULL
	.begin

		.sig sig_stat_1

		xbuf_alloc($p_size, 1, write)
		move($p_size[0], packet_size)
		sram_add($p_size[0], in_port_stat_addr, MPLS_IN_IFC_OCTETS,
					sig_stat_1, sig_stat_1, ___)
		xbuf_free($p_size)
	.end
	#endif
.end
#endm


// ---------------------------------------------------------------------------
// _mpls_update_failed_lookup_counter(input_port)
//
//
//    Description:
//
// 			Updates failed lookup counter for given input port.
//
// ---------------------------------------------------------------------------
#macro _mpls_update_failed_lookup_counter(input_port)
.begin
.reg in_port_stat_addr		// Input port statistics address

	_mpls_get_in_port_stat_addr(in_port_stat_addr, input_port)
	sram_incr(in_port_stat_addr, MPLS_IN_IFC_FAILED_LOOKUP_PKTS)
.end
#endm


// ---------------------------------------------------------------------------
// _mpls_update_in_ifc_error_counter(input_port)
//
//
//    Description:
//
// 			Updates error counter for given input port.
//
// ---------------------------------------------------------------------------
#macro _mpls_update_in_ifc_error_counter(input_port)
.begin
.reg in_port_stat_addr		// Input port statistics address

	_mpls_get_in_port_stat_addr(in_port_stat_addr, input_port)
	sram_incr(in_port_stat_addr, MPLS_IN_IFC_ERROR_PKTS)
.end
#endm


// ---------------------------------------------------------------------------
// _mpls_update_in_ifc_disabled_counter(input_port)
//
//
//    Description:
//
// 			Updates port disabled counter for given input port.
//
// ---------------------------------------------------------------------------
#macro _mpls_update_in_ifc_disabled_counter(input_port)
.begin
.reg in_port_stat_addr		// Input port statistics address

	_mpls_get_in_port_stat_addr(in_port_stat_addr, input_port)
	sram_incr(in_port_stat_addr, MPLS_IN_IFC_DISABLE_PKTS)
.end
#endm


// ---------------------------------------------------------------------------
// _mpls_select_nhlfe_from_set(in_cache, out_nhlfe_offset)
//
//
//    Description:
//
// 			Select an NHLF entry from NHLFE set.
//
//	  Inputs:
//			in_cache - ILM_CACHE or FTN_CACHE
//
// ---------------------------------------------------------------------------
#macro _mpls_select_nhlfe_from_set(in_cache, out_nhlfe_offset)
	alu_shf[out_nhlfe_offset, --, B, in_cache[5], >>12]
#endm

// ---------------------------------------------------------------------------
// _mpls_set_header_type(hdr_type)
//
//
//    Description:
//
// 			Set packet header type: IPv4, IPv6, MPLS, etc.
//
//	  Inputs:
//			hdr_type: encoding header type values.
//
// ---------------------------------------------------------------------------
#macro _mpls_set_header_type(l3prot)

    dl_meta_set_header_type(MPLS_IPV4_HEADER_TYPE)
    br=byte[l3prot, 0, MPLS_L3_IP_V4, header_set#]

    dl_meta_set_header_type(MPLS_IPV6_HEADER_TYPE)
    br=byte[l3prot, 0, MPLS_L3_IP_V6, header_set#]


    dl_meta_set_header_type(MPLS_UNKNOWN_HEADER_TYPE)

header_set#:

#endm

#endif /* MPLS_UTIL_UC */
