/*
 * ============================================================================
 * COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2003 Intel Corporation. All rights reserved.
 *
 * PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *                                                                          */
/** \file      ix_cc_mpls_internal.h
 *
 *  \brief     This file defines MPLS all internal data types and 
 *             function prototypes
 *
 */
#if !defined(IX_CC_MPLS_INTERNAL_H)
#define      IX_CC_MPLS_INTERNAL_H



#if defined(__cplusplus)
extern "C"
{
#endif 






/* Static Configuration Data */
#if !defined (IX_INCLUDE_REGISTRY)

/* MEv2 mask for ME1, ME2, ME5, ME6
 * Note: This is a bitmask used to go in a loop across all possible
 * uEs with left shift. */
#ifndef MPLS_MEV2_NUM_MASK
#define MPLS_MEV2_NUM_MASK                            0x66
#endif

/* The number of all ports in the system.*/
#define MPLS_MAX_GLOBAL_PORTS                         256
#define MPLS_ICMP_QUEUE_MAXSIZE                       8
#define MPLS_ICMP_QUEUE_SLEEPTIME                     100

#else

/* Define strings for repository access */
/* these properties are placed in main MPLS context */
#define IX_MPLS_PROP_ROOT                         "MPLS"
#define IX_MPLS_PROP_MEV2_NUM_MASK                "MEV2_NUM_MASK"
#define IX_MPLS_PROP_MAX_LOCAL_PORTS              "MAX_LOCAL_PORTS"
#define IX_MPLS_PROP_MAX_GLOBAL_PORTS             "MAX_GLOBAL_PORTS"
#define IX_MPLS_PROP_ICMPQ_MAXSIZE                "ICMPQ_MAXSIZE"
#define IX_MPLS_PROP_ICMP_DEFAULT_SLEEP_TIME      "ICMP_DEFAULT_SLEEP_TIME"
#define IX_MPLS_PROP_LABEL_SPACE_CONFIG           "LABEL_SPACE_CONFIG"

/* MPLS_MAX_LOCAL_PORTS * 2 entries*/
static const char mplsPropLabelRange[][16*2]
= {"MPLS/MIN_LABEL_PORT_0",  "MPLS/MAX_LABEL_PORT_0",  "MPLS/MIN_LABEL_PORT_1",  "MPLS/MAX_LABEL_PORT_1",
   "MPLS/MIN_LABEL_PORT_2",  "MPLS/MAX_LABEL_PORT_2",  "MPLS/MIN_LABEL_PORT_3",  "MPLS/MAX_LABEL_PORT_3",
   "MPLS/MIN_LABEL_PORT_4",  "MPLS/MAX_LABEL_PORT_4",  "MPLS/MIN_LABEL_PORT_5",  "MPLS/MAX_LABEL_PORT_5",
   "MPLS/MIN_LABEL_PORT_6",  "MPLS/MAX_LABEL_PORT_6",  "MPLS/MIN_LABEL_PORT_7",  "MPLS/MAX_LABEL_PORT_7",
   "MPLS/MIN_LABEL_PORT_8",  "MPLS/MAX_LABEL_PORT_8",  "MPLS/MIN_LABEL_PORT_9",  "MPLS/MAX_LABEL_PORT_9",
   "MPLS/MIN_LABEL_PORT_10", "MPLS/MAX_LABEL_PORT_10", "MPLS/MIN_LABEL_PORT_11", "MPLS/MAX_LABEL_PORT_11",
   "MPLS/MIN_LABEL_PORT_12", "MPLS/MAX_LABEL_PORT_12", "MPLS/MIN_LABEL_PORT_13", "MPLS/MAX_LABEL_PORT_13",
   "MPLS/MIN_LABEL_PORT_14", "MPLS/MAX_LABEL_PORT_14", "MPLS/MIN_LABEL_PORT_15", "MPLS/MAX_LABEL_PORT_15"};

#endif


#define MPLS_FLAG_VALID_BIT                   0x80
#define MPLS_FLAG_SET_BIT                     0x40




/*
 * TYPENAME:    ix_cc_mpls_context       */
/**
 * ABSTRACT: MPLS CC context data structure
 */
typedef struct ix_s_cc_mpls_context
{
    /* Handle of MPLS forwarder core component */
    ix_cc_handle                                   hMplsHandle;

#if IX_EXCLUDE_CCI
    /* ICMP thread id - this is for level0 support */
    ix_ossl_thread_t                               icmpThreadId;
#else
    /* Event handler handle  */
    ix_event_handle                                hEventHandle;
#endif
    /* Local blade ID.       */
    ix_uint32                                      localBladeId;
    /* Mutex ids used by the component in critical sections. */
    ix_ossl_mutex_t                                mutexId;
    /* MEv2 number mask for MPLS */
    ix_uint32                                      mev2NumMask;
    /* The maximum number of ports per blade. */
    ix_uint32                                      maxLocalPorts;
    /* The number of all ports in the system.*/
    ix_uint32                                      maxGlobalPorts;

    ix_uint32                                      labelSpaceMode;
    /*================= NHLFE context data =======================*/    

    /* Mutex id used in ICMP critical sections. */
    ix_ossl_mutex_t                                nhlfeMutexId;
    /*! Max number of NHLFE table entries.*/
    ix_uint32                                      maxNhlfeEntries;
    /*! head of list containing free CC NHLFE table entries */
    ix_cc_mpls_cc_nhlfe_free_slot                 *pFreeNhlfeListHead;
    /*! tail of list containing free CC NHLFE table entries */
    ix_cc_mpls_cc_nhlfe_free_slot                 *pFreeNhlfeListTail;
    /*! Base address of the CC NHLFE Table */
    ix_cc_mpls_cc_nhlfe                           *pCcNhlfeBaseAddr;
    /*! Base address of the NHLFE Table */
    ix_cc_mpls_nhlfe_internal                     *pNhlfeBaseAddr;
    /*! Type of memory where NHLFE table is held */
    ix_memory_type                                 nhlfeTableMemoryType;
    /*! Memory channel where NHLFE table is held */
    ix_uint32                                      nhlfeTableMemoryChannel;

#if defined (MPLS_COUNTERS_FEATURE)
    /*! Memory channel where NHLFE counters are held
      - they have to be in SRAM due to atomic operations */
    ix_uint32                                      nhlfeCountersMemoryChannel;
    /*! 32-bit FTN counters table */
    ix_cc_mpls_nhlfe_counters_internal            *pFtnCounters;
    /*! 64-bit FTN counters handles table */
    ix_counter_64bit_handle                       *pFtn64BitCountersHdls;
#endif /* MPLS_COUNTERS_FEATURE */


    /*================= LSP context data =======================*/

    /* Mutex id used in ICMP critical sections. */
    ix_ossl_mutex_t                                lspMutexId;
    /* RTM handler */
    ix_cc_rtmv4                                    hRtm;
    /*! Max number of LSP table entries.*/
    ix_uint32                                      maxLspEntries;
    /*! Max number of ILM table entries.*/
    ix_uint32                                      maxIlmEntries;
    /*! Max number of ILM SET entries.*/
    ix_uint32                                      maxIlmSetEntries;
    /*! table contains definitions of allowed Label range per port */
    ix_cc_mpls_label_space                        *pLabelSpaceTable;
    /*! head of stack containing free CC LSP table entries */
    ix_cc_mpls_cc_lsp_free_slot                   *pFreeLspListHead;
    /*! Base address of the CC LSP Table */
    ix_cc_mpls_cc_lsp                             *pCcLspBaseAddr;
    /*! Base address of the ILM Table */
    ix_cc_mpls_ilm_internal                       *pIlmBaseAddr;
    /*! head of stack containing free ILM SET table entries */
    ix_cc_mpls_ilm_set_free_slot                  *pFreeIlmSetListHead;
    /*! Base address of the ILM SET Table */
    ix_cc_mpls_ilm_internal                       *pIlmSetBaseAddr;
    /*! Memory channel where Label space table is held */
    ix_uint32                                      LabelSpaceTableMemoryChannel;
    /*! Type of memory where ILM table is held */
    ix_memory_type                                 IlmTableMemoryType;
    /*! Memory channel where ILM table is held */
    ix_uint32                                      IlmTableMemoryChannel;

#if defined (MPLS_COUNTERS_FEATURE)
    /*! Memory channel where ILM counters are held 
     *  - they have to be in SRAM due to atomic operations */
    ix_uint32                                      IlmCountersMemoryChannel;
    /*! 32-bit ILM counters table */
    ix_cc_mpls_ilm_counters_internal              *pIlmCounters;
    /*! 64-bit ILM counters handles table */
    ix_counter_64bit_handle                       *pIlm64BitCountersHdls;
    /*! 32-bit ILM NHLFE set counters table */
    ix_cc_mpls_ilm_counters_internal              *pIlmNhlfeSetCounters;
    /*! 64-bit ILM NHLFE set counters handles table */
    ix_counter_64bit_handle                       *pIlmNhlfeSet64BitCountersHdls;
    /*! 32-bit input port counters table */
    ix_cc_mpls_input_port_stats                   *pInputPortCounters;
    /*! 64-bit input port counters handles table */
    ix_counter_64bit_handle                       *pInputPort64BitCountersHdls;

#endif /* MPLS_COUNTERS_FEATURE */


    /*================= ICMP context data =======================*/

    /* Mutex id used in ICMP critical sections. */
    ix_ossl_mutex_t                                ICMPmutexId;
    /* Icmp message queue                      */
    ix_buffer_handle*                              pIcmpQueue;
    /*! Icmp message queue head                 */
    ix_uint32                                      icmpQheadIdx;
    /* Icmp message queue tail                 */
    ix_uint32                                      icmpQtailIdx;
    
    /*! icmpQueueMaxSize represents the depth of the ICMP error message queue.
     *  That is, the maximum number of ICMP messages that can fit in the queue,
     *  not the number of messages currently in the queue  */
    ix_uint16                                      icmpQueueMaxSize;

    ix_uint16                                      icmp_ipID;

    /**
    * icmpQueueDrain represents the maximum number of ICMP error messages
    * to send (from the ICMP message queue) each time the queue processing
    * event fires
    */
    ix_uint16                                      icmpQueueDrain;

   /**
    *  icmpSleepTime represents the number of seconds between when the event
    *  will fire causing the ICMP messages to be dequeued and sent. The lower
    *  the value of icmpSleepTime, the more frequent the ICMP messages will
    *  be dequeued and sent.
    */
    ix_uint16                                      icmpSleepTime;

} ix_cc_mpls_context;
 


/*
 * MACRO NAME: MPLS_SET_UBLOCK_SYMBOLS               */
/**
 * DESCRIPTION: This macro sets microblock symbols into the
 *              ix_imported_symbol data structure
 *  
 * \param  argSymbolStruct -  imported symbol structure .
 * \param  argValue        - value of the imported variable.
 * \param  argName         - name of the imported variable
 *
 * \return Fills data structure with the name and value
 */
#define MPLS_SET_UBLOCK_SYMBOLS(argSymbolStruct,argValue,argName) \
{                                                                 \
    argSymbolStruct.m_Value = argValue;                           \
    argSymbolStruct.m_Name = argName;                             \
}




 
/*
 * FUNCTION: _ix_cc_mpls_prop_get_uint_value   */
/**
 * ABSTRACT: Get the value of a specific subproperty
 *
 * DESCRIPTION: Get's the value of a specific subproperty.
 *              The type is validated and an error is passed if it is not
 *              uint32.
 *
 * \param  arg_parent Handle to the parent property.
 * \param  arg_name   Name of the sub-property
 * \param  arg_value  Output parameter for the value.
 *
 * \return IX_SUCCESS upon success or an ix_error with one of the following values.
 * \retval IX_CC_ERROR_ENTRY_NOT_FOUND The property is not the correct type (uint32)
 */
ix_error
_ix_cc_mpls_prop_get_uint_value(ix_configuration_property_handle  arg_parent,
                                const char                       *arg_name,
                                ix_uint32                        *arg_value);




/*
 * FUNCTION: _ix_cc_mpls_get_static_data ()  */
/**
 * ABSTRACT: This function retrieves all MPLS static data from system registry
 *
 * DESCRIPTION: This function retrieves all MPLS static data from 
 *              system registry (uses defaults if registry is absent) 
 *              and store them into the corresponding field in the context 
 *              structure passed in.
 *
 * \param  arg_pContext  - MPLS CC context
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 */
ix_error
_ix_cc_mpls_get_static_data(void *arg_pContext);


/*
 * FUNCTION: _ix_cc_mpls_free_resources ()  */
/**
 * ABSTRACT: This function frees all memory and resources taken by MPLS CC
 *
 * DESCRIPTION: This function frees all memory and resources taken by MPLS CC
 *              Finally, it frees the CC context structure.
 *
 * \param  arg_pContext  - MPLS CC context
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 */
void
_ix_cc_mpls_free_resources(void *arg_pContext);




/*
 * FUNCTION: _ix_cc_mpls_patch_symbols()  */
/**
 * ABSTRACT: Patching of all MPLS symbols in MPLS microblock
 *
 * \param  arg_pContext   - pointer to the MPLS CC context
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 * \n
 * \n Error codes:
 * \n      IX_CC_MPLS_ERROR_RM_GET_PHYS_OFFSET - error by physical offset
 *                                               reading
 * \n      IX_CC_MPLS_ERROR_RM_PATCH_SYMBOL    - error by patching imported
 *                                               symbols into the microcode
 */
ix_error
_ix_cc_mpls_patch_symbols(void *arg_pContext);





#if defined (IX_MPLS_DEBUG)

IX_EXPORT_FUNCTION
void show_mem(void* arg_address, int byteNum);

IX_EXPORT_FUNCTION
void show_metadata (ix_hw_buffer_meta *pHwMeta);

#endif



#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */



#endif /* end !defined(IX_CC_MPLS_INTERNAL_H) */








