/*
 * ============================================================================
 * COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2003 Intel Corporation. All rights reserved.
 *
 * PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *                                                                          */
/** \file      ix_cc_mpls_lsp.h
 *
 *  \brief     The file contains CCI API functions exposed by
 *             MPLS Core Component used to manage LSP table
 *
 */
#if !defined(IX_CC_MPLS_LSP_H)
#define      IX_CC_MPLS_LSP_H



#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */



/* Static Configuration Data */
#if !defined (IX_INCLUDE_REGISTRY)

    /*! Type of memory where ILM table is stored. */
    #if !defined(MPLS_ILM_TABLE_MEM_TYPE)
    #define MPLS_ILM_TABLE_MEM_TYPE                 IX_MEMORY_TYPE_SRAM
    #endif

    /*! Channel of memory where ILM table is stored. */
    #if !defined(MPLS_ILM_TABLE_MEM_CHANNEL)
    #define MPLS_ILM_TABLE_MEM_CHANNEL              MPLS_DEFAULT_SRAM_CH_NUM
    #endif

    /*! Channel of memory where ILM statistics table is stored. */
    #if !defined(MPLS_ILM_COUNTERS_TABLE_MEM_CHANNEL)
    #define MPLS_ILM_COUNTERS_TABLE_MEM_CHANNEL     MPLS_DEFAULT_SRAM_CH_NUM
    #endif

    /*! Channel of memory where Label Space table is stored. */
    #if !defined(MPLS_ILM_LABEL_SPACE_TABLE_MEM_TYPE)
    #define MPLS_ILM_LABEL_SPACE_TABLE_MEM_TYPE     MPLS_DEFAULT_SRAM_CH_NUM
    #endif

    /*! Maximum number of ILM entries. */
    #if !defined(MPLS_ILM_MAX_ENTRIES)
    #define MPLS_ILM_MAX_ENTRIES                    100
    #endif

    /*! Maximum number of ILM SET entries. */
    #if !defined(MPLS_ILM_MAX_SET_ENTRIES)
    #define MPLS_ILM_MAX_SET_ENTRIES                20
    #endif

#else

    /*! Path in registry to MPLS data. */
    #define IX_MPLS_PROP_ILM_ROOT                          IX_MPLS_PROP_ROOT
    /*! Type of memory where table with label space config is kept. */
    #define IX_MPLS_PROP_ILM_LABEL_SPACE_INFO              IX_MPLS_PROP_ROOT
    /*! the size of CC LSP table */
    #define IX_MPLS_PROP_MAX_LSP_ENTRIES                   "MAX_LSP_ENTRIES"
    /*! maximum number of ilm sets */    
    #define IX_MPLS_PROP_MAX_ILM_SET_ENTRIES               "MAX_ILM_SET_ENTRIES"
    /*! Type of memory where ILM table is stored. */
    #define IX_MPLS_PROP_ILM_TABLE_MEM_TYPE                "ILM_TABLE_MEM_TYPE"
    /*! Channel of memory where ILM table is stored. */
    #define IX_MPLS_PROP_ILM_TABLE_MEM_CHANNEL             "ILM_TABLE_MEM_CHANNEL"
    /*! Channel of memory where ILM statistics table is stored. */
    #define IX_MPLS_PROP_ILM_COUNTERS_TABLE_MEM_CHANNEL    "ILM_COUNTERS_MEM_CHANNEL"
    /*! Type of memory where table with label space config is kept. */
    #define IX_MPLS_PROP_ILM_LABEL_RANGE_TABLE_MEM_CHANNEL "ILM_LABEL_RANGE_MEM_CHANNEL"

#endif /* (IX_INCLUDE_REGISTRY) */

/* The base address of the NHLFE table. */
#define IX_MPLS_SYMBOL_ILM_BASE       "MPLS_ILM_TABLE_BASE"
#define IX_MPLS_SYMBOL_ILM_SET_BASE   "MPLS_ILM_NHLFE_SET_TABLE_BASE"
#define IX_MPLS_SYMBOL_ILM_PARAM_BASE "MPLS_ILM_PARAM_TABLE_BASE"

#ifdef MPLS_COUNTERS_FEATURE
    #define IX_MPLS_SYMBOL_ILM_STAT_BASE            "MPLS_ILM_STAT_BASE"
    #define IX_MPLS_SYMBOL_ILM_NHLFE_SET_STAT_BASE  "MPLS_ILM_NHLFE_SET_STAT_BASE"
    #define IX_MPLS_INPUT_PORT_STAT_BASE            "MPLS_INPUT_PORT_STAT_BASE"
    #define IX_MPLS_LSP_SYMBOLS_PATCHED_NUM   6
#else 
    #define IX_MPLS_LSP_SYMBOLS_PATCHED_NUM   3
#endif


/*
 * TYPENAME:    ix_cc_mpls_ilm_internal                    */
/**
 * DESCRIPTION: ILM NHLFE data entry type (intrnal viewpoint)
 *
 */
typedef struct ix_s_cc_mpls_ilm_internal
{
    ix_uint32  flags         :8,
               operation     :2,
               pushCount     :3,
               l3Protocol    :3,
               nextHopId     :16;
    ix_uint32  maxLabPktSize :16,
               outPortId     :16;
    ix_uint32  topLabel      :20,
               reserved_1    :4,
               outPortType   :4,
               ttl           :4;
    ix_uint32  label1        :20,
               reserved_2    :4,
               bladeId       :8;
    ix_uint32  label2        :20,
               reserved_3    :12;
    ix_uint32  label3        :20,
               reserved_4    :12;
    ix_uint32  phb2exp       :16,
               exp2phb       :16;
    ix_uint32  reserved_5;
} ix_cc_mpls_ilm_internal;




/*
 * TYPENAME:    ix_cc_mpls_ilm_set_internal                    */
/**
 * DESCRIPTION: ILM SET data entry type (intrnal viewpoint)
 *
 */
typedef struct ix_s_cc_mpls_ilm_set_internal
{
    ix_uint32   flags       :8,
                exp2phb     :16,
                SetSize     :8;
    ix_uint32   phbMask1;
    ix_uint32   phbMask2;
    ix_uint32   phbMask3;
    ix_uint32   phbMask4;
    ix_uint32   OutSegId1   :20,
                OutSegId2Hi :12;
    ix_uint32   OutSegId2Lo :8,
                OutSegId3   :20,
                OutSegId4Hi :4;
    ix_uint32   OutSegId4Lo :16,
	        reserved    :16;
} ix_cc_mpls_ilm_set_internal;



#define MPLS_ILM_64BIT_COUNTERS_NUM           6
#define MPLS_ILM_32BIT_COUNTER_OVERFLOW_TIME  10

/*
 * TYPENAME:    ix_cc_mpls_ilm_counters_internal                    */
/**
 * DESCRIPTION: ILM statistics data entry type (intrnal viewpoint)
 *
 */
typedef struct ix_s_cc_mpls_ilm_counters_internal
{
   /*! 32-bits counter accumulating the number of octets received by this segment. */
   ix_uint32 mplsInSegmentOctets;
   /*! 32-bits counter accumulating the number of packets received by this segment */
   ix_uint32 mplsInSegmentPkts;
   /*! 32-bits counter accumulating the number of erroneous octets received by 
    *  this segment (e.g. with TTL expired, missed lookups, fragmentation errors)  */
   ix_uint32 mplsInSegErrorOctets;
   /*! 32-bits counter accumulating the number of erroneous packets received by 
    *  this segment (e.g. with TTL expired, missed lookups, fragmentation errors)  */
   ix_uint32 mplsInSegErrorPkts;
   /*! 32-bits counter accumulating the number of octets being the difference 
    *  between the sum of fragments lengths and the fragmented packet's length     */
   ix_uint32 mplsInSegFragmOctDelta;
   /*! 32-bits counter accumulating the number of fragments sent on this segment, 
    *  excluding the first fragment                                                */
   ix_uint32 mplsInSegFragmPktDelta;
} ix_cc_mpls_ilm_counters_internal;
/*
 * TYPENAME:    ix_cc_mpls_label_space               */
/**
 * DESCRIPTION: type allows to define suppoerted label range 
 */
typedef struct ix_s_cc_mpls_label_space
{
   ix_uint32    min_label;
   ix_uint32    max_label;
   ix_uint32    table_offset;
   ix_uint32    stats_offset;
} ix_cc_mpls_label_space;



/*
 * TYPENAME: ix_cc_mpls_cc_lsp Data Type           */
/**
 * ABSTRACT: CC_LSP table entry
 */
typedef struct ix_s_cc_mpls_cc_lsp
{
    #define IX_CC_MPLS_LSP_FREE     0
    #define IX_CC_MPLS_LSP_VALID    1

    ix_uint16               flags;       /* LSP_FREE(0), LSP_VALID(10)  */
    ix_uint8                type;        /* LSP_FEC(1), LSP_ILM(2)      */

    union {

        struct {
            ix_cc_mpls_label   label;     /* Index to ilm table.         */
            ix_uint16          in_port;   /* used in per-interface mode. */
        } ilmData;

        struct {
            ix_uint32          NetAddr;   /* The network address identifying the route.  */
            ix_uint32          NetMask;   /* The subnet mask for the route.              */
            ix_cc_rtmv4_nhid   NextHopId; /* Index to next hop table                     */
        } fecData;

    } inHandle;

    ix_cc_mpls_nhlfe_handle NhlfeHandle; /* NHLFE handle           */
} ix_cc_mpls_cc_lsp;





/*
 * TYPENAME: ix_cc_mpls_cc_lsp_free_slot           */
/**
 * ABSTRACT: Structure used by the list of free LSP entries
 */
typedef struct ix_s_cc_mpls_cc_lsp_free_slot* ix_cc_mpls_cc_lsp_free_slot_ptr;
typedef struct ix_s_cc_mpls_cc_lsp_free_slot
{
    ix_uint32                         LspIndex;
    ix_cc_mpls_cc_lsp_free_slot_ptr   pNextFreeSlot;
} ix_cc_mpls_cc_lsp_free_slot;




/*
 * TYPENAME: ix_cc_mpls_ilm_set_free_slot       */
/**
 * ABSTRACT: Structure used by the list of free ILM SET entries
 */
typedef struct ix_s_cc_mpls_ilm_set_free_slot* ix_cc_mpls_ilm_set_free_slot_ptr;
typedef struct ix_s_cc_mpls_ilm_set_free_slot
{
    ix_uint32                          IlmSetIndex;
    ix_cc_mpls_ilm_set_free_slot_ptr   pNextFreeSlot;
} ix_cc_mpls_ilm_set_free_slot;






/**
 * Internal API
 */

/*
 * FUNCTION: ix_cc_mpls_lsp_init()  */
/**
 * ABSTRACT: function initializes the LSP part of MPLS CC
 *
 * DESCRIPTION: This function creates and initializes the LSP table. It should
 *              be called and returned successfully before any other function in
 *              this library can be called. It returns the base address of the
 *              LSP table.
 *
 * \param  arg_pContext   - pointer to the MPLS CC context
 *                          that will be passed to the core component API function
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 * \n
 * \n Error codes:
 * \n           IX_CC_ERROR_NULL            - invalid pointer
 * \n           IX_CC_ERROR_DUPLICATE_ENTRY - LSP table already exist
 * \n           IX_CC_ERROR_OOM_SYSTEM      - memory allocation failure
 * \n           IX_CC_ERROR_OOM_SRAM        - memory allocation failure
 */
ix_error
ix_cc_mpls_lsp_init(void *arg_pContext);


/*
 * FUNCTION: _ix_cc_mpls_lsp_free_resources()  */
/**
 * ABSTRACT: These function releases all resources taken by LSP library
 *
 * DESCRIPTION: This function frees all memory and resources taken by
 *              LSP library
 *
 * \param  arg_pContext - a pointer to the MPLS LSP context
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 */
void
_ix_cc_mpls_lsp_free_resources(void *arg_pContext);




/*
 * FUNCTION: _ix_cc_mpls_lsp_get_static_data()  */
/**
 * ABSTRACT: reads LSP-relevant data from MPLS configuration data external resource
 *
 * DESCRIPTION: This function retrieves all LSP-relevant static data from
 *              either system repository or ix_cc_mpls.h and store them
 *              into the corresponding field in the control block
 *              structure passed in.
 *
 * \param  arg_pContext  - pointer to component context memory
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 */
ix_error
_ix_cc_mpls_lsp_get_static_data(void *arg_pContext);




/*
 * FUNCTION: _ix_cc_mpls_patch_lsp_symbols()  */
/**
 * ABSTRACT: Patching of all LSP symbols in a microcode in MPLS microblock
 *
 * \param  arg_pContext   - pointer to the MPLS CC context
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 * \n
 * \n Error codes:
 * \n      IX_CC_MPLS_ERROR_RM_GET_PHYS_OFFSET - error by physical offset
 *                                               reading
 * \n      IX_CC_MPLS_ERROR_RM_PATCH_SYMBOL    - error by patching imported
 *                                               symbols into the microcode
 */
ix_error
_ix_cc_mpls_lsp_patch_symbols(void *arg_pContext);



/*
 * FUNCTION: _ix_cc_mpls_lsp_fini()  */
/**
 * ABSTRACT: removes resources taken to generate LSP
 *
 * DESCRIPTION: releases the LSP table.
 *
 * \param  arg_pContext - a pointer to the MPLS context
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 */
ix_error
_ix_cc_mpls_lsp_fini(void *arg_pContext);

    

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(IX_CC_MPLS_LSP_H) */


