/*
 * ============================================================================
 * COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2003 Intel Corporation. All rights reserved.
 *
 * PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *                                                                          */
/** \file      ix_cc_mpls_msg.h
 *
 *  \brief     This file defines MPLS message handler exported data types
 *
 */

#if !defined(IX_CC_MPLS_MSG_H)
#define      IX_CC_MPLS_MSG_H


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */


/*
 * TYPENAME: ix_cc_mpls_message_ctx    */
/** 
 * ABSTRACT: internal context of MPLS message helpers
 *
 * DESCRIPTION: Context data structure for storing info
 *              in async calls to MPLS CC API
 *
 */
typedef struct ix_s_cc_mpls_message_ctx
{
    /* pointer to the user callback function */
    void *pCallBack;
    /* pointer to the user provided context  */
    void *pUserContext;
} ix_cc_mpls_message_ctx;


/* TYPENAME:  ix_cc_mpls_message_lsp_create */
/**
 * ABSTRACT:  Data structure of IX_CC_MPLS_MSG_LSP_CREATE message
 *
 * DESCRIPTION: This structure defines format of the
 *              IX_CC_MPLS_MSG_LSP_CREATE handled by MPLS Core Componenet
 */
typedef struct ix_s_cc_mpls_message_lsp_create
{
  ix_cc_mpls_lsp   Lsp;
  ix_cc_mpls_nhlfe NhlfeSet[MPLS_MAX_NHLFE_IN_SET];
} ix_cc_mpls_message_lsp_create;


/* TYPENAME:  ix_cc_mpls_message_lsp_modify */
/**
 * ABSTRACT:  Data structure of IX_CC_MPLS_MSG_LSP_MODIFY message
 *
 * DESCRIPTION: This structure defines format of the
 *              IX_CC_MPLS_MSG_LSP_MODIFY handled by MPLS Core Componenet
 */
typedef struct ix_s_cc_mpls_message_lsp_modify
{
  ix_cc_mpls_lsp_handle    LspHandle;
  ix_uint8                 nhlfeInfoType;
  ix_cc_mpls_nhlfe_handle  NhlfeSetHandle;
  ix_uint16                nhlfeHandles_num;
  ix_cc_mpls_nhlfe_handle  nhlfeHandles[MPLS_MAX_NHLFE_IN_SET];
  ix_uint16                lspParamsSel[MPLS_MAX_NHLFE_IN_SET];
  ix_cc_mpls_lsp_param     lspParams[MPLS_MAX_NHLFE_IN_SET];
} ix_cc_mpls_message_lsp_modify;
  

/* TYPENAME:  ix_cc_mpls_message_nhlfe_set_create */
/**
 * ABSTRACT:  Data structure of IX_CC_MPLS_MSG_NHLFE_SET_CREATE message
 *
 * DESCRIPTION: This structure defines format of the
 *              IX_CC_MPLS_MSG_NHLFE_SET_CREATE handled by MPLS Core Componenet
 */
typedef struct ix_s_cc_mpls_message_nhlfe_set_create
{
  ix_uint16               NhlfeSetSize;
  ix_cc_mpls_nhlfe_handle NhlfeSetHandleTable[MPLS_MAX_NHLFE_IN_SET];
} ix_cc_mpls_message_nhlfe_set_create;
  
  
/* TYPENAME:  ix_cc_mpls_message_nhlfe_set_modify  */
/**
 * ABSTRACT:  Data structure of IX_CC_MPLS_MSG_NHLFE_SET_MODIFY message
 *
 * DESCRIPTION: This structure defines format of the
 *              IX_CC_MPLS_MSG_NHLFE_SET_MODIFY handled by MPLS Core Componenet
 */
typedef struct ix_s_cc_mpls_message_nhlfe_set_modify
{
  ix_cc_mpls_nhlfe_handle NhlfeHandle;
  ix_uint16               NhlfeSetSize;
  ix_cc_mpls_nhlfe_handle NhlfeSetHandleTable[MPLS_MAX_NHLFE_IN_SET];
} ix_cc_mpls_message_nhlfe_set_modify;


/* TYPENAME:  ix_cc_mpls_message_nhlfe_set_query_rep  */
/**
 * ABSTRACT:  Data structure of a reply to
 *            IX_CC_MPLS_MSG_NHLFE_SET_QUERY message
 *
 * DESCRIPTION: This structure defines format of the reply the MPLS core component
 *              message handler sends after NHLFE set query operation
 */
typedef struct ix_s_cc_mpls_message_nhlfe_set_query_rep {
  ix_uint16               NhlfeSetSize;
  ix_cc_mpls_nhlfe_handle NhlfeSetHandleTable[MPLS_MAX_NHLFE_IN_SET];
} ix_cc_mpls_message_nhlfe_set_query_rep;


/* TYPENAME:  ix_cc_mpls_message_exp2phb_modify */
/**
 * ABSTRACT:  Data structure of IX_CC_MPLS_MSG_EXP2PHB_MODIFY message
 *
 * DESCRIPTION: This structure defines format of the
 *              IX_CC_MPLS_MSG_EXP2PHB_MODIFY handled by MPLS Core Componenet
 */
typedef struct ix_s_cc_mpls_message_exp2phb_modify
{
  ix_uint16          Index;
  ix_cc_mpls_exp2phb Exp2PhbTable[MPLS_MAX_EXP_NUM];
} ix_cc_mpls_message_exp2phb_modify;



/* TYPENAME:  ix_cc_mpls_message_phb2exp_modify */
/**
 * ABSTRACT:  Data structure of IX_CC_MPLS_MSG_PHB2EXP_MODIFY message
 *
 * DESCRIPTION: This structure defines format of the
 *              IX_CC_MPLS_MSG_PHB2EXP_MODIFY  handled by MPLS Core Componenet
 */
typedef struct ix_s_cc_mpls_message_phb2exp_modify
{
  ix_uint16          Index;
  ix_cc_mpls_phb2exp Phb2ExpTable[MPLS_MAX_PHB_NUM];
} ix_cc_mpls_message_phb2exp_modify;
    


/* TYPENAME:  ix_cc_mpls_message_lsp_create_rep     */
/**
 * ABSTRACT:  Data structure of a reply message to
 *            IX_CC_MPLS_MSG_LSP_CREATE message
 *
 * DESCRIPTION: This structure defines format of the reply the MPLS core component
 *              message sent after LSP creation
 */
typedef struct ix_s_cc_mpls_message_lsp_create_rep {
    ix_cc_mpls_nhlfe_handle  nhlfeSetHandle;
    ix_uint16                nhlfe_num;
    ix_cc_mpls_nhlfe_handle  nhlfeHandle[MPLS_MAX_NHLFE_IN_SET];
    ix_cc_mpls_lsp_handle    LspHandle;
} ix_cc_mpls_message_lsp_create_rep;

    
/*
 * FUNCTION: ix_cc_mpls_msg_handler()             */
/**
 * ABSTRACT: the message handler of the MPLS core component.
 *
 * DESCRIPTION: This is the message handler function for the MPLS
 *              core component. The MPLS CC receives messages from
 *              other core component through this message handler function, and
 *              internally it calls appropriate library functions to process
 *              the message.
 * 
 * \param  arg_hDataToken - buffer handle embedding information the
 *                          message passed in arg_UserData.
 * \param  arg_UserData   - message type.
 * \param  arg_pContext   - pointer to the MPLS core component
 *                          context that will be passed to the core
 *                          component when a message arrives. This
 *                          context was defined by the core component
 *                          and passed to core components infrastructure
 *                          through the ix_cc_mpls_init function.
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 * \n
 * \n Error codes:
 *         IX_CC_ERROR_INVALID_MESSAGE - unsupported message
 *         IX_CC_ERROR_INVALID_BUFFER  - buffer handle is unusable
 *         IX_CC_ERROR_INVALID_CONTEXT - context is unusable
 *         IX_CC_ERROR_NULL            - invalid pointer
 *  
 */
ix_error
ix_cc_mpls_msg_handler(ix_buffer_handle arg_hDataToken, 
                       ix_uint32        arg_UserData,
                       void             *arg_pContext);


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(IX_CC_MPLS_MSG_H) */
