/*
 * ============================================================================
 * COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2003 Intel Corporation. All rights reserved.
 *
 * PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *                                                                          */
/** \file      ix_cc_mpls_nhlfe.h
 *
 *  \brief     This file defines NHLFE library exported data types and
 *             function prototypes
 *
 */
#if !defined(IX_CC_MPLS_NHLFE_H)
#define      IX_CC_MPLS_NHLFE_H


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */




/* Static Configuration Data */
#if !defined (IX_INCLUDE_REGISTRY)

#ifndef MPLS_NHLFE_MAX_NHLFE_ENTRIES
#error "MPLS_NHLFE_MAX_NHLFE_ENTRIES undefined!"
#endif

/*! Type of memory where NHLFE table is stored. */
#ifndef MPLS_NHLFE_TABLES_MEM_TYPE
#define MPLS_NHLFE_TABLES_MEM_TYPE                 IX_MEMORY_TYPE_SRAM
#endif

/*! Channel of memory where NHLFE table is stored. */
#ifndef MPLS_NHLFE_TABLES_MEM_CHANNEL
#define MPLS_NHLFE_TABLES_MEM_CHANNEL              MPLS_DEFAULT_SRAM_CH_NUM
#endif

/*! Channel of memory where NHLFE statistics table is stored. */
#ifndef MPLS_NHLFE_COUNTERS_MEM_CHANNEL
#define MPLS_NHLFE_COUNTERS_MEM_CHANNEL            MPLS_DEFAULT_SRAM_CH_NUM
#endif

#else /* (IX_INCLUDE_REGISTRY) */

#define IX_MPLS_PROP_NHLFE_ROOT                   IX_MPLS_PROP_ROOT
/* these properties are placed in NHLFE context */
#define IX_MPLS_PROP_MAX_NHLFE_ENTRIES            "MAX_NHLFE_ENTRIES"
/*! Type of memory where NHLFE table is stored. */
#define IX_MPLS_PROP_NHLFE_TABLE_MEM_TYPE         "NHLFE_TABLE_MEM_TYPE"
/*! Channel of memory where NHLFE table is stored. */
#define IX_MPLS_PROP_NHLFE_TABLE_MEM_CHANNEL      "NHLFE_TABLE_MEM_CHANNEL"
/*! Channel of memory where NHLFE statistics table is stored. */
#define IX_MPLS_PROP_NHLFE_COUNTERS_MEM_CHANNEL   "NHLFE_COUNTERS_MEM_CHANNEL"

#endif /* (IX_INCLUDE_REGISTRY) */


/**
 * Preprocessor symbols and macro definitions
 */

/* The base address of the NHLFE table. */
#define IX_MPLS_SYMBOL_FTN_NHLFE_BASE      "MPLS_FTN_TABLE_BASE"

#ifdef MPLS_COUNTERS_FEATURE
#define IX_MPLS_SYMBOL_FTN_STAT_BASE       "MPLS_FTN_STAT_BASE"
#define MPLS_NHLFE_SYMBOLS_PATCHED_NUM     2
#else /* MPLS_COUNTERS_FEATURE */
#define MPLS_NHLFE_SYMBOLS_PATCHED_NUM     1
#endif /* MPLS_COUNTERS_FEATURE */


/* Allowed range of NHLFE set */
#define MPLS_NHLFE_MIN_SET_SIZE 1
#define MPLS_NHLFE_MAX_SET_SIZE MPLS_MAX_NHLFE_IN_SET

/* Maximum MTU size */
#define MPLS_PORT_MAX_MTU         32767

/* CC NHLFE flags field definitions */
#define MPLS_CC_NHLFE_FLAG_FREE               0x00
#define MPLS_CC_NHLFE_FLAG_NHLFE              0x01
#define MPLS_CC_NHLFE_FLAG_NHLFE_SET          0x02
#define MPLS_CC_NHLFE_FLAG_NHLFE_USED_IN_SET  0x03


/* action field definitions */
#define MPLS_SET_FLAGS		1
#define MPLS_SET_MTU		2
#define MPLS_SET_TTL		4
#define MPLS_SET_FLOWID		8
#define MPLS_SET_EXP		16
#define MPLS_SET_PHB		32
#define MPLS_SET_POLICY		64

/**
 * Types definitions
 */

/**
 * TYPENAME:    ix_cc_mpls_phb2exp_internal
 *
 * DESCRIPTION: PHB to EXP map table entry type definition
 *
 */
typedef struct ix_s_cc_mpls_phb2exp_internal
{
#define MPLS_NHLFE_PHB2EXP_VALID    1

    ix_uint32 valid:1,
              exp0:3,
              reserved0:1,
              exp1:3,
              reserved1:1,
              exp2:3,
              reserved2:1,
              exp3:3,
              reserved3:16;

} ix_cc_mpls_phb2exp_internal;



/**
 * TYPENAME:    ix_cc_mpls_nhlfe_internal
 *
 * DESCRIPTION: NHLFE data entry type (intrnal viewpoint)
 *
 */
typedef struct ix_s_cc_mpls_nhlfe_internal
{
    ix_uint32                                flags:8,
                                             operation:2,
                                             pushCount:3,
                                             l3Protocol:3,
                                             nextHopId:16;
    ix_uint32                                maxLabPktSize:16,
                                             outPortId:16;
    ix_uint32                                topLabel:20,
                                             reserved_1:4,
                                             outPortType:4,
                                             ttl:4;
    ix_uint32                                label1:20,
                                             reserved_2:4,
                                             bladeId:8;
    ix_uint32                                label2:20,
                                             reserved_3:4,
                                             flowIdHigh:8;
    ix_uint32                                label3:20,
                                             flowIdLow:12;
    ix_uint32                                exp2phb:16,
                                             phb2exp:16;
    ix_uint32                                reserved_4;

} ix_cc_mpls_nhlfe_internal;


/**
 * TYPENAME:    ix_cc_mpls_nhlfe_set_internal
 *
 * DESCRIPTION: NHLFE data entry type (internal viewpoint)
 *
 */
typedef struct ix_s_cc_mpls_nhlfe_set_internal
{
    ix_uint32		flags       :  8,
                    exp2phb     : 16,
                    setSize     :  8;
    ix_uint32       phbMask1;
    ix_uint32       phbMask2;
    ix_uint32       phbMask3;
    ix_uint32       phbMask4;
    ix_uint32       outSegId1   : 20,
                    outSegId2Hi : 12;
    ix_uint32       outSegId2Lo :  8,
                    outSegId3   : 20,
                    outSegId4Hi :  4;
    ix_uint32       outSegId4Lo : 16,
                    reserved2   : 16;
}ix_cc_mpls_nhlfe_set_internal;





/*
 * TYPENAME: ix_cc_mpls_cc_nhlfe_free_slot
/**
 * ABSTARCT: Structure used by free NHLFE entries list
 */
typedef struct ix_s_cc_mpls_lsp_list *ix_cc_mpls_lsp_list_ptr;
typedef struct ix_s_cc_mpls_lsp_list
{
    ix_cc_mpls_cc_lsp       *pCcLsp;
    ix_cc_mpls_lsp_list_ptr  pNextElem;
} ix_cc_mpls_lsp_list;


    
/*
 * TYPENAME: ix_cc_mpls_cc_nhlfe     */
/**
 * ABSTRACT: CC NHLFE table entry
 *
 */
typedef struct ix_s_cc_mpls_cc_nhlfe {
	ix_uint16                flags;    /* bit 01 - 00 entry is free
                                        *          01 entry holds a stand-alone NHLFE
                                        *          10 entry points at NHLFE SET
                                        *          11 entry is used in NHLFE SET
                                        */
	ix_uint16                refCount; /* number of LSPs referencing this NHLFE */
	ix_cc_mpls_lsp_list_ptr  lspList;  /* list of referencing LSP entries */
} ix_cc_mpls_cc_nhlfe;




/*
 * TYPENAME: ix_cc_mpls_cc_nhlfe_free_slot
/**
 * ABSTARCT: Structure used by free NHLFE entries list
 */
typedef struct ix_s_cc_mpls_cc_nhlfe_free_slot* ix_cc_mpls_cc_nhlfe_free_slot_ptr;
typedef struct ix_s_cc_mpls_cc_nhlfe_free_slot
{
    ix_cc_mpls_nhlfe_handle           NhlfeHandle;
    ix_cc_mpls_cc_nhlfe_free_slot_ptr pNextFreeSlot;
} ix_cc_mpls_cc_nhlfe_free_slot;




#if defined (MPLS_COUNTERS_FEATURE)

/* 32-bit base of 64-bit counter overflow time */
#define MPLS_NHLFE_32BIT_COUNTER_OVERFLOW_TIME  10

#define MPLS_NHLFE_64BIT_COUNTERS_NUM 6



/* TYPENAME:  ix_s_cc_mpls_nhlfe_counters_internal */
/**
 * ABSTRACT:  Data structure of an NHLFE Counters table entry
 *
 * DESCRIPTION: statistics connected with packets forwarded according 
 *              to a given NHLFE entry. Its entries correspond 1:1
 *              to the NHLFE table entries
 */
typedef struct ix_s_cc_mpls_nhlfe_counters_internal
{
    /*! 32-bit counter accumulating the number of octets received 
     *  through this entry.
     */
    ix_uint32 mplsNhlfeOctets;

    /*
     *! 32-bits counter accumulating the number of packets 
     *  received through this entry
     */
    ix_uint32 mplsNhlfePkts;

    /*! 32-bits counter accumulating the number of octets that were not 
     *  sent through this entry because their TTL expired or they required 
     *  fragmentation but had the DF bit set in the IP header
     */
    ix_uint32 mplsNhlfeErrorOctets;
    
    /*! 32-bits counter accumulating the number of packets that were 
     *  not sent through this entry because their TTL expired or they 
     *  required fragmentation but had the DF bit set in the IP header
     */
    ix_uint32 mplsNhlfeErrorPkts;

    ix_uint32 mplsNhlfeFragmOctetsDelta;
    ix_uint32 mplsNhlfeFragmPktsDelta;
    
} ix_cc_mpls_nhlfe_counters_internal;


#endif


/**
 * Internal API
 */

/*
 * FUNCTION: ix_cc_mpls_nhlfe_init()  */
/**
 * ABSTRACT: function initializes the NHLFE part of MPLS CC
 *
 * DESCRIPTION: This function creates and initializes the NHLFE table. It should
 *              be called and returned successfully before any other function in
 *              this library can be called. It returns the base address of the
 *              NHLFE table.
 *
 * \param  arg_pContext   - pointer to the MPLS CC context
 *                          that will be passed to the core component API function
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 * \n
 * \n Error codes:
 * \n           IX_CC_ERROR_NULL            - invalid pointer
 * \n           IX_CC_ERROR_DUPLICATE_ENTRY - NHLFE table already exist
 * \n           IX_CC_ERROR_OOM_SYSTEM      - memory allocation failure
 * \n           IX_CC_ERROR_OOM_SRAM        - memory allocation failure
 */
ix_error
ix_cc_mpls_nhlfe_init(void *arg_pContext);



/*
 * FUNCTION: _ix_cc_mpls_nhlfe_free_resources()  */
/**
 * ABSTRACT: These function releases all resources taken by NHLFE library
 *
 * DESCRIPTION: This function frees all memories and resources taken by
 *              NHLFE library
 *
 * \param  arg_pContext - a pointer to the MPLS LSP context
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 */
void
_ix_cc_mpls_nhlfe_free_resources(void *arg_pContext);






/*
 * FUNCTION: _ix_cc_mpls_nhlfe_get_static_data()  */
/**
 * ABSTRACT: reads NHLFE-relevant data from MPLS configuration data external resource
 *
 * DESCRIPTION: This function retrieves all NHLFE-relevant static data from
 *              either system repository or ix_cc_mpls.h and store them
 *              into the corresponding field in the control block
 *              structure passed in.
 *
 * \param  arg_pContext  - pointer to component context memory
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 */
ix_error
_ix_cc_mpls_nhlfe_get_static_data(void *arg_pContext);




/*
 * FUNCTION: _ix_cc_mpls_patch_nhlfe_symbols()  */
/**
 * ABSTRACT: Patching of all NHLFE symbols in a microcode in MPLS
 *              microblock
 *
 * \param  arg_pContext   - pointer to the MPLS CC context
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 * \n
 * \n Error codes:
 * \n      IX_CC_MPLS_ERROR_RM_GET_PHYS_OFFSET - error by physical offset
 *                                               reading
 * \n      IX_CC_MPLS_ERROR_RM_PATCH_SYMBOL    - error by patching imported
 *                                               symbols into the microcode
 */
ix_error
_ix_cc_mpls_nhlfe_patch_symbols(void *arg_pContext);





/*
 * FUNCTION: _ix_cc_mpls_nhlfe_inc_ref_count()  */
/**
 * ABSTRACT: increases ref count of a given CC NHLFE entry
 *
 * \param  arg_NhlfeHandle - handle to NHLFE entry which ref count will be increased
 * \param  arg_pContext    - pointer to the MPLS CC context
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 */
ix_error
_ix_cc_mpls_nhlfe_inc_ref_count(ix_cc_mpls_nhlfe_handle  arg_NhlfeHandle,
                                void                    *arg_pContext);




/*
 * FUNCTION: _ix_cc_mpls_nhlfe_dec_ref_count()  */
/**
 * ABSTRACT: decreases ref count of a given CC NHLFE entry
 *
 * \param  arg_NhlfeHandle - handle to NHLFE entry which ref count will be decreased
 * \param  arg_pContext    - pointer to the MPLS CC context
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 */
ix_error
_ix_cc_mpls_nhlfe_dec_ref_count(ix_cc_mpls_nhlfe_handle  arg_NhlfeHandle,
                                void                    *arg_pContext);




/*
 * FUNCTION: _ix_cc_mpls_nhlfe_internal_modify()  */
/**
 * ABSTRACT: This function modifies the NHLFE parameters connected with LSP
 *
 * DESCRIPTION: This function modifies the NHLFE parameters that user passes
 *              as LSP information. This function is invoked internally
 *              by ix_cc_mpls_lsp_create / modify.
 *
 * \param  arg_NhlfeHandle        -	handle identifying an NHLFE entry 
 *                                  containing an NHLFE set
 * \param  arg_lspParamsSel       - flags denoting which fields of lsp has to be modified
 * \param  arg_lspParams          - LSP parameters that should be kept in NHLFE 
 *                                  structure
 * \param  arg_pContext           - pointer to the MPLS core component context 
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 * \n
 * \n Error codes:
 * \n         IX_CC_ERROR_NULL                 - invalid pointer
 * \n         IX_CC_ERROR_ENTRY_NOT_FOUND      - NHLFE does not exist
 * \n         IX_CC_MPLS_ERROR_ENTRY_NHLFE_SET - this is not an NHLFE set entry
 */
ix_error
_ix_cc_mpls_nhlfe_internal_modify(ix_cc_mpls_nhlfe_handle  arg_NhlfeHandle,
                                  ix_uint16                arg_lspParamsSel,
                                  ix_cc_mpls_lsp_param     arg_lspParams,
                                  void                    *arg_pContext);





/*
 * FUNCTION: _ix_cc_mpls_nhlfe_query_params()  */
/**
 * ABSTRACT: This function reads NHLFE parameters which are seen as atributes of LSP
 *
 * DESCRIPTION: This function reads NHLFE parameters which are seen as atributes of LSP.
 *              This function is invoked internally by ix_cc_mpls_lsp_query.
 *
 * \param  arg_NhlfeHandle        -	handle identifying an NHLFE entry 
 *                                  containing an NHLFE set
 * \param  arg_lspParams          - structere where LSP parameters are put
 * \param  arg_pContext           - pointer to the MPLS core component context 
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 * \n
 * \n Error codes:
 * \n         IX_CC_ERROR_NULL                 - invalid pointer
 * \n         IX_CC_ERROR_ENTRY_NOT_FOUND      - NHLFE does not exist
 * \n         IX_CC_MPLS_ERROR_ENTRY_NHLFE_SET - this is not an NHLFE set entry
 */
ix_error
_ix_cc_mpls_nhlfe_query_params(ix_cc_mpls_nhlfe_handle   arg_NhlfeHandle,
                                  ix_cc_mpls_lsp_param  *arg_plspParams,
                                  void                  *arg_pContext);


/*
 * FUNCTION: _ix_cc_mpls_nhlfe_add_lsp()  */
/**
 * ABSTRACT: Add LSP pointer to the list of CC NHLFE structure
 *
 * \param  arg_pCcLsp      - pointer to the CC LSP structure
 * \param  arg_pContext    - pointer to the MPLS CC context
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 * 
 * \n Error codes:
 * \n       IX_CC_ERROR_NULL            - invalid pointer
 * \n       IX_CC_MPLS_ERROR_INVALID_HANDLE - invalid MPLS handle
 */
ix_error
_ix_cc_mpls_nhlfe_add_lsp(ix_cc_mpls_cc_lsp *arg_pCcLsp,
                          void              *arg_pContext);



/*
 * FUNCTION: _ix_cc_mpls_nhlfe_del_lsp()  */
/**
 * ABSTRACT: Remove LSP pointer from the list of CC NHLFE structure
 *
 * \param  arg_pCcLsp      - pointer to the CC LSP structure
 * \param  arg_pContext    - pointer to the MPLS CC context
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 * 
 * \n Error codes:
 * \n       IX_CC_ERROR_NULL            - invalid pointer
 * \n       IX_CC_MPLS_ERROR_INVALID_HANDLE - invalid MPLS handle
 */
ix_error
_ix_cc_mpls_nhlfe_del_lsp(ix_cc_mpls_cc_lsp *arg_pCcLsp,
                          void              *arg_pContext);



/*
 * FUNCTION: _ix_cc_mpls_nhlfe_fini()  */
/**
 * ABSTRACT: removes resources taken to generate NHLFE
 *
 * DESCRIPTION: releases the NHLFE table.
 *
 * \param  arg_pContext - a pointer to the MPLS context
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 */
ix_error
_ix_cc_mpls_nhlfe_fini(void *arg_pContext);


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(IX_CC_MPLS_NHLFE_H) */


