/*
 * ============================================================================
 * COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2003 Intel Corporation. All rights reserved.
 *
 * PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *                                                                          */
/** \file      ix_cc_mpls_pkt_handler.h
 *
 *  \brief     This file defines MPLS packet handler exported functions
 *
 */
#if !defined(IX_CC_MPLS_PKT_HANDLER_H)
#define      IX_CC_MPLS_PKT_HANDLER_H

#if defined(__cplusplus)
extern "C"
{
#endif 



/*
 * FUNCTION: ix_cc_mpls_microblock_pkt_handler()  */
/**
 * ABSTRACT: microblock packet handler function for MPLS Core Component. 
 *
 * DESCRIPTION: This is the registered function to receive exception 
 *              packets from the ILM and FTN Forwarder microblocks.
 *              This function internally calls different functions and
 *              performs various operations based on the exception code
 *              for a given packet.
 *
 * \param  arg_hBuffer       - handle to a buffer which contains exception 
 *                             packets from the ILM and FTN Forwarder microblocks.
 * \param  arg_ExceptionCode - Exception codes  generated by the ILM and FTN Forwarder
 *                             microblocks
 * \param  arg_pContext      - pointer to the MPLS core component context that will 
 *                             be passed to the core component when a packet arrives.
 *                             This context was defined by the core component and 
 *                             passed to core components infrastructure through 
 *                             the ix_cc_mpls_init() function.
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 * \n
 * \n Error codes:
 *         IX_CC_ERROR_INVALID_CODE    - unsupported exception code
 *         IX_CC_ERROR_INVALID_BUFFER  - buffer handle is unusable
 *         IX_CC_ERROR_INVALID_CONTEXT - context is unusable
 *         IX_CC_ERROR_NULL            - invalid pointer
 */
ix_error ix_cc_mpls_microblock_pkt_handler(ix_buffer_handle arg_hBuffer,
    ix_uint32 arg_ExceptionCode,
    void* arg_pContext
    );


/*
 * FUNCTION: ix_cc_mpls_ilm_pkt_handler()  */
/**
 * ABSTRACT: packet handler function for MPLS Core Component. 
 *
 * DESCRIPTION: This is the registered function to receive  
 *              packets from other CC. This function forwards MPLS packets
 *              in the same way as microblock 
 *
 * \param  arg_hBuffer       - handle to a buffer which contains exception 
 *                             packets from the ILM and FTN Forwarder microblocks.
 * \param  arg_ExceptionCode - Exception codes  generated by the ILM and FTN Forwarder
 *                             microblocks
 * \param  arg_pContext      - pointer to the MPLS core component context that will 
 *                             be passed to the core component when a packet arrives.
 *                             This context was defined by the core component and 
 *                             passed to core components infrastructure through 
 *                             the ix_cc_mpls_init() function.
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 * \n
 * \n Error codes:
 *         IX_CC_ERROR_INVALID_BUFFER  - buffer handle is unusable
 *         IX_CC_ERROR_INVALID_CONTEXT - context is unusable
 *         IX_CC_ERROR_NULL            - invalid pointer
 */
ix_error
ix_cc_mpls_ilm_pkt_handler(ix_buffer_handle arg_hBuffer,
                           ix_uint32        arg_ExceptionCode,
                           void            *arg_pContext);







/*
 * FUNCTION: ix_cc_mpls_ftn_pkt_handler()  */
/**
 * ABSTRACT: packet handler function for MPLS Core Component. 
 *
 * DESCRIPTION: This is the registered function to receive  
 *              packets from other CC. This function forwards MPLS packets
 *              in the same way as microblock 
 *
 * \param  arg_hBuffer       - handle to a buffer which contains exception 
 *                             packets from the ILM and FTN Forwarder microblocks.
 * \param  arg_ExceptionCode - Exception codes  generated by the ILM and FTN Forwarder
 *                             microblocks
 * \param  arg_pContext      - pointer to the MPLS core component context that will 
 *                             be passed to the core component when a packet arrives.
 *                             This context was defined by the core component and 
 *                             passed to core components infrastructure through 
 *                             the ix_cc_mpls_init() function.
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 * \n
 * \n Error codes:
 *         IX_CC_ERROR_INVALID_BUFFER  - buffer handle is unusable
 *         IX_CC_ERROR_INVALID_CONTEXT - context is unusable
 *         IX_CC_ERROR_NULL            - invalid pointer
 */
ix_error
ix_cc_mpls_ftn_pkt_handler(ix_buffer_handle arg_hBuffer,
                           ix_uint32        arg_ExceptionCode,
                           void            *arg_pContext);



#if defined(__cplusplus)
}
#endif

#endif /* end !defined(IX_CC_MPLS_PKT_HANDLER_H) */
