#if !defined(IX_CC_CLASSIFIER_6T_HASH_H)
#define      IX_CC_CLASSIFIER_6T_HASH_H
/*
 * ============================================================================
 * COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2003 Intel Corporation. All rights reserved.
 *
 * PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *                                                                               */
/** \file   ix_cc_classifier_6t_hash.h
 *
 *  \brief  The file contains definitions used to control hash table
 *          by 6 tuple classifier Core Component
 *
 */




/*
 * DEFINITION: _IX_CC_CLASSIFIER_6T_EMPTY_FWD_ENTRY   */
/**
 * ABSTRACT: value signifies that Fwd lookup in HashEntry is empty.
 *
 * DESCRIPTIOON: This value is written into nexthopid field of
 *               _ix_cc_classifier_6t_hash_entry structure to mark
 *               the Fwd lookup is not defined.
 */
#define _IX_CC_CLASSIFIER_6T_EMPTY_FWD_ENTRY   ((ix_uint16) 0xFFFF)


/*
 * DEFINITION: _IX_CC_CLASSIFIER_6T_EMPTY_QOS_ENTRY    */
/**
 * ABSTRACT: value signifies that QoS lookup in HashEntry is empty.
 *
 * DESCRIPTIOON: This value is written into flow_id field of
 *               _ix_cc_classifier_6t_hash_entry structure to mark
 *               the Qos lookup is not defined.
 */
#define _IX_CC_CLASSIFIER_6T_EMPTY_QOS_ENTRY   ((ix_uint32) 0xFFFFFFFF)

/*
 * DEFINITION: _IX_CC_CLASSIFIER_6T_EMPTY_FIELD_ENTRY   */
/**
 * ABSTRACT: value signifies that HashEntry is all empty.
 *
 * DESCRIPTIOON: This value is written in all hash entry fields (fields of
 *               _ix_cc_classifier_6t_hash_entry structure) to mark
 *               the entry as empty. Only flow_id and nexthopid fields
 *               are marked differently with _IX_CC_CLASSIFIER_6T_EMPTY_QOS_ENTRY
 *               and _IX_CC_CLASSIFIER_6T_EMPTY_FWD_ENTRY values.
 */
#define _IX_CC_CLASSIFIER_6T_EMPTY_FIELD_ENTRY ((ix_uint32) 0x0)

/*
 * DEFINITION: _IX_CC_CLASSIFIER_6T_EMPTY_CLASS_COLOR_NEXTBLOCK_FIELD   */
/**
 * ABSTRACT: value which is anded with the class_color_nextblock field of a hash entry
 *           if qos lookup entry is empty.
 *
 * DESCRIPTIOON: this value is used by _ix_cc_classifier_6t_clear_hash_entry_lookup_result
 *               function to zero out class, color, and nextblock fields in hash entry
 *               while preserving nexthop_id_type field
 */
#define _IX_CC_CLASSIFIER_6T_EMPTY_CLASS_COLOR_NEXTBLOCK_FIELD ((ix_uint32) 0x0000F000)

/*
 * DEFINITION: _IX_CC_CLASSIFIER_6T_EMPTY_NEXT_HOP_ID_FIELD   */
/**
 * ABSTRACT: value which is anded with the class_color_nextblock fields of a hash entry
 *           if fwd lookup entry is empty
 *
 * DESCRIPTIOON: this value is used by _ix_cc_classifier_6t_clear_hash_entry_lookup_result
 *               function to zero out nexthop_id_type field
 *               while preserving class, color, and nextblock fields.
 */
#define _IX_CC_CLASSIFIER_6T_EMPTY_NEXT_HOP_ID_FIELD ((ix_uint32) 0xFFFF0FFF)

/*
 * DEFINITION: _IX_CC_CLASSIFIER_6T_EMPTY_NEXTBLOCK_FIELD   */
/**
 * ABSTRACT: value which is anded with the class_color_nextblock field of a hash entry
 *           if fwd entry has to fill in next block field.
 *
 * DESCRIPTIOON: this value is used to zero out nextblock field in a hash entry
 *               while preserving the rest of information
 */
#define _IX_CC_CLASSIFIER_6T_EMPTY_NEXTBLOCK_FIELD ((ix_uint32) 0xFFFFFF00)

/*
 * DEFINITION: NULL_NEXT_ENTRY   */
/**
 * ABSTRACT: denotes that the current hash entry is the last element on the list.
 *
 * DESCRIPTION: This value is used in NextHashEntry field of hash table entry
 *              (of _ix_cc_classifier_6t_hash_entry structure). It denotes that
 *              the current element is the last one on the list assigned to a given
 *              hash index.
 */
#define NULL_NEXT_ENTRY  ((ix_uint32) 0)



/*
 * TYPENAME: _ix_cc_classifier_6t_hash_entry  */
/**
 * ABSTRACT: This structure represents layout of a single entry in a hash table.
 *
 * DESCRIPTION: The 6-tuple classifier stores all rules in a hash table.
 *              The collisions (if some 6-tuples produce the same hash index) are resolved
 *              by chaining records on the auxiliary table into list. This structure represents
 *              layout of a single entry in a hash table.
 */
typedef struct {
    /** 32 bit address in RAM of the next entry in a chain.
     *  If this field contains a null value (all zeros), no chained entry exists.
     */
    ix_uint32  NextHashEntry;

    /** 32 bit IPv4 source address of the exact-match rule. */
    ix_uint32  hash_src;

    /** 32 bit IPv4 destination address of the exact-match rule. */
    ix_uint32  hash_dst;

    /** Destination and source port numbers of a layer 4 protocol (e.g. UDP or TCP).
     *  16 most significant bits - source port.
     *  16 less significant bits - destination port. */
    ix_uint32  hash_ports;

    /** Other protocol & interface information.
     *  16 bits - input_port (logical number of an interface on which the packet has been received).
     *  8 bits  - DSCP (DiffServ codepoint stored in Type of Service field of an IPv4 header.
     *            Only 6 left-most bits are significant. Two remaining bits must be 0).
     *  8 bits  - protocol (Layer 4 protocol number e.g. TCP or UDP)  */
    ix_uint32  hash_port_dscp_protocol;

    /** Identifier of a traffic flow to which the packet belongs.
     *  Subsequent blocks can use this value to select a policy instance. */
    ix_uint32  hash_flow;

    /** container for class_id, color_id, next_block_id.
     * 16 bits - class_id (A relative queue number within an output port).
     * 1 bit   - statistics_flag
     * 1 bit   - reserved
     * 4 bits  - nexthopID_type
     * 2 bits  - color_id (Packet drop precedence level: green, yellow or red color;
     * 8 bits  - nextblock_id (Identifier of a next microblock in a processing pipeline). */
    ix_uint32  hash_class_color_nextblock;

#if IX_BYTE_ORDER == IX_BIG_ENDIAN
    ix_uint16  hash_reserved;
    /** Identifier of a next hop information used to forward packets. */
    ix_uint16  hash_nexthop;
#elif IX_BYTE_ORDER == IX_LITTLE_ENDIAN

    /** Identifier of a next hop information used to forward packets. */
    ix_uint16  hash_nexthop;
    ix_uint16  hash_reserved;
#endif

} _ix_cc_classifier_6t_hash_entry;


/*
 * FUNCTION: _ix_cc_classifier_6t_create_hash_table  */
/**
 * ABSTRACT: This function allocates and cleans hash table.
 *
 * DESCRIPTION: This function allocates and cleans hash table. Entries are filled with
 *              _IX_CC_CLASSIFIER_6T_EMPTY_FIELD_ENTRY value which denotes that the entry
 *              is not used. Field NextHashEntry of every entry is set to NULL.
 *
 * \param arg_IndexSize   - size of hash index (in bits). For example if this parameter is 8
 *                          the procedure will allocate memory for 256 entries.
 * \param arg_AuxListSize - the size (number of entries) of auxiliary table which has to be created.
 * \param arg_MemoryType  - the type of memory which will be used to create HashTable.
 * \param arg_MemChannel  - represents the channel from which memory will be allocated.
 * \param HashTable       - on entering it should be NULL. After returning it contains
 *                          the address where the Hash Table is created.
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 * \n
 * \n Error codes:
 * \n      IX_CC_CLASSIFIER_6T_ERROR_HASH_TABLE_EXISTS - when HashTable was not NULL
 * \n      IX_CC_CLASSIFIER_6T_ERROR_HASH_IDX_SIZE - when arg_IndexSize was zero
 * \n      IX_CC_ERROR_OOM_SRAM - if there was not enough memory to create Hash Table.
 */
ix_error
_ix_cc_classifier_6t_create_hash_table(
			ix_uint32   arg_IndexSize,
			ix_uint32   arg_AuxListSize,
			ix_uint32	arg_MemoryType,
			ix_uint32	arg_MemChannel,
			_ix_cc_classifier_6t_hash_entry   **HashTable);


/*
 * FUNCTION: _ix_cc_classifier_6t_destroy_hash_table  */
/**
 * ABSTRACT: This function frees memory taken by hash table.
 *
 * DESCRIPTION: This function frees all memory allocated for hash table.
 *              On return it sets HashTable to NULL.
 *
 * \param HashTable       - hash table created by _ix_cc_classifier_6t_create_hash_table
 *                          function. After completion it will be set to NULL
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 * \n
 * \n Error codes:
 * \n      IX_CC_CLASSIFIER_6T_ERROR_INV_HASH_TABLE - when HashTable was NULL
 * \n      IX_CC_CLASSIFIER_6T_ERROR_FREE_MEM - when freemem call failed.
 */
ix_error
_ix_cc_classifier_6t_destroy_hash_table(
						_ix_cc_classifier_6t_hash_entry  **HashTable);

/*
 * FUNCTION: _ix_cc_classifier_6t_find_free_element  */
/**
 * ABSTRACT: finds the first free element in auxiliary hash table.
 *
 * DESCRIPTION: This function shearches auxiliary hash table for a free element
 *
 * \param FirstElement - pointer to the first element frow which the function starts searching
 * \param LastElement  - pointer to the last element to which the function searches
 *
 * \return pointer to a free hash element if succeeds
 * \n      NULL if fails
 */
_ix_cc_classifier_6t_hash_entry*
_ix_cc_classifier_6t_find_free_element(
    _ix_cc_classifier_6t_hash_entry  *FirstElement,
	_ix_cc_classifier_6t_hash_entry  *LastElement);


/*
 * FUNCTION: _ix_cc_classifier_6t_get_hash_entry  */
/**
 * ABSTRACT: gets data from HashElement.
 *
 * DESCRIPTION: gets data from HashElement. The type of returned result is either
 *              IX_CC_CLASSIFIER_6T_QOS or IX_CC_CLASSIFIER_6T_FWD. It is passed in
 *              arg_pLookupResult->type field.
 *
 * \param arg_pLookupResult - the data container to which data will be copied.
 *                            arg_pLookupResult->type field specifies what data we want
 *                            to copy. It must be either IX_CC_CLASSIFIER_6T_QOS or
 *                            IX_CC_CLASSIFIER_6T_FWD
 * \param HashElement  - hash element from which "LookupResult" is to be taken
 *
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 * \n
 * \n Error codes:
 * \n      IX_CC_ERROR_ENTRY_NOT_FOUND - Hash Table has not the entry of
 *                                       arg_pLookupResult->type defined.
 * \n      IX_CC_CLASSIFIER_6T_ERROR_PARAM - arg_pLookupResult->type in unknown it must
 *                                           be either IX_CC_CLASSIFIER_6T_QOS
 *                                           or X_CC_CLASSIFIER_6T_FWD
 *
 */
ix_error
_ix_cc_classifier_6t_get_hash_entry(
		       ix_cc_classifier_6t_result      *arg_pLookupResult,
		const _ix_cc_classifier_6t_hash_entry  *HashElement);


/*
 * FUNCTION: _ix_cc_classifier_6t_set_hash_entry_lookup_result  */
/**
 * ABSTRACT: puts data into entry of HashTable.
 *
 * DESCRIPTION: The function sets respective fields of Hash Element based on
 *              arg_pLookupResult parameter.
 *
 * \param arg_pLookupResult - the data which will be copied to HashElement given.
 * \param HashElement       - hash element to which "LookupResult" will be written
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 * \n
 * \n Error codes:
 * \n      IX_CC_ERROR_DUPLICATE_ENTRY - HashElement already hash entry of
 *                                       the "arg_pLookupResult->type" type defined.
 * \n      IX_CC_CLASSIFIER_6T_ERROR_PARAM - arg_pLookupResult->type in unknown. It must
 *                                           be either IX_CC_CLASSIFIER_6T_QOS
 *                                           or X_CC_CLASSIFIER_6T_FWD
 *
 */
ix_error
_ix_cc_classifier_6t_set_hash_entry_lookup_result(
	 const  ix_cc_classifier_6t_result      *arg_pLookupResult,
	       _ix_cc_classifier_6t_hash_entry  *HashElement);


/*
 * FUNCTION: _ix_cc_classifier_6t_clear_hash_entry_lookup_result  */
/**
 * ABSTRACT: Cleares given type of lookup result from entry of Hash Table.
 *
 * DESCRIPTION: The function clears fields of a lookup result in Hash Element
 *              based on arg_Type parameter.
 *
 * \param arg_Type          - type of lookup result to be deleted.
 * \param HashElement       - hash element from which lookup result will be deleted.
 *
 * \return IX_CC_ERROR_ENTRY_NOT_FOUND - HashElement has no entry of
 *                           the "arg_Type" type defined.
 * \n      IX_CC_CLASSIFIER_6T_ERROR_PARAM - arg_Type in unknown. It must
 *                           be either IX_CC_CLASSIFIER_6T_QOS or X_CC_CLASSIFIER_6T_FWD
 *
 */
ix_error
_ix_cc_classifier_6t_clear_hash_entry_lookup_result(
	      ix_cc_classifier_6t_table_type  arg_Type,
	     _ix_cc_classifier_6t_hash_entry  *HashElement);



/*
 * FUNCTION: _ix_cc_classifier_6t_zero_out_hash_entry  */
/**
 * ABSTRACT: Filles all fields of haash entry with null values.
 *
 * DESCRIPTION: The function clears all fields of a hash entry uncoditonally.
 *              It does not return any values.
 *
 * \param HashElement       - hash element from which lookup result will be deleted.
 *
 */
void
_ix_cc_classifier_6t_zero_out_hash_entry(
	     _ix_cc_classifier_6t_hash_entry  *HashElement);


/*
 * FUNCTION: _ix_cc_classifier_6t_zero_out_hash_entry_pattern  */
/**
 * ABSTRACT: Cleares all 6-tuple fields of Hash Element.
 *
 * DESCRIPTION: The function clears all 6-tuple fields of Hash Element.
 *              (i.e. source / destination addresses, source / destination TCP ports,
 *              DS, protocol). It does not clears fields of lookup results.
 *              This function must not be called before all lookup result fields
 *              will be cleared.
 *
 * \param HashElement       - hash element from which pattern info will be deleted.
 *
 */
void
_ix_cc_classifier_6t_zero_out_hash_entry_pattern(
	     _ix_cc_classifier_6t_hash_entry  *HashElement);

/*
 * FUNCTION: _ix_cc_classifier_6t_set_hash_entry_pattern */
/**
 * ABSTRACT: sets 6-tuple fields of hash element.
 *
 * DESCRIPTION: The function sets 6-tuple fields of Hash Element.
 *              (i.e. source / destination addresses, source / destination TCP ports,
 *              DS, protocol).
 *
 * \param arg_pKey          - structure containing 6-tuple fields to be copied to Hash Element.
 * \param HashElement       - hash element which will be set up.
 *
 */
void
_ix_cc_classifier_6t_set_hash_entry_pattern(
				ix_cc_classifier_6t_pattern      *arg_pKey,
				_ix_cc_classifier_6t_hash_entry  *HashElement);


/*
 * DEFINITION: _IX_CC_DIFFERENT   */
/**
 * ABSTRACT:  denotes that compared 6-tuple structure is diffrent from the one
 *            in Hash Element.
 *
 * DESCRIPTION: value returned by _ix_cc_classifier_6t_match_entry_pattern function
 *				to signify that 6-tuple structure is different from the one hold
 *              in given Hash Element.
 */
#define _IX_CC_DIFFERENT   0

/*
 * DEFINITION: _IX_CC_MATCH   */
/**
 * ABSTRACT:  denotes that compared 6-tuple structure is the same as the one
 *            in Hash Element.
 *
 * DESCRIPTION: value returned by _ix_cc_classifier_6t_match_entry_pattern function
 *				to signify that 6-tuple structure is the same as the one hold
 *              in given Hash Element.
 */
#define _IX_CC_MATCH       1


/*
 * FUNCTION: _ix_cc_classifier_6t_match_entry_pattern */
/**
 * ABSTRACT: finds if hash element contains the same 6-tuple fields as the structure passed.
 *
 * DESCRIPTION: The function finds if hash element contains the same 6-tuple fields as
 *              the structure passed (i.e. source / destination addresses,
 *              source / destination TCP ports, DS, protocol).
 *
 * \param arg_pKey          - structure containing 6-tuple fields to be checked against Hash Element.
 * \param HashElement       - hash element which will be compared.
 *
 * \return _IX_CC_MATCH     - signifies that 6-tuple structure is the same as the one hold
 *                            in given Hash Element.
 * \n      _IX_CC_DIFFERENT - signifies that 6-tuple structure is different from the one hold
 *                            in given Hash Element.
 */
ix_uint32
_ix_cc_classifier_6t_match_entry_pattern(
				ix_cc_classifier_6t_pattern     *arg_pKey,
			   _ix_cc_classifier_6t_hash_entry  *HashElement);


/*
 * DEFINITION: _IX_CC_CLASSIFIER_6T_ENTRY_EMPTY   */
/**
 * ABSTRACT: denotes that requested entry does not exist.
 *
 * DESCRIPTION: This value is return by _ix_cc_classifier_6t_isEmptyEntry
 *              and _ix_cc_classifier_6t_isEmptyElement functions to denote
 *              that requested entry does not exist.
 */
#define _IX_CC_CLASSIFIER_6T_ENTRY_EMPTY       1

/*
 * DEFINITION: _IX_CC_CLASSIFIER_6T_ENTRY_EXISTS  */
/**
 * ABSTRACT: denotes that requested entry does exist.
 *
 * DESCRIPTION: This value is return by _ix_cc_classifier_6t_isEmptyEntry
 *              and _ix_cc_classifier_6t_isEmptyElement functions to denote
 *              that requested entry does exist.
 */
#define _IX_CC_CLASSIFIER_6T_ENTRY_EXISTS      0


/*
 * FUNCTION: _ix_cc_classifier_6t_isEmptyEntry */
/**
 * ABSTRACT: checks that the HashElement has difened specified type of lookup entry.
 *
 * DESCRIPTION: The function examines whether the HashElement has difened
 *              specified type of lookup entry.
 *
 * \param arg_Type          - type of lookup entry which is searched in the Hash Element.
 * \param HashElement       - hash element which will be searched.
 *
 * \return _IX_CC_CLASSIFIER_6T_ENTRY_EXISTS  - the given hash element contains the specified
 *                                              type of lookup entry.
 * \n      _IX_CC_CLASSIFIER_6T_ENTRY_EMPTY   - the given hash element does not contain
 *                                              the specified type of lookup entry.
 */
ix_uint32
_ix_cc_classifier_6t_isEmptyEntry(
                 ix_cc_classifier_6t_table_type  arg_Type,
          const _ix_cc_classifier_6t_hash_entry  *HashElement);

/*
 * FUNCTION: _ix_cc_classifier_6t_isEmptyElement */
/**
 * ABSTRACT: checks that the HashElement has all fileds cleared.
 *
 * DESCRIPTION: The function examines whether the HashElement has all its fields empty/cleared.
 *              It is done by checking if all fields contain _IX_CC_CLASSIFIER_6T_EMPTY_FIELD_ENTRY
 *              value.
 *
 * \param HashElement       - hash element which will be checked.
 *
 * \return _IX_CC_CLASSIFIER_6T_ENTRY_EXISTS  - the given hash element is not empty
 * \n      _IX_CC_CLASSIFIER_6T_ENTRY_EMPTY   - the given hash element is empty
 */
ix_uint32
_ix_cc_classifier_6t_isEmptyElement(
	  const _ix_cc_classifier_6t_hash_entry  *HashElement);


/*
 * DEFINES: some constants addressing bit fields.
 */
#define _IX_CC_CLASSIFIER_6T_COLOR_ID_MASK         0x03
#define _IX_CC_CLASSIFIER_6T_CLASS_ID_MASK         0xFFFF
#define _IX_CC_CLASSIFIER_6T_NEXTBLOCK_ID_MASK     0xFF
#define _IX_CC_CLASSIFIER_6T_NEXTHOP_ID_TYPE_MASK  0x0F
#define _IX_CC_CLASSIFIER_6T_INPUT_PORT_MASK       0xFFFF
#define _IX_CC_CLASSIFIER_6T_SRC_PORT_MASK         0xFFFF
#define _IX_CC_CLASSIFIER_6T_DST_PORT_MASK         0xFFFF
#define _IX_CC_CLASSIFIER_6T_DSCP_MASK             0x3F
#define _IX_CC_CLASSIFIER_6T_PROTOCOL_MASK         0xFF
#define _IX_CC_CLASSIFIER_6T_COLOR_ID_SHIFT        8
#define _IX_CC_CLASSIFIER_6T_CLASS_ID_SHIFT        16
#define _IX_CC_CLASSIFIER_6T_NEXTHOP_ID_TYPE_SHIFT 12
#define _IX_CC_CLASSIFIER_6T_DSCP_SHIFT            10
#define _IX_CC_CLASSIFIER_6T_INPUT_PORT_SHIFT      16
#define _IX_CC_CLASSIFIER_6T_SRC_PORT_SHIFT        16
#define _IX_CC_CLASSIFIER_6T_STATS_ENABLE_MASK     0x01
#define _IX_CC_CLASSIFIER_6T_STATS_ENABLE_SHIFT    15


#endif

