#if !defined(IX_CC_CLASSIFIER_6T_MSG_H)
#define      IX_CC_CLASSIFIER_6T_MSG_H
/*
 * ============================================================================
 * COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2003 Intel Corporation. All rights reserved.
 *
 * PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *                                                                            */
/** \file    ix_cc_classifier_6t_msg.h
 *
 *  \brief   The file contains definitions used by message helper
 *           functions of 6 tuple classifier Core Component
 *
 */

#include "ix_cc_macros.h"

/*
 * TYPENAME: ix_cc_classifier_6t_message_ctx    */
/** 
 * ABSTRACT:
 *
 * DESCRIPTION: Context data structure for storing info
 *              in async call to add, update, and 
 *              remove an entry
 *
 */
typedef struct ix_s_cc_classifier_6t_message_ctx
{
    /* pointer to the user callback function */
    void *pCallBack;
    /* pointer to the user provided context  */
    void *pUserContext;
} ix_cc_classifier_6t_message_ctx;



/*
 * TYPENAME: ix_cc_classifier_6t_search_message_ctx    */
/** 
 * ABSTRACT:
 *
 * DESCRIPTION: Context data structure for storing info
 *              in async call to search an entry
 *
 */
typedef struct ix_s_cc_classifier_6t_search_message_ctx
{
    /* pointer to the user callback function */
    void                               *pCallBack;
    /* pointer to the user provided context  */
    void                               *pUserContext;
    /* pointer provided by the user where lookup result will be placed  */
    ix_cc_classifier_6t_result         *pUserPlaceholder;

} ix_cc_classifier_6t_search_message_ctx;


/*
 * TYPENAME: ix_cc_classifier_6t_get_stats_message_ctx    */
/** 
 * ABSTRACT:
 *
 * DESCRIPTION: Context data structure for storing info
 *              in async call to get statistics of an entry
 *
 */
typedef struct ix_s_cc_classifier_6t_get_stats_message_ctx
{
    /* pointer to the user callback function */
    void                           *pCallBack;
    /* pointer to the user provided context  */
    void                           *pUserContext;
    /* pointer provided by the user where statistics should be placed  */
    ix_cc_classifier_6t_statistics *pUserPlaceholder;

} ix_cc_classifier_6t_get_stats_message_ctx;


/*
 * TYPENAME: ix_cc_classifier_6t_message_general_data    */
/** 
 * ABSTRACT:
 *
 * DESCRIPTION: Message data structure for storing info
 *              in async call to add, update an entry
 *
 */
typedef struct ix_s_cc_classifier_6t_message_general_data
{
    ix_cc_classifier_6t_pattern Key;
    ix_cc_classifier_6t_result  LookupResult;
} ix_cc_classifier_6t_message_general_data;


#if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)
/*
 * TYPENAME: ix_cc_classifier_6t_message_stats_data  */
/** 
 * ABSTRACT:
 *
 * DESCRIPTION: Message data structure for storing info
 *              in async call to read statistics
 *
 */
typedef struct ix_s_cc_classifier_6t_message_stats_data
{
    ix_cc_classifier_6t_pattern    Key;
}ix_cc_classifier_6t_message_stats_data;
#endif


/*
 * TYPENAME: ix_cc_classifier_6t_message_search_data    */
/** 
 * ABSTRACT:
 *
 * DESCRIPTION: Message data structure for storing info
 *              in async call to search an entry
 *
 */
typedef struct ix_s_cc_classifier_6t_message_search_data
{
    ix_cc_classifier_6t_pattern      Key;
    ix_cc_classifier_6t_table_type   LookupType;
} ix_cc_classifier_6t_message_search_data;


/*
 * TYPENAME: ix_cc_classifier_6t_message_remove_data    */
/** 
 * ABSTRACT:
 *
 * DESCRIPTION: Message data structure for storing info
 *              in async call to remove an entry
 *
 */
typedef struct ix_s_cc_classifier_6t_message_remove_data
{
    ix_cc_classifier_6t_pattern     Key;
    ix_cc_classifier_6t_table_type  Type;
} ix_cc_classifier_6t_message_remove_data;

/*
 * TYPENAME: ix_cc_classifier_6t_cb_add_entry      */
/**
 * ABSTRACT:
 *
 * DESCRIPTION: Client provided callback function. 
 *              The core component invokes this function to report 
 *              the status of an hash entry adding operation
 *
 * \param  arg_Result   - indicates error conditions for the call
 * \n                     IX_SUCCESS - entry successfully added
 * \n                     IX_CC_ERROR_ENTRY_EXISTS - a classification pattern is already configured 
 * \param  arg_pContext - pointer to client-defined context, provided as 
 *                        a parameter in the API call. Used by the caller to match asynchronous 
 *                        request with a response. 
 */
typedef ix_error (*ix_cc_classifier_6t_cb_add_entry) (ix_error arg_Result,
                                                      void* arg_pContext);


/*
 * TYPENAME: ix_cc_classifier_6t_cb_remove_entry  */
/** 
 * ABSTRACT:
 *
 * DESCRIPTION: Client provided callback function. 
 *              The core component invokes this function to report 
 *              the status of a hash entry removing operation
 *
 * \param  arg_Result   - indicates error conditions for the call
 * \n                     IX_SUCCESS                  - entry successfully removed
 * \n                     IX_CC_ERROR_ENTRY_NOT_FOUND - a classification pattern does not exist 
 * \param  arg_pContext - pointer to client-defined context,
 *                        provided as a parameter in the API call. Used by the caller 
 *                        to match asynchronous request with a response.  
 */
typedef ix_error (*ix_cc_classifier_6t_cb_remove_entry) (ix_error arg_Result,
                                                         void* arg_pContext);


/*
 * TYPENAME: ix_cc_classifier_6t_cb_update_entry */
/**
 * ABSTRACT:
 *
 * DESCRIPTION: Client provided callback function. 
 *              The core component invokes this function to report 
 *              the status of a hash entry update operation
 *
 * \param  arg_Result   - indicates error conditions for the call
 * \n                     IX_SUCCESS                  - entry successfully updated
 * \n                     IX_CC_ERROR_ENTRY_NOT_FOUND - a classification pattern is not configured 
 * \param  arg_pContext - pointer to client-defined context,
 *                        provided as a parameter in the API call. Used by the caller 
 *                        to match asynchronous request with a response. 
 * 
 */
typedef ix_error (*ix_cc_classifier_6t_cb_update_entry) (ix_error arg_Result,
                                                         void* arg_pContext);



/*
 * TYPENAME: ix_cc_classifier_6t_cb_search_entry         */
/** 
 * ABSTRACT:
 *
 * DESCRIPTION: Client provided callback function. 
 *              The core component invokes this function to report 
 *              the status of a hash entry search operation
 *
 * \param arg_Result        - indicates error conditions for the call
 * \n                         IX_SUCCESS                  - entry successfully updated
 * \n                         IX_CC_ERROR_ENTRY_NOT_FOUND - a classification pattern is not configured
 * \param arg_pLookupResult - lookup results associated with the requested 
 *                            classification pattern.
 * \param arg_pContext      - pointer to client-defined context, provided as 
 *                            a parameter in the API call. Used by the caller to match
 *                            asynchronous request with a response.
 *
 */ 
typedef ix_error (*ix_cc_classifier_6t_cb_search_entry) (ix_error arg_Result,
                                                         ix_cc_classifier_6t_result *arg_pLookupResult,
                                                         void* arg_pContext);


#if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)
                                                         
/*
 * TYPENAME: ix_cc_classifier_6t_cb_get_statistics         */
/** 
 * ABSTRACT:
 *
 * DESCRIPTION: Client provided callback function. 
 *              The core component invokes this function to report 
 *              the status of reading statistics.
 *
 * \param arg_Result        - indicates error conditions for the call
 * \n                         IX_SUCCESS                  - entry read successful
 * \n                         IX_CC_ERROR_ENTRY_NOT_FOUND - a classification pattern is not configured
 * \n                         IX_CC_CLASSIFIER_6T_ERROR_STATS_NOT_SUPPORTED - the block does not
 * \n                                                                         support statistics
 * \param arg_pStatistics   - if the error code is IX_SUCCESS, the argument contains statistics
 * \n                         associated with the requested classification pattern. otherwise,
 * \n                         the structure is not changed.
 * \param arg_pContext      - pointer to client-defined context, provided as 
 *                            a parameter in the API call. Used by the caller to match
 *                            asynchronous request with a response.
 *
 */ 
typedef ix_error (*ix_cc_classifier_6t_cb_get_statistics) (ix_error arg_Result,
                                                           ix_cc_classifier_6t_statistics *arg_pStatistics,
                                                           void* arg_pContext);


/*
 * TYPENAME: ix_cc_classifier_6t_cb_get_statistics_def         */
/** 
 * ABSTRACT:
 *
 * DESCRIPTION: Client provided callback function. 
 *              The core component invokes this function to report 
 *              the status of reading statistics.
 *
 * \param arg_Result        - indicates error conditions for the call
 * \n                         IX_SUCCESS                  - entry read successful
 * \n                         IX_CC_CLASSIFIER_6T_ERROR_STATS_NOT_SUPPORTED - the block does not
 * \n                                                                         support statistics
 * \param arg_pStatistics   - if the error code is IX_SUCCESS, the argument contains statistics
 * \n                         associated with the default rule. otherwise, the structure
 * \n                         is not changed.
 * \param arg_pContext      - pointer to client-defined context, provided as 
 *                            a parameter in the API call. Used by the caller to match
 *                            asynchronous request with a response.
 *
 */ 
typedef ix_error (*ix_cc_classifier_6t_cb_get_statistics_def) (ix_error arg_Result,
                                                               ix_cc_classifier_6t_statistics *arg_pStatistics,
                                                               void* arg_pContext);


#endif


/*
 * FUNCTION: _ix_cc_classifier_6t_cbi_add_entry                 */
/** 
 * ABSTRACT:    internal callback for the async messaging API
 *
 * DESCRIPTION: This function is an internal callback for the async 
 *              messaging API, ix_cc_classifier_6t_async_add_entry().
 * 
 * \param  arg_Result   - result of the asynchronous operation
 * \param  arg_pContext - context provided by caller
 * \param  arg_Msg      - return message
 * \param  arg_MsgLen   - length of the message
 * 
 * \return IX_SUCCESS                         - operation completed successfully
 * \n       IX_CC_ERROR_NULL                  - invalid pointer to message context, user 
 *                                              callback function, or lookup result structure
 * \n      IX_CC_ERROR_SEND_FAIL  - failure at sending a message 
 */
PRIVATE ix_error
_ix_cc_classifier_6t_cbi_add_entry(ix_error  arg_Result,
                                   void     *arg_pContext,
                                   void     *arg_Msg,
                                   ix_uint32 arg_MsgLen);



/*
 * FUNCTION: ix_cc_classifier_6t_async_add_entry()                                    */
/**
 * ABSTRACT:    sends an entry add message to the 6-tuple classifier CC
 *
 * DESCRIPTION: This helper function builds and sends an entry add message 
 *              to the 6-tuple classifier CC. The core component uses 
 *              the classification pattern, arg_pKey, to prepare a hash input.
 *              Next, it employs a hardware hash unit to generate a hash table index.
 *              If the hash table already contains a specified classification pattern,
 *              the core component reports an error (via callback). It does not 
 *              update lookup results associated with the existing rule. 
 *              Otherwise, if a hash lookup fails, the core component adds a
 *              new entry to the hash table. The lookup result, arg_pLookupResult,
 *              is stored in the newly added entry.
 * 
 * \param  arg_pKey          - pointer to the classification pattern structure,
 *                             which defines an exact-match rule.
 * \param  arg_pLookupResult - it identifies a logical lookup table
 *                             (IX_CC_CLASSIFIER_6T_QOS or IX_CC_CLASSIFIER_6T_FWD)
 *                             and points at the structure of entry data
 *                             to be added to specified classification pattern                        
 * \param  arg_Callback      - a client provided callback function. The core component 
 *                             invokes this function to report an operation status.
 * \param  arg_pUserContext  - pointer to client defined context. This value is
 *                             opaque to the core component. The context is returned in
 *                             a callback function.
 *
 * \return IX_SUCCESS if message was successfully built and sent or 
 *                    a valid error token for failure.
 * \n Error codes:
 * \n      IX_SUCCESS                        - 
 * \n      IX_CC_ERROR_OOM_SYSTEM            - not enought memory to send a message
 * \n      IX_CC_ERROR_NULL                  - invalid pointer in input parameters
 * \n      IX_CC_ERROR_SEND_FAIL - failure at sending a message
 */
ix_error
ix_cc_classifier_6t_async_add_entry(ix_cc_classifier_6t_pattern     *arg_pKey,
                                    ix_cc_classifier_6t_result      *arg_pLookupResult,
                                    ix_cc_classifier_6t_cb_add_entry arg_Callback,
                                    void*                            arg_pUserContext);



/*
 * FUNCTION: _ix_cc_classifier_6t_cbi_remove_entry */
/**
 * ABSTRACT:    internal callback for the async messaging API
 *
 * DESCRIPTION: This function is an internal callback for the async 
 *              messaging API, ix_cc_classifier_6t_async_remove_entry().
 * 
 * \param  arg_Result   - result of the asynchronous operation
 * \param  arg_pContext - context provided by caller
 * \param  arg_Msg      - return message
 * \param  arg_MsgLen   - length of the message
 * 
 * \return  IX_SUCCESS if successful or a valid ix_error for failure.
 * \n       IX_CC_ERROR_NULL                  - invalid pointer to message context, user 
 *                                              callback function, or lookup result structure
 * \n      IX_CC_ERROR_SEND_FAIL  - failure at sending a message 
 */
PRIVATE ix_error
_ix_cc_classifier_6t_cbi_remove_entry(ix_error  arg_Result,
                                  void     *arg_pContext,
                                  void     *arg_Msg,
                                  ix_uint32 arg_MsgLen);


/*
 * FUNCTION: ix_cc_classifier_6t_async_remove_entry() */
/**
 * ABSTRACT:    sends an entry remove message to Classifier Core Component
 *
 * DESCRIPTION: This helper function builds and sends an entry remove message
 *              to the classifier CC. The core component uses the classification
 *              pattern, arg_pKey, to prepare a hash input. Next, it employs 
 *              a hardware hash unit to generate a hash table index. If the hash 
 *              table already contains a specified classification pattern,
 *              the core component removes an entry. Otherwise, if a hash lookup
 *              fails, the core component reports an error. No entry is removed
 *              from the hash table.
 * 
 * \param   arg_pKey         - pointer to the classification pattern structure,
 *                             which defines an exact-match rule.
 * \param   arg_Type         - identifier of a logical lookup table;
 *                             it should be IX_CC_CLASSIFIER_6T_QOS or IX_CC_CLASSIFIER_6T_FWD 
 * \param   arg_Callback     - a client provided callback function. The core component 
 *                             invokes this function to report an operation status.
 * \param   arg_pUserContext - pointer to client defined context. This value is
 *                             opaque to the core component. The context is returned in
 *                             a callback function.
 *
 * \return IX_SUCCESS 				          - message was successfully built and sent 
 * \n      IX_CC_ERROR_NULL                   - invalid pointer in input parameters
 * \n      IX_CC_ERROR_OOM_SYSTEM             - not enough memeory to create context for the message
 * \n      IX_CC_ERROR_SEND_FAIL  - failure at sending a message 
 */
ix_error
ix_cc_classifier_6t_async_remove_entry(ix_cc_classifier_6t_pattern        *arg_pKey,
                                       ix_cc_classifier_6t_table_type      arg_Type,
                                       ix_cc_classifier_6t_cb_remove_entry arg_Callback,
                                       void*                               arg_pUserContext);





/*
 * FUNCTION: _ix_cc_classifier_6t_cbi_update_entry  */
/**
 * ABSTRACT:    internal callback for the async messaging API
 * 
 * DESCRIPTION: This function is an internal callback for the async 
 *              messaging API, ix_cc_classifier_6t_async_update_entry().
 * 
 * \param  arg_Result   - result of the asynchronous operation
 * \param  arg_pContext - context provided by caller
 * \param  arg_Msg      - return message
 * \param  arg_MsgLen   - length of the message
 * 
 * \return  IX_SUCCESS if successful.
 * \n       IX_CC_ERROR_NULL                  - invalid pointer to message context, user 
 *                                              callback function, or lookup result structure
 * \n       IX_CC_CLASSIFIER_6T_ERROR_USER_CB - user provided callback function failed
 */
PRIVATE ix_error
_ix_cc_classifier_6t_cbi_update_entry(ix_error  arg_Result,
                                  void     *arg_pContext,
                                  void     *arg_Msg,
                                  ix_uint32 arg_MsgLen);





/*
 * FUNCTION: ix_cc_classifier_6t_async_update_entry()  */
/**
 * ABSTRACT:    builds and sends an entry update message to 6-tuple Classifier Core Component
 *
 * DESCRIPTION: This helper function builds and sends an entry update message 
 *              to the 6-tuple classifier CC. The core component uses the 
 *              classification pattern, arg_pKey, to prepare a hash input.
 *              Next, it employs a hardware hash unit to generate a hash table index.
 *              If the hash lookup fails (classification pattern is not configured),
 *              the core component reports an error. On successful lookup,
 *              the core component updates result fields in the hash table entry.
 * 
 * \param  arg_pKey           - pointer to the classification pattern structure,
 *                              which defines an exact-match rule.
 * \param  arg_pLookupResult  - it identifies a logical lookup table
 *                              (IX_CC_CLASSIFIER_6T_QOS or IX_CC_CLASSIFIER_6T_FWD)
 *                              and points at the structure of entry data
 *                              to be updated in the specified classification pattern
 * \param  arg_Callback       - a client provided callback function. The core component 
 *                              invokes this function to report an operation status.
 * \param  arg_pUserContext   - pointer to client defined context. This value is
 *                              opaque to the core component. The context is returned in
 *                              a callback function.
 *
 * \return IX_SUCCESS 						 - message was successfully built and sent 
 * \n      IX_CC_ERROR_OOM_SYSTEM            - not enought memory to send a message
 * \n      IX_CC_ERROR_NULL                  - invalid pointer in input parameters
 * \n      IX_CC_ERROR_SEND_FAIL - failure at sending a message
 */
ix_error
ix_cc_classifier_6t_async_update_entry(ix_cc_classifier_6t_pattern        *arg_pKey,
                                       ix_cc_classifier_6t_result         *arg_pLookupResult,
                                       ix_cc_classifier_6t_cb_update_entry arg_Callback,
                                       void                               *arg_pUserContext);


/*
 * FUNCTION: _ix_cc_classifier_6t_cbi_search_entry        */
/**
 * ABSTRACT:    internal callback for the async messaging API
 * 
 * DESCRIPTION: This function is an internal callback for the async 
 *              messaging API, ix_cc_classifier_6t_async_search_entry().
 * 
 * \param  arg_Result   - result of the asynchronous operation
 * \param  arg_pContext - context provided by caller
 * \param  arg_Msg      - return message
 * \param  arg_MsgLen   - length of the message
 * 
 * \return  IX_SUCCESS                        - operation completed successfully
 * \n       IX_CC_ERROR_NULL                  - invalid pointer to message context, user 
 *                                              callback function, or lookup result structure
 * \n       IX_CC_CLASSIFIER_6T_ERROR_USER_CB - user provided callback function failed
 */
PRIVATE ix_error
_ix_cc_classifier_6t_cbi_search_entry(ix_error  arg_Result,
                                      void     *arg_pContext,
                                      void     *arg_Msg,
                                      ix_uint32 arg_MsgLen);



/*
 * FUNCTION: ix_cc_classifier_6t_async_search_entry()   */
/**
 * ABSTRACT:    builds and sends an entry search message to the 6-tuple classifier CC
 * 
 * DESCRIPTION: This helper function builds and sends an entry search message 
 *              to the 6-tuple classifier CC. on receiving the message, the core component
 *              invokes ix_cc_classifier_6t_search_entry API function.
 * 
 * \param  arg_pKey          - pointer to the classification pattern structure,
 *                             which defines an exact-match rule.
 * \param  arg_LookupType    - identifies a logical lookup table;
 *                             (IX_CC_CLASSIFIER_6T_QOS or IX_CC_CLASSIFIER_6T_FWD)
 * \param  arg_LookupResult  - user placeholder where statistics will be returned. User has 
 *                             to allocate and free this memory.
 * \param  arg_Callback      - a client provided callback function. The core component 
 *                             invokes this function to report an operation status.
 * \param  arg_pUserContext  - pointer to client defined context. This value is
 *                             opaque to the core component. The context is returned in
 *                             a callback function.
 *
 * \return  IX_SUCCESS 			      - message was successfully built and sent 
 * \n       IX_CC_ERROR_NULL                  - invalid pointer in input parameters
 * \n       IX_CC_ERROR_SEND_FAIL             - failure at sending a message 
 * \n       IX_CC_CLASSIFIER_6T_ERROR_USER_CB - user provided callback function failed
 */
ix_error
ix_cc_classifier_6t_async_search_entry(ix_cc_classifier_6t_pattern        *arg_pKey,
                                       ix_cc_classifier_6t_table_type      arg_LookupType,
                                       ix_cc_classifier_6t_result         *arg_LookupResult,
                                       ix_cc_classifier_6t_cb_search_entry arg_Callback,
                                       void                               *arg_pUserContext);




#if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)

                                       /*
 * FUNCTION: ix_cc_classifier_6t_cbi_get_statistics_def    */
/**
 * ABSTRACT:    internal callback for the async messaging API
 * 
 * DESCRIPTION: This function is an internal callback for the async 
 *              messaging API, ix_cc_classifier_6t_async_get_statistics_def().
 * 
 * \param  arg_Result   - result of the asynchronous operation
 * \param  arg_pContext - context provided by caller
 * \param  arg_Msg      - return message
 * \param  arg_MsgLen   - length of the message
 * 
 * \return  IX_SUCCESS                        - operation completed successfully
 * \n       IX_CC_ERROR_NULL                  - invalid pointer to message context, user 
 *                                              callback function, or lookup result structure
 * \n       IX_CC_CLASSIFIER_6T_ERROR_USER_CB - user provided callback function failed
 */
PRIVATE ix_error
ix_cc_classifier_6t_cbi_get_statistics_def(ix_error  arg_Result,
                                           void     *arg_pContext,
                                           void     *arg_Msg,
                                           ix_uint32 arg_MsgLen);


/*
 * FUNCTION: ix_cc_classifier_6t_async_get_statistics_def()    */
/**
 * ABSTRACT:    builds and sends a get statistics def message to the 6-tuple classifier CC
 * 
 * DESCRIPTION: This helper function builds and sends a get statistics def message 
 *              to the 6-tuple classifier CC. on receiving the message, the core component
 *              invokes ix_cc_classifier_6t_get_statistics_def API function.
 * 
 * \param  arg_pStatistics   - pointer to a placeholder for the statistics values
 *                             associated with the default rule.
 * \param  arg_Callback      - a client provided callback function. The core component 
 *                             invokes this function to report an operation status.
 * \param  arg_pUserContext  - pointer to client defined context. This value is
 *                             opaque to the core component. The context is returned in
 *                             a callback function.
 *
 * \return  IX_SUCCESS 			      - message was successfully built and sent 
 * \n       IX_CC_ERROR_OOM_SYSTEM            - not enough memory to send a message
 * \n       IX_CC_ERROR_NULL                  - invalid pointer in input parameters
 * \n       IX_CC_ERROR_SEND_FAIL             - failure at sending a message 
 *
 */
ix_error
ix_cc_classifier_6t_async_get_statistics_def(ix_cc_classifier_6t_statistics        *arg_pStatistics,
                                             ix_cc_classifier_6t_cb_get_statistics  arg_Callback,
                                             void                                  *arg_pUserContext);


/*
 * FUNCTION: ix_cc_classifier_6t_cbi_get_statistics    */
/**
 * ABSTRACT:    internal callback for the async messaging API
 * 
 * DESCRIPTION: This function is an internal callback for the async 
 *              messaging API, ix_cc_classifier_6t_async_get_statistics().
 * 
 * \param  arg_Result   - result of the asynchronous operation
 * \param  arg_pContext - context provided by caller
 * \param  arg_Msg      - return message
 * \param  arg_MsgLen   - length of the message
 * 
 * \return  IX_SUCCESS                        - operation completed successfully
 * \n       IX_CC_ERROR_NULL                  - invalid pointer to message context, user 
 *                                              callback function, or lookup result structure
 * \n       IX_CC_CLASSIFIER_6T_ERROR_USER_CB - user provided callback function failed
 */
PRIVATE ix_error
ix_cc_classifier_6t_cbi_get_statistics(ix_error  arg_Result,
                                       void     *arg_pContext,
                                       void     *arg_Msg,
                                       ix_uint32 arg_MsgLen);


/*
 * FUNCTION: ix_cc_classifier_6t_async_get_statistics()    */
/**
 * ABSTRACT:    builds and sends a get statistics message to the 6-tuple classifier CC
 * 
 * DESCRIPTION: This helper function builds and sends a get statistics message 
 *              to the 6-tuple classifier CC. on receiving the message, the core component
 *              invokes ix_cc_classifier_6t_get_statistics API function.
 * 
 * \param  arg_pKey          - pointer to the classification pattern structure,
 *                             which defines an exact-match rule.
 * \param  arg_pStatistics   - pointer to a placeholder for the statistics values
 *                             associated with the default rule.
 * \param  arg_Callback      - a client provided callback function. The core component 
 *                             invokes this function to report an operation status.
 * \param  arg_pUserContext  - pointer to client defined context. This value is
 *                             opaque to the core component. The context is returned in
 *                             a callback function.
 *
 * \return  IX_SUCCESS 			      - message was successfully built and sent 
 * \n       IX_CC_ERROR_OOM_SYSTEM            - not enough memory to send a message
 * \n       IX_CC_ERROR_NULL                  - invalid pointer in input parameters
 * \n       IX_CC_ERROR_SEND_FAIL             - failure at sending a message 
 *
 */
ix_error
ix_cc_classifier_6t_async_get_statistics(ix_cc_classifier_6t_pattern           *arg_pKey,
                                         ix_cc_classifier_6t_statistics        *arg_pStatistics,
                                         ix_cc_classifier_6t_cb_get_statistics  arg_Callback,
                                         void                                  *arg_pUserContext);

#endif


#endif
