#if !defined(IX_CC_CLASSIFIER_6T_STATS_H)
#define      IX_CC_CLASSIFIER_6T_STATS_H
/*
 * ============================================================================
 * COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2003 Intel Corporation. All rights reserved.
 *
 * PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *                                                                               */
/** \file   ix_cc_classifier_6t_stats.h
 *
 *  \brief  The file contains definitions used to manage statistics table
 *          by 6 tuple classifier Core Component
 *
 */

/*
 * TYPENAME: ix_cc_classifier_6t_statistics  */
/**
 * ABSTRACT: This structure represents layout of a single entry in a statistics table.
 *
 * DESCRIPTION: The 6-tuple classifier implements per rule statistics.
 *              Statistics table entry associated with 6-tuple classification rules
 *              consist of two 64-bit counters: packet counter and byte counter.
 *              
 */
typedef struct ix_s_cc_classifier_6t_statistics
{
	/* number of packets matching the rule */
	ix_uint64 packets;

	/* number of bytes matching the rule */
	ix_uint64 bytes;

}ix_cc_classifier_6t_statistics;


/*
 * FUNCTION: ix_cc_classifier_6t_create_stats_table  */
/**
 * DESCRIPTION: This function allocates and zeroes Statistics Table.
 *
 * \param arg_IndexSize   - size of hash index (in bits). For example if this parameter is 8
 *                          the procedure will allocate memory for 256 entries.
 * \param arg_AuxListSize - the size (number of entries) of auxiliary table.
 * \param arg_MemChannel  - represents the channel from which memory will be allocated.
 * \param StatsTable      - on entering it should be NULL. After returning it contains
 *                          the address where the Statistics Table is created.
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 * \n
 * \n Error codes:
 * \n      IX_CC_CLASSIFIER_6T_ERROR_HASH_TABLE_EXISTS - when HashTable was not NULL
 * \n      IX_CC_CLASSIFIER_6T_ERROR_HASH_IDX_SIZE - when arg_IndexSize was zero
 * \n      IX_CC_ERROR_OOM_SRAM - if there was not enough memory to create Hash Table.
 */
ix_error
ix_cc_classifier_6t_create_stats_table(
			ix_uint32   arg_IndexSize,
			ix_uint32   arg_AuxListSize,
			ix_uint32   arg_MemChannel,
			ix_cc_classifier_6t_statistics   **StatsTable,
			ix_cc_classifier_6t_statistics   **DefaultStat);



/*
 * FUNCTION: ix_cc_classifier_6t_destroy_stats_table    */
ix_error
ix_cc_classifier_6t_destroy_stats_table(ix_cc_classifier_6t_statistics  **StatsTable);

/*
 * FUNCTION: ix_cc_classifier_6t_destroy_default_stat   */
ix_error
ix_cc_classifier_6t_destroy_default_stat(ix_cc_classifier_6t_statistics  **DefaultStat);

#endif
