#if !defined(IX_CC_CLASSIFIER_6T_UTILS_H)
#define      IX_CC_CLASSIFIER_6T_UTILS_H
/*
 * ============================================================================
 * COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2003 Intel Corporation. All rights reserved.
 *
 * PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *                                                                           */
/** \file    ix_cc_classifier_6t_utils.h
 *
 *  \brief   The file contains definitions used internally by 
 *           6 tuple classifier Core Component
 *
 */


/* define strings for repository access */
#define IX_CC_CLASSIFIER_6T_REGISTRY_MEV2_NUM_MASK             "CLASSIFIER_6T/MEV2_NUM_MASK"
#define IX_CC_CLASSIFIER_6T_REGISTRY_AUX_HASH_TABLE_SIZE       "CLASSIFIER_6T/AUX_HASH_TABLE_SIZE"
#define IX_CC_CLASSIFIER_6T_REGISTRY_HASH_TABLE_CHANNEL_NUM    "CLASSIFIER_6T/HASH_TABLE_CHANNEL_NUM"
#define IX_CC_CLASSIFIER_6T_REGISTRY_UENG_NUM                  "CLASSIFIER_6T/UENG_NUM"
#define IX_CC_CLASSIFIER_6T_REGISTRY_DEFAULT_COLOR_ID          "CLASSIFIER_6T/DEFAULT_COLOR_ID"
#define IX_CC_CLASSIFIER_6T_REGISTRY_DEFAULT_FLOW_ID           "CLASSIFIER_6T/DEFAULT_FLOW_ID"
#define IX_CC_CLASSIFIER_6T_REGISTRY_DEFAULT_CLASS_ID          "CLASSIFIER_6T/DEFAULT_CLASS_ID"
#define IX_CC_CLASSIFIER_6T_REGISTRY_DEFAULT_NEXTHOP_ID        "CLASSIFIER_6T/DEFAULT_NEXTHOP_ID"
#define IX_CC_CLASSIFIER_6T_REGISTRY_DEFAULT_NEXTHOP_ID_TYPE   "CLASSIFIER_6T/DEFAULT_NEXTHOP_ID_TYPE"

#if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)
#define IX_CC_CLASSIFIER_6T_REGISTRY_STATS_TABLE_CHANNEL_NUM   "CLASSIFIER_6T/STATS_TABLE_CHANNEL_NUM"
#endif /* #if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE) */

/*! default microengines to be patched when there is no system regisrty */
#define IX_CC_CLASSIFIER_6T_DEFAULT_MES_NUMBERS                0x66

/*
 * MACRO NAME: SetUblockSymbols     */
/**
 * ABSTRACT:
 *
 * DESCRIPTION: This macro sets microblock symbols into the ix_imported_symbol data structure
 *
 *  \param  argSymbolStruct - imported symbol structure .
 *  \param  argValue        - value of the imported variable.
 *  \param  argName         - name of the imported variable
 *
 *  \return Fills data structure with the name and value
 */
#define SetUblockSymbols( argSymbolStruct,argValue,argName) { \
            argSymbolStruct.m_Value = argValue;               \
            argSymbolStruct.m_Name = (char*)argName;          \
        }


#if defined(IX_EXCLUDE_CCI)

   /*
	* FUNCTION: _ix_cc_classifier_6t_reg_pkt_msg_handlers()   */
   /**
	* ABSTRACT:
	*
	* DESCRIPTION: Registers message and packet handlers with the Framework RM
	* 
	* \param pContext - classifier core component context that will be passed to the 
	*                   packet handler on each invocation. It maust be valid as long the packet
	*                   handler is registered
	*
	* \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 	*/
	ix_error 
	_ix_cc_classifier_6t_reg_pkt_msg_handlers(_ix_cc_classifier_6t_context *pContext);

#else

   /*
	* FUNCTION: _ix_cc_classifier_6t_reg_pkt_msg_handlers()   */
   /**
	* ABSTRACT:
	*
	* DESCRIPTION: Registers message and packet handlers with the Framework CCI
	* 
	* \param arg_hComponent - handle passed to functions registering message/packet handles
	*                         to give access to the execution engine.
	*
	* \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
	*/
	ix_error
	_ix_cc_classifier_6t_reg_pkt_msg_handlers(const ix_cc_handle arg_hComponent);

#endif


/*
 * FUNCTION: _ix_cc_classifier_6t_dereg_pkt_msg_handlers     */
/**
 * ABSTRACT:    This macro deregisters all packet and message handlers registered for
 *				classifier core component
 *
 * DESCRIPTION: This macro deregisters all packet and message handlers registered for
 *				classifier core component. Depending on the IX_CCI_EXCLUDE constant
 *				it uses CCI API or directly API of Resource Manager.
 *
 * \return IX_SUCCESS or valid ix_error;
 */
ix_error 
_ix_cc_classifier_6t_dereg_pkt_msg_handlers();

/*
 * FUNCTION: _ix_cc_classifier_6t_get_config()             */
/**
 * ABSTRACT: Populates configuration data for classifier core componenet
 *
 * DESCRIPTION: Gets configuration data from the system registry or from 
 *				static definitions if registry is not used and puts it 
 *              into classifier context.
 * 
 * \param   pContext - pointer to the classifier_6t config where configuration data structure 
 *                     will be written
 *
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 * \n
 * \n Error codes:
 * \n      IX_CC_CLASSIFIER_6T_ERROR_REPOSITORY_OPEN - could not open one of the classifier's
 *                                                     register branches
 * \n      IX_CC_CLASSIFIER_6T_ERROR_REPOSITORY_READ - could not read one of the classifier's
 *                                                     register branches 
 *
 */
ix_error 
_ix_cc_classifier_6t_get_config(_ix_cc_classifier_6t_context* pContext);


/*
 * FUNCTION: _ix_cc_classifier_6t_patch_microblock()     */
/**
 * ABSTRACT:    Patches classifier_6t microblock.
 *
 * DESCRIPTION: Patches classifier_6t microblock with configuration data form
 *              context passed.
 * 
 * \param  pConfig - pointer to the classifier_6t core component context 
 *                   which among others contains data to be patched into microcode.
 *
 * \return IX_SUCCESS if the operation has been successful or a valid error token for failure.
 *
 */
ix_error 
_ix_cc_classifier_6t_patch_microblock(const _ix_cc_classifier_6t_context* pContext);


#endif
