/*
 * ============================================================================
 * COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2003 Intel Corporation. All rights reserved.
 *
 * PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *                                                                          */
/** \file      ix_cc_classifier_6t.c
 *
 *  \brief     The file contains common functions that are exposed by all
 *             Core Components(i.e. init, fini, etc.)
 *
 */
#if !defined(IX_CC_CLASSIFIER_6T_C)
#define      IX_CC_CLASSIFIER_6T_C

#include "ix_types.h"
#include "ix_error.h"

#include "ix_ossl.h"
#include "ix_rm.h"
#include "ix_cci.h"
#include "ix_cc.h"
#include "ix_cc_error.h"
#include "ix_cc_macros.h"
#include "ix_netmacros.h"
#include "bindings.h"

#include "cc/ix_cc_msup.h"

#include "../../include/classifier_6t_defs.h"

#include "cc/ix_cc_classifier_6t.h"
#include "cc/ix_cc_classifier_6t_stats.h"
#include "cc/ix_cc_classifier_6t_utils.h"
#include "cc/ix_cc_classifier_6t_msg.h"
#include "cc/ix_cc_classifier_6t_hash_comp.h"



/*
 * FUNCTION: ix_cc_classifier_6t_init()
 */
ix_error ix_cc_classifier_6t_init(ix_cc_handle arg_cc_handle, void **arg_ppContext)
{
    /* holder of physical memory */
    ix_error                     err = IX_SUCCESS;
    void*                        PhysAdd;
    ix_memory_type               memType;
   _ix_cc_classifier_6t_context* pContext;

    #ifdef IX_CLASSIFIER_6T_DEBUG
    ix_ossl_message_log("ix_cc_classifier_6t_init(): ha, ha I'am in :))))))\n");
    #endif

    /* Check the input arguments */
    if (NULL == arg_ppContext)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                ("ix_cc_classifier_6t_init(): got null pointer."));

    /* Check the input arguments */
    if (IX_NULL_HANDLE == arg_cc_handle)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("ix_cc_classifier_6t_init(): Passed null core componenet handle."));

    /* get memory for context structure */
    pContext = (_ix_cc_classifier_6t_context*)ix_ossl_malloc(sizeof(_ix_cc_classifier_6t_context));
    if (NULL == pContext)
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM, ("ix_cc_classifier_6t_init(): No system memory available to allocate room for core component information."));

#ifdef IX_CLASSIFIER_6T_DEBUG
ix_ossl_message_log("ix_cc_classifier_6t context = 0x%x\n", (int) pContext);
#endif

    /* get memory for config structure */
    pContext->pUblockConfig = (ix_cc_classifier_6t_ublock_config*)ix_ossl_malloc(sizeof(ix_cc_classifier_6t_ublock_config));
    if (NULL == pContext->pUblockConfig)
    {
        err = IX_ERROR_WARNING(IX_CC_ERROR_OOM, ("ix_cc_classifier_6t_init(): No system memory available to allocate room for ublock information"));
        goto err_ublockconfig_mem;
    }

    pContext->hClassifier6tHandle = arg_cc_handle;
    pContext->HashTableAddress = NULL;

#if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)
    pContext->StatsTableAddress = NULL;
    pContext->DefaultStatAddress = NULL;
#endif /* #if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE) */

    /* this function cleans own memory in case of error */
    err = (_ix_cc_classifier_6t_get_config(pContext));
    if (err != IX_SUCCESS)
    {
        err = IX_ERROR_WARNING(IX_ERROR_GET_CODE(err), ("ix_cc_classifier_6t_init(): Error during getting configuration"));
        goto err_get_config;
    }

    err = _ix_cc_classifier_6t_create_hash_table(pContext->pUblockConfig->HashIndexSize,
                                                 pContext->AuxEntryEmpty,
                                                 IX_CC_CLASSIFIER_6T_HASH_TABLE_MEM_TYPE,
                                                 pContext->HashTableChannelNum,
                                                 &(pContext->HashTableAddress));

    if (err != IX_SUCCESS)
    {
        err = IX_ERROR_WARNING(IX_ERROR_GET_CODE(err), ("ix_cc_classifier_6t_init(): Error ! Could not create Hash Table."));
        goto err_cr_hash;
    }


    err = ix_rm_get_phys_offset((void*)pContext->HashTableAddress,
                                &memType, &pContext->HashTableChannelNum,
                                &pContext->pUblockConfig->HashTableBase,
                                &PhysAdd);
    if (err != IX_SUCCESS)
    {
        err = IX_ERROR_WARNING(IX_ERROR_GET_CODE(err), ("ix_cc_classifier_6t_init(): Error ! Could not convert virtual address to offset understood by mengine."));
        goto err_get_offset;
    }


     /* set channel number into offset */
     IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(pContext->HashTableChannelNum,
                                        pContext->pUblockConfig->HashTableBase);


     /* Allocate SRAM memory for 64-bit statistics table and initialize it
        with zero values. */
#if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)

    err = ix_cc_classifier_6t_create_stats_table(pContext->pUblockConfig->HashIndexSize,
                                                 pContext->AuxEntryEmpty,
                                                 pContext->StatsTableChannelNum,
                                                 &(pContext->StatsTableAddress),
                                                 &(pContext->DefaultStatAddress));

    if (err != IX_SUCCESS)
    {
        err = IX_ERROR_WARNING(IX_ERROR_GET_CODE(err), ("ix_cc_classifier_6t_init(): Error ! Could not create Statistics Table."));
        goto err_cr_stats;
    }

    err = ix_rm_get_phys_offset((void*)pContext->StatsTableAddress,
                                &memType, &pContext->StatsTableChannelNum,
                                &pContext->pUblockConfig->StatsTableBase,
                                &PhysAdd);
    if (err != IX_SUCCESS)
    {
        err = IX_ERROR_WARNING(IX_ERROR_GET_CODE(err), ("ix_cc_classifier_6t_init(): Error ! Could not convert virtual address of Statistics Table to offset understood by uengine."));
        goto err_stats_get_offset;
    }

     /* set channel number into offset */
     IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(pContext->StatsTableChannelNum,
                                        pContext->pUblockConfig->StatsTableBase);

    err = ix_rm_get_phys_offset((void*)pContext->DefaultStatAddress,
                                &memType, &pContext->StatsTableChannelNum,
                                &pContext->pUblockConfig->DefaultStatAddr,
                                &PhysAdd);
    if (err != IX_SUCCESS)
    {
        err = IX_ERROR_WARNING(IX_ERROR_GET_CODE(err), ("ix_cc_classifier_6t_init(): Error ! Could not convert virtual address of Statistics Table to offset understood by uengine."));
        goto err_def_stat_get_offset;
    }

     /* set channel number into offset */
     IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(pContext->StatsTableChannelNum,
                                        pContext->pUblockConfig->DefaultStatAddr);

#endif /* #if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE) */


#if !defined(IX_CC_SIMULATION)
   /* cleans own memory for errors */
    err = _ix_cc_classifier_6t_patch_microblock(pContext);
    if (err != IX_SUCCESS)
    {
    err = IX_ERROR_WARNING(IX_ERROR_GET_CODE(err), ("ix_cc_classifier_6t_init(): Error in patch microblock"));
    goto err_patch;
    }
#endif /* #if !defined(IX_CC_SIMULATION) */

    /* register packet and message handler */
#if defined(IX_EXCLUDE_CCI)
#define _REGISTRATION_ARGUMENT pContext
#else
#define _REGISTRATION_ARGUMENT arg_cc_handle
#endif
    err = _ix_cc_classifier_6t_reg_pkt_msg_handlers(_REGISTRATION_ARGUMENT);

#undef _REGISTRATION_ARGUMENT

    if (err != IX_SUCCESS)
    {
        err = IX_ERROR_WARNING(IX_ERROR_GET_CODE(err), ("ix_cc_classifier_6t_init(): Error during registration of packet and message handlers"));
        goto err_reg_mess;
    }

    /* Get mutex from OSSL */
    err = ix_ossl_mutex_init(IX_OSSL_MUTEX_UNLOCK, &(pContext->API_mutex));
    if(err != IX_SUCCESS)
    {
        err = IX_ERROR_WARNING(IX_ERROR_GET_CODE(err), ("ix_cc_classifier_6t_init(): Failed to create mutext"));
    goto err_mutex;
    }

    *arg_ppContext = pContext;
    return IX_SUCCESS;

err_mutex:
    _ix_cc_classifier_6t_dereg_pkt_msg_handlers();

err_reg_mess:

#if !defined(IX_CC_SIMULATION)
err_patch:
#endif

#if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)
err_def_stat_get_offset:
    ix_cc_classifier_6t_destroy_default_stat(&pContext->DefaultStatAddress);

err_stats_get_offset:
    ix_cc_classifier_6t_destroy_stats_table(&pContext->StatsTableAddress);

err_cr_stats:
#endif /* #if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE) */

err_get_offset:
    _ix_cc_classifier_6t_destroy_hash_table(&pContext->HashTableAddress);

err_cr_hash:

err_get_config:
    ix_ossl_free(pContext->pUblockConfig);

err_ublockconfig_mem:
    ix_ossl_free(pContext);

    *arg_ppContext = NULL;
    return err;
}

/*
 * FUNCTION: ix_cc_classifier_6t_fini()
 */
ix_error ix_cc_classifier_6t_fini(ix_cc_handle arg_cc_handle, void *arg_pContext)
{
    /* check the arguments */
    ix_error new_err;
    ix_error err = IX_SUCCESS;

    _ix_cc_classifier_6t_context *pContext = (_ix_cc_classifier_6t_context*)arg_pContext;

    if (NULL == pContext)
    return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                 ("ix_cc_classifier_6t_fini(): Error ! NULL 6-tuple classifier Core Context"));

    if (arg_cc_handle != (pContext->hClassifier6tHandle))
    return IX_ERROR_WARNING(IX_CC_CLASSIFIER_6T_ERROR_INVALID_CC_HANDLE,
                 ("ix_cc_classifier_6t_fini(): Error ! NULL handle of 6-tuple classifier"));

    /* remove packet handler */
    new_err = _ix_cc_classifier_6t_dereg_pkt_msg_handlers();
    if (err == IX_SUCCESS) err = new_err;

#if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)
    /* destroy Default Statistics Entry */
    new_err = ix_cc_classifier_6t_destroy_default_stat(&pContext->DefaultStatAddress);
    if (err == IX_SUCCESS) err = new_err;
    /* destroy Statistics Table */
    new_err = ix_cc_classifier_6t_destroy_stats_table(&pContext->StatsTableAddress);
    if (err == IX_SUCCESS) err = new_err;
#endif /* #if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE) */

    /* destroy hash table */
    new_err = _ix_cc_classifier_6t_destroy_hash_table(&pContext->HashTableAddress);
    if (err == IX_SUCCESS) err = new_err;


    /* Free mutex */
    /* new_err = IX_ERROR_CR(ix_ossl_mutex_fini(pContext->API_mutex)); */
    if (err == IX_SUCCESS) err = new_err;

    /* ossl memory deallocation */
    ix_ossl_free(pContext->pUblockConfig);
    ix_ossl_free(pContext);

    return err;
}

/*
 * FUNCTION: ix_cc_classifier_6t_msg_handler()
 */
ix_error
ix_cc_classifier_6t_msg_handler(ix_buffer_handle arg_Msg,
                                ix_uint32        arg_MsgId,
                                void            *arg_pContext)
{
    ix_error                         err      = IX_SUCCESS;   /* ix_uint32 */
    void                            *pMsgData = NULL;
    void                            *pContext = NULL;
    void                            *pMsgReply = NULL;
    ix_uint32                        msgReplyLen = 0;
    /* used to handle search message */
    ix_cc_classifier_6t_result       LookupResult;
    
#if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)
    /* used to handle search message */
    ix_cc_classifier_6t_statistics   StatisticsResult;
#endif


    #ifdef IX_CLASSIFIER_6T_DEBUG
    ix_ossl_message_log("ix_cc_classifier_6t_msg_handler(): ha, ha got something   :)))))) \n");
    ix_ossl_message_log("MsgId : %d \n", arg_MsgId);
    #endif

    /* Check the input arguments */
    if (NULL == arg_pContext)
    return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                 ("ix_cc_classifier_6t_msg_handler(): Error"));
    if (IX_NULL_BUFFER_HANDLE == arg_Msg)
    return IX_ERROR_WARNING(IX_CC_CLASSIFIER_6T_ERROR_INVALID_BUF_HANDLE,
                 ("ix_cc_classifier_6t_msg_handler(): Error"));

    IX_MSUP_EXTRACT_MSG(arg_Msg, &pMsgData, &pContext);
    if ( pMsgData == NULL )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_NULL,
            ( "ix_cc_classifier_6t_msg_handler(): invalid buffer" )
          );
      }

    switch(arg_MsgId) {
        case IX_CC_CLASSIFIER_6T_MSG_ADD_ENTRY:
        {
            ix_cc_classifier_6t_message_general_data *pMsgBody =
                (ix_cc_classifier_6t_message_general_data*) pMsgData;

            err = ix_cc_classifier_6t_add_entry(&pMsgBody->Key,
                                                &pMsgBody->LookupResult,
                                                arg_pContext);
            break;
        }
        case IX_CC_CLASSIFIER_6T_MSG_REMOVE_ENTRY:
        {
            ix_cc_classifier_6t_message_remove_data *pMsgBody =
                (ix_cc_classifier_6t_message_remove_data*) pMsgData;

            err = ix_cc_classifier_6t_remove_entry(&pMsgBody->Key,
                                                   pMsgBody->Type,
                                                   arg_pContext);
            break;
        }
        case IX_CC_CLASSIFIER_6T_MSG_UPDATE_ENTRY:
        {
            ix_cc_classifier_6t_message_general_data *pMsgBody =
                (ix_cc_classifier_6t_message_general_data*) pMsgData;

            err = ix_cc_classifier_6t_update_entry(&pMsgBody->Key,
                                                   &pMsgBody->LookupResult,
                                                   arg_pContext);
            break;
        }
        case IX_CC_CLASSIFIER_6T_MSG_SEARCH_ENTRY:
        {
            ix_cc_classifier_6t_message_search_data  *pMsgBody =
                (ix_cc_classifier_6t_message_search_data*) pMsgData;

            LookupResult.type = pMsgBody->LookupType;

            err = ix_cc_classifier_6t_search_entry(&pMsgBody->Key,
                                                   &LookupResult,
                                                   arg_pContext);
            pMsgReply = &LookupResult;
            msgReplyLen = sizeof(ix_cc_classifier_6t_result);
            break;
        }
#if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)
        case IX_CC_CLASSIFIER_6T_MSG_GET_STATISTICS:
        {
            ix_cc_classifier_6t_message_stats_data *pMsgBody =
                (ix_cc_classifier_6t_message_stats_data*) pMsgData;

            err = ix_cc_classifier_6t_get_statistics(&pMsgBody->Key,
                                                     &StatisticsResult,
                                                     arg_pContext);
            pMsgReply = &StatisticsResult;
            msgReplyLen = sizeof(StatisticsResult);
            break;
        }
        case IX_CC_CLASSIFIER_6T_MSG_GET_STATISTICS_DEF:
        {
            err = ix_cc_classifier_6t_get_statistics_def((ix_cc_classifier_6t_statistics*) pMsgData,
                                                         arg_pContext);
            pMsgReply = pMsgData;
            msgReplyLen = sizeof(ix_cc_classifier_6t_statistics);
            break;
        }
#endif /* #if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE) */
        default:
            err = IX_ERROR_WARNING(IX_CC_ERROR_UNDEFINED_MSG,
                 ("ix_cc_classifier_6t_msg_handler(): Undefined message."));
    }

    /* send return message, length and retrieved context to message support library */
    err = ix_cc_msup_send_reply_msg(pContext, pMsgReply, msgReplyLen, err);
    if(err != IX_SUCCESS)
    {
    err = IX_ERROR_WARNING(IX_CC_ERROR_SEND_FAIL,
             ("ix_cc_classifier_6t_msg_handler(): Send reply message failed"));
    }

    ix_ossl_free(pMsgData);

    if (IX_SUCCESS != ix_rm_buffer_free_chain(arg_Msg))
            err = IX_ERROR_WARNING(IX_CC_CLASSIFIER_6T_ERROR_BUFF_FREE,
                        ("ix_cc_classifier_6t_msg_handler(): Error during releasing buffers"));

    return err;
}

/*
 * FUNCTION: ix_cc_classifier_6t_pkt_handler()
 */
ix_error ix_cc_classifier_6t_pkt_handler(ix_buffer_handle arg_Msg,
                                         ix_uint32        arg_ExceptionCode,
                                         void            *arg_pContext)
{
    _ix_cc_classifier_6t_context    *pContext = (_ix_cc_classifier_6t_context*)arg_pContext;
    _ix_cc_classifier_6t_hash_entry *PreviousHashEntry;
    _ix_cc_classifier_6t_hash_entry *HashEntry;
    ix_cc_classifier_6t_header       IpHdr;
    ix_cc_classifier_6t_pattern      Key;
    ix_cc_classifier_6t_result       LookupResult;
    ix_hw_buffer_meta               *pMetaData;
    ix_uint16                        OutputId;
    ix_uint16                        OutputBid;
    void                            *pData;

    ix_uint16        index          = 0;
    ix_uint32        HashIndex;
    ix_error         err            = IX_SUCCESS;
    ix_uint16        len;
    ix_uint16        ingressPort;

#if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)
    ix_cc_classifier_6t_statistics   *pStats;
    ix_uint32                         pktCountValue;
    ix_uint32                         byteCountValue;
    ix_uint8                          DefaultRuleApplied = 1;
#endif /* #if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE) */

    #ifdef IX_CLASSIFIER_6T_DEBUG
	printf("I've got packet.\n");
    #endif

    /* Check the input arguments */
    if (NULL == arg_pContext)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                         ("ix_cc_classifier_6t_pkt_handler(): Error ! got null context of 6t classifier CC."));

    if (NULL == pContext->HashTableAddress)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                         ("ix_cc_classifier_6t_pkt_handler(): Error ! Hash table has not been created."));

#if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)
    if (NULL == pContext->StatsTableAddress)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                         ("ix_cc_classifier_6t_pkt_handler(): Error ! Statistics Table has not been created."));

    if (NULL == pContext->DefaultStatAddress)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                         ("ix_cc_classifier_6t_pkt_handler(): Error ! Default Statistics Entry has not been created."));
#endif /* #if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE) */

    if (IX_NULL_BUFFER_HANDLE == arg_Msg)
        return IX_ERROR_WARNING(IX_CC_CLASSIFIER_6T_ERROR_INVALID_BUF_HANDLE,
                         ("ix_cc_classifier_6t_pkt_handler(): Error ! Got null buffer handle."));

    err = ix_cc_hw_get_packet_data(arg_Msg, &pData);

    /* Attempt to free the buffer if get data fails  */
    if (err) {
        IX_ERROR_CRT(ix_rm_buffer_free_chain(arg_Msg),
        IX_CC_ERROR_INTERNAL, 0);
        return IX_ERROR_CHAIN(err, IX_CC_ERROR_INTERNAL, 0,
                         ("ix_cc_classifier_6t_pkt_handler(): Failed to get data\n"));
    }


    /* get input interface number */
    err = ix_rm_buffer_get_meta(arg_Msg, (void **)&pMetaData);
    if (err) {
        IX_ERROR_CRT(ix_rm_buffer_free_chain(arg_Msg),
        IX_CC_ERROR_INTERNAL, 0);
        return IX_ERROR_CHAIN(err, IX_CC_ERROR_INTERNAL,
                                   0, ("ix_cc_classifier_6t_pkt_handler(): Failed to get meta\n"));
    }
    ingressPort = (ix_uint16)(IX_RM_MEM_UINT16_READ(&pMetaData->m_InputPort));

    #ifdef IX_CLASSIFIER_6T_DEBUG
    printf("It looks like it comes from port %d \n",ingressPort);
    #endif


    len = (*((ix_uint8 *)pData));
    len = (ix_uint16)((len & 0x0f) << 2);

    #ifdef IX_CLASSIFIER_6T_DEBUG
    printf("and is %d bytes long. \n", len);
    #endif

    /* extract IP header */
    ix_ossl_memcpy(&IpHdr, pData, len + 4);

    #ifdef IX_CLASSIFIER_6T_DEBUG
        show_mem(&IpHdr, 32);
        printf("Whoh It has head ! \n",ingressPort);
        printf("frag_offset ! \n",IpHdr.fragOffset);
    #endif

    Key.ipv4SourceAddress      = IpHdr.sourceAddress;

    #ifdef IX_CLASSIFIER_6T_DEBUG
    printf("from : 0x%x \n", Key.ipv4SourceAddress);
    #endif

    Key.ipv4DestinationAddress = IpHdr.destAddress;
    #ifdef IX_CLASSIFIER_6T_DEBUG
    printf("to : 0x%x \n", Key.ipv4DestinationAddress);
    #endif

    Key.inputPort              = ingressPort;
    #ifdef IX_CLASSIFIER_6T_DEBUG
    printf("in port : 0x%x \n", Key.inputPort);
    #endif

    Key.dscp                   = ((IpHdr.tos & 0xfc) >> 2);
    #ifdef IX_CLASSIFIER_6T_DEBUG
    printf("TOS  : 0x%x \n", IpHdr.tos);
    printf("dscp : 0x%x \n", Key.dscp);
    #endif

    Key.protocol               = IpHdr.protocol;
    #ifdef IX_CLASSIFIER_6T_DEBUG
    printf("prot : 0x%x \n", Key.protocol);
    #endif

    Key.l4DestinationPort      = *(ix_uint16*)(&IpHdr.options[len-18]);
    #ifdef IX_CLASSIFIER_6T_DEBUG
    printf("dst port : 0x%x \n", Key.l4DestinationPort);
    #endif

    Key.l4SourcePort           = *(ix_uint16*)(&IpHdr.options[len-20]);
    #ifdef IX_CLASSIFIER_6T_DEBUG
    printf("src port : 0x%x \n", Key.l4SourcePort);
    #endif

    /*   START CLASSIFICATION ALGORITHM */

    /*  The classification algorithm has to find flow and class id for the packet. First, it constructs
     *  a 128-bit long hash input by concatenating bits of six fields extracted from the processed
     *  IP header (only 6 bits of TOS field are taken). Next, it issues a request to the hardware Hash
     *  Unit and waits for the operation to complete.  When the hash result is ready, a classifier takes
     *  only HASH_KEY_MASK most significant bits (default is 16) from the first returned long word.
     *  These bits are shifted right so they form a DRAM/SRAM offset to a hash table entry.
     *  After the hash entry is retrieved, algorithm compares IP header fields with values stored
     *  in the hash entry. On successful verification, a thread checks if a packet was received from
     *  the allowed input port.
     *  According to DiffServ MIB assumptions, the input port can be specified as either exact number
     *  or a wildcard value (0xFFFF). If a matching entry is found, the classifier writes selected dispatch
     *  loop variables with data stored in a hash entry. Otherwise (if verification fails), the algorithm
     *  loads the next hash entry in a chain, as indicated by next_entry_ptr, and repeats the entry
     *  verification procedure. If a classifier reaches end of chain (next_entry_ptr set to null)
     *  without finding a matching entry, a default rule is applied.
     *  The algorithm supports exact-match classification on an IP header, as well as wildcard classification
     *  on an input port. Such combination accommodates usage scenarios where only a packet header is known,
     *  but not the input port (e.g. IP tunnel endpoints). It also works well with DiffServ,
     *  unless a given 6-tuple rule is configured on selected subset of external ports.
     *  However, such configurations are expected to be very rare: a rule belongs either to a specific
     *  input port or to all interfaces.  */

    ix_ossl_mutex_lock(pContext->API_mutex, IX_OSSL_WAIT_FOREVER);

    HashIndex = _ix_cc_classifier_6t_get_hash_key(&Key, pContext->pUblockConfig->HashIndexSize);

    HashEntry = pContext->HashTableAddress + HashIndex;

    #ifdef IX_CLASSIFIER_6T_DEBUG
    printf("It's high time to look up into hash table ! \n");
    printf("Hash index : 0x%x\n",HashIndex);
    #endif

    if (_ix_cc_classifier_6t_isEmptyElement(HashEntry)) {
        HashEntry = NULL;
        #ifdef IX_CLASSIFIER_6T_DEBUG
            printf("Entry is empty \n");
        #endif
    } else {
        while (_ix_cc_classifier_6t_match_entry_pattern(&Key, HashEntry) == _IX_CC_DIFFERENT)
            if ((void*)HashEntry->NextHashEntry != NULL) {
                PreviousHashEntry = HashEntry;
                HashEntry = pContext->HashTableAddress +
                    ((HashEntry->NextHashEntry - pContext->pUblockConfig->HashTableBase) / sizeof(_ix_cc_classifier_6t_hash_entry));
                #if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)
                HashIndex = HashEntry - pContext->HashTableAddress;
                #endif
            } else {
                HashEntry = NULL;
                break;
			}
	}

	if (HashEntry == NULL) {
        #ifdef IX_CLASSIFIER_6T_DEBUG
            printf("default rule\n");
        #endif
		pMetaData->m_ClassID       = pContext->pUblockConfig->DefaultClassID;
		pMetaData->m_ColorID       = pContext->pUblockConfig->DefaultColorID;
		pMetaData->m_FlowID        = pContext->pUblockConfig->DefaultFlowID;
		pMetaData->m_NextHopID     = pContext->pUblockConfig->DefaultNextHopID;
		pMetaData->m_NextHopIDType = pContext->pUblockConfig->DefaultNextHopIDType;
        OutputId  = IX_CC_CLASSIFIER_6T_DEFAULT_OUTPUT;
        OutputBid = IX_CC_CLASSIFIER_6T_DEFAULT_BID;

	} else {
		LookupResult.type = IX_CC_CLASSIFIER_6T_QOS;
		if (_ix_cc_classifier_6t_get_hash_entry(&LookupResult, HashEntry) != IX_SUCCESS) {
            #ifdef IX_CLASSIFIER_6T_DEBUG
                printf("QOS rule not found in hash table\n");
            #endif
			pMetaData->m_ClassID       = pContext->pUblockConfig->DefaultClassID;
			pMetaData->m_ColorID       = pContext->pUblockConfig->DefaultColorID;
			pMetaData->m_FlowID        = pContext->pUblockConfig->DefaultFlowID;

            OutputId  = IX_CC_CLASSIFIER_6T_DEFAULT_OUTPUT;
            OutputBid = IX_CC_CLASSIFIER_6T_DEFAULT_BID;
        } else {

            #if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)
            DefaultRuleApplied = 0;
            #endif
            
            #ifdef IX_CLASSIFIER_6T_DEBUG
            printf("rule from hash table\n");
            printf("class_id %d\n", LookupResult.result.qos.classId);
            printf("color_id %d\n", LookupResult.result.qos.colorId);
            printf("flow_id %d\n", LookupResult.result.qos.flowId);
            #endif

			pMetaData->m_ClassID = LookupResult.result.qos.classId;
			pMetaData->m_ColorID = LookupResult.result.qos.colorId;
			pMetaData->m_FlowID  = LookupResult.result.qos.flowId;

            switch (LookupResult.result.qos.OutputId) {
               case IX_CC_CLASSIFIER_6T_METER_BID:
                               OutputId  = IX_CC_CLASSIFIER_6T_METER_OUTPUT;
                               OutputBid = IX_CC_CLASSIFIER_6T_METER_BID;
                               break;
               case IX_CC_CLASSIFIER_6T_MARKER_BID:
                   {
                               ix_uint32   new_checksum;
                               ix_uint32   new_tos = (IpHdr.tos & 3) | (((ix_uint8)LookupResult.result.qos.flowId << 2) & 0xfc);
                               ix_uint32   old_tos = ~(*(ix_uint16*)(&IpHdr)) & 0xFFFF;
                               new_tos = new_tos | (~old_tos & 0xFF00);

                               #ifdef IX_CLASSIFIER_6T_DEBUG
                               printf("new_tos : %x\n", new_tos);
                               printf("old_tos : %x\n", old_tos);
                               printf("checksum: %x\n", IpHdr.checksum);
                               #endif

                               new_checksum = ((ix_uint32)IpHdr.checksum - old_tos - new_tos);
                               
                               while (new_checksum > 0xFFFF)
                                   new_checksum = (ix_int32)(new_checksum & 0xFFFF) + ((ix_int16)(new_checksum >> 16));
                               
                               IpHdr.checksum = (ix_uint16)(new_checksum);
                               IpHdr.tos      = (ix_uint8)(new_tos);

                               #ifdef IX_CLASSIFIER_6T_DEBUG
                               printf("new checksum: %x\n", IpHdr.checksum);
                               #endif

                               OutputId  = IX_CC_CLASSIFIER_6T_MARKER_OUTPUT;
                               OutputBid = IX_CC_CLASSIFIER_6T_MARKER_BID;
                               break;
                   }
               case IX_CC_CLASSIFIER_6T_FORWARDER_BID:
                               OutputId  = IX_CC_CLASSIFIER_6T_FORWARDER_OUTPUT;
                               OutputBid = IX_CC_CLASSIFIER_6T_FORWARDER_BID;
                               break;
               default:
                               OutputId  = IX_CC_CLASSIFIER_6T_DEFAULT_OUTPUT;
                               OutputBid = IX_CC_CLASSIFIER_6T_DEFAULT_BID;
            }
        /* LookupResult.result.qos.nextBlockId; */
		}
		LookupResult.type = IX_CC_CLASSIFIER_6T_FWD;
		if (_ix_cc_classifier_6t_get_hash_entry(&LookupResult, HashEntry) != IX_SUCCESS) {
			pMetaData->m_NextHopID     = pContext->pUblockConfig->DefaultNextHopID;
			pMetaData->m_NextHopIDType = pContext->pUblockConfig->DefaultNextHopIDType;

        } else {
			pMetaData->m_NextHopIDType = LookupResult.result.fwd.NextHopIdType;
			pMetaData->m_NextHopID     = LookupResult.result.fwd.NextHopId;
            #if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)
            DefaultRuleApplied = 0;
            #endif
		}
	}

    /* ----------------------------------------------------------*/
	#if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)
    if (DefaultRuleApplied) {
		if (pContext->DefaultStatEnable) {
			pktCountValue = ix_rm_atomic_sram_test_and_add(1, (ix_uint32*)pContext->DefaultStatAddress + 1);
			if ((++pktCountValue) != 0)
		 	  ix_rm_atomic_sram_add(1, (ix_uint32*)pContext->DefaultStatAddress);
			byteCountValue = ix_rm_atomic_sram_test_and_add(pMetaData->m_PacketSize, (ix_uint32*)pContext->DefaultStatAddress + 3);
			if ((++byteCountValue) != 0)
		 	  ix_rm_atomic_sram_add(1, (ix_uint32*)pContext->DefaultStatAddress + 2);
		}
    }  else  {
		if (LookupResult.stats_enable) {
			pStats = pContext->StatsTableAddress + HashIndex;
			pktCountValue = ix_rm_atomic_sram_test_and_add(1, (ix_uint32*)pStats + 1);
			if ((++pktCountValue) != 0)
		 	  ix_rm_atomic_sram_add(1, (ix_uint32*)pStats);
			byteCountValue = ix_rm_atomic_sram_test_and_add(pMetaData->m_PacketSize, (ix_uint32*)pStats + 3);
			if ((++byteCountValue) != 0)
			   ix_rm_atomic_sram_add(1, (ix_uint32*)pStats + 2);
		}
    }
	#endif 
    /* ----------------------------------------------------------*/

    /* save IP header */
    ix_ossl_memcpy(pData, &IpHdr, 12);

    #ifdef IX_CLASSIFIER_6T_DEBUG
        printf(" ------- Changed IP HEADER -----------\n");
        show_mem(pData, 32);
        printf(" ------------------\n");
    #endif

	ix_ossl_mutex_unlock(pContext->API_mutex);

    #ifdef IX_CLASSIFIER_6T_DEBUG
    printf("ix_cc_classifier_6t_pkt_handler(): Sending further ... to %d ! \n",OutputId);
    #endif

    err = ix_rm_packet_send(OutputId, arg_Msg, OutputBid);

    if (err) {
        /* Free the buffer and return error to caller */
        IX_ERROR_CRT(ix_rm_buffer_free(arg_Msg), IX_CC_ERROR_INTERNAL, 0);
        return IX_ERROR_CHAIN(err,IX_CC_ERROR_SEND_FAIL,0,
		        ("ix_cc_classifier_6t_pkt_handler():  Failed to send packet to next target\n"));
    }

    return err;
}



#endif

