/*
 * ============================================================================
 * COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2003 Intel Corporation. All rights reserved.
 *
 * PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *                                                                           */
/** \file   ix_cc_classifier_6t_api.c
 *
 *  \brief  The file contains API functions exposed by 6 tuple classifier
 *          Core Component
 *
 */

#include "ix_error.h"

#include "ix_ossl.h"
#include "ix_cc.h"
#include "ix_cci.h"

#include "ix_cc_error.h"

#include "bindings.h"
#include "../../include/classifier_6t_defs.h"

#include "cc/ix_cc_classifier_6t.h"
#include "cc/ix_cc_classifier_6t_hash_comp.h"
#include "cc/ix_cc_classifier_6t_stats.h"



_ix_cc_classifier_6t_hash_entry* getNextHashEntryAddress(_ix_cc_classifier_6t_context    *pContext,
                                                         _ix_cc_classifier_6t_hash_entry *HashEntry) 
{
  return pContext->HashTableAddress
           + ((HashEntry->NextHashEntry - pContext->pUblockConfig->HashTableBase) 
              / sizeof(_ix_cc_classifier_6t_hash_entry));
}



/*
 * FUNCTION: ix_cc_classifier_6t_add_entry()
 */
ix_error ix_cc_classifier_6t_add_entry(ix_cc_classifier_6t_pattern *arg_pKey,
                                       ix_cc_classifier_6t_result  *arg_pLookupResult,
                                       void *arg_pContext)
{
    ix_error  err   = IX_SUCCESS;
    ix_uint16 index = 0;
    ix_uint32 HashIndex;

    _ix_cc_classifier_6t_context    *pContext  = (_ix_cc_classifier_6t_context*)arg_pContext;
    _ix_cc_classifier_6t_hash_entry *NextHashEntry;
    _ix_cc_classifier_6t_hash_entry *HashEntry;
    _ix_cc_classifier_6t_hash_entry *SubsidiaryHashTableBegin;
    _ix_cc_classifier_6t_hash_entry *SubsidiaryHashTableEnd;

#if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)
    ix_cc_classifier_6t_statistics  *StatsEntry;
#endif /* #if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE) */

    if (NULL == pContext)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
             ("ix_cc_classifier_6t_add_entry(): Error ! classifier has not been created."));

    if (NULL == pContext->HashTableAddress)
	return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
             ("ix_cc_classifier_6t_add_entry(): Error ! Hash table has not been created."));

    if (NULL == arg_pKey)
	return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
             ("ix_cc_classifier_6t_add_entry(): Error ! arg_pKey is NULL."));

    if (NULL == arg_pLookupResult)
 	return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
             ("ix_cc_classifier_6t_add_entry(): Error ! arg_pLookupResult is NULL."));

    if ((IX_CC_CLASSIFIER_6T_QOS != arg_pLookupResult->type) &&
	(IX_CC_CLASSIFIER_6T_FWD != arg_pLookupResult->type))
    {
	return IX_ERROR_WARNING(IX_CC_CLASSIFIER_6T_ERROR_PARAM,
             ("ix_cc_classifier_6t_add_entry(): Error ! Table type unknown."));
    }

    if ((IX_CC_CLASSIFIER_6T_QOS == arg_pLookupResult->type) &&
	(_IX_CC_CLASSIFIER_6T_EMPTY_QOS_ENTRY == arg_pLookupResult->result.qos.flowId))
    {
	return IX_ERROR_WARNING(IX_CC_CLASSIFIER_6T_ERROR_PARAM,
            ("ix_cc_classifier_6t_add_entry(): Error ! Flow ID invalid value."));
    }

    if ((IX_CC_CLASSIFIER_6T_FWD == arg_pLookupResult->type) &&
	(_IX_CC_CLASSIFIER_6T_EMPTY_FWD_ENTRY == arg_pLookupResult->result.fwd.NextHopId))
    {
	return IX_ERROR_WARNING(IX_CC_CLASSIFIER_6T_ERROR_PARAM,
        ("ix_cc_classifier_6t_add_entry(): Error ! Next Hop ID invalid value."));
    }

    if ((IX_CC_CLASSIFIER_6T_QOS == arg_pLookupResult->type) &&
	(IX_CC_CLASSIFIER_6T_METER_BID != arg_pLookupResult->result.qos.OutputId) &&
    (IX_CC_CLASSIFIER_6T_MARKER_BID != arg_pLookupResult->result.qos.OutputId) &&
    (IX_CC_CLASSIFIER_6T_FORWARDER_BID != arg_pLookupResult->result.qos.OutputId))
    {
	return IX_ERROR_WARNING(IX_CC_CLASSIFIER_6T_ERROR_PARAM,
            ("ix_cc_classifier_6t_add_entry(): Error ! Not known Output block ID."));
    }


#if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)

    if (NULL == pContext->StatsTableAddress)
	return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
             ("ix_cc_classifier_6t_add_entry(): Error ! Statistics Table has not been created."));

#endif /* #if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE) */


    ix_ossl_mutex_lock(pContext->API_mutex, IX_OSSL_WAIT_FOREVER);

    HashIndex = _ix_cc_classifier_6t_get_hash_key(arg_pKey, pContext->pUblockConfig->HashIndexSize);

    #if IX_CLASSIFIER_6T_DEBUG
    ix_ossl_message_log("HashIndex=0x%x\n",HashIndex);
    #endif

    HashEntry = pContext->HashTableAddress + HashIndex;

    SubsidiaryHashTableBegin = pContext->HashTableAddress +
	                        (1 << pContext->pUblockConfig->HashIndexSize);
	
    SubsidiaryHashTableEnd   = SubsidiaryHashTableBegin + 
		                         pContext->AuxEntryNo;
		         

    if (_ix_cc_classifier_6t_isEmptyElement(HashEntry)) {

        _ix_cc_classifier_6t_set_hash_entry_pattern(arg_pKey, HashEntry);
        (void*)HashEntry->NextHashEntry = NULL_NEXT_ENTRY;

    } else
    	while (_ix_cc_classifier_6t_match_entry_pattern(arg_pKey, HashEntry) == _IX_CC_DIFFERENT) {
	        if ((void*)HashEntry->NextHashEntry != NULL_NEXT_ENTRY) {
		        HashEntry = getNextHashEntryAddress(pContext, HashEntry);
		    } else {				
                
		        NextHashEntry = _ix_cc_classifier_6t_find_free_element(
		                                  SubsidiaryHashTableBegin,
		                                  SubsidiaryHashTableEnd);
		        
                if (NextHashEntry == NULL_NEXT_ENTRY) {
		            ix_ossl_mutex_unlock(pContext->API_mutex);
    		        return IX_ERROR_WARNING(IX_CC_ERROR_FULL,
	    	            ("ix_cc_classifier_6t_add_entry(): Error ! Hash table too small to complete operation."));
    	    	}
                HashEntry->NextHashEntry = pContext->pUblockConfig->HashTableBase + 
                                           (NextHashEntry - pContext->HashTableAddress) *
                                           sizeof(_ix_cc_classifier_6t_hash_entry);

                HashEntry = NextHashEntry;
		        HashEntry->NextHashEntry = NULL_NEXT_ENTRY;
		        _ix_cc_classifier_6t_set_hash_entry_pattern(arg_pKey, HashEntry);

		        break;
	        }
	}

#if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)
     #ifdef IX_CLASSIFIER_6T_DEBUG
     printf("ix_cc_classifier_6t_add_entry(): pContext->HashTableAddress : 0x%x\n",pContext->HashTableAddress);
     #endif
     HashIndex = (ix_uint32)(HashEntry - pContext->HashTableAddress);
     #ifdef IX_CLASSIFIER_6T_DEBUG
     printf("ix_cc_classifier_6t_add_entry(): new Hash index : 0x%x\n",HashIndex);
     printf("ix_cc_classifier_6t_add_entry(): pContext->StatsTableAddress : 0x%x\n",pContext->StatsTableAddress );
     #endif

     StatsEntry = pContext->StatsTableAddress + HashIndex;
     StatsEntry->packets = 0;
     StatsEntry->bytes = 0;
     #ifdef IX_CLASSIFIER_6T_DEBUG
     printf("ix_cc_classifier_6t_add_entry(): stats addr for this entry : 0x%x\n",StatsEntry);
     #endif
#endif /* #if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE) */

    err = _ix_cc_classifier_6t_set_hash_entry_lookup_result(arg_pLookupResult, HashEntry);

    #ifdef IX_CLASSIFIER_6T_DEBUG
    printf("ix_cc_classifier_6t_add_entry(): new entry: \n");
    show_mem(HashEntry, sizeof(_ix_cc_classifier_6t_hash_entry));
    printf("ix_cc_classifier_6t_add_entry(): set based on user data: \n");
    printf("Lookup result \n");
    show_mem(arg_pLookupResult, sizeof(ix_cc_classifier_6t_result));
    printf("Pattern \n");
    show_mem(arg_pKey, sizeof(ix_cc_classifier_6t_pattern));
    #endif

    ix_ossl_mutex_unlock(pContext->API_mutex);

    return err;
}


/*
 * FUNCTION: ix_error ix_cc_classifier_6t_remove_entry()
 */
ix_error ix_cc_classifier_6t_remove_entry(ix_cc_classifier_6t_pattern    *arg_pKey,
                                          ix_cc_classifier_6t_table_type  arg_Type,
										  void *arg_pContext)
{
    ix_error		 err        = IX_SUCCESS;
    ix_uint16        index      = 0;
    ix_uint32        HashIndex;

    _ix_cc_classifier_6t_context    *pContext  = (_ix_cc_classifier_6t_context*)arg_pContext;
    _ix_cc_classifier_6t_hash_entry *PreviousHashEntry = NULL;
    _ix_cc_classifier_6t_hash_entry *NextHashEntry;
    _ix_cc_classifier_6t_hash_entry *HashEntry;

    if (NULL == pContext)
	return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                 ("ix_cc_classifier_6t_remove_entry(): Error ! classifier has not been created."));

    if (NULL == pContext->HashTableAddress)
	return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                 ("ix_cc_classifier_6t_remove_entry(): Error ! Hash table has not been created."));

    if (NULL == arg_pKey)
	return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                 ("ix_cc_classifier_6t_remove_entry(): Error ! arg_pKey is NULL."));

    if ((IX_CC_CLASSIFIER_6T_QOS != arg_Type) &&
	(IX_CC_CLASSIFIER_6T_FWD != arg_Type))
    {
	return IX_ERROR_WARNING(IX_CC_CLASSIFIER_6T_ERROR_PARAM,
             ("ix_cc_classifier_6t_remove_entry(): Error ! Table type unknown."));
    }

    ix_ossl_mutex_lock(pContext->API_mutex, IX_OSSL_WAIT_FOREVER);

    HashIndex = _ix_cc_classifier_6t_get_hash_key(arg_pKey, pContext->pUblockConfig->HashIndexSize);

    HashEntry = pContext->HashTableAddress + HashIndex;

    #ifdef IX_CLASSIFIER_6T_DEBUG 
    printf(" Hash index = 0x%x    HashEntry =0x%x \n", HashIndex, HashEntry);
    #endif
	
    if (_ix_cc_classifier_6t_isEmptyElement(HashEntry)) {
        #ifdef IX_CLASSIFIER_6T_DEBUG 
            printf("ix_cc_classifier_6t_remove_entry(): Error ! None entry exists. \n");
        #endif
        ix_ossl_mutex_unlock(pContext->API_mutex);
		return IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND,
		       ("ix_cc_classifier_6t_remove_entry(): Error ! Given entry does not exist."));
	} else {
		while (_ix_cc_classifier_6t_match_entry_pattern(arg_pKey, HashEntry) == _IX_CC_DIFFERENT) {
			if ((void*)HashEntry->NextHashEntry != NULL_NEXT_ENTRY) {
				PreviousHashEntry = HashEntry;
				HashEntry = getNextHashEntryAddress(pContext, HashEntry);
			} else {
                                #ifdef IX_CLASSIFIER_6T_DEBUG 
				printf("ix_cc_classifier_6t_remove_entry(): Error ! Given entry does not exist. \n");
                                #endif
                                ix_ossl_mutex_unlock(pContext->API_mutex);
				return IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND,
				       ("ix_cc_classifier_6t_remove_entry(): Error ! Given entry does not exist."));
			}
		}

	}

	switch(arg_Type) {
		case IX_CC_CLASSIFIER_6T_QOS:
                        #ifdef IX_CLASSIFIER_6T_DEBUG 
      			printf(" removing QOS \n");
                        #endif
			if (_ix_cc_classifier_6t_isEmptyEntry(IX_CC_CLASSIFIER_6T_FWD, HashEntry) == _IX_CC_CLASSIFIER_6T_ENTRY_EMPTY) {

                if (PreviousHashEntry != NULL)
                    PreviousHashEntry->NextHashEntry = HashEntry->NextHashEntry;
                else 

                    if (HashEntry->NextHashEntry != NULL_NEXT_ENTRY) {					
                        NextHashEntry = getNextHashEntryAddress(pContext, HashEntry);
					    *HashEntry = *NextHashEntry;
                        _ix_cc_classifier_6t_zero_out_hash_entry(NextHashEntry);
                        break;
                    }
                
                _ix_cc_classifier_6t_zero_out_hash_entry_pattern(HashEntry);
    		}
            err = _ix_cc_classifier_6t_clear_hash_entry_lookup_result(IX_CC_CLASSIFIER_6T_QOS, HashEntry);
			break;

		case IX_CC_CLASSIFIER_6T_FWD:
                     #ifdef IX_CLASSIFIER_6T_DEBUG 
                           printf(" removing FWD \n");
                     #endif
			if (_ix_cc_classifier_6t_isEmptyEntry(IX_CC_CLASSIFIER_6T_QOS, HashEntry) == _IX_CC_CLASSIFIER_6T_ENTRY_EMPTY) {

                if (PreviousHashEntry != NULL)
					PreviousHashEntry->NextHashEntry = HashEntry->NextHashEntry;
                else
                    if (HashEntry->NextHashEntry != NULL_NEXT_ENTRY) {
					    NextHashEntry = getNextHashEntryAddress(pContext, HashEntry);
					    *HashEntry = *NextHashEntry;
                        _ix_cc_classifier_6t_zero_out_hash_entry(NextHashEntry);
                        break;
				    };

                _ix_cc_classifier_6t_zero_out_hash_entry_pattern(HashEntry);
			};
			err = _ix_cc_classifier_6t_clear_hash_entry_lookup_result(IX_CC_CLASSIFIER_6T_FWD, HashEntry);
			break;
	}

	ix_ossl_mutex_unlock(pContext->API_mutex);

	return err;
}



/*
 * FUNCTION: ix_error ix_cc_classifier_6t_update_entry()
 */
ix_error ix_cc_classifier_6t_update_entry(ix_cc_classifier_6t_pattern    *arg_pKey,
                                          ix_cc_classifier_6t_result     *arg_pLookupResult,
										  void *arg_pContext)
{
    ix_error    err   = IX_SUCCESS;
    ix_uint16   index = 0;
    ix_uint32   HashIndex;

    _ix_cc_classifier_6t_context    *pContext  = (_ix_cc_classifier_6t_context*)arg_pContext;
    _ix_cc_classifier_6t_hash_entry *PreviousHashEntry = NULL;
    _ix_cc_classifier_6t_hash_entry *HashEntry;

#if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)
    ix_cc_classifier_6t_statistics  *StatsEntry;
#endif /* #if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE) */


	if (NULL == pContext)
		return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
		                 ("ix_cc_classifier_6t_update_entry(): Error ! classifier has not been created."));

	if (NULL == pContext->HashTableAddress)
		return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
		                 ("ix_cc_classifier_6t_update_entry(): Error ! Hash table has not been created."));

	if (NULL == arg_pKey)
		return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
		                 ("ix_cc_classifier_6t_update_entry(): Error ! arg_pKey is NULL."));

	if (NULL == arg_pLookupResult)
		return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
		                 ("ix_cc_classifier_6t_update_entry(): Error ! arg_pLookupResult is NULL."));

	if ((IX_CC_CLASSIFIER_6T_QOS != arg_pLookupResult->type) &&
		(IX_CC_CLASSIFIER_6T_FWD != arg_pLookupResult->type))
	{
		return IX_ERROR_WARNING(IX_CC_CLASSIFIER_6T_ERROR_PARAM,
		                 ("ix_cc_classifier_6t_update_entry(): Error ! Table type unknown."));
	}

	if ((IX_CC_CLASSIFIER_6T_QOS == arg_pLookupResult->type) &&
		(0xffffffff == arg_pLookupResult->result.qos.flowId))
	{
		return IX_ERROR_WARNING(IX_CC_CLASSIFIER_6T_ERROR_PARAM,
		                 ("ix_cc_classifier_6t_update_entry(): Error ! Flow ID invalid value."));
	}

	if ((IX_CC_CLASSIFIER_6T_FWD == arg_pLookupResult->type) &&
		(0xffff == arg_pLookupResult->result.fwd.NextHopId))
	{
		return IX_ERROR_WARNING(IX_CC_CLASSIFIER_6T_ERROR_PARAM,
		                 ("ix_cc_classifier_6t_update_entry(): Error ! Next Hop ID invalid value."));
	}

    if ((IX_CC_CLASSIFIER_6T_QOS == arg_pLookupResult->type) &&
	(IX_CC_CLASSIFIER_6T_METER_BID != arg_pLookupResult->result.qos.OutputId) &&
    (IX_CC_CLASSIFIER_6T_MARKER_BID != arg_pLookupResult->result.qos.OutputId) &&
    (IX_CC_CLASSIFIER_6T_FORWARDER_BID != arg_pLookupResult->result.qos.OutputId) &&
    (IX_CC_CLASSIFIER_6T_DEFAULT_BID != arg_pLookupResult->result.qos.OutputId))
    {
	return IX_ERROR_WARNING(IX_CC_CLASSIFIER_6T_ERROR_PARAM,
            ("ix_cc_classifier_6t_add_entry(): Error ! Not known Output block ID."));
    }

#if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)

    if (NULL == pContext->StatsTableAddress)
	return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
             ("ix_cc_classifier_6t_update_entry(): Error ! Statistics Table has not been created."));

#endif /* #if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE) */


    ix_ossl_mutex_lock(pContext->API_mutex, IX_OSSL_WAIT_FOREVER);

    HashIndex = _ix_cc_classifier_6t_get_hash_key(arg_pKey, pContext->pUblockConfig->HashIndexSize);

    HashEntry = pContext->HashTableAddress + HashIndex;

    if (_ix_cc_classifier_6t_isEmptyElement(HashEntry)) {
        ix_ossl_mutex_unlock(pContext->API_mutex);
        return IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND,
            ("ix_cc_classifier_6t_update_entry(): Error ! Given entry does not exist."));
    } else {
        while (_ix_cc_classifier_6t_match_entry_pattern(arg_pKey, HashEntry) == _IX_CC_DIFFERENT) {
            if ((void*)HashEntry->NextHashEntry != NULL_NEXT_ENTRY) {
                PreviousHashEntry = HashEntry;
                HashEntry = getNextHashEntryAddress(pContext, HashEntry);
            } else {
                ix_ossl_mutex_unlock(pContext->API_mutex);
                return IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND,
                    ("ix_cc_classifier_6t_update_entry(): Error ! Given entry does not exist."));
            }
        }
    }

#if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)
/* If the statistics flag changes from 0 to 1 and the microblock supports statistics,
   the core component zeroes the corresponding entry in the Statistics Table. */
    if ((arg_pLookupResult->stats_enable) != 0) {
        if (HashEntry->hash_class_color_nextblock & CLASSIFIER_6T_STATISTICS_FLAG) {
            HashIndex = (ix_uint32)HashEntry - (ix_uint32)pContext->HashTableAddress;
            StatsEntry = pContext->StatsTableAddress + HashIndex;
            StatsEntry->packets = 0;
            StatsEntry->bytes = 0;
        }
    }
#endif /* #if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE) */


    err = _ix_cc_classifier_6t_clear_hash_entry_lookup_result(arg_pLookupResult->type, HashEntry);
    if (err != IX_SUCCESS) {
        ix_ossl_mutex_unlock(pContext->API_mutex);
        return IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND,
            ("ix_cc_classifier_6t_update_entry(): Error ! Given entry does not exist."));
    }

    err = _ix_cc_classifier_6t_set_hash_entry_lookup_result(arg_pLookupResult, HashEntry);
    ix_ossl_mutex_unlock(pContext->API_mutex);
    return err;
}


/*
 * FUNCTION: ix_error ix_cc_classifier_6t_search_entry()
 */
ix_error ix_cc_classifier_6t_search_entry(ix_cc_classifier_6t_pattern    *arg_pKey,
                                          ix_cc_classifier_6t_result     *arg_pLookupResult,
									  void *arg_pContext)
{
    ix_error		 err        = IX_SUCCESS;
    ix_uint16        index      = 0;
    ix_uint32        HashIndex;

    _ix_cc_classifier_6t_context    *pContext  = (_ix_cc_classifier_6t_context*)arg_pContext;
    _ix_cc_classifier_6t_hash_entry *PreviousHashEntry = NULL;
    _ix_cc_classifier_6t_hash_entry *HashEntry;

    if (NULL == pContext)
	return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                 ("ix_cc_classifier_6t_search_entry(): Error ! classifier has not been created."));

    if (NULL == pContext->HashTableAddress)
	return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                 ("ix_cc_classifier_6t_search_entry(): Error ! Hash table has not been created."));

    if (NULL == arg_pKey)
	return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                 ("ix_cc_classifier_6t_search_entry(): Error ! arg_pKey is NULL."));

    if (NULL == arg_pLookupResult)
	return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                 ("ix_cc_classifier_6t_search_entry(): Error ! arg_pLookupResult is NULL."));

    if ((IX_CC_CLASSIFIER_6T_QOS != arg_pLookupResult->type) &&
	(IX_CC_CLASSIFIER_6T_FWD != arg_pLookupResult->type))
    {
	return IX_ERROR_WARNING(IX_CC_CLASSIFIER_6T_ERROR_PARAM,
             ("ix_cc_classifier_6t_search(): Error ! Table type unknown."));
    }

    ix_ossl_mutex_lock(pContext->API_mutex, IX_OSSL_WAIT_FOREVER);

    HashIndex = _ix_cc_classifier_6t_get_hash_key(arg_pKey, pContext->pUblockConfig->HashIndexSize);

	HashEntry = pContext->HashTableAddress + HashIndex;

	if (_ix_cc_classifier_6t_isEmptyElement(HashEntry)) {
		ix_ossl_mutex_unlock(pContext->API_mutex);
		return IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND,
		       ("ix_cc_classifier_6t_search_entry(): Error ! Given entry does not exist."));
	} else {
		while (_ix_cc_classifier_6t_match_entry_pattern(arg_pKey, HashEntry) == _IX_CC_DIFFERENT) {
			if ((void*)HashEntry->NextHashEntry != NULL_NEXT_ENTRY) {
				PreviousHashEntry = HashEntry;
				HashEntry = getNextHashEntryAddress(pContext, HashEntry);
			} else {
				ix_ossl_mutex_unlock(pContext->API_mutex);
				return IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND,
				       ("ix_cc_classifier_6t_search_entry(): Error ! Given entry does not exist."));
			}
		}
	}

	err = _ix_cc_classifier_6t_get_hash_entry(arg_pLookupResult, HashEntry);
	
	ix_ossl_mutex_unlock(pContext->API_mutex);
    return err;
}



/* --------------------------------------------------------------------------------------- */
#if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)
/* --------------------------------------------------------------------------------------- */


/*
 * FUNCTION: ix_error ix_cc_classifier_6t_get_statistics()
 */
ix_error ix_cc_classifier_6t_get_statistics(ix_cc_classifier_6t_pattern *arg_pKey,
                                            ix_cc_classifier_6t_statistics *arg_pStatistics,
                                            void *arg_pContext)
{
    ix_error    err = IX_SUCCESS;
    ix_uint32   HashIndex;
    _ix_cc_classifier_6t_context    *pContext  = (_ix_cc_classifier_6t_context*)arg_pContext;
    _ix_cc_classifier_6t_hash_entry *PreviousHashEntry = NULL;
    _ix_cc_classifier_6t_hash_entry *HashEntry;
    ix_cc_classifier_6t_statistics  *StatsEntry;

    if (NULL == pContext)
	return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                 ("ix_cc_classifier_6t_get_statistics(): Error ! classifier has not been created."));

    if (NULL == pContext->HashTableAddress)
	return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                 ("ix_cc_classifier_6t_get_statistics(): Error ! Hash table has not been created."));

    if (NULL == arg_pKey)
	return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                 ("ix_cc_classifier_6t_get_statistics(): Error ! arg_pKey is NULL."));


    ix_ossl_mutex_lock(pContext->API_mutex, IX_OSSL_WAIT_FOREVER);

    HashIndex = _ix_cc_classifier_6t_get_hash_key(arg_pKey, pContext->pUblockConfig->HashIndexSize);

    HashEntry = pContext->HashTableAddress + HashIndex;

    if (_ix_cc_classifier_6t_isEmptyElement(HashEntry)) {
        ix_ossl_mutex_unlock(pContext->API_mutex);
        return IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND,
             ("ix_cc_classifier_6t_get_statistics(): Error ! Given entry does not exist."));
    } else {
        while (_ix_cc_classifier_6t_match_entry_pattern(arg_pKey, HashEntry) == _IX_CC_DIFFERENT) {
            if ((void*)HashEntry->NextHashEntry != NULL_NEXT_ENTRY) {
                PreviousHashEntry = HashEntry;
                HashEntry = getNextHashEntryAddress(pContext, HashEntry);
            } else {
                ix_ossl_mutex_unlock(pContext->API_mutex);
                return IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND,
                       ("ix_cc_classifier_6t_get_statistics(): Error ! Given entry does not exist."));
            }
        }
    }

    if (HashEntry->hash_class_color_nextblock & CLASSIFIER_6T_STATISTICS_FLAG) {
        HashIndex = (ix_uint32)(HashEntry - pContext->HashTableAddress);
        #ifdef IX_CLASSIFIER_6T_DEBUG 
        printf(" I'm taking stats Hash index : 0x%x\n",HashIndex );
	#endif

        StatsEntry = pContext->StatsTableAddress + HashIndex;
        #ifdef IX_CLASSIFIER_6T_DEBUG 
        printf(" Taking from : 0x%x\n",StatsEntry );
	#endif
        
        arg_pStatistics->packets = StatsEntry->packets;
        arg_pStatistics->bytes   = StatsEntry->bytes;
        ix_ossl_mutex_unlock(pContext->API_mutex);
        return err;
    } else {
        ix_ossl_mutex_unlock(pContext->API_mutex);
        return IX_ERROR_WARNING(IX_CC_CLASSIFIER_6T_ERROR_STATS_NOT_SUPPORTED,
                       ("ix_cc_classifier_6t_get_statistics(): Statistics for this rule are not gathered."));
    }
}


/*
 * FUNCTION: ix_error ix_cc_classifier_6t_get_statistics_def()
 */
ix_error ix_cc_classifier_6t_get_statistics_def(ix_cc_classifier_6t_statistics *arg_pStatistics,
                                                void *arg_pContext)
{
    ix_error    err = IX_SUCCESS;
    _ix_cc_classifier_6t_context    *pContext  = (_ix_cc_classifier_6t_context*)arg_pContext;

    if (NULL == pContext)
	return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                 ("ix_cc_classifier_6t_get_statistics(): Error ! classifier has not been created."));

    if (NULL == pContext->DefaultStatAddress)
	return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                 ("ix_cc_classifier_6t_get_statistics(): Error ! Default Statistics Entry has not been created."));


    if (pContext->DefaultStatEnable) {
        ix_ossl_mutex_lock(pContext->API_mutex, IX_OSSL_WAIT_FOREVER);
        arg_pStatistics->packets = pContext->DefaultStatAddress->packets;
        arg_pStatistics->bytes   = pContext->DefaultStatAddress->bytes;
        ix_ossl_mutex_unlock(pContext->API_mutex);
        return err;
    } else
        return IX_ERROR_WARNING(IX_CC_CLASSIFIER_6T_ERROR_STATS_NOT_SUPPORTED,
                       ("ix_cc_classifier_6t_get_statistics_def(): Statistics for default rule are not gathered."));
}

/* --------------------------------------------------------------------------------------- */
#endif  /* CLASSIFIER_6T_PACKET_COUNTER_FEATURE */
/* --------------------------------------------------------------------------------------- */


