/*
 * ============================================================================
 * COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2003 Intel Corporation. All rights reserved.
 *
 * PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *                                                                            */
/** \file    ix_cc_classifier_6t_msg.c
 *
 *  \bief    The file contains message helper functions developed for 
 *           6 tuple classifier core component.
 *
 */

#if !defined(IX_CC_CLASSIFIER_6T_MSG_C)
#define      IX_CC_CLASSIFIER_6T_MSG_C


#include "ix_types.h"
#include "ix_error.h"

#include "ix_ossl.h"
#include "ix_cc.h"
#include "ix_cci.h"

#include "bindings.h"

#include "ix_cc_error.h"

#include "cc/ix_cc_msup.h"
#include "cc/ix_cc_classifier_6t.h"
#include "cc/ix_cc_classifier_6t_msg.h"

/*
 * FUNCTION: _ix_cc_classifier_6t_cbi_add_entry
 */
PRIVATE ix_error
_ix_cc_classifier_6t_cbi_add_entry(ix_error  arg_Result,
                                   void     *arg_pContext,
                                   void     *arg_Msg,
                                   ix_uint32 arg_MsgLen)
{
    void                             *pUserContext = NULL;
    ix_error                          err          = IX_SUCCESS;
    ix_cc_classifier_6t_message_ctx  *pContext     = (ix_cc_classifier_6t_message_ctx*)arg_pContext;
    ix_cc_classifier_6t_cb_add_entry  CallBack;
    
    if (NULL == pContext)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
           ("ix_cc_classifier_6t_cbi_add_entry(): pContext is NULL"));
    #ifdef IX_CLASSIFIER_6T_DEBUG 
    printf("_ix_cc_classifier_6t_cbi_add_entry(\n");
    #endif
    /* Retrieve the user callback and context from the context pointer */
    CallBack = pContext->pCallBack;
    if (NULL == CallBack)
    {
        /* can not call user - free context */
		ix_ossl_free(pContext);
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
           ("ix_cc_classifier_6t_cbi_add_entry(): CallBack is NULL"));
    }

    pUserContext = pContext->pUserContext;

    /* Invoke the user callback to pass back the result and data */
    err = (*CallBack)(arg_Result, pUserContext);

    /* Free the callback context entry */
    ix_ossl_free(pContext);
    pContext = NULL;

    if (IX_SUCCESS != err)
        return IX_ERROR_REMOTE(IX_CC_CLASSIFIER_6T_ERROR_USER_CB,
	           ("ix_cc_classifier_6t_cbi_add_entry(): CallBack function failed"));

    return err;
} 


/*
 * FUNCTION: ix_cc_classifier_6t_async_add_entry()
 */
ix_error
ix_cc_classifier_6t_async_add_entry(ix_cc_classifier_6t_pattern     *arg_pKey,
                                    ix_cc_classifier_6t_result      *arg_pLookupResult,
                                    ix_cc_classifier_6t_cb_add_entry arg_Callback,
                                    void                            *arg_pUserContext)
{
    ix_error                                  err     = IX_SUCCESS;
    ix_cc_classifier_6t_message_ctx          *pIntCtx = NULL;
    ix_cc_classifier_6t_message_general_data  MsgData;

    if (NULL == arg_Callback) 
	return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
	       ("ix_cc_classifier_6t_async_add_entry(): function was called with NULL callback"));

    /* create ix_cc_classifier_6t_message_ctx */
    pIntCtx = ix_ossl_malloc(sizeof(ix_cc_classifier_6t_message_ctx));
    
    if (NULL == pIntCtx) 
	return IX_ERROR_WARNING(IX_CC_ERROR_OOM_SYSTEM,
	       ("ix_cc_classifier_6t_async_add_entry(): Fail to allocate system memory"));
    
    MsgData.Key           = *arg_pKey;
    MsgData.LookupResult  = *arg_pLookupResult;
    pIntCtx->pCallBack    = arg_Callback;
    pIntCtx->pUserContext = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_CLASSIFIER_6T_MSG_INPUT,
                                           (ix_cc_msghlp_callback)_ix_cc_classifier_6t_cbi_add_entry,
                                           pIntCtx,
                                           IX_CC_CLASSIFIER_6T_MSG_ADD_ENTRY,
                                           (void*)&MsgData,
                                           sizeof(ix_cc_classifier_6t_message_general_data)),
                 err,
                 IX_CC_ERROR_SEND_FAIL,
                 IX_ERROR_LEVEL_WARNING, 
                 cleanup_context);

    return IX_SUCCESS;

cleanup_context:

    if(pIntCtx != NULL)
    {
        ix_ossl_free(pIntCtx);
    }/* end if cbContext != NULL */
    return err;
}

/*
 * FUNCTION: _ix_cc_classifier_6t_cbi_remove_entry
 */
PRIVATE ix_error
_ix_cc_classifier_6t_cbi_remove_entry(ix_error  arg_Result,
                                  void     *arg_pContext,
                                  void     *arg_Msg,
                                  ix_uint32 arg_MsgLen)
{
    ix_error                            err = IX_SUCCESS;
    void                               *pUserContext = NULL;
    ix_cc_classifier_6t_cb_remove_entry CallBack;
    ix_cc_classifier_6t_message_ctx    *pContext = (ix_cc_classifier_6t_message_ctx*)arg_pContext;
    
    if (NULL == pContext)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
	         ("ix_cc_classifier_6t_cbi_remove_entry(): pContext is NULL"));

    /* Retrieve the user callback and context from the context pointer */
    CallBack = pContext->pCallBack;
    if (NULL == CallBack)
    {
        /* can not call user - free context */
		ix_ossl_free(pContext);
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
	           ("ix_cc_classifier_6t_cbi_remove_entry(): CallBack is NULL"));
    }

    pUserContext = pContext->pUserContext;
    
    /* Invoke the user callback to pass back the result and data */
    err = (*CallBack)(arg_Result, pUserContext);

    /* Free the callback context entry */
    ix_ossl_free(pContext);
    pContext = NULL;

    if (IX_SUCCESS != err)
        return IX_ERROR_REMOTE(IX_CC_CLASSIFIER_6T_ERROR_USER_CB,
	           ("ix_cc_classifier_6t_cbi_remove_entry(): CallBack function failed"));

    return err;
} 

/*
 * FUNCTION: ix_cc_classifier_6t_async_remove_entry()
 */
ix_error
ix_cc_classifier_6t_async_remove_entry(ix_cc_classifier_6t_pattern        *arg_pKey,
                                       ix_cc_classifier_6t_table_type      arg_Type,
                                       ix_cc_classifier_6t_cb_remove_entry arg_Callback,
                                       void                               *arg_pUserContext)
{
    ix_error err = IX_SUCCESS;
    ix_cc_classifier_6t_message_ctx        *pIntCtx = NULL;
    ix_cc_classifier_6t_message_remove_data MsgData;
	
    if (NULL == arg_Callback) 
	return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
	       ("ix_cc_classifier_6t_async_remove_entry(): function was called with NULL callback"));
    
    /* create ix_cc_classifier_6t_message_ctx */
    pIntCtx = ix_ossl_malloc(sizeof(ix_cc_classifier_6t_message_ctx));
    
    if (NULL == pIntCtx) 
	return IX_ERROR_WARNING(IX_CC_ERROR_OOM_SYSTEM,
	       ("ix_cc_classifier_6t_async_remove_entry(): Fail to allocate system memory"));   
    
    MsgData.Key           = *arg_pKey;
    MsgData.Type          = arg_Type;
    pIntCtx->pCallBack    = arg_Callback;
    pIntCtx->pUserContext = arg_pUserContext;
	
    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_CLASSIFIER_6T_MSG_INPUT,
                                           (ix_cc_msghlp_callback)_ix_cc_classifier_6t_cbi_remove_entry,
                                           pIntCtx,
                                           IX_CC_CLASSIFIER_6T_MSG_REMOVE_ENTRY,
                                           (void*)&MsgData,
                                           sizeof(ix_cc_classifier_6t_message_remove_data)),
                 err,
                 IX_CC_ERROR_SEND_FAIL,
                 IX_ERROR_LEVEL_WARNING, cleanup_context);

    return IX_SUCCESS;

cleanup_context:

    if(pIntCtx != NULL)
    {
        ix_ossl_free(pIntCtx);
    }/* end if cbContext != NULL */
    return err;
}

/*
 * FUNCTION: _ix_cc_classifier_6t_cbi_update_entry
 */
PRIVATE ix_error
_ix_cc_classifier_6t_cbi_update_entry(ix_error  arg_Result,
                                  void     *arg_pContext,
                                  void     *arg_Msg,
                                  ix_uint32 arg_MsgLen)
{
    ix_error                            err = IX_SUCCESS;
    void                               *pUserContext = NULL;
    ix_cc_classifier_6t_cb_update_entry CallBack;
    ix_cc_classifier_6t_message_ctx    *pContext = (ix_cc_classifier_6t_message_ctx*)arg_pContext;
    
    if (NULL == pContext)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
          ("ix_cc_classifier_6t_cbi_update_entry(): pContext is NULL"));

    /* Retrieve the user callback and context from the context pointer */
    CallBack = pContext->pCallBack;
    if (NULL == CallBack)
    {
        /* can not call user - free context */
        ix_ossl_free(pContext);
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
           ("ix_cc_classifier_6t_cbi_update_entry(): CallBack is NULL"));
    }

    pUserContext = pContext->pUserContext;
    
    /* Invoke the user callback to pass back the result and data */
    err = (*CallBack)(arg_Result, pUserContext);

    /* Free the callback context entry */
    ix_ossl_free(pContext);
    pContext = NULL;

    if (IX_SUCCESS != err)
        return IX_ERROR_REMOTE(IX_CC_CLASSIFIER_6T_ERROR_USER_CB,
	           ("ix_cc_classifier_6t_cbi_update_entry(): CallBack function failed"));

    return err;
} 

/*
 * FUNCTION: ix_cc_classifier_6t_async_update_entry()
 */
ix_error
ix_cc_classifier_6t_async_update_entry(ix_cc_classifier_6t_pattern        *arg_pKey,
                                       ix_cc_classifier_6t_result         *arg_pLookupResult,
                                       ix_cc_classifier_6t_cb_update_entry arg_Callback,
                                       void                               *arg_pUserContext)
{
    ix_error err = IX_SUCCESS;
    ix_cc_classifier_6t_message_ctx          *pIntCtx = NULL;
    ix_cc_classifier_6t_message_general_data  MsgData;

    if (NULL == arg_Callback) 
		return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
		       ("ix_cc_classifier_6t_async_update_entry(): function was called with NULL callback"));
    
    /* create ix_cc_classifier_6t_message_ctx */
    pIntCtx = ix_ossl_malloc(sizeof(ix_cc_classifier_6t_message_ctx));
    
    if (NULL == pIntCtx) 
	return IX_ERROR_WARNING(IX_CC_ERROR_OOM_SYSTEM,
	       ("ix_cc_classifier_6t_async_update_entry(): Fail to allocate system memory"));
    
    MsgData.Key           = *arg_pKey;
    MsgData.LookupResult  = *arg_pLookupResult;
    pIntCtx->pCallBack    = arg_Callback;
    pIntCtx->pUserContext = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_CLASSIFIER_6T_MSG_INPUT,
                                           (ix_cc_msghlp_callback)_ix_cc_classifier_6t_cbi_update_entry,
                                           pIntCtx,
                                           IX_CC_CLASSIFIER_6T_MSG_UPDATE_ENTRY,
                                           (void*)&MsgData,
                                           sizeof(ix_cc_classifier_6t_message_general_data)),
                 err,
                 IX_CC_ERROR_SEND_FAIL,
                 IX_ERROR_LEVEL_WARNING, cleanup_context);

    return IX_SUCCESS;

cleanup_context:

    if(pIntCtx != NULL)
    {
        ix_ossl_free(pIntCtx);
    }/* end if cbContext != NULL */
    return err;
}

/*
 * FUNCTION: _ix_cc_classifier_6t_cbi_search_entry
 */
PRIVATE ix_error
_ix_cc_classifier_6t_cbi_search_entry(ix_error  arg_Result,
                                      void     *arg_pContext,
                                      void     *arg_Msg,
                                      ix_uint32 arg_MsgLen)
{
    ix_error  err = IX_SUCCESS;    
    ix_cc_classifier_6t_cb_search_entry     CallBack;
    ix_cc_classifier_6t_search_message_ctx *pContext = 
                                   (ix_cc_classifier_6t_search_message_ctx*)arg_pContext;
    
    #ifdef IX_CLASSIFIER_6T_DEBUG 
    ix_ossl_message_log("_ix_cc_classifier_6t_cbi_search_entry(): ha, ha callback's comming ...\n");
    #endif

    if (NULL == pContext)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
	           ("ix_cc_classifier_6t_cbi_search_entry(): message context is NULL"));

    /* Retrieve the user callback and context from the context pointer */
    CallBack = pContext->pCallBack;
    if (NULL == CallBack)
    {
        /* can not call user - free context */
		ix_ossl_free(pContext);
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
	           ("ix_cc_classifier_6t_cbi_search_entry(): CallBack is NULL"));
    }

    if ( arg_Result == IX_SUCCESS )
    {
        if ( arg_Msg == NULL )
        {
            ix_ossl_free(pContext);
            return IX_ERROR_WARNING(
                IX_CC_ERROR_NULL,
                ("ix_cc_classifier_6t_cbi_search_entry(): Lookup result is NULL"));
        }

        if ( arg_MsgLen != sizeof( ix_cc_classifier_6t_result ) )
        {
            ix_ossl_free(pContext);
            return IX_ERROR_WARNING(
                IX_CC_ERROR_INTERNAL,
                ("ix_cc_classifier_6t_cbi_search_entry(): message length doesn't match " \
                 "expected size" ));
        }
    }
    else
    {
        arg_Msg = NULL;
    }    

    *(pContext->pUserPlaceholder) = *(ix_cc_classifier_6t_result *)arg_Msg;

    /* Invoke the user callback to pass back the result and data */
    err = (*CallBack)(arg_Result, 
                      pContext->pUserPlaceholder,
                      pContext->pUserContext);

    /* Free the callback context entry */
    ix_ossl_free(pContext);
    pContext = NULL;

    if (IX_SUCCESS != err)
        return IX_ERROR_REMOTE(IX_CC_CLASSIFIER_6T_ERROR_USER_CB,
	           ("ix_cc_classifier_6t_cbi_search_entry(): CallBack function failed"));

    return err;
} 

/*
 * FUNCTION: ix_cc_classifier_6t_async_search_entry()
 */
ix_error
ix_cc_classifier_6t_async_search_entry(ix_cc_classifier_6t_pattern        *arg_pKey,
                                       ix_cc_classifier_6t_table_type      arg_LookupType,
                                       ix_cc_classifier_6t_result         *arg_LookupResult,
                                       ix_cc_classifier_6t_cb_search_entry arg_Callback,
                                       void                               *arg_pUserContext)
{
    ix_error err = IX_SUCCESS;
    ix_cc_classifier_6t_search_message_ctx  *pIntCtx = NULL;
    ix_cc_classifier_6t_message_search_data  MsgData;

    #ifdef IX_CLASSIFIER_6T_DEBUG 
    ix_ossl_message_log("ix_cc_classifier_6t_async_search_entry(): ha, ha I'am in :))))))\n");
    #endif

    if (NULL == arg_Callback) 
		return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
		       ("ix_cc_classifier_6t_async_search_entry(): function was called with NULL callback"));
    
    /* create ix_cc_classifier_6t_message_ctx */
    pIntCtx = ix_ossl_malloc(sizeof(ix_cc_classifier_6t_message_ctx));
    
    if (NULL == pIntCtx) 
	return IX_ERROR_WARNING(IX_CC_ERROR_OOM_SYSTEM,
	       ("ix_cc_classifier_6t_async_search_entry(): Fail to allocate system memory"));
    
    MsgData.Key               = *arg_pKey;
    MsgData.LookupType        = arg_LookupType;
    pIntCtx->pCallBack        = arg_Callback;
    pIntCtx->pUserContext     = arg_pUserContext;
    pIntCtx->pUserPlaceholder = arg_LookupResult;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_CLASSIFIER_6T_MSG_INPUT,
                                           (ix_cc_msghlp_callback)_ix_cc_classifier_6t_cbi_search_entry,
                                           pIntCtx,
	                                       IX_CC_CLASSIFIER_6T_MSG_SEARCH_ENTRY,
                                           (void*)&MsgData,
                                           sizeof(ix_cc_classifier_6t_message_search_data)),
                 err,
                 IX_CC_ERROR_SEND_FAIL,
                 IX_ERROR_LEVEL_WARNING, cleanup_context);

    return IX_SUCCESS;


cleanup_context:

    if(pIntCtx != NULL)
    {
        ix_ossl_free(pIntCtx);
    }/* end if cbContext != NULL */
    return err;
}



/* --------------------------------------------------------------------------------------- */
#if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE)
/* --------------------------------------------------------------------------------------- */



/*
 * FUNCTION: ix_cc_classifier_6t_cbi_get_statistics_def
 */
PRIVATE ix_error
ix_cc_classifier_6t_cbi_get_statistics_def(ix_error  arg_Result,
                                           void     *arg_pContext,
                                           void     *arg_Msg,
                                           ix_uint32 arg_MsgLen)
{
    ix_error  err = IX_SUCCESS;    
    ix_cc_classifier_6t_cb_get_statistics_def  CallBack;
    ix_cc_classifier_6t_get_stats_message_ctx *pContext = 
                                 (ix_cc_classifier_6t_get_stats_message_ctx*)arg_pContext;
    
    #ifdef IX_TEST_CLASSIFIER_6T_DEBUG 
    ix_ossl_message_log("ix_cc_classifier_6t_cbi_get_statistics_def(): ha, ha callback's comming ...\n");
    #endif

    if (NULL == pContext)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
	           ("ix_cc_classifier_6t_cbi_get_statistics_def(): message context is NULL"));

    /* Retrieve the user callback and context from the context pointer */
    CallBack = pContext->pCallBack;
    if (NULL == CallBack)
    {
        /* can not call user - free context */
        ix_ossl_free(pContext);
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
	           ("ix_cc_classifier_6t_cbi_get_statistics_def(): CallBack is NULL"));
    }

    if ( arg_Result == IX_SUCCESS )
    {
        if ( arg_Msg == NULL )
        {
            ix_ossl_free(pContext);
            return IX_ERROR_WARNING(
                IX_CC_ERROR_NULL,
                ("ix_cc_classifier_6t_cbi_get_statistics_def(): Lookup result is NULL"));
        }

        if ( arg_MsgLen != sizeof( ix_cc_classifier_6t_statistics ) )
        {
            ix_ossl_free(pContext);
            return IX_ERROR_WARNING(
                IX_CC_ERROR_INTERNAL,
                ("ix_cc_classifier_6t_cbi_get_statistics_def(): message length doesn't match " \
                 "expected size" ));
        }
    }
    else
    {
        arg_Msg = NULL;
    }   

    *(pContext->pUserPlaceholder) = *(ix_cc_classifier_6t_statistics *)arg_Msg;

    /* Invoke the user callback to pass back the result and data */
    err = (*CallBack)(arg_Result, 
                      pContext->pUserPlaceholder, 
                      pContext->pUserContext);

    /* Free the callback context entry */
    ix_ossl_free(pContext);
    pContext = NULL;

    if (IX_SUCCESS != err)
        return IX_ERROR_REMOTE(IX_CC_CLASSIFIER_6T_ERROR_USER_CB,
	           ("ix_cc_classifier_6t_cbi_get_statistics_def(): CallBack function failed"));

    return err;
} 


/*
 * FUNCTION: ix_cc_classifier_6t_async_get_statistics_def()
 */
ix_error
ix_cc_classifier_6t_async_get_statistics_def(ix_cc_classifier_6t_statistics        *arg_pStatistics,
                                             ix_cc_classifier_6t_cb_get_statistics  arg_Callback,
                                             void                                  *arg_pUserContext)
{
    ix_error                                   err = IX_SUCCESS;
    ix_cc_classifier_6t_get_stats_message_ctx *pIntCtx = NULL;
    ix_cc_classifier_6t_statistics             MsgData;

    #ifdef IX_TEST_CLASSIFIER_6T_DEBUG 
    ix_ossl_message_log("ix_cc_classifier_6t_async_get_statistics_def(): ha, ha I'am in :))))))\n");
    #endif

    if (NULL == arg_Callback) 
		return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
		       ("ix_cc_classifier_6t_async_get_statistics_def(): function was called with NULL callback"));
    
    /* create ix_cc_classifier_6t_message_ctx */
    pIntCtx = ix_ossl_malloc(sizeof(ix_cc_classifier_6t_get_stats_message_ctx));
    
    if (NULL == pIntCtx) 
	return IX_ERROR_WARNING(IX_CC_ERROR_OOM_SYSTEM,
	       ("ix_cc_classifier_6t_async_get_statistics_def(): Fail to allocate system memory"));
    
    pIntCtx->pUserPlaceholder = arg_pStatistics;
    pIntCtx->pCallBack        = arg_Callback;
    pIntCtx->pUserContext     = arg_pUserContext;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_CLASSIFIER_6T_MSG_INPUT,
                                           (ix_cc_msghlp_callback)ix_cc_classifier_6t_cbi_get_statistics_def,
                                           pIntCtx,
                                           IX_CC_CLASSIFIER_6T_MSG_GET_STATISTICS_DEF,
                                           (void*)&MsgData,
                                           sizeof(ix_cc_classifier_6t_statistics)),
                 err,
                 IX_CC_ERROR_SEND_FAIL,
                 IX_ERROR_LEVEL_WARNING, cleanup_context);

    return IX_SUCCESS;


cleanup_context:

    if(pIntCtx != NULL)
    {
        ix_ossl_free(pIntCtx);
    }/* end if cbContext != NULL */
    return err;
}


/*
 * FUNCTION: ix_cc_classifier_6t_cbi_get_statistics
 */
PRIVATE ix_error
ix_cc_classifier_6t_cbi_get_statistics(ix_error  arg_Result,
                                       void     *arg_pContext,
                                       void     *arg_Msg,
                                       ix_uint32 arg_MsgLen)
{
    ix_error  err = IX_SUCCESS;    
    ix_cc_classifier_6t_cb_get_statistics      CallBack;
    ix_cc_classifier_6t_get_stats_message_ctx *pContext = 
                                            (ix_cc_classifier_6t_get_stats_message_ctx*)arg_pContext;
    
    #ifdef IX_CLASSIFIER_6T_DEBUG 
    ix_ossl_message_log("ix_cc_classifier_6t_cbi_get_statistics(): ha, ha callback's comming ...\n");
    #endif

    if (NULL == pContext)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
	           ("ix_cc_classifier_6t_cbi_get_statistics(): message context is NULL"));

    /* Retrieve the user callback and context from the context pointer */
    CallBack = pContext->pCallBack;
    if (NULL == CallBack)
    {
        #ifdef IX_CLASSIFIER_6T_DEBUG 
        ix_ossl_message_log("ix_cc_classifier_6t_cbi_get_statistics(): user callback is null\n");
        #endif
        /* can not call user - free context */
        ix_ossl_free(pContext);
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
	           ("ix_cc_classifier_6t_cbi_get_statistics(): CallBack is NULL"));
    }

    if ( arg_Result == IX_SUCCESS )
    {
        if ( arg_Msg == NULL )
        {
            #ifdef IX_CLASSIFIER_6T_DEBUG 
            ix_ossl_message_log("ix_cc_classifier_6t_cbi_get_statistics(): Lookup result is NULL\n");
            #endif
 
            ix_ossl_free(pContext);
            return IX_ERROR_WARNING(
                IX_CC_ERROR_NULL,
                ("ix_cc_classifier_6t_cbi_get_statistics_def():Lookup result is NULL"));
        }

        if ( arg_MsgLen != sizeof( ix_cc_classifier_6t_statistics ) )
        {
            #ifdef IX_CLASSIFIER_6T_DEBUG 
            ix_ossl_message_log("ix_cc_classifier_6t_cbi_get_statistics(): message length doesn't match\n");
            #endif

            ix_ossl_free(pContext);
            return IX_ERROR_WARNING(
                IX_CC_ERROR_INTERNAL,
                ("ix_cc_classifier_6t_cbi_get_statistics_def(): message length doesn't match " \
                 "expected size" ));
        }
    
        #ifdef IX_CLASSIFIER_6T_DEBUG 
        printf("ix_cc_classifier_6t_cbi_get_statistics:  bytes = %d \n",((ix_cc_classifier_6t_statistics *)arg_Msg)->bytes);
        #endif
        
        *(pContext->pUserPlaceholder) = *(ix_cc_classifier_6t_statistics *)arg_Msg;
    }
    else
    {
        arg_Msg = NULL;
        ix_ossl_memset( pContext->pUserPlaceholder, 0, sizeof( ix_cc_classifier_6t_statistics ) );
    }   

    
    /* Invoke the user callback to pass back the result and data */
    err = (*CallBack)(arg_Result, 
                      pContext->pUserPlaceholder,
                      pContext->pUserContext);


    /* Free the callback context entry */
    ix_ossl_free( pContext );
    
    pContext = NULL;

    if (IX_SUCCESS != err)
        return IX_ERROR_REMOTE(IX_CC_CLASSIFIER_6T_ERROR_USER_CB,
	           ("ix_cc_classifier_6t_cbi_get_statistics(): CallBack function failed"));

    return err;
} 


/*
 * FUNCTION: ix_cc_classifier_6t_async_get_statistics()
 */
ix_error
ix_cc_classifier_6t_async_get_statistics(ix_cc_classifier_6t_pattern           *arg_pKey,
                                         ix_cc_classifier_6t_statistics        *arg_pStatistics,
                                         ix_cc_classifier_6t_cb_get_statistics  arg_Callback,
                                         void                                  *arg_pUserContext)
{
    ix_error                                   err = IX_SUCCESS;
    ix_cc_classifier_6t_get_stats_message_ctx *pIntCtx = NULL;
    ix_cc_classifier_6t_message_stats_data     MsgData;

    #ifdef IX_CLASSIFIER_6T_DEBUG 
    ix_ossl_message_log("ix_cc_classifier_6t_async_get_statistics(): ha, ha I'am in :))))))\n");
    #endif

    if (NULL == arg_Callback) 
		return IX_ERROR_WARNING(IX_CC_ERROR_NULL,
		       ("ix_cc_classifier_6t_async_get_statistics(): function was called with NULL callback"));
    
    /* create ix_cc_classifier_6t_message_ctx */
    pIntCtx = ix_ossl_malloc(sizeof(ix_cc_classifier_6t_message_stats_data));
    
    if (NULL == pIntCtx) 
	return IX_ERROR_WARNING(IX_CC_ERROR_OOM_SYSTEM,
	       ("ix_cc_classifier_6t_async_get_statistics(): Fail to allocate system memory"));
    
    pIntCtx->pUserPlaceholder = arg_pStatistics;
    pIntCtx->pCallBack        = arg_Callback;
    pIntCtx->pUserContext     = arg_pUserContext;

    MsgData.Key               = *arg_pKey;

    IX_ERROR_CGT(ix_cc_msup_send_async_msg(IX_CC_CLASSIFIER_6T_MSG_INPUT,
                                           (ix_cc_msghlp_callback)ix_cc_classifier_6t_cbi_get_statistics,
                                           pIntCtx,
                                           IX_CC_CLASSIFIER_6T_MSG_GET_STATISTICS,
                                           (void*)&MsgData,
                                           sizeof(ix_cc_classifier_6t_message_stats_data)),
                 err,
                 IX_CC_ERROR_SEND_FAIL,
                 IX_ERROR_LEVEL_WARNING, cleanup_context);

    return IX_SUCCESS;


cleanup_context:

    if(pIntCtx != NULL)
    {
        ix_ossl_free(pIntCtx);
    }/* end if cbContext != NULL */
    return err;
}

/* --------------------------------------------------------------------------------------- */
#endif  /* CLASSIFIER_6T_PACKET_COUNTER_FEATURE */
/* --------------------------------------------------------------------------------------- */

#endif
