#ifndef CLASSIFIER_6T_DEFS_H
#define CLASSIFIER_6T_DEFS_H

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/
/* "$Id: classifier_6t_defs.h,v 1.17.6.1 2004/03/12 10:28:34 wneubaue Exp $" */

/*
 * FILE  classifier_6t_defs.h
 *
 * Definitions relevant to both 6-tuple IPv4 classifier microblock and
 * core componenet
 *
 * The following global tokens change configuration of a classifier :
 *
 * CLASSIFIER_6T_SRAM_HASH_TABLE - specifies that the hash table will be
 *                                 stored in SRAM. This tocken cannot be defined 
 *                                 simultaneously with 
 *                                 CLASSIFIER_6T_DRAM_HASH_TABLE 
 *
 * CLASSIFIER_6T_DRAM_HASH_TABLE - specifies that the hash table will be
 *                                 stored in DRAM. This tocken cannot be defined 
 *                                 simultaneously with 
 *                                 CLASSIFIER_6T_SRAM_HASH_TABLE 
 *
 * CLASSIFIER_6T_HASH_BY_CRC     - as a default classifier uses hash unit to
 *                                 calculate hash index. Hash unit allows 
 *                                 to transfer the computation to hardware.
 *                                 This option allows to disable the use of 
 *                                 external unit and replace it with crc
 *                                 instruction. This loads a little more 
 *                                 microengine but overall result is better.
 *
 * CLASSIFIER_6T_PACKET_COUNTER_FEATURE - includes a code to collect statistics
 *
 */


/*
 * values to initialize hardware hash multiplicand with.
 * It is done only for simulation in workbench, otherwise it is
 * the job of the 6 tuple core component
 */
#define     CLASSIFIER_6T_HASH_MUL_LW0                  0xAAAAAAAA
#define     CLASSIFIER_6T_HASH_MUL_LW1                  0xAAAAAAAA
#define     CLASSIFIER_6T_HASH_MUL_LW2                  0xAAAAAAAA
#define     CLASSIFIER_6T_HASH_MUL_LW3                  0xAAAAAAAA


/*
 * actual size of a hash table entry in bytes
 */
#define     CLASSIFIER_6T_STATISTICS_FLAG               0x8000



/*
 * byte size of a hash table entry expressed as a power of 2
 */
#define	    CLASSIFIER_6T_HASH_TABLE_ENTRY_LOG_SIZE      5


/*
 * actual size of a hash table entry in bytes
 */
#define     CLASSIFIER_6T_HASH_TABLE_ENTRY_SIZE \
                        (1 << CLASSIFIER_6T_HASH_TABLE_ENTRY_LOG_SIZE)


/*
 * actual size of a statistics entry in bytes
 */
#ifndef     CLASSIFIER_6T_STATISTICS_ENTRY_SIZE
#define     CLASSIFIER_6T_STATISTICS_ENTRY_SIZE         16
#endif


/* Size of a hash table entry expressed as a power of 2
   Rule : int[Log2(CLASSIFIER_6T_STATISTICS_ENTRY_SIZE)]. - number of bits to shift */
#ifndef     CLASSIFIER_6T_STATISTICS_ENTRY_LOG_SIZE
#define     CLASSIFIER_6T_STATISTICS_ENTRY_LOG_SIZE      4
#endif


#endif
/*  CLASSIFIER_6T_DEFS_H  */

