#ifndef CLASSIFIER_6T_H
#define CLASSIFIER_6T_H

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/
/* #define IX_ERROR_FILE_IDENT "$Id: classifier_6t.h,v 1.18.6.1 2004/03/12 10:28:15 wneubaue Exp $" */

/*
 * FILE  classifier_6t.h
 *
 * Contains definitions relevant to 6-tuple IPv4 classifier microblock
 *
 * The following global tokens change configuration of a classifier. They should
 * be defined in classifier_6t_defs.h:
 *
 * #define CLASSIFIER_6T_SRAM_HASH_TABLE - specifies that the hash table will be
 *                                 stored in SRAM. This tocken cannot be defined 
 *                                 simultaneously with 
 *                                 CLASSIFIER_6T_DRAM_HASH_TABLE 
 *
 * #define CLASSIFIER_6T_DRAM_HASH_TABLE - specifies that the hash table will be
 *                                 stored in DRAM. This tocken cannot be defined 
 *                                 simultaneously with 
 *                                 CLASSIFIER_6T_SRAM_HASH_TABLE 
 *
 * #define CLASSIFIER_6T_HASH_BY_CRC - as a default classifier uses hash unit to
 *                                 calculate hash index. Hash unit allows 
 *                                 to transfer the computation to hardware.
 *                                 This option allows to disable the use of 
 *                                 external unit and replace it with crc
 *                                 instruction. This loads a little more 
 *                                 microengine but overall result is better.
 *
 * #define  CLASSIFIER_6T_PACKET_COUNTER_FEATURE - includes a code to collect 
 *                                 statistics
 */


#ifndef CLASSIFIER_6T_DRAM_HASH_TABLE
#ifndef CLASSIFIER_6T_SRAM_HASH_TABLE
	#warning "Define either CLASSIFIER_6T_SRAM_HASH_TABLE "
	#warning "CLASSIFIER_6T_DRAM_HASH_TABLE to let compiler know where "
	#error   "classifier is supposed to keep hash table."
#endif
#endif

#ifndef CLASSIFIER_6T_SRAM_HASH_TABLE
#ifndef CLASSIFIER_6T_DRAM_HASH_TABLE
	#warning "Define either CLASSIFIER_6T_SRAM_HASH_TABLE or "
	#warning "CLASSIFIER_6T_DRAM_HASH_TABLE to let compiler know where "
	#error   "classifier is supposed to keep hash table."
#endif
#endif

#ifdef CLASSIFIER_6T_SRAM_HASH_TABLE
#ifdef CLASSIFIER_6T_DRAM_HASH_TABLE
	#warning "don't know where to put hash table"
	#warning "CLASSIFIER_6T_DRAM_HASH_TABLE and CLASSIFIER_6T_DRAM_HASH_TABLE"
	#error   "constats cannot be defined simultaneously"
#endif
#endif



/*
 * number of bits taken from the result of a hash operation to
 * index hash table in ram
 */
#if (CLASSIFIER_6T_HASH_INDEX_SIZE > 32)
	#error "CLASSIFIER_6T_HASH_INDEX_SIZE cannot be grater than 32 bits"
#endif


/*
 * default values used by 6-tuple classifier for packets 
 * not found in the hash table.
 */
#ifndef USE_IMPORT_VAR

	/*
	 * HASH_TABLE_BASE, PPP_HDR_SIZE are system dependent definitions.
	 * Therefore it should be made either in dl_system.h 
	 * or as a compilation option.
	 */
	#ifndef	CLASSIFIER_6T_HASH_TABLE_BASE
		#error	"CLASSIFIER_6T_HASH_TABLE_BASE not defined."
	#endif

	#ifdef CLASSIFIER_6T_PACKET_COUNTER_FEATURE
		#ifndef	CLASSIFIER_6T_STATS_SRAM_BASE
			#error	"CLASSIFIER_6T_STATS_SRAM_BASE not defined."
		#endif
		#ifdef CLASSIFIER_6T_DEFAULT_STATS_ENABLE
		#ifndef CLASSIFIER_6T_DEFAULT_STATS_ADDRESS
			#error	"CLASSIFIER_6T_DEFAULT_STATS_ADDRESS not defined."
		#endif
		#endif
	#endif

	#ifndef CLASSIFIER_6T_DEFAULT_COLOR_ID
		#warning "CLASSIFIER_6T_DEFAULT_COLOR_ID not defined."
		#warning "Default value GREEN_PACKET will be used."
		#define CLASSIFIER_6T_DEFAULT_COLOR_ID		(GREEN_PACKET)
	#endif
	#ifndef CLASSIFIER_6T_DEFAULT_FLOW_ID
		#warning "CLASSIFIER_6T_DEFAULT_FLOW_ID not defined."
		#warning "Default value 0x01 will be used."
		#define CLASSIFIER_6T_DEFAULT_FLOW_ID		0x01
	#endif
	#ifndef CLASSIFIER_6T_DEFAULT_CLASS_ID
		#warning "CLASSIFIER_6T_DEFAULT_CLASS_ID not defined."
		#warning "Default value 0x00 will be used."
		#define CLASSIFIER_6T_DEFAULT_CLASS_ID		0x00
	#endif
	#ifndef CLASSIFIER_6T_DEFAULT_NEXTHOP_ID
		#warning "CLASSIFIER_6T_DEFAULT_NEXTHOP_ID not defined."
		#warning "Default value 0x00 will be used."
		#define CLASSIFIER_6T_DEFAULT_NEXTHOP_ID	0x00
	#endif
	#ifndef CLASSIFIER_6T_DEFAULT_NEXTHOP_ID_TYPE
		#warning "CLASSIFIER_6T_DEFAULT_NEXTHOP_ID_TYPE not defined."
		#warning "Default value 0x00 will be used."
		#define CLASSIFIER_6T_DEFAULT_NEXTHOP_ID_TYPE	0x00
	#endif

#else
	.import_var CLASSIFIER_6T_HASH_TABLE_BASE
	.import_var CLASSIFIER_6T_DEFAULT_COLOR_ID
	.import_var CLASSIFIER_6T_DEFAULT_FLOW_ID
	.import_var CLASSIFIER_6T_DEFAULT_CLASS_ID
	.import_var CLASSIFIER_6T_DEFAULT_NEXTHOP_ID
	.import_var CLASSIFIER_6T_DEFAULT_NEXTHOP_ID_TYPE

	#ifdef CLASSIFIER_6T_PACKET_COUNTER_FEATURE
		.import_var CLASSIFIER_6T_STATS_SRAM_BASE
		#ifdef CLASSIFIER_6T_DEFAULT_STATS_ENABLE
		.import_var CLASSIFIER_6T_DEFAULT_STATS_ADDRESS
		#endif
	#endif

#endif

#ifndef CLASSIFIER_6T_DEFAULT_NEXT_BLOCK
	#warning "CLASSIFIER_6T_DEFAULT_NEXT_BLOCK not defined."
	#warning "Default value IX_DROP will be used."
	#define CLASSIFIER_6T_DEFAULT_NEXT_BLOCK	IX_DROP
#endif
#ifndef CLASSIFIER_6T_INV_IP_NEXT_BLOCK
	#warning "CLASSIFIER_6T_INV_IP_NEXT_BLOCK not defined."
	#warning "Default value IX_EXCEPTION will be used."
	#define CLASSIFIER_6T_INV_IP_NEXT_BLOCK	     IX_EXCEPTION
#endif

#define CLASSIFIER_6T_NEXTHOP_ID_TYPE_MASK  0x0F
#define CLASSIFIER_6T_COLOR_ID_MASK         0x03

#ifdef CLASSIFIER_6T_PACKET_COUNTER_FEATURE
	#define CLASSIFIER_6T_PACKET_COUNTER_HIGH    0x0
	#define CLASSIFIER_6T_PACKET_COUNTER_LOW     0x4
	#define CLASSIFIER_6T_BYTE_COUNTER_HIGH      0x8
	#define CLASSIFIER_6T_BYTE_COUNTER_LOW       0xC

	#define CLASSIFIER_6T_STATISTICS_ENABLE_BIT      15
#endif

/*
 * TCP / UDP port are kept after IP header.
 * We assume that IP header has minimal length of 20 bytes
 * (it doesn't contain IP options because IP packets with options
 * are handled by core component)
 */
#define TCP_UDP_PORTS 					20, 4

#endif  /*  CLASSIFIER_6T_H  */

