/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILE
 *     dscp_classifier_defaults.h
 *
 * = DESCRIPTION
 *      The DSCP classifier core component internals. This file provides the
 *      static configuration for the DSCP CC. It is used when there is no 
 *      registry data available in the system.
 *
 * ============================================================================
 */

#if !defined( _DSCP_CLASSIFIER_DEFAULTS_H__INCLUDED_ )
#define _DSCP_CLASSIFIER_DEFAULTS_H__INCLUDED_

/**
 * Number of bytes occupied by a single entry in a configuration table. It is
 * used to calculate the size of the configuration table.
 */
#define CLASSIFIER_DSCP_CONFIG_ENTRY_SRAM_SIZE      8

/**
 * Number of bytes occupied by a single entry in a statistics table. It is
 * used to calculate the size of hash table.
 */
#define CLASSIFIER_DSCP_STATS_ENTRY_SRAM_SIZE       16

/**
 * Initial value of flow ID used to populate the configuration table at the 
 * core component initilization. It is applied to all the interfaces and DSCP
 * values.
 */
#define CLASSIFIER_DSCP_DEFAULT_FLOW_ID             0

/**
 * Initial value of class ID used to populate the configuration table at the
 * core component initialization. It is applied to all the interfaces and DSCP
 * values.
 */
#define CLASSIFIER_DSCP_DEFAULT_CLASS_ID            8

/**
 * Initial value of color ID used to populate the configuration table at the
 * core component initilization. It is applied to all the interfaces and DSCP
 * values (default color - green).
 */
#define CLASSIFIER_DSCP_DEFAULT_COLOR_ID            0 

/**
 * Initial value of the statistics flag used to populate the configuration
 * table at the core component initialization. 
 */
#define CLASSIFIER_DSCP_DEFAULT_STATS_ENABLE        0

/**
 * The number of input ports in the system. The value is used to calculate the
 * size of configuration and statistics.
 */
#define CLASSIFIER_DSCP_IN_PORTS                    16

#define CLASSIFIER_DSCP_SRAM_CHANNEL                0

#define CLASSIFIER_DSCP_MEv2_MASK                   4

#endif /* _DSCP_CLASSIFIER_DEFAULTS_H__INCLUDED_ */
