/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILE
 *     dscp_classifier_misc.h
 *
 * = DESCRIPTION
 *      The DSCP classifier core component internals. This file provides
 *      miscellanouse definitions.
 *
 * ============================================================================
 */

#if !defined( _DSCP_CLASSIFIER_MISC_H__INCLUDED_ )
#define _DSCP_CLASSIFIER_MISC_H__INCLUDED_

#if defined( __cplusplus )
extern "C" {
#endif /* __cplusplus */

/**
 *  NAME: _ix_cc_classifier_dscp_convert_a
 *
 *  DESCRIPTION: The function converts classification results passed by the
 *           calling application to the classification rules table entry.
 *
 *  @Param:  - OUT to    - pointer to the table entry to be filled.
 *  @Param:  - IN  from  - 64-element table of the classification results.
 *  @Param:  - IN  cntxt - pointer to the control block internal to the core
 *                         component.
 *
 *  @Return: IX_SUCCESS for successful or valid ix_error token for failure.
 */

ix_error 
_ix_cc_classifier_dscp_convert_a( _T( table_entry )            * to,
                                  ix_cc_classifier_dscp_result * from, 
                                  ix_uint32                      ipc,
                                  void                         * cntxt );

/**
 *  NAME: _ix_cc_classifier_dscp_convert_b
 *
 *  DESCRIPTION: The function converts classification rules table entry to the
 *           classification results.
 *
 *  @Param:  - OUT to    - pointer to the 64-element table of classification 
 *                         results.
 *  @Param:  - IN  from  - pointer to the classification rules table entry.
 *  @Param:  - IN  cntxt - pointer to the control block internal to the core
 *                         component.
 *
 *  @Return: IX_SUCCESS for successful or valid ix_error token for failure.
 */

ix_error 
_ix_cc_classifier_dscp_convert_b( ix_cc_classifier_dscp_result * to,
                                  _T( table_entry )            * from, 
                                  void                         * cntxt );

#if defined(CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE)

/**
 *  NAME: _ix_cc_classifier_dscp_duplicate_entry
 *
 *  DESCRIPTION: The function duplicates the specified classification rules 
 *           table entry. If statistics gathering is supported _flag_stats in 
 *           the destination entry has changed from 0 to 1the function resets 
 *           counters.
 *
 *  @Param:  - OUT dest  - pointer to the destination entry.
 *  @Param:  - IN  src   - pointer to the source entry.
 *  @Param:  - IN  cntxt - pointer to the control block internal to the core
 *                         component.
 *  @Param:  - OUT stats - pointer to the statistics table entry coresponding
 *                         to the destination entry.
 *
 *  @Return: IX_SUCCESS for successful or valid ix_error token for failure.
 */

ix_error
_ix_cc_classifier_dscp_duplicate_entry( _T( table_entry ) * dest,
                                        _T( table_entry ) * src,
                                        void              * cntxt,
                                        _T( stats_entry ) * stats );
#else

/**
 *  NAME: _ix_cc_classifier_dscp_duplicate_entry
 *
 *  DESCRIPTION: The function duplicates the specified classification rules 
 *           table entry. If statistics gathering is supported _flag_stats in 
 *           the destination entry has changed from 0 to 1the function resets 
 *           counters.
 *
 *  @Param:  - OUT dest  - pointer to the destination entry.
 *  @Param:  - IN  src   - pointer to the source entry.
 *  @Param:  - IN  cntxt - pointer to the control block internal to the core
 *                         component.
 *
 *  @Return: IX_SUCCESS for successful or valid ix_error token for failure.
 */

ix_error 
_ix_cc_classifier_dscp_duplicate_entry( _T( table_entry ) * dest,
                                        _T( table_entry ) * src, 
                                        void              * cntxt );
#endif 

/**
 *  NAME: _ix_cc_classifier_dscp_update_config
 */

ix_error 
_ix_cc_classifier_dscp_update_config( _T( config ) * cfg );

/**
 *  NAME: _ix_cc_classifier_dscp_register_pkt_handler
 */

ix_error 
_ix_cc_classifier_dscp_register_pkt_handlers( void * cntxt );

/**
 *  NAME: _ix_cc_classifier_dscp_register_msg_handler
 */

ix_error 
_ix_cc_classifier_dscp_register_msg_handler( void * cntxt );

/** 
 *  NAME: _ix_cc_classifier_dscp_unregister_msg_handler
 */

ix_error 
_ix_cc_classifier_dscp_unregister_msg_handler();

/**
 *  NAME: _ix_cc_classifier_dscp_unregister_pkt_handlers
 */

ix_error 
_ix_cc_classifier_dscp_unregister_pkt_handlers();

/**
 *  NAME: _ix_cc_classifier_dscp_free_memory
 */

ix_error 
_ix_cc_classifier_dscp_free_memory( void ** cntxt );

/**
 *  NAME: _ix_cc_classifier_dscp_update_entries
 */

ix_error 
_ix_cc_classifier_dscp_update_entries( void * cntxt );

/**
 *  NAME: _ix_cc_classifier_dscp_patch_symbols
 */

ix_error 
_ix_cc_classifier_dscp_patch_symbols( _T( context ) * cntxt );


#if defined(CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE)

/**
 *  NAME: _ix_cc_classifier_dscp_reset_counters
 */

ix_error 
_ix_cc_classifier_dscp_reset_counters( _T( stats_entry ) * stats,
                                       _T( table_entry ) * src, 
                                       void              * cntxt );
/**
 *  NAME: _ix_cc_classifier_dscp_zero_stats_entry
 */

ix_error
_ix_cc_classifier_dscp_reset_stats_entry( _T( stats_entry ) * entry );

#endif

#if defined( __cplusplus )
}
#endif /* __cplusplus */

#endif /* _DSCP_CLASSIFIER_MISC_H__INCLUDED_ */
