/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILE
 *     dscp_classifier_registry.h
 *
 * = DESCRIPTION
 *      The DSCP classifier core component internals. This file provides the
 *      dynamic configuration for the DSC CC. It is used when registry data is
 *      present in the system.
 *
 * ============================================================================
 */

#if !defined( _DSCP_CLASSIFIER_REGISTRY_H__INCLUDED_ )
#define _DSCP_CLASSIFIER_REGISTRY_H__INCLUDED_

/**
 * Number of bytes occupied by a single entry in a configuration table. It is
 * used to calculate the size of the configuration table.
 */
#define CLASSIFIER_DSCP_CONFIG_ENTRY_SRAM_SIZE \
  _ix_registry_get_uint32( "CLASSIFIER_DSCP/CONFIG_ENTRY_SRAM_SIZE" )

/**
 * Number of bytes occupied by a single entry in a statistics table. It is used
 * to calculate the size of hash table.
 */
#define CLASSIFIER_DSCP_STATS_ENTRY_SRAM_SIZE \
  _ix_registry_get_uint32( "CLASSIFIER_DSCP/STATS_ENTRY_SRAM_SIZE" )

/**
 * Initial value of flow identifier used to populate the configuration table at
 * the core component initialization.
 */
#define CLASSIFIER_DSCP_DEFAULT_FLOW_ID \
  _ix_registry_get_uint32( "CLASSIFIER_DSCP/DEFAULT_FLOW_ID" )

/**
 * Initial value of class identifier used to populate the configuration table
 * at the core component initialization.
 */
#define CLASSIFIER_DSCP_DEFAULT_CLASS_ID \
  _ix_registry_get_uint32( "CLASSIFIER_DSCP/DEFAULT_CLASS_ID" )

/**
 * Initial value of color identifier used to populate the configuration table
 * at the core comonent initialization.
 */
#define CLASSIFIER_DSCP_DEFAULT_COLOR_ID \
  _ix_registry_get_uint32( "CLASSIFIER_DSCP/DEFAULT_COLOR_ID" )

/**
 * Initial value of the statistics gathering flag used to populate the
 * configuration table at the core component initialization.
 */
#define CLASSIFIER_DSCP_DEFAULT_STATS_ENABLE \
  _ix_registry_get_uint32( "CLASSIFIER_DSCP/DEFAULT_STATS_ENABLE" )

/**
 * The number of input ports in the system. The value is used to calculate the
 * size of configuration and statistics tables.
 */
#define CLASSIFIER_DSCP_IN_PORTS \
  _ix_registry_get_uint32( "CLASSIFIER_DSCP/IN_PORTS" )

#define CLASSIFIER_DSCP_SRAM_CHANNEL \
  _ix_registry_get_uint32( "CLASSIFIER_DSCP/SRAM_CHANNEL_NUM" )

#define CLASSIFIER_DSCP_MEv2_MASK \
  _ix_registry_get_uint32( "CLASSIFIER_DSCP/MEV2_NUM_MASK" )

#if defined( __cplusplus )
extern "C" {
#endif /* __cplusplus */

/**
 * This function retrieves 32-bit unsigned integer from the system registry by
 * a property name.
 *
 * @param  arg_strPropName     name of a property in the system registry.
 *
 * @return If there is a property with the given name in the system registry,
 *         the function returns valid 32-bit unsigned integer. Otherwise, it
 *         return (ix_uint32)-1 value.
 */
ix_uint32 _ix_registry_get_uint32( const ix_uint8 * arg_strPropName );

#if defined( __cplusplus )
}
#endif /* __cplusplus */

#endif /* _DSCP_CLASSIFIER_REGISTRY_H__INCLUDED_ */
