/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILE
 *     ix_cc_dscp_classifier_api.c
 *
 * = DESCRIPTION
 *      DSCP Classifier Library API implementation.
 *
 * ============================================================================
 */

#define IX_ERROR_FILE_IDENT \
    "$Id: ix_cc_dscp_classifier_api.c,v 1.9.6.1 2004/03/12 09:25:36 wneubaue Exp $"



#include "cc/internal/dscp_classifier_internal.h"

/* -------------------------------------- _ix_cc_classifier_add_if_config -- */

ix_error 
ix_cc_classifier_dscp_add_if_config( ix_uint16 input_port,
                                     ix_cc_classifier_dscp_result * results, 
                                     void * cntxt )
  {             
    void * _t;

    /* validity check for cntxt argument */
    IX_ERROR_CHECK_ARG_NULL( 3, cntxt );
       
    /* validity check for input_port argument */
    IX_ERROR_CHECK_ARG_MAX ( 1, input_port, 
      (unsigned int)_CONFIG( cntxt )->_in_ports, "%u" );
    
    /* validity check for results argument */
    IX_ERROR_CHECK_ARG_NULL( 2, results );
  
    if ( __IS_SET( input_port, cntxt ) )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_DUPLICATE_ENTRY,
            ( "entry already has been configured" )
          );
      }
    
    _t = ix_ossl_malloc( TABLE_ENTRY_SIZE( cntxt ) );
    if ( _t == NULL )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_OOM_SYSTEM,
            ( "not enough free system memory available" )
          );
      }

    __SET_BIT( input_port, cntxt );

    /* convert and duplicate entry */
    RESULT_TO_ENTRY( _t, results, 1, cntxt );

#if defined(CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE)
    _ix_cc_classifier_dscp_duplicate_entry(
        _TABLE_ENTRY( cntxt, input_port ),
        _t, cntxt,
        _STATS_ENTRY( cntxt, input_port )
      );
#else 
    _ix_cc_classifier_dscp_duplicate_entry(
        _TABLE_ENTRY( cntxt, input_port ), 
        _t, cntxt );
#endif 

    ix_ossl_free( _t );
     return IX_SUCCESS;
  }

/* ------------------------------- ix_cc_classifier_dscp_update_if_config -- */

ix_error 
ix_cc_classifier_dscp_update_if_config( ix_uint16 input_port,
                                        ix_cc_classifier_dscp_result * results, 
                                        void * cntxt )
  {
    void * _t;

    /* validity check for cntxt argument */
    IX_ERROR_CHECK_ARG_NULL( 3, cntxt );
    
    /* validity check for input_port argument */
    IX_ERROR_CHECK_ARG_MAX ( 1, input_port, 
        (unsigned int)_CONFIG( cntxt )->_in_ports, "%u" );
    
    /* validity check for results */
    IX_ERROR_CHECK_ARG_NULL( 2, results );
            
    if ( !__IS_SET( input_port, cntxt ) )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_DUPLICATE_ENTRY,
            ( "entry has not been configured yet" )
          );
      }
    
    _t = ix_ossl_malloc( TABLE_ENTRY_SIZE( cntxt ) );
    if ( _t == NULL )
      { 
        return IX_ERROR_WARNING(
            IX_CC_ERROR_OOM_SYSTEM,
            ( "not enough free system memory available" )
          );
      }
    
    /* convert and duplicate entry */
    RESULT_TO_ENTRY( _t, results, 1, cntxt );

#if defined(CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE)
    _ix_cc_classifier_dscp_duplicate_entry( 
        _TABLE_ENTRY( cntxt, input_port ),
        _t, cntxt, 
        _STATS_ENTRY( cntxt, input_port ) 
      );
#else 
    _ix_cc_classifier_dscp_duplicate_entry(
        _TABLE_ENTRY( cntxt, input_port ), 
        _t, cntxt );
#endif

    ix_ossl_free( _t );
     return IX_SUCCESS;
  }

/* ------------------------------- ix_cc_classifier_dscp_remove_if_config -- */

ix_error 
ix_cc_classifier_dscp_remove_if_config( ix_uint16 input_port, 
                                        void * cntxt )
  {
    /* validity check for cntxt argument */
    IX_ERROR_CHECK_ARG_NULL( 2, cntxt );

    /* validity check for input_port argument */
    IX_ERROR_CHECK_ARG_MAX ( 1, input_port, 
        (unsigned int)_CONFIG( cntxt )->_in_ports, "%u" );

    if ( !__IS_SET( input_port, cntxt ) )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_ENTRY_NOT_FOUND,
            ( "interface-specific configuration for the port #%d has not " \
              "been defined yet", input_port )
          );
      }
    
    __RESET_BIT( input_port, cntxt );
        
#if defined(CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE)
    _ix_cc_classifier_dscp_duplicate_entry( 
        _TABLE_ENTRY( cntxt, input_port ), 
        _CONTEXT( cntxt )->_dscp_default_rules, 
        cntxt,
        _STATS_ENTRY( cntxt, input_port )
      );
#else
    _ix_cc_classifier_dscp_duplicate_entry(
        _TABLE_ENTRY( cntxt, input_port ),
        _CONTEXT( cntxt )->_dscp_default_rules,
        cntxt );
#endif

     return IX_SUCCESS;
  }
     
/* ---------------------------------- ix_cc_classifier_dscp_get_if_config -- */

ix_error 
ix_cc_classifier_dscp_get_if_config( ix_uint16 input_port, 
                                     ix_cc_classifier_dscp_result * results, 
                                     void * cntxt )
  {
    /* validity check for cntxt argument */
    IX_ERROR_CHECK_ARG_NULL( 3, cntxt );

    /* validity check for input_port argument */
    IX_ERROR_CHECK_ARG_MAX ( 1, input_port, 
        (unsigned int)_CONFIG( cntxt )->_in_ports, "%u" );

    /* validity check for results argument */
    IX_ERROR_CHECK_ARG_NULL( 2, results );

    if ( !__IS_SET( input_port, cntxt ) )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_ENTRY_NOT_FOUND,
            ( "interface-specific configuration for the port #%d has not " \
              "been defined yet", input_port )
          );
      }

    ENTRY_TO_RESULT( results, _TABLE_ENTRY( cntxt, input_port ), cntxt );
    return IX_SUCCESS;
  }

/* ---------------------------------- ix_cc_classifier_dscp_set_def_rules -- */

ix_error 
ix_cc_classifier_dscp_set_def_rules( ix_cc_classifier_dscp_result * results, 
                                     void * cntxt )
  {   
    /* validity check for results argument */
    IX_ERROR_CHECK_ARG_NULL( 1, results );

    /* validity check for cntxt argument */
    IX_ERROR_CHECK_ARG_NULL( 2, cntxt );

    RESULT_TO_ENTRY( _CONTEXT( cntxt )->_dscp_default_rules, results, 0, cntxt );
    return _ix_cc_classifier_dscp_update_entries( cntxt );
  }

/* ---------------------------------- ix_cc_classifier_dscp_get_def_rules -- */

ix_error 
ix_cc_classifier_dscp_get_def_rules( ix_cc_classifier_dscp_result * results, 
                                     void * cntxt )
  {
    /* validity check for cntxt argument */
    IX_ERROR_CHECK_ARG_NULL( 2, cntxt );
    
    /* validity check for results argument */
    IX_ERROR_CHECK_ARG_NULL( 1, results );
       
    ENTRY_TO_RESULT( results, _CONTEXT( cntxt )->_dscp_default_rules, cntxt );
    return IX_SUCCESS;
  }

/* --------------------------------- ix_cc_classifier_dscp_get_statistics -- */

ix_error 
ix_cc_classifier_dscp_get_statistics( ix_uint16 input_port,
                                      ix_uint8 dscp, 
                                      ix_cc_classifier_dscp_statistics * stats, 
                                      void * cntxt )
  {       
#if defined( CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE )

    /* validity check for cntxt argument */
    IX_ERROR_CHECK_ARG_NULL( 4, cntxt );
    
    /* validity check for stats argument */
    IX_ERROR_CHECK_ARG_NULL( 3, stats );

    /* validity check for dscp argument */
    IX_ERROR_CHECK_ARG_MAX ( 2, dscp, DSCP_MAX_VALUE, "%d" );
    
    /* validity check for input_port argument */
    IX_ERROR_CHECK_ARG_MAX ( 1, input_port, 
        (unsigned int)_CONFIG( cntxt )->_in_ports, "%u" );
     
    ix_ossl_memcpy( stats, 
        _STATS_ENTRY( cntxt, input_port ) + dscp,
        _SIZE_OF( stats_entry ) 
      );

    return IX_SUCCESS;

#else 

    return IX_ERROR_WARNING(
        IX_CC_CLASSIFIER_DSCP_ERROR_STATS_NOT_SUPPORTED,
        ( "statistics gathering is not supported by the block" )
      );

#endif
  }
