/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILE
 *     ix_cc_dscp_classifier_debug.c
 *
 * = DESCRIPTION
 *     Diagnostics support implementation.
 *
 * ============================================================================
 */

#define IX_ERROR_FILE_IDENT \
  "$Id: ix_cc_dscp_classifier_debug.c,v 1.2.6.1 2004/03/12 09:25:36 wneubaue Exp $"



/* entire file */
#if defined (IX_DEBUG)

#include "cc/internal/dscp_classifier_internal.h"

/* ------------------------------------------------------- _ix_debug_dump -- */

void 
_ix_debug_dump( void * ptr, 
                ix_ossl_size_t size )
  {
    ix_uint8  buffer[17];
    ix_uint32 _j;    
    
    ix_ossl_memset( buffer, 0, 17 );

    _j = 0;    
    while ( _j < size )
      {
        ix_uint8 _t;

        /* break the line if necessary */
        if ( (_j & 0xf) == 0 )
          {
            if ( *buffer != 0 )
              {
                printf( "| %s\n", buffer ); 
              }

            printf( "0x%08x | ", ptr );
          }
        else if ( (_j & 0x7) == 0 )
          {
            printf( " " );
          }

        printf( "%02x ", (_t = *(ix_uint8 *)ptr) );

        if ( isprint( _t ) )
          {
            buffer[_j & 0xf] = _t;
          }
        else
          {
            buffer[_j & 0xf] = '.';
          }

        ((ix_uint8 *)ptr)++;
        _j += 1;                
      }

    if ( (_j = size & 0xf) != 0 )
      {
        for ( ; _j < 16; _j++ )
          {
            printf( "   " ); buffer[_j] = ' ';
          }
      }

    printf( "| %s\n", buffer );    
  }

/* ------------------------------------------------- _ix_debug_dump_rslts -- */

ix_error 
_ix_debug_dump_rslts( ix_cc_classifier_dscp_result * arg_pResults )
  {
    ix_uint32 _i;

    /* validity check for arg_pResults */
    IX_ERROR_CHECK_ARG_NULL( 1, arg_pResults );

    {
      ix_cc_classifier_dscp_result * _t;

      for ( _i = 0; _i < DSCP_MAX_VALUE; _i++ )
        {
          _t = &arg_pResults[_i];

          ix_ossl_message_log( "dumping classification results at 0x%08x " \
              "(dscp = %02d)\n", (ix_uint32)arg_pResults + _i, _i );

          ix_ossl_message_log( "\tclassId   : %d\n",
            _t->classId );
          ix_ossl_message_log( "\tflowId    : %d\n",
            _t->flowId );
          ix_ossl_message_log( "\tcolorId   : %d\n",
            _t->colorId );
          ix_ossl_message_log( "\tremark    : %s\n",
            (_t->remark)     ? "SET" : "-" );
          ix_ossl_message_log( "\tstatistics: %s\n",
            (_t->statistics) ? "SET" : "-" );
          ix_ossl_message_log( "\tTC        : %s\n",
            (_t->TC)         ? "SET" : "-" );
       }
    }

    return IX_SUCCESS;
  }

/* ------------------------------------------------- _ix_debug_dump_entry -- */

ix_error 
_ix_debug_dump_entry( _T( table_entry ) * entry,  
                      void * cntxt )
  {
    ix_uint32 _i;

    /* validity check for arg_pEntry */
    IX_ERROR_CHECK_ARG_NULL( 1, entry );

    {
      _T( table_entry ) * _t = entry;

      for ( _i = 0; _i < DSCP_MAX_VALUE; _i++ )
        {
          ix_ossl_message_log( "dumping classification rules at 0x%08x " \
            "(dscp = %02d)\n", _t, _i );

          ix_ossl_message_log( "\t_flag_ipc       : %s\n",
            (_t->_flag_ipc)  ? "SET" : "-" );
          ix_ossl_message_log( "\t_flag_statistics: %s\n",
            (_t->_flag_stats) ? "SET" : "-" );
          ix_ossl_message_log( "\t_flag_remark    : %s\n",
            (_t->_flag_remark) ? "SET" : "-" );
          ix_ossl_message_log( "\t_flag_tc        : %s\n",
            (_t->_flag_tc) ? "SET" : "-" );
          ix_ossl_message_log( "\t_id_class       : %d\n",
            _t->_id_class );
          ix_ossl_message_log( "\t_id_flow        : %d\n",
            _t->_id_flow );
          ix_ossl_message_log( "\t_id_color       : %d\n",
            _t->_id_color );

          ((ix_uint8 *)_t) += _CONFIG( cntxt )->_table_entry_size;
        }
    }

    return IX_SUCCESS;
  }

#endif /* IX_DEBUG */

