/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILE
 *     ix_cc_dscp_classifier_fini.c
 *
 * = DESCRIPTION
 *     This file contains implementation of the <code>fini</code> function of
 *     the DSCP Classifier Core Component.
 *
 * ============================================================================
 */

#define IX_ERROR_FILE_IDENT \
    "$Id: ix_cc_dscp_classifier_fini.c,v 1.2.6.1 2004/03/12 09:25:36 wneubaue Exp $"


#include "cc/internal/dscp_classifier_internal.h"

/* ------------------------------------------- ix_cc_classifier_dscp_fini -- */

ix_error ix_cc_classifier_dscp_fini( ix_cc_handle handle, void * cntxt )
  {
    /* validity check for the handle argument */
    if ( handle == IX_NULL_HANDLE )
      {
        return IX_ERROR_WARNING(
            IX_CC_CLASSIFIER_DSCP_ERROR_INVALID_HANDLE,
            ( "null handle specified" )
          );
      }

    /* validity check for the cntxt argument */
    IX_ERROR_CHECK_ARG_NULL( 2, cntxt );

    if ( handle != _CONTEXT( cntxt )->_handle )
      {
        return IX_ERROR_WARNING(
            IX_CC_CLASSIFIER_DSCP_ERROR_INVALID_HANDLE,
            ( "invalid handle specified" )
          );
      }
    
    /* free used memory blocks */
    IX_ERROR_CR( _ix_cc_classifier_dscp_free_memory( &cntxt ) );

    /* unregister packet handlers */
    IX_ERROR_CR( _ix_cc_classifier_dscp_unregister_pkt_handlers() );

    /* unregister message handler */
    IX_ERROR_CR( _ix_cc_classifier_dscp_unregister_msg_handler() );

    return IX_SUCCESS;
  }
