/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILE
 *     ix_cc_dscp_classifier_misc.c
 *
 * = DESCRIPTION
 *     Miscellanouse functions implementation.
 *
 * ============================================================================
 */


#define IX_ERROR_FILE_IDENT \
    "$Id: ix_cc_dscp_classifier_misc.c,v 1.13.4.1 2004/03/12 09:25:35 wneubaue Exp $"


#include "ix_ossl.h"
#include "ix_error.h"
#include "bindings.h"
#include "cc/internal/dscp_classifier_internal.h"

/* -------------------------------------- _ix_cc_classifier_dscp_covert_a -- */

ix_error
_ix_cc_classifier_dscp_convert_a( _T( table_entry )            * to,
                                  ix_cc_classifier_dscp_result * from,
                                  ix_uint32                      ipc,
                                  void                         * cntxt )
  {
    ix_uint32 _i;

    /* validity check for cntxt argument */
    IX_ERROR_CHECK_ARG_NULL( 3, cntxt );

    /* validity check for to argument */
    IX_ERROR_CHECK_ARG_NULL( 1, to );

    /* validity check for from argument */
    IX_ERROR_CHECK_ARG_NULL( 2, from );

    for ( _i = 0; _i < DSCP_MAX_VALUE; _i++ )
      {
        if ( _MUTEX_LOCK( cntxt ) == IX_SUCCESS )
          {
            to->_flag_ipc    = _BIT_FLD( ipc );
            to->_flag_remark = _BIT_FLD( from->remark );
            to->_flag_tc     = _BIT_FLD( from->TC );
            to->_flag_stats  = _BIT_FLD( from->statistics );
            to->_id_class    = from->classId;
            to->_id_flow     = from->flowId;
            to->_id_color    = from->colorId;
            _MUTEX_UNLOCK( cntxt );
          }

        ((ix_uint8 *)to)   +=
            _CONFIG( cntxt )->_table_entry_size;

        ((ix_uint8 *)from) +=
            sizeof( ix_cc_classifier_dscp_result );
     }

    return IX_SUCCESS;
  }

/* ------------------------------------- _ix_cc_classifier_dscp_convert_b -- */

ix_error
_ix_cc_classifier_dscp_convert_b( ix_cc_classifier_dscp_result * to,
                                  _T( table_entry )            * from,
                                  void                         * cntxt )
  {
    ix_uint32 _i;

    /* validity check for cntxt argument */
    IX_ERROR_CHECK_ARG_NULL( 3, cntxt );

    /* validity check for to argument */
    IX_ERROR_CHECK_ARG_NULL( 1, to );

    /* validity check for from argument */
    IX_ERROR_CHECK_ARG_NULL( 2, from );

    for ( _i = 0; _i < DSCP_MAX_VALUE; _i++ )
      {
        if ( _MUTEX_LOCK( cntxt ) == IX_SUCCESS )
          {
            to->classId      = (ix_uint16)from->_id_class;
            to->flowId       = from->_id_flow;
            to->colorId      = (ix_uint8 )from->_id_color;
            to->remark       = (ix_uint8 )from->_flag_remark;
            to->TC           = (ix_uint8 )from->_flag_tc;
            to->statistics   = (ix_uint8 )from->_flag_stats;

            _MUTEX_UNLOCK( cntxt );
          }

        ((ix_uint8 *)to)   +=
            sizeof( ix_cc_classifier_dscp_result );

        ((ix_uint8 *)from) +=
            _CONFIG( cntxt )->_table_entry_size;
      }

    return IX_SUCCESS;
  }

#if defined(CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE)

/* ----------------------------- _ix_cc_classifier_dscp_reset_stats_entry -- */

ix_error
_ix_cc_classifier_dscp_reset_stats_entry( _T( stats_entry ) * entry )
  {
    /* validity check for the entry argument */
    IX_ERROR_CHECK_ARG_NULL( 1, entry );

    /* resetting packet counter */

    ix_rm_atomic_sram_subtract( 0x7fffffffL,
        &((_ix_64bit_counter *)entry)->_l );

    ix_rm_atomic_sram_subtract( 0x7fffffffL,
        &((_ix_64bit_counter *)entry)->_h );

    ((ix_uint8 *)entry) += sizeof( ix_uint64 );

    /* resetting byte counter */

    ix_rm_atomic_sram_subtract( 0x7fffffffL,
        &((_ix_64bit_counter *)entry)->_l );
    ix_rm_atomic_sram_subtract( 0x7fffffffL,
        &((_ix_64bit_counter *)entry)->_h );

    return IX_SUCCESS;
  }

#endif

/* ------------------------------- _ix_cc_classifier_dscp_duplicate_entry -- */

#if defined(CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE)

ix_error
_ix_cc_classifier_dscp_duplicate_entry( _T( table_entry ) * dest,
                                        _T( table_entry ) * src,
                                        void              * cntxt,
                                        _T( stats_entry ) * stats )
#else CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE

ix_error
_ix_cc_classifier_dscp_duplicate_entry( _T( table_entry ) * dest,
                                        _T( table_entry ) * src,
                                        void              * cntxt )
#endif
  {
    ix_uint32 _i;

    /* validity check for cntxt argument */
    IX_ERROR_CHECK_ARG_NULL( 3, cntxt );

    /* validity check for src argument */
    IX_ERROR_CHECK_ARG_NULL( 2, src );

    /* validity check for dest argument */
    IX_ERROR_CHECK_ARG_NULL( 1, dest );

    for ( _i = 0; _i < DSCP_MAX_VALUE; _i++ )
      {
#if defined(CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE)
        /* reset packet and byte counters if flag changed */
        if ( !dest->_flag_stats && src->_flag_stats )
          {
            _ix_cc_classifier_dscp_reset_stats_entry( stats );
          }
#endif

        /* copy classification rules for the particular dscp entry */
        if ( _MUTEX_LOCK( cntxt ) == IX_SUCCESS )
          {
            ix_ossl_memcpy( dest, src, _SIZE_OF( table_entry ) );
            _MUTEX_UNLOCK( cntxt );
          }

        ((ix_uint8 *)dest)  +=
            _CONFIG( cntxt )->_table_entry_size;

        ((ix_uint8 *)src )  +=
            _CONFIG( cntxt )->_table_entry_size;

#if defined(CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE)
        ((ix_uint8 *)stats) +=
            _CONFIG( cntxt )->_stats_entry_size;
#endif
      }

    return IX_SUCCESS;
  }

#if defined(CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE)

/* -------------------------------- _ix_cc_classifier_dscp_reset_counters -- */

ix_error
_ix_cc_classifier_dscp_reset_counters( _T( stats_entry ) * stats,
                                       _T( table_entry ) * src,
                                       void              * cntxt )
  {
    ix_uint32 _i;

    /* validity check for cntxt argument */
    IX_ERROR_CHECK_ARG_NULL( 3, cntxt );

    /* validity check for src argument */
    IX_ERROR_CHECK_ARG_NULL( 2, src );

    /* validity check for stats argument */
    IX_ERROR_CHECK_ARG_NULL( 1, stats );

    for ( _i = 0; _i < DSCP_MAX_VALUE; _i++ )
      {
        if ( src->_flag_stats )
          {
            _ix_cc_classifier_dscp_reset_stats_entry( stats );
          }

        ((ix_uint8 *)(stats)) +=
          _CONFIG( cntxt )->_stats_entry_size;

        ((ix_uint8 *)(src  )) +=
          _CONFIG( cntxt )->_table_entry_size;
      }

    return IX_SUCCESS;
  }

#endif

/* -------------------------------- _ix_cc_classifier_dscp_update_entries -- */

ix_error
_ix_cc_classifier_dscp_update_entries( void * cntxt )
  {
    ix_uint32 _i;

    /* validity check for cntxt argument */
    IX_ERROR_CHECK_ARG_NULL( 1, cntxt );

    for ( _i = 0; _i < _CONFIG( cntxt )->_in_ports; _i++ )
      {
        if ( !__IS_SET( _i, cntxt ) )
          {
#if defined(CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE)
            _ix_cc_classifier_dscp_duplicate_entry(
                _TABLE_ENTRY( cntxt, _i ),
                _CONTEXT( cntxt )->_dscp_default_rules,
                cntxt,
                _STATS_ENTRY( cntxt, _i )
              );
#else /* CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE */
            _ix_cc_classifier_dscp_duplicate_entry(
                _TABLE_ENTRY( cntxt, _i ),
                _CONTEXT( cntxt )->_dscp_default_rules,
                cntxt );
#endif /* !CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE */
          }
      }

    return IX_SUCCESS;
  }

/* --------------------------------- _ix_cc_classifier_dscp_update_config -- */

ix_error
_ix_cc_classifier_dscp_update_config( _T( config ) * cfg )
  {
    /* validity check for cfg agument */
    IX_ERROR_CHECK_ARG_NULL( 1, cfg );

    cfg->_default_class_id      = CLASSIFIER_DSCP_DEFAULT_CLASS_ID;
    cfg->_default_flow_id       = CLASSIFIER_DSCP_DEFAULT_FLOW_ID;
    cfg->_default_color_id      = CLASSIFIER_DSCP_DEFAULT_COLOR_ID;
    cfg->_default_flag_ipc      = 0;
    cfg->_default_flag_remark   = 0;
    cfg->_default_flag_tc       = 0;
    cfg->_default_flag_stats    = CLASSIFIER_DSCP_DEFAULT_STATS_ENABLE;

    cfg->_mev2_mask             = CLASSIFIER_DSCP_MEv2_MASK;
    cfg->_in_ports              = CLASSIFIER_DSCP_IN_PORTS;
    cfg->_sram_channel          = CLASSIFIER_DSCP_SRAM_CHANNEL;

#if defined( CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE )

    cfg->_stats_entry_size      = CLASSIFIER_DSCP_STATS_ENTRY_SRAM_SIZE;
    if ( cfg->_stats_entry_size < _SIZE_OF( stats_entry ) )
      {
        return IX_ERROR_WARNING(
            IX_CC_CLASSIFIER_DSCP_ERROR_INVALID_ENTRY_SIZE,
            ( "invalid size of an entry of the statistics table read from " \
              "the system registry" )
          );
      }

#endif /* CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE */

    cfg->_table_entry_size      = CLASSIFIER_DSCP_CONFIG_ENTRY_SRAM_SIZE;
    if ( cfg->_table_entry_size < _SIZE_OF( table_entry ) )
      {
        return IX_ERROR_WARNING(
            IX_CC_CLASSIFIER_DSCP_ERROR_INVALID_ENTRY_SIZE,
            ( "invalid size of an entry of the configuration table read from "
              "the system registry" )
          );
      }

    return IX_SUCCESS;
  }

/* ----------------------------------- _ix_cc_classifier_dscp_free_memory -- */

ix_error
_ix_cc_classifier_dscp_free_memory( void ** cntxt )
  {
    _T( context ) * _t;

    /* validity check for arg_pContext */
    IX_ERROR_CHECK_ARG_NULL( 1, cntxt );

    _t = (_T( context ) *)(*cntxt);
    if ( _t == NULL )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_NULL,
            ( "null pointer to the control block specified" )
          );
      }

    /* bitmap */
    if ( _t->_cfg_bitmap != NULL )
      {
        ix_ossl_free( (void *)_t->_cfg_bitmap );
      }

    /* configuration */
    if ( _t->_config != NULL )
      {
        ix_ossl_free( (void *)_t->_config );
      }

    /* default rules */
    if ( _t->_dscp_default_rules != NULL )
      {
        ix_ossl_free( (void *)_t->_dscp_default_rules );
      }

#if defined( CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE )

    /* statistics table */
    if ( _t->_dscp_stats_sram_base != NULL )
      {
        ix_rm_mem_free( _t->_dscp_stats_sram_base );
      }

#endif /* CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE */

    /* configuration table */
    if ( _t->_dscp_table_sram_base != NULL )
      {
        ix_rm_mem_free( _t->_dscp_table_sram_base );
      }

    /* mutex */
    if ( _t->_mutex != IX_NULL_HANDLE )
      {
        ix_ossl_mutex_fini( _t->_mutex );
      }

    ix_ossl_free( (void *)*cntxt );
    *cntxt = NULL;

    return IX_SUCCESS;
  }

/* --------------------------------- _ix_cc_classifier_dscp_patch_symbols -- */

ix_error _ix_cc_classifier_dscp_patch_symbols( _T( context ) * cntxt )
  {
    ix_imported_symbol _sym[CLASSIFIER_DSCP_PATCH_SYMBOLS];

    ix_error           _result;
    ix_uint32          _mem_channel;
    ix_memory_type     _mem_type;
    ix_uint32          _mem_offset;
    void *             _phy_address;
    ix_uint32          _i, _j;

    /* configuration table base address */

    _result = ix_rm_get_phys_offset( cntxt->_dscp_table_sram_base, &_mem_type,
        &_mem_channel, &_mem_offset, &_phy_address );

    if ( _result != IX_SUCCESS )
      {
        return IX_ERROR_WARNING(
            IX_CC_CLASSIFIER_DSCP_ERROR_PATCH_SYMBOLS,
            ( "cannot convert virtual address to offset" )
          );
      }

    IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET( _mem_channel, _mem_offset );

    _sym[0].m_Value = _mem_offset;
    _sym[0].m_Name  = CLASSIFIER_DSCP_CONFIG_SRAM_BASE;

#if defined( CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE )

    /* statistics table base address */

    _result = ix_rm_get_phys_offset( cntxt->_dscp_stats_sram_base, &_mem_type,
        &_mem_channel, &_mem_offset, &_phy_address );

    if ( _result != IX_SUCCESS )
      {
        return IX_ERROR_WARNING(
            IX_CC_CLASSIFIER_DSCP_ERROR_PATCH_SYMBOLS,
            ( "cannot convert virtual address to offset" )
          );
      }

    IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET( _mem_channel, _mem_offset );

    _sym[1].m_Value = _mem_offset;
    _sym[1].m_Name  = CLASSIFIER_DSCP_STATS_SRAM_BASE;

#endif /* CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE */

    _i = _CONFIG( cntxt )->_mev2_mask; _j = 0;
    while ( _i != 0 )
      {
        if ( _i & 0x01 )
          {
            ix_rm_ueng_patch_symbols( g_aMicroengineNumberMapping[_j],
                CLASSIFIER_DSCP_PATCH_SYMBOLS, _sym );
          }

        _i >>= 1; _j++;
      }

    return IX_SUCCESS;
  }

/* -------------------------- _ix_cc_classifier_dscp_register_msg_handler -- */

ix_error _ix_cc_classifier_dscp_register_msg_handler( void * arg_pContext )
  {
    ix_error _result;

#if defined( IX_EXCLUDE_CCI )
    _result = ix_rm_message_handler_register(
        IX_CC_CLASSIFIER_DSCP_MSG_INPUT,
        ix_cc_classifier_dscp_msg_handler,
        arg_pContext );

    if ( _result != IX_SUCCESS )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_INTERNAL,
            ( "cannot register message handler" )
          );
      }
#else /* IX_EXCLUDE_CCI */
    _result = ix_cci_cc_add_message_handler(
        _CONTEXT( arg_pContext )->_handle,
        IX_CC_CLASSIFIER_DSCP_MSG_INPUT,
        ix_cc_classifier_dscp_msg_handler,
        IX_INPUT_TYPE_MULTI_SRC );

    if ( _result != IX_SUCCESS )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_INTERNAL,
            ( "cannot register message handler" )
          );
      }
#endif /* !IX_EXCLUDE_CCI */

    return _result;
  }

/* ------------------------ _ix_cc_classifier_dscp_unregister_msg_handler -- */

ix_error _ix_cc_classifier_dscp_unregister_msg_handler()
  {
    ix_error _result;

#if defined( IX_EXCLUDE_CCI )
    _result = ix_rm_message_handler_unregister(
        IX_CC_CLASSIFIER_DSCP_MSG_INPUT );

    if ( _result != IX_SUCCESS )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_INTERNAL,
            ( "cannot unregister message handler" )
          );
      }
#else /* IX_EXCLUDE_CCI */
    _result = ix_cci_cc_remove_message_handler(
        IX_CC_CLASSIFIER_DSCP_MSG_INPUT );

    if ( _result != IX_SUCCESS )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_INTERNAL,
            ( "cannot unregister message handler" )
          );
      }
#endif /* !IX_EXCLUDE_CCI */

    return _result;
  }

/* ------------------------- _ix_cc_classifier_dscp_register_pkt_handlers -- */

ix_error _ix_cc_classifier_dscp_register_pkt_handlers( void * arg_pContext )
  {
    ix_error _result;

#if defined( IX_EXCLUDE_CCI )
    _result = ix_rm_packet_handler_register(
        IX_CC_CLASSIFIER_DSCP_LOCAL_PKT_INPUT,
        ix_cc_classifier_dscp_pkt_handler, arg_pContext );

    if ( _result != IX_SUCCESS )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_INTERNAL,
            ( "cannot register packet handler" )
          );
      }
#else /* IX_EXCLUDE_CCI */
    _result = ix_cci_cc_add_packet_handler(
        _CONTEXT( arg_pContext )->_handle,
        IX_CC_CLASSIFIER_DSCP_LOCAL_PKT_INPUT,
        ix_cc_classifier_dscp_pkt_handler,
        IX_INPUT_TYPE_SINGLE_SRC );

    if ( _result != IX_SUCCESS )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_INTERNAL,
            ( "cannot register packet handler" )
          );
      }
#endif /* !IX_EXCLUDE_CCI */

    return _result;
  }

/* ----------------------- _ix_cc_classifier_dscp_unregister_pkt_handlers -- */

ix_error _ix_cc_classifier_dscp_unregister_pkt_handlers()
  {
    ix_error _result;

#if defined( IX_EXCLUDE_CCI )
    _result = ix_rm_packet_handler_unregister(
        IX_CC_CLASSIFIER_DSCP_LOCAL_PKT_INPUT );

    if ( _result != IX_SUCCESS )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_INTERNAL,
            ( "cannot unregister packet handler" )
          );
      }
#else /* IX_EXCLUDE_CCI */
    _result = ix_cci_cc_remove_packet_handler(
        IX_CC_CLASSIFIER_DSCP_LOCAL_PKT_INPUT );

    if ( _result != IX_SUCCESS )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_INTERNAL,
            ( "cannot unregister packet handler" )
          );
      }
#endif /* !IX_EXCLUDE_CCI */

    return _result;
  }
