/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILE
 *     ix_cc_dscp_classifier_msg_handler.c
 *
 * = DESCRIPTION
 *     Message handler implementation.
 *
 * ============================================================================
 */

#define IX_ERROR_FILE_IDENT \
  "$Id: ix_cc_dscp_classifier_msg_handler.c,v 1.7.6.1 2004/03/12 09:25:35 wneubaue Exp $"


#include "cc/internal/dscp_classifier_internal.h"


/* ------------------------------------------- ix_cc_classifier_dscp_init -- */

ix_error ix_cc_classifier_dscp_msg_handler( ix_cc_handle   arg_hBuffer,
                                            ix_uint32      arg_nMsgType,
                                            void         * arg_pContext )
  {    
    void *                           _pCntxt;
    void *                           _pMsg;
    void *                           _pRetMsg       = NULL;
    ix_uint32                        _nRetMsgLength = 0;        
    ix_cc_classifier_dscp_statistics _pStats;
    ix_cc_classifier_dscp_result     _dscp_result[64];

    ix_error                         _result        = IX_SUCCESS;

    /* validity check for the arg_pContext argument */
    IX_ERROR_CHECK_ARG_NULL( 3, arg_pContext );

    /* validity check for the arg_nMsgType argument */
    IX_ERROR_CHECK_ARG_RANGE( 2, (unsigned int)arg_nMsgType, 
      IX_CC_COMMON_MSG_ID_FIRST, IX_CC_CLASSIFIER_DSCP_MSG_LAST, "%u" );

    /* extracting the message from buffer */
    IX_MSUP_EXTRACT_MSG( arg_hBuffer, &_pMsg, &_pCntxt );
    if ( _pMsg == NULL )
      {
        if ( arg_nMsgType != IX_CC_CLASSIFIER_DSCP_MSG_GET_DEFAULT_RULES )
          {
            return IX_ERROR_WARNING(  
                IX_CC_ERROR_NULL,
                ( "cannot extract the message data from a buffer" )
              );
          }
      }

    /* handle the message depending on its type */
    switch ( arg_nMsgType )
      {
      case IX_CC_CLASSIFIER_DSCP_MSG_ADD_IF_CONFIG:
        {
          _CLASSIFIER_DSCP_ADD_IF_CONFIG( _pMsg, arg_pContext, _result );
          break;
        }
      case IX_CC_CLASSIFIER_DSCP_MSG_UPDATE_IF_CONFIG:
        {
          _CLASSIFIER_DSCP_UPDATE_IF_CONFIG( _pMsg, arg_pContext, _result );
          break;
        }
      case IX_CC_CLASSIFIER_DSCP_MSG_GET_IF_CONFIG:
        {
          _CLASSIFIER_DSCP_GET_IF_CONFIG( _pMsg, _dscp_result, arg_pContext, _result );
          if ( _result == IX_SUCCESS )
            {
              _nRetMsgLength = sizeof(ix_cc_classifier_dscp_result) * 64;
              _pRetMsg       = _dscp_result;
            }
          break;
        }
      case IX_CC_CLASSIFIER_DSCP_MSG_REMOVE_IF_CONFIG:
        {
          _CLASSIFIER_DSCP_REMOVE_IF_CONFIG( _pMsg, arg_pContext, _result );
          break;
        }
      case IX_CC_CLASSIFIER_DSCP_MSG_SET_DEFAULT_RULES:
        {
          _CLASSIFIER_DSCP_SET_DEF_RULES( _pMsg, arg_pContext, _result );
          break;
        }
      case IX_CC_CLASSIFIER_DSCP_MSG_GET_DEFAULT_RULES:
        {
          _CLASSIFIER_DSCP_GET_DEF_RULES( _dscp_result, arg_pContext, _result );
          if ( _result == IX_SUCCESS )
            {
              _nRetMsgLength = sizeof( ix_cc_classifier_dscp_result ) * 64;
              _pRetMsg       = _dscp_result;
            }
          break;
        }
      case IX_CC_CLASSIFIER_DSCP_MSG_GET_STATISTICS:
        {

          _CLASSIFIER_DSCP_GET_STATISTICS( _pMsg, arg_pContext, &_pStats, _result );
          if ( _result == IX_SUCCESS )
          {
              _nRetMsgLength = sizeof( ix_cc_classifier_dscp_statistics );
              _pRetMsg       = &_pStats;
          } else
              ix_error_dump(stderr, _result);
          break;
        }
      default:
        {
          IX_ERROR_CRT( ix_rm_buffer_free( arg_hBuffer ), 
            IX_CC_ERROR_INTERNAL, 0 );

          return IX_ERROR_WARNING(
              IX_CC_ERROR_UNDEFINED_MSG,
              ( "unsupported message type specified" )
            );
        }
      }

    /* send reply message to the client application */
    _result = ix_cc_msup_send_reply_msg(
      _pCntxt, _pRetMsg, _nRetMsgLength, _result );

    if ( _result != IX_SUCCESS )
      {
        _result = IX_ERROR_WARNING(
            IX_CC_ERROR_SEND_FAIL,
            ( "cannot send reply message to the client application" )
          );
      }

    IX_ERROR_CRT( ix_rm_buffer_free_chain( arg_hBuffer ),
        IX_CC_ERROR_INTERNAL, 0 );

    return _result;
  }
