/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILE
 *     ix_cc_dscp_classifier_msghlp.c
 *
 * = DESCRIPTION
 *     Implementation of the message helper API of the DSCP classifier core 
 *     component.
 *
 * ============================================================================
 */

#define IX_ERRPR_FILE_IDENT \
  "$Id: ix_cc_dscp_classifier_msghlp.c,v 1.3.6.1 2004/03/12 09:25:35 wneubaue Exp $"



#include "cc/internal/dscp_classifier_internal.h"

/* -------------------------------------- _ix_cc_classifier_dscp_callback -- */

ix_error 
_ix_cc_classifier_dscp_callback( ix_error  arg_Result,
                                 void *    arg_pCallbackContext,
                                 void *    arg_pMsg,
                                 ix_uint32 arg_nMsgLength )
  {
    /* invoking user-defined callback function... */
    if ( arg_pCallbackContext != NULL )
      {
        ix_error _result = _CALLBACK( 
            arg_pCallbackContext, arg_Result 
          );

        _DELETE_CB_CONTEXT_EX( arg_pCallbackContext );
        
        return _result;
      }
              
    return IX_ERROR_PANIC(
        IX_CC_ERROR_NULL,
        ( "invalid callback context specified" )
      );
  }

/* ----------------------------------- _ix_cc_classifier_dscp_callback_ex -- */

ix_error 
_ix_cc_classifier_dscp_callback_ex( ix_error  arg_Result,
                                    void *    arg_pCallbackContext,
                                    void *    arg_pMsg,
                                    ix_uint32 arg_nMsgLength )
  {
     /* validity check for arg_pMsg */            
    if ( arg_Result == IX_SUCCESS )
      {
        if ( arg_pMsg == NULL )
          {
            _DELETE_CB_CONTEXT_EX( arg_pCallbackContext );

            return IX_ERROR_WARNING(
                IX_CC_ERROR_NULL,
                ( "pointer to the reply message data is NULL" )
              );
          }
      }
    else
      {
        arg_pMsg = NULL;
      }

     /* invoking user-defined callback function... */
    if ( arg_pCallbackContext != NULL )
      {
        ix_error _result = _CALLBACK_EX(
              arg_pCallbackContext, arg_pMsg, arg_Result
            );

        _DELETE_CB_CONTEXT_EX( arg_pCallbackContext );
        
        return _result;
      }

    return IX_ERROR_PANIC(
        IX_CC_ERROR_NULL,
        ( "invalid callback context specified" )
      );
  }

/* ---------------------------- ix_cc_classifier_dscp_async_add_if_config -- */

ix_error 
ix_cc_classifier_dscp_async_add_if_config( 
    ix_uint16                           arg_inputPort,
    ix_cc_classifier_dscp_result      * arg_pResults,
    ix_cc_classifier_dscp_callback      arg_Callback,
    void                              * arg_pUserContext )
  {    
    _ix_cc_classifier_dscp_msg_data _data;    
    
    /* create callback context */
    _CREATE_CB_CONTEXT( arg_Callback, arg_pUserContext );        

#if defined( IX_DEBUG )
    ix_ossl_message_log( "async adding new entry...\n" );
#endif /* IX_DEBUG */

    /* prepare message */
    _data._data_ex._results = arg_pResults;
    _data._input_port       = arg_inputPort;
    
    /* send the message to the core component */
    _SEND_MESSAGE( IX_CC_CLASSIFIER_DSCP_MSG_ADD_IF_CONFIG, 
        &_data, _SIZE_OF( msg_data ) );
    
    return IX_SUCCESS;
  }

/* ------------------------- ix_cc_classifier_dscp_async_update_if_config -- */

ix_error 
ix_cc_classifier_dscp_async_update_if_config( 
    ix_uint16                           arg_inputPort,
    ix_cc_classifier_dscp_result      * arg_pResults,
    ix_cc_classifier_dscp_callback      arg_Callback,
    void                              * arg_pUserContext )
  {
    _ix_cc_classifier_dscp_msg_data _data;

    /* create callback context */
    _CREATE_CB_CONTEXT( arg_Callback, arg_pUserContext );

    /* prepare message */
    _data._data_ex._results = arg_pResults;
    _data._input_port       = arg_inputPort;

    /* send the message to the core component */
    _SEND_MESSAGE( IX_CC_CLASSIFIER_DSCP_MSG_UPDATE_IF_CONFIG,
      &_data, _SIZE_OF( msg_data ) );

    return IX_SUCCESS;
  }
    
/* ------------------------- ix_cc_classifier_dscp_async_remove_if_config -- */

ix_error 
ix_cc_classifier_dscp_async_remove_if_config(
    ix_uint16                           arg_inputPort,
    ix_cc_classifier_dscp_callback      arg_Callback,
    void                              * arg_pUserContext )
  {
    /* create callback context */
    _CREATE_CB_CONTEXT( arg_Callback, arg_pUserContext );

    /* send the message to the core component */
    _SEND_MESSAGE( IX_CC_CLASSIFIER_DSCP_MSG_REMOVE_IF_CONFIG,
        &arg_inputPort, sizeof( ix_uint16 ) );

    return IX_SUCCESS;
  }

/* ---------------------------- ix_cc_classifier_dscp_async_get_if_config -- */

ix_error ix_cc_classifier_dscp_async_get_if_config(
    ix_uint16                           arg_inputPort,
    ix_cc_classifier_dscp_callback_ex   arg_Callback,
    void                              * arg_pUserContext )
  {
    /* create callback context */
    _CREATE_CB_CONTEXT( arg_Callback, arg_pUserContext );
    
    _SEND_MESSAGE_EX( IX_CC_CLASSIFIER_DSCP_MSG_GET_IF_CONFIG,
        &arg_inputPort, sizeof( ix_uint16 ) );

    return IX_SUCCESS;
  } 

/* ---------------------------- ix_cc_classifier_dscp_async_set_def_rules -- */

ix_error ix_cc_classifier_dscp_async_set_def_rules(
    ix_cc_classifier_dscp_result      * arg_pDefaultRules,
    ix_cc_classifier_dscp_callback      arg_Callback,
    void                              * arg_pUserContext )
  {
    /* create callback context */
    _CREATE_CB_CONTEXT( arg_Callback, arg_pUserContext );

    /* send the message to the core component */
    _SEND_MESSAGE( IX_CC_CLASSIFIER_DSCP_MSG_SET_DEFAULT_RULES,
        &arg_pDefaultRules, sizeof( void * ) );

    return IX_SUCCESS;
  }

/* ---------------------------- ix_cc_classifier_dscp_async_get_def_rules -- */

ix_error ix_cc_classifier_dscp_async_get_def_rules(
    ix_cc_classifier_dscp_callback_ex   arg_Callback,
    void                              * arg_pUserContext )
  {
    /* create callback context */
    _CREATE_CB_CONTEXT( arg_Callback, arg_pUserContext );

    /* send the message to the core component */
    _SEND_MESSAGE_EX( IX_CC_CLASSIFIER_DSCP_MSG_GET_DEFAULT_RULES, NULL, 0 );
    return IX_SUCCESS;
  }

/* --------------------------- ix_cc_classifier_dscp_async_get_statistics -- */

ix_error ix_cc_classifier_dscp_async_get_statistics(
    ix_uint16                           arg_inputPort,
    ix_uint8                            arg_dscp,
    ix_cc_classifier_dscp_callback_ex   arg_Callback,
    void                              * arg_pUserContext )
  {
    _ix_cc_classifier_dscp_msg_data _data;

    /* create callback context */
    _CREATE_CB_CONTEXT( arg_Callback, arg_pUserContext );

    _data._data_ex._dscp = arg_dscp;
    _data._input_port    = arg_inputPort;

    _SEND_MESSAGE_EX( IX_CC_CLASSIFIER_DSCP_MSG_GET_STATISTICS,
        &_data, _SIZE_OF( msg_data ) );

    return IX_SUCCESS;
  }
