/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILE
 *     ix_cc_dscp_classifier_registry.c
 *
 * ============================================================================
 */

#define IX_ERROR_FILE_IDENT \
    "$Id: ix_cc_dscp_classifier_registry.c,v 1.2.6.1 2004/03/12 09:25:34 wneubaue Exp $"



#include "cc/internal/dscp_classifier_internal.h"

/* ---------------------------------------------- _ix_registry_get_uint32 -- */

ix_uint32 _ix_registry_get_uint32( const ix_uint8 * arg_strPropName )
  {
    ix_cp_property_info              _info;
    ix_configuration_property_handle _prop = IX_NULL_HANDLE;
    
    ix_error  _result;

    /* validity check for arg_strPropName */
    IX_ERROR_CHECK_ARG_NULL( 1, arg_strPropName );

    _result = ix_rm_cp_property_open( 
      IX_CP_CORE_PROPERTY_ROOT, arg_strPropName, &_prop );

    if ( _result != IX_SUCCESS )
      {
#if defined( IX_DEBUG )        
        ix_ossl_message_log( "cannot open property\n" );
#endif /* IX_DEBUG */

        return (ix_uint32)(-1);
      }

    _result = ix_rm_cp_property_get_info( _prop, &_info );
    if ( _result != IX_SUCCESS )
      {
#if defined( IX_DEBUG )
        ix_ossl_message_log( "cannot get info about the property\n" );
#endif /* IX_DEBUG */

        ix_rm_cp_property_close( _prop );
        return (ix_uint32)(-1);
      }

    if ( _info.m_Options & IX_CP_OPTIONS_FLAGS_DATA_UINT32 )
      {
        ix_uint32 _value;
        
        _result = ix_rm_cp_property_get_value_uint32( _prop, &_value );
        if ( _result != IX_SUCCESS )
          {
#if defined( IX_DEBUG )
            ix_ossl_message_log( "cannot get property value\n" );
#endif /* IX_DEBUG */
            _value = (ix_uint32)(-1);
          }

        ix_rm_cp_property_close( _prop );
        return _value;
      }
#if defined( IX_DEBUG ) 
    else
      {
        ix_ossl_message_log( "invalid property type (all subproperties must " \
            "be unsigned int 32-bit size" );
      }
#endif /* IX_DEBUG */

    ix_rm_cp_property_close( _prop );
    return (ix_uint32)(-1);
  }
