#ifndef DSCP_MARKER_UC
#define DSCP_MARKER_UC

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/
/* #define IX_ERROR_FILE_IDENT "$Id: dscp_marker.uc,v 1.17.6.1 2004/03/12 10:26:38 wneubaue Exp $" */


// FILE  dscp_marker.uc
//
//   Implements a marker microblock for changing a tos field in the IP header

// Exported APIs:
//
//   dscp_marker(out_ip_header,       \
//               in_ip_header,        \
//               IPHDR_WR_START_BYTE, \
//               IPHDR_RD_START_BYTE)

//
// ASSUMPTIONS
//
//   Handles both IPv4 and IPv6 packets.
//

//------------------------------------------------------------------------------
// Included files:

#include <ipv4.h>
#include <dscp_marker.h>



//------------------------------------------------------------------------------
//#macro dscp_marker(out_ip_header,
//                   in_ip_header,
//                   IPHDR_WR_START_BYTE,
//                   IPHDR_RD_START_BYTE)

// dscp_marker
//
// Description:
//
//	Copies a packet header from in_ip_header to out_ip_header.
//  If dl_next_block indicates marker the macro writes flow_id value into
//  a dscp field in the packet header and updates checksum
//	Note the order of parameters.
//
//  Inputs:
//    in_ip_header   - IP header cache. It can span transfer registers, GPRs
//                     LocalMem area. It can be refered to by xbuf_xxx macros.
//    dl_header_type - used to determine the version of ip (because of different
//                     positions of dscp fields in the headers).
//                     It is read using dl_meta_get_header_type macro.
//    dl_flow_id     - the new value of dscp to be written into the header.
//    dl_next_block  - indicates where the current packet is addressed.
//                     BID_DSCP_MARKER value indicates that the current packet
//                     should be processed by a dscp_marker micro block.
//
//  Outputs:
//    out_ip_header  - IP header cache. It can span transfer registers, GPRs
//                     LocalMem area. It can be refered to by xbuf_xxx macros.
//    dl_next_block  - if packet is processed by dscp_marker this variable is set
//                     to DSCP_MARKER_NEXT1
//
//  Constans
//    IPHDR_WR_START_BYTE - byte address relative to the start of out_ip_header
//    IPHDR_RD_START_BYTE - byte address relative to the start of in_ip_header
//
#macro	dscp_marker(out_ip_header, \
                    in_ip_header,  \
					IPHDR_WR_START_BYTE, \
					IPHDR_RD_START_BYTE)


	xbuf_param_set(in_ip_header)  ; sets _cur_xbuf_size at least theoretically.
	#ifdef IN_IP_SIZE
		#error "IN_IP_SIZE has been already defined"
	#endif

	; in order to help this macro achieve this me must do this little trick
	#if (strstr(in_ip_header, $$))
		#define_eval IN_IP_SIZE (_cur_xbuf_d_size * 4) - IPHDR_RD_START_BYTE
    #elif (strstr(in_ip_header, '*l$index')>0)
		#define_eval IN_IP_SIZE (_cur_xbuf_lmem_size * 4) - IPHDR_RD_START_BYTE
    #elif (strstr(in_ip_header, $))
		#define_eval IN_IP_SIZE (_cur_xbuf_size * 4) - IPHDR_RD_START_BYTE
	#else
		#define_eval IN_IP_SIZE (_cur_xbuf_gpr_size * 4) - IPHDR_RD_START_BYTE
	#endif


	xbuf_param_set(out_ip_header) ; sets _cur_xbuf_size
	#ifdef OUT_IP_SIZE
		#error "OUT_IP_SIZE has been already defined"
	#endif

	; in order to help this macro achieve this me must do this little trick
	#if (strstr(out_ip_header, $$))
		#define_eval OUT_IP_SIZE (_cur_xbuf_d_size * 4) - IPHDR_WR_START_BYTE
    #elif (strstr(out_ip_header, '*l$index')>0)
		#define_eval OUT_IP_SIZE (_cur_xbuf_lmem_size * 4) - IPHDR_WR_START_BYTE
    #elif (strstr(out_ip_header, $))
		#define_eval OUT_IP_SIZE (_cur_xbuf_size * 4) - IPHDR_WR_START_BYTE
	#else
		#define_eval OUT_IP_SIZE (_cur_xbuf_gpr_size * 4) - IPHDR_WR_START_BYTE
	#endif


	#if (IN_IP_SIZE > OUT_IP_SIZE)
	   #define_eval  IN_IP_SIZE  OUT_IP_SIZE
	#endif


	.begin
		.reg new_dscp
		.reg new_tos
		.reg old_tos

	    xbuf_copy(out_ip_header, 0, IPHDR_WR_START_BYTE,                \
		          in_ip_header, IPHDR_RD_START_BYTE, 0, IN_IP_SIZE, 0)

		;check block ID. If doesn't match ours, execute bypass path.
		alu[--, dl_next_block, -, BID_DSCP_MARKER]
		bne[marker_end#]


   ;----------------------------------------------------------
   ; Marker path
   ;----------------------------------------------------------

		; get ip_ver and tos fields / 2 bytes @ offset 0
	    xbuf_extract(old_tos, in_ip_header, IPHDR_RD_START_BYTE, 0, 2)

    	dl_meta_get_flow_id(new_dscp)
        alu_shf[new_dscp, 0xFC, AND, new_dscp, <<2]

		.begin
		.reg header_type
			dl_meta_get_header_type(header_type)
			alu[--, header_type, -, DSCP_MARKER_IPV4_PACKET]
			beq[ip4_processing#]
			alu[--, header_type, -, DSCP_MARKER_IPV6_PACKET]
			bne[send_to_next1#]
		.end

   ;----------------------------------------------------------
   ; IPv6 packet processing
   ;----------------------------------------------------------
	ip6_processing#:

        // ipv6 new_tos is in the format:
        //
        //      IP       DSCP
        //     ver   =bits 7..12
        //  +------+------------+----------+
        //  | 0100 |   010100   |  xxxxxx  |
        //  +------+------------+----------+
        //
        //

		alu_shf[new_tos, old_tos, AND~, 0x3F,     <<6]
		alu_shf[new_tos, new_tos, OR,   new_dscp, <<6]

		br[marking#]


   ;----------------------------------------------------------
   ; IPv4 packet processing
   ;----------------------------------------------------------
	ip4_processing#:
        // new_tos is in the format:
        //
        //    ver   len      DSCP = bits 2..7
        //  +-----------+----------+
        //  | 0100 0101 | 000000 xx|
        //  +-----------+----------+
        //
		alu[new_tos, old_tos, AND~, 0xFC]
		alu[new_tos, new_tos, OR, new_dscp]

		.begin
		.reg checksum
		    xbuf_extract(checksum, in_ip_header, \
										IPHDR_RD_START_BYTE, IP_CHECKSUM)

			// formula as in RFC1624
			alu[old_tos,        --,     ~B, old_tos]
			alu[checksum, checksum,      -, old_tos]
			alu[checksum, checksum, -carry, new_tos]
			alu[checksum, checksum, -carry, 0x00]

    		xbuf_insert[out_ip_header, checksum, \
										IPHDR_WR_START_BYTE, IP_CHECKSUM]




		.end


   ;----------------------------------------------------------
   ; marking the packet
   ;----------------------------------------------------------

	marking#:
	   	xbuf_insert[out_ip_header, new_tos, IPHDR_WR_START_BYTE, 0, 2]

	send_to_next1#:
		immed[dl_next_block, DSCP_MARKER_NEXT1]

	marker_end#:

	.end
    #undef IN_IP_SIZE
    #undef OUT_IP_SIZE

#endm


#endif	// DSCP_MARKER_UC
