////////////////////////////////////////////////////////////////////////////////////////
//                                                                      
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119 
//
//		
///////////////////////////////////////////////////////////////////////////////////////


#ifndef QM_DIFFSERV_H
#define QM_DIFFSERV_H

// standard macros from IXPBlocks

#include "stdmac.uc"
#include "localmem.uc"
#include "dl_system.h"
#include "definitions.h"

#ifdef USE_IMPORT_VAR

  .import_var QM_DROP_QUEUE_ENTRY
  .import_var QD_SRAM_BASE
  .import_var QD_TOTAL

#else

/* defined in dl_system.h */


#endif /* USE_IMPORT_VAR */



/////////////////////////////////////////////////////
// Signal Allocation
/////////////////////////////////////////////////////

// All this is for manual allocation of signals till its automated by assembler

#define SIG_1	1
#define SIG_2	2
#define SIG_3	3
#define SIG_4	4
#define SIG_5	5
#define SIG_6	6
#define SIG_7	7
#define SIG_8   8
#define SIG_9 	9
#define SIG_10	10
#define SIG_11	11
#define SIG_12	12
#define SIG_13	13
#define SIG_14	14
#define SIG_15	15

/////////////////////////////////////////////////////
// Ring Definitions
/////////////////////////////////////////////////////



#ifdef EGRESS

//ring numbers
#define_eval ENQ_RING_NUMBER	(QM_RING_IN_0  << 2) 
#define_eval DEQ_RING_NUMBER	(QM_RING_IN_1  << 2)
#define_eval TX_RING_NUMBER_0	(QM_RING_OUT_0 << 2)
#define_eval TX_RING_NUMBER_1	(QM_RING_OUT_1 << 2)

// Depends on the phy mode
#if ( defined(TX_PHY_MODE) && ((TX_PHY_MODE == SPHY_4_8) | (TX_PHY_MODE == MPHY_4) ))
	#define_eval TX_RING_NUMBER_2	(QM_RING_OUT_2 << 2)
	#define_eval TX_RING_NUMBER_3	(QM_RING_OUT_3 << 2)
#endif


// message length

#define NUM_WORDS_ENQ_MESSAGE 	3
#define NUM_WORDS_DEQ_MESSAGE 	1
#define NUM_WORDS_TX_MESSAGE 	1

#else // for INGRESS

#if ( defined(TX_PHY_MODE) && ((TX_PHY_MODE == SPHY_4_8) | (TX_PHY_MODE == MPHY_4) ))
	#define_eval ENQ_RING_NUMBER	(QM_RING_IN_0  << 2) 
	#define_eval DEQ_RING_NUMBER	(QM_RING_IN_1  << 2)
	#define_eval TX_RING_NUMBER_0	(QM_RING_OUT_0 << 2)
	#define_eval TX_RING_NUMBER_1	(QM_RING_OUT_1 << 2)
	#define_eval TX_RING_NUMBER_2	(QM_RING_OUT_2 << 2)
	#define_eval TX_RING_NUMBER_3	(QM_RING_OUT_3 << 2)

#else

// ring numbers

	#define_eval ENQ_RING_NUMBER	(QM_RING_IN    << 2) 
	#define_eval TX_RING_NUMBER	(QM_RING_OUT   << 2)
	#define_eval DEQ_RING_NUMBER	(SCHEDULER_TO_QM_SCR_RING << 2)
	#define_eval TX_RING		QM_RING_OUT

#endif

// message length for each ring

#define NUM_WORDS_ENQ_MESSAGE 	3
#define NUM_WORDS_DEQ_MESSAGE 	1
#define NUM_WORDS_TX_MESSAGE 	2
#define NUM_WORDS_SCHED_MESSAGE 1


#endif


// Ring size [31:30]. Hardware representation
// 00 = 128
// 01 = 256
// 10 = 512
// 11 = 1024

//#define RING_SIZE_128 		0
//#define RING_SIZE_256		1
//#define RING_SIZE_512		2
//#define RING_SIZE_1024		3



// CAM MACRO

#define GET_CAM_HIT 		alu_shf[cam_hit, 0x1, AND, cam_result, >>7]     // ( (x >> 7 ) & 0x1 )
#define GET_CAM_STATUS		alu_shf[cam_status, 0xf, AND, cam_result, >>8]  // ( (x >> 8 ) & 0xf )
#define GET_CAM_ENTRY		alu_shf[cam_entry, 0xf, AND, cam_result, >>3]   // ( (x >> 3 ) & 0xf )

// SRAM QD Defines

/* used to define the qarray entry for q array operations */
#define SRAM_Q_ARRAY_NO_FIELD_SHIFT	24
/* bit of the full sram address where memory channel is encoded. */
#define SRAM_CHANNEL_FIELD_SHIFT	30
/* mask of two bits which is used to separate channel field
   from the full sram address */
#define SRAM_CHANNEL_FIELD_MASK     0x3
/* used to set the SRAM CONTROL CSR */
#define OFFSET_SRAMCONTROL_CSR		0x0
/* during enq/deq the handle ptr is incremented by the define.
   this is used for packet based dequeue and
   not cell based dequeue.  */
#define META_PACKET_NEXT_LW_PTR 	0x7


// for CAM status

#define STATUS_VALID				0x1

// All these masks

#define TR_INV_CHECK_MASK			0x70000000	; schedular message check
#define ADDR_PTR_MASK				0x00ffffff	; to mask of eop,sop and cellcount
#define ENQ_IND_REF_MASK			0x001ff000	; enqueue indirect reference mask
#define SOPRESET_MASK				0xbf		; cell count mask
#define EOPCELLCOUNT_MASK			0x80		; to check eop bit set and last cell
#define MSB_SET						0x80000000	; 
#define DEQ_TRANS_CHECK				0x1			; used to check if there is deq transtion 
#define TX_MESSAGE_QNUM_POSITION	24			; for egress qnum is put in the MSByte of
												; the transmit message.

// Drop Queue defines
#define CELL_COUNT_MASK				0x00ffffff	; to mask of the eop, cellcount
#define QM_LM_DROP_Q_BASE			0x400		; LM address for maintaining drop queue
												; packet count.

// All these bit numbers for shifts

#define TR_INV_CHECK_BITS			28			; schedular message check
#define CELLCOUNT_BIT				24			; shift to get cellcount
#define EOPCELLCOUNT_BITS			24			; shift to get start of cellcount
#define PACKET_SIZE_START_BIT		24			; CSIX RX gives packet length starting at
												; this position
#define PACKET_SIZE_MASK			0x3f		; packet size in put in seven bits.
#define PACKET_OFFSET_IN_MESSAGE	16 
#define EOP_BIT						31	
#define DEQ_Q_NUM_POSITION			0		
#ifndef NUM_CLASS_BITS
    #define NUM_CLASS_BITS				4
    #warning "NUM_CLASS_BITS not defined. Assuming default value of" NUM_CLASS_BITS
#endif
#define ENQ_QUEUE_NUM_MASK			0x8003ffff
// Start of local memory

#define LM_Q_BASE					0x0

#ifndef DL_NEXT_ME
#define			DL_NEXT_ME  0xff
#endif

// Number of queue groups (first level of hierarchical bit vector

#define 		NUMBER_OF_QUEUE_GROUPS       32

// Number of queues per group

#define 		NUMBER_OF_QUEUES_PER_GROUP    32

// The queue group and queue in the group are encoded into one id. LSB 5 bits are used for 
// queue and then 5 bits for queue group id

#define 		NUMBER_OF_BITS_FOR_QUEUE_MASK	0x1f

// bits for queue group

#define 		NUMBER_OF_BITS_FOR_GROUP	5

// Bit number for detecting enqueue transition messages (empty queue getting data)

#define QM_ENQ_TRANSITION_BIT_MASK           0xC0000000

// Bit number for detecting dequeue transition messages (queue going to empty)

#define QM_DEQ_TRANSITION_BIT_MASK           0xC0000000

// Bit number for invalid dequeues
#define QM_INVALID_DEQUEUE_BIT_MASK			 0xA0000000


#endif  // QM_DIFFSERV_H