////////////////////////////////////////////////////////////////////////////////////////////
//                                                                      
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119 
//
///////////////////////////////////////////////////////////////////////////////////

#ifndef QM_DIFFSERV_UC
#define QM_DIFFSERV_UC


//------------------------------------------------------------------------------
// Included files:
#include "qm_diffserv_macro.uc"
#include "qm_diffserv_message.uc"

//------------------------------------------------------------------------------
// Global registers:

// these transfer registers are used to write to transmit scratch ring
.reg          $deq_q, $qa_message
.xfer_order	  $deq_q, $qa_message

// global register used. Most of the registers especially those ending 
// with _reg or _mask are used to store constants. This effort is to reduce an
// instruction to move immediate data into a register. Hopefully the names are
// self explanatory. Also these names will have a corresponding #define with the 
// same name except the "_reg".

.reg    zero
.reg	gl_next_context_sig
.reg    gl_override_bit
.reg    gl_channel_num
.reg    gl_addr_ptr_mask_reg
.reg	gl_qarray_entry_mask_reg
.reg    gl_qm_drop_queue_reg
.reg    gl_qd_sram_base_reg
.reg	gl_enq_queue_num_mask



// these are initialized registers used to store 32bitconstants
.reg  	@enq_transition_bit_mask
.reg    @qm_deq_transition_bit_mask_reg
.reg	@bits_for_queue_mask
.reg    @invalid_dequeue_bit_mask
.reg	@enq_ind_ref_mask_reg
.reg    @sopreset_mask_reg


#ifdef QM_DIFFSERV_DEBUG

.reg	@debug_enq_req
.reg	@debug_deq_req
.reg	@debug_deq_done
.reg	@debug_empty_enq_req
.reg	@debug_empty_deq_req

#endif




//------------------------------------------------------------------------------
// Global signals:

// this signal is used to signal between threads
// basically every thread waits on the previous thread
// to signal
.sig 	volatile sig_prev_thread
.addr 	sig_prev_thread 				SIG_1

// these signals are used to wait on scratch read to complete
.sig    sig_scratch_deq_r_done 

.sig	sig_scratch_enq_r_done
.addr	sig_scratch_enq_r_done			SIG_2

// this signal is used when writing to a scratch ring.
.sig	volatile sig_scratch_w_done
.addr	sig_scratch_w_done				SIG_4

// when a deq request is issued this signal is used to 
// wait for the request to complete.
.sig 	sig_deq_done
.addr	sig_deq_done					SIG_9

// these signals are used to wait on q array read issued for enq and deq
.sig	sig_q_array_deq_r_done
.addr 	sig_q_array_deq_r_done			SIG_8

.sig	sig_q_array_enq_r_done

// this signal is used for any reads and writes to SRAM
.sig	sram_done



//////////////////////////////////////////////////////////////////////////
// Macro Name  : queue_manager
// Description : This is the main macro where the queue manager algorithm
//				 is implemented. The QM is designed to issue an enqueue 
//	 			 dequeue request per beat. Based on this the QM then needs
//				 to send messages to Schedular and Transmit blocks.
// Output      : Nil
// Input       : Nil
// Constant	   : Nil 
// Size        : 82
// Branches    : 19
//////////////////////////////////////////////////////////////////////////
#macro queue_manager[]
	
.begin

// This register is used to determine the qarray entry 
// we are going to act on for enq/deq purposes.

.reg qarray_entry

// This register is used to send transition messages to schedular

.reg sched_message_enq, sched_message_deq

// allocate transfer register for enqueue message 3 LW

.reg  		$enqueuemessage_0, $enqueuemessage_1, $enqueuemessage_2
.xfer_order $enqueuemessage_0, $enqueuemessage_1, $enqueuemessage_2 

// for dequeue message read from scratch ring and message received
// from dequeue
.reg   $dequeuemessage_0  
.reg   $qarray_message

// registers required to read in QDescriptor for dequeue

.reg 		$qd_array_deq_0, $qd_array_deq_1
.xfer_order	$qd_array_deq_0, $qd_array_deq_1

// register required to read in QDescriptor for enqueue

.reg 	 	$qd_array_enq_0, $qd_array_enq_1
.xfer_order	$qd_array_enq_0, $qd_array_enq_1

// this register is required to save the sop handle for enqueue messages

.reg		sop_handle enq_queue_num


; for diffserv a few new declarations are needed

// used to save last idle timestamp to sram every time it changes
.sig  volatile sig_l_timestamp_write

// holds the command to be sent to SEND_ME_SIGNAL in LOCAL_CSR 
// in order to imitate sig_queue_length_write signal from branches when
// we do not actually perfom a sram write.
.reg  skip_l_timestamp_write

.reg $queue0 $queue1 $queue2
.xfer_order $queue0 $queue1 $queue2

// holds the address of a queue descriptor that the current packet 
// is going to / from
.reg qarray_sram_addr



	

	// The following 6 instr are executed only  
	// once and are to jump start the loop

	// signal the next ctx

	_qm_signal_next_ctx[gl_next_context_sig] ;1,0

	// Read enqueue message 

	scratch[get, $enqueuemessage_0, zero, ENQ_RING_NUMBER, NUM_WORDS_ENQ_MESSAGE], sig_done[ sig_scratch_enq_r_done]

	//  swap out waiting on scratch reads to complete. 

	ctx_arb[sig_scratch_enq_r_done, sig_prev_thread]

	// signal the next ctx. this is added because the first time there may not be an 
	// enqueue message and before swapping out should signal the next thread.  

	_qm_signal_next_ctx[gl_next_context_sig] ;1,0



    ///////////////////////////////////////////////////////////////////////////
    //   Diffserv Modification                     ****************************
    ///////////////////////////////////////////////////////////////////////////
	#ifdef DIFFSERV	
		; signal will be sent to the same context we are now in.
		local_csr_rd[ACTIVE_CTX_STS]
		immed[skip_l_timestamp_write, 0]


		; current context is held on the lowest 3 bits.
		alu[skip_l_timestamp_write, skip_l_timestamp_write, and, 7]
		alu_shf[skip_l_timestamp_write, skip_l_timestamp_write, OR,   \
		        &sig_l_timestamp_write, <<3]

		local_csr_wr[SAME_ME_SIGNAL, skip_l_timestamp_write]
		.set_sig sig_l_timestamp_write
	#endif


	#ifdef QM_DIFFSERV_DEBUG

		alu[@debug_enq_req, --, B, 0]
		alu[@debug_deq_req, --, B, 0]
		alu[@debug_deq_done, --, B, 0]
		alu[@debug_empty_enq_req, --, B, 0]
		alu[@debug_empty_deq_req, --, B, 0]

	#endif



	//bypass the hit conditions following this branch.
	
	br[next_iteration#]


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;Loop Begins Here ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;Enq/Deq CAM miss cases handled here ;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
	// This is where the loop begins. Deq CAM miss and Enq CAM miss in a cycle
	// is the worst case for Queue Manager. Therefore, care has been taken
	// that we take minimum branch penalty. Hence as we read on, the code 
	// executes the worst case without taking any branch.

next_iteration#:


	// Read dequeue message
	scratch[get, $dequeuemessage_0, zero, DEQ_RING_NUMBER, NUM_WORDS_DEQ_MESSAGE], sig_done[ sig_scratch_deq_r_done]

	// check if enqueue ring is empty. the read from enqueue scratch ring
	// was issued in the previous phase.

	//.if( $enqueueMessage_0 != 0)
	alu[--, 0, - , $enqueuemessage_0]
	beq[no_enqueue_request#] 			; if there is no request for enq
									; branch to no_enqueue_request# label

	.local enq_queue_num
	// remove the cell count from the queue number for enqueuing.

	alu[enq_queue_num, gl_enq_queue_num_mask, AND, $enqueuemessage_2]

	// this check is for verifying if the enqueue request is for a buffer chain
	// to be dropped. If it matches the drop queue ring number then the special
	// drop queue processsing will be done.

	alu[--, enq_queue_num, -, gl_qm_drop_queue_reg]
	beq[process_drop_queue_enqueue#]


	#ifdef QM_DIFFSERV_DEBUG
		alu[@debug_enq_req, @debug_enq_req, +, 1]
	#endif

	// check if entry is in CAM. if there is a CAM hit then
	// jump to label enq_hit#. if there is a CAM miss then
	// write back the current CAM entry and read in the
	// new entry based on the queue number. 

    ///////////////////////////////////////////////////////////////////////////
    //   Diffserv Modification                     ****************************
    ///////////////////////////////////////////////////////////////////////////
	    _qm_cam_check[qarray_sram_addr,        \
	                  enq_queue_num,           \
	                  enq_hit#,                \
	                  qarray_entry,            \
	                  $qd_array_enq_0,         \
	                  sig_q_array_enq_r_done,  \
	                  0]


	.endlocal // no longer use of reg "enq_queue_num"


	_qm_enqueue_egress[$enqueuemessage_0, $enqueuemessage_1, qarray_entry] ; 5, 1


	// This is the miss path so we
	// wait on Q_Array enq read to complete in addition.


    ///////////////////////////////////////////////////////////////////////////
    //   Diffserv Modification                     ****************************
    ///////////////////////////////////////////////////////////////////////////
	; for diffserv we have to wait for a signal 
	; indicating the end of a queue length writing

    ctx_arb[sig_l_timestamp_write,    \
			sig_q_array_enq_r_done,   \
	        sig_scratch_deq_r_done,   \
			sig_prev_thread], defer[2]
	//save the sop_handle for future use.
	alu[sop_handle, --,b, $enqueuemessage_0]
	// initialize message register. this is the register
	// that is used to send messages to schedular.
	alu[sched_message_enq,--,b,0]
	
	// Set the enqueue transtion message. Since this is a miss we depend on
	// $qd_array_enq_0 that contains the queue count, got from return value 
	// when a read Q_Array request was issued in the previous phase. 

	_qm_set_enqmiss_message[$enqueuemessage_2, $qd_array_enq_0, sched_message_enq];5




check_dequeue#:

	// signal the next ctx well before 

	_qm_signal_next_ctx[gl_next_context_sig] ;1,0	

	// Read enqueue message 
	scratch[get, $enqueuemessage_0, zero, ENQ_RING_NUMBER, NUM_WORDS_ENQ_MESSAGE], sig_done[ sig_scratch_enq_r_done]

	//.if( $dequeueMessage_0 != 0) checking for valid dequeue message. The
	// read for dequeueMessage was issued in the previous phase

	alu[--, 0, - , $dequeuemessage_0]
	beq[no_dequeue_request#]


	#ifdef QM_DIFFSERV_DEBUG
		alu[@debug_deq_req, @debug_deq_req, +, 1]
	#endif

	// check if entry is in CAM. if there is a CAM hit then
	// jump to label deq_hit#. if there is a CAM miss then
	// write back the current CAM entry and read in the
	// new entry based on the queue number. 

    ///////////////////////////////////////////////////////////////////////////
    //   Diffserv Modification                     ****************************
    ///////////////////////////////////////////////////////////////////////////
	_qm_cam_check[qarray_sram_addr,        \
	              $dequeuemessage_0,       \
				  deq_hit#,                \
				  qarray_entry,            \
				  $qd_array_deq_0,         \
				  sig_q_array_deq_r_done,  \
				  0] ; 15, 1, 15

	
	// If we are here it implies that there is a CAM miss.
	// send a dequeue message

	_qm_dequeue[qarray_entry, $qarray_message] ;1, 0

	ctx_arb[sig_scratch_enq_r_done, sig_q_array_deq_r_done, sig_deq_done, sig_prev_thread]


    ///////////////////////////////////////////////////////////////////////////
    //   Diffserv Modification                     ****************************
    ///////////////////////////////////////////////////////////////////////////
	; for diffserv we have to wait for an additional signal 
	; indicating the end of a queue length writing
	.begin
	// registers holding the address of the queue's data in SRAM
	.reg qd_address

		; if ((queue.length = queue.length - 1) == 0)
		;                                     goto save_last_idle_timestamp2#

		alu[*l$index0, $qd_array_deq_0, -, 1]
		beq[save_l_timestamp2#]

		local_csr_wr[SAME_ME_SIGNAL, skip_l_timestamp_write]
		.set_sig sig_l_timestamp_write

		br[skip_save_l_timestamp2#]

	save_l_timestamp2#:

		; change long word address to byte address
		ld_field_w_clr[qd_address, 0111, qarray_sram_addr]
		alu[qd_address, --, B, qd_address, <<2]
		//get the current time
		local_csr_rd[TIMESTAMP_LOW]
		immed[$queue1,0]
		local_csr_rd[TIMESTAMP_HIGH]
		immed[$queue2,0]
		sram[write, $queue1, qd_address, 12, 2], \
                                          sig_done[sig_l_timestamp_write]



	skip_save_l_timestamp2#:

	.end
	
	// To let the assembler know that scratch write 
	// is completed and these transfer registers can be reused

	.io_completed  $qd_array_deq_0, $deq_q

	// Set the dequeue transtion message 

	_qm_set_deq_message[$dequeuemessage_0,  $qarray_message, sched_message_deq]; 11

	// Send Message to the schedular if required

	_qm_send_sched_message[sched_message_enq, sched_message_deq, $qarray_message];4

	// signal the next ctx

	_qm_signal_next_ctx[gl_next_context_sig] ;1,0

	// loop back and read the next enqueue/dequeue request

	br[next_iteration#]




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;No Enqueue/Dequeue Request on Scratch case handled here
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

no_enqueue_request#:

	#ifdef QM_DIFFSERV_DEBUG
		alu[@debug_empty_enq_req, @debug_empty_enq_req, +, 1]
	#endif 


	//  swap out waiting on scratch reads to complete. 

	///////////////////////////////////////////////////////////////////////////
	//   Diffserv Modification                     ****************************
	///////////////////////////////////////////////////////////////////////////
	#ifdef DIFFSERV

		ctx_arb[sig_l_timestamp_write,   \
	            sig_scratch_deq_r_done,  \
	            sig_prev_thread], br[check_dequeue#], defer[2]
		// zero sop handle
		alu[sop_handle, --, b, 0]
		// initialize message register
		alu[sched_message_enq,--,b,0]	

	#else
		
		ctx_arb[ sig_scratch_deq_r_done, sig_prev_thread], defer[2]
			// zero sop handle
			alu[sop_handle, --, b, 0]
			// initialize message register
			alu[sched_message_enq,--,b,0]	
	
		br[check_dequeue#]

	#endif

no_dequeue_request#:

	#ifdef QM_DIFFSERV_DEBUG
		alu[@debug_empty_deq_req, @debug_empty_deq_req, +, 1]
	#endif

	local_csr_wr[SAME_ME_SIGNAL, skip_l_timestamp_write]
	.set_sig sig_l_timestamp_write

	// Check if there is anything to dequeue from the drop queue
	// well we can always get rid of setting the local memory and always issue
	// a dequeue. We can do this optimization for better performance.

	// Send Message to the schedular if required
	// basically we check if we got a valid enqueue message
	// in the previous phase

	alu[--, --, b, sop_handle]
	beq[do_not_send_message#]

	_qm_send_sched_message[sched_message_enq, 0, 0]

do_not_send_message#:

	alu[--,--,b,*l$index1]
	bne[process_drop_queue_dequeue#]

	ctx_arb[sig_scratch_enq_r_done, sig_prev_thread]

	// signal the next ctx

	_qm_signal_next_ctx[gl_next_context_sig] ;1,0

	br[next_iteration#]



;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;Enqueue/Dequeue CAM hit cases handled here;;;;;;;;;;;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


	// this is the code path when there is a valid enqueue
	// and there is a CAM hit.

enq_hit#:

	// send an enqueue message

	_qm_enqueue_egress[$enqueuemessage_0, $enqueuemessage_1, qarray_entry] 

	//  swap out waiting on scratch read for DEQ to complete and the signal
	//  from previous thread. 

    ///////////////////////////////////////////////////////////////////////////
    //   Diffserv Modification                     ****************************
    ///////////////////////////////////////////////////////////////////////////
	; for diffserv we have to wait for an additional signal 
	; indicating the end of a queue length writing

	ctx_arb[sig_l_timestamp_write,    \
			sig_scratch_deq_r_done,   \
	        sig_prev_thread], defer[2] 
	//save the sop_handle for future use.
	alu[sop_handle, --,b, $enqueuemessage_0]
	// initialize message register. this is the register
	// that is used to send messages to schedular.
	alu[sched_message_enq,--,b,0]

	sram[wr_qdesc_count, --, qarray_entry, qarray_sram_addr]


	// Set the enqueue transtion message 

	_qm_set_enqhit_message[$enqueuemessage_2, sched_message_enq]

	// enqueue is done. so now check for dequeue

	br[check_dequeue#]


deq_hit#:

	// send a dequeue message

	_qm_dequeue[qarray_entry, $qarray_message] ;1, 0

	//  swap out waiting on scratch read for ENQ to complete, DEQ to complete
	//  and the signal from previous thread. On returning branch directly to
	// 	to determine dequeue transition message for the CAM hit case.

	ctx_arb[sig_scratch_enq_r_done, sig_deq_done, sig_prev_thread]

	// To let the assembler know that scratch write 
	// is completed and these transfer registers can be reused

	.io_completed  $qd_array_deq_0, $deq_q


    ///////////////////////////////////////////////////////////////////////////
    //   Diffserv Modification                     ****************************
    ///////////////////////////////////////////////////////////////////////////
	; for diffserv we have to wait for an additional signal 
	; indicating the end of a queue length writing
	#ifdef DIFFSERV
	.begin
	.reg temp

		sram[wr_qdesc_count, --, qarray_entry, qarray_sram_addr]
		; decrement the number of dequeue issued counter in local memory 
		; if queue.length == 0 goto save_last_idle_timestamp#
		alu[*l$index0, *l$index0, -, 1]
		beq[save_l_timestamp1#]

		br[skip_save_l_timestamp1#], defer[1]
		.set_sig sig_l_timestamp_write
		local_csr_wr[SAME_ME_SIGNAL, skip_l_timestamp_write]

	save_l_timestamp1#:
		// registers holding the address of the queue's data in SRAM
		.reg qd_address
		//get the current time
		local_csr_rd[TIMESTAMP_LOW]
		immed[$queue0,0]
		local_csr_rd[TIMESTAMP_HIGH]
		immed[$queue1,0]
		; change long word address to byte address
		ld_field_w_clr[qd_address, 0111, qarray_sram_addr]
		alu[qd_address, --, B, qd_address, <<2]
		sram[write, $queue0, qd_address, 12, 2], \
                                          sig_done[sig_l_timestamp_write]


    skip_save_l_timestamp1#:

	.end
	#endif
		
	
	// Set the dequeue transtion message 

	_qm_set_deq_message[$dequeuemessage_0, $qarray_message, sched_message_deq]

	// Send Message to the schedular if required

	_qm_send_sched_message[sched_message_enq, sched_message_deq, $qarray_message]

	// signal the next ctx

	_qm_signal_next_ctx[gl_next_context_sig] ;1,0

	// loop back and read the next enqueue/dequeue request

	br[next_iteration#]




;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;;;;;;;;Enqueue and Dequeue Request for drop queue i.e. dropping a 
;;;;;;;;;;multiple buffer packet is processed here.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


// Handling enqueing to the drop queue

process_drop_queue_enqueue#:

	.local entry, cell_count_masked, cell_count_mask_reg

	// initialize the local memory  pointer to point
	// to the queue count for drop queue

	move[entry, QM_LM_DROP_Q_BASE]
	_qm_lm_init[0, entry]

	// mask the cell count from the head handle
	// this is an optimization that will require
	// only  one dequeue to free that buffer.

	move[cell_count_mask_reg, CELL_COUNT_MASK]
	alu[cell_count_masked, cell_count_mask_reg, AND, $enqueuemessage_0]

	// set the Qarry Entry where the drop queue descriptor
	// is stored and where the packet will be enqueued

	alu[qarray_entry,--,b, QM_DROP_QUEUE_ENTRY, <<SRAM_Q_ARRAY_NO_FIELD_SHIFT]

	alu[qarray_entry, qarray_entry, OR, gl_channel_num]
	
	// enqueue the packet

	_qm_enqueue_ingress[cell_count_masked, $enqueuemessage_1, qarray_entry]
	alu[*l$index0, *l$index0, +, 1] 

	.endlocal	

	//  swap out waiting on scratch read for DEQ to complete. 

	ctx_arb[sig_l_timestamp_write,   \
	        sig_scratch_deq_r_done,  \
			sig_prev_thread], defer[2] ,br[check_dequeue#]
	// zero sop handle
	alu[sop_handle, --, b, 0]
	// initialize message register. this is the register
	// that is used to send messages to schedular.
	alu[sched_message_enq,--,b,0]





process_drop_queue_dequeue#:

	// set the Qarry Entry where the drop queue descriptor
	// is stored and where the packet will be enqueued
	// we can store this information in a global register and
	// avoid these two instruction every time.

	alu[qarray_entry,--,b, QM_DROP_QUEUE_ENTRY, <<SRAM_Q_ARRAY_NO_FIELD_SHIFT]

	alu[qarray_entry, qarray_entry, OR, gl_channel_num]

	// This implies there is need to dequeue so issue a dequeue request

	_qm_dequeue[ qarray_entry, $qarray_message] ;1, 0

	ctx_arb[sig_scratch_enq_r_done, sig_deq_done, sig_prev_thread]

	// Update local memory regarding drop Q queuecount
	// and also enqueue the dequeued buffer to the free list
	// if all cells have been dequeued

	_qm_dropq_deq_handler[deqQueueNum, $qarray_message]


	// signal the next ctx

	_qm_signal_next_ctx[gl_next_context_sig] ;1,0

	br[next_iteration#]

.end
#endm // queue_manager[]




#ifndef DISPATCH_LOOP_EXCLUDED
///////////////////////////////////////////////////
// This is where the code begins..................
///////////////////////////////////////////////////
begin#:

	// Initialization of global and absolute registers, Q_array and local memory

	qm_init[]

	// Here is where the Queue Manager loop begins

	queue_manager[]

error#:
	nop ; place holder will only come here if there is an error
#endif DISPATCH_LOOP_EXCLUDED

#endif // QMCODE_UC
