#ifndef SCHEDULER_DIFFSERV_H
#define SCHEDULER_DIFFSERV_H

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

/* FILE  scheduler_diffserv.h
 *
 * Header file for Egress DiffServ scheduler 
 */




/*
 * this token includes global variables helping with debugging
 *
 
 #define SCHEDULER_DIFFSERV_DEBUG

 */

#ifdef USE_IMPORT_VAR

 .import_var SCHED_WEIGHT_CREDIT_BASE
 .import_var SRAM_ZERO_BLOCK

#endif


/* 
 * size of a queue entry in local memory (experessed as the power of 2).
 * Size of the single Queue Data structure is 8 bytes (2 LWs)
/* bits for port */

#ifndef NUMBER_OF_BITS_FOR_PORT
#define 		NUMBER_OF_BITS_FOR_PORT		4
#warning "NUMBER_OF_BITS_FOR_PORT not defined. Assuming default value of " NUMBER_OF_BITS_FOR_PORT
#endif


/*
 * Number of ports 
 * In the current implementation this number can't be greater than 16
 * It is restricted by the algorithm of chosing port to schedule.
 */
#ifndef NUMBER_OF_PORTS
#define_eval	NUMBER_OF_PORTS              (1 << NUMBER_OF_BITS_FOR_PORT)
#warning "NUMBER_OF_PORTS not defined. Assuming default value of " NUMBER_OF_PORTS
#endif

#if (NUMBER_OF_PORTS > 16)
#error "NUMBER_OF_PORTS can NOT exceed 16. It is restricted with the port schedule algorithm."
#endif 

/* Bit vector that says which ports have credit. Set this up appropriately. For e.g.
 * if there are only 4 ports, then set it up as 0xf. For now we will set up all 16 
 * ports
 */
#define 		PORT_INITIAL_CREDIT_VECTOR	((1 << NUMBER_OF_PORTS) - 1)

/* The port and queue in the port are encoded into one id. LSB 4 bits are used for 
 * queue id and then 4 bits for port id
 */

#define 		NUMBER_OF_BITS_FOR_QUEUE	4

/* Number of queues per port - in fact it is a 16. */
#define 		NUMBER_OF_QUEUES_PER_PORT    (1 << NUMBER_OF_BITS_FOR_QUEUE)


/* 
 * size of a queue entry in local memory (experessed as the power of 2)
 */
#define 		QUEUE_LM_ENTRY_LOG_SIZE		3

/*
 * MTU is a system dependent definition.
 * Therefore it should be made either in dl_system.h or as a compilation option.
 */
#ifndef	MTU
    /* base address where an array of test-result data begins */
	#error	"MTU  not defined."
#endif


/* Number of bits used to encode the packet size */
#define 		BITS_FOR_PACKET_LENGTH          7

/* units in which packet size and queue credit increments are counted */

#define_eval	CHUNK_SIZE                   	(1 << BITS_FOR_PACKET_LENGTH)


/* location in local memory where the data structures per queue are stored
 * This is computed by using 32 bytes for the port structure * number of ports.
 * This is where the port data structures end. 
 */
#define_eval    QUEUE_LM_BASE_OFFSET                (NUMBER_OF_PORTS << 5)

/*
 * actual size of meter entry in LM in bytes 
 */
#define         SCHEDULER_QUEUE_LMEM_SIZE			8

/*
 * actual size of queue entry in LM expressed in the power of 2
 * Rule: int[Log2(LMEM_MTR_SIZE)]
 */
#define         SCHEDULER_QUEUE_LMEM_LOG_SIZE		3

/*
 * Location in Local Memory where start Port Data structures. In fact this is a starting offset for the 
 * Scheduler.
 */
#ifndef PORT_LM_BASE_OFFSET
#warning "PORT_LM_BASE_OFFSET not defined - assuming default value of 0x0."
#define			PORT_LM_BASE_OFFSET				0x0
#endif

/* Location in local memory where the data structures per queue are stored
 * This is computed by using SIZE of the single structure expressed in LWs (so we have to shift 2 more) 
 * bytes for the port structure * number of ports.This is where the port data structures end. 
 */
#define_eval    QUEUE_LM_BASE_OFFSET	        (PORT_LM_BASE_OFFSET + (NUMBER_OF_PORTS << (QUEUE_LM_ENTRY_LOG_SIZE + 2)))

/* 
 * Mask used to get Port structure LM address after all shifts performed on 
 * message received from QM 
 */
#define PORT_LM_BASE_MASK (((1 << NUMBER_OF_BITS_FOR_PORT) - 1) << (QUEUE_LM_ENTRY_LOG_SIZE + 2))

/* 
 * Mask used to get Queue structure LM address after all shifts performed on 
 * message received from QM 
 */
#define QUEUE_LM_BASE_MASK (((1 << (NUMBER_OF_BITS_FOR_QUEUE + NUMBER_OF_BITS_FOR_PORT)) -1) << QUEUE_LM_ENTRY_LOG_SIZE)

/*
 * Define starting number of the physical port assigned to the Ethernet.
 * In the SPI configuration it is possible to assign ports also for another direction.
 */
#define QM_SCHEDULER_PHY_MAP_OFFSET RX_PORT_MAP_VALUE



#ifndef         SCHEDULER_MAX_NO_PENDING_REQUESTS
	#warning "SCHEDULER_MAX_NO_PENDING_REQUESTS not defined. Assuming default value of 12 packets"
	/*
	 * the maximum number of requests that scheduler sent to QM,
	 * but which QM has not yet acknowledged.
	 */
	#define     SCHEDULER_MAX_NO_PENDING_REQUESTS	12
#endif


/*
 * Ring id for scratch ring (from system.h)
 */
#define 		RING_ID					SCHEDULER_RING_OUT

/*
 * Ring full value for testing branch state. This depends on which ring you are using
 * Eventhough 0-15 rings are possible, the "ring full"input state can be used
 * only on rings 0-11. i.e SCR_Ring#_Full is valid only for # = 0-11.
 */
#if (RING_ID > 11)
	#error	"For Ring > 11, we cannot use the SCR_Ring#_Full input state"
#else
	#define_eval    RING_FULL_VALUE	        SCR_Ring/**/RING_ID/***/_Full

#endif

/*
 * Ring size [31:30]. Hardware representation
 * 00 = 128
 * 01 = 256
 * 10 = 512
 * 11 = 1024
 */
#ifndef RING_SIZE_128
#define 		RING_SIZE_128 			0
#endif
#ifndef RING_SIZE_256
#define 		RING_SIZE_256			1
#endif
#ifndef RING_SIZE_512
#define 		RING_SIZE_512			2
#endif
#ifndef RING_SIZE_1024
#define 		RING_SIZE_1024			3
#endif


/* Maximum number of packets in flight */
#define 		MAX_IN_FLIGHT			32


/* Need 16 words of 0 to initialize xfer registers 0..15. Set this up correctly. 
 * Right now points to channel 0, location 0
 */
#ifndef 		SRAM_ZERO_BLOCK
#define 		SRAM_ZERO_BLOCK   		0
#endif


/*
 * Mask defining high priority queues on ports
 */
#ifndef	HIGH_PRIORITY_MASK
	#warning "HIGH_PRIORITY_MASK  not defined."
	#warning "Mask defining high priority queues not defined."
	#warning "Assume 8 first queues to be high default queues."
	#define 	HIGH_PRIORITY_MASK		0x000000FF
#endif

				
/*******************************************************************************
 *
 * For each port there are 8 words in local memory. (16 * 8 = 128 words)
 *
 *
 * PortScheduleVector			: Bit vector for which queues have credit > 0 
 *								  (Uses NUMBER_OF_QUEUES_PER_PORT bits
 *                                in the vector)
 * PortQueueMaskHi              : Vector used to go round high priority ports
 *                                This is a mask which indicates which queues
 *                                form high priority group on the port can be
 *                                handled next
 * PortQueueMaskLo              : Vector used to go round low priority ports
 *                                This is a mask which indicates which queues
 *                                form low priority group on the port can be
 *                                handled next
 * PortQueueEmptyVector			: Bit vector for which queues have data
 *                                for a port
 * PortPacketsScheduled			: Packets scheduled for this port
 * PortCurrentCredit			: Current WRR credit for port
 * PortWeight:					: Weight for the port
 *
 */

#define PORT_SCHEDULE_VECTOR_INDEX			0
#define PORT_QUEUE_MASK_HI                  1
#define PORT_QUEUE_MASK_LO                  2
#define PORT_QUEUE_EMPTY_VECTOR_INDEX		3
#define PORT_PACKETS_SCHEDULED_INDEX		4
#define PORT_CURRENT_CREDIT_INDEX			5
#define PORT_WEIGHT_INDEX					6

/*******************************************************************************
 *
 * For each queue there are 2 words in local memory (256 * 2 = 512 words)
 *
 * QueueCurrentCredit			: Current Credit for the queue 
 * QueueCreditIncrement         : The credit increment to be given to a queue
 *                                at the end of every round
 * QueuePacketCount             : Number of packets stored in queue
 * Reserved[1]                  : reserved to fill the record to 32 bytes
 */

#define QUEUE_CURRENT_CREDIT_INDEX          0
#define QUEUE_CREDIT_INCREMENT_INDEX        0
#define QUEUE_PACKET_COUNT                  1

#define QUEUE_CURRENT_CREDIT_OFFSET        16


#endif 		/*  SCHEDULER_DIFFSERV_H  */