#ifndef SCHEDULER_DIFFSERV_DDR_UC
#define SCHEDULER_DIFFSERV_DDR_UC

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

// FILE  scheduler_diffserv_ddr.uc
//

// 	scheduler algorithm for DiffServ
// 


//------------------------------------------------------------------------------
// Global definitions

#if (SCHEDULER_RING_OUT > 11)
#error			"For Ring Number Greater than 11, we cannot use the SCR_Ring#_Full input state. Check SCHEDULER_RING_OUT"
#else
#define_eval    _RING_NO 	SCHEDULER_RING_OUT
#define_eval	SCHEDULER_RING_OUT_FULL	SCR_Ring/**/_RING_NO/**/_Full
#undef 			_RING_NO
#endif


//------------------------------------------------------------------------------
// #macro _schedule_queue()

// _schedule_queue
//
// Description:
//	 Based on presence of packets, remaining queue credits and queue priority
//   the macro decides which queue should be scheduled.
//
// Inputs:
//    active_queues        -
//    PORT_QUEUE_PRIORITY  -
//    @priority_mask       -
//    in_queue_lm_base     -
//
// Outputs:
//    queue_number         - queue which has been chosen.
//
// Constants:
//    EXIT_LABEL           -
//
#macro _schedule_queue(port_number,              \
                       port_mask,                \
                       queue_number,             \
					   in_deq_message,			 \
                       active_queues,            \
					   PORT_QUEUE_PRIORITY,      \
					   refresh_schedule_vector,  \
					   in_queue_lm_base,         \
                       minus_two,                \
					   EXIT_LABEL)

.begin
.reg queue_mask
.reg queue_lm_offset

	alu[queue_mask, active_queues, and, *l$index0[PORT_QUEUE_PRIORITY]]
	ffs[queue_number, queue_mask]
	bne[ffs_succeeded#], defer[1]

	alu_shf[queue_lm_offset, --,  B, \
            port_number,  <<(QUEUE_LM_ENTRY_LOG_SIZE+NUMBER_OF_BITS_FOR_QUEUE)]


	// If we get here, then ffs failed. So the queue mask must be 0
	// or no queue is schedulable. The case where no queue is schedulable
	// cannot happen because when the schedule vector goes to 0 for a port,
	// it is reset to 1. So queue mask needs to be reset to all 1's. 
	// Recompute ffs straight from active_queues which is 
	// the and of the sched vector and empty vector. 
	// At this point we will always find a queue

	ffs[queue_number, active_queues]

ffs_succeeded#:


	alu_shf[queue_lm_offset,  \
	        queue_lm_offset, OR, queue_number, <<QUEUE_LM_ENTRY_LOG_SIZE]

	alu[queue_lm_offset, queue_lm_offset, +, in_queue_lm_base]

	localmem_set_address(0, queue_lm_offset, LM_HANDLE_1)

	; calculate new queue_mask = ~(1 << queue_no)
	alu[--, queue_number, OR, 0]
	alu[queue_mask, --, ~B, 1, <<indirect]

	alu[--, queue_number, OR, 0]
	; calculate new queueMask  port.queueMask = (0xFFFE << queue_no)
	alu_shf[*l$index0[PORT_QUEUE_PRIORITY], --, B, minus_two, <<indirect]

	// LM[queue_no].packetCount--
	alu[*l$index1[QUEUE_PACKET_COUNT], *l$index1[QUEUE_PACKET_COUNT], -, 1]
	bne[EXIT_LABEL], defer[2]
	 	alu_shf[temp, queue_number, OR, port_number,  <<NUMBER_OF_BITS_FOR_QUEUE]
		alu_shf[in_deq_message, temp, OR, 1, <<31]

	alu_shf[active_queues,      active_queues, AND, queue_mask]

	bne[EXIT_LABEL], defer[1]
	alu[*l$index0[PORT_QUEUE_EMPTY_VECTOR_INDEX], \
	    *l$index0[PORT_QUEUE_EMPTY_VECTOR_INDEX], AND, queue_mask]

	bne[EXIT_LABEL], defer[1]
	alu[*l$index0[PORT_SCHEDULE_VECTOR_INDEX],    \
	    *l$index0[PORT_SCHEDULE_VECTOR_INDEX], OR, refresh_schedule_vector]


	alu[ @port_empty_vector, @port_empty_vector, AND, port_mask]

.end
#endm



/////////////////////////////////////////////////////////////////////////////////////
// 
// schedule[]
//
// Description:
// 	
//		Schedule a packet. When we exit this macro, we will always have scheduled a 
//		packet.  If no data is available on any port, we will loop inside this same 
//		macro. This will ensure that every time we execute this macro we will 
//		execute atleast 37 instructions. This is important for reuse of the xfer 
//		register
//
//
// Outputs/inputs:				
//
//		credit_vector			Current WRR credit vector for port
//
//	
// Inputs:  
//		
//		init_credit_vector 		Initial WRR credit vector for port
//		ring					Scratch ring for deq requests
//		port_mask				Mask used to round robin among ports			
//		minus_two				A constant 0xfffffffe used to compute next port mask
//		deq_message				xfer register for deq message
//		deq_signal				signal for scratch put
//		wait_signal				signal to ctx arb on 				
//
// Constants
//								None
//
// Size: 
//
//								43 cycles
/////////////////////////////////////////////////////////////////////////////////////
#macro schedule(out_credit_vector,     \
                in_init_credit_vector, \
				in_ring,               \
				in_port_round_robin,   \
				in_queue_lm_base,      \
				in_port_lm_base,	   \
				in_deq_message,        \
				minus_two,             \
				DEQ_SIGNAL, WAIT_SIGNAL)

.begin
	.reg	temp
	.reg	temp2
	.reg 	port_number
	.reg	port_mask
	.reg 	queue_number
	.reg 	port_lm_offset
	.reg 	xfer_byte_offset
	.reg 	port_credit_quantum
	.reg 	packets_in_flight


SCHEDULE_PORT#:

        #ifdef SCHEDULER_DELAY
        .reg compare_delay

        sched_delay#:
                alu[@scheduler_delay_counter, @scheduler_delay_counter, +, 1]
                move[compare_delay, SCHEDULER_DELAY]
                alu[--, @scheduler_delay_counter, -, compare_delay]
                beq[start_scheduling#]

                ctx_arb[voluntary] , br [sched_delay#]

        start_scheduling#:
                alu[@scheduler_delay_counter, --, B, 0]

        #endif


	alu[--, @pending_requests, -, SCHEDULER_MAX_NO_PENDING_REQUESTS]
	beq[PENDING_SWAP_OUT#]


	// AND the empty vector and credit vector. 
	alu[temp, @port_empty_vector, AND, out_credit_vector]

	// AND the mask in
	alu[temp2, temp, AND, in_port_round_robin]

	ld_field[temp2, 1100, temp, <<16]

	// find the next port with data to send and with credit 
	ffs[port_number, temp2]

	// If we get here then the port vector is 0. Credit cannot be 0.
	// Since it is reset to 1 when it goes to 0. 
	// So we need to swap out since no port is schedulable
	beq[EMPTY_SWAP_OUT#] 


ADJUST_PORT_CREDIT#:

	// Compute the offset into local memory for this port and set the local memory 
	// handle. Each port data structure is 32 bytes. This operation has a 3 cycle 
	// latency before the local memory is usable 
	alu_shf[port_number,   port_number, and, 0x0F] 
	alu_shf[port_lm_offset,         --,   B, port_number, <<5] 

	alu[port_lm_offset, port_lm_offset, +, in_port_lm_base]

	localmem_set_address(0, port_lm_offset , LM_HANDLE_0)
		
	// Also use defer slot compute transfer register offset for packets
	// transmitted count. Also in bytes

#if (TX_PHY_MODE == SPHY_4_8_SHARED)
    /* in this case the tx feedback is provided via one of the first four transfer registers 
     * indexed by RXTX_ETH_LINE 
     */
	alu_shf[xfer_byte_offset, --, B, RXTX_ETH_LINE, <<2]
#else
	alu_shf[xfer_byte_offset, --, B, port_number, <<2] 
#endif

	// Set the indirect read for xfer register holding packet transmit count. This
	// has a 2 cycle latency

	local_csr_wr[T_INDEX, xfer_byte_offset]


	// compute the current port mask.
	alu[--, port_number, OR, 0]
	alu[port_mask, --, ~B, 1, <<indirect]


	// Decrement the credit. If the port does not get selected because of flow
	// control reasons then it loses this turn and no port is scheduled for the
	// beat
	alu[*l$index0[PORT_CURRENT_CREDIT_INDEX], \
	    *l$index0[PORT_CURRENT_CREDIT_INDEX], -, 1]

	// if the credit is over give it another round and clear the bit in the vector

	bgt[CHECK_IN_FLIGHT_COUNT#]

	// Set port number for indirect operation
	alu[--, port_number, OR, 0]

	// clear the port credit vector. 
	alu_shf[out_credit_vector, out_credit_vector, AND~, 1, <<indirect]

	// AND with the port empty vector 

	alu[--, out_credit_vector, AND, @port_empty_vector]
	
	// check if the result is 0, then the round is over. Reset credit vector

	bne[CHECK_IN_FLIGHT_COUNT#] , defer [2]
		
	// defer 1 - read in the port credit quantum 

	alu[port_credit_quantum, --, B, *l$index0[PORT_WEIGHT_INDEX]]
		
	// defer 2 - increment the credit for the port by quantum

	alu[*l$index0[PORT_CURRENT_CREDIT_INDEX], \
	    *l$index0[PORT_CURRENT_CREDIT_INDEX], +, port_credit_quantum]

	// Set the credit vector to init vector

	alu[out_credit_vector, --, B, in_init_credit_vector]


CHECK_IN_FLIGHT_COUNT#:


	// Compute number of packets in flight.
	alu[packets_in_flight, *l$index0[PORT_PACKETS_SCHEDULED_INDEX], -, *$$index]

	// If packets in flight exceeds MAX_IN_FLIGHT, swap out.

	alu[--, MAX_IN_FLIGHT, -, packets_in_flight]
	
	ble[FLIGHT_SWAP_OUT#]


SCHEDULE_QUEUE#:
	.begin
	.reg    sched_vector
	.reg    active_queues
	.reg    active_hi_queues

		// If we get here we have scheduled a port.
		// Now we schedule a queue on the port

		// Read the port parameters. Assume that the local memory offset 
		// has already been set up.

		alu[sched_vector, --, B, *l$index0[PORT_SCHEDULE_VECTOR_INDEX]]

		// AND the schedule and empty vectors. Empty vector must be nonzero or
		// we would not have selected the port
		alu[active_queues,  \
		    sched_vector, AND, *l$index0[PORT_QUEUE_EMPTY_VECTOR_INDEX]]

		alu[active_hi_queues, active_queues, and, @high_priority_mask]
		beq[LOW_PRIORITY_ROUND#], defer[2]
		// compute next port round robin mask excluding the current port.
		alu[--, port_number, OR, 0]
		alu_shf[in_port_round_robin, --, B, minus_two, <<indirect]

	HIGH_PRIORITY_ROUND#:
	
		_schedule_queue(port_number,
		                port_mask,
					    queue_number,
						in_deq_message,
	        	        active_hi_queues,
						PORT_QUEUE_MASK_HI,
						@high_priority_mask,
						in_queue_lm_base,
						minus_two,
						SEND_DEQUEUE#)

		br[SEND_DEQUEUE#]


	LOW_PRIORITY_ROUND#:

		_schedule_queue(port_number,
		                port_mask,
						queue_number,
						in_deq_message,
	        	        active_queues,
						PORT_QUEUE_MASK_LO,
						@low_priority_mask,
						in_queue_lm_base,
						minus_two,
						SEND_DEQUEUE#)

	.end



SEND_DEQUEUE#:

	// write the data on the scratch ring. We do not have to check whether 
	// the ring is empty because we do not allow to send more than 8 requests
	scratch[put, in_deq_message, 0, in_ring, 1]  , sig_done[DEQ_SIGNAL]

	// If we get here then we scheduled a port.
	// We swap out and then exit the macro
	ctx_arb[WAIT_SIGNAL] , br [SCHEDULE_EXIT#] , defer[2]
		
	// Increment packets scheduled for this port
	alu[*l$index0[PORT_PACKETS_SCHEDULED_INDEX], \
	    *l$index0[PORT_PACKETS_SCHEDULED_INDEX], +, 1]

	// Increment the counter of pending requests on this port
	alu[@pending_requests, @pending_requests, +, 1]



SWAP_OUT#:

	// If we get here, we did not send a packet for some reason or the other. So
	// we swap out and go back to the top of the loop 

	ctx_arb[voluntary] , br[SCHEDULE_PORT#]


EMPTY_SWAP_OUT#:

	// If we get here, we did not send a packet for some reason or the other. So
	// we swap out and go back to the top of the loop 

	#ifdef SCHEDULER_DIFFSERV_DEBUG
		alu[@debug_no_packets, @debug_no_packets, +, 1]
	#endif

	ctx_arb[voluntary] , br [SCHEDULE_PORT#]

		
FLIGHT_SWAP_OUT#:

	// If we get here, we did not send a packet for some reason or the other. So
	// we swap out and go back to the top of the loop 

	#ifdef SCHEDULER_DIFFSERV_DEBUG
		alu[@debug_in_flight_defer, @debug_in_flight_defer, +, 1]
	#endif

	ctx_arb[voluntary] , br [SCHEDULE_PORT#], defer[2]
	// compute next port round robin mask excluding the current port.
	alu[--, port_number, OR, 0]
	alu_shf[in_port_round_robin, --, B, minus_two, <<indirect]


PENDING_SWAP_OUT#:

	// If we get here, we did not send a packet for some reason or the other. So
	// we swap out and go back to the top of the loop 

	#ifdef SCHEDULER_DIFFSERV_DEBUG
		alu[@debug_pending_defer, @debug_pending_defer, +, 1]
	#endif

	ctx_arb[voluntary] , br [SCHEDULE_PORT#]


SCHEDULE_EXIT#:

	// Exit this macro 

.end
#endm



#endif   // SCHEDULER_DIFFSERV_DDR_UC
