/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

/** \file    tc_meter_defs.h
 *
 *  \brief   Three Color Meter Core Component internals - default configuration
 * 
 *           This file provides the default static configuration for the TCM
 *           core component. It is used to compile microcode and core components
 *           with the same configuration.
 */
/* $Id: tc_meter_defs.h,v 1.3.6.2 2004/03/12 10:22:27 wneubaue Exp $ */


#ifndef TC_METER_DEFS_H
#define TC_METER_DEFS_H

/*
 * size of meter entry in SRAM (in bytes)
 */
#define   TC_METER_ENTRY_SRAM_SIZE		        64

/*
 * size of meter entry in SRAM expressed in the power of 2 (in bytes)
 * rule: int[Log2(TC_METER_SRAM_SIZE)]
 */
#define   TC_METER_ENTRY_LOG_SRAM_SIZE           6

/*
 * Amount of SDRAM space taken by one entry with 32 high bits of flow statistics
 * in bytes
 */
#define	  TC_METER_64BIT_STAT_SRAM_SIZE         32

/*
 * size (in bytes) of the entry of meter statistics table where
 * the most significant long words of
 * tcm meter statistics are held.
 * rule: int[Log2(TC_METER_64BIT_STAT_SRAM_SIZE)]
 */
#define   TC_METER_64BIT_STAT_LOG_SRAM_SIZE      5


/*
 * size of meter entry in LM (in bytes)
 */
#define   TC_METER_ENTRY_LMEM_SIZE              64


/*
 * size of meter entry in LM expressed in the power of 2 (in bytes)
 * rule: int[Log2(TC_METER_LMEM_SIZE)]
 */
#define   TC_METER_ENTRY_LOG_LMEM_SIZE           6


/** 
 * The default number of the SRAM memory channel.
 */
#define   TC_METER_DEFAULT_SRAM_CHANNEL       0x01


/** 
 * The default number of entries in a TC Meter statistics table.
 */
#define   TC_METER_DEFAULT_TABLE_SRAM_SIZE  0x0400


#endif
/* TC_METER_DEFS_H */
