#ifndef TC_METER_H
#define TC_METER_H

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/
/* "$Id: tc_meter.h,v 1.8.6.1 2004/03/12 10:23:36 wneubaue Exp $" */

/* FILE  tc_meter.h
 *
 * Contains definitions used by a meter micro-block
 * The following tokens change the meter behavior:
 *
 * #define  TC_METER_COLOR_AWARE_FEATURE    - makes the meter color-aware
 *
 * #define  TC_METER_PACKET_COUNTER_FEATURE - includes a code which generates
 *                                            statistics
 *
 * #define  TC_METER_DISABLE_FIN_SYNC  - disables final thread synchronization
 *                                       if the next block is microblock 
 *                                       which carries out thread synchroniza-
 *                                       tion at the begining this flag can be
 *                                       used to improve performance. (avoid
 *                                       double synchronization one after the
 *                                       other)
 *
 * #define  TC_METER_SRTCM_FEATURE     - this token enables Single Rate mode of
 *                                       meter operation in conformance with 
 *                                       RFC 2697
 *
 * #define  TC_METER_TRTCM_FEATURE     - this token enables Two Rate mode of
 *                                       meter operation in conformance with
 *                                       RFC 2698
 */


#ifndef TC_METER_SRTCM_FEATURE
#ifndef TC_METER_TRTCM_FEATURE
	#warning "Define either TC_METER_SRTCM_FEATURE or TC_METER_TRTCM_FEATURE"
	#warning "to let compiler know what type of meter features should "
	#error   "be compiled in."
#endif
#endif

#ifndef TC_METER_TRTCM_FEATURE
#ifndef TC_METER_SRTCM_FEATURE
	#warning "Define either TC_METER_SRTCM_FEATURE or TC_METER_TRTCM_FEATURE"
	#warning "to let compiler know what type of meter features should "
	#error   "be compiled in."
#endif
#endif


/*
 * DL_NEXT_ME is a system dependent definition.
 * Therefore it should be made either in dl_system.h or as a compilation option.
 * It indicates the next ME executing metering in its functional pipeline
 */
#ifndef	DL_NEXT_ME
    #error "DL_NEXT_ME not defined."
#endif



#ifndef USE_IMPORT_VAR
/* These are system dependent definitions. Therefore they should be made 
 * either in dl_system.h or as a compilation option. */


   /* FIRST_TC_METER_ME It defines which ME send a tcm_next_me_signal to.
    * used during initialization
    */
	#ifndef	FIRST_TC_METER_ME
	  #error "FIRST_TC_METER_ME not defined."
	#endif


   /* TC_METER_TABLE_SRAM_BASE defines the base address of the meter table.
    */
	#ifndef	TC_METER_TABLE_SRAM_BASE
	    #error "TC_METER_TABLE_SRAM_BASE not defined."
	#endif

   /*
    * TC_METER_64BIT_STAT_SRAM_BASE defines the base address of the table 
    * where the most significant long words of meter statistics are held.
    */
	#ifdef TC_METER_PACKET_COUNTER_FEATURE 
	#ifndef	TC_METER_64BIT_STAT_SRAM_BASE
	    #error "TC_METER_64BIT_STAT_SRAM_BASE not defined."
	#endif
	#endif

#else

	.import_var TC_METER_TABLE_SRAM_BASE
	#ifdef TC_METER_PACKET_COUNTER_FEATURE 
	.import_var TC_METER_64BIT_STAT_SRAM_BASE
	#endif

#endif


/*
 * TC_METER_NEXT1 is a system dependent definition.
 * Therefore it should be made either in dl_system.h or as a compilation option.
 * it is definition of a pass exit where packets
 * that are allowed to pass are sent.
 */
#ifndef	TC_METER_NEXT1
#error "TC_METER_NEXT1 not defined."
#endif



/*
 * TC_METER_NEXT2 is a system dependent definition.
 * Therefore it should be made either in dl_system.h or as a compilation option.
 * it is definition of drop exit where packets that are not allowed to pass 
 * are sent.
 */
#ifndef	TC_METER_NEXT2
#error "TC_METER_NEXT2 not defined."
#endif



/*
 * TC_METER_NEXT3 is a system dependent definition.
 * Therefore it should be made either in dl_system.h or as a compilation option.
 * it is definition of drop exit where packets that are not allowed to pass 
 * are sent.
 */
#ifndef	TC_METER_NEXT3
#error "TC_METER_NEXT3 not defined."
#endif


/*
 * TC_METER_NEXT_ME_SIG is a system dependent definition.
 * Therefore it should be made either in dl_system.h or as a compilation option.
 * It is signal used to synchronize meter crtitical sections on different ME
 */
#ifndef	TC_METER_NEXT_ME_SIG
	#warning "TC_METER_NEXT_ME_SIG not defined."
#endif




/* value written into the cam_state bits of each cam_entry */
#define     TC_METER_CAM_VALID                   0x0F

/*
 * values to initialize CSR timestamp register on ME where TC_METER is to work
 * this must be synchronized on each ME with TC_METER
 */
#define     TC_METER_INIT_TIMESTAMP_LOW             0
#define	    TC_METER_INIT_TIMESTAMP_HIGH            0

/*
 * flag indicating which algorithm is used for the given entry of tcmeter 
 * table. if 1 given flow will be metered by trtcm, otherwise by srtcm.
 */
#define     TC_METER_TRTCM_ENABLE_FLAG           0x80

#define     TC_METER_PIR_EXTRACT_SHIFT              5
#define     TC_METER_PIR_EXTRACT_MASK          0x3FFF
#define     TC_METER_CIR_EXTRACT_SHIFT             19

/*
 * define offset in table entry where writable data begins (4 4-byte words).
 */
#define     TC_METER_WRITE_ACCESS_OFFSET            16

/*******************************************************************************
                     TRTC_METER PERTINENT DEFINITIONS
*/


/* TC meter table in local memory */
#define	    TC_METER_LM_TIMESTAMP_L        0
#define	    TC_METER_LM_TIMESTAMP_H        1
#define	    TC_METER_LM_TC                 2
#define	    TC_METER_LM_TE                 3
#define	    TC_METER_LM_TP                 3
#define	    TC_METER_LM_DSCP               4
#define	    TC_METER_LM_METER_TYPE         4
#define	    TC_METER_LM_CIR                5
#define	    TC_METER_LM_PIR                6
#define	    TC_METER_LM_CBS                7
#define	    TC_METER_LM_EBS                8
#define	    TC_METER_LM_PBS                8
#define	    TC_METER_LM_GREEN_PACKETS      9
#define	    TC_METER_LM_GREEN_BYTES        10
#define	    TC_METER_LM_YELLOW_PACKETS     11
#define	    TC_METER_LM_YELLOW_BYTES       12
#define     TC_METER_LM_RED_PACKETS        13
#define	    TC_METER_LM_RED_BYTES          14
#define	    TC_METER_LM_REF_COUNT          15



/*
 * bit number in the status word specifying  
 * wheather or not to count packets marked green
 */
#define     TC_METER_GREEN_COUNTER_FLAG    0x06

/*
 * bit number in the status word specifying
 * wheather or not to count packets marked yellow
 */
#define     TC_METER_YELLOW_COUNTER_FLAG   0x06

/*
 * bit number in the status word specifying
 * wheather or not to count packets marked red
 */
#define     TC_METER_RED_COUNTER_FLAG      0x06


#endif /*  TC_METER_H  */
