/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2000 - 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILENAME
 *      ix_cc_tc_meter_defs.h
 *
 * = DESCRIPTION
 *      TCM Core Component internals - common definitions and macros
 *
 * ============================================================================
 */

#if !defined(IX_CC_TC_METER_DEFS_H)
#define      IX_CC_TC_METER_DEFS_H


/**
 * Unit change factor (kbits/s into bytes/timestamp).
 */
#define TCM_RATE_UNIT_CHANGE_FACTOR    (double)300000

/**
 * The TC Meter table entry availability flag.
 */
#define TC_METER_ENTRY_VALID            1

/**
 *  Number and name of TC_METER symbols to patch.
 */
#if defined(TC_METER_PACKET_COUNTER_FEATURE)
#define TC_METER_PATCH_SYMBOLS         2
#else
#define TC_METER_PATCH_SYMBOLS         1
#endif

#define TC_METER_TABLE_SRAM_BASE       "TC_METER_TABLE_SRAM_BASE"

#if defined(TC_METER_PACKET_COUNTER_FEATURE)
#define TC_METER_64BIT_STAT_SRAM_BASE  "TC_METER_64BIT_STAT_SRAM_BASE"
#endif

/**
 * This enummeration defines messages specific to the TC Meter core component.
 */
typedef enum ix_e_cc_tc_meter_messages
  {
    /* table messages */
    IX_CC_TC_METER_MSG_ADD_ENTRY  = 0x0,
    IX_CC_TC_METER_MSG_REMOVE_ENTRY,
    IX_CC_TC_METER_MSG_UPDATE_ENTRY,
    /* statistics message */
    IX_CC_TC_METER_MSG_GET_STATISTICS,
    /* get parameters message */
    IX_CC_TC_METER_MSG_GET_PARAMETERS,
    /* add new messages before this */
    IX_CC_TC_METER_MSG_LAST
  } ix_cc_tc_meter_messages;

/* user defined macros */


/**
 * This macro shifts 64bit signed integer left.
 *
 * @param  arg_nValue the base value of the expression.
 * @param  arg_nPow   the number of bits to be shifted (up to 64).
 *
 * @return The return value is a 64bit signed integer.
 */
#define _SHIFT_LEFT_64BIT( arg_nValue, arg_nPow ) \
  ((ix_int64)(arg_nValue) << (ix_uint8)arg_nPow)

/**
 * This macro frees the memory allocated in the system and sets the pointer
 * to NULL.
 *
 * @param arg_pPtr the address of the memory to be freed.
 */
#define _SYSTEM_DELETE( arg_pPtr ) \
  if ( ((void *)arg_pPtr) != NULL ) \
    { \
      ix_ossl_free( (void *)arg_pPtr ); \
      ((void *)arg_pPtr) = NULL; \
    } \

/**
 * This macro combines two 32bit unsigned integers into one 64bit unsigned
 * integer.
 *
 * @param  arg_nInt32L low-order 32bits of the new value.
 * @param  arg_nInt32H high-order 32bits of the new value.
 *
 * @return The return value is a 64bit unsigned integer.
 */
#define IX_CC_TC_METER_MAKE_UINT64( arg_nInt32L, arg_nInt32H ) \
  (((ix_uint64)(arg_nInt32H) << 32) + (ix_uint32)(arg_nInt32L))

/**
 * This macro calculates an entry offset.
 *
 * @param  arg_nIndex the entry index.
 * @param  arg_nSize  the entry size in bytes.
 *
 * @return The return value is an 32bit offset value.
 */
#define _GET_OFFSET( arg_nIndex, arg_nSize ) \
  ((ix_uint32)(arg_nIndex) * (ix_uint32)(arg_nSize))

/**
 * This macro set a bit field.
 *
 * @param  arg_bValue condition to be resolved as a boolean expression.
 *
 * @return If <code>arg_bValue</code> is different then 0 it returns 1,
 *         otherwise the macro returns 0.
 */
#define _BIT_FLD( arg_bValue ) \
  (((ix_uint8)(arg_bValue) != 0) ? 1 : 0)

/**
 * This macro returns a pointer of the specified entry. This is common macro
 * used to obtain data from TC Meter table and statistics table (byte array).
 *
 * @param  arg_pBase  base address of a array.
 * @param  arg_nIndex the entry index.
 * @param  arg_nSize  the entry size in bytes.
 *
 * @return The return value is a pointer to the specified entry.
 */
#define _GET_PTR( arg_pBase, arg_nIndex, arg_nSize ) \
  ((ix_uint8 *)(arg_pBase) + _GET_OFFSET( arg_nIndex, arg_nSize ))

/* _CONFIG */
#define _CONFIG( arg_pCntxt ) \
  ((_ix_cc_tc_meter_context *)(arg_pCntxt))->_pConf

/**
 * This macro returns base address of the TC Meter meter table.
 *
 * @param  arg_pCntxt pointer to the TC Meter core component context.
 */
#define _TABLE_BASE( arg_pCntxt ) \
  ((_ix_cc_tc_meter_context *)(arg_pCntxt))->_pTableBase

/**
 * This macro returns the TC Meter core component entry size in bytes.
 *
 * @param  arg_pCntxt ponter to the TC Meter core component context.
 */
#define _TABLE_ENTRY_SIZE( arg_pCntxt ) \
  _CONFIG( arg_pCntxt )->_nEntrySize

/* _STATS_BASE */
#define _STATS_BASE( arg_pCntxt ) \
  ((_ix_cc_tc_meter_context *)(arg_pCntxt))->_pStatsBase

/* _STATS_ENTRY_SIZE */
#define _STATS_ENTRY_SIZE( arg_pCntxt ) \
  _CONFIG( arg_pCntxt )->_nStatsSize

/* _SRAM_CHANNEL */
#define _SRAM_CHANNEL( arg_pCntxt ) \
  _CONFIG( arg_pCntxt )->_nSRAMChnnl

/* _MAX_FLOW_IDS */
#define _MAX_FLOW_IDS( arg_pCntxt ) \
  _CONFIG( arg_pCntxt )->_nTableSize

/* _TABLE_ENTRY */
#define _TABLE_ENTRY( arg_nInstance, arg_pCntxt ) \
  (_ix_cc_tc_meter_table_entry *) \
    _GET_PTR( \
        _TABLE_BASE( arg_pCntxt ), \
        arg_nInstance, \
        _TABLE_ENTRY_SIZE( arg_pCntxt ) \
      ) \

/* _STATS_ENTRY */
#define _STATS_ENTRY( arg_nInstance, arg_pCntxt ) \
  (_ix_cc_tc_meter_stats_entry *) \
    _GET_PTR( \
        _STATS_BASE( arg_pCntxt ), \
        arg_nInstance, \
        _STATS_ENTRY_SIZE( arg_pCntxt ) \
      ) \

/**
 * This macro sets microblock symbols into the ix_imported_symbol data
 * structure.
 *
 * @param  arg_Symbol imported symbol structure .
 * @param  arg_Value  value of the imported variable.
 * @param  arg_Name   name of the imported variable.
 *
 * @return Filled data structure with the name and value.
 */
#define _SET_UBLOCK_SYMBOL( arg_Symbol, arg_Value, arg_Name ) \
    arg_Symbol.m_Value = arg_Value; \
    arg_Symbol.m_Name  = arg_Name; \

#endif /* IX_CC_TC_METER_DEFS_H */
