/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2000 - 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILENAME
 *      ix_cc_tc_meter_types.h
 *
 * = DESCRIPTION
 *      TCM Core Component internals - datatype definitions
 *
 * ============================================================================
 */

#if !defined(IX_CC_TC_METER_TYPES_H)
#define      IX_CC_TC_METER_TYPES_H



/* _ix_cc_tc_meter_config */
typedef struct _ix_s_cc_tc_meter_config
  {
    ix_uint32             _nEntrySize;
    ix_uint32             _nStatsSize;
    ix_uint32             _nTableSize;

    ix_uint32             _nSRAMChnnl;
    ix_uint32             _uEngMask;
  } _ix_cc_tc_meter_config;

/* _ix_cc_tc_meter_context */
typedef struct _ix_s_cc_tc_meter_context
  {
    _ix_cc_tc_meter_config *_pConf;
     ix_cc_handle           _hCC;
    void *                  _pTableBase;
    void *                  _pStatsBase;
  } _ix_cc_tc_meter_context;


#if defined( WIN32 )
#pragma pack( push, 1 )
#endif /* WIN32 */

#if defined( WIN32 )
#pragma pack( pop )
#endif /* WIN32 */


/* _ix_cc_tc_meter_table_entry */
typedef struct _ix_s_cc_tc_meter_table_entry
{
    ix_uint32 _cir:13,
              _pir:14,
              _cir_rate:5;
    ix_uint32 _cbs;
    ix_uint32 _ebs_pbs;
    ix_uint32 _dscp_green:6,
              _output_green:2,
              _dscp_yellow:6,
              _output_yellow:2,
              _dscp_red:6,
              _output_red:2,
              _meter_type:1,
              _stats_enabled:1,
              _valid:1,
              _pir_rate:5;
    ix_uint64 _time_stamp;
    ix_uint32 _tc;
    ix_uint32 _te_tp;
    ix_uint32 _green_pkts;
    ix_uint32 _green_bytes;
    ix_uint32 _yellow_pkts;
    ix_uint32 _yellow_bytes;
    ix_uint32 _red_pkts;
    ix_uint32 _red_bytes;

} _ix_cc_tc_meter_table_entry;



/* _ix_cc_tc_meter_stats_entry */
typedef struct _ix_s_cc_tc_meter_stats_entry
{
    ix_uint32 _green_pkts;
    ix_uint32 _green_bytes;
    ix_uint32 _yellow_pkts;
    ix_uint32 _yellow_bytes;
    ix_uint32 _red_pkts;
    ix_uint32 _red_bytes;

} _ix_cc_tc_meter_stats_entry;




/* _ix_cc_tc_meter_add_entry_data */
typedef struct _ix_s_cc_tc_meter_add_entry_data
{
    ix_uint32                  _nInstance;
    ix_cc_tc_meter_parameters  _Params;
} _ix_cc_tc_meter_add_entry_data;

/* _ix_cc_tc_meter_remove_entry_data */
typedef struct _ix_s_cc_tc_meter_remove_entry_data
{
    ix_uint32 _nInstance;
} _ix_cc_tc_meter_remove_entry_data;

/* _ix_cc_tc_meter_update_entry_data */
typedef struct _ix_s_cc_tc_meter_update_entry_data
  {
    ix_uint32                 _nInstance;
    ix_cc_tc_meter_parameters _Params;
  } _ix_cc_tc_meter_update_entry_data;

/* _ix_cc_tc_meter_get_stats_data */
typedef struct _ix_s_cc_tc_meter_get_stats_data
{
    ix_uint32 _nInstance;
} _ix_cc_tc_meter_get_stats_data;



/* _ix_cc_tc_meter_get_params_data */
typedef struct _ix_s_cc_tc_meter_get_params_data
{
    ix_uint32 _nInstance;
} _ix_cc_tc_meter_get_params_data;



/* _ix_cc_tc_meter_default_cb_context */
typedef struct _ix_s_cc_tc_meter_default_cb_context
{
    ix_cc_tc_meter_callback_op _pCallback;
    void * _pUserContext;
} _ix_cc_tc_meter_default_cb_context;



/* _ix_cc_tc_meter_get_stats_cb_context */
typedef struct _ix_s_cc_tc_meter_get_stats_cb_context
{
    void                       *_pUserContext;
    ix_cc_tc_meter_callback_gs  _pCallback;
    ix_cc_tc_meter_statistics  *_pUserPlaceholder;
} _ix_cc_tc_meter_get_stats_cb_context;



/* _ix_cc_tc_meter_get_params_cb_context */
typedef struct _ix_s_cc_tc_meter_get_params_cb_context
{
    void                       *_pUserContext;
    ix_cc_tc_meter_callback_gp  _pCallback;
    ix_cc_tc_meter_parameters  *_pUserPlaceholder;
} _ix_cc_tc_meter_get_params_cb_context;


#endif /* IX_CC_TC_METER_TYPES_H */
