/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2000 - 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILENAME
 *      ix_cc_tc_meter_fini.c
 *
 * ============================================================================
 */

#define IX_ERROR_FILE_IDENT \
  "$Id: ix_cc_tc_meter_fini.c,v 1.2.6.1 2004/03/12 09:56:48 wneubaue Exp $"


#include <ix_types.h>
#include <ix_error.h>

#include <ix_ossl.h>
#include <ix_rm.h>
#include <ix_cc.h>
#include <ix_cc_error.h>
#include <bindings.h>

#include "cc/ix_cc_tc_meter.h"
#include "cc/ix_cc_tc_meter_msg_helpers.h"
#include "cc/internal/ix_cc_tc_meter_defs.h"
#include "cc/internal/ix_cc_tc_meter_types.h"


/* ---------------------------------- _ix_cc_tc_meter_unregister_msg_handler -- */

ix_error _ix_cc_tc_meter_unregister_msg_handler()
  {
#if defined( IX_EXCLUDE_CCI )
    IX_ERROR_CRT(
        ix_rm_message_handler_unregister( IX_CC_TC_METER_MSG_INPUT ),
          IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_WARNING
      );
#else
    IX_ERROR_CRT(
        ix_cci_cc_remove_message_handler( IX_CC_TC_METER_MSG_INPUT ),
          IX_CC_TC_METER_ERROR_CCI, IX_ERROR_LEVEL_WARNING
      );
#endif /* !IX_EXCLUDE_CCI */
    
    return IX_SUCCESS;
  }

/* ---------------------------------- _ix_cc_tc_meter_unregister_pkt_handler -- */

ix_error _ix_cc_tc_meter_unregister_pkt_handler()
  {
#if defined( IX_EXCLUDE_CCI )
    IX_ERROR_CRT(
        ix_rm_packet_handler_unregister( IX_CC_TC_METER_PKT_INPUT ),
        IX_CC_TC_METER_ERROR_CCI, IX_ERROR_LEVEL_WARNING
      );
#else
    IX_ERROR_CRT(
        ix_cci_cc_remove_packet_handler( IX_CC_TC_METER_PKT_INPUT ),
        IX_CC_TC_METER_ERROR_CCI, IX_ERROR_LEVEL_WARNING
      );
#endif /* IX_EXCLUDE_CCI */

    return IX_SUCCESS;
  }

/* -------------------------------------------- _ix_cc_tc_meter_free_symbols -- */

ix_error _ix_cc_tc_meter_free_symbols( _ix_cc_tc_meter_context * arg_pContext )
  {
    IX_ERROR_CHECK_ARG_NULL( 1, arg_pContext );

    IX_ERROR_CRT(
        ix_rm_mem_free( arg_pContext->_pTableBase ),
        IX_CC_ERROR_OOM_SRAM, IX_ERROR_LEVEL_WARNING
      );
   
    arg_pContext->_pStatsBase = NULL;
    arg_pContext->_pTableBase = NULL;

    return IX_SUCCESS;
  }

/* ----------------------------------------------------- ix_cc_tc_meter_fini -- */

ix_error ix_cc_tc_meter_fini( ix_cc_handle arg_hCC, 
                           void *       arg_pUserContext )
  {   
    _ix_cc_tc_meter_context * _pCntxt = 
      (_ix_cc_tc_meter_context *)arg_pUserContext;

    /* validity check for arg_hCC */
    if ( arg_hCC == IX_NULL_HANDLE )
      {
        return IX_ERROR_WARNING(
          IX_CC_ERROR_NULL,
          ( "ix_cc_tc_meter_fini(): core component handle is NULL" )
        );
      }
    
    /* validity check for arg_pUserContext */    
    IX_ERROR_CHECK_ARG_NULL( 2, arg_pUserContext );
    
    if ( arg_hCC != _pCntxt->_hCC )
      {
        return IX_ERROR_WARNING(
          IX_CC_TC_METER_ERROR_INVALID_HANDLE,
          ( "ix_cc_tc_meter_fini(): invalid core component handle" )
        );
      }
   
    /* rm memory de-allocation */
    IX_ERROR_CR( _ix_cc_tc_meter_free_symbols( _pCntxt ) );   
    /* unregister packet handler */
    IX_ERROR_CR( _ix_cc_tc_meter_unregister_pkt_handler() );      
    /* unregister message handler */
    IX_ERROR_CR( _ix_cc_tc_meter_unregister_msg_handler() );

    /* ossl memory de-allocation */          
    _SYSTEM_DELETE( _pCntxt->_pConf );    
    _SYSTEM_DELETE( _pCntxt );

    return IX_SUCCESS;
  }
