/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2000 - 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILENAME
 *      ix_cc_tc_meter_init.c
 *
 * ============================================================================
 */

#define IX_ERROR_FILE_IDENT \
  "$Id: ix_cc_tc_meter_init.c,v 1.15.6.1 2004/03/12 09:56:48 wneubaue Exp $"



#include <ix_types.h>
#include <ix_error.h>

#include <ix_ossl.h>
#include <ix_rm.h>
#include <ix_cc.h>
#include <ix_cc_error.h>
#include <bindings.h>

#include <diffserv.h>

#include "../../include/tc_meter_defs.h"

#include "cc/ix_cc_tc_meter.h"
#include "cc/ix_cc_tc_meter_msg_helpers.h"
#include "cc/internal/ix_cc_tc_meter_defs.h"
#include "cc/internal/ix_cc_tc_meter_types.h"
#include "cc/internal/ix_cc_tc_meter_misc.h"
#include "cc/internal/ix_cc_tc_meter_registry.h"




/* only include this source code if registry support is present */
#if defined( IX_INCLUDE_REGISTRY )

/* ---------------------------------- _ix_cc_tc_meter_get_property_by_handle -- */

PRIVATE ix_error _ix_cc_tc_meter_get_property( const char * arg_pName,
                                           ix_uint32  * arg_pValue )
  {
    ix_cp_property_info _info;
    ix_configuration_property_handle _hProp = IX_NULL_HANDLE;

    /* validity check for arg_pName */
    IX_ERROR_CHECK_ARG_NULL( 1, arg_pName );
    /* validity check for arg_hValue */
    IX_ERROR_CHECK_ARG_NULL( 2, arg_pValue );

    IX_ERROR_CRT( ix_rm_cp_property_open(
      IX_CP_CORE_PROPERTY_ROOT, arg_pName, &_hProp ),
      IX_CC_TC_METER_ERROR_REGISTRY, IX_ERROR_LEVEL_LOCAL );

    IX_ERROR_CR( ix_rm_cp_property_get_info( _hProp, &_info ) );

    /* validity check for property type */
    if ( _info.m_Options & IX_CP_OPTIONS_FLAGS_DATA_UINT32 )
      {
        IX_ERROR_CR( ix_rm_cp_property_get_value_uint32(
          _hProp, arg_pValue ) );
      }
    else
      {
        return IX_ERROR_LOCAL(
            IX_CC_TC_METER_ERROR_REGISTRY,
            ( "Invalid TC Meter registry property type. All subproperties " \
              "must be uint32" )
          );
      }

    IX_ERROR_CRT( ix_rm_cp_property_close( _hProp ),
        IX_CC_TC_METER_ERROR_REGISTRY, IX_ERROR_LEVEL_LOCAL );

    return IX_SUCCESS;
  }

/* ------------------------------------------------ _ix_cc_tc_meter_registry -- */

PRIVATE ix_error
_ix_cc_tc_meter_update_config( _ix_cc_tc_meter_config * arg_pCfg )
{

    IX_ERROR_CR( _ix_cc_tc_meter_get_property(
      IX_RKEY_TC_METER_ENTRY_SRAM_SIZE, &arg_pCfg->_nEntrySize ) );

    IX_ERROR_CR( _ix_cc_tc_meter_get_property(
      IX_RKEY_TC_METER_TABLE_SRAM_SIZE, &arg_pCfg->_nTableSize ) );

    IX_ERROR_CR( _ix_cc_tc_meter_get_property(
      IX_RKEY_TC_METER_SRAM_CHANNEL, &arg_pCfg->_nSRAMChnnl ) );

    IX_ERROR_CR( _ix_cc_tc_meter_get_property(
      IX_RKEY_TC_METER_STATS_ENTRY_SRAM_SIZE, &arg_pCfg->_nStatsSize ) );

     IX_ERROR_CR( _ix_cc_tc_meter_get_property(
      IX_RKEY_TC_METER_MEV2_NUM_MASK, &arg_pCfg->_uEngMask ) );

    return IX_SUCCESS;
}

#endif /* IX_EXCLUDE_REGISTRY */

/* ------------------------------------------- _ix_cc_tc_meter_patch_symbols -- */

PRIVATE ix_error
_ix_cc_tc_meter_patch_symbols( _ix_cc_tc_meter_context * arg_pCntx )
{
    ix_imported_symbol _Symbols[TC_METER_PATCH_SYMBOLS];
    ix_uint32         i          = 0;
    ix_error          err        = IX_SUCCESS;
    ix_memory_type    MemoryType;
    ix_uint32         MemoryChannel;
    ix_uint32         MemoryOffset;
    void*             PhysicalAddress;


    /*--------------------------------------------*/

    err = ix_rm_get_phys_offset(arg_pCntx->_pTableBase,
                                &MemoryType,
                                &MemoryChannel,
                                &MemoryOffset,
                                &PhysicalAddress);

    if (err != IX_SUCCESS)
        return IX_ERROR_WARNING(err,
              ("Error ! Could not convert virtual address to offset."));

    IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(MemoryChannel, MemoryOffset);

    _Symbols[0].m_Value = MemoryOffset;
    _Symbols[0].m_Name  = TC_METER_TABLE_SRAM_BASE;


    /*--------------------------------------------*/



#if defined(TC_METER_PACKET_COUNTER_FEATURE)

    /*--------------------------------------------*/
    
    err = ix_rm_get_phys_offset(arg_pCntx->_pStatsBase,
                                &MemoryType,
                                &MemoryChannel,
                                &MemoryOffset,
                                &PhysicalAddress);

    if (err != IX_SUCCESS)
        return IX_ERROR_WARNING(err,
              ("Error ! Could not convert virtual address to offset."));

    IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(MemoryChannel, MemoryOffset);

    _Symbols[1].m_Value = MemoryOffset;
    _Symbols[1].m_Name  = TC_METER_64BIT_STAT_SRAM_BASE;

    /*--------------------------------------------*/

#endif



    for(i = 0; i < IX_MAX_MICROENGINE_NUMBER; i++) {

        if (arg_pCntx->_pConf->_uEngMask & (0x1 << i)) {

            ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[i],
                                     TC_METER_PATCH_SYMBOLS,
                                     _Symbols);
        }
    }


    return IX_SUCCESS;
  }

/* ------------------------------------ _ix_cc_tc_meter_register_msg_handler -- */

PRIVATE ix_error _ix_cc_tc_meter_register_msg_handler(
    _ix_cc_tc_meter_context * arg_pUserContext )
  {
#if defined( IX_EXCLUDE_CCI )
    IX_ERROR_CRT(
      ix_rm_message_handler_register(
        IX_CC_TC_METER_MSG_INPUT,
        ix_cc_tc_meter_msg_handler,
        arg_pUserContext
      ),
      IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_WARNING );
#else /* IX_EXCLUDE_CCI */
    IX_ERROR_CRT(
      ix_cci_cc_add_message_handler(
        arg_pUserContext->_hCC,
        IX_CC_TC_METER_MSG_INPUT,
        ix_cc_tc_meter_msg_handler,
        IX_INPUT_TYPE_MULTI_SRC
      ),
      IX_CC_TC_METER_ERROR_CCI, IX_ERROR_LEVEL_WARNING );
#endif /* !IX_EXCLUDE_CCI */

    return IX_SUCCESS;
  }

/* ------------------------------------ _ix_cc_tc_meter_register_pkt_handler -- */

PRIVATE ix_error _ix_cc_tc_meter_register_pkt_handler(
    _ix_cc_tc_meter_context * arg_pUserContext )
  {
#if defined( IX_EXCLUDE_CCI )
    IX_ERROR_CRT(
      ix_rm_packet_handler_register(
        IX_CC_TC_METER_PKT_INPUT,
        ix_cc_tc_meter_pkt_handler,
        arg_pUserContext ),
      IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_WARNING
    );
#else /* IX_EXCLUDE_CCI */
    IX_ERROR_CRT(
      ix_cci_cc_add_packet_handler(
        arg_pUserContext->_hCC,
        IX_CC_TC_METER_PKT_INPUT,
        ix_cc_tc_meter_pkt_handler,
        IX_INPUT_TYPE_MULTI_SRC ),
      IX_CC_TC_METER_ERROR_CCI, IX_ERROR_LEVEL_WARNING );
#endif /* !IX_EXCLUDE_CCI */

    return IX_SUCCESS;
  }

/* ----------------------------------------- _ix_cc_tc_meter_reset_timestamp -- */
ix_error _ix_cc_tc_meter_reset_timestamp( _ix_cc_tc_meter_context * arg_pCntxt )
  {
    ix_uint32 _nMask = 0, i, j;

    /* DiffServ microengines */
    i = arg_pCntxt->_pConf->_uEngMask; j = 0;
    while ( i != 0 )
      {
        if ( i & 0x01 )
          {
            _nMask |= (1 << g_aMicroengineNumberMapping[j]);
          }
        i >>= 1; j++;
      }
    #if defined (IX_TC_METER_DEBUG)
	printf("Reset microengines 0x%x \n",_nMask);
    #endif
    halMe_ResetTimestamp( _nMask );

    return IX_SUCCESS;
  }

/* ----------------------------------------------------- ix_cc_tc_meter_init -- */

ix_error ix_cc_tc_meter_init( ix_cc_handle arg_hCC, void ** arg_ppUserContext )
  {
    ix_error  _result = IX_SUCCESS;
    ix_uint32 _i;

    _ix_cc_tc_meter_config  * _pConf;
    _ix_cc_tc_meter_context * _pCntxt;

    /* validity check for arg_hCC */
    if ( arg_hCC == IX_NULL_HANDLE )
      {
        return IX_ERROR_WARNING(
          IX_CC_ERROR_NULL,
          ( "ix_cc_tc_meter_init(): invalid core component handle" )
        );
      }

    /* validity check for arg_ppUserContext */
    IX_ERROR_CHECK_ARG_NULL( 2, arg_ppUserContext );

    /* allocate memory for the context */
    _pCntxt = (_ix_cc_tc_meter_context *)
      ix_ossl_malloc( sizeof( _ix_cc_tc_meter_context ) );

    if (_pCntxt == NULL )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_OOM_SYSTEM,
            ( "not enough system memory available" \
              "(cannot allocate memory for the context)" )
          );
      }

    /* allocate memory for the core component configuration */
    _pConf = (_ix_cc_tc_meter_config *)
      ix_ossl_malloc( sizeof( _ix_cc_tc_meter_config ) );

    if ( _pConf == NULL )
      {
        _SYSTEM_DELETE( _pCntxt );

        return IX_ERROR_WARNING(
            IX_CC_ERROR_OOM_SYSTEM,
            ( "not enough system memory available" \
              "(cannot allocate memory for the configuration)" )
          );
      }

    _pCntxt->_hCC   = arg_hCC;
    _pCntxt->_pConf = _pConf;

    /* read static configuration from the registry (if available) */
    #if defined( IX_INCLUDE_REGISTRY )
      _result = _ix_cc_tc_meter_update_config( _pCntxt->_pConf );
      if ( _result != IX_SUCCESS )
        {
          _SYSTEM_DELETE( _pCntxt->_pConf );
          _SYSTEM_DELETE( _pCntxt );

          return IX_ERROR_WARNING(
              IX_CC_ERROR_INTERNAL,
              ( "static configuration reading failure" )
            );
        }
    #else  /* IX_INCLUDE_REGISTRY */
      _pCntxt->_pConf->_nEntrySize =
        TC_METER_ENTRY_SRAM_SIZE;

      _pCntxt->_pConf->_nSRAMChnnl =
        TC_METER_DEFAULT_SRAM_CHANNEL;

      _pCntxt->_pConf->_nStatsSize =
        TC_METER_64BIT_STAT_SRAM_SIZE;

      _pCntxt->_pConf->_nTableSize =
        TC_METER_DEFAULT_TABLE_SRAM_SIZE;

    #endif /* !IX_INCLUDE_REGISTRY */

    /* initialize tc_meter core component context */
    _result = ix_rm_mem_alloc(
                    IX_MEMORY_TYPE_SRAM,
                    _pCntxt->_pConf->_nSRAMChnnl,
                    (_pConf->_nEntrySize + _pConf->_nStatsSize) * _pConf->_nTableSize,
                    &_pCntxt->_pTableBase);

    if ( _result != IX_SUCCESS )
      {
        _SYSTEM_DELETE( _pCntxt->_pConf );
        _SYSTEM_DELETE( _pCntxt );

        return IX_ERROR_WARNING(
            IX_CC_ERROR_OOM_SRAM,
            ( "ix_cc_tc_meter_init(): not enough SRAM memory available" )
          );
      }

    /* tc_meter statistics table */
    _pCntxt->_pStatsBase =
          (void *)_GET_PTR(
          _pCntxt->_pTableBase,
          _pConf->_nTableSize,
          _pConf->_nEntrySize
        );

    /* zero CIR and statistics */
    for ( _i = 0; _i <= _pConf->_nTableSize; _i++ )
      {
        _ix_cc_tc_meter_table_entry * _pT =
          _TABLE_ENTRY( _i, _pCntxt );

        _ix_cc_tc_meter_stats_entry * _pS =
          _STATS_ENTRY( _i, _pCntxt );

        _pT->_valid        = 0;

        _pT->_red_bytes    = 0;
        _pT->_red_pkts     = 0;
        _pT->_green_bytes  = 0;
        _pT->_green_pkts   = 0;
        _pT->_yellow_pkts  = 0;
        _pT->_yellow_bytes = 0;

        _pS->_green_bytes  = 0;
        _pS->_green_pkts   = 0;
        _pS->_yellow_bytes = 0;
        _pS->_yellow_pkts  = 0;
        _pS->_red_pkts     = 0;
        _pS->_red_bytes    = 0;
      }

    /* reset timestamp on the specified microengines */
    /* _ix_cc_tc_meter_reset_timestamp( _pCntxt ); */

    _result = _ix_cc_tc_meter_patch_symbols( _pCntxt );
    if ( _result != IX_SUCCESS ) goto error_l1;

    _result = _ix_cc_tc_meter_register_pkt_handler( _pCntxt );
    if ( _result != IX_SUCCESS ) goto error_l2;

    _result = _ix_cc_tc_meter_register_msg_handler( _pCntxt );
    if ( _result != IX_SUCCESS ) goto error_l3;
    

    ix_ossl_message_log("ix_cc_tc_meter_init(): success\n");
    *arg_ppUserContext = (void *)_pCntxt;
    return IX_SUCCESS;

error_l3:
    _ix_cc_tc_meter_unregister_pkt_handler();

error_l2:
     _ix_cc_tc_meter_free_symbols( _pCntxt );

error_l1:

    /* something went wrong... cleanning up */
    _SYSTEM_DELETE( _pCntxt->_pConf );
    _SYSTEM_DELETE( _pCntxt );

    return _result;
  }
