/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2000 - 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILENAME
 *      ix_cc_tc_meter_msg_handler.c
 *
 * ============================================================================
 */

#define IX_ERROR_FILE_IDENT \
  "$Id: ix_cc_tc_meter_msg_handler.c,v 1.6.6.1 2004/03/12 09:56:48 wneubaue Exp $"



#include <ix_types.h>
#include <ix_error.h>

#include <ix_ossl.h>
#include <ix_rm.h>
#include <ix_cc.h>
#include <ix_cc_error.h>

#include <bindings.h>

#include <cc/ix_cc_msup.h>

#include "cc/ix_cc_tc_meter.h"
#include "cc/ix_cc_tc_meter_msg_helpers.h"
#include "cc/internal/ix_cc_tc_meter_defs.h"
#include "cc/internal/ix_cc_tc_meter_types.h"
#include "cc/internal/ix_cc_tc_meter_api.h"


/* ---------------------------------------------- ix_cc_tc_meter_msg_handler -- */

ix_error ix_cc_tc_meter_msg_handler( ix_buffer_handle arg_hData, 
                                  ix_uint32        arg_nMsgType, 
                                  void *           arg_pContext )
  {
    ix_error _result   = IX_SUCCESS;    

    void * _pMsg    = NULL;
    void * _pCntx   = NULL;

    ix_uint32 _nRetMsgLength = 0;
    void *    _pRetMsg = NULL;    
    
    /* used to handle function to get meter entry parameters */
    ix_cc_tc_meter_parameters _MeterParams;
    /* used to handle function to get statistics */
    ix_cc_tc_meter_statistics _MeterStatistics;
    
    /* validity check for arg_pContext */
    if ( arg_pContext == NULL )
      {
        return IX_ERROR_WARNING(
          IX_CC_ERROR_NULL,
          ( "ix_cc_tc_meter_msg_data(): arg_pContext is NULL" )
        );
      } 

    /* validity check for the arg_pUserData */
    if ( (arg_nMsgType < IX_CC_COMMON_MSG_ID_FIRST) || 
         (arg_nMsgType >= IX_CC_TC_METER_MSG_LAST) )
      {
        return IX_ERROR_WARNING(
          IX_CC_ERROR_UNDEFINED_MSG,
          ( "ix_cc_tc_meter_msg_data(): invalid message type" )
        );
      }

    /* extract the message */
    IX_MSUP_EXTRACT_MSG(arg_hData, &_pMsg, &_pCntx);
    if ( _pMsg == NULL )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_NULL,
            ( "ix_cc_tc_meter_msg_handler(): invalid buffer" )
          );
      }

    #ifdef IX_TC_METER_DEBUG
    printf("ix_cc_tc_meter_msg_handler(): got message\n");
    #endif

    /* handle the message */
    switch ( arg_nMsgType )
      {
        case IX_CC_TC_METER_MSG_UPDATE_ENTRY:
          {
            _ix_cc_tc_meter_update_entry_data * _pData =
              (_ix_cc_tc_meter_update_entry_data *)_pMsg;
            
            _result = ix_cc_tc_meter_update_entry( 
                _pData->_nInstance, &(_pData->_Params), arg_pContext
              );
            
            break;
          }

        case IX_CC_TC_METER_MSG_GET_STATISTICS:
          { 
            _ix_cc_tc_meter_get_stats_data * _pData =
              (_ix_cc_tc_meter_get_stats_data *)_pMsg;

            _result = ix_cc_tc_meter_get_statistics(
            _pData->_nInstance, &_MeterStatistics, arg_pContext );
          
            _nRetMsgLength = sizeof(_MeterStatistics);
            _pRetMsg       = &_MeterStatistics;
            break;
          }

        case IX_CC_TC_METER_MSG_ADD_ENTRY:
          {
            _ix_cc_tc_meter_add_entry_data * _pData = 
              (_ix_cc_tc_meter_add_entry_data *)_pMsg;
            _result = ix_cc_tc_meter_add_entry(
              _pData->_nInstance, &(_pData->_Params), arg_pContext );

            break;
          }

        case IX_CC_TC_METER_MSG_REMOVE_ENTRY:
          {
            _ix_cc_tc_meter_remove_entry_data * _pData =
              (_ix_cc_tc_meter_remove_entry_data *)_pMsg;

            _result = ix_cc_tc_meter_remove_entry( 
                _pData->_nInstance, arg_pContext );

            break;
          }

        case IX_CC_TC_METER_MSG_GET_PARAMETERS:
          { 
            _ix_cc_tc_meter_get_params_data * _pData =
                        (_ix_cc_tc_meter_get_params_data *)_pMsg;

            _result = ix_cc_tc_meter_get_parameters(
                  _pData->_nInstance, &_MeterParams, arg_pContext );
          
            _nRetMsgLength = sizeof(_MeterParams);
            _pRetMsg       = &_MeterParams;
            break;
          }

        default:
          IX_ERROR_CRT(
              ix_rm_buffer_free_chain( arg_hData ),
              IX_CC_ERROR_INTERNAL, 0 );

          return IX_ERROR_WARNING(
              IX_CC_ERROR_UNDEFINED_MSG,
              ( "ix_cc_tc_meter_msg_handler(): unsupported message type" )
            );
      }

    /* send reply to the client */
    _result = ix_cc_msup_send_reply_msg(
        _pCntx, _pRetMsg, _nRetMsgLength, _result );
    
    if ( _result != IX_SUCCESS )
      {
        _result = IX_ERROR_WARNING(
            IX_CC_ERROR_SEND_FAIL,
            ( "ix_cc_tc_meter_msg_handler(): an error occured" ) 
          );
      }

    IX_ERROR_CRT(
      ix_rm_buffer_free_chain( arg_hData ),
      IX_CC_ERROR_INTERNAL, 0 );

    return _result;
  }
