/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2000 - 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILENAME
 *      ix_cc_tc_meter_msg_helpers.c
 *
 * ============================================================================
 */

#define IX_ERROR_FILE_IDENT \
  "$Id: ix_cc_tc_meter_msg_helpers.c,v 1.6.6.1 2004/03/12 09:56:48 wneubaue Exp $"



#include <ix_types.h>
#include <ix_error.h>

#include <ix_ossl.h>
#include <ix_rm.h>
#include <ix_cc.h>
#include <ix_cc_error.h>
#include <bindings.h>

#include <cc/ix_cc_msup.h>

#include "cc/ix_cc_tc_meter.h"
#include "cc/ix_cc_tc_meter_msg_helpers.h"
#include "cc/internal/ix_cc_tc_meter_defs.h"
#include "cc/internal/ix_cc_tc_meter_types.h"


/* ---------------------------------------------- _ix_cc_tc_meter_cb_default -- */

PRIVATE ix_error _ix_cc_tc_meter_cb_default(ix_error  arg_Result,
                                            void *    arg_pCallbackContext,
                                            void *    arg_pMsg,
                                            ix_uint32 arg_nMsgLength)
{
    _ix_cc_tc_meter_default_cb_context * _pCntx =  
      (_ix_cc_tc_meter_default_cb_context *)arg_pCallbackContext;

    if ( arg_pCallbackContext != NULL )
      {
        ix_error _result =
          _pCntx->_pCallback( arg_Result, _pCntx->_pUserContext );

        _SYSTEM_DELETE( arg_pCallbackContext );
        return _result;
      }
                
    return IX_ERROR_PANIC(
        IX_CC_ERROR_NULL,
        ( "invalid callback context" )
      );
}


/* -------------------------------------------- _ix_cc_tc_meter_cb_get_stats -- */
#if defined(TC_METER_PACKET_COUNTER_FEATURE)
PRIVATE ix_error _ix_cc_tc_meter_cb_get_stats( ix_error  arg_Result, 
                                            void *    arg_pCallbackContext,
                                            void *    arg_pMsg,
                                            ix_uint32 arg_nMsgLength )
  {
    _ix_cc_tc_meter_get_stats_cb_context * _pCntx =
      (_ix_cc_tc_meter_get_stats_cb_context *)arg_pCallbackContext;

    if ( arg_Result == IX_SUCCESS )
      {
        if ( arg_pMsg == NULL )
          {
            return IX_ERROR_WARNING(
                IX_CC_ERROR_NULL,
                ( "NULL message data pointer" )
              );
          }

        if ( arg_nMsgLength != sizeof( ix_cc_tc_meter_statistics ) )
          {
            return IX_ERROR_WARNING(
                IX_CC_ERROR_INTERNAL,
                ( "message length doesn't match expected size" )
              );
          }
      }
    else
      {
        arg_pMsg = NULL;
      }
  
    if ( arg_pCallbackContext != NULL )
      {  
        ix_error _result;

        *(_pCntx->_pUserPlaceholder) =  *(ix_cc_tc_meter_statistics *)arg_pMsg;

        _result = _pCntx->_pCallback(
            arg_Result, 
            _pCntx->_pUserPlaceholder, 
            _pCntx->_pUserContext
          );
    
        _SYSTEM_DELETE( arg_pCallbackContext );
        return _result;
      }

    return IX_ERROR_PANIC(
        IX_CC_ERROR_NULL,
        ( "invalid callback context" )
      );
  }
#endif
/* --------------------------------------------- _ix_cc_tc_meter_cb_get_params -- */

PRIVATE ix_error _ix_cc_tc_meter_cb_get_params( ix_error  arg_Result, 
                                             void *    arg_pCallbackContext,
                                             void *    arg_pMsg,
                                             ix_uint32 arg_nMsgLength )
  {
    _ix_cc_tc_meter_get_params_cb_context * _pCntxt =
      (_ix_cc_tc_meter_get_params_cb_context *)arg_pCallbackContext;

    if ( arg_Result == IX_SUCCESS )
      {
        if ( arg_pMsg == NULL )
          {
            return IX_ERROR_WARNING(
                IX_CC_ERROR_NULL,
                ( "NULL message data pointer specified" )
              );
          }

        if ( arg_nMsgLength != sizeof( ix_cc_tc_meter_parameters ) )
          {
            return IX_ERROR_WARNING(
                IX_CC_ERROR_INTERNAL,
                ( "message length doesn't match expected size" )
              );
          }
      }
    else
      {
        arg_pMsg = NULL;
      }

    
    if ( arg_pCallbackContext != NULL )
    {
        ix_error _result;
        *(_pCntxt->_pUserPlaceholder) =  *(ix_cc_tc_meter_parameters *)arg_pMsg;

        _result = _pCntxt->_pCallback(
            arg_Result,
            _pCntxt->_pUserPlaceholder,
            _pCntxt->_pUserContext 
          );

        _SYSTEM_DELETE( arg_pCallbackContext );
        return _result;
    }

    return IX_ERROR_PANIC(
        IX_CC_ERROR_NULL,
        ( "invalid callback context" )
      );
  }

/* ------------------------------------------ ix_cc_tc_meter_async_add_entry -- */

ix_error ix_cc_tc_meter_async_add_entry( 
    ix_uint32                   arg_nInstance, 
    ix_cc_tc_meter_parameters  *arg_pParams,
    ix_cc_tc_meter_callback_op  arg_Callback,
    void                       *arg_pUserContext )
  {
    ix_error _result = IX_SUCCESS;
    
    _ix_cc_tc_meter_default_cb_context *_cbCntx;
    _ix_cc_tc_meter_add_entry_data      msgData;

    /* validity check for arg_Callback */
    IX_ERROR_CHECK_ARG_NULL( 3, arg_Callback );

    /* allocate memory for the callback context */
    _cbCntx = ix_ossl_malloc( sizeof( _ix_cc_tc_meter_default_cb_context ) );
    if ( _cbCntx == NULL )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_OOM_SYSTEM,
            ( "not enough system memory available" )
          );
      }

    /* callback contex */
    _cbCntx->_pCallback    = arg_Callback;
    _cbCntx->_pUserContext = arg_pUserContext;
    
    /* message data */
    msgData._Params        = *arg_pParams;
    msgData._nInstance     = arg_nInstance;

    #ifdef IX_TC_METER_DEBUG
    printf(" ix_cc_tc_meter_async_add_entry(): sending message\n");
    #endif

    /* send message to the tc_meter core component */
    _result = ix_cc_msup_send_async_msg(
        IX_CC_TC_METER_MSG_INPUT,
        _ix_cc_tc_meter_cb_default,
        (void *)_cbCntx,
        IX_CC_TC_METER_MSG_ADD_ENTRY,
        (void *)&msgData,
        sizeof( _ix_cc_tc_meter_add_entry_data )
      );

    if 	( _result != IX_SUCCESS )
      {
        ix_ossl_free( _cbCntx );

        return IX_ERROR_WARNING(
            IX_CC_ERROR_SEND_FAIL,
            ( "failed to send asynchronous message" )
          );
      }

    return IX_SUCCESS;
  }

/* ---------------------------------------- ix_cc_tc_meter_async_remove_entry -- */

ix_error ix_cc_tc_meter_async_remove_entry( 
    ix_uint32                  arg_nInstance,
    ix_cc_tc_meter_callback_op arg_Callback,
    void *                     arg_pUserContext)
  {
    ix_error _result = IX_SUCCESS;

    _ix_cc_tc_meter_default_cb_context *_cbCntx;
    _ix_cc_tc_meter_remove_entry_data   msgData;

    /* validity check for arg_Callback */
    IX_ERROR_CHECK_ARG_NULL( 2, arg_Callback );

    /* allocate memory for the callback context */
    _cbCntx = ix_ossl_malloc(
        sizeof( _ix_cc_tc_meter_default_cb_context )
      );

    if ( _cbCntx == NULL )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_OOM_SYSTEM,
            ( "not enough system memory available" )
          );
      }

    _cbCntx->_pCallback    = arg_Callback;
    _cbCntx->_pUserContext = arg_pUserContext;

    msgData._nInstance     = arg_nInstance;

    _result = ix_cc_msup_send_async_msg(
        IX_CC_TC_METER_MSG_INPUT,
        _ix_cc_tc_meter_cb_default,
        (void *)_cbCntx,
        IX_CC_TC_METER_MSG_REMOVE_ENTRY,
        (void *)&msgData,
        sizeof( _ix_cc_tc_meter_remove_entry_data )
      );

    if ( _result != IX_SUCCESS )
      {
        ix_ossl_free( _cbCntx );

        return IX_ERROR_WARNING(
            IX_CC_ERROR_SEND_FAIL,
            ( "failed to send asynchronous message" )
          );
      }

    return IX_SUCCESS;
  }

/* --------------------------------------- ix_cc_tc_meter_async_update_entry -- */

ix_error ix_cc_tc_meter_async_update_entry( 
    ix_uint32                   arg_nInstance,
    ix_cc_tc_meter_parameters  *arg_pParams,
    ix_cc_tc_meter_callback_op  arg_Callback,
    void                       *arg_pUserContext )
  {    
    ix_error _result = IX_SUCCESS;

    _ix_cc_tc_meter_default_cb_context * _cbCntx;
    _ix_cc_tc_meter_update_entry_data msgData;

    /* validity check for arg_Callback */
    IX_ERROR_CHECK_ARG_NULL( 3, arg_Callback );

    /* allocate memory for the callback context */
    _cbCntx = ix_ossl_malloc(
        sizeof( _ix_cc_tc_meter_default_cb_context )
      );

    if ( _cbCntx == NULL )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_OOM_SYSTEM,
            ( "not enough system memory available" )
          );
      }

    _cbCntx->_pCallback    = arg_Callback;
    _cbCntx->_pUserContext = arg_pUserContext;

    msgData._Params        = *arg_pParams;
    msgData._nInstance     = arg_nInstance;

    _result = ix_cc_msup_send_async_msg(
        IX_CC_TC_METER_MSG_INPUT,
        _ix_cc_tc_meter_cb_default,
        (void *)_cbCntx,
        IX_CC_TC_METER_MSG_UPDATE_ENTRY,
        (void *)&msgData,
        sizeof( _ix_cc_tc_meter_update_entry_data )
      );

    if ( _result != IX_SUCCESS )
      {
        ix_ossl_free( _cbCntx );

        return IX_ERROR_WARNING(
            IX_CC_ERROR_SEND_FAIL,
            ( "failed to send asynchronous message" )
          );
      }

    return IX_SUCCESS;
  }

/* ------------------------------------- ix_cc_tc_meter_async_get_statistics -- */
#if defined(TC_METER_PACKET_COUNTER_FEATURE)
ix_error ix_cc_tc_meter_async_get_statistics( 
    ix_uint32                   arg_nInstance,
    ix_cc_tc_meter_statistics  *arg_pStats,
    ix_cc_tc_meter_callback_gs  arg_Callback,
    void                       *arg_pUserContext )
  {
    ix_error _result = IX_SUCCESS;

    _ix_cc_tc_meter_get_stats_cb_context *_cbCntx;
    _ix_cc_tc_meter_get_stats_data        msgData;

    /* validity check for arg_Callback */
    IX_ERROR_CHECK_ARG_NULL( 3, arg_Callback );

    /* allocate memory for the callback context */
    _cbCntx = ix_ossl_malloc(
        sizeof( _ix_cc_tc_meter_get_stats_cb_context )
      );

    if ( _cbCntx == NULL )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_OOM_SYSTEM,
            ( "not enough system memory available" )
          );
      }

    _cbCntx->_pCallback        = arg_Callback;
    _cbCntx->_pUserContext     = arg_pUserContext;
    _cbCntx->_pUserPlaceholder = arg_pStats;

    msgData._nInstance     = arg_nInstance;

    _result = ix_cc_msup_send_async_msg(
        IX_CC_TC_METER_MSG_INPUT,
        _ix_cc_tc_meter_cb_get_stats,
        (void *)_cbCntx,
        IX_CC_TC_METER_MSG_GET_STATISTICS,
        (void *)&msgData,
        sizeof( _ix_cc_tc_meter_get_stats_data )
      );

    if ( _result != IX_SUCCESS )
      {
        ix_ossl_free( _cbCntx );

        return IX_ERROR_WARNING(
            IX_CC_ERROR_SEND_FAIL,
            ( "failed to send asynchronous message" )
          );
      }

    return IX_SUCCESS;
  }
#endif


/* ------------------------------------- ix_cc_tc_meter_async_get_parameters -- */
/**
 * @brief Sends a parameters read message to the TC Meter core component
 *
 * This helper function builds and sends a parameters read message to the TC Meter 
 * core component. The core component checks if the meter entry, pointed by 
 * arg_nInstance, is valid. If the referred table entry is empty, the core 
 * component reports an error (via a callback function). Otherwise, it retrieves
 * parameters values associated with this entry and returns them in a callback
 * function. 
 *
 * @param  arg_nInstance    index of a meter table entry that is queried for
 *                          parameters.
 *         
 * @param  arg_pParams      pointer to the parameters placeholder that should
 *                          be filled with data by the core component.
 *         
 * @param  arg_Callback     a client provided callback function. The core 
 *                          component invokes this function to report an 
 *                          operation status and parameters values.
 *
 * @param  arg_pUserContext pointer to the client defined context. This value
 *                          is opaque to the core componet. The context is
 *                          returned in a callback function.
 *
 * @return IX_SUCCESS if successful or a valid ix_error token for failure. 
 *         The following error codes might be returned via the token.
 *
 * @see    <code>ix_cc_tc_meter_callback_gp</code>\n
 *         <code>ix_cc_tc_meter_parameters</code>
 */
ix_error ix_cc_tc_meter_async_get_parameters(
    ix_uint32                   arg_nInstance,
    ix_cc_tc_meter_parameters  *arg_pParams,
    ix_cc_tc_meter_callback_gp  arg_Callback,
    void *                      arg_pUserContext )
  {    
    ix_error _nResult = IX_SUCCESS;

    _ix_cc_tc_meter_get_params_cb_context *_cbCntx;
    _ix_cc_tc_meter_get_params_data        msgData;

    /* validity check for arg_Callback */
    IX_ERROR_CHECK_ARG_NULL( 3, arg_Callback );

    /* allocate memory for the callback context */
    _cbCntx = ix_ossl_malloc( sizeof( _ix_cc_tc_meter_get_params_cb_context ) );
    if ( _cbCntx == NULL )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_OOM,
            ( "not enough system memory available" )
          );
      }

    _cbCntx->_pCallback        = arg_Callback;
    _cbCntx->_pUserContext     = arg_pUserContext;
    _cbCntx->_pUserPlaceholder = arg_pParams;

    msgData._nInstance         = arg_nInstance;

    _nResult = ix_cc_msup_send_async_msg(
        IX_CC_TC_METER_MSG_INPUT,
        _ix_cc_tc_meter_cb_get_params,
        (void *)_cbCntx,
        IX_CC_TC_METER_MSG_GET_PARAMETERS,
        (void *)&msgData,
        sizeof( _ix_cc_tc_meter_get_params_data )
      );

    if ( _nResult != IX_SUCCESS )
      {
        _SYSTEM_DELETE( _cbCntx );

        return IX_ERROR_WARNING(
            IX_CC_ERROR_SEND_FAIL,
            ( "failed to send asynchronous message" )
          );
      }

    return IX_SUCCESS;
  }
