#ifndef WRED_DEFS_H
#define WRED_DEFS_H

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/
/* "$Id: wred_defs.h,v 1.11.4.1 2004/03/12 10:20:51 wneubaue Exp $" */

/* FILE  wred_defs.h
 *
 * Definitions relevant to both WRED microblock and core componenet
 */

/*
 * Amount of SDRAM space taken by one entry with 32 high bits of wred/queue
 * statistics in bytes
 */
#define     WRED_64BIT_STAT_SRAM_SIZE        32


/*
 * size (in bytes) of the entry of wred statistics table where
 * the most significant long words of WRED statistics are held
 * rule: int[Log2(WRED_64BIT_STAT_RAM_SIZE)]
 */
#define     WRED_64BIT_STAT_LOG_SRAM_SIZE     5


/*
 * maximum rate at which queue can be emptied (in packets per second)
 */
#define     WRED_MAX_PACKET_RATE              6000000



/*
 * These values cannot be changed because of the code optimalization
 * which makes use of the direct lm_addr0 load during CAM lookup.
 */


/*
 * size of WRED entry in LM (in bytes)
 */
#define     WRED_ENTRY_LMEM_SIZE              64


/*
 * size of WRED entry in RAM (in bytes)
 */
#ifdef WRED_PACKET_COUNTER_FEATURE
#define     WRED_ENTRY_RAM_SIZE               64
#else 
#define     WRED_ENTRY_RAM_SIZE               64  /* 32 - but xml not capable */
#endif


/*
 * size of WRED entry in RAM expressed in the power of 2 (in bytes)
 * rule: int[Log2(WRED_ENTRY_RAM_SIZE)]
 */
#ifdef WRED_PACKET_COUNTER_FEATURE
#define     WRED_ENTRY_LOG_RAM_SIZE           6
#else
#define     WRED_ENTRY_LOG_RAM_SIZE           6   /* 5 - but xml not capable */
#endif

/*
 * values to initialize CSR timestamp register on ME where WRED is to work
 * this must be synchronized with CSR timestamp on ME with Queue Manager
 */
#define     WRED_INIT_TIMESTAMP_LOW           0
#define     WRED_INIT_TIMESTAMP_HIGH          0


/* The default number of entries in a WRED table and statistics table */
#define     WRED_DEFAULT_TABLE_RAM_SIZE       0x0100


/* The default RAM memory's chanell number for a WRED Core Component tables */
#define     WRED_DEFAULT_RAM_CHANNEL          0x0



#endif
