#ifndef WRED_INIT_UC
#define WRED_INIT_UC

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

// FILE  wred_init.uc
//
// Implements init and final macros of a wred block.
//

// Exported APIs:
//   wred_init()
//   wred_term()
//

//------------------------------------------------------------------------------
// Included files:
#include <sig_macros.uc>


// --------------------------------------------
#ifdef WRED_DIFFSERV_DEBUG
    .reg @debug_came_in
    .reg @debug_came_out
    .reg @debug_dropped
#endif
// --------------------------------------------


//------------------------------------------------------------------------------
// #macro wred_init()

// wred_init
//
// Description:
//   Initializes CSR timestamp_low and timestamp_high registers 
//   but do not start the timer. Because the start of time measuring affects
//   simultaneously all ME's it should be done by system_init macro
//   It feeds a pseudo random number generator with a seed defined as
//   WRD_INIT_RANDOM
//
#macro    wred_init()
.begin

    br!=ctx[0,init_relative#]

init_absolute#:

    .reg init_random
    move(init_random, WRED_INIT_RANDOM)
    local_csr_wr[PSEUDO_RANDOM_NUMBER, init_random]

      // --------------------------------------------
    #ifdef WRED_DIFFSERV_DEBUG
        alu[@debug_came_in, --, B, 0]
        alu[@debug_came_out, --, B, 0]
        alu[@debug_dropped, --, B, 0]
    #endif
    // --------------------------------------------

init_relative#:

    immed32(queue_base, QD_SRAM_BASE)
    // 8 is added to point at the third longword 
    // of a queue-descriptor structure
    alu[queue_base, queue_base, +, 8]

.end
#endm


//------------------------------------------------------------------------------
// #macro  wred_term()

// wred_term
//
// Description:
//   Terminates the operation of a meter block
//
#macro    wred_term()
#endm


#endif  // WRED_INIT_UC