/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2000 - 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILENAME
 *      ix_cc_wred_api.h
 *
 * = DESCRIPTION
 *      WRED Core Component internals - Libray API
 *      This file defines function prototypes of the Library API
 *
 * ============================================================================
 */

#if !defined(IX_CC_WRED_API_H)
#define      IX_CC_WRED_API_H



#if defined( __cplusplus )
extern "C" {
#endif /* __cplusplus */



/* ix_cc_red_instance */

typedef struct ix_s_cc_red_instance 
  {
    ix_uint8  min_th;
    ix_uint8  max_th;
    ix_uint16 max_prob;
  } ix_cc_red_instance;

/* ix_cc_red_instance */

typedef struct ix_s_cc_wred_parameters 
  {
    ix_uint32 service_rate;
    ix_uint8  weight;
    ix_cc_red_instance green;
    ix_cc_red_instance yellow;
    ix_cc_red_instance red;
    ix_uint8  statsEnabled;
  } ix_cc_wred_parameters;

/* ix_s_cc_wred_parameters */

typedef struct ix_s_cc_wred_statistics
  {
    ix_uint64 greenPacketCount;
    ix_uint64 greenByteCount;
    ix_uint64 yellowPacketCount;
    ix_uint64 yellowByteCount;
    ix_uint64 redPacketCount;
    ix_uint64 redByteCount;
  } ix_cc_wred_statistics;


/* _ix_s_cc_wred_add_entry_data */
typedef struct _ix_s_cc_wred_add_entry_data
  {
    ix_uint32             _nInstance;
    ix_cc_wred_parameters _WredParams;
  } _ix_cc_wred_add_entry_data;


typedef struct _ix_s_cc_wred_add_entry_data_ctx
{    
    void                       *PCoreContext;
    void                       *PUserContext;
    _ix_cc_wred_add_entry_data  EntryData;
    ix_buffer_handle            ReplyMsgBuffer;
} _ix_cc_wred_add_entry_data_ctx;


/* _ix_s_cc_wred_remove_entry_data */
typedef struct _ix_s_cc_wred_remove_entry_data
  {
    ix_uint32 _nInstance;
  } _ix_cc_wred_remove_entry_data;


/* _ix_s_cc_wred_update_entry_data */
typedef struct _ix_s_cc_wred_update_entry_data
  {
    ix_uint32 _nInstance;
    ix_cc_wred_parameters  _Params;
  } _ix_cc_wred_update_entry_data;


/* _ix_s_cc_wred_get_stats_data */
typedef struct _ix_s_cc_wred_get_stats_data
  {
    ix_uint32 _nInstance;
  } _ix_cc_wred_get_stats_data;


/* _ix_s_cc_wred_get_params_data */
typedef struct _ix_s_cc_wred_get_params_data
  {
    ix_uint32 _nInstance;
  } _ix_cc_wred_get_params_data;


/** 
 * @brief  This function removes an entry from the WRED table.
 *
 * The core component first checks if an entry, pointed by arg_Instance, is 
 * valid. If true, it marks the entry as empty and zeroes the statistics. 
 * Otherwise, if the referred entry is already marked empty, the core component
 * reports an error in a callback function. 
 *
 * The empty entries are marked with special value of service_time equal to 0xff. 
 * Note that the WRED microblock does not check the field  against the special 
 * value. It behaves as if the entry was not empty. For this reason the empty 
 * entries must have the max_th and min_th fields for all colors equal to maximal 
 * 16-bit value (0xffff). This ensures that the microblock does not drop packets 
 * for packets directed to the queue. It also zeroes all the statistic fields. 
 * Additionally the core component sets the xxx_const fields for all the packet 
 * colors to zero.
 *
 * @param  arg_nInstance index of a WRED table entry that should be removed. 
 *                       The value must not be higher than WRED\TABLE_RAM_SIZE 
 *                       system property.
 *
 * @param  arg_pContext  pointer to the control block allocated in
 *                       <code>ix_cc_wred_init()</code>
 *
 * @return IX_SUCCESS if successful or a valid ix_error token for failure. The 
 *         following error codes might be returned via the token.
 *\n        
 *         IX_CC_ERROR_ENTRY_NOT_FOUND - entry does not exist
 *         IX_CC_ERROR_RANGE           - WRED instance out of range
 *
 * @see    ix_cc_wred_add_entry()
 */
ix_error ix_cc_wred_remove_entry( ix_uint32 arg_nInstance,
                                  void *    arg_pContext );

/**
 * @brief  This function adds a new antry to the WRED table.
 *
 * 
 */
ix_error ix_cc_wred_add_entry( ix_uint32              arg_nInstance,
                               ix_cc_wred_parameters *arg_pInstanceParams,
                               void                  *arg_pQueueDescriptor,
                               void                  *arg_pContext );



/**
 */
ix_error ix_cc_wred_update_entry( ix_uint32               arg_nInstance,
                                  ix_cc_wred_parameters * arg_pParams,
                                  void *                  arg_pContext );

/**
 */
ix_error ix_cc_wred_get_statistics( ix_uint32               arg_nInstance,
                                    ix_cc_wred_statistics * arg_pStats,
                                    void *                  arg_pContext );

/** 
 * @brief  This function gets parameters of an entry from the WRED table.
 *
 * The core component first checks if an entry, pointed by arg_Instance, is 
 * valid. If true, it reads the entry parameters to the arg_pParams argument. 
 * Otherwise, if the referred entry is marked empty, the core component
 * reports an error in a callback function. 
 *
 *
 * @param  arg_nInstance index of a WRED table entry. 
 *                       The value must not be higher than WRED\TABLE_RAM_SIZE 
 *                       system property.
 *
 * @param  arg_pParams  pointer to the parameters structure which should be 
 *                      filled with entry parameters.
 *
 * @param  arg_pContext  pointer to the control block allocated in
 *                       <code>ix_cc_wred_init()</code>
 *
 * @return IX_SUCCESS if successful or a valid ix_error token for failure. The 
 *         following error codes might be returned via the token.
 *\n        
 *         IX_CC_ERROR_ENTRY_NOT_FOUND - entry does not exist
 *         IX_CC_ERROR_RANGE           - WRED instance out of range
 *
 * @see    ix_cc_wred_add_entry()
 */
ix_error ix_cc_wred_get_parameters( ix_uint32               arg_nInstance, 
                                    ix_cc_wred_parameters * arg_pParams,
                                    void *                  arg_pContext );



#define _GET_INT_CONST(arg_pS, arg_pColor )                                            \
                   ((arg_pS->##arg_pColor.max_prob /                               \
                   (arg_pS->##arg_pColor.max_th - arg_pS->##arg_pColor.min_th)))

#define _GET_FLOAT_CONST(arg_pS, arg_pColor )                                              \
                   ( (float)(arg_pS->##arg_pColor.max_prob) /                             \
                     (float)(arg_pS->##arg_pColor.max_th - arg_pS->##arg_pColor.min_th) )



#define _SERVICE_TIME( arg_nSvrRate ) \
  (ix_uint8)_int( _LOG_2( \
      WRED_UNIT_CHANGE_FACTOR / (double)(arg_nSvrRate) ) \
  )




#if defined( __cplusplus )
}
#endif /* __cplusplus */

#endif /* IX_CC_WRED_API_H */
