/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2000 - 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILENAME
 *      ix_cc_wred_defs.h
 *
 * ============================================================================
 */

#if !defined(IX_CC_WRED_DEFS_H)
#define      IX_CC_WRED_DEFS_H




/* unit change factor */
#define WRED_UNIT_CHANGE_FACTOR     (double)37500000.00

/* table valid entry identificator - entry not empty*/
#define WRED_ENTRY_VALID            0x1

/* color's invalid const value */
#define WRED_NOT_INITIALIZED        (ix_uint16)(-1)

/**
 * Number and names of WRED symbols to patch
 */
#if defined(WRED_PACKET_COUNTER_FEATURE)
#define WRED_PATCH_SYMBOLS          2
#else
#define WRED_PATCH_SYMBOLS          1
#endif

/**
 * Registry defines
 */
#define WRED_PROP_MAX_NAME 128
#define WRED_MAX_PREFIX      6 

/* the location of the wred table depends on 
 * WRED_DRAM_TABLE the definition.
 */
#define WRED_TABLE_RAM_BASE        "WRED_TABLE_RAM_BASE"
#if defined(WRED_PACKET_COUNTER_FEATURE)
#define WRED_64BIT_STAT_SRAM_BASE   "WRED_64BIT_STAT_SRAM_BASE"
#endif

/* ix_cc_wred_messages */

typedef enum ix_e_cc_wred_messages
  {
    /* wred table messages */
    IX_CC_WRED_MSG_ADD_ENTRY = IX_CC_COMMON_MSG_ID_LAST,
    IX_CC_WRED_MSG_REMOVE_ENTRY,
    IX_CC_WRED_MSG_UPDATE_ENTRY,

    /* statistics messages */
    IX_CC_WRED_MSG_GET_STATISTICS,

    /* get parameters messages */
    IX_CC_WRED_MSG_GET_PARAMETERS,

    /* add new messages before this */
    IX_CC_WRED_MSG_LAST
  } ix_cc_wred_messages;

/* user defined macros */

/* IX_CC_WRED_MAKE_UINT64 */
#define IX_CC_WRED_MAKE_UINT64( arg_nInt32L, arg_nInt32H ) \
  (((ix_uint64)(arg_nInt32H) << 32) + (ix_uint32)(arg_nInt32L))

/**
 * This macro frees the memory allocated in the system and sets the pointer
 * to NULL.
 *
 * @param arg_pPtr the address of the memory to be freed.
 */
#define _SYSTEM_DELETE( arg_pPtr ) \
  if ( ((void *)arg_pPtr) != NULL ) \
    { \
      ix_ossl_free( (void *)arg_pPtr ); \
      ((void *)arg_pPtr) = NULL; \
    } \

/* _FRAC */
#define _FRAC( arg_fValue ) \
  fabs( (double)(arg_fValue) - (double)(ix_uint32)(arg_fValue) )

/* _BIT_FLD */
#define _BIT_FLD( arg_bValue ) \
  (((ix_uint8)(arg_bValue) != 0) ? 1 : 0)

/* _LOG_2 */
#define _LOG_2( arg_fValue ) \
  (log( (double)(arg_fValue) ) / (double)0.69314718056)

/* _GET_OFFSET */
#define _GET_OFFSET( arg_nIndex, arg_nSize ) \
  ((ix_uint32)(arg_nIndex) * (ix_uint32)(arg_nSize))

/* _GET_PTR */
#define _GET_PTR( arg_pBase, arg_nIndex, arg_nSize ) \
  ((ix_uint8 *)(arg_pBase) + _GET_OFFSET( arg_nIndex, arg_nSize ))

/* _WRED_TABLE_BASE */
#define _WRED_TABLE_BASE( arg_pCntxt ) \
  ((_ix_cc_wred_context *)(arg_pCntxt))->_pTableBase

/* _WRED_STATS_BASE */
#define _WRED_STATS_BASE( arg_pCntxt ) \
  ((_ix_cc_wred_context *)(arg_pCntxt))->_pStatsBase

/* _WRED_CONFIG */
#define _WRED_CONFIG( arg_pCntxt ) \
  ((_ix_cc_wred_context *)(arg_pCntxt))->_pConf

/* _WRED_TABLE_ENTRY_SIZE */
#define _WRED_TABLE_ENTRY_SIZE( arg_pCntxt ) \
  _WRED_CONFIG( arg_pCntxt )->_nEntrySize

/* _WRED_STATS_ENTRY_SIZE */
#define _WRED_STATS_ENTRY_SIZE( arg_pCntxt ) \
  _WRED_CONFIG( arg_pCntxt )->_nStatsSize

/* _RAM_SIZE */
#define _RAM_SIZE( arg_pCntxt ) \
  _WRED_CONFIG( arg_pCntxt )->_nRAMChnnl

/* _WRED_MAX_FLOW_IDS */
#define _WRED_MAX_FLOW_IDS( arg_pCntxt ) \
  _WRED_CONFIG( arg_pCntxt )->_nTableSize

/* _WRED_TABLE_ENTRY */
#define _WRED_TABLE_ENTRY( arg_nInstance, arg_pCntxt ) \
  (_ix_cc_wred_table_entry *) \
    _GET_PTR( \
        _WRED_TABLE_BASE( arg_pCntxt ), \
        arg_nInstance, \
        _WRED_TABLE_ENTRY_SIZE( arg_pCntxt ) \
      ) \

/* _WRED_STATS_ENTRY */
#define _WRED_STATS_ENTRY( arg_nInstance, arg_pCntxt ) \
  (_ix_cc_wred_stats_entry *) \
    _GET_PTR( \
        _WRED_STATS_BASE( arg_pCntxt ), \
        arg_nInstance, \
        _WRED_STATS_ENTRY_SIZE( arg_pCntxt ) \
      ) \




#endif /* IX_CC_WRED_DEFS_H */
