/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2000 - 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILENAME
 *      ix_cc_wred_types.h
 *
 * ============================================================================
 */

#if !defined(IX_CC_WRED_TYPES_H)
#define      IX_CC_WRED_TYPES_H




/* _ix_cc_wred_config */
typedef struct _ix_s_cc_wred_config
{
    ix_uint32            _nEntrySize;
    #ifdef WRED_PACKET_COUNTER_FEATURE
    ix_uint32            _nStatsSize;
    ix_uint32            _nStatsSRAMChnnl;
    #endif
    ix_uint32            _nTableSize;
    ix_uint32            _nTableRAMChnnl;
    ix_uint32            _uEngMask;
    ix_uint32            _qm_uEngNum;
} _ix_cc_wred_config;


#if defined( WIN32 )
#pragma pack( push, 1 )
#endif /* WIN32 */
#if defined( WIN32 )
#pragma pack( pop )
#endif /* WIN32 */


/* -------------------------------------------- */
#ifdef WRED_CORE_PROBABILITY_CALCULATION




/* _ix_cc_wred_table_entry for LOW-SPEED queues */
typedef struct _ix_s_cc_wred_table_entry
{
    ix_uint16 _reserved_0          : 14,
              _valid               : 1,
              _green_stats_enabled : 1;  /*  0 */
    ix_uint16 _green_probability;        /*  1 */
    ix_uint32 _green_count;              /*  2 */

    #ifdef WRED_PACKET_COUNTER_FEATURE
    ix_uint32 _green_pkts;               /*  3 */
    ix_uint32 _green_bytes;              /*  4 */
    #else
    ix_uint32 _reserved_1;               /*  3 */
    ix_uint32 _reserved_2;              /*  4 */
    #endif

    ix_uint16 _reserved_3           : 15,
              _yellow_stats_enabled : 1; /*  5 */
    ix_uint16 _yellow_probability;       /*  6 */
    ix_uint32 _yellow_count;             /*  7 */

    #ifdef WRED_PACKET_COUNTER_FEATURE
    ix_uint32 _yellow_pkts;              /*  8 */
    ix_uint32 _yellow_bytes;             /*  9 */
    #else
    ix_uint32 _reserved_4;               /*  3 */
    ix_uint32 _reserved_5;              /*  4 */
    #endif

    ix_uint16 _reserved_6           : 15,
              _red_stats_enabled    : 1; /* 10 */
    ix_uint16 _red_probability;          /* 11 */
    ix_uint32 _red_count;                /* 12 */

    #ifdef WRED_PACKET_COUNTER_FEATURE
    ix_uint32 _red_pkts;                 /* 13 */
    ix_uint32 _red_bytes;                /* 14 */
    #else
    ix_uint32 _reserved_7;               /*  3 */
    ix_uint32 _reserved_8;              /*  4 */
    #endif

} _ix_cc_wred_table_entry;



typedef struct _ix_s_cc_wred_updater_entry
{
    ix_cc_wred_parameters    wred_configuartion;
    _ix_cc_wred_table_entry *wred_entry;
    ix_event_handle          event_handle;
    ix_uint32               *p_queue_length;
    double                   average_length;
    float                    green_const;
    float                    yellow_const;
    float                    red_const;
} _ix_cc_wred_updater_entry;


#else /* -----------------------------------------*/



/* _ix_cc_wred_table_entry for LOW-SPEED queues */
typedef struct _ix_s_cc_wred_table_entry
{
    ix_uint16 _green_max_th;
    ix_uint16 _green_min_th;
    ix_uint16 _yellow_max_th;
    ix_uint16 _yellow_min_th;
    ix_uint16 _red_max_th;
    ix_uint16 _red_min_th;
    ix_uint16 _green_const;
    ix_uint16 _service_time  : 8,
              _stats_enabled : 1,
              _valid         : 1,
              _r2            : 1,
              _weight        : 5;
    ix_uint16 _yellow_const;
    ix_uint16 _red_const;
    ix_uint16 _green_count;
    ix_uint16 _avg_length;
    ix_uint16 _yellow_count;
    ix_uint16 _red_count;

    #ifdef WRED_PACKET_COUNTER_FEATURE
    ix_uint32 _green_pkts;
    ix_uint32 _yellow_pkts;
    ix_uint32 _red_pkts;
    ix_uint32 _green_bytes;
    ix_uint32 _yellow_bytes;
    ix_uint32 _red_bytes;
    #endif

} _ix_cc_wred_table_entry;


#endif
/* -------------------------------------------- */


#ifdef WRED_PACKET_COUNTER_FEATURE

/* _ix_cc_wred_stats_entry */
typedef struct _ix_s_cc_wred_stats_entry
  {
    ix_uint32 _green_pkts;
    ix_uint32 _yellow_pkts;
    ix_uint32 _red_pkts;
    ix_uint32 _green_bytes;
    ix_uint32 _yellow_bytes;
    ix_uint32 _red_bytes;
  } _ix_cc_wred_stats_entry;

#endif





/* _ix_cc_wred_context */
typedef struct _ix_s_cc_wred_context
{
    void                      *_pTableBase;

    #ifdef WRED_PACKET_COUNTER_FEATURE
    void                      *_pStatsBase;
    #endif

    _ix_cc_wred_config        *_pConf;
    ix_cc_handle               _hCC;

    #ifdef WRED_CORE_PROBABILITY_CALCULATION
    _ix_cc_wred_updater_entry *pUpdatersTable;
    #endif

} _ix_cc_wred_context;




#endif /* IX_CC_WRED_TYPES_H */
