/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2000 - 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILENAME
 *      ix_cc_wred.h
 *
 * = DESCRIPTION
 *      DiffServ/WRED core component External API
 *      This file defining function prototypes of the SRTCM core component
 *      External API.
 *
 * ============================================================================
 */

#if !defined(IX_CC_WRED_H)
#define      IX_CC_WRED_H


/* The default SRAM memory's chanell number for a WRED Core Component tables */
#define IX_CC_WRED_0_DEFAULT_SRAM_CHANNEL               0x0000
#define IX_CC_WRED_1_DEFAULT_SRAM_CHANNEL               0x0001

typedef struct ix_cc_s_instance_map_wred
{
    ix_uint32 msgIn;
    ix_uint32 pktIn;  
    ix_uint32 pktOut;
    ix_uint32 defaultRamChnnl;  
} ix_cc_instance_map_wred;

#define IX_CC_WRED_INSTANCE_TABLE_DEFINE() \
const ix_cc_instance_map_wred aWredInstanceTable[] = { 

#define IX_CC_WRED_INSTANCE_ENTRY_DEFINE(msgIn, pktIn, pktOut, defaultRamChnnl) \
        { msgIn, pktIn, pktOut, defaultRamChnnl },

#define IX_CC_WRED_INSTANCE_TABLE_END {0} };

#define IX_CC_WRED_INSTANCE_TABLE_GET(instance,paramName) \
        aWredInstanceTable[instance]. ## paramName

#define IX_CC_WRED_INSTANCE_TABLE_DECLARE() \
extern ix_cc_instance_map_wred aWredInstanceTable[];

/* _ix_cc_wred_error_code */

typedef enum ix_e_cc_wred_error_code
  {
    IX_CC_WRED_ERROR_FIRST   = IX_ERROR_MAKE_GROUP( CC_WRED ),
    IX_CC_WRED_ERROR_SUCCESS = IX_CC_WRED_ERROR_FIRST,

    IX_CC_WRED_ERROR_FAILED_PATCHING,
    IX_CC_WRED_ERROR_REGISTRY,

    IX_CC_WRED_ERROR_CCI,

    IX_CC_WRED_ERROR_INVALID_CONTEXT,
    IX_CC_WRED_ERROR_INVALID_PACKET,
    IX_CC_WRED_ERROR_INVALID_PARAM,
    IX_CC_WRED_ERROR_INVALID_HANDLE,
    IX_CC_WRED_ERROR_INVALID_BUFFER,

    /* unknown error code */
    IX_CC_WRED_ERROR_UNKNOWN,
    IX_CC_WRED_ERROR_LAST
  } ix_cc_wred_error_code;

#if defined( __cplusplus )
extern "C" {
#endif /* __cplusplus */

/**
 * @brief    Initialization fo the WRED core component.
 *
 * @param arg_hCC
 * @param arg_ppUserContext
 */
ix_error ix_cc_wred_init( ix_cc_handle arg_hCC,
                          void **      arg_ppUserContext );

/* ix_cc_wred_fini */

ix_error ix_cc_wred_fini( ix_cc_handle arg_hCC,
                          void *       arg_pUserContext );

/* ix_cc_wred_msg_handler */

ix_error ix_cc_wred_msg_handler( ix_buffer_handle arg_hBuffer,
                                 ix_uint32        arg_nMsgType,
                                  void *          arg_pUserContext );

/* ix_cc_wred_pkt_handler */

ix_error ix_cc_wred_pkt_handler( ix_buffer_handle arg_hBuffer,
                                 ix_uint32        arg_nExCode,
                                 void *           arg_pUserContext );

#if defined( __cplusplus )
}
#endif /* __cplusplus */



#endif /* IX_CC_WRED_H */
