/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2000 - 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILENAME
 *      ix_cc_wred_msghlp.h
 *
 * = DESCRIPTION
 *      DiffServ/WRED core component External API
 *      This file defining function prototypes of the SRTCM core component
 *      External API.
 *
 * ============================================================================
 */

#if !defined(IX_CC_WRED_MSGHLP_H)
#define      IX_CC_WRED_MSGHLP_H


#if defined( __cplusplus )
extern "C" {
#endif /* __cplusplus */




/* getting statistics callback definition */
typedef ix_error (* ix_cc_wred_callback_get_stats)
  ( ix_error arg_Result, ix_cc_wred_statistics * arg_pStats, void * arg_pUserContext );

/* default callback definition  */
typedef ix_error (* ix_cc_wred_callback_default)
  ( ix_error arg_Result, void * arg_pUserContext );

/* _ix_cc_wred_default_cb_context */
typedef struct _ix_s_cc_wred_default_cb_context
  {
    ix_cc_wred_callback_default _pCallback;
    void * _pUserContext;
  } _ix_cc_wred_default_cb_context;



#ifdef WRED_PACKET_COUNTER_FEATURE

/* _ix_cc_wred_get_stats_cb_context */
typedef struct _ix_cc_wred_get_stats_cb_context
{
    ix_cc_wred_callback_get_stats  _pCallback;
    ix_cc_wred_statistics         *_pUserPlaceHolder;
    void                          *_pUserContext;

} _ix_cc_wred_get_stats_cb_context;

#endif



  /**
 * @brief Parameters read callback function definition
 *
 * This callback function definition is provided for parameters reading 
 * operation only.
 *
 * @param arg_Result       indicates error conditions for the call: IX_SUCCESS 
 *                         if successful or a valid ix_error token for failure. 
 *                         The following error codes might be returned via the
 *                         token:
 * \n
 * \n                      IX_CC_ERROR_ENTRY_NOT_FOUND - 
 *                                entry is empty (not yet configured).
 *
 * \n                      IX_CC_ERROR_RANGE           - 
 *                                entry index is out of range; the instance 
 *                                identifier is higher than the maximum number 
 *                                of instances that can be configured in the 
 *                                block.
 * \n                        
 * @param arg_pParamsResult pointer to the parameters structure filled with 
 *                         valid data if a WRED entry is not empty.
 *
 * @param arg_pContext     pointer to the client-defined context, provided as
 *                         a parameter in the API call. Used by the caller to
 *                         match asynchronous request with a response.
 *
 * @see   <code>ix_cc_wred_async_get_parameters()</code>,
 *        <code>ix_cc_wred_parameters</code>
 */
typedef ix_error (* ix_cc_wred_callback_get_params)
  ( ix_error arg_Result, 
    ix_cc_wred_parameters * arg_pParams, 
    void * arg_pUserContext );


/* _ix_cc_wred_get_params_cb_context */
typedef struct _ix_cc_wred_get_params_cb_context
  {
    ix_cc_wred_callback_get_params  _pCallback;
    ix_cc_wred_parameters          *_userPlaceHolder;
    void                           *_pUserContext;
  } _ix_cc_wred_get_params_cb_context;


/* ix_cc_wred_async_add_entry */

ix_error ix_cc_wred_async_add_entry( 
    ix_uint32                   arg_nInstance,
    ix_cc_wred_parameters *     arg_pParams,
    ix_cc_wred_callback_default arg_Callback,
    ix_generic_context *        arg_pUserContext
  );

/* ix_cc_wred_async_remove_entry */

ix_error ix_cc_wred_async_remove_entry(
    ix_uint32                   arg_nInstance,
    ix_cc_wred_callback_default arg_Callback,
    ix_generic_context *        arg_pUserContext
  );

/* ix_cc_wred_async_update_entry */

ix_error ix_cc_wred_async_update_entry(
    ix_uint32                   arg_nInstance,
    ix_cc_wred_parameters *     arg_pParams,
    ix_cc_wred_callback_default arg_Callback,
    ix_generic_context *        arg_pUserContext
  );


#ifdef WRED_PACKET_COUNTER_FEATURE

/* ix_cc_wred_async_get_statistics */
ix_error ix_cc_wred_async_get_statistics(
    ix_uint32                     arg_nInstance,
    ix_cc_wred_statistics *       arg_pStats,
    ix_cc_wred_callback_get_stats arg_pCallback,
    ix_generic_context *          arg_pUserContext
  );

#endif

/**
 * @brief Sends a parameters read message to the WRED core component
 *
 * This helper function builds and sends a parameters read message to the WRED 
 * core component. The core component checks if the WRED entry, pointed by 
 * arg_nInstance, is valid. If the referred table entry is empty, the core 
 * component reports an error (via a callback function). Otherwise, it retrieves
 * parameters values associated with this entry and returns them in a callback
 * function. 
 *
 * @param  arg_nInstance    index of a WRED table entry that is queried for
 *                          parameters.
 *         
 * @param  arg_pParams      pointer to the parameters placeholder that should
 *                          be filled with data by the core component.
 *         
 * @param  arg_Callback     a client provided callback function. The core 
 *                          component invokes this function to report an 
 *                          operation status and parameters values.
 *
 * @param  arg_pUserContext pointer to the client defined context. This value
 *                          is opaque to the core componet. The context is
 *                          returned in a callback function.
 *
 * @return IX_SUCCESS if successful or a valid ix_error token for failure. 
 *         The following error codes might be returned via the token.
 *
 * @see    <code>ix_cc_wred_callback_get_params</code>\n
 *         <code>ix_cc_wred_parameters</code>
 */
ix_error ix_cc_wred_async_get_parameters(
    ix_uint32                      arg_nInstance,
    ix_cc_wred_parameters *        arg_pParams,
    ix_cc_wred_callback_get_params arg_Callback,
    ix_generic_context *           arg_pUserContext );










#if defined( __cplusplus )
}
#endif /* __cplusplus */


#endif /* IX_CC_WRED_MSGHLP_H */
