/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2000 - 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILENAME
 *      ix_cc_wred_api.c
 *
 * ============================================================================
 */
#define IX_ERROR_FILE_IDENT "$Id: ix_cc_wred_api.c,v 1.23.2.1 2004/03/12 10:05:17 wneubaue Exp $"

#include <math.h>

#include <ix_types.h>
#include <ix_error.h>
#include <ix_ossl.h>
#include <ix_cc.h>
#include <ix_cc_macros.h>
#include <ix_cc_error.h>
#include <bindings.h>

#include "../../include/wred_defs.h"
#include "cc/ix_cc_wred.h"
#include "cc/internal/ix_cc_wred_defs.h"
#include "cc/internal/ix_cc_wred_api.h"
#include "cc/internal/ix_cc_wred_types.h"
#include "cc/internal/ix_cc_wred_misc.h"
#include "cc/internal/ix_cc_wred_defaults.h"

#if defined( IX_DEBUG_WRED )
extern show_mem(void*, int);
#endif /* IX_DEBUG_WRED */

PRIVATE ix_uint32 _int( double arg_fValue )
{
   return (_FRAC( arg_fValue ) <= 0.5) ?
       (ix_uint32)arg_fValue : ((ix_uint32)arg_fValue + 1);
}


/* ---------------------------------------- _ix_cc_wred_periodic_updater -- */
#ifdef WRED_CORE_PROBABILITY_CALCULATION
ix_error
_ix_cc_wred_periodic_updater(void *arg_pContext)
{

    _ix_cc_wred_updater_entry  *updater_instance = arg_pContext;
    ix_uint32                   queue_length;


    queue_length = *(updater_instance -> p_queue_length);

    if (arg_pContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("It is a null context"));
    }

    updater_instance ->average_length +=  (1/pow(2, updater_instance->wred_configuartion.weight)) *
                                          (queue_length - updater_instance -> average_length);

    /*    the calculation of GREEN probability   */
    if (updater_instance ->average_length < updater_instance ->wred_configuartion.green.min_th)
    {
        updater_instance -> wred_entry -> _green_probability = 0;
    }
    else if (updater_instance ->average_length > updater_instance ->wred_configuartion.green.max_th)
    {
        updater_instance -> wred_entry -> _green_probability = IX_CC_WRED_MAX_DROP_PROBABILITY;
    }
    else
    {
        updater_instance -> wred_entry -> _green_probability =
             (ix_uint16)floor((updater_instance -> average_length - updater_instance ->wred_configuartion.green.min_th) *
             updater_instance -> green_const);
    }

    /*    the calculation of YELLOW probability   */
    if (updater_instance -> average_length < updater_instance ->wred_configuartion.yellow.min_th)
    {
        updater_instance -> wred_entry ->_yellow_probability = 0;
    }
    else if (updater_instance ->average_length > updater_instance ->wred_configuartion.yellow.max_th)
    {
        updater_instance -> wred_entry ->_yellow_probability = IX_CC_WRED_MAX_DROP_PROBABILITY;
    }
    else
    {
        updater_instance -> wred_entry -> _yellow_probability =
             (ix_uint16)floor((updater_instance -> average_length - updater_instance ->wred_configuartion.yellow.min_th) *
             updater_instance -> yellow_const);
    }

    /*    calculation of RED probability   */
    if (updater_instance ->average_length < updater_instance ->wred_configuartion.red.min_th)
    {
        updater_instance -> wred_entry ->_red_probability = 0;
    }
    else if (updater_instance ->average_length > updater_instance ->wred_configuartion.red.max_th)
    {
        updater_instance -> wred_entry ->_red_probability = IX_CC_WRED_MAX_DROP_PROBABILITY;
    }
    else
    {
        updater_instance -> wred_entry ->_red_probability =
             (ix_uint16)floor((updater_instance -> average_length - updater_instance ->wred_configuartion.red.min_th) *
             updater_instance -> red_const);
    }

    #ifdef IX_DEBUG_WRED
    if ((updater_instance->average_length > 1) || (queue_length > 1))
    {
        printf("Wred ID: %x,   queue_addr = %x, queue_length = %d,   average_length = %f \n",
               updater_instance -> wred_entry,
               updater_instance->p_queue_length,
               queue_length,
               updater_instance->average_length);
        printf("probability : green = 0x%x,   yellow = 0x%x,  red = 0x%x\n", 
                updater_instance -> wred_entry -> _green_probability,
                updater_instance -> wred_entry -> _yellow_probability,
                updater_instance -> wred_entry -> _red_probability);
        show_mem(updater_instance->wred_entry,64);
    }
    #endif
    return IX_SUCCESS;
}
#endif


/* ------------------------------------------------- ix_cc_wred_add_entry -- */
ix_error
ix_cc_wred_add_entry(ix_uint32              arg_nInstance,
                     ix_cc_wred_parameters *arg_pParams,
                     void                  *arg_pQueueDescriptor,
                     void                  *arg_pContext)
{
    #ifdef WRED_CORE_PROBABILITY_CALCULATION
    ix_error                    err;
    _ix_cc_wred_updater_entry  *updater_instance;
    #endif
    _ix_cc_wred_table_entry    *_pT;

    #ifdef IX_DEBUG_WRED
    ix_ossl_message_log("ix_cc_wred_add_entry(): check ctx null. \n");
    #endif
    /* validity check for the arg_pContext */
    IX_ERROR_CHECK_ARG_NULL( 3, arg_pContext );
    #ifdef IX_DEBUG_WRED
    ix_ossl_message_log("ix_cc_wred_add_entry(): check ninstance. \n");
    #endif
    /* validity check for arg_nInstance */
    IX_ERROR_CHECK_ARG_MAX( 1, arg_nInstance,
      _WRED_MAX_FLOW_IDS( arg_pContext ), "%u" );
    #ifdef IX_DEBUG_WRED
    ix_ossl_message_log("ix_cc_wred_add_entry(): check nparams. \n");
    #endif
    /* validity check for arg_pParams */
    IX_ERROR_CHECK_ARG_NULL( 2, arg_pParams );

    #ifndef WRED_CORE_PROBABILITY_CALCULATION
        if (( arg_pParams->service_rate > WRED_MAX_PACKET_RATE) || (arg_pParams->service_rate < 0))
        {
            #ifdef IX_DEBUG_WRED
            ix_ossl_message_log("ix_cc_wred_add_entry(): invalid service_rate specified- %d\n",
                                                                       arg_pParams->service_rate);
            #endif
            return IX_ERROR_WARNING(IX_CC_WRED_ERROR_INVALID_PARAM,
                ( "ix_cc_wred_add_entry(): invalid service_rate specified" ));
        }
    #endif

    if ( arg_pParams->weight > 15 )
    {
        #ifdef IX_DEBUG_WRED
        ix_ossl_message_log("ix_cc_wred_add_entry(): weight is %d which is greater then 15. \n",
                                                                              arg_pParams->weight);
        #endif
        return IX_ERROR_WARNING(IX_CC_WRED_ERROR_INVALID_PARAM,
            ( "ix_cc_wred_add_entry(): 'weight' parameter out of range <0;15>" ));
    }

    if ((arg_pParams->green.max_prob  > IX_CC_WRED_MAX_DROP_PROBABILITY) ||
        (arg_pParams->yellow.max_prob > IX_CC_WRED_MAX_DROP_PROBABILITY) ||
        (arg_pParams->red.max_prob    > IX_CC_WRED_MAX_DROP_PROBABILITY))
    {
        #ifdef IX_DEBUG_WRED
        ix_ossl_message_log("ix_cc_wred_add_entry(): too large max probability.\n");
        #endif
        return IX_ERROR_WARNING(IX_CC_WRED_ERROR_INVALID_PARAM,
            ( "ix_cc_wred_add_entry(): 'too large max probability" ));
    }

    _pT = _WRED_TABLE_ENTRY( arg_nInstance, arg_pContext );
    /* validity check for the entry */
    if ( _pT->_valid == WRED_ENTRY_VALID )
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_DUPLICATE_ENTRY,
            ( "ix_cc_wred_add_entry(): entry already exists" ));
    }
    #if defined( IX_DEBUG_WRED )
        printf(" Addning new WRED instance %d at address 0x%x\n",arg_nInstance, _pT);
        printf(" ----- before ------  \n");
        show_mem((void*)_pT, sizeof(_ix_cc_wred_table_entry));
    #endif /* IX_DEBUG_WRED */

    _ix_cc_wred_empty_table_entry( _pT );

    #ifdef WRED_PACKET_COUNTER_FEATURE
    _ix_cc_wred_empty_stats_entry( _WRED_STATS_ENTRY( arg_nInstance, arg_pContext ) );
    #endif

#ifdef WRED_CORE_PROBABILITY_CALCULATION
    updater_instance = (((_ix_cc_wred_context *)(arg_pContext))->pUpdatersTable + arg_nInstance);
    updater_instance -> wred_configuartion = *arg_pParams;
    updater_instance -> wred_entry         = _pT;
    updater_instance -> p_queue_length     = ((ix_uint32*) arg_pQueueDescriptor) + 2,

    updater_instance -> average_length     = 0;
    updater_instance -> green_const        = _GET_FLOAT_CONST(arg_pParams, green);
    updater_instance -> yellow_const       = _GET_FLOAT_CONST(arg_pParams, yellow);
    updater_instance -> red_const          = _GET_FLOAT_CONST(arg_pParams, red);
    #if defined( IX_DEBUG_WRED )
        printf(" service time 0x%x\n",arg_pParams->service_rate);
        printf(" weight       0x%x\n",arg_pParams->weight);
        printf(" updater instance 0x%x\n", updater_instance);
        printf(" core handle  x%x\n",((_ix_cc_wred_context *)(arg_pContext))->_hCC);
    #endif /* IX_DEBUG_WRED */

	if (arg_pParams->service_rate > 0) {
        err = ix_cci_cc_add_event_handler_ex(
                    ((_ix_cc_wred_context *)(arg_pContext))->_hCC,
                    arg_pParams->service_rate,
                    _ix_cc_wred_periodic_updater,
                    IX_EVENT_TYPE_PERIODIC,
                    IX_CC_UPDATE_EVENT_PRIORITY,
                    updater_instance,
                    &(updater_instance->event_handle));
        if (err != IX_SUCCESS)
  	    {
            return IX_ERROR_WARNING(IX_ERROR_GET_CODE(err),
                ( "ix_cc_wred_add_entry(): could not create the handler for time events." ));
        }
    }
    _pT->_green_stats_enabled  = _BIT_FLD( arg_pParams->statsEnabled );
    _pT->_yellow_stats_enabled = _BIT_FLD( arg_pParams->statsEnabled );
    _pT->_red_stats_enabled    = _BIT_FLD( arg_pParams->statsEnabled );
    _pT->_green_count          = 1;
    _pT->_yellow_count         = 1;
    _pT->_red_count            = 1;

#else
    _pT->_service_time  = _SERVICE_TIME( arg_pParams->service_rate );
    _pT->_weight        = arg_pParams->weight;
    _pT->_green_const  = _GET_INT_CONST(arg_pParams, green);
    _pT->_yellow_const = _GET_INT_CONST(arg_pParams, yellow);
    _pT->_red_const    = _GET_INT_CONST(arg_pParams, red);
    _pT->_green_min_th  = arg_pParams->green.min_th << 8;
    _pT->_green_max_th  = arg_pParams->green.max_th << 8;
    _pT->_red_max_th    = arg_pParams->red.max_th << 8;
    _pT->_red_min_th    = arg_pParams->red.min_th << 8;
    _pT->_yellow_max_th = arg_pParams->yellow.max_th << 8;
    _pT->_yellow_min_th = arg_pParams->yellow.min_th << 8;
    _pT->_stats_enabled = _BIT_FLD( arg_pParams->statsEnabled );

#endif
    _pT->_valid         = WRED_ENTRY_VALID;


    #if defined( IX_DEBUG_WRED )
        printf(" arg_pParams =  \n");
        show_mem((void*)arg_pParams, sizeof(ix_cc_wred_parameters ));
        printf(" ----- after ------  \n");
        show_mem((void*)_pT, sizeof(_ix_cc_wred_table_entry));
    #endif /* IX_DEBUG_WRED */

    return IX_SUCCESS;
}


/* ---------------------------------------------- ix_cc_wred_remove_entry -- */
ix_error
ix_cc_wred_remove_entry(ix_uint32  arg_nInstance,
                        void      *arg_pContext)
{
    #ifdef WRED_CORE_PROBABILITY_CALCULATION
    ix_error                    err;
    _ix_cc_wred_updater_entry  *updater_instance;
    #endif
    _ix_cc_wred_table_entry    *_pT;

    /* validity check for arg_pContext */
    IX_ERROR_CHECK_ARG_NULL( 2, arg_pContext );
    /* validity check for arg_nInstance */
    IX_ERROR_CHECK_ARG_MAX( 1, arg_nInstance,
      _WRED_MAX_FLOW_IDS( arg_pContext ), "%u" );
    _pT = _WRED_TABLE_ENTRY( arg_nInstance, arg_pContext );
    if (_pT->_valid != WRED_ENTRY_VALID)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND,
            ( "ix_cc_wred_remove_entry(): entry does not exist" ));
    }

#ifdef WRED_CORE_PROBABILITY_CALCULATION
    updater_instance = (((_ix_cc_wred_context *)(arg_pContext))->pUpdatersTable + arg_nInstance);
    updater_instance -> average_length = 0;
    err = ix_cci_cc_remove_event_handler(
                    ((_ix_cc_wred_context *)(arg_pContext))->_hCC,
                    (updater_instance->event_handle));

    updater_instance -> event_handle = 0;

    if (err != IX_SUCCESS)
    {
        err = IX_ERROR_WARNING(IX_ERROR_GET_CODE(err),
            ( "ix_cc_wred_add_entry(): could not remove the handler for time events." ));
    }
#endif
    _ix_cc_wred_empty_table_entry( _pT );

    #ifdef WRED_PACKET_COUNTER_FEATURE
    _ix_cc_wred_empty_stats_entry( _WRED_STATS_ENTRY( arg_nInstance, arg_pContext ) );
    #endif

#ifdef WRED_CORE_PROBABILITY_CALCULATION
    return err;
#else
    return IX_SUCCESS;
#endif
}


/* ---------------------------------------------- ix_cc_wred_update_entry -- */
ix_error
ix_cc_wred_update_entry(ix_uint32              arg_nInstance,
                        ix_cc_wred_parameters *arg_pParams,
                        void                  *arg_pContext)
{
    #ifdef WRED_CORE_PROBABILITY_CALCULATION
    ix_error                    err;
    _ix_cc_wred_updater_entry  *updater_instance;
    #endif
    _ix_cc_wred_table_entry    *_pT;

    #ifdef IX_DEBUG_WRED
    ix_ossl_message_log("ix_cc_wred_update_entry() : I'm in ..\n");
    #endif

    /* validity check for arg_pContext */
    IX_ERROR_CHECK_ARG_NULL( 3, arg_pContext );
    /* validity check for arg_nInstance */
    IX_ERROR_CHECK_ARG_MAX( 1, arg_nInstance,
    _WRED_MAX_FLOW_IDS( arg_pContext ), "%u" );
    /* validity check for arg_pParams */
    IX_ERROR_CHECK_ARG_NULL( 2, arg_pParams );

    #ifdef IX_DEBUG_WRED
    ix_ossl_message_log("ix_cc_wred_update_entry() : parameters valid ..\n");
    #endif

    #ifndef WRED_CORE_PROBABILITY_CALCULATION
        if ( arg_pParams->service_rate > 1<<8)
        {
            #ifdef IX_DEBUG_WRED
            ix_ossl_message_log("ix_cc_wred_update_entry() : arg_pParams->service_rate > 256 ..\n");
            #endif
            return IX_ERROR_WARNING(IX_CC_WRED_ERROR_INVALID_PARAM,
                ( "ix_cc_wred_update_entry(): invalid arg_pParams specified" ));
        }
    #endif

    if ( arg_pParams->weight > 15 )
    {
        #ifdef IX_DEBUG_WRED
        ix_ossl_message_log("ix_cc_wred_update_entry() : arg_pParams->weight    > 15 ..\n");
        #endif
        return IX_ERROR_WARNING(IX_CC_WRED_ERROR_INVALID_PARAM,
            ( "ix_cc_wred_update_entry(): 'weight' parameter is out of ragne <0;15>" ));
    }

    if ((arg_pParams->green.max_prob  > IX_CC_WRED_MAX_DROP_PROBABILITY) ||
        (arg_pParams->yellow.max_prob > IX_CC_WRED_MAX_DROP_PROBABILITY) ||
        (arg_pParams->red.max_prob    > IX_CC_WRED_MAX_DROP_PROBABILITY))
    {
        #ifdef IX_DEBUG_WRED
        ix_ossl_message_log("ix_cc_wred_update_entry() : too large max probability..\n");
        #endif
        return IX_ERROR_WARNING(IX_CC_WRED_ERROR_INVALID_PARAM,
            ( "ix_cc_wred_add_entry(): 'too large max probability" ));
    }

    _pT = _WRED_TABLE_ENTRY( arg_nInstance, arg_pContext );
    /* check if entry had been configured already */
    if ( _pT->_valid != WRED_ENTRY_VALID )
    {
        #ifdef IX_DEBUG_WRED
        ix_ossl_message_log("ix_cc_wred_update_entry() : entry doesn't exists..\n");
        #endif
        return IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND,
            ( "ix_cc_wred_update_entry(): entry doesn't exists" ));
    }

    /* ----------------------------------------------------*/
    #ifdef WRED_PACKET_COUNTER_FEATURE

    #ifdef WRED_CORE_PROBABILITY_CALCULATION
        /* zero statistics if statistics gathering is being enabled */
        if ( arg_pParams->statsEnabled && !_pT->_green_stats_enabled )
    #else
        /* zero statistics if statistics gathering is being enabled */
        if ( arg_pParams->statsEnabled && !_pT->_stats_enabled )
    #endif
    {
        _pT->_green_bytes  = 0;
        _pT->_green_pkts   = 0;
        _pT->_green_count  = 0;
        _pT->_red_bytes    = 0;
        _pT->_red_pkts     = 0;
        _pT->_red_count    = 0;
        _pT->_yellow_bytes = 0;
        _pT->_yellow_pkts  = 0;
        _pT->_yellow_count = 0;
        _ix_cc_wred_empty_stats_entry(
                    _WRED_STATS_ENTRY( arg_nInstance, arg_pContext ));
    }

    #endif
    /* ----------------------------------------------------*/

#ifdef WRED_CORE_PROBABILITY_CALCULATION
    updater_instance = (((_ix_cc_wred_context *)(arg_pContext))->pUpdatersTable + arg_nInstance);
    updater_instance -> wred_configuartion = *arg_pParams;
    updater_instance -> average_length = 0;
    err = ix_cci_change_event(
                    (updater_instance->event_handle),
                    arg_pParams->service_rate);
    if (err != IX_SUCCESS)
    {
        #ifdef IX_DEBUG_WRED
        ix_ossl_message_log("ix_cc_wred_update_entry() : could not modify the handler for time events..\n");
        #endif
        return IX_ERROR_WARNING(IX_ERROR_GET_CODE(err),
            ( "ix_cc_wred_add_entry(): could not modify the handler for time events." ));
    }
#else
    _pT->_weight = arg_pParams->weight;
    _pT->_stats_enabled = _BIT_FLD( arg_pParams->statsEnabled );
    _pT->_service_time  = _SERVICE_TIME( arg_pParams->service_rate );
    _pT->_green_const  = _GET_INT_CONST(arg_pParams, green);
    _pT->_yellow_const = _GET_INT_CONST(arg_pParams, yellow);
    _pT->_red_const    = _GET_INT_CONST(arg_pParams, red);
    _pT->_green_min_th  = arg_pParams->green.min_th << 8;
    _pT->_green_max_th  = arg_pParams->green.max_th << 8;
    _pT->_red_max_th    = arg_pParams->red.max_th << 8;
    _pT->_red_min_th    = arg_pParams->red.min_th << 8;
    _pT->_yellow_max_th = arg_pParams->yellow.max_th << 8;
    _pT->_yellow_min_th = arg_pParams->yellow.min_th << 8;
#endif
    #ifdef IX_DEBUG_WRED
    ix_ossl_message_log("ix_cc_wred_update_entry() : Tous va bien..\n");
    #endif

    return IX_SUCCESS;
}


#ifdef WRED_PACKET_COUNTER_FEATURE

/* -------------------------------------------- ix_cc_wred_get_statistics -- */
ix_error ix_cc_wred_get_statistics(ix_uint32              arg_nInstance,
                                   ix_cc_wred_statistics *arg_pStats,
                                   void                  *arg_pContext)
{
    _ix_cc_wred_stats_entry * _pS;
    _ix_cc_wred_table_entry * _pT;
    /* validity check for arg_pContext */
    IX_ERROR_CHECK_ARG_NULL( 3, arg_pContext );
    /* validity check for arg_nInstance */
    IX_ERROR_CHECK_ARG_MAX( 1, arg_nInstance,
      _WRED_MAX_FLOW_IDS( arg_pContext ), "%u" );

    _pT = _WRED_TABLE_ENTRY( arg_nInstance, arg_pContext );
    if ( _pT->_valid != WRED_ENTRY_VALID )
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND,
            ( "ix_cc_wred_get_stats(): entry doesn't exist" ));
    }


    /* validity check for arg_pStats */
    IX_ERROR_CHECK_ARG_NULL( 2, arg_pStats );
    _pS = _WRED_STATS_ENTRY( arg_nInstance, arg_pContext );

    #if defined( IX_DEBUG_WRED )
        printf(" WRED entry %d pod adresem 0x%x\n",arg_nInstance, _pT);
        show_mem((void*)_pT, sizeof(_ix_cc_wred_table_entry));
        printf(" WRED stats entry pod adresem 0x%x\n",arg_nInstance, _pS);
        show_mem((void*)_pS, sizeof(_ix_cc_wred_stats_entry));
    #endif /* IX_DEBUG_WRED */

    arg_pStats->greenPacketCount  =
      IX_CC_WRED_MAKE_UINT64(_pT->_green_pkts, _pS->_green_pkts);
    arg_pStats->greenByteCount =
      IX_CC_WRED_MAKE_UINT64(_pT->_green_bytes, _pS->_green_bytes);
    arg_pStats->redPacketCount    =
      IX_CC_WRED_MAKE_UINT64(_pT->_red_pkts, _pS->_red_pkts);
    arg_pStats->redByteCount   =
      IX_CC_WRED_MAKE_UINT64(_pT->_red_bytes, _pS->_red_bytes);
    arg_pStats->yellowPacketCount =
      IX_CC_WRED_MAKE_UINT64(_pT->_yellow_pkts, _pS->_yellow_pkts);
    arg_pStats->yellowByteCount =
      IX_CC_WRED_MAKE_UINT64(_pT->_yellow_bytes, _pS->_yellow_bytes);
    return IX_SUCCESS;
}

#endif



/**
 * @brief  This function gets parameters of an entry from the WRED table.
 *
 * The core component first checks if an entry, pointed by arg_Instance, is
 * valid. If true, it reads the entry parameters to the arg_pParams argument.
 * Otherwise, if the referred entry is marked empty, the core component
 * reports an error in a callback function.
 *
 *
 * @param  arg_nInstance index of a WRED table entry.
 *                       The value must not be higher than WRED\TABLE_RAM_SIZE
 *                       system property.
 *
 * @param  arg_pParams  pointer to the parameters structure which should be
 *                      filled with entry parameters.
 *
 * @param  arg_pContext  pointer to the control block allocated in
 *                       <code>ix_cc_wred_init()</code>
 *
 * @return IX_SUCCESS if successful or a valid ix_error token for failure. The
 *         following error codes might be returned via the token.
 *\n
 *         IX_CC_ERROR_ENTRY_NOT_FOUND - entry does not exist
 *         IX_CC_ERROR_RANGE           - WRED instance out of range
 *
 * @see    ix_cc_wred_add_entry()
 */
ix_error ix_cc_wred_get_parameters( ix_uint32               arg_nInstance,
                                    ix_cc_wred_parameters * arg_pParams,
                                    void *                  arg_pContext )
{
    #ifdef WRED_CORE_PROBABILITY_CALCULATION
    _ix_cc_wred_updater_entry  *updater_instance;
    #endif
    _ix_cc_wred_table_entry    *_pT;

    /* validity check for arg_pContext */
    IX_ERROR_CHECK_ARG_NULL( 3, arg_pContext );
    /* validity check for arg_nInstance */
    IX_ERROR_CHECK_ARG_MAX( 1, arg_nInstance,
      _WRED_MAX_FLOW_IDS( arg_pContext ), "%u" );
    /* validity check for arg_pParams */
    IX_ERROR_CHECK_ARG_NULL( 2, arg_pParams );
    _pT = _WRED_TABLE_ENTRY( arg_nInstance, arg_pContext );
    /* check if entry had been configured */
    if ( _pT->_valid != WRED_ENTRY_VALID )
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND,
            ( "ix_cc_wred_get_parameters(): entry doesn't exists" ));
    }

#ifdef WRED_CORE_PROBABILITY_CALCULATION
    updater_instance = (((_ix_cc_wred_context *)(arg_pContext))->pUpdatersTable + arg_nInstance);
    *arg_pParams = updater_instance -> wred_configuartion;
    arg_pParams->statsEnabled = (ix_uint8) ( ( _pT->_green_stats_enabled != 0 ) ? 1 : 0 );
#else
    /* recalculated value of service rate is returned */
    arg_pParams->service_rate = _pT->_service_time;
    /* weight value stored by WRED table is only 5 bits */
    arg_pParams->weight = (ix_uint8)_pT->_weight;
    arg_pParams->green.min_th = (ix_uint8) ( _pT->_green_min_th >> 8 );
    arg_pParams->green.max_th = (ix_uint8) ( _pT->_green_max_th >> 8 );
    /* recalculated value of green max_prob is returned */
    arg_pParams->green.max_prob = _pT->_green_const;
    arg_pParams->yellow.min_th = (ix_uint8) ( _pT->_yellow_min_th >> 8 );
    arg_pParams->yellow.max_th = (ix_uint8) ( _pT->_yellow_max_th >> 8 );
    /* recalculated value of yellow max_prob is returned */
    arg_pParams->yellow.max_prob = _pT->_yellow_const;
    arg_pParams->red.min_th = (ix_uint8) ( _pT->_red_min_th >> 8 );
    arg_pParams->red.max_th = (ix_uint8) ( _pT->_red_max_th >> 8 );
    /* recalculated value of red max_prob is returned */
    arg_pParams->red.max_prob = _pT->_red_const;
    arg_pParams->statsEnabled = (ix_uint8) ( ( _pT->_stats_enabled != 0 ) ? 1 : 0 );
#endif
    return IX_SUCCESS;
}

