/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2000 - 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILENAME
 *      ix_cc_wred_fini.c
 *
 * ============================================================================
 */

#define IX_ERROR_FILE_IDENT \
  "$Id: ix_cc_wred_fini.c,v 1.12.4.1 2004/03/12 10:05:17 wneubaue Exp $"



#include <ix_types.h>
#include <ix_error.h>

#include <ix_ossl.h>
#include <ix_cc.h>
#include <ix_cc_error.h>
#include <bindings.h>

#include "../../include/wred_defs.h"
#include "cc/ix_cc_wred.h"
#include "cc/internal/ix_cc_wred_defs.h"
#include "cc/internal/ix_cc_wred_api.h"
#include "cc/internal/ix_cc_wred_types.h"

IX_CC_WRED_INSTANCE_TABLE_DECLARE()

/* --------------------------------------------- _ix_cc_wred_free_symbols -- */

ix_error _ix_cc_wred_free_symbols( void * arg_pContext )
  {
    /* validity check for arg_pContext */
    IX_ERROR_CHECK_ARG_NULL( 1, arg_pContext );

    IX_ERROR_CRT(
        ix_rm_mem_free( _WRED_TABLE_BASE( arg_pContext ) ),
        IX_CC_ERROR_OOM, IX_ERROR_LEVEL_LOCAL
      );
    
    _WRED_TABLE_BASE(arg_pContext) = NULL;

    #ifdef WRED_PACKET_COUNTER_FEATURE

    IX_ERROR_CRT(
        ix_rm_mem_free( _WRED_STATS_BASE( arg_pContext ) ),
        IX_CC_ERROR_OOM, IX_ERROR_LEVEL_LOCAL
      );

    _WRED_STATS_BASE(arg_pContext) = NULL;
    
    #endif
    
    return IX_SUCCESS;
  }

/* ----------------------------------- _ix_cc_wred_unregister_msg_handler -- */

ix_error _ix_cc_wred_unregister_msg_handler( ix_cc_handle arg_hCC )
  {
    ix_uint32 instanceCC = IX_HANDLE_TO_INSTANCE(arg_hCC);

#if defined( IX_EXCLUDE_CCI )
    IX_ERROR_CRT(
        ix_rm_message_handler_unregister( IX_CC_WRED_INSTANCE_TABLE_GET(instanceCC,msgIn) ),
        IX_CC_WRED_ERROR_CCI, IX_ERROR_LEVEL_WARNING
      );
#else
    IX_ERROR_CRT(
        ix_cci_cc_remove_message_handler( IX_CC_WRED_INSTANCE_TABLE_GET(instanceCC,msgIn) ),
        IX_CC_WRED_ERROR_CCI, IX_ERROR_LEVEL_WARNING
      );
#endif /* IX_EXCLUDE_CCI */    

    return IX_SUCCESS;
  }

/* ----------------------------------- _ix_cc_wred_unregister_pkt_handler -- */

ix_error _ix_cc_wred_unregister_pkt_handler( ix_cc_handle arg_hCC )
  {
    ix_uint32 instanceCC = IX_HANDLE_TO_INSTANCE(arg_hCC);

#if defined( IX_EXCLUDE_CCI )
    IX_ERROR_CRT(
        ix_rm_packet_handler_unregister( IX_CC_WRED_INSTANCE_TABLE_GET(instanceCC,pktIn) ),
        IX_CC_WRED_ERROR_CCI, IX_ERROR_LEVEL_WARNING
      );
#else
    IX_ERROR_CRT(
        ix_cci_cc_remove_packet_handler( IX_CC_WRED_INSTANCE_TABLE_GET(instanceCC,pktIn) ),
        IX_CC_WRED_ERROR_CCI, IX_ERROR_LEVEL_WARNING
      );
#endif /* IX_EXCLUDE_CCI */
    
    return IX_SUCCESS;
  }




/* ----------------------------------- _ix_cc_wred_unregister_periodic_updater -- */

#ifdef WRED_CORE_PROBABILITY_CALCULATION

ix_error _ix_cc_wred_unregister_all_periodic_updaters(_ix_cc_wred_context *arg_pContext)
{
    ix_error   err;
    ix_error   ret_err = IX_SUCCESS;
    ix_uint32  index;

    #if defined(IX_EXCLUDE_CCI) /* compile time flag */
         #error "You can't define IX_EXCLUDE_CCI together with IX_CC_WRED_PERIODIC_QUEUE_AVG_CALCULATION"        
    #else 

    for (index = 0; index<arg_pContext->_pConf->_nTableSize; index++) {
        if ((arg_pContext->pUpdatersTable + index)->event_handle)
        err = ix_cci_cc_remove_event_handler(arg_pContext->_hCC,
                                             ((arg_pContext->pUpdatersTable) + index)->event_handle);
        if (err != IX_SUCCESS) {
           ret_err = IX_ERROR_WARNING(IX_CC_WRED_ERROR_CCI, 
              ("_ix_cc_wred_unregister_all_periodic_updaters(): couldn't unregister periodic updater handler."));
        }
    }

    #endif


    return ret_err;

} /* end _ix_cc_wred_unregister_all_periodic_updaters  */

#endif



/* ------------------------------------------------------ ix_cc_wred_fini -- */
ix_error ix_cc_wred_fini( ix_cc_handle arg_hCC, 
                          void *       arg_pContext )
  {
    _ix_cc_wred_context * _pCntxt =
      (_ix_cc_wred_context *)arg_pContext;

    /* validity check for arg_pContext */
    IX_ERROR_CHECK_ARG_NULL( 2, arg_pContext );

    /* validity check for arg_hCC */
    IX_ERROR_CHECK_ARG_NULL( 1, arg_hCC );

    if ( arg_hCC != _pCntxt->_hCC )
    {
        return IX_ERROR_WARNING(
            IX_CC_WRED_ERROR_INVALID_HANDLE,
            ( "ix_cc_wred_fini(): invalid core component handle" )
          );
    }

    /* resource manager memory de-allocation */
    IX_ERROR_CR( _ix_cc_wred_free_symbols(arg_pContext) );

    #ifdef WRED_CORE_PROBABILITY_CALCULATION
    /* unregister periodic update */
    IX_ERROR_CR( _ix_cc_wred_unregister_all_periodic_updaters(arg_pContext) );
    ix_ossl_free( (void *)_pCntxt->pUpdatersTable );
    #endif

    /* unregister packet handler */
    IX_ERROR_CR( _ix_cc_wred_unregister_pkt_handler(arg_hCC) );
    /* unregister message handle */
    IX_ERROR_CR( _ix_cc_wred_unregister_msg_handler(arg_hCC) );

    /* system memory de-allocation */
    _SYSTEM_DELETE( _pCntxt->_pConf );    
    _SYSTEM_DELETE( _pCntxt );

    return IX_SUCCESS;
  }


