/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2000 - 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILENAME
 *      ix_cc_wred_init.c
 *
 * ============================================================================
 */

#define IX_ERROR_FILE_IDENT \
  "$Id: ix_cc_wred_init.c,v 1.32.4.1 2004/03/12 10:05:17 wneubaue Exp $"


#include <math.h>

#include <ix_types.h>
#include <ix_error.h>

#include <ix_ossl.h>
#include <ix_cc.h>
#include <ix_cc_macros.h>
#include <ix_cc_error.h>
#include <bindings.h>

#include <diffserv.h>
#include "../../include/wred_defs.h"
#include "cc/ix_cc_wred.h"
#include "cc/internal/ix_cc_wred_defs.h"
#include "cc/internal/ix_cc_wred_defaults.h"
#include "cc/internal/ix_cc_wred_registry.h"
#include "cc/internal/ix_cc_wred_api.h"
#include "cc/internal/ix_cc_wred_types.h"
#include "cc/internal/ix_cc_wred_misc.h"

#ifndef CCI_MULTIINSTANCE

IX_CC_WRED_INSTANCE_TABLE_DEFINE()
IX_CC_WRED_INSTANCE_ENTRY_DEFINE(IX_CC_WRED_MSG_ID,IX_CC_WRED_PKT_ID,IX_CC_WRED_PKT_OUTPUT,WRED_DEFAULT_RAM_CHANNEL)
IX_CC_WRED_INSTANCE_TABLE_END

#else /* CCI_MULTIINSTANCE */

IX_CC_WRED_INSTANCE_TABLE_DECLARE()
    
#endif

/* only include this source code if registry support is present */
#if defined( IX_INCLUDE_REGISTRY )

/* ----------------------------------- _ix_cc_wred_get_property_by_handle -- */

PRIVATE ix_error _ix_cc_wred_get_property( const char * arg_pPropPrefix,
                                           const char * arg_pName,
                                           ix_uint32  * arg_pValue )
  {
    ix_cp_property_info _info;
    ix_configuration_property_handle _hProp = IX_NULL_HANDLE;
    char pName[WRED_PROP_MAX_NAME+1]; 

    /* validity check for arg_pPropPrefix */
    IX_ERROR_CHECK_ARG_NULL( 1, arg_pPropPrefix );
    /* validity check for arg_pName */
    IX_ERROR_CHECK_ARG_NULL( 2, arg_pName );
    /* validity check for arg_hValue */
    IX_ERROR_CHECK_ARG_NULL( 3, arg_pValue );

    sprintf(pName, "%s%s", arg_pPropPrefix, arg_pName);

    IX_ERROR_CRT( ix_rm_cp_property_open(
      IX_CP_CORE_PROPERTY_ROOT, pName, &_hProp ),
      IX_CC_WRED_ERROR_REGISTRY, IX_ERROR_LEVEL_LOCAL );

    IX_ERROR_CR( ix_rm_cp_property_get_info( _hProp, &_info ) );

    /* validity check for property type */
    if ( _info.m_Options & IX_CP_OPTIONS_FLAGS_DATA_UINT32 )
      {
        IX_ERROR_CR(
            ix_rm_cp_property_get_value_uint32( _hProp, arg_pValue )
          );
      }
    else
      {
        return IX_ERROR_LOCAL(
            IX_CC_WRED_ERROR_REGISTRY,
            ( "Invalid WRED registry property type. All subproperties " \
              "must be uint32" )
          );
      }

    IX_ERROR_CRT( ix_rm_cp_property_close( _hProp ),
        IX_CC_WRED_ERROR_REGISTRY, IX_ERROR_LEVEL_LOCAL );

    return IX_SUCCESS;
  }

/* ------------------------------------------------- _ix_cc_wred_registry -- */

PRIVATE ix_error _ix_cc_wred_registry( ix_uint32 arg_instance,
                                       _ix_cc_wred_config * arg_pConf )
  {
      char _pWredInstancePrefix[WRED_MAX_PREFIX+1];

#ifndef CCI_MULTIINSTANCE
      sprintf(_pWredInstancePrefix, "%s%s", "WRED", "/");
#else
      sprintf(_pWredInstancePrefix, "%s%ld%s", "WRED_", arg_instance, "/");
#endif
      
    IX_ERROR_CR( _ix_cc_wred_get_property(
      _pWredInstancePrefix, IX_RKEY_WRED_ENTRY_RAM_SIZE, &arg_pConf->_nEntrySize ) );

    IX_ERROR_CR( _ix_cc_wred_get_property(
      _pWredInstancePrefix, IX_RKEY_WRED_TABLE_MEM_CHANNEL, &arg_pConf->_nTableRAMChnnl ) );

    #ifdef WRED_PACKET_COUNTER_FEATURE

    IX_ERROR_CR( _ix_cc_wred_get_property(
      _pWredInstancePrefix, IX_RKEY_WRED_STATS_MEM_CHANNEL, &arg_pConf->_nStatsSRAMChnnl ) );

    IX_ERROR_CR( _ix_cc_wred_get_property(
      _pWredInstancePrefix, IX_RKEY_WRED_64BITSTAT_ENTRY_SRAM_SIZE, &arg_pConf->_nStatsSize ) );

    #endif

    IX_ERROR_CR( _ix_cc_wred_get_property(
      _pWredInstancePrefix, IX_RKEY_WRED_TABLE_RAM_SIZE, &arg_pConf->_nTableSize ) );

    IX_ERROR_CR( _ix_cc_wred_get_property(
      _pWredInstancePrefix, IX_RKEY_WRED_MEV2_NUM_MASK, &arg_pConf->_uEngMask ) );

    /*
      IX_ERROR_CR( _ix_cc_wred_get_property(
      "QM/QM_UENG_NUM", &arg_pConf->_qm_uEngNum ) );
    */

    return IX_SUCCESS;
  }

#endif /* IX_INCLUDE_REGISTRY */

/* -------------------------------------------- _ix_cc_wred_patch_symbols -- */

PRIVATE ix_error _ix_cc_wred_patch_symbols( _ix_cc_wred_context * arg_pCntx )
  {
    ix_imported_symbol _Symbols[WRED_PATCH_SYMBOLS];
    ix_uint32         i = 0;
    ix_error          err;
    ix_memory_type    MemoryType;
    ix_uint32         MemoryChannel;
    ix_uint32         MemoryOffset;
    void*             PhysicalAddress;

    /*--------------------------------------------*/

    err = ix_rm_get_phys_offset(arg_pCntx->_pTableBase,
                                &MemoryType,
                                &MemoryChannel,
                                &MemoryOffset,
                                &PhysicalAddress);

    if (err != IX_SUCCESS)
        return IX_ERROR_WARNING(err,
              ("Error ! Could not convert virtual address to offset."));

    #ifndef WRED_DRAM_TABLE
    IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(MemoryChannel, MemoryOffset);
    #endif

    _Symbols[0].m_Value = MemoryOffset;
    _Symbols[0].m_Name  = WRED_TABLE_RAM_BASE;

    /*--------------------------------------------*/


#if defined(WRED_PACKET_COUNTER_FEATURE)

    /*--------------------------------------------*/

    err = ix_rm_get_phys_offset(arg_pCntx->_pStatsBase,
                                &MemoryType,
                                &MemoryChannel,
                                &MemoryOffset,
                                &PhysicalAddress);

    if (err != IX_SUCCESS)
        return IX_ERROR_WARNING(err,
              ("Error ! Could not convert virtual address to offset."));

    IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(MemoryChannel, MemoryOffset);

    _Symbols[1].m_Value = MemoryOffset;
    _Symbols[1].m_Name  = WRED_64BIT_STAT_SRAM_BASE;

    /*--------------------------------------------*/

#endif


    for( i = 0; i < IX_MAX_MICROENGINE_NUMBER; i++ ) {
        if ( arg_pCntx->_pConf->_uEngMask & (0x1 << i) ) {
            ix_rm_ueng_patch_symbols(
              g_aMicroengineNumberMapping[i],
              WRED_PATCH_SYMBOLS, _Symbols);
          }
      }

    return IX_SUCCESS;
  }

/* ------------------------------------- _ix_cc_wred_register_msg_handler -- */

ix_error _ix_cc_wred_register_msg_handler( _ix_cc_wred_context * arg_pContext )
  {
    ix_uint32 instanceCC = IX_HANDLE_TO_INSTANCE(arg_pContext->_hCC);

#if defined( IX_EXCLUDE_CCI )
    IX_ERROR_CRT(
      ix_rm_message_handler_register(
        IX_CC_WRED_INSTANCE_TABLE_GET(instanceCC,msgIn),
        ix_cc_wred_msg_handler,
        arg_pContext
      ),
      IX_CC_WRED_ERROR_CCI, IX_ERROR_LEVEL_WARNING );
#else /* IX_EXCLUDE_CCI */
    IX_ERROR_CRT(
      ix_cci_cc_add_message_handler(
        arg_pContext->_hCC,
        IX_CC_WRED_INSTANCE_TABLE_GET(instanceCC,msgIn),
        ix_cc_wred_msg_handler,
        IX_INPUT_TYPE_MULTI_SRC
      ),
      IX_CC_WRED_ERROR_CCI, IX_ERROR_LEVEL_WARNING );
#endif /* !IX_EXCLUDE_CCI */

    return IX_SUCCESS;
  }




/* ------------------------------------- _ix_cc_wred_register_pkt_handler -- */

ix_error _ix_cc_wred_register_pkt_handler( _ix_cc_wred_context * arg_pContext )
  {
    ix_uint32 instanceCC = IX_HANDLE_TO_INSTANCE(arg_pContext->_hCC);

#if defined( IX_EXCLUDE_CCI )
    /* -- rm -- */
    IX_ERROR_CRT(
      ix_rm_packet_handler_register(
        IX_CC_WRED_INSTANCE_TABLE_GET(instanceCC,pktIn),
        ix_cc_wred_pkt_handler,
        arg_pContext
      ),
      IX_CC_WRED_ERROR_CCI, IX_ERROR_LEVEL_WARNING );
#else /* IX_EXCLUDE_CCI */
    IX_ERROR_CRT(
      ix_cci_cc_add_packet_handler(
        arg_pContext->_hCC,
        IX_CC_WRED_INSTANCE_TABLE_GET(instanceCC,pktIn),
        ix_cc_wred_pkt_handler,
        IX_INPUT_TYPE_MULTI_SRC
      ),
      IX_CC_WRED_ERROR_CCI, IX_ERROR_LEVEL_WARNING );
#endif /* !IX_EXCLUDE_CCI */

    return IX_SUCCESS;
  }

/* ---------------------------------------- _ix_cc_wred_empty_table_entry -- */

ix_error _ix_cc_wred_empty_table_entry( _ix_cc_wred_table_entry * _pT )
{
    /* zero wred table entry */


#ifdef WRED_CORE_PROBABILITY_CALCULATION

    _pT->_green_probability  = 0;
    _pT->_yellow_probability = 0;
    _pT->_red_probability    = 0;
    
    _pT->_green_stats_enabled  = 0;
    _pT->_yellow_stats_enabled = 0;
    _pT->_red_stats_enabled    = 0;

#else

    _pT->_avg_length     = 0;

    _pT->_green_min_th   = WRED_NOT_INITIALIZED;
    _pT->_green_max_th   = WRED_NOT_INITIALIZED;
    _pT->_green_const    = 0;

    _pT->_red_max_th     = WRED_NOT_INITIALIZED;
    _pT->_red_min_th     = WRED_NOT_INITIALIZED;
    _pT->_red_const      = 0;

    _pT->_yellow_max_th  = WRED_NOT_INITIALIZED;
    _pT->_yellow_min_th  = WRED_NOT_INITIALIZED;
    _pT->_yellow_const   = 0;

    _pT->_service_time   = 0x0;

    _pT->_weight         = 0;

    _pT->_stats_enabled  = 0;

#endif

    _pT->_red_count      = 0;
    _pT->_yellow_count   = 0;
    _pT->_green_count    = 0;

    #if defined(WRED_PACKET_COUNTER_FEATURE)
    _pT->_red_bytes      = 0;
    _pT->_red_pkts       = 0;
    _pT->_yellow_bytes   = 0;
    _pT->_yellow_pkts    = 0;
    _pT->_green_bytes    = 0;
    _pT->_green_pkts     = 0;
    #endif

    _pT->_valid          = 0;

    return IX_SUCCESS;
}



/* ---------------------------------------- _ix_cc_wred_empty_stats_entry -- */
#if defined(WRED_PACKET_COUNTER_FEATURE)
ix_error _ix_cc_wred_empty_stats_entry( _ix_cc_wred_stats_entry * _pS )
  {
    ix_ossl_memset( _pS, 0, sizeof( _ix_cc_wred_stats_entry ) );
    return IX_SUCCESS;
  }
#endif

/* ------------------------------------------ _ix_cc_wred_reset_timestamp -- */

ix_error _ix_cc_wred_reset_timestamp( _ix_cc_wred_context  *arg_pCntxt )
  {
    ix_uint32 _nMask = 0, i, j;

    /* DiffServ microengines */
    i = arg_pCntxt->_pConf->_uEngMask; j = 0;
    while ( i != 0 )
      {
        if ( i & 0x01 )
          {
            _nMask |= (1 << g_aMicroengineNumberMapping[j]);
          }
        i >>= 1; j++;
      }

    /* Queue Manager microegnines */
    i = arg_pCntxt->_pConf->_qm_uEngNum; j = 0;
    while ( i != 0 )
      {
        if ( i & 0x01 )
          {
            _nMask |= (1 << g_aMicroengineNumberMapping[j]);
          }

        i >>= 1; j++;
      }

/*    halMe_ResetTimestamp( _nMask ); */
    return IX_SUCCESS;
  }

/* ------------------------------------------------------ ix_cc_wred_init -- */

ix_error ix_cc_wred_init(ix_cc_handle arg_hCC,
                         void **      arg_ppContext)
{
    ix_error  _result = IX_SUCCESS;
    ix_uint32 _i;

    _ix_cc_wred_config  * _pConf;
    _ix_cc_wred_context * _pCntxt;

    ix_uint32 _instance = 0;
    char _pInstancePrefix[WRED_MAX_PREFIX+1];

    /* validity check for arg_hCC */
    if ( arg_hCC == IX_NULL_HANDLE )
    {
        return IX_ERROR_WARNING(
            IX_CC_WRED_ERROR_INVALID_HANDLE,
            ( "ix_cc_wred_init(): invalid core component handle" )
          );
    }

    /* validity check for arg_ppUserContext */
    IX_ERROR_CHECK_ARG_NULL( 2, arg_ppContext );

    /* allocate memory for the core component's context */
    _pCntxt = (_ix_cc_wred_context*) ix_ossl_malloc(sizeof( _ix_cc_wred_context ));
    if ( _pCntxt == NULL )
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM_SYSTEM,
                   ( "ix_cc_wred_init(): not enough system memory available " \
                     "(cannot allocate memory for the context)" ));
    }



    /* allocate memory for the microblock context */
    _pConf = (_ix_cc_wred_config *)ix_ossl_malloc( sizeof( _ix_cc_wred_config ) );
    if ( _pConf == NULL )
    {
        _result = IX_ERROR_WARNING(IX_CC_ERROR_OOM_SYSTEM,
                     ( "ix_cc_wred_init(): not enough system memory available " \
                       "(cannot allocate memory for configuration)" ));
        goto err_ublockconfig_mem;
    }



    _pCntxt->_hCC   = arg_hCC;
    _pCntxt->_pConf = _pConf;



    /* read static configuration from the registry (if available) */
    #if defined( IX_INCLUDE_REGISTRY )
        _instance = IX_HANDLE_TO_INSTANCE(arg_hCC);
         
        _result = _ix_cc_wred_registry( _instance, _pCntxt->_pConf );

      if ( _result != IX_SUCCESS )
        {
              _result = IX_ERROR_WARNING(IX_CC_ERROR_INTERNAL,
                            ( "ix_cc_wred_init(): static configuration reading failed" ));
              goto err_no_reg;
        }
    #else /* IX_INCLUDE_REGISTRY */

        _pConf->_nEntrySize         = WRED_ENTRY_RAM_SIZE;

        _pConf->_nSRAMChnnl         = IX_CC_WRED_INSTANCE_TABLE_GET(_instance, defaultRamChnnl);

        _pConf->_nStatsSize         = WRED_64BIT_STAT_SRAM_SIZE;
        
        _pConf->_nStatsSRAMChnnl    = IX_MEMORY_TYPE_SRAM;

        _pConf->_nTableSize         = WRED_DEFAULT_TABLE_RAM_SIZE;

    #endif /* !IX_INCUDE_REGISTRY */


    #ifdef WRED_DRAM_TABLE
    #define WRED_MEMORY_TYPE  IX_MEMORY_TYPE_DRAM
    #else
    #define WRED_MEMORY_TYPE  IX_MEMORY_TYPE_SRAM
    #endif

    /* memory allocation for the wred tables */
    _result = ix_rm_mem_alloc(WRED_MEMORY_TYPE,
                              _pConf->_nTableRAMChnnl,
                              _GET_OFFSET(
                                  _pConf->_nEntrySize,
                                   _pConf->_nTableSize),
                              &_pCntxt->_pTableBase);

    #undef WRED_MEMORY_TYPE

    if ( _result != IX_SUCCESS )
    {
        #ifndef WRED_DRAM_TABLE
            _result = IX_ERROR_WARNING(IX_CC_ERROR_OOM_SRAM,
                   ( "ix_cc_wred_init(): not enough SRAM memory available" ));
        #else
            _result = IX_ERROR_WARNING(IX_CC_ERROR_OOM_DRAM,
                   ( "ix_cc_wred_init(): not enough DRAM memory available" ));
        #endif
        goto err_wred_tab_mem;
    }


    #ifdef WRED_PACKET_COUNTER_FEATURE
    
    /* memory allocation for the wred tables */
    _result = ix_rm_mem_alloc(IX_MEMORY_TYPE_SRAM,
                              _pConf->_nStatsSRAMChnnl,
                              _GET_OFFSET(
                                   _pConf->_nStatsSize,
                                   _pConf->_nTableSize),
                              &_pCntxt->_pStatsBase);

    #endif


    /* zero statistics and WRED table entries */
    for( _i = 0; _i < _pConf->_nTableSize; _i++ )
    {
        _ix_cc_wred_empty_table_entry(_WRED_TABLE_ENTRY( _i, _pCntxt ) );

        #ifdef WRED_PACKET_COUNTER_FEATURE
        _ix_cc_wred_empty_stats_entry(_WRED_STATS_ENTRY( _i, _pCntxt ) );
        #endif
    }


    #ifdef WRED_CORE_PROBABILITY_CALCULATION

    _pCntxt->pUpdatersTable = (_ix_cc_wred_updater_entry*)
            ix_ossl_malloc(sizeof(_ix_cc_wred_updater_entry) * _pConf->_nTableSize);

    if ( _pCntxt->pUpdatersTable  == NULL )
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM_SYSTEM,
                   ( "ix_cc_wred_init(): not enough system memory available " \
                     "(cannot allocate memory for the context)" ));
        goto err_updater_tab_mem;
    }

    #endif


    /* reset timestamp on the specified microengines */
    /* _ix_cc_wred_reset_timestamp( _pCntxt ); */


    _result = _ix_cc_wred_patch_symbols( _pCntxt );
    if ( _result != IX_SUCCESS )
    {
        _result = IX_ERROR_WARNING(IX_ERROR_GET_CODE(_result),
            ( "ix_cc_wred_init(): could not patch symbols" ));
        goto err_patch;
    }

    _result = _ix_cc_wred_register_pkt_handler( _pCntxt );
    if ( _result != IX_SUCCESS )
    {
        _result = IX_ERROR_WARNING(IX_ERROR_GET_CODE(_result),
            ( "ix_cc_wred_init(): could not register packet handler" ));
        goto err_patch;
    }

    _result = _ix_cc_wred_register_msg_handler( _pCntxt );
    if ( _result != IX_SUCCESS )
    {
        _result = IX_ERROR_WARNING(IX_ERROR_GET_CODE(_result),
            ( "ix_cc_wred_init(): could not register message handler" ));
        goto err_msg_handler;
    }

    *arg_ppContext = (void *)_pCntxt;
    return IX_SUCCESS;

    /* -------------------------------------------------------------------- */
    /*                          E R R O R S                                 */

err_msg_handler:
    _ix_cc_wred_unregister_pkt_handler();


err_patch:
    #ifdef WRED_CORE_PROBABILITY_CALCULATION
    ix_ossl_free( (void *)_pCntxt->pUpdatersTable );

err_updater_tab_mem:
    #endif

    _ix_cc_wred_free_symbols( _pCntxt );


err_wred_tab_mem:
#if defined( IX_INCLUDE_REGISTRY )
err_no_reg:
#endif
    ix_ossl_free( (void *)_pCntxt->_pConf);


err_ublockconfig_mem:
    ix_ossl_free( (void *)_pCntxt);


    *arg_ppContext = NULL;
    return _result;
}
