/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2000 - 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILENAME
 *      ix_cc_wred_msg_handler.c
 *
 * ============================================================================
 */
#define IX_ERROR_FILE_IDENT \
  "$Id: ix_cc_wred_msg_handler.c,v 1.13.4.1 2004/03/12 10:05:17 wneubaue Exp $"


#include <ix_types.h>
#include <ix_error.h>

#include <ix_ossl.h>
#include <ix_cc.h>
#include <cc/ix_cc_msup.h>
#include <cc/ix_cc_qm.h>
#include <cc/internal/ix_cc_qm_internal.h>

#include "../../include/wred_defs.h"
#include "cc/internal/ix_cc_wred_defs.h"
#include "cc/internal/ix_cc_wred_api.h"
#include "cc/internal/ix_cc_wred_types.h"
#include "cc/internal/ix_cc_wred_misc.h"

/* ------------------------------------------- ix_cc_wred_async_add_entry -- */
#ifdef WRED_CORE_PROBABILITY_CALCULATION

#include "cc/ix_cc_qm_frmwrk.h"

IX_CC_QM_INSTANCE_TABLE_DECLARE()

ix_error
_ix_cc_wred_cbi_get_queue_descriptor(ix_error   arg_Result,
                                     void      *arg_pAddContext,
                                     void      *arg_pQueueDescriptor,
                                     ix_uint32  arg_nMsgLength)
{
    ix_error _result;

    _ix_cc_wred_add_entry_data_ctx *_pAddContext = (_ix_cc_wred_add_entry_data_ctx *)arg_pAddContext;

    if (arg_Result == IX_SUCCESS)
    {
        #ifdef IX_DEBUG_WRED
        ix_ossl_message_log("_ix_cc_wred_cbi_get_queue_descriptor(): call wred api. \n");
        ix_ossl_message_log("_ix_cc_wred_cbi_get_queue_descriptor(): queue number     = 0x%x. \n", _pAddContext->EntryData._nInstance);
        ix_ossl_message_log("_ix_cc_wred_cbi_get_queue_descriptor(): queue descriptor = 0x%x. \n", *(ix_uint32*)arg_pQueueDescriptor);
        #endif
        _result = ix_cc_wred_add_entry(_pAddContext->EntryData._nInstance,
                                       &_pAddContext->EntryData._WredParams,
                                       *(ix_uint32**)arg_pQueueDescriptor,
                                       _pAddContext->PCoreContext);
    } else {
        #ifdef IX_DEBUG_WRED
        ix_ossl_message_log("_ix_cc_wred_cbi_get_queue_descriptor(): wrong queue_descriptor. \n");
        #endif
        _result = arg_Result;
    }

    #ifdef IX_DEBUG_WRED
    ix_ossl_message_log("_ix_cc_wred_cbi_get_queue_descriptor(): send reply. \n");
    #endif

    /* send return message, length and retrieved context to message support library */
    _result = ix_cc_msup_send_reply_msg(_pAddContext->PUserContext, NULL, 0, _result);

    IX_ERROR_CRT(ix_rm_buffer_free_chain(_pAddContext->ReplyMsgBuffer),
                 IX_CC_ERROR_INTERNAL, 0);

    if(_result != IX_SUCCESS)
    {
        _result = IX_ERROR_WARNING(IX_CC_ERROR_SEND_FAIL,
             ("_ix_cc_wred_cbi_get_queue_descriptor(): Send message to wred failed"));
    }

    return _result;
}

#endif

/* ----------------------------------------------- ix_cc_wred_msg_handler -- */
ix_error ix_cc_wred_msg_handler( ix_buffer_handle arg_hBuffer,
                                 ix_uint32        arg_nMsgType,
                                 void *           arg_pContext )
  {
    
    void * _pMsg    = NULL;
    void * _pCntx   = NULL;
    void * _pRetMsg = NULL;
    ix_uint32   _nRetMsgLength = 0;
    ix_error    _result        = IX_SUCCESS;
    ix_cc_wred_parameters   retParams;
    ix_cc_wred_statistics   retStats;

    /* validity check for arg_pContext */
    if ( arg_pContext == NULL )
      {
        return IX_ERROR_WARNING(
          IX_CC_ERROR_NULL,
          ( "ix_cc_wred_msg_data(): arg_pContext is NULL" )
        );
      }
    /* validity check for the arg_pUserData */
    if ( (arg_nMsgType < IX_CC_COMMON_MSG_ID_FIRST) ||
         (arg_nMsgType >= IX_CC_WRED_MSG_LAST) )
      {
        return IX_ERROR_WARNING(
          IX_CC_ERROR_UNDEFINED_MSG,
          ( "ix_cc_wred_msg_data(): invalid message type" )
        );
      }
    /* extract the message */
    IX_MSUP_EXTRACT_MSG( arg_hBuffer, &_pMsg, &_pCntx );
    if ( _pMsg == NULL )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_NULL,
            ( "ix_cc_wred_msg_handler(): invalid buffer" )
          );
      }
    /* handle the message */
    switch ( arg_nMsgType )
      {
        case IX_CC_WRED_MSG_UPDATE_ENTRY:
          {
            _ix_cc_wred_update_entry_data * _pData =
              (_ix_cc_wred_update_entry_data *)_pMsg;

            #ifdef IX_DEBUG_WRED
            ix_ossl_message_log("ix_cc_wred_msg_handler(): service_rate = %d\n",_pData->_Params.service_rate);
            ix_ossl_message_log("ix_cc_wred_msg_handler(): queue number         0x%x\n",_pData->_nInstance);
            #endif

            _result = ix_cc_wred_update_entry(
                _pData->_nInstance, &(_pData->_Params), arg_pContext
              );

            break;
          }
        case IX_CC_WRED_MSG_GET_STATISTICS:
          {
            _ix_cc_wred_get_stats_data * _pData =
              (_ix_cc_wred_get_stats_data *)_pMsg;

            _result = ix_cc_wred_get_statistics(
                _pData->_nInstance, &retStats, arg_pContext);

            _nRetMsgLength = sizeof(retStats);
            _pRetMsg       = (void *)&retStats;
            break;
          }
        case IX_CC_WRED_MSG_ADD_ENTRY:
        {
              _ix_cc_wred_add_entry_data     *_pData = (_ix_cc_wred_add_entry_data *)_pMsg;

        #ifdef WRED_CORE_PROBABILITY_CALCULATION

              _ix_cc_wred_add_entry_data_ctx *_pAddContext;
              ix_uint32 instanceCC = IX_HANDLE_TO_INSTANCE(((_ix_cc_wred_context *)arg_pContext)->_hCC); 
              
              _pAddContext = ix_ossl_malloc(sizeof(_ix_cc_wred_add_entry_data_ctx));
              _pAddContext -> EntryData      = *((_ix_cc_wred_add_entry_data *)_pMsg);
              _pAddContext -> PCoreContext   = arg_pContext;
              _pAddContext -> PUserContext   = _pCntx;
              _pAddContext -> ReplyMsgBuffer = arg_hBuffer;

              #ifdef IX_DEBUG_WRED
              ix_ossl_message_log("ix_cc_wred_msg_handler(): service_rate = %d\n",_pAddContext->EntryData._WredParams.service_rate);
              ix_ossl_message_log("ix_cc_wred_msg_handler(): queue number         0x%x\n",_pData->_nInstance);
              ix_ossl_message_log("ix_cc_wred_msg_handler(): queue number address 0x%x\n",(ix_uint32*)(&(_pData->_nInstance)));
              #endif

              _result = ix_cc_msup_send_async_msg (IX_CC_QM_INSTANCE_TABLE_GET(instanceCC,msgIn),
                                     (ix_cc_msghlp_callback)_ix_cc_wred_cbi_get_queue_descriptor,
                                     _pAddContext,
                                     IX_CC_QM_MSG_GET_QUEUE_DESCRIPTOR,
                                     (ix_uint32*)(&(_pData->_nInstance)),
                                     sizeof(ix_uint32));

              #ifdef IX_DEBUG_WRED
              ix_ossl_message_log("ix_cc_wred_msg_handler(): message sent.\n");
              #endif

              if ( _result != IX_SUCCESS )
              {
                  ix_ossl_free(_pAddContext);
              }

              ix_ossl_free(_pMsg);

              return _result;

        #else
              _result = ix_cc_wred_add_entry(
                      _pData->_nInstance, &(_pData->_WredParams), NULL, arg_pContext);

              break;
        #endif

        }
        case IX_CC_WRED_MSG_REMOVE_ENTRY:
          {
            _ix_cc_wred_remove_entry_data * _pData =
              (_ix_cc_wred_remove_entry_data *)_pMsg;
            _result = ix_cc_wred_remove_entry(
              _pData->_nInstance, arg_pContext );
            break;
          }
        case IX_CC_WRED_MSG_GET_PARAMETERS:
          {
            _ix_cc_wred_get_params_data * _pData =
              (_ix_cc_wred_get_params_data *)_pMsg;
            
            _result = ix_cc_wred_get_parameters(
                _pData->_nInstance, &retParams, arg_pContext);

            _nRetMsgLength = sizeof(retParams);
            _pRetMsg       = (void *)&(retParams);
            break;
          }
        default:
          IX_ERROR_CRT(
              ix_rm_buffer_free_chain( arg_hBuffer ),
              IX_CC_ERROR_INTERNAL, 0
            );
          return IX_ERROR_WARNING(
              IX_CC_ERROR_UNDEFINED_MSG,
              ( "ix_cc_wred_msg_handler(): unsupported message type" )
            );
    }

    /* send reply to the client */
    _result = ix_cc_msup_send_reply_msg(_pCntx,
                                        _pRetMsg,
                                        _nRetMsgLength,
                                        _result);
    if ( _result != IX_SUCCESS )
    {
         _result = IX_ERROR_WARNING(IX_CC_ERROR_SEND_FAIL,
                         ( "ix_cc_wred_msg_handler(): an error occured"));
    }

    ix_ossl_free(_pMsg);

    IX_ERROR_CRT(ix_rm_buffer_free_chain( arg_hBuffer ),
                 IX_CC_ERROR_INTERNAL, 0);

    return _result;
}
