/*******************************************************************************
  INTEL CONFIDENTIAL

  Copyright 2000 - 2003 Intel Corporation All Rights Reserved.

  The source code contained or described herein and all documents related to
  the source code ("Material") are owned by Intel Corporation or its 
  suppliers or licensors.

  Title to the Material remains with Intel Corporation or its suppliers and
  licensors. The Material contains trade secrets and proprietary and
  confidential information of Intel or its suppliers and licensors.
  The Material is protected by worldwide copyright and trade secret laws and
  treaty provisions. No part of the Material may be used, copied, reproduced,
  modified, published, uploaded, posted, transmitted, distributed,
  or disclosed in any way without Intel's prior express written permission.

  No license under any patent, copyright, trade secret or other intellectual
  property right is granted to or conferred upon you by disclosure
  or delivery of the Materials, either expressly, by implication, inducement,
  estoppel or otherwise. Any license under such intellectual property rights
  must be express and approved by Intel in writing.

*******************************************************************************/

/**
 * ============================================================================
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2000 Network Processor
 *
 * = FILENAME
 *      ix_cc_wred_msghlp.c
 *
 * ============================================================================
 */

#define IX_ERROR_FILE_IDENT \
  "$Id: ix_cc_wred_msghlp.c,v 1.14.4.1 2004/03/12 10:05:17 wneubaue Exp $"




#include <ix_types.h>
#include <ix_error.h>
#include <ix_ossl.h>
#include <ix_cc.h>
#include <cc/ix_cc_msup.h>

#include "../../include/wred_defs.h"
#include "cc/ix_cc_wred.h"
#include "cc/internal/ix_cc_wred_defs.h"
#include "cc/internal/ix_cc_wred_api.h"
#include "cc/internal/ix_cc_wred_types.h"
#include "cc/internal/ix_cc_wred_misc.h"
#include "cc/ix_cc_wred_msghlp.h"

IX_CC_WRED_INSTANCE_TABLE_DECLARE()

/* ----------------------------------------------- _ix_cc_wred_cb_default -- */
PRIVATE ix_error _ix_cc_wred_cb_default( ix_error  arg_Result,
                                         void *    arg_pCallbackContext,
                                         void *    arg_pMsg,
                                         ix_uint32 arg_nMsgLength )
  {
    _ix_cc_wred_default_cb_context * _pCntxt =  
      (_ix_cc_wred_default_cb_context *)arg_pCallbackContext;
    
    if ( arg_pCallbackContext != NULL )
      {
        ix_error _result =
          _pCntxt->_pCallback( arg_Result, _pCntxt->_pUserContext );

        _SYSTEM_DELETE( arg_pCallbackContext );
        return _result;
      }

    return IX_ERROR_PANIC(
        IX_CC_ERROR_NULL,
        ( "invalid callback context" )
      );
  }



/* --------------------------------------------- _ix_cc_wred_cb_get_stats -- */
#ifdef WRED_PACKET_COUNTER_FEATURE
PRIVATE ix_error _ix_cc_wred_cb_get_stats( ix_error  arg_Result, 
                                           void *    arg_pCallbackContext,
                                           void *    arg_pMsg,
                                           ix_uint32 arg_nMsgLength )
  {
    _ix_cc_wred_get_stats_cb_context * _pCntxt =
      (_ix_cc_wred_get_stats_cb_context *)arg_pCallbackContext;

    if ( arg_Result == IX_SUCCESS )
      {
        if ( arg_pMsg == NULL )
          {
            return IX_ERROR_WARNING(
                IX_CC_ERROR_NULL,
                ( "NULL message data pointer specified" )
              );
          }

        if ( arg_nMsgLength != sizeof( ix_cc_wred_statistics ) )
          {
            return IX_ERROR_WARNING(
                IX_CC_ERROR_INTERNAL,
                ( "message length doesn't match expected size" )
              );
          }
      }
    else
      {
        arg_pMsg = NULL;
      }

    
    if ( arg_pCallbackContext != NULL )
    {
        ix_error _result;
        
        *(_pCntxt->_pUserPlaceHolder) = *(ix_cc_wred_statistics *)arg_pMsg;

        _result = _pCntxt->_pCallback(arg_Result,
                                      _pCntxt->_pUserPlaceHolder,
                                      _pCntxt->_pUserContext);
        _SYSTEM_DELETE( arg_pCallbackContext );
        return _result;
    }

    return IX_ERROR_PANIC(
        IX_CC_ERROR_NULL,
        ( "invalid callback context" )
      );
  }
#endif



/* --------------------------------------------- _ix_cc_wred_cb_get_params -- */
PRIVATE ix_error _ix_cc_wred_cb_get_params( ix_error  arg_Result, 
                                           void *    arg_pCallbackContext,
                                           void *    arg_pMsg,
                                           ix_uint32 arg_nMsgLength )
  {
    _ix_cc_wred_get_params_cb_context * _pCntxt =
      (_ix_cc_wred_get_params_cb_context *)arg_pCallbackContext;

    if ( arg_Result == IX_SUCCESS )
      {
        if ( arg_pMsg == NULL )
          {
            return IX_ERROR_WARNING(
                IX_CC_ERROR_NULL,
                ( "NULL message data pointer specified" )
              );
          }

        if ( arg_nMsgLength != sizeof( ix_cc_wred_parameters ) )
          {
            return IX_ERROR_WARNING(
                IX_CC_ERROR_INTERNAL,
                ( "message length doesn't match expected size" )
              );
          }
      }
    else
      {
        arg_pMsg = NULL;
      }

    
    if ( arg_pCallbackContext != NULL )
    {
        ix_error _result;

        *(_pCntxt->_userPlaceHolder) = *(ix_cc_wred_parameters *)arg_pMsg;

        _result = _pCntxt->_pCallback(arg_Result,
                                      _pCntxt->_userPlaceHolder,
                                      _pCntxt->_pUserContext);

        _SYSTEM_DELETE( arg_pCallbackContext );
        return _result;
    }

    return IX_ERROR_PANIC(
                 IX_CC_ERROR_NULL, ( "invalid callback context" ));
  }



/* ------------------------------------------- ix_cc_wred_async_add_entry -- */
ix_error ix_cc_wred_async_add_entry(
    ix_uint32                   arg_nInstance, 
    ix_cc_wred_parameters *     arg_pParams,
    ix_cc_wred_callback_default arg_Callback,
    ix_generic_context *        arg_pUserContext )
  {
    ix_error _nResult = IX_SUCCESS;
    
    _ix_cc_wred_default_cb_context * _cbCntx;
    _ix_cc_wred_add_entry_data msgData;

    ix_uint32 instanceCC;

    /* validity check for arg_Callback */
    IX_ERROR_CHECK_ARG_NULL( 3, arg_Callback );
    /* validity check for arg_pUserContext */
    IX_ERROR_CHECK_ARG_NULL( 4, arg_pUserContext );

    instanceCC = IX_HANDLE_TO_INSTANCE(arg_pUserContext->hCC);

    /* allocate memory for the callback context */
    _cbCntx = ix_ossl_malloc( sizeof( _ix_cc_wred_default_cb_context ) );
    if ( _cbCntx == NULL )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_OOM,
            ( "not enough system memory available" )
          );
      }

    #ifdef IX_DEBUG_WRED
    ix_ossl_message_log("ix_cc_wred_async_add_entry(): adding   :)))))) \n");
    ix_ossl_message_log("ix_cc_wred_async_add_entry(): check serv_rate");
    if ( arg_pParams->service_rate > (1<<8) )
       ix_ossl_message_log("ix_cc_wred_async_add_entry(): invalid service_rate specified- %d\n",
                                                                       arg_pParams->service_rate);
    #endif

    /* callback contex */
    _cbCntx->_pCallback    = arg_Callback;
    _cbCntx->_pUserContext = arg_pUserContext;
    
    /* message data */
    msgData._WredParams    = *arg_pParams;
    msgData._nInstance     = arg_nInstance;

    /* send message to the wred core component */
    _nResult = ix_cc_msup_send_async_msg(
        IX_CC_WRED_INSTANCE_TABLE_GET(instanceCC,msgIn),
        _ix_cc_wred_cb_default,
        (void *)_cbCntx,
        IX_CC_WRED_MSG_ADD_ENTRY,
        (void *)&msgData,
        sizeof( _ix_cc_wred_add_entry_data )
      );

    if ( _nResult != IX_SUCCESS )
      {
        _SYSTEM_DELETE( _cbCntx );

        return IX_ERROR_WARNING(
            IX_CC_ERROR_SEND_FAIL,
            ( "failed to send asynchronous message" )
          );
      }

    return IX_SUCCESS;
  }



/* ---------------------------------------- ix_cc_wred_async_remove_entry -- */
ix_error ix_cc_wred_async_remove_entry(
    ix_uint32                   arg_nInstance,
    ix_cc_wred_callback_default arg_Callback,
    ix_generic_context *        arg_pUserContext )
  {
    ix_error _nResult = IX_SUCCESS;

    _ix_cc_wred_default_cb_context * _cbCntx;
    _ix_cc_wred_remove_entry_data msgData;

    ix_uint32 instanceCC;

    /* validity check for arg_Callback */
    IX_ERROR_CHECK_ARG_NULL( 2, arg_Callback );
    /* validity check for arg_pUserContext */
    IX_ERROR_CHECK_ARG_NULL( 3, arg_pUserContext );

    instanceCC = IX_HANDLE_TO_INSTANCE(arg_pUserContext->hCC);

    /* allocate memory for the callback context */
    _cbCntx = ix_ossl_malloc( sizeof( _ix_cc_wred_default_cb_context ) );
    if ( _cbCntx == NULL )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_OOM,
            ( "not enough system memory available" )
          );
      }

    _cbCntx->_pCallback    = arg_Callback;
    _cbCntx->_pUserContext = arg_pUserContext;

    msgData._nInstance     = arg_nInstance;

    _nResult = ix_cc_msup_send_async_msg(
        IX_CC_WRED_INSTANCE_TABLE_GET(instanceCC,msgIn),
        _ix_cc_wred_cb_default,
        (void *)_cbCntx,
        IX_CC_WRED_MSG_REMOVE_ENTRY,
        (void *)&msgData,
        sizeof( _ix_cc_wred_remove_entry_data )
      );

    if ( _nResult != IX_SUCCESS )
      {
        ix_ossl_free( _cbCntx );

        return IX_ERROR_WARNING(
            IX_CC_ERROR_SEND_FAIL,
            ( "failed to send asynchronous message" )
          );
      }

    return IX_SUCCESS;
  }



/* ---------------------------------------- ix_cc_wred_async_update_entry -- */
ix_error ix_cc_wred_async_update_entry(
    ix_uint32                   arg_nInstance,
    ix_cc_wred_parameters *     arg_pParams,
    ix_cc_wred_callback_default arg_Callback,
    ix_generic_context *        arg_pUserContext )
  {    
    ix_error _nResult = IX_SUCCESS;

    _ix_cc_wred_default_cb_context * _cbCntx;
    _ix_cc_wred_update_entry_data msgData;

    ix_uint32 instanceCC;

    /* validity check for arg_Callback */
    IX_ERROR_CHECK_ARG_NULL( 3, arg_Callback );
    /* validity check for arg_pUserContext */
    IX_ERROR_CHECK_ARG_NULL( 4, arg_pUserContext );

    instanceCC = IX_HANDLE_TO_INSTANCE(arg_pUserContext->hCC);

    /* allocate memory for the callback context */
    _cbCntx = ix_ossl_malloc( sizeof( _ix_cc_wred_default_cb_context ) );
    if ( _cbCntx == NULL )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_OOM,
            ( "not enough system memory available" )
          );
      }

    #ifdef IX_DEBUG_WRED
    ix_ossl_message_log("ix_cc_wred_async_update_entry(): service_rate = %d.\n",arg_pParams->service_rate);
    #endif

    _cbCntx->_pCallback    = arg_Callback;
    _cbCntx->_pUserContext = arg_pUserContext;

    msgData._Params       = *arg_pParams;
    msgData._nInstance    =  arg_nInstance;

    _nResult = ix_cc_msup_send_async_msg(
        IX_CC_WRED_INSTANCE_TABLE_GET(instanceCC,msgIn),
        _ix_cc_wred_cb_default,
        (void *)_cbCntx,
        IX_CC_WRED_MSG_UPDATE_ENTRY,
        (void *)&msgData,
        sizeof( _ix_cc_wred_update_entry_data )
      );

    if ( _nResult != IX_SUCCESS )
      {
        _SYSTEM_DELETE( _cbCntx );

        return IX_ERROR_WARNING(
            IX_CC_ERROR_SEND_FAIL,
            ( "failed to send asynchronous message" )
          );
      }

    return IX_SUCCESS;
  }



/* ------------------------------------------- ix_cc_wred_async_get_stats -- */
#ifdef WRED_PACKET_COUNTER_FEATURE
ix_error ix_cc_wred_async_get_statistics( 
    ix_uint32                     arg_nInstance,
    ix_cc_wred_statistics *       arg_pStats,
    ix_cc_wred_callback_get_stats arg_Callback,
    ix_generic_context *          arg_pUserContext )
  {
    ix_error _nResult = IX_SUCCESS;

    _ix_cc_wred_get_stats_data        msgData;
    _ix_cc_wred_get_stats_cb_context *_cbCntx;
    
    ix_uint32 instanceCC;

    /* validity check for arg_Callback */
    IX_ERROR_CHECK_ARG_NULL( 3, arg_Callback );
    /* validity check for arg_pUserContext */
    IX_ERROR_CHECK_ARG_NULL( 4, arg_pUserContext );

    instanceCC = IX_HANDLE_TO_INSTANCE(arg_pUserContext->hCC);


    /* allocate memory for the callback context */
    _cbCntx = ix_ossl_malloc( sizeof( _ix_cc_wred_get_stats_cb_context ) );
    if ( _cbCntx == NULL )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_OOM,
            ( "not enough system memory available" )
          );
      }

    _cbCntx->_pCallback        = arg_Callback;
    _cbCntx->_pUserContext     = arg_pUserContext;
    _cbCntx->_pUserPlaceHolder = arg_pStats;

    msgData._nInstance     =  arg_nInstance;

    _nResult = ix_cc_msup_send_async_msg(
        IX_CC_WRED_INSTANCE_TABLE_GET(instanceCC,msgIn),
        _ix_cc_wred_cb_get_stats,
        (void *)_cbCntx,
        IX_CC_WRED_MSG_GET_STATISTICS,
        (void *)&msgData,
        sizeof( _ix_cc_wred_get_stats_data )
      );

    if ( _nResult != IX_SUCCESS )
      {
        _SYSTEM_DELETE( _cbCntx );

        return IX_ERROR_WARNING(
            IX_CC_ERROR_SEND_FAIL,
            ( "failed to send asynchronous message" )
          );
      }

    return IX_SUCCESS;
}
#endif



/**
 * @brief Sends a parameters read message to the WRED core component
 *
 * This helper function builds and sends a parameters read message to the WRED 
 * core component. The core component checks if the WRED entry, pointed by 
 * arg_nInstance, is valid. If the referred table entry is empty, the core 
 * component reports an error (via a callback function). Otherwise, it retrieves
 * parameters values associated with this entry and returns them in a callback
 * function. 
 *
 * @param  arg_nInstance    index of a WRED table entry that is queried for
 *                          parameters.
 *         
 * @param  arg_pParams      pointer to the parameters placeholder that should
 *                          be filled with data by the core component.
 *         
 * @param  arg_Callback     a client provided callback function. The core 
 *                          component invokes this function to report an 
 *                          operation status and parameters values.
 *
 * @param  arg_pUserContext pointer to the client defined context. This value
 *                          is opaque to the core componet. The context is
 *                          returned in a callback function.
 *
 * @return IX_SUCCESS if successful or a valid ix_error token for failure. 
 *         The following error codes might be returned via the token.
 *
 * @see    <code>ix_cc_wred_callback_get_params</code>\n
 *         <code>ix_cc_wred_parameters</code>
 */
ix_error ix_cc_wred_async_get_parameters(
    ix_uint32                       arg_nInstance,
    ix_cc_wred_parameters          *arg_pParams,
    ix_cc_wred_callback_get_params  arg_Callback,
    ix_generic_context *        arg_pUserContext)
  {    
    ix_error _nResult = IX_SUCCESS;

    _ix_cc_wred_get_params_cb_context *_cbCntx;
    _ix_cc_wred_get_params_data        msgData;

    ix_uint32 instanceCC;

    /* validity check for arg_Callback */
    IX_ERROR_CHECK_ARG_NULL( 3, arg_Callback );
    /* validity check for arg_pUserContext */
    IX_ERROR_CHECK_ARG_NULL( 4, arg_pUserContext );

    instanceCC = IX_HANDLE_TO_INSTANCE(arg_pUserContext->hCC);

    /* allocate memory for the callback context */
    _cbCntx = ix_ossl_malloc( sizeof( _ix_cc_wred_get_params_cb_context ) );
    if ( _cbCntx == NULL )
      {
        return IX_ERROR_WARNING(
            IX_CC_ERROR_OOM,
            ( "not enough system memory available" )
          );
      }

    _cbCntx->_pCallback       = arg_Callback;
    _cbCntx->_pUserContext    = arg_pUserContext;
    _cbCntx->_userPlaceHolder = arg_pParams;

    msgData._nInstance     =  arg_nInstance;

    _nResult = ix_cc_msup_send_async_msg(
        IX_CC_WRED_INSTANCE_TABLE_GET(instanceCC,msgIn),
        _ix_cc_wred_cb_get_params,
        (void *)_cbCntx,
        IX_CC_WRED_MSG_GET_PARAMETERS,
        (void *)&msgData,
        sizeof( _ix_cc_wred_get_params_data )
      );

    if ( _nResult != IX_SUCCESS )
      {
        _SYSTEM_DELETE( _cbCntx );

        return IX_ERROR_WARNING(
            IX_CC_ERROR_SEND_FAIL,
            ( "failed to send asynchronous message" )
          );
      }

    return IX_SUCCESS;
  }



