#if !defined(__IX_CC_ATMRX_HASH_H__)
#define      __IX_CC_ATMRX_HASH_H__
/*
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor                
 *
 * = MODULE
 *      ATM RX Core Component
 *
 * = FILENAME
 *      ix_cc_atmrx_hash.h
 * 
 */


/**
 * NAME: atmrx_hash_48
 *
 * DESCRIPTION:
 *    Calculates hardware based hash_48
 * 
 * @Param:  IN  arg_pLw :  pointer to table of 2 long words
 * 
 * @Return: value of counted HASH
 */
ix_uint32 atmrx_hash_48(const ix_uint32* arg_pLw);



/**
 * DEFINITION: _IX_CC_ATMRX_HASH_SLEEP_TICKS   */
/**
 * ABSTRACT: time during which thread sleeps before asking again about
 *           completion of hash operation
 *
 * DESCRIPTION: when performing api calls xscale uses hardware hash unit.
 *              Having called that hash it has to wait for result periodically 
 *              polling its internal register to find out if operation has ben completed
 *              This constant defines how many waiting cycles it will wait between 
 *              successive enquiries.
 */ 
#define _IX_CC_ATMRX_HASH_SLEEP_TICKS			1


/**
 * DEFINITION: IX_CC_ATMRX_HASH_TIME_OUT		     
 *
 * ABSTRACT: time after which fail of hash computation decision is made 
 *
 * DESCRIPTION: when performing api calls xscale uses hardware hash unit.
 *              Having called that hash it has to wait for result periodically 
 *              polling its internal register to find out if operation has ben completed
 *              This constant defines how many times it will be polling.
 */ 
#define _IX_CC_ATMRX_HASH_TIME_OUT				10



/**
 * DEFINITION: HASH_CAP_BASE  
 *
 * ABSTRACT:
 *
 * DESCRIPTION: Base addresses of a hash multipliers - The registers which 
 *              contain the programmable hash multiplier for generating hash keys.
 */
#define HASH_CAP_BASE	       0xC0004900


/**
 * DEFINITION: HASH_MULTIPLIER_48_0 
 *
 * ABSTRACT: hardware register containing a part of 128-bit hash multiplier 
 *
 * DESCRIPTION: This register contains the least significant 32 bits of the 
 *              programmable hash multiplier for generating 128-bit hash keys. 
 */
#define HASH_MULTIPLIER_48_0  (HASH_CAP_BASE + 0x00)

/**
 * DEFINITION: HASH_MULTIPLIER_48_1 
 *
 * ABSTRACT: hardware register containing a part of 128-bit hash multiplier 
 *
 * DESCRIPTION: This register contains bits 32 to 63 of the the programmable 
 *              hash multiplier for generating 128-bit hash keys.
 */
#define HASH_MULTIPLIER_48_1  (HASH_CAP_BASE + 0x04)


/*  registers used to perform hardware support hash */

/**
 * DEFINITION: HASH_DATA_48_0   
 *
 * ABSTRACT: hardware register where data to compute hash index on are put.
 *
 * DESCRIPTION: First Hash Data Register where Xscale can write data to be hashed.
 *              Writing to this register in a set initiates the write operation 
 *              to the Hash Unit. Later it contains the result of the operation. 
 *              XScale can poll the Hash Done Register to determine 
 *              when the hash result has been returned.
 */
#define HASH_DATA_48_0        0xD7000000


/**
 * DEFINITION: HASH_DATA_48_1   
 *
 * ABSTRACT:  hardware register where data to compute hash index on are put.
 *
 * DESCRIPTION: Second Hash Data Register where Xscale can write data to be hashed.
 *              Later it contains the result of the operation. 
 *              XScale can poll the Hash Done Register to determine 
 *              when the hash result has been returned.
 */
#define HASH_DATA_48_1        0xD7000004



/**
 * DEFINITION: HASH_DONE    
 *
 * ABSTRACT:  indicates if the hash operation has been completed
 *
 * DESCRIPTION: Hash Done Register to determine when the hash result has been returned
 *              XScale reads the results from the Hash Data Registers.
 *              Because of queuing delays at the Hash Unit, the time to complete an operation 
 *              is not fixed. XScale polls this Hash Done Register to determine
 *              when the hash result has been returned. This register is cleared 
 *              when the Hash Data Registers are written to the Hash Unit. 
 *              Bit [0] of Hash Done Register is set when the Hash Data Registers get the
 *              return result from the Hash Unit (when the last word of the result is returned).
 */
#define HASH_DONE              0xD7000030


/**
 * DEFINITION: HASH_DONE_SIGNAL    
 *
 * ABSTRACT: Value of HASH_DONE register that indicates completion of hash operation
 *
 * DESCRIPTION: Value of HASH_DONE register that indicates completion of hash operation.
 */
#define HASH_DONE_SIGNAL       0x00000001



/**
 * MACRO NAME: SET_HASH_MULTIPLIER_LW_0 
 *
 * ABSTRACT: This macro will put data to the least significant
 *           longword of hash multiplier CSR.  
 *
 * DESCRIPTION: This macro will put data to the least significant
 *              longword of hash multiplier CSR. This register contains 
 *              the programmable hash multiplier for generating 128 bit hash key.
 *
 * @param arg_Data: new value of the multiplier
 */
#define SET_HASH_MULTIPLIER_LW_0( arg_Data ) \
           (*((volatile ix_uint32*) HASH_MULTIPLIER_48_0) = (unsigned int)arg_Data)


/**
 * MACRO NAME: SET_HASH_MULTIPLIER_LW_1 
 *
 * ABSTRACT: The macro will put data to hash multiplier CSR 
 *
 * DESCRIPTION: The macro will put data to hash multiplier CSR 
 *              That register contains the programmable hash multiplier
 *              for generating 128 bit hash key.
 *
 * @param arg_Data: new value of the multiplier
 */

#define SET_HASH_MULTIPLIER_LW_1( arg_Data ) \
          (*((volatile ix_uint32*) HASH_MULTIPLIER_48_1) = (unsigned int)arg_Data)



/**
 * NAME: SET_HASH_DATA_48_0
 *
 * DESCRIPTION:
 *    This macro will put data to HASH_DATA_48_0 register.
 *
 * @param:  IN arg_Data: value to be set in the HASH_DATA_128_0 register.
 *
 */
#define SET_HASH_DATA_48_0( arg_Data ) \
           (*((volatile ix_uint32*)HASH_DATA_48_0) = (unsigned int)arg_Data)


/**
 * MACRO NAME: SET_HASH_DATA_48_1
 *
 * ABSTRACT: This macro will put data to HASH_DATA_48_1 register.
 *
 * DESCRIPTION: This macro will put data to HASH_DATA_48_1 register.
 *
 * @param  arg_Data - value to be set in the HASH_DATA_48_1 register.
 */
#define SET_HASH_DATA_48_1( arg_Data ) \
           (*((volatile ix_uint32*)HASH_DATA_48_1) = (unsigned int)arg_Data)

/**
 * MACRO NAME: GET_HASH_DATA_48_0 
 *
 * ABSTRACT: This macro will get data from HASH_DATA_48_0 register.
 *
 * DESCRIPTION: This macro will get data from HASH_DATA_48_0 register.
 *
 * @return the value from HASH_DATA_48_0 register.
 */
#define GET_HASH_DATA_48_0 \
           (*((volatile ix_uint32*)HASH_DATA_48_0))

/**
 * MACRO NAME: GET_HASH_DATA_48_1
 *
 * ABSTRACT: This macro will get data from HASH_DATA_48_1 register.
 *
 * DESCRIPTION: This macro will get data from HASH_DATA_48_1 register.
 *
 * @return  the value from HASH_DATA_128_1 register.
 */
#define GET_HASH_DATA_48_1 \
           (*((volatile ix_uint32*)HASH_DATA_48_1))


/**
 * MACRO NAME: GET_HASH_DONE 
 *
 * ABSTRACT: This macro will get the value of HASH_DONE register.
 *
 * DESCRIPTION: This macro will get the value of HASH_DONE register.
 *
 * @return     the value of HASH_DONE register. Bit [0] of Hash Done Register is set
 *             when the Hash Data Registers get the return result from 
 *             the Hash Unit (when the last word of the result is returned).
 */
#define GET_HASH_DONE \
           (*((volatile ix_uint32*)HASH_DONE))


#endif

