/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      ATM RX Core Component
 *
 * = FILENAME
 *      ix_cc_atmrx_private.h
 *
 * = DESCRIPTION
 *
 * = AUTHOR
 *      Jaroslaw Kogut
 *      jaroslaw.kogut@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * 
 */

#if !defined(__IX_CC_ATMRX_PRIVATE_H__)
#define __IX_CC_ATMRX_PRIVATE_H__

#include "cc/ix_cc_atmsar.h"
#include "cc/ix_cc_atmsar_plugin.h"
#include "cc/ix_cc_atmsar_list.h"
#include "dl_system_config.h" /* TBD JL dl_system.h */
#include "aal5_rx_defs.h"
#ifdef AAL2_INCLUDED
#include "aal2_rx_defs.h"
#endif

#if defined(__cplusplus)
extern "C" {
#endif /* end defined(__cplusplus) */

#ifdef AAL2_INCLUDED
/* TBD names of some defines should be changed.*/
#define AAL2_CID_HASH_PRIM_BKT_SZ_BYTES AAL2_CID_HASH_PRIM_ENTRY_SIZE
#define AAL2_CID_HASH_SEC_BKT_SZ_BYTES  AAL2_CID_HASH_SEC_ENTRY_SIZE
#endif

/**
 * TYPENAME: _ix_cc_atmrx_symbol_type_t
 * 
 * DESCRIPTION:
 *    This enum defines the symbol types.
 */
typedef enum _ix_cc_atmrx_e_symbol_type
{
    _IX_CC_ATMRX_AAL5_PORT_SYMBOL_TYPE=0,
    _IX_CC_ATMRX_AAL5_VC_SYMBOL_TYPE,

#ifdef AAL2_INCLUDED

    _IX_CC_ATMRX_AAL2_CID_SYMBOL_TYPE,

#endif

    _IX_CC_ATMRX_SYMBOL_TYPE_LAST

} _ix_cc_atmrx_e_symbol_type_t;

/**
 * TYPENAME: _ix_cc_atmrx_symbol_id_t
 * 
 * DESCRIPTION:
 *    This enum defines number of symbols serviced by the core component
 */
typedef enum _ix_cc_atmrx_e_symbol_id
{
    _IX_CC_ATMRX_AAL5_RX_PORT_STAT_SRAM_BASE_SYMBOL_ID=0,
    _IX_CC_ATMRX_AAL5_RX_VC_INFO_SRAM_BASE_SYMBOL_ID,
    _IX_CC_ATMRX_VC_HASH_PRI_BASE_SRAM_SYMBOL_ID,
    _IX_CC_ATMRX_VC_HASH_SEC_BASE_SRAM_SYMBOL_ID,

#ifdef AAL2_INCLUDED

    _IX_CC_ATMRX_VC_LOOKUP_TABLE_BASE_SYMBOL_ID,
    _IX_CC_ATMRX_AAL2_VC_RX_CTX_BASE_SYMBOL_ID,
    _IX_CC_ATMRX_AAL2_CID_HASH_PRIM_SYMBOL_ID,
    _IX_CC_ATMRX_AAL2_CID_HASH_SEC_SYMBOL_ID,
    _IX_CC_ATMRX_AAL2_CID_RX_CTX_BASE_SYMBOL_ID,

#endif /* AAL2_INCLUDED */
    
    _IX_CC_ATMRX_MAX_SYMBOL_ID
} _ix_cc_atmrx_symbol_id_t;


/* size of buffer for prefix name for symbol patched into microcode */
#define _IX_CC_ATMRX_SYMBOL_PREFIX_SIZE 20



#define _IX_CC_ATMRX_HASH_SEC_IDX_INVALID  0
#define _IX_CC_ATMRX_CID_HASH_SEC_IDX_INVALID 0
#define _IX_CC_ATMRX_ERROR_HASH_TIME_OUT 1

#define _IX_CC_ATMRX_VC_C64BITS_IDX_CELL_COUNT         0
#define _IX_CC_ATMRX_VC_C64BITS_IDX_MGT_CELL_COUNT     1
#define _IX_CC_ATMRX_VC_C64BITS_IDX_AAL5_PDU_COUNT     2
#define _IX_CC_ATMRX_VC_C64BITS_IDX_AAL5_PDU_CRC_COUNT 3
#define _IX_CC_ATMRX_VC_C64BITS_IDX_AAL5_PDU_LEN_COUNT 4


#define _IX_CC_ATMRX_PORT_C64BITS_IDX_AAL5_PKT_COUNT      0
#define _IX_CC_ATMRX_PORT_C64BITS_IDX_AAL5_PKT_DROP_COUNT 1
#define _IX_CC_ATMRX_PORT_C64BITS_IDX_AAL5_HW_ERROR       2
#define _IX_CC_ATMRX_PORT_C64BITS_IDX_AAL5_VC_ERROR       3


/**
 * TYPENAME: _ix_cc_atmrx_vc_key_t
 * 
 * DESCRIPTION:
 *   TBD
 */
typedef struct
{
    ix_uint32 key;
    ix_uint32 keyExt;
} _ix_cc_atmrx_vc_key_t;

/**
 * TYPENAME: _ix_cc_atmrx_hash_pos_t
 * 
 * DESCRIPTION:
 *   TBD
 */
typedef struct
{
#define _IX_CC_ATMRX_HASH_POS_PRI 1
#define _IX_CC_ATMRX_HASH_POS_SEC 2
    ix_uint8  tableId;     /**< it points on primary or secondary hash table */
    ix_uint8  key;         /**< key number in entry of table                 */
    ix_uint16 idx;         /**< index in primary or secondary hash table     */
} _ix_cc_atmrx_hash_pos_t;


#ifdef AAL2_INCLUDED

/**
 * TYPENAME: _ix_cc_atmrx_cid_entry_t
 * 
 * DESCRIPTION:
 *    This structure defines format for cid descriptor 
 */
typedef struct _ix_cc_atmrx_s_cid_entry
{
    ix_cc_atm_list_entry_t head;  /*  */
} _ix_cc_atmrx_cid_entry_t;

#endif /* AAL2_INCLUDED */


/**
 * TYPENAME: atmtrx_symbol_descr_t
 * 
 * DESCRIPTION:
 *    ME symbol description: channel number and memory type (local, scratch pad, sram)
 */
typedef struct _ix_cc_atmrx_s_symbol_descr
{
    ix_uint32 channel; /* channel number */
    ix_uint32 memType; /* memory type */
} _ix_cc_atmrx_symbol_descr_t;

/**
 * TYPENAME: _ix_cc_atmrx_context_t
 * 
 * DESCRIPTION:
 *    This structure defines format for L2 Bridge core component context
 */
typedef struct ix_s_cc_atmrx_context
{
    ix_cc_handle hAtmRxHandle;         /* Handle of L2 Bridge core component */
    ix_cc_atmsar_instance_t instance;  /* Instance ID of the core component */

    ix_counter_64bit_handle *hPortCounters; 
    ix_counter_64bit_handle *hVcCounters;
    
    ix_uint32 *pSymbolTable[_IX_CC_ATMRX_MAX_SYMBOL_ID];
    _ix_cc_atmrx_symbol_descr_t symbolDescr[_IX_CC_ATMRX_MAX_SYMBOL_ID];
    ix_uint16 freeSecondaryBucket; /* first free secondary bucket ID */
    ix_uint16 *vcPriIdx;

#ifdef AAL2_INCLUDED

    ix_counter_64bit_handle hCidCounters[MAX_AAL2_CHANNELS * ATMRX_CID_STAT_COUNTERS_NO];
    ix_cc_atm_list_descr_t listCid;           /* descriptor of CID list */
    _ix_cc_atmrx_cid_entry_t cid[MAX_AAL2_CHANNELS]; /* description of CID */
    ix_uint16 freeCidCtxs;                    /* first free secondary bucket ID */
    ix_uint32 cidPriIdx[MAX_AAL2_CHANNELS];
    ix_uint32 mev2Aal2Num; /* mask for microengines controled by the CC */
    
#endif /* AAL2_INCLUDED */
    
    /*
     * Configuration parameters read from registry or from configuration header
     */
    ix_uint32 mev2Num; /* mask for microengines controled by the CC */
    ix_uint32 counters; /* CC counters mapping enabled/disabled */
    
    char symbolPrefix[_IX_CC_ATMRX_SYMBOL_PREFIX_SIZE+1]; /* the prefix is added to patched symbols */

    ix_cc_atmsar_plugin_cfg_params_t pluginCfg;
} _ix_cc_atmrx_context_t;


/**
 * DESCRIPTION: This structure defines message codes for ATM RX  messages
 */
enum ix_e_cc_atmrx_messages
{

    _IX_CC_ATMRX_MSG_FIRST =IX_CC_COMMON_MSG_ID_LAST,

    /* Add new messages before this */
    _IX_CC_ATMRX_MSG_LAST
};


/******************************************************************************
 *                          GLOBAL VARIABLES
 *****************************************************************************/


/******************************************************************************
 *                    PRIMARY HASH TABLE SUPPORT FUNCTIONS
 *****************************************************************************/

/**
 * NAME: _ix_cc_atmrx_hash_pri_get_entry
 *
 * DESCRIPTION:
 *    Find pointer to entry in primary hash table. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN arg_entryId : Id of requested entry
 *
 * @Return: NULL if error occurs, otherwise pointer to secondary bucket.
 */
ix_uint32* _ix_cc_atmrx_hash_pri_get_entry(
_ix_cc_atmrx_context_t *arg_pContext,
ix_uint16 arg_entryId);


/**
 * NAME: _ix_cc_atmrx_hash_pri_init
 *
 * DESCRIPTION:
 *    Initialize primary hash table. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 *
 * @Return: none
 */
void _ix_cc_atmrx_hash_pri_init(
_ix_cc_atmrx_context_t *arg_pContext);


/******************************************************************************
 *                    SECONDARY HASH TABLE SUPPORT FUNCTIONS
 *****************************************************************************/

/**
 * NAME: _ix_cc_atmrx_hash_sec_get_entry
 *
 * DESCRIPTION:
 *    Find pointer to secondary bucket. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN arg_entryId : Id of requested bucket
 *
 * @Return: NULL if error occurs, otherwise pointer to secondary bucket.
 */
ix_uint32* _ix_cc_atmrx_hash_sec_get_entry(
_ix_cc_atmrx_context_t *arg_pContext,
ix_uint16 arg_entryId);


/**
 * NAME: _ix_cc_atmrx_hash_sec_free_entry
 *
 * DESCRIPTION:
 *    Put entry to list. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN arg_pEntry : pointer to entry
 *
 * @Return: none
 */
void _ix_cc_atmrx_hash_sec_free_entry(
_ix_cc_atmrx_context_t *arg_pContext,
ix_uint16 arg_entry);


/**
 * NAME: _ix_cc_atmrx_hash_sec_alloc_entry
 *
 * DESCRIPTION:
 *    Allocate secondary bucket for use.
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 *
 * @Return: ID of allocated entry. If no free entry function returns
 *          _IX_CC_ATMRX_HASH_SEC_IDX_INVALID.
 */
ix_uint16 _ix_cc_atmrx_hash_sec_alloc_entry(
_ix_cc_atmrx_context_t *arg_pContext);


/**
 * NAME: _ix_cc_atmrx_hash_sec_init
 *
 * DESCRIPTION:
 *    Initialize secondary hash table. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 *
 * @Return: none
 */
void _ix_cc_atmrx_hash_sec_init(
_ix_cc_atmrx_context_t *arg_pContext);


/******************************************************************************
 *                       HASH TABLE SUPPORT FUNCTIONS
 ******************************************************************************/

/**
 * NAME: _ix_cc_atmrx_hash_get_vc
 *
 * DESCRIPTION:
 *    Function gets VC key from Hash Table entry
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN  arg_pPos : pointer to structure describing location.
 * @Param:  OUT arg_pVcKey : structure containing VC key (VPI/VCI/port)
 * 
 * @Return: None
 */
void _ix_cc_atmrx_hash_get_vc(
_ix_cc_atmrx_context_t *arg_pContext,
const _ix_cc_atmrx_hash_pos_t* arg_pPos,
_ix_cc_atmrx_vc_key_t* arg_pVcKey);


/**
 * NAME: _ix_cc_atmrx_hash_search_vc_by_ptr
 *
 * DESCRIPTION:
 *    The function searchs position in hash mechanism that has key equal with
 *    requested. It checks primary and secondary hash structures.
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN  arg_pVcKey : pointer to structure describing vc key.
 * @Param:  IN  arg_priIdx : index of entry in primary hash.
 * @Param:  OUT arg_pPos:    position of founded VC.
 * 
 * @Return: 1 - key exists, 0 otherwise
 */
ix_uint8 _ix_cc_atmrx_hash_search_vc_by_ptr(
_ix_cc_atmrx_context_t *arg_pContext,
ix_uint16 arg_ptr,
ix_uint16 arg_priIdx,
_ix_cc_atmrx_hash_pos_t* arg_pPos);

    
/******************************************************************************
 *           FUNCTIONS FOR ATM SAR PLUGIN INFRASTRUCTURE
 *****************************************************************************/

/**
 * TYPENAME: _ix_cc_atmrx_plugin_vc_create
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "VC create" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_hVC :  handle to created VC.
 * @Param:  IN  arg_pVcDescr : parameters describing requested VC.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 */
ix_error _ix_cc_atmrx_plugin_vc_create(
void* arg_pInstance,
const ix_cc_atmsar_vc_handle_t arg_phVC,
const ix_cc_atmsar_vc_descr_t* arg_pVcDescr);

/**
 * TYPENAME: _ix_cc_atmrx_plugin_vc_update
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "VC update" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_hVC :  handle to created VC.
 * @Param:  IN  arg_pVcDescr : parameters describing requested VC.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 */
ix_error _ix_cc_atmrx_plugin_vc_update(
void* arg_pInstance,
const ix_cc_atmsar_vc_handle_t arg_phVC,
const ix_cc_atmsar_update_data_t* arg_pUpdate);

/**
 * TYPENAME: _ix_cc_atmrx_plugin_vc_remove
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "VC remove" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_hVc : handle to VC to be removed
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - vc successfully removed
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 */
ix_error _ix_cc_atmrx_plugin_vc_remove(
void* arg_pInstance,
ix_cc_atmsar_vc_handle_t arg_hVc);


/**
 * TYPENAME: _ix_cc_atmrx_plugin_get_vc_stats
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "GET VC STATS" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_hVc : handle to VC to be removed
 * @Param:  OUT arg_pLookupResult : pointer to statistic structure
 *                  where the requested data will be stored
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - statistics successfully retrieved
 *    IX_CC_INVALID_PORT - specified ATM port does not exist 
 */
ix_error _ix_cc_atmrx_plugin_get_vc_stats(
void* arg_pInstance,
ix_cc_atmsar_vc_handle_t arg_hVc,
ix_cc_atmsar_vc_stats_t *arg_pLookupResult);


/**
 * TYPENAME: _ix_cc_atmrx_plugin_port_create
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "PORT CREATE" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_pPortDescr : pointer to data describing parameters
 *                  for requested port
 * @Param:  OUT arg_hPort : ATM port handle
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 */
ix_error _ix_cc_atmrx_plugin_port_create(
void* arg_pInstance,
ix_cc_atmsar_port_handle_t arg_hPort,
const ix_cc_atmsar_port_descr_t* arg_pPortDescr);


/**
 * TYPENAME: _ix_cc_atmrx_plugin_port_remove
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "PORT REMOVE" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_port : ATM port to be removed
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - port successfully removed
 *    IX_CC_INVALID_PORT - specified port does not exist
 */
ix_error _ix_cc_atmrx_plugin_port_remove(
void* arg_pInstance,
ix_cc_atmsar_port_handle_t arg_port);


/**
 * TYPENAME: _ix_cc_atmrx_plugin_get_port_stats
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "GET PORT STATS" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_port : ATM port ID
 * @Param:  OUT arg_pLookupResult : pointer to ix_cc_sar_port_stats structure
 *                  where the requested data will be stored
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - statistics successfully retrieved
 *    IX_CC_INVALID_PORT - specified ATM port does not exist 
 */
ix_error _ix_cc_atmrx_plugin_get_port_stats(
void* arg_pInstance,
ix_cc_atmsar_port_handle_t arg_port,
ix_cc_atmsar_port_stats_t *arg_pLookupResult);


/******************************************************************************
 *                        AAL2 SUPPORT FUNCTIONS
 ******************************************************************************/


#ifdef AAL2_INCLUDED

/**
 * NAME: _ix_cc_atmrx_plugin_cid_alloc
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it wants to allocate CID for CID creation.
 *    The called function should allocate and return new CID handle.
 *    The 'arg_pCidDescr' agrument is passed to the function because
 *    parameters of CID could influent on assigned handle
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_pCidDescr : pointer to data describing parameters
 *                  for requested CID
 * @Param:  OUT arg_pCid : poiter to data where assigned handle is stored
 *                  CID handle
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 */
ix_error _ix_cc_atmrx_plugin_cid_alloc(
void* arg_pInstance,
ix_cc_atmsar_cid_handle_t *arg_pCid,
const ix_cc_atmsar_cid_descr_t *arg_pCidDescr);


/**
 * NAME: _ix_cc_atmrx_plugin_cid_free
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it wants to free CID for CID destroying.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_cid : handle of freeing CID.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 */
ix_error _ix_cc_atmrx_plugin_cid_free(
void* arg_pInstance,
ix_cc_atmsar_cid_handle_t arg_cid);


/**
 * TYPENAME: _ix_cc_atmrx_plugin_cid_create
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "CID CREATE" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_hCid : CID handle
 * @Param:  IN  arg_pCidDescr : pointer to data describing parameters
 *                  for requested CID
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - CID successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 */
ix_error _ix_cc_atmrx_plugin_cid_create(
void* arg_pInstance,
ix_cc_atmsar_cid_handle_t arg_hCid,
const ix_cc_atmsar_cid_descr_t* arg_pCidDescr);

/**
 * TYPENAME: _ix_cc_atmrx_plugin_cid_update
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "CID UPDATE" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_pCidDescr : pointer to data describing parameters
 *                  for requested CID
 * @Param:  OUT arg_hCid : CID handle
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - CID successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 */
ix_error
_ix_cc_atmrx_plugin_cid_update(
void *arg_pInstance,
ix_cc_atmsar_cid_handle_t arg_hCid,
const ix_cc_atmsar_update_data_t *arg_pUpdate);

/**
 * TYPENAME: _ix_cc_atmrx_plugin_cid_remove
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "CID REMOVE" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_cid : CID handle for removed CID
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - CID successfully removed
 */
ix_error _ix_cc_atmrx_plugin_cid_remove(
void* arg_pInstance,
ix_cc_atmsar_cid_handle_t arg_cid);


/**
 * TYPENAME: _ix_cc_atmrx_plugin_get_cid_stats
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "GET CID STATS" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_cid : CID handle
 * @Param:  OUT arg_pLookupResult : pointer to ix_cc_atmsar_cid_stats_t
 *                  structure where the requested data will be stored
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - statistics successfully retrieved
 *    IX_CC_INVALID_PORT - specified ATM port does not exist 
 */
ix_error _ix_cc_atmrx_plugin_get_cid_stats(
void* arg_pInstance,
ix_cc_atmsar_cid_handle_t arg_cid,
ix_cc_atmsar_cid_stats_t *arg_pLookupResult);


/******************************************************************************
 *                        VC LOOKUP TABLE SUPPORT
 ******************************************************************************/

/**
 * NAME: _ix_cc_atmrx_vc_lookup_get_entry
 *
 * DESCRIPTION:
 *    Find pointer to entry in vc lookup table. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN arg_entryId : Id of requested entry
 *
 * @Return: NULL if error occurs, otherwise pointer to secondary bucket.
 */
ix_uint32* _ix_cc_atmrx_vc_lookup_get_entry(
_ix_cc_atmrx_context_t* arg_pContext,
ix_uint16 arg_entryId);


/**
 * NAME: _ix_cc_atmrx_vc_lookup_init
 *
 * DESCRIPTION:
 *    Initialise VC lookup table. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 */
ix_error _ix_cc_atmrx_vc_lookup_init(
_ix_cc_atmrx_context_t* arg_pContext);


/******************************************************************************
 *                        CID CONTEXT TABLE SUPPORT
 ******************************************************************************/

/**
 * NAME: _ix_cc_atmrx_aal2_cid_rx_ctx_get_entry
 *
 * DESCRIPTION:
 *    Find pointer to entry in aal2 context table. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN  arg_entryId : Id of requested entry
 *
 * @Return: NULL if error occurs, otherwise pointer to aal2 cid context table entry.
 */
ix_uint32* _ix_cc_atmrx_aal2_cid_rx_ctx_get_entry(
_ix_cc_atmrx_context_t* arg_pContext,
ix_uint16 arg_entryId);


/**
 * NAME: _ix_cc_atmrx_aal2_cid_rx_ctx_init
 *
 * DESCRIPTION:
 *    Initialise AAL2 CID Context table. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 */
ix_error _ix_cc_atmrx_aal2_cid_rx_ctx_init(
_ix_cc_atmrx_context_t* arg_pContext);


/******************************************************************************
 *                   PRIMARY CID HASH TABLE SUPPORT FUNCTIONS
 ******************************************************************************/

/**
 * NAME: _ix_cc_atmrx_cid_hash_pri_get_entry
 *
 * DESCRIPTION:
 *    Find pointer to entry in CID primary hash table. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN  arg_entryId : Id of requested entry
 *
 * @Return: NULL if error occurs, otherwise pointer to secondary bucket.
 */
ix_uint32* _ix_cc_atmrx_cid_hash_pri_get_entry(
_ix_cc_atmrx_context_t* arg_pContext,
ix_uint16 arg_entryId);


/**
 * NAME: _ix_cc_atmrx_cid_hash_pri_init
 *
 * DESCRIPTION:
 *    Initialize primary CID hash table. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 *
 * @Return: none
 */
void _ix_cc_atmrx_cid_hash_pri_init(
_ix_cc_atmrx_context_t* arg_pContext);


/******************************************************************************
 *                  SECONDARY CID HASH TABLE SUPPORT FUNCTIONS
 ******************************************************************************/

/**
 * NAME: _ix_cc_atmrx_cid_hash_sec_get_entry
 *
 * DESCRIPTION:
 *    Find pointer to entry in CID secondary hash table. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN arg_entryId : Id of requested entry
 *
 * @Return: NULL if error occurs, otherwise pointer to secondary bucket.
 */
ix_uint32* _ix_cc_atmrx_cid_hash_sec_get_entry(
_ix_cc_atmrx_context_t* arg_pContext,
ix_uint16 arg_entryId);


/**
 * NAME: _ix_cc_atmrx_cid_hash_sec_free_entry
 *
 * DESCRIPTION:
 *    Put bucket entry to free list.
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN  arg_pEntry : pointer to entry
 *
 * @Return: none
 */
void _ix_cc_atmrx_cid_hash_sec_free_entry(
_ix_cc_atmrx_context_t* arg_pContext,
ix_uint16 arg_entry);


/**
 * NAME: _ix_cc_atmrx_cid_hash_sec_alloc_entry
 *
 * DESCRIPTION:
 *    Allocate CID secondary bucket for use.
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 *
 * @Return: ID of allocated entry. If no free entry function returns
 *          _IX_CC_ATMRX_CID_HASH_SEC_IDX_INVALID.
 */
ix_uint16 _ix_cc_atmrx_cid_hash_sec_alloc_entry(
_ix_cc_atmrx_context_t* arg_pContext);


/**
 * NAME: _ix_cc_atmrx_cid_hash_sec_init
 *
 * DESCRIPTION:
 *    Initialize secondary CID hash table. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 *
 * @Return: none
 */
void _ix_cc_atmrx_cid_hash_sec_init(
_ix_cc_atmrx_context_t* arg_pContext);

#endif /* AAL2_INCLUDED */


/******************************************************************************
 *                    VC INFO TABLE SUPPORT FUNCTIONS
 *****************************************************************************/


/**
 * NAME: _ix_cc_atmrx_vc_info_init
 *
 * DESCRIPTION:
 *    Initialise VC info table. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - ATM port successfully inserted
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to initialize
 */
ix_error _ix_cc_atmrx_vc_info_init(
_ix_cc_atmrx_context_t* arg_pContext);


/******************************************************************************
 *                  SUPPORT FOR DEBBUGING AND TRACES
 *****************************************************************************/

extern ix_uint32 g_atmrxDebugMask;

#define _IX_CC_ATMRX_DBG(__mask, __debugInfo) \
    if((__mask & g_atmrxDebugMask)!= 0) ix_ossl_message_log __debugInfo;

#define _IX_CC_ATMRX_DBG_INIT(__debugInfo) _IX_CC_ATMRX_DBG(0x01, __debugInfo)
#define _IX_CC_ATMRX_DBG_VC(__debugInfo) _IX_CC_ATMRX_DBG(0x02, __debugInfo)
#define _IX_CC_ATMRX_DBG_PORT(__debugInfo) _IX_CC_ATMRX_DBG(0x04, __debugInfo)
#define _IX_CC_ATMRX_DBG_CID(__debugInfo) _IX_CC_ATMRX_DBG(0x08, __debugInfo)
#define _IX_CC_ATMRX_DBG_HASH(__debugInfo) _IX_CC_ATMRX_DBG(0x10, __debugInfo)

/******************************************************************************
 *              SUPPORT FOR REGISTRY AND/OR STATIC CONFIGURATION
 *****************************************************************************/


#ifdef IX_INCLUDE_REGISTRY

#define _IX_CC_ATMRX_PROP_MEV2_NUM_MASK      "AAL5_RX_MEV2_NUM_MASK"
#define _IX_CC_ATMRX_PROP_SRAM_CHANNEL       "AAL5_SRAM_CHANNEL"
#define _IX_CC_ATMRX_PROP_SYMBOL_PREFIX      "SYMBOL_PREFIX"
#define _IX_CC_ATMRX_PROP_COUNTERS           "AAL5_RX_COUNTERS"
#define _IX_CC_ATMRX_AAL2_PROP_MEV2_NUM_MASK "AAL2_RX_MEV2_NUM_MASK"
#define _IX_CC_ATMRX_AAL2_PROP_SRAM_CHANNEL  "AAL2_SRAM_CHANNEL"

#else /* IX_INCLUDE_REGISTRY */

#define _IX_CC_ATMRX_DEFAULT_CHANNEL(instance)   0x0
#define _IX_CC_ATMRX_DEFAULT_MEMTYPE(instance)   IX_MEMORY_TYPE_SRAM

#define _IX_CC_ATMRX_MEV2_NUM_MASK(instance)       0x00000001 
#define _IX_CC_ATMRX_SYMBOL_PREFIX(instance)       ""
#define _IX_CC_ATMRX_AAL2_MEV2_NUM_MASK(instance)  0x00000002

#endif /* IX_INCLUDE_REGISTRY */


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_ATMRX_PRIVATE_H__) */


