/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      ATM RX Core Component - CCI - init function
 *
 * = FILENAME
 *      ix_cc_atmrx_init.c
 *
 * = DESCRIPTION
 *      The file defines for init interface and internal functions.
 *
 * = AUTHOR
 *       Jaroslaw Kogut
 *       jaroslaw.kogut@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      11/07/2002
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */
#define IX_ERROR_FILE_IDENT "$Id: ix_cc_atmrx_init.c,v 1.5 2003/12/12 19:14:02 adziecia Exp $";



/**
 * System defined include files
 */
#include "ix_cc_error.h"
#include "ix_rm.h"
#include "ix_ossl.h"
#include "ix_cc.h"
#include "ix_cc_macros.h"
#include "ix_config.h"
#include "bindings.h"
#include "cc/ix_cc_atmsar.h"
#include "cc/ix_cc_atmsar_plugin.h"

/**
 * User defined include files
 */
#include "cc/ix_cc_atmrx_cci.h"
#include "cc/internal/ix_cc_atmrx_private.h"

/**
 * Preprocessor symbols and Macros used in this file
 */
#define _IX_CC_ATMRX_AAL5_SYMBOL 0x1
#define _IX_CC_ATMRX_AAL2_SYMBOL 0x2

/* TBD. Temporary defined SAR instace mechanism */
#define _IX_CC_ATMRX_SAR_INSANCE_0 0


/**
 * Type definitions whose scope is limited to this file
 */

/* This data structure describes the SRAM memory to be allocated, initialized
 * and patched into corresponding symbol */
typedef struct ix_s_atmrx_symbol_table
{
    ix_uint8     *registryName;
    ix_uint8     *symbol;
    ix_uint32    elementSize;
    ix_int8      symbolType;
    ix_uint8     initValue;
    _ix_cc_atmrx_symbol_id_t symbolId;
    ix_uint32    valueToPatch;
    ix_uint32    totalSize;
}_ix_atmrx_symbol_table_t;

/**
 * Variable declarations global to this file only. Externs are followed by 
 * static variables.
 */


/**
 * VARNAME: g_atmrxDebugMask
 *
 * DESCRIPTION:
 *    The global variable contains mask of bits that define what debugging
 *    traces has to be show.
 */
ix_uint32 g_atmrxDebugMask=0x0FFFFFFFF;


static _ix_atmrx_symbol_table_t s_atmrxSymbolTable[] =
{        									 
    {
        "RX_PORT_STAT",
        "AAL5_RX_PORT_STAT_SRAM_BASE",
        AAL5_RX_PORT_STAT_ENTRY_SIZE,
        _IX_CC_ATMRX_AAL5_PORT_SYMBOL_TYPE,
        0x0,
        _IX_CC_ATMRX_AAL5_RX_PORT_STAT_SRAM_BASE_SYMBOL_ID
    },          								 
    {
        "VC_INFO",
        "AAL5_RX_VC_INFO_SRAM_BASE",
        AAL5_RX_VC_INFO_ENTRY_SIZE,
        _IX_CC_ATMRX_AAL5_VC_SYMBOL_TYPE,
        0x0,
        _IX_CC_ATMRX_AAL5_RX_VC_INFO_SRAM_BASE_SYMBOL_ID
    },          								 
    {
        "VC_HASH_PRI",
        "AAL5_RX_VC_HASH_PRI_SRAM_BASE",
        AAL5_RX_VC_HASH_PRI_BKT_ENTRY_SIZE,
        _IX_CC_ATMRX_AAL5_VC_SYMBOL_TYPE,
        0x0,
        _IX_CC_ATMRX_VC_HASH_PRI_BASE_SRAM_SYMBOL_ID
    },          								 
    {
        "VC_HASH_SEC",
        "AAL5_RX_VC_HASH_SEC_SRAM_BASE",
        AAL5_RX_VC_HASH_SEC_BKT_ENTRY_SIZE,
        _IX_CC_ATMRX_AAL5_VC_SYMBOL_TYPE,
        0x0,
        _IX_CC_ATMRX_VC_HASH_SEC_BASE_SRAM_SYMBOL_ID
    },          								 
    {   /* Terminator */          						 
        NULL,
        0,
        0,
        0
    },
};       


#ifdef AAL2_INCLUDED

static _ix_atmrx_symbol_table_t s_atmrxSymbolTableAal2[] =
{
#ifdef VC_LOOKUP_IS_DEFINED
    {
        "VC_LOOKUP_TABLE_BASE",
        "VC_LOOKUP_TABLE_BASE",
        VC_LOOKUP_TABLE_ENTRY_SIZE,
        _IX_CC_ATMRX_AAL5_VC_SYMBOL_TYPE,
        0x0,
        _IX_CC_ATMRX_VC_LOOKUP_TABLE_BASE_SYMBOL_ID
    },
#endif
    {
        "AAL2_VC_RX_CTX_BASE",
        "AAL2_VC_RX_CTX_BASE",
        AAL2_VC_RX_CTX_ENTRY_SIZE,
        _IX_CC_ATMRX_AAL5_VC_SYMBOL_TYPE,
        0x0,
        _IX_CC_ATMRX_AAL2_VC_RX_CTX_BASE_SYMBOL_ID
    },
    {
        "AAL2_CID_HASH_PRIM",
        "AAL2_CID_HASH_PRIM",
        AAL2_CID_HASH_PRIM_BKT_SZ_BYTES,
        _IX_CC_ATMRX_AAL2_CID_SYMBOL_TYPE,
        0x0,
        _IX_CC_ATMRX_AAL2_CID_HASH_PRIM_SYMBOL_ID
    },          								 
    {
        "AAL2_CID_HASH_SEC",
        "AAL2_CID_HASH_SEC",
        AAL2_CID_HASH_SEC_BKT_SZ_BYTES,
        _IX_CC_ATMRX_AAL2_CID_SYMBOL_TYPE,
        0x0,
        _IX_CC_ATMRX_AAL2_CID_HASH_SEC_SYMBOL_ID
    },          								 
    {
        "AAL2_CID_RX_CTX_BASE",
        "AAL2_CID_RX_CTX_BASE",
        AAL2_CID_RX_CTX_ENTRY_SIZE,
        _IX_CC_ATMRX_AAL2_CID_SYMBOL_TYPE,
        0x0,
        _IX_CC_ATMRX_AAL2_CID_RX_CTX_BASE_SYMBOL_ID
    },          								 
    {   /* Terminator */          						 
        NULL,
        0,
        0,
        0
    },
};       

#endif /* AAL2_INCLUDED */


/** 
 * Extern function prototypes.
 */


/** 
 * Static function prototypes.
 */


/** 
 * Function definitions.
 */


/******************************************************************************
 *                            SHUTDOWN FUNCTIONS
 *****************************************************************************/

/**
 * NAME: _ix_cc_atmrx_free_symbols
 *
 * DESCRIPTION:
 *    This primitive will be used to free memory for symbols  allocated for
 *    patching.
 * 
 * @Param:  - IN arg_pContext - pointer to context of current core component.
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ERROR_OOM - failed to free memory for symbols.
 */
static ix_error
_ix_cc_atmrx_free_symbols(
_ix_cc_atmrx_context_t* arg_pContext)
{
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);
    /* Free memory for microblock symbols */
    _IX_CC_ATMRX_DBG_INIT(("_ix_cc_atmrx_free_symbols() TBD. Functionality is not implemented yet\n"));
    return IX_SUCCESS;
}/* end _ix_cc_atmrx_free_symbols */


/**
 * NAME: _ix_cc_atmrx_delete_64bit_counters
 *
 * DESCRIPTION:
 *    This primitive will be used to delete 64 bit counters. 
 * 
 * @Param:  - IN arg_pContext - pointer to context of current core component.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ERROR_OOM_64BIT_COUNTER - failed to delete 64 bit counter.
 */
static ix_error
_ix_cc_atmrx_delete_64bit_counters(
_ix_cc_atmrx_context_t* arg_pContext)
{
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);
    _IX_CC_ATMRX_DBG_INIT(("_ix_cc_atmrx_delete_64bit_counters() TBD. Functionality is not implemented yet\n"));
    return IX_SUCCESS;
}/* end _ix_cc_atmrx_delete_64bit_counters */


/**
 * NAME: _ix_cc_atmrx_unregister_message_handler
 *
 * DESCRIPTION:
 *    This primitive will be used to unregister message handler. It calls CCI
 *    services in level 1 system and resore manager services in level 0 system.
 *
 * @Param:  - IN arg_pContext - pointer to context of current core component.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMRX_ERROR_CCI - error from CCI
 */
static ix_error
_ix_cc_atmrx_unregister_message_handler(
_ix_cc_atmrx_context_t* arg_pContext)
{
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);
    /* Call CCI if CCI is enabled */
#if defined(IX_EXCLUDE_CCI)
    IX_ERROR_CRT(ix_rm_message_handler_unregister(IX_CC_ATMRX_MSG_INPUT),
                 IX_CC_ATMRX_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);
#else
    IX_ERROR_CRT(ix_cci_cc_remove_message_handler(IX_CC_ATMRX_MSG_INPUT),
                 IX_CC_ATMRX_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);
#endif
    return IX_SUCCESS;
}/* end _ix_cc_atmrx_unregister_message_handler */


/**
 * NAME: _ix_cc_atmrx_unregister_cci_packet_handlers
 *
 * DESCRIPTION:
 *    This primitive will be used to unregister packet handler with CCI. 
 * 
 * @Param:  - IN arg_pContext - pointer to context of current core component.
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMRX_ERROR_CCI - error from CCI
 */
static ix_error
_ix_cc_atmrx_unregister_cci_packet_handlers(
_ix_cc_atmrx_context_t* arg_pContext)
{
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);
#if !defined(IX_EXCLUDE_CCI)
    IX_ERROR_CRT(ix_cci_cc_remove_message_handler(IX_CC_ATMRX_PKT_INPUT),
                 IX_CC_ATMRX_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);
#endif
    return IX_SUCCESS;
}/* end _ix_cc_atmrx_unregister_cci_packet_handlers */


/**
 * NAME: _ix_cc_atmrx_unregister_rm_packet_handlers
 *
 * DESCRIPTION:
 *    This primitive will be used to unregister packet handler with 
 *    resource manager
 * 
 * @Param:  - IN arg_pContext - pointer to context of current core component.
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMRX_ERROR_CCI - error from RM
 */
#if defined(IX_EXCLUDE_CCI)
static ix_error
_ix_cc_atmrx_unregister_rm_packet_handlers(
_ix_cc_atmrx_context_t* arg_pContext)
{
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);
    IX_ERROR_CRT(ix_rm_message_handler_unregister(IX_CC_ATMRX_PKT_INPUT),
                 IX_CC_ATMRX_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);
    return IX_SUCCESS;
}/* end _ix_cc_atmrx_unregister_rm_packet_handlers */
#endif


/**
 * NAME: _ix_cc_atmrx_unregister_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to unregister packet handler 
 * with CCI/Resource manager
 *
 * @Param:  - IN arg_pContext - pointer to context of current core component.
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMRX_ERROR_CCI - error from CCI/RM
 */
static ix_error
_ix_cc_atmrx_unregister_packet_handlers(
_ix_cc_atmrx_context_t* arg_pContext)
{
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);
    /* Call CCI if CCI is enabled */
#if defined(IX_EXCLUDE_CCI)
    IX_ERROR_CR(_ix_cc_atmrx_unregister_rm_packet_handlers(arg_pContext));
#else
    IX_ERROR_CR(_ix_cc_atmrx_unregister_cci_packet_handlers(arg_pContext));
#endif
    return IX_SUCCESS;
}/* end _ix_cc_atmrx_unregister_packet_handlers */


/******************************************************************************
 *                            INITIALIZATION FUNCTIONS
 *****************************************************************************/


/**
 * NAME: _ix_cc_atmrx_property_name_set
 *
 * DESCRIPTION:
 *    This primitive builds full path to variable. It adds configuration
 *    section prefix and adds instance to prefix.
 *           .
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  OUT arg_pBuf : destination buffer
 * @Param:  IN  arg_pBufSize : size of destination buffer
 * @Param:  IN  arg_pPropertyName : buffer with property Name
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMTX_ERROR_REGISTRY  - not able to access registry.
 */
static ix_error
_ix_cc_atmrx_property_name_set(
_ix_cc_atmrx_context_t* arg_pContext,
char *arg_pBuf,
ix_uint16 arg_pBufSize,
const char *arg_pProperty,
uint8_t aalType)
{
    char *pSection="ATM_RX";

    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);
    IX_ERROR_DCHECK_ARG_NULL(2, arg_pBuf);
    IX_ERROR_DCHECK_ARG_NULL(4, arg_pProperty);
    IX_ERROR_DCHECK_ARG_MAX(4, (strlen(pSection) + strlen(arg_pProperty) + 6),
                            arg_pBufSize, "%d");

    sprintf(arg_pBuf, "%s_%s%d%c%s",
            pSection,
            ((aalType == _IX_CC_ATMRX_AAL5_SYMBOL) ? "" : "AAL2_"),
            arg_pContext->instance,
            IX_CP_PATH_NAME_DELIMITER,
            arg_pProperty);

    return IX_SUCCESS;
} /* _ix_cc_atmrx_property_name_set */


/**
 * NAME: _ix_cc_atmrx_registry_symbols
 *
 * DESCRIPTION:
 *    This primitive will be used to retreive data from registry for the
 *    core component static data describing symbols to patch.
 *           .
 * @Param:  - IN arg_pContext - pointer to context of current core component.
 * @Param:  - IN arg_pSymbTab - pointer to table with symbols description
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMRX_ERROR_REGISTRY - any internal error during registry accessing.
 *    another error codes from ix_rm_cp_property submodule
 */
static ix_error
_ix_cc_atmrx_registry_symbols(
_ix_cc_atmrx_context_t *arg_pContext,
_ix_atmrx_symbol_table_t *arg_pSymbTab,
uint8_t aalType)
{
    _ix_cc_atmrx_symbol_id_t symbolId;
    ix_int32 i;

#ifdef IX_INCLUDE_REGISTRY

    char name[IX_CP_MAX_PROPERTY_NAME_LENGTH*2];
    ix_configuration_property_handle hCoreRoot = IX_CP_CORE_PROPERTY_ROOT;
    ix_configuration_property_handle hProperty;

    for (i=0; arg_pSymbTab[i].symbol != NULL; i++)
    {
        symbolId=arg_pSymbTab[i].symbolId;
        sprintf(name,"ATM_RX_%s%d%c%s%cCHANNEL",
                ((aalType == _IX_CC_ATMRX_AAL5_SYMBOL) ? "" : "AAL2_"),
                arg_pContext->instance,
                IX_CP_PATH_NAME_DELIMITER,
                arg_pSymbTab[i].registryName,
                IX_CP_PATH_NAME_DELIMITER);

        IX_ERROR_CR(ix_rm_cp_property_open(hCoreRoot, name, &hProperty));
        
        IX_ERROR_CR(ix_rm_cp_property_get_value_uint32(hProperty, &(arg_pContext->symbolDescr[symbolId].channel)));

        _IX_CC_ATMRX_DBG_INIT(("SYMBOL: %s CHANNEL: %ld\n",
                               arg_pSymbTab[i].symbol,
                               arg_pContext->symbolDescr[symbolId].channel));
        
        sprintf(name,"ATM_RX_%s%d%c%s%cMEM_TYPE",
                ((aalType == _IX_CC_ATMRX_AAL5_SYMBOL) ? "" : "AAL2_"),
                arg_pContext->instance,
                IX_CP_PATH_NAME_DELIMITER,
                arg_pSymbTab[i].registryName,
                IX_CP_PATH_NAME_DELIMITER);

        IX_ERROR_CR(ix_rm_cp_property_open(hCoreRoot, name, &hProperty));
        
        IX_ERROR_CR(ix_rm_cp_property_get_value_uint32(hProperty, &(arg_pContext->symbolDescr[symbolId].memType)));

        _IX_CC_ATMRX_DBG_INIT(("SYMBOL: %s MEMTYPE: %ld\n",
                               arg_pSymbTab[i].symbol,
                               arg_pContext->symbolDescr[symbolId].memType));        

        switch(arg_pSymbTab[i].symbolType)
        {        
             case _IX_CC_ATMRX_AAL5_PORT_SYMBOL_TYPE:
                arg_pSymbTab[i].totalSize = arg_pSymbTab[i].elementSize * arg_pContext->pluginCfg.numberOfPorts;
		break;
             case _IX_CC_ATMRX_AAL5_VC_SYMBOL_TYPE:
                arg_pSymbTab[i].totalSize = arg_pSymbTab[i].elementSize * arg_pContext->pluginCfg.numberOfVcs;
		break;
#ifdef AAL2_INCLUDED
             case _IX_CC_ATMRX_AAL2_CID_SYMBOL_TYPE:
                arg_pSymbTab[i].totalSize = arg_pSymbTab[i].elementSize * arg_pContext->pluginCfg.numberOfCids;
		break;
#endif
             default:
                IX_ERROR_GLOBAL(IX_CC_ATMRX_ERROR_REGISTRY,
                               ("Symbol type (%d) is invalid for %s.\n",
                               arg_pSymbTab[i].symbol));
		break;
        }
        
    }

#else /* IX_INCLUDE_REGISTRY */

    for (i=0; arg_pSymbTab[i].symbol != NULL; i++)
    {
        symbolId = arg_pSymbTab[i].symbolId;
        arg_pContext->symbolDescr[symbolId].channel
            = ATM_RX_DEFAULT_CHANNEL(arg_pContext->instance);
        arg_pContext->symbolDescr[symbolId].memType
            = ATM_RX_DEFAULT_MEMTYPE(arg_pContext->instance);        
    }

#endif /* IX_INCLUDE_REGISTRY */
    return IX_SUCCESS;
} /* end _ix_cc_atmrx_registry_symbols */


/**
 * NAME: _ix_cc_atmrx_registry
 *
 * DESCRIPTION:
 *    This primitive will be used to retreive data from registry for all the
 *    core component static data.
 *           .
 * @Param:  - IN arg_pContext - pointer to context of current core component.
 * @Param:  - OUT  none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMRX_ERROR_REGISTRY - any internal error during registry accessing.
 *    another error codes from ix_rm_cp_property submodule
 */
static ix_error
_ix_cc_atmrx_registry(
_ix_cc_atmrx_context_t* arg_pContext)
{
#ifdef IX_INCLUDE_REGISTRY

    char name[IX_CP_MAX_PROPERTY_NAME_LENGTH*2];
    ix_configuration_property_handle hCoreRoot = IX_CP_CORE_PROPERTY_ROOT;
    ix_configuration_property_handle hProperty;
    ix_cp_property_info propertyInfo;

    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);

    /* property AAL5_RX_MEV2_NUM_MASK */
    IX_ERROR_CR(_ix_cc_atmrx_property_name_set(arg_pContext, name,
                                               sizeof(name),
                                               _IX_CC_ATMRX_PROP_MEV2_NUM_MASK,
                                               _IX_CC_ATMRX_AAL5_SYMBOL));
    IX_ERROR_CR(ix_rm_cp_property_open(hCoreRoot, name, &hProperty));
    IX_ERROR_CR(ix_rm_cp_property_get_value_uint32(hProperty, &(arg_pContext->mev2Num)));

#ifdef AAL2_INCLUDED    
    /* property AAL5_RX_MEV2_NUM_MASK */
    IX_ERROR_CR(_ix_cc_atmrx_property_name_set(arg_pContext, name, sizeof(name),
                                               _IX_CC_ATMRX_AAL2_PROP_MEV2_NUM_MASK,
                                               _IX_CC_ATMRX_AAL2_SYMBOL));
    IX_ERROR_CR(ix_rm_cp_property_open(hCoreRoot, name, &hProperty));
    IX_ERROR_CR(ix_rm_cp_property_get_value_uint32(hProperty, &(arg_pContext->mev2Aal2Num)));
#endif    
    /* property SYMBOL_PREFIX */
    IX_ERROR_CR(_ix_cc_atmrx_property_name_set(arg_pContext, name, sizeof(name),
                                               _IX_CC_ATMRX_PROP_SYMBOL_PREFIX,
                                               _IX_CC_ATMRX_AAL5_SYMBOL));
    IX_ERROR_CR(ix_rm_cp_property_open(hCoreRoot, name, &hProperty));
    IX_ERROR_CR(ix_rm_cp_property_get_info(hProperty, &propertyInfo));
    if (propertyInfo.m_DataSize > _IX_CC_ATMRX_SYMBOL_PREFIX_SIZE)
    {
        return IX_ERROR_GLOBAL(IX_CC_ATMRX_ERROR_REGISTRY,
                               ("The prefix symbol size (%d) is too long.\n",
                                propertyInfo.m_DataSize));
    }
    IX_ERROR_CR(ix_rm_cp_property_get_value(hProperty, &propertyInfo.m_DataSize, 
                                            arg_pContext->symbolPrefix));


    /* property AAL5_RX_COUNTERS */
    IX_ERROR_CR(_ix_cc_atmrx_property_name_set(arg_pContext, name, sizeof(name),
                                               _IX_CC_ATMRX_PROP_COUNTERS,
                                               _IX_CC_ATMRX_AAL5_SYMBOL));
    IX_ERROR_CR(ix_rm_cp_property_open(hCoreRoot, name, &hProperty));
    IX_ERROR_CR(ix_rm_cp_property_get_info(hProperty, &propertyInfo));
    if (propertyInfo.m_DataSize > _IX_CC_ATMRX_SYMBOL_PREFIX_SIZE)
    {
        return IX_ERROR_GLOBAL(IX_CC_ATMRX_ERROR_REGISTRY,
                               ("The prefix symbol size (%d) is too long.\n",
                                propertyInfo.m_DataSize));
    }
    IX_ERROR_CR(ix_rm_cp_property_get_value_uint32(hProperty, &(arg_pContext->counters)));
    
#else /* IX_INCLUDE_REGISTRY */

    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);

#ifdef AAL2_INCLUDED
    arg_pContext->mev2Aal2Num = _IX_CC_ATMRX_AAL2_MEV2_NUM_MASK(arg_pContext->instance);
#endif
    arg_pContext->mev2Num = _IX_CC_ATMRX_MEV2_NUM_MASK(arg_pContext->instance);
    strcpy(arg_pContext->symbolPrefix, _IX_CC_ATMRX_SYMBOL_PREFIX(arg_pContext->instance));

#endif /* IX_INCLUDE_REGISTRY */

    /* read SAR general configuration from registry */
    IX_ERROR_CR(ix_cc_atmsar_plugin_get_cfg_params(arg_pContext->instance,
                                                   &(arg_pContext->pluginCfg)));

    IX_ERROR_CR(_ix_cc_atmrx_registry_symbols(arg_pContext,
                                              s_atmrxSymbolTable,
                                              _IX_CC_ATMRX_AAL5_SYMBOL));

#ifdef AAL2_INCLUDED
    if (arg_pContext->pluginCfg.aal2Included == IX_CC_ATMSAR_PLUGIN_AAL2_INLUDED_YES)
    {
        IX_ERROR_CR(_ix_cc_atmrx_registry_symbols(arg_pContext,
                                                  s_atmrxSymbolTableAal2,
                                                  _IX_CC_ATMRX_AAL2_SYMBOL));
    }
    _IX_CC_ATMRX_DBG_INIT(("_ix_cc_atmrx_registry(): Instance:%d  ME mask: %x Aal2 ME mask %x\n",
                           arg_pContext->instance,
                           arg_pContext->mev2Num,
                           arg_pContext->mev2Aal2Num));
#else    
    _IX_CC_ATMRX_DBG_INIT(("_ix_cc_atmrx_registry(): Instance:%d  ME mask: %lx\n",
                           arg_pContext->instance, arg_pContext->mev2Num));
#endif /* AAL2_INCLUDED */

    return IX_SUCCESS;
}/* end _ix_cc_atmrx_registry */


/**
 * NAME: _ix_cc_atmrx_alloc_symbols_table
 *
 * DESCRIPTION:
 *    This primitive will be used to allocate memory for symbols defined in
 *    table to be patched.
 * 
 * @Param:  - IN arg_pContext - pointer to context of current core component.
 * @Param:  - IN arg_pSymbTab - pointer to table with symbols description
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ERROR_OOM  - not able to allocate memory
 */
static ix_error
_ix_cc_atmrx_alloc_symbols_table(
_ix_cc_atmrx_context_t* arg_pContext,
_ix_atmrx_symbol_table_t *arg_pSymbTab)
{
    ix_error  err = IX_SUCCESS;
    ix_uint32 i;
    ix_uint32 **pTable;
    _ix_cc_atmrx_symbol_id_t symbolId;
    ix_int32 memSize;

    /* Allocate context tables and structures. */

    /* port statistics*/
    memSize = (NO_PER_PORT_STAT_COUNTERS * arg_pContext->pluginCfg.numberOfPorts) * sizeof(ix_counter_64bit_handle);
    arg_pContext->hPortCounters = (ix_counter_64bit_handle*) ix_ossl_malloc(memSize);
    if(arg_pContext->hPortCounters == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM_64BIT_COUNTER,
                                ("Memory allocation failed for AAL5 RX port stat table"));
    }    
    ix_ossl_memset(arg_pContext->hPortCounters,0,memSize);

    /* vc statistics  */
    memSize = (ATMRX_VC_STAT_COUNTERS_NO * arg_pContext->pluginCfg.numberOfVcs) * sizeof(ix_counter_64bit_handle);
    arg_pContext->hVcCounters = (ix_counter_64bit_handle*) ix_ossl_malloc(memSize);
    if(arg_pContext->hVcCounters == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM_64BIT_COUNTER,
                                ("Memory allocation failed for AAL5 RX VC stat table"));
    }    
    ix_ossl_memset(arg_pContext->hVcCounters,0,memSize);

    /* pri idx table  */
    memSize = (arg_pContext->pluginCfg.numberOfVcs) * sizeof(ix_uint16);
    arg_pContext->vcPriIdx = (ix_int16*) ix_ossl_malloc(memSize);
    if(arg_pContext->vcPriIdx == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM_64BIT_COUNTER,
                                ("Memory allocation failed for VC primary idx table"));
    }    
    ix_ossl_memset(arg_pContext->vcPriIdx,0,memSize);

    /* Allocate SRAM memory to be used by the microblock */
    for (i=0; arg_pSymbTab[i].symbol != NULL; i++)
    {
        symbolId=arg_pSymbTab[i].symbolId;
        pTable = &arg_pContext->pSymbolTable[symbolId];
        /* Allocate memory for a table */   
        err = ix_rm_mem_alloc(arg_pContext->symbolDescr[symbolId].memType,
                              arg_pContext->symbolDescr[symbolId].channel,
                              arg_pSymbTab[i].totalSize,
                              (void **)pTable);
        if ( err )
        {
            err=IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                 ("Unable to allocate %d memory for symbol %s \n",
                                  arg_pSymbTab[i].totalSize,
                                  arg_pSymbTab[i].symbol));
            return err;
        }
        
        /* Initialize table  */
        ix_ossl_memset((ix_uint8 *)(*pTable),
                       (ix_uint8)arg_pSymbTab[i].initValue,
                       (ix_uint32)arg_pSymbTab[i].totalSize);
        
        _IX_CC_ATMRX_DBG_INIT(("Successfuly allocated %ld bytes for symbol %s\n",
                               arg_pSymbTab[i].totalSize, arg_pSymbTab[i].symbol));
    }
    return IX_SUCCESS;
} /* end _ix_cc_atmrx_alloc_symbols_table */


/**
 * NAME: _ix_cc_atmrx_alloc_symbols
 *
 * DESCRIPTION:
 *    This primitive will be used to allocate memory for symbols to be patched.
 * 
 * @Param:  - IN arg_pContext - pointer to context of current core component.
 * @Param:  - OUT  - none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ERROR_OOM  - not able to allocate memory
 */
static ix_error
_ix_cc_atmrx_alloc_symbols(
_ix_cc_atmrx_context_t* arg_pContext)
{
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);

    /* Allocate SRAM memory to be used by the microblock */
    IX_ERROR_CR(_ix_cc_atmrx_alloc_symbols_table(arg_pContext, s_atmrxSymbolTable));
    
    /* Additional initialization: */

    /* Initialize AAL5_RX_VC_INFO_SRAM_BASE table : */
    _ix_cc_atmrx_vc_info_init(arg_pContext);
    _IX_CC_ATMRX_DBG_INIT(("Successfuly init memory for AAL5_RX_VC_INFO_SRAM_BASE\n" ));

    /* Initialize primary hash table  */
    _ix_cc_atmrx_hash_pri_init(arg_pContext);
    _IX_CC_ATMRX_DBG_INIT(("Successfuly init memory for primary hash table\n" ));

    /* Initialize secondary hash table */
    _ix_cc_atmrx_hash_sec_init(arg_pContext);
    _IX_CC_ATMRX_DBG_INIT(("Successfuly init memory for secondary hash table\n" ));

#ifdef AAL2_INCLUDED

    /* AAL2 Additional initialization: */

    if (arg_pContext->pluginCfg.aal2Included == IX_CC_ATMSAR_PLUGIN_AAL2_INLUDED_YES)
    {
        /* Allocate SRAM memory to be used by the microblock */
        IX_ERROR_CR(_ix_cc_atmrx_alloc_symbols_table(arg_pContext, s_atmrxSymbolTableAal2));

#ifdef VC_LOOKUP_IS_DEFINED
        /* Initialize VC_LOOKUP_TABLE_BASE table: */
        _ix_cc_atmrx_vc_lookup_init(arg_pContext);
        _IX_CC_ATMRX_DBG_INIT (("Successfuly init memory for AAL2_RX_VC_LOOKUP_TABLE\n"));
#endif
        /* Initialize AAL2_CID_RX_CTX table: */
        _ix_cc_atmrx_aal2_cid_rx_ctx_init(arg_pContext);
        _IX_CC_ATMRX_DBG_INIT (("Successfuly init memory for AAL2_CID_RX_CTX_TABLE\n"));

        /* Initialize CID Hash primary table: */
        _ix_cc_atmrx_cid_hash_pri_init(arg_pContext);
        _IX_CC_ATMRX_DBG_INIT (("Successfuly init memory for CID Hash primaary table\n"));

        /* Initialize CID Hash secondary table: */
        _ix_cc_atmrx_cid_hash_sec_init(arg_pContext);
        _IX_CC_ATMRX_DBG_INIT (("Successfuly init memory for CID Hash secondary table\n"));

    }
#endif /* AAL2_INCLUDED */

    return IX_SUCCESS;
}/* end _ix_cc_atmrx_alloc_symbols */


/**
 * NAME: _ix_cc_atmrx_patch_symbols
 *
 * DESCRIPTION:
 *    This primitive will be used to patch symbols to L2 Bridge microblocks.
 * 
 * @Param:  - IN arg_pContext - pointer to context of current core component.
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMRX_ERROR_FAILED_PATCHING - failed patching by calling resource
 *        manager function
 */
static ix_error
_ix_cc_atmrx_patch_symbols(
_ix_cc_atmrx_context_t* arg_pContext)
{

#define _IX_CC_ATMRX_SYMBOL_SIZE 40

    ix_imported_symbol aImportSymbol;
    ix_uint32 i,j;
    ix_memory_type memType;
    ix_uint32 channel = 0;
    ix_uint32 zboffset = 0;
    void *pPhysicalAddress = NULL;
    ix_error err;
    char symbol[_IX_CC_ATMRX_SYMBOL_PREFIX_SIZE + _IX_CC_ATMRX_SYMBOL_SIZE + 1];

    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);

    for (i=0; s_atmrxSymbolTable[i].symbol != NULL; i++)
    {
        IX_ERROR_CT(ix_rm_get_phys_offset((void *)(arg_pContext->pSymbolTable[s_atmrxSymbolTable[i].symbolId]),
                                          &memType,
                                          &channel,
                                          &zboffset,
                                          &pPhysicalAddress),
                    err,
                    IX_CC_ATMRX_ERROR_FAILED_PATCHING, 
                    IX_ERROR_LEVEL_GLOBAL);
        if (err != IX_SUCCESS)
        {
            return(err);
        }

        /* Pack the SRAM channel number into the offset */
        IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(channel, zboffset);

        /** 
         * Prepare the symbol array.  For now only single mode is used 
         * so skip the memOffsetNextAvailTBufElem.
         */   
        if (strlen(s_atmrxSymbolTable[i].symbol) > _IX_CC_ATMRX_SYMBOL_SIZE)
        {
            return IX_ERROR_GLOBAL(IX_CC_ATMRX_ERROR_FAILED_PATCHING, 
                                   ("Variable %s too long", s_atmrxSymbolTable[i].symbol));
        }
        sprintf(symbol, "%s%s", arg_pContext->symbolPrefix, s_atmrxSymbolTable[i].symbol);
        aImportSymbol.m_Value = (ix_uint32)zboffset;
        aImportSymbol.m_Name  = symbol;
    
        for (j=0;j<IX_MAX_MICROENGINE_NUMBER;j++)
        {
            if (arg_pContext->mev2Num & (0x1 << j))
                IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[j],
                                                     1,
                                                     &aImportSymbol));
        }
    }

    /* patch the AAL5_RX_VC_HASH_PRI_INDEX_MASK symbol */
    sprintf(symbol, "%s%s", arg_pContext->symbolPrefix, "AAL5_RX_VC_HASH_PRI_INDEX_MASK");
    aImportSymbol.m_Value = (ix_uint32)(arg_pContext->pluginCfg.numberOfVcs-1);
    aImportSymbol.m_Name  = symbol;
    for (j=0;j<IX_MAX_MICROENGINE_NUMBER;j++)
    {
        if (arg_pContext->mev2Num & (0x1 << j))
            IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[j],
                                                 1,
                                                 &aImportSymbol));
    }    
    
#ifdef AAL2_INCLUDED
    
    channel = 0;
    zboffset = 0;
    pPhysicalAddress = NULL;
    
    for (i=0; s_atmrxSymbolTableAal2[i].symbol != NULL; i++)
    {
        IX_ERROR_CT(ix_rm_get_phys_offset((void *)(arg_pContext->pSymbolTable[s_atmrxSymbolTableAal2[i].symbolId]),
                                          &memType,
                                          &channel,
                                          &zboffset,
                                          &pPhysicalAddress),
                    err,
                    IX_CC_ATMRX_ERROR_FAILED_PATCHING, 
                    IX_ERROR_LEVEL_GLOBAL);
        if (err != IX_SUCCESS)
        {
            return(err);
        }

        /* Pack the SRAM channel number into the offset */
        IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(channel, zboffset);

        /** 
         * Prepare the symbol array.  For now only single mode is used 
         * so skip the memOffsetNextAvailTBufElem.
         */   
        if (strlen(s_atmrxSymbolTableAal2[i].symbol) > _IX_CC_ATMRX_SYMBOL_SIZE)
        {
            return IX_ERROR_GLOBAL(IX_CC_ATMRX_ERROR_FAILED_PATCHING, 
                                   ("Variable %s too long", s_atmrxSymbolTableAal2[i].symbol));
        }
        sprintf(symbol, "%s%s", arg_pContext->symbolPrefix, s_atmrxSymbolTableAal2[i].symbol);
        aImportSymbol.m_Value = (ix_uint32)zboffset;
        aImportSymbol.m_Name  = symbol;
    
        for (j=0;j<IX_MAX_MICROENGINE_NUMBER;j++)
        {
            if (arg_pContext->mev2Aal2Num & (0x1 << j))
                IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[j],
                                                     1,
                                                     &aImportSymbol));
        }
    }
    
#endif
    
    return IX_SUCCESS;
    
#undef _IX_CC_ATMRX_SYMBOL_SIZE
    
}/* end _ix_cc_atmrx_patch_symbols */


/**
 * NAME: _ix_cc_atmrx_create_64bit_counters
 *
 * DESCRIPTION:
 *    This primitive will be used to create 64 bit counters.   .
 * 
 * @Param:  - IN arg_pContext - pointer to context of current core component.
 * @Param:  - IN pInitCtx - pointer to initial context. The context contains
 *                   description of static configuration.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ERROR_OOM_64BIT_COUNTER - error in creating 64 bit counter
 */
static ix_error
_ix_cc_atmrx_create_64bit_counters(
_ix_cc_atmrx_context_t* arg_pContext)
{
    ix_counter_64bit_handle* p64BitCountersHdls;
    ix_uint32* p64BitCountersBase;
    ix_uint16 num64BitCountersHdls;
    ix_uint32 overflowTime;
    ix_uint32 overflowTimeMin;
    ix_uint16 i;
    ix_cc_atmsar_plugin_cfg_port_descr_t *pPortCfgTab;
    ix_error err=IX_SUCCESS;
   
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);

    _IX_CC_ATMRX_DBG_INIT(("ATMRX: READ PORT STATIC CONFIGURATION\n"));

    
    num64BitCountersHdls=NO_PER_PORT_STAT_COUNTERS;
    overflowTimeMin=0x0FFFFFFFF;

    pPortCfgTab = ix_ossl_malloc(sizeof(*pPortCfgTab)*arg_pContext->pluginCfg.numberOfPorts);
    if (pPortCfgTab == NULL)
    {
        return IX_ERROR_GLOBAL(IX_CC_ERROR_NULL, ("Pointer for Port configuration table is NULL"));
    }
    
    IX_ERROR_CG(ix_cc_atmsar_plugin_get_cfg_ports_params(arg_pContext->instance,
                                                         arg_pContext->pluginCfg.numberOfPorts,
                                                         pPortCfgTab),
                err,
                end_of_processing);
    /*
     * read static configuration of ports and create 64bit counters
     * for ports statistic
     */
    for (i = 0; i<arg_pContext->pluginCfg.numberOfPorts; i++)
    {
        if (pPortCfgTab[i].linkRate == 0)
        {
            continue;
        }

        overflowTime=0x0FFFFFFFF/pPortCfgTab[i].linkRate;
        if (overflowTime<700)
        {
            /* Max link speed is equal 5636096 cps (It is OC-48). Minimum
             * overflow time is equal (2^32)/5636096 =~ 762 seconds */
            err = IX_ERROR_GLOBAL(IX_CC_ERROR_ENTRY_NOT_FOUND, 
                                  ("Wrong overflow time %d for port nr %d",
                                   overflowTime, i));
            goto end_of_processing;
        }

        /* change overflow time into safe range. Extra 10 seconds */
        overflowTime-=10;
        if (overflowTime < overflowTimeMin)
        {
            overflowTimeMin=overflowTime;
        }
        
        _IX_CC_ATMRX_DBG_INIT(("ATMRX: ATM PORT(%04d):  speed=%-10ld, overflowTime=%ld\n",
                       i, pPortCfgTab[i].linkRate, overflowTime));
        
        p64BitCountersBase=arg_pContext->pSymbolTable[_IX_CC_ATMRX_AAL5_RX_PORT_STAT_SRAM_BASE_SYMBOL_ID];
        p64BitCountersHdls=arg_pContext->hPortCounters;

        p64BitCountersBase+=(NO_PER_PORT_STAT_COUNTERS*i);
        p64BitCountersHdls+=(NO_PER_PORT_STAT_COUNTERS*i);

        /* Allocate 64-bit Rx counters handles array from RM */
        IX_ERROR_CGT(ix_rm_counter_64bit_new(num64BitCountersHdls,
                                             p64BitCountersBase,
                                             overflowTime,
                                             p64BitCountersHdls),
                     err,
                     IX_CC_ERROR_OOM_64BIT_COUNTER, 
                     IX_ERROR_LEVEL_REMOTE,
                     end_of_processing);
    }
    
    /* create 64 bits counters for VC statistics */

    num64BitCountersHdls=ATMRX_VC_STAT_COUNTERS_NO;
    
    for (i = 0; i <arg_pContext->pluginCfg.numberOfVcs; i++)
    {
        p64BitCountersBase=arg_pContext->pSymbolTable[_IX_CC_ATMRX_AAL5_RX_VC_INFO_SRAM_BASE_SYMBOL_ID];
        p64BitCountersHdls=arg_pContext->hVcCounters;

        p64BitCountersBase+=(((AAL5_RX_VC_INFO_ENTRY_SIZE>>2)*i)+ATMRX_VC_STAT_COUNTERS_FIRST_LW);
        p64BitCountersHdls+=(ATMRX_VC_STAT_COUNTERS_NO*i);

        /* Allocate 64-bit Rx counters handles array from RM */
        IX_ERROR_CGT(ix_rm_counter_64bit_new(num64BitCountersHdls,
                                             p64BitCountersBase,
                                             overflowTimeMin,
                                             p64BitCountersHdls),
                     err,
                     IX_CC_ERROR_OOM_64BIT_COUNTER, 
                     IX_ERROR_LEVEL_REMOTE,
                     end_of_processing);
    }

  end_of_processing:

    ix_ossl_free(pPortCfgTab);
    
    return err;
}/* end _ix_cc_atmrx_create_64bit_counters */


/**
 * NAME: _ix_cc_atmrx_register_message_handler
 *
 * DESCRIPTION:
 *    This primitive will be used to register message handler. 
 * 
 * @Param:  IN  arg_pContext : pointer to context of current core component.
 * @Param:  OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMRX_ERROR_CCI - Error from CCI
 */
static ix_error
_ix_cc_atmrx_register_message_handler(
_ix_cc_atmrx_context_t* arg_pContext)
{
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);
    /* Call CCI message handler if CCI is enabled */
#ifndef CCI_MULTIINSTANCE
#if defined(IX_EXCLUDE_CCI) 
    IX_ERROR_CRT(ix_rm_message_handler_register(IX_CC_ATMRX_MSG_INPUT,
                                                ix_cc_atmrx_msg_handler,
                                                arg_pContext),
                 IX_CC_ATMRX_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);
#else
    IX_ERROR_CRT(ix_cci_cc_add_message_handler(arg_pContext->hAtmRxHandle,
                                               IX_CC_ATMRX_MSG_INPUT,
                                               ix_cc_atmrx_msg_handler,
                                               IX_INPUT_TYPE_MULTI_SRC),
                 IX_CC_ATMRX_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);

#endif
#endif
    return IX_SUCCESS;
}/* end _ix_cc_atmrx_register_message_handler */


/**
 * NAME: _ix_cc_atmrx_register_cci_packet_handlers
 *
 * DESCRIPTION:
 *    This primitive will be used to register packet handler with CCI.
 *
 * @Param:  IN  arg_pContext : pointer to context of current core component.
 * @Param:  OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMRX_ERROR_CCI - error from CCI
 */
static ix_error
_ix_cc_atmrx_register_cci_packet_handlers(
_ix_cc_atmrx_context_t* arg_pContext)
{
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);
#if !defined(IX_EXCLUDE_CCI)
    /* High priority packet handler needs to be registered as a message handler */
    ix_rm_packet_set_receive_mode(IX_CC_ATMRX_PKT_INPUT, IX_COMM_ID_MODE_CALLBACK);

    IX_ERROR_CRT(ix_rm_message_handler_register(IX_CC_ATMRX_PKT_INPUT,
                                                ix_cc_atmrx_pkt_handler,
                                                arg_pContext),
                 IX_CC_ATMRX_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);    

#endif
    return IX_SUCCESS;
}/* end _ix_cc_atmrx_register_cci_packet_handlers */


/**
 * NAME: _ix_cc_atmrx_register_rm_packet_handlers
 *
 * DESCRIPTION:
 *    This primitive will be used to register packet handler with resource
 *    manager
 *
 * @Param:  IN  arg_pContext : pointer to context of current core component.
 * @Param:  OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMRX_ERROR_CCI - error from resource manager
 */
#if defined(IX_EXCLUDE_CCI)
static ix_error
_ix_cc_atmrx_register_rm_packet_handlers(
_ix_cc_atmrx_context_t* arg_pContext)
{
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);
    IX_ERROR_CRT(ix_rm_message_handler_register(IX_CC_ATMRX_PKT_INPUT,
                                                ix_cc_atmrx_pkt_handler,
                                                arg_pContext),
                 IX_CC_ATMRX_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);

    return IX_SUCCESS;
}/* end _ix_cc_atmrx_register_rm_packet_handlers */
#endif


/**
 * NAME: atmrx_register_packet_handlers
 *
 * DESCRIPTION:
 *    This primitive will be used to register packet handlers. It registers with
 *    CCI in level 1 system and register with RM in level 0 system.
 * 
 * @Param:  IN  arg_pContext : pointer to context of current core component.
 * @Param:  OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMRX_ERROR_CCI - error from CCI/resource manager
 */
static ix_error atmrx_register_packet_handlers(
_ix_cc_atmrx_context_t* arg_pContext)
{
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);
    /* Call CCI packet handler if CCI is enabled */
#if defined(IX_EXCLUDE_CCI)
    IX_ERROR_CR(_ix_cc_atmrx_register_rm_packet_handlers(arg_pContext));
#else
    IX_ERROR_CR(_ix_cc_atmrx_register_cci_packet_handlers(arg_pContext));
#endif
    return IX_SUCCESS;
}/* end atmrx_register_packet_handlers */


/******************************************************************************
 *                            CCI API FUNCTIONS
 *****************************************************************************/

#ifdef AAL2_INCLUDED

/**
 * NAME: _ix_cc_atmrx_cid_list_get_entry
 * 
 * DESCRIPTION:
 *    The callback function returns pointer to stucture with
 *    cid description.
 *
 * @Param:  - IN arg_pUserData - pointer to structure that descrige storage
 *                   of entries
 * @Param:  - IN arg_handler - ID of entry
 *
 * @Return:
 *     Pointer if success otherwise NULL.
 */
void*
_ix_cc_atmrx_cid_list_get_entry(
void *arg_pUserData,
ix_uint16 arg_handler)
{
    _ix_cc_atmrx_context_t* pCtx=arg_pUserData;

#if defined(IX_DEBUG)

    if (arg_pUserData == NULL)
    {
        return NULL;
    }

#endif

    return &(pCtx->cid[arg_handler]);
} /* end _ix_cc_atmrx_cid_list_get_entry */

#endif /* AAL2_INCLUDED */


/**
 * NAME: ix_cc_atmrx_init
 *
 * DESCRIPTION:
 *    This primitive shall be called and returned successfully before requesting
 *    any service from the core component. This primitive should be called
 *    only once to initialize the core component. This function performs
 *    allocation of memory for symbols to be patched, Creation of 64 bit
 *    counters, registration of packet and message handlers, allocation and
 *    initialization of internal data structures.
 *              
 * 
 * @Param:  IN arg_hCcHandle - handle to the core component, created by CCI; 
 *                 this shall be used later to get other (to add event handler)
 *                 services from core component infrastructure.
 *
 * @Param:  IN-OUT arg_ppContext - Buffer free list and dynamic property data
 *                 will be given during by system APPs during initialization.
 *                 Upon successfull completion, it represents the location where
 *                 the pointer to the control block allocated by the core
 *                 component will be stored. The control block is internal to
 *                 the core component and it contains information about internal
 *                 data structures, allocated memory and other relevant
 *                 information. Later this pointer to be passed into fini()
 *                 function for freeing memory and other house keeping
 *                 operations when the core component is being destroyed.  
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ERROR_NULL - input parameter arg_ppContext is null
 *    IX_CC_ERROR_OOM   - memory allocation failure      
 *    IX_CC_ATMRX_ERROR_FAILED_PATCHING   - patching failures                      
 *    IX_CC_ERROR_OOM_64BIT_COUNTER - 64 bit counter creation failure
 *    IX_CC_ATMRX_ERROR_REGISTRY  - registry access failures
 *    IX_CC_ATMRX_ERROR_RTM - failure from RTM core component
 *    IX_CC_ATMRX_ERROR_CCI - failure from CCI 
 */
ix_error ix_cc_atmrx_init(
ix_cc_handle arg_hCcHandle,
void** arg_ppContext)
{

#ifdef AAL2_INCLUDED

    ix_uint16 i;

#endif /* AAL2_INCLUDED */

    ix_error err = IX_SUCCESS;
    _ix_cc_atmrx_context_t* pContext;
    ix_cc_init_context *pInitCtx;    /* pointer to variable with static configuration */

    IX_ERROR_CHECK_ARG_NULL(2, arg_ppContext);
    
    _IX_CC_ATMRX_DBG_INIT(("ix_cc_atmrx_init() START\n"));

    /* Allocate memory for L2 Bridge Context */
    pContext = (_ix_cc_atmrx_context_t*)ix_ossl_malloc(sizeof(_ix_cc_atmrx_context_t));
    if (pContext == NULL)
    {
        return IX_ERROR_GLOBAL(IX_CC_ERROR_OOM,
                               ("Failed to allocate memory for L2 Bridge context"));
    }/* end if (pContext == NULL) */

    /* initialize the structure */
    ix_ossl_memset(pContext,0,sizeof(_ix_cc_atmrx_context_t));

    /* store ATM RX  component handle in _ix_cc_atmrx_context_t */
    pContext->hAtmRxHandle = arg_hCcHandle;

#ifdef CCI_MULTIINSTANCE
    /* use multiinstance CC ID if applicable */
    pContext->instance = IX_HANDLE_TO_INSTANCE(arg_hCcHandle);
#else
    /* if not, use default instance number */
    pContext->instance=_IX_CC_ATMRX_SAR_INSANCE_0;
#endif /* CCI_MULTIINSTANCE */


    /* initialize a head of list of free secondary buckets */
    pContext->freeSecondaryBucket= _IX_CC_ATMRX_HASH_SEC_IDX_INVALID;
    
    /* Get the generic init context */
    pInitCtx = (ix_cc_init_context *)*arg_ppContext;

    /* Retrieve static configuration from the registry or fill the data 
     * stuctures using the definition in the header files */
    IX_ERROR_CG(_ix_cc_atmrx_registry(pContext), err, cleanupContext);

    /* Allocate memory for symbols to be patched */
    IX_ERROR_CG(_ix_cc_atmrx_alloc_symbols(pContext), err, cleanupContext);

    /* Patch symbols */
    IX_ERROR_CG(_ix_cc_atmrx_patch_symbols(pContext), err, cleanupMemory);

    /* 64 bit counter creation */
    if (pContext->counters)
    {
        IX_ERROR_CG(_ix_cc_atmrx_create_64bit_counters(pContext), err, cleanupMemory);
    }

    /* Register Message Handler */
    IX_ERROR_CG(_ix_cc_atmrx_register_message_handler(pContext), err, cleanupCounter);

    /* Register Packet Handler */
    IX_ERROR_CG(atmrx_register_packet_handlers(pContext), err, cleanupMessageHandler);

    IX_ERROR_CG(ix_cc_atmsar_plugin_reg_vc_service(IX_CC_ATMRX,
                                                   pContext->instance,
                                                   _ix_cc_atmrx_plugin_vc_create,
                                                   _ix_cc_atmrx_plugin_vc_update,
                                                   _ix_cc_atmrx_plugin_vc_remove,
                                                   _ix_cc_atmrx_plugin_get_vc_stats),
                err,
                cleanupPacketHandlers);
    
    IX_ERROR_CG(ix_cc_atmsar_plugin_reg_port_service(IX_CC_ATMRX,
                                                     pContext->instance,
                                                     _ix_cc_atmrx_plugin_port_create,
                                                     _ix_cc_atmrx_plugin_port_remove,
                                                     _ix_cc_atmrx_plugin_get_port_stats),
                err,
                cleanupPacketHandlers);

#ifdef AAL2_INCLUDED

    if (pContext->pluginCfg.aal2Included == IX_CC_ATMSAR_PLUGIN_AAL2_INLUDED_YES)
    {

        /* initialize CID list */
        pContext->listCid.count = MAX_AAL2_CHANNELS - 1;
        pContext->listCid.getEntry = _ix_cc_atmrx_cid_list_get_entry;
        pContext->listCid.pUserData = pContext;
        if(ix_cc_atm_list_init (&(pContext->listCid),
                                IX_CC_ATM_LIST_INIT_NO_LINK) == IX_CC_ATM_LIST_ERROR)
        {
            err = IX_ERROR_GLOBAL(IX_CC_ERROR_OOM,
                                  ("Failed CID list initialization for ATM RX context"));
            goto cleanupContext;
        }

        pContext->freeCidCtxs=IX_CC_ATM_LIST_INVALID_HANDLE;
        for(i=0; i < MAX_AAL2_CHANNELS; i++) 
        {
            if(ix_cc_atm_list_insert(i, 0, &pContext->freeCidCtxs, &pContext->listCid) != IX_CC_ATM_LIST_OK)
            {
                err = IX_ERROR_GLOBAL(IX_CC_ERROR_OOM,
                                      ("Failed CID list initialization for ATM RX context"));
                goto cleanupContext;
            }
        }
    
        IX_ERROR_CG(ix_cc_atmsar_plugin_reg_cid_service(IX_CC_ATMRX,
                                                        pContext->instance,
                                                        _ix_cc_atmrx_plugin_cid_create,
                                                        _ix_cc_atmrx_plugin_cid_update,
                                                        _ix_cc_atmrx_plugin_cid_remove,
                                                        _ix_cc_atmrx_plugin_get_cid_stats),
                    err,
                    cleanupPacketHandlers);

        IX_ERROR_CG(ix_cc_atmsar_plugin_reg_cid_handle_service(pContext->instance,
                                                               pContext,
                                                               _ix_cc_atmrx_plugin_cid_alloc,
                                                               _ix_cc_atmrx_plugin_cid_free),
                    err,
                    cleanupPacketHandlers);

    }
    
#endif /* AAL2_INCLUDED */
    
    IX_ERROR_CG(ix_cc_atmsar_plugin_reg_done(IX_CC_ATMRX,
                                             pContext->instance,
                                             pContext),
                err,
                cleanupPacketHandlers);
    *arg_ppContext = (void *)pContext;
    _IX_CC_ATMRX_DBG_INIT(("ix_cc_atmrx_init() SUCCESS\n"));
    return IX_SUCCESS;

  cleanupPacketHandlers:
    _ix_cc_atmrx_unregister_packet_handlers(pContext);
  cleanupMessageHandler:
    _ix_cc_atmrx_unregister_message_handler(pContext);
  cleanupCounter:
    if (pContext->counters)
    {
        _ix_cc_atmrx_delete_64bit_counters(pContext);
    }
  cleanupMemory:
    _ix_cc_atmrx_free_symbols(pContext);
  cleanupContext:
    ix_ossl_free(pContext);
    _IX_CC_ATMRX_DBG_INIT(( "ix_cc_atmrx_init() FAILED\n"));
    return err;
} /* end ix_cc_atmrx_init */


/**
 * NAME: ix_cc_atmrx_fini
 *
 * DESCRIPTION:
 *    This primitive shall be called to terminate services from this core
 *    component. This primitive frees memory allocated during initialization,
 *    free all the created resources: delete 64 bit counter. If client claims
 *    for any services from this core component after calling this function,
 *    then the behavior is undefined.
 * 
 * @Param:  IN arg_hCcHandle - handle to this core component
 * @Param:  IN arg_pContext - pointer to the control block memory allocated 
 *                 earlier in ix_cc_atmrx_init function. 
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMRX_ERROR_INVALID_INPUT_PARAM - invalid input parameter(s)
 *    IX_CC_ERROR _OOM - failure of memory deletion                
 *    IX_CC_ERROR_OOM_64BIT_COUNTER - failure of  64 bit counter deletion 
 *    IX_CC_ATMRX_ERROR _RTM - failure from RTM
 *    IX_CC_ATMRX_ERROR_CCI - failure from CCI
 */
ix_error ix_cc_atmrx_fini(
ix_cc_handle arg_hCcHandle,
void* arg_pContext)
{
    _ix_cc_atmrx_context_t* pContext=arg_pContext;

    IX_ERROR_CHECK_ARG_NULL(2, arg_pContext);

    if (arg_hCcHandle != pContext->hAtmRxHandle)
    {
        return IX_ERROR_WARNING(IX_CC_ATMRX_ERROR_INVALID_INPUT_PARAM,
                                ("Invalid input Parameters"));
    }
  
    /* Free memory for symbols */
    IX_ERROR_CR(_ix_cc_atmrx_free_symbols(pContext));

    /* Delete 64 bit counter */
    if (pContext->counters)
    {
        IX_ERROR_CR(_ix_cc_atmrx_delete_64bit_counters(pContext));
    }
    
    /* Unregister Message Handler */
    IX_ERROR_CR(_ix_cc_atmrx_unregister_message_handler(pContext));
  
    /* Unregister Packet Handler */
    IX_ERROR_CR(_ix_cc_atmrx_unregister_packet_handlers(pContext));
    
    /* Free AtmRx Context */ 
    ix_ossl_free(pContext);
        
    return IX_SUCCESS;
} /* end ix_cc_atmrx_fini */

