/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      ATM RX Core Component
 *
 * = FILENAME
 *      ix_cc_atmrx_plugin.c
 *
 * = DESCRIPTION
 *      The file defines functions that are registered in ATM SAR as
 *      plugin functions.
 *
 * = AUTHOR
 *       Jaroslaw Kogut
 *       jaroslaw.kogut@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      11/07/2002
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */
#define IX_ERROR_FILE_IDENT "$Id: ix_cc_atmrx_plugin.c,v 1.3 2003/12/12 19:13:45 adziecia Exp $";

/**
 * System defined include files
 */
#include "ix_cc_error.h"
#include "ix_rm.h"
#include "ix_ossl.h"
#include "ix_cc.h"
#include "ix_cc_macros.h"
#include "ix_config.h"
#include "bindings.h"
#include "cc/ix_cc_atmsar_plugin.h"

/**
 * User defined include files
 */
#include "cc/ix_cc_atmrx_cci.h"
#include "cc/internal/ix_cc_atmrx_private.h"

/**
 * Preprocessor symbols and Macros used in this file
 */


/**
 * Type definitions whose scope is limited to this file
 */

/*                               funcPfx,          varName, lw, offset, mask */
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_pri_hash,   valid,   0,  31,     0x01);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_pri_hash,   key1,    0,  0,      0x07FFFFFFF);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_pri_hash,   ptr2,    1,  16,     0x0FFFF);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_pri_hash,   ptr1,    1,  0,      0x0FFFF);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_pri_hash,   key2,    2,  0,      0x0FFFFFFFF);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_pri_hash,   keyExt1, 3,  28,     0x0F);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_pri_hash,   keyExt2, 3,  24,     0x0F);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_pri_hash,   next,    3,  0,      0x0FFFF);

ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_sec_hash,   valid,   0,  31,      0x01);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_sec_hash,   key1,    0,  0,      0x07FFFFFFF);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_sec_hash,   ptr2,    1,  16,     0x0FFFF);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_sec_hash,   ptr1,    1,  0,      0x0FFFF);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_sec_hash,   key2,    2,  0,      0x0FFFFFFFF);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_sec_hash,   key3,    3,  0,      0x0FFFFFFFF);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_sec_hash,   ptr4,    4,  16,     0x0FFFF);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_sec_hash,   ptr3,    4,  0,      0x0FFFF);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_sec_hash,   key4,    5,  0,      0x0FFFFFFFF);
/* LW6 reserved */
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_sec_hash,   keyExt1, 7,  28,     0x0F);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_sec_hash,   keyExt2, 7,  24,     0x0F);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_sec_hash,   keyExt3, 7,  20,     0x0F);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_sec_hash,   keyExt4, 7,  16,     0x0F);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_sec_hash,   next,    7,  0,      0x0FFFF);

ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_sec_hash,   list,    1,  0,      0x0FFFF); /* it is used for linking into free entry list*/

#ifdef FPGA_HEADER
#define _IX_CC_ATMRX_HASH_PREPARE_KEY(__vcSpec, __vcKey) \
    __vcKey.key=(__vcSpec.vci & 0x0FFFF) | ((__vcSpec.vpi & 0x0FF) << 16) | ((__vcSpec.portNo & 0x07F) << 24); \
    __vcKey.keyExt= ((__vcSpec.portNo >> 7) & 0x0F);
#else
#define _IX_CC_ATMRX_HASH_PREPARE_KEY(__vcSpec, __vcKey) \
    __vcKey.key=(__vcSpec.vci & 0x0FFFF) | ((__vcSpec.vpi & 0x0FF) << 16) | (((__vcSpec.portNo & (~((1<<AAL5_FPGA_PORT_BITS)-1))) & 0x07F) << 24); \
    __vcKey.keyExt= (((__vcSpec.portNo & (~((1<<AAL5_FPGA_PORT_BITS)-1))) >> 7) & 0x0F);
#endif

/**
 * Variable declarations global to this file only. Externs are followed by 
 * static variables.
 */

/** 
 * Extern function prototypes.
 */


/** 
 * Static function prototypes.
 */


/** 
 * Function definitions.
 */


/******************************************************************************
 *                    PRIMARY HASH TABLE SUPPORT FUNCTIONS
 *****************************************************************************/

/**
 * NAME: _ix_cc_atmrx_hash_pri_get_entry
 *
 * DESCRIPTION:
 *    Find pointer to entry in primary hash table. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN  arg_entryId : Id of requested entry
 *
 * @Return: NULL if error occurs, otherwise pointer to secondary bucket.
 */
ix_uint32*
_ix_cc_atmrx_hash_pri_get_entry(
_ix_cc_atmrx_context_t* arg_pContext,
ix_uint16 arg_entryId)
{
    return arg_pContext->pSymbolTable[_IX_CC_ATMRX_VC_HASH_PRI_BASE_SRAM_SYMBOL_ID]+((AAL5_RX_VC_HASH_PRI_BKT_ENTRY_SIZE>>2)*arg_entryId);
} /* end _ix_cc_atmrx_hash_pri_get_entry */


/**
 * NAME: _ix_cc_atmrx_hash_pri_init
 *
 * DESCRIPTION:
 *    Initialize primary hash table. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 *
 * @Return: none
 */
void
_ix_cc_atmrx_hash_pri_init(
_ix_cc_atmrx_context_t* arg_pContext)
{
    ix_uint32* pEntry;
    ix_uint32  i;
    for (i=0; i<arg_pContext->pluginCfg.numberOfVcs; i++)
    {
        pEntry=_ix_cc_atmrx_hash_pri_get_entry(arg_pContext, i);
        _ix_cc_atmrx_pri_hash_set_ptr1(pEntry, IX_CC_ATMSAR_VCID_INVALID);
        _ix_cc_atmrx_pri_hash_set_ptr2(pEntry, IX_CC_ATMSAR_VCID_INVALID);
    }
} /* end _ix_cc_atmrx_hash_pri_init */


/******************************************************************************
 *                    SECONDARY TABLE SUPPORT FUNCTIONS
 *****************************************************************************/

/**
 * NAME: _ix_cc_atmrx_hash_sec_get_entry
 *
 * DESCRIPTION:
 *    Find pointer to secondary bucket. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN arg_entryId : Id of requested bucket
 *
 * @Return: NULL if error occurs, otherwise pointer to secondary bucket.
 */
ix_uint32*
_ix_cc_atmrx_hash_sec_get_entry(
_ix_cc_atmrx_context_t* arg_pContext,
ix_uint16 arg_entryId)
{
    if (arg_entryId>=AAL5_RX_VC_HASH_SEC_SIZE)
    {
        return NULL;
    }
    return arg_pContext->pSymbolTable[_IX_CC_ATMRX_VC_HASH_SEC_BASE_SRAM_SYMBOL_ID]+((AAL5_RX_VC_HASH_SEC_BKT_ENTRY_SIZE>>2)*arg_entryId);
} /* end _ix_cc_atmrx_hash_sec_get_entry */


/**
 * NAME: _ix_cc_atmrx_hash_sec_free_entry
 *
 * DESCRIPTION:
 *    Put entry to list. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN  arg_pEntry : pointer to entry
 *
 * @Return: none
 */
void
_ix_cc_atmrx_hash_sec_free_entry(
_ix_cc_atmrx_context_t *arg_pContext,
ix_uint16 arg_entry)
{
    ix_uint32* pEntry;

    pEntry=_ix_cc_atmrx_hash_sec_get_entry(arg_pContext,arg_entry);
    if (pEntry == NULL)
    {
        return;
    }
    
    /*
     * LW0 contains valid bit.
     * It cannot be used for storing ID of free sec buckets Id
     */
    _ix_cc_atmrx_sec_hash_set_valid(pEntry, 0);
    _ix_cc_atmrx_sec_hash_set_list(pEntry, arg_pContext->freeSecondaryBucket);
    arg_pContext->freeSecondaryBucket=arg_entry;
} /* end _ix_cc_atmrx_hash_sec_free_entry */


/**
 * NAME: _ix_cc_atmrx_hash_sec_alloc_entry
 *
 * DESCRIPTION:
 *    Allocate secondary bucket for use.
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 *
 * @Return: ID of allocated entry. If no free entry function returns
 *          _IX_CC_ATMRX_HASH_SEC_IDX_INVALID.
 */
ix_uint16
_ix_cc_atmrx_hash_sec_alloc_entry(
_ix_cc_atmrx_context_t *arg_pContext)
{
    ix_uint16 retValue;
    ix_uint32* pEntry;
    if (arg_pContext->freeSecondaryBucket == _IX_CC_ATMRX_HASH_SEC_IDX_INVALID)
    {
        return _IX_CC_ATMRX_HASH_SEC_IDX_INVALID;
    }
    retValue=arg_pContext->freeSecondaryBucket;
    pEntry=_ix_cc_atmrx_hash_sec_get_entry(arg_pContext, retValue);
    arg_pContext->freeSecondaryBucket=_ix_cc_atmrx_sec_hash_get_list(pEntry);
    /*ix_ossl_memset((void*)pEntry, 0, AAL5_RX_VC_HASH_SEC_BKT_ENTRY_SIZE);*/
    return retValue;
} /* end _ix_cc_atmrx_hash_sec_alloc_entry */


/**
 * NAME: _ix_cc_atmrx_hash_sec_init
 *
 * DESCRIPTION:
 *    Initialize secondary hash table. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 *
 * @Return: none
 */
void
_ix_cc_atmrx_hash_sec_init(
_ix_cc_atmrx_context_t *arg_pContext)
{
    ix_uint32* pEntry;
    ix_uint32  i;

    /*
     * Initialize secondary hash free list
     * NOTE: index 0 from secondary hash table is unused. Index 0 means invalid
     * index in hash tables
     */

    for (i=1; i<(arg_pContext->pluginCfg.numberOfVcs>>2);i++)
    {
        pEntry=_ix_cc_atmrx_hash_sec_get_entry(arg_pContext, i);
        _ix_cc_atmrx_sec_hash_set_ptr1(pEntry, IX_CC_ATMSAR_VCID_INVALID);
        _ix_cc_atmrx_sec_hash_set_ptr2(pEntry, IX_CC_ATMSAR_VCID_INVALID);
        _ix_cc_atmrx_sec_hash_set_ptr3(pEntry, IX_CC_ATMSAR_VCID_INVALID);
        _ix_cc_atmrx_sec_hash_set_ptr4(pEntry, IX_CC_ATMSAR_VCID_INVALID);
        _ix_cc_atmrx_hash_sec_free_entry(arg_pContext, i);
    }
} /* end _ix_cc_atmrx_hash_sec_init */


/******************************************************************************
 *                    HASH TABLES SUPPORT FUNCTIONS
 *****************************************************************************/


/**
 * NAME: _ix_cc_atmrx_hash_crc32
 *
 * DESCRIPTION:
 *    This is a function counting CRC32 from two long words
 * 
 * @Param:  IN  arg_pLw :  pointer to table of long words
 * 
 * @Return: value of counted CRC
 */
static ix_uint32
_ix_cc_atmrx_hash_crc32(
const ix_uint32* arg_pLw)
{
    ix_uint32 result;
    ix_uint32 i;
    ix_uint32 j;
    ix_uint32 data;

    result=0;
    data=arg_pLw[0];
    for (j=0; j<3; j++)
    {
        for (i=0; i<32; i++)
        {
            if ((result >> 31))
            { 
                result <<=1; result = result | ((data >>(31-i)) & (0x1));
                result = result ^ 0x4C11DB7;
            }
            else
            { 
                result <<=1; result = result | ((data >>(31-i)) & (0x1));  
            }
        }
        if (j==0) { data=arg_pLw[1]; }
        else 	  {	data=0; 	     }
	}
    return result;             /* The complement of the remainder */
} /* end _ix_cc_atmrx_hash_crc32 */


/**
 * NAME: _ix_cc_atmrx_hash_find_primary_idx
 *
 * DESCRIPTION:
 *    This is a function inserts VC into hash tables supported by ATM RX.
 * 
 * @Param:  IN  arg_hVC :  handle to created VC.
 * @Param:  IN  arg_pVcDescr : parameters describing requested VC.
 * 
 * @Return: Index in primary hash table for requested VC
 */
static ix_uint16
_ix_cc_atmrx_hash_find_primary_idx(
_ix_cc_atmrx_context_t *arg_pContext,
const ix_cc_atmsar_vc_spec_t* arg_pSpec)
{
    ix_uint32 lw[2];
    lw[0]= ((arg_pSpec->vpi << 16) | arg_pSpec->vci) & 0x0FFFFFF;
#ifdef FPGA_HEADER
    lw[1]=arg_pSpec->portNo;
#else
    lw[1]=arg_pSpec->portNo & (~((1<<AAL5_FPGA_PORT_BITS)-1));
#endif
    return _ix_cc_atmrx_hash_crc32(lw) & (arg_pContext->pluginCfg.numberOfVcs-1);
} /* end _ix_cc_atmrx_hash_find_primary_idx */


/**
 * NAME: _ix_cc_atmrx_hash_search_vc_by_ptr
 *
 * DESCRIPTION:
 *    The function searchs position in hash mechanism that has key equal with
 *    requested. It checks primary and secondary hash structures.
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN  arg_pVcKey : pointer to structure describing vc key.
 * @Param:  IN  arg_priIdx : index of entry in primary hash.
 * @Param:  OUT arg_pPos:    position of founded VC.
 * 
 * @Return: 1 - key exists, 0 otherwise
 */
ix_uint8
_ix_cc_atmrx_hash_search_vc_by_ptr(
_ix_cc_atmrx_context_t* arg_pContext,
ix_uint16 arg_ptr,
ix_uint16 arg_priIdx,
_ix_cc_atmrx_hash_pos_t* arg_pPos)
{
    ix_uint32 *pEntry;
    ix_uint16 next;

    /* varify primary hash entry */
    pEntry=_ix_cc_atmrx_hash_pri_get_entry(arg_pContext, arg_priIdx);

    if (_ix_cc_atmrx_pri_hash_get_ptr1(pEntry) == arg_ptr)
    {
        arg_pPos->tableId = _IX_CC_ATMRX_HASH_POS_PRI;
        arg_pPos->idx = arg_priIdx;
        arg_pPos->key = 1;
        return 1;
    }
    if (_ix_cc_atmrx_pri_hash_get_ptr2(pEntry) == arg_ptr)
    {
        arg_pPos->tableId = _IX_CC_ATMRX_HASH_POS_PRI;
        arg_pPos->idx = arg_priIdx;
        arg_pPos->key = 2;
        return 1;
    }
    
    next=_ix_cc_atmrx_pri_hash_get_next(pEntry);

    while (next != _IX_CC_ATMRX_HASH_SEC_IDX_INVALID)
    {
        /* varify secondary hash entry */
        pEntry=_ix_cc_atmrx_hash_sec_get_entry(arg_pContext, next);
        
        if (_ix_cc_atmrx_sec_hash_get_ptr1(pEntry) == arg_ptr)
        {
            arg_pPos->tableId = _IX_CC_ATMRX_HASH_POS_SEC;
            arg_pPos->idx = next;
            arg_pPos->key = 1;
            return 1;
        }
        if (_ix_cc_atmrx_sec_hash_get_ptr2(pEntry) == arg_ptr)
        {
            arg_pPos->tableId = _IX_CC_ATMRX_HASH_POS_SEC;
            arg_pPos->idx = next;
            arg_pPos->key = 2;
            return 1;
        }
        if (_ix_cc_atmrx_sec_hash_get_ptr3(pEntry) == arg_ptr)
        {
            arg_pPos->tableId = _IX_CC_ATMRX_HASH_POS_SEC;
            arg_pPos->idx = next;
            arg_pPos->key = 3;
            return 1;
        }
        if (_ix_cc_atmrx_sec_hash_get_ptr4(pEntry) == arg_ptr)
        {
            arg_pPos->tableId = _IX_CC_ATMRX_HASH_POS_SEC;
            arg_pPos->idx = next;
            arg_pPos->key = 4;
            return 1;
        }
        /* get next secondary hash entry */
        next=_ix_cc_atmrx_sec_hash_get_next(pEntry);
    }
    return 0;
} /* end _ix_cc_atmrx_hash_search_vc_by_ptr */


/**
 * NAME: _ix_cc_atmrx_hash_search_vc_by_key
 *
 * DESCRIPTION:
 *    The function searchs position in hash mechanism that has key equal with
 *    requested. It checks primary and secondary hash structures.
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN  arg_pVcKey : pointer to structure describing vc key.
 * @Param:  IN  arg_priIdx : index of entry in primary hash.
 * @Param:  OUT arg_pPos:    position of founded VC.
 * 
 * @Return: 1 - key exists, 0 otherwise
 */
static ix_uint8
_ix_cc_atmrx_hash_search_vc_by_key(
_ix_cc_atmrx_context_t* arg_pContext,
_ix_cc_atmrx_vc_key_t *arg_pVcKey,
ix_uint16 arg_priIdx,
_ix_cc_atmrx_hash_pos_t* arg_pPos)
{
    ix_uint32 *pEntry;
    ix_uint16 next;

    /* varify primary hash entry */
    pEntry=_ix_cc_atmrx_hash_pri_get_entry(arg_pContext, arg_priIdx);

    if ((_ix_cc_atmrx_pri_hash_get_key1(pEntry) == arg_pVcKey->key) &&
        (_ix_cc_atmrx_pri_hash_get_keyExt1(pEntry) == arg_pVcKey->keyExt))
    {
        arg_pPos->tableId = _IX_CC_ATMRX_HASH_POS_PRI;
        arg_pPos->idx = arg_priIdx;
        arg_pPos->key = 1;
        return 1;
    }
    if ((_ix_cc_atmrx_pri_hash_get_key2(pEntry) == arg_pVcKey->key) &&
        (_ix_cc_atmrx_pri_hash_get_keyExt2(pEntry) == arg_pVcKey->keyExt))
    {
        arg_pPos->tableId = _IX_CC_ATMRX_HASH_POS_PRI;
        arg_pPos->idx = arg_priIdx;
        arg_pPos->key = 2;
        return 1;
    }
    
    next=_ix_cc_atmrx_pri_hash_get_next(pEntry);

    while (next != _IX_CC_ATMRX_HASH_SEC_IDX_INVALID)
    {
        /* varify secondary hash entry */
        pEntry=_ix_cc_atmrx_hash_sec_get_entry(arg_pContext, next);
        
        if ((_ix_cc_atmrx_sec_hash_get_key1(pEntry) == arg_pVcKey->key) &&
            (_ix_cc_atmrx_sec_hash_get_keyExt1(pEntry) == arg_pVcKey->keyExt))
        {
            arg_pPos->tableId = _IX_CC_ATMRX_HASH_POS_SEC;
            arg_pPos->idx = next;
            arg_pPos->key = 1;
            return 1;
        }
        if ((_ix_cc_atmrx_sec_hash_get_key2(pEntry) == arg_pVcKey->key) &&
            (_ix_cc_atmrx_sec_hash_get_keyExt2(pEntry) == arg_pVcKey->keyExt))
        {
            arg_pPos->tableId = _IX_CC_ATMRX_HASH_POS_SEC;
            arg_pPos->idx = next;
            arg_pPos->key = 2;
            return 1;
        }
        if ((_ix_cc_atmrx_sec_hash_get_key3(pEntry) == arg_pVcKey->key) &&
            (_ix_cc_atmrx_sec_hash_get_keyExt3(pEntry) == arg_pVcKey->keyExt))
        {
            arg_pPos->tableId = _IX_CC_ATMRX_HASH_POS_SEC;
            arg_pPos->idx = next;
            arg_pPos->key = 3;
            return 1;
        }
        if ((_ix_cc_atmrx_sec_hash_get_key4(pEntry) == arg_pVcKey->key) &&
            (_ix_cc_atmrx_sec_hash_get_keyExt4(pEntry) == arg_pVcKey->keyExt))
        {
            arg_pPos->tableId = _IX_CC_ATMRX_HASH_POS_SEC;
            arg_pPos->idx = next;
            arg_pPos->key = 4;
            return 1;
        }
        /* get next secondary hash entry */
        next=_ix_cc_atmrx_sec_hash_get_next(pEntry);
    }
    return 0;
} /* end _ix_cc_atmrx_hash_search_vc_by_key */


/**
 * NAME: _ix_cc_atmrx_hash_set_vc
 *
 * DESCRIPTION:
 *    the function fills hash entry structure. Argument 'arg_pPos' describes
 *    place of setting, arg_pVcKey and arg_hVc describes what is setting.
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN  arg_pPos : pointer to structure describing location.
 * @Param:  IN  arg_pVcKey : 
 * @Param:  IN  arg_hVc : 
 * 
 * @Return: None
 */
static void
_ix_cc_atmrx_hash_set_vc(
_ix_cc_atmrx_context_t *arg_pContext,
const _ix_cc_atmrx_hash_pos_t *arg_pPos,
const _ix_cc_atmrx_vc_key_t *arg_pVcKey,
const ix_cc_atmsar_vc_handle_t arg_hVc)
{
    ix_uint32 *pEntry;
    switch (arg_pPos->tableId)
    {
    case _IX_CC_ATMRX_HASH_POS_PRI:
        pEntry=_ix_cc_atmrx_hash_pri_get_entry(arg_pContext, arg_pPos->idx);
        switch (arg_pPos->key)
        {
        case 1:
            _ix_cc_atmrx_pri_hash_set_ptr1(pEntry, arg_hVc);
            _ix_cc_atmrx_pri_hash_set_key1(pEntry, arg_pVcKey->key);
            _ix_cc_atmrx_pri_hash_set_keyExt1(pEntry, arg_pVcKey->keyExt);
            break;
        case 2:
            _ix_cc_atmrx_pri_hash_set_ptr2(pEntry, arg_hVc);
            _ix_cc_atmrx_pri_hash_set_key2(pEntry, arg_pVcKey->key);
            _ix_cc_atmrx_pri_hash_set_keyExt2(pEntry, arg_pVcKey->keyExt);
            break;
        default:
            break;
        }
        break;
    case _IX_CC_ATMRX_HASH_POS_SEC:
        pEntry=_ix_cc_atmrx_hash_sec_get_entry(arg_pContext, arg_pPos->idx);
        switch (arg_pPos->key)
        {
        case 1:
            _ix_cc_atmrx_sec_hash_set_ptr1(pEntry, arg_hVc);
            _ix_cc_atmrx_sec_hash_set_key1(pEntry, arg_pVcKey->key);
            _ix_cc_atmrx_sec_hash_set_keyExt1(pEntry, arg_pVcKey->keyExt);
            break;
        case 2:
            _ix_cc_atmrx_sec_hash_set_ptr2(pEntry, arg_hVc);
            _ix_cc_atmrx_sec_hash_set_key2(pEntry, arg_pVcKey->key);
            _ix_cc_atmrx_sec_hash_set_keyExt2(pEntry, arg_pVcKey->keyExt);
            break;
        case 3:
            _ix_cc_atmrx_sec_hash_set_ptr3(pEntry, arg_hVc);
            _ix_cc_atmrx_sec_hash_set_key3(pEntry, arg_pVcKey->key);
            _ix_cc_atmrx_sec_hash_set_keyExt3(pEntry, arg_pVcKey->keyExt);
            break;
        case 4:
            _ix_cc_atmrx_sec_hash_set_ptr4(pEntry, arg_hVc);
            _ix_cc_atmrx_sec_hash_set_key4(pEntry, arg_pVcKey->key);
            _ix_cc_atmrx_sec_hash_set_keyExt4(pEntry, arg_pVcKey->keyExt);
            break;
        default:
            break;
        }
        break;
    default:
        break;
    }
} /* end _ix_cc_atmrx_hash_set_vc */


/**
 * NAME: _ix_cc_atmrx_hash_get_vc
 *
 * DESCRIPTION:
 *    Function gets VC key from Hash Table entry
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN  arg_pPos : pointer to structure describing location.
 * @Param:  OUT arg_pVcKey : structure containing VC key (VPI/VCI/port)
 * 
 * @Return: None
 */
void _ix_cc_atmrx_hash_get_vc(
_ix_cc_atmrx_context_t *arg_pContext,
const _ix_cc_atmrx_hash_pos_t *arg_pPos,
_ix_cc_atmrx_vc_key_t *arg_pVcKey)
{
    ix_uint32 *pEntry;
    switch (arg_pPos->tableId)
    {
    case _IX_CC_ATMRX_HASH_POS_PRI:
        pEntry=_ix_cc_atmrx_hash_pri_get_entry(arg_pContext, arg_pPos->idx);
        switch (arg_pPos->key)
        {
        case 1:
            arg_pVcKey->key = _ix_cc_atmrx_pri_hash_get_key1(pEntry);
            arg_pVcKey->keyExt = _ix_cc_atmrx_pri_hash_get_keyExt1(pEntry);
            break;
        case 2:
            arg_pVcKey->key = _ix_cc_atmrx_pri_hash_get_key2(pEntry);
            arg_pVcKey->keyExt = _ix_cc_atmrx_pri_hash_get_keyExt2(pEntry);
            break;
        default:
            break;
        }
        break;
    case _IX_CC_ATMRX_HASH_POS_SEC:
        pEntry=_ix_cc_atmrx_hash_sec_get_entry(arg_pContext, arg_pPos->idx);
        switch (arg_pPos->key)
        {
        case 1:
            arg_pVcKey->key = _ix_cc_atmrx_sec_hash_get_key1(pEntry);
            arg_pVcKey->keyExt = _ix_cc_atmrx_sec_hash_get_keyExt1(pEntry);
            break;
        case 2:
            arg_pVcKey->key = _ix_cc_atmrx_sec_hash_get_key2(pEntry);
            arg_pVcKey->keyExt = _ix_cc_atmrx_sec_hash_get_keyExt2(pEntry);
            break;
        case 3:
            arg_pVcKey->key = _ix_cc_atmrx_sec_hash_get_key3(pEntry);
            arg_pVcKey->keyExt = _ix_cc_atmrx_sec_hash_get_keyExt3(pEntry);
            break;
        case 4:
            arg_pVcKey->key = _ix_cc_atmrx_sec_hash_get_key4(pEntry);
            arg_pVcKey->keyExt = _ix_cc_atmrx_sec_hash_get_keyExt4(pEntry);
            break;
        default:
            break;
        }
        break;
    default:
        break;
    }
} /* end _ix_cc_atmrx_hash_get_vc */


/**
 * NAME: _ix_cc_atmrx_hash_insert
 *
 * DESCRIPTION:
 *    This is a function inserts VC into hash tables supported by ATM RX.
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN  arg_hVC :  handle to created VC.
 * @Param:  IN  arg_pVcDescr : parameters describing requested VC.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully inserted
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 */
static ix_error
_ix_cc_atmrx_hash_insert(
_ix_cc_atmrx_context_t *arg_pContext,
const ix_cc_atmsar_vc_handle_t arg_hVC,
const ix_cc_atmsar_vc_descr_t *arg_pVcDescr)
{
    ix_uint32 *pEntry;
    ix_uint16 next;
    ix_uint16 priIdx;
    _ix_cc_atmrx_vc_key_t vcKey;
    _ix_cc_atmrx_vc_key_t vcKeyEmpty;
    _ix_cc_atmrx_hash_pos_t hashVcPos;    
    priIdx=_ix_cc_atmrx_hash_find_primary_idx(arg_pContext,&arg_pVcDescr->spec);
    _IX_CC_ATMRX_HASH_PREPARE_KEY(arg_pVcDescr->spec, vcKey);
    _IX_CC_ATMRX_DBG_HASH(("_ix_cc_atmrx_hash_insert: Start. priIdx=%d\n", priIdx));
    if (_ix_cc_atmrx_hash_search_vc_by_key(arg_pContext, &vcKey, priIdx, &hashVcPos) == 1)
    {
        return IX_ERROR_WARNING(IX_CC_ATMSAR_ERROR_INVALID_PARAM,
                                ("VC with the spec exist"));
    }
    if ((vcKey.key == 0) && (vcKey.keyExt == 0))
    {
        return IX_ERROR_WARNING(IX_CC_ATMSAR_ERROR_INVALID_PARAM,
                                ("Invalid VC spec"));
    }

    /* search free position */
    vcKeyEmpty.key=0;
    vcKeyEmpty.keyExt=0;
    if (_ix_cc_atmrx_hash_search_vc_by_key(arg_pContext, &vcKeyEmpty, priIdx, &hashVcPos) == 1)
    {
        /* free position fount */
        /* store vcPri index for release operation */
        arg_pContext->vcPriIdx[arg_hVC]=priIdx;

        /*set VC description into hash mechanism */

        _ix_cc_atmrx_hash_set_vc(arg_pContext, &hashVcPos, &vcKey, arg_hVC);
        if (hashVcPos.tableId == _IX_CC_ATMRX_HASH_POS_PRI)
        {
            pEntry=_ix_cc_atmrx_hash_pri_get_entry(arg_pContext, hashVcPos.idx);
            _ix_cc_atmrx_pri_hash_set_valid(pEntry, 1);
        }

        _IX_CC_ATMRX_DBG_HASH(("_ix_cc_atmrx_hash_insert: Vc added to %s hash entry with idx=%d and idx=%d\n",
                     (hashVcPos.tableId==_IX_CC_ATMRX_HASH_POS_PRI)?"primary":"secondary",
                     hashVcPos.idx, hashVcPos.key));
        return IX_SUCCESS;
    }
    
    /*
     * No free slot in current allocated hash entry.
     * Allocate a new entry,
     * put VC into allocated entry and
     * link to used hash entries.
     */
    
    hashVcPos.tableId = _IX_CC_ATMRX_HASH_POS_SEC;
    hashVcPos.key = 1;
    hashVcPos.idx = _ix_cc_atmrx_hash_sec_alloc_entry(arg_pContext);
    if (hashVcPos.idx == _IX_CC_ATMRX_HASH_SEC_IDX_INVALID)
    {
        return IX_ERROR_WARNING(IX_CC_ATMSAR_ERROR_NO_RESOURCES,
                                ("NO secondary hash entry"));
    }

    /* fill hash entry with new VC */
    _ix_cc_atmrx_hash_set_vc(arg_pContext, &hashVcPos, &vcKey, arg_hVC);
    
    /* set valid bit for the entry */
    pEntry=_ix_cc_atmrx_hash_sec_get_entry(arg_pContext, hashVcPos.idx);
    _ix_cc_atmrx_sec_hash_set_valid(pEntry, 1);
    
    /* store vcPri index for release operation */
    arg_pContext->vcPriIdx[arg_hVC]=priIdx;

    /* link a new hash entry into list */
    pEntry=_ix_cc_atmrx_hash_pri_get_entry(arg_pContext, priIdx);
    next=_ix_cc_atmrx_pri_hash_get_next(pEntry);
    if (next == _IX_CC_ATMRX_HASH_SEC_IDX_INVALID)
    {
        _IX_CC_ATMRX_DBG_HASH(("_ix_cc_atmrx_hash_insert: secondary link (idx=%d) to primary (idx=%d)\n",
                     hashVcPos.idx, priIdx));
        _ix_cc_atmrx_pri_hash_set_next(pEntry, hashVcPos.idx);
    }
    else
    {
        /* find last secondary hash entry */
        while (next != _IX_CC_ATMRX_HASH_SEC_IDX_INVALID)
        {
            _IX_CC_ATMRX_DBG_HASH(("_ix_cc_atmrx_hash_insert: secondary linked %d\n", next));
            priIdx=next;
            pEntry=_ix_cc_atmrx_hash_sec_get_entry(arg_pContext, priIdx);
            next=_ix_cc_atmrx_sec_hash_get_next(pEntry);
        }
        _IX_CC_ATMRX_DBG_HASH(("_ix_cc_atmrx_hash_insert: secondary link (idx=%d) to secondary (idx=%d)\n",
                     hashVcPos.idx, priIdx));
        _ix_cc_atmrx_sec_hash_set_next(pEntry, hashVcPos.idx);
    }
    return IX_SUCCESS;
} /* end _ix_cc_atmrx_hash_insert */


/**
 * NAME: _ix_cc_atmrx_hash_remove
 *
 * DESCRIPTION:
 *    This is a function removing VC fromo hash tables supported by ATM RX.
 *    The empty hash entry will be freed.
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN  arg_hVC :  handle to removed VC.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully inserted
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 */
static ix_error
_ix_cc_atmrx_hash_remove(
_ix_cc_atmrx_context_t *arg_pContext,
const ix_cc_atmsar_vc_handle_t arg_hVc)
{
    ix_uint32 *pEntry;
    ix_uint16 priIdx;
    ix_uint8  removeSecondaryFlag;
    ix_uint16 prevSecondary;
    ix_uint16 currentSecondary;
    ix_uint16 vcPtr;
    ix_uint16 next;
    _ix_cc_atmrx_vc_key_t vcKey;
    _ix_cc_atmrx_hash_pos_t hashVcPos;
    _ix_cc_atmrx_hash_pos_t hashVcPosLast;
    
    priIdx=arg_pContext->vcPriIdx[arg_hVc]; /* you have not to clear the variable */
    _IX_CC_ATMRX_DBG_HASH(("_ix_cc_atmrx_hash_remove: start hVc= %d, priIdx=%d\n", arg_hVc, priIdx));
    if (_ix_cc_atmrx_hash_search_vc_by_ptr(arg_pContext, arg_hVc, priIdx, &hashVcPos)==0)
    {
        return IX_ERROR_WARNING(IX_CC_ATMSAR_ERROR_NO_RESOURCES,
                                ("NO the vc handle in RX hash mechanism "));
    }

    /* remove vc description from slot in found hash entry */
    vcKey.key=0;
    vcKey.keyExt=0;
    _ix_cc_atmrx_hash_set_vc(arg_pContext, &hashVcPos, &vcKey, IX_CC_ATMSAR_VCID_INVALID);

    /* find last entry in hash list */
    pEntry=_ix_cc_atmrx_hash_pri_get_entry(arg_pContext, priIdx);
    next=_ix_cc_atmrx_pri_hash_get_next(pEntry);
    if (next == _IX_CC_ATMRX_HASH_SEC_IDX_INVALID)
    {
        /* check that primary is has keys set */
        if ((_ix_cc_atmrx_pri_hash_get_ptr1(pEntry) == IX_CC_ATMSAR_VCID_INVALID) &&
            (_ix_cc_atmrx_pri_hash_get_ptr2(pEntry) == IX_CC_ATMSAR_VCID_INVALID))
        {
            /* no key in primary entry */
            _IX_CC_ATMRX_DBG_HASH(("_ix_cc_atmrx_hash_remove: set primary entry as invalid\n"));
            _ix_cc_atmrx_pri_hash_set_valid(pEntry, 0);
        }
        _IX_CC_ATMRX_DBG_HASH(("_ix_cc_atmrx_hash_remove: exit primary\n"));
        return IX_SUCCESS;
    }

    /* Now, we start service secondary hash entries !!! */
    
    removeSecondaryFlag=0;
    currentSecondary=_IX_CC_ATMRX_HASH_SEC_IDX_INVALID;
    /* find last secondary hash entry and it predecessor */
    while (next != _IX_CC_ATMRX_HASH_SEC_IDX_INVALID)
    {
        _IX_CC_ATMRX_DBG_HASH(("_ix_cc_atmrx_hash_remove: found secondary %d\n", next));
        currentSecondary=next; /* remember next idx */
        pEntry=_ix_cc_atmrx_hash_sec_get_entry(arg_pContext, next);
        next=_ix_cc_atmrx_sec_hash_get_next(pEntry);
        if (next!= _IX_CC_ATMRX_HASH_SEC_IDX_INVALID)
        {
            /* unlink operation on secondary entry */
            removeSecondaryFlag=1;
            prevSecondary=currentSecondary;
        }
    }
        
    /*
     * 'pEntry' it points on last secondary hash entry
     * 'priIdx' contains index to last secondary hash entry
     * 'currentSecondary' - point to current secondary hash entry
     * 'removeSecondaryFlag' =
     *      1  - unlink from secondary entry
     *      0  - unlink from primary entry
     * 'prevSecondary' - points on secondary entry from which unlink
     *    operation will be performed. The value in the variable is valid
     *    if 'removeSecondaryFlag' is equal '1'.
     */

    /* check that removed VC was in last secondary entry */
    if ((hashVcPos.tableId != _IX_CC_ATMRX_HASH_POS_SEC) ||
        (hashVcPos.idx != currentSecondary))
    {
        /*
         * Removed Vc was not in last secondary hash entry. We need to move
         * any entry from last secondary hash entry to freed slot in hash
         * entry.
         * We try to find no empty slot in last secondary hash entry.
         */
        hashVcPosLast.tableId = _IX_CC_ATMRX_HASH_POS_SEC;
        hashVcPosLast.idx=currentSecondary;
        if ((vcPtr=_ix_cc_atmrx_sec_hash_get_ptr1(pEntry)) != IX_CC_ATMSAR_VCID_INVALID)
        {
            hashVcPosLast.key=1;
            vcKey.key=_ix_cc_atmrx_sec_hash_get_key1(pEntry);
            vcKey.keyExt=_ix_cc_atmrx_sec_hash_get_keyExt1(pEntry);
        }
        else if ((vcPtr=_ix_cc_atmrx_sec_hash_get_ptr2(pEntry)) != IX_CC_ATMSAR_VCID_INVALID)
        {
            hashVcPosLast.key=2;
            vcKey.key=_ix_cc_atmrx_sec_hash_get_key2(pEntry);
            vcKey.keyExt=_ix_cc_atmrx_sec_hash_get_keyExt2(pEntry);
        }
        else if ((vcPtr=_ix_cc_atmrx_sec_hash_get_ptr3(pEntry)) != IX_CC_ATMSAR_VCID_INVALID)
        {
            hashVcPosLast.key=3;
            vcKey.key=_ix_cc_atmrx_sec_hash_get_key3(pEntry);
            vcKey.keyExt=_ix_cc_atmrx_sec_hash_get_keyExt3(pEntry);
        }
        else if ((vcPtr=_ix_cc_atmrx_sec_hash_get_ptr4(pEntry)) != IX_CC_ATMSAR_VCID_INVALID)
        {
            hashVcPosLast.key=4;
            vcKey.key=_ix_cc_atmrx_sec_hash_get_key4(pEntry);
            vcKey.keyExt=_ix_cc_atmrx_sec_hash_get_keyExt4(pEntry);
        }
        else
        {
            _IX_CC_ATMRX_DBG_HASH(("_ix_cc_atmrx_hash_remove: ERROR!!! Something wrong with key\n"));
            hashVcPosLast.key=0;
            vcPtr=IX_CC_ATMSAR_VCID_INVALID;
            vcKey.key=0;
            vcKey.keyExt=0;
        }
            
        _IX_CC_ATMRX_DBG_HASH(("_ix_cc_atmrx_hash_remove: found key to move %d\n", hashVcPosLast.key));
        /*
         * move vc description from last secondary entry to free slot in any
         * previous hash entry (secondary or primary)
         */
        _ix_cc_atmrx_hash_set_vc(arg_pContext, &hashVcPos, &vcKey, vcPtr);
            
        /* remove vc description from last secondary hash entry */
        vcKey.key=0;
        vcKey.keyExt=0;
        _ix_cc_atmrx_hash_set_vc(arg_pContext, &hashVcPosLast, &vcKey, IX_CC_ATMSAR_VCID_INVALID);
            
    }
        
    if ((_ix_cc_atmrx_sec_hash_get_ptr1(pEntry) != IX_CC_ATMSAR_VCID_INVALID) ||
        (_ix_cc_atmrx_sec_hash_get_ptr2(pEntry) != IX_CC_ATMSAR_VCID_INVALID) ||
        (_ix_cc_atmrx_sec_hash_get_ptr3(pEntry) != IX_CC_ATMSAR_VCID_INVALID) ||
        (_ix_cc_atmrx_sec_hash_get_ptr4(pEntry) != IX_CC_ATMSAR_VCID_INVALID))
    {
        /*
         * Any slot in the last secondary hash entry used.
         * Don't free the entry
         */
        _IX_CC_ATMRX_DBG_HASH(("_ix_cc_atmrx_hash_remove: Exit. Last secondary is not empty\n"));
        return IX_SUCCESS;
    }
    
    /*
     * No key in last secondary hash entry:
     *  - set as invalid
     *  - unlink from previous hash entry
     *  - free secondary entry
     */
    _ix_cc_atmrx_sec_hash_set_valid(pEntry, 0);
    
    if (removeSecondaryFlag)
    {
        pEntry=_ix_cc_atmrx_hash_sec_get_entry(arg_pContext, prevSecondary);
        _ix_cc_atmrx_sec_hash_set_next(pEntry, _IX_CC_ATMRX_HASH_SEC_IDX_INVALID);
    }
    else
    {
        pEntry=_ix_cc_atmrx_hash_pri_get_entry(arg_pContext, priIdx);
        _ix_cc_atmrx_pri_hash_set_next(pEntry, _IX_CC_ATMRX_HASH_SEC_IDX_INVALID);
    }
    _ix_cc_atmrx_hash_sec_free_entry(arg_pContext, currentSecondary);
    _IX_CC_ATMRX_DBG_HASH(("_ix_cc_atmrx_hash_remove: Exit. Last secondary is freed, prev=%d, curr=%d\n",
                 prevSecondary, currentSecondary));
    return IX_SUCCESS;
} /* end _ix_cc_atmrx_hash_remove */


/******************************************************************************
 *                    VC INFO TABLE SUPPORT FUNCTIONS
 *****************************************************************************/

/*
 * It is entry description in VC info table:
 *
 *
 * |========================================================================|
 * |exp_SOP(1)|VC_alloc(1)|res(6)|VC_valid(1)|AAL_Type(3)|flist(4) | L2p(16)| LW0
 * |------------------------------------------------------------------------|
 * |                   crc_residue (32)                                     | LW1
 * |------------------------------------------------------------------------|
 * |EOP(1)|SOP(1)|  seg_cnt(6) |               ptr_to_curr_buf(24)          | LW2
 * |------------------------------------------------------------------------|
 * |EOP(1)|SOP(1)|  seg_cnt(6) |               ptr_to_SOP_buf(24)           | LW3
 * |------------------------------------------------------------------------|
 * |           sop_buf_size(16)       |            offset(16)               | LW4
 * |------------------------------------------------------------------------|
 * |           buffer_offset(16)      |        curr_pkt_size (16)           | LW5
 * |------------------------------------------------------------------------|
 * |drop_bit(1)|excep_bit(1)|res(2)| hdr_type(4)|prev_to_prev_buf_addr (24) | LW6
 * |------------------------------------------------------------------------|
 * |         prev_buf_size(16)        |bytes_remaining(8)| cell_count (8)   | LW7
 * |------------------------------------------------------------------------|
 * |                           prev_buf_handle(32)                          | LW8
 * |------------------------------------------------------------------------|
 * |                           receive_cell_count(32)                       | LW9
 * |------------------------------------------------------------------------|
 * |                           receive_mgmnt_cell_count(32)                 | LW10
 * |------------------------------------------------------------------------|
 * |                           receive_aal5_pdu_count(32)                   | LW11
 * |------------------------------------------------------------------------|
 * |                           receive_aal5_pdu_crc_count(32)               | LW12
 * |------------------------------------------------------------------------|
 * |                           receive_aal5_pdu_len_count(32)               | LW13
 * |========================================================================|
 *
 */

/*                               funcPfx,          varName,   lw, offset, mask */
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_vc_info,    suid,      9,  0,      0xFFFFFFFF);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_vc_info,    l2port,    9,  0,      0xFFFF);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_vc_info,    free_list, 0,  16,     0x0F);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_vc_info,    aal_type,  0,  20,     0x07);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_vc_info,    valid,     0,  23,     0x01);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_vc_info,    alloc,     0,  30,     0x01);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_vc_info,    exp_sop,   0,  31,     0x01);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_vc_info,    crc,       1,  0,      0x0FFFFFFFF);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_vc_info,    hdr_type,  6,  24,     0x0F);
ATMSAR_BITS_SUPPORT_FUNC_DEFINE(_ix_cc_atmrx_vc_info,    except,    6,  28,     0x01);

/**
 * NAME: _ix_cc_atmrx_vc_info_get_entry
 *
 * DESCRIPTION:
 *    Find pointer to entry in vc info table. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN  arg_entryId : Id of requested entry
 *
 * @Return: NULL if error occurs, otherwise pointer to secondary bucket.
 */
ix_uint32*
_ix_cc_atmrx_vc_info_get_entry(
_ix_cc_atmrx_context_t* arg_pContext,
ix_uint16 arg_entryId)
{
    return arg_pContext->pSymbolTable[_IX_CC_ATMRX_AAL5_RX_VC_INFO_SRAM_BASE_SYMBOL_ID]+((AAL5_RX_VC_INFO_ENTRY_SIZE>>2)*arg_entryId);
} /* end _ix_cc_atmrx_vc_info_get_entry */


/**
 * NAME: _ix_cc_atmrx_vc_info_init
 *
 * DESCRIPTION:
 *    Initialise VC info table. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully inserted
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to initialize
 */
ix_error
_ix_cc_atmrx_vc_info_init(
_ix_cc_atmrx_context_t* arg_pContext)
{
    ix_uint32 i;
    ix_uint32 *pEntry;

    for (i=0; i<arg_pContext->pluginCfg.numberOfVcs; i++)
    {
        pEntry=_ix_cc_atmrx_vc_info_get_entry(arg_pContext, i);
        _ix_cc_atmrx_vc_info_set_valid(pEntry, 1); /* entry is invalid */
    }
    return IX_SUCCESS;
} /* end _ix_cc_atmrx_vc_info_init */


/**
 * NAME: _ix_cc_atmrx_vc_info_insert
 *
 * DESCRIPTION:
 *    This is a function set VC as valid and set initial parameters for the
 *    VC in VC info table.
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN  arg_hVC :  handle to created VC.
 * @Param:  IN  arg_pVcDescr : parameters describing requested VC.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully inserted
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 */
static ix_error
_ix_cc_atmrx_vc_info_insert(
_ix_cc_atmrx_context_t* arg_pContext,
const ix_cc_atmsar_vc_handle_t arg_hVc,
const ix_cc_atmsar_vc_descr_t* arg_pVcDescr)
{
    ix_uint32 *pEntry;
    pEntry=_ix_cc_atmrx_vc_info_get_entry(arg_pContext, arg_hVc);
    
    ix_ossl_memset((void*)pEntry, 0, AAL5_RX_VC_INFO_ENTRY_SIZE);
    
    switch (arg_pVcDescr->aalType)
    {
    case IX_CC_ATMSAR_AAL5:
        _ix_cc_atmrx_vc_info_set_aal_type(pEntry, 0);
        break;
    case IX_CC_ATMSAR_AAL34:
        _ix_cc_atmrx_vc_info_set_aal_type(pEntry, 1);
        break;
    case IX_CC_ATMSAR_AAL2: 
        _ix_cc_atmrx_vc_info_set_aal_type(pEntry, 4);
        break;
    case IX_CC_ATMSAR_AAL0:
    case IX_CC_ATMSAR_AAL1:
    case IX_CC_ATMSAR_AALRAW:
    default:
        return IX_ERROR_WARNING(IX_CC_ATMSAR_ERROR_INVALID_PARAM,
                                ("AAL type is not serviced"));
    }
    _ix_cc_atmrx_vc_info_set_except(pEntry,
                                    (arg_pVcDescr->packetPath == IX_CC_ATMSAR_PACKET_PATH_REGULAR)?0:1);
    _ix_cc_atmrx_vc_info_set_l2port(pEntry, arg_hVc);/*arg_pVcDescr->l2port);*/
    _ix_cc_atmrx_vc_info_set_free_list(pEntry, arg_pVcDescr->freeListId);
    _ix_cc_atmrx_vc_info_set_hdr_type(pEntry, arg_pVcDescr->hdrType);
    _ix_cc_atmrx_vc_info_set_exp_sop(pEntry, 1);
    _ix_cc_atmrx_vc_info_set_valid(pEntry, 1); /* entry is not valid yet - 'VC update' sets this entry as valid */
    if (arg_pVcDescr->aalType == IX_CC_ATMSAR_AAL2)
    {
	_ix_cc_atmrx_vc_info_set_crc(pEntry, arg_hVc);
    }
    else
    {
	_ix_cc_atmrx_vc_info_set_crc(pEntry, 0x0FFFFFFFF);
    }
    return IX_SUCCESS;
} /* end _ix_cc_atmrx_vc_info_insert */


/**
 * NAME: _ix_cc_atmrx_vc_info_remove
 *
 * DESCRIPTION:
 *    The function sets VC as invalid.
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN  arg_hVC :  handle to removed VC.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully inserted
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 */
static ix_error
_ix_cc_atmrx_vc_info_remove(
_ix_cc_atmrx_context_t *arg_pContext,
const ix_cc_atmsar_vc_handle_t arg_hVc)
{
    ix_uint32 *pEntry;
    pEntry=_ix_cc_atmrx_vc_info_get_entry(arg_pContext, arg_hVc);
    /*
     * TBD. Warning. Who does request alloc bit. It is set by control plane
     * and it is clear by control plane
     */
    /*
      while (_ix_cc_atmrx_vc_info_get_alloc(pEntry)==0)
      {
      ix_ossl_sleep_tick(2);
      }
    */
    _ix_cc_atmrx_vc_info_set_valid(pEntry, 1); /* VC is invalid */
    return IX_SUCCESS;
} /* end _ix_cc_atmrx_vc_info_remove */


/******************************************************************************
 *                       AAL2 SUPPORT FUNCTIONS
 *****************************************************************************/

#ifdef AAL2_INCLUDED
#ifdef VC_LOOKUP_IS_DEFINED

ATMSAR_BITS_SUPPORT_FUNC_DEFINE(atmrx_vc_lookup,    service_type,    0,  29,     0x07);

/**
 * NAME: _ix_cc_atmrx_vc_lookup_get_entry
 *
 * DESCRIPTION:
 *    Find pointer to entry in vc lookup table. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN  arg_entryId : Id of requested entry
 *
 * @Return: NULL if error occurs, otherwise pointer to vc lookup table entry.
 */
ix_uint32*
_ix_cc_atmrx_vc_lookup_get_entry(
_ix_cc_atmrx_context_t* arg_pContext,
ix_uint16 arg_entryId)
{
    return arg_pContext->pSymbolTable[_IX_CC_ATMRX_VC_LOOKUP_TABLE_BASE_SYMBOL_ID]
        + (arg_entryId*VC_LOOKUP_TABLE_ENTRY_SIZE);
} /* end _ix_cc_atmrx_vc_lookup_get_entry */


/**
 * NAME: _ix_cc_atmrx_vc_lookup_init
 *
 * DESCRIPTION:
 *    Initialise VC lookup table. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 */
ix_error _ix_cc_atmrx_vc_lookup_init(
_ix_cc_atmrx_context_t* arg_pContext)
{
    ix_uint32* pEntry;
    ix_uint16 i;

    for (i=0; i<arg_pContext->pluginCfg.numberOfVcs; i++)
    {
        pEntry=_ix_cc_atmrx_vc_lookup_get_entry(arg_pContext, i);
        atmrx_vc_lookup_set_service_type(pEntry, 2);
    }

    return IX_SUCCESS;
} /* end _ix_cc_atmrx_vc_lookup_init */
#endif /* VC_LOOKUP_IS_DEFINED */
#endif /* AAL2_INCLUDED */

/******************************************************************************
 *                       ATM SAR PLUG-IN FUNCTIONS
 *****************************************************************************/


/**
 * NAME: _ix_cc_atmrx_plugin_vc_create
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "VC create" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_hVC :  handle to created VC.
 * @Param:  IN  arg_pVcDescr : parameters describing requested VC.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 */
ix_error
_ix_cc_atmrx_plugin_vc_create(
void* arg_pInstance,
const ix_cc_atmsar_vc_handle_t arg_hVc,
const ix_cc_atmsar_vc_descr_t* arg_pVcDescr)
{
    ix_uint16 i;
    ix_counter_64bit_handle *p64BitCountersHdls;
    _ix_cc_atmrx_context_t *pCtx=arg_pInstance;
    _IX_CC_ATMRX_DBG_VC(("_ix_cc_atmrx_plugin_vc_create\n"));

    /*
     * First prepare VC info data and after it insert VC into
     * into hash table.
     */
    IX_ERROR_CRT(_ix_cc_atmrx_vc_info_insert(pCtx, arg_hVc, arg_pVcDescr),
                 IX_CC_ATMSAR_ERROR_INVALID_PARAM,
                 IX_ERROR_LEVEL_GLOBAL);
    IX_ERROR_CRT(_ix_cc_atmrx_hash_insert(pCtx, arg_hVc, arg_pVcDescr),
                 IX_CC_ATMSAR_ERROR_INVALID_PARAM,
                 IX_ERROR_LEVEL_GLOBAL);

    if (pCtx->counters)
    {
        /* clear statistic counters for VC */
        p64BitCountersHdls=&(pCtx->hVcCounters[ATMRX_VC_STAT_COUNTERS_NO*arg_hVc]);
        for (i=0; i<ATMRX_VC_STAT_COUNTERS_NO; i++)
        {
            IX_ERROR_CRT(ix_rm_counter_64bit_set_value(*(p64BitCountersHdls+i), 0),
                         IX_CC_ATMRX_ERROR_RM_SET_64BIT_COUNTER, 
                         IX_ERROR_LEVEL_REMOTE);
        }
    }
    
    return IX_SUCCESS;
} /* end _ix_cc_atmrx_plugin_vc_create */

/**
 * NAME: _ix_cc_atmrx_plugin_vc_update
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "VC update" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_hVC :  handle to created VC.
 * @Param:  IN  arg_pVcDescr : parameters describing requested VC.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 */
ix_error
_ix_cc_atmrx_plugin_vc_update(
void* arg_pInstance,
const ix_cc_atmsar_vc_handle_t arg_hVc,
const ix_cc_atmsar_update_data_t* arg_pUpdate)
{
    ix_uint32 *pEntry;
    _ix_cc_atmrx_context_t *pCtx=arg_pInstance;
    ix_cc_atmrx_update_data_t *pRxUpdateData = (ix_cc_atmrx_update_data_t*)arg_pUpdate;
    
    _IX_CC_ATMRX_DBG_VC(("_ix_cc_atmrx_plugin_vc_update lw0 %lx lw1 %lx\n",
                         arg_pUpdate->lw0,
                         arg_pUpdate->lw1));

    pEntry=_ix_cc_atmrx_vc_info_get_entry(pCtx, arg_hVc);
    
    _ix_cc_atmrx_vc_info_set_valid(pEntry, 0);

#ifdef OPTIONAL_L2PORT

    _ix_cc_atmrx_vc_info_set_l2port(pEntry, pRxUpdateData->lw0.l2port);
    _ix_cc_atmrx_vc_info_set_hdr_type(pEntry, pRxUpdateData->lw1.hdrType);
    
#elif defined(OPTIONAL_SUID)

    _ix_cc_atmrx_vc_info_set_suid(pEntry, pRxUpdateData->lw0.suId);
    
#else
#error "OPTIONAL_L2PORT or OPTIONAL_SUID should be defined."    
#endif /* OPTIONAL_L2_PORT */
    
    
    return IX_SUCCESS;
} /* end _ix_cc_atmrx_plugin_vc_update */

/**
 * NAME: _ix_cc_atmrx_plugin_vc_remove
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "VC remove" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_hVc : handle to VC to be removed
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - vc successfully removed
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 */
ix_error
_ix_cc_atmrx_plugin_vc_remove(
void* arg_pInstance,
ix_cc_atmsar_vc_handle_t arg_hVc)
{
    _ix_cc_atmrx_context_t *pCtx=arg_pInstance;

    _IX_CC_ATMRX_DBG_VC(("_ix_cc_atmrx_plugin_vc_remove hVC:%d\n", arg_hVc));
    /*
     * First remove VC from hash mechanism and after it clear parameters
     * in VC info table.
     */
    IX_ERROR_CRT(_ix_cc_atmrx_hash_remove(pCtx, arg_hVc),
                 IX_CC_ATMSAR_ERROR_INVALID_PARAM,
                 IX_ERROR_LEVEL_GLOBAL);
    IX_ERROR_CRT(_ix_cc_atmrx_vc_info_remove(pCtx, arg_hVc),
                 IX_CC_ATMSAR_ERROR_INVALID_PARAM,
                 IX_ERROR_LEVEL_GLOBAL);
    return IX_SUCCESS;
} /* end _ix_cc_atmrx_plugin_vc_remove */


/**
 * NAME: ix_cc_atmrx_vc_show_stats
 *
 * DESCRIPTION:
 *    The function shows statistic counters for VC.
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN  arg_hVc : handle to VC to be removed
 * 
 * @Return: none
 */
void
ix_cc_atmrx_vc_show_stats(
_ix_cc_atmrx_context_t* arg_pContext,
ix_cc_atmsar_vc_handle_t arg_hVc)
{
    ix_uint16 i;
    ix_uint64 tmp[ATMRX_VC_STAT_COUNTERS_NO];
    ix_uint32 *ptr;
    ix_counter_64bit_handle *p64BitCountersHdls;

    if (arg_pContext == NULL)
    {
        _IX_CC_ATMRX_DBG_VC(("ERROR. Pointer for ATM RX core component context is null\n"));
        return;
    }

    if (arg_pContext->counters)
    {

        p64BitCountersHdls=&(arg_pContext->hVcCounters[ATMRX_VC_STAT_COUNTERS_NO*arg_hVc]);
        _IX_CC_ATMRX_DBG_VC(("ATM RX: VC[%-5d] STATS\n", arg_hVc));
        if (arg_hVc>=arg_pContext->pluginCfg.numberOfVcs)
        {
            _IX_CC_ATMRX_DBG_VC(("Wrong VC ID. Statistic is not presented\n"));
            return;
        }
        
        for (i=0; i<ATMRX_VC_STAT_COUNTERS_NO; i++)
        {
            if (ix_rm_counter_64bit_get_value(*(p64BitCountersHdls+i), &(tmp[i])) != IX_SUCCESS)
            {
                _IX_CC_ATMRX_DBG_VC(("Error during reading %d counter\n", i));
                return;
            }
        }
        ptr = (ix_uint32*)tmp;
        _IX_CC_ATMRX_DBG_VC(("  cell count         %04lx%04lx\n", ptr[0],ptr[1]));
        _IX_CC_ATMRX_DBG_VC(("  mgmnt. cell count  %04lx%04lx\n", ptr[2],ptr[3]));
        _IX_CC_ATMRX_DBG_VC(("  AAL5 PDU count     %04lx%04lx\n", ptr[4],ptr[5]));
        _IX_CC_ATMRX_DBG_VC(("  AAL5 PDU CRC count %04lx%04lx\n", ptr[6],ptr[7]));
        _IX_CC_ATMRX_DBG_VC(("  AAL5 PDU len count %04lx%04lx\n", ptr[8],ptr[9]));
    }
}


/**
 * NAME: _ix_cc_atmrx_plugin_get_vc_stats
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "GET VC STATS" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_hVc : handle to VC to be removed
 * @Param:  OUT arg_pLookupResult : pointer to statistic structure
 *                  where the requested data will be stored
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - statistics successfully retrieved
 *    IX_CC_INVALID_PORT - specified ATM port does not exist 
 */
ix_error
_ix_cc_atmrx_plugin_get_vc_stats(
void* arg_pInstance,
ix_cc_atmsar_vc_handle_t arg_hVc,
ix_cc_atmsar_vc_stats_t *arg_pLookupResult)
{
    ix_uint64 tmp;
    ix_counter_64bit_handle *p64BitCountersHdls;
    _ix_cc_atmrx_context_t *pCtx=arg_pInstance;

    p64BitCountersHdls=&(pCtx->hVcCounters[ATMRX_VC_STAT_COUNTERS_NO*arg_hVc]);
    _IX_CC_ATMRX_DBG_VC(("_ix_cc_atmrx_plugin_get_vc_stats\n"));

    if (pCtx->counters)
    {
        
        IX_ERROR_CRT(ix_rm_counter_64bit_get_value(*(p64BitCountersHdls+_IX_CC_ATMRX_VC_C64BITS_IDX_CELL_COUNT),
                                                   &(arg_pLookupResult->rx.cells)),
                     IX_CC_ATMRX_ERROR_RM_GET_64BIT_COUNTER, 
                     IX_ERROR_LEVEL_REMOTE);
        
        IX_ERROR_CRT(ix_rm_counter_64bit_get_value(*(p64BitCountersHdls+_IX_CC_ATMRX_VC_C64BITS_IDX_MGT_CELL_COUNT),
                                                   &(tmp)),
                     IX_CC_ATMRX_ERROR_RM_GET_64BIT_COUNTER, 
                     IX_ERROR_LEVEL_REMOTE);
        arg_pLookupResult->rx.cells+=tmp;
        
        IX_ERROR_CRT(ix_rm_counter_64bit_get_value(*(p64BitCountersHdls + _IX_CC_ATMRX_VC_C64BITS_IDX_AAL5_PDU_COUNT),
                                                   &(arg_pLookupResult->rx.packets)),
                     IX_CC_ATMRX_ERROR_RM_GET_64BIT_COUNTER, 
                     IX_ERROR_LEVEL_REMOTE);
        
        IX_ERROR_CRT(ix_rm_counter_64bit_get_value(*(p64BitCountersHdls + _IX_CC_ATMRX_VC_C64BITS_IDX_AAL5_PDU_CRC_COUNT),
                                                   &(arg_pLookupResult->rx.discards)),
                     IX_CC_ATMRX_ERROR_RM_GET_64BIT_COUNTER, 
                     IX_ERROR_LEVEL_REMOTE);
        
        IX_ERROR_CRT(ix_rm_counter_64bit_get_value(*(p64BitCountersHdls + _IX_CC_ATMRX_VC_C64BITS_IDX_AAL5_PDU_LEN_COUNT),
                                                   &(tmp)),
                     IX_CC_ATMRX_ERROR_RM_GET_64BIT_COUNTER, 
                     IX_ERROR_LEVEL_REMOTE);
    }
    
    arg_pLookupResult->rx.discards+=tmp;
    return IX_SUCCESS;
} /* end _ix_cc_atmrx_plugin_get_vc_stats */


/**
 * NAME: _ix_cc_atmrx_plugin_port_create
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "PORT CREATE" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_pPortDescr : pointer to data describing parameters
 *                  for requested port
 * @Param:  OUT arg_hPort : ATM port handle
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 */
ix_error _ix_cc_atmrx_plugin_port_create(
void* arg_pInstance,
ix_cc_atmsar_port_handle_t arg_hPort,
const ix_cc_atmsar_port_descr_t* arg_pPortDescr)
{
    _IX_CC_ATMRX_DBG_PORT(("_ix_cc_atmrx_plugin_port_create\n"));
    return IX_SUCCESS;
} /* end _ix_cc_atmrx_plugin_port_create */


/**
 * NAME: _ix_cc_atmrx_plugin_port_remove
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "PORT REMOVE" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_port : ATM port to be removed
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - port successfully removed
 *    IX_CC_INVALID_PORT - specified port does not exist
 */
ix_error _ix_cc_atmrx_plugin_port_remove(
void* arg_pInstance,
ix_cc_atmsar_port_handle_t arg_port)
{
    _IX_CC_ATMRX_DBG_PORT(("_ix_cc_atmrx_plugin_port_remove\n"));
    return IX_SUCCESS;
} /* end _ix_cc_atmrx_plugin_port_remove */


/**
 * NAME: ix_cc_atmrx_port_show_stats
 *
 * DESCRIPTION:
 *    The function shows statistic counters for VC.
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  IN  arg_port : ATM port to be removed
 * 
 * @Return: none
 */
void ix_cc_atmrx_port_show_stats(
_ix_cc_atmrx_context_t *arg_pContext,
ix_cc_atmsar_port_handle_t arg_port)
{
    ix_uint16 i;
    ix_uint64 tmp[NO_PER_PORT_STAT_COUNTERS];
    ix_uint32 *ptr;
    ix_counter_64bit_handle *p64BitCountersHdls;

    _IX_CC_ATMRX_DBG_PORT(("ATM RX: PORT[%-5d] STATS\n", arg_port));
    if (arg_port>=arg_pContext->pluginCfg.numberOfPorts)
    {
        _IX_CC_ATMRX_DBG_PORT(("Wrong Port ID. Statistic is not presented\n"));
        return;
    }

    if (arg_pContext == NULL)
    {
        _IX_CC_ATMRX_DBG_PORT(("ATM RX context is null\n"));
        return;
    }

    if (arg_pContext->counters)
    {
            
        p64BitCountersHdls=&(arg_pContext->hPortCounters[NO_PER_PORT_STAT_COUNTERS*arg_port]);
        for (i=0; i<NO_PER_PORT_STAT_COUNTERS; i++)
        {
            if (ix_rm_counter_64bit_get_value(*(p64BitCountersHdls+i), &(tmp[i])) != IX_SUCCESS)
            {
                _IX_CC_ATMRX_DBG_VC(("Error during reading %d counter\n", i));
                return;
            }
        }
        ptr = (ix_uint32*)tmp;
        _IX_CC_ATMRX_DBG_PORT(("  AAL5_PKT_COUNT       %04lx%04lx\n", ptr[0],ptr[1]));
        _IX_CC_ATMRX_DBG_PORT(("  AAL5_PKT_DROP_COUNT  %04lx%04lx\n", ptr[2],ptr[3]));
        _IX_CC_ATMRX_DBG_PORT(("  AAL5_HW_ERROR        %04lx%04lx\n", ptr[4],ptr[5]));
        _IX_CC_ATMRX_DBG_PORT(("  AAL5_VC_ERROR        %04lx%04lx\n", ptr[6],ptr[7]));
    }
    
} /* end ix_cc_atmrx_port_show_stats */


/**
 * NAME: _ix_cc_atmrx_plugin_get_port_stats
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "GET PORT STATS" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_port : ATM port ID
 * @Param:  OUT arg_pLookupResult : pointer to ix_cc_sar_port_stats structure
 *                  where the requested data will be stored
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - statistics successfully retrieved
 *    IX_CC_INVALID_PORT - specified ATM port does not exist 
 */
ix_error
_ix_cc_atmrx_plugin_get_port_stats(
void* arg_pInstance,
ix_cc_atmsar_port_handle_t arg_port,
ix_cc_atmsar_port_stats_t *arg_pLookupResult)
{
    ix_counter_64bit_handle *p64BitCountersHdls;
    _ix_cc_atmrx_context_t *pCtx=arg_pInstance;

    _IX_CC_ATMRX_DBG_PORT(("_ix_cc_atmrx_plugin_get_port_stats\n"));
    
    p64BitCountersHdls=&(pCtx->hPortCounters[NO_PER_PORT_STAT_COUNTERS*arg_port]);

    if (pCtx->counters)
    {

        IX_ERROR_CRT(ix_rm_counter_64bit_get_value(*(p64BitCountersHdls+_IX_CC_ATMRX_PORT_C64BITS_IDX_AAL5_PKT_COUNT),
                                                   &(arg_pLookupResult->rx.packets)),
                     IX_CC_ATMRX_ERROR_RM_GET_64BIT_COUNTER, 
                     IX_ERROR_LEVEL_REMOTE);
        
        IX_ERROR_CRT(ix_rm_counter_64bit_get_value(*(p64BitCountersHdls + _IX_CC_ATMRX_PORT_C64BITS_IDX_AAL5_PKT_DROP_COUNT),
                                                   &(arg_pLookupResult->rx.discards)),
                     IX_CC_ATMRX_ERROR_RM_GET_64BIT_COUNTER, 
                     IX_ERROR_LEVEL_REMOTE);
        
        IX_ERROR_CRT(ix_rm_counter_64bit_get_value(*(p64BitCountersHdls + _IX_CC_ATMRX_PORT_C64BITS_IDX_AAL5_VC_ERROR),
                                                   &(arg_pLookupResult->rx.cells)),
                     IX_CC_ATMRX_ERROR_RM_GET_64BIT_COUNTER, 
                     IX_ERROR_LEVEL_REMOTE);
        
        IX_ERROR_CRT(ix_rm_counter_64bit_get_value(*(p64BitCountersHdls + _IX_CC_ATMRX_PORT_C64BITS_IDX_AAL5_HW_ERROR),
                                                   &(arg_pLookupResult->rx.cellErrors)),
                     IX_CC_ATMRX_ERROR_RM_GET_64BIT_COUNTER, 
                     IX_ERROR_LEVEL_REMOTE);
        
    }
    
    return IX_SUCCESS;
} /* end _ix_cc_atmrx_plugin_get_port_stats */



