/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      ix_cc_csix_rx.h
 *
 * = DESCRIPTION
 *      This file will describe the common definitions that will be exported 
 *      by the Csix Rx Core Component.
 *
 *
 * = CHANGE HISTORY
 *      8/19/2002 - Created.
 *
 * ============================================================================
 * $Id: ix_cc_csix_rx.h,v 1.17 2003/09/25 20:35:53 ktseng Exp $
 */

#if !defined(__IX_CC_CSIX_RX_H__)
#define __IX_CC_CSIX_RX_H__


/**
 * System defined include files required.
 */
#include "ix_types.h"
#include "ix_error.h"

#ifndef IX_EXCLUDE_CCI
#include "ix_cci.h"
#endif




#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */




/**
 * The following defines the symbols to be patched into the microblock.
 */
#if (_IX_HARDWARE_TYPE_ == _IX_HW_2800_)
#define CSIX_RX_NUM_SYMBOLS_PATCHED     0 /* there is not a symbol to be patched for 2800 platform */
#else /* (_IX_HARDWARE_TYPE_ == _IX_HW_2800_) */

#ifdef MICROBLOCK_COUNTERS
#define CSIX_RX_NUM_SYMBOLS_PATCHED     2
#else
#define CSIX_RX_NUM_SYMBOLS_PATCHED     1
#endif /* MICROBLOCK_COUNTERS */

#endif /* (_IX_HARDWARE_TYPE_ == _IX_HW_2800_) */

#define SYMBOL_CSIX_RX_CTX_BASE         "CSIX_RX_CTX_SRAM_BASE"
#define SYMBOL_CSIX_RX_COUNTERS_BASE    "CSIX_RX_COUNTERS_SRAM_BASE"


#ifdef IX_INCLUDE_REGISTRY

 /**
 * The following defines the static properties in repository
 */
#define PROP_CSIX_RX_MEV2_NUM_MASK      "CSIX_RX/CSIX_RX_MEV2_NUM_MASK"

#else

 /**
 * The following defines static configuration data.
 * Csix Rx uses these data when system repository is not included
 * in the system.
 */
#if (_IX_HARDWARE_TYPE_ == _IX_HW_2800_)
#define CSIX_RX_MEV2_NUM_MASK             0x0C00     /* ME 10 and 11 */
#else /* (_IX_HARDWARE_TYPE_ == _IX_HW_2800_) */
#define CSIX_RX_MEV2_NUM_MASK             0x00000001 
#endif /* (_IX_HARDWARE_TYPE_ == _IX_HW_2800_) */

#endif


/**
 * Various definitions for maximum number of VoQs, microblock context 
 * memory size, and SRAM channel number for the context memory.
 */
#define CC_CSIX_RX_NUM_VOQS             1024
#define CC_CSIX_RX_NUM_BYTES_PER_CTX_ENTRY      64
#define CC_CSIX_RX_SIZE_CTX_MEM         (CC_CSIX_RX_NUM_VOQS * CC_CSIX_RX_NUM_BYTES_PER_CTX_ENTRY)
#define CC_CSIX_RX_DEFAULT_SRAM_CH_NUM  0

#if (_IX_HARDWARE_TYPE_ == _IX_HW_2800_)
#define IX_CC_CSIX_RX_MAX_ME_NUM        16 /* max. number of MEv2 */
#else
#define IX_CC_CSIX_RX_MAX_ME_NUM        8 /* max. number of MEv2 */
#endif /* (_IX_HARDWARE_TYPE_ == _IX_HW_2800_) */



/**
 * Csix Rx error codes
 */
enum ix_cc_csix_rx_error {
    IX_CC_CSIX_RX_ERROR_USER_CB = IX_ERROR_MAKE_GROUP(CC_CSIX_RX),
    IX_CC_CSIX_RX_ERROR_CCI_REG_HDLR, 
    IX_CC_CSIX_RX_ERROR_CCI_REM_HDLR, 
    IX_CC_CSIX_RX_ERROR_RM_PATCH_SYMBOL, 
    IX_CC_CSIX_RX_ERROR_RM_GET_PHYS_OFFSET, 
    IX_CC_CSIX_RX_ERROR_RM_FREE_BUF, 
    IX_CC_CSIX_RX_ERROR_RM_GET_64BIT_COUNTER,
    IX_CC_CSIX_RX_ERROR_RM_REG_HDLR, 
    IX_CC_CSIX_RX_ERROR_RM_REM_HDLR, 
    IX_CC_CSIX_RX_ERROR_MSUP_SEND_REPLY,
    IX_CC_CSIX_RX_ERROR_MSUP_SEND_MSG,
	IX_CC_CSIX_RX_ERROR_MSUP_EXTRACT_MSG,
    IX_CC_CSIX_RX_ERROR_OSSL_MUTEX_INIT,
    IX_CC_CSIX_RX_ERROR_OSSL_MUTEX_FINI,
    IX_CC_CSIX_RX_ERROR_OSSL_MUTEX_LOCK 
};



/**
 * defines for message types
 */
typedef enum ix_e_cc_csix_rx_msg_type
{
    IX_CC_CSIX_RX_MSG_GET_STATISTICS_INFO = 0
} ix_cc_csix_rx_msg_type;



/**
 * defines for statistics counters
 */


typedef enum ix_e_cc_csix_rx_statistics_info
{
    /**
     * Per VOQ Csix Rx counters.
     */
	/*From CSIX Rx u-code HLD
	Counter	Offset from base for VOQ	
	Packets received	0x0	
	Bytes received	0x4	
	Cframes received	0x8	
	Packets Dropped	0xc	*/

    IX_CC_CSIX_RX_PACKETS = 0,  /* Rx packet count */
    IX_CC_CSIX_RX_BYTES,        /* Rx byte count */
    IX_CC_CSIX_RX_CFRAMES,      /* Rx C-Frame count */
	IX_CC_CSIX_RX_DROP_PACKETS, /* Dropped packets*/
    IX_CC_CSIX_RX_ALL_COUNTERS, /* All above counters of a given VoQ */

    IX_CC_CSIX_RX_COUNTER_LAST  /* End of Csix Rx counter definition */
} ix_cc_csix_rx_statistics_info;

/* 32-bit counter overflow time ??*/
#define CC_CSIX_RX_32BIT_COUNTER_OVERFLOW_TIME  10000
/* For IXP2400, calculation is for byte counter (packet counter can be read less frequently beacause they overflow 
 * less frequently)
 * 10 sec <= 2^32 / ((2.488Gbps/ (8*46)) * (64+16)) 
 * No of packets coming in per second = 2.488 Gbs/8*46. Min packet size for POS is 46
 This is multiplied by (46+16) as CSIX Tx tacks on 16 byte header and CSIX Rx takes off the same header 
 before sending it on. So, within CSIX, the calculation for byte counter overflow is as above*/
 



/**
 * The following define API-level data structures.
 */
typedef struct ix_s_cc_csix_rx_statistics_info_context
{
    ix_cc_csix_rx_statistics_info entity;
    ix_uint32 index;
    void *pUserContext;
} ix_cc_csix_rx_statistics_info_context;


typedef struct ix_s_cc_csix_rx_statistics_info_data
{
    ix_uint32 dataLength; /* length in bytes of the data buffer */
    void *pDataBuffer;    /* buffer to store the statistics data */
} ix_cc_csix_rx_statistics_info_data;



/**
 * prototypes for async messaging user callbacks
 */
typedef ix_error (*ix_cc_csix_rx_cb_get_statistics_info)(
    ix_error arg_Result,
    void *arg_pContext,
    ix_uint64 *arg_pBuffer);



/**
 * Prototypes for interface functions.
 */
ix_error ix_cc_csix_rx_init(
    ix_cc_handle arg_CcHandle, 
    void **arg_ppContext);

ix_error ix_cc_csix_rx_fini(
    ix_cc_handle arg_CcHandle, 
    void *arg_pContext);

ix_error ix_cc_csix_rx_msg_handler(
    ix_buffer_handle arg_Msg, 
    ix_uint32 arg_UserData,
    void *arg_pContext);

ix_error ix_cc_csix_rx_async_get_statistics_info(
    ix_cc_csix_rx_statistics_info_context *arg_pMgInfoContext, 
    ix_cc_csix_rx_cb_get_statistics_info arg_Callback);

ix_error ix_cc_csix_rx_get_statistics_info(
    ix_cc_csix_rx_statistics_info arg_Entity,
    ix_uint32 arg_Index,
    ix_cc_csix_rx_statistics_info_data *arg_pBuffer,
    void *arg_pContext);



#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_CSIX_RX_H__) */

