/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2X00 Network Processor, Release 5
 *
 * = FILENAME
 *      ix_cc_csix_rx_hdlr.c
 *
 * = DESCRIPTION
 *      This file defines various handler routines.
 *
 *
 * = CHANGE HISTORY
 *      8/22/2002 - Created.
 *
 * ============================================================================
 * $Id: ix_cc_csix_rx_hdlr.c,v 1.5 2002/10/17 19:38:55 sharma Exp $
 */

/**
 * System defined include files required.
 */

/**
 * User defined include files required.
 */
#include "ix_cc_error.h"

#include "ix_types.h"
#include "ix_error.h"
#include "ix_cc.h"
#include "bindings.h"
#include "ix_cc_properties.h"

#include "ix_rm.h"
#include "ix_ossl.h"

#ifndef IX_EXCLUDE_CCI
#include "ix_cci.h"
#endif

#include "cc/ix_cc_msup.h"
#include "cc/ix_cc_csix_rx.h"
#include "cc/internal/ix_cc_csix_rx_int.h"



/**
 * Variable declarations global to this file only.  Extern are followed
 * by static variables.
 */



/**
 * Pre-processor symbols and macros used in this file.
 */


/**
 * Extern function prototypes and variables
 */



/**
 * Static function prototypes
 */



/**
 * function definitions.
 */



/**
 * NAME: ix_cc_csix_rx_msg_handler
 * 
 * DESCRIPTION: This function is the common message handler routine for
 *              CSIX Tx CC.
 * 
 * @Param: arg_Msg - IN
 * @Param: arg_UserData - IN
 * @Param: arg_pContext - IN: pointer to the component context allocated
 *                            in the component's init function. 
 * 
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */
ix_error ix_cc_csix_rx_msg_handler(
    ix_buffer_handle arg_Msg, 
    ix_uint32 arg_UserData,
    void *arg_pContext)
{
 
    void *msgData;
    ix_error err1 = IX_SUCCESS;
    ix_error err2 = IX_SUCCESS;
    ix_error result;
    void *pContext;


    /* Check whether arg_pContext is invalid */
    if (arg_pContext == (void *)NULL)
    {
        /* Free the ix_buffer */
        if(arg_Msg != IX_NULL_BUFFER_HANDLE)
        {
            ix_rm_buffer_free_chain(arg_Msg);
        }

        return(IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("NULL argument")));
    }


	/* Get the message sent by the helper function */
    IX_ERROR_CT(ix_cc_msup_extract_msg(arg_Msg, &msgData, &pContext),
                err1,
                IX_CC_CSIX_RX_ERROR_MSUP_EXTRACT_MSG, 
                IX_ERROR_LEVEL_WARNING);

	if (err1 != IX_SUCCESS)
    {
        /* Free the ix_buffer here */
        IX_ERROR_CT(ix_rm_buffer_free_chain(arg_Msg),
                    err2,
                    IX_CC_CSIX_RX_ERROR_RM_FREE_BUF, 
                    IX_ERROR_LEVEL_WARNING);

        return(err1);
    }

    /* Process the message */
    switch(arg_UserData)
    {
        case IX_CC_CSIX_RX_MSG_GET_STATISTICS_INFO:
        {
            ix_cc_csix_rx_statistics_info entity;
            ix_uint32 index;
            ix_cc_csix_rx_statistics_info_data pBuffer;
            ix_uint32 numCountersReturned;

            /* Retrieve embedded information */
            entity = ((ix_cc_csix_rx_statistics_info_context *)msgData)->entity;
            index = ((ix_cc_csix_rx_statistics_info_context *)msgData)->index;

            /* Obtain the number of counters to be returned */
            if (entity == IX_CC_CSIX_RX_ALL_COUNTERS)
            {
                numCountersReturned = CSIX_RX_NUM_COUNTERS_PER_VOQ;
            }
            else /* for all single counters */
            {
                numCountersReturned = 1;
            }

            {
                ix_uint64 dataBuffer[numCountersReturned];

                /* Invoke the corresponding library API */
                pBuffer.dataLength = sizeof(dataBuffer);
                pBuffer.pDataBuffer = (void *)dataBuffer;
                result = ix_cc_csix_rx_get_statistics_info(entity,
                                                           index, 
                                                           &pBuffer,
                                                           arg_pContext);

                /* Send back the result */
                IX_ERROR_CT(ix_cc_msup_send_reply_msg(pContext,
                                                      (void *)dataBuffer, 
                                                      sizeof(dataBuffer),
                                                      result),
	                    err1,
                            IX_CC_CSIX_RX_ERROR_MSUP_SEND_REPLY, 
                            IX_ERROR_LEVEL_WARNING);
            }
            break;
        }
        default:
            err1 = IX_ERROR_WARNING(IX_CC_ERROR_UNDEFINED_MSG, 
                                  ("Undefined message type"));
            break;
    }

    /* This is the function exiting point.  Free the ix_buffer here */
    IX_ERROR_CT(ix_rm_buffer_free_chain(arg_Msg),
                err2,
                IX_CC_CSIX_RX_ERROR_RM_FREE_BUF, 
                IX_ERROR_LEVEL_WARNING);

    /* Always return the first error encountered */
    if (err1 != IX_SUCCESS)
    {
        /* Ignore the return code from ix_rm_buffer_free_chain. Just return the first error */
        return(err1);
    }

    return(err2);

} /* ix_cc_csix_rx_msg_handler */

