/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      ix_cc_pos_rx_int.h
 *
 * = DESCRIPTION
 *      This file contains parameters used internally by Pos Rx core 
 *      component.
 *
 *
 * = CHANGE HISTORY
 *      7/15/2002 - Created.
 *
 * ============================================================================
 * $Id: ix_cc_pos_rx_int.h,v 1.11 2002/10/15 22:20:47 sharma Exp $
 */

#if !defined(__IX_CC_POS_RX_INT_H__)
#define __IX_CC_POS_RX_INT_H__


/**
 * System defined include files required.
 */
#include "cc/ix_cc_pos_rx.h" 
#include "ix_ossl.h"
#include "ix_rm.h"
#include "ix_error.h"


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */



/**
 * Prototypes for internal functions.
 */




/* Size defines for callback context tables */
#define POS_RX_NUM_ENTRIES_CB_CTX_TBL_COMMON          128 /* must be power of 2 */
#define POS_RX_NUM_ENTRIES_CB_CTX_TBL_GET_STAT        POS_RX_NUM_ENTRIES_CB_CTX_TBL_COMMON
#define POS_RX_SIZE_CB_CTX_TBL_GET_STAT               (POS_RX_NUM_ENTRIES_CB_CTX_TBL_GET_STAT * sizeof(ix_cc_pos_rx_cb_ctx_entry))
#define POS_RX_NUM_ENTRIES_CB_CTX_TBL_GET_IF        POS_RX_NUM_ENTRIES_CB_CTX_TBL_COMMON
#define POS_RX_SIZE_CB_CTX_TBL_GET_IF               (POS_RX_NUM_ENTRIES_CB_CTX_TBL_GET_IF * sizeof(ix_cc_pos_rx_cb_ctx_entry))

/* various size definitions for 32bit and 64-bit counter support */
/*Rx packets, Rx bytes, Dropped packets, Exception packets*/
#define POS_RX_NUM_COUNTERS_PER_PORT                   4 
#define POS_RX_TOTAL_NUM_COUNTERS                     (IX_CC_POS_RX_MAX_NUM_PORTS * POS_RX_NUM_COUNTERS_PER_PORT)
#define POS_RX_SIZE_32BIT_COUNTER_BUF                 (POS_RX_TOTAL_NUM_COUNTERS * sizeof(ix_uint32))
#define POS_RX_SIZE_64BIT_COUNTER_HDL                 (POS_RX_TOTAL_NUM_COUNTERS * sizeof(ix_counter_64bit_handle))



/**
 * Callback context entry structure
 */
typedef struct s_cc_pos_rx_cb_ctx_entry
{
    ix_uint8 flags;
    void *pUserCtx;     /* user context */
    void *pUserCb;      /* user callback */
} ix_cc_pos_rx_cb_ctx_entry;



/* values for flags field of the callback context entry structure */
#define CC_POS_RX_ENTRY_FLAGS_NOT_USED     0x00 /* free entry */
#define CC_POS_RX_ENTRY_FLAGS_USED_STATIC  0x01 /* used static entry */
#define CC_POS_RX_ENTRY_FLAGS_USED_DYNAMIC 0x02 /* used dynamic entry */



/**
 * Callback context entry table structure
 */
typedef struct s_cc_pos_rx_cb_ctx_tbl
{
    ix_uint32 index;                 /* index pointing to the next free item */
    ix_cc_pos_rx_cb_ctx_entry *pTable; /* free cb context entry array */ 
} ix_cc_pos_rx_cb_ctx_tbl;



/**
 * Control block structure
 */
typedef struct ix_s_cc_pos_rx_ctl_blk
{

	/**
     * Number of Utopia channels. This will be read in from the properties structure
     */
    ix_uint32 numChannels;

    ix_uint32 mev2Num;        /* Pos Rx MEv2 number */
    ix_uint32 *pRxCtx;        /* Microblock Rx context memory */ /*??*/
	/**
     * Local blade id and port id of each Utopia channel.  
     * This will be derived from ix_cc_init_context structure.
     */
    ix_uint32 portId[IX_CC_POS_RX_MAX_NUM_POS_PORTS];
    ix_uint32 bladeId;

	/**
     * Port status.  This is maintained by the library API to 
     * enable and disable the channel interface.
     */
    ix_cc_pos_rx_if_state portState[IX_CC_POS_RX_MAX_NUM_POS_PORTS];
    ix_counter_64bit_handle *pCounterHandlers64Bit; /* 64-bit counter handler array */
    ix_uint32 *pCounterBase32Bit;                /* 32-bit counters memory */
    ix_cc_pos_rx_cb_ctx_tbl *pCbCtxTblGetStatInfo; /* free cb context table for stats info */
	ix_cc_pos_rx_cb_ctx_tbl *pCbCtxTblGetIfState; /* free cb context table for interface state info */
	ix_ossl_mutex_t mid; /*Mutex id used by the component in critical sections.*/
} ix_cc_pos_rx_ctl_blk;



/**
 * MACRO NAME: IX_CC_POS_RX_ALLOC_ENTRY
 *
 * DESCRIPTION: This macro allocates a user specified structure.
 *              The structure is either allocated from the free entry array
 *              passed in by the user or from OSSL if one is not 
 *              available from the array.
 *
 * @Param: arg_pTbl - IN: pointer to the free entry array.
 * @Param: arg_CurIndex - IN: current index to the array.
 * @Param: arg_TblSize - IN: size of the table (MUST be a power of 2)
 * @Param: arg_Type - IN: data type of the array element.
 *
 * @Param: arg_pEntry - OUT: pointer to the free entry.
 *
 * @Return:  - None
 */
#define IX_CC_POS_RX_ALLOC_ENTRY(                                   \
    arg_pTbl,                                                        \
    arg_CurIndex,                                                    \
    arg_TblSize,                                                     \
    arg_Type,                                                        \
    arg_pEntry)                                                      \
{                                                                    \
                                                                     \
    arg_pEntry = arg_pTbl + arg_CurIndex;                            \
                                                                     \
    if (arg_pEntry->flags == CC_POS_RX_ENTRY_FLAGS_NOT_USED)        \
    {                                                                \
        arg_pEntry->flags = CC_POS_RX_ENTRY_FLAGS_USED_STATIC;      \
        arg_CurIndex = (arg_CurIndex + 1) & (arg_TblSize - 1);       \
    }                                                                \
    else                                                             \
    {                                                                \
        if ((arg_pEntry = (arg_Type *)ix_ossl_malloc(sizeof(arg_Type))) != NULL) \
        {                                                            \
            arg_pEntry->flags = CC_POS_RX_ENTRY_FLAGS_USED_DYNAMIC; \
        }                                                            \
    }                                                                \
}



/**
 * MACRO NAME: IX_CC_POS_RX_FREE_ENTRY
 *
 * DESCRIPTION: This macro frees a structure.
 *              The structure may be freed back to system memory if 
 *              was allocated from it.
 *
 * @Param: arg_pEntry - IN: pointer to the entry to be freed.
 *
 * @Return:  - None
 */
#define IX_CC_POS_RX_FREE_ENTRY(                                      \
    arg_pEntry)                                                        \
{                                                                      \
                                                                       \
    if ((arg_pEntry)->flags == CC_POS_RX_ENTRY_FLAGS_USED_STATIC)       \
    {                                                                  \
        (arg_pEntry)->flags = CC_POS_RX_ENTRY_FLAGS_NOT_USED;           \
    }                                                                  \
    else if ((arg_pEntry)->flags == CC_POS_RX_ENTRY_FLAGS_USED_DYNAMIC) \
    {                                                                  \
        ix_ossl_free(arg_pEntry);                                      \
    }                                                                  \
}



#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_POS_RX_INT_H__) */
