/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      ix_cc_pos_rx.h
 *
 * = DESCRIPTION
 *      This file will describe the common definitions that will be exported 
 *      by the POS Rx Core Component.
 *
 *
 *
 * = CHANGE HISTORY
 *      7/15/2002 - Created.
 *
 * ============================================================================
 * $Id: ix_cc_pos_rx.h,v 1.27 2003/01/27 21:58:13 ktseng Exp $
 */

#if !defined(__IX_CC_POS_RX_H__)
#define __IX_CC_POS_RX_H__


/**
 * System defined include files required.
 */
#include "ix_error.h"
#include "ix_types.h"
#include "ix_cc.h"

#ifndef IX_EXCLUDE_CCI
#include "ix_cci.h"
#endif

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */



/**
 * The following defines the symbols to be patched into the microblock.
 */
#define POS_RX_NUM_SYMBOLS_PATCHED     1
#define SYMBOL_POS_RX_COUNTERS_BASE    "PACKET_COUNTERS_SRAM_BASE"


/**
 * Various definitions for maximum number of ports, SRAM channel 
 * number for the context memory, ME number.
 */
#define IX_CC_POS_RX_MAX_NUM_POS_PORTS  16 /* max. number of POS ports */
#define IX_CC_POS_RX_MAX_ME_NUM     8  /* max. number of MEv2 */
#define IX_CC_POS_RX_DEFAULT_SRAM_CH_NUM   0

/*Blade Id is read from the init context passed by system app; no more a registry entry*/
#ifdef IX_INCLUDE_REGISTRY
/**
 * The following defines the static properties in registry
 */
#define PROP_POS_RX_MEV2_NUM_MASK        "POS_RX/POS_RX_MEV2_NUM_MASK"

#else
/**
 * The following defines static configuration data.
 * POS Rx uses these data when system repository is not included
 * in the system.
 */
#define POS_RX_MEV2_NUM_MASK            0x00000001 /* default ME0 Note: This is a bitmask used to to 
													 go in a loop across all the uEs with left shift. This gives
													 us an index that is mapped to the g_... global array to get the 
													 actual ME no*/

#endif

/**
 * Various definitions for maximum number of PORTS, 
 and default SRAM channel number for the context memory.
 */

#define CC_POS_RX_DEFAULT_SRAM_CH_NUM  0
#define IX_CC_POS_RX_MAX_NUM_PORTS     16

/**
 * defines for exception packet types
 */
#define PPP_CONTROL_PACKET 0x01 /*Changed to 1 as u-code is setting this value; 2 or 4 byte header??*/

/**
 * Various feature options
 */


/*Enable this to get verbose debug information*/
/*#define IX_CC_POS_RX_DEBUG_ENABLE*/




/**
 * defines for statistics counters
 */
typedef enum ix_e_cc_pos_rx_statistics_info
{
    IX_CC_POS_RX_PACKETS = 0,  /* Rx packet count */
	IX_CC_POS_RX_PACKETS_DROPPED,     /* Dropped packets */
	IX_CC_POS_RX_PACKETS_EXCEPTION,  /*Exception Packets*/
    IX_CC_POS_RX_BYTES,        /* Rx byte count */
    IX_CC_POS_RX_ALL_COUNTERS, /* All supported counters of a given Port */
    IX_CC_POS_RX_COUNTER_LAST
} ix_cc_pos_rx_statistics_info;

/**
 * defines for message types
 */
typedef enum ix_e_cc_pos_rx_msg_type{
	IX_CC_POS_RX_MSG_GET_STATISTICS_INFO = (IX_CC_COMMON_MSG_ID_LAST +1),
	IX_CC_POS_RX_MSG_GET_INTERFACE_STATE
} ix_cc_pos_rx_msg_type;



/*error codes*/
enum  {
IX_CC_POS_RX_ERROR_USER_CB = IX_ERROR_MAKE_GROUP(CC_POS_RX), 
IX_CC_POS_RX_ERROR_CCI_REG_HDLR, 
IX_CC_POS_RX_ERROR_CCI_REM_HDLR,
IX_CC_POS_RX_ERROR_RM_REG_HDLR, 
IX_CC_POS_RX_ERROR_RM_REM_HDLR,
IX_CC_POS_RX_ERROR_RM_PATCH_SYMBOL, 
IX_CC_POS_RX_ERROR_RM_GET_PHYS_OFFSET, 
IX_CC_POS_RX_ERROR_RM_FREE_BUF, 
IX_CC_POS_RX_ERROR_RM_GET_64BIT_COUNTER,
IX_CC_POS_RX_ERROR_INVALID_IF_STATE,
IX_CC_POS_RX_ERROR_MSUP_SEND_REPLY,
IX_CC_POS_RX_ERROR_MSUP_SEND_MSG,
IX_CC_POS_RX_ERROR_MSUP_EXTRACT_MSG, 
IX_CC_POS_RX_ERROR_INVALID_BLADE_ID,
IX_CC_POS_RX_ERROR_OSSL_MUTEX_INIT,
IX_CC_POS_RX_ERROR_OSSL_MUTEX_FINI,
IX_CC_POS_RX_ERROR_OSSL_MUTEX_LOCK,
IX_CC_POS_RX_ERROR_OSSL
};




/* 32-bit counter overflow time */
#define CC_POS_RX_32BIT_COUNTER_OVERFLOW_TIME  13000 
/* For IXP2400, calculation is for byte counter (packet counter can be read less frequently beacause they overflow 
 * less frequently)
 * 13 sec <= 2^32 / (2.488Gbps/8)*/ 

typedef struct ix_s_cc_pos_rx_statistics_info_context
{
    ix_cc_pos_rx_statistics_info entity;
    ix_uint32 index;
    void *pUserContext;
} ix_cc_pos_rx_statistics_info_context;

typedef struct ix_s_pos_rx_cc_statistics_info_data
{
    ix_uint32 dataLength; /* length in bytes of the data buffer */
    void *pDataBuffer;    /* buffer to store the statistics data */
} ix_cc_pos_rx_statistics_info_data;


typedef struct ix_s_cc_pos_rx_if_state_context
{
    ix_uint32 portId;
    void *pUserContext;
} ix_cc_pos_rx_if_state_context;


typedef enum ix_e_cc_pos_rx_if_state
{
    IX_CC_POS_RX_IF_STATE_DOWN = 0,
    IX_CC_POS_RX_IF_STATE_UP
} ix_cc_pos_rx_if_state;



/**
 * prototypes for async messaging user callbacks
 */
typedef ix_error (*ix_cc_pos_rx_cb_get_statistics_info)(
    ix_error arg_Result,
    void *arg_pContext,
    ix_uint64 *arg_pBuffer,
    ix_uint32 arg_MsgLen);


typedef ix_error (*ix_cc_pos_rx_cb_get_interface_state)(
    ix_error arg_Result,
    void *arg_pContext,
    ix_cc_pos_rx_if_state *arg_pState);


/**
 * Prototypes for interface functions.
 */
ix_error ix_cc_pos_rx_init(
    ix_cc_handle arg_CcHandle, 
    void **arg_ppContext);

ix_error ix_cc_pos_rx_fini(
    ix_cc_handle arg_CcHandle, 
    void *arg_pContext);

ix_error ix_cc_pos_rx_msg_handler(
	ix_buffer_handle arg_Msg, 
    ix_uint32 arg_UserData,
    void *arg_pContext);


/**
 * NAME: ix_cc_pos_rx_pkt_handler
 *
 * DESCRIPTION: This is the only registered function to receive exception packets from POS rx 
 * microblock. This function sends the packet to POS_RX_PKT_OUTPUT
 * 
 * 
 * @Param:  - IN arg_hDataToken: handle to a buffer which contains exception packets
 *  from POS Rx microblock
 * @Param: - IN arg_ExceptionCode: set to relevant value (PPP_CONTROL_PACKETS for PPP control packets).
 * @Param: - IN arg_pContext: pointer to POS Rx core component context that 
 * will be passed to the core component when a packet arrives. This context was defined by
 * the core component and passed to core components infrastructure through 
 * the ix_cc_pos_rx_init function.
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_SEND_FAIL - error from CCI
 * 
 */

ix_error ix_cc_pos_rx_pkt_handler(
								  ix_buffer_handle arg_hDataBuffer,  
                                  ix_uint32 arg_ExceptionCode,
                                  void* arg_pContext
                                  );


ix_error ix_cc_pos_rx_async_get_statistics_info(
	ix_cc_pos_rx_statistics_info_context *arg_pMgInfoContext, 
	ix_cc_pos_rx_cb_get_statistics_info arg_Callback);

ix_error ix_cc_pos_rx_async_get_interface_state(
    ix_cc_pos_rx_if_state_context *arg_pStateContext, 
    ix_cc_pos_rx_cb_get_interface_state arg_Callback);


ix_error ix_cc_pos_rx_get_statistics_info(
    ix_cc_pos_rx_statistics_info arg_Entity,
    ix_uint32 arg_Index,
    ix_cc_pos_rx_statistics_info_data *arg_pBuffer,
    void *arg_pContext);


ix_error ix_cc_pos_rx_get_interface_state(
    ix_uint32 arg_PortId, 
    ix_cc_pos_rx_if_state *arg_pIfState,
    void *arg_pContext);

ix_error ix_cc_pos_rx_set_property(
    ix_uint32 arg_PropId,
    ix_cc_properties *arg_pProperty,
    void *arg_pContext);


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_POS_RX_H__) */

