/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2X00 Network Processor, Release 5
 *
 * = FILENAME
 *      ix_cc_pos_rx.c
 *
 * = DESCRIPTION
 *      This file contains POS Rx CC interface APIs and internal
 *      functions common to both level-0 and level-1 systems.
 *
 * = CHANGE HISTORY
 *      7/15/2002 - Created.
 *
 * ============================================================================
 * $Id: ix_cc_pos_rx.c,v 1.28 2003/07/18 23:06:14 ktseng Exp $
 */

/**
 * System defined include files required.
 */

/**
 * User defined include files required.
 */
#include "ix_cc_error.h"

#include "ix_types.h"
#include "ix_error.h"
#include "ix_cc.h"
#include "bindings.h"
#include "ix_cc_properties.h"

#include "ix_rm.h"
#include "ix_ossl.h"

#ifndef IX_EXCLUDE_CCI
#include "ix_cci.h"
#endif

#include "cc/ix_cc_pos_rx.h"
#include "cc/internal/ix_cc_pos_rx_int.h"



/**
 * Variable declarations global to this file only.  Extern are followed
 * by static variables.
 */



/**
 * Exported variable declarations global to this component only.
 */
ix_cc_pos_rx_ctl_blk *g_pCcPosRxCtlBlk = NULL; /* control block */


/**
 * Pre-processor symbols and macros used in this file.
 */



/**
 * Extern function prototypes
 */


/**
 * Static function prototypes
 */
PRIVATE ix_error _ix_cc_pos_rx_get_static_data(ix_cc_pos_rx_ctl_blk *);
PRIVATE void _ix_cc_pos_rx_free_ctl_blk(ix_cc_pos_rx_ctl_blk *);
PRIVATE ix_error _ix_cc_pos_rx_create_cb_ctx_tables(ix_cc_pos_rx_ctl_blk *);



/**
 * function definitions.
 */


/**
 * NAME: ix_cc_pos_rx_init
 *
 * DESCRIPTION: This function initializes the pos rx core component.
 *
 * @Param: arg_hCc - IN Given by CCI. We use this for registering pkt/msg hdlr
 * with RM/CCI
 * @Param: arg_ppContext - OUT We return this to CCI which uses this
 * while calling our msg/pkt hdlr. This is a pointer to our (internal) CtrlBlk
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */
ix_error ix_cc_pos_rx_init(
    ix_cc_handle arg_hCc,
    void **arg_ppContext)
{

    ix_cc_pos_rx_ctl_blk *pCtlBlk = NULL;
    ix_imported_symbol symbol[POS_RX_NUM_SYMBOLS_PATCHED];
    ix_uint32 inputId = 0, i=0;
    ix_error err;
    /*next 4 defined to use the ix_rm_get_phys_offset*/
#ifdef IX_PLATFORM_2401
    /* Enhancement: Patch counters depending on MICROBLOCK_COUNTERS flag
     */
#ifdef MICROBLOCK_COUNTERS
    ix_memory_type memType;
    ix_uint32 memCh;
    ix_uint32 memOffset32BitCounter;
    void *pPhyAddr;
#endif
    ix_uint16	numOfSymbols = 0;

#else /* IX_PLATFORM_2401 */
    ix_memory_type memType;
    ix_uint32 memCh;
    ix_uint32 memOffset32BitCounter;
    void *pPhyAddr;
#endif /* IX_PLATFORM_2401 */

    /* Variables used in retrieving static data from ix_cc_init_context */
    ix_cc_init_context *pInitCtx;
    ix_cc_properties_node *pPropNode;



	/* Check whether arg_ppContext is invalid */
    if (arg_ppContext == (void **)NULL)
    {

	return(IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                               ("NULL argument")));
    }


    /* Get the generic init context */
    pInitCtx = (ix_cc_init_context *)*arg_ppContext;

	/* Allocate the control block structure */
    if ((pCtlBlk = (ix_cc_pos_rx_ctl_blk *)ix_ossl_malloc(sizeof(ix_cc_pos_rx_ctl_blk))) == NULL)
    {
	return(IX_ERROR_REMOTE(IX_CC_ERROR_OOM_SYSTEM,
                                ("Out of memory")));

    }

    /* Initialize the structure */
    ix_ossl_memset((void *)pCtlBlk,
                   0,
                   (ix_ossl_size_t)sizeof(ix_cc_pos_rx_ctl_blk));


    /* Retrieve static configuration data */
   IX_ERROR_CG(_ix_cc_pos_rx_get_static_data(pCtlBlk), err, cleanup);

   /**
     * Retrieve the port id information of each channel from the
     * init context (This will work for both level-1 and level-0 modes).
     */
    pPropNode = pInitCtx->pPropertyList;
    i=0;
    while (pPropNode != NULL)
    {
     pCtlBlk->portId[i] = pPropNode->pPropertyInfo->port_id;
	 i++;
    /**
     * Store the local blade id. Assuming all nodes have same
     * blade id so it's ok to keep overwriting.
     */
    pCtlBlk->bladeId = pPropNode->pPropertyInfo->blade_id;
	pPropNode = pPropNode->pNextPropNode;
	/*get the next node*/
	}
	pCtlBlk->numChannels = i;

    /* Set initial interface state of each port to UP */
    for (i = 0; i < pCtlBlk->numChannels; i++)
    {
        pCtlBlk->portState[i] = IX_CC_POS_RX_IF_STATE_UP;
    }

#ifdef IX_PLATFORM_2401
    /* Enhancement: Patch counters depending on MICROBLOCK_COUNTERS flag
     */
#ifdef MICROBLOCK_COUNTERS
	/* allocate memory for 32-bit counters */
   IX_ERROR_CGT(ix_rm_mem_alloc(IX_MEMORY_TYPE_SRAM,
                                  CC_POS_RX_DEFAULT_SRAM_CH_NUM,
                                  POS_RX_SIZE_32BIT_COUNTER_BUF,
                                  (void **)&pCtlBlk->pCounterBase32Bit),
                  err,
                  IX_CC_ERROR_OOM_SRAM,
                  IX_ERROR_LEVEL_REMOTE,
                  cleanup);

    /* Initialize the memory */
    ix_ossl_memset((void *)pCtlBlk->pCounterBase32Bit,
                   0,
                   POS_RX_SIZE_32BIT_COUNTER_BUF);


    /* allocate memory for 64-bit counter handler array */
    if ((pCtlBlk->pCounterHandlers64Bit = (ix_counter_64bit_handle *)ix_ossl_malloc(POS_RX_SIZE_64BIT_COUNTER_HDL)) == NULL)
    {
        /* free all memories allocated earlier */
        _ix_cc_pos_rx_free_ctl_blk(pCtlBlk);

        return(IX_ERROR_REMOTE(IX_CC_ERROR_OOM_SYSTEM,
                                 ("Out of memory")));

    }

    /* allocate 64-bit counters from RM */
    IX_ERROR_CGT(ix_rm_counter_64bit_new(POS_RX_TOTAL_NUM_COUNTERS,
	                          pCtlBlk->pCounterBase32Bit,
                                  CC_POS_RX_32BIT_COUNTER_OVERFLOW_TIME,
                                  pCtlBlk->pCounterHandlers64Bit),
                  err,
                  IX_CC_ERROR_OOM_64BIT_COUNTER,
                  IX_ERROR_LEVEL_REMOTE,
                  cleanup);


    /*
     * patch various symbols and memory bases into Pos Rx microblock:
     *
     * 1. 32-bit counter memory base.
     */

    /* first obtain the base offsets. Then we patch in the channel no*/

    IX_ERROR_CGT(ix_rm_get_phys_offset(pCtlBlk->pCounterBase32Bit,
                                       &memType,
                                       &memCh,
                                       &memOffset32BitCounter,
                                       &pPhyAddr),
                 err,
                 IX_CC_POS_RX_ERROR_RM_GET_PHYS_OFFSET,
                 IX_ERROR_LEVEL_REMOTE,
                 cleanup);

	/* Pack the SRAM channel number into the offset */
     IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(memCh, memOffset32BitCounter);

    symbol[numOfSymbols].m_Value = memOffset32BitCounter;
    symbol[numOfSymbols++].m_Name = SYMBOL_POS_RX_COUNTERS_BASE;
#endif /* MICROBLOCK_COUNTERS */

#else /*IX_PLATFORM_2401*/
    /* allocate memory for 32-bit counters */
   IX_ERROR_CGT(ix_rm_mem_alloc(IX_MEMORY_TYPE_SRAM,
                                  CC_POS_RX_DEFAULT_SRAM_CH_NUM,
                                  POS_RX_SIZE_32BIT_COUNTER_BUF,
                                  (void **)&pCtlBlk->pCounterBase32Bit),
                  err,
                  IX_CC_ERROR_OOM_SRAM,
                  IX_ERROR_LEVEL_REMOTE,
                  cleanup);

    /* Initialize the memory */
    ix_ossl_memset((void *)pCtlBlk->pCounterBase32Bit,
                   0,
                   POS_RX_SIZE_32BIT_COUNTER_BUF);


    /* allocate memory for 64-bit counter handler array */
    if ((pCtlBlk->pCounterHandlers64Bit = (ix_counter_64bit_handle *)ix_ossl_malloc(POS_RX_SIZE_64BIT_COUNTER_HDL)) == NULL)
    {
        /* free all memories allocated earlier */
        _ix_cc_pos_rx_free_ctl_blk(pCtlBlk);

        return(IX_ERROR_REMOTE(IX_CC_ERROR_OOM_SYSTEM,
                                 ("Out of memory")));

    }

    /* allocate 64-bit counters from RM */
    IX_ERROR_CGT(ix_rm_counter_64bit_new(POS_RX_TOTAL_NUM_COUNTERS,
	                          pCtlBlk->pCounterBase32Bit,
                                  CC_POS_RX_32BIT_COUNTER_OVERFLOW_TIME,
                                  pCtlBlk->pCounterHandlers64Bit),
                  err,
                  IX_CC_ERROR_OOM_64BIT_COUNTER,
                  IX_ERROR_LEVEL_REMOTE,
                  cleanup);


    /*
     * patch various symbols and memory bases into Pos Rx microblock:
     *
     * 1. 32-bit counter memory base.
     */

    /* first obtain the base offsets. Then we patch in the channel no*/

    IX_ERROR_CGT(ix_rm_get_phys_offset(pCtlBlk->pCounterBase32Bit,
                                       &memType,
                                       &memCh,
                                       &memOffset32BitCounter,
                                       &pPhyAddr),
                 err,
                 IX_CC_POS_RX_ERROR_RM_GET_PHYS_OFFSET,
                 IX_ERROR_LEVEL_REMOTE,
                 cleanup);

	/* Pack the SRAM channel number into the offset */
     IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(memCh, memOffset32BitCounter);


    symbol[0].m_Value = memOffset32BitCounter; /*?? gives warning-need typecasting?*/
    symbol[0].m_Name = SYMBOL_POS_RX_COUNTERS_BASE;

#endif /*IX_PLATFORM_2401*/

	 /* Patch them into each microengine used by the microblock */
    for (i = 0; i < IX_CC_POS_RX_MAX_ME_NUM; i++)
        {
            if (pCtlBlk->mev2Num & (0x1 << i))
            {
#ifdef IX_PLATFORM_2401
            /* Enhancement: Patch counters depending on MICROBLOCK_COUNTERS flag
             * Note that ix_rm_ueng_patch_symbols() tolerates 0 symbols count.
             */
                IX_ERROR_CT(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[i],
                                                     numOfSymbols,
                                                     symbol),
                            err,
                            IX_CC_POS_RX_ERROR_RM_PATCH_SYMBOL,
                            IX_ERROR_LEVEL_REMOTE);
#else /*IX_PLATFORM_2401*/
                IX_ERROR_CT(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[i],
                                                     POS_RX_NUM_SYMBOLS_PATCHED,
                                                     symbol),
                            err,
                            IX_CC_POS_RX_ERROR_RM_PATCH_SYMBOL,
                            IX_ERROR_LEVEL_REMOTE);
#endif /* IX_PLATFORM_2401 */
                if (err != IX_SUCCESS)
                {
                    /* free all memories allocated earlier */
                    _ix_cc_pos_rx_free_ctl_blk(pCtlBlk);

                    return(err);
                }
            }
        }

    /*
     * Create mutex for critical section.
     */
    IX_ERROR_CGT(ix_ossl_mutex_init(IX_OSSL_MUTEX_UNLOCK,
                                    &(pCtlBlk->mid)),
                  err,
                  IX_CC_POS_RX_ERROR_OSSL_MUTEX_INIT,
                  IX_ERROR_LEVEL_WARNING,
                  cleanup);

	 /*
     * Allocate free callback context entries for each async messaging
     * API function.
     */
    IX_ERROR_CG(_ix_cc_pos_rx_create_cb_ctx_tables(pCtlBlk), err, cleanup);


	inputId = IX_CC_POS_RX_MSG_INPUT;
	/*Level 1*/
#ifndef IX_EXCLUDE_CCI

    /* Register message handler with CCI*/
	IX_ERROR_CGT(ix_cci_cc_add_message_handler(arg_hCc,
                                               inputId,
                                               ix_cc_pos_rx_msg_handler,
                                               IX_INPUT_TYPE_SINGLE_SRC),
                  err,
                  IX_CC_POS_RX_ERROR_CCI_REG_HDLR,
                  IX_ERROR_LEVEL_REMOTE,
                  cleanup);

	/*Level 0*/
#else
	/* Register message handler with RM*/
    IX_ERROR_CGT(ix_rm_message_handler_register(inputId,
                                                ix_cc_pos_rx_msg_handler,
                                                pCtlBlk),
                  err,
                  IX_CC_POS_RX_ERROR_RM_REG_HDLR,
                  IX_ERROR_LEVEL_REMOTE,
                  cleanup);
#endif

    /*register packet handler for POS PPP LCP packets*/
    inputId = IX_CC_POS_RX_PKT_INPUT;
	/*Level 1*/
#ifndef IX_EXCLUDE_CCI
     IX_ERROR_CGT(ix_cci_cc_add_packet_handler(arg_hCc,
                                               inputId,
                                               ix_cc_pos_rx_pkt_handler,
                                               IX_INPUT_TYPE_SINGLE_SRC),
                  err,
                  IX_CC_POS_RX_ERROR_CCI_REG_HDLR,
                  IX_ERROR_LEVEL_REMOTE,
                  cleanup);

	/*Level 0*/
#else
     IX_ERROR_CGT(ix_rm_packet_handler_register(inputId,
                                               ix_cc_pos_rx_pkt_handler,
                                               pCtlBlk),
                  err,
                  IX_CC_POS_RX_ERROR_RM_REG_HDLR,
                  IX_ERROR_LEVEL_REMOTE,
                  cleanup);
#endif



    /* assign output argument */
    *arg_ppContext = (void *)pCtlBlk;

    /*
     * Store the address of the control block structure to a global
     * varible that can be used later in other functions.
     */
    g_pCcPosRxCtlBlk = pCtlBlk;

    return(IX_SUCCESS);

cleanup:

    /* free all memories allocated earlier */
    _ix_cc_pos_rx_free_ctl_blk(pCtlBlk);

#ifdef IX_CC_POS_RX_DEBUG_ENABLE
	ix_error_dump(stderr, err);
#endif
    return(err);

} /* ix_cc_pos_rx_init */



/**
 * NAME: ix_cc_pos_rx_fini
 *
 * DESCRIPTION: This function terminates the pos rx core component.
 *
 * @Param: arg_hCc - IN
 * @Param: arg_pContext - IN
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */
ix_error ix_cc_pos_rx_fini(
    ix_cc_handle arg_hCc,
    void *arg_pContext)
{
    ix_error err;


    /* Check whether arg_pContext is valid */
    if (arg_pContext == (void *)NULL)
    {
        return(IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                               ("NULL argument")));

    }


	/*Currently we do not check for a valid arg_hCcHandle as we do not store it in
	the context in init. We use it only for reg pkt/msg hdlr*/

    /* Start the critical section */
    ix_ossl_mutex_lock(((ix_cc_pos_rx_ctl_blk *)arg_pContext)->mid,
		IX_OSSL_WAIT_FOREVER);

/*Removing message handlers*/

/*Level 1*/
#ifndef IX_EXCLUDE_CCI
	     /* Remove message handler from CCI */
        IX_ERROR_CT(ix_cci_cc_remove_message_handler(IX_CC_POS_RX_MSG_INPUT),
                  err,
                  IX_CC_POS_RX_ERROR_CCI_REM_HDLR,
                  IX_ERROR_LEVEL_WARNING);
		if (err != IX_SUCCESS)
		{
			/* Exit the critical section */
			ix_ossl_mutex_unlock(((ix_cc_pos_rx_ctl_blk *)arg_pContext)->mid);
			return(err);
		}

/*Level 0*/
#else
		/*Remove message handler from RM */
		IX_ERROR_CT(ix_rm_message_handler_unregister(IX_CC_POS_RX_MSG_INPUT),
					 err,
                     IX_CC_POS_RX_ERROR_RM_REM_HDLR,
                     IX_ERROR_LEVEL_WARNING);
		if (err != IX_SUCCESS)
		{
			/* Exit the critical section */
			ix_ossl_mutex_unlock(((ix_cc_pos_rx_ctl_blk *)arg_pContext)->mid);
			return(err);
		}
#endif



		/*Removing packet handlers*/
#ifndef IX_EXCLUDE_CCI
	     /* Remove pcaket handler from CCI */
		IX_ERROR_CT(ix_cci_cc_remove_packet_handler(IX_CC_POS_RX_PKT_INPUT),
					err,
                     IX_CC_POS_RX_ERROR_CCI_REM_HDLR,
                     IX_ERROR_LEVEL_WARNING);
		if (err != IX_SUCCESS)
		{
			/* Exit the critical section */
			ix_ossl_mutex_unlock(((ix_cc_pos_rx_ctl_blk *)arg_pContext)->mid);
			return(err);
		}

#else
		/*Remove pcaket handler from RM*/
		IX_ERROR_CRT(ix_rm_packet_handler_unregister(IX_CC_POS_RX_PKT_INPUT),
                     IX_CC_POS_RX_ERROR_RM_REM_HDLR,
                     IX_ERROR_LEVEL_WARNING);
		if (err != IX_SUCCESS)
		{
			/* Exit the critical section */
			ix_ossl_mutex_unlock(((ix_cc_pos_rx_ctl_blk *)arg_pContext)->mid);
			return(err);
		}
#endif



    /* Exit the critical section */
    ix_ossl_mutex_unlock(((ix_cc_pos_rx_ctl_blk *)arg_pContext)->mid);

    /* Free all memories and resources indicated in the arg_pContext */
    _ix_cc_pos_rx_free_ctl_blk((ix_cc_pos_rx_ctl_blk *)arg_pContext);

    /* Reset the global pointer to the control block structure */
    g_pCcPosRxCtlBlk = NULL;

    return(IX_SUCCESS);

} /* ix_cc_pos_rx_fini */





/**********************
 * Internal functions *
 **********************/


/**
 * NAME: _ix_cc_pos_rx_get_static_data
 *
 * DESCRIPTION: This function retrieves all Pos Rx static data from
 *              either system repository or ix_cc_pos_rx.h and store them
 *              into the corresponding field in the control block
 *              structure passed in.
 *
 * @Param: arg_pCtlBlk - IN
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */
PRIVATE ix_error _ix_cc_pos_rx_get_static_data(
    ix_cc_pos_rx_ctl_blk *arg_pCtlBlk)
{

#ifdef IX_INCLUDE_REGISTRY

    ix_configuration_property_handle hCoreRoot = IX_CP_CORE_PROPERTY_ROOT;
    ix_configuration_property_handle hProperty = (ix_configuration_property_handle)NULL;


     /* open the property */
     IX_ERROR_CRT(ix_rm_cp_property_open(hCoreRoot,
                                         PROP_POS_RX_MEV2_NUM_MASK,
                                         &hProperty),
                  IX_CC_ERROR_ENTRY_NOT_FOUND,
                  IX_ERROR_LEVEL_REMOTE);

     /* read the property */
     IX_ERROR_CRT(ix_rm_cp_property_get_value_uint32(hProperty,
                                                     &arg_pCtlBlk->mev2Num),
                  IX_CC_ERROR_ENTRY_NOT_FOUND,
                  IX_ERROR_LEVEL_REMOTE);

#else  /* IX_INCLUDE_REGISTRY */

     arg_pCtlBlk->mev2Num = POS_RX_MEV2_NUM_MASK;
#endif /* IX_INCLUDE_REGISTRY */

    return(IX_SUCCESS);

} /* _ix_cc_pos_rx_get_static_data */






/**
 * NAME: _ix_cc_pos_rx_create_cb_ctx_tables
 *
 * DESCRIPTION: This function creates callback context tables that
 *              contain free callback context entry structure for
 *              being used by each invocation of helper API
 *              that has callback defined.
 *
 * @Param: arg_pCtlBlk - IN: pointer to the component context allocated
 *                           in the component's init function.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */
PRIVATE ix_error _ix_cc_pos_rx_create_cb_ctx_tables(
    ix_cc_pos_rx_ctl_blk *arg_pCtlBlk)
{

    /**
     * Create table for ix_cc_pos_rx_async_get_statistics_info().
     */

    /* Allocate cb_ctx_table structure first */
    if ((arg_pCtlBlk->pCbCtxTblGetStatInfo = (ix_cc_pos_rx_cb_ctx_tbl *)ix_ossl_malloc(sizeof(ix_cc_pos_rx_cb_ctx_tbl))) == NULL)
    {
        return(IX_ERROR_REMOTE(IX_CC_ERROR_OOM_SYSTEM,
                               ("Out of memory")));
    }

    /* Initialize the structure */
    ix_ossl_memset((void *)arg_pCtlBlk->pCbCtxTblGetStatInfo,
                   0,
                   (ix_ossl_size_t)sizeof(ix_cc_pos_rx_cb_ctx_tbl));

    /* Allocate the callback context entries and put them in the table structure */
    if ((arg_pCtlBlk->pCbCtxTblGetStatInfo->pTable = (ix_cc_pos_rx_cb_ctx_entry *)ix_ossl_malloc(POS_RX_SIZE_CB_CTX_TBL_GET_STAT)) == NULL)
    {
        ix_ossl_free(arg_pCtlBlk->pCbCtxTblGetStatInfo);
        return(IX_ERROR_REMOTE(IX_CC_ERROR_OOM_SYSTEM,
                               ("Out of memory")));
    }

    /**
     * Create table for ix_cc_pos_rx_async_get_interface_state().
     */

    /* Allocate cb_ctx_table structure first */
    if ((arg_pCtlBlk->pCbCtxTblGetIfState = (ix_cc_pos_rx_cb_ctx_tbl *)ix_ossl_malloc(sizeof(ix_cc_pos_rx_cb_ctx_tbl))) == NULL)
    {
        return(IX_ERROR_REMOTE(IX_CC_ERROR_OOM_SYSTEM,
                               ("Out of memory")));
    }

    /* Initialize the structure */
    ix_ossl_memset((void *)arg_pCtlBlk->pCbCtxTblGetIfState,
                   0,
                   (ix_ossl_size_t)sizeof(ix_cc_pos_rx_cb_ctx_tbl));

    /* Allocate the callback context entries and put them in the table structure */
    if ((arg_pCtlBlk->pCbCtxTblGetIfState->pTable = (ix_cc_pos_rx_cb_ctx_entry *)ix_ossl_malloc(POS_RX_SIZE_CB_CTX_TBL_GET_IF)) == NULL)
    {
        ix_ossl_free(arg_pCtlBlk->pCbCtxTblGetStatInfo);
        return(IX_ERROR_REMOTE(IX_CC_ERROR_OOM_SYSTEM,
                               ("Out of memory")));
    }

    return(IX_SUCCESS);

} /* _ix_cc_pos_rx_create_cb_ctx_tables */





/**
 * NAME: _ix_cc_pos_rx_free_ctl_blk
 *
 * DESCRIPTION: This function frees all memories and resources specified
 *              by each pointer member of the control block structure.
 *              Finally, it frees the control block structure.
 *
 * @Param: arg_pCtlBlk - IN
 *
 * @Return: None
 */
PRIVATE void _ix_cc_pos_rx_free_ctl_blk(
    ix_cc_pos_rx_ctl_blk *arg_pCtlBlk)
{
    ix_uint32 i;

    /* Free 64-bit counter handle array */
    if (arg_pCtlBlk->pCounterHandlers64Bit != NULL)
    {
        /* First delete each 64-bit counter from Resource Manager */
        for (i = 0; i < POS_RX_TOTAL_NUM_COUNTERS; i++)
        {
            if(ix_rm_counter_64bit_delete(*(arg_pCtlBlk->pCounterHandlers64Bit + i)) != IX_SUCCESS)
				 ix_ossl_message_log("Warning: Resource Manager 64 bit counter delete failed.\n");
        }

        /* Then free the counter handler array */
        ix_ossl_free(arg_pCtlBlk->pCounterHandlers64Bit);
    }

    /* Free 32-bit counter memory */
    if (arg_pCtlBlk->pCounterBase32Bit != NULL)
    {
       if (ix_rm_mem_free(arg_pCtlBlk->pCounterBase32Bit) != IX_SUCCESS)
			ix_ossl_message_log("Warning: Resource manager memory free failed.\n");
    }


    /* Free async callback context table and entries for getStat */
    if (arg_pCtlBlk->pCbCtxTblGetStatInfo != NULL)
    {
        /* Free the entries first */
        if (arg_pCtlBlk->pCbCtxTblGetStatInfo->pTable != NULL);
        {
            ix_ossl_free(arg_pCtlBlk->pCbCtxTblGetStatInfo->pTable);
        }

        /* Free the table structure */
        ix_ossl_free(arg_pCtlBlk->pCbCtxTblGetStatInfo);
    }

	/* Free async callback context table and entries for get interface state */
    if (arg_pCtlBlk->pCbCtxTblGetIfState != NULL)
    {
        /* Free the entries first */
        if (arg_pCtlBlk->pCbCtxTblGetIfState->pTable != NULL)
        {
            ix_ossl_free(arg_pCtlBlk->pCbCtxTblGetIfState->pTable);
        }

        /* Free the table structure */
        ix_ossl_free(arg_pCtlBlk->pCbCtxTblGetIfState);
    }


    /* Destroy the critical section mutex */
    if (arg_pCtlBlk->mid != NULL)
    {
        if (ix_ossl_mutex_fini(arg_pCtlBlk->mid) != IX_SUCCESS)
        {
            ix_ossl_message_log("Warning: OSSL mutex fini failed.\n");
        }
    }

    /* free the control block structure itself */
    ix_ossl_free((void *)arg_pCtlBlk);

} /*_ix_cc_pos_rx_free_ctl_blk */


