/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2X00 Network Processor, Release 5
 *
 * = FILENAME
 *      ix_cc_pos_hdlr.c
 *
 * = DESCRIPTION
 *      This file defines various message and packet handler routines for level-1 system.
 *
 * = CHANGE HISTORY
 *      7/20/2002 - Created.
 *
 * ============================================================================
 * $Id: ix_cc_pos_rx_hdlr.c,v 1.22 2003/04/23 23:54:52 ktseng Exp $
 */

/**
 * System defined include files required.
 */


#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
#include "ix_ossl.h"
#endif

#include <stdio.h>
/**
 * User defined include files required.
 */
#include "ix_cc_error.h"

#include "ix_types.h"
#include "ix_error.h"
#include "ix_cc.h"
#include "bindings.h"
#include "ix_cc_properties.h"

#include "ix_rm.h"
#include "ix_ossl.h"


#ifndef IX_EXCLUDE_CCI
#include "ix_cci.h"
#endif

#include "cc/ix_cc_msup.h"
#include "cc/ix_cc_pos_rx.h"
#include "cc/internal/ix_cc_pos_rx_int.h"



/**
 * Variable declarations global to this file only.  Extern are followed
 * by static variables.
 */


/**
 * Pre-processor symbols and macros used in this file.
 */


/**
 * Extern function prototypes and variables
 */



/**
 * Static function prototypes
 */



/**
 * function definitions.
 */


/**
 * NAME: ix_cc_pos_rx_msg_handler
 * 
 * DESCRIPTION: This function is the common message handler routine for
 *              POS Rx CC.
 * 
 * @Param: arg_Msg - IN
 * @Param: arg_UserData - IN
 * @Param: arg_pContext - IN
 * 
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */
ix_error ix_cc_pos_rx_msg_handler(
				  ix_buffer_handle arg_Msg, 
				  ix_uint32 arg_UserData,
				  void *arg_pContext)
{
  
  void *msgData;
  ix_error err1 = IX_SUCCESS;
  ix_error err2 = IX_SUCCESS;
  ix_error result;
  void *pContext;
  ix_cc_properties_msg *ccPropMsg;  
  
  
  /* Check whether arg_pContext is invalid */
  if (arg_pContext == (void *)NULL)
    {
      err1 = IX_ERROR_WARNING(IX_CC_ERROR_NULL,
			    ("NULL argument"));
      goto funcRet;
    }
  
  /* Get the message sent by the helper function */
  IX_ERROR_CT(ix_cc_msup_extract_msg(arg_Msg, &msgData, &pContext),
              err1,
              IX_CC_POS_RX_ERROR_MSUP_EXTRACT_MSG, 
              IX_ERROR_LEVEL_WARNING);
  if (err1 != IX_SUCCESS)
    {
      /* Free the ix_buffer (ignore the return code from ix_rm_buffer_free_chain) */
      ix_rm_buffer_free_chain(arg_Msg);
      return(err1);
    }
  
  
  /* Process the message */
  switch(arg_UserData)
  {
    case (IX_CC_POS_RX_MSG_GET_STATISTICS_INFO): 
    {
	ix_cc_pos_rx_statistics_info entity;
	ix_uint32 index;
	ix_cc_pos_rx_statistics_info_data pBuffer;
	ix_uint32 numCountersReturned;
	
	/* Retrieve embedded information */
	entity = ((ix_cc_pos_rx_statistics_info_context *)msgData)->entity;
	index = ((ix_cc_pos_rx_statistics_info_context *)msgData)->index;
	
	
	/* Obtain the number of counters to be returned */
	if (entity == IX_CC_POS_RX_ALL_COUNTERS)
	  {
	    numCountersReturned = POS_RX_NUM_COUNTERS_PER_PORT;
	  }
	else /* for all single counters */
	  {
	    numCountersReturned = 1;
	  }
	/*
	 * Check for the ranges of the arg_Entity and arg_Index moved to 
	 * ix_cc_pos_rx_get_statistics_info.
	 */
	
	{
	  ix_uint64 dataBuffer[numCountersReturned];
	  /* Invoke the corresponding library API */
	  pBuffer.dataLength = sizeof(dataBuffer);
	  pBuffer.pDataBuffer = (void *)dataBuffer;
	  result = ix_cc_pos_rx_get_statistics_info(entity,
						    index, 
						    &pBuffer,
						    arg_pContext); 
	  /*4th arg-argContext. This is used by the lib API ix_cc_pos_rx_get_statistics_info
	    when it tries to get the value of 64 bit counters (context has  a pointer to the linked list 
	    of counters, index is used to get to it. */
	  
	  
	  /* Send back the result */
	  IX_ERROR_CT(ix_cc_msup_send_reply_msg(pContext,
						(void *)dataBuffer, 
						sizeof(dataBuffer),
						result),
		      err1,
		      IX_CC_POS_RX_ERROR_MSUP_SEND_REPLY, 
		      IX_ERROR_LEVEL_WARNING);
	}
    break; 	
   }
    
	/*get interface state case */          
    case IX_CC_POS_RX_MSG_GET_INTERFACE_STATE:
	{
	
	ix_uint32 portId;
	ix_cc_pos_rx_if_state ifState;
	
	/* Retrieve embedded information */
	portId = ((ix_cc_pos_rx_if_state_context *)msgData)->portId;
	
	/* Invoke the corresponding library API */
	result = ix_cc_pos_rx_get_interface_state(portId,
						  &ifState,
						  arg_pContext);
	
	/* Send back the result */
	IX_ERROR_CT(ix_cc_msup_send_reply_msg(pContext,
					      (void *)&ifState, 
					      sizeof(ix_cc_pos_rx_if_state),
					      result),
		    err1,
		    IX_CC_POS_RX_ERROR_MSUP_SEND_REPLY, 
		    IX_ERROR_LEVEL_WARNING);
	break;
      }
      
      /*case for update state; will be sent by stack for Property update for interface state*/
    case IX_CC_COMMON_MSG_ID_PROP_UPDATE:
      /*portId and property is sent to us in the property structure*/
	{
	ccPropMsg = (ix_cc_properties_msg*)msgData;
	/*Make sure the blade id is our local blade id */
	if (ccPropMsg->propData.blade_id != ((ix_cc_pos_rx_ctl_blk *)arg_pContext)->bladeId)
		{
			/* Free the ix_buffer */
			IX_ERROR_CT(ix_rm_buffer_free_chain(arg_Msg),
			err1,
			IX_CC_POS_RX_ERROR_RM_FREE_BUF, 
			IX_ERROR_LEVEL_WARNING);

#ifdef IX_DEBUG
			ix_ossl_message_log("Blade id received does not match with local blade id.\n");
#endif
			/*We do not do anything*/
			return(IX_SUCCESS);
		}
	/* Invoke the corresponding library API - set_property */
	IX_ERROR_C(ix_cc_pos_rx_set_property(ccPropMsg->propID, 
					     &(ccPropMsg->propData),
					     arg_pContext),
		   err1); 
	break;
      }
      
    default :
      err1 = IX_ERROR_WARNING(IX_CC_ERROR_UNDEFINED_MSG, 
			    ("Undefined message type"));
      break;
      
    }  
  
 funcRet:
  /* This is the function exiting point.  Free the ix_buffer here */
  IX_ERROR_CT(ix_rm_buffer_free(arg_Msg),
	      err2,
	      IX_CC_POS_RX_ERROR_RM_FREE_BUF, 
	      IX_ERROR_LEVEL_WARNING);
  
  /* Always return the first error encountered */
  if (err1 != IX_SUCCESS)
    {
      /* Ignore the return code from ix_rm_buffer_free. Just return the first error */
      return(err1);
    }
  
  if (err2 != IX_SUCCESS)
    {
      return(err2);
    }
  
  return(IX_SUCCESS);
}      
/* ix_cc_pos_rx_msg_handler */




/**
 * NAME: ix_cc_pos_rx_pkt_handler
 * 
 * DESCRIPTION: This function is the common packet handler routine for
 *              POS Rx CC. TODO-Check for host/byte order if we inspect packet
 * 
 * @Param: arg_hDataBuffer - IN
 * @Param: arg_ExceptionCode - IN
 * @Param: arg_pContext - IN
 * 
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */  
ix_error ix_cc_pos_rx_pkt_handler(
				  ix_buffer_handle arg_hDataBuffer,  
				  ix_uint32 arg_ExceptionCode,
				  void* arg_pContext)
{
  
  ix_error err = IX_SUCCESS;
  
  
  
  /*If exception code is not PPP_CONTROL_PACKET log a warning*/
  if (arg_ExceptionCode != PPP_CONTROL_PACKET)
    {
#ifdef IX_DEBUG
      IX_ERROR_WARNING(IX_CC_ERROR_UNDEFINED_EXCEP,
		       ("undefined exception code"));
#endif
    }
  /*Using only RM as CCI packet_send also uses RM packet_send*/  
  err = ix_rm_packet_send(IX_CC_POS_RX_PKT_OUTPUT, arg_hDataBuffer,0);
  /*we only free buffer if there is a send error*/
  if(err != IX_SUCCESS)
    {
	  IX_ERROR_CRT(ix_rm_buffer_free(arg_hDataBuffer),IX_CC_POS_RX_ERROR_RM_FREE_BUF, IX_ERROR_LEVEL_WARNING);
	  err = IX_ERROR_WARNING(IX_CC_ERROR_SEND_FAIL, ("Unable to send packet in POS Rx packet handler"));
    }/* end if (err != IX_SUCCESS) */     
  
  
  return err;
}
