#ifndef	_AAL5_RX_H_
#define _AAL5_RX_H_

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2002 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

/*
 *
 *     File Name: 	aal5_rx.h
 *
 *     Purpose: 	This file contains MSF register configuration parameters, 
 *					AAL5 RX configuration parameters, AAL5 RX constants.
 *					
 *
 *      History:
 *
 *
 *     Date            Comment                         By
 *     ------------------------------------------------------------------------
 *
 *	  04/16/2002		Created						
 *    06/20/2003		Merged code with ITP		
 */


#ifdef USE_IMPORT_VAR
    #include <dl_system.h>    
    #include <aal5_rx_defs.h>

	.import_var AAL5_RX_VC_INFO_SRAM_BASE
	.import_var AAL5_RX_VC_HASH_PRI_SRAM_BASE
	.import_var AAL5_RX_VC_HASH_SEC_SRAM_BASE
	.import_var AAL5_RX_PORT_STAT_SRAM_BASE
	.import_var AAL5_RX_VC_HASH_PRI_INDEX_MASK

#else

    #include <dl_system.h> 
       
#endif

// Header file with MSF register constants.
#include <hardware.h>


/*============================================================================*/
/*    			MSF register configuration constants  						  */
/*============================================================================*/



/*----------------------------------------------------------------------------*/
/* RX_UP_CONTROL Settings (CHANGED according to UTOPIA Level3 ATM OC-48 needs)*/
/*----------------------------------------------------------------------------*/

/* For Level 3 DR Time = 2 clock cycles   */
/* 0 = 1cycle, 1 = 2cycles (reset = 0, originally it was 1) */
#define UP_CTRL_DR_TIME 		(RX_DR_TIME_2_CLOCK << RX_DR_TIME_BITS)

/* 0=52 bytes   1=56 bytes  */
#define UP_CTRL_CELL_SIZE 		(RX_CELL_SIZE_52_BYTES << RX_CELL_SIZE_BITS)   

/* Single bit odd parity  */
#define UP_CTRL_PARITY  		(RX_PARITY_ODD << RX_PARITY_BITS)

/* 1 = packet mode, 0 = cell mode (originally it was 1) */
#define UP_CTRL_CP_MODE 		(RX_CP_MODE_CELL << RX_CP_MODE_BITS)   

/* 0 = POS2; 1 = POS3 (reset =0, orignally it was 1) */ 
/*#define UP_CTRL_PP_MODE 		(0 << 5)   - NO NEED */

#define AAL5_RX_UP_CTRL_CONFIG_0	(UP_CTRL_CP_MODE|UP_CTRL_PARITY|\
								UP_CTRL_CELL_SIZE|UP_CTRL_DR_TIME)

#define AAL5_RX_UP_CTRL_CONFIG_1	(UP_CTRL_CP_MODE|UP_CTRL_PARITY|\
								UP_CTRL_CELL_SIZE|UP_CTRL_DR_TIME)

#define AAL5_RX_UP_CTRL_CONFIG_2	(UP_CTRL_CP_MODE|UP_CTRL_PARITY|\
								UP_CTRL_CELL_SIZE|UP_CTRL_DR_TIME)

#define AAL5_RX_UP_CTRL_CONFIG_3	(UP_CTRL_CP_MODE|UP_CTRL_PARITY|\
								UP_CTRL_CELL_SIZE|UP_CTRL_DR_TIME)




/*-----------------------------------------------------------------------------
 MSF_RX_CONTROL settings
-----------------------------------------------------------------------------*/
/*
	[1:0]		Reserved
	[3:2]		RBUF element size 
	[8:4]		Reserved
	[9]			0  ==> CSIX Free list - Data and control cframes goto 
				different Rx_thread_freelist. (Not used here)
	[12:10]		Reserved
	[13]		0  ==> Tx Cbus width is 4 bit
	[15:14]		Reserved
	0[16]		0  ==> Tx CBus Mode is simplex mode
	1[17]		RX Mphy related - Not used here. default value 0
	2[18]		RX Mphy related - Not used here. default value 0
	3[19]		0  ==> RX Single Phy Mode 
	4[21:20]	00 ==> RX width 1 x32
	6[22]		0  ==> RX Mode is UTOPIA/POS mode
	7[27:23]	Reserved
	12[31:28]	0xF ==> Recv enable in channel 0 (SPHY) or all 0,1,2,3 channel(MPHY)
*/


#ifdef IXP2400

    #if (RBUF_ELE_SIZE == 64)

	    #define RX_CTRL_RBUF_ELE_SZ			(RBUF_ELEMENT_SIZE_64 <<RBUF_ELEMENT_SIZE_BITS)

    #elif (RBUF_ELE_SIZE == 128) /* (RBUF_ELE_SIZE == 64) */

   	    #define RX_CTRL_RBUF_ELE_SZ			(RBUF_ELEMENT_SIZE_128 <<RBUF_ELEMENT_SIZE_BITS)

    #elif (RBUF_ELE_SIZE == 256) /* (RBUF_ELE_SIZE == 128) */

        #define RX_CTRL_RBUF_ELE_SZ			(RBUF_ELEMENT_SIZE_256 <<RBUF_ELEMENT_SIZE_BITS)

    #endif /* (RBUF_ELE_SIZE == 64) */

	#define RX_CTRL_CSIX_FLIST_DIFF		(CSIX_FREELIST_DIFF <<CSIX_FREELIST_BITS)
	#define RX_CTRL_CBUS_MD_DUPLEX		(TX_CBUS_MODE_DUPLEX <<TX_CBUS_MODE_BITS)
	#define RX_CTRL_MPHY_EN_SPHY		(RX_MPHY_EN_SPHY <<RX_MPHY_MODE_BITS)


	#if (RX_PHY_MODE == SPHY_4_8)

		#define RX_CTRL_WIDTH			(RX_WIDTH_4_8 <<RX_WIDTH_BITS)
		
	#elif (RX_PHY_MODE == SPHY_2_16)

		#define RX_CTRL_WIDTH			(RX_WIDTH_2_16 <<RX_WIDTH_BITS)	

	#else  /* SPHY_1_32 */

		#define RX_CTRL_WIDTH			(RX_WIDTH_1_32 <<RX_WIDTH_BITS)

	#endif


	#define RX_CTRL_MD_UTPOIA_POS 		(RX_MODE_UOPIA_POS <<RX_MODE_BITS)



	#if (RX_PHY_MODE == SPHY_4_8)

		#define_eval RX_CTRL_EN			((RX_EN_0_ALL <<RX_EN_0_ALL_BITS) | (RX_EN_1 <<RX_EN_1_BITS) |\
										 (RX_EN_2 <<RX_EN_2_BITS) | (RX_EN_3 <<RX_EN_3_BITS))

	#elif (RX_PHY_MODE == SPHY_2_16)

		#define RX_CTRL_EN				(RX_EN_0_ALL <<RX_EN_0_ALL_BITS) | (RX_EN_2 <<RX_EN_2_BITS)

	#else /* SPHY_1_32 */

		#define RX_CTRL_EN				(RX_EN_0_ALL <<RX_EN_0_ALL_BITS)

	#endif


	#define  MSF_RX_CONTROL_CONFIG	(RX_CTRL_RBUF_ELE_SZ|RX_CTRL_CSIX_FLIST_DIFF|\
									 RX_CTRL_CBUS_MD_DUPLEX|RX_CTRL_MPHY_EN_SPHY|\
									 RX_CTRL_WIDTH|RX_CTRL_MD_UTPOIA_POS|\
									 RX_CTRL_EN)

#endif // IXP2400

#ifdef IXP2800

/*
	[1:0]		RBUF Partition = 00
	[3:2]		RBUF element size for Partition 0
	[8:4]		Don't care
	[9]			0  ==> CSIX Free list - Data and control cframes goto 
				different Rx_thread_freelist. (Not used here)
	[16]		1  ==> CSIX Flow control is Full Duplex mode
	[30]		RX_EN_S - Receive Enable for SPI-4 (RPROT = 0).
				Receive section enabled to receive SPI-4 transfers
				as defined in this spec.

*/

    #define RBUF_PARTITION_1		00  // All elements allocated from RBUF_Element_Freelist_0 and
										// RBUF_Element_Freelist_1 based on value in RX_PORT_MAP CSR


	#define RBUF_PARTITION_2		01  // 3/4 elements allocated from RBUF_Element_Freelist_0
										// and 1/4 from RBUF_Element_Freelist_1

	#define RBUF_PARTITION_3		10  // 1/2 elements allocated from RBUF_Element_Freelist_0
										// 3/8 from RBUF_Element_Freelist_1 and
										// 1/8 from RBUF_Element_Freelist_2

	#define CSIX_FLOW_CTRL_SIMPLEX	0
	#define CSIX_FLOW_CTRL_DUPLEX	1
	#define CSIX_FLOW_CTRL_BITS		16

	#define RX_EN_S_IGNORE			0
	#define RX_EN_S					1   // Receive section enabled to receive SPI-4 transfers
	#define RX_EN_S_BITS			30

	#define RX_CTRL_RBUF_PARTITION		(RBUF_PARTITION_1 <<0)

    #if (RBUF_ELE_SIZE == 64)

	    #define RX_CTRL_RBUF_ELE_SZ			(RBUF_ELEMENT_SIZE_64 <<RBUF_ELEMENT_SIZE_BITS)

    #elif (RBUF_ELE_SIZE == 128) /* (RBUF_ELE_SIZE == 64) */

   	    #define RX_CTRL_RBUF_ELE_SZ			(RBUF_ELEMENT_SIZE_128 <<RBUF_ELEMENT_SIZE_BITS)

    #elif (RBUF_ELE_SIZE == 256) /* (RBUF_ELE_SIZE == 128) */

        #define RX_CTRL_RBUF_ELE_SZ			(RBUF_ELEMENT_SIZE_256 <<RBUF_ELEMENT_SIZE_BITS)

    #endif /* (RBUF_ELE_SIZE == 64) */

	#define RX_CTRL_CSIX_FLIST_DIFF		(CSIX_FREELIST_DIFF <<CSIX_FREELIST_BITS)
	#define RX_CTRL_CSIX_FLOW_CTRL		(CSIX_FLOW_CTRL_DUPLEX <<CSIX_FLOW_CTRL_BITS)
	#define RX_CTRL_RX_EN_S				(RX_EN_S <<RX_EN_S_BITS)

	#define MSF_RX_CONTROL_CONFIG	(RX_CTRL_RBUF_PARTITION|RX_CTRL_RBUF_ELE_SZ|\
									 RX_CTRL_CSIX_FLIST_DIFF|RX_CTRL_CSIX_FLOW_CTRL|\
									 RX_CTRL_RX_EN_S)

#endif // IXP2800


/*-----------------------------------------------------------------------------
 RX_THREAD_FREELIST_TIMEOUT_0 settings
-----------------------------------------------------------------------------*/

/* CPP BUS clock time out intervals (half of the ME clock) */
#ifdef IXP2800

#define RX_THREAD_FLIST_TIMEOUT_0   1024

#else  // IXP2800

#if (RX_PHY_MODE == SPHY_4_8)

// Timeout must be lower than ( 424(OC12 cell) * 4 (threads per port) / 2(CPP bus is 2 times slower than ME) )  = 848
#define RX_THREAD_FLIST_TIMEOUT_0	512

#else    /* not SPHY_4_8 */

#define RX_THREAD_FLIST_TIMEOUT_0	1024

#endif   /* SPHY_4_8 */

#endif  // IXP2800

/*-----------------------------------------------------------------------------
	Total number of rbuf elements
-----------------------------------------------------------------------------*/
#define TOTAL_RBUF_ELEMENTS		(TOTAL_RBUF_SIZE / RBUF_ELE_SIZE)

/*--------------------------------------------------------------------------*/

/* In local memory 16LWs space is alloted for each RXC (RXC is only 9LWs)
 as lm index is returned by CAM lookup instr, considering lm entry of 16LWs 
 each.

 LM LW0-LW9 is used for RXC.
 (For TWO_ME_RX_FP - LW15 used to keep reference cnt for this CAM entry)
*/

#define RXC_LM_REF_CNT_OFFSET 		15

/*--------------------------------------------------------------------------*/

/* Fixed Remainder against which the computed CRC remainder is compared  */
#define FIXED_REMAINDER 			0xC704DD7B

/* AAL_type = AAL5 (0x000) 000-AAL5, 001-AAL3/4, 010-AAL4, 011-AAL3, 100-AAL2 */
#define AAL5_TYPE					0x0

/* Data size of an nonEOP cell (EOP cell may have less than 48 data bytes) */ 
#define NONEOP_CELL_DATA_SZ			48

/*	Only buffers with SOP need headroom (for encap/decap). Other buffers
	can have data starting from offset 0  */
#define NON_SOP_BUFF_OFFSET			0 

/* Maximum AAL5 frame size is 64KB  */
#define MAX_AAL5_FR_SZ				65536 


/* AAL_type = AAL5 (0x000) 000-AAL5, 001-AAL3/4, 010-AAL4, 011-AAL3, 100-AAL2 */
#define AAL_TYPE_AAL5				0x0
#define AAL_TYPE_AAL3_4				0x1
#define	AAL_TYPE_AAL4				0x2
#define	AAL_TYPE_AAL3				0x3
#define AAL_TYPE_AAL2				0x4
#define	AAL_TYPE_AAL0				0x5

/*---------------------------------------------------------------------------*/
/* definition of CAM state													 */
/*---------------------------------------------------------------------------*/

#define CAM_STATE_INIT				0    	/*  for initialization  */
#define CAM_STATE_VALID				1		/*  not used now  */

#define CAM_ENTRY_SHF_VAL			3		/*  mask to get cam entry */

/*---------------------------------------------------------------------------*/
/* Random Offset related, for DRAM bank scheduling.													 */
/*---------------------------------------------------------------------------*/

#define	RAND_OFFSET_MASK_AAL5			0x1ff


/*----------------------------------------------------------------------------*/
/* CELL_COUNT #defines														  */
/*----------------------------------------------------------------------------*/
#ifdef NON_AAL5_TX

	#ifdef CSIX_TX

		#define PREPEND_BYTES			8 				// Per cell header size
		#define	PER_PKT_PREPEND_BYTES	8				// Per packet (TM) header size
		#define TX_CELL_SZ				120  			// Cell size (without cell hdr)

	#endif

#endif


/*----------------------------------------------------------------------------*/
/* RXC access #defines														  */
/*----------------------------------------------------------------------------*/

#define SOP_ALLOC_VALID_AAL_FLIST_DEF		    0 /* exp_SOP(1):vc_alloc(1):res(6):
											        vc_st(1):AAL type(3):flist(4):res(16) */

#define CRC_RESIDUE_DEF						    1 /* incremental crc calculated over 
												    the curr pkt(32) */

#define CURR_BUF_HANDLE_DEF				        2 /* curr buffer handle(32) */

#define	SOP_BUF_HANDLE_DEF				        3 /* sop buffer handle(32) */

#define SOP_BUF_SZ_OFFSET_DEF			        4 /* sop_buf_size(16):ptr to header (16) */

#define BUF_OFFSET_CURR_PKT_SZ_DEF		        5 /* buffer offset(16):curr pkt 
												    bytes rcvd till now(16) */

#define DROP_EXCEP_HDRTY_PRV_PRV_BUF_ADDR_DEF	6 /* drop_bit(1):excep_bit(1):res(2):
												    hdr_type(4):prev to prev buf addr(24) */

#define PRV_BUF_SZ_REMAIN_CELL_CNT_DEF	        7 /* prev buf size(16):bytes  
												    remainder(8):cell cnt(8) */

#define PRV_BUF_HANDLE_DEF				        8 /* prev buffer handle(32)*/

#define OPTIONAL_INFO_DEF				        9 /* optional info L2port/suid/etc. */



#define lm_sop_alloc_valid_aal_flist	*l$index0[SOP_ALLOC_VALID_AAL_FLIST_DEF]
#define lm_crc_residue					*l$index0[CRC_RESIDUE_DEF]

#define lm_curr_buf_handle				*l$index0[CURR_BUF_HANDLE_DEF]
#define lm_sop_buf_handle				*l$index0[SOP_BUF_HANDLE_DEF]
#define lm_sop_buf_sz_offset			*l$index0[SOP_BUF_SZ_OFFSET_DEF]

#define lm_buf_offset_curr_pkt_sz		*l$index0[BUF_OFFSET_CURR_PKT_SZ_DEF]
#define lm_prv_buf_handle				*l$index0[PRV_BUF_HANDLE_DEF]
#define lm_prv_bf_sz_remain_cell_cnt	*l$index0[PRV_BUF_SZ_REMAIN_CELL_CNT_DEF]
#define lm_drp_ex_hdr_prv_prv_buf_addr  *l$index0[DROP_EXCEP_HDRTY_PRV_PRV_BUF_ADDR_DEF]
#define lm_optional_info                *l$index0[OPTIONAL_INFO_DEF]

  	
/*------------------------------------------------------------------------------
   To free up some registers, register values are stored in Local memory once, 
   and then used whenever required in the AAL5 RX block.

   Define LM base as 624 to store the register values.
   ***So last 16 LM locations should not be used by RX block for other purpose***

	The #defines for the offsets into this local memory chunk
------------------------------------------------------------------------------ */

#define LM_CONSTANTS_BASE 			624


#define RSW_ERR_MASK_DEF			0 /* Reg to chk rsw idle cell & parity error*/
#define RSW_ERR_CELL_SZ_DEF			1 /* Reg to chk the size of the rcvd cell */

#define FIXED_CRC_REMAIN_DEF		2 /* Has const remainder against which the
							             computed CRC is compared.Its 0xC704DD7B */

#define MAX_AAL5_FR_SZ_DEF			3 /* Maxm AAL5 Frame size supported = 64KB */

#define NON_EOP_CELL_DATA_SZ_DEF	4 /* NON EOP cell size = 48 */

#define MAX_BUF_SIZE_DEF			5 /* MAX buffer size in bytes */

#define RAND_OFFSET_MASK_DEF		6 /* Mask to get random offset (0x1ff) */

#define CRC_INIT_DEF				7 /* To initialize the CRC on SOP */

#define VC_INDEX_MASK_DEF			8 /* Mask used to get vc_index into hash table */

#define lm_rsw_err_chk_mask			*l$index1[RSW_ERR_MASK_DEF]
#define lm_rsw_err_chk_cell_sz		*l$index1[RSW_ERR_CELL_SZ_DEF]

#define lm_fixed_crc_remainder		*l$index1[FIXED_CRC_REMAIN_DEF]	

#define lm_max_aal5_fr_sz			*l$index1[MAX_AAL5_FR_SZ_DEF]	

#define lm_non_eop_cell_data_sz		*l$index1[NON_EOP_CELL_DATA_SZ_DEF]
#define lm_max_buf_size_aal5	    *l$index1[MAX_BUF_SIZE_DEF] 

#define lm_rand_offset_mask			*l$index1[RAND_OFFSET_MASK_DEF]

#define lm_crc_init					*l$index1[CRC_INIT_DEF]	
#define lm_vc_index_mask			*l$index1[VC_INDEX_MASK_DEF]

#define NEXT_THREAD_SIGNAL          2 /* must be manually allocated for TWO_ME_RX_FP */
#define	ENTER_CS1_INTER_ME_SIGNAL	3 /* sig for entering critical section 1 - access of RXC1 */
#define	ENTER_CS2_INTER_ME_SIGNAL	4 /* sig for entering critical section 2 - access of RXC2 */
#define ENTER_CS3_INTER_ME_SIGNAL	5 /* sig for entering critical section 3 - dl_sink[] and putting onto thread freelist */



#ifdef TWO_ME_RX_FP

	#define CS1_INTER_ME_SIG_DEF	 9	/* LM offset for sig cs1_inter_me_sig */
	#define CS2_INTER_ME_SIG_DEF	10  /* LM offset for sig cs2_inter_me_sig */
	#define CS3_INTER_ME_SIG_DEF	11  /* LM offset for sig cs3_inter_me_sig */

	#define lm_cs1_inter_me_sig_val     *l$index1[CS1_INTER_ME_SIG_DEF]
	#define lm_cs2_inter_me_sig_val     *l$index1[CS2_INTER_ME_SIG_DEF]
	#define lm_cs3_inter_me_sig_val     *l$index1[CS3_INTER_ME_SIG_DEF]

#endif

/*-------------------------------------------------------------------------------*
  Following definitions are used for OAM processing.
	OAM processing uses local memory locations:
		1200	to	1328	for storing OAM counters (cached for folding)
		1344	to	1376	for storing counter cache tags
 *-------------------------------------------------------------------------------*/

// defines used to process F5 flows
#define ATM_PTI_F5_SEG_OAM     		0x04	; PTI 4 cells used for F5 segment OAM
#define ATM_PTI_F5_ETE_OAM     		0x05	; PTI 5 cells used for F5 ete OAM
#define F5_PM_INDICATOR_BIT_POS		25		; F5 ETE/SEG PM enabled ? check 2 bit 
#define VC_OAM_RX_TBL_SZ_SHFT		0x04	; VC OAM Rx Table entry size - 4 LW 
#define VC_OAM_LM_TAG				0xAA	; LM cache tag for VC PM counters
#define	F5_OAM_CELL_BIT				0x01	; Whether cell type is F5 OAM cell ?
#define F5_OAM_CELL_BIT_POS     	0		; Bit position for F5_OAM_CELL_BIT

// defines used to process F4 flows
#define ATM_VCI_F4_SEG_OAM			0x03	; VCI 3 used for F4 segment OAM
#define ATM_VCI_F4_ETE_OAM			0x04	; VCI 4 used for F4 ete OAM
#define F4_PM_INDICATOR_BIT_POS		26		; F4 ETE/SEG PM enabled ? check 2 bit 
#define VP_OAM_RX_TBL_SZ_SHFT		0x04	; VP OAM Rx Table entry size - 4 LW 
#define VP_OAM_LM_TAG				0x55	; LM cache tag for VP PM counters
#define MAX_VPI_PER_PORT_SHIFT  	0x0C  	; VPI == 12 bits; multiply by 4096 
#define	F4_OAM_CELL_BIT				0x00	; Whether cell type is F4 OAM cell ?

#define	OAM_ETE_ENDPOINT_BIT		27		; Whether this VC/VP is a ETE endpoint
#define	OAM_SEG_ENDPOINT_BIT		28		; Whether this VC/VP is a SEG endpoint

// defines for F4/F5 FPM cell processing
#define OAM_FPM_CELL_TYPE			0x20	; OAM type- 0010 Function type- 0000
#define	FPM_CELL_BIT				0x02	; Whether F4/F5 OAM cell is FPM cell?
#define	FPM_CELL_BIT_POSITION		1		; Bit position for FPM_CELL_BIT
#define	OAM_CELL_TYPE_BIT			0x04	; Whether OAM cell ?
#define	OAM_CELL_TYPE_BIT_POSITION	2		; Bit position for OAM_CELL_TYPE_BIT

#define_eval OAM_CTR_CACHE_LM_BASE	1200	; Size = 16 * 8 = 128 bytes
#define_eval OAM_CTR_CACHE_LM_SZ	16
#define_eval OAM_CTR_CACHE_0_ADDR	(1200 + 0 * 16)
#define_eval OAM_CTR_CACHE_1_ADDR	(1200 + 1 * 16)
#define_eval OAM_CTR_CACHE_2_ADDR	(1200 + 2 * 16)
#define_eval OAM_CTR_CACHE_3_ADDR	(1200 + 3 * 16)
#define_eval OAM_CTR_CACHE_4_ADDR	(1200 + 4 * 16)
#define_eval OAM_CTR_CACHE_5_ADDR	(1200 + 5 * 16)
#define_eval OAM_CTR_CACHE_6_ADDR	(1200 + 6 * 16)
#define_eval OAM_CTR_CACHE_7_ADDR	(1200 + 7 * 16)

#define_eval OAM_TAG_CACHE_LM_BASE	1344	; Size = 4 * 8 = 32 bytes
#define_eval OAM_TAG_CACHE_0_ADDR	1344
#define_eval OAM_TAG_CACHE_1_ADDR	1348
#define_eval OAM_TAG_CACHE_2_ADDR	1352
#define_eval OAM_TAG_CACHE_3_ADDR	1356
#define_eval OAM_TAG_CACHE_4_ADDR	1360
#define_eval OAM_TAG_CACHE_5_ADDR	1364
#define_eval OAM_TAG_CACHE_6_ADDR	1368
#define_eval OAM_TAG_CACHE_7_ADDR	1372

// Local memory counter names 
#define lm_trcc0 		*l$index0[0]		// Total cells received with CLP=0
#define lm_trcc0_1 		*l$index0[1]		// Total cells with CLP 0 or 1
#define lm_bedc 		*l$index0[2]		// BEDC computed on last block of cells
#define lm_ref_count	*l$index0[3]		// cache reference count (for folding)


#endif	// end of _AAL5_RX_H_
