#ifndef _AAL5_RX_INIT_UC_
#define _AAL5_RX_INIT_UC_

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2002 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/


/*
 *
 *      File Name: 	aal5_rx_init.uc
 *
 *      Purpose: 	This file contains macros for initialization of AAL5 RX 
 *					microblock. It also contains system init code, as AAL5 RX 
 *					is the first block in the pipeline.
 *					
 *
 *      History:
 *
 *
 *      Date            Comment                         By
 *      ------------------------------------------------------------------------
 *
 *		04/16/2002		Created						
 *      06/20/2003		Merged code with ITP		
 */



////////////////////////////////////////////////////////////////////////////////
//	This file has the init macro for AAL5 RX and the system
////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////
// macro : _aal5_rx_read_freelist_qdesc
//
//  Loads Qdescriptor from SRAM to QArray
//
//
//
////////////////////////////////////////////////////////////////////////////////
#macro _aal5_rx_read_freelist_qdesc()
.begin

.reg $reg1
.reg  qaddr src_op1 src_op2 qdesc_addr_2
.reg qnum  check qdesc_addr channel_number
.sig queue_array_read_sig

	immed32(qdesc_addr,TEMPORARY_FREELIST_QDESC)

	immed[qnum, BUF_FREE_LIST0]

	alu_shf[src_op1,--,b,qnum,<<24]

	// Calculate channel number
	immed32[channel_number, TEMPORARY_FREELIST_QDESC]  ; Inserting base
	alu[channel_number, channel_number, and, 0xC0, <<24]    ; Get channel number

	// Calculate QD address (remember about channel and BYTE-2-LW alignment)
	alu[qdesc_addr, qdesc_addr, and~, 0xC0, <<24]   ; Mask-out channel number
	alu[src_op2, channel_number, OR, qdesc_addr,>>2]; shifting to get LW-alignment
					                ; and adding proper channel number
	sram[rd_qdesc_head,$reg1,src_op1,src_op2,2], ctx_swap[queue_array_read_sig]

	sram[rd_qdesc_other,--,src_op1,src_op2]

.end
#endm
////////////////////////////////////////////////////////////////////////////////





///////////////////////////////////////////////////////////////////////////////
// _aal5_rx_get_curr_me_ctx_num:
//
//		Description:
//			Reads the local CSR ACTIVE_CTX_STS and returns the current ME number 
//			and current context number. 
//
//  	Input: 
//			
//	
//  	Output: 
//			me_num:		 current ME number (for Sausalito, range [0,7],
//											for Castine, range [0,15])
//  		ctx_num: 	 current context number
//
//		Size: N/A
//
///////////////////////////////////////////////////////////////////////////////

#macro _aal5_rx_get_curr_me_ctx_num(me_num, ctx_num)

.begin
	
	;	Read local CSR active context status.

	local_csr_rd[ACTIVE_CTX_STS]
	immed[me_num,0] 
	
	; active_ctx_sts[7:3] has ME number and cluster number.
	; active_ctx_sts[2:0] has context number.	
	
	alu[ctx_num, 0x07, and, me_num]					; [2:0] is context number
	alu_shf_right[me_num, 0x1f, and, me_num, 3]		; [7:3] is ME number

	#ifdef IXP2400
		; To convert ME number from [Cluster Num, ME Num] format to 
		; ME Num [0..7].

		// Sausalito active_ctx_sts[7:3] has ME number and cluster number, 
		// and is represented as follows:
		// ME number bits - 7  6  5  4  3  
		//
		//    0              0  0  0  0  0
		//    1              0  0  0  0  1
		//    2              0  0  0  1  0
		//    3              0  0  0  1  1
		//    4              1  0  0  0  0
		//    5              1  0  0  0  1
		//    6              1  0  0  1  0
		//    7              1  0  0  1  1
		//
		// active_ctx_sts[2:0] has context number.
	
		; If me number greater than 3, reset cluster bit and 
		; add 4 to the ME number in order to get correct ME number.
	
		; Check if cluster bit is set.
		br_bclr[me_num, 4, S_FIRST_CLUS_PROC_DONE#] 

		; Second cluster processing, i.e ME number > 3.
		alu[me_num, me_num, and, 0x0f]			; Mask OFF cluster bit.
		alu[me_num, me_num, +, 0x4]				; add 4 to ME number.

		; Here you should get ME number between 0 and 7.

S_FIRST_CLUS_PROC_DONE#:

	#endif // IXP2400
    
    ; For IXP 2800 there is no need to calculate Me number - see PRM 5.8.1
.end
#endm   //  _aal5_rx_get_curr_me_ctx_num()

///////////////////////////////////////////////////////////////////////////////
//
// _aal5_rx_set_thd_freelist_reg:
//
//	 	Description: 
//
//			Given a ME, Thread, signal and xfer register number, form
//			the value that can be used in putting that thread to the 
//			thread freelist.
//
//	 	Outputs:
//
//			freelist_reg	: the value that can be used in putting this
//			thread onto the thread freelist.
//
//		Inputs:
//
//			me_n			: me number.
//
//			ctx_n			: thread number.
//
//			xfer_n			: xfer register number (use &). 
//							  Two transfer registers  xfer and xfer + 1 
//							  are used.
//
//			sig				: signal number to wakeup this thread.
//
//		Size: N/A
//
///////////////////////////////////////////////////////////////////////////////

#macro _aal5_rx_set_thd_freelist_reg(freelist_reg, me_n, ctx_n, xfer_n, sig) 
.begin

	;	Put signal number, me number, thread number and xfer register
	;	numbers in Rx Thread Freelist format. 
	
	alu[freelist_reg, --, B, xfer_n, <<TRNSFER_REG_BITS]		 ; [3:0] xfer reg
	alu[freelist_reg, freelist_reg, OR, ctx_n, <<THREAD_BITS]	 ; [6:4] thd num
	alu[freelist_reg, freelist_reg, OR, me_n, <<ME_NUMBER_BITS]	 ; [9:7] me num, me clus [11]
	alu[freelist_reg, freelist_reg, OR, sig, <<SIGNAL_NUMBER_BITS]; [15:12] sig num

.end
#endm   //   _aal5_rx_set_thd_freelist_reg()


////////////////////////////////////////////////////////////////////////////////
//	_aal5_rx_set_rx_thread_freelist_timeout:
//
//		Description:
//			It initializes MSF register Rx_thread_freelist_timeout which is
//			associated with the Rx_thread_freelist MSF register
// 
//		Input: 
//
//			Thread_freelist_number: thread freelist number value, range [0,3]
//
//       	Timeout: timeout value in CPP bus clock should be less than 0x1fff 
//
//		Output:
//
//		Size: N/A
//		
////////////////////////////////////////////////////////////////////////////////

#macro _aal5_rx_set_rx_thread_freelist_timeout(FREELIST_NUM, timeout_val)
.begin

	.reg tmp, msf_rx_addr

	;	Set Timeout for Rx Thread free list. Timeout specified in CPP
	;	bus clock cycles. Timeout value <= 0x1fff. FREELIST_NUM <= 3.

	move(msf_rx_addr, RX_THREAD_FREELIST_TO_/**/FREELIST_NUM)
	move(tmp, timeout_val)

	; msf[fast_wr,...] requires 
	; timeout value in upper 16 bits  and address in lower 16 bits.
	alu[tmp,--, B, tmp, <<16]					
	msf[fast_wr, --, tmp, msf_rx_addr]			

.end
#endm   //   _aal5_rx_set_rx_thread_freelist_timeout()



///////////////////////////////////////////////////////////////////////////////
// 	_aal5_rx_free_all_rbuf_elements:
//
//	Description:
//		Free rbuf elements one by one and put them onto the rbuf free list. 
//
//		At reset, the rbuf element free list is empty. The software must 
//		initialise by placing required elements in the rbuf free list before 
//		enabling receive operation.
//
//	Input:
//		num_of_rbuf_ele:	Number of rbuf elements you want to put onto the
//							rbuf free list
//	Output:
//
//	Size: N/A		
//
///////////////////////////////////////////////////////////////////////////////
#macro _aal5_rx_free_all_rbuf_elements(num_of_rbuf_ele)
.begin

	.reg element_num, temp_addr

	immed[element_num, 0]

	.while (element_num < num_of_rbuf_ele)			

		_aal5_rx_free_rbuf_element(element_num)		; free rbuf element

		alu[element_num, element_num,  + , 1]		; next element

	.endw

.end
#endm   //   _aal5_rx_free_all_rbuf_elements()

///////////////////////////////////////////////////////////////////////////////
// _aal5_rx_set_rx_up_ctrl:
//
//		Description: Initialize MSF Rx_UP_control reg: clock decode response 
//					time, cell size, parity, CP_mode and PP_mode.
//	
//		Input: 
//			rx_up_ctrl_num:	Rx_UP_control MSF reg, range [0,3]
//	
//		Output: 
//
//		Size: N/A
//
//////////////////////////////////////////////////////////////////////////////
#macro _aal5_rx_set_rx_up_ctrl(RX_UP_CTRL_NUM)
.begin

	.reg tmp, msf_rx_addr

	//	[0]		- 1  ==> UPCtrl_DRTime decode response time, 
	//					  0=1cycle, 1=2cycles (reset = 0, originally it was 1).
	//	[1]		- x	 ==> UPCtrl_CellSize, 0=52 bytes   1=56 bytes.
	//	[3:2]	- 10 ==> UPCtrl_Parity, single bit odd parity.
	//	[4]		- 1  ==> UPCtrl_CP_Mode, 1 = packet mode; 
	//						0 = cell mode (originally it was 1).
	//	[5]		- 1  ==> UPCtrl_PP_Mode, 0 = POS2; 1 = POS3 
	//						(reset =0 , orignally it was 1).
	//	[31:6]	- x  ==> reserved.

	move(msf_rx_addr, RX_UP_CTRL_/**/RX_UP_CTRL_NUM)

#ifdef COMMON_ATM

	#if (RX_UP_CTRL_NUM == RXTX_ATM_UPSTR_LINE)

    	// POS receive (support for FPGA headers)
	    #if (RXTX_ATM_UPSTR_LINE == 0)

        	#undef AAL5_RX_UP_CTRL_CONFIG_0
        	#define AAL5_RX_UP_CTRL_CONFIG_0	((RX_CP_MODE_PACKET << RX_CP_MODE_BITS)|UP_CTRL_PARITY|(RX_CELL_SIZE_OTHER_BYTES << RX_CELL_SIZE_BITS)|UP_CTRL_DR_TIME|(1<<5))

    	#endif // RXTX_ATM_UPSTR_LINE == 0

    	#if (RXTX_ATM_UPSTR_LINE == 1)

        	#undef AAL5_RX_UP_CTRL_CONFIG_1
        	#define AAL5_RX_UP_CTRL_CONFIG_1	((RX_CP_MODE_PACKET << RX_CP_MODE_BITS)|UP_CTRL_PARITY|(RX_CELL_SIZE_OTHER_BYTES << RX_CELL_SIZE_BITS)|UP_CTRL_DR_TIME|(1<<5))

    	#endif // RXTX_ATM_UPSTR_LINE == 1

	#endif

#endif // COMMON_ATM

	; RX_UP_CTRL_{0,3}  has only 6 bits [0:5] relevant, so we can use fast_wr.
	 immed[tmp, AAL5_RX_UP_CTRL_CONFIG_/**/RX_UP_CTRL_NUM]

	; msf[fast_wr,...] requires 
	; rx_up_ctrl_num in upper 16 bits  and address in lower 16 bits.
	alu_shf[tmp, --, b, tmp, <<16]

	msf[fast_wr, --, tmp, msf_rx_addr]				

.end
#endm   //   _aal5_rx_set_rx_up_ctrl()


///////////////////////////////////////////////////////////////////////////////
//	_aal5_rx_config_msf_rx_ctrl:
//
//		Description:
//			Initialize MSF RX control MSF reg for AAL5 RX.
//
//			The MSF is configured as RBUF elements is 128, Multi phy mode 16, 
//			MPHY mode, receive pins is 32 bits, Receive bus pin is UTOPIA/POS 
//			mode and all MPHY-16 channels.
//
//		Input: 
//			
//		Output: 
//
//		Size: N/A
//
///////////////////////////////////////////////////////////////////////////////

#macro _aal5_rx_config_msf_rx_ctrl[] 
.begin

	.reg $tmp, msf_rx_addr
	.sig msf_wr_sig

	//	[1:0]		Reserved.
	//	[3:2]		RBUF element size
	//	[8:4]		Reserved.
	//	[9]			0  ==> CSIX Free list - Data and control cframes goto
	//				different Rx_thread_freelist. (Not used here).
	//	[12:10]		Reserved.
	//	[13]		0  ==> Tx Cbus width is 4 bit.
	//	[15:14]		Reserved.
	//	0[16]		0  ==> Tx CBus Mode is simplex mode.
	//	1[17]		RX Mphy related - Not used here. default value 0.
	//	2[18]		RX Mphy related - Not used here. default value 0.
	//	3[19]		0  ==> RX Single Phy Mode.
	//	4[21:20]	00 ==> RX width 1 x32.
	//	6[22]		0  ==> RX Mode is UTOPIA/POS mode.
	//	7[27:23]	Reserved.
	//	12[31:28]	0xF ==> Receive enable in all (0,1,2,3) channels.

#ifdef UPDATE_MSF_RX_CNTL

	#define_eval EN_MSF_RX_BITS 	((1 << RXTX_ATM_DWNSTR_LINE) + (1 << RXTX_ATM_UPSTR_LINE)) 
    #define_eval MSF_RX_CNTL_WORD	(0x00210000 + (EN_MSF_RX_BITS << 28))

    move($tmp, MSF_RX_CNTL_WORD)
#else
    // For 64B RBUF and SPHY mode MSF_RX_CONTROL_CONFIG has following values:
	// IXP2400/SPHY_4_8 ==> MSF_RX_CONTROL_CONFIG 0xf0210000
    // IXP2400/SPHY_1_32 ==> MSF_RX_CONTROL_CONFIG 0x10010000
	// IXP2800/SPHY_1_32 ==> MSF_RX_CONTROL_CONFIG 0x40010000

	move($tmp, MSF_RX_CONTROL_CONFIG)
#endif


	move(msf_rx_addr, MSF_RX_CNTL)


	msf[write, $tmp, msf_rx_addr, 0, 1], sig_done[msf_wr_sig]
	ctx_arb[msf_wr_sig]

.end
#endm   //   _aal5_rx_config_msf_rx_ctrl()

///////////////////////////////////////////////////////////////////////////////
//	_aal5_rx_hw_init:
//
//		Description:
//			Initializes MSF RX thread freelist, frees RBUF elements, and sets
//			up RX UP CTRL reg
//
//		Input: 
//			
//		Output: 
//
//		Size: N/A
//
///////////////////////////////////////////////////////////////////////////////


#macro _aal5_rx_hw_init()
.begin

#if (RX_PHY_MODE != MPHY_ETH_ATM)

	; timeout in bus clk cycles
	_aal5_rx_set_rx_thread_freelist_timeout[0, RX_THREAD_FLIST_TIMEOUT_0] 

#else /* (RX_PHY_MODE != MPHY_ETH_ATM) */

	; timeout in bus clk cycles
	_aal5_rx_set_rx_thread_freelist_timeout[AAL5_RX_FIRST_FREELIST, RX_THREAD_FLIST_TIMEOUT_0] 

#endif /* (RX_PHY_MODE != MPHY_ETH_ATM) */

#ifdef IXP2800

	.reg $tmp zero
	.sig msf_wr_sig

	// Set PORT MAP for AAL5 RX
	move[$tmp, AAL5_RX_PORT_MAP]
	immed[zero, 0]
	msf[write, $tmp, zero, RX_PORT_MAP, 1], sig_done[msf_wr_sig]

	ctx_arb[msf_wr_sig]

    #if (RX_PHY_MODE != MPHY_ETH_ATM)	

	_aal5_rx_set_rx_thread_freelist_timeout(1, 0x0) 	// No time out freelist_1

    #endif /* (RX_PHY_MODE != MPHY_ETH_ATM)	 */

	_aal5_rx_set_rx_thread_freelist_timeout(2, 0x0) 	// No time out freelist_2
	_aal5_rx_set_rx_thread_freelist_timeout(3, 0x0) 	// No time out freelist_3


#else // IXP2400

    #ifndef AAL5_RX_SKIP_FREEING_RBUFS
    	_aal5_rx_free_all_rbuf_elements(TOTAL_RBUF_ELEMENTS)	// free all rbuf elements
    #endif

	; set up some H/W stuff.
	#if (RX_PHY_MODE == SPHY_4_8)

		; For (ONE_ME_RX_FP, SPHY_4_8)

    	_aal5_rx_set_rx_up_ctrl(0)		

		_aal5_rx_set_rx_thread_freelist_timeout[1, RX_THREAD_FLIST_TIMEOUT_0] 
		_aal5_rx_set_rx_thread_freelist_timeout[2, RX_THREAD_FLIST_TIMEOUT_0] 
		_aal5_rx_set_rx_thread_freelist_timeout[3, RX_THREAD_FLIST_TIMEOUT_0] 

		_aal5_rx_set_rx_up_ctrl(1)	
		_aal5_rx_set_rx_up_ctrl(2)	
		_aal5_rx_set_rx_up_ctrl(3)	
	
	#elif (RX_PHY_MODE == SPHY_4_8_SHARED)

    	#if (RX_ATM_LINES & 0x1)

	    	_aal5_rx_set_rx_thread_freelist_timeout[0, RX_THREAD_FLIST_TIMEOUT_0] 
		    _aal5_rx_set_rx_up_ctrl(0)	

    	#elif  (RX_ATM_LINES & 0x2)

    		_aal5_rx_set_rx_thread_freelist_timeout[1, RX_THREAD_FLIST_TIMEOUT_0] 
	    	_aal5_rx_set_rx_up_ctrl(1)	

    	#elif  (RX_ATM_LINES & 0x4)

	    	_aal5_rx_set_rx_thread_freelist_timeout[2, RX_THREAD_FLIST_TIMEOUT_0] 
		    _aal5_rx_set_rx_up_ctrl(2)	

    	#elif  (RX_ATM_LINES & 0x8)

	    	_aal5_rx_set_rx_thread_freelist_timeout[3, RX_THREAD_FLIST_TIMEOUT_0] 
		    _aal5_rx_set_rx_up_ctrl(3)	

    	#endif

	#elif (RX_PHY_MODE == SPHY_2_16)

			; For (ONE_ME_RX_FP, SPHY_2_16)

			_aal5_rx_set_rx_thread_freelist_timeout[2, RX_THREAD_FLIST_TIMEOUT_0] 

			_aal5_rx_set_rx_up_ctrl(2)

	#else  // SPHY_1_32

    	_aal5_rx_set_rx_up_ctrl(0)		

		_aal5_rx_set_rx_thread_freelist_timeout(1, 0x0) 	// No time out freelist_1
		_aal5_rx_set_rx_thread_freelist_timeout(2, 0x0) 	// No time out freelist_2
		_aal5_rx_set_rx_thread_freelist_timeout(3, 0x0) 	// No time out freelist_3

	#endif					

#endif // IXP2400

.end
#endm  // _aal5_rx_hw_init()

///////////////////////////////////////////////////////////////////////////////
//	_aal5_rx_regs_init:
//
//		Description:
//			Initializes some registers used in aal5_rx().
//
//		Input:
//			
//		Output: 
//
//		Size: N/A
//
///////////////////////////////////////////////////////////////////////////////

#macro _aal5_rx_regs_init()
.begin
	
	move(mask_lower_half, 0xffff)
    move(zero, 0)
    move(l2port, 0)
    move(suid, 0)

	immed32(vc_info_sram_base, AAL5_RX_VC_INFO_SRAM_BASE)       // Has both RXCs and VC Stat counters
	immed32(rbuf_base, RBUF_TBUF_BASE)						    // Has RBUF base
	immed32(hash_pri_base_sram, AAL5_RX_VC_HASH_PRI_SRAM_BASE)	// Has Primary hash table base
	immed32(hash_sec_base_sram, AAL5_RX_VC_HASH_SEC_SRAM_BASE)	// Has Secondary hash table base
	immed32(port_stat_base, AAL5_RX_PORT_STAT_SRAM_BASE)        // Has Port Stat counters

	; Get the current ME number and ctx number.
	_aal5_rx_get_curr_me_ctx_num(me_num, ctx_num)
	; Setup the reg to add context onto the thread freelist.
	_aal5_rx_set_thd_freelist_reg(rx_thd_freelist_reg, me_num, ctx_num, &$rsw0, (&rsw_rcvd_sig))
	; for fast_wr, upper 16bits has data and lower 16 bits has address.
	alu[rx_thd_freelist_reg, --, B, rx_thd_freelist_reg, <<16]


	#if (RX_PHY_MODE == SPHY_4_8)

		; For (ONE_ME_RX_FP, SPHY_4_8)

		; Note that SPHY_4_8 needs ONE_ME_RX_FP config, although AAL5 Rx would
		; run on two MEs.			

		; It's also possible to run SPHY_4_8 on ONE_ME_RX_FP and use only one ME:
		; in this configuration, SINGLE_4_8 must be enabled

		#ifdef FIRST_AAL5_RX_ME 

			#ifdef SINGLE_4_8

				.if((ctx()==0) || (ctx()==1))
					alu[rx_freelist_no,--,B, RX_THREAD_FREELIST_0]
				.elif((ctx()==2) || (ctx()==3)) 
					alu[rx_freelist_no,--,B, RX_THREAD_FREELIST_1]
				.elif((ctx()==4) || (ctx()==5)) 
					alu[rx_freelist_no,--,B, RX_THREAD_FREELIST_2]
				.else //((ctx()==6) || (ctx()==7)) 
					alu[rx_freelist_no,--,B, RX_THREAD_FREELIST_3]
				.endif

			#else /* SINGLE_4_8 */

			    .if((ctx()==0) || (ctx()==1) || (ctx()==2) || (ctx()==3))
				    alu[rx_freelist_no,--,B, RX_THREAD_FREELIST_0]
			    .else //((ctx()==4) || (ctx()==5) || (ctx()==6) || (ctx()==7))
				    alu[rx_freelist_no,--,B, RX_THREAD_FREELIST_1]
			    .endif

			#endif /* SINGLE_4_8 */

		#else /* FIRST_AAL5_RX_ME */

			#ifdef SINGLE_4_8

				.if((ctx()==0) || (ctx()==1))
					alu[rx_freelist_no,--,B, RX_THREAD_FREELIST_0]
				.elif((ctx()==2) || (ctx()==3)) 
					alu[rx_freelist_no,--,B, RX_THREAD_FREELIST_1]
				.elif((ctx()==4) || (ctx()==5)) 
					alu[rx_freelist_no,--,B, RX_THREAD_FREELIST_2]
				.else //((ctx()==6) || (ctx()==7)) 
					alu[rx_freelist_no,--,B, RX_THREAD_FREELIST_3]
				.endif

			#else /* SINGLE_4_8 */

			.if((ctx()==0) || (ctx()==1) || (ctx()==2) || (ctx()==3))
				alu[rx_freelist_no,--,B, RX_THREAD_FREELIST_2]
			.else //((ctx()==4) || (ctx()==5) || (ctx()==6) || (ctx()==7))
				alu[rx_freelist_no,--,B, RX_THREAD_FREELIST_3]
			.endif

			#endif /* SINGLE_4_8 */

		#endif /* FIRST_AAL5_RX_ME */

	#elif (RX_PHY_MODE == MPHY_ETH_ATM) /* (RX_PHY_MODE == SPHY_4_8) */

		alu[rx_freelist_no,--,B, AAL5_RX_THREAD_FREELIST]

	#elif (RX_PHY_MODE == SPHY_4_8_SHARED)

		#if ((RX_ATM_LINES & 0x1) > 0)
			alu[rx_freelist_no,--,B, RX_THREAD_FREELIST_0]
		#elif ((RX_ATM_LINES & 0x2) > 0)
			alu[rx_freelist_no,--,B, RX_THREAD_FREELIST_1]
		#elif ((RX_ATM_LINES & 0x4) > 0)
			alu[rx_freelist_no,--,B, RX_THREAD_FREELIST_2]
		#elif ((RX_ATM_LINES & 0x8) > 0)
			alu[rx_freelist_no,--,B, RX_THREAD_FREELIST_3]
		#endif

	#elif (RX_PHY_MODE == SPHY_2_16)

		.if((ctx()==0) || (ctx()==1) || (ctx()==2) || (ctx()==3))
			alu[rx_freelist_no,--,B, RX_THREAD_FREELIST_0]
		.else //((ctx()==4) || (ctx()==5) || (ctx()==6) || (ctx()==7))
			alu[rx_freelist_no,--,B, RX_THREAD_FREELIST_2]
		.endif

	#else  /* (RX_PHY_MODE == MPHY_ETH_ATM) */

		; For (TWO_ME_RX_FP, SPHY_1_32(default)) & 
		;     (ONE_ME_RX_FP, SPHY_1_32(default))
		alu[rx_freelist_no,--,B, RX_THREAD_FREELIST_0]

	#endif /* (RX_PHY_MODE == MPHY_ETH_ATM) */

	////////////////////////////////////////////////////
	// Set up next_thread signal value in the gpr
	////////////////////////////////////////////////////	

	#if (RX_PHY_MODE == SPHY_4_8)

	.begin

		.if(ctx() == 3)
			alu_shf[next_thread_sig_val, --, b, &next_thread_sig, <<3] ; Signal context 0
		.elif(ctx() == 7)
			immed[tmp, 0x4]
			alu_shf[next_thread_sig_val, tmp, OR, &next_thread_sig, <<3] ; Signal context 4
		.else
			immed[tmp, 0x80]							 
			alu_shf[next_thread_sig_val, tmp, OR, &next_thread_sig, <<3] ; Signal next context
		.endif
	.end

	#else // SPHY_1_32

		// [7] = 1, next context 0->1...7->0, [6:3] = signal_number
		immed[tmp, 0x80]							 
		alu_shf[next_thread_sig_val, tmp, OR, &next_thread_sig, <<3]  ; Signal next context

	#endif

	
.end
#endm  // _aal5_rx_regs_init()


///////////////////////////////////////////////////////////////////////////////
//	_aal5_rx_dl_init:
//
//		Description:
//			Initializes some dispatch loop variables.
//
//		Input:
//
//		Output:
//
//		Size: N/A
//
///////////////////////////////////////////////////////////////////////////////

#macro _aal5_rx_dl_init()
.begin

	; Initialise Dispatch Loop variables to known state.
	alu[dl_next_block, --, B, BID_NULL]			// set dl_next_block NULL
	alu[dl_buf_handle, --, B, DL_BUF_NULL]     	// set dl_buffer_handle NULL
	alu[dl_eop_buf_handle, --, B, DL_BUF_NULL] 	// set dl_eop_buf_handle NULL 

	// In main code, the meta data is set only when it is required. So if we 
	// set it to other than zero value in init, then the value remains
	// unchanged for the part that is not updated in the main code. So wrong 
	// value is written into SRAM META.
	dl_meta_init_cache(zero, zero, zero, zero, zero, zero, 0, 0)
        
    _dl_aal5_sink_init()	; Initialzes the scratch ring number used by dl_sink()

.end
#endm  //  _aal5_rx_dl_init()

///////////////////////////////////////////////////////////////////////////////
//	_aal5_rx_lm_reg_init:
//
//		Description:
//			Initializes some constants in local memory, to save registers.
//
//		Input:
//
//		Output:
//
//		Size: N/A
//
///////////////////////////////////////////////////////////////////////////////

#macro _aal5_rx_lm_reg_init()
.begin

.reg tmp

	move(lm_rsw_err_chk_mask, 0xff22)

    #ifdef FPGA_HEADER

    	move(lm_rsw_err_chk_cell_sz, 0x3800)

    #else /* FPGA_HEADER */

    	move(lm_rsw_err_chk_cell_sz, 0x3400)

    #endif /* FPGA_HEADER */

	move(lm_fixed_crc_remainder, FIXED_REMAINDER)
	move(lm_max_aal5_fr_sz, MAX_AAL5_FR_SZ)
	move(lm_non_eop_cell_data_sz, NONEOP_CELL_DATA_SZ)
	move(lm_max_buf_size_aal5, MAX_BUFF_SIZE)
	move(lm_rand_offset_mask, RAND_OFFSET_MASK_AAL5)
    move(lm_crc_init, 0xffffffff)
	immed32(lm_vc_index_mask, AAL5_RX_VC_HASH_PRI_INDEX_MASK)


.end
#endm  //   _aal5_rx_lm_reg_init()


#ifdef AAL2_INCLUDED

///////////////////////////////////////////////////////////////////////////////
// _aal5_rx_nn_prod_init:
//	 	Description: 
//			Initialise the Next Neighbour Mode (NN mode) for the producer. 
// 			For the producer, NN Registers are wrriten: from This ME. [bit 20 = 1]
//
#macro _aal5_rx_nn_prod_init()

.begin
.reg csr_data
	
	; Only thread 0
	br!=ctx[0, _aal5_rx_nn_prod_init_end#]

	; Read in the CTX_ENABLE csr value
	local_csr_rd[CTX_ENABLES]
	immed[csr_data,0]

	; For the source, NN Registers are writen from This ME. [bit 20 = 0]
	bits_clr[csr_data, 20, 1]	

	local_csr_wr[CTX_ENABLES, csr_data]

_aal5_rx_nn_prod_init_end#:
.end
#endm	//_aal5_rx_nn_prod_init()


#endif //AAL2_INCLUDED

/////////////////////////////////////////////////////////////////////////////////
//            Macro to initialize AAL5 RX
/////////////////////////////////////////////////////////////////////////////////
//	aal5_rx_init:
//
//		Description:
//			It does necessary initialization for AAL5 RX and the pipeline
//				1.  Registers init, local memory init with constant values
//				2.  RX related MSF registers init
//				3.  Does system init (scratch rings init, buffer_free_list creation, 
//					signal all MEs with the SYSTEM_INIT signal)
//				4.  Context, prefetching a buffer
//				5.  Context putting itself onto Thread freelist and waits for 
//					rsw_rcvd_sig
//
//		Inputs: 
//		
//		Outputs: 
//
//		Size: 
//			N/A
//
/////////////////////////////////////////////////////////////////////////////////						

#macro aal5_rx_init()

.begin

	#ifdef		MAX_BUFF_SIZE
		#warning	MAX_BUFF_SIZE is already defined
	#endif

	#define		MAX_BUFF_SIZE		BUFFER_SIZE

	.reg me_num					// current ME number

	.reg lm_addr				// LM addr to store reg values
	.reg next_me				// next ME number to the current one

	.reg tmp


	.sig bd_prefetch_sig		// signal for prefetching buffer handle



	; All threads would execute this part

	; Registers init
	_aal5_rx_regs_init()
		
    #ifdef AAL2_INCLUDED

	    ; NN registers to AAL2 init
    	_aal5_rx_nn_prod_init()

    #endif //AAL2_INCLUDED

	; Dispatch loop init
	_aal5_rx_dl_init()


	; Init the LM handle 1 with the lm addr for storing register values
	immed32[lm_addr, LM_CONSTANTS_BASE]
	alu_shf[lm_addr, --, b, lm_addr, <<2]
	local_csr_wr[ACTIVE_LM_ADDR_1, lm_addr]

	nop
	nop
	nop

	;  Context0 specific init 
	br!=ctx[0, NOT_THD_0#]


        #ifdef DO_FREELIST_INIT
		    #ifdef FREELIST_INIT_VIA_SCRIPT
                _aal5_rx_read_freelist_qdesc()
            #else
                // Create buffer list for upstream direction - only for the first thread
		        dl_buffer_init[]
            #endif
        #endif /* DO_FREELIST_INIT */

		; Constants Init in Local Memory.
		_aal5_rx_lm_reg_init()


 		#ifdef ATM_OAM_ENABLED
		; Initialize OAM PM Counter tag cache in local memory
		_aal5_rx_oam_init()
		#endif // ATM_OAM_ENABLED

		; absolute registers init	
		move(@rand_buf_offset, MAX_BUFFER_OFFSET)	; Starting Random Buf offset

		
		;  Do RX MSF reg init and system init
		#ifdef FIRST_AAL5_RX_ME  				; if(ctx == 0 & FIRST_AAL5_RX_ME)
	
			#ifdef SPHY_4_8_VC_INIT			

				; Do Hash tables and RXC init.
				_aal5_rx_hash_rxc_init()

			#endif

								
		#endif   // FIRST_AAL5_RX_ME


		#ifdef ONE_ME_RX_FP  					; If (ctx == 0 & One ME RX)
	
			; Init CAM and LM with first 16 RXC
			_aal5_rx_init_cam()


		#endif


		#ifdef TWO_ME_RX_FP  			; If (ctx == 0 & two ME RX)

			; Set up the global regs to signal next ME in the RX code
			; (both RX MEs would execute this part)

			// Define DL_NEXT_ME as a part of Project Settings, this is easier.
			// DL_NEXT_ME should be in [clus_no#, me_no#] format

			alu[next_me, --, b, NEXT_AAL5_RX_ME]

			// [0-3]=sig num, [4-6]=cxt num (0, here), [7-11]=ME number (clus#, me#) 
			alu[tmp,--, b, ENTER_CS1_INTER_ME_SIGNAL] 	
			alu_shf[tmp, tmp, OR, next_me, <<7]	
			alu[lm_cs1_inter_me_sig_val, --, b, tmp]
		
			alu[tmp,--, b, ENTER_CS2_INTER_ME_SIGNAL] 	
			alu_shf[tmp, tmp, OR, next_me, <<7]	
			alu[lm_cs2_inter_me_sig_val, --, b, tmp]

			alu[tmp,--, b, ENTER_CS3_INTER_ME_SIGNAL] 
			alu_shf[tmp, tmp, OR, next_me, <<7]		 
			alu[lm_cs3_inter_me_sig_val, --, b, tmp]

			#ifdef FIRST_AAL5_RX_ME	  ; If (ctx == 0 & Two ME RX & FIRST_AAL5_RX_ME)

				; required only for first ME, for the first time, second ME will get 
				; signals from first ME.
	
				; Context 0 signals itself 
							   	
			    // [0-2] = signal num, [3-6] = context num, which is 0 here.
				alu_shf[tmp, --,B,ENTER_CS1_INTER_ME_SIGNAL, <<3]
				local_csr_wr[same_me_signal, tmp]

				alu_shf[tmp, --,B,ENTER_CS2_INTER_ME_SIGNAL, <<3]
				local_csr_wr[same_me_signal, tmp]

				; ENTER_CS3_INTER_ME_SIGNAL is sent in the later part of init.

			#endif // FIRST_AAL5_RX_ME

		#endif // TWO_ME_RX_FP

        #ifdef FIRST_AAL5_RX_ME
            #ifdef DO_SYSTEM_INIT

		    	system_init()

            #endif /* DO_SYSTEM_INIT */
        #endif /* FIRST_AAL5_RX_ME */

NOT_THD_0#:

#if ((RX_PHY_MODE == SPHY_1_32) || (RX_PHY_MODE == SPHY_2_16) || (RX_PHY_MODE == MPHY_ETH_ATM) || (RX_PHY_MODE == SPHY_4_8_SHARED))
	.if(ctx() == 0)
		    ctx_arb[sys_init_signal]
	.else
	
		; All other contexts just wait for a signal from their previous context
		ctx_arb[next_thread_sig]	

	.endif   // if(ctx() == 0)

#else /* ((RX_PHY_MODE == SPHY_1_32) || (RX_PHY_MODE == MPHY_ETH_ATM)) */

	#ifdef SINGLE_4_8
		.if((ctx() == 0) || (ctx() == 2) || (ctx() ==4) || (ctx() ==6)) 

		        ctx_arb[sys_init_signal]

		.else
			; All other contexts just wait for a signal from their previous context
			ctx_arb[next_thread_sig]	
		.endif   // if(ctx() == 0)

	#else /* SINGLE_4_8 */

		.if((ctx() == 0) || (ctx() == 4)) 
		        ctx_arb[sys_init_signal]
		.else
			; All other contexts just wait for a signal from their previous context
			ctx_arb[next_thread_sig]	
		.endif   

	#endif /* SINGLE_4_8 */

#endif

#ifdef FIRST_AAL5_RX_ME
    .if(ctx() == 0)   				; if(ctx == 0 & FIRST_AAL5_RX_ME)
			_aal5_rx_hw_init()
    .endif
#endif

	; Prefetch buffer - take it from a free list
	dl_buf_alloc($prefetch_buf_handle, BUF_FREE_LIST0, bd_prefetch_sig, bd_prefetch_sig)

	; signal the next context
	signal_ctx(next_thread_sig_val)	


	#ifdef FIRST_AAL5_RX_ME	   ; If (FIRST_AAL5_RX_ME & ctx == 7)

		;  config_msf_rx_ctrl enables receive H/W. As soon as it is done, we'll start
		;  receiving packets. Before that we want to make sure all init is done.
		;  Hence we do this init in thread 7.

		br!=ctx[7, NO_MSF_CONFIG#]				; only thread 7 enables receive logic
#ifndef NO_MSF_RX_CTRL_INIT
		_aal5_rx_config_msf_rx_ctrl()    		; start receiving
#endif

NO_MSF_CONFIG#:

	#endif   // FIRST_AAL5_RX_ME



	#ifdef ONE_ME_RX_FP


		; Add thread to freelist.
		msf[fast_wr, --, rx_freelist_no, rx_thd_freelist_reg]	


		#if (RX_PHY_MODE == SPHY_4_8 || RX_PHY_MODE == SPHY_2_16)
			.if((ctx()==7)) // signal thread 4, thread 0 already has next_thread_sig
    		    ; Signal the next thread.
	        	signal_ctx(next_thread_sig_val)
			.endif
		#endif


		ctx_arb[rsw_rcvd_sig]
	

	#else // TWO_ME_RX_FP

		#ifdef FIRST_AAL5_RX_ME	 

			; Add thread to freelist 
			msf[fast_wr, --, rx_freelist_no, rx_thd_freelist_reg]	

			br!=ctx[7, THD_NOT_7#]		// context 7 signals next ME to start getting RBUF
	
			; Signal next ME with enter_cs3_inter_me_sig
			alu[--, --, B, lm_cs3_inter_me_sig_val]
			cap[fast_wr, ALU, INTERTHREAD_SIG]	

THD_NOT_7#:

			ctx_arb[rsw_rcvd_sig]


		#else  // ! FIRST_AAL5_RX_ME

			br!=ctx[0, THD_NOT_0#]		
	
			; Ctx0 waits for [enter_cs3_inter_me_sig, next_thread_sig]
			ctx_arb[enter_cs3_inter_me_sig, next_thread_sig]

			br[THD_FREELIST#]

THD_NOT_0#:

			ctx_arb[next_thread_sig]

THD_FREELIST#:

			; Add thread to freelist 
			msf[fast_wr, --, rx_freelist_no, rx_thd_freelist_reg]	

			br!=ctx[7, THD_NOT_7_1#]		// context 7 signals next ME to start getting RBUF
	
			; Signal next ME with enter_cs3_inter_me_sig
			alu[--, --, B, lm_cs3_inter_me_sig_val]
			cap[fast_wr, ALU, INTERTHREAD_SIG]	

THD_NOT_7_1#:

			; signal the next context
			signal_ctx(next_thread_sig_val)

			ctx_arb[rsw_rcvd_sig]

		#endif  // ! FIRST_AAL5_RX_ME


	#endif  // TWO_ME_RX_FP


#undef MAX_BUFF_SIZE


.end
#endm  // aal5_rx_init()

///////////////////////////////////////////////////////////////////////////////////

#endif 		// _AAL5_RX_INIT_UC_