#ifndef _AAL5_RX_ONE_ME_FP_UTIL_UC_
#define _AAL5_RX_ONE_ME_FP_UTIL_UC_

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2002 By Intel Corporation.  All rights reserved.  
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

/*
 *
 *      File Name: 	aal5_rx_one_me_fp_util.uc
 *
 *      Purpose: 	This file contains utility macros exclusively for 
 *					ONE ME RX FP (Functional Pipeline).
 *
 *					Option (ONE_ME_RX_FP , SPHY_4_8) on  (2 MEs) for OC-48 SPHY_4_8
 *					Option (ONE_ME_RX_FP , SPHY_1_32) on  (1 ME) for OC-24 (or lower) 
 *																	SPHY_1_32
 *
 *
 *      History:
 *
 *
 *      Date            Comment                         By
 *      ------------------------------------------------------------------------
 *
 *		09/19/2002		Created							
 *      06/20/2003		Merged code with ITP		
 *
 */

#ifdef 	SPHY_4_8_VC_INIT

	#include "llc_snap_decap.h"	

#endif


////////////////////////////////////////////////////////////////////////////////
// _aal5_rx_init_cam:
//		
//		Description: 
//			Used for ONE ME RX FP only
//			(1) Load first 16 CAM entries from SRAM set by control plane
//			(2) Load first 16 VC from SRAM set by control plane
//				into local memory as RXCs
//			
//		Input:
//		
//		Output:
//
//		Size: N/A
//
////////////////////////////////////////////////////////////////////////////////
#macro _aal5_rx_init_cam()


#ifndef NUM_OF_INPUTPORTS_16

	; CAM initialization.
	cam_clear

#else  // NUM_OF_INPUTPORTS_16

.begin

	.reg 		tag, tmp, vc_id, entry
	.reg 		lm_addr
	.reg 		tmp1, tmp2, tmp3

	; CAM initialization.

	cam_clear


	; Initialize CAM entries.
	// For ONE_ME_RX_FP, it will be more efficient to use CAM without reference 
	// count. The simplest way is to swap out LRU, when a miss happens. In order 
	// to keep valid receive context (RXC) in sram, when first 16 swapout happens, 
	// we need to initialize CAM and LM entry to make the first 16 swapout not 
	// generating invalid RXC in sram. Initialize CAM entries to first 16 RXCs and 
	// corresponding 16 LWs in LM.

INIT_CAM#:

	; Initialize CAM entries corresponding to RXC 0 through 15.

	// These 16 initial CAM entries are dummy, and we update CAM state with 
	// CAM_STATE_INIT. 

	// The hash lookup returns the index of RXC within the range of [0, 64K-1]
	// As cam tags cannot be zero, cam tags here have absolute rxc_addr.

	// We cannot get rxc_addr from tags, if we have vc_keys as tags.

   	immed[entry, 0x0]
      
	// Here we can init tags starting from vc_id = 500, as we will use approx
	// first 100 VCs for testing SPHY_1_32 and next 50 VCs for SPHY_4_8 
	// testing.
	
	move(vc_id, 0x1F4)			; vc_id/rxc_index = 500

	.while(entry < 16)
	
		alu_shf[tmp, --, b, vc_id, <<AAL5_RX_VC_INFO_SHF_VAL] 
		alu_shf[tag, vc_info_sram_base, +, tmp] 

		cam_write[entry, tag, CAM_STATE_INIT]

	    alu[entry, entry, +, 1] 
		alu[vc_id, vc_id, +, 1]

	.endw

INIT_LM#:

	; Init 16 RXCs in local memory. Each RXC is 9LWs. As cam_lookup returns
	; the local memory address of the entry, considering each entry is 16LWs,
	; we are initializing 16 LWs for each RXC (so 16RXC * 16 LWs = 256 LWs LM).

   immed[lm_addr, 0x0]

	; LW1 = crc_init 
   move(tmp1,0xFFFFFFFF)

   ; ExpSOP(1)|VC_alloc(1)|res (6)|VC_state(1)|AAL_type(3)|free_list(4)|res(16)|
   ; exp_SOP = set for all RXCs
   ; VC_alloc= 0 for allocated VC
   ; vc_state = 0 for valid VC
   ; AAL_type = 0x000 for AAL5
   ; free_list = 0 (for AAL5)
   move(tmp2,0x80000000)

   ; drop_bit(1)|excep_bit(1)|res(2)|hdr_type(4)|prev_to_prev_buf_addr(24)|
   ; hdr_type = 1 (for PKT_PROTO_LLC_SNAP )	
   move(tmp3, 0x1000000)

   move(entry,0)

   local_csr_wr[active_lm_addr_0, lm_addr]	
     
    nop
	nop		; 3 instr after local csr write
	nop

	.while(entry < 16)

		alu[*l$index0++, --, B, tmp2] 	; LW 0
		alu[*l$index0++, --, B, tmp1]   ; LW 1 Crc_residue_init = 0xffff ffff


		alu[*l$index0++, --, B, 0]
		alu[*l$index0++, --, B, 0]

		alu[*l$index0++, --, B, 0]
		alu[*l$index0++, --, B, 0]
		alu[*l$index0++, --, B, tmp3]	; LW 6 
		alu[*l$index0++, --, B, 0]

		alu[*l$index0++, --, B, 0]
		alu[*l$index0++, --, B, 0]
		alu[*l$index0++, --, B, 0]
		alu[*l$index0++, --, B, 0]

		alu[*l$index0++, --, B, 0]
		alu[*l$index0++, --, B, 0] 
		alu[*l$index0++, --, B, 0]
		alu[*l$index0++, --, B, 0]								

	    alu[entry, entry, +, 1] 

	.endw

.end

#endif // NUM_OF_INPUTPORTS_16


#endm   //   _aal5_rx_init_cam()


////////////////////////////////////////////////////////////////////////////////
//    _aal5_rx_get_rxc:
//
//		Description:
//
//			This is for ONE_ME_RX_FP.
//
//			It checks whether the RXC is in LM or not using CAM_lookup.
//			If its CAM_HIT, we are done. If its CAM_MISS, we get the RXC from 
//			SRAM to LM, and update the CAM.
//
//			(Here RXC consists of all 9 LWs of the context).
//
//
//		Input:
//			rxc_index :   index of RXC into the RXC table (VC Info table) 
//						  - range [0, 64K-1].	
//		
//		Outout:
//			new_rxc_tag : absolute address of RXC	(= CAM tag).
//
//		Size:
//			31 cycles (worst case, CAM_MISS), 8 cycles (best case, CAM_HIT).
///////////////////////////////////////////////////////////////////////////////

#macro _aal5_rx_get_rxc(new_rxc_tag, rxc_index)
.begin

	.sig rxc_read_sig   		; signal to read in new RXC from SRAM
	.sig rxc_write_sig			; signal to write out old RXC(LRU) to SRAM

	.reg cam_result, cam_entry

	.reg tmp

	; As VC Info is 16 LWs, (RXC is 9 LWs), <<4, but it wants in bytes so <<6.
	; cam tag = rxc_base + (index <<6) { where index = [0, 64K-1] }
	; cam_tag = rxc_abs_addr - this is because CAM tag cannot be 0.


	alu_shf[tmp, --, b, rxc_index, <<AAL5_RX_VC_INFO_ENTRY_SIZE_LOG2] 
	alu_shf[new_rxc_tag, vc_info_sram_base, +, tmp] 

	; CAM lookup for RXC using rxc_ptr.
	cam_lookup[cam_result, new_rxc_tag], lm_addr0[0]

	; Check for cam_hit or cam_miss.
	br_bset[cam_result, 7, CAM_HIT#]

	
CAM_MISS#:	

	; Get the cam entry.
	alu_shf[cam_entry, 0xf, and, cam_result, >>CAM_ENTRY_SHF_VAL]	

#ifdef USE_OPTIONAL_INFO
	// Allocate xfer registers for reading context.
	xbuf_alloc($rxc_in, 10, read)
#else
	// Allocate xfer registers for reading context.
	xbuf_alloc($rxc_in, 9, read)
#endif
	// Allocate xfer registers for writing context.
	xbuf_alloc($rxc_out, 9, write)
	
	_aal5_rx_swap_out_rxc($rxc_out, cam_entry, rxc_write_sig) ; (12)

	_aal5_rx_swap_in_rxc($rxc_in, new_rxc_tag, rxc_read_sig) ; (2)

	; Update the cam entry with the new RXC's addr first before ctx_arb, so that
	; next thread gets a hit for the same RXC. Or it doesnt evict the same
	; entry again, considering it as LRU.

	; CAM_STATE_VALID state is not used now.
	cam_write[cam_entry, new_rxc_tag, CAM_STATE_VALID] 

	ctx_arb[rxc_read_sig, rxc_write_sig, next_thread_sig], AND

	; Copy RXC from SRAM to LM.
	alu[lm_sop_alloc_valid_aal_flist,   --, B, $rxc_in[0]] 		; LW 0
	alu[lm_crc_residue,                 --, B, $rxc_in[1]] 		; LW 1 Crc_residue_init = 0xffff ffff
	alu[lm_curr_buf_handle,             --, B, $rxc_in[2]]		; LW 2
	alu[lm_sop_buf_handle,              --, B, $rxc_in[3]]		; LW 3
	alu[lm_sop_buf_sz_offset,           --, B, $rxc_in[4]]		; LW 4
	alu[lm_buf_offset_curr_pkt_sz,      --, B, $rxc_in[5]]		; LW 5
	alu[lm_drp_ex_hdr_prv_prv_buf_addr, --, B, $rxc_in[6]]		; LW 6
	alu[lm_prv_bf_sz_remain_cell_cnt,   --, B, $rxc_in[7]]		; LW 7
	alu[lm_prv_buf_handle,              --, B, $rxc_in[8]]		; LW 8
#ifdef USE_OPTIONAL_INFO
    alu[lm_optional_info,               --, B, $rxc_in[9]]		; LW 9
#endif

	br[GET_RXC_DONE#]

	xbuf_free($rxc_in)
	xbuf_free($rxc_out)

CAM_HIT#:

	; Wait for next_thread_signal.
	ctx_arb[next_thread_sig]

GET_RXC_DONE#:

.end
#endm   //   _aal5_rx_get_rxc()

////////////////////////////////////////////////////////////////////////////////
//	_aal5_rx_swap_out_rxc:
//	  
//	Description:
//
//			This is used for ONE_ME_RX_FP only.
//	
//			It writes back old RXC to SRAM.
//			
//		Input:
//			entry :  CAM entry corresponding to the RXC to be written back to SRAM
//		
//			signal_number : signal number to use for SRAM write
//
//			$rxc_out : sxfer regs
//
//		Outout:
//			None
//
//		Size:
//			12 cycles
////////////////////////////////////////////////////////////////////////////////

#macro _aal5_rx_swap_out_rxc($rxc_out, in_entry, in_signal_number)
.begin
	
	.reg lru_addr

	; TAG = rxc abs addr.

	cam_read_tag[lru_addr, in_entry]

	alu[$rxc_out[0], --, B, lm_sop_alloc_valid_aal_flist] 	; LW 0
	alu[$rxc_out[1], --, B, lm_crc_residue] 		        ; LW 1 Crc_residue_init = 0xffff ffff
	alu[$rxc_out[2], --, B, lm_curr_buf_handle]		        ; LW 2
	alu[$rxc_out[3], --, B, lm_sop_buf_handle]		        ; LW 3
	alu[$rxc_out[4], --, B, lm_sop_buf_sz_offset]		    ; LW 4
	alu[$rxc_out[5], --, B, lm_buf_offset_curr_pkt_sz]	    ; LW 5
	alu[$rxc_out[6], --, B, lm_drp_ex_hdr_prv_prv_buf_addr]	; LW 6
	alu[$rxc_out[7], --, B, lm_prv_bf_sz_remain_cell_cnt]	; LW 7
	alu[$rxc_out[8], --, B, lm_prv_buf_handle]		        ; LW 8

	; Set ref_cnt as 8(0x8 - one less than actual) and OV[25] bit to override 
	; reference count
	alu_shf[--, --, B, 0x18, <<21]
	sram[write,$rxc_out[0], lru_addr, 0, max_9], indirect_ref, sig_done[in_signal_number]

.end
#endm   //   _aal5_rx_swap_out_rxc()

////////////////////////////////////////////////////////////////////////////////
//	_aal5_rx_swap_in_rxc:
//	  
//	Description:
//
//			This is used for ONE_ME_RX_FP only.
//	
//			It reads in new RXC from SRAM.
//			
//		Input:
//
//			signal_number : signal number to use for SRAM write
//
//			rxc_addr : the absolute address of new RXC
//
//		Outout:
//			None
//
//		Input/Output:
//			$rxc_in:  The sxfer regs which will have new RXC
//
//		Size:
//			2 cycles
////////////////////////////////////////////////////////////////////////////////

#macro _aal5_rx_swap_in_rxc($rxc_in, rxc_addr, signal_number)
.begin

#ifdef USE_OPTIONAL_INFO
    ; Read in 9 LWs of RXC from SRAM.
 	; Set ref_cnt as 9(0x9 - one less than actual) and OV[25] bit to override 
	; reference count
 	alu_shf[--, --, B, 0x19, <<21]
	sram[read,$rxc_in[0], rxc_addr, 0, max_10], indirect_ref, sig_done[signal_number]
#else
    ; Read in 8 LWs of RXC from SRAM.
 	; Set ref_cnt as 8(0x8 - one less than actual) and OV[25] bit to override 
	; reference count
 	alu_shf[--, --, B, 0x18, <<21]
	sram[read,$rxc_in[0], rxc_addr, 0, max_9], indirect_ref, sig_done[signal_number]
#endif

.end
#endm   //   _aal5_rx_swap_in_rxc()



//  For SPHY_4_8 the initialization of VCs and RXCs is done here in microcode.
//	(This will be later changed to incorporate this initialization in .ind file)
//
//  For SPHY_1_32 the initialization of VCs and RXCs is done here in .ind file.

#ifdef SPHY_4_8_VC_INIT

///////////////////////////////////////////////////////////////////////////////
//
// Adding info into RXC table (VC Info table) and adding VC into HASH TABLE.
// In real scenario, this will be done by Xscale Core, for simulation purposes,
// we do it here.
//
///////////////////////////////////////////////////////////////////////////////



///////////////////////////////////////////////////////////////////////////////
// _aal5_rx_sec_hash_init:
//
// Description:
//		This creates the link list in SRAM which is used for secondary hash table.
//
// Input: N/A
//
// Output: N/A
//
// Size: N/A
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_aal5_rx_sec_hash_init()
.begin

	// Initilize free list for secondary and rest of hash table.
	buf_freelist_create[VC_HASH_SEC_BKT_TOTAL, BUF_FREE_LIST1, VC_HASH_SEC_TABLE_POOL]

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// _aal5_rx_vc_add
//
// Description:
//
//		This is used to add VC entry into hash table (could be either primary table
//		and secondary table)
//
// Input:
//
//		vc_key:	 VPI/VCI/input_port# 
//
//		vc_id:	 The flow id of VC (range [0, 64K - 1])
//	
// Output:
//		N/A
//
// Size: 
//		N/A
///////////////////////////////////////////////////////////////////////////////


#macro _aal5_rx_vc_add(vc_key, vc_id)
.begin

	.reg vpi
	.reg bkt_index
	.reg sec_table
	.reg $buf_handle
	.reg sram_offset
	.reg next_ptr

	.sig alloc_sig
	.sig bkt_read_sig
	.sig bkt_write_sig

	xbuf_alloc($bkt_in, 8, read_write)


	local_csr_wr[CRC_REMAINDER, lm_crc_init]
	nop
	nop
	nop
	crc_be[crc_32,  --,  vc_key] 
	nop
	nop
	nop
	nop
	nop
	// this requires 5 nops between crc_be[] and local_csr_rd[] 
	// Read the actual CRC remainder
	local_csr_rd[CRC_REMAINDER]
	immed[bkt_index,0]
	
	alu_shf[bkt_index, lm_vc_index_mask, AND, bkt_index]

	alu[bkt_index, --, B, bkt_index, <<VC_HASH_PRI_SHF_VAL]
	alu[bkt_index, bkt_index, +, hash_pri_base_sram]
	
	// Get the bucket from sram to sxfer reg.
	sram[read, $bkt_in[0], bkt_index, 0, 4],sig_done[bkt_read_sig]
	ctx_arb[bkt_read_sig], defer[1]
	alu[vc_key, vc_key, OR, 1, <<31]  ; set the msb. This msb is used to check
									  ; whether this has valid VC_key or not.
	

COMPARE0_PRI#:

	// Check whether vc_key1 is occupied or not from msb set
	br_bset[$bkt_in[0], 31, COMPARE1_PRI#]

	// Here vc_id1= NULL
	alu[$bkt_in[0], --, B, vc_key]
	ld_field[$bkt_in[1], 0011, vc_id]

	sram[write, $bkt_in[0], bkt_index, 0, 2],sig_done[bkt_write_sig]
	ctx_arb[bkt_write_sig]
	
	br[END#]	


COMPARE1_PRI#:

	// Check whether vc_key2 is occupied or not from msb set
	br_bset[$bkt_in[2], 31, NO_SPACE_IN_PRI#]

	// Here vc_id2 = NULL
	alu[$bkt_in[1], --, B, $bkt_in[1]]
	ld_field[$bkt_in[1], 1100, vc_id]
	alu[$bkt_in[2], --, B, vc_key]

	sram[write, $bkt_in[1], bkt_index, 4, 2],sig_done[bkt_write_sig]
	ctx_arb[bkt_write_sig]
	
	br[END#]	

NO_SPACE_IN_PRI#:

	alu[sec_table, lm_vc_index_mask, AND, $bkt_in[3]]
	bne[READ_NEXT_TABLE#]        			// br if next_ptr != NULL

ALLOC_NEW_BUFFER#:

	// Allocate one secondary table after primary table
	buf_alloc($buf_handle, BUF_FREE_LIST1, VC_HASH_SEC_TABLE_POOL, \
			alloc_sig, alloc_sig, --)
	
	alu[--, --, B, $buf_handle, <<8]			// [23:0] contains buf handle
	beq[ALLOC_NEW_BUFFER#]

	dl_buf_get_desc(sram_offset, $buf_handle)
	
	// Currently sec table is in SRAM channel 0. But in future when you change
	// to channel 1, remember to do this mapping.

	// dl_buf_get_desc doesn't take care of bank addressing
	// add 0x40000000 for channel 1
	alu_shf[sram_offset, sram_offset, OR, 0x40, <<24]
	
	// Subtract the base address
	alu[next_ptr, sram_offset, -, hash_sec_base_sram]

	// Division by size of sec table i.e. 32 bytes = 8LWs
	// This will give you the index into the VC_HASH_SEC_TABLE
	alu[next_ptr, mask_lower_half, AND, next_ptr, >>VC_HASH_SEC_SHF_VAL]
	
	ld_field[$bkt_in[3], 0011, next_ptr]
	
	// Update the primary table with next ptr	
	sram[write, $bkt_in[3], bkt_index, 12, 1],sig_done[bkt_write_sig]
	ctx_arb[bkt_write_sig]
	
	// Now write to the secondary table
	alu[$bkt_in[0], --, B, vc_key]
	ld_field[$bkt_in[1], 0011, vc_id]
	
	// Update the secondary table
	sram[write, $bkt_in[0], sram_offset, 0, 2],sig_done[bkt_write_sig]
	ctx_arb[bkt_write_sig]

	br[END#]	

READ_NEXT_TABLE#:

	// Multiply sec_table index by size of entry i.e. 32 bytes.
	alu[sec_table, --, B, sec_table, <<VC_HASH_SEC_SHF_VAL]
	
	// Get sram offset.
	alu[sram_offset, sec_table, +, hash_sec_base_sram]

	// Read secondary table.
	sram[read, $bkt_in[0], sram_offset, 0, 8],sig_done[bkt_read_sig]
	ctx_arb[bkt_read_sig]
	
COMPARE1_SEC#:

	// Check whether vc_key1 is occupied or not from msb set
	br_bset[$bkt_in[0], 31, COMPARE2_SEC#]

	// Here vc_id1 = NULL.
	// This case would never be hit, as if we have allocated new sec bkt, then
	// we would have filled up the first vc_key1

	alu[$bkt_in[0], --, B, vc_key]
	ld_field[$bkt_in[1], 0011, vc_id]
	
	// write secondary table.
	sram[write, $bkt_in[0], sram_offset, 0, 2],sig_done[bkt_write_sig]
	ctx_arb[bkt_write_sig]
	
	br[END#]
	 
COMPARE2_SEC#:

	// Check whether vc_key2 is occupied or not from msb set.
	br_bset[$bkt_in[2], 31, COMPARE3_SEC#]


	// Here vc_id2 = NULL
	alu[$bkt_in[1], --, B, $bkt_in[1]]
	ld_field[$bkt_in[1], 1100, vc_id]

	alu[$bkt_in[2], --, B, vc_key]
		
	// write secondary table
	sram[write, $bkt_in[1], sram_offset, 4, 2],sig_done[bkt_write_sig]
	ctx_arb[bkt_write_sig]
	
	br[END#]

COMPARE3_SEC#:

	// Check whether vc_key3 is occupied or not from msb set
	br_bset[$bkt_in[3], 31, COMPARE4_SEC#]

	// Here vc_id3 = NULL
	alu[$bkt_in[3], --, b, vc_key]
	ld_field[$bkt_in[4], 0011, vc_id]

	// write secondary table
	sram[write, $bkt_in[3], sram_offset, 12, 2],sig_done[bkt_write_sig]
	ctx_arb[bkt_write_sig]
	
	br[END#]

COMPARE4_SEC#:

	// Check whether vc_key4 is occupied or not from msb set
	br_bset[$bkt_in[5], 31, NOT_FOUND_IN_SEC#]

	// Here vc_id4 = NULL
	alu[$bkt_in[4], --, B, $bkt_in[4]]
	ld_field[$bkt_in[4], 1100, vc_id]

	alu[$bkt_in[5], --, B, vc_key]
			  	
	// write secondary table
	sram[write, $bkt_in[4], sram_offset, 16, 2],sig_done[bkt_write_sig]
	ctx_arb[bkt_write_sig]
	
	br[END#]

NOT_FOUND_IN_SEC#:

	// Check whether next_ptr = NULL or not
	alu[sec_table, lm_vc_index_mask, AND, $bkt_in[7]]
	bne[READ_NEXT_TABLE#]               // br if next_ptr != NULL
	

	// Copy sram_offset to prv bkt_index	
	move(bkt_index, sram_offset)

ALLOC_NEW_SEC_BUFFER#:

	// Allocate one secondary table after prev secondary table.
	buf_alloc($buf_handle, BUF_FREE_LIST1, VC_HASH_SEC_TABLE_POOL, \
				alloc_sig, alloc_sig, --)
	
	alu[--, --, B, $buf_handle, <<8]	// [23:0] contains buf handle
	beq[ALLOC_NEW_SEC_BUFFER#]

	dl_buf_get_desc(sram_offset, $buf_handle)
	
	// Currently sec table is in SRAM channel 0. But in future when you change
	// to channel 1, remember to do this mapping.

	// dl_buf_get_desc doesn't take care of bank addressing
	// add 0x40000000 for channel 1
	alu_shf[sram_offset, sram_offset, OR, 0x40, <<24]
	
	// Subtract the base address
	alu[next_ptr, sram_offset, -, hash_sec_base_sram]

	// division by size of sec table i.e. 32 bytes
	alu[next_ptr, mask_lower_half, AND, next_ptr, >>VC_HASH_SEC_SHF_VAL]
	
	ld_field[$bkt_in[7], 0011, next_ptr]
	
	// Update the prev sec table with next ptr	
	sram[write, $bkt_in[7], bkt_index, 28, 1],sig_done[bkt_write_sig]
	ctx_arb[bkt_write_sig]
	
	// Now write to the secondary table
	alu[$bkt_in[0], --, B, vc_key]
	ld_field[$bkt_in[1], 0011, vc_id]

	// Update the secondary table
	sram[write, $bkt_in[0], sram_offset, 0, 2],sig_done[bkt_write_sig]
	ctx_arb[bkt_write_sig]
		
					
END#:

	xbuf_free($bkt_in)

.end
#endm

///////////////////////////////////////////////////////////////////////////////
// _aal5_rx_rxcinfo_add
//
// Description:
//
//		This is used to add RXC info entry into RXC table.
//
// Input: 
//		exp_sop:  Expect SOP bit
//
//		vc_id:	  VC ID (range [0 - 64K-1] )
//
//		vc_st:	  VC state (invalid VC = 1, valid VC = 0)
//
//		aal_ty:   AAL_type supported by this VC ( AAL_type = 0 for AAL5)
//
//		flist:    If not AAL5, then use this flist id to get the buffer
//
//		crc :	  crc_init (initialized to 0xffff ffff)
//
//				  from the freelist
//		excep_bit:This bit is set when the VC is reserved to receive signalling
//				  frames
//		hdr_ty:   The type of Header the frames coming on this VC will be having.
//
// Output:
//		N/A
//
// Size: 
//		N/A
//
///////////////////////////////////////////////////////////////////////////////
#macro _aal5_rx_rxcinfo_add(exp_sop, vc_id, vc_st, aal_ty, flist, crc, \
				excep_bit, hdr_ty)
.begin

	.sig rxc_write_sig

	.reg rxc_offset
	.reg tmp

	; Set RXC info here.
	xbuf_alloc($rxc_info, 9, write)

	// Each VC info entry is 16LWs
	alu_shf[rxc_offset, --, B, vc_id, <<AAL5_RX_VC_INFO_SHF_VAL]	

	// | exp_sop(1)|res(7)|vc_state(1)|aal_type(3)|free_list(4)|  res(16)   |
	alu[tmp, --, b, flist, <<16]
	alu[tmp, tmp, OR, aal_ty, <<20]
	alu[tmp, tmp, OR, vc_st, <<23]
	alu[$rxc_info[0], tmp, OR, exp_sop, <<31]

	// | crc_residue(32) |
	move($rxc_info[1], crc)

	// | curr_buf_handle(32) |
	move($rxc_info[2], 0x0)

	// | sop_buf_handle (32) |
	move($rxc_info[3], 0x0)

	// | sop_buf_size(16)  |  offset (16)  |
	move($rxc_info[4], 0x0)

	// | buffer_offset(16) | curr_pkt_size (16)  |
	move($rxc_info[5], 0x0)	

	// | drop_bit(1)|excep_bit(1)|res(2)|hdr_type(4)| prev_to_prev_buf_addr(24)|
	alu[tmp, --, B, hdr_ty, <<24]
	alu[$rxc_info[6], tmp, OR, excep_bit, <<30]

	// | prev_buf_size (16) | bytes_remain(8) | cell_count (8) |
	move($rxc_info[7], 0x0)

	// | prev_buf_handle (32) |
	move($rxc_info[8], 0x0)

	// Set ref_cnt 8(0x8-one less than actual) and OV[25] bit to override refcnt
	alu_shf[--, --, B, 0x18, <<21]
	sram[write, $rxc_info[0], vc_info_sram_base, rxc_offset, max_9], indirect_ref, \
				ctx_swap[rxc_write_sig]

	xbuf_free($rxc_info)

.end
#endm

///////////////////////////////////////////////////////////////////////////////
// _aal5_rx_hash_rxc_init
//
// Description:
//
//		This initializes the hash table and RXC info entry into VC Info table.
//
// Input: 
//		N/A
//
// Output:
//		N/A
//
// Size: 
//		N/A
//
///////////////////////////////////////////////////////////////////////////////

#macro	_aal5_rx_hash_rxc_init()
.begin

	.reg vc_count
	.reg vpi, vci, input_port, vc_id, vc_key
		
	.reg exp_sop_bit, vc_state, aal_type, free_list, crc_init, exception_bit, hdr_type


	//	Initialize the freelist for secondary hash table in sram.
	//	This function will be called from control plane when running on hardware
	_aal5_rx_sec_hash_init()


	// Initilize 20 VCs
	move(vc_count, 0)
	move(vpi, 0x96)
	move(vci, 0x96)
	move(input_port, 0x0)
	move(vc_id, 0)

	.while(vc_count<8)

		// Create a key VPI/VCI/port#(32)=(8/16/4).
		alu[vc_key, input_port, OR, vpi, <<20]
		alu[vc_key, vc_key, OR, vci, <<4]

		// Add VC in the hash table (either primary or secondary table)
		_aal5_rx_vc_add(vc_key, vc_id) 

	
		; (normal VCs)
			move(exp_sop_bit, 1)				
			move(vc_state, 0)	
			move(aal_type, 0)	
			move(free_list, 0)
			move(exception_bit, 0)

			#ifdef MPLS_OVER_ATM
				#ifdef MPLS_INGRESS
					move(hdr_type, PKT_PROTO_LLC_SNAP)
				#else
					move(hdr_type, PKT_PROTO_MPLS)
				#endif
			#else
				move(hdr_type, PKT_PROTO_LLC_SNAP)
			#endif			

			move(crc_init, 0xffffffff)
										
		; Initialize the RXC for a particular vc_id
		_aal5_rx_rxcinfo_add(exp_sop_bit, vc_id, vc_state, aal_type, free_list,
					 crc_init, exception_bit, hdr_type)

		
		alu[vc_id, vc_id, +, 1]
		alu[vc_count, vc_count, +, 1]
	
		alu[vpi, vpi, +, 1]
		alu[vci, vci, +, 1]
    	alu[input_port, input_port, +, 1]

        .if(vc_count == 4)
			move(input_port, 0x0)
        .endif

	.endw

.end
#endm


#endif  // SPHY_4_8_VC_INIT

#endif  // _AAL5_RX_ONE_ME_FP_UTIL_UC_
