 /*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1999-2003 By Intel Corporation.  All rights reserved.
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/*****************************************************************************
 * File:  csix_rx_buf_zero_proc.c
 *
 * Abstract: 
 *        Processes null prefetched buffer case.
 *
 * Contents:
 *        Definitions of following local function:
 *            CsixRxBufZeroProc     
 *
 *****************************************************************************/
#ifndef __CSIX_RX_BUF_ZERO_PROC_C__
#define __CSIX_RX_BUF_ZERO_PROC_C__

#include "csix_rx.h"

/*****************************************************************************
 * Abstract:
 *      Processes null prefetched buffer case.
 *
 * Side Effects:
 *    None
 *
 * Assumptions:
 *    None
 *
 *****************************************************************************/

INLINE void 
CsixRxBufZeroProc(
    uint32_t eleAddr,		/* IN: RBUF element address in receive status word */
	uint32_t payloadLen,	/* IN: payload length in receive status word */
	uint32_t extClass)		/* IN: clas ID in CSIX frame extension header */
{						
	cam_lookup_t lkupResult;
	uint32_t camEntry;
	uint32_t lookupKey;
	uint32_t SeqNum;
    uint32_t ingressBlade;			//ingress blade number

	SIGNAL sig_msfRead;
	SIGNAL sig_bufAlloc;
	__declspec(local_mem shared aligned(8)) rxc_t *prxc;
#ifdef	COUNTERS
	__declspec(sram_write_reg) int byteCount = payloadLen;
	SIGNAL sig_counters;
#endif /* COUNTERS */	

	/* get prepend header (at max 4 LW) */ 
	msf_read((__declspec(sram_read_reg) void *)&prpnd, 
			(volatile void *)(RBUF_TBUF_BASE + eleAddr), 4, sig_done,
			&sig_msfRead);

	//	Prefetch a new buffer so that it can be used next time around.
	Dl_BufAlloc(&prefetchHandle, BUF_FREE_LIST0, BUF_POOL, &sig_bufAlloc, SIG_NONE, ___);

	//	Wait MSF read finish and sig_nextThrd_1 signal
	wait_for_all(&sig_nextThrd_1, &sig_msfRead);
	__implicit_read(&sig_msfRead);

	/* buf_zero_phase_2: */
	SignalNextCtx(ntSigGpr_1);
	ingressBlade=prpnd.ingressSrcBlade;
	lookupKey = extClass | (ingressBlade << 4);	

#ifdef COUNTERS
	if(prpnd.sop)
	{
		sram_incr((volatile void *)(CSIX_RX_COUNTER_BASE + (lookupKey << 4) + 
					PKT_RECEIVED_CNT_IDX));
		sram_incr((volatile void *)(CSIX_RX_COUNTER_BASE + (lookupKey << 4) + 
					PKT_DROPPED_CNT_IDX));
	}
	sram_add((__declspec(sram_write_reg) void *)&byteCount, 
				(volatile void *)(CSIX_RX_COUNTER_BASE + (lookupKey << 4) + 
				BYTE_RECEIVED_CNT_IDX),
				sig_done, &sig_counters);
#endif /* COUNTERS */

	lkupResult = cam_lookup(lookupKey);
	camEntry = lkupResult.entry_num;
	prxc = (__declspec (local_mem shared aligned(8)) rxc_t *) &rxCtxLm[camEntry];
	
	if(!lkupResult.hit)	/* buf_zero_cam_miss */
	{
		uint32_t tmpTag; 
		__declspec(sram_read_reg) rxc_t inRxCtx;
		__declspec(sram_write_reg) rxc_t outRxCtx;
		SIGNAL sig_sramRead;
		SIGNAL sig_sramWrite;
		sram_read((__declspec(sram_read_reg) void *)&inRxCtx, 
				(int *)(CSIX_RX_CTX_SRAM_BASE + (RXC_SIZE * 4) * lookupKey), 
				3, sig_done, &sig_sramRead);
		tmpTag = cam_read_tag(camEntry);
		outRxCtx.value[0] = prxc->value[0];
		outRxCtx.value[1] = prxc->value[1];
		outRxCtx.value[2] = prxc->value[2];
		outRxCtx.value[3] = prxc->value[3];
		outRxCtx.value[4] = prxc->value[4];
		outRxCtx.value[5] = prxc->value[5];
		outRxCtx.value[6] = prxc->value[6];
		outRxCtx.value[7] = prxc->value[7];

		sram_write((__declspec(sram_write_reg) void *)&outRxCtx, 
				(int *)(CSIX_RX_CTX_SRAM_BASE + (RXC_SIZE * 4) * tmpTag),
				8, sig_done, &sig_sramWrite);
		cam_write(camEntry, lookupKey, NORMAL_STATE);

		wait_for_all(&sig_nextThrd_2, &sig_sramRead, &sig_sramWrite);
		__implicit_read((void *)&outRxCtx);
		__implicit_read(&sig_sramRead);
		__implicit_read(&sig_sramWrite);

		/* buf_zero_cam_miss_phase_3: */
		prxc->value[0] = inRxCtx.value[0];
		prxc->value[1] = inRxCtx.value[1];
		prxc->value[2] = inRxCtx.value[2];
	}
	else /* buf_zero cam hit */
	{
		wait_for_all(&sig_nextThrd_2);			
	}

	/* buf_zero_common_phase_3: */
	SignalNextCtx(ntSigGpr_2);
	SeqNum = prxc->cellSeqNum; 
	prxc->cellSeqNum = 0; 

#ifdef COUNTERS 
	wait_for_all(&sig_nextThrd_3, &sig_bufAlloc, &sig_counters);
	__implicit_read(&byteCount);	/* extends life range of byteCount */				
	__implicit_read(&sig_bufAlloc);
	__implicit_read(&sig_counters);
#else /* COUNTERS */
	wait_for_all(&sig_nextThrd_3, &sig_bufAlloc);	
	__implicit_read(&sig_bufAlloc);
#endif /* COUNTERS */

	/* buf_zero_common_phase_4 */
	SignalNextCtx(ntSigGpr_3);

	if(SeqNum)
	{
		DropCurrentPkt(prxc->sopBufHandle , prxc->curBufHandle, lookupKey);
	}

 	prefetchDataHndl = Dl_BufGetData(prefetchHandle);
	FreeRbufElement(eleAddr >> PO2_RBUF_SIZE);
	AddThreadToFreelist();

	wait_for_all(&sig_rxEvent);
	__implicit_read(&sig_rxEvent);
} /* CsixRxBufZeroProc */

#endif // __CSIX_RX_BUF_ZERO_PROC_C__


