 /*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1999-2003 By Intel Corporation.  All rights reserved.
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/*****************************************************************************
 * File:  csix_rx_eop_proc.c
 *
 * Abstract:
 *        Processes csix frame with EOP bit set.
 *
 * Contents:
 *        Definitions of following local function:
 *            CsixRxEopProc     
 *
 *****************************************************************************/
#ifndef __CSIX_RX_EOP_PROC_C__
#define __CSIX_RX_EOP_PROC_C__

#include "csix_rx.h"

/*****************************************************************************
 * Abstract:
 *    Processes csix frame with EOP bit set.
 *
 * Side Effects:
 *    None
 *
 * Assumptions:
 *    None
 *
 *****************************************************************************/

INLINE void
CsixRxEopProc(
    uint32_t eleAddr,		/* IN: RBUF element address in receive status word */
	uint32_t payloadLen,	/* IN: payload length in receive status word */
	uint32_t tmOffset,		/* IN: offset of TM payload */
	uint32_t extClass,		/* IN: clas ID in CSIX frame extension header */
	uint32_t ingressBlade)		/* IN: ingress blade number */
{
	cam_lookup_t lkupResult;		/* cam lookup result */
	uint32_t camEntry;					/* cam entry from lkupResult */
	uint32_t lookupKey;
	uint32_t dataLen;
	uint32_t bufSize;
    uint32_t expectSeqNum;				/* expected seq num */
	uint32_t pktSize;
	dl_buf_handle_t sopBufHandle;
	dl_buf_handle_t curBufHandle;
	uint32_t outputPort;
	uint32_t classId;

	SIGNAL sig_sramRead;
	SIGNAL sig_sramWrite;
	SIGNAL sig_bufAlloc;
	SIGNAL sig_sramOp;
	__declspec(local_mem shared aligned(8)) rxc_t *prxc;
								
#ifdef	COUNTERS
	__declspec(sram_write_reg) int byteCount = payloadLen;	/* byte counter */
	SIGNAL sig_counters;
#endif	/* COUNTERS */

#ifdef	CELL_MODE
	uint32_t cellCount; 
	uint32_t sopCellCount; 
	uint32_t rmdr;
#endif

	lookupKey = extClass | (ingressBlade << 4);	/*	get cam lookup key */

#ifdef COUNTERS
	sram_add((__declspec(sram_write_reg) void *)&byteCount,
				(volatile void *)(CSIX_RX_COUNTER_BASE + (lookupKey << 4) + 
				BYTE_RECEIVED_CNT_IDX),
				sig_done, &sig_counters);
#endif /* COUNTERS */

	/* do cam lookup */
	lkupResult = cam_lookup(lookupKey);
	camEntry = lkupResult.entry_num;
	prxc = (__declspec (local_mem shared aligned(8)) rxc_t *) &rxCtxLm[camEntry];

	/* check lookup result */
	if(!lkupResult.hit)	/* eop_cam_miss */
	{
		uint32_t tmpTag; 
		__declspec(sram_read_reg) rxc_t inRxCtx;
		__declspec(sram_write_reg) rxc_t outRxCtx;

		tmpTag = cam_read_tag(camEntry);
		cam_write(camEntry, lookupKey, NORMAL_STATE);

		sram_read((__declspec(sram_read_reg) void *)&inRxCtx, 
					(int *)(CSIX_RX_CTX_SRAM_BASE + (RXC_SIZE * 4) * lookupKey), 8, sig_done, &sig_sramRead);
		outRxCtx.value[0] = prxc->value[0];
		outRxCtx.value[1] = prxc->value[1];
		outRxCtx.value[2] = prxc->value[2];
		outRxCtx.value[3] = prxc->value[3];
		outRxCtx.value[4] = prxc->value[4];
		outRxCtx.value[5] = prxc->value[5];
		outRxCtx.value[6] = prxc->value[6];
		outRxCtx.value[7] = prxc->value[7];

		sram_write((__declspec(sram_write_reg) void *)&outRxCtx, 
					(int *)(CSIX_RX_CTX_SRAM_BASE + (RXC_SIZE * 4) * tmpTag), 8, sig_done, &sig_sramWrite);

		wait_for_all(&sig_nextThrd_2, &sig_sramRead, &sig_sramWrite);
		__implicit_read((void *)&outRxCtx);
		__implicit_read(&sig_sramRead);
		__implicit_read(&sig_sramWrite);

		/* eop_cam_miss_phase_3: */
		/* move inRxCtx to rxCtxLm in the case of miss */
		prxc->value[0] = inRxCtx.value[0];
		prxc->value[1] = inRxCtx.value[1];
		prxc->value[2] = inRxCtx.value[2];
		prxc->value[3] = inRxCtx.value[3];
		prxc->value[4] = inRxCtx.value[4];
		prxc->value[5] = inRxCtx.value[5];
		prxc->value[6] = inRxCtx.value[6];
		prxc->value[7] = inRxCtx.value[7];
	}
	else /* eop_cam_hit: */
	{
		wait_for_all(&sig_nextThrd_2);			
	}


	/* eop_common_phase_3: */
	/* send next thread sig 2 */
	SignalNextCtx(ntSigGpr_2);

	expectSeqNum = prxc->cellSeqNum;

	/* 
	 * Phase 3 is the only phase in which you can/should access rxc in local memory
	 * Save these for later use in phase 4.
	 */

	sopBufHandle 	= prxc->sopBufHandle;	
	curBufHandle 	= prxc->curBufHandle;
	outputPort		= prxc->outputPort;
	classId		= prxc->classId;

	if(prpnd.seqNumb == expectSeqNum)   /* sequence number match */ 
	{

#ifdef TMP_FIX_DL_METAFLUSH
		// tmp define register to write the result to SRAM.
		__declspec(sram_write_reg) int tmp_sr_wr;
		int sram_addr;
#endif //#ifdef TMP_FIX_DL_METAFLUSH
		prxc->cellSeqNum=0;	//reset for new packet

		/* get cframe length */
		dataLen = payloadLen - NO_SOP_PRPEND_HDR;

		/* get new pkt size */
		pktSize = (int)prxc->pktSize + dataLen;

		/* get data handle in dram */
		dramHandle = (uint32_t)(prxc->curDataHandle);

		/* The RBUF data start address = element_addr + 
		   no sop prepend header length for EOP Cframe */
		eleAddr = eleAddr + NO_SOP_PRPEND_HDR;
	
		/*	move received cframe to dram */
		MoveRbufToPktBuf(dramHandle, eleAddr, dataLen);

#ifdef CELL_MODE
		cellCount = prxc->curBufHandle.seg_count;	/* get cell count */
		rmdr = prxc->remainder;					/* get remainder */
		GetCellCount(&cellCount, &rmdr, dataLen); 	/* get new cell count */
#endif

		/*	
		 * prepare meta data 
		 */
		bufSize = prxc->curBufSize + dataLen;
		if(prxc->sopBufHandle.lw_offset != prxc->curBufHandle.lw_offset)
		{
#ifdef CELL_MODE
			/* get SOP cell count */
			sopCellCount = prxc->sopBufHandle.seg_count;
#endif

			/*
			 * eop_not_sop_buf, first update pktSize in SOP
			 */
			dlMeta.packetSize 		= pktSize;
			dlMeta.freeListId 		= 0;
			dlMeta.rxStat 			= 0;
			dlMeta.headerType 		= 0;

#ifdef TMP_FIX_DL_METAFLUSH
			/*
			 * tmp fix to Dl_MetaFlushCache
			 */
			dlMeta.value[0]=dlMeta.value[2];
#endif /* TMP_FIX_DL_METAFLUSH */

			/* 
			 * Flush the meta data to SRAM. One LW (2) of the meta data is
			 * flushed. 
			 */
#ifndef TMP_FIX_DL_METAFLUSH
			Dl_MetaFlushCache(prxc->sopBufHandle, &sig_sramOp, SIG_NONE, 2, 1);	
#else /* TMP_FIX_DL_METAFLUSH */				
				tmp_sr_wr=dlMeta.value[2];	 //value to write
				sram_addr= (prxc->sopBufHandle.lw_offset << 2) + 8;
				sram_write((__declspec(sram_write_reg) void *)&tmp_sr_wr, 
					(int *)sram_addr, 1, sig_done, &sig_sramOp);
#endif /* TMP_FIX_DL_METAFLUSH */
				
			/*
			 * prepare meta data for EOP
			 */
			dlMeta.bufferNext.value	= IX_DL_BUF_NULL;
			dlMeta.bufferSize 		= bufSize;
			dlMeta.offset 				= 0;
			/*
			 * Flush the meta data to SRAM. All eight LW's (0 to 1) 
			 * of the meta data are flushed.
			 */
			Dl_MetaFlushCache(prxc->curBufHandle, &sig_sramWrite, SIG_NONE, 0, 2);	

#ifdef CELL_MODE
			/*
			 * update current buffer cell count 
			 */
			dlMeta.bufferNext.value = (prxc->curBufHandle.lw_offset) |
										(cellCount << 24);
			/* 
			 * Flush the meta data to SRAM one LW (0) 
			 * of the meta data are flushed. 
			 */
			Dl_MetaFlushCache(prxc->prvBufHandle, &sig_sramOp, SIG_NONE, 0, 1);	
#endif /* CELL_MODE */

#ifdef COUNTERS
#ifdef CELL_MODE
			wait_for_all(&sig_nextThrd_3, &sig_sramOp, &sig_sramWrite, &sig_sramRead,
					&sig_dramWrite, &sig_counters);	
			__implicit_read(&byteCount);	/* extends life range of byteCount */
			__implicit_read(&sig_sramOp);
			__implicit_read(&sig_sramWrite);
			__implicit_read(&sig_sramRead);
			__implicit_read(&sig_dramWrite);
			__implicit_read(&sig_counters);
#ifdef TMP_FIX_DL_METAFLUSH
			__implicit_read(&tmp_sr_wr); /* extends life range of tmp_sr_wr */
#endif /* TMP_FIX_DL_METAFLUSH */												
#else
			wait_for_all(&sig_nextThrd_3, &sig_sramOp, &sig_sramWrite,
					&sig_dramWrite, &sig_counters);
			__implicit_read(&byteCount);	/* extends life range of byteCount */
			__implicit_read(&sig_sramOp);
			__implicit_read(&sig_sramWrite);
			__implicit_read(&sig_dramWrite);
			__implicit_read(&sig_counters);
#ifdef TMP_FIX_DL_METAFLUSH
			__implicit_read(&tmp_sr_wr); /* extends life range of tmp_sr_wr */
#endif /* TMP_FIX_DL_METAFLUSH */				
#endif
#else	/* COUNTERS */
#ifdef CELL_MODE
			wait_for_all(&sig_nextThrd_3, &sig_sramOp, &sig_sramWrite, &sig_sramRead,
					&sig_dramWrite);					
			__implicit_read(&sig_sramOp);
			__implicit_read(&sig_sramWrite);
			__implicit_read(&sig_sramRead);
			__implicit_read(&sig_dramWrite);
#ifdef TMP_FIX_DL_METAFLUSH
			__implicit_read(&tmp_sr_wr); /* extends life range of tmp_sr_wr */
#endif /* TMP_FIX_DL_METAFLUSH */												
#else
			wait_for_all(&sig_nextThrd_3, &sig_sramOp, &sig_sramWrite,
					&sig_dramWrite);
			__implicit_read(&sig_sramOp);
			__implicit_read(&sig_sramWrite);
			__implicit_read(&sig_dramWrite);										
#ifdef TMP_FIX_DL_METAFLUSH
			__implicit_read(&tmp_sr_wr); /* extends life range of tmp_sr_wr */
#endif /* TMP_FIX_DL_METAFLUSH */										
#endif
#endif /* COUNTERS */

		}
		else  //eop_and_sop_buf 
		{

		/*
		 * eop_and_sop_buf
		 */
#ifdef	CELL_MODE
			sopCellCount = cellCount;
#endif
			/*
			 * only update second LW (buffer size, offset) and third LW 
			 * (packet_size field and related fields in the same long word)
			 */				
			dlMeta.bufferSize 		= bufSize;
			dlMeta.offset 			= prxc->sopBufOffset;	
			dlMeta.freeListId	 	= 0;
			dlMeta.rxStat 			= 0;
			dlMeta.packetSize 		= pktSize;
			dlMeta.headerType		= 0;
#ifdef TMP_FIX_DL_METAFLUSH
			/*
			 * tmp fix to Dl_MetaFlushCache
			 */
			dlMeta.value[0]=dlMeta.value[1];
			dlMeta.value[1]=dlMeta.value[2];
#endif //#ifdef TMP_FIX_DL_METAFLUSH

			/* Flush the meta data to SRAM. Two LW (1 to 2) 
			 * of the meta data are flushed. 
			 */
			Dl_MetaFlushCache(prxc->sopBufHandle, &sig_sramOp, SIG_NONE, 1, 2);	
#ifdef COUNTERS
			wait_for_all(&sig_nextThrd_3, &sig_sramOp, &sig_dramWrite, &sig_counters);
			__implicit_read(&byteCount);	/* extends life range of byteCount */
			__implicit_read(&sig_sramOp);
			__implicit_read(&sig_dramWrite);
			__implicit_read(&sig_counters);
#else /* COUNTERS */	
			wait_for_all(&sig_nextThrd_3, &sig_sramOp, &sig_dramWrite);
			__implicit_read(&sig_sramOp);
			__implicit_read(&sig_dramWrite);
#endif /* COUNTERS */
		}

		/*
		 * eop_seq_match_phase_4
		 */

		/* send next thread signal 3 to the next thread */
		SignalNextCtx(ntSigGpr_3);
		
		FreeRbufElement(eleAddr >> PO2_RBUF_SIZE);	/* free RBUF element */

		AddThreadToFreelist();	/* add thread to freelist */

		/*
		 * send the packet to next microblock
		 */

		/* By default, a buffer will have sop and eop bit set. */

		if (sopBufHandle.lw_offset == curBufHandle.lw_offset)
		{
			/* 
			 * One Buffer Case. curBufHandle is treated as the last buffer in 
			 * in the buffer chain, which is null in this case
			 */
			 
			curBufHandle.value = IX_DL_BUF_NULL;	/* sop = eop = count = 0; lw_offset = 0xff */
		}
		else
		{
			/* More than one buffer case */

			sopBufHandle.eop = 0;	
			curBufHandle.sop = 0;

		}


#ifdef CELL_MODE
#ifdef QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER
		CsixRxSink(sopBufHandle, curBufHandle, pktSize, 
			sopCellCount, extClass, outputPort, dlRing, 
			lookupKey, sinkSigMask);
#else /* QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER */
		CsixRxSink(sopBufHandle, curBufHandle, pktSize, 
			sopCellCount, classId, outputPort, dlRing, 
			lookupKey, sinkSigMask);
#endif /* QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER */
#else /* CELL_MODE */
#ifdef QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER
		CsixRxSink(sopBufHandle, curBufHandle, pktSize,
			0, extClass, outputPort, dlRing, 
			lookupKey, sinkSigMask);
#else /* QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER */
		CsixRxSink(sopBufHandle, curBufHandle, pktSize,
			0, classId, outputPort, dlRing, 
			lookupKey, sinkSigMask);
#endif /* QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER */
#endif /* CELL_MODE */

	} /* end of eop_seq_num_match */
	else 
	{
		/*
		 * eop_seq_num mismatch
		 * the original sequence number is stored in expectSeqNum
		 */
		prxc->cellSeqNum=0;		/* reset sequence number */

#ifdef COUNTERS
		wait_for_all(&sig_nextThrd_3, &sig_counters);
		__implicit_read(&byteCount);	/* extends life range of byteCount */
		__implicit_read(&sig_counters);

#else /* COUNTERS */
		wait_for_all(&sig_nextThrd_3);
#endif /* COUNTERS */

		/*
		 * eop_seq_mismatch_phase_4
		 */

		/* send next thread signal 3 to the next thread */
		SignalNextCtx(ntSigGpr_3);

		if(expectSeqNum)	/* expected sequnce number is not zero */
		{											
			/*
			 * drop the incomplete packet
			 */

			DropCurrentPkt(sopBufHandle, curBufHandle, lookupKey);
		}

		FreeRbufElement(eleAddr >> PO2_RBUF_SIZE); /* free RBUF element */
		AddThreadToFreelist();	/*  add thread to freelist */
	
		wait_for_all(&sig_rxEvent);
		__implicit_read(&sig_rxEvent);
	}
} /* CsixRxEopProc */

#endif // __CSIX_RX_EOP_PROC_C__


