 /*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1999-2003 By Intel Corporation.  All rights reserved.
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/*****************************************************************************
 * File:  csix_rx_err_proc.c
 *
 * Abstract: 
 *        MicroC file for CSIX RX microblock utility functions.
  *
 * Contents:
 *        Definitions of following local functions:
 *            CsixRxErrProc       
 *
 *****************************************************************************/
#ifndef __CSIX_RX_ERR_PROC_C__
#define __CSIX_RX_ERR_PROC_C__

#include "csix_rx.h"


INLINE void 
CsixRxErrProc(
    uint32_t eleAddr,		/* IN: RBUF element address in receive status word */
	uint32_t extClass)		/* IN: clas ID in CSIX frame extension header */
{
	cam_lookup_t lkupResult;
	uint32_t camEntry;
	uint32_t lookupKey;
	uint32_t SeqNum;
	uint32_t ingressBlade;		/* ingress blade number */

	__declspec(local_mem shared aligned(8)) rxc_t *prxc;

	if(!(rsw.null)) /* not null error */
	{

		SIGNAL sig_msfRead;

		/* start of normal_proc_phase_1 */
		/* get prepend header (at max 4 LW) */ 
		msf_read((__declspec(sram_read_reg) void *)&prpnd, 
				(volatile void *)(RBUF_TBUF_BASE + eleAddr), 4, sig_done,
				&sig_msfRead);

		wait_for_all(&sig_nextThrd_1, &sig_msfRead);
		__implicit_read(&sig_msfRead);

        /*
		 * err_phase_2
		 */
		SignalNextCtx(ntSigGpr_1);
		ingressBlade=prpnd.ingressSrcBlade;
		lookupKey = extClass | (ingressBlade << 4);	

		/* 
		 * do CAM lookup
		 */
		lkupResult = cam_lookup(lookupKey);
		camEntry = lkupResult.entry_num;
		prxc = (__declspec (local_mem shared aligned(8)) rxc_t *) &rxCtxLm[camEntry];


		if(!lkupResult.hit)	/* eop_cam_miss */
		{

			uint32_t tmpTag; 
			__declspec(sram_read_reg) rxc_t inRxCtx;
			__declspec(sram_write_reg) rxc_t outRxCtx;
			SIGNAL sig_sramRead;
			SIGNAL sig_sramWrite;
			sram_read((__declspec(sram_read_reg) void *)&inRxCtx, 
					(int *)(CSIX_RX_CTX_SRAM_BASE + (RXC_SIZE * 4) * lookupKey), 3, sig_done, &sig_sramRead);
			tmpTag = cam_read_tag(camEntry);
			outRxCtx.value[0] = prxc->value[0];
			outRxCtx.value[1] = prxc->value[1];
			outRxCtx.value[2] = prxc->value[2];
			outRxCtx.value[3] = prxc->value[3];
			outRxCtx.value[4] = prxc->value[4];
			outRxCtx.value[5] = prxc->value[5];
			outRxCtx.value[6] = prxc->value[6];
			outRxCtx.value[7] = prxc->value[7];

			sram_write((__declspec(sram_write_reg) void *)&outRxCtx, 
					(int *)(CSIX_RX_CTX_SRAM_BASE + (RXC_SIZE * 4) * tmpTag), 8, sig_done, &sig_sramWrite);
			cam_write(camEntry, lookupKey, NORMAL_STATE);

			wait_for_all(&sig_nextThrd_2, &sig_sramRead, &sig_sramWrite);
			__implicit_read((void *)&outRxCtx);
			__implicit_read(&sig_sramRead);
			__implicit_read(&sig_sramWrite);

			/*
			 * cam_miss_phase_3
			 */
			/* move $xfers CRX to LM */
			prxc->value[0] = inRxCtx.value[0];
			prxc->value[1] = inRxCtx.value[1];
			prxc->value[2] = inRxCtx.value[2];
		}
		else /* cam hit */
		{
			wait_for_all(&sig_nextThrd_2);			
		}

		/* 
		 * err_common_phase_3
		 */
		SignalNextCtx(ntSigGpr_2);	/* signal next thread */ 

		SeqNum = prxc->cellSeqNum; 	/* get sequence number */
		prxc->cellSeqNum = 0; 		/* set sequence number */

		wait_for_all(&sig_nextThrd_3);			

		/*
		 * err_common_phase_4
		 */
		SignalNextCtx(ntSigGpr_3); /* signal next thread */

		if(SeqNum)
		{
			/* drop incomplete packet */
			DropCurrentPkt(prxc->sopBufHandle , prxc->curBufHandle, 
							lookupKey);
		}

		FreeRbufElement(eleAddr >> PO2_RBUF_SIZE);	/* free current RBUF */
		AddThreadToFreelist();			/* add this thread to freelist */ 

		wait_for_all(&sig_rxEvent);
		__implicit_read(&sig_rxEvent);
	}
	else /* null error */
	{
		wait_for_all(&sig_nextThrd_1);

		/*
		 * null_err_proc_phase_2
		 */
		SignalNextCtx(ntSigGpr_1); /* signal next thread */
		wait_for_all(&sig_nextThrd_2);

		/*
		 * null_err_proc_phase_3
		 */
		SignalNextCtx(ntSigGpr_2);	/* signal next thread */
		wait_for_all(&sig_nextThrd_3);


		/*
		 * null_err_proc_phase_4
		 */
		SignalNextCtx(ntSigGpr_3);	/* signal next thread */

		AddThreadToFreelist();	/* add this thread to freelist */
		wait_for_all(&sig_rxEvent);
		__implicit_read(&sig_rxEvent);
	}
}	/* CsixRxErrProc */

#endif // __CSIX_RX_ERR_PROC_C__
