 /*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1999-2003 By Intel Corporation.  All rights reserved.
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/*****************************************************************************
 * File:  csix_init.c
 *
 * Abstract: 
 *        MicroC file for CSIX RX microblock initialization.
 *
 * Contents:
 *        Definitions of following local function:
 *            CsixRxInit
 *
 *****************************************************************************/
#ifndef __CSIX_RX_INIT_C__
#define __CSIX_RX_INIT_C__

#include "csix_rx.h"

extern void system_init();

/*****************************************************************************
 * Abstract:
 *    Initialized hardware MSF registers, software variables and strucrtures
 *
 * Side Effects:
 *    None
 *
 * Assumptions:
 *    None
 *
 *****************************************************************************/

INLINE void 
CsixRxInit(
)
{

	SIGNAL sig_bufAlloc;

	/* get current thread */
	int thread = GetCurrentCtx();

	/* get current me */
	int thisME = GetCurrentMe();

	/* store the address of signal sig_rxEvent */
	int rxEventSigAddr = __signal_number(&sig_rxEvent);

	/* store the address of rsw */
	int rswAddr = __xfer_reg_number((void *)&rsw);

#ifdef CELL_MODE
	int cell_size = CELL_SIZE_CNST;
#endif /* CELL_MODE */


	/* initialize dispatch loop vaiables to known state */

	/* set dlNextBlock to NULL */
	dlNextBlock = IX_NULL; 

	/* set dlBuHandle NULL */
	dlBufHandle.lw_offset = IX_NULL;

	/* set dlEopBufHandle NULL. */
	dlEopBufHandle.lw_offset = IX_NULL;
	
	if (thread == 0)
	{

		/* ntSigGpr_1 and sig_nextThrd_1 are global */
		ntSigGpr_1 = init_interthread_sig(__signal_number(&sig_nextThrd_1));

		/* ntSigGpr_2 and sig_nextThrd_2 are global */
		ntSigGpr_2 = init_interthread_sig(__signal_number(&sig_nextThrd_2));

		/* ntSigGpr_3 and sig_nextThrd_3 are global */
		ntSigGpr_3 = init_interthread_sig(__signal_number(&sig_nextThrd_3));

		/* clear all CAM entries */
		cam_clear();
		SetAllCamEntryState(NORMAL_STATE);

		/* initialize all RXC's in LM & sram */ 
		CsixRxInitAllRxc();

		/*
		 * Do a fast write to the RX_UP_CTRL_0 CSR. 
		 * Note ms 16 bits is data, i.e. RX_UP_CTRL_0_DATA.
		 * and ls 16 bits is address, i.e. RX_UP_CTRL_0.
		 */
		msf_fast_write((void *)(RX_UP_CTRL_0 | (RX_UP_CTRL_0_DATA << 16)));


		/* dispatch loop system initialization */
		system_init();

		/* setup starting random buffer offset and random offset mask */
		randBufOffset = MAX_BUFFER_OFFSET;
	}
	else
		wait_for_all(&sig_nextThrd_1);

	/* ring number to be used in scratch[put] */
	dlRing = CSIX_RX_RING_OUT <<2;

	/* prefetch a buffer handle from SRAM */
	Dl_BufAlloc(&prefetchHandle, BUF_FREE_LIST0, BUF_POOL, &sig_bufAlloc, __signals(&sig_bufAlloc), ___);
	__implicit_read(&sig_bufAlloc);

	/* compute the buffer data address in DRAM of the prefetched buffer handle */
	prefetchDataHndl = Dl_BufGetData(prefetchHandle);

	/* set the freelist register for each thread */
	rxFreelistReg = rswAddr | (thread << THREAD_BITS) | (thisME << ME_NUMBER_BITS) | \
					  (rxEventSigAddr << SIGNAL_NUMBER_BITS);
	rxFreelistReg	= RX_THREAD_FREELIST_0 | (rxFreelistReg << 16);

	/* add the thread to the receive freelist */
	AddThreadToFreelist();

	/* initialize sinkSigMask */
	sinkSigMask = 1 << __signal_number(&sig_rxEvent);

	SignalNextCtx(ntSigGpr_1);


	if (thread == 7)
	{
		// Setup the MSF control registers
		ConfigMsfRxCtrl();
		SignalNextCtx(ntSigGpr_2);
		SignalNextCtx(ntSigGpr_3);
	}

	/* swap out and wait for the recieve logic to wake the thread */
	wait_for_all(&sig_rxEvent);
	__implicit_read(&sig_rxEvent);
			
} /* CsixRxInit */

#endif // __CSIX_RX_INIT_C__
