  /*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1999-2003 By Intel Corporation.  All rights reserved.
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/*****************************************************************************
 * File:  csix_rx_mop_proc.c
 *
 * Abstract: 
 *        Processes csix frame without SOP or EOP bit set.
 *
 * Contents:
 *        Definitions of following local function:
 *            CsixRxMopProc     
 *
 *****************************************************************************/
#ifndef __CSIX_RX_MOP_PROC_C__
#define __CSIX_RX_MOP_PROC_C__

#include "csix_rx.h"

/*****************************************************************************
 * Abstract:
 *        Processes csix frame without SOP or EOP bit set.
 *
 * Side Effects:
 *    None
 *
 * Assumptions:
 *    None
 *
 *****************************************************************************/

INLINE void 
CsixRxMopProc(
    uint32_t eleAddr,		/* IN: RBUF element address in receive status word */
	uint32_t payloadLen,	/* IN: payload length in receive status word */
	uint32_t tmOffset,		/* IN: offset of TM payload */
	uint32_t extClass,		/* IN: clas ID in CSIX frame extension header */
	uint32_t ingressBlade)		/* IN: ingress blade number */
{
/////////////////////////////////////////////////////////////////////////////
	/* 
	 * local variables and signals declaration 
	 */ 
	cam_lookup_t lkupResult;
	uint32_t camEntry;
	uint32_t lookupKey;
	uint32_t pr_buf_hndl;
	uint32_t dataLen;
	uint32_t pktSize;
 	uint32_t offset = 0;
	uint32_t bufSize;
	uint32_t bufSize_1;
    uint32_t expectSeqNum;						/* expected seq num */
	SIGNAL sig_sramRead;
	SIGNAL sig_sramWrite;
	SIGNAL sig_bufAlloc;
	SIGNAL sig_sramOp;
	__declspec(local_mem shared aligned(8)) rxc_t *prxc;
#ifdef	COUNTERS
	__declspec(sram_write_reg) int byteCount = payloadLen;
	SIGNAL sig_counters;
#endif /* COUNTERS */														

#ifdef	CELL_MODE
	uint32_t CellCount = 0; 
	uint32_t rmdr = 0;
#endif

	lookupKey = extClass | (ingressBlade << 4);	/* get cam lookup key */

#ifdef COUNTERS
	/*
	 * update bytes transmitted 
	 */
	 sram_add((__declspec(sram_write_reg) void *)&byteCount,
				(volatile void *)(CSIX_RX_COUNTER_BASE + (lookupKey << 4) + 
				BYTE_RECEIVED_CNT_IDX),	sig_done, &sig_counters);
#endif	/* COUNTERS */
	
	lkupResult = cam_lookup(lookupKey);
	camEntry = lkupResult.entry_num;
	prxc = (__declspec (local_mem shared aligned(8)) rxc_t *) &rxCtxLm[camEntry];

	if(!lkupResult.hit)	/* mop_cam_miss */
	{
		/*
		 * mop_cam_miss
		 */

		uint32_t tmpTag; 
		__declspec(sram_read_reg) rxc_t inRxCtx;
		__declspec(sram_write_reg) rxc_t outRxCtx;

		sram_read((__declspec(sram_read_reg) void *)&inRxCtx, 
					(int *)(CSIX_RX_CTX_SRAM_BASE + (RXC_SIZE * 4) * lookupKey), 8, sig_done, &sig_sramRead);
		 
		tmpTag = cam_read_tag(camEntry);
		outRxCtx.value[0] = prxc->value[0];
		outRxCtx.value[1] = prxc->value[1];
		outRxCtx.value[2] = prxc->value[2];
		outRxCtx.value[3] = prxc->value[3];
		outRxCtx.value[4] = prxc->value[4];
		outRxCtx.value[5] = prxc->value[5];
		outRxCtx.value[6] = prxc->value[6];
		outRxCtx.value[7] = prxc->value[7];

		sram_write((__declspec(sram_write_reg) void *)&outRxCtx, 
					(int *)(CSIX_RX_CTX_SRAM_BASE + (RXC_SIZE * 4) * tmpTag), 8, sig_done, &sig_sramWrite);
		cam_write(camEntry, lookupKey, NORMAL_STATE);

		wait_for_all(&sig_nextThrd_2, &sig_sramRead, &sig_sramWrite);
		__implicit_read((void *)&outRxCtx);
		__implicit_read(&sig_sramRead);
		__implicit_read(&sig_sramWrite);

		/*
		 * mop_cam_miss_phase_3#
		 */
		prxc->value[0] = inRxCtx.value[0];
		prxc->value[1] = inRxCtx.value[1];
		prxc->value[2] = inRxCtx.value[2];
		prxc->value[3] = inRxCtx.value[3];
		prxc->value[4] = inRxCtx.value[4];
		prxc->value[5] = inRxCtx.value[5];
		prxc->value[6] = inRxCtx.value[6];
		prxc->value[7] = inRxCtx.value[7];
	}
	else /* cam hit */
	{
		wait_for_all(&sig_nextThrd_2);			
	}

	/*
	 * mop_common_phase_3
	 */

	SignalNextCtx(ntSigGpr_2);	/* signal next thread */
	expectSeqNum = prxc->cellSeqNum;	/* save expected sequence number */

	if(prpnd.seqNumb == expectSeqNum)   /* sequence number match */ 
	{

		dataLen = payloadLen - NO_SOP_PRPEND_HDR;	/* get payload length */
		dramHandle = (uint32_t)(prxc->curDataHandle);

		//	The RBUF data start address = element_addr + 
		//  no sop prepend header 
		//  length for a mop Cframe
		eleAddr = eleAddr + NO_SOP_PRPEND_HDR; /* RBUF data start address  */
	
		/*	move received payload to data buffer */
		MoveRbufToPktBuf(dramHandle, eleAddr, dataLen);

#ifdef CELL_MODE
		CellCount = prxc->curBufHandle.seg_count;			/* get cell count */
		rmdr = prxc->remainder;						/* get remainder */
		GetCellCount(&CellCount, &rmdr, dataLen);	/* recalculate */
		prxc->curBufHandle.seg_count = CellCount;			/* update rxc */
		prxc->remainder = rmdr;						/* update rxc */
#endif

		bufSize = prxc->curBufSize + dataLen;	
		pktSize = (int)(prxc->pktSize + dataLen);

		if(prxc->sopBufHandle.lw_offset == prxc->curBufHandle.lw_offset)
		{
			/* 
			 * still sop buffer, offset is not zero
			 */
			offset = prxc->sopBufOffset;	 
		}

		bufSize_1 = bufSize + offset;
		expectSeqNum += 1;  	/* increase by one to get new seq number */

		prxc->pktSize = pktSize; 	/* update packet size */
		prxc->cellSeqNum = expectSeqNum;

	
		if( (bufSize_1 + RBUF_SIZE) <= IX_BUF_MAX) /* mop_buf_not_full */
		{
			prxc->curBufSize = bufSize;
			prxc->curDataHandle = dramHandle + dataLen;
#ifdef COUNTERS
			wait_for_all(&sig_nextThrd_3, &sig_dramWrite, &sig_counters);
			__implicit_read(&byteCount);	/* extends life range of byteCount */
			__implicit_read(&sig_dramWrite);
			__implicit_read(&sig_counters);
#else /* COUNTERS */
			wait_for_all(&sig_nextThrd_3, &sig_dramWrite);
			__implicit_read(&sig_dramWrite);
#endif /* COUNTERS */

			/*
			 * mop_buf_not_full_phase_4
			 */
			SignalNextCtx(ntSigGpr_3); /* signal next thread */

			FreeRbufElement(eleAddr >> PO2_RBUF_SIZE);	/* free current RBUF */
			AddThreadToFreelist();	/* add this thread to freelist */ 
			wait_for_all(&sig_rxEvent);
			__implicit_read(&sig_rxEvent);
		}	/* end of buf_not_full */
		else /* mop_buf_full */
		{
		/* 
		 * mop_buf_full#
		 */

#ifdef CELL_MODE
#ifdef ATM_TX
			rmdr = prxc->remainder;	/* get the rmdr */
			if(rmdr == 0)
			{
				if(rmdr == CELL_SIZE_CNST)
				{
					prxc->remainder = 0;
				}
				else
				{
					prxc->curBufHandle.seg_count -= 1;	/* cell count decrement */
				}			
			}
#endif
#endif

			if(prxc->curBufHandle.lw_offset != prxc->prvBufHandle.lw_offset)
			{
#ifdef TMP_FIX_DL_METAFLUSH
				// Tmp define register to write the result to SRAM.
			__declspec(sram_write_reg) int tmp_sr_wr;
			int sram_addr;
#endif //#ifdef TMP_FIX_DL_METAFLUSH
				/*
				 * clean EOP & SOP bits 
				 */
				dlMeta.bufferNext.value		= 0;	//clean out SOP/EOP bit
				dlMeta.bufferNext.lw_offset = prxc->curBufHandle.lw_offset;
#ifdef CELL_MODE
				/* clean out EOP/SOP bits, reload cell_count */
				dlMeta.bufferNext.value = prxc->curBufHandle.lw_offset |
											prxc->curBufHandle.seg_count;
#endif	/* CELL_MODE */
				/*
				 * only update prvBufHandle's buffer_next with curBufHandle with 
				 * SOP/EOP bits reset
				 * Flush the meta data to SRAM. One LW (0) 
			     * of the meta data are flushed
				 */
#ifndef TMP_FIX_DL_METAFLUSH
				Dl_MetaFlushCache(prxc->prvBufHandle, &sig_sramWrite, SIG_NONE, 0, 1);	
#else // TMP_FIX_DL_METAFLUSH				
				tmp_sr_wr=prxc->curBufHandle.lw_offset;	
				sram_addr= prxc->prvBufHandle.lw_offset << 2;
				sram_write((__declspec(sram_write_reg) void *)&tmp_sr_wr, 
					(int *)sram_addr, 1, sig_done, &sig_sramWrite);
#endif //#ifndef TMP_FIX_DL_METAFLUSH

				prxc->prvBufHandle.lw_offset = prxc->curBufHandle.lw_offset;
				prxc->prvBufHandle.eop = 0;
				prxc->prvBufHandle.sop = 0;

				dlMeta.bufferNext		= prefetchHandle;	
				dlMeta.bufferNext.sop	= 0;		/* clean out SOP */
				dlMeta.bufferSize 		= bufSize;
				dlMeta.offset 			= offset;
				Dl_MetaFlushCache(prxc->curBufHandle, &sig_sramOp, SIG_NONE, 0, 2);
				
				prxc->curBufSize = 0;			/* reset current buffer size*/
				prxc->curBufHandle = prefetchHandle;	/* set new buffer handle */
				prxc->curDataHandle = prefetchDataHndl;	/* set new data buffer handle */

#ifdef CELL_MODE
				prxc->curBufHandle.seg_count = 0;	/* clean out cur cell cnt */
#ifndef ATM_TX
				prxc->remainder = 0;	/* clean out remainder, if no TAM_TX */
#endif

#endif
				/*
				 * get new prefetch buffer
				*/
				Dl_BufAlloc(&prefetchHandle, BUF_FREE_LIST0, BUF_POOL, &sig_bufAlloc, 
						SIG_NONE, ___);
#ifdef COUNTERS
				wait_for_all(&sig_nextThrd_3, &sig_dramWrite, &sig_sramOp, &sig_bufAlloc,
						&sig_sramWrite, &sig_counters);
#ifdef TMP_FIX_DL_METAFLUSH
				__implicit_read(&tmp_sr_wr); /* extends life range of tmp_sr_wr */
#endif /* TMP_FIX_DL_METAFLUSH */										
				__implicit_read(&byteCount);	/* extends life range of byteCount */
				__implicit_read(&sig_dramWrite);
				__implicit_read(&sig_sramOp);
				__implicit_read(&sig_bufAlloc);
				__implicit_read(&sig_sramWrite);
				__implicit_read(&sig_counters);
#else /* COUNTERS */
				wait_for_all(&sig_nextThrd_3, &sig_dramWrite, &sig_sramOp, &sig_bufAlloc,
						&sig_sramWrite);
#ifdef TMP_FIX_DL_METAFLUSH
				__implicit_read(&tmp_sr_wr); /* extends life range of tmp_sr_wr */
#endif /* TMP_FIX_DL_METAFLUSH */										
				__implicit_read(&sig_dramWrite);
				__implicit_read(&sig_sramOp);
				__implicit_read(&sig_bufAlloc);
				__implicit_read(&sig_sramWrite);
#endif /* COUNTERS */
			}
			else /* mop_buf_full_no_eop_reset */
			{	
//mop_buf_full_no_eop_reset#:
				/* 
				 * set previous buf handle without SOP/EOP bits
				 */
				prxc->prvBufHandle.lw_offset = prxc->curBufHandle.lw_offset;
				prxc->prvBufHandle.eop = 0;
				prxc->prvBufHandle.sop = 0;

				dlMeta.bufferNext		= prefetchHandle;	
				dlMeta.bufferNext.sop	= 0;		/* clean out SOP bit */
				dlMeta.bufferSize 		= bufSize;
				dlMeta.offset 			= offset;
				/*
				 * flush the meta data to SRAM. two LW (0 - 1) 
				 * of the meta data are flushed
				 */
				Dl_MetaFlushCache(prxc->curBufHandle, &sig_sramOp, SIG_NONE, 0, 2);

#ifdef CELL_MODE
				/* set cell_count in sop buffer */ 
				prxc->sopBufHandle.seg_count = prxc->curBufHandle.seg_count; 
#ifndef ATM_TX				
				prxc->remainder =0;		/* clean out remainder */
#endif

				/* clean out cur cell cnt */
				prxc->curBufHandle.seg_count =0;
#endif

				prxc->curBufSize = 0;			//reset
				prxc->curBufHandle = prefetchHandle;
				prxc->curDataHandle = prefetchDataHndl;

				/* get new prefetch buffer */
				Dl_BufAlloc(&prefetchHandle, BUF_FREE_LIST0, BUF_POOL, &sig_bufAlloc, SIG_NONE, ___);


#ifdef COUNTERS
				wait_for_all(&sig_nextThrd_3, &sig_sramOp, &sig_dramWrite, &sig_bufAlloc,
					&sig_counters);
				__implicit_read(&byteCount);	/* extends life range of byteCount */
				__implicit_read(&sig_sramOp);
				__implicit_read(&sig_dramWrite);
				__implicit_read(&sig_bufAlloc);
				__implicit_read(&sig_counters);
#else /* COUNTERS */
				wait_for_all(&sig_nextThrd_3, &sig_sramOp, &sig_dramWrite, &sig_bufAlloc);
				__implicit_read(&sig_sramOp);
				__implicit_read(&sig_dramWrite);
				__implicit_read(&sig_bufAlloc);
#endif /* COUNTERS */
			}

			/*
			 * mop_buf_full_phase_4
			 */
			SignalNextCtx(ntSigGpr_3); /* signal next thread */

			/*
			 * get prefetch data handle from prefetch  buffer handle
			 */
			prefetchDataHndl = Dl_BufGetData(prefetchHandle);

			FreeRbufElement(eleAddr >> PO2_RBUF_SIZE); /* free current RBUF */
			AddThreadToFreelist();	/* add this thread to freelist */ 
			wait_for_all(&sig_rxEvent);
			__implicit_read(&sig_rxEvent);
		}
	}
	else /* mop_seq_mismatch */
	{
		/*
		 * mop_seq_mismatch
		 */

		dl_buf_handle_t drop_sop_handle;
		dl_buf_handle_t drop_cur_handle;

		prxc->cellSeqNum=0;		// reset sequence number

		/* 
		 * Phase 3 is the only phase in which you can/should access rxc in local memory
		 */

		drop_sop_handle = prxc->sopBufHandle;	/* used in phase 4 */
		drop_cur_handle = prxc->curBufHandle;

#ifdef COUNTERS
		wait_for_all(&sig_nextThrd_3, &sig_counters);
		__implicit_read(&byteCount);	/* extends life range of byteCount */
		__implicit_read(&sig_counters);
#else /* COUNTERS */
		wait_for_all(&sig_nextThrd_3);
#endif /* COUNTERS */

		/*
		 * mop_seq_mismatch_phase_4
		 */

		SignalNextCtx(ntSigGpr_3);	// signal next thread

		if(expectSeqNum)	/* incomplete packet exist */
		{
			/* drop imcomplete packet */

			DropCurrentPkt(drop_sop_handle, drop_cur_handle, lookupKey);
		}

		FreeRbufElement(eleAddr >> PO2_RBUF_SIZE); /* free current RBUF */
		AddThreadToFreelist();	/* add this thread to freelist */ 
		wait_for_all(&sig_rxEvent);
		__implicit_read(&sig_rxEvent);
	}
} /* CsixRxMopProc */

#endif // __CSIX_RX_MOP_PROC_C__


