  /*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1999-2003 By Intel Corporation.  All rights reserved.
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/*****************************************************************************
 * File:  csix_rx_sep_proc.c
 *
 * Abstract: 
 *        Processes csix frame with both SOP and EOP bit set.
 *
 * Contents:
 *        Definitions of following local function:
 *            CsixRxSepProc     
 *
 *****************************************************************************/
#ifndef __CSIX_RX_SEP_PROC_C__
#define __CSIX_RX_SEP_PROC_C__

#include "csix_rx.h"

/*****************************************************************************
 * Abstract:
 *        Processes csix frame with both SOP and EOP bit set.
 *
 * Side Effects:
 *    None
 *
 * Assumptions:
 *    None
 *
 *****************************************************************************/

INLINE void 
CsixRxSepProc(
    uint32_t eleAddr,		/* IN: RBUF element address in receive status word */
	uint32_t payloadLen,	/* IN: payload length in receive status word */
	uint32_t tmOffset,		/* IN: offset of TM payload */
	uint32_t extClass,		/* IN: clas ID in CSIX frame extension header */
	uint32_t ingressBlade)		/* IN: ingress blade number */
{
	uint32_t bufOffset = GetRandBufOffset();
	cam_lookup_t lkupResult;
	uint32_t bufSize;
	uint32_t lookupKey;
	uint32_t camEntry;
	__declspec(sram_write_reg) dl_meta_t dlMetaOut;
	__declspec(gp_reg) uint32_t tmp=0;
	SIGNAL sig_bufAlloc;
	SIGNAL sig_sramOp;
	__declspec(local_mem shared aligned(8)) rxc_t *prxc;

#ifdef USE_PMETA
	pmeta = (dl_meta_t *) &dlMeta;
#endif /* USE_PMETA */

#ifdef	COUNTERS
	__declspec(sram_write_reg) int byteCount = payloadLen;
	SIGNAL sig_counters;
#endif	/* COUNTERS */
														

#ifdef CELL_MODE
	uint32_t CellCount = 0; 
	uint32_t rmdr = 0;
#endif

	dramHandle = (uint32_t)(prefetchDataHndl + bufOffset);

	//	The RBUF data start address = eleAddr + 
	//  prepend header length for SOP/EOP Cframe
	eleAddr = eleAddr + SOP_PRPEND_HDR;
	
	// The RBUF data length = payload length - 
	// prepend header length
	bufSize= payloadLen - SOP_PRPEND_HDR;

	//	move received cframe to data buffer

	//_rbuf_to_dram[dramHandle, eleAddr, bufSize, \
	//              rbuf_base, dram_sig]
	MoveRbufToPktBuf(dramHandle, eleAddr, bufSize);

	//alu[bufSize, bufSize,-,tmOffset]					; deduct offset
	// deduct offset
	bufSize = bufSize - tmOffset; 					

	// store buf handle
	dlBufHandle = prefetchHandle;

	//	Prefetch a new buffer so that it can be used next time around.
	Dl_BufAlloc(&prefetchHandle, BUF_FREE_LIST0, BUF_POOL, &sig_bufAlloc, SIG_NONE, ___);

	//	Compose CAM lookup key
	lookupKey = extClass | (ingressBlade << 4);	
#ifdef COUNTERS
	sram_incr((volatile void *)(CSIX_RX_COUNTER_BASE + (lookupKey << 4) + PKT_RECEIVED_CNT_IDX));
	sram_add((__declspec(sram_write_reg) void *)&byteCount,
				(volatile void *)(CSIX_RX_COUNTER_BASE + (lookupKey << 4) + BYTE_RECEIVED_CNT_IDX),
				sig_done, &sig_counters);
#endif	/* COUNTERS */

#ifdef CELL_MODE
	/* claculate CellCount from bufSize */
	GetCellCount(&CellCount, &rmdr, bufSize);
#endif

	//	Do CAM lookup & set up local memory address

	lkupResult = cam_lookup(lookupKey);
	camEntry = lkupResult.entry_num;
	prxc = (__declspec (local_mem shared aligned(8)) rxc_t *) &rxCtxLm[camEntry];
	//	Check lookup result

	if(!lkupResult.hit)	// sep_cam_miss
	{
		uint32_t tmpTag; 
		SIGNAL sig_sramRead;
		SIGNAL sig_sramWrite;
		__declspec(sram_read_reg) rxc_t inRxCtx;
		__declspec(sram_write_reg) rxc_t outRxCtx;

		sram_read((__declspec(sram_read_reg) void *)&inRxCtx, 
					(int *)(CSIX_RX_CTX_SRAM_BASE + (RXC_SIZE * 4) * lookupKey), 3, sig_done, &sig_sramRead);
		 
	 
		tmpTag = cam_read_tag(camEntry);
		outRxCtx.value[0] = prxc->value[0];
		outRxCtx.value[1] = prxc->value[1];
		outRxCtx.value[2] = prxc->value[2];
		outRxCtx.value[3] = prxc->value[3];
		outRxCtx.value[4] = prxc->value[4];
		outRxCtx.value[5] = prxc->value[5];
		outRxCtx.value[6] = prxc->value[6];
		outRxCtx.value[7] = prxc->value[7];

		sram_write((__declspec(sram_write_reg) void *)&outRxCtx, 
					(int *)(CSIX_RX_CTX_SRAM_BASE + (RXC_SIZE * 4) * tmpTag), 8, sig_done, &sig_sramWrite);
		tmOffset=tmOffset + bufOffset;
		cam_write(camEntry, lookupKey, NORMAL_STATE);

		wait_for_all(&sig_nextThrd_2, &sig_sramRead, &sig_sramWrite);
		__implicit_read((void *)&outRxCtx);
		__implicit_read(&sig_sramRead);
		__implicit_read(&sig_sramWrite);

//sep_cam_miss_phase_3#:

		//_send_next_thrd_sig[ntSigGpr_2]						; send next thread sig 2
		SignalNextCtx(ntSigGpr_2);

		//	Check if expected cell sequence number is 0
		if(!(inRxCtx.cellSeqNum))	// expected sequence nunber is 0
		{
			//	Update the meta data.

#ifndef USE_SXFER_FOR_DLMETA

#ifndef USE_PMETA
			dlMeta.bufferNext.value	= IX_DL_BUF_NULL;
			dlMeta.bufferSize 		= bufSize;
			dlMeta.offset 			= tmOffset;
			dlMeta.freeListId 		= 0;
			dlMeta.rxStat 			= 0;
			dlMeta.packetSize 		= bufSize;
			dlMeta.headerType		= 0;
			dlMeta.inputPort 		= prpnd.inputPort;
			dlMeta.outputPort 		= prpnd.outputPort;
			dlMeta.nhidType			= prpnd.nextHopId;
			dlMeta.fabricPort		= 0;
			dlMeta.flowId			= prpnd.flowId	;
			dlMeta.classId 			= prpnd.classId;
#else /* USE_PMETA	*/
			pmeta->bufferNext.value	= IX_DL_BUF_NULL;
			pmeta->bufferSize 		= bufSize;
			pmeta->offset 			= tmOffset;
			pmeta->freeListId 		= 0;
			pmeta->rxStat 			= 0;
			pmeta->packetSize 		= bufSize;
			pmeta->headerType		= 0;
			pmeta->inputPort 		= prpnd.inputPort;
			pmeta->outputPort 		= prpnd.outputPort;
			pmeta->nhidType			= prpnd.nextHopId;
			pmeta->fabricPort		= 0;
			pmeta->flowId			= prpnd.flowId	;
			pmeta->classId 			= prpnd.classId;
#endif /* USE_PMETA */

			// Flush the meta data to SRAM seven LW's (0 to 6) 
			// of the meta data are flushed. 
			Dl_MetaFlushCache(dlBufHandle, &sig_sramOp, SIG_NONE, 0, 7);

#else /* USE_SXFER_FOR_DLMETA */

			Dl_MetaWrite(dlBufHandle, &dlMetaOut, IX_DL_BUF_NULL, bufSize,
							tmOffset, bufSize, 0, 0, 0, prpnd.inputPort, 
							prpnd.outputPort, prpnd.nextHopId, 0, 
							prpnd.flowId, prpnd.classId, IX_DL_BUF_NULL,
							&sig_sramOp, SIG_NONE, 0, 7);

#endif /* USE_SXFER_FOR_DLMETA */
	
			//reset cell sequnce number of that entry to 0
			prxc->cellSeqNum = 0;	

#ifdef COUNTERS
			wait_for_all(&sig_nextThrd_3, &sig_sramOp, &sig_dramWrite, &sig_bufAlloc, &sig_counters);
			__implicit_read(&byteCount);	// extends life range of byteCount
			__implicit_read(&sig_sramOp);
			__implicit_read(&sig_dramWrite);
			__implicit_read(&sig_bufAlloc);
			__implicit_read(&sig_counters);
#else /* COUNTERS */
			wait_for_all(&sig_nextThrd_3, &sig_sramOp, &sig_dramWrite, &sig_bufAlloc);
			__implicit_read(&sig_sramOp);
			__implicit_read(&sig_dramWrite);
			__implicit_read(&sig_bufAlloc);
#endif /* COUNTERS */
//sep_common_phase_4#:
			// send next thread signal 3 to the next thread

			SignalNextCtx(ntSigGpr_3);

			// Compute the buffer data address in DRAM of the prefetched buffer handle.
			prefetchDataHndl = Dl_BufGetData(prefetchHandle);
			FreeRbufElement(eleAddr >> PO2_RBUF_SIZE);
			AddThreadToFreelist();
			
			// Send current packet to the next micro block
#ifdef CELL_MODE
#ifdef QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER
			CsixRxSink(dlBufHandle, dlEopBufHandle, bufSize, CellCount,
				extClass, prpnd.outputPort, dlRing, lookupKey, sinkSigMask);
#else /* QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER */
			CsixRxSink(dlBufHandle, dlEopBufHandle, bufSize, CellCount,
				prpnd.classId, prpnd.outputPort, dlRing, lookupKey, sinkSigMask);
#endif /* QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER */
#else /* CELL_MODE */
#ifdef QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER
			CsixRxSink(dlBufHandle, dlEopBufHandle, bufSize, 0,
				extClass, prpnd.outputPort, dlRing, lookupKey, sinkSigMask);
#else /* QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER */
			CsixRxSink(dlBufHandle, dlEopBufHandle, bufSize, 0,
				prpnd.classId, prpnd.outputPort, dlRing, lookupKey, sinkSigMask);
#endif /* QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER */
#endif /* CELL_MODE */
		}
		else // expected cell sequence number is not zero
		{
//sep_cam_miss_seq_not_zero#:

			prxc->cellSeqNum = 0;	

#ifndef USE_SXFER_FOR_DLMETA

#ifndef USE_PMETA
			dlMeta.bufferNext.value	= IX_DL_BUF_NULL;
			dlMeta.bufferSize 		= bufSize;
			dlMeta.offset 			= tmOffset;
			dlMeta.freeListId 		= 0;
			dlMeta.rxStat 			= 0;
			dlMeta.packetSize 		= bufSize;
			dlMeta.headerType		= 0;
			dlMeta.inputPort 		= prpnd.inputPort;
			dlMeta.outputPort 		= prpnd.outputPort;
			dlMeta.nhidType			= prpnd.nextHopId;
			dlMeta.fabricPort		= 0;
			dlMeta.flowId			= prpnd.flowId	;
			dlMeta.classId 			= prpnd.classId;
#else /* USE_PMETA	*/
			pmeta->bufferNext.value	= IX_DL_BUF_NULL;
			pmeta->bufferSize 		= bufSize;
			pmeta->offset 			= tmOffset;
			pmeta->freeListId 		= 0;
			pmeta->rxStat 			= 0;
			pmeta->packetSize 		= bufSize;
			pmeta->headerType		= 0;
			pmeta->inputPort 		= prpnd.inputPort;
			pmeta->outputPort 		= prpnd.outputPort;
			pmeta->nhidType			= prpnd.nextHopId;
			pmeta->fabricPort		= 0;
			pmeta->flowId			= prpnd.flowId	;
			pmeta->classId 			= prpnd.classId;
#endif /* USE_PMETA */

			// Flush the meta data to SRAM seven LW's (0 to 6) 
			// of the meta data are flushed. 
			Dl_MetaFlushCache(dlBufHandle, &sig_sramOp, SIG_NONE, 0, 7);	

#else /* USE_SXFER_FOR_DLMETA */

			Dl_MetaWrite(dlBufHandle, &dlMetaOut, IX_DL_BUF_NULL, bufSize,
							tmOffset, bufSize, 0, 0, 0, prpnd.inputPort, 
							prpnd.outputPort, prpnd.nextHopId, 0, 
							prpnd.flowId, prpnd.classId, IX_DL_BUF_NULL,
							&sig_sramOp, SIG_NONE, 0, 7);
#endif /* USE_SXFER_FOR_DLMETA */


#ifdef COUNTERS
			wait_for_all(&sig_nextThrd_3, &sig_sramOp, &sig_dramWrite, &sig_bufAlloc, &sig_counters);
			__implicit_read(&byteCount);	// extends life range of byteCount
			__implicit_read(&sig_sramOp);
			__implicit_read(&sig_dramWrite);
			__implicit_read(&sig_bufAlloc);
			__implicit_read(&sig_counters);
#else /* COUNTERS */
			wait_for_all(&sig_nextThrd_3, &sig_sramOp, &sig_dramWrite, &sig_bufAlloc);
			__implicit_read(&sig_sramOp);
			__implicit_read(&sig_dramWrite);
			__implicit_read(&sig_bufAlloc);
#endif /* COUNTERS */

//sep_cam_miss_seq_not_zero_phase_4#:

			// send next thread signal 3 to the next thread
			SignalNextCtx(ntSigGpr_3);
			
			DropCurrentPkt((dl_buf_handle_t)inRxCtx.sopBufHandle , 
				(dl_buf_handle_t)inRxCtx.curBufHandle, lookupKey);

			//	get prefetch data handle from 
			//  prefetch buffer handle
			// Compute the buffer data address in DRAM of the prefetched buffer handle.
			prefetchDataHndl = Dl_BufGetData(prefetchHandle);

			FreeRbufElement(eleAddr >> PO2_RBUF_SIZE);
			AddThreadToFreelist();
			
			// Send current packet to the next micro block
#ifdef CELL_MODE
#ifdef QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER
			CsixRxSink(dlBufHandle, dlEopBufHandle, bufSize, CellCount,
				extClass, prpnd.outputPort, dlRing, lookupKey, sinkSigMask);
#else /* QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER */
			CsixRxSink(dlBufHandle, dlEopBufHandle, bufSize, CellCount,
				prpnd.classId, prpnd.outputPort, dlRing, lookupKey, sinkSigMask);
#endif /* QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER */
#else /* CELL_MODE */
#ifdef QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER
			CsixRxSink(dlBufHandle, dlEopBufHandle, bufSize, 0,
				extClass, prpnd.outputPort, dlRing, lookupKey, sinkSigMask);
#else /* QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER */
			CsixRxSink(dlBufHandle, dlEopBufHandle, bufSize, 0,
				prpnd.classId, prpnd.outputPort, dlRing, lookupKey, sinkSigMask);
#endif /* QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER */
#endif /* CELL_MODE */
		}
	}
	else
		// sep_cam_hit
		{
			tmOffset = tmOffset + bufOffset;


#ifndef USE_SXFER_FOR_DLMETA

#ifndef USE_PMETA
			dlMeta.bufferNext.value	= IX_DL_BUF_NULL;
			dlMeta.bufferSize 		= bufSize;
			dlMeta.offset 			= tmOffset;
			dlMeta.freeListId 		= 0;
			dlMeta.rxStat 			= 0;
			dlMeta.packetSize 		= bufSize;
			dlMeta.headerType		= 0;
			dlMeta.inputPort 		= prpnd.inputPort;
			dlMeta.outputPort 		= prpnd.outputPort;
			dlMeta.nhidType			= prpnd.nextHopId;
			dlMeta.fabricPort		= 0;
			dlMeta.flowId			= prpnd.flowId	;
			dlMeta.classId 			= prpnd.classId;
#else /* USE_PMETA	*/
			pmeta->bufferNext.value	= IX_DL_BUF_NULL;
			pmeta->bufferSize 		= bufSize;
			pmeta->offset 			= tmOffset;
			pmeta->freeListId 		= 0;
			pmeta->rxStat 			= 0;
			pmeta->packetSize 		= bufSize;
			pmeta->headerType		= 0;
			pmeta->inputPort 		= prpnd.inputPort;
			pmeta->outputPort 		= prpnd.outputPort;
			pmeta->nhidType			= prpnd.nextHopId;
			pmeta->fabricPort		= 0;
			pmeta->flowId			= prpnd.flowId	;
			pmeta->classId 			= prpnd.classId;
#endif /* USE_PMETA */

			// Flush the meta data to SRAM seven LW's (0 to 6) 
			// of the meta data are flushed. 
			Dl_MetaFlushCache(dlBufHandle, &sig_sramOp, SIG_NONE, 0, 7);

#else /* USE_SXFER_FOR_DLMETA */

			Dl_MetaWrite(dlBufHandle, &dlMetaOut, IX_DL_BUF_NULL, bufSize,
							tmOffset, bufSize, 0, 0, 0, prpnd.inputPort, 
							prpnd.outputPort, prpnd.nextHopId, 0, 
							prpnd.flowId, prpnd.classId, IX_DL_BUF_NULL,
							&sig_sramOp, SIG_NONE, 0, 7);
#endif /* USE_SXFER_FOR_DLMETA */
	
// end of phase 2
			wait_for_all(&sig_nextThrd_2);

//sep_cam_hit_phase_3#:

			SignalNextCtx(ntSigGpr_2);


			//	goto sequence number error proc
			if(!(prxc->cellSeqNum))
			{
#ifdef COUNTERS
				wait_for_all(&sig_nextThrd_3, &sig_sramOp, &sig_dramWrite, &sig_bufAlloc, &sig_counters);
				__implicit_read(&byteCount);	// extends life range of byteCount
				__implicit_read(&sig_sramOp);
				__implicit_read(&sig_dramWrite);
				__implicit_read(&sig_bufAlloc);
				__implicit_read(&sig_counters);
#else /* COUNTERS */
				wait_for_all(&sig_nextThrd_3, &sig_sramOp, &sig_dramWrite, &sig_bufAlloc);
				__implicit_read(&sig_sramOp);
				__implicit_read(&sig_dramWrite);
				__implicit_read(&sig_bufAlloc);
#endif /* COUNTERS */

//sep_cam_hit_phase_3_end#:
			}
			else // sep_cam_hit_seq_not_zero#
			{
//sep_cam_hit_seq_not_zero#:

				dl_buf_handle_t	drop_sop_handle;
				dl_buf_handle_t	drop_cur_handle;

				drop_sop_handle = prxc->sopBufHandle;
				drop_cur_handle = prxc->curBufHandle;

				prxc->cellSeqNum = 0;	

#ifdef COUNTERS
				wait_for_all(&sig_nextThrd_3, &sig_sramOp, &sig_dramWrite, &sig_bufAlloc, &sig_counters);
				__implicit_read(&byteCount);	// extends life range of byteCount
				__implicit_read(&sig_sramOp);
				__implicit_read(&sig_dramWrite);
				__implicit_read(&sig_bufAlloc);
				__implicit_read(&sig_counters);
#else /* COUNTERS */
				wait_for_all(&sig_nextThrd_3, &sig_sramOp, &sig_dramWrite, &sig_bufAlloc);
				__implicit_read(&sig_sramOp);
				__implicit_read(&sig_dramWrite);
				__implicit_read(&sig_bufAlloc);
#endif /* COUNTERS */

				DropCurrentPkt(drop_sop_handle, drop_cur_handle, lookupKey);
			}

//sep_common_phase_4#:
			// send next thread signal 3 to the next thread
			SignalNextCtx(ntSigGpr_3);

			// Compute the buffer data address in DRAM of the prefetched buffer handle.
			prefetchDataHndl = Dl_BufGetData(prefetchHandle);

			FreeRbufElement(eleAddr >> PO2_RBUF_SIZE);
			AddThreadToFreelist();
			
			// Send current packet to the next micro block
#ifdef CELL_MODE
#ifdef QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER
			CsixRxSink(dlBufHandle, dlEopBufHandle, bufSize, CellCount,
				extClass, prpnd.outputPort, dlRing, lookupKey, sinkSigMask);
#else /* QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER */
			CsixRxSink(dlBufHandle, dlEopBufHandle, bufSize, CellCount,
				prpnd.classId, prpnd.outputPort, dlRing, lookupKey, sinkSigMask);
#endif /* QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER */
#else /* CELL_MODE */
#ifdef QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER
			CsixRxSink(dlBufHandle, dlEopBufHandle, bufSize, 0,
				extClass, prpnd.outputPort, dlRing, lookupKey, sinkSigMask);
#else /* QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER */
			CsixRxSink(dlBufHandle, dlEopBufHandle, bufSize, 0,
				prpnd.classId, prpnd.outputPort, dlRing, lookupKey, sinkSigMask);
#endif /* QUEUE_NUMBER_USES_CLASS_ID_IN_EXTENSION_HEADER */
#endif /* CELL_MODE */
		}
} /* CsixRxSepProc */

#endif // __CSIX_RX_SEP_PROC_C__


