 /*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1999-2003 By Intel Corporation.  All rights reserved.
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/*****************************************************************************
 * File:  csix_rx_sop_proc.c
 *
 * Abstract: 
 *        Processes csix frame with SOP bit set.
 *
 * Contents:
 *        Definitions of following local function:
 *            CsixRxSopProc     
 *
 *****************************************************************************/
#ifndef __CSIX_RX_SOP_PROC_C__
#define __CSIX_RX_SOP_PROC_C__

#include "csix_rx.h"

/*****************************************************************************
 * Abstract:
 *    Processes csix frame with SOP bit set.
 *
 * Side Effects:
 *    None
 *
 * Assumptions:
 *    None
 *
 *****************************************************************************/

INLINE void
CsixRxSopProc(
    uint32_t eleAddr,		/* IN: RBUF element address in receive status word */
	uint32_t payloadLen,	/* IN: payload length in receive status word */
	uint32_t tmOffset,		/* IN: offset of TM payload */
	uint32_t extClass,		/* IN: clas ID in CSIX frame extension header */
	uint32_t ingressBlade)		/* IN: ingress blade number */
{
	uint32_t bufOffset = GetRandBufOffset();
	cam_lookup_t lkupResult;
	uint32_t camEntry;
    uint32_t expectSeqNum;						/* expected seq num */
	dl_buf_handle_t drop_sop_hndl;	
	dl_buf_handle_t drop_eop_hndl;		
	/* save prefetchHandle for later use */
	dl_buf_handle_t savePrfHndl = prefetchHandle;

	uint32_t dataLen;
	uint32_t lookupKey;
	SIGNAL sig_sramRead;
	SIGNAL sig_sramWrite;
	SIGNAL sig_bufAlloc;
	SIGNAL sig_sramOp;
	__declspec(local_mem shared aligned(8)) rxc_t *prxc;


#ifdef	COUNTERS							
	__declspec(sram_write_reg) int byteCount = payloadLen; /* bytes transmitted */
	SIGNAL sig_counters;
#endif /* COUNTERS */														

#ifdef	CELL_MODE
	uint32_t CellCount = 0; 		/* cell count */
	uint32_t rmdr = 0;			/* remainder */
#endif


	/*
	 * using prefetch buffer to store the incoming SOP Cframe
	 */

	dramHandle = (uint32_t)(prefetchDataHndl + bufOffset); /* get data handle */
	eleAddr = eleAddr + SOP_PRPEND_HDR; /* RBUF start address for payload */
	dataLen = payloadLen - SOP_PRPEND_HDR;	

	/*
	 *	move received cframe to dram data buffer
	 */
	MoveRbufToPktBuf(dramHandle, eleAddr, dataLen);

	dramHandle = dramHandle + dataLen; /* next payload dram start address */
	dataLen = dataLen - tmOffset;		/* actual data length */
	tmOffset = tmOffset + bufOffset;

	/* 
	 * prefetch a new buffer so that it can be used next time 
	 */
	Dl_BufAlloc(&prefetchHandle, BUF_FREE_LIST0, BUF_POOL, &sig_bufAlloc, SIG_NONE, ___);

	lookupKey = extClass | (ingressBlade << 4);	/* cam lookup key */

#ifdef COUNTERS
	sram_incr((volatile void *)(CSIX_RX_COUNTER_BASE + (lookupKey << 4) + 
				PKT_RECEIVED_CNT_IDX));
	sram_add((__declspec(sram_write_reg) void *)&byteCount,
				(volatile void *)(CSIX_RX_COUNTER_BASE + (lookupKey << 4) + 
				BYTE_RECEIVED_CNT_IDX), sig_done, &sig_counters);
#endif	/* COUNTERS */

#ifdef CELL_MODE
	/* claculate CellCount from dataLen */
	GetCellCount(&CellCount, &rmdr, dataLen);

#endif

	/* cam lookup */     
	lkupResult = cam_lookup(lookupKey);
	camEntry = lkupResult.entry_num;
	prxc = (__declspec (local_mem shared aligned(8)) rxc_t *) &rxCtxLm[camEntry];

	if(!lkupResult.hit)	// sop_cam_miss
	{
		/*
		 * SOP only Cframe CAM miss processing start
		 */
		uint32_t tmpTag; 
		__declspec(sram_read_reg) rxc_t inRxCtx;
		__declspec(sram_write_reg) rxc_t outRxCtx;

		sram_read((__declspec(sram_read_reg) void *)&inRxCtx, 
					(int *)(CSIX_RX_CTX_SRAM_BASE + (RXC_SIZE * 4) * lookupKey), 
					3, sig_done, &sig_sramRead);		 
	 
		tmpTag = cam_read_tag(camEntry);
		outRxCtx.value[0] = prxc->value[0];
		outRxCtx.value[1] = prxc->value[1];
		outRxCtx.value[2] = prxc->value[2];
		outRxCtx.value[3] = prxc->value[3];
		outRxCtx.value[4] = prxc->value[4];
		outRxCtx.value[5] = prxc->value[5];
		outRxCtx.value[6] = prxc->value[6];
		outRxCtx.value[7] = prxc->value[7];

		sram_write((__declspec(sram_write_reg) void *)&outRxCtx, 
				(int *)(CSIX_RX_CTX_SRAM_BASE + (RXC_SIZE * 4) * tmpTag),
				 8, sig_done, &sig_sramWrite);
		cam_write(camEntry, lookupKey, NORMAL_STATE);
		wait_for_all(&sig_nextThrd_2, &sig_sramRead, &sig_sramWrite);
		__implicit_read((void *)&outRxCtx);
		__implicit_read(&sig_sramRead);
		__implicit_read(&sig_sramWrite);

		/*
		 * sop_cam_miss_phase_3
		 */
		/* move following inRxCtx to rxCtxLm in the case of miss */
		prxc->value[0] = inRxCtx.value[0];
		prxc->value[1] = inRxCtx.value[1];
		prxc->value[2] = inRxCtx.value[2];
	}
	else /* cam hit */
	{
		wait_for_all(&sig_nextThrd_2);
	}
	
	/*
	 * sop_common_phase_3
	 */
	SignalNextCtx(ntSigGpr_2);	 /* signal next thread */
	expectSeqNum=prxc->cellSeqNum;
	drop_sop_hndl=prxc->sopBufHandle;
	drop_eop_hndl=prxc->curBufHandle;

	/* update rxCtxLm[camEntry] */
	prxc->sopBufHandle = savePrfHndl;
	prxc->curBufHandle = savePrfHndl;
	prxc->curDataHandle = dramHandle;
	prxc->curBufSize = dataLen;
	prxc->sopBufOffset = tmOffset;
	prxc->prvBufHandle = savePrfHndl;
	prxc->flowId = prpnd.flowId;
	prxc->classId = prpnd.classId;
	prxc->outputPort = prpnd.outputPort;
	prxc->pktSize = dataLen;
	prxc->cellSeqNum = 1;


#ifdef CELL_MODE
	prxc->curBufHandle.seg_count = CellCount;	/* set curCellCount */
	prxc->remainder = rmdr;				/* set remainder */
#endif
	dlMeta.bufferNext.value	= IX_DL_BUF_NULL;
	dlMeta.bufferSize 			= dataLen;
	dlMeta.offset 				= tmOffset;
	dlMeta.freeListId 			= 0;
	dlMeta.rxStat	 			= 0;
	dlMeta.packetSize	 		= dataLen;
	dlMeta.headerType			= 0;
	dlMeta.inputPort 			= prpnd.inputPort;
	dlMeta.outputPort 			= prpnd.outputPort;
	dlMeta.nhidType				= prpnd.nextHopId;
	dlMeta.fabricPort			= 0;
	dlMeta.flowId				= prpnd.flowId	;
	dlMeta.classId 				= prpnd.classId;
	dlMeta.packetNext			= 0;

	// Flush the meta data to SRAM. All eight LW's (0 to 7) 
	// of the meta data are flushed. 
	Dl_MetaFlushCache(savePrfHndl, &sig_sramOp, SIG_NONE, 0, 8);	

#ifdef COUNTERS
	wait_for_all(&sig_nextThrd_3, &sig_sramOp, &sig_dramWrite, &sig_bufAlloc, &sig_counters);
	__implicit_read(&byteCount);	/* extends life range of byteCount */
	__implicit_read(&sig_sramOp);
	__implicit_read(&sig_dramWrite);
	__implicit_read(&sig_bufAlloc);
	__implicit_read(&sig_counters);
#else /* COUNTERS */
	wait_for_all(&sig_nextThrd_3, &sig_sramOp, &sig_dramWrite, &sig_bufAlloc);
	__implicit_read(&sig_sramOp);
	__implicit_read(&sig_dramWrite);
	__implicit_read(&sig_bufAlloc);
#endif /* COUNTERS */

	/*
	 * sop_common_phase_4
	 */
	SignalNextCtx(ntSigGpr_3);	/* signal next thread */

	/*
	 *	check if expected sequence number is 0
	 */
	if(expectSeqNum)
	{
		/* 
		 * drop the incomplete packet
		 */
		DropCurrentPkt(drop_sop_hndl, drop_eop_hndl, lookupKey);
	}

	/* get prefetch data handle from prefetch buffer handle	*/	
	prefetchDataHndl = Dl_BufGetData(prefetchHandle);

	FreeRbufElement(eleAddr >> PO2_RBUF_SIZE);	/* free RBUF element */
	AddThreadToFreelist();			/* add this thread to freelist */

	wait_for_all(&sig_rxEvent);
	__implicit_read(&sig_rxEvent);
}  /* CsixRxSopProc */


#endif // __CSIX_RX_SOP_PROC_C__ 


