///////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
///////////////////////////////////////////////////////////////////////
//
//      File Name: csix_rx.uc
//
//      Purpose: This is the main file for the CSIX RX microblock.
//
//      Date            Comment                         By
//      ---------------------------------------------------------------
//
//      02/26/2002      Created                         David Meng
//		12/03/2002		Modified to add 2 ME solution	asv
//		11/03/2003		Moved 2 ME solution to 			asv
//						a separate file.
//
///////////////////////////////////////////////////////////////////////
#ifndef	__CSIX_RX_UC__
#define	__CSIX_RX_UC__

// Define meta-data size as 7 LW's.
#define META_CACHE_SIZE		7

// Including needed files.
#include <dl_system.h>
#include <stdmac.uc>
#include <dispatch_loop.uc>
#include <hardware.h>

#include <csix_rx.h>				
#include <csix_rx_context.h>		
#include <csix_rx_init.uc>			
#include <csix_rx_common_util.uc>
#include <csix_rx_one_me_util.uc>

//---------------------------------------------------------------------
//	Register definitions.
//---------------------------------------------------------------------

.reg flst_reg						; Free list register
.reg dram_rbuf_ov					; DRAM to RBUF override flag
.reg msf_rx_element_addr			; MSF RX element address
.reg rbuf_base						; RBUF base address
.reg msf_freelist0_addr				; MSF freelist 0 address 
.reg crx_sram_gpr					; CRX SRAM base
.reg nxt_thrd_1_gpr					; next thread 1 adddres
.reg nxt_thrd_2_gpr					; next thread 2 adddres
.reg nxt_thrd_3_gpr					; next thread 3 adddres
.reg prf_data_hndl					; prefetch buf data hndl

.reg ext_class						; extension class
.reg ele_addr						; RBUF element address 
.reg to_qm_ring						; Outgoing CSIX ring number
.reg flags							; SOP/EOP flags
.reg c_offset						; c-frame offset
.reg seq_num						; cell sequence number
.reg in_bld_num						; Input blade number 
.reg class_id						; Class ID in TM header.
.reg ele_num						; RBUF element number
.reg payload_len					; Payload length
.reg err_code						; Error code
.reg next_hop_id					; Next hop id 16 bits.

#if (!IS_IXPTYPE(__IXP28XX))
.reg @rand_buf_offset										; random buffer offset
.reg rand_offset_mask										; mask for wrapping around
#endif

#ifdef CELL_MODE
.reg cell_size						; cell_size											
#endif
	
#ifdef	COUNTERS						
.reg cnt_addr						; counter address
.reg cnt_base						; counter base gpr 
#endif

#ifdef	_DEBUG_COUNTERS_

//---------------------------------------------------------------------
// Declare some counters to ease debugging.
//---------------------------------------------------------------------
.reg @csix_rx_num_cells_rxed		// number of valid C-frames received
.reg @csix_rx_num_buf_alloc			// number of buffers allocated (include prefetch)
.reg @csix_rx_num_def_buf_alloc		// number of buffers allocated (include prefetch) from sram
.reg @csix_rx_num_buf_alloc_fail	// number of buffer alloc failures (lack of buffers)
.reg @csix_rx_num_seq_error			// number of sequence errors while receiving C-frames
.reg @csix_rx_num_qm_sink			// number of pkts successfully sent to QM.
.reg @csix_rx_num_buf_drop_ring_full// number of buffers dropped because of QM ring full
.reg @csix_rx_num_err				// number of error cells recieved.
.reg @csix_rx_num_drop_mpkts		// number of mpkts dropped

#endif

//---------------------------------------------------------------------
// Transfer register definitions.
//---------------------------------------------------------------------

.reg $rsw_1 						; Cache RSW 1
.reg $rsw_2							; Cache RSW 2

.xfer_order 						$rsw_1 $rsw_2
#define rsw_num						&$rsw_1

.reg $prf_hndl						; prefetch buffer

.reg $prpnd0						; prepend header 0
.reg $prpnd1						; prepend header 1
.reg $prpnd2						; prepend header 2
.reg $prpnd3						; prepend header 3

.xfer_order 						$prpnd0, $prpnd1, $prpnd2, $prpnd3

//---------------------------------------------------------------------
// Signal definitions.
//---------------------------------------------------------------------

.sig volatile msf_sig				; MSF read signal
.sig volatile rx_event				; receiving wake up signal

; Hardcoding signal numbers since these signals are used in signal
; masks. Hardcoding guarantees that the same signal number
; is not assigned to these signals.
.sig sram_sig						; SRAM access signal
.addr sram_sig						1
.sig sram_sig_rd					; SRAM read signal
.addr sram_sig_rd					5
.sig sram_sig_wr					; SRAM write signal
.addr sram_sig_wr					9
.sig dram_sig 						; DRAM access signal
.addr dram_sig						14
.sig prf_sig						; prefetch buffer used signal
.addr prf_sig						13

.sig scratch_sig					; scratch ring access signal
.sig volatile nxt_thrd_1			; next thread signal 1
.sig volatile nxt_thrd_2			; next thread signal 2
.sig volatile nxt_thrd_3			; next thread signal 3

//	Map signal to signal address
.addr rx_event						RX_EVENT_SIG
.addr nxt_thrd_1 					NXT_THRD_SIG_1
.addr nxt_thrd_2 					NXT_THRD_SIG_2
.addr nxt_thrd_3 					NXT_THRD_SIG_3
 	 
//------------------------------------------------
//	Main entry point - Code begins execution here.
//------------------------------------------------

	; Initialization phase.
	; This macro also does system initialization.
	csix_rx_init()

system_init_done#:

start#:

	; Recurring phase.

	; When a c-frame is received, the next thread from the
	; thread freelist is signaled. The thread enters here.
	; The RSW for the c-frame is in $rsw_1 and $rsw_2.

	; Check if an erroneous c-frame was received.
	_check_rsw_error(err_code, $rsw_1)

	; If error, goto 'err_proc#'
	bne[err_proc#], defer[2]

	; Get RBUF element number.
	_get_element_num(ele_num,$rsw_1)

	; Compute element address.
	#if IS_IXPTYPE(__IXP28XX)
	alu_shf[ele_addr, --, b, ele_num, <<6]
	#else
	alu_shf[ele_addr, --, b, ele_num, <<7]
	#endif

	_csix_rx_debug_incr_counter(@csix_rx_num_cells_rxed)

	#ifdef CHECK_ZERO_BUFFER_HANDLE

	; Check if prefetched buffer handle was zero, implying null buffer.
	alu[--,$prf_hndl,-,0x0]
	beq[buf_handle_zero#]

	#endif // CHECK_ZERO_BUFFER_HANDLE

phase_1#:

	; Read prepend header.
	msf[read,$prpnd0, rbuf_base, ele_addr, 4], sig_done[msf_sig]

	ctx_arb[nxt_thrd_1, msf_sig], defer[2]

	; Get class ID
	_get_ext_class(ext_class,$rsw_2)
	; Get payload length
	_get_pld_len(payload_len,$rsw_1)

phase_2#:

	; Send next sig 1.
	_send_next_thrd_sig(nxt_thrd_1_gpr)

	; We have used $rsw_1 and $rsw_2 for the current c-frame.
	; So we can add the thread to the receive freelist.
	msf[fast_wr, --, msf_freelist0_addr, flst_reg]

	; Get data offset.
	alu[c_offset, 0xFF, and, $prpnd0, >>24]

	; Get SOP/EOP flags 
	alu[flags, 0x3, and, $prpnd0, >>6]

	; Check if both SOP and EOP flags are set 
	br!=byte[flags, 0, 0x3, nsep#], defer[3]

	; Get ingress blade number.
	alu[in_bld_num, 0x3F, and, $prpnd0]

	; Get class id.
	#ifdef USE_EXT_CLASS_ID
	; Get class id from extension class id (present in csix extension header)
	ld_field_w_clr[class_id, 0011, ext_class]
	#else
	; Get class id from qos class id (present in csix prepend header)
	ld_field_w_clr[class_id, 0011, $prpnd1]
	#endif

	; Get next hop id.
	ld_field_w_clr[next_hop_id, 0011, $prpnd1, >>16]
	
sep#:

	; SOP and EOP flags are set.
	; Do SOP EOP c-frame processing and return to 'start#'
	_csix_rx_sep_proc(start#)

nsep#:
	
	; Check if only SOP flag is set.
	br_bclr[flags, 0, nsop#]

sop#:

	; Only SOP flag is set.
	; Do SOP c-frame processing and return to 'start#'
	_csix_rx_sop_proc(start#)

nsop#:

	; Check if only EOP flag is set.
	br_bclr[flags, 1, mop#]

eop#:

	; Only EOP flag is set.
	; Do EOP c-frame processing and return to 'start#'
	_csix_rx_eop_proc(start#)

mop#:

	; Do MOP c-frame processing and return to 'start#'
	_csix_rx_mop_proc(start#)

err_proc#:

	; Do erroneous c-frame processing and return to 'start#'
	_csix_rx_err_proc(start#)

#ifdef CHECK_ZERO_BUFFER_HANDLE
buf_handle_zero#:

	; Do buffer zero processing and return to 'start#'
	_csix_rx_buf_zero_proc(start#)
#endif


#endif	// __CSIX_RX_UC__