///////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
///////////////////////////////////////////////////////////////////////
//
//
//      File Name: csix_rx_common_util.uc
//
//      Purpose: Contains macros used by CSIX RX that are common to one ME
//               and two ME configurations.
//
//      Date            Comment                         By
//      ---------------------------------------------------------------
//
//      02/26/2002      Created                         David Meng
//		12/03/2002		Modified to add 2 ME solution	asv
//
///////////////////////////////////////////////////////////////////////

#ifndef	CSIX_RX_COMMON_UTIL_UC_
#define	CSIX_RX_COMMON_UTIL_UC_


#include <stdmac.uc>
#include <localmem.uc>
#include <csix_rx.h>
#include <csix_rx_context.h>

/////////////////////////////////////////////////////////////////////////////
//	Locally define s0 to s7 signals to be used in sram initialization

#define s0  msf_sig
#define s1  rx_evet
#define	s2  sram_sig
#define s3	sram_sig_rd
#define s4  sram_sig_wr 
#define s5  dram_sig 
#define s6  scratch_sig
#define s7  prf_sig


/////////////////////////////////////////////////////////////////////////////
// _get_current_me_thread_number
//	
//	Description:
//		This macro this macro gets current 
//		ME numebr and current thread number
//
//	Outputs:	
//		out_me_number--current me number
//  	outthread_num--current thread number
//
//		output_input:	None
//		
//	Inputs:
//		output_input:	None
//		input:	None
//
//		CONSTANTS	:	None
//	
//
//	Size:	7 instructions
/////////////////////////////////////////////////////////////////////////////

#macro _get_current_me_thread_number[out_me_num, out_thread_num]
	
	//	Read active context status register
	local_csr_rd[active_ctx_sts]
	immed[out_me_num,0] 
	//	IXP2400 active_ctx_sts[7:3] are ME number and cluster number, 
	//	and defined as following:
	//	ME number        7  6  5  4  3  This is the latest definition    
	//    0              0  0  0  0  0
	//    1              0  0  0  0  1
	//    2              0  0  0  1  0
	//    3              0  0  0  1  1
	//    4              1  0  0  0  0
	//    5              1  0  0  0  1
	//    6              1  0  0  1  0
	//    7              1  0  0  1  1
	//  bit 5,4,3 are the current thread number	

	alu[out_thread_num, 0x07, and, out_me_num]
	alu_shf_right[out_me_num, 0x1f, and, out_me_num, 3]

#if IS_IXPTYPE(__IXP24XX)
	
	//	For IXP2400, Cluster bit should be set
	//	If me number greater than 4, reset cluster bit and 
	//	add 4 to the ME number in order to get correct ME number 
	
	br_bclr[out_me_num, 4, first_cluster_processing#] 
	alu[out_me_num, out_me_num, and, 0x0f]
	alu[out_me_num, out_me_num, +, 0x4]
first_cluster_processing#:
#endif
//	IXP2800 active_ctx_sts[7:3] are ME number and cluster number, 
//	and defined as following:
//	ME number          7  6  5  4  3  This is the latest definition    
//	    0              0  0  0  0  0
//	    1              0  0  0  0  1
//	    2              0  0  0  1  0
//	    3              0  0  0  1  1
//		4              0  0  1  0  0
//		5              0  0  1  0  1
//		6              0  0  1  1  0
//		7              0  0  1  1  1
//		8              1  0  0  0  0
//		9              1  0  0  0  1
//		10             1  0  0  1  0
//		11             1  0  0  1  1
//		12             1  0  1  0  0
//		13             1  0  1  0  1
//		14             1  0  1  1  0
//		15             1  0  1  1  1
//  bit 5,4,3 are the current thread number

#endm


/////////////////////////////////////////////////////////////////////////////
// _config_msf_rx_ctrl
//	
//	Description:
//			Configure MSF RX control reg for CSIX RX micro code
//
//	Outputs:		
//		None
//		
//	Inputs:
//		None
//		
//	CONSTANTS:	
//		None
//	
//	Size: 8 instructions
/////////////////////////////////////////////////////////////////////////////

#macro _config_msf_rx_ctrl[] 
.begin
/////////////////////////////////////////////////////////////////////////////

	.reg $tmp
	.reg msf_rx_addr

	#if IS_IXPTYPE(__IXP28XX)
	move[$tmp, 0x00000005] 								; config MSF RX as CSIX
	#else
	move[$tmp, 0x00410204] 								; config MSF RX as CSIX
	#endif

	move[msf_rx_addr,MSF_RX_CNTL]
	msf[write, $tmp, msf_rx_addr, 0, 1], \
		sig_done[msf_sig]
	ctx_arb[msf_sig]

	#if (!IS_IXPTYPE(__IXP28XX))
	_free_all_elements[]								; free all RBUF elements
	#endif

	move[$tmp, 0x0F] 	; 
	move[msf_rx_addr,HWM_CONTROL]
	msf[write, $tmp, msf_rx_addr, 0, 1], \
		sig_done[msf_sig]
	ctx_arb[msf_sig]

	_set_csix_type_map[]

	#if IS_IXPTYPE(__IXP28XX)
	_set_rx_thread_freelist_timeout[0,\
		RX_THREAD_FREELIST_TIMEOUT_0]

	; Set RX_PORT_MAP
	.reg $temp zero
	.sig msf_wr_sig
	move[$temp, 0x000000ff]
	immed[zero, 0]
	msf[write, $temp, zero, RX_PORT_MAP, 1], sig_done[msf_wr_sig]
	ctx_arb[msf_wr_sig]


	#else
	_set_rx_thread_freelist_timeout[0,\
		RX_THREAD_FREELIST_TIMEOUT_0]
	_set_rx_thread_freelist_timeout[1,0x0]
	_set_rx_thread_freelist_timeout[2,0x0] 
	_set_rx_thread_freelist_timeout[3,0x0] 
	#endif

	#if IS_IXPTYPE(__IXP28XX)
	move[$tmp, 0x80000005] 								; Enable the CSIX Receive
	#else
	move[$tmp, 0x10410204] 								; Enable the CSIX Receive
	#endif

	move[msf_rx_addr,MSF_RX_CNTL]
	msf[write, $tmp, msf_rx_addr, 0, 1], \
		sig_done[msf_sig]
	ctx_arb[msf_sig]

.end
#endm


/////////////////////////////////////////////////////////////////////////////
// _set_csix_type_map[]
//	
//	Description:
//		set CSIX_TYPE_MAP reg for CSIX RX micro code
//
//	Outputs:	
//		None

//	Inputs:
//		output_input:	None
//		input:	None
//
//		CONSTANTS	:	None
//	
//
//	Size: 3 instructions
/////////////////////////////////////////////////////////////////////////////

#macro _set_csix_type_map[] 
.begin

	.reg $tmp
	.reg msf_rx_addr

#ifdef	MULTICAST_SUPPORT_ENABLE

	move[$tmp, 0x88]

#else

	move[$tmp, 0x08]

#endif

	move[msf_rx_addr,CSIX_TYPE_MAP]
	msf[write, $tmp, msf_rx_addr, 0, 1], sig_done[msf_sig]
	ctx_arb[msf_sig]

.end
#endm

/////////////////////////////////////////////////////////////////////////////
// _disable_msf_interrupt
//	
//	Description:
//		Disable MSF interrupt to the Xcale
//
//	Outputs:	
//		None
//		
//	Inputs:
//		None
//
//	CONSTANTS	:	
//		None	
//
//	Size: 4 instructions
/////////////////////////////////////////////////////////////////////////////

#macro _disable_msf_interrupt[] 
.begin
/////////////////////////////////////////////////////////////////////////////
// Local register declaration
.reg $tmp 												; tmporarily used $xfer
.reg msf_rx_addr										; MSF RX address gpr

	move[$tmp,0]
	move[msf_rx_addr,MSF_INTERRUPT_ENABLE]
	msf[fast_wr, --, msf_rx_addr, tmp, 1]
.end
#endm



/////////////////////////////////////////////////////////////////////////////
// _set_rx_thread_freelist_timeout
//	
//	Description:
//		initialize MSF RX_thread_freelist_timeout 
//		register for CSIX RX micro code
//
//	Outputs:	
//		None

//	Inputs:
//		ofreelist_num:	thread freelist number need to be set
//		timeout_val:timeout value in CPP bus clock should less than 0xffff
//
//		CONSTANTS	:	None
//	
//
//	Size: 4 instructions
/////////////////////////////////////////////////////////////////////////////
#macro _set_rx_thread_freelist_timeout[freelist_num, timeout_val] 
.begin

/////////////////////////////////////////////////////////////////////////////
// Local register declaration

.reg tmp 												; tmporarily used gpr
.reg msf_rx_addr										; MSF RX address gpr

	#if(freelist_num == 0)
		move[msf_rx_addr,RX_THREAD_FREELIST_TO_0]
	#elif(freelist_num == 1)
		move[msf_rx_addr,RX_THREAD_FREELIST_TO_1]
	#elif(freelist_num == 2)
		move[msf_rx_addr,RX_THREAD_FREELIST_TO_2]
	#else
		move[msf_rx_addr,RX_THREAD_FREELIST_TO_3]
	#endif
	move[tmp, timeout_val]
	alu[tmp,--, B, tmp, <<16]
	msf[fast_wr, --, msf_rx_addr, tmp]
.end
#endm

/////////////////////////////////////////////////////////////////////////////
// _set_rx_up_ctrl
//	
//	Description:
//		initialize MSF Rx_UP_control reg: 1 clock decode response time, no parity
//		and CP_mode is packet for CSIX RX micro code//	
//
//	Outputs:	
//		None
//		
//	Inputs:
//		None
//
//	CONSTANTS:	
//		None
//	
//
//	Size: 4 instructions
/////////////////////////////////////////////////////////////////////////////
#macro _set_rx_up_ctrl[rx_up_ctrl_num] 
.begin

/////////////////////////////////////////////////////////////////////////////
// Local register declaration

.reg tmp 												; tmporarily used gpr
.reg msf_rx_addr										; MSF RX address gpr

	#if(rx_up_ctrl_num == 0)
		move[msf_rx_addr,RX_UP_CTRL_0]
	#elif(rx_up_ctrl_num == 1)
		move[msf_rx_addr,RX_UP_CTRL_1]
	#elif(rx_up_ctrl_num == 2)
		move[msf_rx_addr,RX_UP_CTRL_2]
	#else
		move[msf_rx_addr,RX_UP_CTRL_3]
	#endif


	alu[tmp, --, B, 0xd]	

	alu[tmp, --, B, tmp, <<16]
	msf[fast_wr, --, msf_rx_addr, tmp]
.end
#endm

/////////////////////////////////////////////////////////////////////////////
// _set_freelist_reg
//	
//	Description:
//		Set the register used to add a thread to Rx_Thread_freelist
//
//	Outputs:	
//		None
//		
//	Inputs:
//		sig: signal to wake up the thread
//		m_n: ME number 
//		t_n: thread number
//		x_n: the first sram transfer register in a sram register array
//
//
//	CONSTANTS	:	None
//	
//
//	Size: 4 instructions
/////////////////////////////////////////////////////////////////////////////
#macro _set_freelist_reg[freelist_reg, sig, m_n, t_n, x_n] 
.begin
	alu[freelist_reg,--,B,x_n]
	alu_shf[freelist_reg, freelist_reg, OR,sig, <<SIGNAL_NUMBER_BITS]
	alu_shf[freelist_reg, freelist_reg, OR,m_n, <<ME_NUMBER_BITS]
	alu_shf[freelist_reg, freelist_reg, OR,t_n, <<THREAD_BITS]
.end
#endm

/////////////////////////////////////////////////////////////////////////////
// send_next_thrd_sig
//	
//	Description:
//		Send sig_gpr signal to next thread
//
//	Outputs:	
//		None
//  
//
//		
//	Inputs:
//		None
//
//
//	CONSTANTS:	
//		None
//	
//
//	Size: 1 instruction
/////////////////////////////////////////////////////////////////////////////
#macro _send_next_thrd_sig[sig_gpr] 
.begin
	local_csr_wr[SAME_ME_SIGNAL, sig_gpr]
.end
#endm

/////////////////////////////////////////////////////////////////////////////
// _clear_all_lm_crx
//	
//	Description:
//		Clear the all CSIX RX context entries
//
//	Outputs:	
//		None
//		
//	Inputs:
//		None

//
//	CONSTANTS:	
//		None
//	
//
//	Size: 52 instructions
/////////////////////////////////////////////////////////////////////////////
#macro _clear_all_lm_crx[] 
.begin
/////////////////////////////////////////////////////////////////////////////
// Local register declaration
.reg count												; count
.reg lm_addr											; lm address gpr

	move[count, 16]
	move[lm_addr, CRX_LM_BASE]
	local_csr_wr[active_lm_addr_0,lm_addr]
	alu_shf[count,--,B, count, <<4] 
loop#:
	alu[count, count, -, 0x1] 
	bne[loop#],defer[1]
	alu[*l$index0++, --, B, 0x0]
.end
#endm

/////////////////////////////////////////////////////////////////////////////
// _set_all_cam_entry_state
//	
//	Description:
//		set all cam entry state 
//
//	Outputs:	
//		None
//		
//	Inputs:
//		state_value: the state value need to be set
//
//	CONSTANTS	:	None
//	
//
//	Size: 66 instructions
/////////////////////////////////////////////////////////////////////////////
#macro _set_all_cam_entry_state[state_value] 
.begin
/////////////////////////////////////////////////////////////////////////////
// Local register declaration

.reg count													; count gpr
.reg entry_reg												; CAM entry gpr
 
	move[count, 0x0]
	move[entry_reg, 0x0]
set_cam_loop#:
	cam_write[entry_reg, count, state_value]
	alu[--,count,-,15]
	bne[set_cam_loop#],defer[2]
		alu[count, count, +, 0x1]
		alu[entry_reg, entry_reg, +, 0x1]
.end
#endm


/////////////////////////////////////////////////////////////////////////////
// check_rsw_error
//	
//	Description:
//		Get all error fields from RSW
//
//	Outputs:	
//		
//	Inputs:
//		rsw1: receive status word 1
//		tmp_err: all errors
//
//	CONSTANTS:	
//		None
//	
//
//	Size: 1 instruction
/////////////////////////////////////////////////////////////////////////////
#macro _check_rsw_error[tmp_err, rsw1] 
.begin
	alu[--,rsw1,AND,tmp_err ] 
.end
#endm


/////////////////////////////////////////////////////////////////////////////
// _get_element_num 
//	
//	Description:
//		Get receiving element number from rsw1
//
//	Outputs:	
//		element: the receiving elemnet number
//		
//	Inputs:
//		rsw1: receive status word 1
//
//	CONSTANTS:	
//		None
//	
//
//	Size: 1 instruction
/////////////////////////////////////////////////////////////////////////////
#macro _get_element_num[element_num, rsw1] 
.begin
	alu_shf[element_num, 0x7F, and, rsw1, >>24]
.end
#endm

/////////////////////////////////////////////////////////////////////////////
// _get_pld_len
//	
//	Description:
//		Get receiving cframe payload length
//
//	Outputs:	
//		pld_len: payload length
//		
//	Inputs:
//		rsw1: receive status word 1
//
//	CONSTANTS:	
//		None
//	
//
//	Size: 1 instruction
/////////////////////////////////////////////////////////////////////////////
#macro _get_pld_len[pld_len, rsw1] 
.begin
	alu[pld_len,0xff,AND,rsw1,>>16]
.end
#endm

/////////////////////////////////////////////////////////////////////////////
// _get_ext_class
//	
//	Description:
//		Get extension header class ID
//
//	Outputs:	
//		out_class: Class ID
//		
//	Inputs:
//		rsw2: receive status word 2
//
//	CONSTANTS:	
//		None
//	
//
//	Size: 1 instruction
/////////////////////////////////////////////////////////////////////////////
#macro _get_ext_class[out_class, rsw2] 
.begin
	alu_shf[out_class, --, B, rsw2, >>24]
.end
#endm

/////////////////////////////////////////////////////////////////////////////
// _rbuf_to_dram
//	
//	Description:
//		move data from rbuf to DRAM buffer
//
//	Outputs:	
//		None
//		
//	Inputs:
//		dram_addr: the DRAM handle 
//		element_addr: the element address needs to be moved
//		ctx_num: thread num
//		in_rbuf_base: rbuf_base cached in this register
//		sid: signal used in DRAM awrite
//
//	CONSTANTS:	
//		None
//	
//
//	Size: 5 instructions
/////////////////////////////////////////////////////////////////////////////
#macro _rbuf_to_dram[dram_addr, ele_addr, ref_cnt, rbuf_base, sig]
.begin

.reg cnt, addr

	//	compute how many byte shoudl be moved from rbuf to dram 

	alu[cnt, ref_cnt, -, 0x1]

	//	compose the overwrite data

	alu[cnt, --, b, cnt, >>3]
	alu[cnt, dram_rbuf_ov, OR, cnt, <<21]

	//	get element address

	alu[addr,ele_addr, +, rbuf_base]

	//	do the over write for the indirect reference

	alu_shf[--,cnt,OR,addr,<<5]

	//	move data from rbuf to the dram

	dram[rbuf_rd,--,dram_addr,0,max_16],indirect_ref,\
		 sig_done[sig]

.end

#endm

/////////////////////////////////////////////////////////////////////////////
// _drop_current_pkt
//	
//	Description:
//		Drop current partially received packet
//
//	Outputs:	
//		None
//		
//	Inputs:
//		sop_handle: the sop buffer handle 
//		eop_handle: the eop buffer handle
//		voq_num:    virtual queue number
//		cnt_base:	global gpr
//	CONSTANTS:	
//		None
//	
//
//	Size: 5 instructions
/////////////////////////////////////////////////////////////////////////////

#macro _drop_current_pkt[sop_hndl, eop_hndl, voq_num]
.begin
	.reg tmp_sop
	.reg tmp_eop

	#ifdef	_DEBUG_COUNTERS_
	alu[@csix_rx_num_drop_mpkts, @csix_rx_num_drop_mpkts, +, 1]
	#endif

	#ifdef COUNTERS
	.reg addr											; address for counter
	alu[addr, --, B, voq_num, <<4]						; get conter addr
	alu[addr, addr, +, cnt_base]						; add conter base
	sram[incr, --, addr, PKT_DROPPED_CNT_IDX]			; incr pkt dropped count
	#endif

	alu[tmp_sop, sop_hndl, and~, 0xFF, <<24]			; Clean out flags/cell.
	alu[tmp_eop, eop_hndl, and~, 0xFF, <<24]			; Clean out flags/cell.
	alu[--,tmp_sop,-,tmp_eop]							; Campare handles.
	bne[drop_buffer_chain#]								; If need, drop buf chain.

	dl_buf_drop[sop_hndl] 								; Drop one buffer.
	br[drop_current_pkt_done#] 							; goto drop buffer done

drop_buffer_chain#:
	dl_buf_drop_chain[sop_hndl, eop_hndl]	 			; drop buffer chain

drop_current_pkt_done#:
.end
#endm

/////////////////////////////////////////////////////////////////////////////
// _free_all_elements
//	
//	Description:
//		Free all RBUF elements
//
//	Outputs:	
//		None
//		
//	Inputs:
//		None
//
//	CONSTANTS:	
//		None
//	
//
//	Size: 193 instructioncycles
/////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////
#macro _free_all_elements[]
.begin
/////////////////////////////////////////////////////////////////////////////
// Local register declaration
.reg ele_num											; element number
.reg tem_addr											; temporarily used address

	immed[ele_num, 0]
	.while (ele_num < 48)
		alu_shf[temp_addr, --, B, ele_num,<<16] 		 ; get element address
		msf[fast_wr, --, msf_rx_element_addr, temp_addr] ; free the elemnt
		alu[ele_num, ele_num,  + , 1]
	.endw
.end
#endm

/////////////////////////////////////////////////////////////////////////////
// _csix_rx_sink
//	
//	Description:
//		Send received packet to the next processing block by writing scratch ring
//
//	Outputs:	
//		None
//		
//	Inputs:
//		s_hl:			SOP buffer handle register, this register value will be modified
//						after macroc is excuted.  If the caller want to keep this value  
//						the value should be stored some where 
//		r_e_hl:			Raw EOP buffer handle
//		e_e_hl:			Encoded EOP buffer handle: NULL for one buffer cframe
//						otherwise will be real EOP handle  
//		cls_n:			Class number
//		o_n:			Output port number
//		p_sz:			packet size 
// 		c_cnt:			cell count for SOP buffer : used in cell mode 
//		flow_color_id 	flow and color id 
//		rx_sig:			Signal used to wait receving logid wake up  
//		scr_sig:		Scratch ring write signal
//		rt_lbl:			return lable after waked up by receiving logic 
//		voq_num:		virtual queue number
//
//	CONSTANTS:	
//		None
//	
//
//	Size: 10 instructions
/////////////////////////////////////////////////////////////////////////////
#macro _csix_rx_sink[s_hl, r_e_hl, e_e_hl, cls_n, o_n, p_sz, c_cnt, flow_color_id, \
					 rx_sig, scr_sig, rt_lbl, voq_num]

.begin
/////////////////////////////////////////////////////////////////////////////
// Local register declaration
.reg tmp_0													; temporary gpr 0
.reg tmp_1													; temporary gpr 1

	// Check scratch ring fullness

	br_inp_state[CSIX_RING_FULL, csix_rx_sink_scr_full#]	; check ring fullness
	alu[s_hl,s_hl, and~, 0x3F, <<24]						; clean out cell_cnt field

#ifdef CELL_MODE
	// work in cell mode, put cell count in the SOP buf handle	
	alu[s_hl,s_hl, or, c_cnt, <<24]							; set cell_cnt
#else
	alu[tmp_0, --, b, p_sz,>>PKT_DIV]					; pkt size div by 256
	alu[tmp_0, tmp_0, +,0x1]								; pkt size div by 256
	alu[s_hl,s_hl, or, tmp_0, <<24]							; set pkt size
#endif														; to SOP hndl

	//increment the debug counter
	_csix_rx_debug_incr_counter[@csix_rx_num_qm_sink]

	dl_qm_sink[s_hl, e_e_hl, cls_n, o_n, p_sz, flow_color_id, rx_sig, \
	           scr_sig, rt_lbl, to_qm_ring]					; write scratch ring

csix_rx_sink_scr_full#:

	#ifdef TWO_ME_CSIX_RX
	.if (ctx() == 7) 

		; Signal thread 0 of the next me.
		alu[--, --, B, inter_me_sig_2_gpr]
		cap[fast_wr, ALU, interthread_sig]	

		; End of critical section.

	.endif
	#endif // TWO_ME_CSIX_RX

	//	the scratch ring is full, the packet has to be droped

	alu[tmp_0,--,B, s_hl]
	alu[tmp_1,--,B, r_e_hl]

	// increment debug counter
	_csix_rx_debug_incr_counter[@csix_rx_num_buf_drop_ring_full]

	_drop_current_pkt[tmp_0, tmp_1, voq_num]				; drop current pkt
	ctx_arb[rx_sig],br[rt_lbl]								; ctx swiap
.end
#endm

/////////////////////////////////////////////////////////////////////////////
// _get_rand_buf_offst
//	
//	Description:
//		Get virtual random buffer data offset to take the advantage of the
//		4 DRAM banks to improve the speeds
//
//	Outputs:	
//		buf_offset: the return value for a buffer offset
//		
//	Inputs:
//		None	 
//		
//	CONSTANTS:	
//		None
//	
//	Globale variables:
//     @rand_buf_offset		; used and modified
//	   @rand_offset_mask	; used not modified
//
//	Size: 3 instructions
/////////////////////////////////////////////////////////////////////////////

#macro	_get_rand_buf_offst[buf_offset]
.begin

#if IS_IXPTYPE(__IXP28XX)
	// For IXP2800 we do not need to round robin amongst buffer offsets
	// to achieve uniform DRAM bank utilization.s
	immed[buf_offset, 384]
#else
	alu[@rand_buf_offset, rand_offset_mask, AND, @rand_buf_offset]
	alu[@rand_buf_offset, DDR_BANK_SIZE, +, @rand_buf_offset]
	alu[buf_offset, --, B, @rand_buf_offset]
#endif


.end
#endm

/////////////////////////////////////////////////////////////////////////////
// _get_cell_cnt
//	
//	Description:
//		Compute the cell count 
//
//	Outputs:	
//		cell_cnt: the new cell count value 
//		rmdr:	The new value of the remainder
//		
//	Inputs:
//		cell_cnt:	The original value of the cell count
//		rmdr:		The original value of the remainder	 
//		byte_cnt:	The byte count used to compute new cell count
//
//	CONSTANTS:	
//		None
//	
//	Globale variables:
//		cell_size:  This is cached cell size
/////////////////////////////////////////////////////////////////////////////

#macro	_get_cell_cnt[cell_cnt, rmdr, byte_cnt]
.begin
	alu[rmdr, rmdr, +, byte_cnt]						; get data count
cell_cnt_count_loop#:
	alu[--, rmdr, -, cell_size]
	ble[cell_cnt_done#]
	br[cell_cnt_count_loop#], defer[2]
	alu[rmdr,rmdr,-,cell_size]
	alu[cell_cnt, cell_cnt, +, 1]
cell_cnt_done#:
.end
#endm

/////////////////////////////////////////////////////////////////////////////
// _csix_rx_debug_init_counters
//	
//	Description:
//		Initialize the debug counters to zero.
//
//	Outputs:	
//	
//		
//	Inputs:
//
//
//	CONSTANTS:	
//		None
//	
//	Globale variables:
//		The debug counters in absolute registers.
/////////////////////////////////////////////////////////////////////////////
#macro _csix_rx_debug_init_counters[]

#ifdef	_DEBUG_COUNTERS_

	immed[@csix_rx_num_cells_rxed,0]		
	immed[@csix_rx_num_buf_alloc,0]		
	immed[@csix_rx_num_def_buf_alloc, 0]
	immed[@csix_rx_num_buf_alloc_fail,0]		
	immed[@csix_rx_num_seq_error,0]		
	immed[@csix_rx_num_qm_sink,0]		
	immed[@csix_rx_num_buf_drop_ring_full,0]		
	immed[@csix_rx_num_err, 0]
	immed[@csix_rx_num_drop_mpkts, 0]

#endif
#endm


/////////////////////////////////////////////////////////////////////////////
// _csix_rx_debug_incr_counter
//	
//	Description:
//		Initialize the debug counters to zero.
//
//	Outputs:	
//	
//		
//	Inputs:
//
//
//	CONSTANTS:	
//		None
//	
//	Globale variables:
//		The debug counters in absolute registers.
/////////////////////////////////////////////////////////////////////////////
#macro _csix_rx_debug_incr_counter[counter]

#ifdef	_DEBUG_COUNTERS_

	alu[counter,counter,+,1]

#endif
#endm


///////////////////////////////////////////////////////////////////////////////
//
// _csix_rx_buf_alloc
//
//	 	Description: 
//
//			Allocates a buffer either from nn ring or from sram. This macro
//          should be used only when the Packet TX microblock puts free buffers
//          on this ME's nn ring. Currently supported only when TWO_ME_CSIX_RX
//			switch is used.
//
//	 	Outputs:
//			Global registers $prf_hndl and
//          prf_hndl_gpr.
//		Inputs:
//			free_list	:	Specifies the pool from which buffers are to be allocated.
//			req_sig		:	Signal to use in the I/O operation.
//			sig_action	:	What to do with I/O operation - 
//							- wait for signal(s) as specified by sig_action
//							- do not wait for signal, just return (SIG_NONE)
//			sig_mask	:	Signal mask to add 'req_seg' if necessary.
//		Size: 
//
///////////////////////////////////////////////////////////////////////////////
#macro _csix_rx_buf_alloc(free_list, req_sig, sig_action, sig_mask)
.begin

	#if (streq('sig_action', 'SIG_NONE'))

	; First check if a buffer freed by the Packet TX microblock and placed on
	; the nn ring is available. If available, it saves us a trip to sram.
	br_inp_state[NN_EMPTY, default_buf_alloc#]

	; Buffer is available on nn-ring. Copy it into 'prf_hndl_gpr'.
	alu[prf_hndl_gpr, --, b, *n$index++]

	#ifdef FIRST_CSIX_RX_ME
	br_inp_state[NN_FULL, continue#]

	br_inp_state[NN_EMPTY, continue#]
	.reg tmp
	alu[tmp, --, b, *n$index++]
	alu[*n$index++, --, b, tmp]

continue#:
	#endif

	; When we use this buffer, we do not know if it was fetched from nn ring or
	; from sram. This instruction saves us a branch that we would otherwise need.
	; to figure this out. Using this instruction, we insure that the prefetched
	; buffer handle is always in 'prf_hndl_gpr'.
	; Before using 'prf_hndl_gpr', we will do 
	; prf_hndl_gpr = prf_hndl_gpr xor $prf_hndl
	; to get the buffer handle in 'prf_hndl_gpr'.
	alu[prf_hndl_gpr, prf_hndl_gpr, xor, $prf_hndl]
	br[end#]

default_buf_alloc#:

	_csix_rx_debug_incr_counter(@csix_rx_num_def_buf_alloc)

	; Buffer was not available on nn ring. Fetch it from sram.
	dl_buf_alloc($prf_hndl, free_list, req_sig, sig_action)
	alu[sig_mask, sig_mask, or, 1, <<&req_sig]

	; When we use this buffer, we do not know if it was fetched from nn ring or
	; from sram. This instruction saves us a branch that we would otherwise need.
	; to figure this out. Using this instruction, we insure that the prefetched
	; buffer handle is always in 'prf_hndl_gpr'.
	; Before using 'prf_hndl_gpr', we will do 
	; prf_hndl_gpr = prf_hndl_gpr xor $prf_hndl
	; to get the buffer handle in 'prf_hndl_gpr'.
	immed[prf_hndl_gpr, 0]

end#:

	#else

	dl_buf_alloc($prf_hndl, free_list, req_sig, sig_action)
	immed[prf_hndl_gpr, 0]

	#endif

.end
#endm

#endif // CSIX_RX_COMMON_UTIL_UC_