//////////////////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
///////////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: csix_rx_context.h
//
//      Purpose: Define CSIX RX CONTEXT (CRX) data structure
//
//      History: Version 1.0
//
//
//      Date            Comment                         By
//      ---------------------------------------------------------------------
//
//      02/26/2002      Created                         David Meng
//		07/15/2003		Modified to replace unused
//						class id field with header type		asv
//		08/08/03		Converted 2 ME to context pipeline 	urn 
//
///////////////////////////////////////////////////////////////////////////////////

#ifndef _CSIX_RX_CONTEXT_H_
#define _CSIX_RX_CONTEXT_H_

///////////////////////////////////////////////////////////////////////////////////

#include <dl_system.h>

///////////////////////////////////////////////////////////////////////////////////

#define 		CRX_SRAM_BASE			CSIX_RX_CTX_SRAM_BASE	
#define_eval 	CSIX_RING_NUM			CSIX_RX_RING_OUT
#define_eval 	CSIX_RING_FULL			SCR_Ring/**/CSIX_RING_NUM/**/_Full
#ifdef		MULTICAST_SUPPORT_ENABLE
#define_eval	CSIX_MCAST_RING_NUM		CSIX_RX_RING_OUT_1
#define_eval	CSIX_MCAST_RING_FULL	SCR_Ring/**/CSIX_MCAST_RING_NUM/**/_Full
#endif	
#define 		DRAM_DATA_BUF_MAX_SIZE 	BUFFER_SIZE

///////////////////////////////////////////////////////////////////////////////////

#ifndef TWO_ME_CSIX_RX 

//	This header file define the data structure used in the context 
//	processing and packet rea-assembling.  Also, the base address 
//	for the local memory and other addresses are defined in this file

//	using C to define data type
//	#define uint32 unsigned int
//	#define unit8	 char

//	CRX is defined as following in a local memory and sram memory
//	crx{
//lword_0		uint32 pkt_size:16		 	packet size
//lword_0		uint32 cell_seq_num:16   	cell sequence number
//lword_1		uint32 sop_buf_handle:0-23	SOP buffer handle
//lword_2		uint32 sop_cell_count:24-29	SOP cell count
//lword_2		uint32 cur_buf_handle:0-23 	Current buffer handle
//lword_2		uint32 cur_cell_count:24-29	Current buffer cell count
//lword_3		uint32 cur_data_handle  	current data handle
//lword_4		uint32 cur_buf_size:0->15	current buffer size
//lword_4		uint32 sop_buf_ofst:16->31 	SOP buffer offset
//lword_5		uint32 output_port:0->15	output_port number
//lword_5		uint32 header_type:16->23 	header type
//lword_5		uint32 reserved:24->31	 	reserved
//lword_6		uint32 color_id:8			color id 
//lword_6		uint32 flow_id:24       	Flow id
//lword_7		uint32 prv_buf_hndl:0-23	24 bits previuos buffer handle
//lword_7		uint32 remainder:24-31		8 bits reminder 	
//			} //Total is 4X8=32 bytes 	

/////////////////////////////////////////////////////////////////////////////

#ifndef CRX_LM_BASE 
#define CRX_LM_BASE				0
#endif 

/////////////////////////////////////////////////////////////////////////////

#define PKT_SIZE_IDX			0
#define CELL_SEQ_NUM_IDX		0
#define SOP_BUF_HNDL_IDX		1
#define SOP_CELL_CNT_IDX		1
#define CUR_BUF_HNDL_IDX		2
#define CUR_CELL_CNT_IDX		2
#define CUR_DAT_HNDL_IDX		3
#define CUR_BUF_SIZE_IDX		4
#define SOP_BUF_OFST_IDX		4
#define OUTPUT_PORT_IDX			5
#define HEADER_TYPE_IDX			5
#define FLOW_COLOR_ID_IDX		6
#define PRV_BUF_HNDL_IDX		7
#define REMAINDER_IDX			7

/////////////////////////////////////////////////////////////////////////////

#define lm_pkt_size				*l$index0[PKT_SIZE_IDX]
#define lm_cell_seq_num			*l$index0[CELL_SEQ_NUM_IDX]
#define lm_sop_buf_hndl			*l$index0[SOP_BUF_HNDL_IDX]
#define lm_sop_cell_cnt			*l$index0[SOP_CELL_CNT_IDX]
#define lm_cur_buf_hndl			*l$index0[CUR_BUF_HNDL_IDX]
#define lm_cur_cell_cnt			*l$index0[CUR_CELL_CNT_IDX]
#define lm_cur_dat_hndl			*l$index0[CUR_DAT_HNDL_IDX]
#define lm_cur_buf_size			*l$index0[CUR_BUF_SIZE_IDX]
#define lm_sop_buf_ofst			*l$index0[SOP_BUF_OFST_IDX]
#define lm_output_port			*l$index0[OUTPUT_PORT_IDX]
#define lm_header_type			*l$index0[HEADER_TYPE_IDX]
#define lm_flow_color_id		*l$index0[FLOW_COLOR_ID_IDX]
#define lm_prv_buf_hndl			*l$index0[PRV_BUF_HNDL_IDX]
#define lm_remainder			*l$index0[REMAINDER_IDX]

////////////////////////////////////////////////////////////////////////////////////

#else //	TWO_ME_CSIX_RX 

////////////////////////////////////////////////////////////////////////////////////
//
// local memory in CSIX RX ME 0 
//
//
// lword_0		uint32 sop_buffer_offset		:16			SOP buffer offset 
// lword_0		uint32 packet_size				:16		 	Packet size
// lword_1		uint32 current_buffer_offset	:16   		Current Buffer Offset
// lword_1		uint32 header_type				:8			Header Type
// lword_1		uint32 sequence_number			:8			Cell sequence number
//
// local memory in CSIX RX ME 1 
//
// lword_0		uint32 current_buffer_handle			 	Current buffer handle
// lword_1		uint32 sop_buffer_handle					SOP buffer handle 
//
////////////////////////////////////////////////////////////////////////////////////

// Offset at which data starts for non sop buffers

#define NON_SOP_BUFFER_OFFSET 			0

// Number of bits for class id - 16 classes per port 

#define CSIX_RX_BITS_FOR_CLASS	  		4

// Number of bits for VoQ class id on outgoing scratch ring message

#define CSIX_RX_BITS_FOR_VOQ_CLASS		12

// define flags for passing from one microengine to the other in two ME mode 

#define	CSIX_RX_SOP_EOP_FLAGS			0x3
#define CSIX_RX_SOP_EOP_DROP_FLAGS		0x7
#define CSIX_RX_SOP_ONLY_FLAG			0x1

#define CSIX_RX_SOP_BIT					0x0
#define CSIX_RX_EOP_BIT					0x1
#define CSIX_RX_DROP_BIT				0x2
#define CSIX_RX_EOB_BIT					0x3
#define CSIX_RX_NON_EOP_BIT				0x0

// Flags 

#define CSIX_RX_NULL_BUFFER_FLAG 		0
#define CSIX_RX_ERROR_RBUF_FLAG 		1
#define CSIX_RX_CHECK_PREFETCH			0
#define CSIX_RX_DONT_CHECK_PREFETCH 	1
#define CSIX_RX_FREE_RBUF				0
#define CSIX_RX_DONT_FREE_RBUF			1

// Define the size of RBUF in CSIX Rx

#ifndef CSIX_RX_RBUF_SIZE
#define CSIX_RX_RBUF_SIZE		128
#endif 

// define the number of VOQ's supported

#ifndef CSIX_RX_NUMBER_OF_VOQ
#define CSIX_RX_NUMBER_OF_VOQ			256
#endif

// define the prepend sizes 

#define CSIX_RX_SOP_PREPEND_SIZE  		16
#define CSIX_RX_MOP_EOP_PREPEND_SIZE	8
#define CSIX_RX_PREPEND_DIFF			(CSIX_RX_SOP_PREPEND_SIZE - \
											CSIX_RX_MOP_EOP_PREPEND_SIZE)

// local memory base for context in ME0 and ME1 (256 of these)

#ifndef CRX_LM_BASE 
#define CRX_LM_BASE						0
#endif 

// local memory of first microengine 

#define PKT_SIZE_IDX					0
#define SOP_BUF_OFST_IDX				0
#define HEADER_TYPE_IDX					1
#define CELL_SEQ_NUM_IDX				1
#define CURR_BUF_OFST_IDX				1

// local memory of second microengine 

#define CUR_BUF_HNDL_IDX				0
#define SOP_BUF_HNDL_IDX				1

#define lm_pkt_size						*l$index0[PKT_SIZE_IDX]
#define lm_sop_buf_ofst					*l$index0[SOP_BUF_OFST_IDX]
#define lm_cell_seq_num					*l$index0[CELL_SEQ_NUM_IDX]
#define lm_header_type					*l$index0[HEADER_TYPE_IDX]
#define lm_curr_buf_ofst				*l$index0[CURR_BUF_OFST_IDX]
#define lm_sop_buf_hndl					*l$index0[SOP_BUF_HNDL_IDX]
#define lm_cur_buf_hndl					*l$index0[CUR_BUF_HNDL_IDX]

#endif // TWO_ME_CSIX_RX

////////////////////////////////////////////////////////////////////////////////////

#endif	// csix_rx_context.h 

////////////////////////////////////////////////////////////////////////////////////
