///////////////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
///////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: csix_rx_init.uc
//
//      Purpose: CSIX RX and system initialization.
//
//      Date            Comment                         By
//      ---------------------------------------------------------------------
//
//      2/27/2002      Created                         David Meng
//		12/03/2002		Modified to add 2 ME solution	asv
//
/////////////////////////////////////////////////////////////////////////////

#ifndef	CSIX_RX_INIT_UC_
#define	CSIX_RX_INIT_UC_

#include <dl_system.h>
#include <stdmac.uc>
#include <dispatch_loop.uc>
#include <system_init.uc>
#include <hardware.h>

#include <csix_rx.h>				
#include <csix_rx_context.h>		
#include <csix_rx_common_util.uc>			


/////////////////////////////////////////////////////////////////////////////
// csix_rx_init
//	
//	Description:
//		This macro initializes the MSF and put all thread into rx_Freelist_0 in
// 		the order 0 to 7
//
//	Outputs:	
//		This macro will set some globle GPRS
//		
//	Inputs:
//		None
//
//	CONSTANTS:	
//		None
//	
//
//	Size: instructions
/////////////////////////////////////////////////////////////////////////////

#macro csix_rx_init[]
.begin

/////////////////////////////////////////////////////////////////////////////

// Local used register declaration 

.reg temp_addr											; temporary addr gpr
.reg tmp0												; temporary gpr 0
.reg tmp1												; temporary gpr 1
.reg tmp2												; temporary gpr 2
.reg me_num												; me number 
.reg thrd_num											; thread number 


csix_rx_init_start#:

	#ifdef	COUNTERS
	immed32[cnt_base,CSIX_RX_COUNTERS_SRAM_BASE]		; set counter base gpr
	#endif

	#if (!IS_IXPTYPE(__IXP28XX))
	move[rand_offset_mask, OFFSET_MASK]					; set random offset mask
	#endif

	//	Following 4 instructions are called to 
	//  eliminate the warnings from the dispatch macro

	move[tmp0,0x0]
	move[tmp1,0x0]
	move[tmp2,0x0]

	dl_meta_init_cache[tmp0,tmp1,tmp2,tmp0,tmp1,tmp2,tmp0,tmp1]

	move[$rsw_1,0]										; clear RSW_1 write register
	move[$rsw_2,0] 										; clear RSW_2 write register

	sram[read, $rsw_1, tmp0,tmp1,2],ctx_swap[sram_sig]	; get ride off warnings

	move[err_code,ALL_ERROR_CODE] 						; cache error code
	move[msf_rx_element_addr, RBUF_ELEMENT_DONE] 		; RBUF ele done address
	immed32[crx_sram_gpr, CRX_SRAM_BASE]				; set SRAM CRX base address
	alu_shf[to_qm_ring,--, B, CSIX_RX_RING_OUT, <<2] 	; set scratch ring address

	#ifdef CELL_MODE
	alu[cell_size, --, B, CELL_SIZE_CNST]
	#endif

	_get_current_me_thread_number[me_num, thrd_num]

	//	Set next thread signal 1 and 2 gprs

	move[tmp0, 0x80]
	alu_shf[nxt_thrd_1_gpr, tmp0, OR, NXT_THRD_SIG_1, <<3]
	alu_shf[nxt_thrd_2_gpr, tmp0, OR, NXT_THRD_SIG_2, <<3]
	alu_shf[nxt_thrd_3_gpr, tmp0, OR, NXT_THRD_SIG_3, <<3]


	#ifdef TWO_ME_CSIX_RX

	; Setup default signal mask for phase 3.
	alu[phase_3_sigmask_default, --, b, 1, <<&(nxt_thrd_3)]
	alu[phase_3_sigmask_default, phase_3_sigmask_default, or, 1, <<&(sram_sig)]
	alu[phase_3_sigmask_default, phase_3_sigmask_default, or, 3, <<&(dram_sig)]
	#ifdef COUNTERS
	alu[phase_3_sigmask_default, phase_3_sigmask_default, or, 1, <<&(byte_cnt_sig)]
	#endif
	alu[phase_3_sigmask, --, b, phase_3_sigmask_default]

	.begin
	.reg sig

	; Setup gpr to signal end of critical section 1.
	alu[sig, --, B, &inter_me_sig_1]
    alu[sig, sig, OR, NEXT_CSIX_RX_ME, <<7]
    alu[inter_me_sig_1_gpr, --, B, sig]

	; Setup gpr to signal end of critical section 2.
	alu[sig, --, B, &inter_me_sig_2]
    alu[sig, sig, OR, NEXT_CSIX_RX_ME, <<7]
    alu[inter_me_sig_2_gpr, --, B, sig]

	.end

	#endif // TWO_ME_CSIX_RX

	//	set freelist register
	_set_freelist_reg[flst_reg, RX_EVENT_SIG, me_num, thrd_num, rsw_num]

	alu[flst_reg,--, B, flst_reg, <<16]
	move[msf_freelist0_addr,RX_THREAD_FREELIST_0]
	move[rbuf_base, RBUF_TBUF_BASE]							; set RBUF base
	alu[dl_buf_handle,--,B,IX_DL_BUF_NULL]				; clear dl_buffer_handle
	alu[dl_eop_buf_handle,--,B,IX_DL_BUF_NULL]			; clear dl_buffer_handle
	move[dram_rbuf_ov,((1 << 25)|(1<<4))]				; set overwrite gpr

.if(ctx()==0)

	#if (!IS_IXPTYPE(__IXP28XX))
	move[@rand_buf_offset, MAX_BUFFER_OFFSET]			; set global random buffer offset
	#endif

	#ifdef TWO_ME_CSIX_RX
	; Set the NN ring empty threshold to 0 LW's.
	; IN IXP2800 reference designs, the NN ring is used to
	; fetch buffers freed by Packet TX microblock.
	; This saves us a trip to sram to fetch buffers.
	.reg ctx_enable
	local_csr_rd[ctx_enables]
	immed[ctx_enable, 0]
	alu_shf[ctx_enable, ctx_enable, AND~, 3, <<18]
	alu_shf[ctx_enable, ctx_enable, OR, 0, <<18]
	local_csr_wr[ctx_enables, ctx_enable]
	local_csr_wr[nn_get, 0]
	local_csr_wr[nn_put, 0]	
	alu[--, --, b, 0]
	alu[--, --, b, 0]
	alu[--, --, b, 0]

	#endif
	
	//	Clear all CAM entries
	cam_clear											; clear CAM entries

	// If TWO_ME_CSIX_RX is defined, CSIX RX runs as a functional pipeline.
	// The cam of both ME's should start uninitialized.
	#ifndef TWO_ME_CSIX_RX
	_set_all_cam_entry_state[NORMAL_STATE]
	#endif

	//	Clear all local memory CRX entries
	_clear_all_lm_crx[]							

	#if (!IS_IXPTYPE(__IXP28XX))
	// set up value for IXP2400
	_set_rx_up_ctrl[0]
	#endif

	// System initialization
	#ifdef TWO_ME_CSIX_RX
	#ifdef FIRST_CSIX_RX_ME
	system_init[]

	#ifndef IXP_SIMULATION

	; Initialize NN ring with buffer handles so the threads do not 
	; need to do sram[dequeue] to fetch buffer handles.

	.reg count

	; Initialize NN ring of second CSIX RX ME

	immed[count, 0]

	.while(count<72)

	; Fetch a buffer handle from SRAM.
	dl_buf_alloc[$prf_hndl,BUF_FREE_LIST0,sram_sig, SIGNALS(sram_sig)]
	; Store the buffer handle in the NN ring.
	alu[*n$index++, --, b, $prf_hndl]
	alu[count, count, +, 1]

	.endw

	; Set the NN ring so that the ME can write to its own NN ring
	local_csr_rd[ctx_enables]
	immed[ctx_enable, 0]
	alu_shf[ctx_enable, ctx_enable, OR, 1, <<20]
	local_csr_wr[ctx_enables, ctx_enable]
	alu[--, --, b, 0]
	alu[--, --, b, 0]
	alu[--, --, b, 0]

	; Initialize NN ring of first (i.e. this) CSIX RX ME

	immed[count, 0]

	.while(count<72)

	; Fetch a buffer handle from SRAM.
	dl_buf_alloc[$prf_hndl,BUF_FREE_LIST0,sram_sig, SIGNALS(sram_sig)]
	; Store the buffer handle in the NN ring.
	alu[*n$index++, --, b, $prf_hndl]
	alu[count, count, +, 1]

	.endw

	; Reset the NN ring so that the previous ME can write to the NN ring
	local_csr_rd[ctx_enables]
	immed[ctx_enable, 0]
	alu_shf[ctx_enable, ctx_enable, AND~, 1, <<20]
	local_csr_wr[ctx_enables, ctx_enable]
	alu[--, --, b, 0]
	alu[--, --, b, 0]
	alu[--, --, b, 0]

	#endif // IXP_SIMULATION

	#endif
	#else 
	system_init[]
	#endif

	// initialize our debug counters
	_csix_rx_debug_init_counters[]


	#ifdef TWO_ME_CSIX_RX
		
	#ifdef FIRST_CSIX_RX_ME
	
		; Kickstart thread 0 of the first ME of the functional pipeline.

		local_csr_wr[same_me_signal, (&inter_me_sig_1<<3)]
	    local_csr_wr[same_me_signal, (&inter_me_sig_2<<3)]
	#else

		; Thread 0 of all the other ME in the functional pipeline waits 
		; for system initialization.
		.sig volatile system_init_sig
		.addr system_init_sig ME_INIT_SIGNAL
		ctx_arb[system_init_sig]
	#endif

	#endif // TWO_ME_CSIX_RX

.else

	ctx_arb[nxt_thrd_1]	 								; ctx swap

.endif
	
	// increment debug counter
	_csix_rx_debug_incr_counter[@csix_rx_num_buf_alloc]

	//	get prefetch buffer
	dl_buf_alloc[$prf_hndl,BUF_FREE_LIST0,sram_sig, SIGNALS(sram_sig)]

	//	get prefetch data handle
	dl_buf_get_data[prf_data_hndl, $prf_hndl]

	//	Add this thread to the freelist
	msf[fast_wr, --, msf_freelist0_addr, flst_reg]

	// Send next thread signal 1
	_send_next_thrd_sig[nxt_thrd_1_gpr]

initial_end#:

	// check if current thread 7
	alu[--,thrd_num,-,7]	

	// if current thread is not thread 7, go to 
	// no_msf_config
	bne[no_msf_config#]	

	// Signal next CSIX RX ME after adding all threads of this ME
	// to the thread freelist.
	#ifdef TWO_ME_CSIX_RX	
	#ifdef FIRST_CSIX_RX_ME
	signal_me(NEXT_CSIX_RX_ME, ME_INIT_SIGNAL)
	#endif
	#endif

	//	current thread is 7, configure MSF, 
	#ifdef TWO_ME_CSIX_RX
	#ifdef FIRST_CSIX_RX_ME
	_config_msf_rx_ctrl[]	
	#endif
	#else 
	_config_msf_rx_ctrl[]	
	#endif

	//  Send next thread 2 and 3 signals
	_send_next_thrd_sig[nxt_thrd_2_gpr]
	_send_next_thrd_sig[nxt_thrd_3_gpr]

no_msf_config#:

	// swap out and wait for the recieve logic to 
	// wake the thread

	ctx_arb[rx_event]

.end

#endm

#endif	// CSIX_RX_INIT_UC