///////////////////////////////////////////////////////////////////////////////
//
//
//                  I N T E L   P R O P R I E T A R Y
//
//     COPYRIGHT [c]  2002 BY  INTEL  CORPORATION.  ALL RIGHTS
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT
//     THE PRIOR WRITTEN PERMISSION OF :
//
//                        INTEL  CORPORATION
//
//                     2200 MISSION COLLEGE BLVD
//
//               SANTA  CLARA,  CALIFORNIA  95052-8119
//
///////////////////////////////////////////////////////////////////////
//
//      File Name: csix_rx_one_me_util.uc
//
//      Purpose: Contains macros used when CSIX RX runs on one ME.
//
//      Date            Comment                         By
//      ---------------------------------------------------------------
//
//      12/03/2002      Created                         asv
//
///////////////////////////////////////////////////////////////////////

#ifndef	__CSIX_RX_ONE_ME_UTIL_UC__
#define	__CSIX_RX_ONE_ME_UTIL_UC__

#include <dl_system.h>
#include <stdmac.uc>
#include <dispatch_loop.uc>
#include <hardware.h>

#include <csix_rx.h>				
#include <csix_rx_context.h>		
#include <csix_rx_common_util.uc>			

/////////////////////////////////////////////////////////////////////////////
// _csix_rx_sep_proc
//	
//	Description:
//		This macro will handle the SOP/EOP Cframe case when the CSIX RX
//		receives a normal Cframe.
//
//	Outputs:	
//		output:	Some global GPRs may be changed
//				ele_addr is modified
//	Inputs:
//		input:
//			rtn_label : After the macro finishes, return to this label
//
//		CONSTANTS	:	None
//	
//
//	Size:	
/////////////////////////////////////////////////////////////////////////////
#macro _csix_rx_sep_proc[rtn_label]
.begin

.reg buf_offset												; buffer offset
.reg lkup_key												; lookup key 
.reg lkup_result											; lookup result
.reg buf_size												; temporary buffer size
.reg seq_num												; temporary sequnce num

.reg output_port											; used to store output port number
.reg header_type											; used to store header type

.reg dram_handle											; packet data handle
.reg drop_sop_hndl											; drop SOP handle
.reg drop_eop_hndl											; drop EOP handle

#ifdef	COUNTERS
.reg $byte_cnt												; byte counter xfers
#define	byte_cnt_sig	msf_sig								; reuse msf_sig as byte 
#endif														; count sig

#ifdef	CELL_MODE
.reg cell_cnt												; cell count gpr
.reg rmdr 													; reminder gpr
#endif

	_get_rand_buf_offst[buf_offset]							; get random buffer offset

	// Using prefetch buffer to store the incoming 
	// SOP/EOP Cframe

	alu[dram_handle, prf_data_hndl, +, buf_offset]			; Get data handle

	//	The RBUF data start address = element_addr + 
	//  prepend header length for SOP/EOP Cframe

	alu[ele_addr, ele_addr,  +, SOP_PRPEND_HDR]
	
	// The RBUF data length = payload length - 
	// prepend header length

	alu[buf_size, payload_len, -, SOP_PRPEND_HDR]

	//	move received cframe to data buffer

	_rbuf_to_dram[dram_handle, ele_addr, buf_size, \
	              rbuf_base, dram_sig]

	alu[buf_size, buf_size,-,c_offset]						; deduct offset 

	alu[dl_buf_handle,--,B, $prf_hndl]						; store buf handle

	// increment debug counter
	_csix_rx_debug_incr_counter[@csix_rx_num_buf_alloc]

	//	get new prefetch buffer

	dl_buf_alloc[$prf_hndl, BUF_FREE_LIST0, prf_sig, SIG_NONE]

	//	Compose CAM lookup key

	alu[lkup_key, class_id, or, in_bld_num, <<4] 
	
#ifdef COUNTERS
	alu[cnt_addr, --, B, lkup_key, <<4]						; get conter addr
	alu[cnt_addr, cnt_addr, +, cnt_base]					; add base
	sram[incr, --, cnt_addr, PKT_RECEIVED_CNT_IDX]			; increment pkt count
	alu[$byte_cnt, --, B, payload_len]						; get payload length
	sram[add, $byte_cnt, cnt_addr, BYTE_RECEIVED_CNT_IDX],\
	     sig_done[byte_cnt_sig]								; add byte count
#endif

#ifdef CELL_MODE
	alu[cell_cnt,--,B,0x0]
	alu[rmdr,--,B,0x0]
	_get_cell_cnt[cell_cnt, rmdr, buf_size]
#endif

	//	Do CAM lookup & set up local memory address

	cam_lookup[lkup_result, lkup_key],lm_addr0[0]

	//	Check lookup result

	br!=byte[lkup_result, 1, 0x0, sep_cam_hit#],defer[2]

	; Set output port.
	ld_field_w_clr[output_port, 0011, $prpnd2]

	; Set header type.
	ld_field_w_clr[header_type, 0001, $prpnd2, >>16]

sep_cam_miss#:

.begin

	//	This is a CAM miss case
	//  LRU LM CRX=> SRAM, SRAM CRX=>$Xfers

.reg tmp_tag												; temporary tag
.reg tmp_entry												; temporary entry
.reg addr_in												; temporary in address
.reg addr_out												; temporary out address

	xbuf_alloc[$crx_in, 3, read]							; aloc 3 crx_in xbuffers
	xbuf_alloc[$crx_out, 8, write]							; alloc 7 crx_out xbuffers

	alu_shf[addr_in, crx_sram_gpr, or, lkup_key, <<5] 		; get CRX SRAM address  
	sram[read, $crx_in[0], addr_in, 0, 3], \
	     sig_done[sram_sig_rd]

	//	Move LRU LM CRX data to the $crx_out

	alu[$crx_out[0], --, B, *l$index0[0]]
	alu[$crx_out[1], --, B, *l$index0[1]]
	alu[$crx_out[2], --, B, *l$index0[2]]
	alu[$crx_out[3], --, B, *l$index0[3]]
	alu[$crx_out[4], --, B, *l$index0[4]]
	alu[$crx_out[5], --, B, *l$index0[5]]
	alu[$crx_out[6], --, B, *l$index0[6]]
	alu[$crx_out[7], --, B, *l$index0[7]]

	alu[tmp_entry, 0xF,and, lkup_result, >>3]				; get CAM entry
	cam_read_tag[tmp_tag,tmp_entry]							; read CAM tag
	alu[addr_out,crx_sram_gpr, or, tmp_tag, <<5]			; get LRU CRX SRAM address
	sram[write, $crx_out[0], addr_out, 0, 8], \
	     sig_done[sram_sig_wr]

	ctx_arb[nxt_thrd_2, sram_sig_rd, sram_sig_wr], and, \
	        defer[2]
		alu[c_offset, c_offset, +, buf_offset]				; compute  new offset
		cam_write[tmp_entry, lkup_key, NORMAL_STATE]		; update CAM LRU entry

sep_cam_miss_phase_3#:

	_send_next_thrd_sig[nxt_thrd_2_gpr]						; send next thread sig 2

	//	Check if expected cell sequence number is 0

	alu_shf[--, 0x0, or, $crx_in[0], >>16]

	//	If expected cell sequence number is not zero,
	//  goto sequence number error proc

	bne[sep_cam_miss_seq_not_zero#]	
	
	xbuf_alloc[$dl_meta, 7, write]							; alloc $dl_meta

	dl_meta_write[$dl_meta, dl_buf_handle, IX_DL_BUF_NULL,\
	              c_offset, buf_size, buf_size, FREE_LIST_ID, 0, header_type,0,\
				  output_port, 0, next_hop_id, $prpnd3, \
				  class_id, 0, sram_sig, SIG_NONE, 0, 7]	; Write meta data.

#ifdef COUNTERS
	ctx_arb[nxt_thrd_3, sram_sig, dram_sig, prf_sig, \
			byte_cnt_sig], and, defer[2]
#else
	ctx_arb[nxt_thrd_3, sram_sig, dram_sig, prf_sig],\
			and, defer[2]
#endif
		alu_shf[ele_addr, --, B, ele_num,<<16] 				; set element number
		alu[lm_cell_seq_num,--,B,0x0]						; set seq num as 0

	xbuf_free[$dl_meta]										; free $dl_meta

sep_common_phase_4#:

	// send next thread signal 3 to the next thread

	_send_next_thrd_sig[nxt_thrd_3_gpr]

	dl_buf_get_data[prf_data_hndl, $prf_hndl] 				; get data handle
	msf[fast_wr, --, msf_rx_element_addr, ele_addr]			; free current element
	
	// Send current packet to the next micro block

#ifdef CELL_MODE
	_csix_rx_sink[dl_buf_handle, dl_buf_handle,\
	              IX_DL_BUF_NULL, class_id, \
	              output_port, buf_size, cell_cnt, $prpnd3, rx_event, \
				  scratch_sig, rtn_label, lkup_key]
#else

	_csix_rx_sink[dl_buf_handle, dl_buf_handle,\
	              IX_DL_BUF_NULL, class_id, \
	              output_port, buf_size, 0, $prpnd3, rx_event, \
				  scratch_sig, rtn_label, lkup_key]
#endif

sep_cam_miss_seq_not_zero#:

	alu[lm_cell_seq_num,--,B,0x0]							; set sequence num 0
	xbuf_alloc[$dl_meta_2, 7, write]						; alloc $dl_meta_2
	dl_meta_write[$dl_meta_2, dl_buf_handle,IX_DL_BUF_NULL,\
	              c_offset, buf_size, buf_size, FREE_LIST_ID, 0, header_type, 0,\
				  output_port, 0, next_hop_id, $prpnd3,\
				  class_id, 0, sram_sig, SIG_NONE, 0, 7]	; write meta data

#ifdef COUNTERS
	ctx_arb[nxt_thrd_3, sram_sig, dram_sig, prf_sig, \
			byte_cnt_sig], and, defer[2]
#else
	ctx_arb[nxt_thrd_3, sram_sig, dram_sig, prf_sig], and,\
	       	defer[2]
#endif
		alu[drop_sop_hndl, --, B, $crx_in[1]]					; get drop dop handle
		alu[drop_eop_hndl, --, B, $crx_in[2]]					; get drop eop handle

	xbuf_free[$dl_meta_2]									; free the $dl_meta_2 

sep_cam_miss_seq_not_zero_phase_4#:
	// increment debug counter
	_csix_rx_debug_incr_counter[@csix_rx_num_seq_error]

	// send next thread signal 3 to the next thread

	_send_next_thrd_sig[nxt_thrd_3_gpr]

	_drop_current_pkt[drop_sop_hndl, drop_eop_hndl,\
					  lkup_key]								; drop the error packet

	//	get prefetch data handle from 
	//  prefetch buffer handle
		
	dl_buf_get_data[prf_data_hndl, $prf_hndl] 				; get data handle
	alu_shf[ele_addr, --, B, ele_num,<<16] 					; set element number
	msf[fast_wr, --, msf_rx_element_addr, ele_addr]			; free current element
	
	xbuf_free[$crx_in]										; free crx_in $xfers
	xbuf_free[$crx_out]										; free crx_out $xfers


	// Send current packet to the next micro block
#ifdef CELL_MODE
	_csix_rx_sink[dl_buf_handle, dl_buf_handle,\
	              IX_DL_BUF_NULL, class_id, \
	              output_port, buf_size, cell_cnt, $prpnd3, rx_event, \
				  scratch_sig, rtn_label, lkup_key]
#else
	_csix_rx_sink[dl_buf_handle, dl_buf_handle, \
	              IX_DL_BUF_NULL, class_id, \
	              output_port, buf_size, 0, $prpnd3, rx_event, \
				  scratch_sig, rtn_label, lkup_key]
#endif

.end														; sep cam miss proc end


sep_cam_hit#:

	alu[c_offset, c_offset, +, buf_offset]					; compute new offset

	xbuf_alloc[$dl_meta_3, 7, write]						; alloc $dl_meta_3

	dl_meta_write[$dl_meta_3, dl_buf_handle,IX_DL_BUF_NULL,\
	 			  c_offset,buf_size, buf_size, FREE_LIST_ID, 0, header_type, 0, \
				  output_port, 0, next_hop_id, $prpnd3, \
				  class_id, 0, sram_sig, SIG_NONE, 0, 7]	; write meta data

	ctx_arb[nxt_thrd_2]

sep_cam_hit_phase_3#:

	_send_next_thrd_sig[nxt_thrd_2_gpr]

	//	Check if expected cell sequence number is 0

	alu[seq_num, --, B, lm_cell_seq_num, >>16]

	//	If expected cell sequence number is not zero,
	//	goto sequence number error proc

	bne[sep_cam_hit_seq_not_zero#]

#ifdef COUNTERS
	ctx_arb[nxt_thrd_3, sram_sig, dram_sig, prf_sig, \
	        byte_cnt_sig], and
#else
	ctx_arb[nxt_thrd_3, sram_sig, dram_sig, prf_sig], and
#endif

sep_cam_hit_phase_3_end#:

	br[sep_common_phase_4#], defer[1]
		alu_shf[ele_addr, --, B, ele_num,<<16] 					; set element num gpr


sep_cam_hit_seq_not_zero#:

	alu[lm_cell_seq_num,--,B,0x0]								; set seq num 0

#ifdef COUNTERS
	ctx_arb[nxt_thrd_3, sram_sig, dram_sig, prf_sig,\
	        byte_cnt_sig], and,	defer[2]
#else
	ctx_arb[nxt_thrd_3, sram_sig, dram_sig, prf_sig], and, \
			defer[2]
#endif
		alu[drop_sop_hndl, --, B, lm_sop_buf_hndl]				; get drop sop hndl
		alu[drop_eop_hndl, --, B, lm_cur_buf_hndl]				; get drop eop hndl

	// increment debug counter
	_csix_rx_debug_incr_counter[@csix_rx_num_seq_error]

	_drop_current_pkt[drop_sop_hndl, drop_eop_hndl,lkup_key]	; drop error packet
	br[sep_common_phase_4#], defer[1]							; go to common point
		alu_shf[ele_addr, --, B, ele_num,<<16] 					; set element number

	xbuf_free[$dl_meta_3]										; free the $dl_meta_3

.end
#endm


/////////////////////////////////////////////////////////////////////////////
// _csix_rx_sop_proc
//	
//	Description:
//		This macro will handle the SOP only Cframe
//
//	Outputs:	
//		output:	None
//			ele_addr: modified
//			c_offset: modofied
//	Inputs:
//		input:
//			rtn_label : After the macro finishes, return to this label
//
//		CONSTANTS	:	None
//	
//
//	Size:	
/////////////////////////////////////////////////////////////////////////////
#macro _csix_rx_sop_proc[rtn_label]
.begin

.reg buf_offset												; buf_ofset
.reg data_len												; data length
.reg prf_gpr												; prefetch buffer gpr
.reg xpt_seq												; expected seq num
.reg lkup_key												; lookup key 
.reg lkup_result											; lookup result 

.reg dram_handle											; packet data handle
.reg drop_sop_hndl											; drop SOP handle
.reg drop_eop_hndl											; drop EOP handle

#ifdef	COUNTERS							
.reg $byte_cnt												; byte counter xfers
#define	byte_cnt_sig_1	msf_sig								; reuse msf_sig as byte 
#endif														; count sig

#ifdef	CELL_MODE
.reg cell_cnt												; cell count gpr
.reg rmdr 													; reminder gpr
#endif


	_get_rand_buf_offst[buf_offset]							; get buffer offset

	//	Using prefetch buffer to store the incoming SOP Cframe

	alu[dram_handle, buf_offset, +, prf_data_hndl]			; Get data handle

	//	The RBUF data start address = element_addr + 
	//  prepend header length 
	//	for SOP Cframe

	alu[ele_addr, ele_addr,  +, SOP_PRPEND_HDR]
	
	//	The RBUF data length = payload length - prepend header length

	alu[data_len, payload_len, -, SOP_PRPEND_HDR]

	//	move received cframe to data buffer

	_rbuf_to_dram[dram_handle, ele_addr, data_len,\
				  rbuf_base, dram_sig]

	alu[dram_handle, dram_handle,+, data_len]				; compute DRAM handle
	alu[data_len, data_len, -, c_offset]					; deduct offset
	alu[c_offset,c_offset,+,buf_offset]						; compute new offset
	alu[prf_gpr,--, B, $prf_hndl]							; cache pref_hndl
		
	// increment debug counter
	_csix_rx_debug_incr_counter[@csix_rx_num_buf_alloc]

	//	get new prefetch buffer

	dl_buf_alloc[$prf_hndl, BUF_FREE_LIST0, prf_sig,\
	             SIG_NONE]

	//	Compose CAM lookup key

	alu[lkup_key, class_id, or, in_bld_num, <<4] 

#ifdef COUNTERS
	alu[cnt_addr, --, B, lkup_key, <<4]						; get conter addr
	alu[cnt_addr, cnt_addr, +, cnt_base]					; add base
	sram[incr, --, cnt_addr, PKT_RECEIVED_CNT_IDX]			; increment pkt count
	alu[$byte_cnt, --, B, payload_len]						; get payload length
	sram[add, $byte_cnt, cnt_addr, BYTE_RECEIVED_CNT_IDX],\
	     sig_done[byte_cnt_sig_1]							; add byte count
#endif

#ifdef CELL_MODE
	alu[cell_cnt,--,B,0x0]
	alu[rmdr,--,B,0x0]
	_get_cell_cnt[cell_cnt, rmdr, data_len]
#endif

	//	Do CAM lookup & set up local memory address

	cam_lookup[lkup_result, lkup_key],lm_addr0[0]

	//	Check lookup result

	br!=byte[lkup_result, 1, 0x0, sop_cam_hit#]


sop_cam_miss#:

	// SOP only Cframe CAM miss processing start

.begin
	//	This is a CAM miss case; LRU LM CRX=> SRAM;	SRAM CRX=>$Xfers

.reg tmp_tag, tmp_entry, addr_in, addr_out

	xbuf_alloc[$crx_in, 3, read]							; alloc $crx_in
	xbuf_alloc[$crx_out, 8, write]  						; alloc $crx_out
	alu_shf[addr_in, crx_sram_gpr, or, lkup_key, <<5] 		; set CRX SRAM address  
	sram[read, $crx_in[0], addr_in, 0, 3], \
	     sig_done[sram_sig_rd]								; read CRX SRAM 

	//	Move LRU LM CRX data to the $crx_out xfers

	alu[$crx_out[0], --, B, *l$index0[0]]	
	alu[$crx_out[1], --, B, *l$index0[1]]
	alu[$crx_out[2], --, B, *l$index0[2]]
	alu[$crx_out[3], --, B, *l$index0[3]]
	alu[$crx_out[4], --, B, *l$index0[4]]
	alu[$crx_out[5], --, B, *l$index0[5]]
	alu[$crx_out[6], --, B, *l$index0[6]]
	alu[$crx_out[7], --, B, *l$index0[7]]

	alu[tmp_entry, 0xF,and, lkup_result, >>3]				; get CAM entry
	cam_read_tag[tmp_tag,tmp_entry]							; read CAM tag
	alu[addr_out,crx_sram_gpr, or, tmp_tag, <<5]			; get LRU CRX SRAM address
	sram[write, $crx_out[0], addr_out, 0, 8], \
		 sig_done[sram_sig_wr]								; sram write
	ctx_arb[nxt_thrd_2, sram_sig_rd, sram_sig_wr],defer[1]	; ctx swap 
		cam_write[tmp_entry, lkup_key, NORMAL_STATE]		; update CAM LRU entry

sop_cam_miss_phase_3#:

	//	move $crx_in CRX to LM

	alu[*l$index0[0],--,B, $crx_in[0]]	
	alu[*l$index0[1],--,B, $crx_in[1]]
	alu[*l$index0[2],--,B, $crx_in[2]]
	xbuf_free[$crx_in]										; free crx_in $xfers
	xbuf_free[$crx_out]										; free crx_out $xfers
.end														; sop_cam_miss end

sop_common_phase_3#:

	_send_next_thrd_sig[nxt_thrd_2_gpr]						; send next thread sig 2
	alu_shf[xpt_seq, --, B, lm_cell_seq_num, >>16]			; get seq number
	alu[drop_sop_hndl, --, B, lm_sop_buf_hndl]				; get drop_sop_hndl
	alu[drop_eop_hndl, --, B, lm_cur_buf_hndl]				; get drop_eop_hndl

	alu[data_len,0, +16,data_len]		 					; set cur buf size
	alu_shf[lm_cell_seq_num,data_len,OR,0x1,<<16]			; set seq num/pkt size
	alu[lm_sop_buf_hndl,--,B,prf_gpr]						; set sop handle 
	alu[lm_cur_buf_hndl,--,B,prf_gpr]						; set current buf hndle
	alu[lm_cur_dat_hndl,--,B,dram_handle]					; set cur data handle
	alu[lm_sop_buf_ofst,data_len,OR, c_offset, <<16]		; set buf size/offset
	ld_field_w_clr[lm_header_type, 0100, $prpnd2]    			; set header type
	ld_field[lm_output_port,0011,$prpnd2]				; set output port
	alu[lm_prv_buf_hndl,--,B,prf_gpr]						; set current buf hndle

#ifdef CELL_MODE
	alu[lm_cur_cell_cnt, lm_cur_cell_cnt,and~, 0x3F,<<24]	; clean out cell count	
	alu[lm_cur_cell_cnt, lm_cur_cell_cnt, or, cell_cnt,<<24]; set cell count
	alu[lm_remainder, lm_remainder,and~, 0x3F,<<24]			; clean out remainder
	alu[lm_remainder, lm_remainder, or, rmdr,<<24]			; set remainder
#endif

#ifdef COUNTERS
	ctx_arb[nxt_thrd_3, dram_sig, prf_sig, byte_cnt_sig_1],\
			and, defer[2]
#else
	ctx_arb[nxt_thrd_3, dram_sig, prf_sig], and, defer[2]
#endif	
	
		alu[lm_flow_color_id,--,B,$prpnd3]					; store flow id and color
		alu_shf[ele_addr, --, B, ele_num,<<16] 				; store element number

sop_common_phase_4#:

	// send next thread signal 3 to the next thread

	_send_next_thrd_sig[nxt_thrd_3_gpr]

	//	check if expected sequence number is 0

	alu_shf[--, xpt_seq, -, 0x0]
	//	if expect sequence number is zero, no drop pkt

	beq[sop_no_drop_pkt#]
	_drop_current_pkt[drop_sop_hndl, drop_eop_hndl, lkup_key]; drop the error packet

sop_no_drop_pkt#:

	//	get prefetch data handle from prefetch buffer handle	

	dl_buf_get_data[prf_data_hndl, $prf_hndl] 				; get data handle
	msf[fast_wr, --, msf_rx_element_addr, ele_addr]			; Free current element
	
	ctx_arb[rx_event],and,br[rtn_label]						; ctx swap

sop_cam_hit#:
	ctx_arb[nxt_thrd_2], br[sop_common_phase_3#]

.end														; csix_rx_sop_proc end
#endm														; this macro end


/////////////////////////////////////////////////////////////////////////////
// _csix_rx_eop_proc
//	
//	Description:
//		This macro will handle the EOP only Cframe case when the csix rx 
//		received a normal Cframe.
//
//	Outputs:	
//		output:	Some globale GPRs may be modified
//				ele_addr: used and modified
//	Inputs:
//		input:	
//			rtn_label : After the macro finishes, return to this label
//
//		CONSTANTS	:	None
//	
//
//	Size:	
/////////////////////////////////////////////////////////////////////////////
#macro _csix_rx_eop_proc[rtn_label]
.begin

.reg lkup_key												; lookup key 
.reg lkup_result											; look up result
.reg data_len												; Cframe data length
.reg buf_size												; buffer size
.reg seq_num												; sequence number
.reg expt_seq_num											; expect seq number
.reg pkt_size												; packet size
.reg buf_offset												; buffer offset
.reg buf_hndl												; tmp bufer handle
.reg buf_hndl_1												; tmp buffer handle 1
.reg flow_color_id											; flow and color information

.reg output_port											; used to store output port number
.reg header_type											; used to store header type

.reg dram_handle											; packet data handle
.reg drop_sop_hndl											; drop SOP handle
.reg drop_eop_hndl											; drop EOP handle
								
#ifdef	COUNTERS
.reg $byte_cnt												; byte counter xfers
#define	byte_cnt_sig_4	msf_sig								; reuse msf_sig as byte 
#endif														; count sig

#ifdef	CELL_MODE
.reg cell_cnt												; cell count gpr
.reg sop_cell_cnt											; SOP cell count
.reg rmdr 													; reminder gpr
#endif

	alu[lkup_key, class_id, or, in_bld_num, <<4] 			; get CAM lookup key

#ifdef COUNTERS
	alu[cnt_addr, --, B, lkup_key, <<4]						; get conter addr
	alu[cnt_addr, cnt_addr, +, cnt_base]					; add base
	alu[$byte_cnt, --, B, payload_len]						; get payload length
	sram[add, $byte_cnt, cnt_addr, BYTE_RECEIVED_CNT_IDX],\
	     sig_done[byte_cnt_sig_4]							; add byte count
#endif

	//	Do CAM lookup & set up local memory address

	cam_lookup[lkup_result, lkup_key],lm_addr0[0]

	//	Check lookup result

	br!=byte[lkup_result, 1, 0x0, eop_cam_hit#], defer[1]
		ld_field_w_clr[seq_num,0011,$prpnd0,>>8]			; get sequence num

eop_cam_miss#:

.begin

	//	This is a CAM miss case
	//  LRU LM CRX=> SRAM, SRAM CRX=>$Xfers

.reg tmp_tag, tmp_entry, addr_in, addr_out

	xbuf_alloc[$crx_in, 8, read]							; alloc $crx_in 
	xbuf_alloc[$crx_out, 8, write]  						; alloc $crx_out
	alu_shf[addr_in, crx_sram_gpr, or, lkup_key, <<5]		; set CRX SRAM address  
	
	// read CRX data form SRAM to $xfers

	sram[read, $crx_in[0], addr_in, 0, 8], \
	    sig_done[sram_sig_rd]	

	//	move LRU LM CRX data to the $crx_out xfers

	alu[$crx_out[0], --, B, *l$index0[0]]	
	alu[$crx_out[1], --, B, *l$index0[1]]
	alu[$crx_out[2], --, B, *l$index0[2]]
	alu[$crx_out[3], --, B, *l$index0[3]]
	alu[$crx_out[4], --, B, *l$index0[4]]
	alu[$crx_out[5], --, B, *l$index0[5]]
	alu[$crx_out[6], --, B, *l$index0[6]]
	alu[$crx_out[7], --, B, *l$index0[7]]

	alu[tmp_entry, 0xF,and, lkup_result, >>3]				; get CAM entry
	cam_read_tag[tmp_tag,tmp_entry]							; read CAM tag
	alu[addr_out,crx_sram_gpr, or, tmp_tag, <<5]			; get LRU CRX SRAM address
	sram[write, $crx_out[0], addr_out, 0, 8], \
	     sig_done[sram_sig_wr]
	ctx_arb[nxt_thrd_2, sram_sig_rd, sram_sig_wr], and, \
	       defer[1]
		cam_write[tmp_entry, lkup_key, NORMAL_STATE]		; update CAM LRU entry

eop_cam_miss_phase_3#:

	//	move $xfers CRX to LM

	alu[*l$index0[0],--,B, $crx_in[0]]	
	alu[*l$index0[1],--,B, $crx_in[1]]
	alu[*l$index0[2],--,B, $crx_in[2]]
	alu[*l$index0[3],--,B, $crx_in[3]]
	alu[*l$index0[4],--,B, $crx_in[4]]
	alu[*l$index0[5],--,B, $crx_in[5]]
	alu[*l$index0[6],--,B, $crx_in[6]]
	alu[*l$index0[7],--,B, $crx_in[7]]

	xbuf_free[$crx_in]										; free $crx_in
	xbuf_free[$crx_out]										; free $crx_out

eop_common_phase_3#:

	_send_next_thrd_sig[nxt_thrd_2_gpr]						; send next thread sig 2

	// prepare for dop backet
	// get drop SOP and drop EOP buffre handles

	alu[drop_sop_hndl, --, B, lm_sop_buf_hndl]
	alu[drop_eop_hndl, --, B, lm_cur_buf_hndl]

	//	check if expected seq num matches incoming seq num

	alu_shf[expt_seq_num,--, B, lm_cell_seq_num, >>16]
	alu_shf[--, seq_num, -, expt_seq_num]

	// If expected sequence number does not match 
	// incoming seq num goto sequence number missmatch proc

	bne[eop_seq_mismatch#]

	alu[data_len, payload_len, -, NO_SOP_PRPEND_HDR]		; get cframe length
	alu[pkt_size, 0, +16, lm_pkt_size]						; get pkt size
	alu[pkt_size, pkt_size, +, data_len]					; get new pkt size

	// get buffer data handle 

	alu[dram_handle, --, B, lm_cur_dat_hndl]

	//	The RBUF data start address = element_addr + 
	//  no sop prepend header length for EOP Cframe

	alu[ele_addr, ele_addr, +, NO_SOP_PRPEND_HDR]
	
	//	move received cframe to data buffer

	_rbuf_to_dram[dram_handle, ele_addr, data_len, \
	              rbuf_base, dram_sig]

#ifdef CELL_MODE
	alu[cell_cnt,0x3F,and,lm_cur_cell_cnt,>>24]				; get cell count 
	alu[rmdr, 0x3F,and,lm_remainder,>>24]					; get remainder
	_get_cell_cnt[cell_cnt,rmdr,data_len]					; get new cell count
#endif

	//	prepare meta data

	alu[buf_size, 0, +16, lm_cur_buf_size]					; get current buf size
	alu[buf_size, buf_size, +, data_len]					; compute buf size
	alu[buf_hndl, drop_sop_hndl, and~, 0xFF, <<24]			; clean out drop sop hndl
	alu[buf_hndl_1, drop_eop_hndl, and~, 0xFF, <<24]		; clean out drop eop hndl
	alu[--, buf_hndl, -, buf_hndl_1] 						; check if SOP buffer
	beq[eop_and_sop_buf#], defer[2]
	ld_field_w_clr[output_port, 0011, lm_output_port]	; get output port
	ld_field_w_clr[header_type, 0001, lm_header_type, >>16]	; get header type

#ifdef CELL_MODE
	alu[sop_cell_cnt,0x3F,and, lm_sop_cell_cnt,>>24]		; get SOP cell count
#endif

eop_not_sop_buf#:

	xbuf_alloc[$dl_meta, 8, write]							; alloc $dl_meta
	dl_meta_write[$dl_meta, drop_sop_hndl, 0, 0, 0, \
	              pkt_size, FREE_LIST_ID, 0, header_type, 0, output_port,\
				  0, next_hop_id, lm_flow_color_id, \
				  class_id, 0, sram_sig, SIG_NONE, 2, 5]	; rewrite SOP meta data

	// store flow and color into a register
	move(flow_color_id, lm_flow_color_id)

	xbuf_alloc[$dl_meta_1, 2, write]						; alloc $dl_meta_1
	dl_meta_write[$dl_meta_1, drop_eop_hndl, \
	              IX_DL_BUF_NULL, 0, buf_size, 0, \
				  FREE_LIST_ID, 0, 0, 0, 0, 0,0,0,0, \
				  0, sram_sig_wr, SIG_NONE, 0, 2] 			; write EOP meta data

#ifdef CELL_MODE
	xbuf_alloc[$dl_meta_2, 1, write]						; alloc $dl_meta_2

	alu[buf_hndl,--,B,lm_prv_buf_hndl]						; get previous buf hndl
	alu[drop_eop_hndl,lm_cur_buf_hndl,and~,0x7F,<<24]		; clean out SOP bit/cell cnt
	alu[drop_eop_hndl,drop_eop_hndl,or,cell_cnt,<<24]		; set up cell count
	dl_meta_write[$dl_meta_2, buf_hndl, \
	              drop_eop_hndl, 0, 0, 0, \
				  FREE_LIST_ID, 0, 0, 0, 0, 0,0,0,0, \
				  0, sram_sig_rd, SIG_NONE, 0, 1] 			; update current buffer 
				  											; cell count 
#endif

#ifdef COUNTERS
#ifdef CELL_MODE
	ctx_arb[nxt_thrd_3, sram_sig, sram_sig_wr,sram_sig_rd,\
		    dram_sig, byte_cnt_sig_4], and, defer[2]			
#else
	ctx_arb[nxt_thrd_3, sram_sig, sram_sig_wr,\
		    dram_sig, byte_cnt_sig_4], and, defer[2]
#endif
#else
#ifdef CELL_MODE
	ctx_arb[nxt_thrd_3, sram_sig, sram_sig_wr,sram_sig_rd,\
		    dram_sig], and, defer[2]			
#else
	ctx_arb[nxt_thrd_3, sram_sig, sram_sig_wr,\
		    dram_sig], and, defer[2]
#endif
#endif
		alu_shf[ele_addr, --, B, ele_num,<<16]				; set element number
		alu[lm_cell_seq_num, --, B,0x0]						; reset sequence number

	xbuf_free[$dl_meta]										; free $dl_meta
	xbuf_free[$dl_meta_1]									; free $dl_meta_1

#ifdef CELL_MODE
	xbuf_free[$dl_meta_2]									; free $dl_meta_2
#endif

eop_phase_4#:

	// send next thread signal 3 to the next thread

	_send_next_thrd_sig[nxt_thrd_3_gpr]

	msf[fast_wr, --, msf_rx_element_addr, ele_addr]			; free RBUF element
	
	// send the packet to next microblock

	alu[buf_hndl,drop_sop_hndl,and~,0xFF, <<24]				; clean out 8 MSB
	alu[buf_hndl_1,drop_eop_hndl,and~,0xFF, <<24]			; clean out 8 MSB
	alu[--,buf_hndl, -, buf_hndl_1]							; check one buffer case
	bne[eop_phase_4_more_buf#],defer[2]
		alu[drop_sop_hndl,drop_sop_hndl,and~,0x1, <<31]		; clean out EOP bit
		alu[drop_eop_hndl,drop_eop_hndl,and~,0x1, <<30]		; clean out SOP bit

	// here is one buffer case
	alu[drop_sop_hndl, drop_sop_hndl, or, 1, <<31]			; Set EOP bit
#ifdef	CELL_MODE
	_csix_rx_sink[drop_sop_hndl,drop_eop_hndl,\
	              IX_DL_BUF_NULL, class_id, \
				  output_port, pkt_size, sop_cell_cnt, flow_color_id, rx_event,\
				  scratch_sig, rtn_label,lkup_key]
#else
	_csix_rx_sink[drop_sop_hndl,drop_eop_hndl,\
	              IX_DL_BUF_NULL, class_id, \
				  output_port, pkt_size, 0, flow_color_id, rx_event,\
				  scratch_sig, rtn_label,lkup_key]
#endif

eop_phase_4_more_buf#:

	// here is more than one buffer case
	alu[drop_eop_hndl,drop_eop_hndl,or,0x1,<<31]			; set EOP bit in eop hndl	
#ifdef	CELL_MODE
	alu[drop_eop_hndl,drop_eop_hndl,or,cell_cnt,<<24]		; set cell cnt in eop hndl
	_csix_rx_sink[drop_sop_hndl,drop_eop_hndl,\
	              drop_eop_hndl, class_id, \
				  output_port, pkt_size, sop_cell_cnt, flow_color_id, rx_event,\
				  scratch_sig, rtn_label,lkup_key]
#else
	_csix_rx_sink[drop_sop_hndl,drop_eop_hndl,\
	              drop_eop_hndl, class_id, \
				  output_port, pkt_size, 0, flow_color_id, rx_event, \
				  scratch_sig, rtn_label, lkup_key]

#endif

eop_and_sop_buf#:

	xbuf_alloc[$dl_meta_3, 7, write]						; alloc $dl_meta_2
	alu[buf_offset,--,B,lm_sop_buf_ofst,>>16]				; get buf offset

#ifdef	CELL_MODE
	alu[sop_cell_cnt,--,B, cell_cnt]						; get SOP cell count
#endif

	dl_meta_write[$dl_meta_3, drop_sop_hndl, \
				  IX_DL_BUF_NULL, buf_offset,\
				  buf_size, pkt_size, FREE_LIST_ID, 0, header_type, 0,\
				  output_port, 0, next_hop_id, $prpnd3,\
				  class_id, 0, sram_sig, SIG_NONE, 0, 7]	; write meta data
	
	// move flow id and color to a register 
	move(flow_color_id, $prpnd3)

#ifdef COUNTERS
	ctx_arb[nxt_thrd_3, sram_sig, dram_sig, byte_cnt_sig_4],\
	        and, defer[2], br[eop_phase_4#]
#else
	ctx_arb[nxt_thrd_3, sram_sig, dram_sig], and, \
		    defer[2],br[eop_phase_4#]
#endif
		alu_shf[ele_addr, --, B, ele_num,<<16] 				; set element number
		alu[lm_cell_seq_num, --, B,0x0]						; reset sequence number

	xbuf_free[$dl_meta_3]									; free $dl_meta_2

eop_seq_mismatch#:

	
	// the original sequence number is 
	// stored in expt_seq_num

	alu[lm_cell_seq_num,--,B,0x0]							; update LM CRX data

#ifdef COUNTERS
	ctx_arb[nxt_thrd_3, byte_cnt_sig_4]
#else
	ctx_arb[nxt_thrd_3]
#endif

eop_seq_mismatch_phase_4#:

	// send next thread signal 3 to the next thread

	_send_next_thrd_sig[nxt_thrd_3_gpr]

	alu[--,expt_seq_num,-,0x0]								; check seq number is zero
	beq[eop_seq_missmatch_phase_4_no_drop#] 				; if zero, no packet drop

	// increment debug counter
	_csix_rx_debug_incr_counter[@csix_rx_num_seq_error]

	_drop_current_pkt[drop_sop_hndl, drop_eop_hndl,\
					  lkup_key]								; drop partial pkt

eop_seq_missmatch_phase_4_no_drop#:

	alu_shf[ele_addr, --, B, ele_num,<<16]					; set element number gpr
	msf[fast_wr, --, msf_rx_element_addr, ele_addr]			; free RBUF element
	
	ctx_arb[rx_event],and,br[rtn_label]

.end														; eop cam miss proc end


eop_cam_hit#:
	ctx_arb[nxt_thrd_2], br[eop_common_phase_3#]

.end
#endm


/////////////////////////////////////////////////////////////////////////////
// _csix_rx_mop_proc
//	
//	Description:
//		This macro will handle the MOP only Cframe case when the csix rx 
//		received a normal Cframe.
//
//	Outputs:	
//		output:	None
//		
//	Inputs:
//		input:
//			rtn_label : After the macro finishes, return to this label
//
//		CONSTANTS	:	None
//	
//
//	Size:	
/////////////////////////////////////////////////////////////////////////////
#macro _csix_rx_mop_proc[rtn_label]
.begin

.reg lkup_key												; lookup key 
.reg lkup_result											; lookup result
.reg buf_hndl												; buffer handle
.reg pr_buf_hndl											; previous buf handle 
.reg data_len												; buffer data length
.reg pkt_size												; packet size
.reg seq_num												; sequence number
.reg offset													; buffer offset
.reg buf_size												; buffer size
.reg buf_size_1												; buffer size 1
.reg xpt_seq												; expected sequence number

.reg dram_handle											; packet data handle
.reg drop_sop_hndl											; drop SOP handle
.reg drop_eop_hndl											; drop EOP handle

#ifdef	COUNTERS
.reg $byte_cnt												; byte counter xfers
#define	byte_cnt_sig_3	msf_sig								; reuse msf_sig as byte 
#endif														; count sig

#ifdef	CELL_MODE
.reg cell_cnt												; cell count gpr
.reg rmdr 													; reminder gpr
#endif


	alu[lkup_key, class_id, or, in_bld_num, <<4] 			; get CAM lookup key

#ifdef COUNTERS
	alu[cnt_addr, --, B, lkup_key, <<4]						; get conter addr
	alu[cnt_addr, cnt_addr, +, cnt_base]					; add base
	alu[$byte_cnt, --, B, payload_len]						; get payload length
	sram[add, $byte_cnt, cnt_addr, BYTE_RECEIVED_CNT_IDX],\
	     sig_done[byte_cnt_sig_3]							; add byte count
#endif

	//	Do CAM lookup & set up local memory address

	cam_lookup[lkup_result, lkup_key],lm_addr0[0]

	//	Check lookup result

	br!=byte[lkup_result, 1, 0x0, mop_cam_hit#],defer[2]	; if hit go to hit proc
		ld_field_w_clr[seq_num,0011,$prpnd0,>>8]			; get cell sequence num
		alu[offset, --, B, 0x0]								; assume this is not a sop buffer
		 
mop_cam_miss#:

.begin
	//	This is a CAM miss case; LRU LM CRX=> SRAM;	SRAM CRX=>$Xfers

.reg tmp_tag, tmp_entry, addr_in, addr_out

	xbuf_alloc[$crx_in, 8, read]							; aloc 7 crx_in xbuffers
	xbuf_alloc[$crx_out, 8, write]  						; alloc 7 crx_out xbuffers

	alu_shf[addr_in, crx_sram_gpr, or, lkup_key, <<5] 		; set CRX SRAM address  

	// Read SRAM to the $crx_in group

	sram[read, $crx_in[0], addr_in, 0, 8], sig_done[sram_sig_rd]

	//	move LRU LM CRX data to the $crx_out xfers

	alu[$crx_out[0], --, B, *l$index0[0]]
	alu[$crx_out[1], --, B, *l$index0[1]]
	alu[$crx_out[2], --, B, *l$index0[2]]
	alu[$crx_out[3], --, B, *l$index0[3]]
	alu[$crx_out[4], --, B, *l$index0[4]]
	alu[$crx_out[5], --, B, *l$index0[5]]
	alu[$crx_out[6], --, B, *l$index0[6]]
	alu[$crx_out[7], --, B, *l$index0[7]]

	alu[tmp_entry, 0xF,and, lkup_result, >>3]				; get CAM entry
	cam_read_tag[tmp_tag,tmp_entry]							; read CAM tag
	alu[addr_out,crx_sram_gpr, or, tmp_tag, <<5]			; get LRU CRX SRAM address
	sram[write, $crx_out[0], addr_out, 0, 8], \
		 sig_done[sram_sig_wr]


	ctx_arb[nxt_thrd_2, sram_sig_rd, sram_sig_wr], and, \
			defer[1]
		cam_write[tmp_entry, lkup_key, NORMAL_STATE]		; update CAM LRU entry

mop_cam_miss_phase_3#:

	//	move $xfers CRX to LM

	alu[*l$index0[0],--,B, $crx_in[0]]	
	alu[*l$index0[1],--,B, $crx_in[1]]
	alu[*l$index0[2],--,B, $crx_in[2]]
	alu[*l$index0[3],--,B, $crx_in[3]]
	alu[*l$index0[4],--,B, $crx_in[4]]
	alu[*l$index0[5],--,B, $crx_in[5]]
	alu[*l$index0[6],--,B, $crx_in[6]]
	alu[*l$index0[7],--,B, $crx_in[7]]

	xbuf_free[$crx_in]										; free crx_in $xfers
	xbuf_free[$crx_out]										; free crx_out $xfers

.end														; mop_cam_miss done 

mop_common_phase_3#:

	_send_next_thrd_sig[nxt_thrd_2_gpr]						; send next thread sig 2
	
	//	check if expected seq num matches incoming seq num

	ld_field_w_clr[xpt_seq,0011,lm_cell_seq_num, >>16]
	alu[--,seq_num,-,xpt_seq]

	//	if expected cell sequence number does not match 
	//  incoming seq number goto sequence number missmatch proc

	bne[mop_seq_mismatch#]

	alu[data_len, payload_len, -, NO_SOP_PRPEND_HDR]		; get cframe length
	alu[dram_handle, --, B, lm_cur_dat_hndl]				; get data handle

	//	The RBUF data start address = element_addr + 
	//  no sop prepend header 
	//  length for a mop Cframe

	alu[ele_addr, ele_addr,  +, NO_SOP_PRPEND_HDR]
	
	//	move received cframe to data buffer

	_rbuf_to_dram[dram_handle, ele_addr, data_len, \
	              rbuf_base, dram_sig]

#ifdef CELL_MODE
	alu[cell_cnt,0x3F,and,lm_cur_cell_cnt,>>24]				; get cell count 
	alu[rmdr, 0x3F,and,lm_remainder,>>24]					; get remainder
	_get_cell_cnt[cell_cnt,rmdr,data_len]					; get new cell count
	alu[lm_cur_cell_cnt,lm_cur_cell_cnt,and~,0x3F,<<24]		; clean out cell count	
	alu[lm_cur_cell_cnt,lm_cur_cell_cnt,or, cell_cnt,<<24]	; write back cell count
	alu[lm_remainder,lm_remainder,and~,0x3F,<<24]			; clean out remainder	
	alu[lm_remainder,lm_remainder, or, rmdr,<<24]			; write back new remainder
#endif

	alu[buf_hndl,--,B,lm_sop_buf_hndl]						; get SOP buffer handle
	alu[buf_hndl,buf_hndl,and~,0xFF, <<24]					; clean out cell count
	alu[pr_buf_hndl,--,B,lm_cur_buf_hndl]					; get currut buffer handle
	alu[pr_buf_hndl,pr_buf_hndl,and~,0xFF,<<24]				; clean out cell count
	alu[--,buf_hndl,-,pr_buf_hndl]							; check if buf is SOP
	bne[mop_not_sop_buf#], defer[3]
		alu[buf_size,0, +16,lm_cur_buf_size]				; get buffer size	
		alu[buf_size,buf_size,+,data_len]					; compute new buf size
		alu[buf_size_1, --, B, buf_size]    				; store new buffer size

mop_sop_buffer#:
	alu[offset,--,B,lm_sop_buf_ofst, >>16]					; get SOP buffer ofset
	alu[buf_size_1, buf_size_1,+,offset]					; add SOP offset

mop_not_sop_buf#:

	.reg buf_max

	// compute next possible required buffer szie

	alu[buf_size_1,buf_size_1,+,RBUF_SIZE]
	move[buf_max,IX_BUF_MAX]  
	alu[--, buf_size_1,-, buf_max]							; check buf szie 
	bgt[mop_buf_full#], defer[3]							; goto buf full proc
		alu[xpt_seq,xpt_seq,+, 0x1]							; compute new seq num
		alu[pkt_size,0, +16,lm_pkt_size]					; get current pkt size
		alu[pkt_size,pkt_size,+, data_len]					; compute new pkt size

mop_buf_not_full#:

	ld_field[lm_cur_buf_size,0011,buf_size,<<0]				; set new buffer size
	alu[lm_pkt_size,pkt_size,OR,xpt_seq,<<16]				; set pkt size & seq num

#ifdef COUNTERS
	ctx_arb[nxt_thrd_3, dram_sig, byte_cnt_sig_3],\
	        defer[2]										; ctx swap
#else
	ctx_arb[nxt_thrd_3, dram_sig],defer[2]					; ctx swap
#endif
		alu[lm_cur_dat_hndl,dram_handle,+,data_len]			; update data hndle
		alu_shf[ele_addr, --, B, ele_num,<<16]				; set element number

mop_buf_not_full_phase_4#:

	// send next thread signal 3 to the next thread

	_send_next_thrd_sig[nxt_thrd_3_gpr]
	msf[fast_wr, --, msf_rx_element_addr, ele_addr]			; free current element
	
	ctx_arb[rx_event],br[rtn_label]

mop_buf_full#:

#ifdef CELL_MODE
#ifdef ATM_TX
	// For the cell mode and atm tx in the pipe line, the cell count
	// is defined as the number of the cell ending in the buffer. So
	// the reminider should be checked to see if there is a partial cell
	// is counted when the buffer is full.
	alu[rmdr, 0x3F, and, lm_remainder, >>24]				; Get the rmdr.
	alu[--, rmdr, -, cell_size] 							; Check rmdr= cell_size?
	beq[mop_buf_full_rmdr_equal_cell_size#]					; Go to rmdr == cell_size
	alu[--, rmdr, -, 0x0] 									; Check rmdr==0?
	br=0[mop_buf_full_no_rmdr#]								; No remainder.
	alu[cell_cnt, --, B, 1,	<<24]							; Get cell count.
	alu[lm_cur_cell_cnt, lm_cur_cell_cnt, -, cell_cnt]		; Cell count decrement.
	br[mop_buf_full_cell_adj_end#]							; Cell size adjusment done

mop_buf_full_rmdr_equal_cell_size#:
	alu[lm_remainder,lm_remainder, and~, 0x3F,\
			 <<24]  										; Clean out remainder.


mop_buf_full_cell_adj_end#:
#endif
#endif

mop_buf_full_no_rmdr#:			
	alu[dl_buf_handle,--,B,0x0] 							; clean out dl_buf_handle
	alu[pr_buf_hndl,--,B,0x0]  								; clean out pr_buf_hndl
	ld_field[dl_buf_handle,0111,lm_cur_buf_hndl,0]			; get current buf handle
	ld_field[pr_buf_hndl,0111,lm_prv_buf_hndl,0]  			; get prv buf hndl
	alu[--, dl_buf_handle, -, pr_buf_hndl] 					; camprae cur and prv hndl
	beq[mop_buf_full_no_eop_reset#]

#ifdef CELL_MODE
	alu[dl_buf_handle,--,B,lm_cur_buf_hndl]					; reload cur buf handle
															; with cell count in it
	alu_shf[dl_buf_handle,dl_buf_handle,and~,0x3,<<30]		; clean out SOP/EOP bits
#endif

	xbuf_alloc[$dl_meta, 1, write]							; alloc $dl_meta
	dl_meta_write[$dl_meta, pr_buf_hndl, dl_buf_handle, \
	              0, 0, 0, FREE_LIST_ID, 0, 0, 0, 0, \
				  0, 0, 0, 0, 0, sram_sig_wr, SIG_NONE,\
				  0, 1]

	ld_field[lm_prv_buf_hndl,0111,dl_buf_handle,0]  		; update prv buf hndl
	alu_shf[buf_hndl,$prf_hndl, and~, 0x1, <<30]			; clean out SOP

	// write meta data -- next pointer, 
	// offset and buffer_next 
	xbuf_alloc[$dl_meta1, 7, write]							; alloc $dl_meta
	dl_meta_write[$dl_meta1, dl_buf_handle, buf_hndl, \
	              offset, buf_size, 0, FREE_LIST_ID, 0, 0, 0, 0, \
				  0, 0, 0, 0, 0, sram_sig, SIG_NONE, 0, 2]

	ld_field[lm_cur_buf_size,0011,0x0,0]					; reset current buffer size
	alu[lm_pkt_size,pkt_size,or,xpt_seq, <<16]				; set pkt size & seq num
	alu[lm_cur_buf_hndl,--,B,$prf_hndl]  					; set new buffer handle
	alu[lm_cur_dat_hndl,--,B,prf_data_hndl] 				; set new data handle

#ifdef CELL_MODE
	alu[lm_cur_cell_cnt,lm_cur_cell_cnt,and~,0x3F,<<24]		; clean out cur cell cnt

#ifndef ATM_TX
	alu[lm_remainder,lm_remainder, and~, 0x3F,\
			 <<24]  										; Clean out remainder
			 												; if no TAM_TX.
#endif

#endif

	// increment debug counter
	_csix_rx_debug_incr_counter[@csix_rx_num_buf_alloc]

	//	get new prefetch buffer

	dl_buf_alloc[$prf_hndl, BUF_FREE_LIST0, prf_sig,\
				 SIG_NONE]

#ifdef COUNTERS
	ctx_arb[nxt_thrd_3, dram_sig, sram_sig, prf_sig,\
			sram_sig_wr, byte_cnt_sig_3],	defer[1]
#else
	ctx_arb[nxt_thrd_3, dram_sig, sram_sig, sram_sig_wr,\
			prf_sig], defer[1]
#endif
		alu_shf[ele_addr, --, B, ele_num,<<16] 				; set element number	
	xbuf_free[$dl_meta]										; free $dl_meta
	xbuf_free[$dl_meta1]									; free $dl_meta1
	br[mop_buf_full_phase_4#]								
	
mop_buf_full_no_eop_reset#:
	ld_field[lm_prv_buf_hndl,0111,dl_buf_handle,0]  		; set previous buf hndl
	alu_shf[buf_hndl,$prf_hndl, and~, 0x1, <<30]			; clean out SOP bit

	// write meta data -- next pointer, 
	// offset and buffer_next 
	xbuf_alloc[$dl_meta2, 7, write]							; alloc $dl_meta
	dl_meta_write[$dl_meta2, dl_buf_handle, buf_hndl, \
	              offset, buf_size, 0, FREE_LIST_ID, 0, 0, 0, 0, \
				  0, 0, 0, 0, 0, sram_sig, SIG_NONE, 0, 2]

#ifdef CELL_MODE
	alu[cell_cnt,0x3F,and,lm_cur_cell_cnt,>>24]				; get current cell count
	alu[lm_sop_cell_cnt,lm_sop_cell_cnt,and~, 0x3F,\
		<<24]  												; clean out cell count
	alu[lm_sop_cell_cnt,lm_sop_cell_cnt,or, cell_cnt,\
		<<24]  												; set SOP cell count

#ifndef ATM_TX
	alu[lm_remainder,lm_remainder,and~, 0x3F,\
		<<24]  												; Clean out remainder.
#endif

	alu[lm_cur_cell_cnt,lm_cur_cell_cnt,and~,0x3F,<<24]		; clean out cur cell cnt
#endif

	ld_field[lm_cur_buf_size,0011,0x0,0]					; reset cur buf size
	alu[lm_pkt_size,pkt_size,or,xpt_seq, <<16]				; set pkt size & seq num
	alu[lm_cur_buf_hndl,--,B,$prf_hndl]  					; set new buffer handle
	alu[lm_cur_dat_hndl,--,B,prf_data_hndl] 				; set new data handle

	// increment debug counter
	_csix_rx_debug_incr_counter[@csix_rx_num_buf_alloc]

	//	get new prefetch buffer

	dl_buf_alloc[$prf_hndl, BUF_FREE_LIST0, prf_sig,\
				 SIG_NONE]

#ifdef COUNTERS
	ctx_arb[nxt_thrd_3, dram_sig, sram_sig, prf_sig,\
			byte_cnt_sig_3],	defer[1]
#else
	ctx_arb[nxt_thrd_3, dram_sig, sram_sig, prf_sig],\
			defer[1]
#endif
		alu_shf[ele_addr, --, B, ele_num,<<16] 				; set element number
	xbuf_free[$dl_meta2]									; free $dl_meta1

mop_buf_full_phase_4#:

	// send next thread signal 3 to the next thread

	_send_next_thrd_sig[nxt_thrd_3_gpr]

	//	get prefetch data handle from prefetch
	//  buffer handle	

	dl_buf_get_data[prf_data_hndl, $prf_hndl] 				; get data handle
	msf[fast_wr, --, msf_rx_element_addr, ele_addr]			; free current element
	
	ctx_arb[rx_event],and,br[rtn_label]


mop_seq_mismatch#:

	alu[lm_cell_seq_num,--,B,0x0]							; set sequence num 0

#ifdef COUNTERS
	ctx_arb[nxt_thrd_3, byte_cnt_sig_3], and, defer[2]		; ctx swap
#else
	ctx_arb[nxt_thrd_3], and, defer[2]						; ctx swap
#endif
		alu[drop_sop_hndl,--,B,lm_sop_buf_hndl]				; set drop SOP buf hndl
		alu[drop_eop_hndl,--,B,lm_cur_buf_hndl]				; set drop EOP buf hndl

mop_seq_mismatch_phase_4#:

	// send next thread signal 3 to the next thread

	_send_next_thrd_sig[nxt_thrd_3_gpr]

	alu[--,xpt_seq,-,0x0]									; check expect seq num
	beq[mop_seq_mismatch_phase_4_no_drop#] 					; if 0, no packet drop

	// increment debug counter
	_csix_rx_debug_incr_counter[@csix_rx_num_seq_error]

	_drop_current_pkt[drop_sop_hndl, drop_eop_hndl,\
					  lkup_key]								; drop the error packet

mop_seq_mismatch_phase_4_no_drop#:

	alu_shf[ele_addr, --, B, ele_num,<<16] 					; set element number gpr
	msf[fast_wr, --, msf_rx_element_addr, ele_addr]			; free current element
	
	ctx_arb[rx_event],and,br[rtn_label]						; ctx swap

mop_cam_hit#:
	ctx_arb[nxt_thrd_2], br[mop_common_phase_3#]
	
.end
#endm


/////////////////////////////////////////////////////////////////////////////
// _csix_rx_err_proc
//	
//	Description:
//		This macro will handle the error cases when the csix rx received a error
//		Cframe.
//		$rsw_1 is used to decide what error happens in the Cframe
//
//	Outputs:	
//		output:	Some global GPRs may be modified
//				ele_addr modified
//		
//	Inputs:
//		input:
//			rtn_label : After the macro finishes, return to this label
//
//		CONSTANTS	:	None
//
//	Size:	
/////////////////////////////////////////////////////////////////////////////
#macro	_csix_rx_err_proc[rtn_label]
.begin

.reg lkup_key												; lookup key 
.reg lkup_result											; look up result
.reg seq_num												; cell sequence number

.reg drop_sop_hndl											; drop SOP handle
.reg drop_eop_hndl											; drop EOP handle

err_phse_1#:

	//	check if there is null error

	alu[--,$rsw_1, AND, 1, <<9]

	//	null error happens, jump to the null error processing	                                      

	bne[null_err_proc#]

	// Increment error cells received counter
	_csix_rx_debug_incr_counter(@csix_rx_num_err)
	
	//	Read prepend header

	msf[read,$prpnd0, rbuf_base, ele_addr, 4], \
		sig_done[msf_sig]

	//	Wait MSF read finish and nxt_thrd_1 signal

	ctx_arb[nxt_thrd_1, msf_sig],and,defer[1]
		_get_ext_class[ext_class,$rsw_2]					; get class ID


err_phase_2#:

	_send_next_thrd_sig[nxt_thrd_1_gpr]

	; Get class id.
	#ifdef USE_EXT_CLASS_ID
	; Get class id from extension class id (present in csix extension header)
	ld_field_w_clr[class_id, 0011, ext_class]
	#else
	; Get class id from qos class id (present in csix prepend header)
	ld_field_w_clr[class_id, 0011, $prpnd1]
	#endif

	; We have used $rsw_1 and $rsw_2 for the current c-frame.
	; So we can add the thread to the receive freelist.
	msf[fast_wr, --, msf_freelist0_addr, flst_reg]

	alu[in_bld_num,0x3F,and,$prpnd0]						; get ingress blade number
	alu[lkup_key, class_id, or, in_bld_num, <<4] 			; get CAM lookup key

	//	do CAM lookup & set up local memory address

	cam_lookup[lkup_result, lkup_key],lm_addr0[0]

	//	Check lookup result
	br!=byte[lkup_result, 1, 0x0, err_cam_hit#]				; if hit go to hit proc
	
err_cam_miss#:
.begin

	// this is a CAM miss case
	// LRU LM CRX=> SRAM; SRAM CRX=>$Xfers

.reg tmp_tag, tmp_entry, addr_in, addr_out

	xbuf_alloc[$crx_in, 3, read]							; aloc 3 crx_in xbuffers
	xbuf_alloc[$crx_out, 8, write]  						; alloc 7 crx_out xbuffers

	alu_shf[addr_in, crx_sram_gpr, or, lkup_key, <<5] 		; set CRX SRAM address  

	sram[read, $crx_in[0], addr_in, 0, 3], \
		 sig_done[sram_sig_rd]

	//	Move LRU LM CRX data to the $crx_out xfers

	alu[$crx_out[0], --, B, *l$index0[0]]	
	alu[$crx_out[1], --, B, *l$index0[1]]
	alu[$crx_out[2], --, B, *l$index0[2]]
	alu[$crx_out[3], --, B, *l$index0[3]]
	alu[$crx_out[4], --, B, *l$index0[4]]
	alu[$crx_out[5], --, B, *l$index0[5]]
	alu[$crx_out[6], --, B, *l$index0[6]]
	alu[$crx_out[7], --, B, *l$index0[7]]

	alu[tmp_entry, 0xF,and, lkup_result, >>3]				; get CAM entry
	cam_read_tag[tmp_tag,tmp_entry]							; read CAM tag
	alu[addr_out,crx_sram_gpr, or, tmp_tag, <<5]			; get LRU CRX SRAM address
	sram[write, $crx_out[0], addr_out, 0, 8], \
	     sig_done[sram_sig_wr]
	cam_write[tmp_entry, lkup_key, NORMAL_STATE]			; update CAM LRU entry
	ctx_arb[nxt_thrd_2, sram_sig_rd, sram_sig_wr],and,\
	        defer[1]
		cam_write[tmp_entry, lkup_key, NORMAL_STATE]		; update CAM LRU entry

err_cam_miss_phase_3#:
	//	move $xfers CRX to LM

	alu[*l$index0[0],--,B, $crx_in[0]]	
	alu[*l$index0[1],--,B, $crx_in[1]]
	alu[*l$index0[2],--,B, $crx_in[2]]

	xbuf_free[$crx_in]										; free $crx_in
	xbuf_free[$crx_out]										; free $crx_out


err_common_phase_3#:
	_send_next_thrd_sig[nxt_thrd_2_gpr]

	alu[seq_num, --, B, lm_cell_seq_num, >>16]				; get sequence number 
	alu[lm_cell_seq_num,--,B,0x0]							; seq number=0x0

	ctx_arb[nxt_thrd_3],defer[2]
		alu[drop_sop_hndl,--,B,lm_sop_buf_hndl]				; set drop SOP buf hndl
		alu[drop_eop_hndl,--,B,lm_cur_buf_hndl]				; set drop EOP buf hndl

err_common_phase_4#:
	_send_next_thrd_sig[nxt_thrd_3_gpr]

	alu[--,seq_num,-,0x0]									; check current seq num
	beq[err_common_phase_4_no_drop#] 						; if 0, no packet drop
	_drop_current_pkt[drop_sop_hndl, drop_eop_hndl,\
					  lkup_key]								; drop the error packet

err_common_phase_4_no_drop#:

	alu_shf[ele_addr, --, B, ele_num,<<16] 					; set element number
	msf[fast_wr, --, msf_rx_element_addr, ele_addr]			; free current element
	
	ctx_arb[rx_event], br[rtn_label]

.end 														;err CAM lookup miss done

err_cam_hit#:

	ctx_arb[nxt_thrd_2], br[err_common_phase_3#]



null_err_proc#: 											; null cframe processing
	ctx_arb[nxt_thrd_1]

null_err_proc_phase_2#:
	_send_next_thrd_sig[nxt_thrd_1_gpr]

	; We have used $rsw_1 and $rsw_2 for the current c-frame.
	; So we can add the thread to the receive freelist.
	msf[fast_wr, --, msf_freelist0_addr, flst_reg]

	ctx_arb[nxt_thrd_2]

null_err_proc_phase_3#:
	_send_next_thrd_sig[nxt_thrd_2_gpr]
	ctx_arb[nxt_thrd_3]

null_err_proc_phase_4#:
	_send_next_thrd_sig[nxt_thrd_3_gpr]

	ctx_arb[rx_event], br[rtn_label]

.end
#endm


/////////////////////////////////////////////////////////////////////////////
// _csix_rx_buf_zero_proc_proc
//	
//	Description:
//		This macro will handle the buf handle zero case when the csix rx 
//		find the prefetch buffer handle is zero.
//
//	Outputs:	
//		output:	Some global gpr midified
//			ele_addr: modified
//		
//	Inputs:
//		input:	
//			rtn_label : After the macro finishes, return to this label
//
//	CONSTANTS	:	None
//	
//
//	Size:	
/////////////////////////////////////////////////////////////////////////////
#macro	_csix_rx_buf_zero_proc[rtn_label]
.begin

.reg lkup_key												; lookup key 
.reg lkup_result											; lookup result
.reg seq_num												; sequence number

.reg drop_sop_hndl											; drop SOP handle
.reg drop_eop_hndl											; drop EOP handle

#ifdef	COUNTERS
.reg $byte_cnt												; byte counter xfers
#define	byte_cnt_sig_2	msf_sig								; reuse msf_sig as byte 
#endif														; count sig

	// increment debug counter for null buffers
	_csix_rx_debug_incr_counter[@csix_rx_num_buf_alloc_fail]

buf_zero_phse_1#:
	
	//	Read prepend header

	msf[read,$prpnd0, rbuf_base, ele_addr, 4], \
		sig_done[msf_sig]


	// increment debug counter
	_csix_rx_debug_incr_counter[@csix_rx_num_buf_alloc]

	//	get new prefetch buffer

	dl_buf_alloc[$prf_hndl, BUF_FREE_LIST0, prf_sig, SIG_NONE]

	//	Wait MSF read finish and nxt_thrd_1 signal

	ctx_arb[nxt_thrd_1, msf_sig],and,defer[2]
		_get_ext_class[ext_class,$rsw_2]						; get class ID
		_get_pld_len[payload_len,$rsw_1]						; get payload length


buf_zero_phase_2#:
	_send_next_thrd_sig[nxt_thrd_1_gpr]

	; Get class id.
	#ifdef USE_EXT_CLASS_ID
	; Get class id from extension class id (present in csix extension header)
	ld_field_w_clr[class_id, 0011, ext_class]
	#else
	; Get class id from qos class id (present in csix prepend header)
	ld_field_w_clr[class_id, 0011, $prpnd1]
	#endif

	; We have used $rsw_1 and $rsw_2 for the current c-frame.
	; So we can add the thread to the receive freelist.
	msf[fast_wr, --, msf_freelist0_addr, flst_reg]

	alu[in_bld_num,0x3F,and,$prpnd0]							; get ingress blade
	alu[lkup_key, class_id, or, in_bld_num, <<4] 				; get CAM lookup key

#ifdef COUNTERS
	alu[flags, 0x1, and, $prpnd0, >>6]							; get SOP/EOP flags
	beq[buf_zero_no_sop#], defer[2]								; check SOP flag
		alu[cnt_addr, --, B, lkup_key, <<4]						; get conter addr
		alu[cnt_addr, cnt_addr, +, cnt_base]					; add base

buf_zero_sop#:
	sram[incr, --, cnt_addr, PKT_RECEIVED_CNT_IDX]				; increment pkt count
	sram[incr, --, cnt_addr, PKT_DROPPED_CNT_IDX]				; increment dropped
																; pkt count
buf_zero_no_sop#:
	alu[$byte_cnt, --, B, payload_len]							
	sram[add, $byte_cnt, cnt_addr, BYTE_RECEIVED_CNT_IDX],\
	     sig_done[byte_cnt_sig_2]								; add byte count
#endif

	//	Do CAM lookup & set up local memory address

	cam_lookup[lkup_result, lkup_key],lm_addr0[0]

	//	Check lookup result

	br!=byte[lkup_result, 1, 0x0, buf_zero_cam_hit#]			; if hit go hit proc
	
buf_zero_cam_miss#:
.begin
	//	This is a CAM miss case
	//  LRU LM CRX=> SRAM, SRAM CRX=>$Xfers

.reg tmp_tag, tmp_entry, addr_in, addr_out

	xbuf_alloc[$crx_in, 3, read]								; alloc $crx_in
	xbuf_alloc[$crx_out, 8, write]  							; alloc $crx_out
	alu_shf[addr_in, crx_sram_gpr, or, lkup_key, <<5] 			; set CRX SRAM address
	sram[read, $crx_in[0], addr_in, 0, 3], sig_done[sram_sig_rd]	; read CRX SRAM 

	//	Move LRU LM CRX data to the $crx_out xfers

	alu[$crx_out[0], --, B, *l$index0[0]]	
	alu[$crx_out[1], --, B, *l$index0[1]]
	alu[$crx_out[2], --, B, *l$index0[2]]
	alu[$crx_out[3], --, B, *l$index0[3]]
	alu[$crx_out[4], --, B, *l$index0[4]]
	alu[$crx_out[5], --, B, *l$index0[5]]
	alu[$crx_out[6], --, B, *l$index0[6]]
	alu[$crx_out[7], --, B, *l$index0[7]]

	alu[tmp_entry, 0xF,and, lkup_result, >>3]					; get CAM entry
	cam_read_tag[tmp_tag,tmp_entry]								; read CAM tag
	alu[addr_out,crx_sram_gpr, or, tmp_tag, <<5]				; get CRX SRAM addr
	sram[write, $crx_out[0], addr_out, 0, 8], \
	     sig_done[sram_sig_wr]
	ctx_arb[nxt_thrd_2, sram_sig_rd, sram_sig_wr],and,defer[1]
		cam_write[tmp_entry, lkup_key, NORMAL_STATE]			; update CAM LRU entry

buf_zero_cam_miss_phase_3#:

	//	move $xfers CRX to LM

	alu[*l$index0[0],--,B, $crx_in[0]]	
	alu[*l$index0[1],--,B, $crx_in[1]]
	alu[*l$index0[2],--,B, $crx_in[2]]

	xbuf_free[$crx_in]											; free $crx_in
	xbuf_free[$crx_out]											; free $crx_out


buf_zero_common_phase_3#:
	_send_next_thrd_sig[nxt_thrd_2_gpr]

	alu[seq_num, --, B, lm_cell_seq_num, >>16]					; get sequence number 
	alu[lm_cell_seq_num,--,B,0x0]								; seq number=0x0

#ifdef COUNTERS 
	ctx_arb[nxt_thrd_3,prf_sig, byte_cnt_sig_2],defer[2]
#else
	ctx_arb[nxt_thrd_3,prf_sig],defer[2]
#endif
		alu[drop_sop_hndl,--,B,lm_sop_buf_hndl]					; set drop SOP hndl
		alu[drop_eop_hndl,--,B,lm_cur_buf_hndl]					; set drop EOP hndl

buf_zero_common_phase_4#:
	_send_next_thrd_sig[nxt_thrd_3_gpr]

	alu[--,seq_num,-,0x0]										; check sequence num
	beq[buf_zero_common_phase_4_no_drop#] 						; if 0, no packet drop
	_drop_current_pkt[drop_sop_hndl, drop_eop_hndl,lkup_key]	; drop error packet

buf_zero_common_phase_4_no_drop#:

	//	get prefetch data handle from prefetch buffer handle

	dl_buf_get_data[prf_data_hndl, $prf_hndl]    				; get data handle
	alu_shf[ele_addr, --, B, ele_num,<<16] 						; set element num gpr
	msf[fast_wr, --, msf_rx_element_addr, ele_addr]				; Free current element
	
	ctx_arb[rx_event], br[rtn_label]
.end 															; err CAM lookup miss done

buf_zero_cam_hit#:
	ctx_arb[nxt_thrd_2], br[buf_zero_common_phase_3#]

.end
#endm

#endif	// __CSIX_RX_ONE_ME_UTIL_UC__