/////////////////////////////////////////////////////////////////////////////////////
//
//
//                  I N T E L   P R O P R I E T A R Y
//
//     COPYRIGHT [c]  2002 BY  INTEL  CORPORATION.  ALL RIGHTS
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT
//     THE PRIOR WRITTEN PERMISSION OF :
//
//                        INTEL  CORPORATION
//
//                     2200 MISSION COLLEGE BLVD
//
//               SANTA  CLARA,  CALIFORNIA  95052-8119
//
/////////////////////////////////////////////////////////////////////////////////////
//
//      File Name: csix_rx.uc
//
//      Purpose: CSIX Receive Microblock 
//
//      Date            Comment                         				By
//      -------------------------------------------------------------------
//
//		08/08/2003		Rewritten to two ME to context pipeline 		urn
//
/////////////////////////////////////////////////////////////////////////////////////

#ifndef	__CSIX_RX_TWO_ME_UC__
#define	__CSIX_RX_TWO_ME_UC__

/////////////////////////////////////////////////////////////////////////////////////

// This microblock can run on either one ME or two ME's. When run on two ME's, it
// runs as a context pipeline. By default the microblock runs on one ME. If a 
// 'TWO_ME_CSIX_RX' switch is added, it runs on two ME's. The first ME of the 
// pipeline is identified using the 'FIRST_CSIX_RX_ME' switch. The other ME uses
// the 'SECOND_CSIX_RX_ME' switch.

/////////////////////////////////////////////////////////////////////////////////////

// Define meta-data size as 7 LW's.

#define	META_CACHE_SIZE		7

// Define Two ME CSIX RX

#define TWO_ME_CSIX_RX 

/////////////////////////////////////////////////////////////////////////////////////

#include <dl_system.h>
#include <stdmac.uc>
#include <dispatch_loop.uc>
#include <system_init.uc>
#include <hardware.h>

#include <csix_rx.h>				
#include <csix_rx_context.h>				
#include <csix_rx_common_util.uc>

/////////////////////////////////////////////////////////////////////////////////////
//
// csix_rx_init()
//
// Initialization Macro for CSIX RX first ME 
//
/////////////////////////////////////////////////////////////////////////////////////

#macro csix_rx_first_me_init[]
.begin

	// Locally used register declaration 

	.reg temp_addr											; temporary addr gpr
	.reg tmp0												; temporary gpr 0
	.reg tmp1												; temporary gpr 1
	.reg tmp2												; temporary gpr 2
	.reg me_num												; me number 
	.reg thread_num											; thread number 
	.sig sram_sig											; sram signal 
	.sig msf_sig											; msf signal

	#ifdef	COUNTERS

	// Set base for counters 

	immed32[counter_base,CSIX_RX_COUNTERS_SRAM_BASE]	

	#endif

	#if (!IS_IXPTYPE(__IXP28XX))

	// Set mask to randomize offset 

	move[rand_offset_mask, OFFSET_MASK]					

	#endif

	//	Following 4 instructions are called to 
	//  eliminate the warnings from the dispatch macro

	move[tmp0,0x0]
	move[tmp1,0x0]
	move[tmp2,0x0]

	move[rx_stat, 0x0]		; this is unicast, by default											

	dl_meta_init_cache[tmp0,tmp1,tmp2,tmp0,tmp1,tmp2,tmp0,tmp1]

	// Clear RSW write transfer registers

	move[$rsw[0],0]								
	move[$rsw[1],0] 							
	
	// Clear RSW read transfer registers to avoid warnings

	sram[read, $rsw[0], tmp0,tmp1, 2], ctx_swap[sram_sig]	

	// Initialize error code mask 

	move[error_code_mask,ALL_ERROR_CODE] 					

	// Get the thread and microengine number 

	_get_current_me_thread_number[me_num, thread_num]

	//	Set next thread signal gpr 

	move[tmp0, 0x80]
	alu_shf[next_thread_gpr, tmp0, OR, &next_thread_signal, <<3]

	// Setup default signal mask for phase 3

	alu[sigmask_default, --, b, 1, <<&rx_event_signal]
	alu[sigmask_default, sigmask_default, or, 1, <<&meta_write_signal]

	// set signal mask to sigmask_default initially 

	alu[sigmask, --, B, sigmask_default]

	//	set freelist register

	_set_freelist_reg[freelist_reg, &rx_event_signal, me_num, thread_num, &$rsw[0]]

	// Initialize freelist register and MSF freelist address registers to optimize
	// instructions

	alu[freelist_reg,--, B, freelist_reg, <<16]
	move[msf_freelist0_addr,RX_THREAD_FREELIST_0]

	// Set up RBUF base 

	move[rbuf_base, RBUF_TBUF_BASE]		

	// Set up address to free RBUFs

	move[msf_rx_element_addr, RBUF_ELEMENT_DONE] 	

	
	// Set up default offset for buffers
	
	move[buffer_default_offset, 384]
	
	// Set up size to check for buffer full 

	move(buffer_full_check_size, (BUFFER_SIZE - RBUF_SIZE))		

	// Set up a constant corresponding to RBUF element mask in a register 

	#if IS_IXPTYPE(__IXP28XX)

		// 128 addressable elements of 64 bytes each 

		immed32[element_mask, 0x1fC0]

	#else

		// 64 addressable elements of 128 bytes  each 

		immed32[element_mask, 0x1f80]

	#endif

	// Clear dl buffer handle 

	alu[dl_buf_handle,--,B,IX_DL_BUF_NULL]			

	// Do some initialization only in thread 0

	.if (ctx() == 0)

		#if (!IS_IXPTYPE(__IXP28XX))

			// set global random buffer offset

			move[@rand_buf_offset, MAX_BUFFER_OFFSET]	

		#endif

		// Initialize the NN rings. Set empty threshold to 0 LW's 

		_csix_rx_init_nn_ring(0)

		//	Clear all local memory context entries

		_csix_rx_clear_rxc()							

		#if (!IS_IXPTYPE(__IXP28XX))

			// set up value for IXP2400

			_set_rx_up_ctrl[0]

		#endif

		// System initialization

		system_init()

		// Initialize NN ring with buffer handles so the threads do not 
		// need to do sram[dequeue] to fetch buffer handles.

		_csix_rx_prefetch_handles()

		// initialize our debug counters

		#ifdef	_DEBUG_COUNTERS_

		immed[@csix_rx_num_valid_cells_recd,0]		
		immed[@csix_rx_num_err_cells_recd,0]		
		immed[@csix_rx_num_seq_error_cells, 0]
		immed[@csix_rx_num_buf_used,0]		
		immed[@csix_rx_num_sram_buf_alloc,0]		
		immed[@csix_rx_num_buf_alloc_fail,0]		

		#endif

	.else

		// wait for inter thread signal in all other threads 

		ctx_arb[next_thread_signal]	 			

	.endif
	
	// Get prefetch buffer

	dl_buf_alloc($prefetch_handle, BUF_FREE_LIST0, prefetch_signal, \
				 SIGNALS(prefetch_signal))

	// Initialize gpr that holds prefetched buffer handle.to 0. The 
	// correct handled will be xored in

	move(prefetch_handle_gpr, 0)

	//	Add this thread to the freelist

	msf[fast_wr, --, msf_freelist0_addr, freelist_reg]

	// Send next thread signal 

	_send_next_thrd_sig(next_thread_gpr)

	// check if current thread 7

	alu[--, thread_num, -, 7]	

	// if current thread is not thread 7, go to no_msf_config

	bne[no_msf_config#]	

	//	current thread is 7, configure MSF, 

	_config_msf_rx_ctrl[]	
	

no_msf_config#:

	// swap out and wait for the recieve logic to 
	// wake the thread

	ctx_arb[rx_event_signal]

.end

#endm

/////////////////////////////////////////////////////////////////////////////////////
//
// csix_rx_second_me_init()
//
// Initialization Macro for CSIX RX second ME 
//
/////////////////////////////////////////////////////////////////////////////////////

#macro csix_rx_second_me_init[]
.begin

	// Locally used register declaration 

	.reg temp_addr											; temporary addr gpr
	.reg tmp0												; temporary gpr 0
	.reg tmp1												; temporary gpr 1
	.reg tmp2												; temporary gpr 2

	//	Following 4 instructions are called to 
	//  eliminate the warnings from the dispatch macro

	move[tmp0,0x0]
	move[tmp1,0x0]
	move[tmp2,0x0]

	alu_shf[csix_rx_out_ring,--, B, CSIX_RX_RING_OUT, <<2]
	move[rx_stat, 0x0]

#ifdef	MULTICAST_SUPPORT_ENABLE

	alu_shf[csix_rx_mcast_out_ring, --, B, CSIX_RX_RING_OUT_1, <<2]

#endif

	dl_meta_init_cache[tmp0,tmp1,tmp2,tmp0,tmp1,tmp2,tmp0,tmp1]

	//	Set next thread signal gpr 

	move[tmp0, 0x80]
	alu_shf[next_thread_gpr, tmp0, OR, &next_thread_signal, <<3]

	// Setup default signal mask 

	alu[sigmask_default, --, b, 3, <<&dram_write_signal]
	alu[sigmask_default, sigmask_default, or, 1, <<&meta_write_signal]
	alu[sigmask_default, sigmask_default, or, 1, <<&next_thread_signal]
	alu[sigmask_default, sigmask_default, or, 1, <<&scratch_write_signal]

	alu[sigmask, --, B, sigmask_default]

	// Initially there is no scratch write. So remove the scratch write signal

	alu[sigmask, sigmask, AND~, 1, <<&scratch_write_signal]

	// Set up RBUF base 

	move[rbuf_base, RBUF_TBUF_BASE]	

	// Set up address to free RBUFs

	move[msf_rx_element_addr, RBUF_ELEMENT_DONE] 	
	
	// Set up constants in registers to optimize critical path 

	move[rxc_context_ptr_mask, 0x7f8]
	move[element_address_mask, 0x1fff]	
	move[ffff_mask, 0xffff]
	move[dram_rbuf_ov, ((1 << 25) | (1<<4)) ]

#ifdef	MULTICAST_SUPPORT_ENABLE
	// 2 bits are used for rx_stat
	move[output_port_mask, 0x3f0]		
#else
	move[output_port_mask, 0xff0]		
#endif

	// Initialize drop handles and voq 

	move(drop_sop_handle, 0)
	move(drop_eop_handle, 0)
	move(drop_voq, 0)

	// Clear dl buffer handle 

	alu[dl_buf_handle,--,B,IX_DL_BUF_NULL]			

	// Do some initialization only in thread 0

	.if (ctx() == 0)

		// Initialize the NN rings. Set empty threshold to 3 LW's or less 

		_csix_rx_init_nn_ring(0x3)

		//	Clear all local memory context entries

		_csix_rx_clear_rxc()							

		// initialize our debug counters
		
		#ifdef _DEBUG_COUNTERS_

		immed[@csix_rx_num_pkts_sink, 0]
		immed[@csix_rx_num_scratch_ring_full, 0]
		immed[@csix_rx_num_drop_pkts, 0]

		#endif

	.else

		// wait for inter thread signal in all other threads 

		ctx_arb[next_thread_signal]	 			

	.endif
	
	// Send next thread signal 

	_send_next_thrd_sig(next_thread_gpr)

.end

#endm

//////////////////////////////////////////////////////////////////////////////////////
//
// _csix_rx_prefetch_handles()
//
// Prefetch 95 buffer handles into the NN ring
//
//
//////////////////////////////////////////////////////////////////////////////////////

#macro _csix_rx_prefetch_handles()
.begin

	.reg count
	.reg ctx_enable

	//	Set the NN ring so that the ME can write to its own NN ring
	
	local_csr_rd[ctx_enables]
	
	immed[ctx_enable, 0]
	alu_shf[ctx_enable, ctx_enable, OR, 1, <<20]
	
	local_csr_wr[ctx_enables, ctx_enable]
	alu[--, --, b, 0]
	alu[--, --, b, 0]
	alu[--, --, b, 0]

	// Prefetch 95 buffer handles from SRAM and store them into NN ring

	immed[count, 0]

	.while (count < 95)

		// Fetch a buffer handle from SRAM

		dl_buf_alloc ($prefetch_handle, BUF_FREE_LIST0, prefetch_signal, \
					 SIGNALS(prefetch_signal))

		// Store the buffer handle in the NN ring.
	
		alu[*n$index++, --, b, $prefetch_handle]
		alu[count, count, +, 1]

	.endw

	//	Set the NN ring so that the previous ME can write to the NN ring
	
	local_csr_rd[ctx_enables]
	
	immed[ctx_enable, 0]
	alu_shf[ctx_enable, ctx_enable, AND~, 1, <<20]
	
	local_csr_wr[ctx_enables, ctx_enable]
	alu[--, --, b, 0]
	alu[--, --, b, 0]
	alu[--, --, b, 0]

.end
#endm

///////////////////////////////////////////////////////////////////////////////////////
//
// _csix_rx_init_nn_ring()
//
// Initialize the NN ring for CSIX Receive 
//
///////////////////////////////////////////////////////////////////////////////////////

#macro _csix_rx_init_nn_ring(ring_empty_threshold)
.begin

	.reg ctx_enable
	
	// Read the context enable register

	local_csr_rd[ctx_enables]
	immed[ctx_enable, 0]
	
	// Set the empty threshold to 0 - bits 18 and 19

	alu_shf[ctx_enable, ctx_enable, AND~, 3, <<18]
	alu_shf[ctx_enable, ctx_enable, OR, ring_empty_threshold, <<18]

	// Write context enable and get and put pointers

	local_csr_wr[ctx_enables, ctx_enable]
	local_csr_wr[nn_get, 0]
	local_csr_wr[nn_put, 0]	

	alu[--, --, b, 0]
	alu[--, --, b, 0]
	alu[--, --, b, 0]

.end
#endm

///////////////////////////////////////////////////////////////////////////////////////
//
//	_csix_rx_clear_rxc()
//
//	Clear the local memory for CSIX Receive Contexts
//
//
///////////////////////////////////////////////////////////////////////////////////////

#macro _csix_rx_clear_rxc()
.begin

	.reg 	count
	.reg	lm_base

	// Set the local memory index to point to first context 

	immed32[lm_base, CRX_LM_BASE]
	localmem_set_address[0, lm_base, LM_HANDLE_0]

	nop
	nop
	nop

	alu[count, --, B, 0]

	// loop across all contexts resetting to 0 

	.while (count < CSIX_RX_NUMBER_OF_VOQ)
		
		alu[*l$index0++, --, B, 0]
		alu[*l$index0++, --, B, 0]

		alu[count, count, +, 1]

	.endw

.end
#endm

/////////////////////////////////////////////////////////////////////////////////////
//
// csix_rx_read_prepend() : 13 instructions in critical path 
//
/////////////////////////////////////////////////////////////////////////////////////

#macro _csix_rx_read_prepend(buf_handle_zero_label, CHECK_PREFETCH_HANDLE)
.begin

	// Issue a read into the MSF for the prepend. Fill in as many instructions
	// as we can to cover the MSF read latency

	msf[read, $prepend[0], rbuf_base, element_address, 4], sig_done[msf_signal]

	// Restore default signal mask 

	alu[sigmask, --, b, sigmask_default]

	// 	Get the class id in the CSIX extension header in the receive status word

	alu_shf[ext_class_id, --, B, $rsw[1], >>24]

	#if (CHECK_PREFETCH_HANDLE == CSIX_RX_CHECK_PREFETCH)

		// Get the pre-fetched buffer handle. The xor is required because 
		// we may have either got the buffer from the NN ring or the Q-Array

		alu[dl_buf_handle, prefetch_handle_gpr, xor, $prefetch_handle]

		// Check if the prefetched buffer handle was zero, implying null buffer

		beq[buf_handle_zero_label]

	#endif 

#ifdef	MULTICAST_SUPPORT_ENABLE

	// Derive the RX status from the C-frame type field. We currently support only 3
	// types: 00 -> unicast, 01 -> multicast ID, 10 -> broadcast
	alu_shf[rx_stat, 0x3, AND, $rsw[0], >>1]

#endif

	// Get the payload length in the receive status word
	
	alu[payload_size, 0xff, AND, $rsw[0], >>16]

	// Packet buffer length  = payload length -  prepend header length

	alu[payload_size, payload_size, -, CSIX_RX_SOP_PREPEND_SIZE]

	// wait for msf read to complete 

	ctx_arb[next_thread_signal, msf_signal] 

	// We have used $rsw for the current c-frame.
	// So we can add the thread back to the receive freelist

	msf[fast_wr, --, msf_freelist0_addr, freelist_reg]

	// Send next thread signal for phase 1 

	_send_next_thrd_sig(next_thread_gpr)

	// Get SOP/EOP flags 

	alu[flags, 0x3, and, $prepend[0], >>6]

	// Get ingress blade number.
	
	alu[ingress_blade_id, 0x3F, and, $prepend[0]]

	// Get class id.
	
	#ifndef USE_EXT_CLASS_ID
		
		// Get class id from qos class id (present in csix prepend header)
		// This may or may not be the same as the extension class id. The 
		// extension class id is used to determine the VoQ on fabric and the
		// class id is used to determine the queue number on the egress interface
	
		ld_field_w_clr[class_id, 0001, $prepend[1]]
	
	#else

		// Use the extension class id for both voq and queuing on egress

		ld_field_w_clr[class_id, 0001, ext_class_id]

	#endif 


	// Find reassembly context in local memory, 2 LW or 8 bytes per context.
	// Reassembly context is found by voq number = (blade << SHIFT) | ext_class_id

	alu[voq_number, ext_class_id, or, ingress_blade_id, <<CSIX_RX_BITS_FOR_CLASS]
	
	// Get the reassembly context pointer in local memory 

	alu[rxc_context_ptr, --, B, voq_number, <<3]

	#if (CRX_LM_BASE != 0) 

		alu[rxc_context_ptr, CRX_LM_BASE, +, rxc_context_ptr]

	#endif
	



.end
#endm

/////////////////////////////////////////////////////////////////////////////////////
//
// 	csix_rx_first_me:	Packet Reassembly for CSIX Receive first microengine 
//
/////////////////////////////////////////////////////////////////////////////////////

#macro csix_rx_first_me()
.begin

start#:

	.io_completed  meta_write_signal, prefetch_signal

	// When a c-frame is received, the next thread from the thread freelist 
	// is signaled. The thread enters here. The Receive status word for the 
	// c-frame is in $rsw_1 and $rsw_2.

	// Check if an erroneous c-frame was received.

	alu[ --, $rsw[0], AND, error_code_mask] 

	// If error, go to error processing 

	bne[err_proc#] , defer[1]

	// Get RBUF element address

	#if IS_IXPTYPE(__IXP28XX)

		alu_shf[element_address, element_mask, AND, $rsw[0], >>18]

	#else

		alu_shf[element_address, element_mask, AND, $rsw[0], >>17]

	#endif

	// Increment debug counter for no rsw error cells received.

	_csix_rx_debug_incr_counter(@csix_rx_num_valid_cells_recd)

	// Read in the prepend header 

	_csix_rx_read_prepend(buf_handle_zero#, CSIX_RX_CHECK_PREFETCH)

	// Set local memory pointer, 3 cycles to take effect 

	localmem_set_address[0, rxc_context_ptr, LM_HANDLE_0]

	// Check if both SOP and EOP flags are set 

	br!=byte[flags, 0, 0x3, nsep#]
	

sep#:

	// SOP/EOP min packet case 

	_csix_rx_sop_proc(CSIX_RX_SOP_EOP_FLAGS, start#)

nsep#:
	
	// This is the non min packet case. Check if only SOP flag is set.

	br_bclr[flags, 0, nsop#]

sop#:

	// Do SOP c-frame processing and return to 'start#'

	_csix_rx_sop_proc(CSIX_RX_SOP_ONLY_FLAG, start#)


nsop#:

	// This is the non sop case. Check if only EOP flag is set

	br_bclr[flags, 1, mop#]

eop#:

	// Do EOP c-frame processing and return to 'start#'

	_csix_rx_eop_proc(start#)

mop#:

	// Do MOP c-frame processing and return to 'start#'

	_csix_rx_mop_proc(start#)


err_proc#:

	// Do erroneous c-frame processing and return to 'start#'

	_csix_rx_error_proc(start#)

buf_handle_zero#:

	// handle case where no prefetch buffer is available and return to 'start#'

	_csix_rx_null_proc(start#)


.end
#endm // csix_rx_first_me() 

////////////////////////////////////////////////////////////////////////////////////
//
// _csix_rx_sop_proc
//	
//		This macro will handle the SOP/EOP or SOP only c-frame case when the CSIX RX
//		receives a normal c-frame.	
//
////////////////////////////////////////////////////////////////////////////////////

#macro _csix_rx_sop_proc[SOP_EOP_FLAG, return_label]
.begin

	.reg buffer_offset				; offset into packet buffer
	.reg output_port				; output port number
	.reg header_type				; header type
	.reg next_hop_id				; next hop id 
	.reg payload_offset				; offset into cframe payload
	.reg expected_sequence_number	

	#ifdef	COUNTERS

	.reg $byte_counter				; byte counter
	.sig counter_signal				; byte count sig

	#endif

	// Increment debug counter 

	_csix_rx_debug_incr_counter(@csix_rx_num_buf_used)

	// Prefetch buffer for next iteration (7)

	_csix_rx_buffer_alloc(BUF_FREE_LIST0, prefetch_signal, SIG_NONE, sigmask)

	// Get next hop id

	ld_field_w_clr[next_hop_id, 0011, $prepend[1], >>16]
	
	// Get  output port.from prepend 

	ld_field_w_clr[output_port, 0011, $prepend[2]]

	// Get  header type.from prepend 

	ld_field_w_clr[header_type, 0001, $prepend[2], >>16]

	// Check if expected cell sequence number is 0

	ld_field_w_clr[expected_sequence_number, 0001, lm_cell_seq_num] , load_cc

	bne[sequence_check_failed#], defer[1]

	// For SOP/EOP case reset sequence number to 0, else increment it to 1 

	#if (SOP_EOP_FLAG == CSIX_RX_SOP_EOP_FLAGS)

		// Set expected seq num as 0.
	
		alu[lm_cell_seq_num, --, b, 0x0]

	#else 

		// Set expected seq num as 1
	
		alu[lm_cell_seq_num, --, b, 0x1]

	#endif 


sequence_check_done#:


	// Write meta data (8)  

	dl_meta_write[$dl_meta, dl_buf_handle, 0, 0, 0, 0, FREE_LIST_ID, 0, 0, 0, 
				  output_port, 0, next_hop_id, $prepend[3], 
				  class_id, 0, meta_write_signal, SIG_NONE, 3, 4]

	// Get offset into cframe

	alu[payload_offset, 0xFF, and, $prepend[0], >>24]


	#if IS_IXPTYPE(__IXP28XX)

		// For IXP2800 we do not need to round robin among buffer offsets
		// to achieve uniform DRAM bank utilizations

		alu[buffer_offset, buffer_default_offset, +, payload_offset]

	#else

		// Get random buffer offset

		_get_rand_buf_offst[buffer_offset]

		// Add the cframe payload offset

		alu[buffer_offset, buffer_offset, +, payload_offset]

	#endif

	// Also deduct bytes that are not a part of payload

	alu[payload_size, payload_size, -, payload_offset]

	// Increment byte and packet counters for the VoQ for the SOP/EOP case 

	#if (SOP_EOP_FLAG == CSIX_RX_SOP_EOP_FLAGS) 
	#ifdef COUNTERS 

		// Set byte counter to packet size 

		move[$byte_counter, buf_size]

		// Increment counter 

		_csix_rx_increment_counters(counter_base, voq_number, $byte_counter, \
								 	counter_signal, sigmask)
	#endif
	#endif 

	// Send message to the next microengine (7)

#ifdef	MULTICAST_SUPPORT_ENABLE
	
	// send rx_stat also

	_csix_rx_send_to_next_me (dl_buf_handle, buffer_offset, payload_size, \
							  header_type, voq_number, flags, output_port, class_id, rx_stat)

#else

	// Ignore RX_STAT

	_csix_rx_send_to_next_me (dl_buf_handle, buffer_offset, payload_size, \
							  header_type, voq_number, flags, output_port, class_id, 0)

#endif

	// For SOP case, store some variables in local memory 

	#if (SOP_EOP_FLAG == CSIX_RX_SOP_ONLY_FLAG)

		// store buffer offset and packet size into one long word 

		alu[lm_sop_buf_ofst, payload_size, OR, buffer_offset, <<16]

		// Increment buffer offset by RBUF size 

		alu[buffer_offset, buffer_offset, +, payload_size]

		// Set buffer offset .
	
		ld_field[lm_curr_buf_ofst, 1100, buffer_offset, <<16]

		// Set header type  

		ld_field[lm_header_type, 0010, header_type, <<8]


	#endif 
	
	// Wait on all signals including MSF rx event, meta data, and maybe buf alloc
	
	ctx_arb[--], br [return_label], defer[1]
	
	// Use signal mask 

	local_csr_wr[active_ctx_wakeup_events, sigmask]


sequence_check_failed#:

	// Expected cell sequence number is not 0. This implies we
	// received a sop cell while expecting a non-sop cell. Set a 
	// flag so the next ME will drop the previous packet being 
	// reassembled

	alu[flags, flags, OR, 1, <<CSIX_RX_DROP_BIT]

	// Increment debug counter for sequence error cells

	_csix_rx_debug_incr_counter(@csix_rx_num_seq_error_cells)

	// Start processing from where the sequence check is complete 

	br[sequence_check_done#] 

.end
#endm


////////////////////////////////////////////////////////////////////////////////////
//
// _csix_rx_eop_proc
//	
//		This macro will handle the EOP c-frame case 
//
////////////////////////////////////////////////////////////////////////////////////

#macro _csix_rx_eop_proc[return_label]
.begin

	.reg buffer_offset				; offset into packet buffer
	.reg packet_size				; packet size 
	.reg header_type				; header type
	.reg expected_sequence_number	; expected sequence number 
	.reg actual_sequence_number		; actual sequence number
	.reg sop_buffer_offset			; offset into SOP buffer
	.reg element					; used to free

	#ifdef	COUNTERS

		.reg $byte_counter			; byte counter
		.sig counter_signal			; byte count sig

	#endif

	// Get the sequence number from local memory 

	ld_field_w_clr[expected_sequence_number, 0001, lm_cell_seq_num]

	// Get the sequence number from the prepend 

	ld_field_w_clr[actual_sequence_number, 0011, $prepend[0], >>8]

	// Compare the two 

	alu[--, expected_sequence_number, -, actual_sequence_number]

	// If they are not equal the sequence check has failed, drop the packet 

	bne[sequence_check_failed#], defer[1]

	// Set expected seq num as 0.
	
	ld_field[lm_cell_seq_num, 0001, 0x0]

	// We subtracted the SOP prepend size to optimize critical path. But prepend
	// for EOP is smaller. So add back incremental prepend

	alu[payload_size, payload_size, +, CSIX_RX_PREPEND_DIFF]

	// Get packet size from local memory 

	alu[packet_size, payload_size, +16, lm_pkt_size]

	// Get buffer offset from local memory

	alu[buffer_offset, --, B, lm_curr_buf_ofst, >>16]

	// Get header type from local memory 

	ld_field_w_clr[header_type, 0001, lm_header_type, >>8]

	// Get SOP buffer offset from local memory 

	alu[sop_buffer_offset, --, B, lm_sop_buf_ofst, >>16]

	// Increment byte and packet counters for the VoQ 

	#ifdef COUNTERS

		move($byte_counter, packet_size)
		_csix_rx_increment_counters(counter_base, voq_number, $byte_counter, \
							 		counter_signal, sigmask)

	#endif 


sequence_check_done#:

	// Send message to the next microengine. Since output port field is not used 
	// in the message, we use it to pass the SOP buffer offset. Also since buffer
	// handle is not used, we use it to pass packet size 


	_csix_rx_send_to_next_me (packet_size, buffer_offset, payload_size, \
							  header_type, voq_number, flags, sop_buffer_offset, \
							  class_id, 0)


	// Wait on all signals including MSF rx event. Remove other signals from mask

	alu[sigmask, sigmask, AND~, 1, <<&meta_write_signal]
	
	ctx_arb[--], br [return_label], defer[1]
	
	// Use signal mask 

	local_csr_wr[active_ctx_wakeup_events, sigmask]


sequence_check_failed#:

	// Expected cell sequence number is not 0. This implies we
	// received a sop cell while expecting a non-sop cell. Set a 
	// flag so the next ME will drop the previous packet being 
	// reassembled

	alu[flags, flags, OR, 1, <<CSIX_RX_DROP_BIT]

	// To avoid assembler warnings 

	move[header_type, 0]
	move[packet_size, 0]
	move[buffer_offset, 0]
	move[sop_buffer_offset, 0]

	// free the rbuf 

	alu_shf[element, --, B, element_address, <<10]
	msf[fast_wr, --, msf_rx_element_addr, element]

	// Increment debug counter for sequence error cells

	_csix_rx_debug_incr_counter(@csix_rx_num_seq_error_cells)

	// Start processing from where the sequence check is complete 

	br[sequence_check_done#] 

.end
#endm

////////////////////////////////////////////////////////////////////////////////////
//
// _csix_rx_mop_proc
//	
//		This macro will handle an mop cframe 	
//
////////////////////////////////////////////////////////////////////////////////////

#macro _csix_rx_mop_proc[return_label]
.begin

	.reg buffer_offset				; offset into current buffer 
	.reg expected_sequence_number	; expected sequence number 
	.reg actual_sequence_number		; actual sequence number
	.reg new_buffer_offset			; updated buffer offset in current buffer 
	.reg element					; used to free rbuf
	.reg sop_buffer_offset			; sop buffer offset 

	// Get the sequence number from local memory 

	ld_field_w_clr[expected_sequence_number, 0001, lm_cell_seq_num]

	// Get the sequence number from the prepend 

	ld_field_w_clr[actual_sequence_number, 0011, $prepend[0], >>8]

	// Compare the two 

	alu[--, expected_sequence_number, -, actual_sequence_number]

	// If they are not equal the sequence check has failed, drop the packet 

	bne[sequence_check_failed#]

	// Increment expected sequence number. Assuming that this does not overflow beyond
	// 8 bits 
	
	alu[lm_cell_seq_num, 1, +, lm_cell_seq_num]

	// Add back difference in prepend

	alu[payload_size, payload_size, +, CSIX_RX_PREPEND_DIFF]

	// Get packet size from local memory. Assuming that this does not overflow beyond
	// 64k 

	alu[lm_pkt_size, payload_size, +, lm_pkt_size]

	// Get buffer offset from local memory

	alu[buffer_offset, --, B, lm_curr_buf_ofst, >>16]

	// Increment buffer offset and check if buffer is complete 

	alu[new_buffer_offset, buffer_offset, +, payload_size]

	// Now check if it can hold one more RBUF 

	alu[--, new_buffer_offset, -, buffer_full_check_size]

	ble[sequence_check_done#]

	// Buffer is done -- Use prefetched buffer as next buffer and then allocate a 
	// new buffer. Also set the end of buffer bit 

	alu[flags, flags, OR, 1, <<CSIX_RX_EOB_BIT]

	// Prefetch buffer for next iteration (7)

	_csix_rx_buffer_alloc(BUF_FREE_LIST0, prefetch_signal, SIG_NONE, sigmask)

	// Increment debug counter 

	_csix_rx_debug_incr_counter(@csix_rx_num_buf_used)

	// Set the new_buffer_offset to 0 

	alu[new_buffer_offset, --, b, 0]

sequence_check_done#:

	// Get the SOP buffer offset from local memory to pass it to the next ME 

	alu[sop_buffer_offset, --, B, lm_sop_buf_ofst, >>16]


	// Send message to the next microengine 


	_csix_rx_send_to_next_me (dl_buf_handle, buffer_offset, payload_size, \
							  0, voq_number, flags, sop_buffer_offset, 0, 0)


	// clear meta data signal from the signal mask

	alu[sigmask, sigmask, AND~, 1, <<&meta_write_signal]

	// Set the new buffer offset 

	ld_field[lm_curr_buf_ofst, 1100, new_buffer_offset, <<16]
	
	// Wait on all signals including MSF rx event, meta data, and maybe buf alloc
	
	ctx_arb[--], br [return_label], defer[1]
	
	// Use signal mask 

	local_csr_wr[active_ctx_wakeup_events, sigmask]

sequence_check_failed#:

	// Reset expected seq num as 0.
	
	alu[lm_cell_seq_num, --, b, 0x0]

	// Expected cell sequence number is not 0. This implies we
	// received a sop cell while expecting a non-sop cell. Set a 
	// flag so the next ME will drop the previous packet being 
	// reassembled

	alu[flags, flags, OR, 1, <<CSIX_RX_DROP_BIT]

	// free the rbuf 

	alu_shf[element, --, B, element_address, <<10]
	msf[fast_wr, --, msf_rx_element_addr, element]

	// To avoid assembler warnings 

	move[buffer_offset, 0]
	move[new_buffer_offset, 0]

	// Increment debug counter for sequence error cells

	_csix_rx_debug_incr_counter(@csix_rx_num_seq_error_cells)

	// Start processing from where the sequence check is complete 

	br[sequence_check_done#] 

.end
#endm

////////////////////////////////////////////////////////////////////////////////////
//
// _csix_rx_error_proc
//	
//		This macro will handle the SOP/EOP c-frame case when the CSIX RX
//		receives a normal c-frame.	
//
////////////////////////////////////////////////////////////////////////////////////

#macro _csix_rx_error_proc[return_label]
.begin
		
	.reg 	expected_sequence_number
	.reg	element

	// Check if there is null mpacket

	alu[--,$rsw[0], AND, 1, <<9]
			                                      
	bne[null_proc#]

	// Increment debug counter

	_csix_rx_debug_incr_counter(@csix_rx_num_err_cells_recd)

	// Read the prepend header 

	_csix_rx_read_prepend(0, CSIX_RX_DONT_CHECK_PREFETCH)

	// free the rbuf 

	alu_shf[element, --, B, element_address, <<10]
	msf[fast_wr, --, msf_rx_element_addr, element]

	// Set local memory pointer, 3 cycles to take effect 

	localmem_set_address[0, rxc_context_ptr, LM_HANDLE_0]
	nop
	nop
	nop

	// Get the sequence number from local memory 

	ld_field_w_clr[expected_sequence_number, 0001, lm_cell_seq_num] , load_cc

	// Check if it is already 0 

	beq[wait_for_signal#]

	// Otherwise there is a buffer being reassembled. Lets drop it. 
	// Set a flag so the next ME will drop the previous packet being 
	// reassembled

	alu[flags, --, B, 1, <<CSIX_RX_DROP_BIT]

	// Send message to the next microengine. It will free buffer and 
	// also rbuf. 

	_csix_rx_send_to_next_me (0, 0, payload_size, 0, voq_number, flags, 0, class_id, 0)

	// Set expected seq num as 0.
	
	alu[lm_cell_seq_num, --, b, 0x0]

wait_for_signal#:

	// Clear the bit for meta data write 

	alu[sigmask, sigmask, AND~, 1 , <<&meta_write_signal]

	ctx_arb[--], br [return_label], defer[1]
	
	// Use signal mask 

	local_csr_wr[active_ctx_wakeup_events, sigmask]


null_proc#:

	// Wait for next thread signal 

	ctx_arb[next_thread_signal]

	// Add the thread back to the freelist 

	msf[fast_wr, --, msf_freelist0_addr, freelist_reg]

	// Send next thread signal for phase 1 

	_send_next_thrd_sig(next_thread_gpr)

	// wait for receive event 

	ctx_arb[rx_event_signal] , br[return_label]

.end
#endm

////////////////////////////////////////////////////////////////////////////////////
//
// _csix_rx_null_proc
//	
//		This macro handles the case where prefetch buffer is NULL	
//
////////////////////////////////////////////////////////////////////////////////////


#macro _csix_rx_null_proc(return_label)
.begin

	.reg	expected_sequence_number 
	.reg	element

	// Increment debug counter for buf alloc fail error 

	_csix_rx_debug_incr_counter(@csix_rx_num_buf_alloc_fail)

	// wait for msf read to complete 

	ctx_arb[next_thread_signal, msf_signal] 

	// We have used $rsw for the current c-frame.
	// So we can add the thread back to the receive freelist

	msf[fast_wr, --, msf_freelist0_addr, freelist_reg]

	// free the rbuf 

	alu_shf[element, --, B, element_address, <<10]
	msf[fast_wr, --, msf_rx_element_addr, element]

	// Prefetch buffer for next iteration (7)

	_csix_rx_buffer_alloc(BUF_FREE_LIST0, prefetch_signal, SIG_NONE, sigmask)

	// Send next thread signal for phase 1 

	_send_next_thrd_sig(next_thread_gpr)

	// Get ingress blade number.
	
	alu[ingress_blade_id, 0x3F, and, $prepend[0]]

	// find reassembly context in local memory, 2 LW or 8 bytes per context 

	alu[voq_number, ext_class_id, or, ingress_blade_id, <<CSIX_RX_BITS_FOR_CLASS] 
	alu[rxc_context_ptr, --, B, voq_number, <<3]
	alu[rxc_context_ptr, CRX_LM_BASE, +, rxc_context_ptr]

	// Set local memory pointer, 3 cycles to take effect 

	localmem_set_address[0, rxc_context_ptr, LM_HANDLE_0]
	nop
	nop
	nop

	// Get the sequence number from local memory 

	ld_field_w_clr[expected_sequence_number, 0001, lm_cell_seq_num] , load_cc

	// Check if it is already 0 

	beq[wait_for_signal#]

	// Set a flag so the next ME will drop the previous packet being 
	// reassembled

	alu[flags, --, B, 1, <<CSIX_RX_DROP_BIT] 

	// set payload size to 0 to avoid warning in assembler. Ignored by next me 

	move(payload_size, 0)


	_csix_rx_send_to_next_me (0, 0, payload_size, 0, voq_number, flags, 0, ext_class_id, 0)


	// Set expected seq num as 0.
	
	alu[lm_cell_seq_num, --, b, 0x0]

wait_for_signal#:

	// remove the meta data write signal from mask

	alu[sigmask, sigmask, AND~, 1, <<&meta_write_signal]

	// Wait on signals including MSF rx event and maybe buf alloc
	
	ctx_arb[--], br [return_label], defer[1]
	
	// Use signal mask 

	local_csr_wr[active_ctx_wakeup_events, sigmask]

.end
#endm

////////////////////////////////////////////////////////////////////////////////////
//
// _csix_rx_send_to_next_me()
//
// Send a message to the next microengine: 7 instructions or more
//
////////////////////////////////////////////////////////////////////////////////////

#macro _csix_rx_send_to_next_me (handle_or_size, buffer_offset, payload_size, \
								 header_type, voq, flags, port_or_sop_offset, \
								 class_id, rx_stat)
.begin

	.reg tmp 

wait_for_nn#:

	// Wait for space in the NN ring 

	br_inp_state[NN_FULL, wait_for_nn#]

	// Send message to the next microengine

	#if (IS_IXPTYPE(__IXP28XX))

		alu[tmp, --, B, element_address, >>6]				; rbuf element number

	#else

		alu[tmp, --, B, element_address, >>7]				; rbuf element number

	#endif


	alu[tmp, tmp, OR, header_type, <<24]				; header type
	alu[tmp, tmp, OR, voq, <<8]							; voq number 
	alu[*n$index++, tmp, OR, flags, <<16]				; flags 
	
	// This field is used as a buffer handle for SOP and SOP/EOP case and for
	// packet size in EOP case 

	alu[*n$index++, --, B, handle_or_size ]					; buffer handle 
	alu[*n$index++, payload_size, OR, buffer_offset, <<16]	; buffer size and offset 


	#if (isnum(rx_stat))
	
		#if (rx_stat != 0)

			// The field here can be output port for min packets and sop_buffer_offset for 
			// eop mpacket case

			alu[tmp, port_or_sop_offset, OR, class_id, <<16]	; output port and class id 

			// set rx_stat only when it is non-zero

			alu[*n$index++, tmp, OR, rx_stat, <<24]	; rx_stat 

		#else

			// RX stat is ignored.

			alu[*n$index++, port_or_sop_offset, OR, class_id, <<16]	; output port and class id 

		#endif

	#else
	
		// The field here can be output port for min packets and sop_buffer_offset for 
		// eop mpacket case

		alu[tmp, port_or_sop_offset, OR, class_id, <<16]	; output port and class id 

		// set rx_stat only when it is non-zero

		alu[*n$index++, tmp, OR, rx_stat, <<24]	; rx_stat 

	#endif

.end
#endm

////////////////////////////////////////////////////////////////////////////////////
//
// _csix_rx_receive_from_previous_me()
//
// Receive a message from the previous microengine: max of 12 instructions
//
////////////////////////////////////////////////////////////////////////////////////

#macro _csix_rx_receive_from_previous_me (ring_empty_label)
.begin

	br_inp_state[NN_EMPTY, ring_empty_label]

	// Get the flags for SOP, EOP etc

	alu[flags, 0xff , AND , *n$index, >>16]

	// Get the receive context ptr in local memory in bytes and set up pointer 

	alu[rxc_context_ptr, rxc_context_ptr_mask, AND, *n$index, >>5]

	#if (CRX_LM_BASE != 0)

		// Add in the context base 

		alu[rxc_context_ptr, CRX_LM_BASE, +, rxc_context_ptr]

	#endif

	// Set up local memory pointer to context 

	localmem_set_address[0, rxc_context_ptr, LM_HANDLE_0]

	// Get rbuf element address 

	alu[element_address, element_address_mask, AND, *n$index, <<6]

	// Get the header type 

	alu[header_type, 0xff, AND, *n$index++, >>24]

	// Get the buffer handle for current buffer 

	alu[dl_buf_handle, --, B, *n$index++]

	// Get the rbuf payload size to be written 

	ld_field_w_clr[payload_size, 0011, *n$index]

	// Get offset for current buffer 

	alu[buffer_offset, --, B, *n$index++, >>16] 

	// This field can be output port for sop packets or SOP offset for eop case  

	ld_field_w_clr[port_or_sop_offset, 0011, *n$index]


#ifdef	MULTICAST_SUPPORT_ENABLE

	// get the rx_status. For non (SOP and SEP) cases, this is zero.

	alu_shf[rx_stat, 0x3, AND, *n$index, >>24]

#endif

	// Get class id (only 2 to 4 bits are needed)

	alu[class_id, --, B, *n$index++, >>16]

.end
#endm

///////////////////////////////////////////////////////////////////////////////////////
//
// _csix_rx_write_to_dram()
//
// Write cframe to DRAM: max 13 instructions
//
///////////////////////////////////////////////////////////////////////////////////////

#macro _csix_rx_write_to_dram(prepend_size)
.begin

	.reg dram_handle 
	.reg element_offset
	.reg dram_write_size
	.reg payload_offset

	// Get the pointer into DRAM (up to 4)

	dl_buf_get_data(dram_handle, dl_buf_handle)

	// Increment by offet 

	alu[dram_handle, dram_handle, +, buffer_offset]

	// Add the payload offset to write correct number of quadwords for 
	// SOP case. Note that the rbuf to dram command ignores the last 3
	// bits and writes only quad words out.

	alu[payload_offset, buffer_offset, and, 0x7]
	alu[dram_write_size, payload_size, +, payload_offset]

	// Increment element address by the prepend 

	alu[element_offset, element_address, +, prepend_size]

	// Write cframe to DRAM (5)

	_rbuf_to_dram(dram_handle, element_offset, dram_write_size, rbuf_base, 
				  dram_write_signal)

.end
#endm


///////////////////////////////////////////////////////////////////////////////////////
//
// _csix_rx_wait_for_signals()
//
// Wait for signals in the second ME Max of ?? instructions
//
///////////////////////////////////////////////////////////////////////////////////////

#macro _csix_rx_wait_for_signals(sop_handle, eop_handle, packet_size, output_port, \
								  EOP_FLAG, FREE_RBUF_FLAG, rx_stat, return_label)
.begin

	.reg	element

	#if (EOP_FLAG == CSIX_RX_EOP_BIT)

		// Reset the local memory context 

		move(*l$index0++,0)
		move(*l$index0,0)

	#endif 

	// wait for signal 

	ctx_arb[--], defer[2]

	// Use signal mask 

	local_csr_wr[active_ctx_wakeup_events, sigmask]

	// Restore default signal mask 

	alu[sigmask, --, b, sigmask_default]

	// Inform assembler that these signals are done 

	.io_completed meta_write_signal dram_write_signal scratch_write_signal 
	.io_completed meta_write_signal2
	.io_completed buffer_drop_signal buffer_drop_signal2

	// Free RBUF element.

	#if (FREE_RBUF_FLAG == CSIX_RX_FREE_RBUF)

		alu_shf[element, --, B, element_address, <<10]
		msf[fast_wr, --, msf_rx_element_addr, element]

	#endif

	// Send next thread signal  

	_send_next_thrd_sig(next_thread_gpr)


#ifdef	MULTICAST_SUPPORT_ENABLE

	#if (!isnum(rx_stat) || (rx_stat != 0))

		br!=byte[rx_stat, 0, 0, mcast_pkt#]

	#endif

#endif	

	
	#if (EOP_FLAG == CSIX_RX_EOP_BIT)

		// write message to scratch ring and branch to the top of the loop (10)

		_csix_rx_sink(sop_handle, eop_handle, class_id, output_port, packet_size, $sink, \
					  scratch_write_signal, sigmask, rxc_context_ptr, CSIX_RING_NUM, start#)

	#else

		// remove the scratch write signal from the signal mask 

		alu[sigmask, sigmask, AND~, 1, <<&scratch_write_signal]

	#endif

	br [return_label]


#ifdef	MULTICAST_SUPPORT_ENABLE

	#if (!isnum(rx_stat) || (rx_stat != 0))

	mcast_pkt#:

		// for multicast packet, we will produce onto a different scratch ring.

		#if (EOP_FLAG == CSIX_RX_EOP_BIT)

			// write message to scratch ring and branch to the top of the loop (10)

			_csix_rx_sink(sop_handle, eop_handle, class_id, output_port, packet_size, $sink, \
						  scratch_write_signal, sigmask, rxc_context_ptr, CSIX_MCAST_RING_NUM, start#)

		#else

			// remove the scratch write signal from the signal mask 

			alu[sigmask, sigmask, AND~, 1, <<&scratch_write_signal]

		#endif

		br [return_label]

	#endif

#endif	// MULTICAST_SUPPORT_ENABLE

.end
#endm



///////////////////////////////////////////////////////////////////////////////////////

#macro csix_rx_second_me[]
.begin

	.reg		dram_handle
	.reg		voq
	.reg		sop_buffer_handle
	.reg		buf_size
	.reg		new_handle 
	.reg		packet_size


	xbuf_alloc($dl_meta, 7, write)

	xbuf_alloc($sink, 3, write)
	xbuf_alloc($drop, 3, write)		// Used to drop a buffer when scratch is full
	xbuf_alloc($drop2, 3, write)	// Used to drop a buffer containing an error packet

start#:

	// First check if a packet was to be dropped in the previous round

	alu[--, --, B, drop_sop_handle]
	bne[drop_previous_packet#]

receive_message#:
	
	// Receive message from previous microengine (12)

	_csix_rx_receive_from_previous_me(nn_ring_empty#)

	// Check if the error bit is set 

	br_bset[flags, CSIX_RX_DROP_BIT, drop_packet#]

	// Check if flags indicates SOP/EOP case 

	br!=byte[flags, 0, CSIX_RX_SOP_EOP_FLAGS, non_min_packet#]

min_packet#:

	// Write the rbuf to DRAM (11) 

	_csix_rx_write_to_dram(CSIX_RX_SOP_PREPEND_SIZE)

	// write SOP meta data to SRAM - first three long words (5)

	// write the rx_stat also. 
	dl_meta_write[$dl_meta, dl_buf_handle, IX_NULL, buffer_offset, payload_size, \
				  payload_size, FREE_LIST_ID, rx_stat, header_type, 0, 0, 0, 0, 0, \
				  0, 0, meta_write_signal, SIG_NONE, 0, 3]

	// Wait for signals and do scratch write and pass rx_stat

	_csix_rx_wait_for_signals(dl_buf_handle, IX_NULL, payload_size, port_or_sop_offset,\
							  CSIX_RX_EOP_BIT, CSIX_RX_FREE_RBUF, rx_stat, start#)


nn_ring_empty#:

	// Remove the DRAM write signal, meta write signal from the mask 

	alu[sigmask, sigmask, AND~, 3, <<&dram_write_signal]
	alu[sigmask, sigmask, AND~, 1, <<&meta_write_signal]

	// Wait for signals

	_csix_rx_wait_for_signals(IX_NULL, IX_NULL, 0, 0, CSIX_RX_NON_EOP_BIT, \
							  CSIX_RX_DONT_FREE_RBUF, 0, start#)

non_min_packet#:

	// Check if this is SOP case

	br=byte[flags, 0, CSIX_RX_SOP_ONLY_FLAG, sop_only#]

	// For MOP and EOP case, this field contains the sop offset 

	alu[sop_offset, --, B, port_or_sop_offset]

	// Check if the EOP bit is not set  

	br_bclr[flags, CSIX_RX_EOP_BIT, mop#]

	// For eop case, the handle field is used to pass the packet size 

	alu[packet_size, --, B, dl_buf_handle]

	// Get the current buffer handle 	
	
	alu[dl_buf_handle, *l$index0, AND~, 0xff, <<24]

	// Write the rbuf to DRAM (11) 

	_csix_rx_write_to_dram(CSIX_RX_MOP_EOP_PREPEND_SIZE)

	// Get the class id stored in bits 24..27 

	alu[class_id, 0xf, AND , *l$index0, >>24]


	// Get the output port stored in bits 28..31 of word0 and bits 24..29 of word1

	alu[output_port, output_port_mask, AND, *l$index0[1], >>20]
	alu[output_port, output_port, OR,  *l$index0, >>28]

#ifdef	MULTICAST_SUPPORT_ENABLE
	
	// store rx_stat also in bits 30:31. This will limit the number of output 
	// ports to 6 bits.

	alu_shf[rx_stat, --, B, *l$index0[1], >>30]

#endif

	// Get the sop buffer handle 

	alu[sop_buffer_handle, *l$index0[1], AND~, 0xff, <<24] 

	// Check if this is a single or a multi buffer packet 

	alu[--, sop_buffer_handle, -, dl_buf_handle]

	bne  [multi_buffer_eop#]

	// Single buffer eop - write SOP meta data to SRAM - first three long words 

	// write rx_stat also
	dl_meta_write[$dl_meta, sop_buffer_handle, IX_NULL, sop_offset, packet_size, 
				  packet_size, FREE_LIST_ID, rx_stat, header_type, 0, 0, 0, 0, 0, 
				  0, 0, meta_write_signal, SIG_NONE, 0, 3]


	// Set the SOP/EOP bits for buffer handle 

	ld_field[sop_buffer_handle, 1000, 0xc0, <<24]

	// Wait for signals and do scratch write (19)

	_csix_rx_wait_for_signals(sop_buffer_handle, dl_buf_handle, packet_size, output_port, \
							  CSIX_RX_EOP_BIT, CSIX_RX_FREE_RBUF, rx_stat, start#)


multi_buffer_eop#:

	// write SOP meta data to SRAM 

	dl_meta_write[$dl_meta, sop_buffer_handle, 0, 0, 0, packet_size, FREE_LIST_ID, rx_stat, header_type, 0, 
				  0, 0, 0, 0, 0, 0, meta_write_signal, SIG_NONE, 2, 1]

	// Compute size of eop buffer -- Assume multi-buffer packets always start at 
	// offset 0 

	alu[buf_size, buffer_offset, +, payload_size]
	alu[buf_size, buf_size, -, NON_SOP_BUFFER_OFFSET]

	// write meta data for eop buffer. Also assuming here that non-sop buffers 
	// start at offet 0 

	dl_meta_write[$dl_meta, dl_buf_handle, IX_NULL, NON_SOP_BUFFER_OFFSET, buf_size, 
				  0, FREE_LIST_ID, 0, 0, 0, 0, 0, 0, 0, 0, 0, meta_write_signal2, SIG_NONE, 0, 2]

	// Add the second signal to the mask 

	alu[sigmask, sigmask, OR, 1, <<&meta_write_signal2]

	// Set both SOP/EOP bits for aop and eop buffer handle. The bits are used only in cell
	// mode. In packet mode, EOP bit must be set.

	ld_field[sop_buffer_handle, 1000, 0xc0, <<24]
	ld_field[dl_buf_handle, 1000, 0xc0, <<24]

	// Wait for signals and do scratch write 

	_csix_rx_wait_for_signals(sop_buffer_handle, dl_buf_handle, packet_size, output_port, \
							  CSIX_RX_EOP_BIT, CSIX_RX_FREE_RBUF, rx_stat, start#)

drop_packet#:

	// Otherwise drop existing buffer in local memory. Note that the actual 
	// dropping must only be done after all the DRAM writes are complete on
	// the handle. So we wait for the next round. 
	
	// Get the current buffer handle 	
	
	alu[drop_eop_handle, *l$index0, AND~, 0xff, <<24] 

	// check if the handle is NULL

	beq[drop_done#]

	// Get the voq 

	alu[drop_voq, --, B, rxc_context_ptr, >>3]

	// Get the sop buffer handle 

	alu[drop_sop_handle, *l$index0[1], AND~, 0xff, <<24] 

	// Now clear the local memory buffer handles 

	move[*l$index0, 0]
	move[*l$index0[1], 0]

drop_done#:

	// Check if this is a min packet 

	br=byte[flags, 0, CSIX_RX_SOP_EOP_DROP_FLAGS, min_packet#]	

	// Check if this is an SOP m-packet 

	br_bset[flags, CSIX_RX_SOP_BIT, sop_only#]

	// Remove the DRAM write signal, meta write signal from the mask 

	alu[sigmask, sigmask, AND~, 3, <<&dram_write_signal]
	alu[sigmask, sigmask, AND~, 1, <<&meta_write_signal]

	// Otherwise just wait for signals 

	_csix_rx_wait_for_signals(IX_NULL, IX_NULL, 0, 0, CSIX_RX_NON_EOP_BIT, \
							  CSIX_RX_DONT_FREE_RBUF, 0, start#)

mop#:

	// Store the new buffer handle away 

	alu[new_handle, --, B, dl_buf_handle]

	// Get the current buffer handle 	
	
	alu[dl_buf_handle, *l$index0, AND~, 0xff, <<24]

	// Write the rbuf to DRAM (11) 

	_csix_rx_write_to_dram(CSIX_RX_MOP_EOP_PREPEND_SIZE)

	// Otherwise this is a MOP packet. Check if there is an end of buffer condition

	br_bclr[flags, CSIX_RX_EOB_BIT, mop_no_end_of_buffer#]

	// Now check if this is an SOP buffer. Get the sop buffer handle 

	alu[sop_buffer_handle, *l$index0[1], AND~, 0xff, <<24] 

	// Check if this is a single or a multi buffer packet 

	alu[--, sop_buffer_handle, -, dl_buf_handle]

	bne  [multi_buffer_mop#]

	// Compute size  of buffer 

	alu[buf_size, buffer_offset, +, payload_size]
	alu[buf_size, buf_size, -, sop_offset]

	// Write SOP meta data to SRAM - first two long words 

	dl_meta_write[$dl_meta, sop_buffer_handle, new_handle, sop_offset, buf_size, 
				  0, FREE_LIST_ID, 0, 0, 0, 0, 0, 0, 0, 
				  0, 0, meta_write_signal, SIG_NONE, 0, 2]

	// Now store the new buffer handle in LW0 

	ld_field[*l$index0, 0111, new_handle]

	// Wait for signals 

	_csix_rx_wait_for_signals(IX_NULL, IX_NULL, 0, 0, CSIX_RX_NON_EOP_BIT, \
							  CSIX_RX_FREE_RBUF, 0, start#)


multi_buffer_mop#:


	// Compute size  buffer -- Assume multi-buffer packets always start at 
	// offset 0 

	alu[buf_size, buffer_offset, +, payload_size]
	alu[buf_size, buf_size, -, NON_SOP_BUFFER_OFFSET]

	// write meta data for eop buffer. Also assuming here that non-sop buffers 
	// start at offet 0 

	dl_meta_write[$dl_meta, dl_buf_handle, new_handle, NON_SOP_BUFFER_OFFSET, buf_size, 
				  0, FREE_LIST_ID, 0, 0, 0, 0, 0, 0, 0, 0, 0, meta_write_signal, SIG_NONE, 0, 2]	 

	// Now store the new buffer handle in LW0 

	ld_field[*l$index0, 0111, new_handle]

	// Wait for signals 

	_csix_rx_wait_for_signals(IX_NULL, IX_NULL, 0, 0, CSIX_RX_NON_EOP_BIT, \
							  CSIX_RX_FREE_RBUF, 0, start#)

mop_no_end_of_buffer#:

	// No meta data write to wait on 

	alu[sigmask, sigmask, AND~, 1, <<&meta_write_signal]

	// Wait for signals and do scratch write 

	_csix_rx_wait_for_signals(IX_NULL, IX_NULL, 0, 0, CSIX_RX_NON_EOP_BIT, \
							  CSIX_RX_FREE_RBUF, 0, start#)


sop_only#:

	// Write the rbuf to DRAM (11) 

	_csix_rx_write_to_dram(CSIX_RX_SOP_PREPEND_SIZE)

	// Set output port 

	alu[output_port, --, B, port_or_sop_offset]

	// Store the sop handle, class id, output port (lsb 4 bits) in local memory LW0
	
	ld_field_w_clr[*l$index0, 0111, dl_buf_handle]
	ld_field_w_clr[*l$index0[1], 0111, dl_buf_handle]

	alu[*l$index0, *l$index0, OR, class_id, <<24]
	alu[*l$index0, *l$index0, OR, output_port, <<28]

#ifdef	MULTICAST_SUPPORT_ENABLE

	// store rx_stat in addition to the following. A limitation is that now
	// we have only 10 bits for output port as 2 bits are used for rx_stat
	// Store the output port (upper 8 bits) and  sop handle in LW1 

	alu[output_port, 0x3f, AND, output_port, >>4]
	alu[*l$index0[1], *l$index0[1], OR, output_port, <<24]
	alu[*l$index0[1], *l$index0[1], OR, rx_stat, <<30]

#else

	// Store the output port (upper 8 bits) and  sop handle in LW1 

	alu[output_port, --, B, output_port, >>4]
	alu[*l$index0[1], *l$index0[1], OR, output_port, <<24]


#endif

	// No meta data is written. So remove the signal from the mask 

	alu[sigmask, sigmask, AND~, 1, <<&meta_write_signal]

	// Wait for signals and do scratch write 

	_csix_rx_wait_for_signals(IX_NULL, IX_NULL, 0, 0, CSIX_RX_SOP_BIT, \
							  CSIX_RX_FREE_RBUF, 0, start#)

drop_previous_packet#:

	// Drop packet 

	_csix_rx_drop_packet(drop_sop_handle, drop_eop_handle, drop_voq, 
						 $drop2, sigmask, buffer_drop_signal2)

	// Reset the drop handles 

	alu[drop_sop_handle, --, B, 0]
	alu[drop_eop_handle, --, B, 0]

	// Go back to receiving message from previous microengine 

	br[receive_message#]
		
.end
#endm


//////////////////////////////////////////////////////////////////////////////////////
//
// _csix_rx_sink()
//
// Write message to scratch ring 
//
//////////////////////////////////////////////////////////////////////////////////////

#macro _csix_rx_sink (sop_handle, eop_handle, class_id, output_port, packet_size, \
					  $sink, scratch_write_signal, sigmask, rxc_context_ptr, 	  \
					  in_which_ring, return_label)

#if	(!isnum(in_which_ring))

#error	"Input ring must be a constant"

#endif

.begin

	.reg	temp 
	.reg	voq

	// check if the scratch ring is full 

#if	(in_which_ring == CSIX_RING_NUM)

	br_inp_state [CSIX_RING_FULL, csix_rx_sink_scr_full#]	; check ring fullness

#endif

#ifdef	MULTICAST_SUPPORT_ENABLE

#if	(in_which_ring == CSIX_MCAST_RING_NUM)

	br_inp_state [CSIX_MCAST_RING_FULL, csix_rx_sink_scr_full#]	; check ring fullness

#endif

#endif	// MULTICAST_SUPPORT_ENABLE


	// Pass class, output port and packet size for scheduler 
	
	alu[temp, class_id, or, output_port, <<CSIX_RX_BITS_FOR_VOQ_CLASS]		

	alu[$sink[0], temp, or, packet_size, <<16]	
	alu[$sink[1], --, B, sop_handle]						
	alu[$sink[2], --, B, eop_handle]		
	
	// Write to scratch ring

#if	(in_which_ring == CSIX_RING_NUM)

	scratch[put, $sink[0], 0, csix_rx_out_ring, 3], sig_done[scratch_write_signal]

#endif

#ifdef	MULTICAST_SUPPORT_ENABLE

#if	(in_which_ring == CSIX_MCAST_RING_NUM)

	scratch[put, $sink[0], 0, csix_rx_mcast_out_ring, 3], sig_done[scratch_write_signal]

#endif

#endif	// MULTICAST_SUPPORT_ENABLE


	// Increment debug counter

	_csix_rx_debug_incr_counter(@csix_rx_num_pkts_sink)

	// branch to the return label 

	br [return_label]

csix_rx_sink_scr_full#:

	// The scratch ring is full, the packet has to be dropped

	// Increment debug counter

	_csix_rx_debug_incr_counter(@csix_rx_num_scratch_ring_full)

	// Remove the scratch write signal from the mask 

	alu[sigmask, sigmask, AND~, 1, <<&scratch_write_signal]

	// Drop the current packet. First get the VoQ 

	alu[voq, --, B, rxc_context_ptr, >>3]

	_csix_rx_drop_packet(sop_handle, eop_handle, voq, 
						 $drop, sigmask, buffer_drop_signal)

	// Branch to the return label 

	br [return_label]

.end
#endm


/////////////////////////////////////////////////////////////////////////////////////
//
// _csix_rx_drop_packet()
//
// Drop a packet 
//
//////////////////////////////////////////////////////////////////////////////////////

#macro _csix_rx_drop_packet(sop_handle, eop_handle, voq, drop, sigmask, drop_signal)
.begin

	// Increment debug counter 

	_csix_rx_debug_incr_counter(@csix_rx_num_drop_pkts)

	// Increment dropped packet counter in SRAM for VOQ 

	#ifdef COUNTERS

		.reg addr									

		alu[addr, --, B, voq, <<4]					
		alu[addr, addr, +, counter_base]				
		sram[incr, --, addr, PKT_DROPPED_CNT_IDX]		

	#endif

	// Check if it is a single buffer packet 


	#if	(isnum(eop_handle))

		/* we get IX_NULL. This is a one buffer packet, simply drop it */

		// Drop a single buffer 
		dl_buf_drop(sop_handle)								
		br[drop_current_pkt_done#] 						

	#else
	
		/* eop_handle is not a number */

		alu[--, eop_handle, -, IX_NULL]

		bne[drop_buffer_chain#]

		// Drop a single buffer 
		dl_buf_drop(sop_handle)								
		br[drop_current_pkt_done#] 						


	drop_buffer_chain#:

		// Drop a buffer chain 

		dl_buf_drop_chain(sop_handle, eop_handle, drop, sigmask, drop_signal)

	#endif

drop_current_pkt_done#:

.end
#endm


////////////////////////////////////////////////////////////////////////////////////
//
// _csix_rx_increment_counters()
//
// Implements per VoQ byte and packet counters 
//
////////////////////////////////////////////////////////////////////////////////////

#macro _csix_rx_increment_counters(counter_base, voq, byte_counter, counter_sig, \
								   sigmask)

.begin

#ifdef COUNTERS

	.reg 	counter_address

	// Increment per VOQ counters for packets received and bytes received.
	// There are 4 counters per packet

	alu[counter_address, --, B, voq, <<4]					; get conter addr
	alu[counter_address, counter_address, +, counter_base]	; add base

	// Increment the packets received counter 

	sram[incr, --, counter_address, PKT_RECEIVED_CNT_IDX]	; increment pkt count

	// Increment the byte counter 

	sram[add, $byte_counter, counter_address, BYTE_RECEIVED_CNT_IDX],\
	     	sig_done[counter_sig]								; add byte count

	// Add Signal to the signal mask 

	alu[sigmask, sigmask, OR, 1, <<&counter_sig]

#endif

.end
#endm

//////////////////////////////////////////////////////////////////////////////////////
//
// _csix_rx_buffer_alloc
//
//	 	Description: 
//
//			Allocates a buffer either from nn ring or from sram. 
//
//////////////////////////////////////////////////////////////////////////////////////

#macro _csix_rx_buffer_alloc(free_list, req_sig, sig_action, sig_mask)
.begin

	#if (streq('sig_action', 'SIG_NONE'))

		; First check if a buffer freed by the Packet TX microblock and placed on
		; the nn ring is available. If available, it saves us a trip to sram

		br_inp_state[NN_EMPTY, default_buf_alloc#]

		; Buffer is available on nn-ring. Copy it into 'prefetch_handle_gpr'.

		alu[prefetch_handle_gpr, --, b, *n$index++]

continue#:

		; When we use this buffer, we do not know if it was fetched from nn ring or
		; from sram. This instruction saves us a branch that we would otherwise need.
		; to figure this out. Using this instruction, we insure that the prefetched
		; buffer handle is always in 'prefetch_handle_gpr'.
		; Before using 'prefetch_handle_gpr', we will do 
		; prefetch_handle_gpr = prefetch_handle_gpr xor $prefetch_handle
		; to get the buffer handle in 'prf_hndl_gpr'.

		alu[prefetch_handle_gpr, prefetch_handle_gpr, xor, $prefetch_handle]
		br[end#]

default_buf_alloc#:

		// Increment debug counter

		_csix_rx_debug_incr_counter(@csix_rx_num_sram_buf_alloc)

		; Buffer was not available on nn ring. Fetch it from sram.

		dl_buf_alloc($prefetch_handle, free_list, req_sig, sig_action)
		alu[sig_mask, sig_mask, or, 1, <<&req_sig]

		; When we use this buffer, we do not know if it was fetched from nn ring or
		; from sram. This instruction saves us a branch that we would otherwise need.
		; to figure this out. Using this instruction, we insure that the prefetched
		; buffer handle is always in 'prefetch_handle_gpr'.
		; Before using 'prf_hndl_gpr', we will do 
		; prf_hndl_gpr = prf_hndl_gpr xor $prf_hndl
		; to get the buffer handle in 'prf_hndl_gpr'.

		immed[prefetch_handle_gpr, 0]

end#:

	#else

		dl_buf_alloc($prefetch_handle, free_list, req_sig, sig_action)
		immed[prefetch_handle_gpr, 0]

	#endif

.end
#endm

////////////////////////////////////////////////////////////////////////////////////

// Main program starts here 

////////////////////////////////////////////////////////////////////////////////////

#ifdef CSIX_RX_FIRST_MICROENGINE 

main#:

	// Define registers used in the  microblock 

	.reg	rxc_context_ptr				; Byte pointer in local memory to context
	.reg	element_mask 				; Mask used to extract RBUF element
	.reg 	$prefetch_handle			; prefetch handle fetched from SRAM Q-Array
	.reg	prefetch_handle_gpr			; prefetch handle gpr register used with nn
	.reg 	freelist_reg				; Free list register
	.reg 	msf_freelist0_addr			; MSF freelist 0 address 
	.reg 	next_thread_gpr				; next thread 1 adddres to send signal
	.reg 	element_address				; RBUF element address 
	.reg 	flags						; SOP/EOP flags
	.reg 	ingress_blade_id			; Input blade number 
	.reg 	class_id					; Class ID in TM header
	.reg	ext_class_id				; Class ID in CSIX Extension Header 
	.reg 	error_code_mask				; Error code
	.reg 	voq_number					; virtual output queue
	.reg 	payload_size				; buffer size
	.reg	sigmask						; signal mask 
	.reg	sigmask_default				; default value for signal mask 
	.reg	rbuf_base					; base for rbufs
	.reg	buffer_default_offset		; default offset for packet buffers
	.reg	buffer_full_check_size		; size check for buffer full
	.reg	msf_rx_element_addr			; msf rx element address 

	#if (!IS_IXPTYPE(__IXP28XX))

	.reg 	@rand_buf_offset			; random buffer offset
	.reg 	rand_offset_mask			; mask for wrapping around

	#endif
	
	#ifdef	COUNTERS						

	.reg 	counter_base				; counter base gpr 
	.sig	counter_signal				; signal for reading counters

	#endif

	.reg rx_stat						; Rx status of C-frame

	// Define signals used in the microblock 

	.sig 	volatile 	msf_signal					; MSF read signal
	.sig 	volatile 	rx_event_signal				; MSF wake up signal
	.sig	volatile 	next_thread_signal 			; Next Thread Signal  
	.sig				meta_write_signal			; Meta Data Write Signal 
	.sig				prefetch_signal				; Buffer Prefetch Signal 
		
	#ifdef	_DEBUG_COUNTERS_

	// Declare some counters to ease debugging.

	.reg @csix_rx_num_valid_cells_recd	; valid c-frames received
	.reg @csix_rx_num_err_cells_recd	; rsw error c-frames received.
	.reg @csix_rx_num_seq_error_cells	; out-of-sequence c-frames received.
	.reg @csix_rx_num_buf_used			; buffers used (include prefetch)
	.reg @csix_rx_num_sram_buf_alloc	; buffers allocated from sram
	.reg @csix_rx_num_buf_alloc_fail	; buffer alloc failures (lack of buffers)

	#endif

	// Allocate transfer registers for Receive Status Word, Prepend, meta data write

	xbuf_alloc($rsw, 2, read_write)
	xbuf_alloc($prepend, 4, read)
	xbuf_alloc[$dl_meta, 7, write]

	// Initialize the CSIX Receive microblock. Also performs system initialization

	csix_rx_first_me_init()

system_init_done#:

	// Call the CSIX processing code 

	csix_rx_first_me ()

///////////////////////////////////////////////////////////////////////////////////////

#else 

///////////////////////////////////////////////////////////////////////////////////////

// CSIX RX Second Microengine 

main#:

	// Define registers used in the  microblock 

	.reg	rxc_context_ptr				; Byte pointer in local memory to context
	.reg 	next_thread_gpr				; next thread 1 adddres to send signal
	.reg 	element_address				; RBUF element address 
	.reg 	flags						; SOP/EOP flags
	.reg 	class_id					; Class ID in TM header.
	.reg 	buf_size					; buffer size
	.reg	sigmask						; signal mask 
	.reg	sigmask_default				; default value for signal mask 
	.reg	rbuf_base					; base for rbufs
	.reg	buffer_offset				; buffer offet 
	.reg	output_port					; output port 
	.reg	output_port_mask			; output port mask 
	.reg	sop_offset					; sop buffer offset
	.reg	port_or_sop_offset			; output port or sop offset 
	.reg	header_type					; header type 
	.reg	element_address_mask		; constant used to get element address
	.reg	rxc_context_ptr_mask		; constant used to get rxc address 
	.reg	ffff_mask					; constant 
	.reg	dram_rbuf_ov				; dram buffer overwrite constant 
	.reg	msf_rx_element_addr			; msf rx element address 
	.reg	payload_size				; size of rbuf payload 
	.reg	drop_voq					; voq for packet to be dropped
	.reg	drop_sop_handle				; sop handle for packet to be dropped
	.reg	drop_eop_handle				; eop handle for packet to be dropped 
	.reg	csix_rx_out_ring			; ring used to produce the message to next stage
	.reg	rx_stat						; RX Status word for the buffer


	// Define signals used in the microblock 

	.sig	volatile 	next_thread_signal 			; Next Thread Signal  
	.sig				meta_write_signal			; Meta Data Write Signal 
	.sig				meta_write_signal2			; Meta Data Write Signal2 for MOP
	.sig				dram_write_signal			; DRAM write Signal 
	.sig				scratch_write_signal		; Scratch write signal 
	.sig				buffer_drop_signal			; Buffer drop signal for scratch full
	.sig				buffer_drop_signal2			; Buffer drop signal 2 for error packets

	#ifdef	_DEBUG_COUNTERS_

	// Declare some counters to ease debugging.

	.reg @csix_rx_num_pkts_sink			; number of packets sent on outgoing scratch ring
	.reg @csix_rx_num_scratch_ring_full	; number of packets dropped because of scratch ring full
	.reg @csix_rx_num_drop_pkts			; number of packets dropped

	#endif

#ifdef	MULTICAST_SUPPORT_ENABLE

	.reg	csix_rx_mcast_out_ring		; ring onto which multicast packets are produced.

#endif

	// Initialize the CSIX Receive microblock. Also performs system initialization

	csix_rx_second_me_init()

	// Call the CSIX processing code 

	csix_rx_second_me()

#endif // #ifdef CSIX_RX_FIRST_ME 

//////////////////////////////////////////////////////////////////////////////////////

#endif	// __CSIX_RX_TWO_ME_UC__

//////////////////////////////////////////////////////////////////////////////////////
