/*
 *---------------------------------------------------------------------------
 *
 *                  I N T E L   P R O P R I E T A R Y
 *
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT
 *     THE PRIOR WRITTEN PERMISSION OF :
 *
 *                        INTEL  CORPORATION
 *
 *                     2200 MISSION COLLEGE BLVD
 *
 *               SANTA  CLARA,  CALIFORNIA  95052-8119
 *
 *---------------------------------------------------------------------------
 */

#ifndef __POS_RX_H__
#define __POS_RX_H__


#include <ixp.h>
#include <dl_system.h>
#include <sig_functions.h>
#include <dl_buf.c>
#include <dl_meta.c>
#include <dl_source.c>
#include <sig_functions.c>

//	Specifies what is the next block in the pipeline.
//	XXX - To be moved to dl_system.h

#ifdef BID_NEXT_BLOCK
#undef BID_NEXT_BLOCK
#define BID_NEXT_BLOCK			BID_POS
#endif


//---------------------------------------------------------------------
// MSF related constants.
//---------------------------------------------------------------------

//	Following is the definition of MSF register address.
//	XXX should goto ixphw.h file.

#define MSF_RX_CONTROL					0x0000
#define MSF_TX_CONTROL					0x0004
#define MSF_INTERRUPT_STATUS			0x0008
#define MSF_INTERRUPT_ENABLE			0x000c
#define CSIX_TYPE_MAP					0x0010
#define FC_EGRESS_STATUS				0x0014
#define FC_INGRESS_STATUS				0x0018			
#define RESERVED_1						0x001c
#define TX_VALIDATE						0x0020
#define HWM_CONTROL						0x0024
#define SRB_OVERRIDE					0x0028
#define RESERVED_2						0x002c
#define RX_THREAD_FREELIST_0			0x0030
#define RX_THREAD_FREELIST_1			0x0034
#define RX_THREAD_FREELIST_2			0x0038
#define RX_THREAD_FREELIST_3			0x003c 	// Not used in IXP2800
#define RX_PORT_MAP						0x0040 	// Not used in IXP2400	
#define RBUF_ELEMENT_DONE				0x0044
#define RX_MPHY_POLL_LIMIT				0x0048
#define FCEFIFO_VALIDATE				0x004c
#define RX_THREAD_FREELIST_TIMEOUT_0	0x0050
#define RX_THREAD_FREELIST_TIMEOUT_1	0x0054
#define RX_THREAD_FREELIST_TIMEOUT_2	0x0058
#define RX_THREAD_FREELIST_TIMEOUT_3	0x005c
#define TX_SEQUENCE_0					0x0060
#define TX_SEQUENCE_1					0x0064
#define TX_SEQUENCE_2					0x0068
#define TX_SEQUENCE_3					0x006c
#define TX_MPHY_POLL_LIMIT				0x0070
#define TX_MPHY_STATUS					0x0074
#define RESERVED_3						0x0078
#define RESERVED_4						0x007c
#define RX_UP_CTRL_0					0x0080
#define RX_UP_CTRL_1					0x0084
#define RX_UP_CTRL_2					0x0088
#define RX_UP_CTRL_3					0x008c
#define TX_UP_CTRL_0					0x0090
#define TX_UP_CTRL_1					0x0094
#define TX_UP_CTRL_2					0x0098
#define TX_UP_CTRL_3					0x009c
#define FCIFIFO							0x0100	// To 0x013c 16 address for burst access
#define FCEFIFO							0x0140	// TO 0x017c 16 address for burst access

#define RBUF_TBUF						0x2000	// To 0x3FFF Read=RBUF Write=TBUF

//	RX_THREAD_FREELIST register field bits definition.

#define SIGNAL_NUMBER_BITS				12		// 12-15 bits Signal number
#define ME_NUMBER_BITS					7		// 7-9 bits Microengine number
#define THREAD_BITS						4		// 4-6 bits Thread number

//---------------------------------------------------------------------
// MSF related user definitions.
//---------------------------------------------------------------------

// Thread time-out interval (in bus clocks).

#define RX_THREAD_FREELIST_TIMEOUT_0_DATA	1024

#define RX_UP_CTRL_0_DATA					0x39

#define MSF_RX_CONTROL_DATA					0xF0000004

//---------------------------------------------------------------------
// POS RX code related user definitions.
//---------------------------------------------------------------------

// Mask to wrap around at MAX_BUFFER_OFFSET (in dl_system.h)

#define	RAND_OFFSET_MASK		(MAX_BUFFER_OFFSET - 1)

// Maximum payload size is the buffer size (in dl_system.h).

#define MAX_PAYLOAD				BUFFER_SIZE

// User defined RBUF definitions.

#define	RBUF_TOTAL_BYTES		8192
#define RBUF_SIZE				128
#define RBUF_SIZE_IN_8B			16
#define	RBUF_TOTAL_ELEMENTS		(RBUF_TOTAL_BYTES / RBUF_SIZE)

// Defines the size of each RXC in longwords (32 bits).
#define RXC_SIZE				16

#define PO2_RBUF_SIZE			7
#define PO2_RXC_SIZE			6

// Per packet header size

#define	PER_PKT_PREPEND_BYTES	8				

// Cell size

#define CELL_SIZE				120  			

// Defines the maximum number of RXC's that can be present in the
// system at a given time.

#define TOTAL_PORTS				16

typedef enum {

	STATE_START =0,
	STATE_PROC

} RX_STATE;

//---------------------------------------------------------------------
// POS RX microc code related user definitions.
//---------------------------------------------------------------------

#if BIGENDIAN

// 	Defines a structure to represent the RSW in POS-PHY mode.
//	XXX - This should go to ixphw.h

typedef struct
{
 union
 {
  struct
  {
    unsigned int reserved05   : 16;
    unsigned int checksum     : 16;
  };
  struct
  {
    unsigned int reserved04   : 1;
    unsigned int element      : 7;
    unsigned int byteCount	  : 8;
    unsigned int sop          : 1;
    unsigned int eop          : 1;
    unsigned int err          : 1;
    unsigned int reserved03   : 1;
    unsigned int parErr       : 1;
    unsigned int rxErr        : 1;
    unsigned int null         : 1;
    unsigned int reserved02   : 1;
    unsigned int reserved01   : 4;
    unsigned int channel      : 4;
  };
  unsigned int value;
 };
} rsw_pos_phy_t;

#else	//!BIGENDIAN

// Defines a structure to represent the RSW in POS-PHY mode.
typedef struct
{
 union
 {
  struct
  {
    unsigned int channel      : 4;
    unsigned int reserved01   : 4;
    unsigned int reserved02   : 1;
    unsigned int null         : 1;
    unsigned int rxErr        : 1;
    unsigned int parErr       : 1;
    unsigned int reserved03   : 1;
    unsigned int err          : 1;
    unsigned int eop          : 1;
    unsigned int sop          : 1;
    unsigned int byteCount    : 8;
    unsigned int element      : 7;
    unsigned int reserved04   : 1;
  };
  struct
  {
    unsigned int checksum     : 16;
    unsigned int reserved05   : 16;
  };
  unsigned int value;
 };
} rsw_pos_phy_t;

#endif // BIGENDIAN

/**
******************************************************************************
* @ingroup Packer RX 
* Packet RX Context Data structure
*
* @description
*	This structure defines the reassembly Conext data. The size of this
*	context data should be power of 2.
*
******************************************************************************/

typedef struct
{
	int state;					/**< Status of Machine State */
	int pktDst;					/**< Next block the packet is destined to */
	int sopBufSize;				/**< amount of data in SOP Buffer */
	int prevBufSize;			/**< Size of previous Buffer */
	int curData;				/**< Pointer within a buffer where new data will be written */
	int bufSize;				/**< Size of Current Buffer */
	int bufOffset;				/**< Offset within buffer where actual data begins */
	int pktSize;				/**< Size of packet. Large Packet will be in many buffers */
	int errCount;				/**< Counter for errors */
	dl_buf_handle_t sopBuf;		/**< SOP Buffer handle */
	int cellCount;				/**< Number of cells in the current buffer */
	int bytesRemaining;			/**< Buffer size % cell size */ 
	dl_buf_handle_t prevBuf;	/**< Handle of Previous buffer in the chain */
	int sopBufOffset;			/**< Offset within SOP buffer where actual data begins */
	dl_buf_handle_t curBuf;		/**< Handle of Current buffer */
	int reserved;				/**< Reserved for future use */

} rxc_t;


#define NON_SOP_OFFSET		0 

//	XXX - Following defines should be moved to dl_system.h
#ifndef WESTPORT

#define			UC_NULL							0xFF		// NULL for microcode
#define			DL_BUF_NULL						UC_NULL 	// NULL Buffer representation
#define			BID_NULL						UC_NULL		// NULL Block ID
#define			BID_EXCEPTION					0x1			// Exception Block ID
#define			BID_DROP						0x2			// Drop Block ID

#endif

//	extern variables
// 	For details on these definitions, refer to files pos_rx.c, pos_rx_util.c.

extern volatile __declspec(sram_read_reg) rsw_pos_phy_t rsw[2];
extern __declspec(local_mem shared aligned(8)) rxc_t rxc[TOTAL_PORTS];
extern __declspec(sram_read_reg) dl_buf_handle_t prefetchBufHandle;
extern __declspec(gp_reg) int dlNextBlock;			
extern __declspec(gp_reg shared) unsigned int randBufOffset;

#if ((RX_PHY_MODE == SPHY_4_8) || (RX_PHY_MODE == MPHY_4))
extern __declspec(gp_reg) same_me_signal_t ntSigGpr;
#else
extern __declspec(gp_reg shared) same_me_signal_t ntSigGpr;
#endif


extern int rxFreeListReg;
extern int prefetchDataHandle;
extern int	dlRing;				
extern dl_buf_handle_t dlBufHandle;			
extern dl_buf_handle_t dlEopBufHandle;		
extern dl_meta_t dlMeta;
extern SIGNAL nextThreadSig;

//Rx Free List
extern int rxFreeList;

//	extern Fucntions

void PacketRxInit();
void PacketRx();

/*	State machine functions */

void Msp(int portNum, int eleAddr, unsigned int byteCount);
void Mpp(int portNum, int eleAddr, unsigned int byteCount);
void Esp(int portNum, int eleAddr, unsigned int byteCount);
void Epp(int portNum, int eleAddr, unsigned int byteCount);
void Ssp(int portNum, int eleAddr, unsigned int byteCount);
void Spp(int portNum, int eleAddr, unsigned int byteCount);
void Sesp(int portNum, int eleAddr, unsigned int byteCount);
void Sepp(int portNum, int eleAddr, unsigned int byteCount);

/*	Miscellaneous utility functions */

void FreeRbufElement(int);
void FreeAllRbufElements();
void AddThreadToFreelist();
void InitAllRxc();
int  GetRandBufOffset();
void MoveRbufToPktBuf(unsigned int, int, unsigned int);
void drop_current_pkt(dl_buf_handle_t sopBuf, dl_buf_handle_t curBuf);
int  SetFirstCellCount(__declspec(local_mem shared) int *pbytesRemaining, int byteCount);
void IncrCellCount(__declspec(local_mem shared) int *pcellCount, \
					 __declspec(local_mem shared) int *pbytesRemaining, \
					 int byteCount);

#endif // __POS_RX_H__