/*
 *---------------------------------------------------------------------------
 *
 *                  I N T E L   P R O P R I E T A R Y
 *
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT
 *     THE PRIOR WRITTEN PERMISSION OF :
 *
 *                        INTEL  CORPORATION
 *
 *                     2200 MISSION COLLEGE BLVD
 *
 *               SANTA  CLARA,  CALIFORNIA  95052-8119
 *
 *---------------------------------------------------------------------------
 */

#include "packet_rx.h"

extern void system_init();
extern SIGNAL sigRxEvent;

INLINE void 
FreeAllRbufElements()
{
	int elementNum;

	for(elementNum = 0; elementNum < RBUF_TOTAL_ELEMENTS; elementNum++)
		FreeRbufElement(elementNum);
}

INLINE void 
InitAllRxc()
{
	int i;
	
	for(i=0; i<TOTAL_PORTS; i++)
	{			
		rxc[i].state=STATE_START;
		rxc[i].pktDst=0;
		rxc[i].sopBufSize=0;
		rxc[i].prevBufSize=0;
		rxc[i].curData=0;
		rxc[i].bufSize=0;
		rxc[i].bufOffset=0;
		rxc[i].pktSize=0;
		rxc[i].errCount=0;
		rxc[i].sopBuf.value=0;
		rxc[i].cellCount=0;
		rxc[i].bytesRemaining=0;
		rxc[i].prevBuf.value=0;
		rxc[i].sopBufOffset=0;
		rxc[i].curBuf.value=0;
		rxc[i].reserved=0;
	}
}



INLINE void
ConfigMsfRxCtrl()
{

	SIGNAL	msfWrite;
	__declspec(sram_write_reg) int msfRxControlReg;

	//	[1:0]	Reserved
	//	[3:2]	01 ==> RBUF element size is 128
	//	[8:4]	Reserved
	//	[9]		0  ==> CSIX Free list - Data and control cframes goto 
	//			different Rx_thread_freelist. (Not used here)
	//	[12:10]	Reserved
	//	[13]	0  ==> Tx Cbus width is 4 bit
	//	[15:14]	Reserved
	//	[16]	0  ==> Tx CBus Mode is simplex mode
	//	[17]	RX Mphy related - Not used here. default value 0
	//	[18]	RX Mphy Mode: 0: MPHY-4, 1: MPHY-16. default value 0
	//	[19]	0  ==> RX Single Phy Mode, 1: Multi PHY Mode
	//	[21:20]	00 ==> RX width 1 x32, 1:2x16, 2: 4x8, 3:1x16_2x8
	//	[22]	0  ==> RX Mode is UTOPIA/POS mode
	//	[27:23]	Reserved
	//	[31:29]	0xF ==> Receive enable in all (0,1,2,3) channels.
	
	//	The above definitions are for IXP2400. It may differ for IXP2800. [TBD]

#ifdef	IXP2800

	//	The settings are slightly different for IXP2800.
	//	Current, this code supports only SPHY mode (implicit in code).
	//	Support for other modes on IXP2800 will be added later.

	msfRxControlReg = 0x40010004;

#else

	//	Depending on what PHY mode the settings are different.
	//	XXX - These #ifs seem not to be working. Preprocessor problems?

#if	(RX_PHY_MODE == SPHY_1_32)

	msfRxControlReg = 0xf0000004;

#elif (RX_PHY_MODE == SPHY_4_8)

	msfRxControlReg = 0xf0200004;

#elif (RX_PHY_MODE == MPHY_16)

	msfRxControlReg = 0x100E0004;

#else
#error "Unsupported RX_PHY_MODE"
#endif

#endif

	msf_write(&msfRxControlReg, MSF_RX_CONTROL, 1, ctx_swap, &msfWrite);

	__implicit_read(&msfWrite);

}


same_me_signal_t
PacketRxSetNextThreadSig(int thread, int sigval_next_ctx)
{
   int nextThread;
   same_me_signal_t tmpSigGpr;
    
    //	Based on RX_PHY_MODE, setup thread order
	//	0->1->0, 2->3->2, 4->5->4, 6->7->6
	//tmpSigGpr = sigval_next_ctx;
	if (0x1 & thread)     // Check LS Bit to find odd or even
	{
	   nextThread = thread - 1; //Odd: Next thread is one higher
	}
	else
	{
	   nextThread = thread + 1; //Even: Next thread is one lower
	}
	//Set next context bit to zero 
    tmpSigGpr.next_ctx = 0x0;
	//Set thread number to be signalled
	tmpSigGpr.thread = nextThread;
	//Set signal number
	tmpSigGpr.sig = sigval_next_ctx;
	return  tmpSigGpr;
}


INLINE void
SetRxUpCtrl()
{

	unsigned int rxUpCtrlData;

	//	[0]		- 1  ==> decode response time = 2 cycles. (XXX Why?)
	//	[1]		- x	 ==> don't care - 0.
	//	[3:2]	- 10 ==> single bit odd parity, 0x11: Single bit even parity
	//	[4]		- 1  ==> packet mode
	//	[5]		- 1  ==> POS-PHY Level 3 mode
	//	[31:6]	- x  ==> reserved


#ifndef	IXP_SIMULATION

	//	The Angel Island (IXDP2400) board doesn't support POS 3 Mode. So we just use
	//	POS 2 Mode.

	rxUpCtrlData = 0x1d;

#else

	//	On simulation we always use POS 3 Mode, because the Workbench has problems
	//	generating packets for POS 2 mode. This is until WB is fixed.

	rxUpCtrlData = 0x3d;

#endif

	//	Set it up for all four ports (max, when MPHY4 is used. MPHY16 needs only one)
	//	The actual number of ports enabled will depend of what RX_PHY_MODE we select
	//	(See ConfigMsfRxCtrl)

	// 	Note ms 16 bits is data, and ls 16 bits is address, i.e. RX_UP_CTRL_0.

	msf_fast_write((void *)(RX_UP_CTRL_0 | (rxUpCtrlData << 16)));
	msf_fast_write((void *)(RX_UP_CTRL_1 | (rxUpCtrlData << 16)));
	msf_fast_write((void *)(RX_UP_CTRL_2 | (rxUpCtrlData << 16)));
	msf_fast_write((void *)(RX_UP_CTRL_3 | (rxUpCtrlData << 16)));
}




INLINE void
PacketRxSetRxThreadFreelistTimeout(unsigned int timeout)
{

	//	Only SPHY_4_8 mode requires 4 thread free list. (even MPHY16 requires
	//	only 1 thread free list.). XXX What about MPHY4 Mode?

	// Note ms 16 bits is data and ls 16 bits is address, i.e. RX_THREAD_FREELIST_TIMEOUT_X.

	msf_fast_write((void *)(RX_THREAD_FREELIST_TIMEOUT_0 | (timeout << 16)));

#if ((RX_PHY_MODE == SPHY_4_8) || (RX_PHY_MODE == MPHY_4) )

	msf_fast_write((void *)(RX_THREAD_FREELIST_TIMEOUT_1 | (timeout << 16)));
	msf_fast_write((void *)(RX_THREAD_FREELIST_TIMEOUT_2 | (timeout << 16)));
	msf_fast_write((void *)(RX_THREAD_FREELIST_TIMEOUT_3 | (timeout << 16)));

#else

	//	For unused thread freelist set a timeout value of 0 (i.e do not wakeup at all)

	msf_fast_write((void *)(RX_THREAD_FREELIST_TIMEOUT_1));
	msf_fast_write((void *)(RX_THREAD_FREELIST_TIMEOUT_2));
	msf_fast_write((void *)(RX_THREAD_FREELIST_TIMEOUT_3));

#endif

}


void 
PacketRxInit()
{

	SIGNAL bufAllocSig;

	int thread  = GetCurrentCtx();		//	Current thread
	int this_me = GetCurrentMe();			//	Current Micro Engine

	// Initialize a gpr to signal next thread.
	// This is used only during initialisation.

	same_me_signal_t nextThreadSigGpr; 
    
	// 	Store the address of signal and xfer register used for receiving
	//	mpackets from MSF.

	int rxEventAddr 	= __signal_number(&sigRxEvent);
	int rsw0Addr 		= __xfer_reg_number(&(rsw[0]));

	// Initialize dispatch loop vaiables to known state.

	dlNextBlock 				= IX_NULL; 	//	NULL is 0xFF
	dlBufHandle.lw_offset	 	= IX_NULL;	//	NULL is 0xFF
	dlEopBufHandle.lw_offset 	= IX_NULL;	//	NULL is 0xFF


    // Nextthread signal: nextThreadSigGpr is global
	// 0->1->2->3....
	nextThreadSigGpr = InitInterthreadSig(__signal_number(&nextThreadSig));

	//	Global Initialisation (common to all threads) is done by Thread 0.
	if (thread == 0)
	{

		// Next thread signal: ntSigGpr is global
		ntSigGpr = InitInterthreadSig(__signal_number(&nextThreadSig));

		PacketRxSetRxThreadFreelistTimeout(RX_THREAD_FREELIST_TIMEOUT_0_DATA);

		FreeAllRbufElements();

		//	Some settings like POS/Utopia, Packet or cell mode, Parity etc
		//	are set here.

		SetRxUpCtrl();

		// Initialize all Receive Reassembly Context in LM.

		InitAllRxc();

		// Dispatch loop system initialization.

		system_init();						//	In system_init.c

		// Setup starting random buffer offset.

		randBufOffset = MAX_BUFFER_OFFSET;
	}
	else
	{
		//	All other threads wait for Thread 0 to complete Intialisation.

		wait_for_all(&nextThreadSig);	
	}

	// Ring number to be used in scratch[put].

	dlRing = DL_RING_NUMBER <<2;			//	XXX - to be renamed

	// Prefetch a buffer handle from SRAM.

	Dl_BufAlloc(&prefetchBufHandle, BUF_FREE_LIST0, BUF_POOL, &bufAllocSig, __signals(&bufAllocSig), ___);
	__implicit_read(&bufAllocSig);

	// Compute the buffer data address in DRAM of the prefetched buffer handle.

	prefetchDataHandle = Dl_BufGetData(prefetchBufHandle);

	// 	Set the freelist register for each thread.

	rxFreeListReg = rsw0Addr | (thread << THREAD_BITS) | (this_me << ME_NUMBER_BITS) | \
					  (rxEventAddr << SIGNAL_NUMBER_BITS);

    //Set Free List
#if ((RX_PHY_MODE == SPHY_4_8) || (RX_PHY_MODE == MPHY_4))
    if ((thread == 0) || (thread == 1))
	{
    	rxFreeList = RX_THREAD_FREELIST_0;
	}
	else if ((thread == 2) || (thread == 3))
	{
		rxFreeList = RX_THREAD_FREELIST_1;
	}
	else if ((thread == 4) || (thread == 5))
	{
		rxFreeList = RX_THREAD_FREELIST_2;
	}
	else 
	{
		rxFreeList = RX_THREAD_FREELIST_3;
	}
#else
    rxFreeList = RX_THREAD_FREELIST_0;
#endif 

	// 	Add the thread to the receive freelist.
	AddThreadToFreelist(rxFreeList, rxFreeListReg);

#if ((RX_PHY_MODE == SPHY_4_8) || (RX_PHY_MODE == MPHY_4))

    //Wake up next thread
    SignalNextCtx(nextThreadSigGpr);

	//Update signal register for 0->1->0, 2->3->2, 4->5->4, 6->7->6
	ntSigGpr = PacketRxSetNextThreadSig(thread, __signal_number(&nextThreadSig));
#endif

	

	if (thread == 7)
	{
		// 	Setup the MSF_RX_CONTROL CSR.
		// 	This enables receive hardware. As soon as it done, we'll start
		// 	receiving packets. Before that we want to make sure all other threads
		// 	are on the freelist and ready to receive. So we set this CSR in thread 7. 

		ConfigMsfRxCtrl();

	}
	else
	{
		// 	Signal next thread to do its initialization. Thread 7 does not need to 
		// 	signal its next thread (i.e thread 0) since thread 0 is already done 
		//	with initialization. (Thread 0 will subsequently wakeup on receiving
		// 	an mpacket (rx_sig_event))

#if( (RX_PHY_MODE == SPHY_4_8) || (RX_PHY_MODE == MPHY_4) )
	if ((thread==3) || (thread==5) || (thread==7))
	{
		SignalNextCtx(ntSigGpr);
    }
#else
	SignalNextCtx(ntSigGpr);
#endif

	}

	//	Wait for an RBUF element. This is one time only code.

	wait_for_all(&sigRxEvent);
}