/*
 *---------------------------------------------------------------------------
 *
 *                  I N T E L   P R O P R I E T A R Y
 *
 *     COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS
 *     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY
 *     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A
 *     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER
 *     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,
 *     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT
 *     THE PRIOR WRITTEN PERMISSION OF :
 *
 *                        INTEL  CORPORATION
 *
 *                     2200 MISSION COLLEGE BLVD
 *
 *               SANTA  CLARA,  CALIFORNIA  95052-8119
 *
 *---------------------------------------------------------------------------
 */

#include "packet_rx.h"

//	Because of some earlier problem with the compiler, we were using a global variable
//	for DRAM write signal. The problem has been fixed with the compiler. 
//	XXX - will ve cleaned up later.

SIGNAL_PAIR gSigDramWr;

INLINE void 
MoveRbufToPktBuf(unsigned int dramAddr, int elementAddr, unsigned int byteCount)
{
	dram_rbuf_tbuf_ind_t ind ;

	//	initialise to 0 to make all unused fields set 0.

	ind.value = 0;

	ind.ov_buf_addr = 1;
	ind.buf_addr = RBUF_TBUF + elementAddr;
	ind.ov_ref_count = 1;
	ind.ref_count = (byteCount - 1) >> 3;
	dram_rbuf_read_ind((volatile __declspec(dram) void *)dramAddr, 16, ind, sig_done, 
						&gSigDramWr);
}

INLINE void 
FreeRbufElement(int elementNum)
{
	// Do a fast write to the RBUF_ELEMENT_DONE CSR.
	// Note ms 16 bits is data, i.e. elementNum.
	// and ls 16 bits is address, i.e. RBUF_ELEMENT_DONE.

	msf_fast_write((void *)(RBUF_ELEMENT_DONE | (elementNum << 16)));
}

INLINE void 
AddThreadToFreelist(int rxFreeList, int rxFreeListReg)
{
	// Do a fast write to the RX_THREAD_FREELIST_ CSR. 
	// Note ms 16 bits is data, i.e. the freelist_reg.
	// and ls 16 bits is address.

	msf_fast_write((void *)(rxFreeList | (rxFreeListReg << 16)));
}


INLINE int 
GetRandBufOffset()
{
	// Possible values for randBufOffset : 128, 256, 384, 512

	randBufOffset = (randBufOffset & RAND_OFFSET_MASK) + DDR_BANK_SIZE;

	return randBufOffset;
}


INLINE int 
SetFirstCellCount(__declspec(local_mem shared) int *pBytesRemaining, int byteCount)
{
	int cellCount = 0;
	int remainder = byteCount;

	while (remainder > CELL_SIZE)
	{
		cellCount++;
		remainder -= CELL_SIZE;
	}

	*pBytesRemaining = remainder;
	return cellCount;
}

INLINE void 
IncrCellCount(__declspec(local_mem shared) int *pcell_count, __declspec(local_mem shared) int *pBytesRemaining, int byteCount)
{

	int bytesRemaining = *pBytesRemaining + byteCount;
	int cellCount		= *pcell_count;

	while (bytesRemaining > CELL_SIZE) 
	{
		cellCount++;
		bytesRemaining -= CELL_SIZE;
	}

	*pcell_count = cellCount;
	*pBytesRemaining = bytesRemaining;

}

INLINE void 
IncrCounter(int portNum, int counter)
{
#ifdef	RFC_PPP_COUNTERS

#error	"POS counters are not yet implemented"

#endif

}

INLINE void 
DropCurrentPkt(dl_buf_handle_t sopBuf, dl_buf_handle_t curBuf)
{

	//	There are two conditions to handle
	//	1. Only a single buffer needs to be dropped.
	//	2. A buffer chain needs to be dropped. (large packet)
	//
	//	In case 1, sop_handle = curr_handle and both will have the 
	//	SOP bit set. (This is a little peculiar but it makes life
	//	easier elsewhere)
	//
	//	In case 2, sop_handle != curr_handle and only sop_handle
	//	will have the SOP bit set.


	if (curBuf.sop) 
		Dl_BufDrop(curBuf);
	else
		Dl_BufDropChain(sopBuf, curBuf);

}

INLINE void 
Msp(int portNum, int eleAddr, unsigned int byteCount)
{

	// Yes, the parameters are not used. 

	(void) portNum;
	(void) eleAddr;
	(void) byteCount;

	dlNextBlock = BID_NULL;
	wait_for_all(&nextThreadSig);

}

INLINE void 
Esp(int portNum, int eleAddr, unsigned int byteCount)
{

	// Yes, the parameters are not used. 

	(void) portNum;
	(void) eleAddr;
	(void) byteCount;

	dlNextBlock = BID_NULL;
	wait_for_all(&nextThreadSig);

}

INLINE void 
Spp(int portNum, int eleAddr, unsigned int byteCount)
{
	__declspec(local_mem shared) rxc_t *prxc = &rxc[portNum];

	//	Got SOP mpacket while expecting MOP/EOP -  Error case

	//	We do two things. 
	//	[1] We discard the partially (so far) assembled packet.
	//	[2] And since SOP is a new packet we can process it.

	//	[1] Reset State Machine

	prxc->state = STATE_START;

	//	Do the SOP processing for the newly received mpacket

	Ssp(portNum, eleAddr, byteCount);

	//	drop the partially assembled pkt
	//	Prepare to drop the packet. For large packets 
	//	curBuf will be the last buffer in the chain
	//	XXX - Why can't drop packet be done before ssp_processing? .

	DropCurrentPkt(prxc->sopBuf, prxc->curBuf);


}


void 
Sepp(int portNum, int elementAddr, unsigned int byteCount)
{
	__declspec(local_mem shared) rxc_t *prxc = &rxc[portNum];

	//	Got a SOP&EOP (both in same mpacket) while expecting MOP/EOP mpacket - Error
	
	//	We do two things. 
	//	[1] We discard the partially (so far) assembled packet.
	//	[2] And since SOP&EOP is a new but complete packet we can process it.
		
	//	[1] Reset State Machine

	prxc->state = STATE_START;
	 
	//	[2]	Process SOP&EOP mpacket

	Sesp(portNum, elementAddr, byteCount);

	//	drop the partially assembled pkt. For large packets 
	//	curBuf will be the last buffer in the chain
	//	XXX - Why can't drop packet done befor sesp_processing?

	DropCurrentPkt(prxc->sopBuf, prxc->curBuf);
}


INLINE void 
Sesp(int portNum, int elementAddr, unsigned int byteCount)
{
	SIGNAL bufAllocSig;

	// Null buffer.

	if (prefetchBufHandle.lw_offset == 0)
	{

		dlNextBlock = IX_NULL;

		// Prefetch a buffer handle from SRAM for use in next iteration.

		Dl_BufAlloc(&prefetchBufHandle, BUF_FREE_LIST0, BUF_POOL, &bufAllocSig, SIG_NONE, ___);

		wait_for_all(&nextThreadSig, &bufAllocSig);

		__implicit_read(&bufAllocSig);

		// Compute the buffer data address in DRAM of the prefetched buffer handle.

		prefetchDataHandle = Dl_BufGetData(prefetchBufHandle);
	}
	else 
	{
		SIGNAL sigMeta;
		SIGNAL sigMsfRd;
		SIGNAL_PAIR sigDram;

		int protocol;
		int bytesRemaining;

		int bufOffset 		= GetRandBufOffset();
		int dramAddr 		= prefetchDataHandle + bufOffset;
		int elementAddr 	= rsw[0].element << PO2_RBUF_SIZE;

		//	XXX - Now that the compiler is fixed, we should use move_rbuf_to_dram
		//	function here.

		dram_rbuf_tbuf_ind_t ind;

		//	initialise to 0 to make all unused fields set 0.

		ind.value			= 0;

		ind.ov_buf_addr 	= 1;
		ind.buf_addr 		= RBUF_TBUF + elementAddr;
		ind.ov_ref_count 	= 1;
		ind.ref_count 		= (byteCount - 1) >> 3;

		dram_rbuf_read_ind((volatile __declspec(dram) void *)dramAddr, 16, ind, sig_done, &sigDram);

		// Set the address, seg_count, sop_flag and eop_flag fields of dlBufHandle.

		dlBufHandle = prefetchBufHandle;

		// Find out if the received mpacket can be contained in 1 cell 
		// (on the transmit side). Remember that for every packet there is
		// a header (8 bytes) that gets prepended. (This is apart from the per 
		// cell 8 byte header that gets added) This per pkt header is added
		// on the fly. Hence it is not counted in the buffer/packet size.

		if (rsw[0].byteCount > (CELL_SIZE - PER_PKT_PREPEND_BYTES))
			dlBufHandle.seg_count = SetFirstCellCount((__declspec(local_mem shared) int *)&bytesRemaining, rsw[0].byteCount);

		//	Update the meta data.

		dlEopBufHandle.lw_offset 	= IX_NULL;
		dlMeta.bufferNext.value		= 0;
		dlMeta.bufferSize 			= rsw[0].byteCount;
		dlMeta.offset 				= bufOffset;
		dlMeta.freeListId 			= 0;
		dlMeta.rxStat 				= 0;
		dlMeta.packetSize 			= byteCount;
		dlMeta.inputPort 			= portNum;

		// Flush the meta data to SRAM. Only the first three LW's (0, 1, 2) 
		// of the meta data are flushed. The rest of the meta data are not 
		// touched by POS RX.

		dl_meta_flush_cache(dlBufHandle, &sigMeta, SIG_NONE, 0, 2);

		// Prefetch a buffer handle from SRAM for use in next iteration.

		Dl_BufAlloc(&prefetchBufHandle, BUF_FREE_LIST0, BUF_POOL, &bufAllocSig, 0, ___);

		// Wait on the all the previous I/O plus the next thread signal.

		wait_for_all(&nextThreadSig, &sigMeta, &bufAllocSig, &sigDram);

		__implicit_read(&bufAllocSig);
		__implicit_read(&sigMeta);
		__implicit_read(&sigDram);

		dlNextBlock = PACKET_RX_NEXT1;

		// Compute the buffer data address in DRAM of the prefetched buffer handle.

		prefetchDataHandle = Dl_BufGetData(prefetchBufHandle);
	}
}


void 
Ssp(int portNum, int elementAddr, unsigned int byteCount)
{

	SIGNAL bufAllocSig;

	// Null buffer.
	if (prefetchBufHandle.lw_offset == 0)
	{
		dlNextBlock = IX_NULL;

		// Prefetch a buffer handle from SRAM for use in next iteration.

		Dl_BufAlloc(&prefetchBufHandle, BUF_FREE_LIST0, BUF_POOL, &bufAllocSig, SIG_NONE, ___);

		wait_for_all(&nextThreadSig, &bufAllocSig);

		__implicit_read(&bufAllocSig);

		// Compute the buffer data address in DRAM of the prefetched buffer handle.

		prefetchDataHandle = Dl_BufGetData(prefetchBufHandle);
	}
	else 
	{

		SIGNAL 		sigMsfRd;
		SIGNAL_PAIR sigDram;

		__declspec(local_mem shared) rxc_t *prxc = &rxc[portNum];

		int protocol;
		int bufOffset 		= GetRandBufOffset();
		int elementAddr	= rsw[0].element << PO2_RBUF_SIZE;

		prxc->curData = bufOffset + prefetchDataHandle;

		MoveRbufToPktBuf(prxc->curData, elementAddr, byteCount);

		prxc->sopBuf 			= prefetchBufHandle;
		prxc->curBuf 			= prefetchBufHandle;
		prxc->sopBufOffset	 	= bufOffset;
		prxc->bufOffset 		= bufOffset;
		prxc->bufSize 			= byteCount;
		prxc->pktSize 			= byteCount;
		prxc->state 			= STATE_PROC;
		prxc->curData 			+= byteCount;

		//	Cell Count Calculation.
		//	On the transmit side, for every packet we add a prepend hdr
		//	(apart from a cell hdr for every cell). Include this hdr in
		//	cell count calculation. (For a given packet, we'll hit SOP
		//	condition only once so this place fits well for this 
		//	work)

		prxc->cellCount = SetFirstCellCount(&(prxc->bytesRemaining), byteCount+PER_PKT_PREPEND_BYTES);


		//	Prefetch a new buffer so that it can be used next time around.

		Dl_BufAlloc(&prefetchBufHandle, BUF_FREE_LIST0, BUF_POOL, &bufAllocSig, SIG_NONE, ___);

		dlNextBlock = BID_NULL;

		// Wait on the all the previous I/O plus the next thread signal.

		wait_for_all(&nextThreadSig, &bufAllocSig, &gSigDramWr);
		__implicit_read(&bufAllocSig);
		__implicit_read(&gSigDramWr);

		prxc->pktDst = PACKET_RX_NEXT1;

		// Compute the buffer data address in DRAM of the prefetched buffer handle.

		prefetchDataHandle = Dl_BufGetData(prefetchBufHandle);

	}	// end of else

}

INLINE void 
Epp(int portNum, int elementAddr, unsigned int byteCount)
{

	SIGNAL		sigCurMeta;
	SIGNAL		sigPrevMeta;
	SIGNAL_PAIR sigDram;

	__declspec(local_mem shared) rxc_t *prxc = &rxc[portNum];


	MoveRbufToPktBuf(prxc->curData, elementAddr, byteCount);

	IncrCellCount(&prxc->cellCount, &prxc->bytesRemaining, byteCount);

	//	Update context.

	prxc->bufSize += byteCount;
	prxc->pktSize += byteCount;
	prxc->state 	= STATE_START;

	prxc->curBuf.seg_count = prxc->cellCount;

	if (prxc->curBuf.sop) 	
	{

		//	This buffer has SOP. i.e this buffer now contains
		//	a complete packet (SOP ...EOP). So we need to handle only
		//	a single buffer. Set all META data fields and write the 
		//	meta data to SRAM.
		
		dlMeta.offset 			= prxc->bufOffset;
		dlMeta.bufferNext.value = 0;
		dlMeta.bufferSize 		= prxc->bufSize;
		dlMeta.packetSize 		= prxc->pktSize;
		dlMeta.inputPort  		= portNum;

		dl_meta_flush_cache(prxc->curBuf, &sigCurMeta, sig_done, 0, 2);


		//	We are done with this packet. Prepare to send it to
		//	Next block in the pipeline. dlBufHandle should contain 
		//	the SOP buffer handle, dlEopBufHandle is not applicable
		//	because the entire packet is in a single buffer and dlNextBlock
		//	is IPv4 block (based on the protocol info in PPP hdr)

		//	dlNextBlock can be set only in Phase2 here because of a possible race
		//	condition. see the end of this functions

		dlEopBufHandle.value 	= DL_BUF_NULL;
		dlBufHandle 			= prxc->curBuf;

		//	End of Phase 1.
		//	Wait on the all the previous I/O plus the next thread signal.

		wait_for_all(&nextThreadSig, &gSigDramWr, &sigCurMeta);

		__implicit_read(&sigCurMeta);
		__implicit_read(&gSigDramWr);
	}
	else 	
	{
		unsigned int sopBufOffset;
		unsigned int sopBufSize;
		unsigned int pktSize;

		__declspec(sram_write_reg) dl_meta_t prevMeta;
		__declspec(sram_write_reg) dl_meta_t curMeta;

		//	At this point, EOP and SOP are in different buffers i.e large packet.
		//  So we need to write previous buffer's meta data & current buffer's (EOP) 
		//	meta data to SRAM.
		//	There are two cases to handle.

		//	[1] Previous Buffer is also SOP Buffer.
		//	[2] Previous Buffer is not SOP Buffer.

		//	compose meta data for the previous buffer.

		//	Check if the previous buffer is a SOP buffer. If it is not a SOP buffer,
		//	the offset is set as NON_SOP_OFFSET(0) i.e no headroom.
		//	If it is a SOP buffer we have stored the offset in rxc. Use that.

		dlMeta.bufferNext	= prxc->curBuf;
		dlMeta.bufferSize	= prxc->prevBufSize;

		if (prxc->prevBuf.sop)
			dlMeta.offset 	= prxc->sopBufOffset;
		else
			dlMeta.offset	= NON_SOP_OFFSET;

		//	Write META data of previous buffer to SRAM

		Dl_MetaFlushCache2(prxc->prevBuf, &prevMeta, &sigPrevMeta, sig_done, 0, 2);

		//	Set current buffer meta data

		dlMeta.bufferNext.value 	= 0;
		dlMeta.offset				= NON_SOP_OFFSET;
		dlMeta.bufferSize	 		= prxc->bufSize;
			
		//	Write META data of current buffer to SRAM

		Dl_MetaFlushCache2(prxc->curBuf, &curMeta, &sigCurMeta, sig_done, 0, 2);


		//	We are done with this packet. Prepare to send it to
		//	Next block in the pipeline. dlBufHandle should contain 
		//	the SOP buffer handle, dlEopBufHandle should contain EOP buffer
		//	and dlNextBlock should be POS or Ethernet (Packet RX is protocol
		//	agnostic)
 
		//	dlNextBlock can be set only in Phase2 here because of a possible race
		//	condition. see [end#]

		dlBufHandle 	= prxc->sopBuf;
		dlEopBufHandle 	= prxc->curBuf;


		//	Since this is EOP, after a context swap, the contents of local memory could 
		//	be updated by the next thread for a new SOP. So locally save whatever we want
		//	to use after the ctx swap. 

		//	XXX - The compiler optmisies the next 3 instrcutions and uses prxc-> dircectly 
		//	after the context swap there by defeating our purpose. Making these variables
		//	volatile doesn't seem to have any effect. So we do some operations 
		//	that'll avoid this optimisation. Compiler to be fixed.
		 
		sopBufOffset 	= prxc->sopBufOffset-1;
		sopBufSize 		= prxc->sopBufSize-1;
		pktSize			= prxc->pktSize-1;

		//	End of Phase 1.
		//	Wait on the all the previous I/O plus the next thread signal.

		wait_for_all(&nextThreadSig, &sigPrevMeta, &sigCurMeta, &gSigDramWr);

		__implicit_read(&sigCurMeta);
		__implicit_read(&sigPrevMeta);
		__implicit_read(&gSigDramWr);
		__implicit_read((void *) &curMeta);
		__implicit_read((void *) &prevMeta);


		//	The way packets are sent to next block in the pipeline is thro'
		//	dl_sink. (see the outer loop in pos_rx.uc). 
		//	As part of optimisation, two LWs ( 1 & 2) from meta data are sent 
		//	in the scratch ring. dl_sink takes these meta data from the 
		//	variable dlMeta . The values should belong to SOP buffer.

		//	In our case the last dlMeta.* belong to the EOP buffer. So we
		//	need to do a few dlMeta.* settings for SOP buffer here.

		dlMeta.offset 		= sopBufOffset+1;
		dlMeta.bufferSize 	= sopBufSize+1;
		dlMeta.inputPort 	= portNum;
		dlMeta.packetSize 	= pktSize+1;

	}

	//	prxc->pktDst is set in Phase2 in spp_processing. So the value in 
	//	prxc->pktDst is available only in epp Phase2. Be careful about this.
	//	XXX - possibly the contents of locally mem (prxc) is gone by now (next thread
	//	could have updated it for next SOP???).

	dlNextBlock = prxc->pktDst;
}


void 
Mpp(int portNum, int elementAddr, unsigned int byteCount)
{
	int bufSize;
	SIGNAL 			bufAllocSig;
	SIGNAL			sigMeta;
	SIGNAL_PAIR		sigDram;

	__declspec(local_mem shared) rxc_t *prxc = &rxc[portNum];

	//	Increment buffer size by number of bytes received. Buffer size
	//	is the amount of the data currently present in the buffer. We are
	//	guaranteed to have room in the buffer for this mpacket.
	//	[ See the defer slots below]

	prxc->bufSize += byteCount;
	prxc->pktSize += byteCount;
	
	//	Find out if buffer is full (for next mpacket). 
	//	Buffer size + headroom in the buffer will show 
	//	if the buffer is full.

	bufSize = prxc->bufSize + prxc->bufOffset;

	if (bufSize < MAX_PAYLOAD)
	{
		//	Current buffer is not FULL. (i.e it can take current 
		//	mpacket + atleast one more mpacket)

		MoveRbufToPktBuf(prxc->curData, elementAddr, byteCount);

		//	increment cell count depending on the number of bytes received.
		//	embed this cell count in the buffer handle.

		IncrCellCount(&prxc->cellCount, &prxc->bytesRemaining, byteCount);

		prxc->curData += byteCount;

		//	Remember that we have received only a part of the packet so far.
		//	Until we receive EOP, dlNextBlock should be NULL.

		dlNextBlock = BID_NULL;

		//	End of Phase 1.
		//	Wait on the all the previous I/O plus the next thread signal.

		wait_for_all(&nextThreadSig, &gSigDramWr);

		__implicit_read(&gSigDramWr);

	}
	else 
	{
		// 	Buffer full.

		// 	See if a free buffer is available for next time around...

		if (prefetchBufHandle.lw_offset == 0)
		{
			prxc->state = STATE_START;
	
			Dl_BufAlloc(&prefetchBufHandle, BUF_FREE_LIST0, BUF_POOL, &bufAllocSig, SIG_NONE, ___);
			wait_for_all(&nextThreadSig, &bufAllocSig);

			__implicit_read(&bufAllocSig);

			// Compute the buffer data address in DRAM of the prefetched buffer handle.

			prefetchDataHandle = Dl_BufGetData(prefetchBufHandle);
	
			DropCurrentPkt(prxc->sopBuf, prxc->curBuf);
			prxc->errCount++;
			dlNextBlock = BID_NULL;

		}
		else
		{
			//	Yes, free buffer is available

			//	Move rbuf to DRAM

			MoveRbufToPktBuf(prxc->curData, elementAddr, byteCount);

			//	increment cell count depending on the number of bytes received.
			//	embed this cell count in the buffer handle.

			IncrCellCount(&prxc->cellCount, &prxc->bytesRemaining, byteCount);
			prxc->curBuf.seg_count = prxc->cellCount;

			prxc->curBuf.eop = 0;
			if (prxc->curBuf.sop)
			{
				//	[Recap] This buffer is full. This buffer contains SOP. 
				//	A new buffer is available.
		
				//	At this point, no meta data needs to be flushed to SRAM
				//	keep the SOP buffer offset, buffer size and move current
				//	buffer handle to the previous buffer handle in the context data

				prxc->sopBuf 		= prxc->curBuf;
				prxc->prevBuf 		= prxc->curBuf;
				prxc->prevBufSize 	= prxc->bufSize;
				prxc->sopBufSize 	= prxc->bufSize;

				prxc->cellCount 		= 0;
				prxc->bytesRemaining	= 0;
				prxc->bufSize 			= 0;
				prxc->bufOffset 		= NON_SOP_OFFSET;

				prxc->curBuf 		= prefetchBufHandle;
				prxc->curBuf.sop 	= 0;
				prxc->curData 		= prefetchDataHandle;

				//	Prefetch a new buffer so that it can be used next time around.

				Dl_BufAlloc(&prefetchBufHandle, BUF_FREE_LIST0, BUF_POOL, &bufAllocSig, SIG_NONE, ___);
				wait_for_all(&nextThreadSig, &bufAllocSig, &gSigDramWr);

				__implicit_read(&bufAllocSig);
				__implicit_read(&gSigDramWr);

				// Compute the buffer data address in DRAM of the prefetched buffer handle.

				prefetchDataHandle = Dl_BufGetData(prefetchBufHandle);

				dlNextBlock = BID_NULL;
			}
			else 
			{

				//	[Recap] This buffer is full. This buffer does not 
				//	contain SOP. A new buffer is available.

				//	Two slightly different cases to handle here. 
				//	[1] Previous Buffer is SOP. (this will have headroom)
				//	[2] Previous Buffer is not SOP. (this will not)

				//	Now we are ready to flush META data of previous buffer to SRAM.
				//	Remember the META data contains a hw_next pointer which, in this case, 
				//	should point to current buffer. 

				dlMeta.bufferNext = prxc->curBuf;
				dlMeta.bufferSize = prxc->prevBufSize;

				if (prxc->prevBuf.sop)
					dlMeta.offset	= prxc->sopBufOffset;
				else
					dlMeta.offset 	= NON_SOP_OFFSET;


				//	All the required fields of META data have been set
				//	for the previous buffer. Write it to SRAM. The META
				//	Data for !SOP buffers is only a subset of SOP META
				//	and contains only the first 2 longwords of META.

				dl_meta_flush_cache(prxc->prevBuf, &sigMeta, sig_done, 0, 2);

				//	Ok, prepare for our next trip down this lane.

				//	We are done with current buffer. Make it previous buffer 

				prxc->prevBuf 		= prxc->curBuf;
				prxc->prevBufSize	= prxc->bufSize;
					
				prxc->cellCount 	= 0;
				prxc->bufSize 		= 0;
				prxc->bufOffset 	= NON_SOP_OFFSET;
				prxc->bytesRemaining = 0;

				//	Ok, now make the prefetch buffer as current buffer. Any new
				//	mpacket received will be stored in this new buffer.

				prxc->curBuf  = prefetchBufHandle;
				prxc->curBuf.sop = 0;
				prxc->curData = prefetchDataHandle;

				//	Prefetch a new buffer so that it can be used next time around.

				Dl_BufAlloc(&prefetchBufHandle, BUF_FREE_LIST0, BUF_POOL, &bufAllocSig, SIG_NONE, ___);
				wait_for_all(&nextThreadSig, &bufAllocSig, &sigMeta, &gSigDramWr);

				__implicit_read(&bufAllocSig);
				__implicit_read(&sigMeta);
				__implicit_read(&gSigDramWr);

				// Compute the buffer data address in DRAM of the prefetched buffer handle.

				prefetchDataHandle = Dl_BufGetData(prefetchBufHandle);

				dlNextBlock = BID_NULL;
			}	//	Current buffer is not sop

		}	// 	Free buffer is available
	}	//	Buffer full

}
