/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      ATM TX Core Component
 *
 * = FILENAME
 *      ix_cc_atmtx_private.h
 *
 * = DESCRIPTION
 *
 * = AUTHOR
 *      Jaroslaw Kogut
 *      jaroslaw.kogut@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * 
 */

#if !defined(__IX_CC_ATMTX_PRIVATE_H__)
#define __IX_CC_ATMTX_PRIVATE_H__

#include "cc/ix_cc_atmsar.h"
#include "aal5_tx_defs.h"
#include "dl_system.h"

#if defined(__cplusplus)
extern "C" {
#endif /* end defined(__cplusplus) */


/**
 * TYPENAME: _ix_cc_atmtx_symbol_id_t
 * 
 * DESCRIPTION:
 *    This enum defines number of symbols serviced by the core component
 */
typedef enum atmtx_e_symbol_id
{
    _IX_CC_ATMTX_AAL5_TX_CTX_SRAM_BASE_SYMBOL_ID=0,
    _IX_CC_ATMTX_AAL5_TX_PKTS_PER_VCQ_COUNTER_BASE_SYMBOL_ID,
    _IX_CC_ATMTX_AAL5_TX_CELLS_PER_VCQ_COUNTER_BASE_SYMBOL_ID,
    _IX_CC_ATMTX_AAL5_TX_PKTS_PER_OUTPORT_COUNTER_BASE_SYMBOL_ID,
    _IX_CC_ATMTX_AAL5_TX_CELLS_PER_OUTPORT_COUNTER_BASE_SYMBOL_ID,
    _IX_CC_ATMTX_AAL5_TX_CELLS_TX_SCRATCH_BASE_SYMBOL_ID,
    _IX_CC_ATMTX_AAL5_TX_AVAIL_TBUF_ELEMENT_SCRATCH_BASE_SYMBOL_ID,
    _IX_CC_ATMTX_MAX_SYMBOL_ID
} _ix_cc_atmtx_symbol_id_t;


/* size of buffer for prefix name for symbol patched into microcode */
#define _IX_CC_ATMTX_SYMBOL_PREFIX_SIZE 20

/**
 * TYPENAME: _ix_cc_atmtx_symbol_type_t
 * 
 * DESCRIPTION:
 *    This enum defines the symbol types.
 */
typedef enum _ix_cc_atmtx_e_symbol_type
{
    _IX_CC_ATMTX_AAL5_PORT_SYMBOL_TYPE=0,
    _IX_CC_ATMTX_AAL5_VC_SYMBOL_TYPE,
    _IX_CC_ATMTX_AAL5_SINGLE_SYMBOL_TYPE,    

#ifdef AAL2_INCLUDED

    _IX_CC_ATMTX_AAL2_CID_SYMBOL_TYPE,

#endif

    _IX_CC_ATMTX_SYMBOL_TYPE_LAST

} _ix_cc_atmtx_e_symbol_type_t;


/**
 * TYPENAME: _ix_cc_atmtx_symbol_descr_t
 * 
 * DESCRIPTION:
 *    ME symbol description: channel number and memory type (local, scratch pad, sram)
 */
typedef struct atmtx_s_symbol_descr
{
    ix_uint32 channel; /* channel number */
    ix_uint32 memType; /* memory type */
    ix_uint32 patchMask;  /* mask of microengines where symbol should be patched. */
    ix_uint32 memSize;
} _ix_cc_atmtx_symbol_descr_t;

/**
 * TYPENAME: _ix_cc_atmtx_update_data_t
 * 
 * DESCRIPTION:
 *    This structure describes all parameters for binding external object to
 *    ATMSAR object.
 */
typedef struct 
{
    union {
        ix_uint32 lw0;
        ix_uint16 value;
    } l2port;

    union {
        ix_uint32 lw1;
        ix_uint8  value;        
    } hdrType;
    
} _ix_cc_atmtx_update_data_t;

/**
 * TYPENAME: _ix_cc_atmtx_context_t
 * 
 * DESCRIPTION:
 *    This structure defines format for ATM TX core component context
 */
typedef struct ix_s_cc_atmtx_context
{
    ix_cc_handle hAtmTxHandle;            /* Handle of ATM TX core component */
    ix_cc_atmsar_instance_t instance;  /* Instance ID of the core component */
    ix_counter_64bit_handle *hPktsPerVcq; /* 64 bit counter handle  */
    ix_counter_64bit_handle *hCellsPerVcq; /* 64 bit counter handle  */
    ix_counter_64bit_handle *hPktsPerOutPort; /* 64 bit counter handle  */
    ix_counter_64bit_handle *hCellsPerOutPort; /* 64 bit counter handle  */
    
    ix_uint32 *pSymbolTable[_IX_CC_ATMTX_MAX_SYMBOL_ID];
    _ix_cc_atmtx_symbol_descr_t symbolDescr[_IX_CC_ATMTX_MAX_SYMBOL_ID];

    /*
     * Configuration parameters read from registry or from configuration header
     */
    ix_uint32 mev2Num; /* mask for microengines controled by the CC */
    ix_uint32 counters; /* CC counters mapping enabled/disabled */
    /*    ix_uint32 channel; * memory channel of SRAM for patching symbols */
    char symbolPrefix[_IX_CC_ATMTX_SYMBOL_PREFIX_SIZE+1]; /* the prefix is added to patched symbols */
    ix_cc_atmsar_plugin_cfg_params_t pluginCfg;    
} _ix_cc_atmtx_context_t;


/******************************************************************************
 *                          GLOBAL VARIABLES
 *****************************************************************************/


/******************************************************************************
 *           LOCAL FUNCTIONS FOR ATM TX CORE COMPONENT
 *****************************************************************************/


ix_error ix_cc_atmtx_pkt_handler(ix_buffer_handle arg_hBuffer,  
ix_uint32 arg_ExceptionCode,
void* arg_pContext);


/******************************************************************************
 *           FUNCTIONS FOR ATM SAR PLUGIN INFRASTRUCTURE
 *****************************************************************************/


/**
 * NAME: _ix_cc_atmtx_plugin_vc_create
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "VC create" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_hVC :  handle to created VC.
 * @Param:  IN  arg_pVcDescr : parameters describing requested VC.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 */
ix_error _ix_cc_atmtx_plugin_vc_create(
void* arg_pInstance,
const ix_cc_atmsar_vc_handle_t arg_phVC,
const ix_cc_atmsar_vc_descr_t* arg_pVcDescr);


/**
 * NAME: _ix_cc_atmtx_plugin_vc_update
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "VC update" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_hVC :  handle to created VC.
 * @Param:  IN  arg_pVcDescr : parameters describing requested VC.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCESS - ATM port successfully created
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 *    IX_CC_ATMSAR_ERROR_NO_RESOURCES - no resources to add the entry 
 */
ix_error _ix_cc_atmtx_plugin_vc_update(
void* arg_pInstance,
const ix_cc_atmsar_vc_handle_t arg_phVC,
const ix_cc_atmsar_update_data_t* arg_pUpdate);

/**
 * NAME: _ix_cc_atmtx_plugin_vc_remove
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "VC remove" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_hVc : handle to VC to be removed
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - vc successfully removed
 *    IX_CC_ATMSAR_ERROR_INVALID_PARAM - invalid input parameters
 */
ix_error _ix_cc_atmtx_plugin_vc_remove(
void* arg_pInstance,
ix_cc_atmsar_vc_handle_t arg_hVc);


/**
 * NAME: _ix_cc_atmtx_plugin_get_vc_stats
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "GET VC STATS" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_hVc : handle to VC to be removed
 * @Param:  OUT arg_pLookupResult : pointer to statistic structure
 *                  where the requested data will be stored
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - statistics successfully retrieved
 *    IX_CC_INVALID_PORT - specified ATM port does not exist 
 */
ix_error _ix_cc_atmtx_plugin_get_vc_stats(
void* arg_pInstance,
ix_cc_atmsar_vc_handle_t arg_hVc,
ix_cc_atmsar_vc_stats_t *arg_pLookupResult);


/**
 * NAME: _ix_cc_atmtx_plugin_get_port_stats
 *
 * DESCRIPTION:
 *    This is function for plug-in registering. The ATM SAR
 *    calls the function if it services "GET PORT STATS" operation. The called
 *    function should perform the operation on internal structures.
 * 
 * @Param:  IN  arg_pInstance :  pointer to structure describing serviced
 *                  instance.
 * @Param:  IN  arg_port : ATM port ID
 * @Param:  OUT arg_pLookupResult : pointer to ix_cc_sar_port_stats structure
 *                  where the requested data will be stored
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_SUCCESS - statistics successfully retrieved
 *    IX_CC_INVALID_PORT - specified ATM port does not exist 
 */
ix_error _ix_cc_atmtx_plugin_get_port_stats(
void* arg_pInstance,
ix_cc_atmsar_port_handle_t arg_port,
ix_cc_atmsar_port_stats_t *arg_pLookupResult);


/******************************************************************************
 *                  SUPPORT FOR DEBBUGING AND TRACES
 *****************************************************************************/

extern ix_uint32 g_atmTxDebugMask;

#define _IX_CC_ATMTX_DBG(__mask, __debugInfo) \
    if((__mask & g_atmTxDebugMask)!= 0) ix_ossl_message_log __debugInfo;

#define _IX_CC_ATMTX_DBG_INIT(__debugInfo) _IX_CC_ATMTX_DBG(0x01, __debugInfo)
#define _IX_CC_ATMTX_DBG_VC(__debugInfo) _IX_CC_ATMTX_DBG(0x02, __debugInfo)
#define _IX_CC_ATMTX_DBG_PORT(__debugInfo) _IX_CC_ATMTX_DBG(0x04, __debugInfo)


/******************************************************************************
 *              SUPPORT FOR REGISTRY AND/OR STATIC CONFIGURATION
 *****************************************************************************/


#ifdef IX_INCLUDE_REGISTRY

#define _IX_CC_ATMTX_PROP_MEV2_NUM_MASK      "AAL5_TX_MEV2_NUM_MASK"
#define _IX_CC_ATMTX_PROP_SYMBOL_PREFIX      "SYMBOL_PREFIX"
#define _IX_CC_ATMTX_PROP_COUNTERS_ENABLED   "AAL5_TX_COUNTERS"

#else /* IX_INCLUDE_REGISTRY */

#define _IX_CC_ATMTX_DEFAULT_CHANNEL(instance)    0x0
#define _IX_CC_ATMTX_DEFAULT_MEMTYPE(instance)    IX_MEMORY_TYPE_SRAM
#define _IX_CC_ATMTX_DEFAULT_PATCH_MEV2_NUM_MASK(instance)    0x0

#define _IX_CC_ATMTX_MEV2_NUM_MASK(instance)       0x00000001 
#define _IX_CC_ATMTX_SYMBOL_PREFIX(instance)       ""
#define _IX_CC_ATMTX_COUNTERS_ENABLED(instance)    0x00000001

#endif /* IX_INCLUDE_REGISTRY */


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_ATMTX_PRIVATE_H__) */

