/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor, Release 5
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      ATM TX Core Component
 *
 * = FILENAME
 *      ix_cc_atmtx.h
 *
 * = DESCRIPTION
 *      This file defines all exported data types and function prototypes
 *
 * = AUTHOR
 *       Jaroslaw Kogut
 *       jaroslaw.kogut@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *     
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * 
 */

#if !defined(__IX_CC_ATMTX_CCI_H__)
#define __IX_CC_ATMTX_CCI_H__

#include "ix_rm.h"
#include "ix_ossl.h"
#include "ix_cc.h"
#include "ix_cc_properties.h"

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

/**
 * TYPENAME: ix_cc_atmtx_error_code
 * 
 * DESCRIPTION: This enumerated type will describe atmtx error codes
 *
 */
typedef enum ix_e_cc_atmtx_error_code
{
    IX_CC_ATMTX_ERROR_FIRST = IX_ERROR_MAKE_GROUP(CC_ATMTX),
    IX_CC_ATMTX_ERROR_SUCCESS = IX_CC_ATMTX_ERROR_FIRST,

    IX_CC_ATMTX_ERROR_FAILED_PATCHING,
    IX_CC_ATMTX_ERROR_REGISTRY,
    IX_CC_ATMTX_ERROR_BUFFER_FREE,

    IX_CC_ATMTX_ERROR_CCI,
    IX_CC_ATMTX_ERROR_RM_SET_64BIT_COUNTER,
    IX_CC_ATMTX_ERROR_RM_GET_64BIT_COUNTER,
    IX_CC_ATMTX_ERROR_INVALID_CONTEXT,
    IX_CC_ATMTX_ERROR_INVALID_CALLBACK,
    IX_CC_ATMTX_ERROR_INVALID_INPUT_PARAM,
    IX_CC_ATMTX_ERROR_INVALID_BUFFER,
    IX_CC_ATMTX_ERROR_MSG_LIBRARY,
    IX_CC_ATMTX_ERROR_INVALID_PACKET,
    IX_CC_ATMTX_ERROR_INVALID_MASK,

    IX_CC_ATMTX_ERROR_INVALID_FLAGS,
    IX_CC_ATMTX_ERROR_INVALID_PACKETS_TO_DRAIN,
    IX_CC_ATMTX_ERROR_INVALID_PARAM,

    IX_CC_ATMTX_ERROR_CALLBACK,
    
    /* unknown error code */
    IX_CC_ATMTX_ERROR_UNKNOWN,
    IX_CC_ATMTX_ERROR_LAST
} ix_cc_atmtx_error_code;

/******************************************************************************
 *              CORE COMPONENT INFOSTRUCTURE (CCI) FUNCTIONS
 *****************************************************************************/

/**
 * NAME: ix_cc_atmtx_init
 *
 * DESCRIPTION: 
 * 
 * @Param:  IN  arg_hCcHandle: handle to ATM TX core component, created by core
 *                 component infrastructure; This shall be used later to get
 *                 other (to add event handler) services from core component
 *                 infrastructure.
 *
 * @Param:  OUT arg_ppContext: location where the pointer to the control block
 *                 allocated by the core component will be stored. The control
 *                 block is internal to the core component and it contains
 *                 information about  internal data structures, allocated memory
 *                 and other relevant information. Later this pointer to be
 *                 passed into fini() function for freeing memory and other
 *                 house keeping operations when the component is being
 *                 destroyed.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ERROR_OOM   - memory allocation failure      
 *    IX_CC_ATMTX_ERROR_FAILED_PATCHING   - patching failures                      
 *    IX_CC_ERROR_OOM_64BIT_COUNTER - 64 bit counter creation failure
 *    IX_CC_ATMTX_ERROR_CCI - failure from CCI 
 */
ix_error ix_cc_atmtx_init(
ix_cc_handle arg_hCcHandle,
void** arg_ppContext);


/**
 * NAME: ix_cc_atmtx_fini
 *
 * DESCRIPTION: 
 * 
 * @Param:  IN arg_hCcHandle : handle to ATM TX core component
 * @Param:  IN arg_pContext : ppointer to the control block memory
 *                 allocated earlier in ix_cc_atmtx_init function.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMTX_ERROR_INVALID_INPUT_PARAM - invalid input parameter(s)
 *    IX_CC_ERROR_OOM   - failure of memory deletion                
 *    IX_CC_ERROR_OOM_64BIT_COUNTER - failure of  64 bit counter deletion 
 *    IX_CC_ATMTX_ERROR_CCI - failure from CCI
 */
ix_error ix_cc_atmtx_fini(
ix_cc_handle arg_hCcHandle,
void* arg_pContext);



/******************************************************************************
 *                   LEVEL 0 API (LIBRARY) FUNCTIONS
 *****************************************************************************/


/******************************************************************************
 *                    MESSAGE HELPER FUNCTIONS
 *****************************************************************************/


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_ATMTX_CCI_H__) */

