/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      ATM TX Core Component - CCI - init function
 *
 * = FILENAME
 *      ix_cc_atmtx_init.c
 *
 * = DESCRIPTION
 *      The file defines for init interface and internal functions.
 *
 * = AUTHOR
 *       Jaroslaw Kogut
 *       jaroslaw.kogut@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      11/07/2002
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */
#define IX_ERROR_FILE_IDENT "$Id: ix_cc_atmtx_init.c,v 1.5 2003/12/12 19:12:20 adziecia Exp $";


/**
 * System defined include files
 */
#include "ix_cc_error.h"
#include "ix_rm.h"
#include "ix_ossl.h"
#include "ix_cc.h"
#include "ix_cc_macros.h"
#include "ix_config.h"
#include "bindings.h"
#include "cc/ix_cc_atmsar.h"
#include "cc/ix_cc_atmsar_plugin.h"

/**
 * User defined include files
 */
#include "cc/ix_cc_atmtx_cci.h"
#include "cc/internal/ix_cc_atmtx_private.h"

/* TBD: Temporaty solution - during redesing SAR in multinstaces solution.
 * Instead of the definition should be mechanism of multiinstaces */
#define _IX_CC_ATMTX_INSTANCE_SAR_0 0

/**
 * Preprocessor symbols and Macros used in this file
 */

/**
 * Type definitions whose scope is limited to this file
 */

/**
 * TYPENAME: _ix_cc_atmtx_symbol_table_t
 * 
 * DESCRIPTION:
 *    This data structure describes the SRAM memory to be allocated,
 *    initialized and patched into corresponding symbol
 */
typedef struct ix_s_atmtx_symbol_table
{
    ix_uint8     *registryName;
    ix_uint8     *symbol;
    ix_uint32    elementSize;
    ix_uint8     symbolType;
    ix_uint8     initValue;
    _ix_cc_atmtx_symbol_id_t symbolId;
    ix_uint32    valueToPatch;
}_ix_cc_atmtx_symbol_table_t;

/**
 * Variable declarations global to this file only. Externs are followed by 
 * static variables.
 */


/**
 * VARNAME: g_atmTxDebugMask
 *
 * DESCRIPTION:
 *    The global variable contains mask of bits that define what debugging
 *    traces has to be show.
 */
ix_uint32 g_atmTxDebugMask=0x0FFFFFFFF;

static _ix_cc_atmtx_symbol_table_t s_atmTxSymbolTable[] =
{        									 
    {
        "CTX_SRAM_BASE",
        "AAL5_TX_CTX_SRAM_BASE",
        AAL5_TX_CTX_SIZE,
        _IX_CC_ATMTX_AAL5_VC_SYMBOL_TYPE,
        0x0,
        _IX_CC_ATMTX_AAL5_TX_CTX_SRAM_BASE_SYMBOL_ID,
        0
    },          								 
    {
        "PKTS_PER_VCQ_COUNTER",
        "AAL5_TX_PKTS_PER_VCQ_COUNTER_SRAM_BASE",
        AAL5_TX_PKTS_PER_VCQ_COUNTER_SIZE,
        _IX_CC_ATMTX_AAL5_VC_SYMBOL_TYPE,
        0x0,
        _IX_CC_ATMTX_AAL5_TX_PKTS_PER_VCQ_COUNTER_BASE_SYMBOL_ID,
        0
    },          								 
    {
        "CELLS_PER_VCQ_COUNTER",
        "AAL5_TX_CELLS_PER_VCQ_COUNTER_SRAM_BASE",
        AAL5_TX_CELLS_PER_VCQ_COUNTER_SIZE,
        _IX_CC_ATMTX_AAL5_VC_SYMBOL_TYPE,
        0x0,
     	_IX_CC_ATMTX_AAL5_TX_CELLS_PER_VCQ_COUNTER_BASE_SYMBOL_ID,
        0
    },          								 
    {
        "PKTS_PER_OUTPORT_COUNTER",
        "AAL5_TX_PKTS_PER_OUTPORT_COUNTER_SRAM_BASE",
        AAL5_TX_PKTS_PER_OUTPORT_COUNTER_SIZE,
        _IX_CC_ATMTX_AAL5_PORT_SYMBOL_TYPE,
        0x0,
        _IX_CC_ATMTX_AAL5_TX_PKTS_PER_OUTPORT_COUNTER_BASE_SYMBOL_ID,
        0
    },          								 
    { 
        "CELLS_PER_OUTPORT_COUNTER",
        "AAL5_TX_CELLS_PER_OUTPORT_COUNTER_SRAM_BASE",
        AAL5_TX_CELLS_PER_OUTPORT_COUNTER_SIZE,
        _IX_CC_ATMTX_AAL5_PORT_SYMBOL_TYPE,
        0x0,
        _IX_CC_ATMTX_AAL5_TX_CELLS_PER_OUTPORT_COUNTER_BASE_SYMBOL_ID,
        0
    },          								 
    {
        "CELLS_TX_SCRATCH",
        "AAL5_TX_CELLS_TX_SCRATCH_BASE",
        AAL5_TX_CELLS_TX_ELEMENT_SIZE * 4,
        _IX_CC_ATMTX_AAL5_SINGLE_SYMBOL_TYPE,
        0x0,
        _IX_CC_ATMTX_AAL5_TX_CELLS_TX_SCRATCH_BASE_SYMBOL_ID,
        0
    },
    {
        "AVAIL_TBUF_ELEMENT",
        "AAL5_TX_AVAIL_TBUF_ELEMENT_SCRATCH_BASE",
        AAL5_TX_AVAIL_TBUF_ELEMENT_SIZE * 4,
        _IX_CC_ATMTX_AAL5_SINGLE_SYMBOL_TYPE,
        0x0,
        _IX_CC_ATMTX_AAL5_TX_AVAIL_TBUF_ELEMENT_SCRATCH_BASE_SYMBOL_ID,
        0
    },
    {   /* Terminator */          						 
        NULL,
        0,
        0,
        0
    },
};       


/** 
 * Extern function prototypes.
 */


/** 
 * Static function prototypes.
 */


/** 
 * Function definitions.
 */


/******************************************************************************
 *                            SHUTDOWN FUNCTIONS
 *****************************************************************************/

/**
 * NAME: _ix_cc_atmtx_free_symbols
 *
 * DESCRIPTION:
 *    This primitive will be used to free memory for symbols  allocated for
 *    patching.
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ERROR_OOM - failed to free memory for symbols.
 */
static ix_error
_ix_cc_atmtx_free_symbols(
_ix_cc_atmtx_context_t *arg_pContext)
{
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);
    /* Free memory for microblock symbols */
    /*  IX_ERROR_CRT(ix_rm_mem_free(arg_pContext->p),
        IX_CC_ERROR_OOM,IX_ERROR_LEVEL_LOCAL);
    */
    /*
     * Function successfully freed memory for control block, 
     * directed broadcast table and statistics.
     */
    _IX_CC_ATMTX_DBG_INIT(("_ix_cc_atmtx_free_symbols() The functionality is not implemented yet\n"));
    return IX_SUCCESS;
}/* end _ix_cc_atmtx_free_symbols */


/**
 * NAME: _ix_cc_atmtx_delete_64bit_counters
 *
 * DESCRIPTION:
 *    This primitive will be used to delete 64 bit counters. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ERROR_OOM_64BIT_COUNTER - failed to delete 64 bit counter.
 */
static ix_error
_ix_cc_atmtx_delete_64bit_counters(
_ix_cc_atmtx_context_t *arg_pContext)
{
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);
    _IX_CC_ATMTX_DBG_INIT(("_ix_cc_atmtx_delete_64bit_counters() The functionality is not implemented yet\n"));
    return IX_SUCCESS;
}/* end _ix_cc_atmtx_create_64bit_counters */


/**
 * NAME: _ix_cc_atmtx_unregister_message_handler
 *
 * DESCRIPTION:
 *    This primitive will be used to unregister message handler. It calls CCI
 *    services in level 1 system and resore manager services in level 0 system.
 *
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMTX_ERROR_CCI - error from CCI
 */
static ix_error
_ix_cc_atmtx_unregister_message_handler(
_ix_cc_atmtx_context_t *arg_pContext)
{
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);
    /* Call CCI if CCI is enabled */
#if 0
#if defined(IX_EXCLUDE_CCI)
    IX_ERROR_CRT(ix_rm_message_handler_unregister(IX_CC_ATMTX_MSG_INPUT),
                 IX_CC_ATMTX_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);
#else
    IX_ERROR_CRT(ix_cci_cc_remove_message_handler(IX_CC_ATMTX_MSG_INPUT),
                 IX_CC_ATMTX_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);
#endif
#endif
    return IX_SUCCESS;
}/* end _ix_cc_atmtx_unregister_message_handler */


/**
 * NAME: _ix_cc_atmtx_unregister_cci_packet_handlers
 *
 * DESCRIPTION:
 *    This primitive will be used to unregister packet handler with CCI. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMTX_ERROR_CCI - error from CCI
 */
static ix_error
_ix_cc_atmtx_unregister_cci_packet_handlers(
_ix_cc_atmtx_context_t *arg_pContext)
{
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);
#if 0
#if !defined(IX_EXCLUDE_CCI)
    IX_ERROR_CRT(ix_cci_cc_remove_message_handler(IX_CC_ATMTX_PKT_INPUT),
                 IX_CC_ATMTX_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);
#endif
#endif
    return IX_SUCCESS;
}/* end _ix_cc_atmtx_unregister_cci_packet_handlers */


/**
 * NAME: _ix_cc_atmtx_unregister_rm_packet_handlers
 *
 * DESCRIPTION:
 *    This primitive will be used to unregister packet handler with 
 *    resource manager
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMTX_ERROR_CCI - error from RM
 */
#if defined(IX_EXCLUDE_CCI)
static ix_error
_ix_cc_atmtx_unregister_rm_packet_handlers(
_ix_cc_atmtx_context_t *arg_pContext)
{
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);
    IX_ERROR_CRT(ix_rm_message_handler_unregister(IX_CC_ATMTX_PKT_INPUT),
                 IX_CC_ATMTX_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);
    return IX_SUCCESS;
}/* end _ix_cc_atmtx_unregister_rm_packet_handlers */
#endif


/**
 * NAME: _ix_cc_atmtx_unregister_packet_handlers
 *
 * DESCRIPTION: This primitive will be used to unregister packet handler 
 * with CCI/Resource manager
 *
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMTX_ERROR_CCI - error from CCI/RM
 */
static ix_error
_ix_cc_atmtx_unregister_packet_handlers(
_ix_cc_atmtx_context_t *arg_pContext)
{
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);
    /* Call CCI if CCI is enabled */
#if defined(IX_EXCLUDE_CCI)
    IX_ERROR_CR(_ix_cc_atmtx_unregister_rm_packet_handlers(arg_pContext));
#else
    IX_ERROR_CR(_ix_cc_atmtx_unregister_cci_packet_handlers(arg_pContext));
#endif
    return IX_SUCCESS;
}/* end _ix_cc_atmtx_unregister_packet_handlers */


/******************************************************************************
 *                            INITIALIZATION FUNCTIONS
 *****************************************************************************/

/**
 * NAME: _ix_cc_atmtx_property_name_set
 *
 * DESCRIPTION:
 *    This primitive builds full path to variable. It adds configuration
 *    section prefix and adds instance to prefix.
 *           .
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  OUT arg_pBuf : destination buffer
 * @Param:  IN  arg_pBufSize : size of destination buffer
 * @Param:  IN  arg_pPropertyName : buffer with property Name
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMTX_ERROR_REGISTRY  - not able to access registry.
 */
static ix_error
_ix_cc_atmtx_property_name_set(
_ix_cc_atmtx_context_t* arg_pContext,
char *arg_pBuf,
ix_uint16 arg_pBufSize,
const char *arg_pProperty)
{
    char *pSection="ATM_TX";

    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);
    IX_ERROR_DCHECK_ARG_NULL(2, arg_pBuf);
    IX_ERROR_DCHECK_ARG_NULL(4, arg_pProperty);
    IX_ERROR_DCHECK_ARG_MAX(4, (strlen(pSection) + strlen(arg_pProperty) + 6),
                            arg_pBufSize, "%d");

    sprintf(arg_pBuf, "%s_%d%c%s", pSection, arg_pContext->instance,
            IX_CP_PATH_NAME_DELIMITER, arg_pProperty);

    return IX_SUCCESS;
} /* end _ix_cc_atmtx_property_name_set */


/**
 * NAME: _ix_cc_atmtx_registry
 *
 * DESCRIPTION:
 *    This primitive will be used to retreive data from registry for all the
 *    core component static data.
 *           .
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMTX_ERROR_REGISTRY  - not able to access registry.
 *    IX_CC_ERROR_NULL - core component context is null
 */
static ix_error
_ix_cc_atmtx_registry(
_ix_cc_atmtx_context_t* arg_pContext)
{
#if defined (IX_INCLUDE_REGISTRY)
    char name[256];
    ix_configuration_property_handle hCoreRoot = IX_CP_CORE_PROPERTY_ROOT;
    ix_configuration_property_handle hProperty;
    ix_cp_property_info propertyInfo;
    ix_int32 i;

    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);
    
    for (i=0; s_atmTxSymbolTable[i].symbol != NULL; i++)
    {
        sprintf(name,"ATM_TX_%d%c%s%cCHANNEL",
                arg_pContext->instance,
                IX_CP_PATH_NAME_DELIMITER,
                s_atmTxSymbolTable[i].registryName,
                IX_CP_PATH_NAME_DELIMITER);
                                                                             	
        IX_ERROR_CR(ix_rm_cp_property_open(hCoreRoot, name, &hProperty));
        
        IX_ERROR_CR(ix_rm_cp_property_get_value_uint32(hProperty, &(arg_pContext->symbolDescr[i].channel)));

        _IX_CC_ATMTX_DBG_INIT(("SYMBOL: %s CHANNEL: %ld\n",
                               s_atmTxSymbolTable[i].symbol,
                               arg_pContext->symbolDescr[i].channel));
        
        sprintf(name,"ATM_TX_%d%c%s%cMEM_TYPE",
                arg_pContext->instance,
                IX_CP_PATH_NAME_DELIMITER,
                s_atmTxSymbolTable[i].registryName,
                IX_CP_PATH_NAME_DELIMITER);

        IX_ERROR_CR(ix_rm_cp_property_open(hCoreRoot, name, &hProperty));
        
        IX_ERROR_CR(ix_rm_cp_property_get_value_uint32(hProperty, &(arg_pContext->symbolDescr[i].memType)));

        _IX_CC_ATMTX_DBG_INIT(("SYMBOL: %s MEM_TYPE: %ld\n",
                               s_atmTxSymbolTable[i].symbol,
                               arg_pContext->symbolDescr[i].memType));        
        

        sprintf(name,"ATM_TX_%d%c%s%cPATCH_MEV2_NUM_MASK",
                arg_pContext->instance,
                IX_CP_PATH_NAME_DELIMITER,
                s_atmTxSymbolTable[i].registryName,
                IX_CP_PATH_NAME_DELIMITER);

        if (ix_rm_cp_property_open(hCoreRoot, name, &hProperty) == IX_SUCCESS)
        {
            IX_ERROR_CR(ix_rm_cp_property_get_value_uint32(hProperty, &(arg_pContext->symbolDescr[i].patchMask)));

            _IX_CC_ATMTX_DBG_INIT(("SYMBOL: %s PATCH_MEV2_NUM_MASK: %ld\n",
                                   s_atmTxSymbolTable[i].symbol,
                                   arg_pContext->symbolDescr[i].patchMask));        
        }
        else
        {
            /* Default value for patchMask */
         	/*	arg_pContext->symbolDescr[i].patchMask = ATM_TX_DEFAULT_PATCH_MEV2_NUM_MASK(arg_pContext->instance); */

        }

        /* calculate the memory size*/
        switch(s_atmTxSymbolTable[i].symbolType)
        {        
        case _IX_CC_ATMTX_AAL5_PORT_SYMBOL_TYPE:
            arg_pContext->symbolDescr[i].memSize = s_atmTxSymbolTable[i].elementSize * arg_pContext->pluginCfg.numberOfPorts;
            break;
        case _IX_CC_ATMTX_AAL5_VC_SYMBOL_TYPE:
            arg_pContext->symbolDescr[i].memSize = s_atmTxSymbolTable[i].elementSize * arg_pContext->pluginCfg.numberOfVcs;
            break;
        case _IX_CC_ATMTX_AAL5_SINGLE_SYMBOL_TYPE:
            arg_pContext->symbolDescr[i].memSize = s_atmTxSymbolTable[i].elementSize;
            break;
        default:
            IX_ERROR_GLOBAL(IX_CC_ATMTX_ERROR_REGISTRY,
                            ("Symbol type (%d) is invalid for %s.\n",
                             s_atmTxSymbolTable[i].symbol));
            break;
        } /* switch */
        
    }

    /* property AAL5_TX_MEV2_NUM_MASK */
    IX_ERROR_CR(_ix_cc_atmtx_property_name_set(arg_pContext, name, sizeof(name), _IX_CC_ATMTX_PROP_MEV2_NUM_MASK));
    IX_ERROR_CR(ix_rm_cp_property_open(hCoreRoot, name, &hProperty));
    IX_ERROR_CR(ix_rm_cp_property_get_value_uint32(hProperty, &(arg_pContext->mev2Num)));

    /* property SYMBOL_PREFIX */
    IX_ERROR_CR(_ix_cc_atmtx_property_name_set(arg_pContext, name, sizeof(name), _IX_CC_ATMTX_PROP_SYMBOL_PREFIX));
    IX_ERROR_CR(ix_rm_cp_property_open(hCoreRoot, name, &hProperty));
    IX_ERROR_CR(ix_rm_cp_property_get_info(hProperty, &propertyInfo));
    if (propertyInfo.m_DataSize > _IX_CC_ATMTX_SYMBOL_PREFIX_SIZE)
    {
        return IX_ERROR_WARNING(IX_CC_ATMTX_ERROR_REGISTRY,
                                ("The prefix symbol size (%d) is too long.\n",
                                 propertyInfo.m_DataSize));
    }
    IX_ERROR_CR(ix_rm_cp_property_get_value(hProperty, &propertyInfo.m_DataSize, 
                                            arg_pContext->symbolPrefix));

    /* property AAL5_TX_COUNTERS */
    IX_ERROR_CR(_ix_cc_atmtx_property_name_set(arg_pContext, name, sizeof(name), _IX_CC_ATMTX_PROP_COUNTERS_ENABLED));
    IX_ERROR_CR(ix_rm_cp_property_open(hCoreRoot, name, &hProperty));
    IX_ERROR_CR(ix_rm_cp_property_get_value_uint32(hProperty, &(arg_pContext->counters)));
    
#else
    
    ix_int32 i;
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);

    for (i=0; s_atmTxSymbolTable[i].symbol != NULL; i++)
    {
        arg_pContext->symbolDescr[i].channel = _IX_CC_ATMTX_DEFAULT_CHANNEL(arg_pContext->instance);
        arg_pContext->symbolDescr[i].memType = _IX_CC_ATMTX_DEFAULT_MEMTYPE(arg_pContext->instance);        
        arg_pContext->symbolDescr[i].patchMask = _IX_CC_ATMTX_DEFAULT_PATCH_MEV2_NUM_MASK(arg_pContext->instance);
    }
    
    arg_pContext->mev2Num = _IX_CC_ATMTX_DEFAULT_PATCH_MEV2_NUM_MASK(arg_pContext->instance);
    strcpy(arg_pContext->symbolPrefix, _IX_CC_ATMTX_SYMBOL_PREFIX(arg_pContext->instance));
    arg_pContext->counters = _IX_CC_ATMTX_COUNTERS_ENABLED(arg_pContext->instance);
    
#endif

    _IX_CC_ATMTX_DBG_INIT(("_ix_cc_atmtx_registry(): ATM_TX_%d config: ME mask: %lx\n",
                           arg_pContext->instance, arg_pContext->mev2Num));
    
    return IX_SUCCESS;
}/* end _ix_cc_atmtx_registry */


/**
 * NAME: _ix_cc_atmtx_alloc_symbols
 *
 * DESCRIPTION:
 *    This primitive will be used to allocate memory for symbols to be patched.
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ERROR_OOM  - not able to allocate memory
 */
static ix_error
_ix_cc_atmtx_alloc_symbols(
_ix_cc_atmtx_context_t* arg_pContext)
{
    ix_error  err = IX_SUCCESS;
    ix_uint32 i;
    ix_uint32 **pTable;
    ix_uint32 memSize;
    
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);


    /* Allocate SRAM memory to be used by the microblock */
    for (i=0; s_atmTxSymbolTable[i].symbol != NULL; i++)
    {
        pTable = &arg_pContext->pSymbolTable[s_atmTxSymbolTable[i].symbolId];
        
        if ( (s_atmTxSymbolTable[i].symbolId == _IX_CC_ATMTX_AAL5_TX_PKTS_PER_VCQ_COUNTER_BASE_SYMBOL_ID) ||
             (s_atmTxSymbolTable[i].symbolId == _IX_CC_ATMTX_AAL5_TX_CELLS_PER_VCQ_COUNTER_BASE_SYMBOL_ID) ||
             (s_atmTxSymbolTable[i].symbolId == _IX_CC_ATMTX_AAL5_TX_PKTS_PER_OUTPORT_COUNTER_BASE_SYMBOL_ID) ||
             (s_atmTxSymbolTable[i].symbolId == _IX_CC_ATMTX_AAL5_TX_CELLS_PER_OUTPORT_COUNTER_BASE_SYMBOL_ID) )                     
        {
            if (!arg_pContext->counters)
            {
                continue;
            }
        }

        /* Allocate memory for a table */   
        err = ix_rm_mem_alloc(arg_pContext->symbolDescr[i].memType,
                              arg_pContext->symbolDescr[i].channel,
                              arg_pContext->symbolDescr[i].memSize,
                              (void **)pTable);
        if ( err )
        {
            err=IX_ERROR_WARNING(IX_CC_ERROR_NULL,
                                 ("Unable to allocate %d memory for symbol %s \n",
                                  arg_pContext->symbolDescr[i].memSize,
                                  s_atmTxSymbolTable[i].symbol));
            return err;
        }

        /* Initialize table  */
        ix_ossl_memset((ix_uint8 *)(*pTable),
                       (ix_uint8)s_atmTxSymbolTable[i].initValue,
                       (ix_uint32)arg_pContext->symbolDescr[i].memSize);

        _IX_CC_ATMTX_DBG_INIT(("Successfuly allocated %ld bytes for symbol %s Address: %ld\n",
                               arg_pContext->symbolDescr[i].memSize, s_atmTxSymbolTable[i].symbol,(ix_uint32)pTable));
    }
    
    /* Additional initialization: */

    if (arg_pContext->counters)
    {
        memSize = arg_pContext->pluginCfg.numberOfPorts * sizeof(ix_counter_64bit_handle);
        arg_pContext->hPktsPerOutPort = (ix_counter_64bit_handle*) ix_ossl_malloc(memSize);
        if(arg_pContext->hPktsPerOutPort == NULL)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_OOM_64BIT_COUNTER,
                                    ("Memory allocation failed for TX packets counter/port table"));
        }    
        ix_ossl_memset(arg_pContext->hPktsPerOutPort,0,memSize);
    }

    if (arg_pContext->counters)
    {
        memSize = arg_pContext->pluginCfg.numberOfPorts * sizeof(ix_counter_64bit_handle);
        arg_pContext->hCellsPerOutPort = (ix_counter_64bit_handle*) ix_ossl_malloc(memSize);
        if(arg_pContext->hCellsPerOutPort == NULL)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_OOM_64BIT_COUNTER,
                                    ("Memory allocation failed for TX celss counter/port table"));
        }    
        ix_ossl_memset(arg_pContext->hCellsPerOutPort,0,memSize);
    }

    if (arg_pContext->counters)
    {
        memSize = arg_pContext->pluginCfg.numberOfVcs * sizeof(ix_counter_64bit_handle);
        arg_pContext->hPktsPerVcq = (ix_counter_64bit_handle*) ix_ossl_malloc(memSize);
        if(arg_pContext->hPktsPerVcq == NULL)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_OOM_64BIT_COUNTER,
                                    ("Memory allocation failed for TX packets counter/vc table"));
        }    
        ix_ossl_memset(arg_pContext->hPktsPerVcq,0,memSize);
    }
        
    if (arg_pContext->counters)
    {        
        memSize = arg_pContext->pluginCfg.numberOfVcs * sizeof(ix_counter_64bit_handle);
        arg_pContext->hCellsPerVcq = (ix_counter_64bit_handle*) ix_ossl_malloc(memSize);
        if(arg_pContext->hCellsPerVcq == NULL)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_OOM_64BIT_COUNTER,
                                    ("Memory allocation failed for TX cells counter/vc table"));
        }    
        ix_ossl_memset(arg_pContext->hCellsPerVcq,0,memSize);
    }
    
    return IX_SUCCESS;
}/* end _ix_cc_atmtx_alloc_symbols */


/**
 * NAME: _ix_cc_atmtx_patch_symbols
 *
 * DESCRIPTION:
 *    This primitive will be used to patch symbols to ATM TX microblocks.
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMTX_ERROR_FAILED_PATCHING - failed patching by calling resource
 *        manager function
 */
static ix_error
_ix_cc_atmtx_patch_symbols(
_ix_cc_atmtx_context_t* arg_pContext)
{

#define _IX_CC_ATMTX_SYMBOL_SIZE 64

    ix_imported_symbol aImportSymbol;
    ix_uint32 i,j;
    ix_memory_type memType;
    ix_uint32 channel = 0;
    ix_uint32 zboffset = 0;
    void *pPhysicalAddress = NULL;
    ix_error err;
    char symbol[_IX_CC_ATMTX_SYMBOL_PREFIX_SIZE + _IX_CC_ATMTX_SYMBOL_SIZE + 1];

    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);

    for (i=0; s_atmTxSymbolTable[i].symbol != NULL; i++)
    {
        
        if ( (s_atmTxSymbolTable[i].symbolId == _IX_CC_ATMTX_AAL5_TX_PKTS_PER_VCQ_COUNTER_BASE_SYMBOL_ID) ||
             (s_atmTxSymbolTable[i].symbolId == _IX_CC_ATMTX_AAL5_TX_CELLS_PER_VCQ_COUNTER_BASE_SYMBOL_ID) ||
             (s_atmTxSymbolTable[i].symbolId == _IX_CC_ATMTX_AAL5_TX_PKTS_PER_OUTPORT_COUNTER_BASE_SYMBOL_ID) ||
             (s_atmTxSymbolTable[i].symbolId == _IX_CC_ATMTX_AAL5_TX_CELLS_PER_OUTPORT_COUNTER_BASE_SYMBOL_ID) )                     
        {
            if (!arg_pContext->counters)
            {
                continue;
            };
        }        
        
        IX_ERROR_CT(ix_rm_get_phys_offset((void *)(arg_pContext->pSymbolTable[s_atmTxSymbolTable[i].symbolId]),
                                          &memType,
                                          &channel,
                                          &zboffset,
                                          &pPhysicalAddress),
                    err,
                    IX_CC_ATMTX_ERROR_FAILED_PATCHING, 
                    IX_ERROR_LEVEL_REMOTE);
        if (err != IX_SUCCESS)
        {

            return(err);
        }

        /* Pack the SRAM channel number into the offset */
        IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(channel, zboffset);

        /** 
         * Prepare the symbol array.  For now only single mode is used 
         * so skip the memOffsetNextAvailTBufElem.
         */   
        if (strlen(s_atmTxSymbolTable[i].symbol) > _IX_CC_ATMTX_SYMBOL_SIZE)
        {
            return IX_ERROR_GLOBAL(IX_CC_ATMTX_ERROR_FAILED_PATCHING, 
                                   ("Variable %s too long", s_atmTxSymbolTable[i].symbol));
        }
        sprintf(symbol, "%s%s", arg_pContext->symbolPrefix, s_atmTxSymbolTable[i].symbol);
        aImportSymbol.m_Value = (ix_uint32)zboffset;
        aImportSymbol.m_Name  = symbol;

        for (j=0;j<IX_MAX_MICROENGINE_NUMBER;j++)
        {
            if ( (arg_pContext->mev2Num |
                  arg_pContext->symbolDescr[i].patchMask) & (0x1 << j))
            {
                IX_ERROR_CR(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[j], 1, &aImportSymbol));
            }
        }
    }
    return IX_SUCCESS;
    
#undef _IX_CC_ATMTX_SYMBOL_SIZE
    
}/* end _ix_cc_atmtx_patch_symbols */


/**
 * NAME: _ix_cc_atmtx_create_64bit_counters
 *
 * DESCRIPTION:
 *    This primitive will be used to create 64 bit counters.   .
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ERROR_OOM_64BIT_COUNTER - error in creating 64 bit counter
 */
static ix_error
_ix_cc_atmtx_create_64bit_counters(
_ix_cc_atmtx_context_t* arg_pContext)
{
    ix_counter_64bit_handle* p64BitCountersHdls;
    ix_uint32* p64BitCountersBase;
    ix_uint16 num64BitCountersHdls;
    ix_uint32 overflowTime;
    ix_uint32 overflowTimeMin;
    ix_uint16 i;
    ix_cc_atmsar_plugin_cfg_port_descr_t *pPortCfgTab;
    ix_error err=IX_SUCCESS;
   
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);

    _IX_CC_ATMTX_DBG_INIT(("_IX_CC_ATMTX: READ PORT STATIC CONFIGURATION\n"));
    
    overflowTimeMin=0x0FFFFFFFF;

    pPortCfgTab = ix_ossl_malloc(sizeof(*pPortCfgTab)*arg_pContext->pluginCfg.numberOfPorts);
    if (pPortCfgTab == NULL)
    {
        return IX_ERROR_GLOBAL(IX_CC_ERROR_NULL, ("Pointer for Port configuration table is NULL"));
    }

    IX_ERROR_CG(ix_cc_atmsar_plugin_get_cfg_ports_params(arg_pContext->instance,
                                                         arg_pContext->pluginCfg.numberOfPorts,
                                                         pPortCfgTab),
                err,
                end_of_processing);

    /*
     * read static configuration of ports and find minimum overflow time
     */
    for (i = 0; i<arg_pContext->pluginCfg.numberOfPorts; i++)
    {
        if (pPortCfgTab[i].linkRate == 0)
        {
            continue;
        }

        overflowTime=0x0FFFFFFFF/pPortCfgTab[i].linkRate;

        _IX_CC_ATMTX_DBG_INIT(("ATMTX: ATM PORT(%04d):  speed=%-10ld, overflowTime=%ld\n",
                       i, pPortCfgTab[i].linkRate, overflowTime));
        
        if (overflowTime<700)
        {
            /* Max link speed is equal 5636096 cps (It is OC-48). Minimum
             * overflow time is equal (2^32)/5636096 =~ 762 seconds */
            err= IX_ERROR_GLOBAL(IX_CC_ERROR_ENTRY_NOT_FOUND, 
                                 ("Wrong overflow time %ld for port nr %ld",
                                  overflowTime, i));
            goto end_of_processing;
        }

        /* change overflow time into safe range. Extra 10 seconds */
        overflowTime-=10;
        if (overflowTime < overflowTimeMin)
        {
            overflowTimeMin=overflowTime;
        }
    }
    
    _IX_CC_ATMTX_DBG_INIT(("_IX_CC_ATMTX: Minimum overflowTime=%ld\n", overflowTimeMin));

    /* create 64 bits counters for VC statistics */
    num64BitCountersHdls=arg_pContext->pluginCfg.numberOfVcs;
    p64BitCountersBase=arg_pContext->pSymbolTable[_IX_CC_ATMTX_AAL5_TX_PKTS_PER_VCQ_COUNTER_BASE_SYMBOL_ID];
    p64BitCountersHdls=arg_pContext->hPktsPerVcq;
    IX_ERROR_CGT(ix_rm_counter_64bit_new(num64BitCountersHdls,
                                               p64BitCountersBase,
                                               overflowTimeMin,
                                               p64BitCountersHdls),
                 err,
                 IX_CC_ERROR_OOM_64BIT_COUNTER, 
                 IX_ERROR_LEVEL_REMOTE,
                 end_of_processing);

    num64BitCountersHdls=arg_pContext->pluginCfg.numberOfVcs;
    p64BitCountersBase=arg_pContext->pSymbolTable[_IX_CC_ATMTX_AAL5_TX_CELLS_PER_VCQ_COUNTER_BASE_SYMBOL_ID];
    p64BitCountersHdls=arg_pContext->hCellsPerVcq;
    IX_ERROR_CGT(ix_rm_counter_64bit_new(num64BitCountersHdls,
                                         p64BitCountersBase,
                                         overflowTimeMin,
                                         p64BitCountersHdls),
                 err,
                 IX_CC_ERROR_OOM_64BIT_COUNTER, 
                 IX_ERROR_LEVEL_REMOTE,
                 end_of_processing);

    num64BitCountersHdls=arg_pContext->pluginCfg.numberOfPorts;
    p64BitCountersBase=arg_pContext->pSymbolTable[_IX_CC_ATMTX_AAL5_TX_PKTS_PER_OUTPORT_COUNTER_BASE_SYMBOL_ID];
    p64BitCountersHdls=arg_pContext->hPktsPerOutPort;
    IX_ERROR_CGT(ix_rm_counter_64bit_new(num64BitCountersHdls,
                                         p64BitCountersBase,
                                         overflowTimeMin,
                                         p64BitCountersHdls),
                 err,
                 IX_CC_ERROR_OOM_64BIT_COUNTER, 
                 IX_ERROR_LEVEL_REMOTE,
                 end_of_processing);

    num64BitCountersHdls=arg_pContext->pluginCfg.numberOfPorts;
    p64BitCountersBase=arg_pContext->pSymbolTable[_IX_CC_ATMTX_AAL5_TX_CELLS_PER_OUTPORT_COUNTER_BASE_SYMBOL_ID];
    p64BitCountersHdls=arg_pContext->hCellsPerOutPort;
    IX_ERROR_CGT(ix_rm_counter_64bit_new(num64BitCountersHdls,
                                         p64BitCountersBase,
                                         overflowTimeMin,
                                         p64BitCountersHdls),
                 err,
                 IX_CC_ERROR_OOM_64BIT_COUNTER, 
                 IX_ERROR_LEVEL_REMOTE,
                 end_of_processing);

  end_of_processing:

    ix_ossl_free(pPortCfgTab);
    
    return err;
}/* end _ix_cc_atmtx_create_64bit_counters */


/**
 * NAME: _ix_cc_atmtx_register_message_handler
 *
 * DESCRIPTION:
 *    This primitive will be used to register message handler. 
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMTX_ERROR_CCI - Error from CCI
 */
static ix_error
_ix_cc_atmtx_register_message_handler(
_ix_cc_atmtx_context_t *arg_pContext)
{
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);
    return IX_SUCCESS;
}/* end _ix_cc_atmtx_register_message_handler */


/**
 * NAME: _ix_cc_atmtx_register_cci_packet_handlers
 *
 * DESCRIPTION:
 *    This primitive will be used to register packet handler with CCI.
 *
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMTX_ERROR_CCI - error from CCI
 */
static ix_error
_ix_cc_atmtx_register_cci_packet_handlers(
_ix_cc_atmtx_context_t *arg_pContext)
{
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);

#if !defined(IX_EXCLUDE_CCI)
    /* High priority packet handler needs to be registered as a message handler */

    ix_rm_packet_set_receive_mode(IX_CC_ATMTX_PKT_INPUT, IX_COMM_ID_MODE_CALLBACK);

    IX_ERROR_CRT(ix_rm_message_handler_register(IX_CC_ATMTX_PKT_INPUT,
                                                ix_cc_atmtx_pkt_handler,
                                                arg_pContext),
                 IX_CC_ATMTX_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);     
#endif
    return IX_SUCCESS;
}/* end _ix_cc_atmtx_register_cci_packet_handlers */


/**
 * NAME: _ix_cc_atmtx_register_rm_packet_handlers
 *
 * DESCRIPTION:
 *    This primitive will be used to register packet handler with resource
 *    manager
 *
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMTX_ERROR_CCI - error from resource manager
 */
#if defined(IX_EXCLUDE_CCI)
static ix_error
_ix_cc_atmtx_register_rm_packet_handlers(
_ix_cc_atmtx_context_t *arg_pContext)
{
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);
    IX_ERROR_CRT(ix_rm_message_handler_register(IX_CC_ATMTX_PKT_INPUT,
                                                ix_cc_atmtx_pkt_handler,
                                                arg_pContext),
                 IX_CC_ATMTX_ERROR_CCI,
                 IX_ERROR_LEVEL_WARNING);

    return IX_SUCCESS;
}/* end _ix_cc_atmtx_register_rm_packet_handlers */
#endif

/**
 * NAME: _ix_cc_atmtx_register_packet_handlers
 *
 * DESCRIPTION:
 *    This primitive will be used to register packet handlers. It registers with
 *    CCI in level 1 system and register with RM in level 0 system.
 * 
 * @Param:  IN  arg_pContext : pointer of core component context.
 * @Param:  OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMTX_ERROR_CCI - error from CCI/resource manager
 */
static ix_error
_ix_cc_atmtx_register_packet_handlers(
_ix_cc_atmtx_context_t *arg_pContext)
{
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pContext);

    /* Call CCI packet handler if CCI is enabled */
#if defined(IX_EXCLUDE_CCI)
    IX_ERROR_CR(_ix_cc_atmtx_register_rm_packet_handlers(arg_pContext));
#else
    IX_ERROR_CR(_ix_cc_atmtx_register_cci_packet_handlers(arg_pContext));
#endif
    return IX_SUCCESS;
}/* end _ix_cc_atmtx_register_packet_handlers */


/******************************************************************************
 *                            CCI API FUNCTIONS
 *****************************************************************************/

/**
 * NAME: ix_cc_atmtx_init
 *
 * DESCRIPTION:
 *    This primitive shall be called and returned successfully before requesting
 *    any service from the core component. This primitive should be called
 *    only once to initialize the core component. This function performs
 *    allocation of memory for symbols to be patched, Creation of 64 bit
 *    counters, registration of packet and message handlers, allocation and
 *    initialization of internal data structures.
 *              
 * 
 * @Param:  IN arg_hCcHandle - handle to the core component, created by CCI; 
 *                 this shall be used later to get other (to add event handler)
 *                 services from core component infrastructure.
 *
 * @Param:  IN-OUT arg_ppContext - Buffer free list and dynamic property data
 *                 will be given during by system APPs during initialization.
 *                 Upon successfull completion, it represents the location where
 *                 the pointer to the control block allocated by the core
 *                 component will be stored. The control block is internal to
 *                 the core component and it contains information about internal
 *                 data structures, allocated memory and other relevant
 *                 information. Later this pointer to be passed into fini()
 *                 function for freeing memory and other house keeping
 *                 operations when the core component is being destroyed.  
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ERROR_NULL - input parameter arg_ppContext is null
 *    IX_CC_ERROR_OOM   - memory allocation failure      
 *    IX_CC_ATMTX_ERROR_FAILED_PATCHING   - patching failures                      
 *    IX_CC_ERROR_OOM_64BIT_COUNTER - 64 bit counter creation failure
 *    IX_CC_ATMTX_ERROR_REGISTRY  - registry access failures
 *    IX_CC_ATMTX_ERROR_RTM - failure from RTM core component
 *    IX_CC_ATMTX_ERROR_CCI - failure from CCI 
 */
ix_error
ix_cc_atmtx_init(
ix_cc_handle arg_hCcHandle,
void** arg_ppContext)
{
    ix_error err = IX_SUCCESS;
    _ix_cc_atmtx_context_t *pContext;
    ix_cc_init_context *pInitCtx;  /* pointer to variable with static configuration */

    IX_ERROR_CHECK_ARG_NULL(2, arg_ppContext);

    _IX_CC_ATMTX_DBG_INIT(("ix_cc_atmtx_init() START\n"));

    /* Allocate memory for the core component Context */
    pContext = (_ix_cc_atmtx_context_t*)ix_ossl_malloc(sizeof(_ix_cc_atmtx_context_t));
    if (pContext == NULL)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM,
                                ("Failed to allocate memory for ATM TX context"));
    }/* end if (pContext == NULL) */

    /* initialize the structure */
    ix_ossl_memset(pContext, 0, sizeof(_ix_cc_atmtx_context_t));

    /* store ATM TX  component handle in _ix_cc_atmtx_context_t */
    pContext->hAtmTxHandle = arg_hCcHandle;

#ifdef CCI_MULTIINSTANCE
    /* use multiinstance CC ID if applicable */
    pContext->instance = IX_HANDLE_TO_INSTANCE(arg_hCcHandle);
#else
    /* if not, use default instance number */
    pContext->instance = _IX_CC_ATMTX_INSTANCE_SAR_0;
#endif /* CCI_MULTIINSTANCE */

    /* Get the generic init context */
    pInitCtx = (ix_cc_init_context *)*arg_ppContext;

    /* read SAR general configuration from registry */
    IX_ERROR_CR(ix_cc_atmsar_plugin_get_cfg_params(pContext->instance,
                                                   &(pContext->pluginCfg)));
    
    /* Retrieve static configuration from registry. If there is no registry, 
     * then the data structure 
     * will be filled with data from header file. */
    IX_ERROR_CG(_ix_cc_atmtx_registry(pContext), err, cleanupContext);

    /* Allocate memory for symbols to be patched */
    IX_ERROR_CG(_ix_cc_atmtx_alloc_symbols(pContext), err, cleanupContext);

    /* Patch symbols */
    IX_ERROR_CG(_ix_cc_atmtx_patch_symbols(pContext), err, cleanupMemory);

    /* 64 bit counter creation */
    if (pContext->counters)
    {
        IX_ERROR_CG(_ix_cc_atmtx_create_64bit_counters(pContext), err, cleanupMemory);
    }

    /* Register Message Handler */
    IX_ERROR_CG(_ix_cc_atmtx_register_message_handler(pContext), err, cleanupCounter);

    /* Register Packet Handler */
    IX_ERROR_CG(_ix_cc_atmtx_register_packet_handlers(pContext), err, cleanupMessageHandler);

    IX_ERROR_CG(ix_cc_atmsar_plugin_reg_vc_service(IX_CC_ATMTX,
                                                   pContext->instance,
                                                   _ix_cc_atmtx_plugin_vc_create,
                                                   _ix_cc_atmtx_plugin_vc_update,
                                                   _ix_cc_atmtx_plugin_vc_remove,
                                                   _ix_cc_atmtx_plugin_get_vc_stats),
                err,
                cleanupPacketHandlers);

    IX_ERROR_CG(ix_cc_atmsar_plugin_reg_port_service(IX_CC_ATMTX,
                                                     pContext->instance,
                                                     NULL, /* port create is not supported in the CC */
                                                     NULL, /* port remove is not supported in the CC */
                                                     _ix_cc_atmtx_plugin_get_port_stats),
                err,
                cleanupPacketHandlers);

    IX_ERROR_CG(ix_cc_atmsar_plugin_reg_done(IX_CC_ATMTX,
                                             pContext->instance,
                                             pContext),
                err,
                cleanupPacketHandlers);

    *arg_ppContext = (void *)pContext;
    _IX_CC_ATMTX_DBG_INIT(("ix_cc_atmtx_init() SUCCESS\n"));
    return IX_SUCCESS;

  cleanupPacketHandlers:
    _ix_cc_atmtx_unregister_packet_handlers(pContext);
  cleanupMessageHandler:
    _ix_cc_atmtx_unregister_message_handler(pContext);
  cleanupCounter:
    _ix_cc_atmtx_delete_64bit_counters(pContext);
  cleanupMemory:
    _ix_cc_atmtx_free_symbols(pContext);
  cleanupContext:
    ix_ossl_free(pContext);
    _IX_CC_ATMTX_DBG_INIT(( "ix_cc_atmtx_init() FAILED\n"));
    return err;
} /* end ix_cc_atmtx_init */


/**
 * NAME: ix_cc_atmtx_fini
 *
 * DESCRIPTION:
 *    This primitive shall be called to terminate services from this core
 *    component. This primitive frees memory allocated during initialization,
 *    free all the created resources: delete 64 bit counter. If client claims
 *    for any services from this core component after calling this function,
 *    then the behavior is undefined.
 * 
 * @Param:  IN arg_hCcHandle - handle to this core component
 * @Param:  IN arg_pContext - pointer to the control block memory allocated 
 *                 earlier in ix_cc_atmtx_init function. 
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *    IX_CC_ATMTX_ERROR_INVALID_INPUT_PARAM - invalid input parameter(s)
 *    IX_CC_ERROR _OOM - failure of memory deletion                
 *    IX_CC_ERROR_OOM_64BIT_COUNTER - failure of  64 bit counter deletion 
 *    IX_CC_ATMTX_ERROR _RTM - failure from RTM
 *    IX_CC_ATMTX_ERROR_CCI - failure from CCI
 */
ix_error
ix_cc_atmtx_fini(
ix_cc_handle arg_hCcHandle,
void* arg_pContext)
{
    _ix_cc_atmtx_context_t *pCtx=arg_pContext;

    IX_ERROR_CHECK_ARG_NULL(2, arg_pContext);

    if (arg_hCcHandle != pCtx->hAtmTxHandle)
    {
        return IX_ERROR_WARNING(IX_CC_ATMTX_ERROR_INVALID_INPUT_PARAM,
                                ("Invalid input Parameters"));
    }
  
    /* Free memory for symbols */
    IX_ERROR_CR(_ix_cc_atmtx_free_symbols(pCtx));   

    /* Delete 64 bit counter */
    if (pCtx->counters)
    {
        IX_ERROR_CR(_ix_cc_atmtx_delete_64bit_counters(pCtx));
    }
    
    /* Unregister Message Handler */
    IX_ERROR_CR(_ix_cc_atmtx_unregister_message_handler(pCtx));
  
    /* Unregister Packet Handler */
    IX_ERROR_CR(_ix_cc_atmtx_unregister_packet_handlers(pCtx));
    
    /* Free AtmTx Context */ 
    ix_ossl_free(pCtx);
        
    return IX_SUCCESS;
} /* end ix_cc_atmtx_fini */

