/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      ix_cc_csix_tx.h
 *
 * = DESCRIPTION
 *      This file will describe the common definitions that will be exported 
 *      by the CSIX Tx Core Component.
 *
 * = CHANGE HISTORY
 *      6/10/2002 - Created.
 *
 * ============================================================================
 * $Id: ix_cc_csix_tx.h,v 1.27 2003/09/22 15:56:35 ktseng Exp $
 */

#if !defined(__IX_CC_CSIX_TX_H__)
#define __IX_CC_CSIX_TX_H__


/**
 * System defined include files required.
 */
#include "ix_types.h"
#include "ix_error.h"
#include "ix_cc.h"

#ifndef IX_EXCLUDE_CCI
#include "ix_cci.h"
#endif



#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */



/**
 * The following defines the symbols to be patched into the microblock.
 */
#if (_IX_HARDWARE_TYPE_ == _IX_HW_2800_)

#define CSIX_TX_NUM_SYMBOLS_PATCHED     2
#define SYMBOL_CSIX_TX_INGRESS_BLADE_ID "THIS_BLADE_ID"
#define SYMBOL_CSIX_TX_CTX_BASE         "CSIX_TX_CTX_SRAM_BASE"
#define SYMBOL_CSIX_TX_COUNTERS_BASE    "CSIX_TX_COUNTERS_SRAM_BASE"

#else

#define CSIX_TX_NUM_SYMBOLS_PATCHED     3
#define SYMBOL_CSIX_TX_INGRESS_BLADE_ID "THIS_BLADE_ID"
#define SYMBOL_CSIX_TX_CTX_BASE         "CSIX_TX_CTX_SRAM_BASE"
#define SYMBOL_CSIX_TX_COUNTERS_BASE    "CSIX_TX_COUNTERS_SRAM_BASE"

#endif /* (_IX_HARDWARE_TYPE_ == _IX_HW_2800_) */


#ifdef IX_INCLUDE_REGISTRY
/**
 * The following defines the static properties in repository
 */
#define PROP_CSIX_TX_MEV2_NUM_MASK      "CSIX_TX/CSIX_TX_MEV2_NUM_MASK"

#else /* IX_INCLUDE_REGISTRY */

/**
 * The following defines static configuration data.
 * CSIX Tx uses these data when system repository is not included
 * in the system.
 */
#if (_IX_HARDWARE_TYPE_ == _IX_HW_2800_)

#define CSIX_TX_MEV2_NUMBER             0x00000100 /* MEv2 mask for ME8 */

#else /* (_IX_HARDWARE_TYPE_ != _IX_HW_2800_) */

#define CSIX_TX_MEV2_NUMBER             0x00000080 /* MEv2 mask for ME7 */

#endif /* (_IX_HARDWARE_TYPE_ == _IX_HW_2400_) */

#endif /* IX_INCLUDE_REGISTRY */


/**
 * Various definitions for maximum number of VoQs, microblock context 
 * memory size, and SRAM channel number for the context memory.
 */
#define CC_CSIX_TX_NUM_VOQS             1024
#define CC_CSIX_TX_NUM_BYTES_PER_CTX_ENTRY      64
#define CC_CSIX_TX_SIZE_CTX_MEM         (CC_CSIX_TX_NUM_VOQS * CC_CSIX_TX_NUM_BYTES_PER_CTX_ENTRY)
#define CC_CSIX_TX_DEFAULT_SRAM_CH_NUM  0

#if (_IX_HARDWARE_TYPE_ == _IX_HW_2800_)

#define IX_CC_CSIX_TX_MAX_ME_NUM        16 /* max. number of MEv2 */
#define CC_CSIX_TX_DEFAULT_SRAM_CH_NUM_COUNTERS  1 /* Channel number for 32-bit counters */

#else

#define IX_CC_CSIX_TX_MAX_ME_NUM        8 /* max. number of MEv2 */

#endif /* (_IX_HARDWARE_TYPE_ == _IX_HW_2800_) */



/**
 * CSIX Tx error codes
 */
enum ix_cc_csix_tx_error {
    IX_CC_CSIX_TX_ERROR_USER_CB = IX_ERROR_MAKE_GROUP(CC_CSIX_TX),
    IX_CC_CSIX_TX_ERROR_CCI_REG_HDLR, 
    IX_CC_CSIX_TX_ERROR_CCI_REM_HDLR, 
    IX_CC_CSIX_TX_ERROR_RM_PATCH_SYMBOL, 
    IX_CC_CSIX_TX_ERROR_RM_GET_PHYS_OFFSET, 
    IX_CC_CSIX_TX_ERROR_RM_FREE_BUF, 
    IX_CC_CSIX_TX_ERROR_RM_GET_64BIT_COUNTER,
    IX_CC_CSIX_TX_ERROR_RM_REG_HDLR, 
    IX_CC_CSIX_TX_ERROR_RM_REM_HDLR, 
    IX_CC_CSIX_TX_ERROR_MSUP_SEND_REPLY,
    IX_CC_CSIX_TX_ERROR_MSUP_SEND_MSG, 
    IX_CC_CSIX_TX_ERROR_MSUP_EXTRACT_MSG, 
    IX_CC_CSIX_TX_ERROR_OSSL_MUTEX_INIT,
    IX_CC_CSIX_TX_ERROR_OSSL_MUTEX_FINI,
    IX_CC_CSIX_TX_ERROR_OSSL_MUTEX_LOCK 
};



/**
 * defines for message types
 */
typedef enum ix_e_cc_csix_tx_msg_type
{
    IX_CC_CSIX_TX_MSG_GET_STATISTICS_INFO = 0
} ix_cc_csix_tx_msg_type;



/**
 * defines for statistics counters
 */
#define CSIX_TX_NUM_COUNTERS_PER_VOQ    4

typedef enum ix_e_cc_csix_tx_statistics_info
{
    /**
     * Per VOQ CSIX Tx counters.
     */
    IX_CC_CSIX_TX_PACKETS = 0,  /* Tx packet count */
    IX_CC_CSIX_TX_BYTES,        /* Tx byte count */
    IX_CC_CSIX_TX_CFRAMES,      /* Tx C-Frame count */
    IX_CC_CSIX_TX_RESERVED,     /* Reserved counter */
    IX_CC_CSIX_TX_ALL_COUNTERS, /* All above counters of a given VoQ */

    IX_CC_CSIX_TX_COUNTER_LAST  /* End of CSIX Tx counter definition */
} ix_cc_csix_tx_statistics_info;

/*
 * 32-bit counter overflow time.
 * The overflow time is calculated for byte counters which need to be
 * updated more often than the packet counter. For packet counters, we
 * will use the same overflow time since RM 64bit_counter_new function
 * creates array of 64-bit counters with the same overflow time.
 * POS minimum 64-byte packet size and CSIX 16-byte prepend header are 
 * used:
 * 
 *                2^32
 *  ------------------------------ = 10.246 sec
 *   (2.488Gbps/8/46) * (46 + 16)
 * 
 *                2^32
 *  ------------------------------ = 2.549 sec
 *   (10Gbps/8/46) * (46 + 16)
 * 
 */
#if (_IX_HARDWARE_TYPE_ == _IX_HW_2400_)
#define CC_CSIX_TX_32BIT_COUNTER_OVERFLOW_TIME  10000
#elif (_IX_HARDWARE_TYPE_ == _IX_HW_2800_)
#define CC_CSIX_TX_32BIT_COUNTER_OVERFLOW_TIME  2000
#else
#error "Undefined platform type."
#endif


/**
 * The following define API-level data structures.
 */
typedef struct ix_s_cc_csix_tx_statistics_info_context
{
    ix_cc_csix_tx_statistics_info entity;
    ix_uint32 index;
    void *pUserContext;
} ix_cc_csix_tx_statistics_info_context;


typedef struct ix_s_cc_statistics_info_data
{
    ix_uint32 dataLength; /* length in bytes of the data buffer */
    void *pDataBuffer;    /* buffer to store the statistics data */
} ix_cc_csix_tx_statistics_info_data;



/**
 * prototypes for async messaging user callbacks
 */
typedef ix_error (*ix_cc_csix_tx_cb_get_statistics_info)(
    ix_error arg_Result,
    void *arg_pContext,
    ix_uint64 *arg_pBuffer);



/**
 * Prototypes for interface functions.
 */
ix_error ix_cc_csix_tx_init(
    ix_cc_handle arg_CcHandle, 
    void **arg_ppContext);

ix_error ix_cc_csix_tx_fini(
    ix_cc_handle arg_CcHandle, 
    void *arg_pContext);

ix_error ix_cc_csix_tx_msg_handler(
    ix_buffer_handle arg_Msg, 
    ix_uint32 arg_UserData,
    void *arg_pContext);

ix_error ix_cc_csix_tx_async_get_statistics_info(
    ix_cc_csix_tx_statistics_info_context *arg_pMgInfoContext, 
    ix_cc_csix_tx_cb_get_statistics_info arg_Callback);

ix_error ix_cc_csix_tx_get_statistics_info(
    ix_cc_csix_tx_statistics_info arg_Entity,
    ix_uint32 arg_Index,
    ix_cc_csix_tx_statistics_info_data *arg_pBuffer,
    void *arg_pContext);



#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_CSIX_TX_H__) */

