/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.1 for the IXP2000 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      Linux kernel module file.
 *
 * = FILENAME
 *      ix_cc_llc_snap_config_linux_kernel_module.c
 *
 * = DESCRIPTION
 *      The file defines the init and cleanup functions for Linux kernel module.
 *
 * = CREATION TIME
 *      05/06/2003
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: 
 */

#include <linux/module.h>
#include "ix_ossl.h"
#include "ix_cc_llc_snap_config_linux_kernel_module.h"
#include "ix_cc.h"

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

MODULE_LICENSE("Proprietary");

int llc_snap_config_open(struct inode *, struct file *); 
int llc_snap_config_close(struct inode *, struct file *);
int llc_snap_config_ioctl(struct inode *, struct file *, unsigned int, unsigned long);

static struct file_operations llc_snap_config_fops = {
    ioctl: llc_snap_config_ioctl,
    open: llc_snap_config_open,
    release: llc_snap_config_close,
};      

/* dynamic allocation of the Major number from the kernel */
ix_int32 g_MajorNumber = DEV_NUMBER_LLCCONFIG;

/**
 * This function gets called on the insmod system call.
 */
int init_module(void)
{

    ix_ossl_message_log("Loading llc_snap_config utility driver.\n");

    if (register_chrdev(g_MajorNumber, DEV_NAME_LLCCONFIG, &llc_snap_config_fops) == 0)
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("llc_snap_config utility driver is registered with major:%ld\n",g_MajorNumber);
        ix_ossl_message_log("USAGE:\n");
        ix_ossl_message_log("mknod /dev/%s c %ld <minor>\n", DEV_NAME_LLCCONFIG, g_MajorNumber);
        ix_ossl_message_log("with different minor numbers.\n\n");
#endif
    }
    else
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("atsar_config utility driver cannot be registered!!!!.\n");
#endif
        return -1;
    }

    return 0;
} /* init_module */



/**
 * This function gets called on the rmmod system call.
 */
void cleanup_module(void)
{

#ifdef IX_DEBUG
    ix_ossl_message_log("Unloading llc_snap_config utility driver.\n");
#endif

    if (unregister_chrdev(g_MajorNumber, DEV_NAME_LLCCONFIG) == 0)
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("llc_snap_config utility driver cleaned successfully...\n");
#endif
    }
    else
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("llc_snap_config utility driver COULD NOT BE CLEANED UP !!!\n");
#endif
    }

    return;

} /* cleanup_module */



/**
 * This function is called internally, when the user application calls the open
 * function.
 */
int llc_snap_config_open(
    struct inode *llc_snap_config_inode,
    struct file *llc_snap_config_file)
{
#ifdef IX_DEBUG
    ix_ossl_message_log("llc_snap_config_open called.\n");
#endif

    return 0;

} /* llc_snap_config_open */



/**
 * This function is invoked, when the user application calls the close 
 * funtion.
 */
int llc_snap_config_close(
    struct inode *llc_snap_config_inode,
    struct file *llc_snap_config_file)
{
#ifdef IX_DEBUG
    ix_ossl_message_log("llc_snap_config_close called.\n");
#endif

    return 0;

} /* llc_snap_config_close */



/**
 * This function is called by the ioctl command.
 */
int llc_snap_config_ioctl(
    struct inode *llc_snap_config_inode,
    struct file *llc_snap_config_file,
    unsigned int cmd,
    unsigned long arg_IoctlPtr)
{
    ix_uint32 s_Result;
    ix_uint8 cmdString[MAX_NUM_CHAR_CMD_STRING];

#ifdef IX_DEBUG
    ix_ossl_message_log("llc_snap_config_ioctl called.\n");
#endif

    if (llc_snap_config_inode == NULL)
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("llc_snap_config_inode is NULL.\n");
#endif
        return -1;
    }
  
    if (llc_snap_config_file == NULL)
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("llc_snap_config_file is NULL.\n");
#endif
        return -1;
    }        

    /* Copy argument string to kernel space */
    if ((void *)arg_IoctlPtr != NULL)
    {
        strcpy(cmdString, (void *)arg_IoctlPtr);
    }

    switch(cmd)
    {
    case ADD_NEXT_HOP:
        ix_ossl_message_log("ADD_NEXT_HOP called\n");
        s_Result = addNextHop(cmdString);
        break;
    case DEL_NEXT_HOP:
        ix_ossl_message_log("DEL_NEXT_HOP called\n");
        s_Result = delNextHop(cmdString);
        break;
    default:
#ifdef IX_DEBUG
        ix_ossl_message_log("Undefined ioctl command %ld.\n", cmd);
#endif
        s_Result = -1;
        break;
    }

    if (s_Result != 0)
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("llc_snap_config_ioctl failed.\n");
#endif
    }
    else
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("llc_snap_config_ioctl succeeded.\n");
#endif
    }

    return s_Result;

} /* llc_snap_config_ioctl */

#endif




