/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      ix_cc_atm_pos_tx_int.h
 *
 * = DESCRIPTION
 *      This file contains parameters used internally by ATM/POS Tx core 
 *      component.
 *
 * = AUTHOR
 *      Kuo-lang Tseng
 *      Kuo-lang.Tseng@intel.com
 *
 * = CHANGE HISTORY
 *      6/10/2002 - Created.
 *
 * ============================================================================
 * $Id: ix_cc_atm_pos_tx_int.h,v 1.25 2003/10/29 02:14:52 ktseng Exp $
 */

#if !defined(__IX_CC_ATM_POS_TX_INT_H__)
#define __IX_CC_ATM_POS_TX_INT_H__

/**
 * System defined include files required.
 */

#include "ix_types.h"
#include "cc/ix_cc_atm_pos_tx.h"

#include "ix_ossl.h" /* for malloc and free */

#if defined(IX_PLATFORM_2401) || defined(IX_PLATFORM_2801)

/* Platform specific
 * IXDP2401 has a different framer driver
 */
#if (_IX_BOARD_TYPE_ != _IX_IXDP2401_) && (_IX_BOARD_TYPE_ != _IX_IXDP2801_)
/* IXF device driver common includes */
#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h" /* BroadBand Utility Definitions and Function headers */
#include "ixf_api_sonet_d.h"
#include "ixf_api.h"
#include "ixf_api_sonet.h"
#include "ixf6048d.h"
#endif
#else /* IX_PLATFORM_2x01 */
/* IXF device driver common includes */
#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h" /* BroadBand Utility Definitions and Function headers */
#include "ixf_api_sonet_d.h"
#include "ixf_api.h"
#include "ixf_api_sonet.h"
#include "ixf6048d.h"
#endif /* IX_PLATFORM_2x01 */


/**
 * User defined include files required.
 */


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */


#if defined(IX_PLATFORM_2401) || defined(IX_PLATFORM_2801)
/* IGK0100023678 bug fixed */
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
  typedef int BOOL;
#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif
#endif /* _IX_OS_LINUX_KERNEL_ */
#endif /* IX_PLATFORM_2x01 */

/**
 * Prototypes for internal functions.
 */



/* Size defines for callback context tables */
#define ATM_POS_TX_NUM_ENTRIES_CB_CTX_TBL_COMMON        128 /* must be power of 2 */
#define ATM_POS_TX_NUM_ENTRIES_CB_CTX_TBL_GET_STAT      ATM_POS_TX_NUM_ENTRIES_CB_CTX_TBL_COMMON
#define ATM_POS_TX_SIZE_CB_CTX_TBL_GET_STAT             (ATM_POS_TX_NUM_ENTRIES_CB_CTX_TBL_GET_STAT * sizeof(ix_cc_atm_pos_tx_cb_ctx_entry))
#define ATM_POS_TX_NUM_ENTRIES_CB_CTX_TBL_GET_IF        ATM_POS_TX_NUM_ENTRIES_CB_CTX_TBL_COMMON
#define ATM_POS_TX_SIZE_CB_CTX_TBL_GET_IF               (ATM_POS_TX_NUM_ENTRIES_CB_CTX_TBL_GET_IF * sizeof(ix_cc_atm_pos_tx_cb_ctx_entry))


/* various size definitions for 32bit and 64-bit counter support */
#define POS_TX_NUM_COUNTERS_PER_PORT    4
#define POS_TX_TOTAL_NUM_COUNTERS       (IX_CC_POS_TX_MAX_NUM_POS_PORTS * POS_TX_NUM_COUNTERS_PER_PORT)
#define POS_TX_SIZE_32BIT_COUNTER_BUF   (POS_TX_TOTAL_NUM_COUNTERS * sizeof(ix_uint32))
#define POS_TX_SIZE_64BIT_COUNTER_HDL   (POS_TX_TOTAL_NUM_COUNTERS * sizeof(ix_counter_64bit_handle))
#define POS_TX_SIZE_NEXT_AVAIL_TBUF_ELEM 4 /* in byte */



/* Definitions and masks for various operation modes for ATM/POS framer setup */
#define FRAMER_OC_MODE_MASK             0x0000F000 /* Bit [15:12] */
#define FRAMER_LINE_MODE_MASK           0x00000F00 /* Bit [11:8] */
#define FRAMER_CHANNEL_MODE_MASK        0x000000C0 /* Bit [7:6] */
#define FRAMER_UPHY_MODE_MASK           0x0000003F /* Bit [5:0] */

#define FRAMER_OC48C_MODE               0x00000000 /* Bit [15:12] = 0000 */
#define FRAMER_OC48_MODE                0x00001000 /* Bit [15:12] = 0001 */
#define FRAMER_OC12C_MODE               0x00002000 /* Bit [15:12] = 0010 */
#define FRAMER_OC12_MODE                0x00003000 /* Bit [15:12] = 0011 */
#if defined(IX_PLATFORM_2401) || defined(IX_PLATFORM_2801)

  /* Platform specific: Define additional mod used by IXDP2401
   */
#define FRAMER_OC3_MODE                 0x00004000 /* Bit [15:12] = 0100 */
#endif /*IX_PLATFORM_2401 || IX_PLATFORM_2801 */

#define FRAMER_1_16_PECL_LINE_MODE      0x00000000 /* Bit [11:8] = 0000 */
#define FRAMER_4_8_TTL_LINE_MODE        0x00000100 /* Bit [11:8] = 0001 */
#define FRAMER_1_8_TTL_LINE_MODE        0x00000200 /* Bit [11:8] = 0010 */

#define FRAMER_ATM_CHANNEL_MODE         0x00000080 /* Bit [7:6] = 10 */
#define FRAMER_POS_CHANNEL_MODE         0x000000C0 /* Bit [7:6] = 11 */

#define FRAMER_1_32_L2_UPHY_MODE        0x00000000 /* Bit [5:0] = 000000 */
#define FRAMER_4_8_L1_UPHY_MODE         0x00000001 /* Bit [5:0] = 000001 */
#define FRAMER_1_32_L3_UPHY_MODE        0x00000002 /* Bit [5:0] = 000010 */
#define FRAMER_4_8_L3_UPHY_MODE         0x00000003 /* Bit [5:0] = 000011 */



/**
 * Operation mode structure
 */
typedef struct s_ix_cc_atm_pos_op_mode
{
    ix_uint32 ocMode;     /* OC type */
    ix_uint32 lineMode;   /* Line mode */
    ix_uint32 utopiaMode; /* Utopia mode */
} ix_cc_atm_pos_op_mode;



/**
 * The following two definitions define the allowable numbers of POS and ATM 
 * modes based on the specification from ATM/POS media cards. (see also the 
 * component HLD.
 */
#if defined(IX_PLATFORM_2401) || defined(IX_PLATFORM_2801)

  /* Platform specific: with the additional FRAMER_OC3_MODE there are 5
   * possible POS modes now.
   */
#define IX_CC_NUM_POS_MODES 5
#else
#define IX_CC_NUM_POS_MODES 4
#endif /* IX_PLATFORM_2x01 */

#define IX_CC_NUM_ATM_MODES 8



/**
 * Callback context entry structure
 */
typedef struct s_ix_cc_atm_pos_tx_cb_ctx_entry
{
    ix_uint8 flags;     /* entry flag */
    void *pUserCtx;     /* user context */
    void *pUserCb;      /* user callback */
} ix_cc_atm_pos_tx_cb_ctx_entry;



/* values for flags field of the callback context entry structure */
#define CC_ATM_POS_TX_ENTRY_FLAGS_NOT_USED     0x00 /* free entry */
#define CC_ATM_POS_TX_ENTRY_FLAGS_USED_STATIC  0x01 /* used static entry */
#define CC_ATM_POS_TX_ENTRY_FLAGS_USED_DYNAMIC 0x02 /* used dynamic entry */



/**
 * Callback context entry table structure
 */
typedef struct s_cc_atm_pos_tx_cb_ctx_tbl
{
    ix_uint32 index;                       /* array index pointer */
    ix_cc_atm_pos_tx_cb_ctx_entry *pTable; /* free cb context entry array */ 
} cc_atm_pos_tx_cb_ctx_tbl;



/**
 * Control block structure
 */
typedef struct ix_s_cc_atm_pos_tx_ctl_blk
{
    /**
     * Operation modes as defined in HLD.
     */
    ix_uint32 modes;

    /**
     * Number of Utopia channels. This will be derived from the modes.
     */
    ix_uint32 numChannels;

    /**
     * Applicable POS mode number and ATM mode number.
     * (See g_aApplicablePosMode and g_aApplicableAtmMode tables for
     * available mode number.)
     */
    ix_uint32 posModeNum;
    ix_uint32 atmModeNum;

    /**
     * POS mode or ATM mode enable flags.
     */
    BOOL posEnable;
    BOOL atmEnable;

    /**
     * Local blade id and port id of each Utopia channel.  
     * This will be derived from ix_cc_init_context structure.
     */
    ix_uint32 portId[IX_CC_POS_TX_MAX_NUM_POS_PORTS];
    ix_uint32 bladeId;

    /**
     * Port status.  This is maintained by the library API to 
     * enable and disable the channel interface.
     */
    ix_cc_atm_pos_tx_if_state portState[IX_CC_POS_TX_MAX_NUM_POS_PORTS];

    /**
     * MEv2 numbers for ATM Tx and POS Tx microblocks.
     */
    ix_uint32 atmTxMev2Num; /* ATM Tx microblock MEv2 number */
    ix_uint32 posTxMev2Num; /* POS Tx microblock MEv2 number */

    /**
     * Symbols and context base to be patched into the microblocks.
     */
    ix_uint32 pPosTxNextAvailTbufElem; /* POS Tx tbuf element */
    ix_uint32 *pAtmTxCtx;              /* ATM Tx context memory */

    /**
     * based addresses of 64-bit counter handler memories.
     */
    ix_counter_64bit_handle *pAtmCounterHandlers64Bit; /* ATM Tx */
    ix_counter_64bit_handle *pPosCounterHandlers64Bit; /* POS Tx */

    /**
     * base addresses of 32-bit counter memories.
     */
    ix_uint32 *pAtmCounterBase32Bit; /* ATM Tx microblock */
    ix_uint32 *pPosCounterBase32Bit; /* POS Tx microblock */

    /**
     * Free callback context entry tables used by each async helper API.
     */
    cc_atm_pos_tx_cb_ctx_tbl *pCbCtxTblGetStatInfo;
    cc_atm_pos_tx_cb_ctx_tbl *pCbCtxTblGetIfState;

    /**
     * Mutex id used used by the component in critical sections.
     */
    ix_ossl_mutex_t mid;

#if defined(IX_PLATFORM_2801)
#if (_IX_BOARD_TYPE_ == _IX_IXDP2801_)
    /* Platform specific
     * Additionally init context stored for accessing interface list for
     * configuration purposes
     */
    ix_cc_init_context *pInitCtx;
#endif
#endif    

#if defined(IX_PLATFORM_2401) || defined(IX_PLATFORM_2801)
  /* Platform specific
   * Descriptors table for opened driver special files.
   */
#if (_IX_BOARD_TYPE_ == _IX_IXDP2401_) 
    int posFd;
#elif(_IX_BOARD_TYPE_ == _IX_IXDP2801_)
    int posFd[IX_CC_POS_TX_MAX_NUM_POS_PORTS];
#else
    /**
     * Pointer to IXF6048 framer device driver specific handle structure.
     */
    bb_ChipData_t *pChipData;
    ixf6048_ChanCfg_t *pChanCfg;

    /**
     * Pointer to IXF6048 register table.
     */
    InitRegTable_t *ixf6068RegInitTable;
#endif /* _IX_BOARD_TYPE_ */

#else /* IX_PLATFORM_2401 || IX_PLATFORM_2801 */
    /**
     * Pointer to IXF6048 framer device driver specific handle structure.
     */
    bb_ChipData_t *pChipData;
    ixf6048_ChanCfg_t *pChanCfg;

    /**
     * Pointer to IXF6048 register table.
     */
    InitRegTable_t *ixf6068RegInitTable;
#endif /* IX_PLATFORM_2x01 */
} ix_cc_atm_pos_tx_ctl_blk;



/**
 * The following define the data structures for slow port initialization.
 */
typedef struct ix_s_cc_atm_pos_tx_slow_port_reg
{
    ix_uint32 reg;
    ix_uint8 value;
} ix_cc_atm_pos_tx_slow_port_reg;

#define IX_CC_ATM_POS_NUM_SLOW_PORT_REGS        8


/**
 * MACRO NAME: IX_CC_ATM_POS_TX_ALLOC_ENTRY
 *
 * DESCRIPTION: This macro allocates a user specified structure.
 *              The structure is either allocated from the free entry array
 *              passed in by the user or from OSSL if one is not 
 *              available from the array.
 *
 * @Param: arg_pTbl - IN: pointer to the free entry array.
 * @Param: arg_CurIndex - IN: current index to the array.
 * @Param: arg_TblSize - IN: size of the table (MUST be a power of 2)
 * @Param: arg_Type - IN: data type of the array element.
 *
 * @Param: arg_pEntry - OUT: pointer to the free entry.
 *
 * @Return:  - None
 */
#define IX_CC_ATM_POS_TX_ALLOC_ENTRY(                                \
    arg_pTbl,                                                        \
    arg_CurIndex,                                                    \
    arg_TblSize,                                                     \
    arg_Type,                                                        \
    arg_pEntry)                                                      \
{                                                                    \
                                                                     \
    arg_pEntry = arg_pTbl + arg_CurIndex;                            \
                                                                     \
    if (arg_pEntry->flags == CC_ATM_POS_TX_ENTRY_FLAGS_NOT_USED)     \
    {                                                                \
        arg_pEntry->flags = CC_ATM_POS_TX_ENTRY_FLAGS_USED_STATIC;   \
        arg_CurIndex = (arg_CurIndex + 1) & (arg_TblSize - 1);       \
    }                                                                \
    else                                                             \
    {                                                                \
        if ((arg_pEntry = (arg_Type *)ix_ossl_malloc(sizeof(arg_Type))) != NULL) \
        {                                                            \
            arg_pEntry->flags = CC_ATM_POS_TX_ENTRY_FLAGS_USED_DYNAMIC; \
        }                                                            \
    }                                                                \
}



/**
 * MACRO NAME: IX_CC_ATM_POS_TX_FREE_ENTRY
 *
 * DESCRIPTION: This macro frees a structure.
 *              The structure may be freed back to system memory if 
 *              was allocated from it.
 *
 * @Param: arg_pEntry - IN: pointer to the entry to be freed.
 *
 * @Return:  - None
 */
#define IX_CC_ATM_POS_TX_FREE_ENTRY(                                   \
    arg_pEntry)                                                        \
{                                                                      \
                                                                       \
    if (arg_pEntry->flags == CC_ATM_POS_TX_ENTRY_FLAGS_USED_STATIC)    \
    {                                                                  \
        arg_pEntry->flags = CC_ATM_POS_TX_ENTRY_FLAGS_NOT_USED;        \
    }                                                                  \
    else if (arg_pEntry->flags == CC_ATM_POS_TX_ENTRY_FLAGS_USED_DYNAMIC) \
    {                                                                  \
        ix_ossl_free(arg_pEntry);                                      \
    }                                                                  \
}



#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_ATM_POS_TX_INT_H__) */
