/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      ix_cc_atm_pos_tx.h
 *
 * = DESCRIPTION
 *      This file will describe the common definitions that will be exported 
 *      by the ATM/POS Tx Core Component.
 *
 * = CHANGE HISTORY
 *      6/10/2002 - Created.
 *
 * ============================================================================
 * $Id: ix_cc_atm_pos_tx.h,v 1.34 2003/10/29 02:13:23 ktseng Exp $
 */

#if !defined(__IX_CC_ATM_POS_TX_H__)
#define __IX_CC_ATM_POS_TX_H__



/**
 * System defined include files required.
 */
#include "ix_types.h"
#include "ix_error.h"
#include "ix_cc.h"

#ifndef IX_EXCLUDE_CCI
#include "ix_cci.h"
#endif



#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */



/**
 * The following defines static configuration data.
 * ATM/POS Tx uses the #define values when system repository is not included
 * in the system.
 */
#ifdef IX_INCLUDE_REGISTRY

#define PROP_ATM_TX_MEV2_NUM_MASK       "ATM_POS_TX/ATM_TX_MEV2_NUM_MASK"
#define PROP_POS_TX_MEV2_NUM_MASK       "ATM_POS_TX/POS_TX_MEV2_NUM_MASK"

#else /* IX_INCLUDE_REGISTRY */

#define ATM_TX_MEV2_NUMBER              0x00000060 /* default ME5 & 6 */
#define POS_TX_MEV2_NUMBER              0x00000010 /* default ME4 */

#endif /* IX_INCLUDE_REGISTRY */



/**
 * The following defines ATM/POS framer operation mode based on 
 * the build flag. (See also HLD for the definition of each POS mode)
 */
#if (defined IX_PLATFORM_2401) || (defined IX_PLATFORM_2801)

  /* Platform dependent
   * Added POS3 mode for IXDP2401
   */
#if defined(IX_CONFIGURATION_oc48_sphy_pos_egress) 
#define ATM_POS_OPERATION_MODES         0x000000C0   /* POS mode 0 */

#elif defined(IX_CONFIGURATION_oc12_sphy4_pos_egress)
#define ATM_POS_OPERATION_MODES         0x000010C1   /* POS mode 1 */

#elif defined(IX_CONFIGURATION_oc12_mphy4_pos_egress)
#define ATM_POS_OPERATION_MODES         0x000021C1   /* POS mode 2 */

#elif defined(IX_CONFIGURATION_oc3_mphy4_pos_egress)
#define ATM_POS_OPERATION_MODES         0x000041C1   /* POS mode 3 */

#else

/* by default POS mode 3 */
#define ATM_POS_OPERATION_MODES         0x000041C1   /* POS mode 3 */

#endif

#else /* IX_PLATFORM_2401 || IX_PLATFORM_2801*/

#if defined(IX_CONFIGURATION_oc48_sphy_pos_egress) 
#define ATM_POS_OPERATION_MODES         0x000000C0   /* POS mode 0 */

#elif defined(IX_CONFIGURATION_oc12_sphy4_pos_egress)
#define ATM_POS_OPERATION_MODES         0x000010C1   /* POS mode 1 */

#elif defined(IX_CONFIGURATION_oc12_mphy4_pos_egress)
#define ATM_POS_OPERATION_MODES         0x000021C1   /* POS mode 2 */

#else
#error "Framer POS operation mode selection not supported by the device driver."

#endif

#endif /* IX_PLATFORM_2401 || IX_PLATFORM_2801 */

/**
 * The following defines the symbols to be patched into the microblock.
 */
#define POS_TX_NUM_SYMBOLS_PATCHED              1
#define SYMBOL_POS_TX_NEXT_AVAIL_TBUF_ELEM      "PACKET_TX_AVAIL_TBUF_ELE_SCR_ADDR" /* multi mode only */
#define SYMBOL_POS_TX_COUNTERS_BASE             "PACKET_TX_COUNTER_BASE"



/* define for IXF6048 framer device base address */
#define FRAMER_CHIP_BASE_ADDRESS        0xC6000000



/**
 * Various definitions for maximum number of ports, SRAM channel 
 * number for the context memory, ME number.
 */
#if defined(IX_PLATFORM_2801)
/* Platform specific. Constants adjusted for an IXDP2801.
 */
#define IX_CC_POS_TX_MAX_NUM_POS_PORTS  8  /* max. number of POS ports */
#define IX_CC_ATM_POS_TX_MAX_ME_NUM     16 /* max. number of MEv2 */
#define CC_POS_TX_DEFAULT_SRAM_CH_NUM   1  /* SRAM channel number */ 
#define CC_POS_TX_DEFAULT_SCR_CH_NUM    0  /* scratch memory channel number */

#else //NOT IX_PLATFORM_2801

#define IX_CC_POS_TX_MAX_NUM_POS_PORTS  16 /* max. number of POS ports */
#define IX_CC_ATM_POS_TX_MAX_ME_NUM     8  /* max. number of MEv2 */
#define CC_POS_TX_DEFAULT_SRAM_CH_NUM   0  /* SRAM channel number */ 
#define CC_POS_TX_DEFAULT_SCR_CH_NUM    0  /* scratch memory channel number */

#endif //NOT IX_PLATFORM_2801
#if (defined IX_PLATFORM_2401) || (defined IX_PLATFORM_2801)

#define IX_POS_OC48_LINK_SPEED      2488
#define IX_POS_OC12_LINK_SPEED      622
#define IX_POS_OC3_LINK_SPEED       155

#define IX_POS_DB1_MASK             0x0000000F
#define IX_POS_DB2_MASK             0x000000F0

#endif /*defined IX_PLATFORM_2401) || (defined IX_PLATFORM_2801 */

/**
 * ATM/POS Tx error codes. 
 */
enum ix_cc_atm_pos_tx_error {
    IX_CC_ATM_POS_TX_ERROR_USER_CB = IX_ERROR_MAKE_GROUP(CC_POS_TX),
    IX_CC_ATM_POS_TX_ERROR_CCI_REG_HDLR, 
    IX_CC_ATM_POS_TX_ERROR_CCI_REM_HDLR, 
    IX_CC_ATM_POS_TX_ERROR_RM_PATCH_SYMBOL, 
    IX_CC_ATM_POS_TX_ERROR_RM_FREE_MEM, 
    IX_CC_ATM_POS_TX_ERROR_RM_FREE_BUF,
    IX_CC_ATM_POS_TX_ERROR_RM_GET_64BIT_COUNTER,
    IX_CC_ATM_POS_TX_ERROR_RM_GET_DATA,
    IX_CC_ATM_POS_TX_ERROR_RM_REG_HDLR, 
    IX_CC_ATM_POS_TX_ERROR_RM_REM_HDLR, 
    IX_CC_ATM_POS_TX_ERROR_RM_GET_PHYS_OFFSET,
    IX_CC_ATM_POS_TX_ERROR_FRAMER_INIT, 
    IX_CC_ATM_POS_TX_ERROR_FRAMER_RESET, 
    IX_CC_ATM_POS_TX_ERROR_FRAMER_ALLOC, 
    IX_CC_ATM_POS_TX_ERROR_FRAMER_DEALLOC, 
    IX_CC_ATM_POS_TX_ERROR_FRAMER_INIT_CHIP, 
    IX_CC_ATM_POS_TX_ERROR_FRAMER_SET_CHIP_ONLINE, 
    IX_CC_ATM_POS_TX_ERROR_FRAMER_SET_CHIP_OFFLINE, 
    IX_CC_ATM_POS_TX_ERROR_FRAMER_GET_CH_STATUS, 
    IX_CC_ATM_POS_TX_ERROR_FRAMER_ALARM_CB, 
    IX_CC_ATM_POS_TX_ERROR_MSUP_SEND_REPLY,
    IX_CC_ATM_POS_TX_ERROR_MSUP_SEND_MSG, 
    IX_CC_ATM_POS_TX_ERROR_MSUP_EXTRACT_MSG, 
    IX_CC_ATM_POS_TX_ERROR_INVALID_BLADE_ID,
    IX_CC_ATM_POS_TX_ERROR_INVALID_IF_STATE,
    IX_CC_ATM_POS_TX_ERROR_OSSL_MUTEX_INIT,
    IX_CC_ATM_POS_TX_ERROR_OSSL_MUTEX_FINI,
    IX_CC_ATM_POS_TX_ERROR_OSSL_MUTEX_LOCK 
};



/**
 * defines for message types
 */
typedef enum ix_e_cc_atm_pos_tx_msg_type
{
    IX_CC_ATM_POS_TX_MSG_GET_STATISTICS_INFO,
    IX_CC_ATM_POS_TX_MSG_GET_INTERFACE_STATE
} ix_cc_atm_pos_tx_msg_type;



/**
 * defines for statistics counters
 */
#define ATM_TX_NUM_COUNTERS_PER_VCQ     2
#define ATM_TX_NUM_COUNTERS_PER_PORT    2
#define POS_TX_NUM_COUNTERS_PER_PORT    4
#define POS_TX_NUM_COUNTERS_PER_CH      POS_TX_NUM_COUNTERS_PER_PORT

typedef enum ix_e_cc_atm_pos_tx_statistics_info
{
    /**
     * per VCQ ATM Tx counters.
     */
    IX_CC_ATM_TX_PACKETS_VCQ = 0,   /* 64-bit ATM Tx packet count */
    IX_CC_ATM_TX_CELLS_VCQ,         /* 64-bit ATM Tx cell count */
    IX_CC_ATM_TX_ALL_COUNTERS_VCQ,  /* All above ATM counters of a given VCQ */

    /**
     * per port ATM Tx counters.
     */
    IX_CC_ATM_TX_PACKETS_PORT,      /* 64-bit ATM Tx packet count */
    IX_CC_ATM_TX_CELLS_PORT,        /* 64-bit ATM Tx cell count */
    IX_CC_ATM_TX_ALL_COUNTERS_PORT, /* All above ATM counters of a given port */

    /**
     * per port POS Tx counters.
     */
    IX_CC_POS_TX_PACKETS,           /* 64-bit POS Tx packet count */
    IX_CC_POS_TX_DROP_PACKETS,      /* 64-bit POS drop packet count */
    IX_CC_POS_TX_BYTES,             /* 64-bit POS Tx byte count */
    IX_CC_POS_TX_DROP_BYTES,        /* 64-bit POS Tx drop byte count */
    IX_CC_POS_TX_ALL_COUNTERS,      /* All above POS counters of a given port */

    IX_CC_ATM_POS_TX_COUNTER_LAST   /* End of ATM/POS Tx counter definition */
} ix_cc_atm_pos_tx_statistics_info;

/*
 * 32-bit counter overflow time.
 * The overflow time is calculated for byte counters which need to be
 * updated more often than the packet counter. For packet counters, we
 * will use the same overflow time since RM 64bit_counter_new function
 * creates array of 64-bit counters with the same overflow time.
 * POS minimum 64-byte packet size is used:
 * 
 *      2^32
 *  ------------- = 13.81 sec  (IXP2400)
 *   2.488Gbps/8
 * 
 *      2^32
 *  ------------- = 3.436 sec  (IXP2800)
 *   10Gbps/8
 * 
 */
#if (_IX_HARDWARE_TYPE_ == _IX_HW_2400_)
#define CC_POS_TX_32BIT_COUNTER_OVERFLOW_TIME   13000
#elif (_IX_HARDWARE_TYPE_ == _IX_HW_2800_)
#define CC_POS_TX_32BIT_COUNTER_OVERFLOW_TIME   3000
#else
#error "Undefined platform type."
#endif


/**
 * The following define the API-level data structures.
 */
typedef struct ix_s_cc_atm_pos_tx_statistics_info_context
{
    ix_cc_atm_pos_tx_statistics_info entity;
    ix_uint32 index;
    void *pUserContext;
} ix_cc_atm_pos_tx_statistics_info_context;


typedef struct ix_s_cc_atm_pos_tx_statistics_info_data
{
    ix_uint32 dataLength; /* length in bytes of the data buffer */
    void *pDataBuffer;    /* buffer to store the statistics data */
} ix_cc_atm_pos_tx_statistics_info_data;


typedef struct ix_s_cc_atm_pos_tx_if_state_context
{
    ix_uint32 portId;
    void *pUserContext;
} ix_cc_atm_pos_tx_if_state_context;


typedef enum ix_e_cc_atm_pos_tx_if_state
{
    IX_CC_ATM_POS_TX_IF_STATE_DOWN = 0,
    IX_CC_ATM_POS_TX_IF_STATE_UP
} ix_cc_atm_pos_tx_if_state;



/**
 * prototypes for async messaging user callbacks
 */
typedef ix_error (*ix_cc_atm_pos_tx_cb_get_statistics_info)(
    ix_error arg_Result,
    void *arg_pContext,
    ix_uint64 *arg_pBuffer,
    ix_uint32 arg_MsgLen);

typedef ix_error (*ix_cc_atm_pos_tx_cb_get_interface_state)(
    ix_error arg_Result,
    void *arg_pContext,
    ix_cc_atm_pos_tx_if_state *arg_pState);


/**
 * Prototypes for interface functions.
 */
ix_error ix_cc_atm_pos_tx_init(
    ix_cc_handle arg_CcHandle, 
    void **arg_ppContext);

ix_error ix_cc_atm_pos_tx_fini(
    ix_cc_handle arg_CcHandle, 
    void *arg_pContext);

ix_error ix_cc_atm_pos_tx_msg_handler(
    ix_buffer_handle arg_Msg, 
    ix_uint32 arg_UserData,
    void *arg_pContext);

ix_error ix_cc_atm_pos_tx_property_msg_handler(
    ix_buffer_handle arg_Msg, 
    ix_uint32 arg_UserData,
    void *arg_pContext);

ix_error ix_cc_atm_pos_tx_async_get_statistics_info(
    ix_cc_atm_pos_tx_statistics_info_context *arg_pMgInfoContext, 
    ix_cc_atm_pos_tx_cb_get_statistics_info arg_Callback);

ix_error ix_cc_atm_pos_tx_async_get_interface_state(
    ix_cc_atm_pos_tx_if_state_context *arg_pStateContext, 
    ix_cc_atm_pos_tx_cb_get_interface_state arg_Callback);

ix_error ix_cc_atm_pos_tx_get_statistics_info(
    ix_cc_atm_pos_tx_statistics_info arg_Entity,
    ix_uint32 arg_Index,
    ix_cc_atm_pos_tx_statistics_info_data *arg_pBuffer,
    void *arg_pContext);

ix_error ix_cc_atm_pos_tx_get_interface_state(
    ix_uint32 arg_PortId, 
    ix_cc_atm_pos_tx_if_state *arg_pIfState,
    void *arg_pContext);

ix_error ix_cc_atm_pos_tx_set_property(
    ix_uint32 arg_PropId,
    ix_cc_properties *arg_pProperty,
    void *arg_pContext);

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_ATM_POS_TX_H__) */

