/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2X00 Network Processor, Release 5
 *
 * = FILENAME
 *      ix_cc_atm_pos_tx.c
 *
 * = DESCRIPTION
 *      This file contains ATM/POS Tx CC interface APIs and internal
 *      functions common to both level-0 and level-1 systems.
 *
 * = CHANGE HISTORY
 *      6/10/2002 - Created.
 *
 * ============================================================================
 * $Id: ix_cc_atm_pos_tx.c,v 1.53 2003/10/29 02:23:22 ktseng Exp $
 */

/**
 * System defined include files required.
 */
#include <ix_ossl.h>
#include <ix_rm.h>
#ifndef IX_EXCLUDE_CCI
#include <ix_cci.h>
#endif

/**
 * User defined include files required.
 */
#include "ix_cc_error.h"
#include "ix_types.h"
#include "ix_error.h"
#include "ix_cc.h"
#include "bindings.h"
#include "ix_cc_properties.h"

#include "cc/ix_cc_atm_pos_tx.h"
#include "cc/internal/ix_cc_atm_pos_tx_int.h"

#if defined(IX_PLATFORM_2401) || defined(IX_PLATFORM_2801)

/* Platform specific
 * IXDP2401 platform has a different framer driver
 */
#if (_IX_BOARD_TYPE_ == _IX_IXDP2401_) || (_IX_BOARD_TYPE_ == _IX_IXDP2801_)
#include <pos_atm_media_drv.h>
#else
#include "cc/internal/ix_cc_atm_pos_tx_ixf6048_regs.h"
#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h" /* BroadBand Utility Definitions and Function headers */
#include "ixf_api_sonet_d.h"
#include "ixf_api.h"
#include "ixf_api_sonet.h"

/* ixf6048 specific device driver */
#include "ixf6048d.h" /* To fix header dependency problem of ixfapi files */
#include "ixf6048f.h"
#endif /* _IX_BOARD_TYPE_ */

#else  /* IX_PLATFORM_2401 || IX_PLATFORM_2801 */

#include "cc/internal/ix_cc_atm_pos_tx_ixf6048_regs.h"

/* IXF device driver common includes */
#include "common_types.h"
#include "common_def.h"
#include "ixf_api_d.h" /* BroadBand Utility Definitions and Function headers */
#include "ixf_api_sonet_d.h"
#include "ixf_api.h"
#include "ixf_api_sonet.h"

/* ixf6048 specific device driver */
#include "ixf6048d.h" /* To fix header dependency problem of ixfapi files */
#include "ixf6048f.h"

#endif /* IX_PLATFORM_2401 || IX_PLATFORM_2801*/

#if (defined IX_PLATFORM_2401) || (defined IX_PLATFORM_2801)
#if (defined USE_BB_DRV_CONFIG)
/* extern definition for HW_CONFIG information provided in xml config file */
ix_uint32 hwConfigPosPortMask;	/* POS ports Mask */
#endif /* USE_BB_DRV_CONFIG */
#endif /* IX_PLATFORM_2x01 */

ix_error _ix_cc_pos_retrieve_registry_data(const char* arg_pPropertyName,
                                           ix_uint32* arg_pPropertyValue)
{
    ix_configuration_property_handle hPosPropertyHandle;
    ix_error err = IX_SUCCESS;

    /* Get static data from registry */
    err = ix_rm_cp_property_open(IX_CP_CORE_PROPERTY_ROOT,
                                 arg_pPropertyName,
                                 &hPosPropertyHandle);

    if( err == IX_SUCCESS)
    {
    	err = ix_rm_cp_property_get_value_uint32(hPosPropertyHandle,
                                                 arg_pPropertyValue);
    }

    if( err == IX_SUCCESS)
    {
        err = ix_rm_cp_property_close(hPosPropertyHandle);
    }
    return err;
}/* end _ix_cc_pos_retrieve_registry_data */


/**
 * Variable declarations global to this file only.  Extern are followed
 * by static variables.
 */
/**
 * The following two tables define the allowable POS and ATM modes based
 * on the specification from ATM/POS media cards. (see also the
 * component HLD for the definition of each mode)
 */
ix_cc_atm_pos_op_mode g_aApplicablePosMode[IX_CC_NUM_POS_MODES] =
{
    /* POS mode 0 */
    {FRAMER_OC48C_MODE, FRAMER_1_16_PECL_LINE_MODE, FRAMER_1_32_L2_UPHY_MODE},

    /* POS mode 1 */
    {FRAMER_OC48_MODE, FRAMER_1_16_PECL_LINE_MODE, FRAMER_4_8_L1_UPHY_MODE},

    /* POS mode 2 */
    {FRAMER_OC12C_MODE, FRAMER_4_8_TTL_LINE_MODE, FRAMER_4_8_L1_UPHY_MODE},

    /* POS mode 3 */
    {FRAMER_OC12_MODE, FRAMER_1_8_TTL_LINE_MODE, FRAMER_4_8_L1_UPHY_MODE}

#if defined(IX_PLATFORM_2401) || defined(IX_PLATFORM_2801)

  /* Platform dependent
   * Added POS mode for IXDP2401
   */
    /* POS mode 4 */
    ,{FRAMER_OC3_MODE, FRAMER_4_8_TTL_LINE_MODE, FRAMER_4_8_L1_UPHY_MODE}
#endif /*IX_PLATFORM_2401 || IX_PLATFORM_2801 */
};

/* The table below will only be used in PR6 for ATM tx */
ix_cc_atm_pos_op_mode g_aApplicableAtmMode[IX_CC_NUM_ATM_MODES] =
{
    /* ATM mode 0 */
    {FRAMER_OC48C_MODE, FRAMER_1_16_PECL_LINE_MODE, FRAMER_1_32_L3_UPHY_MODE},

    /* ATM mode 1 */
    {FRAMER_OC48_MODE, FRAMER_1_16_PECL_LINE_MODE, FRAMER_4_8_L3_UPHY_MODE},

    /* ATM mode 2 */
    {FRAMER_OC12C_MODE, FRAMER_4_8_TTL_LINE_MODE, FRAMER_1_32_L3_UPHY_MODE},

    /* ATM mode 3 */
    {FRAMER_OC12C_MODE, FRAMER_4_8_TTL_LINE_MODE, FRAMER_4_8_L1_UPHY_MODE},

    /* ATM mode 4 */
    {FRAMER_OC12C_MODE, FRAMER_4_8_TTL_LINE_MODE, FRAMER_4_8_L3_UPHY_MODE},

    /* ATM mode 5 */
    {FRAMER_OC12_MODE, FRAMER_1_8_TTL_LINE_MODE, FRAMER_1_32_L3_UPHY_MODE},

    /* ATM mode 6 */
    {FRAMER_OC12_MODE, FRAMER_1_8_TTL_LINE_MODE, FRAMER_4_8_L1_UPHY_MODE},

    /* ATM mode 7 */
    {FRAMER_OC12_MODE, FRAMER_1_8_TTL_LINE_MODE, FRAMER_4_8_L3_UPHY_MODE}
};



/* slow port initializatiuon register/value pairs */
ix_cc_atm_pos_tx_slow_port_reg g_aSlowPortRegTable[IX_CC_ATM_POS_NUM_SLOW_PORT_REGS] =
{
    {0xC0080000, 0x3},
    {0xC0080004, 0x68},
    {0xC0080008, 0x91},
    {0xC008000C, 0x68},
    {0xC0080010, 0x91},
    {0xC0080014, 0x0},
    {0xC0080018, 0x3},
    {0xC008001C, 0x13}
};


#if defined(IX_PLATFORM_2401) || defined(IX_PLATFORM_2801)

/* Platform specific
 * IXDP2401 platform has a different framer driver
 */
#if (_IX_BOARD_TYPE_ != _IX_IXDP2401_) && (_IX_BOARD_TYPE_ != _IX_IXDP2801_)
/*
 * IXF6048 framer configuration data table. This is user
 * interface generated configuration table that contains the
 * offset/value pairs to be written down to the chip.
 */
extern InitRegTable_t *IxCcPosTxRegInitIXF6048Table[];
#endif
#else
/*
 * IXF6048 framer configuration data table. This is user
 * interface generated configuration table that contains the
 * offset/value pairs to be written down to the chip.
 */
extern InitRegTable_t *IxCcPosTxRegInitIXF6048Table[];
#endif /* IX_PLATFORM_2401 || IX_PLATFORM_2801 */


/**
 * Exported variable declarations global to this component only.
 */
ix_cc_atm_pos_tx_ctl_blk *g_pCcAtmPosTxCtlBlk = NULL; /* control block */



/**
 * Pre-processor symbols and macros used in this file.
 */



/**
 * Extern function prototypes
 */


/**
 * Static function prototypes
 */
PRIVATE ix_error _ix_cc_atm_pos_tx_get_static_data(ix_cc_atm_pos_tx_ctl_blk *);
PRIVATE ix_error _ix_cc_atm_pos_tx_validate_and_set_operation_modes(ix_cc_atm_pos_tx_ctl_blk *);
PRIVATE void _ix_cc_atm_pos_tx_free_ctl_blk(ix_cc_atm_pos_tx_ctl_blk *);
PRIVATE ix_error _ix_cc_atm_pos_tx_create_cb_ctx_tables(ix_cc_atm_pos_tx_ctl_blk *);
PRIVATE ix_error _ix_cc_atm_pos_tx_init_framer(ix_cc_atm_pos_tx_ctl_blk *);
PRIVATE ix_error _ix_cc_atm_pos_tx_fini_framer(ix_cc_atm_pos_tx_ctl_blk *);
#if defined(IX_PLATFORM_2401) || defined(IX_PLATFORM_2801)

/* Platform specific
 * IXDP2401 platform has a different framer driver
 */
#if (_IX_BOARD_TYPE_ != _IX_IXDP2401_) && (_IX_BOARD_TYPE_ != _IX_IXDP2801_)
PRIVATE void _ix_cc_atm_pos_tx_framer_alarm_callback(bb_ChipData_t *, bb_AlarmType_e, void *);
#endif
#else /* IX_PLATFORM_2401 || IX_PLATFORM_2801 */
PRIVATE void _ix_cc_atm_pos_tx_framer_alarm_callback(bb_ChipData_t *, bb_AlarmType_e, void *);
#endif /* IX_PLATFORM_2401 || IX_PLATFORM_2801 */
PRIVATE void _ix_cc_atm_pos_tx_init_slow_port(void);



/**
 * function definitions.
 */


/**
 * NAME: ix_cc_atm_pos_tx_init
 *
 * DESCRIPTION: This function initializes the ATM/POS tx core component.
 *
 * @Param: arg_CcHandle - IN: core component handle created by CCI.
 * @Param: arg_ppContext - IN: pointer to cc init context memory.
 *                         OUT: pointer to component context memory.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */
ix_error ix_cc_atm_pos_tx_init(
    ix_cc_handle arg_CcHandle,
    void **arg_ppContext)
{

    ix_cc_atm_pos_tx_ctl_blk *pCtlBlk = NULL;
    ix_error err = IX_SUCCESS;


#if defined(IX_PLATFORM_2401) || defined(IX_PLATFORM_2801)

    /* Enhancement: Patch counters depending on MICROBLOCK_COUNTERS flag
     */
#ifdef MICROBLOCK_COUNTERS
    /* variables used in patching symbols */
    ix_imported_symbol symbol[POS_TX_NUM_SYMBOLS_PATCHED];
    ix_memory_type memType;
    ix_uint32 memCh;
    ix_uint32 memOffset32BitCounter;
    ix_uint32 memOffsetNextAvailTBufElem;
    void *pPhyAddr;
    ix_uint16 numOfSymbols = 0;
#endif /* MICROBLOCK_COUNTERS */
#else /* IX_PLATFORM_2401 || IX_PLATFORM_2801 */
    /* variables used in patching symbols */
    ix_imported_symbol symbol[POS_TX_NUM_SYMBOLS_PATCHED];
    ix_memory_type memType;
    ix_uint32 memCh;
    ix_uint32 memOffset32BitCounter;
    ix_uint32 memOffsetNextAvailTBufElem;
    void *pPhyAddr;
#endif /* IX_PLATFORM_2401 || IX_PLATFORM_2801 */

    ix_uint32 i;

    /* Variables used in retrieving static data from ix_cc_init_context */
    ix_cc_init_context *pInitCtx;
    ix_cc_properties_node *pPropNode;



    /* Check whether arg_ppContext is invalid */
    if (arg_ppContext == (void **)NULL)
    {
        return(IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                              ("NULL argument")));
    }

    /* Get the generic init context */
    pInitCtx = (ix_cc_init_context *)*arg_ppContext;

    /* Allocate the control block structure */
    if ((pCtlBlk = (ix_cc_atm_pos_tx_ctl_blk *)ix_ossl_malloc(sizeof(ix_cc_atm_pos_tx_ctl_blk))) == NULL)
    {
        return(IX_ERROR_REMOTE(IX_CC_ERROR_OOM_SYSTEM,
                               ("Out of memory")));
    }

    /* Initialize the structure */
    ix_ossl_memset((void *)pCtlBlk,
                   0,
                   (ix_ossl_size_t)sizeof(ix_cc_atm_pos_tx_ctl_blk));

#if defined (IX_PLATFORM_2801)
    /* Platform specific. Storing init context allows searching interface table
     * on purpose of framer initialization.
     */
#if (_IX_BOARD_TYPE_ == _IX_IXDP2801_)

    pCtlBlk->pInitCtx = pInitCtx;

#endif
#endif

    /* Retrieve static configuration data */
    IX_ERROR_C(_ix_cc_atm_pos_tx_get_static_data(pCtlBlk), err);
    if (err != IX_SUCCESS)
    {
        /* free all memories allocated earlier */
        _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

        return(err);
    }

    /**
     * Retrieve the operation mode information
     */
    pCtlBlk->modes = ATM_POS_OPERATION_MODES;

    /**
     * Validate operation modes.
     */
    IX_ERROR_C(_ix_cc_atm_pos_tx_validate_and_set_operation_modes(pCtlBlk), err);
    if (err != IX_SUCCESS)
    {
        /* free all memories allocated earlier */
        _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

        return(err);
    }

    /* Additional check for PR5: only POS mode is supported. */
    /* (This should be removed in PR6) */
    if (pCtlBlk->atmEnable == TRUE)
    {
        /* free all memories allocated earlier */
        _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

        /* Return error for now.  Will be implemented in PR6 */
        return(IX_ERROR_LOCAL(IX_CC_ERROR_UNIMPL,
                              ("ATM feature not implemented")));
    }

    /**
     * Retrieve the port id information of each channel from the
     * init context (This will work for both level-1 and level-0 modes).
     */
    pPropNode = pInitCtx->pPropertyList;
    for (i = 0;
         i < pCtlBlk->numChannels;
         i++, pPropNode = pPropNode->pNextPropNode)
    {
        if (pPropNode == NULL)
        {
            /* free all memories allocated earlier */
            _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

            return(IX_ERROR_REMOTE(IX_CC_ERROR_ENTRY_NOT_FOUND,
                                   ("Number of property nodes does not match with the number of channels configured.")));
        }

        pCtlBlk->portId[i] = pPropNode->pPropertyInfo->port_id;

        /**
         * Store the local blade id. Assuming all nodes have same
         * blade id so it's ok to keep overwriting.
         */
        pCtlBlk->bladeId = pPropNode->pPropertyInfo->blade_id;
    }

    /* Initializa the ATM/POS framer chip based on the modes selected */
    IX_ERROR_C(_ix_cc_atm_pos_tx_init_framer(pCtlBlk), err);
    if (err != IX_SUCCESS)
    {
        /* free all memories allocated earlier */
        _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

        return(err);
    }

    /* Perform POS mode configurations */
    if (pCtlBlk->posEnable)
    {
        /* Allocate next avail TBUF element (see also HLD) */
        IX_ERROR_CT(ix_rm_mem_alloc(IX_MEMORY_TYPE_SCRATCH,
                                    CC_POS_TX_DEFAULT_SCR_CH_NUM,
                                    POS_TX_SIZE_NEXT_AVAIL_TBUF_ELEM,
                                    (void **)&(pCtlBlk->pPosTxNextAvailTbufElem)),
                    err,
                    IX_CC_ERROR_OOM_SRAM,
                    IX_ERROR_LEVEL_REMOTE);
        if (err != IX_SUCCESS)
        {
            /* free all memories allocated earlier */
            _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

            return(err);
        }

#if defined(IX_PLATFORM_2401) || defined(IX_PLATFORM_2801)

        /* Enhancement: Patch counters depending on MICROBLOCK_COUNTERS flag
         */
#ifdef MICROBLOCK_COUNTERS
        /* Allocate 32-bit counter momory */
        IX_ERROR_CT(ix_rm_mem_alloc(IX_MEMORY_TYPE_SRAM,
                                    CC_POS_TX_DEFAULT_SRAM_CH_NUM,
                                    POS_TX_SIZE_32BIT_COUNTER_BUF,
                                    (void **)&(pCtlBlk->pPosCounterBase32Bit)),
                    err,
                    IX_CC_ERROR_OOM_SRAM,
                    IX_ERROR_LEVEL_REMOTE);
        if (err != IX_SUCCESS)
        {
            /* free all memories allocated earlier */
            _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

            return(err);
        }

        /* Initialize the memory */
        ix_ossl_memset((void *)pCtlBlk->pPosCounterBase32Bit,
                       0,
                       POS_TX_SIZE_32BIT_COUNTER_BUF);

        /* Allocate memory for 64-bit counter handler array */
        if ((pCtlBlk->pPosCounterHandlers64Bit = (ix_counter_64bit_handle *)ix_ossl_malloc(POS_TX_SIZE_64BIT_COUNTER_HDL)) == NULL)
        {
            /* Free all memories allocated earlier */
            _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

            return(IX_ERROR_REMOTE(IX_CC_ERROR_OOM_SYSTEM,
                                   ("Out of memory")));
        }

        /* Allocate 64-bit counters from Resource Manager */
        IX_ERROR_CT(ix_rm_counter_64bit_new(POS_TX_TOTAL_NUM_COUNTERS,
                                            pCtlBlk->pPosCounterBase32Bit,
                                            CC_POS_TX_32BIT_COUNTER_OVERFLOW_TIME,
                                            pCtlBlk->pPosCounterHandlers64Bit),
                    err,
                    IX_CC_ERROR_OOM_64BIT_COUNTER,
                    IX_ERROR_LEVEL_REMOTE);
        if (err != IX_SUCCESS)
        {
            /* free all memories allocated earlier */
            _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

            return(err);
        }
        /**
         * patch various symbols and memory bases into POS Tx microblock:
         * 1. POS Tx counter SRAM base.
         */

        IX_ERROR_CT(ix_rm_get_phys_offset((void *)(pCtlBlk->pPosCounterBase32Bit),
                                          &memType,
                                          &memCh,
                                          &memOffset32BitCounter,
                                          &pPhyAddr),
                    err,
                    IX_CC_ATM_POS_TX_ERROR_RM_GET_PHYS_OFFSET,
                    IX_ERROR_LEVEL_REMOTE);
        if (err != IX_SUCCESS)
        {
            /* free all memories allocated earlier */
            _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

            return(err);
        }

        /* Pack the SRAM channel number into the offset */
        IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(memCh, memOffset32BitCounter);

        /**
         * Prepare the symbol array.  For now only single mode is used
         * so skip the memOffsetNextAvailTBufElem.
         */
        symbol[numOfSymbols].m_Value = memOffset32BitCounter;
        symbol[numOfSymbols++].m_Name = SYMBOL_POS_TX_COUNTERS_BASE;

        /* Patch them into each microengine used by the microblock */
        for (i = 0; i < IX_CC_ATM_POS_TX_MAX_ME_NUM; i++)
        {
            if (pCtlBlk->posTxMev2Num & (0x1 << i))
            {
                IX_ERROR_CT(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[i],
                                                     numOfSymbols,
                                                     symbol),
                            err,
                            IX_CC_ATM_POS_TX_ERROR_RM_PATCH_SYMBOL,
                            IX_ERROR_LEVEL_REMOTE);
                if (err != IX_SUCCESS)
                {
                    /* free all memories allocated earlier */
                    _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

                    return(err);
                }
            }
        }
#endif /* MICROBLOCK_COUNTERS */
#else  /* IX_PLATFORM_2x01 */
        /* Allocate 32-bit counter momory */
        IX_ERROR_CT(ix_rm_mem_alloc(IX_MEMORY_TYPE_SRAM,
                                    CC_POS_TX_DEFAULT_SRAM_CH_NUM,
                                    POS_TX_SIZE_32BIT_COUNTER_BUF,
                                    (void **)&(pCtlBlk->pPosCounterBase32Bit)),
                    err,
                    IX_CC_ERROR_OOM_SRAM,
                    IX_ERROR_LEVEL_REMOTE);
        if (err != IX_SUCCESS)
        {
            /* free all memories allocated earlier */
            _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

            return(err);
        }

        /* Initialize the memory */
        ix_ossl_memset((void *)pCtlBlk->pPosCounterBase32Bit,
                       0,
                       POS_TX_SIZE_32BIT_COUNTER_BUF);

        /* Allocate memory for 64-bit counter handler array */
        if ((pCtlBlk->pPosCounterHandlers64Bit = (ix_counter_64bit_handle *)ix_ossl_malloc(POS_TX_SIZE_64BIT_COUNTER_HDL)) == NULL)
        {
            /* Free all memories allocated earlier */
            _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

            return(IX_ERROR_REMOTE(IX_CC_ERROR_OOM_SYSTEM,
                                   ("Out of memory")));
        }

        /* Allocate 64-bit counters from Resource Manager */
        IX_ERROR_CT(ix_rm_counter_64bit_new(POS_TX_TOTAL_NUM_COUNTERS,
                                            pCtlBlk->pPosCounterBase32Bit,
                                            CC_POS_TX_32BIT_COUNTER_OVERFLOW_TIME,
                                            pCtlBlk->pPosCounterHandlers64Bit),
                    err,
                    IX_CC_ERROR_OOM_64BIT_COUNTER,
                    IX_ERROR_LEVEL_REMOTE);
        if (err != IX_SUCCESS)
        {
            /* free all memories allocated earlier */
            _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

            return(err);
        }
        /**
         * patch various symbols and memory bases into POS Tx microblock:
         * 1. Packet Tx next available tbuf element. (multi mode only)
         * 2. POS Tx counter SRAM base.
         */

        /* first obtain the base offsets of these memories */
        IX_ERROR_CT(ix_rm_get_phys_offset((void *)(pCtlBlk->pPosTxNextAvailTbufElem),
                                          &memType,
                                          &memCh,
                                          &memOffsetNextAvailTBufElem,
                                          &pPhyAddr),
                    err,
                    IX_CC_ATM_POS_TX_ERROR_RM_GET_PHYS_OFFSET,
                    IX_ERROR_LEVEL_REMOTE);
        if (err != IX_SUCCESS)
        {
            /* free all memories allocated earlier */
            _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

            return(err);
        }
        IX_ERROR_CT(ix_rm_get_phys_offset((void *)(pCtlBlk->pPosCounterBase32Bit),
                                          &memType,
                                          &memCh,
                                          &memOffset32BitCounter,
                                          &pPhyAddr),
                    err,
                    IX_CC_ATM_POS_TX_ERROR_RM_GET_PHYS_OFFSET,
                    IX_ERROR_LEVEL_REMOTE);
        if (err != IX_SUCCESS)
        {
            /* free all memories allocated earlier */
            _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

            return(err);
        }

        /* Pack the SRAM channel number into the offset */
        IX_CC_SET_SRAM_CHANNEL_INTO_OFFSET(memCh, memOffset32BitCounter);

        /**
         * Prepare the symbol array.  For now only single mode is used
         * so skip the memOffsetNextAvailTBufElem.
         */
        symbol[0].m_Value = memOffset32BitCounter;
        symbol[0].m_Name = SYMBOL_POS_TX_COUNTERS_BASE;

        /* Patch them into each microengine used by the microblock */
        for (i = 0; i < IX_CC_ATM_POS_TX_MAX_ME_NUM; i++)
        {
            if (pCtlBlk->posTxMev2Num & (0x1 << i))
            {
                IX_ERROR_CT(ix_rm_ueng_patch_symbols(g_aMicroengineNumberMapping[i],
                                                     POS_TX_NUM_SYMBOLS_PATCHED,
                                                     symbol),
                            err,
                            IX_CC_ATM_POS_TX_ERROR_RM_PATCH_SYMBOL,
                            IX_ERROR_LEVEL_REMOTE);
                if (err != IX_SUCCESS)
                {
                    /* free all memories allocated earlier */
                    _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

                    return(err);
                }
            }
        }
#endif /* IX_PLATFORM_2401 || IX_PLATFORM_2801 */
    } /* POS mode configuration */

    /*
     * Create mutex for critical section.
     */
    IX_ERROR_CT(ix_ossl_mutex_init(OS_MUTEX_UNLOCK,
                                   &(pCtlBlk->mid)),
                err,
                IX_CC_ATM_POS_TX_ERROR_OSSL_MUTEX_INIT,
                IX_ERROR_LEVEL_REMOTE);
    if (err != IX_SUCCESS)
    {
        /* free all memories allocated earlier */
        _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

        return(err);
    }

    /**
     * Allocate free callback context entries for each async messaging
     * helper API function.
     */
    IX_ERROR_C(_ix_cc_atm_pos_tx_create_cb_ctx_tables(pCtlBlk), err);
    if (err != IX_SUCCESS)
    {
        /* free all memories allocated earlier */
        _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

        return(err);
    }

#ifndef IX_EXCLUDE_CCI

    /* Register message handler */
    IX_ERROR_CT(ix_cci_cc_add_message_handler(arg_CcHandle,
                                              IX_CC_ATM_POS_TX_MSG_INPUT,
                                              ix_cc_atm_pos_tx_msg_handler,
                                              IX_INPUT_TYPE_SINGLE_SRC),
                err,
                IX_CC_ATM_POS_TX_ERROR_CCI_REG_HDLR,
                IX_ERROR_LEVEL_REMOTE);
    if (err != IX_SUCCESS)
    {
        /* free all memories allocated earlier */
        _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

        return(err);
    }

    /* Register the property message handler */
    IX_ERROR_CT(ix_cci_cc_add_message_handler(arg_CcHandle,
                                              IX_CC_ATM_POS_TX_PROPERTY_MSG_INPUT,
                                              ix_cc_atm_pos_tx_property_msg_handler,
                                              IX_INPUT_TYPE_SINGLE_SRC),
                err,
                IX_CC_ATM_POS_TX_ERROR_CCI_REG_HDLR,
                IX_ERROR_LEVEL_REMOTE);
    if (err != IX_SUCCESS)
    {
        /* free all memories allocated earlier */
        _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

        return(err);
    }

#else /* IX_EXCLUDE_CCI */

    /* Register message handler */
    IX_ERROR_CT(ix_rm_message_handler_register(IX_CC_ATM_POS_TX_MSG_INPUT,
                                               ix_cc_atm_pos_tx_msg_handler,
                                               pCtlBlk),
                err,
                IX_CC_ATM_POS_TX_ERROR_RM_REG_HDLR,
                IX_ERROR_LEVEL_REMOTE);
    if (err != IX_SUCCESS)
    {
        /* free all memories allocated earlier */
        _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

        return(err);
    }

    /* Register the property message handler */
    IX_ERROR_CT(ix_rm_message_handler_register(IX_CC_ATM_POS_TX_PROPERTY_MSG_INPUT,
                                               ix_cc_atm_pos_tx_property_msg_handler,
                                               pCtlBlk),
                err,
                IX_CC_ATM_POS_TX_ERROR_RM_REG_HDLR,
                IX_ERROR_LEVEL_REMOTE);
    if (err != IX_SUCCESS)
    {
        /* free all memories allocated earlier */
        _ix_cc_atm_pos_tx_free_ctl_blk(pCtlBlk);

        return(err);
    }

#endif /* IX_EXCLUDE_CCI */

    /* Assign output argument */
    *arg_ppContext = (void *)pCtlBlk;

    /**
     * Store the address of the control block structure to a global
     * varible that can be used later in helper API functions.
     */
    g_pCcAtmPosTxCtlBlk = pCtlBlk;

    return(IX_SUCCESS);

} /* ix_cc_atm_pos_tx_init */



/**
 * NAME: ix_cc_atm_pos_tx_fini
 *
 * DESCRIPTION: This function terminates the ATM/POS tx core component.
 *
 * @Param: arg_CcHandle - IN: CC handle created by CCI.
 * @Param: arg_pContext - IN: pointer to the component context allocated
 *                            in the component's init function.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */
ix_error ix_cc_atm_pos_tx_fini(
    ix_cc_handle arg_CcHandle,
    void *arg_pContext)
{

    ix_error err = IX_SUCCESS;



    /* Check whether arg_pContext is valid */
    if (arg_pContext == (void *)NULL)
    {
        return(IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                              ("NULL argument")));
    }

    /* Start the critical section */
    ix_ossl_mutex_lock(((ix_cc_atm_pos_tx_ctl_blk *)arg_pContext)->mid,
                       IX_OSSL_WAIT_FOREVER);

#ifndef IX_EXCLUDE_CCI

    /* Remove message handler from CCI */
    IX_ERROR_CT(ix_cci_cc_remove_message_handler(IX_CC_ATM_POS_TX_MSG_INPUT),
                err,
                IX_CC_ATM_POS_TX_ERROR_CCI_REM_HDLR,
                IX_ERROR_LEVEL_REMOTE);
    if (err != IX_SUCCESS)
    {
        /* Exit the critical section */
        ix_ossl_mutex_unlock(((ix_cc_atm_pos_tx_ctl_blk *)arg_pContext)->mid);

	return(err);
    }

    /* Remove property message handler from CCI */
    IX_ERROR_CT(ix_cci_cc_remove_message_handler(IX_CC_ATM_POS_TX_PROPERTY_MSG_INPUT),
                err,
                IX_CC_ATM_POS_TX_ERROR_CCI_REM_HDLR,
                IX_ERROR_LEVEL_REMOTE);
    if (err != IX_SUCCESS)
    {
        /* Exit the critical section */
        ix_ossl_mutex_unlock(((ix_cc_atm_pos_tx_ctl_blk *)arg_pContext)->mid);

	return(err);
    }

#else /* IX_EXCLUDE_CCI */

    /* Remove message handler from RM */
    IX_ERROR_CT(ix_rm_message_handler_unregister(IX_CC_ATM_POS_TX_MSG_INPUT),
                err,
                IX_CC_ATM_POS_TX_ERROR_RM_REM_HDLR,
                IX_ERROR_LEVEL_REMOTE);
    if (err != IX_SUCCESS)
    {
        /* Exit the critical section */
        ix_ossl_mutex_unlock(((ix_cc_atm_pos_tx_ctl_blk *)arg_pContext)->mid);

	return(err);
    }

    /* Remove property message handler from RM */
    IX_ERROR_CT(ix_rm_message_handler_unregister(IX_CC_ATM_POS_TX_PROPERTY_MSG_INPUT),
                err,
                IX_CC_ATM_POS_TX_ERROR_RM_REM_HDLR,
                IX_ERROR_LEVEL_REMOTE);
    if (err != IX_SUCCESS)
    {
        /* Exit the critical section */
        ix_ossl_mutex_unlock(((ix_cc_atm_pos_tx_ctl_blk *)arg_pContext)->mid);

	return(err);
    }

#endif /* IX_EXCLUDE_CCI */

    /* Reset the ATM/POS framer device */
    IX_ERROR_C(_ix_cc_atm_pos_tx_fini_framer(arg_pContext), err);
    if (err != IX_SUCCESS)
    {
        /* Exit the critical section */
        ix_ossl_mutex_unlock(((ix_cc_atm_pos_tx_ctl_blk *)arg_pContext)->mid);

	return(err);
    }

    /* Exit the critical section */
    ix_ossl_mutex_unlock(((ix_cc_atm_pos_tx_ctl_blk *)arg_pContext)->mid);

    /* Free all memories and resources indicated in the arg_pContext */
    _ix_cc_atm_pos_tx_free_ctl_blk((ix_cc_atm_pos_tx_ctl_blk *)arg_pContext);

    return(err);

} /* ix_cc_atm_pos_tx_fini */





/**********************
 * Internal functions *
 **********************/


/**
 * NAME: _ix_cc_atm_pos_tx_get_static_data
 *
 * DESCRIPTION: This function retrieves all ATM/POS Tx static data from
 *              either system repository or ix_cc_atm_pos_tx.h and store them
 *              into the corresponding field in the control block
 *              structure passed in.
 *
 * @Param: arg_pCtlBlk - IN: pointer to the component context allocated
 *                           in the component's init function.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */
PRIVATE ix_error _ix_cc_atm_pos_tx_get_static_data(
    ix_cc_atm_pos_tx_ctl_blk *arg_pCtlBlk)
{

    /**
     * Retrieve static configuration data
     */
#ifdef IX_INCLUDE_REGISTRY

    ix_configuration_property_handle hCoreRoot = IX_CP_CORE_PROPERTY_ROOT;
    ix_configuration_property_handle hProperty = (ix_configuration_property_handle)NULL;

    /* open the property */
    IX_ERROR_CRT(ix_rm_cp_property_open(hCoreRoot,
                                        PROP_POS_TX_MEV2_NUM_MASK,
                                        &hProperty),
                 IX_CC_ERROR_ENTRY_NOT_FOUND,
                 IX_ERROR_LEVEL_REMOTE);

    /* read the property */
    IX_ERROR_CRT(ix_rm_cp_property_get_value_uint32(hProperty,
                                                    &arg_pCtlBlk->posTxMev2Num),
                 IX_CC_ERROR_ENTRY_NOT_FOUND,
                 IX_ERROR_LEVEL_REMOTE);

#else /* IX_INCLUDE_REGISTRY */

    arg_pCtlBlk->posTxMev2Num = POS_TX_MEV2_NUMBER;

#endif /* IX_INCLUDE_REGISTRY */

    return(IX_SUCCESS);

} /* _ix_cc_atm_pos_tx_get_static_data */




/**
 * NAME: _ix_cc_atm_pos_tx_validate_and_set_operation_modes
 *
 * DESCRIPTION: This function validates user operation mode selections
 *              based on the supported modes lisetd in the component
 *              HLD.
 *
 * @Param: arg_pCtlBlk - IN: pointer to the component context allocated
 *                           in the component's init function.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */
PRIVATE ix_error _ix_cc_atm_pos_tx_validate_and_set_operation_modes(
    ix_cc_atm_pos_tx_ctl_blk *arg_pCtlBlk)
{

    BOOL isValidMode;
    ix_uint32 i;

    /**
     * Retrieve OC mode, line mode, channel mode, and PHY mode
     */
    ix_uint32 ocMode = arg_pCtlBlk->modes & FRAMER_OC_MODE_MASK;
    ix_uint32 lineMode = arg_pCtlBlk->modes & FRAMER_LINE_MODE_MASK;
    ix_uint32 chMode = arg_pCtlBlk->modes & FRAMER_CHANNEL_MODE_MASK;
    ix_uint32 utopiaMode = arg_pCtlBlk->modes & FRAMER_UPHY_MODE_MASK;

    /* Check if any of the modes is invalid */
    if (((ocMode != FRAMER_OC48C_MODE) &&
         (ocMode != FRAMER_OC48_MODE) &&
         (ocMode != FRAMER_OC12C_MODE) &&
#if defined(IX_PLATFORM_2401) || defined(IX_PLATFORM_2801)

         /* Platform specific
          * Additional OC mode
          */
         (ocMode != FRAMER_OC3_MODE) &&
#endif /* IX_PLATFORM_2401 || IX_PLATFORM_2801 */
         (ocMode != FRAMER_OC12_MODE)) ||
        ((lineMode != FRAMER_1_16_PECL_LINE_MODE) &&
         (lineMode != FRAMER_4_8_TTL_LINE_MODE) &&
         (lineMode != FRAMER_1_8_TTL_LINE_MODE)) ||
        ((chMode != FRAMER_POS_CHANNEL_MODE) &&
         (chMode != FRAMER_ATM_CHANNEL_MODE)) ||
        ((utopiaMode != FRAMER_1_32_L2_UPHY_MODE) &&
         (utopiaMode != FRAMER_4_8_L1_UPHY_MODE) &&
         (utopiaMode != FRAMER_1_32_L3_UPHY_MODE) &&
         (utopiaMode != FRAMER_4_8_L3_UPHY_MODE)))
    {
        return(IX_ERROR_LOCAL(IX_CC_ERROR_UNIMPL,
                              ("Invalid ATM/POS framer mode selected: 0x%ld", arg_pCtlBlk->modes)));
    }

    /* Based on the channel mode, set the ATM and POS mode flags accordingly */
    if (chMode == FRAMER_ATM_CHANNEL_MODE)
    {
        /* ATM mode selected */
        arg_pCtlBlk->atmEnable = TRUE;
    }
    else
    {
        /* POS mode selected */
        arg_pCtlBlk->posEnable = TRUE;
    }

    /**
     * Verify if the combination of these modes is valid. (Only POS mode
     * is checked here.  Checking for ATM mode will be added in PR6.)
     */
    isValidMode = FALSE;
    for (i = 0; i < IX_CC_NUM_POS_MODES; i++)
    {
        if ((ocMode == g_aApplicablePosMode[i].ocMode) &&
            (lineMode == g_aApplicablePosMode[i].lineMode) &&
            (utopiaMode == g_aApplicablePosMode[i].utopiaMode))
        {
            isValidMode = TRUE;
            arg_pCtlBlk->posModeNum = i; /* record the internal POS mode number */
        }
    }

    if (isValidMode == FALSE)
    {
        return(IX_ERROR_LOCAL(IX_CC_ERROR_UNIMPL,
                              ("Invalid ATM/POS framer mode selected: 0x%ld", arg_pCtlBlk->modes)));
    }

    /* Set the number of channels based on the Utopia mode selection */
    switch(utopiaMode)
    {
        case FRAMER_1_32_L2_UPHY_MODE:
        case FRAMER_1_32_L3_UPHY_MODE:
            arg_pCtlBlk->numChannels = 1;
            break;
        case FRAMER_4_8_L1_UPHY_MODE:
        case FRAMER_4_8_L3_UPHY_MODE:

#if defined(IX_PLATFORM_2401) || defined(IX_PLATFORM_2801)

/* Platform specific
 * IXDP2x01 platform has a different framer driver
 */
#if (_IX_BOARD_TYPE_ == _IX_IXDP2401_)

            arg_pCtlBlk->numChannels = 2;

#elif (_IX_BOARD_TYPE_ == _IX_IXDP2801_)

            /* Number of channels depends on the configuration */
            {
                ix_cc_properties_node *pProp;
                /*
                 * Go through all interface properties and recognize POS ports,
                 * then count them
                 */
                arg_pCtlBlk->numChannels = 0;
                for (pProp = arg_pCtlBlk->pInitCtx->pPropertyList;
                     pProp != NULL; pProp = pProp->pNextPropNode)
                {
                    if (pProp->pPropertyInfo->media_type == IX_MEDIA_TYPE_POS)
                    {
                        arg_pCtlBlk->numChannels++;
                    }
                }
            }
#else
            arg_pCtlBlk->numChannels = 4;
#endif /* _IX_BOARD_TYPE_ */
#else /* IX_PLATFORM_2401 || IX_PLATFORM_2801*/
            arg_pCtlBlk->numChannels = 4;
#endif /* IX_PLATFORM_2401 || IX_PLATFORM_2801 */
            break;
        default:
            return(IX_ERROR_LOCAL(IX_CC_ERROR_UNIMPL,
                                  ("Invalid ATM/POS Utopia PHY mode selected")));
    }

    return(IX_SUCCESS);

} /* _ix_cc_atm_pos_tx_validate_and_set_operation_modes */





/**
 * NAME: _ix_cc_atm_pos_tx_create_cb_ctx_tables
 *
 * DESCRIPTION: This function creates callback context tables that
 *              contain free callback context entry structure for
 *              being used by each invocation of helper API
 *              that has callback defined.
 *
 * @Param: arg_pCtlBlk - IN: pointer to the component context allocated
 *                           in the component's init function.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */
PRIVATE ix_error _ix_cc_atm_pos_tx_create_cb_ctx_tables(
    ix_cc_atm_pos_tx_ctl_blk *arg_pCtlBlk)
{

    /**
     * Create table for ix_cc_atm_pos_tx_async_get_statistics_info().
     */

    /* Allocate cb_ctx_table structure first */
    if ((arg_pCtlBlk->pCbCtxTblGetStatInfo = (cc_atm_pos_tx_cb_ctx_tbl *)ix_ossl_malloc(sizeof(cc_atm_pos_tx_cb_ctx_tbl))) == NULL)
    {
        return(IX_ERROR_REMOTE(IX_CC_ERROR_OOM_SYSTEM,
                               ("Out of memory")));
    }

    /* Initialize the structure */
    ix_ossl_memset((void *)arg_pCtlBlk->pCbCtxTblGetStatInfo,
                   0,
                   (ix_ossl_size_t)sizeof(cc_atm_pos_tx_cb_ctx_tbl));

    /* Allocate the callback context entries and put them in the table structure */
    if ((arg_pCtlBlk->pCbCtxTblGetStatInfo->pTable = (ix_cc_atm_pos_tx_cb_ctx_entry *)ix_ossl_malloc(ATM_POS_TX_SIZE_CB_CTX_TBL_GET_STAT)) == NULL)
    {
        ix_ossl_free(arg_pCtlBlk->pCbCtxTblGetStatInfo);
        return(IX_ERROR_REMOTE(IX_CC_ERROR_OOM_SYSTEM,
                               ("Out of memory")));
    }

    /**
     * Create table for ix_cc_atm_pos_tx_async_get_interface_state().
     */

    /* Allocate cb_ctx_table structure first */
    if ((arg_pCtlBlk->pCbCtxTblGetIfState = (cc_atm_pos_tx_cb_ctx_tbl *)ix_ossl_malloc(sizeof(cc_atm_pos_tx_cb_ctx_tbl))) == NULL)
    {
        return(IX_ERROR_REMOTE(IX_CC_ERROR_OOM_SYSTEM,
                               ("Out of memory")));
    }

    /* Initialize the structure */
    ix_ossl_memset((void *)arg_pCtlBlk->pCbCtxTblGetIfState,
                   0,
                   (ix_ossl_size_t)sizeof(cc_atm_pos_tx_cb_ctx_tbl));

    /* Allocate the callback context entries and put them in the table structure */
    if ((arg_pCtlBlk->pCbCtxTblGetIfState->pTable = (ix_cc_atm_pos_tx_cb_ctx_entry *)ix_ossl_malloc(ATM_POS_TX_SIZE_CB_CTX_TBL_GET_IF)) == NULL)
    {
        ix_ossl_free(arg_pCtlBlk->pCbCtxTblGetStatInfo);
        return(IX_ERROR_REMOTE(IX_CC_ERROR_OOM_SYSTEM,
                               ("Out of memory")));
    }

    return(IX_SUCCESS);

} /* _ix_cc_atm_pos_tx_create_cb_ctx_tables */







/**
 * NAME: _ix_cc_atm_pos_tx_free_ctl_blk
 *
 * DESCRIPTION: This function frees all memories and resources specified
 *              by each pointer member of the control block structure.
 *              Finally, it frees the control block structure.
 *
 * @Param: arg_pCtlBlk - IN: pointer to the component context allocated
 *                           in the component's init function.

 *
 * @Return: None
 */
PRIVATE void _ix_cc_atm_pos_tx_free_ctl_blk(
    ix_cc_atm_pos_tx_ctl_blk *arg_pCtlBlk)
{

    ix_uint32 i;



    /* Remove POS mode memories and resources */
    if (arg_pCtlBlk->posEnable)
    {
        /* Free 64-bit counter handle array */
        if (arg_pCtlBlk->pPosCounterHandlers64Bit != NULL)
        {
            /* First delete each 64-bit counter from Resource Manager */
            for (i = 0; i < POS_TX_TOTAL_NUM_COUNTERS; i++)
            {
                ix_rm_counter_64bit_delete(*(arg_pCtlBlk->pPosCounterHandlers64Bit + i));
            }

            /* Then free the counter handler array */
            ix_ossl_free(arg_pCtlBlk->pPosCounterHandlers64Bit);
        }

        /* Free 32-bit counter memory */
        if (arg_pCtlBlk->pPosCounterBase32Bit != NULL)
        {
            if (ix_rm_mem_free(arg_pCtlBlk->pPosCounterBase32Bit) != IX_SUCCESS)
            {
                ix_ossl_message_log("Resource manager memory free failed.\n");
            }
        }

        /* Free the next avail TBUF element memory */
        if (ix_rm_mem_free((void *)(arg_pCtlBlk->pPosTxNextAvailTbufElem)) != IX_SUCCESS)
        {
            ix_ossl_message_log("Resource manager memory free failed.\n");
        }
    } /* posEnable */

    /* Free async callback context table and entries */
    if (arg_pCtlBlk->pCbCtxTblGetStatInfo != NULL)
    {
        /* Free the entries first */
        if (arg_pCtlBlk->pCbCtxTblGetStatInfo->pTable != NULL)
        {
            ix_ossl_free(arg_pCtlBlk->pCbCtxTblGetStatInfo->pTable);
        }

        /* Free the table structure */
        ix_ossl_free(arg_pCtlBlk->pCbCtxTblGetStatInfo);
    }

    /* Free async callback context table and entries */
    if (arg_pCtlBlk->pCbCtxTblGetIfState != NULL)
    {
        /* Free the entries first */
        if (arg_pCtlBlk->pCbCtxTblGetIfState->pTable != NULL)
        {
            ix_ossl_free(arg_pCtlBlk->pCbCtxTblGetIfState->pTable);
        }

        /* Free the table structure */
        ix_ossl_free(arg_pCtlBlk->pCbCtxTblGetIfState);
    }

    /* Destroy the critical section mutex */
    if (arg_pCtlBlk->mid != NULL)
    {
        if (ix_ossl_mutex_fini(arg_pCtlBlk->mid) != IX_SUCCESS)
        {
            ix_ossl_message_log("OSSL mutex fini failed.\n");
        }
    }

    /* free the control block structure itself */
    ix_ossl_free((void *)arg_pCtlBlk);

} /* _ix_cc_atm_pos_tx_free_ctl_blk */


/**
 * NAME: _ix_cc_atm_pos_tx_init_framer
 *
 * DESCRIPTION: This function initializes and configures the
 *              ATM/POS framer media card.
 *
 * @Param: arg_pCtlBlk - IN: pointer to the component context allocated
 *                           in the component's init function.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */
PRIVATE ix_error _ix_cc_atm_pos_tx_init_framer(
    ix_cc_atm_pos_tx_ctl_blk *arg_pCtlBlk)
{

    ix_uint32 i;
#if defined(IX_PLATFORM_2401) || defined(IX_PLATFORM_2801)

    /* Platform spefific
     * IXDP2x01 uses framer driver with different API
     */
#if (_IX_BOARD_TYPE_ == _IX_IXDP2401_)
    char *modeDBStr;
    char oc3ModeStr[] = "oc3,pos";
    char oc12ModeStr[] = "oc12,pos";

    ix_int32 ret;
    ix_uint32 mtu0, mtu1;
    ix_uint32 link_speed;
    pos_atm_ixf6048_pos_cfg_t cfg;

    /* Retrieve data fro registry */
    _ix_cc_pos_retrieve_registry_data("System_Properties/PORT00/MTU", &mtu0);
    _ix_cc_pos_retrieve_registry_data("System_Properties/PORT01/MTU", &mtu1);
    _ix_cc_pos_retrieve_registry_data("System_Properties/PORT00/LINK_SPEED",
                                      &link_speed);

    if (link_speed == IX_POS_OC12_LINK_SPEED)
    {
        modeDBStr = oc12ModeStr;
    }
    else if (link_speed == IX_POS_OC3_LINK_SPEED)
    {
        modeDBStr = oc3ModeStr;
    }
    else
    {
        return(IX_ERROR_LOCAL(IX_CC_ATM_POS_TX_ERROR_FRAMER_INIT,
                              ("Initialization of ATM/POS framer device"
                               " failed. Invalid framer mode selected.")));
    }

    ix_ossl_message_log("DB mode:%s\n", modeDBStr);
    /*
     * Start driver (Linux has this function empty)
     */
    ret = PosAtm_DriverStart(modeDBStr, modeDBStr);
    if (ret < 0)
    {
        return IX_ERROR_REMOTE(IX_CC_ATM_POS_TX_ERROR_FRAMER_INIT,
                               ("Initialization of ATM/POS framer device"
                                " failed"));
    }

#ifdef USE_BB_DRV_CONFIG
	switch(hwConfigPosPortMask)
	{
    	case 0	:	arg_pCtlBlk->posFd = POS_ATM_INV_HANDLE;
        			return IX_ERROR_REMOTE(IX_CC_ATM_POS_TX_ERROR_FRAMER_INIT,
                               			   ("There is no POS card in DB0 or DB1.\n"));
    	case 1	:	arg_pCtlBlk->posFd = PosAtm_Open(0);
					break;
		case 2	:	arg_pCtlBlk->posFd = PosAtm_Open(1);
					break;
		case 3	:	arg_pCtlBlk->posFd = PosAtm_Open(0);
					break;
		default	:	arg_pCtlBlk->posFd = POS_ATM_INV_HANDLE;
        			return IX_ERROR_REMOTE(IX_CC_ATM_POS_TX_ERROR_FRAMER_INIT,
                               			   ("There is no POS card in DB0 or DB1.\n"));
	}
#else /* USE_BB_DRV_CONFIG */
#if defined IX_CONFIGURATION_atmdb0
    /*
     * Open mezzanine DB1
     */
    arg_pCtlBlk->posFd = PosAtm_Open(0);
#else
    /*
     * Open mezzanine DB2
     */
    arg_pCtlBlk->posFd = PosAtm_Open(1);
#endif /* IX_CONFIGURATION_atmdb0 */
#endif /* USE_BB_DRV_CONFIG */

    if (arg_pCtlBlk->posFd == POS_ATM_INV_HANDLE)
    {
        ix_ossl_message_log("\nWarning: Cannot open POS ATM daughter card\n");
    }

    if (arg_pCtlBlk->posFd != POS_ATM_INV_HANDLE)
    {
        if(PosAtm_SetLinkState(arg_pCtlBlk->posFd, 0, POS_ATM_STATE_UP) < 0)
        {
            ix_ossl_message_log(
                "\nERROR: Cannot enable port 0 on daughter card\n");
        }

        if(PosAtm_SetLinkState(arg_pCtlBlk->posFd, 1, POS_ATM_STATE_UP) < 0)
        {
            ix_ossl_message_log(
                "\nERROR: Cannot enable port 1 on daughter card\n");
        }

        cfg.link = 0;
        if (PosAtm_GetPosCfg(arg_pCtlBlk->posFd, &cfg) < 0)
        {
            ix_ossl_message_log(
                "\nERROR: Cannot read port 0 config on daughter card\n");
        }
        else
        {
            cfg.maxPktSize = mtu0;
            if (PosAtm_SetPosCfg(arg_pCtlBlk->posFd, &cfg) < 0)
            {
                ix_ossl_message_log(
                    "\nERROR: Cannot set port 0 config on daughter card\n");
            }
        }


        cfg.link = 1;
        if (PosAtm_GetPosCfg(arg_pCtlBlk->posFd, &cfg) < 0)
        {
            ix_ossl_message_log(
                "\nERROR: Cannot read port 1 config on daughter card \n");
        }
        else
        {
            cfg.maxPktSize = mtu1;
            if (PosAtm_SetPosCfg(arg_pCtlBlk->posFd, &cfg) < 0)
            {
                ix_ossl_message_log(
                    "\nERROR: Cannot set port 1 config on daughter card \n");
            }
        }
    }


#elif  (_IX_BOARD_TYPE_ == _IX_IXDP2801_)

    ix_int32 ret;
    ix_uint32 mtu, port;
    ix_uint32 linkSpeedDB1 = 0, linkSpeedDB2 = 0;

    pos_atm_ixf6048_pos_cfg_t cfg;
    char *modeDB1Str, *modeDB2Str;
    char oc3ModeStr[] = "oc3,pos";
    char oc12ModeStr[] = "oc12,pos";
    char oc48ModeStr[] = "oc48,pos";
    ix_cc_properties_node *pProp;
    ix_cc_init_context *pInitCtx = arg_pCtlBlk->pInitCtx;
    ix_uint32 posMask = 0;

    /*
     * Go through all interface properties and recognize POS ports, then
     * store them in posMask, get per DB link speeds also
     */
    for (pProp = pInitCtx->pPropertyList;
         pProp != NULL; pProp = pProp->pNextPropNode)
    {
        if (pProp->pPropertyInfo->media_type == IX_MEDIA_TYPE_POS)
        {
            posMask |= 1 << pProp->pPropertyInfo->port_id;
            if (IX_POS_DB1_MASK & (1 << pProp->pPropertyInfo->port_id))
            {
                linkSpeedDB1 = pProp->pPropertyInfo->link_speed;
            }
            else if (IX_POS_DB2_MASK & (1 << pProp->pPropertyInfo->port_id))
            {
                linkSpeedDB2 = pProp->pPropertyInfo->link_speed;
            }
        }
    }

    /*
     * Adjust driver mode string according to link speed (this is a trivial
     * solution when all ifs have the same link speed on a particular DB)
     */
    if (posMask & IX_POS_DB1_MASK)
    {
        switch (linkSpeedDB1)
        {
        case IX_POS_OC48_LINK_SPEED:
            modeDB1Str = oc48ModeStr; break;
        case IX_POS_OC12_LINK_SPEED:
            modeDB1Str = oc12ModeStr; break;
        case IX_POS_OC3_LINK_SPEED:
            modeDB1Str = oc3ModeStr; break;
        default:
            return
                (IX_ERROR_LOCAL(IX_CC_ATM_POS_TX_ERROR_FRAMER_INIT,
                                ("Initialization of ATM/POS framer device"
                                 " failed. Invalid framer mode selected.")));
        }
    }
    else /* set by default oc12 pos if there aren't any port on DB1 */
    {
        modeDB1Str = oc12ModeStr;
    }

    if (posMask & IX_POS_DB2_MASK)
    {
        switch (linkSpeedDB2)
        {
        case IX_POS_OC48_LINK_SPEED:
            modeDB2Str = oc48ModeStr; break;
        case IX_POS_OC12_LINK_SPEED:
            modeDB2Str = oc12ModeStr; break;
        case IX_POS_OC3_LINK_SPEED:
            modeDB2Str = oc3ModeStr; break;
        default:
            return
                (IX_ERROR_LOCAL(IX_CC_ATM_POS_TX_ERROR_FRAMER_INIT,
                                ("Initialization of ATM/POS framer device"
                                 " failed. Invalid framer mode selected.")));
        }
    }
    else /* set by default oc12 pos if there aren't any port on DB2 */
    {
        modeDB2Str = oc12ModeStr;
    }

    /*
     * Start driver (Linux has this function empty)
     */
    ret = PosAtm_DriverStart(modeDB1Str, modeDB2Str);
    if (ret < 0)
    {
        return IX_ERROR_REMOTE(IX_CC_ATM_POS_TX_ERROR_FRAMER_INIT,
                               ("Initialization of ATM/POS framer"
                                " device failed"));
    }
    /*
     * Open up to two instances of the driver (depending on the mask)
     */
    if (posMask & IX_POS_DB2_MASK)
    {
        arg_pCtlBlk->posFd[1] = PosAtm_Open(1);
        if (arg_pCtlBlk->posFd[1] == POS_ATM_INV_HANDLE)
        {
            ix_ossl_message_log("\nWarning: Cannot open POS ATM daughter"
                                " card 2\n");
        }
    }
    if (posMask & IX_POS_DB1_MASK)
    {
        arg_pCtlBlk->posFd[0] = PosAtm_Open(0);
        if (arg_pCtlBlk->posFd[0] == POS_ATM_INV_HANDLE)
        {
            ix_ossl_message_log("\nWarning: Cannot open POS ATM daughter"
                                " card 1\n");
        }
    }

    /*
     * Go through all POS ports and init them
     */
    for (port = 0; port < IX_CC_POS_TX_MAX_NUM_POS_PORTS; port++)
    {
        if ((posMask & IX_POS_DB1_MASK) & (1 << port))
        {
            /* recognize MTU */
            for (pProp = pInitCtx->pPropertyList;
                 pProp != NULL; pProp = pProp->pNextPropNode)
            {
                if (pProp->pPropertyInfo->port_id == port)
                {
                    mtu = pProp->pPropertyInfo->mtu;
                }
            }
            /* set MTU and up the link */
            cfg.link = port;
            if (PosAtm_GetPosCfg(arg_pCtlBlk->posFd[0], &cfg) < 0)
            {
                ix_ossl_message_log("\nERROR: Cannot read port on config"
                                    " daughter card 1\n");
            }
            else
            {
                cfg.maxPktSize = mtu;
                if (PosAtm_SetPosCfg(arg_pCtlBlk->posFd[0], &cfg) < 0)
                {
                    ix_ossl_message_log("\nERROR: Cannot set port on config"
                                        " daughter card 1\n");
                }
                if (PosAtm_SetLinkState(arg_pCtlBlk->posFd[0], port,
                                        POS_ATM_STATE_UP) < 0)
                {
                    ix_ossl_message_log("\nERROR: Cannot enable port %d"
                                        " on daughter card 1\n", (int)port);
                }
            }
        } /* end of DB1 section */

        if ((posMask & IX_POS_DB2_MASK) & (1 << port))
        {
            /* recognize MTU */
            for (pProp = pInitCtx->pPropertyList;
                 pProp != NULL; pProp = pProp->pNextPropNode)
            {
                if (pProp->pPropertyInfo->port_id == port)
                {
                    mtu = pProp->pPropertyInfo->mtu;
                }
            }
            /* set MTU and up the link */
            cfg.link = port;
            if (PosAtm_GetPosCfg(arg_pCtlBlk->posFd[1], &cfg) < 0)
            {
                ix_ossl_message_log("\nERROR: Cannot read port on config"
                                    " daughter card 2\n");
            }
            else
            {
                cfg.maxPktSize = mtu;
                if (PosAtm_SetPosCfg(arg_pCtlBlk->posFd[1], &cfg) < 0)
                {
                    ix_ossl_message_log("\nERROR: Cannot set port on config"
                                        " daughter card 2\n");
                }
                if (PosAtm_SetLinkState(arg_pCtlBlk->posFd[1], port,
                                        POS_ATM_STATE_UP) < 0)
                {
                    ix_ossl_message_log("\nERROR: Cannot enable port %d"
                                        " on daughter card 2\n", (int)port);
                }
            }
        } /* end of DB2 section */
    } /* end of for */


#else /* _IX_BOARD_TYPE_ not _IX_IXDP2x01_ */

    ixf6048_ChipCfg_t *pChipCfg;



    /* first initialize the slow port */
    _ix_cc_atm_pos_tx_init_slow_port();

    /* Allocate the bb_ChipData_t structure */
    if ((arg_pCtlBlk->pChipData = (bb_ChipData_t *)ix_ossl_malloc(sizeof(bb_ChipData_t))) == NULL)
    {
        return(IX_ERROR_REMOTE(IX_CC_ERROR_OOM_SYSTEM,
                               ("Out of memory for framer bb_ChipData_t.")));
    }

    /* Allocate the ixf6048_ChipCfg_t structure */
    if ((arg_pCtlBlk->pChipData->pChipCfg = (void *)ix_ossl_malloc(sizeof(ixf6048_ChipCfg_t))) == NULL)
    {
        /* Free framer data structures */
        ix_ossl_free(arg_pCtlBlk->pChipData);

        return(IX_ERROR_REMOTE(IX_CC_ERROR_OOM_SYSTEM,
                               ("Out of memory for framer ixf6048_ChipCfg_t.")));
    }

    /* Allocate the ixf6048_ChanCfg_t structure */
    if ((arg_pCtlBlk->pChanCfg = (ixf6048_ChanCfg_t *)ix_ossl_malloc(sizeof(ixf6048_ChanCfg_t))) == NULL)
    {
        ix_ossl_free(arg_pCtlBlk->pChipData->pChipCfg);
        ix_ossl_free(arg_pCtlBlk->pChipData);

        return(IX_ERROR_REMOTE(IX_CC_ERROR_OOM_SYSTEM,
                               ("Out of memory for framer ixf6048_ChanCfg_t.")));
    }

    /* "Partially" initialize the framer device driver */
    if (IxfApiInit((void *)FRAMER_CHIP_BASE_ADDRESS,
                   bb_6048_CHIP,
                   arg_pCtlBlk->pChipData) != bb_NO_ERROR)
    {
        /* Free framer data structures */
        ix_ossl_free(arg_pCtlBlk->pChanCfg);
        ix_ossl_free(arg_pCtlBlk->pChipData->pChipCfg);
        ix_ossl_free(arg_pCtlBlk->pChipData);

        return(IX_ERROR_LOCAL(IX_CC_ATM_POS_TX_ERROR_FRAMER_INIT,
                              ("Initialization of ATM/POS framer device failed.")));
    }

    /**
     * Allocate memory within the pChipData for members that
     * need dynamic allocation.
     */
    if (IxfApiAllocDataStructureMem(arg_pCtlBlk->pChipData) != bb_NO_ERROR)
    {
        /* Free framer data structures */
        ix_ossl_free(arg_pCtlBlk->pChanCfg);
        ix_ossl_free(arg_pCtlBlk->pChipData->pChipCfg);
        ix_ossl_free(arg_pCtlBlk->pChipData);

        return(IX_ERROR_LOCAL(IX_CC_ATM_POS_TX_ERROR_FRAMER_ALLOC,
                              ("Dynamic memory allocation required by ATM/POS framer device driver failed.")));
    }

    /* Register an alarm callback function into the device driver. */
    if (IxfApiInitAlarmCallback(arg_pCtlBlk->pChipData,
                                (AlarmCallBack)_ix_cc_atm_pos_tx_framer_alarm_callback) != bb_NO_ERROR)
    {
        /* De-allocate memory within the pChipData for members allocated earlier. */
        IxfApiDeAllocMemory(arg_pCtlBlk->pChipData);

        /* Free framer data structures */
        ix_ossl_free(arg_pCtlBlk->pChanCfg);
        ix_ossl_free(arg_pCtlBlk->pChipData->pChipCfg);
        ix_ossl_free(arg_pCtlBlk->pChipData);

        return(IX_ERROR_LOCAL(IX_CC_ATM_POS_TX_ERROR_FRAMER_ALARM_CB,
                              ("Registering of ATM/POS framer alarm callback failed.")));
    }

    /* Configure the framer media card using the pre-generated table. */
    if (IxfApiInitChip(arg_pCtlBlk->pChipData,
                       &IxCcPosTxRegInitIXF6048Table[arg_pCtlBlk->posModeNum][0]) != bb_NO_ERROR)
    {
        /* De-allocate memory within the pChipData for members allocated earlier. */
        IxfApiDeAllocMemory(arg_pCtlBlk->pChipData);

        /* Free framer data structures */
        ix_ossl_free(arg_pCtlBlk->pChanCfg);
        ix_ossl_free(arg_pCtlBlk->pChipData->pChipCfg);
        ix_ossl_free(arg_pCtlBlk->pChipData);

        return(IX_ERROR_LOCAL(IX_CC_ATM_POS_TX_ERROR_FRAMER_INIT_CHIP,
                              ("Configuration of ATM/POS framer device failed.")));
    }

    /**
     * The following code sets up the pChipCfg structure member of the pChipData
     * structure.  This is required  for the operation of ixf6048_SetChanOnline()
     * and ixf6048_SetChanOffline() performed by the ix_cc_atm_pos_tx_set_property().
     * (See also the device driver LLD for details.)
     */
    arg_pCtlBlk->pChanCfg->TrScrEn = ixf6048_eTR_SCR_EN_SCRAMBLER_DISABLED; /* Disable Scrambler -- Valid for Transparent mode only*/
    arg_pCtlBlk->pChanCfg->RepeaterMode = ixf6048_eCHANNEL_IN_NORMAL_MODE; /* Channel is NOT in Repeater Mode */
    arg_pCtlBlk->pChanCfg->pChanIfCfg = NULL; /* Line Side & UTOPIA interface config */
    arg_pCtlBlk->pChanCfg->pOhtChanCfg = NULL; /* SDH configuration for channel */
    arg_pCtlBlk->pChanCfg->pAtmChanCfg = NULL; /* not an ATM channel */
    arg_pCtlBlk->pChanCfg->pPosChanCfg = NULL;  /* POS-specific config for channel */

    if (arg_pCtlBlk->posEnable)
    {
        arg_pCtlBlk->pChanCfg->ChanMode = ixf6048_ePOS_MODE_CHAN;
    }
    else if (arg_pCtlBlk->atmEnable)
    {
        arg_pCtlBlk->pChanCfg->ChanMode = ixf6048_eATM_MODE_CHAN;
    }

    pChipCfg = (ixf6048_ChipCfg_t *)(arg_pCtlBlk->pChipData->pChipCfg);

    pChipCfg->NumChanCfg = ixf6048_eQUAD_DCC_ORDERWIRE_IF;
    pChipCfg->pChipLevelCfg = NULL;
    pChipCfg->pChanCfg[0] = arg_pCtlBlk->pChanCfg;
    pChipCfg->pChanCfg[1] = arg_pCtlBlk->pChanCfg;
    pChipCfg->pChanCfg[2] = arg_pCtlBlk->pChanCfg;
    pChipCfg->pChanCfg[3] = arg_pCtlBlk->pChanCfg;

#endif /* _IX_BOARD_TYPE_ */

#else /*IX_PLATFORM_2401 || IX_PLATFORM_2801 */

    ixf6048_ChipCfg_t *pChipCfg;



    /* first initialize the slow port */
    _ix_cc_atm_pos_tx_init_slow_port();

    /* Allocate the bb_ChipData_t structure */
    if ((arg_pCtlBlk->pChipData = (bb_ChipData_t *)ix_ossl_malloc(sizeof(bb_ChipData_t))) == NULL)
    {
        return(IX_ERROR_REMOTE(IX_CC_ERROR_OOM_SYSTEM,
                               ("Out of memory for framer bb_ChipData_t.")));
    }

    /* Allocate the ixf6048_ChipCfg_t structure */
    if ((arg_pCtlBlk->pChipData->pChipCfg = (void *)ix_ossl_malloc(sizeof(ixf6048_ChipCfg_t))) == NULL)
    {
        /* Free framer data structures */
        ix_ossl_free(arg_pCtlBlk->pChipData);

        return(IX_ERROR_REMOTE(IX_CC_ERROR_OOM_SYSTEM,
                               ("Out of memory for framer ixf6048_ChipCfg_t.")));
    }

    /* Allocate the ixf6048_ChanCfg_t structure */
    if ((arg_pCtlBlk->pChanCfg = (ixf6048_ChanCfg_t *)ix_ossl_malloc(sizeof(ixf6048_ChanCfg_t))) == NULL)
    {
        ix_ossl_free(arg_pCtlBlk->pChipData->pChipCfg);
        ix_ossl_free(arg_pCtlBlk->pChipData);

        return(IX_ERROR_REMOTE(IX_CC_ERROR_OOM_SYSTEM,
                               ("Out of memory for framer ixf6048_ChanCfg_t.")));
    }

    /* "Partially" initialize the framer device driver */
    if (IxfApiInit((void *)FRAMER_CHIP_BASE_ADDRESS,
                   bb_6048_CHIP,
                   arg_pCtlBlk->pChipData) != bb_NO_ERROR)
    {
        /* Free framer data structures */
        ix_ossl_free(arg_pCtlBlk->pChanCfg);
        ix_ossl_free(arg_pCtlBlk->pChipData->pChipCfg);
        ix_ossl_free(arg_pCtlBlk->pChipData);

        return(IX_ERROR_LOCAL(IX_CC_ATM_POS_TX_ERROR_FRAMER_INIT,
                              ("Initialization of ATM/POS framer device failed.")));
    }

    /**
     * Allocate memory within the pChipData for members that
     * need dynamic allocation.
     */
    if (IxfApiAllocDataStructureMem(arg_pCtlBlk->pChipData) != bb_NO_ERROR)
    {
        /* Free framer data structures */
        ix_ossl_free(arg_pCtlBlk->pChanCfg);
        ix_ossl_free(arg_pCtlBlk->pChipData->pChipCfg);
        ix_ossl_free(arg_pCtlBlk->pChipData);

        return(IX_ERROR_LOCAL(IX_CC_ATM_POS_TX_ERROR_FRAMER_ALLOC,
                              ("Dynamic memory allocation required by ATM/POS framer device driver failed.")));
    }

    /* Register an alarm callback function into the device driver. */
    if (IxfApiInitAlarmCallback(arg_pCtlBlk->pChipData,
                                (AlarmCallBack)_ix_cc_atm_pos_tx_framer_alarm_callback) != bb_NO_ERROR)
    {
        /* De-allocate memory within the pChipData for members allocated earlier. */
        IxfApiDeAllocMemory(arg_pCtlBlk->pChipData);

        /* Free framer data structures */
        ix_ossl_free(arg_pCtlBlk->pChanCfg);
        ix_ossl_free(arg_pCtlBlk->pChipData->pChipCfg);
        ix_ossl_free(arg_pCtlBlk->pChipData);

        return(IX_ERROR_LOCAL(IX_CC_ATM_POS_TX_ERROR_FRAMER_ALARM_CB,
                              ("Registering of ATM/POS framer alarm callback failed.")));
    }

    /* Configure the framer media card using the pre-generated table. */
    if (IxfApiInitChip(arg_pCtlBlk->pChipData,
                       &IxCcPosTxRegInitIXF6048Table[arg_pCtlBlk->posModeNum][0]) != bb_NO_ERROR)
    {
        /* De-allocate memory within the pChipData for members allocated earlier. */
        IxfApiDeAllocMemory(arg_pCtlBlk->pChipData);

        /* Free framer data structures */
        ix_ossl_free(arg_pCtlBlk->pChanCfg);
        ix_ossl_free(arg_pCtlBlk->pChipData->pChipCfg);
        ix_ossl_free(arg_pCtlBlk->pChipData);

        return(IX_ERROR_LOCAL(IX_CC_ATM_POS_TX_ERROR_FRAMER_INIT_CHIP,
                              ("Configuration of ATM/POS framer device failed.")));
    }

    /**
     * The following code sets up the pChipCfg structure member of the pChipData
     * structure.  This is required  for the operation of ixf6048_SetChanOnline()
     * and ixf6048_SetChanOffline() performed by the ix_cc_atm_pos_tx_set_property().
     * (See also the device driver LLD for details.)
     */
    arg_pCtlBlk->pChanCfg->TrScrEn = ixf6048_eTR_SCR_EN_SCRAMBLER_DISABLED; /* Disable Scrambler -- Valid for Transparent mode only*/
    arg_pCtlBlk->pChanCfg->RepeaterMode = ixf6048_eCHANNEL_IN_NORMAL_MODE; /* Channel is NOT in Repeater Mode */
    arg_pCtlBlk->pChanCfg->pChanIfCfg = NULL; /* Line Side & UTOPIA interface config */
    arg_pCtlBlk->pChanCfg->pOhtChanCfg = NULL; /* SDH configuration for channel */
    arg_pCtlBlk->pChanCfg->pAtmChanCfg = NULL; /* not an ATM channel */
    arg_pCtlBlk->pChanCfg->pPosChanCfg = NULL;  /* POS-specific config for channel */

    if (arg_pCtlBlk->posEnable)
    {
        arg_pCtlBlk->pChanCfg->ChanMode = ixf6048_ePOS_MODE_CHAN;
    }
    else if (arg_pCtlBlk->atmEnable)
    {
        arg_pCtlBlk->pChanCfg->ChanMode = ixf6048_eATM_MODE_CHAN;
    }

    pChipCfg = (ixf6048_ChipCfg_t *)(arg_pCtlBlk->pChipData->pChipCfg);

    pChipCfg->NumChanCfg = ixf6048_eQUAD_DCC_ORDERWIRE_IF;
    pChipCfg->pChipLevelCfg = NULL;
    pChipCfg->pChanCfg[0] = arg_pCtlBlk->pChanCfg;
    pChipCfg->pChanCfg[1] = arg_pCtlBlk->pChanCfg;
    pChipCfg->pChanCfg[2] = arg_pCtlBlk->pChanCfg;
    pChipCfg->pChanCfg[3] = arg_pCtlBlk->pChanCfg;

#endif /* IX_PLATFORM_2401 || IX_PLATFORM_2801 */

    /* Set initial interface state of each port to UP */
    for (i = 0; i < arg_pCtlBlk->numChannels; i++)
    {
        arg_pCtlBlk->portState[i] = IX_CC_ATM_POS_TX_IF_STATE_UP;
    }

    return(IX_SUCCESS);

} /* _ix_cc_atm_pos_tx_init_framer */



/*#if defined(IX_PLATFORM_2401) || defined(IX_PLATFORM_2801)
 */
/* Platform specific
 * IXDP2401 has a different framer driver
 */
#if (_IX_BOARD_TYPE_ != _IX_IXDP2401_) && (_IX_BOARD_TYPE_ != _IX_IXDP2801_)
/*#endif / * IX_PLATFORM_2401 || IX_PLATFORM_2801 */

/**
 * NAME: _ix_cc_atm_pos_tx_framer_alarm_callback
 *
 * DESCRIPTION: This function is the framer alarm callback function.
 *              The function is registered in the init function and
 *              will be called by the IXF API function, IxfApiChipIsr.
 *
 * @Param: arg_pChipdata - IN: framer chip data structure
 * @Param: arg_AlarmType - IN: alarm type
 * @Param: arg_AlarmStatus - IN: alarm status structure
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */
void _ix_cc_atm_pos_tx_framer_alarm_callback(
    bb_ChipData_t* arg_pChipData,
    bb_AlarmType_e arg_AlarmType,
    void* arg_pAlarmStatus)
{
    ixf6048_OhtAlarmStatus_t *ohtAlarms;
    ix_uint32 channelNum;



    switch(arg_AlarmType)
    {
        case bb_6048_OHT_ALARM:
            ohtAlarms = (ixf6048_OhtAlarmStatus_t *)arg_pAlarmStatus;
            channelNum = ohtAlarms->Channel;

            if (ohtAlarms->RxRst.Status & ixf6048_eRX_RST_LOS_ALM) /* Loss Of Signal */
            {
                g_pCcAtmPosTxCtlBlk->portState[channelNum] = IX_CC_ATM_POS_TX_IF_STATE_DOWN;
            }
            else
            {
                g_pCcAtmPosTxCtlBlk->portState[channelNum] = IX_CC_ATM_POS_TX_IF_STATE_UP;
            }
            break;

        default:
            /* Other alarm will not be processed in release PR5 */
#ifdef IX_DEBUG
            ix_ossl_message_log("Framer alarm type %d is not supported in current release.\n", arg_AlarmType);
#endif
            break;
    }

} /* _ix_cc_atm_pos_tx_framer_alarm_callback */


/*#if defined(IX_PLATFORM_2401) || defined(IX_PLATFORM_2801)
 */
/* Platform specific
 * IXDP2401 has a different framer driver
 */
#endif /* _IX_BOARD_TYPE_ */
/*#endif / * IX_PLATFORM_2401 || IX_PLATFORM_2801 */


/**
 * NAME: _ix_cc_atm_pos_tx_fini_framer
 *
 * DESCRIPTION: This function initializes and configures the
 *              ATM/POS framer media card.
 *
 * @Param: arg_pCtlBlk - IN: pointer to the component context allocated
 *                           in the component's init function.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */
PRIVATE ix_error _ix_cc_atm_pos_tx_fini_framer(
    ix_cc_atm_pos_tx_ctl_blk *arg_pCtlBlk)
{
#if defined(IX_PLATFORM_2401) || defined(IX_PLATFORM_2801)

/* Platform specific
 * IXDP2401 has a different framer driver
 */
#if (_IX_BOARD_TYPE_ == _IX_IXDP2401_)
    PosAtm_Close(arg_pCtlBlk->posFd);
    PosAtm_DriverStop();
#elif (_IX_BOARD_TYPE_ == _IX_IXDP2801_)
    PosAtm_Close(arg_pCtlBlk->posFd[0]);
    PosAtm_Close(arg_pCtlBlk->posFd[1]);
    PosAtm_DriverStop();
#else

    bb_ChipSegment_t section;
    section.group.channel = 0;

    /* Reset the framer chip. */
    if (IxfApiReset(arg_pCtlBlk->pChipData, &section, bb_RESET_CHIP) != bb_NO_ERROR)
    {
        return(IX_ERROR_LOCAL(IX_CC_ATM_POS_TX_ERROR_FRAMER_RESET,
                              ("Reset of ATM/POS framer device failed.")));
    }

    /* Free framer data structures */
    ix_ossl_free(arg_pCtlBlk->pChanCfg);
    ix_ossl_free(arg_pCtlBlk->pChipData->pChipCfg);
    ix_ossl_free(arg_pCtlBlk->pChipData);

#endif /* _IX_BOARD_TYPE_ */

#else /*IX_PLATFORM_2401 || IX_PLATFORM_2801 */

    bb_ChipSegment_t section;
    section.group.channel = 0;

    /* Reset the framer chip. */
    if (IxfApiReset(arg_pCtlBlk->pChipData, &section, bb_RESET_CHIP) != bb_NO_ERROR)
    {
        return(IX_ERROR_LOCAL(IX_CC_ATM_POS_TX_ERROR_FRAMER_RESET,
                              ("Reset of ATM/POS framer device failed.")));
    }

    /* Free framer data structures */
    ix_ossl_free(arg_pCtlBlk->pChanCfg);
    ix_ossl_free(arg_pCtlBlk->pChipData->pChipCfg);
    ix_ossl_free(arg_pCtlBlk->pChipData);

#endif /*IX_PLATFORM_2401 || IX_PLATFORM_2801 */

    return(IX_SUCCESS);

} /* _ix_cc_atm_pos_tx_fini_framer */



/**
 * NAME: _ix_cc_atm_pos_tx_init_slow_port
 *
 * DESCRIPTION: This function initializes slow port.
 *
 * @Param: None
 *
 * @Return: None
 */
PRIVATE void _ix_cc_atm_pos_tx_init_slow_port(void)
{
    ix_uint32 i;

    /* initializing slow port */
    for (i = 0; i < IX_CC_ATM_POS_NUM_SLOW_PORT_REGS; i++)
    {
        /*
         * The following code writes values to some memory locations which can only be
         * executed on hardware.
         */
#if defined(IX_TARGET_XSCALEgnube) || defined(IX_TARGET_XSCALEgnube_Debug)
        *(volatile unsigned int *)(g_aSlowPortRegTable[i].reg) = g_aSlowPortRegTable[i].value;
#endif
    }

} /* _ix_cc_atm_pos_tx_init_slow_port */


#if defined(IX_PLATFORM_2401) || defined(IX_PLATFORM_2801)
#if (_IX_OS_TYPE_ == _IX_OS_VXWORKS_)
#if (_IX_BOARD_TYPE_ == _IX_IXDP2801_)

void framer_stats(void)
{
    pos_atm_ixf6048_counters_t stats;
    ix_int32 ret;
    ix_uint32 port, posMask;
    ix_cc_properties_node *pProp;

    stats.channel = 0; /* never changed below */

    /*
     * Go through all interface properties and recognize POS ports, then
     * store them in posMask
     */
    for (pProp = g_pCcAtmPosTxCtlBlk->pInitCtx->pPropertyList;
         pProp != NULL; pProp = pProp->pNextPropNode)
    {
        if (pProp->pPropertyInfo->media_type == IX_MEDIA_TYPE_POS)
        {
            posMask |= 1 << pProp->pPropertyInfo->port_id;
        }
    }
    for (port = 0; port < IX_CC_POS_TX_MAX_NUM_POS_PORTS; port++)
    {
        if (((posMask & IX_POS_DB1_MASK) & (1 << port)) &&
            !(g_pCcAtmPosTxCtlBlk->posFd[0] < 0))
        {
            stats.link = port;
            if (PosAtm_GetLinkStats(g_pCcAtmPosTxCtlBlk->posFd[0], &stats) < 0)
            {
                printf("\nCannot read POS(/ATM) DB1 statistics for port %d\n",
                       port);
            }
            else
            {
                printf("DB1 POS counters port %d:\n", port);
                printf(" rxFrames:          %d\n",
                       stats.counters.pos.rxFrames);
                printf(" txFrames:          %d\n",
                       stats.counters.pos.txFrames);
                printf(" rxBytes:           %d\n",
                       stats.counters.pos.rxBytes);
                printf(" txBytes:           %d\n",
                       stats.counters.pos.txBytes);
                printf(" rxAbortedFrames:   %d\n",
                       stats.counters.pos.rxAbortedFrames);
                printf(" txAbortedFrames:   %d\n",
                       stats.counters.pos.txAbortedFrames);
                printf(" rxFcsErrors:       %d\n",
                       stats.counters.pos.rxFcsErrors);
                printf(" rxMinLengthErrors: %d\n",
                       stats.counters.pos.rxMinLengthErrors);
                printf(" rxMaxLengthErrors: %d\n",
                       stats.counters.pos.rxMaxLengthErrors);
                printf(" rxFifoOverflows:   %d\n",
                       stats.counters.pos.rxFifoOverflows);
                printf(" txFifoUnderflows:  %d\n",
                       stats.counters.pos.txFifoUnderflows);
            }
        }  /* end of DB1 section */
        else if (((posMask & IX_POS_DB2_MASK) & (1 << port)) &&
                 !(g_pCcAtmPosTxCtlBlk->posFd[1] < 0))
        {
            stats.link = port;
            if (PosAtm_GetLinkStats(g_pCcAtmPosTxCtlBlk->posFd[1], &stats) < 0)
            {
                printf("\nCannot read POS(/ATM) DB2 statistics for port %d\n",
                       port);
            }
            else
            {
                printf("DB2 POS counters for port %d:\n", port);
                printf(" rxFrames:          %d\n",
                       stats.counters.pos.rxFrames);
                printf(" txFrames:          %d\n",
                       stats.counters.pos.txFrames);
                printf(" rxBytes:           %d\n",
                       stats.counters.pos.rxBytes);
                printf(" txBytes:           %d\n",
                       stats.counters.pos.txBytes);
                printf(" rxAbortedFrames:   %d\n",
                       stats.counters.pos.rxAbortedFrames);
                printf(" txAbortedFrames:   %d\n",
                       stats.counters.pos.txAbortedFrames);
                printf(" rxFcsErrors:       %d\n",
                   stats.counters.pos.rxFcsErrors);
                printf(" rxMinLengthErrors: %d\n",
                       stats.counters.pos.rxMinLengthErrors);
                printf(" rxMaxLengthErrors: %d\n",
                       stats.counters.pos.rxMaxLengthErrors);
                printf(" rxFifoOverflows:   %d\n",
                   stats.counters.pos.rxFifoOverflows);
                printf(" txFifoUnderflows:  %d\n",
                       stats.counters.pos.txFifoUnderflows);
            }
        } /* end of DB2 section */
    } /* end of for */
} /* end of framer_stats(void) */

#elif (_IX_BOARD_TYPE_ == _IX_IXDP2401_)

void framer_stats(void)
{
    pos_atm_ixf6048_counters_t stats;

    stats.link = 0;
    stats.channel = 0;

    if (g_pCcAtmPosTxCtlBlk->posFd < 0 ||
        PosAtm_GetLinkStats(g_pCcAtmPosTxCtlBlk->posFd, &stats) < 0)
    {
        printf("\nCannot read POS(/ATM) link 0 statistics\n");
    }
    else
    {
        printf("POS link 0 counters:\n");
        printf(" rxFrames:          %d\n",
               stats.counters.pos.rxFrames);
        printf(" txFrames:          %d\n",
               stats.counters.pos.txFrames);
        printf(" rxBytes:           %d\n",
               stats.counters.pos.rxBytes);
        printf(" txBytes:           %d\n",
               stats.counters.pos.txBytes);
        printf(" rxAbortedFrames:   %d\n",
               stats.counters.pos.rxAbortedFrames);
        printf(" txAbortedFrames:   %d\n",
               stats.counters.pos.txAbortedFrames);
        printf(" rxFcsErrors:       %d\n",
               stats.counters.pos.rxFcsErrors);
        printf(" rxMinLengthErrors: %d\n",
               stats.counters.pos.rxMinLengthErrors);
        printf(" rxMaxLengthErrors: %d\n",
               stats.counters.pos.rxMaxLengthErrors);
        printf(" rxFifoOverflows:   %d\n",
               stats.counters.pos.rxFifoOverflows);
        printf(" txFifoUnderflows:  %d\n",
               stats.counters.pos.txFifoUnderflows);
    }

    stats.link = 1;

    if (g_pCcAtmPosTxCtlBlk->posFd < 0 ||
        PosAtm_GetLinkStats(g_pCcAtmPosTxCtlBlk->posFd, &stats) < 0)
    {
        printf("\nCannot read POS(/ATM) link 1 statistics\n");
    }
    else
    {
        printf("POS link 1 counters:\n");
        printf(" rxFrames:          %d\n",
               stats.counters.pos.rxFrames);
        printf(" txFrames:          %d\n",
               stats.counters.pos.txFrames);
        printf(" rxBytes:           %d\n",
               stats.counters.pos.rxBytes);
        printf(" txBytes:           %d\n",
               stats.counters.pos.txBytes);
        printf(" rxAbortedFrames:   %d\n",
               stats.counters.pos.rxAbortedFrames);
        printf(" txAbortedFrames:   %d\n",
               stats.counters.pos.txAbortedFrames);
        printf(" rxFcsErrors:       %d\n",
               stats.counters.pos.rxFcsErrors);
        printf(" rxMinLengthErrors: %d\n",
               stats.counters.pos.rxMinLengthErrors);
        printf(" rxMaxLengthErrors: %d\n",
               stats.counters.pos.rxMaxLengthErrors);
        printf(" rxFifoOverflows:   %d\n",
               stats.counters.pos.rxFifoOverflows);
        printf(" txFifoUnderflows:  %d\n",
               stats.counters.pos.txFifoUnderflows);
    }

}
#endif /* _IX_BOARD_TYPE == */
#endif /* _IX_OS_TYPE_ == _IX_OS_VXWORKS_ */
#endif /* IX_PLATFORM_2401 || IX_PLATFORM_2801 */


