#ifndef	_AAL5_TX_H_
#define _AAL5_TX_H_

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2002 By Intel Corporation.  All rights reserved.
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

/*
 *
 *      File Name: aal5_tx.h
 *
 *      Description: This file contains code specific and constant
 *                   definitions.
 *
 */

#ifdef USE_IMPORT_VAR

    #include <dl_system.h>
    #include <aal5_tx_defs.h>
    #include <dispatch_loop.uc>

    .import_var AAL5_TX_CELLS_PER_OUTPORT_COUNTER_SRAM_BASE
    .import_var AAL5_TX_CELLS_PER_VCQ_COUNTER_SRAM_BASE
    .import_var AAL5_TX_PKTS_PER_OUTPORT_COUNTER_SRAM_BASE
    .import_var AAL5_TX_PKTS_PER_VCQ_COUNTER_SRAM_BASE

#else

    #include <dl_system.h>
    #include <definitions.h>
	#include <aal5_tx_defs.h>
    #include <dispatch_loop.uc>

#endif

/*
 * This is needed to retrieve calculation of MULT_FACTOR
 */
// #include <dl_buf.uc>

/*-------------------------------------------------------------------*/
/* Check for needed definitions.                                     */
/*-------------------------------------------------------------------*/
// SCR6027 - Added Ending double quote
#if (AAL5_TX_NUMBER_OF_FP_MEs > 1)
#ifdef RAW_MODE_ENABLED
#error "Raw mode not implemented for more than 1 ME"
#endif
#endif

#ifdef ATM_OAM_ENABLED
#ifndef RAW_MODE_ENABLED
#error "Raw mode should be enabled to enable OAM processing"
#endif
#endif

#if (!defined(TX_PHY_MODE) || \
	 !(TX_PHY_MODE == SPHY_4_8 || TX_PHY_MODE == SPHY_1_32 || TX_PHY_MODE == SPHY_4_8_SHARED || TX_PHY_MODE == SPHY_2_16))
#error "Define TX_PHY_MODE as SPHY_4_8, SPHY_1_32, SPHY_2_16 or TX_PHY_MODE == SPHY_4_8_SHARED."
#endif

#if !defined(IXP2400) && !defined(IXP2800)
#error "Define either IXP2400 or IXP2800."
#endif

#if defined(IXP2400) && defined(IXP2800)
#error "Define either IXP2400 or IXP2800, not both."
#endif

#ifdef USE_IMPORT_VAR

    .import_var AAL5_TX_CTX_SRAM_BASE
    .import_var AAL5_TX_AVAIL_TBUF_ELEMENT_SCRATCH_BASE
    .import_var AAL5_TX_CELLS_TX_SCRATCH_BASE

#ifdef COUNTERS

    .import_var AAL5_TX_CELLS_PER_OUTPORT_COUNTER_BASE
    .import_var AAL5_TX_CELLS_PER_VCQ_COUNTER_BASE
    .import_var AAL5_TX_PKTS_PER_OUTPORT_COUNTER_BASE
    .import_var AAL5_TX_PKTS_PER_VCQ_COUNTER_BASE

#endif /* COUNTERS */

#endif /* USE_IMPORT_VAR */

/*-------------------------------------------------------------------*/
/* User specified AAL5 TX code specific definitions.                 */
/*-------------------------------------------------------------------*/
#ifdef		FPGA_HEADER

#define		WRD_OFFSET_0	1
#define		WRD_OFFSET_1	2
#define		WRD_OFFSET_2	3
#define		WRD_OFFSET_3	4
#define		WRD_OFFSET_4	5
#define		WRD_OFFSET_5	6
#define		WRD_OFFSET_6	7
#define		WRD_OFFSET_7	8
#define		WRD_OFFSET_8	9
#define		WRD_OFFSET_9	10
#define		WRD_OFFSET_10	11
#define		WRD_OFFSET_11	12
#define		WRD_OFFSET_12	13
#define		WRD_OFFSET_13	14
#define		WRD_OFFSET_14	15
#define		WRD_OFFSET_15	16

#else

#define		WRD_OFFSET_0	0
#define		WRD_OFFSET_1	1
#define		WRD_OFFSET_2	2
#define		WRD_OFFSET_3	3
#define		WRD_OFFSET_4	4
#define		WRD_OFFSET_5	5
#define		WRD_OFFSET_6	6
#define		WRD_OFFSET_7	7
#define		WRD_OFFSET_8	8
#define		WRD_OFFSET_9	9
#define		WRD_OFFSET_10	10
#define		WRD_OFFSET_11	11
#define		WRD_OFFSET_12	12
#define		WRD_OFFSET_13	13
#define		WRD_OFFSET_14	14
#define		WRD_OFFSET_15	15

#endif


#ifndef AAL5_TX_NUMBER_OF_FP_MEs
#error "AAL5_TX_NUMBER_OF_FP_MEs have to be defined in dl_system.h!"
#endif

/* Define the pipestage 1 signal address. */
#define			PIPESTAGE1_DONE_SIG_VAL		13

/* Define the pipestage 2 signal address. */
#define			PIPESTAGE2_DONE_SIG_VAL		14

/* LM addresses in bytes. */
#define			TXC_BASE_LM					0x0
#define			CONST_BASE_LM				0x400
#define			CELL_BASE_LM				0x440

/* Set number of elements in TBUF */
#define_eval 	TBUF_ELE_NUM		        ((TOTAL_TBUF_SIZE / TBUF_ELE_SIZE) / AAL5_TX_PARTITIONS_NUMBER)

/* Masks. */
#define			CAM_ENTRY_MASK				0x0F
#define			VCQ_MASK					0xFFFF
#define			PORT_MASK					0xFFF
#define 		TX_SEQUENCE_MASK			(TBUF_ELE_NUM - 1)

/* Bit positions. */
#define			SOP_BIT						30
#define			EOP_BIT						31

/* TBUF and TBUF Control Word defines */
#define			TCW_SIZE	        		(1<<TCW_SIZE_LOG2)

/* Shift needed to convert absolute TBUF address to its TCW address. */
#define_eval    TCW_ADDR_SHIFT			    (TBUF_ELE_SIZE_LOG2 - TCW_SIZE_LOG2)

/* Threshold for the TBUF  */
#if (TX_PHY_MODE == SPHY_2_16)
#define			TBUF_THRESHOLD				58
#else
#define			TBUF_THRESHOLD		        (((TOTAL_TBUF_SIZE/TBUF_ELE_SIZE) / AAL5_TX_PARTITIONS_NUMBER) - NUM_OF_THREADS_PER_CHANNEL) 
#endif

/* IMPORTANT: This is an example code - leaved here to allow developers to check how it should be defined. */
/* #if (TX_PHY_MODE == SPHY_1_32)                                                                          */
/*                                                                                                         */
/*    #define			TBUF_THRESHOLD			TX_TBUF_THRESHOLD                                          */
/*                                                                                                         */
/*#elif (TX_PHY_MODE == SPHY_4_8_SHARED)                                                                   */
/*                                                                                                         */
/*	// set tbuf threshold to the total of available tx buffer pool per single                              */
/*	// channel minus number of threads serving single channel                                              */
/*	#define 		NUM_OF_THREADS_PER_CHANNEL	8                                                          */
/*	#define			TBUF_THRESHOLD		(((TOTAL_TBUF_SIZE/TBUF_ELE_SIZE) / 4) - NUM_OF_THREADS_PER_CHANNEL)*/ 
/*                                                                                                         */
/*#elif ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4))                                             */
/*                                                                                                         */
/*    #define			TBUF_THRESHOLD		28                                                             */
/*                                                                                                         */
/*#endif // TX_PHY_MODE                                                                                    */

/* Initial crc residue for a new packet. */
#define INIT_CRC_REMAINDER			    0xFFFFFFFF

/*-------------------------------------------------------------------*/
/* Constant MSF definitions.                                         */
/*-------------------------------------------------------------------*/

/* MSF address map. */
#define			TBUF_VALIDATE				0x1800

/* MSF control bits. */
#define			MsfSinglePhy				0
#define			MsfMultiplePhy				1

#define			MsfSimplex					0
#define			MsfFullDuplex				1

#define			MsfWidth_1x32				0
#define			MsfWidth_2x16				1
#define			MsfWidth_4x8				2
#define			MsfWidth_1x16_2x8			3

#define			MsfCell						0
#define	 		MsfPacket					1

#define			MsfUtopiaPOS				0
#define	 		MsfCSIX						1

#define			MsfElementSize_256			2
#define			MsfElementSize_128			1
#define			MsfElementSize_64			0

#define			MsfParityEven				3
#define			MsfParityOdd				2
#define			MsfParityNone				0

#define			MsfHECEnable				1
#define			MsfHECDisable				0

/* Number have to be evaluated before gluing a string */
#define_eval EVAL_TBUF_ELE_SIZE TBUF_ELE_SIZE

/* AAL5 Tx control bits. */
#ifdef IXP2400

#define_eval	TxTBUFElementSize			MsfElementSize_/**/EVAL_TBUF_ELE_SIZE/**/
#define			RxCBusMode					MsfFullDuplex
#define			TxMPHYMode					MsfSinglePhy
#define			TxMPHYEnable				0
#define			TxMode						MsfUtopiaPOS

#if (TX_PHY_MODE == SPHY_1_32)
#define			TxWidth						MsfWidth_1x32
#define			TxFlush						0x1
#define			TxEn						0x1
#endif /* (TX_PHY_MODE == SPHY_1_32) */

#if (TX_PHY_MODE == SPHY_2_16)
#define			TxWidth						MsfWidth_2x16
#define			TxFlush						0x5
#define			TxEn						0x5
#endif /* (TX_PHY_MODE == SPHY_2_16) */

#if (TX_PHY_MODE == SPHY_4_8) 
#define			TxWidth						MsfWidth_4x8
#define			TxFlush						0xF
#define			TxEn						0xF
#endif /* (TX_PHY_MODE == SPHY_4_8) */

#if (TX_PHY_MODE == SPHY_4_8_SHARED && defined(AAL5_TX_ME_INIT_MSF) )
#define_eval    TxEnMSFBits 	            ((1 << RXTX_ATM_UPSTR_LINE) | (1 << RXTX_ATM_DWNSTR_LINE))
#define			TxWidth						MsfWidth_4x8
#define			TxFlush						TxEnMSFBits
#define			TxEn						TxEnMSFBits
#endif /* (TX_PHY_MODE == SPHY_4_8_SHARED) */
     
#define			TxDRTime					0
#define			TxCellSize					MsfHECDisable
#define			TxParityMode				MsfParityEven
#define			TxCPMode					MsfCell

#endif /* IXP2400 */

#ifdef IXP2800

#define			TxTBUFPartition				0
#define_eval	TxTBUFElementSize			MsfElementSize_/**/EVAL_TBUF_ELE_SIZE/**/
#define			TxDuplexMode				MsfSimplex
#define			TxEnable					1
#define			TxFlush						0xF
#define			TxEnS						1
// Number of bits used to address SPI4, between 0..8
#define			TxSPI4bits					AAL5_TX_SPI4_BITS	

#endif /* IXP2800 */
// SCR6027 - Following defines added for OAM processing
//*************************************************************************
// Following definitions for OAM processing in AAL5 Tx
//*************************************************************************
// Local memory counter cache
#define	lm_vc_tucc0			*l$index1[LWOFFSET0]; LM cached TUCC0
#define lm_vc_tucc0_1		*l$index1[LWOFFSET1]; LM cached TUCC0_1
#define lm_vc_txN_opts		*l$index1[LWOFFSET2]; LM cached txN and options
#define lm_vc_mcsn			*l$index1[LWOFFSET3]; LM cached MCSN
#define lm_vc_bedc			*l$index1[LWOFFSET4]; LM cached BEDC
#define lm_vc_del_bfr		*l$index1[LWOFFSET5]; LM cached delayed buffer
#define lm_vc_del_bfr_hdr	*l$index1[LWOFFSET6]; LM cached delayed bfr header
#define	lm_vc_ref_cnt_flgs	*l$index1[LWOFFSET7]; LM cached ATM flags
												; and reference count

#define	lm_vp_tucc0			*l$index0[LWOFFSET0]; LM cached TUCC0 and TUCC0_1
#define lm_vp_tucc0_1		*l$index0[LWOFFSET1]; LM cached TUCC0_1
#define lm_vp_bedc_txN_mcsn	*l$index0[LWOFFSET2]; LM cached BEDC_txN_MCSN
#define	lm_vp_ref_count		*l$index0[LWOFFSET3]; LM cached ATM flags
												; and reference count

#define OAM_TX_VC_CTR_BASE		1600			; VC Ctr cache LM base
#define OAM_TX_VC_CTR_CTX0		1600			; VC Cache owned by thread 0
#define OAM_TX_VC_CTR_CTX1		1632			; VC Cache owned by thread 1
#define OAM_TX_VC_CTR_CTX2		1664			; VC Cache owned by thread 2
#define OAM_TX_VC_CTR_CTX3		1696			; VC Cache owned by thread 3
#define OAM_TX_VC_CTR_CTX4		1728			; VC Cache owned by thread 4
#define OAM_TX_VC_CTR_CTX5		1760			; VC Cache owned by thread 5
#define OAM_TX_VC_CTR_CTX6		1792			; VC Cache owned by thread 6
#define OAM_TX_VC_CTR_CTX7		1824			; VC Cache owned by thread 7
#define OAM_TX_VC_TAG_BASE		1856			; VC Ctr tag cache LM base

#define OAM_TX_VP_CTR_BASE		1888			; VP Ctr cache LM base
#define OAM_TX_VP_CTR_CTX0		1888			; VP Cache owned by thread 0
#define OAM_TX_VP_CTR_CTX1		1904			; VP Cache owned by thread 1
#define OAM_TX_VP_CTR_CTX2		1920			; VP Cache owned by thread 2
#define OAM_TX_VP_CTR_CTX3		1936			; VP Cache owned by thread 3
#define OAM_TX_VP_CTR_CTX4		1952			; VP Cache owned by thread 4
#define OAM_TX_VP_CTR_CTX5		1968			; VP Cache owned by thread 5
#define OAM_TX_VP_CTR_CTX6		1984			; VP Cache owned by thread 6
#define OAM_TX_VP_CTR_CTX7		2000			; VP Cache owned by thread 7
#define OAM_TX_VP_TAG_BASE		2016			; VP Ctr tag cache LM base

#define CRC_10_TABLE_ADDR		2048			; CRC-10 lookup table - 128 byte

#define VP_OAM_TX_TBL_SZ_SHF 	3				; Each entry = 8 bytes

#ifndef ATM_OAM_FLAG_DESCR
#define ATM_OAM_FLAG_DESCR		0xC1FFFFFF		; dummy descr for PM cell
#endif // ATM_OAM_FLAG_DESCR

#define OAM_TX_DEL_BFR_CH_BT_PS	0x08			; Delayed bfr sts changed?

#define MAX_VPI_PER_PORT_SHIFT 	0x0C			; 4096 VPs per port 

;#define MAX_MCSN_VALUE			0xFF			; Maximum MCSN value 
#define OAM_FPM_CELL_TYPE		0x20			; OAM type- 0010 F/n -0000
#define F5_SEG_OAM_PTI_VAL     	0x04			; PTI 4 cells - F5 seg OAM
#define F5_ETE_OAM_PTI_VAL     	0x05			; PTI 5 cells - F5 ete OAM
#define F4_SEG_OAM_VCI_VAL		0x03			; VCI 3 -for VP segment OAM
#define F4_ETE_OAM_VCI_VAL		0x04			; VCI 4 -for VP ete OAM

#define F4_PM_IND_BITS			0x03			; F4 seg/ETE PM enabled ?
#define F5_PM_IND_BITS			0x0C			; F5 seg/ETE PM enabled ?
#define F4_ETE_PM_IND_BIT_POS	17				; F4 ETE PM enabled ?
#define F4_SEG_PM_IND_BIT_POS	16				; F4 SEG PM enabled ?
#define F5_ETE_PM_IND_BIT_POS	18				; F5 ETE PM enabled ?
#define F5_SEG_PM_IND_BIT_POS	19				; F5 SEG PM enabled ?

#define CELL_1_OFFSET			64				; Delayed cell offset
#define BFRD_CELL_FLIP_FLOP_BT	23				; Use cell offset 0 or 1 ?
#define TRANSPORT_TYPE_BIT       0              ; Transport type bit position
                                                ; Possible bit values: 
												; 0 - Payload (ATM Cell)
												; 1 - OAM Cell
#endif /* _AAL5_TX_H_ */