#ifndef	_AAL5_TX_UC_
#define	_AAL5_TX_UC_

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2002 By Intel Corporation.  All rights reserved.
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

/*
 *
 *      File Name: aal5_tx.uc                                         
 *                                                                   
 *      Description: This is the main file. It contains initialization
 *                   phase macro calls and recurring phase macro calls.
 *                                                                   
 */

#include <aal5_tx.h>
#include <aal5_tx_init.uc>
// SCR6027: Added include file for oam
#include <aal5_tx_oam.uc>
#include <sig_macros.uc>

#if (AAL5_TX_NUMBER_OF_FP_MEs > 1)

#include <aal5_tx_util_fp_me.uc>

#else // (AAL5_TX_NUMBER_OF_FP_MEs == 1)

#include <aal5_tx_util_one_me.uc>

#endif // (AAL5_TX_NUMBER_OF_FP_MEs > 1)

//-------------------------------------------------------------------------------------------
//                            Supported configurations.
//-------------------------------------------------------------------------------------------
// Rate          TX_PHY_MODE     AAL5_TX_NUMBER_OF_FP_MEs  Number of MEs	Definitions
//               (in dl_system.h)  (in dl_system.h)         running		    needed for each
//                                                          AAL5 TX.		AAL5 TX ME.
//-------------------------------------------------------------------------------------------
// * OC-48       SPHY_1_32                 3                 3            	FIRST_AAL5_TX_ME, 
//                                                                	        NEXT_AAL5_TX_ME
// 93% of OC-48  SPHY_1_32                 2                 2            	FIRST_AAL5_TX_ME,
//                                                                 	        NEXT_AAL5_TX_ME
// OC-12         SPHY_1_32                 1                 1            	FIRST_AAL5_TX_ME
//
// OC-12         SPHY_4_8_SHARED           1                 1            	FIRST_AAL5_TX_ME 
//                                                                          ONE_AAL5_TX_ME
//
// Quad OC-12    SPHY_4_8                  1                 2            	FIRST_AAL5_TX_ME,
//                                                                	        SECOND_AAL5_TX_ME
//
// Quad OC-6     SPHY_4_8                  1                 1            	FIRST_AAL5_TX_ME,
//                                                                       	ONE_AAL5_TX_ME
//-------------------------------------------------------------------------------------------
// * Default. For details refer to the AAL5 egress project readme or the AAL5 TX doc.
//-------------------------------------------------------------------------------------------

#ifdef RAW_MODE_ENABLED
#include <raw_tx_util_common.uc>
#endif // RAW_MODE_ENABLED

	;---------------------------------------------------------------
	; One-time initialization phase.
	;---------------------------------------------------------------

    ; Initialize scratch ring number, inter-thread and inter-me signals.
    _aal5_tx_init_threads(next_thread)

	.if(ctx()==0)

		; Local memory has been initialized to AAL5_TX_CELL_PADDING during
		; initialization phase. This is the padding value for an ATM cell.
		; We need to re-initialize a few local memory fields to 0.
		_aal5_tx_init_lm()

	    // SCR6027: Added oam initialization
		; Initialize local memory OAM counter cache and load CRC-10 lookup 
		; tables for ATM OAM processing. This macro executes only when 
		; ATM_OAM_ENABLED is defined to enable ATM OAM processing.
		_aal5_oam_tx_init()

		cam_clear

		#ifdef USE_ME_INIT_SIGNAL
        .sig  volatile sys_init_signal
		.addr sys_init_signal ME_INIT_SIGNAL
		#else
        /* This signal is declared in system_init() macro */
        .sig visible sys_init_signal
		#endif

	    ctx_arb[sys_init_signal]

		#ifdef FIRST_AAL5_TX_ME

		; Thread 0 of the FIRST_AAL5_TX_ME does some MSF 
		; initialization.

		#ifdef AAL5_TX_ME_INIT_MSF
		_aal5_tx_init_msf()
		#else
        #ifdef IXP2800
		// MSF Initialisation is done in Eth TXmicroblock
		move(@spi4_addr, AAL5_TX_SPI4_BASE)
        #endif
		#endif

		; At this point, this thread will start executing.

		#if (AAL5_TX_NUMBER_OF_FP_MEs > 1)

		; In order to start pipestages 1 and 2, the
		; pipestage1_done_sig and pipestage2_done_sig
		; signals are given to self.
		; During normal execution, thread 7 of the
		; prev ME will send these signals when it is
		; done with pipestages 1 and 2 respectively.
		local_csr_wr[same_me_signal, (&pipestage1_done_sig<<3)]
	    local_csr_wr[same_me_signal, (&pipestage2_done_sig<<3)]

		#endif // (AAL5_TX_NUMBER_OF_FP_MEs > 1)

		#endif // FIRST_AAL5_TX_ME

		#if ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4) || (TX_PHY_MODE == SPHY_4_8_SHARED) || (TX_PHY_MODE == SPHY_2_16))

		; Wakeup thread 4 of the same ME.
		local_csr_wr[same_me_signal, ((&sys_init_signal<<3) | (4))]

		#endif // ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4) || (TX_PHY_MODE == SPHY_4_8_SHARED) || (TX_PHY_MODE == SPHY_2_16))

	.else	

		#if (TX_PHY_MODE == SPHY_1_32)

		; Threads 1-7 of all ME's wait for a prev_thread_done_sig
		; signal.
	    ctx_arb[prev_thread_done_sig]

		#endif // (TX_PHY_MODE == SPHY_1_32)

		#if ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4) || (TX_PHY_MODE == SPHY_4_8_SHARED) || (TX_PHY_MODE == SPHY_2_16))

		.if(ctx() == 4)

		; Thread 4 of all ME's wait for the ME_INIT_SIGNAL
		; indicating completion of system initialization.
		; This signal is sent by thread 0 of the same ME after
		; it receives it from the system initialization ME.
		ctx_arb[sys_init_signal]

		.else

		; Threads 1-3, 5-7 of all ME's wait for a prev_thread_done_sig
		; signal.
	    ctx_arb[prev_thread_done_sig]

		.endif

		#endif // ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4) || (TX_PHY_MODE == SPHY_4_8_SHARED) || (TX_PHY_MODE == SPHY_2_16))

	.endif

	; Signal next thread. However, the next thread will not run until
	; this thread swaps out. We send out the next thread signal as
	; early as possible to hide the 10 cycle latency associated 
	; with receiving it.
	signal_ctx(next_thread)

	;---------------------------------------------------------------
	; Processing phase - process one ATM cell at a time in an
	;                    infinite loop.
	;---------------------------------------------------------------

	; Pipestage 1 - phase 1
	; Fetch the dequeue request from scratch ring.
	_aal5_tx_dl_source(scratch_rd_sig)

	; Wait for prev_thread_done and scratch_rd signals.
	ctx_arb[prev_thread_done_sig, scratch_rd_sig]

	; Pipestage 1 - phase 2
	; Check if the dequeue request is null.
	; If it is, follow the null_deq path.
	; If not, setup cam data to start pipestage 1.
	// SCR6027: Added oam_cell_deq# label for enqueu requests related to OAM
	_aal5_tx_check_null_deq_and_setup_cam_data(pipestage1#, null_deq#, oam_cell_deq#)

pipestage1#:

	; Pipestage 1 - phase 3
	; Fetch the transmit context from SRAM into LM.
	; If the AAL5 TX ME's are in a functional pipeline, TXC is split
	; into TXC1 and TXC2. Only TXC1 is fetched here.
	_aal5_tx_fetch_txc_in_lm(RAW_CELL_TRANSMIT#)

	; Allocate s-xfer-rd registers to pre-fetch next buffers' meta-data.
	xbuf_alloc($prefetch_meta, 2, read)

	; Pipestage 1 - phase 4
	; Read cell payload from DRAM and update TXC.
	; Pre-fetch next buffers' meta-data if needed.

	// SCR6027: Moved raw mode handling to raw_cell_transmit# label
	
	_aal5_tx_read_cell_payload_and_update_txc()

	; Pipestage 1 - phase 5
	; Prepare a 48-byte cell in local memory. If cell payload is 40 or
	; 48 bytes skip this step as an optimization.
	; If the AAL5 TX ME's are in a functional pipeline, flush TXC1 from
	; LM to SRAM.

	// SCR6027: Moved raw mode handling to raw_cell_transmit# label
	
	_aal5_tx_prepare_cell()

	; Free allocated s-xfer-rd registers.
	xbuf_free($prefetch_meta)

pipestage2#:

	; Pipestage 2 - phase 1
	; If the AAL5 TX ME's are in a functional pipeline, fetch TXC2 from
	; SRAM into LM. If not, for sop initialize the TXC fields that will
	; be used in pipestage 2.

	_aal5_tx_fetch_txc2_in_lm()

	; Pipestage 2 - phase 2
	; Compute CRC on payload and update TXC.
	; If the AAL5 TX ME's are in a functional pipeline flush TXC2 from 
	; LM to SRAM.
	; Write 52-byte cell to TBUF.
	; Also as an optimization, read the next dequeue request
	; from the scratch ring.

	// SCR6027: Moved raw mode handling to raw_cell_transmit# label
		
	_aal5_tx_compute_crc_and_transmit_cell()

oam_processing#:
	; Pipestage 2 - phase 2_1, 2_2 - Only if ATM_OAM_ENABLED defined
	_aal5_tx_oam_pm_processing(vcq)

	; Pipestage 2 - phase 3
	; Validate cell transmission.

	_aal5_tx_validate_tbuf()

	; Pipestage 1 - phase 2
	; If the dequeue request is null, execute the null_deq# path.
	; else branch to 'pipestage1#' to start the next iteration.
		// SCR6027: Added oam_cell_deq# label to jump to oam processing
	_aal5_tx_check_null_deq_and_setup_cam_data(pipestage1#, null_deq#, oam_cell_deq#)

null_deq#:
	; Null dequeue path. Includes inter-thread and inter-me signalling
	; and reading the dequeue request for the next iteration.
	_aal5_tx_null_deq()
	br[pipestage1#]
// SCR6027: Added function call to bypass phases for oam processing
oam_cell_deq#:
	_aal5_tx_oam_deq()
	br[oam_processing#]

// SCR6027: Added function call to do raw mode cell processing
raw_cell_transmit#:
#ifdef	RAW_MODE_ENABLED
	;Pipestage1 phase 4 in raw mode
	_raw_tx_handle_raw_cell_tx()
#endif
	br[oam_processing#]

#endif	// _AAL5_TX_UC_