#ifndef	_AAL5_TX_INIT_UC_
#define _AAL5_TX_INIT_UC_

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2002 By Intel Corporation.  All rights reserved.
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

/*
 *      File Name: aal5_tx_init.uc                                         
 *                                                                   
 *      Description: This file contains initialization macros.
 *
 */

#include <aal5_tx.h>
#include <aal5_tx_regs.uc>

//------------------------------------------------------------------
// _aal5_tx_init_threads()
//
//    Description: Initializes input scratch ring number,
//                 inter-thread and inter-me signals for each thread.
//
//    Parameters:
//       Outputs:
//       out_next_thread:
//           Register that holds information to signal next thread.
//
//------------------------------------------------------------------
#macro _aal5_tx_init_threads(out_next_thread)
.begin


	#if (TX_PHY_MODE == SPHY_1_32)  

	; Set the scratch ring number on which requests will arrive for
	; this thread.
	alu[ring, --, B, AAL5_TX_IN_0, <<2]

	; Setup inter-thread signalling.

	; next_ctx is the thread that this thread will signal.
	.reg next_ctx

	; When next_ctx is set to 0x80, it will signal the next thread.
	move(next_ctx, 0x80)
    alu[out_next_thread, next_ctx, OR, &prev_thread_done_sig, <<3]

	#endif // (TX_PHY_MODE == SPHY_1_32)

    #if (TX_PHY_MODE == SPHY_4_8_SHARED)

	; next_ctx is the thread that this thread will signal.
	.reg next_ctx

	; Set the scratch ring number on which requests will arrive for
	; this thread.
	alu[ring, --, B, AAL5_TX_IN_0, <<2]

	; Setup inter-thread signalling.
	immed[next_ctx, 0x80]
    
	; Set out_next_thread to contain information to signal the 
	; next thread.
    alu[out_next_thread, next_ctx, OR, &prev_thread_done_sig, <<3]
                                                           
    #endif // TX_PHY_MODE == SPHY_4_8_SHARED
        
	#ifdef ONE_AAL5_TX_ME

	#if (TX_PHY_MODE == SPHY_4_8)

	; next_ctx is the thread that this thread will signal.
	.reg next_ctx

	;Branch according to the current context.
    br=ctx[0, ctx0_init#]
    br=ctx[1, ctx1_init#]
    br=ctx[2, ctx2_init#]
    br=ctx[3, ctx3_init#]
    br=ctx[4, ctx4_init#]
    br=ctx[5, ctx5_init#]
    br=ctx[6, ctx6_init#]
    br=ctx[7, ctx7_init#]

ctx0_init#:

	; Set the scratch ring number on which requests will arrive for
	; this thread.

	#ifdef FIRST_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_0, <<2]
	#endif // FIRST_AAL5_TX_ME

    br[end_thread_init#], defer[2]

	; Setup inter-thread signalling.

	immed[next_ctx, 0x02]
	; Set out_next_thread to contain information to signal the 
	; next thread.
    alu[out_next_thread, next_ctx, OR, &prev_thread_done_sig, <<3]

ctx1_init#:

	; Set the scratch ring number on which requests will arrive for
	; this thread.

	#ifdef FIRST_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_0, <<2]
	#endif // FIRST_AAL5_TX_ME

    br[end_thread_init#], defer[2]

	; Setup inter-thread signalling.

	immed[next_ctx, 0x03]
	; Set out_next_thread to contain information to signal the 
	; next thread.
    alu[out_next_thread, next_ctx, OR, &prev_thread_done_sig, <<3]

ctx2_init#:

	; Set the scratch ring number on which requests will arrive for
	; this thread.

	#ifdef FIRST_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_1, <<2]
	#endif // FIRST_AAL5_TX_ME

    br[end_thread_init#], defer[2]

	; Setup inter-thread signalling.

	immed[next_ctx, 0x04]
	; Set out_next_thread to contain information to signal the 
	; next thread.
    alu[out_next_thread, next_ctx, OR, &prev_thread_done_sig, <<3]

ctx3_init#:

	; Set the scratch ring number on which requests will arrive for
	; this thread.

	#ifdef FIRST_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_1, <<2]
	#endif // FIRST_AAL5_TX_ME

    br[end_thread_init#], defer[2]

	; Setup inter-thread signalling.

	immed[next_ctx, 0x05]
	; Set out_next_thread to contain information to signal the 
	; next thread.
    alu[out_next_thread, next_ctx, OR, &prev_thread_done_sig, <<3]

ctx4_init#:

	; Set the scratch ring number on which requests will arrive for
	; this thread.

	#ifdef FIRST_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_2, <<2]
	#endif // FIRST_AAL5_TX_ME

    br[end_thread_init#], defer[2]

	; Setup inter-thread signalling.

	immed[next_ctx, 0x06]
	; Set out_next_thread to contain information to signal the 
	; next thread.
    alu[out_next_thread, next_ctx, OR, &prev_thread_done_sig, <<3]

ctx5_init#:

	; Set the scratch ring number on which requests will arrive for
	; this thread.
	#ifdef FIRST_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_2, <<2]
	#endif // FIRST_AAL5_TX_ME

    br[end_thread_init#], defer[2]

	; Setup inter-thread signalling.

	immed[next_ctx, 0x07]
	; Set out_next_thread to contain information to signal the 
	; next thread.
    alu[out_next_thread, next_ctx, OR, &prev_thread_done_sig, <<3]

ctx6_init#:

	; Set the scratch ring number on which requests will arrive for
	; this thread.
	#ifdef FIRST_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_3, <<2]
	#endif // FIRST_AAL5_TX_ME

    br[end_thread_init#], defer[2]

	; Setup inter-thread signalling.

	immed[next_ctx, 0x01]
	; Set out_next_thread to contain information to signal the 
	; next thread.
    alu[out_next_thread, next_ctx, OR, &prev_thread_done_sig, <<3]

ctx7_init#:

	; Set the scratch ring number on which requests will arrive for
	; this thread.
	#ifdef FIRST_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_3, <<2]
	#endif // FIRST_AAL5_TX_ME

	; Setup inter-thread signalling.

	immed[next_ctx, 0x00]
	; Set out_next_thread to contain information to signal the 
	; next thread.
    alu[out_next_thread, next_ctx, OR, &prev_thread_done_sig, <<3]

end_thread_init#:

	#endif // (TX_PHY_MODE == SPHY_4_8)


	#else // (not defined ONE_AAL5_TX_ME)

	#if ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4) || (TX_PHY_MODE == SPHY_2_16))

	; next_ctx is the thread that this thread will signal.
	.reg next_ctx

	;Branch according to the current context.
    br=ctx[0, ctx0_init#]
    br=ctx[1, ctx1_init#]
    br=ctx[2, ctx2_init#]
    br=ctx[3, ctx3_init#]
    br=ctx[4, ctx4_init#]
    br=ctx[5, ctx5_init#]
    br=ctx[6, ctx6_init#]
    br=ctx[7, ctx7_init#]

ctx0_init#:

	; Set the scratch ring number on which requests will arrive for
	; this thread.

	#ifdef FIRST_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_0, <<2]
	#endif // FIRST_AAL5_TX_ME

	#ifdef SECOND_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_2, <<2]
	#endif // SECOND_AAL5_TX_ME

    br[end_thread_init#], defer[2]

	; Setup inter-thread signalling.

	immed[next_ctx, 0x01]
	; Set out_next_thread to contain information to signal the 
	; next thread.
    alu[out_next_thread, next_ctx, OR, &prev_thread_done_sig, <<3]

ctx1_init#:

	; Set the scratch ring number on which requests will arrive for
	; this thread.

	#ifdef FIRST_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_0, <<2]
	#endif // FIRST_AAL5_TX_ME

	#ifdef SECOND_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_2, <<2]
	#endif // SECOND_AAL5_TX_ME

    br[end_thread_init#], defer[2]

	; Setup inter-thread signalling.

	immed[next_ctx, 0x02]
	; Set out_next_thread to contain information to signal the 
	; next thread.
    alu[out_next_thread, next_ctx, OR, &prev_thread_done_sig, <<3]

ctx2_init#:

	; Set the scratch ring number on which requests will arrive for
	; this thread.

	#ifdef FIRST_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_0, <<2]
	#endif // FIRST_AAL5_TX_ME

	#ifdef SECOND_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_2, <<2]
	#endif // SECOND_AAL5_TX_ME

    br[end_thread_init#], defer[2]

	; Setup inter-thread signalling.

	immed[next_ctx, 0x03]
	; Set out_next_thread to contain information to signal the 
	; next thread.
    alu[out_next_thread, next_ctx, OR, &prev_thread_done_sig, <<3]

ctx3_init#:

	; Set the scratch ring number on which requests will arrive for
	; this thread.

	#ifdef FIRST_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_0, <<2]
	#endif // FIRST_AAL5_TX_ME

	#ifdef SECOND_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_2, <<2]
	#endif // SECOND_AAL5_TX_ME

    br[end_thread_init#], defer[2]

	; Setup inter-thread signalling.

	immed[next_ctx, 0x00]
	; Set out_next_thread to contain information to signal the 
	; next thread.
    alu[out_next_thread, next_ctx, OR, &prev_thread_done_sig, <<3]

ctx4_init#:

	; Set the scratch ring number on which requests will arrive for
	; this thread.

	#ifdef FIRST_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_1, <<2]
	#endif // FIRST_AAL5_TX_ME

	#ifdef SECOND_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_3, <<2]
	#endif // SECOND_AAL5_TX_ME

    br[end_thread_init#], defer[2]

	; Setup inter-thread signalling.

	immed[next_ctx, 0x05]
	; Set out_next_thread to contain information to signal the 
	; next thread.
    alu[out_next_thread, next_ctx, OR, &prev_thread_done_sig, <<3]

ctx5_init#:

	; Set the scratch ring number on which requests will arrive for
	; this thread.
	#ifdef FIRST_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_1, <<2]
	#endif // FIRST_AAL5_TX_ME

	#ifdef SECOND_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_3, <<2]
	#endif // SECOND_AAL5_TX_ME

    br[end_thread_init#], defer[2]

	; Setup inter-thread signalling.

	immed[next_ctx, 0x06]
	; Set out_next_thread to contain information to signal the 
	; next thread.
    alu[out_next_thread, next_ctx, OR, &prev_thread_done_sig, <<3]

ctx6_init#:

	; Set the scratch ring number on which requests will arrive for
	; this thread.
	#ifdef FIRST_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_1, <<2]
	#endif // FIRST_AAL5_TX_ME

	#ifdef SECOND_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_3, <<2]
	#endif // SECOND_AAL5_TX_ME

    br[end_thread_init#], defer[2]

	; Setup inter-thread signalling.

	immed[next_ctx, 0x07]
	; Set out_next_thread to contain information to signal the 
	; next thread.
    alu[out_next_thread, next_ctx, OR, &prev_thread_done_sig, <<3]

ctx7_init#:

	; Set the scratch ring number on which requests will arrive for
	; this thread.
	#ifdef FIRST_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_1, <<2]
	#endif // FIRST_AAL5_TX_ME

	#ifdef SECOND_AAL5_TX_ME
	alu[ring, --, B, AAL5_TX_IN_3, <<2]
	#endif // SECOND_AAL5_TX_ME

	; Setup inter-thread signalling.

	immed[next_ctx, 0x04]
	; Set out_next_thread to contain information to signal the 
	; next thread.
    alu[out_next_thread, next_ctx, OR, &prev_thread_done_sig, <<3]

end_thread_init#:

	#endif // ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4) || (TX_PHY_MODE == SPHY_2_16))

	#endif // ONE_AAL5_TX_ME


	; Setup inter-me signalling if needed.

	#if (AAL5_TX_NUMBER_OF_FP_MEs > 1)  

	; Used to setup pipestage1 and pipestage2 signals.
	.reg sig

    ; Set start_pipestage1 to contain information to signal pipestage1
	; of the next ME.
	alu[sig, --, B, &pipestage1_done_sig]
    alu[sig, sig, OR, NEXT_AAL5_TX_ME, <<7]
    alu[start_pipestage1, --, B, sig]

    ; Set start_pipestage2 to contain information to signal pipestage2
	; of the next ME.
	alu[sig, --, B, &pipestage2_done_sig]
    alu[sig, sig, OR, NEXT_AAL5_TX_ME, <<7]
    alu[start_pipestage2, --, B, sig]

	#endif // (AAL5_TX_NUMBER_OF_FP_MEs > 1)	

.end
#endm


//------------------------------------------------------------------
// _aal5_tx_init_msf()
//
//    Description: Initializes MSF CSR's.
//
//    Parameters: None
//
//------------------------------------------------------------------
#macro _aal5_tx_init_msf()
.begin

	.reg $msf_data msf_data
	.reg msf_addr
	.sig msf_done_sig

	#ifdef IXP2400

    ; Initialize Tx_UP_Control.
	.reg Tx_UP_Control_data

	immed[Tx_UP_Control_data, ((TX_CP_MODE_CELL << TX_CP_MODE_BITS) | \
							   (TX_PARITY_ODD << TX_PARITY_BITS) | \
							   (TX_CELL_SIZE_52_BYTES << TX_CELL_SIZE_BITS) | \
							   (TX_DR_TIME_2_CLOCK << TX_DR_TIME_BITS))]

	#if defined(AAL5_TX_POS_LINE_0) || defined(AAL5_TX_POS_LINE_1) || \
         defined(AAL5_TX_POS_LINE_2) || defined(AAL5_TX_POS_LINE_3)

	; Initialize Tx_UP_Control to POS-PHY L3.
	.reg Tx_UP_Control_data_pos

	immed[Tx_UP_Control_data_pos, ((TX_CP_MODE_PACKET << TX_CP_MODE_BITS) | \
							   	   (TX_PARITY_EVEN << TX_PARITY_BITS) | \
							   	   (TX_CELL_SIZE_52_BYTES << TX_CELL_SIZE_BITS) | \
							   	   (TX_DR_TIME_2_CLOCK << TX_DR_TIME_BITS))]

	#endif // AAL5_TX_POS_LINE_0 || AAL5_TX_POS_LINE_1 || AAL5_TX_POS_LINE_2 || AAL5_TX_POS_LINE_3

    #ifndef TX_ATM_LINES

    	#ifdef AAL5_TX_POS_LINE_0
	    	alu[msf_data, --, B, Tx_UP_Control_data_pos, <<16]
    	#else
	    	alu[msf_data, --, B, Tx_UP_Control_data, <<16]
    	#endif

	    immed[msf_addr, TX_UP_CTRL_0]
    	msf[fast_wr, --, msf_data, msf_addr]

		#if (TX_PHY_MODE == SPHY_2_16)

		immed[msf_addr, TX_UP_CTRL_2]
		msf[fast_wr, --, msf_data, msf_addr]

		#endif //(TX_PHY_MODE == SPHY_2_16) 

    	#if ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4) || (TX_PHY_MODE == SPHY_4_8_SHARED))

	    #ifdef AAL5_TX_POS_LINE_1
    		alu[msf_data, --, B, Tx_UP_Control_data_pos, <<16]
    	#else
    		alu[msf_data, --, B, Tx_UP_Control_data, <<16]
    	#endif

    	immed[msf_addr, TX_UP_CTRL_1]
    	msf[fast_wr, --, msf_data, msf_addr]

    	#ifdef AAL5_TX_POS_LINE_2
    		alu[msf_data, --, B, Tx_UP_Control_data_pos, <<16]
    	#else
    		alu[msf_data, --, B, Tx_UP_Control_data, <<16]
    	#endif

    	immed[msf_addr, TX_UP_CTRL_2]
    	msf[fast_wr, --, msf_data, msf_addr]

    	#ifdef AAL5_TX_POS_LINE_3
    		alu[msf_data, --, B, Tx_UP_Control_data_pos, <<16]
    	#else
    		alu[msf_data, --, B, Tx_UP_Control_data, <<16]
    	#endif

    	immed[msf_addr, TX_UP_CTRL_3]
    	msf[fast_wr, --, msf_data, msf_addr]

    	#endif // ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4) || (TX_PHY_MODE == SPHY_4_8_SHARED))

    #else // !defined(TX_ATM_LINES)

        #if (TX_ATM_LINES & 0x1)

        	#ifdef AAL5_TX_POS_LINE_0
        		alu[msf_data, --, B, Tx_UP_Control_data_pos, <<16]
        	#else
        		alu[msf_data, --, B, Tx_UP_Control_data, <<16]
        	#endif

        	immed[msf_addr, TX_UP_CTRL_0]
        	msf[fast_wr, --, msf_data, msf_addr]

        #endif // (TX_ATM_LINES & 0x1)

    	#if ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4) || (TX_PHY_MODE == SPHY_4_8_SHARED))

            #if (TX_ATM_LINES & 0x2)

            	#ifdef AAL5_TX_POS_LINE_1
            		alu[msf_data, --, B, Tx_UP_Control_data_pos, <<16]
            	#else
            		alu[msf_data, --, B, Tx_UP_Control_data, <<16]
            	#endif

            	immed[msf_addr, TX_UP_CTRL_1]
            	msf[fast_wr, --, msf_data, msf_addr]

            #endif // (TX_ATM_LINES & 0x2)

            #if (TX_ATM_LINES & 0x4)

            	#ifdef AAL5_TX_POS_LINE_2
            		alu[msf_data, --, B, Tx_UP_Control_data_pos, <<16]
            	#else
               		alu[msf_data, --, B, Tx_UP_Control_data, <<16]
            	#endif

            	immed[msf_addr, TX_UP_CTRL_2]
            	msf[fast_wr, --, msf_data, msf_addr]

            #endif // (TX_ATM_LINES & 0x4)

            #if (TX_ATM_LINES & 0x8)

            	#ifdef AAL5_TX_POS_LINE_3
            		alu[msf_data, --, B, Tx_UP_Control_data_pos, <<16]
            	#else
            		alu[msf_data, --, B, Tx_UP_Control_data, <<16]
            	#endif

            	immed[msf_addr, TX_UP_CTRL_3]
            	msf[fast_wr, --, msf_data, msf_addr]

            #endif // (TX_ATM_LINES & 0x8)

    	#endif // ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4) || (TX_PHY_MODE == SPHY_4_8_SHARED))

    #endif // !defined(TX_ATM_LINES)
     
    ; Initialize MSF_Tx_Control.
	.reg MSF_Tx_Control_data

#ifdef UPDATE_MSF_TX_CNTL

	immed[msf_addr, MSF_TX_CNTL]

	msf[read, $msf_data, msf_addr, 0, 1], ctx_swap[msf_done_sig]
	alu[$msf_data, $msf_data, OR, TX_ATM_LINES, <<28]

#else

	move(MSF_Tx_Control_data, ((0                 << 28) | \
							   (TxFlush           << 24) | \
							   (TxMode            << 22) | \
							   (TxWidth           << 20) | \
							   (TxMPHYEnable      << 19) | \
							   (TxMPHYMode        << 18) | \
							   (RxCBusMode        << 16) | \
							   (TxTBUFElementSize <<  2)))

	alu[$msf_data, --, B, MSF_Tx_Control_data]

	immed[msf_addr, MSF_TX_CNTL]

	msf[write, $msf_data, msf_addr, 0, 1], ctx_swap[msf_done_sig]

    ; Enable transmission.
	alu[MSF_Tx_Control_data, MSF_Tx_Control_data, OR, TxEn, <<28];

	alu[$msf_data, --, B, MSF_Tx_Control_data]

#endif
	immed[msf_addr, MSF_TX_CNTL]

	msf[write, $msf_data, msf_addr, 0, 1], ctx_swap[msf_done_sig]

	#endif // IXP2400

	
	#ifdef IXP2800

    ; Initialize MSF_Tx_Control and enable transmission.
	.reg MSF_Tx_Control_data

	move(MSF_Tx_Control_data,	((TxEnS             << 29) | \
								 (TxFlush           << 24) | \
								 (TxDuplexMode      << 16) | \
								 (TxEnable          << 10) | \
								 (TxTBUFElementSize <<  2) | \
								 (TxTBUFPartition)))

	move(@spi4_addr, AAL5_TX_SPI4_BASE)

	alu[$msf_data, --, B, MSF_Tx_Control_data]

	immed[msf_addr, MSF_TX_CNTL]

	msf[write, $msf_data, msf_addr, 0, 1], ctx_swap[msf_done_sig]

	#endif // IXP2800


	#ifdef IXP2800

	; Set the CSIX_TYPE_MAP register.
	immed[$msf_data, 0x8]

	immed[msf_addr, CSIX_TYPE_MAP]

	msf[write, $msf_data, msf_addr, 0, 1], ctx_swap[msf_done_sig]

	#endif // IXP2800

.end
#endm

//------------------------------------------------------------------
// _aal5_tx_init_txc_lm()
//
//    Description: Initializes ref_cnt and prefetching_in_progress
//    fields for all 16 TXC locations in local memory.
//
//    Parameters: None.
//
//------------------------------------------------------------------
#macro _aal5_tx_init_txc_lm()
.begin
    .reg txc_num, lm_addr_base

	immed[txc_num, 0]
	move(lm_addr_base, TXC_BASE_LM)

	.while(txc_num <= 15)
    
		local_csr_wr[active_lm_addr_0, lm_addr_base]

		//SCR6027 : Corrected comment below
    	// set the TXC base in LM for use in next iteration
    	alu_shf_left(lm_addr_base, TXC_BASE_LM, +, txc_num, 6]
		alu[txc_num, txc_num, +, 1]

		move(*l$index0[0], 0)
		move(*l$index0[1], 0)
		move(*l$index0[2], 0)
		move(*l$index0[3], 0)
		move(*l$index0[4], 0)
		move(*l$index0[5], 0)
		move(*l$index0[6], 0)
		move(*l$index0[7], 0)
		move(*l$index0[8], 0)

	.endw

.end
#endm

//------------------------------------------------------------------
// _aal5_tx_init_ref_cnt_lm()
//
//    Description: Initializes ref_cnt fields for all 16 TXC 
//    locations in local memory.
//
//    Parameters: None.
//
//------------------------------------------------------------------
#macro _aal5_tx_init_ref_cnt_lm()
.begin
   .reg txc_num, lm_addr_base

	immed[txc_num, 0]
	move(lm_addr_base, TXC_BASE_LM)

	.while(txc_num <= 15)
    
		local_csr_wr[active_lm_addr_0, lm_addr_base]

		//SCR6027 : Corrected comment below
    	// set the TXC base in LM for use in next iteration
    	alu_shf_left(lm_addr_base, TXC_BASE_LM, +, txc_num, 6]
		alu[txc_num, txc_num, +, 1]

        // Set initial value to ZERO
		move(*l$index0[9], 0)

	.endw

.end
#endm

//------------------------------------------------------------------
// _aal5_tx_init_cell_lm()
//
//    Description: Initializes place where ATM cells are built and 
//    CRC is calculated for all 16 TXC locations in local memory.
//
//    Parameters: None.
//
//------------------------------------------------------------------
#macro _aal5_tx_init_cell_lm()
.begin
    .reg txc_num, lm_addr_base

	immed[txc_num, 0]
	move(lm_addr_base, CELL_BASE_LM)

// SCR6027: Changed loop from 15 to 7 since there is one buffer
// per thread in this ME.

	.while(txc_num <= 7)
    
		local_csr_wr[active_lm_addr_0, lm_addr_base]

    	; Depending on the current thread, set the cell payload base in LM.
    	alu_shf_left(lm_addr_base, CELL_BASE_LM, +, txc_num, 6]

		alu[txc_num, txc_num, +, 1]

		move(*l$index0[0], AAL5_TX_CELL_PADDING)
		move(*l$index0[1], AAL5_TX_CELL_PADDING)
		move(*l$index0[2], AAL5_TX_CELL_PADDING)
		move(*l$index0[3], AAL5_TX_CELL_PADDING)
		move(*l$index0[4], AAL5_TX_CELL_PADDING)
		move(*l$index0[5], AAL5_TX_CELL_PADDING)
		move(*l$index0[6], AAL5_TX_CELL_PADDING)
		move(*l$index0[7], AAL5_TX_CELL_PADDING)
		move(*l$index0[8], AAL5_TX_CELL_PADDING)
		move(*l$index0[9], AAL5_TX_CELL_PADDING)
		move(*l$index0[10], AAL5_TX_CELL_PADDING)
		move(*l$index0[11], AAL5_TX_CELL_PADDING)
		move(*l$index0[12], AAL5_TX_CELL_PADDING)
		move(*l$index0[13], AAL5_TX_CELL_PADDING)
		move(*l$index0[14], AAL5_TX_CELL_PADDING)
		move(*l$index0[15], AAL5_TX_CELL_PADDING)

	.endw

.end
#endm

//------------------------------------------------------------------
// _aal5_tx_init_lm()
//
//    Description: Initializes ref_cnt and prefetching_in_progress
//    fields for all 16 TXC locations in local memory.
//
//    Parameters: None.
//
//------------------------------------------------------------------
#macro _aal5_tx_init_lm()
.begin

    /* 
     * This part of the initialization consist of initialization of 
     *  - intialization of transmit context (TXC) part
     *  - intialization of reference counters
     *  - intialization of cell storage place
     */

    _aal5_tx_init_txc_lm()
    _aal5_tx_init_ref_cnt_lm()
    _aal5_tx_init_cell_lm()

.end
#endm

#endif // _AAL5_TX_INIT_UC_