#ifndef	_AAL5_TX_REGS_UC_
#define	_AAL5_TX_REGS_UC_

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2002 By Intel Corporation.  All rights reserved.
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/                                                                  

/*                                                                   
 *                                                                   
 *      File Name: aal5_tx_regs.uc                                         
 *                                                                   
 *      Description: This file contains global register and signal
 *
 *                   definitions.
 */                                                                   

#include <aal5_tx.h>
#include <hardware.h> 
#include <xbuf.uc>

;-------------------------------------------------------------------
; Register definitions.
;-------------------------------------------------------------------

	; Scratch ring and CAM related gpr's
	.reg ring vcq \
	     cam_result cam_entry

	; Buffer handle gpr's
	.reg buf_hdl

	; Buffer gpr's
	.reg free_curr_buf, \
		 free_next_buf, \
		 freelist, \
		 prefetched_buffer

	immed[free_curr_buf, 0]
	immed[free_next_buf, 0]
	immed[prefetched_buffer, 0]

	; Sets a bit to indicate that the buffer needs to be freed.
	.reg Ox80000000
	move(Ox80000000, 0x80000000)

	.reg Ox0FFFFFF0
	move(Ox0FFFFFF0, 0x0FFFFFF0)

	; Cell gpr's
	.reg txc_packet_size \
		 txc_sram_offset	\
		 txc_port_and_clp	// 31-16 port, 0 CLP bit

	.reg cell_payload \
		 cell_payload_in_curr_buf \
		 cell_payload_in_next_buf

	.reg curr_payload_addr_offset
	.reg outport

	; Cell transmit gpr's
	.reg tbuf_addr
	.reg cells_tx_by_code
	.reg tcw_payload_offset
	.reg @spi4_addr

	; SRAM and DRAM gpr's for meta data, TXC and payload.

	.reg read	$vcq $buf_hdl_lw
	.xfer_order $vcq $buf_hdl_lw

	.reg 		$cells_tx_by_fabric
	.reg read	$avail_tbuf_element

    xbuf_alloc($$payload, 16, read)

	.reg		thread_num \
				payload_reg_base_addr_lw \
				next_payload_reg_base_lw

	local_csr_rd[active_ctx_sts]
	immed[thread_num, 0]
	alu[thread_num, 0x7, AND, thread_num]

	alu[payload_reg_base_addr_lw, --, B, thread_num, <<4]
	alu[payload_reg_base_addr_lw, payload_reg_base_addr_lw, OR, &$$payload[0]]

	.set next_payload_reg_base_lw

	; Initialize some registers based on phy mode and current thread.

	.reg partition_number
	.reg tbuf_element_base
	.reg tbuf_validate_base

	#if (TX_PHY_MODE == SPHY_1_32)

	; Stores MSF partition number (*4) on which this thread will transmit cells.
	immed[partition_number, (0*4)]

	; Stores base address of RBUF_TBUF_BASE for this thread.
	add(tbuf_element_base, RBUF_TBUF_BASE, 0)

	; Stores base address of TBUF_VALIDATE for this thread.
	add(tbuf_validate_base, TBUF_VALIDATE, 0)

	#endif // (TX_PHY_MODE == SPHY_1_32)

    #if (TX_PHY_MODE == SPHY_4_8_SHARED)

	    ; Stores MSF partition number (*4) on which this thread will transmit cells.
        alu_shf[partition_number, --, B, AAL5_TX_LINE,<<2]

        ; Stores base address of RBUF_TBUF_BASE for this thread.
		alu_shf[tbuf_element_base, --, B, AAL5_TX_LINE,<<11]
        add[tbuf_element_base, RBUF_TBUF_BASE, tbuf_element_base]

   	    ; Stores base address of TBUF_VALIDATE for this thread.

        // TBUF_VALIDATE region starts at 0x1800 and occupies 2 LW per single TBUF.
        // Its partition depends on defined TBUF size. 
		#define_eval TBUF_VAL_LOG2 log2((2 * (8192 / TBUF_ELE_SIZE)))
		alu_shf[tbuf_validate_base, --, B, AAL5_TX_LINE,<<TBUF_VAL_LOG2]
        add[tbuf_validate_base, TBUF_VALIDATE, tbuf_validate_base]
		#undef TBUF_VAL_LOG2

    #endif // (TX_PHY_MODE == SPHY_4_8_SHARED)
    
	#if ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4))

	#ifdef ONE_AAL5_TX_ME

	#ifdef FIRST_AAL5_TX_ME

	.if (ctx() == 0 || ctx() == 1)

	; Stores MSF partition number (*4) on which this thread will transmit cells.
	immed[partition_number, (0*4)]

	; Stores base address of RBUF_TBUF_BASE for this thread.
	add(tbuf_element_base, RBUF_TBUF_BASE, 0)

	; Stores base address of TBUF_VALIDATE for this thread.
	add(tbuf_validate_base, TBUF_VALIDATE, 0)

	.elif (ctx() == 2 || ctx() == 3)

	; Stores MSF partition number (*4) on which this thread will transmit cells.
	immed[partition_number, (1*4)]

	; Stores base address of RBUF_TBUF_BASE for this thread.
	add(tbuf_element_base, RBUF_TBUF_BASE, 2048)

	; Stores base address of TBUF_VALIDATE for this thread.
	add(tbuf_validate_base, TBUF_VALIDATE, 256)

	.elif (ctx() == 4 || ctx() == 5)

	; Stores MSF partition number (*4) on which this thread will transmit cells.
	immed[partition_number, (2*4)]

	; Stores base address of RBUF_TBUF_BASE for this thread.
	add(tbuf_element_base, RBUF_TBUF_BASE, 4096)

	; Stores base address of TBUF_VALIDATE for this thread.
	add(tbuf_validate_base, TBUF_VALIDATE, 512)

	.else

	; Stores MSF partition number (*4) on which this thread will transmit cells.
	immed[partition_number, (3*4)]

	; Stores base address of RBUF_TBUF_BASE for this thread.
	add(tbuf_element_base, RBUF_TBUF_BASE, 6144)

	; Stores base address of TBUF_VALIDATE for this thread.
	add(tbuf_validate_base, TBUF_VALIDATE, 768)

	.endif	

	#endif // FIRST_AAL5_TX_ME

	#else // (not defined ONE_AAL5_TX_ME)

	#ifdef FIRST_AAL5_TX_ME

	.if (ctx() == 0 || ctx() == 1 || ctx() == 2 || ctx() == 3)

	; Stores MSF partition number (*4) on which this thread will transmit cells.
	immed[partition_number, (0*4)]

	; Stores base address of RBUF_TBUF_BASE for this thread.
	add(tbuf_element_base, RBUF_TBUF_BASE, 0)

	; Stores base address of TBUF_VALIDATE for this thread.
	add(tbuf_validate_base, TBUF_VALIDATE, 0)

	.else 

	; Stores MSF partition number (*4) on which this thread will transmit cells.
	immed[partition_number, (1*4)]

	; Stores base address of RBUF_TBUF_BASE for this thread.
	add(tbuf_element_base, RBUF_TBUF_BASE, 2048)

	; Stores base address of TBUF_VALIDATE for this thread.
	add(tbuf_validate_base, TBUF_VALIDATE, 256)

	.endif

	#endif // FIRST_AAL5_TX_ME

	#ifdef SECOND_AAL5_TX_ME

	.if (ctx() == 0 || ctx() == 1 || ctx() == 2 || ctx() == 3)

	; Stores MSF partition number (*4) on which this thread will transmit cells.
	immed[partition_number, (2*4)]

	; Stores base address of RBUF_TBUF_BASE for this thread.
	add(tbuf_element_base, RBUF_TBUF_BASE, 4096)

	; Stores base address of TBUF_VALIDATE for this thread.
	add(tbuf_validate_base, TBUF_VALIDATE, 512)

	.else 

	; Stores MSF partition number (*4) on which this thread will transmit cells.
	immed[partition_number, (3*4)]

	; Stores base address of RBUF_TBUF_BASE for this thread.
	add(tbuf_element_base, RBUF_TBUF_BASE, 6144)

	; Stores base address of TBUF_VALIDATE for this thread.
	add(tbuf_validate_base, TBUF_VALIDATE, 768)

	.endif

	#endif // SECOND_AAL5_TX_ME

	#endif // ONE_AAL5_TX_ME

	#endif // ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4))


	#if (TX_PHY_MODE == SPHY_2_16) 

	#ifdef FIRST_AAL5_TX_ME

	.if (ctx() == 0 || ctx() == 1 || ctx() == 2 || ctx() == 3)

	; Stores MSF partition number (*2) on which this thread will transmit cells.
	immed[partition_number, (0*4)]

	; Stores base address of RBUF_TBUF_BASE for this thread.
	add(tbuf_element_base, RBUF_TBUF_BASE, 0)

	; Stores base address of TBUF_VALIDATE for this thread.
	add(tbuf_validate_base, TBUF_VALIDATE, 0)

	.else 

	; Stores MSF partition number (*4) on which this thread will transmit cells.
	immed[partition_number, (2*4)]

	; Stores base address of RBUF_TBUF_BASE for this thread.
	add(tbuf_element_base, RBUF_TBUF_BASE, 4096)

	; Stores base address of TBUF_VALIDATE for this thread.
	add(tbuf_validate_base, TBUF_VALIDATE, 512)

	.endif

	#endif // FIRST_AAL5_TX_ME

	#ifdef SECOND_AAL5_TX_ME

	.if (ctx() == 0 || ctx() == 1 || ctx() == 2 || ctx() == 3)

	; Stores MSF partition number (*4) on which this thread will transmit cells.
	immed[partition_number, (1*4)]

	; Stores base address of RBUF_TBUF_BASE for this thread.
	add(tbuf_element_base, RBUF_TBUF_BASE, 4096)

	; Stores base address of TBUF_VALIDATE for this thread.
	add(tbuf_validate_base, TBUF_VALIDATE, 512)

	.else 

	; Stores MSF partition number (*4) on which this thread will transmit cells.
	immed[partition_number, (3*4)]

	; Stores base address of RBUF_TBUF_BASE for this thread.
	add(tbuf_element_base, RBUF_TBUF_BASE, 6144)

	; Stores base address of TBUF_VALIDATE for this thread.
	add(tbuf_validate_base, TBUF_VALIDATE, 768)

	.endif

	#endif // SECOND_AAL5_TX_ME

	#endif // (TX_PHY_MODE == SPHY_2_16)

;-------------------------------------------------------------------
; Signal definitions.
;-------------------------------------------------------------------

	; Thread and ME signals

	; Holds the information needed to signal the next thread.
	.reg next_thread

	; Defines the next thread signal.
	.sig volatile prev_thread_done_sig

	#if (AAL5_TX_NUMBER_OF_FP_MEs > 1)

	; Holds the information needed to signal pipestage1 of the functional
	; pipeline.
	.reg start_pipestage1

	; Defines the pipestage1 signal.
	.sig volatile pipestage1_done_sig
	.addr pipestage1_done_sig			PIPESTAGE1_DONE_SIG_VAL

	; Holds the information needed to signal pipestage2 of the functional
	; pipeline.
	.reg start_pipestage2

	; Defines the pipestage2 signal.
	.sig volatile pipestage2_done_sig
	.addr pipestage2_done_sig			PIPESTAGE2_DONE_SIG_VAL

	#endif // (AAL5_TX_NUMBER_OF_FP_MEs > 1)

	; I/O signals.

	; These gpr's hold signals which we will ctx_arb[] on throughout
	; the macro.
	.reg sigmask_a, sigmask_b

	; Init sigmask_a with prev_thread_done_sig for use in pipestage 1.
	alu[sigmask_a, --, B, 1, <<(&prev_thread_done_sig)]

	; Init sigmask_b with prev_thread_done_sig for use in pipestage 1.
	alu[sigmask_b, --, B, 1, <<(&prev_thread_done_sig)]

	#ifdef FLOW_CONTROL
	; Write fire-n-forget to scratch for the scheduler to use.
	.sig sch_sig
	#endif

;-------------------------------------------------------------------
; Local memory initializations.
;-------------------------------------------------------------------

	; Define 16 locations in local memory that will serve as additional
	; gpr's.
	#define	TX_SEQUENCE_INDEX					0

	#define TXC_BASE_SRAM_INDEX					3
	#define AAL5_TX_AVAIL_TBUF_ELEMENT_INDEX	4
	#define AAL5_TX_CELLS_TX_INDEX				5

	#define TBUF_ADDR_MASK_INDEX				6
	#define VCQ_MASK_INDEX						7

	; Used to signal meta data read from QM scratch ring.
	.sig scratch_rd_sig

	; Used to signal meta data read from SRAM.
	.sig meta_rd_sig

	; Used to signal TXC read, TXC write from and to SRAM.
	.sig txc_io_sig

	; Used to signal payload read from DRAM.
	.sig curr_buf_rd_sig
	.sig next_buf_rd_sig

	; Used to signal Tx_Sequence_x CSR read.
	.sig tx_sequence_sig

	; Used to signal next available TBUF element using scratch.
	.sig avail_tbuf_element_sig

	; Used to signal 52-byte cell write to TBUF.
	.sig cell_wr_sig

	; Write to 16 local memory locations. These locations serve as
	; additional gpr's.
	.begin
	.reg const_lm
	move(const_lm, CONST_BASE_LM)

	; Set the signal base LM addr in CSR's.
	local_csr_wr[active_lm_addr_1, const_lm]
		nop
		nop
		nop

	.if (ctx() == 0)

	; Stores address of MSF CSR TX_SEQUENCE_0.
	move(*l$index1[TX_SEQUENCE_INDEX], TX_SEQUENCE_0)

    .begin
    .reg tmp_reg
        immed32[tmp_reg, AAL5_TX_CTX_SRAM_BASE]
    	; Stores the SRAM base address for TXC's.
        immed32[tmp_reg, AAL5_TX_CTX_SRAM_BASE]
        alu[*l$index1[TXC_BASE_SRAM_INDEX], --, b, tmp_reg]
    
    	; Stores the next available TBUF element number.
        immed32[tmp_reg, AAL5_TX_AVAIL_TBUF_ELEMENT_SCRATCH_BASE]
        alu[*l$index1[AAL5_TX_AVAIL_TBUF_ELEMENT_INDEX], --, b, tmp_reg]

	    ; Stores the scratch base address of cells sent info used by scheduler.
        immed32[tmp_reg, AAL5_TX_CELLS_TX_SCRATCH_BASE]
        alu[*l$index1[AAL5_TX_CELLS_TX_INDEX], --, b, tmp_reg]
    .end


	#if (TX_PHY_MODE == SPHY_1_32)

	; Mask that limits the TBUF address space used to
	; transmit a cell to 8KB.
	move(*l$index1[TBUF_ADDR_MASK_INDEX], 0x1FFF)

	#endif // (TX_PHY_MODE == SPHY_1_32)

	#if ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4) || (TX_PHY_MODE == SPHY_4_8_SHARED))
	
	; Mask that limits the TBUF address space used to
	; transmit a cell to 2KB.
	move(*l$index1[TBUF_ADDR_MASK_INDEX], 0x7FF)

	#endif // ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4) || (TX_PHY_MODE == SPHY_4_8_SHARED))

	#if (TX_PHY_MODE == SPHY_2_16)

	; Mask that limits the TBUF address space used to
	; transmit a cell to 4KB.
	move(*l$index1[TBUF_ADDR_MASK_INDEX], 0xFFF)

	#endif //(TX_PHY_MODE == SPHY_2_16)

	; Used to mask incoming queue number from QM
	move(*l$index1[VCQ_MASK_INDEX], VCQ_MASK)

	.endif // (ctx() == 0)

	.end // const_lm
	// SCR6027: Added OAM related register definitions
	#ifdef ATM_OAM_ENABLED

	.reg	cur_bedc
	.reg	atm_cell_copy_header

	; Store the delayed cell with a 1 LW offset such that loading
	; from DRAM to MSF can be done at QW boundary. We set the prepend
	; length to 4 always and these extra bytes will be ignored. To 
	; facilitate this an addittional transfer register $$prepend_header
	; is allocated, though it is never written into

	.reg  write $$prepend_header, $$atm_cell_copy0, $$atm_cell_copy1, $$atm_cell_copy2,\
				$$atm_cell_copy3, $$atm_cell_copy4, $$atm_cell_copy5, $$atm_cell_copy6,\
				$$atm_cell_copy7, $$atm_cell_copy8, $$atm_cell_copy9, $$atm_cell_copy10,\
				$$atm_cell_copy11, $$atm_cell_copy12, $$atm_cell_copy13, $$atm_cell_copy14
	.xfer_order $$prepend_header, $$atm_cell_copy0, $$atm_cell_copy1, $$atm_cell_copy2,\
				$$atm_cell_copy3, $$atm_cell_copy4, $$atm_cell_copy5, $$atm_cell_copy6,\
				$$atm_cell_copy7, $$atm_cell_copy8, $$atm_cell_copy9, $$atm_cell_copy10,\
				$$atm_cell_copy11, $$atm_cell_copy12, $$atm_cell_copy13, $$atm_cell_copy14
	.set		$$prepend_header, $$atm_cell_copy0, $$atm_cell_copy1, $$atm_cell_copy2,\
				$$atm_cell_copy3, $$atm_cell_copy4, $$atm_cell_copy5, $$atm_cell_copy6,\
				$$atm_cell_copy7, $$atm_cell_copy8, $$atm_cell_copy9, $$atm_cell_copy10,\
				$$atm_cell_copy11, $$atm_cell_copy12, $$atm_cell_copy13, $$atm_cell_copy14
	.set		cur_bedc, atm_cell_copy_header
	move($$prepend_header, 0xDeadFace)
	move($$atm_cell_copy14, 0xFaceDead)
	#endif

;-------------------------------------------------------------------
; Debug definitions.
;-------------------------------------------------------------------

#ifdef _DEBUG_
	.reg @null_counter
	.if(ctx()==0)
		immed[@null_counter, 0]
	.endif
#endif

#endif // _AAL5_TX_REGS_UC_