#ifndef	_AAL5_TX_UTIL_FP_ME_UC_
#define _AAL5_TX_UTIL_FP_ME_UC_

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2002 By Intel Corporation.  All rights reserved.
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

/*
 *      File Name: aal5_tx_util_fp_me.uc                                         
 *                                                                   
 *      Description: This file contains macros that implement each
 *                   processing stage of the functional pipeline me
 *                   implementation.
 *                                                                   
 *
 */

#include <aal5_tx.h>
#include <aal5_tx_regs.uc>
#include <aal5_tx_util_common.uc>
#include <aal5_tx_crc.uc>
#include <xbuf.uc>

//------------------------------------------------------------------
// Pipestage 1 - phase 1 : Read dequeue request.
//------------------------------------------------------------------
// _aal5_tx_dl_source()
//
//    Description: Fetch the dequeue request from scratch ring.
//
//    Parameters:
//        Inputs:
//        scratch_rd_sig:
//            Signal used to indicate completion of scratch[get].
//
//------------------------------------------------------------------
#macro _aal5_tx_dl_source(scratch_rd_sig)
.begin

	; Read vcq and buffer handle from scratch ring.
	scratch[get, $vcq, ring, 0, 2], sig_done[scratch_rd_sig]

.end
#endm


//------------------------------------------------------------------
// Pipestage 1 - phase 2
//------------------------------------------------------------------
// _aal5_tx_check_null_deq_and_setup_cam_data()
//
//    Description: Check if the dequeue request is null.
//    If it is, follow the null_deq path.
//    If not, setup cam data to start pipestage 1.
//
//    Parameters:
//        Inputs:
//        NON_NULL_DEQ_LABEL:
//            Label to branch to if dequeue request is non-null.
//        NULL_DEQ_LABEL:
//            Label to branch to if dequeue request is null.
//        OAM_DEQ_LABEL:
//            Label to branch to OAM processing
//
//------------------------------------------------------------------
#macro _aal5_tx_check_null_deq_and_setup_cam_data( \
			NON_NULL_DEQ_LABEL, NULL_DEQ_LABEL, OAM_DEQ_LABEL)
.begin
.reg temp

	signal_ctx(next_thread)

	; if(dl_source() == null), branch to null_path#.
	alu[vcq, *l$index1[VCQ_MASK_INDEX], AND ,$vcq]
	beq[null_path#]

 	; Issue read for next available TBUF element from scratch.
	; This TBUF element will be used later to write out the cell.
	scratch[test_and_incr, $avail_tbuf_element, \
		*l$index1[AAL5_TX_AVAIL_TBUF_ELEMENT_INDEX], partition_number], \
		sig_done[avail_tbuf_element_sig]

	; Compute SRAM addr of the new TXC.
	alu[txc_sram_offset, --, B, vcq, <<6]

	; Store Port# and CLP bit in LM
	move[temp, PORT_MASK]
	alu[txc_port_and_clp, temp, AND, $vcq, >>18]


	; Get the cam_result using vcq and set corres. TXCs' LM address.		
	; Needs 3 nops or eqv. instructions before *l$index0 is used.
	cam_lookup[cam_result, vcq], lm_addr0[0]

	; Current thread==0? If no, branch to not_thread_0#.
	br!=ctx[0, not_thread_0#],defer[2]

	; Get the cam_entry.
	alu[cam_entry, CAM_ENTRY_MASK, AND, cam_result, >>3]

	; Update cam_entry with vcq.	
	cam_write[cam_entry, vcq, 1]

	ctx_arb[prev_thread_done_sig, pipestage1_done_sig], defer[2], \
			br[NON_NULL_DEQ_LABEL]

	; Convert $buf_hdl_lw from lw's to byte address.
	ld_field_w_clr[buf_hdl, 0111, $buf_hdl_lw]
	alu[buf_hdl, --, B, buf_hdl, <<2]

not_thread_0#:

	ctx_arb[prev_thread_done_sig], defer[2], \
			br[NON_NULL_DEQ_LABEL]

	; Convert $buf_hdl_lw from lw's to byte address.
	ld_field_w_clr[buf_hdl, 0111, $buf_hdl_lw]
	alu[buf_hdl, --, B, buf_hdl, <<2]

null_path#:

	; current thread==0? If no, branch to not_thread_0#.
	br!=ctx[0, null_not_thread_0#]

	ctx_arb[prev_thread_done_sig, pipestage1_done_sig], \
			br[NULL_DEQ_LABEL]

null_not_thread_0#:

	ctx_arb[prev_thread_done_sig], \
			br[NULL_DEQ_LABEL]

.end
#endm


//-----------------------------------------------------------------
// Pipestage 1 - phase 3
//-----------------------------------------------------------------
// _aal5_tx_fetch_txc_in_lm()
//
//    Description: Fetch TXC1 in local memory.
//
//    Parameters: RAW_CELL_TX_LABEL -jump to raw mode processing
//
//------------------------------------------------------------------
//SCR6027: Added parameter, needed for RAW Mode
#macro _aal5_tx_fetch_txc_in_lm(RAW_CELL_TX_LABEL)
.begin

;-------------------------------------------------------------------
; Start of critical section.
;-------------------------------------------------------------------

	signal_ctx(next_thread)

#ifdef _DEBUG_
	.begin
	.reg asv_tmp
	alu[asv_tmp, 0xFF, AND, $buf_hdl_lw]
	.if(asv_tmp == 0xff)
	nop
	.endif

	alu[asv_tmp, 0xFF, AND, $buf_hdl_lw]
	.if(asv_tmp == 0x5a)
	nop
	.endif
	.end
#endif

	; Is this is a new packet? If no, branch to non_sop#.
	br_bclr[$buf_hdl_lw, SOP_BIT, non_sop#]

sop#:

	xbuf_alloc($meta, 4, read)

	; Read sop buffer meta data from SRAM.
	sram[read, $meta[0], buf_hdl, 0, 4],\
		sig_done[meta_rd_sig]

	ctx_arb[meta_rd_sig, prev_thread_done_sig], defer[1]

	 // Unfilled defer slot.

	; lm:ref_cnt[cam_entry]++
	alu[*l$index0[9], *l$index0[9], +, 1]

	; Using sop meta data, initialize TXC1.

	; lm:txc_bytes_rem_in_curr_buf = $meta_buf_size
	ld_field_w_clr[*l$index0[0], 0011, $meta[1], >>16]
	; lm:txc_outport = TM 4.1 output port
	ld_field[*l$index0[0], 1100, txc_port_and_clp, <<15]

	; lm:txc_curr_payload_addr =
	; dl_buf_get_data_from_meta($buf_hdl:buf_hdl) + $meta_offset
	dl_buf_get_data_from_meta(*l$index0[1], buf_hdl)

	.reg buf_offset
	ld_field_w_clr[buf_offset, 0011, $meta[1]]
	alu[*l$index0[1], *l$index0[1], +, buf_offset]

	; lm:txc_curr_buf_hdl = buf_hdl
	; lm:txc_bytes_done_in_next_buf = 0
	ld_field_w_clr[*l$index0[2], 0111, buf_hdl]

	; lm:txc_new_curr_buf_hdl = 1
	alu[*l$index0[2], *l$index0[2], OR, 1, <<24]

	; lm:txc_bytes_done_in_packet = 0
	immed[*l$index0[4], 0]

	; lm:txc_end_of_payload = 0
	; lm:txc_freelist = $meta_freelist
	.reg eop_and_freelist
	alu[eop_and_freelist, 0xF, AND, $meta[2], >>12]
	alu[*l$index0[5], --, B, eop_and_freelist, <<24]

	; lm:txc_packet_size = $meta_packet_size
	ld_field[*l$index0[6], 1100, $meta[2]]

	#ifdef COUNTERS
	_aal5_inc_counter(PKTS_PER_VCQ, vcq)
	#endif // COUNTERS

	br[end#], defer[2]

	; lm:txc_next_buf_hdl = bits 23:0 of $meta_next_buf_hdl
	alu[*l$index0[3], $meta[0], AND~, 0xFF, <<24]

	; Convert lm:txc_next_buf_hdl from lw's to byte address.
	alu[*l$index0[3], --, B, *l$index0[3], <<2]

	xbuf_free($meta)

non_sop#:

 	; vcq a hit in CAM? If yes, branch to non_sop_hit#.
	br_bset[cam_result, 7, non_sop_hit#]

non_sop_miss#:

	xbuf_alloc($txc, 7, read)

	; Read TXC1 from SRAM.
	sram[read, $txc[0], *l$index1[TXC_BASE_SRAM_INDEX], \
		txc_sram_offset, 7], sig_done[txc_io_sig]

	ctx_arb[txc_io_sig, prev_thread_done_sig], defer[1]

	 // Unfilled defer slot.

	; lm:ref_cnt[cam_entry]++
	alu[*l$index0[9], *l$index0[9], +, 1]

	; Copy TXC1 from s-xfer-read to LM.
	alu[*l$index0[0], --, B, $txc[0]]	
	alu[*l$index0[1], --, B, $txc[1]]	
	alu[*l$index0[2], --, B, $txc[2]]	
	alu[*l$index0[3], --, B, $txc[3]]	
	alu[*l$index0[4], --, B, $txc[4]]	

	br[end#], defer[2]

	alu[*l$index0[5], --, B, $txc[5]]	
	alu[*l$index0[6], --, B, $txc[6]]	

	xbuf_free($txc)

non_sop_hit#:

	ctx_arb[prev_thread_done_sig], defer[1]

	 // Unfilled defer slot.

	; lm:ref_cnt[cam_entry]++
	alu[*l$index0[9], *l$index0[9], +, 1]

end#:

.end
#endm


//------------------------------------------------------------------
// Pipestage 1 - phase 4 in aal5_tx_util_common.uc
//------------------------------------------------------------------


//------------------------------------------------------------------
// Pipestage 1 - phase 5
//------------------------------------------------------------------
// _aal5_tx_prepare_cell()
//
//    Description: Prepare 48-byte cell in local memory. If cell
//                 payload is 40 or 48 bytes skip this step as an
//                 optimization.
//                 Flush TXC1 from LM to SRAM.
//
//    Parameters: None.
//
//------------------------------------------------------------------
#macro _aal5_tx_prepare_cell()
.begin

	signal_ctx(next_thread)

	; if (prefetched_buffer == 1)
	; update TXC with prefetched buffer information.
	br_bclr[prefetched_buffer, 0, prefetch_udpate_done#]

	 // Unfilled defer slot.

	; lm:txc_next_buf_size = $prefetch_meta_buf_size
	ld_field[*l$index0[4], 1100, $prefetch_meta[1]]

	.reg txc_next_next_buf_hdl

	; lm:txc_next_next_buf_hdl = bits 23:0 of $prefetch_meta_next_buf_hdl
	alu[txc_next_next_buf_hdl, $prefetch_meta[0], AND~, 0xFF, <<24]

	; Convert lm:txc_next_next_buf_hdl from lw's to byte address.
	ld_field[*l$index0[5], 0111, txc_next_next_buf_hdl, <<2]

prefetch_udpate_done#:

	; Set only by thread 7 during msf[read] of Tx_Sequence.
	.set $cells_tx_by_fabric

	xbuf_alloc($txc, 7, write)

	; lm:ref_cnt[cam_entry]--
	alu[*l$index0[9], *l$index0[9], -, 1]

	; if (lm:ref_cnt[vcq] == 0), flush TXC1 to SRAM.
	bne[end_of_critical_section#]

	 // Unfilled defer slot.

	; Copy TXC1 from LM to s-xfer-write.
	alu[$txc[0], --, B, *l$index0[0]]	
	alu[$txc[1], --, B, *l$index0[1]]	
	alu[$txc[2], --, B, *l$index0[2]]	
	alu[$txc[3], --, B, *l$index0[3]]	
	alu[$txc[4], --, B, *l$index0[4]]	
	alu[$txc[5], --, B, *l$index0[5]]	
	alu[$txc[6], --, B, *l$index0[6]]	

	; Add txc_io_sig (txc1_wr) to sigmask_a.
	alu[sigmask_a, sigmask_a, OR, 1, <<(&txc_io_sig)]

	; Write TXC1 to SRAM.
	sram[write, $txc[0], *l$index1[TXC_BASE_SRAM_INDEX], \
		txc_sram_offset, 7], sig_done[txc_io_sig]

	; Current thread==7? If no, branch to end_of_critical_section#.
	br!=ctx[7, end_of_critical_section#]
		
	; Signal thread 0 of the next me.
	alu[--, --, B, start_pipestage1]
	cap[fast_wr, ALU, interthread_sig]	

;-------------------------------------------------------------------
; End of critical section.
;-------------------------------------------------------------------

	; Clear CAM for use in pipestage 2.
	cam_clear

	; Add tx_seq_rd_sig_ to sigmask_a.
	alu[sigmask_a, sigmask_a, OR, 1, <<(&tx_sequence_sig)]

	; Read number of cells actually transmitted
	; on the pins from this partition.
	; 0 <= $cells_tx_by_fabric <= 255.
	msf[read, $cells_tx_by_fabric, *l$index1[TX_SEQUENCE_INDEX], \
		partition_number, 1], sig_done[tx_sequence_sig]

end_of_critical_section#:

	; Swap out to allow thread 7 to signal the next me asap.
	ctx_arb[--], defer[2]

	; ctx_arb on sigmask_b.
	local_csr_wr[active_ctx_wakeup_events, sigmask_b]

	; Reset prefetched_buffer.
	immed[prefetched_buffer, 0]

	.io_completed next_buf_rd_sig
	.io_completed curr_buf_rd_sig
	.io_completed prev_thread_done_sig	

	signal_ctx(next_thread)

	#ifdef COUNTERS
	_aal5_inc_counter(CELLS_PER_VCQ, vcq)
	#endif

	; If the cell payload in current buffer is 48 bytes,
	; do not prepare the cell in local memory. (Optimization).
	alu[--, cell_payload_in_curr_buf, -, 48]
	beq[cell_done#]
 
	; If the cell payload in current buffer is 40 bytes and
	; this is the last cell of the packet, do not prepare the
	; cell in local memory. (Optimization).
	alu[--, cell_payload_in_curr_buf, -, 40]
	bne[prepare_cell_in_lm#]
	alu[--, cell_payload_in_next_buf, -, 0]
	beq[cell_done#]

prepare_cell_in_lm#:

	; Prepares a 48 byte cell in a thread specific location in LM.
	; Pipestage 2 will compute crc on this 48 byte cell and transmit it
	; out through a TBUF element.
	_aal5_tx_prepare_cell_in_lm(curr_payload_addr_offset, 
		cell_payload_in_curr_buf, cell_payload_in_next_buf,
		payload_reg_base_addr_lw,
		next_payload_reg_base_lw)

cell_done#:

	; Current thread==0? If no, branch to wait_for_signals#.
	br!=ctx[0, wait_for_signals#], defer[1]

	 // Unfilled defer slot.

	; Update cam_entry with vcq.	
	cam_write[cam_entry, vcq, 1]

	; Add pipestage2_done_sig to sigmask_a.
	alu[sigmask_a, sigmask_a, OR, 1, <<(&pipestage2_done_sig)]

wait_for_signals#:

	; ctx_arb on sigmask_a.
	ctx_arb[--], defer[2]

	local_csr_wr[active_ctx_wakeup_events, sigmask_a]

	; Set txc_sram_offset for pipestage 2.
	; TXC1 is 7 lw's, so in SRAM TXC2 will start 28 bytes after
	; start of TXC1.
	alu[txc_sram_offset, txc_sram_offset, +, 28]

	.io_completed txc_io_sig // Completed txc1_wr.
	xbuf_free($txc)

	.io_completed tx_sequence_sig
	.io_completed pipestage2_done_sig
	.io_completed prev_thread_done_sig	

.end
#endm


//------------------------------------------------------------------
// Pipestage 2 - phase 1
//------------------------------------------------------------------
// _aal5_tx_fetch_txc2_in_lm()
//
//    Description: Fetch TXC2 in local memory.
//
//    Parameters: None.
//
//------------------------------------------------------------------
#macro _aal5_tx_fetch_txc2_in_lm()
.begin

;-------------------------------------------------------------------
; Start of critical section.
;-------------------------------------------------------------------

	signal_ctx(next_thread)

	; Is this is a new packet? If no, branch to non_sop#.
	br_bclr[$buf_hdl_lw, SOP_BIT, non_sop#]

sop#:

	.begin

	.reg read $txc

	; Read outgoing cell header from SRAM.
	sram[read, $txc, *l$index1[TXC_BASE_SRAM_INDEX], \
			txc_sram_offset, 1], sig_done[txc_io_sig]

	ctx_arb[avail_tbuf_element_sig, txc_io_sig, prev_thread_done_sig],\
		defer[2]

	; lm:ref_cnt[cam_entry]++
	alu[*l$index0[9], *l$index0[9], +, 1]

	; Update cam_entry with vcq.	
	cam_write[cam_entry, vcq, 1]


	; Add outgoing CLP to cell header.
	alu[*l$index0[7], 0x1, AND, txc_port_and_clp]

	; Copy TXC2 from s-xfer-read to LM.
	alu[*l$index0[7], *l$index0[7], OR, $txc]

	; Initialize TXC2:crc.
	; lm:txc_crc = INIT_CRC_REMAINDER
	immed[*l$index0[8], INIT_CRC_REMAINDER]

	br[txc2_in_lm#]

	.end

non_sop#:

 	; vcq a hit in CAM? If yes, branch to non_sop_hit#.
	br_bset[cam_result, 7, non_sop_hit#]

non_sop_miss#:

	.begin

	xbuf_alloc($txc, 2, read)

	; Read TXC2 from SRAM.
	sram[read, $txc[0], *l$index1[TXC_BASE_SRAM_INDEX], \
			txc_sram_offset, 2], sig_done[txc_io_sig]

	ctx_arb[avail_tbuf_element_sig, txc_io_sig, prev_thread_done_sig],\
		defer[2]

	; lm:ref_cnt[cam_entry]++
	alu[*l$index0[9], *l$index0[9], +, 1]

	; Update cam_entry with vcq.	
	cam_write[cam_entry, vcq, 1]

	; Copy TXC2 from s-xfer-read to LM.
	br[txc2_in_lm#], defer[2]

	alu[*l$index0[7], Ox0FFFFFF0, AND, $txc[0], <<4]

	alu[*l$index0[8], --, B, $txc[1]]

	xbuf_free($txc)

	.end

non_sop_hit#:

	ctx_arb[avail_tbuf_element_sig, prev_thread_done_sig], defer[2]

	; lm:ref_cnt[cam_entry]++
	alu[*l$index0[9], *l$index0[9], +, 1]

	; Update cam_entry with vcq.	
	cam_write[cam_entry, vcq, 1]

txc2_in_lm#:

.end
#endm


//------------------------------------------------------------------
// Pipestage 2 - phase 2
//------------------------------------------------------------------
// _aal5_tx_compute_crc_and_transmit_cell()
//
//    Description: Compute CRC on cell, update TXC2 and
//                 flush TXC2 from LM to SRAM and write
//                 52-byte cell to TBUF.
//                 As an optimization, read the next dequeue request
//                 from the scratch ring.
//
//    Parameters: None.
//
//------------------------------------------------------------------
#macro _aal5_tx_compute_crc_and_transmit_cell()
.begin

	signal_ctx(next_thread)

	; Read next dequeue request from scratch (Optimization).
	_aal5_tx_dl_source(scratch_rd_sig)

	; Free current and next buffers if marked.

	; if (free_curr_buf:31 == 1), free current buffer.
	br_bclr[free_curr_buf, 31, check_free_next_buf#], defer[3]

	; Init sigmask_a with prev_thread_done_sig.
	alu[sigmask_a, --, B, 1, <<(&prev_thread_done_sig)]

	; Add cell_wr_sig to sigmask_a.
	alu[sigmask_a, sigmask_a, OR, 1, <<(&cell_wr_sig)]

	; Load CRC_Remainder CSR with lm:txc_crc
	local_csr_wr[CRC_Remainder, *l$index0[8]]

#ifdef _DEBUG_
	.begin
	.reg asv_tmp
	alu[asv_tmp, 0xFF, AND, free_curr_buf]
	.if(asv_tmp == 0xff)
	nop
	.endif

	alu[asv_tmp, 0xFF, AND, free_curr_buf]
	.if(asv_tmp == 0x5a)
	nop
	.endif
	.end
#endif

	; Freeing current buffer.
	dl_buf_free(free_curr_buf, BUF_FREE_LIST0)

	; Reset free_curr_buf.
	immed[free_curr_buf, 0]

check_free_next_buf#:

	.reg tbuf_addr_plus_4

	; if (free_next_buf:31 == 1), free next buffer.
	br_bclr[free_next_buf, 31, crc#], defer[3]

 	; Compute the TBUF address of the available TBUF element.
	alu[tbuf_addr, *l$index1[TBUF_ADDR_MASK_INDEX], AND, \
			$avail_tbuf_element, <<TBUF_ELE_SIZE_LOG2]

	; Add 4 to the tbuf_addr. This new address will be used
	; when writing the 52-byte cell to TBUF. We leave the
	; first 4 bytes unused. These are the 4 unused bytes of the
	; prepend header that we define when validating the TBUF.
	; We need to do this since according to the TCW format,
	; the prepend length must be atleast 8 bytes (1 quadword)
	; and our prepend length is only 4 bytes (cell header).
	alu[tbuf_addr_plus_4, tbuf_addr, +, 4]

	; Compute the cells tx by this code (0..TBUF_ELE_NUM)
	alu[cells_tx_by_code, TX_SEQUENCE_MASK, \
			AND, $avail_tbuf_element]


#ifdef _DEBUG_
	.begin
	.reg asv_tmp
	alu[asv_tmp, 0xFF, AND, free_next_buf]
	.if(asv_tmp == 0xff)
	nop
	.endif

	alu[asv_tmp, 0xFF, AND, free_next_buf]
	.if(asv_tmp == 0x5a)
	nop
	.endif
	.end
#endif

	; Freeing next buffer.
	dl_buf_free(free_next_buf, BUF_FREE_LIST0)

	; Reset free_next_buf.
	immed[free_next_buf, 0]

crc#:

	xbuf_alloc($cell_lw, WRD_OFFSET_14, write)
    .set $cell_lw/**/[WRD_OFFSET_13]

	; If the cell payload in current buffer is 48 bytes compute
	; CRC on cell in d-xfer-rd regs. (Optimization).
	; Copy the 52-byte cell (4-byte cell header) into s-xfer-wr regs. 
	; Also store CRC residue in TXC.
	alu[--, cell_payload_in_curr_buf, -, 48]
	bne[check_40_bytes#], defer[1]

	; Set tcw_payload_offset to 0 by default.
	; This is used when validating the TBUF element containing the
	; 52-byte cell. Macro _aal5_tx_compute_and_store_crc() changes
	; the value of tcw_payload_offset if needed.
	immed[tcw_payload_offset, 0]

	_aal5_tx_compute_and_store_crc(FORTY_EIGHT, curr_payload_addr_offset,
		$cell_lw, crc_done#)

check_40_bytes#:

	; If the cell payload in current buffer is 40 bytes and this is the last
	; cell of the packet, compute CRC on cell in d-xfer-rd regs. (Optimization).
	; Copy the 52-byte cell (4-byte cell header) into s-xfer-wr regs. 
	; Also store CRC residue in TXC.
	alu[--, cell_payload_in_curr_buf, -, 40]
	bne[crc_in_lm#]
	alu[--, cell_payload_in_next_buf, -, 0]
	bne[crc_in_lm#]

	_aal5_tx_compute_and_store_crc(FORTY, curr_payload_addr_offset,
		$cell_lw, crc_done#)

crc_in_lm#:

	; Computes CRC on cell in local memory.
	; Also copies the 52 byte cell into s-xfer-wr regs. 
	_aal5_tx_compute_crc(cell_payload, $cell_lw)

	; Stores CRC residue in TXC.
    _aal5_tx_store_crc(cell_payload, $cell_lw/**/[WRD_OFFSET_12])

crc_done#:

	.use	$$payload[0]  $$payload[1]  $$payload[2]  $$payload[3]  \
			$$payload[4]  $$payload[5]  $$payload[6]  $$payload[7]  \
			$$payload[8]  $$payload[9]  $$payload[10] $$payload[11] \
			$$payload[12] $$payload[13] $$payload[14] $$payload[15]


	#ifdef COUNTERS
	_aal5_inc_counter(CELLS_PER_OUTPORT, outport)
	#endif

	xbuf_alloc($txc, 1, write)

	; lm:ref_cnt[cam_entry]--
	alu[*l$index0[9], *l$index0[9], -, 1]

	; if (lm:ref_cnt[vcq] == 0), flush TXC2 to SRAM.
	bne[transmit_cell#]

	// Unfilled defer slot.

	; Set txc_sram_offset.
	; Advance txc_sram_offset by 4 since we will only be
	; writing out the CRC field.
	alu[txc_sram_offset, txc_sram_offset, +, 4]

	; Copy TXC2 from LM to s-xfer-write.
	; We only need to write back the CRC field since outgoing
	; cell header is not changed by this code.
	alu[$txc[0], --, B, *l$index0[8]]	

	; Add txc_io_sig (for txc2_wr) to sigmask_a.
	alu[sigmask_a, sigmask_a, OR, 1, <<(&txc_io_sig)]

	; Write TXC2 to SRAM.
	sram[write, $txc[0], *l$index1[TXC_BASE_SRAM_INDEX], \
		txc_sram_offset, 1], sig_done[txc_io_sig]

	; Current thread==7? If no, branch to transmit_cell#.
	br!=ctx[7, transmit_cell#]

	; Signal thread 0 of the next me.
	alu[--, --, B, start_pipestage2]
	cap[fast_wr, ALU, interthread_sig]	

;-------------------------------------------------------------------
; End of critical section.
;-------------------------------------------------------------------

	.begin
	.reg cells_pending, cells_tx_by_fabric_masked

	; Clear CAM for use in pipestage 1.
	cam_clear

	// Mask-out the highest bit of the sequence word (sequence number is set on lowest 8 bits)
	alu[cells_tx_by_fabric_masked, $cells_tx_by_fabric, AND, TX_SEQUENCE_MASK]

	// Check if we have a wrap-round case for the cells transmit
	.if (cells_tx_by_code < cells_tx_by_fabric_masked)
		.begin
			.reg cells_tx_by_code_corrected

			alu[cells_tx_by_code_corrected, cells_tx_by_code, +, TBUF_ELE_NUM]
			alu[cells_pending, --, B, cells_tx_by_fabric_masked]

			alu[cells_pending, cells_tx_by_code_corrected, -, cells_pending]			
		.end
	.else

		alu[cells_pending, cells_tx_by_code, -, cells_tx_by_fabric_masked]

	.endif

#ifdef FLOW_CONTROL
	; Write fire-n-forget $cells_tx_by_fabric to scratch for the
	; scheduler to use.

	.io_completed sch_sig

	; Copy $cells_tx_by_fabric to write xfer register.
	alu[$cells_tx_by_fabric, TX_SEQUENCE_MASK, AND, $cells_tx_by_fabric]

	scratch[write, $cells_tx_by_fabric, *l$index1[AAL5_TX_CELLS_TX_INDEX], \
		partition_number, 1], sig_done[sch_sig]


#endif // FLOW_CONTROL
	; Check if there is a TBUF element available
	; for the next thread.
	; 'cells_tx_by_code' gives us a measure of how
	; many TBUF elements this code has transmitted.
	; If the difference between 'cells_tx_by_code'
	; and '$cells_tx_by_fabric' exceeds a certain
	; threshold, stall until the difference falls
	; below the threshold.

	alu[--, cells_pending, -, TBUF_THRESHOLD]	
	.end // cells_pending
	bge[tbuf_full#]	

transmit_cell#:

#ifdef FPGA_HEADER
	alu[$cell_lw[0], 0x03, ~AND, txc_port_and_clp, <<1]

	; Set ref cnt to 14 (to write 15 lw's) using indirect format.
	; Also set the reference count override bit (bit 25).
	alu[--, --, B, (0x10 + WRD_OFFSET_13), <<21]
	; Write 4 byte FPGA header and 52 byte cell from s-xfer-out regs to TBUF.
	msf[write, $cell_lw[0], tbuf_element_base, tbuf_addr, max_/**/WRD_OFFSET_14], \
		indirect_ref, sig_done[cell_wr_sig]

#else

	; Set ref cnt to 13 (to write 14 lw's) using indirect format.
	; Also set the reference count override bit (bit 25).
	alu[--, --, B, (0x10 + WRD_OFFSET_13), <<21]
	; Write 52 byte cell from s-xfer-out regs to TBUF.
	msf[write, $cell_lw[0], tbuf_element_base, tbuf_addr_plus_4, max_/**/WRD_OFFSET_14], \
		indirect_ref, sig_done[cell_wr_sig]

#endif

	; ctx_arb on sigmask_a.
	ctx_arb[--], defer[2], br[transmit_cell_done#]

	local_csr_wr[active_ctx_wakeup_events, sigmask_a]

	; Set tbuf_addr to be used during cell validation.
	alu[tbuf_addr, --, B, tbuf_addr, >>TCW_ADDR_SHIFT]	


tbuf_full#:
	; Stall inside tbuf_full() until more TBUF elements
	; are available. On exit, branch back to transmit_cell#.
	_aal5_tx_tbuf_full()
	br[transmit_cell#]

transmit_cell_done#:
	.io_completed txc_io_sig // Completed txc2_wr
	xbuf_free($txc)

	.io_completed cell_wr_sig
	xbuf_free($cell_lw)

	.io_completed prev_thread_done_sig		

.end
#endm


//------------------------------------------------------------------
// Pipestage 2 - phase 3
//------------------------------------------------------------------
// _aal5_tx_validate_tbuf()
//
//    Description: Validate TBUF element.
//
//    Parameters: None.
//
//------------------------------------------------------------------
#macro _aal5_tx_validate_tbuf()

	; Used to signal TCW write for the written TBUF element.
	.sig tcw_wr_sig
	.reg tmp_port tmp_seop

	signal_ctx(next_thread)

	xbuf_alloc($tcw, 2, write)

	.reg tcw_const
	; Set payload length to 48 bytes (cell payload),
#ifdef FPGA_HEADER
	; prepend offset to 0 and prepend length to 8 bytes (FPGA & cell header).
	immed[tcw_const, ((48 << 8) | (0 << 5) | (8 << 0))]
#else
	; prepend offset to 4 and prepend length to 4 bytes (cell header).
	immed[tcw_const, ((48 << 8) | (4 << 5) | (4 << 0))]
#endif

#ifdef IXP2800
	alu[tmp_seop, --, B, 0x3, <<8]	; setting SoP and EoP
	; Set TCW0. Payload offset has been set and left shifted by 13.
#define_eval INDIRECT_SHIFT	(AAL5_FPGA_PORT_BITS + 1) // +1 bacause we don't need CLP bit
	alu[tmp_base, --, B, txc_port_and_clp, >>INDIRECT_SHIFT]
#undef 	INDIRECT_SHIFT
	alu[tmp_base, @spi4_addr, +, tmp_base]
	alu[tmp_port, tmp_seop, OR, tmp_base]


	alu[tcw_const, tmp_port, OR, tcw_const, <<16]
	alu[$tcw[0], tcw_payload_offset, OR, tcw_const]
#else
	; Set TCW0. Payload offset has been set and left shifted by 13.
	alu[$tcw[0], tcw_payload_offset, OR, tcw_const, <<16]
#endif

	; TCW1 is reserved.
	.set $tcw[1]

	; Write TCW to validate transmit.
	msf[write, $tcw[0], tbuf_validate_base, tbuf_addr, 2], \
		sig_done[tcw_wr_sig]

	ctx_arb[scratch_rd_sig, tcw_wr_sig, prev_thread_done_sig], defer[2]

	; Init sigmask_a with prev_thread_done_sig for use in pipestage 1.
	alu[sigmask_a, --, B, 1, <<(&prev_thread_done_sig)]

	; Init sigmask_b with prev_thread_done_sig for use in pipestage 1.
	alu[sigmask_b, --, B, 1, <<(&prev_thread_done_sig)]

	xbuf_free($tcw)

#endm


//------------------------------------------------------------------
// _aal5_tx_null_deq()
//
//    Description: Path taken by a thread that receives a null
//    dequeue request.
//
//    Parameters: None.
//
//------------------------------------------------------------------
#macro _aal5_tx_null_deq()

null_start#:

#ifdef _DEBUG_
	alu[@null_counter, @null_counter, +, 1]
#endif

;----------------------------
; Pipestage 1 - phase 3
;----------------------------

	signal_ctx(next_thread)

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 5
	alu[--, --, B, 0]
	#endloop

	ctx_arb[prev_thread_done_sig]	

;----------------------------
; Pipestage 1 - phase 4
;----------------------------

	signal_ctx(next_thread)

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 15
	alu[--, --, B, 0]
	#endloop

	ctx_arb[prev_thread_done_sig]	

;----------------------------
; Pipestage 1 - phase 5
;----------------------------

	signal_ctx(next_thread)

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 5
	alu[--, --, B, 0]
	#endloop

	br!=ctx[7, null_not_context_7#]	

	; Signal thread 0 of the next me.
	alu[--, --, B, start_pipestage1]
	cap[fast_wr, ALU, interthread_sig]	

	; Clear CAM.
	cam_clear

null_not_context_7#:	

	ctx_arb[prev_thread_done_sig]

	signal_ctx(next_thread)	

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 15
	alu[--, --, B, 0]
	#endloop

	br!=ctx[0, null_end_pipestage1#]	

	ctx_arb[prev_thread_done_sig, pipestage2_done_sig], \
			br[null_pipestage2#]

null_end_pipestage1#:		

	ctx_arb[prev_thread_done_sig]	

null_pipestage2#:		

;----------------------------
; Pipestage 2 - phase 1
;----------------------------

	signal_ctx(next_thread)

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 1
	alu[--, --, B, 0]
	#endloop

	ctx_arb[prev_thread_done_sig]	

	_aal5_tx_dl_source(scratch_rd_sig)

;----------------------------
; Pipestage 2 - phase 2
;----------------------------

	signal_ctx(next_thread)

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 10
	alu[--, --, B, 0]
	#endloop

	br!=ctx[7, null_transmit_cell#]	

	; Signal thread 0 of the next me.
	alu[--, --, B, start_pipestage2]
	cap[fast_wr, ALU, interthread_sig]	

	; Clear CAM.
	cam_clear

null_transmit_cell#:		

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 5
	alu[--, --, B, 0]
	#endloop

	ctx_arb[prev_thread_done_sig, scratch_rd_sig]	

;----------------------------
; Pipestage 2 - phase 3
;----------------------------

	signal_ctx(next_thread)

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 1
	alu[--, --, B, 0]
	#endloop

	ctx_arb[prev_thread_done_sig]	

;----------------------------
; Pipestage 1 - phase 2
;----------------------------
	// SCR6027: added oam_cell_deq: label
	_aal5_tx_check_null_deq_and_setup_cam_data(null_end#, null_start#, oam_cell_deq#)

null_end#:
	nop

#endm

// SCR6027: added function to bypass all other stages before phase 2 &3 for OAM
//------------------------------------------------------------------
// _aal5_tx_oam_deq()
//
//    Description: Path taken by a thread that receives a
//     request for which OAM flag descriptor is set. So bypass
//     all stages with proper signalling to do oam processing.
//
//    Parameters: None.
//
//------------------------------------------------------------------
#macro _aal5_tx_oam_deq()
.begin
	#ifdef ATM_OAM_ENABLED
	.set tcw_payload_offset
	.set curr_payload_addr_offset
;----------------------------
; Pipestage 1 - phase 3
;----------------------------

	signal_ctx(next_thread)

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 4
	alu[--, --, B, 0]
	#endloop

	; Mark current cell header as invalid since this is a 
	; flag descriptor.
	alu[atm_cell_copy_header, --, B, 0]

	ctx_arb[prev_thread_done_sig]	

;----------------------------
; Pipestage 1 - phase 4
;----------------------------

	signal_ctx(next_thread)

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 15
	alu[--, --, B, 0]
	#endloop

	ctx_arb[prev_thread_done_sig]	

;----------------------------
; Pipestage 1 - phase 5
;----------------------------

	signal_ctx(next_thread)

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 5
	alu[--, --, B, 0]
	#endloop

	ctx_arb[prev_thread_done_sig]

	signal_ctx(next_thread)	

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 15
	alu[--, --, B, 0]
	#endloop

	ctx_arb[prev_thread_done_sig]	

;----------------------------
; Pipestage 2 - phase 1
;----------------------------
	signal_ctx(next_thread)

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 1
	alu[--, --, B, 0]
	#endloop

	ctx_arb[prev_thread_done_sig, avail_tbuf_element_sig]	

	_aal5_tx_dl_source(scratch_rd_sig)

;----------------------------
; Pipestage 2 - phase 2
;----------------------------

	signal_ctx(next_thread)

	; Compute the TBUF address of the available TBUF element.
	alu[tbuf_addr, *l$index1[TBUF_ADDR_MASK_INDEX], AND, \
			$avail_tbuf_element, <<TBUF_ELE_SIZE_LOG2]

	; Set tbuf_addr to be used during cell validation.
	alu[tbuf_addr, --, B, tbuf_addr, >>TCW_ADDR_SHIFT]	

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 8
	alu[--, --, B, 0]
	#endloop

	#endif // ATM_OAM_ENABLED
.end
#endm

#endif // _AAL5_TX_UTIL_FP_ME_UC_