#ifndef	_AAL5_TX_UTIL_ONE_ME_UC_
#define _AAL5_TX_UTIL_ONE_ME_UC_

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2002 By Intel Corporation.  All rights reserved.
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

/*
 *                                                                   
 *      File Name: aal5_tx_util_one_me.uc                                         
 *                                                                   
 *      Description: This file contains macros that implement each
 *                   processing stage of the one me implementation.
 *                                                                   
 *
 */

#include <aal5_tx.h>
#include <aal5_tx_regs.uc>
#include <aal5_tx_util_common.uc>
#include <aal5_tx_crc.uc>
#include <xbuf.uc>

//------------------------------------------------------------------
// Pipestage 1 - phase 1
//------------------------------------------------------------------
// _aal5_tx_dl_source()
//
//    Description: Fetch the dequeue request from scratch ring.
//
//    Parameters:
//        Inputs:
//        scratch_rd_sig:
//            Signal used to indicate completion of scratch[get].
//
//------------------------------------------------------------------
#macro _aal5_tx_dl_source(scratch_rd_sig)
.begin

	; Read vcq and buffer handle from scratch ring.
	scratch[get, $vcq, ring, 0, 2], sig_done[scratch_rd_sig]

.end
#endm


//------------------------------------------------------------------
// Pipestage 1 - phase 2
//------------------------------------------------------------------
// _aal5_tx_check_null_deq_and_setup_cam_data()
//
//    Description: Check if the dequeue request is null.
//    If it is, follow the null_deq path.
//    If not, setup cam data to start pipestage 1.
//
//    Parameters:
//        Inputs:
//        NON_NULL_DEQ_LABEL:
//            Label to branch to if dequeue request is non-null.
//        NULL_DEQ_LABEL:
//            Label to branch to if dequeue request is null.
//        OAM_DEQ_LABEL:
//            Label to branch to OAM processing
//------------------------------------------------------------------
#macro _aal5_tx_check_null_deq_and_setup_cam_data( \
			NON_NULL_DEQ_LABEL, NULL_DEQ_LABEL, OAM_DEQ_LABEL)
.begin

	signal_ctx(next_thread)

	; if(dl_source() == null), branch to null_path#.
	alu[vcq, *l$index1[VCQ_MASK_INDEX], AND ,$vcq]
	beq[null_path#]


 	; Issue read for next available TBUF element from scratch.
	; This TBUF element will be used later to write out the cell.
	scratch[test_and_incr, $avail_tbuf_element, \
		*l$index1[AAL5_TX_AVAIL_TBUF_ELEMENT_INDEX], partition_number], \
		sig_done[avail_tbuf_element_sig]

	; Store Port# and CLP bit in LM
	.reg temp
	move[temp, PORT_MASK]
	alu[txc_port_and_clp, temp, AND, $vcq, >>18]

// SCR6027: Added check for oam flag descriptor
#ifdef ATM_OAM_ENABLED
	.reg	atm_flag_descr_handle
	immed32[atm_flag_descr_handle, ATM_OAM_FLAG_DESCR]
	alu[--, $buf_hdl_lw, XOR, atm_flag_descr_handle]
	beq[atm_oam_tx_path#], defer[1]
	alu[outport, --, B, txc_port_and_clp, >>1]
#endif // ATM_OAM_ENABLED

	; Compute SRAM addr of the new TXC.
	alu[txc_sram_offset, --, B, vcq, <<6]


	; Get the cam_result using vcq and set corres. TXCs' LM address.		
	; Needs 3 nops or eqv. instructions before *l$index0 is used.
	cam_lookup[cam_result, vcq], lm_addr0[0]

	; Get the cam_entry.
	alu[cam_entry, CAM_ENTRY_MASK, AND, cam_result, >>3]

	; Get CAM LRU entry 
.reg lru_txc_sram_offset
	cam_read_tag[lru_txc_sram_offset, cam_entry]

	; Update cam_entry with vcq.	
	cam_write[cam_entry, vcq, 1]

	; Convert $buf_hdl_lw from lw's to byte address.
	ld_field_w_clr[buf_hdl, 0111, $buf_hdl_lw]
	alu[buf_hdl, --, B, buf_hdl, <<2]

	; Save old TXC if we have CAM miss
	br_bclr[cam_result, 7, save_txc#]

	ctx_arb[prev_thread_done_sig], br[NON_NULL_DEQ_LABEL]

null_path#:

	ctx_arb[prev_thread_done_sig], \
			br[NULL_DEQ_LABEL]

save_txc#:
	xbuf_alloc($txc, 9, write)

	.sig lru_txc_io_sig

	alu[lru_txc_sram_offset, --, B, lru_txc_sram_offset, <<6]

	; Copy TXC from LM to s-xfer-write.
	alu[$txc[0], --, B, *l$index0[0]]	
	alu[$txc[1], --, B, *l$index0[1]]	
	alu[$txc[2], --, B, *l$index0[2]]	
	alu[$txc[3], --, B, *l$index0[3]]	
	alu[$txc[4], --, B, *l$index0[4]]	
	alu[$txc[5], --, B, *l$index0[5]]	
	alu[$txc[6], --, B, *l$index0[6]]	
	alu[$txc[7], --, B, *l$index0[7]]	
	alu[$txc[8], --, B, *l$index0[8]]	

	; Write TXC to SRAM.

	; Set ref cnt 8 (to write 9 lw's) using indirect format.
	; Also set the reference count override bit (bit 25).
	alu_shf[--, --, B, 0x18, <<21]
	sram[write, $txc[0], *l$index1[TXC_BASE_SRAM_INDEX], \
		lru_txc_sram_offset, max_9], indirect_ref, \
		sig_done[lru_txc_io_sig]
	
	ctx_arb[prev_thread_done_sig,lru_txc_io_sig], br[NON_NULL_DEQ_LABEL]
	xbuf_free($txc)
// SCR6027: Added OAM related processing
#ifdef ATM_OAM_ENABLED
atm_oam_tx_path#:
	ctx_arb[prev_thread_done_sig], br[OAM_DEQ_LABEL]
#endif // ATM_OAM_ENABLED)
.end
#endm


//-----------------------------------------------------------------
// Pipestage 1 - phase 3
//-----------------------------------------------------------------
// _aal5_tx_fetch_txc_in_lm()
//
//    Description: Fetch TXC in local memory.
//
//    Parameters: RAW_CELL_TX_LABEL -jump to raw mode processing
//
//------------------------------------------------------------------
//SCR6027: Added parameter, needed for RAW Mode
#macro _aal5_tx_fetch_txc_in_lm(RAW_CELL_TX_LABEL)
.begin

	signal_ctx(next_thread)

	; Is this is a new packet? If no, branch to non_sop#.
	br_bclr[$buf_hdl_lw, SOP_BIT, non_sop#]

sop#:
	
	.begin
// SCR6027: in case of raw mode read aal_parameter in meta data
#ifndef	RAW_MODE_ENABLED
	xbuf_alloc($meta, 4, read)

	.reg sram_channel

	alu[sram_channel, --, b, SRAM_REST_CHANNEL_NUMBER, <<30]

	; Read sop buffer meta data from SRAM.
	sram[read, $meta[0], buf_hdl, sram_channel, 4], sig_done[meta_rd_sig]
#else
	xbuf_alloc($meta, 6, read)

	.reg sram_channel

	alu[sram_channel, --, b, SRAM_REST_CHANNEL_NUMBER, <<30]

	; Read sop buffer meta data from SRAM.
	sram[read, $meta[0], buf_hdl, sram_channel, 6],\
		sig_done[meta_rd_sig]
#endif
#ifdef RAW_MODE_ENABLED
    //SCR6027: Read the full TXC context 
	// The OAM cells (like continuity check) transmitted from user can 
	// interleave between two dequeu req. for AAL5 cells. To avoid the 
	// currpotion of TXC in LM, when AAL5 packet processing thread finds
	// the VC locked by the OAM cell processing thread the whole TXC is 
	//loaded in LM 
	.reg $$txc_read[9]
	.xfer_order $$txc_read
	; Set ref cnt 8 (to write 9 lw's) using indirect format.
	; Also set the reference count override bit (bit 25).
	alu_shf[--, --, B, 0x18, <<21]
	sram[read, $$txc_read[0], *l$index1[TXC_BASE_SRAM_INDEX], \
		txc_sram_offset, max_9], indirect_ref, \
		sig_done[txc_io_sig]
	ctx_arb[meta_rd_sig, txc_io_sig, prev_thread_done_sig]

	br_bset[$$txc_read[2], RAW_MODE_BIT, handle_raw_cell_tx#]

	#ifdef ATM_OAM_ENABLED

	; AAL5 VCs could carry OAM cells which need to be transmitted
	; transparently. The transport type bit indicates if this is
	; a raw mode transmission request.

	br_bset[$meta[5], TRANSPORT_TYPE_BIT, handle_raw_cell_tx#]

	#endif

	; Add outgoing vpi/vci to cell header.
	; lm:txc_cell_header = $vpi_vci and CLP bit.
	alu[*l$index0[7], 0x1, AND, txc_port_and_clp]
	alu[*l$index0[7], *l$index0[7], OR, $$txc_read[7]]

#else
    .reg read $txc
    .reg tmp
	; Advance txc_sram_offset so that it points to the
	; outgoing cell header in TXC in SRAM.
	alu[tmp, txc_sram_offset, +, 28]

	; Read TXC2 from SRAM.
	sram[read, $txc, *l$index1[TXC_BASE_SRAM_INDEX], \
			tmp, 1], sig_done[txc_io_sig]

	ctx_arb[meta_rd_sig, prev_thread_done_sig, txc_io_sig]    // added txc_io_sig

	; Add outgoing vpi/vci to cell header.
	; lm:txc_cell_header = $vpi_vci and CLP bit.
	alu[*l$index0[7], 0x1, AND, txc_port_and_clp]
	alu[*l$index0[7], *l$index0[7], OR, $txc]

#endif // RAW_MODE_ENABLED

	 // Unfilled defer slot.

	; Using sop meta data initialize TXC.

	; lm:txc_bytes_rem_in_curr_buf = $meta_buf_size
	ld_field_w_clr[*l$index0[0], 0011, $meta[1], >>16]

	; lm:txc_outport = TM 4.1 output port
	ld_field[*l$index0[0], 1100, txc_port_and_clp, <<15]

	; lm:txc_curr_payload_addr =
	; dl_buf_get_data_from_meta($buf_hdl:buf_hdl) + $meta_offset
	dl_buf_get_data_from_meta(*l$index0[1], buf_hdl)

	.reg buf_offset
	ld_field_w_clr[buf_offset, 0011, $meta[1]]
	alu[*l$index0[1], *l$index0[1], +, buf_offset]

	; lm:txc_curr_buf_hdl = buf_hdl
	; lm:txc_bytes_done_in_next_buf = 0
	ld_field_w_clr[*l$index0[2], 0111, buf_hdl]

	; lm:txc_new_curr_buf_hdl = 1
	alu[*l$index0[2], *l$index0[2], OR, 1, <<24]

	; lm:txc_bytes_done_in_packet = 0
	immed[*l$index0[4], 0]

	; lm:txc_end_of_payload = 0
	; lm:txc_freelist = $meta_freelist
	.reg eop_and_freelist
	alu[eop_and_freelist, 0xF, AND, $meta[2], >>12]
	alu[*l$index0[5], --, B, eop_and_freelist, <<24]

	; lm:txc_packet_size = $meta_packet_size
	ld_field[*l$index0[6], 1100, $meta[2]]

	#ifdef COUNTERS
	_aal5_inc_counter(PKTS_PER_VCQ, vcq)
	#endif // COUNTERS

	br[end#], defer[2]

	; lm:txc_next_buf_hdl = bits 23:0 of $meta_next_buf_hdl
	alu[*l$index0[3], $meta[0], AND~, 0xFF, <<24]

	; Convert lm:txc_next_buf_hdl from lw's to byte address.
	alu[*l$index0[3], --, B, *l$index0[3], <<2]

	xbuf_free($meta)

// SCR6027: Update the LM if its SOP and miss
#ifdef RAW_MODE_ENABLED

handle_raw_cell_tx#:

	; vcq a hit in CAM? If yes, branch to non_sop_hit#.
	br_bset[cam_result, 7, RAW_CELL_TX_LABEL]

	alu[*l$index0[0], --, B, $$txc_read[0]]	
	alu[*l$index0[1], --, B, $$txc_read[1]]	
	alu[*l$index0[2], --, B, $$txc_read[2]]	
	alu[*l$index0[3], --, B, $$txc_read[3]]	
	alu[*l$index0[4], --, B, $$txc_read[4]]	
	alu[*l$index0[5], --, B, $$txc_read[5]]	
	alu[*l$index0[6], --, B, $$txc_read[6]]	
	alu[*l$index0[7], --, B, $$txc_read[7]]	
	alu[*l$index0[8], --, B, $$txc_read[8]]	
	br[RAW_CELL_TX_LABEL]

//	xbuf_free($$txc_read)

#endif

	.end

non_sop#:

	; vcq a hit in CAM? If yes, branch to non_sop_hit#.
	br_bset[cam_result, 7, non_sop_hit#]

non_sop_miss#:

	.begin

	xbuf_alloc($txc, 9, read)

	; Read TXC from SRAM.

	; Set ref cnt 8 (to write 9 lw's) using indirect format.
	; Also set the reference count override bit (bit 25).
	alu_shf[--, --, B, 0x18, <<21]
	sram[read, $txc[0], *l$index1[TXC_BASE_SRAM_INDEX], \
		txc_sram_offset, max_9], indirect_ref, \
		sig_done[txc_io_sig]

	ctx_arb[txc_io_sig, prev_thread_done_sig]

	 // Unfilled defer slot.

	; Copy TXC from s-xfer-read to LM.
	alu[*l$index0[0], --, B, $txc[0]]	
	alu[*l$index0[1], --, B, $txc[1]]	
	alu[*l$index0[2], --, B, $txc[2]]	
	alu[*l$index0[3], --, B, $txc[3]]	
	alu[*l$index0[4], --, B, $txc[4]]	
	alu[*l$index0[5], --, B, $txc[5]]	
	alu[*l$index0[6], --, B, $txc[6]]	
	alu[*l$index0[7], --, B, $txc[7]]	
	alu[*l$index0[8], --, B, $txc[8]]	
	br[end#]

	xbuf_free($txc)

	.end

non_sop_hit#:

	ctx_arb[prev_thread_done_sig]

	 // Unfilled defer slot.

end#:

.end
#endm


//------------------------------------------------------------------
// Pipestage 1 - phase 4 in aal5_tx_util_common.uc
//------------------------------------------------------------------


//------------------------------------------------------------------
// Pipestage 1 - phase 5
//------------------------------------------------------------------
// _aal5_tx_prepare_cell()
//
//    Description: Update TXC.
//                 Prepare 48-byte cell in local memory. If cell
//                 payload is 40 or 48 bytes skip this step as an
//                 optimization.
//
//    Parameters: None.
//
//------------------------------------------------------------------
#macro _aal5_tx_prepare_cell()
.begin

	signal_ctx(next_thread)

	; if (prefetched_buffer == 1)
	; update TXC with prefetched buffer information.
	br_bclr[prefetched_buffer, 0, prefetch_udpate_done#]

	 // Unfilled defer slot.

	; lm:txc_next_buf_size = $prefetch_meta_buf_size
	ld_field[*l$index0[4], 1100, $prefetch_meta[1]]

	.reg txc_next_next_buf_hdl

	; lm:txc_next_next_buf_hdl = bits 23:0 of $prefetch_meta_next_buf_hdl
	alu[txc_next_next_buf_hdl, $prefetch_meta[0], AND~, 0xFF, <<24]

	; Convert lm:txc_next_next_buf_hdl from lw's to byte address.
	ld_field[*l$index0[5], 0111, txc_next_next_buf_hdl, <<2]

prefetch_udpate_done#:
		

	; ctx_arb on sigmask_b.
	ctx_arb[--], defer[2]

	local_csr_wr[active_ctx_wakeup_events, sigmask_b]

	; Reset prefetched_buffer.
	immed[prefetched_buffer, 0]

	.io_completed next_buf_rd_sig
	.io_completed curr_buf_rd_sig
	.io_completed prev_thread_done_sig	

	signal_ctx(next_thread)

	#ifdef COUNTERS
	_aal5_inc_counter(CELLS_PER_VCQ, vcq)
	#endif

	; If the cell payload in current buffer is 48 bytes,
	; do not prepare the cell in local memory. (Optimization).
	alu[--, cell_payload_in_curr_buf, -, 48]
	beq[cell_done#]
 
	; If the cell payload in current buffer is 40 bytes and
	; this is the last cell of the packet, do not prepare the
	; cell in local memory. (Optimization).
	alu[--, cell_payload_in_curr_buf, -, 40]
	bne[prepare_cell_in_lm#]
	alu[--, cell_payload_in_next_buf, -, 0]
	beq[cell_done#]

prepare_cell_in_lm#:

	; Prepares a 48 byte cell in a thread specific location in LM.
	; Pipestage 2 will compute crc on this 48 byte cell and transmit it
	; out through a TBUF element.
	_aal5_tx_prepare_cell_in_lm(curr_payload_addr_offset, 
		cell_payload_in_curr_buf, cell_payload_in_next_buf,
		payload_reg_base_addr_lw,
		next_payload_reg_base_lw)

cell_done#:

	; ctx_arb on sigmask_a.
	ctx_arb[--], defer[2]

	; Add avail_tbuf_element_sig to sigmask_a.
	alu[sigmask_a, sigmask_a, OR, 1, <<(&avail_tbuf_element_sig)]

	local_csr_wr[active_ctx_wakeup_events, sigmask_a]

	.io_completed avail_tbuf_element_sig
	.io_completed tx_sequence_sig
	.io_completed prev_thread_done_sig	

.end
#endm


//------------------------------------------------------------------
// Pipestage 2 - phase 1
//------------------------------------------------------------------
// _aal5_tx_fetch_txc2_in_lm()
//
//    Description: For sop, initialize the TXC fields we will use in
//                 this pipestage.
//
//    Parameters: None.
//
//------------------------------------------------------------------
#macro _aal5_tx_fetch_txc2_in_lm()
.begin

	; Is this is a new packet? If no, branch to non_sop#.
	br_bclr[$buf_hdl_lw, SOP_BIT, non_sop#]

sop#:

	// Unfilled defer slot.

	#ifdef COUNTERS
	_aal5_inc_counter(PKTS_PER_OUTPORT, outport)
	#endif // COUNTERS


	; Initialize TXC:crc.
	; lm:txc_crc = INIT_CRC_REMAINDER
	immed[*l$index0[8], INIT_CRC_REMAINDER]


	br[end#]

non_sop#:

	 // Unfilled defer slot.

end#:

.end
#endm


//------------------------------------------------------------------
// Pipestage 2 - phase 2
//------------------------------------------------------------------
// _aal5_tx_compute_crc_and_transmit_cell()
//
//    Description: Compute CRC on cell, update TXC, write
//                 52-byte cell to TBUF.
//                 As an optimization, read the next dequeue request
//                 from the scratch ring.
//
//    Parameters: None.
//
//------------------------------------------------------------------
#macro _aal5_tx_compute_crc_and_transmit_cell()
.begin

	signal_ctx(next_thread)

	; Read next dequeue request from scratch (Optimization).
	_aal5_tx_dl_source(scratch_rd_sig)

	; Free current and next buffers if marked.

	; if (free_curr_buf:31 == 1), free current buffer.
	br_bclr[free_curr_buf, 31, check_free_next_buf#], defer[1]

	 // Unfilled defer slot.

	; Load CRC_Remainder CSR with lm:txc_crc
	local_csr_wr[CRC_Remainder, *l$index0[8]]

	; Freeing current buffer.
	dl_buf_free(free_curr_buf, BUF_FREE_LIST0)

	; Reset free_curr_buf.
	immed[free_curr_buf, 0]

check_free_next_buf#:

	.reg tbuf_addr_plus_4

	; if (free_next_buf:31 == 1), free next buffer.
	br_bclr[free_next_buf, 31, crc#], defer[3]

	; Compute the TBUF address of the available TBUF element.
	alu[tbuf_addr, *l$index1[TBUF_ADDR_MASK_INDEX], AND, \
			$avail_tbuf_element, <<TBUF_ELE_SIZE_LOG2]

	; Add 4 to the tbuf_addr. This new address will be used
	; when writing the 52-byte cell to TBUF. We leave the
	; first 4 bytes unused. These are the 4 unused bytes of the
	; prepend header that we define when validating the TBUF.
	; We need to do this since according to the TCW format,
	; the prepend length must be atleast 8 bytes (1 quadword)
	; and our prepend length is only 4 bytes (cell header).
	alu[tbuf_addr_plus_4, tbuf_addr, +, 4]

	; Compute the cells tx by this code (0..TBUF_ELE_NUM)
	alu[cells_tx_by_code, TX_SEQUENCE_MASK, \
			AND, $avail_tbuf_element]

	; Freeing next buffer.
	dl_buf_free(free_next_buf, BUF_FREE_LIST0)

	; Reset free_next_buf.
	immed[free_next_buf, 0]

crc#:

	xbuf_alloc($cell_lw, WRD_OFFSET_14, write)
	.set $cell_lw/**/[WRD_OFFSET_13]
// SCR6027: Set atm_cell_copy registers
#ifdef ATM_OAM_ENABLED
	.set $$atm_cell_copy[13]
#endif

	; If the cell payload in current buffer is 48 bytes compute
	; CRC on cell in d-xfer-rd regs. (Optimization).
	; Copy the 52-byte cell (4-byte cell header) into s-xfer-wr regs. 
	; Also store CRC residue in TXC.
	alu[--, cell_payload_in_curr_buf, -, 48]
	bne[check_40_bytes#], defer[1]

	; Set tcw_payload_offset to 0 by default.
	; This is used when validating the TBUF element containing the
	; 52-byte cell. Macro _aal5_tx_compute_and_store_crc() changes
	; the value of tcw_payload_offset if needed.
	immed[tcw_payload_offset, 0]

	_aal5_tx_compute_and_store_crc(FORTY_EIGHT, curr_payload_addr_offset,
		$cell_lw, crc_done#)

check_40_bytes#:

	; If the cell payload in current buffer is 40 bytes and this is the last
	; cell of the packet, compute CRC on cell in d-xfer-rd regs. (Optimization).
	; Copy the 52-byte cell (4-byte cell header) into s-xfer-wr regs. 
	; Also store CRC residue in TXC.
	alu[--, cell_payload_in_curr_buf, -, 40]
	bne[crc_in_lm#]
	alu[--, cell_payload_in_next_buf, -, 0]
	bne[crc_in_lm#]

	_aal5_tx_compute_and_store_crc(FORTY, curr_payload_addr_offset,
		$cell_lw, crc_done#)

crc_in_lm#:

	; Computes CRC on cell in local memory.
	; Also copies the 52 byte cell into s-xfer-wr regs. 
	_aal5_tx_compute_crc(cell_payload, $cell_lw)

	; Stores CRC residue in TXC.
	_aal5_tx_store_crc(cell_payload, $cell_lw[WRD_OFFSET_12])

crc_done#:

	.use	$$payload[0]  $$payload[1]  $$payload[2]  $$payload[3]  \
			$$payload[4]  $$payload[5]  $$payload[6]  $$payload[7]  \
			$$payload[8]  $$payload[9]  $$payload[10] $$payload[11] \
			$$payload[12] $$payload[13] $$payload[14] $$payload[15]


	#ifdef COUNTERS
	_aal5_inc_counter(CELLS_PER_OUTPORT, outport)
	#endif

	
	.begin
	.reg cells_pending, cells_tx_by_fabric_masked


#if (TX_PHY_MODE == SPHY_4_8)

    br=ctx[3, check_tx_sequence#]
    br=ctx[7, check_tx_sequence#]
    br[transmit_cell#]

check_tx_sequence#:    
#endif 

	; Read number of cells actually transmitted from this partition.
	msf[read, $cells_tx_by_fabric, *l$index1[TX_SEQUENCE_INDEX], \
		partition_number, 1], ctx_swap[tx_sequence_sig]

	// Mask-out the highest bit of the sequence word (sequence number is set on lowest 8 bits)
	alu[cells_tx_by_fabric_masked, $cells_tx_by_fabric, AND, TX_SEQUENCE_MASK]

	// Check if we have a wrap-round case for the cells transmit
	.if (cells_tx_by_code >= cells_tx_by_fabric_masked)
		alu[cells_pending, cells_tx_by_code, -, cells_tx_by_fabric_masked]
	.else
		.begin
			.reg cells_tx_by_code_corrected

			alu[cells_tx_by_code_corrected, cells_tx_by_code, +, TBUF_ELE_NUM]
			alu[cells_pending, cells_tx_by_code_corrected, -, cells_tx_by_fabric_masked]
		.end
	.endif

	#ifdef FLOW_CONTROL

	; Write fire-n-forget $cells_tx_by_fabric to scratch for the
	; scheduler to use.

	.io_completed sch_sig

	; Copy $cells_tx_by_fabric to write xfer register.
	alu[$cells_tx_by_fabric, TX_SEQUENCE_MASK, AND, $cells_tx_by_fabric]

	//	scratch[write, $cells_tx_by_fabric, *l$index1[AAL5_TX_CELLS_TX_INDEX], \
	//			partition_number, 1], sig_done[sch_sig]
	// For now, we will write to a scratch location that is
	// independent of partition number from where we read this data. This is
	// since the TM4.1 microblock currently does not support this feature.

	scratch[add, $cells_tx_by_fabric, *l$index1[AAL5_TX_CELLS_TX_INDEX], \
		0], sig_done[sch_sig]

	#endif // FLOW_CONTROL

	; Check if there is a TBUF element available
	; for the next thread.
	; 'cells_tx_by_code' gives us a measure of how
	; many TBUF elements this code has transmitted.
	; If the difference between 'cells_tx_by_code'
	; and '$cells_tx_by_fabric' exceeds a certain
	; threshold, stall until the difference falls
	; below the threshold.

	alu[--, cells_pending, -, TBUF_THRESHOLD]	
	.end // cells_pending
	bge[tbuf_full#]	

transmit_cell#:

#ifdef FPGA_HEADER
	alu[$cell_lw[0], 0x03, ~AND, txc_port_and_clp, <<1]

#ifdef ATM_OAM_ENABLED
	; Store the FPGA header in the 4 byte prepend header
	alu[$$prepend_header, 0x03, ~AND, txc_port_and_clp, <<1]
#endif

	; Set ref cnt to 14 (to write 15 lw's) using indirect format.
	; Also set the reference count override bit (bit 25).
	alu[--, --, B, (0x10 + WRD_OFFSET_13), <<21]
	; Write 4 byte FPGA header and 52 byte cell from s-xfer-out regs to TBUF.
	msf[write, $cell_lw[0], tbuf_element_base, tbuf_addr, max_/**/WRD_OFFSET_14], \
		indirect_ref, sig_done[cell_wr_sig]

#else

	; Set ref cnt to 13 (to write 14 lw's) using indirect format.
	; Also set the reference count override bit (bit 25).
	alu[--, --, B, (0x10 + WRD_OFFSET_13), <<21]
	; Write 52 byte cell from s-xfer-out regs to TBUF.
	msf[write, $cell_lw[0], tbuf_element_base, tbuf_addr_plus_4, max_/**/WRD_OFFSET_14], \
		indirect_ref, sig_done[cell_wr_sig]

#endif

	; ctx_arb on sigmask_a.
	ctx_arb[cell_wr_sig, prev_thread_done_sig], defer[1], \
		br[transmit_cell_done#]

	 // Unfilled defer slot.
	
	; Set tbuf_addr to be used during cell validation.
	alu[tbuf_addr, --, B, tbuf_addr, >>TCW_ADDR_SHIFT]	


tbuf_full#:
	; Stall inside tbuf_full() until more TBUF elements
	; are available. On exit, branch back to transmit_cell#.
	_aal5_tx_tbuf_full()
	br[transmit_cell#]

transmit_cell_done#:

	xbuf_free($cell_lw)

.end
#endm


//------------------------------------------------------------------
// Pipestage 2 - phase 3
//------------------------------------------------------------------
// _aal5_tx_validate_tbuf()
//
//    Description: Validate TBUF element.
//
//    Parameters: None.
//
//------------------------------------------------------------------
#macro _aal5_tx_validate_tbuf()

	; Used to signal TCW write for the written TBUF element.
	.sig tcw_wr_sig
	.reg tmp_port tmp_seop tmp_base

	signal_ctx(next_thread)

	xbuf_alloc($tcw, 2, write)

	.reg tcw_const
	; Set payload length to 48 bytes (cell payload),
//SCR6027: Removed case for RAW_MODE_ENABLED, now it is taken care by the following code

#ifdef FPGA_HEADER
	; prepend offset to 0 and prepend length to 8 bytes (FPGA & cell header).
	immed[tcw_const, ((48 << 8) | (0 << 5) | (8 << 0))]
#else
	; prepend offset to 4 and prepend length to 4 bytes (cell header).
	immed[tcw_const, ((48 << 8) | (4 << 5) | (4 << 0))]
#endif

calculate_control#:

#ifdef IXP2800
	alu[tmp_seop, --, B, 0x3, <<8]	; setting SoP and EoP
	; Set TCW0. Payload offset has been set and left shifted by 13.
#define_eval INDIRECT_SHIFT	(AAL5_FPGA_PORT_BITS + 1) // +1 bacause we don't need CLP bit
	alu[tmp_base, --, B, txc_port_and_clp, >>INDIRECT_SHIFT]
#undef 	INDIRECT_SHIFT
	alu[tmp_base, @spi4_addr, +, tmp_base]
	alu[tmp_port, tmp_seop, OR, tmp_base]

	alu[tcw_const, tmp_port, OR, tcw_const, <<16]
	; Set TCW0. Payload offset has been set and left shifted by 13.
	alu[$tcw[0], tcw_payload_offset, OR, tcw_const]

#else

    #if (defined(AAL5_TX_POS_LINE_0) || defined(AAL5_TX_POS_LINE_1) || \
	     defined(AAL5_TX_POS_LINE_2) || defined(AAL5_TX_POS_LINE_3) )

	    alu[tmp_seop, --, B, 0x3, <<8]	; setting SoP and EoP

  	    alu[tcw_const, tmp_seop, OR, tcw_const, <<16]

        ; Set TCW0. Payload offset has been set and left shifted by 13.
        alu[$tcw[0], tcw_payload_offset, OR, tcw_const]

    #else
        ; Set TCW0. Payload offset has been set and left shifted by 13.
        alu[$tcw[0], tcw_payload_offset, OR, tcw_const, <<16]

    #endif // (defined(AAL5_TX_POS_LINE_0) || defined(AAL5_TX_POS_LINE_1) ||
           //  defined(AAL5_TX_POS_LINE_2) || defined(AAL5_TX_POS_LINE_3))

#endif // IXP2800


	; TCW1 is reserved.
	.set $tcw[1]

	; Write TCW to validate transmit.
	msf[write, $tcw[0], tbuf_validate_base, tbuf_addr, 2], \
		sig_done[tcw_wr_sig]

	ctx_arb[scratch_rd_sig, tcw_wr_sig, prev_thread_done_sig], defer[2]

	; Init sigmask_a with prev_thread_done_sig for use in pipestage 1.
	alu[sigmask_a, --, B, 1, <<(&prev_thread_done_sig)]

	; Init sigmask_b with prev_thread_done_sig for use in pipestage 1.
	alu[sigmask_b, --, B, 1, <<(&prev_thread_done_sig)]

	xbuf_free($tcw)

#endm


//------------------------------------------------------------------
// _aal5_tx_null_deq()
//
//    Description: Path taken by a thread that receives a null
//    dequeue request.
//
//    Parameters: None.
//
//------------------------------------------------------------------
#macro _aal5_tx_null_deq()

null_start#:

;----------------------------
; Pipestage 1 - phase 3
;----------------------------

	signal_ctx(next_thread)

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 5
	alu[--, --, B, 0]
	#endloop

	ctx_arb[prev_thread_done_sig]	

;----------------------------
; Pipestage 1 - phase 4
;----------------------------

	signal_ctx(next_thread)

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 15
	alu[--, --, B, 0]
	#endloop

	ctx_arb[prev_thread_done_sig]	

;----------------------------
; Pipestage 1 - phase 5
;----------------------------

	signal_ctx(next_thread)

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 5
	alu[--, --, B, 0]
	#endloop

	ctx_arb[prev_thread_done_sig]

	signal_ctx(next_thread)	

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 15
	alu[--, --, B, 0]
	#endloop

	ctx_arb[prev_thread_done_sig]	

;----------------------------
; Pipestage 2 - phase 1
;----------------------------

	_aal5_tx_dl_source(scratch_rd_sig)

;----------------------------
; Pipestage 2 - phase 2
;----------------------------

	signal_ctx(next_thread)

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 10
	alu[--, --, B, 0]
	#endloop


null_transmit_cell#:		

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 5
	alu[--, --, B, 0]
	#endloop

	ctx_arb[prev_thread_done_sig, scratch_rd_sig]	

null_transmit_cell_done#:
// SCR6027: OAM processing has added phases 3 & 4 in pipestatge 2 so bypass it.
#ifdef ATM_OAM_ENABLED
;----------------------------
; Pipestage 2 - phase 3 & 4
;----------------------------

	_aal5_oam_tx_bypass_processing()

#endif

;----------------------------
; Pipestage 2 - phase 3
;----------------------------

	signal_ctx(next_thread)

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 1
	alu[--, --, B, 0]
	#endloop

	ctx_arb[prev_thread_done_sig]	

;----------------------------
; Pipestage 1 - phase 2
;----------------------------
	// SCR6027: added oam_cell_deq: label
	_aal5_tx_check_null_deq_and_setup_cam_data(null_end#, null_start#, oam_cell_deq#)

null_end#:
	nop

#endm
// SCR6027: added function to bypass all other stages before phase 2 &3 for OAM
//------------------------------------------------------------------
// _aal5_tx_oam_deq()
//
//    Description: Path taken by a thread that receives a
//     request for which OAM flag descriptor is set. So bypass
//     all stages with proper signalling to do oam processing.
//
//    Parameters: None.
//
//------------------------------------------------------------------
#macro _aal5_tx_oam_deq()
.begin
	#ifdef ATM_OAM_ENABLED
	.set tcw_payload_offset
	.set curr_payload_addr_offset
;----------------------------
; Pipestage 1 - phase 3
;----------------------------

	signal_ctx(next_thread)

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 4
	alu[--, --, B, 0]
	#endloop

	; Mark current cell header as invalid since this is a 
	; flag descriptor.
	alu[atm_cell_copy_header, --, B, 0]

	ctx_arb[prev_thread_done_sig]	

;----------------------------
; Pipestage 1 - phase 4
;----------------------------

	signal_ctx(next_thread)

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 15
	alu[--, --, B, 0]
	#endloop

	ctx_arb[prev_thread_done_sig]	

;----------------------------
; Pipestage 1 - phase 5
;----------------------------

	signal_ctx(next_thread)

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 5
	alu[--, --, B, 0]
	#endloop

	ctx_arb[prev_thread_done_sig]

	signal_ctx(next_thread)	

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 15
	alu[--, --, B, 0]
	#endloop

	ctx_arb[prev_thread_done_sig]	

;----------------------------
; Pipestage 2 - phase 1
;----------------------------
	signal_ctx(next_thread)

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 1
	alu[--, --, B, 0]
	#endloop

	ctx_arb[prev_thread_done_sig, avail_tbuf_element_sig]	

	_aal5_tx_dl_source(scratch_rd_sig)

;----------------------------
; Pipestage 2 - phase 2
;----------------------------

	signal_ctx(next_thread)

	; Compute the TBUF address of the available TBUF element.
	alu[tbuf_addr, *l$index1[TBUF_ADDR_MASK_INDEX], AND, \
			$avail_tbuf_element, <<TBUF_ELE_SIZE_LOG2]

	; Set tbuf_addr to be used during cell validation.
	alu[tbuf_addr, --, B, tbuf_addr, >>TCW_ADDR_SHIFT]	

	; alu here to simulate actual code. We do this to 
	; avoid flooding scratch[read] requests.
	; we can't use nop due to optimalizator 
	#repeat 8
	alu[--, --, B, 0]
	#endloop

	#endif // ATM_OAM_ENABLED
.end
#endm

#endif // _AAL5_TX_UTIL_ONE_ME_UC_