#ifndef	_RAW_TX_UTIL_COMMON_UC_
#define _RAW_TX_UTIL_COMMON_UC_

/*******************************************************************************
                             Intel Proprietary

 Copyright (c) 1998-2002 By Intel Corporation.  All rights reserved.
 No part of this program or publication may be reproduced, transmited,
 transcribed, stored in a retrieval system, or translated into any language
 or computer language in any form or by any means, electronic, mechanical,
 magnetic, optical, chemical, manual, or otherwise, without the prior
 written permission of:
                         Intel Corporation
                         2200 Mission College Blvd.
                         Santa Clara, CA  95052-8119
*******************************************************************************/

/*        
 *      File Name: raw_tx_util_common.uc
 *                                                                   
 *      Description: This file contains utility macros that are used
 *                   during the processing phase.
 *
 */                                                                   

#include <aal5_tx.h>
#include <aal5_tx_regs.uc>


#define RAW_MODE_LW		2
#define RAW_MODE_BIT	31


//------------------------------------------------------------------
// Pipestage 1 - phase 4
//------------------------------------------------------------------
// _raw_tx_read_cell_payload_and_update_txc1()
//
//    Description: Read payload from DRAM.
//                 Update TXC.
//                 Prefetch next buffers' meta data if needed.
//
//    Parameters: buf_offset - This is read from SRAM
//
//------------------------------------------------------------------
#macro _raw_tx_read_cell_payload_and_update_txc(in_buf_offset)
.begin
	.reg sram_channel
	//SCR6027 - Added code to read buffer offset from SRAM
	xbuf_alloc($meta, 2, read)

	signal_ctx(next_thread)

	alu[sram_channel, --, b, SRAM_REST_CHANNEL_NUMBER, <<30]

	; Read sop buffer meta data from SRAM.
	sram[read, $meta[0], buf_hdl, sram_channel, 2], sig_done[meta_rd_sig]

	;The associated buffer is freed (raw mode).
	alu[free_curr_buf, Ox80000000, OR, $buf_hdl_lw]	


	; Store lm:txc_freelist in a gpr for use in pipestage 2.
;	alu[freelist, 0xF, AND, *l$index0[5], >>24]

	ctx_arb[prev_thread_done_sig, meta_rd_sig]

	ld_field_w_clr[in_buf_offset, 0011, $meta[1]]

.end
#endm

//------------------------------------------------------------------
// Pipestage 1 - phase 5
//------------------------------------------------------------------
// _raw_tx_prepare_cell()
//
//    Description: Signal next thread
//
//    Parameters: Input- Buffer offset.
//
//------------------------------------------------------------------
#macro _raw_tx_prepare_cell(in_buf_offset)
.begin
	//SCR6027 - Added code to read payload
	signal_ctx(next_thread)

	; ctx_arb on sigmask_b.
	ctx_arb[--], defer[1]
		local_csr_wr[active_ctx_wakeup_events, sigmask_b]

	.io_completed prev_thread_done_sig	

	signal_ctx(next_thread)
		; Calculate dram address
	dl_buf_get_data_from_meta(curr_payload_addr_offset, buf_hdl)

	; Point the payload address offset to start of data in DRAM
	alu[curr_payload_addr_offset, curr_payload_addr_offset, +, in_buf_offset]

	; Read the cell buffer from DRAM using the 
	; indirect reference format.
	alu[--, --, B, 0x16, <<21]

	; Read payload from current buffer.
	dram[read, $$payload[0], curr_payload_addr_offset, 0, max_7], \
	        indirect_ref, sig_done[curr_buf_rd_sig]

		
	; Drop the ATM cell buffer
	
	; Add curr_buf_rd_sig and curr_buf_rd_sig+1 to signal mask

	alu[sigmask_a, sigmask_a, OR, 3, <<(&curr_buf_rd_sig)]

	; ctx_arb on sigmask_a.
	ctx_arb[--], defer[2]

		; Add avail_tbuf_element_sig to sigmask_a.
		alu[sigmask_a, sigmask_a, OR, 1, <<(&avail_tbuf_element_sig)]

		local_csr_wr[active_ctx_wakeup_events, sigmask_a]

	.io_completed avail_tbuf_element_sig
	.io_completed prev_thread_done_sig	
    .io_completed curr_buf_rd_sig
.end
#endm

//------------------------------------------------------------------
// Pipestage 2 - phase 2
//------------------------------------------------------------------
// _raw_transmit_cell()
//
//    Description: Write payload read from DRAM to MSF. Create a copy
//                 of the payload for OAM processing. Fill FPGA header
//                 in first LW written to MSF if FPGA header insertion
//                 enabled. The below code assumes that the payload 
//				   will be quad word aligned in DRAM.
//
//    Parameters: in_buf_offset	- Data offset in DRAM buffer.
//
//------------------------------------------------------------------
#macro _raw_transmit_cell(in_buf_offset)
.begin
	//SCR6027 - Added code to support OAM processing in case of RAW cells
	.reg tbuf_addr_plus_4		; Offset to write in TBUF
	.reg cells_pending
	.reg cells_tx_by_fabric_masked

	signal_ctx(next_thread)

	; Read next dequeue request from scratch (Optimization).
	_aal5_tx_dl_source(scratch_rd_sig)

	; Compute the TBUF address of the available TBUF element.
	alu[tbuf_addr, *l$index1[TBUF_ADDR_MASK_INDEX], AND, \
			$avail_tbuf_element, <<TBUF_ELE_SIZE_LOG2]

	; Add 4 to the tbuf_addr. This new address will be used
	; when writing the 52-byte cell to TBUF. We leave the
	; first 4 bytes unused. These are the 4 unused bytes of the
	; prepend header that we define when validating the TBUF.
	; We need to do this since according to the TCW format,
	; the prepend length must be atleast 8 bytes (1 quadword)
	; and our prepend length is only 4 bytes (cell header).
	alu[tbuf_addr_plus_4, tbuf_addr, +, 4]

	; Compute the cells tx by this code (0..TBUF_ELE_NUM)
	
	alu[cells_tx_by_code, TX_SEQUENCE_MASK, \
			AND, $avail_tbuf_element]

	; Read number of cells actually transmitted from this partition.
	msf[read, $cells_tx_by_fabric, *l$index1[TX_SEQUENCE_INDEX], \
		partition_number, 1], sig_done[tx_sequence_sig]

waiting_for_tx_seq_rd_sig#:
	br_!signal[tx_sequence_sig, waiting_for_tx_seq_rd_sig#]	

	; Copy $cells_tx_by_fabric to write xfer register.
	alu[$cells_tx_by_fabric, TX_SEQUENCE_MASK, AND, $cells_tx_by_fabric]

	// Mask-out the highest bit of the sequence word (sequence number is set on lowest 8 bits)
	alu[cells_tx_by_fabric_masked, $cells_tx_by_fabric, AND, TX_SEQUENCE_MASK]

	// Check if we have a wrap-round case for the cells transmit
	.if (cells_tx_by_code < cells_tx_by_fabric_masked)
		.begin
			.reg cells_tx_by_code_corrected

			alu[cells_tx_by_code_corrected, cells_tx_by_code, +, TBUF_ELE_NUM]
			alu[cells_pending, --, B, cells_tx_by_fabric_masked]

			alu[cells_pending, cells_tx_by_code_corrected, -, cells_pending]			
		.end
	.else

		alu[cells_pending, cells_tx_by_code, -, cells_tx_by_fabric_masked]

	.endif

#ifdef FLOW_CONTROL
	; Write fire-n-forget $cells_tx_by_fabric to scratch for the
	; scheduler to use.
	.sig sch_sig

	scratch[add, $cells_tx_by_fabric, *l$index1[AAL5_TX_CELLS_TX_INDEX], \
		0], sig_done[sch_sig]

	.io_completed sch_sig
#endif // FLOW_CONTROL

	; Check if there is a TBUF element available
	; for the next thread.
	; 'cells_tx_by_code' gives us a measure of how
	; many TBUF elements this code has transmitted.
	; If the difference between 'cells_tx_by_code'
	; and '$cells_tx_by_fabric' exceeds a certain
	; threshold, stall until the difference falls
	; below the threshold.

	alu[--, cells_pending, -, TBUF_THRESHOLD]	
	bge[tbuf_full#]	

transmit_cell#:

	xbuf_alloc($cell_lw, WRD_OFFSET_13, write)

	#define_eval	i	WRD_OFFSET_0
	#define_eval	j	0

	#while (i < WRD_OFFSET_13)

		alu[$cell_lw[i], --, B, $$payload[j]]

#ifdef ATM_OAM_ENABLED

		alu[$$atm_cell_copy[j], --, B, $$payload[j]]

		#if (i == WRD_OFFSET_0)
			alu[atm_cell_copy_header, --, B, $$payload[j]]
		#else
			#if (i == WRD_OFFSET_1)
				alu[cur_bedc, --, B, $$payload[j]]
			#else
				alu[cur_bedc, cur_bedc, XOR, $$payload[j]]
			#endif
		#endif
#endif

		#define_eval i (i+1)
		#define_eval j (j+1)

	#endloop
	#undef i
	#undef j

#ifdef FPGA_HEADER

	alu[$cell_lw[0], 0x03, ~AND, txc_port_and_clp, <<1]

#ifdef ATM_OAM_ENABLED
	alu[$$prepend_header, 0x03, ~AND, txc_port_and_clp, <<1]
#endif

	; Set ref cnt to 13 (to write 14 lw's) using indirect format.
	; Also set the reference count override bit (bit 25).
	alu[--, --, B, (0x10 + WRD_OFFSET_12), <<21]
	; Write 4 byte FPGA header and 52 byte cell from s-xfer-out regs to TBUF.
	msf[write, $cell_lw[0], tbuf_element_base, tbuf_addr, max_/**/WRD_OFFSET_13], \
		indirect_ref, sig_done[cell_wr_sig]

#else
	; Set ref cnt to 12 (to write 13 lw's) using indirect format.
	; Also set the reference count override bit (bit 25).
	alu[--, --, B, (0x10 + WRD_OFFSET_12), <<21]
	; Write 52 byte cell from s-xfer-out regs to TBUF.
	msf[write, $cell_lw[0], tbuf_element_base, tbuf_addr_plus_4, max_/**/WRD_OFFSET_13], \
		indirect_ref, sig_done[cell_wr_sig]
#endif

	; Freeing current buffer.
	dl_buf_drop(free_curr_buf)

    immed[free_curr_buf, 0]

	; ctx_arb on sigmask_a.
	ctx_arb[cell_wr_sig, prev_thread_done_sig],\
		br[transmit_cell_done#], defer[2]

	 // Unfilled defer slot.

		immed[tcw_payload_offset, 0]
	
		; Set tbuf_addr to be used during cell validation.
		alu[tbuf_addr, --, B, tbuf_addr, >>TCW_ADDR_SHIFT]	

tbuf_full#:
	; Stall inside tbuf_full() until more TBUF elements
	; are available. On exit, branch back to transmit_cell#.
	_aal5_tx_tbuf_full()
	br[transmit_cell#]

transmit_cell_done#:

.end
#endm

//------------------------------------------------------------------
// _raw_tx_handle_raw_cell_tx()
//
//    Description: This macro calls the raw mode cell handling pipestages
//
//    Parameters: None.
//
//------------------------------------------------------------------
#macro _raw_tx_handle_raw_cell_tx()
.begin
	.reg buf_offset
	.set $$payload[0], $$payload[1], $$payload[2], $$payload[3], $$payload[4], $$payload[5]
	.set $$payload[6], $$payload[7], $$payload[8], $$payload[9], $$payload[10], $$payload[11]
	.set $$payload[12], $$payload[13]

#ifdef ATM_OAM_ENABLED
	.set cur_bedc, atm_cell_copy_header
	.set $$atm_cell_copy0, $$atm_cell_copy1, $$atm_cell_copy2, $$atm_cell_copy3
	.set $$atm_cell_copy4, $$atm_cell_copy5, $$atm_cell_copy6, $$atm_cell_copy7
	.set $$atm_cell_copy8, $$atm_cell_copy9, $$atm_cell_copy10, $$atm_cell_copy11
	.set $$atm_cell_copy12, $$atm_cell_copy13
#endif
	;Pipestage1 phase 4 in raw mode
	_raw_tx_read_cell_payload_and_update_txc(buf_offset)

	;Pipestage1 phase 5 in raw mode
	_raw_tx_prepare_cell(buf_offset)

	;Pipestage2 phase 2 in raw mode
	_raw_transmit_cell(buf_offset)

.end
#endm // raw_tx_handle_raw_cell_tx()
#endif // _RAW_TX_UTIL_COMMON_UC_
