/////////////////////////////////////////////////////////////////////////////////////
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//
/////////////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: CsixTx_Init.c
//
//      Abstract: Initialization code for CSIX TX
//
//      History:
//
//      Date            Comment                  				       By
//      ---------------------------------------------------------------------
//
//      07/11/2002      Created from microcode version              Uday Naik
//
/////////////////////////////////////////////////////////////////////////////////////

#include "csix_tx_c.h"

/////////////////////////////////////////////////////////////////////////////////////
// 
// CsixTxSignalInit()
//
// Description:
// 	
//		Initialize the signal masks
// 
//
/////////////////////////////////////////////////////////////////////////////////////

static void CsixTxSignalInit(void)
{	


	// signal mask for phase 1

	sigmask_phase1			= __signals(
											&sig_scratchRead, 
											&sig_tcwWrite,
											&sig_tmhWrite,
											&sig_nextThread2
										);

	// signal mask for phase 2
									
	sigmask_phase2			= __signals(
											&sig_metaRead,
											&sig_dramToTbuf,
											&sig_nextThread1
										);

	// set up the value for the init_complete_signal

	__assign_relative_register(&sig_initComplete, ME_INIT_SIGNAL);

}
	
/////////////////////////////////////////////////////////////////////////////////////
// 
// CsixTxMsfTxControlInit()
//
// Description:
// 	
//		Initialize the msf
// 
//
/////////////////////////////////////////////////////////////////////////////////////

static void CsixTxMsfTxControlInit(void)
{

	__declspec(sram_write_reg) unsigned int data;
	SIGNAL					   sig_writeTxControl;

	data = CSIX_TX_CTL_LW;
	msf_write(&data, (volatile void*) MSF_TX_CNTL, 1, ctx_swap, &sig_writeTxControl);

#ifdef IXP2800

	data = 0;
	msf_write(&data, (volatile void*) SRB_OVERRIDE, 1, ctx_swap, &sig_writeTxControl);

#endif

}


/////////////////////////////////////////////////////////////////////////////////////
// 
// CsixTx_InitCam
//
// Description:
// 	
//		Initialize the cam
// 
//
/////////////////////////////////////////////////////////////////////////////////////

static void CsixTxInitCam(void)
{


	unsigned int i;
	unsigned int tag;

 	// CAM initialization

	cam_clear();

	// Initialize CAM to first 16 queue numbers and corresponding local memory 
	// addresses to 0

	// Set initial tag to queue number of 0 Or'ed with valid bit in MSB
	// The MSB bit is set, so we can compare with the value read from the
	// scratch ring directly.

	tag = (1 << 31);

	for (i = 0; i < 16; i++)
	{
		
		cam_write(i, tag, CAM_STATE_INIT);
		tag++;

	}

}

/////////////////////////////////////////////////////////////////////////////////////
// 
// CsixTxInitLocalMemory
//
// Description:
// 	
//		Initialize the local memory for contexts
// 
//
/////////////////////////////////////////////////////////////////////////////////////

static void CsixTxInitLocalMemory(void)
{

	unsigned int 	i;

	for (i = 0; i < 7; i++)
	{

		// Assume that since txc array is a global, it is initialized to 0
		
		// set the buffer handle to an invalid value 

		globalTxc[i].bufHandle.value = INVALID_HANDLE;

	}

}


/////////////////////////////////////////////////////////////////////////////////////
// 
// CsixTx_Init()
//
// Description:
// 	
//		Initialization of the microblock
// 
//
/////////////////////////////////////////////////////////////////////////////////////

void CsixTx_Init(void)
{
	
	// Initialize the signals 

	CsixTxSignalInit();

	// For the first thread we will do some initialization. every one else does no 
	// initialization

	if (__ctx() == CSIX_TX_FIRST_THREAD)
	{

		// Initialize MSF_TX_CNTL register

		CsixTxMsfTxControlInit();

		// initialize globals shared across threads 

		globalCurrentTbufElementNumber = 0;

		// Initialize the cam 

		CsixTxInitCam();

		// initialize the local memory 

		CsixTxInitLocalMemory();
    
		// Following inter-ME signal init_complete_signal is generated by 
		// the ME which initializes common resources, such as scratch ring, CSIX TX
		// We will wait for this signal to make sure all common resource are 
		// available, before we start CSIX TX processing

#ifdef WAIT_FOR_COMMON_RESOURCE_INITIALIZATION 

		wait_for_all(&sig_initComplete);

#endif 

	    // For kick start, first thread  will send itself 2 initial signals for 
		// phase 1 and 2 . signal number goes into bits 3..6 and thread id into
		// bits 0..2

		signal_same_ME(__signal_number(&sig_nextThread1), CSIX_TX_FIRST_THREAD);
		signal_same_ME(__signal_number(&sig_nextThread2), CSIX_TX_FIRST_THREAD);
	

	}  // end if (__ctx == ...)

}

/////////////////////////////////////////////////////////////////////////////////////

