/////////////////////////////////////////////////////////////////////////////////////
//
//
//                  I N T E L   P R O P R I E T A R Y
//
//     COPYRIGHT [c]  2002 BY  INTEL  CORPORATION.  ALL RIGHTS
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT
//     THE PRIOR WRITTEN PERMISSION OF :
//
//                        INTEL  CORPORATION
//
//                     2200 MISSION COLLEGE BLVD
//
//               SANTA  CLARA,  CALIFORNIA  95052-8119
//
/////////////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: csix_tx.h
//
//      Purpose: header file csix transmit block
//
//
//      History:
//
//      Date            Comment                         				By
//      ---------------------------------------------------------------------------
//
//      11/21/2001      Created                         			David Chou
//
//		3/18/02			Modified to do packet interleaving			Uday Naik	
//						and coding convention compliance
//
/////////////////////////////////////////////////////////////////////////////////////

#ifndef _CSIX_TX_H
#define _CSIX_TX_H


#ifdef USE_IMPORT_VAR

.import_var CSIX_TX_CTX_SRAM_BASE
.import_var CSIX_TX_COUNTERS_SRAM_BASE
.import_var THIS_BLADE_ID
.import_var SRAM_ZERO_BLOCK

#else

/* dl_system.h */


#endif
/////////////////////////////////////////////////////////////////////////////////////

// specify that the meta data is cached in dram xfer registers

#define DL_META_DATA_IN_DXFER		

/////////////////////////////////////////////////////////////////////////////////////
// define the offset in bytes into local memory where we store some globals

#define 		GLOBAL_LM_BASE		2048

// define the offset in bytes into local memory where we store the contexts

#define			TXC_LM_BASE			0

/////////////////////////////////////////////////////////////////////////////////////
// 
// Globals stored in local memory 
//
/////////////////////////////////////////////////////////////////////////////////////

//	We will use local memory to store a few globals. This is better than using 
//  absolute registers if we have a l$index unused because it reduces register
//	pressure and also makes it easier to run the code on fewer threads

#define	GLOBAL_AVAIL_TBUF_ELEMENT_INDEX		0	; next available TBUF element
#define	GLOBAL_MASK_BD_BYTES_SENT_INDEX		1	; mask to get bytes sent from TXC
#define GLOBAL_CTW_VD_BASE_INDEX			2	; base to compute 
												; TBUF_ELEMENT_CTRL_V# address
#define GLOBAL_TBUF_BASE_INDEX				3	; base to compute tbuf address
#define GLOBAL_NUMBER_128_INDEX				4	; constant for sending signals

/////////////////////////////////////////////////////////////////////////////////////

// Signal declaration

/////////////////////////////////////////////////////////////////////////////////////

.sig 	scratch_read_signal 			; signal for scratch ring read
.sig 	meta_read_signal  				; signal for reading meta data from sram
.sig 	txc_read_signal					; signal for reading in txc
.sig 	txc_write_signal				; signal for writing out txc
.sig 	dram_to_tbuf_signal				; signal for writing from dram to tbuf 
.sig 	tcw_write_signal				; signal for writing transmit control word
.sig 	tmh_write_signal				; signal for writing TM header to tbuf


.sig	volatile next_thread_signal_1			; signal for phase 1
.sig	volatile next_thread_signal_2			; signal for phase 2

// signal number for manual signal allocation

#define _TCW_WRITE_SIGNAL			1
#define _TMH_WRITE_SIGNAL			2
#define	_SCRATCH_READ_SIGNAL 		3
#define	_NEXT_THREAD_SIGNAL2 		4
#define _META_READ_SIGNAL			5
#define _DRAM_TO_TBUF_SIGNAL_1		6
#define _DRAM_TO_TBUF_SIGNAL_2		7
#define _NEXT_THREAD_SIGNAL1		8
#define _TXC_READ_SIGNAL			9
#define _TXC_WRITE_SIGNAL			11
#ifdef COUNTERS
// counter updating occurs in phase 1, so share with 
#define_eval _COUNTER_UPDATE_SIGNAL	13
#endif //COUNTERS
// signal mask for phase 1 
#if IS_IXPTYPE(__IXP28XX)  //NEW_28XX_CODE
   #define SIG_MASK_PHASE1				0x16
#else
   #define SIG_MASK_PHASE1				0x1e
#endif
// signal mask for phase 2


#define SIG_MASK_PHASE2				0x1e0

// signal number for manual signal allocation

.addr tcw_write_signal			_TCW_WRITE_SIGNAL			
.addr tmh_write_signal			_TMH_WRITE_SIGNAL
.addr scratch_read_signal		_SCRATCH_READ_SIGNAL 		
.addr next_thread_signal_1		_NEXT_THREAD_SIGNAL1 		
.addr meta_read_signal			_META_READ_SIGNAL			
.addr dram_to_tbuf_signal		_DRAM_TO_TBUF_SIGNAL_1		
.addr next_thread_signal_2		_NEXT_THREAD_SIGNAL2		
.addr txc_read_signal			_TXC_READ_SIGNAL				
.addr txc_write_signal			_TXC_WRITE_SIGNAL		
// This is the inter-ME signal generated by the ME whcih initializes common 
// resources, such as scratch ring, CSIX TX ME will wait for this signal at 
// end of it's init to make sure all common resource are available 

.sig volatile init_complete_signal		
.addr init_complete_signal			ME_INIT_SIGNAL	; defined as 10 in system.h 

#ifdef COUNTERS
.sig counter_update_signal			; signal for update counter
.addr counter_update_signal 		_COUNTER_UPDATE_SIGNAL
#endif //#ifdef COUNTERS		

/////////////////////////////////////////////////////////////////////////////////////



// To get CAM entry# by right shifting 3 ANDing with 0b000001111 (0b000001111000)

#define CAM_ENTRY_MASK	 				0xf
#define CAM_ENTRY_LOC	 				3

// definition of bit location of SOP bit in tx request 2nd longword 

#define SOP_BIT_LOC	30

// definition of bit location of SOP bit in tx request 2nd longword 

#define EOP_BIT_LOC	31

// definition of bit location of valid bd bit in trnasmit context 1st longword 

#define VALID_BD_BIT_LOC	16

// definition of valid bd (bit 16, or bit 0) 

#define VALID_BD_BYTE		0x1			; bit 0

// MSF_TX_CNTL

#if !IS_IXPTYPE(__IXP28XX)

#define CSIX_TX_CTL_LW 	0x10410004	; 0001 0000 0100 0001 Tx enable, CSIX,  \
										; duplex Rx_CBus_Mode					\
										; 0000 0000 0000 0100 32 bits CWORD, 	\
										; CSIX idle,
#endif

// initial first transmit control word for first cell in one packet (includes 
// info for CSIX base header)

#define TCW0_FCIP_BASE_LW	0x00100001	; TM playload length = 0 bytes (will 
										; be overwritten in code), prepend 
										; length = 16 bytes, prepend offset=0,
										; payload offset = 0 (will be overwritten
										; in code), cframe type = 1 (unicast), 
										; other fields are zero


// initial first transmit control word for the cell which is not the first cell
// in one packet (includes info for CSIX base header)

#define TCW0_NOT_FCIP_BASE_LW	0x00080001	; TM playload length = 0 bytes (will 
											; be overwritten in code), prepend 
											; length = 8 bytes, prepend offset=0,
											; payload offset=0, cframe type = 1 
											; (unicast), other fields are zero

// for tbuf elment with is 128 bytes long, we have 48 tbuf elements for data
// (0 to 47) and 16 tbuf elements fo control 

#define MAX_AVAIL_DATA_TBUF		0x30
#define FIRST_AVAIL_DATA_TBUF	0

// definition for mask for CSIX unicast extension header

#define DESTINATION_FIELD_MASK	0x3F

#if IS_IXPTYPE(__IXP28XX)

#define CLASS_FIELD_MASK		0x3
#define DEST_FIELD_SHIFT		2

#else

#define CLASS_FIELD_MASK		0x0F
#define DEST_FIELD_SHIFT		4

#endif

#define CLASS_FIELD_SHIFT		24
  
// definition of size of fully loaded cframe TM payload in bytes
// it will be 112 = 128 (cframe payload size) - 16 (TM header size)
// for the first cell in one packet and
// 120 = 128 (cframe payload size) - 8 (TM header size)
// for cell except the first cell in one packet = 120

#define FULL_TM_PAYLOAD_SIZE_FCIP		112		
#define FULL_TM_PAYLOAD_SIZE_NOT_FCIP	120		

// mask to get cell count remained in this buffer from second long word of
// tx request (this field occupies 6 bits in current meta data)

#define CELL_COUNT_REMAINED_MASK	0x0000003F

// defniton of mask to get egress outpu port from meta data
#define L2_PORT_MASK_LW		0xFF000000

// definition of mask to get freelist id from meta data

#define FREELIST_MASK		0x000F

// definition of the base bits pattern of indir_ref used in instruction which 
// moves packet data from dram to tbuf

#define DRAM_INDIREF_BASE_LW	0x02000010	; overrid bit of Ref_Cnt field (bit
											; 25) override bit of RBUF/TBUF 
											; Byte addr field (bit 4) and must 
											; always be set

// definition of CAM state

#define CAM_STATE_INIT		0    	; for initialization
#define CAM_STATE_VALID		1		; not used now

// The 7th long word of transmit context in local memory keep the last 
// dl_buffer_handle for this queue, it will be initialized to an address
// which is never possible be used as dl_buffer_handle at the initialization 
// to make the initial 16 CAM entries dummy. This means, any CAM hit for these
// initial 16 CAM entries still requires the code to read the meta data from 
// sram as the result of the comparison of it's actual dl_buffer_handle to 
// TXC[7] in local memory. Here we set it to be 0xFF, since dl_buffer_handle
// should be multiles of 32. 
// When the last cframe of one buffer is processed, we also set this TXC[7] 
// to 0xF3, so the code will prevent one new buffer in later runs to use stale
// meta data in local memory, in the case that the transmit context for this 
// queue stays in local memory and the new buffer uses the same 
// dl_buffer_handle as the previous buffer for this queue (the chance of this
// condition to happen should be very low).

#define IMPOSSIBLE_DL_HDL_ADDR	0xFF

// definition of mask to get dl_buf_handle 

#define MASK_FOR_DL_BH_LW	0x03FFFFFF	
#define TRUE_DL_HDL_SHIFT	2

// definition of mask to put freelist in bit 27 to bit 24 from TXC *l$index0[3]

#define MASK_FOR_FREELIST_LW	0x0F000000
	
// definition of lowest bit location of the  remaining cell count in second 
// long word of tx request

#define CELL_COUNT_LOC	24

// since each TXC in sram has 32 bytes, so queue number left shifting 5 bits 
// will give the offset of this TXC entry to sram_txc_base

#define SRAM_TXC_SHIFT	5

//MASK to set the bd valid bit = 0 and bytes already sent in this buffer to 0

#define MASK_OUT_BD_BYTES_SENT_LW 0xFFFE0000

// since CSIX TX microblock doesn't need to refer to next neighbor, following 
// definition is dummy, just to eliminate a warning from dispatch_loop.h

#define DL_NEXT_ME	0

// Transmit context definition 

#define 	TXC_BYTES_SENT_INDEX				0
#define 	TXC_BD_VALID_INDEX					0
#define 	TXC_SEQ_NUMBER_INDEX				1
#define 	TXC_NEXTHOP_CLASS_ID_INDEX			2
#define 	TXC_DRAM_TO_TBUF_SIZE_INDEX			3
#define 	TXC_OFFSET_INDEX					3
#define 	TXC_BUFFER_HANDLE_INDEX				7

// flags for cell 

#define 	SOP_EOP								0	; SOP and EOP bits are set
#define 	MOP_EOP								1	; either MOP or EOP cell
#define 	SOP_ONLY							2	; SOP only cell


/////////////////////////////////////////////////////////////////////////////
// Follow definitions are used for global counters
// There are total 1024 VOQss.  Each VOQ  has a set of counters.
// The counter data structure contains 4 long words
//	PKT_TRANSMITTED_CNT
//	BYTE_TRANSMITTED_CNT
//	RESERVED_CNT
//  The index of the counter data is defiend by CSIX_TX_COUNTER_SRAM_BASE + 
//    voq_offset
//	voq_offset = ((THIS_BLADE_ID << 4)  | (class_id)) << 4 
//	Following are the counter index definitions

#ifdef COUNTERS
#define		PKT_TRANSMITTED_CNT_IDX		0x00
#define		BYTE_TRANSMITTED_CNT_IDX	0x04
#define		CFRAME_TRANSMITTED_CNT_IDX	0x08
#define 	RESERVED_CNT_IDX			0x0C

// misc. definition related to counters updating
// definition of least significant bit location of payload length field
#define	TCW0_PAYLOAD_LEN_LSB_LOC	24
// definition of least significant bit location of prepend length field 
#define TCWO_PREPEND_LEN_LSB_LOC	16
// mask to get prepend length in tcw0
#define TCW0_PREPEND_MASK			0x1F
// mask to get EOP bit and cell count remained in tx_request1
#define EOP_CELL_COUNT_MASK			0xBF
// EOP_CELL_COUNT field for eop cframe
#define EOP_EOP_CELL_COUNT			0x80
#endif // COUNTERS

/////////////////////////////////////////////////////////////////////////////////////
		
#endif // _CSIX_TX_H

/////////////////////////////////////////////////////////////////////////////////////