/////////////////////////////////////////////////////////////////////////////////////
//
//
//                  I N T E L   P R O P R I E T A R Y
//
//     COPYRIGHT [c]  2002 BY  INTEL  CORPORATION.  ALL RIGHTS
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT
//     THE PRIOR WRITTEN PERMISSION OF :
//
//                        INTEL  CORPORATION
//
//                     2200 MISSION COLLEGE BLVD
//
//               SANTA  CLARA,  CALIFORNIA  95052-8119
//
/////////////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: csix_tx2.uc
//
//      Purpose: a context pipe-stage microblock which receives framed transmit request
//				 from previous ME and then send CSIX payload (TM header and payload from packet
//               buffer) and CSIX headers through MSF Transmit State Machine.
//
/////////////////////////////////////////////////////////////////////////////////////

#ifndef __CSIX_TX2_UC__
#define __CSIX_TX2_UC__

/*

The algorithm for this ME is


while (1)
	{
		for (i=0; I < max_phase; i++)
			{
				phase# i:
					issue tbuf validate of cell read in previous iteration and generate signal tbuf_validate_i
					wait for signals  tbuf_validate_i, dram_(i+1), prepend_(i+1)
					read a cell transmit request from NN ring
					grab a tbuf for the mapacket
					issue dram to tbuf transfer request and generate signal dram_i
					add prepend header and generate signal prepend_i
					branch to phase_(i+1);
			}
			// here (i+1) means (i+1) mod max_phase;

	}

*/

// NN ring msg format recevied from prev ME
// lw0 - tx_request0
// lw1 - dram_addr
// lw2 - 0:15 payload_len , 16 (0 is SOP_EOP), 17 ( 1 if MOP_EOP) 24:31 - cell_count_remaining
// lw3-6 - prepend0-3

// include stdmac.uc in IXPblocks Portable library

#include <stdmac.uc>

// include local memory macros in IXPblocks 

#include <localmem.uc>

// global definition of common constants which are shared by all microblocks

#include "dl_system.h"

// definition of hardware register address 

#include <hardware.h>

// definition of constants used

#include "csix_tx.h"

// definition of macros used

#include "csix_tx_util2.uc"

// initialization code for CSIX TX

#include "csix_tx_init2.uc"

// dispatch loop macros

#include "dispatch_loop.uc"

#ifdef	_DEBUG_COUNTERS_
// The following counters are stored in absolute registers. These counters are
// primarily used in debugging.
.reg	@csix_tx_num_cells_txed			// number of C-Frames transmitted
.reg	@csix_tx_num_buf_freed			// number of buffers freed
.reg	@csix_tx_num_def_buf_freed		// number of buffers freed to sram
#endif
														
/////////////////////////////////////////////////////////////////////////////////////
//
// Packet processing entry point for csix microblock
//
//
/////////////////////////////////////////////////////////////////////////////////////

#macro csix_tx2[]

.begin

	// defining number of interleaving phases -- increase number of phases to hide greater latencies.
	// valid values are >= 1
	#ifndef INTERLEAVING_PHASES
	#define INTERLEAVING_PHASES 1
	#endif

	// the code works in several phases. The dram signal generated by phase i is waited 
	// on phase (i-1) mod n. This gives the dram access sufficient time to complete. The allowed
	// latency for dram access = cycle budget * 8 * number of interleaving phases.

	// definitions of registers required for each phase
	#define_eval i 0
	
	#while (i < INTERLEAVING_PHASES)
		.reg tcw_/**/i
		.reg ex_header_/**/i
		.reg tx_request1_/**/i
		.reg cell_count_remaining_/**/i
		.reg default_sigmask_/**/i
		.reg sigmask_/**/i
		.reg addr_of_tx_control_/**/i
		#define_eval i (i + 1)
	#endloop
	#undef i


	.reg		tcw_fcip_base			; base for tcw for first cell of packet
	.reg		tcw_nfcip_base			; base for tcw for not first cell of packet
	.reg		indiref_base			; base for indirect ref of dram 
	.reg		lm_global_base			; base in local memory for globals 

	// defintions of registers 
	.reg 		dram_addr 
	.reg 		payload_length 
	.reg 		sop_eop_reg
	.reg 		indir_ref tmp_addr_of_tbuf
	.reg 		tx_request0
	.reg 		addr_of_tbuf

#ifdef	MULTICAST_SUPPORT_ENABLE

	.reg		mcast_reserved_voq
	.reg		mcast_dst_id

#endif


	// define signals for each phase
	#define_eval i 0

	#while (i < INTERLEAVING_PHASES)
		.sig dram_signal_/**/i
		.sig prepend_signal_/**/i		

		#define_eval i (i + 1)
	#endloop
	#undef i

	.sig tcw_write_signal


	// initialize local memory index 1 to the base for globals 

	immed32[lm_global_base, GLOBAL_LM_BASE]

	// 3 cycles for this to take effect

	localmem_set_address[0, lm_global_base, LM_HANDLE_1]

	nop
	nop
	nop

	// gprs initialized so they may save some cycles 

	_csix_tx_init_gprs2()


	// assign values to default sigmasks and sigmasks
	#define_eval i 0
	#while (i < INTERLEAVING_PHASES)
		#define_eval next_i ( ( i + 1) % INTERLEAVING_PHASES)
		#define_eval prev_i ( ( i + INTERLEAVING_PHASES - 1) % INTERLEAVING_PHASES)
		
		
		alu[default_sigmask_/**/i, --, b, 0]
		alu[sigmask_/**/i, --, b, 0]
		_csix_tx_set_signal[default_sigmask_/**/i, dram_signal_/**/next_i]
		_csix_tx_set_signal[default_sigmask_/**/i, dram_signal_/**/next_i + 1]
		_csix_tx_set_signal[default_sigmask_/**/i, prepend_signal_/**/next_i]
		_csix_tx_set_signal[default_sigmask_/**/i, tcw_write_signal]
		
		// for the first time, sigmasks only has tcw_write_signals
		_csix_tx_set_signal[sigmask_/**/i, tcw_write_signal]
		
			
		#define_eval i (i + 1)
	#endloop
	#undef i

	#define_eval last (INTERLEAVING_PHASES -1)

	// the last mask will initially wait for all signals
	alu[sigmask_/**/last, --, b, default_sigmask_/**/last]

	// perform .set to avoid warnings and other initializations
	
	#define_eval i 0
	#while (i < INTERLEAVING_PHASES)
		
		
		alu[tx_request1_/**/i, --, b, 0]
		.set tcw_/**/i ex_header_/**/i cell_count_remaining_/**/i addr_of_tx_control_/**/i
			
		#define_eval i (i + 1)
	#endloop
	#undef i

	alu[tx_request0, --, b, 0]

	
	// now the main loop
	
	// allocate transfer regs for tcw
	xbuf_alloc[$tcw, 2, write]	; This line doesn't produce any code.


	// main code starts here for each phase

	#define_eval i 0
	#while (i < INTERLEAVING_PHASES)

		// define previous and next phase
		#define_eval next_i ( ( i + 1) % INTERLEAVING_PHASES)
		#define_eval prev_i ( ( i + INTERLEAVING_PHASES - 1) % INTERLEAVING_PHASES)

	
	phase/**/i/**/#:
		.io_completed tcw_write_signal dram_signal_/**/i prepend_signal_/**/i
		
		// check if we had a request in prev iteration
		alu[--, tx_request1_/**/i, -, 0]
		beq[no_transmit_/**/i/**/#]
		
		// transmit the tbuf for packet. This macro also checks if the 
		// buffer handle is no longer required and if so it frees it.

		_csix_tx_transmit_tbuf[ $tcw, addr_of_tx_control_/**/i, tcw_/**/i, ex_header_/**/i, \
							tx_request1_/**/i, cell_count_remaining_/**/i, \
							tcw_write_signal]

		read_next_request_/**/i/**/#:
		
		br_inp_state[NN_EMPTY, no_request_/**/i/**/#]
		
		// read NN request
		xbuf_alloc($prepend_/**/i, 4, write)
		read_nn_request(tx_request0, tx_request1_/**/i, dram_addr, payload_length, sop_eop_reg, cell_count_remaining_/**/i, $prepend_/**/i)
	
		
		// check sop_eop bits
		alu[--, --, b, sop_eop_reg, >>1]
		bne[not_sop_/**/i/**/#]

		// min packet path
		// allocate a new tbuf 
		_csix_tx_get_next_tbuf[addr_of_tbuf, addr_of_tx_control_/**/i]

		// calculate tbuf address
		alu[tmp_addr_of_tbuf, addr_of_tbuf, +, 16]
		alu[indir_ref, indiref_base, OR, tmp_addr_of_tbuf, <<5]

		// get tcw
		_csix_tx_get_transmit_control_word[tcw_/**/i, tcw_fcip_base, 0, payload_length]
	
		// set indirect_ref for dram to tbuf transger
		alu[payload_length, payload_length, -, 0x01]
	  	alu_shf[payload_length, --, B, payload_length, >>3]

		alu[--, indir_ref, OR, payload_length, <<21] 

		// issue dram read

		dram[tbuf_wr, --, dram_addr, 0, max_15], indirect_ref, sig_done[dram_signal_/**/i]

		// issue prepend write

		msf[write, $prepend_/**/i/**/[0], addr_of_tbuf, 0, 4], sig_done[prepend_signal_/**/i]
	
#ifdef	MULTICAST_SUPPORT_ENABLE

		// get the extension header (and update base header) to be used in next iteration
		_csix_tx_cframe_header[ tcw_/**/i, ex_header_/**/i, tx_request0]

#else	
	
		// get the extension header to be used in next iteration
		_csix_tx_get_extension_header[ ex_header_/**/i, tx_request0]

#endif

		wait_for_signals_/**/i/**/#:
			
			// wait for dram and prepend access of (i+1) phase and tbuf validate of ith phase
			local_csr_wr[active_ctx_wakeup_events, sigmask_/**/i]
			ctx_arb[--] , br[phase/**/next_i/**/#], defer[1]
			alu[sigmask_/**/i, --, b, default_sigmask_/**/i]

	no_transmit_/**/i/**/#:
		// no request read in previous iteration
		_csix_tx_clear_signal[sigmask_/**/i, tcw_write_signal]
		br[read_next_request_/**/i/**/#]

	no_request_/**/i/**/#:
		// no request available for this iteration
		alu[tx_request1_/**/i, --, b, 0]
		_csix_tx_clear_signal[sigmask_/**/prev_i, dram_signal_/**/i]
		_csix_tx_clear_signal[sigmask_/**/prev_i, dram_signal_/**/i + 1]
		_csix_tx_clear_signal[sigmask_/**/prev_i, prepend_signal_/**/i]
		br[wait_for_signals_/**/i/**/#]

	not_sop_/**/i/**/#:

		// non min packet case
	
		// allocate a new tbuf (9)

		_csix_tx_get_next_tbuf[addr_of_tbuf, addr_of_tx_control_/**/i]

		// calc tbuf address
		alu[tmp_addr_of_tbuf, addr_of_tbuf, +, 8]
		alu[indir_ref, indiref_base, OR, tmp_addr_of_tbuf, <<5]

		// get tcw
		_csix_tx_get_transmit_control_word[tcw_/**/i, tcw_nfcip_base, 0, payload_length]
	
		// set indirect_ref for dram to tbuf transfer
		alu[payload_length, payload_length, -, 0x01]
	  	alu_shf[payload_length, --, B, payload_length, >>3]
		alu[--, indir_ref, OR, payload_length, <<21] 

		// issue dram to tbuf transfer
		dram[tbuf_wr, --, dram_addr, 0, max_15], indirect_ref, sig_done[dram_signal_/**/i]
		
		// write prepend header
		msf[write, $prepend_/**/i/**//**/[0], addr_of_tbuf, 0, 2], sig_done[prepend_signal_/**/i]
	
#ifdef	MULTICAST_SUPPORT_ENABLE

		// get the extension header (and update base header) to be used in next iteration
		_csix_tx_cframe_header[ tcw_/**/i, ex_header_/**/i, tx_request0]

#else	
	
		// get the extension header to be used in next iteration
		_csix_tx_get_extension_header[ ex_header_/**/i, tx_request0]

#endif

		br[wait_for_signals_/**/i#]


		#define_eval i (i + 1)
	#endloop
	#undef i

	xbuf_free($tcw)

	

.end

#endm


/////////////////////////////////////////////////////////////////////////////////////

// The main code starts here

main#:

	// initialize the microblock

	csix_tx_init2[]

	csix_tx2[]

/////////////////////////////////////////////////////////////////////////////////////

#endif 		// __CSIX_TX2_UC__

/////////////////////////////////////////////////////////////////////////////////////

