/////////////////////////////////////////////////////////////////////////////////////
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//
/////////////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: csix_tx_init.uc
//
//      Purpose: Initialization code for CSIX TX
//
//      History:
//
//      Date            Comment                  				       By
//      ---------------------------------------------------------------------
//
//      01/11/2002      Created                  				       David Chou
//
//		03/25/02		modified for packet interleaving
//						and coding convention						   Uday Naik
//
/////////////////////////////////////////////////////////////////////////////////////

#ifndef __CSIX_TX_INIT2_UC__
#define __CSIX_TX_INIT2_UC__


/////////////////////////////////////////////////////////////////////////////////////
//
// Following definition enable CSIX TX waiting for an inter-ME signal 
// generated by the ME whcih initializes common resources, such as scratch ring,
// near the end of thread 0 of CSIX TX initialization to make sure all common 
// resource are available 
//
/////////////////////////////////////////////////////////////////////////////////////

 
#define WAIT_FOR_COMMON_RESOURCE_INITIALIZATION 	


/////////////////////////////////////////////////////////////////////////////////////
// 
// _csix_tx_msf_tx_control_init
//
// Description:
// 	
//		Initialize the msf
//
// Outputs: 
//
//
// Inputs:  
//
//		
// Constants
//
//		None
//
// Size: 
//
//		5 instructions
// 
//
/////////////////////////////////////////////////////////////////////////////////////

#macro _csix_tx_msf_tx_control_init[]

.begin 

	.reg 	temp_reg 
	.reg	address 
	.reg	$data
	.sig	write_tx_control_signal	

	/* set the configuration before enabling tx */

	move[temp_reg, 0x00100905]
	alu[$data,	--, B, temp_reg]
	alu[address,	--, B, MSF_TX_CNTL]
	msf[write, $data, address, 0, 1], ctx_swap[write_tx_control_signal]

	#if !IS_IXPTYPE(__IXP28XX)

	/* configure the tx up cntl */

	move[temp_reg, 0xd]
	alu[$data,	--, B, temp_reg]
	alu[address,	--, B, TX_UP_CTRL_0]
	msf[write, $data, address, 0, 1], ctx_swap[write_tx_control_signal]

	#endif

	/* now enable the tx */

	move[temp_reg, 0x80110D05]
	alu[$data,	--, B, temp_reg]
	alu[address,	--, B, MSF_TX_CNTL]

	msf[write, $data, address, 0, 1], ctx_swap[write_tx_control_signal]

	//	IXP2800 needs an extra regsiter to be initialised.
	//	This code needs to be moved to a separate macro. [TBD]

	#if IS_IXPTYPE(__IXP28XX)

	alu[$data,	--, B, 0]
	alu[address,	--, B, SRB_OVERRIDE]

	msf[write, $data, address, 0, 1], ctx_swap[write_tx_control_signal]

	#endif

.end 

#endm	


/////////////////////////////////////////////////////////////////////////////////////
// 
// _csix_tx_init_globals
//
// Description:
// 	
//		Initialize the globals stored in local memory
//
// Outputs: 
//
//
// Inputs:  
//
//		
// Constants
//
//		None
//
// Size: 
//
//		14 instructions
// 
//
/////////////////////////////////////////////////////////////////////////////////////

#macro _csix_tx_init_globals[]

.begin
	
	.reg	lm_global_base

	// initialize local memory index 1 to the base for globals 

	immed32[lm_global_base, GLOBAL_LM_BASE]

	// 3 cycles for this to take effect

	localmem_set_address[0, lm_global_base, LM_HANDLE_1]

	nop
	nop
	nop

	// available tbuf element initialized to 0

	move[*l$index1[GLOBAL_AVAIL_TBUF_ELEMENT_INDEX], 0]	
   
	// initialized to 128 to help in signal computation

	move[*l$index1[GLOBAL_NUMBER_128_INDEX], 0x80]

	// mask to set the bd valid bit = 0 and bytes already sent in this buffer to 0

    move[*l$index1[GLOBAL_MASK_BD_BYTES_SENT_INDEX], MASK_OUT_BD_BYTES_SENT_LW]

	// starting address of TBUF_ELEMENT_CTRL_V_# 

   	move[*l$index1[GLOBAL_CTW_VD_BASE_INDEX], TBUF_ELEMENT_CTRL_VALID]

 	// starting address of TBUF 

    move[*l$index1[GLOBAL_TBUF_BASE_INDEX], RBUF_TBUF_BASE] 
	
.end

#endm



/////////////////////////////////////////////////////////////////////////////////////
// 
// csix_tx_init[]
//
// Description:
// 	
//		Initialization of the microblock
//
// Outputs: 
//
//
// Inputs:  
//
//		
// Constants
//
//		None
//
// Size: 
//
//		14 instructions
// 
//
/////////////////////////////////////////////////////////////////////////////////////

#macro csix_tx_init2[]

.begin

	.reg 	sig
	
	// For thread 0 we will do some initialization. every one else waits

	br!=ctx[0, EXIT#]

ctx0_init#:
	
	// init NN ring
	.reg ctx_enable
	local_csr_rd[ctx_enables]
	immed[ctx_enable, 0]
	alu_shf[ctx_enable, ctx_enable, AND~, 3, <<18]
	alu_shf[ctx_enable, ctx_enable, OR, 3, <<18]
	local_csr_wr[ctx_enables, ctx_enable]
	local_csr_wr[nn_put, 0]
	local_csr_wr[nn_get, 0]	
	alu[--, --, b, 0]
	alu[--, --, b, 0]
	alu[--, --, b, 0]

    // Initialize MSF_TX_CNTL register

	_csix_tx_msf_tx_control_init[]

    // start to initialize global GPRs

	_csix_tx_init_globals[]

  	// initialize the debug counters

	_csix_tx_debug_init_counters[]
    

	// following inter-ME signal common_init_complete_sig_num is generated by 
	// the ME whcih initializes common resources, such as scratch ring, CSIX TX
	// ME will wait for this signal to make sure all common resource are 
	// available, when it start CSIX TX processing

#ifdef WAIT_FOR_COMMON_RESOURCE_INITIALIZATION 

	ctx_arb[init_complete_signal]

#endif 


    // For kick start thread 0 will send itself 2 initial signals for 
	// phase 1 and 2 
/*
	immed[sig, _NEXT_THREAD_SIGNAL1]		; prepare value for signaling
    alu_shf[sig, 0x0, OR, sig, <<3]         ; value to signal thread 0
    local_csr_wr[SAME_ME_SIGNAL, sig]   	; signal thread 0

	immed[sig, _NEXT_THREAD_SIGNAL2]		; prepare value for signaling
    alu_shf[sig, 0x0, OR, sig, <<3]         ; value to signal thread 0
    local_csr_wr[SAME_ME_SIGNAL, sig]   	; signal thread 0
*/
	
EXIT#:


.end

#endm  

/////////////////////////////////////////////////////////////////////////////////////

#endif 		//  __CSIX_TX_INIT2_UC__

/////////////////////////////////////////////////////////////////////////////////////
