////////////////////////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
////////////////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: mphy16_tx.h
//
//      Purpose: Header file for PACKRTET TX on MPHY-16 mode 
//
//      History:
//
//
//      Date            Comment                         			By
//      --------------------------------------------------------------------------------	
//
//      03/08/2002      Created                         			David Chou
//
//
/////////////////////////////////////////////////////////////////////////////////////////


#ifndef	__MPHY16_TX_H__
#define	__MPHY16_TX_H__

#ifndef TOTAL_TBUF_SIZE
#define TOTAL_TBUF_SIZE 			8192
#endif	// #ifndef TOTAL_TBUF_SIZE

#ifndef TBUF_ELE_SIZE
#define TBUF_ELE_SIZE	128
#endif	// #ifndef TBUF_ELE_SIZE

#if ((TBUF_ELE_SIZE != 64) && (TBUF_ELE_SIZE != 128))
#error	"**Warning! TBUF_ELE_SIZE must be either 64, or 128."
#endif	// #if ((TBUF_ELE_SIZE != 64) && (TBUF_ELE_SIZE != 128))

#if (TBUF_ELE_SIZE == 128)
#define TBUF_ADDR_SHF		7
#endif 	// #if (TBUF_ELE_SIZE == 128)

#if (TBUF_ELE_SIZE == 64)
#define TBUF_ADDR_SHF		6
#endif	// #if (TBUF_ELE_SIZE == 64)

// MSF_TX_CNTL
#if (TBUF_ELE_SIZE == 128)
#define PACKET_TX_MPHY_16_CTL_LW	0x100E0004	;0001 0000 0000 1110 Tx enable, UTOPIA/POS, MPHY-16
											;0000 0000 0000 0100 128 bytes tbuf size
#endif 	// #if (TBUF_ELE_SIZE == 128)
#if (TBUF_ELE_SIZE == 64)
#define PACKET_TX_MPHY_16_CTL_LW	0x100E0000	;0001 0000 0000 1110 Tx enable, UTOPIA/POS, MPHY-16
											;0000 0000 0000 0000 64 bytes tbuf size
#endif	// #if (TBUF_ELE_SIZE == 64)


#if IS_IXPTYPE(__IXP28XX)
#if (TBUF_ELE_SIZE == 128)
#define PACKET_TX_SPHY_CTL_LW	0x20000454
#endif 	// #if (TBUF_ELE_SIZE == 128)
#if (TBUF_ELE_SIZE == 64)
#define PACKET_TX_SPHY_CTL_LW	0x20000450
#endif	// #if (TBUF_ELE_SIZE == 64)
#define ELE_2_TX_CNTRL_ADDR	4
#else // #ifdef IXP2800
#if (TBUF_ELE_SIZE == 128)
#define PACKET_TX_SPHY_CTL_LW	0x10000004	;0001 0000 0000 0000 Tx enable, UTOPIA/POS, SPHY, Duplex
										;0000 0000 0000 0100 128 bytes tbuf size
#endif 	// #if (TBUF_ELE_SIZE == 128)
#if (TBUF_ELE_SIZE == 64)
#define PACKET_TX_SPHY_CTL_LW	0x10000000	;0001 0000 0000 0000 Tx enable, UTOPIA/POS, SPHY, Duplex
										;0000 0000 0000 0000 64 bytes tbuf size
#endif	// #if (TBUF_ELE_SIZE == 64)
#endif // #ifdef IXP2800


// definition of the threshold to wait to avoid overwriting existing mpackets
// not transmitted out of tbuf yet
#ifndef TBUF_WAIT_THRESHOLD
#if (TBUF_ELE_SIZE == 128)
#define TBUF_WAIT_THRESHOLD        32
#endif 	// #if (TBUF_ELE_SIZE == 128)
#if (TBUF_ELE_SIZE == 64)
#define TBUF_WAIT_THRESHOLD        64
#endif 	// #if (TBUF_ELE_SIZE == 64)
#endif	 // #ifndef TBUF_WAIT_THRESHOLD

#define_eval TBUF_ELE_NUM	(TOTAL_TBUF_SIZE / TBUF_ELE_SIZE)

#ifndef PACKET_TX_PORT_BASE
#define PACKET_TX_PORT_BASE	0
#endif	// #ifndef PACKET_TX_PORT_BASE

// POS_TX_UP_CTL_WD
#define POS_TX_UP_CTL_WD		0x001D	; packet, single bit even parity, 2 clock cycle
// for TX_UP_CTL  0x001C and 0x0018 generated same packets
//#define POS_TX_UP_CTL_WD		0x001C	; packet, single bit even parity, 1 clock cycle
// #define POS_TX_UP_CTL_WD		0x0018	; packet, single bit odd parity, 1 clock cycle
#define ATM_TX_UP_CTL_WD		0x000C	; cell, single bit even parity, 1 clock cycle



// first transmit control word base for SOP_EOP mpacket 
// SOP, EOP bit set, payload offet = 0 (will be filled later),
// channel = 0, will be filled later
#define TCW0_SOP_EOP_BASE_W0 0x0300	
// payload len = 0 (will be filled later), prepend offset = 0, 
// prepend length = 2
#define TCW0_SOP_EOP_BASE_W1 0x0000
// definition of BASE for TCW0 for sop_only mpacket
#define TCW0_SOP_ONLY_BASE_LW	0x00000200

#define MPHY16_PACKET_TX_FIRST_ME		6
#define MPHY16_PACKET_TX_SECOND_ME	7

#ifndef PORTS_CTRLED_BY_FIRST_PACKET_TX_ME
// ports controlled by first POS TX ME (port 0 to 7)
#define			PORTS_CTRLED_BY_FIRST_PACKET_TX_ME		0x00FF
#endif	// #ifndef PORTS_CTRLED_BY_FIRST_PACKET_TX_ME

#ifndef PORTS_CTRLED_BY_SECOND_POS_TX_ME
// ports controlled by second POS TX ME (port 8 to 15)
#define			PORTS_CTRLED_BY_SECOND_POS_TX_ME	0xFF00
#endif 	// #ifndef PORTS_CTRLED_BY_SECOND_POS_TX_ME

#define_eval TBUF_SIZE_MINUS_8 (TBUF_ELE_SIZE -8)

// definition of the mask to get the next available tbuf element from what
// gotten from scratch memory by atomic operation
#define_eval TBUF_ELE_BITS_MASK ( (TOTAL_TBUF_SIZE / TBUF_ELE_SIZE) - 1)

//	Scratch for communicating between Queue Manager and POS Transmit Blocks
//	Ring number 3, base 8192, and size 512 Long Words

#if THIS_ME==MPHY16_PACKET_TX_FIRST_ME
#define			QM_TO_PACKET_TX_SCR_RING			PACKET_TX_IN_0
#endif // #if THIS_ME==MPHY16_PACKET_TX_FIRST_ME

#if THIS_ME==MPHY16_PACKET_TX_SECOND_ME
#define			QM_TO_PACKET_TX_SCR_RING			PACKET_TX_IN_1
#endif // #if THIS_ME==MPHY16_PACKET_TX_SECOND_ME

#if THIS_ME==MPHY16_PACKET_TX_FIRST_ME
#define			PORTS_CTRLED_BY_THIS_ME		(PORTS_CTRLED_BY_FIRST_PACKET_TX_ME)
#endif // #if THIS_ME==MPHY16_PACKET_TX_FIRST_ME

#if THIS_ME==MPHY16_PACKET_TX_SECOND_ME
#define			PORTS_CTRLED_BY_THIS_ME		(PORTS_CTRLED_BY_SECOND_POS_TX_ME)
#endif // #if THIS_ME==MPHY16_PACKET_TX_SECOND_ME

// bits definition in the gloabl execution status flag exe_stat_flag
// definition of the bit location for reading secondary buffer meta data in 
// phase 1, and need to be saved in phase 2 flag in exe_stat_flag GPR
#define READ_SND_BD_FLAG_BIT		0	

// definition of the bit location for some bytes leftover from previous buffer
// flag in exe_stat_flag GPR
#define LEFTOV_FROM_PB_FLAG_BIT		1	 

// definition of the bit location for some bytes leftover from previous buffer
// flag in exe_stat_flag GPR
#define SAVE_LEFTOV_TO_LM_FLAG_BIT	2	 

// definition of the bit location for reading enqueueing sop meta data in 
// phase 2, and need to be saved in phase 3 flag in exe_stat_flag GPR
#define READ_SOP_BD_FLAG_BIT		3


// definition of the bit location to identify taht the tbuf will not be
// transmitted  due to its paylo size less than 4 in exe_stat_flag GPR
#define SKIP_TRANSMIT_FLAG_BIT		4

// definition of the bit location to identify this is an EOB mpacket in 
// exe_stat_flag GPR, so this buffer needs to be freed in phase 3
#define EOB_MPKT_FLAG_BIT			5

// constants definition for queue data structure at local memory
// queue status info starting at 0x800 in local memory
#define QUEUE_INFO_BASE		0x800
 
#define QUEUE_ENTRY_BASE	0

// each port occpies 8 longword
// #define QUEUE_INFO_ENRY_SHFT	5
// each port occpies 16 longword
#define QUEUE_INFO_ENRY_SHFT	6

#define PKTS_IN_QUEUE_SHFT		16

// queue entres for each port occupies 64 lonwords 
#define QUEUE_ENRY_SHFT		8	

#define TAIL_MASK	0xFF
#define TAIL_SHFT	8

#define HEAD_MASK 	0xFF

#define PORT_MASK_IN_TXR	0x0F

// each entry occupies 5 longwords
// #define QUEUE_ENTRY_SIZE	0x14		; has problem with *l$indexn[1] when 
										; index set to odd lw
//#define QUEUE_ENTRY_SIZE	0x20
#define QUEUE_ENTRY_SIZE	0x10		

#define_eval QUEUE_DEPTH	(0x100 / QUEUE_ENTRY_SIZE)
  
#define_eval QUEUE_DEPTH_MINUS_ONE	(QUEUE_DEPTH - 1)

#define_eval HEAD_INCR (QUEUE_ENTRY_SIZE)
#define_eval MAX_PLUS_ONE_QUEUE_OFFSET	(QUEUE_ENTRY_SIZE * QUEUE_DEPTH)

#define_eval MAX_MINUS_ONE_QUEUE_OFFSET	(QUEUE_ENTRY_SIZE * QUEUE_DEPTH_MINUS_ONE)

#define CONST_FF		0xFF

#define CONST_FC		0xFC	

#define CONST_F8		0xF8

#define CONST_0F		0x0F

#define CONST_07		0x07	

#define CONST_04		0x04	



// amount to add to increase pkts_in_queue by one in queue status info
#define INCR_PKTS_IN_QUEUE	0x10000

// amount to add to increase tail_offet to next entry in queue status info
#define_eval INCR_TAILOFFSET		(QUEUE_ENTRY_SIZE <<8)

// bit location for sop flag in queue entry longword 0
#define SOP_BIT_LOC			31

// bit location read next BD flag in queue entry longword 0
#define RD_NBD_BIT_LOC		30

// mask for pkt_in_queue in queue info
#define QPKT_MASK	0x0F
// the least significant bit location of pkt_in_queue in queue info
#define QPKT_SHFT	16



// mask for freelist field
#define FREE_LIST_MASK		0xF

// the least significant bit location of freelist in meta data
#define FLIST_META_LOC		12

// the least significant bit location of freelist in queue entry
#define FLIST_QUEUE_LOC		24

// definition of EOP bit in Transmit control word for POS-PHY
#define TCW0_EOP_BIT_SHFT		8

// definition of SKIP bit in Transmit control word for POS-PHY
#define TCW0_SKIP_BIT_SHFT		11

// definition of mask to get freelist id from meta data
#define FREELIST_MASK		0x000F

// definition of the base bits pattern of indir_ref used in instruction which 
// moves packet data from dram to tbuf
#define DRAM_INDIREF_BASE_LW	0x02000010	; overrid bit of Ref_Cnt field (bit
											; 25) override bit of RBUF/TBUF 
											; Byte addr field (bit 4) and must 
											; always be set


// definition of mask to get dl_buf_handle 
#define TRUE_DL_HDL_SHIFT	2

// definition of mask to put freelist in bit 27 to bit 24 from TXC *l$index0[3]
#define MASK_FOR_FREELIST_LW	0x0F000000
	
// since CSIX TX microblock doesn't need to refer to next neighbor, following 
// definition is dummy, just to eliminate a warning from dispatch_loop.h
#define DL_NEXT_ME	0

// definition of the default PHY TXFIFO depth
// assume PHY has 2K bytes TXFIFO and we set the threshhold to 2 times 
// TBUF_ELE_SIZE
#define DEFAULT_PHY_DEPTH	((2048 / TBUF_ELE_SIZE) - 1)

// definition of the offset of different elements in the queue entry in local
// memory
// definition of the offset of the first element about active buffer  
#define ABD_0_OFFSET		0
// definition of the offset of the second element about active buffer  		
#define ABD_1_OFFSET		1
// definition of the offset of the first element about secondary buffer 
#define SBD_0_OFFSET		2		
// definition of the offset of the first element about secondary buffer
#define SBD_1_OFFSET		3

// definition of the offset of different elements in the queue status info in
// local memory
// definition of the offset of status element 
#define Q_STAT_OFFSET		0
// definition of the offset of remaining mpkt number without checking Tx 
// status element 
#define RMND_AL_MPKT_OFFSET	1
// definition of the offset of PHY Tx FIFO depth element
#define PHY_DEPTH_OFFSET	2
// definition of the offset of packets transmitted for this port element
#define PKT_TXED_OFFSET		3
// definition of the offset of leftover payload status element 
#define LP_STAT_OFFSET		4
// definition of the offset of the first longword of the leftover payload 
// element 
#define LP_LW0_OFFSET		5
// definition of the offset of the second longword of the leftover payload 
// element 
#define SBD_NEXT_BD_OFFSET	6


// definition of bit location of leftover from previous buffer flag in queue 
// status 
#define LP_FLAG_LOC			16


// definition of the least significant bit of the remained payload in queue 
// entry
#define PAYLO_RMND_LOC		16

// definition to the left shift needed on TBUF_ELE_SIZE to calculate the maximum
// bytes (max_bytes_to_move) to move from dram to tbuf for all 8 threads in one 
// ME transmit one mpackt, max_bytes_to_move = TBUF_ELE_SIZE * 8 
#define ME_THREAD_NUM_SHFT	3

#define PKTS_IN_Q_MASK	0xFF

/////////////////////////////////////////////////////////////////////////////
// Follow definitions are used for global counters
// There are total 16 ports.  Each port has a counter
// data.  The counter data structure contains 3 long words
//	PKT_TRANSMITTED_CNT
//	PKT_DROPPED_CNT
//	BYTE_TRANSMITTED_CNT
//	RESERVED_CNT
//  The index of the counter data is defiend by POS_TX_COUNTER_BASE+PORT_OFFSET
//	PORT_OFFSET	= port number << 4 
//	Following are the counter index definitions

#ifdef COUNTERS
#define		PKT_TRANSMITTED_CNT_IDX		0x00
#define		PKT_DROPPED_CNT_IDX			0x04
#define		BYTE_TRANSMITTED_CNT_IDX	0x08
#define 	RESERVED_CNT_IDX			0x0C
#define     PORT_MASK_IN_TCW            0x0F
#endif


// following from msf_regs.h

#define MSF_TX_CNTL				0x0004

#define TX_MPHY_STATUS			0x0074

#define TX_UP_CTRL_0			0x0090
#define TX_UP_CTRL_1			0x0094
#define TX_UP_CTRL_2			0x0098
#define TX_UP_CTRL_3			0x009c

#define TX_SEQUENCE_0			0x0060

// mask to get mpackets actually sent to compare with the next avalble tbuf 
// element number
//#define MPKTS_SENT_MASK		0xFF

// starting address of TBUF_ELEMENT_CTRL_V_# 
#define TBUF_ELEMENT_CTRL_V_LW	 0x00001800    
// starting address of RBUF/TBUF
#define RBUF_TBUF_LW		 0x00002000   

// definition of least significant bit location of fields of Transmit Control Word
// definition of least significant bit location of payload length field
#define	TCW0_PAYLOAD_LEN_LSB_LOC	24
// definition of least significant bit location of payload offset field
#define TCW0_PAYLOAD_OFFSET_LSB_LOC	13
// definition of least significant bit location of prepend length field 
#define TCWO_PREPEND_LEN_LSB_LOC		16
// definition of least significant bit location of prepend offest field  		
#define TCWO_PREPEND_OFFSET_LSB_LOC		21

// definition of the mask to get payload offset (3 bits)
#define TCW0_PAYLOAD_OFFSET_MASK		0x07

// debug
#ifdef DEBUG_TX_PENDING_LOCKED
#define SRAM_WRITE_1_SIG	11
#define FIRST_ME_TBUF_NO_SRAM_BASE		0x100000
#define SECOND_ME_TBUF_NO_SRAM_BASE		0x200000
#endif //#ifdef DEBUG_TX_PENDING_LOCKED


#endif	//	__MPHY16_TX_H__
