////////////////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
////////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: mphy16_tx_init.uc
//
//      Purpose: Initialization macros for PACKRTET TX on MPHY-16 mode 
//
//      History:
//
//
//      Date            Comment                         		By
//      -----------------------------------------------------------------------	
//
//      03/08/2002      Created									David Chou
//
//
///////////////////////////////////////////////////////////////////////////////


#ifndef	__MPHY16_TX_INIT_UC__
#define	__MPHY16_TX_INIT_UC__

#ifndef	MPHY16_PACKET_TX_FIRST_ME
#warning	"MPHY16_PACKET_TX_FIRST_ME not defined. Defaulting to ME 0."
#define	MPHY16_PACKET_TX_FIRST_ME 0
#endif

#ifndef	MPHY16_PACKET_TX_SECOND_ME
#warning	"MPHY16_PACKET_TX_SECOND_ME not defined. Defaulting to ME 1."
#define	MPHY16_PACKET_TX_SECOND_ME 1
#endif

#if	((MPHY16_PACKET_TX_SECOND_ME - MPHY16_PACKET_TX_FIRST_ME) <= 0)
#warning "MPHY16_PACKET_TX_SECOND_ME should be greater than MPHY16_PACKET_TX_FIRST_ME."
#endif

#ifndef	THIS_ME
#error	"**Error! You need to define THIS_ME in assember option."
#endif

#if ((MPHY16_PACKET_TX_FIRST_ME<0) || (MPHY16_PACKET_TX_FIRST_ME>8) || \
     (MPHY16_PACKET_TX_SECOND_ME<0)  || (MPHY16_PACKET_TX_SECOND_ME>8)  || \
	 (THIS_ME<0)     || (THIS_ME>8))
#error	"ME's range from 0-7."
#endif

#ifndef	TBUF_ELE_SIZE
#error	"**Warning! You need to define TBUF_ELE_SIZE."
#endif


#if ((TBUF_ELE_SIZE != 64) && (TBUF_ELE_SIZE != 128) && (TBUF_ELE_SIZE != 256))
#error	"**Warning! TBUF_ELE_SIZE must be 64, or 128, or 256."
#endif

#ifndef TBUF_ELE_SIZE
#define TBUF_ELE_SIZE 128
#error	"**TBUF_ELE_SIZE set to 128."

#endif


///////////////////////////////////////////////////////////////////////////////
// mphy16_tx_init[]
//
//	Description: MSF initialization, CAM initialization, and global variables
//               initialization
//                
//
// Outputs: 
//
//								None
//
// Inputs:  
//
//								None
//
// Constants
//
//								None
//
// Size: 
//
//		around 420  instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro mphy16_tx_init[]
	br=ctx[0, ctx0_init#]
    br=ctx[1, ctx1_init#]
    br=ctx[2, ctx2_init#]
    br=ctx[3, ctx3_init#]
    br=ctx[4, ctx4_init#]
    br=ctx[5, ctx5_init#]
    br=ctx[6, ctx6_init#]
    br=ctx[7, ctx7_init#]

// general ME and global GPRs initializtion and thread 0 context relative
// variables initialization
ctx0_init#:


#if THIS_ME==MPHY16_PACKET_TX_FIRST_ME
.begin 
.reg temp_reg, address, $data

    // Initialize MSF_TX_CNTL register
#ifndef SIMULATE_MPHY_16_WITH_SPHY	
	move[temp_reg, PACKET_TX_MPHY_16_CTL_LW]
#else // #ifndef SIMULATE_MPHY_16_WITH_SPHY	
	move[temp_reg, PACKET_TX_SPHY_CTL_LW]
#endif // #ifndef SIMULATE_MPHY_16_WITH_SPHY
	
	alu[$data,	--, B, temp_reg]
	alu[address,	--, B, MSF_TX_CNTL]

	msf[write, $data, address, 0, 1], sig_done[sig_msf_access_1]
	// swap out to wait for  msf write complete and my turn
	ctx_arb[sig_msf_access_1]

	// Initialization of Tx_UP_Control_0, Tx_UP_Control_1, Tx_UP_Control_2,
	// Tx_UP_Control_3 to POS
	_mphy16_tx_set_16bit_msf_reg[TX_UP_CTRL_0, POS_TX_UP_CTL_WD]
	// _mphy16_tx_set_16bit_msf_reg[TX_UP_CTRL_1, POS_TX_UP_CTL_WD]
	// _mphy16_tx_set_16bit_msf_reg[TX_UP_CTRL_2, POS_TX_UP_CTL_WD]
	// _mphy16_tx_set_16bit_msf_reg[TX_UP_CTRL_3, POS_TX_UP_CTL_WD]
	// Initialization of Tx_UP_Control_0, Tx_UP_Control_1, Tx_UP_Control_2,
	// Tx_UP_Control_3 to UTOPIA(ATM)    
    // _mphy16_tx_set_16bit_msf_reg[TX_UP_CTRL_0, ATM_TX_UP_CTL_WD]
    // _mphy16_tx_set_16bit_msf_reg[TX_UP_CTRL_1, ATM_TX_UP_CTL_WD]
    // _mphy16_tx_set_16bit_msf_reg[TX_UP_CTRL_2, ATM_TX_UP_CTL_WD]
    // _mphy16_tx_set_16bit_msf_reg[TX_UP_CTRL_3, ATM_TX_UP_CTL_WD]
    // end of MSF registers initialization

	// initialize the variable tbuf element in scratch memory 
	// location PACKET_TX_AVAIL_TBUF_ELE_SCR_ADDR to 0, later get and increment
	// available tbuf element by SCRATCH atomic operation SCRACTH[test_and_incr..]  
	alu[$data, --, b, 0]
	move[address, PACKET_TX_AVAIL_TBUF_ELE_SCR_ADDR]
	scratch[write, $data, address, 0, 1], sig_done[sig_scratch_access_2]
	ctx_arb[sig_scratch_access_2]

.end // end of .begin (start of a local area for local variables
    // temp_reg, address, $data)

#endif // #if THIS_ME==MPHY16_PACKET_TX_FIRST_ME

#ifdef DEBUG_TX_PENDING_LOCKED
#if THIS_ME==MPHY16_PACKET_TX_FIRST_ME
	move[@tbuf_no_addr, FIRST_ME_TBUF_NO_SRAM_BASE]
#else
	move[@tbuf_no_addr, SECOND_ME_TBUF_NO_SRAM_BASE]
#endif // #if THIS_ME==MPHY16_PACKET_TX_FIRST_ME
#endif //#ifdef DEBUG_TX_PENDING_LOCKED



	// initialize context relative GPRs
	// thread 0 context relative GPR variables initialization
    _mphy16_tx_thread_specific_init[tbuf_base, tbuf_scr_addr, sring_tr, \
			queue_stat_base, tx_mphy_stat_addr, \
			tx_seq_0_addr, indiref_base, sigmask_phs1_default, \
			sigmask_phs2_default, sig1_next_context_gpr, \
			sig2_next_context_gpr, sig3_next_context_gpr, 1]

    // start to initialize global GPRs

 	// absolute GPR @qstat_vector initialized to 0x0, no packets queued yet
 	move[@qstat_vector, 0x0]

 	// absolute GPR @busy_mask initialized to 0x0, no port busy yet
 	move[@busy_mask, 0x0]

	// starting address of TBUF_ELEMENT_CTRL_V_# 
   	move[@tcw_vd_base, TBUF_ELEMENT_CTRL_V_LW]
 
	// amount to add to increase pkts_in_queue by one in queue status info
	move[@qpkt_incr, INCR_PKTS_IN_QUEUE]	

	// amount to add to increase tail_offet to next entry in queue status info
    move[@tail_incr, INCR_TAILOFFSET]	

.begin 
.reg port_mask
	// bits set in this pot_mask means the transmisson of mpackets of that port
	// is handled by this ME
 	move[port_mask, PORTS_CTRLED_BY_THIS_ME]

 	// absolute GPR @rr_vector initialized to 0xFFFF
 	move[@rr_vector, 0xFFFF]
	alu[@rr_vector, @rr_vector, and, port_mask]
	
 	// absolute GPR @txc_left_paylo_vector initialized to 0xFFFF
 	move[@txc_left_paylo_vector, 0xFFFF]
	alu[@txc_left_paylo_vector, @txc_left_paylo_vector, and, port_mask]

	// absolute GPR @txc_rd_snd_bd_vector initialized to 0xFFFF
 	move[@txc_rd_snd_bd_vector, 0xFFFF]
	alu[@txc_rd_snd_bd_vector, @txc_rd_snd_bd_vector, and, port_mask]

	// wait for port to be ready for transmission
#ifndef SIMULATE_MPHY_16_WITH_SPHY
#if 1
.begin 
.reg tx_pending
.reg tx_status
.reg ready_ports
.reg $stat
wait_for_port_ready#:
	// waiting for port read

	msf[read, $stat, tx_mphy_stat_addr,  0, 1], sig_done[sig_msf_access_2]	
	ctx_arb[sig_msf_access_2]
	ld_field_w_clr[tx_pending, 0011, $stat, >>16]	; get tx_pending
	ld_field_w_clr[tx_status, 0011, $stat]	; get tx_status
	alu[ready_ports, tx_status, and~, tx_pending]
	alu[ready_ports, ready_ports, and, port_mask]
	alu[--, ready_ports, -, port_mask]
	bne[wait_for_port_ready#]
	alu[@mphy_stat_vector, --, b, ready_ports]
.end // tx_pending tx_status ready_ports $stat
#else
   move[@mphy_stat_vector, 0xFFFFFFFF]
	// absolute GPR @mphy_stat_vector_vector initialized to 0xFFFF
 	move[@mphy_stat_vector, 0xFFFF]
	alu[@mphy_stat_vector, @mphy_stat_vector, AND, port_mask]
#endif //#if 0

#else // #ifndef SIMULATE_MPHY_16_WITH_SPHY

    move[@mphy_stat_vector, 0xFFFFFFFF]
	// absolute GPR @mphy_stat_vector_vector initialized to 0xFFFF
 	move[@mphy_stat_vector, 0xFFFF]
	alu[@mphy_stat_vector, @mphy_stat_vector, AND, port_mask]
#endif // #ifndef SIMULATE_MPHY_16_WITH_SPHY

.end //port_mask

#ifdef UNIT_TEST
	move[@buf_freed, 0]
#endif // UNIT_TEST

#ifdef CHECK_DISCARD
	move[@pkts_discarded, 0]
#endif	//CHECK_DISCARD

#ifdef COUNTERS
	move[@cntr_base, POS_TX_COUNTER_BASE]	; initialized to POS_TCX_COUNTER_BASE	
#ifdef UNIT_TEST

#if THIS_ME==MPHY16_PACKET_TX_FIRST_ME	
	// initialize counters area for 16 ports in sram to 0
.begin
.reg cntr_addr ii ss
.reg $w0 $w1 $w2 $w3
.xfer_order $w0 $w1 $w2 $w3
	alu[cntr_addr, --, b, @cntr_base]
	alu[ii, --, b, 0]
	move[ss, 0] 
	alu[$w0, --, b, ss]  
	alu[$w1, --, b, ss]   
	alu[$w2, --, b, ss] 
	alu[$w3, --, b, ss]  
loop_init_counters_in_sram#:	 
	sram[write, $w0, cntr_addr, 0, 4], sig_done[sig_counter_update]
	ctx_arb[sig_counter_update]
	alu[--, ii, -, 16]
	beq[init_counters_done#]
	alu[ii, ii, +, 1]
	alu[cntr_addr, cntr_addr, +, 16]
	br[loop_init_counters_in_sram#]
init_counters_done#:
.end 	
#endif // #if THIS_ME==MPHY16_PACKET_TX_FIRST_ME
#endif // #ifdef UNIT_TEST								
#endif // #ifdef	COUNTERS

	//end of global GPRs initialization

	// initialze local memory, first intialize all 640 lonwords to 0
	// then initialize allowed_mpkts and phy_depth of each port in 
	// queue info to the specific phy_depth of that port  
.begin //begin of local variables entry tag lm_index
.reg lm_entry
.reg max_lm_entry
.reg lm_index

	alu[lm_index, --, b, 0]
	
	local_csr_wr[active_lm_addr_0, lm_index]
	// waiting for local_csr_wr to settle
    	immed[max_lm_entry, 640]
		immed_w1[max_lm_entry, 0]
	    alu[lm_entry,--, b, 0]
	
init_lm#:
   	alu[*l$index0++, --, b, 0x0]			
    alu[lm_entry, lm_entry, +, 1]		; increase by one 	
	alu[--, lm_entry, -, max_lm_entry]
	bne[init_lm#] 				; loop back
.end // lm_entry max_lm_entry lm_index

 
.begin
.reg port
.reg max_port
.reg phy_depth

	move[phy_depth, DEFAULT_PHY_DEPTH]
	move[port, 0]
	move[max_port, 16]

init_phy_depth#:
	_mphy16_tx_init_port_phy_depth[port, phy_depth]
	alu[port, port, +, 1]
	alu[--, port, -, max_port]
	bne[init_phy_depth#]
.end // port max_port phy_depth
	// end of local memory initialization

#if THIS_ME==MPHY16_PACKET_TX_FIRST_ME
	// initialized packets sent for each port in scheduler to 0 by reflect write
.begin
.reg port
.reg max_port
.reg $pkts_txed

	move[port, 0]
	move[max_port, 16]

reset_packets_sent_in_scheduler#:
	_mphy16_tx_notify_scheduler[port, $pkts_txed]
    ctx_arb[sig_reflect_write]
	alu[port, port, +, 1]
	alu[--, port, -, max_port]
	bne[reset_packets_sent_in_scheduler#]
.end // port max_port $pkts_txed

#endif // #if THIS_ME==MPHY16_PACKET_TX_FIRST_ME


#ifdef WAIT_FOR_COMMON_RESOURCE_INITIALIZATION 
	// following inter-ME signal sys_init_signal is generated by 
	// the ME whcih initializes common resources, such as scratch ring, CSIX TX
	// ME will wait for this signal to make sure all common resource are 
	// available, when it start CSIX TX processing
    .begin
	    .sig visible sys_init_signal				
	#ifdef USE_ME_INIT_SIGNAL
	.addr sys_init_signal ME_INIT_SIGNAL
	#endif
	    ctx_arb[sys_init_signal]
    .end
#endif //#ifdef WAIT_FOR_COMMON_RESOURCE_INITIALIZATION

.begin //begin of local variable sig
.reg sig
    alu[sig, --, B,  &sig1_next_context]
    alu_shf[sig, 0x0, OR, sig, <<3]         ; value to signal thread 0
    local_csr_wr[SAME_ME_SIGNAL, sig]   	; signal thread 0

 	alu[sig, --, B,  &sig2_next_context]
    alu_shf[sig, 0x0, OR, sig, <<3]         ; value to signal thread 0
    local_csr_wr[SAME_ME_SIGNAL, sig]   	; signal thread 0

	alu[sig, --, B,  &sig3_next_context]
    alu_shf[sig, 0x0, OR, sig, <<3]         ; value to signal thread 0
    local_csr_wr[SAME_ME_SIGNAL, sig]   	; signal thread 0
.end // end of .begin //begin of local variable sig


	br[sync_exec#]

	
ctx1_init#:
	// thread 1 context relative GPR variables initialization
    _mphy16_tx_thread_specific_init[tbuf_base, tbuf_scr_addr, sring_tr, \
			queue_stat_base, tx_mphy_stat_addr, \
			tx_seq_0_addr, indiref_base, sigmask_phs1_default, \
			sigmask_phs2_default, sig1_next_context_gpr, \
			sig2_next_context_gpr, sig3_next_context_gpr, 2]
	br[sync_exec#]

ctx2_init#:
	// thread 2 context relative GPR variables initialization
    _mphy16_tx_thread_specific_init[tbuf_base, tbuf_scr_addr, sring_tr, \
			queue_stat_base, tx_mphy_stat_addr, \
			tx_seq_0_addr, indiref_base, sigmask_phs1_default, \
			sigmask_phs2_default, sig1_next_context_gpr, \
			sig2_next_context_gpr, sig3_next_context_gpr, 3]
	br[sync_exec#]

	// thread 3 context relative GPR variables initialization
ctx3_init#:
    _mphy16_tx_thread_specific_init[tbuf_base, tbuf_scr_addr, sring_tr, \
			queue_stat_base, tx_mphy_stat_addr, \
			tx_seq_0_addr, indiref_base, sigmask_phs1_default, \
			sigmask_phs2_default, sig1_next_context_gpr, \
			sig2_next_context_gpr, sig3_next_context_gpr, 4]
	br[sync_exec#]

	// thread 4 context relative GPR variables initialization
ctx4_init#:
    _mphy16_tx_thread_specific_init[tbuf_base, tbuf_scr_addr, sring_tr, \
			queue_stat_base, tx_mphy_stat_addr, \
			tx_seq_0_addr, indiref_base, sigmask_phs1_default, \
			sigmask_phs2_default, sig1_next_context_gpr, \
			sig2_next_context_gpr, sig3_next_context_gpr, 5]
	br[sync_exec#]

	// thread 5 context relative GPR variables initialization
ctx5_init#:
    _mphy16_tx_thread_specific_init[tbuf_base, tbuf_scr_addr, sring_tr, \
			queue_stat_base, tx_mphy_stat_addr, \
			tx_seq_0_addr, indiref_base, sigmask_phs1_default, \
			sigmask_phs2_default, sig1_next_context_gpr, \
			sig2_next_context_gpr, sig3_next_context_gpr, 6]
	br[sync_exec#]

	// thread 6 context relative GPR variables initialization
ctx6_init#:
    _mphy16_tx_thread_specific_init[tbuf_base, tbuf_scr_addr, sring_tr, \
			queue_stat_base, tx_mphy_stat_addr, \
			tx_seq_0_addr, indiref_base, sigmask_phs1_default, \
			sigmask_phs2_default, sig1_next_context_gpr, \
			sig2_next_context_gpr, sig3_next_context_gpr, 7]
	br[sync_exec#]

	// thread 7 context relative GPR variables initialization
ctx7_init#:
    _mphy16_tx_thread_specific_init[tbuf_base, tbuf_scr_addr, sring_tr, \
			queue_stat_base, tx_mphy_stat_addr, \
			tx_seq_0_addr, indiref_base, sigmask_phs1_default, \
			sigmask_phs2_default, sig1_next_context_gpr, \
			sig2_next_context_gpr, sig3_next_context_gpr, 0]


// following sync_exec# is used to synchronize execution sequence, so ecah
// thread will execute in order in processing csix transmit 
sync_exec#:

  // get initial Tx_MPHY_Status MSF register value for following infinite
	// looping Packet TX processing 
	_mphy16_tx_get_initial_mphy_stat_for_packet_tx[]

	// wait for my turn
	ctx_arb[sig1_next_context]

#endm // end of macro mphy16_tx_init[]

///////////////////////////////////////////////////////////////////////////////

#endif	// __MPHY16_TX__INIT_UC__
