////////////////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
////////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: mphy16_tx_util.uc
//
//      Purpose: Utility macros for PACKET (POS/Ethernet) TX
//
//      History:
//
//
//      Date            Comment                         		By
//      -----------------------------------------------------------------------	
//
//      03/08/2002      Created									David Chou
//
//
///////////////////////////////////////////////////////////////////////////////


#ifndef	__MPHY16_TX_UTIL_UC__
#define	__MPHY16_TX_UTIL_UC__


///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_set_16bit_msf_reg
//
// Description: Use msf[fast_wr...] to set MSF register 
//
// Outputs: 
//								None
//
// Inputs:  
//		msf_reg_addr:			MSF register address to set
//		msf_reg_val:			MSF register value to set 
//
// Constants:
//								None 
//
// Size: 
//
//		3 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _mphy16_tx_set_16bit_msf_reg[msf_reg_addr, msf_reg_val]
.begin
.reg tmp_addr tmp_val
	  alu[tmp_addr, --, B, msf_reg_addr]			; get 16 bit msf address
	  alu_shf[tmp_val, --, B, msf_reg_val, <<16]	; get value							
	  msf[fast_wr, --, tmp_addr, tmp_val] 			; setting with fast_wr
.end // tmp_addr tmp_val
#endm    // end of macro _mphy16_tx_set_16bit_msf_reg[]



///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_thread_specific_init[]
//
// Description: initialize context relative GPRs for specific thread to save 
//              instruction cycles in Packet TX processing
//
// Outputs: 
//		tbufbase:			base to calculate tbuf address for specific tbuf
//                          element
//      tbuf_scr_loc:		scratch memory location managing next available
//                          tbuf element
//      scratch_ring:		scratch ring number used to get tx requet
//      qstat_base:			base address in local memory for queue status info 
//      addr_mphy_stat:		Tx_MPHY_Status MSF register address
//      addr_tx_seq_0:		Tx_Sequence_0 MSF register address
//      v_indiref_base:		base for indirect refernce in moving payload from 
//                          dram to tbuf
//      v_next_ctx_sig:		value used to write to SAME_ME_SIGNAL local csr 
//                          to wakeup next thread
//
// Inputs:  
//		next_thread:		next thread number of this thread 
//
// Constants:
//		RBUF_TBUF_LW:	 		starting address of TBUF  
//		PACKET_TX_AVAIL_TBUF_ELE_SCR_ADDR: scratch memory address managing 
//                                         next available tbuf element 
//      QM_TO_PACKET_TX_SCR_RING:	scratch ring number used to get tx requet
//      QUEUE_INFO_BASE:		base address in local memory for queue 
//                              status info 
//      TX_MPHY_STATUS:			Tx_MPHY_Status MSF register address
//		TX_SEQUENCE_0:			Tx_SEQUENCE_0 MSF register address
//      DRAM_INDIREF_BASE_LW:	base for indirect refernce in moving 
//								payload from dram to tbuf
//
// Size: 
//
//		16 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _mphy16_tx_thread_specific_init[tbufbase, tbuf_scr_loc, scratch_ring, \
				qstat_base, addr_mphy_stat, addr_tx_seq_0, \
				v_indiref_base, out_default_sigmask1, out_default_sigmask2, \
				v_sig1, v_sig2, v_sig3, next_thread]

	// starting address of TBUF 
    move[tbufbase, RBUF_TBUF_LW]  


	// scratch memory location PACKET_TX_AVAIL_TBUF_ELE_SCR_ADDR keep next 
	// available tbuf element, which is initialized to 0, later get and 
	// increment by one by SCRATCH atomic operation SCRACTH[test_and_incr..] 
	move[tbuf_scr_loc, PACKET_TX_AVAIL_TBUF_ELE_SCR_ADDR]

 	// scratch ring which QM put tx requests
    // scratch instruction requires context relative GPR
	// Scratch for communicating between Queue Manager and Packet Transmit Blocks
	alu_shf[scratch_ring, --, b,  QM_TO_PACKET_TX_SCR_RING, <<2]

	move[qstat_base, QUEUE_INFO_BASE] 

	move[addr_mphy_stat, TX_MPHY_STATUS] 
	 
	move[addr_tx_seq_0, TX_SEQUENCE_0]
	// base to calculate the address field in indirect ref of 
	// dram[tbuf_wr....] instruction
	move[v_indiref_base, DRAM_INDIREF_BASE_LW]

	// default phase 1 signal mask at start of each iteration of loop 
	alu[out_default_sigmask1, --, B, 0]			; initialize to 0
	_mphy16_tx_set_signal[out_default_sigmask1, sig2_next_context]	; add sig2_next_context
	_mphy16_tx_set_signal[out_default_sigmask1, sig_reflect_write]	; add sig_reflect_write
	_mphy16_tx_set_signal[out_default_sigmask1, sig_msf_access_3]	; add sig_msf_access_3
	_mphy16_tx_set_signal[out_default_sigmask1, sig_scratch_access_1]	; add sig_scratch_access_1
	_mphy16_tx_set_signal[out_default_sigmask1, sig_scratch_access_2]	; add sig_scratch_access_2

	// default phase 2 signal mask at start of each iteration of loop 
	alu[out_default_sigmask2, --, B, 0]			; initialize to 0
	_mphy16_tx_set_signal[out_default_sigmask2, sig3_next_context]	; add sig3_next_context
	_mphy16_tx_set_signal[out_default_sigmask2, sig_dram_access_1]	; add sig_dram_access_1
	_mphy16_tx_set_push_signal[out_default_sigmask2, sig_dram_access_1]	; add push signal


  
.begin //local area for local variable next_context
.reg next_context
	// value to write to SAME_ME_SIGNAL csr to wake up next thread
	// will differs for each thread
	move[next_context,next_thread]
	alu_shf[v_sig1, next_context, or, &sig1_next_context, <<3]
	alu_shf[v_sig2, next_context, or, &sig2_next_context, <<3]
	alu_shf[v_sig3, next_context, or, &sig3_next_context, <<3]

.end  // end of .begin //local area for local variable next_context
	
#endm //end of macro _mphy16_tx_thread_specific_init[]

/////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_get_initial_mphy_stat_for_packet_tx[]
//
// Description: get initail Tx_MPHY_Status MSF register value for infinite 
//              looping Packet TX processing
//
// Outputs: 
//								None
//
// Inputs:  
//								None
//
// Constants:
//								None
//
// Size: 
//		5 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _mphy16_tx_get_initial_mphy_stat_for_packet_tx[]
   // get tx request from scratch ring

	// get Tx_MPHY_Status by msf read for later use in phase 1 
	msf[read, $mphy_stat, tx_mphy_stat_addr,  0, 1], sig_done[sig_msf_access_2]	

	// end of reading scratch ring phase
	// swap out to wait scratch ring read complete, and wait for my turn
	ctx_arb[sig_msf_access_2]

#endm // end of macro _mphy16_tx_get_initial_mphy_stat_for_packet_tx[]



///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_set_lmindex1_for_enqueue_sop[]
//
// Description: set local memory index 1 to port queue status info, also 
//              use the delay slots to settle local_csr_wr to prepare some
//				variables used later
//
//
// Outputs: 
//		entry_lm_base:		the begining address of the queue entry in local
//							memory for the port in tx requet
//      port:				port in tx request
//  
// Inputs:  
//		txr_lw0:			tx request from Queue Manager 
//
// Constants:
//						 	None  
//
// Size: 
//
//		7 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_mphy16_tx_set_lmindex1_for_enqueue_sop[entry_lm_base, port, txr_lw0]
	// set local memory index 1 to port queue status info and local
	// memory index 1 to queue entry at queue tail of that port
.begin 
.reg queue_info_lm
.reg port_lsb_3_bits
	alu_shf[port, PORT_MASK_IN_TXR, and, txr_lw0, >>24]	; get port numb	   
  	// get the pointer to queue status info of port_numb in local memory
	// use least 3 significant bits to find base for queue stat and queue entries
	// info, it will applies to both Packet TX first ME (port 0 to 7) and 
	// Packet TX second ME (port 8 to 15)
	alu[port_lsb_3_bits, port, and, CONST_07]	; use least 3 significant bits
	alu_shf[queue_info_lm, queue_stat_base, or, port_lsb_3_bits, \
			<<QUEUE_INFO_ENRY_SHFT]

	local_csr_wr[active_lm_addr_1, queue_info_lm]
	// while waiting for local_csr_wr to settle, do something to save 
	// instruction cycle
	alu_shf[entry_lm_base, --, b, port_lsb_3_bits, <<QUEUE_ENRY_SHFT]
	alu[entry_lm_base, entry_lm_base, +, QUEUE_ENTRY_BASE]
	nop					; for local_csr_wr to settle down

.end // queue_info_lm port_lsb_3_bits
#endm // end of macro _mphy16_tx_set_lmindex1_for_enqueue_sop[]


///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_set_lmindex0_and_enqueue_sop[]
//
// Description: set local memory index 0 to the tail of the queue entry for 
//				the port and put the packet in tx request in queue tail
//
// Outputs:					
//							None 
//
// Inputs:  
//		entry_lm_base:		the begining address of the queue entry in local
//							memory for the port 
//		txr_lw0:			tx request from Queue Manager 
//
// Constants:
//						 	None 
//
//	Size:
//		13 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _mphy16_tx_set_lmindex0_and_enqueue_sop[entry_lm_base, txr_lw0]

	// set local memory index to queue entry at queue tail of that port,
	// update ab_bd and p_sop_flag 
.begin 
.reg queue_entry_lm
.reg tail_offset

  	// get the pointer to queue status info of port_num in local memory
	alu_shf[tail_offset, TAIL_MASK, AND, *l$index1[Q_STAT_OFFSET], >>TAIL_SHFT]
	alu[queue_entry_lm, entry_lm_base, +, tail_offset]
	// get the pointer to the tail of the Q
	// write active_lm_addr0 with the tail
	local_csr_wr[active_lm_addr_0, queue_entry_lm]
	// while waiting for local_csr_wr to settle down,
	// calculate the port_qv_stat for adding port to qstat_vector
	nop


	// advance tail
	alu[tail_offset, tail_offset, +, QUEUE_ENTRY_SIZE]
	alu[--, tail_offset, -, MAX_MINUS_ONE_QUEUE_OFFSET]
	// utilizes defer slots to most
	ble[cont_enq_0#], defer[3]
		ld_field_w_clr[*l$index0[ABD_0_OFFSET], 0111, txr_lw0]	; set ab_bd
		alu_shf[*l$index0[ABD_0_OFFSET], *l$index0[ABD_0_OFFSET], or, \
				1, <<SOP_BIT_LOC]							; set p_sop_flag
   		nop

	// reset tail_offset to 0
	alu[tail_offset, --, b, 0]
cont_enq_0#:
	// update tail_offset
	ld_field[*l$index1[Q_STAT_OFFSET], 0010, tail_offset, <<TAIL_SHFT]

.end 
#endm // end of #macro _set_lmindex0_and_enqueue_sop

	 
///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_save_sop_meta_to_lm()
//
// Description: save sop meta data for the port in tx request to local memory
//
// Outputs: 
//								None
//
// Inputs:  
//      in_port:				port from tx request
//
// Constants:
//								None
//
//	Size: 7 instructions. (Worst case cycle count)
//
///////////////////////////////////////////////////////////////////////////////
#macro _mphy16_tx_save_sop_meta_to_lm(in_port)

.begin 
.reg v_buf_size port_qv_stat whole_dl_meta0
	dl_meta_get_buffer_next[whole_dl_meta0]
	ld_field_w_clr[*l$index0[SBD_0_OFFSET], 0111, whole_dl_meta0]	; get sbd
	alu[--, *l$index0[SBD_0_OFFSET], -, IX_NULL]	; is null (0xFF)?
	beq[finish_save_sop_meta_to_lm#], defer[3]
		alu_shf[*l$index0[ABD_0_OFFSET], *l$index0[ABD_0_OFFSET], or, \
				1, <<SOP_BIT_LOC]
 	  	//update sop buffer size, offset
		dl_meta_get_offset[*l$index0[ABD_1_OFFSET]]
		dl_meta_get_buffer_size[v_buf_size]
	 
	alu_shf[*l$index0[ABD_0_OFFSET], *l$index0[ABD_0_OFFSET], or, 1, \
			<<RD_NBD_BIT_LOC]	; set ab_rd_nbd_flag
finish_save_sop_meta_to_lm#:

	ld_field[*l$index0[ABD_1_OFFSET], 1100, v_buf_size, <<PAYLO_RMND_LOC] 

   	// updating pkts_in_queue, 
	alu[*l$index1[Q_STAT_OFFSET], *l$index1[Q_STAT_OFFSET], +, @qpkt_incr]

	alu[--, in_port, or, 0]
	alu_shf[port_qv_stat, --, b, 1, <<indirect]
	alu[@qstat_vector, @qstat_vector, or, port_qv_stat]

.end // v_buf_size port_qv_stat whole_dl_meta0
#endm // end of #macro _mphy16_tx_save_sop_meta_to_lm()


///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_adjust_mphy_fc_mask[]
//
// Description: update mphy_stat_vector, busy_mask, with updated $mphy_stat
//
// Outputs: 
//								None
//
// Inputs:  
//								None
//
// Constants:
//								None
//
//	Size:
//		23 instructions. (Worst case cycle count, should not happen every
//      packet)
//
///////////////////////////////////////////////////////////////////////////////
#macro _mphy16_tx_adjust_mphy_fc_mask[]

.begin 
.reg tmp
.reg tx_stat
.reg ready_ports
.reg need_update
.reg cur_port
.reg qinfo_lm
.reg port_lsb_3_bits
#ifndef MPHY_STAT_ALWAYS_READY
	ld_field_w_clr[tmp, 0011, $mphy_stat, >>16]	; get tx_pending
	ld_field_w_clr[tx_stat, 0011, $mphy_stat]	; get tx_status
	alu[ready_ports, tx_stat, and~, tmp]
#else
	immed[ready_ports, 0xFFFF]
	immed_w1[ready_ports, 0x0]
#endif //#ifndef MPHY_STAT_ALWAYS_READY
	 
	// busy mask already identify what ports need to check, so port_mask is 
	// not required to decide what ports need update
	alu[need_update, @busy_mask, and, ready_ports]
	// no need to update, is not worst case, can take branchout penalty
	beq[end_of_adjust_mphy_fc_mask#]
	// using defer slots
	// add the ports back to mphy_stat_vector
	alu[@mphy_stat_vector, @mphy_stat_vector, or, need_update]
	// remove the ports back to busy_mask
	alu[@busy_mask, @busy_mask, and~, need_update]


check_again#:

	ffs[cur_port, need_update]
	// all updated can take branchout penalty
	beq[end_of_adjust_mphy_fc_mask#]
	
   	// get the pointer to queue status info of port_numb in local memory
	// use least 3 significant bits to find base for queue stat and queue entries
	// info, it will applies to both Packet TX first ME (port 0 to 7) and Packet
	// TX second ME (port 8 to 15)
	alu[port_lsb_3_bits, cur_port, and, CONST_07]	; use least 3 significant bits
	alu_shf[qinfo_lm, queue_stat_base, or, port_lsb_3_bits, \
			<<QUEUE_INFO_ENRY_SHFT]
	//set local memory index 1
	local_csr_wr[active_lm_addr_1, qinfo_lm]
	//remove current port from need_update
	alu[--, cur_port, or, 0]
	alu_shf[tmp, --, b, 1, <<indirect]

	br[check_again#], defer[2]
		alu[need_update, need_update, and~, tmp]
		// update allowed_mpkts
		alu[*l$index1[RMND_AL_MPKT_OFFSET], --, b, *l$index1[PHY_DEPTH_OFFSET]]

end_of_adjust_mphy_fc_mask#:

.end // 
#endm // end of #macro _mphy16_tx_adjust_mphy_fc_mask[]


///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_update_rond_robin_vecror[]
//
// Description: updating round robin vector (@rr_vector)
//
// Outputs:					
//							None 
//
// Inputs:  
//      port:				port to transmit the current mpacket in processing
//		rr_base:			base used to update round robin vector 
//
// Constants:			
//							None
//
//	Size:
//		3 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_mphy16_tx_update_rond_robin_vecror[port, rr_base]
.begin 
.reg tmp
	alu[--, port, or, 0]
	alu_shf[tmp, --, B, rr_base, <<indirect]
	alu[@rr_vector, --, b, tmp]
.end // 
#endm // end of #macro _mphy16_tx_update_rond_robin_vecror[]


///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_set_lmindex1_for_deq_port_stat[]
//
// Description: set local memory index 1 to port queue status info, also 
//              use the delay slots to settle local_csr_wr to prepare some
//				variables used later
//
// Outputs: 
//		entry_lm_base:		the begining address of the queue entry in local
//							memory for the port to transmit the current mpacket
//  
// Inputs:  
//      port:				port to transmit the current mpacket
//
// Constants:
//						 	None  
//
//	Size:
//		6 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_mphy16_tx_set_lmindex1_for_deq_port_stat[entry_lm_base, port]
	// set local memory index 1 to port queue status info 
.begin 
.reg queue_info_lm 
.reg port_lsb_3_bits
  	// get the pointer to queue status info of port_numb in local memory
	// use least 3 significant bits to find base for queue stat and queue entries
	// info, it will applies to both Packet TX first ME (port 0 to 7) and 
	// Packet TX second ME (port 8 to 15)
	alu[port_lsb_3_bits, port, and, CONST_07]	; use least 3 significant bits
	alu_shf[queue_info_lm, queue_stat_base, or, port_lsb_3_bits, \
			<<QUEUE_INFO_ENRY_SHFT]

	local_csr_wr[active_lm_addr_1, queue_info_lm]
	// while waiting for local_csr_wr to settle, do something to save 
	// instruction cycle
	alu_shf[entry_lm_base, --, b, port_lsb_3_bits, <<QUEUE_ENRY_SHFT]
	alu[entry_lm_base, entry_lm_base, +, QUEUE_ENTRY_BASE]
	nop								; wait for local_csr_wr to settle down

.end // queue_info_lm port_lsb_3_bits
#endm // end of macro _mphy16_tx_set_lmindex1_for_deq_port_stat[]



///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_set_lmindex0_for_deq_entry[]
//
// Description: set local memory index 0 to queue head of this port, also
//           	initialize tcw for SOP_EOP mpacket with chnl in channel 
//              field
//
// Outputs:					
//		tcw:				transmit control word 
//
// Inputs:  
//		entry_lm_base:		the begining address of the queue entry in local
//							memory for the port to transmit the current mpacket 
//      chnl:				port to transmit the current mpacket 
//
// Constants:
//						 	None 
//
//	Size:
//		6 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_mphy16_tx_set_lmindex0_for_deq_entry[tcw, entry_lm_base, chnl]

	// set local memory index to queue entry at queue head of that port,
	
.begin 
.reg queue_entry_lm
.reg head_offset

  	// get the pointer to queue status info in local memory
	alu[head_offset, HEAD_MASK, and, *l$index1[Q_STAT_OFFSET]]
	alu[queue_entry_lm, entry_lm_base, +, head_offset]
	// get the pointer to the tail of the Q
	// write active_lm_addr0 with the tail
	local_csr_wr[active_lm_addr_0, queue_entry_lm]
	immed[tcw, TCW0_SOP_EOP_BASE_W0]
	immed_w1[tcw, TCW0_SOP_EOP_BASE_W1]
	alu[tcw, tcw, or, chnl]
.end 
#endm // end of macro 	_mphy16_tx_set_lmindex0_for_deq_entry[]

	
///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_advance_queue_head[]
//
// Description: advance queue head after the last mpcket of one packet is sent
//              to the port
//
// Outputs:							
//		port_bit:			port bit pattern, port_bit = (1 << port)
//							calculated here by utilizing defer slot to save
//							instruction cycle for worst case
//
// Inputs:  
//      port:				port to transmit the current mpacket 
//
// Constants:
//						 	None 
//
// Size:
//		14 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _mphy16_tx_advance_queue_head[port_bit, port]

.begin 
.reg head_offset
.reg tmp
.reg tmp1


	alu[head_offset, HEAD_MASK, and, *l$index1[Q_STAT_OFFSET]]

	// advance queue head_offset
	alu[head_offset, head_offset, +, QUEUE_ENTRY_SIZE]	
	alu[--, head_offset, -, MAX_MINUS_ONE_QUEUE_OFFSET]

	ble[cont_1_adv_head#], defer[3]
	// use defer slots 
	//decr the pkts_in_queue
		alu[*l$index1[Q_STAT_OFFSET], *l$index1[Q_STAT_OFFSET], -, @qpkt_incr]
		alu[--, port, or, 0]
		alu_shf[port_bit, --, b, 1, <<indirect]

	// need reset head_offset in queue status
	alu[head_offset, --, b, 0]
cont_1_adv_head#:
	ld_field[*l$index1[Q_STAT_OFFSET], 0001, head_offset]

	// check still packets in queue
	alu[--, QPKT_MASK, and, *l$index1[Q_STAT_OFFSET], >>QPKT_SHFT]
	// when branchout, it takes 3 instructions to cont_2_adv_head#, same as not
	// branchout
	bne[cont_2_adv_head#], defer[2]
	// increase transmitted pkt by one
		alu[*l$index1[PKT_TXED_OFFSET], *l$index1[PKT_TXED_OFFSET], +, 1]
		alu[tmp, --, b, @qstat_vector]

	// no pkt left, reset the bit of this port in qstat_vector
	alu[@qstat_vector, tmp, and~, port_bit]

cont_2_adv_head#:

#ifdef COUNTERS
.begin 
.reg cntr_addr
	alu_shf[cntr_addr, --, b, port, <<4]				; get counter addr
	alu[cntr_addr, cntr_addr, +, @cntr_base]			; add base
	sram[incr, --, cntr_addr, PKT_TRANSMITTED_CNT_IDX]	; increment pkts transmitted
.end // cntr_addr
#endif //#ifdef	COUNTERS

.end // head_offset tmp tmp1


#endm // end of macro _mphy16_tx_advance_queue_head[]


///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_decrement_allowed_mpkts_in_this_port[]
//
// Description: decrease the allowed_mpkts of the port by one, if it become 
//              zero, remove this port from @mohy_stat_fc_vector
//
// Outputs:							
//							None
//
// Inputs:  
//      port_bit:			port bit pattern for the port, which the mpacket is
//                          destined to, port_bit = (1 << port)
//						
//
// Constants:
//						 	None 
//
// Size:
//		5 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _mphy16_tx_decrement_allowed_mpkts_in_this_port[port_bit]

	// decrease allowed_mpkts by one
	alu[*l$index1[RMND_AL_MPKT_OFFSET], *l$index1[RMND_AL_MPKT_OFFSET], -, 1]
#ifndef SIMULATE_MPHY_16_WITH_SPHY
	bne[decrement_allowed_mpkts_done#]

	// add port to @busy_mask
	alu[@busy_mask, @busy_mask, or, port_bit]
    // remove port from @mphy_stat_vector
	alu[@mphy_stat_vector, @mphy_stat_vector, and~, port_bit]

decrement_allowed_mpkts_done#:
#endif // #ifndef SIMULATE_MPHY_16_WITH_SPHY

#endm // end of macro _mphy16_tx_decrement_allowed_mpkts_in_this_port[]



///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_notify_scheduler[port, txd_port]
//
// Description: notify scheduler that one packet has been transmitted from 
//				this port 
//
// Outputs:							
//							None 
//
// Inputs:  
//      port:				port to transmit the current mpacket 
//      txd_port:			total packets transmitted th the port (sram 
//                          transfer register)
//
// Constants:
//						 	None 
//
// Size:
//		9 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_mphy16_tx_notify_scheduler[port, txd_port]

.reg remote $$txd_p0 $$txd_p1 $$txd_p2 $$txd_p3 
.reg remote $$txd_p4 $$txd_p5 $$txd_p6 $$txd_p7 
.reg remote $$txd_p8 $$txd_p9 $$txd_p10 $$txd_p11 
.reg remote $$txd_p12 $$txd_p13 $$txd_p14 $$txd_p15 

.begin 
.reg jmp_offset
	alu_shf[jmp_offset, --, b, port, <<1]    ; get jmp_offset by port times two
	// jump to different port
	jump[jmp_offset, jmp_tbl#],defer[3], targets[xp0#,xp1#,xp2#,xp3#,xp4#,xp5#, \
						xp6#,xp7#,xp8#,xp9#,xp10#,xp11#,xp12#,xp13#,xp14#,xp15#] 
		alu[txd_port, --, b, *l$index1[PKT_TXED_OFFSET]] ; get txed pkts for the port
		nop
		nop

//	Start of jump table. 
jmp_tbl#:
  
xp0#:	
	// update scheduler with updated counts of packets transmitted for port 0
	cap[write, txd_port, SCHEDULER_ME, $$txd_p0, 0, 1], \
			sig_done[sig_reflect_write]	; reflect write
  	br[notify_schlr_done#]

xp1#:
	// update scheduler with updated counts of packets transmitted for port 1
	cap[write, txd_port, SCHEDULER_ME, $$txd_p1, 0, 1], \
			sig_done[sig_reflect_write]	; reflect write
  	br[notify_schlr_done#]

xp2#:
	// update scheduler with updated counts of packets transmitted for port 2
	cap[write, txd_port, SCHEDULER_ME, $$txd_p2, 0, 1], \
			sig_done[sig_reflect_write]	; reflect write
  	br[notify_schlr_done#]

xp3#:
	// update scheduler with updated counts of packets transmitted for port 3
	cap[write, txd_port, SCHEDULER_ME, $$txd_p3, 0, 1], \
			sig_done[sig_reflect_write]	; reflect write
  	br[notify_schlr_done#]

xp4#:  
	// update scheduler with updated counts of packets transmitted for port 4
	cap[write, txd_port, SCHEDULER_ME, $$txd_p4, 0, 1], \
			sig_done[sig_reflect_write]	; reflect write
  	br[notify_schlr_done#]

xp5#:
	// update scheduler with updated counts of packets transmitted for port 5
	cap[write, txd_port, SCHEDULER_ME, $$txd_p5, 0, 1], \
			sig_done[sig_reflect_write]	; reflect write
  	br[notify_schlr_done#]

xp6#:
  
	// update scheduler with updated counts of packets transmitted for port 6
	cap[write, txd_port, SCHEDULER_ME, $$txd_p6, 0, 1], \
			sig_done[sig_reflect_write]	; reflect write
  	br[notify_schlr_done#]

xp7#:  
	// update scheduler with updated counts of packets transmitted for port 7
	cap[write, txd_port, SCHEDULER_ME, $$txd_p7, 0, 1], \
			sig_done[sig_reflect_write]	; reflect write
  	br[notify_schlr_done#]

xp8#:  
	// update scheduler with updated counts of packets transmitted for port 8
	cap[write, txd_port, SCHEDULER_ME, $$txd_p8, 0, 1], \
			sig_done[sig_reflect_write]	; reflect write
  	br[notify_schlr_done#]

xp9#:  
	// update scheduler with updated counts of packets transmitted for port 9
	cap[write, txd_port, SCHEDULER_ME, $$txd_p9, 0, 1], \
			sig_done[sig_reflect_write]	; reflect write
   	br[notify_schlr_done#]

xp10#:
	// update scheduler with updated counts of packets transmitted for port 10
	cap[write, txd_port, SCHEDULER_ME, $$txd_p10, 0, 1], \
			sig_done[sig_reflect_write]	; reflect write
   	br[notify_schlr_done#]

xp11#: 
	// update scheduler with updated counts of packets transmitted for port 11
	cap[write, txd_port, SCHEDULER_ME, $$txd_p11, 0, 1], \
			sig_done[sig_reflect_write]	; reflect write
   	br[notify_schlr_done#]

xp12#: 
	// update scheduler with updated counts of packets transmitted for port 12
	cap[write, txd_port, SCHEDULER_ME, $$txd_p12, 0, 1], \
			sig_done[sig_reflect_write]	; reflect write
   	br[notify_schlr_done#]

xp13#: 
	// update scheduler with updated counts of packets transmitted for port 13
	cap[write, txd_port, SCHEDULER_ME, $$txd_p13, 0, 1], \
			sig_done[sig_reflect_write]	; reflect write
    	br[notify_schlr_done#]

xp14#:
	// update scheduler with updated counts of packets transmitted for port 14
	cap[write, txd_port, SCHEDULER_ME, $$txd_p14, 0, 1], \
			sig_done[sig_reflect_write]	; reflect write
  	br[notify_schlr_done#]

xp15#:
	// update scheduler with updated counts of packets transmitted for port 15
	cap[write, txd_port, SCHEDULER_ME, $$txd_p15, 0, 1], \
			sig_done[sig_reflect_write]	; reflect write

notify_schlr_done#:
.end // jmp_offset

#endm // end of macro _mphy16_tx_notify_scheduler[]

	
///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_init_port_phy_depth[]
//
// Description: set phy_depth and allowed_mpkts for port port_no's quee info
// 				 in local memory to phy_depth_val
//
// Outputs:							
//							None 
//
// Inputs:  
//      port:				port to initialize PHY Tx FIFO depth value
//      phy_depth_val:		PHY Tx FIFO depth value
//
// Constants:
//						 	None 
//
// Size:
//		8 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_mphy16_tx_init_port_phy_depth[port_no, phy_depth_val]
.begin 
.reg queue_info_lm
.reg port_lsb_3_bits
   	// get the pointer to queue status info of port_numb in local memory
	// use least 3 significant bits to find base for queue stat and queue entries
	// info, it will applies to both Packet TX first ME (port 0 to 7) and Packet 
	// TX second ME (port 8 to 15)
	alu[port_lsb_3_bits, port_no, and, CONST_07]	; use least 3 significant bits
	alu_shf[queue_info_lm, queue_stat_base, or, port_lsb_3_bits, <<QUEUE_INFO_ENRY_SHFT]

	local_csr_wr[active_lm_addr_1, queue_info_lm]
	// waiting for local_csr_wr to settle
	nop
	nop
	nop
	alu[*l$index1[RMND_AL_MPKT_OFFSET], --, b, phy_depth_val]	; set allowed_mpkts
	alu[*l$index1[PHY_DEPTH_OFFSET], --, b, phy_depth_val]		; set phy_depth

.end // queue_info_lm 
#endm // end of #macro _mphy16_tx_init_port_phy_depth[]


///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_get_tbuf_ele_and_addr[]
//
// Description: get next available tbuf element (tbufele) and corresponding
//              address (tbuf_addr) from input whole_avl_tbufele gotten
//              from atomic scratch access, and tbuf base address
//
// Outputs:							
//		tbufele:			the next available tbuf element 
//		tbuf_addr:			the starting address of the next available tbuf 
//							element
//
// Inputs:  
// 		whole_avl_tbufele:	the 32-bits next available tbuf element gotten
//  						from atomic scratch access
//      tbuf_base: 			the start address of TBUF
//
// Constants:
//		TBUF_ELE_BITS_MASK:	mask to get the next available tbuf element from
//                          input whole_avl_tbufele
//
// Size:
//		2 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _mphy16_tx_get_tbuf_ele_and_addr[tbufele, tbuf_addr, \
										whole_avl_tbufele, tbufbase]
#ifdef DEBUG_TX_PENDING_LOCKED
.begin 
.reg taddr tcount $tcnt
	alu[tcount,--, b, whole_avl_tbufele]
	alu[$tcnt, --, b, tcount]
	alu[taddr, --, b, @tbuf_no_addr]
	sram[write, $tcnt, taddr, 0, 1], sig_done[sig_sram_write_1]
    alu_shf[sig_mask, sig_mask, or, 1, <<SRAM_WRITE_1_SIG]
	alu[@tbuf_no_addr, @tbuf_no_addr, +, 4]
.end //taddr tcount $tcnt
#endif //#ifdef DEBUG_TX_PENDING_LOCKED


	// mask out extra bits of $avail_tbuf_ele
 	alu[tbufele, TBUF_ELE_BITS_MASK, and, whole_avl_tbufele]

    // since TBUF address range is 0x2000 to 0x3FFC
	// we can use one instruction instead of following
	// two instruction to calculate addr_of_tuf 	
	// alu_shf[tmp, --, B, tbufele, <<TBUF_ADDR_SHF]
	// alu[tbuf_addr, tmp, +, tbufbase]
	alu[tbuf_addr, tbufbase, or, tbufele, <<TBUF_ADDR_SHF]
#endm // end of macro _mphy16_tx_get_tbuf_ele_and_addr[]



/////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_move_sop_paylo_to_tbuf[] 
//
// Description: move payload of sop or sop_eop mpacket to tbuf
//
// Outputs:							
//							None
//
// Inputs:  
// 		indir:				indirect reference used in dram[tbuf_wr...] 
//							instruction
//		dramaddr:			dram address of the payload
//		eobbd:				meta data handle 
//		addrtbuf:			the starting address of tbuf for payload 
//		indirbase:			base for  indir
//      offset: 			the starting address of the payload to copy
//		paylo:				payload length
//		offsetmod8:			payload offset
//		sig_dram:			signal number for dram[tbuf_wr...] instruction
//
// Constants:
//							None
//
// Size:
//		8 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_mphy16_tx_move_sop_paylo_to_tbuf[indir, dramaddr, bd, addrtbuf, indirbase, offset, paylo, offsetmod8, sig_dram] 
.begin
.reg tmpx
	// prepare tbuf address for  dram[tbuf_wr......]
	alu[indir, indirbase, or, addrtbuf, <<5]
	// get true eob_bd  
	alu_shf[tmpx, --, b, bd, <<2]
    // use following dl_buf_get_data_from_meta macro from dispatch_loop.uc to
	// get dram address at the beginning of the buffer from true_dl_buf_hdl
	// now it takes one instruction
    dl_buf_get_data_from_meta[dramaddr, tmpx]

	// total we need to fecth (offsetmod8 + paylo) bytes
	// to tbuf
	alu[tmpx, offsetmod8, +, paylo]

    // since ref count 0 will fectch 8 bytes, and ref count 1 will fetch
	// 16 bytes......, and ref count 15 will fetch 128 bytes, 
	// the right way to do it is followinhg:
	//    a) decrease tm_payload_len by one
	//    b) then right shift the result by 3
	// We will do step a first, then combine the right shift in the shift into
	// the ref count in indir_ref.  In this way, we make sure that we fetch 
	// enough and closet to 8 byte alignment of packet data	
	alu[tmpx, tmpx, -, 0x01]	
	// right shift tm_payload_len to get rid of any bits set in bit 0 to 3 to 
	// get ref_cnt for indir_ref 
    alu_shf[tmpx, --, b, tmpx, >>3]
    // left shift 21 bit to ref_cnt location
    alu[--, indir, or, tmpx, <<21] 

    // how many bytes of packet payload are copied to tbuf from dram is 
	// controlled by Ref Cnt field in indir_ref, following max_16 is
	// dummy, offset is added to dramaddr to get the actual address to start 
	// fetch, and the lower 3 bits of the address is ignored by dram channel	 
    dram[tbuf_wr, --, dramaddr, offset, max_16], indirect_ref, \
		sig_done[sig_dram]
.end // tmpx
#endm // end of macro _mphy16_tx_move_sop_paylo_to_tbuf[]


/////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_set_tcw0_with_paylo_len_offset[]
//
// Description: set payload_length and payload_offset in trnsmit control word
//
//
// Outputs:							
//							None
//
// Inputs/Outputs:  
//		tcw:				transmit control word
// 
// Inputs:  
//		paylo_len:			payload length
//		paylo_offse:		payload offset
//
// Constants:
//		TCW0_PAYLOAD_LEN_LSB_LOC:	payload length field location in transmit 
//									control word   
//		TCW0_PAYLOAD_OFFSET_LSB_LOC:	payload offset field location in  
//										transmit contrl word     
//
// Size:
//		2 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_mphy16_tx_set_tcw0_with_paylo_len_offset[tcw, paylo_len, paylo_offset]
	alu_shf[tcw, tcw, or, paylo_len, <<TCW0_PAYLOAD_LEN_LSB_LOC]
	alu_shf[tcw, tcw, or, paylo_offset, <<TCW0_PAYLOAD_OFFSET_LSB_LOC]
#endm // end of macro _mphy16_tx_set_tcw0_with_paylo_len_offset[]
	


/////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_update_ab_paylo_rmnd_and_offset_rpaylo[]
//
// Description: update ab_paylo_rmnd and ab_offset_rpaylo in queue entry 
//
//
// Outputs:							
//							None
//
// Inputs:  
//		mpkt_paylo_len:		payload bytes will transmitted by this mpacket from 
// 							the active buffer
//
// Constants:
//		PAYLO_RMND_LOC:		location of the least significant bit of the 
//							remained payload in queue entry 
//
// Size:
//		3 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _mphy16_tx_update_ab_paylo_rmnd_and_offset_rpaylo[mpkt_ab_payload]
#ifndef AVOID_USING_SKIP_BIT_IN_TCW0
.begin
.reg tmpx
	alu[*l$index0[ABD_1_OFFSET], *l$index0[ABD_1_OFFSET], +, mpkt_ab_payload]
	alu_shf[tmpx, --, b, mpkt_ab_payload, <<PAYLO_RMND_LOC]
	alu[*l$index0[ABD_1_OFFSET], *l$index0[ABD_1_OFFSET], -, tmpx]
.end // tmpx
#else //#ifndef AVOID_USING_SKIP_BIT_IN_TCW0
// adjust current mpkt size to make sure next mpkt in this buffer always have
// more than 4 bytes payload, so SKIT bit never set in TCW0
.begin
.reg next_mpkt_len tmpx
    ld_field_w_clr[tmpx, 0011, *l$index0[ABD_1_OFFSET], >>PAYLO_RMND_LOC]
	alu[next_mpkt_len, tmpx, -, mpkt_ab_payload]
	alu[--, next_mpkt_len, -, 4]
	bge[adjust_this_mpkt_to_avoid_using_skip_bit_done#] 
	alu[mpkt_ab_payload, mpkt_ab_payload, -, 4]
adjust_this_mpkt_to_avoid_using_skip_bit_done#:
	alu[*l$index0[ABD_1_OFFSET], *l$index0[ABD_1_OFFSET], +, mpkt_ab_payload]
	alu_shf[tmpx, --, b, mpkt_ab_payload, <<PAYLO_RMND_LOC]
	alu[*l$index0[ABD_1_OFFSET], *l$index0[ABD_1_OFFSET], -, tmpx]
.end // next_mpkt_len tmpx
#endif //#ifndef AVOID_USING_SKIP_BIT_IN_TCW0







#endm // end of _mphy16_tx_update_ab_paylo_rmnd_and_offset_rpaylo[]
	


///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_set_tcw0_with_paylo_len_offset_chnl[]
//
// Description: set payload_length, payload_offset, and channel fields in 
//              trnsmit control word
//
// Outputs:							
//							None
//
// Inputs/Outputs:  
//		tcw:				transmit control word
// 
// Inputs:  
//		paylo_len:			payload length
//		paylo_offse:		payload offset
//		chnl:				port number
//
// Constants:
//		TCW0_PAYLOAD_LEN_LSB_LOC:	payload length field location in transmit 
//									control word   
//		TCW0_PAYLOAD_OFFSET_LSB_LOC:	payload offset field location in  
//										transmit contrl word     
//
// Size:
//		3 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_mphy16_tx_set_tcw0_with_paylo_len_offset_chnl[tcw, paylo_len, \
													paylo_offset, chnl]
	alu_shf[tcw, tcw, or, paylo_len, <<TCW0_PAYLOAD_LEN_LSB_LOC]
	alu_shf[tcw, tcw, or, paylo_offset, <<TCW0_PAYLOAD_OFFSET_LSB_LOC]
	alu[tcw, tcw, or, chnl]					; set channel field
#endm // end of macro _mphy16_tx_set_tcw0_with_paylo_len_offset_chnl[]


///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_read_meta_data_from_sram()  
//
// Description: read the second and eighth longword of meta data from sram
//
// Outputs:							
//		out_xmeta:			the sram transfer registers to keep meta data 
//
// Inputs/Outputs:  
//		io_sigmask:			signal mask to be added with sig_sram_read_1
// 
// Inputs:  
//		in_buf_handle:		buffer handle to get meta data
//
// Constants:
//		none   
//
// Size:
//		4 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _mphy16_tx_read_meta_data_from_sram(out_xmeta, io_sigmask, \
								in_buf_handle)
	// add sig_sram_read_1 to io_sigmask
	_mphy16_tx_set_signal(io_sigmask, sig_sram_read_1)	; sig_sram_read_1
#ifdef DL_SPLIT_META_DATA
	#if (BUF_SRAM_SPLIT_LW < 3)  
	// add sig_sram_read_11 to io_sigmask only when it is used in 
	// dl_meta_load_cache()necessary, otherwise the transmission will stop, due 
	// to sig_sram_read_11 never return
	_mphy16_tx_set_signal(io_sigmask, sig_sram_read_11)	; sig_sram_read_11
	#endif // #if (BUF_SRAM_SPLIT_LW < 3)  
#endif
	// read meta data by dispatch_loop macro
	dl_meta_load_cache(in_buf_handle, out_xmeta, sig_sram_read_1, 0, 3)

#endm // end of macro _mphy16_tx_read_meta_data_from_sram()


///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_save_sbd_meta_to_lm[]
//
// Description: save secodary buffer data to local memory
//
// Outputs:							
//							None
//
// Inputs:  
//							None
//
// Constants:
//							None  
//
// Size: 7 instructions. (Worst case cycle count)
//
/////////////////////////////////////////////////////////////////////////////
#macro _mphy16_tx_save_sbd_meta_to_lm[]
.begin 
.reg v_buf_size tmp_nsbd tmp_whole
	dl_meta_get_buffer_next[tmp_whole]
	; only need lower three bytes
	ld_field_w_clr[tmp_nsbd, 0111, tmp_whole]
	alu[*l$index1[SBD_NEXT_BD_OFFSET], --, b, tmp_nsbd]
	alu[--, *l$index1[SBD_NEXT_BD_OFFSET], -, IX_NULL]	; is null (0xFF)?
	beq[finish_save_sbd_meta_to_lm#], defer[2]
 	  	//update secondary buffer size, offset always zero
		dl_meta_get_buffer_size[v_buf_size]
		ld_field_w_clr[*l$index0[SBD_1_OFFSET], 1100, v_buf_size, <<PAYLO_RMND_LOC] 
	 
	alu_shf[*l$index0[SBD_0_OFFSET], *l$index0[SBD_0_OFFSET], or, 1, <<RD_NBD_BIT_LOC]	; set ab_rd_nbd_flag
finish_save_sbd_meta_to_lm#:
.end // v_buf_size tmp_nsbd tmp_whole
#endm // end of #macro _mphy16_tx_save_sbd_meta_to_lm[] 


///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_move_not_sop_paylo_to_tbuf[] 
//
// Description: move payload of not_sop mpacket to tbuf 
//
// Outputs:							
//							None
//
// Inputs:  
// 		indir:				indirect reference used in dram[tbuf_wr...] 
//							instruction
//		dramaddr:			dram address of the payload
//		bd:					meta data handle for active buffer 
//		addrtbuf:			the starting address of tbuf for payload 
//		indirbase:			base for  indir
//      offset: 			the starting address of the payload to copy
//		tcw:				transmit control word which already contains 
//							payload length and payload offset
//		sig_dram:			signal number for dram[tbuf_wr...] instruction
//
// Constants:
//							None   
//
// Size:
//		11 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_mphy16_tx_move_not_sop_paylo_to_tbuf[indir, dramaddr, bd, addrtbuf, indirbase, offset, tcw, sig_dram] 
.begin
.reg t_paylo_len
.reg t_paylo_offset
.reg tmpx
	// prepare tbuf address for  dram[tbuf_wr......]
	alu[indir, indirbase, or, addrtbuf, <<5]

	ld_field_w_clr[t_paylo_len, 0001, tcw, >>TCW0_PAYLOAD_LEN_LSB_LOC]
	ld_field_w_clr[t_paylo_offset, 0001, tcw, >>TCW0_PAYLOAD_OFFSET_LSB_LOC]
	alu[t_paylo_offset, t_paylo_offset, and, TCW0_PAYLOAD_OFFSET_MASK]

	// get true bd  
	alu_shf[tmpx, --, b, bd, <<2]
    // use following dl_buf_get_data_from_meta macro from dispatch_loop.uc to
	// get dram address at the beginning of the buffer from true_dl_buf_hdl
	// now it takes one instruction
    dl_buf_get_data_from_meta[dramaddr, tmpx]

	// total we need to fecth (t_paylo_len + t_paylo_offset) bytes
	// to tbuf
	alu[tmpx, t_paylo_len, +, t_paylo_offset]

    // since ref count 0 will fectch 8 bytes, and ref count 1 will fetch
	// 16 bytes......, and ref count 15 will fetch 128 bytes, 
	// the right way to do it is followinhg:
	//    a) decrease tm_payload_len by one
	//    b) then right shift the result by 3
	// We will do step a first, then combine the right shift in the shift into
	// the ref count in indir_ref.  In this way, we make sure that we fetch 
	// enough and closet to 8 byte alignment of packet data	
	alu[tmpx, tmpx, -, 0x01]	
	// right shift tm_payload_len to get rid of any bits set in bit 0 to 3 to 
	// get ref_cnt for indir_ref 
    alu_shf[tmpx, --, b, tmpx, >>3]
    // left shift 21 bit to ref_cnt location
    alu[--, indir, or, tmpx, <<21] 

    // how many bytes of packet payload are copied to tbuf from dram is 
	// controlled by Ref Cnt field in indir_ref, following max_16 is
	// dummy, offset is added to dramaddr to get the actual address to start 
	// fetch, and the lower 3 bits of the address is ignored by dram channel	 
    dram[tbuf_wr, --, dramaddr, offset, max_16], indirect_ref, \
													sig_done[sig_dram]
.end // t_paylo_len t_paylo_offse tmpx
#endm // end of macro _mphy16_tx_move_not_sop_paylo_to_tbuf[]


///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_secondary_buffer_become_active_buffer[]
//
// Description: replace active buffer data with secondary buffer data in  
//              queue entry
//
// Outputs:							
//							None
//
// Inputs:  
//							None
//
// Constants:
//							None 
//
// Size: 
//		5 instructions. (Worst case cycle count)
//
///////////////////////////////////////////////////////////////////////////////
#macro _mphy16_tx_secondary_buffer_become_active_buffer[]
	alu[*l$index0[ABD_0_OFFSET], --, b, *l$index0[SBD_0_OFFSET]]
	alu[*l$index0[ABD_1_OFFSET], --, b, *l$index0[SBD_1_OFFSET]]
	alu[*l$index0[SBD_0_OFFSET], --, b, *l$index1[SBD_NEXT_BD_OFFSET]]
	alu[*l$index0[SBD_1_OFFSET], --, b, 0]		; reset
	alu[*l$index1[SBD_NEXT_BD_OFFSET], --, b, IX_NULL]	; reset
#endm // end of macro 	_mphy16_tx_secondary_buffer_become_active_buffer[]


///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_leftover_bytes_alignment[]
//
// Description: do necessary bytes alignment with offset, so the leftover 
//              will be realigned in first longword without any offset 
//              to the first byte of the long word
//
//
// Outputs:							
//		out1:				leftover longword which will be saved in local
//                          memory
//
// Inputs:  
// 		offset:				offset of leftover starting byte in the input four 
//							dram xfer registers which hold the leftover bytes
//		in0:				fisrt dram xfer registers which hold the leftover
//							bytes
//		in1:				second dram xfer registers which hold the leftover
//							bytes
//		in2:				third dram xfer registers which hold the leftover 
//							bytes
//		in3:				fourth dram xfer registers which hold the leftover 
//							bytes
//
// Constants:
//							None   
//
// Size: 
//		15 instructions. (Worst case cycle count)
//
///////////////////////////////////////////////////////////////////////////////
#macro _mphy16_tx_leftover_bytes_alignment[offset, in0, in1, in2, in3, out0]
	alu[--, offset, -, 4]
	beq[offset_is_four_bytes#]
	blt[offset_less_than_four_bytes#]
offset_greater_than_four_bytes#:
	alu[offset, offset, -, 4]			; make offset less than 3
	local_csr_wr[BYTE_INDEX, offset]	; need 3 cycles for local_csr_wr to settle
	nop
	nop
	nop
	byte_align_be[--, in1]
	byte_align_be[out0, in2]
	br[alignment_done#]
offset_less_than_four_bytes#:
	local_csr_wr[BYTE_INDEX, offset]; need 3 cycles for local_csr_wr to settle
	nop
	nop
	nop
	byte_align_be[--, in0]
	byte_align_be[out0, in1]
	br[alignment_done#]
offset_is_four_bytes#:
	alu[out0, --, b, in1]
alignment_done#:
#endm // end of macro _mphy16_tx_leftover_bytes_alignment[]

#ifdef COUNTERS
///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_update_bytes_transmitted[]
//
// Description: update the bytes tranmitted 
//
// Outputs:							
//							None
//
// Inputs:  
// 		tcw:				transmit control word which has payload length, 
//                          prepend length, and port 
//		sram_add_sig:		signal for sram add operation
//		bytecnt				sram xfer register to keep the value to add to 
// 							counters in sram
//
// Constants:
//							None 
//
// Size: 
//		13 instructions. (Worst case cycle count)
//
///////////////////////////////////////////////////////////////////////////////
#macro _mphy16_tx_update_bytes_transmitted[tcw, sram_add_sig, bytecnt]
.begin
.reg port bytes_txed paylo cntr_addr

	// skip bit set ?
    br_bset[tcw, SKIP_TRANSMIT_FLAG_BIT, bytes_txed_update_done#], defer[2]
		alu[cntr_addr, --, b, 0]		; dummy for SKIP case
		alu[bytecnt, --, b, 0]		; dummy  for SKIP case
	alu[port, tcw, and, PORT_MASK_IN_TCW]			; get port
	alu_shf[cntr_addr, --, b, port, <<4]			; get counter addr
	alu[cntr_addr, cntr_addr, +, @cntr_base]		; add base
	alu_shf[bytes_txed, --, b, tcw, >>TCWO_PREPEND_LEN_LSB_LOC] ; prepend
	alu[bytes_txed, bytes_txed, and, 0x1F]			; mask out
	alu_shf[paylo, --, b, tcw, >>TCW0_PAYLOAD_LEN_LSB_LOC] ; payload
	alu[bytes_txed, bytes_txed, +, paylo]
	alu[bytecnt, --, B, bytes_txed]				; get bytes txed
bytes_txed_update_done#:
	sram[add, bytecnt, cntr_addr, BYTE_TRANSMITTED_CNT_IDX],\
	     sig_done[sram_add_sig]			 	
.end //	port bytes_txed paylo cntr_addr 	 
#endm // end of macro _mphy16_tx_update_bytes_transmitted[]

	
///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_update_pkts_dropped[]
//
// Description: update the packet counts dropped for port due to queue full
//				for that port
//
// Outputs:							
//							None
//
// Inputs:  
// 		port:				the port to discard the packet 
//                        
// Constants:
//							None 
//
// Size: 
//		3 instructions.
//
///////////////////////////////////////////////////////////////////////////////
#macro _mphy16_tx_update_pkts_dropped[port]
.begin
.reg cntr_addr
	alu_shf[cntr_addr, --, b, port, <<4]				; get counter addr
	alu[cntr_addr, cntr_addr, +, @cntr_base]			; add base
	sram[incr, --, cntr_addr, PKT_DROPPED_CNT_IDX]	    ; increment pkts dropped
.end //	cntr_addr	 
#endm // end of macro _mphy16_tx_update_pkts_dropped[]
#endif //#ifdef	COUNTERS


/////////////////////////////////////////////////////////////////////////////////////
// 
// _mphy16_tx_check_tbuf_full()
//
// Description:
// 	
//		Check whether tbuf is full to avoid overwritting tbuf
//
// Outputs: 
//		None
//
// Inputs/Outputs:
//		None
//
// Inputs:  
//		in_tbufele:		tbuf element number which will be used for tx
//
// Constants:
//		None
//
// Labels:
//		TBUF_FULL_LABEL	symbolic address to wait until tbuf not full								not full 

// Size: 
//		10 instructions
// 
////////////////////////////////////////////////////////////////////////////////////
#macro _mphy16_tx_check_tbuf_full(in_tbufele, TBUF_FULL_LABEL)
.begin	
.reg result_reg tmp_tbuf
	alu[result_reg, TBUF_ELE_BITS_MASK, AND, $mpkts_sent0]	; get mpkts actually sent
	alu[--, in_tbufele, -, result_reg]
	bge[cont_comp_tbuf_tx_sequnce#], defer[1]
		alu[tmp_tbuf, --, b, in_tbufele]
	
	alu[tmp_tbuf, tmp_tbuf, +, TBUF_ELE_NUM]	; wrap around, add tbuf element number
cont_comp_tbuf_tx_sequnce#:
	alu[result_reg, tmp_tbuf, -, result_reg]	; compare with totals in tbuf
	alu[--, result_reg, -, TBUF_WAIT_THRESHOLD]	; compare with threshold
	bge[TBUF_FULL_LABEL] 							; go to wait  
.end //result_reg tmp_tbuf
#endm // end of #macro _mphy16_tx_check_tbuf_full()

/////////////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_wait_till_tbuf_not_full()
//
// Description:
// 	
//		Wait till tbuf not full to avoid overwritting
//
// Outputs: 
//								none
//
// Inputs:  
//		in_tbufele:				tbuf element number which will be used for tx
//
// Constants:
//								none
//
// Labels:
//		READTY_TO_TRANSMIT_LABEL	symbolic address to continue trnasmit after 
//									tbuf not full 
//
// Size: 
//		uncertain, depend on hardware performance
//
////////////////////////////////////////////////////////////////////////////////////
#macro _mphy16_tx_wait_till_tbuf_not_full[in_tbufele, READTY_TO_TRANSMIT_LABEL]

.begin	
.reg result tmp_tbuf

read_tx_seuence_loop#:
	// read the actual number of mpkts out of tbuf
	msf[read, $mpkts_sent0, tx_seq_0_addr, 0, 1], sig_done[sig_msf_access_3]	
wait_tx_seq_read_done#:
	nop
	br_!signal[sig_msf_access_3, wait_tx_seq_read_done#]	
	alu[result, TBUF_ELE_BITS_MASK, AND, $mpkts_sent0]	; get mpkts actually sent
	alu[--, in_tbufele, -, result]
	bge[cont_comp_tbuf_tx_sequnce#], defer[1]
		alu[tmp_tbuf, --, b, in_tbufele]		
	alu[tmp_tbuf, tmp_tbuf, +, TBUF_ELE_NUM]	; wrap around, add tbuf element number
cont_comp_tbuf_tx_sequnce#:
	alu[result, tmp_tbuf, -, result]			; compare with totals in tbuf
	alu[--, result, -, TBUF_WAIT_THRESHOLD]		; compare with threshold
	bge[read_tx_seuence_loop#] 					; loop again
	br[READTY_TO_TRANSMIT_LABEL] 			 
.end //result tmp_tbuf

#endm // end of #macro _mphy16_tx_wait_till_tbuf_not_full()

///////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_free_buffer()
//
// Description: free buffer handle by dispatch loop macro dl_buf_free
//
// Outputs:							
// 		none
//
// Inputs:  
//		in_buf_handle:		buffer handle to free 
//
// Constants:
//		none   
//
// Size:
//		4 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro	_mphy16_tx_free_buffer(in_buf_handle)
.begin
	// free buffer handle  
#ifdef UNIT_TEST
	alu[@buf_freed, @buf_freed, +, 1]
#endif //UNIT_TEST

#ifndef SKIP_FREE_BD
     // use dl_buf_free in dl_buf.uc in dispatch_loop to free buffer handle
	dl_buf_free(in_buf_handle, BUF_FREE_LIST0)
#endif //#ifndef SKIP_FREE_BD
.end	

#endm // end of #macro _mphy16_tx_free_buffer()


/////////////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_clear_signal()
//
// Description:
//		Clear a signal from a signal mask 
//
// Outputs: 
//		None 
//
// Inputs/Outputs:  
//		io_sigmask:			signal mask to be cleared with in_sig
//
// Inputs:  
//		in_sig:				signal number to clear
//
// Constants
//
//		None
//
// Size: 
//
//		1 instructions
// 
//
/////////////////////////////////////////////////////////////////////////////////////

#macro _mphy16_tx_clear_signal(io_sigmask, in_sig)

.begin

#if ( isnum(in_sig) )
	
	#define_eval SIG_SHIFT sig
	alu_shf[io_sigmask, io_sigmask, AND~, 1, <<SIG_SHIFT ]
	#undef SIG_SHIFT

#else

	alu[io_sigmask, io_sigmask, AND~, 1, <<(&in_sig)]

#endif

.end

#endm // end of #macro _mphy16_tx_clear_signal()

/////////////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_set_signal()
//
// Description:
// 	
//		Set a signal in a signal mask 
//
// Outputs: 
//		None 
//
// Inputs/Outputs:  
//		io_sigmask:			signal mask to be added with in_sig
//
// Inputs:  
//		in_sig:				signal number to set
//
// Constants
//		None
//
// Size: 
//
//		1 instruction
// 
//
////////////////////////////////////////////////////////////////////////////////////

#macro _mphy16_tx_set_signal(io_sigmask, in_sig)

.begin
#if ( isnum(in_sig) )

	#define_eval SIG_SHIFT in_sig
	alu_shf[io_sigmask, io_sigmask, OR, 1, <<SIG_SHIFT]
	#undef SIG_SHIFT

#else

	alu[io_sigmask, io_sigmask, OR, 1, <<(&in_sig)]

#endif

.end

#endm // end of #macro _mphy16_tx_set_signal()


/////////////////////////////////////////////////////////////////////////////////////
// _mphy16_tx_set_push_signal()
//
// Description:
// 	
//		Add push signal (i.e. dram access has puu and push signal) in a signal mask 
//
// Outputs:
//		None 
//
// Inputs/Outputs:  
//		io_sigmask:			signal mask to be added with push signal of in_sig
//
// Inputs:  
//		in_sig:				signal number to set
//
// Constants
//		None
//
// Size: 
//
//		1-4 instructions
// 
//
////////////////////////////////////////////////////////////////////////////////////
#macro _mphy16_tx_set_push_signal(io_sigmask, in_sig)
.begin
.reg tmp
#if ( isnum(in_sig) )

	#define_eval SIG_SHIFT (in_sig +1)
	alu_shf[io_sigmask, io_sigmask, OR, 1, <<SIG_SHIFT]
	#undef SIG_SHIFT

#else
	alu[tmp, --, B, &in_sig]
	alu[tmp, tmp, +, 1]
	alu[--, tmp, OR, 0]										
	alu_shf[io_sigmask, io_sigmask, OR, 1, <<indirect]

#endif
.end

#endm // end of #macro _mphy16_tx_set_push_signal()


#endif	// __MPHY16_TX_UTIL_UC__
