///////////////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
///////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: pkt_tx_16p.h
//
//      Purpose: Header file for mulitport (up to 16 ports) PACKET transmitter 
//				 for Egress IXP24XX/IXP28XX
//
///////////////////////////////////////////////////////////////////////////////


#ifndef	__PKT_TX_P16_H__
#define	__PKT_TX_P16_H__

///////////////////////////////////////////////////////////////////////////////
//
//	Definition of packet tx ME numbers  
//
///////////////////////////////////////////////////////////////////////////////

#define PACKET_TX_FIRST_ME		6
#define PACKET_TX_SECOND_ME		7

///////////////////////////////////////////////////////////////////////////////
//
//	Definition of port number and handled MEs
//
///////////////////////////////////////////////////////////////////////////////

#if TX_PHY_MODE==MPHY_16

#define TOTAL_PORTS							16

// ports controlled by first Packet TX ME (port 0 to 7)
#define	PORTS_CTRLED_BY_FIRST_PKT_TX_ME		0x00FF

// ports controlled by second Packet TX ME (port 8 to 15)
#define	PORTS_CTRLED_BY_SECOND_PKT_TX_ME	0xFF00

#endif // #if TX_PHY_MODE==MPHY_16

#if TX_PHY_MODE==SPI_4_16PORTS

#define TOTAL_PORTS							16

// ports controlled by first Packet TX ME (port 0 to 7)
#define	PORTS_CTRLED_BY_FIRST_PKT_TX_ME	0x00FF

// ports controlled by second Packet TX ME (port 8 to 15)
#define	PORTS_CTRLED_BY_SECOND_PKT_TX_ME	0xFF00

#endif // #if TX_PHY_MODE==SPI_4_16PORTS

#if TX_PHY_MODE==SPI_4_10PORTS

#define TOTAL_PORTS							10

// ports controlled by first Packet TX ME (port 0 to 4)
#define	PORTS_CTRLED_BY_FIRST_PKT_TX_ME		0x001F

// ports controlled by second Packet TX ME (port 5 to 9)

#define	PORTS_CTRLED_BY_SECOND_PKT_TX_ME	0x03E0

#endif // #if TX_PHY_MODE==SPI_4_10PORTS

#define FIRST_PORT_IN_FIRST_PKT_TX_ME	0

#define FIRST_PORT_IN_SECOND_PKT_TX_ME	(TOTAL_PORTS / 2)

#define LAST_PORT_IN_FIRST_PKT_TX_ME	(FIRST_PORT_IN_SECOND_PKT_TX_ME - 1)

#define LAST_PORT_IN_SECOND_PKT_TX_ME	(TOTAL_PORTS - 1)

#define PORTS_BY_ONE_ME					(TOTAL_PORTS / 2)

///////////////////////////////////////////////////////////////////////////////
//
//	Definition of tbuf element size and derived definitions
//
///////////////////////////////////////////////////////////////////////////////

// TBUF_ELE_SIZE must be 128

#define TBUF_ELE_SIZE				128
#define TBUF_ADDR_SHF				7

#define TOTAL_TBUF_SIZE 			8192
#define TOTOL_THREADS_PER_ME		8


#define_eval TBUF_ELE_NUM	(TOTAL_TBUF_SIZE / TBUF_ELE_SIZE)

// definition of the mask to get the next available tbuf element from what
// gotten from scratch memory by atomic operation

#define_eval TBUF_ELE_BITS_MASK ( (TOTAL_TBUF_SIZE / TBUF_ELE_SIZE) - 1)

// definition of the maximum mapckets per thread

#define MAX_MPKT_PER_THREAD				3

// definition of the threshold to wait to avoid overwriting existing mpackets
// not transmitted out of tbuf yet

#define TBUF_WAIT_THRESHOLD	    (TBUF_ELE_NUM - MAX_MPKT_PER_THREAD)

// definition of the mask to get mpackets transmitted out tbuf from 
// Tx_Sequence_# and get tbuf element number to compare with mpackets
// transmitted  from absolute tbuf element number allocated from scratch 
// memory by atomic operation

#define CHECK_TBUF_ELE_MASK			0xFF

///////////////////////////////////////////////////////////////////////////////
//
//	Definition of MSF control registers
//
///////////////////////////////////////////////////////////////////////////////

// MSF_TX_CNTL

#if IS_IXPTYPE(__IXP24XX)
#ifndef TX_MPHY_LEVEL2
// Tx enable, UTOPIA/POS, MPHY-32, Level 3 operation, 128 bytes tbuf 
#define MPHY_16_POS_TX_CNTL		0x100E0004	;0001 0000 0000 1110 
											;0000 0000 0000 0100
#else TX_MPHY_LEVEL2
// Tx enable, UTOPIA/POS, MPHY-32, Level 2 operation, 128 bytes tbuf 
#define MPHY_16_POS_TX_CNTL		0x100E4004	;0001 0000 0000 1110 
											;0100 0000 0000 0100 
#endif	// #ifndef TX_MPHY_LEVEL2

#define DISABLED_MSF_TX_CNTL	0x00000004	; disbaled MSF Tx Control

// shift to get tcw validate address offset from tbuf element number

#define TCW_VALID_ADDR_SHFT		3			
#endif //#if IS_IXPTYPE(__IXP24XX)

#if IS_IXPTYPE(__IXP28XX)

#ifdef IX_PLATFORM_2801
// For hardware platform IXDP2801 we need specific MSF_TX_CONTROL
// programming consistent with used hardware
//
#ifdef INTERNAL_2800_CLOCK_DIVIDE

#define MPHY_16_POS_TX_CNTL		0x20102d04	;0010 0000 - SPI-4 Tx enable, 
											;0001 0000 - TXCLCK divided, 
											;0010 1101 - Rising CLK edge, RXCLK divided, TX enable
											;0000 0100 - 128B ele, 1 way partition
#else //EXTERNAL clock used

#define MPHY_16_POS_TX_CNTL		0x20002504	;0010 0000 0000 0000 SPI-4 Tx enable
											;0010 0101 0000 0100 Rising CLK edge,128B ele,1 way partition

#endif /* INTERNAL_2800_CLOCK_DIVIDE */

#else //NOT IX_PLATFORM_2801

// SPI-4 Tx enable, 128 bytes tbuf element, 1 way partition

#define MPHY_16_POS_TX_CNTL		0x20000504	;0010 0000 0000 0000 
											;0000 0101 0000 0100 

#endif //#ifdef IX_PLATFORM_2801

#define DISABLED_MSF_TX_CNTL	0x00000004	; disbaled MSF Tx Control

// shift to get tcw validate address offset from tbuf element number

#define TCW_VALID_ADDR_SHFT		4			
#endif //#if IS_IXPTYPE(__IXP28XX)

// POS_TX_UP_CTL_WD

#ifndef ODD_PARITY

//  packet, single bit even parity, 2 clock cycle

#define POS_TX_UP_CTL_WD		0x001D
	
#else	// ODD_PARITY

//  packet, single bit odd parity, 2 clock cycle

#define POS_TX_UP_CTL_WD		0x0019	

#endif	// #ifndef ODD_PARITY


///////////////////////////////////////////////////////////////////////////////
//
//	Definitions specific to each packet tx ME
//
///////////////////////////////////////////////////////////////////////////////

//	Scratch for communicating between Queue Manager and POS Transmit Blocks
//	Ring number 3, base 8192, and size 512 Long Words

#if THIS_ME==PACKET_TX_FIRST_ME
#define	PACKET_TX_SCR_IN_RING		PACKET_TX_IN_0
#endif // #if THIS_ME==PACKET_TX_FIRST_ME

#if THIS_ME==PACKET_TX_SECOND_ME
#define	PACKET_TX_SCR_IN_RING		PACKET_TX_IN_1
#endif // #if THIS_ME==PACKET_TX_SECOND_ME

#if THIS_ME==PACKET_TX_FIRST_ME
#define	PORTS_CTRLED_BY_THIS_ME		PORTS_CTRLED_BY_FIRST_PKT_TX_ME
#define FIRST_PORT_IN_THIS_ME		FIRST_PORT_IN_FIRST_PKT_TX_ME
#define LAST_PORT_IN_THIS_ME		LAST_PORT_IN_FIRST_PKT_TX_ME
#endif // #if THIS_ME==PACKET_TX_FIRST_ME

#if THIS_ME==PACKET_TX_SECOND_ME
#define	PORTS_CTRLED_BY_THIS_ME		PORTS_CTRLED_BY_SECOND_PKT_TX_ME
#define FIRST_PORT_IN_THIS_ME		FIRST_PORT_IN_SECOND_PKT_TX_ME
#define LAST_PORT_IN_THIS_ME		LAST_PORT_IN_SECOND_PKT_TX_ME
#endif // #if THIS_ME==PACKET_TX_SECOND_ME


///////////////////////////////////////////////////////////////////////////////
//
//	Bit definition of the gloabl execution status flag exe_stat_flag
//
///////////////////////////////////////////////////////////////////////////////

// definition of the bit location for freeing buffer in exe_stat_flag 

#define ESF_FREE_BUF_BIT			0
#define_eval ESF_FREE_BUF			(1 << ESF_FREE_BUF_BIT)

#define ESF_PICK_VIRTUAL_QUEUE_BIT	1


#define ESF_EOP_MPKT_THREAD_BIT		2
#define_eval ESF_EOP_MPKT_THREAD	(1 << ESF_EOP_MPKT_THREAD_BIT)

#define ESF_SOP_MPKT_THREAD_BIT		17

// bit signals that leftover read in phase 1 and need to be saved in local 
// memory in phase 2

#define ESF_READ_LEFTOVER_BIT 		18

// bit signals that next buffer meta read in phase 1 and need to be saved in 
// phase 2

#define ESF_READ_NEXT_BUF_META_BIT	19

// bit signals that leftover from previous buffer need to be txed in phase 2

#define ESF_LEFTOVER_TO_TX_BIT		20

// bit signals that the current buffer is the last buffer of the packet

#define ESF_LAST_BUF_BIT			21

///////////////////////////////////////////////////////////////////////////////
//
//	Definition of the queue entry in local memory
//
///////////////////////////////////////////////////////////////////////////////

// each entry occupies 4 longwords (16 bytes)

#define QUEUE_ENTRY_SIZE			0x10		



// queue entries for each port occupies 64 lonwords (256 byts)

#define PORT_QUEUE_ENRY_SHFT		8	

#define_eval PORT_QUEUE_ENTRY_SIZE (1 << PORT_QUEUE_ENRY_SHFT)

// queue depth for each port

#define_eval QUEUE_DEPTH			(PORT_QUEUE_ENTRY_SIZE / QUEUE_ENTRY_SIZE)
  
#define_eval QUEUE_DEPTH_MINUS_ONE	(QUEUE_DEPTH - 1)

// upper bound of queue head/tail offset

#define_eval MAX_QUEUE_HEAD_TAIL	(QUEUE_DEPTH_MINUS_ONE * QUEUE_ENTRY_SIZE)

// least significant bit of queue head offset in port_queue_status longword

#define QUEUE_HEAD_SHFT				0x0
#define HEAD_MASK					0xFF

// least significant bit of queue tail offset in port_queue_status longword

#define QUEUE_TAIL_SHFT		8
#define TAIL_MASK			0xFF

// amount to add to increase tail_offet to next entry in queue status info

#define_eval INCR_TAILOFFSET		(QUEUE_ENTRY_SIZE <<QUEUE_TAIL_SHFT)

// starting local memory address of queue entries for ports

#define PORT_QUEUE_ENTRY_BASE		0

#define QUEUE_STATUS_BASE			0x800
 

// each port occupies 16 longword

#define QUEUE_STATUS_SHFT			6

// the least significant bit location of pkt_in_queue in port queue status

#define PKT_IN_QUEUE_SHFT			16

// mask for pkt_in_queue in port queue staus

#define PKT_IN_QUEUE_MASK			0xFF

#define VIRTUAL_QUEUE_SHFT			24

#define VIRTUAL_QUEUE_MASK			0xFF

#define PORT_MASK_IN_TXR			0x0F

#define_eval HEAD_INCR (QUEUE_ENTRY_SIZE)

// amount to add to increase pkts_in_queue by one in queue status info

#define_eval INCR_PKTS_IN_QUEUE		(1 <<PKT_IN_QUEUE_SHFT)

///////////////////////////////////////////////////////////////////////////////
//
//	Definition of the inex of queue entry in local memory
//
///////////////////////////////////////////////////////////////////////////////

#define CUR_BUF_0		0  

#define CUR_BUF_1		1

#define NEXT_BUF_0		2

#define NEXT_BUF_1		3

#define PAYLO_RMND_LOC	16

#define RD_NEXT_BUF_BIT	30

#define SOP_FLAG_BIT	31

///////////////////////////////////////////////////////////////////////////////
//
//	Definition of the port status entries in local memory
//
///////////////////////////////////////////////////////////////////////////////

// definition of the offset of queue status (queue,  queue tail, queue count) 

#define PORT_QUEUE_STATUS					4

// definition of the offset of packets transmitted for specific port

#define PKT_TXED							5

// definition of the offset of remaining mpacket number without checking Tx 
// status element (flow control credit)

#define FC_CREDIT_REMAINING					6

// definition of the offset of PHY Tx FIFO flow control credit

#if IS_IXPTYPE(__IXP24XX)
#define FC_READY_CREDIT						7
#endif 	// #if IS_IXPTYPE(__IXP24XX)

#if IS_IXPTYPE(__IXP28XX)
#define FC_HUNGRY_CREDIT					7

#define FC_STARVING_CREDIT					8
#endif 	// #if IS_IXPTYPE(__IXP28XX)


// definition of the offset of leftover payload status and next buffer
// handle of next buffer 

#define LEFTOVER_STAT_NEXT_NEXT_HANDLE		9

// definition of the offset of the first longword of the leftover payload 

#define LEFTOVER_LW0						10

// definition of the offset of the second longword of the leftover payload 

#define LEFTOVER_LW1						11

// definition of the offset of the third longword of the leftover payload 

#define LEFTOVER_LW2						12

// definition of the offset of the forth longword of the leftover payload 

#define LEFTOVER_LW3						13

///////////////////////////////////////////////////////////////////////////////
// 
// Globals stored in local memory 
//
///////////////////////////////////////////////////////////////////////////////

//	We will use local memory to store a few globals. This is better than using 
//  absolute registers if we have a l$index unused because it reduces register
//	pressure and also makes it easier to run the code on fewer threads

#define TBUF_BASE_INDEX				0	; base to compute tbuf address
								
#define IND_REF_BASE_INDEX			1	; base for indirect ref for 
										; dram[tbuf_wr... ] 

#define	TCW0_SOP_EOP_BASE_INDEX		2	; base for tcw 0 for sop_eop mpacket

#define TX_REQ_RING_INDEX			3	; scratch ring for tx request

#define TBUF_VD_BASE_INDEX			14	; base to compute tbuf validation addr 

#ifdef CHECK_PORT_QUEUE_OVERFLOW
#define TX_REQUESTS_DISCARDED		15	; tx requests discarded for the port
										; to prevent the port queue overflow
#endif	// #ifdef CHECK_PORT_QUEUE_OVERFLOW

///////////////////////////////////////////////////////////////////////////////
//
//	Definition of some constants
//
///////////////////////////////////////////////////////////////////////////////

#define CONST_FF					0xFF

#define CONST_FC					0xFC	

#define CONST_F8					0xF8

#define CONST_0F					0x0F

#define CONST_07					0x07	

#define CONST_04					0x04	


///////////////////////////////////////////////////////////////////////////////
//
//	Supplemental hardware related definition 
//
///////////////////////////////////////////////////////////////////////////////

// definition of EOP bit in Transmit control word for POS-PHY

#define TCW0_EOP_BIT_SHFT				8

// definition of SKIP bit in Transmit control word for POS-PHY

#define TCW0_SKIP_BIT_SHFT				11

// definition of least significant bit location of fields of Transmit Control Word
// definition of least significant bit location of payload length field

#define	TCW0_PAYLOAD_LEN_LSB_LOC		24

// definition of least significant bit location of payload offset field

#define TCW0_PAYLOAD_OFFSET_LSB_LOC		13

// definition of least significant bit location of prepend length field 

#define TCWO_PREPEND_LEN_LSB_LOC		16

// definition of least significant bit location of prepend offest field  		

#define TCWO_PREPEND_OFFSET_LSB_LOC		21

// definition of the mask to get payload offset (3 bits)

#define TCW0_PAYLOAD_OFFSET_MASK		0x07

// definition of the base bits pattern of indir_ref used in instruction which 
// moves packet data from dram to tbuf

#define DRAM_INDIREF_BASE_LW	0x02000010	; overrid bit of Ref_Cnt field (bit
											; 25) override bit of RBUF/TBUF 
											; Byte addr field (bit 4) and must 
											; always be set


///////////////////////////////////////////////////////////////////////////////
//
//	Miscellaneous definition 
//
///////////////////////////////////////////////////////////////////////////////
	
// since Packet TX microblock doesn't need to refer to next neighbor, following 
// definition is dummy, just to eliminate a warning from dispatch_loop.h

#define DL_NEXT_ME						0

#define NUMBER_OF_THREAD				8


///////////////////////////////////////////////////////////////////////////////
//
//	Definition of thread specific flag
//
///////////////////////////////////////////////////////////////////////////////

// when this bit set, the thread will update flow control based on MSF flow
// control status

#define TSF_UPDATE_FC_BIT				0
#define_eval TSF_UPDATE_FC				(1 << TSF_UPDATE_FC_BIT)

// when this bit set, the thread will check whether enough margin to prevent 
// tbuf overwiting

#define TSF_CHECK_TBUF_FULL_BIT			1	
#define_eval TSF_CHECK_TBUF_FULL		(1 << TSF_CHECK_TBUF_FULL_BIT)

#define THREAD_0_SPECIFIC_FLAGS			0
#define THREAD_1_SPECIFIC_FLAGS			0
#define THREAD_2_SPECIFIC_FLAGS			0
#define THREAD_3_SPECIFIC_FLAGS			0
#define THREAD_4_SPECIFIC_FLAGS			0
#define THREAD_5_SPECIFIC_FLAGS			0
#define THREAD_6_SPECIFIC_FLAGS			0
#define THREAD_7_SPECIFIC_FLAGS			(TSF_UPDATE_FC + TSF_CHECK_TBUF_FULL)


///////////////////////////////////////////////////////////////////////////////
//
//	Definition flow control related information
//
///////////////////////////////////////////////////////////////////////////////


// definition of the default PHY TXFIFO depth

#if IS_IXPTYPE(__IXP24XX)

// assume PHY has 2K bytes TXFIFO and we set the threshhold to 2 times 
// TBUF_ELE_SIZE

#define DEFAULT_PHY_DEPTH	((2048 / TBUF_ELE_SIZE) - 1)
#define_eval  FC_LOW_WATERMARK		NUMBER_OF_THREAD
#endif	// #if IS_IXPTYPE(__IXP24XX)


#if IS_IXPTYPE(__IXP28XX)

#ifdef IX_PLATFORM_2801
/*
 * An IXDP2801 platform has different size of hardware TX buffers (POS has only 2kB per port)
 * so initialize FC depths with diffrent values. Allow to define these values upon compilation.
 */
#ifdef USE_PREDEFINED_FC_DEPTH
/*
 * Here use compilation time definitions for FC depths.
 */
#else /* USE_PREDEFINED_FC_DEPTH */

#define_eval DEFAULT_PHY_HUNGRY_DEPTH	4
#define_eval DEFAULT_PHY_STARVING_DEPTH	10

#define_eval  FC_LOW_WATERMARK		5 /* must be larger than DEFAULT_PHY_HUNGRY_DEPTH */

#endif /* USE_PREDEFINED_FC_DEPTH */

#else // IX_PLATFROM_2801
/* original code */


// following are based on IXF1110 10-Port Gigabit preliminary data sheet
// TX FIDFO: 4.5 KB per channel  4.5x1024 = 4608
// Default TX FIFO high watermark is  1584 bytes, the amount of data
// staored in FIFO exceeds the value, the TX FIFO indicates "SATISFIED"
// Default TX FIFO low watermark is  464 bytes, the amount of data
// staored in FIFO falls below this  value, the TX FIFO status indicates 
// "STAVING".

//#define_eval DEFAULT_PHY_HUNGRY_DEPTH	((4608 - 1584) / TBUF_ELE_SIZE)
//#define_eval DEFAULT_PHY_STARVING_DEPTH	((4608 - 464) / TBUF_ELE_SIZE)
// While the above formula is the theoretically correct computation of the
// hungry and starving depth, different working values are used below. Using
// the theoretical values causes FIFO overflow for jumbo frames. This
// happens because the FIFO satisfied notification when the FIFO goes from
// hungry  to satisfied comes too late. Therefore to avoid FIFO overflow
// we are  more conservative in setting the hungry and starving depths.
#define_eval DEFAULT_PHY_HUNGRY_DEPTH	3
#define_eval DEFAULT_PHY_STARVING_DEPTH	5


#define_eval  FC_LOW_WATERMARK		NUMBER_OF_THREAD

#endif // IX_PLATFROM_2801
// definition  of bit pattern to mask out MSB of flow control status MSF 
// register

#define	FC_CHECK_PATTERN			 0xAAAAAAAA

#endif	// #if IS_IXPTYPE(__IXP28XX)


///////////////////////////////////////////////////////////////////////////////
//
//	Definition specific to Layer 2 haeder
//
///////////////////////////////////////////////////////////////////////////////

#ifdef ADD_L2_HEADER

#define L2_HEADER_ALREADY_EXIST_ID	0xFFFF
#define	L2_HEADER_EXIST_BYTE_FLAG	0xFF
#define L2_HEADER_EXIST_BYTE_LOC	24 

#ifdef POS_TX

#define PPP_IP_PROTOCOL				0x0021
#define PPP_MPLS_PROTOCOL			0x0281	
#define NEXT_HOP_ID_MPLS			0x2000

#define BYTES_FOR_L2_HEADER			8

// bits to shift to get PPP header for IP

#define IP_PROTOCOL_SHFT			16

// prepend offset = 0, prepend length = 2
#define TCW0_SOP_PREPEND_BYTE		0x02

#define_eval TBUF_ELE_SIZE_MINUS_18 (TBUF_ELE_SIZE - 18)

#define L2_HEADER_SIZE				2	/* actual L2 header size in bytes */

#define BYTES_RESERVED_FOR_L2_HEADER_IN_TBUF	8 	/* in the beginning of each
		 tbuf element, the space in bytes used to keep L2 header due to 
		 multiple of 8 bytes limitation */
#endif // POS_TX

#ifdef ETHERNET_TX

#define BYTES_FOR_L2_HEADER			16

#define_eval TBUF_ELE_SIZE_MINUS_26 (TBUF_ELE_SIZE - 26)

#define L2_TABLE_ENTRY_SHFT			4

#define L2_TABLE_VALID_BIT			31

#define L2_TABLE_ENTRY_SIZE_IN_LW	4

#define L2_HEADER_SIZE				14	/* actual L2 header size in bytes */

#define BYTES_RESERVED_FOR_L2_HEADER_IN_TBUF	16	/* in the beginning of each
		 tbuf element, the space in bytes used to keep L2 header due to 
		 multiple of 8 bytes limitation */
#endif // ETHERNET_TX

#endif // ADD_L2_HEADER


#ifndef ADD_L2_HEADER

// first transmit control word base for SOP_EOP mpacket 
// SOP, EOP bit set, payload offet = 0 (will be filled later),
// channel = 0, will be filled later

#define BASE_TCW0_SOP_EOP	0x00000300	

// sop meta data to read in longword

#define META_DATA_LW_TO_READ		3

// max size (payload length + (offset mod by 8)) of a packet which can be 
// fitted into one single mpacket 

#define_eval SINGLE_MPKT_MAX_SIZE			TBUF_ELE_SIZE

#else //ADD_L2_HEADER

#ifdef POS_TX

// payload len = 0 (will be filled later), prepend offset = 0, 
// prepend length = 2

#define BASE_TCW0_SOP_EOP	0x00020300

// sop meta data to read in longword

#define META_DATA_LW_TO_READ		5

#else 	// Ethernet

// payload len = 0 (will be filled later), prepend offset = 0, 
// prepend length = 14

#define BASE_TCW0_SOP_EOP	0x002e0300

// sop meta data to read in longword

#define META_DATA_LW_TO_READ		5

#endif 	// #ifdef POS_TX

// max size (payload length + (offset mod by 8)) of a packet which can be 
// fitted into one single mpacket 

#define_eval SINGLE_MPKT_MAX_SIZE			(TBUF_ELE_SIZE - BYTES_FOR_L2_HEADER)


#endif 	// #ifndef ADD_L2_HEADER


#ifdef STRIP_4BYTES_CRC

// SINGLE_MPKT_MAX_SIZE +  length of CRC

#define_eval SINGLE_MPKT_PLUS_CRC_MAX_SIZE	(SINGLE_MPKT_MAX_SIZE + 4)

#endif	// #ifdef STRIP_4BYTES_CRC

///////////////////////////////////////////////////////////////////////////////
//
//	Short cuts for Reassembly Context in local memory
//
///////////////////////////////////////////////////////////////////////////////

#define lm1_port_queue_stat			*l$index1[PORT_QUEUE_STATUS]
#define lm1_pkt_txed				*l$index1[PKT_TXED]
#define lm1_fc_credit_remaining		*l$index1[FC_CREDIT_REMAINING]
#define lm1_fc_ready_credit			*l$index1[FC_READY_CREDIT]
#define lm1_fc_hungry_credit		*l$index1[FC_HUNGRY_CREDIT]
#define lm1_fc_starving_credit      *l$index1[FC_STARVING_CREDIT]
#define lm1_lov_stat_next_next_buf 	*l$index1[LEFTOVER_STAT_NEXT_NEXT_HANDLE]
#define lm1_leftover_lw0			*l$index1[LEFTOVER_LW0]
#define lm1_leftover_lw1			*l$index1[LEFTOVER_LW1]
#define lm1_leftover_lw2			*l$index1[LEFTOVER_LW2]
#define lm1_leftover_lw3			*l$index1[LEFTOVER_LW3]
#define lm1_tbuf_base				*l$index1[TBUF_BASE_INDEX]
#define lm1_tbuf_valid_base			*l$index1[TBUF_VD_BASE_INDEX]
#define lm1_tx_req_ring				*l$index1[TX_REQ_RING_INDEX]
#define lm1_indiref_base			*l$index1[IND_REF_BASE_INDEX]
#define lm1_tcw0_sop_eop_base		*l$index1[TCW0_SOP_EOP_BASE_INDEX]
#ifdef CHECK_PORT_QUEUE_OVERFLOW
#define lm1_tx_requests_discarded	*l$index1[TX_REQUESTS_DISCARDED]
#endif	// #ifdef CHECK_PORT_QUEUE_OVERFLOW

#define lm0_port_queue_stat			*l$index0[PORT_QUEUE_STATUS]

#define lm0_cur_buf_0				*l$index0[CUR_BUF_0]
#define lm0_cur_buf_1				*l$index0[CUR_BUF_1]
#define lm0_next_buf_0				*l$index0[NEXT_BUF_0]
#define lm0_next_buf_1				*l$index0[NEXT_BUF_1]

///////////////////////////////////////////////////////////////////////////////
//
//	Global Signals
//
///////////////////////////////////////////////////////////////////////////////

.sig sig_scratch_access_1 			; signal for scratch ring read
.sig sig_scratch_access_2 			; signal for scratch memory read
.sig volatile sig_next_context		; signal to wakeup next thread, declared as
									; volatile to eliminate assembling warning
.sig sig_sram_access_1 				; signal 1 for sram read
.sig sig_msf_access_1				; signal 1 for msf access (read or write)
.sig sig_msf_access_2				; signal 2 for msf access (read or write)
.sig sig_dram_access_1				; signal 1 for dram access (dram[tbuf_wr..])
.sig volatile common_init_complete_sig_num		; this is the inter-ME signal 
									; generated by the ME whcih initializes
									; common resources, such as scratch ring,
									; Packet TX MEs will wait for this signal at
									; end of their initialization to make 
									; sure all common resource are available 

//	Manual signal allocation

.addr common_init_complete_sig_num	ME_INIT_SIGNAL	; manual signal allocation,
													; ME_INIT_SIGNAL defined as
													; in dl_system.h  


#ifdef COUNTERS
.sig sig_counter_update								; signal for update counter
.addr sig_counter_update	SCRTACH_ACCESS_1_SIG	; manual signal allocation
													; reuse this signal
													; there is no conflict	
#endif //#ifdef COUNTERS
    
///////////////////////////////////////////////////////////////////////////////
//
//	Definition of sram global counters for core component
//
///////////////////////////////////////////////////////////////////////////////

// There are total 16 ports.  Each port occupies four longword (3 counters 
// plus one reserved longword). They are
//	PKT_TRANSMITTED_CNT
//	PKT_DROPPED_CNT
//	BYTE_TRANSMITTED_CNT
//	RESERVED_CNT
//  The offset of the port counter data is (PACKET_TX_COUNTER_BASE +  PORT_OFFSET)
//	where PORT_OFFSET = port number << 4 

#ifdef COUNTERS
#define		PKT_TRANSMITTED_CNT_IDX		0x00
#define		PKT_DROPPED_CNT_IDX			0x04
#define		BYTE_TRANSMITTED_CNT_IDX	0x08
#define 	RESERVED_CNT_IDX			0x0C
#define     PORT_MASK_IN_TCW            0x0F
#endif	// #ifdef COUNTERS

///////////////////////////////////////////////////////////////////////////////
//
//	Definition of constants related to device bus width
//
///////////////////////////////////////////////////////////////////////////////


#if IS_IXPTYPE(__IXP24XX)

#define BUS_WIDTH_MASK					0xFC
 
#define BUS_WIDTH						0x04

// no matter how long the leftover is, we need to reserve 8 bytes in beginning
// of tbuf to fill leftover with msf[write....]

#define LEFTOVER_TBUF_SPACE				0x08

// in some case, the sum offset_mod_8 + bytes_leftvoer will over 8 bytes 
// (for the worst case, offset_mod_8=7, and bytes_leftover is 3), so in 
// order to cover all cases, read 16 bytes here 

#define LEFTOVER_READ_LENGTH	2

#endif 	// #if IS_IXPTYPE(__IXP24XX)

#if IS_IXPTYPE(__IXP28XX)

#define BUS_WIDTH_MASK					0xF0
 
#define BUS_WIDTH						0x10

// for simplicity we reserve 16 bytes in beginning of tbuf to fill leftover with 
// msf[write....], since maximum leftover in IXP28XX is 15 bytes

#define LEFTOVER_TBUF_SPACE				0x10

// in some case, the sum offset_mod_8 + bytes_leftvoer will over 16 bytes 
// (for the worst case, offset_mod_8=7, and bytes_leftover is 15), so in 
// order to cover all cases, read 24 bytes here 

#define LEFTOVER_READ_LENGTH	3

#endif 	// #if IS_IXPTYPE(__IXP28XX)

///////////////////////////////////////////////////////////////////////////////
//
//	Definition of constants related to leftover from previous buffer
//
///////////////////////////////////////////////////////////////////////////////

#define LM_LEFTOVER_FLAG_BIT		31

#define LM_LEFTOVER_LEN_BIT			24

#define LM_LEFTOVER_LEN_MASK		0x3F	

#define_eval LEFTOVER_MPKT_MAX_LENGTH (TBUF_ELE_SIZE - LEFTOVER_TBUF_SPACE)

#endif	//	__PKT_TX_P16_H__
