///////////////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
///////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: pkt_tx_16p_init.uc
//
//      Purpose: Initialization macros for mulitport (up to 16 ports) PACKET 
//				 transmitter for Egress IXP24XX/IXP28XX
//
///////////////////////////////////////////////////////////////////////////////


#ifndef	__PKT_TX_P16_INIT_UC__
#define	__PKT_TX_P16_INIT_UC__

#ifndef	PACKET_TX_FIRST_ME
#warning	"PACKET_TX_FIRST_ME not defined. Defaulting to ME 0."
#define	PACKET_TX_FIRST_ME 0
#endif

#ifndef	PACKET_TX_SECOND_ME
#warning	"PACKET_TX_SECOND_ME not defined. Defaulting to ME 1."
#define	PACKET_TX_SECOND_ME 1
#endif

#if	((PACKET_TX_SECOND_ME - PACKET_TX_FIRST_ME) <= 0)
#warning "PACKET_TX_SECOND_ME should be greater than PACKET_TX_FIRST_ME."
#endif

#ifndef	THIS_ME
#error	"**Warning! You need to define THIS_ME in assember option."
#endif

#if ((PACKET_TX_FIRST_ME<0) || (PACKET_TX_FIRST_ME>8) || \
     (PACKET_TX_SECOND_ME<0)  || (PACKET_TX_SECOND_ME>8)  || \
	 (THIS_ME<0)     || (THIS_ME>8))
#error	"ME's range from 0-7."
#endif

#ifndef	TBUF_ELE_SIZE
#error	"**Warning! You need to define TBUF_ELE_SIZE."
#endif


#if (TBUF_ELE_SIZE != 128)
#error	"**Warning! TBUF_ELE_SIZE must be 128."
#endif

#ifndef TBUF_ELE_SIZE
#define TBUF_ELE_SIZE 128
#error	"**TBUF_ELE_SIZE set to 128."

#endif


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_gprs_init()
//
// Description: initialize context relative GPRs to save instruction cycles 
//              in Packet TX processing
//
// Outputs: 
//      out_tbuf_scr_loc:		scratch memory location managing next available
//                          	tbuf element
//      out_queue_stat_base:	base address in local memory for queue status 
//								info 
//      out_addr_msf_fc_stat:	MSF status register for PHY flow control
//                          	dram to tbuf
// 		out_cap_addr_base:  	Address base used in CAP (calculated 
//								addressing) to do reflect write to notify 
//								scheduler how many packets have been 
//								transmitted for the specific port
//
// Inputs:  
//		None
//
// Constants:
//		RBUF_TBUF_LW:	 			starting address of TBUF  
//      QM_TO_PACKET_TX_SCR_RING:	scratch ring number used to get tx requet
//      TX_MPHY_STATUS:				Tx_MPHY_Status MSF register address 
//									(IXP24XX)
//		TX_MULTIPLE_PORT_STATUS_0:  Tx_Multiple_Port_Status_0 MSF register 
//									(IXP28XX)
//      DRAM_INDIREF_BASE_LW:		base for indirect refernce in moving 
//									payload from dram to tbuf
//
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_gprs_init(out_tbuf_scr_loc, out_queue_stat_base, \
									out_addr_msf_fc_stat, out_cap_addr_base)

.begin

	// scratch memory location PACKET_TX_AVAIL_TBUF_ELE_SCR_ADDR keep next 
	// available tbuf element, which is initialized to 0, later get and 
	// increment by one by SCRATCH atomic operation SCRACTH[test_and_incr..] 

	immed32(out_tbuf_scr_loc, PACKET_TX_AVAIL_TBUF_ELE_SCR_ADDR)

	// starting address of port status info in local memory

	move(out_queue_stat_base, QUEUE_STATUS_BASE) 

#if IS_IXPTYPE(__IXP24XX)
	move(out_addr_msf_fc_stat, TX_MPHY_STATUS)
#endif	// #if IS_IXPTYPE(__IXP24XX)

#if IS_IXPTYPE(__IXP28XX)
	move(out_addr_msf_fc_stat, TX_MULTIPLE_PORT_STATUS_0) 
#endif	// #if IS_IXPTYPE(__IXP28XX)
  
#ifndef DISABLE_TX2SCHED_FEEDBACK

	// base calculated address used in cap reflect write (calculated address
	// mode) to notify SCHEDULER 

	alu[out_cap_addr_base, --, B, SCHEDULER_ME]
	alu[out_cap_addr_base, out_cap_addr_base, AND, 0x1F]

	// ME cluster number		

	alu_shf[out_cap_addr_base, --, B, out_cap_addr_base, <<10]	
	alu_shf[out_cap_addr_base, out_cap_addr_base, OR, 1, <<15]	; ME
	alu_shf[out_cap_addr_base, out_cap_addr_base, OR, 1, <<9]	; DRAM xfer
#endif	// #ifndef DISABLE_TX2SCHED_FEEDBACK

.end  
	
#endm	// _pkt_tx_16p_gprs_init()


///////////////////////////////////////////////////////////////////////////////
//
// 	_pkt_tx_16p_nn_ring_init()
//
// Description:
//		Initialize next neighbor ring, NN ring will be empty only when there
//      is no valid entry
//
// Outputs: 
//		None
//
// Inputs:  
//		None
//		
// Constants
//		None
//
///////////////////////////////////////////////////////////////////////////////

#macro 	_pkt_tx_16p_nn_ring_init()

.begin

	.reg	ctx_enables_reg

	// NN ring will be empty only where there is no valid entry

	local_csr_rd[ctx_enables]
	immed[ctx_enables_reg, 0]

	alu_shf[ctx_enables_reg, ctx_enables_reg, AND~, 3, <<18]	; [18-19] nn threshold is 0.

	local_csr_wr[ctx_enables, ctx_enables_reg]
	local_csr_wr[nn_get, 0]
	local_csr_wr[nn_put, 0]	

.end	

#endm	// _pkt_tx_16p_nn_ring_init() 


///////////////////////////////////////////////////////////////////////////////
//
// 	_pkt_tx_16p_msf_tx_control_init()
//
// Description:
//		Initialize the transmission related control msf registers
//
// Outputs: 
//		None
//
// Inputs:  
//		None
//		
// Constants
//		None
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_msf_tx_control_init()

.begin 

	.reg	temp_reg
	.reg	address
	.reg	$data
	.sig	sig_msf

#if defined(IX_PLATFORM_2801) && defined(IXP_HARDWARE)
    /* Platform specific. Projects for IXDP2801 hardware just have
     * configured MSF_TX_CALENDAR with baseboard driver.
     */

	//	Set Tx Control 
	alu[address,	--, B, MSF_TX_CNTL]
	move[temp_reg, MPHY_16_POS_TX_CNTL]
	alu[$data,	--, B, temp_reg]
	msf[write, $data, address, 0, 1], ctx_swap[sig_msf]

#else //IX_PLATFORM_2801 && IXP_HARDWARE
	//	Disable Tx before setting any bits

	alu[address,	--, B, MSF_TX_CNTL]
	msf[read, $data, address, 0, 1], ctx_swap[sig_msf]

	alu[$data, $data, AND~, 1, <<10]					; [10] is enable Tx. 
	msf[write, $data, address, 0, 1], ctx_swap[sig_msf]	; Disable Tx

	// Initialize TX_Calendar MSF regs (only IXP28XX)

	_pkt_tx_16p_calendar_init(FIRST_PORT_IN_THIS_ME, TOTAL_PORTS)


	//	Set Tx Control 
	//	[2-3]	: TBUF Size for partition 0. 01 = 128 bytes
	//	[8]		: TSTAT_ENABLE	1 = enable TSTAT
	//	[29]	: TX_EN_S.	1 = Enable SPI4 TX
	//	All other bits are default.

	msf[read, $data, address, 0, 1], ctx_swap[sig_msf]	; first read before setting
	alu[temp_reg, --, B, $data]

	alu[temp_reg, temp_reg, AND~, 1, <<3]				; [2-3] = 01 = 128 byte TBUF
	alu[temp_reg, temp_reg, OR, 1, <<2]					; TBUF size = 128 for Partition 0
	alu[temp_reg, temp_reg, OR, 1, <<8]					; Enable TSAT
	alu[temp_reg, temp_reg, OR, 1, <<29]				; TX_EN_S : Transmit Enbale for SPI4

	alu[$data,	--, B, temp_reg]
	msf[write, $data, address, 0, 1], ctx_swap[sig_msf]	; Write MSF_TX_CONTROL


	//	Enable Tx after setting all required bits

	msf[read, $data, address, 0, 1], ctx_swap[sig_msf]

	alu[$data, $data, OR, 1, <<10]						; [10] is enable Tx.
	msf[write, $data, address, 0, 1], ctx_swap[sig_msf]

#endif //IX_PLATFORM_2801 && IXP_HARDWARE
.end

#endm	// _pkt_tx_16p_msf_tx_control_init() 


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_thread_specific_init()
//
// Description: initialize context relative GPRs for specific thread to save 
//              instruction cycles in Packet TX processing
//
// Outputs: 
//      out_addr_tx_seq_0:	Tx_Sequence_0 MSF register address
//		out_thread_flag:	thread specific flag to identifies some operation
//							only execute in certain threads
//
//      out_next_ctx_sig:	value used to write to SAME_ME_SIGNAL local csr 
//                          to wakeup next thread
//
// Inputs: 
//		in_thread_flag:		thread specific flag to identifies some operation
//							only execute in certain threads
//		in_next_thread:		next thread number of this thread 
//
// Constants:
//      QM_TO_PACKET_TX_SCR_RING:	scratch ring number used to get tx requet
//		TX_SEQUENCE_0:			Tx_SEQUENCE_0 MSF register address
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_thread_specific_init(out_addr_tx_seq, out_thread_flag, \
						out_sig, in_thread_flag, in_next_thread)

.begin

 	.reg next_context

	// tx sequnce address 

	move[out_addr_tx_seq, TX_SEQUENCE_0]
  
  	// thread specific flag
	
	move[out_thread_flag, in_thread_flag]

	// value to write to SAME_ME_SIGNAL csr to wake up next thread
	// will differs for each thread

	move[next_context,in_next_thread]
	alu_shf[out_sig, next_context, or, &sig_next_context, <<3]

.end
	
#endm //end of macro _pkt_tx_16p_thread_specific_init()


///////////////////////////////////////////////////////////////////////////////
//
// 	_pkt_tx_16p_calendar_init()
//
// Description:
//		Initialize TX_Calendar related MSF registers (only IXP28XX)
//
// Outputs: 
//		None
//
// Inputs:  
//		in_first_port:		first port
//		in_total_ports:     total port number
//		
// Constants
//		None
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_calendar_init(in_first_port, in_total_ports)

.begin 

#if IS_IXPTYPE(__IXP28XX)

	.reg	tmp
	.reg	calendar_0_addr
	.reg	$data
	.reg    addr_incr
	.reg 	i
	.reg	calendar_length_addr

	move(i, 0)
	move(tmp, in_first_port)
	alu_shf[addr_incr, --, B, tmp, <<2]
	move(calendar_0_addr, TX_CALENDAR_0)

	// Initialize Tx_calendar_# 

	.while (i < in_total_ports)	

		alu[tmp, i, +, in_first_port]
		alu[$data,	--, B, tmp]
		msf[write, $data, calendar_0_addr, addr_incr, 1], \
												ctx_swap[sig_msf_access_1]
		alu[i, i, +, 1]
		alu[addr_incr, addr_incr, +, 4]

	.endw

	// Initialize Tx_Calendar_Length

	move(calendar_length_addr, TX_CALENDAR_LENGTH)
	alu[$data,	--, B, in_total_ports]
	msf[write, $data, calendar_length_addr, 0, 1], ctx_swap[sig_msf_access_1]

#endif	// #if IS_IXPTYPE(__IXP28XX)

.end

#endm	// _pkt_tx_16p_calendar_init() 


///////////////////////////////////////////////////////////////////////////////
//
// 	_pkt_tx_16p_avail_tbuf_ele_init()
//
// Description:
//		Initialize the next available tbuf element in scartch memory
//
// Outputs: 
//		None
//
// Inputs:  
//		None
//		
// Constants
//		None
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_avail_tbuf_ele_init()

.begin 

	.reg	address
	.reg	$data

	// initialize the next availbale tbuf element in scratch memory location
	// PACKET_TX_AVAIL_TBUF_ELE_SCR_ADDR to 0, later get and increment 
	// available tbuf element by SCRATCH atomic operation   

	alu[$data, --, b, 0]
	immed32(address, PACKET_TX_AVAIL_TBUF_ELE_SCR_ADDR)
	scratch[write, $data, address, 0, 1], ctx_swap[sig_scratch_access_1]

.end 

#endm	// _pkt_tx_16p_avail_tbuf_ele_init() 


/////////////////////////////////////////////////////////////////////////////// 
//
// 	_pkt_tx_16p_wait_ports_ready_for_transmission()
//
// Description:
//		wait for port to be ready for transmission by cheking staus of ports
//
// Outputs: 
//		None
//
// Inputs:  
//		in_portmask: 		ports transmission handled in this ME
//		
// Constants
//		None
//
// NOTE:	This function waits for all 10 ports to get ready (Link up).
//			This means, the user has to connect all 10 ports to a packet generator.
//			To avoid this and enable users to make progres with less than 10 ports 
//			connected this macro is not used for now. In a later release a port_enable
//			mask will be used to identify the number of ports to be used.
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_wait_ports_ready_for_transmission(in_portmask)

.begin

#if IS_IXPTYPE(__IXP24XX)

	.begin 
	.reg	tx_pending
	.reg	tx_status
	.reg	ready_ports
	.reg	$stat

wait_for_port_ready#:

	// waiting for port read
		
	msf[read, $stat, tx_mphy_stat_addr,  0, 1], ctx_swap[sig_msf_access_1]	
	ld_field_w_clr[tx_pending, 0011, $stat, >>16]	; get tx_pending
	ld_field_w_clr[tx_status, 0011, $stat]			; get tx_status
	alu[ready_ports, tx_status, and~, tx_pending]
	alu[ready_ports, ready_ports, and, in_portmask]
	alu[--, ready_ports, -, in_portmask]
	bne[wait_for_port_ready#]
	.end	// tx_pending tx_status ready_ports $stat
	alu[@bfcp, --, b, 0] 

#endif // #if IS_IXPTYPE(__IXP24XX)


#if IS_IXPTYPE(__IXP28XX)

	.begin 
	.reg	$stat
	.reg	stat_mask
	.reg	tmp
	.reg	loop

	immed[loop, 0]

wait_for_port_ready#:
	alu[loop, loop, +, 1]
	alu[--, loop, -, 10]		; loop 10 times for the ports to get ready.
	bgt[ports_not_ready#]

#if THIS_ME==PACKET_TX_FIRST_ME
	alu[stat_mask, --, B, in_portmask]
#endif // #if THIS_ME==PACKET_TX_FIRST_ME

#if THIS_ME==PACKET_TX_SECOND_ME
	alu[tmp, --, B, in_portmask]
	alu[stat_mask, --, B, tmp, <<PORTS_BY_ONE_ME]
#endif // #if THIS_ME==PACKET_TX_SECOND_ME

	alu[tmp, --, B, stat_mask]
	alu_shf[stat_mask, tmp, OR, stat_mask, <<PORTS_BY_ONE_ME]

	msf[read, $stat, tx_mphy_stat_addr,  0, 1], ctx_swap[sig_msf_access_1]	

	// check port status in starving

	alu[--, $stat, AND, stat_mask]

	bne[wait_for_port_ready#]
	alu[@bfcp, --, b, 0]
	br[end#]

ports_not_ready#:

	//	Ideally we'd like to set bfcp (ports blocked by flow control) mask
	//	to indicate which ports are not ready/working/enabled. For now set
	//	it to 0.Even if it's disbaled it'll anyway be flow controlled during 
	//	transmit.

	alu[@bfcp, --, b, 0]

end#:

	.end	// tx_pending tx_status ready_ports $stat

#endif	// #if IS_IXPTYPE(__IXP28XX)


.end

#endm	// _pkt_tx_16p_wait_ports_ready_for_transmission()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_gprs_init()
//
// Description: initialize context relative GPRs to save instruction cycles 
//              in Packet TX processing
//
// Outputs: 
//		out_port_mask:					port mask for ports whose trnsmission
//										are handled by this ME
//		out_vap:						vitual available ports
//		out_busy_port:					busy port
//		out_tcw_valid_base:				tranmit validation base
//		out_fc_check_pattern:			port bit pattern to check fc status
//		out_ep:							empty ports
//		out_blocked_by_read_next_buf:   ports blocked by reading meta data of
//										the next buffer	
//		out_blocked_by_read_next_buf:   ports blocked by reading leftover 
//										payload  to next buffer
//		out_default_sigmask1:			default sig_mask for phase 1	
//		out_default_sigmask2:			default sig_mask for phase 2	
//
// Inputs:  
//		None
//
// Constants:
//		PORTS_CTRLED_BY_THIS_ME:		ports handled by this ME
//		TBUF_ELEMENT_CTRL_VALID:		address bas to tranmit control word  
//      FC_CHECK_PATTERN:				bit pattern to check port PHY status
//		PORTS_CTRLED_BY_THIS_ME:		ports handled by this ME
//
///////////////////////////////////////////////////////////////////////////////


#macro _pkt_tx_16p_absolute_gprs_init(out_port_mask, out_vap, out_busy_port, \
			out_fc_check_pattern, out_ep, out_blocked_by_read_next_buf, \
			out_blocked_by_read_leftover, out_default_sigmask1, \
			out_default_sigmask2)
.begin 

	.reg	tmp
	.reg	sigmask

 	// bits set in this port_mask means the transmisson of mpackets of that
	// port is handled by this ME

 	move(out_port_mask, PORTS_CTRLED_BY_THIS_ME)

 	// absolute GPR virtual availbel port initialized  to 0x0000, no packets 
	// in virtual queue available initially

 	move(out_vap, 0x0000)

 	// absolute GPR busy port initialized to 0x0, no port busy yet

 	move(out_busy_port, 0x0)

#if IS_IXPTYPE(__IXP24XX)
	alu[out_fc_check_pattern, --, B, 0]			; dummy, not used in IXP24XX
#endif	// #if IS_IXPTYPE(__IXP24XX)

#if IS_IXPTYPE(__IXP28XX)
	move(out_fc_check_pattern, FC_CHECK_PATTERN)
#endif	// #if IS_IXPTYPE(__IXP28XX)

  	// absolute GPR empty ports initialized to 0xFFFF, no packets queued yet

 	move(out_ep, 0xFFFF)

	alu[out_ep, out_ep, and, out_port_mask]
	
	// absolute GPR blocked by reading next buffer initialized to 0  

 	move(out_blocked_by_read_next_buf, 0x0)

	// absolute GPR blocked by reading leftover initialized to 0  

 	move(out_blocked_by_read_leftover, 0x0)
	 
	// default phase 1 signal mask at start of each iteration of loop 

	alu[sigmask, --, B, 0]								; initialize to 0
	_pkt_tx_16p_set_signal[sigmask, sig_next_context]	

	// signal for reading tx_request 

	_pkt_tx_16p_set_signal[sigmask, sig_scratch_access_1]	

	// signal for alloacting next availbale tbuf element/elments

	_pkt_tx_16p_set_signal[sigmask, sig_scratch_access_2]	

	// signal for reading flow control msf register

	_pkt_tx_16p_set_signal[sigmask, sig_msf_access_1]	

	// signal for reading  tx_sequence_0 

	_pkt_tx_16p_set_signal[sigmask, sig_msf_access_2]	

	alu[out_default_sigmask1, --, B, sigmask]
	
	// default phase 2 signal mask at start of each iteration of loop 

	alu[sigmask, --, B, 0]			; initialize to 0
	_pkt_tx_16p_set_signal[sigmask, sig_next_context]		
	
	// signal for moving mpkt_1 from dram to tbuf

	_pkt_tx_16p_set_signal[sigmask, sig_dram_access_1]		
	_pkt_tx_16p_set_push_signal[sigmask, sig_dram_access_1]
	
	// signal used for reading meta data 

	_pkt_tx_16p_set_signal[sigmask, sig_sram_access_1]	

	alu[out_default_sigmask2, --, B, sigmask]

.end 

#endm	// _pkt_tx_16p_absolute_gprs_init()


///////////////////////////////////////////////////////////////////////////////
//
// 	_pkt_tx_16p_counters_init()
//
// Description:
//		Counters initialization
//
// Outputs: 
//		None
//
// Inputs:  
//		None
//		
// Constants
//		None
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_counters_init()

.begin

#ifdef COUNTERS

	// initialized to PACKET_TX_COUNTER_BASE	

	move[@cntr_base, PACKET_TX_COUNTER_BASE]	

#ifdef UNIT_TEST

#if THIS_ME==PACKET_TX_FIRST_ME	

	// initialize counters area for 16 ports in sram to 0

	.begin
	.reg 	cntr_addr ii ss
	.reg	$w0 $w1 $w2 $w3
	.xfer_order	$w0 $w1 $w2 $w3

	alu[cntr_addr, --, b, @cntr_base]
	alu[ii, --, b, 0]
	move[ss, 0] 
	alu[$w0, --, b, ss]  
	alu[$w1, --, b, ss]   
	alu[$w2, --, b, ss] 
	alu[$w3, --, b, ss]  

loop_init_counters_in_sram#:	 

	sram[write, $w0, cntr_addr, 0, 4], ctx_swap[sig_counter_update]

	alu[--, ii, -, 16]
	beq[init_counters_done#]

	alu[ii, ii, +, 1]
	alu[cntr_addr, cntr_addr, +, 16]
	br[loop_init_counters_in_sram#]

init_counters_done#:
	
	.end 	

#endif // #if THIS_ME==PACKET_TX_FIRST_ME

#endif // #ifdef UNIT_TEST								

#endif // #ifdef	COUNTERS

.end

#endm	// _pkt_tx_16p_counters_init()


/////////////////////////////////////////////////////////////////////////////// 
//
// 	_pkt_tx_16p_local_memory_init()
//
// Description:
//		local memory initialization
//
// Outputs: 
//		None
//
// Inputs:  
//		None
//		
// Constants
//		None
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_local_memory_init()

.begin

	// initialze local memory, first intialize all 640 entries to 0

	.begin //begin of local variables entry tag lm_index
	.reg	lm_index
	.reg	i


	alu[lm_index, --, b, 0]
	local_csr_wr[active_lm_addr_0, lm_index]	; set local memory index 0 to 0

	// waiting for local_csr_wr to settle ( 3 cycles)

		alu[i, --, B, 0]
		nop
        nop

	.while (i < 640)

		alu[*l$index0++, --, B, 0]
		alu[i, i, +, 1]

	.endw

	.end	// lm_index i

 	// then initialize allowed_mpkts and phy_depth of each port in 
	// queue info to the specific phy_depth of that port  

	.begin

	.reg	port
	.reg	last_port

#if IS_IXPTYPE(__IXP24XX)
	
	.reg	phy_depth
	
	move[phy_depth, DEFAULT_PHY_DEPTH]

#endif 	// #if IS_IXPTYPE(__IXP24XX)

#if IS_IXPTYPE(__IXP28XX)

	.reg	hungry_depth
	.reg	starving_depth

	move[hungry_depth, DEFAULT_PHY_HUNGRY_DEPTH]
	move[starving_depth, DEFAULT_PHY_STARVING_DEPTH]

#endif 	// #if IS_IXPTYPE(__IXP28XX)


	move[port, FIRST_PORT_IN_THIS_ME]
	move[last_port, LAST_PORT_IN_THIS_ME]


	.while (port <= last_port)

	// initialize constants in local memory to resolve insufficient GPR problem

	_pkt_tx_16p_init_constants_in_lm(port, sigmask_phase_1_default, \
					sigmask_phase_2_default)

#if IS_IXPTYPE(__IXP24XX)

	_pkt_tx_16p_init_port_phy_depth(port, phy_depth)

#endif 	// #if IS_IXPTYPE(__IXP24XX)

#if IS_IXPTYPE(__IXP28XX)

	_pkt_tx_16p_init_port_phy_depth(port, hungry_depth, starving_depth)

#endif 	// #if IS_IXPTYPE(__IXP28XX)

		alu[port, port, +, 1]

	.endw

	.end // port max_port phy_depth hungry_depth starving_depth

.end

#endm	// _pkt_tx_16p_local_memory_init()


///////////////////////////////////////////////////////////////////////////////
//
// pkt_tx_16p_init()
//
//	Description: MSF initialization, loacl memory initialization, and global 
//				 variables initialization
//                
//
// Outputs: 
//		None
//
// Inputs:  
//		None
//
// Constants
//		None
//
// Size: 
//		around 420  instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro pkt_tx_16p_init()

	.begin

	.reg	first_port

	// initialize context relative GPRs to save instruction cycles 
    // in Packet TX processing

	_pkt_tx_16p_gprs_init(tbuf_scr_addr, queue_stat_base, tx_mphy_stat_addr, \
												cap_reflect_addr_base)

	alu[first_port, --, B, FIRST_PORT_IN_THIS_ME]

#ifdef ADD_L2_HEADER
#ifdef ETHERNET_TX
//	move(l2_table_base, L2_TABLE_SRAM_BASE)
#endif // #ifdef ETHERNET_TX
#endif 	// #ifdef ADD_L2_HEADER

	br=ctx[0, ctx0_init#]
    br=ctx[1, ctx1_init#]
    br=ctx[2, ctx2_init#]
    br=ctx[3, ctx3_init#]
    br=ctx[4, ctx4_init#]
    br=ctx[5, ctx5_init#]
    br=ctx[6, ctx6_init#]
    br=ctx[7, ctx7_init#]

ctx0_init#:

	// Hardware, global absolute GPRs initializtion and thread 0 specific \
	// context relative variables initialization

	// Initialize NN ring

	_pkt_tx_16p_nn_ring_init()
 
#if THIS_ME==PACKET_TX_FIRST_ME

   // initialize MSF registers for Paket TX

	_pkt_tx_16p_msf_tx_control_init()

	// initialize the next availbale tbuf element in scratch
	
	_pkt_tx_16p_avail_tbuf_ele_init()

#endif // #if THIS_ME==PACKET_TX_FIRST_ME

	// thread 0 specific context relative GPR variables initialization

    _pkt_tx_16p_thread_specific_init(tx_seq_addr, \
			thread_specific_flag, sig_next_context_gpr, \
			THREAD_0_SPECIFIC_FLAGS, 1)

    // initialize absolute GPRs

 	_pkt_tx_16p_absolute_gprs_init(@port_mask, @vap, @busy_port, \
			@fc_check_pattern, @ep, @blocked_by_read_next_buf, \
			@blocked_by_read_leftover, @default_sigmask_1, \
			@default_sigmask_2) 

 	// absolute GPR @rr_vector initialized to 0xFFFF
 	move[@rr_vector, 0xFFFF]
	alu[@rr_vector, @rr_vector, and, @port_mask]

	// wait for port to be ready for transmission

	_pkt_tx_16p_wait_ports_ready_for_transmission(@port_mask)


	alu[@rp, @ep, AND~, @bfcp]
	alu[@vpp, @vap, AND~, @bfcp]

	// debug counters (absolute gprs) initialization

	_pkt_tx_16p_debug_init_counters()

	// sram counters (for core component) initialization

	_pkt_tx_16p_counters_init()


	// initialize local memory 

	_pkt_tx_16p_local_memory_init()	

	// following inter-ME signal common_init_complete_sig_num is generated by 
	// the ME whcih initializes common resources, such as scratch ring, Packet
	// TX ME will wait for this signal to make sure all common resource are 
	// available, when it start Packet TX processing
	ctx_arb[common_init_complete_sig_num]

	.begin //begin of local variable sig
	.reg	sig

    alu[sig, --, B,  &sig_next_context]
    alu_shf[sig, 0x0, OR, sig, <<3]         ; value to signal thread 0
    local_csr_wr[SAME_ME_SIGNAL, sig]   	; signal thread 0


	.end // end of local variable sig
	br[sync_exec#]

	
ctx1_init#:

	// thread 1 specific context relative GPR variables initialization

    _pkt_tx_16p_thread_specific_init(tx_seq_addr, \
			thread_specific_flag, sig_next_context_gpr, \
			THREAD_1_SPECIFIC_FLAGS, 2)

	// initialize local memory index 1, so code can access global constants
	// stored in local memory due to gpr number limitation
	  
	_pkt_tx_16p_set_lm_index_1_to_port_status_info(first_port)
	br[sync_exec#]


	// thread 2 specific context relative GPR variables initialization

ctx2_init#:

    _pkt_tx_16p_thread_specific_init(tx_seq_addr, \
			thread_specific_flag, sig_next_context_gpr, \
			THREAD_2_SPECIFIC_FLAGS, 3)

	// initialize local memory index 1, so code can access global constants
	// stored in local memory due to gpr number limitation
	  
	_pkt_tx_16p_set_lm_index_1_to_port_status_info(first_port)
	br[sync_exec#]


	// thread 3 context specific relative GPR variables initialization

ctx3_init#:

    _pkt_tx_16p_thread_specific_init(tx_seq_addr, \
			thread_specific_flag, sig_next_context_gpr, \
			THREAD_3_SPECIFIC_FLAGS, 4)

	// initialize local memory index 1, so code can access global constants
	// stored in local memory due to gpr number limitation
	  
	_pkt_tx_16p_set_lm_index_1_to_port_status_info(first_port)
	br[sync_exec#]


 	// thread 4 context specific relative GPR variables initialization

ctx4_init#:
 
    _pkt_tx_16p_thread_specific_init(tx_seq_addr, \
			thread_specific_flag, sig_next_context_gpr, \
			THREAD_4_SPECIFIC_FLAGS, 5)

	// initialize local memory index 1, so code can access global constants
	// stored in local memory due to gpr number limitation
	  
	_pkt_tx_16p_set_lm_index_1_to_port_status_info(first_port)
	br[sync_exec#]


	// thread 5 specific context relative GPR variables initialization

ctx5_init#:

    _pkt_tx_16p_thread_specific_init(tx_seq_addr, \
			thread_specific_flag, sig_next_context_gpr, \
			THREAD_5_SPECIFIC_FLAGS, 6)

	// initialize local memory index 1, so code can access global constants
	// stored in local memory due to gpr number limitation
	  
	_pkt_tx_16p_set_lm_index_1_to_port_status_info(first_port)
	br[sync_exec#]


	// thread 6 specific context relative GPR variables initialization

ctx6_init#:

    _pkt_tx_16p_thread_specific_init(tx_seq_addr, \
			thread_specific_flag, sig_next_context_gpr, \
			THREAD_6_SPECIFIC_FLAGS, 7)

	// initialize local memory index 1, so code can access global constants
	// stored in local memory due to gpr number limitation
	  
	_pkt_tx_16p_set_lm_index_1_to_port_status_info(first_port)
	br[sync_exec#]


	// thread 7 specific context relative GPR variables initialization

ctx7_init#:

    _pkt_tx_16p_thread_specific_init(tx_seq_addr, \
			thread_specific_flag, sig_next_context_gpr, \
			THREAD_7_SPECIFIC_FLAGS, 0)

	// initialize local memory index 1, so code can access global constants
	// stored in local memory due to gpr number limitation
	  
	_pkt_tx_16p_set_lm_index_1_to_port_status_info(first_port)

	// following sync_exec# is used to synchronize execution sequence, so ecah
	// thread will execute in order in processing packet transmit 

sync_exec#:

	// wait for my turn

	ctx_arb[sig_next_context]

.end

#endm	// pkt_tx_16p_init()


///////////////////////////////////////////////////////////////////////////////

#endif	// __PKT_TX_P16_INIT_UC__
