/******************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2002 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

 /**
 ******************************************************************************
 * @ingroup Packet Tx
 * File:  Packet_txx.c
 *
 * @description:
 *    This is the SPHY and NPHY4 transmit code main file.  This file 
 *    defines the main function which will be used in the SPHY and MPHY_4
 *	  tramiting microengine C code..
 *****************************************************************************/

#define	INLINE	static __forceinline
#define WAIT_FOR_COMMON_RESOURCE_INITIALIZATION 	


#include <dl_system.h>
#include <hardware.h>
#include <sphy_mphy4_tx.h>
#include <dl_buf.c>
#include <dl_meta.c>
#include <sig_functions.c>
#include <packet_txx_util.c>
#include <packet_txx_init.c>

#if (TX_PHY_MODE == PHY_SPHY_1_32)
	q_port_t qPort[1];								/* Queue entry array     */

#endif

#if ((TX_PHY_MODE == PHY_SPHY_4_8) || (TX_PHY_MODE == PHY_MPHY_4))
	q_port_t qPort[4];								/* Queue entry array     */
#endif

	thread_flag_t threadFlag;						/* Thread execution flag.*/
	SIGNAL sig_next1, sig_next2, sig_next3;
	SIGNAL sig_scratchAccess1; 						/* Scratch ring read sig.*/
	SIGNAL sig_sramRead1; 							/* Sram read sig.        */
	SIGNAL sig_msfAccess1;							/* msf (read/write) sig1.*/
	SIGNAL sig_msfAccess_2; 						/* msf (read/write) sig2.*/
	SIGNAL sig_msfAccess_3;							/* msf (read/write) sig3.*/
	SIGNAL sig_reflectWrite; 						/* reflect write sig.    */
	SIGNAL sig_commonInitComplete;					/* Intial complet sig.   */ 
	SIGNAL_PAIR	sig_dramAccess1;					/* Dram read sig 1.      */ 
	SIGNAL_PAIR sig_dramAccess2;					/* Dram read sig 2.      */
	SIGNAL_MASK sigmask_phs1Default;				/* Signal mask default 1.*/
	SIGNAL_MASK sigmask_phs2Default;				/* Signal mask default 2.*/
	SIGNAL_MASK sigmask_1;							/* Signal mask 1.		 */
	same_me_signal_t ntSig1Gpr;						/* Signal register 1.	 */
	same_me_signal_t ntSig2Gpr;						/* Signal register 2.	 */
	same_me_signal_t ntSig3Gpr;						/* Signal register 3.	 */
	unsigned int scrRingTr; 						/* Scratch ring number.	 */
	unsigned int addrTxSeq;							/* Scratch ring address. */
	unsigned int portEntryTurnaround;				/* Entry turnaround value*/
	__declspec(gp_reg)	int port;  					/* Thread port number.   */
	__declspec(sram_read_reg) input_tx_req_t txReq;	/* Transmit request reg. */
	__declspec(sram_read_reg) int mpktsSent[2]; 	/* mpackets transmitted. */

#ifdef MICROC_INTEGRATION 
#if (TX_PHY_MODE == SPHY_1_32)
	__declspec(remote dram_read_reg) int transmit_count; /*  Remote register.*/
#else
	__declspec(remote dram_read_reg) int transmit_count[16];/*Remote rg array*/
#endif
#else
	__declspec(remote dram_read_reg) int txd_p0;	/* Remote register 0	 */
	__declspec(remote dram_read_reg) int txd_p1;	/* Remote register 1	 */
	__declspec(remote dram_read_reg) int txd_p2; 	/* Remote register 2	 */
	__declspec(remote dram_read_reg) int txd_p3; 	/* Remote register 3	 */
#endif


/********************************************************************************
 * following definition selects sram transfer registers to be used with 
 * dl_meta.uc dispatch loop
 ********************************************************************************/

#define DL_META_DATA_IN_SXFER

#if(TX_PHY_MODE == SPHY_4_8)
#define AVOID_USING_SKIP_BIT_IN_TCW0
#endif 


INLINE PacketTx_ImplicitReadAllSigs(void)
{
	__implicit_read(&sig_next1);
	__implicit_read(&sig_next2);
	__implicit_read(&sig_next3);
	__implicit_read(&sig_scratchAccess1);
	__implicit_read(&sig_sramRead1);
	__implicit_read(&sig_msfAccess1);
	__implicit_read(&sig_msfAccess2);
	__implicit_read(&sig_msfAccess3);
	__implicit_read(&sig_reflectWrite);
	__implicit_read(&sig_dramAccess1);
	__implicit_read(&sig_dramAccess2);
}


INLINE PacketTx_AssignSigReg(void)
{
	__assign_relative_register(&sig_commonInitComplete,
							   ME_INIT_SIGNAL_C);
}

/**
 *******************************************************************************
 * @ingroup Packet Tx
 * 	PacketTx_SopEopMpkt()
 *
 * @description:
 *    Handle the packet which can be fitted into one mpacket 
 *	  dequeue one packet
 *******************************************************************************/
INLINE void											 /* OUT: No return value   */
PacketTxx_SopEopMpkt(q_entry_t *pEntry)			 	 /* IN/OUT: q entry pointer*/
{
	unsigned int tbufEle;							 /*	TBUF element  number   */
	unsigned int addrOfTbuf;						 /* TBUF Elmenet  address. */
	unsigned int addrTxCntrl;						 /* TBUF control address.  */
	unsigned int txCntlWord;						 /* Transmit control word. */
	unsigned int bufDesc;							 /* EOB buffer descripter  */
	unsigned int payloadLen;						 /* Payload length.		   */
	unsigned int offset;						 	 /* Payload offset.		   */
	SIGNAL_MASK sigmask_2;							 /* Signal mask 2.		   */
	__declspec(sram_write_reg ) int cntlWord[2]; 	 /* TBUF control data array*/	

	/*
	 * Phase 1 continue
	 */
	if(pEntry->globaleRdRingFlag == 0)
	{
		pEntry->globaleRdRingFlag =1;
		pEntry->myRdRingFlag = 1;
		sigmask_1 =  __signals(&sig_scratchAccess1, &sig_next2);

		scratch_get_ring(&txReq,
						(volatile __declspec(scratch) void *)scrRingTr,
		              	1,
						sig_done,
						&sig_scratchAccess1);   /* Read scratch ring*/
	}
	else
	{
		sigmask_1 =  __signals(&sig_next2);
	}
	cntlWord[1] = port;
	sigmask_2 = __signal(&sig_next2);
	wait_for_all(sigmask_1);
	
	/*
	 * Phase 2:
	 */
	signal_next_ctx(ntSig2Gpr);
	if(pEntry->myRdRingFlag == 1)
	{
		if(txReq.value != 0x0)
		{
			sigmask_2 = PacketTx_ReadMetaDataFromSram(sigmask_2,
												  	  txReq.value,
												  	  &sig_sramRead1);
			PacketTx_MoveSopPayloToTbuf(bufDesc, 
								addrOfTbuf, 
								offset,
								payloadLen,
								(offset & CONST_07),
								&sig_dramAccess1);

			
			



	txCntlWord = TCW0_SOP_EOP_BASE_W0;
	bufDesc = pEntry->currBufDesc;
	payloadLen = pEntry->payloadLen;
	offset = pEntry->payloadOffset;

	/*
	 * Queue header advanced.
	 * Packet in queue decrement.
	 * Packet transmit Increment.
	 */
	PacketTx_AdvanceQueueHead(pStatus, 
 							  portEntryTurnaround);

#ifndef DISABLE_TX2SCHED_FEEDBACK

	PacketTx_NotifyScheduler(pStatus, port);

#endif

	cntlWord[1] = port;
	sigmask_2 = sigmask_phs2Default;
	wait_for_all(sigmask_1);
	PacketTx_ImplicitReadAllSigs();

 	/*
	 *End of of phase 1
	 *Start Phase 2
	 */
	signal_next_ctx(ntSig2Gpr);

	/*
	 * Get next available tbuf element (tbufEle1) and corresponding address 
	 */
	PacketTx_GetTbufEleAndAddr(pStatus,
							   &tbufEle,
							   &addrOfTbuf);

	/*
	 * check tbuf full to avoid overwritting tbuf.
	 */
	if (PacketTx_CheckTbufFullSopEop(&txCntlWord, 
									 payloadLen,
									 (offset & CONST_07),
									 tbufEle) != 0)
	{
		/*  
		 * Not ready to transmit
		 */
		PacketTx_WaitTillTbufNotFull(tbufEle);
	}
	
	/*
	 * The TBUF is ready, start to transmit
	 */
	PacketTx_MoveSopPayloToTbuf(bufDesc, 
								addrOfTbuf, 
								offset,
								payloadLen,
								(offset & CONST_07),
								&sig_dramAccess1);

#ifdef ADD_PPP_HEADER

    cntlWord[0] = (PPP_IP_PROTOCOL << IP_PROTOCOL_SHFT);
	msf_write(&cntlWord[0], 
	          (void *)addrOfTbufEle,
			  1,
			  sig_done,
			  &sig_msfAccess1);

#endif

 	/*
	 * Now, check whether a valid tx request gotten in the scratch read from the
	 * scratch ring in phase 1. If there is a valid tx request, enqueu that 
	 * packet to the queue of that port.
	 */
	if ((exeStatFlag.getTxRequestFlag == 1) && (txReq.value != 0x0))
	{
		sigmask_2 = PacketTx_ReadMetaDataFromSram(sigmask_2,
												  txReq.value,
												  &sig_sramRead1);
		/*
	 	 * Set sop meta read flag in exe_stst_flag.
	 	 */
		exeStatFlag.readSopDescFlag = 1;
		pEntry = PacketTx_UpdateQueueTail(pStatus,
										  portEntryTurnaround);
	}

	wait_for_all(sigmask_2);
	PacketTx_ImplicitReadAllSigs();

#ifdef IXP2800

	addrTxCntrl = pStatus->ctwVdBase + (tbufEle << ELE_2_TX_CNTRL_ADDR);

#else

	addrTxCntrl = pStatus->ctwVdBase + (tbufEle << 3); 

#endif

 	/*
	 * end of of phase 2 in sop_eop mpacket path
	 * Start sop_eop_mpkt_phase 3
	 */
	signal_next_ctx(ntSig3Gpr);

   	/*
	 * write transmit contrl word (TCW) to TBUF_ELEMENT_CTRL_V_# corresponding
	 * the tbuf_element to initiate transmission
	 */
	cntlWord[0] = txCntlWord;
	msf_write(&cntlWord[0],
			  (void *)addrTxCntrl,
			  2,
			  sig_done,
			  &sig_msfAccess1);

#ifdef COUNTERS

	PacketTx_UpdateBytesTransmitted(txCntlWord,
									sig_counterUpdate,
									byteCnt);
#endif

	PacketTx_FreeBuffer(bufDesc);

	/*
	 * if didn't read meta data in phase 2, just branch out
	 */
	if( exeStatFlag.readSopDescFlag == 1)
	{
		PacketTx_SaveSopMeta(pEntry,
							 pStatus,
							 txReq.sopMetaHndl); 
	}

#ifndef COUNTERS

	wait_for_all(&sig_msfAccess1,
	             &sig_next1);

#else 

	wait_for_all(&sig_msfAccess1, 
				 &sig_CounterUpdate,
				 &sig_next1);

#endif

} /* PacketTx_SopEopMpkt()*/

/*****************************************************************************
 * Abstract:
 *    Packet (POS/Ethernet) transmission main loop, never return
 *
 * Side Effects:
 *    None
 *
 * Assumptions:
 * 	  None
 *****************************************************************************/											
INLINE void											/* OUT: No return value  */
PacketTxx(void)										/* No parameter.		 */
{
	unsigned int tmp;								/* Temp. register.       */
	q_entry_t *pEntry;								/* Queue entry pointer.  */ 

	PacketTxx_AssignSigReg();				/* Wait global initialization.   */
	PacketTxx_Init();	 					/* Packet Txx initialization.    */

	/*
	 * Start processing loop
	 */
	while(1)
	{

		/*
		 * Phase 1:
	 	 */
		wait_for_all(&sig_next1);					/* Wait for my turn*/
 		signal_next_ctx(ntSig1Gpr); 			  	/*send signal to next thread*/
		threadFlag.value = 0; 					  	/*Reset threadFlag.*/
		pEntry = &(qPort[port].q_entry);			/* get the entry */

		if(pEntry->mpktFlag == 0)
		{
			PacketTxx_SopMpkt(pEntry);
		}
		else
		{
			/*
			 *	Phase 2:
			 */	
			wait_for_all(&sig_next2);
			signal_next_ctx(ntSig2Gpr);
			
			/*
			 *	Phase 3:
			 */	
			wait_for_all(&sig_next3);
			signal_next_ctx(ntSig3Gpr);
		}
	}
} /* packetTxx()*/


/*****************************************************************************
 * Abstract:
 *    Packet (POS/Ethernet) transmission main loop
 *
 * Side Effects:
 *    None
 *
 * Assumptions:
 * 	  None
 *****************************************************************************/	     
main( void)
{
	PacketTxx_Init();						/* Initialization fucntion.		 */
	PacketTxx();							/* Transmite function.			 */
}
