/******************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2002 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

 /**
 ******************************************************************************
 * @ingroup Packet Tx
 * File:  PacketTx.c
 *
 * @description:
 *    This is the SPHY and NPHY4 transmit code main file.  This file 
 *    defines the main function which will be used in the SPHY and MPHY_4
 *	  tramiting microengine C code..
 *****************************************************************************/

#define	INLINE	static __forceinline
#define WAIT_FOR_COMMON_RESOURCE_INITIALIZATION 	


#include <dl_system.h>
#include <hardware.h>
#include <sphy_mphy4_tx.h>
#include <dl_buf.c>
#include <dl_meta.c>
#include <sig_functions.c>
#include <sphy_mphy4_tx_util.c>
#include <sphy_mphy4_tx_init.c>

#if (TX_PHY_MODE == SPHY_1_32)
	__declspec(local_mem shared) q_port_t qPort[1];		/* Queue entry array     */
	__declspec(local_mem shared) q_status_t qStatus[1];	/* Queue status array    */
#endif

#if ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4))
	__declspec(local_mem shared) q_port_t qPort[4];		/* Queue entry array     */
	__declspec(local_mem shared) q_status_t qStatus[4];	/* Queue status array    */
#endif

	exe_stat_flag_t exeStatFlag;					/* Thread execution flag.*/
	SIGNAL sig_next1, sig_next2, sig_next3;
	SIGNAL sig_scratchAccess1; 						/* Scratch ring read sig.*/
	SIGNAL sig_sramRead1; 							/* Sram read sig.        */
	SIGNAL sig_msfAccess1;							/* msf (read/write) sig1.*/
	SIGNAL sig_msfAccess_2; 						/* msf (read/write) sig2.*/
	SIGNAL sig_msfAccess_3;							/* msf (read/write) sig3.*/
	SIGNAL sig_reflectWrite; 						/* reflect write sig.    */
	SIGNAL sig_commonInitComplete;					/* Intial complet sig.   */ 
	SIGNAL_PAIR	sig_dramAccess1;					/* Dram read sig 1.      */ 
	SIGNAL_PAIR sig_dramAccess2;					/* Dram read sig 2.      */
	SIGNAL_MASK sigmask_phs1Default;				/* Signal mask default 1.*/
	SIGNAL_MASK sigmask_phs2Default;				/* Signal mask default 2.*/
	SIGNAL_MASK sigmask_1;							/* Signal mask 1.		 */
	same_me_signal_t ntSig1Gpr;						/* Signal register 1.	 */
	same_me_signal_t ntSig2Gpr;						/* Signal register 2.	 */
	same_me_signal_t ntSig3Gpr;						/* Signal register 3.	 */
	unsigned int scrRingTr; 						/* Scratch ring number.	 */
	unsigned int addrTxSeq;							/* Scratch ring address. */
	unsigned int portEntryTurnaround;				/* Entry turnaround value*/
	__declspec(gp_reg)	int port;  					/* Thread port number.   */
	__declspec(sram_read_reg) input_tx_req_t txReq;	/* Transmit request reg. */
	__declspec(sram_read_reg) int mpktsSent[2]; 	/* mpackets transmitted. */

#ifdef MICROC_INTEGRATION 
#if (TX_PHY_MODE == SPHY_1_32)
	__declspec(remote dram_read_reg) int transmit_count; /*  Remote register.*/
#else
	__declspec(remote dram_read_reg) int transmit_count[16];/*Remote rg array*/
#endif
#else
	__declspec(remote dram_read_reg) int txd_p0;	/* Remote register 0	 */
	__declspec(remote dram_read_reg) int txd_p1;	/* Remote register 1	 */
	__declspec(remote dram_read_reg) int txd_p2; 	/* Remote register 2	 */
	__declspec(remote dram_read_reg) int txd_p3; 	/* Remote register 3	 */
#endif


/********************************************************************************
 * following definition selects sram transfer registers to be used with 
 * dl_meta.uc dispatch loop
 ********************************************************************************/

#define DL_META_DATA_IN_SXFER

#if(TX_PHY_MODE == SPHY_4_8)
#define AVOID_USING_SKIP_BIT_IN_TCW0
#endif 


INLINE PacketTx_ImplicitReadAllSigs(void)
{
	__implicit_read(&sig_next1);
	__implicit_read(&sig_next2);
	__implicit_read(&sig_next3);
	__implicit_read(&sig_scratchAccess1);
	__implicit_read(&sig_sramRead1);
	__implicit_read(&sig_msfAccess1);
	__implicit_read(&sig_msfAccess2);
	__implicit_read(&sig_msfAccess3);
	__implicit_read(&sig_reflectWrite);
	__implicit_read(&sig_dramAccess1);
	__implicit_read(&sig_dramAccess2);
}


INLINE PacketTx_AssignSigReg(void)
{
	__assign_relative_register(&sig_commonInitComplete,
							   ME_INIT_SIGNAL_C);
}

/**
 *******************************************************************************
 * @ingroup Packet Tx
 * 	PacketTx_SopEopMpkt()
 *
 * @description:
 *    Handle the packet which can be fitted into one mpacket 
 *	  dequeue one packet
 *******************************************************************************/
INLINE void											 /* OUT: No return value   */
PacketTx_SopEopMpkt(
					q_entry_t *pEntry,  			 /* IN/OUT: Entry pointer  */
					q_status_t *pStatus)			 /* IN/OUT: Status pointer */
{
	unsigned int tbufEle;							 /*	TBUF element  number   */
	unsigned int addrOfTbuf;						 /* TBUF Elmenet  address. */
	unsigned int addrTxCntrl;						 /* TBUF control address.  */
	unsigned int txCntlWord;						 /* Transmit control word. */
	unsigned int bufDesc;							 /* EOB buffer descripter  */
	unsigned int payloadLen;						 /* Payload length.		   */
	unsigned int offset;						 	 /* Payload offset.		   */
	SIGNAL_MASK sigmask_2;							 /* Signal mask 2.		   */
	__declspec(sram_write_reg ) int cntlWord[2]; 	 /* TBUF control data array*/	


	bufDesc = pEntry->currBufDesc;
	payloadLen = pEntry->payloadLen;
	offset = pEntry->payloadOffset;

	/*
	 * Set up control word 
	 */
	txCntlWord = TCW0_SOP_EOP_BASE_W0
				 | ( payloadLen << TCW0_PAYLOAD_LEN_LSB_LOC)
				 | ( (offset & CONST_07) << TCW0_PAYLOAD_OFFSET_LSB_LOC);

	/*
	 * Queue header advanced.
	 * Packet in queue decrement.
	 * Packet transmit Increment.
	 */
	PacketTx_AdvanceQueueHead(pStatus, 
 							  portEntryTurnaround);

#ifndef DISABLE_TX2SCHED_FEEDBACK

	PacketTx_NotifyScheduler(pStatus, port);

#endif

	cntlWord[1] = port;
	sigmask_2 = sigmask_phs2Default;
	wait_for_all(sigmask_1);
	PacketTx_ImplicitReadAllSigs();

 	/*
	 *End of of phase 1
	 *Start Phase 2
	 */
	signal_next_ctx(ntSig2Gpr);

	/*
	 * Get next available tbuf element (tbufEle1) and corresponding address 
	 */
	PacketTx_GetTbufEleAndAddr(pStatus,
							   &tbufEle,
							   &addrOfTbuf);

	/*
	 * check tbuf full to avoid overwritting tbuf.
	 */
	if (PacketTx_CheckTbufFullSopEop(tbufEle) != 0)
	{
		/*  
		 * Not ready to transmit
		 */
		PacketTx_WaitTillTbufNotFull(tbufEle);
	}
	
	/*
	 * The TBUF is ready, start to transmit
	 */
	PacketTx_MoveSopPayloToTbuf(bufDesc, 
								addrOfTbuf, 
								offset,
								payloadLen,
								(offset & CONST_07),
								&sig_dramAccess1);

#ifdef ADD_PPP_HEADER

    cntlWord[0] = (PPP_IP_PROTOCOL << IP_PROTOCOL_SHFT);
	msf_write(&cntlWord[0], 
	          (void *)addrOfTbufEle,
			  1,
			  sig_done,
			  &sig_msfAccess1);

#endif

 	/*
	 * Now, check whether a valid tx request gotten in the scratch read from the
	 * scratch ring in phase 1. If there is a valid tx request, enqueu that 
	 * packet to the queue of that port.
	 */
	if ((exeStatFlag.getTxRequestFlag == 1) && (txReq.value != 0x0))
	{
		sigmask_2 = PacketTx_ReadMetaDataFromSram(sigmask_2,
												  txReq.value,
												  &sig_sramRead1);
		/*
	 	 * Set sop meta read flag in exe_stst_flag.
	 	 */
		exeStatFlag.readSopDescFlag = 1;
		pEntry = PacketTx_UpdateQueueTail(pStatus,
										  portEntryTurnaround);
	}

	wait_for_all(sigmask_2);
	PacketTx_ImplicitReadAllSigs();

#ifdef IXP2800

	addrTxCntrl = pStatus->ctwVdBase + (tbufEle << ELE_2_TX_CNTRL_ADDR);

#else

	addrTxCntrl = pStatus->ctwVdBase + (tbufEle << 3); 

#endif

 	/*
	 * end of of phase 2 in sop_eop mpacket path
	 * Start sop_eop_mpkt_phase 3
	 */
	signal_next_ctx(ntSig3Gpr);

   	/*
	 * write transmit contrl word (TCW) to TBUF_ELEMENT_CTRL_V_# corresponding
	 * the tbuf_element to initiate transmission
	 */
	cntlWord[0] = txCntlWord;
	msf_write(&cntlWord[0],
			  (void *)addrTxCntrl,
			  2,
			  sig_done,
			  &sig_msfAccess1);

#ifdef COUNTERS

	PacketTx_UpdateBytesTransmitted(txCntlWord,
									sig_counterUpdate,
									byteCnt);
#endif

	
	/*
	 * if didn't read meta data in phase 2, just branch out
	 */
	if( exeStatFlag.readSopDescFlag == 1)
	{
		PacketTx_FreeBuffer(bufDesc);

		PacketTx_SaveSopMeta(pEntry,
							 pStatus,
							 txReq.sopMetaHndl); 


#ifndef COUNTERS

     	wait_for_all(&sig_msfAccess1,
	             &sig_next1);

#else 

	    wait_for_all(&sig_msfAccess1, 
				 &sig_CounterUpdate,
				 &sig_next1);

#endif
	}
	else
	{
		PacketTx_FreeBuffer(bufDesc);

#ifndef COUNTERS

     	wait_for_all(&sig_msfAccess1,
	             &sig_next1);

#else 

	    wait_for_all(&sig_msfAccess1, 
				 &sig_CounterUpdate,
				 &sig_next1);

#endif
	}


} /* PacketTx_SopEopMpkt()*/


/*****************************************************************************
 * Abstract:
 *handle no port is ready to transmit case, check and handle 
 *				valid tx request
 *
 * Side Effects:
 *    None
 *
 * Assumptions:
 * 	  None
 *****************************************************************************/
void												/* OUT: No return value  */  
PacketTx_GetTxRequest(
						q_entry_t *pEntry,			/* IN/OUT: Entry pointer */
						q_status_t *pStatus)		/* IN/OUT: Status pointer*/
{
	SIGNAL_MASK sigmask_2;							/* Signal maks 2 		 */

	/*
	 * Not ready for tx, check whether valid tx request received
	 * swap out to wait scratch read Tx request to complete and my turn
	 * clear the signal bit in sig_mask_phase2
	 */

#ifndef DISABLE_TX2SCHED_FEEDBACK

	sigmask_1 = PacketTx_ClearSignal(sigmask_1,
									 &sig_reflectWrite);
#endif

	sigmask_1 = PacketTx_ClearSignal(sigmask_1,
									 &sig_msfAccess3);
	sigmask_2 = PacketTx_SetSignal(0,
								   &sig_next3);

	wait_for_all(sigmask_1);
	PacketTx_ImplicitReadAllSigs();

	/*
	 * end of phase 1 of no port ready to transmit
	 */
	signal_next_ctx(ntSig2Gpr);

	/*
	 * Now, check whether a valid tx request gotten in the scratch read from the
	 * scratch ring in phase 1. If there is a valid tx request, enqueu that 
	 * packet to the queue of that port 
	 */
	if((exeStatFlag.getTxRequestFlag == 1) && (txReq.value != 0))
	{
		sigmask_2 = PacketTx_ReadMetaDataFromSram(sigmask_2,
												  txReq.value,
												  &sig_sramRead1);
		exeStatFlag.readSopDescFlag = 1;
		pEntry = PacketTx_UpdateQueueTail(pStatus,
										  portEntryTurnaround);
	}

	wait_for_all(sigmask_2);
	PacketTx_ImplicitReadAllSigs();

	/*
	 * end of phase 2 of no port ready to transmit
	 * beginning of phase 3 of no port ready to transmit
	 * wake up next thread in beginning of phase 3 to reduce wakeup latency
	 */
	signal_next_ctx(ntSig3Gpr);

	if(exeStatFlag.readSopDescFlag == 1)
	{
		PacketTx_SaveSopMeta(pEntry,
							 pStatus,
							 txReq.sopMetaHndl); 
	}

	wait_for_all(&sig_next1);

} /* PacketTx_GetTxRequest()*/


/*****************************************************************************
 * Abstract:
 * Handling SOP only but not Critical Mpacket
 *
 * Side Effects:
 *    None
 *
 * Assumptions:
 * 	  None
 *****************************************************************************/
INLINE void											/* OUT: No return value  */
PacketTx_SopOnlyNonCriticalMpkt(
								q_entry_t *pEntry,  /* IN/OUT: Entry pointer */
								q_status_t *pStatus)/* IN/OUT: Status pointer*/
{
	
	unsigned int payloadLen;						/* SOP paylod length.	 */
	unsigned int tmp;								/* Temp used reg.    	 */
	unsigned int tbufEle;							/* TBUF element 1 number.*/
	unsigned int addrOfTbuf;						/* TBUF element 1 address*/
	unsigned int txCntlWord;						/* TBUF control word num.*/
	unsigned int addrOfTxControl;					/* TBUF control word addr*/
	unsigned int offset;							/* Payload offset reg.   */ 
	SIGNAL_MASK sigmask_2;							/* Signal mask 2.        */
	__declspec(sram_write_reg) int cntlWord[2];		/* TBUF control array.	 */


	offset = pEntry->payloadOffset;

#ifndef DISABLE_TX2SCHED_FEEDBACK
	/*
	 * since no complete packet is sent with this thread, clear the signal bit \
	 * related reflect write in sigmask_1
	 */
	sigmask_1 = PacketTx_ClearSignal(sigmask_1, &sig_reflectWrite);
#endif

	/*
	 * reset p_sop_flag in queue entry
	 */
	pEntry->sopFlag = 0;

	/*
	 * check whether to read secondary BD
	 */
	if(pEntry->rdNxtMetaFlag == 1)
	{
		/*
		 * read secondary buffer meta data from SRAM 
		 */
		sigmask_1 = PacketTx_ReadMetaDataFromSram(sigmask_1,
												  pEntry->value[2],
												  &sig_sramRead1);
		/* 
		 *reset read secobary buffer meta data bit 
		 */
		pEntry->rdNxtMetaFlag = 0;
		/*
		 * set read secondary BD flag in exeStatFlag for use in phase 2
		 */
		exeStatFlag.rdSecndBufDescFlag = 1;

		tmp = TBUF_ELE_SIZE << ME_THREAD_NUM_SHFT;
		if(pEntry->payloadLen <= tmp) 				/*compare with how much left*/
		{
			pStatus->noLockBufReadFlag = 0;
		}
	}

	/*
	 * now consider the restriction of the limitation that the sum of the prepend
	 * length and payload length must be multiple of device bus width
	 */
#ifndef ADD_PPP_HEADER
	payloadLen = (TBUF_ELE_SIZE - (offset & CONST_07)) &
				DEVICE_BUS_WIDTH_RESTRICTION_MASK;
#else
	payloadLen = ((TBUF_ELE_SIZE_MINUS_8 - (offset & CONST_07)
				 + PPP_HEADER_SIZE) &
	             DEVICE_BUS_WIDTH_RESTRICTION_MASK) - PPP_HEADER_SIZE;
#endif

	/*
	 * update ab_paylo_rmnd and ab_offset_rpaylo in queue entry with 
	 * sop_only_non_critical_paylo [3]
	 */
	PacketTx_UpdateAbPayloRmndAndOffsetRpaylo(pEntry,
											  payloadLen);

 	/*
	 * swap out to wait signals specified in sigmask_1
	 */

	sigmask_2 = sigmask_phs2Default;
	cntlWord[1] = pStatus->portId;
	wait_for_all(sigmask_1);
	PacketTx_ImplicitReadAllSigs();

	/*
	 * end of of phase 1 with sop_only non critical mpkt ready for tx
	 *beginning of phase 2 with sop_only mpkt ready for tx	
	 *wake up next thread in beginning of phase 2 to reduce wakeup latency
	 */
	signal_next_ctx(ntSig2Gpr);

	/*
	 * get next available tbuf element (tbufEle1) and corresponding address 
	 * (addrOfTbufEle) for it [5]
	 */
	PacketTx_GetTbufEleAndAddr(pStatus,
							   &tbufEle,
							   &addrOfTbuf);

	/* 
	 *Check tbuf full to avoid overwritting tbuf.
	 */
	if(PacketTx_CheckTbufFullNotSopEop(tbufEle) != 0)
	{
		PacketTx_WaitTillTbufNotFull(tbufEle);
	}


	/* 
	 *Dram access takes longer latency, so do it first
	 *move payload to tbuf.
	 */
	PacketTx_MoveSopPayloToTbuf(pEntry->currBufDesc, 
								addrOfTbuf,
								offset,
								payloadLen,
								(offset & CONST_07),
								&sig_dramAccess1);

#ifndef ADD_PPP_HEADER

   	/*
	 *setup tcw01 base for sop_only case.
	 */
	txCntlWord = (1 << TCW0_SOP_BIT_LOC);
#else

    /*
	 * write  PPP header for SOP mpacket
	 */
	cntlWord[0] = (PPP_IP_PROTOCOL << IP_PROTOCOL_SHFT);
	msf_write(&cntlWord[0],
	          (void *)addrOfTbufEle,
			  1,
			  sig_done,
			  &sig_msfAccess1);
	
   	/*
	 * setup tcw01 base for sop_only case.
	 */
	tcw01 = (1 << TCW0_SOP_BIT_LOC);
	tcw01 |= (TCW0_SOP_EOP_BASE_W1 << TCWO_W1_SHF);

#endif

	/* 
	 *set payload_length, payload_offset, and channel fields in tcw0.
	 */
	txCntlWord = PacketTx_SetTcw0WithPayloLenOffsetChnl(txCntlWord,
												   		payloadLen,
												   		(offset & CONST_07),
										 		   		pStatus->portId);

	/*
	 *check whether we need to save secondary buffer meta data 
	 *if didn't read meta data in phase 1, just branch out.
	 */
	if(exeStatFlag.rdSecndBufDescFlag == 1)
	{
	 	/* 
		 *Meta data for secondary buffer is ready, move them to local memory.
		 */
		PacketTx_SaveSbdMeta(pEntry,
		                     pStatus);
		pStatus->noLockBufReadFlag = 1;
	}


	/*
	 *now, check whether a valid tx request gotten in the scratch read from the
	 *scratch ring in phase 1. If there is a valid tx request, enqueu that 
	 *packet to the queue of that port.
	 */
	if((exeStatFlag.getTxRequestFlag == 1) && (txReq.value != 0x0))
	{
		/*
		 *Read SOP meta data, read earlier to reduce latency 
		 *read meta data from SRAM by using the BD pointer in tx request
		 */
		sigmask_2 = PacketTx_ReadMetaDataFromSram(sigmask_2,
												  txReq.value,
												  &sig_sramRead1);
		exeStatFlag.readSopDescFlag =1;
		pEntry = PacketTx_UpdateQueueTail(pStatus,
										  portEntryTurnaround);
	}

	/*
	 *End of phase 2.
	 */
	wait_for_all(sigmask_2);
	PacketTx_ImplicitReadAllSigs();

#ifdef IXP2800

	addrOfTxControl = (tbufEle << ELE_2_TX_CNTRL_ADDR) + pStatus->ctwVdBase;

#else

	addrOfTxControl = (tbufEle << 3) + pStatus->ctwVdBase;

#endif

	/*
	 * Phase 3.
	 */
  	signal_next_ctx(ntSig3Gpr);
	cntlWord[0] = txCntlWord;
	msf_write(&cntlWord[0],
			  (void *)addrOfTxControl,
			  2,
			  sig_done,
			  &sig_msfAccess1);

#ifdef COUNTERS

	PacketTx_UpdateBytesTransmitted(txCntlWord,
									sig_CounterUpdate,
									byteCnt)					
#endif

	/*
	 * If didn't read meta data in phase 2, just branch out.
	 */
	if(exeStatFlag.readSopDescFlag == 1)
	{
		PacketTx_SaveSopMeta(pEntry, 
							 pStatus,
							 txReq.sopMetaHndl); 
	}

#ifndef COUNTERS
	wait_for_all(&sig_msfAccess1, 
				 &sig_next1);
#else
	wait_for_all(&sig_msfAccess1, 
				 &sig_counterUpdate,
				 &sig_next1);
#endif
} /* PacketTx_SopOnlyNonCriticalMpkt()*/


/*****************************************************************************
 * Abstract:
 *	For Ethernet, 129 bytes long packet is the critical case, if  
 *				the packet can't be not fitted into one mpacket, but less than 
 *				240 bytes, it can sure be transmitted in two mpackets (since
 *				offset_mod_8 always less than 8), first mpacket will transmit
 *				fixed 120 bytes, the second mpacket will transmit the rest bytes
 *
 * Side Effects:
 *    None
 *
 * Assumptions:
 * 	  None
 *****************************************************************************/
INLINE void												/* OUT: No return value  */
PacketTx_EthernetCriticalPkt(
								q_entry_t *pEntry,  	/* IN/OUT: Entry pointer */
								q_status_t *pStatus) 	/* IN/OUT: Status pointer*/
{
	unsigned int bufDesc;								/* SOP Buffer descriptor.*/
	unsigned int payloadLen;							/* SOP payload length.   */ 
	unsigned int tbufEle1;								/* TBUF element 1 number.*/
	unsigned int addrOfTbuf1;							/* TBUF element 1 address*/
	unsigned int addrOfTxControl1;						/* TBUF element 1 control*/
	unsigned int tbufEle2;								/* TBUF element 2 number.*/
	unsigned int addrOfTbuf2;							/* TBUF element 2 address*/
	unsigned int addrOfTxControl2;						/* TBUF element 2 control*/
	unsigned int tcw01;									/* TBUF control word 1.  */
	unsigned int tcw02;									/* TBUF control word 2.  */
	unsigned int offset;								/* Payload offset.		 */ 
	unsigned int offsetMod8;							/* Offset mode 8.		 */
	unsigned int tmp;									/* Tmp used register.	 */
	SIGNAL_MASK sigmask_2;								/* Signal mask 2.		 */
	__declspec(sram_write_reg ) int cntlWord01[2];		/* TBUF 1 control array. */
	__declspec(sram_write_reg ) int cntlWord23[2];		/* TBUF 2 control array. */
	in_parameter_1_t toTbufPara;						/* Input parameter.		 */

	bufDesc = pEntry->currBufDesc;
	offset = pEntry->payloadOffset;
	offsetMod8 = (pEntry->payloadOffset & CONST_07);
	
	/*
	 * Advanced queue head, so next packet in the queue of this port will be 
	 * processed when this port is selected again
	 */
	PacketTx_AdvanceQueueHead(pStatus,
							  portEntryTurnaround);

#ifndef DISABLE_TX2SCHED_FEEDBACK

	/*
	 * notify scheduler that one packet has been transmitted from this port
	 */
	PacketTx_NotifyScheduler(pStatus,
							 pStatus->portId);
#endif	

 	/*
	 * swap out to wait signals specified in sigmask_1
	 */
	wait_for_all(sigmask_1);
	PacketTx_ImplicitReadAllSigs();
	sigmask_2 = sigmask_phs2Default;
	cntlWord01[1] = pStatus->portId;				/* Keep the channel number */

	/*
	 * Phase 1 finished.
	 * Phase 2  started. 
	 * Wake up next thread in beginning of phase 2 to reduce wakeup latency.
	 */
 	signal_next_ctx(ntSig2Gpr);

	/*
	 * Get next available tbuf element (tbufEle1) and corresponding address 
	 * (addrOfTbuf1) for it.
	 */
	PacketTx_GetTbufEleAndAddr(pStatus, &tbufEle1, &addrOfTbuf1);
	PacketTx_GetTbufEleAndAddr(pStatus, &tbufEle2, &addrOfTbuf2);

	if(PacketTx_CheckTbufFullNotSopEop(tbufEle2) !=0)
	{
		PacketTx_WaitTillTbufNotFull(tbufEle2);
	}

	/*
	 * Ready to transmit.
	 */
#ifndef ADD_PPP_HEADER

	payloadLen = TBUF_ELE_SIZE_MINUS_8;

#else

	payloadLen = TBUF_ELE_SIZE_MINUS_18;

#endif

	/*
	 * move data to tbuf for first mpkt
	 */
	PacketTx_MoveSopPayloToTbuf(bufDesc, 
								addrOfTbuf1,
								pEntry->payloadOffset,
								payloadLen,
								offsetMod8, 
								&sig_dramAccess1) ;

#ifndef ADD_PPP_HEADER

   	/*
	 * setup tcw01 base for sop_only case
	 */
	tcw01 = ( 1 << TCW0_SOP_BIT_LOC);

#else
 
    /*
	 * write  PPP header for SOP mpacket
	 */
	cntlWord01[0] = (PPP_IP_PROTOCOL << IP_PROTOCOL_SHFT);
	msf_write(&cntlWord01[0], 
			  (void *)addrOfTbuf1,
			  1,
			  sig_done,
			  &sig_msfAccess1);

   	/*
	 * setup tcw01 base for sop_only case
	 */
	tcw01 = ((1 << TCW0_SOP_BIT_LOC) |(TCW0_SOP_EOP_BASE_W1 << TCWO_W1_SHF));

#endif

	/*
	 * Set payload_length, payload_offset, and channel fields in tcw0.
	 */
	tcw01 = PacketTx_SetTcw0WithPayloLenOffsetChnl(tcw01, 
												   payloadLen,
												   offsetMod8,
												   pStatus->portId);

   	/*
	 * setup tcw0_2 
	 */

#ifndef ADD_PPP_HEADER

	payloadLen = pEntry->payloadLen - TBUF_ELE_SIZE_MINUS_8;

#else

	payloadLen = pEntry->payloadLen - TBUF_ELE_SIZE_MINUS_18;

	/*
	 * update inOffset_mod_8
	 */
	offsetMod8 += TBUF_ELE_SIZE_MINUS_18;
	offsetMod8 &= 0x7;

#endif

	tcw02 =(1 << TCW0_EOP_BIT_LOC); /*tcw0 base for tcw0_2*/

	/*
	 * set payload_length, payload_offset, and channel fields in tcw0 [3]
	 */
	tcw02 = PacketTx_SetTcw0WithPayloLenOffsetChnl(tcw02,
												   payloadLen,
												   offsetMod8,
												   pStatus->portId);

#ifndef ADD_PPP_HEADER

	offset += TBUF_ELE_SIZE_MINUS_8;

#else

	offset += TBUF_ELE_SIZE_MINUS_18;

#endif

	/*
	 *move data to tbuf for second mpkt, payload offset and payload length can 
	 *be retrived from tcw0_2
	 */
	toTbufPara.inBufDesc = bufDesc;
	toTbufPara.inAddrTbuf = addrOfTbuf2;
	toTbufPara.inOffset = offset;
	toTbufPara.inTcw0 = tcw02;
	toTbufPara.pInSigDram = &sig_dramAccess2;
	PacketTx_MoveNotSopPayloToTbuf(toTbufPara);
	sigmask_2 |= __signals(&sig_dramAccess2);

	/*
	 * now, check whether a valid tx request gotten in the scratch read from the
	 * scratch ring in phase 1. If there is a valid tx request, enqueu that 
	 * packet to the queue of that port 
	 */
	if((exeStatFlag.getTxRequestFlag == 1) && (txReq.value != 0x0))
	{
		/* 
		 *read SOP meta data, read earlier to reduce latency 
		 *read meta data from SRAM by using the BD pointer in tx request
		 */
		sigmask_2 = PacketTx_ReadMetaDataFromSram(sigmask_2,
												  txReq.value,
												  &sig_sramRead1);

		/*
		 *set sop meta read flag in exe_stst_flag 
		 */
		exeStatFlag.readSopDescFlag =1;

		/*
		 *that port and enqueue this packet 
		 */
		pEntry = PacketTx_UpdateQueueTail(pStatus,
										  portEntryTurnaround);
	}

	wait_for_all(sigmask_2);
	PacketTx_ImplicitReadAllSigs();

	/*
	 * End of phase 2.
	 */

#ifdef IXP2800

	tmp = pStatus->ctwVdBase + (tbufEle1 << ELE_2_TX_CNTRL_ADDR);

#else

	tmp = pStatus->ctwVdBase + (tbufEle1 << 3);

#endif

	/*
	 * beginning of phase 3	in ethernet_critical  mpacket path
	 * wake up next thread in the very beginning of phase 3 to reduce wakeup 
	 * latency
	 */
	signal_next_ctx(ntSig3Gpr);
	

	/*
	 * write tcw for first mpkt
     * write transmit contrl word (TCW) to TBUF_ELEMENT_CTRL_V_# corresponding
	 * the tbuf_element to initiate transmission
	 */
	cntlWord01[0] =tcw01;
	msf_write(&cntlWord01[0],
	          (void *)tmp,
			  2,
			  sig_done,
			  &sig_msfAccess1);

#ifdef IXP2800

	addrOfTxControl2 =  pStatus->ctwVdBase + (tbufEle2 << ELE_2_TX_CNTRL_ADDR);

#else

	addrOfTxControl2 = 	 pStatus->ctwVdBase + (tbufEle2 << 3);

#endif

	/*
	 * write tcw for second mpkt
     * write transmit contrl word (TCW) to TBUF_ELEMENT_CTRL_V_# corresponding
	 * the tbuf_element to initiate transmission
	 */
	cntlWord23[0] = tcw02;
	cntlWord23[1] = pStatus->portId;
	msf_write(&cntlWord23[0],
			  (void *)addrOfTxControl2,
			  2,
			  sig_done,
			  &sig_msfAccess2);

#ifdef COUNTERS

	PacketTx_UpdateBytesTransmitted(tcw01, 
									sig_counterUpdate,
									&byteCnt);
#endif

	PacketTx_FreeBuffer(bufDesc);
	if( exeStatFlag.readSopDescFlag == 1)
	{ 
		/*
		 * medat data for sop is ready, move them to local memory
		 */
		PacketTx_SaveSopMeta(pEntry,
							 pStatus,
							 txReq.sopMetaHndl); 
	}

#ifndef COUNTERS

	wait_for_all(&sig_msfAccess1,
				 &sig_msfAccess2,
				 &sig_next1);
#else

	wait_for_all(&sig_msfAccess1,
				 &sig_msfAccess2,
				 &sig_counter_update,
				 &sig_next1);

#endif
	__implicit_read(cntlWord01);
	__implicit_read(cntlWord23);

} /* PacketTx_EthernetCriticalPkt()*/

/*****************************************************************************
 * Abstract:
 * handle mpacket which is not the first mpacket of a 
 *				multiple-mpacket packet
 *
 * Side Effects:
 *    None
 *
 * Assumptions:
 * 	  None
 *****************************************************************************/
void												/* OUT: No return value  */
PacketTx_NotSopMpkt(						
					q_entry_t *pEntry,				/* IN/OUT: Entry pointer */
					q_status_t *pStatus)			/*IN/OUT: Status pointer */
{
	unsigned int leftOvOffset;						/* Left over data offset. */
	unsigned int payloadOffset;						/* Current payload length.*/
	unsigned int tmp;								/* Temp. register.		  */
	unsigned int leftOvLen;							/* Left over data length. */				
	unsigned int allowMaxTxed;						/* Allowed max transmited.*/
	unsigned int needToTransmit;					/* Need to be transmited. */
	unsigned int allowToTransmit;					/* Aloowed to ve transmit.*/
	unsigned int tcw01;								/* Transmit control word. */
	unsigned int offsetMod8;						/* Offset  mode 8.        */
	unsigned int tbufEle;							/* Current TBUF element.  */					
	unsigned int addrOfTbufEle;						/* TBUF element address.  */
	unsigned int currBufdesc;						/* Current buffer desc.	  */
	SIGNAL_MASK sigmask_2;							/* Signal mask 2.		  */
	__declspec(sram_write_reg) int cntlWord[2];		/* TBUF control word.	  */
	__declspec(dram_read_reg) int dramWord[4];		/* Dram word.			  */
	in_parameter_1_t toTbufPara;					/* Input parametr struct. */


#ifndef DISABLE_TX2SCHED_FEEDBACK

	/*
	 * since maybe no complete packet is sent with this thread, clear the 
	 * signal bit related reflect write in sigmask_1 first.
	 */
	sigmask_1 = PacketTx_ClearSignal(sigmask_1, &sig_reflectWrite);
#endif

	currBufdesc = pEntry->currBufDesc;
	tcw01 = pStatus->portId;
    payloadOffset =  pEntry->payloadOffset;

	/* 
	 *check whether to read secondary BD.
	 */
    if(pEntry->rdNxtMetaFlag == 1)
	{
		/* 
		 * Read secondary buffer meta data from SRAM.
		 */
		sigmask_1 = PacketTx_ReadMetaDataFromSram(sigmask_1,
										pEntry->value[2],
										&sig_sramRead1);

		/* 
		 *reset need to read secondary buffer meta data flag.
		 */
		pEntry->rdNxtMetaFlag = 0;

		/*
		 * Set read secondary BD flag in exeStatFlag for use in phase 2.
		 */
		exeStatFlag.rdSecndBufDescFlag = 1;

		/*
		 * check whether this port should be blocked from transmission for any 
		 * following threads which serve the same in the same run to prvent them 
		 * accessing  the secondary buffer meta data (which is not ready yet in this
		 * phase, but will be ready in next phase) when they become active buffer
		 * calculate the max bytes to consume before secondary buffer meta data need
		 * to be ready.
		 */

		tmp = (TBUF_ELE_SIZE  << ME_THREAD_NUM_SHFT);
		if(pEntry->payloadLen <= tmp)
		{
			/*
			 * Set qStatus[port].not_locksndb_read_flag to 0 to prvent any threads
		 	 * handling the same port in the same run to access the secondary buffer data 
		 	 * (which is not ready yet) when it becomes active buffer, set 
		 	 * put qStatus[port].not_locksndb_read_flag to 1
		 	 * when secondary meta data saved in phase 2.
			 */
			pStatus->noLockBufReadFlag = 0;
		}
	}

  	/*
	 * get ab_bd, rmndPaylo, payloadOffset, offset_mod_8, ab_flist for active 
	 * buffer
	 */
	offsetMod8 = (payloadOffset & CONST_07);

	/*
	 *check whether there is some leftover bytes (maximum 8 bytes) from 
	 *previous bufferwhich need to be moved to tbuf at the beginning
	 *of this mpacket.
	 */
	if(pStatus->leftOverFlag == 1)
	{

		/* 
		 *Reset the leftover flag
		 *move leftover bytes to w0 for latter use, also clear that bit,
		 *so later threads in the same run will not repeat the  same thing.
		 */
		qStatus[port].leftOverFlag = 0;
		cntlWord[0] = pStatus->leftOvWord;
		cntlWord[1] = 0;

		/* 
		 * payload leftover will be put into prepend to eliminate any alignment problem
		 * also before saved to local memory, byte alignment is done on leftover bytes
		 * so make sure the offset of those leftover bytes is 0.
		 */
		leftOvLen = pStatus->leftOvLen;
		tcw01 |= (leftOvLen << TCWO_PREPEND_LEN_LSB_LOC);
		exeStatFlag.leftOvFlag = 1;

		/*
		 *check whether current mpacket is eop packet
		 *first check whether this is a last buffer in a packet.
		 */
    	if(pEntry->secndBufDesc == IX_NULL)
		{
			/* 
			 *check whether all remianed payload can fit into one tbuf, leftover already
			 *occupies 8 bytes in tbuf.
			 */
			tmp = offsetMod8 + pEntry->payloadLen;
			if(TBUF_ELE_SIZE_MINUS_8 >= tmp)
			{
				/*
				 *it is EOP case, set EOP bit in tcw01.
				 */
				tcw01  |= (1 << TCW0_EOP_BIT_SHFT);
				
				/* 
				 * set payload_length and payload_offset of tcw01 with rmndPaylo and
	 			 * offsetMod8.
				 */
				tcw01 = PacketTx_SetTcw0WithPayloLenOffset(tcw01,
														   pEntry->payloadLen,
														   offsetMod8);
				
				/* 
				 * Eop mpacket is eob mpacket too, need to free buffer at phase 3.
				 */
				exeStatFlag.eobMpktFlag = 1;
				goto not_sop_need_to_advance_queue_head;
			}

			/* calculate what max can be transmit from this buffer
			 * first total max payload (include lp_paylo) must be multiple of 
		   	 * device bus width
			 */
			allowMaxTxed = ((TBUF_ELE_SIZE_MINUS_8 - offsetMod8 + leftOvLen) &
		 					DEVICE_BUS_WIDTH_RESTRICTION_MASK) - leftOvLen;

			/* 
			 * update ab_paylo_rmnd and ab_payloadOffset in queue entry with 
			 * allowMaxTxed.
			 */
			PacketTx_UpdateAbPayloRmndAndOffsetRpaylo(pEntry, allowMaxTxed);

			/*
			 * Set payload_length and payload_offset of tcw01 with allowMaxTxed
		   	 * and offset_mod_8.
			 */
			tcw01 = PacketTx_SetTcw0WithPayloLenOffset(tcw01,
													   allowMaxTxed,
													   offsetMod8);
			 goto not_sop_end_of_phase1;
		}

		/* 
		 * Since not last buffer in a packet , so only check whether it is a eob 
		 * mpacket which rmndPaylo we can say it is eob mpacket
		 * calculate what max can be transmit from this buffer
	     * first total max payload (include lp_paylo) must be multiple of device 
	     * bus width.
		 */
		allowMaxTxed = ((TBUF_ELE_SIZE_MINUS_8 - offsetMod8 + leftOvLen) &
							 DEVICE_BUS_WIDTH_RESTRICTION_MASK) -
							 leftOvLen;

		/*
		 * If rmndPaylo is less than this max allowable bytes, of cours, we are in 
		 * eob.
		 */ 
		if(allowMaxTxed >= pEntry->payloadLen)
		{
			exeStatFlag.eobMpktFlag = 1;
			needToTransmit = leftOvLen + pEntry->payloadLen;
			allowToTransmit = needToTransmit & DEVICE_BUS_WIDTH_RESTRICTION_MASK;
			allowToTransmit = needToTransmit - leftOvLen;

			/* Set payload_length and payload_offset of tcw01 with allow_to_transmit
			 * and offset_mod_8.
			 */
			tcw01 = PacketTx_SetTcw0WithPayloLenOffset(tcw01,
													   allowToTransmit,
													   offsetMod8);

			/*
			 * Check whether there are some leftover to raed to dramWrod[0], dramWrod[1], dramWrod[2], and
			 * dramWrod[3] check whether we have some leftover to next buffer.
			 */
			if(pEntry->payloadLen != allowToTransmit)		
			{
				exeStatFlag.saveLeftOvFlag = 1;

    			/* 
				 * Use following dl_buf_get_data_from_meta macro from dispatch_loop.uc to
				 * get dram address at the beginning of the buffer from true_dl_buf_hdl
				 * now it takes one instruction.
				 */
				tmp = Dl_BufGetDataFromMeta((currBufdesc << 2));
				tmp += (payloadOffset + allowToTransmit);	
	
				/* 
				 *in some case, the spome of offset_mod_8 + bytes_leftvoer will over 8 bytes 
				 *(for example, offset_mod_8=7, and bytes_leftover is 3), so inorder to 
				 *cover all cases, read 16 bytes here add sig_dramAccess1 to sig_mask
				 * tmp is used as dram Address here.
				 */
				dram_read(&dramWord, 
						  (__declspec(dram) void *)tmp,
						  2,
						  sig_done, 
						  &sig_dramAccess1);	
				sigmask_1 |= __signals(&sig_dramAccess1);

				/* 
				 * This port should be blocked from transmission before those dramWrod[0], dramWrod[1],
				 * dramWrod[2], and dramWrod[3] is gotten and saved to LM.
				 */
				pStatus->noLockLeftOvFlag = 0;
				leftOvOffset = (payloadOffset + allowToTransmit) & CONST_07; 	/* 8 bytes boudary*/
				pStatus->leftOvLen = pEntry->payloadLen - allowToTransmit;
			}
			PacketTx_SecondaryBufferBecomeActiveBuffer(pEntry, pStatus);
			goto not_sop_end_of_phase1;
		}
		
		/* 
		 * Update ab_paylo_rmnd and ab_offsetRpaylo in queue entry with 
		 * allowMaxTxed.
		 */
		PacketTx_UpdateAbPayloRmndAndOffsetRpaylo(pEntry, allowMaxTxed);

		/* 
		 * Set payload_length and payload_offset of tcw01 with allowMaxTxed
		 * and offset_mod_8.
		 */
		tcw01 = PacketTx_SetTcw0WithPayloLenOffset(tcw01, 
												   allowMaxTxed,
												   offsetMod8);
		goto not_sop_end_of_phase1;
	}

	/*
	 * Not sop and no leftover,	check whether current mpacket is eop packet
	 * first check whether this is a last buffer in a packet.
	 */
	
    if(pEntry->secndBufDesc == IX_NULL)
	{
		/* 
		 * Check whether all remianed payload can fit into one tbuf, leftover already
		 * occupies 8 bytes.
		 */
		tmp = offsetMod8 + pEntry->payloadLen;
		if(TBUF_ELE_SIZE >= tmp)
		{
			/*
			 * It is EOP case, set EOP bit in tcw01.
			 */
			tcw01 |= ( 1 << TCW0_EOP_BIT_SHFT);

			/* 
			 * Set payload_length and payload_offset of tcw01 with rmndPaylo
			 * and offset_mod_8.
			 */
			tcw01 = PacketTx_SetTcw0WithPayloLenOffset(tcw01,
													   pEntry->payloadLen,
													   offsetMod8);
			exeStatFlag.eobMpktFlag = 1;
			goto not_sop_need_to_advance_queue_head;
	
		}
		tmp = TBUF_ELE_SIZE - offsetMod8;
		allowMaxTxed = tmp & DEVICE_BUS_WIDTH_RESTRICTION_MASK;

		/*
		 * Update ab_paylo_rmnd and ab_offset_rpaylo in queue entry with 
		 * allowMaxTxed.
		 */
		PacketTx_UpdateAbPayloRmndAndOffsetRpaylo(pEntry,
												  allowMaxTxed);

		/* Set payload_length and payload_offset of tcw01 with allowMaxTxed
		 * and offset_mod_8.
		 */
		tcw01 = PacketTx_SetTcw0WithPayloLenOffset(tcw01,
												   allowMaxTxed,
												   offsetMod8);
		goto not_sop_end_of_phase1;
	}

	/*
	 * No lp not last buffer, check whether is EOB which rmndPaylo we can say it is eob
	 * calculate what max can be transmit from this buffer
	 * first total max payload (include lp_paylo) must be multiple of device 
	 * bus width.
	 */
	tmp =  TBUF_ELE_SIZE - offsetMod8;
	allowMaxTxed = tmp & DEVICE_BUS_WIDTH_RESTRICTION_MASK;

	/*
	 * if rmndPaylo is less than this max allowable bytes, of cours, we are in 
	 * eob.
	 */
	if(allowMaxTxed >= pEntry->payloadLen)
	{
		exeStatFlag.eobMpktFlag = 1;
		allowToTransmit = pEntry->payloadLen & DEVICE_BUS_WIDTH_RESTRICTION_MASK;
	
		/* 
		 * Check whether we have something to transmit.
		 */
		if(allowToTransmit <= 0)
		{
			/*
			 * if it is the case, then set skip bit in tcw01.
			 */
			tcw01 |= ( 1 << TCW0_SKIP_BIT_SHFT);
	
			/* 
			 *Set flag in exeStatFlag for later use.
			 */
			exeStatFlag.skipTransmitFlag = 1;
		}

		/* 
		 * Set payload_length and payload_offset of tcw01 with allow_to_transmit
		 * and offset_mod_8.
		 */
		tcw01 = PacketTx_SetTcw0WithPayloLenOffset(tcw01,
												   allowToTransmit,
												   offsetMod8);

		/*
		 * Check whether there are some leftover to raed to dramWrod[0], dramWrod[1], dramWrod[2], and 
		 * dramWrod[3].
		 */
		if(pEntry->payloadLen != allowToTransmit)
		{
			/*
			 * Set flag in exeStatFlag for later use.
			 */
			exeStatFlag.saveLeftOvFlag = 1;

    		/*
			 * Use following dl_buf_get_data_from_meta macro from dispatch_loop.uc to
	 		 * get dram address at the beginning of the buffer from true_dl_buf_hdl
	 		 * now it takes one instruction
	 		 */ 
    		tmp = Dl_BufGetDataFromMeta((currBufdesc << 2)) + payloadOffset + allowToTransmit;

			/*
			 * In some case, the spome of offset_mod_8 + bytes_leftvoer will over 8 bytes 
		  	 * (for example, offset_mod_8=7, and bytes_leftover is 3), so inorder to 
		 	 * cover all cases, read 16 bytes here.
			 */
			dram_read(&dramWord, 
					  (__declspec(dram) void *)tmp,
					  2,
					  sig_done,
					  &sig_dramAccess1);	
	
			/* 
			 * Add sig_dramAccess1 to sigmask_1.
			 */
			sigmask_1 |= __signals(&sig_dramAccess1);

			/*
			 * This port should be blocked from transmission before those dramWrod[0], dramWrod[1], 
			 * dramWrod[2], and dramWrod[3] is gotten and saved to LM.
			 */
			pStatus->noLockLeftOvFlag = 0;
			leftOvOffset = (payloadOffset + allowToTransmit) & CONST_07; 		/*8 bytes boudary */
			pStatus->leftOvLen = pEntry->payloadLen - allowToTransmit;
		}
		PacketTx_SecondaryBufferBecomeActiveBuffer(pEntry, pStatus);
		goto not_sop_end_of_phase1;
	}

	/*
	 * Update ab_paylo_rmnd and ab_offset_rpaylo in queue entry with 
	 * allowMaxTxed.
	 */
	PacketTx_UpdateAbPayloRmndAndOffsetRpaylo(pEntry, allowMaxTxed);

	/*
	 * Set payload_length and payload_offset of tcw01 with allowMaxTxed
	 * and offset_mod_8.
	 */
	tcw01 = PacketTx_SetTcw0WithPayloLenOffset(tcw01,
											   allowMaxTxed,
											   offsetMod8);
	goto not_sop_end_of_phase1;

	/* 
	 * Advance queue header for EOP mpacket.
	 */
not_sop_need_to_advance_queue_head:

	/* 
	 * Even the deqPort_bit wil be calculated in PacketTx_AdvanceQueueHead()
	 * macro gaian, but it is the same as original deqPort_bit.
	 */
	PacketTx_AdvanceQueueHead(pStatus, portEntryTurnaround);

#ifndef DISABLE_TX2SCHED_FEEDBACK

	/* 
	 * Notify scheduler that one packet has been transmitted from this port.
	 */
	PacketTx_NotifyScheduler(pStatus, pStatus->portId);
	sigmask_1 = PacketTx_SetSignal(sigmask_1, &sig_reflectWrite);
#endif

not_sop_end_of_phase1:

	sigmask_2 = 0;
	cntlWord[1] = pStatus->portId;
	wait_for_all(sigmask_1);
	PacketTx_ImplicitReadAllSigs();

	/*
	 * Beginning of phase 2 with not_sop mpkt ready for tx	
	 * wake up next thread in beginning of phase 2 to reduce wakeup latency.
	 */
 	signal_next_ctx(ntSig2Gpr);
	sigmask_2 = PacketTx_SetSignal(sigmask_2, &sig_next3);

	/*
	 * Get next available tbuf element (tbufEle) and corresponding address 
	 * (addrOfTbufEle) for it [5].
	 */
	PacketTx_GetTbufEleAndAddr(pStatus, &tbufEle, &addrOfTbufEle);

	/*
	 * Check tbuf full to avoid overwritting tbuf.
	 */
	if(PacketTx_CheckTbufFullNotSopEop(tbufEle) != 0)
	{
		PacketTx_WaitTillTbufNotFull( tbufEle );
	}

	/* 
	 * Check whether need to move leftover (w0) to the beginning of tbuf.
	 */
	if(exeStatFlag.leftOvFlag ==1)
	{
		msf_write(&cntlWord[0], 
				  (void *)addrOfTbufEle,
				  1, 
				  sig_done,
				  &sig_msfAccess1);
		sigmask_2 = PacketTx_SetSignal(sigmask_2, &sig_msfAccess1);
		addrOfTbufEle += 8;
	}

	/* 
	 * Check whether need to move payload from active buffer to tbuf.
	 */
	if(exeStatFlag.skipTransmitFlag == 0)
	{
		/* 
		 * Payload offset and payload length can be retrived from tcw01.
		 */
		toTbufPara.inBufDesc = currBufdesc;
		toTbufPara.inAddrTbuf = addrOfTbufEle;
		toTbufPara.inOffset = payloadOffset;
		toTbufPara.inTcw0 = tcw01;
		toTbufPara.pInSigDram = &sig_dramAccess1;
		PacketTx_MoveNotSopPayloToTbuf(toTbufPara);
		sigmask_2 |=__signals(&sig_dramAccess1);
	}

	/*
	 * Check whether we have leftover which needed to be to saved in LM.
	 */
	if(exeStatFlag.saveLeftOvFlag == 1)
	{
		/*
		 * dw is ready now, do necessary bytes alignment with
		 * leftOvOffset, so the leftover will be realigned in first two longword
		 * without any offset to the first byte of the first long word.
		 */
	
		pStatus->leftOvWord = PacketTx_LeftOverBytesAlignment(leftOvOffset,
														 (int *) &dramWord[0]);
		pStatus->leftOverFlag = 1;
		pStatus->noLockLeftOvFlag = 1;
	}

	/*
	 * Check whether we need to save secondary buffer meta data 
	 * if didn't read meta data in phase 2, just branch out.
	 */
	if(exeStatFlag.rdSecndBufDescFlag == 1)
	{
		/*
		 * Medat data for secondary buffer is ready, move them to local memory.
		 */
		PacketTx_SaveSbdMeta(pEntry, pStatus);

		/*
		 * This port was blocked to prvent any following threads (which serves 
		 * this port) in the same run to access the secondary buffer data when it 
		 * becomes active buffer, now secondary meta data saved, put this port back.
		 */
		pStatus->noLockBufReadFlag = 1;
	}

	/*
	 * Now, check whether a valid tx request gotten in the scratch read from the
	 * scratch ring in phase 1. If there is a valid tx request, enqueu that 
	 * packet to the queue of that port.
	 */
	if((exeStatFlag.getTxRequestFlag == 1) && (txReq.value != 0x0))
	{
		sigmask_2 = PacketTx_ReadMetaDataFromSram(sigmask_2,
												  txReq.value,
												  &sig_sramRead1);
		/*
	 	 * Set sop meta read flag in exe_stst_flag.
	 	 */
		exeStatFlag.readSopDescFlag = 1;

		/*
		 * Set local memory index 0 to queue entry at queue tail of 
	 	 * that port and enqueue this packet.
	 	 */
		pEntry = PacketTx_UpdateQueueTail(pStatus, 
										  portEntryTurnaround);
	}

	/* End of phase 2 */

	 wait_for_all(sigmask_2);
	 PacketTx_ImplicitReadAllSigs();

#ifdef IXP2800
	tmp = (tbufEle << ELE_2_TX_CNTRL_ADDR + pStatus->ctwVdBase;
#else
	tmp = (tbufEle << 3) + pStatus->ctwVdBase;
#endif

	/*
	 * Beginning of phase 3.
	 */
	signal_next_ctx(ntSig3Gpr);

    /* 
	 * Write transmit contrl word (TCW) to TBUF_ELEMENT_CTRL_V_# corresponding
	 * the tbuf_element to initiate transmission. Tmp used as control address
	 */
	cntlWord[0] = tcw01;
	cntlWord[1] = pStatus->portId;
	msf_write(&cntlWord[0], (void *)tmp, 2, sig_done , &sig_msfAccess1);

#ifdef COUNTERS
	PacketTx_update_bytes_transmitted(tcw01, sig_CounterUpdate, byteCnt);
#endif 

	/*
	 * Check whether need to free buffer.
	 */
	if(exeStatFlag.eobMpktFlag == 1)
	{
		PacketTx_FreeBuffer(currBufdesc);
	}

	/*
	 * If didn't read meta data in phase 2, just branch out.
	 */
	if(exeStatFlag.readSopDescFlag == 1)
	{
		/*
		 * medat data for sop is ready, move them to local memory.
		 */
		PacketTx_SaveSopMeta(pEntry,
							 pStatus,
							 txReq.sopMetaHndl); 
	}
#ifndef COUNTERS
 	wait_for_all(&sig_msfAccess1, &sig_next1);
#else
	wait_for_All(&sig_msfAccess1, 
				 &sig_counter_update,
				 &sig_next1);
#endif
} /* PacketTx_NotSopMpkt()*/

/*****************************************************************************
 * Abstract:
 *    Packet (POS/Ethernet) transmission main loop, never return
 *
 * Side Effects:
 *    None
 *
 * Assumptions:
 * 	  None
 *****************************************************************************/											
INLINE void											/* OUT: No return value  */
PacketTx(void)										/* No parameter.		 */
{ 
	unsigned int tmp, tmp2;								/* Temp. register.       */
	q_entry_t *pEntry;								/* Queue entry pointer.  */ 
	q_status_t *pStatus = &qStatus[port];			/* Stutas pointer.		 */

	PacketTx_AssignSigReg();	/* Wait global initialization */
	PacketTx_InitGprs(); 		/* Gprs initialized so they may save some cycles*/
	wait_for_all(&sig_next1);	/* Wait for my turn*/

	/*
	 * start phase 1														  
	 */

	while(1)
	{
	    tmp2 = pStatus->pktsInQ;
		exeStatFlag.value = 0; 					  	/*Reset exeStatFlag.*/
		sigmask_1 = sigmask_phs1Default;  		  	/*set signal mask 1*/
		pEntry = (q_entry_t *) pStatus->headOffset;
		tmp = pStatus->noLockBufReadFlag & pStatus->noLockLeftOvFlag; 
		signal_next_ctx(ntSig1Gpr); 			  	/*send signal to next thread*/
		if(tmp2 <= SKIP_GET_TXR_THRESHOLD)
		{
			scratch_get_ring(&txReq,
							(volatile __declspec(scratch) void *)scrRingTr,
		   	              	1,
							sig_done,
							&sig_scratchAccess1);   /* Read scratch ring*/
			exeStatFlag.getTxRequestFlag = 1;
			sigmask_1 |= ( 1 << __signal_number(&sig_scratchAccess1));
		}

		/*
		 * check if there is a mpkt ready for transmiting.
		 */

    	if((pStatus->pktsInQ != 0) && (tmp != 0))
		{
			/* 
			 * get mpackets actually sent out of the tbuf for use in phase 2 to avoid
	   	 	 * overwritting existing mpackets which have not been transmitted out yet
		 	 */
			msf_read(&mpktsSent,
					 (void *)addrTxSeq,
					 1,
					 sig_done,
					 &sig_msfAccess3);
	
			if(pEntry->sopFlag != 0) /*check SOP bit */
			{

#ifndef ADD_PPP_HEADER 

   				tmp = TBUF_ELE_SIZE - (pEntry->payloadOffset & CONST_07);

#else

   				tmp = TBUF_ELE_SIZE_MINUS_8 - (pEntry->payloadOffset & CONST_07);

#endif

				if(pEntry->payloadLen < tmp)
				{
					PacketTx_SopEopMpkt(pEntry,
					                    pStatus);
				}
				else
				{
					if((pEntry->secndBufDesc == IX_NULL) &&

#ifndef ADD_PPP_HEADER

					/* 
	 			 	 * for a packet less than  or equal to 240 bytes, it can be fitted into 
				 	 * two mpackes; make simple, first mpkt take 120 bytes, others in second
				 	 * mpaket
	 			 	 */
					(pEntry->payloadLen <= TWO_TBUF_ELE_SIZE_MINUS_16))

#else 
					/* 
				 	 * for a packet less than  or equal to 230 bytes, it can be fitted into
	 			 	 * two mpackts; make simple, first mpkt take 120 bytes, others in second 
	 			 	 * mpaket
	 			 	 */
					(pEntry->payloadLen <= TWO_TBUF_ELE_SIZE_MINUS_26))

#endif
					{
						PacketTx_EthernetCriticalPkt(pEntry,
													 pStatus);
					}
					else
					{
						PacketTx_SopOnlyNonCriticalMpkt(pEntry,
														pStatus);
					}
				}
			}
			else
			{		
				PacketTx_NotSopMpkt(pEntry,
									pStatus);
			}
		} 												/* Not ready to tx*/
		else
		{
			PacketTx_GetTxRequest(pEntry,
								  pStatus);
		}
	}
} /* packetTx()*/


/*****************************************************************************
 * Abstract:
 *    Packet (POS/Ethernet) transmission main loop
 *
 * Side Effects:
 *    None
 *
 * Assumptions:
 * 	  None
 *****************************************************************************/	     
main( void)
{
	PacketTx_Init();						/* Initialization fucntion.		 */
	PacketTx();								/* Transmite function.			 */
}
