/******************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2002 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *******************************************************************************/
/*****************************************************************************
 *
 * File:  PacketTx_init.c
 *
 * Abstract:
 *      This is the SPHY and NPHY4 transmit code init file.  This file 
 *      defines the initialization functions used in the SPHY and MPHY_4
 *		trasmiting microengine C code..
 * Contents:
 *        Definitions of initialization function.
 *****************************************************************************/
#include <sphy_mphy4_tx.h>


/*****************************************************************************
 * Abstract:
 *    initialize context relative GPRs for specific thread to save 
 *    instruction cycles in POS TX processing
 *
 * Side Effects:
 *    None
 *
 * Assumptions:
 * 	  None
 *****************************************************************************/
INLINE void										/* OUT: No return value      */ 
PacketTx_ThreadSpecificInit(
		int inStringForThread,					/* IN: SCR string for thread */
		int inNextThread,						/* IN: Next thread			 */
		int inPort)								/* IN: ort number			 */
{
 	scrRingTr = inStringForThread << 2; 
	addrTxSeq = TX_SEQUENCE_0 + (inPort << TX_SEQUENCE_ADDR_SHF);
 	portEntryTurnaround = (unsigned int ) &(qPort[inPort].q_entry[0])  
						  + ((inPort + 1) * TOTAOL_QUEUE_ENTRY_SIZE_PER_PORT)
						  - QUEUE_ENTRY_SIZE;
} /* PacketTx_ThreadSpecificInit()*/

/*****************************************************************************
 * Abstract:
 *    Initialize the msf
 *    
 *
 * Side Effects:
 *    None
 *
 * Assumptions:
 * 	  None
 *****************************************************************************/
INLINE void										/* OUT: No return value      */
PacketTx_MsfTxControlInit()
{
	int address;
	SIGNAL sig_temp;
	__declspec(sram_write_reg) int temp;

	temp = 0x4;
	msf_write(&temp, (void *)MSF_TX_CNTL, 1, ctx_swap, &sig_temp);
#if (TX_PHY_MODE == SPHY_1_32)
	temp = PACKET_TX_SPHY_1X32_CTL_LW;
#endif

#if (TX_PHY_MODE == SPHY_4_8)
	temp = PACKET_TX_SPHY_4X8_CTL_LW;
#endif

#if (TX_PHY_MODE == MPHY_4)
	temp = PACKET_TX_MPHY4_CTL_LW;
#endif
	msf_write(&temp, (void *)MSF_TX_CNTL, 1, ctx_swap, &sig_temp);
	PacketTx_Set16BitMsfReg(TX_UP_CTRL_0, POS_TX_UP_CTL_WD);
#if ((TX_PHY_MODE == SPHY_4_8) \
	 || (TX_PHY_MODE == MPHY_4))
	PacketTx_Set16BitMsfReg(TX_UP_CTRL_1, POS_TX_UP_CTL_WD);
	PacketTx_Set16BitMsfReg(TX_UP_CTRL_2, POS_TX_UP_CTL_WD);
	PacketTx_Set16BitMsfReg(TX_UP_CTRL_3, POS_TX_UP_CTL_WD);
#endif
}  /* PacketTx_MsfTxControlInit()*/

/*****************************************************************************
 * Abstract:
 *    	Initialize the the globals in local memoryfor specific port
 *    
 *
 * Side Effects:
 *    None
 *
 * Assumptions:
 * 	  None
 *****************************************************************************/
INLINE void										/* OUT: No return value      */
PacketTx_InitPort(
	__declspec(local_mem shared) q_status_t *pStatus,/* OUT: Status pointer  */
	int inPort)										/* IN: Port number		 */	
{
	int i, base;

	/* initialize local memory index 0 to the base for global info for this port*/
	pStatus->portId = inPort;
	pStatus->ctwVdBase = TBUF_ELEMENT_CTRL_VALID + (inPort << TCW_VD_BASE_SHF);
	pStatus->tbufBase = RBUF_TBUF_BASE + (inPort * TBUF_BASE_SHF) ;
    pStatus->qEntryBase = (unsigned int) &(qPort[inPort].q_entry[0]);
    pStatus->headOffset = (unsigned int) &(qPort[inPort].q_entry[0]);
    pStatus->tailOffset = (unsigned int) &(qPort[inPort].q_entry[0]);
    pStatus->noLockBufReadFlag = 1;
    pStatus->noLockLeftOvFlag = 1;
	for (i =8; i<16; i++)
		pStatus->value[i] = 0;	
 } /* PacketTx_InitPort()*/

/*****************************************************************************
 * Abstract:
 *   Initialize the local memory including queue info and globals for port
 *    
 *
 * Side Effects:
 *    None
 *
 * Assumptions:
 * 	  None
 *****************************************************************************/
INLINE void									/* OUT: No return value			 */
PacketTx_InitLocalMemory()
{
	int i, j, k;

#if (TX_PHY_MODE == SPHY_1_32)
	for(i = 0; i < 16; i++ )
		for(j = 0; j < 4; j++)
			qPort[0].q_entry[i].value[j] = 0;  /* clean the queue entries */	
#endif

#if ((TX_PHY_MODE == SPHY_4_8) \
	 || (TX_PHY_MODE == MPHY_4))
	for(i = 0; i < 4; i++)
		for(j = 0; j < 16; j++)
			for(k = 0; k < 4; k++)
		 		qPort[i].q_entry[j].value[k] = 0;
#endif
 
	PacketTx_InitPort(&qStatus[PORT_0], PORT_0); /* clean the queue entries */

#if ((TX_PHY_MODE == SPHY_4_8) \
	 || (TX_PHY_MODE == MPHY_4))
	PacketTx_InitPort(&qStatus[PORT_1], PORT_1);
	PacketTx_InitPort(&qStatus[PORT_2], PORT_1);
	PacketTx_InitPort(&qStatus[PORT_3], PORT_1);
#endif

} /* PacketTx_init_local_memory()*/

/*****************************************************************************
 * Abstract:
 *   Initialization of the microblock
 *    
 *
 * Side Effects:
 *    None
 *
 * Assumptions:
 * 	  None
 *****************************************************************************/
INLINE void									/* OUT: No return value			 */
PacketTx_Init()
{
	port =0;			/* Thread 0 and 1 always deal with port 0*/
	switch(ctx())
	{
		case 0:
   			/*
			 * Initialize MSF_TX_CNTL register.
			 */
			PacketTx_MsfTxControlInit();
			PacketTx_InitLocalMemory();

#ifdef WAIT_FOR_COMMON_RESOURCE_INITIALIZATION 
			wait_for_all(&sig_commonInitComplete);
#endif

			/*
			 * For kick start thread 0 will send itself 3 initial signals for 
	   		 * phase 1, 2, and 3.
			 */
			signal_same_ME(__signal_number(&sig_next1), 0);
			__implicit_write(&sig_next1);
			signal_same_ME(__signal_number(&sig_next2), 0);
			__implicit_write(&sig_next2);
			signal_same_ME(__signal_number(&sig_next3), 0);
			__implicit_write(&sig_next3);

			/* 
			 *Thread 0 context relative GPR variables initialization.
			 */
			PacketTx_ThreadSpecificInit(SCRATCH_RING_THREAD_0,
										1,
										PORT_HANDLED_BY_THREAD_0);
			break;

		case 1:	/* Thread 1 initialization*/
   			PacketTx_ThreadSpecificInit(SCRATCH_RING_THREAD_1,
										2,
										PORT_HANDLED_BY_THREAD_1);
			break;

		case 2: /* Thread 2 initialization*/
#if ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4))
			port=1; /* Thread 2 and 3 will deal with port 1 for SPHY_4_8 and MPHY_4*/
#endif
	   		PacketTx_ThreadSpecificInit(SCRATCH_RING_THREAD_2,
										3,
										PORT_HANDLED_BY_THREAD_2);
			break;	

		case 3: /* Thread 3 initialization*/
#if ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4))
			port=1; /* Thread 2 and 3 will deal with port 1 for SPHY_4_8 and MPHY_4*/
#endif
   			PacketTx_ThreadSpecificInit(SCRATCH_RING_THREAD_3,
										4,
										PORT_HANDLED_BY_THREAD_3);
			break;
	
		case 4:/* Thread 4 initialization*/
#if ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4))
			port=2; /* Thread 4 and 5 will deal with port 2 for SPHY_4_8 and MPHY_4*/
#endif
   			PacketTx_ThreadSpecificInit(SCRATCH_RING_THREAD_4,
										5,
										PORT_HANDLED_BY_THREAD_4);
			break;

		case 5:/* Thread 5 initialization*/
#if ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4))
			port=2; /* Thread 4 and 5 will deal with port 2 for SPHY_4_8 and MPHY_4*/
#endif
			PacketTx_ThreadSpecificInit(SCRATCH_RING_THREAD_5,
										6,
										PORT_HANDLED_BY_THREAD_5);
			break;

		case 6:/* Thread 6 initialization*/
#if ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4))
			port=3; /* Thread 6 and 7 will deal with port 2 for SPHY_4_8 and MPHY_4*/
#endif
    		PacketTx_ThreadSpecificInit(SCRATCH_RING_THREAD_6,
										7,
										PORT_HANDLED_BY_THREAD_6);
			break;

		case 7:/* Thread 7 initialization*/
#if ((TX_PHY_MODE == SPHY_4_8) || (TX_PHY_MODE == MPHY_4))
			port=3; /* Thread 6 and 7 will deal with port 2 for SPHY_4_8 and MPHY_4*/
#endif
    		PacketTx_ThreadSpecificInit(SCRATCH_RING_THREAD_7,
										0,
										PORT_HANDLED_BY_THREAD_7);
	}

	ntSig1Gpr = init_interthread_sig(__signal_number(&sig_next1));
	ntSig2Gpr = init_interthread_sig(__signal_number(&sig_next2));
	ntSig3Gpr = init_interthread_sig(__signal_number(&sig_next3));

} /* PacketTx_Init()*/
