////////////////////////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
////////////////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: sphy_mphy4_tx.h
//
//      Purpose: Header file for Packet (POS/Ethernet) TX 
//
/////////////////////////////////////////////////////////////////////////////////////////


#ifndef	__SPHY_MPHY4_TX_H__
#define	__SPHY_MPHY4_TX_H__

#ifdef USE_IMPORT_VAR
 
  .import_var PACKET_TX_COUNTER_BASE

#else

  /* defined dl_system.h */
#endif



#define TOTAL_TBUF_SIZE 		8192
#define TOTOL_THREADS_PER_ME		8

#define TBUF_ELE_SIZE		128
#define TBUF_ADDR_SHF		7

#define_eval TBUF_ELE_SIZE_MINUS_8	(TBUF_ELE_SIZE - 8)
#define_eval TBUF_ELE_SIZE_MINUS_16	(TBUF_ELE_SIZE - 16)

// the LSF 8 bits in TX_SEQUENCE_# are valid bits
#define TX_SEQENCE_VALID_BITS_MASK	0xFF

// definition of the threshold to wait to avoid overwriting existing mpackets
// not transmitted out of tbuf yet
#if(TX_PHY_MODE == SPHY_1_32)
#define_eval TBUF_ELE_NUM_PER_SEGMENT	(TOTAL_TBUF_SIZE / TBUF_ELE_SIZE)
#define_eval TBUF_WAIT_THRESHOLD        (TBUF_ELE_NUM_PER_SEGMENT - 1)

#endif // #if(TX_PHY_MODE == SPHY_1_32)

#if(TX_PHY_MODE == SPHY_4_8)
#define TBUF_WAIT_THRESHOLD        8
#define_eval TBUF_ELE_NUM_PER_SEGMENT	(TOTAL_TBUF_SIZE / (TBUF_ELE_SIZE * 4))
#endif //#if(TX_PHY_MODE == SPHY_4_8)

#if(TX_PHY_MODE == MPHY_4)
#define TBUF_WAIT_THRESHOLD        8
#define_eval TBUF_ELE_NUM_PER_SEGMENT	(TOTAL_TBUF_SIZE / (TBUF_ELE_SIZE * 4))
#endif //#if(TX_PHY_MODE == MPHY_4)

#if(TX_PHY_MODE == SPHY_1_32)
// definition of device bus width
#define BUS_WIDTH_IN_BYTES		16
#endif // #if(TX_PHY_MODE == SPHY_1_32)

//#define_eval TBUF_ELE_NUM	(TOTAL_TBUF_SIZE / TBUF_ELE_SIZE)

// POS_TX_UP_CTL_WD
//OK #define POS_TX_UP_CTL_WD		0x0019	; packet, single bit odd parity, 2 clock cycle

#define POS_TX_UP_CTL_WD		0x001D	; packet, single bit even parity, 2 clock cycle
// for TX_UP_CTL  0x001C and 0x0018 generated same packets
//#define POS_TX_UP_CTL_WD		0x001C	; packet, single bit even parity, 1 clock cycle
// #define POS_TX_UP_CTL_WD		0x0018	; packet, single bit odd parity, 1 clock cycle
#define ATM_TX_UP_CTL_WD		0x000C	; cell, single bit even parity, 1 clock cycle

#define MPHY_16_POS_TX_FIRST_ME		6
#define MPHY_16_POS_TX_SECOND_ME	7

// ports controlled by first POS TX ME (port 0 to 7)
#define			PORTS_CTRLED_BY_FIRST_POS_TX_ME		0x00FF

// ports controlled by second POS TX ME (port 8 to 15)
#define			PORTS_CTRLED_BY_SECOND_POS_TX_ME	0xFF00

// definition of the mask to get the next available tbuf element from what
// gotten from scratch memory by atomic operation
#if(TX_PHY_MODE == SPHY_1_32)
#define_eval TBUF_ELE_BITS_MASK ( (TOTAL_TBUF_SIZE / TBUF_ELE_SIZE) - 1)
#endif //#if(TX_PHY_MODE == SPHY_1_32)

#if(TX_PHY_MODE == SPHY_4_8)
#define_eval TBUF_ELE_BITS_MASK ( ((TOTAL_TBUF_SIZE / 4) / TBUF_ELE_SIZE) - 1)
#endif //#if(TX_PHY_MODE == SPHY_4_8)

#if(TX_PHY_MODE == MPHY_4)
#define_eval TBUF_ELE_BITS_MASK ( ((TOTAL_TBUF_SIZE / 4) / TBUF_ELE_SIZE) - 1)
#endif //#if(TX_PHY_MODE == MPHY_4)

//	Scratch for communicating between Queue Manager and POS Transmit Blocks
// bits definition in the gloabl execution status flag exe_stat_flag
// definition of the bit location for reading secondary buffer meta data in 
// phase 1, and need to be saved in phase 2 flag in exe_stat_flag GPR
#define READ_SND_BD_FLAG_BIT		0	

// definition of the bit location for some bytes leftover from previous buffer
// flag in exe_stat_flag GPR
#define LEFTOV_FROM_PB_FLAG_BIT		1	 

// definition of the bit location for some bytes leftover from previous buffer
// flag in exe_stat_flag GPR
#define SAVE_LEFTOV_TO_LM_FLAG_BIT	2	 

// definition of the bit location for reading enqueueing sop meta data in 
// phase 2, and need to be saved in phase 3 flag in exe_stat_flag GPR
#define READ_SOP_BD_FLAG_BIT		3


// definition of the bit location to identify taht the tbuf will not be
// transmitted  due to its paylo size less than 4 in exe_stat_flag GPR
#define SKIP_TRANSMIT_FLAG_BIT		4

// definition of the bit location to identify this is an EOB mpacket in 
// exe_stat_flag GPR, so this buffer needs to be freed in phase 3
#define EOB_MPKT_FLAG_BIT			5

// definition of the bit location to identify that there is no scratch queue
// read to get tx request due to queue full
#define GET_TX_REQUEST_FLAG_BIT	6

// constants definition for queue data structure at local memory
// queue status info starting at 0x800 in local memory
#define QUEUE_INFO_BASE		0x800
 
#define QUEUE_ENTRY_BASE	0

// each port occpies 8 longword
// #define QUEUE_INFO_ENRY_SHFT	5
// each port occpies 16 longword
#define QUEUE_INFO_ENRY_SHFT	6

#define PKTS_IN_QUEUE_SHFT		16

// queue entres for each port occupies 64 lonwords 
#define QUEUE_ENRY_SHFT		8	

#define TAIL_MASK	0xFF
#define TAIL_SHFT	8

#define HEAD_MASK 	0xFF

#define PORT_MASK_IN_TXR	0x0F

// each entry occupies 4 longwords
#define QUEUE_ENTRY_SIZE	0x10

// since this code is designated for one port, in order to increase the
// buffering in the egress pipeline, use all available local memory for 
// the port

#define_eval QUEUE_DEPTH	(0x800 / QUEUE_ENTRY_SIZE)
  
#define_eval QUEUE_DEPTH_MINUS_ONE	(QUEUE_DEPTH - 1)

#define_eval HEAD_INCR (QUEUE_ENTRY_SIZE)
#define_eval MAX_PLUS_ONE_QUEUE_OFFSET	(QUEUE_ENTRY_SIZE * QUEUE_DEPTH)

#define_eval MAX_MINUS_ONE_QUEUE_OFFSET	(QUEUE_ENTRY_SIZE * QUEUE_DEPTH_MINUS_ONE)

#define CONST_FF		0xFF

#define CONST_F8		0xF8

#define CONST_0F		0x0F

#define CONST_07		0x07	

#define CONST_04		0x04	



// amount to add to increase pkts_in_queue by one in queue status info
#define INCR_PKTS_IN_QUEUE	0x10000

// amount to add to increase tail_offet to next entry in queue status info
#define_eval INCR_TAILOFFSET		(QUEUE_ENTRY_SIZE <<8)

// bit location for sop flag in queue entry longword 0
#define SOP_BIT_LOC			31

// bit location read next BD flag in queue entry longword 0
#define RD_NBD_BIT_LOC		30

// mask for pkt_in_queue in queue info
#define QPKT_MASK	0x0F
// the least significant bit location of pkt_in_queue in queue info
#define QPKT_SHFT	16



// mask for freelist field
#define FREE_LIST_MASK		0xF

// the least significant bit location of freelist in meta data
#define FLIST_META_LOC		12

// definition of EOP bit in Transmit control word for POS-PHY
#define TCW0_EOP_BIT_SHFT		8

// definition of SKIP bit in Transmit control word for POS-PHY
#define TCW0_SKIP_BIT_SHFT		11

// definition of mask to get freelist id from meta data
#define FREELIST_MASK		0x000F

// definition of the base bits pattern of indir_ref used in instruction which 
// moves packet data from dram to tbuf
#define DRAM_INDIREF_BASE_LW	0x02000010	; overrid bit of Ref_Cnt field (bit
											; 25) override bit of RBUF/TBUF 
											; Byte addr field (bit 4) and must 
											; always be set


// definition of mask to get dl_buf_handle 
#define MASK_FOR_DL_BH_LW	0x03FFFFFF	
#define TRUE_DL_HDL_SHIFT	2

// definition of mask to put freelist in bit 27 to bit 24 from TXC *l$index0[3]
#define MASK_FOR_FREELIST_LW	0x0F000000
	
// since CSIX TX microblock doesn't need to refer to next neighbor, following 
// definition is dummy, just to eliminate a warning from dispatch_loop.h
#define DL_NEXT_ME	0

// definition of the default PHY TXFIFO depth
// assume PHY has 2K bytes TXFIFO and we set the threshhold to 2 times 
// TBUF_ELE_SIZE
#define DEFAULT_PHY_DEPTH	((2048 / TBUF_ELE_SIZE) - 1)

// definition of the offset of different elements in the queue entry in local
// memory
// definition of the offset of the first element about active buffer  
#define ABD_0_OFFSET		0
// definition of the offset of the second element about active buffer  		
#define ABD_1_OFFSET		1
// definition of the offset of the first element about secondary buffer 
#define SBD_0_OFFSET		2		
// definition of the offset of the first element about secondary buffer
#define SBD_1_OFFSET		3

#ifdef DYNAMIC_PPP_ENCAP
// definition of packet type of the current active buffer
// since this only used by sop mpacket, it can share with SBD_1_OFFSET,
// SBD_1_OFFSET will not be used until nexr buffer meta data read complete
#define ABD_PKT_TYPE_OFFSET		3
#endif

// definition of the least significant bit of the remained payload in queue 
// entry
#define PAYLO_RMND_LOC		16

// definition to the left shift needed on TBUF_ELE_SIZE to calculate the maximum
// bytes (max_bytes_to_move) to move from dram to tbuf for all threads work on one
// port in one ME
#if(TX_PHY_MODE == SPHY_1_32)
// max_bytes_to_move = TBUF_ELE_SIZE * 8 
#define ME_THREAD_NUM_SHFT	3
#endif //#if(TX_PHY_MODE == SPHY_1_32)

#if(TX_PHY_MODE == SPHY_4_8)
// max_bytes_to_move = TBUF_ELE_SIZE * 2 
#define ME_THREAD_NUM_SHFT	1
#endif  //#if(TX_PHY_MODE == SPHY_4_8) 

#if(TX_PHY_MODE == MPHY_4)
// max_bytes_to_move = TBUF_ELE_SIZE * 2 
#define ME_THREAD_NUM_SHFT	1
#endif  //#if(TX_PHY_MODE == MPHY_4) 

#define PKTS_IN_Q_MASK	0xFF

/////////////////////////////////////////////////////////////////////////////
// Follow definitions are used for global counters
// There are total 16 ports.  Each port has a counter
// data.  The counter data structure contains 3 long words
//	PKT_TRANSMITTED_CNT
//	PKT_DROPPED_CNT
//	BYTE_TRANSMITTED_CNT
//	RESERVED_CNT
//  The index of the counter data is defiend by POS_TX_COUNTER_BASE+PORT_OFFSET
//	PORT_OFFSET	= port number << 4 
//	Following are the counter index definitions

#ifdef COUNTERS
#define		PKT_TRANSMITTED_CNT_IDX		0x00
#define		PKT_DROPPED_CNT_IDX			0x04
#define		BYTE_TRANSMITTED_CNT_IDX	0x08
#define 	BYTES_DROPPED_CNT_IDX		0x0C
#define     PORT_MASK_IN_TCW            0x0F
#endif

// definition of least significant bit location of fields of Transmit Control Word
// definition of least significant bit location of payload length field
#define	TCW0_PAYLOAD_LEN_LSB_LOC	24
// definition of least significant bit location of payload offset field
#define TCW0_PAYLOAD_OFFSET_LSB_LOC	13
// definition of least significant bit location of prepend length field 
#define TCWO_PREPEND_LEN_LSB_LOC		16
// definition of least significant bit location of prepend offest field  		
#define TCWO_PREPEND_OFFSET_LSB_LOC		21

// definition of the mask to get payload offset (3 bits)
#define TCW0_PAYLOAD_OFFSET_MASK		0x07

// debug
#ifdef DEBUG_TX_PENDING_LOCKED
#define SRAM_WRITE_1_SIG	11
#define FIRST_ME_TBUF_NO_SRAM_BASE		0x100000
#define SECOND_ME_TBUF_NO_SRAM_BASE		0x200000
#endif //#ifdef DEBUG_TX_PENDING_LOCKED


// base for global info for port 0, each port has 32 longwords
#define PORT_GLOBAL_INFO_BASE	2048

// each port has 32 longwords in local memory for globals
#define PORT_GLOBAL_OFFSET_SHF		7

// for 4 ports case with 128 tbuf element size, tbuf divided into 4 equal
// segements, each segment owns 16 tbuf elements, so each segment owns 
// 128 bytes (8x16) transmit control valid word address space
#define TCW_VD_BASE_SHF		7

// for 4 ports case with 128 tbuf element size, tbuf divided into 4 equal
// segements, each segment owns 16 tbuf elements, so each segment owns 
// 2048 bytes (128x16) tbuf address space
#define TBUF_BASE_SHF		2048

// MSF_Tx_Sequence_{0, 1,2,3} address is 060, 0x64, 0x68, 0x6c
#define TX_SEQUENCE_ADDR_SHF	2
// setting for MSF_Tx_Control for SPHY_1X32 mode
#if IS_IXPTYPE(__IXP28XX)
#define PACKET_TX_SPHY_1X32_CTL_LW	0x20000D04
#define TX_CNTRL_ADDR_SHFT	4
#else // #ifdef IXP2800
#define PACKET_TX_SPHY_1X32_CTL_LW	0x10000004	;0001 0000 0000 0000 Tx enable, UTOPIA/POS, SPHY_1x32, Simplex
										;0000 0000 0000 0100 128 bytes tbuf size
#define TX_CNTRL_ADDR_SHFT	3
#endif // #ifdef IXP2800

// setting for MSF_Tx_Control for SPHY_4x8 mode
#if IS_IXPTYPE(__IXP28XX)
#define PACKET_TX_SPHY_4X8_CTL_LW	0x20000454	//??
//#define TX_CNTRL_ADDR_SHFT	4				//??
#else // #ifdef IXP2800
#define PACKET_TX_SPHY_4X8_CTL_LW	0xFF200004	;0001 0000 0000 0000 Tx enable, UTOPIA/POS, SPHY_4x8, Simplex
											;0000 0000 0000 0100 128 bytes tbuf size
#define TX_CNTRL_ADDR_SHFT	3
#endif // #ifdef IXP2800

// setting for MSF_Tx_Control for MPHY4 mode
#if IS_IXPTYPE(__IXP28XX)
#define PACKET_TX_MPHY4_CTL_LW	0x20000454	//??
//#define TX_CNTRL_ADDR_SHFT	4				//??
#else // #ifdef IXP2800
#define PACKET_TX_MPHY4_CTL_LW	0xFF080004	;0001 0000 0000 0000 Tx enable, UTOPIA/POS, MPHY4, Simplex
											;0000 0000 0000 0100 128 bytes tbuf size
#define TX_CNTRL_ADDR_SHFT	3
#endif // #ifdef IXP2800


/////////////////////////////////////////////////////////////////////////////////////
// 
// Globals stored in local memory 
//
/////////////////////////////////////////////////////////////////////////////////////

//	We will use local memory to store a few globals. These globals will be accessed 
//  *l$index1[], and local memory index 1 is initialized to right value during 
//  initialization based on the operation mode (SPHY(SPHY_1X32), SPHY4(SPHY_4X8), or 
//  MPHY4 (MPHY_4))
#define GLOBAL_CTW_VD_BASE_INDEX			0	; base to compute 
												; TBUF_ELEMENT_CTRL_V# address
												; for the specific port
#define GLOBAL_TBUF_BASE_INDEX				1	; base to compute tbuf address
												; for the specific port
#define GLOBAL_QUEUE_INFO_BASE_INDEX		2	; starting address of queue info 
												; for the specfic port
#define GLOBAL_HEAD_OFFSET_INDEX			3	; queue head for the specific port
												; (bytes offset within queue info)
#define GLOBAL_TAIL_OFFSET_INDEX			4	; queue tail for the specific port
												; (bytes offset within queue info)

#define	GLOBAL_NOT_LOCK_SNDB_RD_FLAG_INDEX	5	; if bit 0 set mans this specific port is 
 												; not prohibited to transmit due to
												; secondary buffer info not ready
#define	GLOBAL_NOT_LOCK_LEFTOVER_FLAG_INDEX	6	; if bit 0 set mans this specific port is 
 												; not prohibited to transmit due to
												; leftover bytes not ready in local memory#d


#define	GLOBAL_AVAIL_TBUF_ELEMENT_INDEX		7	; next available TBUF element for 
												; the specific port
#define GLOBAL_PKTS_IN_QUEUE_INDEX			8	; packets in queue for the specific
												; port
#define GLOBAL_PKTS_TXED_INDEX				9	; packets transmitted for this 
												; specific port
												; (bytes offset within queue info)
#define	GLOBAL_LEFTOVER_FLAG_LEN_INDEX		10	; MSB is the flag bit, if set to 1, 
												; means that there are some leftover
												; bytes from previous buffer to be 
												; transmitted in the beginning of 
												; this mpkt, least significant byte 
												; is the leftover length in bytes
												; specific port 	
#define	GLOBAL_SBD_NEXT_BD_INDEX			11	; next buffer descriptor for secondary 
												; buffer for the specific port
#define	GLOBAL_LEFTOVER_LW0_INDEX			12	; first longword of the leftover payload
 												; (max. 15 bytes) to next mpkt in next
												; buffer due to SPI-4 non-EOP mpkt size 
												; limitation
#define	GLOBAL_LEFTOVER_LW1_INDEX			13	; first longword of the leftover payload
 												; (max. 15 bytes) to next mpkt in next
												; buffer due to SPI-4 non-EOP mpkt size 
												; limitation
#define	GLOBAL_LEFTOVER_LW2_INDEX			14	; first longword of the leftover payload
 												; (max. 15 bytes) to next mpkt in next
												; buffer due to SPI-4 non-EOP mpkt size 
												; limitation
#define	GLOBAL_LEFTOVER_LW3_INDEX			15	; first longword of the leftover payload
 												; (max. 15 bytes) to next mpkt in next
												; buffer due to SPI-4 non-EOP mpkt size 
												; limitation
// base for queue info for port 0, each port has 64 longwords
#define PORT_QUEUE_INFO_BASE	0		; starting address in local memory for queue info

#define PORT_0 	0
#define PORT_1	1
#define PORT_2 	2
#define PORT_3	3

#if(TX_PHY_MODE == SPHY_1_32)
#define PORT_HANDLED_BY_THREAD_0	PORT_0
#define PORT_HANDLED_BY_THREAD_1	PORT_0
#define PORT_HANDLED_BY_THREAD_2	PORT_0
#define PORT_HANDLED_BY_THREAD_3	PORT_0
#define PORT_HANDLED_BY_THREAD_4	PORT_0
#define PORT_HANDLED_BY_THREAD_5	PORT_0
#define PORT_HANDLED_BY_THREAD_6	PORT_0
#define PORT_HANDLED_BY_THREAD_7	PORT_0
#endif //#if(TX_PHY_MODE == SPHY_1_32)

#if(TX_PHY_MODE == SPHY_4_8)
#define PORT_HANDLED_BY_THREAD_0	PORT_0
#define PORT_HANDLED_BY_THREAD_1	PORT_0
#define PORT_HANDLED_BY_THREAD_2	PORT_1
#define PORT_HANDLED_BY_THREAD_3	PORT_1
#define PORT_HANDLED_BY_THREAD_4	PORT_2
#define PORT_HANDLED_BY_THREAD_5	PORT_2
#define PORT_HANDLED_BY_THREAD_6	PORT_3
#define PORT_HANDLED_BY_THREAD_7	PORT_3
#endif //#if(TX_PHY_MODE == SPHY_4_8)

#if(TX_PHY_MODE == MPHY_4)
#define PORT_HANDLED_BY_THREAD_0	PORT_0
#define PORT_HANDLED_BY_THREAD_1	PORT_0
#define PORT_HANDLED_BY_THREAD_2	PORT_1
#define PORT_HANDLED_BY_THREAD_3	PORT_1
#define PORT_HANDLED_BY_THREAD_4	PORT_2
#define PORT_HANDLED_BY_THREAD_5	PORT_2
#define PORT_HANDLED_BY_THREAD_6	PORT_3
#define PORT_HANDLED_BY_THREAD_7	PORT_3
#endif //#if(TX_PHY_MODE == MPHY_4)

#define TCW0_SOP_EOP_BASE_W0 0x0300	


#define_eval TOTAOL_QUEUE_ENTRY_SIZE_PER_PORT	(QUEUE_ENTRY_SIZE * QUEUE_DEPTH)

#if(TX_PHY_MODE == SPHY_1_32)
#define DEVICE_BUS_WIDTH_RESTRICTION_MASK	0xF0
#endif //#if(TX_PHY_MODE == SPHY_1_32)


// definition of BASE for TCW0 for sop_only mpacket
//#define_eval TCW0_SOP_ONLY_BASE_LW	(0x00000200 | (PPPH_LEN <<16))
// SOP bit location in TCW0
#define_eval TCW0_SOP_BIT_LOC	9

#define_eval TCW0_EOP_BIT_LOC	8


#define PE_SPHY_TX_FLAG_CLEAR	0

#define PE_SPHY_TX_FLAG_SET		1

// definition of SKIP_GET_TXR_THRESHOLD: when pkts_in_queue reaches this 
// value, skip reading scratch ring to get tx request to prevent overriding
// queue info which is not processed yet
#if(TX_PHY_MODE == SPHY_1_32)
#define_eval SKIP_GET_TXR_THRESHOLD	(QUEUE_DEPTH - 8)
#endif //#if(TX_PHY_MODE == SPHY_1_32)

#if(TX_PHY_MODE == SPHY_4_8)
#define_eval SKIP_GET_TXR_THRESHOLD	(QUEUE_DEPTH - 2)
#endif //#if(TX_PHY_MODE == SPHY_4_8)

#if(TX_PHY_MODE == MPHY_4)
#define_eval SKIP_GET_TXR_THRESHOLD	(QUEUE_DEPTH - 2)
#endif //#if(TX_PHY_MODE == MPHY_4)

#ifndef ADD_L2_HEADER

	#ifdef SPI4_PREPEND

		// sop meta data to read in longword

		#define META_DATA_LW_TO_READ		3

		// payload len = 0 (will be filled later), prepend offset = 0, 
		// prepend length = 16
		#define TCW0_SOP_EOP_BASE_W1 0x0010

		//TCW0 high word shift
		#define TCWO_W1_SHF	16

		// bytes occupied in tbuf for l2 header due to dram[tbuf_wr] 8 bytes limit

		#define BYTES_FOR_L2_HEADER			16
		#define	SPI4_PREPEND_HEADER_SIZE	16
		#define_eval SOP_EOP_MAX_PAYLOAD	(TBUF_ELE_SIZE - SPI4_PREPEND_HEADER_SIZE)

	#else	// not SPI4_PREPEND

		// sop meta data to read in longword

		#define META_DATA_LW_TO_READ		3

		#define TCW0_SOP_EOP_BASE_W1 0x0000

	#endif	// SPI4_PREPEND

#else // ADD_L2_HEADER

#define L2_HEADER_ALREADY_EXIST_ID	0xFFFF

#define	L2_HEADER_IN_PAYLOAD		1

#define L2_HEADER_NOT_IN_PAYLOAD	0

#ifdef POS_TX

// sop meta data to read in longword

#define META_DATA_LW_TO_READ		3

#ifndef ADD_PPP_ADDR_AND_CNTRL
#ifndef DYNAMIC_PPP_ENCAP
#define PPP_IP_PROTOCOL		0x00210000
#else	// DYNAMIC_PPP_ENCAP
#define PPP_IP_PROTOCOL		0x00000000
#endif	// #ifndef DYNAMIC_PPP_ENCAP
#else	// ADD_PPP_ADDR_AND_CNTRL
#ifndef DYNAMIC_PPP_ENCAP
#define PPP_IP_PROTOCOL		0xFF030021
#else	// DYNAMIC_PPP_ENCAP
#define PPP_IP_PROTOCOL		0xFF030000
#endif	// #ifndef DYNAMIC_PPP_ENCAP
#endif	// #ifndef ADD_PPP_ADDR_AND_CNTRL

#ifndef ADD_PPP_ADDR_AND_CNTRL
// bits to shift to get PPP header for IP
#define IP_PROTOCOL_SHFT	16
// payload len = 0 (will be filled later), prepend offset = 0, 
// prepend length = 2
#define TCW0_SOP_EOP_BASE_W1 0x0002
#define PPP_HEADER_SIZE		2
#else	// ADD_PPP_ADDR_AND_CNTRL
// bits to shift to get PPP header for IP
#define IP_PROTOCOL_SHFT	0
// payload len = 0 (will be filled later), prepend offset = 0, 
// prepend length = 4
#define TCW0_SOP_EOP_BASE_W1 0x0004
#define PPP_HEADER_SIZE		4
#endif	// #ifndef ADD_PPP_ADDR_AND_CNTRL


//TCW0 high word shift
#define TCWO_W1_SHF	16


// bytes occupied in tbuf for l2 header due to dram[tbuf_wr] 8 bytes limit

#define BYTES_FOR_L2_HEADER			8
#endif	// #ifdef POS_TX

#ifdef ETHERNET_TX	

// sop meta data to read in longword

#define META_DATA_LW_TO_READ		5

#define L2_TABLE_ENTRY_SHFT			4

#define L2_TABLE_ENTRY_SIZE_IN_LW	4

// payload len = 0 (will be filled later), prepend offset = 0, 
// prepend length = 14

#define TCW0_SOP_EOP_BASE_W1 0x002e

//TCW0 high word shift
#define TCWO_W1_SHF	16

#define	ETHERNET_HEADER_SIZE	14

// bytes occupied in tbuf for l2 header due to dram[tbuf_wr] 8 bytes limit

#define BYTES_FOR_L2_HEADER			16
#endif	// #ifdef ETHERNET_TX

#define_eval SOP_EOP_MAX_PAYLOAD	(TBUF_ELE_SIZE - BYTES_FOR_L2_HEADER)

#endif //ADD_L2_HEADER

// definition of bits pattern for flags (bit 31: =0 means sop, bit 30: =0 means
// eop, bit 29: =1 means l2 table entry passed from frome first TX ME to second 
// TX ME through NN ring

#define FLAG_BITS_FROM_MSB		7		

#if IS_IXPTYPE(__IXP28XX)
	// Transmit parameters
	#define 			TBUF_SIZE 						8192
	#define_eval 		NUM_TBUF_ELEMS		 			(TBUF_SIZE/TBUF_ELE_SIZE)
	#define_eval 		NUM_TBUF_ELEMS_IN_PART0			NUM_TBUF_ELEMS
	#define_eval		TBUF_MAX_VALID_ELEMENTS			(NUM_TBUF_ELEMS_IN_PART0 - 1)

	#define				TX_SEQUENCE_COUNTER_NUM_BITS	8
	#define_eval		TX_SEQUENCE_MAX_CNT				(1<<TX_SEQUENCE_COUNTER_NUM_BITS)
	#define_eval		TX_SEQUENCE_WRAP_MASK			(TX_SEQUENCE_MAX_CNT-1)

// port TX FIFO status related definition

#ifdef IXP_SIMULATION

// the value set in the TX_CALENDAR_LENGTH register need to be same as the 
// number of port set in device configuration for simulation, it is 1 for this
// single port configuration

#define MEDIA_TX_FSTAT_REPEATS					1

#else 	// hardware

// the number of FIFO status repeats on the FIFO channel default setting 
// for IXF18101 (media chip in Cross Island board of Deer Island system)  

#define IXF18101_DEFAULT_TX_FSTAT_REPEATS		16

// the value set in the TX_CALENDAR_LENGTH register need to be same as the 
// number of FIFO status repeats on the FIFO status chennel in media chip
// use IXF18101 defaul setting as default

#define_eval MEDIA_TX_FSTAT_REPEATS		IXF18101_DEFAULT_TX_FSTAT_REPEATS

#endif	// #ifdef IXP_SIMULATION

// following SIF_TX_MAXBURST_1 and SIF_TX_MAXBURST_2 are  set in media chip
// by XSCALE initialization code

#define IXF18101_SIF_TOTAL_TX_FIFO_SIZE		32768

#define IXF18101_DEFAULT_SIF_TX_MAXBURST_1	512

#define IXF18101_DEFAULT_SIF_TX_MAXBURST_2	256

#define_eval SIF_TOTAL_TX_FIFO_SIZE	IXF18101_SIF_TOTAL_TX_FIFO_SIZE

#define_eval SIF_TX_MAXBURST_1	IXF18101_DEFAULT_SIF_TX_MAXBURST_1

#define_eval SIF_TX_MAXBURST_2	IXF18101_DEFAULT_SIF_TX_MAXBURST_2

#define_eval INITIAL_FC_CREDIT		(SIF_TOTAL_TX_FIFO_SIZE / TBUF_ELE_SIZE)

#define_eval STARVING_FC_CREDIT 	((SIF_TX_MAXBURST_1 * 16) / TBUF_ELE_SIZE)							

#define_eval HUNGRY_FC_CREDIT		((SIF_TX_MAXBURST_2 * 16) / TBUF_ELE_SIZE)		

#define_eval STARVING_FC_CREDIT_MINUS_9	(STARVING_FC_CREDIT - 9)

#define_eval STARVING_FC_CREDIT_MINUS_8	(STARVING_FC_CREDIT - 8)

#define_eval STARVING_FC_CREDIT_MINUS_1	(STARVING_FC_CREDIT - 1)

#define_eval HUNGRY_FC_CREDIT_MINUS_9		(HUNGRY_FC_CREDIT - 9)	

#define_eval HUNGRY_FC_CREDIT_MINUS_8		(HUNGRY_FC_CREDIT - 8)	

#define_eval HUNGRY_FC_CREDIT_MINUS_1		(HUNGRY_FC_CREDIT - 1)	

#define STRAVING_STATUS		0

#define HUNGRY_STATUS		1

#endif	// #if IS_IXPTYPE(__IXP28XX)

// bit location to notify next ME to free buffer

#define FREE_BUFFER_FLAG_BIT		30

///////////////////////////////////////////////////////////////////////////////
//
//	Short cuts for Reassembly Context in local memory
//
///////////////////////////////////////////////////////////////////////////////

#define lm1_leftover_lw0			*l$index1[GLOBAL_LEFTOVER_LW0_INDEX]
#define lm1_leftover_lw1			*l$index1[GLOBAL_LEFTOVER_LW1_INDEX]
#define lm1_leftover_lw2			*l$index1[GLOBAL_LEFTOVER_LW2_INDEX]
#define lm1_leftover_lw3			*l$index1[GLOBAL_LEFTOVER_LW3_INDEX]

#define GLOBAL_MPKTS_TXED_INDEX				9	; mpkts transmitted in TX ME 3

#define META_DATA_PTR_OFFSET		0
#define TCW0_DATA_OFFSET			1
#define FLAG_TBUF_ELEMENT_OFFSET	2

#endif	//	__SPHY_MPHY4_TX_H__
