////////////////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
////////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: sphy_mphy4_tx3.uc
//
//      Purpose: 3rd TX ME in Packet transmitter for Egress IXP2800
//
///////////////////////////////////////////////////////////////////////////////


#ifndef	__SPHY_MPHY4_TX3_UC__
#define	__SPHY_MPHY4_TX3_UC__

// This is 3rd ME in Packet TX microblock
// High-level view of what all MEs do is as follows
/*
1st ME:
	
while (1)
	{
		read request (packet p1) from NN ring from QM.
		Issue SRAM read for metadata of the p1.
		Find first request in LM queue (packet p2) and read the request
		Determine whether the packet is SOP, EOP, none of these or a min packet and call corresponding macro.
		Sends scheduler feedback about packet transmit when reaches EOP
		Calculate DRAM address to read the payload
		Send the request to next ME using NN to transmit a mpacket of p2

		Wait for metadata read signal and int thd signal
		
		Send int thd signal
		Store the metadata read to the tail of LM queue
	}

2nd ME (in two MEs configuration):

while (1)
	{
		for (i=0; I < max_phase; i++)
			{
				phase# i:
					issue tbuf validate of mpacket read in previous iteration and generate signal tbuf_validate_i
					wait for signals tbuf_validate_i, dram_(i+1), prepend_(i+1)
					read a mpacket transmit request from NN ring
					grab a tbuf for the mapacket
					issue dram to tbuf transfer request and generate signal dram_i
					perform flow control function based on port TXFIFO status optionally
					add prepend (l2 header, SPI-4 prepend or leftover) and generate signal prepend_i
					branch to phase_(i+1);
			}
			// here (i+1) means (i+1) mod max_phase;

	}

		
2nd ME (in three MEs configuration):

while (1)
	{
		for (i=0; I < max_phase; i++)
			{
				phase# i:
					issue request for mpacket read in previous iteration to next ME through NN ring
					wait for signals dram_(i+1), prepend_(i+1)
					read a mpacket transmit request from NN ring
					grab a tbuf for the mapacket					
					issue dram to tbuf transfer request and generate signal	dram_i
					add prepend (l2 header, SPI-4 prepend or leftover) and generate signal prepend_i
					branch to phase_(i+1);
			}
			// here (i+1) means (i+1) mod max_phase;

	}


3rd ME (only in three MEs configuration):

while (1)
	{
		for (i=1; I < 3; i++)
			{
				phase# i:
					read mpacket request from NN ring 
					perform flow control function based on port TXFIFO status optionally
					issue tbuf validate of mpacket just read from NN ring
					branch to phase_(i+1);
			}

	}

The interface between 1st ME and 2nd ME is

lw0: meta_data header ptr (used to free a buffer, if buffer is transmitted completeely)
lw1: 0-29 	offset of payload to be transmitted
30	bit clear means the mpacket is eop,
31	bit clear means the mpacket is sop
lw2: 	payload size to be transmitted
 
In case of non-sop mpacket, we have 3 or 6 more LWs on the NN ring, 
The first three more LWs are:

lw3:	leftover_lw0, first longword of leftover from the previous buffer has 
		to be prepended to the current buffer
lw4:	exe_stat_flag  includes information about various condition flags
lw5:	partially created transmit control word 

If LEFTOV_FROM_PB_FLAG_BIT is set in exe_stat_flag, there will be three more 
LWs:

lw6:	leftover_lw1, second longword of leftover from the previous buffer has 
		to be prepended to the current buffer
lw7:	leftover_lw2, third longword of leftover from the previous buffer has 
		to be prepended to the current buffer
lw8:	leftover_lw3, forth longword of leftover from the previous buffer has 
		to be prepended to the current buffer

The interface between 2nd ME and 3rd ME is

lw0: meta_data header ptr (used to free a buffer, if buffer is transmitted completeely)
lw1: transmit control word 0 for this mpacket
lw2: bit 30 - EOB flag: when set, this buffer is completely transmitted, should be freed
	 bit 15 to 0: tbuf element number assigned to this mpacket in 2nd ME

*/


// Following is the optional assembling option for this TX ME:

// CHECK_PORT_TX_FIFO_STATUS: enable this TX ME to perform flow control based
// on meia port TX FIFO status to prevent overwritting of port TX FIFO

// include stdmac.uc in IXPblocks Portable library

#include <stdmac.uc>

// include local memory macros in IXPblocks 

#include <localmem.uc>

// global definition of common constants which are shared by all microblocks

#include "dl_system.h"

// definition of hardware register address 

#include <hardware.h>

// definition of PHY operation mode and misc.

#include <definitions.h>

// definition of constants used

#include "sphy_mphy4_tx.h"

// definition of utilization macros used

#include "sphy_mphy4_tx_util3.uc"

// initialization code for POS/Ethernet TX

#include "sphy_mphy4_tx_init3.uc"

// definition of common utilization macros used

#include "sphy_mphy4_tx_common_util.uc"

// dispatch loop macros

#include "dispatch_loop.uc"


#ifdef CHECK_PORT_TX_FIFO_STATUS
	.reg @fc_credit
	.reg addr_port_stat
	.reg $port_stat_phase_1
	.reg $port_stat_phase_2
	.sig volatile sig_msf_read
#endif  // #ifdef CHECK_PORT_TX_FIFO_STATUS

// following context relative GPR variables are preassigned in initialization 
// to save instrction cycles in POS TX process
.reg sig_next_context_gpr		; value to write to SAME_ME_SIGNAL csr to wake up
								; next thread
.reg port_entry_turnaround		; packet queue head/tail offset value to reset 
								; head/tail offset
.reg $txed_port					; xfer register to keep packet txed to notify
								; egress scheduler by reflect write	
#ifdef COUNTERS
.reg @cntr_base					; start address of Packet TX counters in sram
								; initialized to PACKET_TCX_COUNTER_BASE
.reg $byte_cnt					; sram xfer register to keep value to add to 
								; packets transmitted counter in sram		
#endif //#ifdef	COUNTERS

#ifdef _DEBUG_COUNTERS_
.reg @tbufs_transmitted
.reg @buffers_freed
.reg @default_buffers_freed
#ifdef CHECK_PORT_TX_FIFO_STATUS
.reg @is_satisfied
.reg @is_hungry
.reg @is_starving
#endif  // #ifdef CHECK_PORT_TX_FIFO_STATUS
#endif	// #ifdef _DEBUG_COUNTERS_

#ifdef PORT_STATUS_COUNTER
	.reg @port_not_ready_numb
#endif	// #ifdef PORT_STATUS_COUNTER

#ifdef UNIT_TEST
.reg @buf_freed			; buffer freed for debuging
#endif //UNIT_TEST

///////////////////////////////////////////////////////////////////////////////

//	Global Signals
// following three signals need to be declared as volatile
.sig volatile sig_next_context		; signal to wakeup next thread, declared as
									; volatile to eliminate assembling warning

.sig volatile common_init_complete_sig_num		; this is the inter-ME signal 
										; generated by the ME whcih initializes
										; common resources, such as scratch ring,
										; POS TX MEs will wait for this signal at
										; end of their initialization to make 
										; sure all common resource are available 

//	Manual signal allocation
.addr common_init_complete_sig_num	ME_INIT_SIGNAL	; manual signal allocation,
													; ME_INIT_SIGNAL defined as
													; in dl_system.h  

#ifdef COUNTERS
.sig sig_counter_update									; signal for update counter
#endif //#ifdef COUNTERS		



///////////////////////////////////////////////////////////////////////////////
// sphy_mphy4_tx3()
//
// Description:		Packet (POS/Ethernet) transmission
//
// Outputs:					
//					None
//
// Inputs:  
//					None
//
// Constants:
//					None 
//
//	Size:
//		 			?? instructions for worst case
//
///////////////////////////////////////////////////////////////////////////////

#macro sphy_mphy4_tx3()

.begin 

	.reg	meta_data_ptr_p1
	.reg	meta_data_ptr_p2
	.reg	volatile addr_of_tx_control
	.reg 	volatile eob_flag_tbuf_element
	//.reg	volatile $w0	$w1
	.reg	$w0	$w1
	.xfer_order $w0 $w1
	.sig volatile sig_tbuf_validate

	alu[$w1, --, b, 0]		; initialize w1 

phase_1#:

	.begin
	// beginning of phase 1	
	
	local_csr_wr[SAME_ME_SIGNAL, sig_next_context_gpr]
	
	#ifdef CHECK_PORT_TX_FIFO_STATUS
	
	// get $port_stat_phase_1 for phase_2

	msf[read, $port_stat_phase_1, addr_port_stat, 0, 1], sig_done[sig_msf_read]
	#endif  // #ifdef CHECK_PORT_TX_FIFO_STATUS

	alu[--, *l$index1[GLOBAL_PKTS_IN_QUEUE_INDEX], -, 0] 
	ble[no_mpkt_to_validate_phase_1#]

	_sphy_mphy4_tx_get_mpkt_from_lm(meta_data_ptr_p1, $w0, \
				eob_flag_tbuf_element, addr_of_tx_control)	; (16)
	
	_sphy_mphy4_tx_set_lmindex0_to_queue_tail()		; (4)

	// make sure port ready to prevent media port TX FIFO overwritting

	#ifdef CHECK_PORT_TX_FIFO_STATUS
	
	// check whether TX FIFO of the port ready to receive another
	// mpacket

	alu[@fc_credit, @fc_credit, -, 1]
	ble[port_tx_fifo_full_phase_1#]
	port_tx_fifo_check_done_phase_1#:
	#endif  // #ifdef CHECK_PORT_TX_FIFO_STATUS

	// write transmit contrl word (TCW) to TBUF_ELEMENT_CTRL_V_# corresponding
	// the tbuf_element to initiate transmission

	msf[write, $w0, addr_of_tx_control, 0, 2], sig_done[sig_tbuf_validate]
	_sphy_mphy4_tx_debug_incr_counter(@tbufs_transmitted)

	br_bclr[eob_flag_tbuf_element, FREE_BUFFER_FLAG_BIT, buffer_free_done_phase_1#]

	_sphy_mphy4_tx_free_buffer(meta_data_ptr_p1, buf_free_phase_1#)	; free buffer
	_sphy_mphy4_tx_debug_incr_counter(@buffers_freed)

	buffer_free_done_phase_1#:	

	br_inp_state[nn_empty, read_new_request_done_phase_1#]

	// get mpacket from nn ring and save into local memory

	_sphy_mphy4_tx_get_mpkt_from_nn_ring()			; (10)		
	read_new_request_done_phase_1#:

	#ifndef CHECK_PORT_TX_FIFO_STATUS
	ctx_arb[sig_tbuf_validate, sig_next_context]
	#else	// CHECK_PORT_TX_FIFO_STATUS
	ctx_arb[sig_msf_read, sig_tbuf_validate, sig_next_context]
	#endif  // #ifndef CHECK_PORT_TX_FIFO_STATUS

	.io_completed sig_tbuf_validate
	.io_completed sig_next_context
	#ifdef CHECK_PORT_TX_FIFO_STATUS
	.io_completed sig_msf_read
	#endif  // #ifdef CHECK_PORT_TX_FIFO_STATUS

	.end	// 	addr_of_tx_control eob_flag_tbuf_element

phase_2#:

	.begin
	//.reg	addr_of_tx_control
	//.reg 	eob_flag_tbuf_element
	//.set addr_of_tx_control	
	//.set $w0
	//.set eob_flag_tbuf_element
	
	// beginning of phase 2	
	
	local_csr_wr[SAME_ME_SIGNAL, sig_next_context_gpr]
	
	#ifdef CHECK_PORT_TX_FIFO_STATUS

	// get $port_stat_phase_2 for phase_1

	msf[read, $port_stat_phase_2, addr_port_stat, 0, 1], sig_done[sig_msf_read]
	#endif  // #ifdef CHECK_PORT_TX_FIFO_STATUS
	
	alu[--, *l$index1[GLOBAL_PKTS_IN_QUEUE_INDEX], -, 0] 
	ble[no_mpkt_to_validate_phase_2#]

	_sphy_mphy4_tx_get_mpkt_from_lm(meta_data_ptr_p2, $w0, eob_flag_tbuf_element, \
													addr_of_tx_control)	; (16)


	_sphy_mphy4_tx_set_lmindex0_to_queue_tail()		; (4)
	
	// make sure port ready to prevent media port TX FIFO overwritting

	#ifdef CHECK_PORT_TX_FIFO_STATUS
	
	// check whether TX FIFO of the port ready to receive another
	// mpacket

	alu[@fc_credit, @fc_credit, -, 1]
	ble[port_tx_fifo_full_phase_2#]
	port_tx_fifo_check_done_phase_2#:
	#endif  // #ifdef CHECK_PORT_TX_FIFO_STATUS

	// write transmit contrl word (TCW) to TBUF_ELEMENT_CTRL_V_# corresponding
	// the tbuf_element to initiate transmission

	msf[write, $w0, addr_of_tx_control, 0, 2], sig_done[sig_tbuf_validate]
	_sphy_mphy4_tx_debug_incr_counter(@tbufs_transmitted)
	
	br_bclr[eob_flag_tbuf_element, FREE_BUFFER_FLAG_BIT, buffer_free_done_phase_2#]

	_sphy_mphy4_tx_free_buffer(meta_data_ptr_p2, buf_free_phase_2#)	; free buffer
	_sphy_mphy4_tx_debug_incr_counter(@buffers_freed)

	buffer_free_done_phase_2#:	

	br_inp_state[nn_empty, read_new_request_done_phase_2#]

	// get mpacket from nn ring and save into local memory

	_sphy_mphy4_tx_get_mpkt_from_nn_ring()			; (10)	

	read_new_request_done_phase_2#:

	#ifndef CHECK_PORT_TX_FIFO_STATUS
	ctx_arb[sig_tbuf_validate, sig_next_context]
	#else	// CHECK_PORT_TX_FIFO_STATUS
	ctx_arb[sig_msf_read, sig_tbuf_validate, sig_next_context]
	#endif  // #ifdef CHECK_PORT_TX_FIFO_STATUS

	.io_completed sig_tbuf_validate
	.io_completed sig_next_context
	#ifdef CHECK_PORT_TX_FIFO_STATUS
	.io_completed sig_msf_read
	#endif  // #ifdef CHECK_PORT_TX_FIFO_STATUS

	br[phase_1#]

	.end	// 	addr_of_tx_control eob_flag_tbuf_element

	no_mpkt_to_validate_phase_1#:

	_sphy_mphy4_tx_set_lmindex0_to_queue_tail()

	#ifdef CHECK_PORT_TX_FIFO_STATUS
	_sphy_mphy4_tx_update_fc_credit_for_no_tbuf_validation(@fc_credit,
							$port_stat_phase_2)	
	#endif  // #ifdef CHECK_PORT_TX_FIFO_STATUS

	br_inp_state[nn_empty, no_validation_read_new_request_done_phase_1#]

	// get mpacket from nn ring and save into local memory

	_sphy_mphy4_tx_get_mpkt_from_nn_ring()			; (10)		
	no_validation_read_new_request_done_phase_1#:

	#ifndef CHECK_PORT_TX_FIFO_STATUS
	ctx_arb[sig_next_context]
	#else	// CHECK_PORT_TX_FIFO_STATUS
	ctx_arb[sig_msf_read, sig_next_context]
	#endif  // #ifdef CHECK_PORT_TX_FIFO_STATUS

	.io_completed sig_tbuf_validate
	.io_completed sig_next_context
	#ifdef CHECK_PORT_TX_FIFO_STATUS
	.io_completed sig_msf_read
	#endif  // #ifdef CHECK_PORT_TX_FIFO_STATUS
	br[phase_2#]


	// free buffer by dl_buf_free

	buf_free_phase_1#:
	dl_buf_free(meta_data_ptr_p1, BUF_FREE_LIST0)
	_sphy_mphy4_tx_debug_incr_counter(@default_buffers_freed)
	br[buffer_free_done_phase_1#]


	no_mpkt_to_validate_phase_2#:

	_sphy_mphy4_tx_set_lmindex0_to_queue_tail()

	#ifdef CHECK_PORT_TX_FIFO_STATUS
	_sphy_mphy4_tx_update_fc_credit_for_no_tbuf_validation(@fc_credit,
							$port_stat_phase_1)	
	#endif  // #ifdef CHECK_PORT_TX_FIFO_STATUS

	br_inp_state[nn_empty, no_validation_read_new_request_done_phase_2#]

	// get mpacket from nn ring and save into local memory

	_sphy_mphy4_tx_get_mpkt_from_nn_ring()			; (10)		
	no_validation_read_new_request_done_phase_2#:

	#ifndef CHECK_PORT_TX_FIFO_STATUS
	ctx_arb[sig_next_context]
	#else	// CHECK_PORT_TX_FIFO_STATUS
	ctx_arb[sig_msf_read, sig_next_context]
	#endif  // #ifdef CHECK_PORT_TX_FIFO_STATUS

	.io_completed sig_tbuf_validate
	.io_completed sig_next_context
	#ifdef CHECK_PORT_TX_FIFO_STATUS
	.io_completed sig_msf_read
	#endif  // #ifdef CHECK_PORT_TX_FIFO_STATUS
	 br[phase_1#]

	// free buffer by dl_buf_free

	buf_free_phase_2#:
	dl_buf_free(meta_data_ptr_p2, BUF_FREE_LIST0)
	_sphy_mphy4_tx_debug_incr_counter(@default_buffers_freed)
	br[buffer_free_done_phase_2#]

	#ifdef CHECK_PORT_TX_FIFO_STATUS

	port_tx_fifo_full_phase_1#:
	_sphy_mphy4_tx_wait_port_tx_fifo_ready(@fc_credit,
				$port_stat_phase_2, port_tx_fifo_check_done_phase_1#)


	port_tx_fifo_full_phase_2#:
	_sphy_mphy4_tx_wait_port_tx_fifo_ready(@fc_credit, 
				$port_stat_phase_1, port_tx_fifo_check_done_phase_2#)

	#endif  // #ifdef CHECK_PORT_TX_FIFO_STATUS

.end	 // $w0 $w1

#endm	// sphy_mphy4_tx3()
										


///////////////////////////////////////////////////////////////////////////////
// The main code starts here

main#:
		// initialize the microblock
	#if(TX_PHY_MODE == SPHY_1_32)
		sphy_mphy4_tx_init(port_entry_turnaround, SPHY_1_32)
	#endif // #if(TX_PHY_MODE == SPHY_1_32)

	// call the packet processing block
	sphy_mphy4_tx3()

/////////////////////////////////////////////////////////////////////////////////////

#endif 		// __SPHY_MPHY4_TX3_UC__

/////////////////////////////////////////////////////////////////////////////////////


      

