////////////////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
////////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: sphy_mphy4_tx_common_util.uc
//
//      Purpose: Common utility macros for Packet (POS/Ethernet) TX
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __SPHY_MPHY4_TX_COMMON_UTIL_UC__
#define __SPHY_MPHY4_TX_COMMON_UTIL_UC__

/////////////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_clear_signal()
//
// Description:
//		Clear a signal from a signal mask 
//
// Outputs: 
//		None 
//
// Inputs/Outputs:  
//		io_sigmask:			signal mask to be cleared with in_sig
//
// Inputs:  
//		in_sig:				signal number to clear
//
// Constants
//
//		None
//
// Size: 
//
//		1 instructions
// 
//
/////////////////////////////////////////////////////////////////////////////////////

#macro _sphy_mphy4_tx_clear_signal(io_sigmask, in_sig)

.begin

#if ( isnum(in_sig) )
	
	#define_eval SIG_SHIFT sig
	alu_shf[io_sigmask, io_sigmask, AND~, 1, <<SIG_SHIFT ]
	#undef SIG_SHIFT

#else

	alu[io_sigmask, io_sigmask, AND~, 1, <<(&in_sig)]

#endif

.end

#endm // end of #macro _sphy_mphy4_tx_clear_signal()

/////////////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_set_signal()
//
// Description:
// 	
//		Set a signal in a signal mask 
//
// Outputs: 
//		None 
//
// Inputs/Outputs:  
//		io_sigmask:			signal mask to be added with in_sig
//
// Inputs:  
//		in_sig:				signal number to set
//
// Constants
//		None
//
// Size: 
//
//		1 instruction
// 
//
////////////////////////////////////////////////////////////////////////////////////

#macro _sphy_mphy4_tx_set_signal(io_sigmask, in_sig)

.begin
#if ( isnum(in_sig) )

	#define_eval SIG_SHIFT in_sig
	alu_shf[io_sigmask, io_sigmask, OR, 1, <<SIG_SHIFT]
	#undef SIG_SHIFT

#else

	alu[io_sigmask, io_sigmask, OR, 1, <<(&in_sig)]

#endif

.end

#endm // end of #macro _sphy_mphy4_tx_set_signal()


/////////////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_set_push_signal()
//
// Description:
// 	
//		Add push signal (i.e. dram access has puu and push signal) in a signal mask 
//
// Outputs:
//		None 
//
// Inputs/Outputs:  
//		io_sigmask:			signal mask to be added with push signal of in_sig
//
// Inputs:  
//		in_sig:				signal number to set
//
// Constants
//		None
//
// Size: 
//
//		1-4 instructions
// 
//
////////////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_set_push_signal(io_sigmask, in_sig)
.begin
.reg tmp
#if ( isnum(in_sig) )

	#define_eval SIG_SHIFT (in_sig +1)
	alu_shf[io_sigmask, io_sigmask, OR, 1, <<SIG_SHIFT]
	#undef SIG_SHIFT

#else
	alu[tmp, --, B, &in_sig]
	alu[tmp, tmp, +, 1]
	alu[--, tmp, OR, 0]										
	alu_shf[io_sigmask, io_sigmask, OR, 1, <<indirect]

#endif
.end

#endm // end of #macro _sphy_mphy4_tx_set_push_signal()


///////////////////////////////////////////////////////////////////////////////
//
// _sphy_mphy4_tx_debug_incr_counter()
//	
//	Description:
//		Increment the debug counter by one
//
//	Outputs:	
//		None
//		
//	Inputs:
//		io_counter:		counter to increment
//
//	Inputs:
//		None
//
//	CONSTANTS:	
//		None
//	
//	Instructions:		1
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_debug_incr_counter(io_counter)

#ifdef	_DEBUG_COUNTERS_
.begin

	alu[io_counter,io_counter,+,1]

.end
#endif	// #ifdef	_DEBUG_COUNTERS_
#endm	//	_sphy_mphy4_tx_debug_incr_counter()



#endif 		// __SPHY_MPHY4_TX_COMMON_UTIL_UC__

