////////////////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
////////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: sphy_mphy4_tx_util.uc
//
//      Purpose: Utility macros for 1st TX ME in Packet (POS/Ethernet) TX
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __SPHY_MPHY4_TX_UTIL_UC__
#define __SPHY_MPHY4_TX_UTIL_UC__

///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_set_16bit_msf_reg()
//
// Description: Use msf[fast_wr...] to set MSF register 
//
// Outputs: 
//								None
//
// Inputs:  
//		in_msf_reg_addr:		MSF register address to set
//		in_msf_reg_val:			MSF register value to set 
//
// Constants:
//								None 
//
// Size: 
//
//		3 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_set_16bit_msf_reg(in_msf_reg_addr, in_msf_reg_val)
.begin
.reg tmp_addr tmp_val
	  alu[tmp_addr, --, B, in_msf_reg_addr]			; get 16 bit msf address
	  alu_shf[tmp_val, --, B, in_msf_reg_val, <<16]	; get value							
	  msf[fast_wr, --, tmp_addr, tmp_val] 			; setting with fast_wr
.end // tmp_addr tmp_val
#endm    // end of macro _sphy_mphy4_tx_set_16bit_msf_reg()

///////////////////////////////////////////////////////////////////////////////
// 
// _sphy_mphy4_tx_init_gprs()
//
// Description:
// 	
//		Initialize the common globals in context relative gprs to save 
//      instruction cycles
//
// Outputs: 
//      out_indiref_base:		base for indirect refernce in moving 
//								payload from dram to tbuf
//		out_default_sigmask1:	default sig_mask for phase 1	
//		out_default_sigmask2:	default sig_mask for phase 2	
//		
// Inputs:  
//		None
//		
// Constants
//      DRAM_INDIREF_BASE_LW:	base for indirect refernce in moving 
//								payload from dram to tbuf

//
// Size: 
//
//		10 instructions
// 
//
///////////////////////////////////////////////////////////////////////////////
#macro	_sphy_mphy4_tx_init_gprs(out_indiref_base,  \
							out_default_sigmask)

.begin
	// base to calculate the address field in indirect ref of 
	// dram[tbuf_wr....] instruction
	move(out_indiref_base, DRAM_INDIREF_BASE_LW)

	// default phase 2 signal mask at start of each iteration of loop 
	alu[out_default_sigmask, --, B, 0]			; initialize to 0


	_sphy_mphy4_tx_set_signal[out_default_sigmask, sig3_next_context]	; add sig3_next_context

	_sphy_mphy4_tx_set_signal(out_default_sigmask, sig_sram_read_1)	; add sig_sram_read_1


.end

#endm  // end of #macro _sphy_mphy4_tx_init_gprs()

///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_set_lmindex0_to_queue_head()
//
// Description: set local memory index 0 to the head of the queue entry to 
//				dequeue one packet
//
// Outputs:					
//		out_tcw:			transmit control word 0
//		out_tbufele:		the next available tbuf element 
//
// Inputs:  
//							None
//
// Constants:
//						 	None 
//
//	Size:
//		4 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_sphy_mphy4_tx_set_lmindex0_to_queue_head()

	// set local memory index to dequeu a packet at queue head

	local_csr_wr[active_lm_addr_0, *l$index1[GLOBAL_HEAD_OFFSET_INDEX]]


#endm // end of #macro _sphy_mphy4_tx_set_lmindex0_to_queue_head()


///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_set_lmindex0_to_queue_tail()
//
// Description: set local memory index 0 to the tail of the queue entry for 
//				the port and put the packet in tx request in queue tail
//
// Outputs:					
//							None 
//
// Inputs:  
//      in_turnaround:		queue head/tail offset value for reset offset
//
// Constants:
//						 	None 
//
//	Size:
//		13 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_move_tail_ahead[current_tail, in_turnaround]

	// set local memory index to queue entry at queue tail of that port,
	// update ab_bd and p_sop_flag 
.begin 
.reg tmpxy
 
	//local_csr_wr[active_lm_addr_0, *l$index1[GLOBAL_TAIL_OFFSET_INDEX]]
	alu[current_tail, --, b, *l$index1[GLOBAL_TAIL_OFFSET_INDEX]]
	// while waiting for local_csr_wr to settle down,
	// update queuetail and @queue_count
	alu[--, *l$index1[GLOBAL_TAIL_OFFSET_INDEX], -, in_turnaround]
	bne[set_lmindex0_to_queue_tail_done#], defer[1]
		alu[*l$index1[GLOBAL_TAIL_OFFSET_INDEX], \
			*l$index1[GLOBAL_TAIL_OFFSET_INDEX], +, QUEUE_ENTRY_SIZE]
	alu[*l$index1[GLOBAL_TAIL_OFFSET_INDEX], --, B, \
			*l$index1[GLOBAL_QUEUE_INFO_BASE_INDEX]]
set_lmindex0_to_queue_tail_done#:
.end 
#endm // end of #macro _sphy_mphy4_tx_set_lmindex0_to_queue_tail()

	 
///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_set_lmindex0_to_queue_tail()
//
// Description: set local memory index 0 to the tail of the queue entry for 
//				the port and put the packet in tx request in queue tail
//
// Outputs:					
//							None 
//
// Inputs:  
//      in_turnaround:		queue head/tail offset value for reset offset
//
// Constants:
//						 	None 
//
//	Size:
//		13 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_set_lmindex0_to_queue_tail[in_turnaround]

	// set local memory index to queue entry at queue tail of that port,
	// update ab_bd and p_sop_flag 
.begin 
.reg tmpxy
 
	local_csr_wr[active_lm_addr_0, *l$index1[GLOBAL_TAIL_OFFSET_INDEX]]
	// while waiting for local_csr_wr to settle down,
	// update queuetail and @queue_count
	alu[--, *l$index1[GLOBAL_TAIL_OFFSET_INDEX], -, in_turnaround]
	bne[set_lmindex0_to_queue_tail_done#], defer[1]
		alu[*l$index1[GLOBAL_TAIL_OFFSET_INDEX], \
			*l$index1[GLOBAL_TAIL_OFFSET_INDEX], +, QUEUE_ENTRY_SIZE]
	alu[*l$index1[GLOBAL_TAIL_OFFSET_INDEX], --, B, \
			*l$index1[GLOBAL_QUEUE_INFO_BASE_INDEX]]
set_lmindex0_to_queue_tail_done#:
.end 
#endm // end of #macro _sphy_mphy4_tx_set_lmindex0_to_queue_tail()


///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_advance_queue_head()
//
// Description: advance queue head after the last mpcket of one packet is sent
//              to the port
//
// Outputs:							
//							None
//
// Inputs:  
//      in_turnaround:		queue head/tail offset value for reset offset
//
// Constants:
//						 	None 
//
// Size:
//		6 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_advance_queue_head(in_turnaround)

.begin
continue_advance_queue_head#:
	alu[--, *l$index1[GLOBAL_HEAD_OFFSET_INDEX], -, in_turnaround]
	
	#ifdef DISABLE_TX2SCHED_FEEDBACK
	bne[advance_queue_head_done#], defer[2]
		alu[*l$index1[GLOBAL_PKTS_IN_QUEUE_INDEX], \
			*l$index1[GLOBAL_PKTS_IN_QUEUE_INDEX], -, 1]
		alu[*l$index1[GLOBAL_HEAD_OFFSET_INDEX], \
				*l$index1[GLOBAL_HEAD_OFFSET_INDEX], +, QUEUE_ENTRY_SIZE]
	#else	// TX2SCHED_FEEDBACK
	bne[advance_queue_head_done#], defer[3]
		alu[*l$index1[GLOBAL_PKTS_IN_QUEUE_INDEX], \
			*l$index1[GLOBAL_PKTS_IN_QUEUE_INDEX], -, 1]
		alu[*l$index1[GLOBAL_HEAD_OFFSET_INDEX], \
				*l$index1[GLOBAL_HEAD_OFFSET_INDEX], +, QUEUE_ENTRY_SIZE]
		alu[*l$index1[GLOBAL_PKTS_TXED_INDEX], \
						*l$index1[GLOBAL_PKTS_TXED_INDEX], +, 1]
	#endif	// #ifdef DISABLE_TX2SCHED_FEEDBACK

	alu[*l$index1[GLOBAL_HEAD_OFFSET_INDEX], --, B, \
			*l$index1[GLOBAL_QUEUE_INFO_BASE_INDEX]]
advance_queue_head_done#:

	// send the feedback to scheduler since it is sop mpacket
	// this feedback will be sent once per packet
	#ifndef DISABLE_TX2SCHED_FEEDBACK
		// notify scheduler that one packet has been transmitted from this port [10]
		_sphy_mphy4_tx_notify_scheduler($txed_port, sig_reflector_write)
	#endif	// #ifndef DISABLE_TX2SCHED_FEEDBACK

.end
#endm // end of macro _sphy_mphy4_tx_advance_queue_head()




///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_set_tcw0_with_paylo_len_offset()
//
// Description: set payload_length, payload_offset, and channel fields in 
//              trnsmit control word
//
// Outputs:							
//							None
//
// Inputs/Outputs:  
//		io_tcw:				transmit control word
// 
// Inputs:  
//		in_paylo_len:		payload length
//		in_paylo_offse:		payload offset
//
// Constants:
//		TCW0_PAYLOAD_LEN_LSB_LOC:	payload length field location in transmit 
//									control word   
//		TCW0_PAYLOAD_OFFSET_LSB_LOC:	payload offset field location in  
//										transmit contrl word     
//
// Size:
//		2 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_sphy_mphy4_tx_set_tcw0_with_paylo_len_offset(io_tcw, in_paylo_len, \
													in_paylo_offset)
	alu_shf[io_tcw, io_tcw, or, in_paylo_len, <<TCW0_PAYLOAD_LEN_LSB_LOC]
	alu_shf[io_tcw, io_tcw, or, in_paylo_offset, <<TCW0_PAYLOAD_OFFSET_LSB_LOC]
#endm // end of macro _sphy_mphy4_tx_set_tcw0_with_paylo_len_offset()


///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_save_sop_meta_to_lm()
//
// Description: save sop meta data to local memory
//
// Outputs: 
//								none
//
// Inputs:  
//			in_txr_lw0:			tx request from Queue Manager 
//
// Constants:
//								none
//
//	Size: 10 instructions. (worst case)
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_save_sop_meta_to_lm(in_txr_lw0)
.begin 
.reg v_buf_size whole_dl_meta0
	dl_meta_get_buffer_next[whole_dl_meta0]
	ld_field_w_clr[*l$index0[SBD_0_OFFSET], 0111, whole_dl_meta0]	; get sbd
	alu[--, *l$index0[SBD_0_OFFSET], -, IX_NULL]	; is null (0xFF)?
	beq[finish_save_sop_meta_to_lm#], defer[3]
		ld_field_w_clr[*l$index0[ABD_0_OFFSET], 0111, in_txr_lw0]	; set ab_bd
		alu_shf[*l$index0[ABD_0_OFFSET], *l$index0[ABD_0_OFFSET], or, \
				1, <<SOP_BIT_LOC]
 	  	//update sop buffer size, offset
		dl_meta_get_offset[*l$index0[ABD_1_OFFSET]]
	 
	alu_shf[*l$index0[ABD_0_OFFSET], *l$index0[ABD_0_OFFSET], or, 1, \
			<<RD_NBD_BIT_LOC]	; set ab_rd_nbd_flag
finish_save_sop_meta_to_lm#:
	dl_meta_get_buffer_size[v_buf_size]
	ld_field[*l$index0[ABD_1_OFFSET], 1100, v_buf_size, <<PAYLO_RMND_LOC] 
	alu[*l$index1[GLOBAL_PKTS_IN_QUEUE_INDEX], \
				*l$index1[GLOBAL_PKTS_IN_QUEUE_INDEX], +, 1]

	#ifdef ADD_L2_HEADER
		#ifdef ETHERNET_TX
			dl_meta_get_nexthop_id[*l$index0[SBD_1_OFFSET]]	; save nexthop_id
		#endif	// #ifdef ETHERNET_TX
	#endif // #ifdef ADD_L2_HEADER

	#ifdef POS_TX
		#ifdef DYNAMIC_PPP_ENCAP
			dl_meta_get_header_type[*l$index0[ABD_PKT_TYPE_OFFSET]]		;Dynamic PPP Encap
		#endif // #ifdef POS_TX
	#endif // #ifdef POS_TX

.end // v_buf_size whole_dl_meta0

#endm // end of #macro _sphy_mphy4_tx_save_sop_meta_to_lm()

///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_read_meta_data_from_sram()  
//
// Description: read the first to third longwords of meta data from sram
//
// Outputs:							
//		out_xmeta:			the sram transfer registers to keep meta data 
//
// Inputs:  
//		in_buf_handle:		buffer handle to get meta data
//
// Constants:
//		none   
//
// Size:
//		4 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_read_meta_data_from_sram(out_xmeta, in_buf_handle)
	// read meta data by dispatch_loop macro, signal (sig_sram_read_1)
	// is included in sigmask_phs2_default to save one instruction cycle 
	// in the critical POS min packet case
	dl_meta_load_cache(in_buf_handle, out_xmeta, sig_sram_read_1, 0, \
													META_DATA_LW_TO_READ)

#endm // end of macro _sphy_mphy4_tx_read_meta_data_from_sram()

///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_read_sb_meta_data_from_sram()  
//
// Description: read the first to third longwords of meta data from sram
//
// Outputs:							
//		out_xmeta:			the sram transfer registers to keep meta data 
//
// Inputs/Outputs:  
//		io_sigmask:			signal mask to be added with sig_sram_read_1
// 
// Inputs:  
//		in_buf_handle:		buffer handle to get meta data
//
// Constants:
//		none   
//
// Size:
//		4 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_read_sb_meta_data_from_sram(out_xmeta, io_sigmask, \
								in_buf_handle)
	// add sig_sram_read_1 to io_sigmask
	_sphy_mphy4_tx_set_signal(io_sigmask, sig_sram_read_2)	; sig_sram_read_2

	// read meta data by dispatch_loop macro
	dl_meta_load_cache(in_buf_handle, out_xmeta, sig_sram_read_2, 0, 3)
	
#endm // end of macro _sphy_mphy4_tx_read_sb_meta_data_from_sram()


/////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_update_ab_paylo_rmnd_and_offset_rpaylo()
//
// Description: update ab_paylo_rmnd and ab_offset_rpaylo in queue entry 
//
//
// Outputs:							
//							None
//
// Inputs/Outputs:  
//		io_mpkt_paylo_len:	payload bytes will transmitted by this mpacket from 
// 							the active buffer
//
// Inputs: 
//							None 
//
// Constants:
//		PAYLO_RMND_LOC:		location of the least significant bit of the 
//							remained payload in queue entry 
//
// Size:
//		3 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_update_ab_paylo_rmnd_and_offset_rpaylo(io_mpkt_paylo_len)
#ifndef AVOID_USING_SKIP_BIT_IN_TCW0
.begin
.reg tmpx
	alu[*l$index0[ABD_1_OFFSET], *l$index0[ABD_1_OFFSET], +, io_mpkt_paylo_len]
	alu_shf[tmpx, --, b, io_mpkt_paylo_len, <<PAYLO_RMND_LOC]
	alu[*l$index0[ABD_1_OFFSET], *l$index0[ABD_1_OFFSET], -, tmpx]
.end // tmpx
#else //#ifndef AVOID_USING_SKIP_BIT_IN_TCW0
// adjust current mpkt size to make sure next mpkt in this buffer always have
// more than 4 bytes payload, so SKIT bit never set in TCW0, this only happens
// in SPHY_1x32 mode  
.begin
.reg next_mpkt_len tmpx
    ld_field_w_clr[tmpx, 0011, *l$index0[ABD_1_OFFSET], >>PAYLO_RMND_LOC]
	alu[next_mpkt_len, tmpx, -, io_mpkt_paylo_len]
	alu[--, next_mpkt_len, -, BUS_WIDTH_IN_BYTES]
	bge[adjust_this_mpkt_to_avoid_using_skip_bit_done#] 
	alu[io_mpkt_paylo_len, io_mpkt_paylo_len, -, BUS_WIDTH_IN_BYTES]
adjust_this_mpkt_to_avoid_using_skip_bit_done#:
	alu[*l$index0[ABD_1_OFFSET], *l$index0[ABD_1_OFFSET], +, io_mpkt_paylo_len]
	alu_shf[tmpx, --, b, io_mpkt_paylo_len, <<PAYLO_RMND_LOC]
	alu[*l$index0[ABD_1_OFFSET], *l$index0[ABD_1_OFFSET], -, tmpx]
.end // next_mpkt_len tmpx
#endif //#ifndef AVOID_USING_SKIP_BIT_IN_TCW0
#endm // end of _sphy_mphy4_tx_update_ab_paylo_rmnd_and_offset_rpaylo()


#macro my_dl_meta_get_buffer_next[buf_next, dl_meta]
.begin

	alu[buf_next, --, B, dl_meta[0]]			; dl_meta0[31:0]

.end
#endm

#macro my_dl_meta_get_buffer_size[buf_len, dl_meta]

.begin

	ld_field_w_clr[buf_len, 0011, dl_meta[1], >>16]		; dl_meta1 [31:16]

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_save_sbd_meta_to_lm()
//
// Description: save secodary buffer data to local memory
//
// Outputs:							
//							None
//
// Inputs:  
//							None
//
// Constants:
//							None  
//
// Size: 7 instructions. (Worst case cycle count)
//
/////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_save_sbd_meta_to_lm(dl_meta)
.begin 
.reg v_buf_size tmp_nsbd tmp_whole
	my_dl_meta_get_buffer_next[tmp_whole, dl_meta]
	; only need lower three bytes
	ld_field_w_clr[tmp_nsbd, 0111, tmp_whole]
	alu[*l$index1[GLOBAL_SBD_NEXT_BD_INDEX], --, b, tmp_nsbd]
	alu[--, *l$index1[GLOBAL_SBD_NEXT_BD_INDEX], -, IX_NULL]	; is null (0xFF)?
	beq[finish_save_sbd_meta_to_lm#], defer[2]
 	  	//update secondary buffer size, offset always zero
		my_dl_meta_get_buffer_size[v_buf_size, dl_meta]
		ld_field_w_clr[*l$index0[SBD_1_OFFSET], 1100, v_buf_size, <<PAYLO_RMND_LOC] 
	 
	alu_shf[*l$index0[SBD_0_OFFSET], *l$index0[SBD_0_OFFSET], or, 1, <<RD_NBD_BIT_LOC]	; set ab_rd_nbd_flag
finish_save_sbd_meta_to_lm#:

.end // v_buf_size tmp_nsbd tmp_whole
#endm // end of #macro _sphy_mphy4_tx_save_sbd_meta_to_lm() 

///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_secondary_buffer_become_active_buffer[]
//
// Description: replace active buffer data with secondary buffer data in  
//              queue entry
//
// Outputs:							
//							None
//
// Inputs:  
//							None
//
// Constants:
//							None 
//
// Size: 
//		5 instructions. (Worst case cycle count)
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_secondary_buffer_become_active_buffer()
.begin
	alu[*l$index0[ABD_0_OFFSET], --, b, *l$index0[SBD_0_OFFSET]]
	alu[*l$index0[ABD_1_OFFSET], --, b, *l$index0[SBD_1_OFFSET]]
	alu[*l$index0[SBD_0_OFFSET], --, b, *l$index1[GLOBAL_SBD_NEXT_BD_INDEX]]
	alu[*l$index0[SBD_1_OFFSET], --, b, 0]		; reset
	alu[*l$index1[GLOBAL_SBD_NEXT_BD_INDEX], --, b, IX_NULL]	; reset
.end
#endm // end of #macro _sphy_mphy4_tx_secondary_buffer_become_active_buffer()

///////////////////////////////////////////////////////////////////////////////
//
// _sphy_mphy4_tx_process_and_save_leftover()
//
// Description: align leftover read from dram, save them in local memory
//
// Prerequisite:			local memory index 1 alrady set to queue status 
//							of the port
//
// Outputs:							
// 		None
//
// Inputs:
//		in_leftov_offset:	leftover offset mod by 8 decided in previous phase
//
// Constants:	
//		None   
//
// Size:
//		instructions 
//
///////////////////////////////////////////////////////////////////////////////

#macro _sphy_mphy4_tx_process_and_save_leftover(in_leftov_offset)

.begin

	.reg	tmp

	// jump based on in_leftov_offset

	jump[in_leftov_offset, jmp_tbl#], defer[3], targets[oset_0#, oset_1#, \
		oset_2#, oset_3#, oset_4#, oset_5#, oset_6#, oset_7#] 
	nop
	nop
	nop

//	Start of jump table
 
jmp_tbl#:

oset_0#:
	br[offset_mod_8_eq_0#]
			
oset_1#:
	br[offset_mod_8_eq_1#]

oset_2#:
	br[offset_mod_8_eq_2#]

oset_3#:
	br[offset_mod_8_eq_3#]

oset_4#:
	br[offset_mod_8_eq_4#]

oset_5#:
	br[offset_mod_8_eq_5#]

oset_6#:
	br[offset_mod_8_eq_6#]

oset_7#:
	br[offset_mod_8_eq_7#]

offset_mod_8_eq_0#:					; (8)
	alu[lm1_leftover_lw0, --, B, $$dw0]
	alu[lm1_leftover_lw1, --, B, $$dw1]
	alu[lm1_leftover_lw2, --, B, $$dw2]
	alu[lm1_leftover_lw3, --, B, $$dw3]
	br[align_save_done#]

offset_mod_8_eq_1#:					; (16)
	ld_field_w_clr[tmp, 1110, $$dw0, <<8]
	ld_field[tmp, 0001, $$dw1, >>24]
	alu[lm1_leftover_lw0, --, B, tmp]

	ld_field_w_clr[tmp, 1110, $$dw1, <<8]
	ld_field[tmp, 0001, $$dw2, >>24]
	alu[lm1_leftover_lw1, --, B, tmp]

	ld_field_w_clr[tmp, 1110, $$dw2, <<8]
	ld_field[tmp, 0001, $$dw3, >>24]
	alu[lm1_leftover_lw2, --, B, tmp]

	ld_field_w_clr[tmp, 1110, $$dw3, <<8]
	ld_field[tmp, 0001, $$dw4, >>24]
	alu[lm1_leftover_lw3, --, B, tmp]

	br[align_save_done#]

offset_mod_8_eq_2#:					; (16)
	ld_field_w_clr[tmp, 1100, $$dw0, <<16]
	ld_field[tmp, 0011, $$dw1, >>16]
	alu[lm1_leftover_lw0, --, B, tmp]

	ld_field_w_clr[tmp, 1100, $$dw1, <<16]
	ld_field[tmp, 0011, $$dw2, >>16]
	alu[lm1_leftover_lw1, --, B, tmp]

	ld_field_w_clr[tmp, 1100, $$dw2, <<16]
	ld_field[tmp, 0011, $$dw3, >>16]
	alu[lm1_leftover_lw2, --, B, tmp]

	ld_field_w_clr[tmp, 1100, $$dw3, <<16]
	ld_field[tmp, 0011, $$dw4, >>16]
	alu[lm1_leftover_lw3, --, B, tmp]


	br[align_save_done#]

offset_mod_8_eq_3#:					; (16)
	ld_field_w_clr[tmp, 1000, $$dw0, <<24]
	ld_field[tmp, 0111, $$dw1, >>8]
	alu[lm1_leftover_lw0, --, B, tmp]

	ld_field_w_clr[tmp, 1000, $$dw1, <<24]
	ld_field[tmp, 0111, $$dw2, >>8]
	alu[lm1_leftover_lw1, --, B, tmp]

	ld_field_w_clr[tmp, 1000, $$dw2, <<24]
	ld_field[tmp, 0111, $$dw3, >>8]
	alu[lm1_leftover_lw2, --, B, tmp]

	ld_field_w_clr[tmp, 1000, $$dw3, <<24]
	ld_field[tmp, 0111, $$dw4, >>8]
	alu[lm1_leftover_lw3, --, B, tmp]


	br[align_save_done#]

offset_mod_8_eq_4#:					; (8)
	alu[lm1_leftover_lw0, --, B, $$dw1]
	alu[lm1_leftover_lw1, --, B, $$dw2]
	alu[lm1_leftover_lw2, --, B, $$dw3]
	alu[lm1_leftover_lw3, --, B, $$dw4]
	br[align_save_done#]

offset_mod_8_eq_5#:					; (16)
	ld_field_w_clr[tmp, 1110, $$dw1, <<8]
	ld_field[tmp, 0001, $$dw2, >>24]
	alu[lm1_leftover_lw0, --, B, tmp]

	ld_field_w_clr[tmp, 1110, $$dw2, <<8]
	ld_field[tmp, 0001, $$dw3, >>24]
	alu[lm1_leftover_lw1, --, B, tmp]

	ld_field_w_clr[tmp, 1110, $$dw3, <<8]
	ld_field[tmp, 0001, $$dw4, >>24]
	alu[lm1_leftover_lw2, --, B, tmp]

	ld_field_w_clr[tmp, 1110, $$dw4, <<8]
	ld_field[tmp, 0001, $$dw5, >>24]
	alu[lm1_leftover_lw3, --, B, tmp]

	br[align_save_done#]

offset_mod_8_eq_6#:					; (16)
	ld_field_w_clr[tmp, 1100, $$dw1, <<16]
	ld_field[tmp, 0011, $$dw2, >>16]
	alu[lm1_leftover_lw0, --, B, tmp]

	ld_field_w_clr[tmp, 1100, $$dw2, <<16]
	ld_field[tmp, 0011, $$dw3, >>16]
	alu[lm1_leftover_lw1, --, B, tmp]

	ld_field_w_clr[tmp, 1100, $$dw3, <<16]
	ld_field[tmp, 0011, $$dw4, >>16]
	alu[lm1_leftover_lw2, --, B, tmp]

	ld_field_w_clr[tmp, 1100, $$dw4, <<16]
	ld_field[tmp, 0011, $$dw5, >>16]
	alu[lm1_leftover_lw3, --, B, tmp]

	br[align_save_done#]


offset_mod_8_eq_7#:					; (16)
	ld_field_w_clr[tmp, 1000, $$dw1, <<24]
	ld_field[tmp, 0111, $$dw2, >>8]
	alu[lm1_leftover_lw0, --, B, tmp]

	ld_field_w_clr[tmp, 1000, $$dw2, <<24]
	ld_field[tmp, 0111, $$dw3, >>8]
	alu[lm1_leftover_lw1, --, B, tmp]

	ld_field_w_clr[tmp, 1000, $$dw3, <<24]
	ld_field[tmp, 0111, $$dw4, >>8]
	alu[lm1_leftover_lw2, --, B, tmp]

	ld_field_w_clr[tmp, 1000, $$dw4, <<24]
	ld_field[tmp, 0111, $$dw5, >>8]
	alu[lm1_leftover_lw3, --, B, tmp]

align_save_done#:

.end

#endm	// _sphy_mphy4_tx_process_and_save_leftover()


///////////////////////////////////////////////////////////////////////////////
//
// _sphy_mphy4_decide_next_two_mpkt_paylo()
//
// Description: decide up to two mpkts size in this one buffer packet, in the 
//				case this packet can be fitted into two or three mpackets, 
//              this macro will advance queue head, otherwise it updates the 
//				payload size and offset of the remaining payload
//
// Inputs/Outputs:							
// 		io_sigmask:				signal mask to add reflect write signal
//
// Outputs:							
// 		out_mpkt:				number of mapcket to process in this iteration
//		out_snd_mpkt_paylo:		second mpacket size	
//		out_thrd_mpkt_paylo:	third mpacket size
//
// Inputs:
//		in_pkt_paylo:			packet payload size
//		in_pkt_offset:			packet offset
//  	in_first_mpkt_paylo:	first (sop) mpacket payload size
//
// Constants:
//		None
//
// Size:
//		instructions
//
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_sphy_mphy4_decide_next_two_mpkt_paylo(io_sigmask, out_mpkt, \
			out_snd_mpkt_paylo, out_thrd_mpkt_paylo, in_pkt_paylo, \
			in_pkt_offset, in_first_mpkt_paylo)

.begin

	// decide whether fitting into 2 mpacket and mpacket 2 size

	_sphy_mphy4_tx_decide_snd_mpkt_paylo(out_snd_mpkt_paylo, in_pkt_paylo, \
		in_first_mpkt_paylo, in_pkt_offset, check_three_mpkts_fit#)
				 
	alu[out_mpkt, --,  B, 2]
	alu_shf[exe_stat_flag, exe_stat_flag, or, 1, <<EOB_MPKT_FLAG_BIT]
	// advanced queue head, so next packet in the queue of this port will be 
	// processed when this port is selected again
	_sphy_mphy4_tx_advance_queue_head(port_entry_turnaround)
	#ifndef DISABLE_TX2SCHED_FEEDBACK

	// add signal for reflect write

	_sphy_mphy4_tx_set_signal[io_sigmask, sig_reflector_write]
	#endif	// #ifndef DISABLE_TX2SCHED_FEEDBACK


	br[done#]

check_three_mpkts_fit#:
	// decide whether fitting into 3 mpacket and mpacket 3 size

	_sphy_mphy4_tx_decide_thrd_mpkt_paylo(out_thrd_mpkt_paylo, \
			out_snd_mpkt_paylo, in_pkt_paylo, in_first_mpkt_paylo, \
			in_pkt_offset, more_than_3_mpkts#)					

	alu[out_mpkt, --,  B, 3]
	alu_shf[exe_stat_flag, exe_stat_flag, or, 1, <<EOB_MPKT_FLAG_BIT]
	// advanced queue head, so next packet in the queue of this port will be 
	// processed when this port is selected again
	_sphy_mphy4_tx_advance_queue_head(port_entry_turnaround)	
	#ifndef DISABLE_TX2SCHED_FEEDBACK

	// add signal for reflect write

	_sphy_mphy4_tx_set_signal[io_sigmask, sig_reflector_write]
	#endif	// #ifndef DISABLE_TX2SCHED_FEEDBACK
	br[done#]

more_than_3_mpkts#: 
	alu[out_mpkt, --,  B, 4]
	
	// reset sop_flag in queue entry

	alu_shf[*l$index0[ABD_0_OFFSET], *l$index0[ABD_0_OFFSET], and~, 1, \
															<<SOP_BIT_LOC]

	// update ab_paylo_rmnd and ab_offset_rpaylo in queue entry with 
	// in_first_mpkt_paylo,	out_snd_mpkt_paylo, out_thrd_mpkt_paylo

	_sphy_mphy4_tx_update_ab_paylo_rmnd_and_offset_rpaylo(in_first_mpkt_paylo)
	_sphy_mphy4_tx_update_ab_paylo_rmnd_and_offset_rpaylo(out_snd_mpkt_paylo)
	_sphy_mphy4_tx_update_ab_paylo_rmnd_and_offset_rpaylo(out_thrd_mpkt_paylo)

done#:

.end
#endm	// _sphy_mphy4_decide_next_two_mpkt_paylo()


///////////////////////////////////////////////////////////////////////////////
//
// _sphy_mphy4_tx_decide_snd_mpkt_paylo()
//
// Description: decide the payload size of second mpacket for the packet
//              which has just one buffer  
//
// Outputs:							
// 		out_mpkt_2_size:		size of the second mpacket
//
// Inputs:
//  	in_pkt_size:			payload size of one buffer packet
//		in_mpkt_1_size:			size for the fisrt mpacket 
//		in_pkt_offset:			payload offset of one buffer packet
//
// Constants:
//		None   
//
//	Labels:
//		MORE_THAN_TWO_MPKTS_LABEL:	symbolic address to go when more than
//									two mpacket required
//
// Size:
//		8 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _sphy_mphy4_tx_decide_snd_mpkt_paylo(out_mpkt_2_size, in_pkt_size, \
				in_mpkt_1_size, in_pkt_offset, MORE_THAN_TWO_MPKTS_LABEL)

.begin

	.reg	tmp
	.reg	tmp1

	alu[tmp, in_pkt_offset, +, in_mpkt_1_size]
	alu[tmp, tmp, AND, CONST_07]		; offset_mod_8 for mpkt_2

	// payload in this buffer left after mpkt_1

	alu[tmp1, in_pkt_size, -, in_mpkt_1_size]	; payload_after_mpkt_1	

	// maximum allowable mpkt_2_size

	alu[tmp, TBUF_ELE_SIZE, -, tmp] 	; maximum allowable eop mpkt_2_size
	// mpkt_2_size for not eop	
	alu[out_mpkt_2_size, tmp, AND, DEVICE_BUS_WIDTH_RESTRICTION_MASK]	

	alu[--, tmp1, -, tmp]
	bgt[MORE_THAN_TWO_MPKTS_LABEL]
	alu[out_mpkt_2_size, --, B, tmp1]	; mpkt_2_size for eop mpkt

.end 

#endm	// _sphy_mphy4_tx_decide_snd_mpkt_paylo() 


///////////////////////////////////////////////////////////////////////////////
//
// _sphy_mphy4_tx_decide_thrd_mpkt_paylo()
//
// Description: decide the payload size of third mpacket for the packet which
//              has just one buffer  
//
// Outputs:							
// 		out_mpkt_3_size:		size of the third mpacket
//
// Inputs:
//		in_mpkt_2_size:			size of the second mpacket
//  	in_pkt_size:			payload for one buffer packet
//		in_mpkt_1_size:			size of the first mpacket 
//		in_pkt_offset:			payload offset of one buffer packet
//
// Constants:
//		None   
//
//	Labels:
//		MORE_THAN_THREE_MPKTS_LABEL:	symbolic address to go when more than
//										three mpackets required
//
// Size:
//		10 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _sphy_mphy4_tx_decide_thrd_mpkt_paylo(out_mpkt_3_size, \
						in_mpkt_2_size, in_pkt_size, in_mpkt_1_size, \
						in_pkt_offset, MORE_THAN_THREE_MPKTS_LABEL)

.begin

	.reg	tmp
	.reg	tmp1

	alu[tmp, in_pkt_offset, +, in_mpkt_1_size]
	alu[tmp, tmp, +, in_mpkt_2_size]
	alu[tmp, tmp, AND, CONST_07]		; offset_mod_8 for mpkt_3
	alu[tmp1, in_pkt_size, -, in_mpkt_1_size]
	alu[tmp1, tmp1, -, in_mpkt_2_size]	; payload after mpkt_2

	// maximum allowable mpkt_3_size

	alu[tmp, TBUF_ELE_SIZE, -, tmp] 	; maximum allowable eop mpkt_3_size 
	// mpkt_3_size for not eop
	alu[out_mpkt_3_size, tmp, AND, DEVICE_BUS_WIDTH_RESTRICTION_MASK]	
	alu[--, tmp1, -, tmp]
	bgt[MORE_THAN_THREE_MPKTS_LABEL]
	alu[out_mpkt_3_size, --, B, tmp1]	; mpkt_3_size for eop mpkt
.end 

#endm	// _sphy_mphy4_tx_decide_thrd_mpkt_paylo() 

///////////////////////////////////////////////////////////////////////////////
//
// 	_sphy_mphy4_tx_calendar_init()
//
// Description:
//		Initialize TX_Calendar related MSF registers (only IXP28XX)
//
// Outputs: 
//		None
//
// Inputs:  
//		in_first_port:		first port
//		in_total_ports:     total port number
//		
// Constants
//		None
//
///////////////////////////////////////////////////////////////////////////////

#macro _sphy_mphy4_tx_calendar_init(in_first_port, in_total_ports)

.begin 

#if IS_IXPTYPE(__IXP28XX)

	.reg	tmp
	.reg	calendar_0_addr
	.reg	$data
	.reg    addr_incr
	.reg 	i
	.reg	calendar_length_addr

	move(i, 0)
	move(tmp, in_first_port)
	alu_shf[addr_incr, --, B, tmp, <<2]
	move(calendar_0_addr, TX_CALENDAR_0)

	// Initialize Tx_calendar_# 

	.while (i < in_total_ports)	

		alu[tmp, --, b, in_first_port]
		alu[$data,	--, B, tmp]
		msf[write, $data, calendar_0_addr, addr_incr, 1], \
												ctx_swap[sig_msf_access_1]
		alu[i, i, +, 1]
		alu[addr_incr, addr_incr, +, 4]

	.endw

	// Initialize Tx_Calendar_Length

	move(calendar_length_addr, TX_CALENDAR_LENGTH)
	alu[$data,	--, B, in_total_ports]
	msf[write, $data, calendar_length_addr, 0, 1], ctx_swap[sig_msf_access_1]

#endif	// #if IS_IXPTYPE(__IXP28XX)

.end

#endm	// _sphy_mphy4_tx_calendar_init() 


#ifdef ADD_L2_HEADER
#ifdef ETHERNET_TX
///////////////////////////////////////////////////////////////////////////////
//
// _sphy_mphy4_tx_read_l2_table_entry()  
//
// Description: read the L2 table entry data from sram
//
// Outputs:  
//		none
// 
// Inputs:  
//		in_l2_table_base:	L2 table base
//		in_nexthop_id:		nexthop_id
//		in_sig:				signal for reading l2_table_entry
//
// Constants:
//		none   
//
// Size:
//		3 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_read_l2_table_entry(in_l2_table_base, in_nexthop_id, \
 																	in_sig)
.begin
.reg l2_entry_addr

	alu[l2_entry_addr, --, B, in_nexthop_id, <<L2_TABLE_ENTRY_SHFT]
	sram[read, $l2_entry_lw0, in_l2_table_base, l2_entry_addr, \
			L2_TABLE_ENTRY_SIZE_IN_LW], sig_done[in_sig]
	
.end // l2_entry_addr

#endm // _sphy_mphy4_tx_read_l2_table_entry()


///////////////////////////////////////////////////////////////////////////////
//
// _sphy_mphy4_tx_pass_l2_entry_to_next_me()  
//
// Description: pass l2 table entry to next ME through NN ring
//
// Outputs:  
//		none
// 
// Inputs:  
//		in_l2_header_exist_flag:	1: l2 header already in payload
//									0: need pass to next ME
//
// Constants:
//		none   
//
// Size:
//		5 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_pass_l2_entry_to_next_me(in_l2_header_exist_flag)
.begin

	br=byte[in_l2_header_exist_flag, 0, L2_HEADER_IN_PAYLOAD, \
												l2_header_already_exist#]

	// pass l2 table entry to next ME
	waiting_for_nn_to_become_not_full_1#:
	br_inp_state[nn_full, waiting_for_nn_to_become_not_full_1#]
							
	alu[*n$index++, --, b, $l2_entry_lw0]
	alu[*n$index++, --, b, $l2_entry_lw1]	
	alu[*n$index++, --, b, $l2_entry_lw2]

	waiting_for_nn_to_become_not_full_2#:
	br_inp_state[nn_full, waiting_for_nn_to_become_not_full_2#]
	alu[*n$index++, --, b, $l2_entry_lw3]

	// write two extra two longword (duplicate of the fouth longword of the
	// l2 table entry), because next ME NN ring become empty, when valid
	// element on it less than or equal to 2 based on it's initialization 		

	alu[*n$index++, --, b, $l2_entry_lw3]	
	alu[*n$index++, --, b, $l2_entry_lw3]	

l2_header_already_exist#:
.end 
#endm // _sphy_mphy4_tx_pass_l2_entry_to_next_me(()

#endif // #ifdef ETHERNET_TX
#endif // #ifdef ADD_L2_HEADER


 #ifndef DISABLE_TX2SCHED_FEEDBACK

///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_notify_scheduler()
//
// Description: notify scheduler that one packet has been transmitted from 
//				this port 
//
// Outputs:							
//      out_txd_port:		total packets transmitted by the port (sram 
//                          transfer register)
//
// Inputs:  
//      in_sig:				signal for reflect write
//
// Constants:
//						 	None 
//
// Size:
//		10 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_sphy_mphy4_tx_notify_scheduler(out_txd_port, in_sig)
#if (TX_PHY_MODE == SPHY_1_32)

.begin

.reg remote $$txd_p0 
	alu[out_txd_port, --, b, *l$index1[GLOBAL_PKTS_TXED_INDEX]] ; get txed pkts

	// update scheduler with updated counts of packets transmitted for port 0
	cap[write, out_txd_port, SCHEDULER_ME, $$txd_p0, 0, 1], \
			sig_done[in_sig]	; signal

.end
#endif	// #if (TX_PHY_MODE == SPHY_1_32)

#endm // end of macro _sphy_mphy4_tx_notify_scheduler()
#endif	// #ifndef DISABLE_TX2SCHED_FEEDBACK




#endif 		// __SPHY_MPHY4_TX_UTIL_UC__

///////////////////////////////////////////////////////////////////////////////

