////////////////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
////////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: sphy_mphy4_tx_util2.uc
//
//      Purpose: Utility macros for 2nd TX ME in Packet (POS/Ethernet) TX
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __SPHY_MPHY4_TX_UTIL2_UC__
#define __SPHY_MPHY4_TX_UTIL2_UC__

///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_set_16bit_msf_reg()
//
// Description: Use msf[fast_wr...] to set MSF register 
//
// Outputs: 
//								None
//
// Inputs:  
//		in_msf_reg_addr:		MSF register address to set
//		in_msf_reg_val:			MSF register value to set 
//
// Constants:
//								None 
//
// Size: 
//
//		3 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_set_16bit_msf_reg(in_msf_reg_addr, in_msf_reg_val)
.begin
.reg tmp_addr tmp_val
	  alu[tmp_addr, --, B, in_msf_reg_addr]			; get 16 bit msf address
	  alu_shf[tmp_val, --, B, in_msf_reg_val, <<16]	; get value							
	  msf[fast_wr, --, tmp_addr, tmp_val] 			; setting with fast_wr
.end // tmp_addr tmp_val
#endm    // end of macro _sphy_mphy4_tx_set_16bit_msf_reg()

///////////////////////////////////////////////////////////////////////////////
// 
// _sphy_mphy4_tx_init_gprs()
//
// Description:
// 	
//		Initialize the common globals in context relative gprs to save 
//      instruction cycles
//
// Outputs: 
//      out_indiref_base:		base for indirect refernce in moving 
//								payload from dram to tbuf
//		out_default_sigmask1:	default sig_mask for phase 1	
//		out_default_sigmask2:	default sig_mask for phase 2	
//		
// Inputs:  
//		None
//		
// Constants
//      DRAM_INDIREF_BASE_LW:	base for indirect refernce in moving 
//								payload from dram to tbuf

//
// Size: 
//
//		10 instructions
// 
//
///////////////////////////////////////////////////////////////////////////////
#macro	_sphy_mphy4_tx_init_gprs(out_indiref_base, out_default_sigmask1, \
							out_default_sigmask2)

.begin
	// base to calculate the address field in indirect ref of 
	// dram[tbuf_wr....] instruction
	move(out_indiref_base, DRAM_INDIREF_BASE_LW)


.end

#endm  // end of #macro _sphy_mphy4_tx_init_gprs()

///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_set_lmindex0_to_queue_head()
//
// Description: set local memory index 0 to the head of the queue entry to 
//				dequeue one packet
//
// Outputs:					
//		out_tcw:			transmit control word 0
//		out_tbufele:		the next available tbuf element 
//
// Inputs:  
//							None
//
// Constants:
//						 	None 
//
//	Size:
//		4 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_sphy_mphy4_tx_set_lmindex0_to_queue_head(out_tcw, out_tbufele)

	// set local memory index to dequeu a packet at queue head

	local_csr_wr[active_lm_addr_0, *l$index1[GLOBAL_HEAD_OFFSET_INDEX]]
	// while waiting for local_csr_wr to settle down,
	// calculate the port_qv_stat for adding port to qstat_vector
	immed[out_tcw, TCW0_SOP_EOP_BASE_W0]
	immed_w1[out_tcw, TCW0_SOP_EOP_BASE_W1]
	
	// alok
	#ifdef ONE_ME_PACKET_TX
	 	alu[out_tbufele, --, b, *l$index1[GLOBAL_AVAIL_TBUF_ELEMENT_INDEX]]
	#else // TWO_ME_PACKET_TX
		// nop
	#endif //ONE_ME_PACKET_TX
#endm // end of #macro _sphy_mphy4_tx_set_lmindex0_to_queue_head()

	 
///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_set_lmindex0_to_queue_tail()
//
// Description: set local memory index 0 to the tail of the queue entry for 
//				the port and put the packet in tx request in queue tail
//
// Outputs:					
//							None 
//
// Inputs:  
//      in_turnaround:		queue head/tail offset value for reset offset
//
// Constants:
//						 	None 
//
//	Size:
//		13 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_set_lmindex0_to_queue_tail[in_turnaround]

	// set local memory index to queue entry at queue tail of that port,
	// update ab_bd and p_sop_flag 
.begin 
.reg tmpxy
 
	local_csr_wr[active_lm_addr_0, *l$index1[GLOBAL_TAIL_OFFSET_INDEX]]
	// while waiting for local_csr_wr to settle down,
	// update queuetail and @queue_count
	alu[--, *l$index1[GLOBAL_TAIL_OFFSET_INDEX], -, in_turnaround]
	bne[set_lmindex0_to_queue_tail_done#], defer[1]
		alu[*l$index1[GLOBAL_TAIL_OFFSET_INDEX], \
			*l$index1[GLOBAL_TAIL_OFFSET_INDEX], +, QUEUE_ENTRY_SIZE]
	alu[*l$index1[GLOBAL_TAIL_OFFSET_INDEX], --, B, \
			*l$index1[GLOBAL_QUEUE_INFO_BASE_INDEX]]
set_lmindex0_to_queue_tail_done#:
.end 
#endm // end of #macro _sphy_mphy4_tx_set_lmindex0_to_queue_tail()


///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_advance_queue_head()
//
// Description: advance queue head after the last mpcket of one packet is sent
//              to the port
//
// Outputs:							
//							None
//
// Inputs:  
//      in_turnaround:		queue head/tail offset value for reset offset
//
// Constants:
//						 	None 
//
// Size:
//		6 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_advance_queue_head(in_turnaround)

.begin
continue_advance_queue_head#:
	alu[--, *l$index1[GLOBAL_HEAD_OFFSET_INDEX], -, in_turnaround]
	bne[advance_queue_head_done#], defer[3]
		alu[*l$index1[GLOBAL_PKTS_IN_QUEUE_INDEX], \
			*l$index1[GLOBAL_PKTS_IN_QUEUE_INDEX], -, 1]
		alu[*l$index1[GLOBAL_HEAD_OFFSET_INDEX], \
				*l$index1[GLOBAL_HEAD_OFFSET_INDEX], +, QUEUE_ENTRY_SIZE]
		alu[*l$index1[GLOBAL_PKTS_TXED_INDEX], \
				*l$index1[GLOBAL_PKTS_TXED_INDEX], +, 1]
	alu[*l$index1[GLOBAL_HEAD_OFFSET_INDEX], --, B, \
			*l$index1[GLOBAL_QUEUE_INFO_BASE_INDEX]]
advance_queue_head_done#:

.end
#endm // end of macro _sphy_mphy4_tx_advance_queue_head()


///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_update_next_availble_tbuf_ele()
//
// Description: update next available tbuf element in local memory
//
// Outputs:							
//							None
//
// Inputs:  
//							None
//
// Constants:
//							None
//
// Size:
//		2 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_update_next_availble_tbuf_ele()
.begin 
		alu[*l$index1[GLOBAL_AVAIL_TBUF_ELEMENT_INDEX], \
			*l$index1[GLOBAL_AVAIL_TBUF_ELEMENT_INDEX], +, 1]
		alu[*l$index1[GLOBAL_AVAIL_TBUF_ELEMENT_INDEX], \
			*l$index1[GLOBAL_AVAIL_TBUF_ELEMENT_INDEX], and, TBUF_ELE_BITS_MASK]
.end 

#endm // end of macro _sphy_mphy4_tx_update_next_availble_tbuf_ele()


///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_get_tbuf_addr()
//
// Description: get tbuf element adddress for input tbuf element
//
// Outputs:							
//		out_tbuf_addr:		the starting address of the input tbuf element
//							element
//
// Inputs:  
//		in_tbufele:			the input tbuf element 
//
// Constants:
//							None
//
// Size:
//		2 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_get_tbuf_addr(out_tbuf_addr, in_tbufele)
.begin 
.reg tmpx		
	alu_shf[tmpx, --, B, in_tbufele, <<TBUF_ADDR_SHF]
	alu[out_tbuf_addr, tmpx, +, *l$index1[GLOBAL_TBUF_BASE_INDEX]]
.end // tmpx

#endm // end of macro _sphy_mphy4_tx_get_tbuf_addr()

/////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_move_sop_paylo_to_tbuf() 
//
// Description: move payload of sop or sop_eop mpacket to tbuf
//
// Outputs:							
// 		out_indir:			indirect reference used in dram[tbuf_wr...] 
//							instruction
//		out_dramaddr:		dram address of the payload
//
// Inputs:  
//		in_bd:				meta data handle 
//		in_addrtbuf:		the starting address of tbuf for payload 
//		in_indirbase:		base for  indir
//      in_of_set: 			the starting address of the payload to copy
//		in_paylo:			payload length
//		in_offsetmod8:		payload offset
//		in_sig_dram:		signal number for dram[tbuf_wr...] instruction
//
// Constants:
//							none
//
// Size:
//		8 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_sphy_mphy4_tx_move_sop_paylo_to_tbuf(out_indir, out_dramaddr, in_bd, \
			in_addrtbuf, in_indirbase, in_of_set, in_paylo, in_offsetmod8, \
			in_sig_dram) 
.begin
.reg tmpx
#ifndef ADD_L2_HEADER
	// prepare tbuf address for  dram[tbuf_wr......]
	#ifdef SPI4_PREPEND

		alu[tmpx, in_addrtbuf, +, SPI4_PREPEND_HEADER_SIZE]
		// prepare tbuf address for  dram[tbuf_wr......]
		alu[out_indir, in_indirbase, or, tmpx, <<5]

	#else
		alu[out_indir, in_indirbase, or, in_addrtbuf, <<5]
	#endif

#else // ADD_L2_HEADER
	#ifdef POS_TX
	alu[tmpx, in_addrtbuf, +, BYTES_FOR_L2_HEADER]
	// prepare tbuf address for  dram[tbuf_wr......]
	alu[out_indir, in_indirbase, or, tmpx, <<5]
	#endif	// #ifdef POS_TX
	#ifdef ETHERNET_TX	
	alu[tmpx, in_addrtbuf, +, BYTES_FOR_L2_HEADER]
	br=byte[l2_header_passed, 0, 1, no_need_to_change#]	
		alu[tmpx, --, b, in_addrtbuf]
	no_need_to_change#:

	// prepare tbuf address for  dram[tbuf_wr......]
	alu[out_indir, in_indirbase, or, tmpx, <<5]
	#endif	// #ifdef ETHERNET_TX
#endif // #ifndef ADD_L2_HEADER

	// get buffer handle 
	alu_shf[tmpx, --, b, in_bd, <<2]
    // use following dl_buf_get_data_from_meta macro from dispatch_loop.uc to
	// get dram address at the beginning of the buffer from true_dl_buf_hdl
	// now it takes one instruction
    dl_buf_get_data_from_meta[out_dramaddr, tmpx]

	// total we need to fecth (offsetmod8 + paylo) bytes
	// to tbuf
	alu[tmpx, in_offsetmod8, +, in_paylo]

    // since ref count 0 will fectch 8 bytes, and ref count 1 will fetch
	// 16 bytes......, and ref count 15 will fetch 128 bytes, 
	// the right way to do it is followinhg:
	//    a) decrease tm_payload_len by one
	//    b) then right shift the result by 3
	// We will do step a first, then combine the right shift in the shift into
	// the ref count in indir_ref.  In this way, we make sure that we fetch 
	// enough and closet to 8 byte alignment of packet data	
	alu[tmpx, tmpx, -, 0x01]	
	// right shift tm_payload_len to get rid of any bits set in bit 0 to 3 to 
	// get ref_cnt for indir_ref 
    alu_shf[tmpx, --, b, tmpx, >>3]
    // left shift 21 bit to ref_cnt location
    alu[--, out_indir, or, tmpx, <<21] 

    // how many bytes of packet payload are copied to tbuf from dram is 
	// controlled by Ref Cnt field in indir_ref, following max_16 is
	// dummy, offset is added to dramaddr to get the actual address to start 
	// fetch, and the lower 3 bits of the address is ignored by dram channel	 
    dram[tbuf_wr, --, out_dramaddr, in_of_set, max_16], indirect_ref, \
		sig_done[in_sig_dram]
.end // tmpx
#endm // end of macro _sphy_mphy4_tx_move_sop_paylo_to_tbuf()

///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_set_tcw0_with_paylo_len_offset_chnl()
//
// Description: set payload_length, payload_offset, and channel fields in 
//              trnsmit control word
//
// Outputs:							
//							None
//
// Inputs/Outputs:  
//		io_tcw:				transmit control word
// 
// Inputs:  
//		in_paylo_len:		payload length
//		in_paylo_offse:		payload offset
//		in_chnl:			port number
//
// Constants:
//		TCW0_PAYLOAD_LEN_LSB_LOC:	payload length field location in transmit 
//									control word   
//		TCW0_PAYLOAD_OFFSET_LSB_LOC:	payload offset field location in  
//										transmit contrl word     
//
// Size:
//		3 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_sphy_mphy4_tx_set_tcw0_with_paylo_len_offset_chnl(io_tcw, in_paylo_len, \
													in_paylo_offset, in_chnl)
	.begin
	.reg tmpxy
	alu[tmpxy, --, b, in_paylo_len]
	alu_shf[io_tcw, io_tcw, or, tmpxy, <<TCW0_PAYLOAD_LEN_LSB_LOC]
	.end
	alu_shf[io_tcw, io_tcw, or, in_paylo_offset, <<TCW0_PAYLOAD_OFFSET_LSB_LOC]
	alu[io_tcw, io_tcw, or, in_chnl]					; set channel field
#endm // end of macro _sphy_mphy4_tx_set_tcw0_with_paylo_len_offset_chnl()


///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_set_tcw0_with_paylo_len_offset()
//
// Description: set payload_length, payload_offset, and channel fields in 
//              trnsmit control word
//
// Outputs:							
//							None
//
// Inputs/Outputs:  
//		io_tcw:				transmit control word
// 
// Inputs:  
//		in_paylo_len:		payload length
//		in_paylo_offse:		payload offset
//
// Constants:
//		TCW0_PAYLOAD_LEN_LSB_LOC:	payload length field location in transmit 
//									control word   
//		TCW0_PAYLOAD_OFFSET_LSB_LOC:	payload offset field location in  
//										transmit contrl word     
//
// Size:
//		2 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_sphy_mphy4_tx_set_tcw0_with_paylo_len_offset(io_tcw, in_paylo_len, \
													in_paylo_offset)
	alu_shf[io_tcw, io_tcw, or, in_paylo_len, <<TCW0_PAYLOAD_LEN_LSB_LOC]
	alu_shf[io_tcw, io_tcw, or, in_paylo_offset, <<TCW0_PAYLOAD_OFFSET_LSB_LOC]
#endm // end of macro _sphy_mphy4_tx_set_tcw0_with_paylo_len_offset()


///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_save_sop_meta_to_lm()
//
// Description: save sop meta data to local memory
//
// Outputs: 
//								none
//
// Inputs:  
//			in_txr_lw0:			tx request from Queue Manager 
//
// Constants:
//								none
//
//	Size: 10 instructions. (worst case)
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_save_sop_meta_to_lm(in_txr_lw0)
.begin 
.reg v_buf_size whole_dl_meta0
	dl_meta_get_buffer_next[whole_dl_meta0]
	ld_field_w_clr[*l$index0[SBD_0_OFFSET], 0111, whole_dl_meta0]	; get sbd
	alu[--, *l$index0[SBD_0_OFFSET], -, IX_NULL]	; is null (0xFF)?
	beq[finish_save_sop_meta_to_lm#], defer[3]
		ld_field_w_clr[*l$index0[ABD_0_OFFSET], 0111, in_txr_lw0]	; set ab_bd
		alu_shf[*l$index0[ABD_0_OFFSET], *l$index0[ABD_0_OFFSET], or, \
				1, <<SOP_BIT_LOC]
 	  	//update sop buffer size, offset
		dl_meta_get_offset[*l$index0[ABD_1_OFFSET]]
	 
	alu_shf[*l$index0[ABD_0_OFFSET], *l$index0[ABD_0_OFFSET], or, 1, \
			<<RD_NBD_BIT_LOC]	; set ab_rd_nbd_flag
finish_save_sop_meta_to_lm#:
	dl_meta_get_buffer_size[v_buf_size]
	ld_field[*l$index0[ABD_1_OFFSET], 1100, v_buf_size, <<PAYLO_RMND_LOC] 
	alu[*l$index1[GLOBAL_PKTS_IN_QUEUE_INDEX], \
				*l$index1[GLOBAL_PKTS_IN_QUEUE_INDEX], +, 1]

.end // v_buf_size whole_dl_meta0

#endm // end of #macro _sphy_mphy4_tx_save_sop_meta_to_lm()

///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_read_sb_meta_data_from_sram()  
//
// Description: read the first to third longwords of meta data from sram
//
// Outputs:							
//		out_xmeta:			the sram transfer registers to keep meta data 
//
// Inputs/Outputs:  
//		io_sigmask:			signal mask to be added with sig_sram_read_1
// 
// Inputs:  
//		in_buf_handle:		buffer handle to get meta data
//
// Constants:
//		none   
//
// Size:
//		4 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_read_sb_meta_data_from_sram(out_xmeta, io_sigmask, \
								in_buf_handle)
	// add sig_sram_read_1 to io_sigmask
	_sphy_mphy4_tx_set_signal(io_sigmask, sig_sram_read_1)	; sig_sram_read_1
	// read meta data by dispatch_loop macro
	dl_meta_load_cache(in_buf_handle, out_xmeta, sig_sram_read_1, 0, 3)

#endm // end of macro _sphy_mphy4_tx_read_sb_meta_data_from_sram()


/////////////////////////////////////////////////////////////////////////////////////
// 
// _sphy_mphy4_tx_check_tbuf_full_sop_eop()
//
// Description:
// 	
//		Check whether tbuf is full to avoid overwritting tbuf, also utilizes
//		defer slots to compose tcw0 for sop_eop mpkt
//
// Outputs: 
//		out_tcw_addr	Validation address for this tbuf element
//
// Inputs/Outputs:
//		io_tcw0			tcw0 for this transmit
//		io_tbufele:		tbuf element number which will be used for tx
//
// Inputs:  
//		in_paylo		payload length for tcw0
//
// Constants:
//		None
//
// Labels:	
//		TBUF_FULL_LABEL	symbolic address to wait until tbuf not full								not full 
//	
// Size: 
//
//		9 instructions
// 
//
////////////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_update_tcw_sop_eop(io_tcw0, io_tbufele, in_paylo, )
.begin	
.reg result_reg tmpxy
	//alu[result_reg, TBUF_ELE_BITS_MASK, AND, mpkts_sent0]	; get mpkts actually sent
	//alu[--, io_tbufele, -, result_reg]
	//bge[cont_comp_tbuf_tx_sequnce#], defer[3]
		// set payload_length in io_tcw0 with in_paylo 
		alu[tmpxy, --, b, in_paylo]
		alu_shf[io_tcw0, io_tcw0, OR, tmpxy, <<TCW0_PAYLOAD_LEN_LSB_LOC]

//	alu[io_tbufele, io_tbufele, +, TBUF_ELE_NUM_PER_SEGMENT]	; wrap around, add tbuf element number
//cont_comp_tbuf_tx_sequnce#:
//	alu[result_reg, io_tbufele, -, result_reg]	; compare with totals in tbuf
//	alu[--, result_reg, -, TBUF_WAIT_THRESHOLD]	; compare with threshold
//	bge[TBUF_FULL_LABEL] 								; go to wait  
.end //result_reg tmpy

#endm // end of #macro _sphy_mphy4_tx_check_tbuf_full_sop_eop()


/////////////////////////////////////////////////////////////////////////////////////
// 
// _sphy_mphy4_tx_check_tbuf_full_sop_eop()
//
// Description:
// 	
//		Check whether tbuf is full to avoid overwritting tbuf, also utilizes
//		defer slots to compose tcw0 for sop_eop mpkt
//
// Outputs: 
//		out_tcw_addr	Validation address for this tbuf element
//
// Inputs/Outputs:
//		io_tcw0			tcw0 for this transmit
//		io_tbufele:		tbuf element number which will be used for tx
//
// Inputs:  
//		in_paylo		payload length for tcw0
//
// Constants:
//		None
//
// Labels:	
//		TBUF_FULL_LABEL	symbolic address to wait until tbuf not full								not full 
//	
// Size: 
//
//		9 instructions
// 
//
////////////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_check_tbuf_full_sop_eop(out_tcw_addr, io_tcw0, \
			io_tbufele, in_paylo, TBUF_FULL_LABEL, mpkts_sent0)
.begin	
.reg result_reg tmpxy
	alu[result_reg, TBUF_ELE_BITS_MASK, AND, mpkts_sent0]	; get mpkts actually sent
	alu[--, io_tbufele, -, result_reg]
	bge[cont_comp_tbuf_tx_sequnce#], defer[3]
		// set payload_length in io_tcw0 with in_paylo 
		alu_shf[io_tcw0, io_tcw0, OR, in_paylo, <<TCW0_PAYLOAD_LEN_LSB_LOC]
#if IS_IXPTYPE(__IXP28XX)
		alu_shf[tmpxy, --, B, io_tbufele, <<ELE_2_TX_CNTRL_ADDR]
#else // #ifdef IXP2800
		alu_shf[tmpxy, --, B, io_tbufele, <<3]
#endif // #ifdef IXP2800
		alu[out_tcw_addr, tmpxy, +, *l$index1[GLOBAL_CTW_VD_BASE_INDEX]]
		
	alu[io_tbufele, io_tbufele, +, TBUF_ELE_NUM_PER_SEGMENT]	; wrap around, add tbuf element number
cont_comp_tbuf_tx_sequnce#:
	alu[result_reg, io_tbufele, -, result_reg]	; compare with totals in tbuf
	alu[--, result_reg, -, TBUF_WAIT_THRESHOLD]	; compare with threshold
	bge[TBUF_FULL_LABEL] 								; go to wait  
.end //result_reg tmpy

#endm // end of #macro _sphy_mphy4_tx_check_tbuf_full_sop_eop()



/////////////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_sop_eop_wait_till_tbuf_not_full()
//
// Description:
// 	
//		Wait till tbuf not full to avoid overwritting
//
// Outputs: 
//								none
//
// Inputs:  
//		in_tbufele:				tbuf element number which will be used for tx
//
// Constants:
//								none
//
// Labels:
//		READTY_TO_TRANSMIT_LABEL	symbolic address to continue trnasmit after 
//									tbuf not full 
//
// Size: 
//		uncertain, depend on hardware performance
//
////////////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_sop_eop_wait_till_tbuf_not_full[in_tbufele, \
			 READTY_TO_TRANSMIT_LABEL]

.begin	
.reg result tmp_tbuf
	// in sop_eop case, in_tbufele maybe set to be over TBUF_ELE_NUM_PER_SEGMENT
	// in _sphy_mphy4_tx_check_tbuf_full_sop_eop, corect it first
	alu[in_tbufele, in_tbufele, AND, TBUF_ELE_BITS_MASK]

read_tx_seuence_loop#:
	// read the actual number of mpkts out of tbuf
	msf[read, $mpkts_sent0, addr_tx_seq, 0, 1], sig_done[sig_msf_access_3]	
wait_tx_seq_read_done#:
	nop
	br_!signal[sig_msf_access_3, wait_tx_seq_read_done#]	
	alu[result, TBUF_ELE_BITS_MASK, AND, $mpkts_sent0]	; get mpkts actually sent
	alu[--, in_tbufele, -, result]
	bge[cont_comp_tbuf_tx_sequnce#], defer[1]
		alu[tmp_tbuf, --, b, in_tbufele]		
	alu[tmp_tbuf, tmp_tbuf, +, TBUF_ELE_NUM_PER_SEGMENT]	; wrap around, add tbuf element number
cont_comp_tbuf_tx_sequnce#:
	alu[result, tmp_tbuf, -, result]			; compare with totals in tbuf
	alu[--, result, -, TBUF_WAIT_THRESHOLD]		; compare with threshold
	bge[read_tx_seuence_loop#] 					; loop again
	br[READTY_TO_TRANSMIT_LABEL] 			 
	.end //result tmp_tbuf

#endm // end of #macro _sphy_mphy4_tx_sop_eop_wait_till_tbuf_not_full()

/////////////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_wait_till_tbuf_not_full()
//
// Description:
// 	
//		Wait till tbuf not full to avoid overwritting
//
// Outputs: 
//								none
//
// Inputs:  
//		in_tbufele:				tbuf element number which will be used for tx
//
// Constants:
//								none
//
// Labels:
//		READTY_TO_TRANSMIT_LABEL	symbolic address to continue trnasmit after 
//									tbuf not full 
//
// Size: 
//		uncertain, depend on hardware performance
//
////////////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_wait_till_tbuf_not_full[in_tbufele, READTY_TO_TRANSMIT_LABEL]

.begin	
.reg result tmp_tbuf

read_tx_seuence_loop#:
	// read the actual number of mpkts out of tbuf
	msf[read, $mpkts_sent0, addr_tx_seq, 0, 1], sig_done[sig_msf_access_3]	
wait_tx_seq_read_done#:
	nop
	br_!signal[sig_msf_access_3, wait_tx_seq_read_done#]	
	alu[result, TBUF_ELE_BITS_MASK, AND, $mpkts_sent0]	; get mpkts actually sent
	alu[--, in_tbufele, -, result]
	bge[cont_comp_tbuf_tx_sequnce#], defer[1]
		alu[tmp_tbuf, --, b, in_tbufele]		
	alu[tmp_tbuf, tmp_tbuf, +, TBUF_ELE_NUM_PER_SEGMENT]	; wrap around, add tbuf element number
cont_comp_tbuf_tx_sequnce#:
	alu[result, tmp_tbuf, -, result]			; compare with totals in tbuf
	alu[--, result, -, TBUF_WAIT_THRESHOLD]		; compare with threshold
	bge[read_tx_seuence_loop#] 					; loop again
	br[READTY_TO_TRANSMIT_LABEL] 			 
	.end //result tmp_tbuf

#endm // end of #macro _sphy_mphy4_tx_wait_till_tbuf_not_full()

/////////////////////////////////////////////////////////////////////////////////////
// 
// _sphy_mphy4_tx_check_tbuf_full_not_sop_eop()
//
// Description:
// 	
//		Check whether tbuf is full to avoid overwritting tbuf
//
// Outputs: 
//		None
//
// Inputs/Outputs:
//		None
//
// Inputs:  
//		in_tbufele:		tbuf element number which will be used for tx
//
// Constants:
//		None
//
// Labels:
//		TBUF_FULL_LABEL	symbolic address to wait until tbuf not full								not full 

// Size: 
//		10 instructions
// 
////////////////////////////////////////////////////////////////////////////////////

#macro _sphy_mphy4_tx_check_tbuf_full_not_sop_eop(in_tbufele, TBUF_FULL_LABEL, mpkts_sent0)
.begin	
.reg result_reg tmp_tbuf
	alu[result_reg, TBUF_ELE_BITS_MASK, AND, mpkts_sent0]	; get mpkts actually sent
	alu[--, in_tbufele, -, result_reg]
	bge[cont_comp_tbuf_tx_sequnce#], defer[1]
		alu[tmp_tbuf, --, b, in_tbufele]
	
	alu[tmp_tbuf, tmp_tbuf, +, TBUF_ELE_NUM_PER_SEGMENT]	; wrap around, add tbuf element number
cont_comp_tbuf_tx_sequnce#:
	alu[result_reg, tmp_tbuf, -, result_reg]	; compare with totals in tbuf
	alu[--, result_reg, -, TBUF_WAIT_THRESHOLD]	; compare with threshold
	bge[TBUF_FULL_LABEL] 							; go to wait  
.end //result_reg tmpy
#endm // end of #macro _sphy_mphy4_tx_check_tbuf_full_not_sop_eop()

/////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_update_ab_paylo_rmnd_and_offset_rpaylo()
//
// Description: update ab_paylo_rmnd and ab_offset_rpaylo in queue entry 
//
//
// Outputs:							
//							None
//
// Inputs/Outputs:  
//		io_mpkt_paylo_len:	payload bytes will transmitted by this mpacket from 
// 							the active buffer
//
// Inputs: 
//							None 
//
// Constants:
//		PAYLO_RMND_LOC:		location of the least significant bit of the 
//							remained payload in queue entry 
//
// Size:
//		3 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_update_ab_paylo_rmnd_and_offset_rpaylo(io_mpkt_paylo_len)
#ifndef AVOID_USING_SKIP_BIT_IN_TCW0
.begin
.reg tmpx
	alu[*l$index0[ABD_1_OFFSET], *l$index0[ABD_1_OFFSET], +, io_mpkt_paylo_len]
	alu_shf[tmpx, --, b, io_mpkt_paylo_len, <<PAYLO_RMND_LOC]
	alu[*l$index0[ABD_1_OFFSET], *l$index0[ABD_1_OFFSET], -, tmpx]
.end // tmpx
#else //#ifndef AVOID_USING_SKIP_BIT_IN_TCW0
// adjust current mpkt size to make sure next mpkt in this buffer always have
// more than 4 bytes payload, so SKIT bit never set in TCW0, this only happens
// in SPHY_1x32 mode  
.begin
.reg next_mpkt_len tmpx
    ld_field_w_clr[tmpx, 0011, *l$index0[ABD_1_OFFSET], >>PAYLO_RMND_LOC]
	alu[next_mpkt_len, tmpx, -, io_mpkt_paylo_len]
	alu[--, next_mpkt_len, -, BUS_WIDTH_IN_BYTES]
	bge[adjust_this_mpkt_to_avoid_using_skip_bit_done#] 
	alu[io_mpkt_paylo_len, io_mpkt_paylo_len, -, BUS_WIDTH_IN_BYTES]
adjust_this_mpkt_to_avoid_using_skip_bit_done#:
	alu[*l$index0[ABD_1_OFFSET], *l$index0[ABD_1_OFFSET], +, io_mpkt_paylo_len]
	alu_shf[tmpx, --, b, io_mpkt_paylo_len, <<PAYLO_RMND_LOC]
	alu[*l$index0[ABD_1_OFFSET], *l$index0[ABD_1_OFFSET], -, tmpx]
.end // next_mpkt_len tmpx
#endif //#ifndef AVOID_USING_SKIP_BIT_IN_TCW0
#endm // end of _sphy_mphy4_tx_update_ab_paylo_rmnd_and_offset_rpaylo()


///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_save_sbd_meta_to_lm()
//
// Description: save secodary buffer data to local memory
//
// Outputs:							
//							None
//
// Inputs:  
//							None
//
// Constants:
//							None  
//
// Size: 7 instructions. (Worst case cycle count)
//
/////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_save_sbd_meta_to_lm()
.begin 
.reg v_buf_size tmp_nsbd tmp_whole
	dl_meta_get_buffer_next[tmp_whole]
	; only need lower three bytes
	ld_field_w_clr[tmp_nsbd, 0111, tmp_whole]
	alu[*l$index1[GLOBAL_SBD_NEXT_BD_INDEX], --, b, tmp_nsbd]
	alu[--, *l$index1[GLOBAL_SBD_NEXT_BD_INDEX], -, IX_NULL]	; is null (0xFF)?
	beq[finish_save_sbd_meta_to_lm#], defer[2]
 	  	//update secondary buffer size, offset always zero
		dl_meta_get_buffer_size[v_buf_size]
		ld_field_w_clr[*l$index0[SBD_1_OFFSET], 1100, v_buf_size, <<PAYLO_RMND_LOC] 
	 
	alu_shf[*l$index0[SBD_0_OFFSET], *l$index0[SBD_0_OFFSET], or, 1, <<RD_NBD_BIT_LOC]	; set ab_rd_nbd_flag
finish_save_sbd_meta_to_lm#:

.end // v_buf_size tmp_nsbd tmp_whole
#endm // end of #macro _sphy_mphy4_tx_save_sbd_meta_to_lm() 

///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_secondary_buffer_become_active_buffer[]
//
// Description: replace active buffer data with secondary buffer data in  
//              queue entry
//
// Outputs:							
//							None
//
// Inputs:  
//							None
//
// Constants:
//							None 
//
// Size: 
//		5 instructions. (Worst case cycle count)
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_secondary_buffer_become_active_buffer()
.begin
	alu[*l$index0[ABD_0_OFFSET], --, b, *l$index0[SBD_0_OFFSET]]
	alu[*l$index0[ABD_1_OFFSET], --, b, *l$index0[SBD_1_OFFSET]]
	alu[*l$index0[SBD_0_OFFSET], --, b, *l$index1[GLOBAL_SBD_NEXT_BD_INDEX]]
	alu[*l$index0[SBD_1_OFFSET], --, b, 0]		; reset
	alu[*l$index1[GLOBAL_SBD_NEXT_BD_INDEX], --, b, IX_NULL]	; reset
.end
#endm // end of #macro _sphy_mphy4_tx_secondary_buffer_become_active_buffer()

///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_move_not_sop_paylo_to_tbuf() 
//
// Description: move payload of not_sop mpacket to tbuf 
//
// Outputs:							
// 		out_indir:			indirect reference used in dram[tbuf_wr...] 
//							instruction
//		out_dramaddr:		dram address of the payload
//
// Inputs:  
//		in_bd:				meta data handle for active buffer 
//		in_addrtbuf:		the starting address of tbuf for payload 
//		in_indirbase:		base for  indir
//      in_offset: 			the starting address of the payload to copy
//		in_tcw0:			transmit control word which already contains 
//							payload length and payload offset
//		in_sig_dram:		signal number for dram[tbuf_wr...] instruction
//
// Constants:
//							None   
//
// Size:
//		11 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_sphy_mphy4_tx_move_not_sop_paylo_to_tbuf(out_indir, out_dramaddr, in_bd, \
			in_addrtbuf, in_indirbase, in_offset, in_tcw0, in_sig_dram) 
.begin
.reg t_paylo_len t_paylo_offset tmpx
	// prepare tbuf address for  dram[tbuf_wr......]
	alu[out_indir, in_indirbase, or, in_addrtbuf, <<5]

	ld_field_w_clr[t_paylo_len, 0001, in_tcw0, >>TCW0_PAYLOAD_LEN_LSB_LOC]
	ld_field_w_clr[t_paylo_offset, 0001, in_tcw0, >>TCW0_PAYLOAD_OFFSET_LSB_LOC]
	alu[t_paylo_offset, t_paylo_offset, and, TCW0_PAYLOAD_OFFSET_MASK]

	// get true bd  
	alu_shf[tmpx, --, b, in_bd, <<2]
    // use following dl_buf_get_data_from_meta macro from dispatch_loop.uc to
	// get dram address at the beginning of the buffer from true_dl_buf_hdl
	// now it takes one instruction
    dl_buf_get_data_from_meta[out_dramaddr, tmpx]

	// total we need to fecth (t_paylo_len + t_paylo_offset) bytes
	// to tbuf
	alu[tmpx, t_paylo_len, +, t_paylo_offset]

    // since ref count 0 will fectch 8 bytes, and ref count 1 will fetch
	// 16 bytes......, and ref count 15 will fetch 128 bytes, 
	// the right way to do it is followinhg:
	//    a) decrease tm_payload_len by one
	//    b) then right shift the result by 3
	// We will do step a first, then combine the right shift in the shift into
	// the ref count in indir_ref.  In this way, we make sure that we fetch 
	// enough and closet to 8 byte alignment of packet data	
	alu[tmpx, tmpx, -, 0x01]	
	// right shift tm_payload_len to get rid of any bits set in bit 0 to 3 to 
	// get ref_cnt for indir_ref 
    alu_shf[tmpx, --, b, tmpx, >>3]
    // left shift 21 bit to ref_cnt location
    alu[--, out_indir, or, tmpx, <<21] 

    // how many bytes of packet payload are copied to tbuf from dram is 
	// controlled by Ref Cnt field in indir_ref, following max_16 is
	// dummy, offset is added to dramaddr to get the actual address to start 
	// fetch, and the lower 3 bits of the address is ignored by dram channel	 
    dram[tbuf_wr, --, out_dramaddr, in_offset, max_16], indirect_ref, \
													sig_done[in_sig_dram]
.end // t_paylo_len t_paylo_offse tmpx
#endm // end of macro _sphy_mphy4_tx_move_not_sop_paylo_to_tbuf()

///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_free_buffer()
//
// Description: free buffer handle by dispatch loop macro dl_buf_free
//
// Outputs:							
// 		none
//
// Inputs:  
//		in_buf_handle:		buffer handle to free 
//
// Constants:
//		none   
//
// Size:
//		4 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro	_sphy_mphy4_tx_free_buffer(in_buf_handle, buf_free_label)
.begin
	// free buffer handle  
#ifdef UNIT_TEST
	alu[@buf_freed, @buf_freed, +, 1]
#endif //UNIT_TEST

#ifndef SKIP_FREE_BD
     // use dl_buf_free in dl_buf.uc in dispatch_loop to free buffer handle
writing_to_nn_ring#:
	br_inp_state[NN_FULL, buf_free_label#]
	
	alu[*n$index++, in_buf_handle, or, 0x3, <<30]

exit#:

#endif //#ifndef SKIP_FREE_BD
.end	

#endm // end of #macro _sphy_mphy4_tx_free_buffer()

#ifdef COUNTERS
///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_update_counters_sop_eop()
//
// Description: update the bytes tranmitted by mpkt with SOP bit set in tcw
//
// Outputs:							
//		out_bytecnt			sram xfer register to keep the value to add to 
// 							counters in sram
//
// Inputs:  
// 		in_tcw:				transmit control word which has payload length 
//                          and prepend length
//		in_sram_add_sig:	signal for sram add operation
//
// Constants:
//							None 
//
// Size: 
//		10 instructions.
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_update_counters_sop_eop(out_bytecnt, in_tcw, \
					in_sram_add_sig)
.begin
.reg port bytes_txed paylo cntr_addr
	alu[port, --, B, 0]	; get port
	alu_shf[cntr_addr, --, b, port, <<4]			; get counter addr
	alu[cntr_addr, cntr_addr, +, @cntr_base]		; add base
	alu_shf[bytes_txed, --, b, in_tcw, >>TCWO_PREPEND_LEN_LSB_LOC] ; prepend
	sram[incr, --, cntr_addr, PKT_TRANSMITTED_CNT_IDX]	; increment pkts transmitted
	alu[bytes_txed, bytes_txed, and, 0x1F]			; mask out
	alu_shf[paylo, --, b, in_tcw, >>TCW0_PAYLOAD_LEN_LSB_LOC] ; payload
	alu[bytes_txed, bytes_txed, +, paylo]
	alu[out_bytecnt, --, B, bytes_txed]				; get bytes txed
	sram[add, out_bytecnt, cntr_addr, BYTE_TRANSMITTED_CNT_IDX],\
	     sig_done[in_sram_add_sig]			 	
.end //	port bytes_txed paylo cntr_addr 	 
#endm // end of macro _sphy_mphy4_tx_update_counters_sop_eop()


///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_update_counters_sop_only()
//
// Description: update the bytes tranmitted by mpkt with SOP bit set in tcw
//
// Outputs:							
//		out_bytecnt			sram xfer register to keep the value to add to 
// 							counters in sram
//
// Inputs:  
// 		in_tcw:				transmit control word which has payload length 
//                          and prepend length
//		in_sram_add_sig:	signal for sram add operation
//
// Constants:
//							None 
//
// Size: 
//		9 instructions.
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_update_counters_sop_only(out_bytecnt, in_tcw, \
					in_sram_add_sig)
.begin
.reg port bytes_txed paylo cntr_addr
	alu[port, --, B, 0]	; get port
	alu_shf[cntr_addr, --, b, port, <<4]			; get counter addr
	alu[cntr_addr, cntr_addr, +, @cntr_base]		; add base
	alu_shf[bytes_txed, --, b, in_tcw, >>TCWO_PREPEND_LEN_LSB_LOC] ; prepend
	alu[bytes_txed, bytes_txed, and, 0x1F]			; mask out
	alu_shf[paylo, --, b, in_tcw, >>TCW0_PAYLOAD_LEN_LSB_LOC] ; payload
	alu[bytes_txed, bytes_txed, +, paylo]
	alu[out_bytecnt, --, B, bytes_txed]				; get bytes txed
	sram[add, out_bytecnt, cntr_addr, BYTE_TRANSMITTED_CNT_IDX],\
	     sig_done[in_sram_add_sig]			 	
.end //	port bytes_txed paylo cntr_addr 	 
#endm // end of macro _sphy_mphy4_tx_update_counters_sop_only()

///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_update_counters_not_sop()
//
// Description: update the bytes tranmitted by mpkt without SOP bit set in tcw
//
// Outputs:							
//		out_bytecnt			sram xfer register to keep the value to add to 
// 							counters in sram
//
// Inputs:  
// 		in_tcw:				transmit control word which has payload length 
//                          and prepend length
//		in_sram_add_sig:	signal for sram add operation
//
// Constants:
//							None 
//
// Size: 
//		15 instructions. (Worst case cycle count)
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_update_counters_not_sop(out_bytecnt, in_tcw, \
					in_sram_add_sig)
.begin
.reg port bytes_txed paylo cntr_addr

	// SKIP bit set in in_tcw?
    br_bset[in_tcw, SKIP_TRANSMIT_FLAG_BIT, bytes_txed_update_done#], defer[3]
		alu[cntr_addr, --, b, @cntr_base]	; dummy for SKIP case
		alu[out_bytecnt, --, b, 0]				; dummy  for SKIP case
		alu[port, --, B, 0]	; get port
	alu_shf[cntr_addr, --, b, port, <<4]			; get counter addr
	alu[cntr_addr, cntr_addr, +, @cntr_base]		; add base
 
    br_bclr[in_tcw, TCW0_EOP_BIT_LOC, pkt_txed_update_done#], defer[3]
		alu_shf[bytes_txed, --, b, in_tcw, >>TCWO_PREPEND_LEN_LSB_LOC] ; prepend
		alu[bytes_txed, bytes_txed, and, 0x1F]			; mask out
		alu_shf[paylo, --, b, in_tcw, >>TCW0_PAYLOAD_LEN_LSB_LOC] ; payload
	sram[incr, --, cntr_addr, PKT_TRANSMITTED_CNT_IDX]	; increment pkts transmitted
pkt_txed_update_done#:

	alu[bytes_txed, bytes_txed, +, paylo]
	alu[out_bytecnt, --, B, bytes_txed]				; get bytes txed
bytes_txed_update_done#:
	sram[add, out_bytecnt, cntr_addr, BYTE_TRANSMITTED_CNT_IDX],\
	     sig_done[in_sram_add_sig]			 	
.end //	port bytes_txed paylo cntr_addr 	 
#endm // end of macro _sphy_mphy4_tx_update_counters_not_sop()
	
#endif // COUNTERS



// macro to update count value depending on calendar flow control
#macro update_ok_to_schedule( io_go_for_cnt, new_calendar_seq_no, old_calendar_seq_no, \
				thd_timestamp, timestamp_old )
.begin
.reg diff tmp
	// check if count is new enough
	alu[--, thd_timestamp, -, timestamp_old]
	// using these 2 brances because ble does not work here in case of overflow
	beq[not_valid#]
	bmi[not_valid#]

	alu[timestamp_old, --, b, thd_timestamp]
	alu[tmp, new_calendar_seq_no, AND~, 1, <<31]; clear bit 31
	alu[diff, tmp, -, old_calendar_seq_no]
	
	#define_eval CONST_TMVE_M_TSMC_MASK	(TX_SEQUENCE_MAX_CNT - 1)
	.reg seq_max_mask
	move(seq_max_mask, CONST_TMVE_M_TSMC_MASK);
	alu[diff, diff, and, seq_max_mask];
	alu[old_calendar_seq_no, --, b, tmp]
	alu[io_go_for_cnt, io_go_for_cnt, +, diff]


	/*
	bge[end_of_macro#], defer[2]
		alu[old_calendar_seq_no, --, b, tmp]
		alu[io_go_for_cnt, io_go_for_cnt, +, diff]
	wrap#:
	.begin
		#define_eval CONST_TMVE_M_TSMC	(TX_SEQUENCE_MAX_CNT)
		.reg seq_max
		move(seq_max, CONST_TMVE_M_TSMC);
		#undef CONST_TMVE_M_TSMC
		alu[io_go_for_cnt, io_go_for_cnt, +, seq_max]
		br[end_of_macro#]
	.end
	*/

	not_valid#:

	//alu[--, --, b, io_go_for_cnt]

	end_of_macro#:
.end
#endm


// new macro from refdes code
#macro prevent_tbuf_overwrite(	io_go_for_cnt,				\
								in_tx_seq_addr,				\
								IN_TBUF_MAX_VALID_ELEMENTS, old_calendar_seq_no, branch_label, \
								thd_timestamp, current_timestamp, old_timestamp)
	.begin

	// PR4 will fix this
	// for now define @read_tx_seq_mutex_flag at higher level
	// .reg valid_but_not_xmited @read_tx_seq_mutex_flag @last_go_for_cnt @running_cnt_of_assigns
	//.reg valid_but_not_xmited $tx_sequence tx_sequence assignments_made_since_last_call
	.sig sig_read_tx_seq

		
	tbuf_full_loop#:
	// only one thread reads sequence number at a time
	alu[--, --, b, @read_tx_seq_mutex_flag]
	br>0[wait_for_other_thread_to_read#]

	// read tx sequence number
	.reg $cal_queue
	alu[thd_timestamp, current_timestamp, +, 1]
	alu[current_timestamp, current_timestamp, +, 1]

	msf[read, $cal_queue, in_tx_seq_addr, 0, 1], ctx_swap[sig_read_tx_seq], defer[1]
	alu[@read_tx_seq_mutex_flag, --, b, 1]

	

	update_ok_to_schedule( io_go_for_cnt, $cal_queue , old_calendar_seq_no, thd_timestamp, old_timestamp)
	
	alu[--, --, b, io_go_for_cnt]
	bgt[branch_label], defer[1]
		alu[@read_tx_seq_mutex_flag, --, b, 0]
	br[tbuf_full_loop#]

	
	// -------------------------------------------------------------------------------		
	// Exception processing: If another thead is already reading, then wait here for 
	// it to complete
	wait_for_other_thread_to_read#:
		// we got into this macro because @ok_to_schedule_cnt went negative.  Bounce it back up to zero.
		add(io_go_for_cnt, io_go_for_cnt, 1);
	
	wait_for_other_thread_to_read_loop#:
		ctx_arb[voluntary]
		alu[--, --, b, @read_tx_seq_mutex_flag]
		br>0[wait_for_other_thread_to_read_loop#]

		sub(io_go_for_cnt, io_go_for_cnt, 1)
		bgt[branch_label]
		br[tbuf_full_loop#]
	// -------------------------------------------------------------------------------		


	endm#:
	//alu[@read_tx_seq_mutex_flag, --, b, 0]

	.end
#endm

/////////////////////////////////////////////////////////////////////////////// 
//
// 	_sphy_mphy4_tx_wait_port_ready_for_transmission()
//
// Description:
//		wait for port to be ready for transmission by cheking staus of ports
//
// Outputs: 
//		None
//
// Inputs:  
//		None
//		
// Constants
//		None
//
///////////////////////////////////////////////////////////////////////////////

#macro _sphy_mphy4_tx_wait_port_ready_for_transmission()

.begin

#if IS_IXPTYPE(__IXP28XX)

	.begin 
	.reg	$stat
	.reg 	port_stat_addr
	.sig    sig_msf_read

wait_for_port_ready#:
	
	move(port_stat_addr, TX_PORT_STATUS_0) 

	msf[read, $stat, port_stat_addr,  0, 1], ctx_swap[sig_msf_read]	

	// check port status in starving

	alu[--, $stat, AND, 3]
	bne[wait_for_port_ready#]

	.end	// $stat

#endif	// #if IS_IXPTYPE(__IXP28XX)

.end


#endm	// _sphy_mphy4_tx_wait_port_ready_for_transmission()

///////////////////////////////////////////////////////////////////////////////
//
//	_sphy_mphy4_tx_wait_port_tx_fifo_ready() 
//
// Description: check whether port is ready to receive one or more mpacket 
//				without overwritting mapcket not transmitted yet in tx fifo
//				of the port, if not, wait util ready
//
// Outputs: 
//		io_fc_credit		credit for flow control in unit of tbuf ele size
//
// Inputs:  
//		in_port_stat		port status
//
// Constants:
//		None
//
//	Labels:
//		NORMAL_PATH_LABEL:	symbolic address to go when port is ready to 
//							receive	one or more mpacket
//
//	Size:
//        instructions 
//
///////////////////////////////////////////////////////////////////////////////

#macro _sphy_mphy4_tx_wait_port_tx_fifo_ready(io_fc_credit, in_port_stat, \
															NORMAL_PATH_LABEL)

#ifndef VALIDATE_TBUF_IN_THIRD_TX_ME
#ifdef CHECK_PORT_TX_FIFO_STATUS
.begin 
	.reg	tmp

	alu[tmp, in_port_stat, AND, 3]		; bit 1 and 0, both 0: STARVING
	bne[check_for_hungry_1#]
	alu[io_fc_credit, --, B, STARVING_FC_CREDIT_MINUS_9]	; conservative
	_sphy_mphy4_tx_debug_incr_counter(@is_starving)
	br[NORMAL_PATH_LABEL]

check_for_hungry_1#:
	br!=byte[tmp, 0, HUNGRY_STATUS, read_port_status_loop#] ; not ready

	alu[io_fc_credit, --, B, HUNGRY_FC_CREDIT_MINUS_9]	; conservative
	_sphy_mphy4_tx_debug_incr_counter(@is_hungry)
	br[NORMAL_PATH_LABEL]

read_port_status_loop#:

	alu[--, ind_ref_reg, or, 0]
	msf[read, $port_stat, addr_port_stat, 0, 1], indirect_ref, sig_done[sig_msf_read]

wait_port_stat_read_done#:

	alu[tmp, *$index, AND, 3]		; bit 1 and 0, both 0: STARVING

	bne[check_for_hungry_2#]
	alu[io_fc_credit, --, B, STARVING_FC_CREDIT_MINUS_1]
	_sphy_mphy4_tx_debug_incr_counter(@is_starving)
	br[NORMAL_PATH_LABEL]

check_for_hungry_2#:
	br!=byte[tmp, 0, HUNGRY_STATUS, read_port_status_loop#] ; not ready


	alu[io_fc_credit, --, B, HUNGRY_FC_CREDIT_MINUS_1]
	_sphy_mphy4_tx_debug_incr_counter(@is_hungry)
	br[NORMAL_PATH_LABEL]

.end 
#endif  // #ifdef CHECK_PORT_TX_FIFO_STATUS
#endif	// #ifndef VALIDATE_TBUF_IN_THIRD_TX_ME

#endm	// _sphy_mphy4_tx_wait_port_tx_fifo_ready()


///////////////////////////////////////////////////////////////////////////////
//
//	_sphy_mphy4_tx_update_fc_credit_for_no_tbuf_validation() 
//
// Description: update flow control credit based on the port status read in 
//				previous iteration when there is no new tx request for 
//				later use to optimize the performance
//
// Outputs: 
//		io_fc_credit		credit for flow control in unit of tbuf ele size
//
// Inputs:  
//		in_port_stat		port status
//
// Constants:
//		None
//
//	Size:
//        instructions 
//
///////////////////////////////////////////////////////////////////////////////

#macro _sphy_mphy4_tx_update_fc_credit_for_no_tbuf_validation(io_fc_credit, \
		in_port_stat)

#ifndef VALIDATE_TBUF_IN_THIRD_TX_ME
#ifdef CHECK_PORT_TX_FIFO_STATUS
.begin 
	.reg	tmp

	alu[--, io_fc_credit, -, STARVING_FC_CREDIT_MINUS_8]
	bge[done#]

	alu[tmp, in_port_stat, and, 3]		; bit 1 and 0, both 0: STARVING
	bne[check_for_hungry#]
	alu[io_fc_credit, --, b, STARVING_FC_CREDIT_MINUS_8]
	_sphy_mphy4_tx_debug_incr_counter(@is_starving)
	br[done#]

check_for_hungry#:
	alu[--, io_fc_credit, -, HUNGRY_FC_CREDIT_MINUS_8]
	bgt[done#]

	br!=byte[tmp, 0, HUNGRY_STATUS, done#] 	; not ready

	alu[--, io_fc_credit, -, HUNGRY_FC_CREDIT_MINUS_8]
	_sphy_mphy4_tx_debug_incr_counter(@is_hungry)

done#:
	
.end 
#endif  // #ifdef CHECK_PORT_TX_FIFO_STATUS
#endif	// #ifndef VALIDATE_TBUF_IN_THIRD_TX_ME

#endm	// _sphy_mphy4_tx_update_fc_credit_for_no_tbuf_validation()


#endif 		// __SPHY_MPHY4_TX_UTIL2_UC__

///////////////////////////////////////////////////////////////////////////////

