////////////////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                   
//                                                                      
//     COPYRIGHT (c)  2001-2002 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        
//                                                                      
//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
////////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: sphy_mphy4_tx_util3.uc
//
//      Purpose: Utility macros for 3rd TX ME in Packet (POS/Ethernet) TX
//
///////////////////////////////////////////////////////////////////////////////

#ifndef __SPHY_MPHY4_TX_UTIL3_UC__
#define __SPHY_MPHY4_TX_UTIL3_UC__

///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_set_lmindex0_to_queue_head()
//
// Description: set local memory index 0 to the head of the queue entry to 
//				dequeue one packet
//
// Outputs:					
//		out_tcw:			transmit control word 0
//		out_tbufele:		the next available tbuf element 
//
// Inputs:  
//							None
//
// Constants:
//						 	None 
//
//	Size:
//		4 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_sphy_mphy4_tx_set_lmindex0_to_queue_head()

.begin
	// set local memory index to dequeu a packet at queue head

	local_csr_wr[active_lm_addr_0, *l$index1[GLOBAL_HEAD_OFFSET_INDEX]]
	nop
	nop
	nop
.end
#endm // end of #macro _sphy_mphy4_tx_set_lmindex0_to_queue_head()

	 
///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_advance_queue_head()
//
// Description: advance queue head after the last mpcket of one packet is sent
//              to the port
//
// Outputs:							
//							None
//
// Inputs:  
//      in_turnaround:		queue head/tail offset value for reset offset
//
// Constants:
//						 	None 
//
// Size:
//		6 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_advance_queue_head(in_turnaround)

.begin
continue_advance_queue_head#:
	alu[--, *l$index1[GLOBAL_HEAD_OFFSET_INDEX], -, in_turnaround]
	bne[advance_queue_head_done#], defer[3]
		alu[*l$index1[GLOBAL_PKTS_IN_QUEUE_INDEX], \
			*l$index1[GLOBAL_PKTS_IN_QUEUE_INDEX], -, 1]
		alu[*l$index1[GLOBAL_HEAD_OFFSET_INDEX], \
				*l$index1[GLOBAL_HEAD_OFFSET_INDEX], +, QUEUE_ENTRY_SIZE]
		alu[*l$index1[GLOBAL_MPKTS_TXED_INDEX], \
						*l$index1[GLOBAL_MPKTS_TXED_INDEX], +, 1]
	alu[*l$index1[GLOBAL_HEAD_OFFSET_INDEX], --, B, \
			*l$index1[GLOBAL_QUEUE_INFO_BASE_INDEX]]
advance_queue_head_done#:

.end
#endm // end of macro _sphy_mphy4_tx_advance_queue_head()


///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_set_lmindex0_to_queue_tail()
//
// Description: set local memory index 0 to the tail of the queue entry
//
// Outputs:					
//							None 
//
// Inputs:  
//							None 
//
// Constants:
//						 	None 
//
//	Size:
//		4 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_set_lmindex0_to_queue_tail()
.begin 
	local_csr_wr[active_lm_addr_0, *l$index1[GLOBAL_TAIL_OFFSET_INDEX]]
	nop
	nop
	nop
.end 
#endm // end of #macro _sphy_mphy4_tx_set_lmindex0_to_queue_tail()


///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_advance_queue_tail()
//
// Description: advance queue tail
//
// Outputs:					
//							None 
//
// Inputs:  
//      in_turnaround:		queue head/tail offset value for reset offset
//
// Constants:
//						 	None 
//
//	Size:
//		5 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_advance_queue_tail(in_turnaround)

	// set local memory index to queue entry at queue tail of that port,
	// update ab_bd and p_sop_flag 
.begin 
	alu[--, *l$index1[GLOBAL_TAIL_OFFSET_INDEX], -, in_turnaround]
	bne[set_lmindex0_to_queue_tail_done#], defer[2]
		alu[*l$index1[GLOBAL_TAIL_OFFSET_INDEX], \
			*l$index1[GLOBAL_TAIL_OFFSET_INDEX], +, QUEUE_ENTRY_SIZE]
		alu[*l$index1[GLOBAL_PKTS_IN_QUEUE_INDEX], \
			*l$index1[GLOBAL_PKTS_IN_QUEUE_INDEX], +, 1]
	alu[*l$index1[GLOBAL_TAIL_OFFSET_INDEX], --, B, \
			*l$index1[GLOBAL_QUEUE_INFO_BASE_INDEX]]
set_lmindex0_to_queue_tail_done#:
.end 
#endm // end of #macro _sphy_mphy4_tx_advance_queue_tail()


///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_free_buffer()
//
// Description: free buffer handle by dispatch loop macro dl_buf_free
//
// Outputs:							
// 		none
//
// Inputs:  
//		in_buf_handle:		buffer handle to free 
//
// Constants:
//		none   
//
// Size:
//		4 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro	_sphy_mphy4_tx_free_buffer(in_buf_handle, buf_free_label)
.begin
	// free buffer handle  
#ifdef UNIT_TEST
	alu[@buf_freed, @buf_freed, +, 1]
#endif //UNIT_TEST

#ifndef SKIP_FREE_BD
     // use dl_buf_free in dl_buf.uc in dispatch_loop to free buffer handle
writing_to_nn_ring#:
	br_inp_state[NN_FULL, buf_free_label]
	
	alu[*n$index++, in_buf_handle, or, 0x3, <<30]


exit#:

#endif //#ifndef SKIP_FREE_BD
.end	

#endm // end of #macro _sphy_mphy4_tx_free_buffer()

#ifdef COUNTERS
///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_update_counters_sop_eop()
//
// Description: update the bytes tranmitted by mpkt with SOP bit set in tcw
//
// Outputs:							
//		out_bytecnt			sram xfer register to keep the value to add to 
// 							counters in sram
//
// Inputs:  
// 		in_tcw:				transmit control word which has payload length 
//                          and prepend length
//		in_sram_add_sig:	signal for sram add operation
//
// Constants:
//							None 
//
// Size: 
//		10 instructions.
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_update_counters_sop_eop(out_bytecnt, in_tcw, \
					in_sram_add_sig)
.begin
.reg port bytes_txed paylo cntr_addr
	alu[port, --, B, 0]	; get port
	alu_shf[cntr_addr, --, b, port, <<4]			; get counter addr
	alu[cntr_addr, cntr_addr, +, @cntr_base]		; add base
	alu_shf[bytes_txed, --, b, in_tcw, >>TCWO_PREPEND_LEN_LSB_LOC] ; prepend
	sram[incr, --, cntr_addr, PKT_TRANSMITTED_CNT_IDX]	; increment pkts transmitted
	alu[bytes_txed, bytes_txed, and, 0x1F]			; mask out
	alu_shf[paylo, --, b, in_tcw, >>TCW0_PAYLOAD_LEN_LSB_LOC] ; payload
	alu[bytes_txed, bytes_txed, +, paylo]
	alu[out_bytecnt, --, B, bytes_txed]				; get bytes txed
	sram[add, out_bytecnt, cntr_addr, BYTE_TRANSMITTED_CNT_IDX],\
	     sig_done[in_sram_add_sig]			 	
.end //	port bytes_txed paylo cntr_addr 	 
#endm // end of macro _sphy_mphy4_tx_update_counters_sop_eop()


///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_update_counters_sop_only()
//
// Description: update the bytes tranmitted by mpkt with SOP bit set in tcw
//
// Outputs:							
//		out_bytecnt			sram xfer register to keep the value to add to 
// 							counters in sram
//
// Inputs:  
// 		in_tcw:				transmit control word which has payload length 
//                          and prepend length
//		in_sram_add_sig:	signal for sram add operation
//
// Constants:
//							None 
//
// Size: 
//		9 instructions.
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_update_counters_sop_only(out_bytecnt, in_tcw, \
					in_sram_add_sig)
.begin
.reg port bytes_txed paylo cntr_addr
	alu[port, --, B, 0]	; get port
	alu_shf[cntr_addr, --, b, port, <<4]			; get counter addr
	alu[cntr_addr, cntr_addr, +, @cntr_base]		; add base
	alu_shf[bytes_txed, --, b, in_tcw, >>TCWO_PREPEND_LEN_LSB_LOC] ; prepend
	alu[bytes_txed, bytes_txed, and, 0x1F]			; mask out
	alu_shf[paylo, --, b, in_tcw, >>TCW0_PAYLOAD_LEN_LSB_LOC] ; payload
	alu[bytes_txed, bytes_txed, +, paylo]
	alu[out_bytecnt, --, B, bytes_txed]				; get bytes txed
	sram[add, out_bytecnt, cntr_addr, BYTE_TRANSMITTED_CNT_IDX],\
	     sig_done[in_sram_add_sig]			 	
.end //	port bytes_txed paylo cntr_addr 	 
#endm // end of macro _sphy_mphy4_tx_update_counters_sop_only()

///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_update_counters_not_sop()
//
// Description: update the bytes tranmitted by mpkt without SOP bit set in tcw
//
// Outputs:							
//		out_bytecnt			sram xfer register to keep the value to add to 
// 							counters in sram
//
// Inputs:  
// 		in_tcw:				transmit control word which has payload length 
//                          and prepend length
//		in_sram_add_sig:	signal for sram add operation
//
// Constants:
//							None 
//
// Size: 
//		15 instructions. (Worst case cycle count)
//
///////////////////////////////////////////////////////////////////////////////
#macro _sphy_mphy4_tx_update_counters_not_sop(out_bytecnt, in_tcw, \
					in_sram_add_sig)
.begin
.reg port bytes_txed paylo cntr_addr

	// SKIP bit set in in_tcw?
    br_bset[in_tcw, SKIP_TRANSMIT_FLAG_BIT, bytes_txed_update_done#], defer[3]
		alu[cntr_addr, --, b, @cntr_base]	; dummy for SKIP case
		alu[out_bytecnt, --, b, 0]				; dummy  for SKIP case
		alu[port, --, B, 0]	; get port
	alu_shf[cntr_addr, --, b, port, <<4]			; get counter addr
	alu[cntr_addr, cntr_addr, +, @cntr_base]		; add base
 
    br_bclr[in_tcw, TCW0_EOP_BIT_LOC, pkt_txed_update_done#], defer[3]
		alu_shf[bytes_txed, --, b, in_tcw, >>TCWO_PREPEND_LEN_LSB_LOC] ; prepend
		alu[bytes_txed, bytes_txed, and, 0x1F]			; mask out
		alu_shf[paylo, --, b, in_tcw, >>TCW0_PAYLOAD_LEN_LSB_LOC] ; payload
	sram[incr, --, cntr_addr, PKT_TRANSMITTED_CNT_IDX]	; increment pkts transmitted
pkt_txed_update_done#:

	alu[bytes_txed, bytes_txed, +, paylo]
	alu[out_bytecnt, --, B, bytes_txed]				; get bytes txed
bytes_txed_update_done#:
	sram[add, out_bytecnt, cntr_addr, BYTE_TRANSMITTED_CNT_IDX],\
	     sig_done[in_sram_add_sig]			 	
.end //	port bytes_txed paylo cntr_addr 	 
#endm // end of macro _sphy_mphy4_tx_update_counters_not_sop()
	
#endif // COUNTERS



// macro to update count value depending on calendar flow control
#macro update_ok_to_schedule( io_go_for_cnt, new_calendar_seq_no, old_calendar_seq_no, \
				thd_timestamp, timestamp_old )
.begin
.reg diff tmp
	// check if count is new enough
	alu[--, thd_timestamp, -, timestamp_old]
	// using these 2 brances because ble does not work here in case of overflow
	beq[not_valid#]
	bmi[not_valid#]

	alu[timestamp_old, --, b, thd_timestamp]
	alu[tmp, new_calendar_seq_no, AND~, 1, <<31]; clear bit 31
	alu[diff, tmp, -, old_calendar_seq_no]
	
	#define_eval CONST_TMVE_M_TSMC_MASK	(TX_SEQUENCE_MAX_CNT - 1)
	.reg seq_max_mask
	move(seq_max_mask, CONST_TMVE_M_TSMC_MASK);
	alu[diff, diff, and, seq_max_mask];
	alu[old_calendar_seq_no, --, b, tmp]
	alu[io_go_for_cnt, io_go_for_cnt, +, diff]


	/*
	bge[end_of_macro#], defer[2]
		alu[old_calendar_seq_no, --, b, tmp]
		alu[io_go_for_cnt, io_go_for_cnt, +, diff]
	wrap#:
	.begin
		#define_eval CONST_TMVE_M_TSMC	(TX_SEQUENCE_MAX_CNT)
		.reg seq_max
		move(seq_max, CONST_TMVE_M_TSMC);
		#undef CONST_TMVE_M_TSMC
		alu[io_go_for_cnt, io_go_for_cnt, +, seq_max]
		br[end_of_macro#]
	.end
	*/

	not_valid#:

	//alu[--, --, b, io_go_for_cnt]

	end_of_macro#:
.end
#endm


// new macro from refdes code
#macro prevent_tbuf_overwrite(	io_go_for_cnt,				\
								in_tx_seq_addr,				\
								IN_TBUF_MAX_VALID_ELEMENTS, old_calendar_seq_no, branch_label, \
								thd_timestamp, current_timestamp, old_timestamp)
	.begin

	// PR4 will fix this
	// for now define @read_tx_seq_mutex_flag at higher level
	// .reg valid_but_not_xmited @read_tx_seq_mutex_flag @last_go_for_cnt @running_cnt_of_assigns
	//.reg valid_but_not_xmited $tx_sequence tx_sequence assignments_made_since_last_call
	.sig sig_read_tx_seq

		
	tbuf_full_loop#:
	// only one thread reads sequence number at a time
	alu[--, --, b, @read_tx_seq_mutex_flag]
	br>0[wait_for_other_thread_to_read#]

	// read tx sequence number
	.reg $cal_queue
	alu[thd_timestamp, current_timestamp, +, 1]
	alu[current_timestamp, current_timestamp, +, 1]

	msf[read, $cal_queue, in_tx_seq_addr, 0, 1], ctx_swap[sig_read_tx_seq], defer[1]
	alu[@read_tx_seq_mutex_flag, --, b, 1]

	

	update_ok_to_schedule( io_go_for_cnt, $cal_queue , old_calendar_seq_no, thd_timestamp, old_timestamp)
	
	alu[--, --, b, io_go_for_cnt]
	bgt[branch_label], defer[1]
		alu[@read_tx_seq_mutex_flag, --, b, 0]
	br[tbuf_full_loop#]

	
	// -------------------------------------------------------------------------------		
	// Exception processing: If another thead is already reading, then wait here for 
	// it to complete
	wait_for_other_thread_to_read#:
		// we got into this macro because @ok_to_schedule_cnt went negative.  Bounce it back up to zero.
		add(io_go_for_cnt, io_go_for_cnt, 1);
	
	wait_for_other_thread_to_read_loop#:
		ctx_arb[voluntary]
		alu[--, --, b, @read_tx_seq_mutex_flag]
		br>0[wait_for_other_thread_to_read_loop#]

		sub(io_go_for_cnt, io_go_for_cnt, 1)
		bgt[branch_label]
		br[tbuf_full_loop#]
	// -------------------------------------------------------------------------------		


	endm#:
	//alu[@read_tx_seq_mutex_flag, --, b, 0]

	.end
#endm

/////////////////////////////////////////////////////////////////////////////// 
//
// 	_sphy_mphy4_tx_wait_port_ready_for_transmission()
//
// Description:
//		wait for port to be ready for transmission by cheking staus of ports
//
// Outputs: 
//		None
//
// Inputs:  
//		None
//		
// Constants
//		None
//
///////////////////////////////////////////////////////////////////////////////

#macro _sphy_mphy4_tx_wait_port_ready_for_transmission()

.begin

#if IS_IXPTYPE(__IXP28XX)

	.begin 
	.reg	$stat
	.reg 	port_stat_addr
	.sig    sig_msf_read

wait_for_port_ready#:
	
	move(port_stat_addr, TX_PORT_STATUS_0) 

	msf[read, $stat, port_stat_addr,  0, 1], ctx_swap[sig_msf_read]	

	// check port status in starving

	alu[--, $stat, AND, 3]
	bne[wait_for_port_ready#]

	.end	// $stat

#endif	// #if IS_IXPTYPE(__IXP28XX)

.end


#endm	// _sphy_mphy4_tx_wait_port_ready_for_transmission()


///////////////////////////////////////////////////////////////////////////////
//
//	_sphy_mphy4_tx_wait_port_tx_fifo_ready() 
//
// Description: check whether port is ready to receive one or more mpacket 
//				without overwritting mapcket not transmitted yet in tx fifo
//				of the port, if not, wait util ready
//
// Outputs: 
//		io_fc_credit		credit for flow control in unit of tbuf ele size
//
// Inputs:  
//		in_port_stat		port status
//
// Constants:
//		None
//
//	Labels:
//		NORMAL_PATH_LABEL:	symbolic address to go when port is ready to 
//							receive	one or more mpacket
//
//	Size:
//        instructions 
//
///////////////////////////////////////////////////////////////////////////////

#macro _sphy_mphy4_tx_wait_port_tx_fifo_ready(io_fc_credit, in_port_stat, \
															NORMAL_PATH_LABEL)

#ifdef CHECK_PORT_TX_FIFO_STATUS
.begin 
	.reg	tmp
	.reg read	 $cur_port_stat
	.sig	sig_msf			

	alu[tmp, in_port_stat, AND, 3]		; bit 1 and 0, both 0: STARVING
	bne[check_for_hungry_1#]
	alu[io_fc_credit, --, B, STARVING_FC_CREDIT_MINUS_9]	; conservative
	_sphy_mphy4_tx_debug_incr_counter(@is_starving)
	br[NORMAL_PATH_LABEL]

check_for_hungry_1#:
	br!=byte[tmp, 0, HUNGRY_STATUS, increase_counter_first#] ; not ready

	alu[io_fc_credit, --, B, HUNGRY_FC_CREDIT_MINUS_9]	; conservative
	_sphy_mphy4_tx_debug_incr_counter(@is_hungry)
	br[NORMAL_PATH_LABEL]

increase_counter_first#:
	_sphy_mphy4_tx_debug_incr_counter(@is_satisfied)

read_port_status_loop#:

	msf[read, $cur_port_stat, addr_port_stat, 0, 1], sig_done[sig_msf]	

wait_port_stat_read_done#:

	// read tx request from nn ring into local memory to buffer port FIFO busy

	br_inp_state[nn_empty, no_new_request#]

	// get mpacket from nn ring and save into local memory

	_sphy_mphy4_tx_get_mpkt_from_nn_ring()			; (10)
	
	// set queue tail for next _sphy_mphy4_tx_get_mpkt_from_nn_ring()	

	_sphy_mphy4_tx_set_lmindex0_to_queue_tail()		; (4)

	no_new_request#:

	br_!signal[sig_msf, wait_port_stat_read_done#]	

	alu[tmp, $cur_port_stat, AND, 3]		; bit 1 and 0, both 0: STARVING

	bne[check_for_hungry_2#]
	alu[io_fc_credit, --, B, STARVING_FC_CREDIT_MINUS_1]
	_sphy_mphy4_tx_debug_incr_counter(@is_starving)
	br[NORMAL_PATH_LABEL]

check_for_hungry_2#:
	br!=byte[tmp, 0, HUNGRY_STATUS, go_incease_counter#] ; not ready


	alu[io_fc_credit, --, B, HUNGRY_FC_CREDIT_MINUS_1]
	_sphy_mphy4_tx_debug_incr_counter(@is_hungry)
	br[NORMAL_PATH_LABEL]

go_incease_counter#:
	_sphy_mphy4_tx_debug_incr_counter(@is_satisfied)

	br[read_port_status_loop#] ; not ready

	
.end 
#endif  // #ifdef CHECK_PORT_TX_FIFO_STATUS

#endm	// _sphy_mphy4_tx_wait_port_tx_fifo_ready


///////////////////////////////////////////////////////////////////////////////
//
//	_sphy_mphy4_tx_update_fc_credit_for_no_tbuf_validation() 
//
// Description: update flow control credit based on the port status read in 
//				previous iteration when there is no new tx request for 
//				later use to optimize the performance
//
// Outputs: 
//		io_fc_credit		credit for flow control in unit of tbuf ele size
//
// Inputs:  
//		in_port_stat		port status
//
// Constants:
//		None
//
//	Size:
//        instructions 
//
///////////////////////////////////////////////////////////////////////////////

#macro _sphy_mphy4_tx_update_fc_credit_for_no_tbuf_validation(io_fc_credit, \
		in_port_stat)

#ifdef CHECK_PORT_TX_FIFO_STATUS
.begin 
	.reg	tmp

	alu[--, io_fc_credit, -, STARVING_FC_CREDIT_MINUS_8]
	bge[done#]

	alu[tmp, in_port_stat, and, 3]		; bit 1 and 0, both 0: STARVING
	bne[check_for_hungry#]
	alu[io_fc_credit, --, b, STARVING_FC_CREDIT_MINUS_8]
	_sphy_mphy4_tx_debug_incr_counter(@is_starving)
	br[done#]

check_for_hungry#:
	alu[--, io_fc_credit, -, HUNGRY_FC_CREDIT_MINUS_8]
	bgt[done#]

	// br!=byte[tmp, 0, HUNGRY_STATUS, done#] 	; not ready
	br!=byte[tmp, 0, HUNGRY_STATUS, satisfied#] 	; not ready


	alu[--, io_fc_credit, -, HUNGRY_FC_CREDIT_MINUS_8]
	_sphy_mphy4_tx_debug_incr_counter(@is_hungry)
	br[done#]

satisfied#:
	_sphy_mphy4_tx_debug_incr_counter(@is_satisfied)
done#:
	
.end 
#endif  // #ifdef CHECK_PORT_TX_FIFO_STATUS

#endm	// _sphy_mphy4_tx_update_fc_credit_for_no_tbuf_validation


///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_get_mpkt_from_lm()
//
// Description: get one mpkt from local memory and advance queue head
//
// Outputs:					
//		out_meta_data_ptr:			meta data pointer of this mpkt
//      out_tcw0:					tcw0 of this mpkt
//		out_falg_tbuf_ele:			eob flag and tbuf element number of 
//									this mpkt
//		out_addr_of_tbuf_validatio:	addr of tbuf validation							
//
// Inputs:  
//      None 
//
// Constants:
//		None 
//
//	Size:
//		16 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro	_sphy_mphy4_tx_get_mpkt_from_lm(out_meta_data_ptr, out_tcw0, \
						out_flag_tbuf_ele, out_addr_of_tbuf_validation)
.begin

	_sphy_mphy4_tx_set_lmindex0_to_queue_head()			; (4)
	alu[out_meta_data_ptr, --, b, *l$index0[META_DATA_PTR_OFFSET]]
	alu[out_tcw0, --, b, *l$index0[TCW0_DATA_OFFSET]]
	alu[out_flag_tbuf_ele, --, b, *l$index0[FLAG_TBUF_ELEMENT_OFFSET]]
	.begin
	.reg tmp
	alu[tmp, out_flag_tbuf_ele, and, TBUF_ELE_BITS_MASK]
	alu[tmp, --, b, tmp, <<TX_CNTRL_ADDR_SHFT] 
	alu[addr_of_tx_control, *l$index1[GLOBAL_CTW_VD_BASE_INDEX], +, tmp]
	.end	// tmp

	_sphy_mphy4_tx_advance_queue_head(port_entry_turnaround)	;	(6)


.end 
#endm	// _sphy_mphy4_tx_get_mpkt_from_lm()


///////////////////////////////////////////////////////////////////////////////
// _sphy_mphy4_tx_get_mpkt_from_nn_ring()
//
// Description: get one mpkt from nn ring, save in local memory, and advance 
//				queue tail
//
// Prerequisite:local memory index 0 already set to queue tail
//
// Outputs:					
//      None 
//
// Inputs:  
//      None 
//
// Constants:
//		None 
//
//	Size:
//		10 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro	_sphy_mphy4_tx_get_mpkt_from_nn_ring()
.begin

	// find if LM queue is full
	alu[--, *l$index1[GLOBAL_PKTS_IN_QUEUE_INDEX], -, SKIP_GET_TXR_THRESHOLD]
	// queue full, so don't get tx request, keep tx requests in NN ring
	bgt[get_request_done#]
	alu[*l$index0[META_DATA_PTR_OFFSET], --, b, *n$index++]
	alu[*l$index0[TCW0_DATA_OFFSET], --, b, *n$index++]
	alu[*l$index0[FLAG_TBUF_ELEMENT_OFFSET], --, b, *n$index++]
	_sphy_mphy4_tx_advance_queue_tail(port_entry_turnaround)		;	(5)
get_request_done#:
.end 
#endm	// _sphy_mphy4_tx_get_mpkt_from_nn_ring()


#endif 		// __SPHY_MPHY4_TX_UTIL3_UC__

///////////////////////////////////////////////////////////////////////////////

