#*****************************************************************************
#                            Intel Proprietary
#
# Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
# No part of this program or publication may be reproduced, transmitted,
# transcribed, stored in a retrieval system, or translated into any language
# or computer language in any form or by any means, electronic, mechanical,
# magnetic, optical, chemical, manual, or otherwise, without the prior
# written permission of:
#                        Intel Corporation
#                        2200 Mission College Blvd.
#                        Santa Clara, CA  95052-8119
#****************************************************************************

include $(ROOTDIR)/Rules.make

# A Container should first define the following
# variables, and then include this file
#
# APPLICATION - The container name
#
# COMPONENTS - Subcomponent names
# SRCFILES - Subcomponent names

unexport APPLICATION
unexport COMPONENTS
unexport TARGET
unexport COMPONENTSOBJ
unexport TCOMPONENTSOBJ
unexport SHAREDOBJ
unexport TSHAREDOBJ
unexport COPSOBJ
unexport COMMOBJ
unexport LIBS
unexport COBJ
unexport CCOBJ
unexport CPPOBJ
unexport OBJFILES
unexport OBJS
unexport SRCS
unexport DOTDS

#sub components
TCOMPONENTSOBJ = $(foreach comp,$(COMPONENTS),$(comp)/$(LIBDIR)/$(notdir $(comp)).o)
COMPONENTSOBJ = $(TCOMPONENTSOBJ:%/ControlPlane.o=%/npf_pdk.o)

COMMOBJ =  $(foreach comp,$(COMMCOMPS),$(COMMPATH)/$(comp)/$(LIBDIR)/$(comp).o)
COPSOBJ =  $(foreach comp,$(COPSCOMPS),$(COPSPATH)/$(comp)/$(LIBDIR)/$(TEAM)$(comp).o)

SHAREDOBJ =  $(foreach comp,$(SHAREDLIBS),$(comp)/$(LIBDIR)/lib$(notdir $(comp)).so)

COMMDIRS = $(foreach comp,$(COMMCOMPS),$(COMMPATH)/$(comp))
COPSDIRS = $(foreach comp,$(COPSCOMPS),$(COPSPATH)/$(comp))
SHAREDDIRS = $(SHAREDLIBS)

#local components
COBJ=$(patsubst %.c,%.o,$(filter %.c, $(SRCFILES)))
CCOBJ=$(patsubst %.cc,%.o,$(filter %.cc, $(SRCFILES)))
CPPOBJ=$(patsubst %.cpp,%.o,$(filter %.cpp, $(SRCFILES)))
OBJFILES=$(COBJ) $(CCOBJ) $(CPPOBJ)

OBJS=$(OBJFILES:%=$(OBJDIR)/%)
SRCS=$(SRCFILES:%=src/%)
DOTDS=$(OBJFILES:%.o=$(LIBDIR)/.%.d)

LOCALDIR    := .

INCLUDEPATH := $(shell sh $(ROOTDIR)/.bpfinc.sh) $(INCLUDELOCAL) $(INCLUDESYSTEM)

LIBPATH += $(foreach comp,$(SHAREDLIBS),-L$(comp)/$(LIBDIR))
LIBS += $(foreach comp,$(SHAREDLIBS),-l$(notdir $(comp)))

ifneq ($(TARG_OS),VXWORKS)
TARGET = $(BINDIR)/$(APPLICATION)
else
TARGET = $(BINDIR)/$(APPLICATION).out
endif

default: $(TARGET)

$(TARGET): $(COMPONENTSOBJ) $(COMMOBJ) $(COPSOBJ) $(SHAREDOBJ) $(OBJS)
	@echo "------ building application $(basename $(@F)) ------"
	@mkdir -p $(BINDIR)
ifneq ($(TARG_OS),VXWORKS)
	$(CCC) $(INCLUDEPATH) $(CFLAGS) -o $@ $(COMPONENTSOBJ) $(OBJS) $(COMMOBJ) $(COPSOBJ) $(LIBPATH) $(LIBS) 
else
	$(LD) $(LD_PARTIAL_FLAGS) $(COMPONENTSOBJ) $(OBJS) $(COMMOBJ) $(COPSOBJ) -o partialImage.o
	$(NM) partialImage.o $(COMPONENTSOBJ) $(OBJS) $(COMMOBJ) | wtxtcl $(WIND_BASE)/host/src/hutils/munch.tcl -asm arm > ctdt.c
	$(CC) -fdollars-in-identifiers $(CFLAGS) $(INCLUDESYSTEM) -c ctdt.c -o ctdt.o
	$(LD) $(LD_PARTIAL_FLAGS) partialImage.o ctdt.o -o $@
	rm -f partialImage.o ctdt.*
endif

.PHONY : clean allclean dummy

$(COMPONENTSOBJ): dummy
	@mkdir -p $(@D)
	@$(MAKE) -C $(@D)/../../
#	@$(MAKE) -C $(basename $(@F))

$(COMMOBJ): dummy
	@$(MAKE) -C $(COMMPATH)/$(basename $(@F))

$(COPSOBJ): dummy
	@$(MAKE) -C $(COPSPATH)/cops

$(SHAREDOBJ): dummy
	@mkdir -p $(@D)
	@$(MAKE) -C $(@D)/../../
#	@$(MAKE) -C $(COMMPATH)/$(basename $(@F))

dummy:

clean: dummy
	rm -f $(TARGET) $(OBJS) $(DOTDS)


CLEANDIRS = $(patsubst %, _cleandir_%, $(COMPONENTS) $(COMMDIRS) $(SHAREDDIRS))

$(CLEANDIRS): dummy
	$(MAKE) -C $(patsubst _cleandir_%, %, $@) allclean

allclean: $(CLEANDIRS) clean

project: $(DOTDS)
	@echo APPLICATION=$(APPLICATION) > .temp.m2p
	@echo SRCFILES=$(SRCFILES) >> .temp.m2p
	@echo IXROOT=$(IXROOT) >> .temp.m2p
	@echo IXAHEADERFILES=$(IXAHEADERFILES) >> .temp.m2p
	@$(ROOTDIR)/Tools/m2p.pl .temp.m2p $(ROOTDIR)/Tools/project.template "$(CFLAGS)" $(LIBDIR) "$(INCLUDEPATH)" $(ROOTDIR)/../workspace/ixa_sdk.wsp
	@rm .temp.m2p
	@for comp in $(COMPONENTS) $(SHAREDLIBS); \
	do $(MAKE) -C $$comp project; \
	done

ifneq ($(wildcard $(DOTDS)),)
-include $(DOTDS)
endif
