/*****************************************************************************
 *                  INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 *$Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/APIImplementations/ATM/src/if_atm.c,v $
 *$Revision: 1.5 $
 *$Date: 2003/11/17 16:57:09 $
 * Description:
 *     This file contains the interface management NPF api implementation.
 *
 *****************************************************************************/
#ifdef _INCLUDE_ATM

#include <assert.h>
#include <string.h>
#include "npf_common.h"
#include "npf_cm_types.h"
#include "pdk_cm_types.h"
#include "pdk_callback.h"
#include "pdk_callback_types.h"
#include "pdk_cm.h"
#include "pdkmgr.h"
#include "pil.h"
#include "fpp_common.h"
#include "fppapi.h"
#include "list.h"
#include "npf_if.h"
#include "pdk_ns_helper.h"

#include "fppapi.h"
#include "pdkmgr.h"
#include "cm.h"


#include "npf_atm_types.h"
#include "fppapi_atm_types.h"
#include "if.h"
#include "if_atm.h"
#include "if_atm_callback.h"


IfATM_ControlBlock_t ifAtmCB;
extern DList  IfList;

/* This array store the CB handle registered with FP-Plug In API */
FPPI_CBHANDLE	ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_MAX_CB_TYPES];

/* Extern Function declration of if manager functions */
extern FPPI_CORRELATOR newIfCorrelator(If_t  *ifc, request_t *req);
extern NPF_error_t getInterface(NPF_IfHandle_t ifHandle, If_t  **iface);
extern NPF_error_t getFEandPORTdata(If_t  *iface);
extern NPF_error_t onResponse(request_t *request, void *resp); 

/*****************************************************************************
*                                                                            *
*  -------------------------- Internal Interface --------------------------  *
*                                                                            *
*****************************************************************************/

/*****************************************************************************
*    Name: pdk_atm_init
*
*    Description:
*        Initialize and allocate the global link list, register CB from FPPI
*        and B&D, Init the lock. Called by cm_init
*
*    Parameters:
*        None
*
*    Preconditions:
*        g_initialized = 0
*        called after bd_init, cbthread_init and fppi_init.
*        called before ns_init, ipv4_init, mpls_init
*
*    Postconditions:
*        g_initialized = 1;
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*
********************************************************************************/
NPF_RET pdk_atm_init(void)
{

	/* Take the lock for the Module CB */
	PIL_EnterCriticalSection(&ifAtmCB.lock);

	/* Initialize the List of Request CBs */
	npf_list_init(&(ifAtmCB.apiReqList), IfATM_FreeAtmCbMemory);
	

	/* Set the Initialized flag to TRUE */
	ifAtmCB.initFlag = NPF_TRUE;

	/* Give the lock for the Module CB */
	PIL_LeaveCriticalSection(&ifAtmCB.lock);

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_IfAttrSetFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_ATTR_SET_CB],	/* CB handle */
					FPPI_ATM_IfAttrSet,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ATTR_SET)
					!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_IfDeleteFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_IF_DELETE_CB],	/* CB handle */
					FPPI_ATM_IfDelete,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_DELETE)
					!= FPPI_SUCCESS)
			return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_IfEnableFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_IF_ENABLE_CB],	/* CB handle */
					FPPI_ATM_IfEnable,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ENABLE)
					!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_IfDisableFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_IF_DISABLE_CB],	/* CB handle */
					FPPI_ATM_IfDisable,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_DISABLE)
					!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_IfGenStatsGetFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_IF_GEN_STATS_GET_CB],	/* CB handle */
					FPPI_ATM_IfGenStatsGet,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_STATS_GET)
					!= FPPI_SUCCESS)
			return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_VccSetFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_VCC_SET_CB],	/* CB handle */
					FPPI_ATM_VccSet,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ATM_VCC_SET)
					!= FPPI_SUCCESS)
			return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_VccBindFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_VCC_BIND_CB],	/* CB handle */
					FPPI_ATM_VccBind,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ATM_VCC_BIND)
					!= FPPI_SUCCESS)
			return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_VccDelFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_VCC_DEL_CB],	/* CB handle */
					FPPI_ATM_VccDelete,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ATM_VCC_DELETE)
					!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_VccEnableFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_VCC_ENABLE_CB], /* CB handle */
					FPPI_ATM_VccEnable,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ATM_VCC_ENABLE)
					!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_VccDisableFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_VCC_DISABLE_CB], /* CB handle */
					FPPI_ATM_VccDisable,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ATM_VCC_DISABLE)
					!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;


	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_VccOperStatusGetFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_VCC_OPER_STATUS_GET_CB], /* CB handle */
					FPPI_ATM_VccOperStatusGet,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ATM_VCC_OPER_STATUS_GET)
					!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_VccStatsGetFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_VCC_STATS_GET_CB], /* CB handle */
					FPPI_ATM_VccStatsGet,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ATM_VCC_STATS_GET)
					!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_Aal2ChannelSetFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_AAL2_CHANNEL_SET_CB], /* CB handle */
					FPPI_ATM_AAL2_ChannelSet,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ATM_AAL2_CHANNEL_SET)
					!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_Aal2ChannelDelFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_AAL2_CHANNEL_DEL_CB],	/* CB handle */
					FPPI_ATM_AAL2_ChannelDel,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ATM_AAL2_CHANNEL_DEL)
					!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_ConnXcSetFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_CONNECTION_XC_SET_CB], /* CB handle */
					FPPI_ATM_ConnectionXcSet,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ATM_CONNECTION_XC_SET)
					!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_ConnXcDelFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_CONNECTION_XC_DEL_CB], /* CB handle */
					FPPI_ATM_ConnectionXcDel,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ATM_CONNECTION_XC_DEL)
					!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_OamCpSetFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_OAM_CP_SET_CB], /* CB handle */
					FPPI_ATM_OAM_CP_Set,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ATM_OAM_CP_SET)
					!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_OamCcSetFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_OAM_CC_SET_CB], /* CB handle */
					FPPI_ATM_OAM_CC_Set,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ATM_OAM_CC_SET)
					!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_OamLbSetFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_OAM_LB_SET_CB], /* CB handle */
					FPPI_ATM_OAM_LB_Set,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ATM_OAM_LB_SET)
					!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_OamMonitorFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_OAM_MONITOR_CB], /* CB handle */
					FPPI_ATM_OAM_Monitor,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ATM_OAM_MONITOR)
					!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_OamAlarmSetFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_OAM_ALARM_SET_CB], /* CB handle */
					FPPI_ATM_OAM_AlarmSet,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ATM_OAM_ALARM_SET)
					!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_OamAlarmClearFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_OAM_ALARM_CLEAR_CB], /* CB handle */
					FPPI_ATM_OAM_AlarmClear,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ATM_OAM_ALARM_CLEAR)
					!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_OamPmSetFunc,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_OAM_PM_SET_CB], /* CB handle */
					FPPI_ATM_OAM_PM_Set,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ATM_OAM_PM_SET)
					!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_AtmEvent,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_EVENT_OAM_AIS_ALARM_CB], /* CB handle */
					FPPI_ATM_OAM_AisAlarm_Event,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ATM_EVENT_OAM_AIS_ALARM)
					!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
					(void *)&ifAtmOnCB_AtmEvent,	/* CB func */
					&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_EVENT_OAM_RDI_ALARM_CB], /* CB handle */
					FPPI_ATM_OAM_RdiAlarm_Event,		/* NPF_CBCAT */
					(NPF_USERCONTEXT)NPF_IF_ATM_EVENT_OAM_RDI_ALARM)
					!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
				(void *)&ifAtmOnCB_AtmEvent,	/* CB func */
				&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_EVENT_OAM_LB_CB], /* CB handle */
				FPPI_ATM_OAM_Lb_Event,		/* NPF_CBCAT */
				(NPF_USERCONTEXT)NPF_IF_ATM_EVENT_OAM_LB)
				!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
				(void *)&ifAtmOnCB_AtmEvent,	/* CB func */
				&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_EVENT_OAM_FPM_STATS_CB], /* CB handle */
				FPPI_ATM_OAM_FpmStats_Event,		/* NPF_CBCAT */
				(NPF_USERCONTEXT)NPF_IF_ATM_EVENT_OAM_FPM_STATS)
				!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
				(void *)&ifAtmOnCB_AtmEvent,	/* CB func */
				&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_EVENT_OAM_BR_STATS_CB], /* CB handle */
				FPPI_ATM_OAM_BrStats_Event,		/* NPF_CBCAT */
				(NPF_USERCONTEXT)NPF_IF_ATM_EVENT_OAM_BR_STATS)
				!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
				(void *)&ifAtmOnCB_AtmEvent,	/* CB func */
				&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_EVENT_ATM_VCC_UP_EVENT_CB], /* CB handle */
				FPPI_ATM_VccUp_Event,		/* NPF_CBCAT */
				(NPF_USERCONTEXT)NPF_IF_ATM_EVENT_ATM_VCC_UP_EVENT)
				!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;

	if (FPPAPI_register_callback(
				(void *)&ifAtmOnCB_AtmEvent,	/* CB func */
				&ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_EVENT_ATM_VCC_DOWN_EVENT_CB], /* CB handle */
				FPPI_ATM_VccDown_Event,		/* NPF_CBCAT */
				(NPF_USERCONTEXT)NPF_IF_ATM_EVENT_ATM_VCC_DOWN_EVENT)
				!= FPPI_SUCCESS)
    		return NPF_OUT_OF_MEMORY;


	return NPF_SUCCESS;

}


/*****************************************************************************
*    Name: pdk_atm_shutdown
*
*    Description:
*        Deregister and free the global link list
*
*    Parameters:
*        None
*
*    Preconditions:
*        g_initialized = 1
*        called before bd_shutdown, fppi_shutdown and cbthread_shutdown
*        called after ns_shutdown, ipv4_shutdown, mpls_shutdown ...
*
*    Postconditions:
*        g_initialized = 0;
*
*    Returns:
*        NPF_SUCCESS - always success
*
********************************************************************************/
NPF_RET pdk_atm_shutdown(void)
{
	/* Deregister for the Cal back functions registered with FP-Plugin */
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_ATTR_SET_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_IF_DELETE_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_IF_ENABLE_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_IF_DISABLE_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_IF_GEN_STATS_GET_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_VCC_SET_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_VCC_DEL_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_VCC_ENABLE_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_VCC_DISABLE_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_VCC_BIND_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_VCC_OPER_STATUS_GET_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_VCC_STATS_GET_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_AAL2_CHANNEL_SET_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_AAL2_CHANNEL_DEL_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_CONNECTION_XC_SET_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_CONNECTION_XC_DEL_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_OAM_CP_SET_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_OAM_CC_SET_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_OAM_LB_SET_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_OAM_MONITOR_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_OAM_ALARM_SET_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_OAM_ALARM_CLEAR_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_OAM_PM_SET_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_EVENT_OAM_AIS_ALARM_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_EVENT_OAM_RDI_ALARM_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_EVENT_OAM_LB_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_EVENT_OAM_FPM_STATS_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_EVENT_OAM_BR_STATS_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_EVENT_ATM_VCC_UP_EVENT_CB]);	
	FPPAPI_deregister_callback(ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_EVENT_ATM_VCC_DOWN_EVENT_CB]);	
	
	/* Take the lock for the Module CB */
	PIL_EnterCriticalSection(&ifAtmCB.lock);

	/* Set the Initialized flag to TRUE */
	ifAtmCB.initFlag = NPF_FALSE;

	/* Free the memory allocated for the request list */
	npf_list_destroy(&(ifAtmCB.apiReqList));

	/* Give the lock for the Module CB */
	PIL_LeaveCriticalSection(&ifAtmCB.lock);

	/* Destroy the list of interfaces created in Interface manager.
		This is ideally called for shutdown of interface manager but
		repetition of the same has no harm but instead helps in getting 
		rid of memory leaks */
	npf_list_destroy(&IfList);

   return NPF_SUCCESS;
}

/*********************************************************************
*
*   Function: IfATM_FreeAtmCbMemory 
*
*   Parameters:
*
*   Description:
*			This function Frees the memory associated with the IfATM_RequestCB_t
*			list in ATM control block . 
*
*   return : 
*
*
*****************************************************************************/
void IfATM_FreeAtmCbMemory(void *mem)
{
	IfATM_RequestCB_t *request = NULL;	
	NPF_uint32_t i =0;
	
	request = (IfATM_RequestCB_t *)mem;

	/* Free the memory allocated for the request list */
	npf_list_destroy(&(request->fpTransList));

	/* Free the memory allocated for VCC statistics */
	if((request->cbData).type == NPF_IF_ATM_VCC_STATS_GET)
	{
		for(i=0 ; i<(request->cbData.n_resp); i++)
		{
			/* Free the memory for each VCC */
			if((request->cbData).resp[i].u.vccStats != NULL)
				PIL_FreeMemory((request->cbData).resp[i].u.vccStats);		

		}
	}

	/* Free the memory previously allocated for the response structure */
	PIL_FreeMemory( (request->cbData).resp );

	/* Free the memory previously allocated for the response structure */
	PIL_FreeMemory( (request->apiData).entry );

	/* Free the memory previously allocated for the response structure */
	PIL_FreeMemory(request);

	return;
}/* End of IfATM_FreeAtmCbMemory */ 




/*****************************************************************************
*    Name: IfATM_IfGetFePortFromIfHandle 
*
*    Description:
*			This function Gets FE Id and Port Id from the interface Handle passed 
*			to it. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*
********************************************************************************/
NPF_RET IfATM_IfGetFePortFromIfHandle(
				NPF_IfHandle_t ifHandle,
				FPPI_FEID *feId,
				FPPI_PortID	*portId) 
{
	If_t	*ifc;

	/* Fill up the ifc structure using the ifHandle */
	if(getInterface(ifHandle, &ifc) != NPF_NO_ERROR)
		return NPF_E_UNKNOWN;

	/* Get the FE and Port id in the ifc structure */
	if(getFEandPORTdata(ifc) != NPF_NO_ERROR)
		return NPF_E_UNKNOWN;

	/* Set the Port Id */
	*portId = ifc->PORT.ports[0];

	/* Set the FE Id */
	if((ifc->PORT).portsList != NULL)
		*feId = (ifc->PORT.portsList)->fe_id;

	return NPF_SUCCESS;

}

/*****************************************************************************
*    Name: IfATM_IfAttrSet 
*
*    Description:
*			This function calls the FP-Plugin API to set the attributes of an already
*			created ATM interface.
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*
********************************************************************************/

NPF_RET IfATM_IfAttrSet(If_t *ifc, NPF_IfGeneric_t *iface, request_t *req)
{
	FPPI_FEID feId = 0;
	IfATM_IfAtmAttr_t ifAtmAttr;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	IfATM_IfAtmAttrCorrelator_t *cor = NULL;

	/* Copy the ATM Attributes from the Generic structure */
	memcpy(&(ifAtmAttr.atmAttr),&((iface->u).ATM_Attr),sizeof(NPF_IfATM_t));

	/* Set the FE Id */
	feId = (ifc->PORT.portsList)->fe_id;

	/* Set the Port Speed to the Max Tx Rate of the Port */
	ifAtmAttr.portSpeed = iface->speed;

	/* Create the corelator to be used. This is need to take note of the
		port Speed  */
	cor = (IfATM_IfAtmAttrCorrelator_t *)PIL_AllocateMemory(
					sizeof(IfATM_IfAtmAttrCorrelator_t),PIL_MEM_ZERO_INIT);

	if(cor != NULL)
	{
		cor->ifCorr = (IfCorrelator_t *)newIfCorrelator(ifc, req);
		cor->portSpeed = iface->speed;

		/* Call the FP-Plug function to set ATM attributes */
	  	fpRetVal = FPPAPI_ATM_IfAttrSetFunc( (FPPI_FEID)feId,
									(FPPI_CORRELATOR)cor,
									(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_ATTR_SET_CB],
									(IfATM_IfAtmAttr_t *)&ifAtmAttr,
									(NPF_uint32_t) 1);
	}
	else
	{
		/* Set the fpRetVal to Out of Memory */
		fpRetVal = FPPI_OUT_OF_MEMORY;
	}

	/* Check the return value of the FP-Plug in API */
	if(fpRetVal != FPPI_SUCCESS)
	{
		/* set the interface handle in the request structure */
		req->ihandle = ifc->nsHandle;

		/* Check if the FP-Plug in API succedded */
		switch(fpRetVal)
		{
			/* Set the error code in the request structure
				 according to the response Recvd */
			case FPPI_OUT_OF_MEMORY:
				req->error = NPF_OUT_OF_MEMORY;
				break;
			case FPPI_INVALID_PARAMETERS:
			case FPPI_FAIL:
				req->error =  NPF_IF_E_INVALID_PARAM;
				break;
			case FPPI_NOT_INITIALIZED:
				req->error =  NPF_COMPONENT_UNINITIALIZED;
				break;
			default:
				req->error =  NPF_IF_E_INVALID_PARAM;
				break;
		}/* end of switch */


		/* Increment the resp count in the request structure */
		req->fpResp++;

		/* Call the ifManager function to handle the callback */
		onResponse((request_t *)req, NULL);

		/* Free the memory allocated for the corelator */
		PIL_FreeMemory(cor);

	}/* end of if(fpRetVal != FFPI_SUCCESS ) */

	return (NPF_SUCCESS);		
}


/*****************************************************************************
*    Name: IfATM_IfDelete 
*
*    Description:
*			This function calls the FP-Plugin API to delete an already 
*			created ATM interface.
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*
********************************************************************************/

NPF_RET IfATM_IfDelete(If_t *ifc, request_t *req)
{
	FPPI_PortID		portId = 0;
	FPPI_FEID feId = 0;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	FPPI_CORRELATOR cor = NULL;

	/* Create the correlator */
	cor = newIfCorrelator(ifc, req);

	/* Set the Port Id */
	portId = ifc->PORT.ports[0];

	/* Check if Interface Attributes have been set */
	if(ifc->PORT.portsList != NULL)
	{
		/* Set the FE Id */
		feId = (ifc->PORT.portsList)->fe_id;

		/* Call the FP-Plug function to set ATM attributes */
		fpRetVal = FPPAPI_ATM_IfDeleteFunc((FPPI_FEID)feId,
									(FPPI_CORRELATOR)cor,
									(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_IF_DELETE_CB],
									(FPPI_PortID *)&portId,
									(NPF_uint32_t)1);
	}
	else
	{
		/* Set fpRetVal to FPPI_SUCCESS. We are making the request here as 
			SUCCESS, unlike other places because this seems to be the case
			when user did IfCreate and then immidiately called IfDelete with
			correct ifHandle */
		fpRetVal = FPPI_SUCCESS;

		/* set the interface handle in the request structure */
		req->ihandle = ifc->nsHandle;

		/* Set the async response to SUCCESS */
		req->error = NPF_NO_ERROR;

		/* Increment the resp count in the request structure */
		req->fpResp++;

		/* Call the ifManager function to handle the callback */
		onResponse((request_t *)req, NULL);

		/* Free the memory allocated for the corelator */
		PIL_FreeMemory(cor);
	}

	/* Check the return value of the FP-Plug in API */
	if(fpRetVal != FPPI_SUCCESS)
	{
		/* set the interface handle in the request structure */
		req->ihandle = ifc->nsHandle;

		/* Check if the FP-Plug in API succedded */
		switch(fpRetVal)
		{
			/* Set the error code in the request structure
				 according to the response Recvd */
			case FPPI_OUT_OF_MEMORY:
				req->error = NPF_OUT_OF_MEMORY;
				break;
			case FPPI_INVALID_PARAMETERS:
			case FPPI_FAIL:
				req->error =  NPF_IF_E_INVALID_PARAM;
				break;
			case FPPI_NOT_INITIALIZED:
				req->error =  NPF_COMPONENT_UNINITIALIZED;
				break;
			default:
				req->error =  NPF_IF_E_INVALID_PARAM;
				break;
		}/* end of switch */


		/* Increment the resp count in the request structure */
		req->fpResp++;

		/* Call the ifManager function to handle the callback */
		onResponse((request_t *)req, NULL);

		/* Free the memory allocated for the corelator */
		PIL_FreeMemory(cor);

	}/* end of if(fpRetVal != FFPI_SUCCESS ) */

	return (NPF_SUCCESS);		

}

/*****************************************************************************
*    Name: IfATM_IfEnable 
*
*    Description:
*			This function calls the FP-Plugin API to Enable an already 
*			created ATM interface.
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*
********************************************************************************/

NPF_RET IfATM_IfEnable(If_t *ifc, request_t *req)
{
	FPPI_PortID		portId = 0;
	FPPI_FEID feId = 0;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	FPPI_CORRELATOR cor = NULL;

	/* Create the correlator */
	cor = newIfCorrelator(ifc, req);

	/* Set the Port Id */
	portId = ifc->PORT.ports[0];

	/* Check if Interface Attributes have been set */
	if(ifc->PORT.portsList != NULL)
	{
		/* Set the FE Id */
		feId = (ifc->PORT.portsList)->fe_id;

		/* Call the FP-Plug function to set ATM attributes */
		fpRetVal = FPPAPI_ATM_IfEnableFunc((FPPI_FEID)feId,
									(FPPI_CORRELATOR)cor,
									(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_IF_ENABLE_CB],
									(FPPI_PortID *)&portId,
									(NPF_uint32_t)1);
	}
	else
	{
		/* Set fpRetVal to FPPI_INVALID_PARAMETERS */
		fpRetVal = FPPI_INVALID_PARAMETERS;
	}

	/* Check the return value of the FP-Plug in API */
	if(fpRetVal != FPPI_SUCCESS)
	{
		/* set the interface handle in the request structure */
		req->ihandle = ifc->nsHandle;

		/* Check if the FP-Plug in API succedded */
		switch(fpRetVal)
		{
			/* Set the error code in the request structure
				 according to the response Recvd */
			case FPPI_OUT_OF_MEMORY:
				req->error = NPF_OUT_OF_MEMORY;
				break;
			case FPPI_INVALID_PARAMETERS:
			case FPPI_FAIL:
				req->error =  NPF_IF_E_INVALID_PARAM;
				break;
			case FPPI_NOT_INITIALIZED:
				req->error =  NPF_COMPONENT_UNINITIALIZED;
				break;
			default:
				req->error =  NPF_IF_E_INVALID_PARAM;
				break;
		}/* end of switch */


		/* Increment the resp count in the request structure */
		req->fpResp++;

		/* Call the ifManager function to handle the callback */
		onResponse((request_t *)req, NULL);

		/* Free the memory allocated for the corelator */
		PIL_FreeMemory(cor);

	}/* end of if(fpRetVal != FFPI_SUCCESS ) */

	return (NPF_SUCCESS);		

}


/*****************************************************************************
*    Name: IfATM_IfDisable 
*
*    Description:
*			This function calls the FP-Plugin API to Disable an already 
*			created ATM interface.
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*
********************************************************************************/
NPF_RET IfATM_IfDisable(If_t *ifc, request_t *req)
{
	FPPI_PortID		portId = 0;
	FPPI_FEID feId = 0;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	FPPI_CORRELATOR cor = NULL;

	/* Create the correlator */
	cor = newIfCorrelator(ifc, req);

	/* Set the Port Id */
	portId = ifc->PORT.ports[0];

	/* Check if Interface Attributes have been set */
	if(ifc->PORT.portsList != NULL)
	{
		/* Set the FE Id */
		feId = (ifc->PORT.portsList)->fe_id;

		/* Call the FP-Plug function to set ATM attributes */
		fpRetVal = FPPAPI_ATM_IfDisableFunc((FPPI_FEID)feId,
									(FPPI_CORRELATOR)cor,
									(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_IF_DISABLE_CB],
									(FPPI_PortID *)&portId,
									(NPF_uint32_t)1);
	}
	else
	{
		/* Set fpRetVal to FPPI_INVALID_PARAMETERS */
		fpRetVal = FPPI_INVALID_PARAMETERS;
	}

	/* Check the return value of the FP-Plug in API */
	if(fpRetVal != FPPI_SUCCESS)
	{
		/* set the interface handle in the request structure */
		req->ihandle = ifc->nsHandle;

		/* Check if the FP-Plug in API succedded */
		switch(fpRetVal)
		{
			/* Set the error code in the request structure
				 according to the response Recvd */
			case FPPI_OUT_OF_MEMORY:
				req->error = NPF_OUT_OF_MEMORY;
				break;
			case FPPI_INVALID_PARAMETERS:
			case FPPI_FAIL:
				req->error =  NPF_IF_E_INVALID_PARAM;
				break;
			case FPPI_NOT_INITIALIZED:
				req->error =  NPF_COMPONENT_UNINITIALIZED;
				break;
			default:
				req->error =  NPF_IF_E_INVALID_PARAM;
				break;
		}/* end of switch */


		/* Increment the resp count in the request structure */
		req->fpResp++;

		/* Call the ifManager function to handle the callback */
		onResponse((request_t *)req, NULL);

		/* Free the memory allocated for the corelator */
		PIL_FreeMemory(cor);

	}/* end of if(fpRetVal != FFPI_SUCCESS ) */

	return (NPF_SUCCESS);		
}


/*****************************************************************************
*    Name: IfATM_IfGenericStatsGet 
*
*    Description:
*			This function calls the FP-Plugin API to Get statistics of an already 
*			created ATM interface.
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*
********************************************************************************/
NPF_RET IfATM_IfGenericStatsGet(If_t *ifc, request_t *req)
{
	FPPI_PortID		portId = 0;
	FPPI_FEID feId = 0;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	FPPI_CORRELATOR cor = NULL;

	/* Create the correlator */
	cor = newIfCorrelator(ifc, req);

	/* Set the Port Id */
	portId = ifc->PORT.ports[0];

	/* Check if Interface Attributes have been set */
	if(ifc->PORT.portsList != NULL)
	{
		/* Set the FE Id */
		feId = (ifc->PORT.portsList)->fe_id;

		/* Call the FP-Plug function to set ATM attributes */
		fpRetVal = FPPAPI_ATM_IfGenericStatsGetFunc((FPPI_FEID)feId,
									(FPPI_CORRELATOR)cor,
									(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_IF_GEN_STATS_GET_CB],
									(FPPI_PortID *)&portId,
									(NPF_uint32_t)1);
	}
	else
	{
		/* Set fpRetVal to FPPI_INVALID_PARAMETERS */
		fpRetVal = FPPI_INVALID_PARAMETERS;
	}

	/* Check the return value of the FP-Plug in API */
	if(fpRetVal != FPPI_SUCCESS)
	{
		/* set the interface handle in the request structure */
		req->ihandle = ifc->nsHandle;

		/* Check if the FP-Plug in API succedded */
		switch(fpRetVal)
		{
			/* Set the error code in the request structure
				 according to the response Recvd */
			case FPPI_OUT_OF_MEMORY:
				req->error = NPF_OUT_OF_MEMORY;
				break;
			case FPPI_INVALID_PARAMETERS:
			case FPPI_FAIL:
				req->error =  NPF_IF_E_INVALID_PARAM;
				break;
			case FPPI_NOT_INITIALIZED:
				req->error =  NPF_COMPONENT_UNINITIALIZED;
				break;
			default:
				req->error =  NPF_IF_E_INVALID_PARAM;
				break;
		}/* end of switch */


		/* Increment the resp count in the request structure */
		req->fpResp++;

		/* Call the ifManager function to handle the callback */
		onResponse((request_t *)req, NULL);

		/* Free the memory allocated for the corelator */
		PIL_FreeMemory(cor);

	}/* end of if(fpRetVal != FFPI_SUCCESS ) */

	return (NPF_SUCCESS);		

}


/*****************************************************************************
*    Name: ifATM_CreateRequestCB 
*
*    Description:
*			This function creates a request CB structure and copies the values passed
*        as parameters to it into the CB structure. This Structure is then added to
*        the API request list in IfATM_ControlBlock.
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*
********************************************************************************/
NPF_error_t ifATM_CreateRequestCB(
				NPF_IfATM_CallbackType_t	cbType,
				NPF_callbackHandle_t			cbHandle,
				NPF_correlator_t				correlator,
				NPF_correlator_t				errorReport,
				NPF_IN NPF_IfHandle_t		ifHandle,
				NPF_uint32_t					numEntries,
				void								*entry,
				IfATM_RequestCB_t				**request)
{

	NPF_error_t		ret = NPF_SUCCESS;
	FPPI_PortID		portId = 0;
	NPF_uint32_t	dataSize = 0;
	NPF_uint32_t i = 0;
	
	IfATM_RequestCB_t	*tempCb = NULL;

	/* Trace the function */
	assert(ifATM_CreateRequestCB);

	/* Allocate memory for the Request CB */
	if( (tempCb = (IfATM_RequestCB_t *)PIL_AllocateMemory(
				sizeof(IfATM_RequestCB_t),PIL_MEM_ZERO_INIT)) == NULL)
	{
		ret = NPF_OUT_OF_MEMORY;
		return(ret);
	}

	/* Make the request CB to point to tempCb */
	*request = tempCb;

	/* Store the Values in the Request CB */
	/* Note:- For the members for which no value is passed to this function
				 we need not worry as already memset to ZERO */
	tempCb->cbh = cbHandle;
	tempCb->corr = correlator;
	tempCb->report = errorReport;
		
	/* Initialize the FP Trans List */
	npf_list_init(&(tempCb->fpTransList),PIL_FreeMemory);

	/* Get the FEID associated with the Interface Handle */
	if( IfATM_IfGetFePortFromIfHandle(ifHandle,&((tempCb->feInfo[0]).feId),
									&portId) != NPF_SUCCESS)
	{
		/* Free the memory allocated for tempCb */
		PIL_FreeMemory(tempCb);

		ret = NPF_IF_E_INVALID_PARAM;
		return(ret);
	}
	
	/* Initialize the Call Back Data Structure */
	(tempCb->cbData).type = cbType;
	(tempCb->cbData).allOK = TRUE;
	(tempCb->cbData).n_resp = 0;

	/* Allocate the memory for the AsyncResponse Structure */
	/* Note:- Here we are allocating the MAX possible asyn resp that
	   		could be received which is "numEntries" * 
				"size of NPF_IfAsyncResponse_t ". In case allOK=TRUE 
				than free this memory at the time of call back */
	dataSize = sizeof(NPF_IfAsyncResponse_t) * numEntries;

	if(((tempCb->cbData).resp = (NPF_IfAsyncResponse_t *)PIL_AllocateMemory
										(dataSize,PIL_MEM_ZERO_INIT)) == NULL)
	{
		ret = NPF_OUT_OF_MEMORY;

		/* Free the memory allocated for request CB */
		PIL_FreeMemory(tempCb);

		return(ret);
	}
	
	/* Store the IfHandle for all the asynchronous responses */
	for(i=0; i< numEntries; i++)
	{
		(tempCb->cbData).resp[i].ifHandle = ifHandle;
	}
	

	/* Copy the received parameters in user request in the Request CB */
	(tempCb->apiData).numEntries = numEntries;

	/* Allocate memory for making a copy of the received user params */
	dataSize = ifATM_GetUserDataSize(cbType,numEntries); 
	if( ((tempCb->apiData).entry = PIL_AllocateMemory(dataSize,PIL_MEM_ZERO_INIT))
					== NULL)
	{
		ret = NPF_OUT_OF_MEMORY;

		/* Free the memory allocated for response structure */
		PIL_FreeMemory((tempCb->cbData).resp);

		/* Free the memory allocated for request CB */
		PIL_FreeMemory(tempCb);

		return(ret);
	}

	/* Copy the received user parameters in entry field */
	IfATM_CopyUserParam((tempCb->apiData).entry,entry,cbType,numEntries,portId);

	/* Take the lock for the Module CB */
	PIL_EnterCriticalSection(&ifAtmCB.lock);

	/* Add the created request CB in the apiList of the Module CB */
	npf_list_pushBack((DList *)&(ifAtmCB.apiReqList), (void *)tempCb);

	/* Give the lock for the Module CB */
	PIL_LeaveCriticalSection(&ifAtmCB.lock);

	return(ret);
}	
		
	
/*****************************************************************************
*    Name: ifATM_GetUserDataSize 
*
*    Description:
*			This function computes the memory size for "numEntries" of the 
*			data structure types used for the "cbType" CallBack Type by user.	
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_uint32_t                - dataSize in bytes 
*
********************************************************************************/
NPF_uint32_t ifATM_GetUserDataSize(
				NPF_IfATM_CallbackType_t	cbType,
				NPF_uint32_t					numEntries)
{

	NPF_uint32_t	dataSize = 0;
	
	/* Trace the function */
	assert(ifATM_GetUserDataSize);

	/* Compute the data size depending upon the CB type */
	switch(cbType)
	{
		case NPF_IF_ATM_VCC_SET:
				dataSize = sizeof(IfATM_IfVcc_t) * numEntries;
				break;
		case NPF_IF_ATM_VCC_BIND:
				dataSize = sizeof(IfATM_VccBind_t) * numEntries;
				break;
		case NPF_IF_ATM_VCC_DELETE:
				dataSize = sizeof(IfATM_VccAddr_t) * numEntries;
				break;
		case NPF_IF_ATM_VCC_ENABLE:
				dataSize = sizeof(IfATM_VccAddr_t) * numEntries;
				break;
		case NPF_IF_ATM_VCC_DISABLE:
				dataSize = sizeof(IfATM_VccAddr_t) * numEntries;
				break;
		case NPF_IF_ATM_VCC_OPER_STATUS_GET:
				dataSize = sizeof(IfATM_VccAddr_t) * numEntries;
				break;
		case NPF_IF_ATM_VCC_STATS_GET:
				dataSize = sizeof(IfATM_VccAddr_t) * numEntries;
				break;
		case NPF_IF_ATM_AAL2_CHANNEL_SET:
				dataSize = sizeof(IfATM_AAL2_ChannelInfo_t) * numEntries;
				break;
		case NPF_IF_ATM_AAL2_CHANNEL_DEL:
				dataSize = sizeof(IfATM_AAL2_ConnId_t) * numEntries;
				break;
		case NPF_IF_ATM_CONNECTION_XC_SET:
				dataSize = sizeof(NPF_ATM_ConnectionXc_t) * numEntries;
				break;
		case NPF_IF_ATM_CONNECTION_XC_DEL:
				dataSize = sizeof(NPF_ATM_ConnectionXc_t) * numEntries;
				break;
		case NPF_IF_ATM_OAM_CP_SET:
				dataSize = sizeof(IfATM_OAM_CP_t) * numEntries;
				break;
		case NPF_IF_ATM_OAM_CC_SET:
				dataSize = sizeof(IfATM_OAM_CC_t) * numEntries;
				break;
		case NPF_IF_ATM_OAM_LB_SET:
				dataSize = sizeof(IfATM_OAM_LB_t) * numEntries;
				break;
		case NPF_IF_ATM_OAM_MONITOR:
				dataSize = sizeof(IfATM_OAM_Mon_t) * numEntries;
				break;
		case NPF_IF_ATM_OAM_ALARM_SET:
				dataSize = sizeof(IfATM_OAM_Alarm_t) * numEntries;
				break;
		case NPF_IF_ATM_OAM_ALARM_CLEAR:
				dataSize = sizeof(IfATM_OAM_Alarm_t) * numEntries;
				break;
		case NPF_IF_ATM_OAM_PM_SET:
				dataSize = sizeof(IfATM_OAM_PM_t) * numEntries;
				break;
		default:
				dataSize = 0;
				break;
	} 

	return(dataSize);

}
			
/*****************************************************************************
*    Name: IfATM_CopyUserParam 
*
*    Description:
*			This function copies the user param in srcParam to destParam. 
*			
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_uint32_t                - dataSize in bytes 
*
********************************************************************************/
NPF_uint32_t IfATM_CopyUserParam(
				void *destParam,
				void *srcParam,
				NPF_IfATM_CallbackType_t	cbType,
				NPF_uint32_t					numEntries,
				FPPI_PortID		portId) 
{

	NPF_uint32_t i = 0;


	/* Trace the function */
	assert(IfATM_CopyUserParam);
	
	switch(cbType)
	{
		case NPF_IF_ATM_VCC_SET:
			{
				NPF_IfVcc_t *tempVccInfo2 = NULL;
				IfATM_IfVcc_t *tempVccInfo1 = NULL;
	
				tempVccInfo1 = (IfATM_IfVcc_t *)destParam;
				tempVccInfo2 = (NPF_IfVcc_t *)srcParam;

				for(i=0; i< numEntries; i++)
				{

					/* Copy the parameters from Source to destination */
					tempVccInfo1[i].portId = portId;
					memcpy(&(tempVccInfo1[i].vccInfo),
							 &tempVccInfo2[i],sizeof(NPF_IfVcc_t));
				}
			}
			break;

		case NPF_IF_ATM_VCC_BIND:
			{
				NPF_VccAddr_t *tempVccBind2 = NULL;
				IfATM_VccBind_t *tempVccBind1 = NULL;

				tempVccBind1 = (IfATM_VccBind_t *)destParam;
				tempVccBind2 = (NPF_VccAddr_t *)srcParam;

				for(i=0; i< numEntries; i++)
				{
					/* Copy the parameters from Source to destination */
					tempVccBind1[i].portId = portId;
					memcpy(&(tempVccBind1[i].vccInfo),
								&tempVccBind2[i],sizeof(NPF_VccAddr_t));
				}
			}
			break;


		case NPF_IF_ATM_VCC_DELETE:
			{
				IfATM_VccAddr_t *tempVccAddr1 = NULL;
				NPF_VccAddr_t *tempVccAddr2 = NULL;

				tempVccAddr1 = (IfATM_VccAddr_t *)destParam;
				tempVccAddr2 = (NPF_VccAddr_t *)srcParam;

				for(i=0; i< numEntries; i++)
				{
					/* Copy the parameters from Source to destination */
					tempVccAddr1[i].portId = portId;
					memcpy(&(tempVccAddr1[i].vccAddr),
								&tempVccAddr2[i],sizeof(NPF_VccAddr_t));
				}
			}
			break;

		case NPF_IF_ATM_VCC_ENABLE:
			{
				IfATM_VccAddr_t *tempVccAddr1 = NULL;
				NPF_VccAddr_t *tempVccAddr2 = NULL;

				tempVccAddr1 = (IfATM_VccAddr_t *)destParam;
				tempVccAddr2 = (NPF_VccAddr_t *)srcParam;

				for(i=0; i< numEntries; i++)
				{
					/* Copy the parameters from Source to destination */
					tempVccAddr1[i].portId = portId;
					memcpy(&(tempVccAddr1[i].vccAddr),
							&tempVccAddr2[i],sizeof(NPF_VccAddr_t));
				}
			}
			break;

		case NPF_IF_ATM_VCC_DISABLE:
			{
				IfATM_VccAddr_t *tempVccAddr1 = NULL;
				NPF_VccAddr_t *tempVccAddr2 = NULL;

				tempVccAddr1 = (IfATM_VccAddr_t *)destParam;
				tempVccAddr2 = (NPF_VccAddr_t *)srcParam;

				for(i=0; i< numEntries; i++)
				{
					/* Copy the parameters from Source to destination */
					tempVccAddr1[i].portId = portId;
					memcpy(&(tempVccAddr1[i].vccAddr),
							&tempVccAddr2[i],sizeof(NPF_VccAddr_t));
				}
			}
			break;

		case NPF_IF_ATM_VCC_OPER_STATUS_GET:
			{
				IfATM_VccAddr_t *tempVccAddr1 = NULL;
				NPF_VccAddr_t *tempVccAddr2 = NULL;

				tempVccAddr1 = (IfATM_VccAddr_t *)destParam;
				tempVccAddr2 = (NPF_VccAddr_t *)srcParam;

				for(i=0; i< numEntries; i++)
				{
					/* Copy the parameters from Source to destination */
					tempVccAddr1[i].portId = portId;
					memcpy(&(tempVccAddr1[i].vccAddr),
							&tempVccAddr2[i],sizeof(NPF_VccAddr_t));
				}
			}
			break;

		case NPF_IF_ATM_VCC_STATS_GET:
			{
				IfATM_VccAddr_t *tempVccAddr1 = NULL;
				NPF_VccAddr_t *tempVccAddr2 = NULL;

				tempVccAddr1 = (IfATM_VccAddr_t *)destParam;
				tempVccAddr2 = (NPF_VccAddr_t *)srcParam;

				for(i=0; i< numEntries; i++)
				{
					/* Copy the parameters from Source to destination */
					tempVccAddr1[i].portId = portId;
					memcpy(&(tempVccAddr1[i].vccAddr),
								&tempVccAddr2[i],sizeof(NPF_VccAddr_t));
				}
			}
			break;

		case NPF_IF_ATM_AAL2_CHANNEL_SET:
			{
				NPF_ATM_AAL2_ChannelInfo_t *tempAal2ChanInfo2 = NULL;
				IfATM_AAL2_ChannelInfo_t *tempAal2ChanInfo1 = NULL;

				tempAal2ChanInfo1 = (IfATM_AAL2_ChannelInfo_t *)destParam;
				tempAal2ChanInfo2 = (NPF_ATM_AAL2_ChannelInfo_t *)srcParam;

				for(i=0; i< numEntries; i++)
				{
					/* Copy the Parameters from Source to destination */
					tempAal2ChanInfo1[i].portId = portId;
					memcpy(&(tempAal2ChanInfo1[i].aal2ChanInfo),
								&tempAal2ChanInfo2[i],sizeof(NPF_ATM_AAL2_ChannelInfo_t));
				}
			}
			break;

		case NPF_IF_ATM_AAL2_CHANNEL_DEL:
			{
				IfATM_AAL2_ConnId_t *tempAal2ConId1 = NULL;
				NPF_ATM_AAL2_ConnId_t *tempAal2ConId2 = NULL;

				tempAal2ConId1 = (IfATM_AAL2_ConnId_t *)destParam;
				tempAal2ConId2 = (NPF_ATM_AAL2_ConnId_t *)srcParam;
		
				for(i=0; i< numEntries; i++)
				{
					/* Copy the Parameters from Source to destination */
					tempAal2ConId1[i].portId = portId;
					memcpy(&(tempAal2ConId1[i].aal2ConId),
							&tempAal2ConId2[i],sizeof(NPF_ATM_AAL2_ConnId_t));
				}
			}	
			break;

		case NPF_IF_ATM_CONNECTION_XC_SET:
			break;
		case NPF_IF_ATM_CONNECTION_XC_DEL:
			break;

		case NPF_IF_ATM_OAM_CP_SET:
			{
				NPF_ATM_OAM_CP_t *tempCpInfo2 = NULL;
				IfATM_OAM_CP_t *tempCpInfo1 = NULL;

				tempCpInfo1 = (IfATM_OAM_CP_t *)destParam;
				tempCpInfo2 = (NPF_ATM_OAM_CP_t *)srcParam;	

				for(i=0; i< numEntries; i++)
				{
					/* Copy the Parameters from Source to destination */
					tempCpInfo1[i].portId = portId;
					memcpy(&(tempCpInfo1[i].cpInfo),
							&tempCpInfo2[i],sizeof(NPF_ATM_OAM_CP_t));
				}
			}
			break;

		case NPF_IF_ATM_OAM_CC_SET:
			{
				NPF_ATM_OAM_CC_t *tempCcInfo2 = NULL;
				IfATM_OAM_CC_t *tempCcInfo1 = NULL;
	
				tempCcInfo1 = (IfATM_OAM_CC_t *)destParam;
				tempCcInfo2 = (NPF_ATM_OAM_CC_t *)srcParam;	

				for(i=0; i< numEntries; i++)
				{
					/* Copy the Parameters from Source to destination */
					tempCcInfo1[i].portId = portId;
					memcpy(&(tempCcInfo1[i].ccInfo),
							&tempCcInfo2[i],sizeof(NPF_ATM_OAM_CC_t));
				}
			}	
			break;

		case NPF_IF_ATM_OAM_LB_SET:
			{
				NPF_ATM_OAM_LB_t *tempLbInfo2 = NULL;
				IfATM_OAM_LB_t *tempLbInfo1 = NULL;

				tempLbInfo1 = (IfATM_OAM_LB_t *)destParam;
				tempLbInfo2 = (NPF_ATM_OAM_LB_t *)srcParam;	

				for(i=0; i< numEntries; i++)
				{
					/* Copy the Parameters from Source to destination */
					tempLbInfo1[i].portId = portId;
					memcpy(&(tempLbInfo1[i].lbInfo),
							&tempLbInfo2[i],sizeof(NPF_ATM_OAM_LB_t));
				}
			}
			break;

		case NPF_IF_ATM_OAM_MONITOR:
			{
				NPF_ATM_OAM_Mon_t *tempMonInfo2 = NULL;
				IfATM_OAM_Mon_t *tempMonInfo1 = NULL;

				tempMonInfo1 = (IfATM_OAM_Mon_t *)destParam;
				tempMonInfo2 = (NPF_ATM_OAM_Mon_t *)srcParam;	

				for(i=0; i< numEntries; i++)
				{
					/* Copy the Parameters from Source to destination */
					tempMonInfo1[i].portId = portId;
					memcpy(&(tempMonInfo1[i].monInfo),
							&tempMonInfo2[i],sizeof(NPF_ATM_OAM_Mon_t));
				}
			}
			break;

		case NPF_IF_ATM_OAM_ALARM_SET:
			{
				IfATM_OAM_Alarm_t *tempAlarmInfo1 = NULL;
				NPF_ATM_OAM_Alarm_t *tempAlarmInfo2 = NULL;	

				tempAlarmInfo1 = (IfATM_OAM_Alarm_t *)destParam;
				tempAlarmInfo2 = (NPF_ATM_OAM_Alarm_t *)srcParam;	

				for(i=0; i< numEntries; i++)
				{
					/* Copy the Parameters from Source to destination */
					tempAlarmInfo1[i].portId = portId;
					memcpy(&(tempAlarmInfo1[i].oamAlrmInfo),
							&tempAlarmInfo2[i],sizeof(NPF_ATM_OAM_Alarm_t));
				}
			}	
			break;

		case NPF_IF_ATM_OAM_ALARM_CLEAR:
			{
				IfATM_OAM_Alarm_t *tempAlarmInfo1 = NULL;
				NPF_ATM_OAM_Alarm_t *tempAlarmInfo2 = NULL;	

				tempAlarmInfo1 = (IfATM_OAM_Alarm_t *)destParam;
				tempAlarmInfo2 = (NPF_ATM_OAM_Alarm_t *)srcParam;	

				for(i=0; i< numEntries; i++)
				{
					/* Copy the Parameters from Source to destination */
					tempAlarmInfo1[i].portId = portId;
					memcpy(&(tempAlarmInfo1[i].oamAlrmInfo),
							&tempAlarmInfo2[i],sizeof(NPF_ATM_OAM_Alarm_t));

				}
			}	
			break;

		case NPF_IF_ATM_OAM_PM_SET:
			{
				NPF_ATM_OAM_PM_t *tempPmInfo2 = NULL;
				IfATM_OAM_PM_t *tempPmInfo1 = NULL;

				tempPmInfo1 = (IfATM_OAM_PM_t *)destParam;
				tempPmInfo2 = (NPF_ATM_OAM_PM_t *)srcParam;	
			

				for(i=0; i< numEntries; i++)
				{
					/* Copy the Parameters from Source to destination */
					tempPmInfo1[i].portId = portId;
					memcpy(&(tempPmInfo1[i].pmInfo),
							&tempPmInfo2[i],sizeof(NPF_ATM_OAM_PM_t));
				}
			}
			break;

			default:
					break;
	}/* end of switch */

	return NPF_SUCCESS;

}			
	
	
/*****************************************************************************
*    Name: ifATM_CheckUserParams 
*
*    Description:
*			This function checks the validity of the parameters passed to it. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - Success 
*        NPF_IF_E_INVALID_PARAM      - Invalid Parameters 
*
********************************************************************************/
NPF_error_t ifATM_CheckUserParams(
			NPF_IN NPF_callbackHandle_t	if_cbHandle,
			NPF_IN NPF_errorReporting_t if_errorReporting,
			NPF_IN NPF_IfHandle_t ifHandle,
			NPF_IN NPF_uint32_t count,
			NPF_IN void *ptr)
{
	
	NPF_DATA data = NULL;

	/* Trace the function */
	assert(ifATM_CheckUserParams);

	/* Check if Call back Handle registered */
	if (pdk_callback_isCBExist(if_cbHandle) == NPF_IF_E_INVALID_PARAM)
        return NPF_IF_E_BAD_CALLBACK_FUNCTION;

	/* Check if the interface has been created and its handle exists */
	if (cm_getDataByHandle(ifHandle, &data) == -1)
        return NPF_IF_E_INVALID_HANDLE;

	/* Check if the error reporting value passed is valid */
	if( (if_errorReporting < NPF_REPORT_ALL) || (if_errorReporting > NPF_REPORT_ERRORS) )
        return NPF_IF_E_INVALID_PARAM;

	/* Check values of count and ptr passed */
	if((count != 0) && (ptr == NULL))
        return NPF_IF_E_INVALID_PARAM;
		
	if(count <= 0)
        return NPF_IF_E_BAD_ARRAY_LENGTH;

	return NPF_SUCCESS;	

}



/*****************************************************************************
*    Name: ifATM_ProcessUserRequest 
*
*    Description:
*			This function does the necessary processing required for the received 
* 			user request.
* 
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM		 - Invalid Parameters
*
********************************************************************************/
NPF_error_t ifATM_ProcessUserRequest(IfATM_RequestCB_t	*request)
{

	NPF_error_t		ret = NPF_SUCCESS;
	
	switch( (request->cbData).type)
	{
		case NPF_IF_ATM_VCC_SET:
				ret = ifATM_ProcessVccSetReq(request);
				break;

		case NPF_IF_ATM_VCC_BIND:
				ret = ifATM_ProcessVccBindReq(request);
				break;

		case NPF_IF_ATM_VCC_DELETE:
				ret = ifATM_ProcessVccDelReq(request);
				break;
		
		case NPF_IF_ATM_VCC_ENABLE:
				ret = ifATM_ProcessVccEnableReq(request);
				break;
			
		case NPF_IF_ATM_VCC_DISABLE:
				ret = ifATM_ProcessVccDisableReq(request);
				break;
		
		case NPF_IF_ATM_VCC_OPER_STATUS_GET:
				ret = ifATM_ProcessVccOperStatusGetReq(request);
				break;

		case NPF_IF_ATM_VCC_STATS_GET:
				ret = ifATM_ProcessVccStatsGetReq(request);
				break;

		case NPF_IF_ATM_AAL2_CHANNEL_SET:
				ret = ifATM_ProcessAal2ChanSetReq(request);
				break;

		case NPF_IF_ATM_AAL2_CHANNEL_DEL:
				ret = ifATM_ProcessAal2ChanDelReq(request);
				break;

		case NPF_IF_ATM_CONNECTION_XC_SET:
				ret = ifATM_ProcessConXcSetReq(request);
				break;

		case NPF_IF_ATM_CONNECTION_XC_DEL:
				ret = ifATM_ProcessConXcDelReq(request);
				break;

		case NPF_IF_ATM_OAM_CP_SET:
				ret = ifATM_ProcessOamCpSetReq(request);
				break;

		case NPF_IF_ATM_OAM_CC_SET:
				ret = ifATM_ProcessOamCcSetReq(request);
				break;

		case NPF_IF_ATM_OAM_LB_SET:
				ret = ifATM_ProcessOamLbSetReq(request);
				break;

		case NPF_IF_ATM_OAM_MONITOR:
				ret = ifATM_ProcessOamMonitorReq(request);
				break;

		case NPF_IF_ATM_OAM_ALARM_SET:
				ret = ifATM_ProcessOamAlrmSetReq(request);
				break;

		case NPF_IF_ATM_OAM_ALARM_CLEAR:
				ret = ifATM_ProcessOamAlrmClearReq(request);
				break;

		case NPF_IF_ATM_OAM_PM_SET:
				ret = ifATM_ProcessOamPmSetReq(request);
				break;

		default:
				ret = NPF_IF_E_INVALID_PARAM;
				break;
		}

		return ret;
}


/*****************************************************************************
*    Name: ifATM_ProcessVccSetReq 
*
*    Description:
*			This function does the necessary processing required for the received 
* 			user request for setting a new ATM VCC.
* 
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM		 - invalid parameters
*
********************************************************************************/
NPF_error_t ifATM_ProcessVccSetReq(IfATM_RequestCB_t	*request)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_FeTransInfo_t *corr = NULL;
	NPF_uint32_t i = 0;
	IfATM_IfVcc_t *ptr = NULL;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	IfATM_FpResponse_t tempResp;


	/* Make ptr to point to the entry member of apiData */
	ptr = (IfATM_IfVcc_t *)(request->apiData).entry;

	for(i = 0; i < (request->apiData).numEntries; i++)
	{
		/* Allocate Memory for the correlator */
		corr = (IfATM_FeTransInfo_t *)PIL_AllocateMemory
						(sizeof(IfATM_FeTransInfo_t) , PIL_MEM_ZERO_INIT);	

		if(corr == NULL)
			return NPF_OUT_OF_MEMORY;

		/* Create the Correlator to associate with this request */
		corr->reqCbPtr = request;
		corr->entryPtr = &ptr[i];
		corr->feId = (request->feInfo[0]).feId;
	
		/* Add the Correlator in the fpTransList of the request CB */
		npf_list_pushBack(&(request->fpTransList),(void *)corr);


		/* Increment the Count of num FP Transactions pending */
		request->numTransWait++;
		request->numWaitFEs++;
		((request->feInfo[0]).numWait)++;

		/* Call the FP Plugin API */
		fpRetVal = FPPAPI_ATM_VccSetFunc((FPPI_FEID)(request->feInfo[0]).feId	,
							(FPPI_CORRELATOR)corr,
							(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_VCC_SET_CB],
							(IfATM_IfVcc_t *)&ptr[i],
							1);

		/* Check if the FP-Plug in API succedded */
		if(fpRetVal != FPPI_SUCCESS)
		{

			switch(fpRetVal)
			{
				case FPPI_OUT_OF_MEMORY:
					tempResp.status =  NPF_OUT_OF_MEMORY;
					tempResp.reason =  NPF_OUT_OF_MEMORY;
					break;
				case FPPI_INVALID_PARAMETERS:
				case FPPI_FAIL:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
				case FPPI_NOT_INITIALIZED:
					tempResp.status = NPF_COMPONENT_UNINITIALIZED;
					tempResp.reason = NPF_COMPONENT_UNINITIALIZED;
					break;
				default:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
			}/* end of Switch(fpRetVal) */

			/* Call the ATM callback handler function to handle the failure 
				of the FP-Plugin Call */
			ifAtmOnCB_VccSetFunc((FPPI_FEID)(request->feInfo[0]).feId,
									(FPPI_CORRELATOR)corr,
									(FPPI_CONTEXT)NPF_IF_ATM_VCC_SET,
									(void *)&tempResp);

		}/* end of if condition */

	}/* end of the for loop */

	return ret;
	
}


/*****************************************************************************
*    Name: ifATM_ProcessVccBindReq 
*
*    Description:
*			This function does the necessary processing required for the received 
* 			user request for Binding an ATM VCC with an upper interface.
* 
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM		 - invalid parameters
*
********************************************************************************/
NPF_error_t ifATM_ProcessVccBindReq(IfATM_RequestCB_t	*request)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_FeTransInfo_t *corr = NULL;
	NPF_uint32_t i = 0;
	IfATM_VccBind_t *ptr = NULL;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	IfATM_FpResponse_t tempResp;


	/* Make ptr to point to the entry member of apiData */
	ptr = (IfATM_VccBind_t *)(request->apiData).entry;

	for(i = 0; i < (request->apiData).numEntries; i++)
	{
		/* Allocate Memory for the correlator */
		corr = (IfATM_FeTransInfo_t *)PIL_AllocateMemory
						(sizeof(IfATM_FeTransInfo_t) , PIL_MEM_ZERO_INIT);	

		if(corr == NULL)
			return NPF_OUT_OF_MEMORY;

		/* Create the Correlator to associate with this request */
		corr->reqCbPtr = request;
		corr->entryPtr = &ptr[i];
		corr->feId = (request->feInfo[0]).feId;
	
		/* Add the Correlator in the fpTransList of the request CB */
		npf_list_pushBack(&(request->fpTransList),(void *)corr);


		/* Increment the Count of num FP Transactions pending */
		request->numTransWait++;
		request->numWaitFEs++;
		((request->feInfo[0]).numWait)++;

		/* Call the FP Plugin API */
		fpRetVal = FPPAPI_ATM_VccBindFunc((request->feInfo[0]).feId	,
							(FPPI_CORRELATOR)corr,
							(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_VCC_BIND_CB],
							(IfATM_VccBind_t *)&ptr[i],
							1);

		/* Check if the FP-Plug in API succedded */
		if(fpRetVal != FPPI_SUCCESS)
		{

			switch(fpRetVal)
			{
				case FPPI_OUT_OF_MEMORY:
					tempResp.status =  NPF_OUT_OF_MEMORY;
					tempResp.reason =  NPF_OUT_OF_MEMORY;
					break;
				case FPPI_INVALID_PARAMETERS:
				case FPPI_FAIL:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
				case FPPI_NOT_INITIALIZED:
					tempResp.status = NPF_COMPONENT_UNINITIALIZED;
					tempResp.reason = NPF_COMPONENT_UNINITIALIZED;
					break;
				default:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
			}/* end of Switch(fpRetVal) */

			/* Call the ATM callback handler function to handle the failure 
				of the FP-Plugin Call */
			ifAtmOnCB_VccBindFunc((FPPI_FEID)(request->feInfo[0]).feId,
									(FPPI_CORRELATOR)corr,
									(FPPI_CONTEXT)NPF_IF_ATM_VCC_BIND,
									(void *)&tempResp);

		}/* end of if condition */
	}/* end of the for loop */

	return ret;
	
}



/*****************************************************************************
*    Name: ifATM_ProcessVccDelReq 
*
*    Description:
*			This function does the necessary processing required for the received 
* 			user request for deleting an ATM VCC.
* 
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM		 - invalid parameters
*
********************************************************************************/
NPF_error_t ifATM_ProcessVccDelReq(IfATM_RequestCB_t	*request)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_FeTransInfo_t *corr = NULL;
	NPF_uint32_t i = 0;
	IfATM_VccAddr_t *ptr = NULL;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	IfATM_FpResponse_t tempResp;


	/* Make ptr to point to the entry member of apiData */
	ptr = (IfATM_VccAddr_t *)(request->apiData).entry;

	for(i = 0; i < (request->apiData).numEntries; i++)
	{
		/* Allocate Memory for the correlator */
		corr = (IfATM_FeTransInfo_t *)PIL_AllocateMemory
						(sizeof(IfATM_FeTransInfo_t) , PIL_MEM_ZERO_INIT);	

		if(corr == NULL)
			return NPF_OUT_OF_MEMORY;

		/* Create the Correlator to associate with this request */
		corr->reqCbPtr = request;
		corr->entryPtr = &ptr[i];
		corr->feId = (request->feInfo[0]).feId;
	
		/* Add the Correlator in the fpTransList of the request CB */
		npf_list_pushBack(&(request->fpTransList),(void *)corr);

		/* Increment the Count of num FP Transactions pending */
		request->numTransWait++;
		request->numWaitFEs++;
		((request->feInfo[0]).numWait)++;

		/* Call the FP Plugin API */
		fpRetVal = FPPAPI_ATM_VccDeleteFunc((request->feInfo[0]).feId	,
							(FPPI_CORRELATOR)corr,
							(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_VCC_DEL_CB],
							(IfATM_VccAddr_t *)&ptr[i],
							 1);

		/* Check if the FP-Plug in API succedded */
		if(fpRetVal != FPPI_SUCCESS)
		{

			switch(fpRetVal)
			{
				case FPPI_OUT_OF_MEMORY:
					tempResp.status =  NPF_OUT_OF_MEMORY;
					tempResp.reason =  NPF_OUT_OF_MEMORY;
					break;
				case FPPI_INVALID_PARAMETERS:
				case FPPI_FAIL:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
				case FPPI_NOT_INITIALIZED:
					tempResp.status = NPF_COMPONENT_UNINITIALIZED;
					tempResp.reason = NPF_COMPONENT_UNINITIALIZED;
					break;
				default:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
			}/* end of Switch(fpRetVal) */

			/* Call the ATM callback handler function to handle the failure 
				of the FP-Plugin Call */
			ifAtmOnCB_VccDelFunc((FPPI_FEID)(request->feInfo[0]).feId,
									(FPPI_CORRELATOR)corr,
									(FPPI_CONTEXT)NPF_IF_ATM_VCC_DELETE,
									(void *)&tempResp);

		}/* end of if condition */
	}/* end of the for loop */

	return ret;
	
}



/*****************************************************************************
*    Name: ifATM_ProcessVccEnableReq 
*
*    Description:
*			This function does the necessary processing required for the received 
* 			user request for enabling an ATM VCC.
* 
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM		 - invalid parameters
*
********************************************************************************/
NPF_error_t ifATM_ProcessVccEnableReq(IfATM_RequestCB_t	*request)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_FeTransInfo_t *corr = NULL;
	NPF_uint32_t i = 0;
	IfATM_VccAddr_t *ptr = NULL;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	IfATM_FpResponse_t tempResp;

	/* Make ptr to point to the entry member of apiData */
	ptr = (IfATM_VccAddr_t *)(request->apiData).entry;

	for(i = 0; i < (request->apiData).numEntries; i++)
	{
		/* Allocate Memory for the correlator */
		corr = (IfATM_FeTransInfo_t *)PIL_AllocateMemory
						(sizeof(IfATM_FeTransInfo_t) , PIL_MEM_ZERO_INIT);	

		if(corr == NULL)
			return NPF_OUT_OF_MEMORY;

		/* Create the Correlator to associate with this request */
		corr->reqCbPtr = request;
		corr->entryPtr = &ptr[i];
		corr->feId = (request->feInfo[0]).feId;
	
		/* Add the Correlator in the fpTransList of the request CB */
		npf_list_pushBack(&(request->fpTransList),(void *)corr);

		/* Increment the Count of num FP Transactions pending */
		request->numTransWait++;
		request->numWaitFEs++;
		((request->feInfo[0]).numWait)++;

		/* Call the FP Plugin API */
		fpRetVal = FPPAPI_ATM_VccEnableFunc((FPPI_FEID)(request->feInfo[0]).feId,
							(FPPI_CORRELATOR)corr,
							(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_VCC_ENABLE_CB],
							(IfATM_VccAddr_t *)&ptr[i],
							(NPF_uint32_t)1);

		/* Check if the FP-Plug in API succedded */
		if(fpRetVal != FPPI_SUCCESS)
		{

			switch(fpRetVal)
			{
				case FPPI_OUT_OF_MEMORY:
					tempResp.status =  NPF_OUT_OF_MEMORY;
					tempResp.reason =  NPF_OUT_OF_MEMORY;
					break;
				case FPPI_INVALID_PARAMETERS:
				case FPPI_FAIL:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
				case FPPI_NOT_INITIALIZED:
					tempResp.status = NPF_COMPONENT_UNINITIALIZED;
					tempResp.reason = NPF_COMPONENT_UNINITIALIZED;
					break;
				default:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
			}/* end of Switch(fpRetVal) */

			/* Call the ATM callback handler function to handle the failure 
				of the FP-Plugin Call */
			ifAtmOnCB_VccEnableFunc((FPPI_FEID)(request->feInfo[0]).feId,
									(FPPI_CORRELATOR)corr,
									(FPPI_CONTEXT)NPF_IF_ATM_VCC_ENABLE,
									(void *)&tempResp);

		}/* end of if condition */
	}/* end of the for loop */

	return ret;
}
	


/*****************************************************************************
*    Name: ifATM_ProcessVccDisableReq 
*
*    Description:
*			This function does the necessary processing required for the received 
* 			user request for disabling an ATM VCC.
* 
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM		 - invalid parameters
*
********************************************************************************/
NPF_error_t ifATM_ProcessVccDisableReq(IfATM_RequestCB_t	*request)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_FeTransInfo_t *corr = NULL;
	NPF_uint32_t i = 0;
	IfATM_VccAddr_t *ptr = NULL;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	IfATM_FpResponse_t tempResp;
	


	/* Make ptr to point to the entry member of apiData */
	ptr = (IfATM_VccAddr_t *)(request->apiData).entry;

	for(i = 0; i < (request->apiData).numEntries; i++)
	{
		/* Allocate Memory for the correlator */
		corr = (IfATM_FeTransInfo_t *)PIL_AllocateMemory
						(sizeof(IfATM_FeTransInfo_t) , PIL_MEM_ZERO_INIT);	

		if(corr == NULL)
			return NPF_OUT_OF_MEMORY;

		/* Create the Correlator to associate with this request */
		corr->reqCbPtr = request;
		corr->entryPtr = &ptr[i];
		corr->feId = (request->feInfo[0]).feId;
	
		/* Add the Correlator in the fpTransList of the request CB */
		npf_list_pushBack(&(request->fpTransList),(void *)corr);

		/* Increment the Count of num FP Transactions pending */
		request->numTransWait++;
		request->numWaitFEs++;
		((request->feInfo[0]).numWait)++;

		/* Call the FP Plugin API */
		fpRetVal = FPPAPI_ATM_VccDisableFunc((FPPI_FEID)(request->feInfo[0]).feId,
							(FPPI_CORRELATOR)corr,
							(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_VCC_DISABLE_CB],
							(IfATM_VccAddr_t *)&ptr[i],
							(NPF_uint32_t)1);

		/* Check if the FP-Plug in API succedded */
		if(fpRetVal != FPPI_SUCCESS)
		{

			switch(fpRetVal)
			{
				case FPPI_OUT_OF_MEMORY:
					tempResp.status =  NPF_OUT_OF_MEMORY;
					tempResp.reason =  NPF_OUT_OF_MEMORY;
					break;
				case FPPI_INVALID_PARAMETERS:
				case FPPI_FAIL:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
				case FPPI_NOT_INITIALIZED:
					tempResp.status = NPF_COMPONENT_UNINITIALIZED;
					tempResp.reason = NPF_COMPONENT_UNINITIALIZED;
					break;
				default:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
			}/* end of Switch(fpRetVal) */

			/* Call the ATM callback handler function to handle the failure 
				of the FP-Plugin Call */
			ifAtmOnCB_VccDisableFunc((FPPI_FEID)(request->feInfo[0]).feId,
									(FPPI_CORRELATOR)corr,
									(FPPI_CONTEXT)NPF_IF_ATM_VCC_DISABLE,
									(void *)&tempResp);

		}/* end of if condition */
	}/* end of the for loop */

	return ret;
}
	

/*****************************************************************************
*    Name: ifATM_ProcessVccOperStatusGetReq 
*
*    Description:
*			This function does the necessary processing required for the received 
* 			user request for getting the operational status of an ATM VCC.
* 
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM		 - invalid parameters
*
********************************************************************************/
NPF_error_t ifATM_ProcessVccOperStatusGetReq(IfATM_RequestCB_t	*request)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_FeTransInfo_t *corr = NULL;
	NPF_uint32_t i = 0;
	IfATM_VccAddr_t *ptr = NULL;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	IfATM_FpResponse_t tempResp;


	/* Make ptr to point to the entry member of apiData */
	ptr = (IfATM_VccAddr_t *)(request->apiData).entry;

	for(i = 0; i < (request->apiData).numEntries; i++)
	{
		/* Allocate Memory for the correlator */
		corr = (IfATM_FeTransInfo_t *)PIL_AllocateMemory
						(sizeof(IfATM_FeTransInfo_t) , PIL_MEM_ZERO_INIT);	

		if(corr == NULL)
			return NPF_OUT_OF_MEMORY;

		/* Create the Correlator to associate with this request */
		corr->reqCbPtr = request;
		corr->entryPtr = &ptr[i];
		corr->feId = (request->feInfo[0]).feId;
	
		/* Add the Correlator in the fpTransList of the request CB */
		npf_list_pushBack(&(request->fpTransList),(void *)corr);


		/* Increment the Count of num FP Transactions pending */
		request->numTransWait++;
		request->numWaitFEs++;
		((request->feInfo[0]).numWait)++;

		/* Call the FP Plugin API */
		fpRetVal = FPPAPI_ATM_VccOperStatusGetFunc((FPPI_FEID)(request->feInfo[0]).feId,
							(FPPI_CORRELATOR)corr,
							(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_VCC_OPER_STATUS_GET_CB],
							(IfATM_VccAddr_t *)&ptr[i],
							(NPF_uint32_t)1);

		/* Check if the FP-Plug in API succedded */
		if(fpRetVal != FPPI_SUCCESS)
		{

			switch(fpRetVal)
			{
				case FPPI_OUT_OF_MEMORY:
					tempResp.status =  NPF_OUT_OF_MEMORY;
					tempResp.reason =  NPF_OUT_OF_MEMORY;
					break;
				case FPPI_INVALID_PARAMETERS:
				case FPPI_FAIL:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
				case FPPI_NOT_INITIALIZED:
					tempResp.status = NPF_COMPONENT_UNINITIALIZED;
					tempResp.reason = NPF_COMPONENT_UNINITIALIZED;
					break;
				default:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
			}/* end of Switch(fpRetVal) */

			/* Call the ATM callback handler function to handle the failure 
				of the FP-Plugin Call */
			ifAtmOnCB_VccOperStatusGetFunc((FPPI_FEID)(request->feInfo[0]).feId,
									(FPPI_CORRELATOR)corr,
									(FPPI_CONTEXT)NPF_IF_ATM_VCC_OPER_STATUS_GET,
									(void *)&tempResp);

		}/* end of if condition */
	}/* end of the for loop */


	return ret;
}
	

/*****************************************************************************
*    Name: ifATM_ProcessVccStatsGetReq 
*
*    Description:
*			This function does the necessary processing required for the received 
* 			user request for getting the statistics of an ATM VCC.
* 
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM		 - invalid parameters
*
********************************************************************************/
NPF_error_t ifATM_ProcessVccStatsGetReq(IfATM_RequestCB_t	*request)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_FeTransInfo_t *corr = NULL;
	NPF_uint32_t i = 0;
	IfATM_VccAddr_t *ptr = NULL;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	IfATM_VccStats_t tempResp;


	/* Make ptr to point to the entry member of apiData */
	ptr = (IfATM_VccAddr_t *)(request->apiData).entry;

	for(i = 0; i < (request->apiData).numEntries; i++)
	{
		/* Allocate Memory for the correlator */
		corr = (IfATM_FeTransInfo_t *)PIL_AllocateMemory
						(sizeof(IfATM_FeTransInfo_t) , PIL_MEM_ZERO_INIT);	

		if(corr == NULL)
			return NPF_OUT_OF_MEMORY;

		/* Create the Correlator to associate with this request */
		corr->reqCbPtr = request;
		corr->entryPtr = &ptr[i];
		corr->feId = (request->feInfo[0]).feId;
	
		/* Add the Correlator in the fpTransList of the request CB */
		npf_list_pushBack(&(request->fpTransList),(void *)corr);

		/* Increment the Count of num FP Transactions pending */
		request->numTransWait++;
		request->numWaitFEs++;
		((request->feInfo[0]).numWait)++;

		/* Call the FP Plugin API */
		fpRetVal = FPPAPI_ATM_VccStatsGetFunc((FPPI_FEID)(request->feInfo[0]).feId,
							(FPPI_CORRELATOR)corr,
							(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_VCC_STATS_GET_CB],
							(IfATM_VccAddr_t *)&ptr[i],
							(NPF_uint32_t)1);

		/* Check if the FP-Plug in API succedded */
		if(fpRetVal != FPPI_SUCCESS)
		{

			switch(fpRetVal)
			{
				case FPPI_OUT_OF_MEMORY:
					tempResp.response.status =  NPF_OUT_OF_MEMORY;
					tempResp.response.reason =  NPF_OUT_OF_MEMORY;
					break;
				case FPPI_INVALID_PARAMETERS:
				case FPPI_FAIL:
					tempResp.response.status = NPF_IF_E_INVALID_PARAM;
					tempResp.response.reason = NPF_IF_E_INVALID_PARAM;
					break;
				case FPPI_NOT_INITIALIZED:
					tempResp.response.status = NPF_COMPONENT_UNINITIALIZED;
					tempResp.response.reason = NPF_COMPONENT_UNINITIALIZED;
					break;
				default:
					tempResp.response.status = NPF_IF_E_INVALID_PARAM;
					tempResp.response.reason = NPF_IF_E_INVALID_PARAM;
					break;
			}/* end of Switch(fpRetVal) */

			/* Call the ATM callback handler function to handle the failure 
				of the FP-Plugin Call */
			ifAtmOnCB_VccStatsGetFunc((FPPI_FEID)(request->feInfo[0]).feId,
									(FPPI_CORRELATOR)corr,
									(FPPI_CONTEXT)NPF_IF_ATM_VCC_STATS_GET,
									(void *)&tempResp);

		}/* end of if condition */
	}/* end of the for loop */


	return ret;
}
	


/*****************************************************************************
*    Name: ifATM_ProcessAal2ChanSetReq 
*
*    Description:
*			This function does the necessary processing required for the received 
* 			user request for setting an AAL2 channel on an ATM VCC.
* 
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM		 - invalid parameters
*
********************************************************************************/
NPF_error_t ifATM_ProcessAal2ChanSetReq(IfATM_RequestCB_t	*request)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_FeTransInfo_t *corr = NULL;
	NPF_uint32_t i = 0;
	IfATM_AAL2_ChannelInfo_t *ptr = NULL;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	IfATM_FpResponse_t tempResp;


	/* Make ptr to point to the entry member of apiData */
	ptr = (IfATM_AAL2_ChannelInfo_t *)(request->apiData).entry;

	for(i = 0; i < (request->apiData).numEntries; i++)
	{
		/* Allocate Memory for the correlator */
		corr = (IfATM_FeTransInfo_t *)PIL_AllocateMemory
						(sizeof(IfATM_FeTransInfo_t) , PIL_MEM_ZERO_INIT);	

		if(corr == NULL)
			return NPF_OUT_OF_MEMORY;

		/* Create the Correlator to associate with this request */
		corr->reqCbPtr = request;
		corr->entryPtr = &ptr[i];
		corr->feId = (request->feInfo[0]).feId;
	
		/* Add the Correlator in the fpTransList of the request CB */
		npf_list_pushBack(&(request->fpTransList),(void *)corr);


		/* Increment the Count of num FP Transactions pending */
		request->numTransWait++;
		request->numWaitFEs++;
		((request->feInfo[0]).numWait)++;

		/* Call the FP Plugin API */
		fpRetVal = FPPAPI_ATM_AAL2_ChannelSetFunc((FPPI_FEID)(request->feInfo[0]).feId,
							(FPPI_CORRELATOR)corr,
							(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_AAL2_CHANNEL_SET_CB],
							(IfATM_AAL2_ChannelInfo_t *)&ptr[i],
							(NPF_uint32_t)1);

		/* Check if the FP-Plug in API succedded */
		if(fpRetVal != FPPI_SUCCESS)
		{

			switch(fpRetVal)
			{
				case FPPI_OUT_OF_MEMORY:
					tempResp.status =  NPF_OUT_OF_MEMORY;
					tempResp.reason =  NPF_OUT_OF_MEMORY;
					break;
				case FPPI_INVALID_PARAMETERS:
				case FPPI_FAIL:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
				case FPPI_NOT_INITIALIZED:
					tempResp.status = NPF_COMPONENT_UNINITIALIZED;
					tempResp.reason = NPF_COMPONENT_UNINITIALIZED;
					break;
				default:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
			}/* end of Switch(fpRetVal) */

			/* Call the ATM callback handler function to handle the failure 
				of the FP-Plugin Call */
			ifAtmOnCB_Aal2ChannelSetFunc((FPPI_FEID)(request->feInfo[0]).feId,
									(FPPI_CORRELATOR)corr,
									(FPPI_CONTEXT)NPF_IF_ATM_AAL2_CHANNEL_SET,
									(void *)&tempResp);

		}/* end of if condition */
	}/* end of the for loop */

	return ret;
}
	


/*****************************************************************************
*    Name: ifATM_ProcessAal2ChanDelReq 
*
*    Description:
*			This function does the necessary processing required for the received 
* 			user request for deleting an AAL2 channel on an ATM VCC.
* 
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM		 - invalid parameters
*
********************************************************************************/
NPF_error_t ifATM_ProcessAal2ChanDelReq(IfATM_RequestCB_t	*request)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_FeTransInfo_t *corr = NULL;
	NPF_uint32_t i = 0;
	IfATM_AAL2_ConnId_t *ptr = NULL;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	IfATM_FpResponse_t tempResp;


	/* Make ptr to point to the entry member of apiData */
	ptr = (IfATM_AAL2_ConnId_t *)(request->apiData).entry;

	for(i = 0; i < (request->apiData).numEntries; i++)
	{
		/* Allocate Memory for the correlator */
		corr = (IfATM_FeTransInfo_t *)PIL_AllocateMemory
						(sizeof(IfATM_FeTransInfo_t) , PIL_MEM_ZERO_INIT);	

		if(corr == NULL)
			return NPF_OUT_OF_MEMORY;

		/* Create the Correlator to associate with this request */
		corr->reqCbPtr = request;
		corr->entryPtr = &ptr[i];
		corr->feId = (request->feInfo[0]).feId;
	
		/* Add the Correlator in the fpTransList of the request CB */
		npf_list_pushBack(&(request->fpTransList),(void *)corr);

		/* Increment the Count of num FP Transactions pending */
		request->numTransWait++;
		request->numWaitFEs++;
		((request->feInfo[0]).numWait)++;

		/* Call the FP Plugin API */
		fpRetVal = FPPAPI_ATM_AAL2_ChannelDeleteFunc((FPPI_FEID)(request->feInfo[0]).feId,
							(FPPI_CORRELATOR)corr,
							(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_AAL2_CHANNEL_DEL_CB],
							(IfATM_AAL2_ConnId_t *)&ptr[i],
							(NPF_uint32_t)1);

		/* Check if the FP-Plug in API succedded */
		if(fpRetVal != FPPI_SUCCESS)
		{

			switch(fpRetVal)
			{
				case FPPI_OUT_OF_MEMORY:
					tempResp.status =  NPF_OUT_OF_MEMORY;
					tempResp.reason =  NPF_OUT_OF_MEMORY;
					break;
				case FPPI_INVALID_PARAMETERS:
				case FPPI_FAIL:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
				case FPPI_NOT_INITIALIZED:
					tempResp.status = NPF_COMPONENT_UNINITIALIZED;
					tempResp.reason = NPF_COMPONENT_UNINITIALIZED;
					break;
				default:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
			}/* end of Switch(fpRetVal) */

			/* Call the ATM callback handler function to handle the failure 
				of the FP-Plugin Call */
			ifAtmOnCB_Aal2ChannelDelFunc((FPPI_FEID)(request->feInfo[0]).feId,
									(FPPI_CORRELATOR)corr,
									(FPPI_CONTEXT)NPF_IF_ATM_AAL2_CHANNEL_DEL,
									(void *)&tempResp);

		}/* end of if condition */
	}/* end of the for loop */

	return ret;
}
	


/*****************************************************************************
*    Name: ifATM_ProcessConXcSetReq 
*
*    Description:
*			This function does the necessary processing required for the received 
* 			user request for setting an ATM cross connection.
* 
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM		 - invalid parameters
*
********************************************************************************/
NPF_error_t ifATM_ProcessConXcSetReq(IfATM_RequestCB_t	*request)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_FeTransInfo_t *corr = NULL;
	NPF_uint32_t i = 0;
	NPF_ATM_ConnectionXc_t *ptr = NULL;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	IfATM_FpResponse_t tempResp;


	/* Make ptr to point to the entry member of apiData */
	ptr = (NPF_ATM_ConnectionXc_t *)(request->apiData).entry;

	for(i = 0; i < (request->apiData).numEntries; i++)
	{
		/* Allocate Memory for the correlator */
		corr = (IfATM_FeTransInfo_t *)PIL_AllocateMemory
						(sizeof(IfATM_FeTransInfo_t) , PIL_MEM_ZERO_INIT);	

		if(corr == NULL)
			return NPF_OUT_OF_MEMORY;

		/* Create the Correlator to associate with this request */
		corr->reqCbPtr = request;
		corr->entryPtr = &ptr[i];
		corr->feId = (request->feInfo[0]).feId;
	
		/* Add the Correlator in the fpTransList of the request CB */
		npf_list_pushBack(&(request->fpTransList),(void *)corr);

	
		/* Increment the Count of num FP Transactions pending */
		request->numTransWait++;
		request->numWaitFEs++;
		((request->feInfo[0]).numWait)++;

		/* Call the FP Plugin API */
		fpRetVal = FPPAPI_ATM_ConnectionXcSetFunc((FPPI_FEID)(request->feInfo[0]).feId,
							(FPPI_CORRELATOR)corr,
							(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_CONNECTION_XC_SET_CB],
							(NPF_ATM_ConnectionXc_t *)&ptr[i],
							(NPF_uint32_t)1);

		/* Check if the FP-Plug in API succedded */
		if(fpRetVal != FPPI_SUCCESS)
		{

			switch(fpRetVal)
			{
				case FPPI_OUT_OF_MEMORY:
					tempResp.status =  NPF_OUT_OF_MEMORY;
					tempResp.reason =  NPF_OUT_OF_MEMORY;
					break;
				case FPPI_INVALID_PARAMETERS:
				case FPPI_FAIL:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
				case FPPI_NOT_INITIALIZED:
					tempResp.status = NPF_COMPONENT_UNINITIALIZED;
					tempResp.reason = NPF_COMPONENT_UNINITIALIZED;
					break;
				default:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
			}/* end of Switch(fpRetVal) */

			/* Call the ATM callback handler function to handle the failure 
				of the FP-Plugin Call */
			ifAtmOnCB_ConnXcSetFunc((FPPI_FEID)(request->feInfo[0]).feId,
									(FPPI_CORRELATOR)corr,
									(FPPI_CONTEXT)NPF_IF_ATM_CONNECTION_XC_SET,
									(void *)&tempResp);

		}/* end of if condition */
	}/* end of the for loop */


	return ret;
}
	



/*****************************************************************************
*    Name: ifATM_ProcessConXcDelReq 
*
*    Description:
*			This function does the necessary processing required for the received 
* 			user request for deleting an ATM cross connection.
* 
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM		 - invalid parameters
*
********************************************************************************/
NPF_error_t ifATM_ProcessConXcDelReq(IfATM_RequestCB_t	*request)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_FeTransInfo_t *corr = NULL;
	NPF_uint32_t i = 0;
	NPF_ATM_ConnectionXc_t *ptr = NULL;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	IfATM_FpResponse_t tempResp;


	/* Make ptr to point to the entry member of apiData */
	ptr = (NPF_ATM_ConnectionXc_t *)(request->apiData).entry;

	for(i = 0; i < (request->apiData).numEntries; i++)
	{
		/* Allocate Memory for the correlator */
		corr = (IfATM_FeTransInfo_t *)PIL_AllocateMemory
						(sizeof(IfATM_FeTransInfo_t) , PIL_MEM_ZERO_INIT);	

		if(corr == NULL)
			return NPF_OUT_OF_MEMORY;

		/* Create the Correlator to associate with this request */
		corr->reqCbPtr = request;
		corr->entryPtr = &ptr[i];

		corr->feId = (request->feInfo[0]).feId;
	
		/* Add the Correlator in the fpTransList of the request CB */
		npf_list_pushBack(&(request->fpTransList),(void *)corr);

		/* Increment the Count of num FP Transactions pending */
		request->numTransWait++;
		request->numWaitFEs++;
		((request->feInfo[0]).numWait)++;

		/* Call the FP Plugin API */
		fpRetVal = FPPAPI_ATM_ConnectionXcDeleteFunc((FPPI_FEID)(request->feInfo[0]).feId,
							(FPPI_CORRELATOR)corr,
							(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_CONNECTION_XC_DEL_CB],
							(NPF_ATM_ConnectionXc_t *)&ptr[i],
							(NPF_uint32_t)1);

		/* Check if the FP-Plug in API succedded */
		if(fpRetVal != FPPI_SUCCESS)
		{

			switch(fpRetVal)
			{
				case FPPI_OUT_OF_MEMORY:
					tempResp.status =  NPF_OUT_OF_MEMORY;
					tempResp.reason =  NPF_OUT_OF_MEMORY;
					break;
				case FPPI_INVALID_PARAMETERS:
				case FPPI_FAIL:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
				case FPPI_NOT_INITIALIZED:
					tempResp.status = NPF_COMPONENT_UNINITIALIZED;
					tempResp.reason = NPF_COMPONENT_UNINITIALIZED;
					break;
				default:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
			}/* end of Switch(fpRetVal) */

			/* Call the ATM callback handler function to handle the failure 
				of the FP-Plugin Call */
			ifAtmOnCB_ConnXcDelFunc((FPPI_FEID)(request->feInfo[0]).feId,
									(FPPI_CORRELATOR)corr,
									(FPPI_CONTEXT)NPF_IF_ATM_CONNECTION_XC_DEL,
									(void *)&tempResp);

		}/* end of if condition */
	}/* end of the for loop */


	return ret;
}
	


/*****************************************************************************
*    Name: ifATM_ProcessOamCpSetReq 
*
*    Description:
*			This function does the necessary processing required for the received 
* 			user request for setting an ATM Connection Point used for OAM services.
* 
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM		 - invalid parameters
*
********************************************************************************/
NPF_error_t ifATM_ProcessOamCpSetReq(IfATM_RequestCB_t	*request)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_FeTransInfo_t *corr = NULL;
	NPF_uint32_t i = 0;
	IfATM_OAM_CP_t *ptr = NULL;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	IfATM_FpResponse_t tempResp;


	/* Make ptr to point to the entry member of apiData */
	ptr = (IfATM_OAM_CP_t *)(request->apiData).entry;

	for(i = 0; i < (request->apiData).numEntries; i++)
	{
		/* Allocate Memory for the correlator */
		corr = (IfATM_FeTransInfo_t *)PIL_AllocateMemory
						(sizeof(IfATM_FeTransInfo_t) , PIL_MEM_ZERO_INIT);	

		if(corr == NULL)
			return NPF_OUT_OF_MEMORY;

		/* Create the Correlator to associate with this request */
		corr->reqCbPtr = request;
		corr->entryPtr = &ptr[i];
		corr->feId = (request->feInfo[0]).feId;
	
		/* Add the Correlator in the fpTransList of the request CB */
		npf_list_pushBack(&(request->fpTransList),(void *)corr);


		/* Increment the Count of num FP Transactions pending */
		request->numTransWait++;
		request->numWaitFEs++;
		((request->feInfo[0]).numWait)++;

		/* Call the FP Plugin API */
		fpRetVal = FPPAPI_ATM_OAM_CP_SetFunc((FPPI_FEID)(request->feInfo[0]).feId,
							(FPPI_CORRELATOR)corr,
							(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_OAM_CP_SET_CB],
							(IfATM_OAM_CP_t *)&ptr[i],
							(NPF_uint32_t)1);

		/* Check if the FP-Plug in API succedded */
		if(fpRetVal != FPPI_SUCCESS)
		{

			switch(fpRetVal)
			{
				case FPPI_OUT_OF_MEMORY:
					tempResp.status =  NPF_OUT_OF_MEMORY;
					tempResp.reason =  NPF_OUT_OF_MEMORY;
					break;
				case FPPI_INVALID_PARAMETERS:
				case FPPI_FAIL:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
				case FPPI_NOT_INITIALIZED:
					tempResp.status = NPF_COMPONENT_UNINITIALIZED;
					tempResp.reason = NPF_COMPONENT_UNINITIALIZED;
					break;
				default:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
			}/* end of Switch(fpRetVal) */

			/* Call the ATM callback handler function to handle the failure 
				of the FP-Plugin Call */
			ifAtmOnCB_OamCpSetFunc((FPPI_FEID)(request->feInfo[0]).feId,
									(FPPI_CORRELATOR)corr,
									(FPPI_CONTEXT)NPF_IF_ATM_OAM_CP_SET,
									(void *)&tempResp);

		}/* end of if condition */
	}/* end of the for loop */

	return ret;
}
	


/*****************************************************************************
*    Name: ifATM_ProcessOamCcSetReq 
*
*    Description:
*			This function does the necessary processing required for the received 
* 			user request for setting continutity check mechanism used for OAM services.
* 
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM		 - invalid parameters
*
********************************************************************************/
NPF_error_t ifATM_ProcessOamCcSetReq(IfATM_RequestCB_t	*request)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_FeTransInfo_t *corr = NULL;
	NPF_uint32_t i = 0;
	IfATM_OAM_CC_t *ptr = NULL;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	IfATM_FpResponse_t tempResp;


	/* Make ptr to point to the entry member of apiData */
	ptr = (IfATM_OAM_CC_t *)(request->apiData).entry;

	for(i = 0; i < (request->apiData).numEntries; i++)
	{
		/* Allocate Memory for the correlator */
		corr = (IfATM_FeTransInfo_t *)PIL_AllocateMemory
						(sizeof(IfATM_FeTransInfo_t) , PIL_MEM_ZERO_INIT);	

		if(corr == NULL)
			return NPF_OUT_OF_MEMORY;

		/* Create the Correlator to associate with this request */
		corr->reqCbPtr = request;
		corr->entryPtr = &ptr[i];
		corr->feId = (request->feInfo[0]).feId;
	
		/* Add the Correlator in the fpTransList of the request CB */
		npf_list_pushBack(&(request->fpTransList),(void *)corr);

		/* Increment the Count of num FP Transactions pending */
		request->numTransWait++;
		request->numWaitFEs++;
		((request->feInfo[0]).numWait)++;

		/* Call the FP Plugin API */
		fpRetVal = FPPAPI_ATM_OAM_CC_SetFunc((FPPI_FEID)(request->feInfo[0]).feId,
							(FPPI_CORRELATOR)corr,
							(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_OAM_CC_SET_CB],
							(IfATM_OAM_CC_t *)&ptr[i],
							(NPF_uint32_t)1);

		/* Check if the FP-Plug in API succedded */
		if(fpRetVal != FPPI_SUCCESS)
		{

			switch(fpRetVal)
			{
				case FPPI_OUT_OF_MEMORY:
					tempResp.status =  NPF_OUT_OF_MEMORY;
					tempResp.reason =  NPF_OUT_OF_MEMORY;
					break;
				case FPPI_INVALID_PARAMETERS:
				case FPPI_FAIL:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
				case FPPI_NOT_INITIALIZED:
					tempResp.status = NPF_COMPONENT_UNINITIALIZED;
					tempResp.reason = NPF_COMPONENT_UNINITIALIZED;
					break;
				default:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
			}/* end of Switch(fpRetVal) */

			/* Call the ATM callback handler function to handle the failure 
				of the FP-Plugin Call */
			ifAtmOnCB_OamCcSetFunc((FPPI_FEID)(request->feInfo[0]).feId,
									(FPPI_CORRELATOR)corr,
									(FPPI_CONTEXT)NPF_IF_ATM_OAM_CC_SET,
									(void *)&tempResp);

		}/* end of if condition */
	}/* end of the for loop */

	return ret;
}
	



/*****************************************************************************
*    Name: ifATM_ProcessOamLbSetReq 
*
*    Description:
*			This function does the necessary processing required for the received 
* 			user request for setting LoopBack mechanism used for OAM services.
* 
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM		 - invalid parameters
*
********************************************************************************/
NPF_error_t ifATM_ProcessOamLbSetReq(IfATM_RequestCB_t	*request)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_FeTransInfo_t *corr = NULL;
	NPF_uint32_t i = 0;
	IfATM_OAM_LB_t *ptr = NULL;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	IfATM_FpResponse_t tempResp;


	/* Make ptr to point to the entry member of apiData */
	ptr = (IfATM_OAM_LB_t *)(request->apiData).entry;

	for(i = 0; i < (request->apiData).numEntries; i++)
	{
		/* Allocate Memory for the correlator */
		corr = (IfATM_FeTransInfo_t *)PIL_AllocateMemory
						(sizeof(IfATM_FeTransInfo_t) , PIL_MEM_ZERO_INIT);	

		if(corr == NULL)
			return NPF_OUT_OF_MEMORY;

		/* Create the Correlator to associate with this request */
		corr->reqCbPtr = request;
		corr->entryPtr = &ptr[i];
		corr->feId = (request->feInfo[0]).feId;
	
		/* Add the Correlator in the fpTransList of the request CB */
		npf_list_pushBack(&(request->fpTransList),(void *)corr);

		/* Increment the Count of num FP Transactions pending */
		request->numTransWait++;
		request->numWaitFEs++;
		((request->feInfo[0]).numWait)++;

		/* Call the FP Plugin API */
		fpRetVal = FPPAPI_ATM_OAM_LB_SetFunc((FPPI_FEID)(request->feInfo[0]).feId,
							(FPPI_CORRELATOR)corr,
							(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_OAM_LB_SET_CB],
							(IfATM_OAM_LB_t *)&ptr[i],
							(NPF_uint32_t)1);

		/* Check if the FP-Plug in API succedded */
		if(fpRetVal != FPPI_SUCCESS)
		{

			switch(fpRetVal)
			{
				case FPPI_OUT_OF_MEMORY:
					tempResp.status =  NPF_OUT_OF_MEMORY;
					tempResp.reason =  NPF_OUT_OF_MEMORY;
					break;
				case FPPI_INVALID_PARAMETERS:
				case FPPI_FAIL:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
				case FPPI_NOT_INITIALIZED:
					tempResp.status = NPF_COMPONENT_UNINITIALIZED;
					tempResp.reason = NPF_COMPONENT_UNINITIALIZED;
					break;
				default:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
			}/* end of Switch(fpRetVal) */

			/* Call the ATM callback handler function to handle the failure 
				of the FP-Plugin Call */
			ifAtmOnCB_OamLbSetFunc((FPPI_FEID)(request->feInfo[0]).feId,
									(FPPI_CORRELATOR)corr,
									(FPPI_CONTEXT)NPF_IF_ATM_OAM_LB_SET,
									(void *)&tempResp);

		}/* end of if condition */
	}/* end of the for loop */

	return ret;
}
	



/*****************************************************************************
*    Name: ifATM_ProcessOamMonitorReq 
*
*    Description:
*			This function does the necessary processing required for the received 
* 			user request for setting OAM cell monitorin mechanism used for OAM services.
* 
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM		 - invalid parameters
*
********************************************************************************/
NPF_error_t ifATM_ProcessOamMonitorReq(IfATM_RequestCB_t	*request)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_FeTransInfo_t *corr = NULL;
	NPF_uint32_t i = 0;
	IfATM_OAM_Mon_t *ptr =NULL;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	IfATM_FpResponse_t tempResp;


	/* Make ptr to point to the entry member of apiData */
	ptr = (IfATM_OAM_Mon_t *)(request->apiData).entry;

	for(i = 0; i < (request->apiData).numEntries; i++)
	{
		/* Allocate Memory for the correlator */
		corr = (IfATM_FeTransInfo_t *)PIL_AllocateMemory
						(sizeof(IfATM_FeTransInfo_t) , PIL_MEM_ZERO_INIT);	

		if(corr == NULL)
			return NPF_OUT_OF_MEMORY;

		/* Create the Correlator to associate with this request */
		corr->reqCbPtr = request;
		corr->entryPtr = &ptr[i];
		corr->feId = (request->feInfo[0]).feId;
	
		/* Add the Correlator in the fpTransList of the request CB */
		npf_list_pushBack(&(request->fpTransList),(void *)corr);

		/* Increment the Count of num FP Transactions pending */
		request->numTransWait++;
		request->numWaitFEs++;
		((request->feInfo[0]).numWait)++;

		/* Call the FP Plugin API */
		fpRetVal = FPPAPI_ATM_OAM_MonitorFunc((FPPI_FEID)(request->feInfo[0]).feId,
							(FPPI_CORRELATOR)corr,
							(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_OAM_MONITOR_CB],
							(IfATM_OAM_Mon_t *)&ptr[i],
							(NPF_uint32_t)1);

		/* Check if the FP-Plug in API succedded */
		if(fpRetVal != FPPI_SUCCESS)
		{

			switch(fpRetVal)
			{
				case FPPI_OUT_OF_MEMORY:
					tempResp.status =  NPF_OUT_OF_MEMORY;
					tempResp.reason =  NPF_OUT_OF_MEMORY;
					break;
				case FPPI_INVALID_PARAMETERS:
				case FPPI_FAIL:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
				case FPPI_NOT_INITIALIZED:
					tempResp.status = NPF_COMPONENT_UNINITIALIZED;
					tempResp.reason = NPF_COMPONENT_UNINITIALIZED;
					break;
				default:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
			}/* end of Switch(fpRetVal) */

			/* Call the ATM callback handler function to handle the failure 
				of the FP-Plugin Call */
			ifAtmOnCB_OamMonitorFunc((FPPI_FEID)(request->feInfo[0]).feId,
									(FPPI_CORRELATOR)corr,
									(FPPI_CONTEXT)NPF_IF_ATM_OAM_MONITOR,
									(void *)&tempResp);

		}/* end of if condition */
	}/* end of the for loop */

	return ret;
}
	


/*****************************************************************************
*    Name: ifATM_ProcessOamAlrmSetReq 
*
*    Description:
*			This function does the necessary processing required for the received 
* 			user request for setting OAM Alarms used for OAM services.
* 
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM		 - invalid parameters
*
********************************************************************************/
NPF_error_t ifATM_ProcessOamAlrmSetReq(IfATM_RequestCB_t	*request)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_FeTransInfo_t *corr = NULL;
	NPF_uint32_t i = 0;
	IfATM_OAM_Alarm_t *ptr = NULL;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	IfATM_FpResponse_t tempResp;


	/* Make ptr to point to the entry member of apiData */
	ptr = (IfATM_OAM_Alarm_t *)(request->apiData).entry;

	for(i = 0; i < (request->apiData).numEntries; i++)
	{
		/* Allocate Memory for the correlator */
		corr = (IfATM_FeTransInfo_t *)PIL_AllocateMemory
						(sizeof(IfATM_FeTransInfo_t) , PIL_MEM_ZERO_INIT);	

		if(corr == NULL)
			return NPF_OUT_OF_MEMORY;

		/* Create the Correlator to associate with this request */
		corr->reqCbPtr = request;
		corr->entryPtr = &ptr[i];
		corr->feId = (request->feInfo[0]).feId;
	
		/* Add the Correlator in the fpTransList of the request CB */
		npf_list_pushBack(&(request->fpTransList),(void *)corr);

		/* Increment the Count of num FP Transactions pending */
		request->numTransWait++;
		request->numWaitFEs++;
		((request->feInfo[0]).numWait)++;

		/* Call the FP Plugin API */
		fpRetVal = FPPAPI_ATM_OAM_AlarmSetFunc((FPPI_FEID)(request->feInfo[0]).feId,
							(FPPI_CORRELATOR)corr,
							(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_OAM_ALARM_SET_CB],
							(IfATM_OAM_Alarm_t *)&ptr[i],
							(NPF_uint32_t)1);

		/* Check if the FP-Plug in API succedded */
		if(fpRetVal != FPPI_SUCCESS)
		{

			switch(fpRetVal)
			{
				case FPPI_OUT_OF_MEMORY:
					tempResp.status =  NPF_OUT_OF_MEMORY;
					tempResp.reason =  NPF_OUT_OF_MEMORY;
					break;
				case FPPI_INVALID_PARAMETERS:
				case FPPI_FAIL:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
				case FPPI_NOT_INITIALIZED:
					tempResp.status = NPF_COMPONENT_UNINITIALIZED;
					tempResp.reason = NPF_COMPONENT_UNINITIALIZED;
					break;
				default:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
			}/* end of Switch(fpRetVal) */

			/* Call the ATM callback handler function to handle the failure 
				of the FP-Plugin Call */
			ifAtmOnCB_OamAlarmSetFunc((FPPI_FEID)(request->feInfo[0]).feId,
									(FPPI_CORRELATOR)corr,
									(FPPI_CONTEXT)NPF_IF_ATM_OAM_ALARM_SET,
									(void *)&tempResp);

		}/* end of if condition */
	}/* end of the for loop */


	return ret;
}
	


/*****************************************************************************
*    Name: ifATM_ProcessOamAlrmClearReq 
*
*    Description:
*			This function does the necessary processing required for the received 
* 			user request for disabling OAM Alarms used for OAM services.
* 
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM		 - invalid parameters
*
********************************************************************************/
NPF_error_t ifATM_ProcessOamAlrmClearReq(IfATM_RequestCB_t	*request)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_FeTransInfo_t *corr = NULL;
	NPF_uint32_t i = 0;
	IfATM_OAM_Alarm_t *ptr = NULL;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	IfATM_FpResponse_t tempResp;


	/* Make ptr to point to the entry member of apiData */
	ptr = (IfATM_OAM_Alarm_t *)(request->apiData).entry;

	for(i = 0; i < (request->apiData).numEntries; i++)
	{
		/* Allocate Memory for the correlator */
		corr = (IfATM_FeTransInfo_t *)PIL_AllocateMemory
						(sizeof(IfATM_FeTransInfo_t) , PIL_MEM_ZERO_INIT);	

		if(corr == NULL)
			return NPF_OUT_OF_MEMORY;

		/* Create the Correlator to associate with this request */
		corr->reqCbPtr = request;
		corr->entryPtr = &ptr[i];
		corr->feId = (request->feInfo[0]).feId;
	
		/* Add the Correlator in the fpTransList of the request CB */
		npf_list_pushBack(&(request->fpTransList),(void *)corr);

		/* Increment the Count of num FP Transactions pending */
		request->numTransWait++;
		request->numWaitFEs++;
		((request->feInfo[0]).numWait)++;

		/* Call the FP Plugin API */
		fpRetVal = FPPAPI_ATM_OAM_AlarmClearFunc((FPPI_FEID)(request->feInfo[0]).feId,
							(FPPI_CORRELATOR)corr,
							(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_OAM_ALARM_CLEAR_CB],
							(IfATM_OAM_Alarm_t *)&ptr[i],
							(NPF_uint32_t)1);

		/* Check if the FP-Plug in API succedded */
		if(fpRetVal != FPPI_SUCCESS)
		{

			switch(fpRetVal)
			{
				case FPPI_OUT_OF_MEMORY:
					tempResp.status =  NPF_OUT_OF_MEMORY;
					tempResp.reason =  NPF_OUT_OF_MEMORY;
					break;
				case FPPI_INVALID_PARAMETERS:
				case FPPI_FAIL:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
				case FPPI_NOT_INITIALIZED:
					tempResp.status = NPF_COMPONENT_UNINITIALIZED;
					tempResp.reason = NPF_COMPONENT_UNINITIALIZED;
					break;
				default:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
			}/* end of Switch(fpRetVal) */

			/* Call the ATM callback handler function to handle the failure 
				of the FP-Plugin Call */
			ifAtmOnCB_OamAlarmClearFunc((FPPI_FEID)(request->feInfo[0]).feId,
									(FPPI_CORRELATOR)corr,
									(FPPI_CONTEXT)NPF_IF_ATM_OAM_ALARM_CLEAR,
									(void *)&tempResp);

		}/* end of if condition */
	}/* end of the for loop */

	return ret;
}
	



/*****************************************************************************
*    Name: ifATM_ProcessOamPmSetReq 
*
*    Description:
*			This function does the necessary processing required for the received 
* 			user request for enabling/disabling Performance monitoring mechanism
*			used for OAM services.
* 
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_UNINITIALIZED - FP uninitialized
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM		 - invalid parameters
*
********************************************************************************/
NPF_error_t ifATM_ProcessOamPmSetReq(IfATM_RequestCB_t	*request)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_FeTransInfo_t *corr = NULL;
	NPF_uint32_t i = 0;
	IfATM_OAM_PM_t *ptr = NULL;
	FPPI_RET fpRetVal = FPPI_SUCCESS;
	IfATM_FpResponse_t tempResp;


	/* Make ptr to point to the entry member of apiData */
	ptr = (IfATM_OAM_PM_t *)(request->apiData).entry;

	for(i = 0; i < (request->apiData).numEntries; i++)
	{
		/* Allocate Memory for the correlator */
		corr = (IfATM_FeTransInfo_t *)PIL_AllocateMemory
						(sizeof(IfATM_FeTransInfo_t) , PIL_MEM_ZERO_INIT);	

		if(corr == NULL)
			return NPF_OUT_OF_MEMORY;

		/* Create the Correlator to associate with this request */
		corr->reqCbPtr = request;
		corr->entryPtr = &ptr[i];
		corr->feId = (request->feInfo[0]).feId;
	
		/* Add the Correlator in the fpTransList of the request CB */
		npf_list_pushBack(&(request->fpTransList),(void *)corr);

		/* Increment the Count of num FP Transactions pending */
		request->numTransWait++;
		request->numWaitFEs++;
		((request->feInfo[0]).numWait)++;

		/* Call the FP Plugin API */
		fpRetVal = FPPAPI_ATM_OAM_PM_SetFunc((FPPI_FEID)(request->feInfo[0]).feId,
							(FPPI_CORRELATOR)corr,
							(FPPI_CBHANDLE)ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_OAM_PM_SET_CB],
							(IfATM_OAM_PM_t *)&ptr[i],
							(NPF_uint32_t)1);

		/* Check if the FP-Plug in API succedded */
		if(fpRetVal != FPPI_SUCCESS)
		{

			switch(fpRetVal)
			{
				case FPPI_OUT_OF_MEMORY:
					tempResp.status =  NPF_OUT_OF_MEMORY;
					tempResp.reason =  NPF_OUT_OF_MEMORY;
					break;
				case FPPI_INVALID_PARAMETERS:
				case FPPI_FAIL:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
				case FPPI_NOT_INITIALIZED:
					tempResp.status = NPF_COMPONENT_UNINITIALIZED;
					tempResp.reason = NPF_COMPONENT_UNINITIALIZED;
					break;
				default:
					tempResp.status = NPF_IF_E_INVALID_PARAM;
					tempResp.reason = NPF_IF_E_INVALID_PARAM;
					break;
			}/* end of Switch(fpRetVal) */

			/* Call the ATM callback handler function to handle the failure 
				of the FP-Plugin Call */
			ifAtmOnCB_OamPmSetFunc((FPPI_FEID)(request->feInfo[0]).feId,
									(FPPI_CORRELATOR)corr,
									(FPPI_CONTEXT)NPF_IF_ATM_OAM_PM_SET,
									(void *)&tempResp);

		}/* end of if condition */
	}/* end of the for loop */

	return ret;
}
	




/*****************************************************************************
*                                                                            *
*  -------------------------- External Interface --------------------------  *
*                                                                            *
*****************************************************************************/


/*****************************************************************************
*    Name: NPF_IfATM_VccSet 
*
*    Description:
*			This function adds/creates one or more VCCs on a Single ATM interface.
*    Parameters:
*        None
*
*    Preconditions:
*        called after bd_init, cbthread_init and fppi_init.
*        called before ns_init, ipv4_init, mpls_init
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM      - Invalid Parmaters
*
********************************************************************************/
NPF_error_t NPF_IfATM_VccSet(
			NPF_IN NPF_callbackHandle_t	if_cbHandle,
			NPF_IN NPF_correlator_t if_cbCorrelator,
			NPF_IN NPF_errorReporting_t if_errorReporting,
			NPF_IN NPF_IfHandle_t ifHandle,
			NPF_IN NPF_uint32_t n_Vccs,
			NPF_IN NPF_IfVcc_t *if_VccStructArray)
{
	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_RequestCB_t	*request;

	/* Trace the function */
	assert(NPF_IfATM_VccSet);

	/* Check if the Parameters passed are O.K */
	if((ret = ifATM_CheckUserParams(if_cbHandle,if_errorReporting,ifHandle,
				n_Vccs,(void *)if_VccStructArray)) != NPF_SUCCESS)
		return(ret);


	/* Copy the received the list of Vccs in a request CB structure */
	if((ret = ifATM_CreateRequestCB(NPF_IF_ATM_VCC_SET,if_cbHandle,if_cbCorrelator,
			  if_errorReporting,ifHandle, n_Vccs,(void *)if_VccStructArray,&request))
				 != NPF_SUCCESS)
		return(ret);


	/* Process the user request */
	if((ret = ifATM_ProcessUserRequest(request)) != NPF_SUCCESS)
		return(ret);

	return ret;

}


/*****************************************************************************
*    Name: NPF_IfATM_VccBind 
*
*    Description:
*			This function binds a higher layer interface with 
*			one or more VCCs on a Single ATM interface.
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM      - Invalid Parmaters
*  shabbir this function needs to be redone *
********************************************************************************/
NPF_error_t NPF_IfATM_VccBind(
			NPF_IN NPF_callbackHandle_t	if_cbHandle,
			NPF_IN NPF_correlator_t if_cbCorrelator,
			NPF_IN NPF_errorReporting_t if_errorReporting,
			NPF_IN NPF_IfHandle_t	if_ATM_Handle,
			NPF_IN NPF_IfHandle_t	if_ParentHandle,
			NPF_IN NPF_uint32_t n_Vccs,
			NPF_IN NPF_VccAddr_t *if_VccAddrArray)
{
	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_RequestCB_t	*request;
	NPF_DATA data = NULL;
	NPF_uint32_t i =0;
	IfATM_VccBind_t *vccBind = NULL;

	/* Trace the function */
	assert(NPF_IfATM_VccBind);

	/* Check if the Parameters passed are O.K */
	if((ret = ifATM_CheckUserParams(if_cbHandle,if_errorReporting,if_ATM_Handle,
						n_Vccs,if_VccAddrArray)) != NPF_SUCCESS)
		return(ret);

	/* Check if the Parent Interface Handle is valid */
	if (cm_getDataByHandle(if_ParentHandle, &data) == -1)
        return NPF_IF_E_INVALID_PARAM;


	/* Copy the received the list of Vccs in a request CB structure */
	if((ret = ifATM_CreateRequestCB(NPF_IF_ATM_VCC_BIND,if_cbHandle,if_cbCorrelator,
			  if_errorReporting,if_ATM_Handle, n_Vccs,(void *)if_VccAddrArray,&request))
				 != NPF_SUCCESS)
		return(ret);

	/* Copy the parent interface handle in the request CB entry member structure */
	vccBind = (IfATM_VccBind_t *)(request->apiData).entry;
	for(i=0; i< n_Vccs; i++)
	{
		vccBind[i].parentIfHandle = if_ParentHandle;
	}	


	/* Process the user request */
	if((ret = ifATM_ProcessUserRequest(request)) != NPF_SUCCESS)
		return(ret);

	return ret;

}/* end of NPF_IfATM_VccBind */



/*****************************************************************************
*    Name: NPF_IfATM_VccDelete 
*
*    Description:
*			This function deletes one or more VCCs on a Single ATM interface.
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM      - Invalid Parmaters
*
********************************************************************************/
NPF_error_t NPF_IfATM_VccDelete(
			NPF_IN NPF_callbackHandle_t	if_cbHandle,
			NPF_IN NPF_correlator_t if_cbCorrelator,
			NPF_IN NPF_errorReporting_t if_errorReporting,
			NPF_IN NPF_IfHandle_t ifHandle,
			NPF_IN NPF_uint32_t n_Vccs,
			NPF_IN NPF_VccAddr_t *if_VccAddrArray)
{
	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_RequestCB_t	*request;

	/* Trace the function */
	assert(NPF_IfATM_VccDelete);

	/* Check if the Parameters passed are O.K */
	if((ret = ifATM_CheckUserParams(if_cbHandle,if_errorReporting,ifHandle,
						n_Vccs,if_VccAddrArray)) != NPF_SUCCESS)
		return(ret);


	/* Copy the received the list of Vccs in a request CB structure */
	if((ret = ifATM_CreateRequestCB(NPF_IF_ATM_VCC_DELETE,if_cbHandle,if_cbCorrelator,
			  if_errorReporting,ifHandle, n_Vccs,(void *)if_VccAddrArray,&request))
				 != NPF_SUCCESS)
		return(ret);


	/* Process the user request */
	if((ret = ifATM_ProcessUserRequest(request)) != NPF_SUCCESS)
		return(ret);

	return ret;

}


/*****************************************************************************
*    Name: NPF_IfATM_VccEnable 
*
*    Description:
*			This function Enables one or more VCCs on a Single ATM interface.
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM      - Invalid Parmaters
*
********************************************************************************/
NPF_error_t NPF_IfATM_VccEnable(
			NPF_IN NPF_callbackHandle_t	if_cbHandle,
			NPF_IN NPF_correlator_t if_cbCorrelator,
			NPF_IN NPF_errorReporting_t if_errorReporting,
			NPF_IN NPF_IfHandle_t ifHandle,
			NPF_IN NPF_uint32_t n_Vccs,
			NPF_IN NPF_VccAddr_t *if_VccAddrArray)
{
	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_RequestCB_t	*request;

	/* Trace the function */
	assert(NPF_IfATM_VccEnable);

	/* Check if the Parameters passed are O.K */
	if((ret = ifATM_CheckUserParams(if_cbHandle,if_errorReporting,ifHandle,
						n_Vccs,if_VccAddrArray)) != NPF_SUCCESS)
		return(ret);


	/* Copy the received the list of Vccs in a request CB structure */
	if((ret = ifATM_CreateRequestCB(NPF_IF_ATM_VCC_ENABLE,if_cbHandle,if_cbCorrelator,
			  if_errorReporting,ifHandle, n_Vccs,(void *)if_VccAddrArray,&request))
				 != NPF_SUCCESS)
		return(ret);


	/* Process the user request */
	if((ret = ifATM_ProcessUserRequest(request)) != NPF_SUCCESS)
		return(ret);

	return ret;

}



/*****************************************************************************
*    Name: NPF_IfATM_VccDisable 
*
*    Description:
*			This function Disables one or more VCCs on a Single ATM interface.
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM      - Invalid Parmaters
*
********************************************************************************/
NPF_error_t NPF_IfATM_VccDisable(
			NPF_IN NPF_callbackHandle_t	if_cbHandle,
			NPF_IN NPF_correlator_t if_cbCorrelator,
			NPF_IN NPF_errorReporting_t if_errorReporting,
			NPF_IN NPF_IfHandle_t ifHandle,
			NPF_IN NPF_uint32_t n_Vccs,
			NPF_IN NPF_VccAddr_t *if_VccAddrArray)
{
	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_RequestCB_t	*request;

	/* Trace the function */
	assert(NPF_IfATM_VccDisable);

	/* Check if the Parameters passed are O.K */
	if((ret = ifATM_CheckUserParams(if_cbHandle,if_errorReporting,ifHandle,
						n_Vccs,if_VccAddrArray)) != NPF_SUCCESS)
		return(ret);


	/* Copy the received the list of Vccs in a request CB structure */
	if((ret = ifATM_CreateRequestCB(NPF_IF_ATM_VCC_DISABLE,if_cbHandle,if_cbCorrelator,
			  if_errorReporting,ifHandle, n_Vccs,(void *)if_VccAddrArray,&request))
				 != NPF_SUCCESS)
		return(ret);


	/* Process the user request */
	if((ret = ifATM_ProcessUserRequest(request)) != NPF_SUCCESS)
		return(ret);

	return ret;

}


/*****************************************************************************
*    Name: NPF_IfATM_VccOperStatusGet 
*
*    Description:
*			This function Gets the Operational Status of  one or more VCCs on 
*			a Single ATM interface.
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM      - Invalid Parmaters
*
********************************************************************************/
NPF_error_t NPF_IfATM_VccOperStatusGet(
			NPF_IN NPF_callbackHandle_t	if_cbHandle,
			NPF_IN NPF_correlator_t if_cbCorrelator,
			NPF_IN NPF_errorReporting_t if_errorReporting,
			NPF_IN NPF_IfHandle_t ifHandle,
			NPF_IN NPF_uint32_t n_Vccs,
			NPF_IN NPF_VccAddr_t *if_VccAddrArray)
{
	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_RequestCB_t	*request;

	/* Trace the function */
	assert(NPF_IfATM_VccOperStatusGet);

	/* Check if the Parameters passed are O.K */
	if((ret = ifATM_CheckUserParams(if_cbHandle,if_errorReporting,ifHandle,
						n_Vccs,if_VccAddrArray)) != NPF_SUCCESS)
		return(ret);


	/* Copy the received the list of Vccs in a request CB structure */
	if((ret = ifATM_CreateRequestCB(NPF_IF_ATM_VCC_OPER_STATUS_GET,if_cbHandle,if_cbCorrelator,
			  if_errorReporting,ifHandle, n_Vccs,(void *)if_VccAddrArray,&request))
				 != NPF_SUCCESS)
		return(ret);


	/* Process the user request */
	if((ret = ifATM_ProcessUserRequest(request)) != NPF_SUCCESS)
		return(ret);

	return ret;

}



/*****************************************************************************
*    Name: NPF_IfATM_VccStatsGet 
*
*    Description:
*			This function Gets the statistics of  one or more VCCs on 
*			a Single ATM interface.
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM      - Invalid Parmaters
*
********************************************************************************/
NPF_error_t NPF_IfATM_VccStatsGet(
			NPF_IN NPF_callbackHandle_t	if_cbHandle,
			NPF_IN NPF_correlator_t if_cbCorrelator,
			NPF_IN NPF_errorReporting_t if_errorReporting,
			NPF_IN NPF_IfHandle_t ifHandle,
			NPF_IN NPF_uint32_t n_Vccs,
			NPF_IN NPF_VccAddr_t *if_VccAddrArray)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_RequestCB_t	*request;

	/* Trace the function */
	assert(NPF_IfATM_VccStatsGet);

	/* Check if the Parameters passed are O.K */
	if((ret = ifATM_CheckUserParams(if_cbHandle,if_errorReporting,ifHandle,
						n_Vccs,if_VccAddrArray)) != NPF_SUCCESS)
		return(ret);


	/* Copy the received the list of Vccs in a request CB structure */
	if((ret = ifATM_CreateRequestCB(NPF_IF_ATM_VCC_STATS_GET,if_cbHandle,if_cbCorrelator,
			  if_errorReporting,ifHandle, n_Vccs,(void *)if_VccAddrArray,&request))
				 != NPF_SUCCESS)
		return(ret);


	/* Process the user request */
	if((ret = ifATM_ProcessUserRequest(request)) != NPF_SUCCESS)
		return(ret);

	return ret;

}



/*****************************************************************************
*    Name: NPF_IfATM_ConnectionXcSet 
*
*    Description:
*			This function Adds one or more ATM cross connections on   
*			a Single ATM interface.
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM      - Invalid Parmaters
*
********************************************************************************/
NPF_error_t NPF_IfATM_ConnectionXcSet(
			NPF_IN NPF_callbackHandle_t	if_cbHandle,
			NPF_IN NPF_correlator_t if_cbCorrelator,
			NPF_IN NPF_errorReporting_t if_errorReporting,
			NPF_IN NPF_IfHandle_t ifHandle,
			NPF_IN NPF_uint32_t n_Vccs,
			NPF_IN NPF_ATM_ConnectionXc_t	*if_connXc)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_RequestCB_t	*request;

	/* Trace the function */
	assert(NPF_IfATM_ConnectionXcSet);

	/* Check if the Parameters passed are O.K */
	if((ret = ifATM_CheckUserParams(if_cbHandle,if_errorReporting,ifHandle,
						n_Vccs,if_connXc)) != NPF_SUCCESS)
		return(ret);


	/* Copy the received the list of Vccs in a request CB structure */
	if((ret = ifATM_CreateRequestCB(NPF_IF_ATM_CONNECTION_XC_SET,if_cbHandle,if_cbCorrelator,
			  if_errorReporting,ifHandle, n_Vccs,(void *)if_connXc,&request))
				 != NPF_SUCCESS)
		return(ret);


	/* Process the user request */
	if((ret = ifATM_ProcessUserRequest(request)) != NPF_SUCCESS)
		return(ret);

	return ret;

}


/*****************************************************************************
*    Name: NPF_IfATM_ConnectionXcDelete 
*
*    Description:
*			This function Deletes one or more ATM cross connections on   
*			a Single ATM interface.
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM      - Invalid Parmaters
*
********************************************************************************/
NPF_error_t NPF_IfATM_ConnectionXcDelete(
			NPF_IN NPF_callbackHandle_t	if_cbHandle,
			NPF_IN NPF_correlator_t if_cbCorrelator,
			NPF_IN NPF_errorReporting_t if_errorReporting,
			NPF_IN NPF_IfHandle_t ifHandle,
			NPF_IN NPF_uint32_t n_Vccs,
			NPF_IN NPF_ATM_ConnectionXc_t	*if_connXc)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_RequestCB_t	*request;

	/* Trace the function */
	assert(NPF_IfATM_ConnectionXcDelete);

	/* Check if the Parameters passed are O.K */
	if((ret = ifATM_CheckUserParams(if_cbHandle,if_errorReporting,ifHandle,
						n_Vccs,if_connXc)) != NPF_SUCCESS)
		return(ret);


	/* Copy the received the list of Vccs in a request CB structure */
	if((ret = ifATM_CreateRequestCB(NPF_IF_ATM_CONNECTION_XC_DEL,if_cbHandle,if_cbCorrelator,
			  if_errorReporting,ifHandle, n_Vccs,(void *)if_connXc,&request))
				 != NPF_SUCCESS)
		return(ret);


	/* Process the user request */
	if((ret = ifATM_ProcessUserRequest(request)) != NPF_SUCCESS)
		return(ret);

	return ret;

}



/*****************************************************************************
*    Name: NPF_IfATM_OAM_CP_Set 
*
*    Description:
*			This function Configures one or more ATM Connection Points for OAM services    
*			on a Single ATM interface.
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM      - Invalid Parmaters
*
********************************************************************************/
NPF_error_t NPF_IfATM_OAM_CP_Set(
			NPF_IN NPF_callbackHandle_t	if_cbHandle,
			NPF_IN NPF_correlator_t if_cbCorrelator,
			NPF_IN NPF_errorReporting_t if_errorReporting,
			NPF_IN NPF_IfHandle_t ifHandle,
			NPF_IN NPF_uint32_t n_CPs,
			NPF_IN NPF_ATM_OAM_CP_t *if_oamCP_Array)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_RequestCB_t	*request;

	/* Trace the function */
	assert(NPF_IfATM_OAM_CP_Set);

	/* Check if the Parameters passed are O.K */
	if((ret = ifATM_CheckUserParams(if_cbHandle,if_errorReporting,ifHandle,
						n_CPs,if_oamCP_Array)) != NPF_SUCCESS)
		return(ret);


	/* Copy the received the list of Vccs in a request CB structure */
	if((ret = ifATM_CreateRequestCB(NPF_IF_ATM_OAM_CP_SET,if_cbHandle,if_cbCorrelator,
			  if_errorReporting,ifHandle, n_CPs,(void *)if_oamCP_Array,&request))
				 != NPF_SUCCESS)
		return(ret);


	/* Process the user request */
	if((ret = ifATM_ProcessUserRequest(request)) != NPF_SUCCESS)
		return(ret);

	return ret;

}/* end of NPF_IfATM_OAM_CP_Set function */


/*****************************************************************************
*    Name: NPF_IfATM_OAM_CC_Set 
*
*    Description:
*			This function Activates/Deactivates/Start/Stops Continuity Check procedure 
*			for OAM services on one or more ATM Connection Points associated with    
*			a Single ATM interface.
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM      - Invalid Parmaters
*
********************************************************************************/
NPF_error_t NPF_IfATM_OAM_CC_Set(
			NPF_IN NPF_callbackHandle_t	if_cbHandle,
			NPF_IN NPF_correlator_t if_cbCorrelator,
			NPF_IN NPF_errorReporting_t if_errorReporting,
			NPF_IN NPF_IfHandle_t ifHandle,
			NPF_IN NPF_uint32_t n_CPs,
			NPF_IN NPF_ATM_OAM_CC_t *if_oamCC_Array)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_RequestCB_t	*request;

	/* Trace the function */
	assert(NPF_IfATM_OAM_CC_Set);

	/* Check if the Parameters passed are O.K */
	if((ret = ifATM_CheckUserParams(if_cbHandle,if_errorReporting,ifHandle,
						n_CPs,if_oamCC_Array)) != NPF_SUCCESS)
		return(ret);


	/* Copy the received the list of Vccs in a request CB structure */
	if((ret = ifATM_CreateRequestCB(NPF_IF_ATM_OAM_CC_SET,if_cbHandle,if_cbCorrelator,
			  if_errorReporting,ifHandle, n_CPs,(void *)if_oamCC_Array,&request))
				 != NPF_SUCCESS)
		return(ret);


	/* Process the user request */
	if((ret = ifATM_ProcessUserRequest(request)) != NPF_SUCCESS)
		return(ret);

	return ret;

}/* end of NPF_IfATM_OAM_CC_Set function */



/*****************************************************************************
*    Name: NPF_IfATM_OAM_PM_Set 
*
*    Description:
*			This function Activates/Deactivates/Start/Stops Performance Mgmt procedure 
*			for OAM services on one or more ATM Connection Points associated with    
*			a Single ATM interface.
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM      - Invalid Parmaters
*
********************************************************************************/
NPF_error_t NPF_IfATM_OAM_PM_Set(
			NPF_IN NPF_callbackHandle_t	if_cbHandle,
			NPF_IN NPF_correlator_t if_cbCorrelator,
			NPF_IN NPF_errorReporting_t if_errorReporting,
			NPF_IN NPF_IfHandle_t ifHandle,
			NPF_IN NPF_uint32_t n_CPs,
			NPF_IN NPF_ATM_OAM_PM_t *if_oamPM_Array)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_RequestCB_t	*request;

	/* Trace the function */
	assert(NPF_IfATM_OAM_PM_Set);

	/* Check if the Parameters passed are O.K */
	if((ret = ifATM_CheckUserParams(if_cbHandle,if_errorReporting,ifHandle,
						n_CPs,if_oamPM_Array)) != NPF_SUCCESS)
		return(ret);


	/* Copy the received the list of Vccs in a request CB structure */
	if((ret = ifATM_CreateRequestCB(NPF_IF_ATM_OAM_PM_SET,if_cbHandle,if_cbCorrelator,
			  if_errorReporting,ifHandle, n_CPs,(void *)if_oamPM_Array,&request))
				 != NPF_SUCCESS)
		return(ret);


	/* Process the user request */
	if((ret = ifATM_ProcessUserRequest(request)) != NPF_SUCCESS)
		return(ret);

	return ret;

}/* end of NPF_IfATM_OAM_PM_Set function */


/*****************************************************************************
*    Name: NPF_IfATM_OAM_LB_Set 
*
*    Description:
*			This function Activates/Deactivates/Start/Stops LoopBack procedure 
*			for OAM services on one or more ATM Connection Points associated with    
*			a Single ATM interface.
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM      - Invalid Parmaters
*
********************************************************************************/
NPF_error_t NPF_IfATM_OAM_LB_Set(
			NPF_IN NPF_callbackHandle_t	if_cbHandle,
			NPF_IN NPF_correlator_t if_cbCorrelator,
			NPF_IN NPF_errorReporting_t if_errorReporting,
			NPF_IN NPF_IfHandle_t ifHandle,
			NPF_IN NPF_uint32_t n_CPs,
			NPF_IN NPF_ATM_OAM_LB_t *if_oamLB_Array)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_RequestCB_t	*request;

	/* Trace the function */
	assert(NPF_IfATM_OAM_LB_Set);

	/* Check if the Parameters passed are O.K */
	if((ret = ifATM_CheckUserParams(if_cbHandle,if_errorReporting,ifHandle,
						n_CPs,if_oamLB_Array)) != NPF_SUCCESS)
		return(ret);


	/* Copy the received the list of Vccs in a request CB structure */
	if((ret = ifATM_CreateRequestCB(NPF_IF_ATM_OAM_LB_SET,if_cbHandle,if_cbCorrelator,
			  if_errorReporting,ifHandle, n_CPs,(void *)if_oamLB_Array,&request))
				 != NPF_SUCCESS)
		return(ret);


	/* Process the user request */
	if((ret = ifATM_ProcessUserRequest(request)) != NPF_SUCCESS)
		return(ret);

	return ret;

}/* end of NPF_IfATM_OAM_LB_Set function */




/*****************************************************************************
*    Name: NPF_IfATM_OAM_Monitor 
*
*    Description:
*			This function Activates/Deactivates OAM cell Monitoring 
*			for OAM services on one or more ATM Connection Points associated with    
*			a Single ATM interface.
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM      - Invalid Parmaters
*
********************************************************************************/
NPF_error_t NPF_IfATM_OAM_Monitor(
			NPF_IN NPF_callbackHandle_t	if_cbHandle,
			NPF_IN NPF_correlator_t if_cbCorrelator,
			NPF_IN NPF_errorReporting_t if_errorReporting,
			NPF_IN NPF_IfHandle_t ifHandle,
			NPF_IN NPF_uint32_t n_CPs,
			NPF_IN NPF_ATM_OAM_Mon_t *if_oamMonArray)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_RequestCB_t	*request;

	/* Trace the function */
	assert(NPF_IfATM_OAM_Monitor);

	/* Check if the Parameters passed are O.K */
	if((ret = ifATM_CheckUserParams(if_cbHandle,if_errorReporting,ifHandle,
						n_CPs,if_oamMonArray)) != NPF_SUCCESS)
		return(ret);


	/* Copy the received the list of Vccs in a request CB structure */
	if((ret = ifATM_CreateRequestCB(NPF_IF_ATM_OAM_MONITOR,if_cbHandle,if_cbCorrelator,
			  if_errorReporting,ifHandle, n_CPs,(void *)if_oamMonArray,&request))
				 != NPF_SUCCESS)
		return(ret);


	/* Process the user request */
	if((ret = ifATM_ProcessUserRequest(request)) != NPF_SUCCESS)
		return(ret);

	return ret;

}/* end of NPF_IfATM_OAM_Monitor function */



/*****************************************************************************
*    Name: NPF_IfATM_OAM_AlarmSet 
*
*    Description:
*			This function Enables AIS/RDI Alarm 
*			for OAM services on one or more ATM Connection Points associated with    
*			a Single ATM interface.
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM      - Invalid Parmaters
*
********************************************************************************/
NPF_error_t NPF_IfATM_OAM_AlarmSet(
			NPF_IN NPF_callbackHandle_t	if_cbHandle,
			NPF_IN NPF_correlator_t if_cbCorrelator,
			NPF_IN NPF_errorReporting_t if_errorReporting,
			NPF_IN NPF_IfHandle_t ifHandle,
			NPF_IN NPF_uint32_t n_CPs,
			NPF_IN NPF_ATM_OAM_Alarm_t	*if_oamAlarmArray)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_RequestCB_t	*request;

	/* Trace the function */
	assert(NPF_IfATM_OAM_AlarmSet);

	/* Check if the Parameters passed are O.K */
	if((ret = ifATM_CheckUserParams(if_cbHandle,if_errorReporting,ifHandle,
						n_CPs,if_oamAlarmArray)) != NPF_SUCCESS)
		return(ret);


	/* Copy the received the list of Vccs in a request CB structure */
	if((ret = ifATM_CreateRequestCB(NPF_IF_ATM_OAM_ALARM_SET,if_cbHandle,if_cbCorrelator,
			  if_errorReporting,ifHandle,n_CPs ,(void *)if_oamAlarmArray,&request))
				 != NPF_SUCCESS)
		return(ret);


	/* Process the user request */
	if((ret = ifATM_ProcessUserRequest(request)) != NPF_SUCCESS)
		return(ret);

	return ret;

}/* end of NPF_IfATM_OAM_AlarmSet function */


/*****************************************************************************
*    Name: NPF_IfATM_OAM_AlarmClear 
*
*    Description:
*			This function Disables AIS/RDI Alarm 
*			for OAM services on one or more ATM Connection Points associated with    
*			a Single ATM interface.
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM      - Invalid Parmaters
*
********************************************************************************/
NPF_error_t NPF_IfATM_OAM_AlarmClear(
			NPF_IN NPF_callbackHandle_t	if_cbHandle,
			NPF_IN NPF_correlator_t if_cbCorrelator,
			NPF_IN NPF_errorReporting_t if_errorReporting,
			NPF_IN NPF_IfHandle_t ifHandle,
			NPF_IN NPF_uint32_t n_CPs,
			NPF_IN NPF_ATM_OAM_Alarm_t	*if_oamAlarmArray)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_RequestCB_t	*request;

	/* Trace the function */
	assert(NPF_IfATM_OAM_AlarmClear);

	/* Check if the Parameters passed are O.K */
	if((ret = ifATM_CheckUserParams(if_cbHandle,if_errorReporting,ifHandle,
						n_CPs,if_oamAlarmArray)) != NPF_SUCCESS)
		return(ret);


	/* Copy the received the list of Vccs in a request CB structure */
	if((ret = ifATM_CreateRequestCB(NPF_IF_ATM_OAM_ALARM_CLEAR,if_cbHandle,if_cbCorrelator,
			  if_errorReporting,ifHandle,n_CPs ,(void *)if_oamAlarmArray,&request))
				 != NPF_SUCCESS)
		return(ret);


	/* Process the user request */
	if((ret = ifATM_ProcessUserRequest(request)) != NPF_SUCCESS)
		return(ret);

	return ret;

}/* end of NPF_IfATM_OAM_AlarmClear function */


/*****************************************************************************
*    Name: NPF_IfATM_AAL2_ChannelSet 
*
*    Description:
*			This function sets an AAL2 connection on an ATM connection 
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM      - Invalid Parmaters
*
********************************************************************************/
NPF_error_t NPF_IfATM_AAL2_ChannelSet(
			NPF_IN NPF_callbackHandle_t	if_cbHandle,
			NPF_IN NPF_correlator_t if_cbCorrelator,
			NPF_IN NPF_errorReporting_t if_errorReporting,
			NPF_IN NPF_IfHandle_t ifHandle,
			NPF_IN NPF_uint32_t	n_channels,
			NPF_IN NPF_ATM_AAL2_ChannelInfo_t *if_aal2ChannelArray)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_RequestCB_t	*request;

	/* Trace the function */
	assert(NPF_IfATM_AAL2_ChannelSet);

	/* Check if the Parameters passed are O.K */
	if((ret = ifATM_CheckUserParams(if_cbHandle,if_errorReporting,ifHandle,
						n_channels,if_aal2ChannelArray)) != NPF_SUCCESS)
		return(ret);


	/* Copy the received the list of Vccs in a request CB structure */
	if((ret = ifATM_CreateRequestCB(NPF_IF_ATM_AAL2_CHANNEL_SET,if_cbHandle,if_cbCorrelator,
			  if_errorReporting,ifHandle, n_channels,(void *)if_aal2ChannelArray,&request))
				 != NPF_SUCCESS)
		return(ret);


	/* Process the user request */
	if((ret = ifATM_ProcessUserRequest(request)) != NPF_SUCCESS)
		return(ret);

	return ret;

}/* end of NPF_IfATM_AAL2_ChannelSet  function */



/*****************************************************************************
*    Name: NPF_IfATM_AAL2_ChannelClear 
*
*    Description:
*			This function deletes an AAL2 connection on an ATM connection 
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_OUT_OF_MEMORY           - out of memory
*			NPF_IF_E_INVALID_PARAM      - Invalid Parmaters
*
********************************************************************************/
NPF_error_t NPF_IfATM_AAL2_ChannelClear(
			NPF_IN NPF_callbackHandle_t	if_cbHandle,
			NPF_IN NPF_correlator_t if_cbCorrelator,
			NPF_IN NPF_errorReporting_t if_errorReporting,
			NPF_IN NPF_IfHandle_t ifHandle,
			NPF_IN NPF_uint32_t	n_channels,
			NPF_IN NPF_ATM_AAL2_ConnId_t	*if_connIdArray)
{

	NPF_error_t		ret = NPF_SUCCESS;
	IfATM_RequestCB_t	*request;

	/* Trace the function */
	assert(NPF_IfATM_AAL2_ChannelClear);

	/* Check if the Parameters passed are O.K */
	if((ret = ifATM_CheckUserParams(if_cbHandle,if_errorReporting,ifHandle,
						n_channels,if_connIdArray)) != NPF_SUCCESS)
		return(ret);


	/* Copy the received the list of Vccs in a request CB structure */
	if((ret = ifATM_CreateRequestCB(NPF_IF_ATM_AAL2_CHANNEL_DEL,if_cbHandle,if_cbCorrelator,
			  if_errorReporting,ifHandle, n_channels,(void *)if_connIdArray,&request))
				 != NPF_SUCCESS)
		return(ret);


	/* Process the user request */
	if((ret = ifATM_ProcessUserRequest(request)) != NPF_SUCCESS)
		return(ret);

	return ret;

}/* end of NPF_IfATM_AAL2_ChannelClear  function */



#endif /* _INCLUDE_ATM */
