/*****************************************************************************
 *                  INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 *$Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/APIImplementations/ATM/src/if_atm_callback.c,v $
 *$Revision: 1.5 $
 *$Date: 2003/11/17 16:57:09 $
 * Description:
 *     This file contains the interface management NPF api implementation.
 *
 *****************************************************************************/
#ifdef _INCLUDE_ATM

#include <assert.h>
#include <string.h>
#include "npf_common.h"
#include "npf_cm_types.h"
#include "pdk_cm_types.h"
#include "pdk_callback.h"
#include "pdk_callback_types.h"
#include "pdk_cm.h"
#include "pdkmgr.h"
#include "pil.h"
#include "fpp_common.h"
#include "fppapi.h"
#include "list.h"
#include "npf_if.h"
#include "pdk_ns_helper.h"

#include "fppapi.h"
#include "pdkmgr.h"
#include "cm.h"


#include "npf_atm_types.h"
#include "fppapi_atm_types.h"
#include "if.h"
#include "if_atm.h"
#include "if_atm_callback.h"


/* This array store the CB handle registered with FP-Plug In API */
extern FPPI_CBHANDLE	ifAtmFpPlugInRegCbHndl[NPF_IF_ATM_MAX_CB_TYPES];
extern IfATM_ControlBlock_t ifAtmCB;

/* Extern definitions of if-manager used in ATM */
extern DList EventCallbacks;
extern DList  IfList;
extern PilCriticalSection IM_LOCK;
extern NPF_error_t onResponse(request_t *request, void *resp); 

#ifndef COLOCATED
/* callback functions registered with CM */
extern void on_IntfEvent( NPF_USERCONTEXT context,
         NPF_HANDLE      if_npf_handle,
         NPF_EVENT       if_event,
         NPF_DATA        response_data);
#endif /* COLOCATED */

/*****************************************************************************
*                                                                            *
*  -------------------------- Internal Interface --------------------------  *
*                                                                            *
*****************************************************************************/

/*****************************************************************************
*    Name: ifAtmOnCB_checkParams 
*
*    Description:
*			This function checks the validity of the parameters passed to it. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*			NPF_IF_E_INVALID_PARAM      - invalid parameters
*
********************************************************************************/
NPF_error_t ifAtmOnCB_checkParams(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData,
					FPPI_CONTEXT	 actualContext)
{
	
	IfATM_FeTransInfo_t *tempCor = NULL;
	IfATM_RequestCB_t *request = NULL;

	
	/* Check if the context passed is correct */
	if((NPF_IfATM_CallbackType_t *)context != 
				(NPF_IfATM_CallbackType_t *)actualContext) 
		return NPF_IF_E_INVALID_PARAM;

	/* Check if the correlator passed is NULL */
	if( correlator == NULL)
		return NPF_IF_E_INVALID_PARAM;

	/* Get the Actual Request CB from the correlator passed */	
	tempCor = (IfATM_FeTransInfo_t *)correlator;
	request = tempCor->reqCbPtr;

	/* Check if the FE Id passed is correct */
	if( (request->feInfo[0]).feId != feId)
		return NPF_IF_E_INVALID_PARAM;

	return NPF_SUCCESS;

}



/*****************************************************************************
*    Name: ifAtmOnCB_SendRespToUser 
*
*    Description:
*			This function is used to send the asynchronous response to the user. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_REINITIALIAED - initialized before
*        NPF_OUT_OF_MEMORY           - out of memory
*
********************************************************************************/
NPF_error_t ifAtmOnCB_SendRespToUser(IfATM_RequestCB_t *request)
{
	NPF_GENERAL_CBFUNC cbFunc;
	NPF_userContext_t userContext;
	NPF_USERCONTEXT userContextPtr = (NPF_USERCONTEXT) &userContext;
	DListItr *itr = NULL;
	IfATM_RequestCB_t *tempRequest = NULL;
	NPF_IfCallbackData_t *userResponse = NULL;
	NPF_uint32_t dataSize = 0;
	NPF_uint32_t i = 0;
	

	/* Make a copy of the user Response as this will be freed later */
	if((userResponse = (NPF_IfCallbackData_t *)PIL_AllocateMemory
				(sizeof(NPF_IfCallbackData_t),PIL_MEM_ZERO_INIT)) == NULL)
		return NPF_OUT_OF_MEMORY;

	userResponse->type = (request->cbData).type;

	/* Depending upon the call back type take appropriate action. This is 
		need bec for all Get actions the response member of the callback
		data always has some information which may not be true for set
		actions */
	switch((request->cbData).type)
	{
		case NPF_IF_ATM_VCC_SET:
		case NPF_IF_ATM_VCC_BIND:
		case NPF_IF_ATM_VCC_DELETE:
		case NPF_IF_ATM_VCC_ENABLE:
		case NPF_IF_ATM_VCC_DISABLE:
		case NPF_IF_ATM_AAL2_CHANNEL_SET:
		case NPF_IF_ATM_AAL2_CHANNEL_DEL:
		case NPF_IF_ATM_CONNECTION_XC_SET:
		case NPF_IF_ATM_CONNECTION_XC_DEL:
		case NPF_IF_ATM_OAM_CP_SET:
		case NPF_IF_ATM_OAM_CC_SET:
		case NPF_IF_ATM_OAM_LB_SET:
		case NPF_IF_ATM_OAM_MONITOR:
		case NPF_IF_ATM_OAM_ALARM_SET:
		case NPF_IF_ATM_OAM_ALARM_CLEAR:
		case NPF_IF_ATM_OAM_PM_SET:
				/* Check if all the responses received from the FP are success */
				if( (request->feInfo[0]).numFail == 0)
				{
					/* Mark allOK = TRUE and n_resp = 0 */
					userResponse->allOK = TRUE;
					userResponse->n_resp = 0;
					userResponse->resp = NULL;
				}
				else
				{
		
					/* Not all responses are SUCCESS so build the response structure array */
					userResponse->allOK = FALSE;
					userResponse->n_resp = request->cbData.n_resp;

					dataSize = sizeof(NPF_IfAsyncResponse_t) * ((request->cbData).n_resp);

					if((userResponse->resp = (NPF_IfAsyncResponse_t *)PIL_AllocateMemory
											(dataSize,PIL_MEM_ZERO_INIT)) == NULL)
					{
						PIL_FreeMemory(userResponse);
						return NPF_OUT_OF_MEMORY;
					}
					else
					{
						/* Copy the response information from request CB */
						memcpy(userResponse->resp,(request->cbData).resp, dataSize);
					}
		
				}
				break;
		case NPF_IF_ATM_VCC_OPER_STATUS_GET:
				/* Check if all the responses received from the FP are success */
				if( (request->feInfo[0]).numFail == 0)
				{
					/* Mark allOK = TRUE */
					userResponse->allOK = TRUE;
				}
				else
				{
		
					/* Not all responses are SUCCESS so build the response structure array */
					userResponse->allOK = FALSE;
				}

				/* Set the number of responses in the callback structure */
				userResponse->n_resp = request->cbData.n_resp;
	
				dataSize = sizeof(NPF_IfAsyncResponse_t) * request->cbData.n_resp;

				/* Allocate memory for Asyc response structure */
				if((userResponse->resp = (NPF_IfAsyncResponse_t *)PIL_AllocateMemory
										(dataSize,PIL_MEM_ZERO_INIT)) == NULL)
				{
					PIL_FreeMemory(userResponse);
					return NPF_OUT_OF_MEMORY;
				}

				/* Copy the response information from request CB */
				memcpy(userResponse->resp,(request->cbData).resp,dataSize);
					
				break;
		case NPF_IF_ATM_VCC_STATS_GET:
				/* Check if all the responses received from the FP are success */
				if( (request->feInfo[0]).numFail == 0)
				{
					/* Mark allOK = TRUE */
					userResponse->allOK = TRUE;
				}
				else
				{
		
					/* Not all responses are SUCCESS so build the response structure array */
					userResponse->allOK = FALSE;
				}

				/* Set the number of responses in the callback structure */
				userResponse->n_resp = request->cbData.n_resp;

				dataSize = sizeof(NPF_IfAsyncResponse_t) * request->cbData.n_resp;

				/* Allocate memory for Asyc response structure */
				if((userResponse->resp = (NPF_IfAsyncResponse_t *)PIL_AllocateMemory
										(dataSize,PIL_MEM_ZERO_INIT)) == NULL)
				{
					PIL_FreeMemory(userResponse);
					return NPF_OUT_OF_MEMORY;
				}

				/* Copy the response information from request CB */
				memcpy(userResponse->resp, (request->cbData).resp, dataSize);
					
				for(i = 0; i< request->cbData.n_resp;i++)
				{
					/* Allocate memory for Vcc Stats structure */
					if((userResponse->resp[i].u.vccStats = (NPF_ATM_VccStats_t *)PIL_AllocateMemory
											(sizeof(NPF_ATM_VccStats_t),PIL_MEM_ZERO_INIT)) == NULL)
					{
						PIL_FreeMemory(userResponse);
						return NPF_OUT_OF_MEMORY;
					}
					
					/* Copy the Vcc Stats information from request CB */
					memcpy(userResponse->resp[i].u.vccStats,(request->cbData).resp[i].u.vccStats,
							 sizeof(NPF_ATM_VccStats_t));

				}
				break;
		default:
				break;
	}
 

	/* Get the User registered Context and Call Back Function from the CB handle */
	cbFunc = pdk_callback_func((NPF_CBHANDLE)request->cbh, (NPF_USERCONTEXT *)&userContextPtr);

	/* Send the response to the user */
	cbthread_pushIFAPICB( cbFunc,userContext,request->corr,userResponse);

	/* Free the memory previously allocated for the response structure */
	PIL_FreeMemory(userResponse);

	/* Remove the current requestCB from the ATM CB list */
	/* Take the lock for the Module CB */
	PIL_EnterCriticalSection(&ifAtmCB.lock);

	/* Create the iterator */
	if ((itr = npf_list_itrCreate((DList *)&(ifAtmCB.apiReqList))) != NULL)
	{
		npf_list_itrFirst(itr);
		do
		{
			/* Get the Request CB from the iterator */
			tempRequest = (IfATM_RequestCB_t *) npf_list_itrGetData(itr); 
			
			/* Check if this is the request CB we are looking for */
			if( (tempRequest->cbh == request->cbh)	&&
				(tempRequest->corr == request->corr) &&
				(tempRequest->report == request->report) &&
				(tempRequest == request))
			{
				/* Free all the memory associated with this request block */
				npf_list_itrRemoveFree(itr);
				break;
			}
		}while (npf_list_itrNext(itr) != -1);
	
		/* Delete the iterator that was created */	
		npf_list_itrDelete(itr);
	}

	/* Give the lock for the Module CB */
	PIL_LeaveCriticalSection(&ifAtmCB.lock);

	return NPF_SUCCESS;

}
			





/*****************************************************************************
*    Name: ifAtmOnCB_VccSetFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous VccSet Request. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_VccSetFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	IfATM_FeTransInfo_t *tempCorr = NULL;
	IfATM_RequestCB_t *request = NULL;
	NPF_uint32_t i = 0;
	IfATM_IfVcc_t *tempVcc = NULL;
	IfATM_FpResponse_t *tempResp = NULL;

	/* Check if the received parameters are proper */	
	if(ifAtmOnCB_checkParams
					(feId, correlator, context, responseData,(FPPI_CONTEXT)NPF_IF_ATM_VCC_SET)
					!= NPF_SUCCESS)
		return ;

	/* From the correlator get the request CB */
	tempCorr = (IfATM_FeTransInfo_t *)correlator;

	request = (IfATM_RequestCB_t *)tempCorr->reqCbPtr;

	/* Make the tempVcc pointer to point to entry member in request */
	tempVcc = (IfATM_IfVcc_t *)(request->apiData).entry; 

	/* Make the tempResp pointer point to the responseData */
	tempResp = (IfATM_FpResponse_t *)responseData;


	/* Using the request CB and Correlator get the entry index for which the
		response was received */
	for(i=0;i < (request->apiData).numEntries; i++)
	{
		if( &tempVcc[i] == tempCorr->entryPtr)
		{
			break;
		}
	}

	/* Store the response recvd in the asynchronous response to be sent to user */
	(request->cbData).resp[i].error = tempResp->status;

	/* Store the Vcc Address of the received response */
	(request->cbData).resp[i].u.VccAddr = tempVcc[i].vccInfo.vcc;

	/* Increment the count of responses in the async structure */
	(request->cbData).n_resp ++;


	/* Decrement the count of responses pending in the Request CB */
	(request->feInfo[0]).numWait --;
	request->numTransWait --;

	/* Check if the status received is Failure */
	if( tempResp->status != FPPI_SUCCESS)
		(request->feInfo[0]).numFail ++;

	/* Check if all responses have been received */
	if(request->numTransWait == 0) 
	{
		/* Send the response to the user */
		ifAtmOnCB_SendRespToUser(request);
	}
	
}

/*****************************************************************************
*    Name: ifAtmOnCB_VccBindFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous VccBind Request. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_VccBindFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	IfATM_FeTransInfo_t *tempCorr = NULL;
	IfATM_RequestCB_t *request = NULL;
	NPF_uint32_t i = 0;
	IfATM_VccBind_t *tempVcc = NULL;
	IfATM_FpResponse_t *tempResp = NULL;
#ifndef COLOCATED
	NPF_INTERFACE_t ifAttr;
#endif /* COLOCATED */

	/* Check if the received parameters are proper */	
	if(ifAtmOnCB_checkParams
					(feId, correlator, context, responseData,(FPPI_CONTEXT)NPF_IF_ATM_VCC_BIND)
					!= NPF_SUCCESS)
		return ;

	/* From the correlator get the request CB */
	tempCorr = (IfATM_FeTransInfo_t *)correlator;

	request = (IfATM_RequestCB_t *)tempCorr->reqCbPtr;

	/* Make the tempVcc pointer to point to entry member in request */
	tempVcc = (IfATM_VccBind_t *)(request->apiData).entry; 

	/* Make the tempResp pointer point to the responseData */
	tempResp = (IfATM_FpResponse_t *)responseData;


	/* Using the request CB and Correlator get the entry index for which the
		response was received */
	for(i=0;i < (request->apiData).numEntries; i++)
	{
		if( &tempVcc[i] == tempCorr->entryPtr)
		{
			break;
		}
	}

	/* Store the response recvd in the asynchronous response to be sent to user */
	(request->cbData).resp[i].error = tempResp->status;

	/* Store the Vcc Address of the received response */
	(request->cbData).resp[i].u.VccAddr = tempVcc[i].vccInfo;

	/* Increment the count of responses in the async structure */
	(request->cbData).n_resp ++;


	/* Decrement the count of responses pending in the Request CB */
	(request->feInfo[0]).numWait --;
	request->numTransWait --;

	/* Check if the status received is Failure */
	if( tempResp->status != FPPI_SUCCESS)
		(request->feInfo[0]).numFail ++;


#ifndef COLOCATED
	/* The following section of code is applicable only for REMOTE configuration
		of CP-PDK. For colocated some more code needs to be added which will be
		done in future releases of the code */
	/* If status is SUCCESS than inform the VIDD controller to add a virtual 
		device for this Bind upper interface */
	else
	{
		/* Get the parent (IP) interface attributes */
		cm_if_get_attribs(tempVcc->parentIfHandle,&ifAttr);

		/* Call the VIDD module function to create a virtual device for this 
			interface */
		on_IntfEvent((NPF_USERCONTEXT)PDK_EVT_IF,tempVcc->parentIfHandle,
							NPF_EVENT_IF_UP,(NPF_DATA)&ifAttr);
	}
#endif /* COLOCATED */	

	/* Check if all responses have been received */
	if(request->numTransWait == 0) 
	{
		/* Send the response to the user */
		ifAtmOnCB_SendRespToUser(request);
	}
	
}

/*****************************************************************************
*    Name: ifAtmOnCB_VccDelFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous VccDel Request. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_VccDelFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	IfATM_FeTransInfo_t *tempCorr = NULL;
	IfATM_RequestCB_t *request = NULL;
	NPF_uint32_t i = 0;
	IfATM_VccAddr_t *vcc = NULL;
	IfATM_FpResponse_t *tempResp = NULL;
	

	/* Check if the received parameters are proper */	
	if( ifAtmOnCB_checkParams
					(feId, correlator, context, responseData,(FPPI_CONTEXT)NPF_IF_ATM_VCC_DELETE)
					!= NPF_SUCCESS)
		return ;

	/* From the correlator get the request CB */
	tempCorr = (IfATM_FeTransInfo_t *)correlator;

	request = (IfATM_RequestCB_t *)tempCorr->reqCbPtr;

	/* Make the vcc pointer to point to entry member in request */
	vcc = (IfATM_VccAddr_t *)(request->apiData).entry; 

	/* Make the tempResp pointer point to the responseData */
	tempResp =  (IfATM_FpResponse_t *)responseData;

	/* Using the request CB and Correlator get the entry index for which the
		response was received */
	for(i=0;i < (request->apiData).numEntries; i++)
	{
		if( &vcc[i] == tempCorr->entryPtr)
		{
			break;
		}
	}

	/* Store the response recvd in the asynchronous response to be sent to user */
	(request->cbData).resp[i].error = tempResp->status;

	/* Store the Vcc Address of the received response */
	(request->cbData).resp[i].u.VccAddr = vcc[i].vccAddr;

	/* Increment the count of responses in the async structure */
	(request->cbData).n_resp ++;


	/* Decrement the count of responses pending in the Request CB */
	(request->feInfo[0]).numWait --;
	request->numTransWait --;

	/* Check if the status received is Failure */
	if( tempResp->status != FPPI_SUCCESS)
		(request->feInfo[0]).numFail ++;

	/* Check if all responses have been received */
	if(request->numTransWait == 0) 
	{
		/* Send the response to the user */
		ifAtmOnCB_SendRespToUser(request);
	}
	
}

/*****************************************************************************
*    Name: ifAtmOnCB_VccEnableFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous VccEnable Request. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_VccEnableFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	IfATM_FeTransInfo_t *tempCorr = NULL;
	IfATM_RequestCB_t *request = NULL;
	NPF_uint32_t i = 0;
	IfATM_VccAddr_t *vcc = NULL;
	IfATM_FpResponse_t *tempResp =NULL;

	/* Check if the received parameters are proper */	
	if(ifAtmOnCB_checkParams
					(feId, correlator, context, responseData,(FPPI_CONTEXT)NPF_IF_ATM_VCC_ENABLE)
					!= NPF_SUCCESS)
		return;

	/* From the correlator get the request CB */
	tempCorr = (IfATM_FeTransInfo_t *)correlator;

	request = (IfATM_RequestCB_t *)tempCorr->reqCbPtr;

	/* Make the vcc pointer to point to entry member in request */
	vcc = (IfATM_VccAddr_t *)(request->apiData).entry; 

	/* Make the tempResp pointer point to the responseData */
	tempResp =  (IfATM_FpResponse_t *)responseData;

	/* Using the request CB and Correlator get the entry index for which the
		response was received */
	for(i=0;i < (request->apiData).numEntries; i++)
	{
		if( &vcc[i] == tempCorr->entryPtr)
		{
			break;
		}
	}

	/* Store the response recvd in the asynchronous response to be sent to user */
	(request->cbData).resp[i].error = tempResp->status;

	/* Store the Vcc Address of the received response */
	(request->cbData).resp[i].u.VccAddr = vcc[i].vccAddr;

	/* Increment the count of responses in the async structure */
	(request->cbData).n_resp ++;


	/* Decrement the count of responses pending in the Request CB */
	(request->feInfo[0]).numWait --;
	request->numTransWait --;

	/* Check if the status received is Failure */
	if( tempResp->status != FPPI_SUCCESS)
		(request->feInfo[0]).numFail ++;

	/* Check if all responses have been received */
	if(request->numTransWait == 0) 
	{
		/* Send the response to the user */
		ifAtmOnCB_SendRespToUser(request);
	}
	
}

/*****************************************************************************
*    Name: ifAtmOnCB_VccDisableFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous VccDisable Request. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_VccDisableFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	IfATM_FeTransInfo_t *tempCorr = NULL;
	IfATM_RequestCB_t *request = NULL;
	NPF_uint32_t i = 0;
	IfATM_VccAddr_t *vcc = NULL;
	IfATM_FpResponse_t *tempResp = NULL;

	/* Check if the received parameters are proper */	
	if(ifAtmOnCB_checkParams
					(feId, correlator, context, responseData,(FPPI_CONTEXT)NPF_IF_ATM_VCC_DISABLE)
					!= NPF_SUCCESS)
		return ;

	/* From the correlator get the request CB */
	tempCorr = (IfATM_FeTransInfo_t *)correlator;

	request = (IfATM_RequestCB_t *)tempCorr->reqCbPtr;


	/* Make the vcc pointer to point to entry member in request */
	vcc = (IfATM_VccAddr_t *)(request->apiData).entry; 

	/* Make the tempResp pointer point to the responseData */
	tempResp =  (IfATM_FpResponse_t *)responseData;

	/* Using the request CB and Correlator get the entry index for which the
		response was received */
	for(i=0;i < (request->apiData).numEntries; i++)
	{
		if( &vcc[i] == tempCorr->entryPtr)
		{
			break;
		}
	}

	/* Store the response recvd in the asynchronous response to be sent to user */
	(request->cbData).resp[i].error = tempResp->status;

	/* Store the Vcc Address of the received response */
	(request->cbData).resp[i].u.VccAddr = vcc[i].vccAddr;

	/* Increment the count of responses in the async structure */
	(request->cbData).n_resp ++;


	/* Decrement the count of responses pending in the Request CB */
	(request->feInfo[0]).numWait --;
	request->numTransWait --;

	/* Check if the status received is Failure */
	if( tempResp->status != FPPI_SUCCESS)
		(request->feInfo[0]).numFail ++;

	/* Check if all responses have been received */
	if(request->numTransWait == 0) 
	{
		/* Send the response to the user */
		ifAtmOnCB_SendRespToUser(request);
	}
	
}


/*****************************************************************************
*    Name: ifAtmOnCB_VccOperStatusGetFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous VccOperStatusGet Req. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_VccOperStatusGetFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	IfATM_FeTransInfo_t *tempCorr = NULL;
	IfATM_RequestCB_t *request = NULL;
	NPF_uint32_t i = 0;
	IfATM_VccAddr_t *vcc = NULL;
	IfATM_FpResponse_t *tempResp = NULL;

	/* Check if the received parameters are proper */	
	if(ifAtmOnCB_checkParams
					(feId, correlator, context, responseData,(FPPI_CONTEXT)NPF_IF_ATM_VCC_OPER_STATUS_GET)
					!= NPF_SUCCESS)
		return;

	/* From the correlator get the request CB */
	tempCorr = (IfATM_FeTransInfo_t *)correlator;

	request = (IfATM_RequestCB_t *)tempCorr->reqCbPtr;


	/* Make the vcc pointer to point to entry member in request */
	vcc = (IfATM_VccAddr_t *)(request->apiData).entry; 

	/* Make the tempResp pointer point to the responseData */
	tempResp =  (IfATM_FpResponse_t *)responseData;

	/* Using the request CB and Correlator get the entry index for which the
		response was received */
	for(i=0;i < (request->apiData).numEntries; i++)
	{
		if( &vcc[i] == tempCorr->entryPtr)
		{
			break;
		}
	}

	/* Store the response recvd in the asynchronous response to be sent to user */
	(request->cbData).resp[i].error = tempResp->status;

	/* Store the Vcc Address of the received response */
	(request->cbData).resp[i].u.VccAddr = vcc[i].vccAddr;

	/* Increment the count of responses in the async structure */
	(request->cbData).n_resp ++;


	/* Decrement the count of responses pending in the Request CB */
	(request->feInfo[0]).numWait --;
	request->numTransWait --;

	/* Check if the status received is Failure */
	if( tempResp->status != FPPI_SUCCESS)
		(request->feInfo[0]).numFail ++;

	/* Check if all responses have been received */
	if(request->numTransWait == 0) 
	{
		/* Send the response to the user */
		ifAtmOnCB_SendRespToUser(request);
	}
	
}

/*****************************************************************************
*    Name: ifAtmOnCB_VccStatsGetFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous VccStatsGet Req. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_VccStatsGetFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	IfATM_FeTransInfo_t *tempCorr = NULL;
	IfATM_RequestCB_t *request = NULL;
	NPF_uint32_t i = 0;
	IfATM_VccStats_t *stats = NULL;
	IfATM_VccAddr_t *vcc = NULL;

	/* Check if the received parameters are proper */	
	if(ifAtmOnCB_checkParams
					(feId, correlator, context, responseData,(FPPI_CONTEXT)NPF_IF_ATM_VCC_STATS_GET)
					!= NPF_SUCCESS)
		return ;

	/* From the correlator get the request CB */
	tempCorr = (IfATM_FeTransInfo_t *)correlator;

	request = (IfATM_RequestCB_t *)tempCorr->reqCbPtr;
	stats = (IfATM_VccStats_t *)responseData;


	/* Make the vcc pointer to point to entry member in request */
	vcc = (IfATM_VccAddr_t *)(request->apiData).entry; 


	/* Using the request CB and Correlator get the entry index for which the
		response was received */
	for(i=0;i < (request->apiData).numEntries; i++)
	{
		if( &vcc[i] == tempCorr->entryPtr)
		{
			break;
		}
	}

	/* Store the response recvd in the asynchronous response to be sent to user */
	(request->cbData).resp[i].error = (stats->response).status;

	/* Allocate Memory for the received statistics */
	(request->cbData).resp[i].u.vccStats = (NPF_ATM_VccStats_t *)PIL_AllocateMemory(
							sizeof(NPF_ATM_VccStats_t),PIL_MEM_ZERO_INIT);
	if((request->cbData).resp[i].u.vccStats == NULL)
	{
		return; /* TBD */
	}

	/* Store the Vcc Statistics of the received response */
	memcpy((request->cbData).resp[i].u.vccStats,
				&(stats->vccStats),sizeof(NPF_ATM_VccStats_t));

	/* Increment the count of responses in the async structure */
	(request->cbData).n_resp ++;


	/* Decrement the count of responses pending in the Request CB */
	(request->feInfo[0]).numWait --;
	request->numTransWait --;

	/* Check if the status received is Failure */
	if( (stats->response).status != FPPI_SUCCESS)
		(request->feInfo[0]).numFail ++;

	/* Check if all responses have been received */
	if(request->numTransWait == 0) 
	{
		/* Send the response to the user */
		ifAtmOnCB_SendRespToUser(request);
	}
	
}

/*****************************************************************************
*    Name: ifAtmOnCB_Aal2ChannelSetFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous Aal2ChanSet Req. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_Aal2ChannelSetFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	IfATM_FeTransInfo_t *tempCorr = NULL;
	IfATM_RequestCB_t *request = NULL;
	NPF_uint32_t i = 0;
	IfATM_AAL2_ChannelInfo_t *aal2ChanInfo = NULL;
	IfATM_FpResponse_t *tempResp = NULL;

	/* Check if the received parameters are proper */	
	if(ifAtmOnCB_checkParams
					(feId, correlator, context, responseData,(FPPI_CONTEXT)NPF_IF_ATM_AAL2_CHANNEL_SET)
					!= NPF_SUCCESS)
		return;

	/* From the correlator get the request CB */
	tempCorr = (IfATM_FeTransInfo_t *)correlator;

	request = (IfATM_RequestCB_t *)tempCorr->reqCbPtr;

	/* Make the tempResp pointer point to the responseData */
	tempResp =  (IfATM_FpResponse_t *)responseData;

	/* Make the tempVcc pointer to point to entry member in request */
	aal2ChanInfo = (IfATM_AAL2_ChannelInfo_t *)(request->apiData).entry; 

	/* Using the request CB and Correlator get the entry index for which the
		response was received */
	for(i=0;i < (request->apiData).numEntries; i++)
	{
		if( &aal2ChanInfo[i] == tempCorr->entryPtr)
		{
			break;
		}
	}

	/* Store the response recvd in the asynchronous response to be sent to user */
	(request->cbData).resp[i].error = tempResp->status;

	/* Store the Vcc Address of the received response */
	(request->cbData).resp[i].u.aal2ConnId = aal2ChanInfo[i].aal2ChanInfo.aal2ConnId;

	/* Increment the count of responses in the async structure */
	(request->cbData).n_resp ++;


	/* Decrement the count of responses pending in the Request CB */
	(request->feInfo[0]).numWait --;
	request->numTransWait --;

	/* Check if the status received is Failure */
	if( tempResp->status != FPPI_SUCCESS)
		(request->feInfo[0]).numFail ++;

	/* Check if all responses have been received */
	if(request->numTransWait == 0) 
	{
		/* Send the response to the user */
		ifAtmOnCB_SendRespToUser(request);
	}
	
}

/*****************************************************************************
*    Name: ifAtmOnCB_Aal2ChannelDelFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous Aal2ChanDel Req. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_Aal2ChannelDelFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	IfATM_FeTransInfo_t *tempCorr = NULL;
	IfATM_RequestCB_t *request = NULL;
	NPF_uint32_t i = 0;
	IfATM_AAL2_ConnId_t  *aal2ConnId = NULL; 
	IfATM_FpResponse_t *tempResp = NULL;

	/* Check if the received parameters are proper */	
	if(ifAtmOnCB_checkParams
					(feId, correlator, context, responseData,(FPPI_CONTEXT)NPF_IF_ATM_AAL2_CHANNEL_DEL)
					!= NPF_SUCCESS)
		return;

	/* From the correlator get the request CB */
	tempCorr = (IfATM_FeTransInfo_t *)correlator;

	request = (IfATM_RequestCB_t *)tempCorr->reqCbPtr;

	/* Make the tempResp pointer point to the responseData */
	tempResp =  (IfATM_FpResponse_t *)responseData;

	/* Make the tempVcc pointer to point to entry member in request */
	aal2ConnId = (IfATM_AAL2_ConnId_t *)(request->apiData).entry; 

	/* Using the request CB and Correlator get the entry index for which the
		response was received */
	for(i=0;i < (request->apiData).numEntries; i++)
	{
		if( &aal2ConnId[i] == tempCorr->entryPtr)
		{
			break;
		}
	}

	/* Store the response recvd in the asynchronous response to be sent to user */
	(request->cbData).resp[i].error = tempResp->status;

	/* Store the Vcc Address of the received response */
	(request->cbData).resp[i].u.aal2ConnId = aal2ConnId[i].aal2ConId;

	/* Increment the count of responses in the async structure */
	(request->cbData).n_resp ++;


	/* Decrement the count of responses pending in the Request CB */
	(request->feInfo[0]).numWait --;
	request->numTransWait --;

	/* Check if the status received is Failure */
	if( tempResp->status != FPPI_SUCCESS)
		(request->feInfo[0]).numFail ++;

	/* Check if all responses have been received */
	if(request->numTransWait == 0) 
	{
		/* Send the response to the user */
		ifAtmOnCB_SendRespToUser(request);
	}
	
}




/*****************************************************************************
*    Name: ifAtmOnCB_ConnXcSetFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous ConnXcSet Req. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_ConnXcSetFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	IfATM_FeTransInfo_t *tempCorr = NULL;
	IfATM_RequestCB_t *request = NULL;
	NPF_uint32_t i = 0;
	NPF_ATM_ConnectionXc_t *connXcInfo = NULL;
	IfATM_FpResponse_t *tempResp = NULL;

	/* Check if the received parameters are proper */	
	if(ifAtmOnCB_checkParams
					(feId, correlator, context, responseData,(FPPI_CONTEXT)NPF_IF_ATM_CONNECTION_XC_SET)
					!= NPF_SUCCESS)
		return;

	/* From the correlator get the request CB */
	tempCorr = (IfATM_FeTransInfo_t *)correlator;

	request = (IfATM_RequestCB_t *)tempCorr->reqCbPtr;

	/* Make the tempResp pointer point to the responseData */
	tempResp =  (IfATM_FpResponse_t *)responseData;

	/* Make the tempVcc pointer to point to entry member in request */
	connXcInfo = (NPF_ATM_ConnectionXc_t *)(request->apiData).entry; 

	/* Using the request CB and Correlator get the entry index for which the
		response was received */
	for(i=0;i < (request->apiData).numEntries; i++)
	{
		if( &connXcInfo[i] == tempCorr->entryPtr)
		{
			break;
		}
	}

	/* Store the response recvd in the asynchronous response to be sent to user */
	(request->cbData).resp[i].error = tempResp->status;

	/* Store the Vcc Address of the received response */
	(request->cbData).resp[i].u.connXcInfo = connXcInfo[i];

	/* Increment the count of responses in the async structure */
	(request->cbData).n_resp ++;


	/* Decrement the count of responses pending in the Request CB */
	(request->feInfo[0]).numWait --;
	request->numTransWait --;

	/* Check if the status received is Failure */
	if( tempResp->status != FPPI_SUCCESS)
		(request->feInfo[0]).numFail ++;

	/* Check if all responses have been received */
	if(request->numTransWait == 0) 
	{
		/* Send the response to the user */
		ifAtmOnCB_SendRespToUser(request);
	}
	
}




/*****************************************************************************
*    Name: ifAtmOnCB_ConnXcDelFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous ConnXcDel Req. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_ConnXcDelFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	IfATM_FeTransInfo_t *tempCorr = NULL;
	IfATM_RequestCB_t *request = NULL;
	NPF_uint32_t i = 0;
	NPF_ATM_ConnectionXc_t *connXcInfo = NULL;
	IfATM_FpResponse_t *tempResp = NULL;

	/* Check if the received parameters are proper */	
	if(ifAtmOnCB_checkParams
					(feId, correlator, context, responseData,(FPPI_CONTEXT)NPF_IF_ATM_CONNECTION_XC_DEL)
					!= NPF_SUCCESS)
		return;

	/* From the correlator get the request CB */
	tempCorr = (IfATM_FeTransInfo_t *)correlator;

	request = (IfATM_RequestCB_t *)tempCorr->reqCbPtr;

	/* Make the tempResp pointer point to the responseData */
	tempResp =  (IfATM_FpResponse_t *)responseData;

	/* Make the tempVcc pointer to point to entry member in request */
	connXcInfo = (NPF_ATM_ConnectionXc_t *)(request->apiData).entry; 

	/* Using the request CB and Correlator get the entry index for which the
		response was received */
	for(i=0;i < (request->apiData).numEntries; i++)
	{
		if( &connXcInfo[i] == tempCorr->entryPtr)
		{
			break;
		}
	}

	/* Store the response recvd in the asynchronous response to be sent to user */
	(request->cbData).resp[i].error = tempResp->status;

	/* Store the Vcc Address of the received response */
	(request->cbData).resp[i].u.connXcInfo = connXcInfo[i];

	/* Increment the count of responses in the async structure */
	(request->cbData).n_resp ++;


	/* Decrement the count of responses pending in the Request CB */
	(request->feInfo[0]).numWait --;
	request->numTransWait --;

	/* Check if the status received is Failure */
	if( tempResp->status != FPPI_SUCCESS)
		(request->feInfo[0]).numFail ++;

	/* Check if all responses have been received */
	if(request->numTransWait == 0) 
	{
		/* Send the response to the user */
		ifAtmOnCB_SendRespToUser(request);
	}
	
}



/*****************************************************************************
*    Name: ifAtmOnCB_OamCpSetFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous OAM_CP_Set Req. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_OamCpSetFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	IfATM_FeTransInfo_t *tempCorr = NULL;
	IfATM_RequestCB_t *request = NULL;
	NPF_uint32_t i = 0;
	IfATM_OAM_CP_t *oamCpInfo = NULL;
	IfATM_FpResponse_t *tempResp = NULL;

	/* Check if the received parameters are proper */	
	if(ifAtmOnCB_checkParams
					(feId, correlator, context, responseData,(FPPI_CONTEXT)NPF_IF_ATM_OAM_CP_SET)
					!= NPF_SUCCESS)
		return;

	/* From the correlator get the request CB */
	tempCorr = (IfATM_FeTransInfo_t *)correlator;

	request = (IfATM_RequestCB_t *)tempCorr->reqCbPtr;

	/* Make the tempResp pointer point to the responseData */
	tempResp =  (IfATM_FpResponse_t *)responseData;

	/* Make the oamCpInfo pointer to point to entry member in request */
	oamCpInfo = (IfATM_OAM_CP_t *)(request->apiData).entry; 

	/* Using the request CB and Correlator get the entry index for which the
		response was received */
	for(i=0;i < (request->apiData).numEntries; i++)
	{
		if( &oamCpInfo[i] == tempCorr->entryPtr)
		{
			break;
		}
	}

	/* Store the response recvd in the asynchronous response to be sent to user */
	(request->cbData).resp[i].error = tempResp->status;

	/* Store the Vcc Address of the received response */
	(request->cbData).resp[i].u.VccAddr = oamCpInfo[i].cpInfo.connection;

	/* Increment the count of responses in the async structure */
	(request->cbData).n_resp ++;


	/* Decrement the count of responses pending in the Request CB */
	(request->feInfo[0]).numWait --;
	request->numTransWait --;

	/* Check if the status received is Failure */
	if( tempResp->status != FPPI_SUCCESS)
		(request->feInfo[0]).numFail ++;

	/* Check if all responses have been received */
	if(request->numTransWait == 0) 
	{
		/* Send the response to the user */
		ifAtmOnCB_SendRespToUser(request);
	}
	
}


/*****************************************************************************
*    Name: ifAtmOnCB_OamCcSetFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous OAM_CC_Set Req. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_OamCcSetFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	IfATM_FeTransInfo_t *tempCorr = NULL;
	IfATM_RequestCB_t *request = NULL;
	NPF_uint32_t i = 0;
	IfATM_OAM_CC_t *oamCcInfo = NULL;
	IfATM_FpResponse_t *tempResp = NULL;

	/* Check if the received parameters are proper */	
	if(ifAtmOnCB_checkParams
					(feId, correlator, context, responseData,(FPPI_CONTEXT)NPF_IF_ATM_OAM_CC_SET)
					!= NPF_SUCCESS)
		return;

	/* From the correlator get the request CB */
	tempCorr = (IfATM_FeTransInfo_t *)correlator;

	request = (IfATM_RequestCB_t *)tempCorr->reqCbPtr;

	/* Make the tempResp pointer point to the responseData */
	tempResp =  (IfATM_FpResponse_t *)responseData;

	/* Make the oamCpInfo pointer to point to entry member in request */
	oamCcInfo = (IfATM_OAM_CC_t *)(request->apiData).entry; 

	/* Using the request CB and Correlator get the entry index for which the
		response was received */
	for(i=0;i < (request->apiData).numEntries; i++)
	{
		if( &oamCcInfo[i] == tempCorr->entryPtr)
		{
			break;
		}
	}

	/* Store the response recvd in the asynchronous response to be sent to user */
	(request->cbData).resp[i].error = tempResp->status;

	/* Store the Vcc Address of the received response */
	(request->cbData).resp[i].u.VccAddr = oamCcInfo[i].ccInfo.connection;

	/* Increment the count of responses in the async structure */
	(request->cbData).n_resp ++;


	/* Decrement the count of responses pending in the Request CB */
	(request->feInfo[0]).numWait --;
	request->numTransWait --;

	/* Check if the status received is Failure */
	if( tempResp->status != FPPI_SUCCESS)
		(request->feInfo[0]).numFail ++;

	/* Check if all responses have been received */
	if(request->numTransWait == 0) 
	{
		/* Send the response to the user */
		ifAtmOnCB_SendRespToUser(request);
	}
	
}




/*****************************************************************************
*    Name: ifAtmOnCB_OamLbSetFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous OAM_LB_Set Req. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_OamLbSetFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	IfATM_FeTransInfo_t *tempCorr = NULL;
	IfATM_RequestCB_t *request = NULL;
	NPF_uint32_t i = 0;
	IfATM_OAM_LB_t *oamLbInfo = NULL;
	IfATM_FpResponse_t *tempResp = NULL;

	/* Check if the received parameters are proper */	
	if(ifAtmOnCB_checkParams
					(feId, correlator, context, responseData,(FPPI_CONTEXT)NPF_IF_ATM_OAM_LB_SET)
					!= NPF_SUCCESS)
		return;

	/* From the correlator get the request CB */
	tempCorr = (IfATM_FeTransInfo_t *)correlator;

	request = (IfATM_RequestCB_t *)tempCorr->reqCbPtr;

	/* Make the tempResp pointer point to the responseData */
	tempResp =  (IfATM_FpResponse_t *)responseData;

	/* Make the oamCpInfo pointer to point to entry member in request */
	oamLbInfo = (IfATM_OAM_LB_t *)(request->apiData).entry; 

	/* Using the request CB and Correlator get the entry index for which the
		response was received */
	for(i=0;i < (request->apiData).numEntries; i++)
	{
		if( &oamLbInfo[i] == tempCorr->entryPtr)
		{
			break;
		}
	}

	/* Store the response recvd in the asynchronous response to be sent to user */
	(request->cbData).resp[i].error = tempResp->status;

	/* Store the Vcc Address of the received response */
	(request->cbData).resp[i].u.VccAddr = oamLbInfo[i].lbInfo.connection;

	/* Increment the count of responses in the async structure */
	(request->cbData).n_resp ++;


	/* Decrement the count of responses pending in the Request CB */
	(request->feInfo[0]).numWait --;
	request->numTransWait --;

	/* Check if the status received is Failure */
	if( tempResp->status != FPPI_SUCCESS)
		(request->feInfo[0]).numFail ++;

	/* Check if all responses have been received */
	if(request->numTransWait == 0) 
	{
		/* Send the response to the user */
		ifAtmOnCB_SendRespToUser(request);
	}
	
}



/*****************************************************************************
*    Name: ifAtmOnCB_OamMonitorFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous OAM_Monitor Req. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_OamMonitorFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	IfATM_FeTransInfo_t *tempCorr = NULL;
	IfATM_RequestCB_t *request = NULL;
	NPF_uint32_t i = 0;
	IfATM_OAM_Mon_t *oamMonitorInfo = NULL;
	IfATM_FpResponse_t *tempResp = NULL;

	/* Check if the received parameters are proper */	
	if(ifAtmOnCB_checkParams
					(feId, correlator, context, responseData,(FPPI_CONTEXT)NPF_IF_ATM_OAM_MONITOR)
					!= NPF_SUCCESS)
		return;

	/* From the correlator get the request CB */
	tempCorr = (IfATM_FeTransInfo_t *)correlator;

	request = (IfATM_RequestCB_t *)tempCorr->reqCbPtr;

	/* Make the tempResp pointer point to the responseData */
	tempResp =  (IfATM_FpResponse_t *)responseData;

	/* Make the oamMonitorInfo pointer to point to entry member in request */
	oamMonitorInfo = (IfATM_OAM_Mon_t *)(request->apiData).entry; 

	/* Using the request CB and Correlator get the entry index for which the
		response was received */
	for(i=0;i < (request->apiData).numEntries; i++)
	{
		if( &oamMonitorInfo[i] == tempCorr->entryPtr)
		{
			break;
		}
	}

	/* Store the response recvd in the asynchronous response to be sent to user */
	(request->cbData).resp[i].error = tempResp->status;

	/* Store the Vcc Address of the received response */
	(request->cbData).resp[i].u.VccAddr = oamMonitorInfo[i].monInfo.connection;

	/* Increment the count of responses in the async structure */
	(request->cbData).n_resp ++;


	/* Decrement the count of responses pending in the Request CB */
	(request->feInfo[0]).numWait --;
	request->numTransWait --;

	/* Check if the status received is Failure */
	if( tempResp->status != FPPI_SUCCESS)
		(request->feInfo[0]).numFail ++;

	/* Check if all responses have been received */
	if(request->numTransWait == 0) 
	{
		/* Send the response to the user */
		ifAtmOnCB_SendRespToUser(request);
	}
	
}




/*****************************************************************************
*    Name: ifAtmOnCB_OamAlarmSetFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous OAM_AlarmSet Req. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_OamAlarmSetFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	IfATM_FeTransInfo_t *tempCorr = NULL;
	IfATM_RequestCB_t *request = NULL;
	NPF_uint32_t i = 0;
	IfATM_OAM_Alarm_t *oamAlarmInfo = NULL;
	IfATM_FpResponse_t *tempResp = NULL;

	/* Check if the received parameters are proper */	
	if(ifAtmOnCB_checkParams
					(feId, correlator, context, responseData,(FPPI_CONTEXT)NPF_IF_ATM_OAM_ALARM_SET)
					!= NPF_SUCCESS)
		return;

	/* From the correlator get the request CB */
	tempCorr = (IfATM_FeTransInfo_t *)correlator;

	request = (IfATM_RequestCB_t *)tempCorr->reqCbPtr;

	/* Make the tempResp pointer point to the responseData */
	tempResp =  (IfATM_FpResponse_t *)responseData;

	/* Make the oamAlarmInfo pointer to point to entry member in request */
	oamAlarmInfo = (IfATM_OAM_Alarm_t *)(request->apiData).entry; 

	/* Using the request CB and Correlator get the entry index for which the
		response was received */
	for(i=0;i < (request->apiData).numEntries; i++)
	{
		if( &oamAlarmInfo[i] == tempCorr->entryPtr)
		{
			break;
		}
	}

	/* Store the response recvd in the asynchronous response to be sent to user */
	(request->cbData).resp[i].error = tempResp->status;

	/* Store the Vcc Address of the received response */
	(request->cbData).resp[i].u.VccAddr = oamAlarmInfo[i].oamAlrmInfo.connection;

	/* Increment the count of responses in the async structure */
	(request->cbData).n_resp ++;


	/* Decrement the count of responses pending in the Request CB */
	(request->feInfo[0]).numWait --;
	request->numTransWait --;

	/* Check if the status received is Failure */
	if( tempResp->status != FPPI_SUCCESS)
		(request->feInfo[0]).numFail ++;

	/* Check if all responses have been received */
	if(request->numTransWait == 0) 
	{
		/* Send the response to the user */
		ifAtmOnCB_SendRespToUser(request);
	}
	
}


/*****************************************************************************
*    Name: ifAtmOnCB_OamAlarmClearFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous OAM_AlarmClear Req. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_OamAlarmClearFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	IfATM_FeTransInfo_t *tempCorr = NULL;
	IfATM_RequestCB_t *request = NULL;
	NPF_uint32_t i = 0;
	IfATM_OAM_Alarm_t *oamAlarmInfo = NULL;
	IfATM_FpResponse_t *tempResp = NULL;

	/* Check if the received parameters are proper */	
	if(ifAtmOnCB_checkParams
					(feId, correlator, context, responseData,(FPPI_CONTEXT)NPF_IF_ATM_OAM_ALARM_CLEAR)
					!= NPF_SUCCESS)
		return;

	/* From the correlator get the request CB */
	tempCorr = (IfATM_FeTransInfo_t *)correlator;

	request = (IfATM_RequestCB_t *)tempCorr->reqCbPtr;

	/* Make the tempResp pointer point to the responseData */
	tempResp =  (IfATM_FpResponse_t *)responseData;

	/* Make the oamAlarmInfo pointer to point to entry member in request */
	oamAlarmInfo = (IfATM_OAM_Alarm_t *)(request->apiData).entry; 

	/* Using the request CB and Correlator get the entry index for which the
		response was received */
	for(i=0;i < (request->apiData).numEntries; i++)
	{
		if( &oamAlarmInfo[i] == tempCorr->entryPtr)
		{
			break;
		}
	}

	/* Store the response recvd in the asynchronous response to be sent to user */
	(request->cbData).resp[i].error = tempResp->status;

	/* Store the Vcc Address of the received response */
	(request->cbData).resp[i].u.VccAddr = oamAlarmInfo[i].oamAlrmInfo.connection;

	/* Increment the count of responses in the async structure */
	(request->cbData).n_resp ++;


	/* Decrement the count of responses pending in the Request CB */
	(request->feInfo[0]).numWait --;
	request->numTransWait --;

	/* Check if the status received is Failure */
	if( tempResp->status != FPPI_SUCCESS)
		(request->feInfo[0]).numFail ++;

	/* Check if all responses have been received */
	if(request->numTransWait == 0) 
	{
		/* Send the response to the user */
		ifAtmOnCB_SendRespToUser(request);
	}
	
}



/*****************************************************************************
*    Name: ifAtmOnCB_OamPmSetFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous OAM_PM_Set Req. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_OamPmSetFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	IfATM_FeTransInfo_t *tempCorr = NULL;
	IfATM_RequestCB_t *request = NULL;
	NPF_uint32_t i = 0;
	IfATM_OAM_PM_t *oamPmInfo = NULL;
	IfATM_FpResponse_t *tempResp = NULL;

	/* Check if the received parameters are proper */	
	if(ifAtmOnCB_checkParams
					(feId, correlator, context, responseData,(FPPI_CONTEXT)NPF_IF_ATM_OAM_PM_SET)
					!= NPF_SUCCESS)
		return;

	/* From the correlator get the request CB */
	tempCorr = (IfATM_FeTransInfo_t *)correlator;

	request = (IfATM_RequestCB_t *)tempCorr->reqCbPtr;

	/* Make the tempResp pointer point to the responseData */
	tempResp =  (IfATM_FpResponse_t *)responseData;

	/* Make the oamPmInfo pointer to point to entry member in request */
	oamPmInfo = (IfATM_OAM_PM_t *)(request->apiData).entry; 

	/* Using the request CB and Correlator get the entry index for which the
		response was received */
	for(i=0;i < (request->apiData).numEntries; i++)
	{
		if( &oamPmInfo[i] == tempCorr->entryPtr)
		{
			break;
		}
	}

	/* Store the response recvd in the asynchronous response to be sent to user */
	(request->cbData).resp[i].error = tempResp->status;

	/* Store the Vcc Address of the received response */
	(request->cbData).resp[i].u.VccAddr = oamPmInfo[i].pmInfo.connection;

	/* Increment the count of responses in the async structure */
	(request->cbData).n_resp ++;


	/* Decrement the count of responses pending in the Request CB */
	(request->feInfo[0]).numWait --;
	request->numTransWait --;

	/* Check if the status received is Failure */
	if( tempResp->status != FPPI_SUCCESS)
		(request->feInfo[0]).numFail ++;

	/* Check if all responses have been received */
	if(request->numTransWait == 0) 
	{
		/* Send the response to the user */
		ifAtmOnCB_SendRespToUser(request);
	}
	
}

/*****************************************************************************
*    Name: ifAtmOnCB_IfAttrSetFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous IfAttrSet Request. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_IfAttrSetFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	request_t *request = NULL;
	If_t *ifc = NULL;
	IfATM_FpResponse_t *tempResp = NULL;
	NPF_uint64_t	portSpeed = 0;
	NPF_uint32_t			numFe = 0;
	NPF_uint32_t	i = 0;
	pdk_FEPortList_t   *fePortList = NULL;
	NPF_PORT_ATTRIBS   portAttribs;


	/* Set the port Attributes struture to zero */
	memset(&portAttribs, 0, sizeof(NPF_PORT_ATTRIBS));

	/* Check if the correlator passed is NULL */
	if( correlator == NULL)
		return;

	/* Check if the context passed is correct */
	if((NPF_IfATM_CallbackType_t)context != 
				(NPF_IfATM_CallbackType_t)NPF_IF_ATTR_SET) 
	{
		/* Free the memory of the correlator */
		PIL_FreeMemory(correlator);

		return;
	}

	/* Get the request structure */
	request = ((IfATM_IfAtmAttrCorrelator_t *)correlator)->ifCorr->req;

	/* Get the ifc information from the correlator */
	ifc = ((IfATM_IfAtmAttrCorrelator_t *)correlator)->ifCorr->ifc;

	/* Get the port Speed from the correlator */
	portSpeed = ((IfATM_IfAtmAttrCorrelator_t *)correlator)->portSpeed;

	/* Set the port Speed in the namespace */
	if( pdk_ns_getFEPorts_from_IF(ifc->nsHandle,
						 &numFe,(pdk_FEPortList_t **)&fePortList) == NPF_SUCCESS)
	{
		/* Check if the Number of FE linked to this interface is 0 */
		if((numFe == 0) || (fePortList == NULL))
      {
      }
		else
		{
		
			/* Get the current Port Attributes */ 
			if( cm_port_get_attribs(fePortList->ports[0].port_hdl, &portAttribs) == NPF_SUCCESS)
			{
				/* Set the new Current Port Speed in Namespace */ 
				portAttribs.curTxRate = portSpeed;
				cm_port_set_attribs(fePortList->ports[0].port_hdl, &portAttribs);
			}

			/* Free the memory Allocated for fePorts member. This memory is allocated in 
				pdk_ns_getFEPorts_from_IF function */
			for(i=0;i<numFe;i++)
			{
				if(fePortList[i].ports)
					PIL_FreeMemory(fePortList[i].ports);
			}

			PIL_FreeMemory(fePortList);
		}
   }

	/* set the interface handle in the request structure */
	request->ihandle = ifc->nsHandle;

	/* Make the tempResp pointer point to the responseData */
	tempResp =  (IfATM_FpResponse_t *)responseData;

	/* Set the error code in the request structure according to the response Recvd */
	request->error = tempResp->status;

	/* Increment the resp count in the request structure */
	request->fpResp++;

	/* Call the ifManager function to handle the callback */
	onResponse((request_t *)request, NULL);

	/* Free the memory of the correlator */
	PIL_FreeMemory(correlator);

	return;
}



/*****************************************************************************
*    Name: ifAtmOnCB_IfDeleteFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous IfDelete Request. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_IfDeleteFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	request_t *request = NULL;
	IfATM_FpResponse_t *tempResp = NULL;
	If_t *ifc = NULL;

	/* Check if the correlator passed is NULL */
	if( correlator == NULL)
		return ;

	/* Check if the context passed is correct */
	if((NPF_IfATM_CallbackType_t)context != 
				(NPF_IfATM_CallbackType_t)NPF_IF_DELETE) 
	{
		/* Free the memory of the correlator */
		PIL_FreeMemory(correlator);

		return;
	}

	/* Get the request structure */
	request = ((IfCorrelator_t *)correlator)->req;

	/* Get the ifc information from the correlator */
	ifc = ((IfCorrelator_t *)correlator)->ifc;

	/* set the interface handle in the request structure */
	request->ihandle = ifc->nsHandle;


	/* Make the tempResp pointer point to the responseData */
	tempResp =  (IfATM_FpResponse_t *)responseData;

	/* Set the error code in the request structure according to the response Recvd */
	request->error = tempResp->status;

	/* Increment the resp count in the request structure */
	request->fpResp++;

	/* If Port Delete succedded than updated the ifmanager data structures */
	if(tempResp->status == NPF_NO_ERROR)
	{
		/* The following code is the same as in IfDelete function */
		PIL_EnterCriticalSection( &IM_LOCK );
		if ((npf_list_removeByKey(&IfList, &request->ihandle, (void **)&ifc)) == -1)
		{
			PIL_LeaveCriticalSection( &IM_LOCK );
			request->error = NPF_IF_E_INVALID_HANDLE;
			onResponse(request, NULL);
			PIL_LeaveCriticalSection( &IM_LOCK );

			/* Free the memory of the correlator */
			PIL_FreeMemory(correlator);

			return;
		}

		/* Delete ports list allocated during interface bind */
		npf_ns_close(ifc->nsHandle); /* Check for success */
		npf_ns_delete(ifc->path); /* Check for success */

		if(ifc->PORT.portsList != NULL && ifc->PORT.portsList->ports != NULL)
				PIL_FreeMemory(ifc->PORT.portsList->ports);
		if(ifc->PORT.portsList != NULL)
				PIL_FreeMemory(ifc->PORT.portsList);
		if(ifc != NULL)
				PIL_FreeMemory(ifc);

		PIL_LeaveCriticalSection( &IM_LOCK );

		request->error = NPF_NO_ERROR;
		onResponse(request, NULL);

	}
	else
	{
		request->error = NPF_IF_E_INVALID_HANDLE;

		/* Call the ifManager function to handle the callback */
		onResponse((request_t *)request, NULL);
	}

	/* Free the memory of the correlator */
	PIL_FreeMemory(correlator);

	return;
}


/*****************************************************************************
*    Name: ifAtmOnCB_IfEnableFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous IfEnable Request. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_IfEnableFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	request_t *request = NULL;
	IfATM_FpResponse_t *tempResp = NULL;
	If_t *ifc = NULL;

	/* Check if the correlator passed is NULL */
	if( correlator == NULL)
		return;

	/* Check if the context passed is correct */
	if((NPF_IfATM_CallbackType_t)context != 
				(NPF_IfATM_CallbackType_t)NPF_IF_ENABLE) 
	{
		/* Free the memory of the correlator */
		PIL_FreeMemory(correlator);

		return;
	}

	/* Make the tempResp pointer point to the responseData */
	tempResp =  (IfATM_FpResponse_t *)responseData;

	/* Get the request structure */
	request = ((IfCorrelator_t *)correlator)->req;

	/* Get the ifc information from the correlator */
	ifc = ((IfCorrelator_t *)correlator)->ifc;

	/* set the interface handle in the request structure */
	request->ihandle = ifc->nsHandle;


	/* Set the error code in the request structure according to the response Recvd */
	request->error = tempResp->status;

	/* Check if response received from FP is success than update the ifmanager data */
	if(tempResp->status == NPF_NO_ERROR)
		ifc->admState = NPF_IF_ADMIN_STATUS_UP;

	/* Increment the resp count in the request structure */
	request->fpResp++;

	/* Call the ifManager function to handle the callback */
	onResponse((request_t *)request, NULL);

	/* Free the memory of the correlator */
	PIL_FreeMemory(correlator);

	return;
}


/*****************************************************************************
*    Name: ifAtmOnCB_IfDisableFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous IfDisable Request. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_IfDisableFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	request_t *request = NULL;
	IfATM_FpResponse_t *tempResp = NULL;
	If_t *ifc = NULL;

	/* Check if the correlator passed is NULL */
	if( correlator == NULL)
		return ;

	/* Check if the context passed is correct */
	if((NPF_IfATM_CallbackType_t)context != 
				(NPF_IfATM_CallbackType_t)NPF_IF_DISABLE) 
	{
		/* Free the memory of the correlator */
		PIL_FreeMemory(correlator);

		return ;
	}
	/* Make the tempResp pointer point to the responseData */
	tempResp =  (IfATM_FpResponse_t *)responseData;

	/* Get the request structure */
	request = ((IfCorrelator_t *)correlator)->req;

	/* Get the ifc information from the correlator */
	ifc = ((IfCorrelator_t *)correlator)->ifc;

	/* set the interface handle in the request structure */
	request->ihandle = ifc->nsHandle;


	/* Set the error code in the request structure according to the response Recvd */
	request->error = tempResp->status;

	/* Check if response received from FP is success than update the ifmanager data */
	if(tempResp->status == NPF_NO_ERROR)
		ifc->admState = NPF_IF_ADMIN_STATUS_DOWN;

	/* Increment the resp count in the request structure */
	request->fpResp++;

	/* Call the ifManager function to handle the callback */
	onResponse((request_t *)request, NULL);

	/* Free the memory of the correlator */
	PIL_FreeMemory(correlator);

	return;
}

/*****************************************************************************
*    Name: ifAtmOnCB_IfGenStatsGetFunc 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			response from the Forwarding Plane for a previous IfGenStats Request. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_IfGenStatsGetFunc(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	request_t *request = NULL;
	IfATM_IfStatistics_t *resp = NULL;
	If_t *ifc = NULL;

	/* Check if the correlator passed is NULL */
	if( correlator == NULL)
		return;

	/* Check if the context passed is correct */
	if((NPF_IfATM_CallbackType_t)context != 
				(NPF_IfATM_CallbackType_t)NPF_IF_STATS_GET) 
	{
		/* Free the memory of the correlator */
		PIL_FreeMemory(correlator);

		return;
	}

	/* Get the request structure */
	request = ((IfCorrelator_t *)correlator)->req;

	/* Get the ifc information from the correlator */
	ifc = ((IfCorrelator_t *)correlator)->ifc;

	/* set the interface handle in the request structure */
	request->ihandle = ifc->nsHandle;

	resp = (IfATM_IfStatistics_t *)responseData;

	/* Set the error code in the request structure according to the response Recvd */
	request->error = (resp->response).status;

	/* Increment the resp count in the request structure */
	request->fpResp++;


	/* Call the ifManager function to handle the callback */
	onResponse((request_t *)request, &(resp->stats));

	/* Free the memory of the correlator */
	PIL_FreeMemory(correlator);

	return;
}



#endif /* _INCLUDE_ATM */
