/*****************************************************************************
 *                  INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 *$Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/APIImplementations/DiffServ/inc/ds.h,v $
 *$Revision: 1.5.2.1 $
 *$Date: 2004/03/17 09:51:56 $
 * Description:
 *     This file contains the global data/function used by DiffServ component.
 *
 *****************************************************************************/

#ifndef __DS_H__
#define __DS_H__

#include "log.h"
#include "list.h"
#include "npf_common.h"
#include "pdkmgr.h"
#include "fpp_common.h"
#include "npf_ds.h"

#define DS_MAX_REQS	64

/*trace messages, function entered/exited*/
#define DS_TRACE(format, args...)  npf_logger_write(LOG_DS, LOG_TRACE, format, ##args)
/*informational message, printf debugging*/
#define DS_INFO(format, args...)   npf_logger_write(LOG_DS, LOG_INFO, format, ##args)
/*error message, not critical, recoverable error*/
#define DS_ERROR(format, args...)  npf_logger_write(LOG_DS, LOG_ERR, format, ##args)
/*ciritcal error, unrecoverable*/
#define DS_CRITICAL(format, args...) npf_logger_write(LOG_DS, LOG_CRIT, format, ##args)

#define DS_FUNCTION() DS_TRACE("%s", "\n=>\n")

/*
 * User information storage data structure
 */
typedef struct DSuserInfo
{
    NPF_userContext_t        context;   /* user context */
    NPF_DS_CallbackFunc_t   callback;  /* callback function */
    NPF_callbackHandle_t     handle;    /* callback handle */
} DSuserInfo_t;

/*
 * DPE information storage data structure
 */
typedef struct DS_DpeInfo
{
    NPF_DS_DpeType_t	     type;	  /* DPE type */
    NPF_DS_DpeHandle_t       dpeHandle;   /* DPE handle */
    uint32_t	     	     fec;	  /* FE Count */
    FPPI_FEID		     feid[MAXFENUM];
    FPPI_PortID	     	     port;
    NPF_DS_DpeDirection_t    dir;	/* DPE direction INGRESS/EGRESS */
} DS_DpeInfo_t;

typedef struct DS_ReqResp
{
    FPPI_FEID	    id;		/* FE id */
    uint32_t	    used;	/* tells this request sent to this FE or not */
    uint32_t	    reqs;	/* No. Of APIs called for this FE */
    uint32_t	    resps;	/* No. Of API responses received for this FE */
    uint32_t	    done;	/* FE responded for all*/
    uint32_t	    error;	/* Error reported */
    NPF_DS_AsyncResponse_t *async[DS_MAX_REQS];  /* Array of async responses */
}DS_ReqResp_t;

/*
 * Request Information structure to support array of inputs
 */
typedef struct DiffServRequest
{
    NPF_DS_CallbackType_t  type;       /* Callback type */
    NPF_callbackHandle_t    cbh;        /* Callback handle */
    NPF_userContext_t       context;    /* User context */
    NPF_correlator_t        corr;       /* User correlator */
    NPF_errorReporting_t    report;     /* user error report */
    uint32_t                no;         /* No. of requests generated */
    uint32_t		    fec;	/* Total no of FEs request sent */
    DS_ReqResp_t	    *reqResp;
    NPF_DS_CallbackData_t  data;       /* User callback data */
    NPF_DS_CallbackFunc_t  callback;   /* User registered callback function */
    NPF_error_t             error;      /* Error code */
    uint16_t                feResp;      /* This flag tells response for callback from FE or internal */
    uint32_t                id;         /* Last FE id this request Sent */
    void 		    *temp;
} DiffServRequest_t;

typedef struct DiffServCorrelator
{
    DiffServRequest_t *req;
}DiffServCorrelator_t;

typedef enum DiffServDpeState
{
    DPE_STATE_IDLE,
    DPE_STATE_INUSE,
    DPE_STATE_UNKNOWN
}Dpestate_t;

extern NPF_error_t QOSRegister(NPF_userContext_t      context,
                    NPF_DS_CallbackFunc_t   callback,
                    NPF_callbackHandle_t   *cbhandle);

extern NPF_error_t QOSDeregister(NPF_callbackHandle_t   cbhandle);

extern NPF_error_t DpeCreate(NPF_IfHandle_t interface, NPF_DS_DpeDirection_t dir, NPF_DS_Dpe_t *dpe, DiffServRequest_t *req);
extern NPF_error_t DpeAddAssociation(NPF_DS_DpeSetHandle_t *dpeSet, DiffServRequest_t *req);
extern NPF_error_t DpeDelAssociation(NPF_DS_DpeSetHandle_t *dpeSet, DiffServRequest_t *req);
extern NPF_error_t DpeModify(NPF_DS_DpeHandle_t dpeHandle, NPF_DS_Dpe_t *dpe, DiffServRequest_t *req);
extern NPF_error_t DpeQuery(NPF_DS_DpeHandle_t dpeHandle, DiffServRequest_t *req);
extern NPF_error_t DpeStatistics(NPF_DS_DpeHandle_t dpeHandle, DiffServRequest_t *req);
extern NPF_error_t DpeDelete(NPF_DS_DpeHandle_t dpeHandle, DiffServRequest_t *req);

extern void dsCallback(DiffServRequest_t *resp);
extern NPF_DS_DpeType_t getDpeType(NPF_DS_DpeHandle_t dpeHandle);
extern NPF_error_t newQosRequest(NPF_uint32_t no,
                        NPF_IfCallbackType_t type,
                        NPF_callbackHandle_t cbh,
                        NPF_correlator_t     corr,
                        NPF_errorReporting_t report,
                        DiffServRequest_t            **req);

extern void dpeOperationCallback(
        FPPI_FEID feid,
        FPPI_CORRELATOR fpcorrelator,
        FPPI_CONTEXT fpcontext,
        void *response );

void DsFPPAPIcall(DS_DpeInfo_t *info,
                  FPPI_DS_Dpe_t *elem,
		  NPF_DS_CallbackType_t op,
                  void          *corr,
                  DiffServRequest_t *req);
#endif
