/*****************************************************************************
 *                  INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 *$Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/APIImplementations/IM/inc/if.h,v $
 *$Revision: 1.6 $
 *$Date: 2003/11/17 16:57:10 $
 * Description:
 *     This file contains the global data/function used by nterface management.
 *
 *****************************************************************************/

#ifndef __IF_H__
#define __IF_H__

#include "log.h"
#include "list.h"
#include "npf_common.h"
#include "pdkmgr.h"
#include "fpp_common.h"

/*trace messages, function entered/exited*/
/*trace messages, function entered/exited*/
#define IF_TRACE(format, args...)	npf_logger_write(LOG_IF, LOG_TRACE, format, ##args)
//#define IF_TRACE(format, args...)	printf(format, ##args); npf_logger_write(LOG_IF, LOG_TRACE, format, ##args)
/*informational message, printf debugging*/
#define IF_INFO(format, args...)	npf_logger_write(LOG_IF, LOG_INFO, format, ##args)
/*error message, not critical, recoverable error*/
#define IF_ERROR(format, args...)	npf_logger_write(LOG_IF, LOG_ERR, format, ##args)
/*ciritcal error, unrecoverable*/
#define IF_CRITICAL(format, args...) npf_logger_write(LOG_IF, LOG_CRIT, format, ##args)

#define IF_FUNCTION() IF_TRACE("%s", "=>\n")

#define IF_MAX_NO		10
#define MAX_INTERFACE_LEN	100
#define IF_MAX_ASSOCS		10

#define INTERFACE_ROOT		"/NPFSys/0/Iface"
#define INTERFACE_PORT_ROOT	"/NPFSys/0/IfPort"

/*
* Interface Structure
*/

typedef struct
{
    uint32_t		no; /* Iface no, used to find logical IfPort no and associate */
    char		path[MAX_INTERFACE_LEN]; /* If "name space" path */
    NPF_NSHDL		nsHandle;   /* interface handle, (data handle  for interface node) */
    NPF_NSHDL		inode;   /* "name space" refference to node of type NPF_INTERFACE */
    uint32_t	        count;      /* No of associations with FE Port nodes */
    NPF_NSHDL		list[IF_MAX_ASSOCS];     /* Associations with FE Port nodes */
    NPF_IfGeneric_t	npfIf;	    /* NPF The Interface structure */
    NPF_IfOperStatus_t  operStat; /* Operational status: NPF_IF_OPER_STATUS_UP/DOWN/UNKNOWN */
    NPF_IfAdminStatus_t admState; /* Administrative status: NPF_IF_ADMIN_STATUS_UP/NPF_IF_ADMIN_STATUS_DOWN/UNKNOWN */
    NPF_IfStatistics_t  stats;	/* Interface statistics */
    void                *l2If;  /* L2 Interface to which L3 interface binded, NULL for L2 If */
    void                *l3If;  /* L3 Interface to which L2 interface binded, NULL for L3 If */
    struct{
	uint32_t  	count;  /* Total no. of FEs */
	npf_L3IntfAttrs_t l3Attribs [MAXFENUM][MAXIFOWNPORT]; /* L3 Attribute List */
	npf_L2IntfAttrs_t l2Attribs [MAXFENUM][MAXIFOWNPORT]; /* L2 Attribute List */
    }FE;
    struct{
	uint32_t  	count;  /* no of port IDs */
	FPPI_PortID	ports[MAXIFOWNPORT]; /* array of port IDs */
	pdk_FEPortList_t *portsList; /* List of FE/PORT for this Interface */
    }PORT;
} If_t;

typedef struct
{
    FPPI_FEID		    id; /* FE id */
    NPF_boolean_t	    ok; /* Set to TRUE when req = resp */
    uint32_t		    req; /* No of requests for this FE id */
    uint32_t		    resp; /* No of responses for this FE id */
}feReq_t;

/*
 * Request Information structure to support array of inputs
 */
typedef struct
{
    NPF_IfCallbackType_t    type;	/* Callback type */
    NPF_callbackHandle_t    cbh;	/* Callback handle */
    NPF_userContext_t	    context;    /* User context */
    NPF_correlator_t        corr;	/* User correlator */
    NPF_errorReporting_t    report;	/* user error report */
    uint32_t	    	    no;		/* No. of requests generated */
    uint32_t	    	    fpReq;	/* No of FP API request to be made */
    uint32_t	    	    fpResp;	/* No of FP response generated */
    uint32_t	    	    resp;	/* No. Responses generated for this request */
    NPF_IfCallbackData_t    data;	/* User callback data */
    NPF_IfCallbackFunc_t    callback;   /* User registered callback function */
    NPF_IfHandle_t	    ihandle;	/* interface handle */
    NPF_error_t		    error;	/* Error code */
    uint32_t	    	    id;		/* Last FE id this request Sent */
    void		    *asyncResp[IF_MAX_NO];
    NPF_boolean_t  	    ok;     /* Set to TRUE if all FE of this interface responded */
    feReq_t		    fe[MAXFENUM]; /* Request per FE */
} request_t;

/*
 * IF Correlator data structure
 */
typedef struct
{
    If_t        	*ifc;   /* Interface information */
    request_t   	*req;  /* request information */
    NPF_IfHandle_t      ifHandle; /* Interface handle */
} IfCorrelator_t;


NPF_error_t initIfLog(char *file);

NPF_error_t IfRegister(NPF_userContext_t      context,
                    NPF_IfCallbackFunc_t   callback,
                    NPF_callbackHandle_t   *cbhandle);

NPF_error_t IfDeregister(NPF_callbackHandle_t   cbhandle);

NPF_error_t IfEventRegister(NPF_userContext_t    userContext,
			     NPF_IfEventHandlerFunc_t  ifEventHandlerFunc,
			     NPF_IfEventHandlerHandle_t *ifEventHandlerHandle);

NPF_error_t IfEventDeregister(NPF_IfEventHandlerHandle_t cbhandle);

NPF_error_t IfCreate(NPF_IfType_t type, request_t *req);

NPF_error_t IfDelete(NPF_IfHandle_t ifHandle, request_t *req);

NPF_error_t IfBind(NPF_IfBinding_t   *ifBind, request_t *request);

NPF_error_t IfGenericStatsGet(NPF_IfHandle_t   ifHandle, request_t *req);

NPF_error_t IfEnable(NPF_IfHandle_t   ifHandle, request_t *request);

NPF_error_t IfDisable(NPF_IfHandle_t   ifHandle, request_t *request);

NPF_RET IfAttrSet(NPF_IfHandle_t ifHandle, NPF_IfGeneric_t *iface, request_t *req);

NPF_RET IfCreateAndSet(NPF_IfGeneric_t *iface, request_t *req);


NPF_error_t IfPromiscSet(NPF_IfHandle_t ifHandle, request_t *req);

NPF_error_t IfPromiscClear(NPF_IfHandle_t ifHandle, request_t *req);

NPF_error_t IfMTUSet(NPF_IfHandle_t ifHandle, NPF_uint16_t mtu, request_t *req);

NPF_error_t IfIPv4FibSet(NPF_IfHandle_t ifHandle, NPF_IPv4UC_FwdTableHandle_t fib_npf_hndl, request_t *req);

NPF_error_t IfSpeedSet(NPF_IfHandle_t ifHandle, 
			NPF_IfLAN_Speed_t speed, 
			request_t *req);

NPF_error_t IfAddrSet(NPF_IfHandle_t ifHandle,
                        NPF_IPv4NetAddr_t *addr,
                        request_t *req);

NPF_error_t IfIPv6AddrAdd(NPF_IfHandle_t ifHandle,
                        NPF_IPv6NetAddr_t *addr,
                        request_t *req);

NPF_error_t IfIPv6AddrDelete(NPF_IfHandle_t ifHandle,
                        NPF_IPv6NetAddr_t *addr,
                        request_t *req);

NPF_error_t IfOperStatusGet(NPF_IfHandle_t   ifHandle, request_t *request);

NPF_error_t newRequest(NPF_uint32_t no, 
			NPF_IfCallbackType_t type, 
			NPF_callbackHandle_t cbh,
			NPF_correlator_t     corr, 
			NPF_errorReporting_t report,
			request_t	     **req);

extern void ifAtmOnCB_AtmEvent(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData);
#endif
