/*****************************************************************************
 *                  INTEL CORPORATION PROPRIETARY IF_INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 *$Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/APIImplementations/IM/src/if.c,v $
 *$Revision: 1.7 $
 *$Date: 2003/11/17 16:57:10 $
 * Description:
 *     This file contains the interface management component implementation.
 *
 *****************************************************************************/

#include <assert.h>
#include <string.h>

#include "list.h"
#include "npf_common.h"
#include "pil.h"
#include "log.h"
#include "pdkmgr.h"
#include "fpp_common.h"
#include "pdk_ns_helper.h"
#include "npf_ipv4_types.h"
#include "npf_if.h"
#include "if.h"

#ifdef _INCLUDE_ATM
#include "npf_common.h"
#include "npf_cm_types.h"
#include "npf_atm_types.h"
#include "fppapi_atm_types.h"
#include "pdk_callback.h"
#include "if_atm.h"
#endif /* _INCLUDE_ATM */

/*--------------------------------------------------------------------*
 * I N T E R F A C E  M A N A G E M E N T  G L O B A L  D E F I N E S
 *--------------------------------------------------------------------*/

static DList		UserList;
DList		EventCallbacks;
static DList		RequestList;
DList		IfList;

static uint32_t IfInit = FALSE;

/*
 * Global lock for entire If component
*/
PilCriticalSection IM_LOCK;

#define IF_LOCK()\
	 PIL_EnterCriticalSection( &IM_LOCK )
#define IF_UNLOCK() \
	PIL_LeaveCriticalSection( &IM_LOCK )

static uint32_t interfaceCount = 0; /* No. of Interfaces created */
static char     interfaceRoot[256];

/*
 * FP register callbacke handles
 */
static FPPI_CBHANDLE  ifDelL2PropertiesCallbackHandle;
static FPPI_CBHANDLE  ifDelL3PropertiesCallbackHandle;

#ifdef _INCLUDE_IPV6
static FPPI_CBHANDLE  ifDelL3PropertiesIPv6CallbackHandle;
static FPPI_CBHANDLE  ifFPsetIPaddrV6CallbackHandle;
#endif

static FPPI_CBHANDLE  ifFPsetIPaddrCallbackHandle;
static FPPI_CBHANDLE  ifFPsetIPfwdCallbackHandle;

static FPPI_CBHANDLE  ifFPsetLANSpeedCallbackHandle;
static FPPI_CBHANDLE  ifFPsetMTUCallbackHandle;

static FPPI_CBHANDLE  ifFPgetL3StatsCallbackHandle;
static FPPI_CBHANDLE  ifFPgetL2StatsCallbackHandle;

static FPPI_CBHANDLE  ifFPsetStatusCallbackHandle;

static NPF_USERCONTEXT   ifEventContext;
static NPF_USERCONTEXT   portEventContext;

static FPPI_CBHANDLE  interfaceEventHandle;
static FPPI_CBHANDLE  portEventHandle;

extern int cm_netDataByHandle(NPF_HANDLE handle, NPF_DATA *ppdata);
static int ifSetFlag( NPF_HANDLE ifh, NPF_IF_STATUS flag, int *no, NPF_HANDLE *ifList);

NPF_error_t onResponse(request_t *request, void *resp);
NPF_error_t getInterface(NPF_IfHandle_t ifHandle, If_t  **iface);

static NPF_error_t setL3Attrib(request_t *req, If_t *ifc, NPF_L3ATTRS_TYPE type, void *value);
static NPF_error_t setL2Attrib(request_t *req, If_t *ifc, NPF_L2ATTRS_TYPE type, void *value);
NPF_error_t getIfacePath(char *path);
static NPF_error_t getStats(request_t *req, If_t *ifc);
void IfEvent(NPF_IfEventData_t *event);
void IfPdkEvent(NPF_HANDLE ifh, NPF_EVENT event);
NPF_error_t invokeCallback(request_t *response);
NPF_error_t createIfRoot();
extern int cm_copyFromPDK(NPF_HANDLE handle, NPF_DATA data);
extern NPF_RET pdk_ns_getNSRootHandle( NPF_NSHDL *root_ns_hdl);
extern NPF_error_t IfGetFEid(NPF_IfHandle_t ifHandle, uint32_t *no, FPPI_FEID *feId);

/* Forward Decl */
NPF_error_t IfGetFEIdsFrmFIBHdl(NPF_HANDLE fib_npf_hdl, uint32_t *numFe, FPPI_FEID *feId);
/*
 * User information storage data structure
 */
typedef struct
{
    NPF_userContext_t        context;   /* user context */
    NPF_IfCallbackFunc_t     callback;  /* callback function */
    NPF_callbackHandle_t     handle;    /* callback handle */
} userInfo_t;

/*
 * Event Callback information storage data structure
 */
typedef struct
{
    NPF_userContext_t        context;   /* user context */
    NPF_IfEventHandlerFunc_t    callback;  /* callback function */
    NPF_IfEventHandlerHandle_t     handle;    /* callback handle */
} eventCallbackInfo_t;

/*
 * Callback information storage data structure
 */
typedef struct
{
    NPF_IfCallbackFunc_t   callback;  /* callback function */
    NPF_userContext_t        context;   /* user context */
    NPF_callbackHandle_t     cbhandle;  /* callback handle */
} callbackInfo_t;


/****************************************************************************
 * Function: IfRegister()                                          	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		context: User context to register                           *
 *		callback: User callback function to register                *
 *		cbhandle: Callback handle to be returned		    *
 *  							    		    *
 * Description: 							    *
 *             	This function registers application with specified context  *
 *		and callback function. New callback handle is generated and *
 *		which will be set in cbhandle. Check is made to see user    *
 *		already registered with the same cbhandle.		    *
 * Return: 								    *
 *		NPF_IF_E_BAD_CALLBACK_IF_FUNCTION - callback is NULL	    *
 *		NPF_IF_E_ALREADY_REGISTERED - already registered with 	    *
 *		NPF_E_UNKNOWN - Other errors				    *
 ****************************************************************************/
NPF_error_t IfRegister(NPF_userContext_t      context,
            	    NPF_IfCallbackFunc_t   callback,
            	    NPF_callbackHandle_t   *cbhandle)
{
    NPF_callbackHandle_t tempCbhandle;
    userInfo_t		 *user;
    IF_FUNCTION();
    IF_LOCK();
    if(callback == NULL)
    {
	IF_ERROR("***** %s", "callback == NULL, returning NPF_IF_E_BAD_CALLBACK_IF_FUNCTION");
        IF_UNLOCK();
	return(NPF_IF_E_BAD_CALLBACK_FUNCTION);
    }
    user = (userInfo_t *)PIL_AllocateMemory(sizeof(userInfo_t),PIL_MEM_ZERO_INIT);
    if(user == NULL)
    {
        IF_UNLOCK();
	IF_CRITICAL("***** %s", "user == NULL");
	return NPF_E_UNKNOWN;
    }
    user->context = context;
    user->callback = callback;

    if(npf_list_searchByKey(&UserList, user) != NULL)
    {
        IF_UNLOCK();
	PIL_FreeMemory(user);
	IF_ERROR("***** %s", "NPF_IF_E_ALREADY_REGISTERED");
	return (NPF_IF_E_ALREADY_REGISTERED);
    }

    if (pdk_callback_register (NPF_CAT_IF,
                               (NPF_USERCONTEXT)context,
                               (NPF_GENERAL_CBFUNC)callback,
                               (NPF_CBHANDLE *)&tempCbhandle) != NPF_SUCCESS)
    {
        IF_UNLOCK();
	PIL_FreeMemory(user);
	IF_ERROR("***** %s", "pdk_callback_register result != NPF_SUCCESS");
        return(NPF_E_UNKNOWN);
    }

    *cbhandle = tempCbhandle;
    IF_INFO("cbhandle 	=  %d\n", *cbhandle);
    user->handle = tempCbhandle;
    npf_list_pushBack(&UserList, user);

    IF_UNLOCK();
    return(NPF_NO_ERROR);
}

/****************************************************************************
 * Function: IfDeregister()                                          	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		cbhandle: Callback handle to deregister			    *
 * Description: 							    *
 *             	This function deregisters application with CPPDK.	    *
 *             	Any requests after this is invoked, are not accepted	    *
 * Return: 								    *
 *		NPF_E_BAD_CALLBACK_HANDLE - invalid callback handle	    *
 *		NPF_E_UNKNOWN - Other errors				    *
 ****************************************************************************/
NPF_error_t IfDeregister(NPF_callbackHandle_t   cbhandle)
{
    userInfo_t           key;
    key.context = 0;
    key.callback = NULL;
    key.handle = cbhandle;

    IF_FUNCTION();
    IF_LOCK();

    if ((npf_list_removeByKeyFree(&UserList, &key)) == -1)
    {
	IF_UNLOCK();
	return NPF_E_BAD_CALLBACK_HANDLE;
    }

    if (pdk_callback_deregister((NPF_CBHANDLE)cbhandle) != NPF_SUCCESS)
    {
        IF_UNLOCK();
        return(NPF_E_UNKNOWN);
    }
    IF_UNLOCK();

    return (NPF_NO_ERROR);
}

/****************************************************************************
 * Function: IfEventRegister()                                              *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		userContext: User context to register                       *
 *		ifEventHandlerFunc: User callback function to register      *
 *		ifEventHandlerHandle: Callback handle to be returned	    *
 *  							    		    *
 * Description: 							    *
 *             	This function registers application with specified context  *
 *		and event callback function. New event callback handle is   *
 *		generated and which will be set in cbhandle. 		    *
 *		Check is made to see user already registered with the same  *
 *		cbhandle.						    *
 * Return: 								    *
 *		NPF_IF_E_BAD_CALLBACK_IF_FUNCTION - callback is NULL	    *
 *		NPF_IF_E_ALREADY_REGISTERED - already registered with 	    *
 *		NPF_E_UNKNOWN - Other errors				    *
 ****************************************************************************/
NPF_error_t IfEventRegister(NPF_userContext_t    userContext,
                             NPF_IfEventHandlerFunc_t  ifEventHandlerFunc,
                             NPF_IfEventHandlerHandle_t *ifEventHandlerHandle)
{
    NPF_IfEventHandlerHandle_t	tmpHandle;
    eventCallbackInfo_t		*eventInfo;
    IF_LOCK();
    IF_FUNCTION();
    eventInfo = (eventCallbackInfo_t *)PIL_AllocateMemory(
					sizeof(eventCallbackInfo_t),PIL_MEM_ZERO_INIT);
    if(eventInfo == NULL)
	return(NPF_E_UNKNOWN);
    eventInfo->context = userContext;
    eventInfo->callback = ifEventHandlerFunc;
    eventInfo->handle = tmpHandle;
    if(npf_list_searchByKey(&EventCallbacks, eventInfo) != NULL)
    {
        IF_UNLOCK();
	IF_ERROR("*** NPF_E_CALLBACK_ALREADY_REGISTERED ***");
	return (NPF_E_CALLBACK_ALREADY_REGISTERED);
    }
    npf_list_pushBack(&EventCallbacks, eventInfo);
    IF_TRACE("ifEventHandlerHandle = %d", tmpHandle);
    *ifEventHandlerHandle = tmpHandle;
    IF_UNLOCK();
    return(NPF_NO_ERROR);
}

/****************************************************************************
 * Function: IfEventDeregister()                                            *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		cbhandle: Callback handle to deregister			    *
 * Description: 							    *
 *             	This function deregisters application with CPPDK.	    *
 *             	Any requests after this is invoked, are not accepted	    *
 * Return: 								    *
 *		NPF_E_BAD_CALLBACK_HANDLE - invalid callback handle	    *
 *		NPF_E_UNKNOWN - Other errors				    *
 ****************************************************************************/
NPF_error_t IfEventDeregister(NPF_callbackHandle_t   cbhandle)
{
    eventCallbackInfo_t           key;
    key.context = 0;
    key.callback = NULL;
    key.handle = cbhandle;
    IF_FUNCTION();
    IF_LOCK();
    if ((npf_list_removeByKeyFree(&EventCallbacks, &key)) == -1)
    {
	IF_UNLOCK();
	return NPF_E_BAD_CALLBACK_HANDLE;
    }
    IF_UNLOCK();
    return (NPF_NO_ERROR);
}

/****************************************************************************
 * Function: newIfCorrelator()                                              *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		if: Interface details					    *
 *		req: request details					    *
 * Description: 							    *
 *             	This function creates new correlator to send FPPI request   *
 * Return: 								    *
 *		NULL -on Error						    *
 *		FPPI_CORRELATOR -on success				    *
 ****************************************************************************/
FPPI_CORRELATOR newIfCorrelator(If_t  *ifc, request_t *req)
{
    IfCorrelator_t	*tmp;
    tmp = (IfCorrelator_t *)PIL_AllocateMemory(sizeof(IfCorrelator_t), PIL_MEM_ZERO_INIT);
    if(tmp == NULL) return 0;
    tmp->ifc = ifc;
    tmp->req = req;
    tmp->ifHandle = ifc->nsHandle;
    return((FPPI_CORRELATOR)tmp);
}
/****************************************************************************
 * Function: ifSetFlag()                                          	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		ifh: Interface handle					    *
 *		flag: Status flag UP/DOWN				    *
 *		no: No of interfaces in ifList				    *
 *		ifList: Interface list array				    *
 *  							    		    *
 * Description: 							    *
 *             	This function set the up or down flag of interface and all  *
 *		the assoc. ports checking all the interface which associates*
 *		with those ports too. output all the interfaces handle that *
 *		needs to be trigger up or down event. input ifh is open,    *
 *		ifList[] return in open status				    *
 * Return: 								    *
 *		NPF_IF_E_BAD_CALLBACK_IF_FUNCTION - callback is NULL	    *
 *		NPF_IF_E_ALREADY_REGISTERED - already registered with 	    *
 *		NPF_E_UNKNOWN - Other errors				    *
 ****************************************************************************/
int ifSetFlag( NPF_HANDLE ifh, NPF_IF_STATUS flag, int *no, NPF_HANDLE *ifList)
{
    int             i, j, k, l, portlinkup;
    int             num_ports, num_ifs, tmp_numports;
    NPF_HANDLE      porthlist[MAXIFOWNPORT], ifhlist[MAXIFOWNPORT];
    NPF_HANDLE      tmp_porthlist[MAXIFOWNPORT];
    PDK_PORT_t      *port, *tmpport;
    PDK_INTERFACE_t *interface, *tmpif;

    IF_FUNCTION();

    *no = 0;
    if (cm_getDataByHandle(ifh, (NPF_DATA *)&interface) != NPF_SUCCESS)
        return -1;
    /* set all the port's status, the FE's port should be set already */
    if (cm_getPortByIF(ifh, &num_ports, porthlist) != NPF_SUCCESS)
        return -1;
    /* set the target interface status and push it to the iflist */
    /* If set it up but the linkstatus is down, don't fire IF UP event
       If one of the linkstatus is on, then following algorithm will push
       this IF into the ifList
       interface->status = flag;
       NOTE: open this ifh
       ifList[(*no)++] = ifh; */
    if (flag == NPF_IF_DOWN) {
        for (i=0; i<num_ports; cm_handle_close(porthlist[i++])) {
            cm_getDataByHandle(porthlist[i], (NPF_DATA *)&port);
            if (port->attribs.status == 0)
                /* if this port was admin off before, then skip it */
                continue;
            /* this port was admin on, set if off */
            port->attribs.status = 0;
            /* get the assoc. interfaces and if it is on */
            cm_getIFByPort(porthlist[i], &num_ifs, ifhlist);
            for (j=0; j<num_ifs; j++) {
              cm_getDataByHandle(ifhlist[j], (NPF_DATA *)&tmpif);
                if (tmpif->status == NPF_IF_DOWN) {
                    cm_handle_close(ifhlist[j]);
                    continue;
                }
                /* the associated if was ON */
                /* check if all the assoc. ports are down */
                cm_getPortByIF(ifhlist[j], &tmp_numports, tmp_porthlist);
                portlinkup = 0;
                for (k=0; k<tmp_numports; cm_handle_close(tmp_porthlist[k++])) {
                    cm_getDataByHandle
                        (tmp_porthlist[k], (NPF_DATA *)&tmpport);
                    if (tmpport->attribs.status     == 1 &&
                        tmpport->attribs.linkstatus == 1) {
                        portlinkup = 1;
                        for (l=k; l<tmp_numports; l++)
                            cm_handle_close(tmp_porthlist[l]);
                        break;
                    }
                }
                /* if none of the ports is admin+link on, set the if off */
                if (!portlinkup) {
                    tmpif->status = NPF_IF_DOWN;
                    ifList[(*no)++] = ifhlist[j];
                } else {
                    /* only close the one doesn't need to be set */
                    cm_handle_close(ifhlist[j]);
                }
            } /* done check all the interfaces associ. to this port */
        } /* done check all the ports associates to this if */
    } else if (flag == NPF_IF_UP) {
        for (i=0; i<num_ports; cm_handle_close(porthlist[i++])) {
            cm_getDataByHandle(porthlist[i], (NPF_DATA *)&port);
            if (port->attribs.status == 1)
                /* if this port was admin on before, then skip it */
                continue;
            /* this port was admin off before, make it up */
            port->attribs.status = 1;
            /* get the assoc. interfaces and if it is on */
            cm_getIFByPort(porthlist[i], &num_ifs, ifhlist);
            for (j=0; j<num_ifs; j++) {
                cm_getDataByHandle(ifhlist[j], (NPF_DATA *)&tmpif);
                if (tmpif->type != NPF_INTERFACE)
                {
                    cm_handle_close(ifhlist[j]);
                    continue;
                }
                /* if that interface was on, just skil it */
                if (tmpif->status == NPF_IF_UP) {
                    cm_handle_close(ifhlist[j]);
                    continue;
                }
                /* if that interface was off, check if needs turn it on
                 * check if any of the assoc. ports is admin + link up
                 * and the interface must have ipaddr */
                if (tmpif->ipattribs.addr_u.ipaddr  == 0 &&
                    tmpif->ipattribs.mask_u.submask == 0)
                    continue;
                cm_getPortByIF(ifhlist[j], &tmp_numports, tmp_porthlist);
                portlinkup = 0;
                for (k=0; k<tmp_numports; cm_handle_close(tmp_porthlist[k++])) {
                    cm_getDataByHandle
                        (tmp_porthlist[k], (NPF_DATA *)&tmpport);
                    if (tmpport->attribs.status     == 1 &&
                        tmpport->attribs.linkstatus == 1) {
                        portlinkup = 1;
                        for (l=k; l<tmp_numports; l++)
                            cm_handle_close(tmp_porthlist[l]);
                        break;
                    }
                }
                /* if any of the ports is admin+link on, set the if on */
                if (portlinkup) {
                    tmpif->status = NPF_IF_UP;
                    ifList[(*no)++] = ifhlist[j];
                } else {
                    /* only close the one doesn't need to be set */
                    cm_handle_close(ifhlist[j]);
                }
            } /* done check all the interfaces associ. to this port */
        } /* done check all the ports associates to this if */
    } else
        /* not set UP, not set DOWN, what do you do? */
        assert(0);

    return 1;
}

/****************************************************************************
 * Function: newRequest()                                            	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		no - nof of interface operations to perform		    *
 *		type - Callback type					    *
 *		corr - User specified correlator			    *
 *		report - Error reporting, NPF_REPORT_ALL/NONE/IF_ERROR	    *
 *		req - request_t data, maintains application request state   * 
 *  							    		    *
 * Description: 							    *
 *             	This function creates request_t data in which appl. request *
 *             	state is maintained.					    *
 * Return: 								    *
 *		NPF_INVALID_PARAMETERS - Invalid parameter		    *
 *		NPF_IF_E_BAD_CALLBACK_IF_FUNCTION - no callback registered	    *
 *		NPF_NO_ERROR  - On no error				    *
 *		NPF_E_UNKNOWN  - On other errors			    *
 ****************************************************************************/
NPF_error_t newRequest(NPF_uint32_t no, 
			NPF_IfCallbackType_t type, 
			NPF_callbackHandle_t cbh,
			NPF_correlator_t     corr,
			NPF_errorReporting_t report,
			request_t            **req)
{
    request_t 		*request;
    IF_FUNCTION();
    IF_LOCK();
    if(no < 1)
    {
        IF_UNLOCK();
	IF_ERROR("Requests ( %d ) > %d ***", no, IF_MAX_NO);
	return(NPF_E_UNKNOWN);
    }
    if(no > IF_MAX_NO)
    {
        IF_UNLOCK();
	IF_ERROR("Requests ( %d ) > %d ***", no, IF_MAX_NO);
	return(NPF_E_UNKNOWN);
    }
    request = (request_t *)PIL_AllocateMemory(sizeof(request_t),PIL_MEM_ZERO_INIT);
    if(request == NULL)
    {
        IF_UNLOCK();
	IF_ERROR("request == NULL, returning NPF_E_UNKNOWN");
	return NPF_E_UNKNOWN;
    }

    if (pdk_callback_isCBExist(cbh) == NPF_INVALID_PARAMETERS)
    {
	PIL_FreeMemory(request);
        IF_UNLOCK();
        IF_ERROR("***** No Callback found *****");
        return (NPF_IF_E_BAD_CALLBACK_HANDLE);
    }
    request->callback =  pdk_callback_func((NPF_CBHANDLE)cbh,
                                (NPF_USERCONTEXT *)&request->context);
    if(request->callback == NULL)
    {
	PIL_FreeMemory(request);
        IF_UNLOCK();
        IF_ERROR("***** No Callback found *****");
        return (NPF_IF_E_BAD_CALLBACK_FUNCTION);
    }


    IF_INFO("no= %d, type= %d, handle=%d, corr= %d, report= %d,", no, type, cbh, corr, report);
    request->no = no;
    request->fpReq = 0;
    request->type = type;
    request->cbh = cbh;
    request->corr = corr;
    request->report = report;
    request->data.type = type;
    request->data.allOK = NPF_TRUE;

    npf_list_pushBack(&RequestList, request);
    *req = request;
    IF_UNLOCK();
    return NPF_NO_ERROR;
}


/****************************************************************************
 * Function: getStats()                                          	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		req: Pointer to request_t structure			    *
 *  							    		    *
 * Description: 							    *
 *             	This function Invokes FPPAPI statistics functions based on  *
 *		interface type within request_t. For layer 2 interfaces	    *
 *		it will call "FPPAPI_L2Interface_GetStatistics" and for     *
 *		layer 3 interface type, it will call 			    *
 *		"FPPAPI_L3Interface_GetStatistics"			    *
 * Return: 								    *
 *		NPF_E_UNKNOWN - On error				    *
 *		NPF_NO_ERROR  - On no error				    *
 ****************************************************************************/
static NPF_error_t getStats(request_t *req, If_t *ifc)
{
    uint32_t	count;
    IF_FUNCTION();
    if(ifc->no == 0 || ifc->FE.count == 0 || ifc->PORT.count == 0)
    {
	IF_ERROR("***** ifc->FE.count == 0 || ifc->PORT.count == 0 *****");
	req->error = NPF_IF_E_INVALID_HANDLE;
	onResponse(req, NULL);
	return (NPF_E_UNKNOWN);
    }
    switch(ifc->npfIf.type)
    {
	case NPF_IF_TYPE_LAN:
	case NPF_IF_TYPE_POS:
	case NPF_IF_TYPE_ATM:
	{
    	    for(count = 0; count < ifc->FE.count; count++)
    	    {
	        req->id = ifc->PORT.portsList[count].fe_id;
	        IF_TRACE("Calling FPPAPI_L2Interface_GetStatistics (FE id = %d)", req->id);
                if (FPPAPI_L2Interface_GetStatistics(
               	    req->id,
		    newIfCorrelator(ifc, req),
               	    ifFPgetL2StatsCallbackHandle,
               	    ifc->PORT.ports,
                    ifc->npfIf.type,
               	    ifc->PORT.count) != FPPI_SUCCESS)
                {
           	    IF_ERROR("***** INAVALID L2 Property *****");
           	    req->error = NPF_IF_E_INVALID_HANDLE;
            	    onResponse(req, NULL);
            	    return (NPF_E_UNKNOWN);
                }
	        IF_TRACE("Called FPPAPI_L2Interface_GetStatistics (FE id = %d) !!!", req->id);
    	    }
	}
	break;
	case NPF_IF_TYPE_IPV4:
#ifdef _INCLUDE_IPV6
	case NPF_IF_TYPE_IPV6:
	case NPF_IF_TYPE_IPV6INV4:
#endif
   case NPF_IF_TYPE_MPLS:
	{
    	    for(count = 0; count < ifc->FE.count; count++)
    	    {
	        req->id = ifc->PORT.portsList[count].fe_id;
	        IF_TRACE("Calling FPPAPI_L3Interface_GetStatistics(FE id = %d)", req->id);
                if (FPPAPI_L3Interface_GetStatistics(
               	    req->id,
		    newIfCorrelator(ifc, req),
               	    ifFPgetL3StatsCallbackHandle,
               	    ifc->PORT.ports,
                    ifc->npfIf.type, 
               	    ifc->PORT.count) != FPPI_SUCCESS)
                {
           	    IF_ERROR("***** FPPAPI_L3Interface_GetStatistics failed *****");
           	    req->error = NPF_IF_E_INVALID_HANDLE;
            	    onResponse(req, NULL);
            	    return (NPF_E_UNKNOWN);
                }
    	    }
	}
	break;
	default:
	{
	    IF_ERROR("***** NOT SUPPORTED *****");
	    return(NPF_E_UNKNOWN);
	}
    }
    return(NPF_NO_ERROR);
}

/****************************************************************************
 * Function: delL3Property()                                          	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		req - request_t associated with applications api request    *
 *		type - L3 attribute type to delete			    *
 *		value - L3 attribute value				    *
 *		cb - Callback handle registred with FPPAPI		    *
 *  							    		    *
 * Description: 							    *
 *             	This function invokes "FPPAPI_L3Interface_DelProperties	    *
 *		if interface type in req is of "NPF_IF_TYPE_IPV6"	    *
 * Return: 								    *
 *		NPF_E_UNKNOWN - On error				    *
 *		NPF_NO_ERROR  - On no error				    *
 ****************************************************************************/
static NPF_error_t delL3Property(request_t *req, 
			If_t		*ifc,
			NPF_L3ATTRS_TYPE type, 
			void *value, 
			FPPI_CBHANDLE cb)
{
    uint32_t	count;
    IF_FUNCTION();
    if(req == NULL || ifc == NULL)
    {
	IF_ERROR("req == NULL || ifc == NULL");
	return (NPF_E_UNKNOWN);
    }
    if(setL3Attrib(req, ifc, type, value) != NPF_NO_ERROR)
    {
        IF_ERROR("***** INAVALID L3 Attribute for : %d ***", req->ihandle);
        return (NPF_E_UNKNOWN);
    }
    for(count = 0; count < ifc->FE.count; count++)
    {
	req->id = ifc->PORT.portsList[count].fe_id;
	req->fe[count].id = ifc->PORT.portsList[count].fe_id;
        req->fe[count].req = ifc->FE.count;
	IF_TRACE("- Calling FPPAPI_L3Interface_DelProperties for FE id = %d -", req->id);
	if(ifc->npfIf.type == NPF_IF_TYPE_IPV4)
	{
            if (FPPAPI_L3Interface_DelProperties(
                req->id,
		newIfCorrelator(ifc, req),
                cb,
                ifc->FE.l3Attribs[count],
                ifc->PORT.portsList[count].num_ports ) != FPPI_SUCCESS)
            {
                IF_ERROR("***** INAVALID L3 Property *****");
                req->error = NPF_IF_E_INVALID_L3_HANDLE;
                onResponse(req, NULL);
                return (NPF_E_UNKNOWN);
            }
	}
#ifdef _INCLUDE_IPV6
	else if(ifc->npfIf.type == NPF_IF_TYPE_IPV6)
	{
            if (FPPAPI_L3Interface_DelPropertiesIPv6(
                req->id,
		newIfCorrelator(ifc, req),
                cb,
                ifc->FE.l3Attribs[count],
                ifc->PORT.portsList[count].num_ports ) != FPPI_SUCCESS)
            {
                IF_ERROR("***** INAVALID L3 Property *****");
                req->error = NPF_IF_E_INVALID_L3_HANDLE;
                onResponse(req, NULL);
                return (NPF_E_UNKNOWN);
            }
	}
#endif
	else
	{
            IF_ERROR("***** INAVALID L3 Type *****");
            req->error = NPF_IF_E_INVALID_L3_HANDLE;
            onResponse(req, NULL);
            return (NPF_E_UNKNOWN);
	}
    }
    return(NPF_NO_ERROR);
}

/****************************************************************************
 * Function: setL3Property()                                          	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		req - request_t associated with applications api request    *
 *		type - L3 attribute type to set			    	    *
 *		value - L3 attribute value				    *
 *		cb - Callback handle registred with FPPAPI		    *
 *  							    		    *
 * Description: 							    *
 * Return: 								    *
 *		NPF_E_UNKNOWN - On error				    *
 *		NPF_NO_ERROR  - On no error				    *
 ****************************************************************************/
static NPF_error_t setL3Property(request_t *req, 
			If_t		   *ifc,
			NPF_L3ATTRS_TYPE type, 
			void *value, 
			FPPI_CBHANDLE cb)
{
    uint32_t	count;
    IF_FUNCTION();
    if(req == NULL || ifc == NULL)
    {
	IF_ERROR("req == NULL || ifc == NULL");
	return (NPF_E_UNKNOWN);
    }
    if(setL3Attrib(req, ifc, type, value) != NPF_NO_ERROR)
    {
        IF_ERROR("***** INAVALID L3 Attribute for : %d ***", req->ihandle);
        return (NPF_E_UNKNOWN);
    }
    for(count = 0; count < ifc->FE.count; count++)
    {
	req->id = ifc->PORT.portsList[count].fe_id;
	req->fe[count].id = ifc->PORT.portsList[count].fe_id;
        req->fe[count].req = ifc->FE.count;
	if(ifc->npfIf.type == NPF_IF_TYPE_IPV4 ||
	   ifc->npfIf.type == NPF_IF_TYPE_LAN)
	{
	    IF_TRACE("- Calling FPPAPI_L3Interface_SetProperties for FE id = %d -", req->id);
            if (FPPAPI_L3Interface_SetProperties(
                req->id,
		newIfCorrelator(ifc, req),
                cb,
                ifc->FE.l3Attribs[count],
                ifc->PORT.portsList[count].num_ports ) != FPPI_SUCCESS)
            {
                IF_ERROR("***** INAVALID L3 Property *****");
                req->error = NPF_IF_E_INVALID_PARAM;
                onResponse(req, NULL);
                return (NPF_E_UNKNOWN);
            }
	}
#ifdef _INCLUDE_IPV6
	else if(ifc->npfIf.type == NPF_IF_TYPE_IPV6)
	{
	    IF_TRACE("- Calling FPPAPI_L3Interface_SetPropertiesIPv6 for FE id = %d -", req->id);
            if (FPPAPI_L3Interface_SetPropertiesIPv6(
                req->id,
		newIfCorrelator(ifc, req),
                cb,
                ifc->FE.l3Attribs[count],
                ifc->PORT.portsList[count].num_ports ) != FPPI_SUCCESS)
            {
                IF_ERROR("***** INAVALID L3 Property *****");
                req->error = NPF_IF_E_INVALID_PARAM;
                onResponse(req, NULL);
                return (NPF_E_UNKNOWN);
            }
	    IF_TRACE("- FPPAPI_L3Interface_SetPropertiesIPv6  Called !!!!!!");
	}
#endif
	else
	{
            IF_ERROR("***** INAVALID L3 Interface Type[%d] *****", ifc->npfIf.type);
            req->error = NPF_IF_E_INVALID_PARAM;
            onResponse(req, NULL);
            return (NPF_E_UNKNOWN);
	}
    }
    return(NPF_NO_ERROR);
}

/****************************************************************************
 * Function: setL2Property()                                          	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		req - request_t associated with applications api request    *
 *		type - L3 attribute type to set			    	    *
 *		value - L3 attribute value				    *
 *		cb - Callback handle registred with FPPAPI		    *
 *  							    		    *
 * Description: 							    *
 * Return: 								    *
 *		NPF_E_UNKNOWN - On error				    *
 *		NPF_NO_ERROR  - On no error				    *
 ****************************************************************************/
static NPF_error_t setL2Property(request_t *req, 
			If_t		   *ifc,
			NPF_L2ATTRS_TYPE type, 
			void *value, 
			FPPI_CBHANDLE cb)
{
    uint32_t	count;
    IF_FUNCTION();
    if(req == NULL || ifc == NULL)
    {
	IF_ERROR("req == NULL || ifc == NULL");
	return (NPF_E_UNKNOWN);
    }
    if(setL2Attrib(req, ifc, type, value) != NPF_NO_ERROR)
    {
        IF_ERROR("***** INAVALID L2 Attribute for : %d ***", req->ihandle);
        return (NPF_E_UNKNOWN);
    }
    for(count = 0; count < ifc->FE.count; count++)
    {
	req->id = ifc->PORT.portsList[count].fe_id;
	req->fe[count].id = ifc->PORT.portsList[count].fe_id;
        req->fe[count].req = ifc->FE.count;
	IF_TRACE("-- Calling FPPAPI_L2Interface_SetProperties for FE id = %d --", req->id);
        if (FPPAPI_L2Interface_SetProperties(
                req->id,
		newIfCorrelator(ifc, req),
                cb,
                ifc->FE.l2Attribs[count],
                ifc->PORT.portsList[count].num_ports ) != FPPI_SUCCESS)
        {
            IF_ERROR("***** INAVALID L2 Property *****");
            req->error = NPF_IF_E_INVALID_PARAM;
            onResponse(req, NULL);
            return (NPF_E_UNKNOWN);
        }
    }
    return(NPF_NO_ERROR);
}

/****************************************************************************
 * Function: updateL3Data()                                          	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		req - request_t associated with applications api request    *
 *		type - L3 attribute type to delete			    *
 *		resp - Response from FP module				    *
 *  							    		    *
 * Description: 							    *
 *             	This function is invoked on FP response for L3 set Property.*
 *		updates local data and fires IPADDR_CHANGE/IF UP event	    *
 * Return: 								    *
 *		NPF_E_UNKNOWN - On error				    *
 *		NPF_NO_ERROR  - On no error				    *
 ****************************************************************************/
NPF_error_t updateL3Data(request_t *req, If_t *ifc, NPF_L3ATTRS_TYPE type, void *resp)
{
    uint32_t	    no;
    uint32_t	    i;
    NPF_HANDLE      ifList[MAXIFOWNPORT];
    NPF_IF_STATUS   prevStatus;
    NPF_IfEventData_t ev;
    NPF_boolean_t  ok = NPF_TRUE;
    PDK_INTERFACE_t  *data;
    IF_FUNCTION();
    IF_INFO("Updating L3 System data for FE id = %d", req->id);
    if(cm_getDataByHandle(ifc->nsHandle, (void **)&data) == -1)
    {
	IF_ERROR("*** Invalid Interface Handle = %d ***", ifc->nsHandle);
	return(NPF_E_UNKNOWN);
    }
    if(data == NULL) /* This will Never happen !!!! */
    {
	IF_ERROR("***** PDK_PORT_t data == NULL *****");
	return(NPF_E_UNKNOWN);
    }
    for(req->ok = 0; req->ok < ifc->FE.count; req->ok++)
	ok += req->fe[req->ok].ok;
    if(!ok)
    {
	IF_TRACE("Some more FE response Pending ........");
	return(NPF_NO_ERROR);
    }
    IF_INFO(">>>>>>> ALL FEs RESPONDED <<<<<<<<<<");
    switch(type)
    {
	case NPF_L3ATTRS_IPADDR:
    	    data->ipattribs.addr_u.ipaddr = ifc->FE.l3Attribs[0][0].addr_u.addr;
    	    data->ipattribs.mask_u.submask = ifc->FE.l3Attribs[0][0].mask_u.mask;
	    IF_TRACE("Updated Local ipaddr(%d), submask(%d)",
	    ifc->FE.l3Attribs[0][0].addr_u.addr, ifc->FE.l3Attribs[0][0].mask_u.mask);
	    prevStatus = data->status;
            ifSetFlag(req->ihandle, NPF_IF_UP, (int *)&no, ifList);
	    if(prevStatus == NPF_IF_UP)
            {
		for (i=0; i<no; i++)
		{
                    ev.handle = ifList[i];
                    ev.eventType = NPF_EVENT_IF_IPADDR_CHANGE;
                    IfPdkEvent(req->ihandle, NPF_EVENT_IF_IPADDR_CHANGE);
                    IfEvent(&ev);
		}
            }
            else if(prevStatus == NPF_IF_DOWN)
            {
		for (i=0; i<no; i++)
		{
                    ev.handle = ifList[i];
                    ev.eventType = NPF_IF_OPR_UP;
		    // TODO: update the IF_OPR_STATUS & ADMIN_STATUS
		    /*
		     *Currently no operation state storage defined.
		     *So not updating IF_OPR_STATUS
		    */
		    if(ifc != NULL)
			ifc->npfIf.adminStatus = NPF_IF_ADMIN_STATUS_UP;
                    IfPdkEvent(req->ihandle, NPF_EVENT_IF_UP);
                    IfEvent(&ev);
		}
            }

	break;
	/* Following two types are not supported */
	case NPF_L3ATTRS_ALL:
	case NPF_L3ATTRS_IPFWD:
	    return(NPF_E_UNKNOWN);
	break;
	default:
	    return(NPF_E_UNKNOWN);
    }
    onResponse(req, resp);
    return(NPF_NO_ERROR);
}

/****************************************************************************
 * Function: updateL2Data()                                          	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		req - request_t associated with applications api request    *
 *		type - L3 attribute type to delete			    *
 *		resp - Response from FP module				    *
 *  							    		    *
 * Description: 							    *
 *             	This function is invoked on FP response for L2 set Property.*
 *		updates local data and fires IF UP/DOWN event	    	    *
 * Return: 								    *
 *		NPF_E_UNKNOWN - On error				    *
 *		NPF_NO_ERROR  - On no error				    *
 ****************************************************************************/
NPF_error_t updateL2Data(request_t *req, If_t *ifc, NPF_L2ATTRS_TYPE type, void *resp)
{
    NPF_IfEventData_t ev;
    NPF_boolean_t  ok = NPF_TRUE;
    PDK_PORT_t  *data;
    IF_FUNCTION();
    IF_INFO("Updating L2 System data for FE id = %d", req->id);
    if(cm_getDataByHandle(ifc->nsHandle, (NPF_DATA *)&ifc->inode) == -1)
    {
	IF_ERROR("*** Invalid Interface Handle = %d ***", ifc->nsHandle);
	return(NPF_E_UNKNOWN);
    }
    data = (PDK_PORT_t *)ifc->inode;
    if(data == NULL) /* This will Never happen !!!! */
    {
	IF_ERROR("***** PDK_PORT_t data == NULL *****");
	return(NPF_E_UNKNOWN);
    }

    for(req->ok = 0; req->ok < ifc->FE.count; req->ok++)
	ok += req->fe[req->ok].ok;
    if(!ok)
    {
	IF_TRACE("Some more FE response Pending ........");
	return(NPF_NO_ERROR);
    }
    IF_INFO(">>>>>>> ALL FEs RESPONDED <<<<<<<<<<");
    switch(type)
    {
	case NPF_L2ATTRS_MTU:
	    IF_TRACE("Updating Local MTU = %d", ifc->FE.l2Attribs[0][0].mtu);
    	    data->attribs.MTU = ifc->FE.l2Attribs[0][0].mtu;
	break;
	case NPF_L2ATTRS_LINKSPEED:
	    IF_TRACE("Updating Local curTxRate = %d", ifc->FE.l2Attribs[0][0].link_speed);
    	    data->attribs.curTxRate = ifc->FE.l2Attribs[0][0].link_speed;
	break;
	case NPF_L2ATTRS_STATUS:
	{
	    uint32_t	    i;
	    uint32_t	    no;
	    NPF_HANDLE      ifList[MAXIFOWNPORT];
	    /* assign system data, set all the related interface & ports flags */
	    ifSetFlag(req->ihandle, ifc->FE.l2Attribs[0][0].status, (int *)&no, ifList);
	    switch(ifc->FE.l2Attribs[0][0].status)
	    {
		case NPF_IF_UP:
		    for (i=0; i<no; i++)
		    {
                        ev.handle = ifList[i];
                        ev.eventType = NPF_IF_OPR_UP;
			// TODO: update IM NPF_IF_OPR_UP and NPF_ADMIN_UP
		        /*
		         *Currently no operation state storage defined.
		     	 *So not updating IF_OPR_STATUS
		    	*/
		        if(ifc != NULL)
			    ifc->npfIf.adminStatus = NPF_IF_ADMIN_STATUS_UP;
                        IfPdkEvent(req->ihandle, NPF_EVENT_IF_UP);
                        IfEvent(&ev);
		    }
		    break;
		case NPF_IF_DOWN:
		    for (i=0; i<no; i++)
		    {
                        ev.handle = ifList[i];
                        ev.eventType = NPF_IF_OPR_DOWN;
			// TODO: update IF_OPR_DOWN or ADMIN_DOWN?
		        /*
		         *Currently no operation state storage defined.
		     	 *So not updating IF_OPR_STATUS
		    	*/
		        if(ifc != NULL)
			    ifc->npfIf.adminStatus = NPF_IF_ADMIN_STATUS_DOWN;
                        IfPdkEvent(req->ihandle, NPF_EVENT_IF_DOWN);
                        IfEvent(&ev);
		    }
		    break;
		default:
		    break;

	    }
	}
	break;
	case NPF_L2ATTRS_ALL:
	break;
	default:
	    return(NPF_E_UNKNOWN);
    }
    req->error = NPF_NO_ERROR;
    if(req->fpResp >= req->fpReq)
    {
	IF_TRACE("req->fpResp(%d) >= req->fpReq(%d)", req->fpResp, req->fpReq);
        onResponse(req, resp);
    }
    else
    {
	IF_TRACE("req->fpReq(%d) != req->fpResp(%d)", req->fpReq, req->fpResp);
    }
    return(NPF_NO_ERROR);
}

/****************************************************************************
 * Function: getFEandPORTdata()                                             *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		iface - reference to If_t				    *
 *  							    		    *
 * Description: 							    *
 *             	This function gets FE/PORT information associated with 	    *
 *		interface handle within iface				    *
 * Return: 								    *
 *		NPF_E_UNKNOWN - On error				    *
 *		NPF_NO_ERROR  - On no error				    *
 ****************************************************************************/
NPF_error_t getFEandPORTdata(If_t  *iface)
{
    uint32_t    count;
    uint32_t    pc;
    IF_FUNCTION();
    if(pdk_ns_getFEPorts_from_IF(iface->nsHandle, (int *)&iface->FE.count,
                &iface->PORT.portsList) != NPF_SUCCESS)
    {
        IF_ERROR("*** Iface(%d) Don't have any ports associated *** ", iface->nsHandle);
        if (iface->FE.count != 0)
        {
            PIL_FreeMemory(iface->PORT.portsList->ports);
            PIL_FreeMemory(iface->PORT.portsList);
        }
        return(NPF_E_UNKNOWN);
    }
    for (count = 0; count < iface->FE.count; count++) 
    {
	for (pc = 0; pc< iface->PORT.portsList[count].num_ports; pc++)
	{
    	    iface->PORT.count++;
            iface->PORT.ports[pc] = iface->PORT.portsList[count].ports[pc].port_id;
	}
    }
    IF_INFO("FE Count = %d, Port Id Count = %d", iface->FE.count, iface->PORT.count);
    return(NPF_NO_ERROR);
}

/****************************************************************************
 * Function: setL3Attrib()                                          	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		ifc - local interface data				    *
 *		type - L3 attribute type 				    *
 *		value - L3 attribute value				    *
 *  							    		    *
 * Description: 							    *
 *             	This function update local data for L3 attributes	    *
 * Return: 								    *
 *		NPF_E_UNKNOWN - On error				    *
 *		NPF_NO_ERROR  - On no error				    *
 ****************************************************************************/
NPF_error_t setL3Attrib(request_t *req, If_t *ifc, NPF_L3ATTRS_TYPE type, void *value)
{
    uint32_t    count;
    uint32_t    pc;
    IF_FUNCTION();
    for (count = 0; count < ifc->FE.count; count++) 
    {
	for (pc = 0; pc< ifc->PORT.portsList[count].num_ports; pc++)
	{
	    if(ifc->npfIf.type == NPF_IF_TYPE_LAN)
		ifc->FE.l2Attribs[count][pc].port_type = NPF_PORT_ETHERNET;
	    else if(ifc->npfIf.type == NPF_IF_TYPE_ATM)
		ifc->FE.l2Attribs[count][pc].port_type = NPF_PORT_ATM;
	    else if(ifc->npfIf.type == NPF_PORT_POS)
		ifc->FE.l2Attribs[count][pc].port_type = NPF_PORT_POS;
	    else
		ifc->FE.l2Attribs[count][pc].port_type = ifc->npfIf.type;
 	    ifc->FE.l3Attribs[count][pc].feid = ifc->PORT.portsList[count].fe_id;
	    ifc->FE.l3Attribs[count][pc].type = type;
	    ifc->FE.l3Attribs[count][pc].portid = ifc->PORT.portsList[count].ports[pc].port_id;
	    switch(type)
	    {
		case NPF_L3ATTRS_IPADDR:
		    if(ifc->npfIf.type == NPF_IF_TYPE_IPV4)
		    {

	        	ifc->FE.l3Attribs[count][pc].addr_type = NPF_IPv4_ADDRESS;

			IF_TRACE("Setting NPF_L3ATTRS_IPADDR for NPF_IF_TYPE_IPV4");
			if(!((NPF_IPv4NetAddr_t *)value)->IPv4Addr)
			{
			    IF_ERROR("*** Invalid IPv4Addr [ %d ] ***", ((NPF_IPv4NetAddr_t *)value)->IPv4Addr);
			    req->error = NPF_IF_E_INVALID_IPADDR;
			    req->fpResp++;
			    onResponse(req, NULL);
			    return(NPF_E_UNKNOWN);
			}
			if(!((NPF_IPv4NetAddr_t *)value)->IPv4NetPlen)
			{
			    IF_ERROR("*** Invalid IPv4NetPlen [ %d ] ***", ((NPF_IPv4NetAddr_t *)value)->IPv4NetPlen);
			    req->error = NPF_IF_E_INVALID_NETPLEN;
			    req->fpResp++;
			    onResponse(req, NULL);
			    return(NPF_E_UNKNOWN);
			}
	                ifc->FE.l3Attribs[count][pc].addr_u.addr = ((NPF_IPv4NetAddr_t *)value)->IPv4Addr;
	                ifc->FE.l3Attribs[count][pc].mask_u.mask = (0xFFFFFFFF << (32 - ((NPF_IPv4NetAddr_t *)value)->IPv4NetPlen));
		    }
		    else if(ifc->npfIf.type == NPF_IF_TYPE_IPV6)
		    {

	        	ifc->FE.l3Attribs[count][pc].addr_type = NPF_IPv6_ADDRESS;

			IF_TRACE("Setting NPF_L3ATTRS_IPADDR for NPF_IF_TYPE_IPV6");
			if(!((NPF_IPv6NetAddr_t *)value)->IPv6Addr)
			{
			    IF_ERROR("*** Invalid IPv6Addr [ %d ] ***", ((NPF_IPv6NetAddr_t *)value)->IPv6Addr);
			    req->error = NPF_IF_E_INVALID_IPADDR;
			    req->fpResp++;
			    onResponse(req, NULL);
			    return(NPF_E_UNKNOWN);
			}
			if(!((NPF_IPv6NetAddr_t *)value)->IPv6NetPlen)
			{
			    IF_ERROR("*** Invalid IPv6NetPlen [ %d ] ***", ((NPF_IPv6NetAddr_t *)value)->IPv6NetPlen);
			    req->error = NPF_IF_E_INVALID_NETPLEN;
			    req->fpResp++;
			    onResponse(req, NULL);
			    return(NPF_E_UNKNOWN);
			}
	                memcpy(ifc->FE.l3Attribs[count][pc].addr_u.v6addr , 
                               &((NPF_IPv6NetAddr_t *)value)->IPv6Addr, sizeof(NPF_IPv6Address_t));
	                memcpy(ifc->FE.l3Attribs[count][pc].mask_u.v6mask , 
                               &((NPF_IPv6NetAddr_t *)value)->IPv6NetPlen, sizeof(NPF_uint8_t));
		    }
		break;
		case NPF_L3ATTRS_IPFWD:
	            ifc->FE.l3Attribs[count][pc].ipfwding = *((int *)value);
		break;
		default:
	            return(NPF_E_UNKNOWN);
	    }
	}
    }
    return(NPF_NO_ERROR);
}

/****************************************************************************
 * Function: setL2Attrib()                                          	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		ifc - local interface data				    *
 *		type - L2 attribute type 				    *
 *		value - L2 attribute value				    *
 *  							    		    *
 * Description: 							    *
 *             	This function update local data for L2 attributes	    *
 * Return: 								    *
 *		NPF_E_UNKNOWN - On error				    *
 *		NPF_NO_ERROR  - On no error				    *
 ****************************************************************************/
NPF_error_t setL2Attrib(request_t *req, If_t *ifc, NPF_L2ATTRS_TYPE type, void *value)
{
    uint32_t speed;
    uint32_t    count;
    uint32_t    pc;
    IF_FUNCTION();
    for (count = 0; count < ifc->FE.count; count++) 
    {
	for (pc = 0; pc< ifc->PORT.portsList[count].num_ports; pc++)
	{
	    if(ifc->npfIf.type == NPF_IF_TYPE_LAN)
		ifc->FE.l2Attribs[count][pc].port_type = NPF_PORT_ETHERNET;
	    else if(ifc->npfIf.type == NPF_IF_TYPE_ATM)
		ifc->FE.l2Attribs[count][pc].port_type = NPF_PORT_ATM;
	    else if(ifc->npfIf.type == NPF_PORT_POS)
		ifc->FE.l2Attribs[count][pc].port_type = NPF_PORT_POS;
       else if(ifc->npfIf.type == NPF_IF_TYPE_MPLS)
         ifc->FE.l2Attribs[count][pc].port_type = NPF_PORT_MPLS;
	    else

		ifc->FE.l2Attribs[count][pc].port_type = ifc->npfIf.type;
 	    ifc->FE.l2Attribs[count][pc].feid = ifc->PORT.portsList[count].fe_id;
	    ifc->FE.l2Attribs[count][pc].type = type;
	    ifc->FE.l2Attribs[count][pc].portid = ifc->PORT.portsList[count].ports[pc].port_id;
	    switch(type)
	    {
		case NPF_L2ATTRS_LINKSPEED:
		{
		    speed = *((uint32_t *)value);
		    switch(speed)
		    {
			case NPF_IF_LAN_SPEED_10M: 
			case NPF_IF_LAN_SPEED_100M:
			case NPF_IF_LAN_SPEED_1G:
			case NPF_IF_LAN_SPEED_10G:
			case NPF_IF_LAN_SPEED_AUTO:
			{
	                    ifc->FE.l2Attribs[count][pc].link_speed = speed;
	    		    ifc->npfIf.u.LAN_Attr.speed = speed;
			}
			break;
			default: 
			{
			    IF_ERROR("*** INVALID SPEED : %d ***", *((uint32_t *)value));
			    req->error = NPF_IF_E_INVALID_SPEED;
			    req->fpResp++;
			    onResponse(req, NULL);
			    return(NPF_E_UNKNOWN);
			}
		    }
	        }
		break;
		case NPF_L2ATTRS_MTU:
		    if(!*((NPF_uint16_t *)value))
		    {
			IF_ERROR("*** INVALID MTU : %d ***", *((uint16_t *)value));
			req->error = NPF_IF_E_INVALID_MTU;
			req->fpResp++;
			onResponse(req, NULL);
			return(NPF_E_UNKNOWN);
		    }
	            ifc->FE.l2Attribs[count][pc].mtu = *((NPF_uint16_t *)value);
		break;
		case NPF_L2ATTRS_STATUS:
		    switch(*((int *)value))
		    {
			case NPF_IF_UP:
			case NPF_IF_DOWN:
			{
	            	    ifc->FE.l2Attribs[count][pc].status = *((int *)value);
			}
			break;
			default:
			{
			    IF_ERROR("*** INVALID STATUS : %d ***", *((int *)value));
			    req->error = NPF_IF_E_INVALID_ADMIN_STATUS;
			    req->fpResp++;
			    onResponse(req, NULL);
			    return(NPF_E_UNKNOWN);
			}
			
		    }
		break;
		default:
	            return(NPF_E_UNKNOWN);
	    }
	}
    }
    return(NPF_NO_ERROR);
}

/****************************************************************************
 * Function: IfGenericStatsGet()                                            *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		ifHandle - interface handle				    *
 *		req - request_t data for application's api request	    *
 *  							    		    *
 * Description: 							    *
 *             	This function gets statistics for an interface		    *
 * Return: 								    *
 *		NPF_IF_E_INVALID_HANDLE - Invalid interface handle	    *
 *		NPF_NO_ERROR  - On no error				    *
 ****************************************************************************/
NPF_error_t IfGenericStatsGet(NPF_IfHandle_t   ifHandle, request_t *req)
{
    If_t	*ifc;
    IF_FUNCTION();
    if(getInterface(ifHandle, &ifc) != NPF_NO_ERROR)
    {
	req->error = NPF_IF_E_INVALID_HANDLE;
	req->ihandle = ifHandle;
	onResponse(req, NULL);
	return NPF_E_UNKNOWN;
    }

#ifdef _INCLUDE_ATM
    if(ifc->npfIf.type != NPF_IF_TYPE_ATM)
    {	
    getStats(req, ifc);
    }
    else
    {
		IfATM_IfGenericStatsGet(ifc,req);
    }
#else
    getStats(req, ifc);
#endif /* _INLCUDE_ATM */


    return(NPF_NO_ERROR);
}

/****************************************************************************
 * Function: IfAddrSet()                                            	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		ifHandle - interface handle				    *
 *		addr - IPV4 address					    *
 *		req - request_t data for application's api request	    *
 *  							    		    *
 * Description: 							    *
 *             	This function sets IPV4 address of an interface		    *
 * Return: 								    *
 *		NPF_IF_E_INVALID_HANDLE - Invalid interface handle	    *
 *		NPF_NO_ERROR  - On no error				    *
 ****************************************************************************/
NPF_error_t IfAddrSet(NPF_IfHandle_t ifHandle,
                        NPF_IPv4NetAddr_t *addr,
                        request_t *req)
{
    If_t	*ifc;
    IF_FUNCTION();
    if(getInterface(ifHandle, &ifc) != NPF_NO_ERROR)
    {
	req->error = NPF_IF_E_INVALID_HANDLE;
	req->ihandle = ifHandle;
	onResponse(req, NULL);
	return NPF_E_UNKNOWN;
    }
    setL3Property(req, ifc, NPF_L3ATTRS_IPADDR, addr, ifFPsetIPaddrCallbackHandle);
    return(NPF_NO_ERROR);
}

#ifdef _INCLUDE_IPV6

/****************************************************************************
 * Function: IfIPv6AddrAdd()                                           	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		ifHandle - interface handle				    *
 *		addr - IPV6 address					    *
 *		req - request_t data for application's api request	    *
 *  							    		    *
 * Description: 							    *
 *             	This function sets IPV6 address	of an interface		    *
 * Return: 								    *
 *		NPF_IF_E_INVALID_HANDLE - Invalid interface handle	    *
 *		NPF_NO_ERROR  - On no error				    *
 ****************************************************************************/
NPF_error_t IfIPv6AddrAdd(NPF_IfHandle_t ifHandle,
                        NPF_IPv6NetAddr_t *addr,
                        request_t *req)
{
    If_t	*ifc;
    IF_FUNCTION();
    if(getInterface(ifHandle, &ifc) != NPF_NO_ERROR)
    {
	req->error = NPF_IF_E_INVALID_HANDLE;
	req->ihandle = ifHandle;
	onResponse(req, NULL);
	return NPF_E_UNKNOWN;
    }
    setL3Property(req, ifc, NPF_L3ATTRS_IPADDR, addr, ifFPsetIPaddrCallbackHandle);
    return(NPF_NO_ERROR);
}


/****************************************************************************
 * Function: IfIPv6AddrDelete()                                             *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		ifHandle - interface handle				    *
 *		addr - IPV6 address					    *
 *		req - request_t data for application's api request	    *
 *  							    		    *
 * Description: 							    *
 *             	This function deletes IPV6 address	of an interface	    *
 * Return: 								    *
 *		NPF_IF_E_INVALID_HANDLE - Invalid interface handle	    *
 *		NPF_NO_ERROR  - On no error				    *
 ****************************************************************************/
NPF_error_t IfIPv6AddrDelete(NPF_IfHandle_t ifHandle,
                        NPF_IPv6NetAddr_t *addr,
                        request_t *req)
{
    If_t	*ifc;
    IF_FUNCTION();
    if(getInterface(ifHandle, &ifc) != NPF_NO_ERROR)
    {
	req->error = NPF_IF_E_INVALID_HANDLE;
	req->ihandle = ifHandle;
	onResponse(req, NULL);
	return NPF_E_UNKNOWN;
    }
    delL3Property(req, ifc, NPF_L3ATTRS_IPADDR, addr, ifFPsetIPaddrCallbackHandle);
    return(NPF_NO_ERROR);
}

#endif

/****************************************************************************
 * Function: IfSpeedSet()                                            	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		ifHandle - interface handle				    *
 *		speed - Interfcae speed					    *
 *		req - request_t data for application's api request	    *
 *  							    		    *
 * Description: 							    *
 *             	This function sets speed of an interface		    *
 * Return: 								    *
 *		NPF_IF_E_INVALID_HANDLE - Invalid interface handle	    *
 *		NPF_NO_ERROR  - On no error				    *
 ****************************************************************************/
NPF_error_t IfSpeedSet(NPF_IfHandle_t ifHandle,
                        NPF_IfLAN_Speed_t speed,
                        request_t *req)
{
    If_t	*ifc;
    uint32_t    lanSpeed;
    IF_FUNCTION();
    if(getInterface(ifHandle, &ifc) != NPF_NO_ERROR)
    {
	req->error = NPF_IF_E_INVALID_HANDLE;
	req->ihandle = ifHandle;
	onResponse(req, NULL);
	return NPF_E_UNKNOWN;
    }
    lanSpeed = speed;
    setL2Property(req, ifc, NPF_L2ATTRS_LINKSPEED, &lanSpeed, ifFPsetLANSpeedCallbackHandle);
    return(NPF_NO_ERROR);
}

/****************************************************************************
 * Function: IfIPv4FibSet()                                            	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *               if_cbHandle: the registered callback handle.               *
 *               if_cbCorrelator: the application\222s context for this call*
 *               if_errorReporting: the desired callback.                   *
 *               n_handles: the number of interfaces to set the FIB for.    *
 *               if_HandleArray: pointer to an array of interface handles.  *
 *               if_FIB_Handle: the new FIB handle.                         *
 *  							    		    *
 * Description: 							    *
 *             	This function sets MTU of an interface		    	    *
 * Return: 								    *
 *		NPF_IF_E_INVALID_HANDLE - Invalid interface handle	    *
 *		NPF_NO_ERROR  - On no error				    *
 ****************************************************************************/

NPF_error_t IfIPv4FibSet(NPF_IfHandle_t ifHandle,  NPF_IPv4UC_FwdTableHandle_t fib_npf_hdl, request_t *req)
{
    If_t	*ifc;
    NPF_NSHDL           fib_ns_hdl = 0, fe_ns_hdl = 0;
    NPF_HANDLE          fe_npf_hdl= 0;
    NPF_DATATYPE       npf_data_type;
    uint32_t           numFe=0, i=0;
    FPPI_FEID          fe_id[MAXFENUM], feId=0; 

    IF_FUNCTION();

    /*  Check if the interface is correct */
    if(getInterface(ifHandle, &ifc) != NPF_NO_ERROR)
    {
	    req->error = NPF_IF_E_INVALID_HANDLE;
	    req->ihandle = ifHandle;
	    onResponse(req, NULL);
	    return NPF_E_UNKNOWN;
    }
    /* Check if the Forwarding Handle is correct by finding it as a router
       in Namespace  */
    if(cm_getTypeByHandle(fib_npf_hdl,&npf_data_type) != NPF_SUCCESS)       
    {
	   req->error = NPF_IF_E_INVALID_FIB_HANDLE;
	   req->ihandle = fib_npf_hdl;
	   onResponse(req, NULL);
	   return NPF_E_UNKNOWN;
    }
    
    if( npf_data_type != NPF_IPROUTER)
    {
	   req->error = NPF_IF_E_INVALID_CHILD_HANDLE;
	   req->ihandle = fib_npf_hdl;
	   onResponse(req, NULL);
	   return NPF_E_UNKNOWN;
    }
    /* Get NS Handle for the fib */
    if(cm_getNSHDLByHandle(fib_npf_hdl, &fib_ns_hdl) != NPF_SUCCESS)
    {
         req->error = NPF_IF_E_INVALID_HANDLE;
         req->ihandle = fib_npf_hdl;
         onResponse(req, NULL);
         IF_ERROR("*** Unable to get NS handle for FIB with npf handle %d\n *** ", fib_npf_hdl);
         npf_ns_close(fib_npf_hdl);
         return NPF_E_UNKNOWN;
    }
    /* Close the datahandle */ 
    npf_ns_close(fib_npf_hdl);
    /* Check if already there are any FEs associated with this fib */ 
    if(IfGetFEIdsFrmFIBHdl(fib_npf_hdl, &numFe, &feId) 
                                != NPF_SUCCESS)
    {
      IF_ERROR("Unable to perform check for FEIds assoc with FIB ");
	   req->error = NPF_IF_E_INVALID_HANDLE;
	   req->ihandle = fib_npf_hdl;
	   onResponse(req, NULL);
           npf_ns_close(fib_ns_hdl);
	   return NPF_E_UNKNOWN;
    }
    
    if( numFe > 0) 
    {
      IF_ERROR("Assoc. already exists between FE and FIB ");
	   req->error = NPF_IF_E_INVALID_HANDLE;
	   req->ihandle = fib_npf_hdl;
	   onResponse(req, NULL);
           npf_ns_close(fib_ns_hdl);
	   return NPF_E_UNKNOWN;
    }

    /* Assoc with FEs did not exist */
    /* Get the FE Id from the assoc. of Interface */
    if(IfGetFEid(ifHandle, &numFe,fe_id) != NPF_SUCCESS)
    {
       IF_ERROR("Interface is not associated with the FEId ");
	    req->error = NPF_IF_E_INVALID_HANDLE;
	    req->ihandle = ifHandle;
	    onResponse(req, NULL);
           npf_ns_close(fib_ns_hdl);
	    return(NPF_E_UNKNOWN);
    }
    IF_TRACE("Interface [handle %d] is associated with %d FEs",ifHandle,numFe);
    for(i=0; i < numFe; i++)
    {
       feId = fe_id[i];
       /* Get the FE handle from the FeId */
       if(cm_getFEHandleById(feId, &fe_npf_hdl) != NPF_SUCCESS)  
       {
          IF_ERROR("Unable to get FeId from FEHandle");
	       req->error = NPF_IF_E_INVALID_HANDLE;
	       req->ihandle = ifHandle;
	       onResponse(req, NULL);
               npf_ns_close(fib_ns_hdl);
	       return(NPF_E_UNKNOWN);
       }
       if(cm_getNSHDLByHandle(fe_npf_hdl, &fe_ns_hdl) != NPF_SUCCESS)
       {
	       req->error = NPF_IF_E_INVALID_HANDLE;
	       req->ihandle = ifHandle;
	       onResponse(req, NULL);
	       IF_ERROR("*** Unable to get NS handle for FE node with npf handle %d\n *** ", fib_npf_hdl);
               npf_ns_close(fe_npf_hdl);
	       return NPF_E_UNKNOWN;
       }
       npf_ns_close(fe_npf_hdl);
       /* Build association between FE and Router and vice-versa */
       if(fe_ns_hdl && fib_ns_hdl)
       {    
            /* Add associations from both directions */ 
            if(npf_ns_addAssociate(fe_ns_hdl,fib_ns_hdl) != NPF_SUCCESS) 
            {
                    IF_ERROR("assoc already exists for this FIB/Rounter");
	            req->error = NPF_IF_E_INVALID_HANDLE;
	            req->ihandle = fe_ns_hdl;
	            onResponse(req, NULL);
                    npf_ns_close(fib_ns_hdl);
                    npf_ns_close(fe_ns_hdl);
	            return(NPF_E_UNKNOWN);
            }
            if(npf_ns_addAssociate(fib_ns_hdl,fe_ns_hdl) != NPF_SUCCESS) 
            {
               IF_ERROR("assoc already exists for this FIB/Rounter");
	            req->error = NPF_IF_E_INVALID_HANDLE;
	            req->ihandle = fe_ns_hdl;
	            onResponse(req, NULL);
                    npf_ns_close(fib_ns_hdl);
                    npf_ns_close(fe_ns_hdl);
	            return(NPF_E_UNKNOWN);
            }
            npf_ns_close(fe_ns_hdl);
            IF_INFO("FE [NS handle %d] is associated with FIB Table [NS handle %d]",fe_ns_hdl,fib_ns_hdl);
       }   
       else
       {
            npf_ns_close(fe_ns_hdl);
            IF_ERROR("**ERROR: %s","Invalid NS Handles ; Unable to associate FIB with FE");
           req->error = NPF_IF_E_INVALID_HANDLE;
	        req->ihandle = fe_ns_hdl;
	        onResponse(req, NULL);
           npf_ns_close(fib_ns_hdl);
           npf_ns_close(fe_ns_hdl);
	        return(NPF_E_UNKNOWN);

       } 
    } /* End of for loop for multiple FEs */
    npf_ns_close(fib_ns_hdl);
    /* generate your own callback in this case, as we don't go down */
	 req->error = NPF_NO_ERROR;
	 req->ihandle = ifHandle;
	 onResponse(req, NULL);
    return(NPF_NO_ERROR);
}


/****************************************************************************
 * Function: IfMTUSet()                                            	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		ifHandle - interface handle				    *
 *		mtu - Interfcae MTU value				    *
 *		req - request_t data for application's api request	    *
 *  							    		    *
 * Description: 							    *
 *             	This function sets MTU of an interface		    	    *
 * Return: 								    *
 *		NPF_IF_E_INVALID_HANDLE - Invalid interface handle	    *
 *		NPF_NO_ERROR  - On no error				    *
 ****************************************************************************/
NPF_error_t IfMTUSet(NPF_IfHandle_t ifHandle, NPF_uint16_t mtu, request_t *req)
{
    If_t	*ifc;
    IF_FUNCTION();
    if(getInterface(ifHandle, &ifc) != NPF_NO_ERROR)
    {
	req->error = NPF_IF_E_INVALID_HANDLE;
	req->ihandle = ifHandle;
	onResponse(req, NULL);
	return NPF_E_UNKNOWN;
    }
    ifc->npfIf.u.IPv4_Attr.mtu = mtu;
    setL2Property(req, ifc, NPF_L2ATTRS_MTU, &mtu, ifFPsetMTUCallbackHandle);
    return(NPF_NO_ERROR);
}

/****************************************************************************
 * Function: IfPromiscSet()                                            	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		ifHandle - interface handle				    *
 *		req - request_t data for application's api request	    *
 *  							    		    *
 * Description: 							    *
 *             	This function sets Promiscous mode of an interface	    *
 * Return: 								    *
 *		NPF_IF_E_INVALID_HANDLE - Invalid interface handle	    *
 *		NPF_NO_ERROR  - On no error				    *
 ****************************************************************************/
NPF_error_t IfPromiscSet(NPF_IfHandle_t ifHandle, request_t *req)
{
    If_t	*ifc;
    int		promisc = 1;
    IF_FUNCTION();
    if(getInterface(ifHandle, &ifc) != NPF_NO_ERROR)
    {
	req->error = NPF_IF_E_INVALID_HANDLE;
	req->ihandle = ifHandle;
	onResponse(req, NULL);
	return NPF_E_UNKNOWN;
    }
    setL3Property(req, ifc, NPF_L3ATTRS_IPFWD, &promisc, ifFPsetIPfwdCallbackHandle);
    return(NPF_NO_ERROR);
}

/****************************************************************************
 * Function: IfPromiscClear()                                               *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		ifHandle - interface handle				    *
 *		req - request_t data for application's api request	    *
 *  							    		    *
 * Description: 							    *
 *             	This function clears Promiscous mode of an interface	    *
 * Return: 								    *
 *		NPF_IF_E_INVALID_HANDLE - Invalid interface handle	    *
 *		NPF_NO_ERROR  - On no error				    *
 ****************************************************************************/
NPF_error_t IfPromiscClear(NPF_IfHandle_t ifHandle, request_t *req)
{
    If_t	*ifc;
    int		promisc = 0;
    IF_FUNCTION();
    if(getInterface(ifHandle, &ifc) != NPF_NO_ERROR)
    {
	req->error = NPF_IF_E_INVALID_HANDLE;
	req->ihandle = ifHandle;
	onResponse(req, NULL);
	return NPF_E_UNKNOWN;
    }
    setL3Property(req, ifc, NPF_L3ATTRS_IPFWD, &promisc, ifFPsetIPfwdCallbackHandle);
    return(NPF_NO_ERROR);
}

/****************************************************************************
 * Function: IfCreate()                                            	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		type - interface type					    *
 *		req - request_t data for application's api request	    *
 *  							    		    *
 * Description: 							    *
 *             	This function Creates interface. Always retuns NPF_NO_ERROR *
 *             	or NPF_E_UNKNOWN to the caller. Corresponding error/success *
 *             	is notified through Asyncresponse callback		    *
 * Return: 								    *
 *		NPF_NO_ERROR  - On no error				    *
 *		NPF_E_UNKNOWN  - On error				    *
 ****************************************************************************/
NPF_error_t IfCreate(NPF_IfType_t type, request_t *req)
{
    If_t		*iface;
    NPF_RET             result;
    NPF_NSNODETYPE      nodeType;
    NPF_DATATYPE        dataType;
    NPF_NSHDL           hInstance;
    NPF_NSHDL           hNode;

    IF_FUNCTION();
    nodeType = NPF_NS_INSTNODE;
    dataType = NPF_INTERFACE;
    hInstance = 0;
    iface = (If_t *)PIL_AllocateMemory(sizeof(If_t),PIL_MEM_ZERO_INIT);
    if(iface == NULL)
    {
	IF_ERROR("***** %s", "iface == NULL");
	req->error = NPF_IF_E_NOT_CREATED;
	req->ihandle = 0;
	onResponse(req, NULL);
	return(NPF_NO_ERROR);
    }
    sprintf(iface->path, "%s/%d", interfaceRoot, (int)interfaceCount);
    iface->npfIf.type = type;
    iface->npfIf.speed = 0;
    switch(type)
    {
	case NPF_IF_TYPE_LAN:
	case NPF_IF_TYPE_ATM:
	case NPF_IF_TYPE_POS:
	case NPF_IF_TYPE_IPV4:
	case NPF_IF_TYPE_IPV6:
	case NPF_IF_TYPE_IPV6INV4:
   case NPF_IF_TYPE_MPLS:
	{
	    IF_TRACE("Interface Type : %d, Allowed ....", type);
	}
	break;
	default:
	{
	    IF_ERROR("*** Interface Type : %d, not allowed ....", type);
    	    PIL_FreeMemory(iface);
	    req->error = NPF_IF_E_NOT_CREATED;
	    req->ihandle = 0;
	    onResponse(req, NULL);
	    return(NPF_NO_ERROR);
	}
    }
    IF_INFO("Creating interface type = %d and path = %s", type, iface->path);
    /* check if IF type node exists */
    result = npf_ns_open(iface->path, &hNode);
    if(result == NPF_SUCCESS)
    {
        npf_ns_close(hNode);
        IF_INFO("%s Exists, Sending response NPF_IF_E_NOT_CREATED ...", iface->path);
        PIL_FreeMemory(iface);
        req->error = NPF_IF_E_NOT_CREATED;
        req->ihandle = 0;
        onResponse(req, NULL);
        return(NPF_NO_ERROR);
    }
    else
    {
        IF_INFO("%s Not exists, Creating it ...", iface->path);
        if(npf_ns_create(iface->path, nodeType, dataType, hInstance, &hNode) != NPF_SUCCESS)
        {
            IF_ERROR("***** %s", "npf_ns_create returns NPF_IF_E_NOT_CREATED");
            PIL_FreeMemory(iface);
            req->error = NPF_IF_E_INVALID_PARAM;
            req->ihandle = 0;
            onResponse(req, NULL);
            return(NPF_NO_ERROR);
        }
    }

  
    npf_ns_getDataHandle(hNode, (NPF_HANDLE *)&iface->nsHandle);
    iface->inode = hNode;
    iface->no = interfaceCount++;
    iface->operStat = NPF_IF_OPER_STATUS_DOWN;

    npf_list_pushBack(&IfList, iface);
    IF_TRACE("%s  = {ns_handle= %d, instance %d}", iface->path, hNode, hInstance);
    req->error = NPF_NO_ERROR;
    req->ihandle = iface->nsHandle;
    onResponse(req, NULL);
    return NPF_NO_ERROR;
}

/****************************************************************************
 * Function: associateL3IfandFEPort()                                       *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		l3Iface - L3 interface data				    *
 *		l2Iface - L2 interface data				    *
 *  							    		    *
 * Description: 							    *
 *             	This function associates namespace with l3 interface with   *
 *		namespace with l2 interface. l2Iface contains associations  *
 *		with FE port nodes. Always retuns NPF_NO_ERROR 	or 	    *
 *		NPF_E_UNKNOWN to the caller. Corresponding error/success    *
 *             	is notified through Async Response callback		    *
 * Return: 								    *
 *		NPF_NO_ERROR  - On no error				    *
 *		NPF_E_UNKNOWN  - On error				    *
 ****************************************************************************/
NPF_error_t associateL3IfandFEPort(If_t *l3Iface, If_t  *l2Iface)
{
    NPF_NSHDL           pnode;
    IF_FUNCTION();
    if(l2Iface->count == 0)
    {
	IF_ERROR("***** l2Iface->count == 0, L2 attribute mey not be set ******");
	return(NPF_E_UNKNOWN);
    }
    npf_ns_open(l3Iface->path, &l3Iface->inode);
    switch(l3Iface->npfIf.type)
    {
	/*
	 * Get Childs of this interface which is created through NPF_IfBind
	 * Get FE Port associations for this child node
	 * Associate this If node and FE port node
	 */
	case NPF_IF_TYPE_IPV4:
	case NPF_IF_TYPE_IPV6:
   case NPF_IF_TYPE_MPLS:
	case NPF_IF_TYPE_HIGHEST:
	{
    	    NPF_NSHDL 		hFEportInst;
    	    NPF_NSHDL 		hL3nodeInst;
	    char		port[MAX_INTERFACE_LEN];
	    uint32_t		count;

	    IF_TRACE("Child has (%d) Associations with FE Port", l2Iface->count);
    	    for(count = 0; count < l2Iface->count; count++)
    	    {
	  	hL3nodeInst = l3Iface->inode;
                hFEportInst = l2Iface->list[count];
		IF_INFO("hFEportInst = %d", hFEportInst);
		if(npf_ns_getPath(hFEportInst, port) != NPF_SUCCESS)
		{
		    IF_ERROR("***** Invalid L2 Port, associated, or No L2Iface Port Set ***** ");
    		    npf_ns_close(l3Iface->inode);
	    	    return NPF_E_UNKNOWN;
		}
    	        npf_ns_open(port, &pnode);
		IF_INFO("Associating \"%s(%d, %d)\" and \"%s\"(%d, %d)", 
			l3Iface->path, l3Iface->nsHandle, l3Iface->inode, 
			port, pnode, hFEportInst);
		if(npf_ns_addAssociate(l3Iface->inode, hFEportInst) != NPF_SUCCESS)
		{
		    IF_ERROR("*** Association for \"%s\" and \"%s\" failed ***", 
						l3Iface->path, port);
    		    npf_ns_close(pnode);
    		    npf_ns_close(l3Iface->inode);
    		    return NPF_E_UNKNOWN;
		}
		if(npf_ns_addAssociate(hFEportInst, l3Iface->inode) != NPF_SUCCESS)
		{
		    IF_ERROR("*** Association for \"%s\" and \"%s\" failed ***", 
						port, l3Iface->path);
    		    npf_ns_close(pnode);
    		    npf_ns_close(l3Iface->inode);
    		    return NPF_E_UNKNOWN;
		}
		IF_INFO("\"%s(%d, %d)\" <===> \"%s\"(%d, %d)", 
			l3Iface->path, l3Iface->nsHandle, l3Iface->inode, 
			port, pnode, hFEportInst);
    	    } 
    	    npf_ns_close(pnode);
    	    npf_ns_close(l3Iface->inode);
	    return(NPF_NO_ERROR);
	}
	break;
	default:
	{
	    IF_ERROR("*** Invalid Iface type ***");
	    return NPF_E_UNKNOWN;
	}
    }
    npf_ns_close(l3Iface->inode);
    return NPF_E_UNKNOWN;
}

/****************************************************************************
 * Function: associateL2IfandFEPort()                                       *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		ifc - interface data				    	    *
 *		iface - Generic interface data as defined by NPF	    *
 *  							    		    *
 * Description: 							    *
 *             	This function associates namespace with l2 interface to     *
 *		logical port which is associated to FE/<no>/PORT/<no>	    *
 *             	Always retuns NPF_NO_ERROR or NPF_E_UNKNOWN to the caller.  *
 * Return: 								    *
 *		NPF_NO_ERROR  - On no error				    *
 *		NPF_E_UNKNOWN  - On error				    *
 ****************************************************************************/
NPF_error_t associateL2IfandFEPort(If_t  *ifc, NPF_IfGeneric_t *iface)
{
    NPF_NSHDL           pnode;
    NPF_NSHDL           ifpnode;
    char                ifp[MAX_INTERFACE_LEN];
    char                port[MAX_INTERFACE_LEN];
    NPF_NSHDL 		hIfPortInst;
    NPF_NSHDL 		portNode;
    NODE_ITERATOR 	assocItr;
    uint32_t 		count;
    int 		numAssoc;
    NPF_DATATYPE	type;

    IF_FUNCTION();
    switch(iface->type)
    {
	case NPF_IF_TYPE_LAN:
	case NPF_IF_TYPE_ATM:
	case NPF_IF_TYPE_POS:
	{
    	    IF_INFO("Setting Attribute of interface no : %d, type \"NPF_IF_TYPE_LAN\"", ifc->no);
    	    sprintf(ifp, "%s/%d", INTERFACE_PORT_ROOT, iface->u.LAN_Attr.port);
    	    IF_INFO("IfPort Path = %s", ifp);
	    if(npf_ns_open(ifp, &hIfPortInst) != NPF_SUCCESS)
	    {
	        IF_ERROR("*** No IfacePort found %s ***", ifp);
	        return NPF_E_UNKNOWN;
	    }
	    if(npf_ns_getDataHandle(hIfPortInst, (NPF_HANDLE *)&ifpnode) != NPF_SUCCESS)
	    {
	        IF_ERROR("*** No IfacePort found %s ***", ifp);
	        return NPF_E_UNKNOWN;
	    }
    	    IF_INFO("IfPort Instance = %d, node handle = %d", hIfPortInst, ifpnode);
	    /*
	     * Associate If node and IfPort node
	     */
            if(npf_ns_addAssociate(ifc->inode, hIfPortInst) != NPF_SUCCESS)
	    {
	        IF_ERROR("*** Can't associate Iface and IfPort nodes ***");
	        return NPF_E_UNKNOWN;
	    }
	    IF_INFO("Iface \"%s\" and \"%s\" Associated ......", ifc->path, ifp);

	    /*
	     * Get associations for IfPort Node, where config application
	     * made associations between IfPort nodes and FE Port nodes
	     */
	    if(npf_ns_enumAssociate(hIfPortInst, &assocItr) != NPF_SUCCESS)
	    {
	        IF_ERROR("*** unable to enumerate associations for node : %d\n *** ", hIfPortInst);
	        return NPF_E_UNKNOWN;
	    }
            if(!assocItr) return NPF_E_UNKNOWN;
	    numAssoc = npf_ns_count(assocItr);
	    IF_INFO("L2 Interface has (%d) Assosiations with FE Port", numAssoc);
	    portNode = npf_ns_first(assocItr);
	    count = 0;
    	    do {
		npf_ns_getPath(portNode, port);
		npf_ns_getDataType(portNode, &type);
	        npf_ns_getDataHandle(hIfPortInst, (NPF_HANDLE *)&pnode);
        	IF_INFO("FE Port Node = %d, type = %d handle = %d", portNode, type, pnode);
		if(type == NPF_PORT)
		{
		    ifc->list[count] = portNode;
	    	    IF_INFO("%s(%d, %d)  <==> %s(%d, %d)", 
			ifc->path, ifc->nsHandle, ifc->inode, port, pnode, portNode);
		    if(npf_ns_addAssociate(ifc->inode, portNode) != NPF_SUCCESS)
		    {
			IF_ERROR("*** Association for \"%s\"(%d) and \"%s\"(%d) failed ***", 
							ifc->path, ifc->inode, port, portNode);
    			return NPF_E_UNKNOWN;
		    }
		    if(npf_ns_addAssociate(portNode, ifc->inode) != NPF_SUCCESS)
		    {
			IF_ERROR("*** Association for \"%s\"(%d) and \"%s\"(%d) failed ***", 
							ifc->path, port, portNode, ifc->inode);
    			return NPF_E_UNKNOWN;
		    }
		}
		count ++;
    	    } while ((portNode = npf_ns_next(assocItr)) != 0);
	    if(count > IF_MAX_ASSOCS)
	    {
		IF_ERROR("count (%d) > IF_MAX_ASSOCS(%d)", count, IF_MAX_ASSOCS);
		return NPF_E_UNKNOWN;
	    }
	    ifc->count = count;
            npf_ns_done(assocItr);
	    ifc->npfIf.u.LAN_Attr.port = iface->u.LAN_Attr.port;    
	}
	break;

	default:
	{
	    IF_TRACE("*** Iface %s [ %d ] Not associated with FE port(s)***", ifc->path, iface->type);
	    return (NPF_NO_ERROR);
	}
    }
    return (NPF_NO_ERROR);
}

/****************************************************************************
 * Function: IfCreateAndSet()                                               *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		gen - Generic interface defined by NPF			    *
 *		req - request_t data for application's api request	    *
 *  							    		    *
 * Description: 							    *
 *             	This function implements NPF_IfCreateAndSet		    *
 *             	Always returns NPF_NO_ERROR or NPF_E_UNKNOWN to the caller. *
 *		Corresponding error/success is notified through 	    *
 *		Asyncresponse callback		    			    *
 * Return: 								    *
 *		NPF_NO_ERROR  - On no error				    *
 *		NPF_E_UNKNOWN  - On error				    *
 ****************************************************************************/
NPF_RET IfCreateAndSet(NPF_IfGeneric_t *gen, request_t *req)
{
    If_t		*iface;
    NPF_NSNODETYPE      nodeType;
    NPF_DATATYPE        dataType;
    NPF_NSHDL           hInstance;
    NPF_NSHDL           hNode;
    NPF_RET             result;
    NPF_IfType_t        type;

    IF_FUNCTION();
    IF_LOCK();
    nodeType = NPF_NS_INSTNODE;
    dataType = NPF_INTERFACE;
    hInstance = 0;

    type = gen->type;
    iface = (If_t *)PIL_AllocateMemory(sizeof(If_t),PIL_MEM_ZERO_INIT);
    if(iface == NULL)
    {
	IF_ERROR("***** %s", "iface == NULL");
	req->error = NPF_IF_E_NOT_CREATED;
	req->ihandle = 0;
	onResponse(req, NULL);
        IF_UNLOCK();
	return NPF_E_UNKNOWN;
    }
    sprintf(iface->path, "%s/%d", interfaceRoot, (int)interfaceCount);
    iface->npfIf.type = type;
    iface->npfIf.speed = 0;
    iface->l2If = NULL;
    iface->l3If = NULL;
    IF_INFO("Creating interface type = %d and path = %s", type, iface->path);
    /* check if IF type node exists */
    result = npf_ns_open(iface->path, &hNode);
    if(result == NPF_SUCCESS)
    {
        npf_ns_close(hNode);
        IF_INFO("%s Exists, Sending response NPF_IF_E_NOT_CREATED ...", iface->path);
        PIL_FreeMemory(iface);
        req->error = NPF_IF_E_NOT_CREATED;
        req->ihandle = 0;
        onResponse(req, NULL);
        IF_UNLOCK();
        return(NPF_NO_ERROR);
    }
    else
    {
        IF_INFO("%s Not exists, Creating it ...", iface->path);
        if(npf_ns_create(iface->path, nodeType, dataType, hInstance, &hNode) != NPF_SUCCESS)
        {
            IF_ERROR("***** %s", "npf_ns_create returns NPF_IF_E_NOT_CREATED");
            PIL_FreeMemory(iface);
            req->error = NPF_IF_E_INVALID_PARAM;
            req->ihandle = 0;
            onResponse(req, NULL);
            IF_UNLOCK();
            return(NPF_NO_ERROR);
        }
    }
    npf_ns_getDataHandle(hNode, (NPF_HANDLE *)&iface->nsHandle);
    iface->inode = hNode;
    iface->no = interfaceCount++;
    iface->operStat = NPF_IF_OPER_STATUS_DOWN;
    npf_list_pushBack(&IfList, iface);
    IF_TRACE("%s  = {ns_handle= %d, instance %d}", iface->path, hNode, hInstance);
    req->error = NPF_NO_ERROR;
    req->ihandle = iface->nsHandle;
    IF_UNLOCK();
    IfAttrSet(iface->nsHandle, gen, req);
    return NPF_NO_ERROR;
}

/****************************************************************************
 * Function: IfAttrSet()                                                    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		ifHandle - interface handle				    *
 *		iface - Generic interface structure defined by NPF	    *
 *		req - request_t data for application's api request	    *
 *  							    		    *
 * Description: 							    *
 *             	This function sets an attributes of an interface	    *
 *		Set Interface attributes				    *
 *		Make name space associations for port attribute		    *
 *		Invoke FPP API for other attributes			    *
 *		NPF_IF_TYPE_LAN: port, promisc, speed			    *
 *		NPF_IF_TYPE_ATM: port					    *
 *		NPF_IF_TYPE_POS: port					    *
 *		NPF_IF_TYPE_IPV4: addr, mtu, FIB_Handle			    *
 *		NPF_IF_TYPE_IPV6: n_addr, addr, zoneTable, mtu, FIB_Handle  *
 *		NPF_IF_TYPE_IPV6INV4: tunnelType, IPv4DstAddr, IPv4SrcAddr, *
 *							IPv4TTL, IPv4DSCP   *
 *             	Always returns NPF_NO_ERROR or NPF_E_UNKNOWN to the caller. *
 *		Corresponding error/success is notified through 	    *
 *		Asyncresponse callback		    			    *
 * Return: 								    *
 *		NPF_NO_ERROR  - On no error				    *
 *		NPF_E_UNKNOWN  - On error				    *
 ****************************************************************************/
NPF_RET IfAttrSet(NPF_IfHandle_t ifHandle, NPF_IfGeneric_t *iface, request_t *req)
{
    If_t	*ifc;
    IF_FUNCTION();
    IF_LOCK();
    if(getInterface(ifHandle, &ifc) != NPF_NO_ERROR)
    {
	req->error = NPF_IF_E_INVALID_HANDLE;
	req->ihandle = ifHandle;
	onResponse(req, NULL);
        IF_UNLOCK();
	return NPF_E_UNKNOWN;
    }
    /*
     *Associate If node with FE Port Node
     */
    if(associateL2IfandFEPort(ifc, iface) != NPF_NO_ERROR)
    {
        IF_ERROR("*** Can't associate L2 If and FE Port ***");
	req->error = NPF_IF_E_INVALID_PORT_NUMBER;
	req->ihandle = ifHandle;
	onResponse(req, NULL);
        IF_UNLOCK();
        return NPF_E_UNKNOWN;
    }
    /*
     * Get FE and Port Information for L2 Interface
    */
    IF_TRACE("Getting FE and PORT Information for : %s(%d)", ifc->path,  ifc->nsHandle);
    if(getFEandPORTdata(ifc) != NPF_NO_ERROR)
    {
        IF_ERROR("*** Can't Find FE and PORT Information for : %s(%d)", ifc->path,  ifc->nsHandle);
        req->error = NPF_IF_E_INVALID_HANDLE;
        onResponse(req, NULL);
        IF_UNLOCK();
        return (NPF_E_UNKNOWN);
    }
    switch(ifc->npfIf.type)
    {
	case NPF_IF_TYPE_IPV4: /* Set addr, mtu, FIB_Handle */
	{
	    IF_TRACE("Setting Attributes for: NPF_IF_TYPE_IPV4");
    	    req->fpReq = 2*ifc->FE.count;/* We are making two requests per interface */
    	    setL3Property(req, ifc, NPF_L3ATTRS_IPADDR, &iface->u.IPv4_Attr.addr, ifFPsetIPaddrCallbackHandle);
    	    setL2Property(req, ifc, NPF_L2ATTRS_MTU, &iface->u.IPv4_Attr.mtu, ifFPsetMTUCallbackHandle);
          if(IfIPv4FibSet(ifHandle, iface->u.IPv4_Attr.FIB_Handle,req) !=
            NPF_SUCCESS)
          {
            IF_UNLOCK();
            return (NPF_E_UNKNOWN);
          }
	    ifc->npfIf.u.IPv4_Attr.mtu = iface->u.IPv4_Attr.mtu;
	}
	break;
	case NPF_IF_TYPE_IPV6: /*n_addr, addr, zoneTable, mtu, FIB_Handle*/
	{
	    IF_TRACE("Setting Attributes for: NPF_IF_TYPE_IPV6");
    	    req->fpReq = 2*ifc->FE.count;/* We are making two requests per interface */
    	    setL3Property(req, ifc, NPF_L3ATTRS_IPADDR, &iface->u.IPv6_Attr.addr, ifFPsetIPaddrCallbackHandle);
    	    setL2Property(req, ifc, NPF_L2ATTRS_MTU, &iface->u.IPv4_Attr.mtu, ifFPsetMTUCallbackHandle);
	    ifc->npfIf.u.IPv4_Attr.mtu = iface->u.IPv4_Attr.mtu;
	}
	break;
	case NPF_IF_TYPE_IPV6INV4:
	{
    	    req->error = NPF_IF_E_INVALID_HANDLE;
    	    onResponse(req, &req->ihandle);
	}
	break;
	case NPF_IF_TYPE_LAN: /* Set speed, promisc, status. speed ??? */
	{
	    IF_TRACE("Setting Attributes for: NPF_IF_TYPE_LAN");
    	    req->fpReq = 1*ifc->FE.count;/* We are making two requests per interface */
    	    setL2Property(req, ifc, NPF_L2ATTRS_LINKSPEED, &iface->u.LAN_Attr.speed, ifFPsetLANSpeedCallbackHandle);
	}
	break;
    case NPF_IF_TYPE_MPLS: /* Set Labelled Pkt Size */
    {
       TRACE("Setting Attributes for: NPF_IF_TYPE_MPLS");
               req->fpReq = 1*ifc->FE.count;
       setL2Property(req, ifc, NPF_L2ATTRS_MTU, &iface->u.MPLS_Attr.mtuSize,
                          ifFPsetMTUCallbackHandle);
       ifc->npfIf.u.MPLS_Attr.mtuSize = iface->u.MPLS_Attr.mtuSize;
     }
     break;
#ifdef _INCLUDE_ATM
	case NPF_IF_TYPE_ATM:
   {
       TRACE("Setting Attributes for: NPF_IF_TYPE_ATM");
               req->fpReq = 1*ifc->FE.count;
		IfATM_IfAttrSet(ifc,iface, req);
   }
      break;
#endif /* _INCLUDE_ATM */

	default:
	{
    	    req->error = NPF_NO_ERROR;
    	    onResponse(req, NULL);
	}
    }
    IF_UNLOCK();
    return (NPF_NO_ERROR);
}


/****************************************************************************
 * Function: IfDelete()                                                     *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		ifHandle - interface handle				    *
 *		req - request_t data for application's api request	    *
 *  							    		    *
 * Description: 							    *
 *             	This function implements NPF_IfDelete		    	    *
 *             	Always returns NPF_NO_ERROR or NPF_E_UNKNOWN to the caller. *
 *		Corresponding error/success is notified through 	    *
 *		Asyncresponse callback		    			    *
 * Return: 								    *
 *		NPF_NO_ERROR  - On no error				    *
 *		NPF_E_UNKNOWN  - On error				    *
 ****************************************************************************/

NPF_error_t IfDelete(NPF_IfHandle_t   ifHandle, request_t *req)
{
    If_t                *iface = NULL;
    If_t                ifaceKey;

    ifaceKey.nsHandle = ifHandle;

    /*
     * Destroy interface parents and child list
     */
    IF_FUNCTION();
#ifdef _INCLUDE_ATM
	IF_LOCK();
	if(getInterface(ifHandle, &iface) != NPF_NO_ERROR)
	{
		req->error = NPF_IF_E_INVALID_HANDLE;
		req->ihandle = ifHandle;
		onResponse(req, NULL);
		IF_UNLOCK();
		return NPF_E_UNKNOWN;
	}
	IF_UNLOCK();
    /* Call the ATM FPPAPI function to delete the port in FE */
    if(iface->npfIf.type == NPF_IF_TYPE_ATM)
    {
		IfATM_IfDelete(iface, req);	
    }
	 /* the interface type is not ATM */
	 else
	 {
	    IF_LOCK();
   	 if ((npf_list_removeByKey(&IfList, &ifHandle, (void **)&iface)) == -1)
	    {
   	     IF_UNLOCK();
      	  IF_ERROR("*** No Such Interface Handle : %d ***", ifHandle);
	        req->error = NPF_IF_E_INVALID_HANDLE;
   	     req->ihandle = ifHandle;
      	  onResponse(req, NULL);
	        IF_UNLOCK();
   	     return (NPF_E_UNKNOWN);
	    }

   	 /*
	     * Delete ports list allocated during interface bind
   	  */
	    npf_ns_close(iface->nsHandle); /* Check for success */
   	 npf_ns_delete(iface->path); /* Check for success */
	    if(iface->PORT.portsList != NULL && iface->PORT.portsList->ports != NULL)
   	     PIL_FreeMemory(iface->PORT.portsList->ports);
	    if(iface->PORT.portsList != NULL)
   	     PIL_FreeMemory(iface->PORT.portsList);
	    if(iface != NULL)
   	     PIL_FreeMemory(iface);

	    IF_UNLOCK();

   	 req->error = NPF_NO_ERROR;
	    req->ihandle = ifHandle;
   	 onResponse(req, NULL);
	    IF_UNLOCK();
	 }
#else
    IF_LOCK();
    if ((npf_list_removeByKey(&IfList, &ifHandle, (void **)&iface)) == -1)
    {
        IF_UNLOCK();
        IF_ERROR("*** No Such Interface Handle : %d ***", ifHandle);
        req->error = NPF_IF_E_INVALID_HANDLE;
        req->ihandle = ifHandle;
        onResponse(req, NULL);
        IF_UNLOCK();
        return (NPF_E_UNKNOWN);
    }

    /*
     * Delete ports list allocated during interface bind
     */
    npf_ns_close(iface->nsHandle); /* Check for success */
    npf_ns_delete(iface->path); /* Check for success */
    if(iface->PORT.portsList != NULL && iface->PORT.portsList->ports != NULL)
        PIL_FreeMemory(iface->PORT.portsList->ports);
    if(iface->PORT.portsList != NULL)
        PIL_FreeMemory(iface->PORT.portsList);
    if(iface != NULL)
        PIL_FreeMemory(iface);

    IF_UNLOCK();

    req->error = NPF_NO_ERROR;
    req->ihandle = ifHandle;
    onResponse(req, NULL);
    IF_UNLOCK();
 
#endif /*  _INCLUDE_ATM */

    return NPF_NO_ERROR;
}

/****************************************************************************
 * Function: onResponse()                                                   *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		request - request_t data for application's api request	    *
 *		resp - Response data 					    *
 *  							    		    *
 * Description: 							    *
 *             	This function collects async response data of each request  *
 *             	Prepares NPF_IfAsyncResponse_t, and stores in request state *
 *             	till no of requests equals no of responses. When these two  *
 *		equal it invokes application's callback based on err. report*
 *		registered.						    *
 * Return: 								    *
 *		NPF_NO_ERROR  - On no error				    *
 *		NPF_E_UNKNOWN  - On error				    *
 ****************************************************************************/
NPF_error_t onResponse(request_t *request, void *resp)
{
    NPF_IfCallbackType_t	type;
    NPF_error_t			error;
    NPF_IfHandle_t      	handle;
    NPF_IfAsyncResponse_t 	*asyncResp;

    IF_FUNCTION();
    
    type = request->type;
    error = request->error;
    handle = request->ihandle;
    
    asyncResp = (NPF_IfAsyncResponse_t *)PIL_AllocateMemory\
		(sizeof(NPF_IfAsyncResponse_t), PIL_MEM_ZERO_INIT);
    if(asyncResp == NULL)
    {
	IF_ERROR("***** %s", "PIL_AllocateMemory Failed");
        return NPF_E_UNKNOWN;
    }
    /* This will be freed in invokeCallback function*/
    request->asyncResp[request->resp] = asyncResp;
    asyncResp->error = error;
    asyncResp->ifHandle = handle;
    if(request->fpReq == 0)
        request->resp = request->resp + 1; /* Update no of responses */
    else if(request->fpResp >= request->fpReq)
    {
        request->resp = request->resp + 1; /* Update no of responses */
    }
    if(error != NPF_NO_ERROR)
    {
	request->data.allOK = NPF_FALSE;
    }
    IF_INFO("Callback Type = %d IF HANDLE= %d, IF_ERROR= %d", type, handle, error);
    if(error == NPF_NO_ERROR && resp != NULL)
    {
        switch(type)
        {
	    case NPF_IF_OPER_STATUS_GET:
	    {
	        memcpy(&asyncResp->u.operStat, 
			(NPF_IfOperStatus_t *)resp, 
			sizeof(NPF_IfOperStatus_t));
	    }
	    break;
	    case NPF_IF_BIND:
	    {
	        memcpy(&asyncResp->u.child, 
			(NPF_IfHandle_t *)resp, 
			sizeof(NPF_IfHandle_t));
	    }
	    break;
	    case NPF_IF_ATM_VCC_BIND:
	    case NPF_IF_ATM_VCC_SET:
	    case NPF_IF_ATM_VCC_DELETE:
	    {
	        memcpy(&asyncResp->u.VccAddr, 
			(NPF_VccAddr_t *)resp, 
			sizeof(NPF_VccAddr_t));
	    }
	    break;
	    case NPF_IF_STATS_GET:
	    {
          asyncResp->u.ifStats = (NPF_IfStatistics_t *)PIL_AllocateMemory\
             		(sizeof(NPF_IfStatistics_t), PIL_MEM_ZERO_INIT);
          if(asyncResp->u.ifStats == NULL)
          {
         	IF_ERROR("***** %s", "PIL_AllocateMemory Failed");
            return NPF_E_UNKNOWN;
          }
	        memcpy(asyncResp->u.ifStats,resp,sizeof(NPF_IfStatistics_t));
	    }
	    break;
	    case NPF_IF_VCC_STATS_GET:
	    {
          asyncResp->u.vccStats = (NPF_ATM_VccStats_t *)PIL_AllocateMemory\
             		(sizeof(NPF_ATM_VccStats_t), PIL_MEM_ZERO_INIT);
          if(asyncResp->u.vccStats == NULL)
          {
         	IF_ERROR("***** %s", "PIL_AllocateMemory Failed");
            return NPF_E_UNKNOWN;
          }
	        memcpy(asyncResp->u.vccStats,resp,sizeof(NPF_ATM_VccStats_t));
	    }
	    break;
	    default:
	    {
    	        IF_INFO("union in async response not used for this type");
	    }
	}
    }
    else if(type == NPF_IF_CREATE_AND_SET && error != NPF_NO_ERROR)
    {
        If_t	*ifc;
        if(getInterface(handle, &ifc) != NPF_NO_ERROR)
        {
            /*
             * Delete ports list allocated during interface bind
            */
	    if(ifc != NULL)
	    {
                PIL_FreeMemory(ifc->PORT.portsList);
                npf_ns_close((NPF_NSHDL)&ifc->nsHandle); /* Check for success */
                npf_list_removeByKeyFree(&IfList, &ifc->nsHandle);
	    }
         }
    }
    if(request->no == request->resp)
    {
        IF_INFO("request->no[%d] == request->resp[%d], Invoking Callback",
							 request->no, request->resp);
        invokeCallback(request);
    }
    else
    {
        /* invokeCallback frees the request when it is done with it
           so trying to use it here is a bad thing! */
        IF_INFO("No of requests= %d, No of Responses= %d", request->no, request->resp);
    }
    return NPF_NO_ERROR;
}

/****************************************************************************
 * Function: invokeCallback()                                               *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		response - request_t data for application's api request	    *
 *  							    		    *
 * Description: 							    *
 *             	This function makes single NPF_IfAsyncResponse_t by 	    *
 *             	aggregating individual NPF_IfAsyncResponse_t in request_t   *
 *             	till no of requests equals no of responses. 		    *
 *		Finds the application callback and invokes it with single   *
 *		NPF_IfAsyncResponse_t.					    *
 * Return: 								    *
 *		NPF_NO_ERROR  - On no error				    *
 *		NPF_E_UNKNOWN  - On error				    *
 ****************************************************************************/
NPF_error_t invokeCallback(request_t *response)
{
    NPF_IfCallbackData_t  *data;
    NPF_IfAsyncResponse_t *asyncResp;
    NPF_userContext_t 	  context;
    uint32_t		  count;
    IF_FUNCTION();
    data = &response->data;
    response->data.n_resp = response->resp;
    IF_INFO("type= %d, allOK= %d, n_resp=%d", data->type, data->allOK, data->n_resp);
    /* Allocate memory = total no of responses */
    if ((asyncResp = (NPF_IfAsyncResponse_t *)PIL_AllocateMemory
            (response->resp*sizeof(NPF_IfAsyncResponse_t), PIL_MEM_ZERO_INIT)) == NULL) 
    {
	IF_ERROR("****** PIL_AllocateMemory for NPF_IfAsyncResponse_t Failed *****");
        return (NPF_E_UNKNOWN);
    }
    for(count = 0; count < response->resp; count ++)
    {
	if(response->asyncResp[count] != NULL)
	{
	     memcpy(&asyncResp[count], 
			response->asyncResp[count], 
			sizeof(NPF_IfAsyncResponse_t));
	     PIL_FreeMemory(response->asyncResp[count]);
	}
	else
	{
	    IF_ERROR("***** Something wrong, ..............................");
	}
    }
    IF_TRACE("callback data ready .................");
    
    if(response->callback == NULL)
    {
	IF_ERROR("***** No Callback found *****");
        return (NPF_E_UNKNOWN);
    }
    data->resp = asyncResp;
    cbthread_pushIFAPICB(response->callback, context, response->corr, data);
    npf_list_removeByKeyFree(&RequestList, response);
    return NPF_NO_ERROR;
}

/****************************************************************************
 * Function: getInterface()                                                 *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		ifHandle - interface handle				    *
 *		iface - local interface data				    *
 *  							    		    *
 * Description: 							    *
 *             	This function searches for local interface data for given   *
 *             	interface handle					    *
 * Return: 								    *
 *		NPF_NO_ERROR  - Interface data found for ifHandle	    *
 *		NPF_E_UNKNOWN  - Interface data not found for ifHandle	    *
 ****************************************************************************/
NPF_error_t getInterface(NPF_IfHandle_t ifHandle, If_t  **iface)
{
    DListItr		*itr;
    NPF_boolean_t       found;
    If_t		*data;

    found = NPF_FALSE;
    itr = npf_list_itrCreate (&IfList);
    if (itr == NULL)
    {
        IF_ERROR("%s [ %d ]", "itr == NULL for interface handle", ifHandle);
        return NPF_E_UNKNOWN;
    }
    npf_list_itrFirst (itr);
    do
    {
        data = (If_t *)npf_list_itrGetData (itr);
        assert (data);
        if (data->nsHandle == ifHandle)
        {
	    *iface = data;
            npf_list_itrDelete (itr);
            return(NPF_NO_ERROR);
        }
    } while (npf_list_itrNext (itr) != -1);
    npf_list_itrDelete (itr);
    return(NPF_E_UNKNOWN);
}

/****************************************************************************
 * Function: IfEnable()                                               	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		type - interface handle					    *
 *		req - request_t data for application's api request	    *
 *  							    		    *
 * Description: 							    *
 *             	This function implements NPF_IfEnable		    	    *
 *             	Always returns NPF_NO_ERROR or NPF_E_UNKNOWN to the caller. *
 *		Corresponding error/success is notified through 	    *
 *		Asyncresponse callback		    			    *
 * Return: 								    *
 *		NPF_NO_ERROR  - On no error				    *
 *		NPF_E_UNKNOWN  - On error				    *
 ****************************************************************************/
NPF_error_t IfEnable(NPF_IfHandle_t   ifHandle, request_t *req)
{
    If_t	*ifc;
    uint32_t    status;
    IF_FUNCTION();
    if(getInterface(ifHandle, &ifc) != NPF_NO_ERROR)
    {
	req->error = NPF_IF_E_INVALID_HANDLE;
	req->ihandle = ifHandle;
	onResponse(req, NULL);
	return NPF_E_UNKNOWN;
    }
#ifdef _INCLUDE_ATM
   if(ifc->npfIf.type != NPF_IF_TYPE_ATM)
	{		
    status = NPF_IF_UP;
    setL2Property(req, ifc, NPF_L2ATTRS_STATUS, &status, ifFPsetStatusCallbackHandle);
	}
   else
	{
		IfATM_IfEnable(ifc,req);
	}
#else
    status = NPF_IF_UP;
    setL2Property(req, ifc, NPF_L2ATTRS_STATUS, &status, ifFPsetStatusCallbackHandle);
#endif /* _INCLUDE_ATM */

    return(NPF_NO_ERROR);
}

/****************************************************************************
 * Function: IfDisable()                                               	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		type - interface handle					    *
 *		req - request_t data for application's api request	    *
 *  							    		    *
 * Description: 							    *
 *             	This function implements NPF_IfDisable		    	    *
 *             	Always returns NPF_NO_ERROR or NPF_E_UNKNOWN to the caller. *
 *		Corresponding error/success is notified through 	    *
 *		Asyncresponse callback		    			    *
 * Return: 								    *
 *		NPF_NO_ERROR  - On no error				    *
 *		NPF_E_UNKNOWN  - On error				    *
 ****************************************************************************/
NPF_error_t IfDisable(NPF_IfHandle_t   ifHandle, request_t *req)
{
    If_t	*ifc;
    uint32_t    status;
    IF_FUNCTION();
    status = NPF_IF_DOWN;
    if(getInterface(ifHandle, &ifc) != NPF_NO_ERROR)
    {
	req->error = NPF_IF_E_INVALID_HANDLE;
	req->ihandle = ifHandle;
	onResponse(req, NULL);
	return NPF_E_UNKNOWN;
    }
#ifdef _INCLUDE_ATM
	if(ifc->npfIf.type != NPF_IF_TYPE_ATM)
   {
    setL2Property(req, ifc, NPF_L2ATTRS_STATUS, &status, ifFPsetStatusCallbackHandle);
   }
	else
	{
		IfATM_IfDisable(ifc,req);
	}
#else
    setL2Property(req, ifc, NPF_L2ATTRS_STATUS, &status, ifFPsetStatusCallbackHandle);
#endif /* _INCLUDE_ATM */

    return(NPF_NO_ERROR);
}

/****************************************************************************
 * Function: IfOperStatusGet()                                              *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		type - interface handle					    *
 *		req - request_t data for application's api request	    *
 *  							    		    *
 * Description: 							    *
 *             	This function implements NPF_IfOperStatusGet		    *
 *             	Always returns NPF_NO_ERROR or NPF_E_UNKNOWN to the caller. *
 *		Corresponding error/success is notified through 	    *
 *		Asyncresponse callback		    			    *
 * Return: 								    *
 *		NPF_NO_ERROR  - On no error				    *
 *		NPF_E_UNKNOWN  - On error				    *
 ****************************************************************************/
NPF_error_t IfOperStatusGet(NPF_IfHandle_t   ifHandle, request_t *req)
{
    If_t	*ifc;
    NPF_IfOperStatus_t operStat;
    IF_LOCK();
    IF_FUNCTION();
    if(getInterface(ifHandle, &ifc) != NPF_NO_ERROR)
    {
	req->error = NPF_IF_E_INVALID_HANDLE;
	req->ihandle = ifHandle;
	onResponse(req, NULL);
        IF_UNLOCK();
	return NPF_E_UNKNOWN;
    }
    operStat = ifc->operStat;
    req->error = NPF_NO_ERROR;
    onResponse(req, &operStat);
    IF_UNLOCK();
    return NPF_NO_ERROR;
}

/****************************************************************************
 * Function: validRelation()                                                *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		parent - interface data for parent interface type	    *
 *		child - interface data for child interface type	    	    *
 *  							    		    *
 * Description: 							    *
 *             	This function is called during Interface Bind operation	    *
 *             	Checks for parent and child interface types and associates  *
 *             	namespace data with child interface in case of parent 	    *
 *             	parent interface is of type IPV4/IPV6			    *
 *		Corresponding error/success is notified through 	    *
 *		Asyncresponse callback		    			    *
 * Return: 								    *
 *		NPF_NO_ERROR  - On no error				    *
 *		NPF_E_UNKNOWN  - On error				    *
 ****************************************************************************/
NPF_error_t validRelation(If_t *parent, If_t *child)
{
    IF_FUNCTION();
    /*
     * Check Possible parent and child combinations
     * Now only NPF_IF_TYPE_IPV4 over NPF_IF_TYPE_LAN and
     * NPF_IF_TYPE_IPV4 over NPF_IF_TYPE_POS allowed
     */
    switch(parent->npfIf.type)
    {
	case NPF_IF_TYPE_IPV4:
	case NPF_IF_TYPE_IPV6:
	case NPF_IF_TYPE_IPV6INV4:
   case NPF_IF_TYPE_MPLS:
	{
	    IF_INFO("PARENT :	NPF_IF_TYPE_IPV4");
	    switch(child->npfIf.type)
	    {
		case NPF_IF_TYPE_LAN:
		{
	    	    IF_INFO("CHILD :	NPF_IF_TYPE_LAN");
		    if(associateL3IfandFEPort(parent, child) != NPF_NO_ERROR)
		    {
			IF_ERROR("*** Can't find FE Port relation, may not L2 Port set ***");
		    	return NPF_E_UNKNOWN;
		    }
		    return NPF_NO_ERROR;
		}
		break;
		case NPF_IF_TYPE_POS:
		{
	    	    IF_INFO("CHILD :	NPF_IF_TYPE_POS");
		    if(associateL3IfandFEPort(parent, child) != NPF_NO_ERROR)
		    {
			IF_ERROR("*** Can't find FE Port relation, may not L2 Port set ***");
		    	return NPF_E_UNKNOWN;
		    }
		    return NPF_NO_ERROR;
		}
		break;
		case NPF_IF_TYPE_ATM:
		{
	    	    IF_INFO("CHILD :	NPF_IF_TYPE_ATM");
		    if(associateL3IfandFEPort(parent, child) != NPF_NO_ERROR)
		    {
			IF_ERROR("*** Can't find FE Port relation, may not L2 Port set ***");
		    	return NPF_E_UNKNOWN;
		    }
		    return NPF_NO_ERROR;
		}
		break;
	   	default:
		{
		    IF_ERROR("*** Invalid Parent(parent->nsHandle) Child(%d) ***", 
					parent->nsHandle, child->nsHandle);
		    return NPF_E_UNKNOWN;
	    	}
	    }
	}
	break;
	case NPF_IF_TYPE_ATM:
	{
	    IF_ERROR("*** PARENT :	NPF_IF_TYPE_ATM not allowed ***");
	    return NPF_E_UNKNOWN;
	}
	break;
	default:
	{
	    IF_ERROR("*** PARENT(%d) :	UNKNOWN, not allowed ***", parent->nsHandle);
	    return NPF_E_UNKNOWN;
	}
    }
    return NPF_E_UNKNOWN;
}

/****************************************************************************
 * Function: IfBind()                                               	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		ifBind - NPF_IfBinding_t defined by NPF			    *
 *		req - request_t data for application's api request	    *
 *  							    		    *
 * Description: 							    *
 *             	This function implements NPF_IfBind		    	    *
 *             	Always returns NPF_NO_ERROR or NPF_E_UNKNOWN to the caller. *
 *		Corresponding error/success is notified through 	    *
 *		Asyncresponse callback		    			    *
 * Return: 								    *
 *		NPF_NO_ERROR  - On no error				    *
 *		NPF_E_UNKNOWN  - On error				    *
 ****************************************************************************/
NPF_error_t IfBind(NPF_IfBinding_t   *ifBind, request_t *req)
{
    If_t	*parent = NULL;
    If_t	*child = NULL;
    
    IF_FUNCTION();
    IF_INFO("PARENT : %d, CHILD : %d", ifBind->parent, ifBind->child);
    if(ifBind->parent == ifBind->child)
    {
	IF_ERROR("*** Parent can't bind itself ***");
	req->error = NPF_IF_E_INVALID_CHILD_HANDLE;
	req->ihandle = ifBind->parent;
	onResponse(req, NULL);
	return NPF_E_UNKNOWN;
    }
    if(getInterface(ifBind->parent, &parent) != NPF_NO_ERROR)
    {
	IF_ERROR("No Parent Iface found for handle : %d", ifBind->parent);
	req->error = NPF_IF_E_INVALID_HANDLE;
	req->ihandle = ifBind->parent;
	onResponse(req, NULL);
	return NPF_E_UNKNOWN;
    }
    if(getInterface(ifBind->child, &child) != NPF_NO_ERROR)
    {
	IF_ERROR("No Child Iface found for handle : %d", ifBind->child);
	req->error = NPF_IF_E_INVALID_HANDLE;
	req->ihandle = ifBind->child;
	onResponse(req, NULL);
	return NPF_E_UNKNOWN;
    }
    if(parent->l2If == NULL) /* Always consider First binded L2 Interface */
        parent->l2If = child;
    if(child->l3If == NULL) /* Always consider First binded L2 Interface */
        child->l3If = parent;
    if(validRelation(parent, child) != NPF_NO_ERROR)
    {
	IF_ERROR("*** Invalid Parent Child Binding tried ***");
	req->error = NPF_IF_E_INVALID_HANDLE;
	req->ihandle = parent->nsHandle;
	onResponse(req, NULL);
	return NPF_E_UNKNOWN;
    }
    /*
    if(npf_ns_addAssociate(parent->nsHandle, child->inode) != NPF_SUCCESS)
    {
	IF_ERROR("*** Can't Associate Parent(%d) and Child (%d)*** ", parent->nsHandle, child->nsHandle);
	req->error = NPF_IF_E_INVALID_CHILD_HANDLE;
	req->ihandle = child->nsHandle;
	onResponse(req, NULL);
	return (NPF_E_UNKNOWN);
    }
    IF_INFO("Parent: %d, Child: %d Associated", parent->nsHandle, child->nsHandle);
    if(npf_ns_addAssociate(child->nsHandle, parent->inode) != NPF_SUCCESS)
    {
	IF_ERROR("*** Can't Child(%d) and Parent (%d)*** ", child->nsHandle, parent->nsHandle);
	req->error = NPF_IF_E_INVALID_HANDLE;
	req->ihandle = child->nsHandle;
	onResponse(req, NULL);
	return (NPF_E_UNKNOWN);
    }
    */
    IF_INFO("Child: %d, Parent: %d Associated", child->nsHandle, parent->nsHandle);

    /*
     * Get FE and Port Information for L3 Interface
    */
    if(getFEandPORTdata(parent) != NPF_NO_ERROR)
    {
        IF_ERROR("*** Can't Find FE and PORT Information for : %d", parent->nsHandle);
        req->error = NPF_IF_E_INVALID_HANDLE;
        req->ihandle = child->nsHandle;
        onResponse(req, NULL);
        return (NPF_E_UNKNOWN);
    }
    req->error = NPF_NO_ERROR;
    req->ihandle = parent->nsHandle;
    onResponse(req, &child->nsHandle);
    return NPF_NO_ERROR;
}


/****************************************************************************
 * Function: compareUser()                                                  *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		data - 							    *
 *		key - 							    *
 *  							    		    *
 * Description: 							    *
 *             	compare function for "UserList" DList			    *
 * Return: 								    *
 *		NPF_TRUE  - Data found					    *
 *		NPF_FALSE  - Data Not found				    *
 ****************************************************************************/
static int compareUser (const void* data, const void* key)
{
    userInfo_t *user = (userInfo_t *)data;
    userInfo_t *ky = (userInfo_t *)key;
    if(ky->context == 0 && ky->callback == NULL && ky->handle == user->handle)
    {
        /* Deregister case */
        return 1;
    }
    else if(user->context == ky->context && user->callback == ky->callback)
        return NPF_TRUE;
    else
        return NPF_FALSE;
}

/****************************************************************************
 * Function: compareRequest()                                               *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		data - 							    *
 *		key - 							    *
 *  							    		    *
 * Description: 							    *
 *             	compare function for "RequestList" DList		    *
 * Return: 								    *
 *		NPF_TRUE  - Data found					    *
 *		NPF_FALSE  - Data Not found				    *
 ****************************************************************************/
static int compareRequest (const void* data, const void* ky)
{
    request_t	*request;
    request_t	*key;
    IF_FUNCTION();
    request = (request_t *)data;
    key = (request_t *)ky;
    //if(request->cbh == key->cbh)
    if(request == key)
    {
	IF_TRACE("Request Found ....");
	return(NPF_TRUE);
    }
    return (NPF_FALSE);
}

/****************************************************************************
 * Function: compareIf()                                                    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		data - 							    *
 *		key - 							    *
 *  							    		    *
 * Description: 							    *
 *             	compare function for "IfList" DList		    	    *
 * Return: 								    *
 *		NPF_TRUE  - Data found					    *
 *		NPF_FALSE  - Data Not found				    *
 ****************************************************************************/
static int compareIf (const void* data, const void* key)
{
    If_t *iface;
    NPF_NSHDL *nsh;
    iface = (If_t *)data;
    nsh = (NPF_NSHDL *)key;
    if(iface->nsHandle == *nsh) return (NPF_TRUE);
    return NPF_FALSE;
}

/****************************************************************************
 * Function: compareEventCallback()                                         *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		data - 							    *
 *		key - 							    *
 *  							    		    *
 * Description: 							    *
 *             	compare function for "EventCallbacks" DList		    *
 * Return: 								    *
 *		NPF_TRUE  - Data found					    *
 *		NPF_FALSE  - Data Not found				    *
 ****************************************************************************/
static int compareEventCallback (const void* data, const void* key)
{
    eventCallbackInfo_t *ev = (eventCallbackInfo_t *)data;
    eventCallbackInfo_t *ky = (eventCallbackInfo_t *)key;
    if(ky->context == 0 && ky->callback == NULL && ev->handle == ky->handle)
    {
        /* Deregister case */
        return NPF_TRUE;
    }
    else if(ev->context == ky->context)
    {
        return (NPF_TRUE);
    }
    return(NPF_FALSE);
}

/****************************************************************************
 * Function: IfPdkEvent()                                                   *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		ifh - interface handle					    *
 *		event - See npf_cm_types.h NPF_EVENT			    *
 *  							    		    *
 * Description: 							    *
 *             	This function sends NPF_EVENT to applications which are	    *
 *             	registered through "pdk_XXX_event_register"		    *
 * Return: 								    *
 *		None							    *
 ****************************************************************************/
void IfPdkEvent(NPF_HANDLE ifh, NPF_EVENT event)
{
    NPF_CBHANDLE     cbid, nextcbid;
    NPF_USERCONTEXT  context;
    NPF_INTERFACE_t  *outif;
    NPF_EVENT_CBFUNC clientCBFunc;

    clientCBFunc = (NPF_EVENT_CBFUNC)pdk_callback_firstFunc(PDK_EVT_IF, &context, &nextcbid);
    do 
    {
        if (clientCBFunc == NULL) continue;
        /* create a new NPF_INTERFACE_t for every internal callback */
        if ((outif = (NPF_INTERFACE_t *)PIL_AllocateMemory
                     (sizeof(NPF_INTERFACE_t), PIL_MEM_ZERO_INIT)) == NULL)
            assert(0);
        if (cm_copyFromPDK(ifh, (NPF_DATA)outif) == -1)
            assert(0);
        cbid = nextcbid;
        /* internal just call it, external needs to push to callback thread */
        clientCBFunc(context, ifh, event, (NPF_DATA)outif );
        PIL_FreeMemory(outif);
    }while ((clientCBFunc = (NPF_EVENT_CBFUNC)
              pdk_callback_nextFunc(cbid, &context, &nextcbid)) != NULL);
}

/****************************************************************************
 * Function: IfEvent()                                                      *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		ifh - interface handle					    *
 *		event - See npf_if.h 					    *
 *  							    		    *
 * Description: 							    *
 *             	This function sends EVENTs to applications which are	    *
 *             	registered through "NPF_IfEventRegister"		    *
 * Return: 								    *
 *		None							    *
 ****************************************************************************/
void IfEvent(NPF_IfEventData_t *event)
{
    NPF_IfEventArray_t  events;
    eventCallbackInfo_t *ev;
    DListItr            *eventItr;

    events.n_data = 1;
    events.eventData = PIL_AllocateMemory(sizeof(NPF_IfEventData_t), PIL_MEM_ZERO_INIT);
    /* This will be freed in CallbackRoutine function */
    if(events.eventData == NULL) return;
    memcpy(events.eventData, event, sizeof(NPF_IfEventData_t));
    eventItr = npf_list_itrCreate (&EventCallbacks);
    if(eventItr == NULL)
    {
        PIL_FreeMemory(events.eventData);
	return;
    }
    npf_list_itrFirst (eventItr);
    do
    {
        ev = (eventCallbackInfo_t *)npf_list_itrGetData (eventItr);
        cbthread_pushIFEvent(ev->callback, ev->context, &events);
    } while (npf_list_itrNext (eventItr) != -1);
    npf_list_itrDelete (eventItr);
    return;
}

static void FEresponse(uint32_t feid, IfCorrelator_t *corr, NPF_DATA resp)
{
    request_t	   *req;
    If_t	   *ifc;
    IfCorrelator_t *ifCorr = NULL;
    ifCorr = (IfCorrelator_t *)corr;
    if(ifCorr == NULL) return;
    req = ifCorr->req;
    ifc = ifCorr->ifc;
    req->ihandle = ifCorr->ifHandle;
    PIL_FreeMemory(corr);
    req->fpResp++;
    for(req->ok = 0; req->ok < ifc->FE.count; req->ok++)
    {
        if(req->fe[req->ok].id == feid) /* FE Found */
        {
            req->fe[req->ok].resp = req->fe[req->ok].resp + 1;
            if(req->fe[req->ok].req == req->fe[req->ok].resp)
                req->fe[req->ok].ok = NPF_TRUE;
            IF_TRACE(">> FE id = %d, req = %d, resp = %d, ok = %d <<", 
                        feid, ifc->FE.count, req->fe[req->ok].resp, req->fe[req->ok].ok);
        }
    }
    IF_INFO("NPF_DATA (FPPI_Status) = %d", *(FPPI_Status *)resp);
    if (*(FPPI_Status *)resp != FPPI_SUCCESSFUL)
    {
        IF_ERROR("*** response != FPPI_SUCCESSFUL ***");
        req->error = NPF_IF_E_INVALID_SPEED;
        onResponse(req, NULL);
        return;
    }
    IF_INFO("NPF_DATA (FPPI_Status) = %d", *(FPPI_Status *)resp);
}

/****************************************************************************
 * Function: ifFPgetL3StatsCallback()                                       *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		feid - FE id invoked this callback			    *
 *		corr - Correlator 					    *
 *		context - User Context					    *
 *		resp - NPF_IF_STATS data				    *
 *  							    		    *
 * Description: 							    *
 *             	This function is invoked by FP module on statistics request *
 *             	Constructs  NPF_IfStatistics_t data and invokes application *
 *		callback with NPF_IfStatistics_t as Async Response	    *
 * Return: 								    *
 *		None							    *
 ****************************************************************************/
void ifFPgetL3StatsCallback(uint32_t feid, IfCorrelator_t *corr, NPF_userContext_t context, NPF_DATA resp)
{
    If_t	*ifc;
    request_t   	*req;
    NPF_IF_STATS 	*feStats;
    IF_FUNCTION();
    IF_LOCK();
    req = ((IfCorrelator_t *)corr)->req;
    ifc = ((IfCorrelator_t *)corr)->ifc;
    IF_TRACE("********* ifFPgetL3StatsCallback *********");
    if(context != NPF_IF_STATS_GET || corr == NULL)
	return;
    feStats = (NPF_IF_STATS *)resp;
    req = (request_t *)((IfCorrelator_t *)corr)->req;
    ifc = (If_t *)((IfCorrelator_t *)corr)->ifc;
    for(req->ok = 0; req->ok < ifc->FE.count; req->ok++)
    {
	if(req->fe[req->ok].id == feid) /* FE Found */
	{
	    req->fe[req->ok].resp = req->fe[req->ok].resp + 1;
	    if(req->fe[req->ok].req == req->fe[req->ok].resp)
		req->fe[req->ok].ok = NPF_TRUE;
	}
    }
    if(feStats == NULL || req == NULL)
    {
	IF_UNLOCK();
        //ifc->FE.ok = NPF_FALSE;
	IF_ERROR("***** feStats == NULL || req == NULL*****");
	return;
    }
    ifc->stats.bytesRx += feStats->ifInOctets; /* Receive Bytes */
    ifc->stats.ucPackRx += feStats->ifInUcastPkts; /* Receive Unicast Packets */
    ifc->stats.mcPackRx += feStats->ifInMulticastPkts; /* Receive Multicast Packets */
    ifc->stats.bcPackRx += feStats->ifInMulticastPkts; /* Receive Broadcast packets */
    ifc->stats.dropRx += feStats->ifInDiscards; /* Receive packets dropped */
    ifc->stats.errorRx += feStats->IfInErrors; /* Receive errors */
    ifc->stats.protoRx += feStats->ifInUnknownProtos; /* Receive unknown protocol */
    ifc->stats.bytesTx += feStats->ifOutOctets; /* Transmit bytes */
    ifc->stats.ucPackTx += feStats->ifOutUcastPkts; /* Transmit Unicast Packets */
    ifc->stats.mcPackTx += feStats->OutMulticastPkts; /* Transmit Multicast Packets */
    ifc->stats.bcPackTx += feStats->OutBroadcastPkts; /* Transmit Broadcast Packets */
    ifc->stats.dropTx += feStats->ifOutDiscards; /* Transmit dropped packets */
    ifc->stats.errorTx += feStats->ifOutErrors; /* Transmit errors */
    onResponse(req, &ifc->stats);
    IF_UNLOCK();
}

/****************************************************************************
 * Function: ifDelL2PropertiesCallback()                                    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		feid - FE id invoked this callback			    *
 *		corr - Correlator 					    *
 *		context - User Context					    *
 *		resp - NPF_IF_STATS data				    *
 *  							    		    *
 * Description: 							    *
 *             	Not implemented						    *
 * Return: 								    *
 *		None							    *
 ****************************************************************************/
void ifDelL2PropertiesCallback(uint32_t feid, IfCorrelator_t *corr, NPF_userContext_t context, NPF_DATA resp)
{
    request_t   *req;
    If_t   	*ifc;
    IF_FUNCTION();
    IF_LOCK();
    req = ((IfCorrelator_t *)corr)->req;
    ifc = ((IfCorrelator_t *)corr)->ifc;
    IF_TRACE("********* ifDelL2PropertiesCallback *********");
    FEresponse(feid, corr, resp);
    IF_UNLOCK();
}
/****************************************************************************
 * Function: ifDelL3PropertiesCallback()                                    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		feid - FE id invoked this callback			    *
 *		corr - Correlator 					    *
 *		context - User Context					    *
 *		resp - NPF_IF_STATS data				    *
 *  							    		    *
 * Description: 							    *
 *             	Not implemented						    *
 * Return: 								    *
 *		None							    *
 ****************************************************************************/
void ifDelL3PropertiesCallback(uint32_t feid, IfCorrelator_t *corr, NPF_userContext_t context, NPF_DATA resp)
{
    request_t   *req;
    If_t   	*ifc;
    IF_FUNCTION();
    IF_LOCK();
    req = ((IfCorrelator_t *)corr)->req;
    ifc = ((IfCorrelator_t *)corr)->ifc;
    IF_TRACE("********* ifDelL3PropertiesCallback *********");
    FEresponse(feid, corr, resp);
    IF_UNLOCK();
}

#ifdef _INCLUDE_IPV6

/****************************************************************************
 * Function: ifDelL3PropertiesIPv6Callback()                                    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		feid - FE id invoked this callback			    *
 *		corr - Correlator 					    *
 *		context - User Context					    *
 *		resp - NPF_IF_STATS data				    *
 *  							    		    *
 * Description: 							    *
 *             	Not implemented						    *
 * Return: 								    *
 *		None							    *
 ****************************************************************************/
void ifDelL3PropertiesIPv6Callback(uint32_t feid, IfCorrelator_t *corr, NPF_userContext_t context, NPF_DATA resp)
{
    request_t   *req;
    If_t   	*ifc;
    IF_FUNCTION();
    IF_LOCK();
    req = ((IfCorrelator_t *)corr)->req;
    ifc = ((IfCorrelator_t *)corr)->ifc;
    IF_TRACE("********* ifDelL3PropertiesIPv6Callback *********");
    FEresponse(feid, corr, resp);
    IF_UNLOCK();
}

#endif

/****************************************************************************
 * Function: ifFPgetL2StatsCallback()                                       *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		feid - FE id invoked this callback			    *
 *		corr - Correlator 					    *
 *		context - User Context					    *
 *		resp - NPF_IF_STATS data				    *
 *  							    		    *
 * Description: 							    *
 *             	Not implemented						    *
 * Return: 								    *
 *		None							    *
 ****************************************************************************/
void ifFPgetL2StatsCallback(uint32_t feid, IfCorrelator_t *corr, NPF_userContext_t context, NPF_DATA resp)
{
    request_t   *req;
    If_t   	*ifc;
    IF_FUNCTION();
    IF_LOCK();
    req = ((IfCorrelator_t *)corr)->req;
    ifc = ((IfCorrelator_t *)corr)->ifc;
    IF_TRACE("********* ifFPgetL2StatsCallback *********");
    FEresponse(feid, corr, resp);
    IF_UNLOCK();
}

/****************************************************************************
 * Function: ifSetIPaddrCallback()                                          *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		feid - FE id invoked this callback			    *
 *		corr - Correlator 					    *
 *		context - User Context					    *
 *		resp - NPF_DATA						    *
 *  							    		    *
 * Description: 							    *
 *             	This is invoked by FP module on IPV4 address set request    *
 * Return: 								    *
 *		None							    *
 ****************************************************************************/
void ifSetIPaddrCallback(uint32_t feid, IfCorrelator_t *corr, NPF_userContext_t context, NPF_DATA resp)
{
    request_t   *req;
    If_t   	*ifc;
    IF_FUNCTION();
    IF_LOCK();
    req = ((IfCorrelator_t *)corr)->req;
    ifc = ((IfCorrelator_t *)corr)->ifc;
    IF_TRACE("********* ifSetIPaddrCallback *********");
    if(context != NPF_IF_IPV4ADDR_SET)
    {
        IF_UNLOCK();
	return;
    }
    FEresponse(feid, corr, resp);
    updateL3Data(req, ifc, NPF_L3ATTRS_IPADDR, resp);
    IF_UNLOCK();
}

#ifdef _INCLUDE_IPV6

/****************************************************************************
 * Function: ifSetIPaddrV6Callback()                                        *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		feid - FE id invoked this callback			    *
 *		corr - Correlator 					    *
 *		context - User Context					    *
 *  							    		    *
 * Description: 							    *
 *             	This is invoked by FP module on IPV6 address set request    *
 * Return: 								    *
 *		None							    *
 ****************************************************************************/
void ifSetIPaddrV6Callback(uint32_t feid, IfCorrelator_t *corr, NPF_userContext_t context, NPF_DATA resp)
{
    request_t   *req;
    If_t   	*ifc;
    IF_FUNCTION();
    IF_LOCK();
    req = ((IfCorrelator_t *)corr)->req;
    ifc = ((IfCorrelator_t *)corr)->ifc;
    IF_TRACE("********* ifSetIPaddrV6Callback *********");
    if(context != NPF_IF_IPV6_ADDR_ADD)
    {
        IF_UNLOCK();
	return;
    }
    FEresponse(feid, corr, resp);
    updateL3Data(req, ifc, NPF_L3ATTRS_IPADDR, resp);
    IF_UNLOCK();
}

#endif


/****************************************************************************
 * Function: ifSetIPfwdCallback()                                           *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		feid - FE id invoked this callback			    *
 *		corr - Correlator 					    *
 *		context - User Context					    *
 *  							    		    *
 * Description: 							    *
 *             	This is invoked by FP module on set fwd request 	    *
 * Return: 								    *
 *		None							    *
 ****************************************************************************/
void ifSetIPfwdCallback(uint32_t feid, IfCorrelator_t *corr, NPF_userContext_t context, NPF_DATA resp)
{
    request_t   *req;
    If_t   	*ifc;
    IF_FUNCTION();
    IF_LOCK();
    req = ((IfCorrelator_t *)corr)->req;
    ifc = ((IfCorrelator_t *)corr)->ifc;
    IF_TRACE("********* ifSetIPfwdCallback *********");
    if(context != NPF_IF_LAN_PROMISC_SET)
    {
        IF_UNLOCK();
	return;
    }
    FEresponse(feid, corr, resp);
    updateL3Data(req, ifc, NPF_L3ATTRS_IPFWD, resp);
    IF_UNLOCK();
}

/****************************************************************************
 * Function: ifSetMTUCallback()                                             *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		feid - FE id invoked this callback			    *
 *		corr - Correlator 					    *
 *		context - User Context					    *
 *  							    		    *
 * Description: 							    *
 *             	This is invoked by FP module on MTU set request 	    *
 * Return: 								    *
 *		None							    *
 ****************************************************************************/
void ifSetMTUCallback(uint32_t feid, IfCorrelator_t *corr, NPF_userContext_t context, NPF_DATA resp)
{
    request_t   *req = NULL;
    If_t   	*ifc = NULL;
    IF_FUNCTION();
    IF_LOCK();
    IF_TRACE("********* ifSetMTUCallback *********");
    req = ((IfCorrelator_t *)corr)->req;
    ifc = ((IfCorrelator_t *)corr)->ifc;
    if(context != NPF_IF_IPV4MTU_SET)
    {
        IF_UNLOCK();
	return;
    }
    FEresponse(feid, corr, resp);
    updateL2Data(req, ifc, NPF_L2ATTRS_MTU, resp);
    IF_UNLOCK();
}

/****************************************************************************
 * Function: ifSetLANSpeedCallback()                                        *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		feid - FE id invoked this callback			    *
 *		corr - Correlator 					    *
 *		context - User Context					    *
 *  							    		    *
 * Description: 							    *
 *             	This is invoked by FP module on set speed request	    *
 * Return: 								    *
 *		None							    *
 ****************************************************************************/
void ifSetLANSpeedCallback(uint32_t feid, IfCorrelator_t *corr, NPF_userContext_t context, NPF_DATA resp)
{
    request_t   *req;
    If_t   	*ifc;
    IF_FUNCTION();
    IF_LOCK();
    req = ((IfCorrelator_t *)corr)->req;
    ifc = ((IfCorrelator_t *)corr)->ifc;
    IF_TRACE("********* ifSetLANSpeedCallback *********");
    if(context != NPF_IF_LAN_SPEED_SET)
    {
        IF_UNLOCK();
	return;
    }
    FEresponse(feid, corr, resp);
    updateL2Data(req, ifc, NPF_L2ATTRS_LINKSPEED, resp);
    IF_UNLOCK();
}

/****************************************************************************
 * Function: ifSetStatusCallback()                                          *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		feid - FE id invoked this callback			    *
 *		corr - Correlator 					    *
 *		context - User Context					    *
 *  							    		    *
 * Description: 							    *
 *             	This is invoked by FP module on set interface status request*
 * Return: 								    *
 *		None							    *
 ****************************************************************************/
void ifSetStatusCallback(uint32_t feid, IfCorrelator_t *corr, NPF_userContext_t context, NPF_DATA resp)
{
    request_t   *req;
    If_t   	*ifc;
    IF_FUNCTION();
    IF_LOCK();
    req = ((IfCorrelator_t *)corr)->req;
    ifc = ((IfCorrelator_t *)corr)->ifc;
    IF_TRACE("********* ifSetStatusCallback *********");
    if(context != NPF_IF_ENABLE)
    {
        IF_UNLOCK();
	return;
    }
    FEresponse(feid, corr, resp);
    updateL2Data(req, ifc, NPF_L2ATTRS_STATUS, resp);
    IF_UNLOCK();
}

/****************************************************************************
 * Function: interfaceEvent()                                          	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		context - Registered context				    *
 *		ifHandle - Interface handle				    *
 *		event - event received					    *
 *  							    		    *
 * Description: 							    *
 *             	This is callback function registered through		    *
 *             	"pdk_if_event_register"					    *
 *             	Interface status = link status & port status		    *
 * Return: 								    *
 *		None							    *
 ****************************************************************************/
void interfaceEvent( NPF_USERCONTEXT context,
                   NPF_HANDLE      ifHandle,
                   NPF_EVENT       event,
                   NPF_DATA        data)
{
    If_t              *ifc;
    NPF_IfEventData_t ev;

    if(getInterface(ifHandle, &ifc) != NPF_NO_ERROR)
    {
        IF_ERROR("No Iface found for handle : %d", ifHandle);
        return;
    }

    switch(event)
    {
        case NPF_EVENT_IF_UP:
        {
            IF_TRACE("----- NPF_EVENT_IF_UP(%d) -----", ifHandle);
            ifc->npfIf.adminStatus = NPF_IF_ADMIN_STATUS_UP;
            ev.handle = ifHandle;
            ev.eventType = NPF_IF_OPR_UP;
            IfEvent(&ev);
        }
        break;
        case NPF_EVENT_IF_DOWN:
        {
            IF_TRACE("----- NPF_EVENT_IF_DOWN(%d) -----", ifHandle);
	    // TODO: check the port admin status in C&M
            //ifc->npfIf.adminStatus = NPF_IF_ADMIN_STATUS_DOWN;
            ev.handle = ifHandle;
            ev.eventType = NPF_IF_OPR_DOWN;
            IfEvent(&ev);
        }
        break;
        default:
        break;
    }
}

/****************************************************************************
 * Function: portEvent()                                          	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		context - Registered context				    *
 *		portHandle - port handle				    *
 *		event - event received					    *
 *  							    		    *
 * Description: 							    *
 *             	This is callback function registered through		    *
 *             	"pdk_port_event_register"				    *
 *             	LINKUP event already handled by C&M before triggering       *
 *		this event. IF UP/DOWN Event triggred by C&M, no need to    *
 *		update Interface staus here
 * Return: 								    *
 *		None							    *
 ****************************************************************************/
void portEvent( NPF_USERCONTEXT context,
                   NPF_HANDLE   portHandle,
                   NPF_EVENT    event,
                   NPF_DATA     data)
{
    If_t              *ifc;
    NPF_IfEventData_t ev;
    int             numif = 0;
    int             count = 0;
    NPF_HANDLE      iflist[MAXIFOWNPORT];
    NPF_PORT_t      *port;

    port = (NPF_PORT_t*) data;
    if(port == NULL) return;
    if (cm_getIFByPort(portHandle, &numif, iflist) != NPF_SUCCESS)
	return;
    for(count = 0; count < numif; count ++)
    { 
        if(getInterface(iflist[count], &ifc) != NPF_NO_ERROR)
        {
            IF_ERROR("No Iface found for handle : %d", iflist[count]);
            return;
        }
        switch(event)
        {
            case NPF_EVENT_PORT_LINKUP:
                ev.handle = iflist[count];
                ev.eventType = NPF_EVENT_PORT_LINKUP;
                IfEvent(&ev);
            break;
            case NPF_EVENT_PORT_LINKDOWN:
                ev.handle = iflist[count];
                ev.eventType = NPF_EVENT_PORT_LINKDOWN;
                IfEvent(&ev);
            break;
	    default:
	    break;
        }
    }
}

/****************************************************************************
 * Function: ifListsInit()                                       	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		None							    *
 *  							    		    *
 * Description: 							    *
 *             	This function initializes all internal DList structures	    *
 *             	Called by pdk_if_init function			    	    *
 * Return: 								    *
 *		NPF_NO_ERROR						    *
 ****************************************************************************/
NPF_error_t  ifListsInit()
{
    IF_FUNCTION();
    npf_list_init(&UserList, PIL_FreeMemory);
    npf_list_init(&EventCallbacks, PIL_FreeMemory);
    npf_list_init(&RequestList, PIL_FreeMemory);
    npf_list_init(&IfList, PIL_FreeMemory);

    UserList.compare = &compareUser;
    EventCallbacks.compare = &compareEventCallback;
    RequestList.compare = &compareRequest;
    IfList.compare = &compareIf;
    return NPF_NO_ERROR;
}

/****************************************************************************
 * Function: ifListsClose()                                       	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		None							    *
 *  							    		    *
 * Description: 							    *
 *             	This function destroys all internal DList structures	    *
 *             	Called by pdk_if_shutdown function			    *
 * Return: 								    *
 *		None							    *
 ****************************************************************************/
NPF_error_t  ifListsClose()
{
    IF_FUNCTION();
    npf_list_destroy(&UserList);
    npf_list_destroy(&EventCallbacks);
    npf_list_destroy(&RequestList);
    npf_list_destroy(&IfList);
    return NPF_NO_ERROR;
}


/****************************************************************************
 * Function: ifFPregister()                                       	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		None							    *
 *  							    		    *
 * Description: 							    *
 *             	This function registers callback with FP module		    *
 *             	Called by pdk_if_init function			    	    *
 * Return: 								    *
 *		NPF_NO_ERROR - On no Error				    *
 *		NPF_FAIL - On registration fails			    *
 ****************************************************************************/
NPF_error_t ifFPregister()
{
    IF_FUNCTION();
    IF_INFO("ifSetLANSpeedCallback Registered with handle : %d", ifFPsetLANSpeedCallbackHandle);
    if (FPPAPI_register_callback(
            (void *)&ifSetLANSpeedCallback,        /* func */
            &ifFPsetLANSpeedCallbackHandle,  /* CB handle */
            FPPI_L2_SetProperties,             /* NPF_CBCAT */
            (NPF_USERCONTEXT)NPF_IF_LAN_SPEED_SET)
                != FPPI_SUCCESS)
    {
        IF_ERROR("***** %s", "FPPAPI_register_callback for ifSetLANSpeedCallback != FPPI_SUCCESS");
        return NPF_FAIL;
    }
    IF_INFO("ifSetLANSpeedCallback Registered with handle : %d", ifFPsetLANSpeedCallbackHandle);

    if (FPPAPI_register_callback(
            (void *)&ifSetMTUCallback,        /* func */
            &ifFPsetMTUCallbackHandle,  /* CB handle */
            FPPI_L2_SetProperties,             /* NPF_CBCAT */
            (NPF_USERCONTEXT)NPF_IF_IPV4MTU_SET)
                != FPPI_SUCCESS)
    {
        IF_ERROR("***** %s", "FPPAPI_register_callback for ifSetIPaddrCallback != FPPI_SUCCESS");
        return NPF_FAIL;
    }
    IF_INFO("ifSetMTUCallback Registered with handle : %d", ifFPsetMTUCallbackHandle);

    if (FPPAPI_register_callback(
            (void *)&ifSetStatusCallback,        /* func */
            &ifFPsetStatusCallbackHandle,  /* CB handle */
            FPPI_L2_SetProperties,             /* NPF_CBCAT */
            (NPF_USERCONTEXT)NPF_IF_ENABLE)
                != FPPI_SUCCESS)
    {
        IF_ERROR("***** %s", "FPPAPI_register_callback for ifSetStatusCallback != FPPI_SUCCESS");
        return NPF_FAIL;
    }
    IF_INFO("ifSetStatusCallback Registered with handle : %d", ifFPsetStatusCallbackHandle);

    if (FPPAPI_register_callback(
            (void *)&ifSetStatusCallback,        /* func */
            &ifFPsetStatusCallbackHandle,  /* CB handle */
            FPPI_L2_SetProperties,             /* NPF_CBCAT */
            (NPF_USERCONTEXT)NPF_IF_ENABLE)
                != FPPI_SUCCESS)
    {
        IF_ERROR("***** %s", "FPPAPI_register_callback for ifSetStatusCallback != FPPI_SUCCESS");
        return NPF_FAIL;
    }
    IF_INFO("ifSetStatusCallback Registered with handle : %d", ifFPsetStatusCallbackHandle);

    if (FPPAPI_register_callback(
            (void *)&ifDelL2PropertiesCallback,        /* func */
            &ifDelL2PropertiesCallbackHandle,  /* CB handle */
            FPPI_L2_DelProperties,             /* NPF_CBCAT */
            (NPF_USERCONTEXT)NPF_IF_ENABLE)
                != FPPI_SUCCESS)
    {
        IF_ERROR("***** %s", "FPPAPI_register_callback for ifDelL2PropertiesCallback != FPPI_SUCCESS");
        return NPF_FAIL;
    }
    IF_INFO("ifDelL2PropertiesCallback Registered with handle : %d", ifDelL2PropertiesCallbackHandle);

    if (FPPAPI_register_callback(
            (void *)&ifSetIPaddrCallback,       /* CB func */
            &ifFPsetIPaddrCallbackHandle,      /* CB handle */
            FPPI_L3_SetProperties,                  /* NPF_CBCAT */
            (NPF_USERCONTEXT)NPF_IF_IPV4ADDR_SET) != FPPI_SUCCESS)
    {
        IF_ERROR("***** %s", "FPPAPI_register_callback for ifSetIPaddrCallback != FPPI_SUCCESS");
        return NPF_FAIL;
    }
    IF_INFO("ifSetIPaddrCallback Registered with handle : %d", ifFPsetIPaddrCallbackHandle);

    if (FPPAPI_register_callback(
            (void *)&ifDelL3PropertiesCallback,        /* func */
            &ifDelL3PropertiesCallbackHandle,  /* CB handle */
            FPPI_L3_DelProperties,             /* NPF_CBCAT */
            (NPF_USERCONTEXT)NPF_IF_ENABLE)
                != FPPI_SUCCESS)
    {
        IF_ERROR("***** %s", "FPPAPI_register_callback for ifDelL3PropertiesCallback != FPPI_SUCCESS");
        return NPF_FAIL;
    }
    IF_INFO("ifDelL3PropertiesCallback Registered with handle : %d", ifDelL3PropertiesCallbackHandle);

#ifdef _INCLUDE_IPV6

    if (FPPAPI_register_callback(
            (void *)&ifSetIPaddrV6Callback,       /* CB func */
            &ifFPsetIPaddrV6CallbackHandle,      /* CB handle */
            FPPI_L3_SetPropertiesIPv6,                  /* NPF_CBCAT */
            (NPF_USERCONTEXT)NPF_IF_IPV6_ADDR_ADD) != FPPI_SUCCESS)
    {
        IF_ERROR("***** %s", "FPPAPI_register_callback for ifSetIPaddrV6Callback != FPPI_SUCCESS");
        return NPF_FAIL;
    }
    IF_INFO("ifSetIPaddrV6Callback Registered with handle : %d", ifFPsetIPaddrV6CallbackHandle);

    if (FPPAPI_register_callback(
            (void *)&ifDelL3PropertiesIPv6Callback,       /* CB func */
            &ifDelL3PropertiesIPv6CallbackHandle,      /* CB handle */
            FPPI_L3_DelPropertiesIPv6,                  /* NPF_CBCAT */
            (NPF_USERCONTEXT)NPF_IF_IPV6_ADDR_ADD) != FPPI_SUCCESS)
    {
        IF_ERROR("***** %s", "FPPAPI_register_callback for ifDelL3PropertiesIPv6Callback != FPPI_SUCCESS");
        return NPF_FAIL;
    }
    IF_INFO("ifDelL3PropertiesIPv6Callback Registered with handle : %d", ifDelL3PropertiesIPv6CallbackHandle);

#endif

    if (FPPAPI_register_callback(
            (void *)&ifSetIPfwdCallback,       /* CB func */
            &ifFPsetIPfwdCallbackHandle,      /* CB handle */
            FPPI_L3_SetProperties,                  /* NPF_CBCAT */
            (NPF_USERCONTEXT)NPF_IF_LAN_PROMISC_SET) != FPPI_SUCCESS)
    {
        IF_ERROR("***** %s", "FPPAPI_register_callback for ifSetIPfwdCallback != FPPI_SUCCESS");
        return NPF_FAIL;
    }
    IF_INFO("ifSetIPfwdCallback Registered with handle : %d", ifFPsetIPfwdCallbackHandle);

    if (FPPAPI_register_callback(
            (void *)&ifFPgetL3StatsCallback,    /* func */
            &ifFPgetL3StatsCallbackHandle,   /* CB handle */
            FPPI_L3_GetStatistics,              /* NPF_CBCAT */
            (NPF_USERCONTEXT)NPF_IF_STATS_GET)
                != FPPI_SUCCESS)
    {
        IF_ERROR("***** %s", "FPPAPI_register_callback for ifFPgetL3StatsCallback != FPPI_SUCCESS");
        return NPF_FAIL;
    }
    IF_INFO("ifFPgetL3StatsCallback Registered with handle : %d", ifFPgetL3StatsCallbackHandle);


    if (FPPAPI_register_callback(
            (void *)&ifFPgetL2StatsCallback,    /* func */
            &ifFPgetL2StatsCallbackHandle,   /* CB handle */
            FPPI_L2_GetStatistics,              /* NPF_CBCAT */
            (NPF_USERCONTEXT)NPF_IF_STATS_GET)
                != FPPI_SUCCESS)
    {
        IF_ERROR("***** %s", "FPPAPI_register_callback for ifFPgetL2StatsCallback != FPPI_SUCCESS");
        return NPF_FAIL;
    }
    IF_INFO("ifFPgetL2StatsCallback Registered with handle : %d", ifFPgetL3StatsCallbackHandle);

    /* register for INTERFACE event category */
    ifEventContext =  (void*) PDK_EVT_IF;
    pdk_if_event_register(ifEventContext,
                                (NPF_GENERAL_CBFUNC)&interfaceEvent,
                                &interfaceEventHandle);

    /* register for PORT event category */
    portEventContext =  (void*) PDK_EVT_PORT;
    pdk_port_event_register(portEventContext,
                                  (NPF_GENERAL_CBFUNC)&portEvent,
                                  &portEventHandle);

    IF_TRACE("%s", "ifFPregister Done .............");
    return NPF_NO_ERROR;
}

/****************************************************************************
 * Function: ifFPderegister()                                       	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		None							    *
 *  							    		    *
 * Description: 							    *
 *             	This function Deregisters callbacks which are registered    *
 *             	during pdk_if_init					    *
 *             	Called by pdk_if_shutdown function			    *
 * Return: 								    *
 *		NPF_NO_ERROR - On no Error				    *
 ****************************************************************************/
NPF_error_t ifFPderegister()
{
    IF_FUNCTION();
    IF_LOCK();
    FPPAPI_deregister_callback(ifFPsetLANSpeedCallbackHandle);
    FPPAPI_deregister_callback(ifFPsetMTUCallbackHandle);
    FPPAPI_deregister_callback(ifFPsetStatusCallbackHandle);
    FPPAPI_deregister_callback(ifFPsetIPaddrCallbackHandle);
#ifdef _INCLUDE_IPV6
    FPPAPI_deregister_callback(ifFPsetIPaddrV6CallbackHandle);
    FPPAPI_deregister_callback(ifDelL3PropertiesIPv6CallbackHandle);
#endif
    FPPAPI_deregister_callback(ifFPsetIPfwdCallbackHandle);
    FPPAPI_deregister_callback(ifFPgetL3StatsCallbackHandle);
    FPPAPI_deregister_callback(ifFPgetL2StatsCallbackHandle);
    pdk_if_event_deregister(interfaceEventHandle);
    pdk_port_event_deregister(portEventHandle);
    IF_UNLOCK();
    return NPF_NO_ERROR;
}

/****************************************************************************
 * Function: pdk_if_init()                                       	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		None							    *
 *  							    		    *
 * Description: 							    *
 *             	This is initialization routine for Interface management mod *
 *             	Called by pdk_init					    *
 * Return: 								    *
 *		NPF_FAIL - On Error				    	    *
 *		NPF_SUCCESS - On no Error				    *
 ****************************************************************************/
NPF_RET pdk_if_init()
{

    IF_FUNCTION();
    if(IfInit)
    {
	IF_ERROR("***** %s", "NPF_COMPONENT_REINITIALIZED");
	return NPF_COMPONENT_REINITIALIZED;
    }
    PIL_InitializeCriticalSection(&IM_LOCK);
    IF_LOCK();
    if(ifListsInit() != NPF_NO_ERROR)
    {
        IF_UNLOCK();
	IF_ERROR("***** %s", "ifListsInit() != NPF_NO_ERROR");
	return NPF_FAIL;
    }
    if(ifFPregister() != NPF_NO_ERROR)
    {
        IF_UNLOCK();
	IF_ERROR("***** %s", "ifFPregister() != NPF_NO_ERROR");
	return NPF_FAIL;
    }
    if(createIfRoot() != NPF_NO_ERROR)
    {
        IF_UNLOCK();
	IF_ERROR("***** %s", "createIfRoot() != NPF_NO_ERROR");
	return NPF_FAIL;
    }
    IfInit = NPF_TRUE;
    IF_INFO("%s", "pdk_if_init Done !!!!!!!!!!!!!!!!!!!!!");
    IF_UNLOCK();
    return NPF_SUCCESS;
}

/****************************************************************************
 * Function: pdk_if_shutdown()                                       	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		None							    *
 *  							    		    *
 * Description: 							    *
 *             	This is shutdown routine for Interface management module    *
 *             	Called by pdk_shutdown					    *
 * Return: 								    *
 *		NPF_FAIL - On Error				    	    *
 *		NPF_SUCCESS - On no Error				    *
 ****************************************************************************/
NPF_RET pdk_if_shutdown(void)
{
    IF_FUNCTION();
    PIL_DestroyCriticalSection(&IM_LOCK);
    if(ifListsClose() != NPF_NO_ERROR)
    {
	return NPF_FAIL;
    }
    if(ifFPderegister() != NPF_NO_ERROR)
    {
	return NPF_FAIL;
    }
    #ifdef IF_LOG
    //npf_logger_Stop();
    #endif
    return NPF_SUCCESS;
}


/****************************************************************************
 * Function: initIfLog()                                       	    	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		file - Log file name					    *
 *  							    		    *
 * Description: 							    *
 *             	This is logger initialization routine for interface module  *
 * Return: 								    *
 *		NPF_E_UNKNOWN - On file equlas NULL			    *
 *		NPF_NO_ERROR - On no Error				    *
 ****************************************************************************/
NPF_error_t initIfLog(char *file)
{
    if(file == NULL)
    {
        return (NPF_E_UNKNOWN);
    }
    npf_logger_Start(file);
    npf_logger_SetLevel(LOG_IF, 0xF);
    npf_logger_SetVerbosity(LOG_VERB_MAX);
    return (NPF_NO_ERROR);
}

/****************************************************************************
 * Function: createIfRoot()                                       	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		None							    *
 *  							    		    *
 * Description: 							    *
 *             	This function creates namespace path which is root for	    *
 *             	interface namespace path. Called by pdk_if_init	    	    *
 * Return: 								    *
 *		NPF_E_UNKNOWN - On Error				    *
 *		NPF_NO_ERROR  - On no Error				    *
 ****************************************************************************/
NPF_error_t createIfRoot()
{
    NPF_NSHDL           hNode = 0;
    NPF_NSHDL           rtr_ns_hdl = 0;
    NPF_NSHDL           sys_ns_hdl = 0;
    NPF_NSHDL           if_ns_hdl = 0;
    NPF_HANDLE          sys_npf_hdl = 0;
    NODE_ITERATOR       rtr_itr;
    NODE_ITERATOR       if_itr;
    char                sysname[256], rtrname[256], ifname[256];
    
    /* Zero set the buffers */    
    memset(sysname,0, 256);
    memset(rtrname,0, 256);
    memset(ifname,0, 256);

    if(pdk_ns_getNSRootHandle(&sys_ns_hdl) != NPF_SUCCESS)
    {
        /* Doesn't exists, Create /NPFSys/0" */
        sprintf(sysname, "/NPFSys");
        if(npf_ns_open(sysname, &sys_ns_hdl) == NPF_SUCCESS)
            npf_ns_close(sys_ns_hdl);
	else
	{
            if(npf_ns_create("/NPFSys", NPF_NS_TYPENODE, NPF_SYSTEM, 0, &hNode) != NPF_SUCCESS)
                return(NPF_E_UNKNOWN);
	}
        sprintf(sysname, "/NPFSys/0");
        if(npf_ns_open(sysname, &sys_ns_hdl) == NPF_SUCCESS)
            npf_ns_close(sys_ns_hdl);
        else
	{
            if(npf_ns_create(sysname, NPF_NS_INSTNODE, NPF_SYSTEM, 0, &hNode) != NPF_SUCCESS)
                return(NPF_E_UNKNOWN);
        }
        if(npf_ns_open(sysname, &sys_ns_hdl) != NPF_SUCCESS)
            return(NPF_E_UNKNOWN);
    }
    npf_ns_getPath(sys_ns_hdl, sysname);
    if(npf_ns_getDataHandle(sys_ns_hdl, (NPF_HANDLE *)&sys_npf_hdl) != NPF_SUCCESS)
        return(NPF_E_UNKNOWN);
    /* Close the NS handle but keep the NPF Data Handle */
    npf_ns_close(sys_ns_hdl);
    
   /* System Node is created */
    
    /* Find if Router Node is created; If not, create a TYPE node. Instance will be created by individual modules as and when they are configured  */
    if(pdk_ns_getChildItrByType(sys_npf_hdl, NPF_IPROUTER, &rtr_itr)  != NPF_SUCCESS)
    {
        strcpy(rtrname,sysname); 
        strcat(rtrname, "/Router");
        if(npf_ns_open(rtrname, &rtr_ns_hdl) == NPF_SUCCESS)
            npf_ns_close(rtr_ns_hdl);
	else
        {
            if(npf_ns_create(rtrname, NPF_NS_TYPENODE, NPF_IPROUTER, 0, &hNode) != NPF_SUCCESS)
            {
                npf_ns_close(sys_npf_hdl);
                return(NPF_E_UNKNOWN);
            }
        }
    }
    /* Router TYPE node is created by now !! */

    /* Start creating interfaces under NPFSys/0 */
    if(pdk_ns_getChildItrByType(sys_npf_hdl, NPF_INTERFACE, &if_itr)  != NPF_SUCCESS)
    {
        strcpy(ifname, sysname);
        strcat(ifname, "/Iface");
        if(npf_ns_open(ifname, &if_ns_hdl) == NPF_SUCCESS)
            npf_ns_close(if_ns_hdl);
	else
        {
            if(npf_ns_create(ifname, NPF_NS_TYPENODE, NPF_INTERFACE, 0, &hNode) != NPF_SUCCESS)
            {
                npf_ns_close(sys_npf_hdl);
                return(NPF_E_UNKNOWN);
            }
        }
    }
    npf_ns_close(sys_npf_hdl);
    sprintf(interfaceRoot, "%s",ifname);
    return (NPF_NO_ERROR);
}

/****************************************************************************
 * Function: IfGetPortAttribs()                                       	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		None							    *
 *  							    		    *
 * Description: 							    *
 *             	This function returns Port attributes of an Interface       *
 * Return: 								    *
 *		NPF_E_UNKNOWN - On Error				    *
 *		NPF_NO_ERROR  - On no Error				    *
 ****************************************************************************/
NPF_error_t IfGetPortAttribs(NPF_IfHandle_t ifHandle, 
				int *portId, 
				NPF_IfType_t *portType, 
				NPF_uint16_t *mtu)
{
    If_t        *interface = NULL;
    if(getInterface(ifHandle, &interface) != NPF_NO_ERROR)
    {
        return NPF_E_UNKNOWN;
    }
    switch(interface->npfIf.type)
    {
        case NPF_IF_TYPE_LAN:
        {
            If_t    *l3If;
            /* Always considered first FE in the list */
            *portId = interface->npfIf.u.LAN_Attr.port;
            *portType = NPF_IF_TYPE_LAN;
            if(interface->l3If != NULL)
            {
                l3If = (If_t *)interface->l3If;
                /* Always considered first FE in the list */
                *mtu = l3If->npfIf.u.IPv4_Attr.mtu;
	    }
        }
        break;
        case NPF_IF_TYPE_ATM:
        {
            If_t    *l3If;
            /* Always considered first FE in the list */
            *portId = interface->npfIf.u.ATM_Attr.port;
            *portType = NPF_IF_TYPE_ATM;
            if(interface->l3If != NULL)
            {
                l3If = (If_t *)interface->l3If;
                /* Always considered first FE in the list */
                *mtu = l3If->npfIf.u.IPv4_Attr.mtu;
	    }
	}
	break;
        case NPF_IF_TYPE_POS:
        {
            If_t    *l3If;
            /* Always considered first FE in the list */
            *portId = interface->npfIf.u.POS_Attr.port;
            *portType = NPF_IF_TYPE_POS;
            if(interface->l3If != NULL)
            {
                l3If = (If_t *)interface->l3If;
                /* Always considered first FE in the list */
                *mtu = l3If->npfIf.u.IPv4_Attr.mtu;
	    }
        }
        break;
        case NPF_IF_TYPE_IPV4:
        case NPF_IF_TYPE_IPV6:
        case NPF_IF_TYPE_IPV6INV4:
        {
            if(interface->l2If != NULL)
            {
                If_t    *l2If;
                l2If = (If_t *)interface->l2If;
                /* Always considered first FE in the list */
                *portId = l2If->npfIf.u.LAN_Attr.port;
                *portType = NPF_IF_TYPE_IPV4;
                *mtu = interface->npfIf.u.IPv4_Attr.mtu;
            }
            else
            {
                return(NPF_E_UNKNOWN);
            }
        }
        break;
        default:
            return(NPF_E_UNKNOWN);
       
    }
    return (NPF_NO_ERROR);
}

/****************************************************************************
 * Function: IfGetFEIdsFrmFIBHdl()                                       	    	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		None							    *
 *  							    		    *
 * Description: 							    *
 *             	This function returns FE id associated with the FIB Table *
 * Return: 								    *
 *		NPF_E_UNKNOWN - On Error				    *
 *		NPF_NO_ERROR  - On no Error				    *
 ****************************************************************************/
NPF_error_t IfGetFEIdsFrmFIBHdl(NPF_HANDLE fib_npf_hdl, uint32_t *numFe, FPPI_FEID *feId)
{   
    NPF_NSHDL           fib_ns_hdl = 0, nsNodeHdl = 0, fe_ns_hdl = 0;
    NPF_HANDLE          fe_npf_hdl= 0;
    char                nsNodePath[256];
    NODE_ITERATOR       assocItr;
    NPF_DATATYPE       npf_data_type;
    uint32_t           numAssoc=0,count=0;
    PDK_FE_t           *feInfo;

    IF_FUNCTION();
     
    /* Check if the Forwarding Handle is correct by finding it as a router
       in Namespace  */
    if((fib_npf_hdl == 0) || (cm_getTypeByHandle(fib_npf_hdl,&npf_data_type) != NPF_SUCCESS))       
    {
	   return NPF_IF_E_INVALID_FIB_HANDLE;
    }
    
    if( npf_data_type != NPF_IPROUTER)
    {
	   return NPF_IF_E_INVALID_HANDLE;
    }
 
    if(cm_getNSHDLByHandle(fib_npf_hdl, &fib_ns_hdl) != NPF_SUCCESS)
    {
	    IF_ERROR("*** unable to get NS handle for FIB node with npf handle %d\n *** ", fib_npf_hdl);
	    return NPF_E_UNKNOWN;
    }
       
    /* Check if this router is alreay associated with any FE */
    /* Get associations for Router Node ; It should have NO Assocations with FE*/
    if(npf_ns_enumAssociate(fib_ns_hdl, &assocItr) != NPF_SUCCESS)
    {
	    IF_INFO("*** unable to enumerate associations for FIB node with handle: %d\n *** ", fib_npf_hdl);
       *numFe =0; /* Assign the number as zero */
	    return NPF_SUCCESS;
    }

    /* If assocations are there then check for FE associations, otherwise create*/ 
    if(assocItr)
    { 
       numAssoc = npf_ns_count(assocItr);
       IF_INFO("IPv4 FIB has (%d) Assosiations", numAssoc);
       nsNodeHdl = npf_ns_first(assocItr);
       count = 0;
       while(nsNodeHdl)
       {
          npf_ns_getPath(nsNodeHdl, nsNodePath);
         /* We are interested only in FE Type*/
          npf_ns_getDataType(nsNodeHdl, &npf_data_type);
 
          IF_INFO("Assoc[%d] : Node = 0x%x, Fib Path = %s ",count,
                       nsNodeHdl, nsNodePath);
	       if(npf_data_type == NPF_FE)
	       {
             fe_ns_hdl = nsNodeHdl;
             feInfo = NULL ; /* safe check */
             /* First, get the data handle */
             npf_ns_getDataHandle(fe_ns_hdl, (NPF_HANDLE *)&fe_npf_hdl);
             /* Get FeId From the handle and store it */
             if ((cm_getDataByHandle(fe_npf_hdl, (NPF_DATA *)&feInfo) != NPF_SUCCESS) || (!feInfo))
             { 
                 IF_ERROR("Unable to get datatype from the FE handle %d", fe_npf_hdl);
	              return NPF_E_UNKNOWN;
             }
             /* Storing the FeIds */
             feId[count] = feInfo->id;
             count++;
             break;
          }
         npf_ns_close(nsNodeHdl);
         nsNodeHdl = npf_ns_next(assocItr);
       }
      
       /* Get the association between Router/FIB and FE; We can't have more than one associations */
       if(count > 0 )
       {
          *numFe = count;
          npf_ns_close(nsNodeHdl);
	       return(NPF_SUCCESS);
       }
    } /* If any assoc. existed */
    return (NPF_NO_ERROR);
}


/****************************************************************************
 * Function: IfGetFEid()                                       	    	    *
 * ---------------------------                                              *
 * Parameters: 								    *
 *		None							    *
 *  							    		    *
 * Description: 							    *
 *             	This function returns FE id associated with the Interface   *
 * Return: 								    *
 *		NPF_E_UNKNOWN - On Error				    *
 *		NPF_NO_ERROR  - On no Error				    *
 ****************************************************************************/
NPF_error_t IfGetFEid(NPF_IfHandle_t ifHandle, uint32_t *no, FPPI_FEID *feId)
{
    uint32_t    count = 0;
    If_t        *ifc = NULL;
    if(getInterface(ifHandle, &ifc) != NPF_NO_ERROR)
    {
        return NPF_E_UNKNOWN;
    }
    *no = ifc->FE.count;
    for(count = 0; count < ifc->FE.count; count++)
        feId[count] = ifc->PORT.portsList[count].fe_id;
    return (NPF_NO_ERROR);
}

#ifdef _INCLUDE_ATM
/*****************************************************************************
*    Name: ifAtmOnCB_AtmEvent 
*
*    Description:
*			This function is called by the FP-Plugin module on receiving the 
*			NPF_IF_ATM_EVENT_OAM_AIS_ALARM event from the FP module. 
*
*    Parameters:
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
********************************************************************************/
void ifAtmOnCB_AtmEvent(
					FPPI_FEID       feId,
					FPPI_CORRELATOR correlator,
					FPPI_CONTEXT    context,
					void            *responseData)
{
	NPF_ATM_EventArray_t eventArray;
	NPF_ATM_EventData_t *eventData = (NPF_ATM_EventData_t *)responseData;
	NPF_IfHandle_t ifHandle;
	FPPI_PortID portId;
	eventCallbackInfo_t *eventCallBackInfo = NULL;
	DListItr      *eventItr = NULL;


	/* Get the Port Id hidden as interface_handle in the received event Data */
	portId = eventData->if_handle;

	/* Get the Interface handle from the FE and Port Id */

	/* Set the interface handle value in the eventData structure */
	eventData->if_handle = ifHandle;

	/* Fill up the eventArray Structure */
	eventArray.n_data = 1;
	/* Allocate memory for the eventData member */
	eventArray.eventData = PIL_AllocateMemory
					(sizeof(NPF_IfEventData_t),
					 PIL_MEM_ZERO_INIT);

	/* Copy the event information */
	memcpy(eventArray.eventData,eventData,sizeof(NPF_IfEventData_t));

	/* For all the users that have registered */
	eventItr = npf_list_itrCreate (&EventCallbacks);
	npf_list_itrFirst (eventItr);
	do
	{
		eventCallBackInfo = (eventCallbackInfo_t *)npf_list_itrGetData (eventItr);
		cbthread_pushIFEvent(eventCallBackInfo->callback,
						eventCallBackInfo->context,(NPF_IfEventArray_t *)&eventArray);
	} while (npf_list_itrNext (eventItr) != -1);
	npf_list_itrDelete (eventItr);

	return;
}
#endif /* _INCLUDE_ATM */


