/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/


/*****************************************************************************
* $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/APIImplementations/IPv4/inc/ipv4_macros.h,v $
* $Revision: 1.6 $
* $Date: 2003/11/17 16:57:10 $
*
* Description:
*     A basic set of macros that is used internally by the IPv4 component.
*
*****************************************************************************/

#ifndef IPV4_MACROS_H
#define IPV4_MACROS_H


#define IPV4_DBG_TRACE() npf_logger_Write(LOG_IPV4, LOG_TRACE, "%s\n", "entering");
#ifdef RETURN
#undef RETURN
#endif
#define RETURN( val ) \
    { \
        npf_logger_Write(LOG_IPV4, LOG_TRACE, "%s\n", "leaving"); \
        return val; \
    }

#define IPV4_DBG_PRINT( lvl, fmt, args... ) \
        npf_logger_Write(LOG_IPV4, lvl, fmt, ## args)

#define IPV4_LOCK() { PIL_EnterCriticalSection( &ipv4Mgr.lock);\
        npf_logger_Write(LOG_IPV4, LOG_TRACE, "%s\n", "Mgr Lock is SET");} 
#define IPV4_UNLOCK() { PIL_LeaveCriticalSection( &ipv4Mgr.lock); \
        npf_logger_Write(LOG_IPV4, LOG_TRACE, "%s\n", "Mgr Lock is RESET");} 

#define USER_LOCK() { PIL_EnterCriticalSection( &userInfo->user_lock );\
        npf_logger_Write(LOG_IPV4, LOG_TRACE, "%s\n", "User Lock is SET");} 
#define USER_UNLOCK() { PIL_LeaveCriticalSection( &userInfo->user_lock ); \
        npf_logger_Write(LOG_IPV4, LOG_TRACE, "%s\n", "User Lock is RESET");} 


/*****************************************************************************
* Name: CHK_INIT
*
* Description:
*     Utility macro used to check if the
*     component is properly initialized.
*
*/
#define CHK_INIT() \
    if ( !ipv4Mgr.initialized) {\
        npf_logger_Write(LOG_IPV4, LOG_ERR, "%s\n", "Not Initialized "); \
        return NPF_COMPONENT_UNINITIALIZED;\
    }


/*****************************************************************************
* Name: CHK_UNINIT
*
* Description:
*     Utility macro used to check if the component is uninitialized.
*
*/
#define CHK_UNINIT() \
    if ( ipv4Mgr.initialized) {\
        npf_logger_Write(LOG_IPV4, LOG_ERR, "%s\n", "Already Initialized "); \
        return NPF_ALREADY_INIT;\
    }


#endif
