/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/


/*****************************************************************************
* $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/APIImplementations/IPv4/inc/ipv4_types.h,v $
* $Revision: 1.6 $
* $Date: 2003/11/17 16:57:11 $
*
* Description:
*     Definitions of data structures used internally by IPv4 component. These
*     structures are not exposed externally.
*
*****************************************************************************/

#ifndef IPV4_TYPE_H
#define IPV4_TYPE_H

/***** 
 * Hash defines 
 ******/

#define IPV4_MTU 1500 #define IPV4_MAX_ADDR 256
#define IPV4_NS_MAX_PATH 256
#define IPV4_MAX_NUM_TABLES 4  /* To support Unified */
#define IPV4_MAX_CBACK_NUM  12  /* Check the FPP APIs */
#ifdef IPV4_UNIF_SUPPORT
#define IPV4_MAX_MAP_TRANS 5
#endif

/* Forward references */
typedef struct IPV4UserInfo   IPV4UserInfo_t;   
typedef struct IPV4TableInfo  IPV4TableInfo_t;   
typedef struct IPV4TransInfo  IPV4TransInfo_t; 

/*****************************************************************************
* Name: IPV4TransType_t 
*
* Description:
*     This enum structure holds the type of transactions : Unified or Discrete 
* Parameters:
*
*/

typedef enum {
	IPV4_TRANS_DISC
#ifdef IPV4_UNIF_SUPPORT
	,IPV4_TRANS_UNIF
#endif /* IPV4_UNIF_SUPPORT */
}IPV4TransType_t;


/*****************************************************************************
* Name: IPV4TableType_t 
*
* Description:
*     This data structure holds the information related to various types 
*     of IPv4 Tables like Prefix, Next-Hop, Arp-Request and 
*     FIB Table.  
* Parameters:
*
*/

typedef enum {
	IPV4_TABLE_TYPE_PREFIX = 0,
	IPV4_TABLE_TYPE_NHOP = 1,
	IPV4_TABLE_TYPE_ADDRES = 2,
	IPV4_TABLE_TYPE_FIB = 3,
	IPV4_TABLE_MAX_NUM= 4
} IPV4TableType_t;



/************
 * Data Structures to depict requests from Usr to CP 
 ************/


#ifdef IPV4_UNIF_SUPPORT
typedef enum {
    UNIF_ADD_QUERY_PROGRESS = 1,
    UNIF_ADD_PREFIXENTRY_FOUND,
    UNIF_ADD_PREFIXENTRY_NOTFOUND,
    UNIF_ADD_DELPREFIX_PROGRESS,
    UNIF_ADD_ADDNHOP_PROGRESS,
    UNIF_ADD_ADDPREFIX_PROGRESS,
    UNIF_DEL_QUERY_PROGRESS,
    UNIF_DEL_NHOP_PROGRESS,
    UNIF_DEL_PREFIX_PROGRESS,
    UNIF_FIB_QUERY_PREFIX_PROGRESS,
    UNIF_FIB_QUERY_NHOP_PROGRESS
}IPV4UnifReqState_t;

typedef enum {
    IPV4_USR_UNIF = 1,
    IPV4_USR_DISC
}IPV4UserType_t;
#endif
/*****************************************************************************
* Name: IPV4NhopCntxt_t
*
* Description:
*
* Parameters:
*
*/
typedef struct {
    NPF_uint32_t               numEntries;
    NPF_IPv4UC_NextHopArray_t *nhopArray;
    NPF_uint32_t              *nhIdArray;   
}IPV4NhopCntxt_t; 

#ifdef IPV4_UNIF_SUPPORT
/*****************************************************************************
* Name: IPV4PrefixCntxt_t
*
* Description:
*
* Parameters:
*
*/
typedef struct {
    NPF_uint32_t               numEntries;
    NPF_IN NPF_IPv4UC_Prefix_t *prefixArray;
}IPV4PrefixCntxt_t;
#endif

/*****************************************************************************
* Name: IPV4AddResCntxt_t
*
* Description:
*
* Parameters:
*
*/
typedef struct {
    NPF_uint32_t                    numEntries;
    NPF_IN NPF_IPv4UC_AddResKey_t   *addResKeyArray;
}IPV4AddResCntxt_t;


/*****************************************************************************
* Name: IPV4UsrReqInfo_t
*
* Description:
*/
typedef struct {
    NPF_correlator_t        correlator;    /* User Correlator */
    NPF_errorReporting_t    appVerbosity;  /* Requested Verbosity */
    IPV4TableType_t         tableType;     /* Table Type */
    IPV4TransType_t         transType;     /* Trans. Type */
    NPF_IPv4UC_CallbackType_t reqType;     /* Req Type */
    NPF_callbackHandle_t    callbackHandle;/* Callback Type */
    NPF_uint32_t            tblHandle;     /* Table Handle */
    NPF_uint32_t            bndTblHandle;  /* To support BIND API */
    IPV4NhopCntxt_t         *nhopCntxt;    /* Next-Hop Context */
    IPV4AddResCntxt_t       *addResCntxt;  /* AddRes Context */
#ifdef IPV4_UNIF_SUPPORT
    IPV4PrefixCntxt_t       *prefixCntxt;  /* Prefix Context */ 
#endif
    IPV4TransInfo_t         *transInfo;    /* Ptr to Trans */
} IPV4UsrReqInfo_t;


/************
 * Data Structures to depict requests from CP to FP 
 ************/



/*****************************************************************************
* Name: 
*
* Description:
*     Enumeration used by xxx to track what the state of an
*     FE communication is.
*
* Parameters:
*     PENDING        - No communication with the FE yet.
*     SENT_TO_FE     - A message has been sent to the FE but no response
*                      received.
*     CALLBACK_RECVD - Reply has been received from the FE.
*
*/
typedef enum {
    REQ_TO_FE_PENDING,
    REQ_TO_FE_SENT,
    REQ_TO_FE_CBACK_RECVD
} IPV4ReqToFEState_t;

/*****************************************************************************
* Name: IPV4ReqToFEInfo_t
*
* Description:
*     Keeps track of state for a request sent to a single FE.
*
* Parameters:
*     fppi_correlator - The correlator associated with the pending request.
*     state           - The status of the call(see IPV4ReqToFEState_t).
*     feId            - The FEID of the FE that this contact is associated with.
*
*/
typedef struct {
    FPPI_CORRELATOR 	  fppi_correlator; /* Unique Key to the Req to FP */
    IPV4ReqToFEState_t state;           /* State of Req to the FE */
    FPPI_FEID 		     feId;            /* Req	to one FeId */
    FPPI_RET		     retStatus;       /* Return Status from FPPI */
    IPV4TransInfo_t    *transInfo;      /* Pointer to Trans Info */
} IPV4ReqToFEInfo_t;

#ifdef IPV4_UNIF_SUPPORT
/*****************************************************************************
* Name: IPV4MapTransInfo_t
*
* Description:
*     Holds pointers to the IPV4TransInfo_t structures that are created within
*     the IPV4 Layer on sending Discrete API requests.
*
* Parameters:
*
*/
typedef struct {
	uint32_t          numMapTrans;    /* Number of related trans */
	IPV4TransInfo_t   *mapTransArray[IPV4_MAX_MAP_TRANS]; /* Pointers to those trans */
   IPV4UnifReqState_t unifReqState;
   NPF_IPv4UC_CallbackData_t cbResp;
} IPV4MapTransInfo_t;
#endif /* IPV4_UNIF_SUPPORT */


/*****************************************************************************
* Name: IPV4TransFPInfo_t
*
* Description:
*     Keeps track of all the requests sent to a multiple FEs.
*
* Parameters:
*
*/
typedef struct {
   uint32_t   numFEWait;
   uint32_t   numFEToSend;
   IPV4ReqToFEInfo_t* reqFEPtrArray[MAXFENUM];
}IPV4TransFPInfo_t;
 
/*****************************************************************************
* Name: IPV4TransInfo_t 
*
* Description:
*     This data structure holds the information related to request from the
*     user to IPv4 module to FP. Each request from User can trigger multiple
*     requests to different FEs. 
*     This transaction is uniquely identified by the combination of
*     usr_correlator + fppi_correlator
* Parameters:
*
*/

struct IPV4TransInfo{
    NPF_uint32_t	    transPend;      /* Is transactin pending ? */
    NPF_correlator_t  usr_correlator;  /* Usr correlator: Key for trans */
    IPV4UsrReqInfo_t	*usrReq;    /* Request from User */
    IPV4UserInfo_t	*usrInfo;   /* Quick access to user context */
#ifdef IPV4_UNIF_SUPPORT
    NPF_correlator_t    int_correlator;  /* Internal correlator: For Unif */
    IPV4MapTransInfo_t	*mapTransInfo;    /* Mapped Transactions */
#endif /* IPV4_UNIF_SUPPORT */
    IPV4TransFPInfo_t   transFpInfo;  /* Transaction to FP Info */
   /* Target Table */
    IPV4TableInfo_t		   *tableInfo;  /* Table for trans. */
};


/*****************************************************************************
* Name: IPV4TableState_t 
*
* Description:
*     This enum structure holds the information related to various states
*     which a typical IPv4 Table like Prefix, Next-Hop, Arp-Request and 
*     FIB Table go through.  
* Parameters:
*
*/


typedef enum {
	TABLE_INIT =0,
	TABLE_CREATED =1,
	TABLE_ENTRIES_EXIST =2,
	TABLE_EMPTY =3,
	TABLE_FULL =4,
	TABLE_DELETED =5
} IPV4TableState_t;



/*****************************************************************************
* Name: IPV4TableHndlMap_t 
*
* Description:
*     This data structure holds the pointers to the tables for reference.  
* Parameters:
*
*/
typedef struct {
	IPV4TableInfo_t    *pfxTableInfo;
   IPV4TableInfo_t    *nhopTableInfo;
} IPV4TableHndlMap_t;

	
/*****************************************************************************
* Name: IPV4TableInfo 
*
* Description:
*     This data structure holds the information related to IPv4 Tables like
*     Prefix, Next-Hop, Arp-Request and FIB Table. Each data-struc. is
*     identified uniquely by a handle.     
* Parameters:
*
*/
struct IPV4TableInfo{
	NPF_uint32_t		intHandle; /* Internal Table Handle */
	NPF_uint32_t		extHandle; /* External Table Handle */
	IPV4TableState_t	state;  /* Table State */	
	IPV4TableType_t         type;   /* Table Type */
	IPV4TableHndlMap_t      tblHndlMap; /* Table Map; References to other tables */ 
        uint32_t                numFeTotal;
        FPPI_FEID               feIdArray[MAXFENUM];
};


/*****************************************************************************
* Name: IPV4UserInfo_t
*
* Description:
*     This data structure holds the information related to IPv4 users 
*     e.g., various call-backs, table-handles etc. This structure is created 
*     whenever a user registers itself with IPv4 Module and deleted when 
*     the user de-registers itself or when this module is shutdown. User 
*     is uniquely identified using the context it provides to this module.
*     
* Parameters:
*
*/

struct IPV4UserInfo{
    NPF_userContext_t      	 context;   /* User Cntxt : Unique Key per User*/
    NPF_callbackHandle_t    	 cbHndl; /* Callback handle returned per usr*/
    PilCriticalSection         user_lock; /* User Lock for User App*/
    NPF_IPv4UC_CallbackFunc_t  cbFunc; /* User regd. CBack function */
#ifdef IPV4_UNIF_SUPPORT
    NPF_uint32_t               nhIdCntr;  /* Counter for internal NHId */
    NPF_uint32_t               intCorrelCntr;/* Counter for internal Correlator*/
    IPV4UserType_t             usrType;
    DList                      unifiedTransCpList; /* Transaction list for
                                        unified case */
#endif /* IPV4_UNIF_SUPPORT */
    DList                      transCpList; /* IPV4 Transaction  List */
    NPF_uint32_t               numTables;
    IPV4TableInfo_t            *tableArray; /* Array of tables per user */
};

#ifdef IPV4_EVENT_HANDLING
typedef struct{
    NPF_IPv4UC_EventCallFunc_t  *evntCallFunc; /* Evnt Hndlr Regd. by app.*/
    NPF_IPv4UC_EventArray_t     eventArray;     /* IPv4 events registered */
    NPF_callbackHandle_t 	eventCbHndl; /* Unique Callback Handle for : 
				                eventUsrCntxt+ evntCallFunc.*/	
}IPV4EventHndlrInfo_t;

/* NOTE: Event Handler is supposed to be written by the application */
typedef struct {
    NPF_userContext_t     context;   /* User Cntxt for Event regis. */
    DList			        hndlrList; /* List of event Handlers regis.*/
}IPV4EventUserInfo_t;
#endif /* IPV4_EVENT_HANDLING */

/*****************************************************************************
* Name: IPV4FppCBIndex_t
*
* Description:
*     This enum structure holds the index for each FP Callbacks which are
*     registered with the Forwarding Plane.
*
*     NOTE: Change in this should affect function : ipv4_FillCBInfoToFpp()
*
* Parameters:
*
*/
typedef enum{
   IPV4_FP_CB_ADD_NHOP=0,
   IPV4_FP_CB_DEL_NHOP=1,
   IPV4_FP_CB_FLUSH_NHOP=2,
   IPV4_FP_CB_QUERY_NHOP=3,

   IPV4_FP_CB_ADD_PFX=4,
   IPV4_FP_CB_UPD_PFX=5,
   IPV4_FP_CB_DEL_PFX=6,
   IPV4_FP_CB_FLUSH_PFX=7,
   IPV4_FP_CB_QUERY_PFX=8,

   IPV4_FP_CB_ADD_L2=9,
   IPV4_FP_CB_DEL_L2=10,
   IPV4_FP_CB_FLUSH_L2=11

}IPV4FppCBIndex_t;


/*****************************************************************************
* Name: IPV4FppCBInfo_t
*
* Description:
*     This data structure holds the information related to Callbacks which are
*     registered with the Forwarding Plane.
*     It is initialized when IPV4 PDK module is initialized and deleted when
*     this module is shutdown.
* Parameters:
*
*/
typedef struct {
	NPF_CBCAT      cbType; /* Callback Type */
	void          *cbFunc;/* Callback Function Pointer */
	FPPI_CBHANDLE cbHndl; /* Callback Handle - returned by FPP */
}IPV4FppCBInfo_t;
	

/*****************************************************************************
* Name: IPV4FppInfo_t
*
* Description:
*     This data structure holds the information related to Forwarding Plane 
*     Plugin like various call-backs which IPv4 PDK module registers .
*     It is created when this module is initialized and deleted when
*     this module is shutdown.
* Parameters:
*
*/
typedef struct {
	FPPI_CORRELATOR fppiCorrCntr; /* Counter incremented for new FPPI-req */
	DList           reqFpList;    /* List of requets sent to FPP */
        NPF_uint32_t    numFpCB;      /* Number of CBacks to be registered */
	IPV4FppCBInfo_t *cbInfoArray; /* Array of struc. - FPP CBInfo */
}IPV4FppInfo_t;

/*****************************************************************************
* Name: IPV4IfAttrib_t
*
* Description:
*     This data structure holds the information related to egress If that is
*     fetched from NS. This is a transient information.
*
* Parameters:
*
*/
typedef struct {
   NPF_IfHandle_t ifHndl;
   uint32_t       numFe;
   FPPI_FEID      feId[MAXFENUM]; 
   uint32_t       portId; 
   NPF_IfType_t   portType; 
   uint16_t       mtu;
}IPV4IfAttrib_t;


/*****************************************************************************
* Name: IPV4Mgr_t
*
* Description:
*     This data structure holds the information about the whole IPv4 PDK
*     Module. It is created when this module is initialized and deleted when
*     this module is shutdown.
*     This is the first data-structure that gets created and everything hangs
*     from here.
* Parameters:
*
*/

typedef struct {
    uint32_t 	    	initialized; /* IPv4 Initialized : TRUE/FALSE */
    PilCriticalSection  lock;        /* Lock for IPv4 variables */
    NPF_uint32_t	    	hndlCntr;   /* Global counter to assign unique Ids*/
    DList               userList; /* PDK User List : IPV4UserInfo_t */
#ifdef IPV4_EVENT_HANDLING
    DList	            eventUserList; /* IPV4EventUserInfo_t List */
#endif /* IPV4_EVENT_HANDLING */
    IPV4FppInfo_t   	   fppInfo; /* Forwarding Plane Plugin (FPP) info */
} IPV4Mgr_t;

#endif /* IPV4_TYPE_H */
