/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001-3 Intel Corporation. All Rights Reserved.
*****************************************************************************/


/*****************************************************************************
* $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/APIImplementations/IPv4/src/ipv4_modules.c,v $
* $Revision: 1.8.2.1 $
* $Date: 2004/03/17 09:51:57 $
*
* Description:
*     Contains helper functions to  support NPF API calls implementations in
*     the file npf_ipv4_api.c
*
*****************************************************************************/

/* Standard C Libraries */
#include <string.h>
#include <assert.h>
#include <stdio.h>

/* FPP libraries */
#include "fpp_common.h"
#include "fppCapabilityTypes.h"

/* Helper libraries */
#include "log.h"
#include "list.h"
#include "pil.h"

/* Helper PDK Framework header files */
#include "pdk_callback.h"
#include "pdk_ns_helper.h"
#include "npf_common.h"
#include "pdk_cm.h"

/* Files written as a part of IPv4 Component */
#include "npf_ipv4_types.h"
#include "npf_ipv4_api.h"
#include "npf_ns_api.h"
#include "ipv4_types.h"
#include "ipv4_macros.h"
#include "ipv4_modules.h"

/* Include other Modules that are to be called */
#include "pdkmgr.h"
#include "pdk_l2idm.h"


/*********
 More details of the Interface 
**********/
#ifdef LOG_ENABLE
char *portName[] = {"invalid","unknown", "LAN", "ATM","POS","IPV4","IPV6","IPV6","IPV6INV4","MPLS","NewIf"}; 
#endif

/*****************************************************************************
*   Extern Main Data Structure
*****************************************************************************/

extern IPV4Mgr_t	ipv4Mgr;

/*****************************************************************************
* Function Declaration  
*****************************************************************************/
NPF_RET ipv4_GetTransByCorrelFrmUsr(IPV4UserInfo_t *userInfo, NPF_correlator_t  correl, IPV4TransInfo_t **transInfo);

#ifdef IPV4_UNIF_SUPPORT
NPF_RET ipv4_GetICorrelFrmUsr( IPV4UserInfo_t *userInfo, NPF_uint32_t *correl);
#endif /* IPV4_UNIF_SUPPORT */
NPF_RET ipv4_DelTransInListFrmUsr(IPV4TransInfo_t *transInfo) ;

NPF_RET ipv4_AddTransInListToUsr( IPV4UserInfo_t  *userInfo, IPV4TransInfo_t *transInfo) ;
NPF_RET ipv4_DelNhopCntxt( IPV4NhopCntxt_t   *nhopCntxt);
NPF_RET ipv4_DelAddResCntxt( IPV4AddResCntxt_t  *addResCntxt);

/* IF Management Functions */
/* Helper functions */
extern NPF_error_t IfGetPortAttribs(NPF_IfHandle_t ifHandle, int *portId, NPF_IfType_t *portType, NPF_uint16_t *mtu);
extern NPF_RET pdk_ns_getNSRootHandle( NPF_NSHDL *root_ns_hdl);

extern NPF_RET pdk_ns_getChildByType(NPF_HANDLE parent_npf_hdl,NPF_DATATYPE child_type, char* child_name, NPF_NSHDL *child_hndl);

extern NPF_error_t IfGetFEIdsFrmFIBHdl(NPF_HANDLE fib_npf_hdl, uint32_t *numFe, FPPI_FEID *feId);

extern NPF_error_t IfGetFEid(NPF_IfHandle_t ifHandle, uint32_t *no, FPPI_FEID *feId);

 
/* Callback Functions */

void ipv4_AddPrefixesComplete(FPPI_FEID feid, FPPI_CORRELATOR correlator, 
FPPI_CONTEXT context, void *response );

void ipv4_DeletePrefixesComplete(FPPI_FEID feid, FPPI_CORRELATOR correlator, 
FPPI_CONTEXT context, void *response );

void ipv4_FlushPrefixesComplete(FPPI_FEID feid, FPPI_CORRELATOR correlator, 
FPPI_CONTEXT context, void *response );

void ipv4_QueryPrefixesComplete(FPPI_FEID feid, FPPI_CORRELATOR correlator, 
FPPI_CONTEXT context, void *response );

void ipv4_UpdatePrefixesComplete(FPPI_FEID feid, FPPI_CORRELATOR correlator,
FPPI_CONTEXT context, void *response );

void ipv4_AddNextHopComplete(FPPI_FEID feid, FPPI_CORRELATOR correlator,
FPPI_CONTEXT context, void *response );

void ipv4_DeleteNextHopComplete(FPPI_FEID feid, FPPI_CORRELATOR correlator,
FPPI_CONTEXT context, void *response );

void ipv4_FlushNextHopComplete(FPPI_FEID feid, FPPI_CORRELATOR correlator,
FPPI_CONTEXT context, void *response );

void ipv4_QueryNextHopComplete(FPPI_FEID feid, FPPI_CORRELATOR correlator,
FPPI_CONTEXT context, void *response );

void ipv4_AddL2EntryComplete(FPPI_FEID feid, FPPI_CORRELATOR correlator,
FPPI_CONTEXT context, void *response );

void ipv4_DeleteL2EntryComplete (FPPI_FEID feid, FPPI_CORRELATOR correlator, 
FPPI_CONTEXT context, void *response );

void ipv4_FlushL2EntryComplete (FPPI_FEID feid, FPPI_CORRELATOR correlator, 
FPPI_CONTEXT context, void *response );


/*
   ############################################################################
   IPV4 List Element Comparator Helper Functions
   ############################################################################
*/


/**********************************************************************
*    Name: list comparator functions
*   
*    Description:
*        
*    Parameters:
*        IN: data - element of the list
*            key  - search key
*        
*    Preconditions:
*        
*        IN:
*    Postconditions:
*        
*    
*    Returns:
*        1 : There is a match
*        0 : There is no match 
*    
*/
static int ipv4_CompareReqToFPList( 
		const void *data, 
		const void *key
		)
{
   IPV4ReqToFEInfo_t *reqToFE;
   FPPI_CORRELATOR   *correl;
   
   reqToFE = (IPV4ReqToFEInfo_t*)data;
   correl  = (FPPI_CORRELATOR*)key; 
   return ( reqToFE->fppi_correlator == *correl);
}


static int ipv4_CompareUserList( 
		const void *data, 
		const void *key
		)
{
   IPV4UserInfo_t *userInfo;
   NPF_HANDLE     *cbHndl;

   userInfo = (IPV4UserInfo_t*)data;
   cbHndl   = (NPF_HANDLE*)(key);

   return ( userInfo->cbHndl == *cbHndl);
}

#ifdef IPV4_EVENT_SUPPORT
static int ipv4_CompareEventUserList( 
		const void *data, 
		const void *key
		)
{
   IPV4EventUserInfo_t *evtUserInfo;
   NPF_HANDLE           cbHndl;

   evtUserInfo = (IPV4EventUserInfo_t*)data;
   cbHndl   = (NPF_HANDLE)(*key);

   return ( evtUserInfo->cbHndl == cbHndl);
}
#endif /* IPV4_EVENT_SUPPORT */

static int ipv4_CompareTransCpList( 
		const void *data, 
		const void *key
		)
{
   IPV4TransInfo_t  *transInfo;
   NPF_correlator_t *correl;

   transInfo = (IPV4TransInfo_t*) data;
   correl    = (NPF_correlator_t*) (key);

   return (transInfo->usr_correlator == *correl);
}

#ifdef IPV4_UNIF_SUPPORT
static int ipv4_CompareUnifiedTransCpList( 
		const void *data, 
		const void *key
		)
{
   IPV4TransInfo_t  *transInfo;
   NPF_correlator_t *correl;

   transInfo = (IPV4TransInfo_t*) data;
   correl    = (NPF_correlator_t*) (key);

   if (transInfo->int_correlator > 0) /* If internal correlator is set
					 , compare it with this */
   {
      return (transInfo->int_correlator == *correl);
   }
   return 1;
}
#endif /* IPV4_UNIF_SUPPORT */


/*
   ############################################################################
   IPV4 Lists element Destroy Functions
   ############################################################################
*/


/**********************************************************************
*    Name: list element destroy functions
*   
*    Description:
*        
*    Parameters:
*        IN: data - element of the list
*        
*    Preconditions:
*        
*        IN:
*    Postconditions:
*        
*    
*    Returns:
*        1 : SUCCESS 
*        0 : FAILURE 
*    
*/



static void ipv4_FreeTransCpInfo(void *data)
{
   IPV4TransInfo_t *transInfo;
   
   transInfo = (IPV4TransInfo_t*)(data);
   if(transInfo == NULL || transInfo->usrReq == NULL)
      return ;
   if(ipv4_DeleteUsrReq(transInfo->usrReq) == NPF_SUCCESS)
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR,
                "Unable to delete Trans with correl %d ", 
                          transInfo->usr_correlator);
   }
}


static void ipv4_FreeReqToFP(void *data)
{
   IPV4ReqToFEInfo_t *reqToFE;
   
   reqToFE = (IPV4ReqToFEInfo_t*)(data);
   if(reqToFE == NULL)
      return;
   PIL_FreeMemory(reqToFE);
}


static void ipv4_FreeUserInfo(void *data)
{
   IPV4UserInfo_t *userInfo;

   userInfo = (IPV4UserInfo_t*)(data);
   assert(userInfo);
   assert(ipv4_RemoveUserInfo(userInfo) == NPF_SUCCESS);
}

#ifdef IPV4_EVENT_SUPPORT
static void ipv4_FreeEventUserInfo(void *data)
{
   IPV4EventUserInfo_t *evntUserInfo;
   evntUserInfo = (IPV4EventUserInfo_t *)(data);
   assert(evntUserInfo);
   /* Free any event callback handlers in the list */ 
   npf_list_destroy(&evntUserInfo->hndlrList);
   PIL_FreeMemory(evntUserInfo);
}
#endif /* IPV4_EVENT_SUPPORT */


/*
   ############################################################################
   IPV4 Namespace Interface Module Helper Functions
   ############################################################################
*/


/*****************************************************************************
*    Name: ipv4_GetFEIdAssocWithIfFrmNS
*   
*    Description:
*        Get the FEId corresponding to a given interface
*        
*    Parameters:
*        IN: egressInterfacedata - element of the list
*        OUT: feId - search key
*             portId - returned from search
*             mtu - returned from search
*
*    Preconditions:
*        
*    
*    Postconditions:
*        
*    
*    Returns:
*        void
*    
*/
NPF_RET ipv4_GetFEPortAttribForIfFrmNS(
		IPV4IfAttrib_t  *egressIf
		)
{
    NPF_RET    retVal;
    if((retVal= IfGetFEid(egressIf->ifHndl,&egressIf->numFe,egressIf->feId))
		    != NPF_SUCCESS)
    {
        npf_logger_Write(LOG_IPV4, LOG_ERR,
                "Could not find FEId associated with egressIf [%d]", 
                          egressIf->ifHndl);
	     RETURN(retVal);
    }
   
    /* Get the port details fromt the interface Handle */ 
    if((retVal = IfGetPortAttribs(egressIf->ifHndl,&egressIf->portId,
                                      &egressIf->portType,&egressIf->mtu)) != NPF_SUCCESS )
    {
       npf_logger_Write(LOG_IPV4, LOG_ERR,
                   "Unable to get port attributes");
       RETURN(retVal);
    }

    npf_logger_Write(LOG_IPV4, LOG_TRACE,
       "Egress If Details : FeId = %d,\n PortId = %d \n PortType = %s \n L3-MTU: %d \n",
          egressIf->feId[0],
          egressIf->portId,
          portName[egressIf->portType],
          egressIf->mtu);
    RETURN(NPF_SUCCESS);
} /* ipv4_GetFEPortAttribForIfFrmNS */



/*****************************************************************************
*    Name: ipv4_GetRtrAssocFrmNS
*   
*    Description:
*        Get an iterator to the associations for a Router node frm NS
*        
*    Parameters:
*        OUT: assoc_itr - The iterator for the associations.
*        IN:  ns_node   - The namespace handle.
*        
*    Preconditions:
*        assoc_itr != NULL
*    
*    Postconditions:
*        assoc_itr == iterator to associations if node has associations
*    
*    Returns:
*        The number of associations.
*    
*/   
NPF_RET ipv4_GetRtrAssocFrmNS( 
		NODE_ITERATOR *assoc_itr, 
		NPF_NSHDL     ns_node,
		uint32_t      *numAssoc
		)
{
    NPF_RET      retVal;
    char node_name[NPF_NS_MAXPATH];

    IPV4_DBG_TRACE();

    assert( assoc_itr );
    
    /* Zero the memory */
    memset(node_name, 0 , sizeof(node_name));

    npf_ns_getPath( ns_node, node_name );
    
    if (( retVal = npf_ns_enumAssociate( ns_node, assoc_itr ))
		    != NPF_SUCCESS )
    {
        npf_logger_Write(LOG_IPV4,  LOG_ERR, 
			"unable to enumerate associations for node\n" );
        RETURN( retVal );
    }
		
    if ( !(*assoc_itr) )
    {
        npf_logger_Write(LOG_IPV4,  LOG_ERR, 
			"node has no associations\n" );
        RETURN( NPF_E_UNKNOWN );
    }
    
    /* Get the number of associations */ 
    *numAssoc = npf_ns_count(*assoc_itr);

    RETURN(retVal);
} /* ipv4_GetRtrAssocFrmNS */



/*****************************************************************************
*    Name: ipv4_ChkFENodeFrmNS
*
*    Description:
*        Given a handle to a NS node, find if this node is a FE node. If YES,
*        then give the FEId of that node otherwise return FAILURE.
*    Parameters:
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
*/
NPF_RET ipv4_ChkFENodeFrmNS(
         NPF_NSHDL  ns_node_hndl,
         FPPI_FEID  *feid
		  )
{
    NPF_RET      retVal;
    NPF_HANDLE   ns_data_hndl;
    NPF_DATATYPE ns_node_type;
    IPV4_DBG_TRACE();
    
    /* Find the type of the node */ 
    if ((retVal = npf_ns_getDataType( ns_node_hndl, &ns_node_type)) != NPF_SUCCESS)
    {    
        npf_logger_Write(LOG_IPV4,LOG_ERR,
			"Unable to get type of NS node\n" );
        RETURN( retVal );
    }
    /* We are specifically looking for node which is FE */
    if ( ns_node_type != NPF_FE )
    {
        RETURN(NPF_E_UNKNOWN);
    }
    
    /* This is a FE node; Get Data Handle */ 
    if ( (retVal = npf_ns_getDataHandle( ns_node_hndl, 
				    &ns_data_hndl )) != NPF_SUCCESS )
    {
        npf_logger_Write(LOG_IPV4,LOG_ERR, 
			" Could not get data handle for FE\n" );
        RETURN(retVal);
    }

    /* Get FeId from the CM database for this FE */
    if ( (retVal = cm_fe_get_feid( ns_data_hndl, feid )) != NPF_SUCCESS )
    {
        npf_logger_Write(LOG_IPV4,  LOG_ERR, "Cannot get FEID for FE\n" );
        RETURN(retVal);
    }
    
    /* FEId extracted !! Close the data handle */
    npf_logger_Write(LOG_IPV4,  LOG_INFO, "FEID of FE is 0x%x\n", *feid );
    RETURN (NPF_SUCCESS);
} /* ipv4_ChkFENodeFrmNS */


/*****************************************************************************
*    Name: ipv4_DelAssocNodesForRtrFrmNS
*
*    Description:
*        Given a handle to a router find out all of the associations
*        which are associated with it .The total number of associations
*        (FEs) in the router is returned.
*
*    Parameters:
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
*/
NPF_RET ipv4_DelAssocNodesForRtrFrmNS(
      NPF_NSHDL rtr_ns_hndl,
      FPPI_FEID *feIdArray,
      uint32_t  *numFE
		)
{
    NPF_RET   retVal;
    NODE_ITERATOR assocItr;
    NPF_NSHDL ns_fe_node_hndl;
    int assocLoop;
    uint32_t numAssoc;
    char nodeName[NPF_NS_MAXPATH];
    FPPI_FEID tempFeId;

    IPV4_DBG_TRACE();
    assert( rtr_ns_hndl);
    assert( feIdArray);
    
    /* Initialize number of FEs found as zero */
    *numFE = 0;

    npf_logger_Write(LOG_IPV4, LOG_TRACE,
                     "Looking for FEs associated with router [handle 0x%x]\n", 
		     rtr_ns_hndl);

    /* Get the all associations of this router (includes FEs also )*/
    if( (retVal= ipv4_GetRtrAssocFrmNS( &assocItr, rtr_ns_hndl, &numAssoc))
		                       != NPF_SUCCESS)
    {
       npf_logger_Write(LOG_IPV4, LOG_ERR, "Error in getting associations for router\n" );
       RETURN( NPF_E_UNKNOWN);
    }
    /* If no associations are present, return */ 
    if ( numAssoc == 0 )
    {
        npf_logger_Write(LOG_IPV4, LOG_ERR, "Router has no associations\n" );
        RETURN( NPF_NS_NODE_NOTFOUND);
    }
    /* Associations for the router found */ 
    npf_logger_Write(LOG_IPV4, LOG_ERR,
                     "Router has %d associations\n", numAssoc);
    
    /* Go through all of the associations and look for the
     * node with FE attributes */
    ns_fe_node_hndl = npf_ns_first( assocItr );

    for ( assocLoop = 0; assocLoop< numAssoc; assocLoop++) 
    {
        if(assocLoop > 0) /* Not for first time */
          ns_fe_node_hndl = npf_ns_next( assocItr ); 
	     /* Is this NS node FE ? */
        if (ipv4_ChkFENodeFrmNS(ns_fe_node_hndl, 
					 &tempFeId)== NPF_SUCCESS)
	     {  
           /* Store the FeId in the array passed into thie function */
	        feIdArray[(*numFE)++]=tempFeId;
        
           memset(nodeName, 0 , sizeof(nodeName));
           /* Get the path of the FE */
           npf_ns_getPath( ns_fe_node_hndl, nodeName );
        
           /* Delete FE Node and Rtr Node Association from the Namespace */ 
           if((retVal = npf_ns_delAssociate(rtr_ns_hndl, ns_fe_node_hndl)) != NPF_SUCCESS)
           {
              npf_logger_Write(LOG_IPV4,  LOG_ERR,
                       "Unable to remove assocaition of FE Node \t%s\t \n", nodeName);
              RETURN(retVal);
           }
            /* Take out the assocation in reverse way also */ 
           if((retVal = npf_ns_delAssociate(ns_fe_node_hndl,rtr_ns_hndl)) != NPF_SUCCESS)
           {
              npf_logger_Write(LOG_IPV4,  LOG_ERR,
                       "Unable to remove assocaition of FE Node \t%s\t \n", nodeName);
              RETURN(retVal);
           }
           npf_logger_Write(LOG_IPV4,  LOG_TRACE,
                       "Success: FE Node Assocation removed from NS \t%s\t \n", nodeName);
           npf_ns_close( ns_fe_node_hndl);
	     } /* Should be only FE */
    }
    npf_ns_done( assocItr );

    RETURN (retVal);
} /* ipv4_DelAssocNodesForRtrFrmNS */


/*****************************************************************************
*    Name: ipv4_CheckRtrNodeFromNS 
*
*    Description:
*        Check the handle to the IPV4 router in the Namespace
*
*    Parameters:
*
*    Returns: NPF_SUCCESS: when IP router node is found
*             NPF_E_UNKNOWN when IP router node is not found
*/
NPF_RET ipv4_CheckRtrNodeFrmNS(char *rtr_name, NPF_NSHDL *rtr_hdl)
{
    NPF_RET   retVal;
    NPF_NSHDL sys_ns_hdl, rtr_type_hndl;
    NPF_HANDLE sys_npf_hdl;
    char name[NPF_NS_MAXPATH]; 
    
    IPV4_DBG_TRACE();
    
    /* Zero the memory */
    memset(name,0 , sizeof(name));
    
    /* Get the Namespace handle of the root node (System) */
    if((retVal = pdk_ns_getNSRootHandle (&sys_ns_hdl)) != NPF_SUCCESS)
        RETURN(retVal);     
    
    npf_ns_getPath(sys_ns_hdl, name);
    npf_logger_Write(LOG_IPV4, LOG_TRACE, 
		    "Namespace Root Node Path is %s\n", name);
    
    /* Get the Data Handle for the root node */ 
    if((retVal = npf_ns_getDataHandle(sys_ns_hdl, &sys_npf_hdl))
		    != NPF_SUCCESS)
    {
        npf_ns_close(sys_ns_hdl);  
	     RETURN(retVal);
    }
    /* Get the Node for iterating through the NS tree and select a
     * node of type NPF_IPROUTER */    
    if((retVal = pdk_ns_getChildByType(sys_npf_hdl, NPF_IPROUTER, 
				     rtr_name, &rtr_type_hndl))  != NPF_SUCCESS)
    {
        npf_ns_close(sys_ns_hdl);
        RETURN(retVal); 
    }
    npf_logger_Write(LOG_IPV4, LOG_TRACE, 
		    "Namespace Router Node Path is %s\n", rtr_name);
    *rtr_hdl = rtr_type_hndl;
    npf_ns_close(sys_ns_hdl);
    RETURN(NPF_SUCCESS);
} /* ipv4_CheckRtrNodeFrmNS */


/*****************************************************************************
*    Name: ipv4_GetRtrCountFrmNS 
*
*    Description:
*             Get the number of routers existing in NS
*
*    Parameters:
*
*    Returns: NPF_SUCCESS: 
*             NPF_E_UNKNOWN 
*             
*/
NPF_RET ipv4_GetRtrCountFrmNS(
		uint32_t *rtrCount
		)
{
    NPF_RET   retVal;
    NPF_NSHDL sys_ns_hdl,ns_node_hndl;
    NPF_HANDLE sys_npf_hdl, dataHdl;
    NODE_ITERATOR rtr_itr;
    NPF_ROUTETABLE_t routeTable;
    char name[NPF_NS_MAXPATH]; 
    uint32_t     rtrNum;
    
    IPV4_DBG_TRACE();
    
    /* Zero the memory */
    memset(name,0 , sizeof(name));
    
    /* By default */
    *rtrCount = 0 ;    
    /* Get the Namespace handle of the root node (System) */
    if((retVal = pdk_ns_getNSRootHandle (&sys_ns_hdl)) != NPF_SUCCESS)
        RETURN(retVal);     
    
    npf_ns_getPath(sys_ns_hdl, name);
    npf_logger_Write(LOG_IPV4, LOG_TRACE, 
		    "Namespace Root Node Path is %s\n", name);
    
    /* Get the Data Handle for the root node */ 
    if((retVal = npf_ns_getDataHandle(sys_ns_hdl, &sys_npf_hdl))
		    != NPF_SUCCESS)
    {
        npf_ns_close(sys_ns_hdl);  
	     RETURN(retVal);
    }
     
    /* Get the Interator Node for iterating through the NS tree and select a
     * node of type NPF_IPROUTER */    
    if((retVal = pdk_ns_getChildItrByType(sys_npf_hdl, NPF_IPROUTER, 
				    &rtr_itr))  != NPF_SUCCESS)
    {
        npf_ns_close(sys_ns_hdl);
        RETURN(retVal); 
    }
    
    ns_node_hndl = npf_ns_first(rtr_itr);
    /* Get the IP router instances in a loop */        
    while(ns_node_hndl != (NPF_NSHDL)NULL)
    {
       npf_ns_getPath(ns_node_hndl, name);
       npf_logger_Write(LOG_IPV4, LOG_TRACE, 
		    "Namespace Router Path is %s\n", name);
       /* Get the data handle for this instance of the router */
       if((retVal = npf_ns_getDataHandle(ns_node_hndl, &dataHdl ))
        		    == NPF_SUCCESS)
       {
	       /* Check if the router attribute is IPV4 or (IPV4+IPV6) */
          if(( retVal = cm_router_get_attribs(dataHdl, &routeTable))
	         		== NPF_SUCCESS)
	       {
             if(routeTable.rt_type == NPF_ROUTER_IPv4 ||
                 routeTable.rt_type == NPF_ROUTER_BOTH)
             {
	             rtrNum++; /* Increase the count of IP routers */
             }
	      }
	  else
	  {
             npf_logger_Write(LOG_IPV4, LOG_ERR, " Unable to find attributes\
			of Router with Data Handle  0x%x", dataHdl );
	     goto errHandler;
	  }
          
	  /* Close the NS inst handle */
          npf_ns_close(ns_node_hndl);
          /* Close the data handle */
          npf_ns_close(dataHdl);
       }
       else
       {
           npf_logger_Write(LOG_IPV4, LOG_ERR, " Unable to find data\
		     	handle for 0x%x", ns_node_hndl);
	        goto errHandler; 
       }
       /* Get next node */
       ns_node_hndl = npf_ns_next(rtr_itr);
    } /* end of while */
    
    npf_logger_Write(LOG_IPV4, LOG_TRACE, " Number of routers = %d ", rtrNum);
    npf_ns_close(sys_ns_hdl);
    *rtrCount = rtrNum;
    RETURN(NPF_SUCCESS);

errHandler:
    npf_ns_close(dataHdl);
    npf_ns_close(sys_ns_hdl);
    RETURN(retVal); 
} /* ipv4_GetRtrCountFrmNS */


/*****************************************************************************
*    Name: ipv4_CreateRtrNodeInNS
*   
*    Description:
*        
*    Parameters:
*
*    Preconditions:
*        
*    
*    Postconditions:
*        
*    
*    Returns:
*        void
*    
*/
NPF_RET ipv4_CreateRtrNodeInNS(
      NPF_HANDLE *rtr_npf_hdl
		)
{
    NPF_RET    retVal;
    NPF_NSHDL sys_ns_hdl;
    NPF_NSHDL rtrTypeHndl, rtrInstHndl;
    uint32_t   rtrNumTotal = 0; /*Total Number of IP routers existing */
    uint32_t   newRtrInst  = 0;  /* New router instance Number */
    NPF_ROUTETABLE_t routeTable; /* IPv4 or IPv6 router */
    char       sysPath[NPF_NS_MAXPATH],rtrPath[NPF_NS_MAXPATH];
   
    IPV4_DBG_TRACE();
   
    assert(rtr_npf_hdl); 
    
    /* Zero the memory */
    memset(sysPath,0 , sizeof(sysPath));
    memset(rtrPath,0 , sizeof(rtrPath));
    
    /* Mark the attributes : This is a IPV4 Router */ 
    routeTable.rt_type = NPF_ROUTER_IPv4 ; 
    
    /* Get the Namespace handle of the root node (System) */
    if((retVal = pdk_ns_getNSRootHandle (&sys_ns_hdl)) != NPF_SUCCESS)
        RETURN(retVal);     
    
    npf_ns_getPath(sys_ns_hdl, sysPath);
    npf_ns_close(sys_ns_hdl);  

    if(ipv4_CheckRtrNodeFrmNS(rtrPath, &rtrTypeHndl) != NPF_SUCCESS)
    {
        rtrTypeHndl = 0; /* To be Safe */ 
        sprintf(rtrPath, "%s/Router", sysPath); 
        /* Create Router TYPE Node in NS */ 
        if ((retVal=npf_ns_create(rtrPath, NPF_NS_TYPENODE,NPF_IPROUTER,
				       0,&rtrTypeHndl)) != NPF_SUCCESS)
        {
           npf_logger_Write(LOG_IPV4, LOG_ERR, 
		         "Error in Router Node %s creation\n", rtrPath);
	        RETURN(retVal);
        }
        /* Set the attributes */
        cm_router_set_attribs(rtrTypeHndl, &routeTable);
    }

    /* Find how many routers are there in NS */
    if ((retVal = ipv4_GetRtrCountFrmNS(&rtrNumTotal)) != NPF_SUCCESS)
    {
        npf_logger_Write(LOG_IPV4, LOG_TRACE, "router count is %d", rtrNumTotal);
    }
    
    /* Get the highest Router instance */
    newRtrInst = rtrNumTotal++; 
    
    /* Now, we will create the Router Instance Node */
    sprintf(rtrPath,"%s/%d",rtrPath, newRtrInst);
   
    npf_logger_Write(LOG_IPV4, LOG_TRACE, 
		    "New Router Instance Node Path is %s\n", rtrPath);
   
    /* Create Router INSTANCE Node in NS */ 
    if ((retVal=npf_ns_create(rtrPath, NPF_NS_INSTNODE, NPF_IPROUTER,
				    0,&rtrInstHndl)) != NPF_SUCCESS)
    {
        npf_logger_Write(LOG_IPV4, LOG_ERR, 
		    "Error in Router Node Instance %s creation\n", rtrPath);
        RETURN(retVal);
    }
    /* Set the attributes */
    cm_router_set_attribs(rtrInstHndl, &routeTable);
    
    if ( (retVal = npf_ns_getDataHandle(rtrInstHndl, 
				    rtr_npf_hdl)) != NPF_SUCCESS )
    {
        npf_logger_Write(LOG_IPV4,LOG_ERR, 
			" Could not get data handle for Router %s \n",rtrPath );
        RETURN(retVal);
    }
     /* Close the NS handle */ 
    npf_ns_close(rtrInstHndl); 
    npf_ns_close(rtrTypeHndl);
    RETURN(NPF_SUCCESS)
} /* ipv4_CreateRtrNodeInNS */


/*****************************************************************************
*    Name: ipv4_DeleteRtrNodeInNS
*   
*    Description:
*        
*    Parameters:
*
*    Preconditions:
*        
*    
*    Postconditions:
*        
*    
*    Returns:
*        void
*    
*/
NPF_RET ipv4_DeleteRtrNodeInNS(
		NPF_uint32_t *rtrNpfHndl
		)
{
    NPF_RET retVal;
    char rtrPath[NPF_NS_MAXPATH];
    NPF_NSHDL rtrNSHndl = 0;
    FPPI_FEID feIdArray[MAXFENUM];
    uint32_t  numFE=0;
    
    IPV4_DBG_TRACE();
    
    /* Zero the memory */
    memset(rtrPath,0 , sizeof(rtrPath));
    /* Get the NS Handler from NPF Data Handle */
    retVal = cm_getNSHDLByHandle(*rtrNpfHndl, &rtrNSHndl);
    if(retVal != NPF_SUCCESS || rtrNSHndl == 0)
    {
      npf_logger_Write(LOG_IPV4, LOG_ERR, 
                       "Unable to get Router NS Handle");
      RETURN(retVal);
    }
    /* Get the path to the router */ 
    npf_ns_getPath(rtrNSHndl, rtrPath);

    if((retVal = ipv4_DelAssocNodesForRtrFrmNS(rtrNSHndl,feIdArray,
                              &numFE)) != NPF_SUCCESS)
    {
      npf_logger_Write(LOG_IPV4, LOG_ERR, 
                       "Not able to get the associations");
      npf_ns_close(rtrNSHndl);
      RETURN(retVal);
    }
     /* Close the NS handle */ 
    if (( retVal= npf_ns_close(rtrNSHndl)) != NPF_SUCCESS )
    {
        npf_logger_Write(LOG_IPV4,  LOG_ERR,
                       "Cannot close NS handle\n" );
        RETURN(retVal);
    }
    
    /* Print the router path */
    npf_logger_Write(LOG_IPV4, LOG_TRACE, 
                "Router to be deleted is \t %s\t\n", rtrPath);
    /* Delete it from the Namespace */ 
    if((retVal = npf_ns_delete(rtrPath)) != NPF_SUCCESS)
    {
        npf_logger_Write(LOG_IPV4,  LOG_ERR,
                       "Unable to delete Router \t%s\t \n", rtrPath );
        RETURN(retVal);
    }
    
    npf_logger_Write(LOG_IPV4, LOG_TRACE, 
                "Success : Router deletion in NS : \t %s\t\n", rtrPath);
    
    RETURN(NPF_SUCCESS)
} /* ipv4_DeleteRtrNodeInNS */



/*
   ############################################################################
   IPV4 Transaction Module Helper Functions
   ############################################################################
*/


/*****************************************************************************
*    Name: ipv4_InitReqFrmUsr
*
*    Description:
*
*    Parameters:
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*       NPF_SUCCESS 
*       NPF_FAIL
*
*/
NPF_RET ipv4_InitReqFrmUsr(
   NPF_correlator_t           correlator, 
	NPF_errorReporting_t       appVerb,
	IPV4TableType_t            tableType,
	IPV4TransType_t            transType,
	NPF_IPv4UC_CallbackType_t  reqType,
	NPF_callbackHandle_t       cbHandle,
	NPF_uint32_t               tableHandle,
   IPV4UsrReqInfo_t           **request
	) 
{
   IPV4UsrReqInfo_t  *usrReq=NULL;
   
   IPV4_DBG_TRACE();
   assert(request);
   
   /* Allocate memory for this */
   usrReq = (IPV4UsrReqInfo_t *)PIL_AllocateMemory( 
            sizeof( IPV4UsrReqInfo_t ), PIL_MEM_ZERO_INIT );
   if (!usrReq)
   {
      npf_logger_Write(LOG_IPV4, LOG_ERR, "Memory not allocated ");
      RETURN(NPF_OUT_OF_MEMORY);
   }
   
   /* Assign all the fields with valid values */
   usrReq->correlator = correlator;
   usrReq->appVerbosity = appVerb;
   usrReq->tableType = tableType;
   usrReq->transType = transType;
   usrReq->reqType   = reqType;
   usrReq->callbackHandle  = cbHandle;
   usrReq->tblHandle = tableHandle;
   /* Initalize all the pointer to NULL*/
   usrReq->nhopCntxt   = NULL;
   usrReq->addResCntxt = NULL;
#ifdef IPV4_UNIF_SUPPORT
   usrReq->prefixCntxt = NULL;
#endif /*IPV4_UNIF_SUPPORT*/
   usrReq->transInfo   = NULL;
   
   *request = usrReq; 
   
   RETURN(NPF_SUCCESS);
} /* ipv4_InitReqFrmUsr */


/*****************************************************************************
*    Name: ipv4_FreeUsrReq
*
*    Description:
*
*    Parameters:
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*       NPF_SUCCESS 
*       NPF_FAIL
*
*/
NPF_RET ipv4_FreeUsrReq(
        IPV4UsrReqInfo_t *usrReq
	) 
{
    NPF_RET    retVal;
#ifdef IPV4_UNIF_SUPPORT
   IPV4PrefixCntxt_t *prefixCntxt = NULL;
#endif

   IPV4_DBG_TRACE();
   
   if(usrReq == NULL)
      RETURN(NPF_SUCCESS);

   /* Remove the Next-Hop context if there with this req */ 
   if((retVal = ipv4_DelNhopCntxt(usrReq->nhopCntxt))!=NPF_SUCCESS)
   {
        npf_logger_Write(LOG_IPV4, LOG_ERR, "Unable to Free Nhop Cntxt");
        RETURN(retVal);
   }
   
   /* Remove the AddRes context if there with this req */ 
   if((retVal = ipv4_DelAddResCntxt(usrReq->addResCntxt))!=NPF_SUCCESS)
   {
        npf_logger_Write(LOG_IPV4, LOG_ERR, "Unable to Free AddRes Cntxt");
        RETURN(retVal);
   }
#ifdef IPV4_UNIF_SUPPORT
   prefixCntxt = usrReq->prefixCntxt;
   if (prefixCntxt != NULL)
   {
      if(prefixCntxt->prefixArray)
         PIL_FreeMemory(prefixCntxt->prefixArray);
      PIL_FreeMemory(prefixCntxt);
   }
#endif

   /* Finally Free usrReq */
   PIL_FreeMemory(usrReq);
   RETURN(NPF_SUCCESS);   
} /*  ipv4_FreeUsrReq */

/*****************************************************************************
*    Name: ipv4_ProcessReqFrmUsr
*
*    Description:
*
*    Parameters:
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*       NPF_SUCCESS 
*       NPF_FAIL
*
*/
NPF_RET ipv4_ProcessReqFrmUsr(
        IPV4UsrReqInfo_t *usrReq,
        IPV4UserInfo_t  **user,
        IPV4TableInfo_t **table
	) 
{
   NPF_RET retVal;
   IPV4UserInfo_t  *userInfo  = NULL;
   IPV4TableInfo_t *tableInfo = NULL;
   NPF_uint32_t     tableHandle;     /* Table Handle */

   IPV4_DBG_TRACE();
 
   /* Step1 : Get the User Info corresponding to the callback handle */     
    if((retVal=ipv4_GetUserByHandleFrmMgr(usrReq->callbackHandle, &userInfo))
		          != NPF_SUCCESS)
    {
        npf_logger_Write(LOG_IPV4, LOG_ERR, "Unable to Get the User Info");
        RETURN(retVal);
    }
   
   /* Step2: User Info found. Check if the correlator is unique and not dup : THis
             is true only for discrete. For Unif, trans. will exist */  
 /*   if( usrReq->transType == IPV4_TRANS_DISC) 
    {
       if ((retVal = ipv4_GetTransByCorrelFrmUsr(userInfo,
				          usrReq->correlator,&transInfo)) == NPF_SUCCESS)
       {
           npf_logger_Write(LOG_IPV4, LOG_ERR,
	     "Duplicate transaction with similar correlator %d exists ", usrReq->correlator);
           RETURN(retVal);
       }
    } */

    /* Correlator is not duplicate.*/
 
   /* Step3: Find the table associated with this request */
   /* Get the table handle if it exists */
   tableHandle = usrReq->tblHandle;
   if (tableHandle != (NPF_uint32_t)NULL)
   {
      if((retVal=ipv4_GetTableByHndlFrmUsr(userInfo, usrReq->tableType, 
                         tableHandle, &tableInfo)) != NPF_SUCCESS)
      {
           npf_logger_Write(LOG_IPV4, LOG_ERR, " Target Table [handle = %d] not found ", tableHandle);
  	   RETURN(retVal);
      }
   }
   else /* If the tablehandle is not present, find the table based on type */
   {
      tableInfo = &userInfo->tableArray[usrReq->tableType];    
   }
   assert(tableInfo); /* Table found is success. Safety check */
        
   /* Store the pointers in parameters */
   *user = userInfo ;
   *table = tableInfo ;
   RETURN(NPF_SUCCESS);
} /* ipv4_ProcessReqFrmUsr */




/*****************************************************************************
*    Name: ipv4_InitTransInfo
*
*    Description:
*    	Creates memory and initializes various memebers
*
*    Prequisites: 
*       IPV4TransInfo_t structure is filled already from the NPF API
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If Init is done properly 
*             NPF_E_UNKNOWN : If Init is not done properly 
*/

NPF_RET ipv4_InitTransInfo(
	     IPV4UsrReqInfo_t      *usrReq,
        IPV4UserInfo_t        *userInfo,
        IPV4TableInfo_t       *tableInfo,
        IPV4TransInfo_t       **transInfo
        ) 
{
   NPF_RET retVal;
   IPV4TransInfo_t       *transNew = NULL;
#ifdef IPV4_UNIF_SUPPORT
   uint32_t              i;
#endif /* IPV4_UNIF_SUPPORT */
   IPV4_DBG_TRACE();
  
   assert(usrReq);
   assert(userInfo);
   assert(tableInfo);
   
   /* First Check : search if transaction alrady exists */  
   if(npf_list_searchByKey(&userInfo->transCpList, 
                       (const void *)&usrReq->correlator) != NULL)
   {
      npf_logger_Write(LOG_IPV4, LOG_ERR, 
               " Duplicate Transaction with correlator %d found ",
               usrReq->correlator);
      RETURN( NPF_E_UNKNOWN );
   }
   /* Create a new transaction */
   transNew = (IPV4TransInfo_t *)PIL_AllocateMemory( 
            sizeof( IPV4TransInfo_t ), PIL_MEM_ZERO_INIT );
  
   if (!transNew)
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR, "Memory not allocated ");
       RETURN(NPF_OUT_OF_MEMORY);
   }
   
   transNew->transPend = TRUE;
   transNew->usr_correlator = usrReq->correlator;
#ifdef IPV4_UNIF_SUPPORT
   transNew->int_correlator = 0;    /* zero by default */
   transNew->mapTransInfo   = NULL; /* NULL by default */
   if (usrReq->transType == IPV4_TRANS_UNIF)
   {
      /* get the internal correlator value from User struc */
      ipv4_GetICorrelFrmUsr(userInfo, &transNew->int_correlator);
      transNew->mapTransInfo = (IPV4MapTransInfo_t *)PIL_AllocateMemory(sizeof(IPV4MapTransInfo_t), PIL_MEM_ZERO_INIT);
 
      if (!transNew->mapTransInfo)
      {
         npf_logger_Write(LOG_IPV4, LOG_ERR, "Memory not allocated ");
         RETURN(NPF_OUT_OF_MEMORY);
      }
      /* Initialize mapped transactions */
      transNew->mapTransInfo->numMapTrans = 0;
      transNew->mapTransInfo->cbResp.allOK = NPF_TRUE; 
      transNew->mapTransInfo->cbResp.resp = NULL; 
      transNew->mapTransInfo->cbResp.numResp = 0; 
   
      for (i = 0; i < IPV4_MAX_MAP_TRANS; i++)
         transNew->mapTransInfo->mapTransArray[i] = NULL;
   } /* Only in case of Unified Transaction */
#endif /* IPV4_UNIF_SUPPORT */
   
   /* Transaction now points to the original request */ 
   transNew->usrReq = usrReq;
   /* Transaction points to the User Contxt */ 
   transNew->usrInfo = userInfo;
   /* Transaction points to the Target Table*/ 
   transNew->tableInfo = tableInfo;
   /* Set the request to FE array info to NULL */
   transNew->transFpInfo.numFEWait     = 0;
   transNew->transFpInfo.numFEToSend   = 0;
   /* Set the pointer from User to Trans */ 
   usrReq->transInfo = transNew;
   
   if((retVal = ipv4_AddTransInListToUsr(userInfo, transNew)) != NPF_SUCCESS)
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR, "Unable to add the transaction");
       RETURN(NPF_OUT_OF_MEMORY);
   }

   *transInfo = transNew; /* assing it to param */
   RETURN(NPF_SUCCESS);
} /* ipv4_InitTransInfo */

#ifdef IPV4_UNIF_SUPPORT
/*****************************************************************************
*    Name: ipv4_DupTransInfo
*
*    Description:
*    	Duplicates a transaction control block
*
*    Prequisites: 
*       IPV4TransInfo_t structure is filled already from the NPF API
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If Init is done properly 
*             NPF_E_UNKNOWN : If Init is not done properly 
*/

NPF_RET ipv4_DupTransInfo(
        IPV4TransInfo_t        *srcTransInfo,
        IPV4TransInfo_t        **dstTransInfo
        ) 
{
   IPV4TransInfo_t       *transNew = NULL;

   IPV4_DBG_TRACE();
   assert(srcTransInfo);
 
   transNew = (IPV4TransInfo_t *)PIL_AllocateMemory( 
            sizeof( IPV4TransInfo_t ), PIL_MEM_ZERO_INIT );
  
   if (!transNew)
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR, "Memory not allocated ");
       RETURN(NPF_OUT_OF_MEMORY);
   }
   
   transNew->transPend = TRUE;
   transNew->usr_correlator = srcTransInfo->usr_correlator;
      /* get the internal correlator value from User struc */
   ipv4_GetICorrelFrmUsr(srcTransInfo->usrInfo, &transNew->int_correlator);
   /* Initialize mapped transactions */
   transNew->mapTransInfo = srcTransInfo->mapTransInfo; 
   transNew->mapTransInfo->numMapTrans = 
                  srcTransInfo->mapTransInfo->numMapTrans;
   transNew->mapTransInfo->mapTransArray[transNew->mapTransInfo->numMapTrans]=
       transNew;
   transNew->mapTransInfo->numMapTrans++;
    
   /* Transaction now points to the original request */ 
   transNew->usrReq = srcTransInfo->usrReq;
   /* Transaction points to the User Contxt */ 
   transNew->usrInfo = srcTransInfo->usrInfo;
   /* Transaction points to the Target Table*/ 
   transNew->tableInfo = srcTransInfo->tableInfo;

   *dstTransInfo = transNew; /* assing it to param */
   RETURN(NPF_SUCCESS);
} /* ipv4_DupTransInfo */
#endif

/*****************************************************************************
*    Name: ipv4_DeleteTransInfo
*
*    Description:
*    	Deletes memory and removes any relationships from trans to other
*    	datastrcutres. 
*
*
*    Prequisites: 
*        
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If Init is done properly 
*             NPF_E_UNKNOWN : If Init is not done properly 
*/

NPF_RET ipv4_DeleteTransInfo(
        IPV4TransInfo_t *transInfo
        ) 
{
   NPF_RET retVal;
   IPV4ReqToFEInfo_t **reqToFEArray = NULL;
   uint32_t i;  

   IPV4_DBG_TRACE();
    
   if(transInfo == NULL)
      RETURN(NPF_SUCCESS);
   
   /* Remove the transac from the list first */
   if((retVal = ipv4_DelTransInListFrmUsr(transInfo)) != NPF_SUCCESS)
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR, 
			    " Error in removing Trans from the list in UsrInfo");
       RETURN(retVal)
   }
   /* Store the array in a local variable for easy access */
   reqToFEArray = (IPV4ReqToFEInfo_t**)transInfo->transFpInfo.reqFEPtrArray;
   
   /* Remove the context when all the FE Callbacks are received */ 
   if(transInfo->transFpInfo.numFEWait == 0 &&
                         reqToFEArray != (IPV4ReqToFEInfo_t**)NULL)
   {
       /* Delete - Find the pending requests to FP that need to be deleted */
       for( i=0; i< transInfo->transFpInfo.numFEToSend; i++)
       {
          if((retVal=ipv4_DelFEReqFrmFpp(reqToFEArray[i]->fppi_correlator))
			 != NPF_SUCCESS)
          {
              npf_logger_Write(LOG_IPV4, LOG_ERR, 
			    " Error in deleting FE requests");
              RETURN(retVal)
          }
        }
   }
   /* Finally free transaction */
   PIL_FreeMemory(transInfo);
   transInfo = NULL;
   RETURN(NPF_SUCCESS);
} /* ipv4_DeleteTransInfo */
   
 
/*****************************************************************************
*    Name: ipv4_DeleteUsrReq
*
*    Description:
*    	Deletes memory and removes any relationships with the trans and other
*    	datastrcutres. 
*
*
*    Prequisites: 
*        
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If Init is done properly 
*             NPF_E_UNKNOWN : If Init is not done properly 
*/

NPF_RET ipv4_DeleteUsrReq(
        IPV4UsrReqInfo_t *usrReq
        ) 
{
   NPF_RET retVal;
   IPV4TransInfo_t *transInfo = NULL;

   IPV4_DBG_TRACE();
   
   
   if(usrReq == NULL)
      RETURN(NPF_SUCCESS);
   
   transInfo = usrReq->transInfo; 
   
  
   if(transInfo == NULL)
      RETURN(NPF_SUCCESS);
   
   if((retVal = ipv4_DeleteTransInfo(transInfo)) != NPF_SUCCESS)
   {
      npf_logger_Write(LOG_IPV4, LOG_ERR, 
	    " Error in deleting trans. info ");
      RETURN(retVal);
   }   
   /* Now that transac. is deleted, delete request from user */
   if((retVal = ipv4_FreeUsrReq(usrReq)) != NPF_SUCCESS)
   {
      npf_logger_Write(LOG_IPV4, LOG_ERR, 
	    " Error in deleting User Request from trans. ");
      RETURN(retVal);
   }
   RETURN (NPF_SUCCESS);
} /* ipv4_DeleteUsrReq */



/*
   ############################################################################
   IPV4 L2Id Helper Functions
   ############################################################################
*/


/*****************************************************************************
*    Name: ipv4_PrcUserReqToFPForNhopAdd
*
*    Description:
*       Adds L2Entry in the L2Table, when there is a success received in the
*       callback for the Next-Hop Add.
*
*    Prequisites: 
*        
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If Init is done properly 
*             NPF_E_UNKNOWN : If Init is not done properly 
*/

NPF_RET ipv4_PrcUserReqToFPForNhopAdd(
	NPF_IN IPV4UsrReqInfo_t *usrReq,
	NPF_IN NPF_uint32_t numEntries,
	NPF_IN NPF_uint32_t *nextHopIdArray,
	NPF_IN NPF_IPv4UC_NextHopArray_t *nextHopArrays,
	NPF_IN FPPI_UnicastNextHopUpdateInfo_t *fpNHUpdBlock
        ) 
{
   NPF_RET              retVal;
   NPF_IPv4UC_NextHop_t *nhopEntry = NULL;
   NPF_uint32_t         nhId;
   uint32_t             i, j,k;
   IPV4IfAttrib_t       ifAttrib;
   IPV4TableInfo_t      *tableInfo = NULL;
   FPPI_FEID            feId;
   FPPI_FEandL2IDs      *newL2Id = NULL, *oldL2Ids = NULL;
   uint32_t             numOldL2Ids = 0 , numNewL2Ids = 0;
   BOOL                 bFeExists = FALSE ;
   
    
   IPV4_DBG_TRACE();
  
   assert(usrReq);
   assert(nextHopIdArray);
   assert(nextHopArrays);
   assert(fpNHUpdBlock);
    
   /* Assign to local var */ 
   tableInfo = usrReq->transInfo->tableInfo;

   assert(tableInfo); /* safe check */
  
   /* Basic check to be done */ 
   if( tableInfo->numFeTotal == 0)
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR, 
                   "Nexthop table is not associated with any FE. \n Possible error conditions  are : \n 1) Unsuccessful bind with Prefix Table \n  2) Unsuccessful bind with Layer2 Interface\n 3) FE(s) haven't sent their capability set to CE !!!");
       RETURN(NPF_E_UNKNOWN);
   }
   for ( i=0; i <numEntries; i++)
   {
      /* We are not supporting ECMP as of now. Hence, the USER should not give
       * multiple next-hops for a single nexthopId
       */
      if( nextHopArrays[i].nextHopCount > 1 )
      {
         npf_logger_Write(LOG_IPV4, LOG_ERR,
		    " More than one Next-Hop entry found for single NHId");
         RETURN(NPF_INVALID_PARAMETERS);
      }
      /* We are expecting just a single entry */
      nhopEntry = nextHopArrays[i].nextHopArray; 
      nhId      = nextHopIdArray[i]; 
      
      /* Find egress interface */
      switch(nhopEntry->type)
      {
	     case NPF_IPV4UC_NH_DIRECT_ATTACH:
	     case NPF_IPV4UC_NH_REMOTE:
	     case NPF_IPV4UC_NH_BASIC:
            /* Fill the ifAttrib struc */
            ifAttrib.ifHndl = nhopEntry->egressInterface;
            break;
	     default:
            {
               npf_logger_Write(LOG_IPV4, LOG_ERR,
		               " Invalid NextHop Type ");
               RETURN(NPF_INVALID_PARAMETERS);
            }
      }
     
      /* Egress Interface extracted. Now find the FE assocaited with it */
      if ((retVal= ipv4_GetFEPortAttribForIfFrmNS(&ifAttrib)) 
                                                  != NPF_SUCCESS)
      {
         npf_logger_Write(LOG_IPV4, LOG_ERR,
		    " Couldn't extract FeId associated with Egress If ");
	      RETURN(retVal);
      }
   
      /* We have found valid Port Type for the interface. Also, we have got
       * the associated FeIds with this interface. Now update the NextHop
       * Table on these FEs */

      if(ifAttrib.numFe > MAXFENUM)
      {
         npf_logger_Write(LOG_IPV4, LOG_ERR, 
			 "Interface is assocated with [%d] FEs which \
                           is  more than defined FEs",ifAttrib.numFe);
         RETURN(NPF_E_UNKNOWN);
      }
 
      for ( j =0 ; j<tableInfo->numFeTotal; j++) 
      {
         feId = tableInfo->feIdArray[j];
         /* Verify all associated FEs are existing */ 
         for( k=0; k<ifAttrib.numFe; k++)
         {
             if(feId == ifAttrib.feId[k])
                bFeExists = TRUE;
         }
         if( bFeExists == FALSE)
         {
            npf_logger_Write(LOG_IPV4, LOG_ERR,
                      "Interface's associated FEs are not existing ");
            RETURN(NPF_E_UNKNOWN);
         }

         /* Small check for L2Id addition based on Port Type */
         if( (ifAttrib.portType != NPF_IF_TYPE_LAN) && 
             (ifAttrib.portType != NPF_IF_TYPE_ATM)) 
         {
            npf_logger_Write(LOG_IPV4, LOG_TRACE,
		       " L2Id generation not needed for given Port type");
            /* L2Id sent down is zero */
            fpNHUpdBlock[i].nhInfo->l2id   =  0; 
         }
         else
         {
             npf_logger_Write(LOG_IPV4, LOG_TRACE, 
                 "Going to Update the L2Id [NHId = %d on FEId = %d]"
                 ,nhId,feId);
             
             if((retVal = l2idm_UpdateForNextHopTable(&feId,nhId,1, 
                        		 &nhopEntry->nextHopIP, &numOldL2Ids,
                               &oldL2Ids, &numNewL2Ids, &newL2Id))
	                                      != NPF_SUCCESS)
            {
               npf_logger_Write(LOG_IPV4, LOG_ERR,
                   "Error updating the L2Id corresponding \
    	                  to next hop addresses added");        
               RETURN(retVal);
            }
            
            npf_logger_Write(LOG_IPV4, LOG_TRACE,
             "L2Id Creation  was Success![NHId = %d] [New L2Id = %d] ",
               nhId,newL2Id[0].l2id);
            
            /* Small check */ 
            if (numNewL2Ids == 0)  
            {
              npf_logger_Write(LOG_IPV4, LOG_ERR,
	           " L2Id Manager returned invalid L2Id ");
	           RETURN(NPF_E_UNKNOWN);
            }
            assert(newL2Id); 
                        
            /* Fill in the oldFEandL2Id member */
            for ( k=0; k < numOldL2Ids; k++)
            {
               fpNHUpdBlock[i].oldFEandL2IDs[k].feid = oldL2Ids[k].feid ; 
               fpNHUpdBlock[i].oldFEandL2IDs[k].l2id= oldL2Ids[k].l2id ; 
            } 
            /* Fill a valid L2Id */
            fpNHUpdBlock[i].nhInfo->l2id   =  newL2Id[0].l2id; 
         } /* If Interface doens't need L2Id Interaction */
         
         /* We are not supporting ECMP as yet */ 
         fpNHUpdBlock[i].numNHEntries = 1; 
         
         fpNHUpdBlock[i].nhid= nextHopIdArray[i]; 
         
         /* Fill Next-Hop Info Struc for FPP */
         assert(fpNHUpdBlock[i].nhInfo);  /* defensive check */
         fpNHUpdBlock[i].nhInfo->feid   =  feId ; 
         fpNHUpdBlock[i].nhInfo->portId = ifAttrib.portId; 
         fpNHUpdBlock[i].nhInfo->portType = ifAttrib.portType; 
         fpNHUpdBlock[i].nhInfo->nhAddr   = nhopEntry->nextHopIP; 
         fpNHUpdBlock[i].nhInfo->mediaAddr = nhopEntry->mediaAddress; 
         fpNHUpdBlock[i].nhInfo->mtu      = ifAttrib.mtu; 
         fpNHUpdBlock[i].nhInfo->flags    = 0; 

       } /* end of for */
   } /* For each entry in Nhop */

  RETURN(NPF_SUCCESS);

} /* ipv4_PrcUserReqToFPForNhopAdd */

/*****************************************************************************
*    Name: ipv4_PrcUserReqToFPForNhopDel
*
*    Description:
*       Deletes L2Entry in the L2Table
*
*    Prequisites: 
*        
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS:  
*             NPF_E_UNKNOWN :  
*/

NPF_RET ipv4_PrcUserReqToFPForNhopDel(
	IPV4UsrReqInfo_t *usrReq,
	NPF_uint32_t numEntries,
	NPF_uint32_t *nextHopIdArray,
	NPF_IN FPPI_UnicastNextHopDeleteInfo_t  *fpNHDelBlock
   ) 
{
   NPF_RET              retVal;
   NPF_uint32_t         nhId;
   NPF_IPv4UC_NextHopArray_t *nhopArray=NULL;
   uint32_t             i;
   IPV4IfAttrib_t       ifAttrib;
   IPV4TableInfo_t      *tableInfo = NULL;
   FPPI_FEandL2IDs      *oldL2Ids = NULL;
   uint32_t             numOldL2Ids = 0 ;
    
   IPV4_DBG_TRACE();
  
   assert(usrReq);
   assert(usrReq->transInfo);
   assert(nextHopIdArray);
   assert(fpNHDelBlock);
    
   /* Assign to local var */ 
   tableInfo = usrReq->transInfo->tableInfo;
   
   /*  We will store information in the nhopContext, as we extract it  
    *  from L2Id Manager ; while we get the callback of delete. */
   if((retVal = ipv4_AddNhopCntxtToUsrReq(usrReq, numEntries, 
                     NULL, nextHopIdArray)) != NPF_SUCCESS)
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR, "NextHop Context creation failure ");
       RETURN(NPF_OUT_OF_MEMORY);
   } 

   for ( i=0; i <numEntries; i++)
   {
      /* We are expecting just a single entry */
      nhId      = nextHopIdArray[i]; 
      npf_logger_Write(LOG_IPV4, LOG_TRACE, 
               "Going to Remove the L2Id for NHId=%d ", nhId);
         
     /* Get the context pointer */
     nhopArray = &usrReq->nhopCntxt->nhopArray[i];
     /* We don't support ECMP */
     if(nhopArray->nextHopCount > 1)
     {
        npf_logger_Write(LOG_IPV4, LOG_ERR,
                  "More than one nextHop count entry for each NHID");        
        RETURN(NPF_E_UNKNOWN);
     }
     
     if((retVal = l2idm_GetForNextHopTable(nhId,1, 
                        		 &nhopArray->nextHopArray->nextHopIP, 
                               &numOldL2Ids,
                               &oldL2Ids))
	                            != NPF_SUCCESS)
     {
         npf_logger_Write(LOG_IPV4, LOG_ERR,
                  "Error remvoving the L2Id corresponding \
                  to next hop addresses removed");        
         RETURN(retVal);
     }
            
    /* For Delete, number of L2Ids returned must not be 0*/ 
     if (numOldL2Ids == 0)  
     {
            npf_logger_Write(LOG_IPV4, LOG_ERR,
		                "Zero L2Id provided by L2Id Manager");
	         RETURN(NPF_E_UNKNOWN);
     }
     assert(oldL2Ids); 
     /* Save the context */
     npf_logger_Write(LOG_IPV4, LOG_TRACE, "Removed the L2Id %d ",
        oldL2Ids[0]);
         
     fpNHDelBlock[i].nhid= nextHopIdArray[i]; 
     /* Fill Next-Hop Info Struc for FPP */
     fpNHDelBlock[i].nhInfo.feid     = 0xFFFF;  /* Fill Invalid */
     fpNHDelBlock[i].nhInfo.l2id     =  oldL2Ids[0].l2id; 
     fpNHDelBlock[i].nhInfo.portId   = ifAttrib.portId; 
     fpNHDelBlock[i].nhInfo.portType = ifAttrib.portType; 
     fpNHDelBlock[i].nhInfo.nhAddr   = nhopArray->nextHopArray->nextHopIP; 
     fpNHDelBlock[i].nhInfo.mtu      = ifAttrib.mtu; 
   } /* For each entry in Nhop */
    
  RETURN(NPF_SUCCESS);
}  /* ipv4_PrcUserReqToFPForNhopDel*/ 
/*****************************************************************************
*    Name: ipv4_PrcUserReqToFPForNhopFlush
*
*    Description:
*       Processes Flush for L2Table
*
*    Prequisites: 
*        
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS:  
*             NPF_E_UNKNOWN :  
*/

NPF_RET ipv4_PrcUserReqToFPForNhopFlush(
   FPPI_FEID        feId,
	IPV4UsrReqInfo_t *usrReq,
	NPF_uint32_t     *numEntries,
	FPPI_UnicastNextHopDeleteInfo_t  **fpNhopFlush
   ) 
{
   NPF_RET              retVal;
   uint32_t             i,numL2 =0 ;
   FPPI_L2ID            *l2IdList = NULL; 
	FPPI_UnicastNextHopDeleteInfo_t  *fpNhopFlushBlock = NULL;
   
   IPV4_DBG_TRACE();

   /* Contact L2Id manager to get L2Ids for existing NextHops */ 
   if((retVal = l2idm_GetNumOfL2Ids(feId, &numL2)) 
         != NPF_SUCCESS)
   {
      npf_logger_Write(LOG_IPV4, LOG_ERR,
	      "Unable to get number of Nhops in NHop Table in 
         FeId %d  From L2Id Mgr", feId);
      goto CleanUp;     
   }
   /* Allocate Memory for NhopFlush Block ; */
   if((retVal = ipv4_AllocateFPNhopDelBlock(numL2, 
                            &fpNhopFlushBlock))  != NPF_SUCCESS)
   {
      npf_logger_Write(LOG_IPV4, LOG_ERR,
			    "Unable to Process User req (Flush Nhop) to FP");
      goto CleanUp;     
   }
   /* Allocate memory for L2IdList */
   if((l2IdList = (FPPI_L2ID*)PIL_AllocateMemory(
             (sizeof(FPPI_L2ID)*numL2),
             PIL_MEM_ZERO_INIT)) == NULL)
   {
      npf_logger_Write(LOG_IPV4, LOG_ERR, "Memory Failure");
      retVal = NPF_E_UNKNOWN ;
      goto CleanUp;     
   }
   /* Contact L2Id manager to get existing NextHops */ 
   if((retVal = l2idm_GetL2IdList(feId, numL2,
                      l2IdList)) != NPF_SUCCESS)
   {
      npf_logger_Write(LOG_IPV4, LOG_ERR,
			     "Unable to get Nhop Entries in NHop Table in 
              FeId %d From L2Id Mgr", feId);
      goto CleanUp;     
   }
   for ( i=0; i <numL2; i++)
   { 
         /* Fill the fpNhopFlushBlock structure */
      fpNhopFlushBlock[i].nhid          = 0 ; 
      fpNhopFlushBlock[i].nhInfo.feid   = feId ; 
      fpNhopFlushBlock[i].nhInfo.l2id   = l2IdList[i]; 
   } /* For each entry */
   /* Free the l2Id List ; This was a temporary list */
   PIL_FreeMemory(l2IdList);
   
   /* Return the values of entries back */
   *numEntries = numL2;
   *fpNhopFlush = fpNhopFlushBlock ;
   RETURN(NPF_SUCCESS);

CleanUp:
   if(l2IdList)
      PIL_FreeMemory(l2IdList);
   RETURN(retVal);
} /* ipv4_PrcUserReqToFPForNhopFlush */

/*****************************************************************************
*    Name: ipv4_DelAddResCntxt
*
*    Description:
*       Delete the memory for AddRes Context 
*
*    Prequisites: 
*        
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: 
*             NPF_E_UNKNOWN : 
*/

NPF_RET ipv4_DelAddResCntxt(
        IPV4AddResCntxt_t      *addResCntxt
        ) 
{

   IPV4_DBG_TRACE();
   
   if(addResCntxt == NULL)
     RETURN(NPF_SUCCESS);  

   if (addResCntxt && addResCntxt->addResKeyArray )
      PIL_FreeMemory(addResCntxt->addResKeyArray); 
   if (addResCntxt)
      PIL_FreeMemory(addResCntxt); 

   RETURN(NPF_SUCCESS);
} /*ipv4_DelAddResCntxt*/


/*****************************************************************************
*    Name: ipv4_CreateAddResCntxt
*
*    Description:
*       Initializes the memory for AddRes Context 
*
*    Prequisites: 
*        
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: 
*             NPF_E_UNKNOWN : 
*/

NPF_RET ipv4_CreateAddResCntxt(
	    uint32_t                  numEntries,
	    NPF_IPv4UC_AddResKey_t  *addResArray,
       IPV4AddResCntxt_t         **addResInfo
       ) 
{
   NPF_RET              retVal;
   IPV4AddResCntxt_t    *addResCntxt = NULL;
   uint32_t             i;

   IPV4_DBG_TRACE();
  
   if(!numEntries)
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR, "AddRes Count is zero ");
       RETURN(NPF_E_UNKNOWN);
   } 
  
   /* allocate memory for the context */ 
   addResCntxt= (IPV4AddResCntxt_t*) PIL_AllocateMemory(
		            (sizeof(IPV4AddResCntxt_t)), PIL_MEM_ZERO_INIT); 
   if (!addResCntxt)
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR, "Memory not allocated ");
       retVal = NPF_OUT_OF_MEMORY;
       goto CleanUp;
   }
  
   addResCntxt->numEntries = numEntries;
   
   /* allocate memory for the  entries in context */ 
   addResCntxt->addResKeyArray = (NPF_IPv4UC_AddResKey_t*) 
               PIL_AllocateMemory( (sizeof(NPF_IPv4UC_AddResKey_t)*numEntries), PIL_MEM_ZERO_INIT); 
   if (!addResCntxt->addResKeyArray)
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR, "Memory not allocated ");
       retVal = NPF_OUT_OF_MEMORY;
       goto CleanUp;
   }
  
  
   /* Copy the context in this area */
   for ( i=0 ; i < numEntries; i++)
   {
      /* Copy the contents only if they are provided by the function */
      addResCntxt->addResKeyArray[i].IP_Address = addResArray[i].IP_Address; 
      addResCntxt->addResKeyArray[i].interfaceHandle = addResArray[i].interfaceHandle; 
   }

   /* Assign the pointer */
   *addResInfo = addResCntxt;
   RETURN(NPF_SUCCESS);

CleanUp:
   if(ipv4_DelAddResCntxt(addResCntxt) != NPF_SUCCESS)
      RETURN(NPF_E_UNKNOWN);
   RETURN(retVal);
} /*ipv4_CreateAddResCntxt*/

/*****************************************************************************
*    Name: ipv4_AddAddResCntxtToUsrReq
*
*    Description:
*       Initializes the memory for AddRes Context in user req
*
*    Prequisites: 
*        
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: 
*             NPF_E_UNKNOWN : 
*/

NPF_RET ipv4_AddAddResCntxtToUsrReq(
	      IPV4UsrReqInfo_t *usrReq,
	      uint32_t         numEntries,
	      NPF_IPv4UC_AddResKey_t *addResArray
         ) 
{
   NPF_RET              retVal;
   IPV4AddResCntxt_t    *addResCntxt = NULL;

   IPV4_DBG_TRACE();
  
   assert(usrReq);
  
   if((retVal = ipv4_CreateAddResCntxt(numEntries, addResArray,&addResCntxt))
                                     != NPF_SUCCESS) 
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR, "AddRes Context creation failure ");
       RETURN(NPF_OUT_OF_MEMORY);
   }
   /* Assign it back to the user request */
   usrReq->addResCntxt= addResCntxt;

   RETURN(NPF_SUCCESS);
} /* ipv4_AddAddResCntxtToUsrReq*/



/*****************************************************************************
*    Name: ipv4_DelNhopCntxt
*
*    Description:
*       Delete the memory for Next-Hop Context 
*
*    Prequisites: 
*        
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: 
*             NPF_E_UNKNOWN : 
*/

NPF_RET ipv4_DelNhopCntxt(
        IPV4NhopCntxt_t      *nhopCntxt
        ) 
{

   IPV4_DBG_TRACE();
   
   if(nhopCntxt == NULL)
     RETURN(NPF_SUCCESS);  

   if (nhopCntxt && nhopCntxt->nhopArray && nhopCntxt->nhopArray->nextHopArray)
      PIL_FreeMemory(nhopCntxt->nhopArray->nextHopArray); 
   if (nhopCntxt && nhopCntxt->nhopArray)
      PIL_FreeMemory(nhopCntxt->nhopArray); 
   if (nhopCntxt && nhopCntxt->nhIdArray)
      PIL_FreeMemory(nhopCntxt->nhIdArray); 
   if (nhopCntxt)
      PIL_FreeMemory(nhopCntxt); 

   RETURN(NPF_SUCCESS);
} /*ipv4_DelNhopCntxt*/


/*****************************************************************************
*    Name: ipv4_CreateNhopCntxt
*
*    Description:
*       Initializes the memory for Next-Hop Context 
*
*    Prequisites: 
*        
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: 
*             NPF_E_UNKNOWN : 
*/

NPF_RET ipv4_CreateNhopCntxt(
	uint32_t                  nhCount,
	NPF_IPv4UC_NextHopArray_t *nhopArray,
	uint32_t                  *nhIdArray,
        IPV4NhopCntxt_t           **nhop
        ) 
{
   NPF_RET              retVal;
   IPV4NhopCntxt_t      *nhopCntxt = NULL;
   uint32_t             i;

   IPV4_DBG_TRACE();
  
   if(!nhCount)
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR, "NextHop Count is zero ");
       RETURN(NPF_E_UNKNOWN);
   } 
  
   /* allocate memory for the context */ 
   nhopCntxt= (IPV4NhopCntxt_t*) PIL_AllocateMemory(
		            sizeof(IPV4NhopCntxt_t), PIL_MEM_ZERO_INIT); 
   if (!nhopCntxt)
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR, "Memory not allocated ");
       retVal = NPF_OUT_OF_MEMORY;
       goto CleanUp;
   }
  
    nhopCntxt->numEntries = nhCount;
   
   /* Allocate memory for nextHopArray*/ 
   nhopCntxt->nhopArray = (NPF_IPv4UC_NextHopArray_t*) PIL_AllocateMemory( 
		   (sizeof(NPF_IPv4UC_NextHopArray_t) * nhCount),PIL_MEM_ZERO_INIT); 

   if (!nhopCntxt->nhopArray)
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR, "Memory not allocated ");
       retVal = NPF_OUT_OF_MEMORY;
       goto CleanUp;
   }
   
  
   /* Allocate memory for nhIdArray */ 
   nhopCntxt->nhIdArray = (NPF_uint32_t*) PIL_AllocateMemory( 
		   (sizeof(NPF_uint32_t) * nhCount),PIL_MEM_ZERO_INIT); 

   if (!nhopCntxt->nhIdArray)
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR, "Memory not allocated ");
       retVal = NPF_OUT_OF_MEMORY;
       goto CleanUp;
   }
  
   /* Copy the context in this area */
   for ( i=0 ; i < nhCount; i++)
   {
      /* Allocate memory for nextHopArray*/ 
      nhopCntxt->nhopArray[i].nextHopArray = (NPF_IPv4UC_NextHop_t*) PIL_AllocateMemory(sizeof(NPF_IPv4UC_NextHop_t ),PIL_MEM_ZERO_INIT); 

      if (!nhopCntxt->nhopArray[i].nextHopArray)
      {
         npf_logger_Write(LOG_IPV4, LOG_ERR, "Memory not allocated ");
         retVal = NPF_OUT_OF_MEMORY;
         goto CleanUp;
      }
      /* We don't support more than one nexthops ; No support for ECMP */
      nhopCntxt->nhopArray[i].nextHopCount = 1;

      /* Copy the contents only if they are provided by the function */
      if(nhopArray && nhopArray[i].nextHopArray)
      {
         memcpy(nhopCntxt->nhopArray[i].nextHopArray , nhopArray[i].nextHopArray , sizeof(NPF_IPv4UC_NextHop_t)); 
      }
      if(nhIdArray && nhIdArray[i])
      {
         memcpy(&nhopCntxt->nhIdArray[i] , &nhIdArray[i], sizeof(NPF_uint32_t));
      }
   }

   /* Assign the pointer */
   *nhop = nhopCntxt;
   RETURN(NPF_SUCCESS);

CleanUp:
   if(ipv4_DelNhopCntxt(nhopCntxt) != NPF_SUCCESS)
      RETURN(NPF_E_UNKNOWN);
   RETURN(retVal);
} /*ipv4_CreateNhopCntxt*/


 
/*****************************************************************************
*    Name: ipv4_AddNhopCntxtToUsrReq
*
*    Description:
*       Initializes the memory for Next-Hop Context in user req
*
*    Prequisites: 
*        
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: 
*             NPF_E_UNKNOWN : 
*/

NPF_RET ipv4_AddNhopCntxtToUsrReq(
	IPV4UsrReqInfo_t *usrReq,
	uint32_t         nhCount,
	NPF_IPv4UC_NextHopArray_t *nhopArray,
	uint32_t         *nhIdArray
        ) 
{
   IPV4NhopCntxt_t      *nhopCntxt = NULL;
   NPF_RET              retVal;

   IPV4_DBG_TRACE();
  
   assert(usrReq);
  
   if((retVal = ipv4_CreateNhopCntxt( nhCount, nhopArray, nhIdArray, &nhopCntxt))
                                     != NPF_SUCCESS) 
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR, "NextHop Context creation failure ");
       RETURN(NPF_OUT_OF_MEMORY);
   }
   /* Assign it back to the user request */
   usrReq->nhopCntxt = nhopCntxt;

   RETURN(NPF_SUCCESS);
} /* ipv4_AddNhopCntxtToUsrReq*/

#ifdef IPV4_UNIF_SUPPORT
/*****************************************************************************
*    Name: ipv4_AddPrefixCntxtToUsrReq
*
*    Description:
*       Initializes the memory for Prefix Context in user req
*
*    Prequisites: 
*        
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: 
*             NPF_E_UNKNOWN : 
*/

NPF_RET ipv4_AddPrefixCntxtToUsrReq(
	IPV4UsrReqInfo_t *usrReq,
	uint32_t         prefixCount,
	NPF_IPv4UC_Prefix_t *prefixArray
   ) 
{
   IPV4PrefixCntxt_t      *prefixCntxt = NULL;
   uint32_t               i;  

   IPV4_DBG_TRACE();
  
   assert(usrReq);
   assert(!usrReq->prefixCntxt);
   
   /* allocate memory for the context */ 
   prefixCntxt= (IPV4PrefixCntxt_t*) PIL_AllocateMemory(
		            sizeof(IPV4PrefixCntxt_t), PIL_MEM_ZERO_INIT); 
   if (!prefixCntxt)
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR, "Memory not allocated ");
       RETURN(NPF_OUT_OF_MEMORY);
   }
   prefixCntxt->numEntries = prefixCount;
   /* Allocate memory for nextHopArray*/ 
   prefixCntxt->prefixArray = (NPF_IPv4UC_Prefix_t*) PIL_AllocateMemory( 
		   (sizeof(NPF_IPv4UC_Prefix_t) * prefixCount),PIL_MEM_ZERO_INIT); 

   if (!prefixCntxt->prefixArray)
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR, "Memory not allocated ");
       RETURN(NPF_OUT_OF_MEMORY);
   }

   /* Copy the context in this area */
   for ( i=0 ; i < prefixCount; i++)
   {
      prefixCntxt->prefixArray[i].IPv4Addr = prefixArray[i].IPv4Addr;
      prefixCntxt->prefixArray[i].IPv4NetPlen = prefixArray[i].IPv4NetPlen;
   }

   usrReq->prefixCntxt = prefixCntxt;
   RETURN(NPF_SUCCESS);
} /* ipv4_AddPrefixCntxtToUsrReq*/
#endif

/*****************************************************************************
*    Name: ipv4_PrcUserReqToFPForArpAdd
*
*    Description:
*       Adds L2Entry in the L2Table
*
*    Prequisites: 
*        
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If Init is done properly 
*             NPF_E_UNKNOWN : If Init is not done properly 
*/

NPF_RET ipv4_PrcUserReqToFPForArpAdd(
         	NPF_IN IPV4UsrReqInfo_t *usrReq,
	         NPF_IN NPF_uint32_t numEntries,
	         NPF_IN NPF_IPv4UC_AddResEntry_t *addResArray,
	         NPF_OUT FPPI_IPv4UniNHResolEntry_t *fpL2Array
           ) 
{
   NPF_RET              retVal;
   NPF_IPv4UC_AddResEntry_t *addResEntry = NULL;
   uint32_t            numL2Id = 0, i, j, k;
   FPPI_L2ID            *newL2Id = NULL;
   IPV4IfAttrib_t       ifAttrib;
   IPV4TableInfo_t      *tableInfo = NULL;
   FPPI_FEID            feId;
   BOOL                 bFeExists = FALSE ;
	NPF_IN NPF_IPv4UC_AddResKey_t *addResKeyArray = NULL;

   IPV4_DBG_TRACE();
  
   assert(usrReq);
   assert(addResArray);
   assert(fpL2Array);
    
   /* Assign to local var */ 
   tableInfo = usrReq->transInfo->tableInfo;

   assert(tableInfo); /* safe check */
  
   for ( i=0; i <numEntries; i++)
   {
      addResEntry = &addResArray[i];
      /* Store the handle for query */
      ifAttrib.ifHndl = addResEntry->interfaceHandle;
      
      /* Egress Interface extracted. Now find the FE assocaited with it */
      if ((retVal= ipv4_GetFEPortAttribForIfFrmNS(&ifAttrib)) != NPF_SUCCESS)
      {
         npf_logger_Write(LOG_IPV4, LOG_ERR,
		    " Couldn't extract FeId associated with Egress If ");
	      RETURN(retVal);
      }
      
      /* Small check for L2Id addition based on Port Type */
      if( (ifAttrib.portType != NPF_IF_TYPE_LAN) && 
          (ifAttrib.portType != NPF_IF_TYPE_ATM)) 
      {
         npf_logger_Write(LOG_IPV4, LOG_TRACE,
		    " L2Id generation not needed for given Port type");
	      RETURN(NPF_SUCCESS);
      }
   
      /* We have found valid Port Type for the interface. Also, we have got
       * the associated FeIds with this interface. Now update the NextHop
       * Table on these FEs */

      if(ifAttrib.numFe > MAXFENUM)
      {
         npf_logger_Write(LOG_IPV4, LOG_ERR, 
			 "Interface is assocated with [%d] FEs which \
                           is  more than defined FEs",ifAttrib.numFe);
         RETURN(NPF_E_UNKNOWN);
      }
        
      for ( j =0 ; j<tableInfo->numFeTotal; j++) 
      {
         feId = tableInfo->feIdArray[j];
         /* Verify all associated FEs are existing */ 
         for( k=0; k<ifAttrib.numFe; k++)
         {
             if(feId == ifAttrib.feId[k])
                bFeExists = TRUE;
         }
         if( bFeExists == FALSE)
         {
            npf_logger_Write(LOG_IPV4, LOG_ERR,
                      "Interface's associated FEs are not existing ");
            RETURN(NPF_E_UNKNOWN);
         }

         npf_logger_Write(LOG_IPV4, LOG_TRACE, 
                   "Update AddRes Entry for each FE [FEId=%d]",
                    feId);
         if((retVal = l2idm_UpdateForL2Table(feId,1,
					          &addResEntry->IP_Address, 
					         (uint32_t*)&numL2Id, &newL2Id)) 
		                  != NPF_SUCCESS)
         {
             npf_logger_Write(LOG_IPV4, LOG_ERR,
                         "Error getting the L2Id corresponding \
			                 to ARP addresses added");        
             RETURN(retVal);
         }
         npf_logger_Write(LOG_IPV4, LOG_TRACE, "Got the L2Id for AddRes");
         /* Check for the valid L2Id number; FP doesn't support more than one L2id */
         if(numL2Id != 0 && newL2Id == NULL)
         {
             npf_logger_Write(LOG_IPV4, LOG_ERR,
                         "Error getting the L2Id corresponding \
			                 to ARP addresses added (NumL2Id = %d)",numL2Id);        
             RETURN(retVal);
         }
         /* Start filling the Block which has to go to FP */
         fpL2Array[i].l2id   =  newL2Id[i]; 
         fpL2Array[i].portId   = ifAttrib.portId;
         fpL2Array[i].portType = ifAttrib.portType;
         fpL2Array[i].l2Entry.IP_Address =  addResArray[i].IP_Address;
         fpL2Array[i].l2Entry.interfaceHandle = addResArray[i].interfaceHandle;
         fpL2Array[i].l2Entry.mediaAddress.type= addResArray[i].mediaAddress.type;
         switch(fpL2Array[i].l2Entry.mediaAddress.type)
         {
            case NPF_IF_TYPE_ATM:
            {
               memcpy(&fpL2Array[i].l2Entry.mediaAddress.u.ATM_Vc, 
                      &addResArray[i].mediaAddress.u.ATM_Vc, sizeof(NPF_VccAddr_t));
            }
            break;
            case NPF_IF_TYPE_LAN:
            {
               memcpy(&fpL2Array[i].l2Entry.mediaAddress.u.MAC_Address,
                   &addResArray[i].mediaAddress.u.MAC_Address, sizeof(NPF_MAC_Address_t));
            }
            break;
            default:
            {
               npf_logger_Write(LOG_IPV4, LOG_ERR, "Invalid Interface TYpe");
               RETURN(NPF_E_UNKNOWN);
            }
            break;
         }

       } /* end of for */

   } /* For each entry in AddRes Entry from user */

   /*  We will store information in the addResContext, as we extract
       it from L2Id Manager ; while we get the callback of delete. */
   if(numEntries > 0) 
   {
      if((addResKeyArray = PIL_AllocateMemory(numEntries * sizeof(NPF_IPv4UC_AddResKey_t),PIL_MEM_ZERO_INIT))
         == NULL)
      {
          npf_logger_Write(LOG_IPV4, LOG_ERR, "AddRes Context creation failure ");
          RETURN(NPF_OUT_OF_MEMORY);
      }
      /* Copy into the Key Struc */ 
      for(i=0; i< numEntries; i++)
      {
         memcpy(&addResKeyArray[i].IP_Address, &addResEntry[i].IP_Address, 
                                             sizeof(NPF_IPv4Address_t));
         memcpy(&addResKeyArray[i].interfaceHandle, 
                &addResEntry[i].interfaceHandle, sizeof(NPF_IfHandle_t));
      }
   
      if((retVal = ipv4_AddAddResCntxtToUsrReq(usrReq, numEntries, 
                     addResKeyArray)) != NPF_SUCCESS)
      {
          npf_logger_Write(LOG_IPV4, LOG_ERR, "AddRes Context creation failure ");
          PIL_FreeMemory(addResKeyArray);
          RETURN(NPF_OUT_OF_MEMORY);
      }    
      /* We are going to remove the AddRes key context now */ 
      PIL_FreeMemory(addResKeyArray);
   }
   RETURN(NPF_SUCCESS);
}  /* ipv4_PrcUserReqToFPForArpAdd */ 

/*****************************************************************************
*    Name: ipv4_PrcUserReqToFPForArpDel
*
*    Description:
*       Deletes L2Entry in the L2Table
*
*    Prequisites: 
*        
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS:  
*             NPF_E_UNKNOWN :  
*/

NPF_RET ipv4_PrcUserReqToFPForArpDel(
	IPV4UsrReqInfo_t *usrReq,
	NPF_IN NPF_uint32_t numEntries,
	NPF_IN NPF_IPv4UC_AddResKey_t *addResKeyArray,
	NPF_OUT FPPI_IPv4UniNHResolEntry_t *fpL2Array
        ) 
{
   NPF_RET              retVal;
   uint32_t            numL2Id = 0, i, j, k;
   FPPI_L2ID            *newL2Id = NULL;
   IPV4IfAttrib_t       ifAttrib;
   IPV4TableInfo_t      *tableInfo = NULL;
   FPPI_FEID            feId;
   BOOL                 bFeExists = FALSE ;
	NPF_IN NPF_IPv4UC_AddResKey_t *addResKeyEntry = NULL;

   IPV4_DBG_TRACE();
  
   assert(usrReq);
   assert(addResKeyArray);
   assert(fpL2Array);
    
   /* Assign to local var */ 
   tableInfo = usrReq->transInfo->tableInfo;

   assert(tableInfo); /* safe check */
  
   for ( i=0; i <numEntries; i++)
   {
      addResKeyEntry = &addResKeyArray[i];
      /* Store the handle for query */
      ifAttrib.ifHndl = addResKeyEntry->interfaceHandle;
      
      /* Egress Interface extracted. Now find the FE assocaited with it */
      if ((retVal= ipv4_GetFEPortAttribForIfFrmNS(&ifAttrib)) != NPF_SUCCESS)
      {
         npf_logger_Write(LOG_IPV4, LOG_ERR,
		    " Couldn't extract FeId associated with Egress If ");
	      RETURN(retVal);
      }
      
      /* Small check for L2Id addition based on Port Type */
      if( (ifAttrib.portType != NPF_IF_TYPE_LAN) && 
          (ifAttrib.portType != NPF_IF_TYPE_ATM)) 
      {
         npf_logger_Write(LOG_IPV4, LOG_TRACE,
		    " L2Id generation not needed for given Port type");
	      RETURN(NPF_SUCCESS);
      }
   
      /* We have found valid Port Type for the interface. Also, we have got
       * the associated FeIds with this interface. Now update the NextHop
       * Table on these FEs */

      if(ifAttrib.numFe > MAXFENUM)
      {
         npf_logger_Write(LOG_IPV4, LOG_ERR, 
			 "Interface is assocated with [%d] FEs which \
                           is  more than defined FEs",ifAttrib.numFe);
         RETURN(NPF_E_UNKNOWN);
      }
        
      for ( j =0 ; j<tableInfo->numFeTotal; j++) 
      {
         feId = tableInfo->feIdArray[j];
         /* Verify all associated FEs are existing */ 
         for( k=0; k<ifAttrib.numFe; k++)
         {
             if(feId == ifAttrib.feId[k])
                bFeExists = TRUE;
         }
         if( bFeExists == FALSE)
         {
            npf_logger_Write(LOG_IPV4, LOG_ERR,
                      "Interface's associated FEs are not existing ");
            RETURN(NPF_E_UNKNOWN);
         }

         npf_logger_Write(LOG_IPV4, LOG_TRACE, 
                   "Update AddRes Entry for each FE [FEId=%d]",
                    feId);
         if((retVal = l2idm_GetForL2Table(feId,1,
					          &addResKeyEntry->IP_Address, 
					         (uint32_t*)&numL2Id, &newL2Id)) 
		                  != NPF_SUCCESS)
         {
             npf_logger_Write(LOG_IPV4, LOG_ERR,
                         "Error getting the L2Id corresponding \
			                 to ARP addresses deleted");        
             RETURN(retVal);
         }
         npf_logger_Write(LOG_IPV4, LOG_TRACE, "Deleted the L2Id for AddRes");
         /* Check for the valid L2Id number; FP doesn't support more than one L2id */
         if(numL2Id != 0 && newL2Id == NULL)
         {
             npf_logger_Write(LOG_IPV4, LOG_ERR,
                         "Error getting the L2Id corresponding \
			                 to ARP addresses added (NumL2Id = %d)",numL2Id);        
             RETURN(retVal);
         }
         /* Start filling the Block which has to go to FP */
         fpL2Array[i].l2id   =  newL2Id[i]; 
         fpL2Array[i].portId   = ifAttrib.portId;
         fpL2Array[i].portType = ifAttrib.portType;
         fpL2Array[i].l2Entry.IP_Address =  addResKeyEntry->IP_Address;
         fpL2Array[i].l2Entry.interfaceHandle = addResKeyEntry->interfaceHandle;
       } /* end of for */

   } /* For each entry in AddRes Entry from user */

   /*  We will store information in the addResContext, as we extract
       it from L2Id Manager ; while we get the callback of delete. */
   if((retVal = ipv4_AddAddResCntxtToUsrReq(usrReq, numEntries, 
                     addResKeyArray)) != NPF_SUCCESS)
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR, "AddRes Context creation failure ");
       RETURN(NPF_OUT_OF_MEMORY);
   }    
   RETURN(NPF_SUCCESS);
}  /* ipv4_PrcUserReqToFPForArpDel*/




/*
   ############################################################################
   IPV4 User-Info Module Helper Functions
   ############################################################################
*/

/*****************************************************************************
*    Name: ipv4_AddTableToUsr 
*
*    Description:
*    	Table block enabled in Table Array in User Node. This is the
*    	table enabled when a request comes from the IPV4 User App to create
*    	a table handle.
*
*
*    Prequisites: 
*       For Adding FIB Table, it is assumed that Prefix and NHop tables are
*       already added before calling this routine.
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If Init is done properly 
*             NPF_E_UNKNOWN : If Init is not done properly 
*/

NPF_RET ipv4_AddTableToUsr(
        IPV4UserInfo_t  *userInfo,
	     IPV4TableType_t type
        ) 
{
   NPF_RET retVal;
   IPV4TableInfo_t *tableArray = NULL;
   
   IPV4_DBG_TRACE();
   assert(userInfo);
   assert(userInfo->tableArray);
   
   tableArray = userInfo->tableArray;

#ifdef IPV4_UNIF_SUPPORT
   /* First check if the table added is unified */
   if ( type == IPV4_TABLE_TYPE_FIB)
   {
      /* Check if FIB table has prefix or NHop Tables */
      if((tableArray[type].tblHndlMap.pfxTableInfo == NULL) ||
         (tableArray[type].tblHndlMap.nhopTableInfo == NULL))
      {
         npf_logger_Write(LOG_IPV4, LOG_ERR, " FIB table doesn't\
                     have associated Discrete Table Mapping present");
         RETURN(NPF_E_UNKNOWN);
      }
   }
#endif /* IPV4_UNIF_SUPPORT */
   
   /* We are using table type enum as index in array,for faster access */
   tableArray[type].type= type; 
   
   /* Special NAMESPACE interaction while creating Prefix Table and FIB Table */
   if (type == IPV4_TABLE_TYPE_PREFIX 
#ifdef IPV4_UNIF_SUPPORT
      || type == IPV4_TABLE_TYPE_FIB
#endif /*IPV4_UNIF_SUPPORT*/
      )
   {
      /* Handle value is taken from NS for Pfx Table */
      if( (retVal= ipv4_CreateRtrNodeInNS((NPF_HANDLE*)&tableArray[type].extHandle)
			      != NPF_SUCCESS))
      {
	      tableArray[type].extHandle = (NPF_uint32_t)NULL;
	      tableArray[type].intHandle = (NPF_uint32_t)NULL;
         RETURN(retVal);
      }
   }
   
   /* Create handle values internally */
   ipv4_GetHandleFrmMgr(&tableArray[type].intHandle);

   /* Special processing of external handle for AddRes Table */
   if(type == IPV4_TABLE_TYPE_ADDRES)
   {
      /* For Address Res Table, we need to check if Prefix Table 
       * has been already created. */
      if((tableArray[IPV4_TABLE_TYPE_PREFIX].state == TABLE_INIT) ||
          (tableArray[IPV4_TABLE_TYPE_PREFIX].extHandle == 0))
      {
         npf_logger_Write(LOG_IPV4, LOG_ERR,
                      "Add Res Table doesn't have associated Prefix Table existing!! ");

         RETURN(NPF_E_UNKNOWN);
      }
      /* Assign the external Handle of Addres Table to the Prefix Handle*/
      tableArray[type].extHandle =
               tableArray[IPV4_TABLE_TYPE_PREFIX].extHandle;
   }

   RETURN(NPF_SUCCESS);
}/* ipv4_AddTableToUsr */


/*****************************************************************************
*    Name: ipv4_RemTableFrmUsr 
*
*    Description:
*    	Remove Table block from the Table Array in User Node. This is the
*    	table removed when a request comes from the IPV4 User App to delete 
*    	a table handle.
*
*    Prequisites: 
*       For Removing FIB Table, it is assumed that Prefix and NHop tables are
*       already removed before calling this routine.
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_RemTableFrmUsr(
        IPV4UserInfo_t  *userInfo,
        IPV4TableType_t type
        ) 
{
   NPF_RET retVal = NPF_SUCCESS;
   IPV4TableInfo_t *tableArray = NULL;
   
   IPV4_DBG_TRACE();
   assert(userInfo);
   assert(userInfo->tableArray);

   tableArray = userInfo->tableArray;

#ifdef IPV4_UNIF_SUPPORT

   /* First check if the table added is unified */
   if ( type == IPV4_TABLE_TYPE_FIB)
   {
      if((tableArray[type].tblHndlMap.pfxTableInfo == NULL) || 
          (tableArray[type].tblHndlMap.nhopTableInfo == NULL))
      {
         npf_logger_Write(LOG_IPV4, LOG_ERR,
                      "FIB Table doesn't have associated tables (disc) associated !! ");
	      RETURN (NPF_E_UNKNOWN);
      }
   }
 
#endif /*IPV4_UNIF_SUPPORT*/

   /* Special NAMESPACE interaction while creating Prefix Table */
   if (type == IPV4_TABLE_TYPE_PREFIX || type == IPV4_TABLE_TYPE_FIB )
   {
      /* Handle value is taken from NS for Pfx Table; Ext. Handle 
         is used to delete the router node in NS */
      if( (retVal= ipv4_DeleteRtrNodeInNS(&tableArray[type].extHandle)
			      != NPF_SUCCESS))
      {
         npf_logger_Write(LOG_IPV4, LOG_ERR, 
         "Unable to delete Router Node in [handle=0x%x] in NS ",
                                     tableArray[type].extHandle);
         RETURN(retVal);
      }
   }
   /* Reset the handles to NULL */
   tableArray[type].extHandle = (uint32_t)NULL; 
   tableArray[type].intHandle = (uint32_t)NULL; 
   
   RETURN(retVal);
}/* ipv4_RemTableFrmUsr*/


/*****************************************************************************
*    Name: ipv4_AddTransInListToUsr 
*
*    Description:
*    	Add Transaction block to the transaction list in User Node. This is the
*    	transaction created when a request comes from the IPV4 User App.
*
*    Prequisites: 
*       transInfo should already have been created and filled with requisite
*       info before calling this function.
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If Init is done properly 
*             NPF_E_UNKNOWN : If Init is not done properly 
*/

NPF_RET ipv4_AddTransInListToUsr(
        IPV4UserInfo_t  *userInfo,
	     IPV4TransInfo_t *transInfo
        ) 
{
   NPF_uint32_t correl; 
   IPV4UsrReqInfo_t *usrReq = NULL;

   IPV4_DBG_TRACE();
   assert(transInfo);
   assert(userInfo);
   
   usrReq  = transInfo->usrReq;

#ifdef IPV4_UNIF_SUPPORT
   if(transInfo->usrReq->transType == IPV4_TRANS_DISC)
#endif /* IPV4_UNIF_SUPPORT */
   {
      correl = transInfo->usr_correlator;
      npf_list_pushBack(&userInfo->transCpList, (const void *)transInfo);
   }
#ifdef IPV4_UNIF_SUPPORT
   else
   {
      correl = transInfo->int_correlator;
     /* Before inserting, search if this alrady exists */  
      if(npf_list_searchByKey(&userInfo->unifiedTransCpList, (const void *)&correl) != NULL)
      {
         npf_logger_Write(LOG_IPV4, LOG_ERR, " Duplicate Transaction ");
         RETURN( NPF_E_UNKNOWN );
      }
      npf_list_pushBack(&userInfo->unifiedTransCpList, (const void *)transInfo);
   }
#endif /* IPV4_UNIF_SUPPORT */

   /* Set the back pointer after inserting it into the List */ 
   transInfo->usrInfo = userInfo;
   
   RETURN(NPF_SUCCESS);

}/* ipv4_AddTransInListToUsr */


/*****************************************************************************
*    Name: ipv4_DelTransInListFrmUsr 
*
*    Description:
*    	Removes Transaction block from the transaction list in User Node. 
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If Init is done properly 
*             NPF_E_UNKNOWN : If Init is not done properly 
*/

NPF_RET ipv4_DelTransInListFrmUsr(
	     IPV4TransInfo_t *transInfo
        ) 
{
   NPF_uint32_t correl; 
   IPV4UsrReqInfo_t *usrReq=NULL;
   IPV4UserInfo_t   *usrInfo=NULL;
   IPV4TransInfo_t  *delTrans = NULL;
  
   IPV4_DBG_TRACE();
   
   assert(transInfo);
   
   usrReq  = transInfo->usrReq;
   usrInfo = transInfo->usrInfo;
#ifdef IPV4_UNIF_SUPPORT
   if(usrReq->transType == IPV4_TRANS_DISC)
#endif /* IPV4_UNIF_SUPPORT */
   {
      correl = transInfo->usr_correlator;
      if(npf_list_removeByKey(&usrInfo->transCpList, 
			    (const void *)&correl,
			    (void **)&delTrans) == -1)
      {
          npf_logger_Write(LOG_IPV4, LOG_ERR,
                         "Error finding transaction in list");
          RETURN(NPF_E_UNKNOWN);
      }
   }
#ifdef IPV4_UNIF_SUPPORT
   else
   {
      correl = transInfo->int_correlator;
      if(npf_list_removeByKey(&usrInfo->unifiedTransCpList, 
			    (const void *)&correl,
			    (void **)&delTrans) == -1)
      {
          npf_logger_Write(LOG_IPV4, LOG_ERR,
                         "Error finding transaction in list");
          RETURN(NPF_E_UNKNOWN);
      }
   }
#endif /* IPV4_UNIF_SUPPORT */
    /* Check what is removed was same as what was intended */
    if(delTrans != transInfo)
    {
        npf_logger_Write(LOG_IPV4, LOG_ERR,
                         "Invalid transaction was removed !! ");
        RETURN(NPF_E_UNKNOWN);
    } 
   RETURN(NPF_SUCCESS);
}/* ipv4_DelTransInListFrmUsr */

/*****************************************************************************
*    Name: ipv4_GetTransByUCorrelFrmUsr
*
*    Description:
*    	Gets the trans info by searching on User-Correlator
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If search results in success 
*             NPF_E_UNKNOWN : If search results in failure 
*/

NPF_RET ipv4_GetTransByCorrelFrmUsr(
        IPV4UserInfo_t    *userInfo,
	     NPF_correlator_t  correl, 
        IPV4TransInfo_t **transInfo
        ) 
{
   IPV4_DBG_TRACE();
   assert(userInfo);

   if ((*transInfo = npf_list_searchByKey(&userInfo->transCpList,
				   (const void *)&correl)) == NULL)
   {
        npf_logger_Write(LOG_IPV4, LOG_TRACE, 
                   "No transaction with correlator %d found ", correl);
        RETURN(NPF_E_UNKNOWN);
   }
   RETURN(NPF_SUCCESS);
} /* ipv4_GetTransByCorrelFrmUsr */

#ifdef IPV4_UNIF_SUPPORT
/*****************************************************************************
*    Name: ipv4_GetUnifiedTransByUCorrelFrmUsr
*
*    Description:
*    	Gets the trans info by searching on User-Correlator
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If search results in success 
*             NPF_E_UNKNOWN : If search results in failure 
*/

NPF_RET ipv4_GetUnifiedTransByCorrelFrmUsr(
        IPV4UserInfo_t    *userInfo,
        NPF_correlator_t  correl, 
        IPV4TransInfo_t **transInfo
        ) 
{
   IPV4_DBG_TRACE();
   assert(userInfo);

   if ((*transInfo = npf_list_searchByKey(&userInfo->unifiedTransCpList,
				   (const void *)&correl)) == NULL)
   {
        npf_logger_Write(LOG_IPV4, LOG_TRACE, 
                   "No transaction with correlator %d found ", correl);
        RETURN(NPF_E_UNKNOWN);
   }
   RETURN(NPF_SUCCESS);
} /* ipv4_GetUnifiedTransByCorrelFrmUsr */

/*****************************************************************************
*    Name: ipv4_GetICorrelFrmUsr
*
*    Description:
*    	Get internal correlator from User. 
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_GetICorrelFrmUsr(
		IPV4UserInfo_t *userInfo, 
		NPF_uint32_t *correl
		) 
{
   IPV4_DBG_TRACE();
   USER_LOCK();
   *correl = ++userInfo->intCorrelCntr;
   USER_UNLOCK();

   RETURN(NPF_SUCCESS);
} /* ipv4_GetICorrelFrmUsr*/

 /*****************************************************************************
*    Name: ipv4_GetNhIdFrmUsr
*
*    Description:
*    	Get NHId from User. 
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_GetNhIdFrmUsr(
		IPV4UserInfo_t *userInfo, 
		NPF_uint32_t *nhId
		) 
{
   IPV4_DBG_TRACE();
   USER_LOCK();
   *nhId= ++userInfo->nhIdCntr;
   USER_UNLOCK();
   
   RETURN(NPF_SUCCESS);

} /* ipv4_GetNhIdFrmUsr*/
#endif /* IPV4_UNIF_SUPPORT */ 

/*****************************************************************************
*    Name: ipv4_GetTableByHndlFrmUsr
*
*    Description:
*    	Get Table from User. 
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If Search is successful 
*             NPF_E_UNKNOWN : If search is failure 
*/

NPF_RET ipv4_GetTableByHndlFrmUsr(
      IPV4UserInfo_t *userInfo,
      IPV4TableType_t  tableType,
		NPF_HANDLE   tblHandle,
		IPV4TableInfo_t  **tableInfo
		) 
{
   NPF_RET retVal;
   uint32_t tblNum;

   IPV4_DBG_TRACE();

   USER_LOCK();
   
   retVal = NPF_E_UNKNOWN; 

   for (tblNum=0; tblNum < userInfo->numTables ; tblNum++)
   {
      if(userInfo->tableArray[tblNum].intHandle == tblHandle)
      {
          /* Handle match found; Make an extra check for the type of table */
          if (tblNum == tableType)
          {
	          *tableInfo = &userInfo->tableArray[tblNum];
	          retVal = NPF_SUCCESS;
 	          break;
          }
      }
   }
   USER_UNLOCK();
   
   RETURN(retVal);
} /* ipv4_GetTableByHndlFrmUsr*/



/*****************************************************************************
*    Name: ipv4_InitUserInfo 
*
*    Description:
*    	Initialize User Information.
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If Init is done properly 
*             NPF_E_UNKNOWN : If Init is not done properly 
*/

NPF_RET ipv4_InitUserInfo(
	NPF_userContext_t         userContext,
        NPF_IPv4UC_CallbackFunc_t cbFunc, 
	IPV4UserInfo_t            **usrPtr
	) 
{
   IPV4UserInfo_t  *userInfo = NULL;
   IPV4_DBG_TRACE();
   
   userInfo = (IPV4UserInfo_t *)PIL_AllocateMemory( 
            sizeof( IPV4UserInfo_t ), PIL_MEM_ZERO_INIT );
    
   if (!userInfo)
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR, "Memory not allocated ");
       RETURN( NPF_OUT_OF_MEMORY);
   }
   
   /* Init critical section for this user */ 
   PIL_InitializeCriticalSection( &userInfo->user_lock);
 
   USER_LOCK();
   
   /* Assign the callback func. */
   userInfo->cbFunc = cbFunc;
   
   /* Store the context given by the Application */
   userInfo->context = userContext; 
      
   /* Get the callback handle from the IPV4Mgr_t */
   ipv4_GetHandleFrmMgr(&userInfo->cbHndl);

#ifdef IPV4_UNIF_SUPPORT 
   /* Initialize User specific Counters*/
   userInfo->nhIdCntr = 0;
   userInfo->intCorrelCntr = 0;
#endif /* IPV4_UNIF_SUPPORT */

   /* Init User Lists */
    npf_list_init(&userInfo->transCpList, ipv4_FreeTransCpInfo);/* Trans. List */
    userInfo->transCpList.compare = ipv4_CompareTransCpList;/* Compare Func. for
							       List of Trans. */
#ifdef IPV4_UNIF_SUPPORT 
    npf_list_init(&userInfo->unifiedTransCpList, ipv4_FreeTransCpInfo);/* Trans. List */
    userInfo->unifiedTransCpList.compare = ipv4_CompareUnifiedTransCpList;/* Compare Func. for List of Trans. */
#endif

    /* Create the memory for the max tables (Unified : 4 tables) */
    userInfo->numTables = IPV4_MAX_NUM_TABLES;

    userInfo->tableArray = (IPV4TableInfo_t *)PIL_AllocateMemory( 
                            (sizeof( IPV4TableInfo_t )* userInfo->numTables),
			    PIL_MEM_ZERO_INIT);
  
    if (!userInfo->tableArray)
    {
       npf_logger_Write(LOG_IPV4, LOG_ERR, "Memory not allocated ");
       USER_UNLOCK();
       RETURN(NPF_OUT_OF_MEMORY);
    }
    /* Give the value back */
    *usrPtr = userInfo;
   
   USER_UNLOCK();
   RETURN(NPF_NO_ERROR);
} /* ipv4_InitUserInfo */


/*****************************************************************************
*    Name: ipv4_DelTblCtxtFrmUsr
*
*    Description:
*          Delete context associated with the table.
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_DelTblCtxtFrmUsr(
        IPV4UserInfo_t *userInfo,
        IPV4TableType_t tblType
        ) 
{
   uint32_t i;
   IPV4TableInfo_t *tableInfo = NULL;
   IPV4_DBG_TRACE();
   
   tableInfo = &userInfo->tableArray[tblType]; 
   assert(tableInfo);

   for(i=0 ; i< MAXFENUM ; i++)
   {
      tableInfo->feIdArray[i] = (FPPI_FEID)NULL;
   }
   
   /* Reset the handles for resuse */
   tableInfo->intHandle = (uint32_t)NULL;
   tableInfo->extHandle = (uint32_t)NULL;
   
   tableInfo->tblHndlMap.pfxTableInfo = NULL; 
   tableInfo->tblHndlMap.nhopTableInfo = NULL; 

   RETURN(NPF_SUCCESS);
} /* ipv4_DelTblCtxtFrmUsr */



/*****************************************************************************
*    Name: ipv4_RemoveUserInfo 
*
*    Description:
*    	Delete User Information.
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_RemoveUserInfo(
        IPV4UserInfo_t *userInfo
        ) 
{
   uint16_t i;
   
   IPV4_DBG_TRACE();

   if (!userInfo)
   {
      npf_logger_Write(LOG_IPV4, LOG_ERR, " Invalid Parameters ");
      RETURN(NPF_INVALID_PARAMETERS);
   }
   
   USER_LOCK();
   
   /* Destroy User Lists and Arrays */
   npf_list_destroy(&userInfo->transCpList); /* Trans. List */
#ifdef IPV4_UNIF_SUPPORT
   npf_list_destroy(&userInfo->unifiedTransCpList); /* Trans. List */
#endif   
   /* For each table, there is an array of FeIds which has to be deleted */ 
   if (userInfo->tableArray != NULL)
   {
      for ( i=0; i< userInfo->numTables; i++)
      {
         /* Remove the table from User; Deletes the Namespace 
            association also */
	      ipv4_RemTableFrmUsr(userInfo,i);
    	   ipv4_DelTblCtxtFrmUsr(userInfo,i); 
      }
      PIL_FreeMemory(userInfo->tableArray);
      userInfo->tableArray = NULL;
   }
   USER_UNLOCK();

   /* Destroy User - Lock */
   PIL_DestroyCriticalSection( &userInfo->user_lock );
   /* Finally - destroy the memory */
   PIL_FreeMemory(userInfo);
   
   RETURN(NPF_NO_ERROR);
} /* ipv4_RemoveUserInfo */



/*
   ############################################################################
   IPV4 FPP Module Helper Functions
   ############################################################################
*/

/*****************************************************************************
*    Name: ipv4_AllocateFPNhopDelBlock
*
*    Description:
*              Allocate FP Nhop Block for Deletion of entry
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_AllocateFPNhopDelBlock(
            NPF_IN  uint32_t  numEntries,
	         NPF_OUT FPPI_UnicastNextHopDeleteInfo_t **fpNhopDelBlock
		       ) 
{
   FPPI_UnicastNextHopDeleteInfo_t *fpNhop = NULL;
   IPV4_DBG_TRACE();
   
   if(!numEntries)
   {
      npf_logger_Write(LOG_IPV4, LOG_ERR, " Number of NextHop Entries is zero ");
      RETURN(NPF_E_UNKNOWN);
   }
     /* Allocate memory for the Next-Hop Block to be sent to FP through FPPAPI */ 
   fpNhop = (FPPI_UnicastNextHopDeleteInfo_t *) PIL_AllocateMemory(
                     (sizeof(FPPI_UnicastNextHopDeleteInfo_t)*numEntries), PIL_MEM_ZERO_INIT);
     
   if(fpNhop == NULL)
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR,
			"Memory Allcoation failed ");
       RETURN(NPF_OUT_OF_MEMORY);
   }
  /* return the address */
  *fpNhopDelBlock= fpNhop; 
  RETURN(NPF_SUCCESS); 
} /* ipv4_AllocateFPNhopDelBlock*/ 

/*****************************************************************************
*    Name: ipv4_FreeFPNhopDelBlock
*
*    Description:
*              Free FP Nhop Block used for deletion of entries
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_FreeFPNhopDelBlock(
	         NPF_IN FPPI_UnicastNextHopDeleteInfo_t *fpNextHopBlock
		       ) 
{
   IPV4_DBG_TRACE();

   if(fpNextHopBlock == NULL)
      RETURN(NPF_SUCCESS);
   PIL_FreeMemory(fpNextHopBlock); 
       
   RETURN(NPF_SUCCESS); 
} /* ipv4_FreeFPNhopDelBlock*/ 

/*****************************************************************************
*    Name: ipv4_AllocateFPNhopAddBlock
*
*    Description:
*              Allocate FP Nhop Block
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_AllocateFPNhopAddBlock(
              NPF_IN  uint32_t  numEntries,
	           NPF_OUT FPPI_UnicastNextHopUpdateInfo_t **fpNextHopBlock
		       ) 
{
   FPPI_UnicastNextHopUpdateInfo_t *fpNhop = NULL;
   uint32_t i;

   IPV4_DBG_TRACE();
   
   if(!numEntries)
   {
      npf_logger_Write(LOG_IPV4, LOG_ERR, 
                " Number of NextHop Entries is zero ");
      RETURN(NPF_E_UNKNOWN);
   }
     /* Allocate memory for the Array of Next-Hop Block to be sent to 
     FP through FPPAPI */ 
   fpNhop = (FPPI_UnicastNextHopUpdateInfo_t*) PIL_AllocateMemory(
                  (sizeof(FPPI_UnicastNextHopUpdateInfo_t)*numEntries), 
                  PIL_MEM_ZERO_INIT);
     
   if(fpNhop == NULL)
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR,
			"Memory Allcoation failed ");
       RETURN(NPF_OUT_OF_MEMORY);
   }
   /* For each FP Nhop Block, create space for holding IPA etc. */ 
   for (i =0; i< numEntries; i++)
   {
      /* We are supprint only one entry per nexthop */
      fpNhop[i].nhInfo = (FPPI_NextHopInfo*) PIL_AllocateMemory(
                    sizeof(FPPI_NextHopInfo), PIL_MEM_ZERO_INIT);

      if(fpNhop[i].nhInfo == NULL)
      {
        npf_logger_Write(LOG_IPV4, LOG_ERR,
               "Memory Allcoation failed ");
        RETURN(NPF_OUT_OF_MEMORY);
      }
   }
   /* return the address */
   *fpNextHopBlock = fpNhop; 
   RETURN(NPF_SUCCESS); 
} /* ipv4_AllocateFPNhopAddBlock*/ 

/*****************************************************************************
*    Name: ipv4_FreeFPNhopAddBlock
*
*    Description:
*              Free FP Nhop Block
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_FreeFPNhopAddBlock(
            uint32_t numEntries,
	         NPF_IN FPPI_UnicastNextHopUpdateInfo_t *fpNextHopBlock
		       ) 
{
   uint32_t i;
   IPV4_DBG_TRACE();

   if(fpNextHopBlock == NULL)       
      RETURN(NPF_SUCCESS);
 
   /* De-allocate memory for the Next-Hop Block to 
      be sent to FP through FPPAPI */ 
   for (i=0; i < numEntries; i++)
   {
     if(fpNextHopBlock[i].nhInfo != NULL)
        PIL_FreeMemory(fpNextHopBlock[i].nhInfo); 
   }
   
   PIL_FreeMemory(fpNextHopBlock); 
       
   RETURN(NPF_SUCCESS); 
} /* ipv4_FreeFPNhopAddBlock*/ 


/*****************************************************************************
*    Name: ipv4_ExtractFppResponse
*
*    Description:
*              This functions fills in the NPF_IPv4UC_CallbackData_t function from
*              the FPP returned status
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_ExtractFppResponse(
	         NPF_IN npf_IPv4EntryStatusList *fpEntryStatus,
            NPF_IN NPF_IPv4UC_CallbackData_t *cbData
		      ) 
{
   uint32_t respSize, i;
   BOOL    allOK = TRUE;
   IPV4_DBG_TRACE();

   assert(cbData);
   assert(fpEntryStatus); 
   if(fpEntryStatus->numEntries == 0)
   {
      npf_logger_Write(LOG_IPV4, LOG_TRACE, " Number of responses from FwdPlane is zero !! ");
      cbData->numResp  = 0;
      cbData->allOK = NPF_FALSE;
      RETURN(NPF_SUCCESS);
   } 
   /* Valid response from FP */
   cbData->numResp  = fpEntryStatus->numEntries;
   respSize =  (sizeof(NPF_IPv4UC_AsyncResponse_t)* (fpEntryStatus->numEntries));
   cbData->resp = (NPF_IPv4UC_AsyncResponse_t*) PIL_AllocateMemory( respSize, PIL_MEM_ZERO_INIT);
   if(cbData->resp == NULL)
   {
      npf_logger_Write(LOG_IPV4, LOG_ERR, " Memory Allocation Failure ");
      RETURN(NPF_OUT_OF_MEMORY);
   } 
   /* Fill the App Callback Data */
    for( i =0; i < fpEntryStatus->numEntries; i++)
    {
       cbData->resp[i].returnCode  = fpEntryStatus->entryList[i].retStatus;
       if(cbData->resp[i].returnCode != FPPI_SUCCESS)
         allOK = FALSE; 
    }
    if(allOK == TRUE) 
       cbData->allOK = NPF_TRUE;
    else
       cbData->allOK = NPF_FALSE;
    RETURN(NPF_SUCCESS);
} 



/*****************************************************************************
*    Name: ipv4_CreateFpPfxBlock
*
*    Description:
*        Allocate memory for Prefix Entry/Delete Data block that is to be sent 
*        to FP.
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_CreateFpPfxDataBlock(
	         NPF_IN NPF_uint32_t numEntries, 
	         NPF_IN NPF_IPv4UC_Prefix_t *prefixArray,
	         NPF_IN NPF_uint32_t *nextHopIdArray,
            NPF_OUT FPPI_UnicastPrefixUpdateInfo_t **pfxBlock
		       ) 
{
   FPPI_UnicastPrefixUpdateInfo_t *fpPfxArray = NULL;
   uint32_t i;
   IPV4_DBG_TRACE();
   
   assert(prefixArray);
   assert(nextHopIdArray);
   assert(numEntries);
    
   /* Allocate memory for this FP Data block*/
   fpPfxArray = (FPPI_UnicastPrefixUpdateInfo_t*)PIL_AllocateMemory(
		   (sizeof(FPPI_UnicastPrefixUpdateInfo_t)*numEntries), PIL_MEM_ZERO_INIT);
   
   if (!fpPfxArray)
   {
      npf_logger_Write(LOG_IPV4, LOG_ERR, " Memory Allocation Failure ");
      RETURN(NPF_OUT_OF_MEMORY);
   }
   
   /* Copy the data from user */  
   for ( i=0 ; i < numEntries ; i++)
   {
     fpPfxArray[i].nextHopId = nextHopIdArray[i]; 
     fpPfxArray[i].prefixEntry = prefixArray[i];
   }
   
   /* Return the pointer */ 
   *pfxBlock = fpPfxArray;
   
   RETURN(NPF_SUCCESS);
} /* ipv4_CreateFpPfxDataBlock */

/*****************************************************************************
*    Name: ipv4_DelFpPfxBlock
*
*    Description:
*        Del-Allocate memory for Prefix Entry/Delete Data block that is to be sent 
*        to FP.
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_DelFpPfxDataBlock(
            NPF_IN FPPI_UnicastPrefixUpdateInfo_t *pfxBlock
		       ) 
{
   IPV4_DBG_TRACE();
   if(pfxBlock)
      PIL_FreeMemory(pfxBlock);
    
   RETURN(NPF_SUCCESS);
} /* ipv4_DelFpPfxDataBlock */



/*****************************************************************************
*    Name: ipv4_FindFEReqByCorrelInFpp
*
*    Description:
*       Finds a FE request that is sent to FE in FPP module with
*       FPPI_CORRELATOR as key
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_FindFEReqByCorrelInFpp(
		FPPI_FEID feId,
		FPPI_CORRELATOR fppi_correlator,
		IPV4ReqToFEInfo_t **req
		) 
{
   IPV4ReqToFEInfo_t *reqToFE = NULL;
   IPV4FppInfo_t     *fppInfo = NULL;
   
   IPV4_DBG_TRACE();
   
   assert(fppi_correlator);
   
   /* Local store */ 
   fppInfo = &ipv4Mgr.fppInfo;
   
   /* Search in the DList */ 
   if ((reqToFE = npf_list_searchByKey(&fppInfo->reqFpList,
				   (const void *)&fppi_correlator)) == NULL)
   {
        npf_logger_Write(LOG_IPV4, LOG_ERR,
        	"No ReqToFE element found having fppi_corelator as : %d ",
        	fppi_correlator);
	RETURN(NPF_E_UNKNOWN);
   }
   /* Do one more check */
   /* Match FeId from the cback and stored one */
   if (reqToFE->feId != feId)    
   {
        npf_logger_Write(LOG_IPV4, LOG_ERR,
			"Invalid FEId received from the callback which \
			does not match the stored FeId for this request");
	     RETURN(NPF_E_UNKNOWN);

   }
   /* Return the pointer value */
   *req = reqToFE ; 
   RETURN(NPF_SUCCESS);
} /* ipv4_FindFEReqByCorrelInFpp */


/*****************************************************************************
*    Name: ipv4_AddFEReqToFpp
*
*    Description:
*       Adds a new request that is sent to FE in FPP module.
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_AddFEReqToFpp(
		FPPI_FEID       feId,
		IPV4TransInfo_t *transInfo
		) 
{
   IPV4ReqToFEInfo_t *reqToFE = NULL;
   IPV4FppInfo_t     *fppInfo = NULL;
   uint32_t           i;
   
   IPV4_DBG_TRACE();
   
   assert(transInfo);
  
   fppInfo = &ipv4Mgr.fppInfo;
   
   IPV4_LOCK();  
   
   /* Allocate memory for this new request */
   reqToFE = (IPV4ReqToFEInfo_t *)PIL_AllocateMemory(
		   sizeof(IPV4ReqToFEInfo_t), PIL_MEM_ZERO_INIT);
   
   if (!reqToFE)
   {
      npf_logger_Write(LOG_IPV4, LOG_ERR, 
                      " Memory Allocation Failure ");
      IPV4_UNLOCK();  
      RETURN(NPF_OUT_OF_MEMORY);
   }
  
   /* Initialize various fields here */
   reqToFE->fppi_correlator = ++fppInfo->fppiCorrCntr;
   reqToFE->feId      = feId ;
   reqToFE->state     = REQ_TO_FE_PENDING ;
   reqToFE->retStatus = NPF_E_UNKNOWN; /* Init to error */
   reqToFE->transInfo = transInfo; /* Back pointer to actual trans. */
   
   /* Update list with a new request to FE */
   npf_list_pushBack(&fppInfo->reqFpList, (const void *)reqToFE);
   
   /* Update the pointer for this request in current transInfo; We
      are going to insert the request pointer in the first available
      index in the pointer list*/
    
   for( i=0; i < transInfo->tableInfo->numFeTotal; i++)
   {
      /* CHeck the first empty place in the array */
      if (transInfo->transFpInfo.reqFEPtrArray[i] == (IPV4ReqToFEInfo_t*)NULL) 
	   {
          transInfo->transFpInfo.numFEWait++;
          transInfo->transFpInfo.numFEToSend++;
          transInfo->transFpInfo.reqFEPtrArray[i] = reqToFE;
	   }
   }
  
   IPV4_UNLOCK();  
   
   RETURN (NPF_SUCCESS);
} /* ipv4_AddFEReqToFpp*/


/*****************************************************************************
*    Name: ipv4_DelFEReqFrmFpp
*
*    Description:
*       Delete request to FE from List in FPP  Module
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_DelFEReqFrmFpp(
			  FPPI_CORRELATOR fppi_correlator
			  )
{
   IPV4ReqToFEInfo_t *reqToFE = NULL;
   IPV4FppInfo_t *fppInfo = NULL; 
   IPV4TransInfo_t *transInfo = NULL;
   uint32_t i;
  
   IPV4_DBG_TRACE();
   
   IPV4_LOCK();  
   
   fppInfo = &ipv4Mgr.fppInfo;
   
   
   /* Remove the request from the list and get the pointer to the 
     * removed element */ 
    if(npf_list_removeByKey(&fppInfo->reqFpList, 
			    (const void *)&fppi_correlator, 
			    (void **)&reqToFE) == -1)
    {
        npf_logger_Write(LOG_IPV4, LOG_ERR,
                         "Error finding FPP request in list");
        IPV4_UNLOCK();  
        RETURN(NPF_E_UNKNOWN);
    }
    /* store the Trans. ptr. value in local var */
    transInfo = reqToFE->transInfo;

    /* remove the assocations from the transaction */ 
   if( (transInfo != (IPV4TransInfo_t*)NULL) && 
                (transInfo->tableInfo != (IPV4TableInfo_t*)NULL))
    {
       for(i=0; i < transInfo->tableInfo->numFeTotal; i++)
       {
	       /* If a match is found in the transaction, remove the asoc. */
          if(transInfo->transFpInfo.reqFEPtrArray[i] == reqToFE)
	       {
                   transInfo->transFpInfo.reqFEPtrArray[i]
                                          = (IPV4ReqToFEInfo_t*)NULL; 
                   transInfo->transFpInfo.numFEWait--;
	       }
       }
    }
   
   PIL_FreeMemory(reqToFE);
   
   IPV4_UNLOCK();  
   RETURN (NPF_SUCCESS);
} /* ipv4_DelFEReqFrmFpp*/
 
/*****************************************************************************
*    Name: ipv4_ProcessReqToFPP
*
*    Description:
*
*    Parameters:
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*       NPF_SUCCESS 
*       NPF_FAIL
*
*/
NPF_RET ipv4_ProcessReqToFPP(
	    IPV4UsrReqInfo_t      *usrReq
        ) 
{
   NPF_RET retVal;
   IPV4UserInfo_t        *userInfo  = NULL;
   IPV4TableInfo_t       *tableInfo = NULL;
   IPV4TransInfo_t       *newTrans  = NULL;
   uint32_t               i;
   FPPI_FEID              feId;
   
   IPV4_DBG_TRACE();
   
   newTrans = usrReq->transInfo;
   userInfo =  newTrans->usrInfo;
   tableInfo = newTrans->tableInfo;
   
   /* Step1: Find the Target FEs for this request */   
   if(!tableInfo->extHandle)
   {
       npf_logger_Write(LOG_IPV4, LOG_ERR,
           " Table External handle not set properly; Can't access Namespace ");
       RETURN(NPF_E_UNKNOWN);
   }  
   /* Helper function provided by If Mgmt which gets the 
      FE Ids if table handle is provided : Comes here  */
   if((retVal = IfGetFEIdsFrmFIBHdl(tableInfo->extHandle,
                    &tableInfo->numFeTotal,
                    tableInfo->feIdArray)) != NPF_SUCCESS) 
   {
         npf_logger_Write(LOG_IPV4, LOG_ERR,
           "Unable to get FE Ids associated with FIB Handle from Namespace ");
         RETURN(retVal);
   }
   
   /* Target FEs are available for this table */
   if (tableInfo->numFeTotal > MAXFENUM)
   {
      npf_logger_Write(LOG_IPV4, LOG_ERR,
                "Total number of FEs are more than %d ", MAXFENUM);
      RETURN(NPF_E_UNKNOWN);
   }

   /* Step2: Start initializing requests to each associated FE */
   for(i=0; i < tableInfo->numFeTotal; i++)
   {
      feId = tableInfo->feIdArray[i]; /* Get the FeId */
      /* Create a FP Request intance for this FE */
      if((retVal = ipv4_AddFEReqToFpp(feId,newTrans)) != NPF_SUCCESS)
      {
         npf_logger_Write(LOG_IPV4, LOG_ERR,
           " Associating FP request to FE [FeId = %d] Failed", feId);
         RETURN(retVal);
      }
   }
  RETURN(NPF_SUCCESS);
} /* ipv4_ProcessReqToFPP*/



/*****************************************************************************
*    Name: ipv4_FillCBInfoToFpp 
*
*    Description:
*    	Maps the Callback Functions defined with the Callback Values.
*
*     NOTE: Change in this function should affect enum values of
*           IPV4FppCBIndex_t 
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_FillCBInfoToFpp()
{
   IPV4FppCBInfo_t *cbInfoArray;
   IPV4FppInfo_t *fppInfo;
   
   IPV4_DBG_TRACE();
   
   fppInfo = &ipv4Mgr.fppInfo;
  
   assert(fppInfo->cbInfoArray);
   
   IPV4_LOCK()
   cbInfoArray = fppInfo->cbInfoArray;
   
   /* 1. Fill in the callback-function and Callback-Value info. Handles are
    * initialized to 0. They will be filled at the time of actual registration
    * with FPP
    * 2. In case some Callback is not implemented, the callback function will
    * be written as dummy function - to take care of future additions.
    * */
   cbInfoArray[0].cbType  = FPPI_IPv4_UniAddNextHop;
   cbInfoArray[0].cbFunc  = ipv4_AddNextHopComplete;
   cbInfoArray[0].cbHndl  = (FPPI_CBHANDLE)NULL;
   
   cbInfoArray[1].cbType  = FPPI_IPv4_UniDelNextHop;
   cbInfoArray[1].cbFunc  = ipv4_DeleteNextHopComplete;
   cbInfoArray[1].cbHndl  = (FPPI_CBHANDLE)NULL;
  
   cbInfoArray[2].cbType  = FPPI_IPv4_UniFlushNextHop;
   cbInfoArray[2].cbFunc  = ipv4_FlushNextHopComplete;
   cbInfoArray[2].cbHndl  = (FPPI_CBHANDLE)NULL;

   cbInfoArray[3].cbType  = FPPI_IPv4_UniQueryNextHop;
   cbInfoArray[3].cbFunc  = ipv4_QueryNextHopComplete;
   cbInfoArray[3].cbHndl  = (FPPI_CBHANDLE)NULL;
   
   cbInfoArray[4].cbType  = FPPI_IPv4_UniAddPrefix;
   cbInfoArray[4].cbFunc  = ipv4_AddPrefixesComplete;
   cbInfoArray[4].cbHndl  = (FPPI_CBHANDLE)NULL;
	
   cbInfoArray[5].cbType  = FPPI_IPv4_UniUpdatePrefix;
   cbInfoArray[5].cbFunc  = ipv4_UpdatePrefixesComplete;
   cbInfoArray[5].cbHndl  = (FPPI_CBHANDLE)NULL;

   cbInfoArray[6].cbType  = FPPI_IPv4_UniDelPrefix;
   cbInfoArray[6].cbFunc  = ipv4_DeletePrefixesComplete;
   cbInfoArray[6].cbHndl  = (FPPI_CBHANDLE)NULL;

   cbInfoArray[7].cbType  = FPPI_IPv4_UniFlushPrefix;
   cbInfoArray[7].cbFunc  = ipv4_FlushPrefixesComplete ;
   cbInfoArray[7].cbHndl  = (FPPI_CBHANDLE)NULL;

   cbInfoArray[8].cbType  = FPPI_IPv4_UniQueryPrefix;
   cbInfoArray[8].cbFunc  = ipv4_QueryPrefixesComplete ;
   cbInfoArray[8].cbHndl  = (FPPI_CBHANDLE)NULL;

   cbInfoArray[9].cbType  = FPPI_IPv4_UniAddToL2;
   cbInfoArray[9].cbFunc  = ipv4_AddL2EntryComplete;
   cbInfoArray[9].cbHndl  = (FPPI_CBHANDLE)NULL;
	
   cbInfoArray[10].cbType  = FPPI_IPv4_UniDelToL2;
   cbInfoArray[10].cbFunc  = ipv4_DeleteL2EntryComplete;
   cbInfoArray[10].cbHndl  = (FPPI_CBHANDLE)NULL;
   
   cbInfoArray[11].cbType  = FPPI_IPv4_UniFlushToL2;
   cbInfoArray[11].cbFunc  = ipv4_FlushL2EntryComplete;
   cbInfoArray[11].cbHndl  = (FPPI_CBHANDLE)NULL;
   
   IPV4_UNLOCK();
   RETURN(NPF_NO_ERROR);
} /* ipv4_FillCBInfoToFpp */


/*****************************************************************************
*    Name: ipv4_InitFppInfo
*
*    Description:
*    	Initialize IPV4 Mgr. FPP Information. All the registartions done with
*    	the FPP module are done in this function. This func. is called as a
*    	part of IPV4 Initialization process.
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_InitFppInfo() 
{
   NPF_RET retVal;
   IPV4FppInfo_t *fppInfo;
   NPF_uint32_t cbCount =0;
   
   IPV4_DBG_TRACE();
  
   fppInfo = &ipv4Mgr.fppInfo;
   assert (fppInfo);
    
   IPV4_LOCK()
   
   /* Initialize the FPPI Correlator counter */
   fppInfo->fppiCorrCntr = 0; 
   
   /*Req. to FP List */
   npf_list_init(&fppInfo->reqFpList, ipv4_FreeReqToFP);  
    
   /*Compare Routine - for List of Req to FP */
   fppInfo->reqFpList.compare = ipv4_CompareReqToFPList; 
   
   /* Get the max. number of valid FPPI Callbacks */ 
   fppInfo->numFpCB = IPV4_MAX_CBACK_NUM; 
   /* Alloate memory for the all the Callbacks to FP */
   fppInfo->cbInfoArray = (IPV4FppCBInfo_t*)PIL_AllocateMemory(
      (sizeof(IPV4FppCBInfo_t)*(fppInfo->numFpCB)), PIL_MEM_ZERO_INIT);
   
   if( fppInfo->cbInfoArray == NULL)
   {
      npf_logger_Write(LOG_IPV4, LOG_ERR, "Memory not allocated ");
      IPV4_UNLOCK()
      RETURN(NPF_OUT_OF_MEMORY); 
   }
   
   /* Fill in the Callback Information in the array before registration */
   if ( (retVal = ipv4_FillCBInfoToFpp(fppInfo)) != NPF_SUCCESS)
   {
      npf_logger_Write(LOG_IPV4, LOG_ERR, "Unable to fill IPV4 Callback Info");
      IPV4_UNLOCK()
      RETURN(retVal);
   }

   /* For each FPP CB Info node, register with FP and get the CB Handles */
    for (cbCount=0 ; cbCount<fppInfo->numFpCB; cbCount++)
    {
       if((retVal = FPPAPI_register_callback( 
			    fppInfo->cbInfoArray[cbCount].cbFunc, 
			    &fppInfo->cbInfoArray[cbCount].cbHndl, 
			    fppInfo->cbInfoArray[cbCount].cbType, 
           (NPF_USERCONTEXT)&fppInfo->cbInfoArray/* Pass this as FPP context */
			    ))!= FPPI_SUCCESS)
       {
         npf_logger_Write(LOG_IPV4, LOG_ERR, "Unable to register with FPP ");
         IPV4_UNLOCK()
         RETURN(NPF_E_UNKNOWN);
       }
    }

    /* Register for event callbacks triggered when a prefix or
     * next hop table miss occurs, incomplete next hop
     * or route refresh is triggered
     */
   
#ifdef IPV4_EVENT_HANDLING
    /* TBD : We haven't decided the functions which are going to
             handle events as there is no IPV4 Event Handling done.
             SDK doens't raise any NPF Defined events */
    pdk_if_event_register( (NPF_USERCONTEXT)&ipv4Mgr.fppInfo, 
		    (NPF_GENERAL_CBFUNC)ipv4Mgr.fppInfo.cbInfoArray[cbCount].cbFunc, 
		    (NPF_CBHANDLE *)&ipv4Mgr.fppInfo.cbInfoArray[cbCount].cbHndl);
    
    pdk_fe_event_register( (NPF_USERCONTEXT)&ipv4Mgr.fppInfo, 
		    (NPF_GENERAL_CBFUNC)ipv4Mgr.fppInfo.cbInfoArray[cbCount++].cbFunc, 
		    (NPF_CBHANDLE *)&ipv4Mgr.fppInfo.cbInfoArray[cbCount].cbHndl);
#endif /*IPV4_EVENT_HANDLING */

   IPV4_UNLOCK()
   RETURN(NPF_NO_ERROR);
    
} /* ipv4_InitFppInfo */


/*****************************************************************************
*    Name: ipv4_RemoveFppInfo
*
*    Description:
*    	Destroys IPV4 Mgr. FPP Information. All the de-registartions done with
*    	the FPP module are done in this function. This func. is called as a
*    	part of IPV4 Shutdown process.
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_RemoveFppInfo() 
{
   NPF_RET retVal;
   IPV4FppInfo_t *fppInfo; 
   NPF_uint32_t cbCount =0;
   
   IPV4_DBG_TRACE();
   fppInfo = &ipv4Mgr.fppInfo;
   assert (fppInfo);
   
   /*Req. to FP List */
   npf_list_destroy(&fppInfo->reqFpList);  
   /* For each FPP CB Info node, register with FP and get the CB Handles */
    for (cbCount=0; cbCount<fppInfo->numFpCB; cbCount++)
    {
       if(FPPAPI_deregister_callback( 
			    fppInfo->cbInfoArray[cbCount].cbHndl 
			    )!= FPPI_SUCCESS)
       {
            retVal = NPF_E_UNKNOWN;
       }
    }

#ifdef IPV4_EVENT_SUPPORT
   /* De-Register IF Event CB Handle */
   pdk_if_event_deregister(fppInfo->cbInfoArray[cbCount].cbHndl);
   /* De-Register FE Event CB Handle */
   pdk_fe_event_deregister(fppInfo->cbInfoArray[++cbCount].cbHndl);
#endif /*IPV4_EVENT_SUPPORT*/

   /* De-Alloate memory for the all the Callbacks to FP */
   PIL_FreeMemory(fppInfo->cbInfoArray);
   RETURN(NPF_NO_ERROR);
} /* ipv4_RemoveFppInfo*/


/*
   ############################################################################
   IPV4 Mgr Module Helper Functions
   ############################################################################
*/


/*****************************************************************************
*    Name: ipv4_AddInUserListToMgr
*
*    Description:
*       Adds to User List in Ipv4 Mgr struc.
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_AddInUserListToMgr(IPV4UserInfo_t *userInfo) 
{
   IPV4_DBG_TRACE();
   IPV4_LOCK();
   
   assert(userInfo);
   /* Since SDK Doesn't support more than one User/FIB/ROUTER, hence we 
      will reject any User Registration which is more than ONE here */
   if(ipv4Mgr.userList.size > 1)
   {
      npf_logger_Write(LOG_IPV4, LOG_ERR,
           "More than ONE User registration not allowed !! ");
      IPV4_UNLOCK();    
      RETURN(NPF_E_UNKNOWN);
   }
   npf_list_pushBack(&ipv4Mgr.userList, (const void *)userInfo);
   IPV4_UNLOCK();
   RETURN(NPF_SUCCESS);

} /* ipv4_AddInUserListToMgr*/


/*****************************************************************************
*    Name: ipv4_DelInUserListFrmMgr
*
*    Description:
*       Delete from User List in Ipv4 Mgr struc.
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_DelInUserListFrmMgr(NPF_callbackHandle_t cbHndl) 
{
   IPV4UserInfo_t *userInfo;
   
   IPV4_DBG_TRACE();

   IPV4_LOCK();
    
   /* Search in the list with cbHndl as key */ 
   if(npf_list_removeByKey(&ipv4Mgr.userList, 
		    (const void *)&cbHndl, 
		    (void **)&userInfo) == -1)
   {
      npf_logger_Write(LOG_IPV4, LOG_ERR,
                        "Error finding User Info");
      IPV4_UNLOCK();    
      RETURN(NPF_E_UNKNOWN);
   }
    
   /* Free the memory for this user node */ 
   if(ipv4_RemoveUserInfo(userInfo) != NPF_SUCCESS )
   {
      IPV4_UNLOCK();
      RETURN(NPF_E_UNKNOWN)
   }
   
  IPV4_UNLOCK();
   
  RETURN(NPF_SUCCESS);

} /* ipv4_DelInUserListFrmMgr*/



/*****************************************************************************
*    Name: ipv4_GetHandleFrmMgr
*
*    Description:
*    	Get Handle Count from IPV4 Mgr. 
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_GetHandleFrmMgr(NPF_uint32_t *hndl) 
{
   IPV4_DBG_TRACE();

   IPV4_LOCK();
   
   *hndl = ipv4Mgr.hndlCntr++;

   IPV4_UNLOCK();
   
   RETURN(NPF_SUCCESS);

} /* ipv4_GetHandleFrmMgr */


/*****************************************************************************
*    Name: ipv4_GetUserByHandleFrmMgr 
*
*    Description:
*    	Search User Information based on cbhandle 
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If search is successful 
*             NPF_E_UNKNOWN : If search is failure 
*/

NPF_RET ipv4_GetUserByHandleFrmMgr(
	NPF_callbackHandle_t cbHndl,
	IPV4UserInfo_t **userInfo
        ) 
{
   IPV4_DBG_TRACE();
   
   IPV4_LOCK();

   if ((*userInfo = npf_list_searchByKey(&ipv4Mgr.userList,
				   (const void *)&cbHndl)) == NULL)
   {
        npf_logger_Write(LOG_IPV4, LOG_TRACE, "No callback handle found");
        IPV4_UNLOCK();
        RETURN(NPF_E_BAD_CALLBACK_HANDLE);
   }
  
   IPV4_UNLOCK();
   RETURN(NPF_SUCCESS);
} /* ipv4_GetUserByHandleFrmMgr*/


/*****************************************************************************
*    Name: ipv4_InitIPV4Mgr 
*
*    Description:
*    	Initialize IPV4 Mgr. Information.
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_InitIPV4Mgr() 
{
   NPF_RET retVal;
   
   IPV4_DBG_TRACE();
   
   /* IPV4 Module - Initialization Starts */ 
   if( ipv4Mgr.initialized == TRUE)
   {
      npf_logger_Write(LOG_IPV4, LOG_TRACE, "IPv4 Mgr. is already initialized ");
      RETURN(NPF_SUCCESS);
   }
 
   PIL_InitializeCriticalSection( &ipv4Mgr.lock);
  
   /* Initialize the callback handle counter */
    ipv4Mgr.hndlCntr = 0; 
    
    /* Initialize the Lists, each having destroy func. as param. */
    npf_list_init(&ipv4Mgr.userList, ipv4_FreeUserInfo);          /* Fn. User List */
#ifdef IPV4_EVENT_SUPPORT
    npf_list_init(&ipv4Mgr.eventUserList, ipv4_FreeEventUserInfo);/* Event User List */
#endif /* IPV4_EVENT_SUPPORT*/

    /* Compare function for each list */ 
    ipv4Mgr.userList.compare = ipv4_CompareUserList; /* List of Fn. users */
#ifdef IPV4_EVENT_SUPPORT
    ipv4Mgr.eventUserList.compare = ipv4_CompareEventUserList;/*List of Event users*/
#endif /* IPV4_EVENT_SUPPORT*/
   /* Initalize FPP Info module of IPV4 Mgr */
   if ((retVal=ipv4_InitFppInfo()) != NPF_SUCCESS)
   {
      npf_logger_Write(LOG_IPV4, LOG_ERR, "IPv4 FPP Context not initialized ");
      RETURN(retVal);
   }

   /* IPV4 Module - Initialization Complete */ 
   ipv4Mgr.initialized = TRUE;    
   RETURN(NPF_NO_ERROR);
}/* ipv4_InitIPV4Mgr */ 

/*****************************************************************************
*    Name: ipv4_ShutdownIPV4Mgr 
*
*    Description:
*    	Called in case of shutdown of IPV4 Mgr.
*
*    Parameters:
*    	IN: 
*    	OUT:
*
*    Returns: NPF_SUCCESS: If done properly 
*             NPF_E_UNKNOWN : If not done properly 
*/

NPF_RET ipv4_ShutdownIPV4Mgr() 
{
   NPF_RET retVal;
   
   IPV4_DBG_TRACE();
   
   if (! ipv4Mgr.initialized ) 
   {
      npf_logger_Write(LOG_IPV4, LOG_ERR, "IPv4 module not initalized");
      RETURN(NPF_E_UNKNOWN);
   }
    
    /* Destroy the Lists, each having destroy func. as param. */
    npf_list_destroy(&ipv4Mgr.userList);  /* Fn. User List */
#ifdef IPV4_EVENT_SUPPORT
    npf_list_destroy(&ipv4Mgr.eventUserList);/* Event User List */
#endif /* IPV4_EVENT_SUPPORT*/

   /* Remove FPP Info module of IPV4 Mgr */
   if ((retVal=ipv4_RemoveFppInfo()) == NPF_SUCCESS)
      RETURN(retVal);
   /* IPV4 Module - Initialization */ 
   ipv4Mgr.initialized = FALSE;    
   /* Finallyu, destroy the global lock */ 
   PIL_DestroyCriticalSection(&ipv4Mgr.lock);

   RETURN(NPF_NO_ERROR);
}/* ipv4_ShutdownIPV4Mgr */ 
