/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/


/*****************************************************************************
* $Source:
* /var/cvs/npfpdk/CODEBASE/ControlPlane/APIImplementations/IPv4/inc/ipv4_send_rsv_fp.c,v $
* $Revision: 1.8 $
* $Date: 2003/11/17 16:57:11 $
*
* Description:
*	Implementation of all the FSMs in IPv4 are done in this file.
*
*****************************************************************************/

/* Standard C Libraries */
#include <string.h>
#include <assert.h>
#include <stdio.h>

/* FPP libraries */
#include "fpp_common.h"
#include "fppapi.h"

/* Helper libraries */
#include "log.h"
#include "list.h"
#include "pil.h"

/* Helper PDK Framework header files */
#include "pdk_callback.h"
#include "pdk_ns_helper.h"
#include "npf_common.h"
#include "pdk_cm.h"

/* Files written as a part of IPv4 Component */
#include "npf_ipv4_types.h"
#include "npf_ipv4_api.h"
#include "npf_ns_api.h"
#include "ipv4_types.h"
#include "ipv4_macros.h"
#include "ipv4_modules.h"
#include "ipv4_req_handler.h"

/* Include other Modules that are to be called */
#include "pdkmgr.h"
#include "pdk_l2idm.h"

extern NPF_error_t IfGetFEIdsFrmFIBHdl(NPF_HANDLE fib_npf_hdl, uint32_t *numFe, FPPI_FEID *feId);
extern IPV4Mgr_t	ipv4Mgr;

#define MAX_NUM_NHOP 10

static char* tableName[] = {"Prefix", "NextHop", "AddRes", "Fib"};

/******************************************************************
 * Requests Handling Section
 * *****************************************************************/

/*****************************************************************************
*    Name: ipv4_InvokeUserReqHandler
*
*    Description:
*    Parameters:
*        		 trigger into this state machine. 
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*       NPF_RET : NPF_SUCCESS 
*                 NPF_FAIL    
*
*/
NPF_RET ipv4_InvokeUserReqHandler(
        	IPV4UsrReqInfo_t *usrReq
		)
{
   NPF_RET retVal;
   IPV4UserInfo_t  *userInfo;
   IPV4TableInfo_t *tableInfo;
   IPV4TransInfo_t *transInfo;

   IPV4_DBG_TRACE();
   
   assert(usrReq);
   
   switch (usrReq->reqType)
   {

      case NPF_IPV4UC_PREFIX_TABLE_HANDLE_CREATE: 
      case NPF_IPV4UC_NEXT_HOP_TABLE_HANDLE_CREATE: 
      case NPF_IPV4UC_ADDRESS_RES_TABLE_HANDLE_CREATE: 
	     /** Common Processing for above requests **/
         {
            /* Allocate memory for User Request and intialize it */
            if ((retVal = ipv4_ProcessReqFrmUsr(usrReq,&userInfo,
					       &tableInfo)) != NPF_SUCCESS)
            {
               npf_logger_Write(LOG_IPV4, LOG_ERR,
                    "Error : Error in Processing request from user");
               RETURN(retVal);
            }
           
            if (tableInfo->state != TABLE_EMPTY && 
                tableInfo->state != TABLE_ENTRIES_EXIST && 
		          tableInfo->state != TABLE_FULL
		         )
	         {
	            /* Allocate memory for Transaction and intialize it */
                if ((retVal=ipv4_InitTransInfo(usrReq,userInfo,tableInfo,
					   &transInfo)) != NPF_SUCCESS)
                {
                    npf_logger_Write(LOG_IPV4, LOG_ERR,
		                  " Initialization of Transaction Failed");
                    RETURN(retVal);
                }
                assert(transInfo); /* safe check */ 

	             /* This request is not sent to FP. It is sent to NS. No
         	     * Need to create a transaction in this case.*/
                if((retVal = ipv4_AddTableToUsr(userInfo,usrReq->tableType))
	    	                 != NPF_SUCCESS)
	             {
                    npf_logger_Write(LOG_IPV4, LOG_ERR,
	   		        "Error: %s Table was not created", tableName[usrReq->tableType]);
	                  RETURN(retVal);
	             }
               
	             npf_logger_Write(LOG_IPV4, LOG_TRACE,
	   		        " %s Table with handle %d was created successfully"
                    , tableName[usrReq->tableType],tableInfo->intHandle);
                tableInfo->state = TABLE_EMPTY;
	         }
	         else
	         {
               npf_logger_Write(LOG_IPV4, LOG_ERR,
		           "Error : Request has come in invalid table state");
	            RETURN(NPF_E_UNKNOWN);
	         }
         }
	   break;

      case NPF_IPV4UC_PREFIX_TABLE_HANDLE_DELETE: 
      case NPF_IPV4UC_NEXT_HOP_TABLE_HANDLE_DELETE: 
      case NPF_IPV4UC_ADDRESS_RES_TABLE_HANDLE_DELETE: 
	 /** Common Processing for above requests **/
         {
            if ((retVal = ipv4_ProcessReqFrmUsr(usrReq,&userInfo,
	 			       &tableInfo)) != NPF_SUCCESS)
            {
                npf_logger_Write(LOG_IPV4, LOG_ERR,
                        "Error : Error in Processing request from user");
                RETURN(retVal);
            }
            if (tableInfo->state != TABLE_DELETED)
	    {
                /* Allocate memory for Transaction and intialize it */
               if ((retVal=ipv4_InitTransInfo(usrReq,userInfo,tableInfo,
					   &transInfo)) != NPF_SUCCESS)
               {
                  npf_logger_Write(LOG_IPV4, LOG_ERR,
	 	                " Initialization of Transaction Failed");
                  RETURN(retVal);
               }
               assert(transInfo); /* safe check */ 


	        /* This request is not sent to FP. It is sent to NS. No
	  	      * Need to create a transaction in this case.*/
               
	       if((retVal = ipv4_RemTableFrmUsr(userInfo,usrReq->tableType))
	                          != NPF_SUCCESS)
	       {
                  npf_logger_Write(LOG_IPV4, LOG_ERR,
	   		   "Error: %s Table was not deleted ", tableName[usrReq->tableType]);
		  RETURN(retVal);
	       }
               
	       npf_logger_Write(LOG_IPV4, LOG_TRACE,
	   		        " %s Table with handle %d was deleted successfully", tableName[usrReq->tableType],usrReq->tblHandle);
               tableInfo->state = TABLE_DELETED;
	       
	       /* Now, free the context for this table so that it can be
		* reused */
	       ipv4_DelTblCtxtFrmUsr(userInfo, usrReq->tableType);

	       /* NOTE: Since the Table struct is defined as array element in
		* User Info struct. Hence, it will be remvoed with UserInfo
		* when User de-registers itself
		* */
	    }
	    else
	    {
               npf_logger_Write(LOG_IPV4, LOG_ERR,
		       "Error : Request has come in invalid table state");
	       RETURN(NPF_E_UNKNOWN);
	     }
      }
	   break;

      case NPF_IPV4UC_PREFIX_NEXT_HOP_TABLE_BIND: 
         {
             IPV4TableInfo_t *nhopTableInfo = NULL;

             /* Common Processing for all the requests */
             if ((retVal = ipv4_ProcessReqFrmUsr(usrReq,&userInfo,
	 			       &tableInfo)) != NPF_SUCCESS)
             {
                 npf_logger_Write(LOG_IPV4, LOG_ERR,
                     "Error : Error in Processing request from user");
                 RETURN(retVal);
             }
             /* Allocate memory for Transaction and intialize it */
             if ((retVal=ipv4_InitTransInfo(usrReq,userInfo,tableInfo,
					   &transInfo)) != NPF_SUCCESS)
             {
                 npf_logger_Write(LOG_IPV4, LOG_ERR,
	 	                " Initialization of Transaction Failed");
                  RETURN(retVal);
             }
             assert(transInfo); /* safe check */ 

             /* Special processing for this request; Need to find NextHop Table also */
             if((retVal=ipv4_GetTableByHndlFrmUsr(userInfo, IPV4_TABLE_TYPE_NHOP, 
                         usrReq->bndTblHandle, &nhopTableInfo)) != NPF_SUCCESS)
               {
                  npf_logger_Write(LOG_IPV4, LOG_ERR, 
                      "Target Table [handle = %d] not found ", usrReq->bndTblHandle);
  	          RETURN(retVal);
               }
            /* We should be able to bind tables in any state */
            /* Set the External Handle same as Prefix */ 
            nhopTableInfo->extHandle = tableInfo->extHandle;
            /* Store the FE Intial FE Information for Nhop Table */ 
            if((retVal = IfGetFEIdsFrmFIBHdl(nhopTableInfo->extHandle,&nhopTableInfo->numFeTotal, nhopTableInfo->feIdArray)) != NPF_SUCCESS) 
            {
                 npf_logger_Write(LOG_IPV4, LOG_ERR,
                          "Unable to get FE Ids associated with FIB Handle from Namespace ");
                 RETURN(retVal);
            }
         }
         break;
      
      case NPF_IPV4UC_PREFIX_ENTRY_ADD: 
      case NPF_IPV4UC_NEXT_HOP_ENTRY_ADD: 
      case NPF_IPV4UC_ADDRESS_RES_ENTRY_ADD: 
	 /* Common Processing for above requests */
         {
            if ((retVal = ipv4_ProcessReqFrmUsr(usrReq,&userInfo,
	 	       &tableInfo)) != NPF_SUCCESS)
            {
                npf_logger_Write(LOG_IPV4, LOG_ERR,
                     "Error : Error in Processing request from user");
                RETURN(retVal);
            }
            if( tableInfo->state != TABLE_FULL && 
	                  tableInfo->state != TABLE_DELETED
	        )
	    {
               /* Allocate memory for Transaction and intialize it */
               if ((retVal=ipv4_InitTransInfo(usrReq,userInfo,tableInfo,
					   &transInfo)) != NPF_SUCCESS)
               {
                  npf_logger_Write(LOG_IPV4, LOG_ERR,
		         " Initialization of Transaction Failed");
                  RETURN(retVal);
               }
               assert(transInfo); /* safe check */ 

               /* Process request to be sent to FP */
               if((retVal=ipv4_ProcessReqToFPP(usrReq)) != NPF_SUCCESS)
	            {
                       npf_logger_Write(LOG_IPV4, LOG_ERR,
		             "Error in Processing request to FP ");
	               RETURN(retVal);
	            }

               tableInfo->state = TABLE_ENTRIES_EXIST;
	    }
	    else
	    {
                  npf_logger_Write(LOG_IPV4, LOG_ERR,
		       "Error : Request has come in invalid table state");
	          RETURN(NPF_E_UNKNOWN);
	       }
       }
	    break;
      
      case NPF_IPV4UC_PREFIX_ENTRY_DELETE: 
      case NPF_IPV4UC_NEXT_HOP_ENTRY_DELETE: 
      case NPF_IPV4UC_ADDRESS_RES_ENTRY_DELETE: 
	 /* Common Processing for above requests */
         {
            if ((retVal = ipv4_ProcessReqFrmUsr(usrReq,&userInfo,
	 			       &tableInfo)) != NPF_SUCCESS)
            {
                npf_logger_Write(LOG_IPV4, LOG_ERR,
                     "Error : Error in Processing request from user");
                RETURN(retVal);
            }

            if (((tableInfo->state != TABLE_EMPTY) && 
                (tableInfo->state != TABLE_DELETED)) 
#ifdef IPV4_UNIF_SUPPORT
              || (userInfo->usrType == IPV4_USR_UNIF)
#endif
	           )
	           {
                 /* Allocate memory for Transaction and intialize it */
                 if ((retVal=ipv4_InitTransInfo(usrReq,userInfo,
                              tableInfo, &transInfo)) != NPF_SUCCESS)
                 {
                    npf_logger_Write(LOG_IPV4, LOG_ERR,
		                " Initialization of Transaction Failed");
                    RETURN(retVal);
                 }
                 assert(transInfo); /* safe check */ 
               
	              if((retVal=ipv4_ProcessReqToFPP(usrReq)) 
                                                 != NPF_SUCCESS)
	              {
                   npf_logger_Write(LOG_IPV4, LOG_ERR,
		             "Error in Processing request to FP ");
	                RETURN(retVal);
	              }

    	        }
	           else
	           {
                 npf_logger_Write(LOG_IPV4, LOG_ERR,
		               "Error : Request has come in invalid table state");
	              RETURN(NPF_E_UNKNOWN);
	           }
         }
	      break;
     
      case NPF_IPV4UC_PREFIX_TABLE_FLUSH: 
      case NPF_IPV4UC_NEXT_HOP_TABLE_FLUSH: 
      case NPF_IPV4UC_ADDRESS_RES_TABLE_FLUSH: 
	   /* Common Processing for above requests */
         {
            if ((retVal = ipv4_ProcessReqFrmUsr(usrReq,&userInfo,
	                         &tableInfo)) != NPF_SUCCESS)
            {
                npf_logger_Write(LOG_IPV4, LOG_ERR,
                     "Error : Error in Processing request from user");
                RETURN(retVal);
            }
            if( tableInfo->state == TABLE_ENTRIES_EXIST ||
                tableInfo->state == TABLE_FULL )
	         {
               /* Allocate memory for Transaction and intialize it */
               if ((retVal=ipv4_InitTransInfo(usrReq,userInfo,tableInfo,
					   &transInfo)) != NPF_SUCCESS)
               {
                  npf_logger_Write(LOG_IPV4, LOG_ERR,
		         " Initialization of Transaction Failed");
                  RETURN(retVal);
               }
               assert(transInfo); /* safe check */ 
               
	            if((retVal=ipv4_ProcessReqToFPP(usrReq)) != NPF_SUCCESS)
	            {
                   npf_logger_Write(LOG_IPV4, LOG_ERR,
		               "Error in Processing request to FP ");
	                RETURN(retVal);
	            }

               tableInfo->state = TABLE_EMPTY;
       	   }
	         else
	         {
               npf_logger_Write(LOG_IPV4, LOG_ERR,
		            "Error : Request has come in invalid table state");
	            RETURN(NPF_E_UNKNOWN);
	         }
         }
	      break;
      case NPF_IPV4UC_PREFIX_TABLE_ATTRIBUTE_QUERY: 
      case NPF_IPV4UC_NEXT_HOP_TABLE_ATTRIBUTE_QUERY: 
#ifdef IPV4_UNIF_SUPPORT
      case NPF_IPV4UC_FIB_TABLE_ATTRIBUTE_QUERY: 
#endif  /* IPV4_UNIF_SUPPORT */
      case NPF_IPV4UC_ADDRESS_RES_TABLE_ATTRIBUTE_QUERY: 
      case NPF_IPV4UC_ADDRESS_RES_ENTRY_QUERY: 
	   /* Common Processing for above requests */
         {
	         /* In SDK 3.1, this routine is not supported */
            npf_logger_Write(LOG_IPV4, LOG_ERR,
                     "Info : SDK does not support this API");
	         RETURN(NPF_IPV4UC_FUNCTION_NOT_SUPPORTED);
         }
	      break;

      case NPF_IPV4UC_PREFIX_ENTRY_QUERY: 
      case NPF_IPV4UC_NEXT_HOP_ENTRY_QUERY: 
	 /* Common Processing for above requests */
         {
            if ((retVal = ipv4_ProcessReqFrmUsr(usrReq,&userInfo,
	 			       &tableInfo)) != NPF_SUCCESS)
            {
                npf_logger_Write(LOG_IPV4, LOG_ERR,
                     "Error : Error in Processing request from user");
                RETURN(retVal);
            }
	         if ((tableInfo->state == TABLE_ENTRIES_EXIST) ||
		                  (tableInfo->state == TABLE_FULL) 
#ifdef IPV4_UNIF_SUPPORT
                        || (userInfo->usrType == IPV4_USR_UNIF)
#endif
	            )
	         {
               /*NOTE: Since we have to process request to FP, hence we
                  need to create Transaction ( gluing logic between 
                  CP and FP req)along with the n FP requests 
                  ( n = numFEs) */
               
               /* Allocate memory for Transaction and intialize it */
               if ((retVal=ipv4_InitTransInfo(usrReq,userInfo,tableInfo,
					   &transInfo)) != NPF_SUCCESS)
               {
                  npf_logger_Write(LOG_IPV4, LOG_ERR,
		         " Initialization of Transaction Failed");
                  RETURN(retVal);
               }
               assert(transInfo); /* safe check */ 
               if((retVal=ipv4_ProcessReqToFPP(usrReq)) != NPF_SUCCESS)
	            {
                  npf_logger_Write(LOG_IPV4, LOG_ERR,
		                        "Error in Processing request to FP ");
	               RETURN(retVal);
	            }
	         }
	         else
	         {
               npf_logger_Write(LOG_IPV4, LOG_ERR,
		             "Error : Request has come in invalid table state");
	            RETURN(NPF_E_UNKNOWN);
	         }
         }
	      break;
#ifdef IPV4_UNIF_SUPPORT
      /********************************************************
       * ************* UNIFIED API Handling Section **********
       ********************************************************/
        case NPF_IPV4UC_FIB_TABLE_HANDLE_CREATE: 
         {
            IPV4TransInfo_t *dstTransInfo;

            /* Allocate memory for User Request and intialize it */
            if ((retVal = ipv4_ProcessReqFrmUsr(usrReq,&userInfo,
					       &tableInfo)) != NPF_SUCCESS)
            {
               npf_logger_Write(LOG_IPV4, LOG_ERR,
                    "Error : Error in Processing request from user");
               RETURN(retVal);
            }
           
            if (tableInfo->state != TABLE_EMPTY ||
                tableInfo->state != TABLE_ENTRIES_EXIST || 
		          tableInfo->state != TABLE_FULL
		         )
	         {
	            /* Allocate memory for Transaction and intialize it */
               /* TODO: Is the transaction added to the list maintained by 
                  UserInfo ??? */
               if ((retVal=ipv4_InitTransInfo(usrReq,userInfo,tableInfo,
					   &transInfo)) != NPF_SUCCESS)
               {
                  npf_logger_Write(LOG_IPV4, LOG_ERR,
		                  " Initialization of Transaction Failed");
                  RETURN(retVal);
               }
               assert(transInfo); /* safe check */
               usrReq->transInfo->mapTransInfo->mapTransArray[usrReq->transInfo->mapTransInfo->numMapTrans] = usrReq->transInfo;
               usrReq->transInfo->mapTransInfo->numMapTrans++; 

	           /* This request is not sent to FP. It is sent to NS. No
         	   * Need to create a transaction in this case.*/
             /*  if((retVal = ipv4_AddTableToUsr(userInfo,usrReq->tableType))
	    	                 != NPF_SUCCESS)
	            {
                   npf_logger_Write(LOG_IPV4, LOG_ERR,
	   		        "Error: Table of type %d not created", usrReq->tableType);
	               RETURN(retVal);
	            }
               
	            npf_logger_Write(LOG_IPV4, LOG_TRACE,
	                   "Table of type %d created successfuly", usrReq->tableType);
               tableInfo->state = TABLE_EMPTY; */
	         }
	         else
	         {
                npf_logger_Write(LOG_IPV4, LOG_ERR,
		           "Error : Request has come in invalid table state");
	             RETURN(retVal);
	         }

            if ((retVal=ipv4_DupTransInfo(usrReq->transInfo,
			      &dstTransInfo)) != NPF_SUCCESS)
            {
               npf_logger_Write(LOG_IPV4, LOG_ERR,
		               " Duplication of Transaction Failed");
               RETURN(retVal);
            }
            assert(dstTransInfo); /* safe check */
            npf_list_pushBack(&userInfo->unifiedTransCpList, dstTransInfo); 

            retVal = NPF_IPv4UC_NextHopTableHandleCreate( 
                     usrReq->callbackHandle, 
                     usrReq->transInfo->int_correlator, NPF_REPORT_ALL);
            if(retVal != NPF_SUCCESS)
            {
                  npf_logger_Write(LOG_IPV4, LOG_ERR,
		                  "Nhop Table Creation Failed");
                  RETURN(retVal);
            }

            retVal = NPF_IPv4UC_PrefixTableHandleCreate(
                     usrReq->callbackHandle, dstTransInfo->int_correlator,
                     NPF_REPORT_ALL);
            if(retVal != NPF_SUCCESS)
            {
               npf_logger_Write(LOG_IPV4, LOG_ERR,
		            "Prefix Table Creation Failed");
               RETURN(retVal);
            }
         }
         break;

      case NPF_IPV4UC_FIB_TABLE_HANDLE_DELETE: 
         {
            IPV4TransInfo_t *dstTransInfo;

            /* Allocate memory for User Request and intialize it */
            if ((retVal = ipv4_ProcessReqFrmUsr(usrReq,&userInfo,
					       &tableInfo)) != NPF_SUCCESS)
            {
               npf_logger_Write(LOG_IPV4, LOG_ERR,
                    "Error : Error in Processing request from user");
               RETURN(retVal);
            }
   
            if (tableInfo->state != TABLE_DELETED)
            {
               /* Allocate memory for Transaction and intialize it */
               if ((retVal=ipv4_InitTransInfo(usrReq,userInfo,tableInfo,
                                           &transInfo)) != NPF_SUCCESS)
               {
                  npf_logger_Write(LOG_IPV4, LOG_ERR,
                                " Initialization of Transaction Failed");
                  RETURN(retVal);
               }
               assert(transInfo); /* safe check */
               usrReq->transInfo->mapTransInfo->mapTransArray[usrReq->transInfo->mapTransInfo->numMapTrans] = usrReq->transInfo;
               usrReq->transInfo->mapTransInfo->numMapTrans++; 
               if ((retVal=ipv4_DupTransInfo(usrReq->transInfo,
			      &dstTransInfo)) != NPF_SUCCESS)
               {
                  npf_logger_Write(LOG_IPV4, LOG_ERR,
		               " Initialization of Transaction Failed");
                  RETURN(retVal);
               }
               assert(dstTransInfo); /* safe check */ 
               npf_list_pushBack(&userInfo->unifiedTransCpList, dstTransInfo); 

               retVal = NPF_IPv4UC_PrefixTableHandleDelete( 
                     usrReq->callbackHandle, 
                     usrReq->transInfo->int_correlator,
                     NPF_REPORT_ALL, 
                     usrReq->transInfo->usrInfo->tableArray[IPV4_TABLE_TYPE_PREFIX].intHandle);

               retVal = NPF_IPv4UC_NextHopTableHandleDelete(
                        usrReq->callbackHandle, 
                        dstTransInfo->int_correlator, NPF_REPORT_ALL, 
                        dstTransInfo->usrInfo->tableArray[IPV4_TABLE_TYPE_NHOP].intHandle); 
               if((retVal = ipv4_RemTableFrmUsr(userInfo,usrReq->tableType)) 
                  != NPF_SUCCESS)
               {
                  npf_logger_Write(LOG_IPV4, LOG_ERR,
                      "Error: %s Table was not deleted ", tableName[usrReq->tableType]);
                  RETURN(retVal);
               }
               npf_logger_Write(LOG_IPV4, LOG_TRACE,
                  " %s Table with handle %d was deleted successfully", tableName[usrReq->tableType],usrReq->tblHandle);
               tableInfo->state = TABLE_DELETED; 
               ipv4_DelTblCtxtFrmUsr(userInfo, usrReq->tableType);
            }
            else
            {
               npf_logger_Write(LOG_IPV4, LOG_ERR,
                       "Error : Request has come in invalid table state");
	            RETURN(NPF_E_UNKNOWN);
            }
         }
         break;

      case NPF_IPV4UC_FIB_ENTRY_ADD: 
         {
            if ((retVal = ipv4_ProcessReqFrmUsr(usrReq,&userInfo,
	 	       &tableInfo)) != NPF_SUCCESS)
            {
                npf_logger_Write(LOG_IPV4, LOG_ERR,
                     "Error : Error in Processing request from user");
                RETURN(retVal);
            }

            if( tableInfo->state != TABLE_FULL ||
	             tableInfo->state != TABLE_DELETED )
	         {
               /* Allocate memory for Transaction and intialize it */
               if ((retVal=ipv4_InitTransInfo(usrReq,userInfo,tableInfo,
					   &transInfo)) != NPF_SUCCESS)
               {
                  npf_logger_Write(LOG_IPV4, LOG_ERR,
		         " Initialization of Transaction Failed");
                  RETURN(retVal);
               }
               assert(transInfo); /* safe check */ 
               usrReq->transInfo->mapTransInfo->mapTransArray[usrReq->transInfo->mapTransInfo->numMapTrans] = usrReq->transInfo;
               usrReq->transInfo->mapTransInfo->numMapTrans++; 
         
               tableInfo->state = TABLE_ENTRIES_EXIST;
	         }
	         else
	         {
                npf_logger_Write(LOG_IPV4, LOG_ERR,
		             "Error : Request has come in invalid table state");
	             RETURN(NPF_E_UNKNOWN);
	         }
       }
	    break;

      case NPF_IPV4UC_FIB_ENTRY_DELETE: 
         {
            if ((retVal = ipv4_ProcessReqFrmUsr(usrReq,&userInfo,
	 	       &tableInfo)) != NPF_SUCCESS)
            {
                npf_logger_Write(LOG_IPV4, LOG_ERR,
                     "Error : Error in Processing request from user");
                RETURN(retVal);
            }

            if( tableInfo->state != TABLE_EMPTY ||
	             tableInfo->state != TABLE_DELETED )
	         {
               /* Allocate memory for Transaction and intialize it */
               if ((retVal=ipv4_InitTransInfo(usrReq,userInfo,tableInfo,
					   &transInfo)) != NPF_SUCCESS)
               {
                  npf_logger_Write(LOG_IPV4, LOG_ERR,
		         " Initialization of Transaction Failed");
                  RETURN(retVal);
               }
               assert(transInfo); /* safe check */ 
               usrReq->transInfo->mapTransInfo->mapTransArray[usrReq->transInfo->mapTransInfo->numMapTrans] = usrReq->transInfo;
               usrReq->transInfo->mapTransInfo->numMapTrans++; 
	         }
	         else
	         {
                npf_logger_Write(LOG_IPV4, LOG_ERR,
		             "Error : Request has come in invalid table state");
	             RETURN(NPF_E_UNKNOWN);
	         }
         }
         break;

      case NPF_IPV4UC_FIB_ENTRY_QUERY: 
         {
            if ((retVal = ipv4_ProcessReqFrmUsr(usrReq,&userInfo,
	 	       &tableInfo)) != NPF_SUCCESS)
            {
                npf_logger_Write(LOG_IPV4, LOG_ERR,
                     "Error : Error in Processing request from user");
                RETURN(retVal);
            }

            if((tableInfo->state != TABLE_EMPTY) && 
	            (tableInfo->state != TABLE_DELETED))
	         {
               /* Allocate memory for Transaction and intialize it */
               if ((retVal=ipv4_InitTransInfo(usrReq,userInfo,tableInfo,
					   &transInfo)) != NPF_SUCCESS)
               {
                  npf_logger_Write(LOG_IPV4, LOG_ERR,
		         " Initialization of Transaction Failed");
                  RETURN(retVal);
               }
               assert(transInfo); /* safe check */ 
               usrReq->transInfo->mapTransInfo->mapTransArray[usrReq->transInfo->mapTransInfo->numMapTrans] = usrReq->transInfo;
               usrReq->transInfo->mapTransInfo->numMapTrans++; 
	         }
	         else
	         {
                npf_logger_Write(LOG_IPV4, LOG_ERR,
		             "Error : Request has come in invalid table state");
	             RETURN(NPF_E_UNKNOWN);
	         }
         }
         break;
      case NPF_IPV4UC_FIB_TABLE_FLUSH: 
         {
            IPV4TransInfo_t *dstTransInfo;

            if ((retVal = ipv4_ProcessReqFrmUsr(usrReq,&userInfo,
					       &tableInfo)) != NPF_SUCCESS)
            {
               npf_logger_Write(LOG_IPV4, LOG_ERR,
                    "Error : Error in Processing request from user");
               RETURN(retVal);
            }
            if( tableInfo->state == TABLE_ENTRIES_EXIST ||
                tableInfo->state == TABLE_FULL )
	    {
               /* Allocate memory for Transaction and intialize it */
               if ((retVal=ipv4_InitTransInfo(usrReq,userInfo,tableInfo,
					   &transInfo)) != NPF_SUCCESS)
               {
                  npf_logger_Write(LOG_IPV4, LOG_ERR,
		         " Initialization of Transaction Failed");
                  RETURN(retVal);
               }
               assert(transInfo); /* safe check */ 
               usrReq->transInfo->mapTransInfo->mapTransArray[usrReq->transInfo->mapTransInfo->numMapTrans] = usrReq->transInfo;
               usrReq->transInfo->mapTransInfo->numMapTrans++; 
               
               /* Call Prefix Flush */
               retVal = NPF_IPv4UC_PrefixTableFlush( 
                                  usrReq->callbackHandle,
                                  usrReq->transInfo->int_correlator,
                                  NPF_REPORT_ALL, 
                                  usrReq->transInfo->usrInfo->tableArray[IPV4_TABLE_TYPE_PREFIX].intHandle); 
               if(retVal != NPF_SUCCESS)
               {
                  npf_logger_Write(LOG_IPV4, LOG_ERR,
		               "Preifx Table Flush Failed ");
                  RETURN(retVal);
               }

               if ((retVal=ipv4_DupTransInfo(usrReq->transInfo,
			                                    &dstTransInfo)) 
                                            != NPF_SUCCESS)
               {
                  npf_logger_Write(LOG_IPV4, LOG_ERR,
		               " Initialization of Transaction Failed");
                  RETURN(retVal);
               }
               /* Call Nhop Flush */
               retVal = NPF_IPv4UC_NextHopTableFlush(
                                           usrReq->callbackHandle,
                                           dstTransInfo->int_correlator,
                                           NPF_REPORT_ALL, 
                                           dstTransInfo->usrInfo->tableArray[IPV4_TABLE_TYPE_NHOP].intHandle); 
               if(retVal != NPF_SUCCESS)
               {
                  npf_logger_Write(LOG_IPV4, LOG_ERR,
		               "Nhop Table Flush Failed ");
                  RETURN(retVal);
               }
               tableInfo->state = TABLE_EMPTY;
       	   }
	         else
	         {
               npf_logger_Write(LOG_IPV4, LOG_ERR,
		           "Error : Request has come in invalid table state");
	            RETURN(NPF_E_UNKNOWN);
	         }
         } 
         break;
#endif /* IPV4_UNIF_SUPPORT */
      default:
         npf_logger_Write(LOG_IPV4, LOG_ERR,
                                 "Invalid request from User Encountered !");
         RETURN(NPF_FAIL);
   }
   RETURN(NPF_SUCCESS);
} /* ipv4_InvokeUserReqHandler */



/******************************************************************
 * Callback from FP Handling Section
 * *****************************************************************/
/*****************************************************************************
*    Name: ipv4_SendToAppCallback
*
*    Description:
*
*    Parameters:
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
*/
NPF_RET ipv4_SendToAppCallback(
	     IPV4UsrReqInfo_t          *usrReq, 
        NPF_IPv4UC_CallbackData_t *cbData,
        NPF_IPv4UC_CallbackType_t cbType,
        NPF_correlator_t          corr
	     )
{
    NPF_IPCallbackData_t       thdData;
    IPV4UserInfo_t             *userInfo = NULL; /* LOCK*/

    IPV4_DBG_TRACE();

    assert(cbData); 
    assert(usrReq); 
    
    /* For User Lock */ 
    userInfo = usrReq->transInfo->usrInfo;
    assert(userInfo);
    
    USER_LOCK();
   
   /* Create data to be sent to the thread */
    thdData.type = NPF_IPv4;
    memcpy(&thdData.u.ipv4Data ,cbData, sizeof(NPF_IPv4UC_CallbackData_t));
    thdData.u.ipv4Data.type = cbType; 
    thdData.u.ipv4Data.resp = NULL; 
    /* Memory created in thread data is freed in application thread; */
    thdData.u.ipv4Data.resp = (NPF_IPv4UC_AsyncResponse_t*)PIL_AllocateMemory(sizeof(NPF_IPv4UC_AsyncResponse_t) , PIL_MEM_ZERO_INIT);

    if(thdData.u.ipv4Data.resp == NULL)
    {
        npf_logger_Write(LOG_IPV4, LOG_ERR,
                   "Memory Allocation problem ");
        USER_UNLOCK();
        RETURN(NPF_OUT_OF_MEMORY);
    }
    if(cbData->resp) 
       memcpy(thdData.u.ipv4Data.resp, cbData->resp, sizeof(NPF_IPv4UC_AsyncResponse_t));
    
    /* Give the callback to the User  Application Thread */ 
    cbthread_pushIPAPICB(usrReq->transInfo->usrInfo->cbFunc,
                                          usrReq->transInfo->usrInfo->context, corr, &thdData);
    /* Free the memory allocated for response */
    PIL_FreeMemory(thdData.u.ipv4Data.resp); 
    
    USER_UNLOCK();

    RETURN(NPF_SUCCESS);
} /* ipv4_SendToAppCallback */

/*****************************************************************************
*    Name: ipv4_InvokeAppCallback
*
*    Description:
*
*    Parameters:
*
*    Preconditions:
*
*    Postconditions:
*
*    Returns:
*
*/
NPF_RET ipv4_InvokeAppCallback(
	IPV4UsrReqInfo_t          *usrReq, 
   NPF_IPv4UC_CallbackData_t *fpCBData,
   NPF_IPv4UC_CallbackType_t cbType,
	NPF_IPv4UC_ReturnCode_t   retCode
	)
{
    NPF_correlator_t corr;
    NPF_RET retVal;
    NPF_IPv4UC_CallbackData_t  *cbData = NULL;
    NPF_IPv4UC_AsyncResponse_t *resp = NULL;
    uint32_t                   respSize;

    IPV4_DBG_TRACE();
    
   /* If we hace come here, we are going to give positive or negative 
      response to the application.*/
  
   /* Allocate Callback Data Memory to be passed to Applciation */ 
    cbData = (NPF_IPv4UC_CallbackData_t *) PIL_AllocateMemory
	        (sizeof(NPF_IPv4UC_CallbackData_t), PIL_MEM_ZERO_INIT);
    if(!cbData) 
    {
       npf_logger_Write(LOG_IPV4, LOG_ERR , 
          "Memory Allocation Failed. Unable to return Callback to App  \n" );
       retVal = NPF_OUT_OF_MEMORY;
       goto CleanUp;
    }
   
    /* We need to fill the Callback Data Type here because Application 
       typically make a check for this !! */
    cbData->type = cbType; 
    /* Now, create the memory for the response also */ 
    if(fpCBData == NULL || (fpCBData != NULL && fpCBData->numResp ==0))
    { 
        resp = (NPF_IPv4UC_AsyncResponse_t *)PIL_AllocateMemory (
                    sizeof(NPF_IPv4UC_AsyncResponse_t), PIL_MEM_ZERO_INIT);
    }

    if( fpCBData != NULL && fpCBData->numResp > 0)
    { 
       respSize =  (sizeof(NPF_IPv4UC_AsyncResponse_t)* (fpCBData->numResp));
       resp = (NPF_IPv4UC_AsyncResponse_t *)PIL_AllocateMemory (
                    respSize , PIL_MEM_ZERO_INIT);
    }
    /* Check if resp is still not allocated */    
    if (resp == NULL)
    {
        npf_logger_Write(LOG_IPV4, LOG_ERR , "Memory Allocation Failed. Unable to return Callback \n" );
        retVal = NPF_OUT_OF_MEMORY;
        goto CleanUp;
    }
    /* Memory assigned to cback data */
    cbData->resp  = resp;
    /* Intialize the response with error code */ 
    cbData->resp->returnCode = NPF_E_UNKNOWN;
    /* Callback Data  and response allocated. Now do some fundamental checks */ 
    if(usrReq == NULL)
    {
       /* If we have come here because of invalid parameters */
        npf_logger_Write(LOG_IPV4, LOG_ERR , "Invalid usrReq extracted. Unable to give callback \n" );
        retVal = NPF_E_UNKNOWN;
        goto CleanUp;
    } 
    
    /* Fill the corr with correct correlator value */ 
    corr = usrReq->correlator;
   
    /* Check if we need to give any report to the application */
    if(usrReq->appVerbosity == NPF_REPORT_NONE)
    {
        npf_logger_Write(LOG_IPV4, LOG_TRACE, "No Callback Needed by App \n" );
        retVal = NPF_SUCCESS;
        goto CleanUp;
    }
    
    if(usrReq->transInfo == NULL)
    {
        npf_logger_Write(LOG_IPV4, LOG_ERR, "Transaction not being created. Unable to invoke callback\n" );
        retVal = NPF_E_UNKNOWN;
        goto CleanUp;
    }
   
    /* Fundamental checks done !! Now, as per the request/callback type, 
     * process each request towards the application 
     */ 
 
    switch(cbType)
    {
	 case NPF_IPV4UC_PREFIX_TABLE_HANDLE_CREATE:
        {
	     NPF_IPv4UC_PfxCreateResp_t pfxHndlCreate;
              cbData->numResp = 1;
              if( retCode == NPF_SUCCESS) 
	       {
	          pfxHndlCreate.extHandle =  (NPF_uint32_t) usrReq->transInfo->
		                            tableInfo->extHandle;
	          pfxHndlCreate.intHandle =  (NPF_uint32_t) usrReq->transInfo->
	   	                            tableInfo->intHandle;
	          cbData->allOK = NPF_TRUE;
	       }
	       else
	       {
	          cbData->allOK = NPF_FALSE;
	          pfxHndlCreate.extHandle = (uint32_t)NULL;
	          pfxHndlCreate.intHandle = (uint32_t)NULL;
	       }
              
	        cbData->resp->u.prefixTableHandles = pfxHndlCreate;
                cbData->resp->returnCode = retCode;
#ifdef IPV4_UNIF_SUPPORT
           if (usrReq->transInfo->usrInfo->usrType == IPV4_USR_UNIF)
           {
              ipv4_Unified_CallBack(usrReq->transInfo->usrInfo->context,
                      corr, *cbData);
              PIL_FreeMemory(cbData->resp);  
              PIL_FreeMemory(cbData);  
              RETURN(NPF_SUCCESS);
           }
#endif
	        break;
	     }
        case NPF_IPV4UC_NEXT_HOP_TABLE_HANDLE_CREATE:
	{
           NPF_IPv4UC_NextHopTableHandle_t nhopHndlCreate;
           
           cbData->numResp = 1;
	   if( retCode == NPF_SUCCESS) 
	   {
	      cbData->allOK   = NPF_TRUE;
   	      nhopHndlCreate  = (NPF_uint32_t)
		                     usrReq->transInfo->tableInfo->intHandle;
   	   }
   	   else
   	   {
   	      cbData->allOK = NPF_FALSE;
	      nhopHndlCreate = (NPF_IPv4UC_NextHopTableHandle_t )NULL;
	   }
	   cbData->resp->u.nextHopTableHandle = nhopHndlCreate ;
           cbData->resp->returnCode = retCode;
#ifdef IPV4_UNIF_SUPPORT
           if (usrReq->transInfo->usrInfo->usrType == IPV4_USR_UNIF)
           {
              ipv4_Unified_CallBack(usrReq->transInfo->usrInfo->context,
                      corr, *cbData);
              PIL_FreeMemory(cbData->resp);  
              PIL_FreeMemory(cbData);  
              RETURN(NPF_SUCCESS);
           }
#endif
   	   break;
   	}
        case NPF_IPV4UC_ADDRESS_RES_TABLE_HANDLE_CREATE:
	   {
              NPF_IPv4UC_AddResTableHandle_t arpHndlCreate;
           
              cbData->numResp = 1;
	      if( retCode == NPF_SUCCESS) 
	      {
	         cbData->allOK   = NPF_TRUE;
	         arpHndlCreate  = (NPF_uint32_t)
		                     usrReq->transInfo->tableInfo->intHandle;
	      }
	      else
	      {
	          cbData->allOK = NPF_FALSE;
	          arpHndlCreate = (NPF_IPv4UC_AddResTableHandle_t )NULL;
	      }
	      cbData->resp->u.addResTableHandle = arpHndlCreate;
              cbData->resp->returnCode = retCode;
	      break;
	   }
     case NPF_IPV4UC_FIB_TABLE_HANDLE_CREATE:
	  {
              NPF_IPv4UC_FibCreateResp_t fibHndlCreate;
              cbData->numResp = 1;
	      if( retCode == NPF_SUCCESS) 
	      {
	         cbData->allOK   = NPF_TRUE;
	         fibHndlCreate.intHandle  = (NPF_uint32_t)
		                     usrReq->transInfo->tableInfo->intHandle;
	         fibHndlCreate.extHandle  = (NPF_uint32_t)
		                     usrReq->transInfo->tableInfo->extHandle;
	      }
	      else
	      {
	         cbData->allOK = NPF_FALSE;
	         fibHndlCreate.extHandle = (uint32_t)NULL;
	         fibHndlCreate.intHandle = (uint32_t)NULL;
	      }
	      cbData->resp->u.fibTableHandles = fibHndlCreate ;
              cbData->resp->returnCode = retCode;
	      break;
	   }
      case NPF_IPV4UC_PREFIX_TABLE_HANDLE_DELETE : 
      case NPF_IPV4UC_NEXT_HOP_TABLE_HANDLE_DELETE:
         cbData->numResp = 1;
         if( retCode == NPF_SUCCESS) 
            cbData->allOK = NPF_TRUE;
         else
            cbData->allOK = NPF_FALSE;
         cbData->resp->returnCode = retCode;

#ifdef IPV4_UNIF_SUPPORT
         if (usrReq->transInfo->usrInfo->usrType == IPV4_USR_UNIF)
         {
            ipv4_Unified_CallBack(usrReq->transInfo->usrInfo->context,
              corr, *cbData);
            PIL_FreeMemory(cbData->resp);  
            PIL_FreeMemory(cbData);  
            RETURN(NPF_SUCCESS);
         }
#endif
         break;

      case NPF_IPV4UC_FIB_TABLE_HANDLE_DELETE:
	   {
         cbData->numResp = 1;
	      if( retCode == NPF_SUCCESS) 
	      {
	         cbData->allOK   = NPF_TRUE;
	      }
	      else
	      {
	         cbData->allOK = NPF_FALSE;
	      }
         cbData->resp->returnCode = retCode;
	      break;
	   }

      case NPF_IPV4UC_ADDRESS_RES_TABLE_HANDLE_DELETE:
      case NPF_IPV4UC_PREFIX_NEXT_HOP_TABLE_BIND: 
	   {
         cbData->numResp = 1;
         if( retCode == NPF_SUCCESS) 
	         cbData->allOK = NPF_TRUE;
	      else
	         cbData->allOK = NPF_FALSE;
         cbData->resp->returnCode = retCode;
#ifdef IPV4_UNIF_SUPPORT
           if (usrReq->transInfo->usrInfo->usrType == IPV4_USR_UNIF)
           {
              ipv4_Unified_CallBack(usrReq->transInfo->usrInfo->context,
                      corr, *cbData);
              PIL_FreeMemory(cbData->resp);  
              PIL_FreeMemory(cbData);  
              RETURN(NPF_SUCCESS);
           }
#endif
	  
	      break;
	   }
      case NPF_IPV4UC_PREFIX_TABLE_ATTRIBUTE_QUERY:
      case NPF_IPV4UC_NEXT_HOP_TABLE_ATTRIBUTE_QUERY:
      case NPF_IPV4UC_FIB_TABLE_ATTRIBUTE_QUERY:
      case NPF_IPV4UC_ADDRESS_RES_TABLE_ATTRIBUTE_QUERY:
	   { 
	      cbData->allOK = NPF_FALSE;
	      cbData->numResp = 1;
	      cbData->resp->returnCode = retCode;
	   }
     /*** Callbacks for following requests are receveid from FP ***/
      case NPF_IPV4UC_PREFIX_ENTRY_ADD:
      case NPF_IPV4UC_PREFIX_ENTRY_DELETE: 
      case NPF_IPV4UC_PREFIX_TABLE_FLUSH: 
      case NPF_IPV4UC_PREFIX_ENTRY_QUERY: 
      case NPF_IPV4UC_NEXT_HOP_ENTRY_ADD:
      case NPF_IPV4UC_NEXT_HOP_ENTRY_DELETE :
      case NPF_IPV4UC_NEXT_HOP_TABLE_FLUSH:
      case NPF_IPV4UC_NEXT_HOP_ENTRY_QUERY:
      case NPF_IPV4UC_ADDRESS_RES_ENTRY_ADD:
      case NPF_IPV4UC_ADDRESS_RES_ENTRY_DELETE:
      case NPF_IPV4UC_ADDRESS_RES_TABLE_FLUSH :
      case NPF_IPV4UC_ADDRESS_RES_ENTRY_QUERY:
      {
          /* Do a basic check to see if FP CBData and passed ones are same */ 
         if (fpCBData == NULL || fpCBData->resp == NULL)
         {
            npf_logger_Write(LOG_IPV4, LOG_ERR,
		       "Invalid Callback Info received from FP.\n" );
             retVal = NPF_E_UNKNOWN;
             goto Callback;
         }
         /* Copy the stuff */ 
         cbData->type = fpCBData->type;
         cbData->allOK = fpCBData->allOK;
         cbData->numResp = fpCBData->numResp;
         memcpy(cbData->resp, fpCBData->resp, sizeof(NPF_IPv4UC_AsyncResponse_t));  
#ifdef IPV4_UNIF_SUPPORT
         if (usrReq->transInfo->usrInfo->usrType == IPV4_USR_UNIF)
         {
            cbData->type = cbType;
            ipv4_Unified_CallBack(usrReq->transInfo->usrInfo->context,
              corr, *cbData);
            PIL_FreeMemory(cbData->resp);  
            PIL_FreeMemory(cbData);  
            RETURN(NPF_SUCCESS);
         }
#endif
         break;
      }

      case NPF_IPV4UC_FIB_ENTRY_ADD:
      case NPF_IPV4UC_FIB_ENTRY_DELETE:
      case NPF_IPV4UC_FIB_TABLE_FLUSH:
      case NPF_IPV4UC_FIB_ENTRY_QUERY :
      {
          /* Do a basic check to see if FP CBData and passed ones are same */ 
         if (fpCBData == NULL)
         {
            npf_logger_Write(LOG_IPV4, LOG_ERR,
		       "Invalid Callback received from FP\n" );
             retVal = NPF_E_UNKNOWN;
             goto Callback;
         }
         
         memcpy(cbData, fpCBData, sizeof(NPF_IPv4UC_CallbackData_t));
         /* Number of responses also will get copied here */
         if( fpCBData->resp == NULL )
         {
             npf_logger_Write(LOG_IPV4, LOG_ERR,
		       "Invalid response received from FP \n" );
             retVal = NPF_E_UNKNOWN;
             goto Callback;
         }
         memcpy(cbData->resp, fpCBData->resp, sizeof(NPF_IPv4UC_AsyncResponse_t));  
         break;

      }
      default:
      {
          npf_logger_Write(LOG_IPV4, LOG_ERR,
                         "Invalid request type invoked ");
          retVal = NPF_E_UNKNOWN;
          goto Callback;
      }
    }

Callback:
    /* Final Step : Send it to the Application thread */
    if((retVal = ipv4_SendToAppCallback(usrReq,cbData,cbType,corr)) != NPF_SUCCESS)
    {
        npf_logger_Write(LOG_IPV4, LOG_ERR,
                         "Unable to pass the response to Application ");
        retVal = NPF_E_UNKNOWN;
        goto CleanUp;
    }

CleanUp:
    /* Cleanup the context not needed */
    if(resp != NULL)
       PIL_FreeMemory(resp); 
    if(cbData != NULL)
       PIL_FreeMemory(cbData); 
    RETURN( retVal );
}

/*****************************************************************************
*
*  Forwarding Plane Callback Helper  functions
*
*****************************************************************************/

/*****************************************************************************
*    Name: ipv4_PrcForL2Table
*   
*    Description: function which handles L2 Table processing for 
*                  their callbacks. 
*       
*        
*    Parameters:
*        
*    Preconditions:
*    
*    Postconditions:
*        
*    
*    Returns:
*        Standard error value
*    
*/
NPF_RET ipv4_PrcForL2Table( 
                        FPPI_FEID feId,
                        NPF_IPv4UC_CallbackType_t cbType,
                        IPV4UsrReqInfo_t   *usrReq 
                        )
{
    NPF_RET retVal;
    uint32_t  i;
    IPV4NhopCntxt_t            *nhopCntxt = NULL;
    IPV4AddResCntxt_t          *addResCntxt = NULL;
    NPF_IPv4Address_t          addr;
   NPF_IPv4UC_NextHopArray_t   *nhopArray=NULL;
 
    IPV4_DBG_TRACE();
    
    assert(usrReq);
 
    switch(cbType)
    {
       case NPF_IPV4UC_NEXT_HOP_ENTRY_ADD:
       {
          /* Get the next-hop context */ 
          nhopCntxt = usrReq->nhopCntxt; 
          assert(nhopCntxt); /* Defensive check */
          assert(nhopCntxt->nhIdArray); /* Defensive check */
          assert(nhopCntxt->nhopArray); /* Defensive check */
         /* ### NOTE: There is a design limitation in L2ID Mgr; 
                We are not able to send multiple NextHopIds. 
                So, we will still work with an assumption that one 
                entry at one point.
                This can however affect our array handling in case 
                of NextHopEntry Add API (TBD) 
            ### */
          /* Commit with the L2Table;*/
          for(i=0; i< nhopCntxt->numEntries; i++)
          {
             nhopArray = &nhopCntxt->nhopArray[i];
             /* Get the context pointer */
             nhopArray = &usrReq->nhopCntxt->nhopArray[i];
             /* Safety check ; We don't support ECMP */
             if(nhopArray->nextHopCount > 1)
             {
                npf_logger_Write(LOG_IPV4, LOG_ERR,
                  "More than one nextHop count entry for each NHID");        
                RETURN(NPF_E_UNKNOWN);
             }
             /* Commit only in case it is Eth or ATM */
             if (nhopCntxt->nhopArray->nextHopArray->mediaAddress.type 
                                          == NPF_NO_MEDIA_TYPE) 
               continue;
             if((retVal = l2idm_CommitForNextHopTable(nhopCntxt->nhIdArray[i], 
                                     1, 
                                     &nhopArray->nextHopArray->nextHopIP)) != NPF_SUCCESS)
             { 
                   npf_logger_Write(LOG_IPV4, LOG_ERR,
                        "Could not commit with L2Table !! ");
	                RETURN(retVal);
             }
          } /* Check for all the entries */
       }
       break;
       case NPF_IPV4UC_NEXT_HOP_ENTRY_DELETE:
       {
         /* Get the next-hop context */ 
          nhopCntxt = usrReq->nhopCntxt; 
          assert(nhopCntxt); /* Defensive check */
          assert(nhopCntxt->nhIdArray); /* Defensive check */
          assert(nhopCntxt->nhopArray); /* Defensive check */

          for (i=0; i < nhopCntxt->numEntries; i++) 
          {
             /* Commit only in case it is not POS */
             if (nhopCntxt->nhopArray->nextHopArray->mediaAddress.type 
                                          == NPF_NO_MEDIA_TYPE) 
               continue;

             addr = nhopCntxt->nhopArray[i].nextHopArray->nextHopIP;
             retVal = l2idm_DeleteForNextHopTable(nhopCntxt->nhIdArray[i],1,&addr );
             if(retVal != NPF_SUCCESS)
             {
                 npf_logger_Write(LOG_IPV4, LOG_ERR,
                      "Error cleaning the L2Ids");
	              RETURN(retVal);
             }
          }
       }
       break;
       case NPF_IPV4UC_ADDRESS_RES_ENTRY_ADD:
       {
         /* Get the AddRes context */ 
          addResCntxt = usrReq->addResCntxt; 
          assert(addResCntxt); /* Defensive check */
          assert(addResCntxt->addResKeyArray); /* Defensive check */

          for (i=0; i < addResCntxt->numEntries; i++) 
          {
             addr = addResCntxt->addResKeyArray[i].IP_Address;
             retVal = l2idm_CommitForL2Table(feId,addResCntxt->numEntries,&addr);

             if(retVal != NPF_SUCCESS)
             {
                 npf_logger_Write(LOG_IPV4, LOG_ERR,
                      "Error committing the L2Id corresponding to added ARP addresses");
	              RETURN(retVal);
             }
          }
       }
       break;
       case NPF_IPV4UC_ADDRESS_RES_ENTRY_DELETE:
       {
         /* Get the next-hop context */ 
          addResCntxt = usrReq->addResCntxt; 

          assert(addResCntxt); /* Defensive check */
          assert(addResCntxt->addResKeyArray); /* Defensive check */

          for (i=0; i < addResCntxt->numEntries; i++) 
          {
             addr = addResCntxt->addResKeyArray[i].IP_Address;
             retVal = l2idm_DeleteForL2Table(feId,addResCntxt->numEntries,&addr);
             if(retVal != NPF_SUCCESS)
             {
                 npf_logger_Write(LOG_IPV4, LOG_ERR,
                      "Error cleaning the L2Id corresponding to deleted ARP addresses");
	              RETURN(retVal);
             }
          }
       }
       break;
       case NPF_IPV4UC_NEXT_HOP_TABLE_FLUSH:
       {
          if((retVal = l2idm_FlushNHIDList()) != NPF_SUCCESS)
          {
              npf_logger_Write(LOG_IPV4, LOG_ERR,
                         "Error Flushing L2Id Database ");
          }
          RETURN(retVal);
       }
       break;
       default:
       {
          /* Return Success for other requests */
          break;
       }
    } /* end of switch */
   
    RETURN(NPF_SUCCESS);
} /* ipv4_PrcForL2Table */


/*****************************************************************************
*    Name: ipv4_ReceiveFPResponseEx
*   
*    Description: Generic function which handles commong processing for
*    all types of callbacks. This is a new function which gets the 
*    callback type as defined by NPF
*    Use this routine only for the Newly Added Code.    
*        
*    Parameters:
*        
*    Preconditions:
*        This is valid for all APIs except FLUSH  
*    
*    Postconditions:
*        
*    
*    Returns:
*        Standard error value
*    
*/

NPF_RET ipv4_ReceiveFPResponseEx(  NPF_IPv4UC_CallbackType_t cbType,
                              FPPI_FEID feid,
                              FPPI_CORRELATOR correlator,
                              FPPI_CONTEXT context,
                              void *response )
{
    NPF_RET retVal;
    uint32_t  i;
    IPV4ReqToFEInfo_t   *reqToFE = NULL;
    IPV4TransInfo_t     *transInfo = NULL;
    IPV4UsrReqInfo_t   *usrReq = NULL;
    /* FPP Status Info */
    NPF_IPv4UC_CallbackData_t *cbData = NULL;
    
    IPV4_DBG_TRACE();
   
    /* Get the FPP Response */
    cbData = (NPF_IPv4UC_CallbackData_t*)response;  
    /* Basic Check */
    if(cbData == NULL || cbData->type == 0 || cbData->resp == NULL)
    {
        IPV4_DBG_PRINT( LOG_ERR, "Invalid Response received from FE Id %d \n",feid );
	     goto CLEANUP;
    }
    
    /* Find the request to FE exists - with fppi_correl as key */
    if((retVal=ipv4_FindFEReqByCorrelInFpp(feid, correlator,&reqToFE))
		    != NPF_SUCCESS)
    {
        IPV4_DBG_PRINT( LOG_ERR, "could not find correlator \n" );
	     goto CLEANUP;
    }
    assert(reqToFE); /* safe check */
    
    /* Update the state and see if we have to give callback to the app */
    transInfo = reqToFE->transInfo;
    assert(transInfo); /* safe check */
    usrReq = transInfo->usrReq;
    assert(usrReq); /* safe check */
    
     
    /* Store the response status  and update 
       the state in the callback contexts */ 
    reqToFE->state =  REQ_TO_FE_CBACK_RECVD;
    reqToFE->retStatus = cbData->resp->returnCode ;
    
    /* Check if the callback type returned by FP is correct */
    /* TBD */
    /* Process those requests for which L2Id Manager interaction is needed */
    if((retVal = ipv4_PrcForL2Table(feid, cbType, usrReq)) != NPF_SUCCESS)
    {
        IPV4_DBG_PRINT( LOG_ERR, "Error encountered with L2 Table \n" );
	     goto CLEANUP;
    } 
    /* Decrement the number of FE waits */ 
    transInfo->transFpInfo.numFEWait--;
 
    /* For last FE request , go about calling the callback regd. by the application */ 
    if (transInfo->transFpInfo.numFEWait == 0)
    {

        for(i=0; i< transInfo->transFpInfo.numFEToSend; i++)
        {
            reqToFE =  transInfo->transFpInfo.reqFEPtrArray[i];
            /* Check if all of the requests have completed successfully */
            if(reqToFE->state != REQ_TO_FE_CBACK_RECVD)
            {
                IPV4_DBG_PRINT( LOG_ERR, "Internal error; State incorrect \n" );
	             goto CLEANUP;
            }           
            if(reqToFE->retStatus != NPF_NO_ERROR)
            {
                IPV4_DBG_PRINT( LOG_ERR, "FE with Id %d returned Error \n" );
                /* Override the entryStatus to generate consolidate reply */
                cbData->resp->returnCode = reqToFE->retStatus;
                break;
            }
        }
        
        /* transaction state is pending !! */ 
        transInfo->transPend = FALSE;
      
       /* Get the User request context. Here, FP callback contains the valid
        * return code. Hence, the paramter of returnCode is sent as NULL 
        * as that is not used in this function. */
       
       if((retVal= ipv4_InvokeAppCallback(usrReq,(NPF_IPv4UC_CallbackData_t*)cbData, cbType,0))
		                      != NPF_SUCCESS)
       {
           npf_logger_Write(LOG_IPV4, LOG_ERR,
                         "App Callback could not be invoked !! ");
           retVal = NPF_E_UNKNOWN;
	        goto CLEANUP;
       }
       /*Eventhing went on fine. So, just cleanup the state */
       retVal = NPF_SUCCESS;
       goto CLEANUP;
    }
    else
    {
       npf_logger_Write(LOG_IPV4, LOG_TRACE,
                         "Callback not received from all FEs!! ");
       RETURN(NPF_SUCCESS);
    }
CLEANUP:
       /* Callback invoked. Free the context allocated till now */
    ipv4_DeleteUsrReq(usrReq);
    RETURN(retVal);
} /* ipv4_ReceiveFPResponseEx*/

/*****************************************************************************
*    Name: ipv4_ReceiveFPResponse
*   
*    Description: Generic function which handles commong processing for all types
*                  of callbacks. 
*       
*        
*    Parameters:
*        
*    Preconditions:
*        This is valid for all APIs except FLUSH  
*    
*    Postconditions:
*        
*    
*    Returns:
*        Standard error value
*    
*/

NPF_RET ipv4_ReceiveFPResponse(  NPF_IPv4UC_CallbackType_t cbType,
                              FPPI_FEID feid,
                              FPPI_CORRELATOR correlator,
                              FPPI_CONTEXT context,
                              void *response )
{
    NPF_RET retVal;
    uint32_t  i;
    IPV4ReqToFEInfo_t   *reqToFE = NULL;
    IPV4TransInfo_t     *transInfo = NULL;
    IPV4UsrReqInfo_t   *usrReq = NULL;
    NPF_IPv4UC_CallbackData_t cbData;
    /* FPP Status Info */
    npf_IPv4EntryStatusList    *entryStatus = NULL; /* For everything except flush */  
    
    IPV4_DBG_TRACE();
    
    /* Initialize callback data response to NULL */
    cbData.resp = NULL;
 
    /* Get the FPP Response */
    entryStatus = (npf_IPv4EntryStatusList*)response;  
    /* Basic Check */
    if(entryStatus == NULL || entryStatus->entryList == NULL)
    {
        IPV4_DBG_PRINT( LOG_ERR, "Null Response received from FE Id %d \n",feid );
	     goto CLEANUP;
    }

    /* Find the request to FE exists - with fppi_correl as key */
    if((retVal=ipv4_FindFEReqByCorrelInFpp(feid, correlator,&reqToFE))
		    != NPF_SUCCESS)
    {
        IPV4_DBG_PRINT( LOG_ERR, "could not find correlator \n" );
        retVal = NPF_E_UNKNOWN;
	     goto CLEANUP;
    }
    assert(reqToFE); /* safe check */
    
    /* Update the state and see if we have to give callback to the app */
    transInfo = reqToFE->transInfo;
    assert(transInfo); /* safe check */
    usrReq = transInfo->usrReq;
    assert(usrReq); /* safe check */
    
    /* Store the response status  and update the state in the 
       callback contexts */ 
    reqToFE->state =  REQ_TO_FE_CBACK_RECVD;
    assert(entryStatus->entryList);
    reqToFE->retStatus = entryStatus->entryList->retStatus;
    
    /* Process those requests for which L2Id Manager interaction is needed */
    if((retVal = ipv4_PrcForL2Table(feid, cbType, usrReq)) != NPF_SUCCESS)
    {
        IPV4_DBG_PRINT( LOG_ERR, "Error encountered with L2 Table \n" );
        retVal = NPF_E_UNKNOWN;
	     goto CLEANUP;
    } 
    /* Decrement the number of FE waits */ 
    transInfo->transFpInfo.numFEWait--;
 
    /* For last FE request , go about calling the callback regd.
       by the application */ 
    if (transInfo->transFpInfo.numFEWait == 0)
    {

        for(i=0; i< transInfo->transFpInfo.numFEToSend; i++)
        {
            reqToFE =  transInfo->transFpInfo.reqFEPtrArray[i];
            /* Check if all of the requests have completed successfully */
            if(reqToFE->state != REQ_TO_FE_CBACK_RECVD)
            {
                IPV4_DBG_PRINT( LOG_ERR, "Internal error \n" );
	             goto CLEANUP;
            }           
            if(reqToFE->retStatus != NPF_NO_ERROR)
            {
                IPV4_DBG_PRINT( LOG_ERR, "FE with Id %d returned Error \n",feid);
                /* Override the entryStatus to generate consolidate reply */
                entryStatus->entryList->retStatus = reqToFE->retStatus;
                break;
            }
        }
        /* This function creates the space for the cbData.resp */ 
        if((retVal = ipv4_ExtractFppResponse(entryStatus, &cbData)) != NPF_SUCCESS)
        {
             npf_logger_Write(LOG_IPV4, LOG_ERR,
                   "Unable to extract the response from Fe [Id = %d]", feid);
           retVal = NPF_E_UNKNOWN;
	        goto CLEANUP;
        }
        /* transaction state is pending !! */ 
        transInfo->transPend = FALSE;
      
       /* Get the User request context. Here, FP callback contains the valid
        * return code. Hence, the paramter of returnCode is sent as NULL 
        * as that is not used in this function. */
       
       if((retVal= ipv4_InvokeAppCallback(usrReq,(NPF_IPv4UC_CallbackData_t*)&cbData, cbType,0))
		                      != NPF_SUCCESS)
       {
           npf_logger_Write(LOG_IPV4, LOG_ERR,
                         "App Callback could not be invoked !! ");
           retVal = NPF_E_UNKNOWN;
	        goto CLEANUP;
       }
       
       /*Eventhing went on fine. So, just cleanup the state */
       retVal = NPF_SUCCESS;
       goto CLEANUP;
    }
    else
    {
       npf_logger_Write(LOG_IPV4, LOG_TRACE,
                         "Callback not received from all FEs!! ");
       RETURN(NPF_SUCCESS);
    }
CLEANUP:
    if(cbData.resp != NULL)
       PIL_FreeMemory(cbData.resp);
    ipv4_DeleteUsrReq(usrReq);
    RETURN(retVal);
} /* ipv4_ReceiveFPResponse*/

/*****************************************************************************
*
*  Forwarding Plane Callback Functions
*
*****************************************************************************/

/*****************************************************************************
*    Name: ipv4_AddPrefixesComplete
*   
*    Description: Add prefix entry callback
*       
*        
*    Parameters:
*        IN: feid - forwarding element identifier
*            correlator - callback correlator
*            context - callback context
*            response - response from the FP
*        
*    Preconditions:
*        
*    
*    Postconditions:
*        
*    
*    Returns:
*        Standard error value
*    
*/
void ipv4_AddPrefixesComplete(FPPI_FEID feid,
                              FPPI_CORRELATOR correlator,
                              FPPI_CONTEXT context,
                              void *response )
{
    if(ipv4_ReceiveFPResponse(NPF_IPV4UC_PREFIX_ENTRY_ADD , feid, correlator, context, response)
                                != NPF_SUCCESS)
    {
        IPV4_DBG_PRINT( LOG_ERR, "Encountered an error in Completion function\n" );
    }

    return;
}

/*****************************************************************************
*    Name: ipv4_UpdatePrefixesComplete
*   
*    Description: Update prefix entry callback
*       
*        
*    Parameters:
*        IN: feid - forwarding element identifier
*            correlator - callback correlator
*            context - callback context
*            response - response from the FP
*        
*    Preconditions:
*        
*    
*    Postconditions:
*        
*    
*    Returns:
*        Standard error value
*    
*/
void ipv4_UpdatePrefixesComplete(FPPI_FEID feid,
                              FPPI_CORRELATOR correlator,
                              FPPI_CONTEXT context,
                              void *response )
{
    /* Dummy Function */
    return;
}


/*****************************************************************************
*    Name: ipv4_QueryPrefixesComplete
*   
*    Description: Query next hop entry callback
*       
*        
*    Parameters:
*        IN: feid - forwarding element identifier
*            correlator - callback correlator
*            context - callback context
*            response - response from the FP
*        
*    Preconditions:
*        
*    
*    Postconditions:
*        
*    
*    Returns:
*        Standard error value
*    
*/
void ipv4_QueryPrefixesComplete(FPPI_FEID feid,
                                FPPI_CORRELATOR correlator,
                                FPPI_CONTEXT context,
                                void *response )
{
    if(ipv4_ReceiveFPResponseEx(NPF_IPV4UC_PREFIX_ENTRY_QUERY, feid, correlator, context, response)
                                != NPF_SUCCESS)
    {
        IPV4_DBG_PRINT( LOG_ERR, "Encountered an error in Completion function\n" );
    }
    return;
}


/*****************************************************************************
*    Name: ipv4_DeletePrefixesComplete
*   
*    Description: Delete prefix entry callback
*       
*        
*    Parameters:
*        IN: feid - forwarding element identifier
*            correlator - callback correlator
*            context - callback context
*            response - response from the FP
*        
*    Preconditions:
*        
*    
*    Postconditions:
*        
*    
*    Returns:
*        Standard error value
*    
*/
void ipv4_DeletePrefixesComplete(FPPI_FEID feid,
                                FPPI_CORRELATOR correlator,
                                FPPI_CONTEXT context,
                                void *response )
{
    if(ipv4_ReceiveFPResponse(NPF_IPV4UC_PREFIX_ENTRY_DELETE, feid, correlator, context, response)
                                != NPF_SUCCESS)
    {
        IPV4_DBG_PRINT( LOG_ERR, "Encountered an error in Completion function\n" );
    }
    return;
}
   
/*****************************************************************************
*    Name: ipv4_FlushPrefixesComplete
*   
*    Description: Flush prefix entry callback
*       
*        
*    Parameters:
*        IN: feid - forwarding element identifier
*            correlator - callback correlator
*            context - callback context
*            response - response from the FP
*        
*    Preconditions:
*        
*    
*    Postconditions:
*        
*    
*    Returns:
*        Standard error value
*    
*/
void ipv4_FlushPrefixesComplete(FPPI_FEID feid,
                                FPPI_CORRELATOR correlator,
                                FPPI_CONTEXT context,
                                void *response )
{
    if(ipv4_ReceiveFPResponseEx(NPF_IPV4UC_PREFIX_TABLE_FLUSH, feid, correlator, context, response)
                                != NPF_SUCCESS)
    {
        IPV4_DBG_PRINT( LOG_ERR, "Encountered an error in Completion function\n" );
    }
    return;
}

   
/*****************************************************************************
*    Name: ipv4_AddNextHopComplete
*   
*    Description: Add next hop entry callback
*       
*        
*    Parameters:
*        IN: feid - forwarding element identifier
*            correlator - callback correlator
*            context - callback context
*            response - response from the FP
*        
*    Preconditions:
*        
*    
*    Postconditions:
*        
*    
*    Returns:
*        Standard error value
*    
*/
void ipv4_AddNextHopComplete(FPPI_FEID feid,
                                FPPI_CORRELATOR correlator,
                                FPPI_CONTEXT context,
                                void *response )
{
    if(ipv4_ReceiveFPResponse(NPF_IPV4UC_NEXT_HOP_ENTRY_ADD, feid, correlator, context, response)
                                != NPF_SUCCESS)
    {
        IPV4_DBG_PRINT( LOG_ERR, "Encountered an error in Completion function\n" );
    }
    return;
}

/*****************************************************************************
*    Name: ipv4_DeleteNextHopComplete
*   
*    Description: Delete next hop entry callback
*       
*        
*    Parameters:
*        IN: feid - forwarding element identifier
*            correlator - callback correlator
*            context - callback context
*            response - response from the FP
*        
*    Preconditions:
*        
*    
*    Postconditions:
*        
*    
*    Returns:
*        Standard error value
*    
*/
void ipv4_DeleteNextHopComplete(FPPI_FEID feid,
                                FPPI_CORRELATOR correlator,
                                FPPI_CONTEXT context,
                                void *response )
{ 
    if(ipv4_ReceiveFPResponse(NPF_IPV4UC_NEXT_HOP_ENTRY_DELETE, feid, correlator, context, response)
                                != NPF_SUCCESS)
    {
        IPV4_DBG_PRINT( LOG_ERR, "Encountered an error in Completion function\n" );
    }
    return;
}    

/*****************************************************************************
*    Name: ipv4_FlushNextHopComplete
*   
*    Description: Flush next hop entry callback
*       
*        
*    Parameters:
*        IN: feid - forwarding element identifier
*            correlator - callback correlator
*            context - callback context
*            response - response from the FP
*        
*    Preconditions:
*        
*    
*    Postconditions:
*        
*    
*    Returns:
*        Standard error value
*    
*/
void ipv4_FlushNextHopComplete(FPPI_FEID feid,
                                FPPI_CORRELATOR correlator,
                                FPPI_CONTEXT context,
                                void *response )
{
    if(ipv4_ReceiveFPResponseEx(NPF_IPV4UC_NEXT_HOP_TABLE_FLUSH, feid, correlator, context, response)
                                != NPF_SUCCESS)
    {
        IPV4_DBG_PRINT( LOG_ERR, "Encountered an error in Completion function\n" );
        return;
    }
       return;
}

/*****************************************************************************
*    Name: ipv4_QueryNextHopComplete
*   
*    Description: Query next hop entry callback
*       
*        
*    Parameters:
*        IN: feid - forwarding element identifier
*            correlator - callback correlator
*            context - callback context
*            response - response from the FP
*        
*    Preconditions:
*        
*    
*    Postconditions:
*        
*    
*    Returns:
*        Standard error value
*    
*/
void ipv4_QueryNextHopComplete(FPPI_FEID feid,
                                FPPI_CORRELATOR correlator,
                                FPPI_CONTEXT context,
                                void *response )
{
    /* Calling the new Response Routine */
    if(ipv4_ReceiveFPResponseEx(NPF_IPV4UC_NEXT_HOP_ENTRY_QUERY, feid, correlator, context, response)
                                != NPF_SUCCESS)
    {
        IPV4_DBG_PRINT( LOG_ERR, "Encountered an error in Completion function\n" );
    }
    return;

}


/*****************************************************************************
*    Name: ipv4_AddL2EntryComplete
*   
*    Description: Add address resolution entry callback
*       
*        
*    Parameters:
*        IN: feid - forwarding element identifier
*            correlator - callback correlator
*            context - callback context
*            response - response from the FP
*        
*    Preconditions:
*        
*    
*    Postconditions:
*        
*    
*    Returns:
*        Standard error value
*    
*/
void ipv4_AddL2EntryComplete(FPPI_FEID feid,
                                FPPI_CORRELATOR correlator,
                                FPPI_CONTEXT context,
                                void *response )
{
    if(ipv4_ReceiveFPResponse(NPF_IPV4UC_ADDRESS_RES_ENTRY_ADD, feid, correlator, context, response)
                                != NPF_SUCCESS)
    {
        IPV4_DBG_PRINT( LOG_ERR,
                  "Encountered an error in Completion function\n" );
    }
    return;
}


/*****************************************************************************
*    Name: ipv4_FlushL2EntryComplete
*   
*    Description: Add address resolution entry callback
*       
*        
*    Parameters:
*        IN: feid - forwarding element identifier
*            correlator - callback correlator
*            context - callback context
*            response - response from the FP
*        
*    Preconditions:
*        
*    
*    Postconditions:
*        
*    
*    Returns:
*        Standard error value
*    
*/
void ipv4_FlushL2EntryComplete(FPPI_FEID feid,
                                FPPI_CORRELATOR correlator,
                                FPPI_CONTEXT context,
                                void *response )
{
    /* FPP Status Info */
    IPV4_DBG_PRINT( LOG_ERR, "Completion function Not implemented\n" );
    
    /* Dummy Function */ 
    return;
}


/*****************************************************************************
*    Name: ipv4_DeleteL2EntryComplete
*   
*    Description: Delete address resolution entry callback
*       
*        
*    Parameters:
*        IN: feid - forwarding element identifier
*            correlator - callback correlator
*            context - callback context
*            response - response from the FP
*        
*    Preconditions:
*        
*    
*    Postconditions:
*        
*    
*    Returns:
*        Standard error value
*    
*/
void ipv4_DeleteL2EntryComplete (FPPI_FEID feid,
                                FPPI_CORRELATOR correlator,
                                FPPI_CONTEXT context,
                                void *response )
{
     if(ipv4_ReceiveFPResponse(NPF_IPV4UC_ADDRESS_RES_ENTRY_DELETE, feid, 
                               correlator, context, response) != NPF_SUCCESS)
    {
        IPV4_DBG_PRINT( LOG_ERR,
                  "Encountered an error in Completion function\n" );
    }
    return;
}

#ifdef IPV4_UNIF_SUPPORT
void ipv4_Unified_CallBack(
   NPF_userContext_t userContext,
   NPF_correlator_t correlator,
   NPF_IPv4UC_CallbackData_t data)
{
   DListItr *itr;
   IPV4UserInfo_t *userNode = NULL;
   uint32_t found = FALSE;
   uint32_t transCompleted = TRUE;
   IPV4TransInfo_t *transInfo;
   IPV4TransInfo_t *dstTransInfo;
   NPF_RET retVal;
 
   switch(data.type)
   {
      case  NPF_IPV4UC_PREFIX_TABLE_HANDLE_CREATE:
      case  NPF_IPV4UC_NEXT_HOP_TABLE_HANDLE_CREATE:
         {
            IPV4MapTransInfo_t *mapTransInfo;
            IPV4UsrReqInfo_t   *usrReq;
            uint32_t           i;
            IPV4UserInfo_t  *userInfo;
            IPV4TableInfo_t *tableInfo;
 
            itr = npf_list_itrCreate(&ipv4Mgr.userList);
            if (itr)
            {
               npf_list_itrFirst(itr);
               do 
               {
                  userNode = npf_list_itrGetData(itr);
                  if (userNode->context == userContext)
                  {
                     found = TRUE;
                     break;
                  }
               } while (npf_list_itrNext(itr) != -1);
            }
            if (found)
            {
               if ((transInfo = npf_list_searchByKey(&userNode->unifiedTransCpList,
                                 (const void*)&correlator)) == NULL)
               {
                  npf_logger_Write(LOG_IPV4, LOG_TRACE,
                       "No transaction with correlator %d found", correlator);
                  return;
               }
              
               transInfo->transPend = FALSE;
               /* allOk must have been initialised with TRUE */
               if (transInfo->mapTransInfo->cbResp.allOK == NPF_TRUE)
                  transInfo->mapTransInfo->cbResp.allOK = data.allOK;
 
               if (((data.type == NPF_IPV4UC_PREFIX_TABLE_HANDLE_CREATE) ||
                   (data.type == NPF_IPV4UC_NEXT_HOP_TABLE_HANDLE_CREATE)) &&
                   (data.allOK == NPF_TRUE))
               {
                  switch (data.type)
                  {
                     case NPF_IPV4UC_PREFIX_TABLE_HANDLE_CREATE:
                          /* transInfo->tableInfo->intHandle = data.resp->u.prefixTableHandles.intHandle; */
                          transInfo->tableInfo->extHandle = 
                             data.resp->u.prefixTableHandles.extHandle;
                          break;

                     case NPF_IPV4UC_NEXT_HOP_TABLE_HANDLE_CREATE:
                         /* transInfo->tableInfo[IPV4_TABLE_TYPE_FIB].pairTbl.nhTblHndl = data.resp->u.nextHopTableHandle; */
                          break;
                     default:
                          break;
                   }         
               } 
               for (i = 0; i < transInfo->mapTransInfo->numMapTrans; i++)
               {
                  if (transInfo->mapTransInfo->mapTransArray[i]->transPend ==
                        TRUE)
                  {
                     transCompleted = FALSE; 
                     break;
                  } 
               }
               if (transCompleted == TRUE)
               {
                  usrReq = transInfo->usrReq;
                  mapTransInfo = transInfo->mapTransInfo;
                  userInfo = transInfo->usrInfo;
                  userInfo->tableArray[IPV4_TABLE_TYPE_NHOP].tblHndlMap.pfxTableInfo =
                     &userInfo->tableArray[IPV4_TABLE_TYPE_PREFIX];

                  userInfo->tableArray[IPV4_TABLE_TYPE_PREFIX].tblHndlMap.nhopTableInfo =
                     &userInfo->tableArray[IPV4_TABLE_TYPE_NHOP];
                  userInfo->tableArray[IPV4_TABLE_TYPE_FIB].tblHndlMap.pfxTableInfo = 
                     &userInfo->tableArray[IPV4_TABLE_TYPE_PREFIX];
                  userInfo->tableArray[IPV4_TABLE_TYPE_FIB].tblHndlMap.nhopTableInfo = 
                     &userInfo->tableArray[IPV4_TABLE_TYPE_NHOP];

                  tableInfo = &userInfo->tableArray[IPV4_TABLE_TYPE_FIB]; 
              /*    if((retVal = ipv4_AddTableToUsr(userInfo,usrReq->tableType))
	    	                 != NPF_SUCCESS)
	               {
                      npf_logger_Write(LOG_IPV4, LOG_ERR,
	   		           "Error: Table of type %d not created", usrReq->tableType);
	                   RETURN(retVal);
	               } */
                  tableInfo->type = IPV4_TABLE_TYPE_FIB;
                  ipv4_GetHandleFrmMgr(&tableInfo->intHandle); 
	               npf_logger_Write(LOG_IPV4, LOG_TRACE,
	                   "Table of type %d created successfuly", usrReq->tableType);
                  tableInfo->state = TABLE_EMPTY; 

                  if ((retVal=ipv4_DupTransInfo(transInfo,
                          &dstTransInfo)) != NPF_SUCCESS)
                  {
                     npf_logger_Write(LOG_IPV4, LOG_ERR,
                        " Duplication of Transaction Failed");
                     return;
                  }
                  npf_list_pushBack(&userNode->unifiedTransCpList, dstTransInfo);
                  if ((retVal = NPF_IPv4UC_PrefixNextHopTableBind(
                           transInfo->usrReq->callbackHandle, 
                           dstTransInfo->int_correlator, 
                           transInfo->usrReq->appVerbosity, 
                           dstTransInfo->usrInfo->tableArray[IPV4_TABLE_TYPE_PREFIX].intHandle, 
                           dstTransInfo->usrInfo->tableArray[IPV4_TABLE_TYPE_NHOP].intHandle)) != NPF_SUCCESS)
                  { 
                     npf_logger_Write(LOG_IPV4, LOG_ERR,
                        "Call to PrefixNextHopTableBind failed");
                  }
               } 
            }
            npf_list_itrDelete(itr);
            break;
         }
                   

      case  NPF_IPV4UC_PREFIX_TABLE_HANDLE_DELETE:  
      case  NPF_IPV4UC_NEXT_HOP_TABLE_HANDLE_DELETE :
         {
            IPV4MapTransInfo_t *mapTransInfo;
            IPV4UsrReqInfo_t   *usrReq;
            uint32_t           i;
            IPV4UserInfo_t  *userInfo;
            NPF_IPv4UC_ReturnCode_t retCode;

            itr = npf_list_itrCreate(&ipv4Mgr.userList);
            if (itr)
            {
               npf_list_itrFirst(itr);
               do 
               {
                  userNode = npf_list_itrGetData(itr);
                  if (userNode->context == userContext)
                  {
                     found = TRUE;
                     break;
                  }
               } while (npf_list_itrNext(itr) != -1);
            }
            if (found)
            {
               if ((transInfo = npf_list_searchByKey(&userNode->unifiedTransCpList,
                                 (const void*)&correlator)) == NULL)
               {
                  npf_logger_Write(LOG_IPV4, LOG_TRACE,
                       "No transaction with correlator %d found", correlator);
                  return;
               }
              
               transInfo->transPend = FALSE;
               /* allOK must have been initialised with TRUE */
               if (transInfo->mapTransInfo->cbResp.allOK == NPF_TRUE) 
                  transInfo->mapTransInfo->cbResp.allOK = data.allOK;

               for (i = 0; i < transInfo->mapTransInfo->numMapTrans; i++)
               {
                  if (transInfo->mapTransInfo->mapTransArray[i]->transPend ==
                        TRUE)
                  {
                     transCompleted = FALSE; 
                     break;
                  } 
               }
               if (transCompleted == TRUE)
               {
                  usrReq = transInfo->usrReq;
                  mapTransInfo = transInfo->mapTransInfo;
                  userInfo = transInfo->usrInfo;
                  if (data.allOK == NPF_TRUE)
                     retCode = NPF_SUCCESS;
                  else
                     retCode = NPF_FAIL;

                  ipv4_InvokeAppCallback(usrReq, NULL, 
                                   NPF_IPV4UC_FIB_TABLE_HANDLE_DELETE, 
                                   retCode);
                  for (i = 0; i < transInfo->mapTransInfo->numMapTrans; i++)
                  {
                     if((npf_list_removeByKey(&userInfo->unifiedTransCpList, &mapTransInfo->mapTransArray[i]->int_correlator, (void **)&transInfo)) != 0)
                     {
                        npf_logger_Write(LOG_IPV4, LOG_ERR,
                          " Transaction control block not found ");
                     } 
                     PIL_FreeMemory(mapTransInfo->mapTransArray[i]);
                  }
                  PIL_FreeMemory(mapTransInfo);
                  if ((retVal = ipv4_FreeUsrReq(usrReq)) != 
                       NPF_SUCCESS)
                  {
                     npf_logger_Write(LOG_IPV4, LOG_ERR,
                         " Error in deleting User Request from trans. ");
                     return;
                  }
               } 
            }
            npf_list_itrDelete(itr);
         }
            break;

      case  NPF_IPV4UC_PREFIX_TABLE_FLUSH:
      case  NPF_IPV4UC_NEXT_HOP_TABLE_FLUSH :
         {
            IPV4MapTransInfo_t *mapTransInfo;
            IPV4UsrReqInfo_t   *usrReq;
            uint32_t           i;
            NPF_IPv4UC_ReturnCode_t retCode;
            IPV4UserInfo_t  *userInfo;

            itr = npf_list_itrCreate(&ipv4Mgr.userList);
            if (itr)
            {
               npf_list_itrFirst(itr);
               do 
               {
                  userNode = npf_list_itrGetData(itr);
                  if (userNode->context == userContext)
                  {
                     found = TRUE;
                     break;
                  }
               } while (npf_list_itrNext(itr) != -1);
            }
            if (found)
            {
               if ((transInfo = npf_list_searchByKey(&userNode->unifiedTransCpList,
                                 (const void*)&correlator)) == NULL)
               {
                  npf_logger_Write(LOG_IPV4, LOG_TRACE,
                       "No transaction with correlator %d found", correlator);
                  return;
               }
              
               transInfo->transPend = FALSE;
               /* allOK must have been initialised with TRUE */
               if (transInfo->mapTransInfo->cbResp.allOK == NPF_TRUE)
                  transInfo->mapTransInfo->cbResp.allOK = data.allOK;

               for (i = 0; i < transInfo->mapTransInfo->numMapTrans; i++)
               {
                  if (transInfo->mapTransInfo->mapTransArray[i]->transPend ==
                        TRUE)
                  {
                     transCompleted = FALSE; 
                     break;
                  } 
               }
               if (transCompleted == TRUE)
               {
                  transInfo->mapTransInfo->cbResp.type = 
                                   NPF_IPV4UC_FIB_TABLE_FLUSH; 
                  transInfo->mapTransInfo->cbResp.numResp = data.numResp;  
                  usrReq = transInfo->usrReq;
                  mapTransInfo = transInfo->mapTransInfo;
                  userInfo = transInfo->usrInfo;
                 
                  if (mapTransInfo->cbResp.allOK == NPF_TRUE)
                     retCode = NPF_SUCCESS;
                  else
                     retCode = NPF_FAIL;
 
                  ipv4_InvokeAppCallback(usrReq, 
                                &transInfo->mapTransInfo->cbResp, 
                                NPF_IPV4UC_FIB_TABLE_FLUSH, 
                                retCode);
                  
                  for (i = 0; i < transInfo->mapTransInfo->numMapTrans; i++)
                  {
                     if((npf_list_removeByKey(&userInfo->unifiedTransCpList, &mapTransInfo->mapTransArray[i]->int_correlator, (void **)&transInfo)) != 0)
                     {
                        npf_logger_Write(LOG_IPV4, LOG_ERR,
                          " Transaction control block not found ");
                     } 
                     PIL_FreeMemory(mapTransInfo->mapTransArray[i]);
                  }
                  PIL_FreeMemory(mapTransInfo);
                  if ((retVal = ipv4_FreeUsrReq(usrReq)) != 
                       NPF_SUCCESS)
                  {
                     npf_logger_Write(LOG_IPV4, LOG_ERR,
                         " Error in deleting User Request from trans. ");
                     return;
                  }
               } 
            }
            npf_list_itrDelete(itr);
         }
            break;

      case  NPF_IPV4UC_PREFIX_ENTRY_QUERY:
          {
            uint32_t           allPrefixesExisting = 0;
            /* NPF_uint32_t       nhIdArray[MAX_NUM_NHOP];  */
            uint32_t           i;
            IPV4NhopCntxt_t            *nhopCntxt = NULL;
            NPF_IPv4UC_ReturnCode_t retCode;            

            itr = npf_list_itrCreate(&ipv4Mgr.userList);
            if (itr)
            {
               npf_list_itrFirst(itr);
               do 
               {
                  userNode = npf_list_itrGetData(itr);
                  if (userNode->context == userContext)
                  {
                     found = TRUE;
                     break;
                  }
               } while (npf_list_itrNext(itr) != -1);
            }
            npf_list_itrDelete(itr);

            if (found == FALSE)
              return; 

            if ((transInfo = npf_list_searchByKey(&userNode->unifiedTransCpList,
                              (const void*)&correlator)) == NULL)
            {
               npf_logger_Write(LOG_IPV4, LOG_TRACE,
                    "No transaction with correlator %d found", correlator);
               return;
             }
              
            transInfo->transPend = FALSE;
            if ((retVal=ipv4_DupTransInfo(transInfo,
                   &dstTransInfo)) != NPF_SUCCESS)
            {
               npf_logger_Write(LOG_IPV4, LOG_ERR,
                  " Duplication of Transaction Failed");
               return;
            }

            npf_list_pushBack(&userNode->unifiedTransCpList, dstTransInfo); 

            if(dstTransInfo->mapTransInfo->unifReqState == UNIF_ADD_QUERY_PROGRESS)
            { 
               for (i = 0; i < data.numResp; i++)
               {
                  /* TODO: Does nextHopId = 0 means the prefix is not
                     existing?? */
                  /* The following is the algorithm:
                     If all the prefixes are already existing => result is 1
                     If none of the prefixes are existing => result is 2
                     If some are existing and some others are not, we
                     cannot handle and the result is 3 */

                  if (data.resp[i].returnCode == NPF_SUCCESS)
                  {
                     if (allPrefixesExisting == 0)
                        allPrefixesExisting = 1;
                     else
                     {
                        if (allPrefixesExisting != 1)
                        {
                           allPrefixesExisting = 3; 
                           break;
                        }
                     }
                  }
                  else
                  {
                     if (allPrefixesExisting == 0)
                        allPrefixesExisting = 2;
                     else
                     {
                        if (allPrefixesExisting != 2)
                        { 
                           allPrefixesExisting = 3; 
                           break;
                        }
                     }
                  }
               }
               if (allPrefixesExisting == 3)
                  return;

               switch (allPrefixesExisting)
               {
                  /* All prefixes are existing and hence delete them */
                  case 1:
                     dstTransInfo->mapTransInfo->unifReqState 
                        = UNIF_ADD_DELPREFIX_PROGRESS; 
                     if ((retVal = NPF_IPv4UC_PrefixEntryDelete (
                                 transInfo->usrReq->callbackHandle, 
                                 dstTransInfo->int_correlator, 
                                 transInfo->usrReq->appVerbosity, 
                                 dstTransInfo->usrInfo->tableArray[IPV4_TABLE_TYPE_PREFIX].intHandle, 
                                 transInfo->usrReq->prefixCntxt->numEntries, 
                                 transInfo->usrReq->prefixCntxt->prefixArray)) 
                           != NPF_SUCCESS)
                        return;
                     break;

                     /* None of the prefixes are existing and hence add them */
                  case 2:
                     dstTransInfo->mapTransInfo->unifReqState 
                        = UNIF_ADD_ADDNHOP_PROGRESS; 
                     if ((retVal = NPF_IPv4UC_NextHopEntryAdd(
                                 transInfo->usrReq->callbackHandle, 
                                 dstTransInfo->int_correlator, 
                                 transInfo->usrReq->appVerbosity, 
                                 dstTransInfo->usrInfo->tableArray[IPV4_TABLE_TYPE_NHOP].intHandle, 
                                 transInfo->usrReq->nhopCntxt->numEntries,
                                 transInfo->usrReq->nhopCntxt->nhIdArray,              
                                 transInfo->usrReq->nhopCntxt->nhopArray))
                           != NPF_SUCCESS)
                        return;
                     break;

                  default:
                     break;
               }
            }  /* end of FibAdd case */
            else if(dstTransInfo->mapTransInfo->unifReqState == UNIF_FIB_QUERY_PREFIX_PROGRESS)
            {
               /* allocate memory for the context */
               nhopCntxt= (IPV4NhopCntxt_t*) PIL_AllocateMemory(
                  sizeof(IPV4NhopCntxt_t), PIL_MEM_ZERO_INIT);
               if (!nhopCntxt)
               {
                   npf_logger_Write(LOG_IPV4, LOG_ERR, "Memory not allocated ");
                   return;
               } 
               nhopCntxt->numEntries = data.numResp;
               /* Allocate memory for nhIdArray */
               nhopCntxt->nhIdArray = (NPF_uint32_t*) PIL_AllocateMemory(
                 (sizeof(NPF_uint32_t) * data.numResp),PIL_MEM_ZERO_INIT);
               if (!nhopCntxt->nhIdArray)
               {
                  npf_logger_Write(LOG_IPV4, LOG_ERR, "Memory not allocated ");
                  return;
               }
               for (i = 0; i < data.numResp; i++)
                  nhopCntxt->nhIdArray[i] =
                     data.resp[i].u.prefixQueryResult.nextHopIdentifier; 
               nhopCntxt->nhopArray = NULL;
               transInfo->usrReq->nhopCntxt = nhopCntxt;
               dstTransInfo->mapTransInfo->cbResp.numResp = data.numResp;
               dstTransInfo->mapTransInfo->unifReqState =
                    UNIF_FIB_QUERY_NHOP_PROGRESS;
               if ((retVal = NPF_IPv4UC_NextHopEntryQuery(
                           transInfo->usrReq->callbackHandle, 
                           dstTransInfo->int_correlator, 
                           transInfo->usrReq->appVerbosity, 
                           dstTransInfo->usrInfo->tableArray[IPV4_TABLE_TYPE_NHOP].intHandle, 
                           transInfo->usrReq->nhopCntxt->numEntries,
                           transInfo->usrReq->nhopCntxt->nhIdArray)) 
                     != NPF_SUCCESS)
                  return;        

            }
            else
            {  /* begin Fib delete case */
               /* allocate memory for the context */
               if (data.allOK == NPF_FALSE)
	            {
                  if (transInfo->mapTransInfo->cbResp.allOK == NPF_TRUE)
                     transInfo->mapTransInfo->cbResp.allOK = data.allOK;

                  if (transInfo->mapTransInfo->cbResp.allOK == NPF_TRUE)
                     retCode = NPF_SUCCESS;
                  else
                     retCode = NPF_FAIL;
                  transInfo->mapTransInfo->cbResp.numResp = data.numResp;
		            transInfo->mapTransInfo->cbResp.type = 
			            NPF_IPV4UC_FIB_ENTRY_DELETE; 
                  transInfo->mapTransInfo->cbResp.resp = 
			            (NPF_IPv4UC_AsyncResponse_t*)PIL_AllocateMemory(sizeof(NPF_IPv4UC_AsyncResponse_t)* data.numResp, PIL_MEM_ZERO_INIT);
                  memcpy(transInfo->mapTransInfo->cbResp.resp, 
                         data.resp, 
			                sizeof(NPF_IPv4UC_AsyncResponse_t)*data.numResp);
		            ipv4_InvokeAppCallback(transInfo->usrReq, 
				         &transInfo->mapTransInfo->cbResp, 
				         NPF_IPV4UC_FIB_ENTRY_DELETE, 
				         retCode);

		            for (i = 0; i < transInfo->mapTransInfo->numMapTrans; i++)
		            {
		               if((npf_list_removeByKey(&userNode->unifiedTransCpList, 
			               &transInfo->mapTransInfo->mapTransArray[i]->int_correlator, 
			               (void **)&transInfo)) != 0)
		               {
		                   npf_logger_Write(LOG_IPV4, LOG_ERR,
			                " Transaction control block not found ");
		               } 
		               PIL_FreeMemory(transInfo->mapTransInfo->mapTransArray[i]);
		            }
		            if (data.numResp > 0)
                     PIL_FreeMemory(transInfo->mapTransInfo->cbResp.resp);
		            PIL_FreeMemory(transInfo->mapTransInfo);
		            if ((retVal = ipv4_FreeUsrReq(transInfo->usrReq)) != 
				              NPF_SUCCESS)
		            {
			            npf_logger_Write(LOG_IPV4, LOG_ERR,
					      " Error in deleting User Request from trans. ");
			            return;
		            }
		            return; 
               }
             nhopCntxt= (IPV4NhopCntxt_t*) PIL_AllocateMemory(
                  sizeof(IPV4NhopCntxt_t), PIL_MEM_ZERO_INIT);
             if (!nhopCntxt)
             {
                   npf_logger_Write(LOG_IPV4, LOG_ERR, "Memory not allocated ");
                   return;
              } 
               nhopCntxt->numEntries = data.numResp;
               /* Allocate memory for nhIdArray */
               nhopCntxt->nhIdArray = (NPF_uint32_t*) PIL_AllocateMemory(
                 (sizeof(NPF_uint32_t) * data.numResp),PIL_MEM_ZERO_INIT);
               if (!nhopCntxt->nhIdArray)
               {
                  npf_logger_Write(LOG_IPV4, LOG_ERR, "Memory not allocated ");
                  return;
               }
               for (i = 0; i < data.numResp; i++)
                  nhopCntxt->nhIdArray[i] =
                     data.resp[i].u.prefixQueryResult.nextHopIdentifier; 
               nhopCntxt->nhopArray = NULL;
               transInfo->usrReq->nhopCntxt = nhopCntxt;

               dstTransInfo->mapTransInfo->unifReqState =
                    UNIF_DEL_PREFIX_PROGRESS;
 
               if ((retVal = NPF_IPv4UC_PrefixEntryDelete(
                           transInfo->usrReq->callbackHandle, 
                           dstTransInfo->int_correlator, 
                           transInfo->usrReq->appVerbosity, 
                           dstTransInfo->usrInfo->tableArray[IPV4_TABLE_TYPE_PREFIX].intHandle, 
                           transInfo->usrReq->prefixCntxt->numEntries,
                           transInfo->usrReq->prefixCntxt->prefixArray)) 
                     != NPF_SUCCESS)
                  return;        
 
            }
          }
          break;

      case  NPF_IPV4UC_PREFIX_ENTRY_DELETE:
          {
            uint32_t   i;

            itr = npf_list_itrCreate(&ipv4Mgr.userList);
            if (itr)
            {
               npf_list_itrFirst(itr);
               do 
               {
                  userNode = npf_list_itrGetData(itr);
                  if (userNode->context == userContext)
                  {
                     found = TRUE;
                     break;
                  }
               } while (npf_list_itrNext(itr) != -1);
            }
            npf_list_itrDelete(itr);

            if (found == FALSE)
              return; 

            if ((transInfo = npf_list_searchByKey(&userNode->unifiedTransCpList,
                              (const void*)&correlator)) == NULL)
            {
               npf_logger_Write(LOG_IPV4, LOG_TRACE,
                    "No transaction with correlator %d found", correlator);
               return;
             }
              
            transInfo->transPend = FALSE;
            /* allOK must have been initialised with TRUE */
            if (transInfo->mapTransInfo->cbResp.allOK == NPF_TRUE)
               transInfo->mapTransInfo->cbResp.allOK = data.allOK;

            transInfo->mapTransInfo->cbResp.numResp = data.numResp;
            transInfo->mapTransInfo->cbResp.resp = (NPF_IPv4UC_AsyncResponse_t*)PIL_AllocateMemory(sizeof(NPF_IPv4UC_AsyncResponse_t) * data.numResp, PIL_MEM_ZERO_INIT);
            for (i = 0; i < data.numResp; i++)
               transInfo->mapTransInfo->cbResp.resp[i].returnCode = 
                              data.resp[i].returnCode; 


            if ((retVal=ipv4_DupTransInfo(transInfo,
                   &dstTransInfo)) != NPF_SUCCESS)
            {
               npf_logger_Write(LOG_IPV4, LOG_ERR,
                  " Duplication of Transaction Failed");
               return;
            }

            npf_list_pushBack(&userNode->unifiedTransCpList, dstTransInfo); 

            if ( dstTransInfo->mapTransInfo->unifReqState == UNIF_ADD_DELPREFIX_PROGRESS)
            {
               dstTransInfo->mapTransInfo->unifReqState 
                  = UNIF_ADD_ADDNHOP_PROGRESS; 
               if ((retVal = NPF_IPv4UC_NextHopEntryAdd(
                           transInfo->usrReq->callbackHandle, 
                           dstTransInfo->int_correlator, 
                           transInfo->usrReq->appVerbosity, 
                           dstTransInfo->usrInfo->tableArray[IPV4_TABLE_TYPE_NHOP].intHandle, 
                           transInfo->usrReq->nhopCntxt->numEntries,
                           transInfo->usrReq->nhopCntxt->nhIdArray,              
                           transInfo->usrReq->nhopCntxt->nhopArray))
                     != NPF_SUCCESS)
                  return;
            }
            else
            {
               dstTransInfo->mapTransInfo->unifReqState 
                  = UNIF_DEL_NHOP_PROGRESS; 
               if ((retVal = NPF_IPv4UC_NextHopEntryDelete(
                              transInfo->usrReq->callbackHandle, 
                              dstTransInfo->int_correlator, 
                              transInfo->usrReq->appVerbosity, 
                     dstTransInfo->usrInfo->tableArray[IPV4_TABLE_TYPE_NHOP].intHandle, 
                              transInfo->usrReq->nhopCntxt->numEntries,
                              transInfo->usrReq->nhopCntxt->nhIdArray)) 
                                      != NPF_SUCCESS)
                                 return;     
 
            } 
          }
          break;

      case  NPF_IPV4UC_NEXT_HOP_ENTRY_ADD :
          {
            uint32_t   i;

            itr = npf_list_itrCreate(&ipv4Mgr.userList);
            if (itr)
            {
               npf_list_itrFirst(itr);
               do 
               {
                  userNode = npf_list_itrGetData(itr);
                  if (userNode->context == userContext)
                  {
                     found = TRUE;
                     break;
                  }
               } while (npf_list_itrNext(itr) != -1);
            }
            npf_list_itrDelete(itr);

            if (found == FALSE)
              return; 

            if ((transInfo = npf_list_searchByKey(&userNode->unifiedTransCpList,
                              (const void*)&correlator)) == NULL)
            {
               npf_logger_Write(LOG_IPV4, LOG_TRACE,
                    "No transaction with correlator %d found", correlator);
               return;
             }
              
            transInfo->transPend = FALSE;
            /* allOK must have been initialised with TRUE */
            if (transInfo->mapTransInfo->cbResp.allOK == NPF_TRUE)
               transInfo->mapTransInfo->cbResp.allOK = data.allOK;

            transInfo->mapTransInfo->cbResp.numResp = data.numResp;
            transInfo->mapTransInfo->cbResp.resp = (NPF_IPv4UC_AsyncResponse_t*)PIL_AllocateMemory(sizeof(NPF_IPv4UC_AsyncResponse_t) * data.numResp, PIL_MEM_ZERO_INIT);
            for (i = 0; i < data.numResp; i++)
               transInfo->mapTransInfo->cbResp.resp[i].returnCode = 
                              data.resp[i].returnCode; 

            if ((retVal=ipv4_DupTransInfo(transInfo,
                   &dstTransInfo)) != NPF_SUCCESS)
            {
               npf_logger_Write(LOG_IPV4, LOG_ERR,
                  " Duplication of Transaction Failed");
               return;
            }
            npf_list_pushBack(&userNode->unifiedTransCpList, dstTransInfo); 

            dstTransInfo->mapTransInfo->unifReqState 
                               = UNIF_ADD_ADDPREFIX_PROGRESS; 

            if ((retVal = NPF_IPv4UC_PrefixEntryAdd(
                    transInfo->usrReq->callbackHandle, 
                    dstTransInfo->int_correlator, 
                    transInfo->usrReq->appVerbosity, 
                    dstTransInfo->usrInfo->tableArray[IPV4_TABLE_TYPE_PREFIX].intHandle, 
                    transInfo->usrReq->prefixCntxt->numEntries,
                    transInfo->usrReq->prefixCntxt->prefixArray,              
                    transInfo->usrReq->nhopCntxt->nhIdArray))
                                 != NPF_SUCCESS)
                            return;

          }
          break;
 
      case  NPF_IPV4UC_PREFIX_ENTRY_ADD:
          {
            IPV4MapTransInfo_t *mapTransInfo;
            IPV4UsrReqInfo_t   *usrReq;
            uint32_t           i;
            NPF_IPv4UC_ReturnCode_t retCode;            

            itr = npf_list_itrCreate(&ipv4Mgr.userList);
            if (itr)
            {
               npf_list_itrFirst(itr);
               do 
               {
                  userNode = npf_list_itrGetData(itr);
                  if (userNode->context == userContext)
                  {
                     found = TRUE;
                     break;
                  }
               } while (npf_list_itrNext(itr) != -1);
            }
            npf_list_itrDelete(itr);
 
            if (found == FALSE)
              return; 

            if ((transInfo = npf_list_searchByKey(&userNode->unifiedTransCpList,
                              (const void*)&correlator)) == NULL)
            {
               npf_logger_Write(LOG_IPV4, LOG_TRACE,
                    "No transaction with correlator %d found", correlator);
               return;
             }
              
            transInfo->transPend = FALSE;
            usrReq = transInfo->usrReq;
            mapTransInfo = transInfo->mapTransInfo;
            if (mapTransInfo->cbResp.allOK == NPF_TRUE)
               mapTransInfo->cbResp.allOK = data.allOK;
            if(mapTransInfo->cbResp.numResp != data.numResp)
            {
               npf_logger_Write(LOG_IPV4, LOG_TRACE,
                    "No of responses are not matching");
               return;
            }
            for (i = 0; i < data.numResp; i++)
            {
               if(transInfo->mapTransInfo->cbResp.resp[i].returnCode ==
                         NPF_SUCCESS) 
                  transInfo->mapTransInfo->cbResp.resp[i].returnCode = 
                     data.resp[i].returnCode; 
            } 

            if (mapTransInfo->cbResp.allOK == NPF_TRUE)
               retCode = NPF_SUCCESS;
            else
               retCode = NPF_FAIL;
 
            ipv4_InvokeAppCallback(usrReq, &mapTransInfo->cbResp, 
                             NPF_IPV4UC_FIB_ENTRY_ADD, 
                             retCode);

            for (i = 0; i < transInfo->mapTransInfo->numMapTrans; i++)
            {
               if((npf_list_removeByKey(&userNode->unifiedTransCpList, &mapTransInfo->mapTransArray[i]->int_correlator, (void **)&transInfo)) != 0)
               {
                   npf_logger_Write(LOG_IPV4, LOG_ERR,
                     " Transaction control block not found ");
               } 

               PIL_FreeMemory(mapTransInfo->mapTransArray[i]);
            }
            if (mapTransInfo->cbResp.resp)
              PIL_FreeMemory(mapTransInfo->cbResp.resp);
            PIL_FreeMemory(mapTransInfo);
            if ((retVal = ipv4_FreeUsrReq(usrReq)) != 
                 NPF_SUCCESS)
            {
               npf_logger_Write(LOG_IPV4, LOG_ERR,
                   " Error in deleting User Request from trans. ");
               return;
            }
          }
          break;

      case  NPF_IPV4UC_NEXT_HOP_ENTRY_DELETE :
         {
            IPV4MapTransInfo_t *mapTransInfo;
            IPV4UsrReqInfo_t   *usrReq;
            uint32_t            i;
            NPF_IPv4UC_ReturnCode_t retCode;

            itr = npf_list_itrCreate(&ipv4Mgr.userList);
            if (itr)
            {
               npf_list_itrFirst(itr);
               do 
               {
                  userNode = npf_list_itrGetData(itr);
                  if (userNode->context == userContext)
                  {
                     found = TRUE;
                     break;
                  }
               } while (npf_list_itrNext(itr) != -1);
            }
            npf_list_itrDelete(itr);
 
            if (found == FALSE)
              return; 

            if ((transInfo = npf_list_searchByKey(&userNode->unifiedTransCpList,
                              (const void*)&correlator)) == NULL)
            {
               npf_logger_Write(LOG_IPV4, LOG_TRACE,
                    "No transaction with correlator %d found", correlator);
               return;
             }
              
            transInfo->transPend = FALSE;
            usrReq = transInfo->usrReq;
            mapTransInfo = transInfo->mapTransInfo;
            if (mapTransInfo->cbResp.allOK == NPF_TRUE)
               mapTransInfo->cbResp.allOK = data.allOK;
            if(mapTransInfo->cbResp.numResp != data.numResp)
            {
               npf_logger_Write(LOG_IPV4, LOG_TRACE,
                    "No of responses are not matching");
               return;
            }
            for (i = 0; i < data.numResp; i++)
            {
               if(transInfo->mapTransInfo->cbResp.resp[i].returnCode ==
                         NPF_SUCCESS) 
                  transInfo->mapTransInfo->cbResp.resp[i].returnCode = 
                     data.resp[i].returnCode; 
            } 
            if (mapTransInfo->cbResp.allOK == NPF_TRUE)
               retCode = NPF_SUCCESS;
            else
               retCode = NPF_FAIL;

            ipv4_InvokeAppCallback(usrReq, &mapTransInfo->cbResp, 
                                 NPF_IPV4UC_FIB_ENTRY_DELETE, 
                                 retCode);

            for (i = 0; i < transInfo->mapTransInfo->numMapTrans; i++)
            {
               if((npf_list_removeByKey(&userNode->unifiedTransCpList, 
                   &mapTransInfo->mapTransArray[i]->int_correlator, 
                   (void **)&transInfo)) != 0)
               {
                   npf_logger_Write(LOG_IPV4, LOG_ERR,
                     " Transaction control block not found ");
               } 
               PIL_FreeMemory(mapTransInfo->mapTransArray[i]);
            }
            if (mapTransInfo->cbResp.resp)
               PIL_FreeMemory(mapTransInfo->cbResp.resp);
            PIL_FreeMemory(mapTransInfo);
            if ((retVal = ipv4_FreeUsrReq(usrReq)) != 
                 NPF_SUCCESS)
            {
               npf_logger_Write(LOG_IPV4, LOG_ERR,
                   " Error in deleting User Request from trans. ");
               return;
            }
         }
         break;
  
      case  NPF_IPV4UC_PREFIX_TABLE_ATTRIBUTE_QUERY:
         break;
 
      case  NPF_IPV4UC_PREFIX_NEXT_HOP_TABLE_BIND:
         {
            IPV4MapTransInfo_t *mapTransInfo;
            IPV4UsrReqInfo_t   *usrReq;
            uint32_t           i;
            IPV4UserInfo_t  *userInfo;
            NPF_IPv4UC_ReturnCode_t retCode;
 
            itr = npf_list_itrCreate(&ipv4Mgr.userList);
            if (itr)
            {
               npf_list_itrFirst(itr);
               do 
               {
                  userNode = npf_list_itrGetData(itr);
                  if (userNode->context == userContext)
                  {
                     found = TRUE;
                     break;
                  }
               } while (npf_list_itrNext(itr) != -1);
            }
            if (found)
            {
               if ((transInfo = npf_list_searchByKey(&userNode->unifiedTransCpList,
                                 (const void*)&correlator)) == NULL)
               {
                  npf_logger_Write(LOG_IPV4, LOG_TRACE,
                       "No transaction with correlator %d found", correlator);
                  return;
               }
              
               transInfo->transPend = FALSE;
               /* allOk must have been initialised with TRUE */
               if (transInfo->mapTransInfo->cbResp.allOK == NPF_TRUE)
                  transInfo->mapTransInfo->cbResp.allOK = data.allOK;
 
               for (i = 0; i < transInfo->mapTransInfo->numMapTrans; i++)
               {
                  if (transInfo->mapTransInfo->mapTransArray[i]->transPend ==
                        TRUE)
                  {
                     transCompleted = FALSE; 
                     break;
                  } 
               }
               if (transCompleted == TRUE)
               {
                  usrReq = transInfo->usrReq;
                  mapTransInfo = transInfo->mapTransInfo;
                  userInfo = transInfo->usrInfo;
               
                  if (data.allOK == NPF_TRUE)
                     retCode = NPF_SUCCESS;
                  else
                     retCode = NPF_FAIL; 
 
                  ipv4_InvokeAppCallback(usrReq, NULL, 
                                       NPF_IPV4UC_FIB_TABLE_HANDLE_CREATE, 
                                       retCode);
                  for (i = 0; i < mapTransInfo->numMapTrans; i++)
                  {
                     if((npf_list_removeByKey(&userInfo->unifiedTransCpList, &mapTransInfo->mapTransArray[i]->int_correlator, (void **)&transInfo)) != 0)
                     {
                        npf_logger_Write(LOG_IPV4, LOG_ERR,
                          " Transaction control block not found ");
                     } 
                     PIL_FreeMemory(mapTransInfo->mapTransArray[i]);
                  }
                  PIL_FreeMemory(mapTransInfo);
                  usrReq->transInfo = NULL;
                  if ((retVal = ipv4_FreeUsrReq(usrReq)) != 
                       NPF_SUCCESS)
                  {
                     npf_logger_Write(LOG_IPV4, LOG_ERR,
                         " Error in deleting User Request from trans. ");
                     return;
                  }
               } 
            }
            npf_list_itrDelete(itr);
            break;
         }

      case  NPF_IPV4UC_NEXT_HOP_ENTRY_QUERY :
          {
            IPV4MapTransInfo_t *mapTransInfo;
            IPV4UsrReqInfo_t   *usrReq;
            uint32_t           i;
            NPF_IPv4UC_ReturnCode_t retCode;            

            itr = npf_list_itrCreate(&ipv4Mgr.userList);
            if (itr)
            {
               npf_list_itrFirst(itr);
               do 
               {
                  userNode = npf_list_itrGetData(itr);
                  if (userNode->context == userContext)
                  {
                     found = TRUE;
                     break;
                  }
               } while (npf_list_itrNext(itr) != -1);
            }
            npf_list_itrDelete(itr);
 
            if (found == FALSE)
              return; 

            if ((transInfo = npf_list_searchByKey(&userNode->unifiedTransCpList,
                              (const void*)&correlator)) == NULL)
            {
               npf_logger_Write(LOG_IPV4, LOG_TRACE,
                    "No transaction with correlator %d found", correlator);
               return;
             }
              
            transInfo->transPend = FALSE;
            usrReq = transInfo->usrReq;
            mapTransInfo = transInfo->mapTransInfo;
            if (mapTransInfo->cbResp.allOK == NPF_TRUE)
               mapTransInfo->cbResp.allOK = data.allOK;
          /*  if(mapTransInfo->cbResp.numResp != data.numResp)
            {
               npf_logger_Write(LOG_IPV4, LOG_TRACE,
                    "No of responses are not matching");
               return;
            } */
            mapTransInfo->cbResp.type = NPF_IPV4UC_FIB_ENTRY_QUERY; 
            mapTransInfo->cbResp.numResp = data.numResp;
            mapTransInfo->cbResp.resp = (NPF_IPv4UC_AsyncResponse_t*)PIL_AllocateMemory(sizeof(NPF_IPv4UC_AsyncResponse_t)*data.numResp, PIL_MEM_ZERO_INIT);
            for (i = 0; i < data.numResp; i++)
            {
               mapTransInfo->cbResp.resp[i].u.fibQueryResult.nextHopArray.nextHopCount = data.resp[i].u.nextHopQueryResult.nextHopArray.nextHopCount;
               mapTransInfo->cbResp.resp[i].u.fibQueryResult.nextHopArray.nextHopArray = 
              (NPF_IPv4UC_NextHop_t *)PIL_AllocateMemory(sizeof(NPF_IPv4UC_NextHop_t)*data.resp[i].u.nextHopQueryResult.nextHopArray.nextHopCount, 
                 PIL_MEM_ZERO_INIT);
	           if (data.resp[i].u.nextHopQueryResult.nextHopArray.nextHopCount > 0)
                  memcpy(mapTransInfo->cbResp.resp[i].u.fibQueryResult.nextHopArray.nextHopArray, 
                  data.resp[i].u.nextHopQueryResult.nextHopArray.nextHopArray, 
                  sizeof(NPF_IPv4UC_NextHop_t)*data.resp[i].u.nextHopQueryResult.nextHopArray.nextHopCount);

               memcpy(&mapTransInfo->cbResp.resp[i].u.fibQueryResult.prefix,
                      &usrReq->prefixCntxt->prefixArray[i],
                      sizeof(NPF_IPv4UC_Prefix_t)); 
          
               mapTransInfo->cbResp.resp[i].returnCode = 
                  data.resp[i].returnCode;
            } 
            

            if (mapTransInfo->cbResp.allOK == NPF_TRUE)
               retCode = NPF_SUCCESS;
            else
               retCode = NPF_FAIL;
 
            ipv4_InvokeAppCallback(usrReq, &mapTransInfo->cbResp, 
                             NPF_IPV4UC_FIB_ENTRY_QUERY, 
                             retCode);

            for (i = 0; i < transInfo->mapTransInfo->numMapTrans; i++)
            {
               if((npf_list_removeByKey(&userNode->unifiedTransCpList, &mapTransInfo->mapTransArray[i]->int_correlator, (void **)&transInfo)) != 0)
               {
                   npf_logger_Write(LOG_IPV4, LOG_ERR,
                     " Transaction control block not found ");
               } 

               PIL_FreeMemory(mapTransInfo->mapTransArray[i]);
            }
            if(mapTransInfo->cbResp.numResp)
               for (i = 0; i < mapTransInfo->cbResp.numResp; i++)
               {
                  if(mapTransInfo->cbResp.resp[i].u.fibQueryResult.nextHopArray.nextHopArray)
                     PIL_FreeMemory(mapTransInfo->cbResp.resp[i].u.fibQueryResult.nextHopArray.nextHopArray);
               }
            if (mapTransInfo->cbResp.resp)
               PIL_FreeMemory(mapTransInfo->cbResp.resp);
            PIL_FreeMemory(mapTransInfo);
            if ((retVal = ipv4_FreeUsrReq(usrReq)) != 
                 NPF_SUCCESS)
            {
               npf_logger_Write(LOG_IPV4, LOG_ERR,
                   " Error in deleting User Request from trans. ");
               return;
            }
          }
          break;

      case  NPF_IPV4UC_FIB_ENTRY_QUERY :
      case  NPF_IPV4UC_NEXT_HOP_TABLE_ATTRIBUTE_QUERY :
      case  NPF_IPV4UC_FIB_TABLE_HANDLE_CREATE :
      case  NPF_IPV4UC_FIB_TABLE_HANDLE_DELETE :
      case  NPF_IPV4UC_FIB_ENTRY_ADD :
      case  NPF_IPV4UC_FIB_ENTRY_DELETE :
      case  NPF_IPV4UC_FIB_TABLE_FLUSH :
      case  NPF_IPV4UC_FIB_TABLE_ATTRIBUTE_QUERY :
      case  NPF_IPV4UC_ADDRESS_RES_TABLE_HANDLE_CREATE :
      case  NPF_IPV4UC_ADDRESS_RES_TABLE_HANDLE_DELETE :
      case  NPF_IPV4UC_ADDRESS_RES_ENTRY_ADD :
      case  NPF_IPV4UC_ADDRESS_RES_ENTRY_DELETE :
      case  NPF_IPV4UC_ADDRESS_RES_TABLE_FLUSH :
      case  NPF_IPV4UC_ADDRESS_RES_TABLE_ATTRIBUTE_QUERY :
      case  NPF_IPV4UC_ADDRESS_RES_ENTRY_QUERY:
   }
}
#endif



