/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/


/*****************************************************************************
* $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/CoreBlocks/ConfigManagment/inc/cm.h,v $
* $Revision: 1.6 $
* $Date: 2003/11/17 16:57:12 $
*
* Description:
*
*    The functions and variables here are only for C&M component,
*    Any other component in PDK and external application are _NEVER_
*    include this file nor accessing the variables here.
*
* TODO:
*    1. Add parameter name in function prototype
*
*****************************************************************************/

#ifndef _CM_H_
#define _CM_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "npf_cm_api.h"
#include "pdk_cm.h"
#include "pdk_cm_types.h"
#include "list.h"

/* ---------------------------- C&M Data Type ----------------------------- */
typedef NPF_USERCONTEXT CMCONTEXT;

/* All the possible CM client context, which is the client cookie in
 * CM usercontext pass in to FP plugin at the register time */
typedef enum {	CM_CC_EVENT_FEBIND,
		CM_CC_EVENT_FEUNBIND,
		CM_CC_EVENT_LINKUP,
		CM_CC_EVENT_LINKDOWN,
		CM_CC_IF_SETIPADDR,
		CM_CC_IF_DELIPADDR,
		CM_CC_IF_SETIPv6ADDR,
		CM_CC_IF_DELIPv6ADDR,
		CM_CC_IF_SETIPFWD,
		CM_CC_IF_SETPORTSTATUS,
		CM_CC_IF_GETSTATS,
		CM_CC_ICMP_GETSTATS,
		CM_CC_PORT_SETMTU,
		CM_CC_PORT_SETLINKSPEED,
		CM_CC_PORT_GETSTATS,
		CM_CC_PORT_SETVPIBITS,
		CM_CC_PORT_SETVCIBITS,
                CM_CC_MAXNUM
} CMCLIENTCONTEXT;

#define REQUEST_FE_DONE		-1   /* put in CMCORRELATOR.waitFEid[] */
#define REQUEST_FE_FAIL		-2   /* to indicate the pending FE status */
#define REQUEST_FE_UNBIND	-3   /* if one port FAIL then whole FE FAIL */
typedef struct cmCorrelator_tag {
	NPF_HANDLE	handle;
	NPF_CBHANDLE	cbid;
	NPF_CORRELATOR	correlator;
	NPF_VERBOSITY	verbosity;
        int             cbtype;    /* abstract callback type */
	NPF_DATA	data;      /* save "set" data to onCB time */
        int             numSentFE; /* total num of sent FE */
        FPPI_FEID       waitFEid[MAXFENUM]; /* waiting FE's feid array */
} *CMCORRELATOR;


/* ------------------------- C&M Helper Functions ------------------------- */
int cm_getDataByHandle(NPF_HANDLE, NPF_DATA *);
int cm_copyFromPDK(NPF_HANDLE, NPF_DATA);
int cm_copyToPDK(NPF_HANDLE, NPF_DATA);
int cm_removeFromRequest(CMCORRELATOR);
int cm_requestFailCB(CMCORRELATOR);
int cm_deleteIFEvent(void);

/* ------------------------- FP Plugin onCallBack ------------------------- */
/*
 *    Note: Rename the void * to whatever defines in pdk_fpplugin.h
 */
void onCB_event_FEBind(uint32_t, CMCONTEXT);
void onCB_event_FEUnBind(FPPI_FEID, FPPI_CORRELATOR, FPPI_CONTEXT, void *);
void onCB_event_PortLinkUp(FPPI_FEID, FPPI_CORRELATOR, FPPI_CONTEXT, void *);
void onCB_event_PortLinkDown(FPPI_FEID,FPPI_CORRELATOR,FPPI_CONTEXT,void *);
void onCB_if_set_ipaddr(uint32_t, CMCORRELATOR, CMCONTEXT, NPF_DATA);
void onCB_if_del_ipaddr(uint32_t, CMCORRELATOR, CMCONTEXT, NPF_DATA);
void onCB_if_set_ipv6addr(uint32_t, CMCORRELATOR, CMCONTEXT, NPF_DATA);
void onCB_if_del_ipv6addr(uint32_t, CMCORRELATOR, CMCONTEXT, NPF_DATA);
void onCB_if_set_ipfwd(uint32_t, CMCORRELATOR, CMCONTEXT, NPF_DATA);
void onCB_if_set_portstatus(uint32_t, CMCORRELATOR, CMCONTEXT, NPF_DATA);
void onCB_if_get_statistics(uint32_t, CMCORRELATOR, CMCONTEXT, NPF_DATA);
void onCB_icmp_get_statistics(uint32_t, CMCORRELATOR, CMCONTEXT, NPF_DATA);
void onCB_port_set_mtu(uint32_t, CMCORRELATOR, CMCONTEXT, NPF_DATA);
void onCB_port_set_linkspeed(uint32_t, CMCORRELATOR, CMCONTEXT, NPF_DATA);
void onCB_port_get_statistics(uint32_t, CMCORRELATOR, CMCONTEXT, NPF_DATA);

void onCB_port_set_vpibits(uint32_t, CMCORRELATOR, CMCONTEXT, NPF_DATA);
void onCB_port_set_vcibits(uint32_t, CMCORRELATOR, CMCONTEXT, NPF_DATA);

/* ------------------------ onCallBack by C&M onCB ------------------------ */
void onCB_event_FE(NPF_HANDLE feh, int isInternal, NPF_EVENT);
void onCB_event_IF(NPF_HANDLE ifh, int isInternal, NPF_EVENT); 
void onCB_event_PORT(NPF_HANDLE porth, int isInternal, NPF_EVENT); 

/* ------------------------------ Global Data ----------------------------- */
extern PilCriticalSection g_cmlock;
extern DList              g_outstandingRequest;

#define CM_LOCK()	PIL_EnterCriticalSection(&g_cmlock);
#define CM_UNLOCK()	PIL_LeaveCriticalSection(&g_cmlock);

extern DList	CMRegFppCBIdList;
extern int searchFPPCBId(CMCLIENTCONTEXT tag, NPF_CBHANDLE *fppcbid);

extern NPF_RET cm_check_init(void);

#ifdef __cplusplus
}
#endif

#endif /* _CM_H_ */
