/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/


/*****************************************************************************
* $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/CoreBlocks/ConfigManagment/src/cm.c,v $
* $Revision: 1.6 $
* $Date: 2003/11/17 16:57:12 $
*
* Description:
*     This file is the C&M implementation part. 
*
*****************************************************************************/

#include <string.h>
#include <stdio.h>
#include <assert.h>
#include "npf_ns_api.h"
#include "pdk_callback.h"
#include "fppapi.h"
#include "pdkmgr.h"
#include "cm.h"
#include "log.h"
#include "pdk_ns_helper.h"

/*****************************************************************************
*                                                                            *
*  --------------------------- Static Local Data --------------------------  *
*                                                                            *
*  Can not be accessed in files internal.c and callback.c even if they're    *
*   part of CM                                                               *
*                                                                            *
*****************************************************************************/

static uint32_t	g_initialized = FALSE;

/* available handle count, never -- */
static int	HandleCounter = 16682;

/* keep track of the number of instance */
static int      g_InstanceNum = 0;

/* Internal object list, IF - interface, RT - RouteTable, AT - ARPTable,
 * IC - INTConfig */
static DList	SYSTEMList,
                FEList, 
                PortList, 
                IPRouterList, 
                IFList,
		RTList, 
                PreList, 
                NHList, 
                ATList, 
                ICList;

/* B&D callback handle for FE_Bind event */
static NPF_CBHANDLE	g_regbdcbid = (NPF_CBHANDLE)(-1);

/* Registered CMClientContext to FPPAPI cbid mapping */
FPPI_CBHANDLE	CMRegFppCBId[CM_CC_MAXNUM];

static DList	*ObjArray[] = { 
                              &SYSTEMList, 
                              &FEList, 
                              &PortList, 
                              &IPRouterList,
                              &IFList, 
                              &RTList, 
                              &PreList, 
                              &NHList, 
                              &ATList, 
                              &ICList, 
                            };

/* number of the instance type */
#define NUMOBJARRAY	sizeof(ObjArray)/sizeof(&ObjArray)

/* check if component initialized */
#define CHK_INIT() \
    if (!g_initialized) {\
        npf_logger_write(LOG_CM, LOG_ERR, "C&M is not initialized");\
        return NPF_COMPONENT_UNINITIALIZED;\
    }


/* --------------------------- C&M Global Data --------------------------  */
PilCriticalSection g_cmlock;
DList              g_outstandingRequest;  /* not yet been callback to app. */

/*****************************************************************************
*                                                                            *
*  --------------------------- Private Interface --------------------------  *
*                                                                            *
*  Only C&M itself can access the private interface, MUST always in CM_LOCK  *
*                                                                            *
*****************************************************************************/

/*****************************************************************************
*    Name: cm_getDataByHandle
*
*    Description:
*        Search for the instance by handle, return the actual data pointer
*        and caller needs cast it to proper data structure.
*        ONLY C&M can use this call because it give caller the pointer of
*        the data and C&M NEVER free the ptr.
*        Free data should use internal API cm_delDataByHandle
*
*    Parameters:
*        IN : handle - any NPF_HANDLE to real instance in C&M
*        OUT: ppdata - the pointer to the found data
*
*    Preconditions:
*        CM_LOCK
*        ppdata is double pointer to void
*
*    Postconditions:
*        if found, ppdata points to real data stored in C&M
*
*    Returns:
*        0 if the data was found, -1 otherwise.
*
*/
int cm_getDataByHandle(NPF_HANDLE handle, NPF_DATA *ppdata)
{
    int        i, result;
    PDK_SHIM_t *shim;
    DList      *list;
    DListItr   *itr;

    assert(ppdata);

    result = 0;
    for (i=0; i<NUMOBJARRAY; i++) {
        list = ObjArray[i];
        if ((itr = npf_list_itrCreate(list)) == NULL)
            continue;
        if (npf_list_itrFirst(itr) != -1) {
            do {
                *ppdata = npf_list_itrGetData(itr);
                shim = (PDK_SHIM_t *)(*ppdata);
                if (shim->handle == handle) {
                    result = 1;
                    break;
                }
            } while (npf_list_itrNext(itr) != -1);
        }
        npf_list_itrDelete(itr);
        if (result) return 0;
    }

    return -1;
}

/*****************************************************************************
*    Name: cm_copyFromPDK
*
*    Description:
*        Search for the instance by handle, copy the internal PDK object
*        instance to a paas-in, pre-allocated memory. Does not copy the
*        PDK meta data. Caller can do anything to the new copy.
*
*    Parameters:
*        IN : handle - any NPF_HANDLE to real instance in C&M
*        OUT: data   - the pointer to the new data copy
*
*    Preconditions:
*        CM_LOCK
*        data is ptr to pre-allocated memory and the caller knows the
*        type and the size of the data.
*
*    Postconditions:
*        data points to new copy of the data
*
*    Returns:
*        0 if the data was found, -1 otherwise.
*
*/
int cm_copyFromPDK(NPF_HANDLE handle, NPF_DATA data)
{
    NPF_DATA       sysdata;
    PDK_SHIM_t     *shim;
    PDK_FE_t       *pdkfe;
    NPF_FE_t       *npffe;

    /* This should be always in LOCKED area */
    assert(data);

    if (cm_getDataByHandle(handle, &sysdata) == -1)
        return -1;

    shim = (PDK_SHIM_t *)sysdata;

    switch (shim->type) {
    case NPF_FE:
        pdkfe = (PDK_FE_t *)sysdata;
        npffe = (NPF_FE_t *)data;
        memcpy(&npffe->id, &pdkfe->id, sizeof(uint32_t));
        memcpy(&npffe->attribs, &pdkfe->attribs, sizeof(PDK_FE_ATTRIBS));
        /* NPF_FE_CAPS != PDK_FE_CAPS */
        memcpy(&npffe->caps, &pdkfe->caps, sizeof(NPF_FE_CAPS));
        break;
    case NPF_IPROUTER:
        memcpy(data, (char *)sysdata+sizeof(PDK_SHIM_t), 
               sizeof(NPF_IPROUTER_t));
        break;
    case NPF_INTERFACE:
        memcpy(data, (char *)sysdata+sizeof(PDK_SHIM_t),
               sizeof(NPF_INTERFACE_t));
        break;
    case NPF_PORT:
        memcpy(data, (char *)sysdata+sizeof(PDK_SHIM_t),
               sizeof(NPF_PORT_t));
        break;
    default:
        npf_logger_write(LOG_CM, LOG_ERR, "Not support type in cm_copyFromPDK");
        return -1;
    }
    return 0;
}

/*****************************************************************************
*    Name: cm_copyToPDK
*
*    Description:
*        Search for the instance by handle, copy the pass-in, pre-assigned
*        external data to internal PDK instance. Keep the same meta data
*        in internal instance.
*
*    Parameters:
*        IN: handle - any NPF_HANDLE to real instance in C&M
*        IN: data   - the pointer to the pass-in data
*
*    Preconditions:
*        CM_LOCK
*        data is ptr to pre-assigned instance
*
*    Postconditions:
*        internal instance is changed and caller can free the pass-in data
*
*    Returns:
*        0 if the data was found, -1 otherwise.
*
*/
int cm_copyToPDK(NPF_HANDLE handle, NPF_DATA data)
{
    NPF_DATA   sysdata;
    PDK_SHIM_t *shim;

    /* This should be always in LOCKED area */
    assert(data);

    if (cm_getDataByHandle(handle, &sysdata) == -1)
        return -1;

    shim = (PDK_SHIM_t *)sysdata;

    switch(shim->type) {
    case NPF_IPROUTER:
        memcpy((char *)sysdata+sizeof(PDK_SHIM_t),
                               data, sizeof(NPF_IPROUTER_t));
        break;
    default:
        npf_logger_write(LOG_CM, LOG_ERR, "Not support type in cm_copyToPDK");
        return -1;
    }

    return 0;
}

/*****************************************************************************
*    Name: cm_removeFromRequest
*
*    Description:
*        Remove the cmcorrelator from the g_outstandingRequest. Just
*        remove from the list but not free the correlator.
*
*    Parameters:
*        IN: cmcorrelator - structure contains all the info of one request
*
*    Preconditions:
*        CM_UNLOCK
*        The request maybe failed, either by FEUnBind or cm_shutdown or
*        FE response failure or FPPAPI request call fail.
*        The request might be removed because of the successful callback
*        from FE.
*
*    Postconditions:
*        Request(cmcorrelator) has been removed from g_outstandingRequest
*
*    Returns:
*        0 if the data was found, -1 otherwise.
*
*/
int cm_removeFromRequest(CMCORRELATOR correlator)
{
    int          result;
    DListItr     *itr;
    CMCORRELATOR tmp_correlator, temp;

    /* this should be always in the LOCK area */
    assert(correlator);

    CM_LOCK();

    result = 0;
    if ((itr = npf_list_itrCreate(&g_outstandingRequest)) == NULL)
        return -1;
    if (npf_list_itrFirst(itr) != -1) {
        do {
            tmp_correlator = (CMCORRELATOR)npf_list_itrGetData(itr);
            if (tmp_correlator == correlator) {
                result = 1;
                /* the correlator will be freed the the caller */
                npf_list_itrRemove(itr, (NPF_DATA *)&temp);
                break;
            }
        } while (npf_list_itrNext(itr) != -1);
    }
    npf_list_itrDelete(itr);

    CM_UNLOCK();

    if (!result)
        return -1;
    else
        return 0;

}

/* HACK for cmcallback.c: onCB_event_FEUnBind to fire IF_DELETE
 * event on the left-over interface. See cmcallback.c:onCB_event_FEUnBind
 * for detail. To prevent user explicitly delete IF-PORT association
 * before FE shutdown.
 * The HACK doesn't work in Multiple FE cases */
int cm_deleteIFEvent(void)
{

    DListItr *itr;
    PDK_INTERFACE_t *interface;
    
    CHK_INIT();
    CM_LOCK();
    /* linear search IFList */
    if ((itr = npf_list_itrCreate(&IFList)) == NULL) {
        CM_UNLOCK();
        return NPF_INVALID_PARAMETERS;
    }
    if (npf_list_itrFirst(itr) != -1) {
        do {
            interface = (PDK_INTERFACE_t *)npf_list_itrGetData(itr);
            if (strcmp(interface->vifname, "") == 0) 
                continue;
            /* fire internal interface delete event on existing interface */
            onCB_event_IF(interface->handle, 1, NPF_EVENT_IF_DELETE);
        } while (npf_list_itrNext(itr) != -1);
    }
    npf_list_itrDelete(itr);
    CM_UNLOCK();

    return 0;
}

/*****************************************************************************
*                                                                            *
*  -------------------------- Internal Interface --------------------------  *
*                                                                            *
*****************************************************************************/

/*****************************************************************************
*    Name: cm_init
*
*    Description:
*        Initialize and allocate the global link list, register CB from FPPI
*        and B&D, Init the lock. Called by PDK Manager
*
*    Parameters:
*        None
*
*    Preconditions:
*        g_initialized = 0
*        called after bd_init, cbthread_init and fppi_init.
*        called before ns_init, ipv4_init, mpls_init
*
*    Postconditions:
*        g_initialized = 1;
*
*    Returns:
*        NPF_SUCCESS                 - everything is fine
*        NPF_COMPONENT_REINITIALIAED - initialized before
*        NPF_OUT_OF_MEMORY           - out of memory
*
*/
NPF_RET cm_init(void)
{
    int i;

    if (g_initialized) return NPF_COMPONENT_REINITIALIZED;

    PIL_InitializeCriticalSection(&g_cmlock);

    CM_LOCK();

    /* may define a standalone destroy function instead of free() */
    /* General callback will automatically init at first register */
    npf_list_init(&SYSTEMList, PIL_FreeMemory);
    npf_list_init(&FEList, PIL_FreeMemory);
    npf_list_init(&IFList, PIL_FreeMemory);
    npf_list_init(&PortList, PIL_FreeMemory);
    npf_list_init(&IPRouterList, PIL_FreeMemory);
    npf_list_init(&RTList, PIL_FreeMemory);
    npf_list_init(&PreList, PIL_FreeMemory);
    npf_list_init(&NHList, PIL_FreeMemory);
    npf_list_init(&ATList, PIL_FreeMemory);
    npf_list_init(&ICList, PIL_FreeMemory);
    npf_list_init(&g_outstandingRequest, PIL_FreeMemory);

    for (i=0; i<CM_CC_MAXNUM; i++)
        CMRegFppCBId[i] = -1;

    /* Register FEBIND event from B&D */
    bd_feBind_register_callback((void*)CM_CC_EVENT_FEBIND,
                       (NPF_GENERAL_CBFUNC)onCB_event_FEBind, &g_regbdcbid);

    if (FPPAPI_register_callback(
                (void *)&onCB_if_set_ipaddr,		/* CB func */
                &CMRegFppCBId[CM_CC_IF_SETIPADDR],	/* CB handle */
                FPPI_L3_SetProperties,			/* NPF_CBCAT */
                (NPF_USERCONTEXT)CM_CC_IF_SETIPADDR)
                                            != FPPI_SUCCESS)
        goto nomemory;

    if (FPPAPI_register_callback(
                (void *)&onCB_if_del_ipaddr,		/* CB func */
                &CMRegFppCBId[CM_CC_IF_DELIPADDR],	/* CB handle */
                FPPI_L3_DelProperties,			/* NPF_CBCAT */
                (NPF_USERCONTEXT)CM_CC_IF_DELIPADDR)
                                            != FPPI_SUCCESS)
        goto nomemory;

#ifdef _INCLUDE_IPV6
    if (FPPAPI_register_callback(
                (void *)&onCB_if_set_ipv6addr,		/* CB func */
                &CMRegFppCBId[CM_CC_IF_SETIPv6ADDR],	/* CB handle */
                FPPI_L3_SetPropertiesIPv6,		/* NPF_CBCAT */
                (NPF_USERCONTEXT)CM_CC_IF_SETIPv6ADDR)
                                            != FPPI_SUCCESS)
        goto nomemory;

    if (FPPAPI_register_callback(
                (void *)&onCB_if_del_ipv6addr,		/* CB func */
                &CMRegFppCBId[CM_CC_IF_DELIPv6ADDR],	/* CB handle */
                FPPI_L3_DelPropertiesIPv6,		/* NPF_CBCAT */
                (NPF_USERCONTEXT)CM_CC_IF_DELIPv6ADDR)
                                            != FPPI_SUCCESS)
        goto nomemory;
#endif

    if (FPPAPI_register_callback(
            (void *)&onCB_if_set_ipfwd,          /* func */
            &CMRegFppCBId[CM_CC_IF_SETIPFWD],    /* CB handle */
            FPPI_L3_SetProperties,               /* NPF_CBCAT */
            (NPF_USERCONTEXT)CM_CC_IF_SETIPFWD)
                != FPPI_SUCCESS)
        goto nomemory;

    if (FPPAPI_register_callback(
            (void *)&onCB_if_get_statistics,    /* func */
            &CMRegFppCBId[CM_CC_IF_GETSTATS],   /* CB handle */
            FPPI_L3_GetStatistics,              /* NPF_CBCAT */
            (NPF_USERCONTEXT)CM_CC_IF_GETSTATS)
                != FPPI_SUCCESS)
        goto nomemory;

    if (FPPAPI_register_callback(
            (void *)&onCB_icmp_get_statistics,  /* func */
            &CMRegFppCBId[CM_CC_ICMP_GETSTATS], /* CB handle */
            FPPI_ICMP_GetStatistics,            /* NPF_CBCAT */
            (NPF_USERCONTEXT)CM_CC_ICMP_GETSTATS)
                != FPPI_SUCCESS)
        goto nomemory;

    if (FPPAPI_register_callback(
            (void *)&onCB_port_set_mtu,        /* func */
            &CMRegFppCBId[CM_CC_PORT_SETMTU],  /* CB handle */
            FPPI_L2_SetProperties,             /* NPF_CBCAT */
            (NPF_USERCONTEXT)CM_CC_PORT_SETMTU)
                != FPPI_SUCCESS)
        goto nomemory;

    if (FPPAPI_register_callback(
            (void *)&onCB_port_set_linkspeed,       /* func */
            &CMRegFppCBId[CM_CC_PORT_SETLINKSPEED], /* CB handle */
            FPPI_L2_SetProperties,                  /* NPF_CBCAT */
            (NPF_USERCONTEXT)CM_CC_PORT_SETLINKSPEED)
                != FPPI_SUCCESS)
        goto nomemory;

    if (FPPAPI_register_callback(
            (void *)&onCB_if_set_portstatus,       /* func */
            &CMRegFppCBId[CM_CC_IF_SETPORTSTATUS], /* CB handle */
            FPPI_L2_SetProperties,                 /* NPF_CBCAT */
            (NPF_USERCONTEXT)CM_CC_IF_SETPORTSTATUS)
                != FPPI_SUCCESS)
        goto nomemory;

    if (FPPAPI_register_callback(
            (void *)&onCB_port_get_statistics,  /* func */
            &CMRegFppCBId[CM_CC_PORT_GETSTATS], /* CB handle */
            FPPI_L2_GetStatistics,              /* NPF_CBCAT */
            (NPF_USERCONTEXT)CM_CC_PORT_GETSTATS)
                != FPPI_SUCCESS)
        goto nomemory;

    if (FPPAPI_register_callback(
            (void *)&onCB_event_PortLinkUp,    /* func */
            &CMRegFppCBId[CM_CC_EVENT_LINKUP], /* CB handle */
            FPPI_LinkUp_Event,                 /* NPF_CBCAT */
            (NPF_USERCONTEXT)CM_CC_EVENT_LINKUP)
                != FPPI_SUCCESS)
        goto nomemory;

    if (FPPAPI_register_callback(
            (void *)&onCB_event_PortLinkDown,    /* func */
            &CMRegFppCBId[CM_CC_EVENT_LINKDOWN], /* CB handle */
            FPPI_LinkDown_Event,                 /* NPF_CBCAT */
            (NPF_USERCONTEXT)CM_CC_EVENT_LINKDOWN)
                != FPPI_SUCCESS)
        goto nomemory;

    if (FPPAPI_register_callback(
            (void *)&onCB_event_FEUnBind,       /* func */
            &CMRegFppCBId[CM_CC_EVENT_FEUNBIND],/* CB handle */
            FPPI_FEUnBind_Event,                /* NPF_CBCAT */
            (NPF_USERCONTEXT)CM_CC_EVENT_FEUNBIND)
                != FPPI_SUCCESS)
        goto nomemory;

    g_initialized = TRUE;


    CM_UNLOCK();

    return NPF_SUCCESS;

nomemory:

    /* need unlock first cuz after destroy lock is gone */
    CM_UNLOCK();
    cm_destroy();
    return NPF_OUT_OF_MEMORY;
}

/*****************************************************************************
*    Name: cm_unbindFEs
*
*    Description:
*        Called only at npf_pdk_shutdown by PDK Manager. It's for admin
*        PDK shutdown, needs to simulate the FE unbind event for all
*        UP FEs. Traverse NS, check FE status and fire onCB_event_FEUnBind
*        on the UP FEs. This is for VIDD purpose becasue it listens
*        IF_DELETE to delete the virtual interface, which only happens at
*        FE Unbind time.
*
*    Parameters:
*        None
*
*    Preconditions:
*        FPPI_Stop, no more PDK message could come again from FE
*
*    Postconditions:
*        All FEs are off and related events are fired
*
*    Returns:
*        NPF_SUCCESS
*
*/
NPF_RET cm_unbindFEs(void)
{
    NPF_NSHDL sys_ns_hdl, fe_ns_hdl;
    NPF_HANDLE sys_npf_hdl, fe_npf_hdl;
    NODE_ITERATOR fe_itr;
    PDK_FE_t* pdk_fe;
    int num_fes = 0;

    CHK_INIT();

    CM_LOCK();
    /* Need to traverse Namespace, get all FEs;
       for each FE that is still ON, simulate an UNBIND event */
    if (NPF_SUCCESS != pdk_ns_getNSRootHandle (&sys_ns_hdl))
        goto error;
    if (NPF_SUCCESS != npf_ns_getDataHandle(sys_ns_hdl, &sys_npf_hdl)) {
        npf_ns_close (sys_ns_hdl);
        goto error;
    }
    /* use NS helper fn. to get an iterator to all FEs in the
       system */
    if (NPF_SUCCESS != pdk_ns_getChildItrByType(sys_npf_hdl,NPF_FE,&fe_itr)) {
        npf_ns_close(sys_ns_hdl);
        goto error;
    }

    npf_ns_close (sys_ns_hdl);

    /* iterate through all FEs using this iterator */
    num_fes = npf_ns_count (fe_itr);
    npf_logger_write(LOG_CM, LOG_INFO, "Number of FEs in the system = %d \n",
            num_fes);
    fe_ns_hdl = npf_ns_first (fe_itr);
    do {
        if (NPF_SUCCESS == npf_ns_getDataHandle (fe_ns_hdl, &fe_npf_hdl)) {
            npf_logger_write(LOG_CM, LOG_INFO, "Got FE handle\n");
            if (cm_getDataByHandle(fe_npf_hdl, (void **)&pdk_fe) == -1) {
                npf_logger_write(LOG_CM, LOG_INFO,
                        "Could not get FE data from CM\n");
                npf_ns_close (fe_ns_hdl);
                continue;
            }
            /* check if the handle is FE handle */
            if (((PDK_SHIM_t *)pdk_fe)->type != NPF_FE) {
                npf_logger_write(LOG_CM, LOG_INFO, "FE Data is not valid\n");
                npf_ns_close (fe_ns_hdl);
                continue;
            }
            if (pdk_fe->attribs.status == NPF_FE_UP) {
                npf_logger_write(LOG_CM, LOG_INFO,
                    "Found FE ID %d to be UP\n", pdk_fe->id);
                onCB_event_FEUnBind (
                    pdk_fe->attribs.id,
                    (NPF_CORRELATOR)0,
                    (NPF_USERCONTEXT)CM_CC_EVENT_FEUNBIND,
                    (void*) NULL);
            } /* if (FE was UP) */			    
        } /* end if (got FE NPF handle) */
        npf_ns_close (fe_ns_hdl);
    } while ((fe_ns_hdl = npf_ns_next (fe_itr)) != 0);
    npf_ns_done(fe_itr);
    CM_UNLOCK();
    return NPF_SUCCESS;

error:
    CM_UNLOCK();
    return NPF_INVALID_PARAMETERS;

}

/*****************************************************************************
*    Name: cm_destroy
*
*    Description:
*        Deregister and free the global link list, B&D, FPPI callback
*        destroy the lock and callback all the outstanding request
*
*    Parameters:
*        None
*
*    Preconditions:
*        g_initialized = 1
*        called before bd_shutdown, fppi_shutdown and cbthread_shutdown
*        called after ns_shutdown, ipv4_shutdown, mpls_shutdown ...
*
*    Postconditions:
*        g_initialized = 0;
*
*    Returns:
*        NPF_SUCCESS - always success
*
*/
NPF_RET cm_destroy(void)
{
    int          i;

    CHK_INIT();

    CM_LOCK();
    g_initialized = FALSE;

    /* Deregister Event/Cat Callback from FP Plugin */
    for (i=0; i<CM_CC_MAXNUM; i++)
        if (CMRegFppCBId[i] != -1) {
            FPPAPI_deregister_callback(CMRegFppCBId[i]);
            CMRegFppCBId[i] = -1;
        }
    

    /* this will callback all the outstanding request and destroy it */
    npf_list_destroy(&g_outstandingRequest);
    
    /* if the ObjList is not empty, remove items and free it */
    npf_list_destroy(&ICList);
    npf_list_destroy(&PortList);
    npf_list_destroy(&IFList);
    npf_list_destroy(&ATList);
    npf_list_destroy(&NHList);
    npf_list_destroy(&PreList);
    npf_list_destroy(&RTList);
    npf_list_destroy(&IPRouterList);
    npf_list_destroy(&FEList);
    npf_list_destroy(&SYSTEMList);


    CM_UNLOCK();

    PIL_DestroyCriticalSection(&g_cmlock);

    npf_logger_write(LOG_CM, LOG_INFO,
            "cm successfully shutdown. %d instance left", g_InstanceNum);

    return NPF_SUCCESS;
}


/*****************************************************************************
*    Name: cm_newDataByType
*
*    Description:
*        Create memory space for such datatype, set default value
*        and the meta data. Push into corresponding DList
*
*    Parameters:
*        IN : type    - the datatype needs to be created
*        IN : nsh     - name space handle associates to this object
*        OUT: phandle - NPF_HANDLE to the newly created object
*
*    Preconditions:
*        unique namespace handle is created
*
*    Returns:
*        NPF_SUCCESS
*        NPF_OUT_OF_MEMORY      - fail to create instance
*        NPF_INVALID_PARAMETERS - fail if doen't recognize input type
*
*/
NPF_RET cm_newDataByType(NPF_DATATYPE type,NPF_NSHDL nsh,NPF_HANDLE *phandle)
{

    NPF_DATA   data;
    PDK_SHIM_t *shim;

    assert(phandle);

    CHK_INIT();

    CM_LOCK();

    switch(type) {
        case NPF_SYSTEM:
            if ((data = (void *)PIL_AllocateMemory
                           (sizeof(PDK_SYSTEM_t), PIL_MEM_ZERO_INIT)) == NULL)
                goto error;
            npf_list_pushBack(&SYSTEMList, data);
            strcpy(((PDK_SYSTEM_t *)data)->name, "NPF_SYS");
            ((PDK_SYSTEM_t *)data)->num_FEs = 0;
            break;

        case NPF_FE:
            /* allocate FE memory, free at delData time */
            if ((data = (void *)PIL_AllocateMemory
                        (sizeof(PDK_FE_t), PIL_MEM_ZERO_INIT)) == NULL)
                goto error;
            /* push into the FEList */
            npf_list_pushBack(&FEList, data);
            /* set default value */
            strcpy(((PDK_FE_t *)data)->attribs.name, "N/A");
            strcpy(((PDK_FE_t *)data)->attribs.desc, "N/A");
            /* hack, this should be assigned by B&D */
            ((PDK_FE_t *)data)->attribs.max_arp_table = 1;
            ((PDK_FE_t *)data)->attribs.max_route_table = 1;
            ((PDK_FE_t *)data)->attribs.max_prefix_table = 1;
            ((PDK_FE_t *)data)->attribs.max_nexthop_table = 1;
            break;

        case NPF_PORT:
            if ((data = (void *)PIL_AllocateMemory
                        (sizeof(PDK_PORT_t), PIL_MEM_ZERO_INIT)) == NULL)
                goto error;
            /* push into the PORTList */
            npf_list_pushBack(&PortList, data);
            /* set default value */
            ((PDK_PORT_t *)data)->attribs.id = HandleCounter; /* MPLS Test */
            break;

        case NPF_IPROUTER:
            if ((data = (void *)PIL_AllocateMemory
                        (sizeof(PDK_IPROUTER_t), PIL_MEM_ZERO_INIT)) == NULL)
                goto error;
            /* push into the FEList */
            npf_list_pushBack(&IPRouterList, data);
            /* set default value */
            ((PDK_IPROUTER_t *)data)->num_interfaces = 0;
            break;

        case NPF_INTERFACE:
            if ((data = (void *)PIL_AllocateMemory
                        (sizeof(PDK_INTERFACE_t), PIL_MEM_ZERO_INIT)) == NULL)
                goto error;
            /* push into the IFList */
            npf_list_pushBack(&IFList, data);
            break;

        case NPF_ROUTETABLE:
            if ((data = (void *)PIL_AllocateMemory
                      (sizeof(PDK_ROUTETABLE_t), PIL_MEM_ZERO_INIT)) == NULL)
                goto error;
            /* push into the FEList */
            npf_list_pushBack(&RTList, data);
            break;

    case NPF_PREFIXTABLE:
        if ((data = (void *)PIL_AllocateMemory
             (sizeof(PDK_ROUTETABLE_t), PIL_MEM_ZERO_INIT)) == NULL)
            goto error;
        /* push into the FEList */
        npf_list_pushBack(&PreList, data);
        break;

    case NPF_NEXTHOPTABLE:
        if ((data = (void *)PIL_AllocateMemory
             (sizeof(PDK_ROUTETABLE_t), PIL_MEM_ZERO_INIT)) == NULL)
            goto error;
        /* push into the FEList */
        npf_list_pushBack(&NHList, data);
        break;

    case NPF_ARPTABLE:
            if ((data = (void *)PIL_AllocateMemory
                        (sizeof(PDK_ARPTABLE_t), PIL_MEM_ZERO_INIT)) == NULL)
                goto error;
            /* push into the FEList */
            npf_list_pushBack(&ATList, data);
            break;

        case NPF_INTCONFIG:
            if ((data = (void *)PIL_AllocateMemory
                        (sizeof(PDK_INTCONFIG_t), PIL_MEM_ZERO_INIT)) == NULL)
                goto error;
            npf_list_pushBack(&ICList, data);
            break;

        default:
            npf_logger_write(LOG_CM, LOG_ERR, "Invalid Data Type %d", type);
            CM_UNLOCK();
            return NPF_INVALID_PARAMETERS;
    }

    /* give handle and increment after */
    *phandle = HandleCounter++;
    shim = (PDK_SHIM_t *)data;
    shim->type = type;
    shim->ns_handle = nsh;
    shim->handle = *phandle;
    g_InstanceNum++;

    CM_UNLOCK();

    return NPF_SUCCESS;

error:
    CM_UNLOCK();
    return NPF_OUT_OF_MEMORY;
}

/*****************************************************************************
*    Name: cm_delDataCheck
*
*    Description:
*        Check if a data could be deleted or not, basically see if
*        there are any dependency in CE to stop it from deleting.
*
*    Parameters:
*        IN : handle - the handle of the object that needs to be deleted
*
*    Preconditions:
*        unique handle exists
*
*    Returns:
*        NPF_SUCCESS
*        NPF_INVALID_PARAMETERS - fail if can't find the handle or can't del
*
*/
NPF_RET cm_delDataCheck(NPF_HANDLE handle)
{
    int        i, result;
    PDK_SHIM_t *shim;
    DList      *list;
    DListItr   *itr;

    CHK_INIT();

    CM_LOCK();

    result = 0;
    for (i=0; i<NUMOBJARRAY; i++) {
        list = ObjArray[i];
        if ((itr = npf_list_itrCreate(list))==NULL)
            continue;
        if (npf_list_itrFirst(itr) != -1) {
            do {
                shim = (PDK_SHIM_t *)npf_list_itrGetData(itr);
                if (shim->handle == handle) {
                    result = 1;
                    break;
                }
            } while (npf_list_itrNext(itr) != -1);
        }
        if (result)
            break;
        /* Do not delete itr if found */
        npf_list_itrDelete(itr);
    }

    if (!result) {
        CM_UNLOCK();
        return NPF_INVALID_PARAMETERS;
    }

    /* Check each different type */
    switch (shim->type) {
        case NPF_INTERFACE:
            /* can't delete a IF node if the virtual interface is there */
            if (strcmp(((PDK_INTERFACE_t *)shim)->vifname, "") != 0) {
                npf_list_itrDelete(itr);
                CM_UNLOCK();
                return NPF_INVALID_PARAMETERS;
            }
            break;
        default:
    }

    npf_list_itrDelete(itr);

    CM_UNLOCK();
    return NPF_SUCCESS;

}

/*****************************************************************************
*    Name: cm_delDataByHandle
*
*    Description:
*        Free the memory space of the instance that the Handle points to.
*        Remove from the object list.
*
*    Parameters:
*        IN : handle - the handle of the object that needs to be deleted
*
*    Preconditions:
*        unique handle exists
*
*    Returns:
*        NPF_SUCCESS
*        NPF_INVALID_PARAMETERS - fail if can't find the handle
*
*/
NPF_RET cm_delDataByHandle(NPF_HANDLE handle)
{

    int        i, result;
    PDK_SHIM_t *shim;
    DList      *list;
    DListItr   *itr;

    CHK_INIT();

    CM_LOCK();

    /* Check if this data could be deleted */
    if (cm_delDataCheck(handle) != NPF_SUCCESS) {
        CM_UNLOCK();
        return NPF_INVALID_PARAMETERS;
    }

    result = 0;
    for (i=0; i<NUMOBJARRAY; i++) {
        list = ObjArray[i];
        if ((itr = npf_list_itrCreate(list))==NULL)
            continue;
        if (npf_list_itrFirst(itr) != -1) {
            do {
                shim = (PDK_SHIM_t *)npf_list_itrGetData(itr);
                if (shim->handle == handle) {
                    result = 1;
                    break;
                }
            } while (npf_list_itrNext(itr) != -1);
        }
        if (result)
            break;
        /* Do not delete itr if found */
        npf_list_itrDelete(itr);
    }

    if (!result) {
        CM_UNLOCK();
        return NPF_INVALID_PARAMETERS;
    }

    /* Remove the pointer inside the SYSTEM/FE/IF/PORT/ROUTER if any */

    /* now remove the object */
    if (npf_list_itrRemoveFree(itr) != -1) {
        npf_list_itrDelete(itr);
        g_InstanceNum--;
        CM_UNLOCK();
        return NPF_SUCCESS;
    }
    return NPF_SUCCESS;
}

/*****************************************************************************
*    Name: cm_getFEHandleById
*
*    Description:
*        Get the FE handle by comparing the feid
*
*    Parameters:
*        IN : feid     - the FE id which is assgined by FP Plugin
*        OUT: fehandle - handle of the FE whose id equals input feid
*
*    Preconditions:
*        feid is unique at any giving time
*
*    Returns:
*        NPF_SUCCESS
*        NPF_INVALID_PARAMETERS - fail to find the FE by given feid
*
*/
NPF_RET cm_getFEHandleById(uint32_t feid, NPF_HANDLE *fehandle)
{
    int      result;
    PDK_FE_t *fe;
    DListItr *itr;

    assert(fehandle);

    CHK_INIT();
    CM_LOCK();
    /* linear search for FE handle */
    result = 0;
    if ((itr = npf_list_itrCreate(&FEList)) == NULL) {
        CM_UNLOCK();
        return NPF_INVALID_PARAMETERS;
    }
    if (npf_list_itrFirst(itr) != -1) {
        do {
            fe = (PDK_FE_t *)npf_list_itrGetData(itr);
            if (fe->attribs.id == feid) {
                result = 1;
                *fehandle = fe->handle;
                break;
            }
        } while (npf_list_itrNext(itr) != -1);
    }
    npf_list_itrDelete(itr);
    CM_UNLOCK();

    if (result)
        return NPF_SUCCESS;
    else
        return NPF_INVALID_PARAMETERS;
}

/*****************************************************************************
*    Name: cm_getPortByFE
*
*    Description:
*        Give a FE handle and return port number, port handle list of
*        that FE. Port handle list are pre-allocated by caller.
*
*    Parameters:
*        IN : feh       - FE handle
*        OUT: num_ports - pointer to the number of ports of this FE
*        OUT: porthlist - array of port handles in this FE in open status
*
*    Preconditions:
*        caller pre-allocates port handle array to size MAXFEOWNPORT
*
*    Returns:
*        NPF_SUCCESS
*        NPF_INVALID_PARAMETERS - fail if unable to find port
*
*/
NPF_RET cm_getPortByFE(NPF_HANDLE feh, int *num_ports, NPF_HANDLE *porthlist)
{
    int           result, index;
    NPF_DATATYPE  datatype;
    NPF_NSHDL     nsfeh, nsporth, nsporttypeh;
    NODE_ITERATOR dirItr;
    char path[NPF_NS_MAXPATH] = "";

    assert(num_ports);
    assert(porthlist);

    CHK_INIT();
    *num_ports = 0;
    CM_LOCK();

    if (cm_getNSHDLByHandle(feh, &nsfeh) != NPF_SUCCESS ||
        npf_ns_getPath(nsfeh, path)      != NPF_SUCCESS ||
        npf_ns_readDir(path, &dirItr)    != NPF_SUCCESS)
        goto error;

    nsporttypeh = npf_ns_first(dirItr);
    result = 0;
    do {
        /* bypass the Route/ARP/FTN/ILM Table type */
        npf_ns_getDataType(nsporttypeh, &datatype);
        if (datatype == NPF_PORT) {
            result = 1;
            npf_ns_close(nsporttypeh);
            break;
        }
    } while ((npf_ns_close(nsporttypeh) != -1) && 
                    (nsporttypeh = npf_ns_next(dirItr)) != 0);
    npf_ns_done(dirItr);

    if (!result) goto error;

    memset(path, 0, NPF_NS_MAXPATH);
    if (npf_ns_getPath(nsporttypeh, path) != NPF_SUCCESS ||
        npf_ns_readDir(path, &dirItr)     != NPF_SUCCESS)
        goto error;

    *num_ports = npf_ns_count(dirItr);
    index = 0;
    nsporth = npf_ns_first(dirItr);
    do {
        npf_ns_getDataHandle(nsporth, &porthlist[index++]); 
    } while ((nsporth = npf_ns_next(dirItr)) != 0);
    npf_ns_done(dirItr);

    CM_UNLOCK();
    return NPF_SUCCESS;

error:
    CM_UNLOCK();
    return NPF_INVALID_PARAMETERS;
}

/*****************************************************************************
*    Name: cm_getNSHDLByHandle
*
*    Description:
*        get the namespace handle from the object whose object handle
*        equals to input object handle
*
*    Parameters:
*        IN : objh - object handle of any object instance
*        OUT: nsh  - namespace handle of the pass in object
*
*    Returns:
*        NPF_SUCCESS
*        NPF_INVALID_PARAMETERS - fail to find the object instance in PDK
*
*/
NPF_RET cm_getNSHDLByHandle(NPF_HANDLE objh, NPF_NSHDL *nsh)
{
    int        i, result;
    PDK_SHIM_t *shim;
    DList      *list;
    DListItr   *itr;

    assert(nsh);

    CHK_INIT();
    CM_LOCK();
    result = 0;
    for (i=0; i<NUMOBJARRAY; i++) {
        list = ObjArray[i];
        if ((itr = npf_list_itrCreate(list)) == NULL)
            continue;
        if (npf_list_itrFirst(itr) != -1) {
            do {
                shim = (PDK_SHIM_t *)npf_list_itrGetData(itr);
                if (shim->handle == objh) {
                    result = 1;
                    *nsh = shim->ns_handle;
                    break;
                }
            } while (npf_list_itrNext(itr) != -1);
        }
        npf_list_itrDelete(itr);
        if (result)
            break;
    }
    CM_UNLOCK();

    if (result)
        return NPF_SUCCESS;
    else
        return NPF_INVALID_PARAMETERS;
}

/*****************************************************************************
*    Name: cm_getTypeByHandle
*
*    Description:
*        get the data type from the object whose object handle
*        equals to input object handle
*
*    Parameters:
*        IN : objh - object handle of any object instance
*        OUT: type - data type of the pass in object
*
*    Returns:
*        NPF_SUCCESS
*        NPF_INVALID_PARAMETERS - fail to find the object instance in PDK
*
*/
NPF_RET cm_getTypeByHandle(NPF_HANDLE objh, NPF_DATATYPE *type)
{
    int        i, result;
    PDK_SHIM_t *shim;
    DList      *list;
    DListItr   *itr;

    assert(type);

    CHK_INIT();
    CM_LOCK();
    result = 0;
    for (i=0; i<NUMOBJARRAY; i++) {
        list = ObjArray[i];
        if ((itr = npf_list_itrCreate(list)) == NULL)
            continue;
        if (npf_list_itrFirst(itr) != -1) {
            do {
                shim = (PDK_SHIM_t *)npf_list_itrGetData(itr);
                if (shim->handle == objh) {
                    result = 1;
                    *type = shim->type;
                    break;
                }
            } while (npf_list_itrNext(itr) != -1);
        }
        npf_list_itrDelete(itr);
        if (result)
            break;
    }
    CM_UNLOCK();

    if (result)
        return NPF_SUCCESS;
    else
        return NPF_INVALID_PARAMETERS;
}

/*****************************************************************************
*    Name: cm_check_init
*
*    Description:
*        To check that C & M is initialized
*
*    Parameters:
*
*    Returns:
*        NPF_SUCCESS                 - initialized
*        NPF_COMPONENT_UNINITIALIZED - not initalized
*
*/
NPF_RET cm_check_init()
{
    if (!g_initialized) {
         return NPF_COMPONENT_UNINITIALIZED;
    }

    return NPF_SUCCESS;
}


/*****************************************************************************
*                                                                            *
*  --------------------------- Public Interface ---------------------------  *
*                                                                            *
*    Note: All value needs to be COPY out, don't send out thre pointer       *
*          to the real data inside the PDK                                   *
*    Note: The incoming data need COPY in too, don't use app's memory        *
*                                                                            *
*****************************************************************************/

/* --------------------------- Category Register -------------------------- */

/*****************************************************************************
*    Name: npf_category_register
*
*    Description:
*        Generic callback function which is typedef-ed to all the C&M
*        category register. Simply use common callback library
*
*    Parameters:
*        IN : cat     - the category for this registration
*        IN : context - user context
*        IN : func    - callback function
*        OUT: cbid    - unique callback id and will use for subsequnce calls
*                       in the same category
*
*    Preconditions:
*        the category needs to be list in the pdk_callback.h
*        callback function exists
*
*    Returns:
*        NPF_SUCCESS
*        NPF_OUT_OF_MEMORY      - fail to allocate callback state memory
*        NPF_INVALID_PARAMETERS - fail to find the category
*
*/
NPF_RET npf_category_register(	NPF_CBCAT cat,
				NPF_USERCONTEXT context,
				NPF_GENERAL_CBFUNC func,
				NPF_CBHANDLE	*cbid	)
{
    CHK_PDK_INIT();
    CHK_INIT();

    if (!func || !cbid)
        return NPF_INVALID_PARAMETERS;

    return pdk_callback_register(cat, context, func, cbid);
}

/* This pdk_register is for internal, does not check g_pdk_inig */
NPF_RET pdk_category_register(  NPF_CBCAT cat,
                                NPF_USERCONTEXT context,
                                NPF_GENERAL_CBFUNC func,
                                NPF_CBHANDLE    *cbid   )
{
    CHK_INIT();

    if (!func || !cbid)
        return NPF_INVALID_PARAMETERS;

    return pdk_callback_register(cat, context, func, cbid);
}

/*****************************************************************************
*    Name: npf_category_deregister
*
*    Description:
*        Generic callback function which is typedef-ed to all the C&M
*        category deregister. Simply use common callback library
*
*    Parameters:
*        IN : cbid - the cbid of the category that needs to be deregistered
*
*    Preconditions:
*        callback id was acquired before by npf_category_deregister
*
*    Returns:
*        NPF_SUCCESS
*        NPF_INVALID_PARAMETERS - fail to find the category
*
*/
NPF_RET npf_category_deregister(NPF_CBHANDLE cbid)
{
    CHK_PDK_INIT();
    CHK_INIT();
    return pdk_callback_deregister(cbid);
}

/* This pdk_deregister is for internal, does not check g_pdk_init */
NPF_RET pdk_category_deregister(NPF_CBHANDLE cbid)
{
    CHK_INIT();
    return pdk_callback_deregister(cbid);
}


/*****************************************************************************
*    Name: npf_fe_get_capabilities
*
*    Description:
*        Get the FE capabilities by FE id. 
*
*    Parameters:
*        IN   : feId    - FE id
*        OUT  : feCaps  - FE Capabilities
*
*   Remark : Responsilbilty for freeing Fe Caps & Port Caps buffer is 
*            with User.
*
*    Returns:
*        NPF_SUCCESS
*        NPF_OUT_OF_MEMORY      - fail to allocate the response data
*        NPF_INVALID_PARAMETERS - fail to find fe Id.
*
******************************************************************************/

NPF_RET npf_fe_get_capabilities (uint32_t    	     feId,
				NPF_FeCapability_t   **feCaps)
{
    char                     fe_NsPath[NPF_NS_MAXPATH];
    NPF_NSHDL                fe_ns_hdl, port_ns_hdl;
    NPF_HANDLE               fe_npf_hdl, port_npf_hdl;
    uint32_t                 portIndex = 0;
    NPF_FeCapability_t       *tempFeCaps = NULL;
    NPF_FE_ATTRIBS           feAttribs;
    NPF_PORT_ATTRIBS         portAttribs;
    NODE_ITERATOR            portItr;

    CHK_PDK_INIT();
    CHK_INIT();

    CM_LOCK()
            
    memset(fe_NsPath, 0, NPF_NS_MAXPATH);
    sprintf(fe_NsPath, "/NPFSys/0/FE/%d", feId);

    if(npf_ns_open(fe_NsPath, &fe_ns_hdl) != NPF_SUCCESS)
    {
       CM_UNLOCK()
       return NPF_INVALID_PARAMETERS;
    }

    if (npf_ns_getDataHandle(fe_ns_hdl, &fe_npf_hdl) != NPF_SUCCESS)
    {
       npf_ns_close(fe_ns_hdl);
       CM_UNLOCK()
       return NPF_FAIL;
    }
    
    if (cm_fe_get_attribs(fe_npf_hdl, &feAttribs) != NPF_SUCCESS)
    {
       npf_ns_close(fe_ns_hdl);
       CM_UNLOCK()
       return NPF_FAIL;
    }
    
    tempFeCaps = (NPF_FeCapability_t *)PIL_AllocateMemory(
                      sizeof(NPF_FeCapability_t), PIL_MEM_ZERO_INIT);
    
    if(!tempFeCaps)
    {
       npf_ns_close(fe_ns_hdl);
       CM_UNLOCK()
       return NPF_OUT_OF_MEMORY;
    }
    
    tempFeCaps->feId = feId;
    tempFeCaps->timeStamp = feAttribs.stamp;
    tempFeCaps->feIpAddr = feAttribs.ipaddr;
    memcpy(&tempFeCaps->feMacAddr, &feAttribs.hwaddr, sizeof(HWADDR));
    tempFeCaps->numPorts = feAttribs.num_ports;
    tempFeCaps->status = feAttribs.status;
    
    if(tempFeCaps->numPorts)
    {
       tempFeCaps->ports = (NPF_PORT_ATTRIBS *)PIL_AllocateMemory(
                      sizeof(NPF_PORT_ATTRIBS)*(tempFeCaps->numPorts),
                      PIL_MEM_ZERO_INIT);

       if(!(tempFeCaps->ports))
       {
          PIL_FreeMemory(tempFeCaps);
          npf_ns_close(fe_ns_hdl);
          CM_UNLOCK()
          return NPF_OUT_OF_MEMORY;
       }
    }
    
    if(pdk_ns_getChildItrByType(fe_npf_hdl, NPF_PORT, &portItr) != NPF_SUCCESS)
    {
       npf_ns_close(fe_ns_hdl);
       PIL_FreeMemory(tempFeCaps->ports);
       PIL_FreeMemory(tempFeCaps);
       CM_UNLOCK()
       return NPF_FAIL;
    }

    port_ns_hdl =  npf_ns_first(portItr);
    
    while(port_ns_hdl)
    {
       if(npf_ns_getDataHandle(port_ns_hdl, &port_npf_hdl) == NPF_SUCCESS)
       {
          if (cm_port_get_attribs(port_npf_hdl, &portAttribs) == NPF_SUCCESS)
          {
             memcpy(&tempFeCaps->ports[portIndex], &portAttribs,
                             sizeof(NPF_PORT_ATTRIBS));
          }
       }
       port_ns_hdl = npf_ns_next(portItr);
       portIndex++;
    }
    
    *feCaps = tempFeCaps;

    CM_UNLOCK()

    return NPF_SUCCESS;
} /* end of npf_fe_get_capabilities */

