/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/

/*****************************************************************************
* $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/CoreBlocks/ConfigManagment/src/cmcallback.c,v $
* $Revision: 1.6 $
* $Date: 2003/11/17 16:57:12 $
*
* Description: The Callbacks registered with FPPAPI are defined here.
* 
*****************************************************************************/

#include <string.h>
#include <stdio.h>
#include <assert.h>
#include "pdk_callback.h"
#include "pdk_ns_helper.h"
#include "npf_ns_api.h"
#include "pdkmgr.h"
#include "cm.h"
#include "log.h"

#define INTERNAL_CB	1
#define EXTERNAL_CB	0

/* check the correlator return from FPPI to see if still wait FE back */
static int isStillBindWait(CMCORRELATOR cmcorrelator)
{
    /* this should be always in LOCK area */
    int i, stillBindWait = 0;

    assert(cmcorrelator);

    /* waitFEid[] is the FEID that is still waiting for
     * if the waitFEid[] still contains some value > 1, than
     * it is still waiting for an FE's response. */
    for (i=0; i<cmcorrelator->numSentFE; i++) {
        if (cmcorrelator->waitFEid[i] != REQUEST_FE_DONE &&
            cmcorrelator->waitFEid[i] != REQUEST_FE_FAIL &&
            cmcorrelator->waitFEid[i] != REQUEST_FE_UNBIND)
            stillBindWait = 1;
    }
    return stillBindWait;
}

/* set the up or down flag of interface and all the assoc. ports
 * checking all the interface which associates with those ports too.
 * output all the interfaces handle that needs to be trigger up 
 * or down event
 * input ifh is open, setifhlist[] return in open status */
static int cm_if_setflag(
        NPF_HANDLE    ifh,
        NPF_IF_STATUS flag,
        int           *num_setifs,
        NPF_HANDLE    *setifhlist)
{

    int             i, j, k, l, portlinkup;
    int             num_ports, num_ifs, tmp_numports;
    NPF_HANDLE      porthlist[MAXIFOWNPORT], ifhlist[MAXIFOWNPORT];
    NPF_HANDLE      tmp_porthlist[MAXIFOWNPORT];
    PDK_PORT_t      *port, *tmpport;
    PDK_INTERFACE_t *interface, *tmpif;

    assert(num_setifs);
    assert(setifhlist);

    *num_setifs = 0;

    if (cm_getDataByHandle(ifh, (NPF_DATA *)&interface) != NPF_SUCCESS)
        return -1;
    /* set all the port's status, the FE's port should be set already */
    if (cm_getPortByIF(ifh, &num_ports, porthlist) != NPF_SUCCESS)
        return -1;
    /* set the target interface status and push it to the iflist */
    /* If set it up but the linkstatus is down, don't fire IF UP event
       If one of the linkstatus is on, then following algorithm will push
       this IF into the setifhlist
       interface->status = flag;
       NOTE: open this ifh
       setifhlist[(*num_setifs)++] = ifh; */
    if (flag == NPF_IF_DOWN) {
        for (i=0; i<num_ports; cm_handle_close(porthlist[i++])) {
            cm_getDataByHandle(porthlist[i], (NPF_DATA *)&port);
            if (port->attribs.status == 0)
                /* if this port was admin off before, then skip it */
                continue;
            /* this port was admin on, set if off */
            port->attribs.status = 0;
            /* get the assoc. interfaces and if it is on */
            cm_getIFByPort(porthlist[i], &num_ifs, ifhlist);
            for (j=0; j<num_ifs; j++) {
                cm_getDataByHandle(ifhlist[j], (NPF_DATA *)&tmpif);
                if (tmpif->status == NPF_IF_DOWN) {
                    cm_handle_close(ifhlist[j]);
                    continue;
                }
                /* the associated if was ON */
                /* check if all the assoc. ports are down */
                cm_getPortByIF(ifhlist[j], &tmp_numports, tmp_porthlist);
                portlinkup = 0;
                for (k=0; k<tmp_numports; cm_handle_close(tmp_porthlist[k++])) {
                    cm_getDataByHandle
                        (tmp_porthlist[k], (NPF_DATA *)&tmpport);
                    if (tmpport->attribs.status     == 1 &&
                        tmpport->attribs.linkstatus == 1) {
                        portlinkup = 1;
                        for (l=k; l<tmp_numports; l++)
                            cm_handle_close(tmp_porthlist[l]);
                        break;
                    }
                }
                /* if none of the ports is admin+link on, set the if off */
                if (!portlinkup) {
                    tmpif->status = NPF_IF_DOWN;
                    setifhlist[(*num_setifs)++] = ifhlist[j];
                } else {
                    /* only close the one doesn't need to be set */
                    cm_handle_close(ifhlist[j]);
                }
            } /* done check all the interfaces associ. to this port */
        } /* done check all the ports associates to this if */
    } else if (flag == NPF_IF_UP) {
        for (i=0; i<num_ports; cm_handle_close(porthlist[i++])) {
            cm_getDataByHandle(porthlist[i], (NPF_DATA *)&port);
            if (port->attribs.status == 1)
                /* if this port was admin on before, then skip it */
                continue;
            /* this port was admin off before, make it up */
            port->attribs.status = 1;
            /* get the assoc. interfaces and if it is on */
            cm_getIFByPort(porthlist[i], &num_ifs, ifhlist);
            for (j=0; j<num_ifs; j++) {
                cm_getDataByHandle(ifhlist[j], (NPF_DATA *)&tmpif);
                /* if that interface was on, just skil it */
                if (tmpif->status == NPF_IF_UP) {
                    cm_handle_close(ifhlist[j]);
                    continue;
                }
                /* if that interface was off, check if needs turn it on
                 * check if any of the assoc. ports is admin + link up
                 * and the interface must have ipaddr */
                if (tmpif->ipattribs.addr_u.ipaddr  == 0 && 
                    tmpif->ipattribs.mask_u.submask == 0)
                    continue;
                cm_getPortByIF(ifhlist[j], &tmp_numports, tmp_porthlist);
                portlinkup = 0;
                for (k=0; k<tmp_numports; cm_handle_close(tmp_porthlist[k++])) {
                    cm_getDataByHandle
                        (tmp_porthlist[k], (NPF_DATA *)&tmpport);
                    if (tmpport->attribs.status     == 1 &&
                        tmpport->attribs.linkstatus == 1) {
                        portlinkup = 1;
                        for (l=k; l<tmp_numports; l++)
                            cm_handle_close(tmp_porthlist[l]);
                        break;
                    }
                }
                /* if any of the ports is admin+link on, set the if on */
                if (portlinkup) {
                    tmpif->status = NPF_IF_UP;
                    setifhlist[(*num_setifs)++] = ifhlist[j];
                } else {
                    /* only close the one doesn't need to be set */
                    cm_handle_close(ifhlist[j]);
                }
            } /* done check all the interfaces associ. to this port */
        } /* done check all the ports associates to this if */
    } else 
        /* not set UP, not set DOWN, what do you do? */
        assert(0);

    return 1;
}

/* Note: fire NPF_EVENT_FE_UP after all bind callback are done
 *       this is not perfect but is the only thing we can do
 *       still have the risk that ConfigApp doesn not setup the
 *       router/if attribute done yet then we fire FE_UP event
 *       we don't know when the setup ip/router cb will finish
 *
 * When the FEBind, FE.status, Port.status, Port.linkstatus are on
 *
 * Response Data: NPF_FE_t */
void onCB_event_FEBind(	uint32_t  feid,
			CMCONTEXT cmcontext )
{
    NPF_HANDLE       fehandle;
    NPF_CBHANDLE     cbid, nextcbid;
    NPF_EVENT_CBFUNC clientCBFunc;
    NPF_USERCONTEXT  context;
    PDK_FE_t         *fe;
    NPF_FE_t         *outfe;
    NPF_DATA         data;

    if ((CMCLIENTCONTEXT)cmcontext != CM_CC_EVENT_FEBIND)
        return;
    CM_LOCK();
    /* get FE handle & PDK_FE_t */
    if (cm_getFEHandleById(feid, &fehandle) == -1) 
        goto error;
    if (cm_getDataByHandle(fehandle, &data) == -1)
        goto error;
    fe = (PDK_FE_t *)data;

    /* Note: copy id out of attrib, should expose API to B&D, keep one copy */
    fe->id = fe->attribs.id;

    /* walk the internal CBs to execute all the cb func in FPPI thread */
    clientCBFunc = (NPF_EVENT_CBFUNC)pdk_callback_firstFunc(	PDK_EVT_FE,
								&context,
								&nextcbid);
    /* check in case user de-register */
    if (clientCBFunc != NULL) {
    do {
        /* create a new NPF_FE_t for every internal callback */
        if ((outfe = (NPF_FE_t *)PIL_AllocateMemory(sizeof(NPF_FE_t), 
                                               PIL_MEM_ZERO_INIT)) == NULL)
            assert(0);
        if (cm_copyFromPDK(fehandle, (NPF_DATA)outfe) == -1)
            assert(0);

        cbid = nextcbid;
        clientCBFunc(	context,
 			fehandle,
			NPF_EVENT_FE_BIND,
			(NPF_DATA)outfe	);

        PIL_FreeMemory(outfe);
    } while ((clientCBFunc = (NPF_EVENT_CBFUNC)
              pdk_callback_nextFunc(cbid, &context, &nextcbid)) != NULL);
    } /* clientCBFunc != NULL */

    /* push all the external CBs to mailbox and return, for EACH CB make a
       COPY of response data and this data will be free at the CBthread when
       the app's cb return */

    clientCBFunc = (NPF_EVENT_CBFUNC)pdk_callback_firstFunc(	NPF_EVT_FE,
								&context,
								&nextcbid);
    /* check in case user de-register */
    if (clientCBFunc != NULL) {
    do {
        /* new rsp data need to be deleted in CBThread when App's CB done */
        if ((outfe = (NPF_FE_t *)PIL_AllocateMemory(sizeof(NPF_FE_t), 
                                               PIL_MEM_ZERO_INIT)) == NULL)
            assert(0);
        if (cm_copyFromPDK(fehandle, (NPF_DATA)outfe) == -1)
            assert(0);

        cbid = nextcbid;
        cbthread_pushEventCB(	clientCBFunc,
				context,
				fehandle,
				NPF_EVENT_FE_BIND,
				(NPF_DATA)outfe	);
    } while ((clientCBFunc = (NPF_EVENT_CBFUNC)
              pdk_callback_nextFunc(cbid, &context, &nextcbid)) != NULL);
    } /* clientCBFunc != NULL */

    /* fire NPF_EVENT_FE_UP event for internal then external */
    /* Q: do we want to wait a second here? */
    npf_logger_write(LOG_CM, LOG_INFO,
                     "Delay Transport Thread to have ConfigApp finish onCB");
    PIL_DelayThread((uint32_t)1000);
    onCB_event_FE(fehandle, INTERNAL_CB, NPF_EVENT_FE_UP);	/* internal */
    onCB_event_FE(fehandle, EXTERNAL_CB, NPF_EVENT_FE_UP);	/* external */

    CM_UNLOCK();
    return;

error:
    CM_UNLOCK();
    return;
}

/* Needs callback all the pending request and set all the
 * ports, FE as well as related interface off if necessary
 * Fire IF_DELETE if all the if's ports are deleted in this FE
 * Don't delete any instance here, just set inactive
 *
 * After FEUnbind, FE.status, Port.status, Port.linkstatus are off */
void onCB_event_FEUnBind(
        FPPI_FEID       feid,
        FPPI_CORRELATOR correlator,
        FPPI_CONTEXT    context,
        void            *response)
{
    int             i, j, k, num_ports, num_ifs, tmp_numports, portlinkup;
    int             restartItr;
    DListItr        *itr;
    NPF_DATA        data;
    NPF_NSHDL       sys_nsh;
    NPF_HANDLE      syshandle,fehandle,porthlist[MAXFEOWNPORT],ifhlist[MAXIFOWNPORT];
    NPF_HANDLE      tmp_feh, tmp_porthlist[MAXIFOWNPORT];
    PDK_PORT_t      *port, *tmp_port;
    PDK_INTERFACE_t *interface;
    PDK_FE_t        *fe;
    CMCORRELATOR    tmp_correlator;

    num_ports = num_ifs = tmp_numports = 0;

    /* check if the context is correct */
    if ((CMCLIENTCONTEXT)context != CM_CC_EVENT_FEUNBIND)
        return;
    CM_LOCK();
    /* get FE handle & PDK_FE_t */
    if (cm_getFEHandleById(feid, &fehandle) == -1) 
        goto error;
    if (cm_getDataByHandle(fehandle, (NPF_DATA *)&fe) == -1)
        goto error;

    /* Need callback all the pending request NPF_FAIL here
     * to the one related to this unbinded FE.
     * Note: But don't fire callback failure if other FE that involved 
     * in this pending request is still UP and waiting for response */
    do {
    restartItr = 0;
    itr = npf_list_itrCreate(&g_outstandingRequest);
    if (npf_list_itrFirst(itr) != -1) {
        /* iterate the g_outstandingRequest */
        do {
            tmp_correlator = (CMCORRELATOR)npf_list_itrGetData(itr);
            /* pick one pending correlator, loop it's involved FEs */
            for (i=0; i<tmp_correlator->numSentFE; i++) {
                if (tmp_correlator->waitFEid[i] == feid)
                    /* mark as unbind */
                    tmp_correlator->waitFEid[i] = REQUEST_FE_UNBIND;
            }
            /* if all of the involved FEs are fail/unbind/done, removed it */
            if (!isStillBindWait(tmp_correlator)) {
                /* callback fail and free correlator in cm_requestFailCB */
                npf_list_itrRemoveFree(itr);
                /* iterator invalid after Remove, re-loop g_outstandingReq */
                restartItr = 1;
            }
        } while (!restartItr && npf_list_itrNext(itr) != -1);
    }
    npf_list_itrDelete(itr);
    } while (restartItr);

    /* Fake PortLinkDown event for FPPI, this may trigger related IF off */
    cm_getPortByFE(fehandle, &num_ports, porthlist);
    for (i=0; i<num_ports; i++) {
        cm_getDataByHandle(porthlist[i], (NPF_DATA *)&port);
        onCB_event_PortLinkDown(
                feid,
                0,    /* no correlator is needed */
                (FPPI_CORRELATOR)CM_CC_EVENT_LINKDOWN,
                &port->attribs.id);    /* this should be pointer to portid */
    }

    /* some Port may still on, just link off, so set all Port.status off 
     * and if it's interface is on also check no other
     * associated ports under this interface is both link+port on, set the
     * interface off and fire IF_DOWN event */
    for (i=0; i<num_ports; i++) {
        cm_getDataByHandle(porthlist[i], (NPF_DATA *)&port);
        /* have fired linkdown event above */
        if (port->attribs.linkstatus == 1) assert(0);
        if (port->attribs.status == 0) continue;
        port->attribs.status = 0;
        /* get all the interfaces associated to this port */
        cm_getIFByPort(porthlist[i], &num_ifs, ifhlist);
        for (j=0; j<num_ifs; cm_handle_close(ifhlist[j++])) {
            cm_getDataByHandle(ifhlist[j], (NPF_DATA *)&interface);
            if (interface->status == NPF_IF_DOWN) continue;
            /* ok, this IF is still on, check if other assoc. ports is on */
            cm_getPortByIF(ifhlist[j], &tmp_numports, tmp_porthlist);
            portlinkup = 0;
            for (k=0; k<tmp_numports; cm_handle_close(tmp_porthlist[k++])) {
                cm_getDataByHandle(tmp_porthlist[k], (NPF_DATA)&tmp_port);
                if (tmp_port->attribs.linkstatus == 1 &&
                    tmp_port->attribs.status == 1)
                    portlinkup = 1;
            } /* done the loop of all ports in one interface */
            /* if all the ports are off, fire interface down event */
            if (!portlinkup) {
                interface->status = NPF_IF_DOWN;
                /* fire interface up event */
                onCB_event_IF(ifhlist[j], INTERNAL_CB, NPF_EVENT_IF_DOWN);
                onCB_event_IF(ifhlist[j], EXTERNAL_CB, NPF_EVENT_IF_DOWN);
            }
        } /* done loop of all the interface in this port */
    } /* done loop of all the ports in this FE */

    /* all ports are off and also all interfaces which only connects to
     * this FE are off already. Find them out and fire a IF_DELETE event
     * to VIDD to delete virtual interface */
    for (i=0; i<num_ports; i++) {
        cm_getIFByPort(porthlist[i], &num_ifs, ifhlist);
        for (j=0; j<num_ifs; cm_handle_close(ifhlist[j++])) {
            /* this getSingleFEPortByIF only success when all the ports
             * associated to this IF belong to same FE */
            if (cm_getSingleFEPortByIF(
                    ifhlist[j],
                    &tmp_feh,
                    &tmp_numports,
                    tmp_porthlist) != NPF_SUCCESS) {
                for (k=0; k<tmp_numports; k++)
                    cm_handle_close(tmp_porthlist[k]);
                continue;
            }
            /* if interface only associated with ports with all belong
             * to a same FE, and this FE is current unbind FE, then
             * fire the internal IF_DELETE event, no EXTERNAL!!
             * after fire the NPF_EVENT_IF_DELETE, the vidd should
             * reset the viname to NULL */
            if (fehandle == tmp_feh)
                onCB_event_IF(ifhlist[j], INTERNAL_CB, NPF_EVENT_IF_DELETE);
        } /* done the loop of all interface associated with this port */
    } /* done the loop of all the ports in this down FE */

    /* HACK - we have problem when user explicitly delete the port to
     * interface association, the IF_DELETE event will not fired since
     * C&M doesn't know how the topology looks like, which interface
     * should fire the delete event when a FE goes down.
     *
     * The hack is we assume there is only one FE, so all the virtual intf
     * should be deleted when this FE unbind. If a interfaces's vifname
     * is not "", that means it didn't fire IF_DELETE before, VIDD didn't
     * re-set the vifname. So we need explicitly fire the IF_DELETE on
     * those interfaces. This will happen when user delete the IF-PORT
     * association after set ipaddr and before FE unbind */

    cm_deleteIFEvent();

    /* close all the port handle opened from getPortByFE */
    for (i=0; i<num_ports; i++)
        cm_handle_close(porthlist[i]);

    /* set FE.status off */
    fe->attribs.status = NPF_FE_DOWN;

    /* decrement the System.num_FEs after set FE to Down */
    if (pdk_ns_getNSRootHandle(&sys_nsh) != NPF_SUCCESS ||
        npf_ns_getDataHandle(sys_nsh, &syshandle) != NPF_SUCCESS ||
        cm_getDataByHandle(syshandle, &data) != NPF_SUCCESS ||
        ((PDK_SYSTEM_t *)data)->num_FEs <= 0)
        assert(0);
    ((PDK_SYSTEM_t *)data)->num_FEs--;
    npf_ns_close(sys_nsh);
    
    /* Q: Already disable FE/Ports/IFs here, and fire Port_LinkDown
     *    and IF_Down event already. Is it necessary to fire FE_Down
     *    event? Who is interested?
     *    Same situation, don't know what does FE_Up event mean too.
     * A: Looks internal components need FE_DOWN to cb all pendind Req.
     *    And ConfigApp may want to register FE_DOWN to delete all NS node */
    npf_logger_write(LOG_CM, LOG_INFO,
                     "Delay Transport Thread to have ConfigApp finish onCB");
    PIL_DelayThread((uint32_t)1000);
    onCB_event_FE(fehandle, 1, NPF_EVENT_FE_DOWN);	/* internal */
    onCB_event_FE(fehandle, 0, NPF_EVENT_FE_DOWN);	/* external */

    CM_UNLOCK();
    return;

error:
    CM_UNLOCK();
    return;
}

/* Check if all the link belongs to the same interface are down.
 * If so fire a interface up event otherwise only the one registered
 * port event will get it
 * port can only admin up(off), link can only event up(off)
 * interface can both admin up(off) or event up(off), admin changes
 * port status but link event doesn't
 * Q: what is the linkstatus when new FE bind?
 * A: port.status, port.linkstatus and FE.status are on, if.status off
 *
 * Response Data: NPF_PORT_t */
void onCB_event_PortLinkUp(
        FPPI_FEID       feid,
        FPPI_CORRELATOR correlator,
        FPPI_CONTEXT    context,
        void*           response)
{

    int              i, j, result, num_ports, num_ifs;
    NPF_CBCAT        cat;
    NPF_HANDLE       porth, feh;
    NPF_HANDLE       porthlist[MAXFEOWNPORT], ifhlist[MAXIFOWNPORT];
    NPF_CBHANDLE     cbid, nextcbid;
    NPF_USERCONTEXT  usercontext;
    NPF_EVENT_CBFUNC clientCBFunc;
    PDK_PORT_t       *port;
    NPF_PORT_t       *outport;
    PDK_INTERFACE_t  *interface;
    FPPI_PortID      portid;

    /* check if the context is correct */
    if ((CMCLIENTCONTEXT)context != CM_CC_EVENT_LINKUP)
        return;

    /* FPPI response is ptr to PortID */
    portid = *((int *)response);

    npf_logger_write(LOG_CM, LOG_INFO, "PortLinkUp on FE %d Port %d",
            feid, portid);
    printf("PortLinkUp on FE %d Port %d", (int)feid, (int)portid);

    CM_LOCK();
    /* search for fe and port handle */
    if (cm_getFEHandleById(feid, &feh) != NPF_SUCCESS)
        goto finish;
    if (cm_getPortByFE(feh, &num_ports, porthlist) != NPF_SUCCESS)
        goto finish;
    result = 0;
    for (i=0; i<num_ports; cm_handle_close(porthlist[i++])) {
        if (cm_getDataByHandle(porthlist[i], (NPF_DATA *)&port)!=NPF_SUCCESS)
            continue;
        if (port->attribs.id == portid) {
            /* Q:if the linkstatus was up before, ignore this event? */
            if (port->attribs.linkstatus) {
                /* close the rest of the port handle got from getPortByFE */
                for (j=i; j<num_ports; j++)
                    cm_handle_close(porthlist[j]);
                goto finish;
            }
            port->attribs.linkstatus = 1;
            /* no callback if port is admin off */
            if (!port->attribs.status) {
                /* close the rest of the port handle got from getPortByFE */
                for (j=i; j<num_ports; j++)
                    cm_handle_close(porthlist[j]);
                goto finish;
            }
            porth = porthlist[i];
            result = 1;
            /* close the rest of the port handle got from getPortByFE
             * but leave the current open for notification, close it later */
            for (j=i+1; j<num_ports; j++)
                cm_handle_close(porthlist[j]);
            break;
        }
    }

    /* can't find the port in this FE */
    if (!result) goto finish;

    for (i=0, cat=PDK_EVT_PORT; i<2; i++, cat=NPF_EVT_PORT) {

        /* walk the CBs to execute all the callback func */
        clientCBFunc = (NPF_EVENT_CBFUNC)pdk_callback_firstFunc(
		cat,
		&usercontext,
		&nextcbid);

        /* once registered it must not be NULL, check in case de-register */
        if (clientCBFunc != NULL) {
        do {
            /* create a new NPF_FE_t for every internal callback */
            if ((outport = (NPF_PORT_t *)PIL_AllocateMemory
                        (sizeof(NPF_PORT_t), PIL_MEM_ZERO_INIT)) == NULL)
                assert(0);

            if (cm_copyFromPDK(porth, (NPF_DATA)outport) != NPF_SUCCESS)
                assert(0);

            cbid = nextcbid;
            if (cat == PDK_EVT_PORT) {
                clientCBFunc(	usercontext,
 				porth,
				NPF_EVENT_PORT_LINKUP,
				(NPF_DATA)outport	);
                PIL_FreeMemory(outport);
            } else {
                cbthread_pushEventCB(	clientCBFunc,
					usercontext,
					porth,
					NPF_EVENT_PORT_LINKUP,
					(NPF_DATA)outport	);
            }
        } while ((clientCBFunc = (NPF_EVENT_CBFUNC)
              pdk_callback_nextFunc(cbid, &usercontext, &nextcbid)) != NULL);
        } /* clientCBFunc != NULL */
    }

    if (port->attribs.type != NPF_PORT_ETHERNET)
    {
        cm_handle_close(porth);
        goto finish;
    }

    /* Q: fire interface up event after or before link up? */
    /* fire interface up event if port is admin on but intf down(not admin),
     * and if we reach here the port is absolutely on, otherwise we've
     * returned at early this function */
    cm_getIFByPort(porth, &num_ifs, ifhlist);
    for (i=0; i<num_ifs; cm_handle_close(ifhlist[i++])) {
        if (cm_getDataByHandle(ifhlist[i], (NPF_DATA)&interface)
            != NPF_SUCCESS)
            continue;
        if (interface->type != NPF_INTERFACE)
        {
            continue;
        }
        if (interface->status == NPF_IF_DOWN) {
            interface->status = NPF_IF_UP;
            /* fire interface up event */
            onCB_event_IF(ifhlist[i], INTERNAL_CB, NPF_EVENT_IF_UP);
            onCB_event_IF(ifhlist[i], EXTERNAL_CB, NPF_EVENT_IF_UP);
        }
    }

    /* close the handle of this link now */
    cm_handle_close(porth);

finish:

    CM_UNLOCK();
    return;

}

/* if port is admin off, no callback.
 * if port is admin on but interface is off, callback linkdown
 * if port is admin on and interface is on, callback linkdown and 
 * check all other ports assoc. with this same interface, if their
 * ports are not BOTH link and port on, set interface status off 
 * and fire interface down event
 *
 * Response Data: NPF_PORT_t */
void onCB_event_PortLinkDown(
        FPPI_FEID       feid,
        FPPI_CORRELATOR correlator,
        FPPI_CONTEXT    context,
        void*           response)
{

    int              i, j, result, num_ports, num_ifs;
    NPF_CBCAT        cat;
    NPF_HANDLE       porth, feh;
    NPF_HANDLE       porthlist[MAXFEOWNPORT], ifhlist[MAXIFOWNPORT];
    NPF_CBHANDLE     cbid, nextcbid;
    NPF_USERCONTEXT  usercontext;
    NPF_EVENT_CBFUNC clientCBFunc;
    PDK_PORT_t       *port;
    NPF_PORT_t       *outport;
    PDK_INTERFACE_t  *interface;
    FPPI_PortID      portid;

    /* check if the context is correct */
    if ((CMCLIENTCONTEXT)context != CM_CC_EVENT_LINKDOWN)
        return;

    /* FPPI response data is ptr to PortID */
    portid = *((int *)response);

    npf_logger_write(LOG_CM, LOG_INFO, "PortLinkDown on FE %d Port %d",
            feid, portid);
    printf("PortLinkDown on FE %d Port %d", (int)feid, (int)portid);

    CM_LOCK();

    /* search for fe and port handle */
    if (cm_getFEHandleById(feid, &feh) != NPF_SUCCESS)
        goto finish;
    if (cm_getPortByFE(feh, &num_ports, porthlist) != NPF_SUCCESS)
        goto finish;
    result = 0;
    for (i=0; i<num_ports; cm_handle_close(porthlist[i++])) {
        if (cm_getDataByHandle(porthlist[i], (NPF_DATA *)&port)!=NPF_SUCCESS)
            continue;
        if (port->attribs.id == portid) {
            /* Q: if it was down before, ignore this up event? */
            if (!port->attribs.linkstatus) {
                /* close the rest of the port handle got from getPortByFE */
                for (j=i; j<num_ports; j++)
                    cm_handle_close(porthlist[j]);
                goto finish;
            }
            port->attribs.linkstatus = 0;
            /* no callback if port is admin off */
            if (!port->attribs.status) {
                /* close the rest of the port handle got from getPortByFE */
                for (j=i; j<num_ports; j++)
                    cm_handle_close(porthlist[j]);
                goto finish;
            }
            porth = porthlist[i];
            result = 1;
            /* close the rest of the port handle got from getPortByFE 
             * But leave the current in open for notification, close later */
            for (j=i+1; j<num_ports; j++)
                cm_handle_close(porthlist[j]);
            break;
        }
    }

    for (i=0, cat=PDK_EVT_PORT; i<2; i++, cat=NPF_EVT_PORT) {

        /* walk the CBs to execute all the callback func */
        clientCBFunc = (NPF_EVENT_CBFUNC)pdk_callback_firstFunc(
		cat,
		&usercontext,
		&nextcbid);

        /* once registered it must not be NULL, check in case de-register */
        if (clientCBFunc != NULL) {
        do {
            /* create a new NPF_FE_t for every internal callback */
            if ((outport = (NPF_PORT_t *)PIL_AllocateMemory
                        (sizeof(NPF_PORT_t), PIL_MEM_ZERO_INIT)) == NULL)
                assert(0);

            if (cm_copyFromPDK(porth, (NPF_DATA)outport) != NPF_SUCCESS)
                assert(0);

            cbid = nextcbid;
            if (cat == PDK_EVT_PORT) {
                clientCBFunc(	usercontext,
 				porth,
				NPF_EVENT_PORT_LINKDOWN,
				(NPF_DATA)outport	);
                PIL_FreeMemory(outport);
            } else {
                cbthread_pushEventCB(	clientCBFunc,
					usercontext,
					porth,
					NPF_EVENT_PORT_LINKDOWN,
					(NPF_DATA)outport	);
            }
        } while ((clientCBFunc = (NPF_EVENT_CBFUNC)
              pdk_callback_nextFunc(cbid, &usercontext, &nextcbid)) != NULL);
        } /* clientCBFunc != NULL */
    }

    if (port->attribs.type != NPF_PORT_ETHERNET)
    {
        cm_handle_close(porth);
        goto finish;
    }

    /* Q: fire interface down event after or before link up? */
    /* fire interface down event if this port is admin on and it's interface
     * is on but all the other associated ports are NOT both port.status
     * and port.linkstatus on
     * port must be admin on here otherwise already returned early */
    cm_getIFByPort(porth, &num_ifs, ifhlist);
    for (i=0; i<num_ifs; cm_handle_close(ifhlist[i++])) {
        if (cm_getDataByHandle(ifhlist[i], (NPF_DATA)&interface)
            != NPF_SUCCESS)
            continue;
        if (interface->status == NPF_IF_DOWN)
            continue;
        /* check if all the associated port are link off */
        if (cm_getPortByIF(ifhlist[i], &num_ports, porthlist)
                != NPF_SUCCESS) {
            /* close the rest of the if handle got from getIFByPort */
            for (j=i; j<num_ifs; j++)
                cm_handle_close(ifhlist[j]);
            goto finish;
        }
        result = 0;
        for (j=0; j<num_ports && !result; cm_handle_close(porthlist[j++])) {
            cm_getDataByHandle(porthlist[j], (NPF_DATA)&port);
            if (port->attribs.linkstatus == 1 &&
                port->attribs.status == 1)
                result = 1;
        } /* done the loop of all ports in one interface */
        /* if all the ports are off, fire interface down event */
        if (!result) {
            interface->status = NPF_IF_DOWN;
            /* fire interface up event */
            onCB_event_IF(ifhlist[i], INTERNAL_CB, NPF_EVENT_IF_DOWN);
            onCB_event_IF(ifhlist[i], EXTERNAL_CB, NPF_EVENT_IF_DOWN);
        }
    } /* done check all the interface associated to this linkdown port */

    /* close the current port down handle */
    cm_handle_close(porth);

finish:
    CM_UNLOCK();
    return;
}

/* only fired by onCB_event_FEBind or onCB_event_FEUnBind 
 * after internal/external cb are done
 * DO NOT register it, no cmcorrelator
 *
 * Response Data: NPF_FE_t */
void onCB_event_FE(NPF_HANDLE feh, int isInternal, NPF_EVENT event)
{
    NPF_CBHANDLE     cbid, nextcbid;
    NPF_USERCONTEXT  context;
    NPF_FE_t         *outfe;
    NPF_EVENT_CBFUNC clientCBFunc;

    if (isInternal)
    npf_logger_write(LOG_CM, LOG_INFO, "FE Handle %d %s", feh,
        (event==NPF_EVENT_FE_UP)?"Up":"Down");
    
    /* walk the internal CB to execute all the callback func in FPPI thread */
    clientCBFunc = (NPF_EVENT_CBFUNC)pdk_callback_firstFunc(
		(isInternal==1)?PDK_EVT_FE:NPF_EVT_FE,
		&context,
		&nextcbid);
    /* once registered it must not be NULL, check if de-register */
    if (clientCBFunc != NULL) {
    do {
        /* create a new NPF_INTERFACE_t for every internal callback */
        if ((outfe = (NPF_FE_t *)PIL_AllocateMemory
                     (sizeof(NPF_FE_t), PIL_MEM_ZERO_INIT)) == NULL)
            assert(0);
        if (cm_copyFromPDK(feh, (NPF_DATA)outfe) == -1)
            assert(0);

        cbid = nextcbid;
        /* internal just call it, external needs to push to callback thread */
        if (isInternal) {
            clientCBFunc(	context,
 				feh,
				event,
				(NPF_DATA)outfe	);

            PIL_FreeMemory(outfe);
        } else {
            /* push all the external CBs to mailbox and return, for 
               EACH CB make a COPY of response data and this data 
               will be free at the CBthread when the app's cb return */
            cbthread_pushEventCB(	clientCBFunc,
					context,
					feh,
					event,
					(NPF_DATA)outfe	);
        }
    } while ((clientCBFunc = (NPF_EVENT_CBFUNC)
              pdk_callback_nextFunc(cbid, &context, &nextcbid)) != NULL);
    } /* clientCBFunc != NULL */

    return;
}

/* fired by onCB_if_set_ipaddr, onCB_if_set_ipfwd
 * or onCB_event_PortLinkUp, PortLinkDown
 * DO NOT register it, no cmcorrelator
 * isInternal = 1 means fire the PDK_EVT_IF category, otherwise NPF_EVT_IF
 * event could be NPF_EVENT_IF_IPFORWARDING_CHANGE or NPF_EVENT_IF_IPADDR_CHANGE
 * or NPF_EVENT_IF_DELETE
 *
 * Response Data: NPF_INTERFACE_t */
void onCB_event_IF(NPF_HANDLE ifh, int isInternal, NPF_EVENT event)
{
    NPF_CBHANDLE     cbid, nextcbid;
    NPF_USERCONTEXT  context;
    NPF_INTERFACE_t  *outif;
    NPF_EVENT_CBFUNC clientCBFunc;

    if (isInternal == INTERNAL_CB)
    switch (event) {
    case NPF_EVENT_IF_UP:
        npf_logger_write(LOG_CM,LOG_INFO,"Interface Handle %d Up", ifh);
        break;
    case NPF_EVENT_IF_DOWN:
        npf_logger_write(LOG_CM,LOG_INFO,"Interface Handle %d Down", ifh);
        break;
    case NPF_EVENT_IF_DELETE:
        npf_logger_write(LOG_CM,LOG_INFO,"Interface Handle %d Delete", ifh);
        break;
    case NPF_EVENT_IF_IPADDR_CHANGE:
        npf_logger_write(LOG_CM,LOG_INFO,"Interface Handle %d IP Changed",ifh);
        break;
    default:
    }

    /* walk the internal CB to execute all the callback func in FPPI thread */
    clientCBFunc = (NPF_EVENT_CBFUNC)pdk_callback_firstFunc(
		(isInternal == INTERNAL_CB)?PDK_EVT_IF:NPF_EVT_IF,
		&context,
		&nextcbid);
    /* once registered it must not be NULL, check in case de-register */
    if (clientCBFunc != NULL) {
    do {
        /* create a new NPF_INTERFACE_t for every internal callback */
        if ((outif = (NPF_INTERFACE_t *)PIL_AllocateMemory
                     (sizeof(NPF_INTERFACE_t), PIL_MEM_ZERO_INIT)) == NULL)
            assert(0);
        if (cm_copyFromPDK(ifh, (NPF_DATA)outif) == -1)
            assert(0);

        cbid = nextcbid;

        /* internal just call it, external needs to push to callback thread */
        if (isInternal == INTERNAL_CB) {
            clientCBFunc(	context,
 				ifh,
				event,
				(NPF_DATA)outif	);

            PIL_FreeMemory(outif);
        } else {
            /* push all the external CBs to mailbox and return, for EACH 
               CB make a COPY of response data and this data will be free 
               at the CBthread when the app's cb return */
            cbthread_pushEventCB(	clientCBFunc,
					context,
					ifh,
					event,
					(NPF_DATA)outif	);

        }
    } while ((clientCBFunc = (NPF_EVENT_CBFUNC)
              pdk_callback_nextFunc(cbid, &context, &nextcbid)) != NULL);
    } /* clientCBFunc != NULL */

    return;
}


/* fired by onCB_port_set_mtu, onCB_port_set_linkspeed
 * DO NOT register it, no cmcorrelator
 * isInternal = 1 means fire the PDK_EVT_PORT category, otherwise NPF_EVT_PORT
 * possible event: NPF_EVENT_PORT_MTU_CHANGE,NPF_EVENT_PORT_LINKSPEED_CHANGE
 *
 * Response Data: NPF_PORT_t */
void onCB_event_PORT(NPF_HANDLE porth, int isInternal, NPF_EVENT event)
{
    NPF_CBHANDLE     cbid, nextcbid;
    NPF_USERCONTEXT  context;
    NPF_PORT_t       *outport;
    NPF_EVENT_CBFUNC clientCBFunc;

    if (isInternal == INTERNAL_CB)
    switch (event) {
    case NPF_EVENT_PORT_MTU_CHANGE:
        npf_logger_write(LOG_CM,LOG_INFO,"Port Handle %d MTU Changed", porth);
        break;
    case NPF_EVENT_PORT_LINKSPEED_CHANGE:
        npf_logger_write(LOG_CM,LOG_INFO,"Port Handle %d Speed Changed", porth);
        break;
    default:
    }

    /* walk the internal CB to execute all the callback func in FPPI thread */
    clientCBFunc = (NPF_EVENT_CBFUNC)pdk_callback_firstFunc(
		(isInternal == INTERNAL_CB)?PDK_EVT_PORT:NPF_EVT_PORT,
		&context,
		&nextcbid);
    /* once registered it must not be NULL, check in case de-register */
    if (clientCBFunc != NULL) {
    do {
        /* create a new NPF_INTERFACE_t for every internal callback */
        if ((outport = (NPF_PORT_t *)PIL_AllocateMemory
                     (sizeof(NPF_PORT_t), PIL_MEM_ZERO_INIT)) == NULL)
            assert(0);
        if (cm_copyFromPDK(porth, (NPF_DATA)outport) == -1)
            assert(0);

        cbid = nextcbid;

        /* internal just call it, external needs to push to callback thread */
        if (isInternal == INTERNAL_CB) {
            clientCBFunc(	context,
 				porth,
				event,
				(NPF_DATA)outport	);

            PIL_FreeMemory(outport);
        } else {
            /* push all the external CBs to mailbox and return, for EACH 
               CB make a COPY of response data and this data will be free 
               at the CBthread when the app's cb return */
            cbthread_pushEventCB(	clientCBFunc,
					context,
					porth,
					event,
					(NPF_DATA)outport	);

        }
    } while ((clientCBFunc = (NPF_EVENT_CBFUNC)
              pdk_callback_nextFunc(cbid, &context, &nextcbid)) != NULL);
    } /* clientCBFunc != NULL */

    return;
}

/* Need fire NPF_EVENT_IF_IPADDR_CHANGE event,
 * in evt -> ext cb -> ext evt
 *
 * Response Data: NULL */
void onCB_if_set_ipaddr(uint32_t	feid,
			CMCORRELATOR	cmcorrelator,
			CMCONTEXT	cmcontext,
			NPF_DATA	data	)
{
    int             i, num_setifs;
    NPF_IF_STATUS   prevStatus;
    NPF_HANDLE      setifhlist[MAXIFOWNPORT];
    PDK_INTERFACE_t *interface;

    assert(cmcontext);

    /* cmcorrelator maybe freed before if any related FE communication
     * failed at the request time. Just discard this callback */
    if (!cmcorrelator) return;

    /* Validate if this callback indeed belongs to this type */
    if ((CMCLIENTCONTEXT)cmcontext != CM_CC_IF_SETIPADDR)
	return;

    CM_LOCK();

    /* Add: check RSP for failure response, if ANY PORT FAIL set 
     *      cmcorrelator->waitFEid[?] to REQUEST_FE_FAIL */

    /* Note: Return if still other FPPAPI callbacks is waiting */
    for (i=0; i<cmcorrelator->numSentFE; i++)
        if (cmcorrelator->waitFEid[i] == feid)
            cmcorrelator->waitFEid[i] = REQUEST_FE_DONE;
    if (isStillBindWait(cmcorrelator)) goto finish;

    /* Remove the correlator from g_outstandingRequest but don't free it */
    if (cm_removeFromRequest(cmcorrelator) == -1)
        assert(0);

    /* Add: check if all FEs are DONE or some FAIL? Trouble if any FE FAIL.
     *      rollback the successful FEs and callback fail if not NPF_NOACK
     *      must free cmcorrelator->data & cmcorrelator before return */

    /* assign system data */
    if (cm_getDataByHandle(cmcorrelator->handle, (void **)&interface) == -1)
        goto finish;
    /* make a copy here since we've allocate at the new data time */
    memcpy(&interface->ipattribs.addr_u.ipaddr, cmcorrelator->data, sizeof(IPA));
    memcpy(&interface->ipattribs.mask_u.submask,
        (char *)(cmcorrelator->data)+sizeof(IPA), sizeof(IPA));

    /* Q: Do we still modify the system data if later client CB fail? */

    /* IMPORTANT:
     * In Linux, if do ifconfig eth0 IPA after ifdown eth0, the eth0
     * will ifup automatically. Same here, after the set ipaddr success
     * callback, ASSUME all the related ports in FEs are automatically
     * turn on no matter what status it was. So now we need to set
     * the interface flag on. After that, if it was down before, then
     * fire IF_UP event, otherwise fire IPADDR_CHANGE event
     * This doesn not apply in IPv4 addroute case, if interface is down,
     * the addroute will fail, it won't automatically turn the intf on
     */
    prevStatus = interface->status;
    cm_if_setflag(cmcorrelator->handle, NPF_IF_UP, &num_setifs, setifhlist);
    if (prevStatus == NPF_IF_DOWN) {
        for (i=0; i<num_setifs; i++)
            onCB_event_IF(setifhlist[i], INTERNAL_CB, NPF_EVENT_IF_UP);
    } else if (prevStatus == NPF_IF_UP) {
        for (i=0; i<num_setifs; i++) {
            if (setifhlist[i] == cmcorrelator->handle)
                onCB_event_IF(setifhlist[i], INTERNAL_CB,
                        NPF_EVENT_IF_IPADDR_CHANGE);
            else
                onCB_event_IF(setifhlist[i], INTERNAL_CB, NPF_EVENT_IF_UP);
        }
    } else {
        assert(0);
    }

    /* fire external NPF_EVENT_IF_IPADDR_CHANGE or IF_UP event */
    if (prevStatus == NPF_IF_DOWN) {
        for (i=0; i<num_setifs; i++)
            onCB_event_IF(setifhlist[i], EXTERNAL_CB, NPF_EVENT_IF_UP);
    } else if (prevStatus == NPF_IF_UP) {
        for (i=0; i<num_setifs; i++) {
            if (setifhlist[i] == cmcorrelator->handle)
                onCB_event_IF(setifhlist[i], EXTERNAL_CB,
                        NPF_EVENT_IF_IPADDR_CHANGE);
            else
                onCB_event_IF(setifhlist[i], EXTERNAL_CB, NPF_EVENT_IF_UP);
        }
    } else {
        assert(0);
    }

    /* close all the setifhlist[] handle which was open in if_set_flag */
    for (i=0; i<num_setifs; i++)
        cm_handle_close(setifhlist[i]);

    /* Q: is it right position to free cmcorrelator here? */
    /* Note: Free both cmcorrelator and cmcorrelator->data */
    PIL_FreeMemory(cmcorrelator->data);
    PIL_FreeMemory(cmcorrelator);
     
finish:
    CM_UNLOCK();
    return;
}

void onCB_if_del_ipaddr(uint32_t	feid,
			CMCORRELATOR	cmcorrelator,
			CMCONTEXT	cmcontext,
			NPF_DATA	data	)
{
    int             i, j, num_ports, num_ifs;
    NPF_HANDLE      porthlist[MAXIFOWNPORT], ifhlist[MAXIFOWNPORT];
    PDK_INTERFACE_t *interface;
    PDK_PORT_t      *port;
    NPF_IF_STATUS   prevStatus;

    assert(cmcontext);

    /* cmcorrelator maybe freed before if any related FE communication
     * failed at the request time. Just discard this callback */
    if (!cmcorrelator) return;

    /* Validate if this callback indeed belongs to this type */
    if ((CMCLIENTCONTEXT)cmcontext != CM_CC_IF_DELIPADDR)
	return;

    CM_LOCK();

    /* Add: check RSP for failure response, if ANY PORT FAIL set 
     *      cmcorrelator->waitFEid[?] to REQUEST_FE_FAIL */

    /* Note: Return if still other FPPAPI callbacks is waiting */
    for (i=0; i<cmcorrelator->numSentFE; i++)
        if (cmcorrelator->waitFEid[i] == feid)
            cmcorrelator->waitFEid[i] = REQUEST_FE_DONE;
    if (isStillBindWait(cmcorrelator)) goto finish;

    /* Remove the correlator from g_outstandingRequest but don't free it */
    if (cm_removeFromRequest(cmcorrelator) == -1)
        assert(0);

    /* Add: check if all FEs are DONE or some FAIL? Trouble if any FE FAIL.
     *      rollback the successful FEs and callback fail if not NPF_NOACK
     *      must free cmcorrelator->data & cmcorrelator before return */

    /* assign system data */
    if (cm_getDataByHandle(cmcorrelator->handle, (NPF_DATA *)&interface) == -1)
        goto finish;
    /* make a copy here since we've allocate at the new data time */
    memset(&interface->ipattribs.addr_u.ipaddr, 0, sizeof(IPA));
    memset(&interface->ipattribs.mask_u.submask, 0, sizeof(IPA));

    /* Q: Do we still modify the system data if later client CB fail? */

    /* IMPORTANT:
     * After deleting an IPA, must set this interface DOWN.
     * ASSUME on the FE, delete the last ipaddress of the port will
     * automatically DOWN this port, so check all the ports associated
     * with this interface, if this interface is the ONLY interface
     * that associates with him (means there is only one IPA on this
     * port), and also this port was on before. Then needs to set this
     * port DOWN. But if the associated port has more than one IF
     * associated with it, we don't care whether it was ON or OFF at all.
     * It should remain what it was before
     * Needs fire IF DOWN event, but we don't have PORT down event but
     * link down event, we don't need consider fire linkdown event here */
    prevStatus = interface->status;
    interface->status = NPF_IF_DOWN;
    cm_getPortByIF(cmcorrelator->handle, &num_ports, porthlist);
    for (i = 0; i < num_ports; cm_handle_close(porthlist[i++])) {
        cm_getIFByPort(porthlist[i], &num_ifs, ifhlist);
        /* this port has only one assoc. intf and it's ipaddr has deleted
         * assume in the FE it already been automatically turn port down */
        if (num_ifs != 1 || ifhlist[0] != cmcorrelator->handle) {
            /* close all open interface */
            for (j = 0; j < num_ifs; j++)
                cm_handle_close(ifhlist[j]);
            continue;
        }
        if (cm_getDataByHandle(porthlist[i], (NPF_DATA *)&port) != -1 &&
            ((PDK_SHIM_t *)port)->type == NPF_PORT)
            port->attribs.status = 0;
        /* close all open interface */
        for (j = 0; j < num_ifs; j++)
            cm_handle_close(ifhlist[j]);
    }
    /* fire the internal interface down event if it was on before*/
    if (NPF_IF_UP == prevStatus)
        onCB_event_IF(cmcorrelator->handle, INTERNAL_CB, NPF_EVENT_IF_DOWN);

    /* fire external IF_DOWN event if the interface was UP before */
    if (NPF_IF_UP == prevStatus)
        onCB_event_IF(cmcorrelator->handle, EXTERNAL_CB, NPF_EVENT_IF_DOWN);

    /* Q: is it right position to free cmcorrelator here? */
    /* Note: Free both cmcorrelator and cmcorrelator->data */
    PIL_FreeMemory(cmcorrelator->data);
    PIL_FreeMemory(cmcorrelator);
     
finish:
    CM_UNLOCK();
    return;
}

void onCB_if_set_ipv6addr(uint32_t        feid,
                        CMCORRELATOR    cmcorrelator,
                        CMCONTEXT       cmcontext,
                        NPF_DATA        data    )
{
    printf("Not Supported yet\n");
    assert(0);
}

void onCB_if_del_ipv6addr(uint32_t        feid,
                        CMCORRELATOR    cmcorrelator,
                        CMCONTEXT       cmcontext,
                        NPF_DATA        data    )
{
    printf("Not Supported yet\n");
    assert(0);
}

/* Response Data: NULL */
void onCB_if_set_ipfwd(	uint32_t	feid,
			CMCORRELATOR	cmcorrelator,
			CMCONTEXT	cmcontext,
			NPF_DATA	data	)
{
    int             i;
    NPF_DATA        tmp_data;

    assert(cmcontext);

    /* cmcorrelator maybe freed before if any related FE communication
     * failed at the request time. Just discard this callback */
    if (!cmcorrelator) return;

    /* Validate if this callback indeed belongs to this type */
    if ((CMCLIENTCONTEXT)cmcontext != CM_CC_IF_SETIPFWD)
	return;

    CM_LOCK();

    /* Add: check RSP for failure response, if ANY PORT FAIL set 
     *      cmcorrelator->waitFEid[?] to REQUEST_FE_FAIL */

    /* Note: Return if still other FPPAPI callbacks is waiting */
    for (i=0; i<cmcorrelator->numSentFE; i++) 
        if (cmcorrelator->waitFEid[i] == feid)
            cmcorrelator->waitFEid[i] = REQUEST_FE_DONE;
    if (isStillBindWait(cmcorrelator)) goto finish;

    /* Remove the correlator from g_outstandingRequest but don't free it */
    if (cm_removeFromRequest(cmcorrelator) == -1)
        assert(0);

    /* Add: check if all FEs are DONE or some FAIL? Trouble if any FE FAIL
     *      rollback the successful FEs and callback fail if not NPF_NOACK
     *      must free cmcorrelator->data & cmcorrelator before return */

    /* assign system data */
    if (cm_getDataByHandle(cmcorrelator->handle, &tmp_data) == -1)
        goto finish;
    /* make a copy here since we've allocate at the new data time */
    memcpy(&((PDK_INTERFACE_t *)tmp_data)->ipattribs.ipfwding,
            cmcorrelator->data, sizeof(int));

    /* Q: Do we care about the PORT.status? Above is IF.statys */
    /* Q: Do we still modify the system data if later client CB fail? */

    /* fire internal NPF_EVENT_IF_IPFORWARDING_CHANGE event */
    onCB_event_IF(cmcorrelator->handle, INTERNAL_CB,
                      NPF_EVENT_IF_IPFORWARDING_CHANGE);

    /* fire external NPF_EVENT_IF_IPFORWARDING_CHANGE event */
    onCB_event_IF(cmcorrelator->handle, EXTERNAL_CB,
                      NPF_EVENT_IF_IPFORWARDING_CHANGE);

    /* Q: is it right position to free cmcorrelator here? */
    /* Note: Free both cmcorrelator and cmcorrelator->data */
    PIL_FreeMemory(cmcorrelator->data);
    PIL_FreeMemory(cmcorrelator);
     
finish:
    CM_UNLOCK();
    return;
}

/* Response Data: NPF_IF_STATS */
void onCB_if_get_statistics(	uint32_t	feid,
				CMCORRELATOR	cmcorrelator,
				CMCONTEXT	cmcontext,
				NPF_DATA	data	)
{
    int             i;
    NPF_IF_STATS    *ifportstats, *ifstats;

    assert(cmcontext);

    /* cmcorrelator maybe freed before if any related FE communication
     * failed at the request time. Just discard this callback */
    if (!cmcorrelator) return;

    /* Validate if this callback indeed belongs to this type */
    if ((CMCLIENTCONTEXT)cmcontext != CM_CC_IF_GETSTATS)
	return;

    CM_LOCK();

    /* Add: check RSP for failure response, if ANY PORT FAIL set 
     *      cmcorrelator->waitFEid[?] to REQUEST_FE_FAIL */

    /* Note: Return if still other FPPAPI callbacks is waiting */
    for (i=0; i<cmcorrelator->numSentFE; i++)
        if (cmcorrelator->waitFEid[i] == feid)
            cmcorrelator->waitFEid[i] = REQUEST_FE_DONE;

    /* sum the corrent statistics into the correlator.data */
    ifportstats = (NPF_IF_STATS *)data;
    ifstats = (NPF_IF_STATS *)cmcorrelator->data;
    ifstats->ifInOctets         += ifportstats->ifInOctets;
    ifstats->ifInUcastPkts      += ifportstats->ifInUcastPkts;
    ifstats->ifInDiscards       += ifportstats->ifInDiscards;
    ifstats->IfInErrors         += ifportstats->IfInErrors;
    ifstats->ifInUnknownProtos  += ifportstats->ifInUnknownProtos;
    ifstats->ifOutOctets        += ifportstats->ifOutOctets;
    ifstats->ifOutUcastPkts     += ifportstats->ifOutUcastPkts;
    ifstats->ifOutDiscards      += ifportstats->ifOutDiscards;
    ifstats->ifOutErrors        += ifportstats->ifOutErrors;
    ifstats->ifInMulticastPkts  += ifportstats->ifInMulticastPkts;
    ifstats->OutMulticastPkts   += ifportstats->OutMulticastPkts;
    ifstats->OutBroadcastPkts   += ifportstats->OutBroadcastPkts;
    ifstats->HCInOctets         += ifportstats->HCInOctets;
    ifstats->HCInUcastPkts      += ifportstats->HCInUcastPkts;
    ifstats->HCInMulticastPkts  += ifportstats->HCInMulticastPkts;
    ifstats->HCInBroadcastPkts  += ifportstats->HCInBroadcastPkts;
    ifstats->HCOutOctets        += ifportstats->HCOutOctets;
    ifstats->HCOutUcastPkts     += ifportstats->HCOutUcastPkts;
    ifstats->HCOutMulticastPkts += ifportstats->HCOutMulticastPkts;
    ifstats->HCOutBroadcastPkts += ifportstats->HCOutBroadcastPkts;

    if (isStillBindWait(cmcorrelator)) goto finish;

    /* Remove the correlator from g_outstandingRequest but don't free it */
    if (cm_removeFromRequest(cmcorrelator) == -1)
        assert(0);

    /* Add: check if all FEs are DONE or some FAIL? Trouble if any FE FAIL
     *      rollback the successful FEs and callback fail if not NPF_NOACK
     *      must free cmcorrelator->data & cmcorrelator before return */


    /* Q: is it right position to free cmcorrelator here? */
    /* Note: Free both cmcorrelator and cmcorrelator->data */
    PIL_FreeMemory(cmcorrelator->data);
    PIL_FreeMemory(cmcorrelator);
     
finish:
    CM_UNLOCK();
    return;
}

/* Response Data: NPF_ICMP_STATS */
void onCB_icmp_get_statistics(	uint32_t	feid,
				CMCORRELATOR	cmcorrelator,
				CMCONTEXT	cmcontext,
				NPF_DATA	data	)
{
    int              i;
    NPF_ICMP_STATS_t *icmpportstats, *icmpstats;

    assert(cmcontext);

    /* cmcorrelator maybe freed before if any related FE communication
     * failed at the request time. Just discard this callback */
    if (!cmcorrelator) return;

    /* Validate if this callback indeed belongs to this type */
    if ((CMCLIENTCONTEXT)cmcontext != CM_CC_ICMP_GETSTATS)
	return;

    CM_LOCK();

    /* Add: check RSP for failure response, if ANY PORT FAIL set 
     *      cmcorrelator->waitFEid[?] to REQUEST_FE_FAIL */

    /* Note: Return if still other FPPAPI callbacks is waiting */
    for (i=0; i<cmcorrelator->numSentFE; i++)
        if (cmcorrelator->waitFEid[i] == feid)
            cmcorrelator->waitFEid[i] = REQUEST_FE_DONE;

    /* sum the corrent statistics into the correlator.data */
    icmpportstats = (NPF_ICMP_STATS_t *)data;
    icmpstats = (NPF_ICMP_STATS_t *)cmcorrelator->data;
    icmpstats->icmpInMsgs        += icmpportstats->icmpInMsgs;
    icmpstats->icmpInErrors      += icmpportstats->icmpInErrors;
    icmpstats->icmpInDestUnreach += icmpportstats->icmpInDestUnreach;
    icmpstats->icmpInTimeExceeds += icmpportstats->icmpInTimeExceeds;
    icmpstats->icmpInParamProbs  += icmpportstats->icmpInParamProbs;
    icmpstats->icmpInSrcQuenches += icmpportstats->icmpInSrcQuenches;
    icmpstats->icmpInRedirects   += icmpportstats->icmpInRedirects;
    icmpstats->icmpInEchos       += icmpportstats->icmpInEchos;
    icmpstats->icmpInEchoReps    += icmpportstats->icmpInEchoReps;
    icmpstats->icmpInTimeStamps  += icmpportstats->icmpInTimeStamps;
    icmpstats->icmpInTimeStampsReps  += icmpportstats->icmpInTimeStampsReps;
    icmpstats->icmpInAddrMasks       += icmpportstats->icmpInAddrMasks;
    icmpstats->icmpInAddrMaskReps    += icmpportstats->icmpInAddrMaskReps;
    icmpstats->icmpOutMsgs           += icmpportstats->icmpOutMsgs;
    icmpstats->icmpOutErrors         += icmpportstats->icmpOutErrors;
    icmpstats->icmpOutTimeExceeds    += icmpportstats->icmpOutTimeExceeds;
    icmpstats->icmpOutParamProbs     += icmpportstats->icmpOutParamProbs;
    icmpstats->icmpOutSrcQuenches    += icmpportstats->icmpOutSrcQuenches;
    icmpstats->icmpOutRedirects      += icmpportstats->icmpOutRedirects;
    icmpstats->icmpOutEchos          += icmpportstats->icmpOutEchos;
    icmpstats->icmpOutEchoReps       += icmpportstats->icmpOutEchoReps;
    icmpstats->icmpOutTimeStamps     += icmpportstats->icmpOutTimeStamps;
    icmpstats->icmpOutTimeStampsReps += icmpportstats->icmpOutTimeStampsReps;
    icmpstats->icmpOutAddrMasks      += icmpportstats->icmpOutAddrMasks;
    icmpstats->icmpOutAddrMaskReps   += icmpportstats->icmpOutAddrMaskReps;

    if (isStillBindWait(cmcorrelator)) goto finish;

    /* Remove the correlator from g_outstandingRequest but don't free it */
    if (cm_removeFromRequest(cmcorrelator) == -1)
        assert(0);

    /* Add: check if all FEs are DONE or some FAIL? Trouble if any FE FAIL
     *      rollback the successful FEs and callback fail if not NPF_NOACK
     *      must free cmcorrelator->data & cmcorrelator before return */

    /* Q: is it right position to free cmcorrelator here? */
    /* Note: Free both cmcorrelator and cmcorrelator->data */
    PIL_FreeMemory(cmcorrelator->data);
    PIL_FreeMemory(cmcorrelator);
     
finish:
    CM_UNLOCK();
    return;
}

/* Response Data: real MTU set, -1 if failure */
void onCB_port_set_mtu(	uint32_t	feid,
			CMCORRELATOR	cmcorrelator,
			CMCONTEXT	cmcontext,
			NPF_DATA	data	)
{
    int             i;
    FPPI_Status     *rsp;
    NPF_DATA        tmp_data;

    assert(cmcontext);

    /* cmcorrelator maybe freed before if any related FE communication
     * failed at the request time. Just discard this callback */
    if (!cmcorrelator) return;

    /* Validate if this callback indeed belongs to this type */
    if ((CMCLIENTCONTEXT)cmcontext != CM_CC_PORT_SETMTU)
	return;

    CM_LOCK();

    /* data is the FPPI_Status */
    rsp = (FPPI_Status *)data;

    /* Note: Return if still other FPPAPI callbacks is waiting */
    for (i=0; i<cmcorrelator->numSentFE; i++)
        if (cmcorrelator->waitFEid[i] == feid) {
            if (*rsp == FPPI_SUCCESSFUL)
                cmcorrelator->waitFEid[i] = REQUEST_FE_DONE;
            else
                cmcorrelator->waitFEid[i] = REQUEST_FE_FAIL;
        }
    if (isStillBindWait(cmcorrelator)) goto finish;

    /* Remove the correlator from g_outstandingRequest but don't free it */
    if (cm_removeFromRequest(cmcorrelator) == -1)
        assert(0);

    /* Add: Check all the rsp in the cmcorrelator->waitFEid for all success */

    if (*rsp == FPPI_SUCCESSFUL) {
        /* assign system data */
        if (cm_getDataByHandle(cmcorrelator->handle, &tmp_data) == -1)
            goto finish;
        /* FIX: The real MTU set should check the response data */
        /* make a copy here since we've allocate at the new data time */
        memcpy(&((PDK_PORT_t *)tmp_data)->attribs.MTU,
               cmcorrelator->data, sizeof(int));
        /* fire internal NPF_EVENT_PORT_MTU_CHANGE event */
        onCB_event_PORT(cmcorrelator->handle, INTERNAL_CB, 
                                    NPF_EVENT_PORT_MTU_CHANGE);
    }


    /* fire external NPF_EVENT_PORT_MTUCHANGE event */
    if (*rsp == FPPI_SUCCESSFUL)
        onCB_event_PORT(cmcorrelator->handle, EXTERNAL_CB,
                                        NPF_EVENT_PORT_MTU_CHANGE);

    /* Q: is it right position to free cmcorrelator here? */
    /* Note: Free both cmcorrelator and cmcorrelator->data */
    PIL_FreeMemory(cmcorrelator->data);
    PIL_FreeMemory(cmcorrelator);
     
finish:
    CM_UNLOCK();
    return;
}

/* Response Data: real linkspeed set, -1 if failure */
void onCB_port_set_linkspeed(	uint32_t	feid,
				CMCORRELATOR	cmcorrelator,
				CMCONTEXT	cmcontext,
				NPF_DATA	data	)
{
    int             i;
    NPF_DATA        tmp_data;
    FPPI_Status     *rsp;

    assert(cmcontext);

    /* cmcorrelator maybe freed before if any related FE communication
     * failed at the request time. Just discard this callback */
    if (!cmcorrelator) return;

    /* Validate if this callback indeed belongs to this type */
    if ((CMCLIENTCONTEXT)cmcontext != CM_CC_PORT_SETLINKSPEED)
	return;

    CM_LOCK();

    /* data is the FPPI_Status */
    rsp = (FPPI_Status *)data;

    /* Note: Return if still other FPPAPI callbacks is waiting */
    for (i=0; i<cmcorrelator->numSentFE; i++)
        if (cmcorrelator->waitFEid[i] == feid) {
            if (*rsp == FPPI_SUCCESSFUL)
                cmcorrelator->waitFEid[i] = REQUEST_FE_DONE;
            else
                cmcorrelator->waitFEid[i] = REQUEST_FE_FAIL;
        }
    if (isStillBindWait(cmcorrelator)) goto finish;

    /* Remove the correlator from g_outstandingRequest but don't free it */
    if (cm_removeFromRequest(cmcorrelator) == -1)
        assert(0);

    /* Add: check if all FEs are DONE or some FAIL? Trouble if any FE FAIL
     *      rollback the successful FEs and callback fail if not NPF_NOACK
     *      must free cmcorrelator->data & cmcorrelator before return */

    if (*rsp == FPPI_SUCCESSFUL) {
        /* assign system data */
        if (cm_getDataByHandle(cmcorrelator->handle, &tmp_data) == -1)
            goto finish;
        /* FIX: The real speed set should check the response data */
        /* make a copy here since we've allocate at the new data time */
        memcpy(&((PDK_PORT_t *)tmp_data)->attribs.curTxRate,
               cmcorrelator->data, sizeof(int));
        /* fire internal NPF_EVENT_PORT_LINSKSPEED_CHANGE event */
        onCB_event_PORT(cmcorrelator->handle, INTERNAL_CB,
                                       NPF_EVENT_PORT_LINKSPEED_CHANGE);
    }


    /* fire external NPF_EVENT_PORT_LINKSPEED event */
    if (*rsp == FPPI_SUCCESSFUL)
        onCB_event_PORT(cmcorrelator->handle, EXTERNAL_CB,
                                       NPF_EVENT_PORT_LINKSPEED_CHANGE);

    /* Q: is it right position to free cmcorrelator here? */
    /* Note: Free both cmcorrelator and cmcorrelator->data */
    PIL_FreeMemory(cmcorrelator->data);
    PIL_FreeMemory(cmcorrelator);

finish:
    CM_UNLOCK();
    return;
}

/* Response Data: NPF_PORT_STATS */
void onCB_port_get_statistics(	uint32_t	feid,
				CMCORRELATOR	cmcorrelator,
				CMCONTEXT	cmcontext,
				NPF_DATA	data	)
{
    int             i;

    assert(cmcontext);

    /* cmcorrelator maybe freed before if any related FE communication
     * failed at the request time. Just discard this callback */
    if (!cmcorrelator) return;

    /* Validate if this callback indeed belongs to this type */
    if ((CMCLIENTCONTEXT)cmcontext != CM_CC_PORT_GETSTATS)
	return;

    CM_LOCK();

    /* Add: check RSP for failure response, if ANY PORT FAIL set 
     *      cmcorrelator->waitFEid[?] to REQUEST_FE_FAIL */

    /* Note: Return if still other FPPAPI callbacks is waiting */
    for (i=0; i<cmcorrelator->numSentFE; i++)
        if (cmcorrelator->waitFEid[i] == feid)
            cmcorrelator->waitFEid[i] = REQUEST_FE_DONE;
    if (isStillBindWait(cmcorrelator)) goto finish;

    /* Remove the correlator from g_outstandingRequest but don't free it */
    if (cm_removeFromRequest(cmcorrelator) == -1)
        assert(0);

    /* Add: check if all FEs are DONE or some FAIL? Trouble if any FE FAIL
     *      rollback the successful FEs and callback fail if not NPF_NOACK
     *      must free cmcorrelator->data & cmcorrelator before return */

    /* Q: is it right position to free cmcorrelator here? */
    /* Note: Free both cmcorrelator and cmcorrelator->data */
    PIL_FreeMemory(cmcorrelator->data);
    PIL_FreeMemory(cmcorrelator);
     
finish:
    CM_UNLOCK();
    return;
}

/*
 * Port off:
 * Check all the other ports associates to all the interface which
 * this port belongs to. If they are all admin down, then fire internal intf
 * off event, and check the correlator to see if the interface is the
 * one that completion call operate on. If so then callback the completion
 * call. At the last trigger the external interface off event.
 * Note: A interfaces could be turn off because of all it's associated ports
 *       are associated to other interafces and they all admin turn off.
 *
 * Port On:
 * Check all the other ports associates to all the interfaces which
 * this port belons to. If they are all admin on, then fire internal
 * interface on event, and check the correlator to see if the interface is
 * the one that completion call operate on. If so then callback the completion
 * call. At the last trigger the external interface on event.
 * Note: A interface coule be turn on because of all it's associated ports
 *       are associated to other interfaces and they all admin turn on
 *
 * Q: Then Apps who makes the if_set_portstatus call will receive two response
 *    One is completion callback and the second is interface off event.
 *    Is this fine? Can avoid the second one by checking the correlator to
 *    determinate if the interface is the original request one.
 *
 * FIX: if any of ports callback failure, needs to do a lot to recover
 *
 */
/* This is the completion callback, should wait and receive
 * all the ports on diff. FEs are set properly then call
 * cm_if_setflag to set all the related interface/port flags
 *
 * Response Data: NULL */
void onCB_if_set_portstatus(	FPPI_FEID	feid,
				CMCORRELATOR	cmcorrelator,
				CMCONTEXT	cmcontext,
				NPF_DATA	data	)
{

    int             i, num_setifs;
    NPF_HANDLE      setifhlist[MAXIFOWNPORT];
    NPF_IF_STATUS   status;

    assert(cmcontext);

    /* cmcorrelator maybe freed before if any related FE communication
     * failed at the request time. Just discard this callback */
    if (!cmcorrelator) return;

    /* Validate if this callback indeed belongs to this type */
    if ((CMCLIENTCONTEXT)cmcontext != CM_CC_IF_SETPORTSTATUS)
	return;

    CM_LOCK();

    /* Add: check RSP for failure response, if ANY PORT FAIL set 
     *      cmcorrelator->waitFEid[?] to REQUEST_FE_FAIL */

    /* Note: Return if still other FPPAPI callbacks is waiting */
    for (i=0; i<cmcorrelator->numSentFE; i++)
        if (cmcorrelator->waitFEid[i] == feid)
            cmcorrelator->waitFEid[i] = REQUEST_FE_DONE;
    if (isStillBindWait(cmcorrelator)) goto finish;

    /* Remove the correlator from g_outstandingRequest but don't free it */
    if (cm_removeFromRequest(cmcorrelator) == -1)
        assert(0);

    /* Add: check if all FEs are DONE or some FAIL? Trouble if any FE FAIL
     *      rollback the successful FEs and callback fail if not NPF_NOACK
     *      must free cmcorrelator->data & cmcorrelator before return */

    status = *((NPF_IF_STATUS *)(cmcorrelator->data));

    /* assign system data, set all the related interface & ports flags */
    cm_if_setflag(cmcorrelator->handle,
                  status,
                  &num_setifs,
                  setifhlist);

    /* fire internal interface event */
    for (i=0; i<num_setifs; i++)
        onCB_event_IF(setifhlist[i], INTERNAL_CB,
            (status==NPF_IF_UP)?NPF_EVENT_IF_UP:NPF_EVENT_IF_DOWN);


    /* fire the external interface event */
    for (i=0; i<num_setifs; i++)
        onCB_event_IF(setifhlist[i], EXTERNAL_CB,
            (status==NPF_IF_UP)?NPF_EVENT_IF_UP:NPF_EVENT_IF_DOWN);

    /* Q: is it right position to free cmcorrelator here? */
    /* Note: Free both cmcorrelator and cmcorrelator->data */
    PIL_FreeMemory(cmcorrelator->data);
    PIL_FreeMemory(cmcorrelator);

finish:
    CM_UNLOCK();
    return;
}

