/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/

/*****************************************************************************
* $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/CoreBlocks/ConfigManagment/src/internal.c,v $
* $Revision: 1.6 $
* $Date: 2003/11/17 16:57:12 $
*
* Description:
*    This file provides APIs that can be used by internal components to
*    retrieve the internal object information. They are all SYN calls.
*    Caller (internal components) should pre-allocate the memory for
*    Get case. Get does "COPY out" and Set does "COPY in".
*    
* TODO:
*    1. Check the internal ports whenever do the port search
*
*****************************************************************************/

#include <string.h>
#include <assert.h>
#include <pil.h>
#include "npf_ns_api.h"
#include "pdk_ns_helper.h"
#include "pdk_cm.h"
#include "cm.h"
#include "log.h"

/* Binding Discovery When new FE bound */
NPF_RET cm_sys_incr_numfes(void)
{
    NPF_NSHDL    nsh;
    NPF_HANDLE   handle;
    PDK_SYSTEM_t *sys;
    NPF_DATA     data;

    CM_LOCK();
    if (pdk_ns_getNSRootHandle(&nsh) != NPF_SUCCESS) {
        CM_UNLOCK();
        return NPF_INVALID_PARAMETERS;
    }
    if (npf_ns_getDataHandle(nsh, &handle) != NPF_SUCCESS)
        goto error;
    if (cm_getDataByHandle(handle, &data) == -1)
        goto error;
    sys = (PDK_SYSTEM_t *)data;
    if (NPF_SYSTEM != sys->type)
        goto error;
    sys->num_FEs++;
    npf_ns_close(nsh);
    CM_UNLOCK();
    return NPF_SUCCESS;

error:
    npf_ns_close(nsh);
    CM_UNLOCK();
    return NPF_INVALID_PARAMETERS;

}

/* user should memset the attribs */
NPF_RET cm_fe_set_attribs(NPF_HANDLE handle, PDK_FE_ATTRIBS *attribs)
{
    PDK_FE_t *fe;
    NPF_DATA data;

    assert(attribs);

    CM_LOCK();
    if (cm_getDataByHandle(handle, &data) == -1)
        goto error;
    fe = (PDK_FE_t *)data;
    if (NPF_FE != fe->type)
        goto error;
    memcpy(&fe->attribs, attribs, sizeof(PDK_FE_ATTRIBS));
    CM_UNLOCK();
    return NPF_SUCCESS;

error:
    CM_UNLOCK();
    return NPF_INVALID_PARAMETERS;
}

/* user should memset the caps */
NPF_RET cm_fe_set_caps(NPF_HANDLE handle, PDK_FE_CAPS *caps)
{
    PDK_FE_t *fe;
    NPF_DATA data;

    assert(caps);

    CM_LOCK();
    if (cm_getDataByHandle(handle, &data) == -1)
        goto error;
    fe = (PDK_FE_t *)data;
    if (fe->type != NPF_FE)
        goto error;
    memcpy(&fe->caps, caps, sizeof(PDK_FE_CAPS));
    CM_UNLOCK();
    return NPF_SUCCESS;

error:
    CM_UNLOCK();
    return NPF_INVALID_PARAMETERS;
}

/* Return NPF_FE_ATTRIBS, same as npf_fe_get_attribs */
NPF_RET cm_fe_get_attribs(NPF_HANDLE feh, NPF_FE_ATTRIBS *copyattribs)
{
    NPF_FE_t copyfe;

    assert(copyattribs);

    CM_LOCK();
    if (cm_copyFromPDK(feh, &copyfe) == -1) {
        CM_UNLOCK();
        return NPF_INVALID_PARAMETERS;
    }
    CM_UNLOCK();

    memcpy(copyattribs, &copyfe.attribs, sizeof(NPF_FE_ATTRIBS));
    return NPF_SUCCESS;
}

/* user should memset the attribs */
NPF_RET cm_port_set_attribs(NPF_HANDLE handle, PDK_PORT_ATTRIBS *attribs)
{
    PDK_PORT_t *port;
    NPF_DATA data;

    assert(attribs);

    CM_LOCK();
    if (cm_getDataByHandle(handle, &data) == -1)
        goto error;
    port = (PDK_PORT_t *)data;
    if (port->type != NPF_PORT)
        goto error;
    memcpy(&port->attribs, attribs, sizeof(PDK_PORT_ATTRIBS));
    CM_UNLOCK();
    return NPF_SUCCESS;

error:
    CM_UNLOCK();
    return NPF_INVALID_PARAMETERS;
}

/* user should memset the caps */
NPF_RET cm_port_set_caps(NPF_HANDLE handle, PDK_PORT_CAPS *caps)
{
    PDK_PORT_t *port;
    NPF_DATA data;

    assert(caps);

    CM_LOCK();
    if (cm_getDataByHandle(handle, &data) == -1)
        goto error;
    port = (PDK_PORT_t *)data;
    if (port->type != NPF_PORT)
        goto error;
    memcpy(&port->caps, caps, sizeof(PDK_PORT_CAPS));
    CM_UNLOCK();
    return NPF_SUCCESS;

error:
    CM_UNLOCK();
    return NPF_INVALID_PARAMETERS;
}

NPF_RET cm_port_get_portid(NPF_HANDLE porth, uint32_t *portid)
{
    PDK_PORT_t *port;

    assert(portid);
 
    CM_LOCK();
    if (cm_getDataByHandle(porth, (NPF_DATA *)&port) == -1)
        goto error;
    if (port->type != NPF_PORT)
        goto error;
    *portid = port->attribs.id;
    CM_UNLOCK();
    return NPF_SUCCESS; 

error:
    CM_UNLOCK();
    return NPF_INVALID_PARAMETERS;
}

/* a port can associate with multiple interfaces and may in diff. Routers
 * user should allocate the ifhlist memory first.
 * porth is open and ifhlist[] return open status */
NPF_RET cm_getIFByPort(NPF_HANDLE porth, int *numifs, NPF_HANDLE *ifhlist)
{
    int           i = 0;
    NPF_NSHDL     nsifh, nsporth;
    NODE_ITERATOR itrif;

    assert(numifs);
    assert(ifhlist);

    *numifs = 0;

    CM_LOCK();
    /* get NSHDL of port */
    if (cm_getNSHDLByHandle(porth, &nsporth) != NPF_SUCCESS) {
        CM_UNLOCK();
        return NPF_INVALID_PARAMETERS;
    }
    /* read association */
    if (npf_ns_enumAssociate(nsporth, &itrif) != NPF_SUCCESS) {
        CM_UNLOCK();
        return NPF_INVALID_PARAMETERS;
    }

    /* no association */
    if ((*numifs = npf_ns_count(itrif)) == 0) {
        CM_UNLOCK();
        return NPF_SUCCESS;
    }

    if (*numifs > MAXIFOWNPORT) {
        /* should be MAXPORTOWNIF here but who cares */
        npf_logger_write(LOG_CM, LOG_ERR,
                   "Too many ifs in one port. Limit %d.", MAXIFOWNPORT);
        goto error;
    }

    if ((nsifh = npf_ns_first(itrif)) == 0)
        goto error;

    do {
        npf_ns_getDataHandle(nsifh, &ifhlist[i++]);
    } while ((nsifh = npf_ns_next(itrif)) != 0);
    npf_ns_done(itrif);

    CM_UNLOCK();
    return NPF_SUCCESS;

error:
    npf_ns_done(itrif);
    CM_UNLOCK();
    return NPF_INVALID_PARAMETERS;

}

/* get the arry of port handle assoc. with if, could be in diff. FEs
 * ifh is open and output porth[] return open status */
NPF_RET cm_getPortByIF(	NPF_HANDLE	ifh,
			int		*numports,
			NPF_HANDLE	*porth)
{
    int           i = 0;
    NPF_NSHDL     nsifh, nsporth;
    NODE_ITERATOR itrport;

    assert(numports);
    assert(porth);

    *numports = 0;

    CM_LOCK();
    /* get NSHDL of if */
    if (cm_getNSHDLByHandle(ifh, &nsifh) != NPF_SUCCESS) {
        CM_UNLOCK();
        return NPF_INVALID_PARAMETERS;
    }
    /* read association */
    if (npf_ns_enumAssociate(nsifh, &itrport) != NPF_SUCCESS) {
        CM_UNLOCK();
        return NPF_INVALID_PARAMETERS;
    }

    /* no association */
    if ((*numports = npf_ns_count(itrport)) == 0) {
        CM_UNLOCK();
        return NPF_SUCCESS;
    }

    if (*numports > MAXIFOWNPORT) {
        npf_logger_write(LOG_CM, LOG_ERR,
                   "Too many ports in one if. Limit %d.", MAXIFOWNPORT);
        goto error;
    }

    if ((nsporth = npf_ns_first(itrport)) == 0)
        goto error;

    do {
        npf_ns_getDataHandle(nsporth, &porth[i++]);
    } while ((nsporth = npf_ns_next(itrport)) != 0);
    npf_ns_done(itrport);

    CM_UNLOCK();
    return NPF_SUCCESS;

error:
    npf_ns_done(itrport);
    CM_UNLOCK();
    return NPF_INVALID_PARAMETERS;
}

/* get the array of port handle, FAIL if ports belong to DIFFERENT FE 
 * feh and porth[] are all return CLOSE status
 * if no port associated, return INVALID_PARAMETER */
NPF_RET cm_getSingleFEPortByIF(	NPF_HANDLE	ifh,
				NPF_HANDLE	*feh,
				int		*numports,
				NPF_HANDLE	*porth)
{
    int           i = 0;
    NPF_NSHDL     nsifh, nsporth;
    NPF_HANDLE    nsfeh1, nsfeh2;
    NODE_ITERATOR itrport;

    assert(feh);
    assert(numports);
    assert(porth);

    *numports = 0;
    *feh = -1;

    CM_LOCK();
    /* get NSHDL of if */
    if (cm_getNSHDLByHandle(ifh, &nsifh) != NPF_SUCCESS) {
        CM_UNLOCK();
        return NPF_INVALID_PARAMETERS;
    }
    /* read association */
    if (npf_ns_enumAssociate(nsifh, &itrport) != NPF_SUCCESS) {
        CM_UNLOCK();
        return NPF_INVALID_PARAMETERS;
    }

    if ((*numports = npf_ns_count(itrport)) == 0) {
        CM_UNLOCK();
        return NPF_INVALID_PARAMETERS;
    }

    if (*numports > MAXIFOWNPORT) {
        npf_logger_write(LOG_CM, LOG_ERR,
                   "Too many ports in one if. Limit %d.", MAXIFOWNPORT);
        goto error;
    }

    if ((nsporth = npf_ns_first(itrport)) == 0)
        goto error;

    /* get the ns fehandle of the first port */
    if (npf_ns_getInstParent(nsporth, (NPF_NSHDL *)&nsfeh1) != NPF_SUCCESS) {
        npf_ns_close(nsporth);
        goto error;
    }

    if (npf_ns_getDataHandle(nsfeh1, feh) != NPF_SUCCESS) {
        npf_ns_close(nsporth);
        npf_ns_close(nsfeh1);
        goto error;
    }
    
    do {
        npf_ns_getInstParent(nsporth, (NPF_NSHDL *)&nsfeh2);
        if (nsfeh1 != nsfeh2) {
            npf_logger_write(LOG_CM, LOG_ERR,
                "Ports in one interface belongs to different FEs.");
            *feh = -1;
            npf_ns_close(nsporth);
            npf_ns_close(nsfeh1);
            npf_ns_close(nsfeh2);
            goto error;
        }
        npf_ns_close(nsfeh2);
        npf_ns_getDataHandle(nsporth, &porth[i++]);
    } while ((npf_ns_close(nsporth) != -1) &&
                 (nsporth = npf_ns_next(itrport)) != 0);
    npf_ns_done(itrport);
    npf_ns_close(nsfeh1);

    CM_UNLOCK();
    return NPF_SUCCESS;

error:
    npf_ns_done(itrport);
    CM_UNLOCK();
    return NPF_INVALID_PARAMETERS;
}

/* The following vidd APIs, vidd gets the NPF_INTERFACE_t information by
 * passing it's own memory, C&M copy internal system data to vidd.
 * vidd will set interface->vifname by passing char* and C&M need to do
 * strcpy */

/* Return NPF_INTERFACE_t, same as npf_if_get_attribs */
NPF_RET cm_if_get_attribs(NPF_HANDLE ifh, NPF_INTERFACE_t *copyif)
{
    assert(copyif);

    CM_LOCK();
    if (cm_copyFromPDK(ifh, copyif) == -1) {
        CM_UNLOCK();
        return NPF_INVALID_PARAMETERS;
    }
    CM_UNLOCK();
    return NPF_SUCCESS;
}

NPF_RET cm_if_set_vifname(NPF_HANDLE ifh, char *vifname)
{
    PDK_INTERFACE_t *interface; 
    NPF_DATA        data;

    assert(vifname);

    CM_LOCK();
    if (cm_getDataByHandle(ifh, &data) == -1)
        goto error;
    interface = (PDK_INTERFACE_t *)data;
    if (interface->type != NPF_INTERFACE)
        goto error;
    strcpy(interface->vifname, vifname);
    CM_UNLOCK();
    return NPF_SUCCESS;

error:
    CM_UNLOCK();
    return NPF_INVALID_PARAMETERS;
}

NPF_RET cm_fe_get_feid(NPF_HANDLE feh, uint32_t* feid)
{
    PDK_FE_t *fe; 
    NPF_DATA data;

    assert(feid);

    if (cm_getDataByHandle(feh, &data) == -1)
        goto error;
    fe = (PDK_FE_t *)data;
    if (fe->type != NPF_FE)
        goto error;
    *feid = fe->attribs.id;
    CM_UNLOCK();
    return NPF_SUCCESS;   

error:
    CM_UNLOCK();
    return NPF_INVALID_PARAMETERS;
}

NPF_RET cm_if_set_index(NPF_HANDLE ifh, int index)
{
    PDK_INTERFACE_t *interface; 
    NPF_DATA        data;

    CM_LOCK();
    if (cm_getDataByHandle(ifh, &data) == -1)
        goto error;
    interface = (PDK_INTERFACE_t *)data;
    if (interface->type != NPF_INTERFACE)
        goto error;
    interface->index = index;
    CM_UNLOCK();
    return NPF_SUCCESS;

error:
    CM_UNLOCK();
    return NPF_INVALID_PARAMETERS;
}

/* For DiffServ APIs, DiffServ gets the PDK_FE_CAPS_t information by
 * passing it's own memory, C&M copy internal system data to DiffServ */
NPF_RET cm_fe_get_caps(NPF_HANDLE feh, PDK_FE_CAPS *copycaps)
{
    PDK_FE_t *fe; 
    NPF_DATA data;

    assert(copycaps);

    if (cm_getDataByHandle(feh, &data) == -1)
        goto error;
    fe = (PDK_FE_t *)data;
    if (fe->type != NPF_FE)
        goto error;
    memcpy(copycaps, &fe->caps, sizeof(PDK_FE_CAPS));
    CM_UNLOCK();
    return NPF_SUCCESS;   

error:
    CM_UNLOCK();
    return NPF_INVALID_PARAMETERS;
}

/* Get current active FE numbers, caller passin the numfe memory */
NPF_RET cm_sys_get_numfes(int *numfes)
{
    NPF_NSHDL    nsh;
    NPF_HANDLE   handle;
    PDK_SYSTEM_t *sys;
    NPF_DATA     data;

    if (numfes == NULL) return NPF_INVALID_PARAMETERS;

    *numfes = 0;

    CM_LOCK();
    if (pdk_ns_getNSRootHandle(&nsh) != NPF_SUCCESS) {
        CM_UNLOCK();
        return NPF_INVALID_PARAMETERS;
    }
    if (npf_ns_getDataHandle(nsh, &handle) != NPF_SUCCESS)
        goto error;
    if (cm_getDataByHandle(handle, &data) == -1)
        goto error;
    sys = (PDK_SYSTEM_t *)data;
    if (NPF_SYSTEM != sys->type)
        goto error;
    *numfes = sys->num_FEs;
    npf_ns_close(nsh);
    CM_UNLOCK();
    return NPF_SUCCESS;

error:
    npf_ns_close(nsh);
    CM_UNLOCK();
    return NPF_INVALID_PARAMETERS;

}

/* For Topology Manager */
Bool cm_port_isInternal(NPF_HANDLE porth)
{
    /* check the flag first, then verify the association */
    NPF_NSHDL     nsporth;
    NPF_HANDLE    ifh, intConfigh;
    NPF_DATATYPE  datatype;
    PDK_PORT_t    *port; 
    NODE_ITERATOR ifItr, portItr;

    CM_LOCK();
    if (cm_getDataByHandle(porth, (NPF_DATA *)&port) == -1)
        goto error;
    if (port->type != NPF_PORT)
        goto error;
    if (port->attribs.internal == 0)
        goto error;
    if (cm_getNSHDLByHandle(porth, &nsporth) != NPF_SUCCESS)
        goto error;
    if (npf_ns_enumAssociate(nsporth, &ifItr) != NPF_SUCCESS)
        goto error;
    if (npf_ns_count(ifItr) != 1) {
        npf_ns_done(ifItr);
        goto error;
    }
    if ((ifh = npf_ns_first(ifItr)) == 0) {
        npf_ns_done(ifItr);
        goto error;
    }
    npf_ns_done(ifItr);

    /* internal IF's parent is INTCONF */
    if (npf_ns_getInstParent(ifh, (NPF_NSHDL *)&intConfigh) != NPF_SUCCESS)
        goto error;

    if (npf_ns_getDataType(intConfigh, &datatype) != NPF_SUCCESS) {
        npf_ns_close(intConfigh);
        goto error;
    }
    npf_ns_close(intConfigh);

    if (datatype != NPF_INTCONFIG)
        goto error;

    /* now this is Internal IF, check if only has one associate */
    if (npf_ns_enumAssociate(ifh, &portItr) != NPF_SUCCESS)
        goto error;

    if (npf_ns_count(portItr) != 1) {
        npf_ns_done(portItr);
        goto error;
    }
    npf_ns_done(portItr);

    CM_UNLOCK();
    return TRUE;

error:
    CM_UNLOCK();
    return FALSE;
}

/* For MPLS & IPv4 */
/* Return NPF_IPROUTER_t, same as npf_ipr_get_attribs */
NPF_RET cm_ipr_get_attribs(NPF_HANDLE iprh, NPF_IPROUTER_t *copyipr)
{
    NPF_DATA data;

    assert(copyipr);

    CM_LOCK();
    if (cm_getDataByHandle(iprh, &data) == -1 ||
        ((PDK_SHIM_t *)data)->type != NPF_IPROUTER)
        goto error;
    if (cm_copyFromPDK(iprh, copyipr) == -1)
        goto error;

    CM_UNLOCK();
    return NPF_SUCCESS;

error:
    CM_UNLOCK();
    return NPF_INVALID_PARAMETERS;
}

NPF_RET cm_port_get_attribs(NPF_HANDLE porth, NPF_PORT_ATTRIBS *copyattribs)
{
    NPF_PORT_t copyport;

    assert(copyattribs);

    CM_LOCK();
    if (cm_copyFromPDK(porth, &copyport) == -1) {
        CM_UNLOCK();
        return NPF_INVALID_PARAMETERS;
    }
    CM_UNLOCK();

    memcpy(copyattribs, &copyport.attribs, sizeof(NPF_PORT_ATTRIBS));
    return NPF_SUCCESS;
}

/* Get inter. if by inter. port, 1:1 mapping and parent is INTCONFIG */
NPF_RET cm_getIntIFByPort(	NPF_HANDLE	porth,
				NPF_HANDLE	*ifh)
{
    NPF_NSHDL     nsifh, nsporth;
    NPF_HANDLE    nsparenth;
    NPF_DATATYPE  parenttype;
    NODE_ITERATOR itrif;

    assert(ifh);

    CM_LOCK();
    /* get NSHDL of port */
    if (cm_getNSHDLByHandle(porth, &nsporth) != NPF_SUCCESS) {
        CM_UNLOCK();
        return NPF_INVALID_PARAMETERS;
    }
    /* read association */
    if (npf_ns_enumAssociate(nsporth, &itrif) != NPF_SUCCESS) {
        CM_UNLOCK();
        return NPF_INVALID_PARAMETERS;
    }

    if (npf_ns_count(itrif) != 1) {
        /* should only have one association to internal interface */
        npf_logger_write(LOG_CM, LOG_ERR,
                   "Internal port should only assiciate to one interface");
        goto error;
    }

    if ((nsifh = npf_ns_first(itrif)) == 0)
        goto error;

    /* Add: check if this interface only have one associate to port */

    /* get the ns parent handle of the first if */
    if (npf_ns_getInstParent(nsifh, (NPF_NSHDL *)&nsparenth) != NPF_SUCCESS)
        goto error;

    /* the parent's datatype must be internal config */
    if (npf_ns_getDataType(nsparenth, &parenttype) != NPF_SUCCESS ||
        parenttype != NPF_INTCONFIG)
        goto error;

    npf_ns_getDataHandle(nsifh, ifh);

    if (npf_ns_next(itrif) != 0)
        goto error;

    npf_ns_done(itrif);

    CM_UNLOCK();
    
    return NPF_SUCCESS;

error:
    npf_ns_done(itrif);
    CM_UNLOCK();
    return NPF_INVALID_PARAMETERS;
}

/* close the namespace handle of the pass-in data handle */
NPF_RET cm_handle_close(NPF_HANDLE handle)
{
    NPF_NSHDL nsh;

    CM_LOCK();
    if (cm_getNSHDLByHandle(handle, &nsh) != NPF_SUCCESS)
        goto error;

    npf_ns_close(nsh);

    CM_UNLOCK();
    return NPF_SUCCESS;

error:
    CM_UNLOCK();
    return NPF_INVALID_PARAMETERS;
}

/* Return FALSE if multiple FE exists
 * Return CLOSE FE handle  */
NPF_RET cm_getSingleFEByRouter(NPF_HANDLE routerh, NPF_HANDLE *feh)
{
    int           numfes = 0, numports = 0;
    char          path[NPF_NS_MAXPATH];
    NPF_NSHDL     nsrouterh, nsifh;
    NPF_HANDLE    ifh, porthlist[MAXIFOWNPORT];
    NPF_DATATYPE  iftype;
    NODE_ITERATOR node_itr;

    if (!feh) goto error;

    CM_LOCK();
    if (cm_sys_get_numfes(&numfes) != NPF_SUCCESS) goto release;
    if (numfes != 1) goto release;
    
    /* Traverse NS to get IF, and call cm_getSingleFEPortByIF, only care FE */
    CM_LOCK();

    /* get NSHDL of router */
    if (cm_getNSHDLByHandle(routerh, &nsrouterh) != NPF_SUCCESS)
        goto release;

    /* get path if nsrouterh */
    if (npf_ns_getPath(nsrouterh, path) != NPF_SUCCESS)
        goto release;

    /* read router's directory */
    if (npf_ns_readDir(path, &node_itr) != NPF_SUCCESS)
        goto release;

    /* loop on the directory until found interface type node */
    if ((nsifh = npf_ns_first(node_itr)) == 0) {
        npf_ns_done(node_itr);
        goto release;
    }
    do {
        if (npf_ns_getDataType(nsifh, &iftype) != NPF_SUCCESS) {
            npf_ns_close(nsifh);
            npf_ns_done(node_itr);
            goto release;
        }
        if (iftype == NPF_INTERFACE) {
            npf_ns_close(nsifh);
            break;
        }
        npf_ns_close(nsifh);
    } while ((nsifh = npf_ns_next(node_itr)) != (NPF_NSHDL)NULL);
    npf_ns_done(node_itr);

    /* read interface type node's directory */
    if (npf_ns_getPath(nsifh, path) != NPF_SUCCESS)
        goto release;
    if (npf_ns_readDir(path, &node_itr) != NPF_SUCCESS)
        goto release;

    /* get first interface */
    if ((nsifh = npf_ns_first(node_itr)) == 0) {
        npf_ns_done(node_itr);
        goto release;
    }
    if (npf_ns_getDataHandle(nsifh, &ifh) != NPF_SUCCESS) {
        npf_ns_close(nsifh);
        npf_ns_done(node_itr);
        goto release;
    }
    npf_ns_close(nsifh);
    npf_ns_done(node_itr);

    /* get the FE and Portlist in CLOSE mode */
    if (cm_getSingleFEPortByIF(ifh, feh, &numports, porthlist) != NPF_SUCCESS)
        goto release;

    CM_UNLOCK();
    return NPF_SUCCESS;

release:
    CM_UNLOCK();
error:
    return NPF_INVALID_PARAMETERS;

}

NPF_RET cm_router_set_attribs(	NPF_HANDLE hdl,
                                NPF_ROUTETABLE_t *rtTbl)
{
    PDK_ROUTETABLE_t *rtInfo; 
    NPF_DATA        data;

    assert(rtTbl);

    CM_LOCK();
    if (cm_getDataByHandle(hdl, &data) == -1)
        goto error;
    if(rtInfo->type == NPF_ROUTETABLE)
        rtInfo->rt_type =  rtTbl->rt_type;
    else goto error;

    rtInfo = (PDK_ROUTETABLE_t *)data;
    
    CM_UNLOCK();
    return NPF_SUCCESS;

error:
    CM_UNLOCK();
    return NPF_INVALID_PARAMETERS;
}

NPF_RET cm_router_get_attribs(	NPF_HANDLE hdl,
                                NPF_ROUTETABLE_t *rtTbl)
{
    NPF_DATA data;

    assert(hdl);

    CM_LOCK();
    if (cm_getDataByHandle(hdl, &data) == -1 )
        goto error;
    rtTbl->rt_type = ((NPF_ROUTETABLE_t*)data)->rt_type;
    
    CM_UNLOCK();
    return NPF_SUCCESS;

error:
    CM_UNLOCK();
    return NPF_INVALID_PARAMETERS;
}
