//////////////////////////////////////////////////////////////////////////////
//           INTEL CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Intel Corporation and may not be copied
//  or disclosed except in accordance with the terms of that agreement.
//      Copyright (c) 1999-2000 Intel Corporation.  All Rights Reserved.
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////

/****************************************************************************
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/CoreBlocks/VIDDController/inc/vxworks_vidd.h,v $
 * $Revision: 1.6 $
 * $Date: 2003/11/17 16:57:14 $
 *                                                                          *
 * vxworks_vidd.h                                                           *
 * ------                                                                   *
 * This contains the virtual interface parameters that need to be accessed  *
 * and/or passed in from user space. Additionally, this also defines the    *
 * ioctls that can be invoked from user space.                              *
 *                                                                          *
 ****************************************************************************/


#if !defined(__VXWORKS_VIDD_H__)
#define __VXWORKS_VIDD_H__

#include <stdio.h>

#include "vxWorks.h"
#include "assert.h"
#include "cacheLib.h"
#include "end.h"            /* Common END structures. */
#include "errno.h"
#include "etherLib.h"
#include "etherMultiLib.h"      /* multicast stuff. */
#include "m2Lib.h"
#include "memLib.h"
#include "muxLib.h"
#include "muxTkLib.h"
#include "net/if.h"
#include "net/if_subr.h"
#include "net/mbuf.h"
#include "net/protosw.h"
#include "net/route.h"
#include "netBufLib.h"
#include "netLib.h"
#include "netinet/if_ether.h"
#include "netinet/in.h"
#include "netinet/in_systm.h"
#include "netinet/in_var.h"
#include "netinet/ip.h"
#include "sys/ioctl.h"
#include "sys/socket.h"
#include "pdk_vic.h"
#include "pdk_ph.h"

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

/**
 * Pre-processor symbol and macro definitions.
 */

/* Number of buffers in VIDD buffer pool. */
#define VIDD_NUM_ELEMENTS 2048
#define VIDD_DEV_NAME     "vi"
#define VIDD_IP_ADDR_STR_LEN 16
#define MIN_BUFFER_SIZE 2048
#define IF_STATUS_UP	1
#define IF_STATUS_DOWN	2


typedef enum {  MEDIA_TYPE_FAST_ETHERNET,
		MEDIA_TYPE_1GB_ETHERNET,
		MEDIA_TYPE_ATM,
		MEDIA_TYPE_POS } media_type;

/**
 * Type definitions.
 */

/**
 * TYPENAME: pdk_physical_if_info
 *
 * DESCRIPTION: Structure to contain physical interface properties.
 *
 */
typedef struct pdk_s_physical_if_info pdk_physical_if_info;
struct pdk_s_physical_if_info
{
    uint32_t feId;
    uint32_t portId;
    uint8_t  name[IFNAME_LEN];
    uint32_t unit;
    uint8_t  macAddr[ETH_ALEN];
    uint32_t ipv4Addr;
    uint32_t ipv4Mask;
    uint8_t  if_status;
    media_type mediaType;
    uint16_t MTU;
    uint32_t linkSpeed;
};

   
/**
 * TYPENAME: pdk_vidd_physical_if_node
 * 
 * DESCRIPTION: This list of data structures are created upon
 * initialization by the VIDD and represent the hardware network
 * interfaces.
 *
 */
typedef struct pdk_s_vidd_ctrl pdk_vidd_ctrl;
typedef struct pdk_s_vidd_physical_if_node pdk_vidd_physical_if_node;
struct pdk_s_vidd_physical_if_node
{
    END_OBJ end;
    void* pMuxCookie;
    pdk_vidd_ctrl* pViddCtrl;
    pdk_physical_if_info* pPhysicalIfInfo;
    pdk_vidd_physical_if_node* pNextPhysicalIf;
};

/**
 * TYPENAME: pdk_vidd_fp_node
 * 
 * DESCRIPTION: This is the structure representing a Forwarding Plane Object.
 * It is constructed as a linked list for future expansion of FPs.
 * Created by VIDD during initialization.
 *
 */
typedef struct pdk_s_vidd_fp_node pdk_vidd_fp_node;
struct pdk_s_vidd_fp_node
{
    uint32_t feId;
    pdk_vidd_physical_if_node* pPhysicalIfs;
    uint32_t numPorts;
    pdk_vidd_fp_node* pNextFp;
};


/**
 * TYPENAME: pdk_vidd_ctrl
 * 
 * DESCRIPTION: This structure is defined by the driver and is created during
 * the initialization. It holds the control information about interfaces and
 * forwarding planes, and can be used as a context for all communication
 * between the CC Module and the VIDD.
 *
 */
struct pdk_s_vidd_ctrl
{
/*   pdk_vidd_fp_node* pFps;  */
    NET_POOL_ID pNetPool;
    void* pMclBlkCfg;
    void* pClustMem;
};


/**
 * Prototypes for interface functions.
 */

int pdk_vif_create(
                                PDK_VIC_t* arg_pVif,
                                char* hwaddr
                                );

int pdk_vif_delete(
                                PDK_VIC_t* arg_pVif
                                );

int pdk_vif_setflags(
                                PDK_VIC_t* arg_pVif,
                                uint32_t if_status
                                );

int pdk_vif_ipaddr_update(
                                PDK_VIC_t* arg_pVif,
                                uint32_t ipAddr,
                                uint32_t subMask
                                );

int pdk_vif_mtu_config(
                                PDK_VIC_t* arg_pVif,
                                uint16_t MTU
                                );

int pdk_vidd_receive_pkt(
                                       M_BLK_ID pMblk,
                                        uint32_t portId,
                                        uint32_t feId
                                       );

int pdk_vidd_getNetPool(
                                NET_POOL_ID* arg_pNetPool
                                );


/**
 * Prototypes for device driver functions.
 */

END_OBJ* pdk_vidd_npt_load(
                                    char* initString,
                                    void *pBsp
                                    );

STATUS pdk_vidd_npt_unload(
                                    END_OBJ* pEnd
                                    );

STATUS pdk_vidd_npt_send(
                                  END_OBJ* pEnd,
                                  M_BLK_ID pMblk,
                                  char* dstMacAddr,
                                  long netType,
                                  void* pSpare
                                  );

int pdk_vidd_npt_ioctl(
                                END_OBJ* pEnd,
                                int command,
                                caddr_t buffer
                                );

STATUS pdk_vidd_npt_start(
                                     END_OBJ* pEnd
                                     );

STATUS pdk_vidd_npt_stop(
                                    END_OBJ* pEnd
                                    );

STATUS pdk_vidd_npt_mCastAddrAdd(
                                          END_OBJ * pEnd,
                                          char* pAddress
                                          );

STATUS pdk_vidd_npt_mCastAddrDel(
                                          END_OBJ * pEnd,
                                          char* pAddress
                                          );

STATUS pdk_vidd_npt_mCastAddrGet(
                                          END_OBJ * pEnd,
                                          MULTI_TABLE* pTable
                                          );

STATUS pdk_vidd_npt_pollSend(
                                        END_OBJ* pEND,
                                      M_BLK_ID pPkt,
                                      char* dstAddr,
                                      long netType,
                                      void * pSpareData
                                      );

STATUS pdk_vidd_npt_pollRcv(
                                        END_OBJ* pEND,
                                     M_BLK_ID pMblk,
                                     long* pNetSvc,
                                     long* pNetOffset,
                                     void* pSpareData
                                     );

STATUS pdk_vidd_npt_formAddress(
                                         M_BLK_ID arg_pData,
                                         M_BLK_ID arg_pSrc,
                                         M_BLK_ID arg_pDst
                                         );


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__VXWORKS_VIDD_H__) */
