//////////////////////////////////////////////////////////////////////////////
//           INTEL CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Intel Corporation and may not be copied
//  or disclosed except in accordance with the terms of that agreement.
//      Copyright (c) 1999-2000 Intel Corporation.  All Rights Reserved.
//////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////

/****************************************************************************
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/CoreBlocks/VIDDController/src/pdk_vxworks_vidd.c,v $:
 * $Revision: 1.6 $:
 * $Date: 2003/11/17 16:57:14 $
 *                                                                          *
 * pdk_vxworks_vidd.c                                                       *
 * ------                                                                   *
 * This file contains the VIDD Enhanced Network Driver for VxWorks. This    *
 * driver is used by the pdk on the control plane side.                     *
 *                                                                          *
 ****************************************************************************/


/**
 * User defined include files required.
 */

#include <stdio.h>
#include <string.h>

#ifdef _VXWORKS_

#include "vxworks_vidd.h"

/**
 * Pre-processor symbols and macros used in this file.
 */

#define INCLUDE_NPT 1
#ifdef INCLUDE_NPT
/* NPT requires that we make this defined. */
#define END_MACROS
#include "endLib.h"

#endif /* INCLUDE_NPT */


/**
 * Variable declarations global to this file only.  Externs are followed by
 * static variables.
 */
/****************************************************************************
 * Driver functions:
 *    These routines are provided to the kernel during registration.
 *    These routines are the same across all instances of the driver.
 ****************************************************************************/
static NET_FUNCS pdk_vidd_func_table = {
    (FUNCPTR)pdk_vidd_npt_start,         /* Function to start the device */
    (FUNCPTR)pdk_vidd_npt_stop,          /* Function to stop the device */
    (FUNCPTR)pdk_vidd_npt_unload,        /* Unloading function for the driver */
    (FUNCPTR)pdk_vidd_npt_ioctl,         /* Ioctl function for the driver */
    (FUNCPTR)pdk_vidd_npt_send,          /* Send function for the driver */
    (FUNCPTR)pdk_vidd_npt_mCastAddrAdd,  /* Multicast address add */
    (FUNCPTR)pdk_vidd_npt_mCastAddrDel,  /* Multicast address delete */
    (FUNCPTR)pdk_vidd_npt_mCastAddrGet,  /* Multicast table retrieve */
    (FUNCPTR)pdk_vidd_npt_pollSend,      /* Polling send function for the driver */
    (FUNCPTR)pdk_vidd_npt_pollRcv,   /* Polling receive function for the driver */
    (FUNCPTR)pdk_vidd_npt_formAddress  /* Stub to form L2 address for the driver */

};

static pdk_vidd_ctrl* g_pViddCtrl = NULL;
static pdk_vidd_physical_if_node* g_pFirstPhysicalIf = NULL;
static uint32_t g_numVIFs = 0;


/**
 * Extern function prototypes.
 */
extern int ipAttach(int unit, char* pDevice);

/**
 * Static function prototypes.
 */
static int _pdk_vidd_init_mem(uint32_t bufferSize, pdk_vidd_ctrl* arg_pViddCtrl);
static int _vidd_get_physical_if(uint32_t arg_portId, uint32_t arg_feId,
		    pdk_vidd_physical_if_node** arg_pPhysicalIf, 
		    pdk_vidd_physical_if_node** arg_pPrevPhysicalIf);

/**
 * Functions used by VIDD Controller.
 */

/**
 * NAME: pdk_vif_create
 *
 * DESCRIPTION: This function is called by the CC Module to initialize the
 * local VIDD and register the VIDD's entry points with the CC Module.
 * 
 * @Param:  - INOUT pdk_fp_node* arg_pFP - pointer to the FP structure
 *          used to get data for all ports.
 * @Param:  - INOUT pdk_handler_module* arg_pHandlerModule -
 *          the function will fill in this structure with its entry points
 *          and contexts.
 *
 * @Return: 0, -1
 */
int pdk_vif_create(
                                PDK_VIC_t* arg_pVif,
                                char* hwaddr
                                )
{
    int err = 0;
    pdk_vidd_physical_if_node* pPhysicalIf = NULL;
    pdk_vidd_physical_if_node* pCurrPhysicalIf = NULL;
    void* pDevice = NULL;

    if (arg_pVif == NULL) 
    {
	return -1;
    }

    /* Allocate memory for VIDD control structure. */
    if (g_pViddCtrl == NULL) 
    {
      g_pViddCtrl = malloc(sizeof(pdk_vidd_ctrl));
      if (g_pViddCtrl == NULL)
      {
        return -1;
      }
      memset(g_pViddCtrl, 0, sizeof(pdk_vidd_ctrl));

      /* Allocate memory for and create the shared buffer pool for the driver. */
      err = _pdk_vidd_init_mem(MIN_BUFFER_SIZE, g_pViddCtrl);
      if (err == -1)
	goto label_1;
    }

    /* Allocate memory for physical if structure. */
    pPhysicalIf = malloc(sizeof(pdk_vidd_physical_if_node));
    if (pPhysicalIf == NULL)
    {
        err = -1;
        goto label_1;
    }
    memset(pPhysicalIf, 0, sizeof(pdk_vidd_physical_if_node));
    pPhysicalIf->pPhysicalIfInfo = malloc(sizeof(pdk_physical_if_info));

    /* Store a pointer to the VIDD control structure in the physical if struct*/
    pPhysicalIf->pViddCtrl = g_pViddCtrl;
    /* Set the physical if info */
    pPhysicalIf->pPhysicalIfInfo->feId = arg_pVif->fe_id;
    pPhysicalIf->pPhysicalIfInfo->portId = arg_pVif->port;
    pPhysicalIf->pPhysicalIfInfo->ipv4Addr = htonl(arg_pVif->ipaddr);
    pPhysicalIf->pPhysicalIfInfo->ipv4Mask = htonl(arg_pVif->mask);

    /* This is a unique device unit no. used in vxworks END */
    /* Basically made unique by multiplying FEID by Max Ports + Port ID */
    /* Assumes, Port ID starts from 0 */
    pPhysicalIf->pPhysicalIfInfo->unit = ((arg_pVif->fe_id * 100) + arg_pVif->port);

    strcpy(pPhysicalIf->pPhysicalIfInfo->name, (char*)VIDD_DEV_NAME);

    if (arg_pVif->type == NPF_PORT_ETHERNET)
    {
	pPhysicalIf->pPhysicalIfInfo->mediaType = MEDIA_TYPE_FAST_ETHERNET;
    } 
    else if (arg_pVif->type == NPF_PORT_ATM)
    {
	pPhysicalIf->pPhysicalIfInfo->mediaType = MEDIA_TYPE_ATM;
    }
    pPhysicalIf->pPhysicalIfInfo->MTU = arg_pVif->mtu;
    pPhysicalIf->pPhysicalIfInfo->linkSpeed = arg_pVif->linkspeed;
    pPhysicalIf->pPhysicalIfInfo->if_status = IF_STATUS_UP;

    /* Allocate a mac addr for the VIF */
    pPhysicalIf->pPhysicalIfInfo->macAddr[0] = 0x00;
    pPhysicalIf->pPhysicalIfInfo->macAddr[1] = 0xFF;
    pPhysicalIf->pPhysicalIfInfo->macAddr[2] = (arg_pVif->fe_id) & 0xFF;
    pPhysicalIf->pPhysicalIfInfo->macAddr[3] = (arg_pVif->port) & 0xFF;
    pPhysicalIf->pPhysicalIfInfo->macAddr[4] = (rand() % 256);
    pPhysicalIf->pPhysicalIfInfo->macAddr[5] = (rand() % 256);


    /* Register the physical if with the MUX. */
    pDevice = muxDevLoad(pPhysicalIf->pPhysicalIfInfo->unit, pdk_vidd_npt_load, NULL, 1, (void*)pPhysicalIf);
    if (pDevice == NULL)
    {
        err = -1;
        goto label_1;
    }

    /* Set the netpool for this interface to the VIDD netpool. */
    pPhysicalIf->end.pNetPool = g_pViddCtrl->pNetPool;

    pPhysicalIf->pMuxCookie = pDevice;

    pPhysicalIf->pNextPhysicalIf = NULL;

    /* Bind this MUX device to the IP protocol. */
    {
        STATUS ret_status = OK;
        char* devName = VIDD_DEV_NAME;
        char devUnit[IFNAME_LEN];
        uint32_t ipAddr;
        char ipAddrString[VIDD_IP_ADDR_STR_LEN];
            
        ipAddr = (pPhysicalIf->pPhysicalIfInfo->ipv4Addr);

        sprintf(devUnit, "%s%lu", devName, pPhysicalIf->pPhysicalIfInfo->unit);

        sprintf(ipAddrString, "%lu.%lu.%lu.%lu", ipAddr >> 24, 
	    (ipAddr >> 16) & 0xFF, (ipAddr >> 8) & 0xFF, ipAddr & 0xFF);

        /* Attach this MUX device to the IP stack. */
        ret_status = ipAttach(pPhysicalIf->pPhysicalIfInfo->unit, 
			VIDD_DEV_NAME);
        if(ret_status != OK)
        {
            err = -1;
            goto label_1;
        }

        /* Set the subnet mask for the device. */
        ifMaskSet(devUnit, (pPhysicalIf->pPhysicalIfInfo->ipv4Mask));

        /* Set the IP address for this device. */
        ifAddrSet(devUnit, ipAddrString);
    }        

    /* Link this physical interface into the Interface's list. */
    if(g_pFirstPhysicalIf == NULL)
        g_pFirstPhysicalIf = pPhysicalIf;
    else
    {
	pCurrPhysicalIf = g_pFirstPhysicalIf;
	while (pCurrPhysicalIf->pNextPhysicalIf != NULL)
	  pCurrPhysicalIf = pCurrPhysicalIf->pNextPhysicalIf;
        pCurrPhysicalIf->pNextPhysicalIf = pPhysicalIf;
    }
    g_numVIFs += 1;

    /* Assign the macaddr generated to the hwaddr argument */
    memcpy(hwaddr, pPhysicalIf->pPhysicalIfInfo->macAddr, ETH_ALEN);

    return err;

label_1:
    if(pPhysicalIf != NULL)
    {
        free(pPhysicalIf);
        pPhysicalIf = NULL;
    }

/*    pdk_vif_delete(pViddCtrl); */

    return err;
}

/**
 * NAME: _pdk_vidd_init_mem
 *
 * DESCRIPTION: This routine allocates the single shared memory structure common
 *              to all instances of the driver.
 * 
 * @Param:  - IN uint32_t arg_bufferSize - size in bytes of each buffer
 *          contained in the requested buffer pool.
 * @Param:  - IN pdk_vidd_ctrl* arg_pViddCtrl - pointer to the VIDD
 *          control structure.
 * @Return: 0, -1
 */
int _pdk_vidd_init_mem(
                                     uint32_t arg_bufferSize,
                                     pdk_vidd_ctrl* arg_pViddCtrl
                                     )
{

    int err = 0;

    /****************************************************************************
    * VxWorks network memory pool
    *
    * All instances of the driver share the memory buffers, this is done to
    * help minimize the total size of the pool while still having reasonable
    * response to bursts of packets.  This pool itself is maintained because
    * incoming packets must be copied so that the IP header can be properly
    * double-word aligned.
    *
    * The NULL and 0 fields in the following variables are filled in
    * later in this function.
    */
    M_CL_CONFIG mclConfig = {0, 0, NULL, 0};
    CL_DESC     clDescTbl[] = {
        /* 
        clusterSize      num                             memArea   memSize
        -----------      ----                            -------   -------
        */
        {arg_bufferSize, VIDD_NUM_ELEMENTS, NULL,   0}
    };

    int clDescTblNumElt = (NELEMENTS(clDescTbl));

    int lastAllocCnt; /* Do not change after the for() loop below */
    int totClBlkCnt = 0;

    /* Allocate memory for each of the cluster block sizes.
     * Each cluster is allocated to be of size:
     * (clusterSize + sizeof(long)) * # clusters
     * These cluster (memory regions) are stored in the array
     * clDescTbl.  See netBufLib for more information.
     * Upon exit from this loop:
     *   - clDescTbl[i] will contain the allocated memory
     *   - totClBlkCnt will contain the total number of clusters blocks
     *                 across all allocations
     *   - lastAllocCnt will contain the last index of clDescTbl
     *                  that successfully allocated
     */
    for (lastAllocCnt = 0; lastAllocCnt < clDescTblNumElt; lastAllocCnt++)
    {
        /**
         * Count the total # of blks so that we know how many mBlks
         * to allocate later
         */
        totClBlkCnt += clDescTbl[lastAllocCnt].clNum;

        /* Allocate the cluster */
        clDescTbl[lastAllocCnt].memSize = 
            clDescTbl[lastAllocCnt].clNum * 
            (clDescTbl[lastAllocCnt].clSize + sizeof(long));
        clDescTbl[lastAllocCnt].memArea = malloc(clDescTbl[lastAllocCnt].memSize);
        if ((char*)clDescTbl[lastAllocCnt].memArea == NULL) {
            err = -1;
            goto cleanupClusters;
        }
    }
    /* Store pointer to cluster memory area, to be freed on shutdown. */
    /* We assume that there is only one buffer size in the buffer pool. */
    arg_pViddCtrl->pClustMem = clDescTbl[0].memArea;
        
    /* Determine the mBlk appropriate parameters to pass to netPoolInit.
     * You have to have at least the same number of mBlks as you
     * do clusters.  You have to have 1:1 correspondence between clusters
     * and cluster blocks.
     * Each mBlk also requires memory, the size of which is
     *   M_BLK_SZ + sizeof(long)
     * Each cluster block requires memory, the size of which is
     *   CL_BLK_SZ
     */
    mclConfig.mBlkNum  = totClBlkCnt;  
    mclConfig.clBlkNum = totClBlkCnt; 
    mclConfig.memSize  = (mclConfig.mBlkNum * 
                              (M_BLK_SZ + sizeof(long))) +
                             (mclConfig.clBlkNum * CL_BLK_SZ);

    /* Allocate memory on a 4-byte alignment. */
    mclConfig.memArea = memalign(4, mclConfig.memSize);
    if(mclConfig.memArea == NULL)
    {
        err = -1;
        goto cleanupMblk;
    }
    /* Store pointer to Mblk memory area, to be freed at shutdown. */
    arg_pViddCtrl->pMclBlkCfg = mclConfig.memArea;

    /* Create the shared pool. */
    arg_pViddCtrl->pNetPool = malloc(sizeof(NET_POOL));
    if(arg_pViddCtrl->pNetPool == NULL)
    {
        err = -1;
        goto cleanupMblk;
    }
    memset(arg_pViddCtrl->pNetPool, 0, sizeof(NET_POOL));
    if(netPoolInit(arg_pViddCtrl->pNetPool, &mclConfig, &clDescTbl[0], clDescTblNumElt, NULL) != OK)
    {
        err = -1;
        goto cleanupMblk;
    }
        
    return err;
    
cleanupMblk:
    free(mclConfig.memArea);
    mclConfig.memArea = NULL;
    arg_pViddCtrl->pMclBlkCfg = NULL;
cleanupClusters:
    for (lastAllocCnt--; lastAllocCnt >= 0; lastAllocCnt--)
    {
        free(clDescTbl[lastAllocCnt].memArea);
        clDescTbl[lastAllocCnt].memArea = NULL;
        arg_pViddCtrl->pClustMem = NULL;
    }

    free(arg_pViddCtrl->pNetPool);
    arg_pViddCtrl->pNetPool = NULL;
    return err;
}


/**
 * NAME: pdk_vif_delete
 *
 * DESCRIPTION: This function is called by the CC Module to shutdown the local
 * VIDD and free any memory allocated to it.
 * 
 * @Param:  - IN void* arg_pContext - pointer to context, in this case the VIDD
 *          control structure.
 *
 * @Return: 0, -1
 */
int pdk_vif_delete(
                                PDK_VIC_t* arg_pVif
                                )
{
    STATUS ret_status = OK;

    char* devName = VIDD_DEV_NAME;
    pdk_vidd_physical_if_node* pPhysicalIf = NULL;
    pdk_vidd_physical_if_node* pPrevPhysicalIf;

    _vidd_get_physical_if(arg_pVif->port, arg_pVif->fe_id, &pPhysicalIf,
				&pPrevPhysicalIf);
    if(pPhysicalIf == NULL)
    {
        return -1;
    }

    /* Remove this physical interface from the Interface's list. */
    if (g_pFirstPhysicalIf == pPhysicalIf)
    {
	g_pFirstPhysicalIf = pPhysicalIf->pNextPhysicalIf;
    }
    else
    {
	pPrevPhysicalIf->pNextPhysicalIf = pPhysicalIf->pNextPhysicalIf;
    }

    /* Stop the driver for each port. */
    ret_status = muxDevStop(pPhysicalIf->pMuxCookie);
    if(ret_status != OK) 
    {
        return -1;
    }
            
    /* Unregister the IP addresses for this driver. */
    /* Check if ifAddrDelete() can be used here */
    ret_status = ifRouteDelete(devName, pPhysicalIf->pPhysicalIfInfo->unit);
    if(ret_status == ERROR)
        return -1;

    /* Unregister this driver from the MUX. */
    ret_status = muxDevUnload((char*)VIDD_DEV_NAME,
        pPhysicalIf->pPhysicalIfInfo->unit);
    if(ret_status != OK)
    {
        return -1;
    }

    /* pPhysical is freed in Unload() */
    pPhysicalIf = NULL;

    g_numVIFs -= 1;

    /* Free the common buffer pool ONLY if all interfaces are down */
    if (g_numVIFs == 0)
    {
      if(g_pViddCtrl->pNetPool != NULL)
      {
        /* Free the buffer pool. */
        ret_status = netPoolDelete(g_pViddCtrl->pNetPool);
        if(ret_status != OK)
        {
            return -1;
        }

        free(g_pViddCtrl->pMclBlkCfg);
        g_pViddCtrl->pMclBlkCfg = NULL;
        free(g_pViddCtrl->pClustMem);
        g_pViddCtrl->pClustMem = NULL;
        free(g_pViddCtrl->pNetPool);
        g_pViddCtrl->pNetPool = NULL;
      }
      free(g_pViddCtrl);
      g_pViddCtrl = NULL;
    }

    return 0;
}
    
int pdk_vif_setflags(
                                PDK_VIC_t* arg_pVif,
                                uint32_t if_status
                                )
{
    STATUS ret_status = OK;
    pdk_vidd_physical_if_node* pPhysicalIf = NULL;

    _vidd_get_physical_if(arg_pVif->port, arg_pVif->fe_id, &pPhysicalIf, NULL);
    if(pPhysicalIf == NULL)
    {
        return -1;
    }

    if (if_status == IF_STATUS_UP)
    {
	ret_status = muxDevStart(pPhysicalIf->pMuxCookie);
    } else if (if_status == IF_STATUS_DOWN)
    {
	ret_status = muxDevStop(pPhysicalIf->pMuxCookie);
    } else 
    {
        return -1;
    }
    if(ret_status != OK) 
    {
        return -1;
    }
    return 0;
}

int pdk_vif_ipaddr_update(
                                PDK_VIC_t* arg_pVif,
                                uint32_t ipAddr,
                                uint32_t subMask
                                )
{
    STATUS ret_status = OK;
    pdk_vidd_physical_if_node* pPhysicalIf = NULL;
    char* devName = VIDD_DEV_NAME;

    char devUnit[IFNAME_LEN];
    char ipAddrString[VIDD_IP_ADDR_STR_LEN];

    _vidd_get_physical_if(arg_pVif->port, arg_pVif->fe_id, &pPhysicalIf, NULL);
    if(pPhysicalIf == NULL)
    {
        return -1;
    }

    ret_status = ifRouteDelete(devName, pPhysicalIf->pPhysicalIfInfo->unit);
    if(ret_status == ERROR)
        return -1;

    ipAddr = htonl(ipAddr);

    sprintf(devUnit, "%s%lu", devName, pPhysicalIf->pPhysicalIfInfo->unit);
    sprintf(ipAddrString, "%lu.%lu.%lu.%lu", ipAddr >> 24, (ipAddr >> 16) & 0xFF,
        (ipAddr >> 8) & 0xFF, ipAddr & 0xFF);

    /* Set the subnet mask for the device. */
    ifMaskSet(devUnit, subMask);

    /* Set the IP address for this device. */
    ifAddrSet(devUnit, ipAddrString);

    /* Set the IP address in the physical if node struct */
    pPhysicalIf->pPhysicalIfInfo->ipv4Addr = ipAddr;
    pPhysicalIf->pPhysicalIfInfo->ipv4Mask = subMask;

    return 0;
}


int pdk_vif_mtu_config(
                                PDK_VIC_t* arg_pVif,
                                uint16_t MTU
                                )
{
    pdk_vidd_physical_if_node* pPhysicalIf = NULL;

    _vidd_get_physical_if(arg_pVif->port, arg_pVif->fe_id, &pPhysicalIf, NULL);
    if(pPhysicalIf == NULL)
    {
        return -1;
    }

    /* Cannot be supported - WRS */

    return -1;
}


int pdk_vif_linkspeed_config(
                                PDK_VIC_t* arg_pVif,
                                uint32_t linkspeed
                                )
{
    pdk_vidd_physical_if_node* pPhysicalIf = NULL;

    _vidd_get_physical_if(arg_pVif->port, arg_pVif->fe_id, &pPhysicalIf, NULL);
    if(pPhysicalIf == NULL)
    {
        return -1;
    }

    /* Cannot be supported - WRS */

    return -1;
}

int pdk_vidd_getNetPool(
                                NET_POOL_ID* arg_pNetPool
                                )
{
    if (g_pViddCtrl == NULL)
	return -1;

    *arg_pNetPool = g_pViddCtrl->pNetPool;

    return 0;
}


/**
 * NAME: _vidd_get_physical_if
 *
 * DESCRIPTION: 
 *
 * @Param:  - IN arg_portID - ID of the desired port.
 * @Param:  - IN arg_feID - ID of the FE.
 * @Param:  - INOUT arg_pPhysicalIf - pointer to the list of
 *          ports to traverse.  If we find the desired
 *          port structure, it will be returned in this pointer, otherwise
 *          it will be NULL.
 *
 * @Return: 0, -1
 */

int _vidd_get_physical_if ( 
			    uint32_t arg_portId,
			    uint32_t arg_feId,
			    pdk_vidd_physical_if_node** arg_pPhysicalIf,
			    pdk_vidd_physical_if_node** arg_pPrevPhysicalIf
			    )
{
    pdk_vidd_physical_if_node* pPhysicalIf;
    int ret_status = -1;

    if (arg_pPhysicalIf == NULL)
	return ret_status;

    pPhysicalIf = g_pFirstPhysicalIf;

    /* Traverse the list of VIFs to find the port with the given port ID */
    while(pPhysicalIf != NULL)
    {
        if((pPhysicalIf->pPhysicalIfInfo->portId == arg_portId) &&
	(pPhysicalIf->pPhysicalIfInfo->feId == arg_feId))
	{
	    ret_status = 0;
            break;
	}
	if (arg_pPrevPhysicalIf != NULL)
	    *arg_pPrevPhysicalIf = pPhysicalIf;

        pPhysicalIf = pPhysicalIf->pNextPhysicalIf;
    }
    if (ret_status == 0)
	*arg_pPhysicalIf = pPhysicalIf;

    return ret_status;
}

/****************************************************************************
 * Driver functions:
 *    These routines are provided to the kernel during registration.
 *    These routines are the same across all instances of the driver.
 ****************************************************************************/

/**
 * NAME: pdk_vidd_npt_load
 *
 * DESCRIPTION: This function registers a port with the VxWorks MUX layer.
 * The user does not call this function directly - the VxWorks MUX layer uses it
 * as an entry point when adding a new physical interface.  This function
 * is implemented as a two-pass algorithm - the MUX layer calls this
 * function twice - once with an empty initialization string, and then with
 * the real initialization string.
 * 
 * @Param:  - IN void* pBsp - optional BSP-specific information.  This will
 *          be used as a context, in this case a pointer to the VIDD control
 *          structure.
 * @Param:  - INOUT char* initString - on the first pass of this function,
 *          initString is passed in as an empty allocated string, and the
 *          base name of the interface (e.g. "eth") is copied into it.  On
 *          the second pass, initString is NULL.
 *
 * @Return: Pointer to the END_OBJ structure contained in the VIDD interface data structure.
 *          NULL
 */
END_OBJ* pdk_vidd_npt_load(
                                    char* initString,
                                    void *pBsp
                                    )
{
    pdk_vidd_physical_if_node* pPhysicalIf = NULL;
    STATUS err = OK;

    uint32_t ifType = 0;

    /* Case 1 - return the name of the driver */
    if (initString != NULL)
    {
        if('\0' == initString[0])
        {
            strcpy(initString, (char*)VIDD_DEV_NAME);
/*            memcpy(initString, (char*)VIDD_DEV_NAME, IFNAME_LEN); */
            return (0);
        }
    }
    
    /* Case 2 - create a new instance of the driver */
    if (pBsp == NULL)
    {
        return NULL;
    }

    /* END object is already allocated by the app */
    pPhysicalIf = (pdk_vidd_physical_if_node*)pBsp;

    /* Initialize the END parts of the driver structure. */
    err = END_OBJ_INIT(&pPhysicalIf->end, (DEV_OBJ*)pPhysicalIf, 
                     VIDD_DEV_NAME,
                     pPhysicalIf->pPhysicalIfInfo->unit,
                     &pdk_vidd_func_table,
                     "CP PDK VIDD");
    if(err != OK)
    {
        return NULL;
    }

    /* Initialize the MIB2 parts of the driver structure */
    switch(pPhysicalIf->pPhysicalIfInfo->mediaType)
    {
        case MEDIA_TYPE_FAST_ETHERNET:
            ifType = M2_ifType_fastEther;
            break;
        case MEDIA_TYPE_1GB_ETHERNET:
            ifType = M2_ifType_gigabitEthernet;
            break;
        case MEDIA_TYPE_ATM :
            ifType = M2_ifType_atm;
            break;
        case MEDIA_TYPE_POS:
            ifType = M2_ifType_sonet;
            break;
        default:
            ifType = M2_ifType_other;
    }

    switch(pPhysicalIf->pPhysicalIfInfo->mediaType)
    {
        case MEDIA_TYPE_FAST_ETHERNET:
        case MEDIA_TYPE_1GB_ETHERNET:
            err = END_MIB_INIT(&pPhysicalIf->end, 
                        ifType,
                        pPhysicalIf->pPhysicalIfInfo->macAddr,
                        ETH_ALEN,
                        (uint16_t)pPhysicalIf->pPhysicalIfInfo->MTU,
                        pPhysicalIf->pPhysicalIfInfo->linkSpeed*1000000);
            break;
        case MEDIA_TYPE_ATM:
        case MEDIA_TYPE_POS:
        default:
            err = END_MIB_INIT(&pPhysicalIf->end, 
                    ifType,
                    NULL,
                    0,
                    (uint16_t)pPhysicalIf->pPhysicalIfInfo->MTU,
                    pPhysicalIf->pPhysicalIfInfo->linkSpeed*1000000);   
    }
    
    if(err != OK)
    {
        return NULL;
    }

    
    /* Set the flags to indicate readiness. */
    if(pPhysicalIf->pPhysicalIfInfo->if_status == IF_STATUS_UP)
    {
        END_OBJ_READY(&pPhysicalIf->end,
                   IFF_UP | IFF_RUNNING | IFF_NOTRAILERS | IFF_BROADCAST 
                   | IFF_MULTICAST | IFF_NOARP & ~IFF_PROMISC & ~IFF_RFC2233);
    }
    else
    {
        END_OBJ_READY(&pPhysicalIf->end,
                   ~IFF_UP & IFF_RUNNING | IFF_NOTRAILERS | IFF_BROADCAST
                   | IFF_MULTICAST | IFF_NOARP & ~IFF_PROMISC & ~IFF_RFC2233);
    }
    

    return (&pPhysicalIf->end);    
}


/**
 * NAME: pdk_vidd_receive_pkt
 *
 * DESCRIPTION: This is the VIDD entry point (packet processing
 * callback invoked by the CC Module) that receives packets from
 * the CC Module and passes them to the local network stack.
 * 
 * @Param:  - IN M_BLK_ID pMblk - handle to the input packet.
 * @Param:  - IN void* arg_pCtx - pointer to the context, in this case
 *          the VIDD control structure.
 *
 * @Return: 0, -1
 */
int pdk_vidd_receive_pkt(
                                       M_BLK_ID pMblk,
					uint32_t portId,
					uint32_t feId
                                       )
{
    int err = 0;
    pdk_vidd_physical_if_node* pPhysicalIf = NULL;

    /**
     * Validate input and return -1 if it is invalid.
     */
    if(pMblk == NULL)
    {
        err =  -1;
        goto label_1;
    }

    /**
     * Given the input port ID, look-up the END_OBJ data struct from the list of
     * virtual interfaces.
     */
    _vidd_get_physical_if(portId, feId, &pPhysicalIf, NULL);
    if(pPhysicalIf == NULL)
    {
        err = -1;
        goto label_1;
    }

    /* Any other checks ?? */

    /* Call the receive routine for the stack with a pointer to the mBlk chain. */
    TK_RCV_RTN_CALL(&pPhysicalIf->end, pMblk, 0, 0x800, 0, 0); 

    if(pMblk != NULL)
    {
        netMblkClChainFree(pMblk);
    }

    return err;

label_1:
    /* Should check if end is NULL */
    END_ERR_ADD(&pPhysicalIf->end, MIB2_IN_ERRS, +1);

    /* Free mBlk chain */
    if(pMblk != NULL)
    {
        netMblkClChainFree(pMblk);
    }

    return err;
}


/**
 * NAME: pdk_vidd_npt_send
 *
 * DESCRIPTION: The MUX interface calls this function when the network
 * protocol is sending a network packet. Network buffers are represented 
 * by an mBlk chain in VxWorks. This function takes the packet stored in
 * the mBlk, performs all necessary checks on the packet, and stores the
 * packet data before calling the cppdk API pdk_send_packet() function.
 * 
 * @Param:  - IN END_OBJ* pEnd - pointer to the END_OBJ structure.
 *          It identifies the transmit interface.
 * @Param:  - IN M_BLK_ID pMblk - mBlk chain containing the network
 *          buffer. The data represents the full link layer frame. 
 * @Param:  - IN char* dstMacAddr - destination MAC address from the
 *          OS stack. Ignored by this function as the IPv4 CC will do a
 *          route lookup to obtain the destination MAC address.
 * @Param:  - IN long netType - network service type.  Ignored by VIDD.
 * @Param:  - IN void* pSpare - optional network service data.  Ignored by VIDD.
 *
 * @Return: OK
 *          ERROR
 */
STATUS pdk_vidd_npt_send(
                                  END_OBJ* pEnd, 
                                  M_BLK_ID pMblk, 
                                  char* dstMacAddr,
                                  long netType,
                                  void* pSpare
                                  )
{
    pdk_vidd_physical_if_node* pPhysicalIf = (pdk_vidd_physical_if_node*)pEnd;
    uint32_t portId, feId;
    M_BLK_ID pCurrMblk = pMblk;
    uint16_t dataOffset = 0;

    /* From pEnd pointer extract interface id and store it. */
    portId = pPhysicalIf->pPhysicalIfInfo->portId;
    feId = pPhysicalIf->pPhysicalIfInfo->feId;

    /* If the packet is an IP packet, turn off the most significant bit in the IP ID  - not needed for CP PDK VIDD */

    while(pCurrMblk != NULL)
    {
        /* Dump packet to screen. */

	/* Not Sure about Data OffSet */
        if(pCurrMblk->mBlkHdr.mLen > (MIN_BUFFER_SIZE - dataOffset))
            goto label_1;

        pCurrMblk = pCurrMblk->mBlkHdr.mNext;
    }

    /* Send the packet to the Data Channel */
    /* Not sure if need to pass dstMacAddr */
    ph_write_packet(feId, portId, pMblk);

    /* Return OK to the MUX. */
    return OK;

label_1:
    /* Free the Mblk chain. */
    if(pMblk != NULL)
    {
        netMblkClChainFree(pMblk);
    }

    return (ERROR);
}


/**
 * NAME: pdk_vidd_npt_unload
 *
 * DESCRIPTION: This function is called by the MUX to "release" the device.
 * The function is called for each port that has been activated by call to
 * the pdk_vidd_npt_load().
 * 
 * @Param:  - IN END_OBJ* pEnd - Pointer to END_OBJ data structure
 *          allocated by the Load() function.
 *
 * @Return: OK
 */
STATUS pdk_vidd_npt_unload(
                                    END_OBJ* pEnd
                                    )
{
    pdk_vidd_physical_if_node* pPhysicalIf =
        (pdk_vidd_physical_if_node*)pEnd;

    END_FLAGS_CLR(&pPhysicalIf->end,
                   IFF_UP | IFF_RUNNING | IFF_NOTRAILERS | IFF_BROADCAST
                   | IFF_PROMISC | IFF_MULTICAST | IFF_NOARP);
    
    END_OBJECT_UNLOAD(&pPhysicalIf->end);

#if (!defined(IX_TARGET_SIMNTgnu_Debug) && !defined(IX_TARGET_SIMNTgnu))
    /* Delete the semaphore created by END_OBJ_INIT. */
    /* May need to change to support hot-swappability. */
    if(pPhysicalIf->end.txSem != 0)
    {
        semTake(pPhysicalIf->end.txSem, WAIT_FOREVER);
        semDelete(pPhysicalIf->end.txSem);
        pPhysicalIf->end.txSem = 0;
    }
#endif /* (!defined(IX_TARGET_SIMNTgnu_Debug) && !defined(IX_TARGET_SIMNTgnu)) */

    return OK;
}

/**
 * NAME: pdk_vidd_npt_ioctl
 *
 * DESCRIPTION: The VIDD needs to support ioctl commands to keep
 * the ioctl interface with existing network protocols. The table
 * below gives the list of commonly-used ioctl commands.
 * Command          Function                        Data Type       Supported
 * --------------------------------------------------------------------------
 * SIOCGIFMTU       Get MTU.                        char*           Yes
 * EIOCSFLAGS       Set device flags                int             Yes
 * EIOCGFLAGS       Get device flags                int             Yes
 * EIOCSADDR        Set device address              char*           No
 * EIOCGADDR        Get device address              char*           Yes
 * EIOCMULTIADD     Add multicast address           char*           No
 * EIOCMULTIDEL     Delete multicast address        char*           No
 * EIOCMULTIGET     Get multicast list              MULTI_TABLE*    No
 * EIOCPOLLSTART    Set device into polling mode    NULL            No
 * EIOCPOLLSTOP     Set device into interrupt mode  NULL            No
 * EIOCGFBUF        Get minimum first buffer
 *                  for chaining                    int             Yes
 * EIOCGHDRLEN      Get the size of the data
 *                  link header                     int             Yes
 * EIOCGNPT         Query a driver to determine
 *                  whether it is a NPT driver      int             Yes
 * EIOCGMIB2233     retrieves the RFC2233
 *                  MIB II table                    M2_ID*          Yes
 * EIOCGMIB2        Get the MIB-II counters
 *                  from the driver                 char*           Yes
 *
 * 
 * @Param:  - IN END_OBJ* pEnd - Pointer to END_OBJ data structure
 *          allocated by the Load() function.
 * @Param:  - IN int command - ioctl command.
 * @Param:  - INOUT caddr_t buffer - character buffer holding response from the command.
 *
 * @Return: OK
 *          EINVAL
 */
int pdk_vidd_npt_ioctl(
                                END_OBJ* pEnd,
                                int command,
                                caddr_t buffer
                                )
{
    int value;
    int retval = OK;

    /* Cast pEnd to pdk_vidd_physical_if. */
    pdk_vidd_physical_if_node* pPhysicalIf = (pdk_vidd_physical_if_node*)pEnd;

    /* Switch on command - for all unsupported cases listed above, return EINVAL. */
    switch(command)
    {
        /* For SIOCGIFMTU, copy the MTU from pEnd into buffer. */
        case SIOCGIFMTU:
            *((uint16_t*)buffer) = pPhysicalIf->pPhysicalIfInfo->MTU;
            break;

        
       /**
        * For EIOCSFLAGS, set the END flags - this does not affect the
        * underlying hardware in this release. Use the set_property API
        * instead to bring an interface up or down.
        */
        case EIOCSFLAGS:
            value = (long) buffer;
            if (value < 0)
            {
                value -= value;
                value--;
                END_FLAGS_CLR(pEnd, value);
            }
            else
            {
                END_FLAGS_SET(pEnd, value);
            }
            break;
    
        /* For EIOCGFLAGS, set *buffer to END_FLAGS_GET (pEnd). */
        case EIOCGFLAGS:
            *((uint32_t*)buffer) = END_FLAGS_GET(pEnd);
            break;
    
        /* For EIOCGADDR, copy MAC address from pEnd into buffer. */
        case EIOCGADDR:
            memcpy(buffer, (char*)pPhysicalIf->pPhysicalIfInfo->macAddr, ETH_ALEN);
            break;
        
        /* For EIOCGFBUF, set *buffer to the minimum buffer size used in the VxWorks network buffer pool. */
        case EIOCGFBUF:
            *((int16_t*)buffer) = (int16_t)MIN_BUFFER_SIZE;
            break;

        /* For EIOCGHDRLEN, set *buffer to the L2 header length. */
        case EIOCGHDRLEN:
            *buffer = ETH_ALEN;
            break;

        /* For EIOCGNPT, return OK. */
        case EIOCGNPT:
            break;

        /* For EIOCGMIB2233, set *buffer to the pointer to the MIB2 table stored in pEnd. */
        case EIOCGMIB2233:
            if ((buffer == NULL) || (pEnd->pMib2Tbl == NULL))
            {
               retval = EINVAL;
            }
            else
            {
               *((M2_ID **)buffer) = pEnd->pMib2Tbl;
            }
            break;

        /* For EIOCGMIB2, copy the MIB2 table pointer stored in pEnd into buffer. */
        case EIOCGMIB2:
            if(buffer == NULL)
            {
                retval = EINVAL;
            }
            memcpy((char*)buffer, (char*)&pEnd->mib2Tbl, sizeof(pEnd->mib2Tbl));
            break;

        /* For unsupported ioctl's and the default case, return EINVAL. */
        case EIOCSADDR:
        case EIOCMULTIADD:
        case EIOCMULTIDEL:
        case EIOCMULTIGET:
        case EIOCPOLLSTART:
        case EIOCPOLLSTOP:
        default:
            retval = EINVAL;
    }

    return retval;
}


/**
 * NAME: pdk_vidd_npt_start
 *
 * DESCRIPTION: This function notifies the MUX that an interface is
 *              being brought up.
 * 
 * @Param:  - IN END_OBJ* pEnd - Pointer to END_OBJ data structure
 *          allocated by the Load() function.
 *
 * @Return: OK
 */
STATUS pdk_vidd_npt_start(
                                     END_OBJ* pEnd
                                     )
{
    pdk_vidd_physical_if_node* pPhysicalIf = (pdk_vidd_physical_if_node*)pEnd;
    pPhysicalIf->end.flags |= IFF_UP;
    /**
     * If the driver changes the state of the flags element, the driver
     * should export this change by calling muxError().
     */
    {
        END_ERR flagErr = {END_ERR_UP, NULL, NULL};
        muxError(pEnd, &flagErr);
    }
    return OK;
}

/**
 * NAME: pdk_vidd_npt_stop
 *
 * DESCRIPTION: Stop() notifies the MUX layer that an interface is
 *              being brought down.
 * 
 * @Param:  - IN END_OBJ* pEnd - Pointer to END_OBJ data structure
 *          allocated by the Load() function.
 * @Param:  - INOUT
 * @Param:  - OUT  
 *
 * @Return: OK
 */
STATUS pdk_vidd_npt_stop(
                                    END_OBJ* pEnd
                                    )
{
    pdk_vidd_physical_if_node* pPhysicalIf = (pdk_vidd_physical_if_node*)pEnd;
    pPhysicalIf->end.flags &= ~IFF_UP;
    
    /**
     * If the driver changes the state of the flags element, the driver
     * should export this change by calling muxError().
     */
    {
        END_ERR flagErr = {END_ERR_DOWN, NULL, NULL};
        muxError(pEnd, &flagErr);
    }

    return OK;
}

/**
 * NAME: pdk_vidd_npt_mCastAddrAdd
 *
 * DESCRIPTION: This function adds a new link-layer multicast address
 * to the table of multicast addresses for the interface - in the
 * initial release this call does not affect the underlying hardware.
 * 
 * @Param:  - IN END_OBJ* pEnd - pointer to the END_OBJ structure to
 *          help identify the port.
 * @Param:  - IN char* pAddress - physical address to add to the
 *          Multicast table.
 *
 * @Return: OK
 *          ERROR
 */
STATUS pdk_vidd_npt_mCastAddrAdd(
                                          END_OBJ * pEnd,
                                          char* pAddress
                                          )
{
    pdk_vidd_physical_if_node* pPhysicalIf = (pdk_vidd_physical_if_node*)pEnd;
    return etherMultiAdd(&pPhysicalIf->end.multiList, pAddress);
}

/**
 * NAME: pdk_vidd_npt_mCastAddrDel
 *
 * DESCRIPTION: The function removes previously added link-layer
 * multicast address - in the initial release this does not affect
 * the underlying hardware.
 * 
 * @Param:  - IN END_OBJ* pEnd - pointer to the END_OBJ structure to
 *          help identify the port.
 * @Param:  - IN char* pAddress - physical address to delete from the
 *          Multicast table.
 *
 * @Return: OK
 *          ERROR
 */
STATUS pdk_vidd_npt_mCastAddrDel(
                                          END_OBJ * pEnd,
                                          char* pAddress
                                          )
{
    pdk_vidd_physical_if_node* pPhysicalIf = (pdk_vidd_physical_if_node*)pEnd;
    return etherMultiDel(&pPhysicalIf->end.multiList, pAddress);
}

/**
 * NAME: pdk_vidd_npt_mCastAddrGet
 *
 * DESCRIPTION: This function gets the list of all multicast
 * addresses that are active on the interface. 
 * 
 * @Param:  - IN END_OBJ* pEnd - pointer to the END_OBJ structure to
 *          help identify the port.
 * @Param:  - IN MULTI_TABLE* pTable - pointer to the structure where
 *          the list will be put.
 *
 * @Return: OK
 *          ERROR
 */
STATUS pdk_vidd_npt_mCastAddrGet(
                                          END_OBJ * pEnd,
                                          MULTI_TABLE* pTable
                                          )
{
    pdk_vidd_physical_if_node* pPhysicalIf = (pdk_vidd_physical_if_node*)pEnd;
    return etherMultiGet(&pPhysicalIf->end.multiList, pTable);
}

/**
 * NAME: pdk_vidd_npt_pollSend
 *
 * DESCRIPTION: Polling-mode equivalent to the send() routine.
 * This routine should transfer frames directly to the output core
 * component or exit immediately if the core component is busy -
 * this is not supported in the initial release and always returns ERROR.
 * 
 * @Param:  - IN END_OBJ* pEnd - pointer to the END_OBJ structure. It
 *          identifies the transmit interface.
 * @Param:  - IN M_BLK_ID pMblk - mBlk chain containing the network
 *          buffer. The data represents the full link layer frame.
 * @Param:  - IN char* dstMacAddr - destination MAC address from the
 *          OS stack. Ignored by this function as the VIDD will do its
 *          own route table lookup to obtain the destination MAC address.
 * @Param:  - IN long netType - network service type.
 * @Param:  - IN void* pSpare - optional network service data.
 *
 * @Return: ERROR
 */
STATUS pdk_vidd_npt_pollSend(
					END_OBJ* pEND,
                                      M_BLK_ID pPkt,
                                      char* dstAddr,
                                      long netType,
                                      void * pSpareData
                                      )
{
    return ERROR;
}

/**
 * NAME: pdk_vidd_npt_pollRcv
 *
 * DESCRIPTION: The current implementation of core component model
 * does not support PollReceive().  Therefore this function always
 * returns an error.
 * 
 * @Param:  - IN END_OBJ* pEnd - pointer to the END_OBJ structure. It
 *          identifies the transmit interface.
 * @Param:  - IN M_BLK_ID pMblk - mBlk chain containing the network
 *          buffer. The data represents the full link layer frame.
 * @Param:  - IN long* pNetSvc - network service type.
 * @Param:  - IN long* pNetOffset - offset to network frame.
 * @Param:  - IN void* pSpare - optional network service data.
 *
 * @Return: OK
 *          ERROR
 */
STATUS pdk_vidd_npt_pollRcv(
					END_OBJ* pEND,
                                     M_BLK_ID pMblk,
                                     long* pNetSvc,
                                     long* pNetOffset, 
                                     void* pSpareData
                                     )
{
    return ERROR;
}

/**
 * NAME: pdk_vidd_npt_formAddress
 *
 * DESCRIPTION: For Ethernet interfaces, the stack will attempt to
 *              prepend a MAC address to outgoing packets.  Thus this
 *              function just returns OK, because the MAC address
 *              will be formed by the uBlocks.
 *
 * @Param:  - IN M_BLK_ID arg_pData - mBlk chain containing outgoing data.
 * @Param:  - IN M_BLK_ID arg_pSrc - source address, in an mBlk.
 * @Param:  - IN M_BLK_ID arg_pDst - destination address, in an mBlk.
 *
 * @Return: OK
 */
STATUS pdk_vidd_npt_formAddress(
                                         M_BLK_ID arg_pData,
                                         M_BLK_ID arg_pSrc,
                                         M_BLK_ID arg_pDst
                                         )
{
    return OK;
}


#endif /* _VXWORKS_ */
