/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2002 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/DataRelayShim/inc/msgthreads.h,v $
 * $Revision: 1.6 $
 * $Date: 2003/11/17 16:57:15 $
 *
 ****************************************************************************/

#ifndef DRS_MSG_THREADS_HDR
#define DRS_MSG_THREADS_HDR

#ifdef __cplusplus
extern "C" {
#endif

#include "list.h"
#include "pil.h"
#include "fpp_common.h"

/*******************************************************************************
// Structure containing ptrs to event queue and supporting elements
// for inter-thread communication.
*******************************************************************************/

typedef struct tag_QUEUE_SERVICE_PARAMS
{
    DList               msgQueue;
    PilCriticalSection  queSyncObj;
    PilThread           thrdHandle;
    PilSemaphore        newMsgNotifier;
    PilSemaphore        thrdUpNotifier;
    FPPI_RET            retVal;
} QUEUE_SERVICE_PARAMS, * LPQUEUE_SERVICE_PARAMS;

typedef struct tag_METHOD_INFO
{
    FPPI_CBHANDLE    cbhandle;
    FPPI_CORRELATOR  correlator;
} METHOD_INFO, * LPMETHOD_INFO;


/*******************************************************************************
// Define event types for event loop
*******************************************************************************/

#define DRS_EVENT_SHUTDOWN           0
#define DRS_MSG_TO_FP                1
#define DRS_EVENT_MSG_TO_CP          2
#define DRS_RESPONSE_MSG_TO_CP       3

/*******************************************************************************
// Structures to wrap params before inserting into event queue
*******************************************************************************/

/* Just the event type for generic use */
typedef struct tag_DRS_PARAM_TYPE
{
    uint32_t evType;
} DRS_PARAM_TYPE, * LPDRS_PARAM_TYPE;

/* For general message passing. This is the definition of the object that
   msgs are packed into for subsequent stuffing into the queues. Note that
   the correlator passed into the FP is a ptr to a METHOD_INFO obj. This
   allows the DRS to find the correct correlator and cbhandle when the FP
   calls back into the DRS for the async callback (the methInfo list is
   search for that address). */
typedef struct tag_DRS_GENERAL_EVENT_MSG
{
    uint32_t         evType;
    NPF_CBCAT        cbtype;
    FPPI_FEID        feid;
    void*            correlator;
    void*            outObj;
} DRS_GENERAL_EVENT_MSG, * LPDRS_GENERAL_EVENT_MSG;

/*******************************************************************************
// Method prototypes
*******************************************************************************/

FPPI_RET InitThreadObj(
    LPQUEUE_SERVICE_PARAMS drsThrdParms);

FPPI_RET ShutdownDRSThread(
    LPQUEUE_SERVICE_PARAMS srvcThrdParms);

FPPI_RET CreateDRSThread(
    LPQUEUE_SERVICE_PARAMS drsThrdParms,
    char* threadName);

FPPI_RET DestroyThreadObj(
    LPQUEUE_SERVICE_PARAMS drsThrdParms);

#ifdef __cplusplus
}
#endif

#endif /* DRS_MSG_THREADS_HDR */
