/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/DataRelayShim/src/fppapi.c,v $
 * $Revision: 1.7 $
 * $Date: 2003/11/17 16:57:15 $
 *
 * Description:
 *     Contains the top level API for the data relay shim (DRS) at the
 *     control plane end.
 *
 ****************************************************************************/

#include <string.h>
#include "log.h"
#include "npf_ds_types.h"

#ifdef _INCLUDE_MPLS
#include "fppapi_mpls_types.h"
#endif

#include "fpp_common.h"
#include "drspriv.h"
#include "fppCapabilityTypes.h"
#include "backendapi_types.h"
#include "pdk_callback_types.h"

#ifdef _INCLUDE_ATM

#include "pdk_callback.h"
#include "pdk_cm.h"
#include "npf_if.h"
#include "list.h"
#include "npf_atm_types.h"
#include "fppapi_atm_types.h"

#endif /* _INCLUDE_ATM */
/****************************************************************************
 * Function: FPPAPI_ipv4_unicastAddNextHop()                                *
 * ---------------------------                                              *
 * Parameters: feid       - unique ID for a FE created by TPlugin           *
 *             correlator - used to distinguish multiple invocations        *
 *             cbhandle   - unique ID used to identify the callback fn      *
 *             nhInfo     - the number of entries in the array              *
 *                                                                          *
 * Description: This function is used to add IPv4 unicast next hop entries  *
 *              to a FE.                                                    *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastAddNextHop(
    FPPI_FEID                        feid,
    FPPI_CORRELATOR                  correlator,
    FPPI_CBHANDLE                    cbhandle,
    NPF_uint32_t                     numEntries,
    FPPI_UnicastNextHopUpdateInfo_t* nhInfo)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_IPv4_UniAddNextHop, nhInfo,
        numEntries);
}

/****************************************************************************
 * Function: FPPAPI_ipv4_unicastDelNextHop()                                *
 * ---------------------------                                              *
 * Parameters: feid       - unique ID for a FE created by TPlugin           *
 *             correlator - used to distinguish multiple invocations        *
 *             cbhandle   - unique ID used to identify the callback fn      *
 *             nhid       - user ID used to identify a prefix (n/w addr)    *
 *             numEntries - array of next hop entries to be added           *
 *             nhInfo     - the number of entries in the array              *
 *                                                                          *
 * Description: This function is used to delete IPv4 unicast next hop       *
 *              entries from an FE.                                         *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastDelNextHop(
    FPPI_FEID              feid,
    FPPI_CORRELATOR        correlator,
    FPPI_CBHANDLE          cbhandle,
    NPF_uint32_t           numEntries,
    FPPI_UnicastNextHopDeleteInfo_t* nhDelInfo)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_IPv4_UniDelNextHop, nhDelInfo,
        numEntries);
}

/****************************************************************************
 * Function: FPPAPI_ipv4_unicastFlushNextHop()                              *
 * ---------------------------                                              *
 * Parameters: feid       - unique ID for a FE created by TPlugin           *
 *             correlator - used to distinguish multiple invocations        *
 *             cbhandle   - unique ID used to identify the callback fn      *
 *                                                                          *
 * Description: This function is used to flush all IPv4 unicast next hop    *
 *              entries from an FE.                                         *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastFlushNextHop(
    FPPI_FEID              feid,
    FPPI_CORRELATOR        correlator,
    FPPI_CBHANDLE          cbhandle,
    NPF_uint32_t           numEntries,
    FPPI_UnicastNextHopDeleteInfo_t* nhFlushInfo)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_IPv4_UniFlushNextHop,
        nhFlushInfo,numEntries);
}

/****************************************************************************
 * Function: FPPAPI_ipv4_unicastQueryNextHop()                              *
 * ---------------------------                                              *
 * Parameters: feid       - unique ID for a FE created by TPlugin           *
 *             correlator - used to distinguish multiple invocations        *
 *             cbhandle   - unique ID used to identify the callback fn      *
 *             nhid       - user ID used to identify a prefix (n/w addr)    *
 *                                                                          *
 * Description: This function is used to query all IPv4 unicast next hop    *
 *              entries from an FE.                                         *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastQueryNextHop(
    FPPI_FEID              feid,
    FPPI_CORRELATOR        correlator,
    FPPI_CBHANDLE          cbhandle,
    NPF_uint32_t           numEntries, 
    NPF_uint32_t*          nhid) 
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_IPv4_UniQueryNextHop,
        nhid, numEntries);
}

/***************************************************************************
 * Function: FPPAPI_ipv4_unicastAddPrefix()                                *
 * ---------------------------                                              *
 * Parameters: feid       - unique ID for a FE created by TPlugin           *
 *             correlator - used to distinguish multiple invocations        *
 *             cbhandle   - unique ID used to identify the callback fn      *
 *             numEntries - array of next hop entries to be added           *
 *             prfxInfo   - the number of entries in the array              *
 *                                                                          *
 * Description: This function is used to add IPv4 unicast prefix entries    *
 *              to an FE.                                                   *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastAddPrefix(
    FPPI_FEID                      feid,
    FPPI_CORRELATOR                correlator,
    FPPI_CBHANDLE                  cbhandle,
    NPF_uint32_t                   numEntries,
    FPPI_UnicastPrefixUpdateInfo_t* prfxInfo)
{
    return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
        FPPI_IPv4_UniAddPrefix, prfxInfo, numEntries);
}

/****************************************************************************
 * Function: FPPAPI_ipv4_unicastUpdatePrefix()                              *
 * ---------------------------                                              *
 * Parameters: feid       - unique ID for a FE created by TPlugin           *
 *             correlator - used to distinguish multiple invocations        *
 *             cbhandle   - unique ID used to identify the callback fn      *
 *             numEntries - array of next hop entries to be added           *
 *             prfxInfo   - the number of entries in the array              *
 *                                                                          *
 * Description: This function is used to update IPv4 unicast prefix entries *
 *              on an FE.                                                   *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastUpdatePrefix(
    FPPI_FEID                      feid,
    FPPI_CORRELATOR                correlator,
    FPPI_CBHANDLE                  cbhandle,
    NPF_uint32_t                   numEntries,
    NPF_IPv4UC_Prefix_t*           prfxInfo)
{
    return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
        FPPI_IPv4_UniUpdatePrefix, prfxInfo, numEntries);
}

/****************************************************************************
 * Function: FPPI_RET FPPAPI_ipv4_unicastDelPrefix()                        *
 * ---------------------------                                              *
 * Parameters: feid       - unique ID for a FE created by TPlugin           *
 *             correlator - used to distinguish multiple invocations        *
 *             cbhandle   - unique ID used to identify the callback fn      *
 *             numEntries - array of next hop entries to be added           *
 *             prfxInfo   - the number of entries in the array              *
 *                                                                          *
 * Description: This function is used to delete IPv4 unicast prefix entries *
 *              from an FE.                                                 *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastDelPrefix(
    FPPI_FEID                      feid,
    FPPI_CORRELATOR                correlator,
    FPPI_CBHANDLE                  cbhandle,
    NPF_uint32_t                   numEntries,
    NPF_IPv4UC_Prefix_t*           prfxInfo)
{
    return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
        FPPI_IPv4_UniDelPrefix, prfxInfo, numEntries);
}

/****************************************************************************
 * Function: FPPI_RET FPPAPI_ipv4_unicastFlushPrefix()                      *
 * ---------------------------                                              *
 * Parameters: feid       - unique ID for a FE created by TPlugin           *
 *             correlator - used to distinguish multiple invocations        *
 *             cbhandle   - unique ID used to identify the callback fn      *
 *                                                                          *
 * Description: This function is used to delete all IPv4 unicast prefix     *
 *              entries from an FE.                                         *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastFlushPrefix(
    FPPI_FEID              feid,
    FPPI_CORRELATOR        correlator,
    FPPI_CBHANDLE          cbhandle)
{
    return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
        FPPI_IPv4_UniFlushPrefix, NULL, 0);
}

/****************************************************************************
 * Function: FPPI_RET FPPI_RET FPPAPI_ipv4_unicastQueryPrefix()             *
 * ---------------------------                                              *
 * Parameters: feid       - unique ID for a FE created by TPlugin           *
 *             correlator - used to distinguish multiple invocations        *
 *             cbhandle   - unique ID used to identify the callback fn      *
 *                                                                          *
 * Description: This function is used to query all IPv4 unicast prefix      *
 *              entries on an FE.                                           *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastQueryPrefix(
    FPPI_FEID              feid,
    FPPI_CORRELATOR        correlator,
    FPPI_CBHANDLE          cbhandle,
    NPF_uint32_t           numEntries,
    NPF_IPv4UC_Prefix_t    *prfxInfo)
{
    return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
        FPPI_IPv4_UniQueryPrefix, prfxInfo, numEntries);
}

/****************************************************************************
 * Function: FPPAPI_ipv4_unicastToL2Add()                                   *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID       - unique ID for a FE created by TPlugin      *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE   - unique ID used to identify the callback fn *
 *             entry_count     - the number of entries in the array         *
 *             entry_array     - array of ARP entries to be added           *
 *                                                                          *
 * Description: This function is used to add IPv4 ARP entries to a FE.      *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastToL2Add(
    FPPI_FEID                    feid,
    FPPI_CORRELATOR              correlator,
    FPPI_CBHANDLE                cbhandle,
    uint32_t                     entry_count,
    FPPI_IPv4UniNHResolEntry_t*  entry_array)
{
    return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
        FPPI_IPv4_UniAddToL2, entry_array, entry_count);
}

/****************************************************************************
 * Function: FPPAPI_ipv4_unicastToL2Del()                                   *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID       - unique ID for a FE created by TPlugin      *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE   - unique ID used to identify the callback fn *
 *             entry_count     - the number of entries in the array         *
 *             entry_array     - array of ARP entries to be deleted         *
 *                                                                          *
 * Description: This function is used to delete IPv4 ARP entries from a FE. *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastToL2Del(
    FPPI_FEID                    feid,
    FPPI_CORRELATOR              correlator,
    FPPI_CBHANDLE                cbhandle,
    uint32_t                     entry_count,
    FPPI_IPv4UniNHResolEntry_t*  entry_array)
{
    return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
        FPPI_IPv4_UniDelToL2, entry_array, entry_count);
}

/****************************************************************************
 * Function: FPPAPI_ipv4_unicastToL2Flush()                                 *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID       - unique ID for a FE created by TPlugin      *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE   - unique ID used to identify the callback fn *
 *                                                                          *
 * Description: This function is used to delete all IPv4 ARP entries from   *
 *              an FE. *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastToL2Flush(
    FPPI_FEID        feid,
    FPPI_CORRELATOR  correlator,
    FPPI_CBHANDLE    cbhandle)
{
    return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
        FPPI_IPv4_UniFlushToL2, NULL, 0);
}

/****************************************************************************
 * Function: FPPAPI_ipv4_unicastRouteAdd()                                  *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             route_array - array of route entries to be added             *
 *             route_count - the number of entries in the array             *
 *                                                                          *
 * Description: This function is used to add IPv4 unicast route entries     *
 *              to a FE.                                                    *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

/*
FPPI_RET FPPAPI_ipv4_unicastRouteAdd(
    FPPI_FEID                feid,
    FPPI_CORRELATOR          correlator,
    FPPI_CBHANDLE            cbhandle,
    npf_ipv4_RouteEntry_t*   route_array,
    uint32_t                 route_count)
{
    return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
        FPPI_IPv4_AddRoute, route_array, route_count);
} */

/****************************************************************************
 * Function: FPPAPI_ipv4_unicastRouteDel()                                  *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             key_array - array of route keys to be deleted                *
 *             key_count - the number of keys in the array                  *
 *                                                                          *
 * Description: This function is used to delete IPv4 unicast route entries  *
 *              from a FE.                                                  *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/
/*
FPPI_RET FPPAPI_ipv4_unicastRouteDel(
    FPPI_FEID                feid,
    FPPI_CORRELATOR          correlator,
    FPPI_CBHANDLE            cbhandle,
    npf_ipv4_RouteKey_t*     key_array,
    uint32_t                 key_count)
{
    return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
        FPPI_IPv4_DelRoute, key_array, key_count);
}
*/
/****************************************************************************
 * Function: FPPAPI_ipv4_unicastRoutePurge()                                *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *                                                                          *
 * Description: This function is used to purge IPv4 unicast route entries   *
 *              from a FE.                                                  *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/
/*
FPPI_RET FPPAPI_ipv4_unicastRoutePurge(
    FPPI_FEID                feid,
    FPPI_CORRELATOR          correlator,
    FPPI_CBHANDLE            cbhandle)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_IPv4_PurgeRoute, NULL, 0);
}
*/
/****************************************************************************
 * Function: FPPAPI_ARP_AddEntry()                                          *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             entry_array - array of ARP entries to be added               *
 *             entry_count - the number of entries in the array             *
 *                                                                          *
 * Description: This function is used to add IPv4 ARP entries to a FE.      *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/
/*
FPPI_RET FPPAPI_ARP_AddEntry(
    FPPI_FEID                feid,
    FPPI_CORRELATOR          correlator,
    FPPI_CBHANDLE            cbhandle,
    npf_ipv4_ARPEntry_t*     entry_array,
    uint32_t                 entry_count)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_ARP_AddEntry, entry_array,
        entry_count);
}
*/
/****************************************************************************
 * Function: FPPAPI_ARP_DelEntry()                                          *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             key_array - array of ARP keys to be deleted                  *
 *             key_count - the number of keys in the array                  *
 *                                                                          *
 * Description: This function is used to delete IPv4 ARP entries from a FE. *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/
/*
FPPI_RET FPPAPI_ARP_DelEntry(
    FPPI_FEID                feid,
    FPPI_CORRELATOR          correlator,
    FPPI_CBHANDLE            cbhandle,
    npf_ipv4_ARPKey_t*       key_array,
    uint32_t                 key_count)
{
    return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
        FPPI_ARP_DelEntry, key_array, key_count);
}
*/
/****************************************************************************
 * Function: FPPAPI_ARP_QueryEntry()                                        *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             key_array - array of ARP keys to be queried                  *
 *             key_count - the number of keys in the array                  *
 *                                                                          *
 * Description: This function is used to query IPv4 ARP entries from a FE.  *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/
/*
FPPI_RET FPPAPI_ARP_QueryEntry(
        FPPI_FEID               feid,
        FPPI_CORRELATOR         correlator,
        FPPI_CBHANDLE           cbhandle,
        npf_ipv4_ARPKey_t       *key_array,
        uint32_t                key_count)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_ARP_QueryEntry, key_array,
        key_count);
}
*/
/****************************************************************************
 * Function: FPPAPI_ARP_PurgeEntry()                                        *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *                                                                          *
 * Description: This function is used to purge IPv4 ARP entries from a FE.  *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/
/*
FPPI_RET FPPAPI_ARP_PurgeEntry(
    FPPI_FEID         feid,
    FPPI_CORRELATOR   correlator,
    FPPI_CBHANDLE     cbhandle)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_ARP_PurgeEntry, NULL, 0);
}
*/
/****************************************************************************
 * Function: FPPAPI_SendPacket()                                            *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             packet_array - array of Packet structs to be sent to a FE    *
 *             packet_count - the number of packets in the array            *
 *                                                                          *
 * Description: This function is used to send a data packet to a FE.        *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_SendPacket(
        FPPI_FEID       feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE   cbhandle,
        npf_Packet_t    *packet_array,
        uint32_t        packet_count)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_PacketIO_Event, packet_array,
        packet_count);
}

/****************************************************************************
 * Function: FPPAPI_L2Interface_SetProperties()                             *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             port_array - array of <L2 Intf Attributes, Port ID> pairs    *
 *             port_count - the number of entries in the array              *
 *                                                                          *
 * Description: This function is used to set L2 Interface properties on     *
 *              a FE.                                                       *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_L2Interface_SetProperties(
    FPPI_FEID                feid,
    FPPI_CORRELATOR          correlator,
    FPPI_CBHANDLE            cbhandle,
    npf_L2IntfAttrs_t*       port_array,
    uint32_t                 port_count)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_L2_SetProperties, port_array,
        port_count);
}

/****************************************************************************
 * Function: FPPAPI_L2Interface_GetStatistics()                             *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             port_array - array of port IDs                               *
 *             port_count - the number of entries in the array              *
 *                                                                          *
 * Description: This function is used to get L2 Interface statistics from   *
 *              a FE.                                                       *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_L2Interface_GetStatistics(
    FPPI_FEID                feid,
    FPPI_CORRELATOR          correlator,
    FPPI_CBHANDLE            cbhandle,
    FPPI_PortID*             port_array,
    NPF_IfType_t            interface,
    uint32_t                 port_count)
{
    FPPI_Interface_GetStats_t *port_interface_info;
    FPPI_RET retVal;
    FPPI_PortID*             port;

    port = ( FPPI_PortID *) PIL_AllocateMemory(sizeof(FPPI_PortID)*port_count, PIL_MEM_ZERO_INIT);
    port_interface_info = (FPPI_Interface_GetStats_t  *) PIL_AllocateMemory(sizeof(FPPI_Interface_GetStats_t), PIL_MEM_ZERO_INIT);

   port_interface_info->port_array = port;  

   memcpy(port_interface_info->port_array, port_array, sizeof(FPPI_PortID)*port_count);

    port_interface_info->port_count = port_count;
    port_interface_info->ifType =  interface;
    
    retVal =  fppapi_MsgToFP(feid, 0, correlator, cbhandle, FPPI_L2_GetStatistics, port_interface_info,
        port_count);
    PIL_FreeMemory(port_interface_info);
    PIL_FreeMemory(port);
    return(retVal);
}

/****************************************************************************
 * Function: FPPAPI_L3Interface_SetProperties()                             *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             port_array - array of <L3 Intf Attributes, Port ID> pairs    *
 *             port_count - the number of entries in the array              *
 *                                                                          *
 * Description: This function is used to set L3 Interface properties on     *
 *              a FE.                                                       *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_L3Interface_SetProperties(
    FPPI_FEID          feid,
    FPPI_CORRELATOR    correlator,
    FPPI_CBHANDLE      cbhandle,
    npf_L3IntfAttrs_t* port_array,
    uint32_t           port_count)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_L3_SetProperties, port_array,
        port_count);
}

#ifdef _INCLUDE_IPV6

FPPI_RET FPPAPI_L3Interface_SetPropertiesIPv6(
    FPPI_FEID          feid,
    FPPI_CORRELATOR    correlator,
    FPPI_CBHANDLE      cbhandle,
    npf_L3IntfAttrs_t* port_array,
    uint32_t           port_count)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_L3_SetPropertiesIPv6, port_array,
        port_count);
}

#endif

/****************************************************************************
 * Function: FPPAPI_L3Interface_DelProperties()                             *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             port_array - array of <L3 Intf Attributes, Port ID> pairs    *
 *             port_count - the number of entries in the array              *
 *                                                                          *
 * Description: This function is used to delete L3 Interface properties on  *
 *              a FE.                                                       *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_L3Interface_DelProperties(
    FPPI_FEID          feid,
    FPPI_CORRELATOR    correlator,
    FPPI_CBHANDLE      cbhandle,
    npf_L3IntfAttrs_t* port_array,
    uint32_t           port_count)
{
    return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
        FPPI_L3_DelProperties, port_array, port_count);
}

#ifdef _INCLUDE_IPV6

FPPI_RET FPPAPI_L3Interface_DelPropertiesIPv6(
    FPPI_FEID          feid,
    FPPI_CORRELATOR    correlator,
    FPPI_CBHANDLE      cbhandle,
    npf_L3IntfAttrs_t* port_array,
    uint32_t           port_count)
{
    return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
        FPPI_L3_DelPropertiesIPv6, port_array, port_count);
}

#endif

/****************************************************************************
 * Function: FPPAPI_L3Interface_GetStatistics()                             *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             port_array - array of port IDs                               *
 *             port_count - the number of entries in the array              *
 *                                                                          *
 * Description: This function is used to get L3 Interface statistics from   *
 *              a FE.                                                       *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_L3Interface_GetStatistics(
    FPPI_FEID                feid,
    FPPI_CORRELATOR          correlator,
    FPPI_CBHANDLE            cbhandle,
    FPPI_PortID*             port_array,
    NPF_IfType_t            interface,
    uint32_t                 port_count)
{

    FPPI_Interface_GetStats_t *port_interface_info;
    FPPI_PortID*             port;
    FPPI_RET retVal; 

    port = (FPPI_PortID *) PIL_AllocateMemory(sizeof(FPPI_PortID)*port_count, PIL_MEM_ZERO_INIT);
    port_interface_info = (FPPI_Interface_GetStats_t  *) PIL_AllocateMemory(sizeof(FPPI_Interface_GetStats_t), PIL_MEM_ZERO_INIT);
    port_interface_info->port_array = port;

    memcpy(port_interface_info->port_array, port_array, sizeof(FPPI_PortID)*port_count);

   port_interface_info->port_count = port_count;
   port_interface_info->ifType =  interface;
    
    retVal =  fppapi_MsgToFP(feid, 0, correlator, cbhandle, FPPI_L3_GetStatistics, port_interface_info,
        port_count);
    PIL_FreeMemory(port_interface_info);
    PIL_FreeMemory(port);
    return(retVal);

}

/****************************************************************************
 * Function: FPPAPI_ICMP_GetStatistics()                                    *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             port_array - array of port IDs                               *
 *             port_count - the number of entries in the array              *
 *                                                                          *
 * Description: This function is used to get ICMP statistics from a FE.     *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ICMP_GetStatistics(
    FPPI_FEID                feid,
    FPPI_CORRELATOR          correlator,
    FPPI_CBHANDLE            cbhandle,
    FPPI_PortID*             port_array,
    uint32_t                 port_count)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_ICMP_GetStatistics, port_array,
        port_count);
}

#ifdef _INCLUDE_MPLS

/****************************************************************************
 * Function: FPPAPI_MPLS_Action()                                           *
 * --------------------------------                                         *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             FPPI_MPLS_Data_t - Payload given to FP from Control Plane.     *
 *             numEntry - the number of entries in the array                *
 *                                                                          *
 * Description: This function is used to operate for MPLS Elements in FP.   *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_MPLS_Action(
                        FPPI_FEID                feid,
                        FPPI_CORRELATOR          correlator,
                        FPPI_CBHANDLE            cbhandle,
                        uint32_t                 numEntry,
                        FPPI_MPLS_Data_t         *mplsData)
{
    return fppapi_MsgToFP( feid, 0, correlator, cbhandle, FPPI_CAT_MPLS,
                            mplsData, numEntry);
}

#endif /* MPLS */

#ifdef _INCLUDE_DIFFSERV

/****************************************************************************
 * Function: FPPAPI_QOS_DpeOperation()                                      *
 * --------------------------------                                         *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             FPPI_DS_Dpe_t - Payload given to FP from Control Plane.      *
 *                                                                          *
 * Description: This function is used to operate for DiffServ Elements in FP*
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/
FPPI_RET FPPAPI_QOS_DpeOperation(
    FPPI_FEID             feid,
    FPPI_CORRELATOR       correlator,
    FPPI_CBHANDLE         cbhandle,
    FPPI_DS_Dpe_t         *dpeInfo)
{
    return fppapi_MsgToFP(
        feid,
        0,
        correlator,
        cbhandle,
        FPPI_DS_DpeOperation,
        dpeInfo,
        1);
}

#endif

#ifdef _INCLUDE_IPV6TM

/****************************************************************************
 * Function: FPPAPI_IPv6TM_EnableAuto
 * -----------------------------------                                      *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_PortID - Identifies the interface which is the target   *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             IPA - Source IP address to be passed to FE                   *
 *                                                                          *
 * Description: Enable IPv6 automatic tunneling.                            *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_IPv6TM_EnableAuto(
    FPPI_FEID              feid,
    FPPI_PortID            port_id,
    FPPI_CORRELATOR        correlator,
    FPPI_CBHANDLE          cbhandle)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_IPv6TM_EnableAuto, &port_id,
        1);
}

/****************************************************************************
 * Function: FPPAPI_IPv6TM_DisableAuto
 * -----------------------------------                                      *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_PortID - Identifies the interface which is the target   *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *                                                                          *
 * Description: Disable IPv6 automatic tunneling.                           *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_IPv6TM_DisableAuto(
    FPPI_FEID              feid,
    FPPI_PortID            port_id,
    FPPI_CORRELATOR        correlator,
    FPPI_CBHANDLE          cbhandle)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_IPv6TM_DisableAuto, &port_id,
        1);
}

/****************************************************************************
 * Function: FPPAPI_IPv6TM_EnableConfig
 * -----------------------------------                                      *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_PortID - Identifies the interface which is the target   *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             IPA - Source IP address to be passed to FE                   *
 *                                                                          *
 * Description: Enable IPv6 configured tunneling.                           *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_IPv6TM_EnableConfig(
    FPPI_FEID              feid,
    FPPI_PortID            port_id,
    FPPI_CORRELATOR        correlator,
    FPPI_CBHANDLE          cbhandle,
    IPA                    ipv4Addr)
{
    FPPI_RET retVal = FPPI_SUCCESS;
    npf_ipv6tm_Tunnel_t* tunnel;

    if ((tunnel = (npf_ipv6tm_Tunnel_t*) PIL_AllocateMemory
        (sizeof(npf_ipv6tm_Tunnel_t), PIL_MEM_ZERO_INIT)) == NULL)
    {
        retVal = FPPI_OUT_OF_MEMORY;
        npf_logger_write(LOG_DRS, LOG_ERR, "Could not allocate memory.\n");
        goto DRS_LBL_1;
    }

    tunnel->portid = port_id;
    tunnel->destip = ipv4Addr;

    retVal = fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_IPv6TM_EnableConfig, tunnel,
        1);

/*    PIL_FreeMemory(tunnel); */

DRS_LBL_1:

    return retVal;
}

/****************************************************************************
 * Function: FPPAPI_IPv6TM_DisableConfig
 * -----------------------------------                                      *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_PortID - Identifies the interface which is the target   *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *                                                                          *
 * Description: Disable IPv6 configured tunneling.                          *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_IPv6TM_DisableConfig(
    FPPI_FEID              feid,
    FPPI_PortID            port_id,
    FPPI_CORRELATOR        correlator,
    FPPI_CBHANDLE          cbhandle)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_IPv6TM_DisableConfig, &port_id,
        1);
}

/****************************************************************************
 * Function: FPPAPI_IPv6TM_Enable6to4
 * -----------------------------------                                      *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_PortID - Identifies the interface which is the target   *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             IPA - Source IP address to be passed to FE                   *
 *                                                                          *
 * Description: Enable IPv6 to IPv4 tunneling.                              *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_IPv6TM_Enable6to4(
    FPPI_FEID              feid,
    FPPI_PortID            port_id,
    FPPI_CORRELATOR        correlator,
    FPPI_CBHANDLE          cbhandle)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_IPv6TM_Enable6to4, &port_id,
        1);
}

/****************************************************************************
 * Function: FPPAPI_IPv6TM_Disable6to4
 * -----------------------------------                                      *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_PortID - Identifies the interface which is the target   *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *                                                                          *
 * Description: Disable IPv6 to IPv4 tunneling.                             *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_IPv6TM_Disable6to4(
    FPPI_FEID              feid,
    FPPI_PortID            port_id,
    FPPI_CORRELATOR        correlator,
    FPPI_CBHANDLE          cbhandle)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_IPv6TM_Disable6to4, &port_id,
        1);
}

#endif

#ifdef _INCLUDE_IPV6

/****************************************************************************
 * Function: FPPAPI_IPv6NextHopEntryAdd                                     *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             correlator   - used to distinguish multiple invocations      *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *             numIndices   - the number of entries in the array            *
 *             indicesArray - the number of entries in the array            *
 *                                                                          *
 * Description: Add one or more IPv6 next hop entries.                      *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_IPv6NextHopEntryAdd(
    FPPI_FEID                    feid,
    FPPI_CORRELATOR              correlator,
    FPPI_CBHANDLE                cbhandle,
    FPPI_IPv6_UnicastNextHopUpdateInfo_t  *indicesArray)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_IPv6_AddNextHop,
        indicesArray, 1);
}

/****************************************************************************
 * Function: FPPAPI_IPv6NextHopEntryDelete                                  *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             correlator   - used to distinguish multiple invocations      *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *             numIndices   - the number of entries in the array            *
 *             indicesArray - the number of entries in the array            *
 *                                                                          *
 * Description: Delete one or more IPv6 next hop entries.                   *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_IPv6NextHopEntryDelete(
    FPPI_FEID                    feid,
    FPPI_CORRELATOR              correlator,
    FPPI_CBHANDLE                cbhandle,
    NPF_IPv6NextHopEntryArray_t  *indicesArray)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_IPv6_DelNextHop,
        indicesArray, 1);
}

/****************************************************************************
 * Function: FPPAPI_IPv6NextHopTableFlush                                   *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             correlator   - used to distinguish multiple invocations      *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *                                                                          *
 * Description: Delete all IPv6 next hop entries in FE.                     *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_IPv6NextHopTableFlush(
    FPPI_FEID                    feid,
    FPPI_CORRELATOR              correlator,
    FPPI_CBHANDLE                cbhandle)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_IPv6_FlushNextHop, 0, 0);
}

/****************************************************************************
 * Function: FPPAPI_IPv6NextHopTableAttributeQuery                          *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             correlator   - used to distinguish multiple invocations      *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *                                                                          *
 * Description: Query all IPv6 next hop entries in FE.                      *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_IPv6NextHopTableAttributeQuery(
    FPPI_FEID                    feid,
    FPPI_CORRELATOR              correlator,
    FPPI_CBHANDLE                cbhandle)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_IPv6_QueryNextHop, 0, 0);
}

/****************************************************************************
 * Function: FPPAPI_IPv6PrefixEntryAdd                                      *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             correlator   - used to distinguish multiple invocations      *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *             numEntries   - the number of entries in the array            *
 *             entryArray   - the number of entries in the array            *
 *                                                                          *
 * Description: Add one or more IPv6 prefixes.                              *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_IPv6PrefixEntryAdd(
    FPPI_FEID              feid,
    FPPI_CORRELATOR        correlator,
    FPPI_CBHANDLE          cbhandle,
    NPF_uint32_t           numEntries,
    NPF_IPv6PrefixEntry_t  *entryArray)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_IPv6_AddPrefix,
        entryArray, numEntries);
}

/****************************************************************************
 * Function: FPPAPI_IPv6PrefixEntryDelete                                   *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             correlator   - used to distinguish multiple invocations      *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *             numEntries   - the number of entries in the array            *
 *             entryArray   - the number of entries in the array            *
 *                                                                          *
 * Description: Delete one or more IPv6 prefixes.                           *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_IPv6PrefixEntryDelete(
    FPPI_FEID              feid,
    FPPI_CORRELATOR        correlator,
    FPPI_CBHANDLE          cbhandle,
    NPF_uint32_t           numEntries,
    NPF_IPv6PrefixEntry_t  *entryArray)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_IPv6_DelPrefix,
        entryArray, numEntries);
}

/****************************************************************************
 * Function: FPPAPI_IPv6PrefixTableFlush                                    *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             correlator   - used to distinguish multiple invocations      *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *                                                                          *
 * Description: Delete all IPv6 prefixes from FE.                           *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_IPv6PrefixTableFlush(
    FPPI_FEID              feid,
    FPPI_CORRELATOR        correlator,
    FPPI_CBHANDLE          cbhandle)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_IPv6_FlushPrefix, 0, 0);
}           

/****************************************************************************
 * Function: FPPAPI_IPv6PrefixTableAttributeQuery                           *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             correlator   - used to distinguish multiple invocations      *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *                                                                          *
 * Description: Query all IPv6 prefixes from FE.                            *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_IPv6PrefixTableAttributeQuery(
    FPPI_FEID              feid,
    FPPI_CORRELATOR        correlator,
    FPPI_CBHANDLE          cbhandle)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_IPv6_QueryPrefix, 0, 0);
}           

/****************************************************************************
 * Function: FPPAPI_IPv6_L2EntryAdd                                         *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             correlator   - used to distinguish multiple invocations      *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *             entry        - list of L2 entries                            *
 *                                                                          *
 * Description: Add a layer 2 address.                                      *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_IPv6_L2EntryAdd(
    FPPI_FEID              feid,
    FPPI_CORRELATOR        correlator,
    FPPI_CBHANDLE          cbhandle,
    NPF_IPv6NbrEntry_t     *entry)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_IPv6_AddToL2, entry, 1);
}

/****************************************************************************
 * Function: FPPAPI_IPv6_L2EntryDelete                                      *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             correlator   - used to distinguish multiple invocations      *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *             entry        - list of L2 entries                            *
 *                                                                          *
 * Description: Delete a layer 2 address.                                   *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_IPv6_L2EntryDelete(
    FPPI_FEID              feid,
    FPPI_CORRELATOR        correlator,
    FPPI_CBHANDLE          cbhandle,
    NPF_IPv6NbrEntry_t     *entry)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_IPv6_DelToL2, entry, 1);
}

#endif

#ifdef _INCLUDE_GTP

/****************************************************************************
 * Function: FPPAPI_gtp_rx_setup                                            *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *             correlator   - used to distinguish multiple invocations      *
 *             teid         - tunnel ID                                     *
 *             config       - configuration parameter for tunnel            *
 *                                                                          *
 * Description: Populate a GTP table in an FE.                              *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/
FPPI_RET FPPAPI_gtp_rx_setup(
    FPPI_FEID         feid,
    FPPI_CBHANDLE     cbhandle,
    FPPI_CORRELATOR   correlator,
    uint32_t          teid,
    npf_gtp_Config_t  *config)
{
    FPPI_RET retVal;
    npf_gtp_RxSetupInfo_t* pol;
    uint32_t dataSz;

    dataSz = config != NULL ? sizeof(npf_gtp_Config_t) : 0;

    if ((pol = (npf_gtp_RxSetupInfo_t*) PIL_AllocateMemory
        (sizeof(npf_gtp_RxSetupInfo_t) + dataSz, PIL_MEM_ZERO_INIT)) ==
        NULL)
    {
        retVal = FPPI_OUT_OF_MEMORY;
        npf_logger_write(LOG_DRS, LOG_ERR, "Could not allocate memory.\n");
    }
    else {
        pol->teid   = teid;
        pol->config = NULL;

        if (config != NULL)
        {
            pol->config = (npf_gtp_Config_t*)(pol + 1);
            memcpy(pol->config, config, dataSz);
        }
/*
        retVal = fppapi_MsgToFP(
            feid, 0, correlator, cbhandle, FPPI_GTP_RX_Setup, pol, 1);
*/  }

    return retVal;
}

/****************************************************************************
 * Function: FPPAPI_gtp_rx_release                                          *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *             correlator   - used to distinguish multiple invocations      *
 *             teid         - tunnel ID                                     *
 *                                                                          *
 * Description: Delete an etnry in the FE GTP table.                        *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_rx_release(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t        teid)
{/*
    return fppapi_MsgToFP(
    feid, 0, correlator, cbhandle, FPPI_GTP_RX_Release, &teid, 1);*/
        }

/****************************************************************************
 * Function: FPPAPI_gtp_rx_getTunnelInfo                                    *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *             correlator   - used to distinguish multiple invocations      *
 *             teid         - tunnel ID                                     *
 *                                                                          *
 * Description: Get info from an etnry in the FE GTP table.                 *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_rx_getTunnelInfo(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t        teid)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_GTP_RX_GetTunnelInfo, &teid, 1);
}

/* Uplink */
/****************************************************************************
 * Function: FPPAPI_gtp_tx_setup                                            *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *             correlator   - used to distinguish multiple invocations      *
 *             teid         - tunnel ID                                     *
 *             config       - configuration parameter for tunnel            *
 *             Iu_Association - relevant associations                       *
 *                                                                          *
 * Description: Populate a transmit entry in the FE GTP table.              *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_tx_setup(
    FPPI_FEID           feid,
    FPPI_CBHANDLE       cbhandle,
    FPPI_CORRELATOR     correlator,
    uint32_t            teid,
    npf_gtp_Config_t    *config,
    npf_gtp_IuTunnel_t  *Iu_Association)
{
    FPPI_RET retVal;
    npf_gtp_TxSetupInfo_t* pol;
    uint32_t dataSz1;
    uint32_t dataSz2;
    char* ptr;

    dataSz1 = config != NULL ? sizeof(npf_gtp_Config_t) : 0;
    dataSz2 = Iu_Association != NULL ? sizeof(npf_gtp_IuTunnel_t) : 0;

    if ((pol = (npf_gtp_TxSetupInfo_t*) PIL_AllocateMemory
        (sizeof(npf_gtp_TxSetupInfo_t) + dataSz1 + dataSz2,
        PIL_MEM_ZERO_INIT)) == NULL)
    {
        retVal = FPPI_OUT_OF_MEMORY;
        npf_logger_write(LOG_DRS, LOG_ERR, "Could not allocate memory.\n");
    }
    else {
        pol->teid           = teid;
        pol->config         = NULL;
        pol->Iu_Association = NULL;
        ptr                 = pol + 1;

        if (config != NULL)
        {
            pol->config = (npf_gtp_Config_t*)(pol + 1);
            memcpy(pol->config, config, dataSz1);
            ptr = pol->config;
        }
        if (Iu_Association != NULL)
        {
            pol->Iu_Association = (npf_gtp_IuTunnel_t*)(ptr + dataSz1);
            memcpy(pol->Iu_Association, Iu_Association, dataSz2);
        }

        retVal = fppapi_MsgToFP(
            feid, 0, correlator, cbhandle, FPPI_GTP_TX_Setup, pol, 1);
    }

    return retVal;
}

/****************************************************************************
 * Function: FPPAPI_gtp_tx_release                                          *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *             correlator   - used to distinguish multiple invocations      *
 *             teid         - tunnel ID                                     *
 *                                                                          *
 * Description: Delete a entry in the FE GTP table.                         *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_tx_release(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t        teid)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_GTP_TX_Release, &teid, 1);
}

/****************************************************************************
 * Function: FPPAPI_gtp_tx_getTunnelInfo                                    *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *             correlator   - used to distinguish multiple invocations      *
 *             teid         - tunnel ID                                     *
 *                                                                          *
 * Description: Get info of an entry in the FE GTP table.                   *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_tx_getTunnelInfo(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t       teid)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_GTP_TX_GetTunnelInfo, &teid, 1);
}

/* Source RNC */
/****************************************************************************
 * Function: FPPAPI_gtp_srnc_relocationCmd                                  *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *             correlator   - used to distinguish multiple invocations      *
 *             teid         - tunnel ID                                     *
 *             Iu_Association - relevant associations                       *
 *                                                                          *
 * Description:                                                             *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_srnc_relocationCmd(
    FPPI_FEID           feid,
    FPPI_CBHANDLE       cbhandle,
    FPPI_CORRELATOR     correlator,
    uint32_t            teid,
    npf_gtp_IuTunnel_t  *Iu_Association)
{
    FPPI_RET retVal;
    npf_gtp_SrncReloCmdInfo_t* pol;
    uint32_t dataSz1;

    dataSz1 = Iu_Association != NULL ? sizeof(npf_gtp_IuTunnel_t) : 0;

    if ((pol = (npf_gtp_SrncReloCmdInfo_t*) PIL_AllocateMemory
        (sizeof(npf_gtp_SrncReloCmdInfo_t) + dataSz1,
        PIL_MEM_ZERO_INIT)) == NULL)
    {
        retVal = FPPI_OUT_OF_MEMORY;
        npf_logger_write(LOG_DRS, LOG_ERR, "Could not allocate memory.\n");
    }
    else {
        pol->teid           = teid;
        pol->Iu_Association = NULL;

        if (Iu_Association != NULL)
        {
            pol->Iu_Association = (npf_gtp_IuTunnel_t*)(pol + 1);
            memcpy(pol->Iu_Association, Iu_Association, dataSz1);
        }

        retVal = fppapi_MsgToFP(
            feid, 0, correlator, cbhandle, FPPI_GTP_SRNC_RelocationCmd,
            pol, 1);
    }

    return retVal;
}

/****************************************************************************
 * Function: FPPAPI_gtp_srnc_suspendRAB                                     *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *             correlator   - used to distinguish multiple invocations      *
 *             teid         - tunnel ID                                     *
 *                                                                          *
 * Description:                                                             *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_srnc_suspendRAB(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t        teid)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_GTP_SRNC_SuspendRAB, &teid, 1);
}

/****************************************************************************
 * Function: FPPAPI_gtp_srnc_relocationComplete                             *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *             correlator   - used to distinguish multiple invocations      *
 *             teid         - tunnel ID                                     *
 *                                                                          *
 * Description:                                                             *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_srnc_relocationComplete(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t        teid)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_GTP_SRNC_RelocationComp,
        &teid, 1);
}

/* Target RNC */
/****************************************************************************
 * Function: FPPAPI_gtp_trnc_relocationCmd                                  *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *             correlator   - used to distinguish multiple invocations      *
 *             teid         - tunnel ID                                     *
 *             config       - configuration parameter for tunnel            *
 *                                                                          *
 * Description:                                                             *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_trnc_relocationCmd(
    FPPI_FEID        feid,
    FPPI_CBHANDLE    cbhandle,
    FPPI_CORRELATOR  correlator,
    uint32_t         teid,
    npf_gtp_Config_t *config)
{
    FPPI_RET retVal;
    npf_gtp_RxSetupInfo_t* pol;
    uint32_t dataSz;

    dataSz = config != NULL ? sizeof(npf_gtp_Config_t) : 0;

    if ((pol = (npf_gtp_RxSetupInfo_t*) PIL_AllocateMemory
        (sizeof(npf_gtp_RxSetupInfo_t) + dataSz, PIL_MEM_ZERO_INIT)) ==
        NULL)
    {
        retVal = FPPI_OUT_OF_MEMORY;
        npf_logger_write(LOG_DRS, LOG_ERR, "Could not allocate memory.\n");
    }
    else {
        pol->teid   = teid;
        pol->config = NULL;

        if (config != NULL)
        {
            pol->config = (npf_gtp_Config_t*)(pol + 1);
            memcpy(pol->config, config, dataSz);
        }

        retVal = fppapi_MsgToFP(
            feid, 0, correlator, cbhandle, FPPI_GTP_TRNC_RelocationCmd,
            pol, 1);
    }

    return retVal;
}

/****************************************************************************
 * Function: FPPAPI_gtp_trnc_relocationComplete                             *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *             correlator   - used to distinguish multiple invocations      *
 *             teid         - tunnel ID                                     *
 *             PDCP_seq_num - sequence number                               *
 *                                                                          *
 * Description:                                                             *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_trnc_relocationComplete(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t        teid,
    uint32_t        PDCP_seq_num)
{
    FPPI_RET retVal;
    npf_gtp_TrncReloCompInfo_t* pol;

    if ((pol = (npf_gtp_TrncReloCompInfo_t*) PIL_AllocateMemory
        (sizeof(npf_gtp_TrncReloCompInfo_t), PIL_MEM_ZERO_INIT)) == NULL)
    {
        retVal = FPPI_OUT_OF_MEMORY;
        npf_logger_write(LOG_DRS, LOG_ERR, "Could not allocate memory.\n");
    }
    else {
        pol->teid           = teid;
        pol->PDCP_seq_num   = PDCP_seq_num;

        retVal = fppapi_MsgToFP(
            feid, 0, correlator, cbhandle, FPPI_GTP_TRNC_RelocationComp,
            pol, 1);
    }

    return retVal;
}

/* intersystem handover */
/****************************************************************************
 * Function: FPPAPI_gtp_dataForwardCmd                                      *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *             correlator   - used to distinguish multiple invocations      *
 *             teid         - tunnel ID                                     *
 *                                                                          *
 * Description:                                                             *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_dataForwardCmd(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t        teid)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_GTP_DataForwardCmd, &teid, 1);
}

/****************************************************************************
 * Function: FPPAPI_gtp_suspendRAB                                          *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *             correlator   - used to distinguish multiple invocations      *
 *             teid         - tunnel ID                                     *
 *                                                                          *
 * Description:                                                             *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_suspendRAB(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t        teid)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_GTP_SuspendRAB, &teid, 1);
}

/* Others */
/****************************************************************************
 * Function: FPPAPI_gtp_setBufferSize                                       *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *             correlator   - used to distinguish multiple invocations      *
 *             buffer_size  - size of buffer to set to                      *
 *                                                                          *
 * Description: Set the GTP buffer size on FE.                              *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_setBufferSize(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t        buffer_size)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_GTP_SetBufferSize,
        &buffer_size, 1);
}

/****************************************************************************
 * Function: FPPAPI_gtp_getPDCPSeqNumber                                    *
 * -----------------------------------                                      *
 * Parameters: feid         - unique ID for a FE created by TPlugin         *
 *             cbhandle     - unique ID used to identify the callback fn    *
 *             correlator   - used to distinguish multiple invocations      *
 *             teid         - tunnel ID                                     *
 *                                                                          *
 * Description: Retrieve the PDCP sequence number                           *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_getPDCPSeqNumber(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t         teid)
{
    return fppapi_MsgToFP(
        feid, 0, correlator, cbhandle, FPPI_GTP_GetPDCPSeqNumber,
        &teid, 1);
}

#endif /* GTP */

#ifdef _INCLUDE_ATM
/************************ATM Functions *********************/

/********************************************************************
 * Function:   FPPAPI_ATM_IfAttrSetFunc()
 *
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of ATM interface attributes to be set 
 * count - the number of entries in the array
 *
 * Description: This function is used to set an ATM interface attributes
 *					 on an FE
 *
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_IfAttrSetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_IfAtmAttr_t *info,
        NPF_uint32_t count)
{

        return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
                          FPPI_ATM_IfAttrSet,
                         info, count);

}

/********************************************************************
 * Function:   FPPAPI_ATM_IfDeleteFunc()
 *
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of ATM interfaces to be deleted 
 * count - the number of entries in the array
 *
 * Description: This function is used to delete an ATM interface on an FE 
 *
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_IfDeleteFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        FPPI_PortID *info,
        NPF_uint32_t count)
{

        return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
                          FPPI_ATM_IfDelete,
                         info, count);

}


/********************************************************************
 * Function:   FPPAPI_ATM_IfEnableFunc()
 *
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of ATM interfaces to be enabled 
 * count - the number of entries in the array
 *
 * Description: This function is used to enable an ATM interface on an FE
 *
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_IfEnableFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        FPPI_PortID *info,
        NPF_uint32_t count)
{

        return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
                          FPPI_ATM_IfEnable,
                         info, count);

}


/********************************************************************
 * Function:   FPPAPI_ATM_IfDisableFunc()
 *
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of ATM interfaces to be enabled 
 * count - the number of entries in the array
 *
 * Description: This function is used to disable an ATM interface on an FE
 *
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_IfDisableFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        FPPI_PortID *info,
        NPF_uint32_t count)
{

        return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
                          FPPI_ATM_IfDisable,
                         info, count);

}


/********************************************************************
 * Function:   FPPAPI_ATM_IfGenericStatsGetFunc()
 *
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of ATM interfaces to be enabled 
 * count - the number of entries in the array
 *
 * Description: This function is used to get statistics of an ATM 
 *					interface on an FE
 *
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_IfGenericStatsGetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        FPPI_PortID * info,
        NPF_uint32_t count)
{

        return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
                          FPPI_ATM_IfGenStatsGet,
                         info, count);

}



/********************************************************************
 * Function:   FPPAPI_ATM_VccSetFunc()

 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of connection information to be setup
 * count - the number of entries in the array
 *
 * Description: This function is used to add an ATM connection to a FE
 *
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_VccSetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_IfVcc_t * info,
        NPF_uint32_t count)
{

        return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
                          FPPI_ATM_VccSet,
                         info, count);

}

/********************************************************************
 * Function:   FPPAPI_ATM_VccBindFunc()

 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of connection information to be setup
 * count - the number of entries in the array
 *
 * Description: This function is used to Bind an ATM connection 
 *
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_VccBindFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_VccBind_t * info,
        NPF_uint32_t count)
{

        return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
                          FPPI_ATM_VccBind,
                         info, count);

}

/********************************************************************
 * Function:    FPPAPI_ATM_VccDeleteFunc()
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of connection information to be deleted
 * count - the number of entries in the array
 *
 * Description: This function is used to Delete an ATM connection to a FE
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_VccDeleteFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_VccAddr_t *info,
        NPF_uint32_t count)
{
        return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
                    FPPI_ATM_VccDelete,
                    info, count);
}


/********************************************************************
 * Function:    FPPAPI_ATM_VccEnableFunc()
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of connection information to be deleted
 * count - the number of entries in the array
 *
 * Description: This function is used to Enable an ATM connection to a FE
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_VccEnableFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_VccAddr_t *info,
        NPF_uint32_t count)
{
        return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
                    FPPI_ATM_VccEnable,
                    info, count);

}


/********************************************************************
 * Function:    FPPAPI_ATM_VccDisableFunc()
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of connection information to be deleted
 * count - the number of entries in the array
 *
 * Description: This function is used to Disable an ATM connection to a FE
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_VccDisableFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_VccAddr_t *info,
        NPF_uint32_t count)
{
        return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
                    FPPI_ATM_VccDisable,
                    info, count);

}

/********************************************************************
 * Function:    FPPAPI_ATM_VccOperStatusGetFunc()
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of connection information to be deleted
 * count - the number of entries in the array
 *
 * Description: This function is used to Get the operational status
 *              of an ATM connection to a FE
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_VccOperStatusGetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_VccAddr_t *info,
        NPF_uint32_t count)
{
        return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
                    FPPI_ATM_VccOperStatusGet,
                    info, count);

}


/********************************************************************
 * Function:    FPPAPI_ATM_VccStatsGetFunc()
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of connection information to be deleted
 * count - the number of entries in the array
 *
 * Description: This function is used to Get the Statistics 
 *              of an ATM connection to a FE
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_VccStatsGetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_VccAddr_t *info,
        NPF_uint32_t count)
{
        return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
                    FPPI_ATM_VccStatsGet,
                    info, count);

}




/********************************************************************
 * Function: FPPAPI_ATM_AAL2_ChannelSetFunc()
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * connection - connection information; VPI, VCI and LinkId
 * info - array of connection information to be deleted
 * count - the number of entries
 * Description: This function is used to set the aal2 channels
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/

FPPI_RET FPPAPI_ATM_AAL2_ChannelSetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_AAL2_ChannelInfo_t *info,
        NPF_uint32_t count)
{

        return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
                              FPPI_ATM_AAL2_ChannelSet,
                              info, count);
}
/********************************************************************
 * Function: FPPAPI_ATM_AAL2_ChannelDeleteFunc()
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * connection - connection information; VPI, VCI and LinkId
 * info - array of connection information to be deleted
 * count - the number of entries
 * Description: This function is used to delete the AAL2 channels
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/

FPPI_RET FPPAPI_ATM_AAL2_ChannelDeleteFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_AAL2_ConnId_t *info,
        NPF_uint32_t count)
{

        return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
                                FPPI_ATM_AAL2_ChannelDel,
                                info, count);
}

  
/********************************************************************
 * Function:    FPPAPI_ATM_ConnectionXcSetFunc()
 * ---------------------------
 * Parameters:
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of ATM connection cross connect information
 * count - number of elements present in the info strcuture
 * Description: This function is used to set the ATM cross
 *              Connections
 * Return:  FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/

FPPI_RET FPPAPI_ATM_ConnectionXcSetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        NPF_ATM_ConnectionXc_t * info,
        NPF_uint32_t count)
{
        return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
                        FPPI_ATM_ConnectionXcSet,
                       info, count);

}

/********************************************************************
 * Function:    FPPAPI_ATM_ConnectionXcDeleteFunc()
 * ---------------------------
 * Parameters:
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of ATM connection cross connect information
 * count - number of elements present in the info strcuture
 * Description: This function is used to delete the ATM cross
 *              Connections
 * Return:  FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/

FPPI_RET FPPAPI_ATM_ConnectionXcDeleteFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        NPF_ATM_ConnectionXc_t * info,
        NPF_uint32_t count)
{
        return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
                    FPPI_ATM_ConnectionXcDel,
                    info, count);

}


/********************************************************************
 * Function:     FPPAPI_ATM_OAM_CP_SetFunc()
 * ---------------------------
 * Parameters:
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - ATM OAM connection type
 *
 * Description: This function is used to set the ATM OAM connection
 *              point type
 * Return:  FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/


FPPI_RET FPPAPI_ATM_OAM_CP_SetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_OAM_CP_t * info,
        NPF_uint32_t count)

{
        return fppapi_MsgToFP(feid,0, correlator, cbhandle,
                        FPPI_ATM_OAM_CP_Set,
                        info, count);

}




/********************************************************************
 * Function:   FPPAPI_ATM_OAM_CC_SetFunc()
 * ---------------------------
 * Parameters:
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - ATM OAM contuinity check Parameters
 *
 * Description: This function is used to set the ATM OAM continuity
 *              check 
 * Return:  FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/

FPPI_RET FPPAPI_ATM_OAM_CC_SetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_OAM_CC_t * info,
        NPF_uint32_t count)
  
{
        return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
                        FPPI_ATM_OAM_CC_Set,
                        info, count);

}

/********************************************************************
 * Function:   FPPAPI_ATM_OAM_LB_SetFunc()
 * ---------------------------
 * Parameters:
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - ATM OAM loopBack Parameters
 *
 * Description: This function is used to set the ATM OAM loopback
 *              type
 * Return:  FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/

FPPI_RET FPPAPI_ATM_OAM_LB_SetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_OAM_LB_t * info,
        NPF_uint32_t count)
  
{
        return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
                        FPPI_ATM_OAM_LB_Set,
                        info, count);

}

/********************************************************************
 * Function:   FPPAPI_ATM_OAM_MonitorFunc()
 * ---------------------------
 * Parameters:
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - ATM OAM cell Monitor Parameters
 *
 * Description: This function is used to set the ATM OAM cells  
 *              monitoring function 
 * Return:  FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/

FPPI_RET FPPAPI_ATM_OAM_MonitorFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_OAM_Mon_t * info,
        NPF_uint32_t count)
  
{
        return fppapi_MsgToFP(feid, 0, correlator, cbhandle,
                        FPPI_ATM_OAM_Monitor,
                        info, count);

}
/********************************************************************
 * Function:   FPPAPI_ATM_OAM_AlarmSetFunc()
 * ---------------------------
 * Parameters:
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - ATM OAM Alarm Parameters
 *
 * Description: This function is used to set the ATM OAM Alarm type
 * Return:  FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_OAM_AlarmSetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_OAM_Alarm_t * info,
        uint32_t count)
  
{
        return fppapi_MsgToFP(feid,0, correlator, cbhandle,
                        FPPI_ATM_OAM_AlarmSet,
                        info, count);

}


/********************************************************************
 * Function:   FPPAPI_ATM_OAM_AlarmClearFunc()
 * ---------------------------
 * Parameters:
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - ATM OAM Alarm Parameters
 *
 * Description: This function is used to clear the ATM OAM Alarm type
 * Return:  FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_OAM_AlarmClearFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_OAM_Alarm_t * info,
        NPF_uint32_t count)
  
{
        return fppapi_MsgToFP(feid,0, correlator, cbhandle,
                        FPPI_ATM_OAM_AlarmClear,
                        info, count);

}


/********************************************************************
 * Function:   FPPAPI_ATM_OAM_PM_SetFunc()
 * ---------------------------
 * Parameters:
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - ATM OAM performance monitoring Parameters
 *
 * Description: This function is used to set the ATM OAM PM properties
 * Return:  FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/

FPPI_RET FPPAPI_ATM_OAM_PM_SetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_OAM_PM_t * info,
        NPF_uint32_t count)

{
        return fppapi_MsgToFP(feid,0, correlator, cbhandle,
                        FPPI_ATM_OAM_PM_Set,
                        info, count);
}

#endif /* _INCLUDE_ATM */
