/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/DataRelayShim/src/packinfo.c,v $
 * $Revision: 1.6 $
 * $Date: 2003/11/17 16:57:15 $
 *
 * Description:
 *     Helper methods to copy CP data into local memory and pack into
 *     objects expected by FP.
 *
 ****************************************************************************/

#include "string.h"
#include "log.h"
#include "backendapi_types.h"
#include "fpp_common.h"

#ifdef _INCLUDE_DIFFSERV
#include "npf_ds_types.h"
#endif

#ifdef _INCLUDE_MPLS
#include "fppapi_mpls_types.h"
#endif

#include "drspriv.h"
#include "pdk_callback_types.h"
#include "fppCapabilityTypes.h"

#ifdef _INCLUDE_ATM
#include "pdk_cm_types.h"
#include "npf_atm_types.h"
#include "fppapi_atm_types.h"
#endif /* _INCLUDE_ATM */


/********************************* Globals *********************************/
/* The critical section and BEND ready flags are defined in the Backend
   files. */
extern PilCriticalSection g_beFlagslock;
extern int32_t Bend_Ready;

/****************************************************************************
 * Function: CopyAndPack()                                                  *
 * ---------------------------                                              *
 * Parameters: cbtype      - enum that defines the CB type                  *
 *             portid      - port identifier                                *
 *             dir         - data path direction (ingress, egress, both)    *
 *             tableid     - FE table identifier                            *
 *             instid      - FE table entry instance identifier             *
 *             nexttableid - FE table identifier for next stage             *
 *             nextinstid  - next stage FE table entry instance identifier  *
 *             policy_count- number of policies in list                     *
 *             policy_list - pointer to QoS policy list to encapsulate      *
 *             outObj      - pointer to packed data object                  *
 *                                                                          *
 * Description: Copy the CP data into local memory and pack into structs    *
 *              that the FP expects.                                        *
 *                                                                          *
 * Return: FPPI_SUCCESS if no error otherwise an error code.                *
 *                                                                          *
 ****************************************************************************/

FPPI_RET CopyAndPack(
    NPF_CBCAT             cbtype,
    FPPI_PortID           portid,
    FPPI_NHID             nhid,
    uint32_t    dir,
    uint32_t      tableid,
    uint32_t   instid,
    uint32_t      nexttableid,
    uint32_t   nextinstid,
    uint32_t              policy_count,
    void*                 policy_list,
    void**                outObj)
{
    FPPI_RET retVal = FPPI_SUCCESS;
    uint32_t dataSz;
    uint32_t dataSz2;
    uint32_t i;
   /* type structures */

#ifdef _INCLUDE_ATM

	 IfATM_IfAtmAttr_t *				  atm_attr;
	 FPPI_PortID *						  portId;
    IfATM_IfVcc_t *                conn_info;

    IfATM_VccAddr_t *              vcc_info;
	 IfATM_VccBind_t *				  vcc_bind;

    IfATM_AAL2_ChannelInfo_t *     aal2_info;
	 IfATM_AAL2_ConnId_t *          aal2_chan_id_info;
    NPF_ATM_ConnectionXc_t *       conn_xc_info;

    IfATM_OAM_CP_t *               atm_oam_cp_info;
    IfATM_OAM_CC_t *               atm_oam_cc_info;
    IfATM_OAM_LB_t *               atm_oam_lb_info;
    IfATM_OAM_Mon_t *              atm_oam_monitor_info;
    IfATM_OAM_Alarm_t *            atm_oam_alarm_info;
    IfATM_OAM_PM_t *               atm_oam_pm_info;

#endif /* _INCLUDE_ATM */



    npf_logger_write(LOG_DRS, LOG_TRACE,
        "Entered method.\n");

    switch (cbtype)
    {
    case FPPI_FEBind_Event:
    case FPPI_CEUnBind_Event:
        {
        FPPI_Status* outStatus;

        /* Allocate memory for ojbect to send to FP */
        outStatus = (FPPI_Status*)PIL_AllocateMemory
            (sizeof(FPPI_Status), PIL_MEM_ZERO_INIT);

        if (outStatus == NULL)
        {
            retVal = FPPI_OUT_OF_MEMORY;
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            goto DRS_LBL_1;
        }

        *outStatus = *((FPPI_Status*)policy_list);

        /* Set out param */
        *outObj = outStatus;

        }
        break;

    case FPPI_CEReady_Event:
        {
            /* No data to pack. Just set backend ready flag. */
            PIL_EnterCriticalSection(&g_beFlagslock);
            Bend_Ready = 1;
            PIL_LeaveCriticalSection(&g_beFlagslock);
        }
        break;

#ifdef _INCLUDE_ATM

		 case FPPI_ATM_IfAttrSet:
        {
            dataSz = sizeof(IfATM_IfAtmAttr_t) * policy_count;

            /* Allocate memory for FP struct and embedded obj */
            if ((atm_attr = (IfATM_IfAtmAttr_t *)PIL_AllocateMemory(
                dataSz,
               PIL_MEM_ZERO_INIT)) == NULL)
            {
                retVal = FPPI_OUT_OF_MEMORY;
                goto DRS_LBL_1;
            }

            /* Copy data into new local struct */
            memcpy(atm_attr, policy_list, dataSz);

            /* Set out param */
            *outObj = atm_attr;

        }
		 break;
		 case FPPI_ATM_IfDelete:
		 case FPPI_ATM_IfEnable:
		 case FPPI_ATM_IfDisable:
		 case FPPI_ATM_IfGenStatsGet:
        {
            dataSz = sizeof(FPPI_PortID) * policy_count;

            /* Allocate memory for FP struct and embedded obj */
            if ((portId = (FPPI_PortID *)PIL_AllocateMemory(
                dataSz,
               PIL_MEM_ZERO_INIT)) == NULL)
            {
                retVal = FPPI_OUT_OF_MEMORY;
                goto DRS_LBL_1;
            }

            /* Copy data into new local struct */
            memcpy(portId, policy_list, dataSz);


            /* Set out param */
            *outObj = portId;

        }
		 break;
       case FPPI_ATM_VccSet:
        {
            dataSz = sizeof(IfATM_IfVcc_t) * policy_count;

            /* Allocate memory for FP struct and embedded obj */
            if ((conn_info = (IfATM_IfVcc_t *)PIL_AllocateMemory(
                dataSz,
               PIL_MEM_ZERO_INIT)) == NULL)
            {
                retVal = FPPI_OUT_OF_MEMORY;
                goto DRS_LBL_1;
            }

            /* Copy data into new local struct */
            memcpy(conn_info, policy_list, dataSz);

            /* Set out param */
            *outObj = conn_info;

        }
        break;
       case FPPI_ATM_VccBind:
        {
            dataSz = sizeof(IfATM_VccBind_t) * policy_count;

            /* Allocate memory for FP struct and embedded obj */
            if ((vcc_bind = (IfATM_VccBind_t *)PIL_AllocateMemory(
                dataSz,
               PIL_MEM_ZERO_INIT)) == NULL)
            {
                retVal = FPPI_OUT_OF_MEMORY;
                goto DRS_LBL_1;
            }

            /* Copy data into new local struct */
            memcpy(vcc_bind, policy_list, dataSz);

            /* Set out param */
            *outObj = vcc_bind;

        }
        break;
     case FPPI_ATM_VccDelete:

        {

            dataSz = sizeof(IfATM_VccAddr_t) * policy_count;
            /* Allocate memory for FP struct and embedded obj */
            if ((vcc_info = (IfATM_VccAddr_t *)
                      PIL_AllocateMemory( dataSz, PIL_MEM_ZERO_INIT)) == NULL)
            {
                retVal = FPPI_OUT_OF_MEMORY;
                goto DRS_LBL_1;
            }
            /* Copy data into new local struct */
            memcpy(vcc_info, policy_list, dataSz);
            /* Set out param */
            *outObj = vcc_info;
        }
        break;

     case FPPI_ATM_VccEnable:
        {

            dataSz = sizeof(IfATM_VccAddr_t) * policy_count;
            /* Allocate memory for FP struct and embedded obj */
            if ((vcc_info = (IfATM_VccAddr_t *)
                      PIL_AllocateMemory( dataSz, PIL_MEM_ZERO_INIT)) == NULL)
            {
                retVal = FPPI_OUT_OF_MEMORY;
                goto DRS_LBL_1;
            }
            /* Copy data into new local struct */
            memcpy(vcc_info, policy_list, dataSz);
            /* Set out param */
            *outObj = vcc_info;
        }
        break;

     case FPPI_ATM_VccDisable:
        {

            dataSz = sizeof(IfATM_VccAddr_t) * policy_count;
            /* Allocate memory for FP struct and embedded obj */
            if ((vcc_info = (IfATM_VccAddr_t *)
                      PIL_AllocateMemory( dataSz, PIL_MEM_ZERO_INIT)) == NULL)
            {
                retVal = FPPI_OUT_OF_MEMORY;
                goto DRS_LBL_1;
            }
            /* Copy data into new local struct */
            memcpy(vcc_info, policy_list, dataSz);
            /* Set out param */
            *outObj = vcc_info;
        }
        break;

     case FPPI_ATM_VccOperStatusGet:
        {

            dataSz = sizeof(IfATM_VccAddr_t) * policy_count;
            /* Allocate memory for FP struct and embedded obj */
            if ((vcc_info = (IfATM_VccAddr_t *)
                      PIL_AllocateMemory( dataSz, PIL_MEM_ZERO_INIT)) == NULL)
            {
                retVal = FPPI_OUT_OF_MEMORY;
                goto DRS_LBL_1;
            }
            /* Copy data into new local struct */
            memcpy(vcc_info, policy_list, dataSz);
            /* Set out param */
            *outObj = vcc_info;
        }
        break;

     case FPPI_ATM_VccStatsGet:
        {

            dataSz = sizeof(IfATM_VccAddr_t) * policy_count;
            /* Allocate memory for FP struct and embedded obj */
            if ((vcc_info = (IfATM_VccAddr_t *)
                      PIL_AllocateMemory( dataSz, PIL_MEM_ZERO_INIT)) == NULL)
            {
                retVal = FPPI_OUT_OF_MEMORY;
                goto DRS_LBL_1;
            }
            /* Copy data into new local struct */
            memcpy(vcc_info, policy_list, dataSz);
            /* Set out param */
            *outObj = vcc_info;
        }
        break;

     case FPPI_ATM_AAL2_ChannelSet :
        {
             dataSz = sizeof(IfATM_AAL2_ChannelInfo_t) * policy_count;
            /* Allocate memory for FP struct and embedded obj */
            if ((aal2_info = (IfATM_AAL2_ChannelInfo_t *)PIL_AllocateMemory(
                dataSz,
                PIL_MEM_ZERO_INIT)) == NULL)
            {
                retVal = FPPI_OUT_OF_MEMORY;
                goto DRS_LBL_1;
            }
            /* Copy data into new local struct */
            memcpy(aal2_info, policy_list, dataSz);
            /* Set out param */
            *outObj = aal2_info;
        }
        break;

        case FPPI_ATM_AAL2_ChannelDel :
        {
             dataSz = sizeof(IfATM_AAL2_ConnId_t) * policy_count;
            /* Allocate memory for FP struct and embedded obj */
            if ((aal2_chan_id_info = (IfATM_AAL2_ConnId_t *)PIL_AllocateMemory(
                dataSz,
                PIL_MEM_ZERO_INIT)) == NULL)
            {
                retVal = FPPI_OUT_OF_MEMORY;
                goto DRS_LBL_1;
            }
            /* Copy data into new local struct */
            memcpy(aal2_chan_id_info, policy_list, dataSz);
            /* Set out param */
            *outObj = aal2_chan_id_info;
        }
        break;

      case FPPI_ATM_ConnectionXcSet :
      case FPPI_ATM_ConnectionXcDel :
        {
            dataSz = sizeof(NPF_ATM_ConnectionXc_t) * policy_count;

            /* Allocate memory for FP struct and embedded obj */
            if ((conn_xc_info = (NPF_ATM_ConnectionXc_t *)PIL_AllocateMemory(
                dataSz,
                PIL_MEM_ZERO_INIT)) == NULL)
            {
                retVal = FPPI_OUT_OF_MEMORY;
                goto DRS_LBL_1;
            }

            /* Copy data into new local struct */
            memcpy(conn_xc_info, policy_list, dataSz);

            /* Set out param */
            *outObj = conn_xc_info;
        }
        break;

   case FPPI_ATM_OAM_CP_Set:
        {
            dataSz = sizeof(IfATM_OAM_CP_t) * policy_count;

            /* Allocate memory for FP struct and embedded obj */
            if ((atm_oam_cp_info = (IfATM_OAM_CP_t *) 
                            PIL_AllocateMemory(dataSz,
                PIL_MEM_ZERO_INIT)) == NULL)
            {
                retVal = FPPI_OUT_OF_MEMORY;
                goto DRS_LBL_1;
            }

            /* Copy data into new local struct */
            memcpy(atm_oam_cp_info, policy_list, dataSz);

            /* Set out param */
            *outObj = atm_oam_cp_info;
        }
        break;

   case FPPI_ATM_OAM_CC_Set:
        {
            dataSz = sizeof(IfATM_OAM_CC_t) * policy_count;

            /* Allocate memory for FP struct and embedded obj */
            if ((atm_oam_cc_info = (IfATM_OAM_CC_t *)
                  PIL_AllocateMemory(
                dataSz,
                PIL_MEM_ZERO_INIT)) == NULL)
            {
                retVal = FPPI_OUT_OF_MEMORY;
                goto DRS_LBL_1;
            }

            /* Copy data into new local struct */
            memcpy(atm_oam_cc_info, policy_list, dataSz);

            /* Set out param */
            *outObj = atm_oam_cc_info;
        }
        break;

   case FPPI_ATM_OAM_LB_Set :
        {
            dataSz = sizeof(IfATM_OAM_LB_t) * policy_count;

            /* Allocate memory for FP struct and embedded obj */
            if ((atm_oam_lb_info = (IfATM_OAM_LB_t *)
                    PIL_AllocateMemory(
                dataSz,
                PIL_MEM_ZERO_INIT)) == NULL)
            {
                retVal = FPPI_OUT_OF_MEMORY;
                goto DRS_LBL_1;
            }

            /* Copy data into new local struct */
            memcpy(atm_oam_lb_info, policy_list, dataSz);

            /* Set out param */
            *outObj = atm_oam_lb_info;

        }
        break;

   case FPPI_ATM_OAM_Monitor:
        {
            dataSz = sizeof(IfATM_OAM_Mon_t) * policy_count;

            /* Allocate memory for FP struct and embedded obj */
            if ((atm_oam_monitor_info = (IfATM_OAM_Mon_t *)
                    PIL_AllocateMemory(
                dataSz,
                PIL_MEM_ZERO_INIT)) == NULL)
            {
                retVal = FPPI_OUT_OF_MEMORY;
                goto DRS_LBL_1;
            }

            /* Copy data into new local struct */
            memcpy(atm_oam_monitor_info, policy_list, dataSz);

            /* Set out param */
            *outObj = atm_oam_monitor_info;

        }
        break;

  case FPPI_ATM_OAM_AlarmSet :
        {
            dataSz = sizeof(IfATM_OAM_Alarm_t) * policy_count;

            /* Allocate memory for FP struct and embedded obj */
            if ((atm_oam_alarm_info = (IfATM_OAM_Alarm_t *)
                 PIL_AllocateMemory(
                dataSz,
                PIL_MEM_ZERO_INIT)) == NULL)
            {
                retVal = FPPI_OUT_OF_MEMORY;
                goto DRS_LBL_1;
            }

            /* Copy data into new local struct */
            memcpy(atm_oam_alarm_info, policy_list, dataSz);

            /* Set out param */
            *outObj = atm_oam_alarm_info;

        }
        break;

  case FPPI_ATM_OAM_AlarmClear:
        {
            dataSz = sizeof(IfATM_OAM_Alarm_t) * policy_count;

            /* Allocate memory for FP struct and embedded obj */
            if ((atm_oam_alarm_info = (IfATM_OAM_Alarm_t *)
                 PIL_AllocateMemory(
                dataSz,
                PIL_MEM_ZERO_INIT)) == NULL)
            {
                retVal = FPPI_OUT_OF_MEMORY;
                goto DRS_LBL_1;
            }

            /* Copy data into new local struct */
            memcpy(atm_oam_alarm_info, policy_list, dataSz);

            /* Set out param */
            *outObj = atm_oam_alarm_info;

        }
        break;

    case FPPI_ATM_OAM_PM_Set:
        {
            dataSz = sizeof(IfATM_OAM_PM_t) * policy_count;

            /* Allocate memory for FP struct and embedded obj */
            if ((atm_oam_pm_info = (IfATM_OAM_PM_t *)
                PIL_AllocateMemory(dataSz,
                PIL_MEM_ZERO_INIT)) == NULL)
            {
                retVal = FPPI_OUT_OF_MEMORY;
                goto DRS_LBL_1;
            }

            /* Copy data into new local struct */
            memcpy(atm_oam_pm_info, policy_list, dataSz);

            /* Set out param */
            *outObj = atm_oam_pm_info;

        }
        break;

#endif /* _INCLUDE_ATM */

    case  FPPI_IPv4_UniAddNextHop:
        {
       // FPPI_UnicastNextHopUpdateInfo_t* list;
        npf_ipv4_UnicastNextHopUpdateInfoList_t* list;
        FPPI_UnicastNextHopUpdateInfo_t* inObj =
            (FPPI_UnicastNextHopUpdateInfo_t*)policy_list;
        FPPI_UnicastNextHopUpdateInfo_t*  tmpPtr;


        /* Allocate memory for FP struct and embedded obj */
        if ((list = (npf_ipv4_UnicastNextHopUpdateInfoList_t*)PIL_AllocateMemory(
            sizeof(npf_ipv4_UnicastNextHopUpdateInfoList_t),
            PIL_MEM_ZERO_INIT)) == NULL)
        {
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        }

        list->numEntries = policy_count;
        if ((list->nhUpdateInfo = (FPPI_UnicastNextHopUpdateInfo_t*)PIL_AllocateMemory(sizeof(FPPI_UnicastNextHopUpdateInfo_t) * policy_count,
            PIL_MEM_ZERO_INIT)) == NULL)
        {
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        }

        tmpPtr = list->nhUpdateInfo;

        for (i=0; i<policy_count; i++)
        {
           dataSz = sizeof(FPPI_NextHopInfo) * inObj->numNHEntries;
           dataSz2 = sizeof(inObj->oldFEandL2IDs);
           if ((tmpPtr->nhInfo = (FPPI_NextHopInfo*)PIL_AllocateMemory(dataSz, PIL_MEM_ZERO_INIT)) == NULL)
           {
              npf_logger_write(LOG_DRS, LOG_ERR,
                  "Could not allocate memory.\n");
              retVal = FPPI_OUT_OF_MEMORY;
              goto DRS_LBL_1;
           }

           /* Copy data into new local struct */
           tmpPtr->nhid              = inObj->nhid;
           tmpPtr->numNHEntries      = inObj->numNHEntries;
           tmpPtr->numOldL2IDEntries = inObj->numOldL2IDEntries;

           memcpy(tmpPtr->nhInfo, inObj->nhInfo, dataSz);

           memcpy(tmpPtr->oldFEandL2IDs, inObj->oldFEandL2IDs, dataSz2);
           tmpPtr = tmpPtr + 1;
           inObj = inObj + 1;
       }

        /* Set out param */
        *outObj = list;
        }
        break;

    case FPPI_IPv4_UniQueryNextHop:
    {
        npf_ipv4_UnicastNextHopDeleteInfoList_t* list;
        FPPI_UnicastNextHopDeleteInfo_t* tmpPtr;
        NPF_uint32_t*   tmpQueryInfo;

        tmpQueryInfo = (NPF_uint32_t*)policy_list;
        
        /* Allocate memory for FP struct and embedded obj */
        if ((list = (npf_ipv4_UnicastNextHopDeleteInfoList_t*)PIL_AllocateMemory(
            sizeof(npf_ipv4_UnicastNextHopDeleteInfoList_t),
            PIL_MEM_ZERO_INIT)) == NULL)
        {
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        } 
        list->numEntries = policy_count;
        if ((list->nhDeleteInfo = (FPPI_UnicastNextHopDeleteInfo_t*)PIL_AllocateMemory(sizeof(FPPI_UnicastNextHopDeleteInfo_t) * policy_count,
            PIL_MEM_ZERO_INIT)) == NULL)
        {
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        }

        tmpPtr = list->nhDeleteInfo; 
        for (i=0; i<policy_count; i++)
        {
           /* Copy data into new local struct */
           tmpPtr->nhid        = *tmpQueryInfo;
           tmpQueryInfo = tmpQueryInfo + 1;
           tmpPtr = tmpPtr + 1;
        }
        /* Set out param */
        *outObj = list;
    }
        break;

    case FPPI_IPv4_UniDelNextHop:
        {
        npf_ipv4_UnicastNextHopDeleteInfoList_t* list;
        FPPI_UnicastNextHopDeleteInfo_t* tmpDeleteInfo;
        FPPI_UnicastNextHopDeleteInfo_t* tmpPtr;

        tmpDeleteInfo = (FPPI_UnicastNextHopDeleteInfo_t*)policy_list;

        /* Allocate memory for FP struct and embedded obj */
        if ((list = (npf_ipv4_UnicastNextHopDeleteInfoList_t*)PIL_AllocateMemory(
            sizeof(npf_ipv4_UnicastNextHopDeleteInfoList_t),
            PIL_MEM_ZERO_INIT)) == NULL)
        {
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        }
        list->numEntries = policy_count;
        if ((list->nhDeleteInfo = (FPPI_UnicastNextHopDeleteInfo_t*)PIL_AllocateMemory(sizeof(FPPI_UnicastNextHopDeleteInfo_t) * policy_count,
            PIL_MEM_ZERO_INIT)) == NULL)
        {
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        }

        tmpPtr = list->nhDeleteInfo; 
        for (i=0; i<policy_count; i++)
        {
           /* Copy data into new local struct */
           tmpPtr->nhid        = tmpDeleteInfo->nhid;
           memcpy(&tmpPtr->nhInfo, &tmpDeleteInfo->nhInfo, 
                   sizeof(FPPI_NextHopInfo));
           tmpDeleteInfo = tmpDeleteInfo + 1;
           tmpPtr = tmpPtr + 1;
        }        
        /* Set out param */
        *outObj = list;

        }
        break;

#if 0
    case FPPI_IPv4_AddRoute:
        {
        npf_ipv4_RouteList_t* list;
        dataSz = sizeof(npf_ipv4_RouteEntry_t) * policy_count;

        /* Allocate memory for FP struct and embedded obj */
        if ((list = (npf_ipv4_RouteList_t*)PIL_AllocateMemory(
            sizeof(npf_ipv4_RouteList_t) + dataSz,
            PIL_MEM_ZERO_INIT)) == NULL)
        {
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        }

        /* Copy data into new local struct */
        list->route_list  = (npf_ipv4_RouteEntry_t*)(list + 1);
        list->route_count = policy_count;
        memcpy(list->route_list, policy_list, dataSz);

        /* Set out param */
        *outObj = list;

        }
        break;
#endif

#if 0
    case FPPI_IPv4_DelRoute:
        {
        npf_ipv4_RouteList_t* list;
        npf_ipv4_RouteEntry_t* listEntry;
        npf_ipv4_RouteKey_t* inKeys = (npf_ipv4_RouteKey_t*)policy_list;

        keySz = sizeof(npf_ipv4_RouteKey_t);
        dataSz = sizeof(npf_ipv4_RouteEntry_t) * policy_count;

        /* Allocate memory for FP struct and embedded obj */
        if ((list = (npf_ipv4_RouteList_t*)PIL_AllocateMemory(
            sizeof(npf_ipv4_RouteList_t) + dataSz,
            PIL_MEM_ZERO_INIT)) == NULL)
        {
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        }

        /* Copy data into new local struct */
        list->route_list  = (npf_ipv4_RouteEntry_t*)(list + 1);
        list->route_count = policy_count;

        listEntry = list->route_list;

        for (i = 0; i < policy_count; i++, listEntry++)
        {
            /* Copy the key portion of the entry */
            memcpy(listEntry, inKeys, keySz);
        }

        /* Set out param */
        *outObj = list;

        }
        break;
#endif

#if 0
    case FPPI_IPv4_PurgeRoute:
        /* No data to pack */
        break;
#endif

    case FPPI_IPv4_UniAddPrefix:
    {
         npf_ipv4_UnicastPrefixUpdateInfoList_t* list;
         dataSz = sizeof(FPPI_UnicastPrefixUpdateInfo_t) * policy_count;

         if ((list = (npf_ipv4_UnicastPrefixUpdateInfoList_t*)PIL_AllocateMemory(sizeof(npf_ipv4_UnicastPrefixUpdateInfoList_t) + dataSz, 
             PIL_MEM_ZERO_INIT)) == NULL)
        {
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        }
        /* Copy data into new local struct */
        list->numEntries  = policy_count;
        list->prefixInfo = (FPPI_UnicastPrefixUpdateInfo_t*)(list + 1); 
        memcpy(list->prefixInfo, policy_list, dataSz);

        /* Set out param */
        *outObj = list;
    }
        break;

    case FPPI_IPv4_UniUpdatePrefix:
    case FPPI_IPv4_UniQueryPrefix:
    case FPPI_IPv4_UniDelPrefix:
        {
        npf_ipv4_UnicastPrefixList_t* list;
        dataSz = sizeof(NPF_IPv4UC_Prefix_t) * policy_count;

        /* Allocate memory for FP struct and embedded obj */
        if ((list = (npf_ipv4_UnicastPrefixList_t*)PIL_AllocateMemory(
            sizeof(npf_ipv4_UnicastPrefixList_t) + dataSz,
            PIL_MEM_ZERO_INIT)) == NULL)
        {
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        }

        /* Copy data into new local struct */
        list->numEntries  = policy_count;
        list->prfxInfo    = (NPF_IPv4UC_Prefix_t*)(list + 1);
        memcpy(list->prfxInfo, policy_list, dataSz);

        /* Set out param */
        *outObj = list;

        }
        break;

    case FPPI_IPv4_UniFlushPrefix:
         break;
 
    case FPPI_IPv4_UniFlushNextHop:
        {
           npf_ipv4_UnicastNextHopDeleteInfoList_t*  list;
           dataSz = sizeof(FPPI_UnicastNextHopDeleteInfo_t) * policy_count;
           /* Allocate memory for FP struct and embedded obj */
           if ((list = (npf_ipv4_UnicastNextHopDeleteInfoList_t*)PIL_AllocateMemory(
            sizeof(npf_ipv4_UnicastNextHopDeleteInfoList_t) + dataSz,
            PIL_MEM_ZERO_INIT)) == NULL)
            {
               npf_logger_write(LOG_DRS, LOG_ERR,
                   "Could not allocate memory.\n");
               retVal = FPPI_OUT_OF_MEMORY;
               goto DRS_LBL_1;
            }
           /* Copy data into new local struct */
           list->numEntries  = policy_count;
           list->nhDeleteInfo    = (FPPI_UnicastNextHopDeleteInfo_t*)(list + 1);
           memcpy(list->nhDeleteInfo, policy_list, dataSz);

           /* Set out param */
           *outObj = list;
        /* Nothing to do here */
        }
        break;

    case FPPI_IPv4_UniAddToL2:
    case FPPI_IPv4_UniDelToL2:
        {
        npf_ipv4_UnicastToL2List_t* list;
        dataSz = sizeof(FPPI_IPv4UniNHResolEntry_t) * policy_count;

        /* Allocate memory for FP struct and embedded obj */
        if ((list = (npf_ipv4_UnicastToL2List_t*)PIL_AllocateMemory(
            sizeof(npf_ipv4_UnicastToL2List_t) + dataSz,
            PIL_MEM_ZERO_INIT)) == NULL)
        {
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        }

        /* Copy data into new local struct */
        list->numEntries  = policy_count;
        list->toL2Info    = (FPPI_IPv4UniNHResolEntry_t*)
            (list + 1);
        memcpy(list->toL2Info, policy_list, dataSz);

        /* Set out param */
        *outObj = list;

        }
        break;

    case FPPI_IPv4_UniFlushToL2:

        /* Nothing to do here */
        break;
#if 0
    case FPPI_ARP_AddEntry:
        {
        npf_ipv4_ARPList_t* list;

        dataSz = sizeof(npf_ipv4_ARPEntry_t) * policy_count;

        /* Allocate memory for FP struct and embedded obj */
        if ((list = (npf_ipv4_ARPList_t*)PIL_AllocateMemory(
            sizeof(npf_ipv4_ARPList_t) + dataSz,
            PIL_MEM_ZERO_INIT)) == NULL)
        {
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        }

        /* Copy data into new local struct */
        list->arp_list  = (npf_ipv4_ARPEntry_t*)(list + 1);
        list->arp_count = policy_count;
        memcpy(list->arp_list, policy_list, dataSz);

        /* Set out param */
        *outObj = list;

        }
        break;
#endif

#if 0
    case FPPI_ARP_QueryEntry:
    case FPPI_ARP_DelEntry:
        {
        npf_ipv4_ARPList_t* list;
        npf_ipv4_ARPEntry_t* listEntry;
        npf_ipv4_ARPKey_t* inKeys = (npf_ipv4_ARPKey_t*)policy_list;
        keySz = sizeof(npf_ipv4_ARPKey_t);

        dataSz = sizeof(npf_ipv4_ARPEntry_t) * policy_count;

        /* Allocate memory for FP struct and embedded obj */
        if ((list = (npf_ipv4_ARPList_t*)PIL_AllocateMemory(
            sizeof(npf_ipv4_ARPList_t) + dataSz,
            PIL_MEM_ZERO_INIT)) == NULL)
        {
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        }

        /* Copy data into new local struct */
        list->arp_list  = (npf_ipv4_ARPEntry_t*)(list + 1);
        list->arp_count = policy_count;

        listEntry = list->arp_list;

        for (i = 0; i < policy_count; i++, listEntry++)
        {
            /* Copy the key portion of the entry */
            memcpy(listEntry, inKeys, keySz);
        }

        /* Set out param */
        *outObj = list;

        }
        break;
#endif

    case FPPI_ARP_PurgeEntry:
        /* No data to pack */
        break;

#ifdef _INCLUDE_IPV6TM

    case FPPI_IPv6TM_EnableConfig:

        /* Set out param. The data has already been packed into the
           correct struct. */
        *outObj = policy_list;

        break;

    case FPPI_IPv6TM_EnableAuto:
    case FPPI_IPv6TM_DisableAuto:
    case FPPI_IPv6TM_DisableConfig:
    case FPPI_IPv6TM_Enable6to4:
    case FPPI_IPv6TM_Disable6to4:
        {
        npf_ipv6tm_Tunnel_t* tunnel;

        if ((tunnel = (npf_ipv6tm_Tunnel_t*) PIL_AllocateMemory
            (sizeof(npf_ipv6tm_Tunnel_t), PIL_MEM_ZERO_INIT)) == NULL)
        {
            retVal = FPPI_OUT_OF_MEMORY;
            npf_logger_write(LOG_DRS, LOG_ERR, "Could not allocate memory.\n");
            goto DRS_LBL_1;
        }

        /* Copy data into new local struct */
        tunnel->portid = portid;
        tunnel->destip = 0;

        /* Set out param */
        *outObj = tunnel;

        }
        break;

#endif

#ifdef _INCLUDE_IPV6

    case FPPI_IPv6_AddNextHop:
    case FPPI_IPv6_DelNextHop:
        {
        FPPI_IPv6_UnicastNextHopUpdateInfo_t* list;
        FPPI_IPv6_UnicastNextHopUpdateInfo_t* inObj =
            (FPPI_IPv6_UnicastNextHopUpdateInfo_t*)policy_list;
        dataSz = sizeof(FPPI_IPv6_NextHopInfo) * inObj->numNHEntries;
        dataSz2 = sizeof(inObj->oldFEandL2IDs);

        /* Allocate memory for FP struct and embedded obj */
        if ((list = (FPPI_IPv6_UnicastNextHopUpdateInfo_t*)PIL_AllocateMemory(
            sizeof(FPPI_IPv6_UnicastNextHopUpdateInfo_t) + dataSz,
            PIL_MEM_ZERO_INIT)) == NULL)
        {
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        }

        /* Copy data into new local struct */
        list->nhid              = inObj->nhid;
        list->numNHEntries      = inObj->numNHEntries;
        list->numOldL2IDEntries = inObj->numOldL2IDEntries;

        list->nhInfo   = (FPPI_IPv6_NextHopInfo*)(list + 1);
        memcpy(list->nhInfo, inObj->nhInfo, dataSz);

        memcpy(list->oldFEandL2IDs, inObj->oldFEandL2IDs, dataSz2);

        /* Set out param */
        *outObj = list;

        }
        break;

    case FPPI_IPv6_FlushNextHop:
    case FPPI_IPv6_QueryNextHop:
        /* No data to pack */
        break;

    case FPPI_IPv6_AddPrefix:
    case FPPI_IPv6_DelPrefix:
        {
        npf_ipv6_PrefixList_t* list;
        dataSz = sizeof(NPF_IPv6PrefixEntry_t) * policy_count;

        /* Allocate memory for FP struct and embedded obj */
        if ((list = (npf_ipv6_PrefixList_t*)PIL_AllocateMemory(
            sizeof(npf_ipv6_PrefixList_t) + dataSz,
            PIL_MEM_ZERO_INIT)) == NULL)
        {
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        }

        /* Copy data into new local struct */
        list->prfxInfo    = (NPF_IPv6PrefixEntry_t*)(list + 1);
        list->numEntries  = policy_count;

        memcpy(list->prfxInfo, policy_list, dataSz);

        /* Set out param */
        *outObj = list;

        }
        break;

    case FPPI_IPv6_FlushPrefix:
    case FPPI_IPv6_QueryPrefix:
        /* No data to pack */
        break;

    case FPPI_IPv6_AddToL2:
    case FPPI_IPv6_DelToL2:
        {
        npf_ipv6_L2List_t* list;
        dataSz = sizeof(NPF_IPv6NbrEntry_t) * policy_count;

        /* Allocate memory for FP struct and embedded obj */
        if ((list = (npf_ipv6_L2List_t*)PIL_AllocateMemory(
            sizeof(npf_ipv6_L2List_t) + dataSz,
            PIL_MEM_ZERO_INIT)) == NULL)
        {
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        }

        /* Copy data into new local struct */
        list->L2Info      = (NPF_IPv6NbrEntry_t*)(list + 1);
        list->numEntries  = policy_count;

        memcpy(list->L2Info, policy_list, dataSz);

        /* Set out param */
        *outObj = list;

        }
        break;

#endif

    case FPPI_PacketIO_Event:
        {
        uint32_t pktsSz;
        uint32_t pktObjSz;
        char* pktDataPtr;
        npf_Packet_t* pktObjPtr;
        npf_Packet_t* inPkts = (npf_Packet_t*)policy_list;
        npf_PacketList_t* list;

        pktObjSz = sizeof(npf_Packet_t);
        pktsSz = dataSz = pktObjSz * policy_count;

        /* Add size of embedded pkt data */
        for (i =0; i < policy_count; i++)
        {
            dataSz += inPkts[i].length;
        }

        /* Allocate memory for FP struct and embedded objs */
        if ((list = (npf_PacketList_t*)PIL_AllocateMemory(
            sizeof(npf_PacketList_t) + dataSz, PIL_MEM_ZERO_INIT)) == NULL)
        {
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        }

        /* Copy list object elements */
        list->packet_list  = pktObjPtr = (npf_Packet_t*)(list + 1);
        list->packet_count = policy_count;

        /* Init ptr for copying packet data just beyond the packet objects */
        pktDataPtr = ((char*)pktObjPtr) + pktsSz;

        for (i = 0; i < policy_count; i++)
        {
            /* Copy the packet object */
            memcpy(pktObjPtr, &(inPkts[i]), pktObjSz);

            /* Copy the packet data */
            memcpy(pktDataPtr, inPkts[i].buffer, inPkts[i].length);

            /* Set the new data ptr in this pkt */
            pktObjPtr->buffer = pktDataPtr;

            /* Increment pkt obj ptr and pkt data ptr for next iteration */
            pktObjPtr++;
            pktDataPtr += inPkts[i].length;
        }

        /* Set out param */
        *outObj = list;
        }
        break;

    case FPPI_L2_SetProperties:
        {
        npf_L2IntfAttrsList_t* list;
        dataSz = sizeof(npf_L2IntfAttrs_t) * policy_count;

        /* Allocate memory for FP struct and embedded obj */
        if ((list = (npf_L2IntfAttrsList_t*)PIL_AllocateMemory(
            sizeof(npf_L2IntfAttrsList_t) + dataSz,
            PIL_MEM_ZERO_INIT)) == NULL)
        {
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        }

        /* Copy data into new local struct */
        list->port_list  = (npf_L2IntfAttrs_t*)(list + 1);
        list->port_count = policy_count;
        memcpy(list->port_list, policy_list, dataSz);

        /* Set out param */
        *outObj = list;

        }
        break;

    case FPPI_L2_GetStatistics:
    case FPPI_L3_GetStatistics:
    {
        FPPI_Interface_GetStats_t* list;
        FPPI_Interface_GetStats_t* inObj;
        FPPI_PortID* inPort;

        inObj = (FPPI_Interface_GetStats_t*)policy_list;

        if ((inPort = (FPPI_PortID *)PIL_AllocateMemory(sizeof(FPPI_PortID)*policy_count,PIL_MEM_ZERO_INIT)) == NULL)
        {
             npf_logger_write(LOG_DRS, LOG_ERR,
                 "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        }
        if ((list = (FPPI_Interface_GetStats_t*)PIL_AllocateMemory(sizeof(FPPI_Interface_GetStats_t),PIL_MEM_ZERO_INIT)) == NULL)
        {
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        }
        list->port_array = inPort;
        /* Copy data into new local struct */
        memcpy(list->port_array, inObj->port_array, sizeof(FPPI_PortID)*policy_count);
        list->port_count = inObj->port_count;
        list->ifType = inObj->ifType;
 
        /* Set out param */
        *outObj = list;
    }
    break;

    case FPPI_ICMP_GetStatistics:
        {
        npf_PortList_t* list;
        dataSz = sizeof(FPPI_PortID) * policy_count;

        /* Allocate memory for FP struct and embedded obj */
        if ((list = (npf_PortList_t*)PIL_AllocateMemory(
            sizeof(npf_PortList_t) + dataSz,
            PIL_MEM_ZERO_INIT)) == NULL)
        {
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        }

        /* Copy data into new local struct */
        list->port_list  = (FPPI_PortID*)(list + 1);
        list->port_count = policy_count;
        memcpy(list->port_list, policy_list, dataSz);

        /* Set out param */
        *outObj = list;

        }
        break;

    case FPPI_L3_SetProperties:
    case FPPI_L3_DelProperties:

#ifdef _INCLUDE_IPV6
    case FPPI_L3_SetPropertiesIPv6:
    case FPPI_L3_DelPropertiesIPv6:
#endif
        {
        npf_L3IntfAttrsList_t* list;
        dataSz = sizeof(npf_L3IntfAttrs_t) * policy_count;

        /* Allocate memory for FP struct and embedded obj */
        if ((list = (npf_L3IntfAttrsList_t*)PIL_AllocateMemory(
            sizeof(npf_L3IntfAttrsList_t) + dataSz,
            PIL_MEM_ZERO_INIT)) == NULL)
        {
            npf_logger_write(LOG_DRS, LOG_ERR,
                "Could not allocate memory.\n");
            retVal = FPPI_OUT_OF_MEMORY;
            goto DRS_LBL_1;
        }

        /* Copy data into new local struct */
        list->port_list  = (npf_L3IntfAttrs_t*)(list + 1);
        list->port_count = policy_count;
        memcpy(list->port_list, policy_list, dataSz);

        /* Set out param */
        *outObj = list;

        }
        break;

/* MPLS */
#ifdef _INCLUDE_MPLS

    case FPPI_CAT_MPLS:
         {
            FPPI_MPLS_Data_t *mplsPayload;
            dataSz = sizeof(FPPI_MPLS_Data_t) * policy_count;

            /* Allocate memory for FP struct and embedded obj */
            if ((mplsPayload = (FPPI_MPLS_Data_t *)PIL_AllocateMemory(
                sizeof(FPPI_MPLS_Data_t) + dataSz, PIL_MEM_ZERO_INIT)) == NULL)
            {
                npf_logger_write(LOG_DRS, LOG_ERR,
                     "Could not allocate memory for FPPI_CAT_MPLS\n");
                retVal = FPPI_OUT_OF_MEMORY;
                goto DRS_LBL_1;
            }

            /* Copy data into new local struct */
            memcpy(mplsPayload, policy_list, dataSz);

            /* Set out param */
            *outObj = mplsPayload;

         }
         break;

#endif /* MPLS */


/* DS */
#ifdef _INCLUDE_DIFFSERV

    case FPPI_DS_DpeOperation:
        {
            FPPI_DS_Dpe_t       *inObj;
            FPPI_DS_Dpe_t       *dpeInfo;
            uint32_t            infoSize;
            inObj = (FPPI_DS_Dpe_t *)policy_list;
            infoSize = sizeof(FPPI_DS_Dpe_t);
            dpeInfo = (FPPI_DS_Dpe_t *)PIL_AllocateMemory(infoSize, PIL_MEM_ZERO_INIT);
            if(dpeInfo == NULL)
            {
                npf_logger_write(LOG_DRS, LOG_ERR, "Could not allocate memory.\n");
                retVal = FPPI_OUT_OF_MEMORY;
                goto DRS_LBL_1;
            }
            memcpy(dpeInfo, inObj, sizeof(FPPI_DS_Dpe_t));
            *outObj = dpeInfo;
        }
        break;
#endif

      default:
        break;
    } /* switch (cbtype) */

DRS_LBL_1:

    npf_logger_write(LOG_DRS, LOG_TRACE,
        "Leaving method.\n");

    return retVal;
}


