/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*           Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/

/*****************************************************************************
* $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/PDKManager/src/cbthread.c,v $
* $Revision: 1.7 $
* $Date: 2003/11/17 16:57:15 $
*
* Description:
*    PDK Manager Callback Thread & Mailbox
*    Create a new CB thread executing callback and use PIL mailbox to 
*    communicate between the command thread
*    Msg from writer to mailbox and from mailbox to reader
*    are all copied here.
*
*****************************************************************************/

#include <string.h>
#include <assert.h>

#include "pdkmgr.h"
#include "log.h"

/* Golobal Variable */
PilMailbox g_mailbox;
PilThread  g_cbthread[MAXCBTHREADNUM];


/***
 * Function :ifFreeRespMemory()
 * 
 *   This cleans up embedded pointers in IF Callback Response structure.
 *   after user Callback function is invoked.
 */
static void ifFreeRespMemory(NPF_IfCallbackData_t *ifCbData)
{
   uint32_t  respIndex =0;
	NPF_IfAsyncResponse_t *resp = NULL;

   if((ifCbData->n_resp == 0) &&  (ifCbData->resp == NULL))
      return;

	resp = (NPF_IfAsyncResponse_t *)ifCbData->resp;
   
   switch(ifCbData->type)
   {
      case NPF_IF_ATM_VCC_STATS_GET:
           for(respIndex =0; respIndex < ifCbData->n_resp; respIndex++)
           {
              if(resp[respIndex].u.vccStats)
                 PIL_FreeMemory(resp[respIndex].u.vccStats);
           }
           break;
   
      case NPF_IF_STATS_GET:
           for(respIndex =0; respIndex < ifCbData->n_resp; respIndex++)
           {
              if(resp[respIndex].u.ifStats)
                 PIL_FreeMemory(resp[respIndex].u.ifStats);
           }
           break;
   
               
      default:
           /* No embedded pointer .. so nothing to free */
           break;
   }
   PIL_FreeMemory(ifCbData->resp);

} /* end of ifFreeRespMemory() */



#ifdef _INCLUDE_MPLS

/***
 * Function : mplsClearRespMemory()
 * 
 *   This cleans up embedded pointers in MPLs Callback Response structure.
 *   after user Callback function is invoked.
 */

static void mplsClearRespMemory(NPF_MPLS_CallbackData_t *mplsCbData)
{
   uint32_t  respIndex =0;

   if((mplsCbData->numResp == 0) &&  (mplsCbData->resp == NULL))
      return;
   
   switch(mplsCbData->type)
   {
      case NPF_MPLS_LSP_ENTRY_CREATE:
           for(respIndex =0; respIndex < mplsCbData->numResp; respIndex++)
           {
              if(mplsCbData->resp->u.lspCreateResp.nhlfeSetResp.numNhlfeResp)
                 PIL_FreeMemory(mplsCbData->resp->u.lspCreateResp.nhlfeSetResp.nhlfeResp);
           }
           break;
   
      case NPF_MPLS_NHLFE_SET_CREATE:
           for(respIndex =0; respIndex < mplsCbData->numResp; respIndex++)
           {
              if(mplsCbData->resp->u.nhlfeSetCreateResp.numNhlfeResp)
                 PIL_FreeMemory(mplsCbData->resp->u.nhlfeSetCreateResp.nhlfeResp);
           }
           break;

      case NPF_MPLS_NHLFE_SET_ENTRY_QUERY:
           for(respIndex =0; respIndex < mplsCbData->numResp; respIndex++)
           {
              if(mplsCbData->resp->u.nhlfeSetEntryResp.nhlfeSetEntry.numPolicy)
                 PIL_FreeMemory(
                   mplsCbData->resp->u.nhlfeSetEntryResp.nhlfeSetEntry.policyArray);
           }
           break;
           
           
      default:
           /* No embedded pointer .. so nothing to free */
           break;
   }
   PIL_FreeMemory(mplsCbData->resp);

} /* end of mplsClearRespMemory() */

#endif /* MPLS */

/*
 *  CallbackRoutine function, running in separate thread.
 */
uint32_t CallbackRoutine(void *param) 
{
 
    int                     count = 0;
    PDK_MSG_t               msg;

    NPF_EVENT_CBFUNC         eventCBFunc =NULL;
    NPF_IP_EVENT_CBFUNC      ipEventCBFunc =NULL;
    NPF_IPv4_CBFUNC          ipv4ApiCBFunc =NULL;
    NPF_IfCallbackFunc_t     ifApiCBFunc =NULL;
    NPF_IfEventHandlerFunc_t ifEventCBFunc =NULL;
#ifdef _INCLUDE_IPV6
    NPF_IPv6_CBFUNC          ipv6ApiCBFunc =NULL;
#endif
#ifdef _INCLUDE_IPV6TM
    NPF_IPv6TM_CBFUNC        ipv6TMApiCBFunc =NULL;
#endif
#ifdef _INCLUDE_MPLS
    NPF_MPLS_CallbackFunc_t  mplsApiCBFunc =NULL;
#endif
#ifdef _INCLUDE_DIFFSERV
    NPF_DS_CallbackFunc_t    dsApiCBFunc =NULL;
#endif
#ifdef _INCLUDE_INTSERV
    NPF_IS_CallbackFunc_t    isApiCBFunc =NULL;
#endif
    
    npf_logger_write(LOG_CB, LOG_INFO,
                "CallbackRoutine Thread #%d Created", (int)param);
    
    memset(&msg, 0, sizeof(PDK_MSG_t));
 
    while (PIL_ReadMailbox(g_mailbox, &msg, PIL_INFINITE_TIMEOUT) == PIL_OK)
    {
        count++;
        npf_logger_write(LOG_CB, LOG_INFO, "Got msg of type %d\n", msg.type);

        if (msg.type == PDK_EXITTHREAD) 
        {
            npf_logger_write(LOG_CB, LOG_INFO,
                "Thread %d invoke PDK_EXITTHREAD %d in Callback Mailbox",
                (int)param, count);
            break;
        }

        npf_logger_write(LOG_CB, LOG_INFO,
            "Thread %d invoke %s %d in Callback Mailbox", (int)param,
            (msg.type == PDK_EVENTCB)?"PDK_EVENTCB":"PDK_COMPELETIONCB",count);

        printf("PDK Callback Message Category is %d\n", msg.type);
        switch (msg.type) {
        case PDK_EVENTCB:
            eventCBFunc = msg.cbfunc;
            if (eventCBFunc != NULL);
                eventCBFunc(msg.context, msg.handle, msg.cbtype, msg.data);
            break;

        case PDK_IPEVENTCB:
            ipEventCBFunc = msg.cbfunc;
            if (ipEventCBFunc != NULL);
                ipEventCBFunc((NPF_userContext_t)msg.context, msg.ipevent);
            break;

        case PDK_IPv4COMPLETIONCB:
            ipv4ApiCBFunc = msg.cbfunc;
            if (ipv4ApiCBFunc != NULL)
            {
                printf("popped %d type\n", msg.ipdata.u.ipv4Data.type); 
                ipv4ApiCBFunc((NPF_userContext_t)msg.context, 
				                  (NPF_correlator_t)msg.correlator, 
				                   msg.ipdata.u.ipv4Data);
            }
            break;
#ifdef _INCLUDE_IPV6

        case PDK_IPv6COMPLETIONCB:
            ipv6ApiCBFunc = msg.cbfunc;
            if (ipv6ApiCBFunc != NULL)
                ipv6ApiCBFunc((NPF_userContext_t)msg.context, (NPF_correlator_t)msg.correlator, msg.ipdata.u.ipv6Data);
            if(msg.ipdata.u.ipv6Data.resp != NULL)
              PIL_FreeMemory(msg.ipdata.u.ipv6Data.resp);
            break;
               
#endif

#ifdef _INCLUDE_IPV6TM

        case PDK_IPv6TMCOMPLETIONCB:
            ipv6TMApiCBFunc = msg.cbfunc;
            if (ipv6TMApiCBFunc != NULL)            
                ipv6TMApiCBFunc((NPF_userContext_t)msg.context, (NPF_correlator_t)msg.correlator, msg.ipdata.u.ipv6TMData);
	    if(msg.ipdata.u.ipv6TMData.resp != NULL)
              PIL_FreeMemory(msg.ipdata.u.ipv6TMData.resp);
            break;
#endif

#ifdef _INCLUDE_DIFFSERV
        case PDK_DSCOMPLETIONCB:
            dsApiCBFunc = msg.cbfunc;
            if (dsApiCBFunc != NULL)            
	    {
		npf_logger_write(LOG_DS, LOG_TRACE, 
				"Invoking Callback : %X, Corr: %X, Resp : %X",
				dsApiCBFunc, msg.correlator, msg.dsdata.resp); 
                dsApiCBFunc((NPF_userContext_t)msg.context, (NPF_correlator_t)msg.correlator, &msg.dsdata);
		npf_logger_write(LOG_DS, LOG_TRACE, "Callback Invoked !!!!!");
	    }
	    if(msg.dsdata.resp != NULL)
	    {
	      npf_logger_write(LOG_DS, LOG_TRACE, "Freeing Response: %X", msg.dsdata.resp);
              PIL_FreeMemory(msg.dsdata.resp);
	    }
	    memset(&msg, 0, sizeof(PDK_MSG_t));
            break;

#endif

#ifdef _INCLUDE_INTSERV
        case PDK_ISCOMPLETIONCB:
            isApiCBFunc = msg.cbfunc;
            if (isApiCBFunc != NULL)            
                isApiCBFunc((NPF_userContext_t)msg.context, (NPF_correlator_t)msg.correlator, &msg.isdata);
	    if(msg.isdata.resp != NULL)
              PIL_FreeMemory(msg.isdata.resp);
	    memset(&msg, 0, sizeof(PDK_MSG_t));
            break;
#endif

#ifdef _INCLUDE_MPLS

        case PDK_MPLSCOMPLETIONCB:
            mplsApiCBFunc = msg.cbfunc;
            if (mplsApiCBFunc != NULL)            
                mplsApiCBFunc((NPF_userContext_t)msg.context, 
                              (NPF_correlator_t)msg.correlator, 
                              &msg.mplsdata);
            mplsClearRespMemory(&msg.mplsdata);
	         memset(&msg, 0, sizeof(PDK_MSG_t));
            break;

#endif

        case PDK_IFCOMPLETIONCB:
	    npf_logger_write(LOG_IF, LOG_TRACE, "=> IF CALL BACK <=");
            ifApiCBFunc = msg.cbfunc;
            if (ifApiCBFunc != NULL)            
                ifApiCBFunc((NPF_userContext_t)msg.context, (NPF_correlator_t)msg.correlator, &msg.ifdata);
            if(msg.ifdata.n_resp && msg.ifdata.resp != NULL)
              ifFreeRespMemory((NPF_IfCallbackData_t *)&msg.ifdata);
	    memset(&msg, 0, sizeof(PDK_MSG_t));
            break;
            
        case PDK_IFEVENTCB:
	    npf_logger_write(LOG_IF, LOG_TRACE, "=> IF EVENT CALL BACK <=");
            ifEventCBFunc = msg.cbfunc;
            if (ifEventCBFunc != NULL)            
                ifEventCBFunc((NPF_userContext_t)msg.context, msg.ifEvent);
	    if(msg.ifEvent.eventData != NULL)
              PIL_FreeMemory(msg.ifEvent.eventData);
	    memset(&msg, 0, sizeof(PDK_MSG_t));
            break;

        default:
            npf_logger_write(LOG_CB, LOG_ERR, "Invalid Message Callback Category");
            printf( "Invalid Message Callback Category");
            break;
        } /* switch */

        /* Free Response data which was allocated in components */
        if(msg.data)
        {
            PIL_FreeMemory(msg.data);
            memset(&msg, 0, sizeof(PDK_MSG_t));
        }
        npf_logger_write(LOG_CB, LOG_INFO,
            "Thread %d finished job %d", (int)param, count);
    }

    npf_logger_write(LOG_CB, LOG_INFO,
                "Callback Thread #%d terminated", (int)param);

    PIL_ExitThread(0);
 
    return 0;
}

NPF_RET cbthread_init(void) 
{
    int i;

    g_mailbox = PIL_CreateMailbox(sizeof(PDK_MSG_t), MAXMSGNUM);

    for (i=0; i<MAXCBTHREADNUM; i++)
        /* parameters are func, size, name, piority 1-8, param, options */
        /* Q: FIX How should I know how big the thread stack is ? */
        g_cbthread[i] = PIL_CreateThread(&CallbackRoutine, 1024,
                     "CallbackRoutine", 4, (void*)i, 0);

    return NPF_SUCCESS;
}

/*
 *    Write a msg to mailbox with msg.type = PDK_EXITTHREAD to
 *    ask cbthread ExitThread itself.
 *    Set a timeout to WriteMailbox to prevent app's infinite blocking
 *    Then WaitForThreads to finish all job and exit after getting
 *    EXITTHREADSIG msg. Set a timeout to WaitForThreads to prevent
 *    app's infinite blocking.
 *    Finally Destroy the thread & mailboxk
 */
NPF_RET cbthread_destroy(void) {
    
    int       i;
    PDK_MSG_t exitmsg;

    exitmsg.type = PDK_EXITTHREAD;

    for (i=0; i<MAXCBTHREADNUM; i++)
        PIL_WriteMailbox(g_mailbox, &exitmsg, EXITTIMEOUT);
    for (i=0; i<MAXCBTHREADNUM; i++) {
        if (PIL_TIMEOUT == PIL_WaitForThread(g_cbthread[i], EXITTIMEOUT))
            PIL_TerminateThread(g_cbthread[i]);
        PIL_DestroyThread(g_cbthread[i], NULL);
    }
    PIL_DestroyMailbox(g_mailbox); 

    return NPF_SUCCESS;
}

NPF_RET cbthread_pushIPAPICB(    NPF_GENERAL_CBFUNC    cbfunc,
                NPF_userContext_t        context,
                NPF_correlator_t        correlator,
                NPF_IPCallbackData_t    *data)
{
    PDK_MSG_t w_msg;
    uint32_t  respSize =0;
    uint32_t  numResp =0, i;

    w_msg.cbfunc = cbfunc;
    w_msg.context = (NPF_USERCONTEXT)context;
    w_msg.correlator = (NPF_CORRELATOR)correlator;
    w_msg.data = NULL;
    w_msg.cbtype = data->type;
    if(w_msg.cbtype == NPF_IPv4)
    {
        w_msg.type = PDK_IPv4COMPLETIONCB;
        w_msg.ipdata.u.ipv4Data.type    = data->u.ipv4Data.type;
        w_msg.ipdata.u.ipv4Data.allOK   = data->u.ipv4Data.allOK;
        w_msg.ipdata.u.ipv4Data.numResp = data->u.ipv4Data.numResp;
        numResp = data->u.ipv4Data.numResp;
        /* Response is valid only if there is one recvd. from FP */
        if(numResp > 0)
        {
           respSize = (sizeof(NPF_IPv4UC_AsyncResponse_t)*numResp);
           w_msg.ipdata.u.ipv4Data.resp = 
                     (NPF_IPv4UC_AsyncResponse_t*)
                     PIL_AllocateMemory(respSize, PIL_MEM_ZERO_INIT);
           /* Copy responses in loop */
           for(i=0; i< numResp; i++)
           {
              memcpy(&w_msg.ipdata.u.ipv4Data.resp[i],
                            &data->u.ipv4Data.resp[i],
                           sizeof(NPF_IPv4UC_AsyncResponse_t));
           }
        }
    }

#ifdef _INCLUDE_IPV6
    else if(data.type == NPF_IPv6)
    {
        w_msg.type = PDK_IPv6COMPLETIONCB;
        w_msg.ipdata.type = NPF_IPv6;
        w_msg.ipdata.u.ipv6Data.type = data->u.ipv6Data.type;
        memcpy(&w_msg.ipdata.u.ipv6Data, &data->u.ipv6Data, 
		sizeof(NPF_IPv6CallbackData_t));
    }
#endif
#ifdef _INCLUDE_IPV6TM
    else if(data.type == NPF_IPv6TM)
    {
        w_msg.type = PDK_IPv6TMCOMPLETIONCB;
        w_msg.ipdata.type = data->u.ipv6TMData.type;
        w_msg.ipdata.u.ipv6TMData = data->u.ipv6TMData;
    }
#endif
    PIL_WriteMailbox(g_mailbox, &w_msg, PIL_INFINITE_TIMEOUT);
    printf("Pushed msg of type %d %d %d %d\n", w_msg.type, w_msg.cbtype, w_msg.ipdata.type, correlator);   
    return NPF_SUCCESS;
}

NPF_RET cbthread_pushEventCB(    NPF_GENERAL_CBFUNC    cbfunc,
                NPF_USERCONTEXT        context,
                NPF_HANDLE        handle,
                NPF_EVENT        event,
                NPF_DATA        data    )
{
    PDK_MSG_t w_msg;

    w_msg.type = PDK_EVENTCB;
    w_msg.cbfunc = cbfunc;
    w_msg.context = (NPF_USERCONTEXT)context;
    w_msg.handle = handle;
    w_msg.cbtype = (int)event;
    w_msg.data = data;

    PIL_WriteMailbox(g_mailbox, &w_msg, PIL_INFINITE_TIMEOUT);
 
    return NPF_SUCCESS;

}

NPF_RET cbthread_pushIPEventCB(NPF_GENERAL_CBFUNC cbfunc,
                NPF_userContext_t context,
                NPF_IPEventCallbackData_t data)
{
    IP_EVENT_MSG_t w_msg;

    w_msg.type = PDK_IPEVENTCB;
    w_msg.cbfunc = cbfunc;
    w_msg.context = context;
    w_msg.data.type = data.type;
    if(w_msg.data.type == NPF_IPv4)
    {
        w_msg.data.u.eventCBArray = data.u.eventCBArray;
    }
#ifdef _INCLUDE_IPV6
    else if(data.type == NPF_IPv6)
    {
        w_msg.data.u.eventCBArrayV6 = data.u.eventCBArrayV6;
    }
#endif

    PIL_WriteMailbox(g_mailbox, &w_msg, PIL_INFINITE_TIMEOUT);
    
    return NPF_SUCCESS;
}

#ifdef _INCLUDE_DIFFSERV

NPF_RET cbthread_pushDSAPICB (
        NPF_GENERAL_CBFUNC       cbfunc,
        NPF_userContext_t        context,
        NPF_correlator_t         correlator,
        NPF_DS_CallbackData_t   *data)
{
    PDK_MSG_t w_msg;

    w_msg.type = PDK_DSCOMPLETIONCB;
    w_msg.cbfunc = cbfunc;
    w_msg.context = (NPF_USERCONTEXT)context;
    w_msg.correlator = (NPF_CORRELATOR)correlator;
    w_msg.cbtype = data->type;
    w_msg.dsdata = *data;

    npf_logger_write(LOG_DS, LOG_TRACE, 
		"Writing MailBox Data: %X", data->resp); 
    PIL_WriteMailbox(g_mailbox, &w_msg, PIL_INFINITE_TIMEOUT);
    return NPF_SUCCESS;
}

#endif


#ifdef _INCLUDE_MPLS

NPF_RET cbthread_pushMPLSAPICB (
        NPF_GENERAL_CBFUNC       cbfunc,
        NPF_userContext_t        context,
        NPF_correlator_t         correlator,
        NPF_MPLS_CallbackData_t  *data)
{
    PDK_MSG_t w_msg;

    w_msg.type = PDK_MPLSCOMPLETIONCB;
    w_msg.cbfunc = cbfunc;
    w_msg.context = (NPF_USERCONTEXT)context;
    w_msg.correlator = (NPF_CORRELATOR)correlator;
    w_msg.cbtype = data->type;
    w_msg.mplsdata = *data;

    PIL_WriteMailbox(g_mailbox, &w_msg, PIL_INFINITE_TIMEOUT);
    return NPF_SUCCESS;
}

#endif /* MPLS */

#ifdef _INCLUDE_INTSERV

NPF_RET cbthread_pushISAPICB (
        NPF_GENERAL_CBFUNC       cbfunc,
        NPF_userContext_t        context,
        NPF_correlator_t         correlator,
        NPF_IS_CallbackData_t    *data)
{
    PDK_MSG_t w_msg;

    w_msg.type = PDK_ISCOMPLETIONCB;
    w_msg.cbfunc = cbfunc;
    w_msg.context = (NPF_USERCONTEXT)context;
    w_msg.correlator = (NPF_CORRELATOR)correlator;
    w_msg.cbtype = data->type;
    w_msg.isdata = *data;

    PIL_WriteMailbox(g_mailbox, &w_msg, PIL_INFINITE_TIMEOUT);
    return NPF_SUCCESS;
}

#endif

NPF_RET cbthread_pushIFAPICB (
        NPF_GENERAL_CBFUNC       cbfunc,
        NPF_userContext_t        context,
        NPF_correlator_t         correlator,
        NPF_IfCallbackData_t   *data)
{
    PDK_MSG_t w_msg;

    w_msg.type = PDK_IFCOMPLETIONCB;
    w_msg.cbfunc = cbfunc;
    w_msg.context = (NPF_USERCONTEXT)context;
    w_msg.correlator = (NPF_CORRELATOR)correlator;
    w_msg.cbtype = data->type;
    w_msg.ifdata = *data;
    PIL_WriteMailbox(g_mailbox, &w_msg, PIL_INFINITE_TIMEOUT);
    return NPF_SUCCESS;
}

NPF_RET cbthread_pushIFEvent(NPF_GENERAL_CBFUNC cbfunc,
                NPF_userContext_t context,
                NPF_IfEventArray_t *event)
{
    PDK_MSG_t w_msg;

    w_msg.type = PDK_IFEVENTCB;
    w_msg.cbfunc = cbfunc;
    w_msg.context = (NPF_USERCONTEXT)context;
    w_msg.ifEvent = *event;
    PIL_WriteMailbox(g_mailbox, &w_msg, PIL_INFINITE_TIMEOUT);
    return NPF_SUCCESS;
}
