
/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/


/*****************************************************************************
* $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/PDKManager/src/pdkmgr.c,v $
* $Revision: 1.5.2.2 $
* $Date: 2004/03/22 09:47:37 $
*
* Description:
*     This is the PDK Manager module file.
*
*****************************************************************************/


#include "stdlib.h"
#include "log.h"


#include "pdk_ipv4.h"

#ifdef _INCLUDE_INTSERV
#include "pdk_intserv.h"
#endif

#ifdef _INCLUDE_DIFFSERV
#include "pdk_ds.h"
#endif
#ifdef _INCLUDE_IPV6TM
#include "pdk_ipv6tm.h"
#endif

#include "pdk_if.h"

#ifdef _INCLUDE_MPLS
#include "pdk_mpls.h"
#endif

#ifdef _INCLUDE_GTP
#include "pdk_gtp.h"
#endif

#ifdef _INCLUDE_ATM
#include "pdk_atm.h"
#endif

#include "pdkmgr.h"
#include "pdk_vic.h"
#include <assert.h>
#include <signal.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

#ifdef _LINUX_
#include "dynamicmodule.h"
#endif

#ifdef _VXWORKS_
#include "kernelLib.h"
#include "sysLib.h"
/* Duration of VxWorks system time slices */
#define PDK_VXWORKS_TIME_SLICE_MILLISECS 30
#endif

int g_pdk_init = 0;
static PilThread g_pdk_init_thread = 0;
static PilCriticalSection g_pdkmgrlock;

#define PDK_LOCK()       PIL_EnterCriticalSection(&g_pdkmgrlock);
#define PDK_UNLOCK()     PIL_LeaveCriticalSection(&g_pdkmgrlock);

#ifdef _INCLUDE_IPV6
extern NPF_error_t NPF_ipv6_init(void);
#endif

#ifdef _LINUX_
#include <sys/ipc.h>
#include <sys/msg.h>

static key_t hubkey = HUBKEYID;
#endif

int npf_pdk_init()
{
    int errtype; 

    if (g_pdk_init) assert(0);

    g_pdk_init = 1;

#ifdef _VXWORKS_
    /* Turn time slicing on. Use nominal value for slice duration. */
    kernelTimeSlice((uint32_t)(sysClkRateGet() *
        PDK_VXWORKS_TIME_SLICE_MILLISECS * 0.001));
#endif

    if (PIL_Init() != PIL_OK) assert(0);

    PIL_InitializeCriticalSection(&g_pdkmgrlock);

    /* mark the pdk_init thread */
    g_pdk_init_thread = PIL_GetCurrentThread();

    errtype = pdk_callback_init();
    if (errtype != NPF_SUCCESS)
    {
        npf_logger_write(LOG_PDKMGR, LOG_CRIT,
            "Initialization of the Callback Library failed \n");
        assert(0);
    }

    errtype = cbthread_init();
    if (errtype != NPF_SUCCESS)
    {
        npf_logger_write(LOG_PDKMGR, LOG_CRIT,
            "Initialization of the Callback Thread failed \n");
        assert(0);
    }
    errtype = FPPAPI_Initialize();
    if (errtype != FPPI_SUCCESS)
    {        
        npf_logger_write(LOG_PDKMGR, LOG_CRIT,
            "Initialization of the FP PLugin API failed \n");
        assert(0);
    }
    errtype = bd_init();
    if(errtype != NPF_SUCCESS)
    {
        npf_logger_write(LOG_PDKMGR, LOG_CRIT,
            "Initialization of B&D failed \n");
        assert(0);
    }
    /* Add FPAPIinit() over here     */
    errtype = ns_init();
    if (errtype != NPF_SUCCESS)
    {        
        npf_logger_write(LOG_PDKMGR, LOG_CRIT,
            "Initialization of Namespace failed \n");
        assert(0);
    }    
    errtype = cm_init();   
    if (errtype != NPF_SUCCESS)
    {        
        npf_logger_write(LOG_PDKMGR, LOG_CRIT,
            "Initialization of Config Manager failed \n");
        assert(0);
    }    

    /* Virtual interface needs to be created before IPv4 can use it */

#ifndef COLOCATED
    errtype = vic_init();

    if ( errtype != NPF_SUCCESS ) {
        npf_logger_write(LOG_PDKMGR, LOG_CRIT,
            "Initialization of VIDD Controller failed \n");
        assert(0);

    }

#endif

#ifdef _INCLUDE_MPLS
    errtype = pdk_mpls_init();
    if (errtype != NPF_SUCCESS)
    {
        npf_logger_write(LOG_PDKMGR, LOG_CRIT,
            "Initialization of Config Manager failed \n");
        assert(0);
    }
#endif

    errtype = pdk_if_init();
    if ( errtype != NPF_SUCCESS ) {
        npf_logger_write(LOG_PDKMGR, LOG_CRIT,
            "Initialization of IF failed \n");
        assert(0);
    }    

    errtype = pdk_ipv4_init();
    if ( errtype != NPF_SUCCESS ) {
        npf_logger_write(LOG_PDKMGR, LOG_CRIT,
            "Initialization of IPv4 failed \n");
        assert(0);
    }    

#ifdef _INCLUDE_IPV6
    errtype = NPF_ipv6_init();
    if ( errtype != NPF_SUCCESS ) {
        npf_logger_write(LOG_PDKMGR, LOG_CRIT,
            "Initialization of IPv6 failed \n");
        assert(0);
    }
#endif
    
#ifdef _INCLUDE_IPV6TM
    errtype = pdk_ipv6tm_init();
    if ( errtype != NPF_SUCCESS ) {
        npf_logger_write(LOG_PDKMGR, LOG_CRIT,
            "Initialization of IPv6TM failed \n");
        assert(0);
    }    
#endif

#ifdef _INCLUDE_DIFFSERV
    errtype = pdk_ds_init();
    if ( errtype != NPF_SUCCESS ) {
        npf_logger_write(LOG_PDKMGR, LOG_CRIT,
            "Initialization of DiffServ failed \n");
        assert(0);
    }
#endif

#ifdef _INCLUDE_INTSERV
    errtype = pdk_intserv_init();
    if ( errtype != NPF_SUCCESS ) {
        npf_logger_write(LOG_PDKMGR, LOG_CRIT,
            "Initialization of intserv failed \n");
        assert(0);
    }
#endif

#ifdef _INCLUDE_ATM
    errtype = pdk_atm_init();
    if ( errtype != NPF_SUCCESS ) {
        npf_logger_write(LOG_PDKMGR, LOG_CRIT,
            "Initialization of intserv failed \n");
        assert(0);
    }
#endif

#ifdef _INCLUDE_GTP
    errtype = pdk_gtp_init();
    if ( errtype != NPF_SUCCESS ) {
        npf_logger_write(LOG_GTP, LOG_CRIT,
            "Initialization of gtp failed \n");
        assert(0);
    }
#endif

    return 0;
}

int npf_pdk_start()
{
    int errtype;    

    errtype = FPPAPI_Start();
    if (errtype != FPPI_SUCCESS)
    {        
        npf_logger_write(LOG_PDKMGR, LOG_CRIT,
            "Initialization of the FP PLugin API failed \n");
        assert(0);
    }    
    /* Initialize the Transport Channels over here*/
    return 0;
}

int npf_pdk_shutdown()
{
#ifdef _LINUX_
    int msgqid;
    msgbuf msg;
#endif

    /* only the thread who did npf_pdk_init can do shutdown */
    if (PIL_GetCurrentThread() != g_pdk_init_thread) {
        npf_logger_write(LOG_PDKMGR, LOG_INFO,
            "Skip shutdown on thread %d\n",PIL_GetCurrentThread());
        return 0;
    }

    PDK_LOCK();

    if (!g_pdk_init) {
        PDK_UNLOCK();
        return 0;
    }

    g_pdk_init = 0;

#ifdef _LINUX_
    msgqid = msgget(hubkey,0666);
    if (msgqid != -1)
    {
        msg.mtype = 1;
        strcpy(msg.mtext,SHUTDOWN_MESSAGE);
        msgsnd(msgqid,&msg,MESSAGE_LENGTH,IPC_NOWAIT);
    }
#endif

    FPPAPI_Stop();
	npf_logger_write(LOG_PDKMGR, LOG_INFO, "===>FPPAPI_Stop done\n");

    cm_unbindFEs();
	npf_logger_write(LOG_PDKMGR, LOG_INFO, "===>cm_unbindFEs done\n");
#ifdef _INCLUDE_GTP
        pdk_gtp_shutdown();
	npf_logger_write(LOG_PDKMGR, LOG_INFO, "===>pdk_gtp_shutdown done\n");
#endif
#ifdef _INCLUDE_INTSERV
    pdk_intserv_shutdown();
	npf_logger_write(LOG_PDKMGR, LOG_INFO, "===>pdk_intserv_shtdon done\n");
#endif

#ifdef _INCLUDE_ATM
    pdk_atm_shutdown();
	npf_logger_write(LOG_PDKMGR, LOG_INFO, "===>pdk_atm_shutdown done\n");
#endif

#ifdef _INCLUDE_DIFFSERV
    pdk_ds_shutdown();
	npf_logger_write(LOG_PDKMGR, LOG_INFO, "===>pdk_ds_shutdown done\n");
#endif
    pdk_if_shutdown();
	npf_logger_write(LOG_PDKMGR, LOG_INFO, "===>pdk_if_shutdown done\n");
    pdk_ipv4_shutdown();
	npf_logger_write(LOG_PDKMGR, LOG_INFO, "===>pdk_ipv4_shutdown done\n");

#ifdef _INCLUDE_IPV6TM
    pdk_ipv6tm_shutdown();
	npf_logger_write(LOG_PDKMGR, LOG_INFO, "===>pdk_ipv6tm_shutdown done\n");
#endif

#ifdef _INCLUDE_IPV6
    /* pdk_ipv6_shutdown();
	npf_logger_write(LOG_PDKMGR, LOG_INFO, "===>pdk_ipv6_shutdown done\n");
     */
#endif

#ifdef _INCLUDE_MPLS
    pdk_mpls_shutdown();
	npf_logger_write(LOG_PDKMGR, LOG_INFO, "===>pdk_mpls_shutdown done\n");
#endif

#ifndef COLOCATED
    vic_shutdown();
	npf_logger_write(LOG_PDKMGR, LOG_INFO, "===>vic_shutdown done\n");
#endif

    ns_shutdown();
	npf_logger_write(LOG_PDKMGR, LOG_INFO, "===>ns_shutdown done\n");
    cm_destroy();
	npf_logger_write(LOG_PDKMGR, LOG_INFO, "===>cm_destroy done\n");
    bd_shutdown();
	npf_logger_write(LOG_PDKMGR, LOG_INFO, "===>bd_shutdown done\n");
    FPPAPI_Shutdown();
	npf_logger_write(LOG_PDKMGR, LOG_INFO, "===>FPPAPI_Shutdown done\n");
    cbthread_destroy();
	npf_logger_write(LOG_PDKMGR, LOG_INFO, "===>cbthread_destroy done\n");
 //   pdk_callback_destroy();
	npf_logger_write(LOG_PDKMGR, LOG_INFO, "===>cbthread_destroy done\n");

    PDK_UNLOCK();
    PIL_DestroyCriticalSection(&g_pdkmgrlock);
    PIL_DeInit();
    npf_logger_write(LOG_PDKMGR, LOG_INFO, "===>Success shutdown PDK\n");

    return 0;
}  
