/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/TransportPlugin/inc/fpppriv.h,v $
 * $Revision: 1.6 $
 * $Date: 2003/11/17 16:57:16 $
 *
 *
 ****************************************************************************/

#ifndef _FPP_PRIV_H_
#define _FPP_PRIV_H_

#include "fpp_common.h"
#include "forces_Types.h"
#include "list.h"
#include "pdk_callback.h"
#ifdef _INCLUDE_MPLS
#include "npf_mpls_types.h"
#endif
#include "npf_ds_types.h"
#include "npf_ipv4_types.h"

#ifdef _INCLUDE_IPV6
#include "npf_ipv6_types.h"
#endif

#define FORCES_VERSION 1

#define MAX_MISSED_BEATS_ALLOWED 3

#ifdef __cplusplus
extern "C" {
#endif

/************* Globals *****************/

extern FPPI_FEID FeidCounter;
extern uint32_t CmdCodeCounter;
extern PilCriticalSection fpp_lock;
extern PilCriticalSection fppi_lock;
extern DList   FEList;

/************* TPI local structures *****************/

typedef struct ForCESMessage_t
{
	forces_header_t header; /* alway present, always the same type */
	char		    data[0]; /* basically just a place holder
				                making it easier to find 
				                the actual data for the particular
				                type of message */
} *ForCESMessage;

typedef struct FEInfo_t
{
	FPPI_FEID   feid;
    forces_bind_info_t bindinfo;
	DList       outstandingcommands;

	uint32_t    connectionid; /* identifier supplied by Interconnect
                                 that must be passed back for every
                                 message we want to send to this fe */

    /* capabilities of the fe */
    forces_portlist_t*      portlist;
    forces_blocklist_t*     blocklist;
    FPPI_FE_Caps* fecaps;

    PilCriticalSection lock;

    PilThread heartbeatthread;
    int       msgpipe[2];
    int       missedbeats;
} *FEInfo;

typedef void (*FPPCommandHandler)(
                FPPI_FEID       feid,
                uint32_t        cmd_type,
                uint32_t        block_type,
                FPPI_CBHANDLE   cbhandle,
                FPPI_CORRELATOR correlator,
                uint32_t        global_result,
                char*           data);

extern void default_handler(
                FPPI_FEID       feid,
                uint32_t        cmd_type,
                uint32_t        block_type,
                FPPI_CBHANDLE   cbhandle,
                FPPI_CORRELATOR correlator,
                uint32_t        global_status,
                char*           data);

typedef struct FPPCommand_t
{
    FEInfo          feinfo;
    ForCESMessage   msg;

    uint32_t        cmd_type;
    uint32_t        block_type;
    char*           buffer;
    size_t          size;
    uint32_t        forces_correlator;

    FPPI_CORRELATOR fppi_correlator;
    FPPI_CBHANDLE   fppi_cbhandle;

    FPPCommandHandler   handler;
} *FPPCommand;

/************* Internal Methods *****************/

ForCESMessage newForCESMessage(
                FEInfo      feinfo,
                uint16_t    msg_type,
                uint32_t    correlator,
                size_t      size);

void ForCESMessageSend(
                FEInfo          feinfo,
                ForCESMessage   msg,
                size_t          size);

/* do not call delete message if msg has been sent.  
 * the send frees the resources when it is finished with them */ 
void deleteForCESMessage(ForCESMessage msg);
int ForCESCheckVersion(uint8_t version);

FEInfo newFEInfo(uint32_t connectionid, forces_bind_info_t *bindinfo);
void deleteFEInfo(FEInfo feinfo);

FEInfo FEInfoFind(FPPI_FEID feid);
FEInfo FEInfoFindByConnectId(uint32_t connect_id);
int FEInfoAddOutstandingCommand(FEInfo feinfo, FPPCommand fppcmd);
FPPCommand FEInfoRemoveOutstandingCommand(FEInfo feinfo, uint32_t correlator);
int FEInfoIsBlockTypeSupported(FEInfo feinfo, uint32_t block_type);

void ForCES_DatagramException ( uint32_t ex, uint32_t connect_id);
void ForCES_DatagramReceive (void* buf, uint32_t connect_id, uint32_t len);

FPPCommand newFPPCommand(
                FPPI_FEID           feid,
                FPPI_CORRELATOR     correlator,
                FPPI_CBHANDLE       cbhandle,
                FPPCommandHandler   handler,
                uint32_t            cmd_type,
                uint32_t            block_type,
                size_t              size);

void deleteFPPCommand(FPPCommand fppcmd);
FPPI_RET FPPCommandSend(FPPCommand fppcmd);

#ifdef __cplusplus
}
#endif

#endif /* _FPP_PRIV_H_ */

