/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2003 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/TransportPlugin/src/feinfo.c,v $
 * $Revision: 1.6 $
 * $Date: 2003/11/17 16:57:16 $
 *
 * Description:
 * Contains the utility functions for maintaining FEInfo's
 *
 ****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include "fppapi_types.h"
#include "fppapi.h"
#include "fpp_common.h"
#include "fpppriv.h"

#ifdef _VXWORKS_
#include "sys/times.h"
#include "ioLib.h"
#include "pipeDrv.h"

#else
#include "sys/time.h"
#endif

#include "log.h"

#include "interconnect_api.h"

extern FPPI_RET internal_feUnbindRequest(FEInfo feinfo);

int commandcompare(const void* a, const void* b)
{
    return (((FPPCommand)a)->forces_correlator 
                    == (uint32_t)b);
}

uint32_t FEInfoHeartbeatThread(void* arg)
{
    FEInfo fe = (FEInfo)arg;
    int retval;
    struct timeval tv;
    fd_set rfds;
    ForCESMessage msg;

    npf_logger_write (LOG_TPI, LOG_INFO, "Heart beat thread started\n");
    while (1)
    {
        tv.tv_sec = fe->bindinfo.heartbeat_interval;
        tv.tv_usec = 0;

        FD_ZERO(&rfds);
        FD_SET(fe->msgpipe[0],&rfds);

        retval = select(fe->msgpipe[0]+1,&rfds, NULL,NULL,&tv);
        if (retval)
        {
            break;
        }
        else
        {
            PIL_EnterCriticalSection(&(fe->lock));
            fe->missedbeats++;
            if (fe->missedbeats > MAX_MISSED_BEATS_ALLOWED)
            {
                npf_logger_write (LOG_TPI, LOG_INFO, "Missed more than %d heart beats, unbinding\n",MAX_MISSED_BEATS_ALLOWED);
                PIL_LeaveCriticalSection(&(fe->lock));
                internal_feUnbindRequest(fe);
                return 0;
            }
            PIL_LeaveCriticalSection(&(fe->lock));
            npf_logger_write (LOG_TPI, LOG_INFO, "Sending heart beat\n");
            msg = newForCESMessage(fe, Heartbeat, 0, 0);
            if (!msg)
            {
                continue;
            }
            ForCESMessageSend(fe,msg,0);
        }
    }

    return 0;
}

FEInfo newFEInfo(uint32_t connectionid, forces_bind_info_t *bindinfo)
{
#ifdef _VXWORKS_
    char tmp[128];
#endif
    FEInfo feinfo;

    if (!bindinfo) return 0;

    feinfo = (FEInfo)PIL_AllocateMemory(sizeof(struct FEInfo_t),0);
    if (!feinfo) return NULL;

    npf_list_init(&feinfo->outstandingcommands, PIL_FreeMemory);
    feinfo->outstandingcommands.compare = commandcompare;

    feinfo->connectionid = connectionid;

    feinfo->feid = (FPPI_FEID)bindinfo->feid;
    feinfo->bindinfo =  *bindinfo;
    feinfo->fecaps = NULL; /* will be allocated at capability response time */

    PIL_InitializeCriticalSection(&(feinfo->lock));

#ifdef _VXWORKS_
    sprintf(tmp,"/pipe/tp/fe%d/heartbeat",(int)feinfo->feid);
    pipeDevCreate(tmp,10, 5);
    feinfo->msgpipe[0] = feinfo->msgpipe[1] = open(tmp,O_RDWR,0666);
#else
    pipe(feinfo->msgpipe);
#endif
    feinfo->missedbeats = 0;

    feinfo->heartbeatthread = PIL_CreateThread(FEInfoHeartbeatThread,0,"FEInfoHeartbeatThread",0,feinfo,0);

    npf_list_pushBack(&FEList,feinfo);

    return feinfo;
}

void deleteFEInfo(FEInfo feinfo)
{
#ifdef _VXWORKS_
    char tmp[128];
#endif
    void* tmpdata;
    if (!feinfo) return;

    write(feinfo->msgpipe[1],"exit",5);
    PIL_WaitForThread(feinfo->heartbeatthread,feinfo->bindinfo.heartbeat_interval*2); /* dont wait forever */
    PIL_DestroyThread(feinfo->heartbeatthread,NULL);
    close(feinfo->msgpipe[0]);
#ifndef _VXWORKS_
    close(feinfo->msgpipe[1]);
#else
    sprintf(tmp,"/pipe/tp/fe%d/heartbeat",(int)feinfo->feid);
    pipeDevDelete(tmp,0);
#endif

    if (feinfo->fecaps) PIL_FreeMemory(feinfo->fecaps);

    npf_list_destroy(&feinfo->outstandingcommands);

    /* pop the FEInfo off the FEList */
    npf_list_removeByKey (&FEList, (void*)(feinfo->feid), &tmpdata);

    PIL_FreeMemory(feinfo);
}

FEInfo FEInfoFind(FPPI_FEID feid)
{
    return (npf_list_searchByKey(&FEList,(void*)feid));
}

FEInfo FEInfoFindByConnectId(uint32_t connect_id)
{
    FEInfo feinfo,tmp;
    DListItr *itr;

    feinfo = NULL;

    if ((itr = npf_list_itrCreate(&FEList)) == NULL) 
    {
    	return NULL;
    }
    npf_list_itrFirst(itr);
    do {
        tmp = (FEInfo)npf_list_itrGetData(itr);
        if (tmp->connectionid == connect_id)
        {
            feinfo = tmp;
            break;
        }
    } while (npf_list_itrNext(itr) != -1);
    npf_list_itrDelete(itr);
    return feinfo;
}

int FEInfoAddOutstandingCommand(FEInfo feinfo, FPPCommand fppcmd)
{
    if (!feinfo) return 0;

    npf_list_pushBack(&(feinfo->outstandingcommands),(void*)fppcmd);

    return 0;
}

FPPCommand FEInfoRemoveOutstandingCommand(FEInfo feinfo, uint32_t correlator)
{
    FPPCommand fppcmd,tmp;
    DListItr *itr;

    if (!feinfo) return NULL;

    if ((itr = npf_list_itrCreate(&feinfo->outstandingcommands)) == NULL) 
    {
    	return NULL;
    }
    fppcmd = NULL;
    npf_list_itrFirst(itr);
    do {
        tmp = (FPPCommand)npf_list_itrGetData(itr);
        if (tmp->forces_correlator == correlator)
        {
            fppcmd = tmp;
            npf_list_itrRemove(itr,(void**)&tmp);
            break;
        }
    } while (npf_list_itrNext(itr) != -1);
    npf_list_itrDelete(itr);
    return fppcmd;
}

int FEInfoIsBlockTypeSupported(FEInfo feinfo, uint32_t block_type)
{
    uint32_t i=0,supported=0;
    forces_block_info_t* block;

    block = feinfo->blocklist->block_array;
    for (;i<feinfo->blocklist->block_count;i++,block++)
    {
        if (block->block_type == block_type)
        {
            supported=1;
            break;
        }
    }
    /* block type support information is not supported yet
       just assume all blocks are supported for now */
    return 1;
    return supported;
}
