/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/TransportPlugin/src/fppapi.c,v $
 * $Revision: 1.6 $
 * $Date: 2003/11/17 16:57:16 $
 *
 * Description:
 * Contains the top level API for the transport plugin at the control
 * plane end.
 *
 ****************************************************************************/

#include "fppapi.h"
#include "forces_Types.h"
#include "fppapi_types.h"
#include "pdk_callback.h"
#include "list.h"
#include "stdio.h"
#ifdef _LINUX_
#include "sys/time.h"
#endif
#include "unistd.h"
#include "errno.h"
#include "fpppriv.h"
#include "fpp_common.h"


/****************************************************************************
 * Function: FPPAPI_SendPacket()                                            *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             packet_array - array of Packet structs to be sent to a FE     *
 *             packet_count - the number of packets in the array            *
 *                                                                          *
 * Description: This function is used to send a data packet to a FE.        *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_SendPacket(
        FPPI_FEID       feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE   cbhandle,
        npf_Packet_t    *packet_array,
        uint32_t        packet_count)
{
	return FPPI_FAIL;
}

#ifdef _INCLUDE_GTP

/****************************************************************************
 * Function: FPPAPI_gtp_rx_setup()                                          *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             teid - the tunnel identifier                                 *
 *                                                                          *
 * Description: This function is used to populate the GTP table in a FE,    *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_rx_setup(
    FPPI_FEID         feid,
    FPPI_CBHANDLE     cbhandle,
    FPPI_CORRELATOR   correlator,
    uint32_t         teid,
    npf_gtp_Config_t *config)
{
	return FPPI_FAIL;
}

/****************************************************************************
 * Function: FPPAPI_gtp_rx_release()                                        *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             teid - the tunnel identifier                                 *
 *                                                                          *
 * Description: This function is used to populate the GTP table in a FE,    *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_rx_release(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t       teid)
{
	return FPPI_FAIL;
}

/****************************************************************************
 * Function: FPPAPI_gtp_rx_getTunnelInfo()                                  *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             teid - the tunnel identifier                                 *
 *                                                                          *
 * Description: This function is used to populate the GTP table in a FE,    *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_rx_getTunnelInfo(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t       teid)
{
	return FPPI_FAIL;
}

/****************************************************************************
 * Function: FPPAPI_gtp_tx_setup()                                          *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             teid - the tunnel identifier                                 *
 *                                                                          *
 * Description: This function is used to populate the GTP table in a FE,    *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_tx_setup(
    FPPI_FEID           feid,
    FPPI_CBHANDLE       cbhandle,
    FPPI_CORRELATOR     correlator,
    uint32_t           teid,
    npf_gtp_Config_t   *config,
    npf_gtp_IuTunnel_t *Iu_Association)
{
	return FPPI_FAIL;
}

/****************************************************************************
 * Function: FPPAPI_gtp_tx_release()                                        *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             teid - the tunnel identifier                                 *
 *                                                                          *
 * Description: This function is used to populate the GTP table in a FE,    *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_tx_release(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t       teid)
{
	return FPPI_FAIL;
}

/****************************************************************************
 * Function: FPPAPI_gtp_tx_getTunnelInfo()                                  *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             teid - the tunnel identifier                                 *
 *                                                                          *
 * Description: This function is used to populate the GTP table in a FE,    *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_tx_getTunnelInfo(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t       teid)
{
	return FPPI_FAIL;
}

/****************************************************************************
 * Function: FPPAPI_gtp_srnc_relocationCmd()                                *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             teid - the tunnel identifier                                 *
 *                                                                          *
 * Description: This function is used to populate the GTP table in a FE,    *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_srnc_relocationCmd(
    FPPI_FEID           feid,
    FPPI_CBHANDLE       cbhandle,
    FPPI_CORRELATOR     correlator,
    uint32_t           teid,
    npf_gtp_IuTunnel_t *Iu_Association)
{
	return FPPI_FAIL;
}

/****************************************************************************
 * Function: FPPAPI_gtp_srnc_suspendRAB()                                   *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             teid - the tunnel identifier                                 *
 *                                                                          *
 * Description: This function is used to populate the GTP table in a FE,    *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_srnc_suspendRAB(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t       teid)
{
	return FPPI_FAIL;
}

/****************************************************************************
 * Function: FPPAPI_gtp_srnc_relocationComplete()                           *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             teid - the tunnel identifier                                 *
 *                                                                          *
 * Description: This function is used to populate the GTP table in a FE,    *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_srnc_relocationComplete(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t       teid)
{
	return FPPI_FAIL;
}

/****************************************************************************
 * Function: FPPAPI_gtp_trnc_relocationCmd()                                *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             teid - the tunnel identifier                                 *
 *                                                                          *
 * Description: This function is used to populate the GTP table in a FE,    *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_trnc_relocationCmd(
    FPPI_FEID         feid,
    FPPI_CBHANDLE     cbhandle,
    FPPI_CORRELATOR   correlator,
    uint32_t         teid,
    npf_gtp_Config_t *config)
{
	return FPPI_FAIL;
}

/****************************************************************************
 * Function: FPPAPI_gtp_trnc_relocationComplete()                           *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             teid - the tunnel identifier                                 *
 *                                                                          *
 * Description: This function is used to populate the GTP table in a FE,    *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_trnc_relocationComplete(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t       teid,
    uint32_t       PDCP_seq_num)
{
	return FPPI_FAIL;
}

/****************************************************************************
 * Function: FPPAPI_gtp_dataForwardCmd()                                    *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             teid - the tunnel identifier                                 *
 *                                                                          *
 * Description: This function is used to populate the GTP table in a FE,    *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_dataForwardCmd(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t         teid)
{
	return FPPI_FAIL;
}

/****************************************************************************
 * Function: FPPAPI_gtp_suspendRAB()                                        *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             teid - the tunnel identifier                                 *
 *                                                                          *
 * Description: This function is used to populate the GTP table in a FE,    *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_suspendRAB(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t         teid)
{
	return FPPI_FAIL;
}

/****************************************************************************
 * Function: FPPAPI_gtp_setBufferSize()                                     *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             teid - the tunnel identifier                                 *
 *                                                                          *
 * Description: This function is used to populate the GTP table in a FE,    *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_setBufferSize(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t       buffer_size)
{
	return FPPI_FAIL;
}

/****************************************************************************
 * Function: FPPAPI_gtp_getPDCPSeqNumber()                                  *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             teid - the tunnel identifier                                 *
 *                                                                          *
 * Description: This function is used to populate the GTP table in a FE,    *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_gtp_getPDCPSeqNumber(
    FPPI_FEID       feid,
    FPPI_CBHANDLE   cbhandle,
    FPPI_CORRELATOR correlator,
    uint32_t         teid)
{
	return FPPI_FAIL;
}

#endif

