/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/TransportPlugin/src/fppapi_atm.c,v $
 * $Revision: 1.5 $
 * $Date: 2003/11/17 16:57:16 $
 *
 * Description:
 *     Contains the top level API for the transport plugin at the
 *     control plane end.
 *
 ****************************************************************************/
#ifdef _INCLUDE_ATM

#include <stdio.h>
#include <stdlib.h>
#include "fppapi.h"
#include "fpppriv.h"
#include "fppencap.h"

#include "fppencap_common.h"
#include "pdk_callback_types.h"
#include "npf_atm_types.h"
#include "fppapi_atm_types.h"
#include "fppdecap_npf_atm.h"
#include "fppencap_npf_atm.h"

/************************ATM Functions *********************/

void IfATM_tpHandler(
                FPPI_FEID       feid,
                uint32_t        type, 
                uint32_t        block_type,
                FPPI_CBHANDLE   cbhandle,
                FPPI_CORRELATOR correlator,
                uint32_t        global_result,
                char*           data)
{
    FPPI_CONTEXT    cbcontext;
    fppapi_callback cb;

    switch (block_type)
    {
      case Forces_IfATM_IfAttrSet_Block: 
      case Forces_IfATM_IfDelete_Block: 
      case Forces_IfATM_IfEnable_Block: 
      case Forces_IfATM_IfDisable_Block: 
      case Forces_IfATM_VccSet_Block: 
      case Forces_IfATM_VccBind_Block: 
      case Forces_IfATM_VccDelete_Block: 
      case Forces_IfATM_VccEnable_Block: 
      case Forces_IfATM_VccDisable_Block: 
      case Forces_IfATM_VccOperStatusGet_Block: 
      case Forces_IfATM_AAL2_ChannelSet_Block: 
      case Forces_IfATM_AAL2_ChannelDel_Block: 
      case Forces_IfATM_ConnectionXcSet_Block: 
      case Forces_IfATM_ConnectionXcDel_Block: 
      case Forces_IfATM_OAM_CP_Set_Block: 
      case Forces_IfATM_OAM_CC_Set_Block: 
      case Forces_IfATM_OAM_LB_Set_Block: 
      case Forces_IfATM_OAM_Monitor_Block: 
      case Forces_IfATM_OAM_AlarmSet_Block: 
      case Forces_IfATM_OAM_AlarmClear_Block: 
      case Forces_IfATM_OAM_PM_Set_Block: 
		{
          IfATM_FpResponse_t resp;

          decapsulate_IfATM_FpResponse_t(&resp,data);
          cb = (fppapi_callback)pdk_callback_func(cbhandle,&cbcontext);
          cb(feid,correlator,cbcontext,&resp);
      }
		break;

      case Forces_IfATM_VccStatsGet_Block: 
		{
          IfATM_VccStats_t stats;

          decapsulate_IfATM_VccStats_t(&stats,data);
          cb = (fppapi_callback)pdk_callback_func(cbhandle,&cbcontext);
          cb(feid,correlator,cbcontext,&stats);
      }
		break;

      case Forces_IfATM_IfGenStatsGet_Block: 
		{
          IfATM_IfStatistics_t stats;

          decapsulate_IfATM_IfStatistics_t(&stats,data);
          cb = (fppapi_callback)pdk_callback_func(cbhandle,&cbcontext);
          cb(feid,correlator,cbcontext,&stats);
      }
		break;

		default:
		  break;
    }/* end of switch */
		 

    return;
}


/********************************************************************
 * Function:   FPPAPI_ATM_IfAttrSetFunc()
 *
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of ATM interface attributes to be set 
 * count - the number of entries in the array
 *
 * Description: This function is used to set an ATM interface attributes
 *					 on an FE
 *
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_IfAttrSetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_IfAtmAttr_t *info,
        NPF_uint32_t count)
{
    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_IfATM_IfAtmAttr_t(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_Add_Cmd,     /* comand type */ 
            Forces_IfATM_IfAttrSet_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_IfATM_IfAtmAttr_t(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}

/********************************************************************
 * Function:   FPPAPI_ATM_IfDeleteFunc()
 *
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of ATM interfaces to be deleted 
 * count - the number of entries in the array
 *
 * Description: This function is used to delete an ATM interface on an FE 
 *
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_IfDeleteFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        FPPI_PortID *info,
        NPF_uint32_t count)
{

    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_FPPI_PortID(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_Delete_Cmd,     /* comand type */ 
            Forces_IfATM_IfDelete_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_FPPI_PortID(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;

}


/********************************************************************
 * Function:   FPPAPI_ATM_IfEnableFunc()
 *
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of ATM interfaces to be enabled 
 * count - the number of entries in the array
 *
 * Description: This function is used to enable an ATM interface on an FE
 *
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_IfEnableFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        FPPI_PortID *info,
        NPF_uint32_t count)
{
    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_FPPI_PortID(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_Update_Cmd,     /* comand type */ 
            Forces_IfATM_IfEnable_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_FPPI_PortID(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;

}


/********************************************************************
 * Function:   FPPAPI_ATM_IfDisableFunc()
 *
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of ATM interfaces to be enabled 
 * count - the number of entries in the array
 *
 * Description: This function is used to disable an ATM interface on an FE
 *
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_IfDisableFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        FPPI_PortID *info,
        NPF_uint32_t count)
{

    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_FPPI_PortID(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_Update_Cmd,     /* comand type */ 
            Forces_IfATM_IfDisable_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_FPPI_PortID(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;


}


/********************************************************************
 * Function:   FPPAPI_ATM_IfGenericStatsGetFunc()
 *
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of ATM interfaces to be enabled 
 * count - the number of entries in the array
 *
 * Description: This function is used to get statistics of an ATM 
 *					interface on an FE
 *
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_IfGenericStatsGetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        FPPI_PortID * info,
        NPF_uint32_t count)
{

    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_FPPI_PortID(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_QueryStats_Cmd,     /* comand type */ 
            Forces_IfATM_IfGenStatsGet_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_FPPI_PortID(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;


}



/********************************************************************
 * Function:   FPPAPI_ATM_VccSetFunc()

 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of connection information to be setup
 * count - the number of entries in the array
 *
 * Description: This function is used to add an ATM connection to a FE
 *
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_VccSetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_IfVcc_t * info,
        NPF_uint32_t count)
{
    char* buf;
    size_t size;
    FPPCommand fppcmd;


    size = sizeof(uint32_t);
    size += sizeof_array_IfATM_IfVcc_t(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_Add_Cmd,     /* comand type */ 
            Forces_IfATM_VccSet_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_IfATM_IfVcc_t(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;


}

/********************************************************************
 * Function:   FPPAPI_ATM_VccBindFunc()

 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of connection information to be setup
 * count - the number of entries in the array
 *
 * Description: This function is used to add an ATM connection to a FE
 *
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_VccBindFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_VccBind_t * info,
        NPF_uint32_t count)
{
    char* buf;
    size_t size;
    FPPCommand fppcmd;


    size = sizeof(uint32_t);
    size += sizeof_array_IfATM_VccBind_t(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_Add_Cmd,     /* comand type */ 
            Forces_IfATM_VccBind_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_IfATM_VccBind_t(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;


}

/********************************************************************
 * Function:    FPPAPI_ATM_VccDeleteFunc()
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of connection information to be deleted
 * count - the number of entries in the array
 *
 * Description: This function is used to Delete an ATM connection to a FE
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_VccDeleteFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_VccAddr_t *info,
        NPF_uint32_t count)
{
    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_IfATM_VccAddr_t(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_Delete_Cmd,     /* comand type */ 
            Forces_IfATM_VccDelete_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_IfATM_VccAddr_t(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}


/********************************************************************
 * Function:    FPPAPI_ATM_VccEnableFunc()
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of connection information to be deleted
 * count - the number of entries in the array
 *
 * Description: This function is used to Enable an ATM connection to a FE
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_VccEnableFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_VccAddr_t *info,
        NPF_uint32_t count)
{
    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_IfATM_VccAddr_t(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_Update_Cmd,     /* comand type */ 
            Forces_IfATM_VccEnable_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_IfATM_VccAddr_t(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}



/********************************************************************
 * Function:    FPPAPI_ATM_VccDisableFunc()
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of connection information to be deleted
 * count - the number of entries in the array
 *
 * Description: This function is used to Disable an ATM connection to a FE
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_VccDisableFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_VccAddr_t *info,
        NPF_uint32_t count)
{
    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_IfATM_VccAddr_t(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_Update_Cmd,     /* comand type */ 
            Forces_IfATM_VccDisable_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_IfATM_VccAddr_t(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;

}

/********************************************************************
 * Function:    FPPAPI_ATM_VccOperStatusGetFunc()
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of connection information to be deleted
 * count - the number of entries in the array
 *
 * Description: This function is used to Get the operational status
 *              of an ATM connection to a FE
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_VccOperStatusGetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_VccAddr_t *info,
        NPF_uint32_t count)
{
    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_IfATM_VccAddr_t(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_QueryStats_Cmd,     /* comand type */ 
            Forces_IfATM_VccOperStatusGet_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_IfATM_VccAddr_t(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;

}


/********************************************************************
 * Function:    FPPAPI_ATM_VccStatsGetFunc()
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of connection information to be deleted
 * count - the number of entries in the array
 *
 * Description: This function is used to Get the Statistics 
 *              of an ATM connection to a FE
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_VccStatsGetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_VccAddr_t *info,
        NPF_uint32_t count)
{
    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_IfATM_VccAddr_t(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_QueryStats_Cmd,     /* comand type */ 
            Forces_IfATM_VccStatsGet_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_IfATM_VccAddr_t(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;

}


/********************************************************************
 * Function: FPPAPI_ATM_AAL2_ChannelSetFunc()
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * connection - connection information; VPI, VCI and LinkId
 * info - array of connection information to be deleted
 * count - the number of entries
 * Description: This function is used to set the aal2 channels
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_AAL2_ChannelSetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_AAL2_ChannelInfo_t *info,
        NPF_uint32_t count)
{
    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_IfATM_AAL2_ChannelInfo_t(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_Add_Cmd,     /* comand type */ 
            Forces_IfATM_AAL2_ChannelSet_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_IfATM_AAL2_ChannelInfo_t(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;

}


/********************************************************************
 * Function: FPPAPI_ATM_AAL2_ChannelDeleteFunc()
 * ---------------------------
 * Parameters:
 *
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * connection - connection information; VPI, VCI and LinkId
 * info - array of connection information to be deleted
 * count - the number of entries
 * Description: This function is used to delete the AAL2 channels
 * Return: FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_AAL2_ChannelDeleteFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_AAL2_ConnId_t *info,
        NPF_uint32_t count)
{
    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_IfATM_AAL2_ConnId_t(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_Delete_Cmd,     /* comand type */ 
            Forces_IfATM_AAL2_ChannelDel_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_IfATM_AAL2_ConnId_t(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;


}

  
/********************************************************************
 * Function:    FPPAPI_ATM_ConnectionXcSetFunc()
 * ---------------------------
 * Parameters:
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of ATM connection cross connect information
 * count - number of elements present in the info strcuture
 * Description: This function is used to set the ATM cross
 *              Connections
 * Return:  FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/

FPPI_RET FPPAPI_ATM_ConnectionXcSetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        NPF_ATM_ConnectionXc_t * info,
        NPF_uint32_t count)
{
    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_NPF_ATM_ConnectionXc_t(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_Add_Cmd,     /* comand type */ 
            Forces_IfATM_ConnectionXcSet_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_NPF_ATM_ConnectionXc_t(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;


}

/********************************************************************
 * Function:    FPPAPI_ATM_ConnectionXcDeleteFunc()
 * ---------------------------
 * Parameters:
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - array of ATM connection cross connect information
 * count - number of elements present in the info strcuture
 * Description: This function is used to delete the ATM cross
 *              Connections
 * Return:  FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/

FPPI_RET FPPAPI_ATM_ConnectionXcDeleteFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        NPF_ATM_ConnectionXc_t * info,
        NPF_uint32_t count)
{
    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_NPF_ATM_ConnectionXc_t(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_Delete_Cmd,     /* comand type */ 
            Forces_IfATM_ConnectionXcDel_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_NPF_ATM_ConnectionXc_t(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;


}


/********************************************************************
 * Function:     FPPAPI_ATM_OAM_CP_SetFunc()
 * ---------------------------
 * Parameters:
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - ATM OAM connection type
 *
 * Description: This function is used to set the ATM OAM connection
 *              point type
 * Return:  FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_OAM_CP_SetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_OAM_CP_t * info,
        NPF_uint32_t count)

{
    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_IfATM_OAM_CP_t(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_Add_Cmd,     /* comand type */ 
            Forces_IfATM_OAM_CP_Set_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_IfATM_OAM_CP_t(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;


}


/********************************************************************
 * Function:   FPPAPI_ATM_OAM_CC_SetFunc()
 * ---------------------------
 * Parameters:
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - ATM OAM contuinity check Parameters
 *
 * Description: This function is used to set the ATM OAM continuity
 *              check 
 * Return:  FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_OAM_CC_SetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_OAM_CC_t * info,
        NPF_uint32_t count)
{
    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_IfATM_OAM_CC_t(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_Add_Cmd,     /* comand type */ 
            Forces_IfATM_OAM_CC_Set_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_IfATM_OAM_CC_t(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;

}

/********************************************************************
 * Function:   FPPAPI_ATM_OAM_LB_SetFunc()
 * ---------------------------
 * Parameters:
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - ATM OAM loopBack Parameters
 *
 * Description: This function is used to set the ATM OAM loopback
 *              type
 * Return:  FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_OAM_LB_SetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_OAM_LB_t * info,
        NPF_uint32_t count)
  
{
    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_IfATM_OAM_LB_t(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_Add_Cmd,     /* comand type */ 
            Forces_IfATM_OAM_LB_Set_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_IfATM_OAM_LB_t(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;

}

/********************************************************************
 * Function:   FPPAPI_ATM_OAM_MonitorFunc()
 * ---------------------------
 * Parameters:
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - ATM OAM cell Monitor Parameters
 *
 * Description: This function is used to set the ATM OAM cells  
 *              monitoring function 
 * Return:  FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_OAM_MonitorFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_OAM_Mon_t * info,
        NPF_uint32_t count)
  
{
    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_IfATM_OAM_Mon_t(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_Add_Cmd,     /* comand type */ 
            Forces_IfATM_OAM_Monitor_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_IfATM_OAM_Mon_t(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;

}


/********************************************************************
 * Function:   FPPAPI_ATM_OAM_AlarmSetFunc()
 * ---------------------------
 * Parameters:
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - ATM OAM Alarm Parameters
 *
 * Description: This function is used to set the ATM OAM Alarm type
 * Return:  FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_OAM_AlarmSetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_OAM_Alarm_t * info,
        uint32_t count)
  
{
    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_IfATM_OAM_Alarm_t(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_Add_Cmd,     /* comand type */ 
            Forces_IfATM_OAM_AlarmSet_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_IfATM_OAM_Alarm_t(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;

}


/********************************************************************
 * Function:   FPPAPI_ATM_OAM_AlarmClearFunc()
 * ---------------------------
 * Parameters:
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - ATM OAM Alarm Parameters
 *
 * Description: This function is used to clear the ATM OAM Alarm type
 * Return:  FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/
FPPI_RET FPPAPI_ATM_OAM_AlarmClearFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_OAM_Alarm_t * info,
        NPF_uint32_t count)
  
{
    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_IfATM_OAM_Alarm_t(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_Delete_Cmd,     /* comand type */ 
            Forces_IfATM_OAM_AlarmClear_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_IfATM_OAM_Alarm_t(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;


}


/********************************************************************
 * Function:   FPPAPI_ATM_OAM_PM_SetFunc()
 * ---------------------------
 * Parameters:
 * FPPI_FEID - unique ID for a FE which is generated by TPlugin
 * FPPI_CORRELATOR - used to distinguish multiple invocations
 * FPPI_CBHANDLE - unique ID used to identify the callback function
 * info - ATM OAM performance monitoring Parameters
 *
 * Description: This function is used to set the ATM OAM PM properties
 * Return:  FPPI_SUCCESS, FPPI_FAIL
 ******************************************************************/

FPPI_RET FPPAPI_ATM_OAM_PM_SetFunc(
        FPPI_FEID        feid,
        FPPI_CORRELATOR correlator,
        FPPI_CBHANDLE           cbhandle,
        IfATM_OAM_PM_t * info,
        NPF_uint32_t count)

{
    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_IfATM_OAM_PM_t(info,count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */ 
            IfATM_tpHandler,    /* default handler only decaps status */
            Forces_Add_Cmd,     /* comand type */ 
            Forces_IfATM_OAM_PM_Set_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = (char *)fppcmd->buffer;
    buf = encapsulate_uint32_t(&count,buf);
    encapsulate_array_IfATM_OAM_PM_t(info,count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;

}

#endif /* _INCLUDE_ATM */
