/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/TransportPlugin/src/fppapi_cm.c,v $
 * $Revision: 1.6 $
 * $Date: 2003/11/17 16:57:16 $
 *
 * Description:
 * Contains the top level API for the transport plugin at the control
 * plane end for the Config and Management API.
 *
 ****************************************************************************/

#include <stdio.h>
#include "fppapi.h"
#include "fpppriv.h"
#include "fppencap_npf_cm.h"
#include "fppdecap_npf_cm.h"
#include "fppencap.h"
#include "fppencap_common.h"
#include "fppencap_npf_if.h"

void l2_stats_handler(
                FPPI_FEID       feid,
                uint32_t        cmd_type,
                uint32_t        block_type,
                FPPI_CBHANDLE   cbhandle,
                FPPI_CORRELATOR correlator,
                uint32_t        global_result,
                char*           data)
{
    FPPI_CONTEXT    cbcontext;
    fppapi_callback cb;

    npf_L2StatsList_t* statslist = NULL;

    statslist = (npf_L2StatsList_t*)
            malloc(sizeof(npf_L2StatsList_t));
    if (!statslist) return;

    decapsulate_npf_L2StatsList_t(statslist,data);

    cb = (fppapi_callback)pdk_callback_func(cbhandle,&cbcontext);

    cb(feid,correlator,cbcontext,statslist);

    return;
}

void l3_stats_handler(
                FPPI_FEID       feid,
                uint32_t        cmd_type,
                uint32_t        block_type,
                FPPI_CBHANDLE   cbhandle,
                FPPI_CORRELATOR correlator,
                uint32_t        global_result,
                char*           data)
{
    FPPI_CONTEXT    cbcontext;
    fppapi_callback cb;

    npf_L3StatsList_t* statslist = NULL;

    statslist = (npf_L3StatsList_t*)
            malloc(sizeof(npf_L3StatsList_t));
    if (!statslist) return;

    decapsulate_npf_L3StatsList_t(statslist,data);

    cb = (fppapi_callback)pdk_callback_func(cbhandle,&cbcontext);

    cb(feid,correlator,cbcontext,statslist);

    return;
}

void icmp_stats_handler(
                FPPI_FEID       feid,
                uint32_t        cmd_type,
                uint32_t        block_type,
                FPPI_CBHANDLE   cbhandle,
                FPPI_CORRELATOR correlator,
                uint32_t        global_result,
                char*           data)
{
    FPPI_CONTEXT    cbcontext;
    fppapi_callback cb;

    npf_ICMPStatsList_t* statslist = NULL;

    statslist = (npf_ICMPStatsList_t*)
            malloc(sizeof(npf_ICMPStatsList_t));
    if (!statslist) return;

    decapsulate_npf_ICMPStatsList_t(statslist,data);

    cb = (fppapi_callback)pdk_callback_func(cbhandle,&cbcontext);

    cb(feid,correlator,cbcontext,statslist);

    return;
}
/****************************************************************************
 * Function: FPPAPI_L2Interface_SetProperties()                             *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             port_array - array of <L2 Intf Attributes, Port ID> pairs     *
 *             port_count - the number of entries in the array              *
 *                                                                          *
 * Description: This function is used to set L2 Interface properties on     *
 *              a FE.                                                       *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_L2Interface_SetProperties(
    FPPI_FEID                feid,
    FPPI_CORRELATOR          correlator,
    FPPI_CBHANDLE            cbhandle,
    npf_L2IntfAttrs_t*       port_array,
    uint32_t                 port_count)
{
    char* buf;
    size_t size = 0;
    FPPCommand fppcmd;

    size += sizeof(uint32_t);
    size += sizeof_array_npf_L2IntfAttrs_t(port_array,port_count);

    if (!(fppcmd = newFPPCommand(
            feid,
            correlator,
            cbhandle,
            default_handler,
            Forces_Add_Cmd,
            Forces_L2_Interface_Block,
            size)))
    {
        return FPPI_FAIL;
    }

    buf = fppcmd->buffer;
    buf = encapsulate_uint32_t(&port_count,buf);
    buf = encapsulate_array_npf_L2IntfAttrs_t(port_array,port_count,buf);

    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: FPPAPI_L2Interface_GetStatistics()                             *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             port_array - array of port IDs                                *
 *             port_count - the number of entries in the array              *
 *                                                                          *
 * Description: This function is used to get L2 Interface statistics from   *
 *              a FE.                                                       *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_L2Interface_GetStatistics(
    FPPI_FEID                feid,
    FPPI_CORRELATOR          correlator,
    FPPI_CBHANDLE            cbhandle,
    FPPI_PortID*             port_array,
    NPF_IfType_t            interface,
    uint32_t                 port_count)
{
    char* buf;
    size_t size = 0;
    FPPCommand fppcmd;

    size += sizeof(uint32_t);
    size += sizeof_NPF_IfType_t(&interface);
    size += sizeof_array_FPPI_PortID(port_array,port_count);

    if (!(fppcmd = newFPPCommand(
            feid,
            correlator,
            cbhandle,
            l2_stats_handler,
            Forces_QueryStats_Cmd,
            Forces_L2_Interface_Block,
            size)))
    {
        return FPPI_FAIL;
    }

    buf = fppcmd->buffer;
    buf = encapsulate_NPF_IfType_t(&interface, buf);
    buf = encapsulate_uint32_t(&port_count,buf);
    buf = encapsulate_array_FPPI_PortID(port_array,port_count,buf);

    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: FPPAPI_L3Interface_SetProperties()                             *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             port_array - array of <L3 Intf Attributes, Port ID> pairs     *
 *             port_count - the number of entries in the array              *
 *                                                                          *
 * Description: This function is used to set L3 Interface properties on     *
 *              a FE.                                                       *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_L3Interface_SetProperties(
    FPPI_FEID          feid,
    FPPI_CORRELATOR    correlator,
    FPPI_CBHANDLE      cbhandle,
    npf_L3IntfAttrs_t* port_array,
    uint32_t           port_count)
{
    char* buf;
    size_t size = 0;
    FPPCommand fppcmd;

    size += sizeof(uint32_t);
    size += sizeof_array_npf_L3IntfAttrs_t(port_array,port_count);

    if (!(fppcmd = newFPPCommand(
            feid,
            correlator,
            cbhandle,
            default_handler,
            Forces_Add_Cmd,
            Forces_L3_Interface_Block,
            size)))
    {
        return FPPI_FAIL;
    }

    buf = fppcmd->buffer;
    buf = encapsulate_uint32_t(&port_count,buf);
    buf = encapsulate_array_npf_L3IntfAttrs_t(port_array,port_count,buf);

    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}

#ifdef _INCLUDE_IPV6

FPPI_RET FPPAPI_L3Interface_SetPropertiesIPv6(
    FPPI_FEID          feid,
    FPPI_CORRELATOR    correlator,
    FPPI_CBHANDLE      cbhandle,
    npf_L3IntfAttrs_t* port_array,
    uint32_t           port_count)
{
    char* buf;
    size_t size = 0;
    FPPCommand fppcmd;

    size += sizeof(uint32_t);
    size += sizeof_array_npf_L3IntfAttrs_t(port_array,port_count);

    if (!(fppcmd = newFPPCommand(
            feid,
            correlator,
            cbhandle,
            default_handler,
            Forces_Add_Cmd,
            Forces_L3_IPv6_Interface_Block,
            size)))
    {
        return FPPI_FAIL;
    }

    buf = fppcmd->buffer;
    buf = encapsulate_uint32_t(&port_count,buf);
    buf = encapsulate_array_npf_L3IntfAttrs_t(port_array,port_count,buf);

    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}

#endif

/****************************************************************************
 * Function: FPPAPI_L3Interface_DelProperties()                             *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             port_array - array of <L3 Intf Attributes, Port ID> pairs     *
 *             port_count - the number of entries in the array              *
 *                                                                          *
 * Description: This function is used to delete L3 Interface properties on  *
 *              a FE.                                                       *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_L3Interface_DelProperties(
    FPPI_FEID          feid,
    FPPI_CORRELATOR    correlator,
    FPPI_CBHANDLE      cbhandle,
    npf_L3IntfAttrs_t* port_array,
    uint32_t           port_count)
{
    char* buf;
    size_t size = 0;
    FPPCommand fppcmd;

    size += sizeof(uint32_t);
    size += sizeof_array_npf_L3IntfAttrs_t(port_array,port_count);

    if (!(fppcmd = newFPPCommand(
            feid,
            correlator,
            cbhandle,
            default_handler,
            Forces_Delete_Cmd,
            Forces_L3_Interface_Block,
            size)))
    {
        return FPPI_FAIL;
    }

    buf = fppcmd->buffer;
    buf = encapsulate_uint32_t(&port_count,buf);
    buf = encapsulate_array_npf_L3IntfAttrs_t(port_array,port_count,buf);

    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}

#ifdef _INCLUDE_IPV6

FPPI_RET FPPAPI_L3Interface_DelPropertiesIPv6(
    FPPI_FEID          feid,
    FPPI_CORRELATOR    correlator,
    FPPI_CBHANDLE      cbhandle,
    npf_L3IntfAttrs_t* port_array,
    uint32_t           port_count)
{
    printf ("Not support IPv6 yet\n");
    return FPPI_FAIL;
}

#endif

/****************************************************************************
 * Function: FPPAPI_L3Interface_GetStatistics()                             *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             port_array - array of port IDs                                *
 *             port_count - the number of entries in the array              *
 *                                                                          *
 * Description: This function is used to get L3 Interface statistics from   *
 *              a FE.                                                       *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_L3Interface_GetStatistics(
    FPPI_FEID                feid,
    FPPI_CORRELATOR          correlator,
    FPPI_CBHANDLE            cbhandle,
    FPPI_PortID*             port_array,
    NPF_IfType_t            interface,
    uint32_t                 port_count)
{
    char* buf;
    size_t size = 0;
    FPPCommand fppcmd;

    size += sizeof(uint32_t);
    size += sizeof_NPF_IfType_t(&interface);
    size += sizeof_array_FPPI_PortID(port_array,port_count);

    if (!(fppcmd = newFPPCommand(
            feid,
            correlator,
            cbhandle,
            l3_stats_handler,
            Forces_QueryStats_Cmd,
            Forces_L3_Interface_Block,
            size)))
    {
        return FPPI_FAIL;
    }

    buf = fppcmd->buffer;
    buf = encapsulate_NPF_IfType_t(&interface, buf);
    buf = encapsulate_uint32_t(&port_count,buf);
    buf = encapsulate_array_FPPI_PortID(port_array,port_count,buf);


    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: FPPAPI_ICMP_GetStatistics()                                    *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             port_array - array of port IDs                                *
 *             port_count - the number of entries in the array              *
 *                                                                          *
 * Description: This function is used to get ICMP statistics from a FE.     *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ICMP_GetStatistics(
    FPPI_FEID                feid,
    FPPI_CORRELATOR          correlator,
    FPPI_CBHANDLE            cbhandle,
    FPPI_PortID*             port_array,
    uint32_t                 port_count)
{
    char* buf;
    size_t size = 0;
    FPPCommand fppcmd;

    size += sizeof(uint32_t);
    size += sizeof_array_FPPI_PortID(port_array,port_count);

    if (!(fppcmd = newFPPCommand(
            feid,
            correlator,
            cbhandle,
            icmp_stats_handler,
            Forces_QueryStats_Cmd,
            Forces_ICMP_Block,
            size)))
    {
        return FPPI_FAIL;
    }

    buf = fppcmd->buffer;
    buf = encapsulate_uint32_t(&port_count,buf);
    buf = encapsulate_array_FPPI_PortID(port_array,port_count,buf);

    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}
