/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/TransportPlugin/src/fppapi_ds.c,v $
 * $Revision: 1.6 $
 * $Date: 2003/11/17 16:57:16 $
 *
 * Description:
 * Contains the top level API for the transport plugin at the control
 * plane end.
 *
 ****************************************************************************/

#include <stdio.h>
#include "fppapi.h"
#include "fpppriv.h"
#include "fppencap_common.h"
#include "fppCapabilityTypes.h"
#include "fppencap_npf_ds.h"
#include "fppdecap_npf_ds.h"

void ds_resp_handler(
                FPPI_FEID       feid,
                uint32_t        cmd_type,
                uint32_t        block_type,
                FPPI_CBHANDLE   cbhandle,
                FPPI_CORRELATOR correlator,
                uint32_t        global_result,
                char*           data)
{
    FPPI_CONTEXT    	cbcontext;
    fppapi_callback 	cb;
    FPPI_DS_Dpe_t		*resp;
    uint32_t		size;
    NPF_DS_CallbackType_t op;
    size = sizeof(FPPI_DS_Dpe_t);
    /*
     * To overcome memory alignment in vxworks all paramters 
     * encoded/decoded as uint32_t. Allocate more buffer size
    */
    size = size*2;
    resp = (FPPI_DS_Dpe_t*) PIL_AllocateMemory(size, PIL_MEM_ZERO_INIT);

    if (!resp) return;

    decapsulate_FPPI_DS_Dpe_t(resp, data);

    cb = (fppapi_callback)pdk_callback_func(cbhandle, &cbcontext);

    cb(feid, correlator, cbcontext, resp);
    return;
}

/****************************************************************************
 * Function: FPPAPI_DS_DpeOperation()  	                                    *
 * ---------------------------                                              *
 * Parameters: feid        - unique ID for a FE generated by TPlugin        *
 *             correlator  - used to distinguish multiple invocations       *
 *             cbhandle    - unique ID used to identify the callback fn     *
 *             dpeInfo     - DPE info					    *
 *                                                                          *
 * Description: Send a DiffServ Operation policy to the specified           *
 *              forwarding element.                                         *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/
FPPI_RET FPPAPI_QOS_DpeOperation(
    FPPI_FEID             feid,
    FPPI_CORRELATOR       correlator,
    FPPI_CBHANDLE         cbhandle,
    FPPI_DS_Dpe_t         *dpeInfo)
{
    char* buf;
    size_t size;
    FPPCommand fppcmd;
    size = sizeof_FPPI_DS_Dpe_t(dpeInfo);
    /*
    * To overcome all paramters encoded/decoded as uint32_t.
    * Allocate more buffer size
    */
    size = size*2;
    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */
            ds_resp_handler,    /* default handler only decaps status */
            Forces_Add_Cmd,     /* comand type */
            Forces_DS_Dpe_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = fppcmd->buffer;
    buf = encapsulate_FPPI_DS_Dpe_t(dpeInfo, buf);
    FPPCommandSend(fppcmd);
    return FPPI_SUCCESS;
}
