/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2003 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/TransportPlugin/src/fppapi_ipv4.c,v $
 * $Revision: 1.8 $
 * $Date: 2003/11/17 16:57:17 $
 *
 * Description:
 * Contains the top level API for the transport plugin at the control
 * plane end, and one callback.
 *
 ****************************************************************************/

#include <stdio.h>
#include "fppapi.h"
#include "fpppriv.h"
#include "fppencap_npf_ipv4.h"
#include "fppdecap_npf_ipv4.h"

#include "fppencap.h"

#include "fppencap_npf.h"
#include "fppdecap_capability_types.h"
#include "fppencap_capability_types.h"

/****************************************************************************
 * Function: ipv4_handler()
 * ---------------------------
 * Parameters: feid - FE the response came from
 *             cmd_type - ignored
 *             block_type - ignored
 *             cbhandle - cb to invoke
 *             correlator - correlator supplied by cp when api was invoked
 *             global_result - ignored
 *             data - pointer to buffer to decap results from
 *
 * Description: This function is called when responses come back from
 *              the FE on certain api calls.
 *
 * Return: void
 *
 ****************************************************************************/
void ipv4_handler(
                FPPI_FEID       feid,
                uint32_t        cmd_type,
                uint32_t        block_type,
                FPPI_CBHANDLE   cbhandle,
                FPPI_CORRELATOR correlator,
                uint32_t        global_result,
                char*           data)
{
    FPPI_CONTEXT    cbcontext;
    fppapi_callback cb;
    void* status = NULL;
    switch(cmd_type)
    {
       case Forces_DeleteAll_Cmd:
       case Forces_QueryProps_Cmd:
       {
          status = (NPF_IPv4UC_CallbackData_t*)
               PIL_AllocateMemory(sizeof(NPF_IPv4UC_CallbackData_t),0);
          if (!status) return;
          decapsulate_NPF_IPv4UC_CallbackData_t((
                           NPF_IPv4UC_CallbackData_t*)status, data);
          /* Give callback to User */  
          cb = (fppapi_callback)pdk_callback_func(cbhandle,&cbcontext);
          if (!cb) return;
          cb(feid,correlator,cbcontext,status);
       }
       break;
       default:
       {
          /* Except Flush and Query, we are using old structures */
          status = (npf_IPv4EntryStatusList*)
               PIL_AllocateMemory(sizeof(npf_IPv4EntryStatusList),0);
          if (!status) return;
          decapsulate_npf_IPv4EntryStatusList((
                           npf_IPv4EntryStatusList*)status,data);
         
          /* Give callback to User */  
          cb = (fppapi_callback)pdk_callback_func(cbhandle,&cbcontext);
          if (!cb) return;
          cb(feid,correlator,cbcontext,status);
       
           /* Free buffers taken so far */
          if((((npf_IPv4EntryStatusList*)status)->numEntries) && (((npf_IPv4EntryStatusList*)status)->entryList))
          {
             PIL_FreeMemory(((npf_IPv4EntryStatusList*)status)->entryList);
          }
       }
       break;
    }
    /* Finally, free the status */ 
    if(status)
       PIL_FreeMemory(status);
    return;
}

/****************************************************************************
 * Function: FPPAPI_ipv4_unicastRouteAdd()                                  *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             route_array - array of route entries to be added             *
 *             route_count - the number of entries in the array             *
 *                                                                          *
 * Description: This function is used to add IPv4 unicast route entries     *
 *              to a FE.                                                    *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/
#if 0
FPPI_RET FPPAPI_ipv4_unicastRouteAdd(
    FPPI_FEID                feid,
    FPPI_CORRELATOR          correlator,
    FPPI_CBHANDLE            cbhandle,
    npf_ipv4_RouteEntry_t*   route_array,
    uint32_t                 route_count)
{
    char* buf;
    size_t size;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_npf_ipv4_RouteEntry_t(route_array,route_count);

    if (!(fppcmd = newFPPCommand(
            feid,               /* fe to send command to */
            correlator,         /* pdk's context */
            cbhandle,           /* call when command done */
            default_handler,    /* default handler only decaps status */
            Forces_Add_Cmd,     /* comand type */
            Forces_IPv4_Block,  /* block type to send command to */
            size)))
    {
        return FPPI_FAIL;
    }
    buf = fppcmd->buffer;
    buf = (char *)encapsulate_uint32_t((uint32_t *)&route_count,buf);
    encapsulate_array_npf_ipv4_RouteEntry_t(route_array,route_count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}
#endif

/****************************************************************************
 * Function: FPPAPI_ipv4_unicastRouteDel()                                  *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             key_array - array of route keys to be deleted                *
 *             key_count - the number of keys in the array                  *
 *                                                                          *
 * Description: This function is used to delete IPv4 unicast route entries  *
 *              from a FE.                                                  *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/
#if 0
FPPI_RET FPPAPI_ipv4_unicastRouteDel(
    FPPI_FEID                feid,
    FPPI_CORRELATOR          correlator,
    FPPI_CBHANDLE            cbhandle,
    npf_ipv4_RouteKey_t*     key_array,
    uint32_t                 key_count)
{
    char* buf;
    size_t size;
    npf_ipv4_RouteEntry_t*  route_array;
    int i;
    FPPCommand fppcmd;

    route_array = (npf_ipv4_RouteEntry_t*)PIL_AllocateMemory(sizeof(npf_ipv4_RouteEntry_t)*key_count,0);
    for (i=0;i<key_count;i++)
    {
        route_array[i].data.next_hop_count = 0; 
        route_array[i].data.next_hop_list = NULL; 
        route_array[i].key = key_array[i];
    }

    size = sizeof(uint32_t);
    size += sizeof_array_npf_ipv4_RouteEntry_t(route_array,key_count);

    if (!(fppcmd = newFPPCommand(
                            feid,
                            correlator,
                            cbhandle,
                            default_handler,
                            Forces_Delete_Cmd,
                            Forces_IPv4_Block,
                            size)))
    {
        return FPPI_FAIL;
    }
    buf = fppcmd->buffer;
    buf = (char *)encapsulate_uint32_t(&key_count,buf);
    encapsulate_array_npf_ipv4_RouteEntry_t(route_array,key_count,buf);
    FPPCommandSend(fppcmd);

    free(route_array);

    return FPPI_SUCCESS;
}
#endif

/****************************************************************************
 * Function: FPPAPI_ipv4_unicastRoutePurge()                                *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *                                                                          *
 * Description: This function is used to purge IPv4 unicast route entries   *
 *              from a FE.                                                  *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/
#if 0
FPPI_RET FPPAPI_ipv4_unicastRoutePurge(
    FPPI_FEID                feid,
    FPPI_CORRELATOR          correlator,
    FPPI_CBHANDLE            cbhandle)
{
    FPPCommand fppcmd;

    if (!(fppcmd = newFPPCommand(
                            feid,
                            correlator,
                            cbhandle,
                            default_handler,
                            Forces_DeleteAll_Cmd,
                            Forces_IPv4_Block,
                            0)))
    {
        return FPPI_FAIL;
    }
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}
#endif

/****************************************************************************
 * Function: FPPAPI_ipv4_unicastAddNextHop()                                *
 * ---------------------------                                              *
 * Parameters: feid       - unique ID for a FE created by TPlugin           *
 *             correlator - used to distinguish multiple invocations        *
 *             cbhandle   - unique ID used to identify the callback fn      *
 *             nhInfo     - the number of entries in the array              *
 *                                                                          *
 * Description: This function is used to add IPv4 unicast next hop entries  *
 *              to a FE.                                                    *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastAddNextHop(
    FPPI_FEID                        feid,
    FPPI_CORRELATOR                  correlator,
    FPPI_CBHANDLE                    cbhandle,
    NPF_uint32_t                     numEntries,
    FPPI_UnicastNextHopUpdateInfo_t* nhInfo)
{
    size_t size;
    FPPCommand fppcmd;
    char* buf;

    size = sizeof(NPF_uint32_t);
    size += sizeof_array_FPPI_UnicastNextHopUpdateInfo_t(nhInfo, numEntries);

    if (!(fppcmd = newFPPCommand(
                            feid,
                            correlator,
                            cbhandle,
                            ipv4_handler,
                            Forces_Add_Cmd,
                            Forces_IPv4_NextHop_Block,
                            size)))
    {
        return FPPI_FAIL;
    }

    buf = fppcmd->buffer;
    buf = (char *)encapsulate_NPF_uint32_t((NPF_uint32_t *)&numEntries,buf);
    encapsulate_array_FPPI_UnicastNextHopUpdateInfo_t(nhInfo,numEntries, buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: FPPAPI_ipv4_unicastDelNextHop()                                *
 * ---------------------------                                              *
 * Parameters: feid       - unique ID for a FE created by TPlugin           *
 *             correlator - used to distinguish multiple invocations        *
 *             cbhandle   - unique ID used to identify the callback fn      *
 *             nhid       - user ID used to identify a prefix (n/w addr)    *
 *             numEntries - the number of entries in the array              *
 *             nhInfo     - array of next hop entries to be added           *
 *                                                                          *
 * Description: This function is used to delete IPv4 unicast next hop       *
 *              entries from an FE.                                         *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastDelNextHop(
    FPPI_FEID              feid,
    FPPI_CORRELATOR        correlator,
    FPPI_CBHANDLE          cbhandle,
    NPF_uint32_t           numEntries,
    FPPI_UnicastNextHopDeleteInfo_t*  nhDelInfo)
{
    size_t size;
    char* buf;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_FPPI_UnicastNextHopDeleteInfo_t(nhDelInfo, numEntries);
    if (!(fppcmd = newFPPCommand(
                            feid,
                            correlator,
                            cbhandle,
                            ipv4_handler,
                            Forces_Delete_Cmd,
                            Forces_IPv4_NextHop_Block,
                            size)))
    {
        return FPPI_FAIL;
    }

    buf = fppcmd->buffer;
    buf = (char *)encapsulate_NPF_uint32_t((NPF_uint32_t *)&numEntries,buf);
    encapsulate_array_FPPI_UnicastNextHopDeleteInfo_t(nhDelInfo,numEntries,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: FPPAPI_ipv4_unicastFlushNextHop()                              *
 * ---------------------------                                              *
 * Parameters: feid       - unique ID for a FE created by TPlugin           *
 *             correlator - used to distinguish multiple invocations        *
 *             cbhandle   - unique ID used to identify the callback fn      *
 *                                                                          *
 * Description: This function is used to flush all IPv4 unicast next hop    *
 *              entries from an FE.                                         *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastFlushNextHop(
    FPPI_FEID              feid,
    FPPI_CORRELATOR        correlator,
    FPPI_CBHANDLE          cbhandle,
    NPF_uint32_t           numEntries,
    FPPI_UnicastNextHopDeleteInfo_t*  nhFlushInfo)
{
    size_t size;
    char* buf = NULL;
    FPPCommand fppcmd;
    
    size = sizeof(uint32_t);
    size += sizeof_array_FPPI_UnicastNextHopDeleteInfo_t(nhFlushInfo, numEntries);

    if (!(fppcmd = newFPPCommand(
                            feid,
                            correlator,
                            cbhandle,
                            ipv4_handler,
                            Forces_DeleteAll_Cmd,
                            Forces_IPv4_NextHop_Block,
                            size)))
    {
        return FPPI_FAIL;
    }
    
    buf = fppcmd->buffer;
    buf = (char *)encapsulate_NPF_uint32_t((NPF_uint32_t *)&numEntries,buf);
    encapsulate_array_FPPI_UnicastNextHopDeleteInfo_t(nhFlushInfo,numEntries,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: FPPAPI_ipv4_unicastQueryNextHop()                              *
 * ---------------------------                                              *
 * Parameters: feid       - unique ID for a FE created by TPlugin           *
 *             correlator - used to distinguish multiple invocations        *
 *             cbhandle   - unique ID used to identify the callback fn      *
 *             nhid       - user ID used to identify a prefix (n/w addr)    *
 *                                                                          *
 * Description: This function is used to query all IPv4 unicast next hop    *
 *              entries from an FE.                                         *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastQueryNextHop(
    FPPI_FEID              feid,
    FPPI_CORRELATOR        correlator,
    FPPI_CBHANDLE          cbhandle,
    NPF_uint32_t           numEntries,
    NPF_uint32_t*          nhid)
{
    uint32_t size;
    char* buf;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_array_NPF_uint32_t(nhid, numEntries);

    if (!(fppcmd = newFPPCommand(
                            feid,
                            correlator,
                            cbhandle,
                            ipv4_handler,
                            Forces_QueryProps_Cmd,
                            Forces_IPv4_NextHop_Block,
                            size)))
    {
        return FPPI_FAIL;
    }

    buf = fppcmd->buffer;
    buf = (char *)encapsulate_NPF_uint32_t((NPF_uint32_t *)&numEntries,buf);
    encapsulate_array_NPF_uint32_t(nhid, numEntries, buf);

    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}

/***************************************************************************
 * Function: FPPAPI_ipv4_unicastAddPrefix()                                *
 * ---------------------------                                              *
 * Parameters: feid       - unique ID for a FE created by TPlugin           *
 *             correlator - used to distinguish multiple invocations        *
 *             cbhandle   - unique ID used to identify the callback fn      *
 *             prfxInfo   - array of next hop entries to be added           *
 *             numEntries - the number of entries in the array              *
 *                                                                          *
 * Description: This function is used to add IPv4 unicast prefix entries    *
 *              to an FE.                                                   *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastAddPrefix(
    FPPI_FEID                      feid,
    FPPI_CORRELATOR                correlator,
    FPPI_CBHANDLE                  cbhandle,
    NPF_uint32_t                   numEntries,
    FPPI_UnicastPrefixUpdateInfo_t*  prfxInfo)
{
    size_t size;
    char* buf;
    FPPCommand fppcmd;

    size = sizeof(NPF_uint32_t);
    size += sizeof_array_FPPI_UnicastPrefixUpdateInfo_t(prfxInfo, numEntries);

    if (!(fppcmd = newFPPCommand(
                            feid,
                            correlator,
                            cbhandle,
                            ipv4_handler,
                            Forces_Add_Cmd,
                            Forces_IPv4_Prefix_Block,
                            size)))
    {
        return FPPI_FAIL;
    }

    buf = fppcmd->buffer;
    buf = (char *)encapsulate_NPF_uint32_t((NPF_uint32_t *)&numEntries,buf);
    encapsulate_array_FPPI_UnicastPrefixUpdateInfo_t(prfxInfo,numEntries,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: FPPAPI_ipv4_unicastUpdatePrefix()                              *
 * ---------------------------                                              *
 * Parameters: feid       - unique ID for a FE created by TPlugin           *
 *             correlator - used to distinguish multiple invocations        *
 *             cbhandle   - unique ID used to identify the callback fn      *
 *             numEntries - array of next hop entries to be added           *
 *             prfxInfo   - the number of entries in the array              *
 *                                                                          *
 * Description: This function is used to update IPv4 unicast prefix entries *
 *              on an FE.                                                   *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastUpdatePrefix(
    FPPI_FEID                      feid,
    FPPI_CORRELATOR                correlator,
    FPPI_CBHANDLE                  cbhandle,
    NPF_uint32_t                   numEntries,
    NPF_IPv4UC_Prefix_t*  prfxInfo)
{
    size_t size;
    char* buf;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_NPF_IPv4UC_Prefix_t(prfxInfo);

    if (!(fppcmd = newFPPCommand(
                            feid,
                            correlator,
                            cbhandle,
                            ipv4_handler,
                            Forces_Add_Cmd,
                            Forces_IPv4_Prefix_Block,
                            size)))
    {
        return FPPI_FAIL;
    }

    buf = fppcmd->buffer;
    buf = (char *)encapsulate_uint32_t((uint32_t *)&numEntries,buf);
    encapsulate_array_NPF_IPv4UC_Prefix_t(prfxInfo,numEntries,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: FPPI_RET FPPAPI_ipv4_unicastQueryPrefix()                      *
 * ---------------------------                                              *
 * Parameters: feid       - unique ID for a FE created by TPlugin           *
 *             correlator - used to distinguish multiple invocations        *
 *             cbhandle   - unique ID used to identify the callback fn      *
 *             numEntries - array of prefix entries to be queried           *
 *             prfxInfo   - prefix entry array                              *
 *                                                                          *
 * Description: This function is used to query  IPv4 unicast prefix entries *
 *              from an FE.                                                 *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastQueryPrefix(
    FPPI_FEID                      feid,
    FPPI_CORRELATOR                correlator,
    FPPI_CBHANDLE                  cbhandle,
    NPF_uint32_t                   numEntries,
    NPF_IPv4UC_Prefix_t*           prfxInfo)
{
    size_t size;
    char* buf;
    FPPCommand fppcmd;

    size = sizeof(NPF_uint32_t);
    size += sizeof_array_NPF_IPv4UC_Prefix_t(prfxInfo, numEntries);

    if (!(fppcmd = newFPPCommand(
                            feid,
                            correlator,
                            cbhandle,
                            ipv4_handler,
                            Forces_QueryProps_Cmd,
                            Forces_IPv4_Prefix_Block,
                            size)))
    {
        return FPPI_FAIL;
    }

    buf = fppcmd->buffer;
    buf = (char *)encapsulate_NPF_uint32_t((NPF_uint32_t *)&numEntries,buf);
    encapsulate_array_NPF_IPv4UC_Prefix_t(prfxInfo,numEntries,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}


/****************************************************************************
 * Function: FPPI_RET FPPAPI_ipv4_unicastDelPrefix()                        *
 * ---------------------------                                              *
 * Parameters: feid       - unique ID for a FE created by TPlugin           *
 *             correlator - used to distinguish multiple invocations        *
 *             cbhandle   - unique ID used to identify the callback fn      *
 *             numEntries - array of next hop entries to be added           *
 *             prfxInfo   - the number of entries in the array              *
 *                                                                          *
 * Description: This function is used to delete IPv4 unicast prefix entries *
 *              from an FE.                                                 *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastDelPrefix(
    FPPI_FEID                      feid,
    FPPI_CORRELATOR                correlator,
    FPPI_CBHANDLE                  cbhandle,
    NPF_uint32_t                   numEntries,
    NPF_IPv4UC_Prefix_t*           prfxInfo)
{
    size_t size;
    char* buf;
    FPPCommand fppcmd;

    size = sizeof(NPF_uint32_t);
    size += sizeof_array_NPF_IPv4UC_Prefix_t(prfxInfo, numEntries);

    if (!(fppcmd = newFPPCommand(
                            feid,
                            correlator,
                            cbhandle,
                            ipv4_handler,
                            Forces_Delete_Cmd,
                            Forces_IPv4_Prefix_Block,
                            size)))
    {
        return FPPI_FAIL;
    }

    buf = fppcmd->buffer;
    buf = (char *)encapsulate_NPF_uint32_t((NPF_uint32_t *)&numEntries,buf);
    encapsulate_array_NPF_IPv4UC_Prefix_t(prfxInfo,numEntries,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: FPPI_RET FPPAPI_ipv4_unicastFlushPrefix()                      *
 * ---------------------------                                              *
 * Parameters: feid       - unique ID for a FE created by TPlugin           *
 *             correlator - used to distinguish multiple invocations        *
 *             cbhandle   - unique ID used to identify the callback fn      *
 *                                                                          *
 * Description: This function is used to delete all IPv4 unicast prefix     *
 *              entries from an FE.                                         *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastFlushPrefix(
    FPPI_FEID              feid,
    FPPI_CORRELATOR        correlator,
    FPPI_CBHANDLE          cbhandle)
{
    FPPCommand fppcmd;

    if (!(fppcmd = newFPPCommand(
                            feid,
                            correlator,
                            cbhandle,
                            ipv4_handler,
                            Forces_DeleteAll_Cmd,
                            Forces_IPv4_Prefix_Block,
                            0)))
    {
        return FPPI_FAIL;
    }

    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: FPPAPI_ipv4_unicastToL2Add()                                   *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID       - unique ID for a FE created by TPlugin      *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE   - unique ID used to identify the callback fn *
 *             entry_count     - the number of entries in the array         *
 *             entry_array     - array of ARP entries to be added           *
 *                                                                          *
 * Description: This function is used to add IPv4 ARP entries to a FE.      *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastToL2Add(
    FPPI_FEID                    feid,
    FPPI_CORRELATOR              correlator,
    FPPI_CBHANDLE                cbhandle,
    uint32_t                     entry_count,
    FPPI_IPv4UniNHResolEntry_t*  entry_array)
{
    size_t size;
    char* buf;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_FPPI_IPv4UniNHResolEntry_t(entry_array);

    if (!(fppcmd = newFPPCommand(
                            feid,
                            correlator,
                            cbhandle,
                            ipv4_handler,
                            Forces_Add_Cmd,
                            Forces_L2_ARP_Block,
                            size)))
    {
        return FPPI_FAIL;
    }

    buf = fppcmd->buffer;
    buf = (char *)encapsulate_uint32_t((uint32_t *)&entry_count,buf);
    encapsulate_array_FPPI_IPv4UniNHResolEntry_t(entry_array,entry_count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: FPPAPI_ipv4_unicastToL2Del()                                   *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID       - unique ID for a FE created by TPlugin      *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE   - unique ID used to identify the callback fn *
 *             entry_count     - the number of entries in the array         *
 *             entry_array     - array of ARP entries to be deleted         *
 *                                                                          *
 * Description: This function is used to delete IPv4 ARP entries from a FE. *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastToL2Del(
    FPPI_FEID                    feid,
    FPPI_CORRELATOR              correlator,
    FPPI_CBHANDLE                cbhandle,
    uint32_t                     entry_count,
    FPPI_IPv4UniNHResolEntry_t*  entry_array)
{
    size_t size;
    char* buf;
    FPPCommand fppcmd;

    size = sizeof(uint32_t);
    size += sizeof_FPPI_IPv4UniNHResolEntry_t(entry_array);

    if (!(fppcmd = newFPPCommand(
                            feid,
                            correlator,
                            cbhandle,
                            ipv4_handler,
                            Forces_Delete_Cmd,
                            Forces_L2_ARP_Block,
                            size)))
    {
        return FPPI_FAIL;
    }

    buf = fppcmd->buffer;
    buf = (char *)encapsulate_uint32_t((uint32_t *)&entry_count,buf);
    encapsulate_array_FPPI_IPv4UniNHResolEntry_t(entry_array,entry_count,buf);
    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: FPPAPI_ipv4_unicastToL2Flush()                                 *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID       - unique ID for a FE created by TPlugin      *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE   - unique ID used to identify the callback fn *
 *                                                                          *
 * Description: This function is used to delete all IPv4 ARP entries from   *
 *              an FE. *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/

FPPI_RET FPPAPI_ipv4_unicastToL2Flush(
    FPPI_FEID        feid,
    FPPI_CORRELATOR  correlator,
    FPPI_CBHANDLE    cbhandle)
{
    FPPCommand fppcmd;

    if (!(fppcmd = newFPPCommand(
                            feid,
                            correlator,
                            cbhandle,
                            default_handler,
                            Forces_DeleteAll_Cmd,
                            Forces_L2_ARP_Block,
                            0)))
    {
        return FPPI_FAIL;
    }

    FPPCommandSend(fppcmd);

    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: FPPAPI_ARP_AddEntry()                                          *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             entry_array - array of ARP entries to be added               *
 *             entry_count - the number of entries in the array             *
 *                                                                          *
 * Description: This function is used to add IPv4 ARP entries to a FE.      *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/
#if 0
FPPI_RET FPPAPI_ARP_AddEntry(
    FPPI_FEID                feid,
    FPPI_CORRELATOR          correlator,
    FPPI_CBHANDLE            cbhandle,
    npf_ipv4_ARPEntry_t*     entry_array,
    uint32_t                 entry_count)
{
    /* only the test program uses this right now */
    return FPPI_FAIL;
}
#endif
/****************************************************************************
 * Function: FPPAPI_ARP_DelEntry()                                          *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             key_array - array of ARP keys to be deleted                  *
 *             key_count - the number of keys in the array                  *
 *                                                                          *
 * Description: This function is used to delete IPv4 ARP entries from a FE. *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/
#if 0
FPPI_RET FPPAPI_ARP_DelEntry(
    FPPI_FEID                feid,
    FPPI_CORRELATOR          correlator,
    FPPI_CBHANDLE            cbhandle,
    npf_ipv4_ARPKey_t*       key_array,
    uint32_t                 key_count)
{
    /* only the test program uses this right now */
    return FPPI_FAIL;
}
#endif
/****************************************************************************
 * Function: FPPAPI_ARP_QueryEntry()                                        *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *             key_array - array of ARP keys to be queried                  *
 *             key_count - the number of keys in the array                  *
 *                                                                          *
 * Description: This function is used to query IPv4 ARP entries from a FE.  *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/
#if 0
FPPI_RET FPPAPI_ARP_QueryEntry(
        FPPI_FEID               feid,
        FPPI_CORRELATOR         correlator,
        FPPI_CBHANDLE           cbhandle,
        npf_ipv4_ARPKey_t       *key_array,
        uint32_t                key_count)
{
    /* only the test program uses this right now */
    return FPPI_FAIL;
}
#endif
/****************************************************************************
 * Function: FPPAPI_ARP_PurgeEntry()                                        *
 * ---------------------------                                              *
 * Parameters: FPPI_FEID - unique ID for a FE which is generated by TPlugin *
 *             FPPI_CORRELATOR - used to distinguish multiple invocations   *
 *             FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *                                                                          *
 * Description: This function is used to purge IPv4 ARP entries from a FE.  *
 *                                                                          *
 * Return: FPPI_SUCCESS or error code                                       *
 *                                                                          *
 ****************************************************************************/
#if 0
FPPI_RET FPPAPI_ARP_PurgeEntry(
    FPPI_FEID         feid,
    FPPI_CORRELATOR   correlator,
    FPPI_CBHANDLE     cbhandle)
{
    /* only the test program uses this right now */
    return FPPI_FAIL;
}
#endif
