/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2003 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/TransportPlugin/src/fppcallback.c,v $
 * $Revision: 1.6.2.1 $
 * $Date: 2004/03/17 09:52:01 $
 *
 * Description:
 * This file implements the callback interface defined in pdatagram.h,
 * it uses the decapsulate functions implemented in fppdecap.cc 
 *
 ****************************************************************************/

#include <stdio.h>
#include <string.h> /* for memset */
#include <sys/types.h> /* for memset */
#include <sys/socket.h> /* for memset */
#include <netinet/in.h>
#include <netinet/ip.h>
#include <arpa/inet.h>

#ifdef _VXWORKS_
#include <sockLib.h>
#endif

#include "time.h"
#ifdef FPPI_TIMING
#include "sys/time.h"
#endif

#include "fppapi_types.h"
#include "fppapi.h"
#include "fpp_common.h"
#include "pdk_callback.h"
#include "fpppriv.h"
#include "fppdecap.h"
#include "log.h"

#include "fppdecap.h"
#include "fppdecap_forces.h"
#include "fppdecap_capability_types.h"

#ifdef _INCLUDE_ATM
#include "fppdecap.h"
#include "npf_atm_types.h"
#include "fppapi_atm_types.h"
#endif /* _INCLUDE_ATM */

/****************************************************************************
 * Function: FEBind()
 * ---------------------------
 *
 * Description: Called when a new FE binds
 *
 * Return: none
 *
 ****************************************************************************/
void FEBind(uint32_t connect_id, ForCESMessage msg)
{
    FEInfo feinfo;
    forces_header_t *header;
    forces_bind_info_t bindinfo;

    FPPI_CONTEXT cbcontext;
    FPPI_CBHANDLE cbid, nextcbid;
    fppapi_callback cb = NULL;

    npf_logger_write (LOG_TPI, LOG_INFO, "Received FE Bind\n");

    header = &(msg->header);
    decapsulate_forces_bind_info_t(&bindinfo,(char*)&(msg->data));

    feinfo = newFEInfo(connect_id,&bindinfo);

    // invoke FE_BIND callback
    /* loop the CB to execute all the callback func */
    cb = (fppapi_callback) pdk_callback_firstFunc
        (FPPI_FEBind_Event, &cbcontext, &nextcbid);
    if (cb == NULL)
    {
        npf_logger_write (LOG_TPI, LOG_INFO,
            "No callbacks registered for bind event\n");
    }
    do
    {
       cbid = nextcbid;
       cb (feinfo->feid, NULL, cbcontext, (void *)connect_id);
    }
    while ((cb =
       (fppapi_callback) pdk_callback_nextFunc (cbid, &cbcontext,
                                                    &nextcbid)) != NULL);
}

/****************************************************************************
 * Function: FEUnbind()
 * ---------------------------
 *
 * Description: This is called when an FE unbinds. The Plugin removes
 *        the corresponding entry from the FE List.
 *        It also invokes the FE UnBind callback fn.
 *
 * Return: none
 *
 ****************************************************************************/
void FEUnbind (FEInfo feinfo)
{
    FPPI_CONTEXT cbcontext;
    FPPI_CBHANDLE cbid, nextcbid;
    fppapi_callback cb = NULL;

    npf_logger_write (LOG_TPI, LOG_INFO, "Received Client Close\n");

    // invoke FE_UNBIND callback
    /* loop the CB to execute all the callback func */
    cb = (fppapi_callback) pdk_callback_firstFunc
            (FPPI_FEUnBind_Event, &cbcontext, &nextcbid);
    if (cb == NULL)
    {
        npf_logger_write (LOG_TPI, LOG_INFO,
            "No callbacks registered for unbind event\n");
    }
    do
    {
       cbid = nextcbid;
       cb (feinfo->feid, NULL, cbcontext, NULL);
    }
    while ((cb =
       (fppapi_callback) pdk_callback_nextFunc (cbid, &cbcontext,
                                                &nextcbid)) != NULL);

    deleteFEInfo(feinfo);
}

/****************************************************************************
 * Function: FECapabilityResponse()                                         *
 * ---------------------------                                              *
 *                                                                          *
 * Description: 
 *                                                                          *    * Return: none                                                             *
 *                                                                          *
 ****************************************************************************/
void FECapabilityResponse (FEInfo feinfo, ForCESMessage msg)
{
    FPPI_CONTEXT cbcontext;
    FPPI_CBHANDLE cbid, nextcbid;
    fppapi_callback cb = NULL;
    ForCESMessage startmsg;
    char* buf;

    npf_logger_write (LOG_TPI, LOG_INFO, "Received Capability Response\n");

    feinfo->portlist = (forces_portlist_t*)PIL_AllocateMemory(sizeof(forces_portlist_t),0);
    feinfo->blocklist = (forces_blocklist_t*)PIL_AllocateMemory(sizeof(forces_blocklist_t),0);

    buf = (char*)&(msg->data);

    buf = (char *)decapsulate_forces_portlist_t(feinfo->portlist,buf);
    buf = (char *)decapsulate_forces_blocklist_t(feinfo->blocklist,buf);

    npf_logger_write (LOG_TPI, LOG_INFO,
                    "Received Query Response for FPPI_QUERY_FECAPS\n");


    feinfo->fecaps = (FPPI_FE_Caps*)PIL_AllocateMemory(sizeof(FPPI_FE_Caps),0);

    buf = (char *)decapsulate_FPPI_FE_Caps(feinfo->fecaps,buf);

    npf_logger_write (LOG_TPI, LOG_INFO, "FEName == %s\n",feinfo->fecaps->fe_name);

    /* stick the fe's addr in here */
    {
        int len;
        struct sockaddr_in sin;
        len  = sizeof(struct sockaddr_in);
        getpeername(feinfo->connectionid,(struct sockaddr*)&sin,&len);
        feinfo->fecaps->fe_addr = sin.sin_addr.s_addr;
    }

    /* Finally have all FE Capabilities
     * Now send callbacks
     */
    npf_logger_write (LOG_TPI, LOG_INFO,
                    "Finally Invoking Capability callbacks\n");

    cb = (fppapi_callback) pdk_callback_firstFunc
            (FPPI_FECapability_Event, &cbcontext, &nextcbid);

    if (cb == NULL)
    {
        npf_logger_write (LOG_TPI, LOG_INFO,
            "No callbacks registered for capability event\n");
    }
    do
    {
       cbid = nextcbid;
       cb (feinfo->feid, NULL, cbcontext, (void*)feinfo->fecaps);
    }
    while ((cb =
       (fppapi_callback) pdk_callback_nextFunc (cbid, &cbcontext,
                                                &nextcbid)) != NULL);

    /* Since we now have all capabilities we can tell the FE to start */
    startmsg = newForCESMessage(feinfo,FE_Start_Operation,0,0);
    ForCESMessageSend(feinfo,startmsg,0);

    /* Free allocated memory chunks here as well as from inside decap function*/
    if((feinfo->fecaps->ports_caps.port_count) && 
        (feinfo->fecaps->ports_caps.port_caps_list))
    {
       PIL_FreeMemory(feinfo->fecaps->ports_caps.port_caps_list);
    }
    if((feinfo->portlist->port_count) && (feinfo->portlist->port_array))
    {
       PIL_FreeMemory(feinfo->portlist->port_array);
    }

    /******************
    if(feinfo->blocklist->block_array)
       PIL_FreeMemory(feinfo->blocklist->block_array);
    PIL_FreeMemory(feinfo->portlist);
    PIL_FreeMemory(feinfo->blocklist);
    PIL_FreeMemory(feinfo->fecaps);
    ********************/

    return;
}

/****************************************************************************
 * Function: FEEvent()                                                      *
 * ---------------------------                                              *
 *                                                                          *
 * Description: 
 *                                                                          *    * Return: none                                                             *
 *                                                                          *
 ****************************************************************************/
void FEEvent (FEInfo feinfo, ForCESMessage msg)
{
    FPPI_CONTEXT    cbcontext;
    FPPI_CBHANDLE   cbid, nextcbid;
    fppapi_callback cb;
    NPF_CBCAT       cbtype;

    char* buf;

    void* data;
    forces_command_t cmd;

    npf_logger_write (LOG_TPI, LOG_INFO, "Received FE Event\n");

    buf = (char*)&(msg->data);
    buf = (char *)decapsulate_forces_command_t(&cmd,buf);

    switch(cmd.cmd_type)
    {
        case FORCES_EVENT_DIRECT_CONNECTED_HOST:
                if (cmd.length != sizeof(npf_fpm_ipv4_exception_t))
                {
                    npf_logger_write (LOG_TPI, LOG_ERR,
                        "FE Event data length == %d.  Expected %d\n",
                        cmd.length, sizeof(npf_fpm_ipv4_exception_t));
                }
                data = PIL_AllocateMemory(sizeof(npf_fpm_ipv4_exception_t),0);
                buf = (char *)decapsulate_uint32_t(&(((npf_fpm_ipv4_exception_t *)data)->portid),buf);
                buf = (char *)decapsulate_uint32_t(&(((npf_fpm_ipv4_exception_t *)data)->dest),buf);
                cbtype = FPPI_Directly_Connected_Host_Event;
            break;
        case FORCES_EVENT_PORT_DOWN:
        case FORCES_EVENT_PORT_UP:
            {
            /* common code */
                if (cmd.length != sizeof(FPPI_PortID))
                {
                    npf_logger_write (LOG_TPI, LOG_ERR,
                        "FE Event data length == %d.  Expected %d\n",
                        cmd.length, sizeof(FPPI_PortID));
                }
                /* would rather not malloc space (as that could be slow)
                 * but I think this way will provide a more common
                 * method for other events and commands */
                data = PIL_AllocateMemory(sizeof(uint32_t),0);
                buf = (char *)decapsulate_uint32_t((uint32_t *)data,buf);
                cbtype = FPPI_Directly_Connected_Host_Event;

            /* uncommon code, pardon the extra work with the if */
                if (cmd.cmd_type == FORCES_EVENT_PORT_DOWN)
                {
                    cbtype = FPPI_LinkDown_Event;
                }
                else
                {
                    cbtype = FPPI_LinkUp_Event;
                }
            }
            break;
        case FORCES_EVENT_PACKET_REDIRECT:
            /* this may be obsoleted by side band data channel */
            {
                cbtype = FPPI_PacketIO_Event;
                /* define later
                data = PIL_AllocateMemory(sizeof(npf_PacketList_t),0);
                buf = (char *)decapsulate_npf_PacketList_t((npf_PacketList_t*)data,
                                buf);
                */
            }
            break;
        case FORCES_EVENT_ARPENTRY_ADD:
        case FORCES_EVENT_ARPENTRY_DEL:
            {
                npf_logger_write (LOG_TPI, LOG_ERR,
                    "No idea how to hand arp entry events yet\n");
                return;
            }
            break;
        case FORCES_EVENT_CE_READY:
            {
                npf_logger_write (LOG_TPI, LOG_ERR,
                    "Received CE Ready event from FE?\n");
                return;
            }
            break;

#ifdef _INCLUDE_ATM
        case FORCES_EVENT_ATM_OAM_AIS_ALARM:
            {
                cbtype = FPPI_ATM_OAM_AisAlarm_Event;
                data = malloc(sizeof(NPF_ATM_EventData_t));
                buf = (char *)decapsulate_NPF_ATM_EventData_t((NPF_ATM_EventData_t *)data, buf);
            }
          break;
        case FORCES_EVENT_ATM_OAM_RDI_ALARM:
            {
                cbtype = FPPI_ATM_OAM_RdiAlarm_Event;
                data = malloc(sizeof(NPF_ATM_EventData_t));
                buf = (char *)decapsulate_NPF_ATM_EventData_t((NPF_ATM_EventData_t *)data, buf);
            }
         break;
        case FORCES_EVENT_ATM_OAM_LB :
            {
                cbtype = FPPI_ATM_OAM_Lb_Event;
                data = malloc(sizeof(NPF_ATM_EventData_t));
                buf = (char *)decapsulate_NPF_ATM_EventData_t((NPF_ATM_EventData_t *)data, buf);
            }
         break;
        case FORCES_EVENT_ATM_OAM_FPM_STATS :
            {
                cbtype = FPPI_ATM_OAM_FpmStats_Event;
                data = malloc(sizeof(NPF_ATM_EventData_t));
                buf = (char *)decapsulate_NPF_ATM_EventData_t((NPF_ATM_EventData_t *)data, buf);
            }
         break;
        case FORCES_EVENT_ATM_OAM_BR_STATS :
            {
                cbtype = FPPI_ATM_OAM_BrStats_Event;
                data = malloc(sizeof(NPF_ATM_EventData_t));
                buf = (char *)decapsulate_NPF_ATM_EventData_t((NPF_ATM_EventData_t *)data, buf);
            }
         break;
        case FORCES_EVENT_ATM_VCC_UP_EVENT :
            {
                cbtype = FPPI_ATM_VccUp_Event;
                data = malloc(sizeof(NPF_ATM_EventData_t));
                buf = (char *)decapsulate_NPF_ATM_EventData_t((NPF_ATM_EventData_t *)data, buf);
            }
         break;

        case FORCES_EVENT_ATM_VCC_DOWN_EVENT :
            {
                cbtype = FPPI_ATM_VccDown_Event;
                data = malloc(sizeof(NPF_ATM_EventData_t));
                buf = (char *)decapsulate_NPF_ATM_EventData_t((NPF_ATM_EventData_t *)data, buf);
            }
         break;
#endif /* _INCLUDE_ATM */

        default:
            {
                npf_logger_write (LOG_TPI, LOG_ERR,
                    "Unknown FE Event 0x%x\n",cmd.cmd_type);
                return;
            }
    }

    // invoke FPPI_FECapability_Event callback
    /* loop the CB to execute all the callback func */
    cb = (fppapi_callback) pdk_callback_firstFunc
            (cbtype, &cbcontext, &nextcbid);
    do
    {
       if(cb != NULL)
       {
          cbid = nextcbid;
          /* Invoke the callback routine */
          cb(feinfo->feid, NULL, cbcontext, data);
       }
       else
       {
         npf_logger_write (LOG_TPI, LOG_INFO,
            "No callback registered for event received \n");
       }
    }
    while ((cb =
       (fppapi_callback) pdk_callback_nextFunc (cbid, &cbcontext,
                                                &nextcbid)) != NULL);
    PIL_FreeMemory(data);
}

void FEConfigQueryResponse(
                FEInfo feinfo,
                forces_header_t* header,
                ForCESMessage msg)
{
    forces_command_status_t cmdstatus;
    char* buf;
    FPPCommand fppcmd;

    npf_logger_write (LOG_TPI, LOG_INFO, "Received FE Config Query Response\n");

    buf = (char*)&(msg->data);
    buf = (char *)decapsulate_forces_command_status_t(&cmdstatus,buf);

    fppcmd = FEInfoRemoveOutstandingCommand(feinfo,header->cmd_correlator);
    if (!fppcmd)
    {
        npf_logger_write (LOG_TPI, LOG_ERR,
                        "cmd correlator 0x%x not found in FE %d's list\n",
                        header->cmd_correlator, feinfo->feid);
        return;
    }
    if (fppcmd->cmd_type != cmdstatus.cmd_type)
    {
        npf_logger_write (LOG_TPI, LOG_ERR,
                        "this response (%d) does not match stored cmd_type (%d)\n",
                        cmdstatus.cmd_type,fppcmd->cmd_type);
    }
    else if (fppcmd->block_type != cmdstatus.block_type)
    {
        npf_logger_write (LOG_TPI, LOG_ERR,
                        "this response (%d) does not match stored block_type (%d)\n",
                        cmdstatus.block_type,fppcmd->block_type);
    }
    else if (!fppcmd->handler)
    {
        npf_logger_write (LOG_TPI, LOG_ERR,
                        "No handler for this response (%d), no idea what to do with it\n",
                        fppcmd->cmd_type);
    }
    else
    {
        fppcmd->handler(fppcmd->feinfo->feid,
                        fppcmd->cmd_type,
                        fppcmd->block_type,
                        fppcmd->fppi_cbhandle,
                        fppcmd->fppi_correlator,
                        cmdstatus.global_result,
                        buf);
    }
    PIL_FreeMemory(fppcmd);
}

void ForCES_DatagramException (
        uint32_t ex,
        uint32_t connect_id)
{

}

/****************************************************************************
 * Function: ForCES_DatagramReceive()
 * ---------------------------
 *
 * Description: This is a callback interface registered with the Interconenct
 *        which is invoked when any messages come in from the Interconnct,
 *        such as binds, or when an FE sends a Report message
 *        to the CE.
 *
 * Return: none
 *
 ****************************************************************************/

void ForCES_DatagramReceive (void* buf, uint32_t connect_id, uint32_t len)
{
    ForCESMessage msg;
    forces_header_t* header;
    FEInfo feinfo;

    msg = (ForCESMessage)buf;
    header = &(msg->header);
    header->msg_type = ntohs(header->msg_type);

    if (!(feinfo = FEInfoFindByConnectId(connect_id)))
    {
        npf_logger_write (LOG_TPI, LOG_INFO, 
            "Received Message From unknown FE 0x%x \n",connect_id);
        if (header->msg_type != FE_Bind_Request)
        {
            npf_logger_write (LOG_TPI, LOG_ERR, 
                "Message from unknown FE is not a bind: 0x%x\n",
                header->msg_type);
            deleteForCESMessage(msg);
            return;
        }
    }

    if (!ForCESCheckVersion(header->version))
    {
        npf_logger_write (LOG_TPI, LOG_ERR,
            "Invalid message from FE.  Unsupported version %d\n",
        header->version);
        deleteForCESMessage(msg);
        return;
    }

    switch (header->msg_type)
    {
        case FE_Bind_Response:
            npf_logger_write (LOG_TPI, LOG_ERR,
                "Message type %d not valid at this time\n",header->msg_type);
            break;
        case FE_Bind_Request:
            if (len != sizeof(forces_header_t) + sizeof(forces_bind_info_t))
            {
                npf_logger_write (LOG_TPI, LOG_ERR,
                    "Invalid message from FE.  Incorrect length\n");
                npf_logger_write (LOG_TPI, LOG_ERR,
                    "\texpected length: %d\n",sizeof(forces_header_t) + sizeof(forces_bind_info_t));
                npf_logger_write (LOG_TPI, LOG_ERR,
                    "\treceived length: %d\n",len);
                deleteForCESMessage(msg);
                return;
            }
            FEBind(connect_id,msg);
            break;
        case Unbind:
            FEUnbind(feinfo);
            break;
        case Heartbeat:
            npf_logger_write (LOG_TPI, LOG_INFO, "Received heart beat\n");
            PIL_EnterCriticalSection(&(feinfo->lock));
            feinfo->missedbeats = 0;
            PIL_LeaveCriticalSection(&(feinfo->lock));
            break;
        case Capability_Response:
            FECapabilityResponse(feinfo,msg);
            break;
        case Capability_Request:
        case Topology_Request:
        case Topology_Response:
        case FE_Start_Operation:
        case Config_Query_Request:
            npf_logger_write (LOG_TPI, LOG_ERR,
                "Message type %d not implemented yet\n",header->msg_type);
            break;
        case Config_Query_Response:
            FEConfigQueryResponse(feinfo,header,msg);
            break;
        case FE_Event_Notification:
            FEEvent(feinfo,msg);
            break;
        default:
            npf_logger_write (LOG_TPI, LOG_ERR,
                "Message type %d not recognized\n",header->msg_type);
                break;
    }

    deleteForCESMessage(msg);
    npf_logger_write (LOG_TPI, LOG_INFO, "Finish one job from FP\n");
}
