/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/


/*****************************************************************************
* $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/inc/pdk_cm.h,v $
* $Revision: 1.6 $
* $Date: 2003/11/17 16:57:17 $
*
* Description:
*
*    This header can only be included in internal components file
*    and the api and data struct here can _ONLY_ be used in PDK
*    components. External api and data structure should be in
*    npf_cm_api.h and npf_cm_types.h.
*
* TODO:
*    1. Add parameter name in function prototype
*
*****************************************************************************/

#ifndef _PDK_CM_H_
#define _PDK_CM_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "npf_common.h"
#include "npf_cm_types.h"
#include "npf_cm_api.h"
#include "npf_ns_types.h"
#include "pdk_cm_types.h"
#include "pdk_bd.h"
#include "rwlock.h"

/*****************************************************************************
*                                                                            *
*  -------------------------- Internal Interface --------------------------  *
*                                                                            *
*****************************************************************************/

extern NPF_RET cm_init(void);
extern NPF_RET cm_unbindFEs(void);
extern NPF_RET cm_destroy(void);
extern NPF_RET cm_newDataByType(NPF_DATATYPE, NPF_NSHDL, NPF_HANDLE *);
extern NPF_RET cm_delDataCheck(NPF_HANDLE);
extern NPF_RET cm_delDataByHandle(NPF_HANDLE);
extern NPF_RET cm_getFEHandleById(uint32_t, NPF_HANDLE *);
extern NPF_RET cm_getNSHDLByHandle(NPF_HANDLE, NPF_NSHDL *);
extern NPF_RET cm_port_get_portid(NPF_HANDLE, uint32_t *);
extern NPF_RET cm_getIFByPort(NPF_HANDLE, int *, NPF_HANDLE *);
extern NPF_RET cm_getPortByIF(NPF_HANDLE, int *, NPF_HANDLE *);
extern NPF_RET cm_getSingleFEPortByIF(NPF_HANDLE, NPF_HANDLE *,
                                      int *, NPF_HANDLE *);
extern NPF_RET cm_getPortByFE(NPF_HANDLE, int *, NPF_HANDLE *);
extern NPF_RET cm_handle_close(NPF_HANDLE);
extern NPF_RET cm_getTypeByHandle(NPF_HANDLE, NPF_DATATYPE *);
/* Topology Mgr APIs */
extern Bool cm_port_isInternal(NPF_HANDLE porth);
/* B&D APIs */
extern NPF_RET cm_sys_incr_numfes(void);
extern NPF_RET cm_fe_set_attribs(NPF_HANDLE, PDK_FE_ATTRIBS *);
extern NPF_RET cm_fe_set_caps(NPF_HANDLE, PDK_FE_CAPS *);
extern NPF_RET cm_fe_get_attribs(NPF_HANDLE, NPF_FE_ATTRIBS *);
extern NPF_RET cm_port_set_attribs(NPF_HANDLE, PDK_PORT_ATTRIBS *);
extern NPF_RET cm_port_set_caps(NPF_HANDLE, PDK_PORT_CAPS *);
/* DiffServ APIs */
extern NPF_RET cm_fe_get_caps(NPF_HANDLE feh, PDK_FE_CAPS *);
extern NPF_RET cm_sys_get_numfes(int *numfes);
/* vidd APIs */
extern NPF_RET cm_if_get_attribs(NPF_HANDLE, NPF_INTERFACE_t *);
extern NPF_RET cm_if_set_vifname(NPF_HANDLE, char *);
extern NPF_RET cm_if_set_index(NPF_HANDLE, int);
extern NPF_RET cm_fe_get_feid(NPF_HANDLE, uint32_t *);
/* MPLS & IPv4 APIs */
extern NPF_RET cm_ipr_get_attribs(NPF_HANDLE, NPF_IPROUTER_t *);
extern NPF_RET cm_port_get_attribs(NPF_HANDLE, NPF_PORT_ATTRIBS *);
extern NPF_RET cm_getIntIFByPort(NPF_HANDLE, NPF_HANDLE *);
/* GTP APIs */
extern NPF_RET cm_getSingleFEByRouter(NPF_HANDLE routerh, NPF_HANDLE *feh);

extern NPF_RET cm_router_set_attribs(	NPF_HANDLE, NPF_ROUTETABLE_t *);
extern NPF_RET cm_router_get_attribs(	NPF_HANDLE, NPF_ROUTETABLE_t *);

/* internal registeration */
extern NPF_RET pdk_category_register(   NPF_CBCAT,
                                        NPF_USERCONTEXT,
                                        NPF_GENERAL_CBFUNC,
                                        NPF_CBHANDLE*   );
extern NPF_RET pdk_category_deregister( NPF_CBHANDLE );

/* Internal event notification */
#define pdk_fe_event_register(context, func, cbhandle) \
 pdk_category_register(PDK_EVT_FE, context,(NPF_GENERAL_CBFUNC)func,cbhandle)
#define pdk_fe_event_deregister(cbhandle) pdk_category_deregister(cbhandle)
 
#define pdk_if_event_register(context, func, cbhandle) \
 pdk_category_register(PDK_EVT_IF, context,(NPF_GENERAL_CBFUNC)func,cbhandle)
#define pdk_if_event_deregister(cbhandle) pdk_category_deregister(cbhandle)

#define pdk_port_event_register(context, func, cbhandle) \
 pdk_category_register(PDK_EVT_PORT,context,(NPF_GENERAL_CBFUNC)func,cbhandle)
#define pdk_port_event_deregister(cbhandle) pdk_category_deregister(cbhandle)   
 
#define pdk_ipv4_event_register(context, func, cbhandle) \
 pdk_category_register(PDK_EVT_IPv4,context,(NPF_GENERAL_CBFUNC)func,cbhandle)
#define pdk_ipv4_event_deregister(cbhandle) pdk_category_deregister(cbhandle)

#define pdk_ipr_event_register(context, func, cbhandle) \
 pdk_category_register(PDK_EVT_IPR, context,(NPF_GENERAL_CBFUNC)func,cbhandle)
#define pdk_ipr_event_deregister(cbhandle) pdk_category_deregister(cbhandle)

#ifdef __cplusplus
}
#endif
#endif /* _PDK_CM_H_ */
