/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ControlPlane/inc/pdk_l2idm.h,v $
 * $Revision: 1.7 $
 * $Date: 2003/11/17 16:57:18 $
 *
 * Description:
 * Contains init and shutdown methods, and some helper methods called
 * from fppapi.cc.
 *
 ****************************************************************************/

#ifndef PDK_L2IDM_HDR
#define PDK_L2IDM_HDR


#ifdef __cplusplus
extern "C" {
#endif  

#include "fpp_common.h"
#include "npf_common.h"
#include "fppCapabilityTypes.h"

NPF_RET l2idm_init();

NPF_RET l2idm_shutdown();  

/*
    This set of API manages the L2ID table relevant for the NH table
    on the data plane. When the user wants to add one or more NH addrs
    for the specified NHid, the update method is called followed by
    the commit method after the data plane makes the callback to the
    CP, verifying that the action was indeed performed on the NH table
    in the DP. When the user wants to delete one or more NHaddrs for
    the specified NHid, the get method is called to get the L2IDs
    associated with the passed in NHaddrs. The CP then calls the DP
    delete method. When the DP callback verifies that the delete was
    successful on the DP, the CP calls the l2idm_delete method.

    Note that both the NH and L2 tables in the DP are affected when
    the user adds/dels a NH address.
*/

NPF_RET l2idm_UpdateForNextHopTable(
    FPPI_FEID*          feids,
    FPPI_NHID           nhid,
    uint32_t            countIn,
    NPF_IPv4Address_t*  nhAddrs,
    uint32_t*           numOldL2ids,
    FPPI_FEandL2IDs**   oldL2idList,
    uint32_t*           numNewL2ids,
    FPPI_FEandL2IDs**   newL2idList);

NPF_RET l2idm_CommitForNextHopTable(
    FPPI_NHID           nhid,
    uint32_t            numNHAddrs,
    NPF_IPv4Address_t*  nhAddrs);

NPF_RET l2idm_GetForNextHopTable(
    FPPI_NHID           nhid,
    uint32_t            numNHAddrs,
    NPF_IPv4Address_t*  nhAddrs,
    uint32_t*           numL2ids,
    FPPI_FEandL2IDs**   l2idList);

NPF_RET l2idm_DeleteForNextHopTable(
    FPPI_NHID           nhid,
    uint32_t            numNHAddrs,
    NPF_IPv4Address_t*  nhAddrs);


/* Now the analogous IPv6 methods */

NPF_RET l2idm_ipv6_UpdateForNextHopTable(
    FPPI_FEID*          feids,
    FPPI_NHID           nhid,
    uint32_t            countIn,
    NPF_IPv6Address_t*  nhAddrs,
    uint32_t*           numOldL2ids,
    FPPI_FEandL2IDs**   oldL2idList,
    uint32_t*           numNewL2ids,
    FPPI_FEandL2IDs**   newL2idList);

NPF_RET l2idm_ipv6_CommitForNextHopTable(
    FPPI_NHID           nhid,
    uint32_t            numNHAddrs,
    NPF_IPv6Address_t*  nhAddrs);

NPF_RET l2idm_ipv6_GetForNextHopTable(
    FPPI_NHID           nhid,
    uint32_t            numNHAddrs,
    NPF_IPv6Address_t*  nhAddrs,
    uint32_t*           numL2ids,
    FPPI_FEandL2IDs**   l2idList);

NPF_RET l2idm_ipv6_DeleteForNextHopTable(
    FPPI_NHID           nhid,
    uint32_t            numNHAddrs,
    NPF_IPv6Address_t*  nhAddrs);

NPF_RET l2idm_FlushNHIDList();

NPF_RET l2idm_GetNumOfL2Ids(FPPI_FEID feId, uint32_t *numL2Ids);
NPF_RET l2idm_GetL2IdList(FPPI_FEID feId, uint32_t numEntries,
                          FPPI_L2ID *l2IdArray);


/*
    This set of API manages the L2ID table relevant for the L2 table
    on the data plane. The API is similar to that for the NH table
    but without a nhid and therefore no concept of old L2ids. This
    table is affected when the user adds/dels/ a NH address and also
    when the user adds/deletes an ARP entry.
*/

NPF_RET l2idm_UpdateForL2Table(
    FPPI_FEID           feid,
    uint32_t            numNHAddrs,
    NPF_IPv4Address_t*  nhAddrs,
    uint32_t*           numNewL2ids,
    FPPI_L2ID**         newL2idList);

NPF_RET l2idm_CommitForL2Table(
    FPPI_FEID           feid,
    uint32_t            numNHAddrs,
    NPF_IPv4Address_t*  nhAddrs);

NPF_RET l2idm_GetForL2Table(
    FPPI_FEID           feid,
    uint32_t            numNHAddrs,
    NPF_IPv4Address_t*  nhAddrs,
    uint32_t*           numL2ids,
    FPPI_L2ID**         l2idList);

NPF_RET l2idm_DeleteForL2Table(
    FPPI_FEID           feid,
    uint32_t            numNHAddrs,
    NPF_IPv4Address_t*  nhAddrs);


/* Now the analogous IPv6 methods */

NPF_RET l2idm_ipv6_UpdateForL2Table(
    FPPI_FEID           feid,
    uint32_t            numNHAddrs,
    NPF_IPv6Address_t*  nhAddrs,
    uint32_t*           numNewL2ids,
    FPPI_L2ID**         newL2idList);

NPF_RET l2idm_ipv6_CommitForL2Table(
    FPPI_FEID           feid,
    uint32_t            numNHAddrs,
    NPF_IPv6Address_t*     nhAddrs);

NPF_RET l2idm_ipv6_GetForL2Table(
    FPPI_FEID           feid,
    uint32_t            numNHAddrs,
    NPF_IPv6Address_t*  nhAddrs,
    uint32_t*           numL2ids,
    FPPI_L2ID**         l2idList);

NPF_RET l2idm_ipv6_DeleteForL2Table(
    FPPI_FEID           feid,
    uint32_t            numNHAddrs,
    NPF_IPv6Address_t*  nhAddrs);

#ifdef __cplusplus
}
#endif  

#endif /* PDK_L2IDM_HDR */
