/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/
 
 
/*****************************************************************************
* $Source: /CODEBASE/ControlPlane/inc/pdk_tpm.h
* $Revision: 1.6 $
* $Date: 2003/11/17 16:57:18 $
*
* Description:
*    The interface of topology manager. A client, e.g, MPLS manager, calls
*	 tpm_getFwdPath to get the the forwarding path interms of path segment
*	 iterator, then calls tpm_first(), and tpm_next() sequentially till NULL
*	 is returned.
*
*****************************************************************************/
#ifndef _PDK_TPM_H_
#define _PDK_TPM_H_

#ifdef __cplusplus
extern "C" {
#endif  

#include "npf_common.h"
#include "npf_ns_types.h"

/* a packet forwarding path is an ordered set (DList) of path segment */
typedef struct PATH_SEGMENT_tag
{
    NPF_NSHDL lastPort; /* the path segment leaving port  */
    NPF_NSHDL nextPort; /* the path segment arriving port */

} PATHSEG;

typedef int PATHSEG_ITERATOR;

NPF_RET   tpm_init();
NPF_RET   tpm_getFwdPath(NPF_NSHDL iFE,                  /* ingress FE */
                         NPF_NSHDL eFE,                  /* egress FE  */
                         PATHSEG_ITERATOR * pathSegItr); /* forwarding path */
PATHSEG * tpm_first(PATHSEG_ITERATOR pathSegItr);
PATHSEG * tpm_next(PATHSEG_ITERATOR pathSegItr);
PATHSEG * tpm_prev(PATHSEG_ITERATOR pathSegItr);
int       tpm_count(PATHSEG_ITERATOR pathSegItr);
void      tpm_done(PATHSEG_ITERATOR pathSegItr);
NPF_RET   tpm_shutdown();  

#ifdef __cplusplus
}
#endif  

#endif /* _PDK_TPM_H_ */
