/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/****************************************************************
 *  pdk_vic.h                                                   *
 *  ---------                                                   *
 * This is the main header file for the VIDD Controller module  * 
 * of the NPF PDK                                               *
 *                                                              *
 ****************************************************************/

#ifndef __PDK_VIC_H_
#define __PDK_VIC_H_

#ifdef __cplusplus
extern "C" {
#endif

#ifdef _LINUX_
#include <linux/if_ether.h>    /* ETH_ALEN */
#else
#define ETH_ALEN 6
#endif

#include "npf_common.h"
#include "npf_cm_types.h"
#include "fppapi.h"
#include "pdk_cm.h"
#include "list.h"
#include "pil.h"
#include "pdk_callback.h"

#define IFNAME_LEN 8

/* Enum type passed back in registered VI Update
   callback identifies the event - VI Create/ VI Delete
*/
typedef enum {
    PDK_VIDD_EVT_VI_CREATE,
    PDK_VIDD_EVT_VI_DELETE 
} PDK_VIDD_EVT_CBTYPE;

/* virtual interface data required by packet handler */ 	

typedef struct _pdk_vi {
    char       vi_name[IFNAME_LEN]; /* name of virtual interface */
    IPA        ipaddr;
    IPA        mask;
    NPF_HANDLE intf_hdl; /* handle to interface object in C&M*/
    NPF_HANDLE fe_hdl; /* handle to interface's FE object in C&M*/
} pdk_vif_t;

/* for new tun vidd driver */
typedef struct PDK_VIC_tag
{
    int fd;
    char   name[IFNAME_LEN];

    uint32_t fe_id;
    uint32_t fe_ipaddr;           /* network order */
    uint32_t ipaddr;              /* network order */
    uint32_t mask;                /* network order */
    char     hwaddr[ETH_ALEN];    /* MAC address of VI */
    int      port;                /* port to which the VI connected */
    
    char     name6[IFNAME_LEN];
    IPv6A    ipv6addr;
    int      prefix_len;

    int	     mtu;
    int	     linkspeed;
    NPF_PORT_TYPE      type;

    NPF_HANDLE fe_hdl;
    NPF_HANDLE intf_hdl;

} PDK_VIC_t;

/* VIDD Controller initialization and shutdown routines
   invoked by PDK Manager
*/

extern NPF_RET vic_init();
extern NPF_RET vic_shutdown();
 
/****************************************************************
 * INTERNAL API. SHOULD NOT BE USED BY EXTERNAL APPLICATIONS    *
 * Callbacks other modules (PH) can register with VIDD          *
 ****************************************************************/
 
typedef void (*VIEVENT_CALLBACK) (
				  /* IN: */ NPF_USERCONTEXT context, 
				  /* IN: */ PDK_VIDD_EVT_CBTYPE cbtype,
				  /* IN: */ NPF_DATA* data );

 
extern NPF_RET vic_vi_event_register_callback(
                           /* IN: */ NPF_USERCONTEXT context,
                           /* IN: */ VIEVENT_CALLBACK callback,
                           /* OUT:*/ NPF_CBHANDLE* cb_handle );
 
extern NPF_RET vic_vi_event_deregister_callback( NPF_CBHANDLE cb_handle );

#ifdef __cplusplus
}
#endif

#endif /* __PDK_VIC_H_ */

