/*             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2002 Intel Corporation. All Rights Reserved.
 *
 * Description:
 * This file defines all of the types that are used by the PDK Manager 
 *
 ****************************************************************************/
#ifndef _PDK_MANAGER_H_
#define _PDK_MANAGER_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <stdio.h> 
#include "pdk_ns.h"
#include "pdk_cm.h"
#include "pdk_ipv4.h"
#include "pdk_if.h"
#include "fppapi.h"
#ifdef _INCLUDE_ATM
#include "pdk_atm.h"
#endif

/* Mailbox Message */
typedef enum {
    PDK_COMPLETIONCB,
    PDK_EVENTCB,
    PDK_IPv4COMPLETIONCB,
    PDK_IPv6COMPLETIONCB,
    PDK_IPv6TMCOMPLETIONCB,
    PDK_IPEVENTCB,
    PDK_DSCOMPLETIONCB,
    PDK_ISCOMPLETIONCB,
    PDK_MPLSCOMPLETIONCB,
    PDK_IFCOMPLETIONCB,
    PDK_IFEVENTCB,
    PDK_EXITTHREAD	
} PDK_MSGTYPE;

typedef enum{
    NPF_IPv4,
    NPF_IPv6,
    NPF_IPv6TM  /* Hack for 1.1 */
}NPF_IPType;
    
typedef struct{
    NPF_IPType type;
    union
    {
        NPF_IPv4UC_CallbackData_t ipv4Data;
#ifdef _INCLUDE_IPV6
        NPF_IPv6CallbackData_t ipv6Data;
#endif
#ifdef _INCLUDE_IPV6TM
        NPF_IPv6TMCallbackData_t ipv6TMData;
#endif
    }u;
}NPF_IPCallbackData_t;    


typedef struct{
    NPF_IPType type;
    union
    {
        NPF_IPv4UC_EventArray_t eventCBArray;
#ifdef _INCLUDE_IPV6
        NPF_IPv6_EventArray_t eventCBArrayV6;
#endif
    }u;
}NPF_IPEventCallbackData_t;

    
typedef struct ip_msg_tag {
    PDK_MSGTYPE		type;
    NPF_GENERAL_CBFUNC	cbfunc;
    NPF_userContext_t	context;
    NPF_correlator_t	correlator;	/* ptr to app's correlator */
    int			cbtype;		/* general int abstract all cbtype enum */
    NPF_IPCallbackData_t	data;		/* ptr to PDK created data,
					   should free after app cb return */
} IP_MSG_t;
    
typedef struct ip_event_msg_tag {
    PDK_MSGTYPE		type;
    NPF_GENERAL_CBFUNC	cbfunc;
    NPF_userContext_t	context;
    NPF_correlator_t	correlator;	/* ptr to app's correlator */
    int			cbtype;		/* general int abstract all cbtype enum */
    NPF_IPEventCallbackData_t	data;		/* ptr to PDK created data,
					   should free after app cb return */
} IP_EVENT_MSG_t;
    
typedef struct pdk_msg_tag {
    PDK_MSGTYPE		          type;
    NPF_GENERAL_CBFUNC	       cbfunc;
    NPF_USERCONTEXT	          context;
    NPF_CORRELATOR	          correlator; /* ptr to app's correlator */
    NPF_VERBOSITY	             verbosity;	/* app assigned verbosity */
    NPF_HANDLE	     	          handle;		/* obj handle for this msg */
    int			                cbtype;		/* general abstract all cbtype enum */
    NPF_DATA		             data;		/* ptr to PDK created data*/
    NPF_IPCallbackData_t	    ipdata;
    NPF_IPEventCallbackData_t  ipevent;
#ifdef _INCLUDE_DIFFSERV
    NPF_DS_CallbackData_t	    dsdata;
#endif
#ifdef _INCLUDE_INTSERV
    NPF_IS_CallbackData_t	    isdata;
#endif
#ifdef _INCLUDE_MPLS
    NPF_MPLS_CallbackData_t	 mplsdata;  /* MPLS Cb Data */
#endif
    NPF_IfCallbackData_t	    ifdata;
    NPF_IfEventArray_t		    ifEvent;
} PDK_MSG_t;
    
typedef void (*NPF_IP_EVENT_CBFUNC) (NPF_userContext_t, NPF_IPEventCallbackData_t);

typedef void (*NPF_IPv4_CBFUNC) ( NPF_userContext_t, NPF_correlator_t, NPF_IPv4UC_CallbackData_t);
    
#ifdef _INCLUDE_IPV6
typedef void (*NPF_IPv6_CBFUNC) ( NPF_userContext_t, NPF_correlator_t, NPF_IPv6CallbackData_t);
    
#endif

#ifdef _INCLUDE_IPV6TM
typedef void (*NPF_IPv6TM_CBFUNC) ( NPF_userContext_t, NPF_correlator_t, NPF_IPv6TMCallbackData_t);

#endif
    
/* Global Variable to mark PDK is init-ed */
extern int g_pdk_init;

/* check if pdk initialized - use for external APIs */
#define CHK_PDK_INIT() \
    if (!g_pdk_init) {\
        npf_logger_write(LOG_CM, LOG_ERR, "PDK is not initialized");\
        return NPF_COMPONENT_UNINITIALIZED;\
    }


/* Internal component APIs */
extern NPF_RET cbthread_init(void);
extern NPF_RET cbthread_destroy(void);

extern NPF_RET cbthread_pushIFAPICB(	NPF_GENERAL_CBFUNC,
					NPF_userContext_t,
					NPF_correlator_t,
					NPF_IfCallbackData_t *);

#ifdef _INCLUDE_DIFFSERV
extern NPF_RET cbthread_pushDSAPICB(	NPF_GENERAL_CBFUNC,
					NPF_userContext_t,
					NPF_correlator_t,
					NPF_DS_CallbackData_t *);
#endif

#ifdef _INCLUDE_INTSERV
extern NPF_RET cbthread_pushISAPICB(	NPF_GENERAL_CBFUNC,
					NPF_userContext_t,
					NPF_correlator_t,
					NPF_IS_CallbackData_t *);
#endif

#ifdef _INCLUDE_MPLS
extern NPF_RET cbthread_pushMPLSAPICB(	NPF_GENERAL_CBFUNC ,
					                        NPF_userContext_t ,
					                        NPF_correlator_t ,
					                        NPF_MPLS_CallbackData_t *);
#endif

extern NPF_RET cbthread_pushIPAPICB(	NPF_GENERAL_CBFUNC,
					NPF_userContext_t,
					NPF_correlator_t,
					NPF_IPCallbackData_t*);

extern NPF_RET cbthread_pushAPICB(	NPF_GENERAL_CBFUNC,
					NPF_USERCONTEXT,
					NPF_CORRELATOR,
					NPF_VERBOSITY,
					NPF_HANDLE,
					int,		/* abstract cbtype */
					NPF_DATA	);

extern NPF_RET cbthread_pushIPEventCB(	NPF_GENERAL_CBFUNC,
                                        NPF_userContext_t,
					NPF_IPEventCallbackData_t);

extern NPF_RET cbthread_pushEventCB(	NPF_GENERAL_CBFUNC,
					NPF_USERCONTEXT,
					NPF_HANDLE,
					NPF_EVENT,
					NPF_DATA	);

NPF_RET cbthread_pushIFEvent(NPF_GENERAL_CBFUNC cbfunc,
                NPF_userContext_t context,
                NPF_IfEventArray_t *event);

/* Global Variable */
#define MAXMSGNUM	60
/* Need at least 2 because one may run a BIG job and if that job
   keep making NPF calls, The COP's receiving thread will fill
   the mailbox and finally it becomes full. Once it becomes full
   COP will hang there to wait one job is being pop out from mailbox
   but since all the thread are waiting for COP receiving thread
   to clear to flag to proceed, it will become dead lock.
   The solution is make COP async, or dynamic sporn new thread */
#define MAXCBTHREADNUM	4
/* when terminate thread in cbthread_destroy */
#define EXITTIMEOUT	PIL_INFINITE_TIMEOUT
extern PilMailbox g_mailbox;
extern PilThread  g_cbthread[MAXCBTHREADNUM];

#ifdef __cplusplus
}
#endif

#endif /* _PDK_MANAGER_H_ */
