/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/

/***********************************************************************
*$Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ForwardingPlane/FPManager/src/FPManager.c,v $
*$Revision: 1.7 $
*$Date: 2003/12/01 10:35:24 $
***************************************************************************/

#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include "FPManager.h"
#include "fp_mgr_impl.h"
#include "fp_common.h"
#include "log.h"

#ifdef _INCLUDE_IPV6
extern int fpm_ipv6_mgr_init();
extern int fpm_ipv6_mgr_shutdown();
#endif

#ifdef IXASDKSTUBS
extern NPF_RET ixasdk_stubs_Init();
extern NPF_RET ixasdk_stubs_Shutdown();
#endif /* IXASDKSTUBS */

extern int mpls_test();
extern int g_running;
router_config g_current_router_config;


#ifdef IXASDKSTUBS
void fakeInterfaceUp(int portid)
{
    BENDAPI_sendEvent(FPPI_LinkUp_Event,&portid);
}
void fakeInterfaceDown(int portid)
{
    BENDAPI_sendEvent(FPPI_LinkDown_Event,&portid);
}
#endif

#ifndef COLOCATED

#ifdef _VXWORKS_

int fp_main (char *feid, char *hostid)
{

#else

int main (int argc, char** argv)
{
    char feid [FENAMELEN];    
    char hostid [48];
    char router_config[4];
#endif

    struct timeval tv;
    fd_set rset;
    int retval, ch, portid;
    
    g_running = 1;

#ifdef _VXWORKS_
    g_current_router_config = IPv4_ROUTER;

    if (strlen(feid) >= FENAMELEN) 
	{
	    printf( "FE name must be less than %d characters\n", FENAMELEN);
	    printf( "length of %s is %d\n", feid, (int)strlen(feid));
	    return (1);
	}
    printf("The FEID passed in is %s \n", feid );
    printf("The hostid passed in is %s \n", hostid );
    
#else
    if ( argc < 4 ) 
	{
	    printf("Wrong Usage - Correct Usage"
		   "- FE <feid><host><router_config>\n");
	    printf ("Possible Router Configurations - \n\t"
		    "1 - IPv4 Router\n\t"
		    "2 - IPv6 Router\n\t"
		    "3 - IPv4 & IPv6 Router\n\t"
		    "4 - DS_MPLS_LER: DiffServ with MPLS Label Edge Router \n\t"
		    "5 - DS_MPLS_LSR: DiffServ with MPLS Label Switch Router \n\t"
		    "6 - MPLS: Pure MPLS functionality \n\t");
	    
	    return (1);
	}
    if (strlen(argv[1]) >= FENAMELEN) 
	{
	    printf( "FE name must be less than %d characters\n", FENAMELEN);
	    return (1);
	}
    strcpy (feid, argv[1]);
    printf("The FEID passed in is %s \n", feid );
    strcpy( hostid, argv[2] );
    printf("The hostid passed in is %s \n", hostid );
    strcpy (router_config, argv[3]);
    g_current_router_config = atoi(router_config);

    if (g_current_router_config == DS_CORE ||
	g_current_router_config == DS_EDGE) {
	    printf ("Unsupported router configuration!\n");
	    return (-1);
    }

#endif

    npf_logger_Init();
    npf_logger_Start("stdout");
    npf_logger_SetLevel(LOG_FP_IPV4, 0 );
    npf_logger_SetLevel(LOG_FP_CM, 0 );
    npf_logger_SetLevel(LOG_FP_BOOT, 0 );
    npf_logger_SetLevel(LOG_FP_BD, 0 );
    npf_logger_SetLevel(LOG_DRS, 0 );
    npf_logger_SetLevel(LOG_FP_QOS, 0 );
    npf_logger_SetLevel(LOG_PH, 0 );
    npf_logger_SetLevel(LOG_TPI, 0 );
    npf_logger_SetLevel(LOG_IXA_STUBS, 0 );
    npf_logger_SetVerbosity(LOG_VERB_TIME);
    
    npf_logger_write(LOG_FP_BOOT, LOG_TRACE, "Entering FP Manager");
    if ((pdk_fpmgr_init(feid,hostid))== FALSE)
	{        
	    npf_logger_write(LOG_FP_BOOT, LOG_INFO, 
			     "Initialization of FP Modules in FP Manager"
			     "failed \n");
	    return (-1);
	}
    if ((pdk_fpmgr_run(feid, hostid)) == FALSE)
	{        
	    npf_logger_write(LOG_FP_BOOT, LOG_INFO, 
			     "Start of FP Modules in FP Manager failed \n");
	    return (-1);
	}

    while (g_running)
    {
        tv.tv_sec = 3;
        tv.tv_usec = 0;

#ifdef IXASDKSTUBS
        FD_ZERO(&rset);
        FD_SET(0,&rset);
#endif

        retval = select(1,&rset,NULL,NULL,&tv);

#ifdef IXASDKSTUBS
        if (retval)
        {
            ch = getchar();
            switch (ch)
            {
                case 'u':
                    portid = getchar() - '0';
                    fakeInterfaceUp(portid);
                    break;
                case 'd':
                    portid = getchar() - '0';
                    fakeInterfaceDown(portid);
                    break;
            }
        }
#endif
    }
    if ((pdk_fpmgr_shutdown()) == FALSE)
	printf("Shutdown of FP Modules in FPManager failed \n");

    return 0;
}

#else

int fp_main (char *feid, char *hostid)
{
    npf_logger_Init();
//    npf_logger_Start("stdout");
    npf_logger_SetLevel(LOG_FP_IPV4, LOG_TRACE | LOG_INFO | LOG_ERR | LOG_CRIT);
    npf_logger_SetLevel(LOG_FP_CM, LOG_TRACE | LOG_INFO | LOG_ERR | LOG_CRIT);
    npf_logger_SetLevel(LOG_FP_BOOT, LOG_TRACE | LOG_INFO | LOG_ERR | LOG_CRIT);
    npf_logger_SetLevel(LOG_FP_BD, LOG_TRACE | LOG_INFO | LOG_ERR | LOG_CRIT);
    npf_logger_SetLevel(LOG_DRS, LOG_TRACE | LOG_INFO | LOG_ERR | LOG_CRIT);
    npf_logger_SetLevel(LOG_FP_QOS, LOG_TRACE | LOG_INFO | LOG_ERR | LOG_CRIT);
#ifdef IXASDKSTUBS
    npf_logger_SetLevel(LOG_IXA_STUBS, LOG_TRACE | LOG_INFO | LOG_ERR | LOG_CRIT);
#endif
    npf_logger_SetVerbosity(LOG_VERB_TIME);
    
    npf_logger_write(LOG_FP_BOOT, LOG_TRACE, "Entering FP Manager");
    
    /* TODO: set the default router config is IPv4_ROUTER */
    g_current_router_config = IPv4_ROUTER;

    if (strlen(feid) >= FENAMELEN) 
	{
	    printf( "FE name must be less than %d characters\n", FENAMELEN);
	    printf( "length of %s is %d\n", feid, (int)strlen(feid));
	    return (1);
	}
    printf("The FEID passed in is %s \n", feid );
    printf("The hostid passed in is %s \n", hostid );
    
    npf_logger_write(LOG_FP_BOOT, LOG_INFO, 
		"The FEID passed in is %s \n", feid );
    npf_logger_write(LOG_FP_BOOT, LOG_INFO, 
		"The hostid passed in is %s \n", hostid );

    if ((pdk_fpmgr_init(feid,hostid))== FALSE)
	{        
	    npf_logger_write(LOG_FP_BOOT, LOG_INFO, 
			     "Initialization of FP Modules in FP Manager"
			     "failed \n");
	    return (-1);
	}
    if ((pdk_fpmgr_run(feid, hostid)) == FALSE)
	{        
	    npf_logger_write(LOG_FP_BOOT, LOG_INFO, 
			     "Start of FP Modules in FP Manager failed \n");
	    return (-1);
	}
    return 0;
}    
#endif

int pdk_fpmgr_init(char* feid, char* hostid)
{ 
    int errtype;

	/*set up logger*/
#ifndef COLOCATED
    PIL_Init();
    npf_logger_Init();
#endif

    errtype = pdk_callback_init();
    if (errtype != NPF_SUCCESS)
    {
        npf_logger_write(LOG_FP_BOOT, LOG_ERR, 
			"pdk_callback Library Initialization failed \n");
    	return (FALSE);
    }

    
    errtype = BENDAPI_Initialize();
    if (errtype != FPPI_SUCCESS)
    {
	   npf_logger_write(LOG_FP_BOOT, LOG_ERR, 
			"Backend API Initialization failed \n");
    	return (FALSE);
    }

    /* Call into B&D to register all Bind/Unbind callbacks */
    errtype = fpm_bd_mgr_init();
    if (errtype != FPPI_SUCCESS)
    {
	    printf ("Binding Discovery Initialization Failed \n");
	    return (FALSE);
    }

    /* perform any platform specific initializations
       - Register FPModule as a task in OMS Namespace
     */
    errtype = fpmgr_init ();

#ifndef _VXWORKS_
#ifndef IXASDKSTUBS
    errtype = sh_init (hostid);
#endif
#endif

    /* ADD ALL INDIVIDUAL MGR INITIALIZATIONS HERE */
    /* Initialize all API implementations, etc. */
    errtype = fpm_cm_mgr_init();
    errtype = fpm_ipv4_mgr_init ();
#ifdef _INCLUDE_ATM
    errtype = fpm_atm_conn_mgr_init ();
#endif
    errtype = fpm_event_mgr_init ();
#ifdef _INCLUDE_IPV6TM
    errtype = fpm_ipv6tm_mgr_init ();
#endif
#ifdef _INCLUDE_IPV6
    errtype = fpm_ipv6_mgr_init ();
#endif
#ifndef COLOCATED
#ifdef _VXWORKS_
    errtype = fpm_pkthndlr_mgr_init(hostid);
#endif
#endif

    /*
    errtype = fpm_event_mgr_init ();
    */

#ifdef _INCLUDE_FLOWID
    //errtype = fpm_fid_mgr_init ();
#endif
#ifdef _INCLUDE_DIFFSERV
    errtype = fpm_qos_mgr_init ();
#endif
#ifdef _INCLUDE_MPLS
    errtype = fpm_mpls_mgr_init ();
#endif


	/*initialize the ixasdk simulation stubs */
#ifdef IXASDKSTUBS
	ixasdk_stubs_Init();
#endif /* IXASDKSTUBS */
    return (TRUE);
}

int pdk_fpmgr_run(char* feid , char* hostid)
{
    int errtype; 
    
    npf_logger_write(LOG_FP_BOOT, LOG_TRACE, 
		"Entering pdk_fpmgr_run");
    errtype = fpm_bd_mgr_start(feid, hostid);
    if (errtype != FPPI_SUCCESS)
    {
		npf_logger_write(LOG_FP_BOOT, LOG_ERR, 
			"Binding to CE failed\n");
		return (FALSE);
    }

    return (TRUE);
}

int pdk_fpmgr_shutdown()
{
    int errtype;
    
     npf_logger_write(LOG_FP_BOOT, LOG_TRACE, 
			"Entering pdk_fpmgr_shutdown");
    /* shutdown all managers */
    //fpm_event_mgr_shutdown ();
    
    fpm_cm_mgr_shutdown ();
    
    /* perform platform-specific shutdowns */
    fpmgr_shutdown ();
    
    /* shutdown B&D */
    fpm_bd_mgr_shutdown();
    
#ifndef COLOCATED
#ifdef _VXWORKS_
    fpm_pkthndlr_mgr_shutdown();
#endif
#endif

    fpm_ipv4_mgr_shutdown();
#ifdef _INCLUDE_ATM
    fpm_atm_mgr_shutdown();
#endif
#ifdef _INCLUDE_IPV6TM
    fpm_ipv6tm_mgr_shutdown();
#endif
#ifdef _INCLUDE_IPV6
    fpm_ipv6_mgr_shutdown();
#endif
    /*fpm_arp_mgr_shutdown();*/

#ifdef _INCLUDE_MPLS
    fpm_mpls_mgr_shutdown();
#endif
#ifdef _INCLUDE_DIFFSERV
    fpm_qos_mgr_shutdown();
#endif

	/*initialize the ixasdk simulation stubs */
#ifdef IXASDKSTUBS
	ixasdk_stubs_Shutdown();
#endif /* IXASDKSTUBS */
    /* shutdown Backend API */
    errtype = BENDAPI_Shutdown();
    
    if (errtype != FPPI_SUCCESS )
    {
	    npf_logger_write(LOG_FP_BOOT, LOG_ERR, 
			"Shutdown of FP Manager failed \n");
	    return (FALSE);
    }
    //pdk_callback_destroy();
	npf_logger_Stop();
    return (TRUE);
}
