/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2002 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ForwardingPlane/FPModule/ATMManager/inc/atm_mgr_impl.h,v $
 * $Revision: 1.2 $ 
 * $Date: 2003/11/17 16:57:20 $
 *
 * Description:
 * This file defines the external API which applications use to access
 * ATM functionality.
 *
 ****************************************************************************/
#ifndef __ATM_MGR_H__
#define __ATM_MGR_H__

#ifdef __cplusplus
extern "C" {
#endif

#define ATM_AAL2_ID_HASH_LIST_SIZE 11
#define ATM_VCC_ADDR_HASH_LIST_SIZE 11
#define ATM_PORT_ID_HASH_LIST_SIZE 11

typedef struct
{
		NPF_VccAddr_t  connection;     /* VPI/VCI */
		NPF_uint32_t	portId;			 /* Port Id */
		NPF_uint16_t	vccHandle;		 /* Vcc Addr Handle */
		NPF_IfHandle_t	upperIfHandle;	 /* Upper Interface Handle */
		NPF_IfAAL_t		aalType;			 /* AAL Type */
		PdkHashCb aal2ChanIdHashList;  /* Aal2 Channel Hash List */
}limePointAtmVccHashInfo_t;

typedef struct
{
		NPF_uint16_t   aal2Cid;        /* AAL2 Channel Id */
		NPF_uint16_t	aal2Handle;		 /* AAL2 Handle */
}limePointAtmAal2HashInfo_t;

typedef struct
{
		NPF_uint32_t	portId;			 /* Port Id */
		NPF_uint16_t	portHandle;		 /* Port Handle */
		DList				vccHashInfoList;/* List of Vccs mapped on this port */
}limePointAtmPortHashInfo_t;


typedef struct
{
		NPF_uint32_t	portId;			 /* Port Id */
		FPPI_CBCORRELATOR corr;
}limePointPortCorrelator_t;	

typedef struct
{
		NPF_VccAddr_t  connection;     /* VPI/VCI */
		NPF_uint32_t	portId;			 /* Port Id */
		FPPI_CBCORRELATOR corr;
		NPF_IfAAL_t		aalType;			 /* AAL Type */
}limePointVccCorrelator_t;	

typedef struct
{
		NPF_VccAddr_t  connection;     /* VPI/VCI */
		NPF_uint32_t	portId;			 /* Port Id */
		NPF_uint16_t   aal2Cid;        /* AAL2 Channel Id */
		FPPI_CBCORRELATOR corr;
}limePointAal2Correlator_t;	

extern void fpmIfATM_shutdownFeLimePoint(void );
extern void fpmIfATM_FreeVccHashMemory(void *mem);
extern void fpmIfATM_FreePortHashMemory(void *mem);
extern NPF_RET fpmIfATM_GetVcHandle(NPF_VccAddr_t vccAddr, NPF_uint32_t portId, ix_cc_atmsar_vc_handle_t *vcHandle, NPF_IfAAL_t *aalType);
extern PdkHashComp VccAddrCompareFunc(void *data, void *key);
extern PdkHashComp Aal2ChanIdCompareFunc(void *data, void *key);
extern PdkHashComp PortIdCompareFunc(void *data, void *key);
extern NPF_uint32_t AtmHashIndexByVccAddr(void *key);
extern NPF_uint32_t AtmHashIndexByAal2ChanId(void *key);
extern NPF_uint32_t AtmHashIndexByPortId(void *key);
extern void fpmIfATM_initFeLimePoint(void );
extern void fpmIfATM_IfAttrSetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_IfDelFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_IfEnableFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_IfDisableFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_IfGenStatsGetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_VccSetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_VccBindFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_VccDelFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_VccEnableFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_VccDisableFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_VccOperStatusGetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_VccStatsGetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_AAL2_ChannelSetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_AAL2_ChannelDelFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_ConnectionXcSetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_ConnectionXcDelFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_OAM_CP_SetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_OAM_CC_SetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_OAM_LB_SetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_OAM_MonitorFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_OAM_AlarmSetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_OAM_AlarmClearFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_OAM_PM_SetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);
extern void fpmIfATM_OAM_PM_SetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,FPPI_CONTEXT context,void *response_data);


extern ix_error LimePointVccCreateCBFunc(ix_error arg_Result,
								ix_cc_atmsar_vc_handle_t arg_hVc,void *arg_pContext);
extern ix_error LimePointVccUpdateCBFunc(ix_error arg_Result,
								ix_cc_atmsar_vc_handle_t arg_hVc,void *arg_pContext);
extern ix_error LimePointVccDelCBFunc(ix_error arg_Result,void *arg_pContext);
extern ix_error LimePointVccStatsGetCBFunc(ix_error arg_Result,
					ix_cc_atmsar_vc_stats_t *arg_pLookupResult,	void *arg_pContext);
extern ix_error LimePointAal2ChanSetCBFunc(ix_error arg_Result,
					ix_cc_atmsar_cid_handle_t port,void *arg_pContext);
extern ix_error LimePointAal2ChanDelCBFunc(ix_error arg_Result,void *arg_pContext);
extern ix_error LimePointPortCreateCBFunc(ix_error arg_Result,
					ix_cc_atmsar_port_handle_t port,void *arg_pContext);
extern ix_error LimePointPortDeleteCBFunc(ix_error arg_Result,void *arg_pContext);
extern ix_error LimePointPortStatsGetCBFunc(ix_error arg_Result,
					ix_cc_atmsar_port_stats_t *arg_pLookupResult,void *arg_pContext);
		
extern ix_error LimePointDummyRemoveCBFunc(ix_error arg_Result,void *arg_pContext);

#ifdef __cplusplus
}
#endif

#endif /* __ATM_MGR_H__ */

			
			
			
			
			 
			 


					
			

			
			
			
