/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2002 Intel Corporation. All Rights Reserved.
*****************************************************************************/

#include <assert.h>
#include <string.h>
#include "npf_common.h"
#include "npf_cm_types.h"
#include "pdk_cm_types.h"
#include "pdk_callback.h"
#include "pdk_callback_types.h"
#include "pil.h"
#include "log.h"
#include "fpp_common.h"
#include "list.h"
#include "npf_if.h"
#include "pdk_hash.h"
#include "backendapi.h"
#include "npf_atm_types.h"
#include "fppapi_atm_types.h"

#include "ix_cc_error.h"
#include "cc/ix_cc_atmsar.h"
#include "cc/ix_cc_fpm_core.h"
#include "cc/llc_snap_decap.h"
#include "cc/ix_cc_fpm_core.h"
#include "cc/ix_cc_atmrx_cci.h"

#include "atm_mgr.h"
#include "atm_mgr_translator.h"
#include "atm_mgr_impl.h"
/* Callback Handles - Callbacks registered with Back end API */
extern FPPI_CBHANDLE fppiIfAtmCbHandle[NPF_IF_ATM_MAX_CB_TYPES];


/* Contexts of callbacks registered with BEND API */
extern FPPI_CONTEXT fppiIfAtmContext[NPF_IF_ATM_MAX_CB_TYPES];


static PdkHashCb vccAddrHashList;
static PdkHashCb portIdHashList;


/*********************************************************************
*
*   Function: VccAddrCompareFunc 
*
*   Parameters:
*			data - Actual Data stored in Hash List 
*			key - Key to use for selecting the data 
*
*   Description:
*			This function compares the NPF_VccAddr_t structures stored in
*			the data and key parameters passed to it.
*
*
*   return : 
*			PDK_HASH_EQUAL		- Both are equal
*			PDK_HASH_DIFF		- Mismatch 
*
*****************************************************************************/
PdkHashComp VccAddrCompareFunc(void *data, void *key)
{

	IfATM_VccAddr_t *vccAddrInfo = (IfATM_VccAddr_t *)key;
	limePointAtmVccHashInfo_t *hashInfo = (limePointAtmVccHashInfo_t *)data;

	/* Compare the Vcc Addr */
	if( ((hashInfo->connection).VPI == (vccAddrInfo->vccAddr).VPI ) &&
		 ((hashInfo->connection).VCI == (vccAddrInfo->vccAddr).VCI) &&
			(hashInfo->portId == vccAddrInfo->portId) )
		return (PDK_HASH_EQUAL);
	else
		return (PDK_HASH_DIFF);

} /* End of VccAddrCompareFunc */

/*********************************************************************
*
*   Function: Aal2ChanIdCompareFunc 
*
*   Parameters:
*			data - Actual Data stored in Hash List 
*			key - Key to use for selecting the data 
*
*   Description:
*			This function compares the NPF_AAL2_ConnId_t structures stored in
*			the data and key parameters passed to it.
*
*
*   return : 
*			PDK_HASH_EQUAL		- Both are equal
*			PDK_HASH_DIFF		- Mismatch 
*
*****************************************************************************/
PdkHashComp Aal2ChanIdCompareFunc(void *data, void *key)
{

	limePointAtmAal2HashInfo_t *hashInfo = (limePointAtmAal2HashInfo_t *)data;
	NPF_uint16_t *aal2Cid = (NPF_uint16_t *)key;

	/* Compare the AAL2 Channel Id */
	if(hashInfo->aal2Cid == *aal2Cid )
		return PDK_HASH_EQUAL;
	else
		return PDK_HASH_DIFF;

}/* End of Aal2ChanIdCompareFunc */

/*********************************************************************
*
*   Function: PortIdCompareFunc 
*
*   Parameters:
*			data - Actual Data stored in Hash List 
*			key - Key to use for selecting the data 
*
*   Description:
*			This function compares the NPF_uint32_t structures stored in
*			the data and key parameters passed to it.
*
*
*   return : 
*			PDK_HASH_EQUAL		- Both are equal
*			PDK_HASH_DIFF		- Mismatch 
*
*****************************************************************************/
PdkHashComp PortIdCompareFunc(void *data, void *key)
{

	limePointAtmPortHashInfo_t *hashInfo = (limePointAtmPortHashInfo_t *)data;
	NPF_uint32_t *portId = (NPF_uint32_t *)key;

	/* Compare the AAL2 Channel Id */
	if(hashInfo->portId == *portId )
		return PDK_HASH_EQUAL;
	else
		return PDK_HASH_DIFF;

}/* End of PortIdCompareFunc */



/*********************************************************************
*
*   Function: AtmHashIndexByVccAddr 
*
*   Parameters:
*			key - Key to use for computing the Hash Index 
*
*   Description:
*			This function computes the hash list index, by adding all the 
*			octets of the key parameter and dividing it by the hash list size.  
*
*
*   return : 
*			The Hash List Index
*
*****************************************************************************/
NPF_uint32_t AtmHashIndexByVccAddr(void *key)
{

	NPF_uint32_t index = 0;
	NPF_uint16_t keylen = sizeof(IfATM_VccAddr_t);
	NPF_uint8_t *temp = (NPF_uint8_t *)key;

	while(keylen --)
	{
		index += (NPF_uint32_t)*temp++;
	}

	index %= ATM_VCC_ADDR_HASH_LIST_SIZE;

	return index;

} /* End of AtmHashIndexByVccAddr */
	


/*********************************************************************
*
*   Function: AtmHashIndexByAal2ChanId 
*
*   Parameters:
*			key - Key to use for computing the Hash Index 
*
*   Description:
*			This function computes the hash list index, by adding all the 
*			octets of the key parameter and dividing it by the hash list size.  
*
*
*   return : 
*			The Hash List Index
*
*****************************************************************************/
NPF_uint32_t AtmHashIndexByAal2ChanId(void *key)
{

	NPF_uint32_t index = 0;
	NPF_uint16_t keylen = sizeof(NPF_uint16_t);
	NPF_uint8_t *temp = (NPF_uint8_t *)key;

	while(keylen --)
	{
		index += (NPF_uint32_t)*temp++;
	}

	index %= ATM_AAL2_ID_HASH_LIST_SIZE;

	return index;

} /* End of AtmHashIndexByAal2ChanId */
	

/*********************************************************************
*
*   Function: AtmHashIndexByPortId 
*
*   Parameters:
*			key - Key to use for computing the Hash Index 
*
*   Description:
*			This function computes the hash list index, by adding all the 
*			octets of the key parameter and dividing it by the hash list size.  
*
*
*   return : 
*			The Hash List Index
*
*****************************************************************************/
NPF_uint32_t AtmHashIndexByPortId(void *key)
{

	NPF_uint32_t index = 0;
	NPF_uint16_t keylen = sizeof(NPF_uint32_t);
	NPF_uint8_t *temp = (NPF_uint8_t *)key;

	while(keylen --)
	{
		index += (NPF_uint32_t)*temp++;
	}

	index %= ATM_PORT_ID_HASH_LIST_SIZE;

	return index;

} /* End of AtmHashIndexByPortId */

/*********************************************************************
*
*   Function: fpmIfATM_FreeVccHashMemory 
*
*   Parameters:
*
*   Description:
*			This function Frees the memory associated with a Vcc Hash Node. 
*
*   return : 
*
*
*****************************************************************************/
void fpmIfATM_FreeVccHashMemory(void *mem)
{
	limePointAtmVccHashInfo_t *vccHashInfo = NULL;
	
	if(mem == NULL)
		return;
	
	vccHashInfo = (limePointAtmVccHashInfo_t *)mem;

	/* Destroy the AAL2 hash list */
	if(pdkHashDestroy(&(vccHashInfo->aal2ChanIdHashList))!= PDK_HASH_OK)
	{
	}

	/* Free the memory Allocated for the Vcc Hash Info node */
	PIL_FreeMemory(vccHashInfo);					
					
	
	return;
}/* End of fpmIfATM_FreeVccHashMemory */ 


/*********************************************************************
*
*   Function: fpmIfATM_FreePortHashMemory 
*
*   Parameters:
*
*   Description:
*			This function Frees the memory associated with a Port Hash Node. 
*
*   return : 
*
*
*****************************************************************************/
void fpmIfATM_FreePortHashMemory(void *mem)
{
	limePointAtmPortHashInfo_t *portHashInfo = NULL;
	limePointAtmVccHashInfo_t *vccHashInfo = NULL;
	DListItr *itr = NULL;

	if(mem == NULL)
		return;
	
	portHashInfo = (limePointAtmPortHashInfo_t *)mem;

	/* Destroy the VCC  list */
	/* Create the iterator */
	if ((itr = npf_list_itrCreate((DList *)&(portHashInfo->vccHashInfoList))) != NULL)
	{
		npf_list_itrFirst(itr);
		do
		{
			/* remove the vcc Hash info from the list */
			npf_list_itrRemove(itr,(void **)&vccHashInfo);

		}while (npf_list_itrNext(itr) != -1);
	
		/* Delete the iterator that was created */	
		npf_list_itrDelete(itr);
	}


	/* Free the memory Allocated for the Vcc Hash Info node */
	PIL_FreeMemory(portHashInfo);					
					
	return;
}/* End of fpmIfATM_FreePortHashMemory */ 

/*********************************************************************
*
*   Function: fpmIfATM_GetVcHandle 
*
*   Parameters:
*
*   Description:
*			This function Frees the memory associated with a Port Hash Node. 
*
*   return : 
*
*
*****************************************************************************/
NPF_RET fpmIfATM_GetVcHandle(NPF_VccAddr_t vccAddr, NPF_uint32_t portId, ix_cc_atmsar_vc_handle_t *vcHandle,NPF_IfAAL_t *aalType )
{
	IfATM_VccAddr_t recvVccAddr; 
	PdkHashRet hashRet = PDK_HASH_OK;
	limePointAtmVccHashInfo_t *hashInfo = NULL;

	/* fill up the VccAddr structure */
	memset(&recvVccAddr, 0,sizeof(IfATM_VccAddr_t));
	recvVccAddr.vccAddr = vccAddr;
	recvVccAddr.portId =  portId;

	/* Get the Vcc Handle From the vccAddr */
	hashRet = pdkHashFind(&vccAddrHashList,(void *)&recvVccAddr,(void **)&hashInfo);
	if(hashRet != PDK_HASH_OK)
	{   
			
		return NPF_IF_E_INVALID_PARAM;
	}

	*vcHandle = hashInfo->vccHandle;
	*aalType = hashInfo->aalType;
	

	return NPF_SUCCESS;
					
}/* End of fpmIfATM_GetVcHandle */ 





/*********************************************************************
*
*   Function: fpmIfATM_initFeLimePoint 
*
*   Parameters:
*
*   Description:
*			This function initializes the Hash Lists used for Lime Point. 
*			This function also initializes the Core Components used for Lime Point
*
*   return : 
*
*
*****************************************************************************/
void fpmIfATM_initFeLimePoint(void )
{

	/* Initialize the Vcc Addr Hash lists */
	if(pdkHashInit(&vccAddrHashList,ATM_VCC_ADDR_HASH_LIST_SIZE,
				(PdkHashCompFn)VccAddrCompareFunc,(PdkHashIndexFn)AtmHashIndexByVccAddr,
				(PdkHashFreeFn)fpmIfATM_FreeVccHashMemory)
				!= PDK_HASH_OK)
	{
	}
					
	/* Initialize the PortId Hash lists */
	if(pdkHashInit(&portIdHashList,ATM_PORT_ID_HASH_LIST_SIZE,
				(PdkHashCompFn)PortIdCompareFunc,(PdkHashIndexFn)AtmHashIndexByPortId,
				(PdkHashFreeFn)fpmIfATM_FreePortHashMemory)
				!= PDK_HASH_OK)
	{
	}
					
	
	return;
}/* End of fpmIfATM_initFeLimePoint */ 

/*********************************************************************
*
*   Function: fpmIfATM_shutdownFeLimePoint 
*
*   Parameters:
*
*   Description:
*			This function destroys the Hash Lists used for Lime Point. 
*
*   return : 
*
*
*****************************************************************************/
void fpmIfATM_shutdownFeLimePoint(void )
{


	if(pdkHashDestroy(&vccAddrHashList)!= PDK_HASH_OK)
	{
	}
					
	/* Destroy the Port Id Hash lists */
	if(pdkHashDestroy(&portIdHashList)!= PDK_HASH_OK)
	{
	}

					
	return;
}/* End of fpmIfATM_shutdownFeLimePoint */ 

/*********************************************************************
*
*   Function: fpmIfATM_IfAttrSetFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to set the attributes of an 
*			ATM Interface.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_IfAttrSetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	IfATM_IfAtmAttr_t *attr = NULL;
	ix_cc_atmsar_port_descr_t portDesc;
	ix_error ret = IX_SUCCESS;
	limePointPortCorrelator_t *portCorr = NULL;

	assert(fpmIfATM_IfAttrSetFuncLimePoint);

	/* Get the ATM attributes */
	attr = (IfATM_IfAtmAttr_t *)response_data;

	/* Fill up the port descriptor fields */
	portDesc.portId = (attr->atmAttr).port;

	/* Fill up the port Speed to be passed to Limepoint CC */	
	if(attr->portSpeed <= 64)
		portDesc.bandwidth = IX_CC_ATMSAR_PORT_BANDWIDTH_DS0; 
	else if( (attr->portSpeed > 64) && (attr->portSpeed <= 155000))
		portDesc.bandwidth = IX_CC_ATMSAR_PORT_BANDWIDTH_OC3; 
	else if( (attr->portSpeed > 155000) && (attr->portSpeed <= 622000))
		portDesc.bandwidth = IX_CC_ATMSAR_PORT_BANDWIDTH_OC12; 
	else if( (attr->portSpeed > 622000) && (attr->portSpeed <= 2400000))
		portDesc.bandwidth = IX_CC_ATMSAR_PORT_BANDWIDTH_OC48; 
	else 
		portDesc.bandwidth = IX_CC_ATMSAR_PORT_BANDWIDTH_OC48; 
  
	/* Create the Correlator */
	portCorr = (limePointPortCorrelator_t *)PIL_AllocateMemory(
					sizeof(limePointPortCorrelator_t),PIL_MEM_ZERO_INIT);
	if(portCorr == NULL)
	{
		/* Set the user Response structure to failure */
		userResponse.status = NPF_OUT_OF_MEMORY;
		userResponse.reason = NPF_OUT_OF_MEMORY;						

		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_IfAttrSet, 
						cbcorrelator, 
						&userResponse);
		return;
	}


	portCorr->portId = (attr->atmAttr).port;	
	portCorr->corr = cbcorrelator;

	/* Call the Lime Point message helper function to create a port */
	ret = ix_cc_fpm_async_port_create(0, &portDesc, LimePointPortCreateCBFunc, portCorr);
	if(ret != IX_SUCCESS)
	{
		/* Map the error code to the NPF async error code */
		switch(ret)
		{
			case IX_CC_ATMSAR_ERROR_INVALID_PARAM:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_IF_E_INVALID_PARAM;
				userResponse.reason = NPF_IF_E_INVALID_PARAM;
				break;
#if 0
			case IX_CC_ERROR_MSG_LIBRARY:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_IF_E_INVALID_PARAM;
				userResponse.reason = NPF_IF_E_INVALID_PARAM;
				break;
			case IX_CC_ERROR_UNINIT:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_COMPONENT_UNINITIALIZED;
				userResponse.reason = NPF_COMPONENT_UNINITIALIZED;
				break;
#endif
			default:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_IF_E_INVALID_PARAM;
				userResponse.reason = NPF_IF_E_INVALID_PARAM;
				break;
		}/* end of switch */		


		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_IfAttrSet, 
						cbcorrelator, 
						&userResponse);
	}/* end of if */

	return;
}

/*********************************************************************
*
*   Function: fpmIfATM_IfDelFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to delete an ATM interface. 
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_IfDelFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	FPPI_PortID portId = 0;
	FPPI_PortID *tempPortId = (FPPI_PortID *)response_data;
	ix_error ret = IX_SUCCESS;
	limePointPortCorrelator_t *portCorr = NULL;
	limePointAtmPortHashInfo_t *hashInfo = NULL;
	PdkHashRet hashRet = PDK_HASH_OK;

	assert(fpmIfATM_IfDelFuncLimePoint);

	/* Get the ATM attributes */
	portId = *tempPortId;

	/* Get the Port Handle From the Port Id */
	hashRet = pdkHashFind(&portIdHashList,(void *)&portId,(void **)&hashInfo);
	if(hashRet != PDK_HASH_OK)
	{   
		/* Set the user Response structure to success */
		userResponse.status = NPF_IF_E_INVALID_PARAM;
		userResponse.reason = NPF_IF_E_INVALID_PARAM;
		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_IfDelete, 
						cbcorrelator, 
						&userResponse);
			
		return;
	}

	/* Create the Correlator */
	portCorr = (limePointPortCorrelator_t *)PIL_AllocateMemory(
					sizeof(limePointPortCorrelator_t),PIL_MEM_ZERO_INIT);
	if(portCorr == NULL)
	{
		/* Set the user Response structure to success */
		userResponse.status = NPF_OUT_OF_MEMORY;
		userResponse.reason = NPF_OUT_OF_MEMORY;						

		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_IfDelete, 
						cbcorrelator, 
						&userResponse);
		return;
	}


	portCorr->portId = portId;	
	portCorr->corr = cbcorrelator;

	/* Call the Lime Point message helper function to create a port */
	ret = ix_cc_fpm_async_port_remove(0,
									(ix_cc_atmsar_port_handle_t)hashInfo->portHandle,
									LimePointPortDeleteCBFunc,
									portCorr);
	if(ret != IX_SUCCESS)
	{
		/* Map the error code to the NPF async error code */
		switch(ret)
		{
			case IX_CC_ATMSAR_ERROR_INVALID_PARAM:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_IF_E_INVALID_PARAM;
				userResponse.reason = NPF_IF_E_INVALID_PARAM;
				break;
#if 0
			case IX_CC_ERROR_MSG_LIBRARY:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_IF_E_INVALID_PARAM;
				userResponse.reason = NPF_IF_E_INVALID_PARAM;
				break;
			case IX_CC_ERROR_UNINIT:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_COMPONENT_UNINITIALIZED;
				userResponse.reason = NPF_COMPONENT_UNINITIALIZED;
				break;
#endif
			default:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_IF_E_INVALID_PARAM;
				userResponse.reason = NPF_IF_E_INVALID_PARAM;
				break;
		}/* end of switch */		

		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_IfDelete, 
						cbcorrelator, 
						&userResponse);
	}/* end of if */

	return;
}

/*********************************************************************
*
*   Function: fpmIfATM_IfEnableFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Enable an ATM 
*			interface. 
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_IfEnableFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;

	assert(fpmIfATM_IfEnableFuncLimePoint);

	/* Set the user Response structure to success */
	userResponse.status = NPF_IF_E_INVALID_PARAM;
	userResponse.reason = NPF_IF_E_INVALID_PARAM;						

	BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
					FPPI_ATM_IfEnable, 
					cbcorrelator, 
					&userResponse);
	return;
}

/*********************************************************************
*
*   Function: fpmIfATM_IfDisableFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Disable an ATM 
*			interface. 
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_IfDisableFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;

	assert(fpmIfATM_IfDisableFuncLimePoint);

	/* Set the user Response structure to success */
	userResponse.status = NPF_IF_E_INVALID_PARAM;
	userResponse.reason = NPF_IF_E_INVALID_PARAM;						

	BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
					FPPI_ATM_IfDisable, 
					cbcorrelator, 
					&userResponse);
	return;
}



/*********************************************************************
*
*   Function: fpmIfATM_IfGenStatsGetFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to get Statistics of an ATM 
*			interface. 
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_IfGenStatsGetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_IfStatistics_t userResponse;
	FPPI_PortID portId = 0; 
	FPPI_PortID *tempPortId = (FPPI_PortID *)response_data;
	ix_error ret = IX_SUCCESS;
	limePointPortCorrelator_t *portCorr = NULL;
	limePointAtmPortHashInfo_t *hashInfo = NULL;
	PdkHashRet hashRet = PDK_HASH_OK;

	assert(fpmIfATM_IfGenStatsGetFuncLimePoint);

	/* Get the ATM attributes */
	portId = *tempPortId;

	/* Get the Port Handle From the Port Id */
	hashRet = pdkHashFind(&portIdHashList,(void *)&portId,(void **)&hashInfo);
	if(hashRet != PDK_HASH_OK)
	{   
		/* Set the user Response structure to success */
		userResponse.response.status = NPF_IF_E_INVALID_PARAM;
		userResponse.response.reason = NPF_IF_E_INVALID_PARAM;
		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_IfGenStatsGet, 
						cbcorrelator, 
						&userResponse);
			
		return;
	}

	/* Create the Correlator */
	portCorr = (limePointPortCorrelator_t *)PIL_AllocateMemory(
					sizeof(limePointPortCorrelator_t),PIL_MEM_ZERO_INIT);
	if(portCorr == NULL)
	{
		/* Set the user Response structure to success */
		userResponse.response.status = NPF_OUT_OF_MEMORY;
		userResponse.response.reason = NPF_OUT_OF_MEMORY;						

		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_IfGenStatsGet, 
						cbcorrelator, 
						&userResponse);
		return;
	}


	portCorr->portId = portId;	
	portCorr->corr = cbcorrelator;

	/* Call the Lime Point message helper function to create a port */
	ret = ix_cc_fpm_async_get_port_stats(0,
									(ix_cc_atmsar_port_handle_t)hashInfo->portHandle,
									LimePointPortStatsGetCBFunc,
									portCorr);
	if(ret != IX_SUCCESS)
	{
		/* Map the error code to the NPF async error code */
		switch(ret)
		{
			case IX_CC_ATMSAR_ERROR_INVALID_PARAM:
				/* Set the user Response structure to Failure */
				userResponse.response.status = NPF_IF_E_INVALID_PARAM;
				userResponse.response.reason = NPF_IF_E_INVALID_PARAM;
				break;
#if 0
			case IX_CC_ERROR_MSG_LIBRARY:
				/* Set the user Response structure to Failure */
				userResponse.response.status = NPF_IF_E_INVALID_PARAM;
				userResponse.response.reason = NPF_IF_E_INVALID_PARAM;
				break;
			case IX_CC_ERROR_UNINIT:
				/* Set the user Response structure to Failure */
				userResponse.response.status = NPF_COMPONENT_UNINITIALIZED;
				userResponse.response.reason = NPF_COMPONENT_UNINITIALIZED;
				break;
#endif
			default:
				/* Set the user Response structure to Failure */
				userResponse.response.status = NPF_IF_E_INVALID_PARAM;
				userResponse.response.reason = NPF_IF_E_INVALID_PARAM;
				break;
		}/* end of switch */		


		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_IfGenStatsGet, 
						cbcorrelator, 
						&userResponse);
	}/* end of if */

	return;
}





/*********************************************************************
*
*   Function: fpmIfATM_VccSetFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Setup a new VCC 
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_VccSetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{

	IfATM_IfVcc_t *recvVccInfo = NULL;
	ix_cc_atmsar_vc_descr_t *vccDesc = NULL;
	limePointVccCorrelator_t *vccCorr = NULL;
	ix_error ret = IX_SUCCESS;
	IfATM_FpResponse_t userResponse;


	assert(fpmIfATM_VccSetFuncLimePoint);

	/* Allocate Memory for Vcc Descriptor structure */
	vccDesc = (ix_cc_atmsar_vc_descr_t *)PIL_AllocateMemory(
					sizeof(ix_cc_atmsar_vc_descr_t),PIL_MEM_ZERO_INIT);
	if(vccDesc == NULL)
	{
		/* Set the user Response structure to success */
		userResponse.status = NPF_OUT_OF_MEMORY;
		userResponse.reason = NPF_OUT_OF_MEMORY;						

		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_VccSet, 
						cbcorrelator, 
						&userResponse);
		return;
	}

	/* Get the Vcc Information from the response_data */
	recvVccInfo = (IfATM_IfVcc_t *)response_data;

	/* Fill up the Lime Point Vcc Descriptor structure */
	vccDesc->spec.vpi = (recvVccInfo->vccInfo).vcc.VPI;
	vccDesc->spec.vci = (recvVccInfo->vccInfo).vcc.VCI;
	vccDesc->spec.portNo = recvVccInfo->portId ;


	switch((recvVccInfo->vccInfo).QoS.serviceCategory)
	{
		case NPF_OTHER:
			vccDesc->trafficParams.serviceClass = IX_CC_ATMSAR_ATM_SERVICE_NONE;
			break;	

		case NPF_CBR:
			vccDesc->trafficParams.serviceClass = IX_CC_ATMSAR_ATM_SERVICE_CBR;
			break;	

		case NPF_rtVBR:
			vccDesc->trafficParams.serviceClass = IX_CC_ATMSAR_ATM_SERVICE_RT_VBR;
			break;	

		case NPF_nrtVBR:
			vccDesc->trafficParams.serviceClass = IX_CC_ATMSAR_ATM_SERVICE_NRT_VBR;
			break;	

		case NPF_ABR:
			vccDesc->trafficParams.serviceClass = IX_CC_ATMSAR_ATM_SERVICE_ABR;
			break;	

		case NPF_UBR:
			vccDesc->trafficParams.serviceClass = IX_CC_ATMSAR_ATM_SERVICE_UBR;
			break;	

		case NPF_GFR:
			vccDesc->trafficParams.serviceClass = IX_CC_ATMSAR_ATM_SERVICE_GFR;
			break;	
	}/* end of switch */

	vccDesc->trafficParams.bstEffortReq = (recvVccInfo->vccInfo).QoS.bestEffIndicator; 


	vccDesc->trafficParams.bwd.pcr = (recvVccInfo->vccInfo).QoS.rxPCR1;
	vccDesc->trafficParams.bwd.scr = (recvVccInfo->vccInfo).QoS.rxSCR1;
	vccDesc->trafficParams.bwd.mbs = (recvVccInfo->vccInfo).QoS.rxMBS1;
	vccDesc->trafficParams.bwd.mfs = (recvVccInfo->vccInfo).QoS.rxMaxFrmSize;
	vccDesc->trafficParams.bwd.mcr = (recvVccInfo->vccInfo).QoS.rxMCR;
	vccDesc->trafficParams.bwd.cdvt = (recvVccInfo->vccInfo).QoS.rxCDVT;
	vccDesc->trafficParams.bwd.tagging = (recvVccInfo->vccInfo).QoS.rxTagging;
	vccDesc->trafficParams.bwd.frmDisc = (recvVccInfo->vccInfo).QoS.rxFrameDiscard;

	/* TBD */
	vccDesc->trafficParams.bwd.etoeOamF5FlInd = 0;
	if( (recvVccInfo->vccInfo).QoS.serviceCategory == NPF_UBR)
	{
	/* TBD */
		vccDesc->trafficParams.bwd.ubrBcs = IX_CC_ATMSAR_UBR_BCS_NO_PCR;
	}
	/* TBD */
	vccDesc->trafficParams.bwd.ubrPriority = 0;

	

	vccDesc->trafficParams.fwd.pcr = (recvVccInfo->vccInfo).QoS.txPCR1;
	vccDesc->trafficParams.fwd.scr = (recvVccInfo->vccInfo).QoS.txSCR1;
	vccDesc->trafficParams.fwd.mbs = (recvVccInfo->vccInfo).QoS.txMBS1;
	vccDesc->trafficParams.fwd.mfs = (recvVccInfo->vccInfo).QoS.txMaxFrmSize;
	vccDesc->trafficParams.fwd.mcr = (recvVccInfo->vccInfo).QoS.txMCR;
	vccDesc->trafficParams.fwd.cdvt = (recvVccInfo->vccInfo).QoS.txCDVT;
	vccDesc->trafficParams.fwd.tagging = (recvVccInfo->vccInfo).QoS.txTagging;
	vccDesc->trafficParams.fwd.frmDisc = (recvVccInfo->vccInfo).QoS.txFrameDiscard;

	/* TBD */
	vccDesc->trafficParams.fwd.etoeOamF5FlInd = 0;
	if( (recvVccInfo->vccInfo).QoS.serviceCategory == NPF_UBR)
	{
	/* TBD */
		vccDesc->trafficParams.fwd.ubrBcs = IX_CC_ATMSAR_UBR_BCS_NO_PCR;
	}
	/* TBD */
	vccDesc->trafficParams.fwd.ubrPriority = 0;

	/* TBD */	
	vccDesc->oam.queueNum = 0;
	vccDesc->oam.perfMonitoringAdminState = 0;
	vccDesc->oam.perfMonitoringBlockSize = 	0;	

	switch((recvVccInfo->vccInfo).AAL)
	{
		case NPF_IF_VCC_AAL1:
				vccDesc->aalType = IX_CC_ATMSAR_AAL1;
				break;

		case NPF_IF_VCC_AAL2:
				vccDesc->aalType = IX_CC_ATMSAR_AAL2;
				break;

		case NPF_IF_VCC_AAL3_4:
				vccDesc->aalType = IX_CC_ATMSAR_AAL34;
				break;
		case NPF_IF_VCC_AAL5_LLC:
		case NPF_IF_VCC_AAL5_VCMUX:
				vccDesc->aalType = IX_CC_ATMSAR_AAL5;
				break;
		/* TBD */
		case NPF_IF_VCC_AALRAW:
				vccDesc->aalType = IX_CC_ATMSAR_AALRAW;
				break;

		/* TBD */
		case NPF_IF_VCC_AAL0:
				vccDesc->aalType = IX_CC_ATMSAR_AAL0;
				break;
	} /* End of switch */

	/* If AAL type is AAL5 with LLC_SNAP enapculation set hdrType to
		PKT_PROTO_LLC_SNAP else make it NULL. In this case the other
		module namely IP will take care of hdrType while it does a 
		next hop entry add */
	if((recvVccInfo->vccInfo).AAL == NPF_IF_VCC_AAL5_LLC)
		vccDesc->hdrType = PKT_PROTO_LLC_SNAP;
	else
		vccDesc->hdrType = PKT_PROTO_NULL;
		
	vccDesc->freeListId = 0;
	vccDesc->l2port = 0;
	vccDesc->outputQueueNum = 0;
	vccDesc->packetPath = IX_CC_ATMSAR_PACKET_PATH_REGULAR;

	/* Create the Correlator */
	vccCorr = (limePointVccCorrelator_t *)PIL_AllocateMemory(
					sizeof(limePointVccCorrelator_t),PIL_MEM_ZERO_INIT);
	if(vccCorr == NULL)
	{
		/* Set the user Response structure to success */
		userResponse.status = NPF_OUT_OF_MEMORY;
		userResponse.reason = NPF_OUT_OF_MEMORY;						

		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_VccSet, 
						cbcorrelator, 
						&userResponse);

		PIL_FreeMemory(vccDesc);
		return;
	}


	vccCorr->connection = (recvVccInfo->vccInfo).vcc;
	vccCorr->portId = recvVccInfo->portId;	
	vccCorr->corr = cbcorrelator;
	vccCorr->aalType = (recvVccInfo->vccInfo).AAL;
	
	/* Call the Lime point message helper function */
 	ret = ix_cc_fpm_async_vc_create(0,vccDesc,LimePointVccCreateCBFunc,vccCorr);
	
	if(ret != IX_SUCCESS)
	{
		/* Map the error code to the NPF async error code */
		switch(ret)
		{
			case IX_CC_ATMSAR_ERROR_INVALID_PARAM:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_IF_E_INVALID_PARAM;
				userResponse.reason = NPF_IF_E_INVALID_PARAM;
				break;
#if 0
			case IX_CC_ERROR_MSG_LIBRARY:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_IF_E_INVALID_PARAM;
				userResponse.reason = NPF_IF_E_INVALID_PARAM;
				break;
			case IX_CC_ERROR_UNINIT:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_COMPONENT_UNINITIALIZED;
				userResponse.reason = NPF_COMPONENT_UNINITIALIZED;
				break;
#endif
			default:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_IF_E_INVALID_PARAM;
				userResponse.reason = NPF_IF_E_INVALID_PARAM;
				break;
		}/* end of switch */		


		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_VccSet, 
						cbcorrelator, 
						&userResponse);
	}/* end of if */

	/* Free the memory for the Vcc Descriptor */ 
	PIL_FreeMemory(vccDesc);
	
	return;
}

/*********************************************************************
*
*   Function: fpmIfATM_VccBindFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Bind a VCC connection
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_VccBindFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	IfATM_VccBind_t *recvVccAddr = (IfATM_VccBind_t *)response_data;
	limePointAtmVccHashInfo_t *hashInfo = NULL;
	PdkHashRet hashRet = PDK_HASH_OK;
	IfATM_VccAddr_t vccAddress;

	assert(fpmIfATM_VccBindFuncLimePoint);

	/* fill up the VccAddr structure */
	memset(&vccAddress, 0,sizeof(IfATM_VccAddr_t));
	vccAddress.vccAddr = recvVccAddr->vccInfo;
	vccAddress.portId =  recvVccAddr->portId;


	/* Get the Vcc Handle From the vccAddr */
	hashRet = pdkHashFind(&vccAddrHashList,(void *)&vccAddress,(void **)&hashInfo);
	if(hashRet != PDK_HASH_OK)
	{   
		/* Set the user Response structure to success */
		userResponse.status = NPF_IF_E_INVALID_PARAM;
		userResponse.reason = NPF_IF_E_INVALID_PARAM;
		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_VccBind, 
						cbcorrelator, 
						&userResponse);
			
	}
	else
	{
		/* Store the upper interface handle in the VCC Hash info node */
		hashInfo->upperIfHandle = recvVccAddr->parentIfHandle;


		/* Set the user Response structure to success */
		userResponse.status = NPF_SUCCESS;
		userResponse.reason = NPF_SUCCESS;
		BENDAPI_Report_ATMStatus (FPPI_SUCCESS, 
					FPPI_ATM_VccBind, 
					cbcorrelator, 
					&userResponse);
	}

	return;


}


/*********************************************************************
*
*   Function: fpmIfATM_VccDelFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Delete a VCC connection
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_VccDelFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	IfATM_VccAddr_t recvVccAddr; 
	IfATM_VccAddr_t *temp = (IfATM_VccAddr_t *)response_data; 
	ix_error ret = IX_SUCCESS;
	limePointAtmVccHashInfo_t *hashInfo = NULL;
	PdkHashRet hashRet = PDK_HASH_OK;
	limePointVccCorrelator_t *vccCorr = NULL;

	assert(fpmIfATM_VccDelFuncLimePoint);

	/* fill up the VccAddr structure */
	memset(&recvVccAddr, 0,sizeof(IfATM_VccAddr_t));
	recvVccAddr.vccAddr = temp->vccAddr;
	recvVccAddr.portId =  temp->portId;

	/* Get the Vcc Handle From the vccAddr */
	hashRet = pdkHashFind(&vccAddrHashList,(void *)&recvVccAddr,(void **)&hashInfo);
	if(hashRet != PDK_HASH_OK)
	{   
		/* Set the user Response structure to success */
		userResponse.status = NPF_IF_E_INVALID_PARAM;
		userResponse.reason = NPF_IF_E_INVALID_PARAM;
		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_VccDelete, 
						cbcorrelator, 
						&userResponse);
			
		return;
	}


	/* Create the Correlator */
	vccCorr = (limePointVccCorrelator_t *)PIL_AllocateMemory(
					sizeof(limePointVccCorrelator_t),PIL_MEM_ZERO_INIT);
	if(vccCorr == NULL)
	{
		/* Set the user Response structure to success */
		userResponse.status = NPF_OUT_OF_MEMORY;
		userResponse.reason = NPF_OUT_OF_MEMORY;
		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_VccDelete, 
						cbcorrelator, 
						&userResponse);
		return;
	}


	vccCorr->connection = recvVccAddr.vccAddr;
	vccCorr->portId = recvVccAddr.portId;	
	vccCorr->corr = cbcorrelator;
	

	/* Call the Message helper function of Lime point */
	ret = ix_cc_fpm_async_vc_remove( 0, hashInfo->vccHandle,
								LimePointVccDelCBFunc,vccCorr);
	if(ret != IX_SUCCESS)
	{
		/* Map the error code to the NPF async error code */
		switch(ret)
		{
			case IX_CC_ATMSAR_ERROR_INVALID_PARAM:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_IF_E_INVALID_PARAM;
				userResponse.reason = NPF_IF_E_INVALID_PARAM;
				break;
#if 0
			case IX_CC_ERROR_MSG_LIBRARY:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_IF_E_INVALID_PARAM;
				userResponse.reason = NPF_IF_E_INVALID_PARAM;
				break;
			case IX_CC_ERROR_UNINIT:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_COMPONENT_UNINITIALIZED;
				userResponse.reason = NPF_COMPONENT_UNINITIALIZED;
				break;
#endif
			default:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_IF_E_INVALID_PARAM;
				userResponse.reason = NPF_IF_E_INVALID_PARAM;
				break;
		}/* end of switch */		


		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_VccDelete, 
						cbcorrelator, 
						&userResponse);
	}/* end of if */
		
	return;
}


/*********************************************************************
*
*   Function: fpmIfATM_VccEnableFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Enable a VCC connection
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_VccEnableFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_VccEnableFuncLimePoint);
   
	/* Set the user Response structure to success */
	userResponse.status = NPF_IF_E_INVALID_PARAM;
	userResponse.reason = NPF_IF_E_INVALID_PARAM;

	BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
				FPPI_ATM_VccEnable, 
				cbcorrelator, 
				&userResponse);
	return;
}


/*********************************************************************
*
*   Function: fpmIfATM_VccDisableFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Disable a VCC connection
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_VccDisableFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_VccDisableFuncLimePoint);
   
	/* Set the user Response structure to success */
	userResponse.status = NPF_IF_E_INVALID_PARAM;
	userResponse.reason = NPF_IF_E_INVALID_PARAM;

	BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
					FPPI_ATM_VccDisable, 
					cbcorrelator, 
					&userResponse);
	return;
}


/*********************************************************************
*
*   Function: fpmIfATM_VccOperStatusGetFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Get OperStaus of a VCC conn
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_VccOperStatusGetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_VccOperStatusGetFuncLimePoint);
   
	/* Set the user Response structure to success */
	userResponse.status = NPF_IF_E_INVALID_PARAM;
	userResponse.reason = NPF_IF_E_INVALID_PARAM;

	BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
					FPPI_ATM_VccOperStatusGet, 
					cbcorrelator, 
					&userResponse);
	return;
}


/*********************************************************************
*
*   Function: fpmIfATM_VccStatsGetFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Get Statistics of a VCC conn
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_VccStatsGetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_VccAddr_t recvVccAddr;
	IfATM_VccAddr_t *temp = (IfATM_VccAddr_t *)response_data;
	ix_error ret = IX_SUCCESS;
	limePointAtmVccHashInfo_t *hashInfo = NULL;
	PdkHashRet hashRet = PDK_HASH_OK;
	limePointVccCorrelator_t *vccCorr = NULL;
	IfATM_VccStats_t userResponse;

	assert(fpmIfATM_VccStatsGetFuncLimePoint);


	/* fill up the VccAddr structure */
	memset(&recvVccAddr, 0,sizeof(IfATM_VccAddr_t));
	recvVccAddr.vccAddr = temp->vccAddr;
	recvVccAddr.portId =  temp->portId;

	/* Get the Vcc Handle From the vccAddr */
	hashRet = pdkHashFind(&vccAddrHashList,(void *)&recvVccAddr,(void **)&hashInfo);
	if(hashRet != PDK_HASH_OK)
	{   
		/* Set the user Response structure to success */
		userResponse.response.status = NPF_IF_E_INVALID_PARAM;
		userResponse.response.reason = NPF_IF_E_INVALID_PARAM;

		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_VccStatsGet, 
						cbcorrelator, 
						&userResponse);
			
		return;
	}


	/* Create the Correlator */
	vccCorr = (limePointVccCorrelator_t *)PIL_AllocateMemory(
					sizeof(limePointVccCorrelator_t),PIL_MEM_ZERO_INIT);
	if(vccCorr == NULL)
	{
		/* Set the user Response structure to success */
		userResponse.response.status = NPF_OUT_OF_MEMORY;
		userResponse.response.reason = NPF_OUT_OF_MEMORY;

		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_VccStatsGet, 
						cbcorrelator, 
						&userResponse);
		return;
	}


	vccCorr->connection = recvVccAddr.vccAddr;
	vccCorr->portId = recvVccAddr.portId;	
	vccCorr->corr = cbcorrelator;
	

	/* Call the Message helper function of Lime point */
	ret = ix_cc_fpm_async_get_vc_stats( 0, hashInfo->vccHandle,
								LimePointVccStatsGetCBFunc,vccCorr);
	if(ret != IX_SUCCESS)
	{
		/* Map the error code to the NPF async error code */
		switch(ret)
		{
			case IX_CC_ATMSAR_ERROR_INVALID_PARAM:
				/* Set the user Response structure to Failure */
				userResponse.response.status = NPF_IF_E_INVALID_PARAM;
				userResponse.response.reason = NPF_IF_E_INVALID_PARAM;
				break;
#if 0
			case IX_CC_ERROR_MSG_LIBRARY:
				/* Set the user Response structure to Failure */
				userResponse.response.status = NPF_IF_E_INVALID_PARAM;
				userResponse.response.reason = NPF_IF_E_INVALID_PARAM;
				break;
			case IX_CC_ERROR_UNINIT:
				/* Set the user Response structure to Failure */
				userResponse.response.status = NPF_COMPONENT_UNINITIALIZED;
				userResponse.response.reason = NPF_COMPONENT_UNINITIALIZED;
				break;
#endif
			default:
				/* Set the user Response structure to Failure */
				userResponse.response.status = NPF_IF_E_INVALID_PARAM;
				userResponse.response.reason = NPF_IF_E_INVALID_PARAM;
				break;
		}/* end of switch */		


		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_VccStatsGet, 
						cbcorrelator, 
						&userResponse);
	}/* end of if */
		
	return;
}



/*********************************************************************
*
*   Function: fpmIfATM_AAL2_ChannelSetFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Set an AAL2 channel on a VCC conn
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_AAL2_ChannelSetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_AAL2_ChannelInfo_t *recvAal2ChanInfo = NULL;
	ix_error ret = IX_SUCCESS;
	limePointAtmVccHashInfo_t *vccHashInfo = NULL;
	PdkHashRet hashRet = PDK_HASH_OK;
	limePointAal2Correlator_t *aal2Corr = NULL;
	ix_cc_atmsar_cid_descr_t cidDes;
	IfATM_VccAddr_t vccAddr;
	IfATM_FpResponse_t userResponse;

	assert(fpmIfATM_AAL2_ChannelSetFuncLimePoint);
	recvAal2ChanInfo = (IfATM_AAL2_ChannelInfo_t *)response_data;

	/* Get the Vcc Addr from the received data */
	memset(&vccAddr,0,sizeof(IfATM_VccAddr_t));
	vccAddr.vccAddr = (recvAal2ChanInfo->aal2ChanInfo).aal2ConnId.connection;
	vccAddr.portId = recvAal2ChanInfo->portId;

	
	/* Get the Vcc Handle From the vccAddr */
	hashRet = pdkHashFind(&vccAddrHashList,
						(void *)&vccAddr,
						(void **)&vccHashInfo);
	if(hashRet != PDK_HASH_OK)
	{   
		/* Set the user Response structure to success */
		userResponse.status = NPF_IF_E_INVALID_PARAM;
		userResponse.reason = NPF_IF_E_INVALID_PARAM;

		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_AAL2_ChannelSet, 
						cbcorrelator, 
						&userResponse);
			
		return;
	}


	/* Create the Correlator */
	aal2Corr = (limePointAal2Correlator_t *)PIL_AllocateMemory(
					sizeof(limePointAal2Correlator_t),PIL_MEM_ZERO_INIT);
	if(aal2Corr == NULL)
	{
		/* Set the user Response structure to success */
		userResponse.status = NPF_OUT_OF_MEMORY;
		userResponse.reason = NPF_OUT_OF_MEMORY;

		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_AAL2_ChannelSet, 
						cbcorrelator, 
						&userResponse);
		return;
	}


	aal2Corr->connection = (recvAal2ChanInfo->aal2ChanInfo).aal2ConnId.connection;	
	aal2Corr->portId = recvAal2ChanInfo->portId;
	aal2Corr->aal2Cid = (recvAal2ChanInfo->aal2ChanInfo).aal2ConnId.aal2Cid;
	aal2Corr->corr = cbcorrelator;

	/* Fill the CID descriptor fields */
	cidDes.vc = vccHashInfo->vccHandle;
	cidDes.cid = (recvAal2ChanInfo->aal2ChanInfo).aal2ConnId.aal2Cid;	

	/* Call the Message helper function of Lime point */
	ret = ix_cc_fpm_async_cid_create( 0,&cidDes,
								LimePointAal2ChanSetCBFunc,aal2Corr);
	if(ret != IX_SUCCESS)
	{
		/* Map the error code to the NPF async error code */
		switch(ret)
		{
			case IX_CC_ATMSAR_ERROR_INVALID_PARAM:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_IF_E_INVALID_PARAM;
				userResponse.reason = NPF_IF_E_INVALID_PARAM;
				break;
#if 0
			case IX_CC_ERROR_MSG_LIBRARY:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_IF_E_INVALID_PARAM;
				userResponse.reason = NPF_IF_E_INVALID_PARAM;
				break;
			case IX_CC_ERROR_UNINIT:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_COMPONENT_UNINITIALIZED;
				userResponse.reason = NPF_COMPONENT_UNINITIALIZED;
				break;
#endif
			default:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_IF_E_INVALID_PARAM;
				userResponse.reason = NPF_IF_E_INVALID_PARAM;
				break;
		}/* end of switch */		


		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_AAL2_ChannelSet, 
						cbcorrelator, 
						&userResponse);
	}/* end of if */
		
	return;
}




/*********************************************************************
*
*   Function: fpmIfATM_AAL2_ChannelDelFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Set an AAL2 channel on a VCC conn
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_AAL2_ChannelDelFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_AAL2_ConnId_t *recvAal2ConId = NULL;
	ix_error ret = IX_SUCCESS;
	limePointAtmAal2HashInfo_t *aalHashInfo = NULL;
	limePointAtmVccHashInfo_t *vccHashInfo = NULL;
	PdkHashRet hashRet = PDK_HASH_OK;
	limePointAal2Correlator_t *aal2Corr = NULL;
	IfATM_VccAddr_t vccAddr;
	IfATM_FpResponse_t userResponse;
	NPF_uint16_t tempAal2Cid = 0;

	assert(fpmIfATM_AAL2_ChannelDelFuncLimePoint);
	recvAal2ConId = (IfATM_AAL2_ConnId_t *)response_data;

	tempAal2Cid = (recvAal2ConId->aal2ConId).aal2Cid;

	/* Get the Vcc Addr from the received data */
	memset(&vccAddr,0,sizeof(IfATM_VccAddr_t));
	vccAddr.vccAddr = (recvAal2ConId->aal2ConId).connection;
	vccAddr.portId = recvAal2ConId->portId;


	/* Get the Vcc Handle From the vccAddr */
	hashRet = pdkHashFind(&vccAddrHashList,
						(void *)&vccAddr,
						(void **)&vccHashInfo);
	if(hashRet != PDK_HASH_OK)
	{   
		/* Set the user Response structure to success */
		userResponse.status = NPF_IF_E_INVALID_PARAM;
		userResponse.reason = NPF_IF_E_INVALID_PARAM;

		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_AAL2_ChannelDel, 
						cbcorrelator, 
						&userResponse);
			
		return;
	}

	/* Get the Aal2 Handle From the vccHashInfo */
	hashRet = pdkHashFind(&(vccHashInfo->aal2ChanIdHashList),
						(void *)&tempAal2Cid,
						(void **)&aalHashInfo);
	if(hashRet != PDK_HASH_OK)
	{   
		/* Set the user Response structure to success */
		userResponse.status = NPF_IF_E_INVALID_PARAM;
		userResponse.reason = NPF_IF_E_INVALID_PARAM;

		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_AAL2_ChannelDel, 
						cbcorrelator, 
						&userResponse);
			
		return;
	}


	/* Create the Correlator */
	aal2Corr = (limePointAal2Correlator_t *)PIL_AllocateMemory(
					sizeof(limePointAal2Correlator_t),PIL_MEM_ZERO_INIT);
	if(aal2Corr == NULL)
	{
		/* Set the user Response structure to success */
		userResponse.status = NPF_OUT_OF_MEMORY;
		userResponse.reason = NPF_OUT_OF_MEMORY;

		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_AAL2_ChannelDel, 
						cbcorrelator, 
						&userResponse);
		return;
	}


	aal2Corr->connection = (recvAal2ConId->aal2ConId).connection;	
	aal2Corr->portId = recvAal2ConId->portId;
	aal2Corr->aal2Cid = (recvAal2ConId->aal2ConId).aal2Cid;
	aal2Corr->corr = cbcorrelator;


	/* Call the Message helper function of Lime point */
	ret = ix_cc_fpm_async_cid_remove( 0,aalHashInfo->aal2Handle,
								LimePointAal2ChanDelCBFunc,aal2Corr);
	if(ret != IX_SUCCESS)
	{
		/* Map the error code to the NPF async error code */
		switch(ret)
		{
			case IX_CC_ATMSAR_ERROR_INVALID_PARAM:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_IF_E_INVALID_PARAM;
				userResponse.reason = NPF_IF_E_INVALID_PARAM;
				break;
#if 0
			case IX_CC_ERROR_MSG_LIBRARY:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_IF_E_INVALID_PARAM;
				userResponse.reason = NPF_IF_E_INVALID_PARAM;
				break;
			case IX_CC_ERROR_UNINIT:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_COMPONENT_UNINITIALIZED;
				userResponse.reason = NPF_COMPONENT_UNINITIALIZED;
				break;
#endif
			default:
				/* Set the user Response structure to Failure */
				userResponse.status = NPF_IF_E_INVALID_PARAM;
				userResponse.reason = NPF_IF_E_INVALID_PARAM;
				break;
		}/* end of switch */		


		BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_AAL2_ChannelDel, 
						cbcorrelator, 
						&userResponse);
	}/* end of if */
		
	return;
}




/*********************************************************************
*
*   Function: fpmIfATM_ConnectionXcSetFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Set an ATM Connection XC 
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_ConnectionXcSetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_ConnectionXcSetFuncLimePoint);
   
	/* Set the user Response structure to success */
	userResponse.status = NPF_IF_E_INVALID_PARAM;
	userResponse.reason = NPF_IF_E_INVALID_PARAM;

	BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
					FPPI_ATM_ConnectionXcSet, 
					cbcorrelator, 
					&userResponse);
	return;
}


/*********************************************************************
*
*   Function: fpmIfATM_ConnectionXcDelFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Delete an ATM Connection XC 
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_ConnectionXcDelFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_ConnectionXcDelFuncLimePoint);
   
	/* Set the user Response structure to success */
	userResponse.status = NPF_IF_E_INVALID_PARAM;
	userResponse.reason = NPF_IF_E_INVALID_PARAM;

	BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
					FPPI_ATM_ConnectionXcDel, 
					cbcorrelator, 
					&userResponse);
	return;
}


/*********************************************************************
*
*   Function: fpmIfATM_OAM_CP_SetFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Set connection point 
*			type for OAM services and passes OAM params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_OAM_CP_SetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_OAM_CP_SetFuncLimePoint);
   
	/* Set the user Response structure to success */
	userResponse.status = NPF_IF_E_INVALID_PARAM;
	userResponse.reason = NPF_IF_E_INVALID_PARAM;

	BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
					FPPI_ATM_OAM_CP_Set, 
					cbcorrelator, 
					&userResponse);
	return;
}

/*********************************************************************
*
*   Function: fpmIfATM_OAM_CC_SetFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Set continuity Check  
*			mechanism for OAM services and passes OAM  params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_OAM_CC_SetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_OAM_CC_SetFuncLimePoint);
   
	/* Set the user Response structure to success */
	userResponse.status = NPF_IF_E_INVALID_PARAM;
	userResponse.reason = NPF_IF_E_INVALID_PARAM;

	BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_OAM_CC_Set, 
						cbcorrelator, 
						&userResponse);
	return;
}


/*********************************************************************
*
*   Function: fpmIfATM_OAM_LB_SetFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Set Loopback  
*			mechanism for OAM services and passes OAM  params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_OAM_LB_SetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_OAM_LB_SetFuncLimePoint);
   
	/* Set the user Response structure to success */
	userResponse.status = NPF_IF_E_INVALID_PARAM;
	userResponse.reason = NPF_IF_E_INVALID_PARAM;

	BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
					FPPI_ATM_OAM_LB_Set, 
					cbcorrelator, 
					&userResponse);
	return;
}

/*********************************************************************
*
*   Function: fpmIfATM_OAM_MonitorFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Set OAM Cell Monitoring  
*			mechanism for OAM services and passes OAM  params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_OAM_MonitorFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_OAM_MonitorFuncLimePoint);
   
	/* Set the user Response structure to success */
	userResponse.status = NPF_IF_E_INVALID_PARAM;
	userResponse.reason = NPF_IF_E_INVALID_PARAM;

	BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
					FPPI_ATM_OAM_Monitor, 
					cbcorrelator, 
					&userResponse);
	return;
}



/*********************************************************************
*
*   Function: fpmIfATM_OAM_AlarmSetFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Set OAM Alarms  
*			mechanism for OAM services and passes OAM  params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_OAM_AlarmSetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_OAM_AlarmSetFuncLimePoint);
   
	/* Set the user Response structure to success */
	userResponse.status = NPF_IF_E_INVALID_PARAM;
	userResponse.reason = NPF_IF_E_INVALID_PARAM;

	BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
					FPPI_ATM_OAM_AlarmSet, 
					cbcorrelator, 
					&userResponse);
	return;
}

/*********************************************************************
*
*   Function: fpmIfATM_OAM_AlarmClearFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Clear OAM Alarms  
*			mechanism for OAM services and passes OAM  params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_OAM_AlarmClearFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_OAM_AlarmClearFuncLimePoint);
   
	/* Set the user Response structure to success */
	userResponse.status = NPF_IF_E_INVALID_PARAM;
	userResponse.reason = NPF_IF_E_INVALID_PARAM;

	BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
					FPPI_ATM_OAM_AlarmClear, 
					cbcorrelator, 
					&userResponse);
	return;
}



/*********************************************************************
*
*   Function: fpmIfATM_OAM_PM_SetFuncLimePoint
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to set Performance monitoring 
*			mechanism for OAM services and passes OAM  params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_OAM_PM_SetFuncLimePoint(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;

	assert(fpmIfATM_OAM_PM_SetFuncLimePoint);
   
	/* Set the user Response structure to success */
	userResponse.status = NPF_IF_E_INVALID_PARAM;
	userResponse.reason = NPF_IF_E_INVALID_PARAM;

	BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_OAM_PM_Set, 
						cbcorrelator, 
						&userResponse);
	return;
}


/*********************************************************************
*
*   Function: LimePointVccUpdateCBFunc
*
*   Parameters:
*			arg_Result - Result of the Action Taken 
*			arg_hVc -  Handle of the Vcc Created 
*			arg_pContext - User Passed Correlator 
*
*   Description:
*			This callback function is registered with the Lime Point Core Components.
*			The Lime point Core Components calls this function after creating a  
*			ATM VCC connection.
*
*
*   return : None 
*
*****************************************************************************/
ix_error LimePointVccUpdateCBFunc(
			ix_error arg_Result,
			ix_cc_atmsar_vc_handle_t arg_hVc,
			void *arg_pContext)

{
	limePointVccCorrelator_t *tempCorr = NULL;
	limePointAtmVccHashInfo_t *vccHashInfo = NULL;
	IfATM_VccAddr_t recvVccAddr;
	IfATM_FpResponse_t userResponse;
	FPPI_CBCORRELATOR userCorrelator;
	FPPI_PortID portId = 0;
	limePointAtmPortHashInfo_t *portHashInfo = NULL;

	assert(LimePointVccUpdateCBFunc);

	tempCorr = (limePointVccCorrelator_t *)arg_pContext;

    /* Check if VCC Create is successful */

    /* This is a callback function for VC Update.
       It is after the sucessful VC Update that we shall send the 
       successful creation of VCC.
       VC Update enables the VCC created for carrying traffic.
    */

    if(arg_Result == IX_SUCCESS)
    {

        /* Get the user supplied correlator hidden in the received correlator */
        userCorrelator = tempCorr->corr;

        /* Create the VCC Hash Info structure and insert it in the VCC Hash 
           List 
        */
        vccHashInfo = (limePointAtmVccHashInfo_t *)PIL_AllocateMemory(
						sizeof(limePointAtmVccHashInfo_t),PIL_MEM_ZERO_INIT);

	    if(vccHashInfo == NULL)
	    {
	        /* Delete the Vcc just created. This is done for consistency of 
               CP-PDK with SDK */
	        ix_cc_fpm_async_vc_remove( 0, arg_hVc,LimePointDummyRemoveCBFunc,0);

            /* Set the user Response structure to Failure */
		    userResponse.status = NPF_OUT_OF_MEMORY;
		    userResponse.reason = NPF_OUT_OF_MEMORY;						
   
	        /* Send the response back to the CP user */
		    BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
						FPPI_ATM_VccSet, 
						userCorrelator, 
						&userResponse);

		    /* Free the memory allocated for the received correlator */
		    PIL_FreeMemory(tempCorr);
		    return IX_CC_ERROR_OOM;
	    }

	    /* Get the VCC Addr Information hidden in the correlator */
	    vccHashInfo->connection = tempCorr->connection;
	    vccHashInfo->portId = tempCorr->portId;
	    vccHashInfo->vccHandle = arg_hVc;
	    vccHashInfo->aalType = tempCorr->aalType;	

	    /* Intialize the AAL2 Hash List */
	    pdkHashInit(&(vccHashInfo->aal2ChanIdHashList),
					ATM_AAL2_ID_HASH_LIST_SIZE,
					(PdkHashCompFn)Aal2ChanIdCompareFunc,						
					(PdkHashIndexFn)AtmHashIndexByAal2ChanId,
					PIL_FreeMemory);


	    /* Insert the Vcc Hash Info Node in the Port Hash Info associated with 
           the VCC 
        */

	    /* Get the Port Hash Info to which this VCC is mapped */

	    portId = tempCorr->portId;

	    /* Get the Port Handle From the Port Id */
	    if(pdkHashFind(&portIdHashList,(void *)&portId,(void **)&portHashInfo) 
							!= PDK_HASH_OK)
	    {
	        /* Delete the Vcc just created. 
               This is done for consistency of CP-PDK with SDK */

		    ix_cc_fpm_async_vc_remove( 0, arg_hVc,LimePointDummyRemoveCBFunc,0);

		    /* Set the user Response structure to Failure */
		    userResponse.status = NPF_IF_E_INVALID_PARAM;
		    userResponse.reason = NPF_IF_E_INVALID_PARAM;						
   
		    /* Send the response back to the CP user */
		    BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
		                         FPPI_ATM_VccSet, 
							     userCorrelator, 
							     &userResponse);

		    /* Free the memory allocated for the received correlator */
		    PIL_FreeMemory(tempCorr);
		    PIL_FreeMemory(vccHashInfo);
		    return IX_CC_ERROR_ENTRY_NOT_FOUND;

        }		

	    /* Insert the pointer of the vcc Hash Info in the Port Hash Info List 
           of Vccs
        */
	    npf_list_pushBack(&(portHashInfo->vccHashInfoList),vccHashInfo);
					

	    /* Create the key with which to insert the vcc hash info node in the 
           hash list 
        */
	    memset(&recvVccAddr,0,sizeof(IfATM_VccAddr_t));
	    recvVccAddr.vccAddr = tempCorr->connection;
	    recvVccAddr.portId = tempCorr->portId;

	    /* Insert the Vcc Hash Info Node in the Vcc Hash List */
	    if( pdkHashInsert(&vccAddrHashList,&recvVccAddr,vccHashInfo) != PDK_HASH_OK)
	    {
	        /* Delete the Vcc just created. 
               This is done for consistency of CP-PDK with SDK */
	        ix_cc_fpm_async_vc_remove( 0, arg_hVc,LimePointDummyRemoveCBFunc,0);

	        /* Set the user Response structure to Failure */
	        userResponse.status = NPF_IF_E_INVALID_PARAM;
	        userResponse.reason = NPF_IF_E_INVALID_PARAM;						
   
	        /* Send the response back to the CP user */
	        BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_VccSet, 
							userCorrelator, 
							&userResponse);

	        /* Free the memory allocated for the received correlator */
	        PIL_FreeMemory(tempCorr);
	        PIL_FreeMemory(vccHashInfo);
	        return IX_CC_ERROR_INTERNAL;
	    }

	    /* Set the user Response structure to success */
	    userResponse.status = NPF_NO_ERROR;
	    userResponse.reason = NPF_NO_ERROR;						
   
	    /* Send the response back to the CP user */
	    BENDAPI_Report_ATMStatus (FPPI_SUCCESS, 
							FPPI_ATM_VccSet, 
							userCorrelator, 
							&userResponse);
	}
	else
	{
		/* Set the user Response structure to success */
		userResponse.status = NPF_IF_E_INVALID_PARAM;
		userResponse.reason = NPF_IF_E_INVALID_PARAM;						
   
		/* Send the response back to the CP user */
		BENDAPI_Report_ATMStatus (FPPI_SUCCESS, 
							FPPI_ATM_VccSet, 
							userCorrelator, 
							&userResponse);

	} /* end of else */

	/* Free the memory allocated for the received correlator */
	PIL_FreeMemory(tempCorr);

	return IX_SUCCESS;
}

/*********************************************************************
*
*   Function: LimePointVccCreateCBFunc
*
*   Parameters:
*			arg_Result - Result of the Action Taken 
*			arg_hVc -  Handle of the Vcc Created 
*			arg_pContext - User Passed Correlator 
*
*   Description:
*			This callback function is registered with the Lime Point Core Components.
*			The Lime point Core Components calls this function after creating an  
*			ATM VCC connection.
*
*
*   return : None 
*
*****************************************************************************/
ix_error LimePointVccCreateCBFunc(
			ix_error arg_Result,
			ix_cc_atmsar_vc_handle_t arg_hVc,
			void *arg_pContext)

{
    limePointVccCorrelator_t *tempCorr = NULL;
    IfATM_FpResponse_t userResponse;
    FPPI_CBCORRELATOR userCorrelator;
    ix_cc_atmrx_update_data_t tmpVcUpdateData;
    ix_cc_atmsar_update_data_t *vcUpdateData = NULL;
    NPF_IfAAL_t   aalType;
    ix_cc_atmsar_vc_handle_t vcHandle = 0;


	assert(LimePointVccCreateCBFunc);

	tempCorr = (limePointVccCorrelator_t *)arg_pContext;

	/* Get the user supplied correlator hidden in the received correlator */
	userCorrelator = tempCorr->corr;

        vcHandle = arg_hVc;            /* VC Handle */
        aalType = tempCorr->aalType;   /* AAL Type  */

    /* Now we need to enable the VCC to carry the traffic. The VCC Create
       only creates the API but we need to enable it to carry the ATM traffic
       by an VC Update call.
    */

	/* Check if VCC Created successfully */
	if(arg_Result == IX_SUCCESS)
	{
        if (aalType == NPF_IF_VCC_AAL5_LLC)
            tmpVcUpdateData.lw1.hdrType = PKT_PROTO_LLC_SNAP;
        else if (aalType == NPF_IF_VCC_AAL5_VCMUX)
        /* We are carrying IPv4 traffic over this VCC */
            tmpVcUpdateData.lw1.hdrType = PKT_PROTO_IPv4;
        else
            tmpVcUpdateData.lw1.hdrType = PKT_PROTO_NULL;

       tmpVcUpdateData.lw0.l2port = vcHandle;
       vcUpdateData = (ix_cc_atmsar_update_data_t *)&tmpVcUpdateData;
   
        /* We are carrying the same context as the VCC Create */
        /* Context will be freed in the Callback of this function */
        ix_cc_fpm_async_vc_update(0,
                              vcHandle,
                              vcUpdateData,
                              LimePointVccUpdateCBFunc,
                              arg_pContext);
    }
	else
	{
		/* Set the user Response structure to success */
		userResponse.status = NPF_IF_E_INVALID_PARAM;
		userResponse.reason = NPF_IF_E_INVALID_PARAM;						
   
		/* Send the response back to the CP user */
		BENDAPI_Report_ATMStatus (FPPI_SUCCESS, 
							FPPI_ATM_VccSet, 
							userCorrelator, 
							&userResponse);

    /* We are no longer in need of tempCorr if the VCC Create is unsuccessful */

	/* Free the memory allocated for the received correlator */
	PIL_FreeMemory(tempCorr);

	} /* end of else */

	return IX_SUCCESS;
}



/*********************************************************************
*
*   Function: LimePointVccDelCBFunc
*
*   Parameters:
*			arg_Result - Result of the Action Taken 
*			arg_hVc -  Handle of the Vcc Created 
*			arg_pContext - User Passed Correlator 
*
*   Description:
*			This callback function is registered with the Lime Point Core Components.
*			The Lime point Core Components calls this function after deleting an  
*			ATM VCC connection.
*
*
*   return : None 
*
*****************************************************************************/
ix_error LimePointVccDelCBFunc(
			ix_error arg_Result,
			void *arg_pContext)

{
	limePointVccCorrelator_t *tempCorr = NULL;
	limePointAtmVccHashInfo_t *vccHashInfo = NULL;
	limePointAtmVccHashInfo_t *tempVccHashInfo = NULL;
	IfATM_VccAddr_t recvVccAddr;
	IfATM_FpResponse_t userResponse;
	FPPI_CBCORRELATOR userCorrelator;
	FPPI_PortID portId = 0;
	limePointAtmPortHashInfo_t *portHashInfo = NULL;
	DListItr *itr = NULL;



	assert(LimePointVccDelCBFunc);

	tempCorr = (limePointVccCorrelator_t *)arg_pContext;

	/* Get the User Correlator from the received correlator */
	userCorrelator = tempCorr->corr; 

	if(arg_Result == IX_SUCCESS)
	{

		/* Get the VCC Addr from the received correlator */
		memset(&recvVccAddr,0,sizeof(IfATM_VccAddr_t));
		recvVccAddr.vccAddr = tempCorr->connection;
		recvVccAddr.portId = tempCorr->portId;

		/* Remove the Hash Info for this VCC from the VCC hash list */
		if( pdkHashDelete(&vccAddrHashList,(void *)&recvVccAddr,(void **)&vccHashInfo) == PDK_HASH_OK)
		{
			/* Destroy the Aal2 hash list if it exists */
			pdkHashDestroy(&(vccHashInfo->aal2ChanIdHashList));

			/* Remove the Vcc HashInfo node from the VCC list in the port Hash list */
			/* Get the Port Hash Info to which this VCC is mapped */
			portId = tempCorr->portId;

			/* Get the Port Handle From the Port Id */
			pdkHashFind(&portIdHashList,(void *)&portId,(void **)&portHashInfo);

			/* Remove the pointer of the vcc Hash Info From the Port Hash Info List of Vccs */
			/* Create the iterator */
			if ((itr = npf_list_itrCreate((DList *)&(portHashInfo->vccHashInfoList))) != NULL)
			{
				npf_list_itrFirst(itr);
				do
				{
					/* Get the Vcc Hash Info CB from the iterator */
					tempVccHashInfo = (limePointAtmVccHashInfo_t *) npf_list_itrGetData(itr); 
		
					/* Check if this is the VCC we are looking for */
					if(tempVccHashInfo == vccHashInfo)
					{
						/* remove the vcc Hash info from the list */
						npf_list_itrRemove(itr,(void **)&vccHashInfo);

						break;
					}

				}while (npf_list_itrNext(itr) != -1);
	
				/* Delete the iterator that was created */	
				npf_list_itrDelete(itr);
			}

			/* Free the memory for the Vcc Hash Info Node */
			PIL_FreeMemory(vccHashInfo);
		}

		/* Set the user Response structure to success */
		userResponse.status = NPF_NO_ERROR;
		userResponse.reason = NPF_NO_ERROR;						
   
	}
	else
	{
		/* Set the user Response structure to success */
		userResponse.status = NPF_IF_E_INVALID_PARAM;
		userResponse.reason = NPF_IF_E_INVALID_PARAM;						
	}
	
	
	/* Send the response back to the CP user */
	BENDAPI_Report_ATMStatus (FPPI_SUCCESS, 
							FPPI_ATM_VccDelete, 
							userCorrelator, 
							&userResponse);

	/* Free the memory allocated for the received correlator */
	PIL_FreeMemory(tempCorr);

	return IX_SUCCESS;
}
		

/*********************************************************************
*
*   Function: LimePointVccStatsGetCBFunc
*
*   Parameters:
*			arg_Result - Result of the Action Taken 
*			arg_pLookupResult -  Statistics of the Vcc  
*			arg_pContext - User Passed Correlator 
*
*   Description:
*			This callback function is registered with the Lime Point Core Components.
*			The Lime point Core Components calls this function after getting statistics  
*			of an ATM VCC connection.
*
*
*   return : None 
*
*****************************************************************************/
ix_error LimePointVccStatsGetCBFunc(
			ix_error arg_Result,
			ix_cc_atmsar_vc_stats_t *arg_pLookupResult,
			void *arg_pContext)

{
	limePointVccCorrelator_t *tempCorr = NULL;
	FPPI_CBCORRELATOR userCorrelator;
	ix_cc_atmsar_vc_stats_t *stats = NULL;
	IfATM_VccStats_t userResponse;


	assert(LimePointVccStatsGetCBFunc);

	tempCorr = (limePointVccCorrelator_t *)arg_pContext;

	/* Get the User Correlator from the received correlator */
	userCorrelator = tempCorr->corr; 

	if(arg_Result == IX_SUCCESS)
	{
		/* Get the statistics received */
		stats = (ix_cc_atmsar_stats_t *)arg_pLookupResult;

		/* Set the user Response structure to success */
		userResponse.response.status = NPF_NO_ERROR;
		userResponse.response.reason = NPF_NO_ERROR;						

		/* Fill up the vcc Statistics structure */
		userResponse.vccStats.vccAddr = tempCorr->connection;
		userResponse.vccStats.cellsClp1Rx = stats->rx.cells;
		userResponse.vccStats.cellsClp0Rx = 0;
		userResponse.vccStats.cellsTaggedRx = 0;
		userResponse.vccStats.cellsDiscardedRx = stats->rx.cellErrors;
		userResponse.vccStats.cellsClp1Tx = stats->tx.cells;
		userResponse.vccStats.cellsClp0Tx = 	0;	
		userResponse.vccStats.framesRx = stats->rx.packets;
		userResponse.vccStats.framesDiscardedRx = stats->rx.discards;
		userResponse.vccStats.framesTx = stats->tx.packets;
		userResponse.vccStats.bytesRx = 0;
		userResponse.vccStats.bytesTx = 0;

		/* Send the response back to the CP user */
		BENDAPI_Report_ATMStatus (FPPI_SUCCESS, 
								FPPI_ATM_VccStatsGet, 
								userCorrelator, 
								&userResponse);
	}
	else
	{
		/* Set the user Response structure to success */
		userResponse.response.status = NPF_IF_E_INVALID_PARAM;
		userResponse.response.reason = NPF_IF_E_INVALID_PARAM;						

		/* Send the response back to the CP user */
		BENDAPI_Report_ATMStatus (FPPI_SUCCESS, 
								FPPI_ATM_VccStatsGet, 
								userCorrelator, 
								&userResponse);
	}

	/* Free the memory allocated for the received correlator */
	PIL_FreeMemory(tempCorr);

	return IX_SUCCESS;
}
		

/*********************************************************************
*
*   Function: LimePointAal2ChanSetCBFunc
*
*   Parameters:
*			arg_Result - Result of the Action Taken 
*			port -  AAL2 Channel Handle  
*			arg_pContext - User Passed Correlator 
*
*   Description:
*			This callback function is registered with the Lime Point Core Components.
*			The Lime point Core Components calls this function after setting an AAL2 Chan
*			on an ATM VCC connection.
*
*
*   return : None 
*
*****************************************************************************/
ix_error LimePointAal2ChanSetCBFunc(
			ix_error arg_Result,
			ix_cc_atmsar_cid_handle_t port,
			void *arg_pContext)
{
	limePointAal2Correlator_t *tempCorr = NULL;
	FPPI_CBCORRELATOR userCorrelator;
	IfATM_VccAddr_t recvVccAddr;
	limePointAtmVccHashInfo_t *vccHashInfo = NULL;
	PdkHashRet hashRet = PDK_HASH_OK;
	NPF_uint16_t   aal2Cid = 0; 
	limePointAtmAal2HashInfo_t *aalHashInfo = NULL;
	IfATM_FpResponse_t userResponse;


	assert(LimePointAal2ChanSetCBFunc);

	tempCorr = (limePointAal2Correlator_t *)arg_pContext;

	/* Get the User Correlator from the received correlator */
	userCorrelator = tempCorr->corr; 

	if(arg_Result == IX_SUCCESS)
	{
		/* Get the VCC Addr from the received correlator */
		memset(&recvVccAddr,0,sizeof(IfATM_VccAddr_t));
		recvVccAddr.vccAddr = tempCorr->connection;
		recvVccAddr.portId = tempCorr->portId;
		aal2Cid = tempCorr->aal2Cid;

		/* Find the VCC Hash Info node for the received AAL2 channel */
		hashRet = pdkHashFind(&vccAddrHashList,(void *)&recvVccAddr,(void **)&vccHashInfo);
		if(hashRet != PDK_HASH_OK)
		{   
			/* Call the ix_cc function to delete the AAL2 Channel */
			ix_cc_fpm_async_cid_remove( 0,port,LimePointDummyRemoveCBFunc,0);

			/* Set the user Response structure to success */
			userResponse.status = NPF_IF_E_INVALID_PARAM;
			userResponse.reason = NPF_IF_E_INVALID_PARAM;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_AAL2_ChannelSet, 
							userCorrelator, 
							&userResponse);
			
			/* Free the memory allocated for the received correlator */
			PIL_FreeMemory(tempCorr);
			return IX_CC_ERROR_INTERNAL;
		}

		/* Allocate memory for the AAL2 Hash Info node */
		aalHashInfo = (limePointAtmAal2HashInfo_t *)PIL_AllocateMemory(
							sizeof(limePointAtmAal2HashInfo_t),PIL_MEM_ZERO_INIT);
		if(aalHashInfo == NULL)
		{
			/* Call the ix_cc function to delete the AAL2 Channel */
			ix_cc_fpm_async_cid_remove( 0,port,LimePointDummyRemoveCBFunc,0);

			/* Set the user Response structure to success */
			userResponse.status = NPF_OUT_OF_MEMORY;
			userResponse.reason = NPF_OUT_OF_MEMORY;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_AAL2_ChannelSet, 
							userCorrelator, 
							&userResponse);
			
			/* Free the memory allocated for the received correlator */
			PIL_FreeMemory(tempCorr);
			return IX_CC_ERROR_OOM;
		}

		/* Store the received AAL2 Channel handle in the hash node */
		aalHashInfo->aal2Cid = aal2Cid;	
		aalHashInfo->aal2Handle = port;

		/* Insert the AAL2 Hash node in the AAL hash list of the VCC Hash node */
		hashRet = pdkHashInsert(&vccHashInfo->aal2ChanIdHashList,&aal2Cid,aalHashInfo);
		if(hashRet != PDK_HASH_OK)
		{   
			/* Call the ix_cc function to delete the AAL2 Channel */
			ix_cc_fpm_async_cid_remove( 0,port,LimePointDummyRemoveCBFunc,0);

			/* Set the user Response structure to success */
			userResponse.status = NPF_IF_E_INVALID_PARAM;
			userResponse.reason = NPF_IF_E_INVALID_PARAM;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_AAL2_ChannelSet, 
							userCorrelator, 
							&userResponse);
			
			/* Free the memory allocated for the received correlator */
			PIL_FreeMemory(tempCorr);

			return IX_CC_ERROR_INTERNAL;
		}	


		/* Set the user Response structure to success */
		userResponse.status = NPF_NO_ERROR;
		userResponse.reason = NPF_NO_ERROR;						

		/* Send the response back to the CP user */
		BENDAPI_Report_ATMStatus (FPPI_SUCCESS, 
								FPPI_ATM_AAL2_ChannelSet, 
								userCorrelator, 
								&userResponse);
	}
	else
	{
		/* Set the user Response structure to success */
		userResponse.status = NPF_IF_E_INVALID_PARAM;
		userResponse.reason = NPF_IF_E_INVALID_PARAM;						

		/* Send the response back to the CP user */
		BENDAPI_Report_ATMStatus (FPPI_SUCCESS, 
								FPPI_ATM_AAL2_ChannelSet, 
								userCorrelator, 
								&userResponse);
	}

	/* Free the memory allocated for the received correlator */
	PIL_FreeMemory(tempCorr);
	return IX_SUCCESS;
}
		

/*********************************************************************
*
*   Function: LimePointAal2ChanDelCBFunc
*
*   Parameters:
*			arg_Result - Result of the Action Taken 
*			arg_pContext - User Passed Correlator 
*
*   Description:
*			This callback function is registered with the Lime Point Core Components.
*			The Lime point Core Components calls this function after deleting an AAL2 Chan
*			on an ATM VCC connection.
*
*
*   return : None 
*
*****************************************************************************/
ix_error LimePointAal2ChanDelCBFunc(
			ix_error arg_Result,
			void *arg_pContext)

{
	limePointAal2Correlator_t *tempCorr = NULL;
	FPPI_CBCORRELATOR userCorrelator;
	IfATM_VccAddr_t recvVccAddr;
	limePointAtmVccHashInfo_t *vccHashInfo = NULL;
	PdkHashRet hashRet = PDK_HASH_OK;
	NPF_uint16_t   aal2Cid = 0; 
	IfATM_FpResponse_t userResponse;


	assert(LimePointAal2ChanDelCBFunc);

	tempCorr = (limePointAal2Correlator_t *)arg_pContext;

	/* Get the User Correlator from the received correlator */
	userCorrelator = tempCorr->corr; 

	if(arg_Result == IX_SUCCESS)
	{
		/* Get the VCC Addr from the received correlator */
		memset(&recvVccAddr,0,sizeof(IfATM_VccAddr_t));
		recvVccAddr.vccAddr = tempCorr->connection;
		recvVccAddr.portId = tempCorr->portId;
		aal2Cid = tempCorr->aal2Cid;

		/* Find the VCC Hash Info node for the received AAL2 channel */
		hashRet = pdkHashFind(&vccAddrHashList,(void *)&recvVccAddr,(void **)&vccHashInfo);
		if(hashRet != PDK_HASH_OK)
		{   
			npf_logger_write(LOG_FP_ATM, LOG_ERR, 
						"AAL2 Hash info node associated with the AAL2 channel not found \n");
		}

		/* Remove and Free the AAL2 Hash Node from the AAL Hash list in Vcc Hash Node */
		hashRet = pdkHashDeleteFree(&(vccHashInfo->aal2ChanIdHashList),&aal2Cid);
		if(hashRet != PDK_HASH_OK)
		{   
			npf_logger_write(LOG_FP_ATM, LOG_ERR, 
						"Failed to delete AAL2 Hash node from the AAL2 hash list \n");
		}

		/* Set the user Response structure to success */
		userResponse.status = NPF_NO_ERROR;
		userResponse.reason = NPF_NO_ERROR;						

		BENDAPI_Report_ATMStatus (FPPI_SUCCESS, 
						FPPI_ATM_AAL2_ChannelDel, 
						userCorrelator, 
						&userResponse);
	}
	else
	{
		/* Set the user Response structure to success */
		userResponse.status = NPF_IF_E_INVALID_PARAM;
		userResponse.reason = NPF_IF_E_INVALID_PARAM;						

		/* Send the response back to the CP user */
		BENDAPI_Report_ATMStatus (FPPI_SUCCESS, 
								FPPI_ATM_AAL2_ChannelDel, 
								userCorrelator, 
								&userResponse);
	}

	/* Free the memory allocated for the received correlator */
	PIL_FreeMemory(tempCorr);

	return IX_SUCCESS;
}



/*********************************************************************
*
*   Function: LimePointPortCreateCBFunc
*
*   Parameters:
*			arg_Result - Result of the Action Taken 
*			arg_hVc -  Handle of the Vcc Created 
*			arg_pContext - User Passed Correlator 
*
*   Description:
*			This callback function is registered with the Lime Point Core Components.
*			The Lime point Core Components calls this function after creating a  
*			ATM Port.
*
*
*   return : None 
*
*****************************************************************************/
ix_error LimePointPortCreateCBFunc(
			ix_error arg_Result,
			ix_cc_atmsar_port_handle_t port,
			void *arg_pContext)

{
	limePointPortCorrelator_t *tempCorr = NULL;
	limePointAtmPortHashInfo_t *portHashInfo = NULL;
	IfATM_FpResponse_t userResponse;
	FPPI_CBCORRELATOR userCorrelator;


	assert(LimePointPortCreateCBFunc);


	tempCorr = (limePointPortCorrelator_t *)arg_pContext;

	/* Get the user supplied correlator hidden in the received correlator */
	userCorrelator = tempCorr->corr;

	/* The following line shall be removed once LimePoint starts supporting
		dynamic Port Creation */
	arg_Result = IX_SUCCESS;


	/* Check if VCC Created successfully */
	if(arg_Result == IX_SUCCESS)
	{
		/* Create the Port Hash Info structure and insert it in the Port Hash List */
		portHashInfo = (limePointAtmPortHashInfo_t *)PIL_AllocateMemory(
							sizeof(limePointAtmPortHashInfo_t),PIL_MEM_ZERO_INIT);
		if(portHashInfo == NULL)
		{
			/* Call the Lime Point message helper function to delete the port */
			ix_cc_fpm_async_port_remove(0,port,LimePointDummyRemoveCBFunc,0);

			/* Set the user Response structure to success */
			userResponse.status = NPF_OUT_OF_MEMORY;
			userResponse.reason = NPF_OUT_OF_MEMORY;						
   
			/* Send the response back to the CP user */
			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_IfAttrSet, 
							userCorrelator, 
							&userResponse);

			/* Free the memory allocated for the received correlator */
			PIL_FreeMemory(tempCorr);
			return IX_CC_ERROR_OOM;
		}

		/* Get the Port Id Information hidden in the correlator */
		portHashInfo->portId = tempCorr->portId;
		portHashInfo->portHandle = port;	
		
		/* Initialize the list of Vccs Mapped to this port */
		npf_list_init(&(portHashInfo->vccHashInfoList), PIL_FreeMemory);

		/* Insert the Port Hash Info Node in the Port Hash List */
		pdkHashInsert(&portIdHashList,&(tempCorr->portId),portHashInfo);

		/* Set the user Response structure to success */
		userResponse.status = NPF_NO_ERROR;
		userResponse.reason = NPF_NO_ERROR;						
   
		/* Send the response back to the CP user */
		BENDAPI_Report_ATMStatus (FPPI_SUCCESS, 
							FPPI_ATM_IfAttrSet, 
							userCorrelator, 
							&userResponse);
	}
	else
	{
		/* Set the user Response structure to success */
		userResponse.status = NPF_IF_E_INVALID_PARAM;
		userResponse.reason = NPF_IF_E_INVALID_PARAM;						
   
		/* Send the response back to the CP user */
		BENDAPI_Report_ATMStatus (FPPI_SUCCESS, 
							FPPI_ATM_IfAttrSet, 
							userCorrelator, 
							&userResponse);

	} /* end of else */

	/* Free the memory allocated for the received correlator */
	PIL_FreeMemory(tempCorr);

	return IX_SUCCESS;
} /* end of LimePointPortCreateCBFunc */



/*********************************************************************
*
*   Function: LimePointPortDeleteCBFunc
*
*   Parameters:
*			arg_Result - Result of the Action Taken 
*			arg_pContext - User Passed Correlator 
*
*   Description:
*			This callback function is registered with the Lime Point Core Components.
*			The Lime point Core Components calls this function after deleting an  
*			ATM port.
*
*
*   return : None 
*
*****************************************************************************/
ix_error LimePointPortDeleteCBFunc(
			ix_error arg_Result,
			void *arg_pContext)

{
	limePointPortCorrelator_t *tempCorr = NULL;
	IfATM_FpResponse_t userResponse;
	FPPI_CBCORRELATOR userCorrelator;
	FPPI_PortID portId = 0;
	limePointAtmPortHashInfo_t *portHashInfo = NULL;
	DListItr *itr = NULL;
	limePointAtmVccHashInfo_t *vccHashInfo = NULL;
	IfATM_VccAddr_t recvVccAddr;


	assert(LimePointPortDeleteCBFunc);

	tempCorr = (limePointPortCorrelator_t *)arg_pContext;

	/* Get the User Correlator from the received correlator */
	userCorrelator = tempCorr->corr; 

	if(arg_Result == IX_SUCCESS)
	{

		/* Get the Port Id from the received correlator */
		portId = tempCorr->portId;

		/* Remove the Hash Info for this Port from the Port hash list */
		if( pdkHashDelete(&portIdHashList,&portId,(void **)&portHashInfo )== PDK_HASH_OK)
		{
			/* Set the user Response structure to success */
			userResponse.status = NPF_NO_ERROR;
			userResponse.reason = NPF_NO_ERROR;			

			/* From the Port Hash Info structure get all the VCCs mapped to this
				port and delete them as well */
			/* Create the iterator */
			if ((itr = npf_list_itrCreate((DList *)&(portHashInfo->vccHashInfoList))) != NULL)
			{
				npf_list_itrFirst(itr);
				do
				{
					/* remove the vcc Hash info from the list */
					npf_list_itrRemove(itr,(void **)&vccHashInfo);

					/* Get the VCC Addr from the received correlator */
					memset(&recvVccAddr,0,sizeof(IfATM_VccAddr_t));
					recvVccAddr.vccAddr = vccHashInfo->connection;
					recvVccAddr.portId = vccHashInfo->portId;

					/* Remove the Hash Info for this VCC from the VCC hash list */
					if( pdkHashDelete(&vccAddrHashList,
							(void *)&recvVccAddr,(void **)&vccHashInfo) == PDK_HASH_OK)
					{
						/* Destroy the Aal2 hash list if it exists */
						pdkHashDestroy(&(vccHashInfo->aal2ChanIdHashList));

						/* Free the memory for the Vcc Hash Info Node */
						PIL_FreeMemory(vccHashInfo);
					}

				}while (npf_list_itrNext(itr) != -1);
	
				/* Delete the iterator that was created */	
				npf_list_itrDelete(itr);
			}


			/* Free the memory for the portHashInfo structure */
			PIL_FreeMemory(portHashInfo);
			
		}
	}
	else
	{
		/* Set the user Response structure to success */
		userResponse.status = NPF_IF_E_INVALID_PARAM;
		userResponse.reason = NPF_IF_E_INVALID_PARAM;						
	}
	
	
	/* Send the response back to the CP user */
	BENDAPI_Report_ATMStatus (FPPI_SUCCESS, 
							FPPI_ATM_IfDelete, 
							userCorrelator, 
							&userResponse);

	/* Free the memory allocated for the received correlator */
	PIL_FreeMemory(tempCorr);

	return IX_SUCCESS;
}
		

/*********************************************************************
*
*   Function: LimePointPortStatsCBFunc
*
*   Parameters:
*			arg_Result - Result of the Action Taken 
*			arg_pLookupResult -  Statistics of the Port  
*			arg_pContext - User Passed Correlator 
*
*   Description:
*			This callback function is registered with the Lime Point Core Components.
*			The Lime point Core Components calls this function after getting statistics  
*			of an ATM Port. 
*
*
*   return : None 
*
*****************************************************************************/
ix_error LimePointPortStatsGetCBFunc(
			ix_error arg_Result,
			ix_cc_atmsar_port_stats_t *arg_pLookupResult,
			void *arg_pContext)

{
	limePointPortCorrelator_t *tempCorr = NULL;
	FPPI_CBCORRELATOR userCorrelator;
	ix_cc_atmsar_port_stats_t *stats = NULL;
	IfATM_IfStatistics_t userResponse;


	assert(LimePointPortStatsGetCBFunc);

	tempCorr = (limePointPortCorrelator_t *)arg_pContext;

	/* Get the User Correlator from the received correlator */
	userCorrelator = tempCorr->corr; 

	if(arg_Result == IX_SUCCESS)
	{
		/* Get the statistics received */
		stats = (ix_cc_atmsar_port_stats_t *)arg_pLookupResult;

		/* Set the user Response structure to success */
		userResponse.response.status = NPF_NO_ERROR;
		userResponse.response.reason = NPF_NO_ERROR;						

		/* Fill up the vcc Statistics structure */
		userResponse.stats.bytesRx = 0;
		userResponse.stats.ucPackRx = (stats->rx).packets;
		userResponse.stats.mcPackRx = 0;
		userResponse.stats.bcPackRx = 0;
		userResponse.stats.dropRx = (stats->rx).discards;
		userResponse.stats.errorRx = 0;
		userResponse.stats.protoRx = 0;
		userResponse.stats.bytesTx = 0;
		userResponse.stats.ucPackTx = (stats->tx).packets;
		userResponse.stats.mcPackTx = 0;
		userResponse.stats.bcPackTx = 0;
		userResponse.stats.dropTx = (stats->tx).discards;
		userResponse.stats.errorTx = 0;

		/* Send the response back to the CP user */
		BENDAPI_Report_ATMStatus (FPPI_SUCCESS, 
								FPPI_ATM_IfGenStatsGet, 
								userCorrelator, 
								&userResponse);
	}
	else
	{
		/* Set the user Response structure to success */
		userResponse.response.status = NPF_IF_E_INVALID_PARAM;
		userResponse.response.reason = NPF_IF_E_INVALID_PARAM;						

		/* Send the response back to the CP user */
		BENDAPI_Report_ATMStatus (FPPI_SUCCESS, 
								FPPI_ATM_IfGenStatsGet, 
								userCorrelator, 
								&userResponse);
	}

	/* Free the memory allocated for the received correlator */
	PIL_FreeMemory(tempCorr);

	return IX_SUCCESS;
}

		
/*********************************************************************
*
*   Function: LimePointDummyRemoveCBFunc
*
*   Parameters:
*			arg_Result - Result of the Action Taken 
*			arg_pContext - User Passed Correlator 
*
*   Description:
*			This callback function is registered with the Lime Point Core Components.
*			The Lime point Core Components calls this function after deleting an  
*			ATM port.This is a dummy function used only when CP-PDK fails to update
*			its tables even when SDK has done so.
*
*
*   return : None 
*
*****************************************************************************/
ix_error LimePointDummyRemoveCBFunc(
			ix_error arg_Result,
			void *arg_pContext)

{
	return IX_SUCCESS;
}

