/*****************************************************************************^M
*                  INTEL CORPORATION PROPRIETARY INFORMATION^M
*  This software is supplied under the terms of a license agreement or^M
*  nondisclosure agreement with Intel Corporation and may not be copied^M
*  or disclosed except in accordance with the terms of that agreement.^M
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.^M
*****************************************************************************/


#ifndef _VXWORKS_
#include <stdint.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <net/if.h>
#include <sys/ioctl.h>
#include "FPModule.h"
#include "FidMgr.h"
#include "fp_common.h"
#include "atm_mgr_impl.h"
#include "ix_atm.h"
#include "ix_cc_atm.h"
#include "pil.h"
#include "log.h"

#define IX_SUCCESS 0
//typedef uint32_t        ix_error;
//typedef uint32_t        ix_uint32;

static DList g_atm_pendinglist;

ix_error atm_cb(ix_error arg_Result,void* arg_pContext);
ix_error atm_oam_cb(ix_error arg_Result,void* arg_pContext);
ix_error atm_linkAttribs_cb(ix_error arg_Result,void* arg_pContext);
ix_error atm_OamProperties_cb(ix_error arg_Result,void* arg_pContext);
ix_error atm_OamStatistics_cb(ix_error arg_Result,void* arg_pContext);
ix_error atm_ConnStatistics_cb(ix_error arg_Result,void* arg_pContext);

int atm_InitFPEngine()
{
    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Entering atm_InitFPEngine");
    /*initialize a linked list to hold the Fid/Qid*/
    npf_list_init(&g_atm_pendinglist, PIL_FreeMemory);
    return 0;
}

int atm_DestroyFPEngine()
{
    /*free resources for any pending u_c*/
    atm_free_resources();

    /*Destroy List*/
    npf_list_destroy(&g_atm_pendinglist);
    return 0;
}


/*****************************************************************************
*   Function:  atm_connectionSet 
*
*   Parameters: 
*                t_c : translator context
*                Connection info
*   Description: 
*                This function requests the IXASDK for connection set 
*                It provides a callback function to the IXASDK to
                 call back with 
*                the status on connection set          
*   Return: none
*****************************************************************************/

void atm_connectionSet(atm_TranslatorContext_t *t_c, 
NPF_ATM_ConnectionInfo_t *conn_info)
{
    int32_t iRet = 0;
    atm_uc_info *uc_info = NULL;
    atm_UserContext_t *u_c = NULL;
    ix_error err;
    ix_atm_connect_req conn;

    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Entering atm Connection Setup");

    u_c = (atm_UserContext_t *)PIL_AllocateMemory(sizeof(atm_UserContext_t), PIL_MEM_ZERO_INIT);
    if(!u_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Mem allocation for u_c failed");
        iRet = -1;
        goto finish;
    }    
    u_c->t_c = t_c;
    
    uc_info = (atm_uc_info *)PIL_AllocateMemory(
    sizeof(atm_uc_info ), PIL_MEM_ZERO_INIT);
    if(!uc_info)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "Mem allocation failed for uc_info");
        iRet = -1;
        goto finish;
    }
    uc_info->type = FPPI_ATM_ConnectionSet;
    uc_info->u_c = (void *)u_c;
    /*insert this u_c info into the u_c pending list*/
    npf_list_pushBack(&g_atm_pendinglist, uc_info);

    /* Fill the connection information */


    /*Call into the IXASDK*/

    if(err != IX_SUCCESS)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "IXASDK30 API call failed error code:%d",err);
        iRet = -1;
        goto finish;
    }
    
finish:
    if(iRet == -1)
    {
        if(u_c)    
            PIL_FreeMemory(u_c);
        /*inform the Translator component about failure via the up-call*/
            
        fpm_atm_mgr_upcall(iRet, t_c);
        
    }
    
    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Leaving atm Connection Setup");

}

/*****************************************************************************
*   Function:  atm_connectionDel
*
*   Parameters:
*                t_c : translator context
*                Connection info
*   Description: 
*                This function requests the IXASDK for connection delete
*                It provides a callback function to the IXASDK to
                 call back with 
*                the status on connection delete          
*   Return: none
*****************************************************************************/
void atm_connectionDel(atm_TranslatorContext_t *t_c)
{

    int32_t iRet = 0;
    atm_uc_info *uc_info = NULL;
    atm_UserContext_t *u_c = NULL;
    ix_error err;
    ix_atm_disconnect_req disConnection;

    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Entering atm Connection Delete");

    u_c = (atm_UserContext_t *)PIL_AllocateMemory(sizeof(atm_UserContext_t), PIL_MEM_ZERO_INIT);
    if(!u_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Mem allocation for u_c failed");
        iRet = -1;
        goto finish;
    }    
    u_c->t_c = t_c;

    uc_info = (atm_uc_info *)PIL_AllocateMemory(
    sizeof(atm_uc_info ), PIL_MEM_ZERO_INIT);
    if(!uc_info)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "Mem allocation failed for uc_info");
        iRet = -1;
        goto finish;
    }
    uc_info->type = FPPI_ATM_ConnectionDel;
    uc_info->u_c = (void *)u_c;
    /*insert this u_c info into the u_c pending list*/
    npf_list_pushBack(&g_atm_pendinglist, uc_info);

    /* Fill the disconnection information */
    
    /*Call into the IXASDK*/

    if(err != IX_SUCCESS)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "IXASDK30 API call failed error code:%d",err);
        iRet = -1;
        goto finish;
    }
    
finish:
    if(iRet == -1)
    {
        if(u_c)    
            PIL_FreeMemory(u_c);
        /*inform the Translator component about failure via the up-call*/
            
        fpm_atm_mgr_upcall(iRet, t_c);
        
    }
    
    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Leaving atm Connection Deletion");

}

/*****************************************************************************
*   Function:  atm_aal5MuxSet
*
*   Parameters:
*                t_c : translator context
*                mux : type of the AAL5
*                reserved : reserved field 
*   Description: 
*                This function requests the IXASDK for mux set 
*                It provides a callback function to the IXASDK to
                 call back with 
*                the status on mux set        
*   Return: none
*****************************************************************************/

void atm_aal5MuxSet(atm_TranslatorContext_t *t_c, 
NPF_ATM_AAL5_MuxType_t mux, 
NPF_uint32_t reserved)
{
    int32_t iRet = 0;
    atm_uc_info *uc_info = NULL;
    atm_UserContext_t *u_c = NULL;
    ix_error err;
    ix_atm_conn_id conn;
    ix_uint8 mux_flag;
    ix_uint8 res;

    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Entering mux  Set");

    u_c = (atm_UserContext_t *)PIL_AllocateMemory(sizeof(atm_UserContext_t), PIL_MEM_ZERO_INIT);
    if(!u_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Mem allocation for u_c failed");
        iRet = -1;
        goto finish;
    }
    u_c->t_c = t_c;

    uc_info = (atm_uc_info *)PIL_AllocateMemory(
    sizeof(atm_uc_info ), PIL_MEM_ZERO_INIT);
    if(!uc_info)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR,
            "Mem allocation failed for uc_info");
        iRet = -1;
        goto finish;
    }
    uc_info->type = FPPI_ATM_AAL5MuxSet;
    uc_info->u_c = (void *)u_c;
    /*insert this u_c info into the u_c pending list*/
    npf_list_pushBack(&g_atm_pendinglist, uc_info);

    /* Fill the MUX information */

    /*Call into the IXASDK*/

    if(err != IX_SUCCESS)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR,
            "IXASDK30 API call failed error code:%d",err);
        iRet = -1;
        goto finish;
    }

finish:
    if(iRet == -1)
    {
        if(u_c)   
            PIL_FreeMemory(u_c);
        /*inform the Translator component about failure via the up-call*/

        fpm_atm_mgr_upcall(iRet, t_c);

    }

    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Leaving atm aal5 Mux Set");


}

/*****************************************************************************
*   Function:  atm_aal2ChannelSet
*
*   Parameters:
*                t_c : translator context
*   Description: 
*                This function requests the IXASDK for channel set 
*                It provides a callback function to the IXASDK to
                 call back with 
*                the status on channel set          
*   Return: none
*****************************************************************************/
void atm_aal2ChannelSet(atm_TranslatorContext_t* t_c)
{
    int32_t iRet = 0;
    atm_uc_info *uc_info = NULL;
    atm_UserContext_t *u_c = NULL;
    ix_error err;
    ix_atm_conn_id conn;
    ix_uint32 length;
    ix_uint8 cid;

    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Entering atm aal2 channel Set");

    u_c = (atm_UserContext_t *)PIL_AllocateMemory(sizeof(atm_UserContext_t), PIL_MEM_ZERO_INIT);
    if(!u_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Mem allocation for u_c failed");
        iRet = -1;
        goto finish;
    }    
    u_c->t_c = t_c;

    uc_info = (atm_uc_info *)PIL_AllocateMemory(
    sizeof(atm_uc_info ), PIL_MEM_ZERO_INIT);
    if(!uc_info)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "Mem allocation failed for uc_info");
        iRet = -1;
        goto finish;
    }
    uc_info->type = FPPI_ATM_Aal2ChannelSet;
    uc_info->u_c = (void *)u_c;
    /*insert this u_c info into the u_c pending list*/
    npf_list_pushBack(&g_atm_pendinglist, uc_info);

    /* Fill the aal2 information */
    /*Call into the IXASDK*/

    if(err != IX_SUCCESS)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "IXASDK30 API call failed error code:%d",err);
        iRet = -1;
        goto finish;
    }
    
finish:
    if(iRet == -1)
    {
        if(u_c)    
            PIL_FreeMemory(u_c);
        /*inform the Translator component about failure via the up-call*/
            
        fpm_atm_mgr_upcall(iRet, t_c);
        
    }
    
    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Leaving atm aal2 channel Set");

}

/*****************************************************************************
*   Function:  atm_aal2ChannelDel
*
*   Parameters:
*                t_c : translator context
*   Description:
*                This function requests the IXASDK for channel delete
*                It provides a callback function to the IXASDK to
                 call back with
*                the status on channel del
*   Return: none
*****************************************************************************/


void atm_aal2ChannelDel(atm_TranslatorContext_t* t_c)
{
    int32_t iRet = 0;
    atm_uc_info *uc_info = NULL;
    atm_UserContext_t *u_c = NULL;
    ix_error err;
    ix_atm_conn_id conn;
    ix_uint32 length;
    ix_uint8 cid;

    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Entering atm aal2 channel Delete");

    u_c = (atm_UserContext_t *)PIL_AllocateMemory(sizeof(atm_UserContext_t), PIL_MEM_ZERO_INIT);
    if(!u_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Mem allocation for u_c failed");
        iRet = -1;
        goto finish;
    }    
    u_c->t_c = t_c;

    uc_info = (atm_uc_info *)PIL_AllocateMemory(
    sizeof(atm_uc_info ), PIL_MEM_ZERO_INIT);
    if(!uc_info)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "Mem allocation failed for uc_info");
        iRet = -1;
        goto finish;
    }
    uc_info->type = FPPI_ATM_Aal2ChannelDel;
    uc_info->u_c = (void *)u_c;
    /*insert this u_c info into the u_c pending list*/
    npf_list_pushBack(&g_atm_pendinglist, uc_info);


    /* Fill the deletion information */

    /*Call into the IXASDK*/

    if(err != IX_SUCCESS)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "IXASDK30 API call failed error code:%d",err);
        iRet = -1;
        goto finish;
    }
    
finish:
    if(iRet == -1)
    {
        if(u_c)    
            PIL_FreeMemory(u_c);
        /*inform the Translator component about failure via the up-call*/
            
        fpm_atm_mgr_upcall(iRet, t_c);
        
    }
    
    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Leaving atm aal2 channel delete");

}
/*****************************************************************************
*   Function:  atm_connectionXcSet
*
*   Parameters:
*                t_c : translator context
*                Connection info
*   Description:
*                This function requests the IXASDK for connection xconnect set
*                It provides a callback function to the IXASDK to
                 call back with
*                the status on connection xconnect set
*   Return: none
*****************************************************************************/

void atm_ConnectionXcSet(atm_TranslatorContext_t* t_c)
{
    int32_t iRet = 0;
    atm_uc_info *uc_info = NULL;
    atm_UserContext_t *u_c = NULL;
    ix_error err;
    ix_atm_conn_id source; 
    ix_atm_conn_id destination; 

    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Entering atm xc connect Set");

    u_c = (atm_UserContext_t *)PIL_AllocateMemory(sizeof(atm_UserContext_t), PIL_MEM_ZERO_INIT);
    if(!u_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Mem allocation for u_c failed");
        iRet = -1;
        goto finish;
    }    
    u_c->t_c = t_c;

    uc_info = (atm_uc_info *)PIL_AllocateMemory(
    sizeof(atm_uc_info ), PIL_MEM_ZERO_INIT);
    if(!uc_info)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "Mem allocation failed for uc_info");
        iRet = -1;
        goto finish;
    }
    uc_info->type = FPPI_ATM_ConnectionXcSet;
    uc_info->u_c = (void *)u_c;
    /*insert this u_c info into the u_c pending list*/
    npf_list_pushBack(&g_atm_pendinglist, uc_info);


    /* Fill the disconnection information */

    /*Call into the IXASDK*/

    if(err != IX_SUCCESS)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "IXASDK30 API call failed error code:%d",err);
        iRet = -1;
        goto finish;
    }
    
finish:
    if(iRet == -1)
    {
        if(u_c)    
            PIL_FreeMemory(u_c);
        /*inform the Translator component about failure via the up-call*/
            
        fpm_atm_mgr_upcall(iRet, t_c);
    }
    
    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Leaving atm xc connect    set");


}

/*****************************************************************************
*   Function:  atm_connectionXcDel
*
*   Parameters:
*                t_c : translator context
*                Connection info
*   Description:
*                This function requests the IXASDK for connection xconnect del
*                It provides a callback function to the IXASDK to
                 call back with
*                the status on connection xconnect del
*   Return: none
*****************************************************************************/


void atm_ConnectionXcDel(atm_TranslatorContext_t* t_c)
{
    int32_t iRet = 0;
    atm_uc_info *uc_info = NULL;
    atm_UserContext_t *u_c = NULL;
    ix_error err;
    ix_atm_conn_id source; 
    ix_atm_conn_id destination; 

    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Entering atm xc connect Delete");

    u_c = (atm_UserContext_t *)PIL_AllocateMemory(sizeof(atm_UserContext_t), PIL_MEM_ZERO_INIT);
    if(!u_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Mem allocation for u_c failed");
        iRet = -1;
        goto finish;
    }    
    u_c->t_c = t_c;


    uc_info = (atm_uc_info *)PIL_AllocateMemory(
    sizeof(atm_uc_info ), PIL_MEM_ZERO_INIT);
    if(!uc_info)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "Mem allocation failed for uc_info");
        iRet = -1;
        goto finish;
    }
    uc_info->type = FPPI_ATM_ConnectionXcDel;
    uc_info->u_c = (void *)u_c;
    /*insert this u_c info into the u_c pending list*/
    npf_list_pushBack(&g_atm_pendinglist, uc_info);


    /* Fill the disconnection information */

    /*Call into the IXASDK*/

    if(err != IX_SUCCESS)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "IXASDK30 API call failed error code:%d",err);
        iRet = -1;
        goto finish;
    }
    
finish:
    if(iRet == -1)
    {
        if(u_c)    
            PIL_FreeMemory(u_c);
        /*inform the Translator component about failure via the up-call*/
            
        fpm_atm_mgr_upcall(iRet, t_c);
    }
    
    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Leaving atm xc connect    delete");


}
/*****************************************************************************
*   Function:  atm_tmParamGet
*
*   Parameters:
*                t_c : translator context
*   Description:
*                This function requests the IXASDK for tm param get
*                It provides a callback function to the IXASDK to
                 call back with
*                the status on param get
*   Return: none
*****************************************************************************/
void atm_tmParamGet(atm_TranslatorContext_t* t_c)
{
    int32_t iRet = 0;
    atm_uc_info *uc_info = NULL;
    atm_UserContext_t *u_c = NULL;
    ix_error err;
    ix_atm_conn_id conn;

    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Entering atm traffic Params get ");

    u_c = (atm_UserContext_t *)PIL_AllocateMemory(sizeof(atm_UserContext_t), PIL_MEM_ZERO_INIT);
    if(!u_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Mem allocation for u_c failed");
        iRet = -1;
        goto finish;
    }    
    u_c->t_c = t_c;

    uc_info = (atm_uc_info *)PIL_AllocateMemory(
    sizeof(atm_uc_info ), PIL_MEM_ZERO_INIT);
    if(!uc_info)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "Mem allocation failed for uc_info");
        iRet = -1;
        goto finish;
    }
    uc_info->type = FPPI_ATM_TMParamGet;
    uc_info->u_c = (void *)u_c;
    /*insert this u_c info into the u_c pending list*/
    npf_list_pushBack(&g_atm_pendinglist, uc_info);


    /* Fill the disconnection information */

    /*Call into the IXASDK*/

    if(err != IX_SUCCESS)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "IXASDK30 API call failed error code:%d",err);
        iRet = -1;
        goto finish;
    }
    
finish:
    if(iRet == -1)
    {
        if(u_c)    
            PIL_FreeMemory(u_c);
        /*inform the Translator component about failure via the up-call*/
            
        fpm_atm_mgr_upcall(iRet, t_c);
    }
    
    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Leaving atm traffic Params get");


}
/*****************************************************************************
*   Function:  atm_linkPropSet
*
*   Parameters:
*                t_c : translator context
*   Description:
*                This function requests the IXASDK for link porperties
*                It provides a callback function to the IXASDK to
                 call back with
*                the status on set of link properties
*   Return: none
*****************************************************************************/

void atm_linkPropSet(atm_linkAttribs_TranslatorContext_t* t_c)
{
    int32_t iRet = 0;
    uint8_t i;
    atm_uc_info *uc_info = NULL;
    atm_linkAttribs_UserContext_t *u_c = NULL;
    ix_error err;
    ix_atm_port_attribs attribs;

    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Entering atm link attribs get ");

    u_c = (atm_linkAttribs_UserContext_t  *)PIL_AllocateMemory(
           sizeof(atm_linkAttribs_UserContext_t), PIL_MEM_ZERO_INIT);
    if(!u_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Mem allocation for u_c failed");
        iRet = -1;
        goto finish;
    }    
    u_c->t_c = ( atm_linkAttribs_TranslatorContext_t*)t_c;


    uc_info = (atm_uc_info *)PIL_AllocateMemory(
    sizeof(atm_uc_info ), PIL_MEM_ZERO_INIT);
    if(!uc_info)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "Mem allocation failed for uc_info");
        iRet = -1;
        goto finish;
    }
    uc_info->type = FPPI_ATM_LinkPropSet;
    uc_info->u_c = (void *)u_c;
    /*insert this u_c info into the u_c pending list*/
    npf_list_pushBack(&g_atm_pendinglist, uc_info);


    /* Fill the disconnection information */
 
    /*Call into the IXASDK*/
    
    if(err != IX_SUCCESS)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "IXASDK30 API call failed error code:%d",err);
        iRet = -1;
        goto finish;
    }
    
finish:
    if(iRet == -1)
    {
        if(u_c)    
            PIL_FreeMemory(u_c);
        /*inform the Translator component about failure via the up-call*/
            
        fpm_atm_mgr_linkAttribs_upcall(iRet, t_c);
    }
    
    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Leaving atm link attribs get");

}
/*****************************************************************************
*   Function:  atm_linkPropGet
*
*   Parameters:
*                t_c : translator context
*   Description:
*                This function requests the IXASDK for link porperties
*                It provides a callback function to the IXASDK to
                 call back with
*                the status on get of link properties
*   Return: none
*****************************************************************************/

void atm_linkPropGet(atm_linkAttribs_TranslatorContext_t* t_c)
{
    int32_t iRet = 0;
    atm_uc_info *uc_info = NULL;
    atm_linkAttribs_UserContext_t *u_c = NULL;
    ix_error err;
    ix_uint32  port;

    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Entering atm link properties get ");

    u_c = (atm_linkAttribs_UserContext_t *)
       PIL_AllocateMemory(sizeof(atm_linkAttribs_UserContext_t), 
        PIL_MEM_ZERO_INIT);
    if(!u_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Mem allocation for u_c failed");
        iRet = -1;
        goto finish;
    }    
    u_c->t_c = t_c;


    uc_info = (atm_uc_info *)PIL_AllocateMemory(
    sizeof(atm_uc_info ), PIL_MEM_ZERO_INIT);
    if(!uc_info)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "Mem allocation failed for uc_info");
        iRet = -1;
        goto finish;
    }
    uc_info->type = FPPI_ATM_LinkPropGet;
    uc_info->u_c = (void *)u_c;
    /*insert this u_c info into the u_c pending list*/
    npf_list_pushBack(&g_atm_pendinglist, uc_info);


    /* Fill the disconnection information */
    port = t_c->port;
    /*Call into the IXASDK*/
    
    if(err != IX_SUCCESS)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "IXASDK30 API call failed error code:%d",err);
        iRet = -1;
        goto finish;
    }
    
finish:
    if(iRet == -1)
    {
        if(u_c)    
            PIL_FreeMemory(u_c);
        /*inform the Translator component about failure via the up-call*/
            
        fpm_atm_mgr_linkAttribs_upcall(iRet, t_c);
    }
    
    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Leaving atm link properties get");

}

/*****************************************************************************
*   Function:  atm_oam_setup_req
*
*   Parameters:
*                t_c : translator context
*   Description:
*                This function requests the IXASDK for oam setup
*                It provides a callback function to the IXASDK to
                 call back with
*                the status on oam setup
*   Return: none
*****************************************************************************/

void atm_oam_setup_req(atm_oam_TranslatorContext_t* t_c)
{
    int32_t iRet = 0;
    int32_t i;
    atm_uc_info *uc_info = NULL;
    atm_Oam_UserContext_t  *u_c = NULL;
    ix_error err;
    ix_atm_oam_setup_req  oam_info;

    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Entering oam setup request ");

    u_c = (atm_Oam_UserContext_t  *)
           PIL_AllocateMemory(sizeof(atm_Oam_UserContext_t), 
           PIL_MEM_ZERO_INIT);
    if(!u_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Mem allocation for u_c failed");
        iRet = -1;
        goto finish;
    }

    uc_info = (atm_uc_info *)PIL_AllocateMemory(
    sizeof(atm_uc_info ), PIL_MEM_ZERO_INIT);
    if(!uc_info)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "Mem allocation failed for uc_info");
        iRet = -1;
        goto finish;
    }
    /* initialize the oam_info structure */

    u_c->t_c = t_c;

    uc_info->u_c = (void *)u_c;
    /*insert this u_c info into the u_c pending list*/
    npf_list_pushBack(&g_atm_pendinglist, uc_info);


    /*Call into the IXASDK*/
    
    if(err != IX_SUCCESS)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "IXASDK30 API call failed error code:%d",err);
        iRet = -1;
        goto finish;
    }
    
finish:
    if(iRet == -1)
    {
        if(u_c)    
            PIL_FreeMemory(u_c);
        /*inform the Translator component about failure via the up-call*/
            
        fpm_atm_mgr_oam_upcall(iRet, t_c);
    }
    
    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Leaving oam setup request");

}

/*****************************************************************************
*   Function:  atm_oam_prop_get_req
*
*   Parameters:
*                t_c : translator context
*   Description:
*                This function requests the IXASDK for oam get
*                It provides a callback function to the IXASDK to
                 call back with
*                the status on oam get
*   Return: none
*****************************************************************************/


void atm_oam_prop_get_req(atm_OamProperties_TranslatorContext_t* t_c)
{

    int32_t iRet = 0;
    atm_uc_info *uc_info;
    atm_OamProperties_UserContext_t *u_c = NULL;
    ix_error err;
    ix_atm_conn_id conn;
    ix_uint8 direction;

    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Entering oam setup request ");

    u_c = (atm_OamProperties_UserContext_t *)
          PIL_AllocateMemory(sizeof(atm_OamProperties_UserContext_t), 
          PIL_MEM_ZERO_INIT);
    if(!u_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Mem allocation for u_c failed");
        iRet = -1;
        goto finish;
    }

    /* Fill the oam setup information as per the cbType */
    
    u_c->t_c = t_c;

    uc_info = (atm_uc_info *)PIL_AllocateMemory(
    sizeof(atm_uc_info ), PIL_MEM_ZERO_INIT);
    if(!uc_info)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "Mem allocation failed for uc_info");
        iRet = -1;
        goto finish;
    }
    uc_info->type = FPPI_ATM_OamPropGet;
    uc_info->u_c = (void *)u_c;
    /*insert this u_c info into the u_c pending list*/
    npf_list_pushBack(&g_atm_pendinglist, uc_info);


    /*Call into the IXASDK*/

    if(err != IX_SUCCESS)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "IXASDK30 API call failed error code:%d",err);
        iRet = -1;
        goto finish;
    }
    
finish:
    if(iRet == -1)
    {
        if(u_c)    
            PIL_FreeMemory(u_c);
        /*inform the Translator component about failure via the up-call*/
            
        fpm_atm_mgr_OamProperties_upcall(iRet, t_c);
    }
    
    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Leaving oam prop get");
}

/*****************************************************************************
*   Function:  atm_oam_stat_get_req
*
*   Parameters:
*                t_c : translator context
*   Description:
*                This function requests the IXASDK for oam stat get
*                It provides a callback function to the IXASDK to
                 call back with
*                the status on oam stat get
*   Return: none
*****************************************************************************/


void atm_oam_stat_get_req(atm_OamStatistics_TranslatorContext_t* t_c)
{
    int32_t iRet = 0;
    atm_uc_info *uc_info;
    atm_OamStatistics_UserContext_t *u_c = NULL;
    ix_error err;
    ix_atm_conn_id conn;
    ix_uint8 direction;

    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Entering oam setup request ");

    u_c = (atm_OamStatistics_UserContext_t *)PIL_AllocateMemory(
           sizeof(atm_OamStatistics_UserContext_t), PIL_MEM_ZERO_INIT);
    if(!u_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Mem allocation for u_c failed");
        iRet = -1;
        goto finish;
    }

    /* Fill the oam setup information as per the cbType */

    
    u_c->t_c = t_c;

    uc_info = (atm_uc_info *)PIL_AllocateMemory(
    sizeof(atm_uc_info ), PIL_MEM_ZERO_INIT);
    if(!uc_info)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "Mem allocation failed for uc_info");
        iRet = -1;
        goto finish;
    }
    uc_info->type = FPPI_ATM_OamStatGet;
    uc_info->u_c = (void *)u_c;
    /*insert this u_c info into the u_c pending list*/
    npf_list_pushBack(&g_atm_pendinglist, uc_info);


    /*Call into the IXASDK*/


    if(err != IX_SUCCESS)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "IXASDK30 API call failed error code:%d",err);
        iRet = -1;
        goto finish;
    }
    
finish:
    if(iRet == -1)
    {
        if(u_c)    
            PIL_FreeMemory(u_c);
        /*inform the Translator component about failure via the up-call*/
            
        fpm_atm_mgr_OamStatistics_upcall(iRet, t_c);
    }
    
    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Leaving oam stat get ");

}

/*****************************************************************************
*   Function:  atm_conn_stat_get_req
*
*   Parameters:
*                t_c : translator context
*   Description:
*                This function requests the IXASDK for conn stat get
*                It provides a callback function to the IXASDK to
                 call back with
*                the status on conn stat get
*   Return: none
*****************************************************************************/

void atm_conn_stat_get_req(atm_ConnStatistics_TranslatorContext_t* t_c)
{
    int32_t iRet = 0;
    atm_uc_info *uc_info;
    atm_ConnStatistics_UserContext_t *u_c = NULL;
    ix_error err;
    ix_atm_conn_id conn;

    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Entering oam setup request ");

    u_c = (atm_ConnStatistics_UserContext_t *)PIL_AllocateMemory(
           sizeof(atm_ConnStatistics_UserContext_t), PIL_MEM_ZERO_INIT);
    if(!u_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Mem allocation for u_c failed");
        iRet = -1;
        goto finish;
    }

    /* Fill the oam setup information as per the cbType */

    
    u_c->t_c = t_c;

    uc_info = (atm_uc_info *)PIL_AllocateMemory(
    sizeof(atm_uc_info ), PIL_MEM_ZERO_INIT);
    if(!uc_info)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR,
            "Mem allocation failed for uc_info");
        iRet = -1;
        goto finish;
    }
    uc_info->type = FPPI_ATM_ConnStatGet;
    uc_info->u_c = (void *)u_c;
    /*insert this u_c info into the u_c pending list*/
    npf_list_pushBack(&g_atm_pendinglist, uc_info);

    /*Call into the IXASDK*/

    if(err != IX_SUCCESS)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "IXASDK30 API call failed error code:%d",err);
        iRet = -1;
        goto finish;
    }
    
finish:
    if(iRet == -1)
    {
        if(u_c)    
            PIL_FreeMemory(u_c);
        /*inform the Translator component about failure via the up-call*/
            
        fpm_atm_mgr_ConnStatistics_upcall(iRet, t_c);
    }
    
    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "leaving conn stat get");

}

/*****************************************************************************
*   Function:  atm_trafficIndicatorSet
*
*   Parameters:
*                t_c : translator context
*   Description:
*                This function requests the IXASDK for traffic indicator set
*                It provides a callback function to the IXASDK to
                 call back with
*                the status on set
*   Return: none
*****************************************************************************/

void atm_trafficIndicatorSet(atm_TranslatorContext_t* t_c)
{
    int32_t iRet = 0;
    atm_uc_info *uc_info;
    atm_UserContext_t *u_c = NULL;
    ix_error err;
    ix_atm_conn_id conn;
    ix_uint8 flag;

    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Entering oam setup request ");

    u_c = (atm_UserContext_t *)PIL_AllocateMemory(sizeof(atm_UserContext_t), PIL_MEM_ZERO_INIT);
    if(!u_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Mem allocation for u_c failed");
        iRet = -1;
        goto finish;
    }

    /* Fill the oam setup information as per the cbType */

    
    u_c->t_c = t_c;

    uc_info = (atm_uc_info *)PIL_AllocateMemory(
    sizeof(atm_uc_info ), PIL_MEM_ZERO_INIT);
    if(!uc_info)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "Mem allocation failed for uc_info");
        iRet = -1;
        goto finish;
    }
    uc_info->type = FPPI_ATM_TrafficIndicatorSet;
    uc_info->u_c = (void *)u_c;
    /*insert this u_c info into the u_c pending list*/
    npf_list_pushBack(&g_atm_pendinglist, uc_info);


    /*Call into the IXASDK*/

    if(err != IX_SUCCESS)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "IXASDK30 API call failed error code:%d",err);
        iRet = -1;
        goto finish;
    }
    
finish:
    if(iRet == -1)
    {
        if(u_c)    
            PIL_FreeMemory(u_c);
        /*inform the Translator component about failure via the up-call*/
            
        fpm_atm_mgr_upcall(iRet, t_c);
    }
    
    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Leaving oam setup request");

}

/*****************************************************************************
*   Function:  atm_stats_req_desc
*
*   Parameters:
*                t_c : translator context
*   Description:
*                This function requests the IXASDK for conn statistics set
*                It provides a callback function to the IXASDK to
                 call back with
*                the status on setup
*   Return: none
*****************************************************************************/


void atm_stats_req_desc(atm_TranslatorContext_t* t_c)
{
    int32_t iRet = 0;
    atm_uc_info *uc_info;
    atm_UserContext_t *u_c = NULL;
    ix_error err;
    ix_atm_statistics_req stat_req;

    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Entering oam setup request ");

    u_c = (atm_UserContext_t *)PIL_AllocateMemory(sizeof(atm_UserContext_t), PIL_MEM_ZERO_INIT);
    if(!u_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Mem allocation for u_c failed");
        iRet = -1;
        goto finish;
    }

    /* copy the statistics request information into the corresponding IX structure */

    
    u_c->t_c = t_c;

    uc_info = (atm_uc_info *)PIL_AllocateMemory(
    sizeof(atm_uc_info ), PIL_MEM_ZERO_INIT);
    if(!uc_info)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "Mem allocation failed for uc_info");
        iRet = -1;
        goto finish;
    }
    uc_info->type = FPPI_ATM_ConnStatIndSet;
    uc_info->u_c = (void *)u_c;
    /*insert this u_c info into the u_c pending list*/
    npf_list_pushBack(&g_atm_pendinglist, uc_info);

    /*Call into the IXASDK*/
    

    if(err != IX_SUCCESS)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "IXASDK30 API call failed error code:%d",err);
        iRet = -1;
        goto finish;
    }
    
finish:
    if(iRet == -1)
    {
        if(u_c)    
            PIL_FreeMemory(u_c);
        /*inform the Translator component about failure via the up-call*/
            
        fpm_atm_mgr_upcall(iRet, t_c);
    }
    
    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Leaving oam setup request");
}






/*****************************************************************************
******************************************************************************
*                            Callback Function Implementations
******************************************************************************
*****************************************************************************/


/*****************************************************************************
*
*   Function:   atm_cb()
*
*   Parameters: 
*            arg_Result:error code returned from the IXASDK messaging operation.
*            arg_pContext:pointer to user context        
*
*   Description:Callback function that contains the status information 
*                Once all the required status is available the translator 
*                fpm_atm_mgr_upcall() is invoked.
*
*   Return: 0 on success, -1 on failure
*****************************************************************************/

ix_error atm_cb(ix_error arg_Result,void* arg_pContext)
{
    int32_t iRet = 0;
    atm_UserContext_t * u_c = NULL;
    atm_TranslatorContext_t * t_c = NULL;
    
    if(!arg_pContext)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Invalid pContext");
        iRet = -1;
        goto finish;
    }
    /*validate the u_c*/
    if(!atm_is_Valid(arg_pContext))
    {
        iRet =  -1;
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "invalid u_c:0x%x", 
            arg_pContext);
        goto finish;
    }

    u_c = (atm_UserContext_t *)arg_pContext;
    t_c = (atm_TranslatorContext_t *)u_c->t_c;
    if(!t_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, 
            "t_c is invalid");
        iRet =  -1;
        goto finish;
    }
    
    
    npf_logger_write(LOG_FP_ATM, LOG_INFO, "arg_Result:%d", arg_Result);

    
    if(arg_Result == NPF_NO_ERROR)
            fpm_atm_mgr_upcall(0, t_c);
        else
            fpm_atm_mgr_upcall(-1, t_c);

    /*free the u_c for this callback*/
    PIL_FreeMemory(u_c);

finish:/*only if something is seriously wrong. Note that the u_c's and the 
       related resources they point to will be anyhow freed during shut down*/

    if(iRet == -1)
        fpm_atm_mgr_upcall(iRet, t_c);
    return iRet;
}


/*****************************************************************************
*
*   Function:   atm_oam_cb()
*
*   Parameters:
*            arg_Result:error code returned from the IXASDK messaging operation.
*            arg_pContext:pointer to user context
*
*   Description:Callback function that contains the status information
*                Once all the required status is available the translator
*                fpm_atm_mgr_upcall() is invoked.
*
*   Return: 0 on success, -1 on failure
*****************************************************************************/



ix_error atm_oam_cb(ix_error arg_Result,void* arg_pContext)
{
    int32_t iRet = 0;
    atm_Oam_UserContext_t * u_c = NULL;
    atm_oam_TranslatorContext_t * t_c = NULL;
   
    if(!arg_pContext)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Invalid pContext");
        iRet = -1;
        goto finish;
    }
    /*validate the u_c*/
    if(!atm_is_Valid(arg_pContext))
    {
        iRet =  -1;
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "invalid u_c:0x%x",
            arg_pContext);
        goto finish;
    }

    u_c = (atm_Oam_UserContext_t *)arg_pContext;
    t_c = (atm_oam_TranslatorContext_t *)u_c->t_c;
    if(!t_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR,
            "t_c is invalid");
        iRet =  -1;
        goto finish;
    }


    npf_logger_write(LOG_FP_ATM, LOG_INFO, "arg_Result:%d", arg_Result);

   
    if(arg_Result == NPF_NO_ERROR)
            fpm_atm_mgr_oam_upcall(0, t_c);
        else
            fpm_atm_mgr_oam_upcall(-1, t_c);

    /*free the u_c for this callback*/
    PIL_FreeMemory(u_c);

finish:/*only if something is seriously wrong. Note that the u_c's and the
       related resources they point to will be anyhow freed during shut down*/

    if(iRet == -1)
        fpm_atm_mgr_oam_upcall(iRet, t_c);
    return iRet;
}


/*****************************************************************************
*
*   Function:   atm_linkAttribs_cb()
*
*   Parameters:
*            arg_Result:error code returned from the IXASDK messaging operation.
*            arg_pContext:pointer to user context
*
*   Description:Callback function that contains the status information
*                Once all the required status is available the translator
*                fpm_atm_mgr_upcall() is invoked.
*
*   Return: 0 on success, -1 on failure
*****************************************************************************/

ix_error atm_linkAttribs_cb(ix_error arg_Result,void* arg_pContext)
{
    int32_t iRet = 0;
    atm_linkAttribs_UserContext_t * u_c = NULL;
    atm_linkAttribs_TranslatorContext_t * t_c = NULL;
   
    if(!arg_pContext)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Invalid pContext");
        iRet = -1;
        goto finish;
    }
    /*validate the u_c*/
    if(!atm_is_Valid(arg_pContext))
    {
        iRet =  -1;
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "invalid u_c:0x%x",
            arg_pContext);
        goto finish;
    }

    u_c = (atm_linkAttribs_UserContext_t *)arg_pContext;
    t_c = (atm_linkAttribs_TranslatorContext_t *)u_c->t_c;
    if(!t_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR,
            "t_c is invalid");
        iRet =  -1;
        goto finish;
    }


    npf_logger_write(LOG_FP_ATM, LOG_INFO, "arg_Result:%d", arg_Result);

   
    if(arg_Result == NPF_NO_ERROR)
            fpm_atm_mgr_linkAttribs_upcall(0, t_c);
        else
            fpm_atm_mgr_linkAttribs_upcall(-1, t_c);

    /*free the u_c for this callback*/
    PIL_FreeMemory(u_c);

finish:/*only if something is seriously wrong. Note that the u_c's and the
       related resources they point to will be anyhow freed during shut down*/

    if(iRet == -1)
        fpm_atm_mgr_linkAttribs_upcall(iRet, t_c);
    return iRet;
}


/*****************************************************************************
*
*   Function:   atm_OamProperties_cb()
*
*   Parameters:
*            arg_Result:error code returned from the IXASDK messaging operation.
*            arg_pContext:pointer to user context
*
*   Description:Callback function that contains the status information
*                Once all the required status is available the translator
*                fpm_atm_mgr_upcall() is invoked.
*
*   Return: 0 on success, -1 on failure
*****************************************************************************/

ix_error atm_OamProperties_cb(ix_error arg_Result,void* arg_pContext)
{
    int32_t iRet = 0;
    atm_OamProperties_UserContext_t * u_c = NULL;
    atm_OamProperties_TranslatorContext_t * t_c = NULL;
   
    if(!arg_pContext)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Invalid pContext");
        iRet = -1;
        goto finish;
    }
    /*validate the u_c*/
    if(!atm_is_Valid(arg_pContext))
    {
        iRet =  -1;
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "invalid u_c:0x%x",
            arg_pContext);
        goto finish;
    }

    u_c = (atm_OamProperties_UserContext_t *)arg_pContext;
    t_c = (atm_OamProperties_TranslatorContext_t *)u_c->t_c;
    if(!t_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR,
            "t_c is invalid");
        iRet =  -1;
        goto finish;
    }


    npf_logger_write(LOG_FP_ATM, LOG_INFO, "arg_Result:%d", arg_Result);

   
    if(arg_Result == NPF_NO_ERROR)
            fpm_atm_mgr_OamProperties_upcall(0, t_c);
        else
            fpm_atm_mgr_OamProperties_upcall(-1, t_c);

    /*free the u_c for this callback*/
    PIL_FreeMemory(u_c);

finish:/*only if something is seriously wrong. Note that the u_c's and the
       related resources they point to will be anyhow freed during shut down*/

    if(iRet == -1)
        fpm_atm_mgr_OamProperties_upcall(iRet, t_c);
    return iRet;
}

/*****************************************************************************
*
*   Function:   atm_OamStatistics_cb()
*
*   Parameters:
*            arg_Result:error code returned from the IXASDK messaging operation.
*            arg_pContext:pointer to user context
*
*   Description:Callback function that contains the status information
*                Once all the required status is available the translator
*                fpm_atm_mgr_upcall() is invoked.
*
*   Return: 0 on success, -1 on failure
*****************************************************************************/


ix_error atm_OamStatistics_cb(ix_error arg_Result,void* arg_pContext)
{
    int32_t iRet = 0;
    atm_OamStatistics_UserContext_t * u_c = NULL;
    atm_OamStatistics_TranslatorContext_t * t_c = NULL;
   
    if(!arg_pContext)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Invalid pContext");
        iRet = -1;
        goto finish;
    }
    /*validate the u_c*/
    if(!atm_is_Valid(arg_pContext))
    {
        iRet =  -1;
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "invalid u_c:0x%x",
            arg_pContext);
        goto finish;
    }

    u_c = (atm_OamStatistics_UserContext_t *)arg_pContext;
    t_c = (atm_OamStatistics_TranslatorContext_t *)u_c->t_c;
    if(!t_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR,
            "t_c is invalid");
        iRet =  -1;
        goto finish;
    }


    npf_logger_write(LOG_FP_ATM, LOG_INFO, "arg_Result:%d", arg_Result);

   
    if(arg_Result == NPF_NO_ERROR)
            fpm_atm_mgr_OamStatistics_upcall(0, t_c);
        else
            fpm_atm_mgr_OamStatistics_upcall(-1, t_c);

    /*free the u_c for this callback*/
    PIL_FreeMemory(u_c);

finish:/*only if something is seriously wrong. Note that the u_c's and the
       related resources they point to will be anyhow freed during shut down*/

    if(iRet == -1)
        fpm_atm_mgr_OamStatistics_upcall(iRet, t_c);
    return iRet;
}


/*****************************************************************************
*
*   Function:   atm_ConnStatistics_cb()
*
*   Parameters:
*            arg_Result:error code returned from the IXASDK messaging operation.
*            arg_pContext:pointer to user context
*
*   Description:Callback function that contains the status information
*                Once all the required status is available the translator
*                fpm_atm_mgr_upcall() is invoked.
*
*   Return: 0 on success, -1 on failure
*****************************************************************************/

ix_error atm_ConnStatistics_cb(ix_error arg_Result,void* arg_pContext)
{
    int32_t iRet = 0;
    atm_ConnStatistics_UserContext_t * u_c = NULL;
    atm_ConnStatistics_TranslatorContext_t * t_c = NULL;
   
    if(!arg_pContext)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Invalid pContext");
        iRet = -1;
        goto finish;
    }
    /*validate the u_c*/
    if(!atm_is_Valid(arg_pContext))
    {
        iRet =  -1;
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "invalid u_c:0x%x",
            arg_pContext);
        goto finish;
    }

    u_c = (atm_ConnStatistics_UserContext_t *)arg_pContext;
    t_c = (atm_ConnStatistics_TranslatorContext_t *)u_c->t_c;
    if(!t_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR,
            "t_c is invalid");
        iRet =  -1;
        goto finish;
    }


    npf_logger_write(LOG_FP_ATM, LOG_INFO, "arg_Result:%d", arg_Result);

   
    if(arg_Result == NPF_NO_ERROR)
            fpm_atm_mgr_ConnStatistics_upcall(0, t_c);
        else
            fpm_atm_mgr_ConnStatistics_upcall(-1, t_c);

    /*free the u_c for this callback*/
    PIL_FreeMemory(u_c);

finish:/*only if something is seriously wrong. Note that the u_c's and the
       related resources they point to will be anyhow freed during shut down*/

    if(iRet == -1)
        fpm_atm_mgr_ConnStatistics_upcall(iRet, t_c);
    return iRet;
}



/*****************************************************************************
*
*   Function:   atm_is_Valid()
*
*   Parameters: 
*            u_c: Pointer to u_c
*
*   Description: 
*                Searches for the u_c in the pending u_c list, pops out the
*                entry if the match is found 
*
*   Return: 1 for if match found, 0 if match is not found
*****************************************************************************/
uint32_t atm_is_Valid(void *u_c)
{
    atm_uc_info    *uc_info = NULL;
    DListItr *itr;
    int found = 0;

    if(!u_c)
    {
        npf_logger_write(LOG_FP_ATM, LOG_ERR, "Invalid u_c");
        return 0;
    }

    npf_logger_write(LOG_FP_ATM, LOG_TRACE,
        "Entering cm_is_Valid with u_c:0x%x", u_c);
    
    /*walk through the list    to look for a match*/
    itr = npf_list_itrCreate(&g_atm_pendinglist);
    if (npf_list_itrFirst(itr) != -1) 
    {
        do {
                uc_info = (atm_uc_info *)npf_list_itrGetData(itr);
                if(uc_info->u_c == u_c)
                {
                    found = 1;
                    npf_logger_write(LOG_FP_ATM, LOG_TRACE,
                        "Found a match");
                    npf_list_itrRemoveFree(itr);
                }
            } while (!found && npf_list_itrNext(itr) != -1);
    }    
    npf_list_itrDelete(itr);    
    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Leaving atm_is_Valid");
    
    if(found)
        return 1;
    return 0;

}


/*****************************************************************************
*
*   Function:   atm_free_resources()
*
*   Parameters: 
*            none
*
*   Description: 
*                Walks thro' the u_c pending list and frees up the resources
                and appropriately calls into the translator upcall
*
*   Return: none
*****************************************************************************/
void atm_free_resources()
{
    atm_uc_info    *uc_info = NULL;
    DListItr        *itr;

    npf_logger_write(LOG_FP_ATM, LOG_TRACE,
        "Entering atm_free_resources with");
    
    /*walk through the list    to look for a match*/
    itr = npf_list_itrCreate(&g_atm_pendinglist);
    if (npf_list_itrFirst(itr) != -1) 
    {
        do {
                uc_info = (atm_uc_info *)npf_list_itrGetData(itr);

                if(uc_info)
                {
                    if(uc_info->u_c)
                    {
                        /*call into the translator*/
                        fpm_atm_mgr_upcall( -1,
                            ((atm_UserContext_t *)(uc_info->u_c))->t_c
                            );
                        
                        PIL_FreeMemory(uc_info->u_c);                            
                    }
                    npf_list_itrRemoveFree(itr);
                }
            } while (npf_list_itrNext(itr) != -1);
    }    
    npf_list_itrDelete(itr);    
    npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Leaving atm_free_resources");    
}

