/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2002 Intel Corporation. All Rights Reserved.
*****************************************************************************/

#include <assert.h>
#include "npf_common.h"
#include "npf_cm_types.h"
#include "pdk_cm_types.h"
#include "pdk_callback.h"
#include "pdk_callback_types.h"
#include "pil.h"
#include "log.h"
#include "fpp_common.h"
#include "list.h"
#include "npf_if.h"
#include "FPModule.h"

#include "pdk_hash.h"
#include "backendapi.h"


#include "npf_atm_types.h"
#include "fppapi_atm_types.h"

#include "atm_mgr.h"
#include "atm_mgr_translator.h"
#include "ix_cc_error.h"
#include "cc/ix_cc_atmsar.h"
#include "atm_mgr_impl.h"


/* Callback Handles - Callbacks registered with Back end API */
FPPI_CBHANDLE fppiIfAtmCbHandle[NPF_IF_ATM_MAX_CB_TYPES];


/* Contexts of callbacks registered with BEND API */

FPPI_CONTEXT fppiIfAtmContext[NPF_IF_ATM_MAX_CB_TYPES];



/*****************************************************************************
*    Name: fpm_atm_conn_mgr_init
*
*    Description:
*			This function registers with the Back End API all the call back functions.
*			It also call the Core Component Initialization function.
*
*    Parameters:
*
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*
*    Returns:
*        FPPI_SUCCESS                 - everything is fine
*        FPPI_FAIL                  - Failed to register CB with backend API 
*
********************************************************************************/
NPF_uint32_t fpm_atm_conn_mgr_init(void)
{

	NPF_uint32_t ret = FPPI_SUCCESS;

    /* Assigning Contexts for registering for callbacks */

	fppiIfAtmContext[NPF_IF_ATM_ATTR_SET_CB] = (void *)FPPI_ATM_IfAttrSet;
	fppiIfAtmContext[NPF_IF_ATM_IF_DELETE_CB] = (void *)FPPI_ATM_IfDelete;
	fppiIfAtmContext[NPF_IF_ATM_IF_ENABLE_CB] = (void *)FPPI_ATM_IfEnable;
	fppiIfAtmContext[NPF_IF_ATM_IF_DISABLE_CB] = (void *)FPPI_ATM_IfDisable;
	fppiIfAtmContext[NPF_IF_ATM_IF_GEN_STATS_GET_CB] = (void *)FPPI_ATM_IfGenStatsGet;
	fppiIfAtmContext[NPF_IF_ATM_VCC_SET_CB] = (void *)FPPI_ATM_VccSet;
	fppiIfAtmContext[NPF_IF_ATM_VCC_BIND_CB] = (void *)FPPI_ATM_VccBind;
	fppiIfAtmContext[NPF_IF_ATM_VCC_DEL_CB] = (void *)FPPI_ATM_VccDelete;
	fppiIfAtmContext[NPF_IF_ATM_VCC_ENABLE_CB] =	(void *)FPPI_ATM_VccEnable;
	fppiIfAtmContext[NPF_IF_ATM_VCC_DISABLE_CB] = (void *)FPPI_ATM_VccDisable;
	fppiIfAtmContext[NPF_IF_ATM_VCC_OPER_STATUS_GET_CB] = (void *)FPPI_ATM_VccOperStatusGet;
	fppiIfAtmContext[NPF_IF_ATM_VCC_STATS_GET_CB] = (void *)FPPI_ATM_VccStatsGet;
	fppiIfAtmContext[NPF_IF_ATM_AAL2_CHANNEL_SET_CB] = (void *)FPPI_ATM_AAL2_ChannelSet;
	fppiIfAtmContext[NPF_IF_ATM_AAL2_CHANNEL_DEL_CB] = (void *)FPPI_ATM_AAL2_ChannelDel;
	fppiIfAtmContext[NPF_IF_ATM_CONNECTION_XC_SET_CB] = (void *)FPPI_ATM_ConnectionXcSet;
	fppiIfAtmContext[NPF_IF_ATM_CONNECTION_XC_DEL_CB] = (void *)FPPI_ATM_ConnectionXcDel;
	fppiIfAtmContext[NPF_IF_ATM_OAM_CP_SET_CB] = (void *)FPPI_ATM_OAM_CP_Set;
	fppiIfAtmContext[NPF_IF_ATM_OAM_CC_SET_CB] = (void *)FPPI_ATM_OAM_CC_Set;
	fppiIfAtmContext[NPF_IF_ATM_OAM_LB_SET_CB] = (void *)FPPI_ATM_OAM_LB_Set;
	fppiIfAtmContext[NPF_IF_ATM_OAM_MONITOR_CB] = (void *)FPPI_ATM_OAM_Monitor;
	fppiIfAtmContext[NPF_IF_ATM_OAM_ALARM_SET_CB] = (void *)FPPI_ATM_OAM_AlarmSet;
	fppiIfAtmContext[NPF_IF_ATM_OAM_ALARM_CLEAR_CB] = (void *)FPPI_ATM_OAM_AlarmClear;
	fppiIfAtmContext[NPF_IF_ATM_OAM_PM_SET_CB] = (void *)FPPI_ATM_OAM_PM_Set;

	ret = BENDAPI_register_callback (&fpmIfATM_IfAttrSetFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_ATTR_SET_CB], 
							FPPI_ATM_IfAttrSet, 
							fppiIfAtmContext[NPF_IF_ATM_ATTR_SET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for IfAttrSet  Failed\n");

		return FPPI_FAIL;
	}

	
	ret = BENDAPI_register_callback (&fpmIfATM_IfDeleteFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_IF_DELETE_CB], 
							FPPI_ATM_IfDelete, 
							fppiIfAtmContext[NPF_IF_ATM_IF_DELETE_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for IfDelete  Failed\n");

		return FPPI_FAIL;
	}

	ret = BENDAPI_register_callback (&fpmIfATM_IfEnableFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_IF_ENABLE_CB], 
							FPPI_ATM_IfEnable, 
							fppiIfAtmContext[NPF_IF_ATM_IF_ENABLE_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for IfEnable  Failed\n");

		return FPPI_FAIL;
	}

	ret = BENDAPI_register_callback (&fpmIfATM_IfDisableFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_IF_DISABLE_CB], 
							FPPI_ATM_IfDisable, 
							fppiIfAtmContext[NPF_IF_ATM_IF_DISABLE_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for IfDisable  Failed\n");

		return FPPI_FAIL;
	}

	ret = BENDAPI_register_callback (&fpmIfATM_IfGenStatsGetFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_IF_GEN_STATS_GET_CB], 
							FPPI_ATM_IfGenStatsGet, 
							fppiIfAtmContext[NPF_IF_ATM_IF_GEN_STATS_GET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for IfGenStatsGet  Failed\n");

		return FPPI_FAIL;
	}


	ret = BENDAPI_register_callback (&fpmIfATM_VccSetFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_VCC_SET_CB], 
							FPPI_ATM_VccSet, 
							fppiIfAtmContext[NPF_IF_ATM_VCC_SET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for VccSet  Failed\n");

		return FPPI_FAIL;
	}

	ret = BENDAPI_register_callback (&fpmIfATM_VccBindFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_VCC_BIND_CB], 
							FPPI_ATM_VccBind, 
							fppiIfAtmContext[NPF_IF_ATM_VCC_BIND_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for VccBind  Failed\n");

		return FPPI_FAIL;
	}



	ret = BENDAPI_register_callback (&fpmIfATM_VccDelFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_VCC_DEL_CB], 
							FPPI_ATM_VccDelete, 
							fppiIfAtmContext[NPF_IF_ATM_VCC_DEL_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for VccDel  Failed\n");

		return FPPI_FAIL;
	}

	ret = BENDAPI_register_callback (&fpmIfATM_VccEnableFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_VCC_ENABLE_CB], 
							FPPI_ATM_VccEnable, 
							fppiIfAtmContext[NPF_IF_ATM_VCC_ENABLE_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for VccEnable  Failed\n");

		return FPPI_FAIL;
	}

	
	ret = BENDAPI_register_callback (&fpmIfATM_VccDisableFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_VCC_DISABLE_CB], 
							FPPI_ATM_VccDisable, 
							fppiIfAtmContext[NPF_IF_ATM_VCC_DISABLE_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for VccDisable  Failed\n");

		return FPPI_FAIL;
	}

	

	ret = BENDAPI_register_callback (&fpmIfATM_VccOperStatusGetFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_VCC_OPER_STATUS_GET_CB], 
							FPPI_ATM_VccOperStatusGet, 
							fppiIfAtmContext[NPF_IF_ATM_VCC_OPER_STATUS_GET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for VccOperStatusGet  Failed\n");

		return FPPI_FAIL;
	}

	
	ret = BENDAPI_register_callback (&fpmIfATM_VccStatsGetFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_VCC_STATS_GET_CB], 
							FPPI_ATM_VccStatsGet, 
							fppiIfAtmContext[NPF_IF_ATM_VCC_STATS_GET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for VccStatsGet  Failed\n");

		return FPPI_FAIL;
	}

	
	ret = BENDAPI_register_callback (&fpmIfATM_AAL2_ChannelSetFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_AAL2_CHANNEL_SET_CB], 
							FPPI_ATM_AAL2_ChannelSet, 
							fppiIfAtmContext[NPF_IF_ATM_AAL2_CHANNEL_SET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for AAL2_ChannelSet  Failed\n");

		return FPPI_FAIL;
	}

	
	ret = BENDAPI_register_callback (&fpmIfATM_AAL2_ChannelDelFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_AAL2_CHANNEL_DEL_CB], 
							FPPI_ATM_AAL2_ChannelDel, 
							fppiIfAtmContext[NPF_IF_ATM_AAL2_CHANNEL_DEL_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for AAL2_ChannelDel  Failed\n");

		return FPPI_FAIL;
	}

	
	ret = BENDAPI_register_callback (&fpmIfATM_ConnectionXcSetFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_CONNECTION_XC_SET_CB], 
							FPPI_ATM_ConnectionXcSet, 
							fppiIfAtmContext[NPF_IF_ATM_CONNECTION_XC_SET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for ATM_ConnectionXcSet  Failed\n");

		return FPPI_FAIL;
	}

	
	ret = BENDAPI_register_callback (&fpmIfATM_ConnectionXcDelFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_CONNECTION_XC_DEL_CB], 
							FPPI_ATM_ConnectionXcDel, 
							fppiIfAtmContext[NPF_IF_ATM_CONNECTION_XC_DEL_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for ATM_ConnectionXcDel  Failed\n");

		return FPPI_FAIL;
	}

	ret = BENDAPI_register_callback (&fpmIfATM_OAM_CP_SetFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_OAM_CP_SET_CB], 
							FPPI_ATM_OAM_CP_Set, 
							fppiIfAtmContext[NPF_IF_ATM_OAM_CP_SET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for ATM_OAM_CP_Set  Failed\n");

		return FPPI_FAIL;
	}

	
	ret = BENDAPI_register_callback (&fpmIfATM_OAM_CC_SetFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_OAM_CC_SET_CB], 
							FPPI_ATM_OAM_CC_Set, 
							fppiIfAtmContext[NPF_IF_ATM_OAM_CC_SET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for ATM_OAM_CC_Set  Failed\n");

		return FPPI_FAIL;
	}

	
	ret = BENDAPI_register_callback (&fpmIfATM_OAM_LB_SetFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_OAM_LB_SET_CB], 
							FPPI_ATM_OAM_LB_Set, 
							fppiIfAtmContext[NPF_IF_ATM_OAM_LB_SET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for ATM_OAM_LB_Set  Failed\n");

		return FPPI_FAIL;
	}

	ret = BENDAPI_register_callback (&fpmIfATM_OAM_MonitorFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_OAM_MONITOR_CB], 
							FPPI_ATM_OAM_Monitor, 
							fppiIfAtmContext[NPF_IF_ATM_OAM_MONITOR_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for ATM_OAM_Monitor  Failed\n");

		return FPPI_FAIL;
	}

	
	ret = BENDAPI_register_callback (&fpmIfATM_OAM_AlarmSetFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_OAM_ALARM_SET_CB], 
							FPPI_ATM_OAM_AlarmSet, 
							fppiIfAtmContext[NPF_IF_ATM_OAM_ALARM_SET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for ATM_OAM_AlarmSet  Failed\n");

		return FPPI_FAIL;
	}

	
	ret = BENDAPI_register_callback (&fpmIfATM_OAM_AlarmClearFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_OAM_ALARM_CLEAR_CB], 
							FPPI_ATM_OAM_AlarmClear, 
							fppiIfAtmContext[NPF_IF_ATM_OAM_ALARM_CLEAR_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for ATM_OAM_AlarmClear  Failed\n");

		return FPPI_FAIL;
	}

	
	ret = BENDAPI_register_callback (&fpmIfATM_OAM_PM_SetFunc, 
							&fppiIfAtmCbHandle[NPF_IF_ATM_OAM_PM_SET_CB], 
							FPPI_ATM_OAM_PM_Set, 
							fppiIfAtmContext[NPF_IF_ATM_OAM_PM_SET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"Registering for ATM_OAM_PM_Set  Failed\n");

		return FPPI_FAIL;
	}

	
	/* Call the FP blade SDK specifics for Initialization*/
	fpmIfATM_initFeLimePoint();
	return FPPI_SUCCESS;
}



/*****************************************************************************
*    Name: fpm_atm_mgr_shutdown
*
*    Description:
*			This function registers with the Back End API all the call back functions.
*			It also call the Core Component Initialization function.
*   Description: This function shuts-down the FPModule. It deregisters for 
*					callbacks with the Back-end API, de-initializes the FP Engine
*					Cleans up the threads and sets the FP Manager shutdown flag. It
*					Calls the specific SDK function for respective cleanup.
*
*    Parameters:
*
*        None
*
*    Preconditions:
*
*    Postconditions:
*
*
*    Returns:
*        FPPI_SUCCESS                 - everything is fine
*			D_PARAMETERSURE                  - Failed to register CB with backend API 
*
********************************************************************************/
NPF_uint32_t fpm_atm_mgr_shutdown(void)
{
	NPF_uint32_t ret = FPPI_SUCCESS;
     
	npf_logger_write(LOG_FP_ATM, LOG_TRACE, "Shutdown will happen here \n");
    
	ret = BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_ATTR_SET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"De-registering for  Attr Set Failed\n");
		return (FPPI_FAILURE);
	}

	ret = BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_IF_DELETE_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"De-registering for  If Delete Failed\n");
		return (FPPI_FAILURE);
	}

	ret = BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_IF_ENABLE_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"De-registering for  If Enable Failed\n");
		return (FPPI_FAILURE);
	}

	ret = BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_IF_DISABLE_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"De-registering for  If Disable Failed\n");
		return (FPPI_FAILURE);
	}

	ret = BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_IF_GEN_STATS_GET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"De-registering for  If Gen Stats Get Failed\n");
		return (FPPI_FAILURE);
	}

	ret = BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_VCC_SET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"De-registering for  Vcc Set Failed\n");
		return (FPPI_FAILURE);
	}

	ret = BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_VCC_BIND_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
			"De-registering for  Vcc Bind Failed\n");
		return (FPPI_FAILURE);
	}

	ret =BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_VCC_DEL_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
				"De-registering for  Vcc Delete Failed\n");
		return (FPPI_FAILURE);
	}

	ret =BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_VCC_ENABLE_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
				"De-registering for Vcc Enable Failed\n");
		return (FPPI_FAILURE);
	}

	ret =BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_VCC_DISABLE_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
				"De-registering for Vcc Disable Failed\n");
		return (FPPI_FAILURE);
	}


	ret =BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_VCC_OPER_STATUS_GET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
				"De-registering for VccOperStatusGet  Failed\n");
		return (FPPI_FAILURE);
	}


	ret =BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_VCC_STATS_GET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
				"De-registering for VccStatsGet  Failed\n");
		return (FPPI_FAILURE);
	}

	ret =BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_AAL2_CHANNEL_SET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
				"De-registering for AAL2_ChannelSet  Failed\n");
		return (FPPI_FAILURE);
	}


	ret =BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_AAL2_CHANNEL_DEL_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
				"De-registering for AAL2_ChannelDel  Failed\n");
		return (FPPI_FAILURE);
	}


	ret =BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_CONNECTION_XC_SET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
				"De-registering for ATM_ConnectionXcSet  Failed\n");
		return (FPPI_FAILURE);
	}


	ret =BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_CONNECTION_XC_DEL_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
				"De-registering for ATM_ConnectionXcDel  Failed\n");
		return (FPPI_FAILURE);
	}


	ret =BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_OAM_CP_SET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
				"De-registering for ATM_OAM_CP_Set  Failed\n");
		return (FPPI_FAILURE);
	}


	ret =BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_OAM_CC_SET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
				"De-registering for ATM_OAM_CC_Set  Failed\n");
		return (FPPI_FAILURE);
	}


	ret =BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_OAM_LB_SET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
				"De-registering for ATM_OAM_LB_Set  Failed\n");
		return (FPPI_FAILURE);
	}


	ret =BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_OAM_MONITOR_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
				"De-registering for ATM_OAM_Monitor  Failed\n");
		return (FPPI_FAILURE);
	}


	ret =BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_OAM_ALARM_SET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
				"De-registering for ATM_OAM_AlarmSet  Failed\n");
		return (FPPI_FAILURE);
	}


	ret =BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_OAM_ALARM_CLEAR_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
				"De-registering for ATM_OAM_AlarmClear  Failed\n");
		return (FPPI_FAILURE);
	}


	ret =BENDAPI_deregister_callback (fppiIfAtmCbHandle[NPF_IF_ATM_OAM_PM_SET_CB]);
	if(ret != FPPI_SUCCESS)
	{
		npf_logger_write(LOG_FP_ATM, LOG_ERR, 
				"De-registering for ATM_OAM_PM_Set  Failed\n");
		return (FPPI_FAILURE);
	}

	/* Call the FP blade SDK specifics for Shut down */
	fpmIfATM_shutdownFeLimePoint();
    return(FPPI_SUCCESS);
}

/*********************************************************************
*
*   Function: fpmIfATM_IfAttrSetFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Setup Attributes of ATM i/f 
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_IfAttrSetFunc (
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_IfAttrSetFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_ATTR_SET_CB])
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_IfAttrSet, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_IfAttrSet, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_IfAttrSetFuncLimePoint(cbcorrelator,context,response_data);

	
	return;
}

/*********************************************************************
*
*   Function: fpmIfATM_IfDeleteFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to delete an ATM i/f 
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_IfDeleteFunc(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_IfDeleteFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_IF_DELETE_CB])
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_IfDelete, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_IfDelete, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_IfDelFuncLimePoint(cbcorrelator,context,response_data);

	
	return;
}



/*********************************************************************
*
*   Function: fpmIfATM_IfEnableFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Enable an ATM i/f 
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_IfEnableFunc(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_IfEnableFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_IF_ENABLE_CB])
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_IfEnable, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_IfEnable, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_IfEnableFuncLimePoint(cbcorrelator,context,response_data);
	
	return;
}


/*********************************************************************
*
*   Function: fpmIfATM_IfDisableFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Disable an ATM i/f 
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_IfDisableFunc(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_IfDisableFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_IF_DISABLE_CB])
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_IfDisable, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_IfDisable, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_IfDisableFuncLimePoint(cbcorrelator,context,response_data);
	
	return;
}


/*********************************************************************
*
*   Function: fpmIfATM_IfGenStatsGetFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Get Statistics of an ATM i/f 
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_IfGenStatsGetFunc(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_IfStatistics_t userResponse;
	assert(fpmIfATM_IfGenStatsGetFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_IF_GEN_STATS_GET_CB])
	{
			/* Set the user Response structure to success */
			userResponse.response.status = NPF_INVALID_PARAMETERS;
			userResponse.response.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_IfGenStatsGet, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.response.status = NPF_INVALID_PARAMETERS;
			userResponse.response.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_IfGenStatsGet, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_IfGenStatsGetFuncLimePoint(cbcorrelator,context,response_data);
	return;
}


/*********************************************************************
*
*   Function: fpmIfATM_VccSetFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Setup a new VCC 
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_VccSetFunc (
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_VccSetFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_VCC_SET_CB])
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_VccSet, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_VccSet, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_VccSetFuncLimePoint(cbcorrelator,context,response_data);
	
	return;
}

/*********************************************************************
*
*   Function: fpmIfATM_VccBindFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to bind a VCC with parent Interface
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_VccBindFunc (
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_VccBindFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_VCC_BIND_CB])
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_VccBind, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_VccBind, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_VccBindFuncLimePoint(cbcorrelator,context,response_data);
	
	return;
}

/*********************************************************************
*
*   Function: fpmIfATM_VccDelFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Delete a VCC connection
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_VccDelFunc (
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_VccDelFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_VCC_DEL_CB])
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_VccDelete, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_VccDelete, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_VccDelFuncLimePoint(cbcorrelator,context,response_data);
	
	return;
}


/*********************************************************************
*
*   Function: fpmIfATM_VccEnableFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Enable a VCC connection
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_VccEnableFunc(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_VccEnableFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_VCC_ENABLE_CB])
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_VccEnable, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_VccEnable, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_VccEnableFuncLimePoint(cbcorrelator,context,response_data);
	
	return;
}


/*********************************************************************
*
*   Function: fpmIfATM_VccDisableFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Disable a VCC connection
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_VccDisableFunc(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_VccDisableFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_VCC_DISABLE_CB])
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_VccDisable, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_VccDisable, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_VccDisableFuncLimePoint(cbcorrelator,context,response_data);
	
	return;
}


/*********************************************************************
*
*   Function: fpmIfATM_VccOperStatusGetFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Get OperStaus of a VCC conn
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_VccOperStatusGetFunc(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_VccOperStatusGetFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_VCC_OPER_STATUS_GET_CB])
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_VccOperStatusGet, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_VccOperStatusGet, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_VccOperStatusGetFuncLimePoint(cbcorrelator,context,response_data);
	return;
}


/*********************************************************************
*
*   Function: fpmIfATM_VccStatsGetFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Get Statistics of a VCC conn
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_VccStatsGetFunc(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_VccStats_t userResponse;
	assert(fpmIfATM_VccStatsGetFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_VCC_STATS_GET_CB])
	{
			/* Set the user Response structure to success */
			userResponse.response.status = NPF_INVALID_PARAMETERS;
			userResponse.response.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_VccStatsGet, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.response.status = NPF_INVALID_PARAMETERS;
			userResponse.response.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_VccStatsGet, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_VccStatsGetFuncLimePoint(cbcorrelator,context,response_data);
	return;
}


/*********************************************************************
*
*   Function: fpmIfATM_AAL2_ChannelSetFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Set an AAL2 channel on a VCC conn
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_AAL2_ChannelSetFunc(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_AAL2_ChannelSetFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_AAL2_CHANNEL_SET_CB])
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_AAL2_ChannelSet, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_AAL2_ChannelSet, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_AAL2_ChannelSetFuncLimePoint(cbcorrelator,context,response_data);
	return;
}


/*********************************************************************
*
*   Function: fpmIfATM_AAL2_ChannelDelFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Set an AAL2 channel on a VCC conn
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_AAL2_ChannelDelFunc(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_AAL2_ChannelDelFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_AAL2_CHANNEL_DEL_CB])
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_AAL2_ChannelDel, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_AAL2_ChannelDel, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_AAL2_ChannelDelFuncLimePoint(cbcorrelator,context,response_data);
	return;
}


/*********************************************************************
*
*   Function: fpmIfATM_ConnectionXcSetFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Set an ATM Connection XC 
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_ConnectionXcSetFunc(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_ConnectionXcSetFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_CONNECTION_XC_SET_CB])
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_ConnectionXcSet, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_ConnectionXcSet, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_ConnectionXcSetFuncLimePoint(cbcorrelator,context,response_data);
	return;
}



/*********************************************************************
*
*   Function: fpmIfATM_ConnectionXcDelFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Delete an ATM Connection XC 
*			and passes connection params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_ConnectionXcDelFunc(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_ConnectionXcDelFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_CONNECTION_XC_DEL_CB])
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_ConnectionXcDel, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_ConnectionXcDel, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_ConnectionXcDelFuncLimePoint(cbcorrelator,context,response_data);
	return;
}



/*********************************************************************
*
*   Function: fpmIfATM_OAM_CP_SetFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Set connection point 
*			type for OAM services and passes OAM params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_OAM_CP_SetFunc(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_OAM_CP_SetFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_OAM_CP_SET_CB])
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_OAM_CP_Set, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_OAM_CP_Set, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_OAM_CP_SetFuncLimePoint(cbcorrelator,context,response_data);
	
   return;
}


/*********************************************************************
*
*   Function: fpmIfATM_OAM_CC_SetFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Set continuity Check  
*			mechanism for OAM services and passes OAM  params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_OAM_CC_SetFunc(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_OAM_CC_SetFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_OAM_CC_SET_CB])
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_OAM_CC_Set, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_OAM_CC_Set, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_OAM_CC_SetFuncLimePoint(cbcorrelator,context,response_data);
	
	return;
}



/*********************************************************************
*
*   Function: fpmIfATM_OAM_LB_SetFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Set Loopback  
*			mechanism for OAM services and passes OAM  params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_OAM_LB_SetFunc(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_OAM_LB_SetFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_OAM_LB_SET_CB])
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_OAM_LB_Set, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_OAM_LB_Set, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_OAM_LB_SetFuncLimePoint(cbcorrelator,context,response_data);
	
	return;
}


/*********************************************************************
*
*   Function: fpmIfATM_OAM_MonitorFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Set OAM Cell Monitoring  
*			mechanism for OAM services and passes OAM  params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_OAM_MonitorFunc(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_OAM_MonitorFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_OAM_MONITOR_CB])
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_OAM_Monitor, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_OAM_Monitor, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_OAM_MonitorFuncLimePoint(cbcorrelator,context,response_data);
	return;
}


/*********************************************************************
*
*   Function: fpmIfATM_OAM_AlarmSetFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Set OAM Alarms  
*			mechanism for OAM services and passes OAM  params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_OAM_AlarmSetFunc(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_OAM_AlarmSetFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_OAM_ALARM_SET_CB])
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_OAM_AlarmSet, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_OAM_AlarmSet, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_OAM_AlarmSetFuncLimePoint(cbcorrelator,context,response_data);
	return;
}


/*********************************************************************
*
*   Function: fpmIfATM_OAM_AlarmClearFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to Clear OAM Alarms  
*			mechanism for OAM services and passes OAM  params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_OAM_AlarmClearFunc(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_OAM_AlarmClearFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_OAM_ALARM_CLEAR_CB])
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_OAM_AlarmClear, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_OAM_AlarmClear, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_OAM_AlarmClearFuncLimePoint(cbcorrelator,context,response_data);
	return;
}


/*********************************************************************
*
*   Function: fpmIfATM_OAM_PM_SetFunc
*
*   Parameters:
*			cbcorrelator - CallBack Correlator
*			context - CallBack Context
*			response_data - Callback response data 
*
*   Description:
*			This callback function is registered with the Back-end API.
*			The Back-end API calls this function to set Performance monitoring 
*			mechanism for OAM services and passes OAM  params to the FE.
*
*
*   return : None 
*
*****************************************************************************/
void fpmIfATM_OAM_PM_SetFunc(
				FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data)
{
	IfATM_FpResponse_t userResponse;
	assert(fpmIfATM_OAM_PM_SetFunc);
   
	/* Check if Context is correct */ 
	if(context != fppiIfAtmContext[NPF_IF_ATM_OAM_PM_SET_CB])
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_OAM_PM_Set, 
							cbcorrelator, 
							&userResponse);
			return;
	}

	/* Check if response_data is not NULL */
	if(response_data == NULL)
	{
			/* Set the user Response structure to success */
			userResponse.status = NPF_INVALID_PARAMETERS;
			userResponse.reason = NPF_INVALID_PARAMETERS;						

			BENDAPI_Report_ATMStatus (FPPI_FAILURE, 
							FPPI_ATM_OAM_PM_Set, 
							cbcorrelator, 
							&userResponse);
			return;
	}
  
	/* Call the platform specific function */
	fpmIfATM_OAM_PM_SetFuncLimePoint(cbcorrelator,context,response_data);
	
	return;
}







/*****************************************************************************
*
*   Function:   fpmIfATM_SendEventFunc
*
*   Parameters: 
*                event_type: event type
*                event_data : data 
*   Description:
*                This function is called by the platform specific layer to send
*					  an ATM event to the Control Plane User.
*
*   Return: 0 on success, -1 on failure 
*****************************************************************************/
NPF_uint32_t fpmIfATM_SendEventFunc(
			NPF_CBCAT  event_type, 
			NPF_ATM_EventData_t *     event_data)
{
	npf_logger_write(LOG_FP_ATM, LOG_TRACE, 
		"Entering fpmIfATM_SendEventFunc");
	switch(event_type)
	{
		case FPPI_ATM_OAM_AisAlarm_Event:
		case FPPI_ATM_OAM_RdiAlarm_Event:
		case FPPI_ATM_OAM_FpmStats_Event:
		case FPPI_ATM_OAM_BrStats_Event:
		case FPPI_ATM_OAM_Lb_Event:
			if (BENDAPI_sendATMEvent(event_type, event_data) != FPPI_SUCCESS) 
			{        
				npf_logger_write(LOG_FP_ATM, LOG_ERR, 
						"Sending Report Failed");
			}
			break;
		default : 
			break;
	} /* end of switch */

	return 0;
}

