/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/

/* **************************************************************************
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ForwardingPlane/FPModule/BindingDiscovery/src/bd-translator.c,v $ 
 * $Revision: 1.6 $
 * $Date: 2003/11/17 16:57:21 $
 ****************************************************************************/

#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "pdk_macros.h"
#include "FPModule.h"
#include "bd_mgr.h"
#include "cm_mgr.h"
#include "log.h"
#include "fp_common.h"
#ifdef _INCLUDE_DIFFSERV
#include "qos_mgr.h"
#endif

extern int pdk_fpmgr_shutdown();
extern router_config g_current_router_config;

/* Macros for enabling traces */
//#ifdef FPM_BDMGR_DEBUG
#define BD_TRACE() \
npf_logger_write(LOG_FP_BD, LOG_TRACE, "%s\n", "entering");
//#ifdef BD_RETURN
//#undef BD_RETURN
//#endif //#if BD_RETURN
#define BD_RETURN( val ) \
    { \
        npf_logger_write(LOG_FP_BD, LOG_TRACE, "%s\n", "leaving"); \
        return val; \
    }
#define BD_PRINT( lvl, fmt, args... ) \
        npf_logger_write(LOG_FP_BD, lvl, fmt, ## args)

//#define BD_PRINT(lvl, fmt, args...) /* nothing */
//#endif //#ifdef DEBUG

/* Callback Handles - Callbacks registered with Back end API */
FPPI_CBHANDLE        FEbindCBHdl, FEunbindCBHdl, CEReadyCBHdl; 
FPPI_CBCORRELATOR    g_fe_cap_correlator = 1;

/* Contexts of callbacks registered with BEND API */
static FPPI_CONTEXT  g_FE_bind_context;
static FPPI_CONTEXT  g_FE_unbind_context;
static FPPI_CONTEXT  g_CE_ready_context;
static int           g_init = 0;
static FENAME        g_fe_name; /* FE name */
static char          g_host[20]; /* CE to connect to  */
int                  PortID[MAX_IXP_LINKS];
int                  g_running; /* global variable used to set the state 
				   of the FE. This variable when set to 0 
				   triggers the FE shutdown 
				*/
static FPPI_FE_Caps* g_fe_caps; /* global variable that contains the FE
				   capabilities. Initialized during startup,
				   used for reporting capabilities to CE,
				   destroyed in bd_mgr_bindResponse_cb()
				*/

static PilEvent g_fe_bind_pilevent;


/**************************************************************************
*
*   Function: fpm_FECapResponse_cb()
*
*   Parameters:
*        None 
*
*   Description: This function is called by the Back-end API. It signals to the
*                FE that the CE has received the FE Attributes and is ready. 
*                The FE then starts the EventHandler which is a poll thread 
*                to monitor Link Status
*
*   BD_RETURN : NONE 
*
***************************************************************************/

void fpm_FECapResponse_cb (FPPI_CBCORRELATOR cbcorrelator,            
			   FPPI_CONTEXT context,
			   void *response_data)
{
    BD_TRACE ();
    PIL_SetEvent(g_fe_bind_pilevent);
    BD_RETURN (/*void*/);
}        

FPPI_RET fpm_bd_mgr_start (char * fe_name, char* host)
{

    BD_TRACE ();
    strcpy (g_fe_name, fe_name);
    strcpy (g_host, host);

    /* invoke into CM Manager to get FE capabilities. We need the Blade
       ID to bind to the CE */
    
    fpm_cm_mgr_get_fecaps (g_fe_name, (uint32_t) g_fe_cap_correlator);

    PIL_WaitForEvent(g_fe_bind_pilevent,PIL_INFINITE_TIMEOUT);

    BD_PRINT (LOG_INFO, "Getting the FE Attributes for FE: %s \n", 
	      fe_name);
    BD_RETURN (FPPI_SUCCESS);
}    

/*****************************************************************************
*
*   Function: fpm_bd_mgr_shutdown()
*
*   Parameters:
*        None 
*
*   Description: This function shuts-down the FPModule. It deregisters for 
*                callbacks with the Back-end API, de-initializes the FP Engine
*                Cleans up the threads and sets the FP Manager shutdown flag.
*
*
*   BD_RETURN :  FPPI_SUCCESS if succesful
*            FPPI_FAILURE otherwise
*
*****************************************************************************/

FPPI_RET fpm_bd_mgr_shutdown()
{
    int errtype;     

    BD_TRACE ();
    errtype = BENDAPI_deregister_callback (FEbindCBHdl);    
    if (FPPI_SUCCESS != errtype)
    {
        BD_PRINT (LOG_INFO, "De-registering for  FE BIND Failed\n");
        BD_RETURN (FPPI_FAILURE);
    }    
    errtype = BENDAPI_deregister_callback (FEunbindCBHdl);
    if (FPPI_SUCCESS != errtype)
    {
        BD_PRINT (LOG_INFO, "De-registering for  FE UNBIND Failed\n");
        BD_RETURN (FPPI_FAILURE);
    }    
    errtype = BENDAPI_deregister_callback (CEReadyCBHdl);
    if (FPPI_SUCCESS != errtype)
    {
        BD_PRINT (LOG_INFO, "De-registering for  CE Ready Failed\n");
        BD_RETURN (FPPI_FAILURE);
    }    
 
    /* call into Event Manager */
    BD_RETURN (FPPI_SUCCESS); 
}


/*****************************************************************************
*
*   Function: fpm_unbindResponse_cb()
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data
*
*   Description: This function s called by the Back-End API. It sets 
*                the running state of the FE to 0. The CE signals the 
*                 FE to unbind.
*
*
*   BD_RETURN :  None 
*
*****************************************************************************/

void fpm_unbindResponse_cb (FPPI_CBCORRELATOR cbcorrelator,            
			    FPPI_CONTEXT context,
			    void *response_data)
{
    BD_TRACE ();
    if(context != g_FE_unbind_context)
        BD_RETURN (/*void*/);
#ifdef COLOCATED
	if ((pdk_fpmgr_shutdown()) == FALSE)
	    printf("Shutdown of FP Modules in FPManager failed \n");
#else
    g_running = 0;
#endif
    BD_RETURN (/*void*/);
}        

/*************************************************************************
*
*   Function: fpm_bindResponse_cb()
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data
*
*   Description: This function is called by the Backend API after the FE 
*                boots up and sends a bind request. The FE capabilities 
*                are queried and sent to the CE.
*
*
*   BD_RETURN : None 
*
*************************************************************************/

void fpm_bindResponse_cb (FPPI_CBCORRELATOR cbcorrelator,
			  FPPI_CONTEXT context,
			  void *response_data)
{
    BD_TRACE ();
    if(context != g_FE_bind_context)
        BD_RETURN (/*void*/);

	
    BD_PRINT (LOG_INFO, "***fename****:%s\n", g_fe_name);
    BENDAPI_capabilityRequest (g_fe_caps, CEReadyCBHdl);

    /* we dont need the FE capability anymore in B&D, so free the
       memory */
    PIL_FreeMemory ((void*) g_fe_caps->ports_caps.port_caps_list);
    PIL_FreeMemory ((void*) g_fe_caps);
    BD_RETURN (/*void*/);
}

/*************************************************************************
*
*   Function: bd_mgr_report_fecaps_cb()
*
*   Parameters: FPPI_FE_Caps - FE capabilities 
*               cbcorrelator - CallBack Correlator
*
*   Description: This function is called by Config & Mgmt Manager to 
*                report the FE capabilities it has queried from the
*                underlying platform
*
*   RETURN : None 
*
*************************************************************************/

void bd_mgr_report_fecaps_cb (uint32_t feid, FPPI_FE_Caps* fecaps, 
							  uint32_t correlator)
{
    int errtype = FPPI_SUCCESS;

    BD_TRACE ();
    if (! fecaps) {
	BD_PRINT (LOG_ERR, "NULL memory from CMM!");
	BD_RETURN (/*void*/);
    }
    /* copy this capability got from CMM into our global variable */
    g_fe_caps = (FPPI_FE_Caps*) PIL_AllocateMemory(sizeof (FPPI_FE_Caps),
						   PIL_MEM_ZERO_INIT);
    if (!g_fe_caps) {
	BD_PRINT (LOG_ERR, "Unable to allocate memory!\n Exiting!\n");
	exit (1);
    }
    strcpy (g_fe_caps->fe_name, g_fe_name);

    /* TODO: need to figure out actual values for this */
    switch (g_current_router_config) {
    case IPv4_ROUTER:
        g_fe_caps->caps_bitmap = 1;  /* bit 1 */
        break;
    case IPv6_ROUTER:
        g_fe_caps->caps_bitmap = 2;  /* bit 2 */
        break;
    case IPv4v6_ROUTER:
        g_fe_caps->caps_bitmap = 3;  /* bit 1&2 */
        break;
    case MPLS:
        g_fe_caps->caps_bitmap = 12; /* bit 3&4 */
        break;
    case DS_MPLS_LER:
        g_fe_caps->caps_bitmap = 20; /* bit 3&5 */
        break;
    case DS_MPLS_LSR:
        g_fe_caps->caps_bitmap = 24; /* bit 4&5 */
        break;
    default:
        assert(0);
        break;
    }
    g_fe_caps->ports_caps.port_count = fecaps->ports_caps.port_count;
    g_fe_caps->ports_caps.port_caps_list = (FPPI_Port_Caps*) 
	PIL_AllocateMemory (
	sizeof (FPPI_Port_Caps) * fecaps->ports_caps.port_count, 
	PIL_MEM_ZERO_INIT
	);
    if (!g_fe_caps->ports_caps.port_caps_list) {
	BD_PRINT (LOG_ERR, "Unable to allocate memory!\n Exiting!\n");
	PIL_FreeMemory ((void*) g_fe_caps);
	exit (1);
    }
    memcpy (g_fe_caps->ports_caps.port_caps_list, 
	    fecaps->ports_caps.port_caps_list,
	    sizeof (FPPI_Port_Caps) * g_fe_caps->ports_caps.port_count);

    /* now send a bind request to the CP */
    sprintf (g_fe_name, "fe%d", (int)feid);
    errtype = BENDAPI_bindRequest(g_host, g_fe_name, feid, FEbindCBHdl);
    BD_PRINT (LOG_INFO, "************Host address: %s FE Name: %s FE id = %d********** \n", g_host, 
	      g_fe_name, feid);

    if (errtype != FPPI_SUCCESS)
    {
        BD_PRINT (LOG_INFO, "FE Bind Request Failed \n");
        BD_RETURN (/*void*/);
    }
    BD_PRINT (LOG_INFO, "*********** SENT BIND REQUEST******** \n");
    BD_RETURN (/*void*/);
}


/****************************************************************************
*   Function: fpm_bd_mgr_init()
*
*   Parameters:  None 
*
*   Description: This function intializes BindingDiscovery. This registers
*                callback functions with the Back-end API for Control 
*                Operations and Events such as Bind, Unbind, etc.
*
*   Return:  FPPI_SUCCESS if succesful
*            FPPI_FAILURE otherwise
****************************************************************************/  

FPPI_RET fpm_bd_mgr_init()
{
    int errtype;
    BD_TRACE();

    /* Checking to see if the FPM is already initialized */
    if (g_init == 0)
        g_init = 1;
    else if (g_init == 1)
        return (FPPI_FAILURE);
    /* Assigning Contexts for registering for callbacks */

    g_fe_bind_pilevent = PIL_CreateEvent(FALSE,FALSE);

    g_FE_bind_context = (void *) FPPI_FEBind_Event;
    g_FE_unbind_context = (void *) FPPI_CEUnBind_Event;
    g_CE_ready_context = (void *) FPPI_CEReady_Event;
    
    errtype = BENDAPI_register_callback (&fpm_bindResponse_cb, 
					 &FEbindCBHdl, 
					 FPPI_FEBind_Event, 
					 g_FE_bind_context);
    if (NPF_SUCCESS != errtype)
    {
        BD_PRINT (LOG_INFO, "Registering for FE Bind Failed\n");
        BD_RETURN (FPPI_FAILURE);
    }
    errtype = BENDAPI_register_callback (&fpm_unbindResponse_cb, 
					 &FEunbindCBHdl, 
					 FPPI_CEUnBind_Event, 
					 g_FE_unbind_context);
    if (FPPI_SUCCESS != errtype)
    {
        BD_PRINT (LOG_INFO, "Registering for FE UnBind Failed\n");
        BD_RETURN (FPPI_FAILURE);
    }
    errtype = BENDAPI_register_callback (&fpm_FECapResponse_cb, 
					 &CEReadyCBHdl, 
					 FPPI_CEReady_Event, 
					 g_CE_ready_context);
    if (FPPI_SUCCESS != errtype)
    {
        BD_PRINT (LOG_INFO, "Registering for FE Capabilities Event  Failed\n");
        BD_RETURN (FPPI_FAILURE);
    }    

    BD_RETURN (FPPI_SUCCESS);
} /* end of pdk_fpm_init() */
