/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/

#ifndef _CM_MGR_SDK_H
#define _CM_MGR_SDK_H

#ifdef __cplusplus
extern "C" {
#endif

#define CM_CAPS		1
#define CM_PORTNUM	2
#define CM_STATS	3

/****************************************************************************
*																			*
*						TRANSLATOR CONTEXT STRUCUTURES						*
*																			*			
*****************************************************************************/
/*****************************************************************************
*   Name: cm_TranslatorContext_t
*
*   Description:
*		Stores the genaric translator's context 
*
*   Parameters:
*		cbCorrleator:	Higher level instance differentiator, opaque to the
*						FP module. Passed back to the FP Module's client 
*						(Backenc API) as is.
*       
*
*/

typedef struct
{	
	uint32_t cbCorrelator;
} cm_TranslatorContext_t;

/*****************************************************************************
*   Name: cm_portnum_TranslatorContext_t
*
*   Description:
*		Stores the translator's context related to getting num of ports.
*
*   Parameters:
*		get_fecaps:		boolean, 1 indicatdes fecaps is to be retrieved
*		fename:			pointer to the name of the FE
*		port_num:		Pointer to memory allocated for holding port number
*		cbCorrleator:	Higher level instance differentiator, opaque to the
*						FP module. Passed back to the FP Module's client 
*						(Backenc API) as is.
*       
*
*/

typedef struct
{
	char get_fecaps;
	char *fename;
	uint32_t *port_num;
	uint32_t cbCorrelator;
} cm_portnum_TranslatorContext_t;

/*****************************************************************************
*   Name: cm_fecaps_TranslatorContext_t
*
*   Description:
*		Stores the translator's context related to getting the FE caps. 
*
*   Parameters:
*		fecaps:	Pointer to memory allocated for FE caps.
*		cbCorrleator:	Higher level instance differentiator, opaque to the
*						FP module. Passed back to the FP Module's client 
*						(Backenc API) as is.
*       
*
*/

typedef struct
{
	FPPI_FE_Caps *fecaps;
	uint32_t cbCorrelator;
} cm_fecaps_TranslatorContext_t;

/*****************************************************************************
*   Name: cm_L3_stats_TranslatorContext_t
*
*   Description:
*		Stores the translator's context related to getting the FE caps. 
*
*   Parameters:
*		stats_list:		Pointer to memory allocated for statistics.
*		num_entries:	number of ports
*		cbCorrleator:	Higher level instance differentiator, opaque to the
*						FP module. Passed back to the FP Module's client 
*						(Backend API) as is.
*       
*
*/
typedef struct {
	npf_L3StatsEntry_t *stats_list;
	uint32_t num_entries;
	uint32_t cbCorrelator;
} cm_L3_stats_TranslatorContext_t;


/****************************************************************************
*																			*
*				PLATFORM SPECIFIC USER CONTEXT STRUCUTURES					*	
*																			*			
*****************************************************************************/

/*****************************************************************************
*   Name: cm_portnum_UserContext_t
*
*   Description:
*		Stores the user context related to getting the port number. 
*
*   Parameters:
*				t_c: Translator context.
*		
*       
*
*/

typedef struct
{
	cm_portnum_TranslatorContext_t *t_c;
} cm_portnum_UserContext_t;

/*****************************************************************************
*   Name: cm_fecaps_UserContext_t
*
*   Description:
*		Stores the user context related to getting the FE caps. 
*
*   Parameters:
*				t_c: Translator context.
*		
*       
*
*/

typedef struct
{
	cm_fecaps_TranslatorContext_t *t_c;
} cm_fecaps_UserContext_t;

/*****************************************************************************
*   Name: cm_stats_UserContext_t
*
*   Description:
*		Stores the user context related to getting the FE statistics. 
*
*   Parameters:
*				reqComp_Indctr: Pointer to counter to keep track of translator 
*								request completion
*				t_c: Translator context.
*				port_index: Port index required by the IXASDK3.0
*				portID: ID of the port
*		
*       
*
*/

typedef struct {
	uint32_t *reqComp_Indctr;
	cm_L3_stats_TranslatorContext_t *t_c;
	uint32_t port_index;
	uint32_t portId;	
} cm_L3_stats_UserContext_t;

/*****************************************************************************
*   Name: cm_uc_info_t
*
*   Description:
*		Stores the pending user context info 
*
*   Parameters:
*				type: Type of u_c
*				timestamp: Indicates the time of the down-call
*				u_c: Pointer to the user context
*		
*       
*
*/

typedef struct {
	uint32_t type;
	uint32_t timestamp;
	void * u_c;
} cm_uc_info;

int	cm_InitFPEngine();
int	cm_DestroyFPEngine();
uint32_t cm_is_Valid(void *u_c);
void cm_free_resources();

/********************************************************************
*					TRANSLATOR UP - CALLS							*
*********************************************************************/

int32_t fpm_cm_mgr_get_port_num_upcall(int32_t result, 
									cm_portnum_TranslatorContext_t *t_c);
int32_t fpm_cm_mgr_get_fecaps_upcall(int32_t result, 
									  cm_fecaps_TranslatorContext_t *t_c);
int32_t fpm_cm_mgr_set_L2_attrib_upcall(int32_t result, 
										 cm_TranslatorContext_t* t_c);
int32_t fpm_cm_mgr_set_L3_attrib_upcall(int32_t result, 
										 cm_TranslatorContext_t* t_c);
int32_t fpm_cm_mgr_delete_L3_attrib_upcall(int32_t result, 
										 cm_TranslatorContext_t* t_c);
int32_t fpm_cm_mgr_get_L3_stats_upcall(int32_t result,
									cm_L3_stats_TranslatorContext_t *t_c);

/********************************************************************
*					PLATFORM SPECIFIC APIS							*
********************************************************************/
void cm_get_port_num(cm_portnum_TranslatorContext_t *t_c);

void cm_get_fecaps(uint32_t numports, 
				   cm_fecaps_TranslatorContext_t *t_c);

void cm_set_L2_attrib(npf_L2IntfAttrsList_t* l2attr, 
							  cm_TranslatorContext_t* t_c);

void cm_set_L3_attrib(npf_L3IntfAttrsList_t* l3attr, 
							  cm_TranslatorContext_t* t_c);
void cm_delete_L3_attrib(npf_L3IntfAttrsList_t* l3attr, 
							  cm_TranslatorContext_t* t_c);
void cm_get_generic_stats(FPPI_Interface_GetStats_t *portlist, 
				   cm_L3_stats_TranslatorContext_t *t_c);

#ifdef __cplusplus
}
#endif

#endif /*_CM_MGR_SDK_H */

