/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2002 Intel Corporation. All Rights Reserved.
*****************************************************************************/

/**************************************************************************
*$Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ForwardingPlane/FPModule/ConfigManager/src/cm_mgr-ixasdk30.c,v $
*$Revision: 1.10 $
*$Date: 2003/12/01 10:35:25 $
*****************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "FPModule.h"
#include <cm_mgr_impl.h>
#include "pil.h"
#include <log.h>

#include "ix_cc.h"
#include "ix_cc_error.h"
#include "ix_types.h"
#include "cc/ix_cc_stkdrv_common.h"
#include "cc/ix_cc_stkdrv_pktclass.h"
#include "cc/ix_cc_stkdrv.h"
#include "cc/ix_cc_ipv4.h"
#include "cc/ix_cc_rtmv4.h"
#ifdef _INCLUDE_POS 
#include "cc/ix_cc_pos_rx.h"
#include "cc/ix_cc_atm_pos_tx.h"
#endif 
#include "cc/ix_cc_fpm_core.h"
#include "cc/ix_cc_eth_rx.h"
#ifdef _INCLUDE_MPLS
#include "cc/ix_cc_mpls.h"
#endif /* _INCLUDE_MPLS */


extern int	g_max_num_ports;
uint32_t	g_mybladeID;
static	DList	g_cm_pendinglist;
static uint32_t g_nexthop_id = MAX_NHID;

/* Note (PR6): Next Hop Ids for Local delivery packets have to ideally be 
   something that is agreed upon between the SDK and PDK, since we dont
   want the NH Ids to clash. 
   For local delivery packets, the FPModule needs to add an exact match
   route into IPv4 CC and use some range of NH IDs. This range of NHIds
   should not clash with the NHIds that are generated in the control plane.
   A similar understanding is required for NHIds that are to be added for
   adding subnet routes also (support for directly connected hosts). However
   Directly connected hosts will not be supported in PR6. This will have
   to be done for PR7, I guess.
 */

static int32_t cm_set_default_routes(uint32_t addr, uint32_t mask, 
				     uint32_t feid, uint32_t portid);

ix_error cm_dummy_cb(ix_error arg_Result, void *arg_pContext)
{
    npf_logger_write(LOG_FP_CM, LOG_INFO, "cm_dummy_cb called, error = %d\n", 
		     arg_Result);
    switch (arg_Result)
      {
      case IX_SUCCESS:
	  npf_logger_write (LOG_FP_CM, LOG_INFO, "NPF_NO_ERROR");
	  break;
      case IX_CC_IPV4_ERROR_DUPLICATE_NEXT_HOP:
	  npf_logger_write (LOG_FP_CM, LOG_INFO, "DUPLICATE NEXT HOP");
	  break;
      case IX_CC_IPV4_ERROR_NEXT_HOP_ID_IN_USE:
	  npf_logger_write (LOG_FP_CM, LOG_INFO, "NEXT HOP ID IN USE");
	  break;
      case IX_CC_IPV4_ERROR_INVALID_NH_INFO:
	  npf_logger_write (LOG_FP_CM, LOG_INFO, "INVALID NH INFO");
	  break;
      case IX_CC_IPV4_ERROR_INVALID_MASK:
	  npf_logger_write (LOG_FP_CM, LOG_INFO, "INVALID MASK");
	  break;
      case IX_CC_IPV4_ERROR_INVALID_CONTEXT:
	  npf_logger_write (LOG_FP_CM, LOG_INFO, "INVALID CONTEXT");
	  break;
      case IX_CC_IPV4_ERROR_INVALID_FLAGS:
	  npf_logger_write (LOG_FP_CM, LOG_INFO, "INVALID FLAGS");
	  break;
      case IX_CC_IPV4_ERROR_CALLBACK:
	  npf_logger_write (LOG_FP_CM, LOG_INFO, "ERROR CALLBACK");
	  break;
      default:
	  npf_logger_write (LOG_FP_CM, LOG_INFO, "Unknown error code");
	  break;
      }
    return IX_SUCCESS;
}

uint32_t get_nhid ()
{
  g_nexthop_id--;
  if (g_nexthop_id < MAX_RESERVED_NHID) {
    npf_logger_write (LOG_FP_CM, LOG_ERR, "Out of Next Hop Ids!\n");
    return -1;
  }
  return g_nexthop_id;
}

/*****************************************************************************
*
*			Callback prototypes
*
*****************************************************************************/
ix_error cm_get_port_num_cb(ix_error arg_Result, void *arg_pContext, 
			    ix_uint32* arg_pNumPorts);
ix_error cm_get_port_fecaps_cb(ix_error arg_Result, void *arg_pContext,
			       ix_cc_properties* arg_pProperty);
ix_error cm_get_generic_tx_stats_cb(ix_error arg_Result, void *arg_pContext,
				    ix_uint64 *arg_pBuffer);
ix_error cm_get_generic_rx_stats_cb(ix_error arg_Result, void *arg_pContext,
				    ix_uint64 *arg_pBuffer);

/*****************************************************************************
*
*   Function:   cm_InitFPEngine()
*
*   Parameters: None 
*
*   Description: This function starts up the platform specific component of 
*				 the CMM.
*
*   Return: None 
*****************************************************************************/
int cm_InitFPEngine()
{
    npf_logger_write(LOG_FP_CM, LOG_TRACE, "Entering cm_InitFPEngine");
    
	/*initialize a linked list to hold pendinh u_c info*/
    npf_list_init(&g_cm_pendinglist, PIL_FreeMemory);
    
    npf_logger_write(LOG_FP_CM, LOG_TRACE, "Leaving cm_InitFPEngine");
    return 0;
}

/*****************************************************************************
*
*   Function:   cm_DestroyFPEngine()
*
*   Parameters: None 
*
*   Description: This function destroys the platform specific component of 
*				 the CMM.
*
*   Return: None 
*****************************************************************************/
int cm_DestroyFPEngine()
{

    npf_logger_write(LOG_FP_CM, LOG_TRACE, "Entering cm_DestroyFPEngine");
    
    /*free resources for any pending u_c*/
    cm_free_resources();
    
    /*Destroy List*/
    npf_list_destroy(&g_cm_pendinglist);
    
    npf_logger_write(LOG_FP_CM, LOG_TRACE, "Leaving cm_DestroyFPEngine");
    return 0;
}

/*****************************************************************************
*
*   Function:   cm_get_port_num()
*
*   Parameters: 
*			t_c: Translator context 
*
*
*   Description:
*		This function queries the IXASDK for number of ports on an FE.
*		It provides a callback function to the IXASDK to call back with 
*		the number of ports on this FE.
*
*   Return: none
*****************************************************************************/
void cm_get_port_num(cm_portnum_TranslatorContext_t *t_c)
{
    int32_t iRet = 0;
    cm_uc_info *uc_info = NULL;
    cm_portnum_UserContext_t *u_c = NULL;
    ix_error err;
    
    npf_logger_write(LOG_FP_CM, LOG_TRACE, "Entering cm_get_port_num");
    
    /*build the platform specific user context*/
    u_c = (cm_portnum_UserContext_t *)
	PIL_AllocateMemory (sizeof(cm_portnum_UserContext_t), PIL_MEM_ZERO_INIT);
    if(!u_c)
	{
	    npf_logger_write(LOG_FP_CM, LOG_ERR, "Mem allocation failed for u_c");
	    iRet = -1;
	    goto finish;
	}
    npf_logger_write(LOG_FP_CM, LOG_INFO, "Allocated u_c:0x%x", u_c);
    u_c->t_c = t_c;
    
    uc_info = (cm_uc_info *)
	PIL_AllocateMemory (sizeof(cm_uc_info), PIL_MEM_ZERO_INIT);
    if(!uc_info)
	{
	    npf_logger_write(LOG_FP_CM, LOG_ERR, 
			     "Mem allocation failed for uc_info");
	    iRet = -1;
	    goto finish;
	}
    uc_info->type = CM_PORTNUM;
    uc_info->u_c = (void *)u_c;
    /*insert this u_c info into the u_c pending list*/
    npf_list_pushBack(&g_cm_pendinglist, uc_info);
    
    /*Call into the IXASDK*/
    err = ix_cc_stkdrv_async_get_num_ports(cm_get_port_num_cb, (void *)u_c);
    if(err != IX_SUCCESS)
	{
	    npf_logger_write(LOG_FP_CM, LOG_ERR,
			"IXASDK30 API call failed error code:%d", IX_ERROR_GET_CODE(err));
		ix_error_dump(stdout, err);
		iRet = -1;
		goto finish;
	}

finish:
	if(iRet == -1)
	{
	    if(cm_is_Valid(u_c))
		PIL_FreeMemory(u_c);
	    /*inform the Translator component about failure via the up-call*/
	    fpm_cm_mgr_get_port_num_upcall(iRet, t_c);
	}
	npf_logger_write(LOG_FP_CM, LOG_TRACE, "Leaving cm_get_port_num");
}

/*****************************************************************************
*
*   Function:   cm_get_fecaps()
*
*   Parameters: 
*			numports:	 Number of ports
*			portnum_t_c: Translator context 
*			
*
*   Description: 
*				This function queries the IXASDK for capabilities of the FE.
*				It provides a callback function to the IXASDK to call back with 
*				the FE capabilities information.
*
*   Return: none
*****************************************************************************/
void cm_get_fecaps(uint32_t numports, 
		   cm_fecaps_TranslatorContext_t *t_c)
{
    int32_t iRet = 0;
    cm_fecaps_UserContext_t *u_c = NULL;
    cm_uc_info *uc_info = NULL;
    ix_uint32 propId;
    ix_error err;
	
    npf_logger_write(LOG_FP_CM, LOG_TRACE, "Entering cm_get_fecaps");
    
    /*build the platform specific user context.*/
    u_c = (cm_fecaps_UserContext_t *)
	PIL_AllocateMemory (sizeof(cm_fecaps_UserContext_t), PIL_MEM_ZERO_INIT);
    if(!u_c)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, "Mem allocation for u_c failed");
		iRet = -1;
		goto finish;
	}	
	npf_logger_write(LOG_FP_CM, LOG_INFO, "Allocated u_c:0x%x", u_c);	
	u_c->t_c = t_c;
	
	uc_info = (cm_uc_info *)PIL_AllocateMemory(
		sizeof(cm_uc_info), PIL_MEM_ZERO_INIT);
	if(!uc_info)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, "Mem allocation for uc_info failed");
		iRet = -1;
		goto finish;
	}
	uc_info->type = CM_CAPS;
	uc_info->u_c = (void *)u_c;
	/*insert this u_c info into the u_c pending list*/
	npf_list_pushBack(&g_cm_pendinglist, uc_info);

	/*set the property ID to get all the super set of properties*/		
	propId = (IX_CC_GET_PROPERTY_MTU | IX_CC_GET_PROPERTY_MAC_ADDR 
		| IX_CC_GET_PROPERTY_LINK_SPEED | IX_CC_GET_PROPERTY_MEDIA_TYPE);
	
	/*Call into the IXASDK*/
	err = ix_cc_stkdrv_async_get_property(cm_get_port_fecaps_cb,
					      (void *)u_c, 
					      propId, IX_CC_GET_ALL_PORTS
					      );
	if(err != IX_SUCCESS)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR,
			"IXASDK30 API call failed error code:%d", IX_ERROR_GET_CODE(err));
		ix_error_dump(stdout, err);
		iRet = -1;
		goto finish;
	}

finish:
	if(iRet == -1)
	{
		if(cm_is_Valid(u_c))
		    PIL_FreeMemory(u_c);
		/*inform the Translator component about failure via the up-call*/
		fpm_cm_mgr_get_fecaps_upcall(iRet, t_c);
	}
	npf_logger_write(LOG_FP_CM, LOG_TRACE, "Leaving cm_get_fecaps");
}


/*****************************************************************************
*
*   Function:   cm_set_L2_attrib()
*
*   Parameters: 
*			l2attr:	Pointer to NPF L2 attributes strucure
*			t_c		Translator context 			
*
*   Description: 
*				This function queries the IXASDK for L2 attributes of ports.
*
*   Return: none
*****************************************************************************/
void cm_set_L2_attrib(npf_L2IntfAttrsList_t* l2attr, 
		      cm_TranslatorContext_t* t_c)
{
    uint32_t              i = 0;
    int32_t               iRet = 0;
    ix_cc_properties      property;
    ix_uint32             propId;
    ix_error              err;
    NPF_IfLAN_Speed_t     link_speed;
#ifdef _INCLUDE_MPLS
    ix_cc_mpls_port_param mplsPortInfo;
#endif
    
    npf_logger_write(LOG_FP_CM, LOG_TRACE, "Entering  cm_set_L2_attrib");
    
    for (i = 0; i < l2attr->port_count; i++)
    {     
        memset(&property, 0, sizeof(ix_cc_properties));
	    
        property.blade_id = l2attr->port_list[i].feid;
	    
        npf_logger_write(LOG_FP_CM, LOG_INFO, 
			     "Attribs for portID=%d,"
			     "l2attr->port_list[i].type = %d\n", 
			     l2attr->port_list[i].portid, 
			     l2attr->port_list[i].type);
	    
	property.port_id = l2attr->port_list[i].portid;

        switch(l2attr->port_list[i].type) {
	case NPF_L2ATTRS_ALL:
	    {
		npf_logger_write(LOG_FP_CM, LOG_INFO, "The new MTU is %d\n", 
				 l2attr->port_list[i].mtu);
		npf_logger_write(LOG_FP_CM, LOG_INFO, "The new speed is %d\n",
				 l2attr->port_list[i].link_speed);
		npf_logger_write(LOG_FP_CM, LOG_INFO, "The new status is %d\n",
				 l2attr->port_list[i].status);				
		/*set the property ID to get all the super set of properties*/
		propId = (
			  IX_CC_SET_PROPERTY_MTU |
			  IX_CC_SET_PROPERTY_PHYSICAL_IF_STATUS |
			  IX_CC_SET_PROPERTY_LINK_SPEED
			  );
		link_speed = l2attr->port_list[i].link_speed;
		/* the Link speed coming from the control plane is an enum value.
		   We need to convert it before calling the SDK
		*/
		switch (link_speed) 
		  {
		  case NPF_IF_LAN_SPEED_10M: /*= 1 - 10 megabits/second */
		    property.link_speed = 10;
		    npf_logger_write(LOG_FP_CM, LOG_INFO, "Setting speed to 10Mbps");
		    break;
		  case NPF_IF_LAN_SPEED_100M: /* = 2 100 megabits/second */
		    property.link_speed = 100;
		    npf_logger_write(LOG_FP_CM, LOG_INFO, "Setting speed to 100Mbps");
		    break;
		  case NPF_IF_LAN_SPEED_1G: /* = 3 1 gigabit/second */
		    property.link_speed = 1000;
		    npf_logger_write(LOG_FP_CM, LOG_INFO, "Setting speed to 1Gbps");
		    break;
		  case NPF_IF_LAN_SPEED_10G: /* = 4 10 gigabit/second */
		    property.link_speed = 100000;
		    npf_logger_write(LOG_FP_CM, LOG_INFO, "Setting speed to 10Gbps");
		    break;
		  case NPF_IF_LAN_SPEED_AUTO: /* = 5 Set autosense */
		    break;
		} /* end switch(linkspeed) */

		property.mtu = l2attr->port_list[i].mtu;
		property.physical_if_state = l2attr->port_list[i].status;			
		
		/*call into IXASDK API*/
		err = ix_cc_async_set_property(propId, &property);

		if(err != IX_SUCCESS)
		    {
			npf_logger_write(LOG_FP_CM, LOG_ERR, 
					 "IXASDK30 API call failed for portId:%d"
					 "error code:%d", 
					 l2attr->port_list[i].portid, err);
			ix_error_dump(stdout, err);
			iRet = -1;
		    }
		break;
	    }
	case NPF_L2ATTRS_MTU:
	    {
#ifdef _INCLUDE_MPLS
          if (l2attr->port_list[i].port_type == NPF_PORT_MPLS)
          {
	 	       npf_logger_write(LOG_FP_CM, LOG_INFO, "Add MPLS Port\n");
                        
             memset(&mplsPortInfo, 0, sizeof(ix_cc_mpls_port_param));

             /* Make global Port from blade & local port */

             mplsPortInfo.globalPortIndex = l2attr->port_list[i].feid;
             mplsPortInfo.globalPortIndex = 
                                       mplsPortInfo.globalPortIndex << 16;
             mplsPortInfo.globalPortIndex = 
                 (mplsPortInfo.globalPortIndex) | (l2attr->port_list[i].portid);
             mplsPortInfo.flags = MPLS_PORT_FLAG_ENABLED;
             mplsPortInfo.maxLabPktSize = l2attr->port_list[i].mtu;

		       /*call into IXASDK API*/
	  	       err = ix_cc_mpls_async_add_port(&mplsPortInfo, 
                                   cm_dummy_cb, (void *)t_c);
          }
          else
#endif /* MPLS */
          {
	 	       npf_logger_write(LOG_FP_CM, LOG_INFO, "The new MTU is %d\n", 
				 l2attr->port_list[i].mtu);				
		       /*set the property ID to get all the super set of properties*/
  	  	       propId = (IX_CC_SET_PROPERTY_MTU);
     		    property.mtu = l2attr->port_list[i].mtu;
		
		       /*call into IXASDK API*/
		       err = ix_cc_async_set_property(propId, &property);
           }
		
	        if(err != IX_SUCCESS)
	        {
	   	     npf_logger_write(LOG_FP_CM, LOG_ERR, 
		           "IXASDK30 API call failed for portId:%d" "error code:%d", 
					     l2attr->port_list[i].portid, 
					     IX_ERROR_GET_CODE(err));
		        ix_error_dump(stdout, err);
		        iRet = -1;
	        }
           break;
	    }

	case  NPF_L2ATTRS_LINKSPEED:
	    {
		npf_logger_write(LOG_FP_CM, LOG_INFO, "The new speed is %d\n",
				 l2attr->port_list[i].link_speed);				
		/*set the property ID to get all the super set of properties*/
		propId = (IX_CC_SET_PROPERTY_LINK_SPEED);
		
		link_speed = l2attr->port_list[i].link_speed;
		/* the Link speed coming from the control plane is an enum value.
		   We need to convert it before calling the SDK
		*/
		switch (link_speed) 
		  {
		  case NPF_IF_LAN_SPEED_10M: /*= 1 - 10 megabits/second */
		    property.link_speed = 10;
		    npf_logger_write(LOG_FP_CM, LOG_INFO, "Setting speed to 10Mbps");
		    break;
		  case NPF_IF_LAN_SPEED_100M: /* = 2 100 megabits/second */
		    property.link_speed = 100;
		    npf_logger_write(LOG_FP_CM, LOG_INFO, "Setting speed to 100Mbps");
		    break;
		  case NPF_IF_LAN_SPEED_1G: /* = 3 1 gigabit/second */
		    property.link_speed = 1000;
		    npf_logger_write(LOG_FP_CM, LOG_INFO, "Setting speed to 1Gbps");
		    break;
		  case NPF_IF_LAN_SPEED_10G: /* = 4 10 gigabit/second */
		    property.link_speed = 100000;
		    npf_logger_write(LOG_FP_CM, LOG_INFO, "Setting speed to 1Gbps");
		    break;
		  case NPF_IF_LAN_SPEED_AUTO: /* = 5 Set autosense */
		    npf_logger_write(LOG_FP_CM, LOG_INFO, "Setting speed to AutoSense");
		    break;
		} /* end switch(linkspeed) */
		
		/*call into IXASDK API*/
		err = ix_cc_async_set_property(propId, &property);
		
		if(err != IX_SUCCESS)
		    {
			npf_logger_write(LOG_FP_CM, LOG_ERR, 
					 "IXASDK30 API call failed for portId:%d"
					 " error code:%d", 
					 l2attr->port_list[i].portid, 
					 IX_ERROR_GET_CODE(err));
			ix_error_dump(stdout, err);
			iRet = -1;
		    }
		break;
	    }
	case  NPF_L2ATTRS_STATUS:
	    {
		npf_logger_write(LOG_FP_CM, LOG_INFO, "The new status is %d\n",
				 l2attr->port_list[i].status);				
		/*set the property ID to get all the super set of properties*/
		propId = (IX_CC_SET_PROPERTY_PHYSICAL_IF_STATUS);
		property.physical_if_state = l2attr->port_list[i].status;			
		
		/*call into IXASDK API*/
		err = ix_cc_async_set_property(propId, &property) ;
		
		if(err != IX_SUCCESS)
		    {
			npf_logger_write(LOG_FP_CM, LOG_ERR, 
					 "IXASDK30 API call failed for portId:%d"
					 "error code:%d", 
					 l2attr->port_list[i].portid, 
					 IX_ERROR_GET_CODE(err));
			ix_error_dump(stdout, err);
			iRet = -1;
		    }
		/*TBD. if status == 1; SetPortFlagsforMulticast(num);*/
		break;
	    }
	default:
	    npf_logger_write(LOG_FP_CM, LOG_INFO, 
			     "unknown action type \n");
        } /* end (switch)*/
	if(iRet == -1)
	    {
		npf_logger_write(LOG_FP_CM, LOG_ERR, 
				 "Error while setting property");
		break;
	    }
	} /* end (for loop) */	
    fpm_cm_mgr_set_L2_attrib_upcall(iRet, t_c);
    npf_logger_write(LOG_FP_CM, LOG_TRACE, "Leaving cm_set_L2_attrib");
}

/*****************************************************************************
*
*   Function:cm_set_L3_attrib()
*
*   Parameters: 
*			l3attr:	Pointer to NPF L3 attributes strucure
*			t_c		Translator context
*
*   Description: 
*				This function queries the IXASDK for L3 attributes of ports.
*
*   Return: none
*
*
*****************************************************************************/
void cm_set_L3_attrib(npf_L3IntfAttrsList_t* l3attr, 
		      cm_TranslatorContext_t* t_c)
{    
    uint32_t i = 0;
    int32_t iRet = 0;
    ix_uint32 propId;
    ix_cc_properties property;
    ix_error err;
    
    npf_logger_write(LOG_FP_CM, LOG_TRACE, "Entering  cm_set_L3_attrib");
    
    for (i = 0; i < l3attr->port_count; i++) {
        memset(&property, 0, sizeof(ix_cc_properties));
	property.blade_id = l3attr->port_list[i].feid;
	
	npf_logger_write(LOG_FP_CM, LOG_INFO, 
			 "Attribs for portID=%d,"
			 "l3attr->port_list[i].type = %d\n", 
			 l3attr->port_list[i].portid, 
			 l3attr->port_list[i].type);
	
	property.port_id = l3attr->port_list[i].portid;

		
   switch (l3attr->port_list[i].type) {
	case NPF_L3ATTRS_ALL:
	    {
		npf_logger_write(LOG_FP_CM, LOG_INFO, "The IPv4 addr is 0x%x\n", 
				 l3attr->port_list[i].addr_u.addr);
		npf_logger_write(LOG_FP_CM, LOG_INFO, "The mask is 0x%x\n",
				 l3attr->port_list[i].mask_u.mask);
		npf_logger_write(LOG_FP_CM, LOG_INFO, "The ipfwding is %d\n",
				 l3attr->port_list[i].ipfwding);


		/*set the property ID */
		propId = IX_CC_SET_PROPERTY_ADD_INTERFACE_IPV4 |
		    IX_CC_SET_PROPERTY_PHYSICAL_IF_STATUS;
		
		property.ip_prop[0].protocol.ipv4_prop.ipv4_address = 
		    l3attr->port_list[i].addr_u.addr;
		property.ip_prop[0].protocol.ipv4_prop.ipv4_subnet_mask = 
		    l3attr->port_list[i].mask_u.mask;
		property.physical_if_state = 1;
                property.no_virtual_if = 1;
		/*call into IXASDK API*/
		err = ix_cc_async_set_property(propId, &property);
		if(err != IX_SUCCESS)
		    {
			npf_logger_write(LOG_FP_CM, LOG_ERR, 
					 "IXASDK30 API call failed for portId:%d"
					 "error code:%d", 
					 l3attr->port_list[i].portid, 
					 IX_ERROR_GET_CODE(err));
			ix_error_dump(stdout, err);
			iRet = -1;
		    }
		else
		    {
			/*Set the default routes*/
			if(cm_set_default_routes(l3attr->port_list[i].addr_u.addr, 
						 l3attr->port_list[i].mask_u.mask, 
						 l3attr->port_list[i].feid, 
						 l3attr->port_list[i].portid) == -1)
			    {
				npf_logger_write(LOG_FP_CM, LOG_ERR, 
						 "IXASDK30 API call failed to"
						 "add default routes");
				iRet = -1;
			    }
		    }
		break;
	    }
	case NPF_L3ATTRS_IPADDR:
	    {
		npf_logger_write(LOG_FP_CM, LOG_INFO, "The IPv4 addr is 0x%x\n", 
				 l3attr->port_list[i].addr_u.addr);
		npf_logger_write(LOG_FP_CM, LOG_INFO, "The mask is 0x%x\n",
				 l3attr->port_list[i].mask_u.mask);
		/*set the property ID*/
		propId = IX_CC_SET_PROPERTY_ADD_INTERFACE_IPV4 |
		    IX_CC_SET_PROPERTY_PHYSICAL_IF_STATUS;
		
		property.ip_prop[0].protocol.ipv4_prop.ipv4_address = 
		    l3attr->port_list[i].addr_u.addr;
		property.ip_prop[0].protocol.ipv4_prop.ipv4_subnet_mask = 
		    l3attr->port_list[i].mask_u.mask;
		property.physical_if_state = 1;
                property.no_virtual_if = 1;

		/*call into IXASDK API*/
		err = ix_cc_async_set_property(propId, &property);
		
		if(err != IX_SUCCESS)
		    {
			npf_logger_write(LOG_FP_CM, LOG_ERR, 
					 "IXASDK30 API call failed for portId:%d"
					 "error code:%d", 
					 l3attr->port_list[i].portid, 
					 IX_ERROR_GET_CODE(err));
			ix_error_dump(stdout, err);
			iRet = -1;
		    }
		else
		    {
			/*Set the default routes*/
			if(cm_set_default_routes(l3attr->port_list[i].addr_u.addr, 
						 l3attr->port_list[i].mask_u.mask, 
						 l3attr->port_list[i].feid, 
						 l3attr->port_list[i].portid) == -1)
					{
					    npf_logger_write(LOG_FP_CM, LOG_ERR, 
							     "IXASDK30 API call"
							     " failed to add"
							     "default routes");
					    iRet = -1;
					}
		    }
		break;
	    }
	case NPF_L3ATTRS_IPFWD:
	    {
		npf_logger_write(LOG_FP_CM, LOG_INFO, 
				 "IP FWDING OPTION NOT SUPPORTED ***\n");
		npf_logger_write(LOG_FP_CM, LOG_INFO, 
				 "IPFWDNG Value is %d", 
				 l3attr->port_list[i].ipfwding);
		break;
	    }
	default:
	    npf_logger_write(LOG_FP_CM, LOG_ERR, 
			     "Unknown TYPE OF L3 ATTR \n");
        }
    }	
    fpm_cm_mgr_set_L3_attrib_upcall(iRet, t_c);
    npf_logger_write(LOG_FP_CM, LOG_TRACE, "Leaving cm_set_L3_attrib");
}

/*****************************************************************************
*
*   Function:cm_delete_L3_attrib()
*
*   Parameters: 
*			l3attr:	Pointer to NPF L3 attributes strucure
*			t_c		Translator context
*
*   Description: 
*				This function requests the IXASDK for deleting L3 attributes of
*				ports.
*
*   Return: none
*
*
*****************************************************************************/
void cm_delete_L3_attrib(npf_L3IntfAttrsList_t* l3attr, 
			 cm_TranslatorContext_t* t_c)
{    
    uint32_t i = 0;
    int32_t iRet = 0;
    ix_uint32 propId;
    ix_cc_properties property;
    ix_error err;
    
    npf_logger_write(LOG_FP_CM, LOG_TRACE, "Entering  cm_delete_L3_attrib");
    
    for (i = 0; i < l3attr->port_count; i++)
	{
	    memset(&property, 0, sizeof(ix_cc_properties));
	    property.blade_id = l3attr->port_list[i].feid;

	    npf_logger_write(LOG_FP_CM, LOG_INFO, 
			     "Attribs for portID=%d,"
			     "l3attr->port_list[i].type = %d\n", 
			     i, l3attr->port_list[i].type);
	    
	    property.port_id = l3attr->port_list[i].portid;
	    
	    switch (l3attr->port_list[i].type)
		{
		case NPF_L3ATTRS_ALL:
		    {
			npf_logger_write(LOG_FP_CM, LOG_INFO, 
					 "The IPv4 addr is 0x%x\n", 
					 l3attr->port_list[i].addr_u.addr);
			npf_logger_write(LOG_FP_CM, LOG_INFO, 
					 "The mask is 0x%x\n",
					 l3attr->port_list[i].mask_u.mask);
			npf_logger_write(LOG_FP_CM, LOG_INFO, 
					 "The ipfwding is %d\n",
					 l3attr->port_list[i].ipfwding);
			/*set the property ID */
			propId = IX_CC_SET_PROPERTY_ADD_INTERFACE_IPV4 |
			    IX_CC_SET_PROPERTY_PHYSICAL_IF_STATUS;
			
			property.ip_prop[0].protocol.ipv4_prop.ipv4_address = 
			    l3attr->port_list[i].addr_u.addr;
			property.ip_prop[0].protocol.ipv4_prop.ipv4_subnet_mask = 
			    l3attr->port_list[i].mask_u.mask;
			property.physical_if_state = 0;
			/*call into IXASDK API*/
			if(ix_cc_async_set_property(propId, &property) 
			   != IX_SUCCESS)
			    {
				npf_logger_write(LOG_FP_CM, LOG_ERR, 
						 "IXASDK30 API call failed for"
						 "portId:%d", 
						 l3attr->port_list[i].portid);
				iRet = -1;
			    }
			
			break;
		    }
		case NPF_L3ATTRS_IPADDR:
			{
			    npf_logger_write(LOG_FP_CM, LOG_INFO, 
					     "The IPv4 addr is 0x%x\n", 
					     l3attr->port_list[i].addr_u.addr);
			    npf_logger_write(LOG_FP_CM, LOG_INFO, 
					     "The mask is 0x%x\n",
					     l3attr->port_list[i].mask_u.mask);
			    npf_logger_write(LOG_FP_CM, LOG_INFO, 
					     "The ipfwding is %d\n",
					     l3attr->port_list[i].ipfwding);
			    
			    /*set the property ID */
			    propId = IX_CC_SET_PROPERTY_ADD_INTERFACE_IPV4 |
				IX_CC_SET_PROPERTY_PHYSICAL_IF_STATUS;
			    
			    property.ip_prop[0].protocol.ipv4_prop.ipv4_address = 
				l3attr->port_list[i].addr_u.addr;
			    property.ip_prop[0].protocol.ipv4_prop.ipv4_subnet_mask = 
				l3attr->port_list[i].mask_u.mask;
			    property.physical_if_state = 0;
			    /*call into IXASDK API*/
			    err = ix_cc_async_set_property(propId, &property);
			    
			    if(err != IX_SUCCESS)
				{
				    npf_logger_write(LOG_FP_CM, LOG_ERR, 
						     "IXASDK30 API call failed"
						     "for portId:%d error code:%d", 
						     l3attr->port_list[i].portid,
						     IX_ERROR_GET_CODE(err));
				    ix_error_dump(stdout, err);
				    iRet = -1;
				}
			    break;
			}
		case NPF_L3ATTRS_IPFWD:
		    {
			npf_logger_write(LOG_FP_CM, LOG_INFO, 
					 "IP FWDING OPTION NOT SUPPORTED ***\n");
			npf_logger_write(LOG_FP_CM, LOG_INFO, 
					 "IPFWDNG Value is %d", 
					 l3attr->port_list[i].ipfwding);
			break;
		    }
		default:
		    npf_logger_write(LOG_FP_CM, LOG_ERR, 
				     "Unknown TYPE OF L3 ATTR \n");
		}
	}
    fpm_cm_mgr_set_L3_attrib_upcall(iRet, t_c);
    npf_logger_write(LOG_FP_CM, LOG_TRACE, "Leaving cm_delete_L3_attrib");
}

/******************************************************************************
	TBD NOTE: The stats information gathering is just temporary, it gives the
	generic stats on L3 stats request, this will change when the CP 
	aligns with the latest NPF, but the code can be easily reused for rel 1.2

  *****************************************************************************/

/*****************************************************************************
*
*   Function:   cm_get_generic_stats()
*
*   Parameters: 
*			portlist: Pointer to NPF npf_PortList_t struct.
*			t_c:	  Translator context 
*			
*
*   Description:
*				This function queries the IXASDK for statistics.
*				It provides a callback function to the IXASDK to call back with 
*				the statistics information. 
*
*   Return: none
*****************************************************************************/
void cm_get_generic_stats(FPPI_Interface_GetStats_t *portlist, 
			  cm_L3_stats_TranslatorContext_t *t_c)
{
    int32_t iRet = 0;
    uint32_t i = 0, j = 0;
    cm_L3_stats_UserContext_t *u_c = NULL;
    cm_uc_info *uc_info = NULL;
    uint32_t *reqComp_Indctr = NULL;
    

    reqComp_Indctr = (uint32_t *)
	PIL_AllocateMemory (sizeof(uint32_t), PIL_MEM_ZERO_INIT);
    if(!reqComp_Indctr)
	{
	    npf_logger_write(LOG_FP_CM, LOG_ERR, 
			     "Mem allocation for reqComp_Indctr failed");
	    iRet = -1;
	    goto finish;
	}
    
    /*set the request completion indicator counter equal to:
      numport * 2 (= Rx + Tx, Stats)
      as the statistics have to be got from the the Tx intf and the RX intf
    */	
    *reqComp_Indctr = portlist->port_count * 2;
    
    for(i = 0; i < portlist->port_count; ++i) {	
	/*build the platform specific user context*/
	for(j = 0; j < 2; ++j) {
	    u_c = (cm_L3_stats_UserContext_t *)
		PIL_AllocateMemory (sizeof(cm_L3_stats_UserContext_t),
				    PIL_MEM_ZERO_INIT);
	    if(!u_c) {
		npf_logger_write(LOG_FP_CM, LOG_ERR, 
				 "Mem allocation for u_c failed");
		iRet = -1;
		goto finish;
	    }			
	    npf_logger_write(LOG_FP_CM, LOG_INFO, "Allocated u_c:0x%x", 
			     u_c);
	    u_c->reqComp_Indctr = reqComp_Indctr;
	    u_c->t_c = t_c;
	    u_c->port_index = i;
	    u_c->portId = portlist->port_array[i];
	    //u_c->portType = portlist->port_array[i].port_type;
	    uc_info = (cm_uc_info *)
		PIL_AllocateMemory (sizeof(cm_uc_info), PIL_MEM_ZERO_INIT);
	    if(!uc_info)
		{
		    npf_logger_write(LOG_FP_CM, LOG_ERR, 
				     "Mem allocation for uc_info failed");
		    iRet = -1;
		    (*(u_c->reqComp_Indctr))--;
		    goto finish;
		}
	    uc_info->type = CM_STATS;
	    uc_info->u_c = (void *)u_c;
	    /*insert this u_c info into the u_c pending list*/
	    npf_list_pushBack(&g_cm_pendinglist, uc_info);
	    
	    switch(portlist->ifType)
		{
#ifdef _INCLUDE_POS 
		case 0://POS_TYPE:
		    {
			   ix_cc_atm_pos_tx_statistics_info_context *pMgInfoContext = 
			    (ix_cc_atm_pos_tx_statistics_info_context *)
			    PIL_AllocateMemory
			    (sizeof(ix_cc_atm_pos_tx_statistics_info_context),
			     PIL_MEM_ZERO_INIT);
			
			if(!pMgInfoContext)
			    {
				npf_logger_write(LOG_FP_CM, LOG_ERR, 
						 "Mem allocation for pMgInfoContext"
						 "failed");
				iRet = -1;
				goto finish;
			    }
			if(j == 0)
			    pMgInfoContext->entity = IX_CC_POS_TX_ALL_COUNTERS;
			else if(j == 1)
			    pMgInfoContext->entity = IX_CC_POS_RX_ALL_COUNTERS;
			
			pMgInfoContext->index = i;
			pMgInfoContext->pUserContext = (void *)u_c;
			
			/*Call into the IXASDK for stats infor on TX and RX 
			  interfaces*/
			if(j == 0)//TX
			    {
#ifdef USE_FPM_CC
				if(ix_cc_fpm_async_get_pos_tx_stats 
				   (pMgInfoContext,
				    cm_get_generic_tx_stats_cb) != IX_SUCCESS)
				    {
					npf_logger_write(LOG_FP_CM, LOG_ERR, 
							 "IXASDK30 API call"
							 " failed for portId:%d", 
							 i);
					iRet = -1;
					(*(reqComp_Indctr))--;
					goto finish;
				    }
#endif /* FPM */
				/* just decrement the counter so that we dont 
				   have to wait for the callback for this
				*/
				(*(reqComp_Indctr))--;
			    }
			/*free the pMgInfoContext, as IXASDK should have made a copy*/
			if(pMgInfoContext)
			    PIL_FreeMemory(pMgInfoContext);
			break;
		    }
#endif /*  POS Specific */
		case 1://ETHERNET_TYPE:
		    {
			ix_cc_eth_tx_statistics_info_context* pMgInfoContext = 
			    (ix_cc_eth_tx_statistics_info_context *)
			    PIL_AllocateMemory 
			    (sizeof(ix_cc_eth_tx_statistics_info_context),
			     PIL_MEM_ZERO_INIT);
			
			if(!pMgInfoContext)
			    {
				npf_logger_write(LOG_FP_CM, LOG_ERR, 
						 "Mem allocation for pMgInfoContext"
						 "failed");
				iRet = -1;
				goto finish;
			    }
			if(j == 0)
			  pMgInfoContext->entity = IX_CC_ETH_TX_ALL_DRIVER_COUNTERS;
			else if(j == 1)
			  pMgInfoContext->entity = IX_CC_ETH_RX_ALL_DRIVER_COUNTERS;
			
			pMgInfoContext->index = i;
			pMgInfoContext->pUserContext = (void *)u_c;
			
			/*Call into the IXASDK for stats infor on TX and RX interfaces*/
			if(j == 0)//TX
			    {
#ifdef USE_FPM_CC
			      if(ix_cc_fpm_async_get_eth_tx_stats 
				 (pMgInfoContext,
				  cm_get_generic_tx_stats_cb) != IX_SUCCESS)
				{
				  npf_logger_write(LOG_FP_CM, LOG_ERR, 
						   "IXASDK30 API call failed"
						   "for portId:%d", i);
				  iRet = -1; 
				  
				  (*(reqComp_Indctr))--;
				  goto finish;
				}
#endif
			      /* simply decrement the completion counter so that
				 we dont have to wait for this callback
			      */
			      (*(reqComp_Indctr))--;
			      
			    }
/* NOTE: Change IX_CC_ETH to _INCLUDE_ETH later on for consistency ! */
#ifdef IX_CC_ETH 
			else if (j == 1)//RX
			  {
			    #ifdef IXASDKSTUBS
			    if(ix_cc_eth_rx_async_get_statistics_info
			    #else
			    if(fpm_ix_cc_eth_rx_async_get_statistics_info
			    #endif
			       ((ix_cc_eth_rx_statistics_info_context*)pMgInfoContext, 
				(ix_cc_eth_rx_cb_get_statistics_info)cm_get_generic_rx_stats_cb) != IX_SUCCESS)
			      {
				npf_logger_write(LOG_FP_CM, LOG_ERR, 
						 "IXASDK30 API call failed"
						 "for portId:%d", i);
				iRet = -1;
				(*(reqComp_Indctr))--;
				goto finish;
			      }
			  }
#endif /* IX_CC_ETH */
			/*free the pMgInfoContext, as IXASDK should have made a copy*/
			if(pMgInfoContext)
			  PIL_FreeMemory(pMgInfoContext);
			break;
		    }
		case 2://ATM_TYPE:
		    break;
		default:
		}
	}		
    }
    
 finish: /*Something is seriously wrong here, should never get here*/
    if(iRet == -1)
	{
	    /*Note that any unfreed u_c's and the related resources they point to 
	      will be anyhow freed during shut down*/
	    if(cm_is_Valid(u_c))
		PIL_FreeMemory(u_c);
	    
	    if(!reqComp_Indctr || *reqComp_Indctr == 0)
		{
		    /*inform the Translator component about failure via the up-call*/
		    fpm_cm_mgr_get_L3_stats_upcall(iRet, t_c);
		    if(reqComp_Indctr)
			PIL_FreeMemory(reqComp_Indctr);
		}
	}
}

/*****************************************************************************
******************************************************************************
*	Callback Function Implementations
******************************************************************************
*****************************************************************************/

/*****************************************************************************
*
*   Function:   cm_get_port_num_cb()
*
*   Parameters: 
*			arg_Result:error code returned from the IXASDK messaging operation.
*			arg_pContext:pointer to user context		
*			arg_pNumPorts: pointer to where the number of ports is stored.
*
*   Description: Callback function that contains the port_num information.
*				 This invokes the translator fpm_cm_mgr_get_port_num_upcall().
*
*   Return: 0 on success, -1 on failure
*****************************************************************************/
ix_error cm_get_port_num_cb(ix_error arg_Result, void *arg_pContext, 
			    ix_uint32* arg_pNumPorts)
{
	ix_error err = IX_SUCCESS;
	cm_portnum_UserContext_t * u_c = NULL;
	cm_portnum_TranslatorContext_t * t_c = NULL;
	
	npf_logger_write(LOG_FP_CM, LOG_TRACE, "Entering cm_get_port_num_cb");

	if(!arg_pContext)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, "Invalid pContext");
		err = IX_ERROR_NEW(IX_CC_ERROR_NULL, IX_ERROR_LEVEL_WARNING);
		goto finish;
	}
	if(arg_Result != IX_SUCCESS)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"Invalid result from the messaging operation: %d", arg_Result);
		ix_error_dump(stdout, arg_Result);
		err = IX_ERROR_NEW(IX_CC_ERROR_NULL, IX_ERROR_LEVEL_WARNING);
		goto finish;
	}	
	/*validate the u_c*/
	if(!cm_is_Valid(arg_pContext))
	{
		err =  IX_ERROR_NEW(IX_CC_ERROR_RANGE, IX_ERROR_LEVEL_WARNING);
		npf_logger_write(LOG_FP_CM, LOG_ERR, "invalid u_c:0x%x", 
			arg_pContext);
		goto finish;
	}

	u_c = (cm_portnum_UserContext_t *)arg_pContext;
	t_c = (cm_portnum_TranslatorContext_t *)u_c->t_c;
	if(!t_c)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"t_c is invalid");
		err =  IX_ERROR_NEW(IX_CC_ERROR_RANGE, IX_ERROR_LEVEL_WARNING);;
		goto finish;
	}
	/*fill in the translator context with port number*/
	npf_logger_write(LOG_FP_CM, LOG_INFO, "Number of ports:%d",*arg_pNumPorts);
	memcpy(t_c->port_num , arg_pNumPorts, sizeof(uint32_t));
	
	/*Free the User context memory here as we are done with it*/
	if(u_c)
	{
		npf_logger_write(LOG_FP_CM, LOG_INFO, "Freeing u_c:0x%x", u_c);
		PIL_FreeMemory(u_c);
	}
	
	/*call into the translator upcall with the received information*/	
	fpm_cm_mgr_get_port_num_upcall(0, t_c);	

finish:/*only if something is seriously wrong*/
	if(err != IX_SUCCESS)
		fpm_cm_mgr_get_port_num_upcall(-1, t_c);
	
	npf_logger_write(LOG_FP_CM, LOG_TRACE, "Leaving cm_get_port_num_cb");
	return err;
}

/*****************************************************************************
*
*   Function:   cm_get_port_fecaps_cb()
*
*   Parameters: 
*			arg_Result:error code returned from the IXASDK messaging operation.
*			arg_pContext:pointer to user context		
*			arg_pProperty: pointer to properties structure.
*
*   Description: Callback function that contains the property information 
*				 for a specific port. Once all the required caps is available
*				 the translator fpm_cm_mgr_get_fecaps_upcall() is invoked.
*
*   Return: 0 on success, -1 on failure
*****************************************************************************/
ix_error cm_get_port_fecaps_cb(ix_error arg_Result, void *arg_pContext,
			       ix_cc_properties* arg_pProperty)
{
    ix_error err = IX_SUCCESS;
    uint32_t i = 0, port_idx = 0;
    cm_fecaps_UserContext_t * u_c = NULL;
    cm_fecaps_TranslatorContext_t * t_c = NULL;
    
    npf_logger_write(LOG_FP_CM, LOG_TRACE, "Entering cm_get_port_fecaps_cb");
    
    if(!arg_pContext)
	{
	    npf_logger_write(LOG_FP_CM, LOG_ERR, "Invalid arg_pContext");
	    err = IX_ERROR_NEW(IX_CC_ERROR_NULL, IX_ERROR_LEVEL_WARNING);
	    goto finish;
	}
    if(!arg_pProperty)
	{
	    npf_logger_write(LOG_FP_CM, LOG_ERR, "Invalid arg_pProperty");
	    err = IX_ERROR_NEW(IX_CC_ERROR_NULL, IX_ERROR_LEVEL_WARNING);
	    goto finish;
	}
    if(arg_Result != IX_SUCCESS)
	{
	    npf_logger_write(LOG_FP_CM, LOG_ERR, 
			     "Invalid result from the messaging operation: %d", 
			     arg_Result);
	    ix_error_dump(stdout, arg_Result);
	    err = IX_ERROR_NEW(IX_CC_ERROR_NULL, IX_ERROR_LEVEL_WARNING);
	    goto finish;
	}	
    /*validate the u_c*/
    if(!cm_is_Valid(arg_pContext))
	{
	    err =  IX_CC_ERROR_RANGE;
	    npf_logger_write(LOG_FP_CM, LOG_ERR, "invalid u_c:0x%x", 
			     arg_pContext);
	    goto finish;
	}
    u_c = (cm_fecaps_UserContext_t *)arg_pContext;
    t_c = (cm_fecaps_TranslatorContext_t *)u_c->t_c;
    if(!t_c)
	{
	    npf_logger_write(LOG_FP_CM, LOG_ERR, 
			     "t_c is invalid");
	    err =  IX_ERROR_NEW(IX_CC_ERROR_RANGE, IX_ERROR_LEVEL_WARNING);
	    goto finish;
	}
    /*
      Here we need to make some processing. Basically IXA SDK 3.0 returns
      the ports in the reverse order of port ids (3, 2, 1, 0). This causes
      problems in the control plane since the Namespace nodes are now
      numbered in the reverse order as the port ids.
      
      A good way to solve this is to sort the port list array wrt port ids,
      but for the time being, since we know that the port ids will be 
      between 0-3, we will just use the port id to index into the array
      of ports that we are going to store the information in.
    */
    for(i = 0; i < t_c->fecaps->ports_caps.port_count; ++i)
	{
	    /*this will be same for all the ports*/
	    g_mybladeID = arg_pProperty[i].blade_id;
	    npf_logger_write(LOG_FP_CM, LOG_INFO, "g_mybladeID=%d\n",
			     g_mybladeID);
	    
	    port_idx = arg_pProperty[i].port_id;
	    /*fill in the translator context fecaps struct with FE capabilities*/
	    t_c->fecaps->ports_caps.port_caps_list[port_idx].portid = 
		arg_pProperty[i].port_id;
	    /* now translate the media type to NPF defined NPF_PORT_TYPE */
	    switch (arg_pProperty[i].media_type)
		{
		case IX_MEDIA_TYPE_FAST_ETHERNET:
		    t_c->fecaps->ports_caps.port_caps_list[port_idx].port_type =
			NPF_PORT_ETHERNET;
		    break;
		case IX_MEDIA_TYPE_1GB_ETHERNET:
		    t_c->fecaps->ports_caps.port_caps_list[port_idx].port_type =
			NPF_PORT_ETHERNET;
		    break;
		case IX_MEDIA_TYPE_ATM:
		    t_c->fecaps->ports_caps.port_caps_list[port_idx].port_type =
			NPF_PORT_ATM;
		    break;
		case IX_MEDIA_TYPE_POS:
		    t_c->fecaps->ports_caps.port_caps_list[port_idx].port_type =
			NPF_PORT_POS;
		    break;
		case IX_MEDIA_TYPE_UNKNOWN:
		    t_c->fecaps->ports_caps.port_caps_list[port_idx].port_type =
			NPF_PORT_UNKNOWN;
		    break;
		case IX_MEDIA_TYPE_LAST:
		    t_c->fecaps->ports_caps.port_caps_list[port_idx].port_type =
			NPF_PORT_UNKNOWN;
		    break;
	 	default:
		    t_c->fecaps->ports_caps.port_caps_list[port_idx].port_type =
			NPF_PORT_UNKNOWN;
		    break;
		} /* end switch */

		//arg_pProperty[i].media_type;
	    memcpy (t_c->fecaps->ports_caps.port_caps_list[port_idx].mac_addr,
		    arg_pProperty[i].mac_addr, IX_CC_MAC_ADDR_LEN);
	    t_c->fecaps->ports_caps.port_caps_list[port_idx].link_speed = 
		arg_pProperty[i].link_speed;
	    t_c->fecaps->ports_caps.port_caps_list[port_idx].mtu = 
		arg_pProperty[i].mtu;
	    
	    /*at this point we have got all the information for the 
	      translator request. We are ready to call into the translator 
	      upcall with the received information*/
	    
	    npf_logger_write
		(LOG_FP_CM, LOG_INFO, 
		 "port id--->%d", 
		 t_c->fecaps->ports_caps.port_caps_list[port_idx].portid);
	    npf_logger_write
		(LOG_FP_CM, LOG_INFO, 
		 "port_type--->%d", 
		 t_c->fecaps->ports_caps.port_caps_list[port_idx].port_type);
	    npf_logger_write
		(LOG_FP_CM, LOG_INFO, 
		 "mac_addr--->0x%x::%x::%x::%x::%x::%x", 
		 t_c->fecaps->ports_caps.port_caps_list[port_idx].mac_addr[0],
		 t_c->fecaps->ports_caps.port_caps_list[port_idx].mac_addr[1],
		 t_c->fecaps->ports_caps.port_caps_list[port_idx].mac_addr[2],
		 t_c->fecaps->ports_caps.port_caps_list[port_idx].mac_addr[3],
		 t_c->fecaps->ports_caps.port_caps_list[port_idx].mac_addr[4],
		 t_c->fecaps->ports_caps.port_caps_list[port_idx].mac_addr[5]
		 );
	    npf_logger_write
		(LOG_FP_CM, LOG_INFO, 
		 "link_speed--->%d", 
		 t_c->fecaps->ports_caps.port_caps_list[port_idx].link_speed);
	    npf_logger_write(LOG_FP_CM, LOG_INFO, 
			     "mtu--->%d", 
			     t_c->fecaps->ports_caps.port_caps_list[port_idx].mtu);
	}
    
    if(u_c)
	{
	    npf_logger_write(LOG_FP_CM, LOG_INFO, "Freeing u_c:0x%x", u_c);
	    PIL_FreeMemory(u_c);
	}
    
    fpm_cm_mgr_get_fecaps_upcall(0, t_c);
    
 finish:/*only if something is seriously wrong*/
    if(err != IX_SUCCESS)
	fpm_cm_mgr_get_fecaps_upcall(-1, t_c);
    
    npf_logger_write(LOG_FP_CM, LOG_TRACE, "Leaving cm_get_port_fecaps_cb");
    return err;
}

/*****************************************************************************
*
*   Function:   cm_get_generic_tx_stats_cb
*
*   Parameters: 
*			arg_Result:error code returned from the IXASDK messaging operation.
*			arg_pContext:pointer to user context		
*			arg_pNumPorts: pointer to properties structure.
*
*   Description: Callback function that contains the property information 
*				 for a specific port.Once all the required stats is available
*				 the translator fpm_cm_get_port_num_upcall() is invoked.
*
*   Return: IX_SUCCESS on success, -1 on failure
*****************************************************************************/
ix_error cm_get_generic_tx_stats_cb(ix_error arg_Result, void *arg_pContext,
				    ix_uint64 *arg_pBuffer)
{
	ix_error err = IX_SUCCESS;
	uint32_t i = 0;
	cm_L3_stats_UserContext_t *u_c = NULL;
	cm_L3_stats_TranslatorContext_t *t_c = NULL;
	
	if(!arg_pContext)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, "Invalid arg_pContext");
		err = IX_ERROR_NEW(IX_CC_ERROR_NULL, IX_ERROR_LEVEL_WARNING);
		goto finish;
	}
	
	if(!arg_pBuffer)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, "Invalid arg_pBuffer");
		err = IX_ERROR_NEW(IX_CC_ERROR_NULL, IX_ERROR_LEVEL_WARNING);
		goto finish;
	}
	if(arg_Result != IX_SUCCESS)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"Invalid result from the messaging operation: %d", arg_Result);
		ix_error_dump(stdout, arg_Result);
		err = IX_ERROR_NEW(IX_CC_ERROR_NULL, IX_ERROR_LEVEL_WARNING);
		goto finish;
	}
	
	/*validate the u_c*/
	if(!cm_is_Valid(arg_pContext))
	{
		err = IX_ERROR_NEW(IX_CC_ERROR_RANGE, IX_ERROR_LEVEL_WARNING);
		npf_logger_write(LOG_FP_CM, LOG_ERR, "invalid u_c:0x%x", 
			arg_pContext);
		goto finish;
	}
	u_c = (cm_L3_stats_UserContext_t *)arg_pContext;
	t_c = (cm_L3_stats_TranslatorContext_t *)u_c->t_c;
	if(!t_c)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"t_c is invalid");
		err =  IX_ERROR_NEW(IX_CC_ERROR_RANGE, IX_ERROR_LEVEL_WARNING);
		goto finish;
	}

	if(*(u_c->reqComp_Indctr) != 0)
	{
	    /*decrement the request completion indicator counter*/
	    (*(u_c->reqComp_Indctr))--;
	    
	    /*fill in the translator context fecaps struct with FE capabilities*/
	    t_c->stats_list[u_c->port_index].portid = u_c->portId;
	    t_c->stats_list[u_c->port_index].l3stats.bytesRx = 0;
	    t_c->stats_list[u_c->port_index].l3stats.ucPackRx = 0;
	    t_c->stats_list[u_c->port_index].l3stats.mcPackRx = 0;
	    t_c->stats_list[u_c->port_index].l3stats.bcPackRx = 0;
	    t_c->stats_list[u_c->port_index].l3stats.dropRx = 0;
	    t_c->stats_list[u_c->port_index].l3stats.errorRx = 0;
	    t_c->stats_list[u_c->port_index].l3stats.protoRx = 0;
	    t_c->stats_list[u_c->port_index].l3stats.mcPackRx = 0;
	    t_c->stats_list[u_c->port_index].l3stats.bytesTx = arg_pBuffer[3];
	    t_c->stats_list[u_c->port_index].l3stats.ucPackTx = arg_pBuffer[7];
	    t_c->stats_list[u_c->port_index].l3stats.mcPackTx = arg_pBuffer[8];
	    t_c->stats_list[u_c->port_index].l3stats.bcPackTx = arg_pBuffer[9];
	    t_c->stats_list[u_c->port_index].l3stats.dropTx = arg_pBuffer[4];
	    t_c->stats_list[u_c->port_index].l3stats.errorTx = 0;
	}
	if(*(u_c->reqComp_Indctr) == 0)
	{
		/*at this point we have got all the information for the 
		translator request. We are ready to call into the translator 
		upcall with the received information...also, free the 
		reqComp_Indctr*/
		for(i = 0; i < t_c->num_entries; ++i)
		{
		  /*
		    npf_logger_write(LOG_FP_CM, LOG_INFO, 
		    "TX BYTES--->%d for port index:%d", 
		    t_c->stats_list[i].l3stats.ifOutOctets, i);
		    npf_logger_write(LOG_FP_CM, LOG_INFO, 
		    "TX PACKETS--->%d for port index:%d", 
		    t_c->stats_list[i].l3stats.ifOutUcastPkts, i);
		    npf_logger_write(LOG_FP_CM, LOG_INFO, 
		    "TX DEOPPED PACKETS--->%d for port index:%d", 
		    t_c->stats_list[i].l3stats.ifOutErrors, i);
		  */
		}
		fpm_cm_mgr_get_L3_stats_upcall(0, t_c);				
		PIL_FreeMemory(u_c->reqComp_Indctr);
	}

	/*free the u_c for this callback*/
	{
		npf_logger_write(LOG_FP_CM, LOG_INFO, "Freeing u_c:0x%x", u_c);
		PIL_FreeMemory(u_c);
	}	

finish:/*only if something is seriously wrong. Note that the u_c's and the 
	   related resources they point to will be anyhow freed during shut down*/

	if(err != IX_SUCCESS)
		fpm_cm_mgr_get_L3_stats_upcall(-1, t_c);
	return err;
}

/*****************************************************************************
*
*   Function:   cm_get_generic_rx_stats_cb
*
*   Parameters: 
*			arg_Result:error code returned from the IXASDK messaging operation.
*			arg_pContext:pointer to user context		
*			arg_pNumPorts: pointer to properties structure.
*
*   Description: Callback function that contains the property information 
*				 for a specific port. Once all the required caps is available
*				 the translator fpm_cm_mgr_get_fecaps_upcall() is invoked.
*
*   Return: 0 on success, -1 on failure
*****************************************************************************/
ix_error cm_get_generic_rx_stats_cb(ix_error arg_Result, void *arg_pContext,
				    ix_uint64 *arg_pBuffer)
{
	ix_error err = IX_SUCCESS;
	uint32_t i = 0;
	cm_L3_stats_UserContext_t *u_c = NULL;
	cm_L3_stats_TranslatorContext_t *t_c = NULL;
	
	if(!arg_pContext)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, "Invalid arg_pContext");
		err =  IX_ERROR_NEW(IX_CC_ERROR_NULL, IX_ERROR_LEVEL_WARNING);
		goto finish;
	}

	if(!arg_pBuffer)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, "Invalid arg_pBuffer");
		err =  IX_CC_ERROR_NULL;
		goto finish;
	}
	if(arg_Result != IX_SUCCESS)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"Invalid result from the messaging operation: %d", arg_Result);
		ix_error_dump(stdout, arg_Result);
		err = IX_ERROR_NEW(IX_CC_ERROR_NULL, IX_ERROR_LEVEL_WARNING);
		goto finish;
	}
	
	/*validate the u_c*/
	if(!cm_is_Valid(arg_pContext))
	{
		err = IX_ERROR_NEW(IX_CC_ERROR_RANGE, IX_ERROR_LEVEL_WARNING);
		npf_logger_write(LOG_FP_CM, LOG_ERR, "invalid u_c:0x%x", 
			arg_pContext);
		goto finish;
	}
	u_c = (cm_L3_stats_UserContext_t *)arg_pContext;
	t_c = (cm_L3_stats_TranslatorContext_t *)u_c->t_c;
	if(!t_c)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"t_c is invalid");
		err =  IX_ERROR_NEW(IX_CC_ERROR_RANGE, IX_ERROR_LEVEL_WARNING);
		goto finish;
	}

	if(*(u_c->reqComp_Indctr) != 0)
	{
		/*decrement the request completion indicator counter*/
		(*(u_c->reqComp_Indctr))--;

		/*fill in the translator context fecaps struct with FE capabilities*/
		t_c->stats_list[u_c->port_index].portid = u_c->portId;
		t_c->stats_list[u_c->port_index].l3stats.bytesRx = arg_pBuffer[4];
		t_c->stats_list[u_c->port_index].l3stats.ucPackRx = arg_pBuffer[8];
		t_c->stats_list[u_c->port_index].l3stats.mcPackRx = arg_pBuffer[9];;
		t_c->stats_list[u_c->port_index].l3stats.bcPackRx = arg_pBuffer[10];;
		t_c->stats_list[u_c->port_index].l3stats.dropRx = 0;
		t_c->stats_list[u_c->port_index].l3stats.errorRx = 0;
		t_c->stats_list[u_c->port_index].l3stats.protoRx = 0;
		t_c->stats_list[u_c->port_index].l3stats.mcPackRx = 0;
	}
	if(*u_c->reqComp_Indctr == 0)
	{
		/*at this point we have got all the information for the 
		translator request. We are ready to call into the translator 
		upcall with the received information...also, free the 
		reqComp_Indctr*/
		for(i = 0; i < t_c->num_entries; ++i)
		{
		  /*
		    npf_logger_write(LOG_FP_CM, LOG_INFO, 
		    "RX BYTES--->%d for port index:%d", 
		    t_c->stats_list[i].l3stats.ifInOctets, i);
		    npf_logger_write(LOG_FP_CM, LOG_INFO, 
		    "RX PACKETS--->%d for port index:%d", 
		    t_c->stats_list[i].l3stats.ifInUcastPkts, i);
		    npf_logger_write(LOG_FP_CM, LOG_INFO, 
		    "RX DEOPPED PACKETS--->%d for port index:%d", 
		    t_c->stats_list[i].l3stats.IfInErrors, i);
		  */
		}
		fpm_cm_mgr_get_L3_stats_upcall(0, t_c);		
		
		/*free the user context resources for the the related down-call*/
		PIL_FreeMemory(u_c->reqComp_Indctr);
	}
	/*free the u_c for this callback*/
	{
		npf_logger_write(LOG_FP_CM, LOG_INFO, "Freeing u_c:0x%x", u_c);
		PIL_FreeMemory(u_c);
	}

finish:/*only if something is seriously wrong. Note that the u_c's and the 
	   related resources they point to will be anyhow freed during shut down*/
	if(err != IX_SUCCESS)
		fpm_cm_mgr_get_L3_stats_upcall(-1, t_c);
	return err;
}

/*****************************************************************************
*
*   Function:   cm_is_Valid()
*
*   Parameters: 
*			u_c: Pointer to u_c
*
*   Description: 
*				Searches for the u_c in the pending u_c list, pops out the
*				entry if the match is found 
*
*   Return: 1 for if match found, 0 if match is not found
*****************************************************************************/
uint32_t cm_is_Valid(void *u_c)
{
	cm_uc_info	*uc_info = NULL;
	DListItr *itr;
	int found = 0;

	if(!u_c)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, "Invalid u_c");
		return 0;
	}

	npf_logger_write(LOG_FP_CM, LOG_TRACE,
		"Entering cm_is_Valid with u_c:0x%x", u_c);
	
	/*walk through the list	to look for a match*/
	itr = npf_list_itrCreate(&g_cm_pendinglist);
	if (npf_list_itrFirst(itr) != -1) 
	{
        do {
				uc_info = (cm_uc_info *)npf_list_itrGetData(itr);
				if(uc_info->u_c == u_c)
				{
					found = 1;
					npf_logger_write(LOG_FP_CM, LOG_TRACE,
						"Found a match");
					npf_list_itrRemoveFree(itr);
				}
			} while (!found && npf_list_itrNext(itr) != -1);
    }	
	npf_list_itrDelete(itr);	
	npf_logger_write(LOG_FP_CM, LOG_TRACE, "Leaving cm_is_Valid");
	
	if(found)
		return 1;
	return 0;

}

/*****************************************************************************
*
*   Function:   cm_free_resources()
*
*   Parameters: 
*			none
*
*   Description: 
*				Walks thro' the u_c pending list and frees up the resources
				and appropriately calls into the translator upcall
*
*   Return: none
*****************************************************************************/
void cm_free_resources()
{
    cm_uc_info *uc_info = NULL;
    
    npf_logger_write(LOG_FP_CM, LOG_TRACE,
		     "Entering cm_free_resources");
    
    /*walk through the list and delete the pending resources*/	
    while (npf_list_popBack(&g_cm_pendinglist, (void **)&uc_info) != -1) 
	{
	    if(uc_info)
		{
		    switch(uc_info->type)
			{
			case CM_STATS:
				if(uc_info->u_c)
				{
					/*call into the translator*/
					fpm_cm_mgr_get_L3_stats_upcall( -1,
						((cm_L3_stats_UserContext_t *)(uc_info->u_c))->t_c
						);
					/*free the related resources*/
					if(((cm_L3_stats_UserContext_t *)
						(uc_info->u_c))->reqComp_Indctr)
					{
						PIL_FreeMemory(
							((cm_L3_stats_UserContext_t *)
							(uc_info->u_c))->reqComp_Indctr
							);
						((cm_L3_stats_UserContext_t *)
							(uc_info->u_c))->reqComp_Indctr = NULL;
					}						
					PIL_FreeMemory(uc_info->u_c);
					npf_logger_write(LOG_FP_CM, LOG_INFO,
						"Freed CM_STATS related resources");
				}
				break;
			case CM_PORTNUM:
				if(uc_info->u_c)
				{
					/*call into the translator*/
					fpm_cm_mgr_get_port_num_upcall( -1,
						((cm_portnum_UserContext_t *)(uc_info->u_c))->t_c
						);	
					/*free the related resources*/
					PIL_FreeMemory(uc_info->u_c);
					npf_logger_write(LOG_FP_CM, LOG_INFO,
						"Freed CM_PORTNUM related resources");
				}						
				break;
			case CM_CAPS:
				if(uc_info->u_c)
				{
					/*call into the translator*/
					fpm_cm_mgr_get_fecaps_upcall( -1,
						((cm_fecaps_UserContext_t *)(uc_info->u_c))->t_c
						);
					/*free the related resources*/
					PIL_FreeMemory(uc_info->u_c);
					npf_logger_write(LOG_FP_CM, LOG_INFO,
						"Freed CM_CAPS related resources");
				}	
				break;
			}
			PIL_FreeMemory(uc_info);
		}
    }	
	npf_logger_write(LOG_FP_CM, LOG_TRACE, "Leaving cm_free_resources");	
}

/*****************************************************************************
*
*
*   Function:   mask_to_prefixLength()
*
*   Parameters: mask: Subnet Mask
*               
*
*   Description: This function returns the prefix Length for the given mask
*             
*
*   Return: Prefix Length
*
*
*****************************************************************************/
uint32_t mask_to_prefixLength(unsigned int mask)
{
	int i, prefixLen = 0;
	for(i = 0; i < 32; ++i)
	{
		if(mask & 1)
			++prefixLen;
		mask = mask >> 1;
	}
	return prefixLen;
}

/*****************************************************************************
*
*
*   Function:   cm_set_default_routes()
*
*   Parameters: 
*               
*
*   Description: 
*             
*
*   Return: 
*
*
*****************************************************************************/
int32_t cm_set_default_routes(uint32_t addr, uint32_t mask_len, 
			      uint32_t feid, uint32_t portid)
{
    uint32_t i, subnet_route_addr;
    uint32_t nhid, mask;
    ix_cc_rtmv4_next_hop_info nh_info;
    mask = mask_len;

    npf_logger_write (LOG_FP_CM, LOG_INFO, "Mask = 0x%x!\n", mask);
    
    for( i = 0; i < 2; ++i)
	{
	    nhid = get_nhid(); //get some reserved values
	    if (nhid != -1) {
	      if(i == 0) { //subnet route
		npf_logger_write (LOG_FP_CM, LOG_INFO, "Adding Subnet Route!\n"
				  "Next Hop ID = 0x%x\n", nhid);
		subnet_route_addr = addr & mask;
		nh_info.bladeID = feid;
		nh_info.l2Index = 0; /* reservered L2 Id = 0 */
		nh_info.portID = portid;
		nh_info.mtu = 1500;
		nh_info.flags = 0;
		nh_info.ipAddr = 0;
		nh_info.l2IndexType = 0;
	      }
	      else { //exact match route
		/* Mask Len for Exact match for IPv4 is 32 */
		nh_info.flags = 1; /* LOCAL DELIVERY */
		mask = 0xFFFFFFFF;
		nh_info.bladeID = feid;
		nh_info.portID = portid;
		nh_info.mtu = 1500;
		nh_info.ipAddr = 0;
		nh_info.l2IndexType = 0;
		npf_logger_write (LOG_FP_CM, LOG_INFO, "Adding Exact Match Route!\n"
				  "Next Hop ID = 0x%x\n", nhid);
	      }
	      if(ix_cc_ipv4_async_add_next_hop (nhid, &nh_info, 
						(ix_cc_ipv4_cb_route_op)cm_dummy_cb,
						(void*) &i) 
		 != IX_SUCCESS)
		{
		  npf_logger_write(LOG_FP_CM, LOG_ERR, 
				   "IXASDK30 API call failed for adding a"
				   "next hop");
		  return  -1;
		}
	      if(ix_cc_ipv4_async_add_route(
					    addr, mask, 
					    nhid,
					    (ix_cc_ipv4_cb_route_op) cm_dummy_cb,
					    (void *)&nhid) != IX_SUCCESS)
		{
		  npf_logger_write(LOG_FP_CM, LOG_ERR, 
				   "IXASDK30 API call failed for adding a route");
		  return  -1;
		}
	    } /* end if (valid NH ID) */
	} /* end for loop */
    return 0;
}


