/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2002 Intel Corporation. All Rights Reserved.
*****************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "FPModule.h"
#include "cm_mgr_impl.h"
#include "bd_mgr.h"
#include "pil.h"
#include "log.h"

/* Callback Handles - Callbacks registered with Back end API */
static FPPI_CBHANDLE L2GetStatsCBHdl, L3GetStatsCBHdl, L2SetAttrCBHdl;
static FPPI_CBHANDLE L3SetAttrCBHdl, L3DelArrCBHdl;

/* Contexts of callbacks registered with BEND API */
static FPPI_CONTEXT g_L2_setAttr_context;
static FPPI_CONTEXT g_L2_getStats_context;
static FPPI_CONTEXT g_L3_getStats_context;
static FPPI_CONTEXT g_L3_setAttr_context;
static FPPI_CONTEXT g_L3_delAttr_context;
static FPPI_CONTEXT g_ICMP_getStats_context;

int g_totallinks = MAX_IXP_LINKS;
extern uint32_t g_mybladeID;

void fpm_cm_mgr_get_port_num(char* name, FPPI_CBCORRELATOR cbCorrelator, 
							 char get_fecaps);
/*****************************************************************************
*
*   Function:   fpm_cm_mgr_init()
*
*   Parameters: None 
*
*   Description: This function starts up the CM manager. It registers 
*				 appropriate for callbacks with the Back-end API, initializes 
*				 the platform specific component.
*
*   Return: 0 on success, FPPAPI_FAILURE on failure 
*****************************************************************************/
int fpm_cm_mgr_init()
{	
	int errtype;

	/* Assigning Contexts for registering for callbacks */
	g_L2_setAttr_context = (void *) FPPI_L2_SetProperties;
    g_L2_getStats_context = (void *) FPPI_L2_GetStatistics;
	g_L3_getStats_context = (void *) FPPI_L3_GetStatistics;
	g_L3_setAttr_context = (void *) FPPI_L3_SetProperties;
	g_ICMP_getStats_context = (void *) FPPI_ICMP_GetStatistics;
	g_L3_delAttr_context = (void *) FPPI_L3_DelProperties;

	errtype = BENDAPI_register_callback (&fpm_cm_mgr_set_L2_attrib, 
					 &L2SetAttrCBHdl, 
					 FPPI_L2_SetProperties, 
					 g_L2_setAttr_context);
    if (FPPI_SUCCESS != errtype)
    {
        npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"Registering for Set L2 Attributes  Failed\n");
        return (FPPI_FAILURE);
    }

	errtype = BENDAPI_register_callback (&fpm_cm_mgr_get_L2_stats, 
					 &L2GetStatsCBHdl, 
					 FPPI_L2_GetStatistics, 
					 g_L2_getStats_context);
    if (FPPI_SUCCESS != errtype)
    {
        npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"Registering for  L2 Statistics  Failed\n");
        return (FPPI_FAILURE);
    }
	errtype = BENDAPI_register_callback (&fpm_cm_mgr_get_L3_stats, 
					 &L3GetStatsCBHdl, 
					 FPPI_L3_GetStatistics, 
					 g_L3_getStats_context);
    if (FPPI_SUCCESS != errtype)
    {
        npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"Registering for  L3 Statistics  Failed\n");
        return (FPPI_FAILURE);
    }
	errtype = BENDAPI_register_callback (&fpm_cm_mgr_set_L3_attrib, 
					 &L3SetAttrCBHdl, 
					 FPPI_L3_SetProperties, 
					 g_L3_setAttr_context);
    if (FPPI_SUCCESS != errtype)
    {
        npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"Registering for L3 Set Properties Failed\n");
        return (FPPI_FAILURE);
    }
	errtype = BENDAPI_register_callback (&fpm_cm_mgr_delete_L3_attrib, 
					 &L3DelArrCBHdl, 
					 FPPI_L3_DelProperties, 
					 g_L3_delAttr_context);
    if (FPPI_SUCCESS != errtype)
    {
        npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"Registering for L3 Delete Properties Failed\n");
        return (FPPI_FAILURE);
    }
	/*errtype = BENDAPI_register_callback (&fpm_cm_mgr_get_icmp_stats, 
					 &ICMPGetStatsCBHdl, 
					 FPPI_ICMP_GetStatistics, 
					 g_ICMP_getStats_context);
    if (FPPI_SUCCESS != errtype)
    {
        npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"Registering for  ICMP Statistics  Failed\n");
        return (FPPI_FAILURE);
    }*/

	/* Call the FP blade SDK specifics for Initialization*/
	cm_InitFPEngine();

	return (0);
}

/*****************************************************************************
*
*   Function:   fpm_cm_mgr_shutdown()
*
*   Parameters: None 
*
*   Description: This function shuts-down the CMM. It deregisters for 
*                callbacks with the Back-end API, de-initializes the platform 
*				 specific component
*
*   Return: 0 on success, FPPAPI_FAILURE on failure 
*****************************************************************************/
int fpm_cm_mgr_shutdown()
{
	int errtype;

	errtype =  BENDAPI_deregister_callback (L2SetAttrCBHdl);
    if (FPPI_SUCCESS != errtype)
    {
        npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"De-registering for  L2 Set Attr Failed\n");
        //return (FPPI_FAILURE);
    }    
	errtype = BENDAPI_deregister_callback (L2GetStatsCBHdl);
    if (FPPI_SUCCESS != errtype)
    {
        npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"De-registering for  L2 Get Stats Failed\n");
        //return (FPPI_FAILURE);
    }
	errtype = BENDAPI_deregister_callback (L3GetStatsCBHdl);
    if (FPPI_SUCCESS != errtype)
    {
        npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"De-registering for  L3 Get Stats Failed\n");
        //return (FPPI_FAILURE);
    }
	errtype = BENDAPI_deregister_callback (L3SetAttrCBHdl);
    if (FPPI_SUCCESS != errtype)
    {
        npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"De-registering for L3 Set Attr Failed\n");
        //return (FPPI_FAILURE);
    }
	errtype = BENDAPI_deregister_callback (L3DelArrCBHdl);
    if (FPPI_SUCCESS != errtype)
    {
        npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"De-registering for L3 Del Attr Failed\n");
        //return (FPPI_FAILURE);
    }
	/* Call the FP blade SDK specifics for shutdown*/
	cm_DestroyFPEngine();

    return (FPPI_SUCCESS);
}

/****************************************************************************
*																			*
*																			*
*								  FP Module									*
*							CM Manager EXTERNAL APIs						*
*																			*
*																			*
****************************************************************************/

/***************************************************************************
*
*   Function:   fpm_cm_mgr_get_fecaps
*
*   Parameters:
*			name: Name of the fe that also contains the fe id
*			cbCorrelator: CallBack Correlator: 
*
*   Description:
*				This function is invoked by the B&D Manager to get the 
*				capabilities of the Forwarding Element (FE). 
*
*   Return: None 
****************************************************************************/
void fpm_cm_mgr_get_fecaps(char* name, FPPI_CBCORRELATOR cbCorrelator)
{
	npf_logger_write(LOG_FP_CM, LOG_TRACE, "Entering fpm_cm_mgr_get_fecaps");
	
	fpm_cm_mgr_get_port_num(name, cbCorrelator, 1);

	npf_logger_write(LOG_FP_CM, LOG_TRACE, "Leaving fpm_cm_mgr_get_fecaps");
}


/***************************************************************************
*
*   Function:   fpm_cm_mgr_get_port_num
*
*   Parameters:
*			name: Name of the fe that also contains the fe id
*			cbCorrelator: CallBack Correlator: 
*			get_fecaps: A flag, 0 if only port number is to be retrieved
*						1 if the fecaps also needs to be retrieved.
*
*   Description: This function is invoked by the fpm_cm_mgr_get_fecaps()
*				 prior to getting the fecaps. This can also be used by any
*				 external module to just get the numner of ports on th FE
*
*   Return: None 
****************************************************************************/
void fpm_cm_mgr_get_port_num(char* name, FPPI_CBCORRELATOR cbCorrelator, 
							 char get_fecaps)
{
	cm_portnum_TranslatorContext_t *t_c = NULL;
	uint32_t *port_num = NULL; 
	char *fename = NULL; 

	npf_logger_write(LOG_FP_CM, LOG_TRACE, "Entering fpm_cm_mgr_get_port_num");
	
	t_c = (cm_portnum_TranslatorContext_t *)PIL_AllocateMemory
		(sizeof(cm_portnum_TranslatorContext_t), PIL_MEM_ZERO_INIT);

	port_num = (uint32_t *)PIL_AllocateMemory(sizeof(uint32_t), 
		PIL_MEM_ZERO_INIT);

	fename = (char *)PIL_AllocateMemory(sizeof(char) * (strlen(name) +1), 
		PIL_MEM_ZERO_INIT);

	if(!t_c || !fename || !port_num)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"Memory allocation failed");
		goto finish;
	}
	npf_logger_write(LOG_FP_CM, LOG_INFO, "Allocated t_c:0x%x", t_c);
	/*Build the respective translator context*/
	t_c->get_fecaps = get_fecaps;
	t_c->cbCorrelator = cbCorrelator;
	t_c->port_num = port_num;	
	t_c->fename = fename;
	memcpy(t_c->fename, name, strlen(name));
	
	/*Call into the platform specific component and wait for 
	callback to get number of ports*/
	cm_get_port_num(t_c);
	
	npf_logger_write(LOG_FP_CM, LOG_TRACE, "Leaving fpm_cm_mgr_get_port_num");
	
	return;
finish:
	if(t_c)	
		PIL_FreeMemory(t_c);
	if(fename)	
		PIL_FreeMemory(fename);
	if(port_num)	
		PIL_FreeMemory(port_num);
	
	bd_mgr_report_fecaps_cb(g_mybladeID, NULL, t_c->cbCorrelator);
}

/*****************************************************************************
*
*   Function: fpm_cm_mgr_set_L2_attrib
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data - npf_L2IntfAttrsList_t 
*
*   Description: This callback function is registered with the Back-end API.
*                The Back-end API calls this function to set the L2 attributes
*                like MTU, LinkSpeed, Status on the ports of the FE.
*
*   PRETURN : None 
*
*****************************************************************************/
void fpm_cm_mgr_set_L2_attrib(  FPPI_CBCORRELATOR cbcorrelator,            
                         FPPI_CONTEXT context,
                         void *response_data)
{
    npf_L2IntfAttrsList_t* l2attr;
	cm_TranslatorContext_t *t_c;

    npf_logger_write(LOG_FP_CM, LOG_TRACE, "Entering fpm_cm_mgr_set_L2_attrib");

	if(context != g_L2_setAttr_context)
        return;

	t_c = (cm_TranslatorContext_t *)PIL_AllocateMemory
		(sizeof(cm_TranslatorContext_t), PIL_MEM_ZERO_INIT);
	npf_logger_write(LOG_FP_CM, LOG_INFO, "Allocated t_c:0x%x", t_c);
	
	t_c->cbCorrelator = cbcorrelator;    

    l2attr = (npf_L2IntfAttrsList_t *)response_data;

	cm_set_L2_attrib(l2attr, t_c);

	npf_logger_write(LOG_FP_CM, LOG_TRACE, "Leaving fpm_cm_mgr_set_L2_attrib");
}

/*****************************************************************************
*
*   Function: fpm_cm_mgr_set_L3_attrib
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data - npf_L3IntfAttrsList_t 
*
*   Description: This callback function is registered with the Back-end API.
*                The Back-end API calls this function to set the L3 attributes
*                like IP addresses
*
*   PRETURN : None 
*
*****************************************************************************/
void fpm_cm_mgr_set_L3_attrib(  FPPI_CBCORRELATOR cbcorrelator,            
                         FPPI_CONTEXT context,
                         void *response_data)
{
    npf_L3IntfAttrsList_t* l3attr;
	cm_TranslatorContext_t *t_c;

	npf_logger_write(LOG_FP_CM, LOG_TRACE, "Entering fpm_cm_mgr_set_L3_attrib");

    if(context != g_L3_setAttr_context)
        return;

	t_c = (cm_TranslatorContext_t *)PIL_AllocateMemory
		(sizeof(cm_TranslatorContext_t), PIL_MEM_ZERO_INIT);
	npf_logger_write(LOG_FP_CM, LOG_INFO, "Allocated t_c:0x%x", t_c);
	
	t_c->cbCorrelator = cbcorrelator;    
	l3attr = (npf_L3IntfAttrsList_t *) response_data;    

	cm_set_L3_attrib(l3attr, t_c);

	/*TBD. Add routes for all the interfaces:
	1. Subnet Route
	2. Exact Match route
	3. Subnet directed Broadcast*/

	npf_logger_write(LOG_FP_CM, LOG_TRACE, "Leaving fpm_cm_mgr_set_L3_attrib");
}

/*****************************************************************************
*
*   Function: fpm_cm_mgr_delete_L3_attrib
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data - npf_L3IntfAttrsList_t 
*
*   Description: This callback function is registered with the Back-end API.
*                The Back-end API calls this function to delete the L3 
*				attributes like IP addresses
*
*   return: None 
*
*****************************************************************************/
void fpm_cm_mgr_delete_L3_attrib(  FPPI_CBCORRELATOR cbcorrelator,            
                         FPPI_CONTEXT context,
                         void *response_data)
{
    npf_L3IntfAttrsList_t* l3attr;
	cm_TranslatorContext_t *t_c;

	npf_logger_write(LOG_FP_CM, LOG_TRACE, "Entering fpm_cm_mgr_delete_L3_attrib");

    if(context != g_L3_delAttr_context)
        return;

	t_c = (cm_TranslatorContext_t *)PIL_AllocateMemory
		(sizeof(cm_TranslatorContext_t), PIL_MEM_ZERO_INIT);
	npf_logger_write(LOG_FP_CM, LOG_INFO, "Allocated t_c:0x%x", t_c);

	t_c->cbCorrelator = cbcorrelator;    
	l3attr = (npf_L3IntfAttrsList_t *) response_data;    

	cm_delete_L3_attrib(l3attr, t_c);

	npf_logger_write(LOG_FP_CM, LOG_TRACE, "Leaving fpm_cm_mgr_delete_L3_attrib");
}

/*****************************************************************************
*
*   Function:   fpm_cm_get_L3_statistics
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data - npf_PortList_t 
*
*   Description: This callback function is registered with the Back-end API.
*                The Back-end API calls this function to get L3 statistics
*
*   return: None 
*****************************************************************************/
void fpm_cm_mgr_get_L3_stats(FPPI_CBCORRELATOR cbCorrelator, 
			     FPPI_CONTEXT context, void *response_data)
{
	cm_L3_stats_TranslatorContext_t *t_c = NULL;
	FPPI_Interface_GetStats_t *portlist = NULL;
	npf_L3StatsEntry_t *stats_list= NULL;

	npf_logger_write(LOG_FP_CM, LOG_TRACE, "Entering fpm_cm_get_L3_statistics");
	
	if(context != g_L3_getStats_context)
        return;

        portlist = (FPPI_Interface_GetStats_t *)response_data;

	/*Allocate the necessary memory for building the translator context*/
	t_c = (cm_L3_stats_TranslatorContext_t *)PIL_AllocateMemory
		(sizeof(cm_L3_stats_TranslatorContext_t), PIL_MEM_ZERO_INIT);
	stats_list = (npf_L3StatsEntry_t *)PIL_AllocateMemory(
		sizeof(npf_L3StatsEntry_t) * portlist->port_count, PIL_MEM_ZERO_INIT);
	if(t_c == NULL || stats_list == NULL)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"Memory allocation failed");
		goto finish;
	}
	npf_logger_write(LOG_FP_CM, LOG_INFO, "Allocated t_c:0x%x", t_c);
	/*Build the respective translator context*/
	t_c->cbCorrelator = cbCorrelator;
	t_c->stats_list = stats_list;
	t_c->num_entries = portlist->port_count;

	/*Call into the platform specific component and wait for 
	callback to L3 stats*/
	cm_get_generic_stats(portlist, t_c);
	
	npf_logger_write(LOG_FP_CM, LOG_TRACE, "Leaving fpm_cm_get_L3_statistics");
	
	return;

finish:
	if(t_c)	
		PIL_FreeMemory(t_c);
	if(stats_list)	
		PIL_FreeMemory(stats_list);
	
	BENDAPI_Report_Status(FPPI_FAILURE, 
                        FPPI_L3_GetStatistics, 
                        t_c->cbCorrelator,
                        NULL);
}

/*****************************************************************************
*
*   Function:   fpm_cm_mgr_get_L2_stats
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data - npf_PortList_t 
*
*   Description: This callback function is registered with the Back-end API.
*                The Back-end API calls this function to get L2 statistics
*
*   return: None 
*****************************************************************************/
void fpm_cm_mgr_get_L2_stats(FPPI_CBCORRELATOR cbcorrelator,
			FPPI_CONTEXT context,
			void *response_data)
{

}

/*****************************************************************************
*
*   Function:   fpm_cm_mgr_get_icmp_stats
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data - npf_PortList_t 
*
*   Description: This callback function is registered with the Back-end API.
*                The Back-end API calls this function to get icmp statistics
*
*   return: None 
*****************************************************************************/
void fpm_cm_mgr_get_icmp_stats(FPPI_CBCORRELATOR cbcorrelator,
			FPPI_CONTEXT context,
			void *response_data)
{

}

/******************************************************************************
*******************************************************************************
*								Translator UP-CALLS							  *
*																			  * 
*******************************************************************************
******************************************************************************/

/*****************************************************************************
*
*   Function:   fpm_cm_mgr_get_port_num_upcall
*
*   Parameters: 
*				result: Result of the down call request
*				t_c: Translator context containing the num of ports 
*   Description:
*				This upcall function is called the the platform specific layer, 
*				to report the number of ports on this FE 
*
*   Return: 0 on success, -1 on failure 
*****************************************************************************/
int32_t fpm_cm_mgr_get_port_num_upcall(int32_t result,
									cm_portnum_TranslatorContext_t *t_c)
{	
	int32_t iRet = 0;
	FPPI_FE_Caps *fecaps;
	uint32_t numports; 
	cm_fecaps_TranslatorContext_t *fecaps_t_c = NULL;

	npf_logger_write(LOG_FP_CM, LOG_TRACE, 
		"Entering fpm_cm_mgr_get_port_num_upcall");

	if(!t_c)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"t_c is invalid");
		return -1;
	}
	if(result == -1)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, "NPU API returned error");
		iRet = -1;
		goto finish;
	}
	
	numports = *(t_c->port_num);

	if(t_c->get_fecaps) /*we need to get the fecaps*/
	{
		fecaps_t_c = PIL_AllocateMemory(sizeof(cm_fecaps_TranslatorContext_t), 
			PIL_MEM_ZERO_INIT);
		fecaps = PIL_AllocateMemory(sizeof(FPPI_FE_Caps), 
			PIL_MEM_ZERO_INIT);		
		fecaps->ports_caps.port_count = numports;
		fecaps->ports_caps.port_caps_list =
			(FPPI_Port_Caps *) PIL_AllocateMemory(
			numports * sizeof(FPPI_Port_Caps), PIL_MEM_ZERO_INIT);
		if (!fecaps_t_c || !fecaps ||
			!fecaps->ports_caps.port_caps_list
			) 
		{
			npf_logger_write(LOG_FP_CM, LOG_ERR, 
				"Memory Allocation failed!!\n");
			iRet = -1;
			goto finish;
		}	
		npf_logger_write(LOG_FP_CM, LOG_INFO, "Allocated t_c:0x%x", t_c);		
		/*build the fecaps translator context*/
		fecaps_t_c->cbCorrelator = t_c->cbCorrelator;
		fecaps_t_c->fecaps = fecaps;
		memcpy(fecaps_t_c->fecaps->fe_name, t_c->fename, strlen(t_c->fename));
		cm_get_fecaps(numports, fecaps_t_c);
	}
	else /*call the user's callback with number of ports*/
	{
		
	}

finish:
	/*free resources related to this t_c*/
	if(t_c)
	{
		if(t_c->port_num)
			PIL_FreeMemory(t_c->port_num);
		if(t_c->fename)
			PIL_FreeMemory(t_c->fename);
		npf_logger_write(LOG_FP_CM, LOG_INFO, "Freeing t_c:0x%x", t_c);
		PIL_FreeMemory(t_c);
	}	
	npf_logger_write(LOG_FP_CM, LOG_TRACE, 
		"Leaving fpm_cm_mgr_get_port_num_upcall");
	return iRet;
}

/*****************************************************************************
*
*   Function:   fpm_cm_mgr_get_fecaps_upcall
*
*   Parameters: 
*			result: Result of the down call request
*			t_c: Translator context containing FPPI_FE_Caps structure populated
*
*   Description:
*				This upcall function is called the the platform specific layer,
*				to report the capabilities for this FE 
*
*   Return: 0 on success, -1 on failure 
*****************************************************************************/
int32_t fpm_cm_mgr_get_fecaps_upcall(int32_t result, 
									  cm_fecaps_TranslatorContext_t *t_c)
{
	int32_t iRet = 0;

	if(!t_c)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"t_c is invalid");
		return -1;
	}
	if(result == -1)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, "NPU API returned error");
		iRet = -1;
		goto finish;
	}
	
	/*Call into BindingDiscovery Manager with the FE caps*/
	bd_mgr_report_fecaps_cb(g_mybladeID, t_c->fecaps, t_c->cbCorrelator);
finish:
	/*free resources related to this t_c*/	
	PIL_FreeMemory((t_c->fecaps->ports_caps.port_caps_list));
	PIL_FreeMemory((t_c->fecaps));
	npf_logger_write(LOG_FP_CM, LOG_INFO, "Freeing t_c:0x%x", t_c);
	PIL_FreeMemory(t_c);
	return 0;
}

/*****************************************************************************
*
*   Function:   fpm_cm_mgr_set_L2_attrib_upcall
*
*   Parameters: 
*			result: Result of the down call request
*			t_c:	Translator Context containing the cbcorrelator
*
*   Description:
*				This upcall function is called the the platform specific layer,
*				to report the status on setting L2 properties
*
*   Return: 0 on success, -1 on failure 
*****************************************************************************/
int32_t fpm_cm_mgr_set_L2_attrib_upcall(int32_t result, 
										 cm_TranslatorContext_t* t_c)
{
	if(!t_c)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"t_c is invalid");
		return -1;
	}
	if(result == -1)
		BENDAPI_Report_Status (FPPI_FAILURE, 
                              FPPI_L2_SetProperties, 
                              t_c->cbCorrelator, 
                              NULL);
	else
		BENDAPI_Report_Status (FPPI_SUCCESS, 
                              FPPI_L2_SetProperties, 
                              t_c->cbCorrelator, 
                              NULL);
	npf_logger_write(LOG_FP_CM, LOG_INFO, "Freeing t_c:0x%x", t_c);
	PIL_FreeMemory(t_c);
	return 0;
}

/*****************************************************************************
*
*   Function:   fpm_cm_mgr_set_L3_attrib_upcall
*
*   Parameters: 
*			result: Result of the down call request
*			t_c:	Translator Context containing the cbcorrelator
*
*   Description:
*				This upcall function is called the the platform specific layer,
*				to report the status on setting L3 attributes
*
*   Return: 0 on success, -1 on failure 
*****************************************************************************/
int32_t fpm_cm_mgr_set_L3_attrib_upcall(int32_t result, 
										 cm_TranslatorContext_t* t_c)
{
	if(!t_c)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"t_c is invalid");
		return -1;
	}
	if(result == -1)
		BENDAPI_Report_Status (FPPI_FAILURE, 
                              FPPI_L3_SetProperties, 
                              t_c->cbCorrelator, 
                              NULL);
	else
		BENDAPI_Report_Status (FPPI_SUCCESS, 
                              FPPI_L3_SetProperties, 
                              t_c->cbCorrelator, 
                              NULL);
	npf_logger_write(LOG_FP_CM, LOG_INFO, "Freeing t_c:0x%x", t_c);
	PIL_FreeMemory(t_c);
	return 0;
}

/*****************************************************************************
*
*   Function:   fpm_cm_mgr_delete_L3_attrib_upcall
*
*   Parameters: 
*			result: Result of the down call request
*			t_c:	Translator Context containing the cbcorrelator
*
*   Description:
*				This upcall function is called the the platform specific layer,
*				to report status on delete of the L3 attributes
*
*   Return: 0 on success, -1 on failure 
*****************************************************************************/
int32_t fpm_cm_mgr_delete_L3_attrib_upcall(int32_t result, 
										 cm_TranslatorContext_t* t_c)
{
	if(!t_c)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"t_c is invalid");
		return -1;
	}
	if(result == -1)
		BENDAPI_Report_Status (FPPI_FAILURE, 
                              FPPI_L3_SetProperties, 
                              t_c->cbCorrelator, 
                              NULL);
	else
		BENDAPI_Report_Status (FPPI_SUCCESS, 
                              FPPI_L3_SetProperties, 
                              t_c->cbCorrelator, 
                              NULL);
	npf_logger_write(LOG_FP_CM, LOG_INFO, "Freeing t_c:0x%x", t_c);
	PIL_FreeMemory(t_c);
	return 0;
}

/*****************************************************************************
*
*   Function:   fpm_cm_mgr_get_L3_stats_upcall
*
*   Parameters: 
*				result: Result of the down call request
*				t_c: Translator context containing the statistics information
*
*   Description:
*				This upcall function is called the the platform specific layer,
*				to report L3 statistics
*
*   Return: 0 on success, -1 on failure 
*****************************************************************************/
int32_t fpm_cm_mgr_get_L3_stats_upcall(int32_t result,
									cm_L3_stats_TranslatorContext_t *t_c)
{		
	npf_L3StatsList_t l3statList;

	npf_logger_write(LOG_FP_CM, LOG_TRACE, 
		"Entering fpm_cm_mgr_get_L3_stats_upcall");

	if(!t_c)
	{
		npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"t_c is invalid");
		return -1;
	}

	/*package stats info for the Backend API to send it up*/
	l3statList.port_count = t_c->num_entries;
	l3statList.stats_list = t_c->stats_list;

	if (BENDAPI_Report_Status( FPPI_SUCCESS, FPPI_L3_GetStatistics, 
		t_c->cbCorrelator,(void *)&l3statList ) != FPPI_SUCCESS)
	{		
		npf_logger_write(LOG_FP_CM, LOG_ERR, 
			"L3 Stats sending Report Failed \n");
	}
	
	/*free resources related to this t_c*/
	PIL_FreeMemory(t_c->stats_list);
	npf_logger_write(LOG_FP_CM, LOG_INFO, "Freeing t_c:0x%x", t_c);
	PIL_FreeMemory(t_c);

	npf_logger_write(LOG_FP_CM, LOG_TRACE, 
		"Leaving fpm_cm_mgr_get_L3_stats_upcall");
	return 0;
}

#if 0
/*****************************************************************************
*
*   Function: fpm_getICMPstats_cb()
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data 
*
*   Description: This  callback function is called by the Back-end API to 
*                get the ICMP statistics from the FE 
*
*   PRETURN : npf_ICMPStatsEntry_t 
*
*****************************************************************************/
void fpm_getICMPstats_cb (FPPI_CBCORRELATOR cbcorrelator,            
			  FPPI_CONTEXT context,
			  void *response_data)
{
    int i;
    int errtype;
    npf_ICMPStatsList_t icmpstatList;
    icmpstatistics icmpstatval;
    npf_PortList_t *portlist;


    PDBG_TRACE ();
    if(context != g_ICMP_getStats_context)
        PRETURN (/*void*/);
    portlist = (npf_PortList_t *)response_data;
    
    icmpstatList.port_count = portlist->port_count;
    icmpstatList.stats_list = 
	(npf_ICMPStatsEntry_t *)
	PIL_AllocateMemory(icmpstatList.port_count*sizeof(npf_ICMPStatsEntry_t), PIL_MEM_ZERO_INIT);
    for (i = 0; i < icmpstatList.port_count; i++)
    {
        icmpstatval = GetICMPStats(portlist->port_list[i]);  
        icmpstatList.stats_list[i].portid = portlist->port_list[i];
        icmpstatList.stats_list[i].icmpstats.icmpInMsgs = 0;
        icmpstatList.stats_list[i].icmpstats.icmpInErrors = 0;
        icmpstatList.stats_list[i].icmpstats.icmpInDestUnreach = 0;
        icmpstatList.stats_list[i].icmpstats.icmpInTimeExceeds = 0;
        icmpstatList.stats_list[i].icmpstats.icmpInParamProbs = 0;
        icmpstatList.stats_list[i].icmpstats.icmpInSrcQuenches = 0;
        icmpstatList.stats_list[i].icmpstats.icmpInRedirects = 0;
        icmpstatList.stats_list[i].icmpstats.icmpInEchos = 0;
        icmpstatList.stats_list[i].icmpstats.icmpInEchoReps = 0;
        icmpstatList.stats_list[i].icmpstats.icmpInTimeStamps = 0;
        icmpstatList.stats_list[i].icmpstats.icmpInTimeStampsReps = 0;
        icmpstatList.stats_list[i].icmpstats.icmpInAddrMasks = 0;
        icmpstatList.stats_list[i].icmpstats.icmpInAddrMaskReps = 0;

        icmpstatList.stats_list[i].icmpstats.icmpOutSrcQuenches = 0;
        icmpstatList.stats_list[i].icmpstats.icmpOutRedirects = 0;
        icmpstatList.stats_list[i].icmpstats.icmpOutEchos = 0;
        icmpstatList.stats_list[i].icmpstats.icmpOutEchoReps = 0;
        icmpstatList.stats_list[i].icmpstats.icmpOutTimeStamps = 0;
        icmpstatList.stats_list[i].icmpstats.icmpOutTimeStampsReps = 0;
        icmpstatList.stats_list[i].icmpstats.icmpOutAddrMasks = 0;
        icmpstatList.stats_list[i].icmpstats.icmpOutAddrMaskReps = 0;

        
        icmpstatList.stats_list[i].icmpstats.icmpOutMsgs = icmpstatval.msgs;
        icmpstatList.stats_list[i].icmpstats.icmpOutErrors = icmpstatval.errors;
        icmpstatList.stats_list[i].icmpstats.icmpOutTimeExceeds = 
	    icmpstatval.timeexcd;
        icmpstatList.stats_list[i].icmpstats.icmpOutParamProbs = 
	    icmpstatval.paramprobs;
    }
     errtype = BENDAPI_Report_Status( FPPI_SUCCESS, 
                                      FPPI_ICMP_GetStatistics, 
                                      cbcorrelator,
                                      (void *)&icmpstatList );
     if (errtype != FPPI_SUCCESS)
         PDBG_PRINT ("REPORT STATUS AT ICMP STATS FAILED \n");
     free (icmpstatList.stats_list);

     PRETURN (/*void*/);
}

/*****************************************************************************
*
*   Function: fpm_getL2stats_cb()
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data -NONE 
*
*   Description: This  callback function is called by the Back-end API to 
*                get the L2 statistics from the FE 
*
*   PRETURN : npf_L2StatsList_t * 
*
*****************************************************************************/
void fpm_getL2stats_cb (  FPPI_CBCORRELATOR cbcorrelator,            
                          FPPI_CONTEXT context,
                          void *response_data)
{
     npf_L2StatsList_t l2statList;
     int i;
     int errtype;
     l2statistics l2st;
     npf_PortList_t *portlist;

     PDBG_TRACE ();
    if(context != g_L2_getStats_context)
        PRETURN (/*void*/);
     portlist = (npf_PortList_t *)response_data;
     
     l2statList.port_count = portlist->port_count;
     l2statList.stats_list = 
	 (npf_L2StatsEntry_t *) 
	 PIL_AllocateMemory(g_totallinks * sizeof(npf_L2StatsEntry_t), PIL_MEM_ZERO_INIT);
     for(i = 0; i < l2statList.port_count; i++)
     {
         l2st = GetL2Stats(portlist->port_list[i]);    
         l2statList.stats_list[i].portid = portlist->port_list[i];
         l2statList.stats_list[i].l2stats.dot3StatsAlignmentErrors = 
	     l2st.AlignmentErrors;
         l2statList.stats_list[i].l2stats.dot3StatsFCSErrors = 
	     l2st.FramesCheckSequenceErrors;
         l2statList.stats_list[i].l2stats.dot3StatsSingleCollisionFrames = 
	     l2st.SingleCollisionFrames;
         l2statList.stats_list[i].l2stats.dot3StatsMultipleCollisionFrames = 
	     l2st.MultipleCollisionFrames;
         l2statList.stats_list[i].l2stats.dot3StatsSQETestErrors = 0;
         l2statList.stats_list[i].l2stats.dot3StatsDeferredTransmissions= 
	     l2st.FramesWithDeferredXmissions;
         l2statList.stats_list[i].l2stats.dot3StatsLateCollisions= 
	     l2st.LateCollisions;
         l2statList.stats_list[i].l2stats.dot3StatsExcessiveCollisions= 0;
         l2statList.stats_list[i].l2stats.dot3StatsInternalMacTransmitErrors= 0;
         l2statList.stats_list[i].l2stats.dot3StatsCarrierSenseErrors= 0;
         l2statList.stats_list[i].l2stats.dot3StatsFrameTooLongs = 
	     l2st.FrameTooLongErrors;
         l2statList.stats_list[i].l2stats.dot3StatsInternalMacReceiveErrors= 0;
         l2statList.stats_list[i].l2stats.dot3StatsSymbolErrors= 0;
         l2statList.stats_list[i].l2stats.dot3StatsDuplexStatus= 0;

     }
     PDBG_PRINT ("l2statList.port_count = %d\n", l2statList.port_count);
     PDBG_PRINT ("l2statList.stats_list[0].dot3StatsFCSErrors = %d\n",
         l2statList.stats_list[0].l2stats.dot3StatsFCSErrors);

     errtype = BENDAPI_Report_Status( FPPI_SUCCESS, 
                                      FPPI_L2_GetStatistics, 
                                      cbcorrelator,
                                      (void *)&l2statList );
     if (errtype != FPPI_SUCCESS)
         PDBG_PRINT ("Report Status for L2 Stats failed \n");
     free (l2statList.stats_list);

     PRETURN (/*void*/);
}
#endif

