/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/
#include "ix_cc.h"
#include "cc/ix_cc_fpm_core.h"
#include "FPModule.h"

static ix_cc_fpm_link_status_cb onCB_cc_link_status \
    (ix_uint32 bladeId, ix_uint32 portId, ix_cc_fpm_link_status linkState);
static ix_cc_fpm_ipv4_exception_cb onCB_cc_ipv4_exception \
    (ix_uint32 bladeId, ix_uint32 portId, ix_uint32 dest);

/* Register LinkStatus and Non-Directed-Connected-Host Event from
 * IXASDK 3.0 Core Component */
int event_InitFPEngine()
{
    ix_error err;

    err = ix_cc_fpm_register_link_status_cb ((void *)&onCB_cc_link_status);

    if (err != IX_SUCCESS) {
        PDBG_PRINT("Unable to register link status from IXASDK 3.0 CC\n");
        return(-1);
    }

    err = ix_cc_fpm_register_ipv4_exception_cb \
            ((void *)&onCB_cc_ipv4_exception);

    if (err != IX_SUCCESS) {
        PDBG_PRINT("Unable to register direct host from IXASDK 3.0 CC\n");
        return(-1);
    }
    
    return 0;
}

int event_DestroyFPEngine()
{
    ix_cc_fpm_deregister_link_status_cb();
    ix_cc_fpm_deregister_ipv4_exception_cb();

    return 0;
}

ix_cc_fpm_link_status_cb onCB_cc_link_status \
    (ix_uint32 bladeId, ix_uint32 portId, ix_cc_fpm_link_status linkState)
{
    int errtype;

    if (linkState) {
        PDBG_PRINT("Sending Link UP Event for Link %d \n", (int)portId);
        errtype = BENDAPI_sendEvent(FPPI_LinkUp_Event, &portId);
        if (FPPI_SUCCESS == errtype)
            PDBG_PRINT ("Sent Port Up Event \n");
    } else {
        PDBG_PRINT("Sending Link Down Event for Link %d \n", (int)portId);
        errtype = BENDAPI_sendEvent(FPPI_LinkDown_Event, &portId);
        if (FPPI_SUCCESS == errtype)
            PDBG_PRINT ("Sent Port Down Event \n");
    }
    return 0;
}

ix_cc_fpm_ipv4_exception_cb onCB_cc_ipv4_exception \
    (ix_uint32 bladeId, ix_uint32 portId, ix_uint32 dest)
{
    int errtype;
    npf_fpm_ipv4_exception_t wrapper;
    wrapper.portid = portId;
    wrapper.dest = dest;

    PDBG_PRINT("Sending Direct Connected Event for Link %d \n", (int)portId);
    errtype = BENDAPI_sendEvent(FPPI_Directly_Connected_Host_Event, &wrapper);
    if (FPPI_SUCCESS == errtype)
        PDBG_PRINT ("Sent Port Up Event \n");

    return 0;
}
