/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/

#ifndef _IPV4_MGR_SDK_H
#define _IPV4_MGR_SDK_H

#ifdef __cplusplus
extern "C" {
#endif

/****************************************************************************
*																			*
*						TRANSLATOR CONTEXT STRUCUTURES						*
*																			*			
*****************************************************************************/

/*****************************************************************************
*   Name: ipv4_TranslatorContext_t
*
*   Description:
*		Stores the translator's context related to ipv4 down-calls, like
*		adding/deleting prefixes/next hops/arps etc
*
*   Parameters:
*		overall_status:	0 is no single error, in the struct npf_IPv4_Status_t
*						pointed by status, 1 if there is any error.
*		status:			pointer to the npf_IPv4_Status_t struct
*		num_entries:	number of prefix entries 
*		cbType:			Type of the down-call 
*		cbCorrleator:	Higher level instance differentiator, opaque to the
*						FP module. Passed back to the FP Module's client 
*						(Backend API) as is.
*       
*
*/

typedef struct
{
	uint32_t overall_status;
	npf_IPv4_Status_t *status;
	uint32_t num_entries;
	uint32_t cbType;
	uint32_t cbCorrelator;
        NPF_IPv4UC_CallbackData_t cbData;
} ipv4_TranslatorContext_t;

/****************************************************************************
*																			*
*				PLATFORM SPECIFIC USER CONTEXT STRUCUTURES					*	
*																			*			
*****************************************************************************/

/*****************************************************************************
*   Name: ipv4_UserContext_t
*
*   Description:
*		Stores the user context related to add/del prefixes/nexthops etc.
*
*   Parameters:
*				reqComp_Indctr: Pointer to counter to keep track of translator 
*								request completion
*				t_c:			Translator context.
*				index:			offset that ensures the location of the prefix
*								or nexthop etc to be in the same order as the 
*								TP passes in.
*		
*       
*
*/

typedef struct
{
	uint32_t *reqComp_Indctr;
	ipv4_TranslatorContext_t *t_c;
	PilSemaphore lock;
	uint32_t index;
} ipv4_UserContext_t;

/********************************************************************
*					TRANSLATOR UP - CALLS							*
*********************************************************************/

int32_t fpm_ipv4_mgr_upcall(int32_t result, 
									ipv4_TranslatorContext_t *t_c);

/********************************************************************
*					PLATFORM SPECIFIC APIS							*
********************************************************************/
void ipv4_add_prefix(
					 npf_ipv4_UnicastPrefixUpdateInfoList_t *prefixes, 
					 ipv4_TranslatorContext_t *t_c);
void ipv4_query_prefix(
					 npf_ipv4_UnicastPrefixList_t *prefixes, 
					 ipv4_TranslatorContext_t *t_c);
void ipv4_delete_prefix(
					 npf_ipv4_UnicastPrefixList_t *prefixes, 
					 ipv4_TranslatorContext_t *t_c);
void ipv4_update_prefix(
					 npf_ipv4_UnicastPrefixList_t *prefixes, 
					 ipv4_TranslatorContext_t *t_c);
void ipv4_flush_prefixes(ipv4_TranslatorContext_t* t_c);
void ipv4_add_next_hop(
					 npf_ipv4_UnicastNextHopUpdateInfoList_t *nexthops, 
					 ipv4_TranslatorContext_t *t_c);
void ipv4_query_next_hop(
					 npf_ipv4_UnicastNextHopDeleteInfoList_t *nexthops, 
					 ipv4_TranslatorContext_t *t_c);
void ipv4_delete_next_hop(
					 npf_ipv4_UnicastNextHopDeleteInfoList_t *nexthops, 
					 ipv4_TranslatorContext_t *t_c);
void ipv4_flush_next_hops(
                    npf_ipv4_UnicastNextHopDeleteInfoList_t *nexthops,
						  ipv4_TranslatorContext_t *t_c);

void ipv4_add_arp(
				  npf_ipv4_UnicastToL2List_t *arpentry, 
				  ipv4_TranslatorContext_t *t_c);
void ipv4_delete_arp(
					 npf_ipv4_UnicastToL2List_t *arpentry,
					 ipv4_TranslatorContext_t *t_c);
void ipv4_flush_arps(ipv4_TranslatorContext_t *t_c);

/*****************************************************************************
*   Name: ipv4_uc_info_t
*
*   Description:
*		Stores the pending user context info 
*
*   Parameters:
*				type:		Type of u_c
*				timestamp:	Holds time of actual IXASDK3.0 invocation 
*							(currently not used)
*				u_c:		Pointer to the user context
*		
*       
*
*/

typedef struct {
	uint32_t type;
	uint32_t timestamp;
	void * u_c;
} ipv4_uc_info;

int ipv4_InitFPEngine();
int ipv4_DestroyFPEngine();
void ipv4_free_resources();
uint32_t ipv4_is_Valid(void *u_c);

#ifdef __cplusplus
}
#endif

#endif /*_IPV4_MGR_SDK_H */

