/*****************************************************************************
 *                  INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *  Copyright (c) 2002 Intel Corporation. All Rights Reserved.
 *
 *	Description:
 *	This file implements the IXASD30 specifc component of the IPv4 manager, 
 *****************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "ix_error.h"
#include "cc/ix_cc_rtmv4.h"
#include "cc/ix_cc_ipv4.h"
#include "cc/ix_cc_fpm_core.h"
#include "ix_cc_ipv4_init.h"
#include "ix_cc_error.h"
#ifdef _INCLUDE_ATM
#include "cc/ix_cc_atmsar.h"
#include "cc/ix_cc_atmrx_cci.h"
#include "cc/llc_snap_decap.h"
#endif

#include "pil.h"
#include "FPModule.h"
#include "ipv4_mgr_impl.h"
#include "fp_common.h"
#include "log.h"


#ifdef _VXWORKS_
#include "routeLib.h"
#endif


#ifdef _INCLUDE_ATM
extern NPF_RET fpmIfATM_GetVcHandle(NPF_VccAddr_t, NPF_uint32_t,ix_cc_atmsar_vc_handle_t*, NPF_IfAAL_t *aalType );
#endif 
extern router_config g_current_router_config;


/*QoS call to get best effort flow tuple*/
extern int dpe_get_besteffort_flow_tuple (uint32_t* flowid, uint32_t* queueid);
extern uint32_t mask_to_prefixLength(unsigned int mask);
/*static uint32_t addRouteToLinuxStack(uint32_t addr, uint32_t mask, 
  uint32_t nhAddr, uint32_t interface);
  static uint32_t delRouteToLinuxStack(uint32_t addr, uint32_t mask);
*/
static int bits_to_mask( int prefixLength);
static uint32_t convert_to_npf(ix_error error);
static void ipv4_display_cb_type(uint32_t cbType);

static void ipv4_sync_routes (uint32_t prefix, uint32_t mask);
static ix_error ipv4_lookup_route_cb (ix_error arg_Result,
				      void* arg_pContext,
				      ix_cc_rtmv4_next_hop_info* arg_pNHInfo);
void fpm_dump_arp_info (ix_cc_eth_tx_next_hop_info *arp_info);
void fpm_dump_l2_info (uint32_t l2_id, uint32_t port_id,
		       ix_cc_l2tm_entry* l2_info);

static	DList   g_ipv4_pendinglist;
extern uint32_t g_mybladeID;

typedef struct _s_fpm_ipv4_ctx {
  uint32_t prefix;
  uint32_t mask;
} fpm_ipv4_context;

/*****************************************************************************
 *
 *						Callback prototypes
 *
 *****************************************************************************/

ix_error ipv4_cb(ix_error arg_Result,void* arg_pContext);
ix_error l2_op_cb(ix_error arg_Result,void* arg_pContext);

#ifdef _INCLUDE_ATM

/*****************************************************************************
 *
 *						ATM Specific Callback routines 
 *
 *****************************************************************************/

ix_error LimePointUpdateCBFunc(ix_error arg_Result,
		ix_cc_atmsar_vc_handle_t arg_hVc,
		void *arg_pContext)
{
  FPPI_UnicastNextHopUpdateInfo_t *nhUpdCntxt = NULL;
  ix_error err = IX_SUCCESS;

  if( (!arg_pContext) )
  {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR,
                      "Callback error ; Invalid pContext");
      err = IX_ERROR_NEW(IX_CC_ERROR_NULL, IX_ERROR_LEVEL_WARNING);
      return err;
    }
   
   if( (arg_Result != IX_SUCCESS))
    {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR,
              "Callback error ;Arg REsult not success");
      err = IX_ERROR_NEW(IX_CC_ERROR_NULL, IX_ERROR_LEVEL_WARNING);
      ix_error_dump(stdout,arg_Result);
      return err;
    }
    /* Get the context here */
    nhUpdCntxt = (FPPI_UnicastNextHopUpdateInfo_t*)arg_pContext;
    /* We are not doing anything in this function as of now. */
    /* Free the context */
    PIL_FreeMemory(nhUpdCntxt);
	 
    return IX_SUCCESS;
}
/* This is a callback for adding the entry in LLC-Snap Table */
ix_error LimePointLlcSnapAddCBFunc(ix_error arg_Result,
		void *arg_pContext)
{
  FPPI_UnicastNextHopUpdateInfo_t *nhUpdCntxt = NULL;
  ix_cc_atmsar_update_data_t *vcUpdateData = NULL;
  ix_cc_atmrx_update_data_t tempVcUpdateData;
  NPF_VccAddr_t vccAddr;    
  NPF_uint32_t portId =0;
  ix_cc_atmsar_vc_handle_t vcHandle = 0;
  ix_error err = IX_SUCCESS;
  NPF_IfAAL_t   aaltype;
 
  npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Entering  LimePointLlcSnapAddCBFunc");
  
  if( (!arg_pContext) )
  {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, "Callback error ; Invalid pContext");
      err = IX_ERROR_NEW(IX_CC_ERROR_NULL, IX_ERROR_LEVEL_WARNING);
      return err;
    }
   
   if( (arg_Result != IX_SUCCESS))
    {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, "Callback error ;Arg REsult not success");
      err = IX_ERROR_NEW(IX_CC_ERROR_NULL, IX_ERROR_LEVEL_WARNING);
     ix_error_dump(stdout,arg_Result);
      return err;
    }
    nhUpdCntxt = (FPPI_UnicastNextHopUpdateInfo_t*)arg_pContext;
    vccAddr.VPI = nhUpdCntxt->nhInfo->mediaAddr.u.ATM_Vc.VPI;
    vccAddr.VCI = nhUpdCntxt->nhInfo->mediaAddr.u.ATM_Vc.VCI;
    portId = nhUpdCntxt->nhInfo->portId; 

    fpmIfATM_GetVcHandle(vccAddr,portId,&vcHandle,&aaltype);
    tempVcUpdateData.lw0.l2port = vcHandle;
    switch(aaltype)
    {
	case NPF_IF_VCC_AAL5_LLC:
        {
           tempVcUpdateData.lw1.hdrType = PKT_PROTO_LLC_SNAP;
        }
	break;
	case NPF_IF_VCC_AAL5_VCMUX:
	{
           tempVcUpdateData.lw1.hdrType = PKT_PROTO_IPv4;
	}
	break;
	default:
	{
           tempVcUpdateData.lw1.hdrType = PKT_PROTO_NULL;
	   break;
	}
    }
    vcUpdateData = (ix_cc_atmsar_update_data_t *)&tempVcUpdateData;
	
    /* Context will be freed in the Callback of this function */ 
    ix_cc_fpm_async_vc_update(0,
                              vcHandle,
                              vcUpdateData,
                              LimePointUpdateCBFunc,
                              nhUpdCntxt);
    return IX_SUCCESS;
}

/* This is a callback for deleting the entry in LLC-Snap Table */
ix_error LimePointLlcSnapDeleteCBFunc(ix_error arg_Result,
		void *arg_pContext)
{
  ix_error err = IX_SUCCESS;
 
  npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Entering  LimePointLlcSnapDeleteCBFunc");
  
  if( (!arg_pContext) )
  {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, "Callback error ; Invalid pContext");
      err = IX_ERROR_NEW(IX_CC_ERROR_NULL, IX_ERROR_LEVEL_WARNING);
      return err;
    }
   
   if( (arg_Result != IX_SUCCESS))
    {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, "Callback error ;Arg REsult not success");
      err = IX_ERROR_NEW(IX_CC_ERROR_NULL, IX_ERROR_LEVEL_WARNING);
     ix_error_dump(stdout,arg_Result);
      return err;
    }
    return IX_SUCCESS;
}

#endif /* _INCLUDE_ATM */



/*****************************************************************************
 *
 *   Function:   ipv4_InitFPEngine()
 *
 *   Parameters: None 
 *
 *   Description: This function starts up the ipv4 platform specific component. 
 *
 *   Return: 0 on success
 *****************************************************************************/
int ipv4_InitFPEngine()
{
    npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Entering ipv4_InitFPEngine");
    /*initialize a linked list to hold the Fid/Qid*/
    npf_list_init(&g_ipv4_pendinglist, PIL_FreeMemory);
    return 0;
}

/*****************************************************************************
 *
 *   Function:   ipv4_DestroyFPEngine()
 *
 *   Parameters: None 
 *
 *   Description: This function shuts down the ipv4 platform specific component. 
 *
 *   Return: 0 on success
 *****************************************************************************/
int ipv4_DestroyFPEngine()
{
    /*free resources for any pending u_c*/
    ipv4_free_resources();
    
    /*Destroy List*/
    npf_list_destroy(&g_ipv4_pendinglist);
    return 0;
}

/*****************************************************************************
 *
 *
 *   Function:   ipv4_add_prefix()
 *
 *   Parameters: 
 *      prefixes: Pointer to prefix list
 *	t_c: Translator context, the embedded status struct to be 
 *						 filled once IXASDK3.0 responds.
 *					
 *
 *   Description: 
 *				This function requests the IXASDK adding prefixes.
 *				It provides a callback function to the IXASDK to call back with 
 *				the status on adding prefixes.           
 *
 *   Return: none
 *
 *****************************************************************************/
void ipv4_add_prefix(npf_ipv4_UnicastPrefixUpdateInfoList_t *prefixes, 
		     ipv4_TranslatorContext_t *t_c)
{
   int32_t iRet = 0;
   uint32_t i = 0;
   uint32_t success_counter = 0;
   ipv4_uc_info *uc_info = NULL;
   ipv4_UserContext_t *u_c = NULL;
   uint32_t *reqComp_Indctr = NULL;
   ix_error err;
   PilSemaphore lock;

   npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Entering  ipv4_add_prefix");

   lock = PIL_CreateBinarySemaphore(TRUE);

   reqComp_Indctr = (uint32_t *)
      PIL_AllocateMemory(sizeof(uint32_t), PIL_MEM_ZERO_INIT);
   if(!reqComp_Indctr)
   {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
            "Mem allocation for reqComp_Indctr failed");
      iRet = -1;
      goto finish;
   }

   /*set the request completion indicator counter*/
   *reqComp_Indctr = prefixes->numEntries;

   for (i = 0; i < prefixes->numEntries; i++)
   {
      u_c = (ipv4_UserContext_t *)
         PIL_AllocateMemory (sizeof(ipv4_UserContext_t), PIL_MEM_ZERO_INIT);
      if(!u_c) {
         npf_logger_write(LOG_FP_IPV4, LOG_ERR, "Mem allocation for u_c failed");
         iRet = -1;
         PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
         *reqComp_Indctr = success_counter;
         PIL_ReleaseSemaphore(lock);
         t_c->status[i].retStatus = NPF_E_UNKNOWN;
         t_c->overall_status = NPF_E_UNKNOWN;
         goto finish;
      }	
      npf_logger_write(LOG_FP_IPV4, LOG_INFO, "Allocated u_c:0x%x", u_c);
      u_c->reqComp_Indctr = reqComp_Indctr;
      u_c->t_c = t_c;
      u_c->lock = lock;
      /*set the index to 'i' aligning with the same offset as the addr
        when the callback comes, we use index i to set the error code
        for the respective down call addr*/
      u_c->index = i; 

      uc_info = (ipv4_uc_info *)
         PIL_AllocateMemory (sizeof(ipv4_uc_info ), PIL_MEM_ZERO_INIT);
      if(!uc_info) {
         npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
               "Mem allocation failed for uc_info");
         iRet = -1;
         PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
         *reqComp_Indctr = success_counter;
         PIL_ReleaseSemaphore(lock);
         t_c->status[i].retStatus = NPF_E_UNKNOWN;
         t_c->overall_status = NPF_E_UNKNOWN;
         goto finish;
      }
      uc_info->type = FPPI_IPv4_UniAddPrefix;
      uc_info->u_c = (void *)u_c;
      /*insert this u_c info into the u_c pending list*/
      npf_list_pushBack(&g_ipv4_pendinglist, uc_info);

      /*Call into the IXASDK*/
      err = ix_cc_ipv4_async_add_route(
            prefixes->prefixInfo[i].prefixEntry.IPv4Addr,
            bits_to_mask(prefixes->prefixInfo[i].prefixEntry.IPv4NetPlen), 
            prefixes->prefixInfo[i].nextHopId, ipv4_cb,
            (void *)u_c);

      if(err != IX_SUCCESS) {
         npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
               "IXASDK30 API call failed for adding a route error code:%d",
               IX_ERROR_GET_CODE(err));
         ix_error_dump(stdout, err);			
         PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
         *reqComp_Indctr = success_counter;
         PIL_ReleaseSemaphore(lock);
         t_c->status[i].retStatus = 
            convert_to_npf(IX_ERROR_GET_CODE(err));
         t_c->overall_status = NPF_E_UNKNOWN;
         iRet = -1;
         goto finish;
      }
      success_counter++;
      /* now we need to synchronize this route with the kernel route table */
      ipv4_sync_routes (prefixes->prefixInfo[i].prefixEntry.IPv4Addr,
            bits_to_mask (prefixes->prefixInfo[i].prefixEntry.IPv4NetPlen));
   }

finish:
   if(iRet == -1)
   {
      /*Note that any unfreed u_c's and the related resources they point to 
        will be anyhow freed during shut down*/
      if(ipv4_is_Valid(u_c))
         PIL_FreeMemory(u_c);

      if(!reqComp_Indctr || success_counter == 0)
      {
         /*inform the Translator component about failure via the up-call*/
         fpm_ipv4_mgr_upcall(iRet, t_c);
         if(reqComp_Indctr)
            PIL_FreeMemory(reqComp_Indctr);
      }
   }
   npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Leaving  ipv4_add_prefix");
}

/*****************************************************************************
*
*
*   Function:   ipv4_query_prefix()
*
*   Parameters: 
*               prefixes:Pointer to prefix list
*				t_c:	 Translator context, the embedded status struct to be 
*						 filled once IXASDK3.0 responds.
*   Description: 
*				This function requests the IXASDK for prefix information.
*				It provides a callback function to the IXASDK to call back with 
*				the status.
*
*   Return: none
*
*****************************************************************************/
void ipv4_query_prefix(npf_ipv4_UnicastPrefixList_t *prefixes, 
			ipv4_TranslatorContext_t *t_c)
{
   int32_t iRet = 0;
   uint32_t i = 0;
   uint32_t netMask = 0;
   ipv4_uc_info *uc_info = NULL;
   ipv4_UserContext_t *u_c = NULL;
   uint32_t *reqComp_Indctr = NULL;
   ix_error err;
   uint32_t success_counter = 0;
   ix_cc_rtmv4_nhid nhid;
   PilSemaphore lock;

   npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Entering  ipv4_query_prefix");	

   lock = PIL_CreateBinarySemaphore(TRUE);

   reqComp_Indctr = (uint32_t *)
      PIL_AllocateMemory (sizeof(uint32_t), PIL_MEM_ZERO_INIT);
   if(!reqComp_Indctr)
   {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
            "Mem allocation for reqComp_Indctr failed");
      iRet = -1;
      goto finish;
   }

   /*set the request completion indicator counter*/
   *reqComp_Indctr = prefixes->numEntries;

   
   for (i = 0; i < prefixes->numEntries; i++) 
   {
      u_c = (ipv4_UserContext_t *)
         PIL_AllocateMemory (sizeof(ipv4_UserContext_t), PIL_MEM_ZERO_INIT);
      if(!u_c)
      {
         npf_logger_write(LOG_FP_IPV4, LOG_ERR, "Mem allocation for u_c failed");
         iRet = -1;
         PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
         *reqComp_Indctr = success_counter;
         PIL_ReleaseSemaphore(lock);
         t_c->status[i].retStatus = NPF_E_UNKNOWN;
         t_c->overall_status = NPF_E_UNKNOWN;
         goto finish;
      }
      npf_logger_write(LOG_FP_IPV4, LOG_INFO, "Allocated u_c:0x%x", u_c);
      u_c->reqComp_Indctr = reqComp_Indctr;
      u_c->t_c = t_c;
      u_c->lock = lock;
      /*set the index to 'i' aligning with the same offset as the addr
        when the callback comes, we use index i to set the error code
        for the respective down call addr*/
      u_c->index = i; 
      
       /* TBD: In Array Handling, change this;Allocate response*/
       u_c->t_c->cbData.resp = (NPF_IPv4UC_AsyncResponse_t *)PIL_AllocateMemory(sizeof(NPF_IPv4UC_AsyncResponse_t)*prefixes->numEntries, PIL_MEM_ZERO_INIT);
      uc_info = (ipv4_uc_info *)
         PIL_AllocateMemory(sizeof(ipv4_uc_info ), PIL_MEM_ZERO_INIT);
      if(!uc_info)
      {
         npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
               "Mem allocation failed for uc_info");
         iRet = -1;
         PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
         *reqComp_Indctr = success_counter;
         PIL_ReleaseSemaphore(lock);
         t_c->status[i].retStatus = NPF_E_UNKNOWN;
         t_c->overall_status = NPF_E_UNKNOWN;
         u_c->t_c->cbData.allOK = NPF_FALSE;
         u_c->t_c->cbData.numResp = 1;
         u_c->t_c->cbData.type = NPF_IPV4UC_PREFIX_ENTRY_QUERY;
         u_c->t_c->cbData.resp[i].returnCode = NPF_FAIL;
         u_c->t_c->cbData.resp[i].u.prefixQueryResult.nextHopIdentifier = 0;
         goto finish;
      }
      uc_info->type = FPPI_IPv4_UniQueryPrefix;
      uc_info->u_c = (void *)u_c;
      /*insert this u_c info into the u_c pending list*/
      npf_list_pushBack(&g_ipv4_pendinglist, uc_info);	

      /* Make an SDK call here */
      netMask = 0xffffffff << (32 - prefixes->prfxInfo[i].IPv4NetPlen);
      err = ix_cc_rtmv4_get_route(g_pIpv4Context->hRtm, 
                       prefixes->prfxInfo[i].IPv4Addr, netMask, &nhid);
      if(err != IX_SUCCESS)
      {
         npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
               "IXASDK30 API call failed for query a route;  error code:%d",
         IX_ERROR_GET_CODE(err));
         ix_error_dump(stdout, err);			
         PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
         *reqComp_Indctr = success_counter;
         PIL_ReleaseSemaphore(lock);						
         t_c->status[i].retStatus = 
            convert_to_npf(IX_ERROR_GET_CODE(err));
         t_c->overall_status = NPF_E_UNKNOWN;
         iRet = -1;
         u_c->t_c->cbData.allOK = NPF_FALSE;
         u_c->t_c->cbData.numResp = 1;
         u_c->t_c->cbData.type = NPF_IPV4UC_PREFIX_ENTRY_QUERY;
         u_c->t_c->cbData.resp[i].returnCode = NPF_FAIL;
         u_c->t_c->cbData.resp[i].u.prefixQueryResult.nextHopIdentifier = 0;

         goto finish;
      }	
   
      u_c->t_c->cbData.resp[i].returnCode = NPF_SUCCESS;
      u_c->t_c->cbData.resp[i].u.prefixQueryResult.nextHopIdentifier = nhid;
      success_counter++;
   }
      u_c->t_c->cbData.allOK = NPF_TRUE;
      u_c->t_c->cbData.numResp = prefixes->numEntries;
      u_c->t_c->cbData.type = NPF_IPV4UC_PREFIX_ENTRY_QUERY;
      /* TODO: Change the following function call */
      ipv4_cb(IX_SUCCESS, (void *)u_c);


finish:
   if(iRet == -1)
   {
      /*Note that any unfreed u_c's and the related resources they point to 
        will be anyhow freed during shut down*/
      if(ipv4_is_Valid(u_c))
         PIL_FreeMemory(u_c);

      if(!reqComp_Indctr || success_counter == 0)
      {
         /*inform the Translator component about failure via the up-call*/
         fpm_ipv4_mgr_upcall(iRet, t_c);
         if(reqComp_Indctr)
            PIL_FreeMemory(reqComp_Indctr);
      }
   }

   npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Leaving  ipv4_query_prefix");
}

/*****************************************************************************
*
*
*   Function:   ipv4_delete_prefix()
*
*   Parameters: 
*               prefixes:Pointer to prefix list
*				t_c:	 Translator context, the embedded status struct to be 
*						 filled once IXASDK3.0 responds.
*   Description: 
*				This function requests the IXASDK deleting prefixes.
*				It provides a callback function to the IXASDK to call back with 
*				the status on deleting prefixes.           
*
*   Return: none
*
*****************************************************************************/
void ipv4_delete_prefix(npf_ipv4_UnicastPrefixList_t *prefixes, 
			ipv4_TranslatorContext_t *t_c)
{
   int32_t iRet = 0;
   uint32_t i = 0;
   ipv4_uc_info *uc_info = NULL;
   ipv4_UserContext_t *u_c = NULL;
   uint32_t *reqComp_Indctr = NULL;
   ix_error err;
   uint32_t success_counter = 0;
   PilSemaphore lock;

   npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Entering  ipv4_delete_prefix");	

   lock = PIL_CreateBinarySemaphore(TRUE);

   reqComp_Indctr = (uint32_t *)
      PIL_AllocateMemory (sizeof(uint32_t), PIL_MEM_ZERO_INIT);
   if(!reqComp_Indctr)
   {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
            "Mem allocation for reqComp_Indctr failed");
      iRet = -1;
      goto finish;
   }

   /*set the request completion indicator counter*/
   *reqComp_Indctr = prefixes->numEntries;

   for (i = 0; i < prefixes->numEntries; i++) {
      u_c = (ipv4_UserContext_t *)
         PIL_AllocateMemory (sizeof(ipv4_UserContext_t), PIL_MEM_ZERO_INIT);
      if(!u_c)
      {
         npf_logger_write(LOG_FP_IPV4, LOG_ERR, "Mem allocation for u_c failed");
         iRet = -1;
         PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
         *reqComp_Indctr = success_counter;
         PIL_ReleaseSemaphore(lock);
         t_c->status[i].retStatus = NPF_E_UNKNOWN;
         t_c->overall_status = NPF_E_UNKNOWN;
         goto finish;
      }
      npf_logger_write(LOG_FP_IPV4, LOG_INFO, "Allocated u_c:0x%x", u_c);
      u_c->reqComp_Indctr = reqComp_Indctr;
      u_c->t_c = t_c;
      u_c->lock = lock;
      /*set the index to 'i' aligning with the same offset as the addr
        when the callback comes, we use index i to set the error code
        for the respective down call addr*/
      u_c->index = i; 

      uc_info = (ipv4_uc_info *)
         PIL_AllocateMemory(sizeof(ipv4_uc_info ), PIL_MEM_ZERO_INIT);
      if(!uc_info)
      {
         npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
               "Mem allocation failed for uc_info");
         iRet = -1;
         PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
         *reqComp_Indctr = success_counter;
         PIL_ReleaseSemaphore(lock);
         t_c->status[i].retStatus = NPF_E_UNKNOWN;
         t_c->overall_status = NPF_E_UNKNOWN;
         goto finish;
      }
      uc_info->type = FPPI_IPv4_UniDelPrefix;
      uc_info->u_c = (void *)u_c;
      /*insert this u_c info into the u_c pending list*/
      npf_list_pushBack(&g_ipv4_pendinglist, uc_info);	

      /*Call into the IXASDK*/
      err = ix_cc_ipv4_async_delete_route(
            prefixes->prfxInfo[i].IPv4Addr,
            bits_to_mask(prefixes->prfxInfo[i].IPv4NetPlen), 
            ipv4_cb, (void *)u_c);

      if(err != IX_SUCCESS)
      {
         npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
               "IXASDK30 API call failed for deleting a route error code:%d",
               IX_ERROR_GET_CODE(err));
         ix_error_dump(stdout, err);			
         PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
         *reqComp_Indctr = success_counter;
         PIL_ReleaseSemaphore(lock);						
         t_c->status[i].retStatus = 
            convert_to_npf(IX_ERROR_GET_CODE(err));
         t_c->overall_status = NPF_E_UNKNOWN;
         iRet = -1;
         goto finish;
      }	
      success_counter++;
   }

finish:
   if(iRet == -1)
   {
      /*Note that any unfreed u_c's and the related resources they point to 
        will be anyhow freed during shut down*/
      if(ipv4_is_Valid(u_c))
         PIL_FreeMemory(u_c);

      if(!reqComp_Indctr || success_counter == 0)
      {
         /*inform the Translator component about failure via the up-call*/
         fpm_ipv4_mgr_upcall(iRet, t_c);
         if(reqComp_Indctr)
            PIL_FreeMemory(reqComp_Indctr);
      }
   }

   npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Leaving  ipv4_delete_prefix");
}

/*****************************************************************************
*
*
*   Function:   ipv4_update_prefix()
*
*   Parameters: 
*               prefixes:Pointer to prefix list
*				t_c:	 Translator context, the embedded status struct to be 
*						 filled once IXASDK3.0 responds.
*
*   Description: 
*				This function requests the IXASDK updating prefixes.
*				It provides a callback function to the IXASDK to call back with 
*				the status on updating prefixes.           
*
*   Return: none
*
*****************************************************************************/
void ipv4_update_prefix(npf_ipv4_UnicastPrefixList_t *prefixes, 
					 ipv4_TranslatorContext_t *t_c)
{
	int32_t iRet = 0;
	uint32_t i = 0;
	ipv4_uc_info *uc_info = NULL;
	ipv4_UserContext_t *u_c = NULL;
	uint32_t *reqComp_Indctr = NULL;
	ix_error err;
	uint32_t success_counter = 0;
	PilSemaphore lock;

	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Entering ipv4_update_prefix");
	
	lock = PIL_CreateBinarySemaphore(TRUE);
	reqComp_Indctr = (uint32_t *)PIL_AllocateMemory(
		sizeof(uint32_t), PIL_MEM_ZERO_INIT);
	if(!reqComp_Indctr)
	{
		npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"Mem allocation for reqComp_Indctr failed");
		iRet = -1;
		goto finish;
	}
	
	/*set the request completion indicator counter*/
	*reqComp_Indctr = prefixes->numEntries;

	for (i = 0; i < prefixes->numEntries; i++)
	{
		u_c = (ipv4_UserContext_t *)PIL_AllocateMemory(
		sizeof(ipv4_UserContext_t),PIL_MEM_ZERO_INIT);
		if(!u_c)
		{
			npf_logger_write(LOG_FP_IPV4, LOG_ERR, "Mem allocation for u_c failed");
			iRet = -1;
			PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
			*reqComp_Indctr = success_counter;
			PIL_ReleaseSemaphore(lock);
			t_c->status[i].retStatus = NPF_E_UNKNOWN;
			t_c->overall_status = NPF_E_UNKNOWN;
			goto finish;
		}
		npf_logger_write(LOG_FP_IPV4, LOG_INFO, "Allocated u_c:0x%x", u_c);
		u_c->reqComp_Indctr = reqComp_Indctr;
		u_c->t_c = t_c;
		u_c->lock = lock;
		/*set the index to 'i' aligning with the same offset as the addr
		when the callback comes, we use index i to set the error code
		for the respective down call addr*/
		u_c->index = i; 
		
		uc_info = (ipv4_uc_info *)PIL_AllocateMemory(
		sizeof(ipv4_uc_info ), PIL_MEM_ZERO_INIT);
		if(!uc_info)
		{
			npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
				"Mem allocation failed for uc_info");
			iRet = -1;
			PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
			*reqComp_Indctr = success_counter;
			PIL_ReleaseSemaphore(lock);
			t_c->status[i].retStatus = NPF_E_UNKNOWN;
			t_c->overall_status = NPF_E_UNKNOWN;
			goto finish;
		}
		uc_info->type = FPPI_IPv4_UniUpdatePrefix;
		uc_info->u_c = (void *)u_c;
		/*insert this u_c info into the u_c pending list*/
		npf_list_pushBack(&g_ipv4_pendinglist, uc_info);

		/*Call into the IXASDK*/
		err = ix_cc_ipv4_async_update_route(
			prefixes->prfxInfo[i].IPv4Addr,
			bits_to_mask(prefixes->prfxInfo[i].IPv4NetPlen), 
			0,
			ipv4_cb, (void *)u_c);
		if(err != IX_SUCCESS)
		{
			npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
				"IXASDK30 API call failed for updating route error code:%d",
				IX_ERROR_GET_CODE(err));
			ix_error_dump(stdout, err);			
			PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
			*reqComp_Indctr = success_counter;
			PIL_ReleaseSemaphore(lock);						
			t_c->status[i].retStatus = 
				convert_to_npf(IX_ERROR_GET_CODE(err));
			t_c->overall_status = NPF_E_UNKNOWN;
			iRet = -1;
			goto finish;
		}
		success_counter++;
	}

finish:
	if(iRet == -1)
	{
		/*Note that any unfreed u_c's and the related resources they point to 
		will be anyhow freed during shut down*/
		if(ipv4_is_Valid(u_c))
			PIL_FreeMemory(u_c);

		if(!reqComp_Indctr || success_counter == 0)
		{
			/*inform the Translator component about failure via the up-call*/
			fpm_ipv4_mgr_upcall(iRet, t_c);
			if(reqComp_Indctr)
				PIL_FreeMemory(reqComp_Indctr);
		}
	}

	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Leaving ipv4_update_prefix");
}

/*****************************************************************************
*
*   Function:ipv4_flush_prefixes()
*
*   Parameters: 
*				t_c:	 Translator context
*
*   Description: 
*				This function purges the prefixes
*
*   Return: none
*
*
*****************************************************************************/
void ipv4_flush_prefixes(ipv4_TranslatorContext_t* t_c)
{    
	int32_t iRet = 0;
   uint32_t *reqComp_Indctr = NULL;
   ix_error err;
   uint32_t success_counter = 0;
   PilSemaphore lock;
   ipv4_uc_info *uc_info = NULL;
   ipv4_UserContext_t *u_c = NULL;

	
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Entering  ipv4_flush_prefixes");
   lock = PIL_CreateBinarySemaphore(TRUE);

   reqComp_Indctr = (uint32_t *)
      PIL_AllocateMemory (sizeof(uint32_t), PIL_MEM_ZERO_INIT);

   if(!reqComp_Indctr)
   {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
            "Mem allocation for reqComp_Indctr failed");
      iRet = -1;
      goto finish;
   }

   /*set the request completion indicator counter*/
   *reqComp_Indctr = 1;

   u_c = (ipv4_UserContext_t *)
         PIL_AllocateMemory (sizeof(ipv4_UserContext_t), PIL_MEM_ZERO_INIT);
   if(!u_c)
   {
         npf_logger_write(LOG_FP_IPV4, LOG_ERR, "Mem allocation for u_c failed");
         iRet = -1;
         PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
         *reqComp_Indctr = success_counter;
         PIL_ReleaseSemaphore(lock);
         t_c->status->retStatus = NPF_E_UNKNOWN;
         t_c->overall_status = NPF_E_UNKNOWN;
         goto finish;
   }
   npf_logger_write(LOG_FP_IPV4, LOG_INFO, "Allocated u_c:0x%x", u_c);
   u_c->reqComp_Indctr = reqComp_Indctr;
   u_c->t_c = t_c;
   u_c->lock = lock;
   u_c->index = 0;  /* Single entry */

   uc_info = (ipv4_uc_info *)
         PIL_AllocateMemory(sizeof(ipv4_uc_info ), PIL_MEM_ZERO_INIT);
   if(!uc_info)
   {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
               "Mem allocation failed for uc_info");
      iRet = -1;
      PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
      *reqComp_Indctr = success_counter;
      PIL_ReleaseSemaphore(lock);
      t_c->status->retStatus = NPF_E_UNKNOWN;
      t_c->overall_status = NPF_E_UNKNOWN;
      goto finish;
   }
   uc_info->type = FPPI_IPv4_UniFlushPrefix;
   uc_info->u_c = (void *)u_c;
   /*insert this u_c info into the u_c pending list*/
   npf_list_pushBack(&g_ipv4_pendinglist, uc_info);	


	/*Call into the IXASDK*/
	err = ix_cc_ipv4_async_purge_routes() ;

   if(err != IX_SUCCESS)
   {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
               "IXASDK30 API call failed for deleting a nexthop error code:%d",
      IX_ERROR_GET_CODE(err));
      ix_error_dump(stdout, err);			
      PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
      *reqComp_Indctr = success_counter;
      PIL_ReleaseSemaphore(lock);						
      t_c->status->retStatus = 
      convert_to_npf(IX_ERROR_GET_CODE(err));
      t_c->overall_status = NPF_E_UNKNOWN;
      iRet = -1;
      goto finish;
   }

   u_c->t_c->cbData.resp = (NPF_IPv4UC_AsyncResponse_t *)PIL_AllocateMemory(sizeof(NPF_IPv4UC_AsyncResponse_t), PIL_MEM_ZERO_INIT);
   
   u_c->t_c->cbData.resp->returnCode = NPF_SUCCESS;
   success_counter++;
   u_c->t_c->cbData.allOK = NPF_TRUE;
   u_c->t_c->cbData.numResp = 1;
   u_c->t_c->cbData.type = NPF_IPV4UC_PREFIX_TABLE_FLUSH;
   /* Explicitly call the Callback */
   ipv4_cb(IX_SUCCESS, (void *)u_c);

finish:
   if(iRet == -1)
   {
      /*Note that any unfreed u_c's and the related resources they point to 
        will be anyhow freed during shut down*/
      if(ipv4_is_Valid(u_c))
         PIL_FreeMemory(u_c);

      if(!reqComp_Indctr || success_counter == 0)
      {
         /*inform the Translator component about failure via the up-call*/
         fpm_ipv4_mgr_upcall(iRet, t_c);
         if(reqComp_Indctr)
            PIL_FreeMemory(reqComp_Indctr);
      }
   }
   npf_logger_write(LOG_FP_IPV4, LOG_TRACE, 
                        "Leaving  ipv4_Flush_Prefix_table");
}


/*****************************************************************************
*
*
*   Function:   ipv4_add_next_hop()
*
*   Parameters: 
*               nexthops:Pointer to nexthop list
*				t_c:	 Translator context, the embedded status struct to be 
*						 filled once IXASDK3.0 responds.
*
*   Description: 
*				This function requests the IXASDK adding nextops.
*				It provides a callback function to the IXASDK to call back with 
*				the status on adding prefixes.           
*
*   Return: none
*
*****************************************************************************/
void ipv4_add_next_hop(npf_ipv4_UnicastNextHopUpdateInfoList_t *nexthops, 
		       ipv4_TranslatorContext_t *t_c)
{
    int32_t iRet = 0;
    uint32_t i = 0;
    ipv4_uc_info *uc_info = NULL;
    ipv4_UserContext_t *u_c = NULL;
    uint32_t *reqComp_Indctr = NULL;
    ix_cc_rtmv4_next_hop_info nh_info;
    ix_error err;
    uint32_t success_counter = 0;
    PilSemaphore lock;
    ix_cc_l2tm_entry l2_info;
    FPPI_UnicastNextHopUpdateInfo_t *nhUpdCntxt = NULL;
    uint32_t      aal5Valid =0;
#ifdef _INCLUDE_ATM
    NPF_uint32_t portId =0;
    NPF_VccAddr_t vccAddr;    
    ix_cc_atmsar_vc_handle_t vcHandle = 0;
    NPF_IfAAL_t   aaltype;
    uint32_t      encap_bit;
#endif /* _INCLUDE_ATM */
   
   
   npf_logger_write(LOG_FP_IPV4, LOG_TRACE, 
		     "Entering ipv4_add_next_hop. numEntries:%d",
		     nexthops->numEntries);
    
	if(nexthops == NULL || nexthops->nhUpdateInfo == NULL)
   {  
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			     "NULL NextHop Information received from CP ");
	   iRet = -1;
	   goto finish;
   }
   lock = PIL_CreateBinarySemaphore(TRUE);
    
   reqComp_Indctr = (uint32_t *)
	PIL_AllocateMemory (sizeof(uint32_t), PIL_MEM_ZERO_INIT);
    if(!reqComp_Indctr)
	{
	    npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			     "Mem allocation for reqComp_Indctr failed");
	    iRet = -1;
	    goto finish;
	}
		
	/*set the request completion indicator counter, each add next hop is 
	completed when it gets a callback from ipv4 cc for adding to NH 
	table, and from  fp cc for adding to L2 table*/
    *reqComp_Indctr = nexthops->numEntries;
    
   for (i = 0; i < nexthops->numEntries; i++)
	{
           if(nexthops->nhUpdateInfo[i].numNHEntries > 1/* || nexthops->numOldL2IDEntries > 1*/)
	   {
	       npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			     "IXASDK30 does not support ECMP numEntries:%d,"
			     "numOldL2IDEntries:%d",
			     nexthops->nhUpdateInfo[i].numNHEntries, 
			     nexthops->nhUpdateInfo[i].numOldL2IDEntries);
	       iRet = -1;
	       goto finish;
	   }
     
	   u_c = (ipv4_UserContext_t *)
		PIL_AllocateMemory (sizeof(ipv4_UserContext_t), PIL_MEM_ZERO_INIT);
	    if(!u_c)
		{
		    npf_logger_write(LOG_FP_IPV4, LOG_ERR, "Mem allocation for u_c failed");
		    iRet = -1;
		    PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
		    *reqComp_Indctr = success_counter;
		    PIL_ReleaseSemaphore(lock);
		    t_c->status[i].retStatus = NPF_E_UNKNOWN;
		    t_c->overall_status = NPF_E_UNKNOWN;
			goto finish;
		}
	    npf_logger_write(LOG_FP_IPV4, LOG_INFO, "Allocated u_c:0x%x", u_c);
	    u_c->reqComp_Indctr = reqComp_Indctr;
	    u_c->t_c = t_c;
	    u_c->lock = lock;
	    /*set the index to 'i' aligning with the same offset as the addr
	      when the callback comes, we use index i to set the error code
	      for the respective down call addr*/
	    u_c->index = i; 
	  
       if(nexthops->nhUpdateInfo[i].nhInfo == NULL)
       {
         npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			     "NextHop Info at index [%d] is NULL",i);
	      iRet = -1;
		   PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
	      *reqComp_Indctr = success_counter;
	      PIL_ReleaseSemaphore(lock);
	   	t_c->status[i].retStatus = NPF_E_UNKNOWN;
	   	t_c->overall_status = NPF_E_UNKNOWN;
	      goto finish;
      }
  
	    /*populate the nh_info struct for IXASDK30
	      This information will be populated on all the 
	      blades*/		
	    nh_info.bladeID = nexthops->nhUpdateInfo[i].nhInfo->feid;
	    nh_info.l2Index = nexthops->nhUpdateInfo[i].nhInfo->l2id;
	    nh_info.portID = nexthops->nhUpdateInfo[i].nhInfo->portId;
	    nh_info.mtu = nexthops->nhUpdateInfo[i].nhInfo->mtu;
	    nh_info.flags = nexthops->nhUpdateInfo[i].nhInfo->flags;
	    nh_info.ipAddr = nexthops->nhUpdateInfo[i].nhInfo->nhAddr;
	    nh_info.l2IndexType = 0;
	    
	    uc_info = (ipv4_uc_info *)
		PIL_AllocateMemory (sizeof(ipv4_uc_info ), PIL_MEM_ZERO_INIT);
	    if(!uc_info)
		{
		    npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
				     "Mem allocation failed for uc_info");
		    iRet = -1;
		    PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
		    *reqComp_Indctr = success_counter;
		    PIL_ReleaseSemaphore(lock);
		    t_c->status[i].retStatus = NPF_E_UNKNOWN;
		    t_c->overall_status = NPF_E_UNKNOWN;
		    goto finish;
		}
	    uc_info->type = FPPI_IPv4_UniAddNextHop;
	    uc_info->u_c = (void *)u_c;
	    /*insert this u_c info into the u_c pending list*/
	    npf_list_pushBack(&g_ipv4_pendinglist, uc_info);
	   
      
      /* Allocate the memory for the context, Ths mem is freed in the callback
       function */
       nhUpdCntxt = PIL_AllocateMemory(sizeof(FPPI_UnicastNextHopUpdateInfo_t),PIL_MEM_ZERO_INIT);
       

       if(nhUpdCntxt == NULL)
       {
            npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
				    " Memory Allocation Problem ");
		      t_c->overall_status = NPF_E_UNKNOWN;
		      iRet = -1;
		      goto finish;
       }
       
       memcpy(nhUpdCntxt, &nexthops->nhUpdateInfo[i], sizeof(FPPI_UnicastNextHopUpdateInfo_t));
      /*Call into the IXASDK to add entries into the L2 Table */		
	   if(nhUpdCntxt->nhInfo->feid == g_mybladeID)
      {
          switch (nhUpdCntxt->nhInfo->portType)
          {
              case NPF_IF_TYPE_LAN:
              {
		             l2_info.l2AddrType = IX_CC_L2TM_L2ADDR_ETHER;
              }
              break;
              case NPF_IF_TYPE_ATM:
              {
		            l2_info.l2AddrType = IX_CC_L2TM_L2ADDR_ATM;
              }
              break;
              case NPF_IF_TYPE_POS:
              {
                  /*Not a valid header for L2. Store an invalid type */
		            l2_info.l2AddrType = IX_CC_L2TM_L2ADDR_LAST;
              }
              break;
              default:
              {		
                   npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
				                " Port Type not supported");
		             t_c->overall_status = NPF_E_UNKNOWN;
		             iRet = -1;
		             goto finish;
                 }
                 break;
           }
	        /* TBD : In case the addres is IPV6, we need to check here */	
           l2_info.ipAddrType = IX_CC_L2TM_IPADDR_IPV4;
           /* Fill the right  values */
		     l2_info.l3Address.ipv4Address = nexthops->nhUpdateInfo[i].nhInfo->nhAddr;

#ifdef USE_FPM_CC
           if(l2_info.l2AddrType == IX_CC_L2TM_L2ADDR_ETHER)
           {
	           /*add an L2 entry into the L2 table via FP CC*/
	           fpm_dump_l2_info (nexthops->nhUpdateInfo[i].nhInfo->l2id,
				  nexthops->nhUpdateInfo[i].nhInfo->portId, &l2_info);

	           err = ix_cc_fpm_async_add_l2_entry (nexthops->nhUpdateInfo[i].nhInfo->l2id,
						  nexthops->nhUpdateInfo[i].nhInfo->portId,
						  &l2_info,
						  l2_op_cb,
						  (void*)&(nexthops->nhUpdateInfo[i]));
	           if(err != IX_SUCCESS)
              {
	        	     npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
				           "IXASDK30 API call failed for na L2 entry error code:%d", IX_ERROR_GET_CODE(err));

		           ix_error_dump(stdout, err);			
		           PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
		           *reqComp_Indctr = success_counter;
		           PIL_ReleaseSemaphore(lock);						
		           t_c->status[i].retStatus = 
		           convert_to_npf(IX_ERROR_GET_CODE(err));
		           t_c->overall_status = NPF_E_UNKNOWN;
		           iRet = -1;
		           goto finish;
	           }
            } /* This block is only for ethernet. Other Tech. will update their own L2 Tables. */

#ifdef _INCLUDE_ATM
            /* We will do the ATM Stuff here; We are going to update 
               ATM specific L2 Table now  on Egress side ( AI)*/
            
            if(l2_info.l2AddrType == IX_CC_L2TM_L2ADDR_ATM)
            {
  
               npf_logger_write(LOG_FP_IPV4, LOG_TRACE, 
                             "Going to send to ATM L2 Table ");
               /* One more check here */ 
               if (nhUpdCntxt->nhInfo->mediaAddr.type != NPF_ATM_VC)
               {
                  npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
                           "Invalid ATM VC Address encountered ");
               }
               else
               {
                  vccAddr.VPI = nhUpdCntxt->nhInfo->mediaAddr.u.ATM_Vc.VPI;
                  vccAddr.VCI = nhUpdCntxt->nhInfo->mediaAddr.u.ATM_Vc.VCI;
                  portId = nhUpdCntxt->nhInfo->portId; 
               
                  fpmIfATM_GetVcHandle(vccAddr,portId,&vcHandle,&aaltype);
                  if(aaltype == NPF_IF_VCC_AAL5_LLC)
                  {
                       aal5Valid = 1;
                       encap_bit = 0;
                  }
	               else if (aaltype == NPF_IF_VCC_AAL5_VCMUX)
	               {
                       aal5Valid = 1;
                       encap_bit = 1;
	               }
                  else
                     aal5Valid =0;
		            
                  if(aal5Valid == 1) 
                  {
                     /* Call the LLC SNAP Encap Function */ 
                     ix_cc_llc_snap_encap_fpm_add_next_hop( 
	                  nhUpdCntxt->nhInfo->l2id,
	                  encap_bit,
	                  vccAddr.VPI,
	                  vccAddr.VCI,
	                  vcHandle, 
	                  0xaa, 
                     0xaa, 
                     0x03, 
                     0, 
                     0x0800,
                     LimePointLlcSnapAddCBFunc,
    	               nhUpdCntxt);
                  }

               } /* Valid ATM VC Address processing block */
           } /* If the Interface is ATM */
#endif /* _INCLUDE_ATM */
#endif /* FPM ifdef */
           if(aal5Valid == 0)
           {
               /* Onlye if we haven't sent the llc encap nhop */ 
                PIL_FreeMemory(nhUpdCntxt);
           }

            /* All L2 Table updations are done. 
               Now, let us do NextHop Table updations on Ingress */
           if(nexthops->nhUpdateInfo[i].numOldL2IDEntries > 0)	
           {
		        /*this is an update to existing nhid*/
		        /*add an entry into the NH Table on IPv4 CC*/
		        err = ix_cc_ipv4_async_update_next_hop(nexthops->nhUpdateInfo[i].nhid, &nh_info, ipv4_cb, (void *)u_c);
		        if(err != IX_SUCCESS)
		        {
			        npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
					    "IXASDK30 API call failed for adding a next hop error code:%d",
					 IX_ERROR_GET_CODE(err));
   			    ix_error_dump(stdout, err);			
   			    PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
   			    *reqComp_Indctr = success_counter;
   			    PIL_ReleaseSemaphore(lock);						
   			    t_c->status[i].retStatus = 
   			    convert_to_npf(IX_ERROR_GET_CODE(err));
   			    t_c->overall_status = NPF_E_UNKNOWN;
   			    iRet = -1;
   			    goto finish;
   		    }
   	    }
   	    else 
          {
	   	    /*add an entry into the NH Table on IPv4 CC*/
	   	    if(ix_cc_ipv4_async_add_next_hop(nexthops->nhUpdateInfo[i].nhid, &nh_info, 
	   					     ipv4_cb, (void *)u_c) != IX_SUCCESS)
	   		{
	   		    npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
	   				     "IXASDK30 API call failed for adding a next hop:%d",
	   				     IX_ERROR_GET_CODE(err));
	   		    iRet = -1;
	   		    PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
	   		    *reqComp_Indctr = success_counter;
	   		    PIL_ReleaseSemaphore(lock);
	   		    t_c->status[i].retStatus = NPF_E_UNKNOWN;
	   		    t_c->overall_status = NPF_E_UNKNOWN;
	   		    goto finish;
	   		}
	   	}
	      success_counter++;
	   } /* Only if it is my bladeId */
       
   }
 
 finish:
    if(iRet == -1)
	{
	    /*Note that any unfreed u_c's and the related resources they point to 
	      will be anyhow freed during shut down*/
	    if(ipv4_is_Valid(u_c))
		PIL_FreeMemory(u_c);
	    
	    if(!reqComp_Indctr || success_counter == 0)
		{
		    /*inform the Translator component about failure via the up-call*/
		    fpm_ipv4_mgr_upcall(iRet, t_c);
		    if(reqComp_Indctr)
			PIL_FreeMemory(reqComp_Indctr);
		}
	}
    npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Leaving ipv4_add_next_hop");
}

/*****************************************************************************
*
*
*   Function:   ipv4_query_next_hop()
*
*   Parameters: 
*               nexthops:Pointer to nexthops list
*				t_c:	 Translator context, the embedded status struct to be 
*						 filled once IXASDK3.0 responds.
*
*   Description: 
*				This function requests the IXASDK for next hop information.
*				It provides a callback function to the IXASDK to call back with 
*				the status.           
*
*   Return: none
*
*****************************************************************************/
void ipv4_query_next_hop(npf_ipv4_UnicastNextHopDeleteInfoList_t *nexthops, 
			  ipv4_TranslatorContext_t *t_c)
{
   int32_t iRet = 0;
   uint32_t i = 0;
   ipv4_uc_info *uc_info = NULL;
   ipv4_UserContext_t *u_c = NULL;
   uint32_t *reqComp_Indctr = NULL;
   ix_error err;
   uint32_t success_counter = 0;
   PilSemaphore lock;
   ix_cc_rtmv4_next_hop_info nextHopInfo;

   npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Entering  ipv4_query_next_hop");	

   lock = PIL_CreateBinarySemaphore(TRUE);
   reqComp_Indctr = (uint32_t *)
      PIL_AllocateMemory (sizeof(uint32_t), PIL_MEM_ZERO_INIT);
   if(!reqComp_Indctr)
   {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
            "Mem allocation for reqComp_Indctr failed");
      iRet = -1;
      goto finish;
   }

   /*set the request completion indicator counter*/
   *reqComp_Indctr = nexthops->numEntries;

   for (i = 0; i < nexthops->numEntries; i++)
   {
      u_c = (ipv4_UserContext_t *)
         PIL_AllocateMemory (sizeof(ipv4_UserContext_t), PIL_MEM_ZERO_INIT);
      if(!u_c)
      {
         npf_logger_write(LOG_FP_IPV4, LOG_ERR, "Mem allocation for u_c failed");
         iRet = -1;
         PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
         *reqComp_Indctr = success_counter;
         PIL_ReleaseSemaphore(lock);
         t_c->status[i].retStatus = NPF_E_UNKNOWN;
         t_c->overall_status = NPF_E_UNKNOWN;
         goto finish;
      }
      npf_logger_write(LOG_FP_IPV4, LOG_INFO, "Allocated u_c:0x%x", u_c);
      u_c->reqComp_Indctr = reqComp_Indctr;
      u_c->t_c = t_c;
      u_c->lock = lock;
      /*set the index to 'i' aligning with the same offset as the addr
        when the callback comes, we use index i to set the error code
        for the respective down call addr*/
      u_c->index = i; 

      uc_info = (ipv4_uc_info *)
         PIL_AllocateMemory (sizeof(ipv4_uc_info ), PIL_MEM_ZERO_INIT);
      if(!uc_info)
      {
         npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
               "Mem allocation failed for uc_info");
         iRet = -1;
         PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
         *reqComp_Indctr = success_counter;
         PIL_ReleaseSemaphore(lock);	
         t_c->status[i].retStatus = NPF_E_UNKNOWN;
         t_c->overall_status = NPF_E_UNKNOWN;
         goto finish;
      }
      uc_info->type = FPPI_IPv4_UniQueryNextHop;
      uc_info->u_c = (void *)u_c;
      /*insert this u_c info into the u_c pending list*/
      npf_list_pushBack(&g_ipv4_pendinglist, uc_info);

      /*Call into the IXASDK*/
      err = ix_cc_rtmv4_get_next_hop(g_pIpv4Context->hRtm, 
            nexthops->nhDeleteInfo[i].nhid, 
            &nextHopInfo);
      
      if(err != IX_SUCCESS)
      {
         npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
               "IXASDK30 API call failed for deleting a nexthop error code:%d",
               IX_ERROR_GET_CODE(err));
         ix_error_dump(stdout, err);			
         PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
         *reqComp_Indctr = success_counter;
         PIL_ReleaseSemaphore(lock);						
         t_c->status[i].retStatus = 
            convert_to_npf(IX_ERROR_GET_CODE(err));
         t_c->overall_status = NPF_E_UNKNOWN;
         iRet = -1;
         u_c->t_c->cbData.allOK = NPF_FALSE;
         u_c->t_c->cbData.numResp = 1;
         u_c->t_c->cbData.type = NPF_IPV4UC_NEXT_HOP_ENTRY_QUERY;
         if (i == 0) 
            u_c->t_c->cbData.resp = (NPF_IPv4UC_AsyncResponse_t *)PIL_AllocateMemory(sizeof(NPF_IPv4UC_AsyncResponse_t)*nexthops->numEntries, PIL_MEM_ZERO_INIT);
   
         u_c->t_c->cbData.resp[i].returnCode = NPF_FAIL;
         u_c->t_c->cbData.resp[i].u.nextHopQueryResult.nextHopIdentifier = 
                  0;
         u_c->t_c->cbData.resp[i].u.nextHopQueryResult.nextHopArray.nextHopCount = 0;
         u_c->t_c->cbData.resp[i].u.nextHopQueryResult.nextHopArray.nextHopArray = 
         (NPF_IPv4UC_NextHop_t *)PIL_AllocateMemory(sizeof(NPF_IPv4UC_NextHop_t), PIL_MEM_ZERO_INIT);
         u_c->t_c->cbData.resp[i].u.nextHopQueryResult.nextHopArray.nextHopArray->type = 0;
         u_c->t_c->cbData.resp[i].u.nextHopQueryResult.nextHopArray.nextHopArray->weight = 0;
         u_c->t_c->cbData.resp[i].u.nextHopQueryResult.nextHopArray.nextHopArray->egressInterface = 0;
         u_c->t_c->cbData.resp[i].u.nextHopQueryResult.nextHopArray.nextHopArray->nextHopIP = 0;

         goto finish;
      }

      if (i == 0) 
         u_c->t_c->cbData.resp = (NPF_IPv4UC_AsyncResponse_t *)PIL_AllocateMemory(sizeof(NPF_IPv4UC_AsyncResponse_t)*nexthops->numEntries, PIL_MEM_ZERO_INIT);
   
      u_c->t_c->cbData.resp[i].returnCode = NPF_SUCCESS;
      u_c->t_c->cbData.resp[i].u.nextHopQueryResult.nextHopIdentifier = 
                  nexthops->nhDeleteInfo[i].nhid;
      u_c->t_c->cbData.resp[i].u.nextHopQueryResult.nextHopArray.nextHopCount = 1;
      u_c->t_c->cbData.resp[i].u.nextHopQueryResult.nextHopArray.nextHopArray = 
         (NPF_IPv4UC_NextHop_t *)PIL_AllocateMemory(sizeof(NPF_IPv4UC_NextHop_t), PIL_MEM_ZERO_INIT);
      u_c->t_c->cbData.resp[i].u.nextHopQueryResult.nextHopArray.nextHopArray->type = 0;
      u_c->t_c->cbData.resp[i].u.nextHopQueryResult.nextHopArray.nextHopArray->weight = 0;
      u_c->t_c->cbData.resp[i].u.nextHopQueryResult.nextHopArray.nextHopArray->egressInterface = 0;
      u_c->t_c->cbData.resp[i].u.nextHopQueryResult.nextHopArray.nextHopArray->nextHopIP = nextHopInfo.ipAddr;


      success_counter++;
   }
      u_c->t_c->cbData.allOK = NPF_TRUE;
      u_c->t_c->cbData.numResp = nexthops->numEntries;
      u_c->t_c->cbData.type = NPF_IPV4UC_NEXT_HOP_ENTRY_QUERY;

      /* TODO: Change the following */
        ipv4_cb(IX_SUCCESS, (void *)u_c);
/*      InsertMethodIntoList(
         Method_ix_cc_rtmv4_sync_query_next_hop,
         STUBS_IPv4_ROUTE_OP,
         ipv4_cb,
         (void *)u_c);  */

finish:
   if(iRet == -1)
   {
      /*Note that any unfreed u_c's and the related resources they point to 
        will be anyhow freed during shut down*/
      if(ipv4_is_Valid(u_c))
         PIL_FreeMemory(u_c);

      if(!reqComp_Indctr || success_counter == 0)
      {
         /*inform the Translator component about failure via the up-call*/
         fpm_ipv4_mgr_upcall(iRet, t_c);
         if(reqComp_Indctr)
            PIL_FreeMemory(reqComp_Indctr);
      }
   }
   npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Leaving  ipv4_delete_next_hop");
}
/*****************************************************************************
*
*
*   Function:   ipv4_delete_next_hop()
*
*   Parameters: 
*               nexthops:Pointer to nexthops list
*				t_c:	 Translator context, the embedded status struct to be 
*						 filled once IXASDK3.0 responds.
*
*   Description: 
*				This function requests the IXASDK deleting prefixes.
*				It provides a callback function to the IXASDK to call back with 
*				the status on deleting nexthops.           
*
*   Return: none
*
*****************************************************************************/
void ipv4_delete_next_hop(npf_ipv4_UnicastNextHopDeleteInfoList_t *nexthops, 
			  ipv4_TranslatorContext_t *t_c)
{
    int32_t iRet = 0;
    uint32_t i = 0;
    ipv4_uc_info *uc_info = NULL;
    ipv4_UserContext_t *u_c = NULL;
    uint32_t *reqComp_Indctr = NULL;
    ix_error err;
    uint32_t success_counter = 0;
    PilSemaphore lock;
    
    npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Entering  ipv4_delete_next_hop");	
    
    lock = PIL_CreateBinarySemaphore(TRUE);
    reqComp_Indctr = (uint32_t *)
	PIL_AllocateMemory (sizeof(uint32_t), PIL_MEM_ZERO_INIT);
    if(!reqComp_Indctr)
	{
	    npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			     "Mem allocation for reqComp_Indctr failed");
	    iRet = -1;
	    goto finish;
	}
    
    /*set the request completion indicator counter*/
    *reqComp_Indctr = nexthops->numEntries;
    
    for (i = 0; i < nexthops->numEntries; i++)
	{
	    
	    u_c = (ipv4_UserContext_t *)
		PIL_AllocateMemory (sizeof(ipv4_UserContext_t), PIL_MEM_ZERO_INIT);
	    if(!u_c)
		{
		    npf_logger_write(LOG_FP_IPV4, LOG_ERR, "Mem allocation for u_c failed");
		    iRet = -1;
		    PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
		    *reqComp_Indctr = success_counter;
		    PIL_ReleaseSemaphore(lock);
		    t_c->status[i].retStatus = NPF_E_UNKNOWN;
		    t_c->overall_status = NPF_E_UNKNOWN;
		    goto finish;
		}
	    npf_logger_write(LOG_FP_IPV4, LOG_INFO, "Allocated u_c:0x%x", u_c);
	    u_c->reqComp_Indctr = reqComp_Indctr;
	    u_c->t_c = t_c;
	    u_c->lock = lock;
	    /*set the index to 'i' aligning with the same offset as the addr
	      when the callback comes, we use index i to set the error code
	      for the respective down call addr*/
	    u_c->index = i; 
	    
	    uc_info = (ipv4_uc_info *)
		PIL_AllocateMemory (sizeof(ipv4_uc_info ), PIL_MEM_ZERO_INIT);
	    if(!uc_info)
		{
		    npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
				     "Mem allocation failed for uc_info");
		    iRet = -1;
		    PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
		    *reqComp_Indctr = success_counter;
		    PIL_ReleaseSemaphore(lock);	
		    t_c->status[i].retStatus = NPF_E_UNKNOWN;
		    t_c->overall_status = NPF_E_UNKNOWN;
		    goto finish;
		}
	    uc_info->type = FPPI_IPv4_UniDelNextHop;
	    uc_info->u_c = (void *)u_c;
	    /*insert this u_c info into the u_c pending list*/
	    npf_list_pushBack(&g_ipv4_pendinglist, uc_info);
	    
	    /*Call into the IXASDK*/
	    err = ix_cc_ipv4_async_delete_next_hop(nexthops->nhDeleteInfo[i].nhid,
						   ipv4_cb, (void *)u_c);
	    if(err != IX_SUCCESS)
		{
		    npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
				     "IXASDK30 API call failed for deleting a nexthop error code:%d",
				     IX_ERROR_GET_CODE(err));
		    ix_error_dump(stdout, err);			
		    PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
		    *reqComp_Indctr = success_counter;
		    PIL_ReleaseSemaphore(lock);						
		    t_c->status[i].retStatus = 
			convert_to_npf(IX_ERROR_GET_CODE(err));
		    t_c->overall_status = NPF_E_UNKNOWN;
		    iRet = -1;
		    goto finish;
		}
	    /*
	      needs to be fixed for 1.2, where all the next hop and arp calls 
	      should pass the port type as well
	    */
	    if(nexthops->nhDeleteInfo[i].nhInfo.feid == g_mybladeID)	{
		/*del an L2 entry from the L2 table via FP CC*/
#ifdef USE_FPM_CC
	      npf_logger_write (LOG_FP_IPV4, LOG_INFO, "L2 Id being deleted = %d \n", 
				nexthops->nhDeleteInfo[i].nhInfo.l2id);

	      if(ix_cc_fpm_async_del_l2_entry(nexthops->nhDeleteInfo[i].nhInfo.l2id,
					      l2_op_cb,
					      (void*) nexthops->nhDeleteInfo[i].nhInfo.l2id) 
		 != IX_SUCCESS)
		    {
			npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
					 "IXASDK30 API call failed for na L2 entry");
			iRet = -1;
			PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
			*reqComp_Indctr = success_counter;
			PIL_ReleaseSemaphore(lock);
			t_c->status[i].retStatus = NPF_E_UNKNOWN;
			t_c->overall_status = NPF_E_UNKNOWN;
			goto finish;
		    }

#ifdef _INCLUDE_ATM
            /* We will do the ATM Stuff here; We are going to delete 
               ATM specific LLC SNAP Table now  on Egress side ( AI)*/
          npf_logger_write(LOG_FP_IPV4, LOG_TRACE, 
                   "Going to send delete request to LLC Snap Table ");
          /* Call the LLC SNAP Encap Function */ 
          ix_cc_llc_snap_encap_fpm_del_next_hop( 
                     nexthops->nhDeleteInfo[i].nhInfo.l2id,
	                  LimePointLlcSnapDeleteCBFunc,
    	               NULL);
#endif /* _INCLUDE_ATM */
#endif /* USE FPM CC*/
	
	    }
	    success_counter++;
	}
 finish:
    if(iRet == -1)
	{
	    /*Note that any unfreed u_c's and the related resources they point to 
	      will be anyhow freed during shut down*/
	    if(ipv4_is_Valid(u_c))
		PIL_FreeMemory(u_c);
	    
	    if(!reqComp_Indctr || success_counter == 0)
		{
		    /*inform the Translator component about failure via the up-call*/
		    fpm_ipv4_mgr_upcall(iRet, t_c);
		    if(reqComp_Indctr)
			PIL_FreeMemory(reqComp_Indctr);
		}
	}
    npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Leaving  ipv4_delete_next_hop");
}

/*****************************************************************************
*
*   Function:ipv4_flush_next_hops()
*
*   Parameters: 
*               nexthops:Pointer to nexthops list
*				t_c:	 Translator context
*
*   Description: 
*				This function purges all routes and next hops
*
*   Return: none
*
*
*****************************************************************************/
void ipv4_flush_next_hops(npf_ipv4_UnicastNextHopDeleteInfoList_t *nexthops, ipv4_TranslatorContext_t* t_c)
{    
    int32_t iRet = 0, i;
    ix_error err;
    uint32_t success_counter = 0;
	
    npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Entering  ipv4_flush_next_hops");
    /* Flush the L2Table on egress side first */
    for (i = 0; i < nexthops->numEntries; i++) 
    {
      if(nexthops->nhDeleteInfo[i].nhInfo.feid == g_mybladeID) 
      {
	
	      if(ix_cc_fpm_async_del_l2_entry (nexthops->nhDeleteInfo[i].nhInfo.l2id,
					 l2_op_cb,
					 (void*) nexthops->nhDeleteInfo[i].nhInfo.l2id) 
	             != IX_SUCCESS) 
         {
	         npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			   "IXASDK30 API call failed for na L2 entry");
	         iRet = -1;
	         goto finish;
	      }
      }
    }
    
    /*Call into the IXASDK to purge the RTM */
    if((err = ix_cc_ipv4_async_purge_rtm())
                                                      != IX_SUCCESS) 
    {
	    npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			 "IXASDK30 API call failed for purging rtm");
       ix_error_dump(stdout, err);			
       t_c->status->retStatus = 
                 convert_to_npf(IX_ERROR_GET_CODE(err));
	    iRet = -1;
       goto finish;
    }		  
    /* Allocate mem for response */ 
    t_c->cbData.resp = (NPF_IPv4UC_AsyncResponse_t *)PIL_AllocateMemory(sizeof(NPF_IPv4UC_AsyncResponse_t), PIL_MEM_ZERO_INIT);
   
    t_c->cbData.resp->returnCode = NPF_SUCCESS;
    success_counter++;
    t_c->cbData.allOK = NPF_TRUE;
    t_c->cbData.numResp = 1;
    t_c->cbData.type = NPF_IPV4UC_NEXT_HOP_TABLE_FLUSH;
 
    fpm_ipv4_mgr_upcall(iRet, t_c);
    npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Leaving ipv4_flush_next_hops");
    
    return;
 finish:
    if(iRet == -1) 
    {		
	   /*inform the Translator component about failure via the up-call*/
    	fpm_ipv4_mgr_upcall(iRet, t_c);
    }
}

/*****************************************************************************
*
*
*   Function:   ipv4_add_arp()
*
*   Parameters: 
*               arpentry:Pointer to arp list
*	 t_c:	 Translator context, the embedded status struct to be 
*				 filled once IXASDK3.0 responds.
*
*   Description: 
*		This function requests the IXASDK adding arp eintries.
*		It provides a callback function to the IXASDK to call back 
* with the status on adding arp entry.           
*
*   Return: none
*
*****************************************************************************/
void ipv4_add_arp(npf_ipv4_UnicastToL2List_t *arpentry, 
		  ipv4_TranslatorContext_t *t_c)
{
    int32_t iRet = 0;
    uint32_t i = 0;
#ifdef USE_FPM_CC
    ix_error err;
#endif
    ipv4_uc_info *uc_info = NULL;
    ipv4_UserContext_t *u_c = NULL;
    uint32_t *reqComp_Indctr = NULL;
    uint32_t success_counter = 0;
    PilSemaphore lock;
    ix_cc_eth_tx_next_hop_info arp_info;
    
    npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Entering ipv4_add_arp");
    
    lock = PIL_CreateBinarySemaphore(TRUE);
    reqComp_Indctr = (uint32_t *)PIL_AllocateMemory(
		sizeof(uint32_t), PIL_MEM_ZERO_INIT);
    if(!reqComp_Indctr)
	{
	    npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			     "Mem allocation for reqComp_Indctr failed");
	    iRet = -1;
	    goto finish;
	}
    
    /*set the request completion indicator counter*/
    *reqComp_Indctr = arpentry->numEntries;
    
    for (i = 0; i < arpentry->numEntries; i++)
	{
	    u_c = (ipv4_UserContext_t *)
		PIL_AllocateMemory (sizeof(ipv4_UserContext_t),
				    PIL_MEM_ZERO_INIT);
	    if(!u_c)
		{
		  npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
				   "Mem allocation for u_c failed");
		  iRet = -1;
		  PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
		  *reqComp_Indctr = success_counter;
		  PIL_ReleaseSemaphore(lock);
		  t_c->status[i].retStatus = NPF_E_UNKNOWN;
		  t_c->overall_status = NPF_E_UNKNOWN;
		  goto finish;
		}	
	    npf_logger_write(LOG_FP_IPV4, LOG_INFO, "Allocated u_c:0x%x", u_c);
	    u_c->reqComp_Indctr = reqComp_Indctr;
	    u_c->t_c = t_c;		
	    u_c->lock = lock;
	    /*set the index to 'i' aligning with the same offset as the addr
	      when the callback comes, we use index i to set the error code
	      for the respective down call addr*/
	    u_c->index = i; 
	    
	    uc_info = (ipv4_uc_info *)
		PIL_AllocateMemory (sizeof(ipv4_uc_info ), PIL_MEM_ZERO_INIT);
	    if(!uc_info)
		{
		    npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
				     "Mem allocation failed for uc_info");
		    iRet = -1;
		    PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
		    *reqComp_Indctr = success_counter;
		    PIL_ReleaseSemaphore(lock);
		    t_c->status[i].retStatus = NPF_E_UNKNOWN;
		    t_c->overall_status = NPF_E_UNKNOWN;
		    goto finish;
		}
	    uc_info->type = FPPI_IPv4_UniAddToL2;
	    uc_info->u_c = (void *)u_c;
	    /*insert this u_c info into the u_c pending list*/
	    npf_list_pushBack(&g_ipv4_pendinglist, uc_info);

	    /* Assume the L2 entry is for Ethernet & IPv4 */
	    
	    arp_info.l2Index = arpentry->toL2Info[i].l2id;
	    arp_info.nextHopIp = 
	      arpentry->toL2Info[i].l2Entry.IP_Address;
	    arp_info.outputPort = 
	      arpentry->toL2Info[i].portId;
	    memcpy(&arp_info.l2Addr.etherAddr, 
		   &arpentry->toL2Info[i].l2Entry.mediaAddress.u.MAC_Address,
		   ETH_ALEN);

	    switch(arpentry->toL2Info[i].l2Entry.mediaAddress.type)
		{
		case NPF_MAC_ADDRESS:
		    npf_logger_write(LOG_FP_IPV4, LOG_INFO, 
				     "MAC Address type");
		    arp_info.l2Index = arpentry->toL2Info[i].l2id;
		    arp_info.nextHopIp = 
		      arpentry->toL2Info[i].l2Entry.IP_Address;
		    arp_info.outputPort = 
			arpentry->toL2Info[i].portId;
		    memcpy(&arp_info.l2Addr.etherAddr, 
			   &arpentry->toL2Info[i].l2Entry.mediaAddress.u.MAC_Address,
			   ETH_ALEN);
		    
		    npf_logger_write(LOG_FP_IPV4, LOG_INFO, 
				     "MAC Addr: 0x%x::%x::%x::%x::%x::%x", 
				     arp_info.l2Addr.etherAddr[0],
				     arp_info.l2Addr.etherAddr[1],
				     arp_info.l2Addr.etherAddr[2],
				     arp_info.l2Addr.etherAddr[3],
				     arp_info.l2Addr.etherAddr[4],
				     arp_info.l2Addr.etherAddr[5]);
		    break;
		case NPF_ATM_VC:
		    npf_logger_write(LOG_FP_IPV4, LOG_INFO, 
				"ATM Not currently supported");
		    iRet = -1;
		    goto finish;
		default:
		    npf_logger_write(LOG_FP_IPV4, LOG_INFO, 
				"OTHER Not currently supported");
		    iRet = -1;
		    goto finish;
		}
#if 0
	    switch(arpentry->toL2Info[i].l2Entry.interfaceHandle.type)
		{
		case NPF_IF_TYPE_POS:
		    npf_logger_write(LOG_FP_IPV4, LOG_INFO, 
				     "POS Interface type");
		    arg_OutputPort = 
			arpentry->toL2Info[i].l2Entry.interfaceHandle.u.POS_Attr.port;
		    npf_logger_write(LOG_FP_IPV4, LOG_INFO, 
				     "arg_OutputPort:%d", arg_OutputPort);
		    break;
		case NPF_IF_TYPE_IPV4:
		    npf_logger_write(LOG_FP_IPV4, LOG_INFO, 
				     "NPF_IF_TYPE_IPV4 Interface type");
		    break;
		case NPF_IF_TYPE_LAN:
		    npf_logger_write(LOG_FP_IPV4, LOG_INFO, 
				     "NPF_IF_TYPE_LAN Interface type");
		    arg_OutputPort = 
			arpentry->toL2Info[i].l2Entry.interfaceHandle.u.LAN_Attr.port;
		    npf_logger_write(LOG_FP_IPV4, LOG_INFO, 
				     "arg_OutputPort:%d", arg_OutputPort);
		    break;
		case NPF_IF_TYPE_ATM:
		    npf_logger_write(LOG_FP_IPV4, LOG_INFO, 
				     "ATM interface Not currently supported");
		    iRet = -1;
		    goto finish;
		default:
		    npf_logger_write(LOG_FP_IPV4, LOG_INFO, 
				     "interface Not currently supported");
		    iRet = -1;
		    goto finish;
		}
#endif
	    /*Call into the IXASDK*/
#ifdef USE_FPM_CC
	    fpm_dump_arp_info (&arp_info);

	    err = ix_cc_fpm_async_add_arp_entry (&arp_info,
						 ipv4_cb, 
						 (void *)u_c);	
	    if(err != IX_SUCCESS)
	      {
		npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
				 "IXASDK30 API call failed for adding arp error code:%d",
				 IX_ERROR_GET_CODE(err));
		ix_error_dump(stdout, err);			
		PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
		*reqComp_Indctr = success_counter;
		PIL_ReleaseSemaphore(lock);						
		t_c->status[i].retStatus = 
		  convert_to_npf(IX_ERROR_GET_CODE(err));
		t_c->overall_status = NPF_E_UNKNOWN;
		iRet = -1;
		goto finish;
	      }
#endif
	    success_counter++;
	}
    
 finish:
	if(iRet == -1)
	    {
	    /*Note that any unfreed u_c's and the related resources they 
	      point to will be anyhow freed during shut down*/
	    if(ipv4_is_Valid(u_c))
	      PIL_FreeMemory(u_c);
	    
	    if(!reqComp_Indctr || success_counter == 0)
	      {
		/*inform the Translator component about failure via the 
		  up-call*/
		fpm_ipv4_mgr_upcall(iRet, t_c);
		if(reqComp_Indctr)
		  PIL_FreeMemory(reqComp_Indctr);
	      }
	  }
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Leaving ipv4_add_arp");
}

/*****************************************************************************
*
*
*   Function:   ipv4_delete_arp()
*
*   Parameters: 
*               arpentry:Pointer to arp list
*				t_c:	 Translator context, the embedded status struct to be 
*						 filled once IXASDK3.0 responds.
*
*   Description: 
*				This function requests the IXASDK deleting arp.
*				It provides a callback function to the IXASDK to call back with 
*				the status on deleting arp.           
*
*   Return: none
*
*****************************************************************************/
void ipv4_delete_arp(npf_ipv4_UnicastToL2List_t *arpentry, 
		     ipv4_TranslatorContext_t *t_c)
{
	int32_t iRet = 0;
	uint32_t i = 0;
#ifdef USE_FPM_CC
    ix_error err;
#endif
	ipv4_uc_info *uc_info = NULL;
	ipv4_UserContext_t *u_c = NULL;
	uint32_t *reqComp_Indctr = NULL;
	uint32_t  arg_L2Id;
	uint32_t success_counter = 0;
	PilSemaphore lock;

	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Entering ipv4_delete_arp");
	
	lock = PIL_CreateBinarySemaphore(TRUE);
	reqComp_Indctr = (uint32_t *)PIL_AllocateMemory(
		sizeof(uint32_t), PIL_MEM_ZERO_INIT);
	if(!reqComp_Indctr)
	{
		npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"Mem allocation for reqComp_Indctr failed");
		iRet = -1;
		goto finish;
	}
	
	/*set the request completion indicator counter*/
	*reqComp_Indctr = arpentry->numEntries;

	for (i = 0; i < arpentry->numEntries; i++)
	{
		u_c = (ipv4_UserContext_t *)PIL_AllocateMemory(
		sizeof(ipv4_UserContext_t),PIL_MEM_ZERO_INIT);
		if(!u_c)
		{
			npf_logger_write(LOG_FP_IPV4, LOG_ERR, "Mem allocation for u_c failed");
			iRet = -1;
			PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
			*reqComp_Indctr = success_counter;
			PIL_ReleaseSemaphore(lock);
			t_c->status[i].retStatus = NPF_E_UNKNOWN;
			t_c->overall_status = NPF_E_UNKNOWN;
			goto finish;
		}	
		npf_logger_write(LOG_FP_IPV4, LOG_INFO, "Allocated u_c:0x%x", u_c);
		u_c->reqComp_Indctr = reqComp_Indctr;
		u_c->t_c = t_c;		
		u_c->lock = lock;
		/*set the index to 'i' aligning with the same offset as the addr
		when the callback comes, we use index i to set the error code
		for the respective down call addr*/
		u_c->index = i; 

		uc_info = (ipv4_uc_info *)PIL_AllocateMemory(
		sizeof(ipv4_uc_info ), PIL_MEM_ZERO_INIT);
		if(!uc_info)
		{
			npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
				"Mem allocation failed for uc_info");
			iRet = -1;
			PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
			*reqComp_Indctr = success_counter;
			PIL_ReleaseSemaphore(lock);
			t_c->status[i].retStatus = NPF_E_UNKNOWN;
			t_c->overall_status = NPF_E_UNKNOWN;
			goto finish;
		}
		uc_info->type = FPPI_IPv4_UniDelToL2;
		uc_info->u_c = (void *)u_c;
		/*insert this u_c info into the u_c pending list*/
		npf_list_pushBack(&g_ipv4_pendinglist, uc_info);

		/*Call into the IXASDK*/
		arg_L2Id = arpentry->toL2Info[i].l2id;
#ifdef USE_FPM_CC
	      npf_logger_write (LOG_FP_IPV4, LOG_INFO, "L2 Id being deleted = %d \n", 
				arg_L2Id);

		err = ix_cc_fpm_async_del_arp_entry(arg_L2Id,
						    ipv4_cb, (void *)u_c);
		if(err != IX_SUCCESS)
		{
		    npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
				     "IXASDK30 API call failed for deleting arp error code:%d",
				     IX_ERROR_GET_CODE(err));
		    ix_error_dump(stdout, err);			
		    PIL_AcquireSemaphore(lock, PIL_INFINITE_TIMEOUT);
		    *reqComp_Indctr = success_counter;
		    PIL_ReleaseSemaphore(lock);						
		    t_c->status[i].retStatus = 
			convert_to_npf(IX_ERROR_GET_CODE(err));
		    t_c->overall_status = NPF_E_UNKNOWN;
		    iRet = -1;
		    goto finish;
		}
#endif
		success_counter++;
	}

finish:
	if(iRet == -1)
	{
	    /*Note that any unfreed u_c's and the related resources they point to 
	      will be anyhow freed during shut down*/
	    if(ipv4_is_Valid(u_c))
		PIL_FreeMemory(u_c);
	    
	    if(!reqComp_Indctr || success_counter == 0)
		{
		    /*inform the Translator component about failure via the up-call*/
		    fpm_ipv4_mgr_upcall(iRet, t_c);
		    if(reqComp_Indctr)
			PIL_FreeMemory(reqComp_Indctr);
		}
	}
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Leaving ipv4_delete_arp");
}

/*****************************************************************************
*
*   Function:ipv4_flush_arps()
*
*   Parameters: 
*				t_c:Translator context
*
*   Description: 
*				This function purges all arps
*
*   Return: none
*
*
*****************************************************************************/
void ipv4_flush_arps(ipv4_TranslatorContext_t* t_c)
{    
    int32_t iRet = 0;
    
    npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Entering  ipv4_flush_arps");
	
    /*Call into the IXASDK*/
#ifdef USE_FPM_CC
    if(ix_cc_fpm_sync_purge_arp_table() != IX_SUCCESS)
	{
	    npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			     "IXASDK30 API call failed for purging arp table");
	    iRet = -1;
	}
#endif
    fpm_ipv4_mgr_upcall(iRet, t_c);
    npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Leaving ipv4_flush_arps");
}

/*****************************************************************************
******************************************************************************
*	Callback Function Implementations
******************************************************************************
*****************************************************************************/


/*****************************************************************************
*
*   Function:   ipv4_cb()
*
*   Parameters: 
*			arg_Result:error code returned from the IXASDK messaging operation.
*			arg_pContext:pointer to user context		
*
*   Description:Callback function that contains the status information 
*				for a specific add/delete prefix/nh/arp. 
*				Once all the required status is available the translator 
*				fpm_ipv4_mgr_upcall() is invoked.
*
*   Return: IX_SUCCESS on success, appropriate IX_ERROR on failure
*****************************************************************************/
ix_error ipv4_cb(ix_error arg_Result,void* arg_pContext)
{
	ix_error err = IX_SUCCESS;
	ipv4_UserContext_t * u_c = NULL;
	ipv4_TranslatorContext_t * t_c = NULL;
	
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Entering ipv4_cb");

	if(!arg_pContext)
	{
		npf_logger_write(LOG_FP_IPV4, LOG_ERR, "Invalid pContext");
		err = IX_ERROR_NEW(IX_CC_ERROR_NULL, IX_ERROR_LEVEL_WARNING);
		goto finish;
	}
	/*validate the u_c*/
	if(!ipv4_is_Valid(arg_pContext))
	{
		err =  IX_CC_ERROR_RANGE;
		npf_logger_write(LOG_FP_IPV4, LOG_ERR, "invalid u_c:0x%x", 
			arg_pContext);
		goto finish;
	}
	u_c = (ipv4_UserContext_t *)arg_pContext;
	t_c = (ipv4_TranslatorContext_t *)u_c->t_c;
	if(!t_c)
	{
		npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"t_c is invalid");
		err =  IX_ERROR_NEW(IX_CC_ERROR_RANGE, IX_ERROR_LEVEL_WARNING);
		goto finish;
	}

	ipv4_display_cb_type(t_c->cbType);
	
	npf_logger_write(LOG_FP_IPV4, LOG_INFO, "arg_Result:%d", 
			IX_ERROR_GET_CODE(arg_Result));
	ix_error_dump(stdout, arg_Result);

	npf_logger_write(LOG_FP_IPV4, LOG_INFO, "reqComp_Indctr:%d", 
		*(u_c->reqComp_Indctr));
  

	if(*(u_c->reqComp_Indctr) != 0)
	{
		/*decrement the request completion indicator counter*/
		PIL_AcquireSemaphore(u_c->lock, PIL_INFINITE_TIMEOUT);
		(*(u_c->reqComp_Indctr))--;
		PIL_ReleaseSemaphore((u_c->lock));
		/*set the error code*/	
		npf_logger_write(LOG_FP_IPV4, LOG_INFO, "error code: %d", 
					IX_ERROR_GET_CODE(arg_Result));
		ix_error_dump(stdout, arg_Result);
		t_c->status[u_c->index].retStatus = 
				convert_to_npf(IX_ERROR_GET_CODE(arg_Result));
		if(t_c->status[u_c->index].retStatus != NPF_NO_ERROR)
			t_c->overall_status = NPF_E_UNKNOWN;
	}
   /* When the loop for multiple entries is over */
	if(*(u_c->reqComp_Indctr) == 0)
	{
		/*at this point we have got all the information for the 
		translator request. We are ready to call into the translator 
		upcall with the received information*/	
		
      if(t_c->overall_status == NPF_NO_ERROR)
			fpm_ipv4_mgr_upcall(0, t_c);
		else
			fpm_ipv4_mgr_upcall(-1, t_c);
		
		PIL_FreeMemory(u_c->reqComp_Indctr);
		PIL_DestroySemaphore(u_c->lock);
	}
	/*free the u_c for this callback*/
	npf_logger_write(LOG_FP_IPV4, LOG_INFO, "Freeing u_c:0x%x", u_c);
	PIL_FreeMemory(u_c);

finish:/*only if something is seriously wrong. Note that the u_c's and the 
	   related resources they point to will be anyhow freed during shut down*/
	if(err != IX_SUCCESS)
		fpm_ipv4_mgr_upcall(-1, t_c);

	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Leaving ipv4_cb");
	return err;
}


ix_error l2_op_cb(ix_error arg_Result,void* arg_pContext)
{
  ix_error err = IX_SUCCESS;
  
  npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Entering l2_op_cb");
  
  if(!arg_pContext)
    {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, "Invalid pContext");
      err = IX_ERROR_NEW(IX_CC_ERROR_NULL, IX_ERROR_LEVEL_WARNING);
    }
  if (arg_Result != IX_SUCCESS) {
    npf_logger_write (LOG_FP_IPV4, LOG_ERR, "L2 OP Error");
  }

  return IX_SUCCESS;
}

/*****************************************************************************
*   Function: bits_to_mask()
*
*   Parameters: prefixLength: prefix length
*
*   Description: Gets the mask out of prefix length
*
*   return : Mask formed out of the pefixLength
*****************************************************************************/
int bits_to_mask( int prefixLength) 
{
    int temp = 1;
    int i = 0;

    for ( i = 0; i < prefixLength; i++ ) 
	{
        temp = temp << 1;
        temp = temp | 1;
    }
    temp = temp << ( 32 - prefixLength );

    //printf( "0x%x\n", temp );
    return temp;
}

/*****************************************************************************
*   Function: convert_to_npf()
*
*   Parameters: error: ixasdk3.0 error code
*
*   Description: converst the ixa error to npf error
*
*   return : npf error
*****************************************************************************/
uint32_t convert_to_npf(ix_error error)
{
	uint32_t npf_error;
	
	switch (error)
	{
	case IX_SUCCESS:
		npf_error = NPF_NO_ERROR;
		break;
	case IX_CC_IPV4_ERROR_DUPLICATE_NEXT_HOP:
		npf_error = NPF_E_UNKNOWN;
		break;
	case IX_CC_IPV4_ERROR_NEXT_HOP_ID_IN_USE:
		npf_error = NPF_E_UNKNOWN;
		break;
	case IX_CC_IPV4_ERROR_INVALID_NH_INFO:
		npf_error = NPF_E_UNKNOWN;
		break;
	case IX_CC_IPV4_ERROR_INVALID_MASK:
		npf_error = NPF_E_UNKNOWN;
		break;
	case IX_CC_IPV4_ERROR_INVALID_CONTEXT:
		npf_error = NPF_E_UNKNOWN;
		break;
	case IX_CC_IPV4_ERROR_INVALID_FLAGS:
		npf_error = NPF_E_UNKNOWN;
		break;
	case IX_CC_IPV4_ERROR_CALLBACK:
		npf_error = NPF_E_UNKNOWN;
		break;
	default:
		npf_error = NPF_E_UNKNOWN;
	}
	npf_logger_write(LOG_FP_IPV4, LOG_INFO,
		"Returning error code -->:%d", npf_error);
	return npf_error;
}

/*****************************************************************************
*
*   Function:   ipv4_is_Valid()
*
*   Parameters: 
*			u_c: Pointer to u_c
*
*   Description: 
*				Searches for the u_c in the pending u_c list, pops out the
*				entry if the match is found 
*
*   Return: 1 for if match found, 0 if match is not found
*****************************************************************************/
uint32_t ipv4_is_Valid(void *u_c)
{
	ipv4_uc_info	*uc_info = NULL;
	DListItr *itr;
	int found = 0;

	if(!u_c)
	{
		npf_logger_write(LOG_FP_IPV4, LOG_ERR, "Invalid u_c");
		return 0;
	}

	npf_logger_write(LOG_FP_IPV4, LOG_TRACE,
		"Entering ipv4_is_Valid with u_c:0x%x", u_c);
	
	/*walk through the list	to look for a match*/
	itr = npf_list_itrCreate(&g_ipv4_pendinglist);
	if (npf_list_itrFirst(itr) != -1) 
	{
        do {
				uc_info = (ipv4_uc_info *)npf_list_itrGetData(itr);
				if(uc_info->u_c == u_c)
				{
					found = 1;
					npf_logger_write(LOG_FP_IPV4, LOG_TRACE,
						"Found a match");
					npf_list_itrRemoveFree(itr);
				}
			} while (!found && npf_list_itrNext(itr) != -1);
    }	
	npf_list_itrDelete(itr);	
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Leaving ipv4_is_Valid");
	
	if(found)
		return 1;
	return 0;

}

/*****************************************************************************
*
*   Function:   ipv4_free_resources()
*
*   Parameters: 
*			none
*
*   Description: 
*				Walks thro' the u_c pending list and frees up the resources
				and appropriately calls into the translator upcall
*
*   Return: none
*****************************************************************************/
void ipv4_free_resources()
{
	ipv4_uc_info	*uc_info = NULL;
	uint32_t		valid_type = 1;

	npf_logger_write(LOG_FP_IPV4, LOG_TRACE,
		"Entering ipv4_free_resources with");
	
	/*walk through the list	and delete the pending resources*/
	while (npf_list_popBack(&g_ipv4_pendinglist, (void **)&uc_info) != -1) 
	{
		valid_type = 1;

		if(uc_info)
		{
			switch(uc_info->type)
			{
				case FPPI_IPv4_UniAddPrefix:
					npf_logger_write(LOG_FP_IPV4, LOG_INFO,
						"Freeing FPPI_IPv4_UniAddPrefix related resources");
					break;
				case FPPI_IPv4_UniDelPrefix:
					npf_logger_write(LOG_FP_IPV4, LOG_INFO,
					"Freeing FPPI_IPv4_UniDelPrefix related resources");
					break;
				case FPPI_IPv4_UniUpdatePrefix:
					npf_logger_write(LOG_FP_IPV4, LOG_INFO,
					"Freeing FPPI_IPv4_UniUpdatePrefix related resources");
					break;
				case FPPI_IPv4_UniAddNextHop:
					npf_logger_write(LOG_FP_IPV4, LOG_INFO,
					"Freeing FPPI_IPv4_UniAddNextHop related resources");
					break;
				case FPPI_IPv4_UniDelNextHop:
					npf_logger_write(LOG_FP_IPV4, LOG_INFO,
					"Freeing FPPI_IPv4_UniDelNextHop related resources");
					break;
				case FPPI_IPv4_UniAddToL2:
					npf_logger_write(LOG_FP_IPV4, LOG_INFO,
					"Freeing FPPI_IPv4_UniAddToL2 related resources");
					break;
				case FPPI_IPv4_UniDelToL2:
					npf_logger_write(LOG_FP_IPV4, LOG_INFO,
					"Freeing FPPI_IPv4_UniDelToL2 related resources");
					break;
				default:
					npf_logger_write(LOG_FP_IPV4, LOG_ERR,
						"Unknown Type");
					valid_type = 0;
			}
			if(valid_type && uc_info->u_c)
			{
				/*call into the translator*/
				fpm_ipv4_mgr_upcall( -1,
					((ipv4_UserContext_t *)(uc_info->u_c))->t_c
					);
				/*free the related resources*/
				if(((ipv4_UserContext_t *)
					(uc_info->u_c))->reqComp_Indctr)
				{
					PIL_FreeMemory(
						((ipv4_UserContext_t *)
						(uc_info->u_c))->reqComp_Indctr
						);
					((ipv4_UserContext_t *)
						(uc_info->u_c))->reqComp_Indctr = NULL;
				}
				PIL_FreeMemory(uc_info->u_c);
				PIL_FreeMemory(uc_info);
			}
		}
    }	
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Leaving ipv4_free_resources");	
}

/*****************************************************************************
*
*   Function:   ipv4_display_cb_type()
*
*   Parameters: 
*			cbType: callback type.
*
*   Description: Dsiplays the the name of the callback type
*
*   Return: 0 on success, -1 on failure
*****************************************************************************/
void ipv4_display_cb_type(uint32_t cbType)
{
	switch(cbType)
	{
	case FPPI_IPv4_UniAddPrefix:
		npf_logger_write(LOG_FP_IPV4, LOG_INFO,
			"This callback is for request to add Prefix");
		break;
	case FPPI_IPv4_UniDelPrefix:
		npf_logger_write(LOG_FP_IPV4, LOG_INFO,
			"This callback is for request to delete Prefix");
		break;
	case FPPI_IPv4_UniUpdatePrefix:
		npf_logger_write(LOG_FP_IPV4, LOG_INFO,
			"This callback is for request to update Prefix");
		break;
	case FPPI_IPv4_UniAddNextHop:
		npf_logger_write(LOG_FP_IPV4, LOG_INFO,
			"This callback is for request to add nexthop");
		break;
	case FPPI_IPv4_UniDelNextHop:
		npf_logger_write(LOG_FP_IPV4, LOG_INFO,
			"This callback is for request to delete nexthop");
		break;
	case FPPI_IPv4_UniAddToL2:
		npf_logger_write(LOG_FP_IPV4, LOG_INFO,
			"This callback is for request to add arp");
		break;
	case FPPI_IPv4_UniDelToL2:
		npf_logger_write(LOG_FP_IPV4, LOG_INFO,
			"This callback is for request to delete arp");
		break;
	case FPPI_IPv4_UniQueryPrefix:
		npf_logger_write(LOG_FP_IPV4, LOG_INFO,
			"This callback is for request to Query Prefix ");
		break;
	case FPPI_IPv4_UniQueryNextHop:
		npf_logger_write(LOG_FP_IPV4, LOG_INFO,
			"This callback is for request to Query NextHop");
		break;
	case FPPI_IPv4_UniFlushNextHop:
		npf_logger_write(LOG_FP_IPV4, LOG_INFO,
			"This callback is for request to Flush NextHop");
		break;
	case FPPI_IPv4_UniFlushPrefix:
		npf_logger_write(LOG_FP_IPV4, LOG_INFO,
			"This callback is for request to Flush Prefix ");
		break;
	default:
		npf_logger_write(LOG_FP_IPV4, LOG_ERR,
			"Invalid callback Type");
	}
}

void ipv4_sync_routes (uint32_t arg_Prefix, uint32_t arg_Mask)
{
    ix_error err = IX_SUCCESS;
    fpm_ipv4_context ipv4_context;

    /* call into the IPv4 Core component to look up the route.
       We will get back the NHinfo for the prefix we just added. Use this
       to populate the kernel route table
     */

    ipv4_context.prefix = arg_Prefix;
    ipv4_context.mask = arg_Mask;

    err = ix_cc_ipv4_async_lookup_route (arg_Prefix,
					 ipv4_lookup_route_cb,
					 (void*) &ipv4_context);
    if (err != IX_SUCCESS) {
	npf_logger_write (LOG_FP_IPV4, LOG_ERR, 
			  "Route lookup failed");
    }
    return;
}

ix_error ipv4_lookup_route_cb (ix_error arg_Result,
			       void* arg_pContext,
			       ix_cc_rtmv4_next_hop_info* arg_pNHInfo)
{
#ifdef _VXWORKS_
  fpm_ipv4_context* context = (fpm_ipv4_context*) arg_pContext;
  
  mRouteEntryAdd (context->prefix, arg_pNHInfo->ipAddr, 0, context->mask,
		  0, 0);
#endif    
  return IX_SUCCESS; 
}


void fpm_dump_l2_info (uint32_t l2_id, uint32_t port_id,
		       ix_cc_l2tm_entry* l2_info)
{
  
  npf_logger_write (LOG_FP_IPV4, LOG_INFO, "L2 Information\n"
		    "--------------------\n"
		    "L2 ID   = %d\n"
		    "PORT ID = %d\n", l2_id, port_id);
  
  switch (l2_info->l2AddrType) {
  case IX_CC_L2TM_L2ADDR_ETHER:
    npf_logger_write (LOG_FP_IPV4, LOG_INFO, "L2 addr type = Ethernet\n");
    break;
  case IX_CC_L2TM_L2ADDR_ATM:
    npf_logger_write (LOG_FP_IPV4, LOG_INFO, "L2 addr type = ATM\n");
    break;
  default:
      npf_logger_write (LOG_FP_IPV4, LOG_INFO, "L2 addr type = Unknown!\n");
      break;
  }
  
    switch (l2_info->ipAddrType) {
    case IX_CC_L2TM_IPADDR_IPV4:
      npf_logger_write (LOG_FP_IPV4, LOG_INFO, "IP addr type = IPv4\n IPAddress = 0x%x\n",
			l2_info->l3Address.ipv4Address);
      break;
    case IX_CC_L2TM_IPADDR_IPV6:
      npf_logger_write (LOG_FP_IPV4, LOG_INFO, "IP addr type = IPv6\n"
			"IPAddress = 0x%2x:0x%2x:0x%2x:0x%2x:0x%2x:0x%2x:0x%2x:0x%2x\n",
			l2_info->l3Address.ipv6Address);
      
      break;
    default:
      npf_logger_write (LOG_FP_IPV4, LOG_INFO, "IP addr type = Unknown!\n");
      break;
    }
  return;
}

void fpm_dump_arp_info (ix_cc_eth_tx_next_hop_info *arp_info)
{
  npf_logger_write (LOG_FP_IPV4, LOG_INFO, "L2 ID = %d \n Output Port = %d \n"
		    "Next hop IP = 0x%x \n L2 Addr = %lX:%lX:%lX:%lX:%lX:%lX",
		    arp_info->l2Index, 
		    arp_info->outputPort, 
		    arp_info->nextHopIp, 
		    arp_info->l2Addr.etherAddr[0],arp_info->l2Addr.etherAddr[1],
		    arp_info->l2Addr.etherAddr[2],arp_info->l2Addr.etherAddr[3],
		    arp_info->l2Addr.etherAddr[4],arp_info->l2Addr.etherAddr[5]
		    );
  return;
}
