/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2002 Intel Corporation. All Rights Reserved.
*****************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include "FPModule.h"
#include "ipv4_mgr_impl.h"
#include "pil.h"
#include "log.h"

/* Extern Declarations */
extern uint32_t ix_cc_ipv4_initStub();
extern void ix_cc_ipv4_finiStub();

/* Callback Handles - Callbacks registered with Back end API */
FPPI_CBHANDLE IPv4AddRouteCBHdl,  IPv4DelRouteCBHdl, PurgeRouteCBHdl, UpdateRouteCBHdl, IPv4QueryRouteCBHdl;
FPPI_CBHANDLE ARPAddEntryCBHdl, ARPDelEntryCBHdl, ARPQueryCBHdl,ARPPurgeCBHdl;
FPPI_CBHANDLE AddNHCBHdl, DelNHCBHdl, PurgeNHCBHdl, QueryNHCBHdl;

/* Contexts of callbacks registered with BEND API */
static FPPI_CONTEXT g_IPv4_AddPrefix_context;
static FPPI_CONTEXT g_IPv4_QueryPrefix_context;
static FPPI_CONTEXT g_IPv4_DelPrefix_context;
static FPPI_CONTEXT g_IPv4_FlushPrefix_context;
static FPPI_CONTEXT g_IPv4_UpdatePrefix_context;
static FPPI_CONTEXT g_IPv4_AddNexthopPrefix_context;
static FPPI_CONTEXT g_IPv4_QueryNexthopPrefix_context;
static FPPI_CONTEXT g_IPv4_DelNexthopPrefix_context;
static FPPI_CONTEXT g_IPv4_FlushNexthopPrefix_context;
static FPPI_CONTEXT g_IPv4_AddArp_context;
static FPPI_CONTEXT g_IPv4_DelArp_context;
static FPPI_CONTEXT g_IPv4_FlushArp_context;

/*****************************************************************************
*
*   Function: fpm_ipv4_mgr_query_next_hop()
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data - 
*												npf_ipv4_UnicastNextHopList_t
*
*   Description: This callback function is registered with the Back-end API.
*                The Back-end API calls this function to query next hops.
*
*
*   PRETURN : None 
*
*****************************************************************************/
void fpm_ipv4_mgr_query_next_hop(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context, void *response_data)
{
	uint32_t i;
	ipv4_TranslatorContext_t *t_c = NULL;
	npf_ipv4_UnicastNextHopDeleteInfoList_t *nexthops = NULL;
	npf_IPv4_Status_t *status = NULL;
    
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE,
		"Entering fpm_ipv4_mgr_delete_next_hop");

    if(context != g_IPv4_QueryNexthopPrefix_context)
        return;

    nexthops = (npf_ipv4_UnicastNextHopDeleteInfoList_t *) response_data;
	
	/*Allocate the necessary memory for building the translator context*/
	t_c = (ipv4_TranslatorContext_t *)PIL_AllocateMemory
		(sizeof(ipv4_TranslatorContext_t), PIL_MEM_ZERO_INIT);
	status = (npf_IPv4_Status_t *)PIL_AllocateMemory(
		sizeof(npf_IPv4_Status_t) * nexthops->numEntries, PIL_MEM_ZERO_INIT);
	if(t_c == NULL || status == NULL)
	{
		npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"Memory allocation failed");
		goto finish;
	}

	/*Build the respective translator context*/
	t_c->cbCorrelator = cbcorrelator;
	t_c->cbType = FPPI_IPv4_UniQueryNextHop;
	t_c->status = status;
	t_c->num_entries = nexthops->numEntries;

	/*set the fields of the status structure appropriately*/
	for(i = 0; i < nexthops->numEntries; ++i)
	{
		t_c->status[i].type = NPF_IPv4_ADDRESS;	
		t_c->status[i].retStatus = NPF_E_UNKNOWN;
		t_c->status[i].u.ipv4Addr.IPv4Addr = nexthops->nhDeleteInfo[i].nhInfo.nhAddr;
		//t_c->status[i].u.ipv4Addr.IPv4NetPlen = 0xffffffff;
		t_c->status[i].u.ipv4Addr.IPv4NetPlen = 0xff;
			
	}
	/*Call into the platform specific component and wait for 
	callback on status of deleteing nexthops*/
	ipv4_query_next_hop(nexthops, t_c);
	
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, 
		"Leaving fpm_ipv4_mgr_delete_next_hop");

	return;

finish:
	if(t_c)	
		PIL_FreeMemory(t_c);
	if(status)	
		PIL_FreeMemory(status);
	
	BENDAPI_Report_Status(FPPI_FAILURE, 
                        FPPI_IPv4_UniQueryNextHop, 
                        cbcorrelator,
                        NULL);	
}

/*****************************************************************************
*
*   Function: fpm_ipv4_mgr_query_prefix()
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data - 
*												npf_ipv4_UnicastPrefixList_t
*
*   Description: This callback function is registered with the Back-end API.
*                The Back-end API calls this function to query for prefixes.
*
*
*   PRETURN : None 
*
*****************************************************************************/
void fpm_ipv4_mgr_query_prefix(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context, void *response_data)
{
	uint32_t i;
	ipv4_TranslatorContext_t *t_c = NULL;
	npf_ipv4_UnicastPrefixList_t *prefixes = NULL;
	npf_IPv4_Status_t *status = NULL;
    
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE,
		"Entering fpm_ipv4_mgr_delete_prefix");

    if(context != g_IPv4_QueryPrefix_context)
        return;

    prefixes = (npf_ipv4_UnicastPrefixList_t *) response_data;
	
	/*Allocate the necessary memory for building the translator context*/
	t_c = (ipv4_TranslatorContext_t *)PIL_AllocateMemory
		(sizeof(ipv4_TranslatorContext_t), PIL_MEM_ZERO_INIT);
	status = (npf_IPv4_Status_t *)PIL_AllocateMemory(
		sizeof(npf_IPv4_Status_t) * prefixes->numEntries, PIL_MEM_ZERO_INIT);
	if(t_c == NULL || status == NULL)
	{
		npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"Memory allocation failed");
		goto finish;
	}

	/*Build the respective translator context*/
	t_c->cbCorrelator = cbcorrelator;
	t_c->cbType = FPPI_IPv4_UniQueryPrefix;
	t_c->status = status;
	t_c->num_entries = prefixes->numEntries;

	/*set the fields of the status structure appropriately*/
	for(i = 0; i < prefixes->numEntries; ++i)
	{
		t_c->status[i].type = NPF_IPv4_ADDRESS;	
		t_c->status[i].retStatus = NPF_E_UNKNOWN;
		t_c->status[i].u.ipv4Addr.IPv4Addr = 
			prefixes->prfxInfo[i].IPv4Addr;
		t_c->status[i].u.ipv4Addr.IPv4NetPlen = 
			prefixes->prfxInfo[i].IPv4NetPlen;
	}
	/*Call into the platform specific component and wait for 
	callback on status of querying for prefixes*/
	ipv4_query_prefix(prefixes, t_c);
	
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, 
		"Leaving fpm_ipv4_mgr_delete_prefix");

	return;

finish:
	if(t_c)	
		PIL_FreeMemory(t_c);
	if(status)	
		PIL_FreeMemory(status);
	
	BENDAPI_Report_Status(FPPI_FAILURE, 
                        FPPI_IPv4_UniQueryPrefix, 
                        cbcorrelator,
                        NULL);	
}

/*****************************************************************************
*
*   Function:   fpm_ipv4_mgr_init()
*
*   Parameters: None 
*
*   Description: This function starts up the ipv4 manager. It registers 
*				 appropriate for callbacks with the Back-end API, initializes 
*				 the platform specific component.
*
*   Return: 0 on success, FPPAPI_FAILURE on failure 
*****************************************************************************/
int fpm_ipv4_mgr_init()
{
   int errtype;

   /* Assigning Contexts for registering for callbacks */
   g_IPv4_AddPrefix_context = (void *) FPPI_IPv4_UniAddPrefix;
   g_IPv4_DelPrefix_context = (void *) FPPI_IPv4_UniDelPrefix;
   g_IPv4_FlushPrefix_context = (void *) FPPI_IPv4_UniFlushPrefix;	
   g_IPv4_UpdatePrefix_context = (void *) FPPI_IPv4_UniUpdatePrefix;
   g_IPv4_AddNexthopPrefix_context = (void *) FPPI_IPv4_UniAddNextHop;
   g_IPv4_DelNexthopPrefix_context = (void *) FPPI_IPv4_UniDelNextHop;
   g_IPv4_FlushNexthopPrefix_context = (void *) FPPI_IPv4_UniFlushNextHop;	
   g_IPv4_AddArp_context = (void *) FPPI_IPv4_UniAddToL2;
   g_IPv4_DelArp_context = (void *) FPPI_IPv4_UniDelToL2;
   g_IPv4_FlushArp_context = (void *) FPPI_IPv4_UniFlushToL2;
   g_IPv4_QueryPrefix_context = (void *) FPPI_IPv4_UniQueryPrefix;
   g_IPv4_QueryNexthopPrefix_context = (void *) FPPI_IPv4_UniQueryNextHop;

#ifndef _VXWORKS_
#ifndef IXASDKSTUBS
   sh_ipv4_init();
#endif
#endif 

#ifdef IXASDKSTUBS
   ix_cc_ipv4_initStub();
#endif
   /*call backs for Prefixes*/
   errtype = BENDAPI_register_callback (&fpm_ipv4_mgr_add_prefix, 
         &IPv4AddRouteCBHdl, 
         FPPI_IPv4_UniAddPrefix, 
         g_IPv4_AddPrefix_context);
   if (FPPI_SUCCESS != errtype)
   {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
            "Registering Add Route Failed\n");
      return (FPPI_FAILURE);
   }

   errtype = BENDAPI_register_callback (&fpm_ipv4_mgr_query_prefix, 
         &IPv4QueryRouteCBHdl, 
         FPPI_IPv4_UniQueryPrefix, 
         g_IPv4_QueryPrefix_context);
   if (FPPI_SUCCESS != errtype)
   {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
            "Registering Query Prefix Failed\n");
      return (FPPI_FAILURE);
   }


   errtype = BENDAPI_register_callback (&fpm_ipv4_mgr_delete_prefix, 
         &IPv4DelRouteCBHdl, 
         FPPI_IPv4_UniDelPrefix, 
         g_IPv4_DelPrefix_context);
   if (FPPI_SUCCESS != errtype)
   {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
            "Registering for Delete Route Failed\n");
      return (FPPI_FAILURE);
   }
   errtype = BENDAPI_register_callback (&fpm_ipv4_mgr_flush_prefixes, 
         &PurgeRouteCBHdl, 
         FPPI_IPv4_UniFlushPrefix, 
         g_IPv4_FlushPrefix_context);
   if (FPPI_SUCCESS != errtype)
   {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
            "Registering for flush prefixes Failed\n");
      return (FPPI_FAILURE);
   }
   errtype = BENDAPI_register_callback (&fpm_ipv4_mgr_update_prefix, 
         &UpdateRouteCBHdl, 
         FPPI_IPv4_UniUpdatePrefix, 
         g_IPv4_UpdatePrefix_context);
   if (FPPI_SUCCESS != errtype)
   {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
            "Registering for update prefixes Failed\n");
      return (FPPI_FAILURE);
   }

   /*call backs for Arps*/
   errtype = BENDAPI_register_callback (&fpm_ipv4_mgr_add_arp, 
         &ARPAddEntryCBHdl, 
         FPPI_IPv4_UniAddToL2, 
         g_IPv4_AddArp_context);
   if (FPPI_SUCCESS != errtype)
   {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
            "Registering for Add ARP Entry Failed\n");
      return (FPPI_FAILURE);
   }
   errtype = BENDAPI_register_callback (&fpm_ipv4_mgr_delete_arp, 
         &ARPDelEntryCBHdl, 
         FPPI_IPv4_UniDelToL2, 
         g_IPv4_DelArp_context);
   if (FPPI_SUCCESS != errtype)
   {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
            "Registering for Delete ARP Entry Failed\n");
      return (FPPI_FAILURE);
   }

   errtype = BENDAPI_register_callback (&fpm_ipv4_mgr_flush_arps, 
         &ARPPurgeCBHdl, 
         FPPI_IPv4_UniFlushToL2, 
         g_IPv4_FlushArp_context);
   if (FPPI_SUCCESS != errtype)
   {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
            "Registering for Purge ARP Entry Failed\n");
      return (FPPI_FAILURE);
   }

   /*call backs for Next Hops*/
   errtype = BENDAPI_register_callback (&fpm_ipv4_mgr_add_next_hop, 
         &AddNHCBHdl, 
         FPPI_IPv4_UniAddNextHop, 
         g_IPv4_AddNexthopPrefix_context);
   if (FPPI_SUCCESS != errtype)
   {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
            "Registering for Add NH Entry Failed\n");
      return (FPPI_FAILURE);
   }

   errtype = BENDAPI_register_callback (&fpm_ipv4_mgr_query_next_hop, 
         &QueryNHCBHdl, 
         FPPI_IPv4_UniQueryNextHop, 
         g_IPv4_QueryNexthopPrefix_context);
   if (FPPI_SUCCESS != errtype)
   {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
            "Registering Query Next Hop Failed\n");
      return (FPPI_FAILURE);
   }

   errtype = BENDAPI_register_callback (&fpm_ipv4_mgr_delete_next_hop, 
         &DelNHCBHdl, 
         FPPI_IPv4_UniDelNextHop, 
         g_IPv4_DelNexthopPrefix_context);
   if (FPPI_SUCCESS != errtype)
   {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
            "Registering for Delete NH Entry Failed\n");
      return (FPPI_FAILURE);
   }
   errtype = BENDAPI_register_callback (&fpm_ipv4_mgr_flush_next_hops, 
         &PurgeNHCBHdl, 
         FPPI_IPv4_UniFlushNextHop, 
         g_IPv4_FlushNexthopPrefix_context);
   if (FPPI_SUCCESS != errtype)
   {
      npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
            "Registering for flush NH Failed\n");
      return (FPPI_FAILURE);
   }
   /* Call the FP blade SDK specifics for Initialization*/
   ipv4_InitFPEngine();

   return (0);
}

/*****************************************************************************
*
*   Function:   fpm_ipv4_mgr_shutdown()
*
*   Parameters: None 
*
*   Description: This function shuts-down the ipv4 manager. It deregisters for 
*                callbacks with the Back-end API, de-initializes the platform 
*				 specific component
*
*   Return: 0 on success, FPPAPI_FAILURE on failure 
*****************************************************************************/

int fpm_ipv4_mgr_shutdown ()
{
	int errtype;     
    npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Shutdown will happen here \n");

#ifndef _VXWORKS_
#ifndef IXASDKSTUBS
   sh_ipv4_finish();
#endif
#endif 

#ifdef IXASDKSTUBS
   ix_cc_ipv4_finiStub();
#endif	
	errtype =BENDAPI_deregister_callback (IPv4AddRouteCBHdl);
    if (FPPI_SUCCESS != errtype)
    {
        npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"De-registering for  Add Route Failed\n");
        //return (FPPI_FAILURE);
    }
	errtype = BENDAPI_deregister_callback (IPv4DelRouteCBHdl);    
    if (FPPI_SUCCESS != errtype)
    {
        npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"De-registering for Delete Route Failed\n");
        //return (FPPI_FAILURE);
    }
	errtype = BENDAPI_deregister_callback (PurgeRouteCBHdl);
    if (FPPI_SUCCESS != errtype)
    {
        npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"De-registering for Purge Route Failed\n");
        //return(FPPI_FAILURE);
    }
	errtype = BENDAPI_deregister_callback (ARPAddEntryCBHdl);
    if (FPPI_SUCCESS != errtype)
    {
        npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"De-registering for  ARP Add Entry Failed\n");
        //return(FPPI_FAILURE);
    }    
    errtype = BENDAPI_deregister_callback (ARPDelEntryCBHdl);
    if (FPPI_SUCCESS != errtype)
    {
        npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"De-registering for  ARP Delete Entry Failed\n");
        //return(FPPI_FAILURE);
    }
	errtype = BENDAPI_deregister_callback (ARPPurgeCBHdl);
    if (FPPI_SUCCESS != errtype)
    {
        npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"De-registering for  ARP Purge Failed\n");
        //return(FPPI_FAILURE);
    }
	/*errtype = BENDAPI_deregister_callback (ARPQueryCBHdl);
    if (FPPI_SUCCESS != errtype)
    {
        npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"De-registering for  ARP Query Failed\n");
        return(FPPI_FAILURE);
    }*/
	errtype = BENDAPI_deregister_callback (AddNHCBHdl);
    if (FPPI_SUCCESS != errtype)
    {
        npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"De-registering for add next_hop Failed\n");
        //return(FPPI_FAILURE);
    }
	errtype = BENDAPI_deregister_callback (DelNHCBHdl);
    if (FPPI_SUCCESS != errtype)
    {
        npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"De-registering for del next_hop Failed\n");
        //return(FPPI_FAILURE);
    }
    errtype = BENDAPI_deregister_callback (PurgeNHCBHdl);
    if (FPPI_SUCCESS != errtype)
    {
        npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"De-registering for purge next_hop Failed\n");
        //return(FPPI_FAILURE);
    }
	/* Call the FP blade SDK specifics for shutdown*/
	ipv4_DestroyFPEngine();

    return(FPPI_SUCCESS);
}


/****************************************************************************
*																			*
*																			*
*								  FP Module									*
*							IPV4 Manager EXTERNAL APIs						*
*																			*
*																			*
****************************************************************************/

/*****************************************************************************
*
*   Function: fpm_ipv4_mgr_add_route()
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data - 
*												npf_ipv4_UnicastPrefixList_t
*
*   Description: This callback function is registered with the Back-end API.
*                The Back-end API calls this function to add routes to the FE.
*
*
*   PRETURN : None 
*
*****************************************************************************/
void fpm_ipv4_mgr_add_prefix(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context, void *response_data)
{
	uint32_t i;
	ipv4_TranslatorContext_t *t_c = NULL;
	npf_ipv4_UnicastPrefixUpdateInfoList_t *prefixes = NULL;
	npf_IPv4_Status_t *status = NULL;
    
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Entering fpm_ipv4_mgr_add_route");

    if(context != g_IPv4_AddPrefix_context)
        return;

    prefixes = (npf_ipv4_UnicastPrefixUpdateInfoList_t *) response_data;
	
	/*Allocate the necessary memory for building the translator context*/
	t_c = (ipv4_TranslatorContext_t *)PIL_AllocateMemory
		(sizeof(ipv4_TranslatorContext_t), PIL_MEM_ZERO_INIT);
	status = (npf_IPv4_Status_t *)PIL_AllocateMemory(
		sizeof(npf_IPv4_Status_t) * prefixes->numEntries, PIL_MEM_ZERO_INIT);
	if(t_c == NULL || status == NULL)
	{
		npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"Memory allocation failed");
		goto finish;
	}

	/*Build the respective translator context*/
	t_c->cbCorrelator = cbcorrelator;
	t_c->cbType = FPPI_IPv4_UniAddPrefix;
	t_c->status = status;
	t_c->num_entries = prefixes->numEntries;
	
	/*set the fields of the status strucrure appropriately*/
	for(i = 0; i < prefixes->numEntries; ++i)
	{
		t_c->status[i].type = NPF_IPv4_ADDRESS;	
		t_c->status[i].retStatus = NPF_E_UNKNOWN;
		t_c->status[i].u.ipv4Addr.IPv4Addr = 
			prefixes->prefixInfo[i].prefixEntry.IPv4Addr;
		t_c->status[i].u.ipv4Addr.IPv4NetPlen = 
			prefixes->prefixInfo[i].prefixEntry.IPv4NetPlen;
	}

	/*Call into the platform specific component and wait for 
	callback on status of adding prefixes*/
	ipv4_add_prefix(prefixes, t_c);
	
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Leaving fpm_ipv4_mgr_add_route");

	return;

finish:
	if(t_c)	
		PIL_FreeMemory(t_c);
	if(status)	
		PIL_FreeMemory(status);
	
	BENDAPI_Report_Status(FPPI_FAILURE, 
                        FPPI_IPv4_UniAddPrefix, 
                        cbcorrelator,
                        NULL);	
}


/*****************************************************************************
*
*   Function: fpm_ipv4_mgr_delete_prefix()
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data - 
*												npf_ipv4_UnicastPrefixList_t
*
*   Description: This callback function is registered with the Back-end API.
*                The Back-end API calls this function to deletes routes.
*
*
*   PRETURN : None 
*
*****************************************************************************/
void fpm_ipv4_mgr_delete_prefix(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context, void *response_data)
{
	uint32_t i;
	ipv4_TranslatorContext_t *t_c = NULL;
	npf_ipv4_UnicastPrefixList_t *prefixes = NULL;
	npf_IPv4_Status_t *status = NULL;
    
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE,
		"Entering fpm_ipv4_mgr_delete_prefix");

    if(context != g_IPv4_DelPrefix_context)
        return;

    prefixes = (npf_ipv4_UnicastPrefixList_t *) response_data;
	
	/*Allocate the necessary memory for building the translator context*/
	t_c = (ipv4_TranslatorContext_t *)PIL_AllocateMemory
		(sizeof(ipv4_TranslatorContext_t), PIL_MEM_ZERO_INIT);
	status = (npf_IPv4_Status_t *)PIL_AllocateMemory(
		sizeof(npf_IPv4_Status_t) * prefixes->numEntries, PIL_MEM_ZERO_INIT);
	if(t_c == NULL || status == NULL)
	{
		npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"Memory allocation failed");
		goto finish;
	}

	/*Build the respective translator context*/
	t_c->cbCorrelator = cbcorrelator;
	t_c->cbType = FPPI_IPv4_UniDelPrefix;
	t_c->status = status;
	t_c->num_entries = prefixes->numEntries;

	/*set the fields of the status structure appropriately*/
	for(i = 0; i < prefixes->numEntries; ++i)
	{
		t_c->status[i].type = NPF_IPv4_ADDRESS;	
		t_c->status[i].retStatus = NPF_E_UNKNOWN;
		t_c->status[i].u.ipv4Addr.IPv4Addr = 
			prefixes->prfxInfo[i].IPv4Addr;
		t_c->status[i].u.ipv4Addr.IPv4NetPlen = 
			prefixes->prfxInfo[i].IPv4NetPlen;
	}
	/*Call into the platform specific component and wait for 
	callback on status of deleting prefixes*/
	ipv4_delete_prefix(prefixes, t_c);
	
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, 
		"Leaving fpm_ipv4_mgr_delete_prefix");

	return;

finish:
	if(t_c)	
		PIL_FreeMemory(t_c);
	if(status)	
		PIL_FreeMemory(status);
	
	BENDAPI_Report_Status(FPPI_FAILURE, 
                        FPPI_IPv4_UniDelPrefix, 
                        cbcorrelator,
                        NULL);	
}

/*****************************************************************************
*
*   Function: fpm_ipv4_mgr_update_prefix()
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data - 
*												npf_ipv4_UnicastPrefixList_t
*
*   Description: This callback function is registered with the Back-end API.
*                The Back-end API calls this function to update routes to 
*				 the FE for the same NHIDs
*
*
*   PRETURN : None 
*
*****************************************************************************/
void fpm_ipv4_mgr_update_prefix(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context, void *response_data)
{
	uint32_t i;
	ipv4_TranslatorContext_t *t_c = NULL;
	npf_ipv4_UnicastPrefixList_t *prefixes = NULL;
	npf_IPv4_Status_t *status = NULL;
    
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, 
		"Entering fpm_ipv4_mgr_update_prefix");

    if(context != g_IPv4_UpdatePrefix_context)
        return;

    prefixes = (npf_ipv4_UnicastPrefixList_t *) response_data;
	
	/*Allocate the necessary memory for building the translator context*/
	t_c = (ipv4_TranslatorContext_t *)PIL_AllocateMemory
		(sizeof(ipv4_TranslatorContext_t), PIL_MEM_ZERO_INIT);
	status = (npf_IPv4_Status_t *)PIL_AllocateMemory(
		sizeof(npf_IPv4_Status_t) * prefixes->numEntries, PIL_MEM_ZERO_INIT);
	if(t_c == NULL || status == NULL)
	{
		npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"Memory allocation failed");
		goto finish;
	}

	/*Build the respective translator context*/
	t_c->cbCorrelator = cbcorrelator;
	t_c->cbType = FPPI_IPv4_UniUpdatePrefix;
	t_c->status = status;
	t_c->num_entries = prefixes->numEntries;

	/*set the fields of the status structure appropriately*/
	for(i = 0; i < prefixes->numEntries; ++i)
	{
		t_c->status[i].type = NPF_IPv4_ADDRESS;	
		t_c->status[i].retStatus = NPF_E_UNKNOWN;
		t_c->status[i].u.ipv4Addr.IPv4Addr = 
			prefixes->prfxInfo[i].IPv4Addr;
		t_c->status[i].u.ipv4Addr.IPv4NetPlen = 
			prefixes->prfxInfo[i].IPv4NetPlen;
	}
	/*Call into the platform specific component and wait for 
	callback on status of adding prefixes*/
	ipv4_update_prefix(prefixes, t_c);
	
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, 
		"Leaving fpm_ipv4_mgr_update_prefix");

	return;

finish:
	if(t_c)	
		PIL_FreeMemory(t_c);
	if(status)	
		PIL_FreeMemory(status);
	
	BENDAPI_Report_Status(FPPI_FAILURE, 
                        FPPI_IPv4_UniUpdatePrefix, 
                        cbcorrelator,
                        NULL);	
}

/*****************************************************************************
*
*   Function: fpm_ipv4_mgr_add_next_hop()
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data - 
*												FPPI_UnicastNextHopUpdateInfo_t
*
*   Description: This callback function is registered with the Back-end API.
*                The Back-end API calls this function to add next hop to 
*				 the FE.
*
*
*   PRETURN : None 
*
*****************************************************************************/
void fpm_ipv4_mgr_add_next_hop(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context, void *response_data)
{
	uint32_t i;
	ipv4_TranslatorContext_t *t_c = NULL;
	npf_ipv4_UnicastNextHopUpdateInfoList_t *nexthops = NULL;
	npf_IPv4_Status_t *status = NULL;
    
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, 
		"Entering fpm_ipv4_mgr_add_next_hop");

    if(context != g_IPv4_AddNexthopPrefix_context)
        return;

    nexthops = (npf_ipv4_UnicastNextHopUpdateInfoList_t *) response_data;
	
	/*Allocate the necessary memory for building the translator context*/
	t_c = (ipv4_TranslatorContext_t *)PIL_AllocateMemory
		(sizeof(ipv4_TranslatorContext_t), PIL_MEM_ZERO_INIT);
	status = (npf_IPv4_Status_t *)PIL_AllocateMemory(
		sizeof(npf_IPv4_Status_t) * nexthops->numEntries, PIL_MEM_ZERO_INIT);
	if(t_c == NULL || status == NULL)
	{
		npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"Memory allocation failed");
		goto finish;
	}

	/*Build the respective translator context*/
	t_c->cbCorrelator = cbcorrelator;
	t_c->cbType = FPPI_IPv4_UniAddNextHop;
	t_c->status = status;
	t_c->num_entries = nexthops->numEntries;

	/*set the fields of the status structure appropriately*/
	for(i = 0; i < nexthops->numEntries; ++i)
	{
		t_c->status[i].type = NPF_IPv4_ADDRESS;	
		t_c->status[i].retStatus = NPF_E_UNKNOWN;
      if(nexthops->nhUpdateInfo[i].nhInfo)
		   t_c->status[i].u.ipv4Addr.IPv4Addr = nexthops->nhUpdateInfo[i].nhInfo->nhAddr;
		//t_c->status[i].u.ipv4Addr.IPv4NetPlen = 0xffffffff;
		t_c->status[i].u.ipv4Addr.IPv4NetPlen = 0xff;
			
	}
	/*Call into the platform specific component and wait for 
	callback on status of adding next hops*/
	ipv4_add_next_hop(nexthops, t_c);
	
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, 
		"Leaving fpm_ipv4_mgr_add_next_hop");

	return;

finish:
	if(t_c)	
		PIL_FreeMemory(t_c);
	if(status)	
		PIL_FreeMemory(status);
	
	BENDAPI_Report_Status(FPPI_FAILURE, 
                        FPPI_IPv4_UniAddNextHop, 
                        cbcorrelator,
                        NULL);	
}


/*****************************************************************************
*
*   Function: fpm_ipv4_mgr_delete_next_hop()
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data - 
*												npf_ipv4_UnicastNextHopList_t
*
*   Description: This callback function is registered with the Back-end API.
*                The Back-end API calls this function to deletes next hops.
*
*
*   PRETURN : None 
*
*****************************************************************************/
void fpm_ipv4_mgr_delete_next_hop(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context, void *response_data)
{
	uint32_t i;
	ipv4_TranslatorContext_t *t_c = NULL;
	npf_ipv4_UnicastNextHopDeleteInfoList_t *nexthops = NULL;
	npf_IPv4_Status_t *status = NULL;
    
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE,
		"Entering fpm_ipv4_mgr_delete_next_hop");

    if(context != g_IPv4_DelNexthopPrefix_context)
        return;

    nexthops = (npf_ipv4_UnicastNextHopDeleteInfoList_t *) response_data;
	
	/*Allocate the necessary memory for building the translator context*/
	t_c = (ipv4_TranslatorContext_t *)PIL_AllocateMemory
		(sizeof(ipv4_TranslatorContext_t), PIL_MEM_ZERO_INIT);
	status = (npf_IPv4_Status_t *)PIL_AllocateMemory(
		sizeof(npf_IPv4_Status_t) * nexthops->numEntries, PIL_MEM_ZERO_INIT);
	if(t_c == NULL || status == NULL)
	{
		npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"Memory allocation failed");
		goto finish;
	}

	/*Build the respective translator context*/
	t_c->cbCorrelator = cbcorrelator;
	t_c->cbType = FPPI_IPv4_UniDelNextHop;
	t_c->status = status;
	t_c->num_entries = nexthops->numEntries;

	/*set the fields of the status structure appropriately*/
	for(i = 0; i < nexthops->numEntries; ++i)
	{
		t_c->status[i].type = NPF_IPv4_ADDRESS;	
		t_c->status[i].retStatus = NPF_E_UNKNOWN;
		t_c->status[i].u.ipv4Addr.IPv4Addr = nexthops->nhDeleteInfo[i].nhInfo.nhAddr;
		//t_c->status[i].u.ipv4Addr.IPv4NetPlen = 0xffffffff;
		t_c->status[i].u.ipv4Addr.IPv4NetPlen = 0xff;
			
	}
	/*Call into the platform specific component and wait for 
	callback on status of deleteing nexthops*/
	ipv4_delete_next_hop(nexthops, t_c);
	
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, 
		"Leaving fpm_ipv4_mgr_delete_next_hop");

	return;

finish:
	if(t_c)	
		PIL_FreeMemory(t_c);
	if(status)	
		PIL_FreeMemory(status);
	
	BENDAPI_Report_Status(FPPI_FAILURE, 
                        FPPI_IPv4_UniDelNextHop, 
                        cbcorrelator,
                        NULL);	
}

/*****************************************************************************
*
*   Function: fpm_ipv4_mgr_add_arp()
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data - 
*												npf_ipv4_UnicastToL2List_t
*
*   Description: This callback function is registered with the Back-end API.
*                The Back-end API calls this function to add arps to the FE.
*
*
*   PRETURN : None 
*
*****************************************************************************/
void fpm_ipv4_mgr_add_arp(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context, void *response_data)
{
	uint32_t i;
	ipv4_TranslatorContext_t *t_c = NULL;
	npf_ipv4_UnicastToL2List_t *arpentry = NULL;
	npf_IPv4_Status_t *status = NULL;
    
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Entering fpm_ipv4_mgr_add_arp");

    if(context != g_IPv4_AddArp_context)
        return;

    arpentry = (npf_ipv4_UnicastToL2List_t *) response_data;
	
	/*Allocate the necessary memory for building the translator context*/
	t_c = (ipv4_TranslatorContext_t *)PIL_AllocateMemory
		(sizeof(ipv4_TranslatorContext_t), PIL_MEM_ZERO_INIT);
	status = (npf_IPv4_Status_t *)PIL_AllocateMemory(
		sizeof(npf_IPv4_Status_t) * arpentry->numEntries, PIL_MEM_ZERO_INIT);
	if(t_c == NULL || status == NULL)
	{
		npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"Memory allocation failed");
		goto finish;
	}

	/*Build the respective translator context*/
	t_c->cbCorrelator = cbcorrelator;
	t_c->cbType = FPPI_IPv4_UniAddToL2;
	t_c->status = status;
	t_c->num_entries = arpentry->numEntries;

	/*set the fields of the status structure appropriately*/
	for(i = 0; i < arpentry->numEntries; ++i)
	{
		t_c->status[i].type = NPF_L2_ADDRESS;	
		t_c->status[i].retStatus = NPF_E_UNKNOWN;
		t_c->status[i].u.l2Addr.type = NPF_MAC_ADDRESS;
		memcpy(t_c->status[i].u.l2Addr.u.MAC_address, 
			arpentry->toL2Info[i].l2Entry.mediaAddress.u.MAC_Address, 6);
	}
	/*Call into the platform specific component and wait for 
	callback on status of adding prefixes*/
	ipv4_add_arp(arpentry, t_c);
	
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, "Leaving fpm_ipv4_mgr_add_arp");

	return;

finish:
	if(t_c)	
		PIL_FreeMemory(t_c);
	if(status)	
		PIL_FreeMemory(status);
	
	BENDAPI_Report_Status(FPPI_FAILURE, 
                        FPPI_IPv4_UniAddToL2, 
                        cbcorrelator,
                        NULL);	
}

/*****************************************************************************
*
*   Function: fpm_ipv4_mgr_delete_arp()
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data - 
*												npf_ipv4_UnicastToL2List_t
*
*   Description: This callback function is registered with the Back-end API.
*                The Back-end API calls this function to deletes arps.
*
*
*   PRETURN : None 
*
*****************************************************************************/
void fpm_ipv4_mgr_delete_arp(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context, void *response_data)
{
	uint32_t i;
	ipv4_TranslatorContext_t *t_c = NULL;
	npf_ipv4_UnicastToL2List_t *arpentry = NULL;
	npf_IPv4_Status_t *status = NULL;
    
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE,
		"Entering fpm_ipv4_mgr_delete_arp");

    if(context != g_IPv4_DelArp_context)
        return;

    arpentry = (npf_ipv4_UnicastToL2List_t *) response_data;
	
	/*Allocate the necessary memory for building the translator context*/
	t_c = (ipv4_TranslatorContext_t *)PIL_AllocateMemory
		(sizeof(ipv4_TranslatorContext_t), PIL_MEM_ZERO_INIT);
	status = (npf_IPv4_Status_t *)PIL_AllocateMemory(
		sizeof(npf_IPv4_Status_t) * arpentry->numEntries, PIL_MEM_ZERO_INIT);
	if(t_c == NULL || status == NULL)
	{
		npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"Memory allocation failed");
		goto finish;
	}

	/*Build the respective translator context*/
	t_c->cbCorrelator = cbcorrelator;
	t_c->cbType = FPPI_IPv4_UniDelToL2;
	t_c->status = status;
	t_c->num_entries = arpentry->numEntries;

	/*set the fields of the status structure appropriately*/
	for(i = 0; i < arpentry->numEntries; ++i)
	{
		t_c->status[i].type = NPF_L2_ADDRESS;	
		t_c->status[i].retStatus = NPF_E_UNKNOWN;
		t_c->status[i].u.l2Addr.type = NPF_MAC_ADDRESS;
		memcpy(t_c->status[i].u.l2Addr.u.MAC_address, 
			arpentry->toL2Info[i].l2Entry.mediaAddress.u.MAC_Address, 6);
	}
	/*Call into the platform specific component and wait for 
	callback on status of deleting prefixes*/
	ipv4_delete_arp(arpentry, t_c);
	
	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, 
		"Leaving fpm_ipv4_mgr_delete_arp");

	return;

finish:
	if(t_c)	
		PIL_FreeMemory(t_c);
	if(status)	
		PIL_FreeMemory(status);
	
	BENDAPI_Report_Status(FPPI_FAILURE, 
                        FPPI_IPv4_UniDelToL2, 
                        cbcorrelator,
                        NULL);	
}


/*****************************************************************************
*
*   Function: fpm_ipv4_mgr_flush_prefixes()
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data - NULL
*
*   Description: This callback function is registered with the Back-end API.
*                The Back-end API calls this function to purge routes from 
*                the FE.
*
*   PRETURN : None 
*
*****************************************************************************/
void fpm_ipv4_mgr_flush_prefixes(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context,
		      void *response_data)
{
	ipv4_TranslatorContext_t *t_c;
	npf_IPv4_Status_t *status = NULL;

	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, 
		"fpm_ipv4_mgr_flush_prefixes");

    if(context != g_IPv4_FlushPrefix_context)
        return;

	t_c = (ipv4_TranslatorContext_t *)PIL_AllocateMemory
		(sizeof(ipv4_TranslatorContext_t), PIL_MEM_ZERO_INIT);
	status = (npf_IPv4_Status_t *)PIL_AllocateMemory(
		sizeof(npf_IPv4_Status_t), PIL_MEM_ZERO_INIT);
	if(t_c == NULL || status == NULL)
	{
		npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"Memory allocation failed");
		goto finish;
	}


	t_c->cbCorrelator = cbcorrelator;    
	t_c->cbType = FPPI_IPv4_UniFlushPrefix;
   t_c->status = status;
	t_c->num_entries = 1;

	/*set the fields of the status structure appropriately*/
   t_c->status->retStatus = NPF_E_UNKNOWN;

	ipv4_flush_prefixes(t_c);

	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, 
		"fpm_ipv4_mgr_flush_prefixes");

   return;

finish:
	if(t_c)	
		PIL_FreeMemory(t_c);
	if(status)	
		PIL_FreeMemory(status);
	
	BENDAPI_Report_Status(FPPI_FAILURE, 
                        FPPI_IPv4_UniFlushPrefix, 
                        cbcorrelator,
                        NULL);	
}

/*****************************************************************************
*
*   Function: fpm_ipv4_mgr_flush_next_hops()
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data - NULL
*
*   Description: This callback function is registered with the Back-end API.
*                The Back-end API calls this function to purge routes and 
*				 next hops
*
*   PRETURN : None 
*
*****************************************************************************/
void fpm_ipv4_mgr_flush_next_hops(FPPI_CBCORRELATOR cbcorrelator,
				  FPPI_CONTEXT context,
				  void *response_data)
{
  ipv4_TranslatorContext_t *t_c;
  npf_ipv4_UnicastNextHopDeleteInfoList_t* nexthops = NULL;
  
  npf_logger_write(LOG_FP_IPV4, LOG_TRACE, 
		   "fpm_ipv4_mgr_flush_next_hops");
  
  if(context != g_IPv4_FlushNexthopPrefix_context)
    return;
#if USE_FPM_CC
  nexthops = (npf_ipv4_UnicastNextHopDeleteInfoList_t*) response_data;
#endif

  t_c = (ipv4_TranslatorContext_t *)
    PIL_AllocateMemory
    (sizeof(ipv4_TranslatorContext_t), PIL_MEM_ZERO_INIT);
  
  t_c->cbCorrelator = cbcorrelator;    
  t_c->cbType = FPPI_IPv4_UniFlushNextHop;

#if USE_FPM_CC
  ipv4_flush_next_hops(nexthops, t_c);
#endif
  
  npf_logger_write(LOG_FP_IPV4, LOG_TRACE, 
		   "fpm_ipv4_mgr_flush_next_hops");
}

/*****************************************************************************
*
*   Function: fpm_ipv4_mgr_flush_arps()
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data - NULL
*
*   Description: This callback function is registered with the Back-end API.
*                The Back-end API calls this function to purge arps from 
*                the FE.
*
*   PRETURN : None 
*
*****************************************************************************/
void fpm_ipv4_mgr_flush_arps(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context,
		      void *response_data)
{
	ipv4_TranslatorContext_t *t_c;

	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, 
		"fpm_ipv4_mgr_flush_arps");

    if(context != g_IPv4_FlushArp_context)
        return;

	t_c = (ipv4_TranslatorContext_t *)PIL_AllocateMemory
		(sizeof(ipv4_TranslatorContext_t), PIL_MEM_ZERO_INIT);
	
	t_c->cbCorrelator = cbcorrelator;    
	t_c->cbType = FPPI_IPv4_UniFlushToL2;
	ipv4_flush_arps(t_c);

	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, 
		"fpm_ipv4_mgr_flush_arps");
}

/******************************************************************************
*******************************************************************************
*								Translator UP-CALLS							  *
*																			  *
*******************************************************************************
******************************************************************************/

/*****************************************************************************
*
*   Function:   fpm_ipv4_mgr_upcall
*
*   Parameters: 
*				result: Result of the down call request
*				t_c: Translator context containing status on adding/deleting
*				prefixes/nh/arps
*   Description:
*				This upcall function is called the the platform specific layer, 
*				to the status on adding/deleting prefixes/nh/arps
*
*   Return: 0 on success, -1 on failure 
*****************************************************************************/
int32_t fpm_ipv4_mgr_upcall(int32_t result, ipv4_TranslatorContext_t *t_c)
{
   uint32_t i;
	npf_IPv4EntryStatusList statusList;

	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, 
		"Entering fpm_ipv4_mgr_upcall");

	if(!t_c)
	{
		npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			"t_c is invalid");
		return -1;
	}

	/*package stats info for the Backend API to send it up*/
	statusList.numEntries = t_c->num_entries;
	statusList.entryList = t_c->status;

   switch(t_c->cbType)
   {
      case FPPI_IPv4_UniQueryNextHop:
      case FPPI_IPv4_UniQueryPrefix :
	   case FPPI_IPv4_UniFlushPrefix :
	   case FPPI_IPv4_UniFlushNextHop:
	   case FPPI_IPv4_UniFlushToL2:
      {
	      if(result == -1)
	      {
	         if (BENDAPI_Report_Status( FPPI_FAILURE, t_c->cbType, 
	   	      t_c->cbCorrelator,(void *)&t_c->cbData) != FPPI_SUCCESS)
	         {		
	           	npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			       "Sending Report Failed");
	         }
         }
         else /* Success */
         {
             /* All new API additions support NPF Callback DataStruc */ 
	         if (BENDAPI_Report_Status( FPPI_SUCCESS, t_c->cbType, 
			       t_c->cbCorrelator,(void *)&t_c->cbData) != FPPI_SUCCESS)
	         {		
			        npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
				         "Sending Report Failed");
            }

         }
      }
      break;
      default:
      { 
		   if (BENDAPI_Report_Status( FPPI_FAILURE, t_c->cbType, 
		 	   t_c->cbCorrelator,(void *)&statusList ) != FPPI_SUCCESS)
		   {		
		    	npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
				                     "Sending Report Failed");
		   }
         else
         {
            if (BENDAPI_Report_Status( FPPI_SUCCESS, t_c->cbType, 
			      t_c->cbCorrelator,(void *)&statusList ) != FPPI_SUCCESS)
		      {		
			       npf_logger_write(LOG_FP_IPV4, LOG_ERR, 
			        "Sending Report Failed");
		      }
         }
      }
   } /* end of switch */
	
   /*Free the user context resources for the the related down-call*/
   /*** Free the memory after giving it up ***/
   switch(t_c->cbType)
   {
      case FPPI_IPv4_UniQueryPrefix: 
      case FPPI_IPv4_UniFlushPrefix:
      case FPPI_IPv4_UniFlushNextHop:
      {
         if(t_c->cbData.resp != NULL)
         {
            PIL_FreeMemory(t_c->cbData.resp);
         }
      }
      break;
      case  FPPI_IPv4_UniQueryNextHop:  
      {
         if(t_c->cbData.resp != NULL)
         {
            for(i=0; i< t_c->cbData.numResp; i++)
            {
               PIL_FreeMemory(t_c->cbData.resp[i].u.nextHopQueryResult.nextHopArray.nextHopArray);
            }
            PIL_FreeMemory(t_c->cbData.resp);
         }
       }
       break;
       default:
       break;
   }
	/*free resources related to this t_c*/
   if(t_c->status)
	   PIL_FreeMemory(t_c->status);
	PIL_FreeMemory(t_c);

	npf_logger_write(LOG_FP_IPV4, LOG_TRACE, 
		"Leaving fpm_ipv4_mgr_upcall");

	return 0;
}


#if 0
/*****************************************************************************
*
*   Function: fpm_ipv4_mgr_query_arp()
*
*   Parameters: cbcorrelator - CallBack Correlator
*               context - CallBack Context
*               response_data - Callback response data - npf_ipv4_ARPKeyList_t 
*
*   Description: This callback function is registered with the Back-end API.
*                The Back-end API calls this function to query for ARP Entries
*                from the FE.
*
*   PRETURN : None 
*
*****************************************************************************/
void fpm_ipv4_mgr_query_arp(  FPPI_CBCORRELATOR cbcorrelator,            
                        FPPI_CONTEXT context,
                        void *response_data)
{
    int i;
    ARPQreturn  result;
    int flag = 0;
    npf_ipv4_ARPKeyList_t* arpKeyList;
    npf_ipv4_ARPList_t arpList;

    PDBG_TRACE ();
    if(context != g_ARP_Query_context)
        PRETURN (/*void*/);
    arpKeyList = (npf_ipv4_ARPKeyList_t *) response_data;
    arpList.arp_count = arpKeyList->key_count;
    arpList.arp_list = (npf_ipv4_ARPEntry_t *)
	malloc (arpList.arp_count * sizeof (npf_ipv4_ARPEntry_t));
    PDBG_PRINT ( "arplist addr 0x%x\n", arpList.arp_list );
    for (i = 0; i < arpKeyList->key_count; i++)
    {
        arpList.arp_list[i].key.ip_addr = arpKeyList->key_list[i].ip_addr;
        result =  QueryforMACAddr(arpKeyList->key_list[i].ip_addr);
        memcpy (arpList.arp_list[i].data.mac_addr, result.hwaddr, 6);
        PDBG_PRINT ("The MAC Addr is %x:%x:%x:%x:%x:%x, result is %d\n", 
		    arpList.arp_list[i].data.mac_addr[0], 
		    arpList.arp_list[i].data.mac_addr[1], 
		    arpList.arp_list[i].data.mac_addr[2], 
		    arpList.arp_list[i].data.mac_addr[3], 
		    arpList.arp_list[i].data.mac_addr[4], 
		    arpList.arp_list[i].data.mac_addr[4], 
		    result.value);
        arpList.arp_list[i].data.lint_number = 0; /*ACE does not support this*/
        arpList.arp_list[i].data.aging = 0;
        if (result.value == -1)
	    flag = 1;
        PDBG_PRINT ( "arplist addr 0x%x\n", arpList.arp_list );
    }
    PDBG_PRINT ( "arplist addr 0x%x\n", arpList.arp_list );
    if (flag == 1)
        BENDAPI_Report_Status (FPPI_FAILURE, 
                               FPPI_ARP_QueryEntry, 
                               cbcorrelator, 
                               NULL);
    else     
        BENDAPI_Report_Status (FPPI_SUCCESS, 
                               FPPI_ARP_QueryEntry, 
                               cbcorrelator, 
                               (void *) &arpList );
    PDBG_PRINT ( "arplist addr 0x%x\n", arpList.arp_list );
    free (arpList.arp_list);

    PRETURN (/*void*/);
}

#endif

