/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2002 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ForwardingPlane/FPModule/IXASDKStubs/inc/ixasdk_stubs_pri.h,v $
 * $Revision: 1.6 $
 * $Date: 2003/11/17 16:57:22 $
 *
 ****************************************************************************/

#ifndef IXA_STUBS_PRI_HDR
#define IXA_STUBS_PRI_HDR

#ifdef __cplusplus
extern "C" {
#endif
    
#include "list.h" 
#include "pil.h"
#include "fpp_common.h"
#include "ix_error.h"

/*******************************************************************************
// Structure containing ptrs to event queue and supporting elements
// for inter-thread communication.
*******************************************************************************/

typedef struct tag_QUEUE_SERVICE_PARAMS
{   
    DList               msgQueue;
    PilCriticalSection  queSyncObj;
    PilThread           thrdHandle;
    PilEvent            newMsgNotifier;
    PilEvent            thrdUpNotifier;
    FPPI_RET            retVal;
} QUEUE_SERVICE_PARAMS, * LPQUEUE_SERVICE_PARAMS;


/*******************************************************************************
// Define event types for event loop
*******************************************************************************/

#define STUBS_EVENT_SHUTDOWN           0
#define STUBS_IPv4_ROUTE_OP            1
#define STUBS_IPv4_ROUTE_DUMP          2
#define STUBS_IPv4_NEXT_HOP            3
#define STUBS_IPv4_SLEEP_TIME          4
#define STUBS_IPv4_QUEUE_DEPTH         5
#define STUBS_IPv4_PKT_DRAIN           6
#define STUBS_IPv4_GET_STATS           7
#define STUBS_IPv4_LOOKUP              8
#define STUBS_STKDRV_NUM_PORTS         9
#define STUBS_STKDRV_GET_PROPERTIES   10
#define STUBS_ARP_OP                  11
#define STUBS_L2_ATM_POS_TX_STATS     12
#define STUBS_POS_RX_STATS            13
#define STUBS_IPv6_ROUTE_OP           14
#define STUBS_IPV6TM_ADD_OP           15
#define STUBS_ATM_GET_LINK_ATTRIBUTE  16
#define STUBS_ATM_SET_LINK_ATTRIBUTE  17
#define STUBS_ATM_GET_CONN_STATISTICS 18
#define STUBS_ATM_SET_CONN_STATISTICS_IND 19
#define STUBS_ATM_SET_CONN_TRAFFIC_IND    20
#define STUBS_ATM_GET_OAM_STATISTICS      21
#define STUBS_ATM_SET_OAM_CC              22
#define STUBS_ATM_GET_OAM_PROPERTIES      23
#define STUBS_ATM_CONNECTION_SETUP        24
#define STUBS_ATM_CONNECTION_DEL          25
#define STUBS_ATM_SET_AAL5_MUX            26
#define STUBS_ATM_SET_AAL2_CHANNEL        27
#define STUBS_ATM_DEL_AAL2_CHANNEL        28
#define STUBS_ATM_SET_XC_CONNECTION       29
#define STUBS_ATM_DEL_XC_CONNECTION       30
#define STUBS_ATM_DATA_REQ                31
#define STUBS_IPV6TM_DEL_OP               32
#define STUBS_L2_ETH_TX_STATS             33
#define STUBS_L2_OP                       34
#define STUBS_ETH_RX_STATS                35
#define STUBS_CM_MSG_TO_CP             100

/*******************************************************************************
// Structures to wrap params before inserting into event queue
*******************************************************************************/

/* Just the event type for generic use */
typedef struct tag_STUBS_PARAM_TYPE
{
    uint32_t evType;
} STUBS_PARAM_TYPE, * LPSTUBS_PARAM_TYPE;

/* For general message passing. This is the definition of the object that
   msgs are packed into for subsequent stuffing into the queues. Note that
   the correlator passed into the FP is a ptr to a METHOD_INFO obj. This
   allows the STUBS to find the correct correlator and cbhandle when the FP
   calls back into the STUBS for the async callback (the methInfo list is
   search for that address). */
typedef struct tag_STUBS_GENERAL_EVENT_MSG
{
    uint32_t  evType;
    void*     cbMethod;
    void*     context;
    ix_error  ixReturn;
} STUBS_GENERAL_EVENT_MSG, * LPSTUBS_GENERAL_EVENT_MSG;

/*******************************************************************************
// Method prototypes
*******************************************************************************/

NPF_RET InsertMethodIntoList(
    uint32_t methodID,
    uint32_t eventType,
    void*    arg_Callback,
    void*    arg_pUserContext);

void SetIXAReturnObject(
    ix_error* retObj);

void DelayRandomTime();


#ifdef __cplusplus
}
#endif

#endif /* IXA_STUBS_PRI_HDR */

