/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ForwardingPlane/FPModule/IXASDKStubs/src/ixasdk_binding_stubs.c,v $
 * $Revision: 1.6.2.1 $
 * $Date: 2004/03/17 09:52:06 $
 *
 * Description:
 *      Implements method to create event loops in new threads. The thread
 *      method is also contained here. The event threads wait for and
 *      processes messages from the CP and FP and then call the appropriate
 *      methods in the FP or CP.
 *
 ****************************************************************************/

#include "list.h"
#include "log.h"
#include "ixasdk_methodmap.h"
#include "ixasdk_stubs_pri.h"
#include "cci/cc.h"
#include "ix_cc.h"
#include "cc/ix_cc_rtmv4.h"
#include "cc/ix_cc_ipv4.h"
#include "cc/ix_cc_stkdrv_common.h"
#include "cc/ix_cc_stkdrv_pktclass.h"
#include "cc/ix_cc_stkdrv.h"
#ifdef _INCLUDE_ATM
#include "cc/ix_cc_atm_pos_tx.h"
#endif
#ifdef _INCLUDE_DIFFSERV
#include "cc/ix_cc_pos_rx.h"
#endif
#include "cc/ix_cc_fpm_core.h"
//#include "cc/ix_cc_eth_tx.h"
//#include "cc/ix_cc_l2tm.h"
//#include "cc/ix_cc_arp.h"

extern QUEUE_SERVICE_PARAMS g_stubsParms;

/**
 * NAME: ix_cc_stkdrv_async_get_num_ports
 *
 * DESCRIPTION: This function is called by the FP Module to get the number of
 * ports configured on the Stack Driver.
 * 
 * @Param:  - IN ix_cc_stkdrv_cb_get_num_ports arg_pCallback - pointer to the
 *          client's callback function to be invoked once the message returns.
 * @Param:  - IN void* arg_pContext - pointer to client context.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_OOM_SYSTEM
 */

ix_error ix_cc_stkdrv_async_get_num_ports(
    ix_cc_stkdrv_cb_get_num_ports arg_pCallback,
    void* arg_pContext)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);

    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_pCallback : 0x%x
            \n\targ_pContext : 0x%x
            \n",
            __FUNCTION__,
            arg_pCallback,
            arg_pContext
            );

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_stkdrv_async_get_num_ports,
            STUBS_STKDRV_NUM_PORTS,
            arg_pCallback,
            arg_pContext);

    }

    return retObj;
}

/**
 * NAME: ix_cc_stkdrv_async_get_property
 *
 * DESCRIPTION: This function is called by the FP Module to get the interface
 *              properties of a specific port.
 * 
 * @Param:  - IN ix_cc_stkdrv_cb_get_property arg_pCallback - pointer to the
 *          client's callback function to be invoked once the message returns.
 * @Param:  - IN void* arg_pContext - pointer to client context.
 * @Param:  - IN ix_uint32 arg_propId - property IDs for the properties
 *          being requested.
 * @Param:  - IN ix_uint32 arg_portId - ID of port being queried. Use 
 *          IX_CC_GET_ALL_PORTS to get all port properties.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_OOM_SYSTEM
 */
ix_error ix_cc_stkdrv_async_get_property(
    ix_cc_stkdrv_cb_get_property arg_pCallback,
    void* arg_pContext,
    ix_uint32 arg_propId,
    ix_uint32 arg_portId)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_pCallback : 0x%x
            \n\targ_pContext : 0x%x
            \n\targ_propId : 0x%x
            \n\targ_portId : 0x%x
            \n",
            __FUNCTION__,
            arg_pCallback,
            arg_pContext,
            arg_propId,
            arg_portId
            );

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_stkdrv_async_get_property,
            STUBS_STKDRV_GET_PROPERTIES,
            arg_pCallback,
            arg_pContext);

    }

    return retObj;
}

/**
 * NAME: ix_cc_async_set_property
 *
 * DESCRIPTION: This function sends update messages (to the registered clients
 * who are interested in the respective property changes, for the properties
 * of which the Stack Driver is a master. If this function is called for
 * properties of which the Stack Driver is not a master, it will return an error.
 * Note that although this is an asynchronous function, no callback will be
 * provided to indicate success or error of the actual property updates.
 * 
 * @Param:  - IN ix_uint32 arg_PropId - id of the property to set.
 * @Param:  - IN ix_cc_properties* arg_pProperty - pointer to
 *          ix_cc_properties data structure.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_OOM_SYSTEM
 */
ix_error ix_cc_async_set_property(
				  ix_uint32 arg_PropId,
				  ix_cc_properties* arg_pProperty
				  )
{
    ix_error retObj = IX_SUCCESS;

    /* IXA SDK says no callback for this method */
#ifndef SQESTUBS
    npf_logger_write(LOG_IXA_STUBS, LOG_TRACE,
        "Entered %s .\n",
        StubsMethodMap(Method_ix_cc_async_set_property));

#else /* SQESTUBS */
    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s
        \n\targ_PropId : 0x%x
        \n\targ_pProperty : 0x%x
        \n\t\targ_pProperty->blade_id : %d
        \n\t\targ_pProperty->port_id : %d 
        \n\t\targ_pProperty->ip_version : %d 
        \n\t\t\targ_pProperty->ip_prop.ipv4_address : 0x%x
        \n\t\t\targ_pProperty->ip_prop.ipv4_subnet_mask : 0x%x
        \n\t\t\targ_pProperty->ip_prop.ipv4_broadcast : 0x%x
        \n\t\t\targ_pProperty->ip_prop.ipv4_gateway : 0x%x
        \n\t\targ_pProperty->mtu : %d
        \n\t\targ_pProperty->physical_if_state : %d
        \n\t\targ_pProperty->link_speed : %d
        \n\t\targ_pProperty->mac_addr[IX_CC_MAC_ADDR_LEN] : %d
        \n\t\targ_pProperty->media_type; : %d
        \n",
        __FUNCTION__,
        arg_PropId,
        arg_pProperty,
        (arg_pProperty) ? arg_pProperty->blade_id : 0,
        (arg_pProperty) ? arg_pProperty->port_id : 0 ,
        (arg_pProperty) ? arg_pProperty->ip_version : 0, 
        (arg_pProperty) ? arg_pProperty->ip_prop.ipv4_prop.ipv4_address : 0 ,
        (arg_pProperty) ? arg_pProperty->ip_prop.ipv4_prop.ipv4_subnet_mask : 0 ,
        (arg_pProperty) ? arg_pProperty->ip_prop.ipv4_prop.ipv4_broadcast : 0 ,
        (arg_pProperty) ? arg_pProperty->ip_prop.ipv4_prop.ipv4_gateway : 0 ,
        (arg_pProperty) ? arg_pProperty->mtu : 0,
        (arg_pProperty) ? arg_pProperty->physical_if_state : 0,
        (arg_pProperty) ? arg_pProperty->link_speed : 0,
        (arg_pProperty) ? arg_pProperty->mac_addr[IX_CC_MAC_ADDR_LEN] : 0,
        (arg_pProperty) ? arg_pProperty->media_type : 0);
#endif /* SQESTUBS */
    return retObj;
}

void
ix_error_dump(FILE * fp, ix_error err)
{
#ifdef SQESTUBS
    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s
        \n\tfp : 0x%x
        \n\terr : %d
        \n",
        __FUNCTION__,
        fp,
           err
        );
#endif /* SQESTUBS */
    return;
}
