/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ForwardingPlane/FPModule/IXASDKStubs/src/ixasdk_fpm_stubs.c,v $
 * $Revision: 1.6.2.1 $
 * $Date: 2004/03/17 09:52:07 $
 *
 * Description:
 *      Implements method to create event loops in new threads. The thread
 *      method is also contained here. The event threads wait for and
 *      processes messages from the CP and FP and then call the appropriate
 *      methods in the FP or CP.
 *
 ****************************************************************************/

#include "list.h"
#include "log.h"
#include "ixasdk_methodmap.h"
#include "ixasdk_stubs_pri.h"
#include "cci/cc.h"
#include "ix_cc.h"
#include "cc/ix_cc_rtmv4.h"
#include "cc/ix_cc_ipv4.h"
#include "cc/ix_cc_stkdrv_common.h"
#include "cc/ix_cc_stkdrv_pktclass.h"
#include "cc/ix_cc_stkdrv.h"
#ifdef _INCLUDE_ATM
#include "cc/ix_cc_atm_pos_tx.h"
#endif
#ifdef _INCLUDE_DIFFSERV
#include "cc/ix_cc_pos_rx.h"
#endif
#include "cc/ix_cc_fpm_core.h"
#include "cc/ix_cc_eth_rx.h"
//#include "cc/ix_cc_l2tm.h"
//#include "cc/ix_cc_arp.h"

#ifdef _LINUX_
#include "netinet/ip.h"
#endif

extern QUEUE_SERVICE_PARAMS g_stubsParms;

/**
 * NAME: ix_cc_fpm_register_link_status_cb
 *
 * DESCRIPTION: This message helper primitive helps a client to register
 *              interest in Link status update events.
 * 
 * @Param:  - IN arg_LinkState_cb
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_register_link_status_cb (ix_cc_fpm_link_status_cb 
					    arg_LinkState_cb)
{
    ix_error retObj;

    /* no callback for this method */
#ifndef SQESTUBS
    npf_logger_write(LOG_IXA_STUBS, LOG_TRACE,
        "Entered %s .\n",
        StubsMethodMap(Method_ix_cc_fpm_register_link_status_cb));
#endif /* ! SQESTUBS */

    /* Set up return object */
    SetIXAReturnObject(&retObj);

#ifdef SQESTUBS
       npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
			" Registering Link status cb \n",
			__FUNCTION__
			);
#endif /* SQESTUBS */

    return retObj;
}

/**
 * NAME: ix_cc_fpm_register_pkt_hdlr_cb
 *
 * DESCRIPTION: This message helper primitive helps a client to register
 *              interest in receiving packets
 * 
 * @Param:  - IN arg_pktHdlr_cb
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_register_pkt_hdlr_cb (ix_cc_fpm_pkt_hdlr_cb
					 arg_Packet_cb)
{
    ix_error retObj;

    /* no callback for this method */
#ifndef SQESTUBS
    npf_logger_write(LOG_IXA_STUBS, LOG_TRACE,
        "Entered %s .\n",
        StubsMethodMap(Method_ix_cc_fpm_sync_add_l2_entry));
#endif /* ! SQESTUBS */

    /* Set up return object */
    SetIXAReturnObject(&retObj);

#ifdef SQESTUBS
       npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
			" Registering packet handler cb \n",
			__FUNCTION__
			);
#endif /* SQESTUBS */
    return retObj;
}

/**
 * NAME: ix_cc_fpm_register_ipv4_exception_cb
 *
 * DESCRIPTION: This message helper primitive helps a client to register
 *              interest in receiving IPv4 exceptions (Direct Host Route)
 * 
 * @Param:  - IN arg_ipv4Exception_cb
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_register_ipv4_exception_cb (ix_cc_fpm_ipv4_exception_cb
					       arg_ipv4Exception_cb)
{
    ix_error retObj;

    /* no callback for this method */
#ifndef SQESTUBS
    npf_logger_write(LOG_IXA_STUBS, LOG_TRACE,
        "Entered %s .\n",
        StubsMethodMap(Method_ix_cc_fpm_sync_add_l2_entry));
#endif /* ! SQESTUBS */

    /* Set up return object */
    SetIXAReturnObject(&retObj);

#ifdef SQESTUBS
       npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
			" Registering IPv4 exception cb \n",
			__FUNCTION__
			);
#endif /* SQESTUBS */
    return retObj;
}

/**
 * NAME: ix_cc_fpm_deregister_link_status_cb
 *
 * DESCRIPTION: This message helper primitive helps a client to deregister
 *              from receiving link status updates
 * 
 * @Param:  - IN - None
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_deregister_link_status_cb ()
{
    ix_error retObj;

    /* no callback for this method */
#ifndef SQESTUBS
    npf_logger_write(LOG_IXA_STUBS, LOG_TRACE,
        "Entered %s .\n",
        StubsMethodMap(Method_ix_cc_fpm_sync_add_l2_entry));
#endif /* ! SQESTUBS */

    /* Set up return object */
    SetIXAReturnObject(&retObj);

#ifdef SQESTUBS
       npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
			" DeRegistering Link status exception cb \n",
			__FUNCTION__
			);
#endif /* SQESTUBS */
    return retObj;
}

/**
 * NAME: ix_cc_fpm_deregister_pktHdlr_cb
 *
 * DESCRIPTION: This message helper primitive helps a client to deregister
 *              from receiving packets
 * 
 * @Param:  - IN - None
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_deregister_pkt_hdlr_cb ()
{
    ix_error retObj;

    /* no callback for this method */
#ifndef SQESTUBS
    npf_logger_write(LOG_IXA_STUBS, LOG_TRACE,
        "Entered %s .\n",
        StubsMethodMap(Method_ix_cc_fpm_sync_add_l2_entry));
#endif /* ! SQESTUBS */

    /* Set up return object */
    SetIXAReturnObject(&retObj);

#ifdef SQESTUBS
       npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
			" DeRegistering packet cb \n",
			__FUNCTION__
			);
#endif /* SQESTUBS */
    return retObj;
}

/**
 * NAME: ix_cc_fpm_deregister_ipv4_exception_cb
 *
 * DESCRIPTION: This message helper primitive helps a client to deregister
 *              from receiving IPv4 exceptions (Direct Host Route)
 * 
 * @Param:  - IN arg_ipv4Exception_cb
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_deregister_ipv4_exception_cb ()
{
    ix_error retObj;

    /* no callback for this method */
#ifndef SQESTUBS
    npf_logger_write(LOG_IXA_STUBS, LOG_TRACE,
        "Entered %s .\n",
        StubsMethodMap(Method_ix_cc_fpm_sync_add_l2_entry));
#endif /* ! SQESTUBS */

    /* Set up return object */
    SetIXAReturnObject(&retObj);

#ifdef SQESTUBS
       npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
			" DeRegistering IPv4 exception cb \n",
			__FUNCTION__
			);
#endif /* SQESTUBS */
    return retObj;
}

/**
 * NAME: ix_cc_fpm_sync_add_l2_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to add a L2 table 
 *              entry into the L2 Table maintained on the egress.
 *              This is a synchronous function that will block till the result
 *              is available or a timeout occurs.
 * 
 * @Param:  - IN arg_L2Id: L2 Id being added
 * @Param:  - IN arg_NextHopIp: IP address of the next hop 
 * @Param:  - IN arg_OutputPort: Outgoing port for the specified next hop
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_sync_add_l2_entry (
				      ix_uint32 arg_L2ID,
				      ix_uint32 arg_OutputPort,
				      ix_cc_l2tm_entry *arg_l2Info
				      )
{
    ix_error retObj;

    /* IXA SDK says no callback for this method */
#ifndef SQESTUBS
    npf_logger_write(LOG_IXA_STUBS, LOG_TRACE,
        "Entered %s .\n",
        StubsMethodMap(Method_ix_cc_fpm_sync_add_l2_entry));
#endif /* ! SQESTUBS */

    /* Set up return object */
    SetIXAReturnObject(&retObj);

#ifdef SQESTUBS
       npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s
        \n\targ_L2Id : 0x%x
        \n\targ_OutputPort : 0x%x
        \n\targ_l2Info : 0x%x
        \n",
        __FUNCTION__,
        arg_L2ID,
        arg_OutputPort,
        arg_l2Info
        );

#endif /* SQESTUBS */
    return retObj;
}

/**
 * NAME: ix_cc_fpm_sync_del_l2_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to del a L2 table 
 *              entry from the L2 Table maintained on the egress.
 *              This is a synchronous function that will block till the result
 *              is available or a timeout occurs.
 * 
 * @Param:  - IN arg_L2Id: L2 Id being removed
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_sync_del_l2_entry (
                      ix_uint32 arg_L2Id
                      )
{
    ix_error retObj;

    /* IXA SDK says no callback for this method */
#ifndef SQESTUBS
    npf_logger_write(LOG_IXA_STUBS, LOG_TRACE,
        "Entered %s .\n",
        StubsMethodMap(Method_ix_cc_fpm_sync_del_l2_entry));

#endif /* ! SQESTUBS */
    /* Set up return object */
    SetIXAReturnObject(&retObj);

#ifdef SQESTUBS
    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s
        \n\targ_L2Id : 0x%x
        \n",
        __FUNCTION__,
        arg_L2Id
        );

#endif /* SQESTUBS */
    return retObj;
}

/**
 * NAME: ix_cc_fpm_async_add_l2_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to add a L2
 *              entry into the L2 table maintained on the egress.
 * 
 * @Param:  - IN arg_L2Id: L2 Id being added
 * @Param:  - IN arg_OutputPort: Outgoing port for the specified next hop
 * @Param:  - IN arg_L2Info: ix_cc_l2tm_entry structure for L2 Table
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_async_add_l2_entry (
				       ix_uint32 arg_L2ID,
				       ix_uint32 arg_OutputPort,
				       ix_cc_l2tm_entry *arg_l2Info,
				       ix_cc_fpm_l2_op_cb arg_Callback,
				       void* arg_pUserContext
				       )
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_L2ID : 0x%x
            \n\targ_OutputPort : 0x%x
            \n\targ_l2Info : 0x%x
            \n\targ_Callback : 0x%x
            \n\targ_pUserContext : 0x%x
            \n",
            __FUNCTION__,
            arg_L2ID,
            arg_OutputPort,
            arg_l2Info,
            arg_Callback,
            arg_pUserContext
            );

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_fpm_async_add_l2_entry,
            STUBS_ARP_OP,
            arg_Callback,
            arg_pUserContext);

    }

    return retObj;
}


/**
 * NAME: ix_cc_fpm_async_del_l2_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to delete a L2
 *              entry from the L2 table maintained on the egress.
 * 
 * @Param:  - IN arg_L2Id: L2 Id being deleted
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_async_del_l2_entry (
				       ix_uint32 arg_L2ID,
				       ix_cc_fpm_l2_op_cb arg_Callback,
				       void* arg_pUserContext
				       )
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_L2ID : 0x%x
            \n\targ_Callback : 0x%x
            \n\targ_pUserContext : 0x%x
            \n",
            __FUNCTION__,
            arg_L2ID,
            arg_Callback,
            arg_pUserContext
            );

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_fpm_async_del_l2_entry,
            STUBS_ARP_OP,
            arg_Callback,
            arg_pUserContext);

    }
    return retObj;
}

/**
 * NAME: ix_cc_fpm_async_add_arp_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to add a ARP
 *              entry into the ARP table maintained on the egress.
 * 
 * @Param:  - IN arg_L2Id: L2 Id being added
 * @Param:  - IN arg_NextHopIp: IP address of the next hop 
 * @Param:  - IN arg_OutputPort: Outgoing port for the specified next hop
 * @Param:  - IN arg_L2Address: ARP entry being added (L2 Address)
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_async_add_arp_entry (
					ix_cc_eth_tx_next_hop_info* arg_ArpInfo,
					ix_cc_fpm_arp_op_cb arg_Callback,
					void* arg_pUserContext
					)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_ArpInfo : 0x%x
            \n\targ_Callback : 0x%x
            \n\targ_pUserContext : 0x%x
            \n",
            __FUNCTION__,
            arg_ArpInfo,
            arg_Callback,
            arg_pUserContext
            );

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_fpm_async_add_arp_entry,
            STUBS_ARP_OP,
            arg_Callback,
            arg_pUserContext);

    }

    return retObj;
}

/**
 * NAME: ix_cc_fpm_async_del_arp_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to add a ARP
 *              entry into the ARP table maintained on the egress.
 * 
 * @Param:  - IN arg_L2Id: L2 Id being deleted
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_async_del_arp_entry (
					ix_uint32 arg_L2Id,
					ix_cc_fpm_arp_op_cb arg_Callback,
					void* arg_pUserContext
					)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_L2Id : 0x%x
            \n\targ_Callback : 0x%x
            \n\targ_pUserContext : 0x%x
            \n",
            __FUNCTION__,
            arg_L2Id,
            arg_Callback,
            arg_pUserContext
            );

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_fpm_async_del_arp_entry,
            STUBS_ARP_OP,
            arg_Callback,
            arg_pUserContext);

    }

    return retObj;
}

/**
 * NAME: ix_cc_fpm_async_purge_arp_table
 *
 * DESCRIPTION: This message helper primitive helps a client to purge the ARP
 *              table maintained on the egress.
 * 
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_sync_purge_arp_table ()
{
    ix_error retObj;

    /* IXA SDK says no callback for this method */
#ifndef SQESTUBS
    npf_logger_write(LOG_IXA_STUBS, LOG_TRACE,
        "Entered %s .\n",
        StubsMethodMap(Method_ix_cc_fpm_async_purge_arp_table));

#endif /* ! SQESTUBS */
    /* Set up return object */
    SetIXAReturnObject(&retObj);

#ifdef SQESTUBS
    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s
        \n",
        __FUNCTION__
        );
#endif /* SQESTUBS */
    return retObj;
}

#ifdef _INCLUDE_ATM
/**
 * NAME: ix_cc_fpm_async_get_atm_pos_tx_stats
 *
 * DESCRIPTION: This message helper primitive helps a client to retrieve
 *              L2 (ATM/POS) transmit statistics maintained on the egress.
 * 
 * @Param:  - IN arg_PortId: port to get statistics for
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */
ix_error ix_cc_fpm_async_get_pos_tx_stats (
    ix_cc_atm_pos_tx_statistics_info_context* arg_pStatsInfo,
    ix_cc_fpm_atm_pos_tx_stats_op_cb arg_Callback)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS
        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_pStatsInfo : 0x%x
            \n\t\targ_pStatsInfo->entity : %d
            \n\t\targ_pStatsInfo->index: %d
            \n\t\targ_pStatsInfo->pUserContext : 0x%x
            \n\targ_Callback : 0x%x
            \n",
            __FUNCTION__,
            arg_pStatsInfo,
            (arg_pStatsInfo) ? arg_pStatsInfo->entity  : 0,
            (arg_pStatsInfo) ? arg_pStatsInfo->index : 0,
            (arg_pStatsInfo) ? arg_pStatsInfo->pUserContext : 0,
            arg_Callback
            );

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_fpm_async_get_pos_tx_stats,
            STUBS_L2_ATM_POS_TX_STATS,
            arg_Callback,
            arg_pStatsInfo->pUserContext);

    }

    return retObj;
}
#endif /* _INCLUDE_ATM */


ix_error ix_cc_fpm_sync_send_packet (
    ix_uint32 arg_PortID,
    char*     arg_pBuffer,
    ix_uint32 arg_PacketSize
    )
{
    ix_error retObj;
#ifdef _LINUX_
    struct iphdr *h;
#endif

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS
        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_PortID : 0x%x
            \n\targ_pBuffer : 0x%x
            \n\t\targ_PacketSize : %d
            \n",
            __FUNCTION__,
            arg_PortID,
            arg_pBuffer,
            arg_PacketSize
            );

#else /* SQESTUBS */
        npf_logger_write(LOG_IXA_STUBS, LOG_TRACE,
            "Entered %s .\n",
            __FUNCTION__);
#endif
#ifdef _LINUX_
        h = (struct iphdr *)arg_pBuffer;
        npf_logger_write(LOG_IXA_STUBS, LOG_TRACE,
            "SRC: 0x%x -> DST: 0x%x | PROTOCOL %x\n",
            h->saddr,h->daddr,h->protocol);
#endif
    }

    return retObj;
}

/**
 * NAME: ix_cc_fpm_async_get_eth_tx_stats
 *
 * DESCRIPTION: This message helper primitive helps a client to retrieve
 *              L2 (ETH) transmit statistics maintained on the egress.
 * 
 * @Param:  - IN arg_PortId: port to get statistics for
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */
ix_error ix_cc_fpm_async_get_eth_tx_stats (
    ix_cc_eth_tx_statistics_info_context* arg_pStatsInfo,
    ix_cc_fpm_eth_tx_stats_op_cb arg_Callback)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS
        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_pStatsInfo : 0x%x
            \n\t\targ_pStatsInfo->entity : %d
            \n\t\targ_pStatsInfo->index: %d
            \n\t\targ_pStatsInfo->pUserContext : 0x%x
            \n\targ_Callback : 0x%x
            \n",
            __FUNCTION__,
            arg_pStatsInfo,
            (arg_pStatsInfo) ? arg_pStatsInfo->entity  : 0,
            (arg_pStatsInfo) ? arg_pStatsInfo->index : 0,
            (arg_pStatsInfo) ? arg_pStatsInfo->pUserContext : 0,
            arg_Callback
            );

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_fpm_async_get_eth_tx_stats,
            STUBS_L2_ETH_TX_STATS,
            arg_Callback,
            arg_pStatsInfo->pUserContext);

    }

    return retObj;
}

/**
 * NAME: ix_cc_fpm_async_get_eth_rx_stats
 *
 * DESCRIPTION: This message helper primitive helps a client to retrieve
 *              L2 (ETH) transmit statistics maintained on the egress.
 * 
 * @Param:  - IN arg_PortId: port to get statistics for
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */
ix_error ix_cc_eth_rx_async_get_statistics_info (
    ix_cc_eth_rx_statistics_info_context* arg_pStatsInfo,
    ix_cc_eth_rx_cb_get_statistics_info arg_Callback)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS
        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_pStatsInfo : 0x%x
            \n\t\targ_pStatsInfo->entity : %d
            \n\t\targ_pStatsInfo->index: %d
            \n\t\targ_pStatsInfo->pUserContext : 0x%x
            \n\targ_Callback : 0x%x
            \n",
            __FUNCTION__,
            arg_pStatsInfo,
            (arg_pStatsInfo) ? arg_pStatsInfo->entity  : 0,
            (arg_pStatsInfo) ? arg_pStatsInfo->index : 0,
            (arg_pStatsInfo) ? arg_pStatsInfo->pUserContext : 0,
            arg_Callback
            );

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
			     Method_ix_cc_eth_rx_async_get_statistics_info,
			     STUBS_ETH_RX_STATS,
			     arg_Callback,
			     arg_pStatsInfo->pUserContext);
	
    }

    return retObj;
}

#ifdef _INCLUDE_ATM

/**
 * NAME: ix_cc_pos_rx_async_get_statistics_info
 *
 * DESCRIPTION: This message helper primitive helps a client to retrieve
 *              L2 (POS) receive statistics.
 * 
 * @Param:  - IN arg_PortId: port to get statistics for
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */
ix_error ix_cc_pos_rx_async_get_statistics_info(
    ix_cc_pos_rx_statistics_info_context *arg_pMgInfoContext,
    ix_cc_pos_rx_cb_get_statistics_info arg_Callback)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_pMgInfoContext : 0x%x
            \n\t\targ_pMgInfoContext->entity : %d
            \n\t\targ_pMgInfoContext->index: %d
            \n\t\targ_pMgInfoContext->pUserContext : 0x%x
            \n\targ_Callback : 0x%x
            \n",
            __FUNCTION__,
            arg_pMgInfoContext,
            (arg_pMgInfoContext) ? arg_pMgInfoContext->entity : 0,
            (arg_pMgInfoContext) ? arg_pMgInfoContext->index : 0,
            (arg_pMgInfoContext) ? arg_pMgInfoContext->pUserContext : 0,
            arg_Callback
            );

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_pos_rx_async_get_statistics_info,
            STUBS_POS_RX_STATS,
            arg_Callback,
            arg_pMgInfoContext->pUserContext);

    }

    return retObj;
}
#endif /*_INCLUDE_ATM */
