/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ForwardingPlane/FPModule/IXASDKStubs/src/ixasdk_ipv4_stubs.c,v $
 * $Revision: 1.8 $
 * $Date: 2003/11/17 16:57:23 $
 *
 * Description:
 *      Implements method to create event loops in new threads. The thread
 *      method is also contained here. The event threads wait for and
 *      processes messages from the CP and FP and then call the appropriate
 *      methods in the FP or CP.
 *
 ****************************************************************************/

#include <time.h>
#include "list.h"
#include "log.h"
#include "ixasdk_methodmap.h"
#include "ixasdk_stubs_pri.h"

#include "cci/cc.h"
#include "ix_cc.h"
#include "cc/ix_cc_rtmv4.h"
#include "cc/ix_cc_ipv4.h"
#include "cc/ix_cc_fpm_core.h"
#include "ix_cc_ipv4_init.h"

//#include "ix_cc_ipv4.h"
//#include "ix_error.h"


extern QUEUE_SERVICE_PARAMS g_stubsParms;
ix_cc_ipv4_context* g_pIpv4Context;

/**
 * NAME: ix_cc_ipv4_async_add_route
 *
 * DESCRIPTION: This message helper primitive helps client to add a route in RTM. 
 * Client needs to successfully add nextHopId to RTM by calling ix_cc_ipv4_async_add_next_hop
 * messaging helper primitive before sending this message to IPv4 Forwarder core component. 
 * 
 * @Param:  - IN arg_IpAddr: network IP address identifying the route.
 * @Param:  - IN arg_NetMask: network mask for the route.
 * @Param:  - IN arg_NextHopId: next hop identifier for route.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */


ix_error ix_cc_ipv4_async_add_route(ix_uint32 arg_IpAddr,
                                    ix_uint32 arg_NetMask,
                                    ix_cc_rtmv4_nhid arg_NextHopId,
                                    ix_cc_ipv4_cb_route_op arg_Callback,
                                    void* arg_pUserContext)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s\n\targ_IpAddr : 0x%x\n\targ_NetMask : 0x%x\n\targ_NextHopId : 0x%x\n\targ_Callback : 0x%x\n\targ_pUserContext : 0x%x\n",__FUNCTION__,arg_IpAddr,arg_NetMask,arg_NextHopId,arg_Callback,arg_pUserContext);

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv4_async_add_route,
            STUBS_IPv4_ROUTE_OP,
            arg_Callback,
            arg_pUserContext);

    }

    return retObj;
}

/**
 * NAME: ix_cc_ipv4_async_delete_route
 *
 * DESCRIPTION:This message helper primitive helps a client to delete a route in RTM. 
 * Client needs to delete all routes associate with a next hop to delete the next hop
 * by calling ix_cc_ipv4_async_delete_next_hop message helper primitive. 
 * 
 * @Param:  - IN arg_IpAddr: network IP address identifying the route.
 * @Param:  - IN arg_NetMask: network mask for the route.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */


ix_error ix_cc_ipv4_async_delete_route(ix_uint32 arg_IpAddr,
                                       ix_uint32 arg_NetMask,
                                       ix_cc_ipv4_cb_route_op arg_Callback,
                                       void* arg_pUserContext
                                      )
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
               "%s
            \n\targ_IpAddr : 0x%x
            \n\targ_NetMask : 0x%x
            \n\targ_Callback : 0x%x
            \n\targ_pUserContext : 0x%x
            \n",
            __FUNCTION__,
            arg_IpAddr,
            arg_NetMask,
            arg_Callback,
            arg_pUserContext
            );

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv4_async_delete_route,
            STUBS_IPv4_ROUTE_OP,
            arg_Callback,
            arg_pUserContext);

    }

    return retObj;
}

/**
 * NAME: ix_cc_ipv4_async_update_route
 *
 * DESCRIPTION: This message helper primitive helps client to update a route in RTM. 
 * Client needs to successfully add nextHopId to RTM by calling ix_cc_ipv4_async_add_next_hop
 * messaging helper primitive before sending this message to IPv4 Forwarder core component. 
 * 
 * @Param:  - IN arg_IpAddr: network IP address identifying the route.
 * @Param:  - IN arg_NetMask: network mask for the route.
 * @Param:  - IN arg_NextHopId: next hop identifier for the route.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */

ix_error ix_cc_ipv4_async_update_route(ix_uint32 arg_IpAddr,
                                    ix_uint32 arg_NetMask,
                                    ix_cc_rtmv4_nhid arg_NextHopId,
                                    ix_cc_ipv4_cb_route_op arg_Callback,
                                    void* arg_pUserContext
                                   )
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s 
            \n\targ_IpAddr : 0x%x
            \n\targ_NetMask : 0x%x
            \n\targ_NextHopId : 0x%x
            \n\targ_Callback : 0x%x
            \n\targ_pUserContext : 0x%x
            \n",
            __FUNCTION__,
            arg_IpAddr,
            arg_NetMask,
            arg_NextHopId,
            arg_Callback,
            arg_pUserContext
            );

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv4_async_update_route,
            STUBS_IPv4_ROUTE_OP,
            arg_Callback,
            arg_pUserContext);

    }

    return retObj;
}

/**
 * NAME: ix_cc_ipv4_async_lookup_route
 *
 * DESCRIPTION: This primitive helps a client to look up routing information 
 * for a given IP address. If there is a successful match, then the information will be 
 * given to client are nextHopId (including the blade id), portId, mtu and flags. 
 * If there is no match, then IX_CC_RTMV4_NHID_NO_ROUTE will be assigned to nextHopId
 * field of pNextHopData and all other information are invalid.
 * 
 * @Param:  - IN arg_IpAddr: network IP address identifying the route.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_ipv4_async_lookup_route(ix_uint32 arg_IpAddr,
                                       ix_cc_ipv4_cb_lookup_route arg_Callback,
                                       void* arg_pUserContext
                                      )
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s 
            \n\targ_IpAddr : 0x%x
            \n\targ_Callback : 0x%x
            \n\targ_pUserContext : 0x%x
            \n",
            __FUNCTION__,
            arg_IpAddr,
            arg_Callback,
            arg_pUserContext);

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv4_async_lookup_route,
            STUBS_IPv4_LOOKUP,
            arg_Callback,
            arg_pUserContext);

    }

    return retObj;
}

/**
 * NAME: ix_cc_ipv4_async_purge_routes
 *
 * DESCRIPTION: This primitive helps a client to remove all routes from RTM.
 * 
 * @Param:  - IN arg_CcHandle - 
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_ipv4_async_purge_routes()
{
    ix_error retObj;

    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS
    if (retObj == (ix_error)0)
    {

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n",
            __FUNCTION__);

    }
#endif /* SQESTUBS */
    
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_async_dump_routes
 *
 * DESCRIPTION: This primitive helps to dump all routes of RTM in memory
 * 
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_ipv4_async_dump_routes(ix_cc_ipv4_cb_dump_data arg_Callback,
                                      void* arg_pUserContext
                                     )
{
    ix_error retObj;
    
    /* Set up return object */
    SetIXAReturnObject(&retObj);

    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_Callback : 0x%x
            \n\targ_pUserContext : 0x%x
            \n",
            __FUNCTION__,
            arg_Callback,
            arg_pUserContext);

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv4_async_dump_routes,
            STUBS_IPv4_ROUTE_DUMP,
            arg_Callback,
            arg_pUserContext);
    
    }

    return retObj;
}

/**
 * NAME: ix_cc_ipv4_async_add_next_hop
 *
 * DESCRIPTION: This primitive helps a client to add next hop information to RTM database. 
 * Client needs to add next hop to RTM in order to add routes referring to that next hop.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop to be added
 * @Param:  - IN arg_pNextHopData: pointer to next hop information
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined contex
 * 
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_ipv4_async_add_next_hop(ix_cc_rtmv4_nhid arg_NextHopId,
                                       ix_cc_rtmv4_next_hop_info* arg_pNextHopData,
                                       ix_cc_ipv4_cb_route_op arg_Callback,
                                       void* arg_pUserContext
                                      )
{
    ix_error retObj;
    
    /* Set up return object */
    SetIXAReturnObject(&retObj);

    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_NextHopId : %d
            \n\targ_pNextHopData : %d
            \n\t\targ_pNextHopData->bladeID : %d
            \n\t\targ_pNextHopData->l2Index : %d
            \n\t\targ_pNextHopData->portID : %d
            \n\t\targ_pNextHopData->mtu : %d
            \n\t\targ_pNextHopData->flags : %d
            \n\t\targ_pNextHopData->ipAddr : %d
            \n\targ_Callback : 0x%x
            \n\targ_pUserContext : 0x%x
            \n",
            __FUNCTION__,
            arg_NextHopId,
            arg_pNextHopData,
            (arg_pNextHopData) ? arg_pNextHopData->bladeID : 0,
            (arg_pNextHopData) ? arg_pNextHopData->l2Index : 0,
            (arg_pNextHopData) ? arg_pNextHopData->portID : 0,
            (arg_pNextHopData) ? arg_pNextHopData->mtu : 0,
            (arg_pNextHopData) ? arg_pNextHopData->flags : 0,
            (arg_pNextHopData) ? arg_pNextHopData->ipAddr : 0,
            arg_Callback,
            arg_pUserContext);

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv4_async_add_next_hop,
            STUBS_IPv4_ROUTE_OP,
            arg_Callback,
            arg_pUserContext);
    
    }

    return retObj;
}

/**
 * NAME: ix_cc_ipv4_async_delete_next_hop
 *
 * DESCRIPTION: This message helper primitive helps a client to delete next hop information 
 * from RTM database. Client needs to remove all routes associated with next hop before 
 * calling this primitive. If not, client gets an error.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop to be removed.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */


ix_error ix_cc_ipv4_async_delete_next_hop(ix_cc_rtmv4_nhid arg_NextHopId,
                                          ix_cc_ipv4_cb_route_op arg_Callback,
                                          void* arg_pUserContext
                                          )
{
    ix_error retObj;
    
    /* Set up return object */
    SetIXAReturnObject(&retObj);

    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_NextHopId : %d
            \n\targ_Callback : 0x%x
            \n\targ_pUserContext : 0x%x
            \n",
            __FUNCTION__,
            arg_NextHopId, 
            arg_Callback,
            arg_pUserContext);

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv4_async_delete_next_hop,
            STUBS_IPv4_ROUTE_OP,
            arg_Callback,
            arg_pUserContext);
    
    }

    return retObj;
}

/**
 * NAME: ix_cc_ipv4_async_update_next_hop
 *
 * DESCRIPTION: This primitive helps a client to update next hop information to RTM database. 
 * Client needs to add next hop to RTM in order to add routes referring to that next hop.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop to be updated
 * @Param:  - IN arg_pNextHopData: pointer to next hop information
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined contex
 * 
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_ipv4_async_update_next_hop(ix_cc_rtmv4_nhid arg_NextHopId,
                                       ix_cc_rtmv4_next_hop_info* arg_pNextHopData,
                                       ix_cc_ipv4_cb_route_op arg_Callback,
                                       void* arg_pUserContext
                                      )
{
    ix_error retObj;
    
    /* Set up return object */
    SetIXAReturnObject(&retObj);

    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_NextHopId : %d
            \n\targ_pNextHopData : %d
            \n\t\targ_pNextHopData->bladeID : %d
            \n\t\targ_pNextHopData->l2Index : %d
            \n\t\targ_pNextHopData->portID : %d
            \n\t\targ_pNextHopData->mtu : %d
            \n\t\targ_pNextHopData->flags : %d
            \n\t\targ_pNextHopData->ipAddr : %d
            \n\targ_Callback : 0x%x
            \n\targ_pUserContext : 0x%x
            \n",
            __FUNCTION__,
            arg_NextHopId,
            arg_pNextHopData,
            (arg_pNextHopData) ? arg_pNextHopData->bladeID : 0,
            (arg_pNextHopData) ? arg_pNextHopData->l2Index : 0,
            (arg_pNextHopData) ? arg_pNextHopData->portID : 0,
            (arg_pNextHopData) ? arg_pNextHopData->mtu : 0,
            (arg_pNextHopData) ? arg_pNextHopData->flags : 0,
            (arg_pNextHopData) ? arg_pNextHopData->ipAddr : 0,
            arg_Callback,
            arg_pUserContext);

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv4_async_update_next_hop,
            STUBS_IPv4_ROUTE_OP,
            arg_Callback,
            arg_pUserContext);
    
    }

    return retObj;
}

/**
 * NAME: ix_cc_ipv4_async_get_next_hop
 *
 * DESCRIPTION: This message helper primitive helps a client to retrieve 
 * next hop information from RTM database. 
 * 
 * @Param:  - IN arg_NextHopId: Identifier of the next hop information to be retrieved from RTM.
 * @Param:  - IN arg_Callback: pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_ipv4_async_get_next_hop(ix_cc_rtmv4_nhid arg_NextHopId,
                                       ix_cc_ipv4_cb_get_next_hop arg_Callback,
                                       void* arg_pUserContext
                                      )
{
    ix_error retObj;
    
    /* Set up return object */
    SetIXAReturnObject(&retObj);

    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_NextHopId : %d
            \n\targ_Callback : 0x%x
            arg_pUserContext : 0x%x
            \n",
            __FUNCTION__,
            arg_NextHopId,
            arg_Callback,
            arg_pUserContext);

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv4_async_get_next_hop,
            STUBS_IPv4_NEXT_HOP,
            arg_Callback,
            arg_pUserContext);
    
    }

    return retObj;
}

/**
 * NAME: ix_cc_ipv4_async_purge_rtm
 *
 * DESCRIPTION: This primitive helps a client to removes all routes and next hops from RTM database
 * 
 * @Param:  - IN  - none
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */


ix_error ix_cc_ipv4_async_purge_rtm()
{
    ix_error retObj;

    SetIXAReturnObject(&retObj);
    
#ifdef SQESTUBS
    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
          "%s
        \n",
        __FUNCTION__);

#endif /* SQESTUBS */
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_async_dump_next_hops
 *
 * DESCRIPTION:   This primitive helps to dump all next hops of RTM in memory.
 * 
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_ipv4_async_dump_next_hops(ix_cc_ipv4_cb_dump_data arg_Callback,
                                         void* arg_pUserContext
                                        )
{
    ix_error retObj;
    
    /* Set up return object */
    SetIXAReturnObject(&retObj);

    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_Callback : 0x%x
            \n\targ_pUserContext : 0x%x
            \n",
            __FUNCTION__,
            arg_Callback,
            arg_pUserContext);

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv4_async_dump_next_hops,
            STUBS_IPv4_ROUTE_DUMP,
            arg_Callback,
            arg_pUserContext);
    
    }

    return retObj;
}

/**
 * NAME: ix_cc_ipv4_async_set_mtu
 *
 * DESCRIPTION:This primitive helps client to update MTU for a given next hop. 
 * Before calling this message helper, client needs to call ix_cc_ipv4_async_add_next_hop to 
 * add next hop identifier in RTM database. The validity of MTU shall be done by the client.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop.
 * @Param:  - IN arg_Mtu: new maximum transmission unit for the given next hop.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */


ix_error ix_cc_ipv4_async_set_mtu(ix_cc_rtmv4_nhid arg_NextHopId,
                                  ix_uint32 arg_Mtu,
                                  ix_cc_ipv4_cb_route_op arg_Callback,
                                  void* arg_pUserContext
                                 )
{
    ix_error retObj;
    
    /* Set up return object */
    SetIXAReturnObject(&retObj);

    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_NextHopId : 0x%x
            \n\targ_Mtu : 0x%x
            \n\targ_Callback : 0x%x
            arg_pUserContext : 0x%x
            \n",
            __FUNCTION__,
            arg_NextHopId,
            arg_Mtu,
            arg_Callback,
            arg_pUserContext);

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv4_async_set_mtu,
            STUBS_IPv4_ROUTE_OP,
            arg_Callback,
            arg_pUserContext);
    
    }

    return retObj;
}

/**
 * NAME: ix_cc_ipv4_async_set_flags
 *
 * DESCRIPTION:This primitive helps client to update flags for a given next hop.
 * Before calling this message helper, client needs to call ix_cc_ipv4_async_add_next_hop to add next hop 
 * identifier in RTM database. The supported flags[BLDBLK] are defined by IPv4 Forwarder microblock.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop.
 * @Param:  - IN arg_Flags: new flags for the given next hop 
 * @Param:  - IN arg_pUserContext: pointer to client defined context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */
ix_error ix_cc_ipv4_async_set_flags(ix_cc_rtmv4_nhid arg_NextHopId,
                                    ix_uint32 arg_Flags,
                                    ix_cc_ipv4_cb_route_op arg_Callback,
                                    void* arg_pUserContext
                                   )
{
    ix_error retObj;
    
    /* Set up return object */
    SetIXAReturnObject(&retObj);

    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_NextHopId : 0x%x
            \n\targ_Flags : 0x%x
            \n\targ_Callback : 0x%x
            \n\targ_pUserContext : 0x%x
            \n",
            __FUNCTION__,
            arg_NextHopId,
            arg_Flags,
            arg_Callback,
            arg_pUserContext);

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv4_async_set_flags,
            STUBS_IPv4_ROUTE_OP,
            arg_Callback,
            arg_pUserContext);
    
    }

    return retObj;
}

/**
 * NAME: ix_cc_ipv4_async_get_sleep_time
 *
 * DESCRIPTION:This message helper primitive helps client to retrieve the number of seconds 
 * between when the event will fire causing the ICMP messages to dequeue and send. 
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_ipv4_async_get_sleep_time(ix_cc_ipv4_cb_get_sleep_time arg_Callback,
                                         void* arg_pUserContext
                                        )
{
    ix_error retObj;
    
    /* Set up return object */
    SetIXAReturnObject(&retObj);

    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_Callback : 0x%x
            \n\targ_pUserContext : 0x%x
            \n",
            __FUNCTION__,
            arg_Callback,
            arg_pUserContext);

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv4_async_get_sleep_time,
            STUBS_IPv4_SLEEP_TIME,
            arg_Callback,
            arg_pUserContext);
    
    }

    return retObj;
}

/**
 * NAME: ix_cc_ipv4_async_set_sleep_time
 *
 * DESCRIPTION: This message helper primitive helps client to set the number of seconds between 
 * when the event will fire causing the ICMP messages to dequeue and send.  The lower the value of arg_SleepTime, 
 * the more frequent the ICMP messages will be dequeued and sent.
 * 
 * @Param:  - IN arg_SleepTime: minimum value is 1 second and maximum value is 32 seconds.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV4_ERROR_INVALID_SLEEP_TIME - invalid sleep time
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_ipv4_async_set_sleep_time(ix_uint16 arg_SleepTime)
{
    ix_error retObj;

    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS

    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
           "%s
        \n\targ_SleepTime : 0x%x
        \n",
        __FUNCTION__,
        arg_SleepTime);
#endif /* SQESTUBS */
    
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_async_get_queue_depth
 *
 * DESCRIPTION:This message helper primitive helps client to retrieves the depth of the ICMP error message queue. 
 * Client retrieves the maximum number of ICMP messages that can fit in the queue, not the number of messages
 * currently in the queue.
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */
ix_error ix_cc_ipv4_async_get_queue_depth(ix_cc_ipv4_cb_get_queue_depth arg_Callback,
                                          void* arg_pUserContext
                                         )
{
    ix_error retObj;
    
    /* Set up return object */
    SetIXAReturnObject(&retObj);

    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_Callback : 0x%x
            \n\targ_pUserContext : 0x%x
            \n",
            __FUNCTION__,
            arg_Callback,
            arg_pUserContext);

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv4_async_get_queue_depth,
            STUBS_IPv4_QUEUE_DEPTH,
            arg_Callback,
            arg_pUserContext);
    
    }

    return retObj;
}

/**
 * NAME: ix_cc_ipv4_async_get_packets_to_drain
 *
 * DESCRIPTION:This message helper primitive helps client to retrieve the maximum number of ICMP error messages 
 * to send each time the queue processing event fires. The processing event will send up to this many ICMP error
 * packets from the queue before going back to sleep. 
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */
ix_error ix_cc_ipv4_async_get_packets_to_drain(ix_cc_ipv4_cb_get_packets_to_drain arg_Callback,
                                               void* arg_pUserContext
                                              )
{
    ix_error retObj;
    
    /* Set up return object */
    SetIXAReturnObject(&retObj);

    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_Callback : 0x%x
            \n\targ_pUserContext : 0x%x
            \n",
            __FUNCTION__,    
            arg_Callback,
            arg_pUserContext);

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv4_async_get_packets_to_drain,
            STUBS_IPv4_PKT_DRAIN,
            arg_Callback,
            arg_pUserContext);
    
    }

    return retObj;
}

/**
 * NAME: ix_cc_ipv4_async_set_packets_to_drain
 *
 * DESCRIPTION:This message helper primitive helps client to set the maximum number of ICMP error messages to 
 * send each time the queue processing event fires. The value to arg_PacketsToDrain must not exceed the depth of 
 * the queue.
 * 
 * @Param:  - IN arg_PacketsToDrain: The maximum number of packets to send each time the queue processing event fires.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV4_ERROR_INVALID_PACKETS_TO_DRAIN - invalid value for packets to drain
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */
ix_error ix_cc_ipv4_async_set_packets_to_drain(ix_uint16 arg_PacketsToDrain)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
#ifdef SQESTUBS
    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
           "%s
        \n\targ_PacketsToDrain : 0x%x
        \n",
        __FUNCTION__,
        arg_PacketsToDrain);
    
#endif /* SQESTUBS */
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_async_get_statistics
 *
 * DESCRIPTION:This message helper primitive helps client to retrieve statistics report from IPv4 Forwarder core component.
 * This primitive will give the statistics from IPv4 microblock, RTM and IPV4 Forwarder core component.  
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pUserContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_ipv4_async_get_statistics(ix_cc_ipv4_cb_get_statistics arg_Callback,
                                         void* arg_pUserContext
                                        )
{
    ix_error retObj;
    
    /* Set up return object */
    SetIXAReturnObject(&retObj);

    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_Callback : 0x%x
            \n\targ_pUserContext : 0x%x
            \n",
            __FUNCTION__,
            arg_Callback,
            arg_pUserContext);

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv4_async_get_statistics,
            STUBS_IPv4_GET_STATS,
            arg_Callback,
            arg_pUserContext);
    
    }

    return retObj;
}

ix_error ix_cc_ipv4_initStub()
{
    ix_error retObj;
    
    /* Set up return object */
    SetIXAReturnObject(&retObj);

   g_pIpv4Context = (ix_cc_ipv4_context *)PIL_AllocateMemory(sizeof(ix_cc_ipv4_context), PIL_MEM_ZERO_INIT);    
   return retObj;
}

void ix_cc_ipv4_finiStub()
{
   PIL_FreeMemory(g_pIpv4Context); 
}
 
#if 0
/**
 * NAME: ix_cc_ipv4_init
 *
 * DESCRIPTION: This primitive shall be called and returned successfully before requesting any service
 *              from IPv4 Forwarder component. This primitive should be called only once to initialize 
 *              IPv4 Forwarder component.This function performs allocation of memory for symbols to be patched,
 *              Creation of 64 bit counters, registration of packet and message handlers,
 *              initialization and configuration of RTM, allocation and initialization of internal data structures
 *              and creation of event handler for ICMP.
 * 
 * @Param:  - IN arg_hCcHandle - handle to IPV4 core component, created by core component infrastructure; 
 *            this shall be used later to get other (to add event handler) services from core component infrastructure.
 *
 * @Param:  - OUT arg_ppContext - location where the pointer to the control block allocated by 
 *            IPv4 Forwarder core component will be stored. The control block is internal to IPv4 
 *            Forwarder core component and it contains information about IPv4 internal data structures, 
 *            allocated memory and other relevant information. Later this pointer to be passed into  
 *            ix_cc_ipv4_fini function for freeing memory and other house keeping operations 
 *            when IPv4 component is being destroyed.  
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOM   - memory allocation failure      
 * IX_CC_IPV4_ERROR_FAILED_PATCHING   - patching failures                      
 * IX_CC_ERROR_OOM_64BIT_COUNTER - 64 bit counter creation failure
 * IX_CC_IPV4_ERROR_REGISTRY  - registry access failures
 * IX_CC_IPV4_ERROR_RTM - failure from RTM core component
 * IX_CC_IPV4_ERROR_CCI - failure from CCI 
 * 
 */

ix_error ix_cc_ipv4_init(
                         ix_cc_handle arg_hCcHandle,
                         void** arg_ppContext
                        )
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS

       npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s
            \n\targ_hCcHandle : %d
            \n\targ_ppContext : 0x%x
            \n\t\t*arg_ppContext : 0x%x
            \n",
            __FUNCTION__,
            arg_hCcHandle,
            arg_ppContext,
            (arg_ppContext) ? *arg_ppContext : 0);
#endif /* SQESTUBS */
    
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_fini
 *
 * DESCRIPTION: This primitive shall be called to terminate services from IPv4 forwarder 
 * component. This primitive frees memory allocated during initialization, shut down RTM, 
 * free all the created resources: delete 64 bit counter. If client claims for any services 
 * from IPv4 after scalling this function, then the behavior is undefined.
 * 
 * @Param:  - IN arg_hCcHandle - handle to IPv4 Forwader core component
 *
 * @Param:  - IN arg_pContext - ppointer to the control block memory allocated 
 * earlier in ix_cc_ipv4_init function. 
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV4_ERROR_INVALID_INPUT_PARAM - invalid input parameter(s)
 * IX_CC_ERROR _OOM   - failure of memory deletion                
 * IX_CC_ERROR_OOM_64BIT_COUNTER - failure of  64 bit counter deletion 
 * IX_CC_IPV4_ERROR _RTM - failure from RTM
 * IX_CC_IPV4_ERROR_CCI - failure from CCI
 * 
 */
ix_error ix_cc_ipv4_fini(
                         ix_cc_handle arg_hCcHandle,
                         void* arg_pContext
                        )
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS

       npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s 
            \n\targ_hCcHandle : %d
            \n\targ_pContext : 0x%x
            \n",
            __FUNCTION__,
            arg_hCcHandle,
            arg_pContext);
#endif /* SQESTUBS */
    
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_msg_handler
 *
 * DESCRIPTION: This primitive is the message handler function for IPv4 Forwarder
 * core component. IPV4 Forwarder core component receives messages from other core component
 * through this message handler function and internally it calls appropriate library function
 * to process the message. This message handler shall be used to update dynamic properties.
 * 
 * @Param:  - IN arg_hDataToken - buffer handle embedding information for the message
 * passed in arg_UserData.
 *
 * arg_UserData - message type
 *
 * @Param:  - OUT - arg_pComponentContext: pointer to IPv4 Forwarder core component context 
 * that will be passed to the core component when a message arrives. This context was defined
 * by the core component and passed to core components infrastructure through 
 * ix_cc_ipv4_init function.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - null argument
 * IX_CC_IPV4_ERROR_INVALID_CONTEXT - context is unusable
 * IX_CC_ERROR_UNDEFINED_MSG - un supported message
 * IX_CC_IPV4_ERROR_MSG_LIBRARY - error from message support library
 * 
 */
ix_error ix_cc_ipv4_msg_handler(
                                ix_buffer_handle arg_hDataToken,
                                ix_uint32 arg_UserData,
                                void* arg_pContext
                                )
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS

       npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s 
            \n\targ_hDataToken : %d
            \n\targ_UserData : 0x%x
            \n\targ_pContext : 0x%x
            \n",
            __FUNCTION__,
            arg_hDataToken,
            arg_UserData,
            arg_pContext);
#endif /* SQESTUBS */
    
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_microblock_high_priority_pkt_handler
 *
 * DESCRIPTION: This is the only registered function to receive exception packets from high priority
 * queue of IPv4 microblock. This function sends the packet to stack driver
 * (ie, recieved only local delivery packets). 
 * 
 * @Param:  - IN arg_hDataToken: handle to a buffer which contains exception packets
 *  from IPv4 microblock
 * @Param: - IN arg_ExceptionCode: will be ignored.
 * @Param: - IN arg_pContext: pointer to IPV4 Forwarder core component context that 
 * will be passed to the core component when a packet arrives. This context was defined by
 * the core component and passed to core components infrastructure through 
 * the ix_cc_ipv4_init function.
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV4_ERROR_CCI - error from CCI
 * 
 */

ix_error ix_cc_ipv4_microblock_high_priority_pkt_handler(
                                                         ix_buffer_handle arg_hDataToken,  
                                                         ix_uint32 arg_ExceptionCode,
                                                         void* arg_pContext
                                                        )
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS

    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_hDataToken : %d
        \n\targ_ExceptionCode : 0x%x
        \n\targ_pContext : 0x%x
        \n",
        __FUNCTION__,
        arg_hDataToken,
        arg_ExceptionCode,
        arg_pContext);
#endif /* SQESTUBS */
    
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_microblock_low_priority_pkt_handler
 *
 * DESCRIPTION: TThis is the registered function to receive exception packets from 
 * IPv4 microblock. This function internally calls/performs different functions/operations
 * based on exception code for the given packet.
 * 
 * @Param:  - IN arg_hDataToken: handle to a buffer which contains exception packets
 * from IPv4 microblock
 * @Param: - IN arg_ExceptionCode: Exception codes generated by IPv4 forwarder microblock.
 * @Param: - IN arg_pContext: pointer to IPV4 Forwarder core component context that 
 * will be passed to the core component when a packet arrives. This context was defined by
 * the core component and passed to core components infrastructure through 
 * the ix_cc_ipv4_init function.
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV4_ERROR_CCI - error from CCI
 * IX_CC_ERROR_UNDEFINED_EXCEP - unsupported exception code
 * IX_CC_ERROR_INTERNAL - an internal error
 * IX_CC_ERROR_ALIGN - pointer not aligned properly
 * IX_CC_ERROR_NULL - null pointer
 * IX_CC_IPV4_ERROR_RTM - error from RTM
 *
 *
 */
ix_error ix_cc_ipv4_microblock_low_priority_pkt_handler(
                                                        ix_buffer_handle arg_hDataToken,  
                                                        ix_uint32 arg_ExceptionCode,
                                                        void* arg_pContext
                                                       )
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS
    
    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_hDataToken : %d
        \n\targ_ExceptionCode : 0x%x
        \n\targ_pContext : 0x%x
        \n",
        __FUNCTION__,
        arg_hDataToken,
        arg_ExceptionCode,
        arg_pContext);

#endif /* SQESTUBS */
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_stkdrv_pkt_handler
 *
 * DESCRIPTION: This is the registered function to receive packets from stack driver 
 * core component. Packet comes from stack driver need special processing by 
 * IPv4 core component: no TTL decrement, no IP header validation.
 * 
 * @Param:  - IN arg_hDataToken: handle to a buffer which contains packet
 * from stack driver core component. - 
 *
 * @Param:  - IN arg_ExceptionCode:  will be Ignored 
 *
 * @Param:  - IN arg_pComponnetContext: pointer to IPV4 Forwarder core component context
 * that will be passed to the core component when a packet arrives. This context was 
 * defined by the core component and passed to core components infrastructure through 
 * the ix_cc_ipv4_init function. 
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV4_ERROR_CCI - error from CCI
 * IX_CC_ERROR_INTERNAL - an internal error
 * IX_CC_ERROR_ALIGN - pointer not aligned properly
 * IX_CC_ERROR_NULL - null pointer
 * 
 */
ix_error ix_cc_ipv4_stkdrv_pkt_handler(
                                           ix_buffer_handle arg_hDataToken,
                                           ix_uint32 arg_ExceptionCode,
                                           void* arg_pContext
                                           )
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS

    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_hDataToken : %d
        \n\targ_ExceptionCode : 0x%x
        \n\targ_pContext : 0x%x
        \n",
        __FUNCTION__,
        arg_hDataToken,
        arg_ExceptionCode,
        arg_pContext);
#endif /* SQESTUBS */
    
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_common_pkt_handler
 *
 * DESCRIPTION: This is the registered function to receive packets 
 * from any core component other than stack driver: interface Rx core component.
 * 
 * @Param:  - IN arg_hDataToken: handle to a buffer which contains packet from any core component other than stack driver.
 * @Param:  - INarg_ExceptionCode:  will be Ignored
 * @Param:  - INarg_pComponnetContext: pointer to IPV4 Forwarder core component context 
 * that will be passed to the core component when a packet arrives. 
 * This context was defined by the core component and passed to core components infrastructure
 * through the ix_cc_ipv4_init function.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_INTERNAL - an internal error
 * IX_CC_ERROR_ALIGN - pointer not aligned properly
 * IX_CC_ERROR_NULL - null pointer
 * 
 */
ix_error ix_cc_ipv4_common_pkt_handler(
                                           ix_buffer_handle arg_hDataToken,
                                           ix_uint32 arg_ExceptionCode,
                                           void* arg_pContext
                                           )
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS

    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_hDataToken : %d
        \n\targ_ExceptionCode : 0x%x
        \n\targ_pContext : 0x%x
        \n",
        __FUNCTION__,
        arg_hDataToken,
        arg_ExceptionCode,
        arg_pContext);
#endif /* SQESTUBS */
    
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_add_route
 *
 * DESCRIPTION:This library primitive helps to add a route in RTM. 
 * This API internally calls ix_cc_rtmv4_add_route API of RTM core component to add route in RTM database.
 * 
 * @Param:  - IN arg_IpAddr: network IP address identifying the route.
 * @Param:  - IN arg_NetMask: network mask for the route.
 *  @Param:  - IN arg_NextHopId: next hop identifier for route.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_ENTRY_NOT_FOUND - invalid next hop identifier
 * IX_CC_ERROR_DUPLICATE_ROUTE - route was already added to this NHID
 * IX_CC_ERROR_CONFLICTING_ROUTE - route was already added to another NHID
 * IX_CC_IPV4_ERROR_RTM - RTM internal failure
 * 
 */

ix_error ix_cc_ipv4_add_route(
                              ix_uint32 arg_IpAddr,
                              ix_uint32 arg_NetMask,
                              ix_cc_rtmv4_nhid arg_NextHopId
                              )
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS

    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_IpAddr : 0x%x
        \n\targ_NetMask : 0x%x
        \n\targ_NextHopId : 0x%x
        \n",
        __FUNCTION__,
        arg_IpAddr,
        arg_NetMask,
        arg_NextHopId);
#endif /* SQESTUBS */
    
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_delete_route
 *
 * DESCRIPTION: This library primitive helps to delete a route from RTM. 
 * This API internally calls ix_cc_rtmv4_del_route API of RTM core component to delete route from RTM database.
 * 
 * @Param:  - IN arg_IpAddr: network IP address identifying the route
 *
 * @Param:  - IN arg_NetMask: network mask for the route
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_ENTRY_NOT_FOUND - entry not found in RTM
 * IX_CC_IPV4_ERROR_RTM - RTM internal failure
 * 
 */

ix_error ix_cc_ipv4_delete_route(
                                 ix_uint32 arg_IpAddr,
                                 ix_uint32 arg_NetMask
                                )
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS

    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_IpAddr : 0x%x
        \n\targ_NetMask : 0x%x
        \n",
        __FUNCTION__,
        arg_IpAddr,
        arg_NetMask);
#endif /* SQESTUBS */
    
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_update_route
 *
 * DESCRIPTION:This library primitive helps to add a route in RTM. 
 * This API internally calls ix_cc_rtmv4_update_route API of RTM core component to 
 * add route in RTM database. 
 * 
 * @Param:  - IN arg_IpAddr: network IP address the route
 * @Param:  - IN arg_NetMask: network mask for the route
 * @Param:  - IN arg_NextHopId: hop identifier for route to be updated
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_ENTRY_NOT_FOUND - invalid next hop identifier
 * IX_CC_IPV4_ERROR_INVALID_MASK - invalid netmask
 * IX_CC_ERROR_CONFLICTING_ROUTE - route was already added to another NHID
 * IX_CC_IPV4_ERROR_RTM - RTM internal failure
 * 
 */

ix_error ix_cc_ipv4_update_route(ix_uint32 arg_IpAddr,
                              ix_uint32 arg_NetMask,
                              ix_cc_rtmv4_nhid arg_NextHopId
                             )
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS

    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_IpAddr : 0x%x
        \n\targ_NetMask : 0x%x
        \n\targ_NextHopId : 0x%x
        \n",
        __FUNCTION__,
        arg_IpAddr,
        arg_NetMask,
        arg_NextHopId);
#endif /* SQESTUBS */
    
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_lookup_route
 *
 * DESCRIPTION: This primitive helps a client to look up routing information for a given IP address.
 * If there is a successful match, then the information will be given to client are nextHopId (including the blade id),
 * portId, mtu and flags. 
 * 
 * @Param:  - IN arg_IpAddr: network IP address identifying the route to be updated.
 * @Param:  - INarg_NetMask: network mask for the route to be updated.
 * @Param:  - INarg_NextHopId: next hop identifier for the route to be updated.
- 
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - null pinter
 * IX_CC_IPV4_ERROR_RTM - RTM internal error
 * 
 */

ix_error ix_cc_ipv4_lookup_route(
                                 ix_uint32 arg_IpAddr,
                                 ix_cc_rtmv4_next_hop_info *arg_pNextHopInfo)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS

       npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_IpAddr : 0x%x
        \n\targ_pNextHopInfo : 0x%x
        \n\t\targ_pNextHopInfo->bladeID : 0x%x
        \n\t\targ_pNextHopInfo->l2Index : 0x%x
        \n\t\targ_pNextHopInfo->portID : 0x%x
        \n\t\targ_pNextHopInfo->mtu : 0x%x
        \n\t\targ_pNextHopInfo->flags : 0x%x
        \n\t\targ_pNextHopInfo->ipAddr : 0x%x
        \n",
        __FUNCTION__,
        arg_IpAddr,
        arg_pNextHopInfo,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->bladeID : 0,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->l2Index : 0,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->portID : 0,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->mtu : 0,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->flags : 0,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->ipAddr : 0);
#endif /* SQESTUBS */
    
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_purge_routes
 *
 * DESCRIPTION: This primitive helps a client to remove all routes from RTM.
 * 
 * @Param:  - IN  -  none
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - null pointer
 * IX_CC_ERROR_ALIGN - pointer not aligned properly
 * 
 */
ix_error ix_cc_ipv4_purge_routes()
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS
       npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n",
        __FUNCTION__);
#endif /* SQESTUBS */
    
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_dump_routes
 *
 * DESCRIPTION: This primitive helps to dump all routes of RTM in memory
 * 
 * @Param:  - IN arg_pBuffer - pointer to a block of memory where routes are to be stored.
 * @Param:  - IN arg_Size - size of buffer in bytes.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - null pointer
 * IX_CC_ERROR_ALIGN - pointer not aligned properly
 * IX_CC_ERROR_FULL -  buffer is too small
 */
ix_error ix_cc_ipv4_dump_routes(
                                char* arg_pBuffer,
                                ix_uint32 arg_Size)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS
       npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_pBuffer : 0x%x
        \n\targ_Size : 0x%x
        \n",
        __FUNCTION__,
        arg_pBuffer,
        arg_Size);
#endif /* SQESTUBS */
    
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_add_next_hop
 *
 * DESCRIPTION: This primitive helps a client to add next hop information to RTM database.
 * Client needs to add next hop to RTM in order to add routes referring to that next hop. 
 * 
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop to be added.
 * @Param:  - IN arg_pNextHopData: defined by RTM
 *
 * @Param:  - OUT  none 
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_ERROR_DUPLICATE_ROUTE - route was already added to this NHID
 * IX_CC_ERROR_CONFLICTING_ROUTE - route was already added to another NHID
 * IX_CC_IPV4_ERROR_RTM - RTM internal failure
 * IX_CC_IPV4_ERROR_INVALID_NH_INFO - invalid next hop information
 */

ix_error ix_cc_ipv4_add_next_hop(
                               ix_cc_rtmv4_nhid arg_NextHopId,
                               ix_cc_rtmv4_next_hop_info* arg_pNextHopInfo)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
#ifdef SQESTUBS
       npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_NextHopId : %d
        \n\targ_pNextHopInfo : %d
        \n\t\targ_pNextHopInfo->bladeID : %d
        \n\t\targ_pNextHopInfo->l2Index : %d
        \n\t\targ_pNextHopInfo->portID : %d
        \n\t\targ_pNextHopInfo->mtu : %d
        \n\t\targ_pNextHopInfo->flags : %d
        \n\t\targ_pNextHopInfo->ipAddr : %d
        \n",
        __FUNCTION__,
        arg_NextHopId,
        arg_pNextHopInfo,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->bladeID : 0,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->l2Index : 0,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->portID : 0,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->mtu : 0,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->flags : 0,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->ipAddr : 0);

#endif /* SQESTUBS */
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_delete_next_hop
 *
 * DESCRIPTION: This primitive helps a client to delete next hop information from RTM database. 
 * Client needs to remove all routes associated with next hop before calling this primitive. 
 * If not, client gets an error.This API internally calls ix_cc_rtmv4_delete_next_hop API of RTM
 * to delete next hop information from RTM.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop to be removed.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV4_ERROR_RTM - RTM internal failure
 * IX_CC_ERROR_ENTRY_NOT_FOUND - invalid next hop id
 * IX_CC_IPV4_ERROR_NEXT_HOP_ID_IN_USE -next hop id is in use
 */

ix_error ix_cc_ipv4_delete_next_hop(ix_cc_rtmv4_nhid arg_pNextHopId)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
#ifdef SQESTUBS
       npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_pNextHopId : %d
        \n",
        __FUNCTION__,
        arg_pNextHopId);

#endif /* SQESTUBS */
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_update_next_hop
 *
 * DESCRIPTION: This primitive helps a client to add next hop information to RTM database.
 * Client needs to add next hop to RTM in order to add routes referring to that next hop. 
 * 
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop to be added.
 * @Param:  - IN arg_pNextHopData: defined by RTM
 *
 * @Param:  - OUT  none 
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_RTM - RTM internal failure
 * IX_CC_IPV4_ERROR_INVALID_NH_INFO - invalid next hop information
 * IX_CC_ERROR_ENTRY_NOT_FOUND - invalid next hop id
 */

ix_error ix_cc_ipv4_update_next_hop(
                               ix_cc_rtmv4_nhid arg_NextHopId,
                               ix_cc_rtmv4_next_hop_info* arg_pNextHopInfo)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
#ifdef SQESTUBS
       npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_NextHopId : %d
        \n\targ_pNextHopInfo : %d
        \n\t\targ_pNextHopInfo->bladeID : %d
        \n\t\targ_pNextHopInfo->l2Index : %d
        \n\t\targ_pNextHopInfo->portID : %d
        \n\t\targ_pNextHopInfo->mtu : %d
        \n\t\targ_pNextHopInfo->flags : %d
        \n\t\targ_pNextHopInfo->ipAddr : %d
        \n",
        __FUNCTION__,
        arg_NextHopId,
        arg_pNextHopInfo,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->bladeID : 0,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->l2Index : 0,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->portID : 0,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->mtu : 0,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->flags : 0,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->ipAddr : 0);

#endif /* SQESTUBS */
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_get_next_hop
 *
 * DESCRIPTION:This library primitive helps to retrieve next hop information from RTM.
 * This API internally calls ix_cc_rtmv4_get_next_hop API of RTM core component to 
 * retrieve next hop information. 
 * 
 * @Param:  - IN arg_NextHopId: Identifier of the next hop information to be retrieved from RTM.
 *
 * @Param:  - OUT arg_pNextHopInfo: Result will be placed into this structure
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV4_ERROR_RTM - RTM internal failure
 * IX_CC_ERROR_ENTRY_NOT_FOUND - invalid next hop id
 * 
 */
ix_error ix_cc_ipv4_get_next_hop(
                                 ix_cc_rtmv4_nhid arg_NextHopId,
                                 ix_cc_rtmv4_next_hop_info* arg_pNextHopInfo
                                )
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
#ifdef SQESTUBS
       npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_NextHopId : %d
        \n\targ_pNextHopInfo : %d
        \n\t\targ_pNextHopInfo->bladeID : %d
        \n\t\targ_pNextHopInfo->l2Index : %d
        \n\t\targ_pNextHopInfo->portID : %d
        \n\t\targ_pNextHopInfo->mtu : %d
        \n\t\targ_pNextHopInfo->flags : %d
        \n\t\targ_pNextHopInfo->ipAddr : %d
        \n",
        __FUNCTION__,
        arg_NextHopId,
        arg_pNextHopInfo,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->bladeID : 0,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->l2Index : 0,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->portID : 0,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->mtu : 0,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->flags : 0,
        (arg_pNextHopInfo) ? arg_pNextHopInfo->ipAddr : 0);

#endif /* SQESTUBS */
    return retObj;
}



/**
 * NAME: ix_cc_ipv4_dump_next_hops
 *
 * DESCRIPTION: This API internally calls ix_cc_rtmv4_dump_next_hop API of RTM core component. 
 * Client shall use IX_CC_RTMV4_DUMP_NEXT_HOP_SIZE to determine how many bytes to allocate.
 * 
 * @Param:  - IN arg_pBuffer - pointer to a block of memory where next hops are to be stored.
 * @Param:  - IN arg_Size - size of buffer in bytes
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV4_ERROR_RTM - RTM failure
 * 
 */

ix_error ix_cc_ipv4_dump_next_hops(
                                char* arg_pBuffer,
                                ix_uint32 arg_Size)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS
       npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_pBuffer : 0x%x
        \n\targ_Size : 0x%x
        \n",
        __FUNCTION__,
        arg_pBuffer,
        arg_Size);
#endif /* SQESTUBS */
    
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_purge_rtm
 *
 * DESCRIPTION: This primitive helps a client to removes all routes and next hops from RTM database.
 * 
 * @Param:  - IN - none 
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV4_ERROR_RTM - RTM failure
 * 
 */
ix_error ix_cc_ipv4_purge_rtm()
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS

    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n",
        __FUNCTION__);
#endif /* SQESTUBS */
    
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_set_mtu
 *
 * DESCRIPTION: This primitive helps client to update MTU for a given next hop. 
 * Before calling this, client needs to call ix_cc_ipv4_add_next_hop to add next hop 
 * identifier in RTM database. The validity of MTU shall be done by the client.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop.
 * @Param:  - arg_Mtu: new maximum transmission unit for the given next hop.
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV4_ERROR_RTM - RTM failure
 * IX_CC_ERROR_ENTRY_NOT_FOUND - invalid next hop id 
 * IX_CC_IPV4_ERROR_INVALID_NH_INFO - invalid mtu
 */

ix_error ix_cc_ipv4_set_mtu(
                            ix_cc_rtmv4_nhid arg_NextHopId,
                            ix_uint32 arg_Mtu)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS

    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_NextHopId : %d
        \n\targ_Mtu : 0x%x
        \n",
        __FUNCTION__,
        arg_NextHopId,
        arg_Mtu);
#endif /* SQESTUBS */
    
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_set_flags
 *
 * DESCRIPTION:This primitive helps client to update flags for a given next hop.
 * Before calling this primitive, client needs to call ix_cc_ipv4_add_next_hop to 
 * add next hop identifier in RTM database. 
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop.
 * @Param:  - IN arg_Flags: new flags for the given next hop 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV4_ERROR_RTM - RTM failure
 * IX_CC_ERROR_ENTRY_NOT_FOUND - invalid next hop id 
 * IX_CC_IPV4_ERROR_INVALID_FLAGS - invalid flags 
 * 
 */
ix_error ix_cc_ipv4_set_flags(
                              ix_cc_rtmv4_nhid arg_NextHopId,
                              ix_uint32 arg_Flags)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS
    
    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_NextHopId : %d
        \n\targ_Flags : 0x%x
        \n",
        __FUNCTION__,
        arg_NextHopId,
        arg_Flags);

#endif /* SQESTUBS */
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_get_sleep_time
 *
 * DESCRIPTION: This library primitive helps to retrieve the number of seconds 
 * between when the event will fire causing the ICMP messages to be dequeued and sent. 
 * The lower the value of arg_pSleepTime, the more frequently the ICMP messages will be 
 * dequeued and sent.  
 * 
 * @Param:  - IN  none 
 *
 * @Param:  - OUT arg_pSleepTime: ICMP sleep period 
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - null pointer
 * 
 */
ix_error ix_cc_ipv4_get_sleep_time(ix_uint16* arg_pSleepTime)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS
    
    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_pSleepTime : 0x%x
        \n\t\t*arg_pSleepTime : %d
        \n",
        __FUNCTION__,
        arg_pSleepTime,
        (arg_pSleepTime) ? *arg_pSleepTime : 0);

#endif /* SQESTUBS */
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_set_sleep_time
 *
 * DESCRIPTION: This library primitive helps to set the period between events firing to process 
 * the ICMP error message queue. The lower the value of arg_SleepTime, the more frequently
 * the output queue will be processed. The minimum value of arg_SleepTime is 1 sec and 
 * maximum value can be 32sec.
 * 
 * @Param:  - IN arg_SleepTime: the number of seconds to wait between processing
 * the ICMP error message queue.
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV4_INVALID_SLEEP_TIME - invalid sleep time
 * 
 */

ix_error ix_cc_ipv4_set_sleep_time(ix_uint16 arg_pSleepTime)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
#ifdef SQESTUBS
    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_pSleepTime : 0x%x
        \n",
        __FUNCTION__,
        arg_pSleepTime);

#endif /* SQESTUBS */
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_get_queue_depth
 *
 * DESCRIPTION: This library primitive helps to retrieves the depth of the ICMP error message 
 * queue. That is, the maximum number of ICMP messages that can fit in the queue, not the
 * number of messages currently in the queue.
 * 
 * @Param:  - IN none 
 *
 * @Param:  - OUT arg_pQueueDepth: the current number of entries that the ICMP error message queue can hold.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - null pointer
 * 
 */
ix_error ix_cc_ipv4_get_queue_depth(ix_uint16* arg_pQueueDepth)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS

    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_pQueueDepth : 0x%x
        \n\t\t*arg_pQueueDepth : %d
        \n",
        __FUNCTION__,
        arg_pQueueDepth,
        (arg_pQueueDepth) ? *arg_pQueueDepth : 0);

#endif /* SQESTUBS */
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_get_packets_to_drain
 *
 * DESCRIPTION:This library primitive helps to retrieve the maximum number of ICMP error messages
 * to send (from the ICMP message queue ) each time the queue processing event fires. 
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT arg_pPacketsToDrain: the maximum number of packets to send each time the 
 * event fires for processing the queue.  
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * 
 */
ix_error ix_cc_ipv4_get_packets_to_drain(ix_uint16* arg_pPacketsToDrain)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
#ifdef SQESTUBS
    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_pPacketsToDrain : 0x%x
        \n\t\t*arg_pPacketsToDrain : %d
        \n",
        __FUNCTION__,
        arg_pPacketsToDrain,
        (arg_pPacketsToDrain) ? *arg_pPacketsToDrain : 0);

#endif /* SQESTUBS */
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_set_packets_to_drain
 *
 * DESCRIPTION: This library primitive helps to set the maximum number of ICMP error messages 
 * to send each time the queue processing event fires. The value of arg_PacketsToDrain
 * must not exceed the depth of the queue.
 * 
 * @Param:  - IN arg_PacketsToDrain: The maximum number of packets to send each time the queue processing
 * event fires. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV4_INVALID_PACKETS_TO_DRAIN - invalid value
 * 
 */
ix_error ix_cc_ipv4_set_packets_to_drain(ix_uint16 arg_pPacketsToDrain)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS

    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_pPacketsToDrain : %d
        \n",
        __FUNCTION__,
        arg_pPacketsToDrain);

#endif /* SQESTUBS */
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_get_statistics
 *
 * DESCRIPTION: This library primitive helps to retrieve statistics report from IPV4 Forwarder 
 * core component. This primitive will give the statistics of IPV4 microblock, RTM and 
 * IPV4 Forwarder core component. Client needs to allocate memory to hold data.
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT arg_pBuffer: pointer to memory to hold statistics data.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - null pointer
 * 
 */
ix_error ix_cc_ipv4_get_statistics(ix_cc_ipv4_stats_data* arg_pBuffer)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS
      
      npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_pBuffer : 0x%x
        \n",
        __FUNCTION__,
        arg_pBuffer);
   
#endif /* SQESTUBS */
    return retObj;
}

/**
 * NAME: ix_cc_ipv4_set_property
 *
 * DESCRIPTION: This library primitive helps to set dynamic properties of 
 * IPv4 Forwarder core component. 
 * 
 * @Param:  - IN arg_PropId: Identifier of a property or properties(ORed) to be updated 
 * for a port
 * @Param:  - IN arg_pPropData: pointer to dynamic property data structure
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * IX_CC_ERROR_NUL - invalid pointer
 * IX_CC_IPV4_ERROR_INVALID_INPUT_PARAM - invalid input param
 * IX_ERROR_FULL - table is full
 * IX_CC_ERROR_ENTRY_NOT_FOUND - entry not found in table
 *
 */
#ifdef SQESTUBS
#define build_char_string ( x, size, str ) \
    int i;    \
    char str[size*3 + 1];    \
    for (i=0;i<size;i++)\
    {    \
        sprinf("%02x ",x[i]); \
    }    \
    str[size*3+1]=`\0`;
#endif /* SQESTUBS */
ix_error ix_cc_ipv4_set_property(ix_uint32 arg_PropId,
                                 ix_cc_properties* arg_pPropData)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);

#ifdef SQESTUBS
    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s 
        \n\targ_PropId : 0x%x
        \n\targ_pPropData : 0x%x
          \n\t\targ_pPropData->blade_id : 0x%x
        \n\t\targ_pPropData->port_id : 0x%x
        \n\t\targ_pPropData->ip_version : 0x%x
        \n\t\t\targ_pPropData->ip_prop.ipv4_address : 0x%x
        \n\t\t\targ_pPropData->ip_prop.ipv4_subnet_mask : 0x%x
        \n\t\t\targ_pPropData->ip_prop.ipv4_broadcast : 0x%x
        \n\t\t\targ_pPropData->ip_prop.ipv4_gateway : 0x%x
        \n\t\targ_pPropData->mtu : 0x%x
        \n\t\targ_pPropData->physical_if_state : 0x%x
        \n\t\targ_pPropData->link_speed : 0x%x
        \n\t\targ_pPropData->mac_addr[IX_CC_MAC_ADDR_LEN] : 0x%x
        \n\t\targ_pPropData->media_type; : 0x%x
        \n",
        __FUNCTION__,
        arg_PropId,
        arg_pPropData,
           (arg_pPropData) ? arg_pPropData->blade_id : 0 ,
        (arg_pPropData) ? arg_pPropData->port_id : 0 , 
        (arg_pPropData) ? arg_pPropData->ip_version : 0 ,
        (arg_pPropData) ? arg_pPropData->ip_prop.ipv4_prop.ipv4_address : 0 ,
        (arg_pPropData) ? arg_pPropData->ip_prop.ipv4_prop.ipv4_subnet_mask : 0 ,
        (arg_pPropData) ? arg_pPropData->ip_prop.ipv4_prop.ipv4_broadcast : 0 ,
        (arg_pPropData) ? arg_pPropData->ip_prop.ipv4_prop.ipv4_gateway : 0 ,
        (arg_pPropData) ? arg_pPropData->mtu : 0,
        (arg_pPropData) ? arg_pPropData->physical_if_state : 0,
        (arg_pPropData) ? arg_pPropData->link_speed : 0,
        (arg_pPropData) ? arg_pPropData->mac_addr[IX_CC_MAC_ADDR_LEN] : 0,
        (arg_pPropData) ? arg_pPropData->media_type : 0
        );
   
#endif /* SQESTUBS */
    return retObj;
}
#endif

ix_error ix_cc_rtmv4_get_next_hop (
         ix_cc_rtmv4 arg_hRtm,
         ix_cc_rtmv4_nhid arg_NextHopId,
         ix_cc_rtmv4_next_hop_info *arg_pNextHopInfo)
{
   static ix_uint32 counter = 0;
   ix_error retObj;

   SetIXAReturnObject(&retObj);

   arg_pNextHopInfo->bladeID = 2;
   arg_pNextHopInfo->l2Index = counter++;
   arg_pNextHopInfo->portID = 1;
   arg_pNextHopInfo->mtu = 1450;
   arg_pNextHopInfo->flags = 0;
   arg_pNextHopInfo->ipAddr = 0x0a010007;

   return retObj;
}  

ix_error ix_cc_rtmv4_get_route (
         ix_cc_rtmv4 arg_hRtm,
         ix_uint32 arg_NetAddr,
         ix_uint32 arg_NetMask,
         ix_cc_rtmv4_nhid *arg_pNextHopId)
{
   static ix_uint32 counter = 1;
   ix_error retObj;
   static ix_uint32 flag = 0;

   SetIXAReturnObject(&retObj);

   *arg_pNextHopId = counter++;
   
   /* Toggle for Unified case to simulate real
      FP scenario. For Discrete, we are going to 
      get FAIL/SUCCESS alternately */
   
   if (flag == 0)
   {
      flag = 1; 
      counter--;
      return 2;
   }
   else
      flag = 2;
   return 2;
} 
