/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ForwardingPlane/FPModule/IXASDKStubs/src/ixasdk_ipv6_stubs.c,v $
 * $Revision: 1.5 $
 * $Date: 2003/11/17 16:57:23 $
 *
 * Description:
 *      Implements method to create event loops in new threads. The thread
 *      method is also contained here. The event threads wait for and
 *      processes messages from the CP and FP and then call the appropriate
 *      methods in the FP or CP.
 *
 ****************************************************************************/

#include <time.h>
#include "list.h"
#include "log.h"
#include "ixasdk_methodmap.h"
#include "ixasdk_stubs_pri.h"

#include "cci/cc.h"
#include "ix_cc.h"
#include "cc/ix_cc_rtmv4.h"
#include "cc/ix_cc_rtmv6.h"
#include "cc/ix_cc_ipv6.h"
#include "cc/ix_cc_fpm_core.h"

//#include "ix_cc_ipv6.h"
//#include "ix_error.h"


extern QUEUE_SERVICE_PARAMS g_stubsParms;


/**
 * NAME: ix_cc_ipv6_async_add_prefix
 *
 * DESCRIPTION: This message helper primitive helps client to add a prefix in RTM. 
 * Client needs to successfully add nextHopId to RTM by calling ix_cc_ipv6_async_add_next_hop
 * messaging helper primitive before sending this message to IPv6 Forwarder core component. 
 * 
 * @Param:  - IN arg_IpAddr: network IP address identifying the prefix.
 * @Param:  - IN arg_NetMask: network mask for the prefix.
 * @Param:  - IN arg_NextHopId: next hop identifier for prefix.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library
 * 
 */


ix_error ix_cc_ipv6_async_add_prefix(ix_uint128 arg_IPv6Addr,
                                    ix_uint8 arg_PrefixLen,
                                    ix_cc_rtmv6_nhid arg_NextHopId,
                                    ix_cc_ipv6_cb_route_op arg_Callback,
                                    void* arg_pUserContext)
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s"
            "\n\targ_IPv6Addr : 0x%x:0x%x:0x%x:0x%x"
            "\n\targ_PrefixLen : 0x%x"
            "\n\targ_NextHopId : 0x%x"
            "\n\targ_Callback : 0x%x"
            "\n\targ_pUserContext : 0x%x"
            "\n",
            __FUNCTION__,
            ((uint32_t*)&arg_IPv6Addr)[0],((uint32_t*)&arg_IPv6Addr)[1],((uint32_t*)&arg_IPv6Addr)[2],((uint32_t*)&arg_IPv6Addr)[3],
            arg_PrefixLen,
            arg_NextHopId,
            arg_Callback,
            arg_pUserContext
            );

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv6_async_add_prefix,
            STUBS_IPv6_ROUTE_OP,
            arg_Callback,
            arg_pUserContext);

    }

    return retObj;
}

/**
 * NAME: ix_cc_ipv6_async_delete_prefix
 *
 * DESCRIPTION:This message helper primitive helps a client to delete a prefix in RTM. 
 * Client needs to delete all prefixes associate with a next hop to delete the next hop
 * by calling ix_cc_ipv6_async_delete_next_hop message helper primitive. 
 * 
 * @Param:  - IN arg_IpAddr: network IP address identifying the prefix.
 * @Param:  - IN arg_NetMask: network mask for the prefix.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */


ix_error ix_cc_ipv6_async_delete_prefix(ix_uint128 arg_IPv6Addr,
                                       ix_uint8 arg_PrefixLen,
                                       ix_cc_ipv6_cb_route_op arg_Callback,
                                       void* arg_pUserContext
                                      )
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s"
            "\n\targ_IPv6Addr : 0x%x:0x%x:0x%x:0x%x"
            "\n\targ_PrefixLen : 0x%x"
            "\n\targ_Callback : 0x%x"
            "\n\targ_pUserContext : 0x%x"
            "\n",
            __FUNCTION__,
            ((uint32_t*)&arg_IPv6Addr)[0],((uint32_t*)&arg_IPv6Addr)[1],((uint32_t*)&arg_IPv6Addr)[2],((uint32_t*)&arg_IPv6Addr)[3],
            arg_PrefixLen,
            arg_Callback,
            arg_pUserContext
            );

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv6_async_delete_prefix,
            STUBS_IPv6_ROUTE_OP,
            arg_Callback,
            arg_pUserContext);

    }

    return retObj;
}


/**
 * NAME: ix_cc_ipv6_async_flush_prefixes
 *
 * DESCRIPTION: This primitive helps a client to remove all prefixes from RTM.
 * 
 * @Param:  - IN arg_CcHandle - 
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */

ix_error ix_cc_ipv6_async_purge_prefixes()
{
    ix_error retObj;

    SetIXAReturnObject(&retObj);
    
    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s"
            "\n",
            __FUNCTION__
            );

#endif /* SQESTUBS */

    }
    return retObj;
}


/**
 * NAME: ix_cc_ipv6_async_add_next_hop
 *
 * DESCRIPTION: This primitive helps a client to add next hop information to RTM database. 
 * Client needs to add next hop to RTM in order to add routes referring to that next hop.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop to be added
 * @Param:  - IN arg_pNextHopData: pointer to next hop information
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined contex
 * 
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_ipv6_async_add_next_hop(
                               ix_cc_rtmv6_nhid arg_NextHopId,
                               ix_uint32        arg_NumNextHops,
                               ix_cc_rtmv6_next_hop_info* arg_pNextHopData,
                               ix_cc_ipv6_cb_route_op arg_Callback,
                               void* arg_pUserContext
                               )
{
    ix_error retObj;
    
    /* Set up return object */
    SetIXAReturnObject(&retObj);

    if (retObj == (ix_error)0)
    {
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv6_async_add_next_hop,
            STUBS_IPv6_ROUTE_OP,
            arg_Callback,
            arg_pUserContext);
    }

    return retObj;
}

/**
 * NAME: ix_cc_ipv6_async_update_next_hop
 *
 * DESCRIPTION: This primitive helps a client to update next hop information in
 *             the  RTM database. 
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop to be added
 * @Param:  - IN arg_NumNextHops: The number of next hops in the next hop entry.
 * @Param:  - IN arg_pNextHopData: pointer to next hop information
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined contex
 * 
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_ipv6_async_update_next_hop(
                                ix_cc_rtmv6_nhid arg_NextHopId,
                                ix_uint32        arg_NumNextHops,
                                ix_cc_rtmv6_next_hop_info* arg_pNextHopData,
                                ix_cc_ipv6_cb_route_op arg_Callback,
                                void* arg_pUserContext
                                )
{
    ix_error retObj;
    
    /* Set up return object */
    SetIXAReturnObject(&retObj);

    if (retObj == (ix_error)0)
    {
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv6_async_update_next_hop,
            STUBS_IPv6_ROUTE_OP,
            arg_Callback,
            arg_pUserContext);

    }

    return retObj;
}

/**
 * NAME: ix_cc_ipv6_async_delete_next_hop
 *
 * DESCRIPTION: This message helper primitive helps a client to delete next hop information 
 * from RTM database. Client needs to remove all routes associated with next hop before 
 * calling this primitive. If not, client gets an error.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop to be removed.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 */


ix_error ix_cc_ipv6_async_delete_next_hop(ix_cc_rtmv6_nhid arg_NextHopId,
                                          ix_cc_ipv6_cb_route_op arg_Callback,
                                          void* arg_pUserContext
                                          )
{
    ix_error retObj;
    
    /* Set up return object */
    SetIXAReturnObject(&retObj);

    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s"
            "\n\targ_NextHopId : 0x%x"
            "\n\targ_Callback : 0x%x"
            "\n\targ_pContext : 0x%x"
            "\n",
            __FUNCTION__,
            arg_NextHopId,
            arg_Callback,
            arg_pUserContext
            );

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv6_async_delete_next_hop,
            STUBS_IPv6_ROUTE_OP,
            arg_Callback,
            arg_pUserContext);

    }

    return retObj;
}

/**
 * NAME: ix_cc_ipv6_async_purge_rtm
 *
 * DESCRIPTION: This primitive helps a client to removes all routes and next hops from RTM database
 * 
 * @Param:  - IN  - none
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_IPV6_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */


ix_error ix_cc_ipv6_async_purge_rtm()
{
    ix_error retObj;

    SetIXAReturnObject(&retObj);
    
    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s"
            "\n",
            __FUNCTION__
            );

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */

    }
    return retObj;
}


/**
 * NAME: ix_cc_ipv6_init
 *
 * DESCRIPTION: This primitive shall be called and returned successfully before requesting any service
 *              from IPv6 Forwarder component. This primitive should be called only once to initialize 
 *              IPv6 Forwarder component.This function performs allocation of memory for symbols to be patched,
 *              Creation of 64 bit counters, registration of packet and message handlers,
 *              initialization and configuration of RTM, allocation and initialization of internal data structures
 *              and creation of event handler for ICMP.
 * 
 * @Param:  - IN arg_hCcHandle - handle to IPV6 core component, created by core component infrastructure; 
 *            this shall be used later to get other (to add event handler) services from core component infrastructure.
 *
 * @Param:  - OUT arg_ppContext - location where the pointer to the control block allocated by 
 *            IPv6 Forwarder core component will be stored. The control block is internal to IPv6 
 *            Forwarder core component and it contains information about IPv6 internal data structures, 
 *            allocated memory and other relevant information. Later this pointer to be passed into  
 *            ix_cc_ipv6_fini function for freeing memory and other house keeping operations 
 *            when IPv6 component is being destroyed.  
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOM   - memory allocation failure      
 * IX_CC_IPV6_ERROR_FAILED_PATCHING   - patching failures                      
 * IX_CC_ERROR_OOM_64BIT_COUNTER - 64 bit counter creation failure
 * IX_CC_IPV6_ERROR_REGISTRY  - registry access failures
 * IX_CC_IPV6_ERROR_RTM - failure from RTM core component
 * IX_CC_IPV6_ERROR_CCI - failure from CCI 
 * 
 */

ix_error ix_cc_ipv6_init(
                         ix_cc_handle arg_hCcHandle,
                         void** arg_ppContext
                        )
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
#ifdef SQESTUBS

    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s"
        "\n\targ_hCcHandle : 0x%x"
        "\n\targ_ppContext : 0x%x"
        "\n",
        __FUNCTION__,
        arg_hCcHandle,
        arg_ppContext
        );
#endif /* SQESTUBS */
    
    return retObj;
}

/**
 * NAME: ix_cc_ipv6_fini
 *
 * DESCRIPTION: This primitive shall be called to terminate services from IPv6 forwarder 
 * component. This primitive frees memory allocated during initialization, shut down RTM, 
 * free all the created resources: delete 64 bit counter. If client claims for any services 
 * from IPv6 after scalling this function, then the behavior is undefined.
 * 
 * @Param:  - IN arg_hCcHandle - handle to IPv6 Forwader core component
 *
 * @Param:  - IN arg_pContext - ppointer to the control block memory allocated 
 * earlier in ix_cc_ipv6_init function. 
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_IPV6_ERROR_INVALID_INPUT_PARAM - invalid input parameter(s)
 * IX_CC_ERROR _OOM   - failure of memory deletion                
 * IX_CC_ERROR_OOM_64BIT_COUNTER - failure of  64 bit counter deletion 
 * IX_CC_IPV6_ERROR _RTM - failure from RTM
 * IX_CC_IPV6_ERROR_CCI - failure from CCI
 * 
 */
ix_error ix_cc_ipv6_fini(
                         ix_cc_handle arg_hCcHandle,
                         void* arg_pContext
                        )
{
    ix_error retObj;

    /* Set up return object */
    SetIXAReturnObject(&retObj);
    
#ifdef SQESTUBS
    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s"
        "\n\targ_hCcHandle : 0x%x"
        "\n\targ_pContext : 0x%x"
        "\n",
        __FUNCTION__,
        arg_hCcHandle,
        arg_pContext
        );
#endif /* SQESTUBS */
    return retObj;
}

ix_error ix_cc_ipv6_async_add_neighbor(ix_uint16   arg_l2Index,
                                  ix_uint128  arg_NbrIp,
                                  ix_uint32 arg_OutputPort,
                                  ix_uint8  *agr_L2Addr,
                                  ix_cc_ipv6_cb_route_op arg_Callback,
                                  void* arg_pUserContext)
{
    ix_error retObj;

    SetIXAReturnObject(&retObj);
    
    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s"
            "\n\targ_l2Index : 0x%x"
            "\n\targ_NbrIp : 0x%x:0x%x:0x%x:0x%x"
            "\n\targ_OutputPort : 0x%x"
            "\n\targ_agr_L2Addr : 0x%x"
            "\n\targ_Callback : 0x%x"
            "\n\targ_pUserContext : 0x%x"
            "\n",
            __FUNCTION__,
            arg_l2Index,
            ((uint32_t*)&arg_NbrIp)[0],((uint32_t*)&arg_NbrIp)[1],((uint32_t*)&arg_NbrIp)[2],((uint32_t*)&arg_NbrIp)[3],
            arg_OutputPort,
            agr_L2Addr,
            arg_Callback,
            arg_pUserContext
            );

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv6_async_add_nbr,
            STUBS_IPv6_ROUTE_OP,
            arg_Callback,
            arg_pUserContext);

    }
    return retObj;
}

ix_error ix_cc_ipv6_async_del_neighbor(
                                    ix_uint16 arg_L2Index,
                                    ix_cc_ipv6_cb_route_op arg_Callback,
                                    void* arg_pUserContext)
{
    ix_error retObj;

    SetIXAReturnObject(&retObj);
    
    if (retObj == (ix_error)0)
    {
#ifdef SQESTUBS

        npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
            "%s"
            "\n\targ_L2Index: 0x%x"
            "\n\targ_Callback : 0x%x"
            "\n\targ_pUserContext : 0x%x"
            "\n",
            __FUNCTION__,
            arg_L2Index,
            arg_Callback,
            arg_pUserContext
            );

#endif /* SQESTUBS */
        /* Copy params into local memory and add method call to queue */
        InsertMethodIntoList(
            Method_ix_cc_ipv6_async_del_nbr,
            STUBS_IPv6_ROUTE_OP,
            arg_Callback,
            arg_pUserContext);

    }
    return retObj;
}

