/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.1 for the IXP2000 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      Linux kernel module file.
 *
 * = FILENAME
 *      ix_ph_core_linux_kernel_module.c
 *
 * = DESCRIPTION
 *      The file defines the init and cleanup functions for Linux kernel module for PacketHandler.
 *
 * = AUTHOR
 *       Sankaralingam Thirunavukkarasu 
 *       sankaralingam.thirunavukkarasu@intel.com
 *
 * = AKNOWLEDGEMENTS
 *
 *      
 * = CREATION TIME
 *      09/03/2004
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: 
 */

#include <linux/module.h>
#include "ix_ossl.h"
#include "ix_ph.h"

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

extern ix_error ix_ph_receive_packet_cp (struct sk_buff*);

/* structure for registering with kernel to receive the packets from remote cc */
static struct inet_protocol vipip_protocol = {
	handler:	ix_ph_receive_packet_cp,
	err_handler:	NULL,
	protocol:	IP_PROTO_VIP,
	name:	"IP_PROTO_VIP"
};

MODULE_LICENSE("Proprietary");

int init_module(void)
{
#ifdef IX_DEBUG
     ix_ossl_message_log( "Loading PH core module.\n");
#endif
    /* register with kernel to receive IP_PROTO_VIP packets */
    inet_add_protocol(&vipip_protocol);

    return 0;
}

void cleanup_module(void)
{
#ifdef IX_DEBUG
    ix_ossl_message_log( "Unloading PH core module.\n");
#endif
    /* de-register with kernel*/
    inet_del_protocol(&vipip_protocol);

    return;
}

#endif /* _IX_OS_LINUX_KERNEL_ */
