/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/

#include "pil.h"

#include <stdio.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <arpa/inet.h>
#include <string.h>
#include <unistd.h>

#ifdef _VXWORKS_
#include <sockLib.h>
#include <netBufLib.h>

#include "ioLib.h"
#include "pipeDrv.h"

#define SHUTDOWN_PIPE_NAME "/pipe/packethandler"
#endif
#define SHUTDOWN_PIPE_MSG "exit"

#include <cc/ix_cc_fpm_core.h>

#include "fpp_common.h"
#include "log.h"

#define PH_IP_PROTOCOL 105
#define MAX_MESSAGE_LEN 1500
static int g_ph_socketd;
struct sockaddr_in g_ph_ce_addr;
PilThread g_ph_RcvrThread;

static int g_shutdownPipe[2];
static int phInitialized = 0;

typedef struct ph_header_t
{
    unsigned int portid;
    unsigned int length;
} ph_header;

void ph_write_packet (ix_uint32 bladeid, ix_uint32 portid, 
		      ix_uint32 len, char* buffer);

uint32_t ph_receiver_thread(void* a)
{
    char buf[MAX_MESSAGE_LEN + 20];
    int sd = (int)a;
    struct sockaddr_in from;
    int len;
    int ret;
    struct ip *iph;
    ph_header *ph;

    fd_set rfds;
    int maxfd, retval;

    maxfd = sd;
    if (g_shutdownPipe[0] > sd) maxfd = g_shutdownPipe[0];
    maxfd++;

    npf_logger_write(LOG_PH, LOG_TRACE, 
		     "Entering packet handler receiver thread\n");
    while(1)
    {
        FD_ZERO(&rfds);
        FD_SET(g_shutdownPipe[0],&rfds);
        FD_SET(sd,&rfds);

        retval = select(maxfd, &rfds, NULL, NULL, NULL);

        if (retval < 0)
        {
            npf_logger_write(LOG_PH, LOG_ERR, "Error returned from select\n");
            return -1;
        }
        if (FD_ISSET(sd,&rfds))
        {
            npf_logger_write(LOG_PH, LOG_TRACE, "looking for data\n");
            ret = recvfrom(sd,buf,sizeof(buf),0,(struct sockaddr*)&from,&len);
            if (ret < 0)
            {
                npf_logger_write(LOG_PH, LOG_ERR, "Error from recvfrom %d",errno);
                return 0;
            }
            iph = (struct ip*)buf;
    
            npf_logger_write(LOG_PH, LOG_TRACE, "got some!\n");
            npf_logger_write(LOG_PH, LOG_TRACE,
                "protocol %x, saddr %x, daddr %x, len %d\n",
                iph->ip_p,iph->ip_src,iph->ip_dst,ntohs(iph->ip_len));

            ph = (ph_header*)(buf + sizeof(struct ip));

            npf_logger_write(LOG_PH, LOG_TRACE,
                "portid %d, length %d\n",
                ntohl(ph->portid),ntohl(ph->length));

            ix_cc_fpm_sync_send_packet(
                ntohl(ph->portid),
                (char*)(ph + 1),
                ntohl(ph->length)
	        );
        }
        if (FD_ISSET(g_shutdownPipe[0],&rfds))
        {
            npf_logger_write(LOG_PH, LOG_TRACE, "Got a shutdown event...exiting\n");
            return 0;
        }
    }
    return -1;
}

NPF_RET ph_init(char* ceaddr)
{
    NPF_RET ret = NPF_FAIL;
    int optval = 64*1024;
    int ret1 = 0;

    g_ph_socketd = socket(AF_INET,SOCK_RAW,PH_IP_PROTOCOL);
    if (g_ph_socketd < 0)
    {
        perror("Error creating socket\n");
        return ret;
    }
    ret1 = setsockopt (g_ph_socketd, SOL_SOCKET, SO_RCVBUF, 
                                          &optval, sizeof (optval));
    if (ret1 < 0)
    {
        perror("Error setting SO_RCVBUF for socket\n");
        return ret;
    }
    optval=64*1024;
    ret1 = setsockopt (g_ph_socketd, SOL_SOCKET, SO_SNDBUF, 
                                          &optval, sizeof (optval));
    if (ret1 < 0)
    {
        perror("Error setting SO_SNDBUF for socket\n");
        return ret;
    }

#ifdef _VXWORKS_
    pipeDrv();
    pipeDevCreate(SHUTDOWN_PIPE_NAME, 10, 5);

    g_shutdownPipe[0] = g_shutdownPipe[1] =
        open(SHUTDOWN_PIPE_NAME, O_RDWR, 0666);
#else
    pipe( g_shutdownPipe );
#endif

    g_ph_RcvrThread = PIL_CreateThread (ph_receiver_thread, 0,
					"ph_receiver_thread",0,
					(void*)g_ph_socketd,0);

    memset(&g_ph_ce_addr,0,sizeof(struct sockaddr_in));
    g_ph_ce_addr.sin_family = AF_INET;
    g_ph_ce_addr.sin_addr.s_addr = inet_addr(ceaddr);

    //register callback
    ix_cc_fpm_register_pkt_hdlr_cb((ix_cc_fpm_pkt_hdlr_cb)ph_write_packet);
    phInitialized = 1;
    return ret;
}

void shutdownReceiveThread()
{
    write( g_shutdownPipe[1], SHUTDOWN_PIPE_MSG, strlen(SHUTDOWN_PIPE_MSG)+1 );
    if(phInitialized)
    {
        PIL_WaitForThread(g_ph_RcvrThread,PIL_INFINITE_TIMEOUT);
        PIL_DestroyThread (g_ph_RcvrThread, NULL);
	phInitialized = 0;
    }

    close(g_shutdownPipe[0]);
#ifndef _VXWORKS_
    close(g_shutdownPipe[1]);
#endif
    return;
}

NPF_RET ph_shutdown()
{
    NPF_RET ret;

    shutdownReceiveThread();

    ix_cc_fpm_deregister_pkt_hdlr_cb();

    return ret;
}

void ph_write_packet (ix_uint32 bladeid, ix_uint32 portid, 
		      ix_uint32 len, char* buffer)
{
    char buf[MAX_MESSAGE_LEN + 20];
    ph_header* ph;
    int retval;

    npf_logger_write(LOG_PH, LOG_TRACE, 
		     "Entering packet handler write packet\n");
    memcpy(buf+sizeof(ph_header),buffer,len);

    ph = (ph_header*)buf;
    ph->portid = htonl(portid);
    ph->length = len;

    if(len > (MAX_MESSAGE_LEN - 50))
    {
        npf_logger_write(LOG_PH, LOG_ERR, 
		     "Packet len is more than 1450, Change the mtu of all direct connected host to below 1450\n");
        return;
    } 

    retval = sendto(g_ph_socketd,
            buf,len+sizeof(ph_header),
            0,
            (struct sockaddr*)&(g_ph_ce_addr),
            sizeof(struct sockaddr_in));

    if (-1 == retval)
    {
        npf_logger_write(LOG_PH, LOG_TRACE, 
			 "error sending packet (0x%x)\n",errno);
    }
}

