/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/

#ifndef _QoS_MGR_IMPL_H
#define _QoS_MGR_IMPL_H

#include "npf_ds.h"
#include "cci/cci_types.h"

#define EF_CLASS_ID     0

#define AF11_CLASS_ID   1
#define AF12_CLASS_ID   2
#define AF13_CLASS_ID   3

#define AF21_CLASS_ID   4
#define AF22_CLASS_ID   5
#define AF23_CLASS_ID   6

#define AF31_CLASS_ID   7
#define AF32_CLASS_ID   8
#define AF33_CLASS_ID   9

#define AF41_CLASS_ID   10
#define AF42_CLASS_ID   11
#define AF43_CLASS_ID   12

#define BF_CLASS_ID     15

/*trace messages, function entered/exited*/
#define DS_TRACE(format, args...)  npf_logger_write(LOG_DS, LOG_TRACE, format, ##args)
/*informational message, printf debugging*/
#define DS_INFO(format, args...)   npf_logger_write(LOG_DS, LOG_INFO, format, ##args)
/*error message, not critical, recoverable error*/
#define DS_ERROR(format, args...)  npf_logger_write(LOG_DS, LOG_ERR, format, ##args)
/*ciritcal error, unrecoverable*/
#define DS_CRITICAL(format, args...) npf_logger_write(LOG_DS, LOG_CRIT, format, ##args)

#define DS_FUNCTION() DS_TRACE("%s", "\n=>\n")

typedef enum QOS_DPE_State
{
    DPE_STATE_IDLE,
    DPE_STATE_INUSE,
    DPE_STATE_UNKNOWN
}QOS_DPE_State_t;

typedef struct fpm_qos_dpe
{
    QOS_DPE_State_t	state;
    uint32_t		assocs;
    NPF_DS_DpeHandle_t  dpeHandle;      /* DPE handle */
    NPF_DS_DpeHandle_t  prevDpeHandle;  /* DPE handle */
    NPF_DS_DpeDirection_t dir;            /* Direction, INGRESS/EGRESS */
    ix_uint32 		flowId;		/* Flow id */
    ix_uint32 		classId;	/* Calss id, relative Queue id */
    FPPI_PortID 	port;		/* Port */
    NPF_DS_Dpe_t	dpe;	        /* DPE attributes */
    void		*temp;		/* Used for temporary storage */
} fpm_qos_dpe_t;

typedef struct fpm_qos_request
{
    FPPI_CBCORRELATOR	corr;
    FPPI_CONTEXT	context;
    FPPI_DS_Dpe_t	dpe;
}fpm_qos_request_t;

extern int fpm_qos_dpe_init();
extern int fpm_qos_dpe_shutdown();
extern void fpm_qos_dpe_operation(fpm_qos_request_t *req);

extern void fpm_query_dpe(fpm_qos_dpe_t *dpe, fpm_qos_request_t *req);

extern void fpm_6t_classifier_op(fpm_qos_dpe_t *classifier, 
					fpm_qos_request_t *req, 
					NPF_DS_CallbackType_t op);

extern void fpm_dscp_classifier_op(fpm_qos_dpe_t *classifier,
                                fpm_qos_request_t *req,
                                NPF_DS_CallbackType_t op);

void fpm_meter_op(fpm_qos_dpe_t *meter,
                        fpm_qos_request_t *req,
                        NPF_DS_CallbackType_t op);

void fpm_marker_op(fpm_qos_dpe_t *marker,
                        fpm_qos_request_t *req,
                        NPF_DS_CallbackType_t op);

void fpm_dropper_op(fpm_qos_dpe_t *dropper,
                        fpm_qos_request_t *req,
                        NPF_DS_CallbackType_t op);

extern void send_ix_cc_error(NPF_DS_ErrorType_t error);
extern void send_ix_cc_async_response(fpm_qos_request_t *resp);

#endif
