/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <assert.h>
#include <errno.h>

#include "FPModule.h"
#include "npf_ds.h"
#include "log.h"
#include "list.h"
#include "qos_mgr_impl.h"

#define _IX_HARDWARE_TYPE_  _IX_HW_2400_
#include "ix_cc_microengines_bindings.h"
#include "bindings.h"
#include "ix_cc_error.h"
#include "cc/ix_cc_fpm_core.h"
#include "cc/ix_cc_classifier_6t.h"
#include "cc/ix_cc_classifier_6t_stats.h"
#include "cc/ix_cc_classifier_6t_api.h"
#include "cc/ix_cc_classifier_6t_msg.h"
#include "cc/ix_cc_dscp_classifier_msghlp.h"
#include "cc/ix_cc_tc_meter_msg_helpers.h"
#include "cc/ix_cc_wred_msghlp.h"

static uint32_t		dscpEntries = 0;
#define 	 	DSCP_CLASSIFIER_SIZE 64
ix_cc_classifier_dscp_result dscp_classifier_entry[DSCP_CLASSIFIER_SIZE];

static DList		DpeList;

uint8_t genClassId(uint8_t  dscp);
static int compareDpe(const void* data, const void* key);
static ix_error fpm_classifier_operation_callback(ix_error arg_Result, void* arg_pContext);
static ix_error fpm_meter_operation_callback(ix_error arg_Result, void* arg_pContext);
static ix_error fpm_dropper_operation_callback(ix_error arg_Result, void* arg_pContext);
void fpm_dropper_stats(fpm_qos_dpe_t *dpe, fpm_qos_request_t *req);
#ifdef CLASSIFIER_6T_PACKET_COUNTER_FEATURE
ix_error fpm_classifier_6t_stats_callback(ix_error arg_Result, 
					ix_cc_classifier_6t_statistics * arg_pStatsResult, 
					void* arg_pContext);
#endif
ix_error fpm_classifier_dscp_stats_callback(ix_error arg_Result, 
					ix_cc_classifier_dscp_statistics * arg_pStatsResult, 
					void* arg_pContext);
static ix_error fpm_meter_stats_callback(ix_error arg_Result, 
					ix_cc_tc_meter_statistics * arg_pStatsResult, 
					void* arg_pContext);
static ix_error fpm_dropper_stats_callback(ix_error arg_Result, 
					ix_cc_wred_statistics * arg_pStatsResult, 
					void* arg_pContext);
static void dpeSetState(NPF_DS_DpeHandle_t  dpeHandle, NPF_DS_CallbackType_t op, QOS_DPE_State_t state);

#define    CHECK_IX_CC_ERROR(err, resp) \
    switch(err)\
    {\
	case IX_SUCCESS:\
	{\
	}\
	break;\
	case IX_CC_ERROR_OOM_SYSTEM:\
	    resp->dpe.error = NPF_DS_E_RESOURCE;\
	    send_ix_cc_async_response(resp);\
	break;\
	case IX_CC_ERROR_NULL:\
	    resp->dpe.error = NPF_DS_E_RESOURCE;\
	    send_ix_cc_async_response(resp);\
	break;\
	case IX_CC_ERROR_SEND_FAIL:\
	    resp->dpe.error = NPF_DS_E_RESOURCE;\
	    send_ix_cc_async_response(resp);\
	break;\
	default:\
	    resp->dpe.error = NPF_DS_E_UNKNOWN;\
	    send_ix_cc_async_response(resp);\
    }\
    return\

/********************************************************************************
* Function:	fpm_qos_dpe_init						*
* Description:  Initialization routine for DiffServ Manager			*
*		Initializes the Lists DpeList(used to store DPE) 	 	*
*		(used to store DPE temporarily for modification)		*
********************************************************************************/
int fpm_qos_dpe_init()
{
    DS_FUNCTION();
    npf_list_init(&DpeList, PIL_FreeMemory);
    DpeList.compare = compareDpe;
    return(FPPI_SUCCESS);
}

/********************************************************************************
* Function:	fpm_qos_dpe_shutdown						*
* Description:  Sutdown routine for DiffServ Manager				*
*		Destroys the Lists DpeList(used to store DPE) 			*
*		(used to store DPE temporarily for modification)		*
********************************************************************************/
int fpm_qos_dpe_shutdown()
{
    DS_FUNCTION();
    npf_list_destroy(&DpeList);
    return(FPPI_SUCCESS);
}

#define FPM_RETURN_ERROR(err) \
    {\
        req->dpe.error = err;\
        send_ix_cc_async_response(req);\
        return;\
    }\

/********************************************************************************
* Function:	fpm_qos_dpe_operation						*
* Description:  Entry point for all DPE operation requests.			*
*		Based on operation and DPE type, validation is made using	*
*		DPE handle and DPE specific API are invoked			*
*		For ADD_ASSOC and DEL_ASSOC DPE direction check is made		*
*		DSCP classifier and WRED dropper must have direction EGRESS	*
********************************************************************************/
void fpm_qos_dpe_operation(fpm_qos_request_t *req)
{
    fpm_qos_dpe_t	*dpe;
    fpm_qos_dpe_t	*prevDpe;

    DS_FUNCTION();
    req->dpe.error = NPF_NO_ERROR;
    switch(req->dpe.op)
    {
	case NPF_DS_DPE_CREATE:
	{
    	    dpe = (fpm_qos_dpe_t *)PIL_AllocateMemory(sizeof(fpm_qos_dpe_t), PIL_MEM_ZERO_INIT);
            if(dpe == NULL) 
	    {
		DS_ERROR("dpe == NULL");
	 	FPM_RETURN_ERROR(NPF_DS_E_RESOURCE);
	        return;
	    }
	    dpe->dpeHandle = req->dpe.dpeHandle;
	    dpe->prevDpeHandle = req->dpe.prevDpeHandle;
	    dpe->port = req->dpe.port;
	    dpe->dir = req->dpe.dir;
	    dpe->temp = NULL;
	    memcpy(&dpe->dpe, &req->dpe.u.dpe, sizeof(NPF_DS_Dpe_t));
	    DS_TRACE("NPF_DS_DPE_CREATE(Handle: %d, Type: %d)", dpe->dpeHandle, dpe->dpe.type);
	    switch(dpe->dpe.type)
	    {
		case NPF_DS_CLASSIFIER:
		{
	            /*
	             * Classifier is the first elemnt in data path.
	             * There are no previous dpe to associate, so push it to FP
	            */
	    	    DS_TRACE("NPF_DS_CLASSIFIER");
		    if(dpe->dir != NPF_DS_INGRESS)
	     	        FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
		    dpe->state = DPE_STATE_IDLE;
	    	    npf_list_pushBack(&DpeList, dpe);
    		    if(dpe->dpe.u.classifier.type == NPF_DS_IPV4_FILTER)
		        fpm_6t_classifier_op(dpe, req, NPF_DS_DPE_CREATE);
		    else if(dpe->dpe.u.classifier.type == NPF_DS_DSCP_FILTER)
		        fpm_dscp_classifier_op(dpe, req, NPF_DS_DPE_CREATE);
		    else
	     	        FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
		    return;
		}
		break;
		case NPF_DS_DROPPER:
	        {
		    if(dpe->dir != NPF_DS_EGRESS)
	     	        FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
		    dpe->state = DPE_STATE_IDLE;
	    	    npf_list_pushBack(&DpeList, dpe);
	    	    send_ix_cc_async_response(req);
		}
		case NPF_DS_METER:
		case NPF_DS_MARKER:
	        {
		    dpe->state = DPE_STATE_IDLE;
	    	    dpe->temp = NULL;
	    	    npf_list_pushBack(&DpeList, dpe);
	    	    send_ix_cc_async_response(req);
		}
	 	break;
	       default:
	       {
	     	    FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
	       }
	    }
	}
	break;
	case NPF_DS_DPE_ADD_ASSOC:
	{
	    if((dpe = (fpm_qos_dpe_t *)npf_list_searchByKey(&DpeList, &req->dpe.dpeHandle)) == NULL)
		FPM_RETURN_ERROR(NPF_DS_E_INVALID_DPE)
	    if((prevDpe = (fpm_qos_dpe_t *)npf_list_searchByKey(&DpeList, &req->dpe.prevDpeHandle)) == NULL)
		FPM_RETURN_ERROR(NPF_DS_E_INVALID_DPE_PATH)
	    dpe->prevDpeHandle = req->dpe.prevDpeHandle;
	    DS_TRACE("Dpe Handle : %d, Dpe Direction: %d, PrevDpeHandle : %d, 
			PrevDpeType: %d, PrevDpeDirection: %d", 
			dpe->dpeHandle, dpe->dir, prevDpe->dpeHandle, prevDpe->dpe.type, prevDpe->dir);
	    switch(dpe->dpe.type)
	    {
		case NPF_DS_CLASSIFIER:
		{
	    	    DS_TRACE("NPF_DS_DPE_ADD_ASSOC:	NPF_DS_CLASSIFIER");
	     	    FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
		}
		break;
		case NPF_DS_METER:
		{
		    
	    	    DS_TRACE("NPF_DS_METER");
	    	    if(dpe->dir == NPF_DS_INGRESS && prevDpe->state != DPE_STATE_IDLE)
			FPM_RETURN_ERROR(NPF_DS_E_DPE_IN_USE)
		    if(prevDpe->dpe.type == NPF_DS_CLASSIFIER && 
			prevDpe->dir == NPF_DS_INGRESS)
		    {
			dpe->flowId = prevDpe->flowId;
			dpe->classId = prevDpe->classId;
			dpe->port = prevDpe->port;
		        fpm_meter_op(dpe, req, NPF_DS_DPE_ADD_ASSOC);
		    }
		    else
	 		FPM_RETURN_ERROR(NPF_DS_E_INVALID_DPE_PATH)
		}
		break;
		case NPF_DS_MARKER:
		{
		    /*Unless METER attached to CALSSIFIER, don't allow marker attch to METER */
		    /*If METER state is IN_USE means it is attached to CLASSFIER */
	    	    DS_TRACE("NPF_DS_METER");
		    if(prevDpe->dpe.type == NPF_DS_METER && 
		        prevDpe->state == DPE_STATE_INUSE)
		    {
			dpe->flowId = prevDpe->flowId;
			dpe->classId = prevDpe->classId;
			dpe->temp = prevDpe->temp;	//Meter Parameters
		        fpm_marker_op(dpe, req, NPF_DS_DPE_ADD_ASSOC);
		    }
		    else
	 		FPM_RETURN_ERROR(NPF_DS_E_INVALID_DPE_PATH)
		}
		break;
		case NPF_DS_DROPPER:
		{
	    	    DS_TRACE("NPF_DS_METER");
		    if(dpe->dir == NPF_DS_EGRESS &&
			(prevDpe->dpe.type == NPF_DS_MARKER || 
			prevDpe->dpe.type == NPF_DS_CLASSIFIER))
		    {
			dpe->flowId = prevDpe->flowId;
			dpe->classId = prevDpe->classId;
			dpe->port = prevDpe->port;
		        fpm_dropper_op(dpe, req, NPF_DS_DPE_ADD_ASSOC);
		    }
		    else
	 		FPM_RETURN_ERROR(NPF_DS_E_INVALID_DPE_PATH)
		}
		break;
		default:
	 	    FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
	    }
	}
	break;
	case NPF_DS_DPE_DELETE_ASSOC:
	{
	    DS_TRACE("NPF_DS_DPE_DELETE_ASSOC");
	    if((dpe = (fpm_qos_dpe_t *)npf_list_searchByKey(&DpeList, &req->dpe.dpeHandle)) == NULL)
		FPM_RETURN_ERROR(NPF_DS_E_INVALID_DPE)
	    if((prevDpe = (fpm_qos_dpe_t *)npf_list_searchByKey(&DpeList, &req->dpe.prevDpeHandle)) == NULL)
		FPM_RETURN_ERROR(NPF_DS_E_INVALID_DPE_PATH)
	    DS_TRACE("Dpe Handle : %d, Dpe Direction: %d, PrevDpeHandle : %d, 
			PrevDpeType: %d, PrevDpeDirection: %d", 
			dpe->dpeHandle, dpe->dir, prevDpe->dpeHandle, prevDpe->dpe.type, prevDpe->dir);
	    switch(dpe->dpe.type)
	    {
		case NPF_DS_CLASSIFIER:
		{
	    	    DS_TRACE("NPF_DS_CLASSIFIER");
	     	    FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
		}
		break;
		case NPF_DS_METER:
		{
	    	    DS_TRACE("NPF_DS_METER");
		    if(prevDpe->dpe.type == NPF_DS_CLASSIFIER && 
			prevDpe->dir == NPF_DS_INGRESS &&
			prevDpe->dpeHandle == dpe->prevDpeHandle)
		    {
		        fpm_meter_op(dpe, req, NPF_DS_DPE_DELETE_ASSOC);
		    }
		    else
	 		FPM_RETURN_ERROR(NPF_DS_E_INVALID_DPE_PATH)
		}
		break;
		case NPF_DS_MARKER:
		{
	    	    DS_TRACE("NPF_DS_MARKER");
		    if(prevDpe->dpe.type == NPF_DS_METER && 
			prevDpe->dpeHandle == dpe->prevDpeHandle &&
		        prevDpe->state == DPE_STATE_INUSE)
		    {
		        fpm_marker_op(dpe, req, NPF_DS_DPE_DELETE_ASSOC);
		    }
		    else
	 		FPM_RETURN_ERROR(NPF_DS_E_INVALID_DPE_PATH)
		}
		break;
		case NPF_DS_DROPPER:
		{
	    	    DS_TRACE("NPF_DS_DROPPER");
		    if(dpe->dir == NPF_DS_EGRESS &&
			(prevDpe->dpe.type == NPF_DS_MARKER || 
			prevDpe->dpe.type == NPF_DS_CLASSIFIER))
		    {
		        fpm_dropper_op(dpe, req, NPF_DS_DPE_DELETE_ASSOC);
		    }
		    else
	 		FPM_RETURN_ERROR(NPF_DS_E_INVALID_DPE_PATH)
		}
		break;
		default:
	 	    FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
	    }
	}
	break;
	case NPF_DS_DPE_MODIFY:
	{
	    fpm_qos_dpe_t *tmp;
	    if((dpe = (fpm_qos_dpe_t *)npf_list_searchByKey(&DpeList, &req->dpe.dpeHandle)) == NULL)
		FPM_RETURN_ERROR(NPF_DS_E_INVALID_DPE)
	    DS_TRACE("NPF_DS_DPE_MODIFY(%d)", dpe->dpeHandle);
    	    tmp = (fpm_qos_dpe_t *)PIL_AllocateMemory(sizeof(fpm_qos_dpe_t), PIL_MEM_ZERO_INIT);
            if(tmp == NULL) 
	    {
		DS_ERROR("tmp == NULL");
	 	FPM_RETURN_ERROR(NPF_DS_E_RESOURCE);
	        return;
	    }
	    DS_TRACE("Dpe Handle:	%d", dpe->dpeHandle);
	    memcpy(tmp, dpe, sizeof(fpm_qos_dpe_t)); //Store old values
	    dpe->temp = tmp;
	    memcpy(&dpe->dpe, &req->dpe.u.dpe, sizeof(NPF_DS_Dpe_t)); //Copy new Values
	    dpe->dpe.type = tmp->dpe.type;
	    switch(dpe->dpe.type)
	    {
		case NPF_DS_CLASSIFIER:
    		    if(dpe->dpe.u.classifier.type == NPF_DS_IPV4_FILTER)
		        fpm_6t_classifier_op(dpe, req, NPF_DS_DPE_MODIFY);
		    else if(dpe->dpe.u.classifier.type == NPF_DS_DSCP_FILTER)
		        fpm_dscp_classifier_op(dpe, req, NPF_DS_DPE_MODIFY);
		    else
		        FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
		break;
		case NPF_DS_METER:
		    fpm_meter_op(dpe, req, NPF_DS_DPE_MODIFY );
		break;
		case NPF_DS_MARKER:
		    fpm_marker_op(dpe, req, NPF_DS_DPE_MODIFY);
		break;
		case NPF_DS_DROPPER:
		    fpm_dropper_op(dpe, req, NPF_DS_DPE_MODIFY);
		break;
		case NPF_DS_QUEUE:
		case NPF_DS_SCHEDULER:
		    FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
		break;
		default:
		    FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
	    }
	}
	break;
	case NPF_DS_DPE_DELETE:
	{
	    if((dpe = (fpm_qos_dpe_t *)npf_list_searchByKey(&DpeList, &req->dpe.dpeHandle)) == NULL)
		FPM_RETURN_ERROR(NPF_DS_E_INVALID_DPE)
	    DS_TRACE("NPF_DS_DPE_DELETE(%d)", dpe->dpeHandle);
	    switch(dpe->dpe.type)
	    {
		case NPF_DS_CLASSIFIER:
	    	    DS_TRACE("NPF_DS_CLASSIFIER(Hanldle: %d, State: %d)", dpe->dpeHandle, dpe->state);
		    if(dpe->state != DPE_STATE_IDLE)
			FPM_RETURN_ERROR(NPF_DS_E_DPE_IN_USE)
    		    if(dpe->dpe.u.classifier.type == NPF_DS_IPV4_FILTER)
		        fpm_6t_classifier_op(dpe, req, NPF_DS_DPE_DELETE);
		    else if(dpe->dpe.u.classifier.type == NPF_DS_DSCP_FILTER)
		        fpm_dscp_classifier_op(dpe, req, NPF_DS_DPE_DELETE);
		    else
	     	        FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
		    return;
		break;
		case NPF_DS_METER:
		{
	    	    DS_TRACE("NPF_DS_METER(Hanldle: %d, State: %d)", dpe->dpeHandle, dpe->state);
		    if(dpe->state != DPE_STATE_IDLE)
			FPM_RETURN_ERROR(NPF_DS_E_DPE_IN_USE)
		    npf_list_removeByKeyFree(&DpeList, &dpe->dpeHandle);
	    	    send_ix_cc_async_response(req);
		    return;
		}
		break;
		case NPF_DS_MARKER:
		{
	    	    DS_TRACE("NPF_DS_MARKER(Hanldle: %d, State: %d)", dpe->dpeHandle, dpe->state);
		    if(dpe->state != DPE_STATE_IDLE)
			FPM_RETURN_ERROR(NPF_DS_E_DPE_IN_USE)
		    npf_list_removeByKeyFree(&DpeList, (void *)&dpe->dpeHandle);
	    	    send_ix_cc_async_response(req);
		    return;
		}
		break;
		case NPF_DS_DROPPER:
	        {
	    	    DS_TRACE("NPF_DS_DROPPER(Hanldle: %d, State: %d)", dpe->dpeHandle, dpe->state);
		    if(dpe->state != DPE_STATE_IDLE)
			FPM_RETURN_ERROR(NPF_DS_E_DPE_IN_USE)
		    npf_list_removeByKeyFree(&DpeList, &dpe->dpeHandle);
	    	    send_ix_cc_async_response(req);
		    return;
		}
		break;
		case NPF_DS_QUEUE:
		case NPF_DS_SCHEDULER:
		    FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
		break;
		default:
		    FPM_RETURN_ERROR(NPF_DS_E_INVALID_DPE)
	    }
	}
	break;
	case NPF_DS_DPE_QUERY:
	{
	    if((dpe = (fpm_qos_dpe_t *)npf_list_searchByKey(&DpeList, &req->dpe.dpeHandle)) == NULL)
		FPM_RETURN_ERROR(NPF_DS_E_INVALID_DPE)
	    DS_TRACE("NPF_DS_DPE_QUERY(%d)", dpe->dpeHandle);
	    switch(dpe->dpe.type)
	    {
		case NPF_DS_CLASSIFIER:
		case NPF_DS_METER:
		case NPF_DS_MARKER:
		case NPF_DS_DROPPER:
		    fpm_query_dpe(dpe, req);
		break;
		case NPF_DS_QUEUE:
		case NPF_DS_SCHEDULER:
		    FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
		break;
		default:
		    FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
	    }
	}
	break;
	case NPF_DS_DPE_STATISTICS:
	{
	    if((dpe = (fpm_qos_dpe_t *)npf_list_searchByKey(&DpeList, &req->dpe.dpeHandle)) == NULL)
		FPM_RETURN_ERROR(NPF_DS_E_INVALID_DPE)
	    DS_TRACE("NPF_DS_DPE_STATISTICS(%d)", dpe->dpeHandle);
	    switch(dpe->dpe.type)
	    {
		case NPF_DS_CLASSIFIER:
    		    if(dpe->dpe.u.classifier.type == NPF_DS_IPV4_FILTER)
		        fpm_6t_classifier_op(dpe, req, NPF_DS_DPE_STATISTICS);
		    else if(dpe->dpe.u.classifier.type == NPF_DS_DSCP_FILTER)
		        fpm_dscp_classifier_op(dpe, req, NPF_DS_DPE_STATISTICS);
		    else
	     	        FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
		break;
		case NPF_DS_METER:
		    fpm_meter_op(dpe, req, NPF_DS_DPE_STATISTICS);
		break;
		case NPF_DS_DROPPER:
		    fpm_dropper_op(dpe, req, NPF_DS_DPE_STATISTICS);
		break;
		break;
		case NPF_DS_MARKER:
		case NPF_DS_QUEUE:
		case NPF_DS_SCHEDULER:
		    FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
		break;
		default:
		    FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
	    }
	}
	break;
        default:
        {
            FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
        }
    }
}

/********************************************************************************
* Function:	fpm_query_dpe							*
* Description:  Returns attributes of DPE					*
********************************************************************************/
void fpm_query_dpe(fpm_qos_dpe_t *dpe, fpm_qos_request_t *req)
{
    DS_FUNCTION();
    memcpy(&req->dpe.u.dpe, &dpe->dpe, sizeof(NPF_DS_Dpe_t));
    send_ix_cc_async_response(req);
}

/********************************************************************************
* Function:	fpm_6t_classifier_op						*
* Description:  Creates/Modify/Delete 6t-Classifier in INGRESS CC		*
********************************************************************************/
void fpm_6t_classifier_op(fpm_qos_dpe_t *classifier, 
				fpm_qos_request_t *req, 
				NPF_DS_CallbackType_t op)
{
    ix_error			error = IX_CC_ERROR_NULL;
    ix_cc_classifier_6t_pattern	pattern;
    ix_cc_classifier_6t_result    lookupResult;
    DS_FUNCTION();
    memset(&lookupResult, 0, sizeof(ix_cc_classifier_6t_result));
    #ifdef FPM_QOSMGR_DEBUG
    printf("fpm_6t_classifier_op: %d\n", (int)classifier->dpeHandle);
    #endif
    lookupResult.type = IX_CC_CLASSIFIER_6T_QOS;
    #ifdef CLASSIFIER_6T_PACKET_COUNTER_FEATURE
    lookupResult.stats_enable = 1;
    #endif /* #if defined(CLASSIFIER_6T_PACKET_COUNTER_FEATURE) */
    lookupResult.result.qos.colorId = classifier->dpe.u.classifier.precedence;
    lookupResult.result.qos.OutputId = IX_CC_CLASSIFIER_6T_METER_BID;

    switch(classifier->dpe.u.classifier.type)
    {
	case NPF_DS_IPV4_FILTER:
	{
	    NPF_DS_IPV4_Filter_t	*ipv4Filter;
	    ipv4Filter = &classifier->dpe.u.classifier.filterEntry.filter_IPV4;
	    pattern.ipv4SourceAddress          = ipv4Filter->srcAddr;
            pattern.ipv4DestinationAddress     = ipv4Filter->dstAddr;
            pattern.inputPort                  = classifier->port;
            pattern.padding                    = 0;
            pattern.dscp                       = ipv4Filter->dscp;
            pattern.protocol                   = ipv4Filter->protocol;
            pattern.l4SourcePort               = ipv4Filter->srcL4Port;
            pattern.l4DestinationPort          = ipv4Filter->dstL4Port;
	    switch(op)
	    {
		case NPF_DS_DPE_CREATE:
		{
    		    classifier->flowId = lookupResult.result.qos.flowId = classifier->dpeHandle;
    		    lookupResult.result.qos.classId = genClassId(ipv4Filter->dscp);
    		    classifier->classId = lookupResult.result.qos.classId;
    		    #ifdef FPM_QOSMGR_DEBUG
    		    printf("\n-------------------------------------------------------------\n");
    		    printf("\nSourceAddress: 0x%x", (int)pattern.ipv4SourceAddress);
    		    printf("\nDestinationAddress: 0x%x", (int)pattern.ipv4DestinationAddress);
    		    printf("\nProtocol: 0x%x", (int)pattern.protocol);
    		    printf("\nSourcePort: 0x%x", (int)pattern.l4SourcePort);
    		    printf("\nDestinationPort: 0x%x", (int)pattern.l4DestinationPort);
    		    printf("\nDscp: 0x%x", (int)pattern.dscp);
    		    printf("\n-------------------------------------------------------------\n");
    		    printf("FlowId: %d, ClassId: %d\n", (int)lookupResult.result.qos.flowId, 
							(int)lookupResult.result.qos.classId);
    		    printf("\n-------------------------------------------------------------\n");
    		    #endif
    	    	    error = ix_cc_classifier_6t_async_add_entry(&pattern, 
					&lookupResult,
					&fpm_classifier_operation_callback, 
					(void *)req);
		}
		break;
		case NPF_DS_DPE_MODIFY:
		{
    		    #ifdef FPM_QOSMGR_DEBUG
    		    printf("\n-------------------------------------------------------------\n");
    		    printf("\nSourceAddress: 0x%x", (int)pattern.ipv4SourceAddress);
    		    printf("\nDestinationAddress: 0x%x", (int)pattern.ipv4DestinationAddress);
    		    printf("\nProtocol: 0x%x", (int)pattern.protocol);
    		    printf("\nSourcePort: 0x%x", (int)pattern.l4SourcePort);
    		    printf("\nDestinationPort: 0x%x", (int)pattern.l4DestinationPort);
    		    printf("\nDscp: 0x%x", (int)pattern.dscp);
    		    printf("\n-------------------------------------------------------------\n");
    		    printf("FlowId: %d, ClassId: %d\n", (int)lookupResult.result.qos.flowId, 
							(int)lookupResult.result.qos.classId);
    		    printf("\n-------------------------------------------------------------\n");
    		    #endif
    	    	    error = ix_cc_classifier_6t_async_update_entry(&pattern, 
					&lookupResult,
					&fpm_classifier_operation_callback, 
					(void *)req);
		}
		break;
		case NPF_DS_DPE_DELETE:
		{
    	    	    error = ix_cc_classifier_6t_async_remove_entry(&pattern, 
					IX_CC_CLASSIFIER_6T_QOS,
					&fpm_classifier_operation_callback, 
					(void *)req);
		}
		break;
		#ifdef CLASSIFIER_6T_PACKET_COUNTER_FEATURE
		case NPF_DS_DPE_STATISTICS:
		{
    	    	    ix_cc_classifier_6t_statistics 	*arg_pStats;
    	    	    if((arg_pStats = PIL_AllocateMemory(sizeof(ix_cc_classifier_6t_statistics), 
		                                                PIL_MEM_ZERO_INIT)) == NULL)
        	    FPM_RETURN_ERROR(NPF_DS_E_RESOURCE);
     	    	    error = ix_cc_classifier_6t_async_get_statistics(&pattern,
					arg_pStats,
					fpm_classifier_6t_stats_callback,
					(void *)req);
	        }
	        break;
	        #endif
		default:
		{
            	    FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
		}
	    }
	}
	break;
	case NPF_DS_IPV6_FILTER:
            FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
	break;
	case NPF_DS_MPLS_FILTER:
            FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
	default:
            FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
	break;
    }
    CHECK_IX_CC_ERROR(error, req);
}

/********************************************************************************
* Function:	fpm_dscp_classifier_op						*
* Description:  Create/Modify/Delete DSCP-Classifier entry in INRESS CC		*
********************************************************************************/
void fpm_dscp_classifier_op(fpm_qos_dpe_t *classifier, 
				fpm_qos_request_t *req,
				NPF_DS_CallbackType_t op)
{
    ix_error		error = IX_CC_ERROR_NULL;
    ix_cc_classifier_dscp_result lookupResult;
    DS_FUNCTION();
    memset(&lookupResult, 0, sizeof(ix_cc_classifier_dscp_result));
    lookupResult.colorId = classifier->dpe.u.classifier.precedence;
    lookupResult.TC = 1;
    lookupResult.remark = 1;
    #ifdef CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE
    lookupResult.statistics = 1;
    #endif
    switch(op)
    {
	case NPF_DS_DPE_CREATE:
	{
	    NPF_DS_DSCP_Filter_t	*dscpFilter;
    	    classifier->flowId = lookupResult.flowId = classifier->dpeHandle;
	    dscpFilter = &classifier->dpe.u.classifier.filterEntry.filter_DSCP;
    	    lookupResult.classId = genClassId(dscpFilter->dscp);
    	    classifier->classId = lookupResult.classId;
    	    #ifdef FPM_QOSMGR_DEBUG
    	    printf("\n-------------------------------------------------------------\n");
    	    printf("\nport: %d", classifier->port);
    	    printf("\nflowId: %d", (int)lookupResult.flowId);
    	    printf("\nclassId: %d", lookupResult.classId);
    	    printf("\ncolorId: %d", lookupResult.colorId);
    	    printf("\nTC: %d", lookupResult.TC);
    	    printf("\nremark: %d", lookupResult.remark);
    	    printf("\nstatistics: %d", lookupResult.statistics);
    	    printf("\n-------------------------------------------------------------\n");
	    #endif
	    /*
	     * prevDpeHandle filed used to hold the total no. of entries
	     * This is valid only for DSCP Classifier entry
	    */
	    if(classifier->prevDpeHandle > 1)
	    {
		memcpy(&dscp_classifier_entry[dscpEntries++], 
			&lookupResult, 
			sizeof(ix_cc_classifier_dscp_result));
		if(dscpEntries < classifier->prevDpeHandle)
		{
    	    	    #ifdef FPM_QOSMGR_DEBUG
		    printf("Pending Entries: %d, Storing Locally !!!!!!1\n", classifier->prevDpeHandle - dscpEntries);
		    #endif
    	            send_ix_cc_async_response(req);
		    return;
		}
	    }
	    else
	    {
	        memcpy(&dscp_classifier_entry[0], 
			&lookupResult, 
	    		sizeof(ix_cc_classifier_dscp_result));
	    }
    	    error = ix_cc_classifier_dscp_async_add_if_config(classifier->port,
					dscp_classifier_entry,
					&fpm_classifier_operation_callback, 
					(void *)req);
	}
	break;
	case NPF_DS_DPE_MODIFY:
	{
    	    error = ix_cc_classifier_dscp_async_update_if_config(classifier->port,
					dscp_classifier_entry,
					&fpm_classifier_operation_callback, 
					(void *)req);
	}
	break;
	#ifdef CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE
	case NPF_DS_DPE_STATISTICS:
	{
	    NPF_DS_DSCP_Filter_t	*dscpFilter;
	    dscpFilter = &classifier->dpe.u.classifier.filterEntry.filter_DSCP;
    	    error = ix_cc_classifier_dscp_async_get_statistics(classifier->port,
					dscpFilter->dscp,
					&fpm_classifier_dscp_stats_callback, 
					(void *)req);
	}
	break;
	#endif
	case NPF_DS_DPE_DELETE:
	{
    	    error = ix_cc_classifier_dscp_async_remove_if_config(classifier->port,
					&fpm_classifier_operation_callback, 
					(void *)req);
	}
	break;
	default:
	{
       	    FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
	}
    }
    CHECK_IX_CC_ERROR(error, req);
}

/********************************************************************************
* Function:	fpm_meter_op							*
* Description:  Create/Modify/Delete local METER DPE with user values. 		*
*		Since METER is part of MARKER, no SDK api is called till MARKER *
*		associates with METER SDK METER object is filled with values and*
*		stored in temp filed of local METER DPE				*
********************************************************************************/
void fpm_meter_op(fpm_qos_dpe_t *meter, 
			fpm_qos_request_t *req,
			NPF_DS_CallbackType_t op)
{
    ix_error		error = IX_CC_ERROR_NULL;
    ix_cc_tc_meter_parameters	*localMeter = NULL;
    ix_cc_fpm_dpe	tc_meter;
    DS_FUNCTION();
    localMeter = (ix_cc_tc_meter_parameters *)
    PIL_AllocateMemory(sizeof(ix_cc_tc_meter_parameters), PIL_MEM_ZERO_INIT);
    tc_meter.type = IX_CC_FPM_DPE_METER;
    if(localMeter == NULL)
    {
        FPM_RETURN_ERROR(NPF_DS_E_RESOURCE)
    }
    #ifdef FPM_QOSMGR_DEBUG
    printf("fpm_meter_op : flowId = %d, classId = %d\n", (int)meter->flowId, (int)meter->classId);
    #endif
    switch(op)
    {
	case NPF_DS_DPE_ADD_ASSOC:
	case NPF_DS_DPE_MODIFY:
	{
	    localMeter->statsEnabled = meter->dpe.u.meter.statsRequired;
    	    if(meter->dpe.u.meter.type == NPF_DS_TRTCM_AWARE)
    	    {
                localMeter->meter_type_id = IX_CC_TC_METER_TRTCM_TYPE;
                localMeter->meter_type.trtcm.cir = meter->dpe.u.meter.meterEntry.cir;
                localMeter->meter_type.trtcm.pir = meter->dpe.u.meter.meterEntry.pir;
                localMeter->meter_type.trtcm.cbs = meter->dpe.u.meter.meterEntry.cbs;
                localMeter->meter_type.trtcm.pbs = meter->dpe.u.meter.meterEntry.pbs;
    	    }
    	    else if (meter->dpe.u.meter.type == NPF_DS_SRTCM_AWARE)
    	    {
                localMeter->meter_type_id = IX_CC_TC_METER_SRTCM_TYPE;
                localMeter->meter_type.srtcm.cir = meter->dpe.u.meter.meterEntry.cir;
                localMeter->meter_type.srtcm.cbs = meter->dpe.u.meter.meterEntry.cbs;
                localMeter->meter_type.srtcm.ebs = meter->dpe.u.meter.meterEntry.ebs;
    	    }
    	    else
    	    {
                FPM_RETURN_ERROR(NPF_DS_E_DPE_NOT_SUPPORTED);
    	    }
	    
    	    switch(meter->dpe.u.meter.meterEntry.nextGreenDpe)
    	    {
	        case NPF_DS_MARKER:
    	            localMeter->greenOutput = IX_CC_TC_METER_NEXT1;
	        break;
	        case NPF_DS_DROPPER:
    	            localMeter->greenOutput = IX_CC_TC_METER_NEXT2;
	        break;
	        default:
       	            FPM_RETURN_ERROR(NPF_DS_E_INVALID_DPE_PATH)
    	    }
    	    switch(meter->dpe.u.meter.meterEntry.nextYellowDpe)
    	    {
	        case NPF_DS_MARKER:
	             localMeter->yellowOutput = IX_CC_TC_METER_NEXT1;
	        break;
	        case NPF_DS_DROPPER:
	            localMeter->yellowOutput = IX_CC_TC_METER_NEXT2;
	        break;
	        default:
                    FPM_RETURN_ERROR(NPF_DS_E_INVALID_DPE_PATH)
    	    }
    	    switch(meter->dpe.u.meter.meterEntry.nextRedDpe)
    	    {
	        case NPF_DS_MARKER:
	            localMeter->redOutput = IX_CC_TC_METER_NEXT1;
	        break;
	        case NPF_DS_DROPPER:
	            localMeter->redOutput = IX_CC_TC_METER_NEXT2;
	        break;
	        default:
                    FPM_RETURN_ERROR(NPF_DS_E_INVALID_DPE_PATH)
    	    }
	
    	    /*
		Save meter parameters, till marker is added
    	    */
    	    meter->temp = localMeter;
	    if(op == NPF_DS_DPE_ADD_ASSOC)
	    {
    	        dpeSetState(meter->dpeHandle, NPF_DS_DPE_ADD_ASSOC, DPE_STATE_INUSE);
    	        send_ix_cc_async_response(req);
    	        return;
	    }
	    if(op == NPF_DS_DPE_MODIFY)
	    {
    	        #ifdef FPM_QOSMGR_DEBUG
    	        printf("FlowId: %d\n", (int)meter->flowId);
    	        printf("ClassId: %d\n", (int)meter->classId);
    	        #endif
		if(meter->dir == NPF_DS_INGRESS)
		{
		    /*
		     * On egress for wred and meter instance is used queue_id 
		     *	(i.e. class_id + port_id << 4)
		    */
    	            error = ix_cc_tc_meter_async_update_entry(meter->flowId,
					localMeter,
					fpm_meter_operation_callback,
					(void *)req);
		}
	 	else
		{
	    	    if(op == NPF_DS_DPE_ADD_ASSOC)
    	        	tc_meter.op = IX_CC_FPM_DPE_ADD;
	    	    else
    	        	tc_meter.op = IX_CC_FPM_DPE_UPDATE;
		    /*
		     * On egress for wred and meter instance is used queue_id 
		     *	(i.e. class_id + port_id << 4)
		    */
		    tc_meter.fid = meter->classId;
		    memcpy(&tc_meter.dpe.tc_meter, localMeter, sizeof(ix_cc_tc_meter_parameters));
    		    #ifdef IXASDKSTUBS
    	            error = ix_cc_tc_meter_async_update_entry(meter->flowId,
					localMeter,
					fpm_meter_operation_callback,
					(void *)req);
		    #else
    	            error = ix_cc_fpm_async_dpe_op(&tc_meter, fpm_meter_operation_callback, (void *)req);
		    #endif
		}
	    }
	}
	break;
	case NPF_DS_DPE_DELETE_ASSOC:
	{
	    if(meter->dir == NPF_DS_INGRESS)
	    {
    	        error = ix_cc_tc_meter_async_remove_entry(meter->flowId, 
							fpm_meter_operation_callback, 
							(void *)req);
	    }
	    else
	    {
    	       	tc_meter.op = IX_CC_FPM_DPE_REMOVE;
		/*
		 * On egress for wred and meter instance is used queue_id 
		 * (i.e. class_id + port_id << 4)
		 */
	        tc_meter.fid = meter->classId;
    		#ifdef IXASDKSTUBS
    	        error = ix_cc_tc_meter_async_remove_entry(meter->flowId, 
							fpm_meter_operation_callback, 
							(void *)req);
		#else
    	        error = ix_cc_fpm_async_dpe_op(&tc_meter, fpm_meter_operation_callback, (void *)req);
		#endif
	    }
	}
	break;
	case NPF_DS_DPE_STATISTICS:
	{
    	    ix_cc_tc_meter_statistics 	*arg_pStats;
    	    if((arg_pStats = PIL_AllocateMemory(sizeof(ix_cc_tc_meter_statistics), 
			PIL_MEM_ZERO_INIT)) == NULL)
        	FPM_RETURN_ERROR(NPF_DS_E_RESOURCE);
    	    #ifdef FPM_QOSMGR_DEBUG
    	    printf("FlowId: %d\n", (int)meter->flowId);
    	    #endif
	    if(meter->dir == NPF_DS_INGRESS)
	    {
     	        error = ix_cc_tc_meter_async_get_statistics(meter->flowId,
					arg_pStats,
					fpm_meter_stats_callback,
					(void *)req);
	    }
	    else
	    {
    	       	tc_meter.op = IX_CC_FPM_DPE_STATS;
		/*
		 * On egress for wred and meter instance is used queue_id 
		 * (i.e. class_id + port_id << 4)
		 */
	        tc_meter.fid = meter->classId;
    		#ifdef IXASDKSTUBS
     	        error = ix_cc_tc_meter_async_get_statistics(meter->flowId,
					arg_pStats,
					fpm_meter_stats_callback,
					(void *)req);
		#else
    	        error = ix_cc_fpm_async_dpe_op(&tc_meter, fpm_meter_stats_callback, (void *)req);
		#endif
	    }
	}
	break;
	default:
	{
       	    FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
	}
    }
    CHECK_IX_CC_ERROR(error, req);
}

/********************************************************************************
* Function:	fpm_marker_op							*
* Description:  								*
********************************************************************************/
void fpm_marker_op(fpm_qos_dpe_t *marker, 
			fpm_qos_request_t *req,
			NPF_DS_CallbackType_t op)
{
    ix_error		error = IX_CC_ERROR_NULL;
    ix_cc_tc_meter_parameters	*localMeter;
    fpm_qos_dpe_t		*meter;
    ix_cc_fpm_dpe	tc_meter;
    DS_FUNCTION();
    if((meter = (fpm_qos_dpe_t *)npf_list_searchByKey(&DpeList, &marker->prevDpeHandle)) == NULL)
	FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
    #ifdef FPM_QOSMGR_DEBUG
    printf("fpm_marker_op : flowId = %d, classId = %d\n", (int)marker->flowId, (int)marker->classId);
    #endif
    tc_meter.type = IX_CC_FPM_DPE_METER;
    switch(op)
    {
	case NPF_DS_DPE_ADD_ASSOC:
	case NPF_DS_DPE_MODIFY:
	{
    	    localMeter = (ix_cc_tc_meter_parameters *)marker->temp;
	    if(localMeter == NULL)
	    {
	 	FPM_RETURN_ERROR(NPF_DS_E_INVALID_DPE_PATH)
	    }
    	    localMeter->greenDscp = marker->dpe.u.marker.markerEntry.dscp.greenDscp;
    	    localMeter->yellowDscp = marker->dpe.u.marker.markerEntry.dscp.yellowDscp;
    	    localMeter->redDscp = marker->dpe.u.marker.markerEntry.dscp.redDscp;
	    if(op == NPF_DS_DPE_ADD_ASSOC)
	    {
    	        #ifdef FPM_QOSMGR_DEBUG
    	        printf("FlowId: %d\n", (int)marker->flowId);
    	        #endif
		if(marker->dir == NPF_DS_INGRESS && meter->dir == NPF_DS_INGRESS)
		{
    	            error = ix_cc_tc_meter_async_add_entry(marker->flowId,
					localMeter,
					fpm_meter_operation_callback,
					(void *)req);
		}
		else if(marker->dir == NPF_DS_EGRESS && meter->dir == NPF_DS_EGRESS)
		{
    	    	    #ifdef FPM_QOSMGR_DEBUG
    	    	    printf("TC_METER: EGRESS:FlowId: %d\n", (int)marker->flowId);
    	    	    #endif
    	            tc_meter.op = IX_CC_FPM_DPE_ADD;
		    /*
		     * On egress for wred and meter instance is used queue_id 
		     *	(i.e. class_id + port_id << 4)
		    */
		    tc_meter.fid = marker->classId;
		    memcpy(&tc_meter.dpe.tc_meter, localMeter, sizeof(ix_cc_tc_meter_parameters));
    		    #ifdef IXASDKSTUBS
    	            error = ix_cc_tc_meter_async_add_entry(marker->flowId,
					localMeter,
					fpm_meter_operation_callback,
					(void *)req);
		    #else
    	            error = ix_cc_fpm_async_dpe_op(&tc_meter, fpm_meter_operation_callback, (void *)req);
		    #endif
		}
		else
		{
	 	    FPM_RETURN_ERROR(NPF_DS_E_INVALID_DPE_PATH)
		}
		PIL_FreeMemory(localMeter);
		localMeter = NULL;
	    }
	    if(op == NPF_DS_DPE_MODIFY)
	    {
    	        #ifdef FPM_QOSMGR_DEBUG
    	        printf("FlowId: %d\n", (int)marker->flowId);
    	        #endif
		if(marker->dir == NPF_DS_INGRESS)
		{
    	            error = ix_cc_tc_meter_async_update_entry(marker->flowId,
					localMeter,
					fpm_meter_operation_callback,
					(void *)req);
		}
	 	else
		{
    	            tc_meter.op = IX_CC_FPM_DPE_UPDATE;
		    /*
		     * On egress for wred and meter instance is used queue_id 
		     *	(i.e. class_id + port_id << 4)
		    */
		    tc_meter.fid = marker->classId;
		    memcpy(&tc_meter.dpe.tc_meter, localMeter, sizeof(ix_cc_tc_meter_parameters));
    		    #ifdef IXASDKSTUBS
    	            error = ix_cc_tc_meter_async_update_entry(marker->flowId,
					localMeter,
					fpm_meter_operation_callback,
					(void *)req);
		    #else
    	            error = ix_cc_fpm_async_dpe_op(&tc_meter, fpm_meter_operation_callback, (void *)req);
		    #endif
		}
	    }
	}
	break;
	case NPF_DS_DPE_DELETE_ASSOC:
	{
	    /* Description:  Since Marker is embeded in Meter, accepts DELETE request
	       and sets the local DPE state to DPE_STATE_IDLE
	       No SDK api call is made
	    */
            dpeSetState(marker->dpeHandle, NPF_DS_DPE_DELETE_ASSOC, DPE_STATE_IDLE);
            send_ix_cc_async_response(req);
	    return;
	}
	break;
	default:
	{
       	    FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
	}
    }
    CHECK_IX_CC_ERROR(error, req);
}

/********************************************************************************
* Function:	fpm_dropper_op							*
* Description:  Create/Modify/Delete/Get Statistics  WRED dropper entry		*
*		Since WRED dropper micro blocks are maintained at EGRESS side	*
*		this API makes use of IX_CC_FPM_CORE and IX_CC_FPM_PROXY modules* 
********************************************************************************/
void fpm_dropper_op(fpm_qos_dpe_t *dropper,
                        fpm_qos_request_t *req,
                        NPF_DS_CallbackType_t op)
{
    ix_error		error = IX_CC_ERROR_NULL;
    #ifdef IXASDKSTUBS
    ix_cc_wred_parameters wred_parameters;
    #endif
    ix_cc_fpm_dpe	wred;
    DS_FUNCTION();
    #ifdef FPM_QOSMGR_DEBUG
    printf("fpm_dropper_op : flowId = %d, classId = %d\n", (int)dropper->flowId, (int)dropper->classId);
    #endif
    switch(op)
    {
	case NPF_DS_DPE_ADD_ASSOC:
	case NPF_DS_DPE_MODIFY:
	{
	    if(op == NPF_DS_DPE_ADD_ASSOC)
    	        wred.op = IX_CC_FPM_DPE_ADD;
	    else
    	        wred.op = IX_CC_FPM_DPE_UPDATE;

    	    wred.type = IX_CC_FPM_DPE_WRED_DROPPER;
	    /*
	     * On egress for wred and meter instance is used queue_id 
	     *	(i.e. class_id + port_id << 4)
	    */
    	    wred.fid = dropper->classId;
    	    wred.dpe.wred_dropper.service_rate = dropper->dpe.u.dropper.serviceRate;
    	    wred.dpe.wred_dropper.weight =  dropper->dpe.u.dropper.weight;
    	    wred.dpe.wred_dropper.green.min_th =  dropper->dpe.u.dropper.dropperEntry.green.minThreshold;
    	    wred.dpe.wred_dropper.green.max_th =  dropper->dpe.u.dropper.dropperEntry.green.maxThreshold;
    	    wred.dpe.wred_dropper.green.max_prob =  dropper->dpe.u.dropper.dropperEntry.green.maxProbability;
    	    wred.dpe.wred_dropper.yellow.min_th =  dropper->dpe.u.dropper.dropperEntry.yellow.minThreshold;
    	    wred.dpe.wred_dropper.yellow.max_th =  dropper->dpe.u.dropper.dropperEntry.yellow.maxThreshold;
    	    wred.dpe.wred_dropper.yellow.max_prob =  dropper->dpe.u.dropper.dropperEntry.yellow.maxProbability;
    	    wred.dpe.wred_dropper.red.min_th =  dropper->dpe.u.dropper.dropperEntry.red.minThreshold;
    	    wred.dpe.wred_dropper.red.max_th =  dropper->dpe.u.dropper.dropperEntry.red.maxThreshold;
    	    wred.dpe.wred_dropper.red.max_prob =  dropper->dpe.u.dropper.dropperEntry.red.maxProbability;
	    wred.dpe.wred_dropper.statsEnabled = dropper->dpe.u.dropper.statsRequired;

	    if(op == NPF_DS_DPE_ADD_ASSOC)
	    {
    	        #ifdef IXASDKSTUBS
    	        error = ix_cc_wred_async_add_entry(dropper->flowId,
					&wred_parameters,
					&fpm_dropper_operation_callback,
					(void *)req);
    	        #else
    	        error = ix_cc_fpm_async_dpe_op(&wred, fpm_dropper_operation_callback, (void *)req);
    	        #endif
	    }
	    if(op == NPF_DS_DPE_MODIFY)
	    {
    		#ifdef IXASDKSTUBS
    		error = ix_cc_wred_async_update_entry(dropper->flowId,
					&wred_parameters,
					&fpm_dropper_operation_callback,
					(void *)req);
    		#else
    		error = ix_cc_fpm_async_dpe_op(&wred, fpm_dropper_operation_callback, (void *)req);
    		#endif
	    }
	}
	break;
	case NPF_DS_DPE_DELETE_ASSOC:
	{
    	    wred.op = IX_CC_FPM_DPE_REMOVE;
    	    wred.type = IX_CC_FPM_DPE_WRED_DROPPER;
	    /*
	     * On egress for wred and meter instance is used queue_id 
	     *	(i.e. class_id + port_id << 4)
	    */
    	    wred.fid = dropper->classId;
    	    #ifdef IXASDKSTUBS
    	    error = ix_cc_wred_async_remove_entry(dropper->flowId,
					&fpm_dropper_operation_callback,
					(void *)req);
    	    #else
    	    error = ix_cc_fpm_async_dpe_op(&wred, fpm_dropper_operation_callback, (void *)req);
    	    #endif
	}
	break;
	#ifdef WRED_PACKET_COUNTER_FEATURE
	case NPF_DS_DPE_STATISTICS:
	{
	    ix_cc_wred_statistics	*arg_pStats = NULL;
    	    wred.op = IX_CC_FPM_DPE_STATS;
    	    wred.type = IX_CC_FPM_DPE_WRED_DROPPER;
	    /*
	     * On egress for wred and meter instance is used queue_id 
	     *	(i.e. class_id + port_id << 4)
	    */
    	    wred.fid = dropper->classId;
    	    if((arg_pStats = PIL_AllocateMemory(sizeof(ix_cc_wred_statistics), PIL_MEM_ZERO_INIT)) == NULL)
        	    FPM_RETURN_ERROR(NPF_DS_E_RESOURCE);
    	    #ifdef IXASDKSTUBS
    	    error = ix_cc_wred_async_get_statistics(dropper->flowId,
					arg_pStats,
					fpm_dropper_stats_callback,
					(void *)req);
    	    #else
    	    error = ix_cc_fpm_async_dpe_op(&wred, fpm_dropper_stats_callback, (void *)req);
    	    #endif
	}
	break;
	#endif
	default:
	{
       	    FPM_RETURN_ERROR(NPF_DS_E_NOT_SUPPORTED)
	}
    }
    CHECK_IX_CC_ERROR(error, req);
}

/********************************************************************************
* Function:	dpeSetState							*
* Description:  Helper function to set the state of DPE				*
*		When operation is ADD_ASSOC and state is INUSE, assocs count of *
*		DPE and previous DPE is incremented and state is set to IN_USE. *
*		When operation is DELETE_ASSOC and state is IDLE, assoc count of*
*		DPE and previous DPE is decremented. When assoc count is zero	*
*		state is set to IDLE						*
********************************************************************************/
static void dpeSetState(NPF_DS_DpeHandle_t  dpeHandle, NPF_DS_CallbackType_t op, QOS_DPE_State_t state)
{
    fpm_qos_dpe_t *dpe;
    fpm_qos_dpe_t *prev;
    DS_FUNCTION();
    if((dpe = (fpm_qos_dpe_t *)npf_list_searchByKey(&DpeList, (void *)&dpeHandle)) != NULL)
    {
	if(op == NPF_DS_DPE_ADD_ASSOC && state == DPE_STATE_INUSE)
	{
	    dpe->assocs = dpe->assocs + 1;
	    dpe->state = state;
	}
	else if(op == NPF_DS_DPE_DELETE_ASSOC && state == DPE_STATE_IDLE)
	{
	    dpe->assocs = dpe->assocs - 1;
	    if(dpe->assocs == 0)
	        dpe->state = state;
	}
        if((prev = (fpm_qos_dpe_t *)npf_list_searchByKey(&DpeList, (void *)&dpe->prevDpeHandle)) != NULL)
	{
	    if(op == NPF_DS_DPE_ADD_ASSOC && state == DPE_STATE_INUSE)
	    {
	        prev->assocs = prev->assocs + 1;
	        prev->state = state;
	    }
	    else if(op == NPF_DS_DPE_DELETE_ASSOC && state == DPE_STATE_IDLE)
	    {
	        prev->assocs = prev->assocs - 1;
	        if(prev->assocs == 0)
	            prev->state = state;
	    }
	}
    }
}

/********************************************************************************
* Function:	compareDpe							*
* Description:  compare function for DpeList					*
********************************************************************************/
static int compareDpe(const void* data, const void* key)
{
    if(((fpm_qos_dpe_t *)data)->dpeHandle == *(NPF_DS_DpeHandle_t *)key)
	return(1);
    else
	return(0);
}

/********************************************************************************
* Function:	fpm_classifier_operation_callback				*
* Description:  Classifier operation callback					*
*		If operation os MODIFY and result is not SUCCESS, original	*
*		parameters are restored in local DPE. If operation is DELETE and*
*		result is SUCCESS, local DPE is removed from DpeList		*
*		In all cases, response is sent to user				*
********************************************************************************/
static ix_error fpm_classifier_operation_callback(ix_error arg_Result, void* arg_pContext)
{
    fpm_qos_dpe_t 	   *tmp;
    fpm_qos_dpe_t 	   *dpe = NULL;
    fpm_qos_request_t	   *resp;
    ix_error		   error;
    NPF_DS_DpeHandle_t	   handle;

    #ifdef IXASDKSTUBS
    error = arg_Result;
    #else
    error = IX_ERROR_GET_CODE(arg_Result);
    #endif
 
    resp = (fpm_qos_request_t *)arg_pContext;
    DS_FUNCTION();
    #ifdef FPM_QOSMGR_DEBUG
    printf("\n============ fpm_classifier_operation_callback(%d) ==============\n", (int)error);
    #endif
    handle = resp->dpe.dpeHandle;
    dpe = (fpm_qos_dpe_t *)npf_list_searchByKey(&DpeList, &handle);
    if(dpe == NULL)
    {
    	#ifdef FPM_QOSMGR_DEBUG
	printf("******** ERROR: No DPE information Found ?????????********\n");
	#endif
	resp->dpe.error = NPF_DS_E_UNKNOWN;
        send_ix_cc_async_response(resp);
	return(IX_SUCCESS);
    }
    if(resp->dpe.op == NPF_DS_DPE_MODIFY)
    {
	 if(error == IX_SUCCESS)
	 {
	    PIL_FreeMemory(dpe->temp); /* Old Values */
	 }
	 else
	 {
    	    #ifdef FPM_QOSMGR_DEBUG
	    printf("NPF_DS_DPE_MODIFY(%d): Failed ???, Reverting old values ....\n", (int)handle);
	    #endif
	    tmp = (fpm_qos_dpe_t *)dpe->temp;
	    memcpy(&dpe->dpe, &tmp->dpe, sizeof(FPPI_DS_Dpe_t));
	 }
    }
    switch(error)
    {
	case IX_SUCCESS:
 	{
    	    #ifdef FPM_QOSMGR_DEBUG
	    printf("\n>>>>>>>>>> IX_SUCCESS <<<<<<<<<<<<<\n");
    	    #endif
    	    if(resp->dpe.op == NPF_DS_DPE_DELETE)
    	    {
		npf_list_removeByKeyFree(&DpeList, &resp->dpe.dpeHandle);
	    }
	}
	break;
	default:
	{
    	    #ifdef FPM_QOSMGR_DEBUG
	    printf("\n>>>>>>>>>> NPF_DS_E_UNKNOWN <<<<<<<<<<<<<\n");
    	    #endif
	    resp->dpe.error = NPF_DS_E_UNKNOWN;
	}
    }
    send_ix_cc_async_response(resp);
    return(IX_SUCCESS);
}
#ifdef CLASSIFIER_6T_PACKET_COUNTER_FEATURE
/********************************************************************************
* Function:	fpm_classifier_6t_stats_default_callback			*
* Description:  6t Classifier default Statistics callback			*
*		SDK statistics are mapped to NPF_DS_StatsResp_t parameters and  *
*		response is send to user					*
********************************************************************************/
static ix_error fpm_classifier_6t_stats_default_callback(ix_error arg_Result, 
					ix_cc_classifier_6t_statistics * arg_pStatsResult, 
					void* arg_pContext)
{
    fpm_qos_request_t	   *resp;
    ix_error		   error;
    DS_FUNCTION();
    #ifdef IXASDKSTUBS
    error = arg_Result;
    #else
    error = IX_ERROR_GET_CODE(arg_Result);
    #endif
    #ifdef FPM_QOSMGR_DEBUG
    printf("\n============ fpm_classifier_6t_stats_default_callback(%d) ==============\n", (int)error);
    #endif
    resp = (fpm_qos_request_t *)arg_pContext;
    switch(error)
    {
	case IX_SUCCESS:
 	{
    	    #ifdef FPM_QOSMGR_DEBUG
	    printf("\n>>>>>>>>>> IX_SUCCESS <<<<<<<<<<<<<\n");
	    printf("defaultPackets : %d\n", (int)arg_pStatsResult->packets);
	    printf("defaultBytes : %d\n", (int)arg_pStatsResult->bytes);
	    printf("totalPackets : %d\n", (int)resp->dpe.u.stats.u.classfierStats.totalPackets);
	    printf("totalBytes : %d\n", (int)resp->dpe.u.stats.u.classfierStats.totalBytes);
    	    #endif
	    resp->dpe.u.stats.u.classfierStats.defaultPackects = arg_pStatsResult->packets;
	    resp->dpe.u.stats.u.classfierStats.defaultBytes	= arg_pStatsResult->bytes;
	}
	break;
	default:
	{
    	    #ifdef FPM_QOSMGR_DEBUG
	    printf("\n>>>>>>>>>> NPF_DS_E_UNKNOWN <<<<<<<<<<<<<\n");
    	    #endif
	   /* resp->dpe.error = NPF_DS_E_UNKNOWN; */
	}
    }
    send_ix_cc_async_response(resp);
    return(IX_SUCCESS);
}


/********************************************************************************
* Function:	fpm_classifier_6t_stats_callback				*
* Description:  6t Classifier Statistics callback				*
*		SDK statistics are mapped to NPF_DS_StatsResp_t parameters and  *
*		response is send to user					*
********************************************************************************/
ix_error fpm_classifier_6t_stats_callback(ix_error arg_Result, 
					ix_cc_classifier_6t_statistics * arg_pStatsResult, 
					void* arg_pContext)
{
    fpm_qos_request_t	   *resp;
    ix_error		   error;
    DS_FUNCTION();
    #ifdef IXASDKSTUBS
    error = arg_Result;
    #else
    error = IX_ERROR_GET_CODE(arg_Result);
    #endif
    #ifdef FPM_QOSMGR_DEBUG
    printf("\n============ ix_cc_classifier_6t_statistics(%d) ==============\n", (int)error);
    #endif
    resp = (fpm_qos_request_t *)arg_pContext;
    switch(error)
    {
	case IX_SUCCESS:
 	{
    	    #ifdef FPM_QOSMGR_DEBUG
	    printf("\n>>>>>>>>>> IX_SUCCESS <<<<<<<<<<<<<\n");
	    printf("totalPackets : %d\n", (int)arg_pStatsResult->packets);
	    printf("totalBytes : %d\n", (int)arg_pStatsResult->bytes);
    	    #endif
	    resp->dpe.u.stats.u.classfierStats.defaultPackects = 0;
	    resp->dpe.u.stats.u.classfierStats.defaultBytes	= 0;
	    resp->dpe.u.stats.u.classfierStats.totalPackets = arg_pStatsResult->packets;
	    resp->dpe.u.stats.u.classfierStats.totalBytes = arg_pStatsResult->bytes;
	}
	break;
	default:
	{
    	    #ifdef FPM_QOSMGR_DEBUG
	    printf("\n>>>>>>>>>> NPF_DS_E_UNKNOWN <<<<<<<<<<<<<\n");
    	    #endif
	}
    }
    #ifdef FPM_QOSMGR_DEBUG
    printf("\n.......... Getting Default Statistics .............\n");
    #endif
    error = ix_cc_classifier_6t_async_get_statistics_def(arg_pStatsResult, 
							fpm_classifier_6t_stats_default_callback,
							arg_pContext);
    if(error != IX_SUCCESS)
    {
        resp->dpe.error = NPF_DS_E_UNKNOWN;
        send_ix_cc_async_response(resp);
    }
    return(IX_SUCCESS);
}
#endif

#ifdef CLASSIFIER_DSCP_PACKET_COUNTER_FEATURE
/********************************************************************************
* Function:	fpm_classifier_dscp_stats_callback				*
* Description:  DSCP Classifier Statistics callback				*
*		SDK statistics are mapped to NPF_DS_StatsResp_t parameters and  *
*		response is send to user					*
********************************************************************************/
ix_error fpm_classifier_dscp_stats_callback(ix_error arg_Result, 
					ix_cc_classifier_dscp_statistics * arg_pStatsResult, 
					void* arg_pContext)
{
    fpm_qos_request_t	   *resp;
    ix_error		   error;
    DS_FUNCTION();
    #ifdef IXASDKSTUBS
    error = arg_Result;
    #else
    error = IX_ERROR_GET_CODE(arg_Result);
    #endif
    #ifdef FPM_QOSMGR_DEBUG
    printf("\n============ fpm_classifier_dscp_stats_callback(%d) ==============\n", (int)error);
    #endif
    resp = (fpm_qos_request_t *)arg_pContext;
    switch(error)
    {
	case IX_SUCCESS:
 	{
    	    #ifdef FPM_QOSMGR_DEBUG
	    printf("\n>>>>>>>>>> IX_SUCCESS <<<<<<<<<<<<<\n");
	    printf("totalPackets : %d\n", (int)arg_pStatsResult->packets);
	    printf("totalBytes : %d\n", (int)arg_pStatsResult->bytes);
    	    #endif
	    resp->dpe.u.stats.u.classfierStats.defaultPackects = 0;
	    resp->dpe.u.stats.u.classfierStats.defaultBytes	= 0;
	    resp->dpe.u.stats.u.classfierStats.totalPackets = arg_pStatsResult->packets;
	    resp->dpe.u.stats.u.classfierStats.totalBytes = arg_pStatsResult->bytes;
	}
	break;
	default:
	{
    	    #ifdef FPM_QOSMGR_DEBUG
	    printf("\n>>>>>>>>>> NPF_DS_E_UNKNOWN <<<<<<<<<<<<<\n");
    	    #endif
            resp->dpe.error = NPF_DS_E_UNKNOWN;
	}
    }
    send_ix_cc_async_response(resp);
    return(IX_SUCCESS);
}
#endif

/********************************************************************************
* Function:	fpm_meter_operation_callback					*
* Description:  Meter operation callback					*
*		If operation os MODIFY and result is not SUCCESS, original	*
*		parameters are restored in local DPE. If operation is DELETE and*
*		result is SUCCESS, local DPE is removed from DpeList		*
*		If operation is ADD_ASSOC and result is SUCCESS, local DPE state*
*		is set to IDLE. If operation is DELETE_ASSOC and result is	*
*		SUCCESS, local DPE state is set to IDLE				*
*		In all cases, response is sent to user				*
********************************************************************************/
static ix_error fpm_meter_operation_callback(ix_error arg_Result, void* arg_pContext)
{
    fpm_qos_dpe_t 	   *tmp;
    fpm_qos_dpe_t 	   *dpe = NULL;
    fpm_qos_request_t	   *resp;
    NPF_DS_DpeHandle_t	   handle;
    ix_error		   error;
    DS_FUNCTION();
    #ifdef IXASDKSTUBS
    error = arg_Result;
    #else
    error = IX_ERROR_GET_CODE(arg_Result);
    #endif
    #ifdef FPM_QOSMGR_DEBUG
    printf("\n============ fpm_meter_operation_callback(%d) ==============\n", (int)arg_Result);
    #endif
    resp = (fpm_qos_request_t *)arg_pContext;
    handle = resp->dpe.dpeHandle;
    dpe = (fpm_qos_dpe_t *)npf_list_searchByKey(&DpeList, &handle);
    if(dpe == NULL)
    {
    	#ifdef FPM_QOSMGR_DEBUG
	printf("******** ERROR: No DPE information Found ?????????********\n");
	#endif
	resp->dpe.error = NPF_DS_E_UNKNOWN;
        send_ix_cc_async_response(resp);
	return(IX_SUCCESS);
    }

    if(resp->dpe.op == NPF_DS_DPE_MODIFY)
    {
	 if(error == IX_SUCCESS)
	 {
	    PIL_FreeMemory(dpe->temp); /* Old Values */
	 }
	 else
	 {
    	    #ifdef FPM_QOSMGR_DEBUG
	    printf("NPF_DS_DPE_MODIFY(%d): Failed ???, Reverting old values ....\n", (int)handle);
	    #endif
	    tmp = (fpm_qos_dpe_t *)dpe->temp;
	    memcpy(&dpe->dpe, &tmp->dpe, sizeof(FPPI_DS_Dpe_t));
	 }
    }
    switch(error)
    {
	case IX_SUCCESS:
 	{
    	    #ifdef FPM_QOSMGR_DEBUG
	    printf("\n>>>>>>>>>> IX_SUCCESS <<<<<<<<<<<<<\n");
    	    #endif
	    if(resp->dpe.op == NPF_DS_DPE_DELETE_ASSOC)
	    {
	        dpeSetState(resp->dpe.dpeHandle, resp->dpe.op, DPE_STATE_IDLE);
	    }
	    else if(resp->dpe.op == NPF_DS_DPE_ADD_ASSOC)
	    {
	        dpeSetState(resp->dpe.dpeHandle, resp->dpe.op, DPE_STATE_INUSE);
	    }
	    else
	        dpeSetState(resp->dpe.dpeHandle, resp->dpe.op, DPE_STATE_UNKNOWN);
	}
	break;
	default:
	{
    	    #ifdef FPM_QOSMGR_DEBUG
	    printf("\n>>>>>>>>>> NPF_DS_E_UNKNOWN <<<<<<<<<<<<<\n");
    	    #endif
	    resp->dpe.error = NPF_DS_E_UNKNOWN;
	}
    }
    send_ix_cc_async_response(resp);
    return(IX_SUCCESS);
}

/********************************************************************************
* Function:	fpm_meter_stats_callback					*
* Description:  Meter Statistics callback					*
*		SDK statistics are mapped to NPF_DS_StatsResp_t parameters and  *
*		response is send to user					*
********************************************************************************/
static ix_error fpm_meter_stats_callback(ix_error arg_Result, 
					ix_cc_tc_meter_statistics * arg_pStatsResult, 
					void* arg_pContext)
{
    fpm_qos_request_t	   *resp;
    ix_error		   error;
    DS_FUNCTION();
    #ifdef FPM_QOSMGR_DEBUG
    printf("\n============ fpm_meter_stats_callback(%d) ==============\n", (int)arg_Result);
    #endif
    #ifdef IXASDKSTUBS
    error = arg_Result;
    #else
    error = IX_ERROR_GET_CODE(arg_Result);
    #endif
    resp = (fpm_qos_request_t *)arg_pContext;
    switch(error)
    {
	case IX_SUCCESS:
 	{
    	    #ifdef FPM_QOSMGR_DEBUG
	    printf("\n>>>>>>>>>> IX_SUCCESS <<<<<<<<<<<<<\n");
    	    #endif
	    resp->dpe.u.stats.u.meterStats.type = NPF_DS_SRTCM_AWARE;
	    resp->dpe.u.stats.u.meterStats.u.colorAwareStats.greenByteCount = arg_pStatsResult->greenByteCount;
	    resp->dpe.u.stats.u.meterStats.u.colorAwareStats.greenPacketCount = arg_pStatsResult->greenPacketCount;
	    resp->dpe.u.stats.u.meterStats.u.colorAwareStats.yellowByteCount = arg_pStatsResult->yellowByteCount;
	    resp->dpe.u.stats.u.meterStats.u.colorAwareStats.yellowPacketCount = arg_pStatsResult->yellowPacketCount;
	    resp->dpe.u.stats.u.meterStats.u.colorAwareStats.redByteCount = arg_pStatsResult->redByteCount;
	    resp->dpe.u.stats.u.meterStats.u.colorAwareStats.redPacketCount = arg_pStatsResult->redPacketCount;
	}
	break;
	default:
	{
    	    #ifdef FPM_QOSMGR_DEBUG
	    printf("\n>>>>>>>>>> NPF_DS_E_UNKNOWN <<<<<<<<<<<<<\n");
    	    #endif
	    resp->dpe.error = NPF_DS_E_UNKNOWN;
	}
    }
    send_ix_cc_async_response(resp);
    return(IX_SUCCESS);
}

/********************************************************************************
* Function:	fpm_dropper_operation_callback					*
* Description:  WRED dropper operation callback					*
*		If operation os MODIFY and result is not SUCCESS, original	*
*		parameters are restored in local DPE. If operation is DELETE and*
*		result is SUCCESS, local DPE is removed from DpeList		*
*		If operation is ADD_ASSOC and result is SUCCESS, local DPE state*
*		is set to IDLE. If operation is DELETE_ASSOC and result is	*
*		SUCCESS, local DPE state is set to IDLE				*
*		In all cases, response is sent to user				*
********************************************************************************/
static ix_error fpm_dropper_operation_callback(ix_error arg_Result, void* arg_pContext)
{
    fpm_qos_dpe_t 	   *tmp;
    fpm_qos_dpe_t 	   *dpe = NULL;
    fpm_qos_request_t	   *resp;
    NPF_DS_DpeHandle_t	   handle;
    ix_error		   error;
    DS_FUNCTION();
    #ifdef IXASDKSTUBS
    error = arg_Result;
    #else
    error = IX_ERROR_GET_CODE(arg_Result);
    #endif
    resp = (fpm_qos_request_t *)arg_pContext;
    handle = resp->dpe.dpeHandle;
    dpe = (fpm_qos_dpe_t *)npf_list_searchByKey(&DpeList, &handle);
    if(dpe == NULL)
    {
    	#ifdef FPM_QOSMGR_DEBUG
	printf("******** ERROR: No DPE information Found ?????????********\n");
	#endif
	resp->dpe.error = NPF_DS_E_UNKNOWN;
        send_ix_cc_async_response(resp);
	return(IX_SUCCESS);
    }

    if(resp->dpe.op == NPF_DS_DPE_MODIFY)
    {
	 if(error == IX_SUCCESS)
	 {
	    PIL_FreeMemory(dpe->temp); /* Old Values */
	 }
	 else
	 {
    	    #ifdef FPM_QOSMGR_DEBUG
	    printf("NPF_DS_DPE_MODIFY(%d): Failed ???, Reverting old values ....\n", (int)handle);
	    #endif
	    tmp = (fpm_qos_dpe_t *)dpe->temp;
	    memcpy(&dpe->dpe, &tmp->dpe, sizeof(FPPI_DS_Dpe_t));
	 }
    }
    switch(error)
    {
	case IX_SUCCESS:
 	{
	    if(resp->dpe.op == NPF_DS_DPE_DELETE_ASSOC)
	    {
	        dpeSetState(resp->dpe.dpeHandle, resp->dpe.op, DPE_STATE_IDLE);
	    }
	    else if(resp->dpe.op == NPF_DS_DPE_ADD_ASSOC)
	    {
	        dpeSetState(resp->dpe.dpeHandle, resp->dpe.op, DPE_STATE_INUSE);
	    }
	    else
	        dpeSetState(resp->dpe.dpeHandle, resp->dpe.op, DPE_STATE_UNKNOWN);
	}
	break;
	default:
	    resp->dpe.error = NPF_DS_E_UNKNOWN;
    }
    send_ix_cc_async_response(resp);
    return(IX_SUCCESS);
}

#ifdef WRED_PACKET_COUNTER_FEATURE
/********************************************************************************
* Function:	fpm_dropper_stats_callback					*
* Description:  WRED dropper Statistics callback				*
*		SDK statistics are mapped to NPF_DS_StatsResp_t parameters and  *
*		response is send to user					*
********************************************************************************/
static ix_error fpm_dropper_stats_callback(ix_error arg_Result, 
					ix_cc_wred_statistics * arg_pStatsResult, 
					void* arg_pContext)
{
    fpm_qos_request_t	   *resp;
    ix_error		   error;
    DS_FUNCTION();
    #ifdef IXASDKSTUBS
    error = arg_Result;
    #else
    error = IX_ERROR_GET_CODE(arg_Result);
    #endif
    resp = (fpm_qos_request_t *)arg_pContext;
    switch(error)
    {
	case IX_SUCCESS:
 	{
	    resp->dpe.u.stats.u.dropperStats.type = NPF_DS_SRTCM_AWARE;
	    resp->dpe.u.stats.u.dropperStats.u.colorAwareDropperStats.greenByteDropped = arg_pStatsResult->greenByteCount;
	    resp->dpe.u.stats.u.dropperStats.u.colorAwareDropperStats.greenPacketDropped = arg_pStatsResult->greenPacketCount;
	    resp->dpe.u.stats.u.dropperStats.u.colorAwareDropperStats.yellowByteDropped = arg_pStatsResult->yellowByteCount;
	    resp->dpe.u.stats.u.dropperStats.u.colorAwareDropperStats.yellowPacketDropped = arg_pStatsResult->yellowPacketCount;
	    resp->dpe.u.stats.u.dropperStats.u.colorAwareDropperStats.redByteDropped = arg_pStatsResult->redByteCount;
	    resp->dpe.u.stats.u.dropperStats.u.colorAwareDropperStats.redPacketDropped = arg_pStatsResult->redPacketCount;
	}
	break;
	default:
	    resp->dpe.error = NPF_DS_E_UNKNOWN;
    }
    send_ix_cc_async_response(resp);
    return(IX_SUCCESS);
}

#endif

/********************************************************************************
*                                                                               *
*       0   1   2   3   4   5   6   7                                           *
*      +---+---+---+---+---+---+---+---+                                        *
*      |         DSCP          |  CU   |                                        *
*      +---+---+---+---+---+---+---+---+                                        *
*                                                                               *
*        DSCP: differentiated services codepoint                                *
*        CU:   currently unused                                                 *
*               Class1          Class2            Class3        Class 4         *
*Low Drop    (AF11)001010(10) (AF21)010010(18) (AF31)011010(26) (AF41)100010(34)*
*Medium Drop (AF12)001100(12) (AF22)10100(20)  (AF32)011100(28) (AF42)100100(36)*
*High Drop   (AF13)001110(14) (AF23)010110(22) (AF33)011110(30) (AF43)100110(38)*
********************************************************************************/
/*
 * In SDK, 16 Qs maintained per port.
 * First 8 Qs treated as hogh prority
*/
uint8_t genClassId(uint8_t  dscp)
{
    uint8_t   classId = BF_CLASS_ID;
    DS_FUNCTION();
    switch(dscp)
    {
        case 46: classId = EF_CLASS_ID; break;

        case 10: classId = AF11_CLASS_ID; break;
        case 12: classId = AF12_CLASS_ID; break;
        case 14: classId = AF13_CLASS_ID; break;

        case 18: classId = AF21_CLASS_ID; break;
        case 20: classId = AF22_CLASS_ID; break;
        case 22: classId = AF23_CLASS_ID; break;

        case 26: classId = AF31_CLASS_ID; break;
        case 28: classId = AF32_CLASS_ID; break;
        case 30: classId = AF33_CLASS_ID; break;

        case 34: classId = AF41_CLASS_ID; break;
        case 36: classId = AF42_CLASS_ID; break;
        case 38: classId = AF43_CLASS_ID; break;

        default:
            classId = BF_CLASS_ID;
    }
    DS_TRACE("classId   :       %d\n", classId);
    return(classId);
}
