/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/
#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include "FPModule.h"
#include "fp_common.h"
#include "npf_ds_types.h"
#include "dpe_mgr.h"
#include "log.h"
#include "qos_mgr.h"

extern router_config    g_current_router_config;

/* these variables define how data path elements can be linked together
   on the specific platform/NPU */
static uint32_t         g_clfr_count, g_marker_count, g_meter_count; 
static uint32_t         g_dropper_count, g_queue_count, g_scheduler_count;
static uint32_t         g_shaper_count;

static uint32_t         g_max_clfr_next_tbl_count, g_max_meter_next_tbl_count;
static uint32_t         g_max_marker_next_tbl_count;
static uint32_t         g_max_dropper_next_tbl_count;
static uint32_t         g_max_queue_next_tbl_count;
static uint32_t         g_max_scheduler_next_tbl_count;
static uint32_t         g_max_shaper_next_tbl_count;

int qos_init_fp ()
{
    QOS_DBG_TRACE ();

    InitFlowEventCallback ();

    switch (g_current_router_config) {
    case IPv4_ROUTER:
    case MPLS:
	break;
    case DS_MPLS_LER:
    case DS_MPLS_LSR:
	{
	    /* # of instances data path elements  */
	    g_clfr_count = g_meter_count = g_shaper_count = 1;
	    g_dropper_count = g_queue_count = g_scheduler_count = 1;
	    g_marker_count = 0;
 
	    g_max_clfr_next_tbl_count = 1; /* Classifier --> Meter */
	    g_max_marker_next_tbl_count = 0; /* NO Marker*/
	    g_max_meter_next_tbl_count = 2; /* Meter --> Dropper, Q */
	    g_max_dropper_next_tbl_count = 1; /* Dropper --> Queue */
	    g_max_queue_next_tbl_count = 1; /* Queue --> Scheduler */
	    g_max_shaper_next_tbl_count = 0; /* Shaper --> Scheduler */
	    g_max_scheduler_next_tbl_count = 0; /* Scheduler --> 0 */
	}
	break;
    case DS_EDGE:
	{
	    /* # of instances data path elements  */
	    g_clfr_count = g_marker_count = g_meter_count = g_shaper_count = 1;
	    g_dropper_count = g_queue_count = g_scheduler_count = 1;
	    
	    g_max_clfr_next_tbl_count = 4; /* Clfr --> Meter, Marker, Dropper,
					    Queue */
	    g_max_meter_next_tbl_count = 3; /* Meter --> Marker, Dropper, 
					       Queue */
	    g_max_marker_next_tbl_count = 3; /* Marker --> Meter, Dropper,
						Queue */
	    g_max_dropper_next_tbl_count = 1; /* Dropper --> Queue */
	    g_max_queue_next_tbl_count = 1; /* Queue --> Shaper */
	    g_max_shaper_next_tbl_count = 1; /* Shaper --> Scheduler */
	    g_max_scheduler_next_tbl_count = 0; /* Scheduler --> 0 */
	}
	break;
    case DS_CORE:
	{
	    /* # of instances data path elements  */
	    g_clfr_count = g_meter_count = g_shaper_count = 1;
	    g_dropper_count = g_queue_count = g_scheduler_count = 1;
	    g_marker_count = 0;

	    g_max_clfr_next_tbl_count = 3; /* Classifier --> Meter, Dropper, 
					      Queue */
	    g_max_marker_next_tbl_count = 0; /* NO Marker */
	    g_max_meter_next_tbl_count = 2; /* Meter --> Dropper, Queue */
	    g_max_dropper_next_tbl_count = 1; /* Dropper --> Queue */
	    g_max_queue_next_tbl_count = 1; /* Queue --> Shaper */
	    g_max_shaper_next_tbl_count = 1; /* Shaper --> Scheduler */
	    g_max_scheduler_next_tbl_count = 0; /* Scheduler --> 0 */
	}
	break;
    }
    PRETURN (0);
}

int qos_shutdown_fp ()
{    
    PRETURN (0);
}

/**************************************************************************
*
*   Function: qos_get_caps
*
*   Parameters: npf_ds_IfCaps_t* Interface capabilities
*
*   Description: Returns the QoS Capabilities supported on this Fwd. Plane
*
*   PRETURN : None 
*************************************************************************/

int qos_get_caps (npf_ds_IfCaps_t* qos_caps)
{
    QOS_DBG_TRACE ();

    /*	Common components - SRTC Meter, WRED Buf Mgr, DRR Scheduler */

    /* Basic Classification capabilities */
    qos_caps->ClfrCaps_Count = g_clfr_count;
    qos_caps->ClfrCaps_list[0].Base.type = npf_ds_ClfrCaps;
    qos_caps->ClfrCaps_list[0].Base.tableid = DPE_CLASSIFIER_TABLE_ID;
    qos_caps->ClfrCaps_list[0].Base.Direction = npf_ds_IngressOnly;
    qos_caps->ClfrCaps_list[0].Spec = 0;
    qos_caps->ClfrCaps_list[0].Base.stats_supported = 0;
    qos_caps->ClfrCaps_list[0].Base.next_tables_count = 
	g_max_clfr_next_tbl_count;
    switch (g_current_router_config) {
    case IPv4_ROUTER:
    case MPLS:
	break;
    case DS_MPLS_LER:
	{
	    /* Classifier = CIDR-based FEC classifier
	       Marker     = MPLS marker
	    */
	    QOS_DBG_PRINT (LOG_INFO, "Collecting Caps for DS_MPLS_ILER!\n");
	    qos_caps->ClfrCaps_list[0].Spec |= 0x031F; 
	    /* Bit 10 - MPLS FEC
	       Bit 9  - MPLS Label
	       Bits 0-5 - IPv4 and DSCP filter. */
	    /****************************************************
	     * this is a hack, actually. Even though FP can report IPv4 and
	     * DSCP filters separately, code in the DiffServ module inthe
	     * control plane checks only for IPv4. So, if FP reports DSCP,
	     * this cannot be validated
	     ******************************************************/
	    qos_caps->ClfrCaps_list[0].Base.stats_supported = 0x0001; 
	    /* Pkt Count */
	    /* Classifier can be followed by Marker ONLY in MPLS+QoS */
	    qos_caps->ClfrCaps_list[0].Base.next_tables_list[0] = 
		DPE_METER_TABLE_ID;

	    /* MPLS Marker Capabilities */
	    qos_caps->MarkerCaps_Count = 0;

	       /* fill Meter Capabilities */
	    qos_caps->MeterCaps_Count = g_meter_count;
	    qos_caps->MeterCaps_list[0].Spec = 0;
	    //qos_caps->MeterCaps_list[0].Spec |= 0x0004; /* Bit 3 - srTCMBlind */
	    qos_caps->MeterCaps_list[0].Spec |= 0x0001; /* Bit 1 - Simple */
	    qos_caps->MeterCaps_list[0].Base.type = npf_ds_MeterCaps;
	    qos_caps->MeterCaps_list[0].Base.tableid = DPE_METER_TABLE_ID;
	    qos_caps->MeterCaps_list[0].Base.Direction = npf_ds_IngressOnly;
	    qos_caps->MeterCaps_list[0].Base.stats_supported = 0;
	    qos_caps->MeterCaps_list[0].Base.stats_supported = 0x0003; 
	    /* Byte count&
	       pkt count */
	    qos_caps->MeterCaps_list[0].Base.next_tables_count = 
		g_max_meter_next_tbl_count;
	    qos_caps->MeterCaps_list[0].Base.next_tables_list[0] = 
		DPE_ALG_DROPPER_TABLE_ID;
	    qos_caps->MeterCaps_list[0].Base.next_tables_list[1] = 
		DPE_QUEUE_TABLE_ID;
	    qos_caps->MeterCaps_list[0].failnext_tables_count =
		g_max_meter_next_tbl_count;
	    qos_caps->MeterCaps_list[0].failnext_tables_list[0] =
		DPE_ALG_DROPPER_TABLE_ID;
	    qos_caps->MeterCaps_list[0].failnext_tables_list[1] =
		DPE_QUEUE_TABLE_ID;
	}
	break;
    case DS_MPLS_LSR:
	{
	    /* Classifier = Label-based classifier
	       Marker     = MPLS marker
	    */
	    QOS_DBG_PRINT (LOG_INFO, "Collecting Caps for DS_MPLS_LSR!\n");
	    qos_caps->ClfrCaps_list[0].Spec = 0x011F; 
	    /* bit 9 - MPLS Label
	       bit 0-5 - IPv4 + DSCP classifier */
	    /****************************************************
	     * this is a hack, actually. Even though FP can report IPv4 and
	     * DSCP filters separately, code in the DiffServ module inthe
	     * control plane checks only for IPv4. So, if FP reports DSCP,
	     * this cannot be validated
	     ******************************************************/
	    qos_caps->ClfrCaps_list[0].Base.stats_supported = 0x0001; 
	    /* Pkt Count */
	    /* Classifier can be followed by Marker ONLY in MPLS+QoS */
	    qos_caps->ClfrCaps_list[0].Base.next_tables_list[0] = 
		DPE_METER_TABLE_ID;

	    /* MPLS Marker Capabilities */
	    qos_caps->MarkerCaps_Count = 0;

	    /* fill Meter Capabilities */
	    qos_caps->MeterCaps_Count = g_meter_count;
	    qos_caps->MeterCaps_list[0].Spec = 0;
	    //qos_caps->MeterCaps_list[0].Spec |= 0x0004; /* Bit 3 - srTCMBlind */
	    qos_caps->MeterCaps_list[0].Spec |= 0x0001; /* Bit 1 - Simple */
	    qos_caps->MeterCaps_list[0].Base.type = npf_ds_MeterCaps;
	    qos_caps->MeterCaps_list[0].Base.tableid = DPE_METER_TABLE_ID;
	    qos_caps->MeterCaps_list[0].Base.Direction = npf_ds_IngressOnly;
	    qos_caps->MeterCaps_list[0].Base.stats_supported = 0;
	    qos_caps->MeterCaps_list[0].Base.stats_supported = 0x0003; 
	    /* Byte count & pkt count */	    
	    qos_caps->MeterCaps_list[0].Base.next_tables_count = 
		g_max_meter_next_tbl_count;
	    qos_caps->MeterCaps_list[0].Base.next_tables_list[0] = 
		DPE_ALG_DROPPER_TABLE_ID;
	    qos_caps->MeterCaps_list[0].Base.next_tables_list[1] = 
		DPE_QUEUE_TABLE_ID;
	    qos_caps->MeterCaps_list[0].failnext_tables_count =
		g_max_meter_next_tbl_count;
	    qos_caps->MeterCaps_list[0].failnext_tables_list[0] =
		DPE_ALG_DROPPER_TABLE_ID;
	    qos_caps->MeterCaps_list[0].failnext_tables_list[1] =
		DPE_QUEUE_TABLE_ID;
	}
	break;
    case DS_EDGE:
	{
	    /* IPv4 Range Matching Classifier
	       DSCP Marker
	    */
	    QOS_DBG_PRINT (LOG_INFO, "Collecting Capabilities for DS_EDGE !\n");
	    qos_caps->ClfrCaps_list[0].Spec |= 0x001F; /* IPv4 Range Matching Clfr */
	    qos_caps->ClfrCaps_list[0].Base.stats_supported = 0x0001; /* Pkt Count */
	    /* Classifier can be followed by 
	       -- Marker
	       -- Meter
	       -- Buf Mgr (Algo Dropper)
	    */
	    qos_caps->ClfrCaps_list[0].Base.next_tables_list[0] = 
		DPE_MARKER_TABLE_ID;
	    qos_caps->ClfrCaps_list[0].Base.next_tables_list[1] = 
		DPE_METER_TABLE_ID;
	    qos_caps->ClfrCaps_list[0].Base.next_tables_list[2] = 
		DPE_ALG_DROPPER_TABLE_ID;
	    qos_caps->ClfrCaps_list[0].Base.next_tables_list[3] = 
		DPE_QUEUE_TABLE_ID;
	
	    /* DSCP Marker Capabilities */
	    qos_caps->MarkerCaps_Count = g_marker_count;
	    qos_caps->MarkerCaps_list[0].Base.type = npf_ds_MarkerCaps;
	    qos_caps->MarkerCaps_list[0].Base.tableid = DPE_MARKER_TABLE_ID;
	    qos_caps->MarkerCaps_list[0].Base.Direction = npf_ds_IngressOnly;
	    qos_caps->MarkerCaps_list[0].Spec = 0; 
	    qos_caps->MarkerCaps_list[0].Spec |= 0x0001; /* DSCP Marker */
	    qos_caps->MarkerCaps_list[0].Base.stats_supported = 0; 
	    qos_caps->MarkerCaps_list[0].Base.stats_supported = 0x0001; /*Pkt Count*/
	    qos_caps->MarkerCaps_list[0].Base.next_tables_count = 
		g_max_marker_next_tbl_count;
	    /* Marker --> Meter
	       Marker --> Dropper */
	    qos_caps->MarkerCaps_list[0].Base.next_tables_list[0] = 
		DPE_METER_TABLE_ID;
	    qos_caps->MarkerCaps_list[0].Base.next_tables_list[1] = 
		DPE_ALG_DROPPER_TABLE_ID;
	    qos_caps->MarkerCaps_list[0].Base.next_tables_list[2] = 
		DPE_QUEUE_TABLE_ID;

	    /* fill Meter Capabilities */
	    qos_caps->MeterCaps_Count = g_meter_count;
	    qos_caps->MeterCaps_list[0].Spec = 0;
	    //qos_caps->MeterCaps_list[0].Spec |= 0x0004; /* Bit 3 - srTCMBlind */
	    qos_caps->MeterCaps_list[0].Spec |= 0x0001; /* Bit 1 - Simple */
	    qos_caps->MeterCaps_list[0].Base.type = npf_ds_MeterCaps;
	    qos_caps->MeterCaps_list[0].Base.tableid = DPE_METER_TABLE_ID;
	    qos_caps->MeterCaps_list[0].Base.Direction = npf_ds_IngressOnly;
	    qos_caps->MeterCaps_list[0].Base.stats_supported = 0;
	    qos_caps->MeterCaps_list[0].Base.stats_supported = 0x0003; 
	    /* Byte count & pkt count */
	    qos_caps->MeterCaps_list[0].Base.next_tables_count = 
		g_max_meter_next_tbl_count;
	    qos_caps->MeterCaps_list[0].Base.next_tables_list[0] = 
		DPE_MARKER_TABLE_ID;
	    qos_caps->MeterCaps_list[0].Base.next_tables_list[1] = 
		DPE_ALG_DROPPER_TABLE_ID;
	    qos_caps->MeterCaps_list[0].Base.next_tables_list[2] = 
		DPE_QUEUE_TABLE_ID;
	    qos_caps->MeterCaps_list[0].failnext_tables_count =
		g_max_meter_next_tbl_count;

	    qos_caps->MeterCaps_list[0].failnext_tables_list[0] = 
		DPE_MARKER_TABLE_ID;
	    qos_caps->MeterCaps_list[0].failnext_tables_list[1] = 
		DPE_ALG_DROPPER_TABLE_ID;
	    qos_caps->MeterCaps_list[0].failnext_tables_list[2] = 
		DPE_QUEUE_TABLE_ID;
	}
	break;
    case DS_CORE:
	{
	    /* DSCP Classifier
	       No marker
	    */
	    QOS_DBG_PRINT (LOG_INFO, "Collecting Caps for DS_MPLS_LSR!\n");
	    qos_caps->ClfrCaps_list[0].Spec = 0x001F; /* DSCP */
	    qos_caps->ClfrCaps_list[0].Base.stats_supported = 0x0001; 
	    /* Pkt Count */
	    /* Classifier can be followed by 
	       -- Meter
	       -- Buf Mgr (Algo Dropper)
	    */
	    qos_caps->ClfrCaps_list[0].Base.next_tables_list[0] = 
		DPE_METER_TABLE_ID;
	    qos_caps->ClfrCaps_list[0].Base.next_tables_list[1] = 
		DPE_ALG_DROPPER_TABLE_ID;
	    qos_caps->ClfrCaps_list[0].Base.next_tables_list[2] = 
		DPE_QUEUE_TABLE_ID;
	    /* fill Meter Capabilities */
	    qos_caps->MeterCaps_Count = g_meter_count;
	    qos_caps->MeterCaps_list[0].Spec = 0;
	    //qos_caps->MeterCaps_list[0].Spec |= 0x0004; /* Bit 3 - srTCMBlind */
	    qos_caps->MeterCaps_list[0].Spec |= 0x0001; /* Bit 1 - Simple */
	    qos_caps->MeterCaps_list[0].Base.type = npf_ds_MeterCaps;
	    qos_caps->MeterCaps_list[0].Base.tableid = DPE_METER_TABLE_ID;
	    qos_caps->MeterCaps_list[0].Base.Direction = npf_ds_IngressOnly;
	    qos_caps->MeterCaps_list[0].Base.stats_supported = 0;
	    qos_caps->MeterCaps_list[0].Base.stats_supported = 0x0003; 
	    /* Byte count & pkt count */
	    qos_caps->MeterCaps_list[0].Base.next_tables_count = 
		g_max_meter_next_tbl_count;
	    qos_caps->MeterCaps_list[0].Base.next_tables_list[0] = 
		DPE_ALG_DROPPER_TABLE_ID;
	    qos_caps->MeterCaps_list[0].Base.next_tables_list[1] = 
		DPE_QUEUE_TABLE_ID;

	    qos_caps->MeterCaps_list[0].failnext_tables_count =
		g_max_meter_next_tbl_count;
	    qos_caps->MeterCaps_list[0].failnext_tables_list[0] =
		DPE_ALG_DROPPER_TABLE_ID;
	    qos_caps->MeterCaps_list[0].failnext_tables_list[1] =
		DPE_QUEUE_TABLE_ID;
	}
	break;
    } /*end switch (router_config) */
 
     /* fill in Algo Dropper capabilities */
    qos_caps->DropperCaps_Count = g_dropper_count;
    qos_caps->DropperCaps_list[0].Type = 0;
    qos_caps->DropperCaps_list[0].Type |= 0x0018; /* Bit 4 - mQDrop */
    qos_caps->DropperCaps_list[0].Base.type = npf_ds_AlgDropCaps;
    qos_caps->DropperCaps_list[0].Base.tableid = DPE_ALG_DROPPER_TABLE_ID;
    qos_caps->DropperCaps_list[0].Base.Direction = npf_ds_IngressOnly;
    qos_caps->DropperCaps_list[0].Base.stats_supported = 0;
    qos_caps->DropperCaps_list[0].Base.stats_supported |= 0x0001; /* Pkt count
								   */ 
    qos_caps->DropperCaps_list[0].Base.next_tables_count = 
	g_max_dropper_next_tbl_count;
    qos_caps->DropperCaps_list[0].Base.next_tables_list[0] = 
	DPE_QUEUE_TABLE_ID;

    /* fill in Queue (Buffer Manager) capabilities */
    qos_caps->QueueCaps_Count = g_queue_count;
    qos_caps->QueueCaps_list[0].Base.type = npf_ds_QCaps;
    qos_caps->QueueCaps_list[0].Base.tableid = DPE_QUEUE_TABLE_ID;
    qos_caps->QueueCaps_list[0].Base.Direction = npf_ds_EgressOnly;
    qos_caps->DropperCaps_list[0].Base.stats_supported = 0; 
    qos_caps->DropperCaps_list[0].Base.stats_supported |= 0x0001; /* Pkt count
								   */
    //qos_caps->QueueCaps_list[0].MinQueueSize = ;
    //qos_caps->QueueCaps_list[0].MaxQueueSize = ;
    //qos_caps->QueueCaps_list[0].TotalQueueSize = ;
    
    qos_caps->QueueCaps_list[0].Base.next_tables_count = 
	g_max_queue_next_tbl_count;
    qos_caps->QueueCaps_list[0].Base.next_tables_list[0] = DPE_SCHEDULER_TABLE_ID;;
    
    /* fill in Shaper capabilities */
    qos_caps->ShaperCaps_Count = g_shaper_count;
    qos_caps->ShaperCaps_list[0].Base.type = npf_ds_ShaperCaps;
    qos_caps->ShaperCaps_list[0].Base.tableid = DPE_SHAPER_TABLE_ID;
    qos_caps->ShaperCaps_list[0].Base.Direction = npf_ds_EgressOnly;
    qos_caps->ShaperCaps_list[0].MaxLevels = 1; /* One level supported */
    qos_caps->ShaperCaps_list[0].Base.next_tables_count = 0;
    //g_max_shaper_next_tbl_count;
    qos_caps->ShaperCaps_list[0].Base.next_tables_list[0] = -1;
    /* not followed by anything */

    /* fill in Scheduler capabilities */
    qos_caps->SchedulerCaps_Count = g_scheduler_count;
    qos_caps->SchedulerCaps_list[0].Base.type = npf_ds_SchedulerCaps;
    qos_caps->SchedulerCaps_list[0].Base.tableid = DPE_SCHEDULER_TABLE_ID;
    qos_caps->SchedulerCaps_list[0].Base.Direction = npf_ds_EgressOnly;
    qos_caps->SchedulerCaps_list[0].Base.stats_supported = 0;
    qos_caps->SchedulerCaps_list[0].Base.stats_supported |= 0x0003; 
    /* Pkt & byte count */
    qos_caps->SchedulerCaps_list[0].ServiceDisc = npf_ds_SchedulerDRR;
    //qos_caps->SchedulerCaps_list[0].MaxInputs = ;
    qos_caps->SchedulerCaps_list[0].MinMaxRate = 0x0003; 
    /* Minrate only - bit 1*/
    qos_caps->SchedulerCaps_list[0].Base.next_tables_count = 0; 

    PRETURN (0);
}

/*************************************************************************
 *                                                                       *
 *                         ADD ENTRY ROUTINES                            *
 *                                                                       *
 *************************************************************************/
/**************************************************************************
*
*   Function: qos_add_classifier()
*
*   Parameters: IN: Flow Tuple
*               IN: npf_ds_BaseFilter_t 
*               OUT: filter handle
*
*   Description: Adds a DSCP or IPv4 Range Matching filter and returns
*                a filter id that should be used for deleting
*
*   PRETURN : None 
*************************************************************************/
int qos_add_classifier (dpe_flow_tuple_t flow_tpl, npf_ds_BaseFilter_t* clfr,
			filter_handle_t* filter_handle)
{
    char                 dscp;
    npf_ds_IPv4Filter_t* ipv4_filter;
    unsigned long        result;
    int                  err = 0;

    QOS_DBG_TRACE ();
    QOS_DBG_PRINT (LOG_INFO, "Classifier params\n\t"
		   "Flow Id              = %d \n\t"
		   "Queue Id             = %d \n\t"
		   "Next Blk Id          = %d \n\t",
		   flow_tpl.flow_id, flow_tpl.queue_id,
		   flow_tpl.next_block_id);

    if (clfr->type == npf_ds_IPv4Filter)
	ipv4_filter = (npf_ds_IPv4Filter_t*) clfr;
    else {
	QOS_DBG_PRINT (LOG_ERR, "Unknown filter type!\n");
	err = -1;
	PRETURN (err);
    }
    switch (g_current_router_config) {
    case DS_EDGE:
	{
	    /* Edge routers have a IPv4 Range Matching classifier */

	    QOS_DBG_PRINT (LOG_INFO, "IPv4 Clfr params!\n\t"
			   "Src Addr     = 0x%x \n\t"
			   "Dst Addr     = 0x%x \n\t"
			   "Src Port Min = 0x%x \n\t"
			   "Src Port Max = 0x%x \n\t"
			   "Dst Port Min = 0x%x \n\t"
			   "Dst Port Max = 0x%x \n\t"
			   "Priority     = 0x%x \n\t"
			   "Protocol     = 0x%x \n\t",
			   ipv4_filter->SrcAddr, ipv4_filter->DstAddr,
			   ipv4_filter->SrcL4PortMin,
			   ipv4_filter->SrcL4PortMax,
			   ipv4_filter->DstL4PortMin,
			   ipv4_filter->DstL4PortMax, clfr->Precedence,
			   ipv4_filter->Protocol);
	}
	break;
    case DS_CORE:
	{
	    /* Core routers have a DSCP/TOS classifier */
	    dscp = ipv4_filter->Dscp;
	    QOS_DBG_PRINT (LOG_INFO, "IPv4 Clfr params!\n\t"
			   " DSCP = 0x%x \n\t", dscp);
	}
	break;
    } /* end switch (router config) */

    PRETURN (err);
}

/**************************************************************************
*
*   Function: qos_add_marker()
*
*   Parameters: IN: Flow Tuple
*               IN: DSCP mark to apply
*
*   Description: sets up a mapping between a DSCP and Flow ID in the DSCP
*                Marker
*
*   PRETURN : None 
*************************************************************************/
int qos_add_marker (dpe_flow_tuple_t flow_tuple, npf_ds_Action_t* action)
{
    npf_ds_DscpMarkAct_t* dscp_mark;
    unsigned long         result;
    int                   err = 0;
    
    QOS_DBG_TRACE ();
    QOS_DBG_PRINT (LOG_INFO, "Marker params\n\t"
		   "Flow Id              = %d \n\t"
		   "Queue Id             = %d \n\t"
		   "Next Blk Id          = %d \n\t",
		   flow_tuple.flow_id, flow_tuple.queue_id,
		   flow_tuple.next_block_id);

    if (*action->Specific == npf_ds_DscpMarkAct)
      dscp_mark = (npf_ds_DscpMarkAct_t*) action->Specific;

    QOS_DBG_PRINT (LOG_INFO, "DSCP mark = 0x%x \n\t", dscp_mark->Dscp);
    
    PRETURN (err);
}

/**************************************************************************
*
*   Function: qos_add_scheduler_entry()
*
*   Parameters: Interface (port/queue_group id)
*               Flow tuple
*               npf_ds_MinRate_t*, npf_ds_MaxRate_t*
*
*   Description: Sets up a scheduler for the specified queue
*
*   PRETURN : None 
*************************************************************************/

int qos_add_scheduler_entry (uint32_t intf, dpe_flow_tuple_t flow_tuple,
			     npf_ds_MinRate_t* min_rate, 
			     npf_ds_MaxRate_t* max_rate)
{
    int            err = 0;

    QOS_DBG_TRACE ();
    QOS_DBG_PRINT (LOG_INFO, "Scheduler Parameters!\n\t"
		   "Interface Id = %d \n\t"
		   " Flow Id     = %d \n\t"
		   " Queue Id    = %d \n\t", intf, flow_tuple.flow_id,
		   flow_tuple.queue_id);

    PRETURN (err);
}

/**************************************************************************
*
*   Function: qos_add_default_scheduler_entry()
*
*   Parameters: Interface (port/queue_group id)
*               Flow tuple
*
*   Description: Sets up a default scheduler for the specified queue
*
*   PRETURN : None 
*************************************************************************/

int qos_add_default_scheduler_entry (uint32_t intf, dpe_flow_tuple_t flow_tuple)
{
    int            err = 0;

    QOS_DBG_TRACE ();
    QOS_DBG_PRINT (LOG_INFO, "Scheduler Parameters!\n\t"
		   "Interface Id = %d \n\t"
		   " Flow Id     = %d \n\t"
		   " Queue Id    = %d \n\t", intf, flow_tuple.flow_id,
		   flow_tuple.queue_id);


    PRETURN (err);
}

/**************************************************************************
*
*   Function: qos_add_default_queue_entry()
*
*   Parameters: Interface (port/queue_group id)
*               Flow id
*
*   Description: Sets up the EWMA constant for the corr. Queue Id.
*
*   PRETURN : None 
*************************************************************************/

int qos_add_default_queue_entry (uint32_t intf, uint32_t q_id)
{

    int            err = 0;

    QOS_DBG_TRACE ();
    QOS_DBG_PRINT (LOG_INFO, "Queue Parameters!\n\t"
		   "Interface Id = %d \n\t"
		   " Queue Id    = %d \n\t", intf, q_id);

     PRETURN (err);
}

/**************************************************************************
 *
 *   Function: qos_set_dropper_params()
 *
 *   Parameters: Interface (port/queue_group id)
 *               Flow id
 *
 *   Description: Sets up the EWMA constant, etc. for the corr. Queue Id.
 *
 *   PRETURN : None 
 *************************************************************************/

int qos_set_dropper_params (uint32_t intf, dpe_flow_tuple_t flow_tuple,
			    npf_ds_AlgDrop_t* dropper)
{
    unsigned long        ewma;
    npf_ds_RandomDrop_t* random_dropper;
    unsigned long        result;
    int                  err = 0;

    QOS_DBG_TRACE ();
    QOS_DBG_PRINT (LOG_INFO, "Queue Parameters!\n\t"
		   "Interface Id = %d \n\t"
		   " Queue Id    = %d \n\t", intf, flow_tuple.queue_id);

    /* Now set the Queue parameters */
    if (dropper->dropper.sqalgdrop.Type == Type_randomDrop) {
	random_dropper = (npf_ds_RandomDrop_t*) 
	    dropper->dropper.sqalgdrop.Specific;
    	ewma = (unsigned long) random_dropper->Weight;
    	QOS_DBG_PRINT (LOG_INFO, "Setting EWMA for Q id = %d\n\t"
			   "Queue grp id = %d \n\t"
			   "Ewma = %d \n\t", flow_tuple.queue_id, intf, ewma);
    }

    if (dropper->dropper.sqalgdrop.Type == Type_alwaysDrop) { 
	/* NULL if absolute dropper */
	QOS_DBG_PRINT (LOG_INFO, "Adding absolute dropper for Flow ID = %d Queue Id = %d\n",
		       flow_tuple.flow_id, flow_tuple.queue_id);
    }
    PRETURN (err);
}

/**************************************************************************
*
*   Function: qos_add_default_red_instance()
*
*   Parameters: Flow id
*
*   Description: Installs a default RED parameter for the corr. flow id
*
*   PRETURN : None 
*************************************************************************/

int qos_add_default_red_instance (uint32_t flow_id)
{
    int            err = 0;

    QOS_DBG_TRACE ();
    QOS_DBG_PRINT (LOG_INFO, " WRED Parameters!\n\t"
		   " Flow Id    = %d \n\t",flow_id);

    PRETURN (err);
}

/**************************************************************************
*
*   Function: qos_create_meter()
*
*   Parameters: Flow Tuple
*
*   Description: Installs a new meter 
*
*   PRETURN : None 
*************************************************************************/

int qos_create_meter (dpe_flow_tuple_t flow_tuple)
{
     int                  err = 0;

    QOS_DBG_TRACE ();
    QOS_DBG_PRINT (LOG_INFO, "SRTC Meter Parameters!\n\t"
		   "Flow Id              = %d \n\t"
		   "Queue Id             = %d \n\t"
		   "Next Blk Id          = %d \n",
		   flow_tuple.flow_id, flow_tuple.queue_id,
		   flow_tuple.next_block_id);

    PRETURN (err);
}

/**************************************************************************
*
*   Function: qos_setup_meter()
*
*   Parameters: IN: Flow Tuple
*               IN: Downstream flow tuple
*               IN: Fail next flow tuple
*
*   Description: Sets up a meter for updating incoming flow ids to map to
*                the corresponding outgoing downstream flow ids, for both
*                conforming and non-conforming traffic 
*
*   PRETURN : None 
*************************************************************************/

int qos_setup_meter (dpe_flow_tuple_t flow_tuple, 
		     dpe_flow_tuple_t downstream_flow_tuple,
		     dpe_flow_tuple_t failnext_flow_tuple)
{
    int                  err = 0;
    
    QOS_DBG_TRACE ();
    QOS_DBG_PRINT (LOG_INFO, "SRTC Meter Parameters!\n\t"
		   "Flow Id              = %d \n\t"
		   "Queue Id             = %d \n\t"
		   "Next Blk Id          = %d \n"
		   "Downstream flow id   = %d \n\t"
		   "Downstream queue id  = %d \n\t"
		   "Downstream NBlk id   = %d \n\t"
		   "Fail next Flow Id    = %d \n\t"
		   "fail next Queue Id   = %d \n\t"
		   "fail next Next Blk Id= %d \n",
		   flow_tuple.flow_id, 
		   flow_tuple.queue_id,
		   flow_tuple.next_block_id,
		   downstream_flow_tuple.flow_id, 
		   downstream_flow_tuple.queue_id,
		   downstream_flow_tuple.next_block_id, 
		   failnext_flow_tuple.flow_id, 
		   failnext_flow_tuple.queue_id, 
		   failnext_flow_tuple.next_block_id);

     PRETURN (err);
}

/**************************************************************************
*
*   Function: qos_setup_meter()
*
*   Parameters: IN: Flow Tuple
*               IN: Downstream flow tuple
*               IN: Fail next flow tuple
*
*   Description: Sets up a meter for updating incoming flow ids to map to
*                the corresponding outgoing downstream flow ids, for both
*                conforming and non-conforming traffic 
*
*   PRETURN : None 
*************************************************************************/
int qos_set_meter_params (dpe_flow_tuple_t flow_tuple, npf_ds_Meter_t* meter)
{
    npf_ds_TBParam_t*    tb_params;
    int                  err = 0;

    QOS_DBG_TRACE ();
    QOS_DBG_PRINT (LOG_INFO, "SRTC Meter Parameters!\n\t"
		   "Flow Id              = %d \n\t"
		   "Queue Id             = %d \n\t"
		   "Next Blk Id          = %d \n",
		   flow_tuple.flow_id, flow_tuple.queue_id,
		   flow_tuple.next_block_id);

     /* now set the SRTC parameters */
    if (*meter->Specific == npf_ds_TBParam)
	tb_params = (npf_ds_TBParam_t*) (meter->Specific);
    else {
	QOS_DBG_PRINT (LOG_ERR, "Unsupported Meter type - %d\n",
		       meter->Specific);
	//PRETURN (-1);
    }
    QOS_DBG_PRINT (LOG_INFO, "SRTC parameters\n\t Rate: %d \n\t"
		   "Burst = %d \n\t Interval = %d \n\t",
		   tb_params->Rate, tb_params->BurstSize, 
		   tb_params->Interval); 

     PRETURN (err);
}

/*************************************************************************
 *                                                                       *
 *                         MODIFY ENTRY ROUTINES                         *
 *                                                                       *
 *************************************************************************/

/**************************************************************************
*
*   Function: qos_modify_scheduler_entry()
*
*   Parameters: Interface (port/queue_group id)
*               Flow tuple
*               npf_ds_Scheduler
*
*   Description: Modifies a scheduler for the specified queue
*
*   PRETURN : None 
*************************************************************************/

int qos_modify_scheduler_entry (uint32_t intf, dpe_flow_tuple_t flow_tuple,
				npf_ds_MinRate_t* min_rate,
				npf_ds_MaxRate_t* max_rate)
{
    int            err = 0;

    QOS_DBG_TRACE ();
    QOS_DBG_PRINT (LOG_INFO, "Scheduler Parameters!\n\t"
		   "Interface Id = %d \n\t"
		   " Flow Id     = %d \n\t"
		   " Queue Id    = %d \n\t", intf, flow_tuple.flow_id,
		   flow_tuple.queue_id);

     PRETURN (err);
}

/**************************************************************************
*
*   Function: qos_modify_meter()
*
*   Parameters: Flow Tuple
*               npf_ds_TBParam_t - token bucket params
*
*   Description: Modifies a meter 
*
*   PRETURN : None 
*************************************************************************/

int qos_modify_meter (dpe_flow_tuple_t flow_tuple, npf_ds_Meter_t* meter)
{
    npf_ds_TBParam_t*    tb_params;
    int                  err = 0;

    QOS_DBG_TRACE ();
    QOS_DBG_PRINT (LOG_INFO, "SRTC Meter Parameters!\n\t"
		   "Flow Id              = %d \n\t",
		   flow_tuple.flow_id);


    /* now set the SRTC parameters */
    if (meter->Specific == npf_ds_TBParam)
	tb_params = (npf_ds_TBParam_t*) (meter->Specific);
    else {
	QOS_DBG_PRINT (LOG_ERR, "Unsupported Meter type - %d\n",
		       meter->Specific);
	PRETURN (-1);
    }
    QOS_DBG_PRINT (LOG_INFO, "SRTC parameters\n\t Rate: %d \n\t"
		   "Burst = %d \n\t Interval = %d \n\t",
		   tb_params->Rate, tb_params->BurstSize, 
		   tb_params->Interval); 

     PRETURN (err);
}

/*************************************************************************
 *                                                                       *
 *                         DELETE ENTRY ROUTINES                         *
 *                                                                       *
 *************************************************************************/
/**************************************************************************
*
*   Function: qos_delete_classifier()
*
*   Parameters: IN: Filter ID
*
*   Description: Deletes a classifier installed earlier
*
*   PRETURN : None 
*************************************************************************/
int qos_delete_classifier (filter_handle_t filter_handle)
{
    unsigned long        result;
    int                  err = 0;

    QOS_DBG_TRACE ();

    QOS_DBG_PRINT (LOG_INFO, "Delete Classifier params \n\t"
		   "Filter id = 0x%x \n", filter_handle);

    PRETURN (err);
}


/**************************************************************************
*
*   Function: qos_delete_marker()
*
*   Parameters: IN: Flow ID
*
*   Description: Deletes a marker mapping installed earlier
*
*   PRETURN : None 
*************************************************************************/
int qos_delete_marker (dpe_flow_tuple_t flow_tuple)
{
    unsigned long result;
    int           err = 0;

    QOS_DBG_TRACE ();

    QOS_DBG_PRINT (LOG_INFO, "Delete marker params \n\t"
		   "Flow id = %d \n", flow_tuple.flow_id);

    PRETURN (err);
}

/**************************************************************************
*
*   Function: qos_delete_meter()
*
*   Parameters: Flow Tuple
*               npf_ds_TBParam_t - token bucket params
*
*   Description: Deletes a meter 
*
*   PRETURN : 0 on success, -1 for failure
*************************************************************************/

int qos_delete_meter (dpe_flow_tuple_t flow_tuple, 
		      dpe_flow_tuple_t failnext_flow_tuple)
{
    int           err = 0;

    QOS_DBG_TRACE ();

    PRETURN (err);
}

/**************************************************************************
*
*   Function: qos_delete_dropper()
*
*   Parameters: Flow Tuple
*
*   Description: Deletes a dropper 
*
*   PRETURN : 0 on success, -1 for failure
*************************************************************************/

int qos_delete_dropper (dpe_flow_tuple_t flow_tuple) 
{
     int           err = 0;

    QOS_DBG_TRACE ();

    PRETURN (err);
}

/**************************************************************************
*
*   Function: qos_delete_queue()
*
*   Parameters: IN: Flow Tuple
*               IN: Queue Group Id (Interface)
*   Description: Deletes a queue
*
*   PRETURN : 0 on success, -1 for failure
*************************************************************************/

int qos_delete_queue (dpe_flow_tuple_t flow_tuple, uint32_t intf) 
{
     int           err = 0;

    QOS_DBG_TRACE ();

    PRETURN (err);
}

/**************************************************************************
*
*   Function: qos_delete_scheduler()
*
*   Parameters: IN: Flow Tuple
*               IN: Queue Group Id (Interface)
*   Description: Deletes a Queue in Scheduler
*
*   PRETURN : 0 on success, -1 for failure
*************************************************************************/

int qos_delete_scheduler (dpe_flow_tuple_t flow_tuple, uint32_t intf) 
{
    int           err = 0;

    QOS_DBG_TRACE ();


    PRETURN (err);
}

/*************************************************************************
 *                                                                       *
 *                         GET STATISTICS ROUTINES                       *
 *                                                                       *
 *************************************************************************/
/**************************************************************************
*
*   Function: qos_get_clfr_stats
*
*   Parameters: IN: Flow Id
*               OUT:Packet count, Byte count
* 
*   Description: Gets statistics counters from the specified classifier
*
*   PRETURN : None 
*************************************************************************/

int qos_get_clfr_stats (filter_handle_t filter_hdl,
			unsigned long long* defaulted_pkt_count,
			unsigned long long* pkt_count,
			unsigned long long* dropped_pkt_count,
			unsigned long long* defaulted_byte_count,
			unsigned long long* byte_count,
			unsigned long long* dropped_byte_count)
{
    long           result;
    int            err = 0;
    
    QOS_DBG_TRACE ();
    QOS_DBG_PRINT (LOG_INFO, "Get Clfr Statistics Parameters!\n\t"
		   "Filter handle = %d \n", filter_hdl);

    PRETURN (err);
}

/**************************************************************************
*
*   Function: qos_get_marker_stats
*
*   Parameters: OUT: number marked
*               OUT: number dropped
* 
*   Description: Gets statistics counters from the specified marker
*
*   PRETURN : None 
*************************************************************************/

int qos_get_marker_stats (unsigned long long* pkts_marked,
			  unsigned long long* pkts_dropped,
			  unsigned long long* bytes_marked,
			  unsigned long long* bytes_dropped)
{
    long           result;
    int            err = 0;
    
    QOS_DBG_TRACE ();


    PRETURN (err);
}

/**************************************************************************
*
*   Function: qos_get_scheduler_stats
*
*   Parameters: IN: Queue Group Id
*               IN: Queue Id
*               OUT:Packet Count, Byte Count
* 
*   Description: Gets statistics counters from the specified scheduler
*
*   PRETURN : None 
*************************************************************************/

int qos_get_scheduler_stats (uint32_t queue_grp_id, uint32_t queue_id,
			     unsigned long long* packet_count,
			     unsigned long long* byte_count)
{
    int         err = 0;
    
    QOS_DBG_TRACE ();
    QOS_DBG_PRINT (LOG_INFO, "Get Scheduler Statistics Parameters!\n\t"
		   "Queue Id    = %d \n\t"
		   "Queue Grp Id= %d \n",queue_id, queue_grp_id);

 
    PRETURN (err);
}

/**************************************************************************
*
*   Function: qos_get_queue_stats
*
*   Parameters: IN: Queue Group Id
*               IN: Queue Id
*               OUT:Packets dropped, Packets consumed
* 
*   Description: Gets statistics counters from the specified queue
*
*   PRETURN : None 
*************************************************************************/

int qos_get_queue_stats (uint32_t queue_grp_id, uint32_t queue_id,
			 unsigned long long* pkts_dropped,
			 unsigned long long* pkts_consumed)
{
    int            err = 0;
    
    QOS_DBG_TRACE ();
    QOS_DBG_PRINT (LOG_INFO, "Get Queue Statistics Parameters!\n\t"
		   "Queue Id    = %d \n\t"
		   "Queue Grp Id= %d \n",queue_id, queue_grp_id);

    PRETURN (err);
}

/**************************************************************************
*
*   Function: qos_get_dropper_stats
*
*   Parameters: IN: Flow Id
*               OUT:Packets dropped, Packets consumed
* 
*   Description: Gets statistics counters from the specified dropper
*
*   PRETURN : None 
*************************************************************************/

int qos_get_dropper_stats (uint32_t flow_id,
			   unsigned long long* pkts_dropped,
			   unsigned long long* pkts_consumed)
{
     int            err = 0;
    
    QOS_DBG_TRACE ();
    QOS_DBG_PRINT (LOG_INFO, "Get Dropper Statistics Parameters!\n\t"
		   "Flow Id    = %d \n", flow_id);

     PRETURN (err);
}

/**************************************************************************
*
*   Function: qos_get_meter_stats
*
*   Parameters: IN: Flow Id
*               OUT:Packet count, Byte count
* 
*   Description: Gets statistics counters from the specified meter
*
*   PRETURN : None 
*************************************************************************/

int qos_get_meter_stats (uint32_t flow_id,
			 unsigned long long* conf_pkt_count,
			 unsigned long long* conf_byte_count,
			 unsigned long long* nonconf_pkt_count,
			 unsigned long long* nonconf_byte_count)
{
    int            err = 0;
    
    QOS_DBG_TRACE ();
    QOS_DBG_PRINT (LOG_INFO, "Get Meter Statistics Parameters!\n\t"
		   "Flow Id    = %d \n", flow_id);

    PRETURN (err);
}
