/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/

#ifndef _SHIM_MGR_H
#define _SHIM_MGR_H

#ifdef __cplusplus
extern "C" {
#endif

#define MAX_NUM_PARAMS  10
#define MAX_NUM_RESP    10
#define MAX_RESPONSE_SIZE 3000
#define IX_ERROR        20

typedef enum
{
   SH_IPV4_CC = 0,
   SH_FPM_CC,
   SH_FPM_DATA_CC,
   SH_STKDRV_CC
} SH_COMPONENT;

typedef struct
{
   SH_COMPONENT    compName;   /* Indiactes the core component
                                 to which the request is directed */
   void*          userContext; /* Client context */
   void*          callback;    /* Client defined callback function */
   union                       /* The action identifies the core component
                                  call to be made */
   {
      SH_IPV4_ACTION  ipv4Action;
      SH_FPM_ACTION   fpmAction;
      SH_STKDRV_ACTION stkdrvAction; 
   } u;
   ix_error        requestStatus;  /* This error code will be set if an error
                                      is encountered in the request path */
   ix_error        responseStatus; /* This error code will be set if the 
                                      callback returns an error */
   void*           params[MAX_NUM_PARAMS]; /* Pointers to parameter
                                              structures */
} SH_Context_t;

typedef struct
{
   SH_COMPONENT    compName; /* Indicates the core component that
                                originated this response */
   void*           shContext; /* Client context pointer */
   ix_error   responseStatus; /* This error code will be set if the 
                                      callback returns an error */
   union                      /* This action identifies the core
                                 component call made */
   {
      SH_IPV4_ACTION  ipv4Action;
      SH_FPM_ACTION   fpmAction;
      SH_STKDRV_ACTION stkdrvAction; 
   } u;
} SH_ResponseMessageHeader;

typedef struct 
{
   SH_ResponseMessageHeader  respHeader; /* Header information */
   struct 
   {
      ix_uint32    length;       /* Number of bytes in the next field */
      void*        responseData; /* Callback data */    
   } outData[MAX_NUM_RESP];
} SH_GeneralResponseMessage;

typedef struct
{
   SH_COMPONENT    compName; /* Indicates the core component that
                                originated this response */
   void*           callback; /* Call back function to be called 
                                for the event */
   union                     /* This action identifies the core
                                component call made */
   {
      SH_IPV4_ACTION  ipv4Action;
      SH_FPM_ACTION   fpmAction;
      SH_STKDRV_ACTION stkdrvAction; 
   } u;
} SH_EventInfo_t;

#ifdef __cplusplus
}
#endif

#endif /*SHIM_MGR_H */

