/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ForwardingPlane/FPModule/Shim/src/ixasdk_binding.c,v $
 * $Revision: 1.3 $
 * $Date: 2003/12/01 10:35:25 $
 *
 * Description:
 *      Implements method to create event loops in new threads. The thread
 *      method is also contained here. The event threads wait for and
 *      processes messages from the CP and FP and then call the appropriate
 *      methods in the FP or CP.
 *
 ****************************************************************************/

#include "list.h"
#include "log.h"
/* #include "ixasdk_methodmap.h"
#include "ixasdk_stubs_pri.h" */
#include "cci/cc.h"
#include "ix_cc.h"
#include "cc/ix_cc_rtmv4.h"
#include "cc/ix_cc_ipv4.h"
#include "cc/ix_cc_stkdrv_common.h"
#include "cc/ix_cc_stkdrv_pktclass.h"
#include "cc/ix_cc_stkdrv.h"
#include "cc/ix_cc_fpm_core.h"

#include "shim_ipv4_mgr.h"
#include "shim_mgr.h"

extern ix_uint32 g_shim_fd;
/* extern QUEUE_SERVICE_PARAMS g_stubsParms; */

/**
 * NAME: ix_cc_stkdrv_async_get_num_ports
 *
 * DESCRIPTION: This function is called by the FP Module to get the number of
 * ports configured on the Stack Driver.
 * 
 * @Param:  - IN ix_cc_stkdrv_cb_get_num_ports arg_pCallback - pointer to the
 *          client's callback function to be invoked once the message returns.
 * @Param:  - IN void* arg_pContext - pointer to client context.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_OOM_SYSTEM
 */

ix_error ix_cc_stkdrv_async_get_num_ports(
    ix_cc_stkdrv_cb_get_num_ports arg_pCallback,
    void* arg_pContext)
{
   SH_Context_t*   shContext;  /* Pointer to Shim context */
   ix_error        error;      /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
       
   shContext->compName = SH_STKDRV_CC;
   shContext->userContext = arg_pContext;
   shContext->callback = (void*)arg_pCallback;
   shContext->u.stkdrvAction = SH_STKDRV_ASYNC_GET_NUMPORTS;

   if (write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n");    
      error = shContext->requestStatus;
      PIL_FreeMemory(shContext); 
      return error;
   }     
   return IX_SUCCESS;
}

/**
 * NAME: ix_cc_stkdrv_async_get_property
 *
 * DESCRIPTION: This function is called by the FP Module to get the interface
 *              properties of a specific port.
 * 
 * @Param:  - IN ix_cc_stkdrv_cb_get_property arg_pCallback - pointer to the
 *          client's callback function to be invoked once the message returns.
 * @Param:  - IN void* arg_pContext - pointer to client context.
 * @Param:  - IN ix_uint32 arg_propId - property IDs for the properties
 *          being requested.
 * @Param:  - IN ix_uint32 arg_portId - ID of port being queried. Use 
 *          IX_CC_GET_ALL_PORTS to get all port properties.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_OOM_SYSTEM
 */
ix_error ix_cc_stkdrv_async_get_property(
    ix_cc_stkdrv_cb_get_property arg_pCallback,
    void* arg_pContext,
    ix_uint32 arg_propId,
    ix_uint32 arg_portId)
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */ 
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
       
   shContext->compName = SH_STKDRV_CC;
   shContext->userContext = arg_pContext;
   shContext->callback = (void*)arg_pCallback;
   shContext->u.stkdrvAction = SH_STKDRV_ASYNC_GET_PROPERTY;
   shContext->params[0] = &arg_propId;
   shContext->params[1] = &arg_portId;
   if (write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n");    

      error = shContext->requestStatus;
      PIL_FreeMemory(shContext); 
      return error;
   }     
   return IX_SUCCESS;
}

/**
 * NAME: ix_cc_async_set_property
 *
 * DESCRIPTION: This function sends update messages (to the registered clients
 * who are interested in the respective property changes, for the properties
 * of which the Stack Driver is a master. If this function is called for
 * properties of which the Stack Driver is not a master, it will return an error.
 * Note that although this is an asynchronous function, no callback will be
 * provided to indicate success or error of the actual property updates.
 * 
 * @Param:  - IN ix_uint32 arg_PropId - id of the property to set.
 * @Param:  - IN ix_cc_properties* arg_pProperty - pointer to
 *          ix_cc_properties data structure.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_OOM_SYSTEM
 */
ix_error ix_cc_async_set_property(
				  ix_uint32 arg_PropId,
				  ix_cc_properties* arg_pProperty
				  )
{
   SH_Context_t*   shContext;
   ix_error        error;

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
   
   shContext->compName = SH_STKDRV_CC;
   shContext->userContext = NULL;
   shContext->callback = NULL;
   shContext->u.stkdrvAction = SH_STKDRV_ASYNC_SET_PROPERTY;
   shContext->params[0] = &arg_PropId;
   shContext->params[1] = arg_pProperty;
   if(write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n");    
   
   error = shContext->requestStatus;
   PIL_FreeMemory(shContext); 
   return error;
}

void
ix_error_dump(FILE * fp, ix_error err)
{
#ifdef SQESTUBS
    npf_logger_write(LOG_SQE_IXA_STUBS, LOG_TRACE,
        "%s
        \n\tfp : 0x%x
        \n\terr : %d
        \n",
        __FUNCTION__,
        fp,
           err
        );
#endif /* SQESTUBS */
    return;
}
