/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: 
 * $Revision: 1.3 $
 * $Date: 2003/12/01 10:35:26 $
 *
 * Description:
 *      Implements method to create event loops in new threads. The thread
 *      method is also contained here. The event threads wait for and
 *      processes messages from the CP and FP and then call the appropriate
 *      methods in the FP or CP.
 *
 ****************************************************************************/

#include "list.h"
#include "log.h"
/* #include "ixasdk_methodmap.h"
#include "ixasdk_stubs_pri.h" */
#include "cci/cc.h"
#include "ix_cc.h"
#include "cc/ix_cc_rtmv4.h"
#include "cc/ix_cc_ipv4.h"
#include "cc/ix_cc_stkdrv_common.h"
#include "cc/ix_cc_stkdrv_pktclass.h"
#include "cc/ix_cc_stkdrv.h"
#ifdef _INCLUDE_ATM
#include "cc/ix_cc_atm_pos_tx.h"
#endif
#ifdef _INCLUDE_DIFFSERV
#include "cc/ix_cc_pos_rx.h"
#endif
#include "cc/ix_cc_fpm_core.h"
#include "cc/ix_cc_eth_rx.h"

#include "shim_ipv4_mgr.h"
#include "shim_mgr.h"

#ifdef _LINUX_
#include "netinet/ip.h"
#endif

extern ix_uint32 g_shim_fd;
extern DList eventCallbackList;

/* extern QUEUE_SERVICE_PARAMS g_stubsParms; */

/**
 * NAME: ix_cc_fpm_register_link_status_cb
 *
 * DESCRIPTION: This message helper primitive helps a client to register
 *              interest in Link status update events.
 * 
 * @Param:  - IN arg_LinkState_cb
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_register_link_status_cb (ix_cc_fpm_link_status_cb 
					    arg_LinkState_cb)
{
   SH_Context_t*   shContext;     /* Pointer to Shim context */
   ix_error        error;         /* Error code */
   SH_EventInfo_t* pEvntListNode; /* Pointer to events list node */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }

   shContext->compName = SH_FPM_CC;
   shContext->userContext = NULL;
   shContext->callback = NULL;
   shContext->u.fpmAction = SH_FPM_REGISTER_LINKST;
   shContext->params[0] = arg_LinkState_cb;
   if(write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n"); 
   else
   {
      pEvntListNode = (SH_EventInfo_t*)PIL_AllocateMemory(sizeof(SH_EventInfo_t), PIL_MEM_ZERO_INIT);
      if (pEvntListNode == NULL)
      {
         npf_logger_write(LOG_SH, LOG_ERR,
            "Could not allocate memory for Shim context \n");    
         error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
         return error;
      }
      pEvntListNode->compName = SH_FPM_CC;
      pEvntListNode->callback = (void*)arg_LinkState_cb;
      pEvntListNode->u.fpmAction = SH_FPM_EVENT_LINKST; 
      npf_list_pushBack(&eventCallbackList, pEvntListNode);
   }
   error = shContext->requestStatus;
   PIL_FreeMemory(shContext); 
   return error;
}

/**
 * NAME: ix_cc_fpm_register_pkt_hdlr_cb
 *
 * DESCRIPTION: This message helper primitive helps a client to register
 *              interest in receiving packets
 * 
 * @Param:  - IN arg_pktHdlr_cb
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_register_pkt_hdlr_cb (ix_cc_fpm_pkt_hdlr_cb
					 arg_Packet_cb)
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }

   shContext->compName = SH_FPM_CC;
   shContext->userContext = NULL;
   shContext->callback = NULL;
   shContext->u.fpmAction = SH_FPM_REGISTER_PKTHDLR;
   shContext->params[0] = arg_Packet_cb;
   if(write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n"); 

   error = shContext->requestStatus;
   PIL_FreeMemory(shContext); 
   return error;
}

/**
 * NAME: ix_cc_fpm_register_ipv4_exception_cb
 *
 * DESCRIPTION: This message helper primitive helps a client to register
 *              interest in receiving IPv4 exceptions (Direct Host Route)
 * 
 * @Param:  - IN arg_ipv4Exception_cb
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_register_ipv4_exception_cb (ix_cc_fpm_ipv4_exception_cb
					       arg_ipv4Exception_cb)
{
   SH_Context_t*   shContext;     /* Pointer to Shim context */
   ix_error        error;         /* Error code */
   SH_EventInfo_t* pEvntListNode; /* Pointer to events list node */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }

   shContext->compName = SH_FPM_CC;
   shContext->userContext = NULL;
   shContext->callback = NULL;
   shContext->u.fpmAction = SH_FPM_REGISTER_IPV4EXCP;
   shContext->params[0] = arg_ipv4Exception_cb;
   if(write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n");
   {
      pEvntListNode = (SH_EventInfo_t*)PIL_AllocateMemory(sizeof(SH_EventInfo_t), PIL_MEM_ZERO_INIT);
      if (pEvntListNode == NULL)
      {
         npf_logger_write(LOG_SH, LOG_ERR,
            "Could not allocate memory for Shim context \n");    
         error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
         return error;
      }
      pEvntListNode->compName = SH_FPM_CC;
      pEvntListNode->callback = (void*)arg_ipv4Exception_cb;
      pEvntListNode->u.fpmAction = SH_FPM_EVENT_IPV4EXCP; 
      npf_list_pushBack(&eventCallbackList, pEvntListNode);
   } 
   
   error = shContext->requestStatus;
   PIL_FreeMemory(shContext); 
   return error;
}

/**
 * NAME: ix_cc_fpm_deregister_link_status_cb
 *
 * DESCRIPTION: This message helper primitive helps a client to deregister
 *              from receiving link status updates
 * 
 * @Param:  - IN - None
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_deregister_link_status_cb ()
{
   SH_Context_t*   shContext;    /* Pointer to Shim context */
   ix_error        error;        /* Error code */
   SH_EventInfo_t* pEvntListNode; /* Pointer to events list node */
   SH_ResponseMessageHeader  respMsgHdr;  /* Used as key for the events 
                                             callback list */ 

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
   shContext->compName = SH_FPM_CC;
   shContext->userContext = NULL;
   shContext->callback = NULL;
   shContext->u.fpmAction = SH_FPM_DEREGISTER_LINKST;
   respMsgHdr.compName = SH_FPM_CC;
   respMsgHdr.shContext = NULL;
   respMsgHdr.u.fpmAction = SH_FPM_EVENT_LINKST; 
   
   if(write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n"); 
   else
   {
      if(npf_list_removeByKey(&eventCallbackList, &respMsgHdr, 
         (void **)&pEvntListNode) != 0)
      {
         npf_logger_write(LOG_SH, LOG_ERR,
            "Error while removing node from events list\n");    
      }
      PIL_FreeMemory(pEvntListNode);   
   }
   error = shContext->requestStatus;
   PIL_FreeMemory(shContext); 
   return error;
}

/**
 * NAME: ix_cc_fpm_deregister_pktHdlr_cb
 *
 * DESCRIPTION: This message helper primitive helps a client to deregister
 *              from receiving packets
 * 
 * @Param:  - IN - None
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_deregister_pkt_hdlr_cb ()
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }

   shContext->compName = SH_FPM_CC;
   shContext->userContext = NULL;
   shContext->callback = NULL;
   shContext->u.fpmAction = SH_FPM_DEREGISTER_PKTHDLR;
   if(write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n"); 

   error = shContext->requestStatus;
   PIL_FreeMemory(shContext); 
   return error;
}

/**
 * NAME: ix_cc_fpm_deregister_ipv4_exception_cb
 *
 * DESCRIPTION: This message helper primitive helps a client to deregister
 *              from receiving IPv4 exceptions (Direct Host Route)
 * 
 * @Param:  - IN arg_ipv4Exception_cb
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_deregister_ipv4_exception_cb ()
{
   SH_Context_t*   shContext;     /* Pointer to Shim context */
   ix_error        error;         /* Error code */
   SH_EventInfo_t* pEvntListNode; /* Pointer to events list node */
   SH_ResponseMessageHeader  respMsgHdr;  /* Used as key for the events 
                                             callback list */ 

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }

   shContext->compName = SH_FPM_CC;
   shContext->userContext = NULL;
   shContext->callback = NULL;
   shContext->u.fpmAction = SH_FPM_DEREGISTER_IPV4EXCP;
   respMsgHdr.compName = SH_FPM_CC;
   respMsgHdr.shContext = NULL;
   respMsgHdr.u.fpmAction = SH_FPM_EVENT_IPV4EXCP; 

   if(write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n"); 
   else
   {
      if(npf_list_removeByKey(&eventCallbackList, &respMsgHdr, 
         (void **)&pEvntListNode) != 0)
      {
         npf_logger_write(LOG_SH, LOG_ERR,
            "Error while removing node from events list\n");    
      }
      PIL_FreeMemory(pEvntListNode);   
   }

   error = shContext->requestStatus;
   PIL_FreeMemory(shContext); 
   return error;
}

/**
 * NAME: ix_cc_fpm_sync_add_l2_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to add a L2 table 
 *              entry into the L2 Table maintained on the egress.
 *              This is a synchronous function that will block till the result
 *              is available or a timeout occurs.
 * 
 * @Param:  - IN arg_L2Id: L2 Id being added
 * @Param:  - IN arg_NextHopIp: IP address of the next hop 
 * @Param:  - IN arg_OutputPort: Outgoing port for the specified next hop
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_sync_add_l2_entry (
				      ix_uint32 arg_L2ID,
				      ix_uint32 arg_OutputPort,
				      ix_cc_l2tm_entry *arg_l2Info
				      )
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
   
   shContext->compName = SH_FPM_CC;
   shContext->userContext = NULL;
   shContext->callback = NULL;
   shContext->u.fpmAction = SH_FPM_ADD_L2ENTRY;
   shContext->params[0] = &arg_L2ID;
   shContext->params[1] = &arg_OutputPort;
   shContext->params[2] = arg_l2Info;
   if(write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n"); 

   error = shContext->requestStatus;
   PIL_FreeMemory(shContext); 
   return error;
}

/**
 * NAME: ix_cc_fpm_sync_del_l2_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to del a L2 table 
 *              entry from the L2 Table maintained on the egress.
 *              This is a synchronous function that will block till the result
 *              is available or a timeout occurs.
 * 
 * @Param:  - IN arg_L2Id: L2 Id being removed
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_sync_del_l2_entry (
                      ix_uint32 arg_L2Id
                      )
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
   
       
   shContext->compName = SH_FPM_CC;
   shContext->userContext = NULL;
   shContext->callback = NULL;
   shContext->u.fpmAction = SH_FPM_DEL_L2ENTRY;
   shContext->params[0] = &arg_L2Id;
   if(write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n"); 

   error = shContext->requestStatus;
   PIL_FreeMemory(shContext); 
   return error;
}

/**
 * NAME: ix_cc_fpm_async_add_l2_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to add a L2
 *              entry into the L2 table maintained on the egress.
 * 
 * @Param:  - IN arg_L2Id: L2 Id being added
 * @Param:  - IN arg_OutputPort: Outgoing port for the specified next hop
 * @Param:  - IN arg_L2Info: ix_cc_l2tm_entry structure for L2 Table
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_async_add_l2_entry (
				       ix_uint32 arg_L2ID,
				       ix_uint32 arg_OutputPort,
				       ix_cc_l2tm_entry *arg_l2Info,
				       ix_cc_fpm_l2_op_cb arg_Callback,
				       void* arg_pUserContext
				       )
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
       
   shContext->compName = SH_FPM_CC;
   shContext->userContext = arg_pUserContext;
   shContext->callback = arg_Callback;
   shContext->u.fpmAction = SH_FPM_ASYNC_ADD_L2ENTRY;
   shContext->params[0] = &arg_L2ID;
   shContext->params[1] = &arg_OutputPort;
   shContext->params[2] = arg_l2Info;

   if (write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n");  
      error = shContext->requestStatus;
      PIL_FreeMemory(shContext); 
      return error;
   }
   return IX_SUCCESS;
}


/**
 * NAME: ix_cc_fpm_async_del_l2_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to delete a L2
 *              entry from the L2 table maintained on the egress.
 * 
 * @Param:  - IN arg_L2Id: L2 Id being deleted
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_async_del_l2_entry (
				       ix_uint32 arg_L2ID,
				       ix_cc_fpm_l2_op_cb arg_Callback,
				       void* arg_pUserContext
				       )
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
   
       
   shContext->compName = SH_FPM_CC;
   shContext->userContext = arg_pUserContext;
   shContext->callback = arg_Callback;
   shContext->u.fpmAction = SH_FPM_ASYNC_DEL_L2ENTRY;
   shContext->params[0] = &arg_L2ID;

   if (write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n");  
      error = shContext->requestStatus;
      PIL_FreeMemory(shContext); 
      return error;
   }
   return IX_SUCCESS;
}

/**
 * NAME: ix_cc_fpm_async_add_arp_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to add a ARP
 *              entry into the ARP table maintained on the egress.
 * 
 * @Param:  - IN arg_L2Id: L2 Id being added
 * @Param:  - IN arg_NextHopIp: IP address of the next hop 
 * @Param:  - IN arg_OutputPort: Outgoing port for the specified next hop
 * @Param:  - IN arg_L2Address: ARP entry being added (L2 Address)
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_async_add_arp_entry (
					ix_cc_eth_tx_next_hop_info* arg_ArpInfo,
					ix_cc_fpm_arp_op_cb arg_Callback,
					void* arg_pUserContext
					)
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
       
   shContext->compName = SH_FPM_CC;
   shContext->userContext = arg_pUserContext;
   shContext->callback = arg_Callback;
   shContext->u.fpmAction = SH_FPM_ASYNC_ADD_ARPENTRY;
   shContext->params[0] = arg_ArpInfo;

   if (write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n");  
      error = shContext->requestStatus;
      PIL_FreeMemory(shContext); 
      return error;
   }
   return IX_SUCCESS;
}

/**
 * NAME: ix_cc_fpm_async_del_arp_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to add a ARP
 *              entry into the ARP table maintained on the egress.
 * 
 * @Param:  - IN arg_L2Id: L2 Id being deleted
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_async_del_arp_entry (
					ix_uint32 arg_L2Id,
					ix_cc_fpm_arp_op_cb arg_Callback,
					void* arg_pUserContext
					)
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
       
   shContext->compName = SH_FPM_CC;
   shContext->userContext = arg_pUserContext;
   shContext->callback = arg_Callback;
   shContext->u.fpmAction = SH_FPM_ASYNC_DEL_ARPENTRY;
   shContext->params[0] = &arg_L2Id;

   if (write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n");  
      error = shContext->requestStatus;
      PIL_FreeMemory(shContext); 
      return error;
   }
   return IX_SUCCESS;
}

/**
 * NAME: ix_cc_fpm_async_purge_arp_table
 *
 * DESCRIPTION: This message helper primitive helps a client to purge the ARP
 *              table maintained on the egress.
 * 
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_sync_purge_arp_table ()
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
       
   shContext->compName = SH_FPM_CC;
   shContext->userContext = NULL;
   shContext->callback = NULL;
   shContext->u.fpmAction = SH_FPM_FLUSH_ARP;

   if(write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n");  
   error = shContext->requestStatus;
   PIL_FreeMemory(shContext); 
   return error;
}

#ifdef _INCLUDE_ATM
/**
 * NAME: ix_cc_fpm_async_get_atm_pos_tx_stats
 *
 * DESCRIPTION: This message helper primitive helps a client to retrieve
 *              L2 (ATM/POS) transmit statistics maintained on the egress.
 * 
 * @Param:  - IN arg_PortId: port to get statistics for
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_async_get_pos_tx_stats (
    ix_cc_atm_pos_tx_statistics_info_context* arg_pStatsInfo,
    ix_cc_fpm_atm_pos_tx_stats_op_cb arg_Callback)
{
   /* This function is not implemented */
   return IX_ERROR;
}
#endif /* _INCLUDE_ATM */

ix_error ix_cc_fpm_sync_send_packet (
    ix_uint32 arg_PortID,
    char*     arg_pBuffer,
    ix_uint32 arg_PacketSize
    )
{
   /* This function is not implemented */
   return IX_ERROR;
}

/**
 * NAME: ix_cc_fpm_async_get_eth_tx_stats
 *
 * DESCRIPTION: This message helper primitive helps a client to retrieve
 *              L2 (ETH) transmit statistics maintained on the egress.
 * 
 * @Param:  - IN arg_PortId: port to get statistics for
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */
ix_error ix_cc_fpm_async_get_eth_tx_stats (
    ix_cc_eth_tx_statistics_info_context* arg_pStatsInfo,
    ix_cc_fpm_eth_tx_stats_op_cb arg_Callback)
{
   /* This function is not implemented */
   return IX_ERROR;
}

/**
 * NAME: ix_cc_fpm_async_get_eth_rx_stats
 *
 * DESCRIPTION: This message helper primitive helps a client to retrieve
 *              L2 (ETH) transmit statistics maintained on the egress.
 * 
 * @Param:  - IN arg_PortId: port to get statistics for
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */
ix_error ix_cc_eth_rx_async_get_statistics_info (
    ix_cc_eth_rx_statistics_info_context* arg_pStatsInfo,
    ix_cc_eth_rx_cb_get_statistics_info arg_Callback)
{
   /* This function is not implemented */
   return IX_ERROR;
}

ix_error fpm_ix_cc_eth_rx_async_get_statistics_info (
    ix_cc_eth_rx_statistics_info_context* arg_pStatsInfo,
    ix_cc_eth_rx_cb_get_statistics_info arg_Callback)
{
   /* This function is not implemented */
   return IX_ERROR;
}

#ifdef _INCLUDE_DIFFSERV
/**
 * NAME: ix_cc_pos_rx_async_get_statistics_info
 *
 * DESCRIPTION: This message helper primitive helps a client to retrieve
 *              L2 (POS) receive statistics.
 * 
 * @Param:  - IN arg_PortId: port to get statistics for
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */
ix_error ix_cc_pos_rx_async_get_statistics_info(
    ix_cc_pos_rx_statistics_info_context *arg_pMgInfoContext,
    ix_cc_pos_rx_cb_get_statistics_info arg_Callback)
{
   /* This function is not implemented */
   return IX_ERROR;
}
#endif
