/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ForwardingPlane/FPModule/Shim/src/ixasdk_ipv4.c,v $
 * $Revision: 1.2 $
 * $Date: 2003/11/17 16:57:27 $
 *
 * Description:
 *      Implements method to create event loops in new threads. The thread
 *      method is also contained here. The event threads wait for and
 *      processes messages from the CP and FP and then call the appropriate
 *      methods in the FP or CP.
 *
 ****************************************************************************/

#include <time.h>
#include "list.h"
#include "log.h"
/* #include "ixasdk_methodmap.h"
#include "ixasdk_stubs_pri.h" */

#include "cci/cc.h"
#include "ix_cc.h"
#include "cc/ix_cc_rtmv4.h"
#include "cc/ix_cc_ipv4.h"
#include "cc/ix_cc_fpm_core.h"
#include "ix_cc_ipv4_init.h"

#include "shim_ipv4_mgr.h"
#include "shim_mgr.h"

/* extern QUEUE_SERVICE_PARAMS g_stubsParms; */
extern ix_uint32 g_shim_fd;
ix_cc_ipv4_context* g_pIpv4Context;

/**
 * NAME: ix_cc_ipv4_async_add_route
 *
 * DESCRIPTION: This message helper primitive helps client to add a route in RTM. 
 * Client needs to successfully add nextHopId to RTM by calling ix_cc_ipv4_async_add_next_hop
 * messaging helper primitive before sending this message to IPv4 Forwarder core component. 
 * 
 * @Param:  - IN arg_IpAddr: network IP address identifying the route.
 * @Param:  - IN arg_NetMask: network mask for the route.
 * @Param:  - IN arg_NextHopId: next hop identifier for route.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOM - memory allocation failure
 * 
 */


ix_error ix_cc_ipv4_async_add_route(ix_uint32 arg_IpAddr,
                                    ix_uint32 arg_NetMask,
                                    ix_cc_rtmv4_nhid arg_NextHopId,
                                    ix_cc_ipv4_cb_route_op arg_Callback,
                                    void* arg_pUserContext)
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
 
   shContext->compName = SH_IPV4_CC;
   shContext->userContext = arg_pUserContext;
   shContext->callback = (void*)arg_Callback;
   shContext->u.ipv4Action = SH_IPV4_ASYNC_ADD_ROUTE;
   shContext->params[0] = &arg_IpAddr;
   shContext->params[1] = &arg_NetMask;
   shContext->params[2] = &arg_NextHopId;
   if (write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n"); 
      error = shContext->requestStatus;
      PIL_FreeMemory(shContext); 
      return error;
   }     
   return IX_SUCCESS;
}

/**
 * NAME: ix_cc_ipv4_async_delete_route
 *
 * DESCRIPTION:This message helper primitive helps a client to delete a route in RTM. 
 * Client needs to delete all routes associate with a next hop to delete the next hop
 * by calling ix_cc_ipv4_async_delete_next_hop message helper primitive. 
 * 
 * @Param:  - IN arg_IpAddr: network IP address identifying the route.
 * @Param:  - IN arg_NetMask: network mask for the route.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_OOM - memory allocation failure
 * 
 */


ix_error ix_cc_ipv4_async_delete_route(ix_uint32 arg_IpAddr,
                                       ix_uint32 arg_NetMask,
                                       ix_cc_ipv4_cb_route_op arg_Callback,
                                       void* arg_pUserContext
                                      )
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
       
   shContext->compName = SH_IPV4_CC;
   shContext->userContext = arg_pUserContext;
   shContext->callback = (void*)arg_Callback;
   shContext->u.ipv4Action = SH_IPV4_ASYNC_DEL_ROUTE;
   shContext->params[0] = &arg_IpAddr;
   shContext->params[1] = &arg_NetMask;
   if (write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n"); 

      error = shContext->requestStatus;
      PIL_FreeMemory(shContext); 
      return error;
   }     
   return IX_SUCCESS;
}

/**
 * NAME: ix_cc_ipv4_async_update_route
 *
 * DESCRIPTION: This message helper primitive helps client to update a route in RTM. 
 * Client needs to successfully add nextHopId to RTM by calling ix_cc_ipv4_async_add_next_hop
 * messaging helper primitive before sending this message to IPv4 Forwarder core component. 
 * 
 * @Param:  - IN arg_IpAddr: network IP address identifying the route.
 * @Param:  - IN arg_NetMask: network mask for the route.
 * @Param:  - IN arg_NextHopId: next hop identifier for the route.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_ipv4_async_update_route(ix_uint32 arg_IpAddr,
                                    ix_uint32 arg_NetMask,
                                    ix_cc_rtmv4_nhid arg_NextHopId,
                                    ix_cc_ipv4_cb_route_op arg_Callback,
                                    void* arg_pUserContext
                                   )
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
   
   shContext->compName = SH_IPV4_CC;
   shContext->userContext = arg_pUserContext;
   shContext->callback = (void*)arg_Callback;
   shContext->u.ipv4Action = SH_IPV4_ASYNC_UPDATE_ROUTE;
   shContext->params[0] = &arg_IpAddr;
   shContext->params[1] = &arg_NetMask;
   shContext->params[2] = &arg_NextHopId;

   if (write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n"); 

      error = shContext->requestStatus;
      PIL_FreeMemory(shContext); 
      return error;
   }     
   return IX_SUCCESS;
}

/**
 * NAME: ix_cc_ipv4_async_lookup_route
 *
 * DESCRIPTION: This primitive helps a client to look up routing information 
 * for a given IP address. If there is a successful match, then the information will be 
 * given to client are nextHopId (including the blade id), portId, mtu and flags. 
 * If there is no match, then IX_CC_RTMV4_NHID_NO_ROUTE will be assigned to nextHopId
 * field of pNextHopData and all other information are invalid.
 * 
 * @Param:  - IN arg_IpAddr: network IP address identifying the route.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT -none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_OOM - memory allocation failure
 */

ix_error ix_cc_ipv4_async_lookup_route(ix_uint32 arg_IpAddr,
                                       ix_cc_ipv4_cb_lookup_route arg_Callback,
                                       void* arg_pUserContext
                                      )
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
   
   shContext->compName = SH_IPV4_CC;
   shContext->userContext = arg_pUserContext;
   shContext->callback = (void*)arg_Callback;
   shContext->u.ipv4Action = SH_IPV4_ASYNC_LOOKUP_ROUTE;
   shContext->params[0] = &arg_IpAddr;

   if (write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n"); 

      error = shContext->requestStatus;
      PIL_FreeMemory(shContext); 
      return error;
   }     
   return IX_SUCCESS;
}

/**
 * NAME: ix_cc_ipv4_async_purge_routes
 *
 * DESCRIPTION: This primitive helps a client to remove all routes from RTM.
 * 
 * @Param:  - IN arg_CcHandle - 
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_OOM - memory allocation failure
 */

ix_error ix_cc_ipv4_async_purge_routes()
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
       
   shContext->compName = SH_IPV4_CC;
   shContext->userContext = NULL;
   shContext->callback = NULL;
   shContext->u.ipv4Action = SH_IPV4_ASYNC_FLUSH_PREFIX;

   if(write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n"); 

   error = shContext->requestStatus;
   PIL_FreeMemory(shContext); 
   return error;
}

/**
 * NAME: ix_cc_ipv4_async_dump_routes
 *
 * DESCRIPTION: This primitive helps to dump all routes of RTM in memory
 * 
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_IPV4_ERROR_MSG_LIBRARY- failure from Message Support Library 
 * 
 */

ix_error ix_cc_ipv4_async_dump_routes(ix_cc_ipv4_cb_dump_data arg_Callback,
                                      void* arg_pUserContext
                                     )
{
   /* This API is not implemented */
   return IX_ERROR;
}

/**
 * NAME: ix_cc_ipv4_async_add_next_hop
 *
 * DESCRIPTION: This primitive helps a client to add next hop information to RTM database. 
 * Client needs to add next hop to RTM in order to add routes referring to that next hop.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop to be added
 * @Param:  - IN arg_pNextHopData: pointer to next hop information
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined contex
 * 
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_ipv4_async_add_next_hop(ix_cc_rtmv4_nhid arg_NextHopId,
                                       ix_cc_rtmv4_next_hop_info* arg_pNextHopData,
                                       ix_cc_ipv4_cb_route_op arg_Callback,
                                       void* arg_pUserContext
                                      )
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
       
   shContext->compName = SH_IPV4_CC;
   shContext->userContext = arg_pUserContext;
   shContext->callback = (void*)arg_Callback;
   shContext->u.ipv4Action = SH_IPV4_ASYNC_ADD_NHOP;
   shContext->params[0] = &arg_NextHopId;
   shContext->params[1] = arg_pNextHopData;

   if (write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n"); 

      error = shContext->requestStatus;
      PIL_FreeMemory(shContext); 
      return error;
   }     
   return IX_SUCCESS;
}

/**
 * NAME: ix_cc_ipv4_async_delete_next_hop
 *
 * DESCRIPTION: This message helper primitive helps a client to delete next hop information 
 * from RTM database. Client needs to remove all routes associated with next hop before 
 * calling this primitive. If not, client gets an error.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop to be removed.
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_OOM - memory allocation failure
 */


ix_error ix_cc_ipv4_async_delete_next_hop(ix_cc_rtmv4_nhid arg_NextHopId,
                                          ix_cc_ipv4_cb_route_op arg_Callback,
                                          void* arg_pUserContext
                                          )
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
   
       
   shContext->compName = SH_IPV4_CC;
   shContext->userContext = arg_pUserContext;
   shContext->callback = (void*)arg_Callback;
   shContext->u.ipv4Action = SH_IPV4_ASYNC_DEL_NHOP;
   shContext->params[0] = &arg_NextHopId;

   if (write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n"); 

      error = shContext->requestStatus;
      PIL_FreeMemory(shContext); 
      return error;
   }     
   return IX_SUCCESS;
}

/**
 * NAME: ix_cc_ipv4_async_update_next_hop
 *
 * DESCRIPTION: This primitive helps a client to update next hop information to RTM database. 
 * Client needs to add next hop to RTM in order to add routes referring to that next hop.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop to be updated
 * @Param:  - IN arg_pNextHopData: pointer to next hop information
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined contex
 * 
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_ipv4_async_update_next_hop(ix_cc_rtmv4_nhid arg_NextHopId,
                                       ix_cc_rtmv4_next_hop_info* arg_pNextHopData,
                                       ix_cc_ipv4_cb_route_op arg_Callback,
                                       void* arg_pUserContext
                                      )
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
       
   shContext->compName = SH_IPV4_CC;
   shContext->userContext = arg_pUserContext;
   shContext->callback = (void*)arg_Callback;
   shContext->u.ipv4Action = SH_IPV4_ASYNC_UPDATE_NHOP;
   shContext->params[0] = &arg_NextHopId;
   shContext->params[1] = arg_pNextHopData;

   if (write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n"); 

      error = shContext->requestStatus;
      PIL_FreeMemory(shContext); 
      return error;
   }     
   return IX_SUCCESS;
}

/**
 * NAME: ix_cc_ipv4_async_get_next_hop
 *
 * DESCRIPTION: This message helper primitive helps a client to retrieve 
 * next hop information from RTM database. 
 * 
 * @Param:  - IN arg_NextHopId: Identifier of the next hop information to be retrieved from RTM.
 * @Param:  - IN arg_Callback: pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_OOM - memory allocation failure
 */

ix_error ix_cc_ipv4_async_get_next_hop(ix_cc_rtmv4_nhid arg_NextHopId,
                                       ix_cc_ipv4_cb_get_next_hop arg_Callback,
                                       void* arg_pUserContext
                                      )
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
       
   shContext->compName = SH_IPV4_CC;
   shContext->userContext = arg_pUserContext;
   shContext->callback = (void*)arg_Callback;
   shContext->u.ipv4Action = SH_IPV4_ASYNC_GET_NHOP;
   shContext->params[0] = &arg_NextHopId;

   if (write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n"); 

      error = shContext->requestStatus;
      PIL_FreeMemory(shContext); 
      return error;
   }     
   return IX_SUCCESS;
}

/**
 * NAME: ix_cc_ipv4_async_purge_rtm
 *
 * DESCRIPTION: This primitive helps a client to removes all routes and next hops from RTM database
 * 
 * @Param:  - IN  - none
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_ipv4_async_purge_rtm()
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
   
   shContext->compName = SH_IPV4_CC;
   shContext->userContext = NULL;
   shContext->callback = NULL;
   shContext->u.ipv4Action = SH_IPV4_ASYNC_FLUSH_NHOP;

   if(write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n"); 

   error = shContext->requestStatus;
   PIL_FreeMemory(shContext); 
   return error;
}

/**
 * NAME: ix_cc_ipv4_async_dump_next_hops
 *
 * DESCRIPTION:   This primitive helps to dump all next hops of RTM in memory.
 * 
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 */

ix_error ix_cc_ipv4_async_dump_next_hops(ix_cc_ipv4_cb_dump_data arg_Callback,
                                         void* arg_pUserContext
                                        )
{
   /* This API is not implemented */
   return IX_ERROR;
}

/**
 * NAME: ix_cc_ipv4_async_set_mtu
 *
 * DESCRIPTION:This primitive helps client to update MTU for a given next hop. 
 * Before calling this message helper, client needs to call ix_cc_ipv4_async_add_next_hop to 
 * add next hop identifier in RTM database. The validity of MTU shall be done by the client.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop.
 * @Param:  - IN arg_Mtu: new maximum transmission unit for the given next hop.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 */


ix_error ix_cc_ipv4_async_set_mtu(ix_cc_rtmv4_nhid arg_NextHopId,
                                  ix_uint32 arg_Mtu,
                                  ix_cc_ipv4_cb_route_op arg_Callback,
                                  void* arg_pUserContext
                                 )
{
   /* This API is not implemented */
   return IX_ERROR;
}

/**
 * NAME: ix_cc_ipv4_async_set_flags
 *
 * DESCRIPTION:This primitive helps client to update flags for a given next hop.
 * Before calling this message helper, client needs to call ix_cc_ipv4_async_add_next_hop to add next hop 
 * identifier in RTM database. The supported flags[BLDBLK] are defined by IPv4 Forwarder microblock.
 * 
 * @Param:  - IN arg_NextHopId: identifier of next hop.
 * @Param:  - IN arg_Flags: new flags for the given next hop 
 * @Param:  - IN arg_pUserContext: pointer to client defined context
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * 
 */
ix_error ix_cc_ipv4_async_set_flags(ix_cc_rtmv4_nhid arg_NextHopId,
                                    ix_uint32 arg_Flags,
                                    ix_cc_ipv4_cb_route_op arg_Callback,
                                    void* arg_pUserContext
                                   )
{
   /* This API is not implemented */
   return IX_ERROR;
}

/**
 * NAME: ix_cc_ipv4_async_get_sleep_time
 *
 * DESCRIPTION:This message helper primitive helps client to retrieve the number of seconds 
 * between when the event will fire causing the ICMP messages to dequeue and send. 
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * 
 */

ix_error ix_cc_ipv4_async_get_sleep_time(ix_cc_ipv4_cb_get_sleep_time arg_Callback,
                                         void* arg_pUserContext
                                        )
{
   /* This API is not implemented */
   return IX_ERROR;
}

/**
 * NAME: ix_cc_ipv4_async_set_sleep_time
 *
 * DESCRIPTION: This message helper primitive helps client to set the number of seconds between 
 * when the event will fire causing the ICMP messages to dequeue and send.  The lower the value of arg_SleepTime, 
 * the more frequent the ICMP messages will be dequeued and sent.
 * 
 * @Param:  - IN arg_SleepTime: minimum value is 1 second and maximum value is 32 seconds.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * 
 */

ix_error ix_cc_ipv4_async_set_sleep_time(ix_uint16 arg_SleepTime)
{
   /* This API is not implemented */
   return IX_ERROR;
}

/**
 * NAME: ix_cc_ipv4_async_get_queue_depth
 *
 * DESCRIPTION:This message helper primitive helps client to retrieves the depth of the ICMP error message queue. 
 * Client retrieves the maximum number of ICMP messages that can fit in the queue, not the number of messages
 * currently in the queue.
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * 
 */
ix_error ix_cc_ipv4_async_get_queue_depth(ix_cc_ipv4_cb_get_queue_depth arg_Callback,
                                          void* arg_pUserContext
                                         )
{
   /* This API is not implemented */
   return IX_ERROR;
}

/**
 * NAME: ix_cc_ipv4_async_get_packets_to_drain
 *
 * DESCRIPTION:This message helper primitive helps client to retrieve the maximum number of ICMP error messages 
 * to send each time the queue processing event fires. The processing event will send up to this many ICMP error
 * packets from the queue before going back to sleep. 
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * 
 */
ix_error ix_cc_ipv4_async_get_packets_to_drain(ix_cc_ipv4_cb_get_packets_to_drain arg_Callback,
                                               void* arg_pUserContext
                                              )
{
   /* This API is not implemented */
   return IX_ERROR;
}

/**
 * NAME: ix_cc_ipv4_async_set_packets_to_drain
 *
 * DESCRIPTION:This message helper primitive helps client to set the maximum number of ICMP error messages to 
 * send each time the queue processing event fires. The value to arg_PacketsToDrain must not exceed the depth of 
 * the queue.
 * 
 * @Param:  - IN arg_PacketsToDrain: The maximum number of packets to send each time the queue processing event fires.
 *
 * @Param:  - OUT
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * 
 */
ix_error ix_cc_ipv4_async_set_packets_to_drain(ix_uint16 arg_PacketsToDrain)
{
   /* This API is not implemented */
   return IX_ERROR;
}

/**
 * NAME: ix_cc_ipv4_async_get_statistics
 *
 * DESCRIPTION:This message helper primitive helps client to retrieve statistics report from IPv4 Forwarder core component.
 * This primitive will give the statistics from IPv4 microblock, RTM and IPV4 Forwarder core component.  
 * 
 * @Param:  - IN arg_Callback: pointer to client provided callback.
 * @Param:  - IN arg_pUserContext: pointer to client defined context. 
 *
 * @Param:  - OUT none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 */

ix_error ix_cc_ipv4_async_get_statistics(ix_cc_ipv4_cb_get_statistics arg_Callback,
                                         void* arg_pUserContext
                                        )
{
   /* This API is not implemented */
   return IX_ERROR;
}

void sh_ipv4_init()
{
   g_pIpv4Context = (ix_cc_ipv4_context *)PIL_AllocateMemory(sizeof(ix_cc_ipv4_context), PIL_MEM_ZERO_INIT);    
}

void sh_ipv4_finish()
{
   PIL_FreeMemory(g_pIpv4Context); 
}

/**
 * NAME: ix_cc_rtmv4_get_next_hop
 *
 * DESCRIPTION: 
 * 
 * @Param:  - IN arg_hRtm: Handle to RTM table
 *          - IN arg_NextHopId: Next hop identifier to be queried
 *
 * @Param:  - OUT arg_pNextHopInfo: Next hop information
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_OOM - memory allocation failure
 * 
 */
 
ix_error ix_cc_rtmv4_get_next_hop (
         ix_cc_rtmv4 arg_hRtm,
         ix_cc_rtmv4_nhid arg_NextHopId,
         ix_cc_rtmv4_next_hop_info *arg_pNextHopInfo)
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
       
   shContext->compName = SH_IPV4_CC;
   shContext->userContext = NULL;
   shContext->callback = NULL;
   shContext->u.ipv4Action = SH_IPV4_GET_NHOP;
   shContext->params[0] = &arg_hRtm;
   shContext->params[1] = &arg_NextHopId;
   shContext->params[2] = arg_pNextHopInfo;

   if (write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n"); 

      error = shContext->requestStatus;
      PIL_FreeMemory(shContext); 
      return error;
   }     
   return IX_SUCCESS;
}
  
/**
 * NAME: ix_cc_rtmv4_get_route
 *
 * DESCRIPTION: 
 * 
 * @Param:  - IN arg_hRtm: Handle to RTM table
 *          - IN arg_NetAddr: Network address
 *          - IN arg_NetMask: Network mask
 *
 * @Param:  - OUT arg_pNextHopId: Next hop identifier
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *
 * IX_CC_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_rtmv4_get_route (
         ix_cc_rtmv4 arg_hRtm,
         ix_uint32 arg_NetAddr,
         ix_uint32 arg_NetMask,
         ix_cc_rtmv4_nhid *arg_pNextHopId)
{
   SH_Context_t*   shContext;   /* Pointer to Shim context */
   ix_error        error;       /* Error code */

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if(shContext == NULL)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Could not allocate memory for Shim context \n");    
      error = IX_ERROR_NEW(IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
      return error;
   }
       
   shContext->compName = SH_IPV4_CC;
   shContext->userContext = NULL;
   shContext->callback = NULL;
   shContext->u.ipv4Action = SH_IPV4_GET_ROUTE;
   shContext->params[0] = &arg_hRtm;
   shContext->params[1] = &arg_NetAddr;
   shContext->params[2] = &arg_NetMask;
   shContext->params[3] = arg_pNextHopId;

   if (write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
   {
      npf_logger_write(LOG_SH, LOG_ERR,
         "Error while writing into Shim device driver \n"); 

      error = shContext->requestStatus;
      PIL_FreeMemory(shContext); 
      return error;
   }     
   return IX_SUCCESS;
} 
