/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: 
 * $Revision: 1.2 $
 * $Date: 2003/11/17 16:57:27 $
 *
 * Description:
 *
 ****************************************************************************/
#include <time.h>
#include <fcntl.h>
#include "list.h"
#include "log.h"
/* #include "ixasdk_methodmap.h"
#include "ixasdk_stubs_pri.h" */

#include "cci/cc.h"
#include "ix_cc.h"
#include "cc/ix_cc_rtmv4.h"
#include "cc/ix_cc_ipv4.h"
#include "cc/ix_cc_fpm_core.h"
#include "ix_cc_ipv4_init.h"
#include "cc/ix_cc_stkdrv.h"

#include "shim_ipv4_mgr.h"
#include "shim_mgr.h"

int32_t             g_shim_fd;
PilThread           g_sh_RdThread;
static uint32_t     shInitialized = 0;
DList               eventCallbackList;              

/*****************************************************************************
*    Name: CompareEventList
*
*    Description:
*        Compare function for the list that maintains callback functions
*        for events.
*
*    Parameters:
*        Data and the key.
*
*    Preconditions:
*        None.
*
*    Postconditions:
*        None.        
*
*    Returns:
*        TRUE if a match is found and FALSE if no match is found. 
*
*/

int sh_CompareEventList (const void* data,
    const void* key)
{
   SH_ResponseMessageHeader*  pKey;
   SH_EventInfo_t*            pData;

   pData = (SH_EventInfo_t*)data;
   pKey = (SH_ResponseMessageHeader*)key;

   if (pData->compName == pKey->compName)
   {
      switch(pKey->compName)
      {
         case SH_IPV4_CC:
            return (pData->u.ipv4Action == pKey->u.ipv4Action);
            break;

         case SH_FPM_CC:
            return (pData->u.fpmAction == pKey->u.fpmAction);
            break;

         case SH_STKDRV_CC:
            return (pData->u.stkdrvAction == pKey->u.stkdrvAction);
            break;

         case SH_FPM_DATA_CC:
            break;
      }
   }
   else
      return (pData->compName == pKey->compName);      
}

/*****************************************************************************
*    Name: sh_reader_thread 
*
*    Description:
*        This thread continuosly polls for any message from the character
*        device. If there is no message available it blocks on a semaphore.
*        Whenever a message is available, this thread decodes it and then
*        calls the respective client registered function.
*
*    Parameters:
*        File descriptor on which to read. 
*
*    Preconditions:
*        None.
*
*    Postconditions:
*        None.        
*
*    Returns:
*        None.
*
*/

uint32_t sh_reader_thread(void* a)
{
   char           buf[MAX_RESPONSE_SIZE];   /* Buffer into which the 
                                               response or event message 
                                               is read */
   int32_t        fd = *((int32_t*)a);      /* The file descriptor for the
                                               Shim character device */
   size_t         len;                      /* Length of the message read */
   SH_ResponseMessageHeader* respHdr;         /* Pointer to the common message
                                               structure */
   SH_Context_t*  pContext;                /* Pointer to the shim context */
   DListItr*      itr;                     /* List iterator */
   SH_EventInfo_t* eventListNode;
   uint32_t       found = FALSE;           /* Keeps track of whether a match
                                              is found in the events list
                                              or not */ 
   char*          pos;                     /* Pointer that keeps of the 
                                              next data to be read from
                                              buffer */
   while (1)
   {
      len = read(fd, buf, MAX_RESPONSE_SIZE);
      if (len == 0)
      {
         continue;
      } 
      respHdr = (SH_ResponseMessageHeader*)buf;
      pContext = (SH_Context_t*)(respHdr->shContext);
      pos = buf + sizeof(SH_ResponseMessageHeader);
      switch(respHdr->compName)
      {
         case SH_IPV4_CC:
            switch(respHdr->u.ipv4Action) 
            {
               case SH_IPV4_ASYNC_ADD_ROUTE:
               case SH_IPV4_ASYNC_DEL_ROUTE:
               case SH_IPV4_ASYNC_UPDATE_ROUTE:
               case SH_IPV4_ASYNC_ADD_NHOP:
               case SH_IPV4_ASYNC_DEL_NHOP:
               case SH_IPV4_ASYNC_UPDATE_NHOP:
               {
                 (*((ix_cc_ipv4_cb_route_op)(pContext->callback)))(
                        pContext->responseStatus,
                        pContext->userContext); 
                 PIL_FreeMemory(pContext);
               }
               break;

               case SH_IPV4_ASYNC_LOOKUP_ROUTE:
               {
                  ix_cc_rtmv4_next_hop_info* next_hop_info;

                  pos = pos + sizeof(ix_uint32);
                  next_hop_info = (ix_cc_rtmv4_next_hop_info*)pos;    
                  (*((ix_cc_ipv4_cb_lookup_route)(pContext->callback)))(
                        pContext->responseStatus,
                        pContext->userContext,
                        next_hop_info); 
                  PIL_FreeMemory(pContext);
               }
               break;

               case SH_IPV4_ASYNC_GET_NHOP:
               {
                  ix_cc_rtmv4_next_hop_info* next_hop_info;

                  pos = pos + sizeof(ix_uint32);
                  next_hop_info = (ix_cc_rtmv4_next_hop_info*)pos;    
                  (*((ix_cc_ipv4_cb_get_next_hop)(pContext->callback)))(
                        pContext->responseStatus,
                        pContext->userContext,
                        next_hop_info);
                  PIL_FreeMemory(pContext);
               }
               break;

               case SH_IPV4_ASYNC_FLUSH_PREFIX:
               case SH_IPV4_ASYNC_FLUSH_NHOP:
               case SH_IPV4_GET_NHOP:
               case SH_IPV4_GET_ROUTE:
                  npf_logger_write(LOG_SH, LOG_ERR, 
                      "Control must not reach here for IPv4 CC\n");
                  break;

               default:
                  npf_logger_write(LOG_SH, LOG_ERR, 
                      "Control must not reach here for IPv4 CC\n");
           }
         break;
     
         case SH_FPM_CC:
            switch(respHdr->u.fpmAction) 
            {
               case SH_FPM_ASYNC_ADD_L2ENTRY:
               case SH_FPM_ASYNC_DEL_L2ENTRY:
               {
                  (*((ix_cc_fpm_l2_op_cb)(pContext->callback)))(
                        pContext->responseStatus,
                        pContext->userContext);
                  PIL_FreeMemory(pContext);
               } 
               break;
                         
               case SH_FPM_ASYNC_ADD_ARPENTRY:
               case SH_FPM_ASYNC_DEL_ARPENTRY:
               {
                  (*((ix_cc_fpm_arp_op_cb)(pContext->callback)))(
                        pContext->responseStatus,
                        pContext->userContext);
                  PIL_FreeMemory(pContext);
               }
               break;
                
               case SH_FPM_ADD_L2ENTRY:
               case SH_FPM_DEL_L2ENTRY:
               case SH_FPM_REGISTER_LINKST:
               case SH_FPM_REGISTER_IPV4EXCP:
               case SH_FPM_REGISTER_PKTHDLR:
               case SH_FPM_DEREGISTER_LINKST:
               case SH_FPM_DEREGISTER_IPV4EXCP:
               case SH_FPM_DEREGISTER_PKTHDLR:
               case SH_FPM_FLUSH_ARP:
                  npf_logger_write(LOG_SH, LOG_ERR, 
                      "Control must not reach here for FPM CC\n");
                  break;

               case SH_FPM_EVENT_LINKST:
               case SH_FPM_EVENT_IPV4EXCP:
                  itr = npf_list_itrCreate(&eventCallbackList);
                  if (itr)
                  {
                     npf_list_itrFirst(itr);
                     do
                     {
                        eventListNode = npf_list_itrGetData(itr);
                        if (eventListNode->compName == respHdr->compName)
                        {
                           switch(respHdr->compName)
                           {
                              case SH_FPM_CC:
                                 if(eventListNode->u.fpmAction ==
                                     respHdr->u.fpmAction)
                                    found = TRUE;
                                 break;

                              case SH_IPV4_CC:
                                 if(eventListNode->u.ipv4Action ==
                                     respHdr->u.ipv4Action)
                                    found = TRUE;
                                 break;

                              case SH_STKDRV_CC:
                                 if(eventListNode->u.stkdrvAction ==
                                     respHdr->u.stkdrvAction)
                                    found = TRUE;
                                 break;
    
                              default:
                                 npf_logger_write(LOG_SH, LOG_ERR, 
                                    "Control must not reach here for events\n");
                             
                           }
                        }  
                     }  while ((npf_list_itrNext(itr) != -1) && 
                               (found == FALSE));
                     /* Call the callback function here */

                     switch(eventListNode->compName)
                     {
                        case SH_FPM_CC:
                        {
                           ix_uint32*             arg1;
                           ix_uint32*             arg2;
                           ix_uint32*             arg3;
                           ix_uint32              tmpLen;
                           ix_cc_fpm_link_status* link_status; 

                           if (eventListNode->u.fpmAction == SH_FPM_EVENT_LINKST)
                           {
                              tmpLen = *((ix_uint32*)pos);
                              pos = pos + sizeof(ix_uint32);
                              arg1 = (ix_uint32*)pos;
                              pos = pos + tmpLen;
                              tmpLen = *((ix_uint32*)pos);
                              pos = pos + sizeof(ix_uint32);
                              arg2 = (ix_uint32*)pos;
                              pos = pos + tmpLen;
                              tmpLen = *((ix_uint32*)pos);
                              pos = pos + sizeof(ix_uint32);
                              link_status = (ix_cc_fpm_link_status*)pos;
                              ((ix_cc_fpm_link_status_cb)(eventListNode->callback))
                                (*arg1, *arg2, *link_status);
                           }
                           else if (eventListNode->u.fpmAction == SH_FPM_EVENT_IPV4EXCP)
                           {
                              tmpLen = *((ix_uint32*)pos);
                              pos = pos + sizeof(ix_uint32);
                              arg1 = (ix_uint32*)pos;
                              pos = pos + tmpLen;
                              tmpLen = *((ix_uint32*)pos);
                              pos = pos + sizeof(ix_uint32);
                              arg2 = (ix_uint32*)pos;
                              pos = pos + tmpLen;
                              tmpLen = *((ix_uint32*)pos);
                              pos = pos + sizeof(ix_uint32);
                              arg3 = (ix_uint32*)pos;

                              ((ix_cc_fpm_ipv4_exception_cb)(eventListNode->callback))
                              (*arg1, *arg2, *arg3);
                           }
                        }
                        break;

                        case SH_IPV4_CC:
                        case SH_STKDRV_CC:
                        case SH_FPM_DATA_CC:
                           break;

                     }
                     npf_list_itrDelete(itr); 
                  }

                  break; 
               
               default:
                  npf_logger_write(LOG_SH, LOG_ERR, 
                      "Control must not reach here for FPM CC\n");
            }
         break; 

         case SH_STKDRV_CC:
           switch(respHdr->u.fpmAction)
           {
              case SH_STKDRV_ASYNC_GET_NUMPORTS:
              {
                 ix_uint32*             arg1;

                 pos = pos + sizeof(ix_uint32);
                 arg1 = (ix_uint32*)pos;
 
                 (*((ix_cc_stkdrv_cb_get_num_ports)(pContext->callback)))(
                        pContext->responseStatus,
                        pContext->userContext,
                        (ix_uint32*)arg1);
                 PIL_FreeMemory(pContext);
              }
              break;
                
              case SH_STKDRV_ASYNC_GET_PROPERTY:
              {
                 ix_cc_properties* ccProperties;

                 pos = pos + sizeof(ix_uint32);
                 ccProperties = (ix_cc_properties*)pos;
                 (*((ix_cc_stkdrv_cb_get_property)(pContext->callback)))(
                        pContext->responseStatus,
                        pContext->userContext,
                        (ix_cc_properties*)ccProperties);
                 PIL_FreeMemory(pContext);
              }
              break;
                            
              case SH_STKDRV_ASYNC_SET_PROPERTY:
                  npf_logger_write(LOG_SH, LOG_ERR, 
                      "Control must not reach here for Stack Driver CC\n");
                 break;

              default:
                  npf_logger_write(LOG_SH, LOG_ERR, 
                      "Control must not reach here for Stack Driver CC\n");
          }
          break;

       case SH_FPM_DATA_CC:    /* Fall through */
       default:
         npf_logger_write(LOG_SH, LOG_ERR, 
            "There is no support for this CC\n");

     } /* end of switch for core component */
   }  /* end of while loop */
   return IX_SUCCESS;
}

/*****************************************************************************
*    Name: sh_init 
*
*    Description:
*        This is the Shim initialisation routine. It creates the 
*        reader thread.
*
*    Parameters:
*        None.
*
*    Preconditions:
*        None.
*
*    Postconditions:
*        The Shim reader thread is created.        
*
*    Returns:
*        None.
*
*/

NPF_RET sh_init(char* host)
{
   SH_Context_t*   shContext;
   uint32_t        address;

   address = inet_addr(host);
   g_shim_fd = open("/dev/Shim", O_RDWR);

   npf_list_init(&eventCallbackList, PIL_FreeMemory);
   eventCallbackList.compare = sh_CompareEventList;
   g_sh_RdThread = PIL_CreateThread(sh_reader_thread, 0,
                                    "sh_reader_thread", 0,
                                    (void*)&g_shim_fd , 0); 
   shInitialized = 1;  
   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if (shContext == NULL)
   {
      return NPF_FAIL;
   }
   shContext->compName = SH_FPM_CC;
   shContext->u.fpmAction = SH_FPM_CONFIG_CPADDR;
   shContext->params[0] = &address;
   if (write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
   {
      PIL_FreeMemory(shContext);
      return NPF_FAIL;
   }

   shContext->compName = SH_FPM_CC;
   shContext->u.fpmAction = SH_FPM_REGISTER_PKTHDLR;
   shContext->params[0] = NULL;
   if (write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
   {
      PIL_FreeMemory(shContext);
      return NPF_FAIL;
   }

   PIL_FreeMemory(shContext);
   return NPF_SUCCESS;
}

/*****************************************************************************
*    Name: sh_shutdown 
*
*    Description:
*        This is the Shim shutdown routine. It destroys the event callback
*        list. 
*
*    Parameters:
*        None.
*
*    Preconditions:
*        None.
*
*    Postconditions:
*        The Shim reader thread is created.        
*
*    Returns:
*        None.
*
*/

NPF_RET sh_shutdown()
{
   SH_Context_t*   shContext;

   shContext = (SH_Context_t*)PIL_AllocateMemory(sizeof(SH_Context_t),
                              PIL_MEM_ZERO_INIT);
   if (shContext == NULL)
   {
      return NPF_FAIL;
   }
   shContext->compName = SH_FPM_CC;
   shContext->u.fpmAction = SH_FPM_DEREGISTER_PKTHDLR;
   if (write(g_shim_fd, shContext, sizeof(SH_Context_t)) == -1)
   {
      PIL_FreeMemory(shContext);
      return NPF_FAIL;
   }
   PIL_FreeMemory(shContext);
   npf_list_destroy(&eventCallbackList);

   if(shInitialized)
   {
      PIL_DestroyThread(g_sh_RdThread, NULL);
      shInitialized = 0;
   } 
   return NPF_SUCCESS;
}
