/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ForwardingPlane/TransportPlugin/inc/bpriv.h,v $
 * $Revision: 1.6 $
 * $Date: 2003/11/17 16:57:49 $
 *
 * Description:
 * Internal method prototypes and symbol definitions are contained here.
 *
 ****************************************************************************/

#ifndef _B_PRIV_H_
#define _B_PRIV_H_

#include "fpp_common.h"
#include "forces_Types.h"
#include "interconnect_api.h"

#ifdef __cplusplus
extern "C" {
#endif

#define FORCES_VERSION 1

#define QUEUE_SIZE 6


/************* Globals *****************/
/* globals */
/*callback info*/

typedef struct __FE_Caps_Info_t {
	uint32_t Caps_req;	 
   FPPI_FE_Caps*   fe_caps;
   FPPI_CBHANDLE   cbhandle;
} FE_Caps_Info_t;

typedef struct __bken_cb_info {
	int	init;			// Interconnect init param.
	CBClientDatagramFuncs clientparams;	// parameters used to initialize Client Interconnect.
	uint32_t connect_id;	// Client connection Id, return by the interconnect.

	FE_Caps_Info_t fe_caps_info;	
} BEND_cb_info;

extern BEND_cb_info g_BEND_cb_info;

typedef struct ForCESMessage_t
{
	forces_header_t header; /* alway present, always the same type */
	char		data[0]; /* basically just a place holder
				   making it easier to find 
				   the actual data for the particular
				   type of message */
} *ForCESMessage;


inline int ForCESGetConnectionId();
int ForCESReceivedCapsRequest();

typedef struct BEReport_t
{
    ForCESMessage msg;
    uint32_t size;
    char* buf;
    uint32_t connectionid;
}* BEReport;

BEReport newBEReport(
        uint32_t connectionid,
        uint32_t cmd_type,
        uint32_t block_type,
        uint32_t status,
		  uint32_t corr,
        size_t   size);
void deleteBEReport(BEReport rpt);
void BEReportSend(BEReport rpt);

/************* Internal Methods *****************/

int ForCES_InitInterconnect (char *ipaddr);

void ForCES_DatagramReceive (void* buf, uint32_t connect_id, uint32_t len);
void ForCES_DatagramException (
        uint32_t ex,
        uint32_t connect_id);
ForCESMessage newForCESMessage(uint32_t connid ,uint16_t msg_type,uint32_t correlator,size_t size);

/* feinfo of fe to send message to */
void ForCESMessageSend(uint32_t connid, ForCESMessage msg, uint32_t size);

/* do not call if msg has been sent.  
 * the send frees the resources when it is finished with them */ 
void deleteForCESMessage(ForCESMessage msg);
int ForCESCheckVersion(uint8_t version);
int ForCES_DisconnectInterconnect ();
int ForCES_Unbind(uint32_t connectionid);
int ForCES_Store_FECaps( 
	    FPPI_FE_Caps*   fe_caps,
       FPPI_CBHANDLE   cbhandle);

#define sizeof_uint32_t(a) sizeof(uint32_t)

#define sizeof_array_char(a,l) sizeof(char)*l

#define sizeof_array_uint8_t(l) sizeof(uint8_t)*l

// Check if this needs to be here
int ForCES_MessageReceive (
        void *pbuf,
        uint32_t connect_id,
        uint32_t length);

int ForCES_encapsulate_FECaps(
    	FPPI_FE_Caps*   fe_caps,
       FPPI_CBHANDLE   cbhandle,
       uint32_t 	   connectionid,
       char**		   buf,
       int*		   size);
int ForCES_encapsulate_Event(
       uint32_t 	   connectionid,
       uint32_t event, 
       uint32_t block_type, 
       uint32_t block_handle, 
       uint32_t 	   length,
       void* 		   event_data,
       char**		   buf,
       int*		   size);
int ForCES_encapsulate_PacketRedirect(
       uint32_t 	   connectionid,
       uint32_t         block_handle, 
       npf_Packet_t     *packet_list,
       char**		   buf,
       int*		   size);
int ForCES_encapsulate_Report_Status(
       uint32_t 	   connectionid,
       NPF_CBCAT	   cbtype,
	uint32_t         corr,
       uint32_t         block_handle, 
       void             *response_data,
       char**		   buf,
       int*		   size);
              
/* Encap / Decap Funcations */
/* FPPI_FE_Caps */
char* encapsulate_forces_bind_info_t (
                forces_bind_info_t* a,
                char* buf
                );
char* encapsulate_FPPI_FE_Caps_t(
	         FPPI_FE_Caps*   fe_caps,
                char* buf
                );
uint32_t sizeof_FPPI_FE_Caps_t(
	    	FPPI_FE_Caps*   fe_caps
                );

/* FPPI_FE_event_t */
char* encapsulate_FE_event_t(
		  uint32_t event_t, 
		  uint32_t block_type, 
		  uint32_t block_handle, 
		  uint32_t l, 
		  void *a,
                char* buf
                );
uint32_t sizeof_FE_event_t(
	    	uint32_t   l
                );
/* PacketRedirection */
char* encapsulate_sendPacket(
		  uint32_t block_handle, 
                npf_Packet_t    *packet_list,
                char* buf
                );
uint32_t sizeof_PacketRedirection(
              npf_Packet_t    *packet_list
                );
/* npf_IPv4EntryStausList_t */
char* encapsulate_npf_IPv4EntryStatusList_t(
                npf_IPv4EntryStatusList    *a,
                char* buf
                );
uint32_t sizeof_npf_IPv4EntryStatusList_t(
              npf_IPv4EntryStatusList    *a
                );
/* npf_IPv4_Status_t */
char *encapsulate_npf_IPv4_Status_t(
              npf_IPv4_Status_t    *a,
	       char *buf
                );
uint32_t sizeof_npf_IPv4_Status_t(
              npf_IPv4_Status_t    *a
                );
char *encapsulate_Report_Status(
		uint32_t block_handle,
		NPF_CBCAT cbtype, 
		npf_IPv4EntryStatusList    *a, 
		char *buf
		);
uint32_t sizeof_encapsulate_Report_Status(
		void *response_data);

char* decapsulate_FPPI_FEandL2IDs_t(
                FPPI_FEandL2IDs   *a,
                char* buf
                );
char* decapsulate_FPPI_NextHopInfo_t(
                FPPI_NextHopInfo   *a,
                char* buf
                );
char* decapsulate_FPPI_UnicastNextHopUpdateInfo_t(
                FPPI_UnicastNextHopUpdateInfo_t* a,
                char* buf
                );
char* decapsulate_Forces_Message_t(
                ForCESMessage *a,
                char* buf
                );
/*
char *decapsulate_forces_command_t (
		forces_command_t *a, 
		char *buf);*/

int ForCES_Capability_Response_t(
    	FPPI_FE_Caps*   fe_caps,
       FPPI_CBHANDLE   cbhandle,
       uint32_t 	   connectionid,
       char**		   buf,
       int*		   size);

uint32_t sizeof_Capability_Response_t(
                FPPI_FE_Caps*   fe_caps
                );

char* encapsulate_Capability_Response_t(
                 FPPI_FE_Caps*   fe_caps,
                char* buf
                );

char* encapsulate_PacketRedirection(
                  uint32_t block_handle,
                npf_Packet_t    *packet_list,
                char* buf
                );
		
#define _IPv6_ 1

#ifdef __cplusplus
}
#endif

#endif /* _B_PRIV_H_ */
