/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2003 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ForwardingPlane/TransportPlugin/src/backendapi.c,v $
 * $Revision: 1.7 $
 * $Date: 2003/11/17 16:57:49 $
 *
 * Description:
 * This file implements the Backend API defined in backendapi.h, it also 
 * implements the CbPibResponse() callback interface exposed by the ForCES PIB 
 * layer. In addition, it contains the impl. of some of the compare functions 
 * used by the ReportList.
 *
 ****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <assert.h>
#include "backendapi.h"
#include "fpp_common.h"
#include "forces_Types.h"
#include "pil.h"
#include "bpriv.h"
#include "log.h"
#include "funcp.h"
#include "fppdecap_npf.h"
#include "fppdecap_common.h"
#include "fppencap_capability_types.h"
#include "pdk_cm_types.h"

#ifdef _INCLUDE_ATM
#include "npf_atm_types.h"
#include "fppapi_atm_types.h"
#include "fppdecap_npf_atm.h"
#include "fppencap_npf_atm.h"
#endif

#include "fppdecap_npf_ipv4.h"
#include "fppencap_npf_ipv4.h"

#ifdef _INCLUDE_IPV6
#include "fppdecap_npf_ipv6.h"
#include "fppencap_npf_ipv6.h"
#endif

#ifdef _INCLUDE_DIFFSERV
#include "fppdecap_npf_ds.h"
#include "fppencap_npf_ds.h"
#endif

#ifdef _INCLUDE_MPLS
#include "fppapi_mpls_types.h"
#include "fppdecap_mpls.h"
#include "fppencap_mpls.h"
#endif

#include "fppdecap_npf_if.h"
#include "fppencap_npf_if.h"



/********************************* Globals *********************************/
//DList   ReportList;
int ClientAcceptFlag = 0;
int DecisionCount = 0;
int Bend_Initialized = 0;
int Bend_Ready = 0;
PilCriticalSection fpp_lock;

void free_object ( void* object, uint32_t cbtype);
extern void ForCESStartHeartbeat(int heartbeat_interval);

/****************************************************************************
 * Function: BENDAPI_register_callback()                                    *
 * ---------------------------                                              *
 * Parameters: backendapi_callback - the callback function being registered *
 *             FPPI_CBHANDLE - unique ID generated by the Plugin, used to   *
 *                              identify the callback function              *
 *             NPF_CBCAT - the callback type for which the fn is registered *
 *             FPPI_CONTEXT - value passed by client which should be        *
 *                              returned when callback is invoked           *
 *                                                                          *
 * Description: This is the generic function used by the client             *
 *              to register a callback function for a particular event/     *
 *              call identified by the callback type.                       *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET BENDAPI_register_callback(
    backendapi_callback    callback,
    FPPI_CBHANDLE        *cbhandle,
    NPF_CBCAT        cbtype,
    FPPI_CONTEXT        context)
{
    if (Bend_Initialized == 0)
        return FPPI_FAIL;
    if (pdk_callback_register(cbtype, context, (NPF_GENERAL_CBFUNC)callback, cbhandle) != NPF_SUCCESS) {
    npf_logger_write(LOG_TPI, LOG_ERR, "register failed \n");
    return FPPI_FAIL;
    }
    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: BENDAPI_deregister_callback()                                  *
 * ---------------------------                                              *
 * Parameters: FPPI_CBHANDLE - unique ID used to identify the callback fn   *
 *                                                                          *
 * Description: This is the generic function used by the client             *
 *              to de-register a callback function.                         *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET BENDAPI_deregister_callback(
    FPPI_CBHANDLE    cbhandle)
{
    if (Bend_Initialized == 0)
        return FPPI_FAIL;
    if (pdk_callback_deregister(cbhandle) != NPF_SUCCESS)
    return FPPI_FAIL;
    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: BENDAPI_Initialize()                                           *
 * ---------------------------                                              *
 * Parameters: none                                                         *
 *                                                                          *
 * Description: This function is used to initialize the Transport Plugin    *
 *              module in the forwarding plane. It initializes the different*
 *              lists used by the plugin module.                            *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET BENDAPI_Initialize (void)
{
    if (Bend_Initialized == 1)
        return FPPI_FAIL;
    Bend_Initialized = 1;

    //npf_list_init(&ReportList, PIL_FreeMemory);
    //ReportList.compare = &compareReport;

    PIL_InitializeCriticalSection(&fpp_lock);

    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: BENDAPI_Shutdown()                                             *
 * ---------------------------                                              *
 * Parameters: none                                                         *
 *                                                                          *
 * Description: This function is used to de-initialize the Transport Plugin *
 *              module. It destroys the lists initialized by the module.    *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET BENDAPI_Shutdown (void)
{
    if (Bend_Initialized == 0)
        return FPPI_FAIL;
    Bend_Initialized = 0;

    printf("BENAPI_Shutdown\n");
    ForCES_DisconnectInterconnect ();

    //npf_list_destroy(&ReportList);
    PIL_DestroyCriticalSection(&fpp_lock);

    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: void ForCES_MessageReceive()                                   *
 * ---------------------------                                              *
 *                                                                          *
 * Description: This is a callback fn which is registered with the COPS     *
 *              client module and is invoked when the CE sends COPS Decision*
 *              messages or on other events such as CE Bind and CE UnBind.  *
 *                                                                          *
 * Return: none                                                             *
 *                                                                          *
 ****************************************************************************/

int ForCES_MessageReceive (
        void *pbuf,
        uint32_t connect_id,
        uint32_t length)
{
    uint32_t retval;
    
    ForCESMessage msg;
    forces_header_t *header;
    char *buf;
    uint32_t /*index = 0,*/ cbtype = FPPI_INVALID_CBTYPE;
    FPPI_CONTEXT context;
    FPPI_CBHANDLE cbid, nextcbid;
    //	FPPI_CBCORRELATOR cbcorrelator = 0;
    //	FPPI_Status* status;
    backendapi_callback cb;
    void *object = NULL;
    //	ReportInfo* newRepInfo;
   
    decapsulate_Forces_Message_t(&msg, pbuf);
    header = &(msg->header);
        
    header->msg_type= ntohs(header->msg_type);
    buf = msg->data;
    switch (header->msg_type)
    {
    	case FE_Bind_Request :
       case Capability_Response :
       case Topology_Request :
       case Topology_Response :
	case FE_Start_Operation :
	case FE_Event_Notification :
	case Unbind :
	case Heartbeat :
    	case Config_Query_Request :
	case Config_Query_Response :
	    retval = MSG_HANDLER(header, msg, connect_id, length, &object, &cbtype);
	    break;
    	case FE_Bind_Response :
	    retval = MSG_HANDLER( header, msg, connect_id, length, &object, &cbtype);
	    ClientAcceptFlag = 1; /* Check WHY this has to be done */
	    Bend_Ready = 1; /* Check WHY this has to be done */
           break;
       case Capability_Request :
           printf ("\n Cap req\n");
	    retval = MSG_HANDLER( header, msg, connect_id, length, &object, &cbtype);
	    break;

       default :
           printf ( "FLEX MSG type %d not yet handled\n", header->msg_type);
           break;
    }
    if (cbtype != FPPI_INVALID_CBTYPE)
    {
        /* loop the CB to execute all the callback func */
        cb = (backendapi_callback)pdk_callback_firstFunc(
        			cbtype, &context, &nextcbid);
        if (cb == NULL)
        {
    	     goto TPI_LBL;
        }
        do
        {
            cbid = nextcbid;
            	cb( header->cmd_correlator, context, object );
        } while ((cb = (backendapi_callback)pdk_callback_nextFunc(
         		cbid, &context, &nextcbid)) != NULL);
    } /* if (cbtype != FPPI_INVALID_CBTYPE) */

   TPI_LBL:

    if (object != NULL)
    {
        free_object(object, cbtype);
        free(object);
    }
    return 1;
}

/****************************************************************************
 * Function: free_object ()                                                 *
 * ---------------------------                                              *
 * Parameters: object - the object/struct to be freed                       *
 *             cbtype - the callback type of the object/struct              *
 *                                                                          *
 * Description: This fn is used to free objects/structs acc to their cbtype *
 *                                                                          *
 * Return: none                                                             *
 *                                                                          *
 ****************************************************************************/

void free_object (
    void* object, uint32_t cbtype)
{
   /*  npf_ipv4_RouteList_t* rlist; 
    npf_ipv4_ARPList_t* alist; 
    npf_ipv4_ARPKeyList_t* akeylist; */
    npf_L3IntfAttrsList_t* l3list;
    npf_L2IntfAttrsList_t* l2list;
    npf_PacketList_t* packlist;
    npf_PortList_t* plist;
    npf_ipv4_UnicastNextHopUpdateInfoList_t *nhAdd;
    npf_ipv4_UnicastPrefixList_t    *prefix;
    npf_ipv4_UnicastNextHopDeleteInfoList_t  *nhDel;

    switch(cbtype)
    {
#if 0
    case FPPI_IPv4_AddRoute:
    case FPPI_IPv4_DelRoute:
        rlist = (npf_ipv4_RouteList_t *)object;
        free(rlist->route_list->data.next_hop_list);
        free(rlist->route_list);
        break;
#endif

    case FPPI_IPv4_UniAddNextHop:
         nhAdd = (npf_ipv4_UnicastNextHopUpdateInfoList_t*)object;
         if((nhAdd) && (nhAdd->numEntries) && (nhAdd->nhUpdateInfo) && (nhAdd->nhUpdateInfo->nhInfo))
            PIL_FreeMemory(nhAdd->nhUpdateInfo->nhInfo);
         if((nhAdd) && (nhAdd->numEntries) && (nhAdd->nhUpdateInfo))
            PIL_FreeMemory(nhAdd->nhUpdateInfo);
         /* nhAdd is being freed as object by the caller of this func */
         break;

    case FPPI_IPv4_UniDelNextHop:
         nhDel =  (npf_ipv4_UnicastNextHopDeleteInfoList_t*)object;
         if((nhDel) && (nhDel->numEntries) && (nhDel->nhDeleteInfo))
            PIL_FreeMemory(nhDel->nhDeleteInfo);
         break;

    case FPPI_IPv4_UniAddPrefix:
    case FPPI_IPv4_UniDelPrefix:
         prefix = (npf_ipv4_UnicastPrefixList_t *)object;
         PIL_FreeMemory(prefix->prfxInfo);
         break;

#if 0        
    case FPPI_ARP_AddEntry:
    case FPPI_ARP_DelEntry:
        alist = (npf_ipv4_ARPList_t *)object;
        free(alist->arp_list);
        break;
#endif
#ifdef _INCLUDE_MPLS
    case FPPI_CAT_MPLS:
         /* No embedded pointers in object to be freed */
        break;
#endif

#if 0
    case FPPI_ARP_QueryEntry:
        akeylist = (npf_ipv4_ARPKeyList_t *)object;
        free(akeylist->key_list);
        break;
#endif
    case FPPI_L3_SetProperties:
    case FPPI_L3_DelProperties:
        l3list = (npf_L3IntfAttrsList_t *)object;
        free(l3list->port_list);
        break;
    case FPPI_L2_SetProperties:
    case FPPI_L2_DelProperties:
        l2list = (npf_L2IntfAttrsList_t *)object;
        free(l2list->port_list);
        break;
    case FPPI_PacketIO_Event:
    packlist = (npf_PacketList_t *)object;
        free(packlist->packet_list);
        break;
    case FPPI_L2_GetStatistics:
    case FPPI_L3_GetStatistics:
    case FPPI_ICMP_GetStatistics:
        plist = (npf_PortList_t *)object;
        free(plist->port_list);
        break;
    default:
        /* The delete and get stats for the diffserv methods have
           no embedded objs to free so do nothing for them here. */
    }
}

/****************************************************************************
 * Function: BENDAPI_bindRequest()                                          *
 * ---------------------------                                              *
 * Parameters: ipaddr - the IP Address of the Control Element               *
 *             FPPI_CBHANDLE - the ID used to identify the callback fn      *
 *                                                                          *
 * Description: This fn is used to send a Bind Request to the Control Plane *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET BENDAPI_bindRequest(
    char*          ipaddr,
    char*          fename,
    FPPI_FEID      fpFeid,
    FPPI_CBHANDLE  cbhandle)
{
    // CBClientDatagramFuncs * cbParams;
    uint32_t connectionid;
    ForCESMessage msg;
    forces_bind_info_t bind_info;
    
    // CS_SECURITY* copssec = NULL;
    // uint32_t reqtimeout = 200000;
    char pepIdAndFeid[FPPI_MAX_NAME_LEN];

    if (Bend_Initialized == 0)
        return FPPI_FAIL;

    PIL_EnterCriticalSection(&fpp_lock);

    /* Interconnect Initialize : Func in bcallback.c*/
    if (ForCES_InitInterconnect (ipaddr))
    {
        PIL_LeaveCriticalSection(&fpp_lock);
        return FPPI_FAIL;
    }
    
    /* Put the fename and feid in the same string, using a unique
       delimiter so can be separated at CP end. */
    sprintf(pepIdAndFeid, "%s%s%d", fename, g_pdk_feidNameSeparator, (int)fpFeid);

    // send Client Open message
    connectionid = ForCESGetConnectionId();

    msg = newForCESMessage(connectionid, FE_Bind_Request,cbhandle,sizeof(forces_bind_info_t));

    bind_info.feid = fpFeid;
    bind_info.control_encap_type = 0;
    bind_info.data_encap_type = 0;
    bind_info.bind_status = 0;
    bind_info.heartbeat_interval = 100;

    encapsulate_forces_bind_info_t(&bind_info, msg->data);
    ForCESMessageSend(connectionid , msg, sizeof(forces_bind_info_t));
    
    ForCESStartHeartbeat(bind_info.heartbeat_interval);

    PIL_LeaveCriticalSection(&fpp_lock);

    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: BENDAPI_unbindRequest()                                        *
 * ---------------------------                                              *
 * Parameters: none                                                         *
 *                                                                          *
 * Description: This fn is used to send a UnBind Request to the Control Pl. *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET BENDAPI_unbindRequest(void)
{
    uint32_t connectionid;
    if (Bend_Initialized == 0)
        return FPPI_FAIL;
    // send Client Close message
    printf("BENAPI_unbindRequest\n");
    connectionid = ForCESGetConnectionId();
    ForCES_Unbind(connectionid);
    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: BENDAPI_capabilityRequest()                                    *
 * ---------------------------                                              *
 * Parameters: FPPI_FE_Caps - FE Capability struct                          *
 *             FPPI_CBHANDLE - the ID used to identify the callback fn      *
 *                                                                          *
 * Description: This fn is used to send FE Capabilities to the Control Plane*
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET BENDAPI_capabilityRequest(
    FPPI_FE_Caps*   fe_caps,
        FPPI_CBHANDLE   cbhandle)
{
    char *buf;
    int size;
    uint32_t connect_id;
    if ((Bend_Initialized == 0) || (ClientAcceptFlag == 0))
        return FPPI_FAIL;
    
    connect_id = ForCESGetConnectionId();
    ForCES_Store_FECaps( fe_caps,cbhandle);    
	
    /* Already received Caps Request */
    if ( ForCESReceivedCapsRequest())
    {
	ForCES_Capability_Response_t( fe_caps,cbhandle,connect_id, &buf, &size);
        ForCESMessageSend(connect_id, (ForCESMessage)buf, size);
    }

    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: BENDAPI_sendPacket()                                           *
 * ---------------------------                                              *
 * Parameters: packet_list - array of npf_Packet_t structs                  *
 *             packet_count - the number of entries in the array            *
 *                                                                          *
 * Description: This fn is used to send control Packets to the Control Plane*
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET BENDAPI_sendPacket(
        npf_Packet_t    *packet_list,
        uint32_t        packet_count)
{
    char* buffer;
    int /*res,*/ i;
    uint32_t connect_id;
    uint32_t block_handle=0;
    int size;
    	
    if ((Bend_Initialized == 0) || (Bend_Ready == 0))
        return FPPI_FAIL;
    
    connect_id = ForCESGetConnectionId();

    PIL_EnterCriticalSection(&fpp_lock);
#ifdef BEND_DEBUG
    npf_logger_write(LOG_TPI, LOG_INFO,
        "In BENDAPI_SendPacket():: Packet count = %d\n", packet_count);
#endif

    for (i = 0; i < packet_count; i++) {
#ifdef BEND_DEBUG
    npf_logger_write(LOG_TPI, LOG_INFO,
        "Packet len = %d\n", packet_list->length);
#endif
 
    ForCES_encapsulate_PacketRedirect(connect_id, block_handle, packet_list, &buffer, &size);
    ForCESMessageSend(connect_id, (ForCESMessage)buffer, size);
    
    packet_list += 1;
    }

    PIL_LeaveCriticalSection(&fpp_lock);
    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: BENDAPI_sendEvent()                                            *
 * ---------------------------                                              *
 * Parameters: event_type - the event type defined in pdk_callback.h        *
 *             event_data - the struct corresponding to the event           *
 *                                                                          *
 * Description: This fn is used to send async Events to the Control Plane   *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

FPPI_RET BENDAPI_sendEvent(
        NPF_CBCAT       event_type,
        void*           event_data)
{
    // int res;
    FPPI_PortID    portid;
    uint32_t connect_id;
    void *data;
    char *buf;
    int size;
    int length;
    uint32_t forcesevent;
    uint32_t blocktype;
    uint32_t blockhandle;

    if ((Bend_Initialized == 0) || (Bend_Ready == 0))
        return FPPI_FAIL;
    
    connect_id = ForCESGetConnectionId();
    
    PIL_EnterCriticalSection(&fpp_lock);

    switch (event_type) {
    case FPPI_Directly_Connected_Host_Event:
    {
        ForCESMessage msg_direct_connected;
        char *direct_event;
        forcesevent = FORCES_EVENT_DIRECT_CONNECTED_HOST;
        blocktype = blockhandle = 0;
        length = sizeof(npf_fpm_ipv4_exception_t);
    
        size= sizeof_FE_event_t(length); 
        msg_direct_connected = newForCESMessage(connect_id, FE_Event_Notification, 
                                       0, size);
    
        direct_event = encapsulate_uint32_t (&forcesevent, msg_direct_connected->data);
        direct_event = encapsulate_uint32_t (&blocktype, direct_event);
        direct_event = encapsulate_uint32_t (&blockhandle, direct_event);
        direct_event = encapsulate_uint32_t ((uint32_t *)&length, direct_event);
        direct_event = encapsulate_uint32_t (&(((npf_fpm_ipv4_exception_t *)event_data)->portid), direct_event);
        direct_event = encapsulate_uint32_t (&(((npf_fpm_ipv4_exception_t *)event_data)->dest), direct_event);
        buf=(char *)msg_direct_connected;
    }

    ForCESMessageSend(connect_id, (ForCESMessage)buf, size);

    PIL_LeaveCriticalSection(&fpp_lock);
    return FPPI_SUCCESS;

    case FPPI_LinkUp_Event:
       forcesevent = FORCES_EVENT_PORT_UP;
       blocktype = blockhandle = 0;
       length = 4;
       portid = *(FPPI_PortID*)event_data;
       portid = htonl(portid);
       data = &portid;
	break;
    case FPPI_LinkDown_Event:
       forcesevent = FORCES_EVENT_PORT_DOWN;
       blocktype = blockhandle = 0;
       length = 4;
       portid = *(FPPI_PortID*)event_data;
       portid = htonl(portid);
       data = &portid;
	break;
    case FPPI_ARP_AddEntry_Event:
       forcesevent = FORCES_EVENT_ARPENTRY_ADD;
       blocktype = blockhandle = 0;
       length = 20;
       data = event_data;
	break;
    case FPPI_ARP_DelEntry_Event:
       forcesevent = FORCES_EVENT_ARPENTRY_DEL;
       blocktype = blockhandle = 0;
       length = 20;
       data = event_data;
	break;
    default:
    // error - type not supported
	break;
    }

    // send Accounting report
    ForCES_encapsulate_Event(connect_id,forcesevent,blocktype,blockhandle,length, data, &buf, &size);
    ForCESMessageSend(connect_id, (ForCESMessage)buf, size);

    PIL_LeaveCriticalSection(&fpp_lock);
    return FPPI_SUCCESS;
}

/****************************************************************************
 * Function: BENDAPI_Report_Status()                                        *
 * ---------------------------                                              *
 * Parameters: FPPI_Status - the status of the call (success or failure)    *
 *             NPF_CBCAT - the callback type for which the status is being  *
 *                     reported                                             *
 *             FPPI_CBCORRELATOR - the unique ID generated by the Plugin    *
 *               which is passed to the client during callback invocation   *
 *             response_data - the struct containing any response data      *
 *                                                                          *
 * Description: This fn is used to notify the Control Plane about the       *
 *              status of a call which was sent by the CP (decision msg)    *
 *              to be executed on the Forwarding Plane, using a COPS Report *
 *                                                                          *
 * Return: FPPI_SUCCESS, FPPI_FAIL                                          *
 *                                                                          *
 ****************************************************************************/

static uint32_t cmdTypes(uint32_t cbtype)
{
    switch(cbtype)
    {    
        case FPPI_IPv4_AddRoute:
        case FPPI_ARP_AddEntry:
        case FPPI_IPv4_UniAddNextHop:
        case FPPI_IPv4_UniAddPrefix:
        case FPPI_IPv4_UniAddToL2:
#ifdef _INCLUDE_IPV6
        case FPPI_IPv6_AddNextHop:
        case FPPI_IPv6_AddPrefix:
        case FPPI_IPv6_AddToL2:
#endif
        case FPPI_L2_SetProperties:
        case FPPI_L3_SetProperties:
        case FPPI_L3_SetPropertiesIPv6:
#ifdef _INCLUDE_IPV6TM
        case FPPI_IPv6TM_EnableAuto:
        case FPPI_IPv6TM_EnableConfig:
        case FPPI_IPv6TM_Enable6to4:
#endif
#ifdef _INCLUDE_ATM
        case FPPI_ATM_IfAttrSet: /* Start ATM */
        case FPPI_ATM_VccSet:
        case FPPI_ATM_VccBind:
        case FPPI_ATM_AAL2_ChannelSet:
        case FPPI_ATM_ConnectionXcSet:
        case FPPI_ATM_OAM_CP_Set:
        case FPPI_ATM_OAM_CC_Set:
        case FPPI_ATM_OAM_LB_Set:
        case FPPI_ATM_OAM_Monitor:
        case FPPI_ATM_OAM_AlarmSet:
        case FPPI_ATM_OAM_PM_Set: /* end of ATM */
#endif /* _INCLUDE_ATM */
#ifdef _INCLUDE_DIFFSERV
        case FPPI_DS_DpeOperation:
#endif
            return Forces_Add_Cmd;

        case FPPI_IPv4_DelRoute:
#ifdef _INCLUDE_ATM
        case FPPI_ATM_IfDelete:   /* start -ATM */
        case FPPI_ATM_VccDelete:
        case FPPI_ATM_AAL2_ChannelDel:
        case FPPI_ATM_OAM_AlarmClear:
        case FPPI_ATM_ConnectionXcDel: /* end - ATM */
#endif /* _INCLUDE_ATM */
        case FPPI_ARP_DelEntry:
        case FPPI_IPv4_UniDelNextHop:
        case FPPI_IPv4_UniDelPrefix:
        case FPPI_IPv4_UniDelToL2:
#ifdef _INCLUDE_IPV6
        case FPPI_IPv6_DelNextHop:
        case FPPI_IPv6_DelPrefix:
        case FPPI_IPv6_DelToL2:
#endif
        case FPPI_L2_DelProperties:
        case FPPI_L3_DelProperties:
        case FPPI_L3_DelPropertiesIPv6:
#ifdef _INCLUDE_IPV6TM
        case FPPI_IPv6TM_DisableAuto:
        case FPPI_IPv6TM_DisableConfig:
        case FPPI_IPv6TM_Disable6to4:
#endif
            return Forces_Delete_Cmd;

        case FPPI_IPv4_PurgeRoute:
        case FPPI_ARP_PurgeEntry:
        case FPPI_IPv4_UniFlushNextHop:
        case FPPI_IPv4_UniFlushPrefix:
        case FPPI_IPv4_UniFlushToL2:
#ifdef _INCLUDE_IPV6
        case FPPI_IPv6_FlushNextHop:
        case FPPI_IPv6_FlushPrefix:
#endif
            return Forces_DeleteAll_Cmd;

#ifdef _INCLUDE_MPLS
        case FPPI_CAT_MPLS:    /* MPLS  Category */
#endif

        case FPPI_L2_GetStatistics:
        case FPPI_L3_GetStatistics:
        case FPPI_ICMP_GetStatistics:
#ifdef _INCLUDE_ATM
        case FPPI_ATM_VccOperStatusGet:
        case FPPI_ATM_VccStatsGet:
        case FPPI_ATM_IfGenStatsGet:
#endif /* _INCLUDE_ATM */
	     return Forces_QueryStats_Cmd;

#ifdef _INCLUDE_ATM
        case FPPI_ATM_VccEnable:
        case FPPI_ATM_VccDisable:
        case FPPI_ATM_IfEnable:
        case FPPI_ATM_IfDisable:
             return Forces_Update_Cmd;
#endif /* _INCLUDE_ATM */

        case FPPI_IPv4_UniQueryNextHop:
        case FPPI_IPv4_UniQueryPrefix:
           return Forces_QueryProps_Cmd;

        default :
           break;
    }
    return 0;
}

static uint32_t blockTypes(uint32_t cbtype)
{
    switch(cbtype)
    {
#if 0 /* Was applicable only for Unified */
        case FPPI_IPv4_AddRoute:
        case FPPI_IPv4_DelRoute:
        case FPPI_IPv4_PurgeRoute:
            return Forces_IPv4_Block;
            break;
#endif
        case FPPI_ARP_AddEntry:
        case FPPI_ARP_DelEntry:
        case FPPI_ARP_PurgeEntry:
            return Forces_L2_ARP_Block;
            break;
        case FPPI_IPv4_UniAddNextHop:
        case FPPI_IPv4_UniDelNextHop:
        case FPPI_IPv4_UniFlushNextHop:
        case FPPI_IPv4_UniQueryNextHop:
            return Forces_IPv4_NextHop_Block;
            break;
        case FPPI_IPv4_UniAddPrefix:
        case FPPI_IPv4_UniDelPrefix:
        case FPPI_IPv4_UniFlushPrefix:
        case FPPI_IPv4_UniQueryPrefix:
            return Forces_IPv4_Prefix_Block;
            break;
        case FPPI_L2_SetProperties:
        case FPPI_L2_DelProperties:
        case FPPI_L2_GetStatistics:
		return Forces_L2_Interface_Block;

        case FPPI_L3_SetProperties:
        case FPPI_L3_DelProperties:
        case FPPI_L3_GetStatistics:
		return Forces_L3_Interface_Block;

#ifdef _INCLUDE_IPV6
        case FPPI_L3_SetPropertiesIPv6:
        case FPPI_L3_DelPropertiesIPv6:
		return Forces_L3_IPv6_Interface_Block;
#endif

	 case FPPI_ICMP_GetStatistics:
		return Forces_ICMP_Block;
#ifdef _INCLUDE_ATM
         case FPPI_ATM_IfAttrSet:
                return Forces_IfATM_IfAttrSet_Block;
         case FPPI_ATM_IfDelete:
                return Forces_IfATM_IfDelete_Block;
         case FPPI_ATM_IfEnable:
                return Forces_IfATM_IfEnable_Block;
         case FPPI_ATM_IfDisable:
                return Forces_IfATM_IfDisable_Block;
         case FPPI_ATM_IfGenStatsGet:
                return Forces_IfATM_IfGenStatsGet_Block;
         case FPPI_ATM_VccSet:
                return Forces_IfATM_VccSet_Block;
         case FPPI_ATM_VccBind:
                return Forces_IfATM_VccBind_Block;
         case FPPI_ATM_VccDelete:
                return Forces_IfATM_VccDelete_Block;
         case FPPI_ATM_VccEnable:
                return Forces_IfATM_VccEnable_Block;
         case FPPI_ATM_VccDisable:
                return Forces_IfATM_VccDisable_Block;
         case FPPI_ATM_VccOperStatusGet:
                return Forces_IfATM_VccOperStatusGet_Block;
         case FPPI_ATM_VccStatsGet:
                return Forces_IfATM_VccStatsGet_Block;
         case FPPI_ATM_AAL2_ChannelSet:
                return Forces_IfATM_AAL2_ChannelSet_Block;
         case FPPI_ATM_AAL2_ChannelDel:
                return Forces_IfATM_AAL2_ChannelDel_Block;
         case FPPI_ATM_ConnectionXcSet:
                return Forces_IfATM_ConnectionXcSet_Block;
         case FPPI_ATM_ConnectionXcDel:
                return Forces_IfATM_ConnectionXcDel_Block;
         case FPPI_ATM_OAM_CP_Set:
                return Forces_IfATM_OAM_CP_Set_Block;
         case FPPI_ATM_OAM_CC_Set:
                return Forces_IfATM_OAM_CC_Set_Block;
         case FPPI_ATM_OAM_LB_Set:
                return Forces_IfATM_OAM_LB_Set_Block;
         case FPPI_ATM_OAM_Monitor:
                return Forces_IfATM_OAM_Monitor_Block;
         case FPPI_ATM_OAM_AlarmSet:
                return Forces_IfATM_OAM_AlarmSet_Block;
         case FPPI_ATM_OAM_AlarmClear:
                return Forces_IfATM_OAM_AlarmClear_Block;
         case FPPI_ATM_OAM_PM_Set:
                return Forces_IfATM_OAM_PM_Set_Block;
#endif /* _INCLUDE_ATM */

#ifdef _INCLUDE_IPV6
        case FPPI_IPv6_AddNextHop:
	     case FPPI_IPv6_DelNextHop:
        case FPPI_IPv6_FlushNextHop:
        case FPPI_IPv6_QueryNextHop:
            return Forces_IPv6_NextHop_Block;

	     case FPPI_IPv6_AddPrefix:
        case FPPI_IPv6_DelPrefix:
        case FPPI_IPv6_FlushPrefix:
        case FPPI_IPv6_QueryPrefix:
            return Forces_IPv6_Prefix_Block;
#endif

#ifdef _INCLUDE_MPLS
        case FPPI_CAT_MPLS:              /* MPLS */
            return Forces_MPLS_Block;
#endif

#ifdef _INCLUDE_IPV6
        case FPPI_IPv6_AddToL2:
        case FPPI_IPv6_DelToL2:
            return Forces_IPv6_L2_Block;
#endif
#ifdef _INCLUDE_IPV6TM
	     case FPPI_IPv6TM_EnableConfig:
        case FPPI_IPv6TM_DisableConfig:
            return Forces_IPv6TM_Config_Block;
        case FPPI_IPv6TM_EnableAuto:
        case FPPI_IPv6TM_DisableAuto:
            return Forces_IPv6TM_Auto_Block;
        case FPPI_IPv6TM_Enable6to4:
        case FPPI_IPv6TM_Disable6to4:
            return Forces_IPv6TM_6to4_Block;
#endif

#ifdef _INCLUDE_DIFFSERV
	case FPPI_DS_DpeOperation:
            return Forces_DS_Dpe_Block;
#endif
        case FPPI_IPv4_UniAddToL2:
        case FPPI_IPv4_UniDelToL2:
        case FPPI_IPv4_UniFlushToL2:
            return Forces_L2_ARP_Block;
            break;
        /* etcetera */
    }
    return 0;
}

FPPI_RET BENDAPI_Report_Status(
    FPPI_Status       status,
    NPF_CBCAT         cbtype,
    FPPI_CBCORRELATOR cbcorrelator,
    void              *response_data)
{
    uint32_t connect_id;
    int size;
    BEReport rpt;

    if ((Bend_Initialized == 0) || (Bend_Ready == 0))
        return FPPI_FAIL;
	
    PIL_EnterCriticalSection(&fpp_lock);
    connect_id = ForCESGetConnectionId();
    if ( response_data == NULL )
    {
       npf_logger_write(LOG_TPI, LOG_ERR, "Null Response !! \n");
	     rpt = newBEReport(
                    connect_id,
                    cmdTypes(cbtype),
                    blockTypes(cbtype),
                    status,
                    cbcorrelator,
                    0);
            if(rpt == NULL)
            {
               npf_logger_write(LOG_TPI, LOG_ERR,
                        "ForCES Buffer Allocation failed \n");
		         return FPPI_FAIL;
            }
    }
    else
    {
    
        switch (cbtype)
        {
        case FPPI_IPv4_UniQueryPrefix:
        case FPPI_IPv4_UniQueryNextHop:
        case FPPI_IPv4_UniFlushNextHop:
        case FPPI_IPv4_UniFlushPrefix:
        case FPPI_IPv4_UniFlushToL2:
             size = sizeof_NPF_IPv4UC_CallbackData_t((NPF_IPv4UC_CallbackData_t*)response_data);
            rpt = newBEReport(
                     connect_id,
                     cmdTypes(cbtype),
                     blockTypes(cbtype),
                     status,
                     cbcorrelator,
                     size);
            if(rpt == NULL)
            {
               npf_logger_write(LOG_TPI, LOG_ERR,
                        "ForCES Buffer Allocation failed \n");
		         return FPPI_FAIL;
            }
            encapsulate_NPF_IPv4UC_CallbackData_t(
                    (NPF_IPv4UC_CallbackData_t*)response_data,
                    rpt->buf);
            break;
          
        case FPPI_IPv4_UniAddToL2:
        case FPPI_IPv4_UniDelToL2:
            size = sizeof_npf_IPv4EntryStatusList_t((npf_IPv4EntryStatusList *)response_data);
            rpt = newBEReport(
                     connect_id,
                     cmdTypes(cbtype),
                     blockTypes(cbtype),
                     status,
                     cbcorrelator,
                     size);
            if(rpt == NULL)
            {
               npf_logger_write(LOG_TPI, LOG_ERR,
                        "ForCES Buffer Allocation failed \n");
		         return FPPI_FAIL;
            }
            encapsulate_npf_IPv4EntryStatusList_t(
                        (npf_IPv4EntryStatusList *)response_data,
                        rpt->buf);
            break;
       
        case FPPI_IPv4_UniAddNextHop:
        case FPPI_IPv4_UniDelNextHop:
        case FPPI_IPv4_UniAddPrefix:
        case FPPI_IPv4_UniDelPrefix:
            size = sizeof_npf_IPv4EntryStatusList_t((npf_IPv4EntryStatusList *)response_data);
            rpt = newBEReport(
                     connect_id,
                     cmdTypes(cbtype),
                     blockTypes(cbtype),
                     status,
                     cbcorrelator,
                     size);
            if(rpt == NULL)
            {
               npf_logger_write(LOG_TPI, LOG_ERR,
                        "ForCES Buffer Allocation failed \n");
		         return FPPI_FAIL;
            }
            encapsulate_npf_IPv4EntryStatusList_t(
                        (npf_IPv4EntryStatusList *)response_data,
                        rpt->buf);
            break;
	 case FPPI_L2_GetStatistics :
            size = sizeof_npf_L2StatsList_t((npf_L2StatsList_t *)response_data);
            rpt = newBEReport(
                        connect_id,
                        cmdTypes(cbtype),
                        blockTypes(cbtype),
                        status,
                        cbcorrelator,
                        size);
            if(rpt == NULL)
            {
               npf_logger_write(LOG_TPI, LOG_ERR,
                        "ForCES Buffer Allocation failed \n");
		         return FPPI_FAIL;
            }
            encapsulate_npf_L2StatsList_t(
                        (npf_L2StatsList_t *)response_data,
                        rpt->buf);
	     break;
        case FPPI_L3_GetStatistics :
            size = sizeof_npf_L3StatsList_t((npf_L3StatsList_t *)response_data);
            rpt = newBEReport(
                        connect_id,
                        cmdTypes(cbtype),
                        blockTypes(cbtype),
                        status,
                        cbcorrelator,
                        size);
            if(rpt == NULL)
            {
               npf_logger_write(LOG_TPI, LOG_ERR,
                        "ForCES Buffer Allocation failed \n");
		         return FPPI_FAIL;
            }
            encapsulate_npf_L3StatsList_t(
                        (npf_L3StatsList_t *)response_data,
                        rpt->buf);
	     break;
        case FPPI_ICMP_GetStatistics :
            size = sizeof_npf_ICMPStatsList_t((npf_ICMPStatsList_t *)response_data);
            rpt = newBEReport(
                        connect_id,
                        cmdTypes(cbtype),
                        blockTypes(cbtype),
                        status,
                        cbcorrelator,
                        size);
            if(rpt == NULL)
            {
               npf_logger_write(LOG_TPI, LOG_ERR,
                        "ForCES Buffer Allocation failed \n");
		         return FPPI_FAIL;
            }
            encapsulate_npf_ICMPStatsList_t(
                        (npf_ICMPStatsList_t *)response_data,
                        rpt->buf);
	     break;
#ifdef _INCLUDE_IPV6
        case FPPI_IPv6_AddNextHop:
        case FPPI_IPv6_DelNextHop:
        case FPPI_IPv6_AddPrefix:
        case FPPI_IPv6_DelPrefix:
        case FPPI_IPv6_AddToL2:
        case FPPI_IPv6_DelToL2:
            size = sizeof_npf_IPv6_Status_t((npf_IPv6_Status_t *)response_data);
            rpt = newBEReport(
                        connect_id,
                        cmdTypes(cbtype),
                        blockTypes(cbtype),
                        status,
                        cbcorrelator,
                        size);
            if(rpt == NULL)
            {
               npf_logger_write(LOG_TPI, LOG_ERR,
                        "ForCES Buffer Allocation failed \n");
		         return FPPI_FAIL;
            }
            encapsulate_npf_IPv6_Status_t(
                        (npf_IPv6_Status_t *)response_data,
                        rpt->buf);
            break;
#endif

/* MPLS */
#ifdef _INCLUDE_MPLS

        case FPPI_CAT_MPLS:
            size = sizeof_FPPI_MPLS_Resp_t((FPPI_MPLS_Resp_t *)response_data);
            rpt = newBEReport(
                        connect_id,
                        cmdTypes(cbtype),
                        blockTypes(cbtype),
                        status,
                        cbcorrelator,
                        size);
            if(rpt == NULL)
            {
               npf_logger_write(LOG_TPI, LOG_ERR,
                        "ForCES Buffer Allocation failed \n");
		         return FPPI_FAIL;
            }
            encapsulate_FPPI_MPLS_Resp_t(
                        (FPPI_MPLS_Resp_t *)response_data,
                        rpt->buf);
            break;

#endif /* MPLS */

#ifdef _INCLUDE_DIFFSERV
	case FPPI_DS_DpeOperation:
            size = sizeof_FPPI_DS_Dpe_t((FPPI_DS_Dpe_t *)response_data);
	    /*
	     * To overcome memory alignment in vxworks all paramters 
	     * encoded/decoded as uint32_t. Allocate more buffer size
	    */
	    size = size*2;
            rpt = newBEReport(
                        connect_id,
                        cmdTypes(cbtype),
                        blockTypes(cbtype),
                        status,
                        cbcorrelator,
                        size);
            if(rpt == NULL)
            {
               npf_logger_write(LOG_TPI, LOG_ERR,
                        "ForCES Buffer Allocation failed \n");
		         return FPPI_FAIL;
            }
            encapsulate_FPPI_DS_Dpe_t(
                        (FPPI_DS_Dpe_t *)response_data,
                        rpt->buf);
            break;
#endif /* DIFFSERV */

        default:
           /* default is for all types that only send back the status */
            rpt = newBEReport(connect_id, Forces_NULL_Cmd, 0, status, cbcorrelator, 0);
            if(rpt == NULL)
            {
               npf_logger_write(LOG_TPI, LOG_ERR,
                        "ForCES Buffer Allocation failed \n");
		         return FPPI_FAIL;
            }
	    break; 

       } /* switch */
    }
    BEReportSend(rpt);

    PIL_LeaveCriticalSection(&fpp_lock);

    return FPPI_SUCCESS;
}


/* ATM */
#ifdef _INCLUDE_ATM
FPPI_RET BENDAPI_sendATMEvent(
        NPF_CBCAT       event_type,
        void*           event_data)
{
	uint32_t connect_id;
	void *data;
	char *buf;
	int size;
	int length;
	uint32_t forcesevent;
	uint32_t blocktype;
	uint32_t blockhandle;

	if((Bend_Initialized == 0) || (Bend_Ready == 0))
		return FPPI_FAIL;

	connect_id = ForCESGetConnectionId();

	PIL_EnterCriticalSection(&fpp_lock);

	switch(event_type)
	{
		case FPPI_ATM_OAM_AisAlarm_Event:
			forcesevent = FORCES_EVENT_ATM_OAM_AIS_ALARM;
			blocktype = blockhandle = 0;
			length = sizeof(NPF_ATM_EventData_t);
			data = event_data;
			break;

		case FPPI_ATM_OAM_RdiAlarm_Event:
			forcesevent = FORCES_EVENT_ATM_OAM_RDI_ALARM;
			blocktype = blockhandle = 0;
			length = sizeof(NPF_ATM_EventData_t);
			data = event_data;
			break;

		case FPPI_ATM_OAM_Lb_Event:
			forcesevent = FORCES_EVENT_ATM_OAM_LB;
			blocktype = blockhandle = 0;
			length = sizeof(NPF_ATM_EventData_t);
			data = event_data;
			break;

		case FPPI_ATM_OAM_FpmStats_Event:
			forcesevent = FORCES_EVENT_ATM_OAM_FPM_STATS;
			blocktype = blockhandle = 0;
			length = sizeof(NPF_ATM_EventData_t);
			data = event_data;
			break;

		case FPPI_ATM_OAM_BrStats_Event:
			forcesevent = FORCES_EVENT_ATM_OAM_BR_STATS;
			blocktype = blockhandle = 0;
			length = sizeof(NPF_ATM_EventData_t);
			data = event_data;
			break;

		case FPPI_ATM_VccUp_Event :
			forcesevent = FORCES_EVENT_ATM_VCC_UP_EVENT;
			blocktype = blockhandle = 0;
			length = sizeof(NPF_ATM_EventData_t);
			data = event_data;
			break;

		case FPPI_ATM_VccDown_Event :
			forcesevent = FORCES_EVENT_ATM_VCC_DOWN_EVENT;
			blocktype = blockhandle = 0;
			length = sizeof(NPF_ATM_EventData_t);
			data = event_data;
			break;

		default :
			break;

	} /* end of switch */

	/* Encapsulate the event information */
	ForCES_encapsulate_Event(connect_id,forcesevent,blocktype,blockhandle,length,
									data, &buf, &size);
	
	/* Send the FORCES message */
	ForCESMessageSend(connect_id, (ForCESMessage)buf, size);

	PIL_LeaveCriticalSection(&fpp_lock);
	return FPPI_SUCCESS;

}

FPPI_RET BENDAPI_Report_ATMStatus(
    FPPI_Status       status,
    NPF_CBCAT         cbtype,
    FPPI_CBCORRELATOR cbcorrelator,
    void              *response_data)
{

    uint32_t connect_id;
    int size;
    BEReport rpt;

    if ((Bend_Initialized == 0) || (Bend_Ready == 0))
        return FPPI_FAIL;
    PIL_EnterCriticalSection(&fpp_lock);

    connect_id = ForCESGetConnectionId();

    switch (cbtype)
    {
      case FPPI_ATM_IfAttrSet: 
      case FPPI_ATM_IfDelete: 
      case FPPI_ATM_IfEnable: 
      case FPPI_ATM_IfDisable: 
      case FPPI_ATM_VccSet: 
      case FPPI_ATM_VccBind: 
      case FPPI_ATM_VccDelete: 
      case FPPI_ATM_VccEnable: 
      case FPPI_ATM_VccDisable: 
      case FPPI_ATM_VccOperStatusGet: 
      case FPPI_ATM_AAL2_ChannelSet: 
      case FPPI_ATM_AAL2_ChannelDel: 
      case FPPI_ATM_ConnectionXcSet: 
      case FPPI_ATM_ConnectionXcDel: 
      case FPPI_ATM_OAM_CP_Set: 
      case FPPI_ATM_OAM_CC_Set: 
      case FPPI_ATM_OAM_LB_Set: 
      case FPPI_ATM_OAM_Monitor: 
      case FPPI_ATM_OAM_AlarmSet: 
      case FPPI_ATM_OAM_AlarmClear: 
      case FPPI_ATM_OAM_PM_Set: 
         if (response_data != NULL)
         {
             size = sizeof_IfATM_FpResponse_t((IfATM_FpResponse_t *)response_data);
             rpt = newBEReport(
                        connect_id,
                        cmdTypes(cbtype),
                        blockTypes(cbtype),
                        status,
                        cbcorrelator,
                        size);
            if(rpt == NULL)
            {
               npf_logger_write(LOG_TPI, LOG_ERR,
                        "ForCES Buffer Allocation failed \n");
		         return FPPI_FAIL;
            }
             encapsulate_IfATM_FpResponse_t((IfATM_FpResponse_t *)response_data,rpt->buf);
             BEReportSend(rpt);
         }
	      break;

      case FPPI_ATM_VccStatsGet: 
         if (response_data != NULL)
         {
             size = sizeof_IfATM_VccStats_t((IfATM_VccStats_t *)response_data);
             rpt = newBEReport(
                        connect_id,
                        cmdTypes(cbtype),
                        blockTypes(cbtype),
                        status,
                        cbcorrelator,
                        size);
            if(rpt == NULL)
            {
               npf_logger_write(LOG_TPI, LOG_ERR,
                        "ForCES Buffer Allocation failed \n");
		         return FPPI_FAIL;
            }
             encapsulate_IfATM_VccStats_t((IfATM_VccStats_t *)response_data,rpt->buf);
             BEReportSend(rpt);
         }
		   break;
      case FPPI_ATM_IfGenStatsGet: 
        if (response_data != NULL)
        {
             size = sizeof_IfATM_IfStatistics_t((IfATM_IfStatistics_t *)response_data);
             rpt = newBEReport(
                        connect_id,
                        cmdTypes(cbtype),
                        blockTypes(cbtype),
                        status,
                        cbcorrelator,
                        size);
            if(rpt == NULL)
            {
               npf_logger_write(LOG_TPI, LOG_ERR,
                        "ForCES Buffer Allocation failed \n");
		         return FPPI_FAIL;
            }
             encapsulate_IfATM_IfStatistics_t((IfATM_IfStatistics_t *)response_data,rpt->buf);
             BEReportSend(rpt);
        }
		  break;

        default:
         /* default is for all types that only send back the status */
        {
            rpt = newBEReport(connect_id, Forces_NULL_Cmd, 0, status, cbcorrelator, 0);
            if(rpt == NULL)
            {
               npf_logger_write(LOG_TPI, LOG_ERR,
                        "ForCES Buffer Allocation failed \n");
		         return FPPI_FAIL;
            }
            BEReportSend(rpt);
        }
        break;

    }/* end of switch */
    PIL_LeaveCriticalSection(&fpp_lock);
    return FPPI_SUCCESS;

}
#endif /* _INCLUDE_ATM */


