/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2003 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ForwardingPlane/TransportPlugin/src/bcallback.c,v $
 * $Revision: 1.6 $
 * $Date: 2003/11/17 16:57:49 $
 *
 * Description:
 * This file implements the callback interface defined in pdatagram.h,
 * it uses the decapsulate functions implemented in fppdecap.cc 
 *
 ****************************************************************************/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "forces_Types.h"
#include "backendapi.h"
#include "fpp_common.h"
#include "pdk_callback.h"
#ifdef FPPI_TIMING
#include "sys/time.h"
#endif
#include "bpriv.h"
#include "log.h"
#include "time.h"

void ForCES_DatagramException (
        uint32_t ex,
        uint32_t connect_id)
{

}

/****************************************************************************
 * Function: ForCES_DatagramReceive()                                       *
 * ---------------------------                                              *
 *                                                                          *
 * Description: This is a callback interface fn registered with
 *              the Interconnect which is invoked whenever messages come
 *              in from the CE
 *                                                                          *    * Return: none                                                             *
 *                                                                          *
 ****************************************************************************/

void ForCES_DatagramReceive (void* buf, uint32_t connect_id, uint32_t len)
{
    ForCESMessage msg;
    forces_header_t *header;

    msg = (ForCESMessage)buf;
    header = &(msg->header);

    if (!ForCESCheckVersion(header->version))
    {
        npf_logger_write (LOG_TPI, LOG_ERR,
            "Invalid message from FE.  Unsupported version %d\n",
	    header->version);
        deleteForCESMessage(msg);
	return;
    }

// Call ForcesMessage;
   if (ForCES_MessageReceive (buf,connect_id,len))
   {
   npf_logger_write (LOG_TPI, LOG_INFO, "deleting recv buffer\n");
    	deleteForCESMessage(msg);
   }
   npf_logger_write (LOG_TPI, LOG_INFO, "Finish one job from CP\n");
}

int ForCES_Capability_Response_t(
    	FPPI_FE_Caps*   fe_caps,
       FPPI_CBHANDLE   cbhandle,
       uint32_t 	   connectionid,
       char**		   buf,
       int*		   size)
{
    ForCESMessage msg;
  
    *size = sizeof_Capability_Response_t(fe_caps);
    msg = newForCESMessage(connectionid, Capability_Response, cbhandle, *size);
    encapsulate_Capability_Response_t(fe_caps, msg->data);
    *buf = (char *)msg;
    return 0;
}

int ForCES_encapsulate_FECaps(
    	FPPI_FE_Caps*   fe_caps,
       FPPI_CBHANDLE   cbhandle,
       uint32_t 	   connectionid,
       char**		   buf,
       int*		   size)
{
    ForCESMessage msg;
  
    *size = sizeof_FPPI_FE_Caps_t(fe_caps);
    msg = newForCESMessage(connectionid, Config_Query_Response, cbhandle, *size);
    encapsulate_FPPI_FE_Caps_t(fe_caps, msg->data);
    *buf = (char *)msg;
    return 0;
}

int ForCES_Store_FECaps( 
	FPPI_FE_Caps*   fe_caps,
       FPPI_CBHANDLE   cbhandle)
{
    int i;
    if (g_BEND_cb_info.fe_caps_info.fe_caps==NULL)
    {
        g_BEND_cb_info.fe_caps_info.fe_caps = (FPPI_FE_Caps *)malloc (sizeof (FPPI_FE_Caps));
	 g_BEND_cb_info.fe_caps_info.fe_caps->ports_caps.port_caps_list = (FPPI_Port_Caps *)malloc(fe_caps->ports_caps.port_count*sizeof (FPPI_Port_Caps));
    }
    // start copying everything
    memcpy (g_BEND_cb_info.fe_caps_info.fe_caps->fe_name, fe_caps->fe_name, FENAMELEN);
    g_BEND_cb_info.fe_caps_info.fe_caps->ports_caps.port_count  = fe_caps->ports_caps.port_count;
    for (i=0 ; i < fe_caps->ports_caps.port_count; i++)
    {
        g_BEND_cb_info.fe_caps_info.fe_caps->ports_caps.port_caps_list[i].portid
		=fe_caps->ports_caps.port_caps_list[i].portid;
        g_BEND_cb_info.fe_caps_info.fe_caps->ports_caps.port_caps_list[i].port_type
		=fe_caps->ports_caps.port_caps_list[i].port_type;
        memcpy ( g_BEND_cb_info.fe_caps_info.fe_caps->ports_caps.port_caps_list[i].mac_addr,
		fe_caps->ports_caps.port_caps_list[i].mac_addr, MACADDRLEN);
        g_BEND_cb_info.fe_caps_info.fe_caps->ports_caps.port_caps_list[i].link_speed
		=fe_caps->ports_caps.port_caps_list[i].link_speed;
        g_BEND_cb_info.fe_caps_info.fe_caps->ports_caps.port_caps_list[i].mtu
		=fe_caps->ports_caps.port_caps_list[i].mtu;
    }
    return 1;
}


int ForCES_encapsulate_Event(
       uint32_t 	   connectionid,
       uint32_t event, 
       uint32_t block_type, 
       uint32_t block_handle, 
       uint32_t 	   length,
       void* 		   event_data,
       char**		   buf,
       int*		   size)
{
    ForCESMessage msg;
  
    *size = sizeof_FE_event_t(length);
    msg = newForCESMessage(connectionid, FE_Event_Notification, 0, *size);
    encapsulate_FE_event_t(event, block_type, block_handle, length, event_data, msg->data);
    *buf = (char *)msg;
    return 0;
}       

int ForCES_encapsulate_PacketRedirect(
       uint32_t 	   connectionid,
       uint32_t         block_handle, 
       npf_Packet_t     *packet_list,
       char**		   buf,
       int*		   size)
{
    ForCESMessage msg;
  
    *size = sizeof_PacketRedirection(packet_list);
    msg = newForCESMessage(connectionid, FE_Event_Notification, 0, *size);
    encapsulate_PacketRedirection(block_handle, packet_list, msg->data);
    *buf = (char *)msg;
    return 0;
}       

int ForCES_encapsulate_Report_Status(
       uint32_t 	   connectionid,
       NPF_CBCAT	   cbtype,
       uint32_t 	   corr,
       uint32_t         block_handle, 
       void             *response_data,
       char**		   buf,
       int*		   size)
{
    ForCESMessage msg;
  
    *size = sizeof_encapsulate_Report_Status(response_data);
    msg = newForCESMessage(connectionid, Config_Query_Response, corr, *size);
    encapsulate_Report_Status(block_handle,cbtype, response_data, msg->data);
    *buf = (char *)msg;
    return 0;

}

int Forces_Cap_Req(
	ForCESMessage msg,
	uint32_t connect_id,
	int length)
{
    char *buf;
    int size;
    if (g_BEND_cb_info.fe_caps_info.fe_caps)
    {
        ForCES_Capability_Response_t( g_BEND_cb_info.fe_caps_info.fe_caps,
					g_BEND_cb_info.fe_caps_info.cbhandle,
					connect_id, &buf, &size);
        ForCESMessageSend(connect_id, (ForCESMessage)buf, size);
	 g_BEND_cb_info.fe_caps_info.Caps_req = 0;
	 return 1;
    }
    else
    {
	 g_BEND_cb_info.fe_caps_info.Caps_req = 1;
	 return 1;
    }	
}

int Forces_FeCap_Req(
	ForCESMessage msg,
	uint32_t connect_id,
	int length)
{
    char *buf;
    int size;
    ForCESMessage rsp;
    if (g_BEND_cb_info.fe_caps_info.fe_caps)
    {
printf("Sending  Forces_FeCap_Req\n");
        ForCES_encapsulate_FECaps( g_BEND_cb_info.fe_caps_info.fe_caps,
					g_BEND_cb_info.fe_caps_info.cbhandle,
					connect_id, &buf, &size);
	 rsp = (ForCESMessage)buf;
	 rsp->header.cmd_correlator =  msg->header.cmd_correlator; 				
        ForCESMessageSend(connect_id, rsp, size);
	 return 1;
    }
    else
    {
printf("NOT Sending  Forces_FeCap_Req : error?\n");
        // Return ERROR
	 return 1;
    }	
}

int ForCES_Unbind(uint32_t connectionid)
{
    ForCESMessage msg;
    if (connectionid == 0)
	return 1;
    msg = newForCESMessage(connectionid, Unbind,0,0);
    ForCESMessageSend(connectionid , msg, 0);
    return 1;
}

