/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ForwardingPlane/TransportPlugin/src/forces.c,v $
 * $Revision: 1.6 $
 * $Date: 2003/11/17 16:57:50 $
 *
 * Description:
 * Contains the utility functions for maintaining ForCESMessage's
 *
 ****************************************************************************/

#include <unistd.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>
#include "backendapi.h"
#include "forces_Types.h"
#include "bpriv.h"
#include "log.h"
#ifdef _VXWORKS_
#include "ioLib.h"
#include "pipeDrv.h"
#endif

BEND_cb_info g_BEND_cb_info;

#define MAX_MISSED_BEATS_ALLOWED 3

PilThread g_BEND_HeartbeatThreadId = 0;
static int g_BEND_missedbeats;
static int g_BEND_msgpipe[2];

uint32_t BEND_HeartbeatThread(void* arg)
{
    int retval;
    struct timeval tv; 
    fd_set rfds;
    ForCESMessage msg;
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Entering BEND_HeartbeatThread\n");
    while (1)
    {
        tv.tv_sec = (int)arg;
        tv.tv_usec = 0;

        FD_ZERO(&rfds);
        FD_SET(g_BEND_msgpipe[0],&rfds);
    
        retval = select(g_BEND_msgpipe[0]+1,&rfds, NULL,NULL,&tv);
        if (retval)
        {
            break;
        }
        else
        {
            g_BEND_missedbeats++;
            if (g_BEND_missedbeats > MAX_MISSED_BEATS_ALLOWED)
            {
                npf_logger_write(LOG_TPI, LOG_INFO, "Missed more than %d heart beats\n",MAX_MISSED_BEATS_ALLOWED);
                BENDAPI_unbindRequest();
                return 0;
            }
            npf_logger_write(LOG_TPI, LOG_INFO, "Sending heart beat\n");
            msg = newForCESMessage(ForCESGetConnectionId(), Heartbeat, 0, 0);
            if (!msg)
            {
                continue; 
            }
            ForCESMessageSend(ForCESGetConnectionId(),msg,0);
        }
    }

    return 0;
}

void ForCESStartHeartbeat(int heartbeat_interval)
{
#ifdef _VXWORKS_
    pipeDevCreate("/pipe/tp/forces", 10, 1);
    g_BEND_msgpipe[0] = g_BEND_msgpipe[1] = open("/pipe/tp/forces", O_RDWR, 0666);
#else
    pipe(g_BEND_msgpipe);
#endif
    g_BEND_missedbeats = 0;
    g_BEND_HeartbeatThreadId = PIL_CreateThread(BEND_HeartbeatThread,0,"BEND_HeartbeatThread",0,(void*)heartbeat_interval,0);
}

void ForCESGotHeartbeat()
{
    npf_logger_write(LOG_TPI, LOG_INFO, "Received heart beat\n");
    g_BEND_missedbeats = 0;
}

void ForCESStopHeartbeat()
{
    if (!g_BEND_HeartbeatThreadId) return;

    npf_logger_write(LOG_TPI, LOG_INFO, "Shutting down heart beat thread\n");
    write(g_BEND_msgpipe[1],"exit",5);
    PIL_WaitForThread(g_BEND_HeartbeatThreadId,500); /* dont wait forever */
    PIL_DestroyThread(g_BEND_HeartbeatThreadId,NULL);
    close(g_BEND_msgpipe[0]);
#ifndef _VXWORKS_
    close(g_BEND_msgpipe[1]);
#endif
    g_BEND_HeartbeatThreadId = 0;
}

int ForCESGetConnectionId()
{
	return (g_BEND_cb_info.init)? g_BEND_cb_info.connect_id: 0 ;
}

int ForCESReceivedCapsRequest()
{
	return g_BEND_cb_info.fe_caps_info.Caps_req ;
}

ForCESMessage newForCESMessage(uint32_t connectionid, uint16_t msg_type,
		uint32_t correlator, size_t size)
{
    ForCESMessage msg;
    void* pbuf;

    if (PL_SUCCESS != pb_getBuffer(connectionid,sizeof(forces_header_t)+size,0,&pbuf))
    {
        return 0;
    }
    msg = (ForCESMessage)pbuf;
    msg->header.version = FORCES_VERSION;
    msg->header.flags = 0;
    msg->header.msg_type = htons(msg_type);
    msg->header.cmd_correlator = correlator;

    return msg;
}

void deleteForCESMessage(ForCESMessage msg)
{
    pb_freeBuffer((void*)msg);
}

void ForCESMessageSend(uint32_t connectionid, ForCESMessage msg, uint32_t size)
{
    SendDatagram(connectionid,(void*)msg,sizeof(forces_header_t)+size,0);
}

int ForCESCheckVersion(uint8_t version)
{
    return (version <= FORCES_VERSION);
}

int ForCES_InitInterconnect (char *ipaddr)
{
	g_BEND_cb_info.clientparams.connect_info.flags = PL_DATAGRAM_IPV4;
	inet_aton (ipaddr, &(	g_BEND_cb_info.clientparams.connect_info.saddr.ipv4_addr.sin_addr));
	g_BEND_cb_info.clientparams.connect_info.saddr.ipv4_addr.sin_port = PL_DATAGRAM_SERVER_PORT;
	
    g_BEND_cb_info.clientparams.datagram_receive = ForCES_DatagramReceive;
    g_BEND_cb_info.clientparams.datagram_exception = ForCES_DatagramException;
	if ( pb_PlBufferInit (QUEUE_SIZE))
		return 1;
	if ( InitializeDatagramClient (&g_BEND_cb_info.clientparams, &g_BEND_cb_info.connect_id))
		return 1;
	g_BEND_cb_info.init = 1;
	return 0;
}

int ForCES_DisconnectInterconnect ()
{
    ForCESStopHeartbeat();

    if ( pb_PlBufferDeInit ())
	return 1;
    if ( DeInitializeDatagram ())
	return 1;
    g_BEND_cb_info.init = 1;
    if (g_BEND_cb_info.fe_caps_info.fe_caps)
    {
        if (g_BEND_cb_info.fe_caps_info.fe_caps->ports_caps.port_caps_list)
	 {
            free(g_BEND_cb_info.fe_caps_info.fe_caps->ports_caps.port_caps_list);
	 }
        free(g_BEND_cb_info.fe_caps_info.fe_caps);
    }
    return 0;
}

