/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ForwardingPlane/TransportPlugin/src/funcp.c,v $
 * $Revision: 1.7 $
 * $Date: 2003/11/17 16:57:50 $
 *
 * Description:
 *
 ****************************************************************************/
#include <string.h>
#include <stdlib.h>
#include <assert.h>

#include "backendapi.h"
#include "fpp_common.h"
#include "forces_Types.h"
#include "fppCapabilityTypes.h"
#include "bpriv.h"
#include "log.h"
#include "funcp.h"
#include "fppencap_forces.h"
#include "fppdecap_forces.h"
#include "bedecap_api_types.h"
#include "fppdecap_capability_types.h"
#include "fppdecap_npf.h"
#include "fppdecap_common.h"
#include "fppencap_capability_types.h"
#include "fppdecap_npf_if.h"
#include "fppencap_npf_if.h"

#include "fppdecap_npf_ipv4.h"
#include "fppencap_npf_ipv4.h"

#ifdef _INCLUDE_MPLS
#include "fppapi_mpls_types.h"
#include "fppdecap_mpls.h"
#include "fppencap_mpls.h"
#endif  /* MPLS */

#ifdef _INCLUDE_IPV6
#include "fppdecap_npf_ipv6.h"
#include "fppencap_npf_ipv6.h"
#endif  /* IPV6 */

#ifdef _INCLUDE_DIFFSERV
#include "fppdecap_npf_ds.h"
#include "fppencap_npf_ds.h"
#endif /* DS */

#ifdef _INCLUDE_ATM
#include "pdk_cm_types.h"
#include "npf_atm_types.h"
#include "fppapi_atm_types.h"
#include "fppdecap_npf_atm.h"
#include "fppencap_npf_atm.h"
#endif /* _INCLUDE_ATM */

forCES_msg_func_ptr msgfp[] = 
{
    { FE_Bind_Request,			FPPI_INVALID_CBTYPE,  FE_Bind_Request_msgfp},
    { FE_Bind_Response,	    		FPPI_FEBind_Event,  FE_Bind_Response_msgfp},
    { Capability_Request,	    	FPPI_INVALID_CBTYPE,  Capability_Request_msgfp},
    { Capability_Response,	    	FPPI_INVALID_CBTYPE,  Capability_Response_msgfp},
    { Topology_Request,	    		FPPI_INVALID_CBTYPE,  Topology_Request_msgfp},
    { Topology_Response,	    	FPPI_INVALID_CBTYPE,  Topology_Response_msgfp},
    { FE_Start_Operation,	    	FPPI_INVALID_CBTYPE,  FE_Start_Operation_msgfp },
    { Config_Query_Request,	    	FPPI_INVALID_CBTYPE,  Config_Query_Request_msgfp },
    { Config_Query_Response,    	FPPI_INVALID_CBTYPE,  Config_Query_Response_msgfp },
    { FE_Event_Notification,    	FPPI_INVALID_CBTYPE,  FE_Event_Notification_msgfp },
    { Unbind,			    	FPPI_INVALID_CBTYPE,  Unbind_msgfp },
    { Heartbeat,		    	FPPI_INVALID_CBTYPE,  Heartbeat_msgfp }
};

forCES_evnt_cmd_ptr cmdfp[] = 
{
    { Forces_NULL_Cmd,	    		FPPI_INVALID_CBTYPE,  Forces_NULL_Cmd_msgfp},
    { Forces_Add_Cmd,		    	FPPI_INVALID_CBTYPE,  Forces_Add_Cmd_msgfp},
    { Forces_Update_Cmd,                FPPI_INVALID_CBTYPE,  Forces_Update_Cmd_msgfp},
    { Forces_Delete_Cmd,	    	FPPI_INVALID_CBTYPE,  Forces_Delete_Cmd_msgfp},
    { Forces_DeleteAll_Cmd,	    	FPPI_INVALID_CBTYPE,  Forces_DeleteAll_Cmd_msgfp},
    { Forces_SendPacket_Cmd,    	FPPI_INVALID_CBTYPE,  Forces_SendPacket_Cmd_msgfp},
    { Forces_QueryStats_Cmd,    	FPPI_INVALID_CBTYPE,  Forces_QueryStats_Cmd_msgfp},
    { Forces_QueryProps_Cmd,    	FPPI_INVALID_CBTYPE,  Forces_QueryProps_Cmd_msgfp }
};

forCES_evnt_cmd_ptr evntfp[] = 
{
   { Forces_Port_Event, 	   	FPPI_INVALID_CBTYPE,  Forces_Port_Event_msgfp},
   { Forces_BlockSpecific_Event, 	FPPI_INVALID_CBTYPE,  Forces_BlockSpecific_Event_msgfp},
   { Forces_PacketRedirect_Event, 	FPPI_INVALID_CBTYPE,  Forces_PacketRedirect_Event_msgfp},
   { Forces_Capability_Event, 		FPPI_INVALID_CBTYPE,  Forces_Capability_Event_msgfp}
};

/*Adding New objects make sure that the objects are in the exact same order as ForceBlockEnum */
forCES_evnt_cmd_ptr blkfp[] = 
{
   { Forces_Ingress_Intf_Block, 	FPPI_INVALID_CBTYPE,  Forces_Ingress_Intf_Block_msgfp},
   { Forces_Egress_Intf_Block, 		FPPI_INVALID_CBTYPE,  Forces_Egress_Intf_Block_msgfp},
  /* { Forces_IPv4_Block, 		FPPI_INVALID_CBTYPE,  Forces_IPv4_Block_msgfp}, */
   { Forces_IPv4_Prefix_Block, 		FPPI_INVALID_CBTYPE,  Forces_IPv4_Prefix_Block_msgfp},
   { Forces_IPv4_NextHop_Block, 	FPPI_INVALID_CBTYPE,  Forces_IPv4_NextHop_Block_msgfp},
   { Forces_L2_ARP_Block, 		FPPI_INVALID_CBTYPE,  Forces_L2_ARP_Block_msgfp},
   { Forces_L2_Interface_Block, 	FPPI_INVALID_CBTYPE,  Forces_L2_Interface_Block_msgfp},
   { Forces_L3_Interface_Block, 	FPPI_INVALID_CBTYPE,  Forces_L3_Interface_Block_msgfp},
   { Forces_ICMP_Block, 		FPPI_INVALID_CBTYPE,  Forces_ICMP_Block_msgfp},
#ifdef _INCLUDE_IPV6
   { Forces_L3_IPv6_Interface_Block, 	FPPI_INVALID_CBTYPE,  Forces_L3_IPv6_Interface_Block_msgfp},
   { Forces_IPv6_Block, 		FPPI_INVALID_CBTYPE,  Forces_IPv6_Block_msgfp},
   { Forces_IPv6_Prefix_Block, 		FPPI_INVALID_CBTYPE,  Forces_IPv6_Prefix_Block_msgfp},
   { Forces_IPv6_NextHop_Block, 	FPPI_INVALID_CBTYPE,  Forces_IPv6_NextHop_Block_msgfp},
   { Forces_IPv6_L2_Block, 	FPPI_INVALID_CBTYPE,  Forces_IPv6_L2_Block_msgfp},
#endif /* IPV6 */
#ifdef _INCLUDE_IPV6TM
   { Forces_IPv6TM_Block, 		FPPI_INVALID_CBTYPE,  Forces_IPv6TM_Block_msgfp},
   { Forces_IPv6TM_Auto_Block, 		FPPI_INVALID_CBTYPE,  Forces_IPv6TM_Auto_Block_msgfp},
   { Forces_IPv6TM_Config_Block, 	FPPI_INVALID_CBTYPE,  Forces_IPv6TM_Config_Block_msgfp},
   { Forces_IPv6TM_6to4_Block, 	FPPI_INVALID_CBTYPE,  Forces_IPv6TM_6to4_Block_msgfp},
#endif /* IPV6TM */

#ifdef _INCLUDE_MPLS
   { Forces_MPLS_Block,		FPPI_INVALID_CBTYPE,  Forces_MPLS_Block_msgfp},
#endif  /* MPLS */

#ifdef _INCLUDE_ATM
   { Forces_IfATM_IfAttrSet_Block,    FPPI_INVALID_CBTYPE,  Forces_IfATM_IfAttrSet_Block_msgfp},
   { Forces_IfATM_IfDelete_Block, FPPI_INVALID_CBTYPE, Forces_IfATM_IfDelete_Block_msgfp}, 
   { Forces_IfATM_IfEnable_Block, FPPI_INVALID_CBTYPE,Forces_IfATM_IfEnable_Block_msgfp}, 
   { Forces_IfATM_IfDisable_Block, FPPI_INVALID_CBTYPE, Forces_IfATM_IfDisable_Block_msgfp}, 
   { Forces_IfATM_IfGenStatsGet_Block, FPPI_INVALID_CBTYPE, Forces_IfATM_IfGenStatsGet_Block_msgfp}, 
   { Forces_IfATM_VccSet_Block, FPPI_INVALID_CBTYPE, Forces_IfATM_VccSet_Block_msgfp}, 
   { Forces_IfATM_VccBind_Block, FPPI_INVALID_CBTYPE, Forces_IfATM_VccBind_Block_msgfp}, 
   { Forces_IfATM_VccDelete_Block, FPPI_INVALID_CBTYPE, Forces_IfATM_VccDelete_Block_msgfp}, 
   { Forces_IfATM_VccEnable_Block, FPPI_INVALID_CBTYPE, Forces_IfATM_VccEnable_Block_msgfp}, 
   { Forces_IfATM_VccDisable_Block, FPPI_INVALID_CBTYPE, Forces_IfATM_VccDisable_Block_msgfp}, 
   { Forces_IfATM_VccOperStatusGet_Block, FPPI_INVALID_CBTYPE, Forces_IfATM_VccOperStatusGet_Block_msgfp }, 
   { Forces_IfATM_VccStatsGet_Block, FPPI_INVALID_CBTYPE, Forces_IfATM_VccStatsGet_Block_msgfp}, 
   { Forces_IfATM_AAL2_ChannelSet_Block, FPPI_INVALID_CBTYPE, Forces_IfATM_AAL2_ChannelSet_Block_msgfp}, 
   { Forces_IfATM_AAL2_ChannelDel_Block, FPPI_INVALID_CBTYPE, Forces_IfATM_AAL2_ChannelDel_Block_msgfp}, 
   { Forces_IfATM_ConnectionXcSet_Block, FPPI_INVALID_CBTYPE, Forces_IfATM_ConnectionXcSet_Block_msgfp}, 
   { Forces_IfATM_ConnectionXcDel_Block, FPPI_INVALID_CBTYPE, Forces_IfATM_ConnectionXcDel_Block_msgfp}, 
   { Forces_IfATM_OAM_CP_Set_Block, FPPI_INVALID_CBTYPE, Forces_IfATM_OAM_CP_Set_Block_msgfp}, 
   { Forces_IfATM_OAM_CC_Set_Block, FPPI_INVALID_CBTYPE, Forces_IfATM_OAM_CC_Set_Block_msgfp}, 
   { Forces_IfATM_OAM_LB_Set_Block, FPPI_INVALID_CBTYPE, Forces_IfATM_OAM_LB_Set_Block_msgfp}, 
   { Forces_IfATM_OAM_Monitor_Block, FPPI_INVALID_CBTYPE, Forces_IfATM_OAM_Monitor_Block_msgfp }, 
   { Forces_IfATM_OAM_AlarmSet_Block, FPPI_INVALID_CBTYPE, Forces_IfATM_OAM_AlarmSet_Block_msgfp }, 
   { Forces_IfATM_OAM_AlarmClear_Block, FPPI_INVALID_CBTYPE, Forces_IfATM_OAM_AlarmClear_Block_msgfp }, 
   { Forces_IfATM_OAM_PM_Set_Block, FPPI_INVALID_CBTYPE, Forces_IfATM_OAM_PM_Set_Block_msgfp }, 
#endif /* ATM */

#ifdef _INCLUDE_DIFFSERV
   { Forces_DS_Dpe_Block, FPPI_INVALID_CBTYPE,Forces_DS_Dpe_Block_msgfp }, 
#endif /* DIFFSERV */
   { -1, 	FPPI_INVALID_CBTYPE,  NULL}
};

extern int Forces_Cap_Req( ForCESMessage msg, uint32_t connect_id, int length);
extern  void ForCESStopHeartbeat(void);
extern  void ForCESGotHeartbeat(void);


/* Message handlers */
uint32_t FE_Bind_Request_msgfp (ForCESMessage msg ,
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    // char *buf = msg->data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    
    npf_logger_write(LOG_TPI, LOG_INFO, "FE_Bind_Request\n");

    
    // set pobj & pcbtype
    *pobj = retobj;	// Maybe allocated to be passed into callback handler and freed later
    *pcbtype = msgfp[FE_Bind_Request - MSG_OFFSET].cb_type;
    return retval;
}

uint32_t FE_Bind_Response_msgfp (ForCESMessage msg ,
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    char *buf = msg->data;
    int retval = FUNC_SUCCESS;
    // fill declare
    forces_bind_resp_t bind_r;
    FPPI_Status* status;
    
    npf_logger_write(LOG_TPI, LOG_INFO, "FE_Bind_Response\n");

    /* Check if this handler code is the right way to go*/
    buf = decapsulate_forces_bind_resp_t (&bind_r, buf);
    
    // set pobj & pcbtype
    status = PIL_AllocateMemory(sizeof(FPPI_Status),0);
    *status = FPPI_SUCCESSFUL;
    *pobj = status;	// Maybe allocated to be passed into callback handler and freed later
    *pcbtype = FPPI_FEBind_Event;

    return retval;
}

uint32_t Capability_Request_msgfp (ForCESMessage msg ,
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    // char *buf = msg->data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Capability_Request\n");

    retval = Forces_Cap_Req(msg,connectionid,length);
    
    // set pobj & pcbtype
    *pobj = retobj;	// Maybe allocated to be passed into callback handler and freed later
    *pcbtype = msgfp[ Capability_Request - MSG_OFFSET].cb_type;
    return retval;
}

uint32_t Capability_Response_msgfp (ForCESMessage msg ,
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    // char *buf = msg->data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Capability_Response\n");

    npf_logger_write(LOG_TPI, LOG_INFO, "Capability_Response : Not sure what todo with this on FE\n");

    // set pobj & pcbtype
    *pobj = retobj;	// Maybe allocated to be passed into callback handler and freed later
    *pcbtype = msgfp[Capability_Response - MSG_OFFSET].cb_type;
    return retval;
}

uint32_t Topology_Request_msgfp (ForCESMessage msg ,
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    // char *buf = msg->data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Topology_Request\n");

    
    // set pobj & pcbtype
    *pobj = retobj;	// Maybe allocated to be passed into callback handler and freed later
    *pcbtype = msgfp[Topology_Request - MSG_OFFSET].cb_type;
    return retval;
}

uint32_t Topology_Response_msgfp (ForCESMessage msg ,
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    // char *buf = msg->data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Topology_Response\n");

    
    // set pobj & pcbtype
    *pobj = retobj;	// Maybe allocated to be passed into callback handler and freed later
    *pcbtype = msgfp[Topology_Response - MSG_OFFSET].cb_type;
    return retval;
}

uint32_t FE_Start_Operation_msgfp (ForCESMessage msg ,
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    // char *buf = msg->data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    
    npf_logger_write(LOG_TPI, LOG_INFO, "FE_Start_Operation\n");

    
    // set pobj & pcbtype
    *pobj = retobj;	// Maybe allocated to be passed into callback handler and freed later
    *pcbtype = FPPI_CEReady_Event;
    return retval;
}

uint32_t Config_Query_Request_msgfp (ForCESMessage msg ,
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    char *buf = msg->data;
    int retval = FUNC_SUCCESS;
    // fill declare
    forces_command_t *cmd=(forces_command_t *)msg->data;
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Config_Query_Request\n");

    buf = decapsulate_forces_command_t (cmd, buf);
    npf_logger_write(LOG_TPI, LOG_INFO, ":: type %d\n ", cmd->cmd_type );
    retval = CMD_HANDLER ( cmd, msg, connectionid, length, pobj, pcbtype);
    
    // set pobj & pcbtype

    // *pobj = retobj;	// Maybe allocated to be passed into callback handler and freed later
    // *pcbtype = msgfp[Config_Query_Request - MSG_OFFSET].cb_type;
    return retval;
}

uint32_t Config_Query_Response_msgfp (ForCESMessage msg ,
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    // char *buf = msg->data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Config_Query_Response\n");

    // set pobj & pcbtype
    *pobj = retobj;	// Maybe allocated to be passed into callback handler and freed later
    *pcbtype = msgfp[Config_Query_Request - MSG_OFFSET].cb_type;

    return retval;
}

uint32_t FE_Event_Notification_msgfp (ForCESMessage msg ,
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    // char *buf = msg->data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    
    npf_logger_write(LOG_TPI, LOG_INFO, "FE_Event_Notification\n");
    
    // set pobj & pcbtype
    *pobj = retobj;	// Maybe allocated to be passed into callback handler and freed later
    *pcbtype = msgfp[FE_Event_Notification - MSG_OFFSET].cb_type;
    return retval;
}

uint32_t Unbind_msgfp (ForCESMessage msg ,
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    // char *buf = msg->data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Unbind\n");

    ForCESStopHeartbeat();
    
    // set pobj & pcbtype
    *pobj = retobj;	// Maybe allocated to be passed into callback handler and freed later
    *pcbtype = FPPI_CEUnBind_Event;
    return retval;
}

uint32_t Heartbeat_msgfp (ForCESMessage msg ,
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    // char *buf = msg->data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Heartbeat\n");

    ForCESGotHeartbeat();
    
    // set pobj & pcbtype
    *pobj = retobj;	// Maybe allocated to be passed into callback handler and freed later
    *pcbtype = msgfp[Heartbeat - MSG_OFFSET].cb_type;
    return retval;
}

/* Command handlers */
uint32_t Forces_NULL_Cmd_msgfp (ForCESMessage msg, 
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    // char *buf = msg->data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_NULL_Cmd\n");
    
    // set pobj & pcbtype
    *pobj = retobj;	// Maybe allocated to be passed into callback handler and freed later
    *pcbtype = cmdfp[Forces_NULL_Cmd - CMD_OFFSET].cb_type;
    return retval;
}

uint32_t Forces_Add_Cmd_msgfp (ForCESMessage msg, 
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    // char *buf = msg->data;
    int retval = FUNC_SUCCESS;
    forces_command_t *cmd;
    // fill declare
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_Add_Cmd\n");

    cmd = (forces_command_t *) msg->data;
    
    retval = BLK_HANDLER(cmd,msg,connectionid,length,pobj,pcbtype); 

    return retval;
}

uint32_t Forces_Update_Cmd_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
   // char *buf = msg->data;
    int retval = FUNC_SUCCESS;
    forces_command_t *cmd;
    // fill declare
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_Add_Cmd\n");

    cmd = (forces_command_t *) msg->data;
   
    retval = BLK_HANDLER(cmd,msg,connectionid,length,pobj,pcbtype);
    return retval;
}

uint32_t Forces_Delete_Cmd_msgfp (ForCESMessage msg, 
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    int retval = FUNC_SUCCESS;
    // fill declare
    forces_command_t *cmd;
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_Delete_Cmd\n");

    cmd = (forces_command_t *) msg->data;
    
    retval = BLK_HANDLER(cmd, msg,connectionid,length,pobj,pcbtype); 
    
    return retval;
}

uint32_t Forces_DeleteAll_Cmd_msgfp (ForCESMessage msg, 
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    int retval = FUNC_SUCCESS;
    // fill declare
    forces_command_t *cmd;
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_DeleteAll_Cmd\n");

    cmd = (forces_command_t *) msg->data;
    
    retval = BLK_HANDLER(cmd, msg,connectionid,length,pobj,pcbtype);
    
    return retval;
}

uint32_t Forces_SendPacket_Cmd_msgfp (ForCESMessage msg, 
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    int retval = FUNC_SUCCESS;
    // fill declare
    forces_command_t *cmd;

    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_SendPacket_Cmd\n");

    cmd = (forces_command_t *) msg->data;

    retval = BLK_HANDLER(cmd, msg,connectionid,length,pobj,pcbtype);

    return retval;

}

uint32_t Forces_QueryStats_Cmd_msgfp (ForCESMessage msg, 
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    int retval = FUNC_SUCCESS;
    // fill declare
    forces_command_t *cmd;
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_QueryStats_Cmd\n");

    cmd = (forces_command_t *) msg->data;
    
    retval = BLK_HANDLER(cmd,msg,connectionid,length,pobj,pcbtype); 
    
    return retval;
}

uint32_t Forces_QueryProps_Cmd_msgfp (ForCESMessage msg,
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    int retval = FUNC_SUCCESS;
    // fill declare
    forces_command_t *cmd;
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_QueryProps_Cmd\n");

    cmd = (forces_command_t *) msg->data;
    
    retval = BLK_HANDLER(cmd,msg,connectionid,length,pobj,pcbtype); 
    
    return retval;
}

/* Event handlers */
uint32_t Forces_Port_Event_msgfp (ForCESMessage msg,
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_Port_Event\n");

    
    // set pobj & pcbtype
    *pobj = retobj;	// Maybe allocated to be passed into callback handler and freed later
    *pcbtype = evntfp[Forces_Port_Event - EVT_OFFSET].cb_type;
    return retval;
}

uint32_t Forces_BlockSpecific_Event_msgfp (ForCESMessage msg,
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_BlockSpecific_Event\n");

    
    // set pobj & pcbtype
    *pobj = retobj;	// Maybe allocated to be passed into callback handler and freed later
    *pcbtype = evntfp[Forces_BlockSpecific_Event - EVT_OFFSET].cb_type;
    return retval;
}

uint32_t Forces_PacketRedirect_Event_msgfp (ForCESMessage msg,
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_PacketRedirect_Event\n");

    
    // set pobj & pcbtype
    *pobj = retobj;	// Maybe allocated to be passed into callback handler and freed later
    *pcbtype = evntfp[Forces_PacketRedirect_Event - EVT_OFFSET].cb_type;
    return retval;
}

uint32_t Forces_Capability_Event_msgfp (ForCESMessage msg,
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_Capability_Event\n");

    
    // set pobj & pcbtype
    *pobj = retobj;	// Maybe allocated to be passed into callback handler and freed later
    *pcbtype = evntfp[Forces_Capability_Event - EVT_OFFSET].cb_type;
    return retval;
}

/* Block Handlers */
uint32_t Forces_Ingress_Intf_Block_msgfp (ForCESMessage msg,
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    int retval = FUNC_SUCCESS;
    // fill declare
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_Ingress_Intf_Block\n");

    
    // set pobj & pcbtype
    return retval;
}

uint32_t Forces_Egress_Intf_Block_msgfp (ForCESMessage msg,
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_Egress_Intf_Block\n");

    
    // set pobj & pcbtype
    *pobj = retobj;	// Maybe allocated to be passed into callback handler and freed later
    *pcbtype = blkfp[Forces_Egress_Intf_Block - BLK_OFFSET].cb_type;
    return retval;
}

#if 0
uint32_t Forces_IPv4_Block_msgfp (ForCESMessage msg,
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    npf_ipv4_RouteList_t* routelist;
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IPv4_Prefix_Block\n");

    switch (type )
    {
        case Forces_Add_Cmd :
	 case Forces_Delete_Cmd : 
            routelist = (npf_ipv4_RouteList_t*)PIL_AllocateMemory(sizeof(npf_ipv4_RouteList_t),0);
            buf = decapsulate_uint32_t(&(routelist->route_count),buf);
            routelist->route_list = (npf_ipv4_RouteEntry_t*)PIL_AllocateMemory(sizeof(npf_ipv4_RouteEntry_t)*routelist->route_count,0);
	     decapsulate_array_npf_ipv4_RouteEntry_t(routelist->route_list, routelist->route_count, buf);
	     retobj = (void *)routelist;
 	     *pcbtype = (type == Forces_Add_Cmd ) ? FPPI_IPv4_AddRoute : FPPI_IPv4_DelRoute;
	     break;
        case Forces_DeleteAll_Cmd :
 	     *pcbtype = FPPI_IPv4_PurgeRoute;
	     break; 
        case Forces_QueryStats_Cmd :
	 case Forces_NULL_Cmd : 
	 case Forces_SendPacket_Cmd :
        case Forces_QueryProps_Cmd :
	 default :
    		npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IPv4_Prefix_Block : Add CODE\n");
		break;
    }
     *pobj = (void *)retobj;
    return retval;
}
#endif

uint32_t Forces_IPv4_Prefix_Block_msgfp (ForCESMessage msg,
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    uint32_t      size;
    char *buf =   ((forces_command_t *)msg->data)->cmd_data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    npf_ipv4_UnicastPrefixList_t *prefixlist;
    npf_ipv4_UnicastPrefixUpdateInfoList_t* preUpdateList;

    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IPv4_Prefix_Block\n");

    switch (type)
    {
       case Forces_Add_Cmd :
          preUpdateList = (npf_ipv4_UnicastPrefixUpdateInfoList_t *)PIL_AllocateMemory(sizeof(npf_ipv4_UnicastPrefixUpdateInfoList_t), 0); 
          buf = decapsulate_NPF_uint32_t(&(preUpdateList->numEntries), buf);
          size = sizeof(FPPI_UnicastPrefixUpdateInfo_t) * 
                                            preUpdateList->numEntries;
          preUpdateList->prefixInfo = (FPPI_UnicastPrefixUpdateInfo_t *)PIL_AllocateMemory(size, 0);
          decapsulate_array_FPPI_UnicastPrefixUpdateInfo_t(preUpdateList->prefixInfo, preUpdateList->numEntries, buf);             
          retobj = (void *)preUpdateList;
          *pcbtype = FPPI_IPv4_UniAddPrefix;
          break;
 
	    case Forces_Delete_Cmd : 
          prefixlist = (npf_ipv4_UnicastPrefixList_t *)PIL_AllocateMemory(sizeof (npf_ipv4_UnicastPrefixList_t ),0);
     	    buf = decapsulate_uint32_t(&(prefixlist->numEntries), buf);
          prefixlist->prfxInfo = (NPF_IPv4UC_Prefix_t *)PIL_AllocateMemory(sizeof (NPF_IPv4UC_Prefix_t )*prefixlist->numEntries,0);
	       decapsulate_array_NPF_IPv4UC_Prefix_t(prefixlist->prfxInfo, prefixlist->numEntries, buf);
	       retobj = (void *)prefixlist;
          *pcbtype = FPPI_IPv4_UniDelPrefix;
	       break;

      case Forces_DeleteAll_Cmd :
 	      *pcbtype = FPPI_IPv4_UniFlushPrefix;
	      break; 

      case Forces_QueryStats_Cmd :
 	      *pcbtype = FPPI_IPv4_UniQueryPrefix;
	      break;

      case Forces_QueryProps_Cmd :
          prefixlist = (npf_ipv4_UnicastPrefixList_t *)PIL_AllocateMemory(sizeof (npf_ipv4_UnicastPrefixList_t ),0);
     	    buf = decapsulate_uint32_t(&(prefixlist->numEntries), buf);
          prefixlist->prfxInfo = (NPF_IPv4UC_Prefix_t *)PIL_AllocateMemory(sizeof (NPF_IPv4UC_Prefix_t )*prefixlist->numEntries,0);
	       decapsulate_array_NPF_IPv4UC_Prefix_t(prefixlist->prfxInfo, prefixlist->numEntries, buf);
	       retobj = (void *)prefixlist;
 	      *pcbtype = FPPI_IPv4_UniQueryPrefix;
         break;
 
	   case Forces_NULL_Cmd : 
	   case Forces_SendPacket_Cmd :

	   default :
         npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IPv4_Prefix_Block : Add CODE for the other Cmd types\n");
	      break;
     }

    *pobj = (void *)retobj;
    return retval;
}

uint32_t Forces_IPv4_NextHop_Block_msgfp (ForCESMessage msg,
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    npf_ipv4_UnicastNextHopUpdateInfoList_t* nUpdateList; 
    uint32_t size;
    npf_ipv4_UnicastNextHopDeleteInfoList_t *nDeleteList;
    npf_ipv4_UnicastNextHopQueryList_t *nQueryList;
 
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IPv4_NextHop_Block\n");

    switch (type)
    {
     case Forces_Add_Cmd :
        nUpdateList = (npf_ipv4_UnicastNextHopUpdateInfoList_t *)PIL_AllocateMemory(sizeof(npf_ipv4_UnicastNextHopUpdateInfoList_t), 0); 
        buf = decapsulate_NPF_uint32_t(&(nUpdateList->numEntries), buf);
         if(nUpdateList->numEntries == 0)
         {
            npf_logger_write(LOG_TPI, LOG_ERR, "Number of Entries is zero "); 
            retval = NPF_E_UNKNOWN;
            break;
         }
        size = sizeof(FPPI_UnicastNextHopUpdateInfo_t) * 
                                            nUpdateList->numEntries;
        nUpdateList->nhUpdateInfo = (FPPI_UnicastNextHopUpdateInfo_t *)PIL_AllocateMemory(size, 0);
        decapsulate_array_FPPI_UnicastNextHopUpdateInfo_t(nUpdateList->nhUpdateInfo, nUpdateList->numEntries, buf); 
	     retobj = (void *)nUpdateList;
 	     *pcbtype = FPPI_IPv4_UniAddNextHop;
	 break;

	 case Forces_Delete_Cmd : 
         nDeleteList = (npf_ipv4_UnicastNextHopDeleteInfoList_t *)PIL_AllocateMemory(sizeof (npf_ipv4_UnicastNextHopDeleteInfoList_t ),0);
     	   buf = decapsulate_NPF_uint32_t(&(nDeleteList->numEntries), buf);
         if(nDeleteList->numEntries == 0)
         {
            npf_logger_write(LOG_TPI, LOG_ERR, "Number of Entries is zero "); 
            retval = NPF_E_UNKNOWN;
            break;
         }
         size = sizeof(FPPI_UnicastNextHopDeleteInfo_t) * 
                                      nDeleteList->numEntries;
         nDeleteList->nhDeleteInfo = (FPPI_UnicastNextHopDeleteInfo_t *)PIL_AllocateMemory(size, 0);
         decapsulate_array_FPPI_UnicastNextHopDeleteInfo_t(nDeleteList->nhDeleteInfo, nDeleteList->numEntries, buf);
	      retobj = (void *)nDeleteList;
        *pcbtype = FPPI_IPv4_UniDelNextHop;
	 break;

    case Forces_DeleteAll_Cmd :
         nDeleteList = (npf_ipv4_UnicastNextHopDeleteInfoList_t *)PIL_AllocateMemory(sizeof (npf_ipv4_UnicastNextHopDeleteInfoList_t ),0);
     	   buf = decapsulate_NPF_uint32_t(&(nDeleteList->numEntries), buf);
         if(nDeleteList->numEntries == 0)
         {
            npf_logger_write(LOG_TPI, LOG_ERR, "Number of Entries is zero "); 
            retval = NPF_E_UNKNOWN;
            break;
         }
         size = sizeof(FPPI_UnicastNextHopDeleteInfo_t) * 
                                      nDeleteList->numEntries;
         nDeleteList->nhDeleteInfo = (FPPI_UnicastNextHopDeleteInfo_t *)PIL_AllocateMemory(size, 0);
         decapsulate_array_FPPI_UnicastNextHopDeleteInfo_t(nDeleteList->nhDeleteInfo, nDeleteList->numEntries, buf);
	      retobj = (void *)nDeleteList;
  	      *pcbtype = FPPI_IPv4_UniFlushNextHop;
	 break;

    case Forces_QueryStats_Cmd :
         *pcbtype = FPPI_IPv4_UniQueryNextHop;
    break; 

    case Forces_QueryProps_Cmd :
         nQueryList = (npf_ipv4_UnicastNextHopQueryList_t *)PIL_AllocateMemory(sizeof (npf_ipv4_UnicastNextHopQueryList_t ),0);
     	   buf = decapsulate_NPF_uint32_t(&(nQueryList->numEntries), buf);
         if(nQueryList->numEntries == 0)
         {
            npf_logger_write(LOG_TPI, LOG_ERR, "Number of Entries is zero "); 
            retval = NPF_E_UNKNOWN;
            break;
         }
         size = sizeof(NPF_uint32_t) * nQueryList->numEntries;
         nQueryList->nhIdInfo = (NPF_uint32_t *)PIL_AllocateMemory(size, 0);
         decapsulate_array_NPF_uint32_t(nQueryList->nhIdInfo, nQueryList->numEntries, buf);
	      retobj = (void *)nQueryList;

        *pcbtype = FPPI_IPv4_UniQueryNextHop;
    break;

	 case Forces_NULL_Cmd : 
	 case Forces_SendPacket_Cmd :
	 default :
        {
            npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IPv4_NextHop_Block : Add CODE for 
            the other Cmd types\n");
        }
	 break;
    }
     *pobj = (void *)retobj;
    return retval;
}

uint32_t Forces_L2_ARP_Block_msgfp (ForCESMessage msg,
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    npf_ipv4_UnicastToL2List_t* list;
    uint32_t size;
 
    switch(type) 
    {
       case Forces_Add_Cmd :
          list = (npf_ipv4_UnicastToL2List_t *)PIL_AllocateMemory(sizeof(npf_ipv4_UnicastToL2List_t), 0);
     	    buf = decapsulate_uint32_t(&(list->numEntries), buf);
          size = sizeof(FPPI_IPv4UniNHResolEntry_t) * 
             list->numEntries;
          list->toL2Info = (FPPI_IPv4UniNHResolEntry_t *)PIL_AllocateMemory(size, 0);
          decapsulate_array_FPPI_IPv4UniNHResolEntry_t(list->toL2Info, list->numEntries, buf); 
	       retobj = (void *)list;
 	       *pcbtype = FPPI_IPv4_UniAddToL2;
          break;

	    case Forces_Delete_Cmd : 
          list = (npf_ipv4_UnicastToL2List_t *)PIL_AllocateMemory(sizeof(npf_ipv4_UnicastToL2List_t), 0);
     	    buf = decapsulate_uint32_t(&(list->numEntries), buf);
          size = sizeof(FPPI_IPv4UniNHResolEntry_t) * 
             list->numEntries;
          list->toL2Info = (FPPI_IPv4UniNHResolEntry_t *)PIL_AllocateMemory(size, 0);
          decapsulate_array_FPPI_IPv4UniNHResolEntry_t(list->toL2Info, list->numEntries, buf); 
	       retobj = (void *)list;
 	       *pcbtype = FPPI_IPv4_UniDelToL2;
          break;

	    default :
          {
             npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IPv4_ARP_Block :
                Case not handled\n");
          }
	       break;

    }
 
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_L2_ARP_Block\n");

    *pobj = (void *)retobj;	// Maybe allocated to be passed into callback handler and freed later
    return retval;
}

uint32_t Forces_L2_Interface_Block_msgfp (ForCESMessage msg,
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    npf_L2IntfAttrsList_t *L2AttrsList;
    FPPI_Interface_GetStats_t *port_interface_info; 
    FPPI_PortID *port; 
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_L2_Interface_Block\n");

    switch (type)
    {
        case Forces_Add_Cmd :
	 case Forces_Delete_Cmd : 
            L2AttrsList = (npf_L2IntfAttrsList_t *)malloc (sizeof(npf_L2IntfAttrsList_t));
	     buf = decapsulate_npf_L2IntfAttrsList_t (L2AttrsList, buf);
	     retobj = (void*)L2AttrsList;
            *pcbtype = (type == Forces_Add_Cmd) ? FPPI_L2_SetProperties : FPPI_L2_DelProperties;
	     break;
        case Forces_QueryStats_Cmd :
            port_interface_info = ( FPPI_Interface_GetStats_t  *)malloc (sizeof( FPPI_Interface_GetStats_t ));
            buf = decapsulate_NPF_IfType_t(&port_interface_info->ifType, buf);
            buf = decapsulate_uint32_t(&port_interface_info->port_count, buf);
            port = (FPPI_PortID  *)PIL_AllocateMemory(sizeof(FPPI_PortID)*(port_interface_info->port_count),0);
            port_interface_info->port_array = port;
            buf = decapsulate_array_FPPI_PortID(port_interface_info->port_array, port_interface_info->port_count, buf);
            retobj = (void*)port_interface_info;
            *pcbtype = FPPI_L2_GetStatistics;
            break;
        case Forces_QueryProps_Cmd :
        case Forces_DeleteAll_Cmd :
	 case Forces_NULL_Cmd : 
	 case Forces_SendPacket_Cmd :
	 default :
    		npf_logger_write(LOG_TPI, LOG_INFO, "Forces_L2_Interface_Block : Cmd %d not supported\n",type);
		break;
    }
    *pobj = retobj;	// Maybe allocated to be passed into callback handler and freed later
    // set pobj & pcbtype
    return retval;
}

uint32_t Forces_L3_Interface_Block_msgfp (ForCESMessage msg,
			uint32_t type,
			uint32_t connectionid,
			uint32_t length,
			void **pobj,
			uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    npf_L3IntfAttrsList_t *L3AttrsList;
    FPPI_Interface_GetStats_t *port_interface_info;
    FPPI_PortID * port;
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_L3_Interface_Block\n");

    switch (type)
    {
        case Forces_Add_Cmd :
	 case Forces_Delete_Cmd : 
            L3AttrsList = (npf_L3IntfAttrsList_t *)malloc (sizeof(npf_L3IntfAttrsList_t));
	     buf = decapsulate_npf_L3IntfAttrsList_t (L3AttrsList, buf);
	     retobj = (void*)L3AttrsList;
            *pcbtype = (type == Forces_Add_Cmd) ? FPPI_L3_SetProperties : FPPI_L3_DelProperties;
	     break;
        case Forces_QueryStats_Cmd :
            port_interface_info = ( FPPI_Interface_GetStats_t  *)malloc (sizeof( FPPI_Interface_GetStats_t ));
            buf = decapsulate_NPF_IfType_t(&port_interface_info->ifType, buf);
            buf = decapsulate_uint32_t(&port_interface_info->port_count, buf);
            port = (FPPI_PortID  *)PIL_AllocateMemory(sizeof(FPPI_PortID)*(port_interface_info->port_count),0);
            port_interface_info->port_array = port;
            buf = decapsulate_array_FPPI_PortID(port_interface_info->port_array, port_interface_info->port_count, buf);

            retobj = (void*)port_interface_info;
            *pcbtype = FPPI_L3_GetStatistics;
	     break;
        case Forces_QueryProps_Cmd :
        case Forces_DeleteAll_Cmd :
	 case Forces_NULL_Cmd : 
	 case Forces_SendPacket_Cmd :
	 default :
    		npf_logger_write(LOG_TPI, LOG_INFO, "Forces_L3_Interface_Block : Cmd %d not supported\n",type);
		break;
    }
    
    *pobj = retobj;	// Maybe allocated to be passed into callback handler and freed later
    // set pobj & pcbtype
    return retval;
}

uint32_t Forces_ICMP_Block_msgfp (ForCESMessage msg,
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj,
			uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    npf_PortList_t *portList;

    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_ICMP_Block\n");

    switch (type)
    {
        case Forces_QueryStats_Cmd :
            portList = (npf_PortList_t *)malloc (sizeof(npf_PortList_t));
            buf = decapsulate_npf_PortList_t (portList, buf);
            retobj = (void*)portList;
            *pcbtype = FPPI_ICMP_GetStatistics;
	     break;
        case Forces_Add_Cmd :
	 case Forces_Delete_Cmd : 
        case Forces_QueryProps_Cmd :
        case Forces_DeleteAll_Cmd :
	 case Forces_NULL_Cmd : 
	 case Forces_SendPacket_Cmd :
	 default :
    		npf_logger_write(LOG_TPI, LOG_INFO, "Forces_ICMP_Block : Cmd %d not supported\n",type);
		break;
    }
    // set pobj & pcbtype
    return retval;
}

#ifdef _INCLUDE_IPV6

uint32_t Forces_L3_IPv6_Interface_Block_msgfp (ForCESMessage msg,
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    int retval = FUNC_SUCCESS;
    // fill declare
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_L3_IPv6_Interface_Block\n");

    switch (type)
    {
        case Forces_Add_Cmd :
	 case Forces_Delete_Cmd : 
        case Forces_QueryStats_Cmd :
        case Forces_QueryProps_Cmd :
        case Forces_DeleteAll_Cmd :
	 case Forces_NULL_Cmd :
	 case Forces_SendPacket_Cmd :
	 default :
    		npf_logger_write(LOG_TPI, LOG_INFO, "Forces_L3_IPv6_Interface_Block : Add CODE\n");
		break;
    }
    
    // set pobj & pcbtype
    return retval;
}

uint32_t Forces_IPv6_Block_msgfp (ForCESMessage msg,
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;

     *pobj = (void *)retobj;
    return retval;
}
   
uint32_t Forces_IPv6_Prefix_Block_msgfp (ForCESMessage msg,
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    npf_ipv6_PrefixList_t *prefixlist;
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IPv6_Prefix_Block\n");

    switch (type)
    {
        case Forces_Add_Cmd :
	 case Forces_Delete_Cmd : 
        {
            prefixlist = (npf_ipv6_PrefixList_t *)PIL_AllocateMemory(sizeof (npf_ipv6_PrefixList_t ),0);
	     decapsulate_npf_ipv6_PrefixList_t(prefixlist, buf);
	     retobj = (void *)prefixlist;
      	     *pcbtype = (type == Forces_Add_Cmd ) ? FPPI_IPv6_AddPrefix : FPPI_IPv6_DelPrefix ;
        }
	 break;
        case Forces_DeleteAll_Cmd :
 	     *pcbtype = FPPI_IPv6_FlushPrefix;
	     break; 
        case Forces_QueryStats_Cmd :
 	     *pcbtype = FPPI_IPv6_QueryPrefix;
	     break; 
	 case Forces_NULL_Cmd : 
	 case Forces_SendPacket_Cmd :
        case Forces_QueryProps_Cmd :
	 default :
        {
            npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IPv6_Prefix_Block : Add CODE for 
            the other Cmd types\n");
        }
	 break;
     }
    // set pobj & pcbtype
     *pobj = (void *)retobj;
    return retval;
}

uint32_t Forces_IPv6_NextHop_Block_msgfp (ForCESMessage msg,
			uint32_t type,
			uint32_t connectionid,
			uint32_t length,
			void **pobj,
			uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    FPPI_IPv6_UnicastNextHopUpdateInfo_t *nhop;
    // NPF_IPv6NextHopEntryArray_t *nlist;

    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IPv6_NextHop_Block\n");

    switch (type)
    {
        case Forces_Add_Cmd :
	 case Forces_Delete_Cmd :
        {
            nhop = (FPPI_IPv6_UnicastNextHopUpdateInfo_t *)PIL_AllocateMemory(sizeof (FPPI_IPv6_UnicastNextHopUpdateInfo_t ),0);
	     decapsulate_FPPI_IPv6_UnicastNextHopUpdateInfo_t(nhop, buf);
	     retobj = (void *)nhop;
	     *pcbtype = (type == Forces_Add_Cmd) ? FPPI_IPv6_AddNextHop : FPPI_IPv6_DelNextHop;

        }
	 break;
        case Forces_DeleteAll_Cmd :
      	     *pcbtype = FPPI_IPv6_FlushNextHop;
	 break;
        case Forces_QueryStats_Cmd :
 	     *pcbtype = FPPI_IPv6_QueryNextHop;
	     break;
	 case Forces_NULL_Cmd :
	 case Forces_SendPacket_Cmd :
        case Forces_QueryProps_Cmd :
	 default :
        {
            npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IPv6_NextHop_Block : Add CODE for 
            the other Cmd types\n");
        }
	 break;
     }
     *pobj = (void *)retobj;
    return retval;
}

uint32_t Forces_IPv6_L2_Block_msgfp (ForCESMessage msg,
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    FPPI_IPv6NbrEntry_t *l2info;
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IPv6_L2_Block_msgfp\n");

    switch (type)
    {
        case Forces_Add_Cmd :
	 case Forces_Delete_Cmd : 
        {
            l2info = (FPPI_IPv6NbrEntry_t *)PIL_AllocateMemory(sizeof (FPPI_IPv6NbrEntry_t ),0);
	     decapsulate_FPPI_IPv6NbrEntry_t (l2info, buf);
	     retobj = (void *)l2info;
 	     *pcbtype = (type == Forces_Add_Cmd) ? FPPI_IPv6_AddToL2 : FPPI_IPv6_DelToL2;
        }
	 break;
        case Forces_DeleteAll_Cmd :
        case Forces_QueryStats_Cmd :
	 case Forces_NULL_Cmd : 
	 case Forces_SendPacket_Cmd :
        case Forces_QueryProps_Cmd :
	 default :
        {
            npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IPv6_L2_Block_msgfp : Add CODE for 
            the other Cmd types\n");
        }
	 break;
     }
     *pobj = (void *)retobj;
    return retval;
}

#endif

#ifdef _INCLUDE_IPV6TM

uint32_t Forces_IPv6TM_Block_msgfp (ForCESMessage msg,
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IPv6TM_Block_msgfp : Add CODE for 
            the other Cmd types\n");
     *pobj = (void *)retobj;
    return retval;
}

uint32_t Forces_IPv6TM_Auto_Block_msgfp (ForCESMessage msg,
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    npf_ipv6tm_Tunnel_t *tunnel;
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IPv6TM_Auto_Block_msgfp\n");

    switch (type)
    {
        case Forces_Add_Cmd :
	 case Forces_Delete_Cmd : 
        {
            tunnel = (npf_ipv6tm_Tunnel_t *)PIL_AllocateMemory(sizeof (npf_ipv6tm_Tunnel_t ),0);
	     decapsulate_npf_ipv6tm_Tunnel_t (tunnel, buf);
	     retobj = (void *)tunnel;
 	     *pcbtype = (type == Forces_Add_Cmd) ? FPPI_IPv6TM_EnableAuto : FPPI_IPv6TM_DisableAuto;
        }
	 break;
        case Forces_DeleteAll_Cmd :
        case Forces_QueryStats_Cmd :
	 case Forces_NULL_Cmd : 
	 case Forces_SendPacket_Cmd :
        case Forces_QueryProps_Cmd :
	 default :
        {
            npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IPv6TM_Auto_Block_msgfp : Add CODE for 
            the other Cmd types\n");
        }
	 break;
     }
     *pobj = (void *)retobj;
    return retval;
}

uint32_t Forces_IPv6TM_Config_Block_msgfp (ForCESMessage msg,
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    npf_ipv6tm_Tunnel_t *tunnel;
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IPv6TM_Config_Block_msgfp\n");

    switch (type)
    {
        case Forces_Add_Cmd :
	 case Forces_Delete_Cmd : 
        {
            tunnel = (npf_ipv6tm_Tunnel_t *)PIL_AllocateMemory(sizeof (npf_ipv6tm_Tunnel_t ),0);
	     decapsulate_npf_ipv6tm_Tunnel_t (tunnel, buf);
	     retobj = (void *)tunnel;
 	     *pcbtype = (type == Forces_Add_Cmd) ? FPPI_IPv6TM_EnableConfig : FPPI_IPv6TM_DisableConfig;
        }
	 break;
        case Forces_DeleteAll_Cmd :
        case Forces_QueryStats_Cmd :
	 case Forces_NULL_Cmd : 
	 case Forces_SendPacket_Cmd :
        case Forces_QueryProps_Cmd :
	 default :
        {
            npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IPv6TM_Config_Block_msgfp : Add CODE for 
            the other Cmd types\n");
        }
	 break;
     }
     *pobj = (void *)retobj;
    return retval;
}

uint32_t Forces_IPv6TM_6to4_Block_msgfp (ForCESMessage msg,
			uint32_t type, 
			uint32_t connectionid, 
			uint32_t length, 
			void **pobj, 
			uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    void *retobj=NULL;
    int retval = FUNC_SUCCESS;
    // fill declare
    npf_ipv6tm_Tunnel_t *tunnel;
    
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IPv6TM_6to4_Block_msgfp\n");

    switch (type)
    {
        case Forces_Add_Cmd :
	 case Forces_Delete_Cmd : 
        {
            tunnel = (npf_ipv6tm_Tunnel_t *)PIL_AllocateMemory(sizeof (npf_ipv6tm_Tunnel_t ),0);
	     decapsulate_npf_ipv6tm_Tunnel_t (tunnel, buf);
	     retobj = (void *)tunnel;
 	     *pcbtype = (type == Forces_Add_Cmd) ? FPPI_IPv6TM_Enable6to4 : FPPI_IPv6TM_Disable6to4;
        }
	 break;
        case Forces_DeleteAll_Cmd :
        case Forces_QueryStats_Cmd :
	 case Forces_NULL_Cmd : 
	 case Forces_SendPacket_Cmd :
        case Forces_QueryProps_Cmd :
	 default :
        {
            npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IPv6TM_6to4_Block_msgfp : Add CODE for 
            the other Cmd types\n");
        }
	 break;
     }
     *pobj = (void *)retobj;
    return retval;
}

#endif

/* MPLS */
#ifdef _INCLUDE_MPLS

uint32_t Forces_MPLS_Block_msgfp (
                        ForCESMessage msg,
			uint32_t      type, 
			uint32_t      connectionid, 
			uint32_t      length, 
			void          **pobj, 
			uint32_t      *pcbtype)
{
    char             *buf    = ((forces_command_t *)msg->data)->cmd_data;
    void             *retobj = NULL;
    int              retval  = FUNC_SUCCESS;
    FPPI_MPLS_Data_t *mplsData;
    uint32_t          count =0;

    
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_MPLS_Block_msgfp\n");

    switch (type)
    {
        case Forces_Add_Cmd :
        case Forces_Delete_Cmd : 
        case Forces_DeleteAll_Cmd :
	     case Forces_NULL_Cmd : 
	     case Forces_SendPacket_Cmd :
        case Forces_QueryProps_Cmd :
        case Forces_QueryStats_Cmd :
            mplsData = (FPPI_MPLS_Data_t *)PIL_AllocateMemory(sizeof (FPPI_MPLS_Data_t ),0);
            /* Decapsulate Array Size */
            buf = decapsulate_uint32_t(&count, buf);
            /* Decapsulate Array of MPLS Entries */
	         buf = decapsulate_array_FPPI_MPLS_Data_t (mplsData, count, buf);
	         retobj = (void *)mplsData;
 	         *pcbtype = FPPI_CAT_MPLS;
	         break; 

	      default :
            npf_logger_write(LOG_TPI, LOG_INFO, 
              "Forces_MPLS_Block_msgfp : Add CODE for other Cmd types\n");
	         break;
    }
    *pobj = (void *)retobj;
    return retval;
}

#endif /* MPLS */

/* Start - ATM */
#ifdef _INCLUDE_ATM
uint32_t Forces_IfATM_IfAttrSet_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    IfATM_IfAtmAttr_t *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_IfAttrSet_Block_msgfp\n");

    switch (type )
    {
        case Forces_Add_Cmd :
            info = (IfATM_IfAtmAttr_t *)PIL_AllocateMemory(sizeof(IfATM_IfAtmAttr_t),PIL_MEM_ZERO_INIT);
            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_IfATM_IfAtmAttr_t(info, count, buf);

            *pcbtype = FPPI_ATM_IfAttrSet;
             break;

        case Forces_Delete_Cmd:
        case Forces_DeleteAll_Cmd:
        case Forces_QueryStats_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_IfAttrSet_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}


uint32_t Forces_IfATM_IfDelete_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    FPPI_PortID *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_IfDelete_Block_msgfp \n");

    switch (type )
    {
        case Forces_Delete_Cmd:
            info = (FPPI_PortID *)PIL_AllocateMemory(sizeof(FPPI_PortID),PIL_MEM_ZERO_INIT);
            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_FPPI_PortID(info, count, buf);

            *pcbtype = FPPI_ATM_IfDelete;
             break;

        case Forces_Add_Cmd :
        case Forces_DeleteAll_Cmd:
        case Forces_QueryStats_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_IfDelete_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}


uint32_t Forces_IfATM_IfEnable_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    FPPI_PortID *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_IfEnable_Block_msgfp \n");

    switch (type )
    {
        case Forces_Update_Cmd:
            info = (FPPI_PortID *)PIL_AllocateMemory(sizeof(FPPI_PortID),PIL_MEM_ZERO_INIT);
            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_FPPI_PortID(info, count, buf);

            *pcbtype = FPPI_ATM_IfEnable;
             break;

        case Forces_Add_Cmd :
        case Forces_Delete_Cmd:
        case Forces_DeleteAll_Cmd:
        case Forces_QueryStats_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_IfEnable_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}


uint32_t Forces_IfATM_IfDisable_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    FPPI_PortID *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_IfDisable_Block_msgfp \n");

    switch (type )
    {
        case Forces_Update_Cmd:
            info = (FPPI_PortID *)PIL_AllocateMemory(sizeof(FPPI_PortID),PIL_MEM_ZERO_INIT);

            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_FPPI_PortID(info, count, buf);

            *pcbtype = FPPI_ATM_IfDisable;
             break;

        case Forces_Add_Cmd :
        case Forces_Delete_Cmd:
        case Forces_DeleteAll_Cmd:
        case Forces_QueryStats_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_IfDisable_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}

uint32_t Forces_IfATM_IfGenStatsGet_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    FPPI_PortID *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_IfGenStatsGet_Block_msgfp \n");

    switch (type )
    {
        case Forces_QueryStats_Cmd:
            info = (FPPI_PortID *)PIL_AllocateMemory(sizeof(FPPI_PortID),PIL_MEM_ZERO_INIT);

            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_FPPI_PortID(info, count, buf);

            *pcbtype = FPPI_ATM_IfGenStatsGet;
             break;

        case Forces_Add_Cmd :
        case Forces_Delete_Cmd:
        case Forces_DeleteAll_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_IfGenStatsGet_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}


uint32_t Forces_IfATM_VccSet_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    IfATM_IfVcc_t *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_VccSet_Block_msgfp \n");

    switch (type )
    {
        case Forces_Add_Cmd :
            info = (IfATM_IfVcc_t *)PIL_AllocateMemory(sizeof(IfATM_IfVcc_t),PIL_MEM_ZERO_INIT);

            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_IfATM_IfVcc_t(info, count, buf);

            *pcbtype = FPPI_ATM_VccSet;
             break;

        case Forces_Delete_Cmd:
        case Forces_DeleteAll_Cmd:
        case Forces_QueryStats_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_VccSet_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}

uint32_t Forces_IfATM_VccBind_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    IfATM_VccBind_t *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_VccSet_Block_msgfp \n");

    switch (type )
    {
        case Forces_Add_Cmd :
            info = (IfATM_VccBind_t *)PIL_AllocateMemory(sizeof(IfATM_VccBind_t),PIL_MEM_ZERO_INIT);

            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_IfATM_VccBind_t(info, count, buf);

            *pcbtype = FPPI_ATM_VccBind;
             break;

        case Forces_Delete_Cmd:
        case Forces_DeleteAll_Cmd:
        case Forces_QueryStats_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_VccBind_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}


uint32_t Forces_IfATM_VccDelete_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    IfATM_VccAddr_t *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_VccDelete_Block_msgfp \n");

    switch (type )
    {
        case Forces_Delete_Cmd:
            info = (IfATM_VccAddr_t *)PIL_AllocateMemory(sizeof(IfATM_VccAddr_t),PIL_MEM_ZERO_INIT);

            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_IfATM_VccAddr_t(info, count, buf);

            *pcbtype = FPPI_ATM_VccDelete;
             break;

        case Forces_Add_Cmd :
        case Forces_DeleteAll_Cmd:
        case Forces_QueryStats_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_VccDelete_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}


uint32_t Forces_IfATM_VccEnable_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    IfATM_VccAddr_t *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_VccEnable_Block_msgfp \n");

    switch (type )
    {
        case Forces_Update_Cmd :
            info = (IfATM_VccAddr_t *)PIL_AllocateMemory(sizeof(IfATM_VccAddr_t),PIL_MEM_ZERO_INIT);

            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_IfATM_VccAddr_t(info, count, buf);

            *pcbtype = FPPI_ATM_VccEnable;
             break;

        case Forces_Add_Cmd :
        case Forces_Delete_Cmd:
        case Forces_DeleteAll_Cmd:
        case Forces_QueryStats_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_VccEnable_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}


uint32_t Forces_IfATM_VccDisable_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    IfATM_VccAddr_t *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_VccDisable_Block_msgfp \n");

    switch (type )
    {
        case Forces_Update_Cmd :
            info = (IfATM_VccAddr_t *)PIL_AllocateMemory(sizeof(IfATM_VccAddr_t),PIL_MEM_ZERO_INIT);

            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_IfATM_VccAddr_t(info, count, buf);

            *pcbtype = FPPI_ATM_VccDisable;
             break;

        case Forces_Add_Cmd :
        case Forces_Delete_Cmd:
        case Forces_DeleteAll_Cmd:
        case Forces_QueryStats_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_VccDisable_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}


uint32_t Forces_IfATM_VccOperStatusGet_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    IfATM_VccAddr_t *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_VccOperStatusGet_Block_msgfp \n");

    switch (type )
    {
        case Forces_QueryStats_Cmd:
            info = (IfATM_VccAddr_t *)PIL_AllocateMemory(sizeof(IfATM_VccAddr_t),PIL_MEM_ZERO_INIT);

            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_IfATM_VccAddr_t(info, count, buf);

            *pcbtype = FPPI_ATM_VccOperStatusGet;
             break;

        case Forces_Add_Cmd :
        case Forces_Delete_Cmd:
        case Forces_DeleteAll_Cmd:
        case Forces_Update_Cmd :
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_VccOperStatusGet_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}

uint32_t Forces_IfATM_VccStatsGet_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    IfATM_VccAddr_t *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_VccStatsGet_Block_msgfp \n");

    switch (type )
    {
        case Forces_QueryStats_Cmd:
            info = (IfATM_VccAddr_t *)PIL_AllocateMemory(sizeof(IfATM_VccAddr_t),PIL_MEM_ZERO_INIT);

            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_IfATM_VccAddr_t(info, count, buf);

            *pcbtype = FPPI_ATM_VccStatsGet;
             break;

        case Forces_Add_Cmd :
        case Forces_Delete_Cmd:
        case Forces_DeleteAll_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_VccStatsGet_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}


uint32_t Forces_IfATM_AAL2_ChannelSet_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    IfATM_AAL2_ChannelInfo_t *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_AAL2_ChannelSet_Block_msgfp \n");

    switch (type )
    {
        case Forces_Add_Cmd :
            info = (IfATM_AAL2_ChannelInfo_t *)PIL_AllocateMemory(sizeof(IfATM_AAL2_ChannelInfo_t)
                                     ,PIL_MEM_ZERO_INIT);

            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_IfATM_AAL2_ChannelInfo_t(info, count, buf);

            *pcbtype = FPPI_ATM_AAL2_ChannelSet;
             break;

        case Forces_Delete_Cmd:
        case Forces_DeleteAll_Cmd:
        case Forces_QueryStats_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_AAL2_ChannelSet_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}


uint32_t Forces_IfATM_AAL2_ChannelDel_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    IfATM_AAL2_ConnId_t *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_AAL2_ChannelDel_Block_msgfp \n");

    switch (type )
    {
        case Forces_Delete_Cmd:
            info = (IfATM_AAL2_ConnId_t *)PIL_AllocateMemory(sizeof(IfATM_AAL2_ConnId_t),PIL_MEM_ZERO_INIT);

            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_IfATM_AAL2_ConnId_t(info, count, buf);

            *pcbtype = FPPI_ATM_AAL2_ChannelDel;
             break;

        case Forces_Add_Cmd :
        case Forces_DeleteAll_Cmd:
        case Forces_QueryStats_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_AAL2_ChannelDel_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}


uint32_t Forces_IfATM_ConnectionXcSet_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    NPF_ATM_ConnectionXc_t *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_ConnectionXcSet_Block_msgfp \n");

    switch (type )
    {
        case Forces_Add_Cmd :
            info = (NPF_ATM_ConnectionXc_t *)PIL_AllocateMemory(sizeof(NPF_ATM_ConnectionXc_t),PIL_MEM_ZERO_INIT);

            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_NPF_ATM_ConnectionXc_t(info, count, buf);

            *pcbtype = FPPI_ATM_ConnectionXcSet;
             break;

        case Forces_Delete_Cmd:
        case Forces_DeleteAll_Cmd:
        case Forces_QueryStats_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_ConnectionXcSet_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}



uint32_t Forces_IfATM_ConnectionXcDel_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    NPF_ATM_ConnectionXc_t *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_ConnectionXcDel_Block_msgfp \n");

    switch (type )
    {
        case Forces_Delete_Cmd:
            info = (NPF_ATM_ConnectionXc_t *)PIL_AllocateMemory(sizeof(NPF_ATM_ConnectionXc_t),PIL_MEM_ZERO_INIT);

            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_NPF_ATM_ConnectionXc_t(info, count, buf);

            *pcbtype = FPPI_ATM_ConnectionXcDel;
             break;

        case Forces_Add_Cmd :
        case Forces_DeleteAll_Cmd:
        case Forces_QueryStats_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_ConnectionXcDel_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}


uint32_t Forces_IfATM_OAM_CP_Set_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    IfATM_OAM_CP_t *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_OAM_CP_Set_Block_msgfp \n");

    switch (type )
    {
        case Forces_Add_Cmd :
            info = (IfATM_OAM_CP_t *)PIL_AllocateMemory(sizeof(IfATM_OAM_CP_t),PIL_MEM_ZERO_INIT);

            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_IfATM_OAM_CP_t(info, count, buf);

            *pcbtype = FPPI_ATM_OAM_CP_Set;
             break;

        case Forces_Delete_Cmd:
        case Forces_DeleteAll_Cmd:
        case Forces_QueryStats_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_OAM_CP_Set_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}



uint32_t Forces_IfATM_OAM_CC_Set_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    IfATM_OAM_CC_t *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_OAM_CC_Set_Block_msgfp \n");

    switch (type )
    {
        case Forces_Add_Cmd :
            info = (IfATM_OAM_CC_t *)PIL_AllocateMemory(sizeof(IfATM_OAM_CC_t),PIL_MEM_ZERO_INIT);

            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_IfATM_OAM_CC_t(info, count, buf);

            *pcbtype = FPPI_ATM_OAM_CC_Set;
             break;

        case Forces_Delete_Cmd:
        case Forces_DeleteAll_Cmd:
        case Forces_QueryStats_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_OAM_CC_Set_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}


uint32_t Forces_IfATM_OAM_LB_Set_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    IfATM_OAM_LB_t *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_OAM_LB_Set_Block_msgfp \n");

    switch (type )
    {
        case Forces_Add_Cmd :
            info = (IfATM_OAM_LB_t *)PIL_AllocateMemory(sizeof(IfATM_OAM_LB_t),PIL_MEM_ZERO_INIT);

            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_IfATM_OAM_LB_t(info, count, buf);

            *pcbtype = FPPI_ATM_OAM_LB_Set;
             break;

        case Forces_Delete_Cmd:
        case Forces_DeleteAll_Cmd:
        case Forces_QueryStats_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_OAM_LB_Set_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}



uint32_t Forces_IfATM_OAM_Monitor_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    IfATM_OAM_Mon_t *info = NULL;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_OAM_Monitor_Block_msgfp \n");

    switch (type )
    {
        case Forces_Add_Cmd :
            info = (IfATM_OAM_Mon_t *)PIL_AllocateMemory(sizeof(IfATM_OAM_Mon_t),PIL_MEM_ZERO_INIT);

            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_IfATM_OAM_Mon_t(info, count, buf);

            *pcbtype = FPPI_ATM_OAM_Monitor;
             break;

        case Forces_Delete_Cmd:
        case Forces_DeleteAll_Cmd:
        case Forces_QueryStats_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_OAM_Monitor_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}


uint32_t Forces_IfATM_OAM_AlarmSet_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    IfATM_OAM_Alarm_t *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_OAM_AlarmSet_Block_msgfp \n");

    switch (type )
    {
        case Forces_Add_Cmd :
            info = (IfATM_OAM_Alarm_t *)PIL_AllocateMemory(sizeof(IfATM_OAM_Alarm_t),PIL_MEM_ZERO_INIT);

            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_IfATM_OAM_Alarm_t(info, count, buf);

            *pcbtype = FPPI_ATM_OAM_AlarmSet;
             break;

        case Forces_Delete_Cmd:
        case Forces_DeleteAll_Cmd:
        case Forces_QueryStats_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_OAM_AlarmSet_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}



uint32_t Forces_IfATM_OAM_AlarmClear_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    IfATM_OAM_Alarm_t *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_OAM_AlarmClear_Block_msgfp \n");

    switch (type )
    {
        case Forces_Delete_Cmd:
            info = (IfATM_OAM_Alarm_t *)PIL_AllocateMemory(sizeof(IfATM_OAM_Alarm_t),PIL_MEM_ZERO_INIT);

            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_IfATM_OAM_Alarm_t(info, count, buf);

            *pcbtype = FPPI_ATM_OAM_AlarmClear;
             break;

        case Forces_Add_Cmd :
        case Forces_DeleteAll_Cmd:
        case Forces_QueryStats_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_OAM_AlarmClear_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}



uint32_t Forces_IfATM_OAM_PM_Set_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    int retval = FUNC_SUCCESS;
    IfATM_OAM_PM_t *info = NULL ;
    uint32_t count;
   
    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_OAM_PM_Set_Block_msgfp \n");

    switch (type )
    {
        case Forces_Add_Cmd :
            info = (IfATM_OAM_PM_t *)PIL_AllocateMemory(sizeof(IfATM_OAM_PM_t),PIL_MEM_ZERO_INIT);

            buf = decapsulate_uint32_t(&count,buf);
            decapsulate_array_IfATM_OAM_PM_t(info, count, buf);

            *pcbtype = FPPI_ATM_OAM_PM_Set;
             break;

        case Forces_Delete_Cmd:
        case Forces_DeleteAll_Cmd:
        case Forces_QueryStats_Cmd:
        case Forces_NULL_Cmd:
        case Forces_SendPacket_Cmd:
        case Forces_QueryProps_Cmd:
        default :
                npf_logger_write(LOG_TPI, LOG_INFO, "Forces_IfATM_OAM_PM_Set_Block_msgfp : Add CODE\n");
                break;
    } /* end of switch */

     *pobj = (void *)info;
    return retval;
}


/* end - ATM */
#endif /* _INCLUDE_ATM */

#ifdef _INCLUDE_DIFFSERV
uint32_t Forces_DS_Dpe_Block_msgfp (ForCESMessage msg,
                        uint32_t type,
                        uint32_t connectionid,
                        uint32_t length,
                        void **pobj,
                        uint32_t *pcbtype)
{
    char *buf = ((forces_command_t *)msg->data)->cmd_data;
    void *retobj=NULL;
    FPPI_DS_Dpe_t *dpe=NULL;
    int retval = FUNC_SUCCESS;

    npf_logger_write(LOG_TPI, LOG_INFO, "Forces_DS_Dpe_Block_msgfp\n");

    switch (type )
    {
        case Forces_Add_Cmd :
 	    *pcbtype = FPPI_DS_DpeOperation;
            dpe = (FPPI_DS_Dpe_t *)PIL_AllocateMemory(sizeof (FPPI_DS_Dpe_t ),0);
            decapsulate_FPPI_DS_Dpe_t(dpe, buf);
	    retobj = (void *)dpe ;
	default :
        {
            npf_logger_write(LOG_TPI, LOG_INFO, "Forces_DS_Classifier_Block: Add CODE\n");
            break;
        }
    }
    *pobj = (void *)retobj;
    return retval;
}
#endif
