/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ForwardingPlane/TransportPlugin/src/misc.c,v $
 * $Revision: 1.6 $
 * $Date: 2003/11/17 16:57:50 $
 *
 * Description:
 * This file implements the decapsulate functions used by backendapi.c
 *
 ****************************************************************************/

#include <string.h>
#include "backendapi_types.h"
#include "forces_Types.h"
#include "bpriv.h"
#include "log.h"
#include "fppencap.h"
#include "fppdecap.h"
#include "fppencap_forces.h"
#include "fppencap_capability_types.h"

extern char* encapsulate_array_uint8_t ( uint8_t* a, uint32_t l, char* buf);

char* decapsulate_FPPI_FEandL2IDs_t(
                FPPI_FEandL2IDs   *a,
                char* buf
                )
{
    buf = decapsulate_uint32_t (&(a->feid),buf);
    buf = decapsulate_uint32_t (&(a->l2id),buf);
    return buf;
}
char* decapsulate_FPPI_NextHopInfo_t(
                FPPI_NextHopInfo   *a,
                char* buf
                )
{
    buf = (char *)decapsulate_uint32_t (&(a->feid),buf);
    buf = (char *)decapsulate_uint32_t (&(a->l2id),buf);
    buf = (char *)decapsulate_uint32_t (&(a->portId),buf);
    buf = (char *)decapsulate_uint32_t ((uint32_t *)&(a->nhAddr),buf);
    buf = (char *)decapsulate_uint32_t (&(a->mtu),buf);
    buf = (char *)decapsulate_uint32_t ((uint32_t *)&(a->flags),buf);
    return buf;
}

char* decapsulate_Forces_Message_t(
                ForCESMessage *a,
                char* buf
                )
{
    *a = (ForCESMessage )buf;
    return buf;
}
#if 0
char *decapsulate_forces_command_t (
		forces_command_t *a, 
		char *buf)
{
    buf = decapsulate_uint32_t(&(a->cmd_type),buf);
    buf = decapsulate_uint32_t(&(a->block_type),buf);
    buf = decapsulate_uint32_t(&(a->block_handle),buf);
    buf = decapsulate_uint32_t(&(a->length),buf);
    /* do not care about cmd_data */
    return buf;
}

char *decapsulate_forces_command_status_t (
		forces_command_status_t *a, 
		char *buf)
{
    buf = decapsulate_uint32_t(&(a->cmd_type),buf);
    buf = decapsulate_uint32_t(&(a->block_type),buf);
    buf = decapsulate_uint32_t(&(a->block_handle),buf);
    buf = decapsulate_uint32_t(&(a->global_result),buf);
    buf = decapsulate_uint32_t(&(a->length),buf);
    /* do not care about cmd_data */
    return buf;
}

/* encap misc */
char *encapsulate_forces_command_t (
		forces_command_t *a, 
		char *buf)
{
    buf = encapsulate_uint32_t(&(a->cmd_type),buf);
    buf = encapsulate_uint32_t(&(a->block_type),buf);
    buf = encapsulate_uint32_t(&(a->block_handle),buf);
    buf = encapsulate_uint32_t(&(a->length),buf);
    /* do not care about cmd_data */
    return buf;
}

#define  sizeof_forces_command_t(a) sizeof(forces_command_t)

char *encapsulate_forces_command_status_t (
		forces_command_status_t *a, 
		char *buf)
{
    buf = encapsulate_uint32_t(&(a->cmd_type),buf);
    buf = encapsulate_uint32_t(&(a->block_type),buf);
    buf = encapsulate_uint32_t(&(a->block_handle),buf);
    buf = encapsulate_uint32_t(&(a->global_result),buf);
    buf = encapsulate_uint32_t(&(a->length),buf);
    /* do not care about cmd_data */
    return buf;
}
#define  sizeof_forces_command_status_t(a) sizeof(forces_command_status_t)
#endif

/* Forces Capability Query response */
char* encapsulate_Capability_Response_t(
	         FPPI_FE_Caps*   fe_caps,
                char* buf
                )
{
    uint32_t i;
    forces_portlist_t port_list;
    forces_port_info_t port_info;
    FPPI_Port_Caps *portcaps;

    port_list.port_count = fe_caps->ports_caps.port_count;

    //buf = encapsulate_array_char(fe_caps->fe_name,FENAMELEN,buf);
    buf = encapsulate_uint32_t((uint32_t *)&(port_list.port_count),buf);

    portcaps = fe_caps->ports_caps.port_caps_list;
    	
    for ( i=0; i < port_list.port_count ; i++, portcaps++)
    {
    	 // Not 1:1 mapping between port infos
	 port_info.port_id = portcaps->portid;
	 port_info.port_type = portcaps->port_type;
        buf = (char *)encapsulate_forces_port_info_t(&port_info,buf);
    }
    i = 0;
    buf = (char *)encapsulate_uint32_t ((uint32_t *)&i, buf);
  	
/* this isn't in the forces (FLEX) doc, but i'm gonna do it anyway! */
    buf = (char *)encapsulate_FPPI_FE_Caps(fe_caps,buf);
    return buf;
}

uint32_t sizeof_Capability_Response_t(
	    	FPPI_FE_Caps*   fe_caps
                )
{
    int size = 0;
    forces_portlist_t a;

    //size = sizeof_array_char(fe_caps->fe_name,FENAMELEN);
    a.port_count = fe_caps->ports_caps.port_count;
    size += sizeof_forces_portlist_t(&a);
    size += sizeof_uint32_t(&a);
    
    size += sizeof_FPPI_FE_Caps(fe_caps);

    return size;
}

/* FPPI_FE_Caps */
char* encapsulate_FPPI_FE_Caps_t(
	         FPPI_FE_Caps*   fe_caps,
                 char* buf
                )
{
    int i;
    FPPI_Port_Caps *portcaps;
    forces_command_t *cmd=(forces_command_t * )buf;
    
    cmd->cmd_type = FPPI_QUERY_FECAPS;
    cmd->block_type = 0;
    cmd->block_handle = 0;
    cmd->length = sizeof_FPPI_FE_Caps_t(fe_caps);
	
    buf = (char *)encapsulate_forces_command_t(cmd,buf);
//    buf = encapsulate_array_char(fe_caps->fe_name,FENAMELEN,buf);
    buf = (char *)encapsulate_uint32_t((uint32_t *)&fe_caps->ports_caps.port_count,buf);

    portcaps = fe_caps->ports_caps.port_caps_list;
    	
    for ( i=0; i < fe_caps->ports_caps.port_count ; i++, portcaps++)
    {
    	buf = (char *)encapsulate_uint32_t((uint32_t *)&portcaps->portid,buf);
	buf = (char *)encapsulate_uint32_t((uint32_t *)&portcaps->port_type, buf);
	buf = (char *)encapsulate_array_uint8_t(portcaps->mac_addr, MACADDRLEN,buf);
	buf = (char *)encapsulate_uint32_t((uint32_t *)&portcaps->link_speed,buf);
	buf = (char *)encapsulate_uint32_t(&portcaps->mtu,buf);
    } 
    i = 0;
    encapsulate_uint32_t (&i, buf);
    return buf;
}

uint32_t sizeof_FPPI_FE_Caps_t(
	    	FPPI_FE_Caps*   fe_caps
                )
{
    int i;
    int size;
    FPPI_Port_Caps *portcaps;
    forces_command_t cmd;

    size = sizeof_forces_command_t(&cmd);
    size += sizeof_array_char(fe_caps->fe_name,FENAMELEN);
    size += sizeof_uint32_t(fe_caps->ports_caps.port_count);

    portcaps = fe_caps->ports_caps.port_caps_list;
    	
    for ( i=0; i < fe_caps->ports_caps.port_count ; i++, portcaps++)
    {
    	size += sizeof_uint32_t(portcaps->portid);
	size += sizeof_uint32_t(portcaps->port_type);
	size += sizeof_array_uint8_t(MACADDRLEN);
	size += sizeof_uint32_t(portcaps->link_speed);
	size += sizeof_uint32_t(portcaps->mtu);
    } 
    size += sizeof_uint32_t (0);
    return size;
}


/* FPPI_FE_event_t */
char* encapsulate_FE_event_t(
		  uint32_t event_t, 
		  uint32_t block_type, 
		  uint32_t block_handle, 
		  uint32_t l, 
		  void *a,
          char* buf
          )
{
    uint8_t *u8a = (uint8_t *)a;
    buf = (char *)encapsulate_uint32_t (&event_t, buf);
    buf = (char *)encapsulate_uint32_t (&block_type, buf);
    buf = (char *)encapsulate_uint32_t (&block_handle, buf);
    buf = (char *)encapsulate_uint32_t (&l, buf);
    encapsulate_array_uint8_t (u8a,l,buf);
    return buf;
}

uint32_t sizeof_FE_event_t(
	    	uint32_t   l
                )
{
    int size;
    size = sizeof_uint32_t(l);
    size += sizeof_uint32_t(l);
    size += sizeof_uint32_t(l);
    size += sizeof_uint32_t(l);
    size += sizeof_array_uint8_t(l);
    return size;
}


/* PacketRedirection */
char* encapsulate_PacketRedirection(
		  uint32_t block_handle, 
                npf_Packet_t    *packet_list,
                char* buf
                )
{
    int uin=FORCES_EVENT_PACKET_REDIRECT;	
    char *a = (char *)packet_list->buffer;
    buf = (char *)encapsulate_uint32_t ((uint32_t *)&uin, buf);
    buf = (char *)encapsulate_uint32_t ((uint32_t *)&block_handle, buf);
    buf = (char *)encapsulate_uint32_t ((uint32_t *)&packet_list->length, buf);
    buf = (char *)encapsulate_array_char (a,packet_list->length,buf);
    encapsulate_uint32_t (&packet_list->portid, buf);
    return buf;
}

uint32_t sizeof_PacketRedirection(
              npf_Packet_t    *packet_list
                )
{
    int size;
    size = sizeof_uint32_t(size);
    size += sizeof_uint32_t(size);
    size += sizeof_uint32_t(size);
    size += sizeof_array_char (packet_list->buffer,packet_list->length);
    size += sizeof_uint32_t(size);
    return size;
}

/* npf_IPv4EntryStausList_t */
char* encapsulate_npf_IPv4EntryStatusList_t(
                npf_IPv4EntryStatusList    *a,
                char* buf
                )
{
    int i;
    npf_IPv4_Status_t    *st;
    buf = (char *)encapsulate_uint32_t(&a->numEntries, buf);
    st=a->entryList;
    for ( i=0; i<a->numEntries; i++,st++)
       buf = (char *)encapsulate_npf_IPv4_Status_t(st,buf);
    return buf;
}

uint32_t sizeof_npf_IPv4EntryStatusList_t(
              npf_IPv4EntryStatusList    *a
                )
{
    int size;
    int i;
    npf_IPv4_Status_t    *st;
    size = sizeof_uint32_t(a->numEntries);
    st=a->entryList;
    for ( i=0; i<a->numEntries; i++,st++)
       size += sizeof_npf_IPv4_Status_t(st);
    return size;
}

char *encapsulate_Report_Status(
		uint32_t block_handle,
		NPF_CBCAT cbtype, 
		npf_IPv4EntryStatusList    *a, 
		char *buf
		)
{
    int size;
    int cmd_type;
    size = sizeof_npf_IPv4EntryStatusList_t(a);
    if (cbtype == FPPI_IPv4_UniAddNextHop)
    {
    	cmd_type = FPPI_CONFIG_IPV4_NHOP_ADD;
    }
    buf = (char *)encapsulate_uint32_t((uint32_t *)&cmd_type, buf);
    buf = (char *)encapsulate_uint32_t(0, buf);
    buf = (char *)encapsulate_uint32_t((uint32_t *)&block_handle, buf);
    buf = (char *)encapsulate_uint32_t((uint32_t *)&size, buf);
    buf = (char *)encapsulate_npf_IPv4EntryStatusList_t(a, buf);
    return buf;
}

uint32_t sizeof_encapsulate_Report_Status(
		void *response_data)
{
    int size;
    size = sizeof_uint32_t(size);
    size += sizeof_uint32_t(size);
    size += sizeof_uint32_t(size);
    size += sizeof_uint32_t(size);
    size += sizeof_npf_IPv4EntryStatusList_t((npf_IPv4EntryStatusList    *)response_data);
    return size;
}

