/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/

/***************************************************************************
*$Source: /cvs/constellations/ixa_3.0/src/cp_pdk/ForwardingPlane/inc/FPModule.h,v $
*$Revision: 1.6 $
*$Date: 2003/11/17 16:57:50 $
****************************************************************************/

#ifndef _FPMODULE_H_
#define _FPMODULE_H_

#include <stdio.h>
#include <assert.h>
#include<signal.h>
#include "backendapi.h"
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "pdk_macros.h"

#define MAX_IXP1200_ETH_PORTS 8 

#define MAX_IXP_LINKS       8  /* Current maximum number of links */

/* 
   This variable will decide how many link UPs are faked on a single FE
   Change this variable to report a different number of LINKs from the EF
 */
#define MAX_PORT_LINKUP 5

#define MAC_ADDR_LEN 6
#define MAXLINK 20
#define MAXPORT 20

/* Macros for debugging */
#define PDBG_PRINT(fmt, args...) printf(fmt,## args)
#define PDBG_TRACE() DBG_TRACE() /* macro */
#define PRETURN(val) RETURN(val) 

extern int shutdown_flag;
extern int g_running;
char  fename [48];
char  hostid [48];

typedef struct config_info
{
    uint32_t ipaddr;
    uint32_t mask;
    uint32_t ifNum;
    uint32_t flags;
}configInfo;

typedef struct l2config_info
{
    uint32_t type;
    uint32_t portnum;
    uint32_t mtu;
    char m_hwaddr[70];
}l2configInfo;  

typedef struct l2statistics_info
{
    uint32_t SingleCollisionFrames;
    uint32_t MultipleCollisionFrames;
    uint32_t FramesCheckSequenceErrors;
    uint32_t AlignmentErrors;
    uint32_t FramesWithDeferredXmissions;
    uint32_t LateCollisions;
    uint32_t FrameTooLongErrors;
}l2statistics;
        
typedef struct l3statistics_info
{
    uint32_t rxPkt;
    uint32_t txPkt;
    uint32_t rxBytes;
    uint32_t txBytes;
    uint32_t rxMcastPkt;
    uint32_t txMcastPkt;
    uint32_t rxBcastPkt;
    uint32_t txBcastPkt;
}l3statistics;  


typedef struct icmpstatistics_info
{
    uint32_t msgs;
    uint32_t errors;
    uint32_t dstunreachable;
    uint32_t timeexcd;
    uint32_t paramprobs;
}icmpstatistics;

/* Calls made in FPModule.c */
FPPI_RET pdk_fpm_init(void);
FPPI_RET pdk_fpm_start(char* name, char* hostname);
FPPI_RET pdk_fpm_shutdown(void);

/*Event Handler Functions */
void createEventHandler(void);
void * EventHandlerLoop (void *);

/* Callback functions registered with the back-end API */

void fpm_bindResponse_cb (FPPI_CBCORRELATOR cbcorrelator,
			  FPPI_CONTEXT context,
			  void *response_data);

void fpm_unbindResponse_cb (FPPI_CBCORRELATOR cbcorrelator,
			    FPPI_CONTEXT context,
			    void *response_data);

/*IPv4 Manager External APIs*/
void fpm_ipv4_mgr_add_prefix(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context,
		      void *response_data);
void fpm_ipv4_mgr_delete_prefix(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context,
		      void *response_data);
void fpm_ipv4_mgr_flush_prefixes(FPPI_CBCORRELATOR cbcorrelator,
			FPPI_CONTEXT context,
			void *response_data);
void fpm_ipv4_mgr_update_prefix(FPPI_CBCORRELATOR cbcorrelator,
			FPPI_CONTEXT context,
			void *response_data);
void fpm_ipv4_mgr_add_arp(FPPI_CBCORRELATOR cbcorrelator,
			 FPPI_CONTEXT context,
			 void *response_data);
void fpm_ipv4_mgr_delete_arp(FPPI_CBCORRELATOR cbcorrelator,
			 FPPI_CONTEXT context,
			 void *response_data);
void fpm_ipv4_mgr_flush_arps(FPPI_CBCORRELATOR cbcorrelator,
			   FPPI_CONTEXT context,
			   void *response_data);
void fpm_ipv4_mgr_query_arp(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context,
		      void *response_data);
void fpm_ipv4_mgr_add_next_hop(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context,
		      void *response_data);
void fpm_ipv4_mgr_delete_next_hop(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context,
		      void *response_data);
void fpm_ipv4_mgr_flush_next_hops(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context,
		      void *response_data);

/*MPLS Manager External APIs*/
void fpm_mpls_FpCallback (FPPI_CBCORRELATOR cbcorrelator,
		          FPPI_CONTEXT context,
		          void *response_data);

/*CMM External APIs*/
void fpm_cm_mgr_get_L2_stats(FPPI_CBCORRELATOR cbcorrelator,
			FPPI_CONTEXT context,
			void *response_data);
void fpm_cm_mgr_get_L3_stats(FPPI_CBCORRELATOR cbcorrelator,
			FPPI_CONTEXT context,
			void *response_data);
void fpm_cm_mgr_get_icmp_stats(FPPI_CBCORRELATOR cbcorrelator,
			  FPPI_CONTEXT context,
			  void *response_data);
void fpm_cm_mgr_set_L3_attrib(FPPI_CBCORRELATOR cbcorrelator,
			 FPPI_CONTEXT context,
			 void *response_data);
void fpm_cm_mgr_delete_L3_attrib(FPPI_CBCORRELATOR cbcorrelator,
			     FPPI_CONTEXT context,
			     void *response_data);
void fpm_cm_mgr_set_L2_attrib(FPPI_CBCORRELATOR cbcorrelator,
		       FPPI_CONTEXT context,
		       void *response_data);

/*IPv6TM Manager External APIs*/
void fpm_ipv6tm_mgr_enable_auto(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context,
		      void *response_data);
void fpm_ipv6tm_mgr_disable_auto(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context,
		      void *response_data);
void fpm_ipv6tm_mgr_enable_config(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context,
		      void *response_data);
void fpm_ipv6tm_mgr_disable_config(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context,
		      void *response_data);
void fpm_ipv6tm_mgr_enable_6to4(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context,
		      void *response_data);
void fpm_ipv6tm_mgr_disable_6to4(FPPI_CBCORRELATOR cbcorrelator,
		      FPPI_CONTEXT context,
		      void *response_data);

/*IPv6 Manager External APIs*/
void fpm_ipv6_mgr_add_prefix(FPPI_CBCORRELATOR cbcorrelator,
	                     FPPI_CONTEXT context,
                             void *response_data);
void fpm_ipv6_mgr_delete_prefix(FPPI_CBCORRELATOR cbcorrelator,
    	                        FPPI_CONTEXT context,
                            void *response_data);
void fpm_ipv6_mgr_flush_prefixes(FPPI_CBCORRELATOR cbcorrelator,
	                        FPPI_CONTEXT context,
	                        void *response_data);
/*void fpm_ipv6_mgr_update_prefix(FPPI_CBCORRELATOR cbcorrelator,
	                        FPPI_CONTEXT context,
	                        void *response_data);*/
void fpm_ipv6_mgr_add_nbr(FPPI_CBCORRELATOR cbcorrelator,
                          FPPI_CONTEXT context,
                          void *response_data);
void fpm_ipv6_mgr_delete_nbr(FPPI_CBCORRELATOR cbcorrelator,
                         FPPI_CONTEXT context,
                          void *response_data);
/*void fpm_ipv6_mgr_flush_nbr(FPPI_CBCORRELATOR cbcorrelator,
                           FPPI_CONTEXT context,
                           void *response_data);*/
void fpm_ipv6_mgr_add_next_hop(FPPI_CBCORRELATOR cbcorrelator,
	                      FPPI_CONTEXT context,
                              void *response_data);
void fpm_ipv6_mgr_delete_next_hop(FPPI_CBCORRELATOR cbcorrelator,
	                      FPPI_CONTEXT context,
                              void *response_data);
void fpm_ipv6_mgr_flush_next_hops(FPPI_CBCORRELATOR cbcorrelator,
	                      FPPI_CONTEXT context,
				                            void *response_data);

#ifdef COLOCATED
int fp_main(char *feid,  char *hostid);
#endif

void fpm_delL3Properties_cb (FPPI_CBCORRELATOR cbcorrelator,
			     FPPI_CONTEXT context,
			     void *response_data);
void fpm_getFTNstats_cb (FPPI_CBCORRELATOR cbcorrelator,
			  FPPI_CONTEXT context,
			  void *response_data);
void fpm_getILMstats_cb (FPPI_CBCORRELATOR cbcorrelator,
			  FPPI_CONTEXT context,
			  void *response_data);

/* GTP API */
void fpm_gtp_rxSetup_cb (FPPI_CBCORRELATOR cbcorrelator,
			 FPPI_CONTEXT context,
			 void *response_data);

void fpm_gtp_rxRelease_cb (FPPI_CBCORRELATOR cbcorrelator,
			   FPPI_CONTEXT context,
			   void *response_data);

void fpm_gtp_rxGetTunnelInfo_cb (FPPI_CBCORRELATOR cbcorrelator,
				 FPPI_CONTEXT context,
				 void *response_data);

void fpm_gtp_txSetup_cb (FPPI_CBCORRELATOR cbcorrelator,
			 FPPI_CONTEXT context,
			 void *response_data);

void fpm_gtp_txRelease_cb (FPPI_CBCORRELATOR cbcorrelator,
			   FPPI_CONTEXT context,
			   void *response_data);

void fpm_gtp_txGetTunnelInfo_cb (FPPI_CBCORRELATOR cbcorrelator,
				 FPPI_CONTEXT context,
				 void *response_data);

void fpm_gtp_srncRelocationCmd_cb (FPPI_CBCORRELATOR cbcorrelator,
				   FPPI_CONTEXT context,
				   void *response_data);

void fpm_gtp_srncSuspendRAB_cb (FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data);

void  fpm_gtp_srncRelocationComplete_cb (FPPI_CBCORRELATOR cbcorrelator,
					 FPPI_CONTEXT context,
					 void *response_data);

void fpm_gtp_trncRelocationCmd_cb (FPPI_CBCORRELATOR cbcorrelator,
				   FPPI_CONTEXT context,
				   void *response_data);

void  fpm_gtp_trncRelocationComplete_cb (FPPI_CBCORRELATOR cbcorrelator,
					 FPPI_CONTEXT context,
					 void *response_data);

void fpm_gtp_setBufferSize_cb (FPPI_CBCORRELATOR cbcorrelator,
			       FPPI_CONTEXT context,
			       void *response_data);

void fpm_gtp_dataForwardCmd_cb (FPPI_CBCORRELATOR cbcorrelator,
				FPPI_CONTEXT context,
				void *response_data);

void fpm_gtp_suspendRAB_cb (FPPI_CBCORRELATOR cbcorrelator,
			    FPPI_CONTEXT context,
			    void *response_data);

void fpm_gtp_getPDCPSeqNumber_cb (FPPI_CBCORRELATOR cbcorrelator,
				  FPPI_CONTEXT context,
				  void *response_data);

#ifdef _INCLUDE_ATM

extern void fpmIfATM_IfAttrSetFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_IfDeleteFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_IfEnableFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_IfDisableFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_IfGenStatsGetFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_VccSetFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_VccBindFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_VccDelFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_VccEnableFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_VccDisableFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_VccOperStatusGetFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_VccStatsGetFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_AAL2_ChannelSetFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_AAL2_ChannelDelFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_ConnectionXcSetFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_ConnectionXcDelFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_OAM_CP_SetFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_OAM_CC_SetFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_OAM_LB_SetFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_OAM_MonitorFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_OAM_AlarmSetFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_OAM_AlarmClearFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);
extern void fpmIfATM_OAM_PM_SetFunc(FPPI_CBCORRELATOR cbcorrelator,
						FPPI_CONTEXT context,
						void *response_data);

#endif /* _INCLUDE_ATM */

#endif /* _FPMODULE_H */
