/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

#include <string.h>
#include <stdio.h>
#include "pdk_callback.h"

#ifdef TESTCB
#include "npf_cm_types.h"
#include "npf_cm_api.h"
#endif

/*
 *    Local Variable
*/

/*
static int                CBCounter = 10;
static DList              CBList[MAXCBCATNUM];
*/

static uint32_t           g_pdkcallback_init = FALSE;
static uint32_t           g_pdkcallback_refcount = 0;
static PilCriticalSection g_cblock;

typedef struct cbsystem_t
{
  int                CBCounter;
  DList              CBList[MAXCBCATNUM];
  PilCriticalSection lock;
} cbsystem;

/*
 * Backwards compatibility hack
 */

void * CPCallbackHandle;
void * FPCallbackHandle;

static DList              CBSystemsList;
 
#define GLOBAL_LOCK()       PIL_EnterCriticalSection(&g_cblock)
#define GLOBAL_UNLOCK()     PIL_LeaveCriticalSection(&g_cblock)

#define CB_LOCK(s)       PIL_EnterCriticalSection(&((s)->lock))
#define CB_UNLOCK(s)     PIL_LeaveCriticalSection(&((s)->lock))

/* check if initialized */
#define CHK_INIT() \
    if (!g_pdkcallback_init) {\
        printf("Callback library is not initialized\n");\
        return NPF_COMPONENT_UNINITIALIZED;\
    }

#define CHK_INIT_NULL() \
    if (!g_pdkcallback_init) {\
        printf("Callback library is not initialized\n");\
        return NULL;\
    }

int pdk_callback_compare_systems(const void *a, const void *b)
{
    return (a==b);
}

NPF_RET pdk_callback_initEx(void** handle)
{
    cbsystem *s;
    int i;

    if (!handle)
        return NPF_INVALID_PARAMETERS;

    if (!g_pdkcallback_init)
    {
        PIL_InitializeCriticalSection(&g_cblock);
        GLOBAL_LOCK();
        g_pdkcallback_init = TRUE;
        npf_list_init(&CBSystemsList, PIL_FreeMemory);
        CBSystemsList.compare = pdk_callback_compare_systems;
        GLOBAL_UNLOCK();
    }

    GLOBAL_LOCK();
    ++g_pdkcallback_refcount;
    GLOBAL_UNLOCK();

    s = PIL_AllocateMemory(sizeof(cbsystem),0);
    /* to make each cbid different start in diff pdk_callback_init */
    s->CBCounter = 1977+g_pdkcallback_refcount*MAXCBCATNUM;
    PIL_InitializeCriticalSection(&(s->lock));
    CB_LOCK(s);
    for (i=0; i<MAXCBCATNUM; i++)
        npf_list_init(&(s->CBList[i]), PIL_FreeMemory);
    CB_UNLOCK(s);

    *handle = (void*)s;
    npf_list_pushBack(&CBSystemsList,*handle);

    return NPF_SUCCESS;
}

/*
 *    Destroy all the callbacks, only called at the shutdown
 */
NPF_RET pdk_callback_destroyEx(void* handle)
{
    int   i;
    DList *list;
    cbsystem *s, *tmpHandle;

    if (!handle)
        return NPF_INVALID_PARAMETERS;

    CHK_INIT();

    s = (cbsystem*)handle;

    CB_LOCK(s);

    for (i=0; i<MAXCBCATNUM; i++) 
    {
        list = &(s->CBList[i]);
        pdk_callback_flushCatEx(s,(NPF_CBCAT)i);
        PIL_DestroyCriticalSection(&list->lock);
        memset(list, 0, sizeof(DList));
    }
    
    CB_UNLOCK(s);
    PIL_DestroyCriticalSection(&(s->lock));
    npf_list_removeByKey(&CBSystemsList,s,(void **)&tmpHandle);
    PIL_FreeMemory(s);

    GLOBAL_LOCK();
    if (--g_pdkcallback_refcount == 0)
    {
        g_pdkcallback_init = FALSE;
        PIL_DestroyCriticalSection(&g_cblock);
        memset(&CBSystemsList,0,sizeof(DList));
    }
    GLOBAL_UNLOCK();
    return NPF_SUCCESS;
}

/*
 *    npf_xxx_register may want to validate input or cast
 *    argument before making this general callback register
 */
NPF_RET pdk_callback_registerEx(	void*			handle,
				NPF_CBCAT		cat,
				NPF_USERCONTEXT		context,
				NPF_GENERAL_CBFUNC	func,
				NPF_CBHANDLE		*cbid )
{
    cbsystem *s;
    CB_t *newCB;

    CHK_INIT();

    if (!handle) return NPF_INVALID_PARAMETERS;

    /* Make sure the func is valid */
    if (func == NULL) return NPF_INVALID_PARAMETERS;

    /* Make sure the cat is valid */
    if ((int)cat >= MAXCBCATNUM) return NPF_INVALID_PARAMETERS;

    /* Q: Do we check if the context/client/server exist here? */

    s = (cbsystem*)handle;

    /* allocate memory here and destroy at the deregister() */
    if ((newCB = (CB_t *)PIL_AllocateMemory
                            (sizeof(CB_t), PIL_MEM_ZERO_INIT)) == NULL)
        return NPF_OUT_OF_MEMORY;

    CB_LOCK(s);

    *cbid = s->CBCounter++;

    /* push the CB into the category row */
    npf_list_pushBack(&(s->CBList[cat]), (void *)newCB);

    CB_UNLOCK(s);

    newCB->cbid = *cbid;
    newCB->context = context;
    newCB->category = cat;
    /* Be sure you cast it back before you callback */
    newCB->func = func;
    
    return NPF_SUCCESS;
}

/*
 *    Self-contained so can just do a #define to npf_xxx_deregister call
 */
NPF_RET pdk_callback_deregisterEx(void* handle, NPF_CBHANDLE cbid)
{
    cbsystem *s;
    int      i, found;
    CB_t     *cb;
    DList    *list;
    DListItr *itr;

    CHK_INIT();
    if (!handle) return NPF_INVALID_PARAMETERS;

    s = (cbsystem*)handle;
    found = 0;
    CB_LOCK(s);
    for (i=0; i<MAXCBCATNUM && !found; i++) 
    {
        list = &(s->CBList[i]);
        if ((itr = npf_list_itrCreate(list)) == NULL)
            continue;
        npf_list_itrFirst(itr);
        do {
            cb = (CB_t *)npf_list_itrGetData(itr);
            if (cb->cbid == cbid) {
                found = 1;
                npf_list_itrRemoveFree(itr);
                break;
            }
        } while (npf_list_itrNext(itr) != -1);        
        npf_list_itrDelete(itr);
    }
    CB_UNLOCK(s);

    if (!found)
        return NPF_INVALID_PARAMETERS;
    else
        return NPF_SUCCESS;
}

/*
 *    Remove all the registered callback from a category
 *    Does not destroy the list itself
 */
NPF_RET pdk_callback_flushCatEx(void* handle, NPF_CBCAT cat)
{
    cbsystem *s;
    void  *data;
    DList *list;

    CHK_INIT();
    if (!handle) return NPF_INVALID_PARAMETERS;

    s = (cbsystem*)handle;

    list = &(s->CBList[cat]);
    PIL_EnterCriticalSection(&list->lock);
    while (npf_list_size(list) > 0) {
       if (npf_list_remove(list, npf_list_tail(list), (void **)&data) == 0 && 
           list->destroy != NULL) {
           /* user-defined function to free dynamically allocated data. */
           list->destroy(data);
       }
    }
    PIL_LeaveCriticalSection(&list->lock);
    return NPF_SUCCESS;
}

/*
 *    Return NPF_SUCCESS if cb handle exists NPF_INVALID_PARAMETER otherwise
 */
NPF_RET pdk_callback_isCBExistEx(void* handle, NPF_CBHANDLE cbid)
{
    cbsystem *s;
    int      i, found;
    CB_t     *cb;
    DList    *list;
    DListItr *itr;

    CHK_INIT();
    if (!handle) return NPF_INVALID_PARAMETERS;

    s = (cbsystem*)handle;

    found = 0;
    CB_LOCK(s);
    for (i=0; i<MAXCBCATNUM && !found; i++) {
        list = &(s->CBList[i]);
        if ((itr = npf_list_itrCreate(list)) == NULL)
            continue;
        npf_list_itrFirst(itr);
        do {
            cb = (CB_t *)npf_list_itrGetData(itr);
            if (cb->cbid == cbid)
                found = 1;
        } while (npf_list_itrNext(itr) != -1 && !found);        
        npf_list_itrDelete(itr);
    }
    CB_UNLOCK(s);

    if (!found)
        return NPF_INVALID_PARAMETERS;
    else
        return NPF_SUCCESS;

}

/*
 *    Return NPF_SUCCESS if cb handle exists NPF_INVALID_PARAMETERS otherwise
 *    Output NPF_CBCAT
 */
NPF_RET pdk_callback_getCategoryEx(void* handle, NPF_CBHANDLE cbid, NPF_CBCAT *category)
{
    cbsystem *s;
    int      i, found;
    CB_t     *cb;
    DList    *list;
    DListItr *itr;

    CHK_INIT();
    if (!handle) return NPF_INVALID_PARAMETERS;

    s = (cbsystem*)handle;

    found = 0;
    CB_LOCK(s);
    for (i=0; i<MAXCBCATNUM && !found; i++) {
        list = &(s->CBList[i]);
        if ((itr = npf_list_itrCreate(list)) == NULL)
            continue;
        npf_list_itrFirst(itr);
        do {
            cb = (CB_t *)npf_list_itrGetData(itr);
            if (cb->cbid == cbid)
                found = 1;
        } while (npf_list_itrNext(itr) != -1 && !found);        
        npf_list_itrDelete(itr);
    }
    *category = cb->category;
    CB_UNLOCK(s);

    if (!found)
        return NPF_INVALID_PARAMETERS;
    else
        return NPF_SUCCESS;

}

NPF_GENERAL_CBFUNC pdk_callback_funcEx(void* handle, NPF_CBHANDLE cbid, NPF_USERCONTEXT *context)
{
    cbsystem *s;
    int      i, found;
    CB_t     *cb;
    DList    *list;
    DListItr *itr;

    CHK_INIT_NULL();
    if (!handle) return NULL;

    s = (cbsystem*)handle;

    found = 0;
    CB_LOCK(s);
    for (i=0; i<MAXCBCATNUM; i++) {
        list = &(s->CBList[i]);
        if ((itr = npf_list_itrCreate(list)) == NULL)
            continue;
        npf_list_itrFirst(itr);
        do {
            cb = (CB_t *)npf_list_itrGetData(itr);
            if (cb->cbid == cbid) {
                found = 1;
		*context = cb->context;
                break;
            }
        } while (npf_list_itrNext(itr) != -1);        
        npf_list_itrDelete(itr);
        if (found)
            break;
    }
    CB_UNLOCK(s);

    if (!found)
        return NULL;

    return (cb->func);
}

NPF_GENERAL_CBFUNC pdk_callback_firstFuncEx(	void* handle,
						NPF_CBCAT       category,
						NPF_USERCONTEXT *context,
						NPF_CBHANDLE	*cbid)
{
    cbsystem *s;
    CB_t  *cb;
    DList *list;

    CHK_INIT_NULL();
    if (!handle) return NULL;

    s = (cbsystem*)handle;

    list = &(s->CBList[category]);
    cb = npf_list_getFirstData(list);
    if (cb == NULL) return NULL;
    *context = cb->context;
    *cbid = cb->cbid;
    return (cb->func);
}

/*
 *    Return NULL if passin cbid or nextFunc doesn't exist
 */
NPF_GENERAL_CBFUNC pdk_callback_nextFuncEx(	void* handle,
						NPF_CBHANDLE    cbid,
						NPF_USERCONTEXT *context,
						NPF_CBHANDLE	*nextcbid)
{
    cbsystem *s;
    int      i, found;
    CB_t     *cb, *nextcb;
    DList    *list;
    DListItr *itr;

    CHK_INIT_NULL();
    if (!handle) return NULL;

    s = (cbsystem*)handle;

    found = 0;
    CB_LOCK(s);
    for (i=0; i<MAXCBCATNUM; i++) {
        list = &(s->CBList[i]);
        if ((itr = npf_list_itrCreate(list)) == NULL)
            continue;
        npf_list_itrFirst(itr);
        do {
            cb = (CB_t *)npf_list_itrGetData(itr);
            if (cb->cbid == cbid) {
                /* check if the next itr exists */
                if (npf_list_itrNext(itr) != -1)
                    found = 1;
                else {
                    npf_list_itrDelete(itr);
                    CB_UNLOCK(s);
                    return NULL;
                }
                break;
            }
        } while (npf_list_itrNext(itr) != -1);
        if (found)
            break;
        npf_list_itrDelete(itr);
    }

    if (!found) {
        CB_UNLOCK(s);
        return NULL;
    }

    nextcb = (CB_t *)npf_list_itrGetData(itr);
    npf_list_itrDelete(itr);
    *nextcbid = nextcb->cbid;
    *context = nextcb->context;
    CB_UNLOCK(s);
    return (nextcb->func);

}

NPF_RET pdk_callback_printEx(void* handle)
{
    cbsystem *s;
    int      i;
    CB_t     *cb;
    DList    *list;
    DListItr *itr;

    CHK_INIT();
    if (!handle) return NPF_INVALID_PARAMETERS;

    s = (cbsystem*)handle;

    printf("--------------- TOP -----------------\n");
    CB_LOCK(s);
    for (i=0; i<MAXCBCATNUM; i++) {
        list = &(s->CBList[i]);
        if ((itr = npf_list_itrCreate(list)) == NULL)
            continue;
        printf("[%2d] ",i);
        npf_list_itrFirst(itr);
        do {
            cb = (CB_t *)npf_list_itrGetData(itr);
            printf("%2d ", (int)cb->cbid);
        } while (npf_list_itrNext(itr) != -1);
        npf_list_itrDelete(itr);
        printf("\n");
    }
    CB_UNLOCK(s);
    printf("--------------- END -----------------\n");
    return NPF_SUCCESS;
}

#ifdef TESTCB
void onCB_IP(	NPF_USERCONTEXT context,
		NPF_CORRELATOR	correlator,
		NPF_VERBOSITY	verbosity,
		NPF_HANDLE	handle,
		NPF_IP_CBTYPE	type,
		NPF_DATA	data)
{
    printf("Application onCB_IP gets callback from Main\n");
    printf("Context %s Correlator %s Handle %d\n", 
           (char *)context, (char *)correlator, handle);

    switch(type) {
    case NPF_IP_SET_IPADDR:
        printf("Get NPF_IP_SET_IPADDR\n");
        break;
    default:
        printf("Wrong\n");
    }
}

int main() {

    NPF_CBHANDLE cbid1, cbid2, cbid3;
    NPF_IP_CBFUNC ipcbfunc;

    /* Register Callback */
    ipcbfunc = &onCB_IP;
    pdk_callback_register(NPF_CAT_IP, "ConA", 
                         (NPF_GENERAL_CBFUNC)ipcbfunc, &cbid1);
    pdk_callback_register(NPF_CAT_IF, "ConB", 
                         (NPF_GENERAL_CBFUNC)ipcbfunc, &cbid2);
    pdk_callback_register(PDK_EVT_FE, "ConC", 
                         (NPF_GENERAL_CBFUNC)ipcbfunc, &cbid3);

    /* Invoke callback (should be in another thread) */
    {
        NPF_IP_CBFUNC   clientCBFunc;
	NPF_USERCONTEXT context;
        clientCBFunc = (NPF_IP_CBFUNC)pdk_callback_func(cbid1, &context);
        clientCBFunc(	context,
			(void *)0,		/* CORRELATOR */
			NPF_ACK,		/* VERBOSITY */
			100,			/* NPF HANDLE */
			NPF_IP_SET_IPADDR,	/* CBTYPE */
			(void *)0);		/* RSP DATA */
    }

    /* Check and Deregister callback */
    pdk_callback_print();
    pdk_callback_deregister(cbid1);
    pdk_callback_print();
    pdk_callback_flushCat(NPF_CAT_IF);
    pdk_callback_print();
    pdk_callback_destroy();
    return 0;
}
#endif /* TESTCB */
