/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/common/Encap/src/fppdecap.c,v $
 * $Revision: 1.6 $
 * $Date: 2003/11/17 16:57:51 $
 *
 * Description:
 * This file implements the decapsulate functions which are used by fpibNotify.cc
 * It contains the code which uses the objects defined in the ForCES PIB layer
 *
 ****************************************************************************/

#include "fppdecap.h"

#include <string.h>

/* here are the decapsulation functions for the built in types */
/* not yet sure how to handle pointers/arrays.  The built in */
/* types will assume preallocated for now */
char* decapsulate_char (
                char* c, /* to be decapsulated */
                char* buf /* to decapsulate into */
                )
{
    memcpy(c,buf,sizeof(char));
    return buf + sizeof(char);
}

char* decapsulate_array_char (
                char* a,
                uint32_t l, /* length of a to decapsulate */
                char* buf
                )
{
    uint32_t i;
    for (i=0;i<l;i++)
    {
        buf = decapsulate_char(&(a[i]),buf);
    }
    return buf;
}

char* decapsulate_uchar (
                unsigned char* c, /* to be decapsulated */
                char* buf /* to decapsulate into */
                )
{
    memcpy(c,buf,sizeof(unsigned char));
    return buf + sizeof(char);
}

char* decapsulate_array_uchar (
                unsigned char* a,
                uint32_t l, /* length of a to decapsulate */
                char* buf
                )
{
    uint32_t i;
    for (i=0;i<l;i++)
    {
        buf = decapsulate_uchar(&(a[i]),buf);
    }
    return buf;
}

char* decapsulate_uint8_t(
                uint8_t* a,
                char* buf
                )
{
    memcpy(a,buf,sizeof(uint8_t));
    return buf + sizeof(uint8_t);
}

char* decapsulate_array_uint8_t(
                uint8_t* a,
                uint32_t l,
                char* buf
                )
{
    uint32_t i;
    for (i=0;i<l;i++)
    {
        buf = decapsulate_uint8_t(&(a[i]),buf);
    }
    return buf;
}

char* decapsulate_uint16_t (
                uint16_t* i,
                char* buf
                )
{
    memcpy(i,buf,sizeof(uint16_t));
    *i = ntohs(*i);
    return buf + sizeof(uint16_t);
}

char* decapsulate_array_uint16_t (
                uint16_t* a,
                uint32_t l,
                char* buf
                )
{
    uint32_t i;
    for (i=0;i<l;i++)
    {
        buf = decapsulate_uint16_t(&(a[i]),buf);
    }
    return buf;
}

char* decapsulate_uint32_t (
                uint32_t* i,
                char* buf
                )
{
    memcpy(i,buf,sizeof(uint32_t));
    *i = ntohl(*i);
    return buf + sizeof(uint32_t);
}

char* decapsulate_array_uint32_t (
                uint32_t* a,
                uint32_t l,
                char* buf
                )
{
    uint32_t i;
    for (i=0;i<l;i++)
    {
        buf = decapsulate_uint32_t(&(a[i]),buf);
    }
    return buf;
}

char* decapsulate_uint64_t (
                uint64_t* i,
                char* buf
                )
{
    *i = ntohl(*(uint64_t*)buf);
    return buf + sizeof(uint64_t);
}

char* decapsulate_array_uint64_t (
                uint64_t* a,
                uint32_t l,
                char* buf
                )
{
    uint32_t i;
    for (i=0;i<l;i++)
    {
        buf = decapsulate_uint64_t(&(a[i]),buf);
    }
    return buf;
}

char *decapsulate_IPv6A (
                IPv6A* a,
                char* buf
                )
{
    return decapsulate_array_uint32_t((uint32_t *)a,4,buf);
}

char *decapsulate_HWADDR(
                HWADDR* a,
                char* buf
                )
{
    return decapsulate_array_char((char *)a,MAX_ADDR_LEN,buf);
}

char *decapsulate_IPADDR(
                IPADDR* a,
                char* buf
                )
{
    return decapsulate_uint32_t((uint32_t *)a,buf);
}
