/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/common/Encap/src/fppencap.c,v $
 * $Revision: 1.6 $
 * $Date: 2003/11/17 16:57:52 $
 *
 * Description:
 * This file implements the encapsulate functions which are used by fppapi.cc 
 * It contains the code which uses the objects defined in the ForCES PIB layer
 *
 ****************************************************************************/

#include "fppencap.h"

#include <string.h> /* for memcpy */

/* CHAR */
char* encapsulate_char (
                char* c, /* to be encapsulated */
                char* buf /* to encapsulate into */
                )
{
    *buf = *c;
    return buf + sizeof(char);
}

char* encapsulate_array_char (
                char* a,
                uint32_t l, /* length of a to encapsulate */
                char* buf
                )
{
/*
    uint32_t i;
    for (i=0;i<l;i++,a++)
    {
        buf = encapsulate_char(a,buf);
    }
    return buf;
 */
    memcpy(buf,a,l*sizeof(char));
    return buf + l*sizeof(char);
}

/* UINT8_T */
char* encapsulate_uint8_t (
                uint8_t* i,
                char* buf
                )
{
    *(uint8_t*)buf = *i;
    return buf + sizeof(uint8_t);
}

char* encapsulate_array_uint8_t (
                uint8_t* a,
                uint32_t l,
                char* buf
                )
{
/*
    uint32_t i;
    for (i=0;i<l;i++,a++)
    {
        buf = encapsulate_uint8_t(a,buf);
    }
    return buf;
 */
    memcpy(buf,a,l*sizeof(uint8_t));
    return buf + l*sizeof(uint8_t);
}

/* UINT16_T */
char* encapsulate_uint16_t (
                uint16_t* i,
                char* buf
                )
{
    /* use memcpy instead, incase of alignment problems
    *(uint16_t*)buf = htons(*i);
    */
    uint16_t t = htons(*i);
    memcpy(buf,&t,sizeof(uint16_t));
    return buf + sizeof(uint16_t);
}

char* encapsulate_array_uint16_t (
                uint16_t* a,
                uint32_t l,
                char* buf
                )
{
    uint32_t i;
    for (i=0;i<l;i++,a++)
    {
        buf = encapsulate_uint16_t(a,buf);
    }
    return buf;
}

/* UINT32_T */
char* encapsulate_uint32_t (
                uint32_t* i,
                char* buf
                )
{
    /* see above
    *(uint32_t*)buf = htonl(*i);
    */
    uint32_t t = htonl(*i);
    memcpy(buf,&t,sizeof(uint32_t));
    return buf + sizeof(uint32_t);
}

char* encapsulate_array_uint32_t (
                uint32_t* a,
                uint32_t l,
                char* buf
                )
{
    uint32_t i;
    for (i=0;i<l;i++,a++)
    {
        buf = encapsulate_uint32_t(a,buf);
    }
    return buf;
}

char* encapsulate_uint64_t (
                uint64_t* i,
                char* buf
                )
{
    /* see above
    *(uint64_t*)buf = htonl(*i);
    */
    uint64_t t = htonl(*i);
    memcpy(buf,&t,sizeof(uint64_t));
    return buf + sizeof(uint64_t);
}

char* encapsulate_array_uint64_t (
                uint64_t* a,
                uint32_t l,
                char* buf
                )
{
    uint32_t i;
    for (i=0;i<l;i++,a++)
    {
        buf = encapsulate_uint64_t(a,buf);
    }
    return buf;
}

char *encapsulate_IPv6A (
                IPv6A* a,
                char* buf
                )
{
    return encapsulate_array_uint32_t((uint32_t *)a,4,buf);
}

char *encapsulate_HWADDR(
                HWADDR* a,
                char* buf
                )
{
    return encapsulate_array_char((char *)a,MAX_ADDR_LEN,buf);
}

char *encapsulate_IPADDR(
                IPADDR* a,
                char* buf
                )
{
    return encapsulate_uint32_t((uint32_t *)a,buf);
}
