/**************************************************************************** 
//             INTEL CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or 
//  nondisclosure agreement with Intel Corporation and may not be copied 
//  or disclosed except in accordance with the terms of that agreement. 
//  Copyright (c) 2000-2001 Intel Corporation. All Rights Reserved.
****************************************************************************/

/****************************************************************************
//  $Workfile:   cserror.h  $
//  $Revision: 1.6 $
//  $Date: 2003/11/17 16:57:52 $
//
//  Purpose:
//      This is the main header file for COPS SDK - COPS Server module API.
//      It contains all the required class definitions and return/error
//      codes and other flags and codes.
//
****************************************************************************/
#ifndef  _CSERROR_HDR
#define _CSERROR_HDR

/* COPS SDK function return codes/error codes */

/* Success */
#define CS_RET_SUCCESS                      0

/* Unknown */
#define CS_RET_UNKNOWN                        999

/* General */
#define CS_RET_INVALIDHANDLE                21
#define CS_RET_INVALIDARGS                  22
#define CS_RET_NOTINITIALIZED               23
#define CS_RET_ALREADYINITED                24
#define CS_RET_NOTALLOWED                   25

/* COPS related */
#define CS_RET_NOSECURITYINFO               41
#define CS_RET_INVALIDMESSAGE               42
#define CS_RET_SECURITYFAILED               43

/* System related */
#define CS_RET_OUTOFMEMORY                  61
#define CS_RET_INVALIDEVENT                 62    
#define CS_RET_EVENTOBJFAILED               63
#define CS_RET_SYSTEMERROR                  64

/* Timer Related */
#define CS_RET_NOTIMER                      81
#define CS_RET_TIMEOUT                      82
#define CS_RET_TIMERLISTDELERR              83
#define CS_RET_TIMERLISTINSERR              84
#define CS_RET_TIMERLISTADDERR              85

/* Network related */
#define CS_RET_INVALIDHOSTNAME              101
#define CS_RET_INVALIDSERVER                102    
#define CS_RET_NOSERVER                     103    
#define CS_RET_NOCLIENT                     104
#define CS_RET_NOCLIENTSERVER               105
#define CS_RET_ALREADYRUNNING               106    
#define CS_RET_ALREADYCONNECTED             107    
#define CS_RET_ALREADYINLISTEN              108   
#define CS_RET_RECVINCOMPLETE               109    
#define CS_RET_SOCKETCREATERR               110
#define CS_RET_SOCKETCLOSEERR               111
#define CS_RET_SOCKETLISTADDERR             112
#define CS_RET_SOCKETLISTDELERR             113
#define CS_RET_SERVERSTOPERR                114
#define CS_RET_ACTINGASCLIENT               115    
#define CS_RET_ACTINGASSERVER               116   
#define CS_RET_LISTENFAILED                 117    
#define CS_RET_THREADFAILED                 118
#define CS_RET_CONNECTFAILED                119
#define CS_RET_NETWORKFAILED                120
#define CS_RET_NOMOREDATA                   121
#define CS_RET_ALREADYSENT                  122
#define CS_RET_SOCKETCLOSED                 123

/* COPS-PR specific  */

#define CPR_RET_INVALIDOBJTYPE              201
#define CPR_RET_INVALIDOBJLENGTH            202
#define CPR_RET_OBJECTMISSING               203
#define CPR_RET_UNKNOWNENCODING             204
#define CPR_RET_INVALIDPRID                 205
#define CPR_RET_INVALIDEPD                  206
#define CPR_RET_INVALID_LENGTH              207
#define CPR_RET_VALUERANGEERR               208
#define CPR_RET_CONNECTIONCLOSED            209
#define CPR_RET_REQUESTDELETED              210
#define CPR_RET_MAXLENGTHEXCEEDED           211

#define CPR_RET_NOMOREDEC		            212
#define CPR_RET_NOMOREDATA		            213
#define CPR_RET_DECODEERROR		            214
#define CPR_RET_ENCODEERROR                 215
#define CPR_RET_HASHERR		                216
#define CPR_RET_INVALIDUSE                  217
#define CPR_RET_LISTERR		                218

#define CPR_RET_INVALID_TAG                 219

/* PIB specific */

#define PIB_RET_UNKNOWNPRC                  300
#define PIB_RET_WRONGPRCTYPE                301
#define PIB_RET_UNEXPECTEDDATATYPE          302
#define PIB_RET_UNKNOWNPEP                  303
#define PIB_RET_PEPWRONGSTATE               304
#define PIB_RET_PEPNOTATTACHED              305
#define PIB_RET_UNKNOWNREQUEST              306
#define PIB_RET_MISSINGATTRIB               307
#define PIB_RET_PEPALREADYATTACHED          308
#define PIB_RET_INVALIDREFERENCE            309
#define PIB_RET_INVALIDPOLDATA              310
#define PIB_RET_NOSUCHPOLICY                311
#define PIB_RET_NOSUCHTARGET                312
#define PIB_RET_INVALIDLENGTH               313
#define PIB_RET_INVALIDROLE                 314
#define PIB_RET_UNKNOWNPOLDATATYPE          315
#define PIB_RET_UNKNOWNPOLICYTYPE           316
#define PIB_RET_INVALIDATTRVALUE            317
#define PIB_RET_MAXPRISEXCEEDED             318
#define PIB_RET_STREAMSEEKERROR             319
#define PIB_RET_STREAMREADERROR             320
#define PIB_RET_STREAMWRITEERROR            321
#define PIB_RET_STREAMDATAERROR             322

#define PIB_RET_LOOKUP_ERROR                323
#define PIB_RET_LIST_ERROR                  324
#define PIB_RET_ROLLBACK_ERROR              325
#define PIB_RET_DEC_PENDING                 326
#define PIB_RET_RPT_PENDING                 327
#define PIB_RET_PRINOTFOUND                 328
#define PIB_RET_SUPPATTRS_INVALIDLEN        329
#define PIB_RET_CFG_CTXT_EXISTS             330

#define PIB_RET_UNKNOWNPIB                  331
#define PIB_RET_PIBALREADYEXISTS            332
#define PIB_RET_PEPNOTCONNECTED             333
#define PIB_RET_PRCALREADYEXISTS            334
#define PIB_RET_WRONGPOLICYTYPE             335
#define PIB_RET_POLICYINUSE                 336
#define PIB_RET_UNEXPECTED_PRC              337
#define PIB_RET_UNKNOWNPRCATTR              338
#define PIB_RET_NOACTION                    339
#define PIB_RET_POLICYNAMEEXISTS            340
#define PIB_RET_NOSUCHPRI                   341
#define PIB_RET_UPDATECONFLICT              342

#endif /* _CSERROR_HDR */












