/******************************************************************************
//             INTEL CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or 
//  nondisclosure agreement with Intel Corporation and may not be copied 
//  or disclosed except in accordance with the terms of that agreement. 
//  Copyright (c) 2000-2001 Intel Corporation. All Rights Reserved.
//
//
//      $Workfile:   csportability.h  $
//      $Revision: 1.6 $
//      $Date: 2003/11/17 16:57:52 $
//
//  Purpose:    Upper layer using Portability layer need to use this 
//              header file.
//              
//
******************************************************************************/

#if !defined _CSPORTABILITY_H_
#define _CSPORTABILITY_H_
 
#ifdef __cplusplus
extern "C" {
#endif

#include <OsRedefs.h>
#include <csdatatypes.h>

#define CSINADDR_NONE 0xffffffff
typedef void (*TIMER_CALLBACK)(ULONG handle);

#define DEFAULT_STACK_SIZE 2000
#define DEFAULT_THREAD_PRIORITY 8

/* General Functions */
/******************************************************************************
//
// METHOD:      ULONG CsPortInit( void (*pCbCsNetReceive)
//                                (ULONG,const char *,int),
//                                  void (*pCbCsNetDisconnect)(ULONG),  
//                                  void (*pCbCsTimerExpire)(ULONG),
//                                  void (*pCbCsNetAccept)(ULONG, char *)
//                                )
//
// ARGUMENTS:   Callback function pointers for Receive,Disconnect,Timeout
//              and accept are the arguments
//
// DESCRIPTION: Initialize the portability required variables and starts
//              Client/Server Threads and Initializes Windows Socket Library.
//
// RETURNS:     CS_RET_SUCCESS
//              CS_RET_INVALIDARGS
//              CS_RET_SYSTEMERROR
//              CS_RET_ALREADYINITED
//              
//
******************************************************************************/

ULONG CsPortInit( void (*pCbCsNetReceive)(ULONG,const char *,int),
                  void (*pCbCsNetDisconnect)(ULONG),  
                  void (*pCbCsTimerExpire)(ULONG),
                  void (*pCbCsNetAccept)(ULONG, char *)
                 );

/******************************************************************************
//
// METHOD:      void CsPortDeinit( void )
//
// ARGUMENTS:   None.
//
// DESCRIPTION: Clean up internal values, object state...etc., and shutdown
//              the server or client.
//
// RETURNS:     Nothing.
//              
//
******************************************************************************/

void  CsPortDeinit( void );

/******************************************************************************
//
// METHOD:      ULONG CsLogStatus( USHORT severity,  USHORT event, char *Msg)
//
// ARGUMENTS:   Severity - Error severity to log
//              event    - error event value
//              Msg      - Error message string pointer.
//
// DESCRIPTION: Logs the passed information to a Log file.
//              
//
// RETURNS:     CS_ERR_SUCCESS
//              
//
******************************************************************************/

ULONG CsLogStatus( USHORT severity,  USHORT event, char *Msg);


/*Error Display levels.*/

#define CSINFO  0x0001  
#define CSERR   0x0010      
#define CSWARN  0x0100
     
/*You can use the following #define, to Trace various seviery levels
//of Trace messages.*/

#ifdef _DEBUG
    #ifdef CSTRACE
    #undef CSTRACE
    #endif
    
    #ifdef CSTRACE_INFO     /*Trace informations*/
    #undef CSTRACE_INFO
    #endif

    #ifdef CSTRACE_ERR      /*Trace Errors*/
    #undef CSTRACE_ERR
    #endif

    #ifdef CSTRACE_WARN     /*Trace Warnings*/
    #undef CSTRACE_WARN
    #endif

    #ifdef CSPRINT
    #undef CSPRINT
    #endif

    #define CSTRACE_ERR        CSERR, __FILE__, __LINE__

    #define CSTRACE_WARN       CSWARN, __FILE__, __LINE__

    #define CSTRACE_INFO       CSINFO, __FILE__, __LINE__

    #define CSTRACE(VARGLST)   PrintTraceMsg VARGLST
#else
    #undef  CSTRACE_INFO
    #undef  CSTRACE_ERR
    #undef  CSTRACE_WARN

    #define CSTRACE(VARGLST)    ((void)0)

#endif

#ifdef _DEBUG

/******************************************************************************
//
// METHOD:      void PrintTraceMsg(int level, LPCTSTR lpszFile, int nLine, 
//                                  LPCTSTR lpszFormat, ...)
//
// ARGUMENTS:   level - can be CSTRACE_INFO, CSTRACE_WARN, CSTRACE_ERR
//              used to suppress various levels of trace messages while
//              tracing the error in debug mode.
//              lpszFile - Name of the file and Thread name
//              nLine - Line number of the current thread
//              lpszFormat - printf style format string for output string.
//
// DESCRIPTION: Prints trace string to output window while in debug mode.
//              
//
// RETURNS:     Nothing.
//              
//
******************************************************************************/

void PrintTraceMsg(int level, char *lpszFile,
                   int nLine, char *lpszFormat, ...);
#endif


/* Synchronization Functions */
/******************************************************************************
//
// METHOD:      ULONG CsSyncCreate( ULONG *handle )
//
// ARGUMENTS:   handle - Sync object handle. This is out variable
//              filled by this function with valid object handle
//              if succeeds.
//
// DESCRIPTION: Creates a sync object and returns a unique handle
//              
//
// RETURNS:     CS_RET_SUCCESS
//              CS_RET_OUTOFMEMORY
//              CS_RET_INVALIDHANDLE
//
//
******************************************************************************/

ULONG CsSyncCreate( ULONG *handle );

/******************************************************************************
//
// METHOD:      ULONG CsSyncDestroy( ULONG handle )
//
// ARGUMENTS:   handle - Unique sync object handle
//
// DESCRIPTION: Destroies the given sync object if present.
//              
//
// RETURNS:     CS_RET_SUCCESS
//              CS_RET_INVALIDHANDLE
//
//
******************************************************************************/

ULONG CsSyncDestroy( ULONG handle );

/******************************************************************************
//
// METHOD:      ULONG CsSyncLock( ULONG handle )
//
// ARGUMENTS:   handle - Unique sync object handle
//
// DESCRIPTION: Lock the sync object.
//              
//
// RETURNS:     CS_RET_SUCCESS
//              CS_RET_INVALIDHANDLE
//
//              
//
******************************************************************************/

ULONG CsSyncLock( ULONG handle );

/******************************************************************************
//
// METHOD:      ULONG CsSyncUnlock( ULONG handle )
//
// ARGUMENTS:   handle - Unique sync object handle.
//
// DESCRIPTION: Unlocks the sync object.
//              
//
// RETURNS:     CS_RET_SUCCESS
//              CS_RET_INVALIDHANDLE
//
//
******************************************************************************/

ULONG CsSyncUnlock( ULONG handle );


/* TCP/IP Functions */

/******************************************************************************
//
// METHOD:      ULONG CsNetConnect(char *addr,USHORT port,ULONG *handle,
//                                CS_IPADDR *ServerIp);
//
// ARGUMENTS:   addr    - IP address or Domain name of the Server
//              port    - port of the Server
//              handle  - This is a out variable, returned by the Server   
//                        if it connects successfully.
//              ServerIp- Backup Server IP address
//  
// DESCRIPTION: Connects to the server which listening at the port give
//              port.
//
// RETURNS:     CS_RET_SUCCESS
//              CS_RET_ALREADYCONNECTED
//              CS_RET_INVALIDHOSTNAME
//              CS_RET_SOCKETCREATERR
//              CS_RET_EVENTOBJFAILED
//              CS_RET_NOTINITIALIZED
//              CS_RET_CONNECTFAILED
//              CS_RET_THREADFAILED
//              Or what ever value returned by AddNewSocketToSocketList()
//
//
******************************************************************************/

ULONG CsNetConnect(char *addr,USHORT port,ULONG *handle,CS_IPADDR *ServerIp);


/******************************************************************************
//
// METHOD:      ULONG CsNetListen( USHORT port )
//
// ARGUMENTS:   port - To Listen for connection.
//
// DESCRIPTION: Starts a server and waits for the client connection.
//              
//
// RETURNS:     CS_RET_SUCCESS
//              CS_RET_ALREADYINLISTEN
//              CS_RET_ACTINGASCLIENT
//              CS_RET_INVALIDHANDLE
//              CS_RET_LISTENFAILED
//              
//
******************************************************************************/

ULONG CsNetListen( USHORT port );


/******************************************************************************
//
// METHOD:      ULONG CsNetStopListen()
//
// ARGUMENTS:   None
//
// DESCRIPTION: Stops the Server.
//              
//
// RETURNS:     CS_ERR_SUCCESS
//              CS_RET_NOSERVER
//              CS_RET_SOCKETCLOSEERR
//              CS_RET_SERVERSTOPERR
//
//
******************************************************************************/

ULONG CsNetStopListen(void);


/******************************************************************************
//
// METHOD:      ULONG CsNetDisconnect ( ULONG handle )
//
// ARGUMENTS:   handle - Socket handle
//
// DESCRIPTION: Disconnect a client from the server.
//              
//
// RETURNS:     CS_RET_SUCCESS
//              CS_RET_NOCLIENTSERVER
//              CS_RET_NOSERVER
//              CS_RET_SOCKETCLOSEERR
//              
//
******************************************************************************/

ULONG CsNetDisconnect ( ULONG handle );


/******************************************************************************
//
// METHOD:      ULONG CsNetSend( ULONG handle,  LPBYTE message, int length)
//
// ARGUMENTS:   handle  - Socket handle
//              message : Pointer to the data to be sent  
//              length  : Size of the data
//  
// DESCRIPTION: Sends data to the specified socket.
//              
//
// RETURNS:     CS_ERR_SUCCESS
//              CS_RET_NETWORKFAILED
//              CS_RET_NOCLIENTSERVER
//              
******************************************************************************/

ULONG CsNetSend( ULONG handle, UCHAR * message,  int length);


/******************************************************************************
//
// METHOD:      ULONG CSHTONL( ULONG value )
//
// ARGUMENTS:   value - host byte order
//
// DESCRIPTION: function converts a u_long from host 
//              to TCP/IP network byte order.
//              
//
// RETURNS:     Returns the TCP/IP network byte order
//              
//
******************************************************************************/

ULONG CSHTONL( ULONG value ); 


/******************************************************************************
//
// METHOD:      ULONG CSNTOHL( ULONG value )
//
// ARGUMENTS:   value - Network order byte.
//
// DESCRIPTION: function converts a u_long from TCP/IP 
//              network order to host byte order.
//              
//
// RETURNS:     Returns host byte order.
//              
//
******************************************************************************/

ULONG CSNTOHL( ULONG value ); 


/******************************************************************************
//
// METHOD:      USHORT CSHTONS( USHORT value )
//
// ARGUMENTS:   value - host byte order
//
// DESCRIPTION: function converts a u_short from host 
//              to TCP/IP network byte order.
//              
//
// RETURNS:     Returns network byte order.
//              
//
******************************************************************************/

USHORT CSHTONS( USHORT value );


/******************************************************************************
//
// METHOD:      USHORT CSNTOHS( USHORT value )
//
// ARGUMENTS:   value - network byte order
//
// DESCRIPTION: function converts a u_short from TCP/IP 
//              network byte order to host byte order.
//              
//
// RETURNS:     Returns host byte order.
//              
//
******************************************************************************/

USHORT CSNTOHS( USHORT value );


/******************************************************************************
//
// METHOD:      ULONG CSINET_ADDR(char *cp)
//
// ARGUMENTS:   cp - Dotted IP address string.
//
// DESCRIPTION: function converts a string containing an (Ipv4) 
//              Internet Protocol dotted address into a proper address 
//              for the IN_ADDR structure.
//
//              
//
// RETURNS:     returns an unsigned long value containing a suitable 
//              binary representation of the Internet address given.
//              
//
******************************************************************************/

ULONG CSINET_ADDR(char *cp);


/******************************************************************************
//
// METHOD:      char *CSINET_NTOA(const CS_IPADDR *in)
//
// ARGUMENTS:   in - structure that represents an Internet host address.
//
// DESCRIPTION: function converts an (Ipv4) Internet network address 
//              into a string in Internet standard dotted format.

//              
//
// RETURNS:     returns a char pointer to a buffer containing the text 
//              address in standard ".'' notation.
//              
//
******************************************************************************/

char *CSINET_NTOA(const CS_IPADDR *in);


/* Timer Functions */
/******************************************************************************
//
// METHOD:      ULONG CsTimerSet( ULONG handle, ULONG seconds )
//
// ARGUMENTS:   handle - Unique timer handle
//              seconds - Time out seconds, when timer expires
//
// DESCRIPTION: Used to Set the timer for 'n' seconds.
//              
//
// RETURNS:     CS_RET_SUCCESS
//              CS_RET_INVALIDHANDLE
//              CS_RET_UNKNOWN
//
//
******************************************************************************/

ULONG CsTimerSet( ULONG handle, ULONG seconds );

/******************************************************************************
//
// METHOD: ULONG CsTimerSetEx( ULONG *handle,ULONG seconds,TIMER_CALLBACK cbFunc)
//
// ARGUMENTS:   handle - timer handle returned
//              seconds - Time out seconds, when timer expires
//              cbFunc - callback func
//
// DESCRIPTION: Used to Set the timer for 'n' seconds.
//
// RETURNS:     CS_RET_SUCCESS
//              CS_RET_INVALIDHANDLE
//              CS_RET_UNKNOWN
//
//
******************************************************************************/

ULONG CsTimerSetEx( ULONG *handle, ULONG seconds, TIMER_CALLBACK cbFunc );

/******************************************************************************
//
// METHOD:      ULONG CsTimerDelete( ULONG handle )
//
// ARGUMENTS:   handle - Unique timer handle.
//
// DESCRIPTION: Call this to delete a timer.
//              
//
// RETURNS:     CS_ERR_SUCCESS
//              CS_RET_TIMERLISTDELERR
//              CS_RET_INVALIDHANDLE
//
//
******************************************************************************/

ULONG CsTimerDelete( ULONG handle );


/******************************************************************************
//
// METHOD:      ULONG CsTimerDeleteEx( ULONG handle )
//
// ARGUMENTS:   handle - timer handle received from CsTimerSetEx function
//
// DESCRIPTION: Call this to del a timer created by calling CsTimerSetEx only.
//              
//
// RETURNS:     CS_ERR_SUCCESS
//              CS_RET_TIMERLISTDELERR
//              CS_RET_INVALIDHANDLE
//
//
******************************************************************************/

ULONG CsTimerDeleteEx( ULONG handle );


/******************************************************************************
//
// METHOD:      ULONG64 CSNTOHL64( ULONG64 value )
//
// ARGUMENTS:   value - network byte order
//
// DESCRIPTION: function converts a ULONG64 from TCP/IP 
//              network byte order to host byte order.
//              
//
// RETURNS:     Returns host byte order.
//              
//
******************************************************************************/

ULONG64 CSNTOHL64( ULONG64 value );

/******************************************************************************
//
// METHOD:      ULONG64 CSHTONL64( ULONG64 value )
//
// ARGUMENTS:   value - network byte order
//
// DESCRIPTION: function converts a ULONG64 from host byte order to TCP/IP 
//              network byte order.
//              
//
// RETURNS:     Returns network byte order.
//              
//
******************************************************************************/

ULONG64 CSHTONL64( ULONG64 value );

int CSGetHostName(char *name, int length);

CS_IPADDR CSResolveIpAddress(char *addr);

#ifdef __cplusplus
}
#endif


#endif /* _CSPORTABILITY_H_ */
