/******************************************************************************
//             INTEL CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Intel Corporation and may not be copied
//  or disclosed except in accordance with the terms of that agreement.
//      Copyright (c) 2000 Intel Corporation. All Rights Reserved.
//
//  $Workfile:   Interconnect_api.h  $
//  $Revision: 1.6 $
//  $Date: 2003/11/17 16:57:52 $
//
//  Contents: Exposed Datagram and Buffer function header file
//
******************************************************************************/


#if !defined _INTERCONNECT_API_H
#define _INTERCONNECT_API_H

#ifdef __cplusplus
extern "C" {
#endif

#include "cserror.h"

/* Return Codes */
#define PL_SUCCESS				0x0 
#define PL_ERROR_INVALID_SIZE 		0x1
#define PL_ERROR_INVALID_ARG 		0x2
#define PL_ERROR_OUTOFMEMORY 		0x3
#define PL_ERROR_UNKNOWN 	       	0x4
#define PL_ERROR_NOTINITIALIZED 		0x5
#define PL_ERROR_BUFFERUNITIALIZED		0x6 

#define PL_DATAGRAM_SERVER_PORT		5544
/* Exceptions : Used in the CALLBACK */
#define PL_DATAGRAM_EX_CONNECTION_CLOSE	0x01
/* Client Connect Flags : Used CBClientDatagramParams */
#define PL_DATAGRAM_IPV4   0x01
#define PL_DATAGRAM_IPV6   0x02

/*  CALLBACK FUNCTIONS */
/******************************************************************************
//
// CALLBACK METHOD : CB_DatagramReceive 
//
// ARGUMENTS:  pbuf - buffer containing data.
//	       connect_id - connection identifier (sock fd)
//             length - length of buffer 
//
// DESCRIPTION: Callback to recieve datagrams.
//
// RETURNS:     PL_SUCCESS
//
// NOTE : Call pb_freebuffer after using the pbuf
******************************************************************************/

typedef void (*CB_DatagramReceive)(
		void 		*pbuf, 
		uint32_t 	connect_id, 
		uint32_t 	length);

/******************************************************************************
//
// CALLBACK METHOD : CB_DatagramException 
//
// ARGUMENTS:  ex - exception number.
//	       connect_id - connection identifier (sock fd)
//
// DESCRIPTION: Callback to recieve datagrams. 
//
// RETURNS:     PL_SUCCESS
//
******************************************************************************/
typedef void (*CB_DatagramException)(
		uint32_t 	ex, 
		uint32_t 	connect_id);


typedef struct _CBServerDatagramParams {
    		CB_DatagramReceive	datagram_receive; 
    		CB_DatagramException	datagram_exception;	
} CBServerDatagramFuncs;


typedef struct _datagram_connect {
    		uint32_t		flags; 
    		union {
        		struct sockaddr_in	ipv4_addr; 
#ifndef _VXWORKS_
	 		struct sockaddr_in6	ipv6_addr; 
#endif
		// Place holder for the other connection protocols.
    			}		saddr;
		}datagram_connect;

typedef struct _CBClientDatagramParams {
    		datagram_connect      connect_info;
    		CB_DatagramReceive	datagram_receive;	
    		CB_DatagramException	datagram_exception;	
		} CBClientDatagramFuncs;

/******************************************************************************
//
// uint32_t InitializeDatagramServer (CBServerDatagramParams *serverparams);
//
// ARGUMENTS:  serverparams - pointer to CBServerDatagramParams 
//
// DESCRIPTION: Initialize the Datagram Server API.
//
// RETURNS:     
//		PL_SUCCESS				
//  		PL_ERROR_INVALID_ARG 		
//   		PL_ERROR_OUTOFMEMORY 		
//    		PL_ERROR_UNKNOWN 	       	
//      	PL_ERROR_BUFFERUNITIALIZED		 
******************************************************************************/
uint32_t InitializeDatagramServer (
		CBServerDatagramFuncs 	*serverparams);

/******************************************************************************
//
// uint32_t InitializeDatagramClient (CBClientDatagramParams *clientparams);
//
// ARGUMENTS:  clientparams - pointer to CBClientDatagramParams 
//		connect_id - pointer to CBClientDatagramParams
// DESCRIPTION: Initialize the Datagram Client API.
//
// RETURNS:     
//		PL_SUCCESS				
//  		PL_ERROR_INVALID_ARG 		
//   		PL_ERROR_OUTOFMEMORY 		
//    		PL_ERROR_UNKNOWN 	       	
//      	PL_ERROR_BUFFERUNITIALIZED		 
******************************************************************************/

uint32_t InitializeDatagramClient (
		CBClientDatagramFuncs 	*clientparams,
		uint32_t 		  	*pconnect_id);

/******************************************************************************
//
// uint32_t SendDatagram(uint32_t connect_id, void* msg, uint32_t size, uint32_t flags);
//
// ARGUMENTS:    connect_id - connection identifier
//              msg - buffer containing data
//              size - size of msg
//		 flags - reserved ( pass 0) for later use.
//
// DESCRIPTION: Send Client/Server Datagram
//
// RETURNS:     
//		PL_SUCCESS				
// 		PL_ERROR_INVALID_SIZE 		
//  		PL_ERROR_INVALID_ARG 		
//    		PL_ERROR_UNKNOWN 	       	
//     		PL_ERROR_NOTINITIALIZED 		
// NOTE : Call pb_getbuffer for allocating buf
******************************************************************************/

uint32_t SendDatagram(
		uint32_t 			connect_id, 
		void 				*msg, 
		uint32_t 			size, 
		uint32_t 			flags);

/******************************************************************************
//
// uint32_t DeInitializeDatagram();
//
// ARGUMENTS: 
//
// DESCRIPTION: Deinitialize Datagram API
//
// RETURNS:     
//		PL_SUCCESS				
//     		PL_ERROR_NOTINITIALIZED 		
******************************************************************************/

uint32_t DeInitializeDatagram();

/******************************************************************************
//
// uint32_t pb_PlBufferInit();
//
// ARGUMENTS: 
//	Qsize - total number of Buffers to be Allocated 
//		(includes send and recv buffers)
//
// DESCRIPTION: Initialize Buffer API; Tobe called buffer Datagram Initialize
//
// RETURNS:     
//		PL_SUCCESS				
// 		PL_ERROR_INVALID_SIZE 		
//  		PL_ERROR_INVALID_ARG 		
//   		PL_ERROR_OUTOFMEMORY 		
******************************************************************************/

uint32_t pb_PlBufferInit(
		uint32_t 		Qsize);

/******************************************************************************
//
// uint32_t pb_PlBufferDeInit();
//
// ARGUMENTS: 
//
// DESCRIPTION: Deinitialize Buffer API
//
// RETURNS:     PL_SUCCESS
//
******************************************************************************/
uint32_t pb_PlBufferDeInit(void);

/******************************************************************************
//
// uint32_t pb_getBuffer();
//
// ARGUMENTS: 
//	connect_id : connect_id for the requested buffer
//		"0" is a valid value, but buffer extra security lost.
//	size	: size in bytes
//	flags	: Reserved.
//	pbuf	: Out parameter to be filled
//
// DESCRIPTION: request a buffer of "size" bytes
//
// RETURNS:     
//		PL_SUCCESS				
// 		PL_ERROR_INVALID_SIZE 		
//  		PL_ERROR_INVALID_ARG 		
//   		PL_ERROR_OUTOFMEMORY 		
//    		PL_ERROR_UNKNOWN 	       	
//     		PL_ERROR_NOTINITIALIZED 		
******************************************************************************/

uint32_t pb_getBuffer (
		uint32_t 	connect_id,
		uint32_t 	size,
		uint32_t 	flags,
		void 		**pbuf);

/******************************************************************************
//
// uint32_t pb_freeBuffer();
//
// ARGUMENTS: 
//
// DESCRIPTION: free buffer
//
// RETURNS:     
//		PL_SUCCESS				
//  		PL_ERROR_INVALID_ARG 		
//     		PL_ERROR_NOTINITIALIZED 		
******************************************************************************/

uint32_t pb_freeBuffer(
			void 		* buf);

#ifdef __cplusplus
}
#endif

#endif // _INTERCONNECT_API_H

