
/******************************************************************************
//             INTEL CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or 
//  nondisclosure agreement with Intel Corporation and may not be copied 
//  or disclosed except in accordance with the terms of that agreement. 
//      Copyright (c) 2000 Intel Corporation. All Rights Reserved.
//
//  $Workfile:   ptcpip.h  $
//  $Revision: 1.6 $
//  $Date: 2003/11/17 16:57:53 $
//
//  Contents: Buffer API function header file 
//
******************************************************************************/


#if !defined _PTCPBUFFER_H 
#define _PTCPBUFFER_H

#ifdef __cplusplus
extern "C" {
#endif

#include "cserror.h"
#include "slist.h"

#define MAX_BUF_SIZE 1024
#define MAX_USER_BUF 1000
#define COOKIE 1234

/* Internal Buffer Management Data Structures */

#define PLBUF_SUCCESS 			PL_SUCCESS	 
#define PLBUF_ERROR_INVALID_SIZE 		PL_ERROR_INVALID_SIZE
#define PLBUF_ERROR_INVALID_ARG 		PL_ERROR_INVALID_ARG
#define PLBUF_ERROR_OUTOFMEMORY 		PL_ERROR_OUTOFMEMORY
#define PLBUF_ERROR_UNKNOWN 		PL_ERROR_UNKNOWN
#define PLBUF_ERROR_NOTINITIALIZED		PL_ERROR_NOTINITIALIZED 
#define PLBUF_ERROR_BUFFERUNITIALIZED 	PL_ERROR_BUFFERUNITIALIZED

/* FLAGS */
#define PLBUF_FREE_BUFFER 		0x00
#define PLBUF_ALLOC_BUFFER 		0x01
#define PLBUF_SEND_BUFFER 		0x02
#define PLBUF_RECV_BUFFER 	       0x04
#define PLBUF_CHAIN_BUFFER 		0x08

#define SEMAPHORE_TIMEOUT 100

typedef struct _plbufMemory {
    uint32_t total_Q_size;
    uint32_t buffers_used;
    unsigned char *pBufMem;
    uint32_t pBufFreeMem;
    uint32_t pBufFreeTail;
    uint32_t pBufSend;
    uint32_t pSendTail;
    uint32_t pBufRecv;
    uint32_t pRecvTail;
} plbufMemory, *pplbufMemory;

/* Struct of the buffer header used by each buffer Mgt */
typedef struct _plBufHeader {
    uint32_t		connect_id;		
    uint32_t		flags;
} plBufHeader;


typedef struct _plBuf {
    plBufHeader	pbufheader;	
    plHeader plheader;
    unsigned  char	pbuf[0];
} plBuf;

typedef enum {
   plbuf_free = PLBUF_FREE_BUFFER,
   plbuf_send = PLBUF_SEND_BUFFER,
   plbuf_Recv = PLBUF_RECV_BUFFER
} e_buff_type ;

typedef struct _plMemoryMgt {
	e_buff_type status;
	int next;
	int prev;
	} plMemoryMgt;

extern plbufMemory g_plbufMemory;
extern plMemoryMgt *g_plMemoryMgt;

extern int g_bufferInit;

/*
Memory 
--------------------------------------------
--------------------------------------------
*/

/* Internal functions used by the Datagram API */
uint32_t pb_addToSendQ(plBuf *buf);
plBuf *pb_removeFromSendQ(void );
uint32_t pb_addToRecvQ(plBuf *buf);
plBuf *pb_removeFromRecvQ(void );
uint32_t pb_PLBufToBuf(plBuf *buf, void **);
uint32_t pb_BufToPLBuf(void *buf, plBuf **);
uint32_t pb_getRecvBuffer (uint32_t connect_id,uint32_t nBufSize, uint32_t flags, void ** buf);


#ifdef __cplusplus
}
#endif
#endif // _PTCPBUFFER_H
