/**************************************************************************** 
//             INTEL CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or 
//  nondisclosure agreement with Intel Corporation and may not be copied 
//  or disclosed except in accordance with the terms of that agreement. 
//      Copyright (c) 2000 Intel Corporation. All Rights Reserved.
//
//
//  $Workfile:   slist.h  $
//  $Revision: 1.6 $
//  $Date: 2003/11/17 16:57:53 $
//
//  Purpose:    Provides prototypes for all the methods used in slist.c
//
****************************************************************************/

#if !defined SLIST_HDR
#define SLIST_HDR

#include <csdatatypes.h>

#ifdef __cplusplus
extern "C" {
#endif

/* this is an item in the list and stores pointers to data, next and
   previous list items */
typedef struct tag_LISTITEM
{
    void *      data;
    struct tag_LISTITEM *  next;      
    struct tag_LISTITEM *  prev;      
} LISTITEM, * LPLISTITEM;

typedef const LISTITEM * LPCLISTITEM;
    

/* this is created when Createlist is called and stores the head and tail
   of the list. */
typedef struct tag_LIST
{
    LPLISTITEM  head;
    LPLISTITEM  tail;
    short       count;
} SLIST, * LPLIST;

typedef const LPLIST LPCLIST;
    

/* this is created whenever an iterator is needed for iterating the list */
typedef struct tag_LISTITR
{
    LPLIST      list;
    LPLISTITEM  current;
} LISTITR, * LPLISTITR;

typedef const LISTITR * LPCLISTITR;
    

/* Function prototypes */

/* List operations */
extern LPLIST ListCreate(void);
extern BOOL ListAddItem(LPLIST list, void * data);
extern BOOL ListRemoveItem(LPLIST list, void * data);
extern BOOL ListFindItem(LPLIST list, void * data);
extern void * ListGetFirst(LPLIST list);
extern void * ListGetLast(LPLIST list);
extern int ListGetCount(LPLIST list);
extern void ListDelete(LPLIST list);
extern void ListFreeDelete(LPLIST list);


/* List Iterator operations */
extern LPLISTITR ListItrCreate(LPLIST list);
extern LPLISTITR ListItrFirst(LPLIST list, LPLISTITR listItr);
extern void * ListItrGetNext(LPLISTITR listItr);
extern BOOL ListItrInsertAfter(LPLISTITR listItr, void * data);
extern BOOL ListItrInsertBefore(LPLISTITR listItr, void * data);
extern BOOL ListItrRemoveItem(LPLISTITR listItr);
extern void ListItrDelete(LPLISTITR listItr);

#ifdef __cplusplus
}
#endif


#endif /* SLIST_HDR */
