/******************************************************************************
//           INTEL CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or
//  nondisclosure agreement with Intel Corporation and may not be copied
//  or disclosed except in accordance with the terms of that agreement.
//      Copyright (c) 1999-2000 Intel Corporation.  All Rights Reserved.
//
// $Workfile:   timer.h  $
// $Revision: 1.6 $
// $Date: 2003/11/17 16:57:53 $
//
// Description:
//
// Contents:
//
******************************************************************************/




#ifdef __cplusplus
extern "C" {
#endif

#ifndef TIMER_H
#define TIMER_H

#include <time.h>
#include <pil.h>
#include "csdatatypes.h"
#include "csportability.h"


/******************************************************************************
// Struct:  TimerInfo
//
// Description:
//     Used to hold a single timer entry in the linked list of active 
//     timers.
//
******************************************************************************/
struct TimerInfo 
{
    ULONG handle;
    time_t time;
    struct TimerInfo *next;
    TIMER_CALLBACK cbFunc; /*will be called if handle was generated
                            *from a layer other than the cops client
                            *- is NULL otherwise*/
};


/******************************************************************************
//
// Global variables used within the Timer module.
//
******************************************************************************/
extern void (*TimeoutCallback)(ULONG handle);  // Pointer to timer callback


/******************************************************************************
// Method:  TimerInit
//
// Description:
//     This will initialize the timer module. Allocating space and
//     creating any threads as needed. This routine should only be called
//     once. If it is called a second time it will simply return
//     immediately.
//
******************************************************************************/

int TimerInit( void (*CbCsTimeout)(ULONG handle) );


/******************************************************************************
// Method:  TimerDeinit
//
// Description:
//     This routine is used to deinitialize the timer module. It will join
//     any threads and deallocate any memory as needed. It will also
//     stop all current timers without signalling them first. In other
//     words, if there are active timers when this is called, they will
//     never go off. This routine may only be called if the timer module
//     has already been initialized. If the timer module hasn't been
//     initialized, this routine will do nothing.
//
******************************************************************************/

int TimerDeinit( void );


#endif

#ifdef __cplusplus
}
#endif

