/******************************************************************************
//             INTEL CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or 
//  nondisclosure agreement with Intel Corporation and may not be copied 
//  or disclosed except in accordance with the terms of that agreement. 
//      Copyright (c) 2000-2001 Intel Corporation. All Rights Reserved.
//
//  $Workfile:   pdatagram.c  $
//  $Revision: 1.6 $
//  $Date: 2003/11/17 16:57:53 $
//
//  Description: Implementation of datagram functions
//
******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <time.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <pil.h>
#include <stddef.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#ifdef _VXWORKS_
#include <ioLib.h>
#include <pipeDrv.h>
#else
#include <fcntl.h>
#endif

#include "csportability.h" 
#include "pdatagram.h"
#include "ptcpbuffer.h"

/* Callback funcs */
void DatagramCbCsNetReceive(ULONG,const char *,int);
void DatagramCbCsNetDisconnect(ULONG);
void DatagramCbCsTimerExpire(ULONG);
void DatagramCbCsNetAccept(ULONG, char *);

#define DG_SERVER	0x00
#define DG_CLIENT	0x01

#define DATAGRAM_COOKIE 0xFFFFFFFF

ULONG clientconnecthandle;
ULONG serverconnecthandle;
ULONG connecthandle;

/* globals */
CBServerDatagramFuncs g_pDatagramServer;
CBClientDatagramFuncs g_pDatagramClient;
CB_DatagramReceive _g_fpCBRecv;
CB_DatagramException _g_fpCBExp;

int g_server=DG_SERVER;

void DatagramCbBufferReceive(ULONG connect_id , const char * buf, int size)
{
    CSTRACE((CSTRACE_ERR,"ADD "));
    _g_fpCBRecv( (void *)buf, connect_id, size);
    CSTRACE((CSTRACE_ERR,"ADD 1"));
}

void DatagramCbCsNetReceive(ULONG connect_id , const char * msg, int size)
{
    plBuf *pbuf;
    plmsg *pmsg; 
    plHeader tmpHeader;
    void *buf;
    int length;
    int bufsize;
    int notdone = 1;
    int ndone = 0;
    CSTRACE((CSTRACE_INFO,"[DATAGRAM RECV]\n"));
    pmsg = (plmsg *)(msg+ndone);
    
    if (pmsg->plheader.cookie != DATAGRAM_COOKIE)	
    {
        CSTRACE((CSTRACE_ERR,"[PORTAB]recieved data with invalid cookie \n"));
        return ;  
    }
    
    while (notdone)
    {
        if (size < sizeof(plHeader))
        {
            memcpy(&tmpHeader,msg+ndone,size);
            read(connect_id, ((unsigned char *)&tmpHeader + size), sizeof(plHeader)-size);
            ndone += size;
            size = 0;
            pmsg = (plmsg*)(&tmpHeader);
        }
        else
        {
            pmsg = (plmsg *)(msg+ndone);
        }

        length = CSNTOHL(pmsg->plheader.length); 
        bufsize = length - sizeof (plHeader);
        if (pb_getRecvBuffer (connect_id, bufsize, 0, &buf))
            return ;  
        pb_BufToPLBuf(buf,&pbuf);
        pbuf->pbufheader.connect_id = connect_id;
	 
        if ( length == size )
        {
            memcpy ((void *)&pbuf->plheader , (msg+ndone), length);
            // We recv a full buffer
            pmsg->plheader.length = bufsize;
            notdone = 0;
            ndone +=length;      	 
        }
        else if ( length > size )
        {
            int nread;
            // Recv incomplete buffer
            if (size==0)
            {
                memcpy ((void *)&pbuf->plheader , &tmpHeader, sizeof(plHeader));
                size = sizeof(plHeader);
            }
            else
            {
                memcpy ((void *)&pbuf->plheader , (msg + ndone), size);
                ndone +=size;      	 
            }
            nread = read(connect_id, ((unsigned char *)&pbuf->plheader + size), length-size);
            // ASSERT that we read all the data;
            pmsg->plheader.length = bufsize;
            notdone = 0;
            ndone+=nread;
        }
        else if  ( length < size )
        {
            memcpy ((void *)&pbuf->plheader , (msg + ndone), length);
            pmsg->plheader.length = bufsize;
            ndone +=length;      	 
            // Add this buffer to the recv Q
            pb_addToRecvQ(pbuf);
            // And deal with the rest
            size = size - length;
        }
    }
    // Add to the recv Q
    CSTRACE((CSTRACE_ERR,"ADD recv%x", pbuf));
    pb_addToRecvQ(pbuf);
    CSTRACE((CSTRACE_ERR,"ADD recv%x", pbuf));
}

void DatagramCbCsNetDisconnect(ULONG connect_id)
{
        _g_fpCBExp(DATAGRAM_EX_CONNECTION_CLOSE, connect_id);	
}

void DatagramCbCsTimerExpire(ULONG x)
{
    if (g_server==DG_SERVER)
    {
    }
    else
    {
    }	
}

void DatagramCbCsNetAccept(ULONG connect, char *client_ip)
{
    if (g_server==DG_SERVER)
    {
    /*
        CSTRACE ("connection from client\n");
	 serverconnecthandle = x;
	 connecthandle = serverconnecthandle;
	 CsNetSend(connecthandle, "abc", 3); */
        CSTRACE((CSTRACE_INFO, "[PORTAB] Client connection from %s connect_id %d\n",
		client_ip, connect));
    }
}

/******************************************************************************
//
// uint32_t InitializeDatagramServer (CBServerDatagramParams *serverparams);
//
// ARGUMENTS:  serverparams - pointer to CBServerDatagramParams 
//
// DESCRIPTION: Initialize the Datagram Server API.
//
// RETURNS:     CS_RET_SUCCESS
//
//
******************************************************************************/
uint32_t InitializeDatagramServer (
	CBServerDatagramFuncs *serverparams)
{
    unsigned short int csport=DATAGRAM_SERVER_PORT;	
    if (g_bufferInit == FALSE)
    {
        CSTRACE((CSTRACE_ERR, "[PORTAB] Initialize Buffer API before Datagram\n"));
	 return FALSE;
    }

    g_server = DG_SERVER;
    g_pDatagramServer = *serverparams;
    _g_fpCBRecv = g_pDatagramServer.datagram_receive;
    _g_fpCBExp = g_pDatagramServer.datagram_exception;
    
    CsPortInit( DatagramCbCsNetReceive,
                  DatagramCbCsNetDisconnect,
                  DatagramCbCsTimerExpire,
                  DatagramCbCsNetAccept
                 );

    CsNetListen(csport);
    return CS_RET_SUCCESS;
}

/******************************************************************************
//
// uint32_t InitializeDatagramClient (CBClientDatagramParams *clientparams);
//
// ARGUMENTS:  clientparams - pointer to CBClientDatagramParams 
//
// DESCRIPTION: Initialize the Datagram Client API.
//
// RETURNS:     CS_RET_SUCCESS
//
******************************************************************************/

uint32_t InitializeDatagramClient (
	CBClientDatagramFuncs *clientparams,
	uint32_t 		  *pconnect_id)
{
    CS_IPADDR csserverip;
    unsigned short int csport;
    
    if (!clientparams || !pconnect_id )
    	return FALSE;
    if (g_bufferInit == FALSE)
    {
        CSTRACE((CSTRACE_ERR, "[PORTAB] Initialize Buffer API before Datagram\n"));
	 return FALSE;
    }

    if (clientparams->connect_info.flags != DATAGRAM_IPV4)
    {
        CSTRACE((CSTRACE_ERR, "[PORTAB] Only IPV4 Datagram Connections supported\n"));
	 return FALSE;
    }

    g_server = DG_CLIENT;
    g_pDatagramClient = *clientparams;
    _g_fpCBRecv = g_pDatagramClient.datagram_receive;
    _g_fpCBExp = g_pDatagramClient.datagram_exception;

    CsPortInit( DatagramCbCsNetReceive,
                  DatagramCbCsNetDisconnect,
                  DatagramCbCsTimerExpire,
                  DatagramCbCsNetAccept
                 );

    csserverip.addr = g_pDatagramClient.connect_info.saddr.ipv4_addr.sin_addr.s_addr ;
    csport = g_pDatagramClient.connect_info.saddr.ipv4_addr.sin_port;

    CsNetConnect(CSINET_NTOA(&csserverip), csport , (ULONG *) pconnect_id , &csserverip);
    return CS_RET_SUCCESS;
}

/******************************************************************************
//
// uint32_t SendDatagram(uint32_t connect_id, void* msg, uint32_t size, uint32_t flags);
//
// ARGUMENTS:   connect_id - connection identifier
//              msg - buffer containing data
//              size - size of msg
//		flags - reserved ( pass 0) for later use.
//
// DESCRIPTION: Send Client/Server Datagram
//
// RETURNS:     CS_RET_SUCCESS
//
// NOTE : Call pb_getbuffer for allocating buf
******************************************************************************/

uint32_t SendDatagram(
	uint32_t connect_id, 
	void* msg, 
	uint32_t size, 
	uint32_t flags)
{
	plBuf *pbuf;
	if (PLBUF_SUCCESS == pb_BufToPLBuf(msg, &pbuf ))
	{
    	    if (pbuf->pbufheader.flags != PLBUF_ALLOC_BUFFER)
	        CSTRACE((CSTRACE_WARN, "[PORTAB] WARN Possible Buffer Corruption\n"));
    	    if (pbuf->pbufheader.connect_id != connect_id)
	        CSTRACE((CSTRACE_WARN, "[PORTAB] WARN Buffer requested on connect_id %d SENT on %d\n",
		pbuf->pbufheader.connect_id , connect_id));
    	    pbuf->pbufheader.connect_id = connect_id;
           pbuf->plheader.cookie = DATAGRAM_COOKIE;	
	    if (pbuf->plheader.length != size)
	        CSTRACE((CSTRACE_WARN, "[PORTAB] WARN Buffer requested length %d SENT size %d\n",
		pbuf->plheader.length , size));

	    pbuf->plheader.length = size;  
	    //printf ( "[DG]sending %s\n", pbuf->pbuf);
	    pb_addToSendQ(pbuf);
	    
	    CSTRACE((CSTRACE_INFO,"[DATAGRAM SEND]\n"));
	    return CS_RET_SUCCESS;
	}
       CSTRACE((CSTRACE_INFO,"[DATAGRAM SEND ERR]\n"));
	return FALSE;
}

/******************************************************************************
//
// uint32_t DeInitializeDatagram();
//
// ARGUMENTS: 
//
// DESCRIPTION: Deinitialize Datagram API
//
// RETURNS:     CS_RET_SUCCESS
//
******************************************************************************/
uint32_t DeInitializeDatagram()
{
	CsPortDeinit();
	return CS_RET_SUCCESS;
}

