/******************************************************************************
//             INTEL CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or 
//  nondisclosure agreement with Intel Corporation and may not be copied 
//  or disclosed except in accordance with the terms of that agreement. 
//      Copyright (c) 2000-2001 Intel Corporation. All Rights Reserved.
//
//  $Workfile:   ptcpbuffer.c  $
//  $Revision: 1.6 $
//  $Date: 2003/11/17 16:57:55 $
//
//  Description: Implementation of networking functions
//
******************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/types.h>
#include <time.h>
#include <netinet/in.h>
#include <netdb.h>
#include <arpa/inet.h>
#include <pil.h>
#include <stddef.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#ifdef _VXWORKS_
#include <ioLib.h>
#include <pipeDrv.h>
#else
#include <fcntl.h>
#endif

#include "cserror.h"
#include "csdatatypes.h"
#include "csportability.h"
#include "pgeneral.h"
#include "pdatagram.h"
#include "ptcpbuffer.h"
#include "slist.h"

#ifdef SOLARIS
 #include <strings.h>
#endif

#define TERMINATE_STR "Terminate thread"

/* global variables*/
static int _exit_all_threads=0;

static int g_bufferPipe[2]; /*pipe used to signal a thread to close*/
static PilThread g_SendBufferThreadHandle;
static PilThread g_RecvBufferThreadHandle;

static ULONG g_BufferLock;
PilSemaphore g_bufferSendSem;
PilSemaphore g_bufferRecvSem;

static ULONG g_SendLock;
PilSemaphore g_sendCountSem;

static ULONG g_RecvLock;
PilSemaphore g_recvCountSem;

int g_bufferInit=0;
plbufMemory g_plbufMemory;
plMemoryMgt *g_plMemoryMgt;

uint32_t SendBufferThread(void * arg);
uint32_t RecvBufferThread(void * arg);

int signalThreads ()
{
       PIL_ReleaseSemaphore ( g_bufferSendSem);
       PIL_ReleaseSemaphore ( g_bufferRecvSem);
       PIL_ReleaseSemaphore ( g_sendCountSem);
       PIL_ReleaseSemaphore ( g_recvCountSem);
       return PLBUF_SUCCESS;
}

uint32_t pb_PLBufIndex(plBuf *plbuf)
{
    unsigned char * bufaddress = (unsigned char *)plbuf;
    uint32_t index = (bufaddress - g_plbufMemory.pBufMem) / MAX_BUF_SIZE;
    return index ;
}/*pb_PLBufToBuf*/

int free_plbuf ( plBuf * ppbuf)
{
    ppbuf->pbufheader.connect_id = -1;
    ppbuf->pbufheader.flags = PLBUF_FREE_BUFFER;
    ppbuf->plheader.cookie = COOKIE;
    ppbuf->plheader.length = 0;
    return PLBUF_SUCCESS;
}

int FindFreeMem ( plbufMemory *pplBufMemory)
{
	return pplBufMemory->pBufFreeMem;
}

int SetFreeMem ( plbufMemory *pplBufMemory, int index)
{
//    ASSERT ( pplBufMemory->pBufFreeMem, index);
    if (index == pplBufMemory->pBufFreeTail )
    {
    	pplBufMemory->pBufFreeTail = -1;
    	pplBufMemory->pBufFreeMem = -1;
    }
    else
    {
    	pplBufMemory->pBufFreeMem = g_plMemoryMgt[index].next;
    	g_plMemoryMgt[pplBufMemory->pBufFreeMem].prev = -1;
    }
    return pplBufMemory->pBufFreeMem;
}

uint32_t ReturnFreeMem(plbufMemory *pplBufMemory, int index)
{
    int temp;
    if (pplBufMemory->pBufFreeTail != -1 )
    {
    	temp = pplBufMemory->pBufFreeTail;
       pplBufMemory->pBufFreeTail = index;
    
       g_plMemoryMgt[temp].next = index;
       g_plMemoryMgt[index].prev = temp;
       g_plMemoryMgt[index].next = -1;
       return pplBufMemory->pBufFreeMem;
    }
    else
    {
        pplBufMemory->pBufFreeTail = index;
        pplBufMemory->pBufFreeMem = index;

	 g_plMemoryMgt[index].next = -1;
	 g_plMemoryMgt[index].prev = -1;
	 return pplBufMemory->pBufFreeMem;
    }
}/*ReturnFreeMem*/

uint32_t AddSendMem(plbufMemory *pplBufMemory, int index)
{
    int temp;
//    //printf ("Adding %d to SendQ head %d tail %d\n", index, pplBufMemory->pBufSend, pplBufMemory->pSendTail);
    if (pplBufMemory->pSendTail != -1 )
    {
	 temp = pplBufMemory->pSendTail;
        pplBufMemory->pSendTail = index;
        g_plMemoryMgt[temp].next = index;
        g_plMemoryMgt[index].prev = temp;
        g_plMemoryMgt[index].next = -1;
	 return pplBufMemory->pBufSend;
    }	 
    else
    {
        pplBufMemory->pSendTail = index;
        pplBufMemory->pBufSend = index;

	 g_plMemoryMgt[index].next = -1;
	 g_plMemoryMgt[index].prev = -1;

	 return pplBufMemory->pBufSend;
    }
}/*AddSendMem*/

uint32_t RemoveSendMem(plbufMemory *pplBufMemory)
{
    int index;
    index = pplBufMemory->pBufSend;

  
    if (pplBufMemory->pBufSend == pplBufMemory->pSendTail/* NEVER == -1 ASSERT*/ )
    {
        pplBufMemory->pSendTail = -1;
        pplBufMemory->pBufSend = -1;
    }	 
    else
    {
        pplBufMemory->pBufSend = g_plMemoryMgt[index].next;
	 g_plMemoryMgt[pplBufMemory->pBufSend].prev = -1;
    }
//    //printf ("removing %d to SendQ head %d tail %d\n", index, pplBufMemory->pBufSend, pplBufMemory->pSendTail);
    return index;
}/*RemoveSendMem*/

uint32_t AddRecvMem(plbufMemory *pplBufMemory, int index)
{
    int temp;
 //   //printf ("Adding %d to RecvQ head %d tail %d\n", index, pplBufMemory->pBufRecv, pplBufMemory->pRecvTail);
    if (pplBufMemory->pRecvTail != -1 )
    {
	 temp = pplBufMemory->pRecvTail;
        pplBufMemory->pRecvTail = index;
        g_plMemoryMgt[temp].next = index;
        g_plMemoryMgt[index].prev = temp;
        g_plMemoryMgt[index].next = -1;
	 return pplBufMemory->pBufRecv;
    }	 
    else
    {
        pplBufMemory->pRecvTail = index;
        pplBufMemory->pBufRecv = index;

	 g_plMemoryMgt[index].next = -1;
	 g_plMemoryMgt[index].prev = -1;

	 return pplBufMemory->pBufRecv;
    }
}/*AddRecvMem*/

uint32_t RemoveRecvMem(plbufMemory *pplBufMemory)
{
    int index;
    index = pplBufMemory->pBufRecv;
    
    if (pplBufMemory->pBufRecv == pplBufMemory->pRecvTail/* NEVER == -1 ASSERT*/ )
    {
        pplBufMemory->pRecvTail = -1;
        pplBufMemory->pBufRecv = -1;
    }	 
    else
    {
        pplBufMemory->pBufRecv = g_plMemoryMgt[index].next;
	 g_plMemoryMgt[pplBufMemory->pBufRecv].prev = -1;
    }
//    //printf ("removing %d to RecvQ head %d tail %d\n", index, pplBufMemory->pBufRecv, pplBufMemory->pRecvTail);
    return index;
}/*RemoveRecvMem*/

int CreateSyncObjects (int nQSize)
{
    //All synchronization objects need to be created globally
    if(CsSyncCreate(&g_BufferLock) != CS_RET_SUCCESS)
        goto DELBUFLOCK;        
    if(CsSyncCreate(&g_SendLock) != CS_RET_SUCCESS)
        goto DELSENDLOCK;        
    if(CsSyncCreate(&g_RecvLock) != CS_RET_SUCCESS)
        goto DELRECVLOCK;        

    if ((g_bufferSendSem = PIL_CreateSemaphore(nQSize,nQSize))==NULL)
	goto DELBUFSEND;
    if ((g_bufferRecvSem = PIL_CreateSemaphore(nQSize,nQSize))==NULL)
	goto DELBUFRECV;
    if ((g_sendCountSem = PIL_CreateSemaphore(0,nQSize))==NULL)
	goto DELSENDSEM;
    if ((g_recvCountSem = PIL_CreateSemaphore(0,nQSize)))
	return TRUE;
    PIL_DestroySemaphore(g_recvCountSem);
DELSENDSEM :	PIL_DestroySemaphore(g_sendCountSem);
DELBUFRECV :	PIL_DestroySemaphore(g_bufferRecvSem);
DELBUFSEND :	PIL_DestroySemaphore(g_bufferSendSem);
DELRECVLOCK : CsSyncDestroy(g_RecvLock);   
DELSENDLOCK : CsSyncDestroy(g_SendLock);
DELBUFLOCK :   CsSyncDestroy(g_BufferLock);
    return FALSE;        
}
int DestroySyncObjects ()
{
    PIL_DestroySemaphore(g_recvCountSem);
    PIL_DestroySemaphore(g_sendCountSem);
    PIL_DestroySemaphore(g_bufferRecvSem);
    PIL_DestroySemaphore(g_bufferSendSem);
    CsSyncDestroy(g_RecvLock);   
    CsSyncDestroy(g_SendLock);
    CsSyncDestroy(g_BufferLock);
    return TRUE;
}

int PreAllocateAndInitialize (int qsize)
{
    int size = 0;
    int memsize = 0;
    plBuf *ppbuf;
    int Qsize=2*qsize;
    
    g_plbufMemory.total_Q_size =Qsize;
    memsize = Qsize * MAX_BUF_SIZE ;
    g_plbufMemory.pBufMem = (char *)malloc (memsize * sizeof(char));
    if (!g_plbufMemory.pBufMem)
    	return 0;

    g_plbufMemory.pBufFreeMem = 0;
    g_plbufMemory.pBufFreeTail = Qsize - 1;
    g_plbufMemory.pBufSend = -1;
    g_plbufMemory.pSendTail = -1;
    g_plbufMemory.pBufRecv = -1;
    g_plbufMemory.pRecvTail = -1;
    g_plbufMemory.buffers_used=0;

    g_plMemoryMgt = (plMemoryMgt *) calloc (Qsize , sizeof (plMemoryMgt));
    if (!g_plMemoryMgt)
    {
       free (g_plbufMemory.pBufMem);
    	return 0;
    }

    for (size=0; size < Qsize; size++)
    {
	ppbuf = (plBuf *)((unsigned char *) g_plbufMemory.pBufMem	+ (size*MAX_BUF_SIZE ));
	free_plbuf (ppbuf);

	if ( size == (Qsize -1))
		g_plMemoryMgt[size].next = -1;
	else
		g_plMemoryMgt[size].next = ( size+1 ) % Qsize;
	
	if ( size == 0)
		g_plMemoryMgt[size].prev = -1;
	else
		g_plMemoryMgt[size].prev = (size - 1) % Qsize;
    }
    return 1;
}

int DestroyAllocate ()
{
    free (g_plbufMemory.pBufMem);
    free (g_plMemoryMgt);
    return TRUE;
}

/*Function prototypes*/

uint32_t getBuffer (uint32_t connect_id,uint32_t nBufSize, uint32_t flags, void ** buf, PilSemaphore *Sem)
{
    uint32_t timeout=SEMAPHORE_TIMEOUT;
    int multiplier=2;
    int retval;
     	
    plBuf *fplbuf; 	
    uint32_t freebuffIndex=0;
    
    if (nBufSize== 0 || ( nBufSize > MAX_USER_BUF) )
	return PLBUF_ERROR_INVALID_SIZE ; // Return Error Condition
    if (!buf)
       return PLBUF_ERROR_INVALID_ARG;
    
    // NOT enough flexiblity in the semaphore implementation Of PIL
    while (multiplier)
    {      
    	retval = PIL_AcquireSemaphore ( *Sem,
    				timeout*multiplier);
	if (_exit_all_threads == 1)
	{
	     // Something wrong get out.
	    return PLBUF_ERROR_OUTOFMEMORY;
	}
	if (retval == PIL_OK)
	    multiplier = 0;
	else
	{
	    multiplier = (multiplier == 65536)? 2 : (multiplier * multiplier);
	}
    }
    
// get the lock to the global data.
    CsSyncLock(g_BufferLock);

    g_plbufMemory.buffers_used++;
    freebuffIndex = FindFreeMem(&g_plbufMemory);
    //printf ( "GET BUFF %d\n", freebuffIndex );
    fplbuf = (plBuf *) ( g_plbufMemory.pBufMem + (MAX_BUF_SIZE * freebuffIndex));
    CSTRACE (( CSTRACE_ERR, " [%d] %x, %x , %x\n", freebuffIndex, g_plbufMemory.pBufMem
    ,fplbuf,fplbuf->pbuf));
    SetFreeMem(&g_plbufMemory, freebuffIndex);
    
// release lock
    CsSyncUnlock(g_BufferLock);
    
    fplbuf->pbufheader.flags = PLBUF_ALLOC_BUFFER;
    fplbuf->pbufheader.connect_id = connect_id;
    fplbuf->plheader.cookie = COOKIE;	
    fplbuf->plheader.length = nBufSize;	
    *buf = fplbuf->pbuf;
    return PLBUF_SUCCESS;
}/*pb_getBuffer*/

/******************************************************************************
// Method:  void *pb_getBuffer (uint32_t connect_id, uint32_t nBufSize, uint32_t flags)
//
// Parameters:
//     connect_id - connection identifier for the session.
//     nBufSize - size of buffer requested.
//     flags - flags for session and buffer.
//
// Description:
//     Used to request a buffer of size nBufSize. This is a blocking call. 
//     If free buffers are available a buff is given. If Buffers are not 
//     available, this call blocks until buffers are available.
//
// Returns:
// retuen a pointer to the usable buffer area.
// 
******************************************************************************/
uint32_t pb_getBuffer (uint32_t connect_id,uint32_t nBufSize, uint32_t flags, void ** buf)
{
    return getBuffer (connect_id,nBufSize, flags, buf, &g_bufferSendSem);
}

uint32_t pb_getRecvBuffer (uint32_t connect_id,uint32_t nBufSize, uint32_t flags, void ** buf)
{
    return getBuffer (connect_id,nBufSize, flags, buf, &g_bufferRecvSem);
}

/******************************************************************************
// Method:  void *pb_freeBuffer (void *pbuf)
//
// Parameters:
//     pbuf - buffer to free.
//
// Description:
//     Use this call to free the buffer at the layer that discards the buffer and 
//     has no need for the buffer anymore.. 
//
// Returns:
// retuen a pointer to the usable buffer area.
// 
******************************************************************************/
uint32_t freeBuffer (void *pbuf, PilSemaphore *Sem)
{
    plBuf *ppbuf;
    uint32_t index ;
    if ( !pbuf)
        return PLBUF_ERROR_INVALID_ARG;
    // Check if pbuf falls in a valid boundary of the preinitialized memory
    if ( pb_BufToPLBuf ( pbuf , &ppbuf))
        return PLBUF_ERROR_INVALID_ARG;
    CSTRACE (( CSTRACE_ERR,"free_buffer\n")); 		
    index = pb_PLBufIndex(ppbuf);

// get the lock
    CsSyncLock(g_BufferLock);
    g_plbufMemory.buffers_used--;
    g_plbufMemory.pBufFreeMem = ReturnFreeMem(&g_plbufMemory, index);
// release lock
    CsSyncUnlock(g_BufferLock);

    ppbuf->pbufheader.flags = PLBUF_FREE_BUFFER;
    PIL_ReleaseSemaphore ( *Sem);
    return PLBUF_SUCCESS;
}/*pb_freeBuffer*/

uint32_t pb_freeSendBuffer (void * pbuf)
{
    return freeBuffer (pbuf, &g_bufferSendSem);
}

uint32_t pb_freeBuffer (void *pbuf)
{
    return freeBuffer (pbuf, &g_bufferRecvSem);
}
/******************************************************************************
// Method:  void *pb_addToSendQ (plBuf *plbuf)
//
// Parameters:
//     plbuf - buffer to add to the send Q  .
//
// Description:
//     This call is used to add a buffer to the send Q	
//
// Returns:
// return a pointer to the usable buffer area.
// 
******************************************************************************/
uint32_t pb_addToSendQ (plBuf *plbuf)
{
    void *pbuf;
    uint32_t index;
    if ( !plbuf)
        return PLBUF_ERROR_INVALID_ARG;
    // Check if plbuf falls in a valid boundary of the preinitialized memory
    // If if doesnot, then return error.
    // If it falls on a valid boundary then add the buffer to the Q of
    // buffer to be sent over the interconnect.
    if ( pb_PLBufToBuf(plbuf, &pbuf))
        return PLBUF_ERROR_INVALID_ARG;
    plbuf->pbufheader.flags = PLBUF_SEND_BUFFER;
    index = pb_PLBufIndex(plbuf);

    CsSyncLock(g_SendLock);
    AddSendMem(&g_plbufMemory,index);
    CsSyncUnlock(g_SendLock);

    PIL_ReleaseSemaphore ( g_sendCountSem);
    return PLBUF_SUCCESS;
}/*pb_addToSendQ*/
 
/******************************************************************************
// Method:  plBuf *pb_removeFromSendQ()
//
// Parameters:
//
// Description:
//     This call is used to retrieve a buffer from the send Q and deliver it 
//     over the interconnect. This is a blocking call if there are no buffers
//     to send over the interconnect.
//
// Returns:
//     CS_RET_SUCCESS if successful.
// 
******************************************************************************/
plBuf *pb_removeFromSendQ()
{
    // Check if there are any send buffers in the sendQ.
    // If there are no buffers in the send Q, then block until there are 
    // buffers available.
    plBuf *buf;
    int index;
    uint32_t timeout=SEMAPHORE_TIMEOUT;
    int multiplier=2;
    int retval;
    while (multiplier)
    {      
        // Wait For send buffers.
       retval = PIL_AcquireSemaphore ( g_sendCountSem,
    				multiplier*timeout);
	if (retval == PIL_OK)
	    multiplier = 0;
	else
	{
	    multiplier = (multiplier == 65536)? 0 : (multiplier * multiplier);
		CSTRACE((CSTRACE_INFO, "%s timeout sem %d\n", __FUNCTION__, multiplier));
            if (_exit_all_threads == 1)
            {
     	         // Something wrong get out.
	        return NULL;
            }
	}
    }
    if (retval != PIL_OK)
    	return NULL;
    

    // Lock Buffer sync
    CsSyncLock(g_SendLock);
    index = RemoveSendMem(&g_plbufMemory);
    CsSyncUnlock(g_SendLock);

    if (index == -1)
		return NULL;
    else
    {
        buf = (plBuf *) ((unsigned char *) g_plbufMemory.pBufMem + (index *MAX_BUF_SIZE));
        return buf;
    }
}/*pb_removeFromSendQ*/

/******************************************************************************
// Method:  void *pb_addToRecvQ (plBuf *plbuf)
//
// Parameters:
//     pbuf - buffer to free.
//
// Description:
//     This call is used by the thread listening for msgs over the interconnect to
//     add a buffer to the Recvd Q. The buffers can be retrived by another blocking
//     call pb_removeFromTheRecvQ.
//
// Returns:
// 
******************************************************************************/
uint32_t pb_addToRecvQ (plBuf *plbuf)
{
    void *pbuf;
    uint32_t index;
    if ( !plbuf)
        return PLBUF_ERROR_INVALID_ARG;
    // Check if plbuf falls in a valid boundary of the preinitialized memory
    // If if doesnot, then return error.
    // If it falls on a valid boundary then add the buffer to the Q of
    // buffer to be sent over the interconnect.
    if ( pb_PLBufToBuf(plbuf, &pbuf))
        return PLBUF_ERROR_INVALID_ARG;
    plbuf->pbufheader.flags = PLBUF_RECV_BUFFER;
    index = pb_PLBufIndex(plbuf);

    CsSyncLock(g_RecvLock);
    AddRecvMem(&g_plbufMemory,index);
    CsSyncUnlock(g_RecvLock);

    PIL_ReleaseSemaphore ( g_recvCountSem);

    return PLBUF_SUCCESS;
}/*pb_addToRecvQ*/
 
/******************************************************************************
// Method:  plBuf *pb_removeFromRecvQ()
//
// Parameters:
//
// Description:
//     This call is used to retrieve buffers from the RecvQ. Blocking call blocks
//     until their are available buffers.
//
// Returns:
//     CS_RET_SUCCESS if successful.
// 
******************************************************************************/
plBuf *pb_removeFromRecvQ()
{
    // Check if there are any send buffers in the sendQ.
    // If there are no buffers in the send Q, then block until there are 
    // buffers available.
    plBuf *buf;
    int index;
    uint32_t timeout=SEMAPHORE_TIMEOUT;
    int multiplier=2;
    int retval;
    
    while (multiplier)
    {      
        // Wait For send buffers.
       retval = PIL_AcquireSemaphore ( g_recvCountSem,
    				multiplier*timeout);
       if (_exit_all_threads == 1)
       {
    	// Something wrong get out.
	return NULL;
       }
	if (retval == PIL_OK)
	    multiplier = 0;
	else
	{
	    multiplier = (multiplier == 65536)? 0 : (multiplier * multiplier);
		CSTRACE((CSTRACE_INFO, "%s timeout sem %d\n", __FUNCTION__, multiplier));
	}
    }
    if (retval != PIL_OK)
    	return NULL;
 
    // Lock Buffer sync
    CsSyncLock(g_RecvLock);
    index = RemoveRecvMem(&g_plbufMemory);
    CsSyncUnlock(g_RecvLock);

    buf = (plBuf *) ((unsigned char *) g_plbufMemory.pBufMem + (index *MAX_BUF_SIZE));
    return buf;
}/*pb_removeFromRecvQ*/

/******************************************************************************
// Method:  uint32_t pb_PLBufToBuf(plBuf *plbuf, void **buf)
//
// Parameters:
//
// Description:
//     Use this call to convert a plbuf pointer to a void* to used to fill or access
//     the data part.
//
// Returns:
//     CS_RET_SUCCESS if successful.
// 
******************************************************************************/

uint32_t pb_PLBufToBuf(plBuf *plbuf, void **buf)
{
    unsigned char * bufaddress = (unsigned char *)plbuf;
     
    if ( (g_plbufMemory.pBufMem - bufaddress) % MAX_BUF_SIZE )
    	return PLBUF_ERROR_INVALID_ARG;
    *buf = plbuf->pbuf;	
    return PLBUF_SUCCESS;
}/*pb_PLBufToBuf*/

/******************************************************************************
// Method:  uint32_t pb_BufToPLBuf(void *buf, plbuf **pplbuf)
//
// Parameters:
//
// Description:
//     Use this call to convert a void* to a plbuf pointer, used before adding 
//     to the Q's.
//
// Returns:
//     CS_RET_SUCCESS if successful.
// 
******************************************************************************/

uint32_t pb_BufToPLBuf(void *buf, plBuf **pplbuf)
{
    unsigned char *bufaddress = buf;
    uint32_t off =  bufaddress - g_plbufMemory.pBufMem;
    int bufnum = off / MAX_BUF_SIZE;
    plBuf *plbuf = (plBuf *) (g_plbufMemory.pBufMem + (bufnum * MAX_BUF_SIZE) );

    CSTRACE(( CSTRACE_ERR, " %x %x %d %x [%x] %x %x\n", g_plbufMemory.pBufMem, buf, bufnum, plbuf->pbuf, MAX_BUF_SIZE, bufaddress, off));	
    if ( plbuf->pbuf != buf) 
    	return PLBUF_ERROR_INVALID_ARG;
    *pplbuf = plbuf;
    return PLBUF_SUCCESS;
}/*pb_BufToPLBuf*/

/******************************************************************************
// Method:  BOOL pb_PlBufferInit (int nQSize)
//
// Parameters:
//     Buf - Buffer to return to the Q.
//
// Description:
//     Used to request a buffer of size nBufSize. This is a blocking call. 
//     If free buffers are available a buff is given. If Buffers are not 
//     available, this call blocks until buffers are available.
//
// Returns:
//     PLBUF_SUCCESS if successful.
//     PLBUF_OUTOFMEMORY if successful.
// 
******************************************************************************/
uint32_t pb_PlBufferInit (uint32_t nQSize)
{
    int retval;
    CSTRACE ( (CSTRACE_INFO, "[PORTAB]ULONG BufferInit(...)\n") );

    if ( 0 == PreAllocateAndInitialize(nQSize))
       return PLBUF_ERROR_OUTOFMEMORY;

#ifdef _VXWORKS_
    pipeDevCreate("/pipe/Interconnect/buffer",1,strlen(TERMINATE_STR)+1);
    g_bufferPipe[0] = g_bufferPipe[1] = open("/pipe/Interconnect/buffer",O_RDWR,0777);
#else
    if(pipe(g_bufferPipe))
    {
        CSTRACE((CSTRACE_ERR, "[PORTAB] Unable to create pipe.\n"));
        return PLBUF_ERROR_OUTOFMEMORY;    
    }
#endif
   
    if (!CreateSyncObjects(nQSize))
    {
       close(g_bufferPipe[0]);
#ifndef _VXWORKS_
       close(g_bufferPipe[1]);
#endif 
       DestroyAllocate();
       return PLBUF_ERROR_OUTOFMEMORY;    
    }
     
    if (0!= (g_SendBufferThreadHandle = PIL_CreateThread(
                                          SendBufferThread,
                                          DEFAULT_STACK_SIZE,
                                          NULL,
                                          DEFAULT_THREAD_PRIORITY,
                                          NULL,
                                          0)))
    {
        if (0 != (g_RecvBufferThreadHandle = PIL_CreateThread(
                                          RecvBufferThread,
                                          DEFAULT_STACK_SIZE,
                                          NULL,
                                          DEFAULT_THREAD_PRIORITY,
                                          NULL,
                                          0)))
	{
	    g_bufferInit = 1;
       	    CSTRACE((CSTRACE_INFO,"Buffer Initiallized, started threads\n", retval));
           return PLBUF_SUCCESS;
	}
	// Recv Thread Creation failed
       retval = write (g_bufferPipe[1], TERMINATE_STR, sizeof(TERMINATE_STR));
       CSTRACE((CSTRACE_INFO,"WRITE RETUENED %d BYTES\n", retval));
       if(retval == sizeof(TERMINATE_STR))
       {
	    _exit_all_threads = 1;
	    signalThreads ();
    	    CSTRACE ((CSTRACE_INFO,"[PORTAB]: Waiting for SendBufferThread\n"));
	    PIL_WaitForThread(g_SendBufferThreadHandle, PIL_INFINITE_TIMEOUT);
	}
    }
    // Send Thread Creation failed
    CSTRACE((CSTRACE_INFO,"[PORTAB] could not create SendBufferThread\n"));
    DestroySyncObjects();
    close(g_bufferPipe[0]);
#ifndef _VXWORKS_
    close(g_bufferPipe[1]);
#endif
    DestroyAllocate ();
    return PLBUF_ERROR_OUTOFMEMORY;
}/*pb_PlBufferInit*/

/******************************************************************************
// Method:  BOOL pb_PlBufferDeinit ()
//
// Parameters:
//
// Description:
//
// Returns:
//     CS_RET_SUCCESS if successful.
// 
******************************************************************************/
uint32_t pb_PlBufferDeInit()
{
    int retval;

    if( g_bufferInit == FALSE )
    {
    	CSTRACE ((CSTRACE_ERR, "[PORTAB] Need to call pb_PlBufferInit() first\n"));
	return PLBUF_ERROR_NOTINITIALIZED;
    }
   
    retval = write (g_bufferPipe[1], TERMINATE_STR, sizeof(TERMINATE_STR));
    CSTRACE((CSTRACE_INFO,"WRITE RETUENED %d BYTES\n", retval));
    if(retval == sizeof(TERMINATE_STR))
    {
	_exit_all_threads = 1;
	signalThreads ();
    	CSTRACE ((CSTRACE_INFO,"[PORTAB]: Waiting for SendBufferThread\n"));
	PIL_WaitForThread(g_SendBufferThreadHandle, PIL_INFINITE_TIMEOUT);
    	CSTRACE ((CSTRACE_INFO,"[PORTAB]: Waiting for RecvBufferThread\n"));
	PIL_WaitForThread(g_RecvBufferThreadHandle, PIL_INFINITE_TIMEOUT);

       DestroySyncObjects();
	DestroyAllocate ();
       g_bufferInit = 0;
	return PLBUF_SUCCESS;
    }
    CSTRACE ((CSTRACE_INFO,"[PORTAB] Couldn't terminate  SendBufferThread\n"));
    return PLBUF_ERROR_UNKNOWN;
}/*pb_PlBufferDeinit()*/


/******************************************************************************
// Method:  void *SendBufferThread (void *arg)
//
// Parameters:
//     arg - arg to the SendBufferThread thread.
//
// Description:
//
// Returns:
// 
******************************************************************************/
uint32_t SendBufferThread(void * arg)
{
    plBuf *buf=NULL;
    pplmsg *msg;
    uint32_t size;
    // Algo
    // Have a while loop that checks the Q buffer for available buffers and sends them over the tcp connection.
    // When a buffer is available make the socket call to send the packet.
    // Return the buffer back to the Q.
   
	CSTRACE((CSTRACE_ERR," %s  0x%08x, 0x%08x\n",__FUNCTION__,PIL_GetCurrentThreadId(), g_SendBufferThreadHandle));
   while (1)
   {
   	buf = pb_removeFromSendQ();	

	CSTRACE((CSTRACE_ERR," %s\n", __FUNCTION__));
	if (buf)
	{
	    msg = (pplmsg *) &buf->plheader;
	    // Use the socket to send the message
	    size = buf->plheader.length + sizeof( plHeader);
	    //printf ( "[BU]sending %s\n", buf->pbuf);
	    buf->plheader.length = CSHTONL( size );

	    CsNetSend( buf->pbufheader.connect_id , (void *)msg,  size);
           CSTRACE (( CSTRACE_ERR, " [BUFFER] %d	%d\n", buf->pbufheader.connect_id , size));
           // Free the buffer
	    pb_freeSendBuffer (buf->pbuf);
	}
	else
	{
	    /* USE_ THE TERMINATE STR TO CHECK FOR TERMINATE CONDITIONS. */
	    /* For Now use the flag, check what can be done later. */
	    if (_exit_all_threads )
	    {
		CSTRACE((CSTRACE_ERR,"exiting %s\n", __FUNCTION__));
               PIL_ExitThread(0);                        
            }
	}
   }
}

/******************************************************************************
// Method:  void *RecvBufferThread (void *arg)
//
// Parameters:
//     arg - arg to the SendBufferThread thread.
//
// Description:
//
// Returns:
// 
******************************************************************************/
uint32_t RecvBufferThread(void * arg)
{
   int size;
   plBuf *plbuf=NULL;
   
	CSTRACE((CSTRACE_ERR,"1 %s\n", __FUNCTION__));
   while (1)
   {
   	plbuf = pb_removeFromRecvQ();
	/* USE_ THE TERMINATE STR TO CHECK FOR TERMINATE CONDITIONS. */
	/* For Now use the flag, check what can be done later. */
	CSTRACE((CSTRACE_ERR," %s\n", __FUNCTION__));
	if (_exit_all_threads )
	{
		CSTRACE((CSTRACE_ERR,"exiting %s\n", __FUNCTION__));
            PIL_ExitThread(0);     
	}                   
	if (plbuf)
	{
	    	//printf ( "[BU]recv %s\n", plbuf->pbuf);
	    	//printf ( "[BU]sending %s\n", buf->pbuf);
	  	//  buf->plheader.length = CSHTONL( size );
        	plbuf->plheader.length  = CSNTOHL(plbuf->plheader.length); 
	    	size = plbuf->plheader.length - sizeof( plHeader);

        	plbuf->plheader.length  = size; 
	
		DatagramCbBufferReceive(plbuf->pbufheader.connect_id , plbuf->pbuf, size);
	}
		
	plbuf = NULL;
   }
}

