/****************************************************************************** 
//             INTEL CORPORATION PROPRIETARY INFORMATION
//  This software is supplied under the terms of a license agreement or 
//  nondisclosure agreement with Intel Corporation and may not be copied 
//  or disclosed except in accordance with the terms of that agreement. 
//      Copyright (c) 2000 Intel Corporation. All Rights Reserved.
//
// $Workfile:   ptrace.c  $
// $Revision: 1.6 $
// $Date: 2003/11/17 16:57:55 $
//
//  Purpose:    Provides a function to display a string to the output
//              windows while tracing the application in Debug mode.
//              you can pass variable number of parameters to the 
//              function.
//
******************************************************************************/

#include <stdio.h>
#include <stdarg.h>
#include "csportability.h"

#ifdef _DEBUG
/******************************************************************************
//
// METHOD:      void PrintTraceMsg(int level, LPCTSTR lpszFile, int nLine, 
//                                  LPCTSTR lpszFormat, ...)
//
// ARGUMENTS:   level : can be CSTRACE_INFO, CSTRACE_WARN, CSTRACE_ERR
//              used to suppress various levels of trace messages while
//              tracing the error in debug mode.
//              lpszFile : Name of the file and Thread name
//              nLine : Line number of the current thread
//              lpszFormat : printf style format string for output string.
//
// DESCRIPTION: Prints trace string to output window while in debug mode.
//              
//
// RETURNS:
//              
//
******************************************************************************/

void PrintTraceMsg(int level, char *lpszFile,
                   int nLine, char *lpszFormat, ...)
{
    char           szTraceMsg[200];
    char*           lpszPfx;
    int             cbMsg;
    va_list         lpArgv;

    lpszPfx = "[%08X] %s(%d) : ";

    // Format trace msg prefix
    cbMsg = sprintf(szTraceMsg, 
                    lpszPfx, 
                    PIL_GetCurrentThread(),
                    lpszFile, 
                    nLine);
    
    // Append trace msg to prefix.
    va_start(lpArgv, lpszFormat);
    cbMsg = vsprintf(szTraceMsg + cbMsg, lpszFormat, lpArgv);
    va_end(lpArgv);

    fprintf(stderr, szTraceMsg);
}

#endif









