/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/*
 * This file contains the functions and stuff that will generate the
 * decapsulation routines for a given list of "types"
 */

#include <stdio.h>
#include <stdlib.h>
#include <list.h>
#include "types.h"
#include "typesinternal.h"

extern itype globaltypelist;

char* format_decap_type_header = 
"char* decapsulate_%s (\n"\
"        %1$s* a,\n"\
"        char* buf\n"\
"        )";

char* format_decap_type_array_header = 
"char* decapsulate_array_%s (\n"\
"        %1$s* a,\n"\
"        uint32_t l,\n"\
"        char* buf\n"\
"        )";

char* format_decap_type_array_body = 
"\n"\
"{\n"\
"    uint32_t i;\n"\
"    for(i=0;i<l;i++,a++)\n"\
"    {\n"\
"        buf = decapsulate_%1$s(a,buf);\n"\
"    }\n";

char* format_decap_type_footer =
"    return buf;\n"\
"}\n\n";

/* types */
char* format_decap_type_user =
"    buf = decapsulate_%s(a,buf);\n";

char* format_decap_type_pointer =
"    buf = decapsulate_uint32_t(a,buf);\n"\
"    if (*a)\n"\
"    {\n"\
"        *a = malloc(sizeof(%s));\n"\
"        buf = decapsulate_%1$s(*a,buf);\n"\
"    }\n";

char* format_decap_type_array =
"    buf = decapsulate_array_%s(a,%s,buf);\n";

/* fields of a type */
char* format_decap_field=
"    buf = decapsulate_%s(&(a->%s),buf);\n";

char* format_decap_field_void =
"    /* ignoring void field %s */\n";

char* format_decap_field_array =
"    buf = decapsulate_array_%s(a->%s,%s,buf);\n";

char* format_decap_field_pointer =
"    buf = decapsulate_uint32_t(&(a->%1$s),buf);\n"\
"    if (a->%1$2s)\n"\
"    {\n"\
"        a->%s = (%s*)malloc(sizeof(%2$s));\n"\
"        buf = decapsulate_%2$s(a->%1$s,buf);\n"\
"    }\n";

char* format_decap_field_pointer_array =
"    if (a->%3$s)\n"\
"    {\n"\
"        a->%s = (%s*)malloc(sizeof(%2$s)*a->%s);\n"\
"        buf = decapsulate_array_%2$s(a->%1$s,a->%3$s,buf);\n"\
"    }\n"\
"    else\n"\
"    {\n"\
"        a->%1$s = NULL;\n"\
"    }\n";


/*** union stuff ***/
char* format_decap_field_union_header = 
    "    switch(a->%s)\n"\
    "    {\n";

char* format_decap_field_union_footer = 
    "        default:\n"\
    "            /* unknown type */\n"\
    "            break;\n"\
    "    }\n";

char* format_decap_field_union_case_basic =
    "        case %s:\n"\
    "            buf = decapsulate_%s(&(a->%s%s),buf);\n"\
    "            break;\n";

char* format_decap_field_union_case_pointer =
    "        case %s:\n"\
    "            buf = decapsulate_uint32_t(&(a->%3$s%4$s),buf);\n"\
    "            if (a->%3$s%4$s)\n"\
    "            {\n"\
    "                a->%3$s%4$s = (%2$s*)malloc(sizeof(%2$s));\n"\
    "                buf = decapsulate_%s(a->%3$s%4$s,buf);\n"\
    "            }\n"\
    "            break;\n";

int writedecap_union(FILE* f, itype t)
{
    char unionname[32];
    itype field;
    listitr itr;

    fprintf(f,format_decap_field_union_header,t->size);

    strcpy(unionname,t->name);
    if (t->name[0])
    {
        strcat(unionname,".");
    }
    itr = newlistitr(t->fields);
    while (listitrnext(itr))
    {
        field = (itype)listitrdata(itr);
        switch (field->kindoftype)
        {
            case TYPE_BASIC:
                fprintf(f,format_decap_field_union_case_basic,
                                field->kase,
                                field->basetype->name,
                                unionname,
                                field->name);
                break;
            case TYPE_POINTER:
                fprintf(f,format_decap_field_union_case_pointer,
                                field->kase,
                                field->basetype->name,
                                unionname,
                                field->name);
                break;
        }
    }
    deletelistitr(itr);

    fprintf(f,format_decap_field_union_footer);
    return 1;
}

int writedecap_struct_field(FILE* f, char* parent, itype field)
{
    char fieldname[128];

    if (parent)
    {
        strcpy(fieldname,parent);
        strcat(fieldname,".");
    }
    else
    {
        fieldname[0] = 0;
    }
    strcat(fieldname,field->name);

    switch (field->kindoftype)
    {
        case TYPE_BASIC:
            fprintf(f,format_decap_field,
                field->basetype->name,
                fieldname);
            break;
        case TYPE_POINTER:
            if (field->size) /* an array */
                fprintf(f,format_decap_field_pointer_array,
                    fieldname,
                    field->basetype->name,
                    field->size);
            else /* already a pointer */
                fprintf(f,format_decap_field_pointer,
                    fieldname,
                    field->basetype->name);
            break;
        case TYPE_ARRAY:
            fprintf(f,format_decap_field_array,
                    field->basetype->name,
                    fieldname,
                    field->size);
            break;
/*
        case TYPE_UNION:
            writedecap_union(f,field);
            break;
        case TYPE_STRUCT:
            writedecap_sub_struct(f,field);
            break;
 */
    }
}


char* format_decap_field_struct_array_header =
"    for (i=0;i<%s;i++)\n"\
"    {\n";

char* format_decap_field_struct_array_footer =
"    }\n";

int writedecap_sub_struct(FILE* f, itype t)
{
    char structname[128];

    itype field;
    listitr itr;

    strcpy(structname,t->name);
    if (t->size) /* struct array :( */
    {
        fprintf(f,format_decap_field_struct_array_header,t->size);
        strcat(structname,"[i]");
    }

    itr = newlistitr(t->fields);
    while (listitrnext(itr))
    {
        field = (itype)listitrdata(itr);
        if (!(strcmp(field->basetype->name,"void")))
        {
            /* no idea how to decap a void */
            fprintf(f,format_decap_field_void,field->name);
            continue;
        }
        writedecap_struct_field(f,structname,field);
    }
    deletelistitr(itr);

    if (t->size) /* struct array :( */
    {
        fprintf(f,format_decap_field_struct_array_footer);
    }

    return 1;
}


int writedecap_struct(FILE* f, itype t)
{
    itype field;
    listitr itr;

    printIndexIfNeeded(f,t);

    itr = newlistitr(t->fields);
    while (listitrnext(itr))
    {
        field = (itype)listitrdata(itr);
        if (!(strcmp(field->basetype->name,"void")))
        {
            /* no idea how to decap a void */
            fprintf(f,format_decap_field_void,field->name); 
            continue;
        }
        switch (field->kindoftype)
        {
            case TYPE_BASIC:
                fprintf(f,format_decap_field,
                                field->basetype->name,
                                field->name);
                break;
            case TYPE_POINTER:
                if (field->size) /* an array */
                    fprintf(f,format_decap_field_pointer_array,
                                    field->name,
                                    field->basetype->name,
                                    field->size);
                else /* already a pointer */
                    fprintf(f,format_decap_field_pointer,
                                    field->name,
                                    field->basetype->name);
                break;
            case TYPE_ARRAY:
                fprintf(f,format_decap_field_array,
                                field->basetype->name,
                                field->name,
                                field->size);
                break;
            case TYPE_UNION:
                writedecap_union(f,field);
                break;
            case TYPE_STRUCT:
                writedecap_sub_struct(f,field);
                break;
        }
    }
    deletelistitr(itr);

    return 1;
}

int writedecap(FILE* f, itype t, int defnonly)
{
    if (!(strcmp(t->basetype->name,"void")))
    {
        /* no idea how to decap a void! */
        return 1;
    }
    fprintf(f,format_decap_type_header,t->name);

    if (defnonly)
    {
        fprintf(f,";\n");
    }
    else
    {
        fprintf(f,"\n{\n");
        switch (t->kindoftype)
        {
            case TYPE_BASIC:
                fprintf(f,format_decap_type_user,t->basetype->name);
                break;
            case TYPE_POINTER:
                fprintf(f,format_decap_type_pointer,t->basetype->name);
                break;
            case TYPE_STRUCT:
                writedecap_struct(f,t);
                break;
            case TYPE_ARRAY:
                fprintf(f,format_decap_type_array,t->basetype->name,t->size);
                break;
        }
        fprintf(f,format_decap_type_footer);
    }
    
    fprintf(f,format_decap_type_array_header,t->name);
    if (defnonly)
    {
        fprintf(f,";\n");
    }
    else
    {
        fprintf(f,format_decap_type_array_body,t->name);
        fprintf(f,format_decap_type_footer);
    }
    return 1;
}

int  writealldecaps(FILE* f, int defnonly)
{
    listitr itr;
    itype t;

    itr = newlistitr(globaltypelist->fields);
    itr = newlistitr(globaltypelist->fields);
    while (listitrnext(itr))
    {
        t = (itype)listitrdata(itr);
        if (t->basetype) /* not built in */
        {
            writedecap(f,t,defnonly);
        }
    }
    deletelistitr(itr);
}
