/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

%{
/*
 *	test program for generating definitions and declarations
 */

#include <string.h>
#include "y.defs.h"

extern g_linenumber;
%}

ws	[ \t]+
id	[a-zA-Z][a-zA-Z0-9_]*
size	[0-9]+
file    [a-zA-Z0-9.\/_]+

command [a-z][a-z-]*


begincommand (@<<)
nl	[\n]

%x COMMENT
%x CPPCOMMENT
%x COMMAND
%x PREPROC

%%	

"//"            { BEGIN CPPCOMMENT; }
<CPPCOMMENT>.   { }
<CPPCOMMENT>\n  { g_linenumber++; BEGIN 0; }

"/*"            { BEGIN COMMENT; }

<COMMENT>"*/"   { BEGIN 0; }
<COMMENT>"@!"   {
                    BEGIN COMMAND;
                    return yytext[0];
                }
<COMMENT>\n     { g_linenumber++; }
<COMMENT>.      { }

<COMMAND>{id} {
        yylval.string = strdup(yytext);
        return ID;
        }
<COMMAND>{command} {
        yylval.string = strdup(yytext);
        return ID;
        }
<COMMAND>{size} {
        yylval.string = strdup(yytext);
        return SIZE;
        }
<COMMAND>"!"    { BEGIN COMMENT; }
<COMMAND>{ws}   ;
<COMMAND>{nl}   {  g_linenumber++; }
<COMMAND>.      { return yytext[0]; }

^"#"            { BEGIN PREPROC; }
<PREPROC>{size}      { g_linenumber = atoi(yytext) - 1; }
<PREPROC>.      { }
<PREPROC>\n     { g_linenumber++; BEGIN 0; }

{ws}	;
{nl}	{ g_linenumber++; }
{id}	{
            if (yylval.keyword = keyword(yytext))
                return yylval.keyword;
            yylval.string = strdup(yytext);
            return ID;
        }
{size}  {
            yylval.size = strdup(yytext);
            return SIZE;
        }
{file}  {
            yylval.string = strdup(yytext);
            return INCFILE;
        }
.       { return yytext[0]; }


%%

static struct keyword {
	char* name; /* text string */
	int value;  /* token */
} keywords[] = 
{
"typedef", TYPEDEF,
"unsigned", UNSIGNED,
"signed", SIGNED,
"short", SHORT,
"long", LONG,
/*
"int8_t", INT8,
"int16_t", INT16,
"int32_t", INT32,
"uint8_t", UINT8,
"uint16_t", UINT16,
"uint32_t", UINT32,
"void", VOID,
"char", CHAR,
*/
"struct", STRUCT,
"union", UNION,
"enum", ENUM,
NULL, 0,
};

int keyword(char* k)
{
  int i = 0;
  while (keywords[i].name)
  {
    if (!strcmp(k,keywords[i].name)) break;
    i++;
  }
  return keywords[i].value;
}
