/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

%{
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "list.h"
#include "stack.h"
#include "types.h"

extern stack globalscopestack;
extern int g_linenumber;
%}

%union {
	char* string;
	char* size;
	int keyword;
    void* type;
}

%token <string> ID INCFILE CMD
%token <size> SIZE
%token <keyword> TYPEDEF STRUCT UNION ENUM
%token <keyword> INT FLOAT DOUBLE
%token <keyword> UINT8 UINT16 UINT32
%token <keyword> INT8 INT16 INT32 CHAR
%token <keyword> VOID
%token <keyword> UNSIGNED SIGNED SHORT LONG 

%type <string> id optid command ;
%type <size> size optarr;
%type <keyword> type struct modifier;
%type <type> declaration;

%start statements

%%

statements:
            statements statement
        |   statement
        |
        ;

statement:
            definition
        |   functiondef
        ;

definition:	  typedef modifieddeclaration
		;

functiondef:	  type id {
				newfunction($1,$2);
			}
			'(' arglist ')' ';'
		;

typedef:	  TYPEDEF { }
		;

declaration:
            type id ';' 
            { 
				type t;

				t = newdeclaration($1,$2,TYPE_BASIC,0,0); 

				stackpush(globalscopestack,(void*)t);
                $$ = t;
            }
		|   type '*' id ';' 
            {
				type  t;

				t = newdeclaration($1,$3,TYPE_POINTER,0,0); 

				stackpush(globalscopestack,(void*)t);
                $$ = t;
			}
        /* hard core array.  already allocated within
           the struct. size is a value or define or something */
		|   type id '[' size ']' ';' 
            {
				type t;

				t = newdeclaration($1,$2,TYPE_ARRAY,$4,0); 

				stackpush(globalscopestack,(void*)t);
                $$ = t;
			}
		|   struct optid
            { 
				type t;
				t = (type)newdeclaration(STRUCT,$2,TYPE_STRUCT,0,0);
				stackpush(globalscopestack,(void*)t);
			}

			decllist id optarr ';'
            { 
				type t;
				t = (type)stackpop(globalscopestack);
                renametype(t,$5);

                if ($6)
                {
                    changeToArray(t,$6);
                }

				stackpush(globalscopestack,(void*)t);
                $$ = t;
			}
        |   union optid '@' id ':' size
            {
				type t;
				t = (type)newdeclaration(UNION,$2,TYPE_UNION,$6,0);
				stackpush(globalscopestack,(void*)t);
            }
            decllist optid ';'
            /*uniondeclist optid ';'*/
            {
				type t;
				t = (type)stackpop(globalscopestack);
                renametype(t,$9);

				stackpush(globalscopestack,(void*)t);

                $$ = t;
            }
        |   ENUM optid
            {
            }
            '{' enumlist '}' id ';'
            {
				type t;

				t = newdeclaration(lookupTypeId("uint32_t"),$7,TYPE_BASIC,0,0); 

				stackpush(globalscopestack,(void*)t);
                $$ = t;
            }
        |   type '(' '*' id ')' '(' arglist ')' ';'
            {
                printf("Function pointer type decl:%d: %s\n",g_linenumber,$4);
                $$ = 0;
            }
        |   type '*' '(' id ')' ';'
            {
                printf("Function pointer ??? type decl:%d: %s\n",g_linenumber,$4);
                $$ = 0;
            }
		;

optarr:
        '[' size ']'    { $$ = strdup($2); }
    |   { $$ = 0; }
    ;

type:
        modifier id { $$ = lookupTypeIdMod($2,$1); }
    |   modifier modifier { $$ = lookupTypeIdMod("int",$2); }
    |   id { $$ = lookupTypeId($1); }
	;

modifier:
        UNSIGNED { $$ = $1; }
    |   SIGNED { $$ = $1; }
    |   SHORT { $$ = $1; }
    |   LONG { $$ = $1; }
    ;

struct:	  STRUCT { $$ = $1; }
	;

command: '@' id ':' size 
            { 
                type t;
				t = (type)stackpop(globalscopestack);

                modifyType(t,$2,$4);

				stackpush(globalscopestack,(void*)t);
                $$ = t;
            }
    ;

decllist:	  '{' declarationlist '}'
		;

commandchain:
          commandchain command
        | command
        |
        ;

modifieddeclaration:
            declaration commandchain
            {
				type enc, t;
				t = (type)stackpop(globalscopestack);
				enc = (type)stackpop(globalscopestack);

				addtotype(enc,(void*)t);
				stackpush(globalscopestack,(void*)enc);
            }
            ;

declarationlist:
			| declarationlist modifieddeclaration
			| modifieddeclaration
			|
			;

union:      UNION
        ;

optval: '=' SIZE
        |
        ;

enum:   id optval
        ;

enumlist:   enum ',' enumlist
        |   enum
        |
        ;

arglist:	  arg
		| arglist ',' arg
		|
		;

arg:	  type id { addargtofunction($1,$2,0); }
	| type '*' id { addargtofunction($1,$3,1); }
	;

size:	SIZE { $$ = strdup($1); } /* dont really care what value is
				     and it might be a macro anyway */
    | id { $$ = $1; }
	;

id:	ID { $$ = strdup($1); }
	;

optid: id { $$ = $1; }
    | { $$ = strdup(""); }
    ;

%%

void yyerror(char *s)
{
  printf("line %d: parse error : %s\n",g_linenumber,s);
}
