/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/*
 * This file contains the functions and stuff that will generate the
 * encapsulation routines for a given list of "types"
 */

#include <stdio.h>
#include <stdlib.h>
#include <list.h>
#include "types.h"
#include "typesinternal.h"

extern itype globaltypelist;

extern void writeheader(FILE*);

char* format_encap_file_header = 
"#include \"%s\"\n\n";

/****
 * FORMATS
 ****/

/*** root types ***/
char* format_encap_type_header = 
    "char* encapsulate_%s (\n"\
    "        %1$s* a,\n"\
    "        char* buf\n"\
    "        )";

char* format_encap_type_array_header = 
    "char* encapsulate_array_%s (\n"\
    "        %1$s* a,\n"\
    "        uint32_t l,\n"\
    "        char* buf\n"\
    "        )";

char* format_encap_type_array_body = 
    "\n"\
    "{\n"\
    "    uint32_t i;\n"\
    "    for(i=0;i<l;i++,a++)\n"\
    "    {\n"\
    "        buf = encapsulate_%s(a,buf);\n"\
    "    }\n";

char* format_encap_type_footer =
    "    return buf;\n"\
    "}\n\n";

char* format_encap_type_user =
    "    buf = encapsulate_%s(a,buf);\n";

char* format_encap_type_user_pointer =
    "    buf = encapsulate_uint32_t(a,buf);\n"\
    "    if (*a)\n"\
    "         buf = encapsulate_%s(*a,buf);\n";

char* format_encap_type_array =
    "    buf = encapsulate_array_%s(a,%s,buf);\n";

/*** fields of a struct ***/
char* format_encap_field_void =
    "        /* ignoring void field %s */\n";

char* format_encap_field_user =
    "    buf = encapsulate_%s(&(a->%s),buf);\n";

char* format_encap_field_pointer =
    "    buf = encapsulate_uint32_t(&(a->%2$s),buf);\n"\
    "    if (a->%2$s)\n"\
    "        buf = encapsulate_%s(a->%s,buf);\n";

char* format_encap_field_array =
    "    buf = encapsulate_array_%s(a->%s,%s,buf);\n";

char* format_encap_field_pointer_array =
    "    buf = encapsulate_array_%s(a->%s,a->%s,buf);\n";

/*** sizeof stuff ***/
char* format_sizeof_type_header = 
    "uint32_t sizeof_%s (\n"\
    "        %1$s* a\n"\
    "        )";

char* format_sizeof_type_array_header = 
    "uint32_t sizeof_array_%s (\n"\
    "        %1$s* a,\n"\
    "        uint32_t l\n"\
    "        )";

char* format_sizeof_type_footer =
    "}\n\n";

/* size of arrays */
char* format_sizeof_type_array_simple =
    "    return sizeof(%s)*l;\n";

char* format_sizeof_type_array_user =
    "    uint32_t i,s=0;\n"\
    "    for(i=0;i<l;i++,a++)\n"\
    "    {\n"\
    "        s += sizeof_%s(a);\n"\
    "    }\n"\
    "    return s;\n";

/*** size of types ***/
char* format_sizeof_type_builtin =
    "    return sizeof(%s);\n";

char* format_sizeof_type_user =
    "    return sizeof_%s(a);\n";

char* format_sizeof_type_user_pointer_user =
    "    uint32_t s=0;\n"\
    "    s += sizeof(uint32_t);\n"\
    "    if (*a)\n"\
    "        s += sizeof_%s(*a);\n";

char* format_sizeof_type_user_pointer_builtin =
    "    uint32_t s=0;\n"\
    "    s += sizeof(uint32_t);\n"\
    "    if (*a)\n"\
    "        s += sizeof(%s);\n";

char* format_sizeof_type_array =
    "    return sizeof(%s)*%s;\n";

/* size of fields */
char* format_sizeof_field_header =
    "    uint32_t s=0;\n";

char* format_sizeof_field_footer =
    "    return s;\n";

char* format_sizeof_field_void =
    "        /* ignoring void field %s */\n";

char* format_sizeof_field_builtin =
    "        s += sizeof(%s);\n";

char* format_sizeof_field_user =
    "        s += sizeof_%s(&(a->%s));\n";

char* format_sizeof_field_pointer_user =
    "        s += sizeof(uint32_t);\n"\
    "        if (a->%2$s)\n"\
    "            s += sizeof_%s(a->%s);\n";

char* format_sizeof_field_pointer_builtin =
    "        s += sizeof(uint32_t);\n"\
    "        if (a->%2$s)\n"\
    "            s += sizeof(%s);\n";

char* format_sizeof_field_array_builtin =
    "        s += sizeof(%s)*%s;\n";

char* format_sizeof_field_array_user =
    "        s += sizeof_array_%s(a->%s,%s);\n";

char* format_sizeof_field_pointer_array_user =
    "        s += sizeof_array_%s(a->%s,a->%s);\n";

char* format_sizeof_field_pointer_array_builtin =
    "        s += sizeof(%s)*(a->%s);\n";

/*** encap union related stuff ***/
char* format_encap_field_union_header =
    "    switch(a->%s)\n"\
    "    {\n";

char* format_encap_field_union_footer =
    "        default:\n"\
    "            /* unknown type */\n"\
    "            break;\n"\
    "    }\n";

char* format_encap_field_union_case_void =
    "        case %s:\n"\
    "            /* ignoring void field %s */\n"\
    "            break;\n";

char* format_encap_field_union_case_user =
    "        case %s:\n"\
    "            buf = encapsulate_%s(&(a->%s%s),buf);\n"\
    "            break;\n";

char* format_encap_field_union_case_pointer =
    "        case %s:\n"\
    "            buf = encapsulate_uint32_t(&(a->%3$s%4$s),buf);\n"\
    "            if (a->%3$s%4$s)\n"\
    "                buf = encapsulate_%s(a->%3$s%4$s,buf);\n"\
    "            break;\n";

/*** sizeof union related stuff ***/
char* format_sizeof_field_union_header =
    "    switch(a->%s)\n"\
    "    {\n";

char* format_sizeof_field_union_footer =
    "        default:\n"\
    "            /* unknown type */\n"\
    "            break;\n"\
    "    }\n";

char* format_sizeof_field_union_case_void =
    "        case %s:\n"\
    "            /* ignoring void field %s */\n"\
    "            break;\n";

char* format_sizeof_field_union_case_user =
    "        case %s:\n"\
    "            s += sizeof_%s(&(a->%s.%s));\n"\
    "            break;\n";

char* format_sizeof_field_union_case_pointer =
    "        case %s:\n"\
    "            s += sizeof(uint32_t);\n"\
    "            if (a->%3$s.%4$s)\n"\
    "                s += sizeof_%s(a->%3$s.%4$s);\n"\
    "            break;\n";

char* format_sizeof_field_union_case_builtin =
    "        case %s:\n"\
    "            s += sizeof(%s);\n"\
    "            break;\n";
    
/*****
 * FUNCTIONS
 */
int writeencap_union(FILE* f, itype t)
{
    char unionname[32];
    itype field;
    listitr itr;

    fprintf(f,format_encap_field_union_header,t->size);

    strcpy(unionname,t->name);
    if (t->name[0])
    {
        strcat(unionname,".");
    }
    itr = newlistitr(t->fields);
    while (listitrnext(itr))
    {
        field = (itype)listitrdata(itr);
        if (!(strcmp(field->basetype->name,"void")))
        {
            /* no idea how to encap a void */
            fprintf(f,format_encap_field_union_case_void,field->name); 
            continue;
        }
        switch (field->kindoftype)
        {
            case TYPE_BASIC:
                fprintf(f,format_encap_field_union_case_user,
                                field->kase,
                                field->basetype->name,
                                unionname,
                                field->name);
                break;
            case TYPE_POINTER:
                fprintf(f,format_encap_field_union_case_pointer,
                                field->kase,
                                field->basetype->name,
                                unionname,
                                field->name);
                break;
        }
    }
    deletelistitr(itr);

    fprintf(f,format_encap_field_union_footer);
    return 1;
}

int writeencap_struct_field(FILE* f, char* parent, itype field)
{
    char fieldname[128];

    if (parent)
    {
        strcpy(fieldname,parent);
        strcat(fieldname,".");
    }
    else
    {
        fieldname[0] = 0;
    }
    strcat(fieldname,field->name);

    switch (field->kindoftype)
    {
        case TYPE_BASIC:
            fprintf(f,format_encap_field_user,
                            field->basetype->name,
                            fieldname);
            break;
        case TYPE_POINTER:
            if (field->size) /* an array */
                fprintf(f,format_encap_field_pointer_array,
                                field->basetype->name,
                                fieldname,
                                field->size);
            else /* already a pointer */
                fprintf(f,format_encap_field_pointer,
                                field->basetype->name,
                                fieldname);
            break;
        case TYPE_ARRAY:
            fprintf(f,format_encap_field_array,
                            field->basetype->name,
                            fieldname,
                            field->size);
            break;
/* brain is imploding
        case TYPE_UNION:
            writeencap_union(f,field);
            break;
        case TYPE_STRUCT:
            writeencap_sub_struct(f,field);
            break;
 */
    }
}

int printIndexIfNeeded(FILE* f, itype t)
{
    itype field;
    listitr itr;

    itr = newlistitr(t->fields);
    while (listitrnext(itr))
    {
        field = (itype)listitrdata(itr);
        if (field->kindoftype == TYPE_STRUCT)
        {
            if (field->size)
            {
                fprintf(f,"    uint32_t i;\n");
                deletelistitr(itr);
                return 1;
            }
        }
    }
    deletelistitr(itr);

    return 0;
}

char* format_encap_field_struct_array_header = 
"    for (i=0;i<%s;i++)\n"\
"    {\n";

char* format_encap_field_struct_array_footer = 
"    }\n";

int writeencap_sub_struct(FILE* f, itype t)
{
    char structname[128];

    itype field;
    listitr itr;

    strcpy(structname,t->name);
    if (t->size) /* struct array :( */
    {
        fprintf(f,format_encap_field_struct_array_header,t->size);
        strcat(structname,"[i]");
    }

    itr = newlistitr(t->fields);
    while (listitrnext(itr))
    {
        field = (itype)listitrdata(itr);
        if (!(strcmp(field->basetype->name,"void")))
        {
            /* no idea how to encap a void */
            fprintf(f,format_encap_field_void,field->name); 
            continue;
        }
        writeencap_struct_field(f,structname,field);
    }
    deletelistitr(itr);

    if (t->size) /* struct array :( */
    {
        fprintf(f,format_encap_field_struct_array_footer);
    }

    return 1;
}

int writeencap_struct(FILE* f, itype t)
{
    itype field;
    listitr itr;

    printIndexIfNeeded(f,t);

    itr = newlistitr(t->fields);
    while (listitrnext(itr))
    {
        field = (itype)listitrdata(itr);
        if (!(strcmp(field->basetype->name,"void")))
        {
            /* no idea how to encap a void */
            fprintf(f,format_encap_field_void,field->name); 
            continue;
        }
        switch (field->kindoftype)
        {
            case TYPE_BASIC:
                fprintf(f,format_encap_field_user,
                                field->basetype->name,
                                field->name);
                break;
            case TYPE_POINTER:
                if (field->size) /* an array */
                    fprintf(f,format_encap_field_pointer_array,
                                    field->basetype->name,
                                    field->name,
                                    field->size);
                else /* already a pointer */
                    fprintf(f,format_encap_field_pointer,
                                    field->basetype->name,
                                    field->name);
                break;
            case TYPE_ARRAY:
                fprintf(f,format_encap_field_array,
                                field->basetype->name,
                                field->name,
                                field->size);
                break;
            case TYPE_UNION:
                writeencap_union(f,field);
                break;
            case TYPE_STRUCT:
                writeencap_sub_struct(f,field);
                break;
        }
    }
    deletelistitr(itr);

    return 1;
}

char* format_sizeof_field_union_builtin = 
"        case %s:\n"\
"            s += sizeof(%s);\n"\
"            break;\n";

char* format_sizeof_field_union_user = 
"        case %s:\n"\
"            s += sizeof_%s(&(a->%s%s));\n"\
"            break;\n";

char* format_sizeof_field_union_pointer_builtin = 
"        case %s:\n"\
"            s += sizeof(uint32_t);\n"\
"            if (a->%3$s%4$s)\n"\
"               s += sizeof(%2$s);\n"\
"            break;\n";

char* format_sizeof_field_union_pointer_user = 
"        case %s:\n"\
"            s += sizeof(uint32_t);\n"\
"            if (a->%3$s%4$s)\n"\
"               s += sizeof_%2$s(a->%3$s%4$s);\n"\
"            break;\n";

/* assuming unions are always inside structs */
int writesizeof_union(FILE* f, itype t)
{
    char unionname[32];
    itype field;
    listitr itr;

    fprintf(f,format_sizeof_field_union_header,t->size);

    strcpy(unionname,t->name);
    if (t->name[0])
    {
        strcat(unionname,".");
    }
    itr = newlistitr(t->fields);
    while (listitrnext(itr))
    {
        field = (itype)listitrdata(itr);
        if (!(strcmp(field->basetype->name,"void")))
        {
            /* no idea how to encap a void */
            fprintf(f,format_sizeof_field_void,field->name); 
            continue;
        }

        switch (field->kindoftype)
        {
        case TYPE_POINTER:
            if (field->basetype->kindoftype == TYPE_BUILTIN)
                fprintf(f,format_sizeof_field_union_pointer_builtin,
                                field->kase,
                                field->basetype->name,
                                unionname,
                                field->name);
            else
                fprintf(f,format_sizeof_field_union_pointer_user,
                                field->kase,
                                field->basetype->name,
                                unionname,
                                field->name);
            break;
        case TYPE_BASIC:
            if (field->basetype->kindoftype == TYPE_BUILTIN)
                fprintf(f,format_sizeof_field_union_builtin,
                                field->kase,
                                field->basetype->name);
            else
                fprintf(f,format_sizeof_field_union_user,
                                field->kase,
                                field->basetype->name,
                                unionname,
                                field->name);
            break;
        }
    }
    deletelistitr(itr);
    fprintf(f,format_sizeof_field_union_footer);

    return 1;
}

int writesizeof_struct(FILE* f, itype t)
{
    itype field;
    listitr itr;

    fprintf(f,format_sizeof_field_header);
    itr = newlistitr(t->fields);
    while (listitrnext(itr))
    {
        field = (itype)listitrdata(itr);
        if (!(strcmp(field->basetype->name,"void")))
        {
            /* no idea how to encap a void */
            fprintf(f,format_sizeof_field_void,field->name); 
            continue;
        }
        switch (field->kindoftype)
        {
            case TYPE_BASIC:
                if (field->basetype->kindoftype == TYPE_BUILTIN)
                    fprintf(f,format_sizeof_field_builtin,
                                    field->basetype->name,
                                    field->name);
                else
                    fprintf(f,format_sizeof_field_user,
                                    field->basetype->name,
                                    field->name);
                break;
            case TYPE_POINTER:
                if (field->size) /* an array */
                    if (field->basetype->kindoftype == TYPE_BUILTIN)
                    {
                        fprintf(f,format_sizeof_field_pointer_array_builtin,
                                        field->basetype->name,
                                        field->size);
                    }
                        else
                    {
                        fprintf(f,format_sizeof_field_pointer_array_user,
                                        field->basetype->name,
                                        field->name,
                                        field->size);
                    }
                else /* already a pointer */
                {
                    if (field->basetype->kindoftype == TYPE_BUILTIN)
                        fprintf(f,format_sizeof_field_pointer_builtin,
                                        field->basetype->name,
                                        field->name);
                    else
                        fprintf(f,format_sizeof_field_pointer_user,
                                        field->basetype->name,
                                        field->name);
                }
                break;
            case TYPE_ARRAY:
                if (field->basetype->kindoftype == TYPE_BUILTIN)
                    fprintf(f,format_sizeof_field_array_builtin,
                                    field->basetype->name,
                                    field->size);
                else
                    fprintf(f,format_sizeof_field_array_user,
                                    field->basetype->name,
                                    field->name,
                                    field->size);
                break;
            case TYPE_UNION:
                writesizeof_union(f,field);
                break;
        }
    }
    deletelistitr(itr);
    fprintf(f,format_sizeof_field_footer);

    return 1;
}

int writeencap(FILE* f, itype t, int defnonly)
{
    if (!(strcmp(t->basetype->name,"void")))
    {
        /* no idea how to decap a void! */
        return 1;
    }
    fprintf(f,format_encap_type_header,t->name);

    if (defnonly)
    {
        fprintf(f,";\n");
    }
    else
    {
        fprintf(f,"\n{\n");
        switch (t->kindoftype)
        {
            case TYPE_BASIC:
                fprintf(f,format_encap_type_user,t->basetype->name);
                break;
            case TYPE_STRUCT:
                writeencap_struct(f,t);
                break;
            case TYPE_POINTER:
                fprintf(f,format_encap_type_user_pointer,t->basetype->name);
                break;
            case TYPE_ARRAY:
                fprintf(f,format_encap_type_array,t->basetype->name,t->size);
                break;
        }
        fprintf(f,format_encap_type_footer);
    }

    fprintf(f,format_encap_type_array_header,t->name);
    if (defnonly)
    {
        fprintf(f,";\n");
    }
    else
    {
        fprintf(f,format_encap_type_array_body,t->name);
        fprintf(f,format_encap_type_footer);
    }
    return 1;
}

int issimpletype(itype t)
{
    if (t->kindoftype == TYPE_POINTER) return 0;

    if (!t->basetype)
    {
        if (t->kindoftype == TYPE_BUILTIN) return 1;
        return 0;
    }

    switch (t->kindoftype)
    {
        case TYPE_BASIC:
        case TYPE_ARRAY:
            return issimpletype(t->basetype);
            break;
        case TYPE_STRUCT:
        case TYPE_UNION:
            {
                int simple = 1;
                listitr itr;
                itype field;

                itr = newlistitr(t->fields);
                while (simple && listitrnext(itr))
                {
                    field = (itype)listitrdata(itr);
                    simple = issimpletype(field);
                }
                deletelistitr(itr);
                return simple;
            }
            break;
    }
    return 0; /* dont know */
}

int writesizeof(FILE* f, itype t, int defnonly)
{
    if (!(strcmp(t->basetype->name,"void")))
    {
        /* no idea how to encap a void! */
        return 1;
    }
    fprintf(f,format_sizeof_type_header,t->name);

    if (defnonly)
    {
        fprintf(f,";\n");
    }
    else
    {
        fprintf(f,"\n{\n");
        if (issimpletype(t))
            fprintf(f,format_sizeof_type_builtin,t->name);
        else
        {
            switch (t->kindoftype)
            {
                case TYPE_BASIC:
                    if (t->basetype->kindoftype == TYPE_BUILTIN)
                        fprintf(f,format_sizeof_type_builtin,t->basetype->name);
                    else
                        fprintf(f,format_sizeof_type_user,t->basetype->name);
                    break;
                case TYPE_STRUCT:
                    writesizeof_struct(f,t);
                    break;
                case TYPE_POINTER:
                    if (t->basetype->kindoftype == TYPE_BUILTIN)
                        fprintf(f,format_sizeof_type_user_pointer_builtin,t->basetype->name);
                    else
                        fprintf(f,format_sizeof_type_user_pointer_user,t->basetype->name);
                    break;
                case TYPE_ARRAY:
                    fprintf(f,format_sizeof_type_array,t->basetype->name,t->size);
                    break;
            }
        }
        fprintf(f,format_sizeof_type_footer);
    }

    fprintf(f,format_sizeof_type_array_header,t->name);
    if (defnonly)
    {
        fprintf(f,";\n");
    }
    else
    {
        fprintf(f,"\n{\n");
        if (issimpletype(t))
            fprintf(f,format_sizeof_type_array_simple,t->name);
        else
            fprintf(f,format_sizeof_type_array_user,t->name);
        fprintf(f,format_sizeof_type_footer);
    }
}

int  writeallencaps(FILE* f, int defnonly)
{
    listitr itr;
    itype t;

    itr = newlistitr(globaltypelist->fields);
    while (listitrnext(itr))
    {
        t = (itype)listitrdata(itr);
        if (t->basetype) /* not built in && not unknown */
        {
            writeencap(f,t,defnonly);
            writesizeof(f,t,defnonly);
        }
    }
    deletelistitr(itr);
}
