/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/*
 * creates and maintains a list of function prototypes
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "y.defs.h"

#include "list.h"
#include "stack.h"
#include "types.h"
#include "typesinternal.h"
#include "functions.h"

list globalfunctionlist = NULL;
int globalfunctioncount = 1024;

typedef struct function_t {
	char* name;
	int functionid;
	itype returntype;
	list arglist;
} * function;

typedef struct arg_t {
	char* name;
	int kindofarg; /* 0: regular, 1: pointer, 2: array ... */
	itype argtype;
} * arg;

void* newfunction(int returntype, char* name)
{
	function f;
	f = (function)malloc(sizeof(struct function_t));

	f->name = strdup(name);
	f->functionid = ++globalfunctioncount;
	f->returntype = (itype)findTypeByTypeId(returntype);
	f->arglist = newlist();

	listappend(globalfunctionlist,(void*)f);
}

void addargtofunction(int argtype, char* name, int kindofarg)
{
	function f;
	arg a;
	f = (function)listremoveback(globalfunctionlist);
	if (!f) return; /* serious parse error */

	a = (arg)malloc(sizeof(struct arg_t));
	a->name = strdup(name);
	a->argtype = (itype)findTypeByTypeId(argtype);
	a->kindofarg = kindofarg;

	listappend(f->arglist,(void*)a);

	listappend(globalfunctionlist,(void*)f);
}

void initfunctions()
{
	globalfunctionlist = newlist();
}

void printarg(arg a)
{
	if (!a) return;
	printf(" %s%s %s",a->argtype->name, a->kindofarg?"*":"",a->name);
}

void printfunction(function f)
{
	int first = 1;
	arg a;
	listitr itr;

	if (!f) return;

	printf("%s %s ( ",f->returntype->name, f->name);
	itr = newlistitr(f->arglist);
	while (listitrnext(itr))
	{
		if (!first)
			printf(",");
		else
			first = 0;
		a = (arg)listitrdata(itr);
		printarg(a);
	}
	deletelistitr(itr);
	printf(" )\n");
}

void printallfunctions()
{
	function f;
	listitr itr;

	printf("All Functions:\n");
	itr = newlistitr(globalfunctionlist);
	while (listitrnext(itr))
	{
		printf("     ");
		f = (function)listitrdata(itr);
		printfunction(f);
	}
	deletelistitr(itr);
}
