/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/*
 * generic list code
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "list.h"

typedef struct listnode_t
{
	void *data;
	struct listnode_t *prev, *next;
} *listnode;

typedef struct list_t
{
	listnode head;
	listnode tail;
} *internal_list;

list newlist()
{
	internal_list l;
	l = (internal_list)malloc(sizeof(struct list_t));
	l->head = l->tail = NULL;
	return (list)l;
}

void deletelist(list ll)
{
	internal_list l;
	listnode n;
	l = (internal_list)ll;
	
	n = l->head;
	while (n)
	{
		 l->head = n->next;
		 free(n);
		 n = l->head;
	}
	free(l);
}

int listisempty(list ll)
{
	internal_list l;
	l = (internal_list)ll;
	if (!l) return 1;

	return l->head?0:1;
}

listnode newnode(void *d)
{
	listnode n;
	n = (listnode)malloc(sizeof(struct listnode_t));
	n->data = d;
	n->prev = n->next = NULL;
}

void listappend(list ll, void *d)
{
	internal_list l;
	listnode n;
	l = (internal_list)ll;
	if (!l) return;

	n = newnode(d);
	n->prev = l->tail;
	if (l->tail)
		l->tail->next = n;
	if (!l->head)
		l->head = n;
	l->tail = n;
}

void listprepend(list ll, void *d)
{
	internal_list l;
	listnode n;
	l = (internal_list)ll;

	if (!l) return;

	n = newnode(d);
	n->next = l->head;
	if (l->head)
		l->head->prev = n;
	if (!l->tail)
		l->tail = n;
	l->head = n;
}

void* listremovefront(list ll)
{
	internal_list l;
	listnode n;
	void* d;
	l = (internal_list)ll;

	if (!l) return NULL; /* not a very good error code since anyone
			     could have added NULL to the list */
	if (!l->head) return NULL;

	n = l->head;
	l->head = n->next;
	if (l->head)
		l->head->prev = NULL;
	else
		l->tail = NULL;
	n->next = NULL;

	d = n->data;
	free(n);
	return d;
}

void* listremoveback(list ll)
{
	internal_list l;
	listnode n;
	void* d;
	l = (internal_list)ll;

	if (!l) return NULL; /* not a very good error code since anyone
			     could have added NULL to the list */
	if (!l->tail) return NULL;

	n = l->tail;
	l->tail = n->prev;
	if (l->tail)
		l->tail->next = NULL;
	else
		l->head = NULL;
	n->prev = NULL;

	d = n->data;
	free(n);
	return d;
}

typedef struct listitr_t
{
	internal_list list;
	listnode current;
}*internal_listitr;

listitr newlistitr(list l)
{
	internal_listitr i;
	i = (internal_listitr)malloc(sizeof(struct listitr_t));
	i->list = l;
	i->current = NULL;
	return (listitr)i;
}

int listitrnext(listitr ii)
{
	internal_listitr i;
	i = (internal_listitr)ii;
	if (!i) return 0;

	if (!i->list) return 0; /* list empty */

	if (!i->list->head) return 0; /* list empty */

	if (i->current == i->list->tail) return 0; /* already at end of list */

	if (!i->current) /* first time called */
		i->current = i->list->head;
	else
		i->current = i->current->next; /* not gonna be null,
						  otherwise tail check
						  would have succeeded */

	return 1;
}

void* listitrdata(listitr ii)
{
	internal_listitr i;
	i = (internal_listitr)ii;
	if (!i) return NULL; /* not much of an error value, since anyone
			        could have added NULL to the list */

	if (!i->current) return NULL;

	return i->current->data;
}

void deletelistitr(listitr ii)
{
	internal_listitr i;
	i = (internal_listitr)ii;

	free(i);
}

