/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/*
 * creates and maintains a list of user defined types
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "y.defs.h"

#include "list.h"
#include "stack.h"

#include "types.h"
#include "typesinternal.h"

int g_linenumber = 1;
stack globalscopestack = NULL;
int globaltypecount = 1024;

itype globaltypelist = NULL;

type findTypeByName(char* name)
{
	listitr itr;
	itype t;
	int found;

	found = 0;
	if (!globaltypelist) return 0;
	itr = newlistitr(globaltypelist->fields);

	while (listitrnext(itr))
	{
		t = (itype)listitrdata(itr);
		if (!strcmp(name,t->name))
		{
			found = 1;
			break;
		}
	}
	deletelistitr(itr);
	if (found)
		return (type)t;
	fprintf(stderr,"line %d: findTypeByName: Unable to find type %s\n",g_linenumber,name);
	return 0;
}

type findTypeByTypeId(int typeid)
{
	listitr itr;
	itype t;
	int found;

	found = 0;
	if (!globaltypelist) return 0;
	itr = newlistitr(globaltypelist->fields);

	while (listitrnext(itr))
	{
		t = (itype)listitrdata(itr);
		if (typeid == t->typeid)
		{
			found = 1;
			break;
		}
	}
	deletelistitr(itr);
	if (found)
		return (type)t;
	fprintf(stderr,"line %d: findTypeByTypeId: Unable to find type %d\n",g_linenumber,typeid);
	return 0;
}

void* newdeclaration(
        int basetype,
        char* name,
        int kindoftype,
        char* size,
        char* k)
{
	itype t;
	t = malloc(sizeof(struct type_t));
	t->typeid = ++globaltypecount;
	t->kindoftype = kindoftype;
	t->name = strdup(name);
    t->size = size?strdup(size):NULL;
    t->kase = k?strdup(k):NULL;
	t->basetype = (itype)findTypeByTypeId(basetype);
	t->fields = NULL;
	return t;
}

void* builtindeclaration(int typeid, char* name)
{
	itype t;
	t = malloc(sizeof(struct type_t));
	t->typeid = typeid;
	t->name = strdup(name);
	t->basetype = NULL;
	t->fields = NULL;
	t->kindoftype = TYPE_BUILTIN;
	t->size = NULL;
	return t;
}

int lookupTypeId(char *name)
{
	itype t;
	t = findTypeByName(name);
	if (t)
		return t->typeid;
	fprintf(stderr,
        "line %d: "
        "lookupTypeId: Unable to find type %s : "
        "assuming defined elsewehere!\n",
        g_linenumber,name);
    t = builtindeclaration(++globaltypecount,name);
    t->kindoftype = TYPE_BASIC;
	addtotype(globaltypelist,t);
	return t->typeid;
}

char* modstrings[] = 
{
    "unsigned",
    "signed",
    "short",
    "long",
    0
};

int lookupTypeIdMod(char* name, int mod)
{
	itype t;
    char moddedname[128];

    sprintf(moddedname,"%s%s",modstrings[mod - UNSIGNED],name);

	t = findTypeByName(moddedname);
	if (t)
		return t->typeid;
	fprintf(stderr,
        "line %d: "
        "lookupTypeId: Unable to find type %s: "
        "assuming defined elsewehere!\n",
        g_linenumber,moddedname);
    t = builtindeclaration(++globaltypecount,moddedname);
    t->kindoftype = TYPE_BASIC;
	addtotype(globaltypelist,t);
	return t->typeid;
}

/* add type tt to the fields list of enc */
void addtotype(type ee, type tt)
{
    itype enc, t;
    enc = (itype)ee;
    t = (itype)tt;
	if (!enc->fields) enc->fields = newlist();
	listappend(enc->fields,t);
}

void renametype(type tt, char* name)
{
    itype t;
    t = (itype)tt;
    if (t->name) free(t->name);
    t->name = strdup(name);
}

void modifyType(type tt, char* cmd, char* val)
{
    itype t;
    t = (itype)tt;

    if (!strcmp(cmd,"array-length"))
    {
        t->size = strdup(val);
    }
    else
    if (!strcmp(cmd,"union-case"))
    {
        t->kase = strdup(val);
    }
}

int changeToArray(type tt, char* size)
{
    itype t;
    t = (itype)tt;

    if (t->kindoftype != TYPE_STRUCT)
    {
        return 0;
    }

    /* maybe do more type changing? */
    t->size = strdup(size);

    return 1;
}

int isintype(type ee, char* name)
{
	listitr itr;
	itype t,enc;
	int found;

    enc = (itype)ee;
	found = 0;
	itr = newlistitr(enc->fields);

	while (listitrnext(itr))
	{
		t = (itype)listitrdata(itr);
		if (!strcmp(name,t->name))
		{
			found = 1;
			break;
		}
	}
	deletelistitr(itr);
	if (found)
		return 1;
	return 0;
}

void inittypes()
{
	globalscopestack = newstack();

	globaltypelist = builtindeclaration(0,"ROOT(FAKETYPE)");
	stackpush(globalscopestack,globaltypelist);

	addtotype(globaltypelist,builtindeclaration(++globaltypecount,"int"));
	addtotype(globaltypelist,builtindeclaration(++globaltypecount,"float"));
	addtotype(globaltypelist,builtindeclaration(++globaltypecount,"double"));
	addtotype(globaltypelist,builtindeclaration(++globaltypecount,"long"));

	addtotype(globaltypelist,builtindeclaration(++globaltypecount,"int8_t"));
	addtotype(globaltypelist,builtindeclaration(++globaltypecount,"int16_t"));
	addtotype(globaltypelist,builtindeclaration(++globaltypecount,"int32_t"));
	addtotype(globaltypelist,builtindeclaration(++globaltypecount,"char"));

	addtotype(globaltypelist,builtindeclaration(++globaltypecount,"uint8_t"));
	addtotype(globaltypelist,builtindeclaration(++globaltypecount,"uint16_t"));
	addtotype(globaltypelist,builtindeclaration(++globaltypecount,"uint32_t"));

	addtotype(globaltypelist,builtindeclaration(++globaltypecount,"void"));

    /* not really types */
	addtotype(globaltypelist,builtindeclaration(STRUCT,"struct"));
	addtotype(globaltypelist,builtindeclaration(UNION,"union"));
}

void printindent(int indent)
{
	int i=indent;
	while (i--) printf(" ");
}
void printtype(itype t, int indent)
{
	itype child;
	listitr itr;

	if (!t) return;

	printindent(indent);
	if (t->kindoftype != TYPE_BUILTIN && t->basetype)
	{
		printf("%s : %s",t->name,t->basetype->name);
		switch (t->kindoftype)
		{
			case TYPE_POINTER:
				printf(" Pointer");
                if (t->size)
                {
                    printf(" array, length determined by %s\n",t->size);
                }
				break;
			case TYPE_UNION:
				printf(" switched off of : %s",t->size);
				break;
			case TYPE_ARRAY:
				printf(" [ %s ]",t->size);
				break;
            case TYPE_STRUCT:
                if (t->size) /* struct array */
                {
				    printf(" [ %s ]",t->size);
                }
		}
		printf("\n");
	}
	else
		printf("%s : %d\tbuiltin\n",t->name,t->typeid);
		

	itr = newlistitr(t->fields);
	while (listitrnext(itr))
	{
		child = listitrdata(itr);
		printtype(child,indent+5);
	}
	deletelistitr(itr);
}

void printalltypes()
{
	itype child;
	listitr itr;

	printf("All Types:\n");
	itr = newlistitr(globaltypelist->fields);
	while (listitrnext(itr))
	{
		child = listitrdata(itr);
		printtype(child,5);
	}
	deletelistitr(itr);
}
