/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2002 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/common/inc/fppapi_atm_types.h,v $
 * $Revision: 1.5 $
 * $Date: 2003/11/17 16:57:56 $
 *
 * Description:
 * This file defines all of the types that are used by the ATM API.
 *
 ****************************************************************************/

#ifndef __FPPAPI_ATM_TYPES_H__
#define __FPPAPI_ATM_TYPES_H__

#ifdef _INCLUDE_ATM

#ifdef __cplusplus
extern "C" {
#endif

typedef NPF_uint32_t NPF_IfATM_ReturnCode_t;
#define MAX_ATM_ENTRIES 10

typedef enum
{
	NPF_IF_ATM_ATTR_SET_CB						= 0,
	NPF_IF_ATM_IF_DELETE_CB						= 1,
	NPF_IF_ATM_IF_ENABLE_CB						= 2,
	NPF_IF_ATM_IF_DISABLE_CB					= 3,
	NPF_IF_ATM_IF_GEN_STATS_GET_CB			= 4,
	NPF_IF_ATM_VCC_SET_CB						= 5, 
	NPF_IF_ATM_VCC_DEL_CB						= 6,
	NPF_IF_ATM_VCC_ENABLE_CB					= 7,
	NPF_IF_ATM_VCC_DISABLE_CB					= 8,
	NPF_IF_ATM_VCC_BIND_CB                 = 9,
	NPF_IF_ATM_VCC_OPER_STATUS_GET_CB		= 10,
	NPF_IF_ATM_VCC_STATS_GET_CB				= 11,
	NPF_IF_ATM_AAL2_CHANNEL_SET_CB			= 12,
	NPF_IF_ATM_AAL2_CHANNEL_DEL_CB			= 13,
	NPF_IF_ATM_CONNECTION_XC_SET_CB			= 14,
	NPF_IF_ATM_CONNECTION_XC_DEL_CB			= 15,
	NPF_IF_ATM_OAM_CP_SET_CB					= 16,
	NPF_IF_ATM_OAM_CC_SET_CB					= 17,	
	NPF_IF_ATM_OAM_LB_SET_CB					= 18,
	NPF_IF_ATM_OAM_MONITOR_CB					= 19,
	NPF_IF_ATM_OAM_ALARM_SET_CB				= 20,
	NPF_IF_ATM_OAM_ALARM_CLEAR_CB				= 21,
	NPF_IF_ATM_OAM_PM_SET_CB					= 22,
	NPF_IF_ATM_EVENT_OAM_AIS_ALARM_CB		= 23,
	NPF_IF_ATM_EVENT_OAM_RDI_ALARM_CB		= 24,
	NPF_IF_ATM_EVENT_OAM_LB_CB					= 25,
	NPF_IF_ATM_EVENT_OAM_FPM_STATS_CB		= 26,
	NPF_IF_ATM_EVENT_OAM_BR_STATS_CB			= 27,
	NPF_IF_ATM_EVENT_ATM_VCC_UP_EVENT_CB	= 28,
	NPF_IF_ATM_EVENT_ATM_VCC_DOWN_EVENT_CB	= 29,
	NPF_IF_ATM_MAX_CB_TYPES					   = 30 
}NPF_IfATM_CallbackType_t;
	


typedef struct
{
	NPF_boolean_t				initFlag;					/* Initialization Flag */
	PilCriticalSection		lock;							/* If ATM Lock */
	DList							apiReqList;					/* List of Transaction Blocks */
}IfATM_ControlBlock_t;


typedef struct
{
	NPF_callbackHandle_t			cbh;				/* User Supplied CallBackHandle */
	NPF_correlator_t				corr;				/* User Supplied Correlator */
	NPF_errorReporting_t			report;			/* User supplied error Reporting */
	
	NPF_uint32_t					numTransWait;	/* Nos. of Pending FP Transaction */
	NPF_uint32_t					numTransFail;	/* Nos. of Failed FP Transaction */

	DList								fpTransList;	/* List of ATM FE Trans Info */
	
	NPF_uint32_t					numWaitFEs;		/* Nos of FEs to be interacted with for this req */

	struct
	{
		FPPI_FEID					feId;				/* FE Id */
		NPF_uint32_t				numWait;			/* */
		NPF_uint32_t				numFail;			/* */
	}feInfo[MAXFENUM];

	NPF_IfCallbackData_t		cbData;			/* CallBack Data Structure */
	
	struct
	{
		NPF_uint32_t				numEntries;		/* Nos. of entries */
		NPF_IfATM_ReturnCode_t	entryRet[MAX_ATM_ENTRIES];
		void						   *entry;			/* Actual entries recvd in called user API */
		void 							*resp;			
	}apiData;
}IfATM_RequestCB_t;

typedef struct
{
	IfATM_RequestCB_t				*reqCbPtr;		/* Request CB Ptr */
	void								*entryPtr;		/* Element Entry Ptr */
	FPPI_FEID						feId;				/* FE ID */
	FPPI_Status						transStatus;	/* Transaction Status */
}IfATM_FeTransInfo_t;


typedef struct
{
	NPF_uint8_t				status;			/* Status of response */
	NPF_uint8_t				reason;			/* Reason for failure */
}IfATM_FpResponse_t;



/*
 *	ATM per Interface statistics
 */
typedef struct
{
		IfATM_FpResponse_t response;  /* Response success/failure */
		NPF_IfStatistics_t stats;  /* Per Vcc Statistics */
}IfATM_IfStatistics_t;


/*
 *	ATM per VCC statistics
 */
typedef struct
{
		IfATM_FpResponse_t response;  /* Response success/failure */
		NPF_ATM_VccStats_t vccStats;  /* Per Vcc Statistics */
}IfATM_VccStats_t;


/*
 * ATM Vcc attributes
 */
typedef struct
{
		NPF_IfVcc_t			vccInfo;		/* VCC attributes */
		NPF_uint32_t		portId;		/* Port ID */
}IfATM_IfVcc_t;

/* 
 * Vcc Bind information
 */
typedef struct
{
		NPF_VccAddr_t		vccInfo;			/* VCC attributes */
		NPF_uint32_t		portId;			/* Port ID */
		NPF_IfHandle_t		parentIfHandle;	/* If Handle of parent interface */
}IfATM_VccBind_t;
	

typedef struct
{
		NPF_VccAddr_t		vccAddr;		/* VPI/VCI value */
		NPF_uint32_t		portId;		/* Port ID */
}IfATM_VccAddr_t;


/*
 *	OAM CP configuration
 */
typedef struct
{
		NPF_ATM_OAM_CP_t		cpInfo;		/* OAM_CP_t Info */
		NPF_uint32_t			portId;		/* Port ID */
}IfATM_OAM_CP_t;

/*
 *	OAM CC Activation/Deactivation
 */
typedef struct{
		NPF_ATM_OAM_CC_t		ccInfo;		/* OAM_CC_t Info */
		NPF_uint32_t			portId;		/* Port ID */
}IfATM_OAM_CC_t;

/*
 *	OAM PM Activation/Deactivation -  FPM-BR/FPM
 */
typedef struct{
		NPF_ATM_OAM_PM_t		pmInfo;		/* OAM_PM_t Info */
		NPF_uint32_t			portId;		/* Port ID */
}IfATM_OAM_PM_t;

/*
 *	OAM Loopback
 */
typedef struct{
		NPF_ATM_OAM_LB_t		lbInfo;		/* OAM_LB_t Info */
		NPF_uint32_t			portId;		/* Port ID */
}IfATM_OAM_LB_t;




/*
* OAM Non-Intrusive Monitoring 
*/
typedef struct {
		NPF_ATM_OAM_Mon_t		monInfo; 	 /* OAM_Mon_t Info */
		NPF_uint32_t			portId;		/* Port ID */
}IfATM_OAM_Mon_t;


/*
 * OAM Alarms - Declare and Release 
 */
typedef struct
{
		NPF_ATM_OAM_Alarm_t	oamAlrmInfo;		/* VPI/VCI */
		NPF_uint32_t			portId;				/* Port ID */
}IfATM_OAM_Alarm_t;


/*
 *   AAL2 Connection ID.
 */
typedef struct
{
		NPF_ATM_AAL2_ConnId_t	aal2ConId;		/* Aal2 Connection Id */
		NPF_uint32_t		portId;			/* Port ID */
}IfATM_AAL2_ConnId_t;

typedef struct
{		NPF_ATM_AAL2_ChannelInfo_t aal2ChanInfo;	/* AAL2 channel Info */
		NPF_uint32_t		portId;			/* Port ID */
}IfATM_AAL2_ChannelInfo_t;


typedef struct
{		NPF_IfATM_t		atmAttr;				/* ATM interface Attributes */
		NPF_uint64_t	portSpeed;			/* Port Speed */
}IfATM_IfAtmAttr_t;



#ifdef __cplusplus
}
#endif

#endif /* _INCLUDE_ATM */

#endif /* __FPPAPI_ATM_TYPES_H__ */
