/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2002 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/common/inc/ipaddr.h,v $
 * $Revision: 1.6 $
 * $Date: 2003/11/17 16:57:56 $
 *
 *
 ****************************************************************************/

#ifndef _IPADDR_H_
#define _IPADDR_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <npf.h>

#define FALSE 0
#define TRUE  1

typedef struct in_addr IPADDR;
typedef struct in_addr SUBMASK;
typedef uint32_t IPv6A[4];

/*****************************************************************************
*                                                                            *
*  --------------------------- Public Interface ---------------------------  *
*                                                                            *
*****************************************************************************/

/* Convert binary in_addr to number-and-dot format */
int npf_ipaddr_addr2char(struct in_addr in, char *buff);

/* Validate the uint32_t ipaddr */
int npf_ipaddr_validate(uint32_t ipaddr, uint32_t submask);
int npf_ipaddr_prefixlen(uint32_t ipaddr);
int npf_ipv6addr_validate(IPv6A addr, IPv6A submask);
int npf_ipv6addr_compare(IPv6A addr1, IPv6A addr2);
int npf_ipv6addr_copy(IPv6A dest, IPv6A src);
int npf_ipv6addr_isZero(IPv6A addr);

/* Byte swap a 16 char array */
void npf_ipv6addr_swapByteOrder(char *addr);

# if __BYTE_ORDER == __BIG_ENDIAN
/* No op - host byte order same as network byte order */
#   define htonlIPv6(x) (x)
#   define ntohlIPv6(x) (x)
# elif __BYTE_ORDER == __LITTLE_ENDIAN
#   define htonlIPv6(x) npf_ipv6addr_swapByteOrder(x)
#   define ntohlIPv6(x) npf_ipv6addr_swapByteOrder(x)
# endif

/* Calculate IPv6 prefix length. Assume the passing address fileds 
 * (unsigned 32-bit int) are in network byte order.
 */
int PrefixLen(IPv6A v6mask);

/* Calculate IPv6 prefix length. */
int Equalv6Addrs(IPv6A addr1, IPv6A addr2);

/*
    Here are standard C libraries:

    dot-and-number ip:  90.2.3.4
    network byte order: 0x0403025a
    host byte order:    0x5a020304

    struct in_addr {unsigned long s_addr;}

    unsigned long int htonl(unsigned long int hostlong);
    unsigned short int htons(unsigned short int hostshort);
    unsigned long int ntohl(unsigned long int netlong);
    unsigned short int ntohs(unsigned short int netshort); 

    "x.x.x.x" -> binary in_addra network byte order
    int inet_aton(const char *cp, struct in_addr *inp);

    "x.x.x.x" -> network byte order
    unsigned long int inet_addr(const char *cp);
    "x" or "x.x" or "x.x.x" or "x.x.x.x" -> network byte order
    unsigned long int inet_network(const char *cp);

    in_addr network byte order -> "x.x.x.x"
    char *inet_ntoa(struct in_addr in);

    struct in_addr inet_makeaddr(int net, int host);
    unsigned long int inet_lnaof(struct in_addr in);
    unsigned long int inet_netof(struct in_addr in);

 */

#ifdef __cplusplus
}
#endif

#endif  /* _IPADDR_H_ */

