/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

#ifndef _LOG_H_
#define _LOG_H_

#ifdef __cplusplus
extern "C" {
#endif

// TODO
// Log to multiple files
// Circular log file
// Truncate long log entries

#include "npf_common.h"

// Global log enable/disable flag. By setting this to 0 all logger write
// commands are removed by the preprocessor. This should be done for release
// builds that need maximum performance. By disabling the logger with this
// switch there is no runtime checking of log levels done, the logger is
// removed at compile time.

#ifndef LOG_ENABLE
#define LOG_ENABLE 1
#endif

// Set the default verbosity level of the logger. This controls how much info
// is provided in the log message prefix. This value should be set to one of
// the available values in npf_logger_verbosity below. 
// Setting this value sets the high water mark for the prefix. For instance if
// 4 is chosen then verbosity levels 3, 2, 1 and 0 are included implicitly.

#ifndef LOG_VERBOSITY
#define LOG_VERBOSITY 6
#endif

// Define the default log levels for each PDK component. These are masks that
// are created by ORing together the possible values in npf_logger_levels.
// These defaults can be overridden by defining the same symbol in a makefile
// and can also be configured at runtime using available logger API functions.
// Note that setting a component to 0 will disable logging for a component
// but this check will be performed at runtime. If highest performance is
// required then use the LOG_ENABLE flag above to disable the entire logger
// at compile time.

#ifndef LOG_APP_LVL
#define LOG_APP_LVL 0
#endif
#ifndef LOG_PDKMGR_LVL
#define LOG_PDKMGR_LVL 0
#endif
#ifndef LOG_NS_LVL
#define LOG_NS_LVL 0
#endif
#ifndef LOG_LBM_LVL
#define LOG_LBM_LVL 0
#endif
#ifndef LOG_TPM_LVL
#define LOG_TPM_LVL 0
#endif
#ifndef LOG_CM_LVL
#define LOG_CM_LVL 0
#endif
#ifndef LOG_IF_LVL
#define LOG_IF_LVL 0
#endif
#ifndef LOG_BD_LVL
#define LOG_BD_LVL 0
#endif
#ifndef LOG_ATM_LVL
#define LOG_ATM_LVL 0
#endif
#ifndef LOG_IPV4_LVL
#define LOG_IPV4_LVL 0
#endif
#ifndef LOG_IPV6_LVL
#define LOG_IPV6_LVL 0
#endif
#ifndef LOG_IPV6TM_LVL
#define LOG_IPV6TM_LVL 0
#endif
#ifndef LOG_DS_LVL
#define LOG_DS_LVL 0
#endif
#ifndef LOG_INTSERV_LVL
#define LOG_INTSERV_LVL 0
#endif
#ifndef LOG_MPLS_LVL
#define LOG_MPLS_LVL 0
#endif
#ifndef LOG_TOPO_LVL
#define LOG_TOPO_LVL 0
#endif
#ifndef LOG_CB_LVL
#define LOG_CB_LVL 0
#endif
#ifndef LOG_VIC_LVL
#define LOG_VIC_LVL 0
#endif
#ifndef LOG_PH_LVL
#define LOG_PH_LVL 0
#endif
#ifndef LOG_DRS_LVL
#define LOG_DRS_LVL 0
#endif
#ifndef LOG_GTP_LVL
#define LOG_GTP_LVL 0
#endif
#ifndef LOG_QOS_LVL
#define LOG_QOS_LVL 0
#endif
#ifndef LOG_TPI_LVL
#define LOG_TPI_LVL 0
#endif
#ifndef LOG_FP_QOS_LVL
#define LOG_FP_QOS_LVL 0
#endif
#ifndef LOG_FP_MPLS_LVL
#define LOG_FP_MPLS_LVL 0
#endif
#ifndef LOG_CNF_TST_LVL
#define LOG_CNF_TST_LVL 0
#endif
#ifndef LOG_FP_CM_LVL
#define LOG_FP_CM_LVL 0
#endif
#ifndef LOG_FP_ATM_LVL
#define LOG_FP_ATM_LVL 0
#endif
#ifndef LOG_FP_IPV4_LVL
#define LOG_FP_IPV4_LVL 0
#endif
#ifndef LOG_FP_IPV6_LVL
#define LOG_FP_IPV6_LVL 0
#endif
#ifndef LOG_FP_IPV6TM_LVL
#define LOG_FP_IPV6TM_LVL 0
#endif
#ifndef LOG_STUBS_LVL
#define LOG_STUBS_LVL 0
#endif
#ifndef LOG_ATM_LVL
#define LOG_ATM_LVL 0
#endif

// This enumeration defines the possible verbosity levels. The default
// verbosity level can be set at compile time using the LOG_VERBOSITY
// flag above. Setting the verbosity to one of these levels also turns on
// the verbosity levels below it implicitly. For example, if LOG_VERB_FUNC
// is set then LOG_VERB_LEVEL and LOG_VERB_COMP are also included implicitly.
typedef enum {
    LOG_VERB_COMP = 0,
    LOG_VERB_LEVEL = 1,
    LOG_VERB_FUNC = 2,
    LOG_VERB_LINE = 3,
    LOG_VERB_FILE = 4,
    LOG_VERB_PID = 5,
    LOG_VERB_TIME = 6,
    LOG_VERB_MAX = LOG_VERB_TIME
} npf_logger_verbosity;

// This enumeration defines the possible log levels that can be set for each
// component. These values are ORd together to yield the log level for the
// component. For instance, if LOG_TRACE and LOG_CRIT are to be enabled the
// actual log level would be LOG_TRACE | LOG_CRIT which would be 0x9. The
// default log level can be set with the #defines above. If all logging needs
// to be disabled for performance reasons the LOG_ENABLE flag above should be
// used. The npf_logger_level values are examined at runtime only. 
// Additional log levels may be added in the future as needed however the
// intent is to keep the number of levels small so that they are used in a
// consistent manner.
// NOTE THE ARRAY BELOW MUST BE CHANGED IF THIS ENUM IS CHANGED
typedef enum {
    LOG_DISABLED = 0x0, // logging is disabled
    LOG_TRACE = 0x1,    // trace messages, function entered/exited
    LOG_INFO  = 0x2,    // informational message, printf debugging
    LOG_ERR   = 0x4,    // error message, not critical, recoverable error
    LOG_CRIT  = 0x8,    // ciritcal error, unrecoverable
    LOG_LEVEL_MAX = LOG_CRIT
} npf_logger_level; // debug-level message 

#define LOG_NUM_LEVELS 4

// This array is used to translate the enum value to a string for output in
// the log message prefix.
// NOTE THIS MUST BE UPDATED IF THE ENUM IS CHANGED
//static char *npf_logger_level_names[LOG_LEVEL_MAX+1] = { "UNUSED", "TRACE", "INFO", "UNUSED", "ERR", "UNUSED", "UNUSED", "UNUSED", "CRIT" };
// Above variable is moved to log.c

// This enumeration defines the possible components that can be logged. Each
// component in the PDK should have a value in this enum and should only use
// its own enum value when printing a log message. 
// NOTE THE ARRAY BELOW MUST BE CHANGED IF THIS ENUM IS CHANGED
typedef enum {
    LOG_PDKMGR = 0, // pdk manager
    LOG_NS,         // namespace
    LOG_LBM,        // label manager
    LOG_TPM,        // topology manager
    LOG_CM,         // configuration & management
    LOG_IF,	    // interface management
    LOG_BD,         // binding & discovery
    LOG_IPV4,       // ipv4 implementation
    LOG_IPV6,       // ipv6 implementation
    LOG_IPV6TM,     // ipv6tm implementation
    LOG_DS,    	    // DiffServ implementation
    LOG_INTSERV,    // intserv implementation
    LOG_MPLS,       // mpls tsm
    LOG_TOPO,       // inter-fe forwarding tsm
    LOG_CB,         // callback thread
    LOG_VIC,        // VIDD Controller
    LOG_PH,         // Packet Handler
    LOG_SH,         // Shim layer
    LOG_APP,        // Application
    LOG_TPI,        // transport plug-in
    LOG_DRS,        // data relay shim (co-located TPI)
    LOG_GTP,        // gprs tunnel protocol
    LOG_FP_GTP,     // FP GTPManager
    LOG_FP_QOS,     // FP QoSManager
    LOG_FP_MPLS,    // FP MPLS Manager
    LOG_FP_CM,      // FP CM Manager
    LOG_FP_ATM,    // FP ATM Manager 
    LOG_FP_IPV4,    // FP IPV4 Manager 
    LOG_FP_IPV6,    // FP IPV6 Manager 
    LOG_FP_IPV6TM,  // FP IPV6TM Manager 
    LOG_FP_BOOT,    // FP Boot Manager
    LOG_FP_BD,      // FP Binding & Discovery
    LOG_IXA_STUBS,  // IXA SDK stubs
    LOG_CNF_TST,    // Conformance Test    
    LOG_SQE_IXA_STUBS,  // IXA SDK stubs
    LOG_ATM,        // ATM API 
    LOG_MAX_COMPS 
} npf_logger_component;

// This array is used to translate the enum value to a string for output in
// the log message prefix.
// NOTE THIS MUST BE UPDATED IF THE ENUM IS CHANGED
//static char *npf_logger_comp_names[LOG_MAX_COMPS] = { "PDKMGR", "NS", "LBM", "TPM", "CM", "IF", "BD", "IPV4", "IPV6", "IPV6TM", "DIFFSERV", "INTSERV", "MPLS", "TOPO", "CB", "VIC", "PH", "APP", "TPI", "DRS", "GTP", "FP_GTP", "FP_QOS", "FP_MPLS", "FP_CM", "FP_IPV4", "FP_IPV6", "FP_IPV6TM", "FP_BOOT", "FP_BD","IXA_STUBS", "CNF_TST", "SQE_STUBS", "ATM" };
// Above variable is moved to log.c

/*
 * hack for backwards compatibility
 */
extern void * CPLoggerHandle;
extern void * FPLoggerHandle;

#ifdef _TEAMCP_
#define TEAMLOGGERHANDLE CPLoggerHandle
#else

#ifdef _TEAMFP_
#define TEAMLOGGERHANDLE FPLoggerHandle
#else
#define TEAMLOGGERHANDLE void
#endif

#endif

/* Local Interface - Don't use it in other file */
void npf_logger_logEx(void *handle, char *file, const char *func, int line,
    npf_logger_component comp, npf_logger_level level, char *format, ...);

#define npf_logger_log(file, func, line, \
    comp, level, format, args...) \
	npf_logger_logEx( TEAMLOGGERHANDLE, file, func, line, \
		comp, level, format, ##args)

void npf_logger_truncate(void);

/*****************************************************************************
*                                                                            *
*  --------------------------- Public Interface ---------------------------  *
*                                                                            *
*****************************************************************************/
NPF_RET npf_logger_Init( );
NPF_RET npf_logger_StartEx( void **handle, char *filename );
NPF_RET npf_logger_StopEx(void *handle );
NPF_RET npf_logger_SetLevelEx( void *handle, npf_logger_component comp, uint32_t level );
NPF_RET npf_logger_SetVerbosityEx( void *handle, npf_logger_verbosity verb );


/*
 * backwards compatiblity
 */
#define npf_logger_Start( filename )\
	npf_logger_StartEx( & TEAMLOGGERHANDLE, filename )

#define npf_logger_Stop()\
	npf_logger_StopEx( TEAMLOGGERHANDLE )

#define npf_logger_SetLevel( comp, level )\
	npf_logger_SetLevelEx( TEAMLOGGERHANDLE, comp, level )

#define npf_logger_SetVerbosity( verb )\
	npf_logger_SetVerbosityEx( TEAMLOGGERHANDLE, verb )

#define npf_logger_MyWrite(component, level, format, args...) \
        npf_logger_log(__FILE__, __FUNCTION__, __LINE__, component, level, format, ##args)

#if LOG_ENABLE
#define npf_logger_Write(component, level, format, args...) \
        npf_logger_log(__FILE__, __FUNCTION__, __LINE__, component, level, format, ##args)
/*#define npf_logger_Write(component, level, format, args...)*/ 
#define npf_logger_WriteEx(handle, component, level, format, args...) \
        npf_logger_logEx(handle, __FILE__, __FUNCTION__, __LINE__, component, level, format, ##args)
#else
#define npf_logger_Write(component, level, format, args...)
#define npf_logger_WriteEx(handle, component, level, format, args...)
#endif


//
// Old interface, to be removed after all code is migrated to new logger
// interface. DO NOT USE THIS INTERFACE FOR NEW CODE!!
//
#define npf_logger_open( filename) \
    { \
    printf( "npf_logger_open is deprecated, use npf_logger_Start instead\n" ); \
    npf_logger_Start( filename ); \
    }
#define npf_logger_close()\
    { \
    printf( "npf_logger_close is deprecated, use npf_logger_Stop instead\n" ); \
    npf_logger_Stop();\
    }
#define npf_logger_setdisplay(display) \
    printf( "npf_logger_setdisplay is deprecate, do not use\n" );
#define npf_logger_setlevel(level) \
    printf( "npf_logger_setlevel is deprecated, use npf_logger_SetLevel instead\n" );
#define npf_logger_setmaxline(maxline) \
    printf( "npf_logger_setmaxline is deprecated, do not use\n" );
#define npf_logger_setcomponent(A, args...) \
    printf( "npf_logger_setcomponent is deprecated, use npf_logger_SetLevel instead\n" );
#define npf_logger_write(component, level, format, args...) \
        npf_logger_log(__FILE__, __FUNCTION__, __LINE__, component, level, format, ##args)
/*#define npf_logger_write(component, level, format, args...)*/ 
        
#ifdef __cplusplus
}
#endif

#endif  /* _LOG_H_ */
