/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/*
 *    pdk_callback.h can only be included in the PDK internal components
 *    external user program should use existing npf api for registering
 *    callback service. it's better to put in CODEBASE/inc but I don't 
 *    know where to put the source file then.
 *
 */


#ifndef _PDK_CALLBACK_H_
#define _PDK_CALLBACK_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "npf_common.h"
#include "list.h"
#include "pdk_callback_types.h"

/*
 * Backwards compatibility hack
 */

extern void * CPCallbackHandle;
extern void * FPCallbackHandle;

#ifdef _TEAMCP_
#define TEAMCALLBACKHANDLE CPCallbackHandle
#else

#ifdef _TEAMFP_
#define TEAMCALLBACKHANDLE FPCallbackHandle
#else
#define TEAMCALLBACKHANDLE void
#endif

#endif

/* Register callback in a category */
extern NPF_RET pdk_callback_registerEx(	void* handle,
					NPF_CBCAT		category,
					NPF_USERCONTEXT		context,
					NPF_GENERAL_CBFUNC	func,
					NPF_CBHANDLE 		*cbid );

/* Deregister callback */
extern NPF_RET pdk_callback_deregisterEx( void* handle, NPF_CBHANDLE cbid);

/* Remove all the callback in a category */
extern NPF_RET pdk_callback_flushCatEx( void* handle, NPF_CBCAT category );

/* Initialize the callback library at the boot time */
extern NPF_RET pdk_callback_initEx( void** handle);

/* Destroy the callback at shutdown time */
extern NPF_RET pdk_callback_destroyEx( void* handle );

/* Exam if the callback registered? */
extern NPF_RET pdk_callback_isCBExistEx( void* handle, NPF_CBHANDLE cbid);

/* Get the callback category by checking callback id */
extern NPF_RET pdk_callback_getCategoryEx( void* handle, NPF_CBHANDLE cbid,
						NPF_CBCAT	*category);

/* Retrive the callback function */
extern NPF_GENERAL_CBFUNC pdk_callback_funcEx( void* handle, NPF_CBHANDLE cbid,
						NPF_USERCONTEXT	*context);

/* Retrive the first callback func in a category */
extern NPF_GENERAL_CBFUNC pdk_callback_firstFuncEx( void* handle, NPF_CBCAT category,
                                                 NPF_USERCONTEXT *context,
                                                 NPF_CBHANDLE	 *cbid);

/* Retrive the next callback func in same category */
extern NPF_GENERAL_CBFUNC pdk_callback_nextFuncEx( void* handle, NPF_CBHANDLE cbid,
                                                NPF_USERCONTEXT *context,
						NPF_CBHANDLE	*nextcbid);

/* Private Member */
NPF_RET pdk_callback_printEx(void* handle);

/* redefine old names to preserve existing code */
#define pdk_callback_register(	category, context, func, cbid )\
        pdk_callback_registerEx( TEAMCALLBACKHANDLE, category, context, func, cbid )

#define pdk_callback_deregister(cbid)\
        pdk_callback_deregisterEx(TEAMCALLBACKHANDLE, cbid)

#define pdk_callback_flushCat(category )\
        pdk_callback_flushCatEx(TEAMCALLBACKHANDLE, category )

#define pdk_callback_init()\
        pdk_callback_initEx(&TEAMCALLBACKHANDLE)

#define pdk_callback_destroy()\
        pdk_callback_destroyEx(TEAMCALLBACKHANDLE)

#define pdk_callback_isCBExist(cbid)\
        pdk_callback_isCBExistEx(TEAMCALLBACKHANDLE, cbid)

#define pdk_callback_getCategory(cbid, category)\
        pdk_callback_getCategoryEx(TEAMCALLBACKHANDLE, cbid, category)

#define pdk_callback_func(cbid, context)\
        pdk_callback_funcEx(TEAMCALLBACKHANDLE, cbid, context)

#define pdk_callback_firstFunc(category, context,cbid)\
        pdk_callback_firstFuncEx(TEAMCALLBACKHANDLE, category, context,cbid)

#define pdk_callback_nextFunc(cbid, context, nextcbid)\
        pdk_callback_nextFuncEx(TEAMCALLBACKHANDLE, cbid, context, nextcbid)

#define pdk_callback_print(void)\
        pdk_callback_printEx(TEAMCALLBACKHANDLE)

#ifdef __cplusplus
}
#endif

#endif /* _PDK_CALLBACK_H_ */
