/*****************************************************************************
 *                  INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 ******************************************************************************/

/*****************************************************************************
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/common/inc/pdk_hash.h,v $
 * $Revision: 1.5 $
 * $Date: 2003/11/17 16:57:56 $
 * 
 *  Description: Header File for Hash.
 * 
 ******************************************************************************/



#ifndef __PDK_HASH_H__
#define __PDK_HASH_H__

#include "list.h"                              /* D-List Header */

#ifdef __cplusplus
extern "C" {
#endif

/* Return values for Compare functions */
typedef enum 
{
   PDK_HASH_EQUAL = 1,                    /* Matched Return  */
   PDK_HASH_DIFF  = 2                     /* Mismatch Return */
} PdkHashComp;

/* 
 * Return values for Hash External User functions 
 */
typedef enum 
{
   PDK_HASH_OK        = 0,                /* Successful Operation  */
   PDK_HASH_FAILED    = 1,                /* Failed  Operation     */
   PDK_HASH_DUPLICATE = 2                 /* Duplicate Entry found */
} PdkHashRet;


/***************************************************************************
 * Description: Compare key with the key present in the data
 *
 * Params:
 * IN data: Pointer to data block to be hashed. The data contains
 *          its unique key.
 * IN key: Pointer to the key structure to which the key of the
 *         data has to be compared.
 **************************************************************************/

typedef PdkHashComp (*PdkHashCompFn)(const void  *data, const void *key);


/*************************************************************************
 * Description: Pointer to a function that computes the hash index.
 *
 * Params:
 * IN key: Pointer to the key structure.
 **************************************************************************/

typedef uint32_t (*PdkHashIndexFn) (const void *key);

/*************************************************************************
 * Description: Pointer to a function that frees the memory.
 *
 * Params:
 * IN key: Pointer to the allocated block.
 **************************************************************************/

typedef void (*PdkHashFreeFn) (void *mem);


/*
 * Hash Control Block.
 */
typedef struct 
{
   uint32_t        len;               /* No of bins in Hash Table      */
   PdkHashCompFn   compFn;            /* Comparison fn. used in search */
   PdkHashIndexFn  indFn;             /* Hash Index Computation fn.    */
   DList           *array;            /* Array of D-List Control Blocks*/
} PdkHashCb;


/*
 * Hash Initialization function
 */
extern PdkHashRet pdkHashInit(PdkHashCb         *cb, 
                              uint32_t          len,
                              PdkHashCompFn     compFn, 
                              PdkHashIndexFn    indFn, 
                              PdkHashFreeFn     freeFn);

/*
 * Hash Node Insertion function
 */
extern PdkHashRet pdkHashInsert(PdkHashCb       *cb,
                                void            *key,
                                void            *data);

/*
 * Hash Node Deletion function (Does not Free buffer )
 */
extern PdkHashRet pdkHashDelete(PdkHashCb         *cb, 
                                void              *key,
                                void              **data);

/*
 * Hash Node Deletion & Free function (Frees buffer )
 */
extern PdkHashRet pdkHashDeleteFree(PdkHashCb         *cb, 
                                    void              *key);

/*
 * Hash Node Find function  (search node)
 */
extern PdkHashRet pdkHashFind(PdkHashCb            *cb, 
                              void                 *key,
                              void                 **data);

/*
 * Hash Destroy function  (purge (cleanup) entire Hash Table)
 */
extern PdkHashRet pdkHashDestroy(PdkHashCb     *cb);



#ifdef __cplusplus
}
#endif

#endif /* __PDK_HASH_H__ */
