/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/
 
 
/*****************************************************************************
* $Source: /CODEBASE/common/inc/rwlock.h
* $Revision: 1.6 $
* $Date: 2003/11/17 16:57:56 $
*
* Description:
*    Define a single lock interface which maximize concurrency of 
*	 read/write operations in the folowing sense: When the data is read,
*	 others can join to read it, but cannot write to it; when one is writing
*    to the data, no one else can read or write to it.
*
* Assumption:
*	 It uses critical section from PIL
*
*****************************************************************************/   
#ifndef _RWLOCK_H_
#define _RWLOCK_H_


#ifdef __cplusplus
extern "C" {
#endif 

#include "pil.h"

typedef enum {UNINITIALIZED, INITIALIZED} STATE;

typedef enum {RW_READ, RW_WRITE} RW_OPERATION;

#undef TRUE
#define TRUE    1
 
#undef FALSE
#define FALSE   0
 
typedef char Bool; 

typedef struct RWLOCK_t
{
        STATE              state; /* if the lock is initialized         */
        RW_OPERATION          mode;  /* ongoing operation is READ or WRITE */
        int                nr;    /* number of active readers           */
        int                nw;    /* number of outstanding writers      */
        PilCriticalSection r;     /* read mutex                         */
        PilCriticalSection w;     /* write mutex                        */
} RWLOCK;
 
Bool init(RWLOCK * lock);
Bool acquire(RWLOCK * lock, RW_OPERATION op);
Bool release(RWLOCK * lock);     
Bool destroy(RWLOCK * lock);

#ifdef __cplusplus
}
#endif

#endif /* _RWLOCK_H_ */
