/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/*
 * common list code for every arch to share
 *
 * you must make sure that you have defined the 
 * PILLIST_LOCK and PILLIST_UNLOCK macros for your specific arch
 * and have initialized the critical section in pilinit.c
 *
 * we cannot use PilCriticalSections here because the pil cs functions call these
 * functions and we'd get into nasty infinite loops
 *
 * - stephen d .goglin
 */
#include "pillist.h"

#include "pil_int.h"

typedef struct pillist_s 
{
    struct pillist_s *prev,  *next; //must be first
    void* o;
} *pillist_t;

PILLIST_MUTEX_TYPE  pillistmutex;

void _PIL_AddToList(pillist *l, void* o)
{
    pillist_t n;

    if (!l) return;

    PILLIST_LOCK(pillistmutex);

    n = PIL_AllocateMemory(sizeof(struct pillist_s),0);

    n->o = o;
    n->prev = NULL;

    if (!*l)
    {
        *l= n;
        n->next = NULL;
    }
    else
    {
        n->next = (pillist_t)*l;
        n->next->prev = n;
        *l= n;
    }

    PILLIST_UNLOCK(pillistmutex);
}

static int _PIL_CompareList(void* a, void* b)
{
    return ((int)a==(int)b);
}

pillist_t _PIL_FindNodeInList(pillist *l, void *o, int (*compare)(void *, void *))
{
    pillist_t p;

    if (!l || !*l) return NULL;

    PILLIST_LOCK(pillistmutex);

    p = *l;
    while (p)
    {
        if (compare(p->o,o)) break;
        p = p->next;
    }
    PILLIST_UNLOCK(pillistmutex);

    return p;
}

void _PIL_RemoveFromList(pillist *l, void* o)
{
    pillist_t n;

    PILLIST_LOCK(pillistmutex);

    n = _PIL_FindNodeInList(l,o,_PIL_CompareList);

    if (!n) return;

    if (n->next)
        n->next->prev = n->prev;

    if (n->prev)
        n->prev->next = n->next;
    else
        *l = n->next;

    PILLIST_UNLOCK(pillistmutex);
}

void* _PIL_FindInList(pillist *l, void *o, int (*compare)(void *, void *))
{
    pillist_t n;

    n = _PIL_FindNodeInList(l,o,compare);

    if (!n) return NULL;

    return n->o;
}

void _PIL_ForEachInList(pillist *l, void (*foreach)(void *))
{
    pillist_t p;

    if (!l || !*l) return;

    PILLIST_LOCK(pillistmutex);

    p = *l;
    while (p)
    {
        foreach(p->o);
        p = p->next;
    }
    PILLIST_UNLOCK(pillistmutex);

    return;
}
