/////////////////////////////////////////////////////////////////////////
//                                                                     //
// Filename: CQUEUE.H                                                  //
// Module:   OS Isolation Layer Mailbox Services                       //
// Author:   Venky Gokulrangan                                         //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
//                                                                     //
//      Copyright (C) 1996 Intel Corporation  ALL RIGHTS RESERVED      //
//                                                                     //
//             INTEL CORPORATION PROPRIETARY INFORMATION               //
//                                                                     //
//     This software is supplied under the terms of a license agree-   //
//     ment or non-disclosure agreement with Intel Corporation         //
//     and may not be copied nor disclosed except in accordance        //
//     with the terms of that agreement.                               //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
//                                                                     //
// PURPOSE: Implement Fixed Sized Circular Queue                       //
//                                                                     //
// REQUIRES: Some functions from cqueue.c are called                   //
//                                                                     //
/////////////////////////////////////////////////////////////////////////


#ifndef __CQUEUE_H_DEFINED__
#define __CQUEUE_H_DEFINED__

#include "pil.h"

#ifdef __cplusplus
    extern "C" {
#endif

typedef enum
{
   QUEUE_OK     = 0x00,
   QUEUE_FULL   = 0x01,
   QUEUE_EMPTY  = 0x02,
   QUEUE_ERROR  = 0x03,
   QUEUE_NOMEM  = 0x10
} QueueReturn ;


// This contains the functions for mailbox..
// The functions are intended to be used on a per process basis
// using the static library mechanism

typedef DWORD CQHandle;  // Internal to the module.. So PIL Prefix is
                           // not present

// Self contained object
typedef struct PilCQ_t
{
   DWORD         dwHead ,
                 dwTail ,
                 dwQLen,
                 dwMailSize,
                 dwCount ;
   PVOID         pQ ; // Array of Queue Objects.. 
   PilSemaphore AccessSem ; // to access the variables and the Queue itself

} PilCQ ;

//////////////////////////////////////////////////////////////////////////////
//
// Function:   CreateCQ
// Parameters: Number of Units in the Queue
// Purpose:    Creates a circular queue of the size as specified.
// Returns:    Handle of the created circular queue
//             NULL, if unable to create the circular queue
//
// Notes:      In any case, if an error occurs, all the resources allocated
//             until the previous point are removed
//

CQHandle
CreateCQ ( 
    DWORD dwQLen,
    DWORD dwMailSize ) ;


//////////////////////////////////////////////////////////////////////////////
//
// Function:   Enqueue
// Parameters: 1. Handle to the Circular Queue
//             2. The address to copy the mail message from
// Purpose:    If a mail slot is available, this function queues the message
//             into that slot
//
// Returns:    QUEUE_ERROR if unable to perform the operation
//             QUEUE_FULL  if no space is available
//             QUEUE_OK    if succesful
//

QueueReturn
Enqueue ( 
   CQHandle hCQ,
   PVOID Object ) ; // Object itself is an address

//////////////////////////////////////////////////////////////////////////////
//
// Function:   DeQueue
// Parameters: 1. Handle to the Circular Queue
//             2. The address to copy the mail message to
// Purpose:    If message is available in a mail slot, this function copies
//             the message to the user space.
//
// Returns:    QUEUE_ERROR if unable to perform the operation
//             QUEUE_FULL  if no element is available to copy from to user
//             QUEUE_OK    if succesful
//
QueueReturn
DeQueue (
         CQHandle hCQ,
         PVOID pObject) ;

#ifdef __cplusplus
   }
#endif // cplusplus

#endif // __CQUEUE_H_DEFINED__
