/***********************************************************************
 *                                                                     *
 * Filename: HEAP.H                                                    *
 * Module:   OIL Local Heap Management Internal Declarations           *
 * Author:   Daniel Baumberger                                         *
 *                                                                     *
 ***********************************************************************
 *                                                                     *
 *      Copyright (C) 1996 Intel Corporation  ALL RIGHTS RESERVED      *
 *                                                                     *
 *             INTEL CORPORATION PROPRIETARY INFORMATION               *
 *                                                                     *
 *     This software is supplied under the terms of a license agree-   *
 *     ment or non-disclosure agreement with Intel Corporation         *
 *     and may not be copied nor disclosed except in accordance        *
 *     with the terms of that agreement.                               *
 *                                                                     *
 ***********************************************************************
 *                                                                     *
 * PURPOSE:                                                            *
 ***********************************************************************/

#ifndef _HEAP_H_INCLUDED_
#define _HEAP_H_INCLUDED_

#ifdef __cplusplus
   extern "C" {
#endif

#pragma pack (1)   // Byte align

typedef struct tHeapHeader
{
    struct tHeapHeader* pPrev; /* Pointer to the previous block */
  //    struct tHeapHeader* pNext; /* Pointer to the next block     */
    DWORD  dwSize:31;             /* The size of the block         */
    BOOL   bInUse:1;            /* TRUE if allocated, or FALSE   */
} HeapHeader, *PHeapHeader;

typedef struct tHeapDesc
{
    PHeapHeader phhHead;    /* The header record of the mem pool */
    PHeapHeader phhTail;    /* The tail of the memory pool       */
    DWORD       dwSize;     /* The size of the entire pool       */
} HeapDesc;

#pragma pack ()

typedef DWORD    HeapHandle ;
typedef PVOID*   PPVOID ;


// External Functions for the world to use

//
// Create a heap and return the handle to the heap
//

HeapHandle
PIL_CreateHeap (
        DWORD wMaxHeapSize ) ;  // The maximum amount of mem in the heap

VOID
PIL_DeleteHeap (
        HeapHandle hHeap  ) ;  // The heap to delete


PVOID
PIL_HeapAlloc (
	HeapHandle hHeap,   // The heap to allocate from
        DWORD wSize  )  ;   // The size of block to allocate


BOOL
PIL_HeapFree (
        HeapHandle hHeap,    // The heap to deallocate from
	PVOID      pBuf ) ;  // The pointer to free


BOOL
PIL_HeapShrink (
    HeapHandle hHeap,         // The heap to shrink a buffer from
    PVOID      pBuf,          // The pointer to the buffer to shrink
    DWORD       wNewSize );   // The new (smaller) size

#ifdef DEBUG
VOID
PIL_HeapScan (
    HeapHandle hHeap ) ;  // Debug Routine
#endif


#ifdef __cplusplus
   }
#endif

#endif /* _HEAP_H_INCLUDED_ */
