////////////////////////////////////////////////////////////////////////////////
//
// INTEL CORPORATION PROPRIETARY INFORMATION               
// This software is supplied under the terms of a license agreement 
// or non-disclosure agreement with Intel Corporation and may not be 
// copied nor disclosed except in accordance with the terms of that 
// agreement.                               
//
// Copyright  1999 Intel Corporation. All Rights Reserved.
//
// ABSTRACT:
//   Contains declarations and macros to manipulate a single or double linked
//   list.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef _LLIST_H_INCLUDED_
#define _LLIST_H_INCLUDED_

//------------------------------------------------------------------------------
//
// LIST_NODE
//		Data structure used to maintain doubly linked lists throughout the 
//		driver
//
//------------------------------------------------------------------------------
typedef struct tLIST_NODE {
   struct tLIST_NODE * volatile Flink;
   struct tLIST_NODE * volatile Blink;
} LIST_NODE, *PLIST_NODE;

//------------------------------------------------------------------------------
//
// INITIALIZE_LIST_HEAD
//		Initializes the LIST_NODE structure at the head of a linked list.
//		Must be called prior to using any of the other list manipulation
//		macros.
//
//------------------------------------------------------------------------------
#define INITIALIZE_LIST_HEAD(ListHead) ((ListHead)->Flink = (ListHead)->Blink = (ListHead))

//------------------------------------------------------------------------------
//
// IS_LIST_EMPTY
//		Returns TRUE if the given list is empty
//
//------------------------------------------------------------------------------
#define IS_LIST_EMPTY(ListHead) ((ListHead)->Flink == (ListHead))

//------------------------------------------------------------------------------
//
// REMOVE_HEAD_LIST
//		Returns the entry at the head of the list.
//		Must NEVER be called with an empty list.  Use IS_LIST_EMPTY to determine
//		if the list is empty prior to calling this function.
//
//------------------------------------------------------------------------------
#define REMOVE_HEAD_LIST(ListHead) (ListHead)->Flink; {REMOVE_ENTRY_LIST((ListHead)->Flink)}

//------------------------------------------------------------------------------
//
// REMOVE_TAIL_LIST
//		Returns the entry at the tail of the list.
//		Must NEVER be called with an empty list.  Use IS_LIST_EMPTY to determine
//		if the list is empty prior to calling this function.
//
//------------------------------------------------------------------------------
#define REMOVE_TAIL_LIST(ListHead) (ListHead)->Blink; {REMOVE_ENTRY_LIST((ListHead)->Blink)}

//------------------------------------------------------------------------------
//
// REMOVE_ENTRY_LIST
//		Removes the given entry from the list
//
//------------------------------------------------------------------------------
#define REMOVE_ENTRY_LIST(Entry) { PLIST_NODE _EX_Blink; PLIST_NODE _EX_Flink; _EX_Flink = (Entry)->Flink; _EX_Blink = (Entry)->Blink; _EX_Blink->Flink = _EX_Flink; _EX_Flink->Blink = _EX_Blink; }

//------------------------------------------------------------------------------
//
// INSERT_TAIL_LIST
//		Inserts the given entry at the tail of the list specified by ListHead
//
//------------------------------------------------------------------------------
#define INSERT_TAIL_LIST(ListHead,Entry) {PLIST_NODE _EX_Blink;PLIST_NODE _EX_ListHead;_EX_ListHead = (ListHead);_EX_Blink = _EX_ListHead->Blink;(Entry)->Flink = _EX_ListHead;(Entry)->Blink = _EX_Blink;_EX_Blink->Flink = (Entry);_EX_ListHead->Blink = (Entry);}

//------------------------------------------------------------------------------
//
// INSERT_HEAD_LIST
//		Inserts the given entry at the head of the list specified by ListHead
//
//------------------------------------------------------------------------------
#define INSERT_HEAD_LIST(ListHead,Entry) {PLIST_NODE _EX_Flink;PLIST_NODE _EX_ListHead;_EX_ListHead = (ListHead);_EX_Flink = _EX_ListHead->Flink;(Entry)->Flink = _EX_Flink;(Entry)->Blink = _EX_ListHead;_EX_Flink->Blink = (Entry);_EX_ListHead->Flink = (Entry);}

//------------------------------------------------------------------------------
//
// GET_FIRST_ENTRY_FROM_LIST
//		Returns the first entry in the list without removing the entry from
//		the list (if the list is empty, NULL is returned)
//
//------------------------------------------------------------------------------
#define GET_FIRST_ENTRY_FROM_LIST( ListHead) ((IS_LIST_EMPTY(ListHead)) ? NULL : ((ListHead)->Flink))

//------------------------------------------------------------------------------
//
// GET_NEXT_ENTRY_FROM_LIST
//		Returns the entry in the list following the given entry.  If the given
//		entry is the last in the list, NULL is returned.
//
//------------------------------------------------------------------------------
#define GET_NEXT_ENTRY_FROM_LIST( ListHead,Entry) (((((PLIST_NODE)(Entry))->Flink) == (ListHead)) ? NULL: (((PLIST_ENTRY)(Entry))->Flink))

//------------------------------------------------------------------------------
//
// SINGLE_LIST_NODE
//		Data structure used to maintain singly linked lists throughout the 
//		driver
//
//------------------------------------------------------------------------------
typedef struct tSINGLE_LIST_NODE {
    struct tSINGLE_LIST_NODE *Next;
} SINGLE_LIST_NODE, *PSINGLE_LIST_NODE;

//------------------------------------------------------------------------------
//
// POP_ENTRY_LIST
//		Pops the an entry from the head of a singly linked list
//
//------------------------------------------------------------------------------
#define POP_ENTRY_LIST(ListHead) (ListHead)->Next;{PSINGLE_LIST_NODE FirstEntry;FirstEntry = (ListHead)->Next;if (FirstEntry != NULL) {(ListHead)->Next = FirstEntry->Next;}}

//------------------------------------------------------------------------------
//
// PUSH_ENTRY_LIST
//		Pushes an entry onto the head of a singly linked list
//
//------------------------------------------------------------------------------
#define PUSH_ENTRY_LIST(ListHead,Entry) (Entry)->Next = (ListHead)->Next; (ListHead)->Next = (Entry)


#endif // _LLIST_H_INCLUDED_
