/////////////////////////////////////////////////////////////////////////
//                                                                     //
// Filename: PILINIT.H                                                 //
// Module:   Platform Isolation Layer Initialization/De-Initialization //
// Author:   Daniel Baumberger/Jose Puthenkulam                        //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
//                                                                     //
//      Copyright (C) 1999 Intel Corporation  ALL RIGHTS RESERVED      //
//                                                                     //
//             INTEL CORPORATION PROPRIETARY INFORMATION               //
//                                                                     //
//     This software is supplied under the terms of a license agree-   //
//     ment or non-disclosure agreement with Intel Corporation         //
//     and may not be copied nor disclosed except in accordance        //
//     with the terms of that agreement.                               //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
//                                                                     //
// PURPOSE:                                                            //
//                                                                     //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
//
// HISTORY:
//
// 
/////////////////////////////////////////////////////////////////////////

#ifndef __PILINIT_H_INCLUDED__
#define __PILINIT_H_INCLUDED__

//
// @doc
// @topic Initialization and De-initialization (Required) |
//
// PIL has an explicit initialization and de-initalization function.
// These are required to be called by any application before using
// any PIL function and after they are finished with the PIL layer.
// This allows PIL to perform and implementation-specific initialization
// and de-initialization in a portable fashion.  The behavior of calling
// a PIL function without first calling <f PIL_Init> is undefined.
//

///////////////////////////////////////////////////////////////////////
//
// PIL_Init
//
// @func <f PIL_Init> must be called before any other PIL function may
//   be used, including any utility or run-time functions that are
//   exported by PIL.  This allows PIL to do implementation-specific
//   initialization in a portable manner.
//
//   <f PIL_Init> is a synchronous call.  It will not return until the
//   initialization is complete.  If another call is made into 
//   <f PIL_Init> before the first call completes, it will also block
//   until initialization is complete.  Once the initialization is
//   complete, subsequent calls to <f PIL_Init> will return an error
//   code of <ms PIL_INITIALIZED>.
//	
// @rdesc Returns one of the following:
//
// @flag PIL_OK          | PIL initialization is successful.
// @flag PIL_NOMEM       | There is insufficient memory for PIL to 
//                         initialize.
// @flag PIL_INITIALIZED | <f PIL_Init> has already been completed.
//
//

PilResult
PIL_Init();

///////////////////////////////////////////////////////////////////////
//
// PIL_DeInit
//
// @func <f PIL_DeInit> must be called when PIL is no longer required.
//   No PIL function may be called after this function is called.  This
//   allows PIL to release any implementation-specific resources in a
//   portable manner.
//
//   <f PIL_DeInit> is a synchronous call.  It will not return until
//   the deinitialization is complete.  If another call is made into
//   <f PIL_DeInit> before the first call complete, it will also block
//   until deinitialization is complete.  Once the deinitialization is
//   complete, subsequent calls to will simply return.
//
// @rdesc This function does not return a value.
//
//

void
PIL_DeInit();


#endif // __PILINIT_H_INCLUDED__
