/////////////////////////////////////////////////////////////////////////
//                                                                     //
// Filename: PILINTRL.H                                                //
// Module:   Platform Isolation Layer Interlocked Services             //
// Author:   Daniel Baumberger                                         //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
//                                                                     //
//      Copyright (C) 1999 Intel Corporation  ALL RIGHTS RESERVED      //
//                                                                     //
//             INTEL CORPORATION PROPRIETARY INFORMATION               //
//                                                                     //
//     This software is supplied under the terms of a license agree-   //
//     ment or non-disclosure agreement with Intel Corporation         //
//     and may not be copied nor disclosed except in accordance        //
//     with the terms of that agreement.                               //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
//                                                                     //
// Declares the Interlocked Services API of PIL.                       //
//                                                                     //
/////////////////////////////////////////////////////////////////////////

#ifndef __PILINTRL_H_INCLUDED__
#define __PILINTRL_H_INCLUDED__

//
// @doc
//
// @topic Interlocked services (Optional) |
//
// Interlocked services allow operations to be performed on memory in an
// atomic way, protecting the operation from other threads and other
// processors.  In a multiprocessor environment, these functions assure
// the processor has exclusive use of any shared memory.
//

///////////////////////////////////////////////////////////////////////
//
// PILOS_InterlockedCompareExchange
//
// @func <f PILOS_InterlockedCompareExchange> atomically compares the 
//   <p Comperand> value with the <p Destination>value, exchanging the
//   <p Destination> value and <p Exchange> value if they are equal.  The values
//   are treated as 32-bit unsigned values.
//
// @rdesc The original value of <p Destination>.
//
//

int32_t
PILOS_InterlockedCompareExchange(
    int32_t* Destination,    // @parm Address of the destination variable.
    int32_t  Exchange,       // @parm The exchange value if the comparison
                             //   succeeds.
    int32_t  Comperand       // @parm The value to compare with the destination.
    );

///////////////////////////////////////////////////////////////////////
//
// PILOS_InterlockedDecrement
//
// @func <f PILOS_InterlockedDecrement> atomically decrements the given
//   32-bit value by one and returns the result.
//
// @rdesc The resulting decremented value.
//
//

int32_t
PILOS_InterlockedDecrement(
    int32_t* Value    // @parm Pointer to value to decrement
    );

///////////////////////////////////////////////////////////////////////
//
// PILOS_InterlockedExchange
//
// @func <f PILOS_InterlockedExchange> atomically assigns the value
//   of <p Value> to <p Target>, returning the old value of <p Target>
//    as a result.
//
// @rdesc The prior value of target before the exchange.
//
//

int32_t
PILOS_InterlockedExchange(
    int32_t* Target, // @parm Pointer to the long value to exchange.
    int32_t  Value   // @parm The new value for the target.
    );

///////////////////////////////////////////////////////////////////////
//
// PILOS_InterlockedIncrement
//
// @func <f PILOS_InterlockedIncrement> atomically increments <p Value>
//   and returns the result.
//
// @rdesc The 32-bit resulting incremented value.
//
//

int32_t
PILOS_InterlockedIncrement(
    int32_t* Value    // @parm Pointer to value to Increment
    );

#endif // __PILINTRL_H_INCLUDED__
