/////////////////////////////////////////////////////////////////////////
//                                                                     //
// Filename: PILLIB.H                                                  //
// Module:   Platform Isolation Layer Dynamic Library Routines         //
// Author:   Daniel Baumberger                                         //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
//                                                                     //
//      Copyright (C) 1999 Intel Corporation  ALL RIGHTS RESERVED      //
//                                                                     //
//             INTEL CORPORATION PROPRIETARY INFORMATION               //
//                                                                     //
//     This software is supplied under the terms of a license agree-   //
//     ment or non-disclosure agreement with Intel Corporation         //
//     and may not be copied nor disclosed except in accordance        //
//     with the terms of that agreement.                               //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
//                                                                     //
// PURPOSE:                                                            //
//                                                                     //
// Declares the PIL Dynamic Library routines.                          //
//                                                                     //
/////////////////////////////////////////////////////////////////////////

#ifndef __PILLIB_H_INCLUDED__
#define __PILLIB_H_INCLUDED__

// 
// @doc
//
// @topic Dynamic Libraries (Optional) |
//
// Dynamic libraries allow modules to be loaded and symbols be resolved
// at run-time.  For those environments where dynamic libraries are 
// supported by the underlying OS, PIL will utilize those services.  In
// other environments, however, PIL will have to emulate the dynamic
// loading mechanism.  The actual implementation of this emulation is
// platform specific.  Refer to the platform specific notes for each
// implemenation too see if dynamic libraries are native or emulated.
//
  
typedef void* PilLib;

typedef int32_t (*PilProc)();

///////////////////////////////////////////////////////////////////////
//
// PILOS_FreeLibrary
//
// @func <f PILOS_FreeLibrary> frees a library loaded via 
//   <f PILOS_LoadLibrary>.
//
// @rdesc Returns one of the following:
//
// @flag PIL_OK             | The library was successfully freed.
// @flag PIL_INVALID_HANDLE | The handle of the library is not a valid 
//                            library handle.
//
  
PilResult
PILOS_FreeLibrary(
    PilLib Lib          // @parm The handle of the library to free
    );

///////////////////////////////////////////////////////////////////////
//
// PILOS_GetProcAddress
//
// @func <f PILOS_GetProcAddress> looks up a function in a dynamic 
//   library loaded via <f PILOS_LoadLibrary>.  The address of the function
//   is returned and should be casted to the appropriate function prototype.
//   The PilProc type is defined as:
//
//   <ms typedef int32_t (*PilProc)();>
//
// @rdesc Returns one of the following:
//
// @flag PilProc | The address of the symbol.
// @flag NULL    | The symbol could not be found in this dynamic library.
//

PilProc
PILOS_GetProcAddress(
    PilLib Lib,         // @parm The library in which to look up a function in.
    char*  ProcName     // @parm The name of the function to retrieve.
    );

///////////////////////////////////////////////////////////////////////
//
// PILOS_LoadLibrary
//
// @func <f PILOS_LoadLibrary> loads a dynamic library module into memory, 
//   returning the handle of the library.  Function points can be retrieved
//   from this module by using <f PILOS_GetProcAddress>. 
//
// @rdesc Returns one of the following:
//
// @flag PilLib | The handle of the newly loaded library.
// @flag NULL   | The library could not be loaded.
//
//

PilLib
PILOS_LoadLibrary(
    char* LibraryName  // @parm The name of the library to load.
    );

#endif // __PILLIB_H_INCLUDED__
