/////////////////////////////////////////////////////////////////////////
//                                                                     //
// Filename: PILMISC.H                                                 //
// Module:   Platform Independent Layer Miscellaneous Routines         //
// Author:   Dan Baumberger                                            //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
//                                                                     //
//      Copyright (C) 1999 Intel Corporation  ALL RIGHTS RESERVED      //
//                                                                     //
//             INTEL CORPORATION PROPRIETARY INFORMATION               //
//                                                                     //
//     This software is supplied under the terms of a license agree-   //
//     ment or non-disclosure agreement with Intel Corporation         //
//     and may not be copied nor disclosed except in accordance        //
//     with the terms of that agreement.                               //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
//                                                                     //
// PURPOSE:                                                            //
//                                                                     //
//   Declares the PIL Miscellaneous Services API.                      //
//                                                                     //
/////////////////////////////////////////////////////////////////////////

#ifndef __PILMISC_H_INCLUDED__
#define __PILMISC_H_INCLUDED__

//
// @doc
//
// @topic Miscellaneous services |
//
// The miscellaneous services of PIL is a grab bag of extra functions
// that do not belong anywhere else.  
//

///////////////////////////////////////////////////////////////////////
//
// PIL_Abort
//
// @func <f PIL_Abort> will display the printf-style message on the 
//   appropriate device and immediately abort execution.  The actual 
//   effect of aborting execution is implementation dependent other 
//   than this function will never return.
//
// @rdesc This function does not return.
//
// 

void
PIL_Abort(
    char* fmt,          // @parm A print-style format string
    ...                 // followed by optional additional parameters.
    );


///////////////////////////////////////////////////////////////////////
//
// PIL_GetCurrentTime
//
// @func <f PIL_GetCurrentTime> returns a 64-bit time value in 
//   milliseconds.  Depending on the implementation, this function may 
//   not return the actual clock time but rather may be some number of 
//   milliseconds since an event in the past (usually system start).  
//   However, it is guaranteed that the difference between any two calls 
//   of this function be the actual elapsed time in milliseconds.
//
// @rdesc The current time value in milliseconds.
//
//

unsigned long long
PIL_GetCurrentTime(
    );

///////////////////////////////////////////////////////////////////////
//
// PIL_Sleep
//
// @func <f PIL_Sleep> delays the execution of the process for at least
// the time specified.
//
// @rdesc return zero on success.
//
//

int
PIL_Sleep(
	int numsec,
	long nanosec
    );

///////////////////////////////////////////////////////////////////////
//
// PIL_GetHostByName
//
// @func <f PIL_GetHostByName> returns the DNS resolved IP Address 
//  for the given host.  
//
//////////////////////////////////////////////////////////////////////

struct hostent *  
PIL_GetHostByName( 
	char *pHostName
	);

#ifdef _VXWORKS_
#define gethostbyname PIL_GetHostByName
#define sleep(x)         PIL_Sleep(x, 0)
#endif // _VXWORKS_


#endif // __PILMISC_H_INCLUDED__

