/////////////////////////////////////////////////////////////////////////
//                                                                     //
// Filename: PIL_INT.H                                                 //
// Module:   Platform Isolation Layer Internal Header                  //
// Author:   JP Sugarbroad                                             //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
//                                                                     //
//      Copyright (C) 1999 Intel Corporation  ALL RIGHTS RESERVED      //
//                                                                     //
//             INTEL CORPORATION PROPRIETARY INFORMATION               //
//                                                                     //
//     This software is supplied under the terms of a license agree-   //
//     ment or non-disclosure agreement with Intel Corporation         //
//     and may not be copied nor disclosed except in accordance        //
//     with the terms of that agreement.                               //
//                                                                     //
/////////////////////////////////////////////////////////////////////////

#ifndef _PIL_INT_H_
#define _PIL_INT_H_

#include <features.h>

#include "pil.h"

#include <pthread.h>
#include <errno.h>
#include <signal.h>
#include <sys/time.h>

#include "pillist.h"

#ifdef DEBUG
typedef enum {
    THREAD_INVALID,
    THREAD_ACTIVE,
    THREAD_SUSPENDED,
    THREAD_EXITED
} threadstatus;
#endif

// The structure behind PilThread
typedef struct thread_s {
#ifdef DEBUG
    uint32_t magic;
#endif
    char *name;
    pthread_t thread;
#ifdef DEBUG
    threadstatus status;
#endif
} thread_t;

#ifdef DEBUG
typedef struct csinfo_s{
    char *name;
    thread_t *owner;
    pillist blocking; //which threads are blocking on this cs
} csinfo_t;
#endif

// The structure behind PilCriticalSection
typedef struct {
#ifdef DEBUG
    uint32_t magic;
#endif
    pthread_mutex_t mutex;
#ifdef DEBUG
    csinfo_t *info;
#endif
} pilcs_t;

#define PILLIST_MUTEX_TYPE pthread_mutex_t

#define PILLIST_LOCK(l) \
	pthread_mutex_lock(&l)

#define PILLIST_UNLOCK(l) \
	pthread_mutex_unlock(&l)

#endif
