/////////////////////////////////////////////////////////////////////////
//                                                                     //
// Filename: PILINTRL.C                                                //
// Module:   Platform Isolation Layer Interlocked Services             //
// Author:   Daniel Baumberger                                         //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
//                                                                     //
//      Copyright (C) 1999 Intel Corporation  ALL RIGHTS RESERVED      //
//                                                                     //
//             INTEL CORPORATION PROPRIETARY INFORMATION               //
//                                                                     //
//     This software is supplied under the terms of a license agree-   //
//     ment or non-disclosure agreement with Intel Corporation         //
//     and may not be copied nor disclosed except in accordance        //
//     with the terms of that agreement.                               //
//                                                                     //
/////////////////////////////////////////////////////////////////////////

// DO NOT include pil_int.h or pil.h

#include "stdint.h"

#ifdef _IXP_
#include <asm-arm/system.h>

#ifndef __cli
#include <asm-arm/proc/system.h>
#endif
#else
#include <asm/system.h>

#ifndef cli
#include <asm/proc/system.h>
#endif
#endif


///////////////////////////////////////////////////////////////////////
//
// PILOS_InterlockedCompareExchange
//
// Atomically compares two values, exchanging the destination with
// another value if the comparison is TRUE.
//

int32_t
PILOS_InterlockedCompareExchange(
    int32_t* Destination,    // @parm Address of the destination variable.
    int32_t  Exchange,       // @parm The exchange value if the comparison
                             //   succeeds.
    int32_t  Comperand       // @parm The value to compare with the destination.
    )
{
    int32_t ret = *Destination;
#ifdef USE_ASM
    __asm__ __volatile__ (
        "lock; cmpxchg %1, %0"
        : "=m" (*Destination)
        : "r" (Exchange), "a" (Comperand));
#else
#ifdef _IXP_
    unsigned long flags;

    __save_flags(flags);
    __cli();
    if (*Destination == Comperand)
    {
        *Destination = Exchange;
    }
    __restore_flags(flags);

#else
    unsigned long flags;

    save_flags(flags);
    cli();
    if (*Destination == Comperand)
    {
        *Destination = Exchange;
    }
    restore_flags(flags);
#endif
#endif    
    return ret;
}

///////////////////////////////////////////////////////////////////////
//
// PILOS_InterlockedDecrement
//
// Atomically decrements a variable.
//

int32_t
PILOS_InterlockedDecrement(
    int32_t* Value    // @parm Pointer to value to decrement
    )
{
#ifdef USE_ASM
    __asm__ __volatile__ (
        "lock; decl %0"
        : "=m" (*Value));
#else
#ifdef _IXP_
    unsigned long flags;

    __save_flags(flags);
    __cli();
    *Value--;
    __restore_flags(flags);
#else
    unsigned long flags;

    save_flags(flags);
    cli();
    *Value--;
    restore_flags(flags);
#endif
#endif    
    return *Value;
}

///////////////////////////////////////////////////////////////////////
//
// PILOS_InterlockedExchange
//
// Atomically exchanges two values.
//

int32_t
PILOS_InterlockedExchange(
    int32_t* Target, // @parm Pointer to the long value to exchange.
    int32_t  Value   // @parm The new value for the target.
    )
{
#ifdef USE_ASM
    __asm__ __volatile__ (
        "lock; xchgl %1, %0"
        : "=m" (*Target), "+r" (Value));
#else
#ifdef _IXP_
    unsigned long flags;

    __save_flags(flags);
    __cli();
    *Target = Value;
    __restore_flags(flags);
#else
    unsigned long flags;

    save_flags(flags);
    cli();
    *Target = Value;
    restore_flags(flags);
#endif
#endif    
    return Value;
}

///////////////////////////////////////////////////////////////////////
//
// PILOS_InterlockedIncrement
//
// Atomically increments a variable.
//

int32_t
PILOS_InterlockedIncrement(
    int32_t* Value    // @parm Pointer to value to Increment
    )
{
#ifdef USE_ASM
    __asm__ __volatile__ (
        "lock; incl %0"
        : "=m" (*Value));
#else
#ifdef _IXP_
    unsigned long flags;

    __save_flags(flags);
    __cli();
    *Value++;
    __restore_flags(flags);
#else
    unsigned long flags;

    save_flags(flags);
    cli();
    *Value++;
    restore_flags(flags);
#endif
#endif
    return *Value;
}

