/////////////////////////////////////////////////////////////////////////
//                                                                     //
// Filename: PILLIB.C                                                  //
// Module:   Platform Isolation Layer Dynamic Library Routines         //
// Author:   JP Sugarbroad                                             //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
//                                                                     //
//      Copyright (C) 1999 Intel Corporation  ALL RIGHTS RESERVED      //
//                                                                     //
//             INTEL CORPORATION PROPRIETARY INFORMATION               //
//                                                                     //
//     This software is supplied under the terms of a license agree-   //
//     ment or non-disclosure agreement with Intel Corporation         //
//     and may not be copied nor disclosed except in accordance        //
//     with the terms of that agreement.                               //
//                                                                     //
/////////////////////////////////////////////////////////////////////////

#include "pil_int.h"
#include <dlfcn.h>
#include <string.h>
#include <stdio.h>

#ifdef DEBUG
    // Magic number for debug checking
#   define PIL_DL_MAGIC (0xAF008732)

#   define pil_checklib(lib,r) do { \
        if (!(lib) || (lib)->magic != PIL_DL_MAGIC) { \
            ASSERTMSG(FALSE,"Attempt to use bad libaphore."); \
            return r; \
        } \
    } while (0);
#else // !DEBUG
#   define pil_checklib(lib,r)
#endif

// The structure behind PilLib
typedef struct lib_s
{
#ifdef DEBUG
    uint32_t magic;
#endif
    void *dl;
} lib_t;

///////////////////////////////////////////////////////////////////////
//
// PILOS_LoadLibrary
//
// Loads a dynamic library into memory.
//

PilLib PILOS_LoadLibrary(char *LibraryName)
{
    lib_t *result;
    char name[256];

    if (!LibraryName) return (PilLib) NULL;

    // Allocate and initialize
    
    result = (lib_t *) PIL_AllocateMemory(sizeof(lib_t),0);
    if (result == NULL) return (PilLib) NULL;

#ifdef DEBUG
    result->magic = PIL_DL_MAGIC;
#endif
    strcpy(name,"lib");
    strcat(name,LibraryName);
    strcat(name,".so");
    result->dl = dlopen(name,RTLD_NOW);
    if (result->dl == NULL) {
        PIL_FreeMemory(result);
#ifdef DEBUG
        fprintf(stderr, "\n!! PILOS_LoadLibrary:dlopen(%s) failed - %s !!\n", name, dlerror()); //[jmy]
#endif
        return (PilLib) NULL;
    }
    
    return (PilLib) result;
}

///////////////////////////////////////////////////////////////////////
//
// PILOS_GetProcAddress
//
// Retrieves the address of a symbol from a previously loaded library.
//

PilProc PILOS_GetProcAddress(PilLib Lib, char *ProcName)
{
    lib_t *lib = (lib_t *) Lib;

    pil_checklib(lib,(PilProc) NULL);

    return (PilProc) dlsym(lib->dl,ProcName);
}

///////////////////////////////////////////////////////////////////////
//
// PILOS_FreeLibrary
//
// Frees a dynamic library loaded via PIL_LoadLibrary.
//

PilResult PILOS_FreeLibrary(PilLib Lib)
{
    lib_t *lib = (lib_t *) Lib;

    pil_checklib(lib,PIL_INVALID_HANDLE);

#ifdef DEBUG
    lib->magic = 0;
#endif
    dlclose(lib->dl);
    return PIL_OK;
}
