/////////////////////////////////////////////////////////////////////////
//                                                                     //
// Filename: PILMEM.C                                                  //
// Module:   Platform Isolation Layer Memory Routines                  //
// Author:   JP Sugarbroad                                             //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
//                                                                     //
//      Copyright (C) 1999 Intel Corporation  ALL RIGHTS RESERVED      //
//                                                                     //
//             INTEL CORPORATION PROPRIETARY INFORMATION               //
//                                                                     //
//     This software is supplied under the terms of a license agree-   //
//     ment or non-disclosure agreement with Intel Corporation         //
//     and may not be copied nor disclosed except in accordance        //
//     with the terms of that agreement.                               //
//                                                                     //
/////////////////////////////////////////////////////////////////////////

#include "pil_int.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

////////////////////////////////////////////////////////////////////////
//
// PIL_AllocateMemory
//
// Allocates a block of memory of at least dwSize bytes in size.
//

void * PIL_AllocateMemory(uint32_t dwSize, uint32_t dwFlags)
{
    void * result;

    ASSERT(dwSize >= 0);

    if (dwSize == 0) return NULL;

    result = malloc(dwSize);
    if (result == NULL) return NULL;

    // Zero init if requested
    if (dwFlags & PIL_MEM_ZERO_INIT) memset((char *) result, 0, dwSize);

    return result;
}

////////////////////////////////////////////////////////////////////////
//
// PIL_FreeMemory
//
// Frees memory allocated using PIL_AllocateMemory.
//

void PIL_FreeMemory(void * pMem)
{
    ASSERT(pMem != NULL);

    free(pMem);
}

////////////////////////////////////////////////////////////////////////
//
// PIL_AllocatePages
//
// Allocates dwNumPages of contiguous pages of memory (min 1 page).
//

void * PIL_AllocatePages(uint32_t dwNumPages, uint32_t dwFlags, uint32_t *pdwPhysAddr)
{
    ASSERTMSG(FALSE,"PIL_AllocatePages is unimplemented");
    return NULL;
}

////////////////////////////////////////////////////////////////////////
//
// PIL_FreePages
//
// Frees dwNumPages allocated through PIL_AllocatePages.
//

void PIL_FreePages(void * pMem, uint32_t dwNumPages)
{
    ASSERTMSG(FALSE,"PIL_FreePages is unimplemented");
}
