/////////////////////////////////////////////////////////////////////////
//                                                                     //
// Filename: PILMISC.C                                                 //
// Module:   Platform Isolation Layer Miscellaneous Routines           //
// Author:   JP Sugarbroad                                             //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
//                                                                     //
//      Copyright (C) 1999 Intel Corporation  ALL RIGHTS RESERVED      //
//                                                                     //
//             INTEL CORPORATION PROPRIETARY INFORMATION               //
//                                                                     //
//     This software is supplied under the terms of a license agree-   //
//     ment or non-disclosure agreement with Intel Corporation         //
//     and may not be copied nor disclosed except in accordance        //
//     with the terms of that agreement.                               //
//                                                                     //
/////////////////////////////////////////////////////////////////////////

#include "pil_int.h"
#include <stdarg.h>
#include <stdio.h>
#include <sys/time.h>

///////////////////////////////////////////////////////////////////////
//
// PIL_Abort
//
// Displays a printf-style message on an appropriate output device
// and terminates execution.
//

void PIL_Abort(char *fmt, ...)
{
    va_list ap;

    va_start(ap,fmt);
    vfprintf(stderr,fmt,ap);
    fflush(stderr);
    va_end(ap);

    exit(1);
}

///////////////////////////////////////////////////////////////////////
//
// PIL_GetCurrentTime
//
// Returns a 64-bit time value in milliseconds.  Note that this may not
// be the actual clock time in milliseconds but instead may be the
// number of milliseconds since system start.  It is guaranteed that
// the difference between any two calls of this function is the elapsed
// real time in milliseconds.
//

uint64_t PIL_GetCurrentTime()
{
    struct timeval now;

    gettimeofday(&now,NULL);
    return ((unsigned long) now.tv_sec)*1000+now.tv_usec/1000;
}


///////////////////////////////////////////////////////////////////////
//
// GetLastError
//
// Is being created as an empty stub for the windows call that has been
// used in MSM.

unsigned long GetLastError()
{
    return((unsigned long)NULL);
}



