/////////////////////////////////////////////////////////////////////////
//                                                                     //
// Filename: PIL_VX.H                                                  //
// Module:   Platform Isolation Layer Internal Declaration File        //
// Author:   Venky Gokulrangan                                         //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
//                                                                     //
//      Copyright (C) 1999 Intel Corporation  ALL RIGHTS RESERVED      //
//                                                                     //
//             INTEL CORPORATION PROPRIETARY INFORMATION               //
//                                                                     //
//     This software is supplied under the terms of a license agree-   //
//     ment or non-disclosure agreement with Intel Corporation         //
//     and may not be copied nor disclosed except in accordance        //
//     with the terms of that agreement.                               //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
//                                                                     //
// PURPOSE: Declare fucntions and data structures for Internal Usage   //
//          to implement PIL layer for VxWorks                         //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
//
// HISTORY:
//
/////////////////////////////////////////////////////////////////////////

#ifndef __PIL_VX_INCLUDED__
#define __PIL_VX_INCLUDED__

// VxWorks specific header files
#include "vxWorks.h"
#include "semLib.h"
#include "taskLib.h"
#include "wdLib.h"

#define inline

typedef char CHAR;
typedef CHAR* PCHAR;
typedef char BYTE;
typedef BYTE* PBYTE;

typedef unsigned short WORD;
typedef WORD* PWORD;

typedef unsigned long long uint64_t;
typedef unsigned long DWORD;
typedef DWORD* PDWORD;

typedef void VOID;
typedef void* PVOID;

#include "pil.h"
#include "pildebug.h"
#include "llist.h"

#include "cqueue.h"

#define PIL_BYTES_PER_PAGE 4096

//
// undefine the Binary Semaphore create and use our own here
//

#ifdef PIL_CreateBinarySemaphore
#undef PIL_CreateBinarySemaphore
#endif

// Our own binary interface VxWorks
// REASON:
//  Event Implementation in VxWorks requires that the initial count
//  be 1, but the number of people waiting on the event can be any
//  arbitrary number. Every time when PIL_ReleaseSemaphore works
//  if will check the current number + 1 against the max and it would
//  not release, but instead flag it as an error..
//

PilSemaphore
PIL_CreateBinarySemaphore (
               DWORD dwInitialCount,
               DWORD dwMaximumCount ) ;


//////////////////////////////////////////////////////////////////////////////
//
//  Semaphore  Structures
//
//////////////////////////////////////////////////////////////////////////////

typedef struct _PilSemaphore_t
{
#ifdef RESTRACK
   LIST_NODE Link ;
#endif

   SEM_ID  Semid ;
   DWORD   dwMax ;
   DWORD   dwCurrCount ; // VxWorks specific count

} PilSemaphore_t ;

//////////////////////////////////////////////////////////////////////////////
//
//  Thread Structures
//
//////////////////////////////////////////////////////////////////////////////


typedef struct _PilThread_t
{
#ifdef RESTRACK
   LIST_NODE Link ;
#endif

  PilThreadRoutine StartAddress ; // Address of the 
  PVOID            ThreadParam ;  // User supplied thread input parameter
  DWORD            dwOptions ;    // Thread Options
  DWORD            Status ;       // We need to store the statu
  PilEvent         ExitEvent ;    // Set event when thread exits

  // Just in case other threads need this info..
  int              Tid;  // Id of the thread
} PilThread_t ;

//////////////////////////////////////////////////////////////////////////////
//
//  Event Structures
//
//////////////////////////////////////////////////////////////////////////////


typedef struct _PilEvent_t
{

#ifdef RESTRACK
   LIST_NODE Link ;
#endif

   BOOL      bManualReset ;
   BOOL      bSignalled ;

   int       NumWaiters ;
   BOOL      Flag ; // Status of the event

   // events are built on top of semaphores
   PilSemaphore AccessSem ;
   PilSemaphore EventSem ; // Event Data synch

} PilEvent_t ;

//////////////////////////////////////////////////////////////////////////////
//
//  Timer  Structure 
//
// Implementation Notes:
// We are going to maintain the time as it comes in the function call. We 
// assume the a clock resolution to be 10 msec..
// To support 1 msec level resolution, we need to change the system clock
// rate into 1000Hz
//////////////////////////////////////////////////////////////////////////////

typedef struct _PilTimer_t
{
#ifdef RESTRACK
   LIST_NODE Link;
#endif
   int               Recur ;       // recurring timer - could have used
   WDOG_ID           wdId ;        // Timer Id from VxWorks -- wdId
   DWORD             dwInterval ;  // in millisec -- firing interval

   PilTimerRoutine   ProcedureAddress ; // Function to trigger
   PVOID             Data;              // Data to be passed to the routine   

} PilTimer_t ;


int getTicks ( DWORD dwInterval ) ; // get in ticks of the system


//////////////////////////////////////////////////////////////////////////////
//
//  Mailbox Structures
//
//////////////////////////////////////////////////////////////////////////////

// This contains the functions for mailbox..
// The functions are intended to be used on a per process basis
// using the static library mechanism

//typedef DWORD CQHandle ;

typedef struct _MboxDesc_t
{

  // Mailbox Handle
   CQHandle       Handle ;    // Handle to the Queue
   DWORD          dwQLen;     // Maximum number of messages in the queue
   DWORD          dwMailSize; // Size of individual Message

   PilSemaphore   hReadSem ;  // Reader Semaphore
   PilSemaphore   hWriteSem ; // Writer Semaphore

   PilSemaphore   pLock;      // Semaphore protects the queue

} MboxDesc;

#if 0

typedef enum
{
   QUEUE_OK     = 0x00,
   QUEUE_FULL   = 0x01,
   QUEUE_EMPTY  = 0x02,
   QUEUE_ERROR  = 0x03,
   QUEUE_NOMEM  = 0x10
} QueueReturn ;

// Self contained object
typedef struct PilCQ_t
{
   DWORD   dwHead ;
   DWORD   dwTail ;
   DWORD   dwSize ;
   DWORD   dwCount ;
   PDWORD  pQ ; // Array of Queue Objects.. 

   PilSemaphore AccessSem ; // to access the variables and the Queue itself

} PilCQ ;


//////////////////////////////////////////////////////////////////////////////
//
// Function:   CreateCQ
// Parameters: Number of Units in the Queue
// Purpose:    Creates a circular queue of the size as specified.
// Returns:    Handle of the created circular queue
//             NULL, if unable to create the circular queue
//
// Notes:      In any case, if an error occurs, all the resources allocated
//             until the previous point are removed
//

CQHandle
CreateCQ ( 
    DWORD dwQLen,
    DWORD dwMailSize ) ;


//////////////////////////////////////////////////////////////////////////////
//
// Function:   Enqueue
// Parameters: 1. Handle to the Circular Queue
//             2. The address to copy the mail message from
// Purpose:    If a mail slot is available, this function queues the message
//             into that slot
//
// Returns:    QUEUE_ERROR if unable to perform the operation
//             QUEUE_FULL  if no space is available
//             QUEUE_OK    if succesful
//

QueueReturn
Enqueue ( 
   CQHandle hCQ,
   PVOID Object ) ; // Object itself is an address

//////////////////////////////////////////////////////////////////////////////
//
// Function:   DeQueue
// Parameters: 1. Handle to the Circular Queue
//             2. The address to copy the mail message to
// Purpose:    If message is available in a mail slot, this function copies
//             the message to the user space.
//
// Returns:    QUEUE_ERROR if unable to perform the operation
//             QUEUE_FULL  if no element is available to copy from to user
//             QUEUE_OK    if succesful
//
QueueReturn
DeQueue (
         CQHandle hCQ,
         PVOID pObject) ;

#endif

#ifdef __cplusplus
}
#endif

#endif
