/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

#include "pil_vx.h"
#include "pil_int.h"

#ifdef DEBUG
static pillist cslist = NULL;
pillist _PIL_GetCritcalSectionList() { return cslist; }
#endif

VOID
_PIL_InitializeCriticalSection(
    PilCriticalSection* pCriticalSection 
  , const char* name
    )  // Pointer to critical section
{   
   PilCS* p = (PilCS *)pCriticalSection ;

   p->Sem = semMCreate (SEM_Q_PRIORITY) ;
   ASSERT(p->Sem) ;
#ifdef DEBUG
    p->info = (csinfo_t*)PIL_AllocateMemory(sizeof(csinfo_t),0);
    p->info->owner = NULL;
    p->info->blocking = NULL;

    if (!name)
        name = "unnamed";

    p->info->name = (char*)PIL_AllocateMemory(strlen(name)+1,0);
    strcpy(p->info->name,name);

    _PIL_AddToList(&cslist,p->info);
#endif
}


VOID
PIL_EnterCriticalSection(
    PilCriticalSection* pCriticalSection    // Pointer to critical section
    )
{
#ifdef DEBUG
   PilThread_t *t;
#endif
   PilCS* p = (PilCS *)pCriticalSection ;

#ifdef DEBUG
    t = (PilThread_t*)PIL_GetCurrentThread();
    _PIL_AddToList(&(p->info->blocking),t);
#endif

   semTake (p->Sem, WAIT_FOREVER);

#ifdef DEBUG
   _PIL_RemoveFromList(&(p->info->blocking),t);
   p->info->owner = t;
#endif
}


VOID
PIL_LeaveCriticalSection(
    PilCriticalSection* pCriticalSection    // Pointer to critical section
    )
{
   PilCS* p = (PilCS *)pCriticalSection ;

#ifdef DEBUG
    p->info->owner = NULL;
#endif

   semGive (p->Sem) ;
}


BOOL
PIL_TryEnterCriticalSection(
    PilCriticalSection* pCriticalSection    // Pointer to critical section
    )
{
  // try for 1 clock tick to see if you can get it.. if not leave
  // immediately

   PilCS* p = (PilCS *)pCriticalSection ;
   return semTake (p->Sem, 1) == OK ;
}

VOID
PIL_DestroyCriticalSection(
    PilCriticalSection* pCriticalSection )  // Pointer to critical section
{   
#ifdef DEBUG
    csinfo_t *info;
#endif

   PilCS* p = (PilCS *)pCriticalSection ;

#ifdef DEBUG
    info = p->info;
    _PIL_RemoveFromList(&cslist,info);

    PIL_FreeMemory(info);
#endif

   semDelete(p->Sem) ;
}


