/////////////////////////////////////////////////////////////////////////
//                                                                     //
// Filename: PILINIT.C                                                 //
// Module:   Platform Isolation Layer Initialization/De-Initialization //
// Author:   JP Sugarbroad                                             //
//                                                                     //
/////////////////////////////////////////////////////////////////////////
//                                                                     //
//      Copyright (C) 1999 Intel Corporation  ALL RIGHTS RESERVED      //
//                                                                     //
//             INTEL CORPORATION PROPRIETARY INFORMATION               //
//                                                                     //
//     This software is supplied under the terms of a license agree-   //
//     ment or non-disclosure agreement with Intel Corporation         //
//     and may not be copied nor disclosed except in accordance        //
//     with the terms of that agreement.                               //
//                                                                     //
/////////////////////////////////////////////////////////////////////////

#include "pil_int.h"

int nPilRefCount = 0;

///////////////////////////////////////////////////////////////////////
//
// PIL_Init
//
// Initializes the PIL library. All global variables used by PIL 
// implementations need to be initialized here to account for
// embedded implementations where reset may not guarantee that
// globals are re-initialized. 
// This function should be called before any other PIL library
// call is made by the application. 
// 

extern PILLIST_MUTEX_TYPE  pillistmutex;

PilResult PIL_Init()
{
    PilResult r = PIL_OK;

    if (!nPilRefCount)
    {
        pillistmutex = semBCreate (SEM_Q_PRIORITY, SEM_FULL) ;
        nPilRefCount = 1;
    }
    else
    {
        // Already initialized, just increment the reference count
        nPilRefCount++;
        r = PIL_OK;
    }
    
    return r;
}

///////////////////////////////////////////////////////////////////////
//
// PIL_DeInit
//
// De-initializes the PIL library. Any allocations done in PIL_Init
// can be freed at this time. All PIL objects must be freed before
// this call is made. No other PIL library call can be made after this
// call is made.
//

void PIL_DeInit()
{
    if (nPilRefCount)
    {
        nPilRefCount--;
        if (nPilRefCount == 0)
        {
        }
    }
}
