/*****************************************************************************
 *                  INTEL CORPORATION PROPRIETARY IF_INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 *$Source: /cvs/constellations/ixa_3.0/src/cp_pdk/cppui/build/src/cppui_event.c,v $
 *$Revision: 1.5 $
 *$Date: 2003/11/17 16:57:56 $
 * Description:
 *     This file contains the event handling of CPPUI module
 *
 *****************************************************************************/

#include "cppui_common.h"

static NPF_userContext_t    eventContext = 0;
static FPPI_CBHANDLE    FPPfeBindEventHandle = 0;
static FPPI_CBHANDLE    FPPfeUnBindEventHandle = 0;
static FPPI_CBHANDLE    FPPfeCapabilityEventHandle = 0;
static FPPI_CBHANDLE    FPPfePortCapabilityEventHandle = 0;
static FPPI_CBHANDLE    FPPPortLinkUpEventHandle = 0;
static FPPI_CBHANDLE    portLinkDownEventHandle = 0;
static FPPI_CBHANDLE    directlyConnectedHostHandle = 0;

void CppuiEventDeregister()
{
    FPPAPI_deregister_callback(FPPfeBindEventHandle);
    FPPAPI_deregister_callback(FPPfeUnBindEventHandle);
    FPPAPI_deregister_callback(FPPfeCapabilityEventHandle);
    FPPAPI_deregister_callback(FPPfePortCapabilityEventHandle);
    FPPAPI_deregister_callback(FPPPortLinkUpEventHandle);
    FPPAPI_deregister_callback(portLinkDownEventHandle);
    FPPAPI_deregister_callback(directlyConnectedHostHandle);
}

void FPPfeBindEvent(FPPI_FEID feid, 
			FPPI_CORRELATOR corr, 
			FPPI_CONTEXT context, 
			void *resp)
{
    int id = 0;
    id = (int)feid;
    cppui_send_FE_BIND_event(id);
}

void FPPfeUnBindEvent(FPPI_FEID feid, 
			FPPI_CORRELATOR corr, 
			FPPI_CONTEXT context, 
			void *resp)
{
    int id = 0;
    id = (int)feid;
    cppui_send_FE_UNBIND_event(id);
}

void FPPfeCapabilityEvent(FPPI_FEID feid, 
				FPPI_CORRELATOR corr, 
				FPPI_CONTEXT context, 
				void *resp)
{
    int          num_ports = 0;
    int          i = 0;
    int          portId = 0;
    FPPI_FE_Caps*  fe_caps;
    FPPI_Port_Caps*          port_caps;
    fe_caps = (FPPI_FE_Caps*) resp;
    port_caps = fe_caps->ports_caps.port_caps_list;
    num_ports = fe_caps->ports_caps.port_count;
    for(i=0; i<num_ports; i++)
    {
 	portId = port_caps->portid;
        cppui_send_FE_PORT_event(feid, portId);
        port_caps += 1;
    }
}

void FPPfePortCapabilityEvent(FPPI_FEID feid, 
				FPPI_CORRELATOR corr, 
				FPPI_CONTEXT context, 
				void *resp)
{
    //cppui_send_PORT_CAPABILITY_event(id);
}

void FPPportLinkUpEvent(FPPI_FEID feid, 
				FPPI_CORRELATOR corr, 
				FPPI_CONTEXT context, 
				void *resp)
{
    int id = 0;
    FPPI_PortID      portid;
    id = (int)feid;
    portid = *((int *)resp);
    cppui_send_PORT_LINK_UP_event(feid, portid);
}

void FPPportLinkDownEvent(FPPI_FEID feid, 
				FPPI_CORRELATOR corr, 
				FPPI_CONTEXT context, 
				void *resp)
{
    int id = 0;
    FPPI_PortID      portid;
    id = (int)feid;
    portid = *((int *)resp);
    cppui_send_PORT_LINK_DOWN_event(feid, portid);
}

void RegisterFPPfeBindEvent()
{
    FPPAPI_register_callback(&FPPfeBindEvent, 
				&FPPfeBindEventHandle, 
				FPPI_FEBind_Event,  
				(NPF_USERCONTEXT)eventContext);
}

void RegisterFPPfeUnBindEvent()
{
    FPPAPI_register_callback(&FPPfeUnBindEvent, 
				&FPPfeUnBindEventHandle, 
				FPPI_FEUnBind_Event,  
				(NPF_USERCONTEXT)eventContext);
}

void RegisterFPPfeCapabilityEvent()
{
    FPPAPI_register_callback(&FPPfeCapabilityEvent, 
				&FPPfeCapabilityEventHandle, 
				FPPI_FECapability_Event,  (NPF_USERCONTEXT)eventContext);
}

void RegisterFPPfePortCapabilityEvent()
{
    FPPAPI_register_callback(&FPPfePortCapabilityEvent, 
				&FPPfePortCapabilityEventHandle, 
				FPPI_FEPortCap_Event,  
				(NPF_USERCONTEXT)eventContext);
}

void RegisterFPPportLinkUpEvent()
{
    FPPAPI_register_callback(&FPPportLinkUpEvent, 
				&FPPPortLinkUpEventHandle, 
				FPPI_LinkUp_Event,  
				(NPF_USERCONTEXT)eventContext);
}

void RegisterFPPportLinkDownEvent()
{
    FPPAPI_register_callback(&FPPportLinkDownEvent, 
				&portLinkDownEventHandle, 
				FPPI_LinkDown_Event,  
				(NPF_USERCONTEXT)eventContext);
}

void CppuiEventRegister()
{
    RegisterFPPfeBindEvent();
    RegisterFPPfeUnBindEvent();
    RegisterFPPfeCapabilityEvent();
    RegisterFPPfePortCapabilityEvent();
    RegisterFPPportLinkUpEvent();
    RegisterFPPportLinkDownEvent();
    /*
    RegisterFePdkEvent();
    RegisterIfPdkEvent();
    RegisterPortPdkEvent();
    */
}
