/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2003 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

         
        #include "cppui_common.h"
         
        void cppuiIPv4Callback(NPF_userContext_t userContext, NPF_correlator_t correlator, NPF_IPv4UC_CallbackData_t cbData)
        {
        
    static NPF_IPv4UC_CallbackData_t  data;
    uint32_t   sizeResp=0;
    CPPUI_LOCK();
    data.numResp= cbData.numResp;
    if(data.numResp == 0)
      data.numResp = 1; /* This is to make sure that we have resp */
    data.type  = cbData.type;
    data.allOK = cbData.allOK;
    
    sizeResp =  (data.numResp)*sizeof(NPF_IPv4UC_AsyncResponse_t);
    data.resp = (NPF_IPv4UC_AsyncResponse_t*)PIL_AllocateMemory(sizeResp, PIL_MEM_ZERO_INIT);
    assert(data.resp);
    memcpy(data.resp, cbData.resp,sizeResp);
    cppuiCallback(userContext, correlator, &data); 
    PIL_FreeMemory(data.resp);
    CPPUI_UNLOCK();

        }
        void getPrefixEntry(char *input, NPF_IPv4UC_Prefix_t *prefixEntry, uint32_t param )
        {
        
  static NPF_IPv4UC_Prefix_t pfxEntry;
  switch(param)
    {
	case 1:	/*Address */
	{
	     getipv4address(input,&pfxEntry.IPv4Addr);
	}
	break;
	case 2:	/*Prefix Length */
	{
	    pfxEntry.IPv4NetPlen = (NPF_uint8_t)atol(input);
	}
	break;
    }
    memcpy(prefixEntry, &pfxEntry, sizeof(NPF_IPv4UC_Prefix_t));

        }
        void getNextHopId(char *input, uint32_t *nhId, uint32_t param )
        {
        
  static NPF_uint32_t  id;
  switch(param)
    {
	case 1:	/*NH-ID*/
	{
	    id = (NPF_uint32_t)atol(input);
	}
	break;
     }
  *nhId = id;

        }
        void getNextHopParams(char *input, NPF_IPv4UC_NextHopArray_t *nha, int param)
        {
        
    static NPF_IPv4UC_NextHop_t nh;
    switch(param)
    {
	case 1:	/*NextHopType*/
	{
	    nh.type = (NPF_IPv4UC_NextHopType_t)atol(input);
	}
	break;
	case 2: /*NextHopWeight*/
	{
	    nh.weight = (NPF_uint16_t)atol(input);
	}
	break;
	case 3: /*NextHop_IP_Address*/
	{
	    getipv4address(input, &nh.nextHopIP);
            nh.type = 1;
	}
	break;
	case 4: /*Media_Address_Type*/
	{
	    nh.mediaAddress.type = (NPF_MediaType_t)atol(input);
	}
	break;
	case 5: /*Media_Address*/
	{
	    if(nh.mediaAddress.type == NPF_MAC_ADDRESS)
		    getmacaddress(input, (uint8_t*)&nh.mediaAddress.u.MAC_Address);
	    if(nh.mediaAddress.type == NPF_ATM_VC)
		    getatmaddress(input, &nh.mediaAddress.u.ATM_Vc);
	}
	break;
    }
	nha->nextHopCount = 1;
        /* Assigning the same memory that is created here */
        nha->nextHopArray = &nh;

        }
        void getAddResParams(char *input, NPF_IPv4UC_AddResEntry_t *addResEntry, int param)
        {
        
    switch(param)
    {
	   case 1: /*IP_Address of Interface*/
	   {
	       getipv4address(input, &addResEntry->IP_Address);
	   }
	   break;
	   case 2: /*Media_Address_Type*/
	   {
	      addResEntry->mediaAddress.type = (NPF_MediaType_t)atol(input);
	   }
	   break;
	   case 3: /*Media_Address*/
	   {
	      if(addResEntry->mediaAddress.type == NPF_MAC_ADDRESS)
		      getmacaddress(input, (uint8_t*)&addResEntry->mediaAddress.u.MAC_Address);
	      if(addResEntry->mediaAddress.type == NPF_ATM_VC)
		      getatmaddress(input, &addResEntry->mediaAddress.u.ATM_Vc);
	   }
	   break;
    }

        }
        int formatModeResp(int cmd,uint32_t mode, char *resp)
        {
        
    int pos = 0;
   if(cmd == 158) /* Supported */
   {
      NPF_IPv4UC_SupportedMode_t supMode = mode;
      switch(supMode)
      {
        case NPF_IPV4UC_UNIFIED_ONLY:
         pos += sprintf(&resp[pos], "%s", "UNIFIED ONLY");
         break;
         case NPF_IPV4UC_BOTH_SUPPORTED:
         pos += sprintf(&resp[pos], "%s", "UNIFIED AND DISCRETE");
         break;
         default:
         return 1;
      }
   }
   else if (cmd == 159) /* Preferrence */
   {
      NPF_IPv4UC_PreferredMode_t prefMode = mode;
      switch(prefMode)
      {
        case NPF_IPV4UC_DISCRETE_PREFERRED:
         pos += sprintf(&resp[pos], "%s", "DISCRETE PREFERRED");
         break;
         case NPF_IPV4UC_UNIFIED_PREFERRED:
         pos += sprintf(&resp[pos], "%s", "UNIFIED PREFERRED");
         break;
         case NPF_IPV4UC_NO_PREFERENCE:
         pos += sprintf(&resp[pos], "%s", "NO PREFERENCE");
         break;
         default:
         return 1;
      }
   }
   return 0;


        }
        int formatIPv4Response(int cmd, NPF_IPv4UC_CallbackData_t *data,  char *resp)
        {
        
    NPF_IPv4UC_AsyncResponse_t *async;
    int pos = 0;
    if(data == NULL || data->resp == NULL)
    {
         pos += sprintf(&resp[pos], "%s", "NULL RESPONSE");
         return(1);
    }

    if(data->allOK == FALSE )
    {
         pos += sprintf(&resp[pos], "\t Response Received. allOK is FALSE \n");
         return(1);
    }
    pos += sprintf(&resp[pos], "\t Response Received. allOK is TRUE \n");
    async = data->resp;
    switch(cmd)
    {
	case 160:	/*Create_Prefix_Table_Handle*/
	{
	    if(data->type == NPF_IPV4UC_PREFIX_TABLE_HANDLE_CREATE && async->returnCode == 0)
	    {
                storeHandle(CPPUI_IPV4_PREFIX_HANDLE, async->u.prefixTableHandles.intHandle);
                storeHandle(CPPUI_IPV4_FWD_HANDLE, async->u.prefixTableHandles.extHandle);
		pos += sprintf(&resp[pos], "Prefix Internal Table Handle : %d\n", async->u.prefixTableHandles.intHandle);
		pos += sprintf(&resp[pos], "Forwarding Table Handle : %d\n", async->u.prefixTableHandles.extHandle);
		return(0);
	    }
	    else
	    {
		pos += sprintf(&resp[pos], "Prefix Table Create : Unknown Response\n");
		return(1);
	    }
	}
	case 161:	/*Create_NextHop_Table_Handle*/
	{
	    if(data->type == NPF_IPV4UC_NEXT_HOP_TABLE_HANDLE_CREATE && async->returnCode == 0)
	    {
                storeHandle(CPPUI_IPV4_NEXTHOP_HANDLE, async->u.nextHopTableHandle);
		pos += sprintf(&resp[pos], "NextHop Table Handle : %d\n", async->u.nextHopTableHandle);
		return(0);
	    }
	    else
	    {
		pos += sprintf(&resp[pos], "NextHop Table Create : Unknown Response\n");
		return(1);
	    }
	}
	case 162:	/*Create_AddRes_Table_Handle*/
	{
	    if(data->type == NPF_IPV4UC_ADDRESS_RES_TABLE_HANDLE_CREATE && async->returnCode == 0)
	    {
                storeHandle(CPPUI_IPV4_ADDRESS_HANDLE, async->u.addResTableHandle);
		pos += sprintf(&resp[pos], "Address Resolution Table Handle : %d\n", async->u.addResTableHandle);
		return(0);
	    }
	    else
	    {
		pos += sprintf(&resp[pos], "Address Resolution Table Create : Unknown Response\n");
		return(1);
	    }
	}
	break;
		case 163:	/*Delete_Prefix_Table_Handle*/
	{
	    if(data->type == NPF_IPV4UC_PREFIX_TABLE_HANDLE_DELETE && async->returnCode == 0)
	    {
		pos += sprintf(&resp[pos], "Prefix Table Deleted\n");
		return(0);
	    }
	    else
	    {
		pos += sprintf(&resp[pos], "Prefix Table Delete : Unknown Response\n");
		return(1);
	    }
	}
	case 164:	/*Delete_Nexthop_Table_Handle*/
	{
	    if(data->type == NPF_IPV4UC_NEXT_HOP_TABLE_HANDLE_DELETE && async->returnCode == 0)
	    {
		pos += sprintf(&resp[pos], "NextHop Table Deleted\n");
		return(0);
	    }
	    else
	    {
		pos += sprintf(&resp[pos], "NextHop Table Delete : Unknown Response\n");
		return(1);
	    }
	}
	case 165:	/*Delete_Address_Table_Handle*/
	{
	    if(data->type == NPF_IPV4UC_ADDRESS_RES_TABLE_HANDLE_DELETE && async->returnCode == 0)
	    {
		pos += sprintf(&resp[pos], "Address Resolution Table Deleted\n");
		return(0);
	    }
	    else
	    {
		pos += sprintf(&resp[pos], "Address Resolution Table Delete : Unknown Response\n");
		return(1);
	    }
	}

	case 166:	/*Bind_Prefix_NextHop_Table*/
	{
	    if(data->type == NPF_IPV4UC_PREFIX_NEXT_HOP_TABLE_BIND && async->returnCode == 0)
	    {
		pos += sprintf(&resp[pos], "Table Bind successful \n");
		return(0);
	    }
	    else
	    {
		pos += sprintf(&resp[pos], "Table Bind failure : Unknown Response\n");
		return(1);
	    }
	}

	case 167:	/*Add_Prefix_Table_Entry*/
	{
	    if(data->type == NPF_IPV4UC_PREFIX_ENTRY_ADD && async->returnCode == 0)
	    {
		pos += sprintf(&resp[pos], "Prefix Added \n");
		return(0);
	    }
	    else
	    {
		pos += sprintf(&resp[pos], "Prefix Add Failed \n");
		return(1);
	    }
	}
	break;
	case 168:	/*Add_NextHop_Table_Entry*/
	{
	    if(data->type == NPF_IPV4UC_NEXT_HOP_ENTRY_ADD && async->returnCode == 0)
	    {
		pos += sprintf(&resp[pos], "NextHop Added \n");
		return(0);
	    }
	    else
	    {
		pos += sprintf(&resp[pos], "NextHop Add Failed \n");
		return(1);
	    }
	}
	break;
	case 169:	/*Add_AddRes_Table_Entry*/
	{
	    if(data->type == NPF_IPV4UC_ADDRESS_RES_ENTRY_ADD && async->returnCode == 0)
	    {
		pos += sprintf(&resp[pos], "Address Res Entry Added \n");
		return(0);
	    }
	    else
	    {
		pos += sprintf(&resp[pos], "Address Res Entry Addition Failed \n");
		return(1);
	    }
	}
	break;
        case 170:	/*Delete_Prefix_Table_Entry*/
        {
             if(data->type == NPF_IPV4UC_PREFIX_ENTRY_DELETE && async->returnCode == 0)
	    {
                pos += sprintf(&resp[pos], "Prefix Entry Deleted \n");
	        return(0);
	    }
	    else
	    {
	       pos += sprintf(&resp[pos], "Prefix Entry Deletion Failed \n");
	       return(1);
            }
        }
        break;
        case 171:	/*Delete_NextHop_Table_Entry*/
        {
             if(data->type == NPF_IPV4UC_NEXT_HOP_ENTRY_DELETE && async->returnCode == 0)
	    {
                pos += sprintf(&resp[pos], "Next-Hop Entry Deleted \n");
	        return(0);
	    }
	    else
	    {
	       pos += sprintf(&resp[pos], "Next-Hop Entry Deletion Failed \n");
	       return(1);
            }
        }
        break;
        case 172:	/*Delete_AddRes_Table_Entry*/
        {
             if(data->type == NPF_IPV4UC_ADDRESS_RES_ENTRY_DELETE && async->returnCode == 0)
	    {
                pos += sprintf(&resp[pos], "Address Res Entry Deleted \n");
	        return(0);
	    }
	    else
	    {
	       pos += sprintf(&resp[pos], "Address Res Entry Deletion Failed \n");
	       return(1);
            }
        }
        break;
        case 173:	/*Flush_Prefix_Table*/
        {
             if(data->type == NPF_IPV4UC_PREFIX_TABLE_FLUSH && async->returnCode == 0)
	    {
                pos += sprintf(&resp[pos], "Prefix Table Flushed \n");
	        return(0);
	    }
	    else
	    {
	       pos += sprintf(&resp[pos], "Prefix Table Flush Failed \n");
	       return(1);
            }
        }
        break;
        case 174:	/*Flush_NextHop_Table*/
        {
             if(data->type == NPF_IPV4UC_NEXT_HOP_TABLE_FLUSH && async->returnCode == 0)
	    {
                pos += sprintf(&resp[pos], "NextHop Table Flushed \n");
	        return(0);
	    }
	    else
	    {
	       pos += sprintf(&resp[pos], "NextHop Table Flush Failed \n");
	       return(1);
            }
        }
        break;
        case 175:	/*Flush_AddRes_Table*/
        {
             if(data->type == NPF_IPV4UC_ADDRESS_RES_TABLE_FLUSH && async->returnCode == 0)
	    {
                pos += sprintf(&resp[pos], "AddRes Table Flushed \n");
	        return(0);
	    }
	    else
	    {
	       pos += sprintf(&resp[pos], "AddRes Table Flush Failed \n");
	       return(1);
            }
        }
        break;
	case 176:	/*Query_Prefix_Table_Entry*/
	{
	    if(data->type == NPF_IPV4UC_PREFIX_ENTRY_QUERY && async->returnCode == 0)
	    {
		    pos += sprintf(&resp[pos], "Prefix Query Successful \n");
		    pos += sprintf(&resp[pos], "\t NhId :%d  \n",
                       async->u.prefixQueryResult.nextHopIdentifier);
		    return(0);
	    }
	    else
	    {
		    pos += sprintf(&resp[pos], "Prefix Query Failed \n");
		    return(1);
	    }
	}
	break;
	case 177:	/*Query_NextHop_Table_Entry*/
	{
       uint32_t i, ipAddr =0;
	    if(data->type == NPF_IPV4UC_NEXT_HOP_ENTRY_QUERY && async->returnCode == 0)
	    {
		    pos += sprintf(&resp[pos], "NextHop Query Successful \n");
		    pos += sprintf(&resp[pos], "\t NextHop Entries received : %d \n",
                 async->u.nextHopQueryResult.nextHopArray.nextHopCount);

          for(i=0; i < async->u.nextHopQueryResult.nextHopArray.nextHopCount;
          i++)
          {
              char* ipString = NULL;
             ipAddr = async->u.nextHopQueryResult.nextHopArray.nextHopArray[i].nextHopIP; 
             ipString = getIPStringFromInt(ipAddr);
		       pos += sprintf(&resp[pos],
                 "\t NHId:%d \n \t NextHop IP Address : %s \n",
                 async->u.nextHopQueryResult.nextHopIdentifier,
                 ipString);
          }
		    return(0);
	    }
	    else
	    {
		    pos += sprintf(&resp[pos], "NextHop Query Failed \n");
		    return(1);
	    }
	}
	break;
	case 178:	/*Query_AddRes_Table_Entry*/
	{
	    if(data->type == NPF_IPV4UC_ADDRESS_RES_ENTRY_QUERY && async->returnCode == 0)
	    {
		   pos += sprintf(&resp[pos], "Address Res Entry Query Successful \n");
		   return(0);
	    }
	    else
	    {
		    pos += sprintf(&resp[pos], "Address Res Entry Query Failed \n");
		    return(1);
	    }
	}
	break;



        /*** UNIFIED TEST CASES START FROM 180 */

        case 180:	/*Create_Fib_Table_Handle*/
	     {
	       if(data->type == NPF_IPV4UC_FIB_TABLE_HANDLE_CREATE && async->returnCode == 0)
	       {
             storeHandle(CPPUI_IPV4_FIB_HANDLE, async->u.fibTableHandles.intHandle);
             storeHandle(CPPUI_IPV4_FWD_HANDLE, async->u.fibTableHandles.extHandle);
		       pos += sprintf(&resp[pos], "FIB Internal Table Handle : %d\n", async->u.fibTableHandles.intHandle);
		       pos += sprintf(&resp[pos], "FIB External Table Handle : %d\n", async->u.fibTableHandles.extHandle);
		       return(0);
	       }
	       else
	       {
		       pos += sprintf(&resp[pos], "FIB Table Create : Unknown Response\n");
		       return(1);
	       }
	     }
        break;
        case 181:	/*Delete_FIB_Table_Handle*/
	     {
	        if(data->type == NPF_IPV4UC_FIB_TABLE_HANDLE_DELETE && async->returnCode == 0)
	        {
		        pos += sprintf(&resp[pos], "FIB Table Deleted\n");
		        return(0);
	        }
	        else
	        {
		        pos += sprintf(&resp[pos], "FIB Table Delete : Unknown Response\n");
		        return(1);
	         }
	      }
         break;
         case 182:	/*Add_Fib_Table_Entry*/
         {
	         if(data->type == NPF_IPV4UC_FIB_ENTRY_ADD && async->returnCode == 0)
	         {
               pos += sprintf(&resp[pos], "FIB Entry Added \n");
	            return(0);
	          }
	          else
	          {
	             pos += sprintf(&resp[pos], "FIB Entry Addition Failed \n");
	             return(1);
             }
          }
          break;
          case 183:	/*Delete_Fib_Table_Entry*/
          {
             if(data->type == NPF_IPV4UC_FIB_ENTRY_DELETE && async->returnCode == 0)
      	    {
                pos += sprintf(&resp[pos], "FIB Entry Deleted \n");
	             return(0);
	          }
	          else
	          {
	            pos += sprintf(&resp[pos], "FIB Entry Deletion Failed \n");
	            return(1);
              }
          }
          break;

          case 184:	/*Flush_Fib_Table*/
          {
             if(data->type == NPF_IPV4UC_FIB_TABLE_FLUSH && async->returnCode == 0)
	          {
                pos += sprintf(&resp[pos], "FIB Table Flushed \n");
	             return(0);
	          }
	          else
	          {
	             pos += sprintf(&resp[pos], "FIB Table Flush Failed \n");
	             return(1);
             }
          }
          break;
          case 185:	/*Query_Fib_Table_Entry*/
          {
             uint32_t i, ipAddr =0;
             if(data->type == NPF_IPV4UC_FIB_ENTRY_QUERY && async->returnCode == 0)
             {
                pos += sprintf(&resp[pos], "FIB Query Successful \n");
                pos += sprintf(&resp[pos], "\t FIB Entries received : %d \n",
                      async->u.fibQueryResult.nextHopArray.nextHopCount);

                for(i=0; i < async->u.fibQueryResult.nextHopArray.nextHopCount;
                      i++)
                {
                   char* ipString = NULL;
                   ipAddr = async->u.fibQueryResult.nextHopArray.nextHopArray[i].nextHopIP; 
                   ipString = getIPStringFromInt(ipAddr);
                   pos += sprintf(&resp[pos],
                         "\t NextHop IP Address : %s \n",
                         ipString);
                }
                return(0);
             }
             else
             {
                pos += sprintf(&resp[pos], "NextHop Query Failed \n");
                return(1);
             }

          }
          break;

    } /* end of switch */
    PIL_FreeMemory(data->resp);
    return(0);

        }
         
         
        int NPF_IPv4UC_PrefixEntryAdd_SelfTest(   NPF_IPv4UC_PrefixTableHandle_t *prefixTableHandle, NPF_uint32_t *no, NPF_IPv4UC_Prefix_t *prefixEntry, NPF_uint32_t *nhId, void *corr)
        {
        
            NPF_uint32_t *nhIdArray = NULL;
            NPF_IPv4UC_Prefix_t *prefixEntryArray = NULL;
            uint32_t i;
            uint32_t num = *no;
            prefixEntryArray = (NPF_IPv4UC_Prefix_t*) PIL_AllocateMemory(sizeof(NPF_IPv4UC_Prefix_t)*num, PIL_MEM_ZERO_INIT);
            nhIdArray = (NPF_uint32_t*) PIL_AllocateMemory(sizeof(NPF_uint32_t)*num, PIL_MEM_ZERO_INIT);
            if(prefixEntryArray == NULL || nhIdArray == NULL)
               return 1;
            /* Fill the values here */ 
            for(i=0; i < num; i++)
            {  
               nhIdArray[i] = *nhId + i;
               prefixEntryArray[i].IPv4Addr = prefixEntry->IPv4Addr + i;
               prefixEntryArray[i].IPv4NetPlen = prefixEntry->IPv4NetPlen;
            }
             
            NPF_IPv4UC_PrefixEntryAdd(IPV4cbh,(NPF_correlator_t)corr,NPF_REPORT_ALL,*prefixTableHandle, num , prefixEntryArray,nhIdArray);
            /* Free the Memory */
            PIL_FreeMemory(prefixEntryArray);
            PIL_FreeMemory(nhIdArray);
            return 0;
         
        }
         
         
        int NPF_IPv4UC_NextHopEntryAdd_SelfTest(             NPF_IPv4UC_NextHopTableHandle_t *nxthopTableHandle, NPF_uint32_t *no, NPF_uint32_t *nhId, NPF_IPv4UC_NextHopArray_t *nextHopEntry, void *corr)
        {
        
           NPF_IPv4UC_NextHopArray_t *nextHopArray=NULL;
           NPF_IPv4UC_NextHop_t   *nextHopEntries=NULL;
           NPF_uint32_t *nhIdArray=NULL;
           uint32_t i; 
           uint32_t num = *no; 

           nextHopArray = (NPF_IPv4UC_NextHopArray_t*) PIL_AllocateMemory(sizeof(NPF_IPv4UC_NextHopArray_t)*num, PIL_MEM_ZERO_INIT);
           nextHopEntries = (NPF_IPv4UC_NextHop_t*) PIL_AllocateMemory(sizeof(NPF_IPv4UC_NextHop_t)*num, PIL_MEM_ZERO_INIT);
           nhIdArray = (NPF_uint32_t*) PIL_AllocateMemory(sizeof(NPF_uint32_t)*num, PIL_MEM_ZERO_INIT);
            
           if(nextHopArray == NULL || nextHopEntries == NULL || nhIdArray == NULL)
               return 1;

           for(i=0; i < num; i++)
           {
              nextHopArray[i].nextHopCount = 1;
              nextHopArray[i].nextHopArray = &nextHopEntries[i]; 
              nextHopArray[i].nextHopArray->type = 
                                  nextHopEntry->nextHopArray->type;     
              nextHopArray[i].nextHopArray->weight =     
                                  nextHopEntry->nextHopArray->weight;     
              nextHopArray[i].nextHopArray->egressInterface =     
                                  nextHopEntry->nextHopArray->egressInterface;   
              nextHopArray[i].nextHopArray->nextHopIP =     
                                  nextHopEntry->nextHopArray->nextHopIP+i;   
              nextHopArray[i].nextHopArray->mediaAddress.type =     
                                  nextHopEntry->nextHopArray->mediaAddress.type; 
       /*     nextHopArray[i].nextHopArray->mediaAddress.u.MAC_Address = 
                   nextHopEntry->nextHopArray->mediaAddress.u.MAC_Address;  */
              nhIdArray[i] = *nhId + i;
           }                      
           NPF_IPv4UC_NextHopEntryAdd(IPV4cbh,(NPF_correlator_t)corr,NPF_REPORT_ALL,
              *nxthopTableHandle, num, nhIdArray, nextHopArray);

           /* Free the Memory */
           PIL_FreeMemory(nextHopArray);
           PIL_FreeMemory(nextHopEntries);
           PIL_FreeMemory(nhIdArray);
           return 0;
       
        }
         
         
        int NPF_IPv4UC_PrefixEntryDelete_SelfTest(             NPF_IPv4UC_PrefixTableHandle_t *prefixTableHandle, NPF_uint32_t *no, NPF_IPv4UC_Prefix_t *prefixEntry, void *corr)
        {
        
            NPF_IPv4UC_Prefix_t *prefixEntryArray=NULL;
            uint32_t i; 
            uint32_t num = *no; 
            
            prefixEntryArray = (NPF_IPv4UC_Prefix_t*) PIL_AllocateMemory(sizeof(NPF_IPv4UC_Prefix_t)*num, PIL_MEM_ZERO_INIT);
            if(prefixEntryArray == NULL)
               return 1;
            
            for(i=0; i < num; i++)
            {  
               prefixEntryArray[i].IPv4Addr = prefixEntry->IPv4Addr + i;
               prefixEntryArray[i].IPv4NetPlen = prefixEntry->IPv4NetPlen;
            }    
            NPF_IPv4UC_PrefixEntryDelete(IPV4cbh,(NPF_correlator_t)corr,NPF_REPORT_ALL,
               *prefixTableHandle, num, prefixEntryArray);
            PIL_FreeMemory(prefixEntryArray);
            return 0;
            
        }
         
         
        int NPF_IPv4UC_NextHopEntryDelete_SelfTest(                 NPF_IPv4UC_NextHopTableHandle_t *nxthopTableHandle, NPF_uint32_t *no, NPF_uint32_t *nhId, void *corr)
        {
        
              NPF_uint32_t *nhIdArray=NULL;
              uint32_t i; 
              uint32_t num = *no; 

               nhIdArray = (NPF_uint32_t*) PIL_AllocateMemory(sizeof(NPF_uint32_t)*num, PIL_MEM_ZERO_INIT);
               if(nhIdArray == NULL)
                  return 1;

              for(i=0; i < num; i++)
                 nhIdArray[i] = *nhId + i;
              NPF_IPv4UC_NextHopEntryDelete(IPV4cbh,(NPF_correlator_t)corr,NPF_REPORT_ALL,
                 *nxthopTableHandle, num, nhIdArray);  
              PIL_FreeMemory(nhIdArray);
              return 0;
            
        }
         
void cppui_ipv4_message(CppuiRequest *req)
{
    switch(req->type)
    {
        case 158: /*IPV4Get_Supported_Mode */
        {
            
            NPF_error_t ret = 1;
            
            
            ret = NPF_IPv4UC_GetSupportedModes();
            
            
            req->error = formatModeResp(158, ret, req->resp);
            getTime(&req->respts);
            sendResponse(req);
            
        }
        break;
        case 159: /*IPV4Get_Preferred_Mode */
        {
            
            NPF_error_t ret = 1;
            
            
            ret = NPF_IPv4UC_GetPreferredMode();
            
            
            req->error = formatModeResp(159, ret, req->resp);
            getTime(&req->respts);
            sendResponse(req);
            
        }
        break;
        case 160: /*IPV4Create_Prefix_Table_Handle */
        {
            
            NPF_error_t ret = 1;
            
            
            ret = NPF_IPv4UC_PrefixTableHandleCreate(IPV4_COMMON_ARGS);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 161: /*IPV4Create_NextHop_Table_Handle */
        {
            
            NPF_error_t ret = 1;
            
            
            ret = NPF_IPv4UC_NextHopTableHandleCreate(IPV4_COMMON_ARGS);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 162: /*IPV4Create_AddRes_Table_Handle */
        {
            
            NPF_error_t ret = 1;
            
            
            ret = NPF_IPv4UC_AddResTableHandleCreate(IPV4_COMMON_ARGS);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 163: /*IPV4Delete_Prefix_Table_Handle */
        {
            NPF_IPv4UC_PrefixTableHandle_t prefixTableHandle;
            
            NPF_error_t ret = 1;
            
            
            prefixTableHandle = (int )getHandle(req->args[2]); /*PrefixTableHandle*/
            ret = NPF_IPv4UC_PrefixTableHandleDelete(IPV4_COMMON_ARGS, prefixTableHandle);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 164: /*IPV4Delete_NextHop_Table_Handle */
        {
            NPF_IPv4UC_NextHopTableHandle_t nxthopTableHandle;
            
            NPF_error_t ret = 1;
            
            
            nxthopTableHandle = (int )getHandle(req->args[2]); /*PrefixTableHandle*/
            ret = NPF_IPv4UC_NextHopTableHandleDelete(IPV4_COMMON_ARGS, nxthopTableHandle);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 165: /*IPV4Delete_Address_Table_Handle */
        {
            NPF_IPv4UC_AddResTableHandle_t addResTableHandle;
            
            NPF_error_t ret = 1;
            
            
            addResTableHandle = (int )getHandle(req->args[2]); /*AddResTableHandle*/
            ret = NPF_IPv4UC_AddResTableHandleDelete(IPV4_COMMON_ARGS, addResTableHandle);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 166: /*IPV4Bind_Prefix_NextHop_Table */
        {
            NPF_IPv4UC_PrefixTableHandle_t prefixTableHandle;
            NPF_IPv4UC_NextHopTableHandle_t nxthopTableHandle;
            
            NPF_error_t ret = 1;
            
            
            prefixTableHandle = (int )getHandle(req->args[2]); /*PrefixTableHandle*/
            nxthopTableHandle = (int )getHandle(req->args[3]); /*NextHopTableHandle*/
            ret = NPF_IPv4UC_PrefixNextHopTableBind(IPV4_COMMON_ARGS, prefixTableHandle, nxthopTableHandle);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 167: /*IPV4Add_Prefix_Table_Entry */
        {
            NPF_IPv4UC_PrefixTableHandle_t prefixTableHandle;
            NPF_uint32_t no;
            NPF_IPv4UC_Prefix_t prefixEntry;
            NPF_uint32_t nhId;
            
            NPF_error_t ret = 1;
            
            int selftest = 0;
            
            prefixTableHandle = (int )getHandle(req->args[2]); /*PrefixTableHandle*/
            no = (int )atol(req->args[3]); /*NoOfEntries*/
            getPrefixEntry(req->args[4], &prefixEntry, 1); /*IPv4Addr*/
            getPrefixEntry(req->args[5], &prefixEntry, 2); /*IPv4NetPlen*/
            nhId = (int )atol(req->args[6]); /*NextHopId*/
            selftest = (int)atol(req->args[7]);
             
            if(selftest)
                ret = NPF_IPv4UC_PrefixEntryAdd_SelfTest(&prefixTableHandle, &no, &prefixEntry, &nhId, req);
            else
                ret = NPF_IPv4UC_PrefixEntryAdd(IPV4_COMMON_ARGS, prefixTableHandle, no, &prefixEntry, &nhId);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 168: /*IPV4Add_NextHop_Table_Entry */
        {
            NPF_IPv4UC_NextHopTableHandle_t nxthopTableHandle;
            NPF_uint32_t no;
            NPF_uint32_t nhId;
            NPF_IPv4UC_NextHopArray_t nextHopEntry;
            
            NPF_error_t ret = 1;
            
            int selftest = 0;
            
            no = (int )atol(req->args[2]); /*No*/
            nxthopTableHandle = (int )getHandle(req->args[3]); /*NextHopTableHandle*/
            nhId = (int )atol(req->args[4]); /*NextHopId*/
            getNextHopParams(req->args[5], &nextHopEntry, 1); /*NextHopType*/
            getNextHopParams(req->args[6], &nextHopEntry, 2); /*NextHopWeight*/
            nextHopEntry.nextHopArray->egressInterface = (int )getHandle(req->args[7]); /*EgressIfaceHandle*/
            getNextHopParams(req->args[8], &nextHopEntry, 3); /*NextHop_IP_Address*/
            getNextHopParams(req->args[9], &nextHopEntry, 4); /*Media_Address_Type*/
            getNextHopParams(req->args[10], &nextHopEntry, 5); /*Media_Address*/
            selftest = (int)atol(req->args[11]);
             
            if(selftest)
                ret = NPF_IPv4UC_NextHopEntryAdd_SelfTest(&nxthopTableHandle, &no, &nhId, &nextHopEntry, req);
            else
                ret = NPF_IPv4UC_NextHopEntryAdd(IPV4_COMMON_ARGS, nxthopTableHandle, no, &nhId, &nextHopEntry);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 169: /*IPV4Add_AddRes_Table_Entry */
        {
            NPF_IPv4UC_AddResTableHandle_t addResTableHandle;
            NPF_uint32_t no;
            NPF_IPv4UC_AddResEntry_t addResEntry;
            
            NPF_error_t ret = 1;
            
            
            no = (int )atol(req->args[2]); /*NoOfEntries*/
            addResTableHandle = (int )getHandle(req->args[3]); /*AddResTableHandle*/
            getAddResParams(req->args[4], &addResEntry,1); /*IP_Address*/
            addResEntry.interfaceHandle = (int )getHandle(req->args[5]); /*Interface_Handle*/
            getAddResParams(req->args[6], &addResEntry, 2); /*Media_Address_Type*/
            getAddResParams(req->args[7], &addResEntry, 3); /*Media_Address*/
            ret = NPF_IPv4UC_AddResEntryAdd(IPV4_COMMON_ARGS, addResTableHandle, no, &addResEntry);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 170: /*IPV4Delete_Prefix_Table_Entry */
        {
            NPF_IPv4UC_PrefixTableHandle_t prefixTableHandle;
            NPF_uint32_t no;
            NPF_IPv4UC_Prefix_t prefixEntry;
            
            NPF_error_t ret = 1;
            
            int selftest = 0;
            
            prefixTableHandle = (int )getHandle(req->args[2]); /*PrefixTableHandle*/
            no = (int )atol(req->args[3]); /*NoOfEntries*/
            getPrefixEntry(req->args[4], &prefixEntry, 1); /*IPv4Addr*/
            getPrefixEntry(req->args[5], &prefixEntry, 2); /*IPv4NetPlen*/
            getPrefixEntry(req->args[6], &prefixEntry, 3); /*NextHopIndex*/
            selftest = (int)atol(req->args[7]);
             
            if(selftest)
                ret = NPF_IPv4UC_PrefixEntryDelete_SelfTest(&prefixTableHandle, &no, &prefixEntry, req);
            else
                ret = NPF_IPv4UC_PrefixEntryDelete(IPV4_COMMON_ARGS, prefixTableHandle, no, &prefixEntry);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 171: /*IPV4Delete_NextHop_Table_Entry */
        {
            NPF_IPv4UC_NextHopTableHandle_t nxthopTableHandle;
            NPF_uint32_t no;
            NPF_uint32_t nhId;
            
            NPF_error_t ret = 1;
            
            int selftest = 0;
            
            no = (int )atol(req->args[2]); /*NoOfEntries*/
            nxthopTableHandle = (int )getHandle(req->args[3]); /*NextHopTableHandle*/
            nhId = (int )atol(req->args[4]); /*NextHopId*/
            selftest = (int)atol(req->args[5]);
             
            if(selftest)
                ret = NPF_IPv4UC_NextHopEntryDelete_SelfTest(&nxthopTableHandle, &no, &nhId, req);
            else
                ret = NPF_IPv4UC_NextHopEntryDelete(IPV4_COMMON_ARGS, nxthopTableHandle, no, &nhId);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 172: /*IPV4Delete_AddRes_Entry */
        {
            NPF_IPv4UC_AddResTableHandle_t addResTableHandle;
            NPF_uint32_t no;
            NPF_IPv4UC_AddResKey_t addResKeyEntry;
            
            NPF_error_t ret = 1;
            
            
            no = (int )atol(req->args[2]); /*NoOfEntries*/
            addResTableHandle = (int )getHandle(req->args[3]); /*AddResTableHandle*/
            getipv4address(req->args[4],  &addResKeyEntry.IP_Address ); /*IP_Address*/
            addResKeyEntry.interfaceHandle = (int )getHandle(req->args[5]); /*Interface_Handle*/
            ret = NPF_IPv4UC_AddResEntryDelete(IPV4_COMMON_ARGS, addResTableHandle, no, &addResKeyEntry);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 173: /*IPV4Flush_Prefix_Table */
        {
            NPF_IPv4UC_PrefixTableHandle_t prefixTableHandle;
            
            NPF_error_t ret = 1;
            
            
            prefixTableHandle = (int )getHandle(req->args[2]); /*PrefixTableHandle*/
            ret = NPF_IPv4UC_PrefixTableFlush(IPV4_COMMON_ARGS, prefixTableHandle);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 174: /*IPV4Flush_NextHop_Table */
        {
            NPF_IPv4UC_NextHopTableHandle_t nxthopTableHandle;
            
            NPF_error_t ret = 1;
            
            
            nxthopTableHandle = (int )getHandle(req->args[2]); /*NextHopTableHandle*/
            ret = NPF_IPv4UC_NextHopTableFlush(IPV4_COMMON_ARGS, nxthopTableHandle);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 175: /*IPV4Flush_AddRes_Table */
        {
            NPF_IPv4UC_AddResTableHandle_t addResTableHandle;
            
            NPF_error_t ret = 1;
            
            
            addResTableHandle = (int )getHandle(req->args[2]); /*AddResTableHandle*/
            ret = NPF_IPv4UC_AddResTableFlush(IPV4_COMMON_ARGS, addResTableHandle);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 176: /*IPV4Query_Prefix_Table_Entry */
        {
            NPF_IPv4UC_PrefixTableHandle_t prefixTableHandle;
            NPF_uint32_t no;
            NPF_IPv4UC_Prefix_t prefixEntry;
            
            NPF_error_t ret = 1;
            
            
            prefixTableHandle = (int )getHandle(req->args[2]); /*PrefixTableHandle*/
            no = (int )atol(req->args[3]); /*NoOfEntries*/
            getPrefixEntry(req->args[4], &prefixEntry, 1); /*IPv4Addr*/
            getPrefixEntry(req->args[5], &prefixEntry, 2); /*IPv4NetPlen*/
            ret = NPF_IPv4UC_PrefixEntryQuery(IPV4_COMMON_ARGS, prefixTableHandle, no, &prefixEntry);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 177: /*IPV4Query_NextHop_Table_Entry */
        {
            NPF_IPv4UC_NextHopTableHandle_t nxthopTableHandle;
            NPF_uint32_t no;
            NPF_uint32_t nhId;
            
            NPF_error_t ret = 1;
            
            
            no = (int )atol(req->args[2]); /*NoOfEntries*/
            nxthopTableHandle = (int )getHandle(req->args[3]); /*NextHopTableHandle*/
            nhId = (int )atol(req->args[4]); /*NextHopId*/
            ret = NPF_IPv4UC_NextHopEntryQuery(IPV4_COMMON_ARGS, nxthopTableHandle, no, &nhId);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 178: /*IPV4Query_AddRes_Table_Entry */
        {
            NPF_IPv4UC_AddResTableHandle_t addResTableHandle;
            NPF_uint32_t no;
            NPF_IPv4UC_AddResKey_t addResKeyEntry;
            
            NPF_error_t ret = 1;
            
            
            no = (int )atol(req->args[2]); /*NoOfEntries*/
            addResTableHandle = (int )getHandle(req->args[3]); /*AddResTableHandle*/
            getipv4address(req->args[4],  &addResKeyEntry.IP_Address ); /*IP_Address*/
            addResKeyEntry.interfaceHandle = (int )getHandle(req->args[5]); /*Interface_Handle*/
            ret = NPF_IPv4UC_AddResEntryQuery(IPV4_COMMON_ARGS, addResTableHandle, no, &addResKeyEntry);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 198: /*IPV4Reset_IPV4_Handles_For_Test_Repeat */
        {
            
            NPF_error_t ret = 1;
            
            
            ret = Reset_IPV4_Handles();
            
            
            req->error = 0;
            sprintf(req->resp, "IPV4Reset_IPV4_Handles_For_Test_Repeat");
            getTime(&req->respts);
            sendResponse(req);
            
        }
        break;
    }
}
void cppui_ipv4_unified_message(CppuiRequest *req)
{
    switch(req->type)
    {
        case 180: /*IPV4_UNIFIEDCreate_Fib_Table_Handle */
        {
            
            NPF_error_t ret = 1;
            
            
            ret = NPF_IPv4UC_FibTableHandleCreate(IPV4_COMMON_ARGS);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 181: /*IPV4_UNIFIEDDelete_Fib_Table_Handle */
        {
            NPF_IPv4UC_FibTableHandle_t fibTableHandle;
            
            NPF_error_t ret = 1;
            
            
            fibTableHandle = (int )getHandle(req->args[2]); /*FibTableHandle*/
            ret = NPF_IPv4UC_FibTableHandleDelete(IPV4_COMMON_ARGS, fibTableHandle);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 182: /*IPV4_UNIFIEDAdd_Fib_Table_Entry */
        {
            NPF_IPv4UC_FibTableHandle_t fibTableHandle;
            NPF_uint32_t no;
            NPF_IPv4UC_Prefix_t prefixEntry;
            NPF_IPv4UC_NextHopArray_t nextHopEntry;
            
            NPF_error_t ret = 1;
            
            
            fibTableHandle = (int )getHandle(req->args[2]); /*FibTableHandle*/
            no = (int )atol(req->args[3]); /*NoOfEntries*/
            getPrefixEntry(req->args[4], &prefixEntry, 1); /*IPv4Addr*/
            getPrefixEntry(req->args[5], &prefixEntry, 2); /*IPv4NetPlen*/
            getNextHopParams(req->args[6], &nextHopEntry, 3); /*NextHop_IP_Address*/
            getNextHopParams(req->args[7], &nextHopEntry, 1); /*NextHopType*/
            getNextHopParams(req->args[8], &nextHopEntry, 2); /*NextHopWeight*/
            nextHopEntry.nextHopArray->egressInterface = (int )getHandle(req->args[9]); /*EgressIfaceHandle*/
            getNextHopParams(req->args[10], &nextHopEntry, 4); /*Media_Address_Type*/
            getNextHopParams(req->args[11], &nextHopEntry, 5); /*Media_Address*/
            ret = NPF_IPv4UC_FibEntryAdd(IPV4_COMMON_ARGS, fibTableHandle, no, &prefixEntry, &nextHopEntry);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 183: /*IPV4_UNIFIEDDelete_Fib_Table_Entry */
        {
            NPF_IPv4UC_FibTableHandle_t fibTableHandle;
            NPF_uint32_t no;
            NPF_IPv4UC_Prefix_t prefixEntry;
            
            NPF_error_t ret = 1;
            
            
            fibTableHandle = (int )getHandle(req->args[2]); /*FibTableHandle*/
            no = (int )atol(req->args[3]); /*NoOfEntries*/
            getPrefixEntry(req->args[4], &prefixEntry, 1); /*IPv4Addr*/
            getPrefixEntry(req->args[5], &prefixEntry, 2); /*IPv4NetPlen*/
            ret = NPF_IPv4UC_FibEntryDelete(IPV4_COMMON_ARGS, fibTableHandle, no, &prefixEntry);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 184: /*IPV4_UNIFIEDFlush_Fib_Table */
        {
            NPF_IPv4UC_FibTableHandle_t fibTableHandle;
            
            NPF_error_t ret = 1;
            
            
            fibTableHandle = (int )getHandle(req->args[2]); /*FibTableHandle*/
            ret = NPF_IPv4UC_FibTableFlush(IPV4_COMMON_ARGS, fibTableHandle);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
        case 185: /*IPV4_UNIFIEDQuery_Fib_Table_Entry */
        {
            NPF_IPv4UC_FibTableHandle_t fibTableHandle;
            NPF_uint32_t no;
            NPF_IPv4UC_Prefix_t prefixEntry;
            
            NPF_error_t ret = 1;
            
            
            fibTableHandle = (int )getHandle(req->args[2]); /*FibTableHandle*/
            no = (int )atol(req->args[3]); /*NoOfEntries*/
            getPrefixEntry(req->args[4], &prefixEntry, 1); /*IPv4Addr*/
            getPrefixEntry(req->args[5], &prefixEntry, 2); /*IPv4NetPlen*/
            ret = NPF_IPv4UC_FibEntryQuery(IPV4_COMMON_ARGS, fibTableHandle, no, &prefixEntry);
            
            if(ret == NPF_NO_ERROR)
            {
                #ifdef CPPUI_WAIT_RESP
                waitAsyncResponse(req);
                #endif
            }
            else
            {
                req->error = ret;
                sprintf(req->resp, "%s", "Command Rejected");
                sendResponse(req);
            }
        }
        break;
    }
}
void cppui_ipv4_response(CppuiRequest *req, void *data)
{
    switch(req->type)
    {
        case 160: /*IPV4Create_Prefix_Table_Handle */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(160, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 161: /*IPV4Create_NextHop_Table_Handle */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(161, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 162: /*IPV4Create_AddRes_Table_Handle */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(162, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 163: /*IPV4Delete_Prefix_Table_Handle */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(163, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 164: /*IPV4Delete_NextHop_Table_Handle */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(164, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 165: /*IPV4Delete_Address_Table_Handle */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(165, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 166: /*IPV4Bind_Prefix_NextHop_Table */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(166, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 167: /*IPV4Add_Prefix_Table_Entry */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(167, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 168: /*IPV4Add_NextHop_Table_Entry */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(168, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 169: /*IPV4Add_AddRes_Table_Entry */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(169, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 170: /*IPV4Delete_Prefix_Table_Entry */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(170, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 171: /*IPV4Delete_NextHop_Table_Entry */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(171, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 172: /*IPV4Delete_AddRes_Entry */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(172, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 173: /*IPV4Flush_Prefix_Table */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(173, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 174: /*IPV4Flush_NextHop_Table */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(174, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 175: /*IPV4Flush_AddRes_Table */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(175, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 176: /*IPV4Query_Prefix_Table_Entry */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(176, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 177: /*IPV4Query_NextHop_Table_Entry */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(177, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 178: /*IPV4Query_AddRes_Table_Entry */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(178, cbdata, req->resp);
            sendResponse(req);
        }
        break;
    } /*switch(req->type) */
}
void cppui_ipv4_unified_response(CppuiRequest *req, void *data)
{
    switch(req->type)
    {
        case 180: /*IPV4_UNIFIEDCreate_Fib_Table_Handle */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(180, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 181: /*IPV4_UNIFIEDDelete_Fib_Table_Handle */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(181, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 182: /*IPV4_UNIFIEDAdd_Fib_Table_Entry */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(182, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 183: /*IPV4_UNIFIEDDelete_Fib_Table_Entry */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(183, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 184: /*IPV4_UNIFIEDFlush_Fib_Table */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(184, cbdata, req->resp);
            sendResponse(req);
        }
        break;
        case 185: /*IPV4_UNIFIEDQuery_Fib_Table_Entry */
        {
            NPF_IPv4UC_CallbackData_t *cbdata = (NPF_IPv4UC_CallbackData_t*)data;
            req->error = formatIPv4Response(185, cbdata, req->resp);
            sendResponse(req);
        }
        break;
    } /*switch(req->type) */
}
