/*****************************************************************************
 *                  INTEL CORPORATION PROPRIETARY IF_INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 *$Source: /cvs/constellations/ixa_3.0/src/cp_pdk/cppui/build/src/cppui_server.c,v $
 *$Revision: 1.5 $
 *$Date: 2003/11/17 16:57:57 $
 * Description:
 *     This file contains the UDP server implementation of CPPUI module
 *
 *****************************************************************************/

#include "cppui.h"

char EventList[MAX_EVENT_COUNT][MAX_MSG_SIZE];
static int eventCount = 0;
void sendPendingEvents(CppuiRequest *listner);

static int EventListnerCompare(const void *data, const void *key)
{
   CPPUI_LOCK();
   if(!(memcmp((CppuiRequest *)data, (CppuiRequest *)key, sizeof(CppuiRequest))))
   {
      CPPUI_UNLOCK();
      return 1;
   }
   CPPUI_UNLOCK();
   return 0;
}

void cppui(char *args)
{
   int port = CPPDK_PORT;
   int sd, rc, n, cliLen;
   struct sockaddr_in cliAddr, servAddr;
   char *tok = NULL;
   CppuiRequest *req = NULL;
   int err = 0;
   #ifdef _VXWORKS_
   int fd;
   #endif
   
   #ifdef _VXWORKS_
   #define MAX_MSG_LEN 256
   pipeDrv();
   pipeDevCreate("/pipe/cppui", 10, MAX_MSG_LEN);
   fd = open("/pipe/cppui", O_RDWR, 0666);
   if (fd == -1)
      err = -1;
   else
      g_pipefds[0] = g_pipefds[1] = fd;
   #else
   err = pipe( g_pipefds );
   #endif
   if (err == -1)
   {
      printf("***** ERROR: pipe creation failed **** \n");
      exit(0);
   }
    
   PIL_InitializeCriticalSection(&LOCK);
   npf_list_init(&EventListners, PIL_FreeMemory);
   EventListners.compare = &EventListnerCompare;
    
   cppdkInit();
    
   sd=socket(AF_INET, SOCK_DGRAM, 0);
   if(sd<0)
   {
      printf("*** ERROR: Socket Creation failed ???????? ***\n");
      exit(1);
   }
   if(getenv("CPPDK_PORT") != NULL)
      port = atoi(getenv("CPPDK_PORT"));
   printf("Starting UDP server with port : %d\n", (int)port);
   servAddr.sin_family = AF_INET;
   servAddr.sin_addr.s_addr = htonl(INADDR_ANY);
   servAddr.sin_port = htons(port);
   rc = bind (sd, (struct sockaddr *) &servAddr,sizeof(servAddr));
   if(rc<0)
   {
      printf("*** ERROR: Socket Bind failed ???????? ***\n");
      exit(1);
   }
   while(1)
   {
      cliLen = sizeof(cliAddr);
      req = (CppuiRequest *)PIL_AllocateMemory(sizeof(CppuiRequest), PIL_MEM_ZERO_INIT);
      if(req == NULL) continue;
      n = recvfrom(sd, req->msg, MAX_MSG_SIZE, 0, (struct sockaddr *) &cliAddr, &cliLen);
      if(n<0){PIL_FreeMemory(req); continue;}
      tok = strtok(req->msg, ";");
      while(tok)
      {
         strcpy(req->args[req->no++], tok);
         tok = strtok( NULL, ";" );
      }
      req->sd = sd;
      memcpy(&req->to, &cliAddr, sizeof(cliAddr));
      req->pos = 0;
      req->pos += sprintf(&req->resp[req->pos], "%s;", req->args[0]);
      process(req);
   }
   PIL_DestroyCriticalSection(&LOCK);
   npf_list_destroy(&EventListners);
   npf_pdk_shutdown();
}
 
 
void onCppuiConnect(CppuiRequest *req)
{
   addEventListner(req);
   req->error = registerCppdk;
   if(registerCppdk == 0)
       sprintf(req->resp, "Connect Response(OK)");
   else
       sprintf(req->resp, "Connection Failed, May Registering all modules failed");
   sendPendingEvents(req);  
   sendResponse(req);
}

void onCppuiDisconnect(CppuiRequest *req)
{
   deleteEventListner(req);
   req->error = 0;
   cppdkShutdown();
   sprintf(req->resp, "Disconnect Response(OK)");
   sendResponse(req);
}

void sendResponse(CppuiRequest *req)
{
   time_t secs;
   struct tm *tmp;
   unsigned long long millis;
   uint32_t seconds;
   millis = req->respts - req->reqts;
   secs = (time_t)millis / 1000;
   millis = millis % 1000;
   tmp = localtime(&secs);
   seconds = tmp->tm_sec;
   if(req == NULL || req->type == 0) return;
   sprintf(&req->resp[ strlen(req->resp)], "#%d;%d;%d#%d sec:%d ms", 
		(int)req->type, (int)req->seq, (int)req->error, (int)seconds, (int)millis);
   sendto(req->sd, req->resp, strlen(req->resp), 0, (struct sockaddr *) &req->to, sizeof(req->to));
   memset(req, 0, sizeof(CppuiRequest));
   PIL_FreeMemory(req);
}
 
void addEventListner(CppuiRequest *req)
{
   CppuiRequest *listner = (CppuiRequest *)PIL_AllocateMemory(sizeof(CppuiRequest), PIL_MEM_ZERO_INIT);
   if(listner == NULL) return;
   memcpy(listner, req, sizeof(CppuiRequest));
   npf_list_pushBack(&EventListners, listner);
}
 
void deleteEventListner(CppuiRequest *req)
{
   npf_list_removeByKeyFree(&EventListners, req);
}
 
void sendPendingEvents(CppuiRequest *listner)
{
    int i = 0;
    for(i = 0; i < eventCount; i++)
    {
        sendto(listner->sd, 
	     EventList[i], 
	     strlen(EventList[i]), 
	     0, 
	     (struct sockaddr *) &listner->to, 
	     sizeof(listner->to));
    }
}

void sendEvent(char *event)
{
   DListItr            *eventItr;
   CppuiRequest        *listner;
   if(eventCount >= MAX_EVENT_COUNT)
	eventCount = 0;
   strcpy(EventList[eventCount++], event);
   eventItr = npf_list_itrCreate (&EventListners);
   if(!eventItr) return;
   npf_list_itrFirst (eventItr);
   do
   {
      listner = (CppuiRequest *)npf_list_itrGetData (eventItr);
      if(!listner) continue;
      sendto(listner->sd, event, strlen(event), 0, (struct sockaddr *) &listner->to, sizeof(listner->to));
   }while (npf_list_itrNext (eventItr) != -1);
   npf_list_itrDelete(eventItr);
}
 
#ifdef _LINUX_
int main( int argc, char **argv )
{
   cppui("CPPUI");
   return 0;
}
#endif

