/*****************************************************************************
 *                  INTEL CORPORATION PROPRIETARY IF_INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 *$Source: /cvs/constellations/ixa_3.0/src/cp_pdk/cppui/build/src/cppui_util.c,v $
 *$Revision: 1.6 $
 *$Date: 2003/11/17 16:57:57 $
 * Description:
 *     This file contains the utility functions of CPPUI module
 *
 *****************************************************************************/

#include "cppui_common.h"

NPF_error_t             npfRet = 0;
NPF_error_t             registerCppdk = 0;
NPF_userContext_t       context = 0;
NPF_callbackHandle_t    IFcbh = 0;
NPF_callbackHandle_t    DScbh = 0;
NPF_callbackHandle_t    IPV4cbh = 0;
NPF_callbackHandle_t    MPLScbh = 0;
NPF_callbackHandle_t    ATMcbh = 0;
NPF_IfEventHandlerHandle_t    IfEvtHandle = 0;

#define CPPUI_MAX_HANDLES	1000

uint32_t  		LANInterfaceHandleCount = 0;
NPF_IfHandle_t  	LANInterfaceHandleList[CPPUI_MAX_HANDLES];
uint32_t  		POSInterfaceHandleCount = 0;
NPF_IfHandle_t  	POSInterfaceHandleList[CPPUI_MAX_HANDLES];
uint32_t  		IPV4InterfaceHandleCount = 0;
NPF_IfHandle_t  	IPV4InterfaceHandleList[CPPUI_MAX_HANDLES];

/* IPV4 Handles */
uint32_t  		IPV4PrefixHandleCount = 0;
NPF_IPv4UC_PrefixTableHandle_t IPV4PrefixHandleList[CPPUI_MAX_HANDLES];
uint32_t  		IPV4NexthopHandleCount = 0;
NPF_IPv4UC_NextHopTableHandle_t IPV4NexthopHandleList[CPPUI_MAX_HANDLES];
uint32_t  		IPV4AddressHandleCount = 0;
NPF_IPv4UC_AddResTableHandle_t IPV4AddressHandleList[CPPUI_MAX_HANDLES];
uint32_t  		IPV4FibHandleCount = 0;
NPF_IPv4UC_FibTableHandle_t     IPV4FibHandleList[CPPUI_MAX_HANDLES];
uint32_t  		IPV4FwdHandleCount = 0;
NPF_IPv4UC_FwdTableHandle_t    IPV4FwdHandleList[CPPUI_MAX_HANDLES];

#ifdef MPLS
uint32_t  		MPLSInterfaceHandleCount = 0;
NPF_IfHandle_t  	MPLSInterfaceHandleList[CPPUI_MAX_HANDLES];
uint32_t  		MPLSLSPHandleCount = 0;
NPF_IfHandle_t  	MPLSLSPHandleList[CPPUI_MAX_HANDLES];
uint32_t  		MPLSNhlfeHandleCount = 0;
NPF_IfHandle_t  	MPLSNhlfeHandleList[CPPUI_MAX_HANDLES];
uint32_t  		MPLSNhlfeSetHandleCount = 0;
NPF_IfHandle_t  	MPLSNhlfeSetHandleList[CPPUI_MAX_HANDLES];
#endif

#ifdef ATM
uint32_t  		ATMInterfaceHandleCount = 0;
NPF_IfHandle_t  	ATMInterfaceHandleList[CPPUI_MAX_HANDLES];
#endif

#ifdef DIFFSERV
uint32_t  		ClassifierHandleCount = 0;
NPF_DS_DpeHandle_t  	ClassifierHandleList[CPPUI_MAX_HANDLES];
uint32_t  		MeterHandleCount = 0;
NPF_DS_DpeHandle_t  	MeterHandleList[CPPUI_MAX_HANDLES];
uint32_t  		MarkerHandleCount = 0;
NPF_DS_DpeHandle_t  	MarkerHandleList[CPPUI_MAX_HANDLES];
uint32_t  		DropperHandleCount = 0;
NPF_DS_DpeHandle_t  	DropperHandleList[CPPUI_MAX_HANDLES];
#endif

int compareHandle(const void *data, const void *key)
{
    return(1);
}

void cppdkInit()
{
    #ifndef PRINT_LOG
    char *log = "cppui.log";
    #endif

    npf_pdk_init();
    npf_pdk_start();

    npf_logger_Init();

    #ifdef PRINT_LOG
    npf_logger_Start("stdout");
    #else
    npf_logger_Start(log);
    #endif

    #ifdef IF_LOG
    npf_logger_SetLevel(LOG_IF, 0xF);
    #endif

    #ifdef DS_LOG
    npf_logger_SetLevel(LOG_DS, 0xF);
    #endif

    #ifdef IPV4_LOG
    npf_logger_SetLevel(LOG_IPV4, 0xF);
    #endif

    #ifdef PDKMGR_LOG
    npf_logger_SetLevel(LOG_PDKMGR, 0xF);
    #endif

    #ifdef MPLS_LOG
    npf_logger_SetLevel(LOG_MPLS, 0xF);
    #endif

    #ifdef ATM_LOG
    npf_logger_SetLevel(LOG_ATM, 0xF);
    #endif

    npf_logger_SetVerbosity(LOG_VERB_MAX);

    PIL_InitializeCriticalSection(&LOCK);

    registerCppdk += NPF_IfRegister(context, (NPF_IfCallbackFunc_t)cppuiCallback, &IFcbh);

    registerCppdk += NPF_IPv4UC_Register(context, (NPF_IPv4UC_CallbackFunc_t)cppuiIPv4Callback, &IPV4cbh);

    #ifdef DIFFSERV
    registerCppdk += NPF_DS_Register(context, (NPF_DS_CallbackFunc_t)cppuiCallback, &DScbh);
    #endif

    #ifdef MPLS
    registerCppdk += NPF_MPLS_Register(context, (NPF_MPLS_CallbackFunc_t)cppuiCallback, &MPLScbh);
    #endif

    CppuiEventRegister();
}

void cppdkShutdown()
{

    #ifdef EXIT_ON_DISCONNECT

    PIL_DestroyCriticalSection(&LOCK);

    registerCppdk += NPF_IfDeregister(IFcbh);

    #ifdef DIFFSERV
    registerCppdk += NPF_DS_Deregister(&DScbh);
    #endif

    #ifdef MPLS
    registerCppdk += NPF_MPLS_Deregister(&MPLScbh);
    #endif

    CppuiEventDeregister();
    npf_pdk_shutdown();
    exit(0);

    #endif
}

void waitAsyncResponse(CppuiRequest  *req)
{
   fd_set         rfds;
   struct timeval tv;
   uint32_t       len = 0;
   tv.tv_sec = ASYNC_RESP_WAIT_TIME;
   tv.tv_usec = 0;
   FD_ZERO( &rfds );
   FD_SET( g_pipefds[0], &rfds );
   select( g_pipefds[0] + 1, &rfds, NULL, NULL, (ASYNC_RESP_WAIT_TIME == 0)?NULL :&tv );
   if( FD_ISSET(g_pipefds[0], &rfds ))
   {
      read(g_pipefds[0], (char *)&len, 4);
      FD_CLR( g_pipefds[0], &rfds );
   }
   else
   {
      if(req == NULL || req->type == 0) return;
      req->error  = (int)ERROR_REQUEST_TIMEOUT;
      req->pos += sprintf(req->resp, "%s(%d);", "REQUEST TIMED OUT", ASYNC_RESP_WAIT_TIME);
      sendResponse(req);
      write(g_pipefds[1], (char *)&len, sizeof(len));
   }
}


char* getIPStringFromInt(uint32_t ipAddr)
{
   static char buffer[128];
   memset(buffer, 0, sizeof(char)*128);

   sprintf((char*)buffer,"%d.%d.%d.%d",
           (ipAddr & 0xff000000) >> 24,
           (ipAddr & 0x00ff0000) >> 16,
           (ipAddr & 0x0000ff00) >> 8,
           (ipAddr & 0x000000ff)) ;
   return ((char*)buffer);
}

int getipv4address(char *input, NPF_uint32_t *addr)
{
   struct in_addr ipaddr;
   if(!inet_aton( input, &ipaddr)) return 1;
   *addr = ntohl( ipaddr.s_addr );
   return(0);
}
 
int getmacaddress(char *input, uint8_t *macaddr)
{
   char *endptr;
   uint32_t i;
   endptr = input;
   for( i = 0; i<6; i++ )
   {
      macaddr[i] = strtoul( endptr, &endptr, 16 );
      if((*endptr != ':') && (*endptr != 0)) return 1;
      endptr++;
   }
   return 0;
}

int getatmaddress(char *input, NPF_VccAddr_t *atmaddr)
{
   char *vcistr = NULL, *vpistr = NULL;
   vpistr = strtok(input, ",");
   vcistr = strtok(NULL,",");
   if(vcistr && vpistr)
   {
      atmaddr->VPI  = (NPF_uint16_t)atol(vpistr);
      atmaddr->VCI  = (NPF_uint16_t)atol(vcistr);
      return 0;
   }
   return 1;
}

void getTime(unsigned long long *ms)
{

    //time_t secs;
    unsigned long long millis;
    //struct tm *tmp;
    //static char time_string[32];
    millis = PIL_GetCurrentTime();
    *ms = millis;
    //secs = (time_t)millis / 1000;
    //millis = millis % 1000;
    //tmp = localtime(&secs);
    //strftime(time_string, sizeof(time_string), "%m/%d %H:%M:%S", tmp);
    //sprintf(&time_string[strlen(time_string)], ".%03d", (int)millis);
    
}
char *getTimeString()
{

    time_t secs;
    unsigned long long millis;
    struct tm *tmp;
    static char time_string[32];
    millis = PIL_GetCurrentTime();
    secs = (time_t)millis / 1000;
    millis = millis % 1000;
    tmp = localtime(&secs);
    strftime(time_string, sizeof(time_string), "%m/%d %H:%M:%S", tmp);
    sprintf(&time_string[strlen(time_string)], ".%03d", (int)millis);
    return(time_string);
}

int storeHandle(CppuiHandleType_t type, uint32_t handle)
{
    CPPUI_LOCK();
    switch(type)
    {
	case CPPUI_LAN_INTERFACE_HANDLE:
	    if(LANInterfaceHandleCount > CPPUI_MAX_HANDLES) 
	    {
		CPPUI_UNLOCK(); 
		return(1);
	    }
	    LANInterfaceHandleList[LANInterfaceHandleCount++] = handle;
	break;
	case CPPUI_POS_INTERFACE_HANDLE:
	    if(POSInterfaceHandleCount > CPPUI_MAX_HANDLES) 
	    {
		CPPUI_UNLOCK(); 
		return(1);
	    }
	    POSInterfaceHandleList[POSInterfaceHandleCount++] = handle;
	break;
	case CPPUI_IPV4_INTERFACE_HANDLE:
	    if(IPV4InterfaceHandleCount > CPPUI_MAX_HANDLES) 
	    {
		CPPUI_UNLOCK(); 
		return(1);
	    }
	    IPV4InterfaceHandleList[IPV4InterfaceHandleCount++] = handle;
	break;
        /* IPV4 Handling */ 
        case CPPUI_IPV4_FWD_HANDLE:
	    if(IPV4FwdHandleCount > CPPUI_MAX_HANDLES) 
	    {
		CPPUI_UNLOCK(); 
		return(1);
	    }
	    IPV4FwdHandleList[IPV4FwdHandleCount++] = handle;
	break;
        case CPPUI_IPV4_PREFIX_HANDLE:
	    if(IPV4PrefixHandleCount > CPPUI_MAX_HANDLES) 
	    {
		CPPUI_UNLOCK(); 
		return(1);
	    }
	    IPV4PrefixHandleList[IPV4PrefixHandleCount++] = handle;
	break;
        case CPPUI_IPV4_NEXTHOP_HANDLE:
	    if(IPV4NexthopHandleCount > CPPUI_MAX_HANDLES) {
		CPPUI_UNLOCK(); 
		return(1);
	    }
	    IPV4NexthopHandleList[IPV4NexthopHandleCount++] = handle;
	break;
        case CPPUI_IPV4_ADDRESS_HANDLE:
	    if(IPV4AddressHandleCount > CPPUI_MAX_HANDLES) 
	    {
		CPPUI_UNLOCK(); 
		return(1);
	    }
	    IPV4AddressHandleList[IPV4AddressHandleCount++] = handle;
	break;
        case CPPUI_IPV4_FIB_HANDLE:
	    if(IPV4FibHandleCount > CPPUI_MAX_HANDLES) 
	    {
		CPPUI_UNLOCK(); 
		return(1);
	    }
	    IPV4FibHandleList[IPV4FibHandleCount++] = handle;
	break;
	#ifdef MPLS
	case CPPUI_MPLS_INTERFACE_HANDLE:
	    if(MPLSInterfaceHandleCount > CPPUI_MAX_HANDLES) 
	    {
		CPPUI_UNLOCK(); 
		return(1);
	    }
	    MPLSInterfaceHandleList[MPLSInterfaceHandleCount++] = handle;
	break;
	case CPPUI_MPLS_LSP_HANDLE:
	    if(MPLSLSPHandleCount > CPPUI_MAX_HANDLES) 
	    {
		CPPUI_UNLOCK(); 
		return(1);
	    }
	    MPLSLSPHandleList[MPLSLSPHandleCount++] = handle;
	break;
	case CPPUI_MPLS_NHLFE_HANDLE:
	    if(MPLSNhlfeHandleCount > CPPUI_MAX_HANDLES) 
	    {
		CPPUI_UNLOCK(); 
		return(1);
	    }
	    MPLSNhlfeHandleList[MPLSNhlfeHandleCount++] = handle;
	break;
	case CPPUI_MPLS_NHLFE_SET_HANDLE:
	    if(MPLSNhlfeSetHandleCount > CPPUI_MAX_HANDLES) 
	    {
		CPPUI_UNLOCK(); 
		return(1);
	    }
	    MPLSNhlfeSetHandleList[MPLSNhlfeSetHandleCount++] = handle;
	break;
	#endif
	#ifdef ATM
	case CPPUI_ATM_INTERFACE_HANDLE:
	    if(ATMInterfaceHandleCount > CPPUI_MAX_HANDLES) 
	    {
		CPPUI_UNLOCK(); 
		return(1);
	    }
	    ATMInterfaceHandleList[ATMInterfaceHandleCount++] = handle;
	break;
	#endif
	#ifdef DIFFSERV
	case CPPUI_CLASSIFIER_HANDLE:
	    if(ClassifierHandleCount > CPPUI_MAX_HANDLES) 
	    {
		CPPUI_UNLOCK(); 
		return(1);
	    }
	    ClassifierHandleList[ClassifierHandleCount++] = handle;
	break;
	case CPPUI_METER_HANDLE:
	    if(MeterHandleCount > CPPUI_MAX_HANDLES) 
	    {
		CPPUI_UNLOCK(); 
		return(1);
	    }
	    MeterHandleList[MeterHandleCount++] = handle;
	break;
	case CPPUI_MARKER_HANDLE:
	    if(MarkerHandleCount > CPPUI_MAX_HANDLES) 
	    {
		CPPUI_UNLOCK(); 
		return(1);
	    }
	    MarkerHandleList[MarkerHandleCount++] = handle;
	break;
	case CPPUI_DROPPER_HANDLE:
	    if(DropperHandleCount > CPPUI_MAX_HANDLES) 
	    {
		CPPUI_UNLOCK(); 
		return(1);
	    }
	    DropperHandleList[DropperHandleCount++] = handle;
	break;
	#endif
	default:
	{
	    CPPUI_UNLOCK(); 
	    return(1);
	}
    }
    CPPUI_UNLOCK();
    return(0);
}

int getHandle(char *input)
{
    int  index = 0;
    char *tmp = NULL;
    tmp = strchr(input, ':');
    if(tmp == NULL) return((int)atol(input));
    CPPUI_LOCK();
    index = (int)atol(tmp + sizeof(char));
    if(index > CPPUI_MAX_HANDLES)
    {
        CPPUI_UNLOCK();
	return(index);
    }
    if(!strncmp(input, "PREFIX", strlen("PREFIX"))) 
    {
        CPPUI_UNLOCK();
	     return(IPV4PrefixHandleList[index]);
    }

    if(!strncmp(input, "NEXTHOP", strlen("NEXTHOP"))) 
    {
        CPPUI_UNLOCK();
	     return(IPV4NexthopHandleList[index]);
    }
    
    if(!strncmp(input, "ADDRESS", strlen("ADDRESS"))) 
    {
        CPPUI_UNLOCK();
	return(IPV4AddressHandleList[index]);
    }
    
    if(!strncmp(input, "FIB", strlen("FIB"))) 
    {
        CPPUI_UNLOCK();
	return(IPV4FibHandleList[index]);
    }
    
    if(!strncmp(input, "FWD", strlen("FWD"))) 
    {
        CPPUI_UNLOCK();
	     return(IPV4FwdHandleList[index]);
    }
    
    if(!strncmp(input, "LAN", strlen("LAN"))) 
    {
        CPPUI_UNLOCK();
	return(LANInterfaceHandleList[index]);
    }
    if(!strncmp(input, "POS", strlen("POS"))) 
    {
        CPPUI_UNLOCK();
	return(POSInterfaceHandleList[index]);
    }
    else if(!strncmp(input, "IPV4", strlen("IPV4"))) 
    {
        CPPUI_UNLOCK();
	return(IPV4InterfaceHandleList[index]);
    }
    #ifdef ATM
    else if(!strncmp(input, "ATM", strlen("ATM")))
    {
        CPPUI_UNLOCK();
	return(ATMInterfaceHandleList[index]);
    }
    #endif
    #ifdef MPLS
    else if(!strncmp(input, "MPLS", strlen("MPLS"))) 
    {
        CPPUI_UNLOCK();
	return(MPLSInterfaceHandleList[index]);
    }
    else if(!strncmp(input, "LSP", strlen("LSP")))
    {
        CPPUI_UNLOCK();
	return(MPLSLSPHandleList[index]);
    }
    else if(!strncmp(input, "NHLFE", strlen("NHLFE")))
    {
        CPPUI_UNLOCK();
	return(MPLSNhlfeHandleList[index]);
    }
    else if(!strncmp(input, "NHLFESET", strlen("NHLFESET")))
    {
        CPPUI_UNLOCK();
	return(MPLSNhlfeSetHandleList[index]);
    }
    #endif
    #ifdef DIFFSERV
    else if(!strncmp(input, "CLASSIFIER", strlen("CLASSIFIER")))
    {
        CPPUI_UNLOCK();
	return(ClassifierHandleList[index]);
    }
    else if(!strncmp(input, "METER", strlen("METER")))
    {
        CPPUI_UNLOCK();
	return(MeterHandleList[index]);
    }
    else if(!strncmp(input, "MARKER", strlen("MARKER")))
    {
        CPPUI_UNLOCK();
	return(MarkerHandleList[index]);
    }
    else if(!strncmp(input, "DROPPER", strlen("DROPPER")))
    {
        CPPUI_UNLOCK();
	return(DropperHandleList[index]);
    }
    #endif
    CPPUI_UNLOCK();
    return(index);
}

int Reset_IPV4_Handles()
{
   IPV4PrefixHandleCount = 0;
   IPV4NexthopHandleCount = 0;
   IPV4AddressHandleCount = 0;
   IPV4FibHandleCount = 0;
   IPV4FwdHandleCount = 0;
   
   return(0);
}

#ifdef MPLS
int Reset_MPLS_Handles()
{
    MPLSInterfaceHandleCount = 0;
    MPLSLSPHandleCount = 0;
    MPLSNhlfeHandleCount = 0;
    MPLSNhlfeSetHandleCount = 0;
    return 0;
}
#endif /* MPLS */

#ifdef DIFFSERV
int Reset_DS_Handles()
{
    ClassifierHandleCount = 0;
    MeterHandleCount = 0;
    MarkerHandleCount = 0;
    DropperHandleCount = 0;
    return(0);
}
#endif /* MPLS */

int Reset_IF_Handles()
{
    LANInterfaceHandleCount = 0;
    POSInterfaceHandleCount = 0;
    return(0);
}
