/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor, Release 5
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      FP Module Core Component
 *
 * = FILENAME
 *      ix_cc_fpm_core.h
 *
 * = DESCRIPTION
 *      This file defines all exported data types and function prototypes
 *      for the FP Module ingress Core Component. This will run on the ingress 
 *      side of the IXP 2400/2800 (Angel Island platform).
 *
 * = AUTHOR
 *       Rajeev Muralidhar
 *       Rajeev.D.Muralidhar@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      08/16/2002 11:45 AM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: ix_cc_fpm_core.h,v 1.7.2.1 2004/03/24 11:39:52 smds Exp $
 */

#if !defined(__IX_CC_FPM_H__)
#define __IX_CC_FPM_H__

#include "ix_cc_error.h"
#if (_IX_OS_TYPE_ != _IX_OS_LINUX_KERNEL_)
#include <stdio.h>
#include <string.h>
#endif
#include "ix_ossl.h"
#include "ix_rm.h"
#include "ix_cc.h"
#include "ix_cc_properties.h"
#include "cc/ix_cc_msup.h"
#include "ix_netmacros.h"

#include "cc/ix_cc_l2tm.h"
#include "bindings.h"

#if (_IX_OS_TYPE_ != _IX_OS_LINUX_KERNEL_)
#include "cc/ix_cc_fpm_common.h"
#endif

#if !defined (IX_EXCLUDE_CCI)
#include "ix_cci.h"
#endif

#include "cc/ix_cc_arp.h"
#include "cc/ix_cc_eth_tx.h"
#include "cc/ix_cc_eth_rx.h"

#ifdef IX_CC_QOS
#include "cc/ix_cc_tc_meter_msg_helpers.h"
#include "cc/internal/ix_cc_wred_api.h"
#include "cc/ix_cc_wred_msghlp.h"
#endif /* IX_CC_QOS */

#ifdef IX_CC_ATM
#include "ix_cc_atmsar.h"
#include "ix_cc_llc_snap_encap.h"
#endif /* IX_CC_ATM */

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

/* Macros etc. */
#ifdef IX_DEBUG
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
#define FPM_CC_DBG_PRINT(fmt, args...) ix_ossl_message_log (fmt, ##args)
#define FPM_CC_DBG_TRACE() ix_ossl_message_log ("Entering %s \n", __FUNCTION__)
#else
#define FPM_CC_DBG_PRINT(fmt, args...) printf (fmt, ##args)
#define FPM_CC_DBG_TRACE() printf ("Entering %s \n", __FUNCTION__)
#endif
#else
#define FPM_CC_DBG_PRINT(fmt, args...)  /* nothing */
#define FPM_CC_DBG_TRACE() /* nothing */
#endif

/**
 * TYPENAME: ix_cc_fpm_messages
 * 
 * DESCRIPTION: This structure defines the different FP Module messages
 *
 */
typedef enum ix_e_cc_fpm_messages
{

    /* L2 Table operations */
    IX_CC_FPM_MSG_ADD_L2_ENTRY = IX_CC_COMMON_MSG_ID_LAST,
    IX_CC_FPM_MSG_DEL_L2_ENTRY,
    /* ARP operations */
    IX_CC_FPM_MSG_ADD_ARP_ENTRY,
    IX_CC_FPM_MSG_DEL_ARP_ENTRY,
    IX_CC_FPM_MSG_PURGE_ARP_TABLE,
    /* Events and exceptions */
    IX_CC_FPM_MSG_NO_DIRECT_HOST_ROUTE,
    /* Statistics messages */
    IX_CC_FPM_MSG_ETH_TX_STATS,

    /* DiffServ Messages */
    IX_CC_FPM_MSG_DPE_OP,

    /* ATM Messages */
    IX_CC_FPM_MSG_ATM_POS_TX_STATS,
    IX_CC_FPM_MSG_ATM_PORT_CREATE,
    IX_CC_FPM_MSG_ATM_PORT_DELETE,
    IX_CC_FPM_MSG_ATM_PORT_STATS_GET,
    IX_CC_FPM_MSG_ATM_VCC_CREATE,
    IX_CC_FPM_MSG_ATM_VCC_UPDATE, 
    IX_CC_FPM_MSG_ATM_VCC_DELETE, 
    IX_CC_FPM_MSG_ATM_VCC_STATS_GET, 
    IX_CC_FPM_MSG_ATM_CID_CREATE, 
    IX_CC_FPM_MSG_ATM_CID_DELETE, 
    IX_CC_FPM_MSG_ATM_CID_STATS_GET, 

    /* LLC Snap Encap Next Hop Add message */
    IX_CC_FPM_MSG_LLC_SNAP_ENCAP_ADD_NEXT_HOP,
    IX_CC_FPM_MSG_LLC_SNAP_ENCAP_DEL_NEXT_HOP,
    /* Add new messages before this */

    IX_CC_FPM_MSG_LAST     
} ix_cc_fpm_messages;

/**
 * TYPENAME: ix_cc_fpm_error_code
 * 
 * DESCRIPTION: This enumerated type will describe the FPModule core component's
 *              error codes
 *
 */

typedef enum ix_e_cc_fpm_error_code
{
    IX_CC_FPM_ERROR_FIRST = IX_ERROR_MAKE_GROUP(CC_FPM),
    IX_CC_FPM_ERROR_SUCCESS = IX_CC_FPM_ERROR_FIRST,

    IX_CC_FPM_ERROR_CCI,
    IX_CC_FPM_ERROR_INVALID_CONTEXT,
    IX_CC_FPM_ERROR_INVALID_INPUT_PARAM,
    IX_CC_FPM_ERROR_INVALID_BUFFER,
    IX_CC_FPM_ERROR_MSG_LIBRARY,
    IX_CC_FPM_ERROR_DUPLICATE_NEXT_HOP,
    IX_CC_FPM_ERROR_NEXT_HOP_ID_IN_USE,
    IX_CC_FPM_ERROR_CALLBACK,
    IX_CC_FPM_ERROR_BUFFER_FREE,
    IX_CC_FPM_ERROR_ADD_HANDLER,
    IX_CC_FPM_ERROR_REPOSITORY_OPEN,
    IX_CC_FPM_ERROR_REPOSITORY_READ,
    IX_CC_FPM_ERROR_PACKET_SEND,
    IX_CC_FPM_ERROR_L2TM,
    IX_CC_FPM_ERROR_ARP,

    /* unknown error code */
    IX_CC_FPM_ERROR_UNKNOWN,
    IX_CC_FPM_ERROR_LAST
} ix_cc_fpm_error_code;

/**
 * TYPENAME: ix_cc_fpm_add_l2_entry_data
 * 
 * DESCRIPTION: This structure defines the structure for adding L2 entries.
 *              This is passed across the message support library between the 
 *              ingress and egress FP Module core components.
 *
 */
typedef struct ix_s_cc_fpm_add_l2_entry_data
{
    ix_uint32        l2_id;
    ix_uint32        output_port;
    ix_cc_l2tm_entry l2tm_info;
} ix_cc_fpm_add_l2_entry_data;


#ifdef IX_CC_QOS
typedef enum ix_s_fpm_dpe_op
{
    IX_CC_FPM_DPE_ADD,
    IX_CC_FPM_DPE_UPDATE,
    IX_CC_FPM_DPE_REMOVE,
    IX_CC_FPM_DPE_STATS
}ix_cc_fpm_dpe_op;

typedef enum ix_s_fpm_dpe_type
{
    IX_CC_FPM_DPE_DSCP_CLASSIFIER,
    IX_CC_FPM_DPE_METER,
    IX_CC_FPM_DPE_MARKER,
    IX_CC_FPM_DPE_WRED_DROPPER
}ix_cc_fpm_dpe_type;

/**
 * TYPENAME: ix_s_cc_fpm_ix_cc_fpm_dpe
 * 
 * DESCRIPTION: This structure defines the structure for DPE operations
 *              This is passed across the message support library between the 
 *              ingress and egress FP Module core components.
 *
 */
typedef struct ix_s_cc_fpm_ix_cc_fpm_dpe
{
    ix_cc_fpm_dpe_op   op;	/*Add, Update, Remove, statistics */
    ix_cc_fpm_dpe_type type;	/* DPE type */
    ix_uint32	       fid;	/* Flow Id */
    ix_uint16	       port;	/* Port Id */
    ix_uint8	       dscp;	/* Port Id */
    union
    {
	ix_cc_tc_meter_parameters	tc_meter; /* DSCP classifier parameters */
	ix_cc_wred_parameters		wred_dropper; /* WRED dropper parameters */
	ix_cc_wred_statistics		wred_stats; /* WRED statistics */
	ix_cc_tc_meter_statistics	meter_stats; /* tc-meter statistics */
    }dpe;
} ix_cc_fpm_dpe;
#endif /* IX_CC_QOS */

/**
 * TYPENAME: ix_cc_fpm_link_status
 * 
 * DESCRIPTION: This enumerated type will describe the link status of a physical
 *              port
 *
 */
typedef enum ix_e_cc_fpm_link_status
{
    IX_CC_FPM_LINK_DOWN,
    IX_CC_FPM_LINK_UP
} ix_cc_fpm_link_status;



#ifdef IX_CC_ATM
/*************** Data Types used for ATM *******************************/

/**
 * TYPENAME: ix_cc_fpm_port_descr_t
 * 
 * DESCRIPTION: This data type describes the Attributes of a physical ATM port
 *
 */
typedef struct ix_cc_fpm_atm_port_descr ix_cc_fpm_atm_port_descr_t;

struct ix_cc_fpm_atm_port_descr
{
	ix_cc_atmsar_instance_t arg_sarInstance;
	ix_cc_atmsar_port_descr_t arg_pPortDescr;
};


/**
 * TYPENAME: ix_cc_fpm_port_create_cb_t
 * 
 * DESCRIPTION: This data types stores the callback function for ATM port 
 *					 create call 
 *
 */
typedef struct ix_cc_fpm_atm_port_create_cb ix_cc_fpm_atm_port_create_cb_t;

struct ix_cc_fpm_atm_port_create_cb
{
	ix_cc_atmsar_cb_port_create_t arg_Callback;
	void *arg_pUserContext;
};


/**
 * TYPENAME: ix_cc_fpm_atm_port_create_resp_t
 * 
 * DESCRIPTION: This data types stores the status of the ATM port create 
 *					 call 
 *
 */
typedef struct ix_cc_fpm_atm_port_create_resp ix_cc_fpm_atm_port_create_resp_t;

struct ix_cc_fpm_atm_port_create_resp
{
	ix_error arg_Result;
	ix_cc_atmsar_port_handle_t port;
};

/**
 * TYPENAME: ix_cc_fpm_port_handle_t
 * 
 * DESCRIPTION: This data type describes the handle of a physical ATM port
 *
 */
typedef struct ix_cc_fpm_atm_port_handle ix_cc_fpm_atm_port_handle_t;

struct ix_cc_fpm_atm_port_handle
{
	ix_cc_atmsar_instance_t arg_sarInstance;
	ix_cc_atmsar_port_handle_t arg_port;
};

/**
 * TYPENAME: ix_cc_fpm_atm_port_del_cb_t
 * 
 * DESCRIPTION: This data types stores the callback function for ATM port 
 *					 delete call 
 *
 */
typedef struct ix_cc_fpm_atm_port_del_cb ix_cc_fpm_atm_port_del_cb_t;

struct ix_cc_fpm_atm_port_del_cb
{
	ix_cc_atmsar_cb_port_remove_t arg_Callback;
	void *arg_pUserContext;
};

/**
 * TYPENAME: ix_cc_fpm_atm_port_stats_get_cb_t
 * 
 * DESCRIPTION: This data types stores the callback function for ATM port 
 *					 statistics get call 
 *
 */
typedef struct ix_cc_fpm_atm_port_stats_get_cb ix_cc_fpm_atm_port_stats_get_cb_t;

struct ix_cc_fpm_atm_port_stats_get_cb
{
	ix_cc_atmsar_cb_get_port_stats_t arg_Callback;
	void *arg_pUserContext;
};


/**
 * TYPENAME: ix_cc_fpm_atm_port_stats_get_resp_t
 * 
 * DESCRIPTION: This data types stores the status of the ATM port Statistics 
 *					 get call 
 *
 */
typedef struct ix_cc_fpm_atm_port_stats_get_resp ix_cc_fpm_atm_port_stats_get_resp_t;

struct ix_cc_fpm_atm_port_stats_get_resp
{
	ix_error arg_Result;
	ix_cc_atmsar_port_stats_t arg_pLookupResult;
};


/**
 * TYPENAME: ix_cc_fpm_atm_vc_descr_t
 * 
 * DESCRIPTION: This data type describes the Attributes of a ATM VCC 
 *
 */
typedef struct ix_cc_fpm_atm_vc_descr ix_cc_fpm_atm_vc_descr_t;

struct ix_cc_fpm_atm_vc_descr
{
	ix_cc_atmsar_instance_t arg_sarInstance;
	ix_cc_atmsar_vc_descr_t arg_pVcDescr;
};


/**
 * TYPENAME: ix_cc_fpm_vcc_create_cb_t
 * 
 * DESCRIPTION: This data types stores the callback function for ATM VCC 
 *					 create call 
 *
 */
typedef struct ix_cc_fpm_atm_vc_create_cb ix_cc_fpm_atm_vc_create_cb_t;

struct ix_cc_fpm_atm_vc_create_cb
{
	ix_cc_atmsar_cb_vc_create_t arg_Callback;
	void *arg_pUserContext;
};

/**
 * TYPENAME: ix_cc_fpm_atm_vc_create_resp_t
 * 
 * DESCRIPTION: This data types stores the status of the ATM VCC create 
 *					 call 
 *
 */
typedef struct ix_cc_fpm_atm_vc_create_resp ix_cc_fpm_atm_vc_create_resp_t;

struct ix_cc_fpm_atm_vc_create_resp
{
	ix_error arg_Result;
	ix_cc_atmsar_vc_handle_t arg_hVc;
};


/**
 * TYPENAME: ix_cc_fpm_atm_vc_update_t
 * 
 * DESCRIPTION: This data type describes the Attributes of a ATM VCC 
 *
 */
typedef struct ix_cc_fpm_atm_vc_update ix_cc_fpm_atm_vc_update_t;

struct ix_cc_fpm_atm_vc_update
{
	ix_cc_atmsar_instance_t arg_sarInstance;
	ix_cc_atmsar_vc_handle_t arg_hVc;
	ix_cc_atmsar_update_data_t arg_pUpdateData;
};


/**
 * TYPENAME: ix_cc_fpm_atm_vc_update_cb_t
 * 
 * DESCRIPTION: This data types stores the callback function for ATM VCC 
 *					 create call 
 *
 */
typedef struct ix_cc_fpm_atm_vc_update_cb ix_cc_fpm_atm_vc_update_cb_t;

struct ix_cc_fpm_atm_vc_update_cb
{
	ix_cc_atmsar_cb_vc_update_t arg_Callback;
	void *arg_pUserContext;
};


/**
 * TYPENAME: ix_cc_fpm_atm_vc_handle_t
 * 
 * DESCRIPTION: This data type describes the Attributes of a ATM VCC 
 *
 */
typedef struct ix_cc_fpm_atm_vc_handle ix_cc_fpm_atm_vc_handle_t;

struct ix_cc_fpm_atm_vc_handle
{
	ix_cc_atmsar_instance_t arg_sarInstance;
	ix_cc_atmsar_vc_handle_t arg_hVc;
};

/**
 * TYPENAME: ix_cc_fpm_atm_vc_remove_cb_t
 * 
 * DESCRIPTION: This data types stores the callback function for ATM VCC 
 *					 delete call 
 *
 */
typedef struct ix_cc_fpm_atm_vc_remove_cb ix_cc_fpm_atm_vc_remove_cb_t;

struct ix_cc_fpm_atm_vc_remove_cb
{
	ix_cc_atmsar_cb_vc_remove_t arg_Callback;
	void *arg_pUserContext;
};


/**
 * TYPENAME: ix_cc_fpm_atm_vc_stats_get_cb_t
 * 
 * DESCRIPTION: This data types stores the callback function for ATM VCC 
 *					 Statistics Get call 
 *
 */
typedef struct ix_cc_fpm_atm_vc_stats_get_cb ix_cc_fpm_atm_vc_stats_get_cb_t;

struct ix_cc_fpm_atm_vc_stats_get_cb
{
	ix_cc_atmsar_cb_get_vc_stats_t arg_Callback;
	void *arg_pUserContext;
};


/**
 * TYPENAME: ix_cc_fpm_atm_vc_stats_get_resp_t
 * 
 * DESCRIPTION: This data types stores the status of the ATM VCC  
 *					 statistics get call 
 *
 */
typedef struct ix_cc_fpm_atm_vc_stats_get_resp ix_cc_fpm_atm_vc_stats_get_resp_t;

struct ix_cc_fpm_atm_vc_stats_get_resp
{
	ix_error arg_Result;
	ix_cc_atmsar_vc_stats_t arg_pLookupResult;
};

/*************** ATM section ends here *********************************/

/*********** LLC SNAP section ******************************************/
/**
 * TYPENAME: ix_cc_fpm_llc_snap_encap_add_next_hop_t
 * 
 * DESCRIPTION: This data type describes the Attributes of a next Hop
 *
 */
typedef struct ix_cc_fpm_llc_snap_encap_add_next_hop ix_cc_fpm_llc_snap_encap_add_next_hop_t;

struct ix_cc_fpm_llc_snap_encap_add_next_hop
{
				ix_uint16 nextHop;
				ix_uint8 encapBit;
				ix_uint16 vpi;
				ix_uint16 vci;
				ix_uint16 vcq;
				ix_uint8 dSap;
				ix_uint8 sSap;
				ix_uint8 ctrl;
				ix_uint32 oui;
				ix_uint16 pid;
};


/**
 * TYPENAME: ix_cc_fpm_llc_snap_encap_add_next_hop_cb_t
 * 
 * DESCRIPTION: This data types stores the callback function for LLC 
 *              SNAP next hop add call 
 *
 */
typedef struct ix_cc_fpm_llc_snap_encap_add_next_hop_cb ix_cc_fpm_llc_snap_encap_add_next_hop_cb_t;

struct ix_cc_fpm_llc_snap_encap_add_next_hop_cb
{
				ix_cc_llc_snap_encap_cb_add_next_hop pCallback;
				void *pUserContext;
};

/**
 * TYPENAME: ix_cc_fpm_llc_snap_encap_del_next_hop_cb_t
 * 
 * DESCRIPTION: This data types stores the callback function for LLC 
 *              SNAP next hop delete call 
 *
 */
typedef struct ix_cc_fpm_llc_snap_encap_del_next_hop ix_cc_fpm_llc_snap_encap_del_next_hop_t;

struct ix_cc_fpm_llc_snap_encap_del_next_hop
{
	ix_uint16         l2Index;
};


typedef struct ix_cc_fpm_llc_snap_encap_del_next_hop_cb ix_cc_fpm_llc_snap_encap_del_next_hop_cb_t;

struct ix_cc_fpm_llc_snap_encap_del_next_hop_cb
{
				ix_cc_llc_snap_encap_cb_delete_next_hop pCallback;
				void *pUserContext;
};



/*********** LLC SNAP section ends here *********************************/
#endif /* IX_CC_ATM */



/**
 * TYPENAME: ix_cc_fpm_l2_op_cb
 * 
 * DESCRIPTION: Type definition for general L2 callbacks (async functions).
 *
 */

typedef ix_error (*ix_cc_fpm_l2_op_cb) (ix_error arg_Result, 
					 void* arg_pContext);
#ifdef IX_CC_QOS
/**
 * TYPENAME: ix_cc_fpm_dpe_cb
 * 
 * DESCRIPTION: Type definition for DiffServ DPE callbacks (async functions).
 *
 */

typedef ix_error (*ix_cc_fpm_dpe_cb) (ix_error arg_Result, 
					 void* arg_pContext);
/**
 * TYPENAME: ix_cc_fpm_dpe_meter_stats_cb
 * 
 * DESCRIPTION: Type definition for DiffServ DPE callbacks (async functions).
 *
 */

typedef ix_error (*ix_cc_fpm_dpe_meter_stats_cb) (ix_error arg_Result, 
					 ix_cc_tc_meter_statistics *stats,
					 void* arg_pContext);
/**
 * TYPENAME: ix_cc_fpm_dpe_wred_dropper_stats_cb
 * 
 * DESCRIPTION: Type definition for DiffServ DPE callbacks (async functions).
 *
 */

typedef ix_error (*ix_cc_fpm_dpe_wred_dropper_stats_cb) (ix_error arg_Result, 
					 ix_cc_wred_statistics *stats,
					 void* arg_pContext);
#endif /* IX_CC_QOS */

/**
 * TYPENAME: ix_cc_fpm_arp_op_cb
 * 
 * DESCRIPTION: Type definition for general ARP callbacks.
 *
 */

typedef ix_error (*ix_cc_fpm_arp_op_cb) (ix_error arg_Result, 
					 void* arg_pContext);

/**
 * TYPENAME: ix_cc_fpm_eth_tx_stats_op_cb
 * 
 * DESCRIPTION: Type definition for Ethernet statistics
 *
 */

typedef ix_error (*ix_cc_fpm_eth_tx_stats_op_cb) 
     (ix_error arg_Result,
      void* arg_pContext,
      ix_uint64* arg_pBuffer);

/**
 * TYPENAME: ix_cc_fpm_atm_pos_tx_stats_op_cb
 * 
 * DESCRIPTION: Type definition for POS/ATM statistics callback
 *
 */
#ifdef IX_CC_ATM
typedef ix_error (*ix_cc_fpm_atm_pos_tx_stats_op_cb) 
     (ix_error arg_Result,
      void* arg_pContext,
      ix_uint64* arg_pBuffer);
#endif
/**
 * TYPENAME: ix_cc_fpm_arp_callback_data
 * 
 * DESCRIPTION: This structure defines callback data for ARP operations
 *
 */
typedef struct ix_s_cc_fpm_arp_callback_data
{
    ix_cc_fpm_arp_op_cb pUserCallback;
    void* pUserContext;
} ix_cc_fpm_arp_callback_data;

/**
 * TYPENAME: ix_cc_fpm_l2_callback_data
 * 
 * DESCRIPTION: This structure defines callback data for L2 operations
 *
 */
typedef struct ix_s_cc_fpm_l2_callback_data
{
    ix_cc_fpm_l2_op_cb pUserCallback;
    void* pUserContext;
} ix_cc_fpm_l2_callback_data;

/**
 * TYPENAME: ix_cc_fpm_eth_stats_callback_data
 * 
 * DESCRIPTION: This structure defines callback data for statistics
 *              retrieval.
 *
 */
typedef struct ix_s_cc_fpm_eth_stats_callback_data
{
    ix_cc_fpm_eth_tx_stats_op_cb pUserCallback;
    void* pUserContext;
} ix_cc_fpm_eth_stats_callback_data;

/**
 * TYPENAME: ix_cc_fpm_atm_pos_tx_stats_callback_data
 * 
 * DESCRIPTION: This structure defines callback data for statistics
 *              retrieval.
 *
 */
#ifdef IX_CC_ATM
typedef struct ix_s_cc_fpm_atm_pos_stats_callback_data
{
    ix_cc_fpm_atm_pos_tx_stats_op_cb pUserCallback;
    void* pUserContext;
} ix_cc_fpm_atm_pos_stats_callback_data;
#endif

#ifdef IX_CC_QOS
/**
 * TYPENAME: ix_cc_fpm_dpe_callback_data
 * 
 * DESCRIPTION: This structure defines callback data for statistics
 *              retrieval.
 *
 */
typedef struct ix_s_cc_fpm_dpe_callback_data
{
    ix_cc_fpm_dpe_cb       pUserCallback;
    ix_cc_fpm_dpe_wred_dropper_stats_cb p_wred_dropper_stats_callback;
    ix_cc_fpm_dpe_meter_stats_cb p_meter_stats_callback;
    void	           *pUserContext;
    ix_error	           error;	/* Error of DPE operation response */
    ix_cc_fpm_dpe          dpe;
} ix_cc_fpm_dpe_callback_data;
#endif /*IX_CC_QOS*/

/**
 * NAME: ix_cc_fpm_link_status_cb
 *
 * DESCRIPTION: This is the signature of the callback function registered 
 *              by the FPModule to receive link status updates.
 *  
 * @Param:  - IN  arg_BladeId: Blade id of the FP
 * @Param:  - IN  arg_PortId: port id that registered a link status change
 * @Param:  - IN  arg_LinkStatus: LINK UP or LINK DOWN
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */
typedef ix_uint32 (*ix_cc_fpm_link_status_cb) 
     (ix_uint32             arg_BladeId,
      ix_uint32             arg_PortId,
      ix_cc_fpm_link_status arg_LinkState);

/**
 * NAME: ix_cc_fpm_ipv4_exception_cb
 *
 * DESCRIPTION: This is the signature of the callback function registered 
 *              by the FPModule to receive exceptions for Directly Connected
 *              Hosts - a NO_DIRECT_HOST_ROUTE_EXCEPTION is delivered to the CP;
 *              subsequently an exact match route is downloaded to the FP. This
 *              is an IPv4 exception since this packet was unrouteable.
 *  
 * @Param:  - IN  arg_BladeId: Blade id of the FP
 * @Param:  - IN  arg_PortId: outport port that the packet was to be sent on
 * @Param:  - IN  arg_DestIP: Destination IP address of the packet - this would
 *                            be a directly connected host.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */
typedef ix_uint32 (*ix_cc_fpm_ipv4_exception_cb) 
     (ix_uint32 arg_BladeId,
      ix_uint32 arg_PortId,
      ix_uint32 arg_DestIP);

/**
 * NAME: ix_cc_fpm_pkt_hdlr_cb
 *
 * DESCRIPTION: This is the signature of the callback function registered 
 *              by the FPModule to receive packets destined to the control
 *              plane.
 *  
 * @Param:  - IN  arg_BladeId: Blade id of the FP
 * @Param:  - IN  arg_PortId: port id that recvd a packet
 * @Param:  - IN  arg_Packet: buffer (PDU) that was recvd
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */
typedef ix_uint32 (*ix_cc_fpm_pkt_hdlr_cb)
     (ix_uint32 arg_BladeId,
      ix_uint32 arg_PortId,
      ix_uint32 arg_PacketSize,
      char*     arg_Packet);

/**
 * TYPENAME: ix_cc_fpm_context
 * 
 * DESCRIPTION: This structure defines the FP Module core component context.
 *              This is initialized during the CC initialization, and is stored
 *              in the CC context that is passed into the init() function. During
 *              shutdown, the fini() function passes this context into the CC, so
 *              that corresponding shutdown can be performed. 
 *
 */
typedef struct ix_s_cc_fpm_context
{
    ix_cc_handle                fpm_hFPMHandle; /* Handle of FPModule CC */
    ix_uint32                   fpm_localBladeId; /* Local Blade id */
    /* handle to freelist of ix_buffers used to contain packets. */
    ix_buffer_free_list_handle  fpm_hFreeList;  
    ix_cc_fpm_link_status_cb    fpm_linkState_cb;
    ix_cc_fpm_ipv4_exception_cb fpm_ipv4Exception_cb;
    ix_cc_fpm_pkt_hdlr_cb       fpm_pktHdlr_cb;
} ix_cc_fpm_context;

/****************************************************************************
 *               INIIALIZATION, SHUTDOWN, MESSAGE HANDLER                   *
 ***************************************************************************/
/**
 * NAME: ix_cc_fpm_init
 *
 * DESCRIPTION: This primitive shall be called and returned successfully before 
 *              requesting any service from FPModule Core component. This 
 *              primitive should be called only once.
 *              On the ingress, this function performs all initializations 
 *              required by a core component (registering msg handler). NO 
 *              packet handler is required by this component. Also, this does 
 *              not have any micro block component, so symbol patching is done. 
 *              On the egress, other than the above basic initialization, it 
 *              initializes the L2 Table Manager.
 * 
 * @Param:  - IN arg_hCcHandle - handle to FP Module core component, created by 
 *            core component infrastructure; this shall be used later to get 
 *            other services from core component infrastructure.
 *
 * @Param:  - OUT arg_ppContext - location where the pointer to the control 
 *            block allocated by FP Module core component will be stored. 
 *            The control block is internal to Forwarder core component and it 
 *            contains information about internal data structures, allocated 
 *            memory and other relevant information. Later this pointer to be 
 *            passed into ix_cc_fpm_fini function for freeing memory and other 
 *            house keeping operations when FPM component is being destroyed.  
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOM   - memory allocation failure      
  * IX_CC_FPM_ERROR_CCI - failure from CCI 
 * 
 */

ix_error ix_cc_fpm_init(
			ix_cc_handle arg_hCcHandle,
			void** arg_ppContext
                        );


/**
 * NAME: ix_cc_fpm_fini
 *
 * DESCRIPTION: This primitive shall be called to terminate services from 
 *              FP Module Core component. This primitive frees memory allocated 
 *              during initialization, any other cleaning up that needs to be
 *              done. On the egress, it shuts down the L2 Table Manager.
 *              If client claims for any services from FPModule CC after calling
 *              this function, the behavior is undefined.
 * 
 * @Param:  - IN arg_hCcHandle - handle to FP Module core component
 *
 * @Param:  - IN arg_pContext - pointer to the control block memory was 
 *            allocated earlier in ix_cc_fpm_init function. 
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_INVALID_INPUT_PARAM - invalid input parameter(s)
 * IX_CC_ERROR_OOM   - failure of memory deletion                
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * 
 */
ix_error ix_cc_fpm_fini(
                         ix_cc_handle arg_hCcHandle,
                         void* arg_pContext
			 );


/**
 * NAME: ix_cc_fpm_msg_handler
 *
 * DESCRIPTION: This primitive is the message handler function for FP Module
 *              core component. FP Module ingress core component receives 
 *              messages from its egress counterpart, and vice versa. 
 *              through this message handler function and internally it calls 
 *              appropriate library function to process the message. 
 * 
 * @Param:  - IN arg_hDataToken - buffer handle embedding information for the 
 *            message passed in arg_UserData.
 *
 * arg_UserData - message type
 *
 * @Param:  - OUT - arg_pComponentContext: pointer to FP Module core component 
 *            context that will be passed to the core component when a message 
 *            arrives. This context was defined by the core component and passed
 *            to core components infrastructure through ix_cc_fpm_init function.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - null argument
 * IX_CC_FPM_ERROR_INVALID_CONTEXT - context is unusable
 * IX_CC_ERROR_UNDEFINED_MSG - un supported message
 * IX_CC_FPM_ERROR_MSG_LIBRARY - error from message support library
 * 
 */
ix_error ix_cc_fpm_msg_handler (
				ix_buffer_handle arg_hDataToken,
				ix_uint32 arg_UserData,
				void* arg_pContext
				);

/**
 * NAME: ix_cc_fpm_pkt_handler
 *
 * DESCRIPTION: This primitive is the packet handler function for FP Module
 *              core component. FP Module ingress core component receives 
 *              packets destined for remote control plane (protocol packets)
 *              from the stack driver; these packets will then be tunneled
 *              to the control plane.
 * 
 * @Param:  - IN ix_buffer_handle arg_hDataToken - buffer handle containing
 *                                                 the packet
 * @Param:  - IN ix_uint32 arg_ExceptionCode - Exception code for the packet.
 * @Param:  - IN void* arg_ComponentContext - pointer to FPModule core component
 *               context. This context was defined by the core component and 
 *               passed to CCI through ix_cc_fpm_init function.
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 */
ix_error ix_cc_fpm_pkt_handler (
				ix_buffer_handle arg_hDataToken,
				ix_uint32 arg_ExceptionCode,
				void* arg_pContext
				);

/****************************************************************************
 *               MESSAGE API (FOR EXTERNAL CLIENTS)                         *
 ***************************************************************************/
/**
 * NAME: ix_cc_fpm_sync_add_l2_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to add a L2 table 
 *              entry into the L2 Table maintained on the egress.
 *              This is a synchronous function that will block till the result
 *              is available or a timeout occurs.
 * 
 * @Param:  - IN ix_uint32 arg_L2Index: L2 Id being added
 * @Param:  - IN ix_uint32 arg_OutputPort: Port on which NH IP is connected
 * @Param:  - IN arg_L2_info: L2 Info being added. Defined by L2 Table Manager
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_sync_add_l2_entry (
				      ix_uint32         arg_L2Index,
				      ix_uint32         arg_OutputPort,
				      ix_cc_l2tm_entry* arg_L2_info
				      ); 
/**
 * NAME: ix_cc_fpm_sync_del_l2_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to del a L2 table 
 *              entry from the L2 Table maintained on the egress.
 *              This is a synchronous function that will block till the result
 *              is available or a timeout occurs.
 * 
 * @Param:  - IN arg_L2Id: L2 Id being removed
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_sync_del_l2_entry (
				      ix_uint32 arg_L2Id
				      );

/**
 * NAME: ix_cc_fpm_async_add_l2_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to add a L2 table 
 *              entry into the L2 Table maintained on the egress.
 *              This is an asynchronous function that will return the result in a
 *              subsequent callback.
 * 
 * @Param:  - IN ix_uint32 arg_L2Index: L2 Id being added
 * @Param:  - IN ix_uint32 arg_OutputPort: Port on which NH IP is connected
 * @Param:  - IN arg_L2_info: L2 Info being added. Defined by L2 Table Manager
 * @Param:  - IN arg_pCallback: Callback function to be invoked.
 * @Param:  - IN arg_pContext: User context.
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_async_add_l2_entry (
				       ix_uint32          arg_L2Index,
				       ix_uint32          arg_OutputPort,
				       ix_cc_l2tm_entry*  arg_L2_info,
				       ix_cc_fpm_l2_op_cb arg_Callback,
				       void*              arg_pUserContext
				      ); 
/**
 * NAME: ix_cc_fpm_async_del_l2_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to del a L2 table 
 *              entry from the L2 Table maintained on the egress.
 *              This is an asynchronous function that will return the result of
 *              the operation in a subsequent callback.
 * 
 * @Param:  - IN arg_L2Id: L2 Id being removed
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_async_del_l2_entry (
				       ix_uint32          arg_L2Id,
				       ix_cc_fpm_l2_op_cb arg_Callback,
				       void*              arg_pUserContext
				      );

/**
 * NAME: ix_cc_fpm_async_add_arp_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to add a ARP
 *              entry into the ARP table maintained on the egress.
 * 
 * @Param:  - IN ix_cc_eth_tx_next_hop_info arg_L2Id: ARP info being added
 *               Defined by the Ethernet Tx core component
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_async_add_arp_entry (
					ix_cc_eth_tx_next_hop_info *arg_ARPInfo,
					ix_cc_fpm_arp_op_cb arg_Callback,
					void* arg_pUserContext
					);

/**
 * NAME: ix_cc_fpm_async_del_arp_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to del a ARP
 *              entry from the ARP table maintained on the egress.
 * 
 * @Param:  - IN arg_L2Id: L2 Id being added
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_async_del_arp_entry (
					ix_uint32  arg_L2Id,
					ix_cc_fpm_arp_op_cb arg_Callback,
					void* arg_pUserContext
					);

/**
 * NAME: ix_cc_fpm_sync_purge_arp_table
 *
 * DESCRIPTION: This message helper primitive helps a client to purge the ARP
 *              table maintained on the egress. This is a fire-and-forget call.
 * 
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_sync_purge_arp_table (void);

/**
 * NAME: ix_cc_fpm_async_get_eth_tx_stats
 *
 * DESCRIPTION: This message helper primitive helps a client to retrieve
 *              Ethernet transmit statistics maintained on the egress.
 * 
 * @Param:  - IN arg_pStatsInfo: pointer to the statistics info structure
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_async_get_eth_tx_stats (
					   ix_cc_eth_tx_statistics_info_context 
					   *arg_pStatsInfo,
					   ix_cc_fpm_eth_tx_stats_op_cb 
					   arg_Callback
					   );

/**
 * NAME: ix_cc_fpm_sync_send_packet
 *
 * DESCRIPTION: This function is invoked by FP Module (or the tunnel module)
 *              to send packets from the remote CP out of the system.
 *              These packets will be converted to ix_buffer and then
 *              handed off to the stack driver.
 * 
 * @Param:  - IN ix_uint32 arg_PortId - Port on which packet is to be sent
 * @Param:  - IN char* arg_pBuffer - Char buffer containing the packet
 * @Param:  - IN ix_uint32 arg_PacketLen - Packet length
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 */
ix_error ix_cc_fpm_sync_send_packet (
				     ix_uint32 arg_PortID,
				     char*     arg_pBuffer,
				     ix_uint32 arg_PacketLen
				     );


/**
 * NAME: ix_cc_fpm_register_link_status_cb
 *
 * DESCRIPTION: This function is called by the FP Module's platform
 *              specific component to receive link status upates.
 *  
 * @Param:  - IN  arg_LinkState_cb: Link state callback function
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *          Return code is not interpreted by the core component.
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error ix_cc_fpm_register_link_status_cb (ix_cc_fpm_link_status_cb 
					    arg_LinkState_cb);

/**
 * NAME: ix_cc_fpm_deregister_link_status_cb
 *
 * DESCRIPTION: This function is called by the FP Module's platform
 *              specific component to deregister its callback
 *  
 * @Param:  - IN  - None
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *          Return code is not interpreted by the core component.
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error ix_cc_fpm_deregister_link_status_cb (void);

/**
 * NAME: ix_cc_fpm_register_packet_cb
 *
 * DESCRIPTION: This function is called by the FP Module's platform
 *              specific component to receive packets destined to the CP
 *  
 * @Param:  - IN  arg_Packet_cb: Packet receive callback function
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *          Return code is not interpreted by the core component.
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error ix_cc_fpm_register_pkt_hdlr_cb (ix_cc_fpm_pkt_hdlr_cb arg_Packet_cb);

/**
 * NAME: ix_cc_fpm_deregister_pkt_hdlr_cb
 *
 * DESCRIPTION: This function is called by the FP Module's platform
 *              specific component to deregister its callback
 *  
 * @Param:  - IN  - None
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *          Return code is not interpreted by the core component.
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error ix_cc_fpm_deregister_pkt_hdlr_cb (void);


/**
 * NAME: ix_cc_fpm_register_ipv4_exception_cb 
 *
 * DESCRIPTION: This function is called by the FP Module's platform
 *              specific component to receive exceptions for directly
 *              connected hosts. This is an IPv4 exception.
 *  
 * @Param:  - IN  arg_DirectHostRouteEvent_cb: Callback function that is 
 *            to be invoked for the event
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *          Return code is not interpreted by the core component.
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error ix_cc_fpm_register_ipv4_exception_cb (ix_cc_fpm_ipv4_exception_cb 
					       arg_ipv4Exception_cb);

/**
 * NAME: ix_cc_fpm_deregister_ipv4_exception_cb
 *
 * DESCRIPTION: This function is called by the FP Module's platform
 *              specific component to deregister its callback
 *  
 * @Param:  - IN  - None
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *          Return code is not interpreted by the core component.
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error ix_cc_fpm_deregister_ipv4_exception_cb (void);

#ifdef IX_CC_QOS
/**
 * NAME: ix_cc_fpm_async_dpe_op
 *
 * DESCRIPTION: This message helper primitive helps a client to perform DPE
 *              operation in the egress.
 *              This is an asynchronous function that will return the result in a
 *              subsequent callback.
 * 
 * @Param:  - IN arg_DPE_info: DPE info
 * @Param:  - IN arg_pCallback: Callback function to be invoked.
 * @Param:  - IN arg_pContext: User context.
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

extern ix_error ix_cc_fpm_async_dpe_op (
				       ix_cc_fpm_dpe  *arg_DPE_info,
				       ix_cc_fpm_l2_op_cb arg_Callback,
				       void*              arg_pUserContext
				      );
#endif /*IX_CC_QOS*/

#ifdef IX_CC_ATM
extern ix_error ix_cc_fpm_async_port_create(
				const ix_cc_atmsar_instance_t arg_sarInstance,
				const ix_cc_atmsar_port_descr_t* arg_pPortDescr,
				ix_cc_atmsar_cb_port_create_t arg_Callback,
				void *arg_pUserContext);

extern ix_error ix_cc_fpm_async_vc_create(
				const ix_cc_atmsar_instance_t arg_sarInstance,
				const ix_cc_atmsar_vc_descr_t* arg_pVcDescr,
				ix_cc_atmsar_cb_vc_create_t arg_Callback,
				void *arg_pUserContext);

extern ix_error ix_cc_fpm_async_vc_update(
				const ix_cc_atmsar_instance_t arg_sarInstance,
				ix_cc_atmsar_vc_handle_t arg_hVc,
				const ix_cc_atmsar_update_data_t* arg_pUpdateData,
				ix_cc_atmsar_cb_vc_update_t arg_Callback,
				void *arg_pUserContext);

extern ix_error ix_cc_fpm_async_vc_remove(
				const ix_cc_atmsar_instance_t arg_sarInstance,
				const ix_cc_atmsar_vc_handle_t arg_hVc,
				ix_cc_atmsar_cb_vc_remove_t arg_Callback,
				void *arg_pUserContext);

extern ix_error ix_cc_fpm_async_get_vc_stats(
				const ix_cc_atmsar_instance_t arg_sarInstance,
				ix_cc_atmsar_vc_handle_t arg_hVc,
				ix_cc_atmsar_cb_get_vc_stats_t arg_Callback,
				void *arg_pUserContext);

extern ix_error ix_cc_fpm_async_port_create(
				const ix_cc_atmsar_instance_t arg_sarInstance,
				const ix_cc_atmsar_port_descr_t* arg_pPortDescr,
				ix_cc_atmsar_cb_port_create_t arg_Callback,
				void *arg_pUserContext);

extern ix_error ix_cc_fpm_async_port_remove(
				const ix_cc_atmsar_instance_t arg_sarInstance,
				ix_cc_atmsar_port_handle_t arg_port,
				ix_cc_atmsar_cb_port_remove_t arg_Callback,
				void *arg_pUserContext);

extern ix_error ix_cc_fpm_async_get_port_stats(
				const ix_cc_atmsar_instance_t arg_sarInstance,
				ix_cc_atmsar_port_handle_t arg_port,
				ix_cc_atmsar_cb_get_port_stats_t arg_Callback,
				void *arg_pUserContext);

extern ix_error ix_cc_fpm_async_cid_create(
				const ix_cc_atmsar_instance_t arg_sarInstance,
				const ix_cc_atmsar_cid_descr_t* arg_pCidDescr,
				ix_cc_atmsar_cb_cid_create_t arg_callback,
				void *arg_pUserContext);

extern ix_error ix_cc_fpm_async_cid_remove(
				const ix_cc_atmsar_instance_t arg_sarInstance,
				ix_cc_atmsar_cid_handle_t arg_cid,
				ix_cc_atmsar_cb_cid_remove_t arg_callback,
				void *arg_pUserContext);

extern ix_error ix_cc_fpm_async_get_cid_stats(
				const ix_cc_atmsar_instance_t arg_sarInstance,
				ix_cc_atmsar_cid_handle_t arg_cid,
				ix_cc_atmsar_cb_get_cid_stats_t arg_callback,
				void *arg_pUserContext);


extern ix_error ix_cc_llc_snap_encap_fpm_add_next_hop(
				ix_uint16 nextHop,
				ix_uint8 encapBit,
				ix_uint16 vpi,
				ix_uint16 vci,
				ix_uint16 vcq,
				ix_uint8 dSap,
				ix_uint8 sSap,
				ix_uint8 ctrl,
				ix_uint32 oui,
				ix_uint16 pid,
				ix_cc_llc_snap_encap_cb_add_next_hop pCallback,
				void *pUserContext);

extern ix_error ix_cc_llc_snap_encap_fpm_del_next_hop(
				ix_uint16 nextHop,
				ix_cc_llc_snap_encap_cb_delete_next_hop pCallback,
				void *pUserContext);
#endif /*IX_CC_ATM*/



#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_FPM_H__) */
