/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 5
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      FP Module Core Component - message helpers
 *
 * = FILENAME
 *      ix_cc_fpm_msg_helpers.c
 *
 * = DESCRIPTION
 *      The file defines message helper interface functions. This contains all 
 *      the external API exported by the Core component - async/sync calls,
 *      register/deregister functions, etc.
 *
 * = AUTHOR
 *       Rajeev Muralidhar
 *       Rajeev.D.Muralidhar@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      07/08/2002
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */
/*#define IX_FILE_IDENT "$Id: ix_cc_fpm_msg_helpers.c,v 1.8.2.1 2004/03/24 11:39:53 smds Exp $";*/

#include "cc/ix_cc_fpm_core.h"

#ifdef IX_DEBUG
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
#define FPM_CC_DBG_PRINT(fmt, args...) ix_ossl_message_log (fmt, ##args) 
#define FPM_CC_DBG_TRACE() ix_ossl_message_log ("Entering %s \n", __FUNCTION__)
#else
#define FPM_CC_DBG_PRINT(fmt, args...) printf (fmt, ##args)
#define FPM_CC_DBG_TRACE() printf ("Entering %s \n", __FUNCTION__)
#endif
#else
#define FPM_CC_DBG_PRINT(fmt, args...)  /* nothing */
#define FPM_CC_DBG_TRACE() /* nothing */
#endif

/* Local functions */
#ifdef IX_CC_ETH
static void ix_cc_fpm_dump_l2_info (ix_cc_fpm_add_l2_entry_data *l2_info);
static void ix_cc_fpm_dump_arp_info (ix_cc_eth_tx_next_hop_info *arp_info);
#endif /*IX_CC_ETH*/


#ifdef IX_CC_ATM
/**
 * NAME: ix_cc_llc_snap_encap_fpm_add_next_hop_cb_core
 *
 * DESCRIPTION: This is the internal callback function for 
 *					 LLC SNAP Add Next Hop operation
 *					  
 * @Param:  - IN ix_error arg_Result - error code returned from
 *          the asynchronous operation.
 * @Param:  - IN void* arg_pContext - pointer to reply message
 *          context, 
 * @Param:  - IN void* arg_pMsg - pointer to reply message.
 * @Param:  - IN ix_uint32 arg_MsgLen - length of reply message.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_NULL
 */
ix_error ix_cc_llc_snap_encap_fpm_add_next_hop_cb_core(
			  ix_error arg_Result,
			  void* arg_pContext,
			  void* arg_pMsg,
			  ix_uint32 arg_MsgLen
			  )
{
	ix_error err = IX_SUCCESS;
	ix_error *result = NULL;
	ix_cc_fpm_llc_snap_encap_add_next_hop_cb_t *context = NULL;

	FPM_CC_DBG_TRACE ();
	result = (ix_error *)arg_pMsg;
	context = (ix_cc_fpm_llc_snap_encap_add_next_hop_cb_t *)arg_pContext;

	if(arg_Result != IX_SUCCESS)
	{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM CORE: LLC SNAP NEXT ADD CB Failed: %d\n", arg_Result); 
#else
		printf(" FPM CORE : LLC SNAP NEXT ADD CB Failed");
		ix_error_dump(stdout,arg_Result);
#endif
		return arg_Result;
	}

	if(context == NULL)
		return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
				("Internal callback invoked with NULL context"));

	/* Call user's callback function. */
	IX_ERROR_C((context->pCallback)
			(*result,context->pUserContext), err);
    
	/* Call user's callback function. */
	ix_ossl_free(context);

	return err;
}

/**
 * NAME: ix_cc_llc_snap_encap_fpm_del_next_hop_cb_core
 *
 * DESCRIPTION: This is the internal callback function for  
 *					 LLC SNAP delete next Hop operation
 * 
 * @Param:  - IN ix_error arg_Result - error code returned from
 *          the asynchronous operation.
 * @Param:  - IN void* arg_pContext - pointer to reply message
 *          context, 
 * @Param:  - IN void* arg_pMsg - pointer to reply message.
 * @Param:  - IN ix_uint32 arg_MsgLen - length of reply message.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_NULL
 */
ix_error ix_cc_llc_snap_encap_fpm_del_next_hop_cb_core(
			  ix_error arg_Result,
			  void* arg_pContext,
			  void* arg_pMsg,
			  ix_uint32 arg_MsgLen
			  )
{
	ix_error err = IX_SUCCESS;
	ix_error *result = NULL;
	ix_cc_fpm_llc_snap_encap_del_next_hop_cb_t *context = NULL;

	FPM_CC_DBG_TRACE ();
	result = (ix_error *)arg_pMsg;
	context = (ix_cc_fpm_llc_snap_encap_del_next_hop_cb_t *)arg_pContext;

	if(arg_Result != IX_SUCCESS)
	{
		printf(" FPM CORE : LLC SNAP NEXT HOP DEL CB Failed");
		ix_error_dump(stdout,arg_Result);
		return arg_Result;
	}

	if(context == NULL)
		return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
				("Internal callback invoked with NULL context"));

	/* Call user's callback function. */
	IX_ERROR_C((context->pCallback)
			(*result,context->pUserContext), err);
    
	/* Call user's callback function. */
	ix_ossl_free(context);

	return err;
}

/**
 * NAME: ix_cc_fpm_atm_port_create_cb_core
 *
 * DESCRIPTION: This is the internal callback function for ATM port
 *					 create operation
 * 
 * @Param:  - IN ix_error arg_Result - error code returned from
 *          the asynchronous operation.
 * @Param:  - IN void* arg_pContext - pointer to reply message
 *          context, 
 * @Param:  - IN void* arg_pMsg - pointer to reply message.
 * @Param:  - IN ix_uint32 arg_MsgLen - length of reply message.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_NULL
 */
ix_error ix_cc_fpm_atm_port_create_cb_core(
			  ix_error arg_Result,
			  void* arg_pContext,
			  void* arg_pMsg,
			  ix_uint32 arg_MsgLen
			  )
{
	ix_cc_fpm_atm_port_create_cb_t *context = NULL;
	ix_error err = IX_SUCCESS;
	ix_cc_fpm_atm_port_create_resp_t *result = NULL;

	FPM_CC_DBG_TRACE ();
	context = (ix_cc_fpm_atm_port_create_cb_t *)arg_pContext;
	result = (ix_cc_fpm_atm_port_create_resp_t *)arg_pMsg;

	if(arg_Result != IX_SUCCESS)
	{
#if(_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log(" FPM CORE : Port Create CB Failed: %d\n",arg_Result);
#else
		printf(" FPM CORE : Port Create CB Failed");
		ix_error_dump(stdout,arg_Result);
#endif
		return arg_Result;
	}

	if(context == NULL)
		return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
				("Internal callback invoked with NULL context"));

	/* Call user's callback function. */
	IX_ERROR_C((context->arg_Callback)
			(result->arg_Result,result->port, context->arg_pUserContext), err);
    
	/* Call user's callback function. */
	ix_ossl_free(context);

	return err;
}

/**
 * NAME: ix_cc_fpm_atm_port_del_cb_core
 *
 * DESCRIPTION: This is the internal callback function for ATM port
 *					 delete operation
 * 
 * @Param:  - IN ix_error arg_Result - error code returned from
 *          the asynchronous operation.
 * @Param:  - IN void* arg_pContext - pointer to reply message
 *          context, 
 * @Param:  - IN void* arg_pMsg - pointer to reply message.
 * @Param:  - IN ix_uint32 arg_MsgLen - length of reply message.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_NULL
 */
ix_error ix_cc_fpm_atm_port_del_cb_core(
			  ix_error arg_Result,
			  void* arg_pContext,
			  void* arg_pMsg,
			  ix_uint32 arg_MsgLen
			  )
{
	ix_cc_fpm_atm_port_del_cb_t *context = NULL;
	ix_error err = IX_SUCCESS;
	ix_error result = IX_SUCCESS;

	FPM_CC_DBG_TRACE ();
	context = (ix_cc_fpm_atm_port_del_cb_t *)arg_pContext;
	result = (ix_error)arg_pMsg;

	if(arg_Result != IX_SUCCESS)
	{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM CORE: Port Delete CB Failed: %d\n", arg_Result); 
#else
		printf(" FPM CORE : Port Delete CB Failed");
		ix_error_dump(stdout,arg_Result);
#endif

		return arg_Result;
	}

	if(context == NULL)
		return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
				("Internal callback invoked with NULL context"));

	/* Call user's callback function. */
	IX_ERROR_C((context->arg_Callback)
			(result, context->arg_pUserContext), err);
    
	/* Call user's callback function. */
	ix_ossl_free(context);

	return err;
}


/**
 * NAME: ix_cc_fpm_atm_port_stats_get_cb_core
 *
 * DESCRIPTION: This is the internal callback function for ATM port
 *					 statistics get operation
 * 
 * @Param:  - IN ix_error arg_Result - error code returned from
 *          the asynchronous operation.
 * @Param:  - IN void* arg_pContext - pointer to reply message
 *          context, 
 * @Param:  - IN void* arg_pMsg - pointer to reply message.
 * @Param:  - IN ix_uint32 arg_MsgLen - length of reply message.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_NULL
 */
ix_error ix_cc_fpm_atm_port_stats_get_cb_core(
			  ix_error arg_Result,
			  void* arg_pContext,
			  void* arg_pMsg,
			  ix_uint32 arg_MsgLen
			  )
{
	ix_cc_fpm_atm_port_stats_get_cb_t *context = NULL;
	ix_error err = IX_SUCCESS;
	ix_cc_fpm_atm_port_stats_get_resp_t *result = NULL;

	FPM_CC_DBG_TRACE ();
	context = (ix_cc_fpm_atm_port_stats_get_cb_t *)arg_pContext;
	result = (ix_cc_fpm_atm_port_stats_get_resp_t *)arg_pMsg;

	if(arg_Result != IX_SUCCESS)
	{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM CORE: Port Statistics CB Failed: %d\n", arg_Result); 
#else
		printf(" FPM CORE : Port Statistics CB Failed");
		ix_error_dump(stdout,arg_Result);
#endif

		return arg_Result;
	}

	if(context == NULL)
		return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
				("Internal callback invoked with NULL context"));

	/* Call user's callback function. */
	IX_ERROR_C((context->arg_Callback)
			(result->arg_Result,&(result->arg_pLookupResult),
			 context->arg_pUserContext), err);
    
	/* Call user's callback function. */
	ix_ossl_free(context);

	return err;
}

/**
 * NAME: ix_cc_fpm_atm_vc_create_cb_core
 *
 * DESCRIPTION: This is the internal callback function for ATM VCC 
 *					 create operation
 * 
 * @Param:  - IN ix_error arg_Result - error code returned from
 *          the asynchronous operation.
 * @Param:  - IN void* arg_pContext - pointer to reply message
 *          context, 
 * @Param:  - IN void* arg_pMsg - pointer to reply message.
 * @Param:  - IN ix_uint32 arg_MsgLen - length of reply message.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_NULL
 */
ix_error ix_cc_fpm_atm_vc_create_cb_core(
			  ix_error arg_Result,
			  void* arg_pContext,
			  void* arg_pMsg,
			  ix_uint32 arg_MsgLen
			  )
{
	ix_cc_fpm_atm_vc_create_cb_t *context = NULL;
	ix_error err = IX_SUCCESS;
	ix_cc_fpm_atm_vc_create_resp_t *result = NULL;

	FPM_CC_DBG_TRACE ();
	context = (ix_cc_fpm_atm_vc_create_cb_t *)arg_pContext;
	result = (ix_cc_fpm_atm_vc_create_resp_t *)arg_pMsg;

	if(arg_Result != IX_SUCCESS)
	{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM CORE: VC Create CB Failed: %d\n", arg_Result); 
#else
		printf(" FPM CORE : VC Create CB Failed");
		ix_error_dump(stdout,arg_Result);
#endif

		return arg_Result;
	}

	if(context == NULL)
		return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
				("Internal callback invoked with NULL context"));

	/* Call user's callback function. */
	IX_ERROR_C((context->arg_Callback)
			(result->arg_Result,result->arg_hVc, context->arg_pUserContext), err);
    
	/* Call user's callback function. */
	ix_ossl_free(context);

	return err;
}


/**
 * NAME: ix_cc_fpm_atm_vc_update_cb_core
 *
 * DESCRIPTION: This is the internal callback function for ATM VCC 
 *					 update operation
 * 
 * @Param:  - IN ix_error arg_Result - error code returned from
 *          the asynchronous operation.
 * @Param:  - IN void* arg_pContext - pointer to reply message
 *          context, 
 * @Param:  - IN void* arg_pMsg - pointer to reply message.
 * @Param:  - IN ix_uint32 arg_MsgLen - length of reply message.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_NULL
 */
ix_error ix_cc_fpm_atm_vc_update_cb_core(
			  ix_error arg_Result,
			  void* arg_pContext,
			  void* arg_pMsg,
			  ix_uint32 arg_MsgLen
			  )
{
	ix_cc_fpm_atm_vc_update_cb_t *context = NULL;
	ix_error err = IX_SUCCESS;
	ix_cc_fpm_atm_vc_create_resp_t *result = NULL;

	FPM_CC_DBG_TRACE ();
	context = (ix_cc_fpm_atm_vc_update_cb_t *)arg_pContext;
	result = (ix_cc_fpm_atm_vc_create_resp_t *)arg_pMsg;

	if(arg_Result != IX_SUCCESS)
	{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM CORE: VC Update Failed: %d\n", arg_Result); 
#else
		printf(" FPM CORE : Vc Update Failed");
		ix_error_dump(stdout,arg_Result);
#endif

		return arg_Result;
	}

	if(context == NULL)
		return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
				("Internal callback invoked with NULL context"));

	/* Call user's callback function. */
	IX_ERROR_C((context->arg_Callback)
			(result->arg_Result,result->arg_hVc, context->arg_pUserContext), err);
    
	/* Call user's callback function. */
	ix_ossl_free(context);

	return err;
}

/**
 * NAME: ix_cc_fpm_atm_vc_remove_cb_core
 *
 * DESCRIPTION: This is the internal callback function for ATM VCC 
 *					 Delete operation
 * 
 * @Param:  - IN ix_error arg_Result - error code returned from
 *          the asynchronous operation.
 * @Param:  - IN void* arg_pContext - pointer to reply message
 *          context, 
 * @Param:  - IN void* arg_pMsg - pointer to reply message.
 * @Param:  - IN ix_uint32 arg_MsgLen - length of reply message.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_NULL
 */
ix_error ix_cc_fpm_atm_vc_remove_cb_core(
			  ix_error arg_Result,
			  void* arg_pContext,
			  void* arg_pMsg,
			  ix_uint32 arg_MsgLen
			  )
{
	ix_cc_fpm_atm_vc_remove_cb_t *context = NULL;
	ix_error err = IX_SUCCESS;
	ix_error *result = NULL;

	FPM_CC_DBG_TRACE ();
	context = (ix_cc_fpm_atm_vc_remove_cb_t *)arg_pContext;
	result = (ix_error *)(arg_pMsg);

	if(arg_Result != IX_SUCCESS)
	{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM CORE: VC Remove Failed: %d\n", arg_Result); 
#else
		printf(" FPM CORE : Vc Remove Failed");
		ix_error_dump(stdout,arg_Result);
#endif

		return arg_Result;
	}

	if(context == NULL)
		return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
				("Internal callback invoked with NULL context"));

	/* Call user's callback function. */
	IX_ERROR_C((context->arg_Callback)
			(*result,context->arg_pUserContext), err);
    
	/* Call user's callback function. */
	ix_ossl_free(context);

	return err;
}



/**
 * NAME: ix_cc_fpm_atm_vc_stats_get_cb_core
 *
 * DESCRIPTION: This is the internal callback function for ATM VCC 
 *					 Statistics Get operation
 * 
 * @Param:  - IN ix_error arg_Result - error code returned from
 *          the asynchronous operation.
 * @Param:  - IN void* arg_pContext - pointer to reply message
 *          context, 
 * @Param:  - IN void* arg_pMsg - pointer to reply message.
 * @Param:  - IN ix_uint32 arg_MsgLen - length of reply message.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_NULL
 */
ix_error ix_cc_fpm_atm_vc_stats_get_cb_core(
			  ix_error arg_Result,
			  void* arg_pContext,
			  void* arg_pMsg,
			  ix_uint32 arg_MsgLen
			  )
{
	ix_cc_fpm_atm_vc_stats_get_cb_t *context = NULL;
	ix_error err = IX_SUCCESS;
	ix_cc_fpm_atm_vc_stats_get_resp_t *result = NULL;

	FPM_CC_DBG_TRACE ();
	context = (ix_cc_fpm_atm_vc_stats_get_cb_t *)arg_pContext;
	result = (ix_cc_fpm_atm_vc_stats_get_resp_t *)arg_pMsg;

	if(arg_Result != IX_SUCCESS)
	{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM CORE: VC Statistics Get Failed: %d\n", arg_Result); 
#else
		printf(" FPM CORE : Vc Statistics Get Failed");
		ix_error_dump(stdout,arg_Result);
#endif

		return arg_Result;
	}

	if(context == NULL)
		return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
				("Internal callback invoked with NULL context"));

	/* Call user's callback function. */
	IX_ERROR_C((context->arg_Callback)
			(result->arg_Result,&result->arg_pLookupResult,context->arg_pUserContext), err);
    
	/* Call user's callback function. */
	ix_ossl_free(context);

	return err;
}

#endif /*IX_CC_ATM*/

#ifdef IX_CC_ETH
ix_error fpm_ix_cc_eth_rx_async_get_statistics_info(ix_cc_eth_rx_statistics_info_context *pMgInfoContext,
					ix_cc_eth_rx_cb_get_statistics_info cm_get_generic_rx_stats_cb)
{
    return(ix_cc_eth_rx_async_get_statistics_info
                               ((ix_cc_eth_rx_statistics_info_context*)pMgInfoContext,
                                (ix_cc_eth_rx_cb_get_statistics_info)cm_get_generic_rx_stats_cb));
    return(IX_CC_ERROR_NULL);
}
#endif /* IX_CC_ETH */

#ifdef IX_CC_QOS
ix_error ix_cc_fpm_dpe_op_cb (
			  ix_error arg_Result,
			  void* arg_pContext,
			  void* arg_pMsg,
			  ix_uint32 arg_MsgLen
			  )
{
    ix_error err = IX_SUCCESS;
    ix_uint64* buffer = NULL;
    ix_cc_fpm_dpe_callback_data* pContext;

    FPM_CC_DBG_TRACE ();
    pContext = (ix_cc_fpm_dpe_callback_data*)arg_pContext;
    
    if(pContext == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
				("Internal callback invoked with NULL context"));

    switch(pContext->dpe.op)
    {
	case IX_CC_FPM_DPE_ADD:
	case IX_CC_FPM_DPE_UPDATE:
	case IX_CC_FPM_DPE_REMOVE:
	{
    	    if(pContext->pUserCallback == NULL)
	    {
    		ix_ossl_free (pContext);
        	return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("Internal callback invoked with NULL context"));
	    }
    	    IX_ERROR_C((pContext->pUserCallback)(arg_Result, pContext->pUserContext), err);
        }
	break;
	case IX_CC_FPM_DPE_STATS:
	{
	    switch(pContext->dpe.type)
	    {
		case IX_CC_FPM_DPE_METER:
		{
		    ix_cc_tc_meter_statistics   stats;
    	    	    if(pContext->p_meter_stats_callback == NULL)
	    	    {
    			ix_ossl_free (pContext);
        		return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("Internal callback invoked with NULL context"));
	    	    }
		    ix_ossl_memcpy(&stats, (ix_cc_tc_meter_statistics *)arg_pMsg, arg_MsgLen);
    	    	    IX_ERROR_C((pContext->p_meter_stats_callback)
	       		(arg_Result, &stats, pContext->pUserContext), err);
		}
		break;
		case IX_CC_FPM_DPE_WRED_DROPPER:
		{
		    ix_cc_wred_statistics   stats;
    	    	    if(pContext->p_wred_dropper_stats_callback == NULL)
	    	    {
    			ix_ossl_free (pContext);
        		return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("Internal callback invoked with NULL context"));
	    	    }
		    ix_ossl_memcpy(&stats, (ix_cc_wred_statistics *)arg_pMsg, arg_MsgLen);
    	    	    IX_ERROR_C((pContext->p_wred_dropper_stats_callback)
	       		(arg_Result, &stats, pContext->pUserContext), err);
		}
		break;
	  	default:
		{
		}
	    }

	}
	break;
	default:
	{
	}
    }
    ix_ossl_free (pContext);
    return err;
}
#endif /*IX_CC_QOS*/

/**
 * NAME: _ix_cc_fpm_l2_cb
 *
 * DESCRIPTION: This is the internal callback function for ARP operations
 * 
 * @Param:  - IN ix_error arg_Result - error code returned from
 *          the asynchronous operation.
 * @Param:  - IN void* arg_pContext - pointer to reply message
 *          context, 
 * @Param:  - IN void* arg_pMsg - pointer to reply message.
 * @Param:  - IN ix_uint32 arg_MsgLen - length of reply message.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_NULL
 */
ix_error ix_cc_fpm_l2_cb (
			  ix_error arg_Result,
			  void* arg_pContext,
			  void* arg_pMsg,
			  ix_uint32 arg_MsgLen
			  )
{
    ix_error err = IX_SUCCESS;
#ifdef IX_CC_ETH
    ix_cc_fpm_l2_callback_data* pContext;

    FPM_CC_DBG_TRACE ();
    pContext = (ix_cc_fpm_l2_callback_data*)arg_pContext;

    if(pContext == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
				("Internal callback invoked with NULL context"));

    /* Call user's callback function. */
    IX_ERROR_C((pContext->pUserCallback)
	       (arg_Result, pContext->pUserContext), err);
#endif
    return err;
}

/**
 * NAME: _ix_cc_fpm_arp_cb
 *
 * DESCRIPTION: This is the internal callback function for ARP operations
 * 
 * @Param:  - IN ix_error arg_Result - error code returned from
 *          the asynchronous operation.
 * @Param:  - IN void* arg_pContext - pointer to reply message
 *          context, 
 * @Param:  - IN void* arg_pMsg - pointer to reply message.
 * @Param:  - IN ix_uint32 arg_MsgLen - length of reply message.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_NULL
 */
ix_error ix_cc_fpm_arp_cb (
			   ix_error arg_Result,
			   void* arg_pContext,
			   void* arg_pMsg,
			   ix_uint32 arg_MsgLen
			   )
{
    ix_error err = IX_SUCCESS;
#ifdef IX_CC_ETH
    ix_cc_fpm_arp_callback_data* pContext;

    FPM_CC_DBG_TRACE ();
    pContext = (ix_cc_fpm_arp_callback_data*)arg_pContext;

    if(pContext == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
				("Internal callback invoked with NULL context"));

    /* Call user's callback function. */
    IX_ERROR_C((pContext->pUserCallback)
	       (arg_Result, pContext->pUserContext), err);
#endif
    return err;
}

/**
 * NAME: _ix_cc_fpm_eth_stats_cb
 *
 * DESCRIPTION: This is the internal callback function for retrieving Eth
 *              Tx statistics
 * 
 * @Param:  - IN ix_error arg_Result - error code returned from
 *          the asynchronous operation.
 * @Param:  - IN void* arg_pContext - pointer to reply message
 *          context, 
 * @Param:  - IN void* arg_pMsg - pointer to reply message.
 * @Param:  - IN ix_uint32 arg_MsgLen - length of reply message.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_NULL
 */
ix_error ix_cc_fpm_eth_stats_cb (
				 ix_error arg_Result,
				 void* arg_pContext,
				 void* arg_pMsg,
				 ix_uint32 arg_MsgLen
				 )
{
    ix_error err = IX_SUCCESS;
#ifdef IX_CC_ETH
    ix_uint64* buffer = NULL;
    ix_cc_fpm_eth_stats_callback_data* pContext;

    FPM_CC_DBG_TRACE ();
    pContext = (ix_cc_fpm_eth_stats_callback_data*)arg_pContext;
    
    if(pContext == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
				("Internal callback invoked with NULL context"));

    /* Call user's callback function. */
    IX_ERROR_C((pContext->pUserCallback)
	       (arg_Result, pContext->pUserContext, buffer), err);
#endif
    return err;
}

/**
 * NAME: ix_cc_fpm_atm_pos_stats_cb
 *
 * DESCRIPTION: This is the internal callback function for retrieving ATM/POS
 *              Tx statistics
 * 
 * @Param:  - IN ix_error arg_Result - error code returned from
 *          the asynchronous operation.
 * @Param:  - IN void* arg_pContext - pointer to reply message
 *          context, 
 * @Param:  - IN void* arg_pMsg - pointer to reply message.
 * @Param:  - IN ix_uint32 arg_MsgLen - length of reply message.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_NULL
 */
#ifdef IX_CC_ATM
ix_error ix_cc_fpm_atm_pos_stats_cb (
				     ix_error arg_Result,
				     void* arg_pContext,
				     void* arg_pMsg,
				     ix_uint32 arg_MsgLen
				     )
{
    ix_error err = IX_SUCCESS;
    ix_uint64* buffer = NULL;
    ix_cc_fpm_atm_pos_stats_callback_data* pContext;

    FPM_CC_DBG_TRACE ();
    pContext = (ix_cc_fpm_atm_pos_stats_callback_data*)arg_pContext;
    
    if(pContext == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
				("Internal callback invoked with NULL context"));
    
    /* Call user's callback function. */
    IX_ERROR_C((pContext->pUserCallback)
	       (arg_Result, pContext->pUserContext, buffer), err);
    
    return err;
}
#endif

/**
 * NAME: ix_cc_fpm_sync_add_l2_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to add a L2 table 
 *              entry into the L2 Table maintained on the egress.
 * 
 * @Param:  - IN arg_L2Id: L2 Id being added
 * @Param:  - IN arg_OutputPort: Outgoing port for the specified next hop
 * @Param:  - IN arg_l2tmInfo: ix_cc_l2tm_entry with NH IP.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_sync_add_l2_entry (
				      ix_uint32         arg_L2Index,
				      ix_uint32         arg_OutputPort,
				      ix_cc_l2tm_entry* arg_L2_info
				      )
{
#ifdef IX_CC_ETH
    ix_error                    *err;
    ix_uint32                   ret_msg_len = 0;
    ix_cc_fpm_add_l2_entry_data l2_params;

    FPM_CC_DBG_TRACE ();

    l2_params.l2_id = arg_L2Index;
    l2_params.output_port = arg_OutputPort;
    ix_ossl_memcpy (&l2_params.l2tm_info, arg_L2_info, 
		    sizeof (ix_cc_l2tm_entry));
    ix_cc_fpm_dump_l2_info (&l2_params);
    IX_ERROR_CGT (ix_cc_msup_send_sync_msg (IX_CC_FP_MODULE_INGRESS_OUTPUT,
					    IX_CC_FPM_MSG_ADD_L2_ENTRY,
					    (void*) &l2_params,
					    sizeof (ix_cc_fpm_add_l2_entry_data),
					    (void**) &err,
					    &ret_msg_len),
		  *err, IX_CC_FPM_ERROR_MSG_LIBRARY,
		  IX_ERROR_LEVEL_WARNING, label);
    FPM_CC_DBG_PRINT ("Leaving %s\n", __FUNCTION__);
    
    return IX_SUCCESS;

 label:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM CORE: %ld\n", (ix_uint32)*err); 
#else
		ix_error_dump(stdout,*err);
#endif

    return *err;
#else
    return IX_SUCCESS;
#endif
} /* end ix_cc_fpm_sync_add_l2_entry */

/**
 * NAME: ix_cc_fpm_sync_del_l2_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to del a L2 table 
 *              entry from the L2 Table maintained on the egress.
 *              This is a synchronous function that will block till the result
 *              is available or a timeout occurs.
 * 
 * @Param:  - IN arg_L2Id: L2 Id being removed
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_sync_del_l2_entry (
				      ix_uint32 arg_L2Id
				      )
{
#ifdef IX_CC_ETH
    ix_error  *err;
    ix_uint32 ret_msg_len = 0;

    FPM_CC_DBG_TRACE ();
    FPM_CC_DBG_PRINT ("L2 ID being deleted = %ld\n", (ix_uint32)arg_L2Id);

    IX_ERROR_CGT (ix_cc_msup_send_sync_msg (IX_CC_FP_MODULE_INGRESS_OUTPUT,
					    IX_CC_FPM_MSG_DEL_L2_ENTRY,
					    (void*) &arg_L2Id,
					    sizeof (ix_uint32),
					    (void**) &err,
					    &ret_msg_len),
		  *err, IX_CC_FPM_ERROR_MSG_LIBRARY,
		  IX_ERROR_LEVEL_WARNING, label);
    return IX_SUCCESS;
 label:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM CORE: %ld\n", (ix_uint32)*err); 
#else
		ix_error_dump(stdout,*err);
#endif
    return *err;
#else
    return IX_SUCCESS;
#endif
}

/**
 * NAME: ix_cc_fpm_async_add_l2_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to add a L2
 *              entry into the L2 table maintained on the egress.
 * 
 * @Param:  - IN arg_L2Id: L2 Id being added
 * @Param:  - IN arg_OutputPort: Outgoing port for the specified next hop
 * @Param:  - IN arg_l2tmInfo: ix_cc_l2tm_entry with NH IP.
 * @Param:  - IN ix_cc_eth_tx_next_hop_info arg_ARPInfo: ARP info to be added
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_async_add_l2_entry (
				       ix_uint32          arg_L2Index,
				       ix_uint32          arg_OutputPort,
				       ix_cc_l2tm_entry*  arg_L2_info,
				       ix_cc_fpm_l2_op_cb arg_Callback,
				       void*              arg_pUserContext
				       )
{
#ifdef IX_CC_ETH
    ix_cc_fpm_l2_callback_data* cbContext;
    ix_cc_fpm_add_l2_entry_data l2_params;
    ix_error                    err = IX_SUCCESS;

    FPM_CC_DBG_TRACE ();
    if (arg_Callback == NULL) {
	return IX_ERROR_WARNING (IX_CC_ERROR_NULL, 
				 ("Invalid callback funtion pointer\n"));
    }
    cbContext = (ix_cc_fpm_l2_callback_data*) 
	ix_ossl_malloc (sizeof (ix_cc_fpm_l2_callback_data));
    if (cbContext == NULL) {
	return IX_ERROR_WARNING (IX_CC_ERROR_OOM, ("Mem allocation for context"
						   "failed"));
    }

    l2_params.l2_id = arg_L2Index;
    l2_params.output_port = arg_OutputPort;
    ix_ossl_memcpy (&l2_params.l2tm_info, arg_L2_info, 
		    sizeof (ix_cc_l2tm_entry));
    ix_cc_fpm_dump_l2_info (&l2_params);

    cbContext->pUserCallback = arg_Callback;
    cbContext->pUserContext = arg_pUserContext;

    FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

    IX_ERROR_CGT (ix_cc_msup_send_async_msg (IX_CC_FP_MODULE_INGRESS_OUTPUT,
					     (ix_cc_msghlp_callback)
					     ix_cc_fpm_l2_cb,
					     (void*) cbContext,
					     IX_CC_FPM_MSG_ADD_L2_ENTRY,
					     (void*) &l2_params,
					     sizeof 
					     (ix_cc_fpm_add_l2_entry_data)),
		  err,
		  IX_CC_FPM_ERROR_MSG_LIBRARY,
		  IX_ERROR_LEVEL_WARNING,
		  cleanup);
    FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);
    
    return IX_SUCCESS;
 cleanup:
    if (cbContext != NULL)
      ix_ossl_free (cbContext);

    return err;
#else
    return IX_SUCCESS;
#endif
}

/**
 * NAME: ix_cc_fpm_async_del_l2_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to del a L2
 *              entry from the L2 table maintained on the egress.
 * 
 * @Param:  - IN arg_L2Id: L2 Id being deleted
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_async_del_l2_entry (
					ix_uint32  arg_L2Id,
					ix_cc_fpm_l2_op_cb arg_Callback,
					void* arg_pUserContext
					)
{
#ifdef IX_CC_ETH
    ix_cc_fpm_l2_callback_data* cbContext;
    ix_error                     err = IX_SUCCESS;

    FPM_CC_DBG_TRACE ();
    if (arg_Callback == NULL) {
	return IX_ERROR_WARNING (IX_CC_ERROR_NULL, 
				 ("Invalid callback funtion pointer\n"));
    }
    cbContext = (ix_cc_fpm_l2_callback_data*) 
	ix_ossl_malloc (sizeof (ix_cc_fpm_l2_callback_data));
    if (cbContext == NULL) {
	return IX_ERROR_WARNING (IX_CC_ERROR_OOM, ("Mem allocation for context"
						   "failed"));
    }

    cbContext->pUserCallback = arg_Callback;
    cbContext->pUserContext = arg_pUserContext;

    IX_ERROR_CGT (ix_cc_msup_send_async_msg (IX_CC_FP_MODULE_INGRESS_OUTPUT,
					     (ix_cc_msghlp_callback)
					     ix_cc_fpm_l2_cb,
					     (void*) cbContext,
					     IX_CC_FPM_MSG_DEL_L2_ENTRY,
					     (void*) &arg_L2Id,
					     sizeof (ix_uint32)),
		  err,
		  IX_CC_FPM_ERROR_MSG_LIBRARY,
		  IX_ERROR_LEVEL_WARNING,
		  cleanup);
    
    return IX_SUCCESS;
 cleanup:
    if (cbContext != NULL)
      ix_ossl_free (cbContext);

    return err;
#else
    return IX_SUCCESS;
#endif
}

/**
 * NAME: ix_cc_fpm_async_add_arp_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to add a ARP
 *              entry into the ARP table maintained on the egress.
 * 
 * @Param:  - IN ix_cc_eth_tx_next_hop_info arg_ARPInfo: ARP info to be added
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_async_add_arp_entry (
					ix_cc_eth_tx_next_hop_info* arg_ARPInfo,
					ix_cc_fpm_arp_op_cb arg_Callback,
					void* arg_pUserContext
					)
{
#ifdef IX_CC_ETH
    ix_cc_fpm_arp_callback_data* cbContext;
    ix_error                     err = IX_SUCCESS;

    FPM_CC_DBG_TRACE ();
    if (arg_Callback == NULL) {
	return IX_ERROR_WARNING (IX_CC_ERROR_NULL, 
				 ("Invalid callback funtion pointer\n"));
    }
    cbContext = (ix_cc_fpm_arp_callback_data*) 
	ix_ossl_malloc (sizeof (ix_cc_fpm_arp_callback_data));
    if (cbContext == NULL) {
	return IX_ERROR_WARNING (IX_CC_ERROR_OOM, ("Mem allocation for context"
						   "failed"));
    }

    ix_cc_fpm_dump_arp_info (arg_ARPInfo);

    cbContext->pUserCallback = arg_Callback;
    cbContext->pUserContext = arg_pUserContext;

    FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

    IX_ERROR_CGT (ix_cc_msup_send_async_msg (IX_CC_FP_MODULE_INGRESS_OUTPUT,
					     (ix_cc_msghlp_callback)
					     ix_cc_fpm_arp_cb,
					     (void*) cbContext,
					     IX_CC_FPM_MSG_ADD_ARP_ENTRY,
					     (void*) arg_ARPInfo,
					     sizeof (ix_cc_eth_tx_next_hop_info)),
		  err,
		  IX_CC_FPM_ERROR_MSG_LIBRARY,
		  IX_ERROR_LEVEL_WARNING,
		  cleanup);
    FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);
    
    return IX_SUCCESS;
 cleanup:
    FPM_CC_DBG_PRINT ("Error at %s : %d \n", __FUNCTION__, __LINE__);
    if (cbContext != NULL)
      ix_ossl_free (cbContext);

    return err;
#else
    return IX_SUCCESS;
#endif
}

/**
 * NAME: ix_cc_fpm_async_del_arp_entry
 *
 * DESCRIPTION: This message helper primitive helps a client to del a ARP
 *              entry from the ARP table maintained on the egress.
 * 
 * @Param:  - IN arg_L2Id: L2 Id being deleted
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_async_del_arp_entry (
					ix_uint32  arg_L2Id,
					ix_cc_fpm_arp_op_cb arg_Callback,
					void* arg_pUserContext
					)
{
#ifdef IX_CC_ETH
    ix_cc_fpm_arp_callback_data* cbContext;
    ix_error                     err = IX_SUCCESS;

    FPM_CC_DBG_TRACE ();
    if (arg_Callback == NULL) {
	return IX_ERROR_WARNING (IX_CC_ERROR_NULL, 
				 ("Invalid callback funtion pointer\n"));
    }
    cbContext = (ix_cc_fpm_arp_callback_data*) 
	ix_ossl_malloc (sizeof (ix_cc_fpm_arp_callback_data));
    if (cbContext == NULL) {
	return IX_ERROR_WARNING (IX_CC_ERROR_OOM, ("Mem allocation for context"
						   "failed"));
    }

    cbContext->pUserCallback = arg_Callback;
    cbContext->pUserContext = arg_pUserContext;

    IX_ERROR_CGT (ix_cc_msup_send_async_msg (IX_CC_FP_MODULE_INGRESS_OUTPUT,
					     (ix_cc_msghlp_callback)
					     ix_cc_fpm_arp_cb,
					     (void*) cbContext,
					     IX_CC_FPM_MSG_DEL_ARP_ENTRY,
					     (void*) &arg_L2Id,
					     sizeof (ix_uint32)),
		  err,
		  IX_CC_FPM_ERROR_MSG_LIBRARY,
		  IX_ERROR_LEVEL_WARNING,
		  cleanup);
    FPM_CC_DBG_PRINT ("%s : %d \n", __FUNCTION__, __LINE__);

    return IX_SUCCESS;
 cleanup:
    FPM_CC_DBG_PRINT ("Error at %s : %d \n", __FUNCTION__, __LINE__);
    if (cbContext != NULL)
      ix_ossl_free (cbContext);

    return err;
#else
    return IX_SUCCESS;
#endif
}

/**
 * NAME: ix_cc_fpm_sync_purge_arp_table
 *
 * DESCRIPTION: This message helper primitive helps a client to purge the ARP
 *              table maintained on the egress.
 * 
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_sync_purge_arp_table ()
{
#ifdef IX_CC_ETH
    FPM_CC_DBG_TRACE ();
    IX_ERROR_CRT (ix_cc_msup_send_msg (IX_CC_FP_MODULE_INGRESS_OUTPUT,
				       IX_CC_FPM_MSG_PURGE_ARP_TABLE,
				       (void*) NULL,
				       0),
		  IX_CC_FPM_ERROR_MSG_LIBRARY,
		  IX_ERROR_LEVEL_WARNING);

    return IX_SUCCESS;
#else
    return IX_SUCCESS;
#endif
}


/**
 * NAME: ix_cc_fpm_async_get_eth_tx_stats
 *
 * DESCRIPTION: This message helper primitive helps a client to retrieve
 *              Ethernet transmit statistics maintained on the egress.
 * 
 * @Param:  - IN ix_cc_eth_tx_statistics_info_context arg_pStatsInfo: 
 *               statistics info to be retrieved
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_async_get_eth_tx_stats (
					   ix_cc_eth_tx_statistics_info_context*
					   arg_pStatsInfo,
					   ix_cc_fpm_eth_tx_stats_op_cb 
					   arg_Callback
					   )
{
#ifdef IX_CC_ETH
    ix_cc_fpm_eth_stats_callback_data* cbContext;
    ix_error err = IX_SUCCESS;

    FPM_CC_DBG_TRACE ();
    if (arg_Callback == NULL) {
	return IX_ERROR_WARNING (IX_CC_ERROR_NULL, 
				 ("Invalid callback funtion pointer\n"));
    }
    cbContext = (ix_cc_fpm_eth_stats_callback_data*) 
	ix_ossl_malloc (sizeof (ix_cc_fpm_eth_stats_callback_data));
    if (cbContext == NULL) {
	return IX_ERROR_WARNING (IX_CC_ERROR_OOM, ("Mem allocation for context"
						   "failed"));
    }

    FPM_CC_DBG_PRINT ("Requested Eth tx stats for port num %ld\n",
		      (ix_uint32)arg_pStatsInfo->index);

    cbContext->pUserCallback = arg_Callback;
    cbContext->pUserContext = arg_pStatsInfo->pUserContext;

    IX_ERROR_CGT (ix_cc_msup_send_async_msg (IX_CC_FP_MODULE_INGRESS_OUTPUT,
					     (ix_cc_msghlp_callback)
					     ix_cc_fpm_eth_stats_cb,
					     (void*) cbContext,
					     IX_CC_FPM_MSG_ETH_TX_STATS,
					     (void*) arg_pStatsInfo,
					     sizeof (ix_cc_eth_tx_statistics_info_context)),
		  err,
		  IX_CC_FPM_ERROR_MSG_LIBRARY,
		  IX_ERROR_LEVEL_WARNING,
		  cleanup);

 cleanup:
    if (cbContext != NULL)
	ix_ossl_free (cbContext);
    
    return err;
#else
    return IX_SUCCESS;
#endif
}


void ix_cc_fpm_dump_l2_info (ix_cc_fpm_add_l2_entry_data *l2_info)
{
#ifdef IX_CC_ETH
    FPM_CC_DBG_TRACE ();

    FPM_CC_DBG_PRINT ("L2 ID = %ld \n Output Port = %ld \n", (ix_uint32)l2_info->l2_id,
		      (ix_uint32)l2_info->output_port);
    switch (l2_info->l2tm_info.l2AddrType) {
    case IX_CC_L2TM_L2ADDR_ETHER:
	FPM_CC_DBG_PRINT ("L2 addr type = Ethernet\n");
	break;
    case IX_CC_L2TM_L2ADDR_ATM:
	FPM_CC_DBG_PRINT ("L2 addr type = ATM\n");
	break;
    default:
	FPM_CC_DBG_PRINT ("L2 addr type = Unknown!\n");
	break;
    }
    
    switch (l2_info->l2tm_info.ipAddrType) {
    case IX_CC_L2TM_IPADDR_IPV4:
	FPM_CC_DBG_PRINT ("IP addr type = IPv4\n IPAddress = 0x%x\n",
			  l2_info->l2tm_info.l3Address.ipv4Address);
	break;
    case IX_CC_L2TM_IPADDR_IPV6:
	FPM_CC_DBG_PRINT ("IP addr type = IPv6\n IPAddress = 0x%x\n",
			  l2_info->l2tm_info.l3Address.ipv4Address);
	break;
    default:
	FPM_CC_DBG_PRINT ("IP addr type = Unknown!\n");
	break;
    }
#endif
    return;
}

void ix_cc_fpm_dump_arp_info (ix_cc_eth_tx_next_hop_info *arp_info)
{
#ifdef IX_CC_ETH
    FPM_CC_DBG_TRACE ();

    FPM_CC_DBG_PRINT ("L2 ID = %ld \n Output Port = %ld \n"
		      "Next hop IP = 0x%x \n L2 Addr = %X:%X:%X:%X:%X:%X",
		      (ix_uint32)arp_info->l2Index, 
		      (ix_uint32)arp_info->outputPort, 
		      arp_info->nextHopIp, 
		      arp_info->l2Addr.etherAddr[0],arp_info->l2Addr.etherAddr[1],
		      arp_info->l2Addr.etherAddr[2],arp_info->l2Addr.etherAddr[3],
		      arp_info->l2Addr.etherAddr[4],arp_info->l2Addr.etherAddr[5]
		      );
#endif
    return;
}

#ifdef IX_CC_QOS
/**
 * NAME: ix_cc_fpm_async_dpe_op
 *
 * DESCRIPTION: This message helper primitive helps a client to perform WRED dropper
 *              maintained on the egress.
 * 
 * @Param:  - IN ix_cc_fpm_dpe arg_ARPInfo: ARP info to be added
 * @Param:  - IN arg_Callback : pointer to client provided callback function.
 * @Param:  - IN arg_pUserContext: pointer to client defined context.
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - invalid pointer
 * IX_CC_FPM_ERROR_MSG_LIBRARY- failure from Message Support Library
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * IX_CC_FPM_ERROR_OOM - memory allocation failure
 * 
 */

ix_error ix_cc_fpm_async_dpe_op (
				ix_cc_fpm_dpe *arg_DpeInfo,
				ix_cc_fpm_dpe_cb arg_Callback,
				void* arg_pUserContext
				)
{
    ix_cc_fpm_dpe_callback_data  *cbContext;
    ix_error                     err = IX_SUCCESS;

    FPM_CC_DBG_TRACE ();
    if (arg_Callback == NULL) {
	return IX_ERROR_WARNING (IX_CC_ERROR_NULL, 
				 ("Invalid callback funtion pointer\n"));
    }
    cbContext = (ix_cc_fpm_dpe_callback_data*) 
	ix_ossl_malloc (sizeof (ix_cc_fpm_dpe_callback_data));
    if (cbContext == NULL) {
	return IX_ERROR_WARNING (IX_CC_ERROR_OOM, ("Mem allocation for context"
						   "failed"));
    }
    ix_ossl_memcpy(&cbContext->dpe, arg_DpeInfo, sizeof(ix_cc_fpm_dpe));
    if(arg_DpeInfo->type == IX_CC_FPM_DPE_WRED_DROPPER && arg_DpeInfo->op == IX_CC_FPM_DPE_STATS)
    {
        cbContext->p_wred_dropper_stats_callback = arg_Callback;
    }
    else if(arg_DpeInfo->type == IX_CC_FPM_DPE_METER && arg_DpeInfo->op == IX_CC_FPM_DPE_STATS)
    {
        cbContext->p_meter_stats_callback = arg_Callback;
    }
    else
    {
        cbContext->pUserCallback = arg_Callback;
    }
    cbContext->pUserContext = arg_pUserContext;

    FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

    IX_ERROR_CGT (ix_cc_msup_send_async_msg (IX_CC_FP_MODULE_INGRESS_OUTPUT,
					     (ix_cc_msghlp_callback)
					     ix_cc_fpm_dpe_op_cb,
					     (void*) cbContext,
					     IX_CC_FPM_MSG_DPE_OP,
					     (void*) arg_DpeInfo,
					     sizeof (ix_cc_fpm_dpe)),
		  			     err,
		  			     IX_CC_FPM_ERROR_MSG_LIBRARY,
		  			     IX_ERROR_LEVEL_WARNING,
		  			     cleanup);
    FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);
    return IX_SUCCESS;
 cleanup:
    FPM_CC_DBG_PRINT ("Error at %s : %d \n", __FUNCTION__, __LINE__);
    if (cbContext != NULL)
      ix_ossl_free (cbContext);
    return err;
}
#endif /*IX_CC_QOS*/

#ifdef IX_CC_ATM
ix_error ix_cc_llc_snap_encap_fpm_add_next_hop(
				ix_uint16 nextHop,
				ix_uint8 encapBit,
				ix_uint16 vpi,
				ix_uint16 vci,
				ix_uint16 vcq,
				ix_uint8 dSap,
				ix_uint8 sSap,
				ix_uint8 ctrl,
				ix_uint32 oui,
				ix_uint16 pid,
				ix_cc_llc_snap_encap_cb_add_next_hop pCallback,
				void *pUserContext)
{
	ix_cc_fpm_llc_snap_encap_add_next_hop_cb_t *cbContext = NULL;
	ix_cc_fpm_llc_snap_encap_add_next_hop_t params;
	ix_error  err = IX_SUCCESS;

	FPM_CC_DBG_TRACE ();

	/* Check if the Call back Function is NULL */
	if (pCallback == NULL)
	{
		return IX_ERROR_WARNING (IX_CC_ERROR_NULL, 
				("Invalid callback funtion pointer\n"));
	}

	/* Allocate Memory for the Call Back data structure */
	cbContext = (ix_cc_fpm_llc_snap_encap_add_next_hop_cb_t *) 
						ix_ossl_malloc (sizeof (ix_cc_fpm_llc_snap_encap_add_next_hop_cb_t));
	if (cbContext == NULL)
	{
			return IX_ERROR_WARNING
					(IX_CC_ERROR_OOM, ("Mem allocation for context failed"));
	}

	/* Fill up the Call back data structure */
	cbContext->pCallback = pCallback;
	cbContext->pUserContext = pUserContext;

	/* Fill up the Parameters to be passed to actual CC function */
	ix_ossl_memcpy(&params, 0, sizeof (ix_cc_fpm_llc_snap_encap_add_next_hop_t));

	params.nextHop = nextHop;
	params.encapBit = encapBit; 
	params.vpi = vpi;
	params.vci = vci;
	params.vcq = vcq;
	params.dSap = dSap;
	params.sSap = sSap;
	params.ctrl = ctrl;
	params.oui = oui;
	params.pid = pid;

	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

	IX_ERROR_CGT (
		ix_cc_msup_send_async_msg (IX_CC_FP_MODULE_INGRESS_OUTPUT,
							(ix_cc_msghlp_callback)ix_cc_llc_snap_encap_fpm_add_next_hop_cb_core,
							(void*) cbContext,
							IX_CC_FPM_MSG_LLC_SNAP_ENCAP_ADD_NEXT_HOP,
							(void*) &params,
							sizeof(ix_cc_fpm_llc_snap_encap_add_next_hop_t)), 
		err,
		IX_CC_FPM_ERROR_MSG_LIBRARY,
		IX_ERROR_LEVEL_WARNING,
		cleanup);
	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);
    
	return IX_SUCCESS;

 cleanup:
    if (cbContext != NULL)
      ix_ossl_free (cbContext);

    return err;
}


ix_error ix_cc_llc_snap_encap_fpm_del_next_hop(
				ix_uint16 nextHop,
				ix_cc_llc_snap_encap_cb_delete_next_hop pCallback,
				void *pUserContext)
{
	ix_cc_fpm_llc_snap_encap_del_next_hop_cb_t *cbContext = NULL;
	ix_cc_fpm_llc_snap_encap_del_next_hop_t params;
	ix_error  err = IX_SUCCESS;

	FPM_CC_DBG_TRACE ();

	/* Check if the Call back Function is NULL */
	if (pCallback == NULL)
	{
		return IX_ERROR_WARNING (IX_CC_ERROR_NULL, 
				("Invalid callback funtion pointer\n"));
	}

	/* Allocate Memory for the Call Back data structure */
	cbContext = (ix_cc_fpm_llc_snap_encap_del_next_hop_cb_t *) 
						ix_ossl_malloc (sizeof (ix_cc_fpm_llc_snap_encap_del_next_hop_cb_t));
	if (cbContext == NULL)
	{
			return IX_ERROR_WARNING
					(IX_CC_ERROR_OOM, ("Mem allocation for context failed"));
	}

	/* Fill up the Call back data structure */
	cbContext->pCallback = pCallback;
	cbContext->pUserContext = pUserContext;

	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

	IX_ERROR_CGT (
		ix_cc_msup_send_async_msg (IX_CC_FP_MODULE_INGRESS_OUTPUT,
							(ix_cc_msghlp_callback)ix_cc_llc_snap_encap_fpm_del_next_hop_cb_core,
							(void*) cbContext,
							IX_CC_FPM_MSG_LLC_SNAP_ENCAP_DEL_NEXT_HOP,
							(void*) &nextHop,
							sizeof(ix_uint16)), 
		err,
		IX_CC_FPM_ERROR_MSG_LIBRARY,
		IX_ERROR_LEVEL_WARNING,
		cleanup);
	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);
    
	return IX_SUCCESS;

 cleanup:
    if (cbContext != NULL)
      ix_ossl_free (cbContext);

    return err;
}

ix_error ix_cc_fpm_async_port_create(
				const ix_cc_atmsar_instance_t arg_sarInstance,
				const ix_cc_atmsar_port_descr_t* arg_pPortDescr,
				ix_cc_atmsar_cb_port_create_t arg_Callback,
				void *arg_pUserContext)

{
	ix_cc_fpm_atm_port_create_cb_t *cbContext = NULL;
	ix_cc_fpm_atm_port_descr_t port_params;
	ix_error  err = IX_SUCCESS;

	FPM_CC_DBG_TRACE ();

	/* Check if the Call back Function is NULL */
	if (arg_Callback == NULL)
	{
		return IX_ERROR_WARNING (IX_CC_ERROR_NULL, 
				("Invalid callback funtion pointer\n"));
	}

	/* Allocate Memory for the Call Back data structure */
	cbContext = (ix_cc_fpm_atm_port_create_cb_t *) 
						ix_ossl_malloc (sizeof (ix_cc_fpm_atm_port_create_cb_t));
	if (cbContext == NULL)
	{
			return IX_ERROR_WARNING
					(IX_CC_ERROR_OOM, ("Mem allocation for context failed"));
	}

	/* Fill up the Call back data structure */
	cbContext->arg_Callback = arg_Callback;
	cbContext->arg_pUserContext = arg_pUserContext;

	/* Fill up the Parameters to be passed to actual CC function */
	ix_ossl_memcpy(&port_params, 0, sizeof (ix_cc_atmsar_port_descr_t));
	port_params.arg_sarInstance = arg_sarInstance;
	ix_ossl_memcpy (&port_params.arg_pPortDescr, arg_pPortDescr, 
					sizeof (ix_cc_atmsar_port_descr_t));

	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

	IX_ERROR_CGT (
		ix_cc_msup_send_async_msg (IX_CC_FP_MODULE_INGRESS_OUTPUT,
							(ix_cc_msghlp_callback)ix_cc_fpm_atm_port_create_cb_core,
							(void*) cbContext,
							IX_CC_FPM_MSG_ATM_PORT_CREATE,
							(void*) &port_params,
							sizeof(ix_cc_fpm_atm_port_descr_t)), 
		err,
		IX_CC_FPM_ERROR_MSG_LIBRARY,
		IX_ERROR_LEVEL_WARNING,
		cleanup);
	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);
    
	return IX_SUCCESS;

 cleanup:
    if (cbContext != NULL)
      ix_ossl_free (cbContext);

    return err;
}

ix_error ix_cc_fpm_async_port_remove(
				const ix_cc_atmsar_instance_t arg_sarInstance,
				ix_cc_atmsar_port_handle_t arg_port,
				ix_cc_atmsar_cb_port_remove_t arg_Callback,
				void *arg_pUserContext)
{
	ix_cc_fpm_atm_port_del_cb_t *cbContext = NULL;
	ix_cc_fpm_atm_port_handle_t port_params;
	ix_error  err = IX_SUCCESS;

	FPM_CC_DBG_TRACE ();

	/* Check if the Call back Function is NULL */
	if (arg_Callback == NULL)
	{
		return IX_ERROR_WARNING (IX_CC_ERROR_NULL, 
				("Invalid callback funtion pointer\n"));
	}

	/* Allocate Memory for the Call Back data structure */
	cbContext = (ix_cc_fpm_atm_port_del_cb_t *) 
						ix_ossl_malloc (sizeof (ix_cc_fpm_atm_port_del_cb_t));
	if (cbContext == NULL)
	{
			return IX_ERROR_WARNING
					(IX_CC_ERROR_OOM, ("Mem allocation for context failed"));
	}

	/* Fill up the Call back data structure */
	cbContext->arg_Callback = arg_Callback;
	cbContext->arg_pUserContext = arg_pUserContext;

	/* Fill up the Parameters to be passed to actual CC function */
	ix_ossl_memcpy(&port_params, 0, sizeof (ix_cc_fpm_atm_port_handle_t));
	port_params.arg_sarInstance = arg_sarInstance;
	port_params.arg_port = arg_port;

	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

	IX_ERROR_CGT (
		ix_cc_msup_send_async_msg (IX_CC_FP_MODULE_INGRESS_OUTPUT,
							(ix_cc_msghlp_callback)ix_cc_fpm_atm_port_del_cb_core,
							(void*) cbContext,
							IX_CC_FPM_MSG_ATM_PORT_DELETE,
							(void*) &port_params,
							sizeof(ix_cc_fpm_atm_port_handle_t)), 
		err,
		IX_CC_FPM_ERROR_MSG_LIBRARY,
		IX_ERROR_LEVEL_WARNING,
		cleanup);
	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);
    
	return IX_SUCCESS;

 cleanup:
    if (cbContext != NULL)
      ix_ossl_free (cbContext);

    return err;
}


ix_error ix_cc_fpm_async_get_port_stats(
				const ix_cc_atmsar_instance_t arg_sarInstance,
				ix_cc_atmsar_port_handle_t arg_port,
				ix_cc_atmsar_cb_get_port_stats_t arg_Callback,
				void *arg_pUserContext)
{
	ix_cc_fpm_atm_port_stats_get_cb_t *cbContext = NULL;
	ix_cc_fpm_atm_port_handle_t port_params;
	ix_error  err = IX_SUCCESS;

	FPM_CC_DBG_TRACE ();

	/* Check if the Call back Function is NULL */
	if (arg_Callback == NULL)
	{
		return IX_ERROR_WARNING (IX_CC_ERROR_NULL, 
				("Invalid callback funtion pointer\n"));
	}

	/* Allocate Memory for the Call Back data structure */
	cbContext = (ix_cc_fpm_atm_port_stats_get_cb_t *) 
						ix_ossl_malloc (sizeof (ix_cc_fpm_atm_port_stats_get_cb_t));
	if (cbContext == NULL)
	{
			return IX_ERROR_WARNING
					(IX_CC_ERROR_OOM, ("Mem allocation for context failed"));
	}

	/* Fill up the Call back data structure */
	cbContext->arg_Callback = arg_Callback;
	cbContext->arg_pUserContext = arg_pUserContext;

	/* Fill up the Parameters to be passed to actual CC function */
	ix_ossl_memcpy(&port_params, 0, sizeof (ix_cc_fpm_atm_port_handle_t));
	port_params.arg_sarInstance = arg_sarInstance;
	port_params.arg_port = arg_port;

	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

	IX_ERROR_CGT (
		ix_cc_msup_send_async_msg (IX_CC_FP_MODULE_INGRESS_OUTPUT,
							(ix_cc_msghlp_callback)ix_cc_fpm_atm_port_stats_get_cb_core,
							(void*) cbContext,
							IX_CC_FPM_MSG_ATM_PORT_STATS_GET,
							(void*) &port_params,
							sizeof(ix_cc_fpm_atm_port_handle_t)), 
		err,
		IX_CC_FPM_ERROR_MSG_LIBRARY,
		IX_ERROR_LEVEL_WARNING,
		cleanup);
	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);
    
	return IX_SUCCESS;

 cleanup:
    if (cbContext != NULL)
      ix_ossl_free (cbContext);

    return err;
}

ix_error ix_cc_fpm_async_vc_create(
				const ix_cc_atmsar_instance_t arg_sarInstance,
				const ix_cc_atmsar_vc_descr_t* arg_pVcDescr,
				ix_cc_atmsar_cb_vc_create_t arg_Callback,
				void *arg_pUserContext)
{
	ix_cc_fpm_atm_vc_create_cb_t *cbContext = NULL;
	ix_cc_fpm_atm_vc_descr_t vc_params;
	ix_error  err = IX_SUCCESS;

	FPM_CC_DBG_TRACE ();

	/* Check if the Call back Function is NULL */
	if (arg_Callback == NULL)
	{
		return IX_ERROR_WARNING (IX_CC_ERROR_NULL, 
				("Invalid callback funtion pointer\n"));
	}

	/* Allocate Memory for the Call Back data structure */
	cbContext = (ix_cc_fpm_atm_vc_create_cb_t *) 
						ix_ossl_malloc (sizeof (ix_cc_fpm_atm_vc_create_cb_t));
	if (cbContext == NULL)
	{
			return IX_ERROR_WARNING
					(IX_CC_ERROR_OOM, ("Mem allocation for context failed"));
	}

	/* Fill up the Parameters to be passed to actual CC function */
	ix_ossl_memcpy(&vc_params, 0, sizeof (ix_cc_fpm_atm_vc_descr_t));
	vc_params.arg_sarInstance = arg_sarInstance;
	ix_ossl_memcpy(&vc_params.arg_pVcDescr, arg_pVcDescr, 
					sizeof (ix_cc_atmsar_vc_descr_t));

	/* Fill up the Call back data structure */
	cbContext->arg_Callback = arg_Callback;
	cbContext->arg_pUserContext = arg_pUserContext;

	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

	IX_ERROR_CGT (
		ix_cc_msup_send_async_msg (IX_CC_FP_MODULE_INGRESS_OUTPUT,
							(ix_cc_msghlp_callback)ix_cc_fpm_atm_vc_create_cb_core,
							(void*) cbContext,
							IX_CC_FPM_MSG_ATM_VCC_CREATE,
							(void*) &vc_params,
                     sizeof(ix_cc_fpm_atm_vc_descr_t)), 
		               err,
		               IX_CC_FPM_ERROR_MSG_LIBRARY,
		               IX_ERROR_LEVEL_WARNING,
		               cleanup);
	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);
    
	return IX_SUCCESS;

 cleanup:
    if (cbContext != NULL)
      ix_ossl_free (cbContext);

    return err;
}


ix_error ix_cc_fpm_async_vc_update(
				const ix_cc_atmsar_instance_t arg_sarInstance,
				ix_cc_atmsar_vc_handle_t arg_hVc,
				const ix_cc_atmsar_update_data_t* arg_pUpdateData,
				ix_cc_atmsar_cb_vc_update_t arg_Callback,
				void *arg_pUserContext)
{
	ix_cc_fpm_atm_vc_update_cb_t *cbContext = NULL;
	ix_cc_fpm_atm_vc_update_t vc_params;
	ix_error  err = IX_SUCCESS;

	FPM_CC_DBG_TRACE ();

	/* Check if the Call back Function is NULL */
	if (arg_Callback == NULL)
	{
		return IX_ERROR_WARNING (IX_CC_ERROR_NULL, 
				("Invalid callback funtion pointer\n"));
	}

	/* Allocate Memory for the Call Back data structure */
	cbContext = (ix_cc_fpm_atm_vc_update_cb_t *) 
						ix_ossl_malloc (sizeof (ix_cc_fpm_atm_vc_update_cb_t));
	if (cbContext == NULL)
	{
			return IX_ERROR_WARNING
					(IX_CC_ERROR_OOM, ("Mem allocation for context failed"));
	}

	/* Fill up the Parameters to be passed to actual CC function */
	ix_ossl_memcpy(&vc_params, 0, sizeof (ix_cc_fpm_atm_vc_update_t));
	vc_params.arg_sarInstance = arg_sarInstance;
	vc_params.arg_hVc = arg_hVc;
	ix_ossl_memcpy(&vc_params.arg_pUpdateData, arg_pUpdateData, 
					sizeof (ix_cc_atmsar_update_data_t));

	/* Fill up the Call back data structure */
	cbContext->arg_Callback = arg_Callback;
	cbContext->arg_pUserContext = arg_pUserContext;

	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

	IX_ERROR_CGT (
		ix_cc_msup_send_async_msg (IX_CC_FP_MODULE_INGRESS_OUTPUT,
							(ix_cc_msghlp_callback)ix_cc_fpm_atm_vc_update_cb_core,
							(void*) cbContext,
							IX_CC_FPM_MSG_ATM_VCC_UPDATE,
							(void*) &vc_params,
							sizeof(ix_cc_fpm_atm_vc_update_t)), 
		err,
		IX_CC_FPM_ERROR_MSG_LIBRARY,
		IX_ERROR_LEVEL_WARNING,
		cleanup);
	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);
    
	return IX_SUCCESS;

 cleanup:
    if (cbContext != NULL)
      ix_ossl_free (cbContext);

    return err;
}



ix_error ix_cc_fpm_async_vc_remove(
				const ix_cc_atmsar_instance_t arg_sarInstance,
				const ix_cc_atmsar_vc_handle_t arg_hVc,
				ix_cc_atmsar_cb_vc_remove_t arg_Callback,
				void *arg_pUserContext)
{
	ix_cc_fpm_atm_vc_remove_cb_t *cbContext = NULL;
	ix_cc_fpm_atm_vc_handle_t vc_params;
	ix_error  err = IX_SUCCESS;

	FPM_CC_DBG_TRACE ();

	/* Check if the Call back Function is NULL */
	if (arg_Callback == NULL)
	{
		return IX_ERROR_WARNING (IX_CC_ERROR_NULL, 
				("Invalid callback funtion pointer\n"));
	}

	/* Allocate Memory for the Call Back data structure */
	cbContext = (ix_cc_fpm_atm_vc_remove_cb_t *) 
						ix_ossl_malloc (sizeof (ix_cc_fpm_atm_vc_remove_cb_t));
	if (cbContext == NULL)
	{
			return IX_ERROR_WARNING
					(IX_CC_ERROR_OOM, ("Mem allocation for context failed"));
	}

	/* Fill up the Parameters to be passed to actual CC function */
	ix_ossl_memcpy(&vc_params, 0, sizeof (ix_cc_fpm_atm_vc_handle_t));
	vc_params.arg_sarInstance = arg_sarInstance;
	vc_params.arg_hVc = arg_hVc;

	/* Fill up the Call back data structure */
	cbContext->arg_Callback = arg_Callback;
	cbContext->arg_pUserContext = arg_pUserContext;

	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

	IX_ERROR_CGT (
		ix_cc_msup_send_async_msg (IX_CC_FP_MODULE_INGRESS_OUTPUT,
							(ix_cc_msghlp_callback)ix_cc_fpm_atm_vc_remove_cb_core,
							(void*) cbContext,
							IX_CC_FPM_MSG_ATM_VCC_DELETE,
							(void*) &vc_params,
							sizeof(ix_cc_fpm_atm_vc_handle_t)), 
		err,
		IX_CC_FPM_ERROR_MSG_LIBRARY,
		IX_ERROR_LEVEL_WARNING,
		cleanup);
	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);
    
	return IX_SUCCESS;

 cleanup:
    if (cbContext != NULL)
      ix_ossl_free (cbContext);

    return err;
}

ix_error ix_cc_fpm_async_get_vc_stats(
				const ix_cc_atmsar_instance_t arg_sarInstance,
				ix_cc_atmsar_vc_handle_t arg_hVc,
				ix_cc_atmsar_cb_get_vc_stats_t arg_Callback,
				void *arg_pUserContext)
{
	ix_cc_fpm_atm_vc_stats_get_cb_t *cbContext = NULL;
	ix_cc_fpm_atm_vc_handle_t vc_params;
	ix_error  err = IX_SUCCESS;

	FPM_CC_DBG_TRACE ();

	/* Check if the Call back Function is NULL */
	if (arg_Callback == NULL)
	{
		return IX_ERROR_WARNING (IX_CC_ERROR_NULL, 
				("Invalid callback funtion pointer\n"));
	}

	/* Allocate Memory for the Call Back data structure */
	cbContext = (ix_cc_fpm_atm_vc_stats_get_cb_t *) 
						ix_ossl_malloc (sizeof (ix_cc_fpm_atm_vc_stats_get_cb_t));
	if (cbContext == NULL)
	{
			return IX_ERROR_WARNING
					(IX_CC_ERROR_OOM, ("Mem allocation for context failed"));
	}

	/* Fill up the Parameters to be passed to actual CC function */
	ix_ossl_memcpy(&vc_params, 0, sizeof (ix_cc_fpm_atm_vc_handle_t));
	vc_params.arg_sarInstance = arg_sarInstance;
	vc_params.arg_hVc = arg_hVc;

	/* Fill up the Call back data structure */
	cbContext->arg_Callback = arg_Callback;
	cbContext->arg_pUserContext = arg_pUserContext;

	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

	IX_ERROR_CGT (
		ix_cc_msup_send_async_msg (IX_CC_FP_MODULE_INGRESS_OUTPUT,
							(ix_cc_msghlp_callback)ix_cc_fpm_atm_vc_stats_get_cb_core,
							(void*) cbContext,
							IX_CC_FPM_MSG_ATM_VCC_STATS_GET,
							(void*) &vc_params,
							sizeof(ix_cc_fpm_atm_vc_handle_t)), 
		err,
		IX_CC_FPM_ERROR_MSG_LIBRARY,
		IX_ERROR_LEVEL_WARNING,
		cleanup);
	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);
    
	return IX_SUCCESS;

 cleanup:
    if (cbContext != NULL)
      ix_ossl_free (cbContext);

    return err;
}




ix_error ix_cc_fpm_async_cid_create(
				const ix_cc_atmsar_instance_t arg_sarInstance,
				const ix_cc_atmsar_cid_descr_t* arg_pCidDescr,
				ix_cc_atmsar_cb_cid_create_t arg_callback,
				void *arg_pUserContext)
{
	return 0;
}

ix_error ix_cc_fpm_async_cid_remove(
				const ix_cc_atmsar_instance_t arg_sarInstance,
				ix_cc_atmsar_cid_handle_t arg_cid,
				ix_cc_atmsar_cb_cid_remove_t arg_callback,
				void *arg_pUserContext)
{
	return 0;
}

ix_error ix_cc_fpm_async_get_cid_stats(
				const ix_cc_atmsar_instance_t arg_sarInstance,
				ix_cc_atmsar_cid_handle_t arg_cid,
				ix_cc_atmsar_cb_get_cid_stats_t arg_callback,
				void *arg_pUserContext)
{
	return 0;
}
#endif /*IX_CC_ATM*/

