/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2400 Network Processor, Release 5
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      FP Module Core Component
 *
 * = FILENAME
 *      ix_cc_fpm_proxy.h
 *
 * = DESCRIPTION
 *      This file defines the data types required by the FP Module egress
 *      "proxy" core component. This contains the basic function definitions
 *      of the FPModule egress CC. There are no external clients of this Core
 *      component.
 *
 * = AUTHOR
 *       Rajeev Muralidhar
 *       Rajeev.D.Muralidhar@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      12/28/2003
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: ix_cc_fpm_proxy.h,v 1.6.2.1 2004/03/24 11:39:53 smds Exp $
 */

#if !defined(__IX_CC_FPM_PROXY_H__)
#define __IX_CC_FPM_PROXY_H__

#include "ix_cc_error.h"
#if (_IX_OS_TYPE_ != _IX_OS_LINUX_KERNEL_)
#include <stdio.h>
#include <string.h>
#endif
#include "ix_ossl.h"
#include "ix_rm.h"
#include "ix_cc.h"
#include "ix_cc_properties.h"
#include "cc/ix_cc_msup.h"

#if !defined (IX_EXCLUDE_CCI)
#include "ix_cci.h"
#endif

#include "bindings.h"

#ifdef IX_CC_ETH
#include "cc/ix_cc_arp.h"
#include "cc/ix_cc_eth_tx.h"
#include "cc/ix_cc_l2tm.h"
#endif /* IX_CC_ETH */


#ifdef IX_CC_ATM_POS
#include "cc/ix_cc_atm_pos_tx.h"
#endif /* IX_CC_ATM */

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

/* Macros etc. */
#ifdef IX_DEBUG
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
#define FPM_CC_DBG_PRINT(fmt, args...) ix_ossl_message_log (fmt, ##args)
#define FPM_CC_DBG_TRACE() ix_ossl_message_log ("Entering %s \n", __FUNCTION__)
#else
#define FPM_CC_DBG_PRINT(fmt, args...) printf (fmt, ##args)
#define FPM_CC_DBG_TRACE() printf ("Entering %s \n", __FUNCTION__)
#endif
#else
#define FPM_CC_DBG_PRINT(fmt, args...)  /* nothing */
#define FPM_CC_DBG_TRACE() /* nothing */
#endif

typedef struct ix_s_cc_fpm_proxy_context
{
    ix_cc_handle   fpm_hFPMHandle; /* Handle of FPModule CC */
    ix_uint32      fpm_localBladeId; /* Local Blade id */
#ifdef IX_CC_ETH
    ix_cc_l2tm     fpm_hL2tm;
#endif
} ix_cc_fpm_proxy_context;

/****************************************************************************
 *               INIIALIZATION, SHUTDOWN, MESSAGE HANDLER                   *
 ***************************************************************************/
/**
 * NAME: ix_cc_fpm_proxy_init
 *
 * DESCRIPTION: This primitive shall be called and returned successfully before 
 *              requesting any service from FPModule Core component. This 
 *              primitive should be called only once.
 *              This function performs all initializations required by a core 
 *              component (registering msg handler). NO packet handler is required
 *              by this component. Also, this does not have any micro block 
 *              component, so symbol patching is not required. It also
 *              initializes the L2 Table Manager.
 * 
 * @Param:  - IN arg_hCcHandle - handle to FP Module core component, created by 
 *            core component infrastructure; this shall be used later to get 
 *            other services from core component infrastructure.
 *
 * @Param:  - OUT arg_ppContext - location where the pointer to the control 
 *            block allocated by FP Module core component will be stored. 
 *            The control block is internal to Forwarder core component and it 
 *            contains information about internal data structures, allocated 
 *            memory and other relevant information. Later this pointer to be 
 *            passed into ix_cc_fpm_fini function for freeing memory and other 
 *            house keeping operations when FPM component is being destroyed.  
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOM   - memory allocation failure      
  * IX_CC_FPM_ERROR_CCI - failure from CCI 
 * 
 */

ix_error ix_cc_fpm_proxy_init(
			      ix_cc_handle arg_hCcHandle,
			      void** arg_ppContext
			      );

/**
 * NAME: ix_cc_fpm_proxy_fini
 *
 * DESCRIPTION: This primitive shall be called to terminate services from 
 *              FP Module Core component. This primitive frees memory allocated 
 *              during initialization, any other cleaning up that needs to be
 *              done. It shuts down the L2 Table Manager.
 *              If client claims for any services from FPModule CC after calling
 *              this function, the behavior is undefined.
 * 
 * @Param:  - IN arg_hCcHandle - handle to FP Module core component
 *
 * @Param:  - IN arg_pContext - pointer to the control block memory was 
 *            allocated earlier in ix_cc_fpm_init function. 
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_INVALID_INPUT_PARAM - invalid input parameter(s)
 * IX_CC_ERROR_OOM   - failure of memory deletion                
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * 
 */
ix_error ix_cc_fpm_proxy_fini(
			      ix_cc_handle arg_hCcHandle,
			      void* arg_pContext
			      );

/**
 * NAME: ix_cc_fpm_proxy_msg_handler
 *
 * DESCRIPTION: This primitive is the message handler function for FP Module
 *              core component. FP Module egress core component receives 
 *              messages from its ingress counterpart through this message 
 *              handler function and internally it processes the message. 
 * 
 * @Param:  - IN arg_hDataToken - buffer handle embedding information for the 
 *            message passed in arg_UserData.
 *
 * arg_UserData - message type
 *
 * @Param:  - OUT - arg_pComponentContext: pointer to FP Module core component 
 *            context that will be passed to the core component when a message 
 *            arrives. This context was defined by the core component and passed
 *            to core components infrastructure through ix_cc_fpm_init function.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - null argument
 * IX_CC_FPM_ERROR_INVALID_CONTEXT - context is unusable
 * IX_CC_ERROR_UNDEFINED_MSG - un supported message
 * IX_CC_FPM_ERROR_MSG_LIBRARY - error from message support library
 * 
 */
ix_error ix_cc_fpm_proxy_msg_handler (
				      ix_buffer_handle arg_hDataToken,
				      ix_uint32 arg_UserData,
				      void* arg_pContext
				      );

ix_error ix_cc_fpm_init_l2tm (void);

ix_error ix_cc_fpm_terminate_l2tm (void);


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_FPM_PROXY_H__) */


