/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      FP Module Core Component - CCI - init function
 *
 * = FILENAME
 *      ix_cc_fpm_init.c
 *
 * = DESCRIPTION
 *      This file the basic functions of the FPModule egress CC - init() and 
 *      fini() functions, message handler & internal library API, and internal
 *      callback functions. There are no external clients of this Core
 *      component.
 *
 * = AUTHOR
 *       Rajeev Muralidhar
 *       Rajeev.D.Muralidahr@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      12/28/2002
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */
/*#define IX_FILE_IDENT "$Id: ix_cc_fpm_proxy.c,v 1.7.2.1 2004/03/24 11:39:53 smds Exp $";*/

#include "cc/ix_cc_fpm_core.h"
#include "cc/ix_cc_fpm_proxy.h"
#include "cc/internal/internal_msup.h"

#if !defined(IX_INCLUDE_REGISTRY) /* configuration values header file */
#include "ix_config.h"
#endif

#define REPOSITORY_PATH_LOCAL_BLADE_ID "System_Properties/BLADE_ID"

/* Macros, local functions, etc. */
ix_cc_fpm_proxy_context* g_pFPMContext;

#ifdef IX_CC_ETH
static void ix_cc_fpm_dump_context (ix_cc_fpm_proxy_context* context);
static void ix_cc_fpm_dump_l2_info (ix_cc_fpm_add_l2_entry_data *l2_info);
static void ix_cc_fpm_dump_arp_info (ix_cc_eth_tx_next_hop_info *arp_info);
#endif /*IX_CC_ETH*/

#ifdef IX_CC_ATM

/**
 * NAME: ix_cc_llc_snap_encap_fpm_add_next_hop_cb_proxy
 *
 * DESCRIPTION: This callback function for LLC SNAP Add next hop 
 * operation
 * @Param:  - IN  - ix_error 
 *						- void *arg_pContext
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *          Return code is not interpreted by the core component.
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error ix_cc_llc_snap_encap_fpm_add_next_hop_cb_proxy(
			ix_error arg_Result,
			void *arg_pContext)
{

	ix_error err = IX_SUCCESS;
	ix_error result = IX_SUCCESS;

	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);    
	FPM_CC_DBG_TRACE ();

	if(arg_Result != IX_SUCCESS)
	{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("\nFPM PROXY - LLC SNAP NEXT HOP ADD ERROR\n");
#else		
      printf("\nFPM PROXY - LLC SNAP NEXT HOP ADD ERROR\n");
		ix_error_dump(stdout,arg_Result);
#endif
		return arg_Result;
	}

	if (arg_pContext == NULL)
	{
		return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
			("Internal callback invoked with NULL context"));
	}
	result = arg_Result;

	IX_ERROR_CGT (ix_cc_msup_send_reply_msg (arg_pContext,
								(void*)&result,
								sizeof(ix_error),
								IX_SUCCESS),
				err,
				IX_CC_FPM_MSG_LLC_SNAP_ENCAP_ADD_NEXT_HOP,
				IX_ERROR_LEVEL_WARNING,
				label);

	return IX_SUCCESS;

	label:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
		return err;
}

/**
 * NAME: ix_cc_llc_snap_encap_fpm_del_next_hop_cb_proxy
 *
 * DESCRIPTION: This callback function for LLC SNAP Delete next hop 
 * operation
 * @Param:  - IN  - ix_error 
 *						- void *arg_pContext
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *          Return code is not interpreted by the core component.
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */

ix_error ix_cc_llc_snap_encap_fpm_del_next_hop_cb_proxy(
			ix_error arg_Result,
			void *arg_pContext)
{

	ix_error err = IX_SUCCESS;
	ix_error result = IX_SUCCESS;

	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);    
	FPM_CC_DBG_TRACE ();

	if(arg_Result != IX_SUCCESS)
	{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("\nFPM PROXY - LLC SNAP NEXT HOP DEL ERROR\n");
#else		
      printf("\nFPM PROXY - LLC SNAP NEXT HOP DEL ERROR\n");
		ix_error_dump(stdout,arg_Result);
#endif
		return arg_Result;
	}

	if (arg_pContext == NULL)
	{
		return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
			("Internal callback invoked with NULL context"));
	}
	result = arg_Result;

	IX_ERROR_CGT (ix_cc_msup_send_reply_msg (arg_pContext,
								(void*)&result,
								sizeof(ix_error),
								IX_SUCCESS),
				err,
				IX_CC_FPM_MSG_LLC_SNAP_ENCAP_DEL_NEXT_HOP,
				IX_ERROR_LEVEL_WARNING,
				label);

	return IX_SUCCESS;

	label:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
		return err;
}





/**
 * NAME: ix_cc_fpm_atm_port_create_cb_proxy
 *
 * DESCRIPTION: This callback function for ATM Port Create operation
 *  
 * @Param:  - IN  - ix_error 
 *						- ix_cc_atmsar_port_handle_t
 *						- void *arg_pContext
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *          Return code is not interpreted by the core component.
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error ix_cc_fpm_atm_port_create_cb_proxy(
			ix_error arg_Result,
			ix_cc_atmsar_port_handle_t port,
			void *arg_pContext)
{
	ix_error err = IX_SUCCESS;
	ix_cc_fpm_atm_port_create_resp_t result;

	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);    
	FPM_CC_DBG_TRACE ();

	if(arg_Result != IX_SUCCESS)
	{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("\nFPM PROXY - PORT CREATE ERROR: %ld\n", arg_Result);
#else		
		printf("\nFPM PROXY - PORT CREATE ERROR\n");
		ix_error_dump(stdout,arg_Result);
#endif

	}

	if (arg_pContext == NULL)
			return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
					("Internal callback invoked with NULL context"));

	/* Initialize the result structure */
	ix_ossl_memcpy(&result, 0, sizeof (ix_cc_fpm_atm_port_create_resp_t));
	result.arg_Result = arg_Result;
	result.port = port;


	IX_ERROR_CGT (ix_cc_msup_send_reply_msg (arg_pContext,
								(void*)&result,
								sizeof(ix_cc_fpm_atm_port_create_resp_t),
								IX_SUCCESS),
				err,
				IX_CC_FPM_MSG_ATM_PORT_CREATE,
				IX_ERROR_LEVEL_WARNING,
				label);

	return IX_SUCCESS;

	label:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
		return err;
}



/**
 * NAME: ix_cc_fpm_atm_port_del_cb_proxy
 *
 * DESCRIPTION: This callback function for ATM Port delete operation
 *  
 * @Param:  - IN  - ix_error 
 *						- ix_cc_atmsar_port_handle_t
 *						- void *arg_pContext
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *          Return code is not interpreted by the core component.
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error ix_cc_fpm_atm_port_del_cb_proxy(
			ix_error arg_Result,
			void *arg_pContext)
{
	ix_error err = IX_SUCCESS;

	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);    
	FPM_CC_DBG_TRACE ();

	if(arg_Result != IX_SUCCESS)
	{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("\nFPM PROXY - PORT DELETE ERROR\n");
#else		
		printf("\nFPM PROXY - PORT DELETE ERROR\n");
		ix_error_dump(stdout,arg_Result);
#endif

		return arg_Result;
	}


	if (arg_pContext == NULL)
			return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
					("Internal callback invoked with NULL context"));


	IX_ERROR_CGT (ix_cc_msup_send_reply_msg (arg_pContext,
								(void*)arg_Result,
								sizeof(ix_error),
								IX_SUCCESS),
				err,
				IX_CC_FPM_MSG_ATM_PORT_DELETE,
				IX_ERROR_LEVEL_WARNING,
				label);

	return IX_SUCCESS;

	label:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
		return err;
}


/**
 * NAME: ix_cc_fpm_atm_port_stats_get_cb_proxy
 *
 * DESCRIPTION: This callback function for ATM Port Statistics Get operation
 *  
 * @Param:  - IN  - ix_error 
 *						- ix_cc_atmsar_port_handle_t
 *						- void *arg_pContext
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *          Return code is not interpreted by the core component.
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error ix_cc_fpm_atm_port_stats_get_cb_proxy(
			ix_error arg_Result,
			ix_cc_atmsar_port_stats_t *arg_pLookupResult,
			void *arg_pContext)
{
	ix_error err = IX_SUCCESS;
	ix_cc_fpm_atm_port_stats_get_resp_t result;

	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);    
	FPM_CC_DBG_TRACE ();

	if(arg_Result != IX_SUCCESS)
	{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("\nFPM PROXY - PORT DELETE ERROR: %ld\n", arg_Result);
#else		
		printf("\nFPM PROXY - PORT DELETE ERROR\n");
		ix_error_dump(stdout,arg_Result);
#endif

		return arg_Result;
	}


	if (arg_pContext == NULL)
			return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
					("Internal callback invoked with NULL context"));

	/* Initialize the result structure */
	ix_ossl_memcpy(&result, 0, sizeof (ix_cc_fpm_atm_port_stats_get_resp_t));
	result.arg_Result = arg_Result;
	ix_ossl_memcpy(&result.arg_pLookupResult, arg_pLookupResult, sizeof (ix_cc_atmsar_port_stats_t));

	IX_ERROR_CGT (ix_cc_msup_send_reply_msg (arg_pContext,
								(void*)&result,
								sizeof(ix_cc_fpm_atm_port_stats_get_resp_t),
								IX_SUCCESS),
				err,
				IX_CC_FPM_MSG_ATM_PORT_STATS_GET,
				IX_ERROR_LEVEL_WARNING,
				label);

	return IX_SUCCESS;

	label:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
		return err;
}



/**
 * NAME: ix_cc_fpm_atm_vcc_create_cb_proxy
 *
 * DESCRIPTION: This callback function for ATM VCC Create operation
 *  
 * @Param:  - IN  - ix_error 
 *						- ix_cc_atmsar_port_handle_t
 *						- void *arg_pContext
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *          Return code is not interpreted by the core component.
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error ix_cc_fpm_atm_vcc_create_cb_proxy(
			ix_error arg_Result,
			ix_cc_atmsar_vc_handle_t arg_hVc,
			void *arg_pContext)
{
	ix_error err = IX_SUCCESS;
	ix_cc_fpm_atm_vc_create_resp_t result;

	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);    
	FPM_CC_DBG_TRACE ();

	if(arg_Result != IX_SUCCESS)
	{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("\nFPM PROXY - VCC CREATE ERROR: %ld\n", arg_Result);
#else		
		printf("\n FPM PROXY - VCC CREATE ERROR \n");
		ix_error_dump(stdout,arg_Result);
#endif

		return arg_Result;
	}

	if (arg_pContext == NULL)
			return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
					("Internal callback invoked with NULL context"));

	/* Initialize the result structure */
	ix_ossl_memcpy(&result, 0, sizeof (ix_cc_fpm_atm_vc_create_resp_t));
	result.arg_Result = arg_Result;
	result.arg_hVc = arg_hVc;


	IX_ERROR_CGT (ix_cc_msup_send_reply_msg (arg_pContext,
								(void*)&result,
								sizeof(ix_cc_fpm_atm_vc_create_resp_t),
								IX_SUCCESS),
				err,
				IX_CC_FPM_MSG_ATM_VCC_CREATE,
				IX_ERROR_LEVEL_WARNING,
				label);

	return IX_SUCCESS;

	label:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
		return err;
}



/**
 * NAME: ix_cc_fpm_atm_vcc_update_cb_proxy
 *
 * DESCRIPTION: This callback function for ATM VCC Update operation
 *  
 * @Param:  - IN  - ix_error 
 *						- ix_cc_atmsar_port_handle_t
 *						- void *arg_pContext
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *          Return code is not interpreted by the core component.
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */

ix_error ix_cc_fpm_atm_vcc_update_cb_proxy(
			ix_error arg_Result,
			ix_cc_atmsar_vc_handle_t arg_hVc,
			void *arg_pContext)
{
	ix_error err = IX_SUCCESS;
	ix_cc_fpm_atm_vc_create_resp_t result;

	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);    
	FPM_CC_DBG_TRACE ();

	if(arg_Result != IX_SUCCESS)
	{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("\nFPM PROXY - VC UPDATE ERROR: %ld\n", arg_Result);
#else		
		printf("\nFPM PROXY - VC UPDATE ERROR\n");
		ix_error_dump(stdout,arg_Result);
#endif

		return arg_Result;
	}

	if (arg_pContext == NULL)
			return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
					("Internal callback invoked with NULL context"));

	/* Initialize the result structure */
	ix_ossl_memcpy(&result, 0, sizeof (ix_cc_fpm_atm_vc_create_resp_t));
	result.arg_Result = arg_Result;
	result.arg_hVc = arg_hVc;


	IX_ERROR_CGT (ix_cc_msup_send_reply_msg (arg_pContext,
								(void*)&result,
								sizeof(ix_cc_fpm_atm_vc_create_resp_t),
								IX_SUCCESS),
				err,
				IX_CC_FPM_MSG_ATM_VCC_UPDATE,
				IX_ERROR_LEVEL_WARNING,
				label);

	return IX_SUCCESS;

	label:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
		return err;
}

/**
 * NAME: ix_cc_fpm_atm_vcc_remove_cb_proxy
 *
 * DESCRIPTION: This callback function for ATM VCC Delete operation
 *  
 * @Param:  - IN  - ix_error 
 *						- ix_cc_atmsar_port_handle_t
 *						- void *arg_pContext
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *          Return code is not interpreted by the core component.
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */

ix_error ix_cc_fpm_atm_vcc_remove_cb_proxy(
			ix_error arg_Result,
			void *arg_pContext)
{
	ix_error err = IX_SUCCESS;
	ix_error result = IX_SUCCESS;

	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);    
	FPM_CC_DBG_TRACE ();

	if(arg_Result != IX_SUCCESS)
	{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("\nFPM PROXY - VC DELETE ERROR: %d\n", arg_Result);
#else		
		printf("\nFPM PROXY - VC DELETE ERROR\n");
		ix_error_dump(stdout,arg_Result);
#endif

		return arg_Result;
	}

	if (arg_pContext == NULL)
			return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
					("Internal callback invoked with NULL context"));
	result = arg_Result;

	IX_ERROR_CGT (ix_cc_msup_send_reply_msg (arg_pContext,
								(void*)&result,
								sizeof(ix_error),
								IX_SUCCESS),
				err,
				IX_CC_FPM_MSG_ATM_VCC_DELETE,
				IX_ERROR_LEVEL_WARNING,
				label);

	return IX_SUCCESS;

	label:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
		return err;
}

/**
 * NAME: ix_cc_fpm_atm_vcc_stats_get_cb_proxy
 *
 * DESCRIPTION: This callback function for ATM VCC Statistics Get operation
 *  
 * @Param:  - IN  - ix_error 
 *						- ix_cc_atmsar_port_handle_t
 *						- void *arg_pContext
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *          Return code is not interpreted by the core component.
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error ix_cc_fpm_atm_vcc_stats_get_cb_proxy(
			ix_error arg_Result,
			ix_cc_atmsar_vc_stats_t *arg_pLookupResult,
			void *arg_pContext)
{
	ix_error err = IX_SUCCESS;
	ix_cc_fpm_atm_vc_stats_get_resp_t result;

	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);    
	FPM_CC_DBG_TRACE ();

	if(arg_Result != IX_SUCCESS)
	{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("\nFPM PROXY - VC Statistics get error: %d\n", arg_Result);
#else		
		printf("\nFPM PROXY - VC Statistics Get ERROR\n");
		ix_error_dump(stdout,arg_Result);
#endif

		return arg_Result;
	}

	if (arg_pContext == NULL)
			return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
					("Internal callback invoked with NULL context"));


	/* Initialize the result structure */
	ix_ossl_memcpy(&result, 0, sizeof (ix_cc_fpm_atm_vc_stats_get_resp_t));
	result.arg_Result = arg_Result;
	ix_ossl_memcpy(&result.arg_pLookupResult,arg_pLookupResult, sizeof (ix_cc_atmsar_vc_stats_t));

	IX_ERROR_CGT (ix_cc_msup_send_reply_msg (arg_pContext,
								(void*)&result,
								sizeof(ix_cc_fpm_atm_vc_stats_get_resp_t),
								IX_SUCCESS),
				err,
				IX_CC_FPM_MSG_ATM_VCC_STATS_GET,
				IX_ERROR_LEVEL_WARNING,
				label);

	return IX_SUCCESS;

	label:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
		return err;
}

#endif /*IX_CC_ATM*/

#ifdef IX_CC_QOS
/**
 * NAME: ix_cc_fpm_dpe_callback
 *
 * DESCRIPTION: This callback function for WRED DPE operation
 *  
 * @Param:  - IN  - ix_cc_fpm_proxy_context* context 
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *          Return code is not interpreted by the core component.
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error ix_cc_fpm_dpe_callback (ix_error arg_Result, void * arg_pUserContext )
{
    ix_error err = IX_SUCCESS;
    FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);    
    FPM_CC_DBG_TRACE ();
    if (arg_pUserContext == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
				("Internal callback invoked with NULL context"));
    IX_ERROR_CGT (ix_cc_msup_send_reply_msg (arg_pUserContext,
					     (void*) arg_Result,
					     sizeof(arg_Result),
					     IX_SUCCESS),
		  err,
		  IX_CC_FPM_MSG_DPE_OP, IX_ERROR_LEVEL_WARNING,
		  label);
    return IX_SUCCESS;

 label:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
    return err;
}

/**
 * NAME: ix_cc_fpm_dpe_meter_statisticss_cb
 *
 * DESCRIPTION: This callback function for WRED DPE operation
 *  
 * @Param:  - IN  - ix_cc_fpm_proxy_context* context 
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *          Return code is not interpreted by the core component.
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error ix_cc_fpm_dpe_meter_statistics_cb (ix_error arg_Result, 
				ix_cc_tc_meter_statistics * arg_pStats,
				void * arg_pUserContext )
{
    ix_error err = IX_SUCCESS;
    FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);    
    FPM_CC_DBG_TRACE ();
    if (arg_pUserContext == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
				("Internal callback invoked with NULL context"));
    IX_ERROR_CGT (ix_cc_msup_send_reply_msg (arg_pUserContext,
					     (void*) arg_pStats,
					     sizeof(ix_cc_wred_statistics),
					     IX_SUCCESS),
		  err,
		  IX_CC_FPM_MSG_DPE_OP, IX_ERROR_LEVEL_WARNING,
		  label);
    return IX_SUCCESS;

 label:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
    return err;
}

/**
 * NAME: ix_cc_fpm_dpe_wred_dropper_statisticss_cb
 *
 * DESCRIPTION: This callback function for WRED DPE operation
 *  
 * @Param:  - IN  - ix_cc_fpm_proxy_context* context 
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *          Return code is not interpreted by the core component.
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */
ix_error ix_cc_fpm_dpe_wred_dropper_statistics_cb (ix_error arg_Result, 
				ix_cc_wred_statistics * arg_pStats,
				void * arg_pUserContext )
{
    ix_error err = IX_SUCCESS;
    FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);    
    FPM_CC_DBG_TRACE ();
    if (arg_pUserContext == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
				("Internal callback invoked with NULL context"));
    IX_ERROR_CGT (ix_cc_msup_send_reply_msg (arg_pUserContext,
					     (void*) arg_pStats,
					     sizeof(ix_cc_wred_statistics),
					     IX_SUCCESS),
		  err,
		  IX_CC_FPM_MSG_DPE_OP, IX_ERROR_LEVEL_WARNING,
		  label);
    return IX_SUCCESS;

 label:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
    return err;
}
#endif /*IX_CC_QOS*/

#ifdef IX_CC_ETH

/**
 * NAME: _ix_cc_fpm_configure_l2tm
 *
 * DESCRIPTION: This primitive will be used to configure L2 Table Mgr.
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT  none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_L2TM - error from L2 table mgr
 * 
 */

ix_error ix_cc_fpm_init_l2tm ()
{
    ix_error err = IX_SUCCESS;
    char name[32];

    /* Initalize L2 Table Mgr */
    
    strcpy (name, "DEFAULT");
    
    IX_ERROR_CGT (ix_cc_l2tm_create ("DEFAULT", &g_pFPMContext->fpm_hL2tm),
		  err,
		  IX_CC_FPM_ERROR_L2TM,
		  IX_ERROR_LEVEL_WARNING,
		  label);
    
    return IX_SUCCESS;
    
 label:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
    return err;
}/* _ix_cc_fpm_init_l2tm */
 
/**
 * NAME: x_cc_fpm_terminate_l2tm()
 *
 * DESCRIPTION: This primitive will be used to terminate L2 TM services: 
 * 
 * @Param:  - IN none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_RTM - error from RTM
 * 
 */

ix_error ix_cc_fpm_terminate_l2tm ()
{
    ix_error err = IX_SUCCESS;
    
    /* Destroy L2 Table Mgr */
    
    IX_ERROR_CGT (ix_cc_l2tm_destroy (g_pFPMContext->fpm_hL2tm),
		  err,
		  IX_CC_FPM_ERROR_L2TM,
		  IX_ERROR_LEVEL_WARNING,
		  label);

    return IX_SUCCESS;
 label:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
    return err;
}/* end _ix_cc_fpm_terminate_rtm_services */
#endif /*IX_CC_ETH*/
   
/**
 * NAME: ix_cc_fpm_proxy_init
 *
 * DESCRIPTION: This primitive shall be called and returned successfully before 
 *              requesting any service from FPModule Core component. This 
 *              primitive should be called only once.
 *              On the ingress, this function performs all initializations 
 *              required by a core component (registering msg handler). NO 
 *              packet handler is required by this component. Also, this does 
 *              not have any micro block component, so symbol patching is done. 
 *              It also initializes the L2 Table Manager.
 * 
 * @Param:  - IN arg_hCcHandle - handle to FP Module core component, created by 
 *            core component infrastructure; this shall be used later to get 
 *            other services from core component infrastructure.
 *
 * @Param:  - OUT arg_ppContext - location where the pointer to the control 
 *            block allocated by FP Module core component will be stored. 
 *            The control block is internal to Forwarder core component and it 
 *            contains information about internal data structures, allocated 
 *            memory and other relevant information. Later this pointer to be 
 *            passed into ix_cc_fpm_fini function for freeing memory and other 
 *            house keeping operations when FPM component is being destroyed.  
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_OOM   - memory allocation failure      
  * IX_CC_FPM_ERROR_CCI - failure from CCI 
 * 
 */
ix_error ix_cc_fpm_proxy_init(
			      ix_cc_handle arg_hCcHandle,
			      void** arg_ppContext
			      )
{
    ix_error err = IX_SUCCESS;
    ix_cc_init_context* init_context = (ix_cc_init_context*) *arg_ppContext;
    ix_cc_properties_node *pPropNode;


    FPM_CC_DBG_TRACE ();
    if (*arg_ppContext == NULL) 
	return IX_ERROR_WARNING (IX_CC_ERROR_NULL, ("FP Module CC init called"
						    " with NULL context"));

    /* allocate memory for FP module control structure */
    g_pFPMContext = ix_ossl_malloc (sizeof (ix_cc_fpm_proxy_context));
    if (g_pFPMContext == NULL)
	return IX_ERROR_WARNING (IX_CC_ERROR_OOM_SYSTEM, 
				 ("Could not allocate"
				  "memory for FPM ctx"));

    g_pFPMContext->fpm_hFPMHandle = arg_hCcHandle;
    /**
    * Retrieve local blade property information from init context
    * (This will work for both level-1 and level-0 modes)
    */
    pPropNode = init_context->pPropertyList;
    /* Get blade Id from first node */
    if(pPropNode != NULL)
    {
        g_pFPMContext->fpm_localBladeId  =
        pPropNode->pPropertyInfo->blade_id;
    }/* end if */


#ifdef IX_CC_ETH
    ix_cc_fpm_init_l2tm();
#endif /*IX_CC_ETH*/
    
    /* now register message handler with CCI/RM */
#if defined(IX_EXCLUDE_CCI)
    FPM_CC_DBG_PRINT ("IN %s %d \n", __FUNCTION__, __LINE__);
    IX_ERROR_CRT (ix_rm_message_handler_register (IX_CC_FP_MODULE_EGRESS_INPUT,
						  ix_cc_fpm_proxy_msg_handler,
						  g_pFPMContext),
		  IX_CC_FPM_ERROR_ADD_HANDLER,
		  IX_ERROR_LEVEL_WARNING);
#else
    FPM_CC_DBG_PRINT ("IN %s %d \n", __FUNCTION__, __LINE__);
    IX_ERROR_CGT (ix_cci_cc_add_message_handler  (g_pFPMContext->fpm_hFPMHandle,
						  IX_CC_FP_MODULE_EGRESS_INPUT,
						  ix_cc_fpm_proxy_msg_handler,
						  IX_INPUT_TYPE_SINGLE_SRC),
		  err,
		  IX_CC_FPM_ERROR_ADD_HANDLER,
		  IX_ERROR_LEVEL_WARNING, label_1);
    FPM_CC_DBG_PRINT ("IN %s %d \n", __FUNCTION__, __LINE__);
#endif

    /* store the FPM context in the context passed in */
    *arg_ppContext = (void*) g_pFPMContext;
    ix_cc_fpm_dump_context (g_pFPMContext);

    return IX_SUCCESS;

 label_1:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
    return err;
} /* end ix_cc_fpm_init */

/**
 * NAME: ix_cc_fpm_proxy_fini
 *
 * DESCRIPTION: This primitive shall be called to terminate services from 
 *              FP Module core component. 
 * 
 * @Param:  - IN arg_hCcHandle - handle to FP Module core component
 *
 * @Param:  - IN arg_pContext - pointer to the control block memory allocated 
 * earlier in ix_cc_fpm_init function. 
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_INVALID_INPUT_PARAM - invalid input parameter(s)
 * IX_CC_ERROR_OOM - failure of memory deletion
 * IX_CC_FPM_ERROR_CCI - failure from CCI
 * 
 */

ix_error ix_cc_fpm_proxy_fini (
			       ix_cc_handle arg_hCcHandle,
			       void* arg_pContext
			       )
{
    if (arg_pContext == NULL)
	return IX_ERROR_WARNING (IX_CC_ERROR_NULL, ("FP Module CC fini called"
						    " with NULL context"));
    g_pFPMContext = (ix_cc_fpm_proxy_context*) arg_pContext;
    ix_cc_fpm_dump_context (g_pFPMContext);

#ifdef IX_CC_ETH
    /* shutdown L2 Table Manager */
    ix_cc_fpm_terminate_l2tm ();
#endif /*IX_CC_ETH*/
    
#if defined(IX_EXCLUDE_CCI)
    IX_ERROR_CRT(ix_rm_message_handler_unregister(IX_CC_FP_MODULE_EGRESS_INPUT),
		 IX_CC_FPM_ERROR_CCI,
		 IX_ERROR_LEVEL_WARNING);
#else
    IX_ERROR_CRT(ix_cci_cc_remove_message_handler(IX_CC_FP_MODULE_EGRESS_INPUT),
		 IX_CC_FPM_ERROR_CCI,
		 IX_ERROR_LEVEL_WARNING);
#endif
    ix_ossl_free ((ix_cc_fpm_proxy_context*) arg_pContext);
    
    return IX_SUCCESS;
} /* end ix_cc_fpm_fini */

#ifdef IX_CC_ETH
/**
 * NAME: ix_cc_fpm_create_arp_cb
 *
 * DESCRIPTION: This is the internal callback function for create ARP operation
 * 
 * @Param:  - IN ix_error arg_Result - error code returned from
 *          the asynchronous operation.
 * @Param:  - IN void* arg_pContext - pointer to context 
 * @Param:  - IN void* arg_pMsg - pointer to reply message.
 * @Param:  - IN ix_uint32 arg_MsgLen - length of reply message.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_NULL
 */
ix_error ix_cc_fpm_create_arp_cb (ix_uint32 arg_Result,
				  void*     arg_pContext)
{
    ix_error err = IX_SUCCESS;

    FPM_CC_DBG_TRACE ();
    
    if (arg_pContext == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
				("Internal callback invoked with NULL context"));
    IX_ERROR_CGT (ix_cc_msup_send_reply_msg (arg_pContext,
					     (void*) arg_Result,
					     sizeof (ix_uint32),
					     IX_SUCCESS),
		  err,
		  IX_CC_FPM_ERROR_ARP, IX_ERROR_LEVEL_WARNING,
		  label);
    return IX_SUCCESS;

 label:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
    return err;
}

/**
 * NAME: ix_cc_fpm_add_arp_cb
 *
 * DESCRIPTION: This is the internal callback function for add ARP operation
 * 
 * @Param:  - IN ix_error arg_Result - error code returned from
 *          the asynchronous operation.
 * @Param:  - IN void* arg_pContext - pointer to context 
 * @Param:  - IN void* arg_pMsg - pointer to reply message.
 * @Param:  - IN ix_uint32 arg_MsgLen - length of reply message.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_NULL
 */
ix_error ix_cc_fpm_add_arp_cb (ix_uint32 arg_Result,
			       void*     arg_pContext)
{
    ix_error err = IX_SUCCESS;

    FPM_CC_DBG_TRACE ();
    
    if (arg_pContext == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
				("Internal callback invoked with NULL context"));
    IX_ERROR_CGT (ix_cc_msup_send_reply_msg (arg_pContext,
					     (void*) arg_Result,
					     sizeof (ix_uint32),
					     IX_SUCCESS),
		  err,
		  IX_CC_FPM_ERROR_ARP, IX_ERROR_LEVEL_WARNING,
		  label);
    return IX_SUCCESS;

 label:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
    return err;
}

/**
 * NAME: ix_cc_fpm_del_arp_cb
 *
 * DESCRIPTION: This is the internal callback function for del ARP operation
 * 
 * @Param:  - IN ix_error arg_Result - error code returned from
 *          the asynchronous operation.
 * @Param:  - IN void* arg_pContext - pointer to context 
 * @Param:  - IN void* arg_pMsg - pointer to reply message.
 * @Param:  - IN ix_uint32 arg_MsgLen - length of reply message.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_NULL
 */
ix_error ix_cc_fpm_del_arp_cb (ix_uint32 arg_Result,
			       void*     arg_pContext)
{
    ix_error err = IX_SUCCESS;

    FPM_CC_DBG_TRACE ();
    
    if (arg_pContext == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
				("Internal callback invoked with NULL context"));
    IX_ERROR_CGT (ix_cc_msup_send_reply_msg (arg_pContext,
					(void*) arg_Result,
					sizeof (ix_uint32),
					IX_SUCCESS),
		  err,
		  IX_CC_FPM_ERROR_ARP, IX_ERROR_LEVEL_WARNING,
		  label);
    return IX_SUCCESS;

 label:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
    return err;
}

/**
 * NAME: ix_cc_fpm_eth_tx_stats_cb
 *
 * DESCRIPTION: This is the internal callback function for retrieving Eth Tx
 *              statistics.
 * 
 * @Param:  - IN ix_error arg_Result - error code returned from
 *          the asynchronous operation.
 * @Param:  - IN void* arg_pContext - pointer to context 
 * @Param:  - IN void* arg_pMsg - pointer to reply message.
 * @Param:  - IN ix_uint32 arg_MsgLen - length of reply message.
 * @Return: IX_SUCCESS
 *          IX_CC_ERROR_NULL
 */
ix_error ix_cc_fpm_eth_tx_stats_cb (ix_uint32 arg_Result,
				    void*     arg_pContext,
				    ix_uint64 *arg_pBuffer,
				    ix_uint32 arg_MsgLen)
{
    ix_error err = IX_SUCCESS;

    FPM_CC_DBG_TRACE ();
    
    if (arg_pContext == NULL)
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, 
				("Internal callback invoked with NULL context"));
    IX_ERROR_CGT (ix_cc_msup_send_reply_msg (arg_pContext,
					     (void*) arg_pBuffer,
					     arg_MsgLen,
					     IX_SUCCESS),
		  err,
		  IX_CC_FPM_ERROR_ARP, IX_ERROR_LEVEL_WARNING,
		  label);
    return IX_SUCCESS;

 label:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
    return err;
}

/**
 * NAME: ix_cc_fpm_add_l2_entry
 *
 * DESCRIPTION: This library function is used to add a L2 entry into the 
 *              L2 Table Manager. FP Module egress core component receives 
 *              this message from its ingress counterpart in its message
 *              handler. 
 * 
 * @Param:  - IN l2_id - L2 ID to be added to the L2 Table
 *            IN nexthop_ip - IP Address of the Next hop
 *            IN output_port - outgoing port for the Next hop
 *            IN arg_pContext - pointer to context
 *
 * @Param:  - OUT - None
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_ERROR_NULL - null argument
 * IX_CC_FPM_ERROR_INVALID_CONTEXT - context is unusable
 * IX_CC_ERROR_UNDEFINED_MSG - un supported message
 * IX_CC_FPM_ERROR_MSG_LIBRARY - error from message support library
 * 
 */
ix_error ix_cc_fpm_add_l2_entry (
				 ix_cc_fpm_add_l2_entry_data* arg_L2Info,
				 void* arg_pContext
				 )
{
    ix_error err = IX_SUCCESS;
    ix_cc_l2tm_entry* l2_info;
    ix_cc_eth_tx_next_hop_info arpinfo;

    FPM_CC_DBG_TRACE ();
    
    ix_cc_fpm_dump_l2_info (arg_L2Info);
    l2_info = (ix_cc_l2tm_entry*) &arg_L2Info->l2tm_info;

    IX_ERROR_CGT (ix_cc_l2tm_add_l3_info (g_pFPMContext->fpm_hL2tm,
					  arg_L2Info->l2_id, 
					  ( const ix_cc_l2tm_ipaddr*)&arg_L2Info->l2tm_info.l3Address.ipv4Address,
					  IX_CC_L2TM_IPADDR_IPV4),
		  err, 
		  IX_CC_FPM_ERROR_L2TM,
		  IX_ERROR_LEVEL_LOCAL, 
		  label);
    
    /* Make call into Ethernet Tx core component to generate ARP request */
    
    arpinfo.l2Index = arg_L2Info->l2_id;
    
    arpinfo.outputPort = arg_L2Info->output_port;

    switch (l2_info->l2AddrType) {
    case IX_CC_L2TM_L2ADDR_ETHER:
	FPM_CC_DBG_PRINT ("L2 addr type = Ethernet\n");
	break;
    case IX_CC_L2TM_L2ADDR_ATM:
	FPM_CC_DBG_PRINT ("L2 addr type = ATM\n");
	break;
    default:
	FPM_CC_DBG_PRINT ("L2 addr type = Unknown!\n");
	break;
    }
    
    switch (l2_info->ipAddrType) {
    case IX_CC_L2TM_IPADDR_IPV4:
	FPM_CC_DBG_PRINT ("IP addr type = IPv4\n IPAddress = 0x%x\n",
			  (ix_uint32)l2_info->l3Address.ipv4Address);
	arpinfo.nextHopIp = l2_info->l3Address.ipv4Address;
	break;
    case IX_CC_L2TM_IPADDR_IPV6:
	FPM_CC_DBG_PRINT ("IP addr type = IPv6\n IPAddress = 0x%x\n",
			  l2_info->l3Address.ipv6Address);
/*
  No need to anything regarding ARP for an IPv6 address since IPv6 has a separate
  ARP/Neighbor discovery component.
  arpinfo.nextHopIp = l2_info->l3Address.ipv6Address;
*/
	break;
    default:
	FPM_CC_DBG_PRINT ("IP addr type = Unknown!\n");
	break;
    }


    IX_ERROR_CGT (ix_cc_eth_tx_async_create_arp_entry (&arpinfo, 
						       ix_cc_fpm_create_arp_cb,
						       arg_pContext),
		  err, IX_CC_FPM_ERROR_ARP, IX_ERROR_LEVEL_WARNING,
		  label);
		  
    return IX_SUCCESS;

 label:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
    /* send error back to caller with error */
    err = ix_cc_msup_send_reply_msg (arg_pContext, 
				     (void*) err,
				     sizeof (ix_uint32),
				     err);
    return err;
}

/**
 * NAME: ix_cc_fpm_del_l2_entry
 *
 * DESCRIPTION: This library function is used to remove a L2 entry from the 
 *              L2 Table Manager. FP Module egress core component receives 
 *              this message from its ingress counterpart in its message
 *              handler. 
 * 
 * @Param:  - IN l2_id - L2 ID to be removed from the L2 Table
 *            IN arg_pContext - pointer to context
 *
 * @Param:  - OUT - None
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_ERROR_NULL - null argument
 * IX_CC_FPM_ERROR_INVALID_CONTEXT - context is unusable
 * IX_CC_ERROR_UNDEFINED_MSG - un supported message
 * IX_CC_FPM_ERROR_MSG_LIBRARY - error from message support library
 * 
 */
ix_error ix_cc_fpm_del_l2_entry (
				 ix_uint32 arg_L2ID,
				 void* arg_pContext
				)
{
    ix_error err = IX_SUCCESS;

    FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);    
    FPM_CC_DBG_PRINT ("L2 ID = %d \n", (ix_int16)arg_L2ID);
        
    IX_ERROR_CGT (ix_cc_l2tm_delete_entry (g_pFPMContext->fpm_hL2tm,
					    arg_L2ID),
		   err, IX_CC_FPM_ERROR_L2TM,
		   IX_ERROR_LEVEL_LOCAL, label_1);

    err = ix_cc_msup_send_reply_msg (arg_pContext, 
				     (void*) IX_SUCCESS,
				     sizeof (ix_uint32),
				     IX_SUCCESS);    
    return IX_SUCCESS;

 label_1:
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
    /* send error back to caller with error */
    err = ix_cc_msup_send_reply_msg (arg_pContext, 
				     (void*) err,
				     sizeof (ix_uint32),
				     err);
    return err;
}

/**
 * NAME: ix_cc_fpm_get_eth_tx_stats
 *
 * DESCRIPTION: This library function is used to retrieve Ethernet Transmit
 *              statistics.
 *              FP Module egress core component receives this message from its 
 *              ingress counterpart in its message
 *              handler. 
 * 
 * @Param:  - IN port_id - port for which statistics is to be retrieved
 *            IN arg_pContext - pointer to context
 *
 * @Param:  - OUT - None
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_ERROR_NULL - null argument
 * IX_CC_FPM_ERROR_INVALID_CONTEXT - context is unusable
 * IX_CC_ERROR_UNDEFINED_MSG - un supported message
 * IX_CC_FPM_ERROR_MSG_LIBRARY - error from message support library
 * 
 */
ix_error ix_cc_fpm_get_eth_tx_stats (
				     ix_uint32 arg_PortId
				     )
{
    ix_error                           err = IX_SUCCESS;
    /*
    ix_cc_eth_tx_statistics_info       stats_info;
    ix_cc_eth_tx_statistics_info_data* stats;
    
    stats_info = FPM_CC_ETH_ALL_DRIVER_COUNTERS;
    */
    FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);
    FPM_CC_DBG_PRINT ("Getting statistics for Port ID = %d \n", (ix_int16)arg_PortId);
    /* 
       IX_ERROR_CR (ix_cc_eth_tx_get_statistics_info (stats_info,
       arg_PortId,
       stats);
    */
    return err;
}
#endif /*IX_CC_ETH*/


/****************************************************************************
 *      MESSAGE HANDLER, GENERAL CALLBACK FUNCTION
 ***************************************************************************/
/**
 * NAME: ix_cc_fpm_proxy_msg_handler
 *
 * DESCRIPTION: This primitive is the message handler function for FP Module
 *              core component. FP Module egress core component receives 
 *              messages from its egress counterpart through this message 
 *              handler function and it appropriately processes the messages. 
 * 
 * @Param:  - IN arg_hDataToken - buffer handle embedding information for the 
 *            message passed in arg_UserData.
 *
 * arg_UserData - message type
 *
 * @Param:  - OUT - arg_pComponentContext: pointer to FP Module core component 
 *            context that will be passed to the core component when a message 
 *            arrives. This context was defined by the core component and passed
 *            to core components infrastructure through ix_cc_fpm_init function.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 * 
 * IX_CC_ERROR_NULL - null argument
 * IX_CC_FPM_ERROR_INVALID_CONTEXT - context is unusable
 * IX_CC_ERROR_UNDEFINED_MSG - un supported message
 * IX_CC_FPM_ERROR_MSG_LIBRARY - error from message support library
 * 
 */
ix_error ix_cc_fpm_proxy_msg_handler (
				      ix_buffer_handle arg_hDataToken,
				      ix_uint32 arg_UserData,
				      void* arg_pContext
				      )
{
    ix_error                          err = IX_SUCCESS;
    void                              *pContext = NULL;
    void                              *pMsg = NULL;
    void                              *pTmpContext = NULL;


    FPM_CC_DBG_TRACE ();
    
    /* Validate arg_pContext */
    if (arg_pContext == NULL) {
	return IX_ERROR_WARNING (IX_CC_ERROR_NULL,
				 ("arg_pContext is NULL"));
    }/* end if(arg_pContext == NULL) */
    
    if ((arg_UserData < IX_CC_COMMON_MSG_ID_FIRST) ||
	(arg_UserData >= IX_CC_FPM_MSG_LAST)) {
	
        return IX_ERROR_WARNING (IX_CC_ERROR_UNDEFINED_MSG,
				 ("Invalid Message Type"));
    }/* end if */

    IX_MSUP_EXTRACT_MSG (arg_hDataToken,&pMsg,&pContext);

    #ifdef IX_CC_ETH
    if (arg_UserData != IX_CC_FPM_MSG_PURGE_ARP_TABLE) {
	if (pMsg == NULL) {
	    return IX_ERROR_WARNING (IX_CC_ERROR_NULL,
				     ("Invalid Buffer"));
	}/* end if(pContext == NULL) */
    }
    #endif
    
    FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);
    /* Check for message type */
    switch (arg_UserData) {
#ifdef IX_CC_ETH
    case IX_CC_FPM_MSG_ADD_L2_ENTRY:
      {
	ix_cc_fpm_add_l2_entry_data* l2_info = 
	  (ix_cc_fpm_add_l2_entry_data*) pMsg;
	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

/*	err = ix_cc_fpm_add_l2_entry (l2_info, arg_pContext); KLT */
	err = ix_cc_fpm_add_l2_entry (l2_info, pContext); /* KLT */
	if(err != IX_SUCCESS) {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
	    err = IX_ERROR_WARNING (err, ("Adding L2 entry failed"));
	}
	break;	
      }
    case IX_CC_FPM_MSG_DEL_L2_ENTRY:
      {
	ix_uint32 l2_id = *(ix_uint32*)pMsg;
	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);
	FPM_CC_DBG_PRINT ("L2 Id being deleted = %d \n", (ix_int16)l2_id);
/*	err = ix_cc_fpm_del_l2_entry (l2_id, arg_pContext); KLT */
	err = ix_cc_fpm_del_l2_entry (l2_id, pContext); /* KLT */
	if(err != IX_SUCCESS) {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
	    err = IX_ERROR_WARNING (err, ("Deleting L2 entry failed"));
	}
	break;
      }
    case IX_CC_FPM_MSG_ADD_ARP_ENTRY:
      {
	ix_cc_l2tm_entry l2_info;
	ix_cc_eth_tx_next_hop_info* pData = (ix_cc_eth_tx_next_hop_info*) pMsg;
	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

	ix_cc_fpm_dump_arp_info (pData);

	l2_info.l2AddrType = IX_CC_L2TM_L2ADDR_ETHER;
	l2_info.ipAddrType = IX_CC_L2TM_IPADDR_IPV4;
	l2_info.l3Address.ipv4Address = pData->nextHopIp;

	IX_ERROR_CRT (ix_cc_l2tm_add_entry (g_pFPMContext->fpm_hL2tm,
					    pData->l2Index, &l2_info),
		      IX_CC_FPM_ERROR_L2TM,
		      IX_ERROR_LEVEL_LOCAL); 
	
	IX_ERROR_CRT (ix_cc_eth_tx_async_add_arp_entry (pData, 
							ix_cc_fpm_add_arp_cb,
/*							arg_pContext), KLT */
							pContext), /* KLT */
		      IX_CC_FPM_ERROR_ARP, IX_ERROR_LEVEL_WARNING);
	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);
	break;	
      }
    case IX_CC_FPM_MSG_DEL_ARP_ENTRY:
      {
	ix_uint32 l2_id = *(ix_uint32*) pMsg;
	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);
	FPM_CC_DBG_PRINT ("L2 Id being deleted = %ld \n", (ix_uint32)l2_id);
	IX_ERROR_CRT (ix_cc_eth_tx_async_del_arp_entry (l2_id, 
							ix_cc_fpm_del_arp_cb,
/*							arg_pContext), KLT */
							pContext), /* KLT */
		      IX_CC_FPM_ERROR_ARP, IX_ERROR_LEVEL_WARNING);
	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);
	break;
      }
    case IX_CC_FPM_MSG_PURGE_ARP_TABLE:
      {
	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);
	IX_ERROR_CRT (ix_cc_eth_tx_async_purge_arp_cache (),
		      IX_CC_FPM_ERROR_ARP, IX_ERROR_LEVEL_WARNING);
	/* NOTE: This is a FIRE and FORGET call, no message needs to be 
	   send a reply message back to ingress - no one is waiting for
	   this.
	*/
	FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);
	break;
      }
    case IX_CC_FPM_MSG_ETH_TX_STATS:
	{
	    /* has to be handled only on egress */	
	    ix_cc_eth_tx_statistics_info_context* stats_info = 
		(ix_cc_eth_tx_statistics_info_context*) pMsg;
	    FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

	    IX_ERROR_CRT (ix_cc_eth_tx_async_get_statistics_info 
			  (stats_info, ix_cc_fpm_eth_tx_stats_cb),
			  IX_CC_FPM_ERROR_ARP, IX_ERROR_LEVEL_WARNING);
	    FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);
	    break;
	}
#endif /*IX_CC_ETH*/
#ifdef IX_CC_QOS
    case IX_CC_FPM_MSG_DPE_OP:
      {
        ix_error err = IX_SUCCESS;
	ix_cc_wred_parameters wred_parameters;
	ix_cc_fpm_dpe *dpe = (ix_cc_fpm_dpe*) pMsg;
        if(dpe == NULL) return;
	switch(dpe->type)
	{
	    case IX_CC_FPM_DPE_METER:
	    {
	        if(dpe->op == IX_CC_FPM_DPE_ADD)
	        {
	            ix_cc_tc_meter_async_add_entry(dpe->fid,
						&dpe->dpe.tc_meter,
						ix_cc_fpm_dpe_callback,
						pContext);
	        }
	        else if(dpe->op == IX_CC_FPM_DPE_UPDATE)
	        {
	            ix_cc_tc_meter_async_update_entry(dpe->fid,
						&dpe->dpe.tc_meter,
						ix_cc_fpm_dpe_callback,
						pContext);
	        }
	        else if(dpe->op == IX_CC_FPM_DPE_REMOVE)
	        {
	            ix_cc_tc_meter_async_remove_entry(dpe->fid,
						ix_cc_fpm_dpe_callback,
						pContext);
	        }
	        else if(dpe->op == IX_CC_FPM_DPE_STATS)
	        {
	            ix_cc_tc_meter_async_get_statistics(dpe->fid,
						&dpe->dpe.meter_stats,
						ix_cc_fpm_dpe_meter_statistics_cb,
						pContext);
	        }
	    }
	    break;
	    case IX_CC_FPM_DPE_WRED_DROPPER:
	    {
	        if(dpe->op == IX_CC_FPM_DPE_ADD)
	        {
	            ix_cc_wred_async_add_entry(dpe->fid,
						&dpe->dpe.wred_dropper,
						ix_cc_fpm_dpe_callback,
						pContext);
	        }
	        else if(dpe->op == IX_CC_FPM_DPE_UPDATE)
	        {
	            ix_cc_wred_async_update_entry(dpe->fid,
						&dpe->dpe.wred_dropper,
						ix_cc_fpm_dpe_callback,
						pContext);
	        }
	        else if(dpe->op == IX_CC_FPM_DPE_REMOVE)
	        {
	            ix_cc_wred_async_remove_entry(dpe->fid,
						ix_cc_fpm_dpe_callback,
						pContext);
	        }
	        else if(dpe->op == IX_CC_FPM_DPE_STATS)
	        {
	            ix_cc_wred_async_get_statistics(dpe->fid,
						&dpe->dpe.wred_stats,
						ix_cc_fpm_dpe_wred_dropper_statistics_cb,
						pContext);
	        }
	    }
	    default:
	    {
	    }
	}
      }
	break;	
#endif /*IX_CC_QOS*/
#ifdef IX_CC_ATM
	case IX_CC_FPM_MSG_LLC_SNAP_ENCAP_ADD_NEXT_HOP:
	{
		/* Get the Port descriptor from the pMsg */
		ix_cc_fpm_llc_snap_encap_add_next_hop_t *data  = 
				(ix_cc_fpm_llc_snap_encap_add_next_hop_t *) pMsg;

		FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

		/* Call the CC function */
		err = ix_cc_llc_snap_encap_async_add_next_hop(
									data->nextHop,
									data->encapBit,
									data->vpi,
									data->vci,
									data->vcq,
									data->dSap,
									data->sSap,
									data->ctrl,
									data->oui,
									data->pid,
									ix_cc_llc_snap_encap_fpm_add_next_hop_cb_proxy,
									(void *)pContext);
									
		if(err != IX_SUCCESS)
		{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
			err = IX_ERROR_WARNING (err, ("LLC SNAP NEXT HOP ADD failed"));
		}


	}
	break;
	case IX_CC_FPM_MSG_LLC_SNAP_ENCAP_DEL_NEXT_HOP:
	{
		/* Get the Next Hop Id from the pMsg */
		ix_uint16 *data  = (ix_uint16 *) pMsg;
				

		FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

		/* Call the CC function */
		err = ix_cc_llc_snap_encap_async_delete_next_hop(
									(ix_uint16)*data,
									ix_cc_llc_snap_encap_fpm_del_next_hop_cb_proxy,
									(void *)pContext);
									
		if(err != IX_SUCCESS)
		{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
			err = IX_ERROR_WARNING (err, ("LLC SNAP NEXT HOP DEL failed"));
			ix_error_dump (stdout, err);
		}
	}
	break;
	case IX_CC_FPM_MSG_ATM_PORT_CREATE:
	{
		/* Get the Port descriptor from the pMsg */
		ix_cc_fpm_atm_port_descr_t *data  = 
				(ix_cc_fpm_atm_port_descr_t *) pMsg;

		/* Save the RM context received. This is needed because the RM
			will overwrite on this context when the ATM CC on egress
			and ingress communicate. This leads to the context getting
			lost. The memory for this has to be freed. The exact position
			for freeing the memory is TBD */
    	pTmpContext = (ix_msup_msg_header*)
							ix_ossl_malloc(sizeof(ix_msup_msg_header));
    	memcpy(pTmpContext,pContext,sizeof(ix_msup_msg_header));

		FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

		/* Call the CC function */
		err = ix_cc_atmsar_async_port_create(
									data->arg_sarInstance,
									&data->arg_pPortDescr,
									ix_cc_fpm_atm_port_create_cb_proxy,
									(void *)pTmpContext);
									
		if(err != IX_SUCCESS)
		{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
			err = IX_ERROR_WARNING (err, ("ATM Port create failed"));
		}

	}
	break;

	case IX_CC_FPM_MSG_ATM_PORT_DELETE:
	{
		/* Get the Port descriptor from the pMsg */
		ix_cc_fpm_atm_port_handle_t *data  = 
				(ix_cc_fpm_atm_port_handle_t *) pMsg;

		/* Save the RM context received. This is needed because the RM
			will overwrite on this context when the ATM CC on egress
			and ingress communicate. This leads to the context getting
			lost. The memory for this has to be freed. The exact position
			for freeing the memory is TBD */
    	pTmpContext = (ix_msup_msg_header*)
							ix_ossl_malloc(sizeof(ix_msup_msg_header));
    	memcpy(pTmpContext,pContext,sizeof(ix_msup_msg_header));

		FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

		/* Call the CC function */
		err = ix_cc_atmsar_async_port_remove(
									data->arg_sarInstance,
									data->arg_port,
									ix_cc_fpm_atm_port_del_cb_proxy,
									(void *)pTmpContext);
									
		if(err != IX_SUCCESS)
		{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
			err = IX_ERROR_WARNING (err, ("ATM Port delete failed"));
		}

	}
	break;

	case IX_CC_FPM_MSG_ATM_PORT_STATS_GET:
	{
		/* Get the Port descriptor from the pMsg */
		ix_cc_fpm_atm_port_handle_t *data  = 
				(ix_cc_fpm_atm_port_handle_t *) pMsg;

		/* Save the RM context received. This is needed because the RM
			will overwrite on this context when the ATM CC on egress
			and ingress communicate. This leads to the context getting
			lost. The memory for this has to be freed. The exact position
			for freeing the memory is TBD */
    	pTmpContext = (ix_msup_msg_header*)
							ix_ossl_malloc(sizeof(ix_msup_msg_header));
    	memcpy(pTmpContext,pContext,sizeof(ix_msup_msg_header));

		FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

		/* Call the CC function */
		err = ix_cc_atmsar_async_get_port_stats(
									data->arg_sarInstance,
									data->arg_port,
									ix_cc_fpm_atm_port_stats_get_cb_proxy,
									(void *)pTmpContext);
									
		if(err != IX_SUCCESS)
		{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
			err = IX_ERROR_WARNING (err, ("ATM Port Statistics Get failed"));
		}

	}
	break;

	case IX_CC_FPM_MSG_ATM_CID_CREATE:
	break;
	
	case IX_CC_FPM_MSG_ATM_CID_DELETE:
	break;

	case IX_CC_FPM_MSG_ATM_CID_STATS_GET:
	break;


	case IX_CC_FPM_MSG_ATM_VCC_CREATE:
	{
		/* Get the Vcc Info from the pMsg */
		ix_cc_fpm_atm_vc_descr_t *data  = 
				(ix_cc_fpm_atm_vc_descr_t *) pMsg;

		/* Save the RM context received. This is needed because the RM
			will overwrite on this context when the ATM CC on egress
			and ingress communicate. This leads to the context getting
			lost. The memory for this has to be freed. The exact position
			for freeing the memory is TBD */
    	pTmpContext = (ix_msup_msg_header*)
							ix_ossl_malloc(sizeof(ix_msup_msg_header));
    	memcpy(pTmpContext,pContext,sizeof(ix_msup_msg_header));

		FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

		/* Call the CC function */
		err = ix_cc_atmsar_async_vc_create(
									data->arg_sarInstance,
									&data->arg_pVcDescr,
									ix_cc_fpm_atm_vcc_create_cb_proxy,
							(void *)pTmpContext);
									
		if(err != IX_SUCCESS)
		{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
			err = IX_ERROR_WARNING (err, ("ATM Vcc Set failed"));
		}

	}
	break;
	case IX_CC_FPM_MSG_ATM_VCC_UPDATE:
	{
		/* Get the Vcc Info from the pMsg */
		ix_cc_fpm_atm_vc_update_t *data  = 
				(ix_cc_fpm_atm_vc_update_t *) pMsg;

		/* Save the RM context received. This is needed because the RM
			will overwrite on this context when the ATM CC on egress
			and ingress communicate. This leads to the context getting
			lost. The memory for this has to be freed. The exact position
			for freeing the memory is TBD */
    	pTmpContext = (ix_msup_msg_header*)
							ix_ossl_malloc(sizeof(ix_msup_msg_header));
    	memcpy(pTmpContext,pContext,sizeof(ix_msup_msg_header));

		FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

		/* Call the CC function */
		err = ix_cc_atmsar_async_vc_update(
									data->arg_sarInstance,
									data->arg_hVc,
									&data->arg_pUpdateData,
									ix_cc_fpm_atm_vcc_update_cb_proxy,
									(void *)pTmpContext);
									
		if(err != IX_SUCCESS)
		{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
			err = IX_ERROR_WARNING (err, ("ATM Vcc Update failed"));
		}

	}
	break;
	case IX_CC_FPM_MSG_ATM_VCC_DELETE:
	{
		/* Get the Vcc Info from the pMsg */
		ix_cc_fpm_atm_vc_handle_t *data  = 
				(ix_cc_fpm_atm_vc_handle_t *) pMsg;

		/* Save the RM context received. This is needed because the RM
			will overwrite on this context when the ATM CC on egress
			and ingress communicate. This leads to the context getting
			lost. The memory for this has to be freed. The exact position
			for freeing the memory is TBD */
    	pTmpContext = (ix_msup_msg_header*)
							ix_ossl_malloc(sizeof(ix_msup_msg_header));
    	memcpy(pTmpContext,pContext,sizeof(ix_msup_msg_header));

		FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

		/* Call the CC function */
		err = ix_cc_atmsar_async_vc_remove(
									data->arg_sarInstance,
									data->arg_hVc,
									ix_cc_fpm_atm_vcc_remove_cb_proxy,
									(void *)pTmpContext);
									
		if(err != IX_SUCCESS)
		{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
			err = IX_ERROR_WARNING (err, ("ATM Vcc Delete failed"));
		}

	}
	break;
	
	case IX_CC_FPM_MSG_ATM_VCC_STATS_GET:
	{
		/* Get the Vcc Info from the pMsg */
		ix_cc_fpm_atm_vc_handle_t *data  = 
				(ix_cc_fpm_atm_vc_handle_t *) pMsg;

		/* Save the RM context received. This is needed because the RM
			will overwrite on this context when the ATM CC on egress
			and ingress communicate. This leads to the context getting
			lost. The memory for this has to be freed. The exact position
			for freeing the memory is TBD */
    	pTmpContext = (ix_msup_msg_header*)
							ix_ossl_malloc(sizeof(ix_msup_msg_header));
    	memcpy(pTmpContext,pContext,sizeof(ix_msup_msg_header));

		FPM_CC_DBG_PRINT ("%s %d\n", __FUNCTION__, __LINE__);

		/* Call the CC function */
		err = ix_cc_atmsar_async_get_vc_stats(
									data->arg_sarInstance,
									data->arg_hVc,
									ix_cc_fpm_atm_vcc_stats_get_cb_proxy,
									(void *)pTmpContext);
									
		if(err != IX_SUCCESS)
		{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
      ix_ossl_message_log("FPM PROXY: %ld\n", (ix_uint32)err);
#else
		ix_error_dump (stdout, err);
#endif
			err = IX_ERROR_WARNING (err, ("ATM Vcc Statistics Get failed"));
		}

	}
	break;

#endif /*IX_CC_ATM*/

    default:
	err = IX_ERROR_WARNING (IX_CC_ERROR_UNDEFINED_MSG,
				("Msg handler invoked with invalid msg ID &d", 
				 arg_UserData));
	break;
    }/* end switch(arg_UserData) */
    
    IX_ERROR_CRT (ix_rm_buffer_free_chain (arg_hDataToken),
		  IX_CC_FPM_ERROR_BUFFER_FREE, 0); 
    return err;
    
} /* end ix_cc_fpm_proxy_msg_handler */


#ifdef IX_CC_ETH
/**
 * NAME: ix_cc_fpm_dump_l2_info
 *
 * DESCRIPTION: This function dumps the contents of the FPM L2 entry structure
 *  
 * @Param:  - IN  - ix_cc_fpm_add_l2_entry_data* l2_info 
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *          Return code is not interpreted by the core component.
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */
void ix_cc_fpm_dump_l2_info (ix_cc_fpm_add_l2_entry_data *l2_info)
{
    FPM_CC_DBG_TRACE ();

    FPM_CC_DBG_PRINT ("L2 ID = %d \n Output Port = %d \n", (ix_int16)l2_info->l2_id,
		      (ix_int16)l2_info->output_port);
    switch (l2_info->l2tm_info.l2AddrType) {
    case IX_CC_L2TM_L2ADDR_ETHER:
	FPM_CC_DBG_PRINT ("L2 addr type = Ethernet\n");
	break;
    case IX_CC_L2TM_L2ADDR_ATM:
	FPM_CC_DBG_PRINT ("L2 addr type = ATM\n");
	break;
    default:
	FPM_CC_DBG_PRINT ("L2 addr type = Unknown!\n");
	break;
    }
    
    switch (l2_info->l2tm_info.ipAddrType) {
    case IX_CC_L2TM_IPADDR_IPV4:
	FPM_CC_DBG_PRINT ("IP addr type = IPv4\n IPAddress = 0x%x\n",
			  (ix_uint32)l2_info->l2tm_info.l3Address.ipv4Address);
	break;
    case IX_CC_L2TM_IPADDR_IPV6:
	FPM_CC_DBG_PRINT ("IP addr type = IPv6\n"
			  "IPAddress = 0x%2x:0x%2x:0x%2x:0x%2x:0x%2x:0x%2x:0x%2x:0x%2x\n",
			  l2_info->l2tm_info.l3Address.ipv6Address);
	break;
    default:
	FPM_CC_DBG_PRINT ("IP addr type = Unknown!\n");
	break;
    }
    return;
}

/**
 * NAME: ix_cc_fpm_dump_arp_info
 *
 * DESCRIPTION: This function dumps the contents of the ARP structure
 *  
 * @Param:  - IN  - ix_cc_eth_tx_next_hop_info* arp_info 
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *          Return code is not interpreted by the core component.
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */
void ix_cc_fpm_dump_arp_info (ix_cc_eth_tx_next_hop_info *arp_info)
{
    FPM_CC_DBG_TRACE ();

    FPM_CC_DBG_PRINT ("L2 ID = %d \n Output Port = %d \n"
		      "Next hop IP = 0x%x \n L2 Addr = %X:%X:%X:%X:%X:%X",
		      (ix_int16)arp_info->l2Index, 
		      (ix_int16)arp_info->outputPort, 
		      (ix_int16)arp_info->nextHopIp, 
		      arp_info->l2Addr.etherAddr[0],arp_info->l2Addr.etherAddr[1],
		      arp_info->l2Addr.etherAddr[2],arp_info->l2Addr.etherAddr[3],
		      arp_info->l2Addr.etherAddr[4],arp_info->l2Addr.etherAddr[5]
		      );
    return;
}
#endif /*IX_CC_ETH*/

/**
 * NAME: ix_cc_fpm_dump_context
 *
 * DESCRIPTION: This function dumps the content of the FP Module cc context
 *  
 * @Param:  - IN  - ix_cc_fpm_proxy_context* context 
 *
 * @Param:  - OUT - none
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 *          Return code is not interpreted by the core component.
 * Error Codes:
 * 
 * IX_CC_FPM_ERROR_CALLBACK - user callback returns error message
 * 
 */
void ix_cc_fpm_dump_context (ix_cc_fpm_proxy_context* context)
{
    FPM_CC_DBG_PRINT ("In %s \n Printing FPM Context \n", __FUNCTION__);
    FPM_CC_DBG_PRINT ("Blade ID = %d\n", (ix_int16)context->fpm_localBladeId);
    return;
}

