/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/inc/forces_Types.h,v $
 * $Revision: 1.6 $
 * $Date: 2003/11/17 16:57:59 $
 *
 *
 ****************************************************************************/

#ifndef _FORCES_TYPES_H_
#define _FORCES_TYPES_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "npf_common.h"

/******************************************/
/******* NEW FORCES MESSAGE TYPES *********/
/******************************************/

#define FE_Bind_Request			1
#define FE_Bind_Response		2
#define Capability_Request		3
#define Capability_Response		4
#define Topology_Request		5
#define Topology_Response		6
#define FE_Start_Operation		7
#define Config_Query_Request		8
#define Config_Query_Response		9
#define FE_Event_Notification		10
#define Unbind				11
#define Heartbeat			12

#define Forces_TLV_Encap		1
#define Forces_BER_Encap		2
#define Forces_XML_Encap		3
#define Forces_GRE_Encap		4
#define Forces_IPinIP_Encap		5

#define Forces_NULL_Cmd			1
#define Forces_Add_Cmd			2
#define Forces_Update_Cmd		3
#define Forces_Delete_Cmd		4
#define Forces_DeleteAll_Cmd		5
#define Forces_SendPacket_Cmd		6
#define Forces_QueryStats_Cmd		7
#define Forces_QueryProps_Cmd		8

#define Forces_Port_Event		11
#define Forces_BlockSpecific_Event	12
#define Forces_PacketRedirect_Event	13
#define Forces_Capability_Event		14


typedef enum {
    Forces_Null_Block,
    Forces_Ingress_Intf_Block,
    Forces_Egress_Intf_Block,
    /*
    Forces_IPv4_Block,
    */
    Forces_IPv4_Prefix_Block,
    Forces_IPv4_NextHop_Block,
    Forces_L2_ARP_Block,
    Forces_L2_Interface_Block,
    Forces_L3_Interface_Block,
    Forces_ICMP_Block,

#ifdef _INCLUDE_IPV6
    Forces_L3_IPv6_Interface_Block,
    Forces_IPv6_Block,
    Forces_IPv6_Prefix_Block,
    Forces_IPv6_NextHop_Block,
    Forces_IPv6_L2_Block,
#endif /* IPV6 */

#ifdef _INCLUDE_IPV6TM
    Forces_IPv6TM_Block,
    Forces_IPv6TM_Auto_Block,
    Forces_IPv6TM_Config_Block,
    Forces_IPv6TM_6to4_Block,
#endif /* IPV6TM */
    
#ifdef _INCLUDE_MPLS
    Forces_MPLS_Block,
#endif  /* MPLS */

/* ATM */
#ifdef _INCLUDE_ATM
    Forces_IfATM_IfAttrSet_Block,
    Forces_IfATM_IfDelete_Block,
    Forces_IfATM_IfEnable_Block,
    Forces_IfATM_IfDisable_Block,
    Forces_IfATM_IfGenStatsGet_Block,
    Forces_IfATM_VccSet_Block,
    Forces_IfATM_VccBind_Block,
    Forces_IfATM_VccDelete_Block,
    Forces_IfATM_VccEnable_Block,
    Forces_IfATM_VccDisable_Block,
    Forces_IfATM_VccOperStatusGet_Block,
    Forces_IfATM_VccStatsGet_Block,
    Forces_IfATM_AAL2_ChannelSet_Block,
    Forces_IfATM_AAL2_ChannelDel_Block,
    Forces_IfATM_ConnectionXcSet_Block,
    Forces_IfATM_ConnectionXcDel_Block,
    Forces_IfATM_OAM_CP_Set_Block,
    Forces_IfATM_OAM_CC_Set_Block,
    Forces_IfATM_OAM_LB_Set_Block,
    Forces_IfATM_OAM_Monitor_Block,
    Forces_IfATM_OAM_AlarmSet_Block,
    Forces_IfATM_OAM_AlarmClear_Block,
    Forces_IfATM_OAM_PM_Set_Block,
#endif /* _INCLUDE_ATM */

/* DS */
#ifdef _INCLUDE_DIFFSERV
    Forces_DS_Dpe_Block,
#endif

/* This identifies the last Block */
/* Add New Block types above this */

    Forces_End_Block
} ForceBlocksEnum;

typedef struct {
    uint8_t	version;
    uint8_t	flags;
    uint16_t	msg_type;
    uint32_t	cmd_correlator;
} forces_header_t;

typedef struct {
    uint32_t	feid;
    uint16_t	control_encap_type;
    uint16_t	data_encap_type;
    uint32_t	bind_status;
    uint32_t	heartbeat_interval;
} forces_bind_info_t;

typedef struct {
    uint32_t	bind_result;
    uint32_t	fe_behavior;
} forces_bind_resp_t;

typedef struct {
    uint32_t	port_id;
    uint32_t	port_type;
    MACA        mac_addr;
    uint32_t    link_speed;
    uint32_t    mtu;
} forces_port_info_t;

typedef struct {
    uint32_t	port_count; 
    forces_port_info_t	*port_array; /* @!array-length:port_count! */
} forces_portlist_t;

typedef struct {
    uint32_t	block_type;
    uint32_t	block_handle;
    uint32_t	downstream_block_count;
    uint32_t	*downstream_block_array; /* @!array-length:downstream_block_count! */
} forces_block_info_t;

typedef struct {
    uint32_t		block_count;
    forces_block_info_t	*block_array; /* @!array-length:block_count! */
} forces_blocklist_t;

typedef struct {
    uint32_t	fe_count;
    uint32_t	*feid_array; /* @!array-length:fe_count! */
} forces_conn_FElist_t;

typedef struct {
    uint32_t	cmd_type;
    uint32_t	block_type;
    uint32_t	block_handle;
    uint32_t	length;
    char	cmd_data[0];
} forces_command_t;

typedef struct {
    uint32_t	cmd_type;
    uint32_t	block_type;
    uint32_t	block_handle;
    uint32_t	global_result;
    uint32_t	length;
    char	cmd_data[0];
} forces_command_status_t;

typedef struct {
    uint32_t	reason;
    uint32_t	fe_behavior;
} forces_unbind_t;

#ifdef __cplusplus
}
#endif

#endif /* _FORCES_TYPES_H_ */

