/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/inc/fppCapabilityTypes.h,v $
 * $Revision: 1.6 $
 * $Date: 2003/11/17 16:57:59 $
 *
 *
 ****************************************************************************/

#ifndef _FPP_CAPABILITY_TYPES_H_
#define _FPP_CAPABILITY_TYPES_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "fpp_common.h"
#include "npf_common.h"
#include "npf_cm_types.h"
#include "npf_ipv4_types.h"

#ifdef _INCLUDE_IPV6
#include "npf_ipv6_types.h"
#endif

#ifdef _INCLUDE_DIFFSERV
#include "npf_ds.h"
#endif


#ifdef _INCLUDE_GTP
#include "npf_gtp_types.h"
#endif

#define FENAMELEN 10
#define MAX_OLD_IDS 10

#define DRS_FENAME_SZ_MAX     64
#define DRS_FEIPADDR_SZ_MAX   64

typedef char FENAME[FENAMELEN];

typedef struct fpp_Port_Caps_tag {
    FPPI_PortID		portid;
    uint32_t		port_type;  /* IXA set should match NPF_PORT_TYPE */
    MACA	        mac_addr;
    uint32_t		link_speed;
    uint32_t		mtu;
    NPF_ATM_PortCaps    atmCaps;
} FPPI_Port_Caps;

typedef struct fpp_Ports_Caps_tag {
    uint32_t		port_count;
    FPPI_Port_Caps	*port_caps_list; /* @!array-length:port_count! */
} FPPI_Ports_Caps;

typedef struct fpp_FE_Caps_tag {
    FENAME		fe_name;
    uint32_t    fe_addr;
    FPPI_Ports_Caps	ports_caps;
    uint32_t		caps_bitmap; /* bit 1 - IPv4 Fwd, 
					bit 2 - IPv6 Fwd,
					bit 3 - MPLS LER,
					bit 4 - MPLS LSR,
					bit 5 - DiffServ  */
} FPPI_FE_Caps;

typedef struct fpp_FPPI_FEID_MAP
{
    FPPI_FEID  feid;
    char       feName[DRS_FENAME_SZ_MAX];
    char       feIPAddr[DRS_FEIPADDR_SZ_MAX];
    BOOL       isSet;
} FPPI_FEID_MAP;

/******************* Response Data Structs ********************/

/* ARP list struct being deprecated */
/* typedef struct {
    uint32_t arp_count;
    npf_ipv4_ARPEntry_t* arp_list; */  /* @!array-length:arp_count! */
/* } npf_ipv4_ARPList_t; */

typedef struct {
    uint32_t length;
    char* buffer; /* @!array-length:length! */
    FPPI_PortID	portid;
} npf_Packet_t;

typedef struct {
    npf_Packet_t* packet_list; /* @!array-length:packet_count! */
    uint32_t packet_count;
} npf_PacketList_t;

typedef struct {
    uint32_t port_count;
    FPPI_PortID* port_list; /* @!array-length:port_count! */
} npf_PortList_t;

typedef struct {
    FPPI_PortID	portid;
    NPF_PORT_STATS l2stats;
} npf_L2StatsEntry_t;

typedef struct FPPI_Interface_GetStats{
    FPPI_PortID	*port_array;
    uint32_t port_count;
    NPF_IfType_t  ifType;
} FPPI_Interface_GetStats_t;

typedef struct {
    uint32_t port_count;
    npf_L2StatsEntry_t* stats_list; /* @!array-length:port_count! */
} npf_L2StatsList_t;

typedef struct {
    FPPI_PortID	portid;
    NPF_IfStatistics_t l3stats;
} npf_L3StatsEntry_t;

typedef struct {
    uint32_t port_count;
    npf_L3StatsEntry_t* stats_list; /* @!array-length:port_count! */
} npf_L3StatsList_t;

typedef struct {
    FPPI_PortID	portid;
    NPF_ICMP_STATS_t icmpstats;
} npf_ICMPStatsEntry_t;

typedef struct {
    uint32_t port_count;
    npf_ICMPStatsEntry_t* stats_list; /* @!array-length:port_count! */
} npf_ICMPStatsList_t;

typedef enum
{
    NPF_L3ATTRS_ALL,
    NPF_L3ATTRS_IPADDR,
    NPF_L3ATTRS_IPFWD
} NPF_L3ATTRS_TYPE;

typedef enum {
    NPF_L2_ADDRESS   = 1,
    NPF_IPv4_ADDRESS = 2,
    NPF_IPv6_ADDRESS = 3,
    NPF_NEXT_HOP_ID  = 4
} npf_AddressType_t;

typedef struct {
    NPF_L3ATTRS_TYPE type;
    FPPI_FEID        feid;
    FPPI_PortID      portid;
    npf_AddressType_t addr_type; /* IPv4 or IPv6 */
    union /* @!union-switch:addr_type! */
    {
        IPA     addr;  /* @!union-case:NPF_IPv4_ADDRESS! */
        IPv6A   v6addr; /* @!union-case:NPF_IPv6_ADDRESS! */
    }addr_u;
    union /* @!union-switch:addr_type! */
    {
        IPA     mask; /* @!union-case:NPF_IPv4_ADDRESS! */
        IPv6A   v6mask; /* @!union-case:NPF_IPv6_ADDRESS! */
    }mask_u;
    int		ipfwding;
} npf_L3IntfAttrs_t;

typedef struct {
    uint32_t port_count;
    npf_L3IntfAttrs_t* port_list; /* @!array-length:port_count! */
} npf_L3IntfAttrsList_t;

typedef enum {  NPF_L2ATTRS_ALL,
		NPF_L2ATTRS_MTU,
                NPF_L2ATTRS_LINKSPEED,
		NPF_L2ATTRS_STATUS,
		NPF_L2ATTRS_VPIBITS,
		NPF_L2ATTRS_VCIBITS,
                NPF_L2ATTRS_MPLS_MTU
} NPF_L2ATTRS_TYPE;

typedef struct {
    NPF_L2ATTRS_TYPE type;
    FPPI_FEID   feid;
    FPPI_PortID	portid;
    uint32_t	port_type;  /* what IXA set should match NPF_PORT_TYPE */

    uint32_t	link_speed;
    uint32_t	mtu;
    int		status;
    uint32_t vpi_bits;
    uint32_t vci_bits;
} npf_L2IntfAttrs_t;

typedef struct {
    uint32_t port_count;
    npf_L2IntfAttrs_t* port_list; /* @!array-length:port_count! */
} npf_L2IntfAttrsList_t;

#ifdef _INCLUDE_IPV6

typedef struct FPPI_IPv6_NextHopInfo_tag
{
    FPPI_FEID              feid;
    FPPI_L2ID              l2id;
    FPPI_PortID            portId;
    NPF_IPv6Address_t      nhAddr;
    NPF_uint32_t           weight;
    NPF_MTU                mtu;
    NPF_uint32_t           flags;
} FPPI_IPv6_NextHopInfo;

#endif


typedef struct FPPI_NextHopInfo_tag
{
    FPPI_FEID              feid;
    FPPI_L2ID              l2id;
    FPPI_PortID            portId;
    uint32_t               portType;
    NPF_IPv4Address_t      nhAddr;
    NPF_MediaAddress_t     mediaAddr;
    NPF_MTU                mtu;
    NPF_uint32_t           flags;
} FPPI_NextHopInfo;

typedef struct
{
    FPPI_FEID  feid;
    FPPI_L2ID  l2id;
} FPPI_FEandL2IDs;

/* For adding and deleting IPv4 and Ipv6 unicast next hop entries */
typedef struct
{
    FPPI_NHID          nhid;
    uint32_t           numNHEntries;
    FPPI_NextHopInfo*  nhInfo; /* @! array-length : numNHEntries ! */
    uint32_t           numOldL2IDEntries;
    FPPI_FEandL2IDs    oldFEandL2IDs[MAX_OLD_IDS];

} FPPI_UnicastNextHopUpdateInfo_t;


typedef struct 
{
    FPPI_NHID          nhid;
    FPPI_NextHopInfo   nhInfo;
} FPPI_UnicastNextHopDeleteInfo_t;

typedef struct
{
   NPF_uint32_t             nextHopId;
   NPF_IPv4UC_Prefix_t      prefixEntry;
} FPPI_UnicastPrefixUpdateInfo_t;

#ifdef _INCLUDE_IPV6
typedef struct
{
    FPPI_NHID          nhid;
    uint32_t           numNHEntries;
    FPPI_IPv6_NextHopInfo*  nhInfo; /* @! array-length : numNHEntries ! */
    uint32_t           numOldL2IDEntries;
    FPPI_FEandL2IDs    oldFEandL2IDs[MAX_OLD_IDS];

} FPPI_IPv6_UnicastNextHopUpdateInfo_t;

#endif

typedef struct FPPI_IPv4UniNHResolEntry_tag
{
    FPPI_L2ID                               l2id;
    FPPI_PortID                             portId;
    uint32_t                                portType;    
    NPF_IPv4UC_AddResEntry_t                l2Entry;
} FPPI_IPv4UniNHResolEntry_t;

#ifdef _INCLUDE_IPV6

typedef struct FPPI_IPv6NbrEntry_tag
{
    FPPI_L2ID                               l2id;
    NPF_IPv6NbrEntry_t                      l2Entry;
} FPPI_IPv6NbrEntry_t;

#endif

typedef struct {
    uint32_t            retStatus;
    npf_AddressType_t  type; 
    union /* @!union-switch:type! */
    {
        NPF_IPv4NetAddr_t        ipv4Addr; /* @!union-case:NPF_IPv4_ADDRESS! */
        NPF_MediaAddressEntry_t  l2Addr; /* @!union-case:NPF_L2_ADDRESS! */
    } u;
} npf_IPv4_Status_t;

/* So the FP can return success/error on each prefix/next hop
   it attempts to add/del/update. */
typedef struct FPPI_IPv4EntryStatusList_tag
{
    uint32_t           numEntries;
    npf_IPv4_Status_t* entryList; /* @!array-length:numEntries! */
} npf_IPv4EntryStatusList;


#ifdef _INCLUDE_IPV6TM

typedef struct {
    FPPI_PortID  portid;
    IPA          ipv4addr;
} npf_IPv6TMInfo_t;

#endif

#ifdef _INCLUDE_IPV6

/*NOTE: The union has been left in place below for any future additions*/
typedef struct {
    uint32_t            retStatus;
    npf_AddressType_t  type;
    union /* @!union-switch:type! */
    {
        NPF_IPv6NetAddr_t        ipv6Addr; /* @!union-case:NPF_IPv6_ADDRESS!*/
        NPF_uint32_t             nhId;     /* @!union-case:NPF_NEXT_HOP_ID!*/
    } u;
} npf_IPv6_Status_t;

/* So the FP can return success/error on each prefix/next hop
 *    it attempts to add/del/update. */
typedef struct FPPI_IPv6EntryStatusList_tag
{
    uint32_t           numEntries;
    npf_IPv6_Status_t* entryList; /* @!array-length:numEntries! */
} npf_IPv6EntryStatusList;

#endif

#ifdef _INCLUDE_GTP

typedef struct {
    uint32_t            teid;
    npf_gtp_Config_t*   config;
} npf_gtp_RxSetupInfo_t;

typedef struct {
    uint32_t            teid;
    npf_gtp_Config_t*   config;
    npf_gtp_IuTunnel_t* Iu_Association;
} npf_gtp_TxSetupInfo_t;

typedef struct {
    uint32_t            teid;
    npf_gtp_IuTunnel_t* Iu_Association;
} npf_gtp_SrncReloCmdInfo_t;

typedef struct {
    uint32_t            teid;
    npf_gtp_Config_t*   config;
} npf_gtp_TrncReloCmdInfo_t;

typedef struct {
    uint32_t            teid;
    uint32_t            PDCP_seq_num;
} npf_gtp_TrncReloCompInfo_t;

#endif


typedef struct {
    uint32_t		portid;
    uint32_t		dest;
} npf_fpm_ipv4_exception_t;

#ifdef _INCLUDE_DIFFSERV

typedef struct
{
    NPF_DS_CallbackType_t       op;             /* DPE Operation */
    NPF_DS_DpeType_t            type;           /* DPE Type */
    NPF_DS_ErrorType_t          error;          /* Error Code */
    NPF_DS_DpeHandle_t          dpeHandle;      /* DPE handle */
    NPF_DS_DpeHandle_t          prevDpeHandle;  /* DPE handle */
    FPPI_PortID                 port;           /* Port id */
    NPF_DS_DpeDirection_t	dir;		/* Direction, INGRESS/EGRESS */
    union
    {
        NPF_DS_Dpe_t            dpe;           /* Data path element */
        NPF_DS_StatsResp_t      stats;
    }u;
}FPPI_DS_Dpe_t;

#endif

#ifdef __cplusplus
}
#endif

#endif /* _FPP_CAPABILITY_TYPES_H_ */

