/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/


/*****************************************************************************
* $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/inc/pdk_cm_types.h,v $
* $Revision: 1.6 $
* $Date: 2003/11/17 16:57:59 $
*
* Description:
*
*    This file should only be included inside PDK internally
*    All the external apps should use npf_api/npf_cm_types.h
*
*****************************************************************************/

#ifndef _PDK_CM_TYPES_H_
#define _PDK_CM_TYPES_H_

#ifdef __cplusplus
extern "C" {
#endif 

#include "npf_cm_types.h"
#include "npf_ns_types.h"
#include "npf_cm_api.h"
#include "list.h"
#include "ipaddr.h"
#include "hwaddr.h"

/* Do not put the association in the data structure, it
 * should be maintained by the namespace
 *
 * Note: if modifing the following data structure,
 *       make sure you've taken care of the cm_copyFromPDK
 *       and cm_copyToPDK of the corresponding NPF_ data structure */

/* - Shim Layer - */
typedef struct PDK_SHIM_tag
{
	NPF_DATATYPE	type;
	NPF_HANDLE	handle;
	NPF_NSHDL	ns_handle;
} PDK_SHIM_t;

/* - SYSTEM - */
typedef struct PDK_SYSTEM_tag
{
	NPF_DATATYPE	type;		/* always put as first */
	NPF_HANDLE	handle;
	NPF_NSHDL	ns_handle;	/* back reference to its NS node */
	char		name[128];
	char		desc[256];
	int		num_FEs;
} PDK_SYSTEM_t;

/* - FE - */
#define MAXFENUM	12		/* max number of FE */
typedef NPF_FE_ATTRIBS PDK_FE_ATTRIBS;

typedef struct PDK_FE_CAPS_tag {
        int              max_arp_table;
        int              max_route_table;
        int              max_prefix_table;
        int              max_nexthop_table;
        int              max_ftn_table;
        int              max_ilm_table;
        int              route_table_type; /* IPv4, IPv6 or BOTH */
} PDK_FE_CAPS;

typedef struct PDK_FE_tag
{
	NPF_DATATYPE	type;		/* always put at first */
	NPF_HANDLE	handle;
	NPF_NSHDL	ns_handle;	/* back reference to its NS node */
	PDK_FE_ATTRIBS	attribs;
	PDK_FE_CAPS	caps;
        uint32_t	id;
} PDK_FE_t;

/* - Port - */
#define MAXFEOWNPORT	12		/* max number port belongs to a FE */
typedef NPF_PORT_ATTRIBS PDK_PORT_ATTRIBS;
typedef NPF_PORT_CAPS PDK_PORT_CAPS;
typedef struct PDK_port_tag
{
	NPF_DATATYPE		type;		/* always put at first */
	NPF_HANDLE		handle;		/* port handle */
	NPF_NSHDL		ns_handle;	/* back ref to its NS node */
	PDK_PORT_ATTRIBS	attribs;
	PDK_PORT_CAPS		caps;
} PDK_PORT_t;

/* - Router - */
typedef struct PDK_iprouter_tag
{
	NPF_DATATYPE	type;		/* always put at first */
	NPF_HANDLE	handle;
	NPF_NSHDL	ns_handle;
	char		name[128];
	char		desc[256];
	int		num_interfaces;
} PDK_IPROUTER_t;

/* - Interface - */
#define MAXIFOWNPORT	24		/* max number port associated to if */
typedef NPF_IF_IP_ATTRIBS PDK_IF_IP_ATTRIBS;
typedef struct PDK_interface_tag
{
	NPF_DATATYPE		type;		/* always put at first */
	NPF_HANDLE		handle;
	NPF_NSHDL		ns_handle;	/* back ref to its NS node */
	char			name[128];
	char			desc[256];
	int			status;		/* UP, DOWN, TESTING */
	int			index;		/* If index - hack fehdl vidd */
	char			vifname[128];	/* virtual interface name */
	char			vifname6[128];	/* virtual interface name */
	NPF_IF_IP_ATTRIBS	ipattribs;	
} PDK_INTERFACE_t;

/*
typedef enum
{
    NPF_ROUTER_IPv4 = 0,
    NPF_ROUTER_IPv6 = 1,
    NPF_ROUTER_BOTH = 2
} PDK_ROUTER_TYPE_t;
*/
/* - Route Table - */
typedef struct PDK_routetable_tag
{
    	NPF_DATATYPE		type;		/* always put at first */
	NPF_HANDLE		handle;
	NPF_NSHDL		ns_handle;	/* back ref to its NS node */
	int			rt_type;	/* IPv4, IPv6 or BOTH */
} PDK_ROUTETABLE_t;

/* - ARP Table - */
typedef struct PDK_arptable_tag
{
	NPF_DATATYPE		type;		/* always put as first */
	NPF_HANDLE		handle;
	NPF_NSHDL		ns_handle;	/* back ref to its NS node */
	int			bogus;		/* not use at all */
} PDK_ARPTABLE_t;

/* - FTN Table - */
typedef struct PDK_ftntable_tag
{
        NPF_DATATYPE            type;           /* always put as first */
        NPF_HANDLE              handle;
        NPF_NSHDL               ns_handle;      /* back ref to its NS node */
        int                     bogus;          /* not use at all */
} PDK_FTNTABLE_t;

/* - ILM Table - */
typedef struct PDK_ilmtable_tag
{
        NPF_DATATYPE            type;           /* always put as first */
        NPF_HANDLE              handle;
        NPF_NSHDL               ns_handle;      /* back ref to its NS node */
        int                     bogus;          /* not use at all */
} PDK_ILMTABLE_t;

/* - Internal Config - */
typedef struct PDK_intconfig_tag
{
	NPF_DATATYPE		type;		/* always put as first */
	NPF_HANDLE		handle;
	NPF_NSHDL		ns_handle;	/* back ref to its NS node */
	int			bogus;		/* not use at all */
} PDK_INTCONFIG_t;

#ifdef __cplusplus
}
#endif

#endif /* _PDK_CM_TYPES_H_ */
