/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2002 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/npf_api/npf_atm_api.h,v $
 * $Revision: 1.5 $ 
 * $Date: 2003/11/17 16:57:59 $
 *
 * Description:
 * This file defines the external API which applications use to access
 * ATM functionality.
 *
 ****************************************************************************/
#ifndef __NPF_ATM_API_H__
#define __NPF_ATM_API_H__

#ifdef _INCLUDE_ATM

#ifdef __cplusplus
extern "C" {
#endif

/* Add or Modify an ATM VCC */
NPF_error_t NPF_IfATM_VccSet(
				NPF_IN		NPF_callbackHandle_t 	if_cbHandle,
				NPF_IN		NPF_correlator_t			if_cbCorrelator,
				NPF_IN		NPF_errorReporting_t		if_errorReporting,
				NPF_IN		NPF_IfHandle_t				if_handle,
				NPF_IN		NPF_uint32_t				n_Vccs,
				NPF_IN		NPF_IfVcc_t					*if_VccStructArray);

/* Bind  a higher layer interface to an ATM VCC */
NPF_error_t	NPF_IfATM_VccBind(
				NPF_IN		NPF_callbackHandle_t 	if_cbHandle,
				NPF_IN		NPF_correlator_t			if_cbCorrelator,
				NPF_IN		NPF_errorReporting_t		if_errorReporting,
				NPF_IN		NPF_IfHandle_t				if_ATM_Handle,
				NPF_IN		NPF_IfHandle_t				if_ParentHandle,
				NPF_IN		NPF_uint32_t				n_Vccs,
				NPF_IN		NPF_VccAddr_t				*if_VccAddrArray);

/* Delete an ATM VCC */
NPF_error_t	NPF_IfATM_VccDelete(
				NPF_IN		NPF_callbackHandle_t 	if_cbHandle,
				NPF_IN		NPF_correlator_t			if_cbCorrelator,
				NPF_IN		NPF_errorReporting_t		if_errorReporting,
				NPF_IN		NPF_IfHandle_t				if_handle,
				NPF_IN		NPF_uint32_t				n_Vccs,
				NPF_IN		NPF_VccAddr_t				*if_VccAddrArray);

/* Enable an ATM VCC */
NPF_error_t	NPF_IfATM_VccEnable(
				NPF_IN		NPF_callbackHandle_t 	if_cbHandle,
				NPF_IN		NPF_correlator_t			if_cbCorrelator,
				NPF_IN		NPF_errorReporting_t		if_errorReporting,
				NPF_IN		NPF_IfHandle_t				if_handle,
				NPF_IN		NPF_uint32_t				n_Vccs,
				NPF_IN		NPF_VccAddr_t				*if_VccAddrArray);

				
/* Disable an ATM VCC */
NPF_error_t	NPF_IfATM_VccDisable(
				NPF_IN		NPF_callbackHandle_t 	if_cbHandle,
				NPF_IN		NPF_correlator_t			if_cbCorrelator,
				NPF_IN		NPF_errorReporting_t		if_errorReporting,
				NPF_IN		NPF_IfHandle_t				if_handle,
				NPF_IN		NPF_uint32_t				n_Vccs,
				NPF_IN		NPF_VccAddr_t				*if_VccAddrArray);


/* Fetch Operational Status of an ATM VCC */
NPF_error_t	NPF_IfATM_VccOperStatusGet(
				NPF_IN		NPF_callbackHandle_t 	if_cbHandle,
				NPF_IN		NPF_correlator_t			if_cbCorrelator,
				NPF_IN		NPF_errorReporting_t		if_errorReporting,
				NPF_IN		NPF_IfHandle_t				if_handle,
				NPF_IN		NPF_uint32_t				n_Vccs,
				NPF_IN		NPF_VccAddr_t				*if_VccAddrArray);

/* Read ATM VCC Statistics */
NPF_error_t	NPF_IfATM_VccStatsGet(
				NPF_IN		NPF_callbackHandle_t 	if_cbHandle,
				NPF_IN		NPF_correlator_t			if_cbCorrelator,
				NPF_IN		NPF_errorReporting_t		if_errorReporting,
				NPF_IN		NPF_IfHandle_t				if_handle,
				NPF_IN		NPF_uint32_t				n_Vccs,
				NPF_IN		NPF_VccAddr_t				*if_VccAddrArray);

/* Add Connection Cross-connect */
NPF_error_t	NPF_IfATM_ConnectionXcSet(
				NPF_IN		NPF_callbackHandle_t 	if_cbHandle,
				NPF_IN		NPF_correlator_t			if_cbCorrelator,
				NPF_IN		NPF_errorReporting_t		if_errorReporting,
				NPF_IN		NPF_IfHandle_t				if_handle,
				NPF_IN		NPF_uint32_t				n_Vccs,
				NPF_IN		NPF_ATM_ConnectionXc_t	*if_connXc);


/* Delete Connection Cross-connect */
NPF_error_t	NPF_IfATM_ConnectionXcDelete(
				NPF_IN		NPF_callbackHandle_t 	if_cbHandle,
				NPF_IN		NPF_correlator_t			if_cbCorrelator,
				NPF_IN		NPF_errorReporting_t		if_errorReporting,
				NPF_IN		NPF_IfHandle_t				if_handle,
				NPF_IN		NPF_uint32_t				n_Vccs,
				NPF_IN		NPF_ATM_ConnectionXc_t	*if_connXc);

/* Configure a Connection Point with OAM Attributes */
NPF_error_t	NPF_IfATM_OAM_CP_Set(
				NPF_IN		NPF_callbackHandle_t 	if_cbHandle,
				NPF_IN		NPF_correlator_t			if_cbCorrelator,
				NPF_IN		NPF_errorReporting_t		if_errorReporting,
				NPF_IN		NPF_IfHandle_t				if_handle,
				NPF_IN		NPF_uint32_t				n_CPs,
				NPF_IN		NPF_ATM_OAM_CP_t			*if_oamCP_Array);

/* Activate, Deactivate, Start/Stop OAM Continuity Check */
NPF_error_t	NPF_IfATM_OAM_CC_Set(
				NPF_IN		NPF_callbackHandle_t 	if_cbHandle,
				NPF_IN		NPF_correlator_t			if_cbCorrelator,
				NPF_IN		NPF_errorReporting_t		if_errorReporting,
				NPF_IN		NPF_IfHandle_t				if_handle,
				NPF_IN		NPF_uint32_t				n_CPs,
				NPF_IN		NPF_ATM_OAM_CC_t			*if_oamCC_Array);

/* Activate, Deactivate, Start and Stop OAM Performance */
NPF_error_t	NPF_IfATM_OAM_PM_Set(
				NPF_IN		NPF_callbackHandle_t 	if_cbHandle,
				NPF_IN		NPF_correlator_t			if_cbCorrelator,
				NPF_IN		NPF_errorReporting_t		if_errorReporting,
				NPF_IN		NPF_IfHandle_t				if_handle,
				NPF_IN		NPF_uint32_t				n_CPs,
				NPF_IN		NPF_ATM_OAM_PM_t			*if_oamPM_Array);

/* Initiate an OAM Loopback Procedure */
NPF_error_t	NPF_IfATM_OAM_LB_Set(
				NPF_IN		NPF_callbackHandle_t 	if_cbHandle,
				NPF_IN		NPF_correlator_t			if_cbCorrelator,
				NPF_IN		NPF_errorReporting_t		if_errorReporting,
				NPF_IN		NPF_IfHandle_t				if_handle,
				NPF_IN		NPF_uint32_t				n_CPs,
				NPF_IN		NPF_ATM_OAM_LB_t			*if_oamLB_Array);

/* Activate, Deactivate Cell Monitoring */
NPF_error_t	NPF_IfATM_OAM_Monitor(
				NPF_IN		NPF_callbackHandle_t 	if_cbHandle,
				NPF_IN		NPF_correlator_t			if_cbCorrelator,
				NPF_IN		NPF_errorReporting_t		if_errorReporting,
				NPF_IN		NPF_IfHandle_t				if_handle,
				NPF_IN		NPF_uint32_t				n_CPs,
				NPF_IN		NPF_ATM_OAM_Mon_t			*if_oamMonArray);

/* Enable AIS/RDI Alarms */
NPF_error_t	NPF_IfATM_OAM_AlarmSet(
				NPF_IN		NPF_callbackHandle_t 	if_cbHandle,
				NPF_IN		NPF_correlator_t			if_cbCorrelator,
				NPF_IN		NPF_errorReporting_t		if_errorReporting,
				NPF_IN		NPF_IfHandle_t				if_handle,
				NPF_IN		NPF_uint32_t				n_CPs,
				NPF_IN		NPF_ATM_OAM_Alarm_t		*if_oamAlarmArray);

/* Disable AIS/RDI Alarms */
NPF_error_t	NPF_IfATM_OAM_AlarmClear(
				NPF_IN		NPF_callbackHandle_t 	if_cbHandle,
				NPF_IN		NPF_correlator_t			if_cbCorrelator,
				NPF_IN		NPF_errorReporting_t		if_errorReporting,
				NPF_IN		NPF_IfHandle_t				if_handle,
				NPF_IN		NPF_uint32_t				n_CPs,
				NPF_IN		NPF_ATM_OAM_Alarm_t		*if_oamAlarmArray);

/* Create an AAL2 Channel */
NPF_error_t	NPF_IfATM_AAL2_ChannelSet(
				NPF_IN		NPF_callbackHandle_t 	if_cbHandle,
				NPF_IN		NPF_correlator_t			if_cbCorrelator,
				NPF_IN		NPF_errorReporting_t		if_errorReporting,
				NPF_IN		NPF_IfHandle_t				if_handle,
				NPF_IN		NPF_uint32_t				n_channels,
				NPF_IN		NPF_ATM_AAL2_ChannelInfo_t		*if_aal2ChannelArray);

/* Delete an AAL2 Channel */
NPF_error_t	NPF_IfATM_AAL2_ChannelClear(
				NPF_IN		NPF_callbackHandle_t 	if_cbHandle,
				NPF_IN		NPF_correlator_t			if_cbCorrelator,
				NPF_IN		NPF_errorReporting_t		if_errorReporting,
				NPF_IN		NPF_IfHandle_t				if_handle,
				NPF_IN		NPF_uint32_t				n_channels,
				NPF_IN		NPF_ATM_AAL2_ConnId_t	*if_connIdArray);


/* ATM Event Handler Registration Function */
NPF_error_t	NPF_ATM_EventRegister(
				NPF_IN		NPF_userContext_t			userContext,
				NPF_IN		NPF_ATM_EventHandlerFunc_t	atmEventHandlerFunc,
				NPF_IN		NPF_IfHandle_t				ifHandle,
				NPF_IN		NPF_uint32_t				n_Vccs,
				NPF_IN		NPF_VccAddr_t				*vccAddrArray,
				NPF_OUT		NPF_ATM_EventHandlerHandle_t	*atmEventHandlerHandle);

/* ATM Event Handler De-registration Function */
NPF_error_t	NPF_ATM_EventDeregister(
				NPF_IN		NPF_ATM_EventHandlerHandle_t	atmEventHandlerHandle);





#ifdef __cplusplus
}
#endif
#endif /* _INCLUDE_ATM */
#endif /* __NPF_ATM_API_H__ */
