/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2002 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/npf_api/npf_atm_types.h,v $
 * $Revision: 1.5 $
 * $Date: 2003/11/17 16:58:00 $
 *
 * Description:
 * This file defines all of the types that are used by the ATM API.
 *
 ****************************************************************************/

#ifndef __NPF_ATM_TYPES_H__
#define __NPF_ATM_TYPES_H__

#ifdef _INCLUDE_ATM

#ifdef __cplusplus
extern "C" {
#endif

typedef NPF_uint32_t NPF_ATM_EventHandlerHandle_t; 
/*
 *	Service Category of the virtual channel connection
 */
typedef enum
{
		NPF_OTHER		= 0,				
		NPF_CBR			= 1,				/* CBR service */
		NPF_rtVBR		= 2,				/* Real time VBR service */
		NPF_nrtVBR		= 3,				/* Non-real time VBR service */
		NPF_ABR			= 4,				/* ABR service */
		NPF_UBR			= 5,				/* UBR service */
		NPF_GFR			= 6				/* GFR service */
}NPF_ServiceCategory_t; 

/* 
 *	ATM QOS profile
 */
typedef struct
{
		NPF_ServiceCategory_t 	serviceCategory;	/* Service Category */
		NPF_uint32_t				rxPCR1;				/* Ingress Peak Cell Rate (CLP = 0+1) */
		NPF_uint32_t				rxPCR0;				/* Ingress Peak Cell Rate (CLP = 0) */
		NPF_uint32_t				rxSCR1;				/* Ingress Sustainable Cell Rate (CLP = 0+1) */
		NPF_uint32_t				rxSCR0;				/* Ingress Sustainable Cell Rate (CLP = 0) */
		NPF_uint32_t				rxMBS1;				/* Ingress Max Burst Size (CLP = 0+1) */
		NPF_uint32_t				rxMBS0;				/* Ingress Max Burst Size (CLP = 0) */
		NPF_uint32_t				rxMCR;				/* Ingress Min Cell Rate */
		NPF_uint32_t				rxMaxFrmSize;		/* Ingress Max Frame Size */
		NPF_uint32_t				rxCTD;				/* Ingress Cell Transfer Delay */
		NPF_uint32_t				rxCDV;				/* Ingress Peak-to-Peak Cell Delay variation */
		NPF_uint32_t				rxCLR;				/* Ingress Cell Loss Ration */
		NPF_uint32_t				rxCDVT;				/* Ingress Cell Delay Variation Tolerance */
		NPF_uint16_t				rxQosClass;			/* QOS class */
		NPF_boolean_t				rxFrameDiscard;	/* Receive Frame Discard */
		NPF_boolean_t				rxTagging;			/* Tagging option */

		NPF_uint32_t				txPCR1;				/* Egress Peak Cell Rate (CLP = 0+1) */
		NPF_uint32_t				txPCR0;				/* Egress Peak Cell Rate (CLP = 0) */
		NPF_uint32_t				txSCR1;				/* Egress Sustainable Cell Rate (CLP = 0+1) */
		NPF_uint32_t				txSCR0;				/* Egress Sustainable Cell Rate (CLP = 0) */
		NPF_uint32_t				txMBS1;				/* Egress Max Burst Size (CLP = 0+1) */
		NPF_uint32_t				txMBS0;				/* Egress Max Burst Size (CLP = 0) */
		NPF_uint32_t				txMCR;				/* Egress Min Cell Rate */
		NPF_uint32_t				txMaxFrmSize;		/* Egress Max Frame Size */
		NPF_uint32_t				txCTD;				/* Egress Cell Transfer Delay */
		NPF_uint32_t				txCDV;				/* Egress Peak-to-Peak Cell Delay variation */
		NPF_uint32_t				txCLR;				/* Egress Cell Loss Ration */
		NPF_uint32_t				txCDVT;				/* Egress Cell Delay Variation Tolerance */
		NPF_uint16_t				txQosClass;			/* Egress QOS class */
		NPF_boolean_t				txFrameDiscard;	/* Egress Frame Discard */
		NPF_boolean_t				txTagging;			/* Egress Tagging option */

		NPF_boolean_t				bestEffIndicator;	/* Best Effort Indicator */
}NPF_IfATM_QoS_t;



/*
 *	AAL1 subtype used by CBR service application
 */
typedef enum
{
		NPF_NULL 									= 0,
		NPF_VOICEBAND 								= 1,
		NPF_CIRCUIT_EMULATION_SYNCHRONOUS 	= 2,
		NPF_CIRCUIT_EMULATION_ASYNCHRONOUS 	= 3,
		NPF_HIGH_QUALITY_AUDIO 					= 4,
		NPF_VIDEO 									= 5
}NPF_AAL1Subtype_t;

/*
 *	CBR Rate
 */
typedef enum
{
		NPF_64_KBPS				= 0,
		NPF_1544_KBPS			= 1,
		NPF_6312_KBPS			= 2,
		NPF_32064_KBPS			= 3,
		NPF_44736_KBPS			= 4,
		NPF_97728_KBPS			= 5,
		NPF_2048_KBPS			= 6,
		NPF_8448_KBPS			= 7,
		NPF_34368_KBPS			= 8,
		NPF_139264_KBPS		= 9,
		NPF_N64_KBPS			= 10,
		NPF_N8_KBPS				= 11
}NPF_CBR_t;

/*
 *	Clock Recovery Type
 */
typedef enum
{
		NPF_SYNCHRONOUS		= 0,			/* Synchronous */
		NPF_SRTS					= 1,			/* Asynchronous-SRTS(Sync residual time stamp) */
		NPF_ADAPTIVE			= 2			/* Asynchronous - Adaptive Clock Recovery */
}NPF_AAL1ClkRecoveryType_t;


 
/*
 *	Frame Error Check (FEC) Method
 */
typedef enum
{
		NPF_NO_FEC 								= 0,			/* No FEC */
		NPF_LOSS_SENSITIVE_SIGNAL_FEC		= 1,			/* FEC for Loss sensitive signal transport */
		NPF_DELAY_SENSITIVE_SIGNAL_FEC	= 2			/* FEC for delay sensitive */
}NPF_AAL1FEC_t;

/*
 * AAL1 Profile 
 */
typedef struct
{
		NPF_AAL1Subtype_t				subtype;				/* AAL1 sub-type */
		NPF_CBR_t						cbrRate;				/* CBR service */
		NPF_uint32_t					rateMultiplier;	/* Rate Multiplier */
		NPF_AAL1ClkRecoveryType_t	clkRecoveryType;	/* Clock Recovery Type */
		NPF_AAL1FEC_t					fecType;				/* Frame Error Check method */
		NPF_boolean_t					sdtSupport;			/* SDT configured */
		NPF_boolean_t					sdtBlockSize;		/* SDT Block Size */
		NPF_uint32_t					partiallyFilledCells;	/* Nos of partially fill cells */
		NPF_uint32_t					cellLossIntegrPeriod;
}NPF_AAL1_Profile_t;

typedef enum
{
		NPF_AUDIO 					= 1,
		NPF_MULTIRATE				= 2,
		NPF_AUDIO_AND_MULTIRATE = 3
}NPF_AAL2SscsServiceCategory;

typedef enum
{
		NPF_OTHER_SSCS_PROFILE	 = 0,
		NPF_ITUT			 			 = 1
}NPF_AAL2SscsProfileSource_t;

typedef enum
{
		NPF_ALAW		 = 1,
		NPF_ULAW		 = 2
}NPF_AAL2SscsPcmEncoding_t;



/*
 *	ATM Trunking AAL2 Profile 
 */
typedef struct
{
		NPF_uint32_t		vcci;					/* Uniquely identifies a VCC among IWFs */
		NPF_uint32_t		sigVcci;				/* VCCI to carry the CCS */
}NPF_AAL2TrunkingProfileEntry_t;

typedef enum
{
		NPF_SINGLE_CPS_PACKET_PER_CPS_PDU_NO_OVERLAP			= 1,
		NPF_MULTIPLE_CPS_PACKETS_PER_CPS_PDU_WITH_OVERLAP	= 2
}NPF_AAL2CpsOptimisation_t;


/*
 *	ATM LES AAL2 Profile 
 */
typedef struct
{
		NPF_uint32_t					cpsCIDLowerLimit;		/* Min Value of CID */
		NPF_uint32_t					cpsCIDUpperLimit;		/* Max Value of CID */
		NPF_AAL2CpsOptimisation_t	cpsOptimisation;		
}NPF_AAL2LES_ProfileEntry_t;

/*
 *	SSCS Type
 */
typedef enum
{
		NPF_SSCS_NULL	 		= 0,
		NPF_DATA_SSURED		= 1,
		NPF_DATA_NON_ASSURED	= 2,
		NPF_FRAME_RELAY		= 4,
		NPF_SSCF_CONS			= 8,
		NPF_SSCF_COTS			= 9,
		NPF_SAR					= 16,
		NPF_TRUNKING			= 17
}NPF_SscsType_t;



/*
 *	AAL2 Profile
 */
typedef struct
{
	
		NPF_uint32_t 	cpsMaxMultiplexedChannels;			/* Max Mux Channels */
		NPF_uint32_t	cpsMaxSduLength;						/* Max CPS-SDU size */
		NPF_uint32_t	cpsTimer_CU;							/* Combined use Timer CU */
		NPF_SscsType_t	sscsType;								/* SSCS Type */
		NPF_uint32_t	appId;									/* Application Id */
		
		/* Common SSCS Parameters (SSCS type = SAR) as specified in I.366.1 */			
		NPF_boolean_t	sstedStatus;							/* SSTED selected */
		NPF_boolean_t	ssadtStatus;							/* SSADT selected */
		NPF_uint32_t	maxSssarSduLength;					/* Max SSSAR-SDU size */
		NPF_uint32_t	maxSssarSegLength;					/* Max SSSAR-PDU size (1...MaxCPSDeliverLength) */
		NPF_uint32_t	rasTimer;								/* RAS Timer */
		
		/* Common SSCS Parameters (SSCS type = trunking) as specified in I.366.2 */
		NPF_AAL2SscsServiceCategory	srvCategory;		/* Service Category */
		NPF_boolean_t	circuitModeDataTransport;			/* Circuit Mode Data Support */
		NPF_boolean_t	frameModeDataTransport;				/* Frame Mode Data Support */
		NPF_boolean_t	faxDemodTransport;					/* Demodulated Fax Data Support */
		NPF_boolean_t	casTransport;							/* CAS Support */
		NPF_boolean_t	dtmfDigitPacketTransport;			/* DTMF dialed digit support */
		NPF_boolean_t	mfR1DigitPacketTransport;			/* MF-R1 dialed digit support */
		NPF_boolean_t	mfR2DigitPacketTransport;			/* MF-R2 dialed digit support */
		NPF_boolean_t	audioServiceTransport;				/* Audio Transport */
		NPF_AAL2SscsPcmEncoding_t	pcmEncoding;			/* PCM Encoding Type */
		NPF_uint32_t	maxFrameModeDataLength;				/* Max I.366.2 frame data length */
		NPF_AAL2SscsProfileSource_t	profileSource;		/* Profile Source */
		NPF_uint32_t	predefinedProfileIdentifier;		/* Predefined Profile Id */
		NPF_uint32_t	ieeeOui;									/* IEEE OUI */
		NPF_uint32_t	circuitModeDataNumChannels;		/* N*64Kbits/s circuit mode data */

		/* Common SSCOP Parameters as specified in Q.2110 */
		NPF_uint32_t	sscopSduLength;						/* Max SSCOP-SDU Length */
		NPF_uint32_t	sscopUuFieldLength;					/* Max SSCOP-UU Length */
		
		NPF_AAL2TrunkingProfileEntry_t	aal2TrunkingProfile; /* AAL2 Trunking profile */

		NPF_AAL2LES_ProfileEntry_t			aal2LESProfile;		/* AAL2 CPS Profile for LES */
}NPF_AAL2_Profile_t;
		
			


typedef enum
{
		NPF_MESSAGE_ASSURED		= 0,
		NPF_MESSAGE_UNASSURED	= 1,
		NPF_STREAMING_ASSURED	= 2,
		NPF_STREAMING_UNASSURED	= 3
}NPF_AAL_Mode_t;			
		

/* 
 *	AAL3/4 Profile 
 */
typedef struct
{
		NPF_uint32_t		maxCpcsSduSizeForward;		/* Max incoming CPCS PDU size */
		NPF_uint32_t		maxCpcsSduSizeBackward;		/* Max outgoing CPCS PDU size */
		NPF_uint32_t		midRangeLow;					/* Low Mid Value for VCC */
		NPF_uint32_t		midRangeHigh;					/* high Mid Value for VCC */
		NPF_AAL_Mode_t		aalMode;							/* AAL Mode */
		NPF_SscsType_t		sscsType;						/* SSCS Type */
}NPF_AAL3_4_Profile_t;	


/* 
 *	AAL5 Profile
 */
typedef struct
{
		NPF_uint32_t		maxCpcsSduSizeForward;	/* Max outgoing CPCS PDU size */
		NPF_uint32_t		maxCpcsSduSizeBackward;	/* Max incoming CPCS PDU size */
		NPF_AAL_Mode_t		aalMode;						/* AAL Mode */
		NPF_SscsType_t		sscsType;					/* SSCS Type */
}NPF_AAL5_Profile_t;

/*
 * AAL Profile
 */
typedef struct 
{
		NPF_IfAAL_t					AAL;					/* AAL type */
		union{
			NPF_AAL1_Profile_t			aal1Profile;		/* AAL1 Profile */
			NPF_AAL2_Profile_t			aal2Profile;		/* AAL2 Profile */
			NPF_AAL3_4_Profile_t			aal34Profile;		/* AAL3/4 Profile */
			NPF_AAL5_Profile_t			aal5Profile;		/* AAL5 Profile */
		}u;
}NPF_IfAAL_Profile_t;

/*
 * ATM Vcc attributes
 */
typedef struct
{
		NPF_VccAddr_t				vcc;					/* VPI/VCI value of VCC */
		NPF_IfAAL_t					AAL;					/* AAL type */
		NPF_IfAAL_Profile_t		aalProfile;			/* AAL Profile */
		NPF_IfHandle_t				parent;				/* Parent interface handle */
		NPF_IfATM_QoS_t			QoS;					/* QoS Profile */
}NPF_IfVcc_t;


/* 
 * OAM Alarm Type
 */
typedef enum
{
		NPF_IF_ATM_F4_SEG_AIS_ALARM = 0,  /* F4 Segment AIS alarm */
		NPF_IF_ATM_F4_SEG_RDI_ALARM = 1,  /* F4 Segment RDI alarm */
		NPF_IF_ATM_F4_EP_AIS_ALARM  = 2,  /* F4 End Point AIS alarm */
		NPF_IF_ATM_F4_EP_RDI_ALARM  = 3,  /* F4 End Point RDI alarm */
		NPF_IF_ATM_F5_SEG_AIS_ALARM = 4,  /* F5 Segment AIS alarm */
		NPF_IF_ATM_F5_SEG_RDI_ALARM = 5,  /* F5 Segment RDI alarm */
		NPF_IF_ATM_F5_EP_AIS_ALARM  = 6,  /* F5 End Point AIS alarm */
		NPF_IF_ATM_F5_EP_RDI_ALARM  = 7  /* F5 End Point RDI alarm */
}NPF_OAM_AlarmType_t;


/* 
* OAM Fault Management(AIS/RDI/CC/LB) and Performance Management(FPM/BR)
* cell type
*/
typedef enum
{
		NPF_AIS_CELL = 0,  /* Alarm Indication Signal Cell */
		NPF_RDI_CELL = 1,  /* Remote Defect Indicator Cell */
		NPF_CC_CELL  = 2,  /* Continuity Check Cell */
		NPF_LB_CELL  = 3,  /* Loopback Cell */
		NPF_FPM_CELL = 4,  /* Forward Performance Monitoring Cell */
		NPF_BR_CELL  = 5   /* Backward Reporting Cell */
}NPF_OAM_CellType_t;


/* 
 * OAM Loopback Application Type
 */
typedef enum
{
		NPF_LB_END_TO_END	= 0,
		NPF_ACCESS_LINE	= 1,
		NPF_INTER_DOMAIN	= 2,
		NPF_NET_TO_EP		= 3,
		NPF_INTRA_DOMAIN	= 4
}NPF_OAM_LB_AppType_t;


/* 
 * OAM Flow Type
 */
typedef enum
{
		NPF_SEGMENT = 0,
		NPF_END_TO_END = 1
}NPF_OAM_FlowType_t;


/* 
 * OAM Oper Type
 */
typedef enum
{
		NPF_ACTIVATE    = 0,
		NPF_DEACTIVATE  = 1,
		NPF_START       = 2,
		NPF_STOP        = 3
}NPF_OAM_OperType_t;


/* 
 * OAM CP Type
 */
typedef enum
{
		NPF_ENDPOINT             = 0,
		NPF_CP_SEGMENT              = 1,
		NPF_ENDPOINT_AND_SEGMENT = 2,
		NPF_INTERMEDIATE         = 3
}NPF_OAM_CP_Type_t;

/* 
 * OAM PM Function Type
 */
typedef enum
{
		NPF_IF_FPM   	= 1,
		NPF_IF_FPM_BR	= 2
}NPF_OAM_FuncType_t;

/* 
 * OAM Role
 */
typedef enum
{
		NPF_SOURCE          = 0,
		NPF_SINK            = 1,
		NPF_SOURCE_AND_SINK = 2
}NPF_OAM_Role_t;


/* 
 * OAM PM Block Size
 */
typedef enum
{
		NPF_SIZE_32768 = 0,
		NPF_SIZE_16384 = 1,
		NPF_SIZE_8192  = 2,
		NPF_SIZE_4096  = 3,
		NPF_SIZE_2048  = 4,
		NPF_SIZE_1024  = 5,
		NPF_SIZE_512   = 6,
		NPF_SIZE_256   = 7,
		NPF_SIZE_128   = 8
}NPF_OAM_BlkSize_t;


/* 
 * OAM Direction
 */
typedef enum
{
		NPF_FORWARD        = 0,    /* A-B */
		NPF_BACKWARD       = 1,    /* B-A */
		NPF_TWO_WAY        = 2,    /* TWO-WAY */
		NPF_NOT_APPLICABLE = 3     /* NOT APPLICABLE */
}NPF_OAM_Direction_t;

/*
 *	OAM CP configuration
 */
typedef struct
{
		NPF_VccAddr_t			connection;		/* VPI/VCI */
		NPF_OAM_CP_Type_t		cpType;			/* Endpoint/Segment/both/intermediate */
		NPF_OAM_Role_t			roleType;		/* Source/Sink/Both */
		NPF_OAM_Direction_t	oamDir;			/* OAM direction (Forward/Backward/Both) */
}NPF_ATM_OAM_CP_t;

/*
 *	OAM CC Activation/Deactivation
 */
typedef struct{
		NPF_VccAddr_t			connection;		/* VPI/VCI */
		NPF_OAM_FlowType_t	flowType;		/* Segment/End-to-End */
		NPF_OAM_OperType_t	operType;		/* Activate/Deactivate/Start/Stop */
}NPF_ATM_OAM_CC_t;

/*
 *	OAM PM Activation/Deactivation -  FPM-BR/FPM
 */
typedef struct{
		NPF_VccAddr_t			connection;		/* VPI/VCI */
		NPF_OAM_FlowType_t	flowType;		/* Segment/end-to-end */
		NPF_OAM_FuncType_t	funcType;		/* FPM-BR/FPM */
		NPF_OAM_Direction_t	dir;				/* Away/Towards/Both */
		NPF_OAM_BlkSize_t		fwdSize;			/* A-B block Size */
		NPF_OAM_BlkSize_t		bwdSize;			/* B-A block Size */
		NPF_OAM_OperType_t	operType;		/* Activate/Deactivate/Start/Stop */
}NPF_ATM_OAM_PM_t;

/*
 *	OAM Loopback
 */
typedef struct{
		NPF_VccAddr_t			connection;		/* VPI/VCI */
		NPF_OAM_FlowType_t	flowType;		/* Segment/end-to-end */
		NPF_OAM_LB_AppType_t	lbAppType;		/* Loopback application type */ 		
		NPF_uchar8_t           llId[16];    /* Loopback location Id */
		NPF_boolean_t          remCell;     /* Remove Loopback Cells */
} NPF_ATM_OAM_LB_t;




/*
* OAM Non-Intrusive Monitoring 
*/
typedef struct {
    NPF_VccAddr_t          connection;  /* VPI/VCI */
    NPF_OAM_FlowType_t     flowType;    /* segment/end-to-end */
    NPF_OAM_CellType_t     cellType;    /* OAM FM and PM Cell Type */
} NPF_ATM_OAM_Mon_t;


/*
 * OAM Alarms - Declare and Release 
 */
typedef struct
{
		NPF_VccAddr_t			connection;		/* VPI/VCI */
		NPF_OAM_AlarmType_t	cellType;		/* OAM AIS/RDI Cell Type */
} NPF_ATM_OAM_Alarm_t;


/*
 *   AAL2 Connection Info.
 */
typedef struct
{
		NPF_ATM_AAL2_ConnId_t  aal2ConnId;             /* AAL2 Connection Id */

		/* Specified in I.363.2 */
		NPF_uint16_t     maxCpsSduDeliverLength; /* Maximum length of CPS SDU */
}NPF_ATM_AAL2_ChannelInfo_t;


/*
 *	Data Structures for ATM Event Notification
 */


/*
 *   ATM Event Types
 */
typedef enum
{
		NPF_IF_ATM_VCC_UP = 1,  /* VCC on the Interface is UP */
		NPF_IF_ATM_VCC_DWN = 2, /* VCC on the Interface is DOWN */

		NPF_IF_ATM_F4_SEG_AIS =  3,  /* F4 Segment AIS alarm */
		NPF_IF_ATM_F4_SEG_RDI =  4,  /* F4 Segment RDI alarm */
		NPF_IF_ATM_F4_EP_AIS  =  5,  /* F4 End Point AIS alarm */
		NPF_IF_ATM_F4_EP_RDI  =  6,  /* F4 End Point RDI alarm */
		NPF_IF_ATM_F5_SEG_AIS =  7,  /* F5 Segment AIS alarm */
		NPF_IF_ATM_F5_SEG_RDI =  8,  /* F5 Segment RDI alarm */
		NPF_IF_ATM_F5_EP_AIS  =  9,  /* F5 End Point AIS alarm */
		NPF_IF_ATM_F5_EP_RDI  = 10,  /* F5 End Point RDI alarm */

		NPF_IF_ATM_F4_SEG_CC = 11,   /* F4 Segment CC */
		NPF_IF_ATM_F4_EP_CC  = 12,   /* F4 End Point CC */
		NPF_IF_ATM_F5_SEG_CC = 13,   /* F5 Segment CC */
		NPF_IF_ATM_F5_EP_CC  = 14,   /* F5 End Point CC */

		NPF_IF_ATM_F4_SEG_FPM  = 15, /* F4 Segment FPM */
		NPF_IF_ATM_F4_EP_FPM   = 16, /* F4 End Point FPM */
		NPF_IF_ATM_F5_SEG_FPM  = 17, /* F5 Segment FPM */
		NPF_IF_ATM_F5_EP_FPM   = 18, /* F5 End Point FPM */

		NPF_IF_ATM_F4_SEG_BR = 19, /* F4 Segment BR */
		NPF_IF_ATM_F4_EP_BR  = 20, /* F4 End Point BR */
		NPF_IF_ATM_F5_SEG_BR = 21, /* F5 Segment BR */
		NPF_IF_ATM_F5_EP_BR  = 22,  /* F5 End Point BR */

		NPF_IF_ATM_F4_SEG_LB = 23,   /* F4 Segment LB */
		NPF_IF_ATM_F4_EP_LB  = 24,   /* F4 End Point LB */
		NPF_IF_ATM_F5_SEG_LB = 25,   /* F5 Segment LB */
		NPF_IF_ATM_F5_EP_LB  = 26   /* F5 End Point LB */
}NPF_ATM_Event_t;


/*
 *   AIS/RDI Fault Management Cell Info
 */
typedef struct
{
		NPF_uint16_t defectType;         /* Defect type */
		NPF_char8_t  defectLocation[16]; /* Defect Location */
}NPF_OAM_AisRdi_EventInfo_t;


/*
 *   FPM Cell Info
 */
typedef struct
{
		NPF_uchar8_t  mscn;           /* FPM Monitoring Cell Seq. No. */
		NPF_uint16_t  totUsrCell1;    /* Total User Cell(CLP-0+1) */
		NPF_uint16_t  totUsrCell0;    /* Total User Cell(CLP-0) */
		NPF_uint16_t  blkErrDetCode;  /* Block Error Detection Code(CLP-0+1) */
		NPF_uint32_t  timeStamp;      /* Time Stamp */
}NPF_OAM_FPM_EventInfo_t;


/*
 *   BR Cell Info
 */
typedef struct
{
		NPF_uchar8_t  mscn;            /* BR Monitoring Cell Seq. No. */
		NPF_uint16_t  totUsrCell1;     /* Total User Cell(CLP-0+1) */
		NPF_uint16_t  totUsrCell0;     /* Total User Cell(CLP-0) */
		NPF_uint32_t  timeStamp;       /* Time Stamp */
		NPF_uchar8_t  repMscn;         /* Reported Monitoring Cell Seq. No. */
		NPF_uchar8_t  secbc;           /* Severely Errored Cell Sequence Number */
		NPF_uint16_t  totRcvdUsrCell0; /* Total Received User Cell(CLP-0) */
		NPF_uint8_t   blkErr;          /* Block Error Result(CLP-0+1) */
		NPF_uint16_t  totRcvdUsrCell1; /* Total Received User Cell(CLP-0+1) */
}NPF_OAM_BR_EventInfo_t;


/*
 *   Loopback Cell Info
 */
typedef struct
{
		NPF_uchar8_t  llId[16];          /* Loopback Location Id */
}NPF_OAM_LB_EventInfo_t;


/*
*   ATM Event Notification 
*/
typedef struct
{
		NPF_ATM_Event_t eventType;     /* Event type */
		NPF_IfHandle_t if_handle;     /* Interface Handle */
		NPF_VccAddr_t  vcc_Addr;      /* VPI/VCI */
		union
		{
		   NPF_OAM_AisRdi_EventInfo_t oamAisRdiInfo; /* AIS/RDI event info */
			NPF_OAM_FPM_EventInfo_t   fpmStats;			/* FPM event info */
			NPF_OAM_BR_EventInfo_t    brStats;			/* BR event info */
   		NPF_OAM_LB_EventInfo_t    oamLBInfo;		/* LB event info */
		}u;
}NPF_ATM_EventData_t;


typedef struct
{
		NPF_uint16_t 	     n_data;       /* Number of events in array */
		NPF_ATM_EventData_t *eventData;   /* Array of event notifications */
}NPF_ATM_EventArray_t;




/*
 *   ATM Event Handler Function
 */
typedef void (*NPF_ATM_EventHandlerFunc_t)(
			NPF_IN NPF_userContext_t     userContext,
			NPF_IN NPF_ATM_EventArray_t  atmEventArray);



#ifdef __cplusplus
}
#endif

#endif /* _INCLUDE_ATM */
#endif /* __NPF_ATM_TYPES_H__ */
