/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/

/*****************************************************************************
* $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/npf_api/npf_cm_types.h,v $
* $Revision: 1.6 $
* $Date: 2003/11/17 16:58:00 $
*
* Description: It has C&M typedefs.
*****************************************************************************/

#ifndef _NPF_CM_TYPES_H_
#define _NPF_CM_TYPES_H_

#ifdef __cplusplus
extern "C" {
#endif 

#include <time.h>
#include "ipaddr.h"
#include "hwaddr.h"

/* ------------------------------- Event Types ----------------------------- */
typedef enum {	NPF_EVENT_FE_BIND,
		NPF_EVENT_FE_UP,
		NPF_EVENT_FE_DOWN,
		NPF_EVENT_IF_UP,
		NPF_EVENT_IF_DOWN,
		NPF_EVENT_IF_DELETE,
		NPF_EVENT_PORT_LINKUP,
		NPF_EVENT_PORT_LINKDOWN,
		NPF_EVENT_PORT_MTU_CHANGE,
		NPF_EVENT_PORT_LINKSPEED_CHANGE,
		NPF_EVENT_IF_IPADDR_CHANGE,
		NPF_EVENT_IF_IPFORWARDING_CHANGE,
		NPF_EVENT_ATM_LINK_UP,
		NPF_EVENT_ATM_LINK_DOWN,
		NPF_EVENT_ATM_CONN_UP,
		NPF_EVENT_ATM_CONN_FAILURE,
		NPF_EVENT_ATM_CONN_STATISTICS,
		NPF_EVENT_ATM_OAM_AIS_ALARM,
		NPF_EVENT_ATM_OAM_RDI_ALARM,
		NPF_EVENT_ATM_OAM_LOOPBACK,
		NPF_EVENT_ATM_OAM_PM_STATISTICS } NPF_EVENT; 

/* ------------------------------- FE CB Types ----------------------------- */

typedef enum {	NPF_FE_GET_ATTRIBS,
		NPF_FE_SET_NAME,  /* not used */
		NPF_FE_SET_DESC,
		NPF_FE_SET_IPADDR,  /* not used */
		NPF_FE_INITIALIZE,
		NPF_FE_SHUTDOWN    } NPF_FE_CBTYPE;    /* FE Mgt */


/* ------------------------------- IF CB Types ----------------------------- */
typedef enum {	NPF_IF_GET_ATTRIBS,
		NPF_IF_SET_PORT_STATUS,
		NPF_IF_SET_NAME,
		NPF_IF_SET_DESC,
		NPF_IF_SET_IPADDR,
		NPF_IF_DEL_IPADDR,
		NPF_IF_SET_FORWARDING,
		NPF_IF_GET_STATISTICS    } NPF_IF_CBTYPE;    /* IF Mgt */

/* ------------------------------- PORT Types ----------------------------- */
typedef enum {	NPF_PORT_ETHERNET,
		NPF_PORT_ATM,
		NPF_PORT_MPLS,
		NPF_PORT_POS,
		NPF_PORT_UNKNOWN }  NPF_PORT_TYPE;
                
/* ------------------------------- ICMP CB Types --------------------------- */
typedef enum {  NPF_ICMP_GET_STATISTICS    } NPF_ICMP_CBTYPE;    /* ICMP Mgt */


/* ------------------------------- PORT CB Types --------------------------- */
typedef enum {	NPF_PORT_GET_STATISTICS,
		NPF_PORT_GET_ATTRIBS,
		NPF_PORT_SET_MTU,
		NPF_PORT_SET_LINKSPEED,
		NPF_PORT_SET_INTERNAL,
		NPF_PORT_SET_VPIBITS,
		NPF_PORT_SET_VCIBITS } NPF_PORT_CBTYPE;    /* Port Mgt */

/* ------------------------------- ROUTER Types --------------------------- */
typedef enum {  NPF_ROUTER_IPv4 = 1,
                NPF_ROUTER_IPv6 = 2,
                NPF_ROUTER_BOTH = 3 } NPF_ROUTER_TYPE_t;

/* ------------------------------- IP ROUTER CB Types --------------------- */
typedef enum {	NPF_IPR_GET_ATTRIBS,
		NPF_IPR_SET_ATTRIBS    } NPF_IPR_CBTYPE;    /* IPRouter */

/* ------------------------------- Interface Status --------------------- */
typedef enum {	NPF_IF_DOWN = 0, 
              	NPF_IF_UP, 
              	NPF_IF_TESTING} NPF_IF_STATUS;

/* ------------------------------- FE Status --------------------- */
typedef enum {	NPF_FE_DOWN = 0, 
              	NPF_FE_UP } NPF_FE_STATUS;

/* ------------------------------- System Info --------------------- */
typedef struct NPF_SYSTEM_tag
{
    char        name[128];
    char        desc[256];
    int         num_FEs;      /* Q: Who use and assign it? */
} NPF_SYSTEM_t;

/* ------------------------------- FE  Attribs --------------------- */
typedef struct NPF_FE_ATTRIBS_tag
{
    char        desc[256];
    char        name[128];    /* identify if rebind! must be unique */
    time_t      stamp;
    uint32_t    id;           /* assigned by FE, diff. after re-bind */
    int         num_ports;
    int         status;        
    HWADDR      hwaddr;
    IPADDR      ipaddr;
    /* this should be in caps but for here temprary */
    int         max_arp_table;
    int         max_route_table;
    int         max_prefix_table;
    int         max_nexthop_table;
    int         max_ftn_table;
    int         max_ilm_table;
} NPF_FE_ATTRIBS;

/* ------------------------------- FE  CAPS --------------------- */
typedef struct NPF_FE_CAPS_tag {
    int         max_arp_table;
    int         max_route_table;
    int         max_prefix_table;
    int         max_nexthop_table;
    int         max_ftn_table;
    int         max_ilm_table;
    NPF_ROUTER_TYPE_t        route_table_type;  /* IPv4, IPv6 or BOTH */
    /* diffserv_caps & gtp_caps are retrieved by DiffServ/GTP API,
     * put in PDK_FE_CAPS */
} NPF_FE_CAPS;

/* ------------------------------- FE  Info --------------------- */
typedef struct NPF_FE_tag
{
    NPF_FE_ATTRIBS attribs;
    NPF_FE_CAPS    caps;
    uint32_t       id;        /* equals attribs.id */
} NPF_FE_t;

/* ------------------------------- PORT  Attribs --------------------- */
typedef struct NPF_port_attribs_tag
{
    NPF_PORT_TYPE  type;      /* ETHERNET, ATM, etc. */
    int            status;    /* admin control purpose */
    int            linkstatus;/* FE reports it */
    uint32_t       id;        /* should be FPPI_PortID but weird to be here */
    HWADDR         macaddr;
    int            minTxRate; /* bps */
    int            maxTxRate; /* bps */
    int            curTxRate; /* bps */
    int            MTU;
    int            internal;  /* if it is internal port or not */
} NPF_PORT_ATTRIBS;

/* ------------------------------- ATM PORT  Caps --------------------- */
typedef struct
{
    uint32_t max_link_rate;  /* Kbps */
    int      max_vpi_bits;   /* Maximum VPI bits */
    int      max_vci_bits;   /* Maximum VCI bits */
    int      max_vpc;        /* Maximum number of VPCs */
    int      max_vci;        /* Maximum number of VCCs */
    int      max_svpc_vpi;   /* Maximum Switched VPC VPI */
    int      max_svcc_vpi;   /* Maximum Switched VCC VPI */
    int      min_svcc_vpi;   /* Minimum Switched VCC VCI */
} NPF_ATM_PortCaps;

/* ------------------------------- PORT  Caps --------------------- */
typedef union NPF_PORT_CAPS_tag 
{
    NPF_ATM_PortCaps atm_port_caps;
} NPF_PORT_CAPS;

/* ------------------------------- PORT  Stats --------------------- */
typedef struct NPF_PORT_STATS_t
{
                 /* Number of frames with an alignment error */
    unsigned long dot3StatsAlignmentErrors;
                 /* Number of frames with frame check errors */
    unsigned long dot3StatsFCSErrors;
                 /* Number of successfully transmitted frames for which
                  * there was exactly one collision */
    unsigned long dot3StatsSingleCollisionFrames;
                 /* Number of successfully transmitted frames for which
                  * there were multiple collisions */
    unsigned long dot3StatsMultipleCollisionFrames;
                 /* Number of times the Signal Quality Error Test Error was
                  * generated by the interface */
    unsigned long dot3StatsSQETestErrors;
                 /* Number of the first transmission attempt was delayed
                  * because the medium was busy */
    unsigned long dot3StatsDeferredTransmissions;
                 /* Number of times that a collision was detected later than
                  * 64 octets into the transmission (also added into
                  * CollisionCount) */
    unsigned long dot3StatsLateCollisions;
                 /* Number of frames for which transmission failed because
                  * of excessive collisions */
    unsigned long dot3StatsExcessiveCollisions;
                 /* Number of frames for which transmission failed because of
                  * an internal MAC layer transmit error */
    unsigned long dot3StatsInternalMacTransmitErrors;
                 /* Number of failed transmission attempts because the carrier
                  * sense condition was lost or never asserted */
    unsigned long dot3StatsCarrierSenseErrors;
                 /* Number of received frames that were bigger than the
                  * maximum permitted size */
    unsigned long dot3StatsFrameTooLongs;
                 /* Number of frames for which reception failed due to an
                  * internal MAC layer receive error */
    unsigned long dot3StatsInternalMacReceiveErrors;
                 /* For an interface operating at 100Mbps, the number of
                  * times there was an invalid data symbol when a valid carrier
                  * was present */
    unsigned long dot3StatsSymbolErrors;
                 /* The current mode of operation of the MAC entity */
    unsigned long dot3StatsDuplexStatus;

} NPF_PORT_STATS;

/* ------------------------------- ICMP  Stats --------------------- */
typedef struct NPF_ICMP_STATS_tag
{
                 /* Number of incoming ICMP messages */
    unsigned long icmpInMsgs; 
                 /* Number of incoming ICMP messages containing errors */
    unsigned long icmpInErrors; 
                 /* Number of incoming ICMP Destination Unreachable messages */
    unsigned long icmpInDestUnreach; 
                 /* Number of incoming ICMP Time Exceeded messages */
    unsigned long icmpInTimeExceeds; 
                 /* Number of incoming ICMP Param Problem message - 
                  * A router MUST generate a Parameter Problem message for any 
                  * error not specifically covered by another ICMP message - RFC 1812*/
    unsigned long icmpInParamProbs;
                 /* Number of incoming ICMP Source Quench messages */
    unsigned long icmpInSrcQuenches;
                 /* Number of incoming ICMP Redirect messages */
    unsigned long icmpInRedirects;
                 /* Number of incoming ICMP Echo messages */
    unsigned long icmpInEchos;
                 /* Number of incoming ICMP Echo reply messages */
    unsigned long icmpInEchoReps;
                 /* Number of incoming ICMP Time Stamp messages */
    unsigned long icmpInTimeStamps;
                 /* Number of incoming ICMP Time Stamp reply messages */
    unsigned long icmpInTimeStampsReps;
                 /* Number of incoming ICMP Address mask messages - 
                  * A router SHOULD have a configuration option for each logical 
                  * interface specifying whether the router is allowed to answer 
                  * Address Mask Requests for that interface; this option MUST 
                  * default to allowing responses*/
    unsigned long icmpInAddrMasks;
                 /* Number of incoming ICMP Address mask reply messages */
    unsigned long icmpInAddrMaskReps;
                 /* Corresponding ICMP outgoing messages */
    unsigned long icmpOutMsgs;
    unsigned long icmpOutErrors;
    unsigned long icmpOutTimeExceeds;
    unsigned long icmpOutParamProbs;
    unsigned long icmpOutSrcQuenches;
    unsigned long icmpOutRedirects;
    unsigned long icmpOutEchos;
    unsigned long icmpOutEchoReps;
    unsigned long icmpOutTimeStamps;
    unsigned long icmpOutTimeStampsReps;
    unsigned long icmpOutAddrMasks;
    unsigned long icmpOutAddrMaskReps; 

} NPF_ICMP_STATS_t;

/* ------------------------------- PORT  Info --------------------- */
typedef struct NPF_port_tag
{
    NPF_PORT_ATTRIBS attribs;
    NPF_PORT_CAPS    caps;
} NPF_PORT_t;

/* ------------------------------- IpRouter  Info --------------------- */
typedef struct NPF_iprouter_tag
{
    char name[128];
    char desc[256];
    int  num_interfaces;     
} NPF_IPROUTER_t;

/* - Interface - */
#define MAXNUMIPADDRS	12


typedef struct NPF_IF_IP_ATTRIBS_tag
{
    int type;            // IPv4 = 0, IPv6 = 1
    union /* @!union-switch:type! */
    {
        IPA   ipaddr; /* @!union-case:0! */
        IPv6A ipv6addr; /* @!union-case:1! */
    }addr_u;
    union /* @!union-switch:type! */
    {
        IPA   submask; /* @!union-case:0! */
        IPv6A ipv6submask; /* @!union-case:1! */
    }mask_u;
    int ipfwding;    /* ENABLED, DISABLED */
} NPF_IF_IP_ATTRIBS;

/* ------------------------------- IpRouter  Info --------------------- */
typedef struct NPF_IF_STATS_t {
                 /* Number of octets received on this interface */
    unsigned long ifInOctets; 
                 /* Number of unicast packets received on this interface */
    unsigned long ifInUcastPkts; 
                 /* Num of inbound packets discarded in absence of errors */
    unsigned long ifInDiscards; 
                 /* Number of inbound packets discarded due to errors */
    unsigned long IfInErrors; 
                 /* Num of inbound pkts discarded due to unknown protocol */
    unsigned long ifInUnknownProtos; 
                 /* Num of transmitted NettoMedia octets on this interface */
    unsigned long ifOutOctets; 
                 /* Number of unicast packets transmitted */
    unsigned long ifOutUcastPkts;
                 /* Num of outbound packets discarded but were free of errors */
    unsigned long ifOutDiscards; 
                 /* Number of outbound packets discarded due to errors */
    unsigned long ifOutErrors;
                 /* Number of inbound multicast packets */
    unsigned long ifInMulticastPkts;
                 /* Number of outgoing multicast packets */
    unsigned long OutMulticastPkts; 
                 /* Number of outgoing broadcast packets */
    unsigned long OutBroadcastPkts;
                 /* 64-bit version of counters, for > 20 Mbps */
    unsigned long HCInOctets;
    unsigned long HCInUcastPkts;
    unsigned long HCInMulticastPkts;
    unsigned long HCInBroadcastPkts;
    unsigned long HCOutOctets;
    unsigned long HCOutUcastPkts;
    unsigned long HCOutMulticastPkts;
    unsigned long HCOutBroadcastPkts;

} NPF_IF_STATS;

typedef struct NPF_interface_tag
{   
    char name[128];
    char desc[256];
    int  status;          /* UP, DOWN, TESTING, admin or all link down */
    int  index;           /* If index, no one use it, hack for fehdl in vidd */
    char vifname[128];    /* virtual interface name */
    char vifname6[128];   /* virtual interface name */
    NPF_IF_IP_ATTRIBS ipattribs;
} NPF_INTERFACE_t;

/* - Route Table - */
typedef struct NPF_routetable_tag
{
    int rt_type;	/* IPv4, IPv6 or BOTH */
} NPF_ROUTETABLE_t;

/* - Prefix Table - */
typedef struct NPF_prefixtable_tag
{
    int bogus;	/* not use at all */
} NPF_PREFIXTABLE_t;

/* - NextHop Table - */
typedef struct NPF_nexthoptable_tag
{
    int bogus;	/* not use at all */
} NPF_NEXTHOPTABLE_t;

/* - ARP Table - */
typedef struct NPF_arptable_tag
{
    int bogus;	/* not use at all */
} NPF_ARPTABLE_t;

/* - FTN Table -  */
typedef struct NPF_ftntable_tag
{
    int bogus;  /* not use at all */
} NPF_FTNTABLE_t;

/* - ILM Table - */
typedef struct NPF_ilmtable_tag
{
    int bogus;  /* not use at all */
} NPF_ILMTABLE_t;

/* - Internal Configuration - */
typedef struct NPF_intconfig_tag
{
    int bogus;  /* not use at all */
} NPF_INTCONFIG_t;

/*
 * FE Capability Struct
 */

typedef struct NPF_FeCapability
{
   uint32_t              feId;
   time_t                timeStamp;
   IPADDR                feIpAddr;
   HWADDR                feMacAddr;
   int                   status;
   uint32_t              numPorts;
   NPF_PORT_ATTRIBS      *ports;
} NPF_FeCapability_t;


/* ---------------------- Event Notification Callback -------------------- */
typedef void (*NPF_EVENT_CBFUNC) ( 	NPF_USERCONTEXT,
					NPF_HANDLE,
					NPF_EVENT,
					NPF_DATA	);



/* ---------------------------- FE Management ----------------------------- */
typedef void (*NPF_FE_CBFUNC) (	NPF_USERCONTEXT,
				NPF_CORRELATOR,
				NPF_VERBOSITY,
				NPF_HANDLE,
				NPF_FE_CBTYPE,
				NPF_DATA	);


/* --------------------------- Interface Management ----------------------- */
typedef void (*NPF_IF_CBFUNC) (	NPF_USERCONTEXT,
				NPF_CORRELATOR,
				NPF_VERBOSITY,
				NPF_HANDLE,
				NPF_IF_CBTYPE,
				NPF_DATA	);


/* ------------------------------ ICMP Management ------------------------- */
typedef void (*NPF_ICMP_CBFUNC) (	NPF_USERCONTEXT,
					NPF_CORRELATOR,
					NPF_VERBOSITY,
					NPF_HANDLE,
					NPF_ICMP_CBTYPE,
					NPF_DATA	);

/* ------------------------------ PORT Management ------------------------- */

typedef void (*NPF_PORT_CBFUNC) (	NPF_USERCONTEXT,
					NPF_CORRELATOR,
					NPF_VERBOSITY,
					NPF_HANDLE,
					NPF_PORT_CBTYPE,
					NPF_DATA	);

/* --------------------------- IP Router Management ----------------------- */
typedef void (*NPF_IPR_CBFUNC) (	NPF_USERCONTEXT,
					NPF_CORRELATOR,
					NPF_VERBOSITY,
					NPF_HANDLE,
					NPF_IPR_CBTYPE,
					NPF_DATA	);


#ifdef __cplusplus
}
#endif

#endif /* _NPF_CM_TYPES_H_ */
