
/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2002 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/npf_api/npf_common.h,v $
 * $Revision: 1.6 $
 * $Date: 2003/11/17 16:58:00 $
 *
 * Description:  This file defines the common NPF defines which all modules
 * use.
 *
****************************************************************************/

#ifndef _NPF_COMMON_H_
#define _NPF_COMMON_H_

#ifdef __cplusplus
extern "C" {
#endif
        
#include "npf.h"

/*
* IPv4 Interface attributes, used in the union within
* NPF_IfGeneric_t.
*/

/*
 * IPv4 address prefix structure (Defined globally)
 */
typedef struct {
	NPF_IPv4Address_t IPv4Addr;       /* IPv4 address */
	NPF_uint8_t       IPv4NetPlen;    /* Prefix length in bits (1-32) */
} NPF_IPv4NetAddr_t;

typedef NPF_uint32_t NPF_IPv4UnicastPrefixTableHandle_t;

/*
 * IPv6 Address Flags
 */
typedef NPF_uint8_t NPF_IPv6AddrFlags_t;

/*
 * IPv6 address prefix structure (Defined globally)
 */
typedef struct {
    NPF_IPv6Address_t   IPv6Addr;
    NPF_uint8_t         IPv6NetPlen;
    NPF_IPv6AddrFlags_t IPv6Flags;
} NPF_IPv6NetAddr_t;

/*
 * IPv6 unicast zone structure
 */
typedef NPF_uint32_t NPF_IPv6UCZoneTable_t[3];

/*
 * IPv6 Interface attributes
 */
typedef NPF_uint32_t NPF_IPv6PrefixTableHandle_t;

/*
 * IPv6 in IPv4 Tunnel types
 */
typedef enum {
    NPF_IF_V6INV4_AUTOMATIC = 1,
    NPF_IF_V6INV4_CONFIGURED =2,
    NPF_IF_V6INV4_6TO4 = 3
} NPF_IPv6inv4TunnelType_t;

#include "npf_if.h"

#ifdef _VXWORKS_
#include <unistd.h>
#else
#include <stdint.h>
#endif
#include "pil.h"

#define    PTR_UNDEFINED    0xFFFFFFFF


typedef enum {
        NPF_SUCCESS,
        NPF_FAIL,
        NPF_OSAS_FAIL,
        NPF_ALREADY_INIT,
        NPF_NOT_INITIALIZED,
        NPF_INVALID_PARAMETERS,
        NPF_INVALID_HANDLE,
        NPF_OUT_OF_MEMORY,
        NPF_DUPLICATE_CONTEXT,
        NPF_NO_INTERNAL_PORT,
        NPF_COMPONENT_UNINITIALIZED,
        NPF_COMPONENT_REINITIALIZED,
        NPF_NS_PATH_OVER_LENGTH,
        NPF_NS_NODE_IN_USE,
        NPF_NS_NODE_EXISTS,
        NPF_NS_NODE_NOTFOUND,
        NPF_CALLBACK_FAIL,
        NPF_IS_OUT_OF_HANDLES,
        NPF_IS_NOSUCH_INTF,
        NPF_IS_NOSUCH_FLOW,
        NPF_IS_NOSUCH_FILTER,
        NPF_IS_NOSUCH_FS,
        NPF_IS_NOFLOWSPEC,
        NPF_IS_FS_ALREADY_ATTACHED,
        NPF_IS_TSPECTYPE_UNSUPPORTED,
        NPF_IS_RSPECTYPE_UNSUPPORTED,
        NPF_IS_UNKNOWN_SERV_TYPE,
        NPF_IS_UNKNOWN_CHAR_PARM,
        NPF_IS_PARM_OUT_OF_RANGE,
        NPF_IS_PARM_UNSPECIFIED,     /*Param value is not set*/
        NPF_IS_FAILED_CAPS_VALIDATION,
        NPF_FE_CONNECTION_FAIL,
        NPF_IF_INVALID_PORT_ASSOCIATION,
        NPF_IPV6TM_TUNNEL_NOTFOUND,
        NPF_IPV6TM_TUNNEL_ALREADY_ENABLED    } NPF_RET;
        

typedef enum {    
        NPF_SYSTEM,
        NPF_IPROUTER,
        NPF_ROUTETABLE,
        NPF_PREFIXTABLE,
        NPF_NEXTHOPTABLE,
        NPF_ARPTABLE,
        NPF_FTNTABLE,
        NPF_ILMTABLE,
        NPF_FE,
        NPF_PORT,
        NPF_INTERFACE,
        NPF_INTCONFIG,
	NPF_ATM_LINK,
	NPF_ATM_CONNECTION,
	NPF_ATM_VIRTUAL_DEVICE,
        NPF_MAX_DATATYPE } NPF_DATATYPE;

typedef enum {    
        NPF_ACK,
        NPF_NOACK,
        NPF_FAILACK
} NPF_VERBOSITY;

typedef uint32_t NPF_HANDLE;        // the handle to the data,
typedef uint32_t NPF_CBHANDLE;
typedef void * NPF_USERCONTEXT;
typedef void * NPF_CORRELATOR;        // correlator


/*****************************************************************************
*                                                                            *
*  ---------------------------- Internal Usage ----------------------------  *
*                                                                            *
*****************************************************************************/ 

typedef void *(NPF_GENERAL_CBFUNC);    // pdk cast specific cbfunc to general
typedef void * NPF_DATA;        // pointer to whatever data inside npf
                    // need cast to (struct X *)
                    // some compiler don't like (void *)


#define MACADDRLEN 6

#define OID uint32_t

#define FAMILY_NETADDR_IPV4 1
#define FAMILY_NETADDR_IPV6 2

#ifdef _VXWORKS_
typedef long long          int64_t;
typedef unsigned long long uint64_t;
#endif

typedef uint32_t IPA;
typedef uint8_t MACA[MACADDRLEN];
typedef float float32_t; 

/*-----------------------------------
  Changes for IPv4 1.1
  -----------------------------------*/
typedef void * NPF_event_t;

#if 0 // Already defined in npf_if.h
#ifndef IF_HANDLE
#define IF_HANDLE
typedef NPF_uint32_t NPF_IfHandle_t;
#endif
#endif /* 0 */ // Already defined in npf_if.h



typedef enum {
    NPF_NO_MEDIA_TYPE = 1,
    NPF_MAC_ADDRESS = 2,
    NPF_ATM_VC = 3
}NPF_MediaType_t;

typedef NPF_uint32_t NPF_VccAddress_t;

typedef struct {
    NPF_MediaType_t type;
    union /* @!union-switch:type! */
    {
        NPF_MAC_Address_t MAC_address; /* @!union-case:NPF_MAC_ADDRESS! */
        NPF_VccAddress_t  ATM_VC; /* @!union-case:NPF_ATM_VC! */
    }u;
}NPF_MediaAddressEntry_t;


#define NPF_FOUNDATIONS_MAX_ERR          (NPF_FOUNDATIONS_BASE_ERR + 98)

#define NPF_IPV4_BASE_ERR  (NPF_FOUNDATIONS_MAX_ERR + 1)
#define NPF_IPV4_MAX_ERR   (NPF_IPV4_BASE_ERR + 99)

#define NPF_INTERFACES_BASE_ERR (NPF_IPV4_MAX_ERR + 1)
#define NPF_INTERFACES_MAX_ERR (NPF_INTERFACES_BASE_ERR + 99)

#define NPF_ATM_BASE_ERR (NPF_INTERFACES_MAX_ERR + 1)
#define NPF_ATM_MAX_ERR (NPF_ATM_BASE_ERR + 99)

#define NPF_MPLS_BASE_ERR (NPF_ATM_MAX_ERR + 1)
#define NPF_MPLS_MAX_ERR  (NPF_MPLS_BASE_ERR + 99)

#define NPF_IS_BASE_ERR   (NPF_MPLS_MAX_ERR + 1)
#define NPF_IS_MAX_ERR    (NPF_IS_BASE_ERR + 99)

#define NPF_DS_BASE_ERR   (NPF_IS_MAX_ERR + 1)
#define NPF_DS_MAX_ERR    (NPF_DS_BASE_ERR + 99)

#define NPF_GTP_BASE_ERR  (NPF_DS_MAX_ERR + 1)
#define NPF_GTP_MAX_ERR   (NPF_GTP_BASE_ERR + 99)


int npf_pdk_init();
int npf_pdk_start();
int npf_pdk_shutdown();

typedef uint32_t ipv4addr;
typedef	char ipv6addr[16];



#ifdef __cplusplus
}
#endif

#endif /* _NPF_COMMON_H_ */
