/****************************************************************************
 *             INTEL CORPORATION PROPRIETARY INFORMATION
 *  This software is supplied under the terms of a license agreement or
 *  nondisclosure agreement with Intel Corporation and may not be copied
 *  or disclosed except in accordance with the terms of that agreement.
 *      Copyright (c) 2001 Intel Corporation. All Rights Reserved.
 *
 * $Source: /cvs/constellations/ixa_3.0/src/cp_pdk/npf_api/npf_ds.h,v $
 * $Revision: 1.5 $
 * $Date: 2003/11/17 16:58:00 $
 *
 * Description:
 * This file defines all of the types that are used by the DiffServ API.
 *
 ****************************************************************************/

#ifndef _NPF_DS_H_
#define _NPF_DS_H_

#include "npf.h"
#include "npf_common.h"
#include "npf_if.h"
#include "npf_ipv4_types.h"

#ifdef _INCLUDE_MPLS
#include "npf_mpls_types.h"
#endif

typedef 	NPF_uint32_t  	NPF_DS_DpeHandle_t;
typedef 	NPF_uint64_t  	NPF_DS_Counter_t;
typedef 	NPF_uint32_t  	NPF_DS_ErrorType_t;

typedef enum
{
    NPF_DS_INGRESS = 1,
    NPF_DS_EGRESS = 2
}NPF_DS_DpeDirection_t;

typedef enum
{
    NPF_DS_CLASSIFIER = 1,
    NPF_DS_METER = 2,
    NPF_DS_MARKER = 3,
    NPF_DS_DROPPER = 4,
    NPF_DS_QUEUE = 5,
    NPF_DS_SCHEDULER = 6,
    NPF_DS_SHAPER = 7	
}NPF_DS_DpeType_t;

typedef struct 
{
    NPF_IPv4Address_t	dstAddr;	/* IPv4 destination address */
    NPF_uint8_t		dstNetpLen;	/* IPv4 destination Prefix length in bits (1-32) */
    NPF_IPv4Address_t   srcAddr;	/* IPv4 source address */
    NPF_uint8_t		srcNetpLen;	/* IPv4 source Prefix length in bits (1-32) */
    NPF_int32_t         dscp;		/* DiffServ codepoint stored in Type of Service field of an IPv4 header */
    NPF_int32_t         protocol;	/* Layer 4 protocol number (e.g. TCP or UDP) */
    NPF_uint32_t        dstL4Port; 	/* Destination port number of a layer 4 protocol (e.g. UDP or TCP)*/
    NPF_uint32_t        srcL4Port;	/* Source port number of a layer 4 protocol (e.g. UDP or TCP)*/
}NPF_DS_IPV4_Filter_t;

typedef struct 
{
    NPF_IPv6Address_t	dstAddr;	/* IPv6 destination address */
    NPF_uint8_t		dstNetpLen;	/* IPv6 destination Prefix length in bits (1-128) */
    NPF_IPv6Address_t   srcAddr;	/* IPv6 source address */
    NPF_uint8_t		srcNetpLen;	/* IPv6 source Prefix length in bits (1-128) */
    NPF_int32_t         fid;		/* Flow label in the IPV6 header */
    NPF_int32_t         dscp;		/* DiffServ codepoint stored in Type of Service field of an IPv6 header */
    NPF_int32_t         protocol;	/* Layer 4 protocol number (e.g. TCP or UDP) */
    NPF_uint32_t        dstL4Port; 	/* Destination port number of a layer 4 protocol (e.g. UDP or TCP)*/
    NPF_uint32_t        srcL4Port;	/* Source port number of a layer 4 protocol (e.g. UDP or TCP)*/
}NPF_DS_IPV6_Filter_t;

typedef enum
{
    NPF_DS_MPLS_FEC = 1,
    NPF_DS_MPLS_LABEL = 2			
}NPF_DS_MPLS_FilterType_t;

typedef struct 
{
    NPF_DS_MPLS_FilterType_t	type;		/* MPLS Filter Type */
#ifdef _INCLUDE_MPLS
    union
    {
        NPF_MPLS_Fec_t          fec;
        NPF_MPLS_Label_t        label;
    } filter;
#endif
}NPF_DS_MPLS_Filter_t;

typedef struct 
{
     NPF_uint8_t          dscp;  /* DSCP value */
}NPF_DS_DSCP_Filter_t;

typedef enum
{
    NPF_DS_IPV4_FILTER = 1,
    NPF_DS_IPV6_FILTER = 2,
    NPF_DS_MPLS_FILTER = 3,
    NPF_DS_DSCP_FILTER  = 4
}NPF_DS_ClassifierType_t;

typedef union
{
    NPF_DS_IPV4_Filter_t	filter_IPV4;	/* IPV4 Filter */
    NPF_DS_IPV6_Filter_t	filter_IPV6; 	/* IPV6 Filter */
    NPF_DS_MPLS_Filter_t	filter_MPLS;	/* MPLS Filter */
    NPF_DS_DSCP_Filter_t	filter_DSCP;	/* DSCP Filter */
}NPF_DS_Filter_t;

typedef struct
{
    NPF_DS_ClassifierType_t	type;		/* Classifier type */
    NPF_uint32_t             	precedence; 	/* ColorId, Packet drop precedence level: green, yellow or red color */
    NPF_DS_Filter_t          	filterEntry; 	/* filter Entry */
}NPF_DS_Classifier_t;

typedef enum {
    NPF_DS_SIMPLE_TB = 1,  			/* Simple Token Bucket Metering */
    NPF_DS_AVG_RATE = 2,   			/* Average rate metering */
    NPF_DS_SRTCM_BLIND = 3,			/*Single Rate Tricolor metering in color blind  */
    NPF_DS_SRTCM_AWARE = 4,			/* SRTCM color aware mode */
    NPF_DS_TRTCM_BLIND = 5,			/* Two Rate Tricolor Metering in color blind */
    NPF_DS_TRTCM_AWARE = 6,			/* TRTCM color aware mode */ 
    NPF_DS_TSW_TCM = 7    			/*Time Sliding Window TCM */
} NPF_DS_MeterType_t;

typedef enum {
    NPF_DS_MARKER_DSCP = 1,
    NPF_DS_MARKER_MPLS = 2
} NPF_DS_MarkerType_t;

typedef struct
{
    NPF_uint8_t	greenDscp;	/* DSCP value to mark for green packets */
    NPF_uint8_t	yellowDscp;	/* DSCP value to mark for yellow packets */
    NPF_uint8_t	redDscp;	/* DSCP value to mark for red packets */
}NPF_DS_DSCP_Marker_t;

typedef struct
{
#ifdef _INCLUDE_MPLS
    NPF_MPLS_Label_t	tos_label; /*top of stack label including Exp Bits */
#endif
}NPF_DS_MPLS_Marker_t;

typedef union
{
    	NPF_DS_DSCP_Marker_t	dscp;
    	//NPF_DS_MPLS_Marker_t	label;
}NPF_DS_MarkerEntry_t;


typedef struct
{
    NPF_DS_MarkerType_t	  	type;
    NPF_boolean_t	  	statsRequired;	/* boolean value; if true, statistics are gathered */
    NPF_DS_MarkerEntry_t  	markerEntry;    /* Marker Entry */ 
}NPF_DS_Marker_t;

typedef struct
{
    NPF_uint32_t	cir;		  	/* Committed Information Rate in kbits/sec */
    NPF_uint32_t	pir;		  	/* Peak Inforamtion Rate in kbit/sec, used in NPF_DS_TRTCM_AWARE type */
    NPF_uint32_t	cbs;		  	/* Committed Burst Size in bytes */
    NPF_uint32_t	pbs;		  	/* Peak Burst Size in bytes, used in NPF_DS_TRTCM_AWARE type */
    NPF_uint32_t	ebs;		  	/* Excess Burst Size in bytes */
    NPF_DS_DpeType_t    nextGreenDpe; 		/* The next DPE should be attached for conformance (Green) flow */
    NPF_DS_DpeType_t    nextYellowDpe; 		/* The next DPE should be attached for yellow packets */
    NPF_DS_DpeType_t    nextRedDpe;   		/* The next DPE should be attached for red packets */
}NPF_DS_MeterEntry_t;

typedef struct
{
    NPF_DS_MeterType_t	  type;
    NPF_boolean_t	  statsRequired;	/* boolean value; if true, statistics are gathered */
    NPF_DS_MeterEntry_t   meterEntry;     	/* one Meter Entry */ 
}NPF_DS_Meter_t;

typedef enum
{
    NPF_DS_RED = 1,	/* Random Early Detection */
    NPF_DS_WRED = 2	/* Weighted Random Early Detection */
}NPF_DS_DropperType_t;

typedef struct
{
    NPF_uint8_t		minThreshold;  	/* minimum queue threshold in packets  (0-255) */
			  		/* no packets are dropped, if the average */
    				   	/* queue length is below this threshold */
    NPF_uint8_t		maxThreshold; 	/* maximum queue threshold in packets (0-255) */
				  	/* packets are always dropped, if the average */
				 	/* queue length exceeds this threshold */
    NPF_uint16_t	maxProbability; /* dropping probability at maxThreshold, expressed as a fraction with denominator of 65535 */
}NPF_DS_RED_Instance_t;


typedef struct
{
    NPF_DS_RED_Instance_t	green;	/* RED params for green packets */
    NPF_DS_RED_Instance_t	yellow;	/* RED params for yellow packets */
    NPF_DS_RED_Instance_t	red;		/* RED params for red packets */
}NPF_DS_DropperEntry_t;

typedef struct
{
    NPF_uint32_t		serviceRate;	 /* Queue service rate in packets/sec */ 	
    NPF_uint8_t			weight;		 /*  EWMA factor, negative power of 2  */ 
    NPF_DS_DropperType_t	type;		/*Dropper Alogorithm */
    NPF_boolean_t	      	statsRequired;	/* boolean value; if true, statistics are gathered */
    NPF_DS_DropperEntry_t     	dropperEntry;  /* one dropper Entry */
}NPF_DS_Dropper_t;

typedef enum
{
    NPF_DS_SPQ =  1,  	/* Strict Priority Queuing */
    NPF_DS_CBQ =  2,	/* Class Based Queuing */
    NPF_DS_CBWFQ =3	/* Class Based Weighted Fair Queuing */
}NPF_DS_QueueType_t;

typedef enum
{
    NPF_DS_CLASS_EF =  1,  		/* EF class */
    NPF_DS_CLASS_AF1 =  2,		/* AF class-1 */
    NPF_DS_CLASS_AF2 =  3,		/* AF class-2 */
    NPF_DS_CLASS_AF3 =  4,		/* AF class-3 */
    NPF_DS_CLASS_AF4 =  5,		/* AF class-4 */
    NPF_DS_CLASS_BF =   6		/* BF class */
}NPF_DS_ClassType_t;

typedef struct
{
    NPF_uint32_t	priority;	/* Priority of input to the associated Queue/scheduler */
    NPF_uint32_t        abs;	/* Minimum absolute rate, in kbps that downstream queue/scheduler should allocate*/
    NPF_uint32_t        rel;	/* Minimum rate that downstream scheduler should allocate to this queue */
} NPF_DS_MinRate_t;


typedef struct
{
	NPF_uint32_t       level; /*indicates which level of multi rate shaper being given its parameters */
    	NPF_uint32_t       abs;	  /* Maximum rate, in kbps that downstream scheduler should allocate to this queue */
	NPF_uint32_t       rel;	/* The maximum rate that a downstream scheduler 
					element should allocate to this queue, relative 
					to the maximum rate of the interface as 
					reported by ifSpeed in units of 1/1000 of 1.*/
	NPF_uint32_t       threshold;	/* The number of bytes of queue depth at 
					which the rate of a multi-rate scheduler will 
					increase to next output rate */
} NPF_DS_SlMaxRate_t;

typedef struct
{
    NPF_uint32_t          maxRateCount;	/* No of multi-rate entries */
    NPF_DS_SlMaxRate_t    *maxRateList;   /* Array of multirate entries */
} NPF_DS_MlMaxRate_t;

typedef enum {
    NPF_DS_SL_MAXRATE = 1,			/* Single rate */
    NPF_DS_ML_MAXRATE = 2			/* Multi rate */
} NPF_DS_MaxRateType_t;

typedef struct
{
    NPF_DS_MaxRateType_t        type;
    union 
    {
        NPF_DS_SlMaxRate_t      slMaxRate;
        NPF_DS_MlMaxRate_t      mlMaxRate;
    } maxRate;
} NPF_DS_MaxRate_t;

typedef struct
{
    NPF_DS_QueueType_t        type;
    NPF_DS_ClassType_t        classId;
    NPF_DS_MinRate_t	      minRate;
    NPF_DS_MaxRate_t	      maxRate; 
} NPF_DS_Q_t;

typedef enum
{
    NPF_DS_SCHEDULER_SP =  1,		/* Strict Priority Scheduler */
    NPF_DS_SCHEDULER_RR =  2,		/* Round Robin Scheduler*/
    NPF_DS_SCHEDULER_WRR =  3,		/* Weighted Round Robin Scheduler*/
    NPF_DS_SCHEDULER_WFQ =  3,		/* Weighted Fair Queuing scheduler*/
    NPF_DS_SCHEDULER_DRR =  4		/* Deficit Round Robin Scheduler*/
}NPF_DS_SchedMethod_t;

typedef struct
{
    NPF_DS_SchedMethod_t      type;
    NPF_DS_MinRate_t	      minRate;
    NPF_DS_MaxRate_t	      maxRate; 
} NPF_DS_Scheduler_t;

typedef struct
{
    NPF_DS_DpeType_t		type;
    union
    {
	NPF_DS_Classifier_t	classifier;
	NPF_DS_Marker_t		marker;
	NPF_DS_Meter_t		meter;
	NPF_DS_Dropper_t	dropper;
	NPF_DS_Q_t		queue;
	NPF_DS_Scheduler_t	scheduler;
    }u;
}NPF_DS_Dpe_t;

typedef struct
{
    NPF_DS_DpeHandle_t    dpeHandle ; /* handle of DPE part of a flow */ 
    NPF_DS_DpeHandle_t    prevDpeHandle ; /* handle of DPE part of a flow */ 
} NPF_DS_DpeSetHandle_t;

typedef struct
{
    NPF_DS_Counter_t		defaultPackects;
    NPF_DS_Counter_t		defaultBytes;
    NPF_DS_Counter_t		totalBytes;
    NPF_DS_Counter_t		totalPackets;
}NPF_DS_ClassifierStats_t;

typedef struct {
    NPF_DS_Counter_t		greenByteCount;
    NPF_DS_Counter_t		greenPacketCount;
    NPF_DS_Counter_t		yellowByteCount;
    NPF_DS_Counter_t		yellowPacketCount;
    NPF_DS_Counter_t		redByteCount;
    NPF_DS_Counter_t		redPacketCount;
} NPF_DS_ColorAwareMeterStats_t;

typedef struct {
    NPF_DS_Counter_t		confByteCount;
    NPF_DS_Counter_t		confPacketCount;
    NPF_DS_Counter_t		nonConfByteCount;
    NPF_DS_Counter_t		nonConfPacketCount;
} NPF_DS_SimpleMeterStats_t;

typedef struct
{
    NPF_DS_MeterType_t		type;		/* Meter type */
    union
    {
	NPF_DS_ColorAwareMeterStats_t	colorAwareStats; /*meant for all color aware metering algorithms */ 
	NPF_DS_SimpleMeterStats_t	simpleStats;
    }u;
}NPF_DS_MeterStats_t;

typedef struct
{
    NPF_DS_Counter_t		bytesMarked;
    NPF_DS_Counter_t		packetsMarked;
}NPF_DS_MarkerStats_t;

typedef struct
{
    NPF_DS_Counter_t	greenByteDropped;		/* Green bytes dropped */
    NPF_DS_Counter_t	greenPacketDropped;	/* Green packets dropped */
    NPF_DS_Counter_t	yellowByteDropped;	/* Yellow bytes dropped */
    NPF_DS_Counter_t	yellowPacketDropped;	/* Yellow packets dropped */
    NPF_DS_Counter_t	redByteDropped;		/* Red bytes dropped */
    NPF_DS_Counter_t	redPacketDropped;		/* Red packets dropped */
}NPF_DS_ColorAwareDropperStats_t;

typedef struct
{
    NPF_DS_Counter_t		bytesConsumed;		
    NPF_DS_Counter_t		packetsConsumed;		
    NPF_DS_Counter_t		bytesDropped;		
    NPF_DS_Counter_t		packetsDropped;		
}NPF_DS_SimpleDropperStats_t;

typedef struct
{
    NPF_DS_MeterType_t		type;				/* Meter type */
    union
    {
	NPF_DS_ColorAwareDropperStats_t 	colorAwareDropperStats;
	NPF_DS_SimpleDropperStats_t		simpleDropperStats;
    }u;
}NPF_DS_DropperStats_t;

typedef enum {
    NPF_DS_DPE_CREATE = 1,
    NPF_DS_DPE_MODIFY = 2,
    NPF_DS_DPE_DELETE = 3,
    NPF_DS_DPE_QUERY = 4,
    NPF_DS_DPE_ADD_ASSOC = 5,
    NPF_DS_DPE_DELETE_ASSOC = 6,
    NPF_DS_DPE_STATISTICS = 7
} NPF_DS_CallbackType_t;

typedef struct
{
    union 
    {
	NPF_DS_ClassifierStats_t	classfierStats;
	NPF_DS_MeterStats_t		meterStats;
	NPF_DS_MarkerStats_t		markerStats;
	NPF_DS_DropperStats_t		dropperStats;
	//NPF_DS_QueueStats_t		queueStats;
	//NPF_DS_SchedulerStats_t		schedulerStats;
    } u;
} NPF_DS_StatsResp_t;

typedef struct
{
    NPF_DS_ErrorType_t   error;
    NPF_DS_DpeHandle_t   dpeHandle;
    NPF_DS_DpeHandle_t   prevDpeHandle;
    union 
    {
       NPF_DS_Dpe_t	  dpe;
       NPF_DS_StatsResp_t statsResp;
    } u;
} NPF_DS_AsyncResponse_t;

typedef struct
{
    NPF_DS_CallbackType_t	type;
    NPF_boolean_t		allOK;
    NPF_uint32_t		nResp;
    NPF_DS_AsyncResponse_t	*resp;
} NPF_DS_CallbackData_t;

#define NPF_DS_BASE_ERROR			700
#define NPF_DS_MAX_ERROR			NPF_DS_BASE_ERROR + 99

#define NPF_DS_E_INVALID_DPE			((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 0)
#define NPF_DS_E_NO_SUCH_DPE			((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 1)
#define NPF_DS_E_INVALID_INTERFACE		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 2)
#define	NPF_DS_E_INVALID_DPE_PARAMETER 		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 3)
#define	NPF_DS_E_DPE_NOT_SUPPORTED 		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 4)
#define NPF_DS_E_INVALID_DPE_HANDLE		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 5)
#define NPF_DS_E_NO_STATS_AVAILABLE		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 6)
#define NPF_DS_E_NOT_SUPPORTED			((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 7)
#define	NPF_DS_E_UNKNOWN			((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 8)
#define	NPF_DS_E_RESOURCE			((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 9)
#define	NPF_DS_E_INVALID_DPE_PATH		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 10)
#define	NPF_DS_E_INTERNAL			((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 11)
#define	NPF_DS_E_DPE_NOT_CERATED 		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 12)
#define	NPF_DS_E_DPE_NOT_DELETED 		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 13)
#define	NPF_DS_E_DPE_ADD_ASSOC_FAILED		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 14)
#define	NPF_DS_E_DPE_DEL_ASSOC_FAILED		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 15)
#define	NPF_DS_E_DPE_INVALID_STATE		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 16)
#define	NPF_DS_E_DPE_IN_USE			((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 17)

#define	NPF_DS_E_INVALID_CLASSIFIER_TYPE	((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 20)
#define	NPF_DS_E_INVALID_CLASSIFIER_PARAMETER	((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 21)
#define	NPF_DS_E_INVALID_PRECEDENCE		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 22)
#define	NPF_DS_E_INVALID_FILTER			((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 23)
#define	NPF_DS_E_INVALID_SOURCE_ADDRESS		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 24)
#define	NPF_DS_E_INVALID_DEST_ADDRESS		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 25)
#define	NPF_DS_E_INVALID_SOURCE_PREFIX		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 26)
#define	NPF_DS_E_INVALID_DEST_PREFIX		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 27)
#define	NPF_DS_E_INVALID_SOURCE_PORT		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 28)
#define	NPF_DS_E_INVALID_DEST_PORT		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 29)
#define	NPF_DS_E_INVALID_DSCP			((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 30)
#define	NPF_DS_E_INVALID_PROTOCOL		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 31)
#define	NPF_DS_E_INVALID_FLOW_ID		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 32)
#define	NPF_DS_E_INVALID_CLASS_ID		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 33)
#define	NPF_DS_E_INVALID_FEC			((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 34)
#define	NPF_DS_E_INVALID_LABEL			((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 35)
#define	NPF_DS_E_INVALID_FILETR_PARAMETER	((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 36)

#define	NPF_DS_E_INVALID_METER_TYPE		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 40)
#define	NPF_DS_E_INVALID_METER_PARAMETER	((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 41)
#define	NPF_DS_E_INVALID_CIR			((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 42)
#define	NPF_DS_E_INVALID_CBS			((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 43)
#define	NPF_DS_E_INVALID_EBS			((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 44)

#define	NPF_DS_E_INVALID_MARKER_TYPE		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 50)
#define	NPF_DS_E_INVALID_MARKER_PARAMETER	((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 51)

#define	NPF_DS_E_INVALID_DROPPER_TYPE		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 60)
#define	NPF_DS_E_INVALID_DROPPER_PARAMETER	((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 61)
#define	NPF_DS_E_INVALID_MIN_THRESHOLD		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 62)
#define	NPF_DS_E_INVALID_MAX_THRESHOLD		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 63)
#define	NPF_DS_E_INVALID_PROBABILITY		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 64)
#define	NPF_DS_E_INVALID_SERVICE_RATE		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 65)
#define	NPF_DS_E_INVALID_WEIGHT			((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 66)

#define	NPF_DS_E_INVALID_Q_TYPE			((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 70)
#define	NPF_DS_E_INVALID_Q_PARAMETER		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 71)
#define	NPF_DS_E_INVALID_CLASS_TYPE		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 72)
#define	NPF_DS_E_INVALID_PRIORITY		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 73)
#define	NPF_DS_E_INVALID_ABSOLUTE_RATE		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 74)
#define	NPF_DS_E_INVALID_RATE			((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 75)
#define	NPF_DS_E_INVALID_RATE_TYPE		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 76)
#define	NPF_DS_E_INVALID_THRESHOLD		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 77)
#define	NPF_DS_E_INVALID_LEVEL			((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 78)

#define	NPF_DS_E_INVALID_SCHEDULER_TYPE		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 80)
#define	NPF_DS_E_INVALID_SCHEDULER_PARAMETER	((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 81)
#define	NPF_DS_E_INVALID_MIN_RATE		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 82)
#define	NPF_DS_E_INVALID_MAX_RATE		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 83)

#define	NPF_DS_E_INVALID_SHAPER_TYPE		((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 90)
#define	NPF_DS_E_INVALID_SHAPER_PARAMETER	((NPF_DS_ErrorType_t)NPF_DS_BASE_ERROR + 91)


typedef void (*NPF_DS_CallbackFunc_t) (
            NPF_IN      NPF_userContext_t         context,
            NPF_IN      NPF_correlator_t          correlator,
            NPF_IN      NPF_DS_CallbackData_t     *callbackData);

NPF_error_t NPF_DS_Register (
            NPF_IN 	NPF_userContext_t      context,
            NPF_IN 	NPF_DS_CallbackFunc_t  callback,
            NPF_OUT 	NPF_callbackHandle_t   *cbhandle);

NPF_error_t NPF_DS_Deregister(
    NPF_IN NPF_callbackHandle_t cbhandle);

NPF_error_t NPF_DS_DpeCreate(
       NPF_IN    NPF_callbackHandle_t      cbhandle,
       NPF_IN    NPF_correlator_t          correlator,
       NPF_IN    NPF_errorReporting_t      errorReporting,
       NPF_IN    NPF_IfHandle_t            interface,
       NPF_IN    NPF_DS_DpeDirection_t     dir,
       NPF_IN    NPF_uint32_t              noOfEntries, 
       NPF_IN    NPF_DS_Dpe_t       	   *dpe);

NPF_error_t NPF_DS_DpeAddAssociation(
       NPF_IN    NPF_callbackHandle_t      cbhandle,
       NPF_IN    NPF_correlator_t          correlator,
       NPF_IN    NPF_errorReporting_t      errorReporting,
       NPF_IN    NPF_uint32_t              noOfEntries,
       NPF_IN    NPF_DS_DpeSetHandle_t	   *dpeSetHandle);

NPF_error_t NPF_DS_DpeDelAssociation(
       NPF_IN    NPF_callbackHandle_t      cbhandle,
       NPF_IN    NPF_correlator_t          correlator,
       NPF_IN    NPF_errorReporting_t      errorReporting,
       NPF_IN    NPF_uint32_t              noOfEntries,
       NPF_IN    NPF_DS_DpeSetHandle_t     *dpeSetHandle);

NPF_error_t NPF_DS_DpeModify(
       NPF_IN    NPF_callbackHandle_t      cbhandle,
       NPF_IN    NPF_correlator_t          correlator,
       NPF_IN    NPF_errorReporting_t      errorReporting,
       NPF_IN    NPF_uint32_t              noOfEntries,
       NPF_IN    NPF_DS_DpeHandle_t        *dpeHandle,
       NPF_IN    NPF_DS_Dpe_t      	   *dpe);

NPF_error_t NPF_DS_DpeDelete(
       NPF_IN    NPF_callbackHandle_t      cbhandle,
       NPF_IN    NPF_correlator_t          correlator,
       NPF_IN    NPF_errorReporting_t      errorReporting,
       NPF_IN    NPF_uint32_t          	   nHandles,
       NPF_IN    NPF_DS_DpeHandle_t	   *dpeHandle);

NPF_error_t NPF_DS_DpeQueryAttributes(
       NPF_IN    NPF_callbackHandle_t      cbhandle,
       NPF_IN    NPF_correlator_t          correlator,
       NPF_IN    NPF_errorReporting_t      errorReporting,
       NPF_IN    NPF_uint32_t          	   nHandles,
       NPF_IN    NPF_DS_DpeHandle_t	   *dpeHandle);

NPF_error_t NPF_DS_DpeQueryStats(
       NPF_IN    NPF_callbackHandle_t      cbhandle,
       NPF_IN    NPF_correlator_t          correlator,
       NPF_IN    NPF_errorReporting_t      errorReporting,
       NPF_IN    NPF_uint32_t          	   nHandles,
       NPF_IN    NPF_DS_DpeHandle_t        *dpeHandle);

#endif
