/*****************************************************************************
*                  INTEL CORPORATION PROPRIETARY INFORMATION
*  This software is supplied under the terms of a license agreement or
*  nondisclosure agreement with Intel Corporation and may not be copied
*  or disclosed except in accordance with the terms of that agreement.
*  Copyright (c) 2001 Intel Corporation. All Rights Reserved.
*****************************************************************************/


/*****************************************************************************
 * Description: 
 * This header file defines functions for the NP Forum 
 * IPv4 Unicast Forwarding API
******************************************************************************/

#ifndef __NPF_IPV4_API_H
#define __NPF_IPV4_API_H


#ifdef __cplusplus
extern "C" {
#endif

#include "npf_ipv4_types.h"



/*----------------------------------------------------------------- 
*
* Function Call Prototypes
*
*-----------------------------------------------------------------*/

/* TODO : Define it in User application and here, make it extern */
typedef void (*NPF_IPv4UC_CallbackFunc_t) (
	NPF_IN NPF_userContext_t userContext,
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_IPv4UC_CallbackData_t data);

/* TODO : Define it in User application and here, make it extern */
typedef void (*NPF_IPv4UC_EventCallFunc_t) ( 
		NPF_IN NPF_userContext_t userContext,
		NPF_IN NPF_IPv4UC_EventArray_t data);

extern NPF_error_t NPF_IPv4UC_Register(
	NPF_IN NPF_userContext_t userContext, 
	NPF_IN NPF_IPv4UC_CallbackFunc_t callbackFunc,
	NPF_OUT NPF_callbackHandle_t *callbackHandle);


extern NPF_error_t NPF_IPv4UC_Deregister(
	NPF_IN NPF_callbackHandle_t callbackHandle); 


extern NPF_error_t NPF_IPv4UC_EventRegister(
	NPF_IN NPF_userContext_t userContext,
	NPF_IN NPF_IPv4UC_EventCallFunc_t eventCallFunc,
	NPF_OUT NPF_callbackHandle_t *eventCallHandle); 


extern NPF_error_t NPF_IPv4UC_EventDeregister(
	NPF_IN NPF_callbackHandle_t eventCallHandle);

extern NPF_IPv4UC_SupportedMode_t NPF_IPv4UC_GetSupportedModes (void); 

extern NPF_IPv4UC_PreferredMode_t NPF_IPv4UC_GetPreferredMode(void);


extern NPF_error_t NPF_IPv4UC_PrefixTableHandleCreate(
	NPF_IN NPF_callbackHandle_t callbackHandle, 
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting);


extern NPF_error_t NPF_IPv4UC_PrefixTableHandleDelete(
	NPF_IN NPF_callbackHandle_t callbackHandle, 
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting,
	NPF_IN NPF_IPv4UC_PrefixTableHandle_t tableHandle);


extern NPF_error_t NPF_IPv4UC_PrefixEntryAdd( 
	NPF_IN NPF_callbackHandle_t callbackHandle,
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting,
	NPF_IN NPF_IPv4UC_PrefixTableHandle_t tableHandle,
	NPF_IN NPF_uint32_t numEntries, 
	NPF_IN NPF_IPv4UC_Prefix_t *prefixArray,
	NPF_IN NPF_uint32_t *nextHopIdArray);


extern NPF_error_t NPF_IPv4UC_PrefixEntryDelete(
	NPF_IN NPF_callbackHandle_t callbackHandle, 
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting,
	NPF_IN NPF_IPv4UC_PrefixTableHandle_t tableHandle,
	NPF_IN NPF_uint32_t numEntries,
	NPF_IN NPF_IPv4UC_Prefix_t *prefixArray); 


extern NPF_error_t NPF_IPv4UC_PrefixTableFlush(
	NPF_IN NPF_callbackHandle_t callbackHandle,
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting, 
	NPF_IN NPF_IPv4UC_PrefixTableHandle_t tableHandle);


extern NPF_error_t NPF_IPv4UC_PrefixTableAttributeQuery(
	NPF_IN NPF_callbackHandle_t callbackHandle,
	NPF_IN NPF_correlator_t correlator, 
	NPF_IN NPF_errorReporting_t errorReporting,
	NPF_IN NPF_IPv4UC_PrefixTableHandle_t tableHandle);


extern NPF_error_t NPF_IPv4UC_PrefixEntryQuery(
	NPF_IN NPF_callbackHandle_t callbackHandle, 
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting,
	NPF_IN NPF_IPv4UC_PrefixTableHandle_t tableHandle,
	NPF_IN NPF_uint32_t numEntries,
	NPF_IN NPF_IPv4UC_Prefix_t *prefixArray); 


extern NPF_error_t NPF_IPv4UC_PrefixNextHopTableBind(
	NPF_IN NPF_callbackHandle_t callbackHandle,
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting, 
	NPF_IN NPF_IPv4UC_PrefixTableHandle_t prefixTableHandle,
	NPF_IN NPF_IPv4UC_NextHopTableHandle_t nextHopTableHandle);


extern NPF_error_t NPF_IPv4UC_NextHopTableHandleCreate( 
	NPF_IN NPF_callbackHandle_t callbackHandle,
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting);


extern NPF_error_t NPF_IPv4UC_NextHopTableHandleDelete( 
	NPF_IN NPF_callbackHandle_t callbackHandle,
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting,
	NPF_IN NPF_IPv4UC_NextHopTableHandle_t tableHandle);


extern NPF_error_t NPF_IPv4UC_NextHopEntryAdd(
	NPF_IN NPF_callbackHandle_t callbackHandle,
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting,
	NPF_IN NPF_IPv4UC_NextHopTableHandle_t tableHandle, 
	NPF_IN NPF_uint32_t numEntries,
	NPF_IN NPF_uint32_t *nextHopIdArray,
	NPF_IN NPF_IPv4UC_NextHopArray_t *nextHopArrays);


extern NPF_error_t NPF_IPv4UC_NextHopEntryDelete( 
	NPF_IN NPF_callbackHandle_t callbackHandle,
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting,
	NPF_IN NPF_IPv4UC_NextHopTableHandle_t tableHandle,
	NPF_IN NPF_uint32_t numEntries, 
	NPF_IN NPF_uint32_t *nextHopIdArray);


extern NPF_error_t NPF_IPv4UC_NextHopTableFlush(
	NPF_IN NPF_callbackHandle_t callbackHandle,
	NPF_IN NPF_correlator_t correlator, 
	NPF_IN NPF_errorReporting_t errorReporting,
	NPF_IN NPF_IPv4UC_NextHopTableHandle_t tableHandle);


extern NPF_error_t NPF_IPv4UC_NextHopTableAttributeQuery(
	NPF_IN NPF_callbackHandle_t callbackHandle, 
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting,
	NPF_IN NPF_IPv4UC_NextHopTableHandle_t tableHandle);


extern NPF_error_t NPF_IPv4UC_NextHopEntryQuery( 
	NPF_IN NPF_callbackHandle_t callbackHandle,
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting,
	NPF_IN NPF_IPv4UC_NextHopTableHandle_t tableHandle,
	NPF_IN NPF_uint32_t numEntries, 
	NPF_IN NPF_uint32_t *nextHopIdArray);


extern NPF_error_t NPF_IPv4UC_FibTableHandleCreate(
	NPF_IN NPF_callbackHandle_t callbackHandle, 
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting);


extern NPF_error_t NPF_IPv4UC_FibTableHandleDelete(
	NPF_IN NPF_callbackHandle_t callbackHandle, 
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting,
	NPF_IN NPF_IPv4UC_FibTableHandle_t tableHandle);


extern NPF_error_t NPF_IPv4UC_FibEntryAdd( 
	NPF_IN NPF_callbackHandle_t callbackHandle,
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting,
	NPF_IN NPF_IPv4UC_FibTableHandle_t tableHandle,
	NPF_IN NPF_uint32_t numEntries, 
	NPF_IN NPF_IPv4UC_Prefix_t *prefixArray,
	NPF_IN NPF_IPv4UC_NextHopArray_t *nextHopArrays);


extern NPF_error_t NPF_IPv4UC_FibEntryDelete(
	NPF_IN NPF_callbackHandle_t callbackHandle, 
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting,
	NPF_IN NPF_IPv4UC_FibTableHandle_t tableHandle,
	NPF_IN NPF_uint32_t numEntries,
	NPF_IN NPF_IPv4UC_Prefix_t *prefixArray); 


extern NPF_error_t NPF_IPv4UC_FibTableFlush(
	NPF_IN NPF_callbackHandle_t callbackHandle,
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting, 
	NPF_IN NPF_IPv4UC_FibTableHandle_t tableHandle);


extern NPF_error_t NPF_IPv4UC_FibAttributeQuery(
	NPF_IN NPF_callbackHandle_t callbackHandle,
	NPF_IN NPF_correlator_t correlator, 
	NPF_IN NPF_errorReporting_t errorReporting,
	NPF_IN NPF_IPv4UC_FibTableHandle_t tableHandle);


extern NPF_error_t NPF_IPv4UC_FibEntryQuery(
	NPF_IN NPF_callbackHandle_t callbackHandle, 
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting,
	NPF_IN NPF_IPv4UC_FibTableHandle_t tableHandle,
	NPF_IN NPF_uint32_t numEntries,
	NPF_IN NPF_IPv4UC_Prefix_t *prefixArray); 


extern NPF_error_t NPF_IPv4UC_AddResTableHandleCreate(
	NPF_IN NPF_callbackHandle_t callbackHandle,
	NPF_IN NPF_correlator_t correlator, 
	NPF_IN NPF_errorReporting_t errorReporting);


extern NPF_error_t NPF_IPv4UC_AddResTableHandleDelete(
	NPF_IN NPF_callbackHandle_t callbackHandle,
	NPF_IN NPF_correlator_t correlator, 
	NPF_IN NPF_errorReporting_t errorReporting,
	NPF_IN NPF_IPv4UC_AddResTableHandle_t tableHandle);


extern NPF_error_t NPF_IPv4UC_AddResEntryAdd(
	NPF_IN NPF_callbackHandle_t callbackHandle, 
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting,
	NPF_IN NPF_IPv4UC_AddResTableHandle_t tableHandle,
	NPF_IN NPF_uint32_t numEntries,
	NPF_IN NPF_IPv4UC_AddResEntry_t *entryArray); 


extern NPF_error_t NPF_IPv4UC_AddResEntryDelete(
	NPF_IN NPF_callbackHandle_t callbackHandle,
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting, 
	NPF_IN NPF_IPv4UC_AddResTableHandle_t tableHandle,
	NPF_IN NPF_uint32_t numEntries,
	NPF_IN NPF_IPv4UC_AddResKey_t *entryArray);


extern NPF_error_t NPF_IPv4UC_AddResTableFlush( 
	NPF_IN NPF_callbackHandle_t callbackHandle,
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting,
	NPF_IN NPF_IPv4UC_AddResTableHandle_t tableHandle);


extern NPF_error_t NPF_IPv4UC_AddResTableAttributeQuery(
	NPF_IN NPF_callbackHandle_t callbackHandle,
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting,
	NPF_IN NPF_IPv4UC_AddResTableHandle_t tableHandle); 


extern NPF_error_t NPF_IPv4UC_AddResEntryQuery(
	NPF_IN NPF_callbackHandle_t callbackHandle,
	NPF_IN NPF_correlator_t correlator,
	NPF_IN NPF_errorReporting_t errorReporting, 
	NPF_IN NPF_IPv4UC_AddResTableHandle_t tableHandle,
	NPF_IN NPF_uint32_t numEntries,
	NPF_IN NPF_IPv4UC_AddResKey_t *entryArray);

#ifdef __cplusplus
}
#endif
#endif /* __NPF_IPV4_API_H */ 
