/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation.All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 5
 *
 * = FILENAME
 *      cci_defs.h
 *
 * = DESCRIPTION
 *      Tuneable definitions in the core component infrastructure.
 *
 * = AUTHOR
 *       Roland Arajs
 *       roland.arajs@intel.com
 *
 * = CHANGE HISTORY
 *      4/11/2002 11:24:41 AM - creation time
 *
 * ============================================================================
 * $Id: cci_defs.h,v 1.13 2003/03/04 19:30:25 ageorges Exp $
 */

#if !defined(__CCI_DEFS_H__)
#define __CCI_DEFS_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

#include "ix_hardware_configuration.h"

/**
 * The following value sets the default frequency at which execution engines 
 * will check for events and shutdown flag. The check interval is in units of 
 * milliseconds. If an event is set that is of shorter duration than this 
 * interval, the engine semaphore's timeout will use the event timeout instead.
 * Therfore, events can be handled even if this value is set to wait forever. 
 * However, if the value is set to wait forever, it might not be possible to
 * shut down the execution engines. 
 * Do not set this value to zero.
 */
#define IX_CCI_EVENT_CHECK_INTERVAL     1000
    

#define IX_CCI_MEMORY_CHANNEL           IX_DRAM_CHANNELS_NUMBER - 1
#define IX_CCI_SHARED_MEMORY_TYPE       IX_MEMORY_TYPE_DRAM




#define IX_CCI_MAX_INPUT_IDS            IX_COMM_LOCAL_ID_NUMBER - \
                                        IX_COMM_LAST_UBLOCK_ID - 1


/* Maximum number of engines on an ingress or egress board.
 * This value does not need to be a power of 2. It should not be 
 * greater than 64 in the current implementation
 */
#define IX_CCI_MAX_ENGINES              64

/* Maximum number of policies on an ingress or egress board.
 * This value does not need to be a power of 2. It should not be 
 * greater than 512 in the current implementation
 */
#define IX_CCI_MAX_POLICIES             256



/* Maximum number of CCs in an engine. This value does not need to be a 
 * power of 2. It should not be greater than 256 in the current implementation
 */
#define IX_CCI_MAX_CCS                  64


/* Maximum number of events in an engine. This value does not need to be a 
 * power of 2. It should not be greater than 2M in the current implementation
 */
#define IX_CCI_MAX_EVENTS               256


/**
 * Number of default engines associated with non-engine threads. The code presently
 * only allows for 1 default engine.
 */
#define IX_CCI_NUM_DEFAULT_ENGINES  1



/**
 * The number of handlers the executable engine will run before passing control
 * to the next execution engine
 */
#if !defined(IX_CCI_EXE_RUN_GRANULARITY)
#define IX_CCI_EXE_RUN_GRANULARITY      384
#endif



/**
 * Represents the default number of credits (messages to be processed) by a CC
 * message handler
 */
#define IX_CCI_MESSAGE_RUN_GRANULARITY          128

/**
 * Represents the default number of credits (packets to be processed) by a CC
 * packet handler
 */
#define IX_CCI_PACKET_RUN_GRANULARITY           128


/**
 * Represents the default number of credits (data tokens to be processed) by an
 * execution engine message policy
 */
#define IX_CCI_MESSAGE_POLICY_RUN_GRANULARITY   256

/**
 * Represents the default number of credits (data tokens to be processed) by an
 * execution engine packet policy
 */
#define IX_CCI_PACKET_POLICY_RUN_GRANULARITY    256


/**
 * The following constant determines the initial capacity of the policy container.
 * Policy containers are growable, so if most execution engines have few handlers
 * associated with them, this number can be set to a low value to save memory
 * usage. Each entry in a container consumes 
 */

#define IX_CCI_INIT_WRR_POLICY_SIZE     1
#define IX_CCI_INIT_RR_POLICY_SIZE      1
#define IX_CCI_INIT_SP_POLICY_SIZE      2
#define IX_CCI_WRR_POLICY_GROW_INCR     1
#define IX_CCI_RR_POLICY_GROW_INCR      1
#define IX_CCI_SP_POLICY_GROW_INCR      1


#define g_szCciSharedPtrMsgTkq  "cci_SharedPtrMsgTkq"
#define g_szCciSharedPtrPktTkq  "cci_SharedPtrPktTkq"
#define g_szCciSharedPtrPolicies "cci_SharedPtrPolicies"
#define g_szCciSharedPtrEngines  "cci_SharedPtrEngines"
#define g_szCciSharedPtrShutdownFlags "cci_SharedPtrShutdownFlags"
#define g_szCciSharedPtrShmemSema "cci_SharedPtrShmemSema"

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__CCI_DEFS_H__) */
