/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation.All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 5
 *
 * = FILENAME
 *      internal_cci_error.h
 *
 * = DESCRIPTION
 *      Error macros specifically created for the core-component framework
 *
 * = AUTHOR
 *       Roland Arajs
 *       roland.arajs@intel.com
 *
 * = CHANGE HISTORY
 *      3/11/2002 11:04:12 AM - creation time
 *
 * ============================================================================
 * $Id: cci_error.h,v 1.7 2002/09/18 17:15:37 lbmiller Exp $
 */

#if !defined(CCI_ERROR_H__)
#define CCI_ERROR_H__

#include "cci/cci_types.h"


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */



/* Comment out following line to take out programming logic checks */
#if defined(_DEBUG)
#define _IX_CCI_DEBUG_  
#endif


/**
 * TYPENAME: ix_cci_err
 * 
 * DESCRIPTION: A central location for the definitions of all the error codes 
 *              generated in the core-component infrastructure.
 *
 */
typedef enum ix_e_cci_err
{
    IX_CCI_ERR_FIRST = IX_ERROR_MAKE_GROUP(CC_INFRASTRUCTURE),
    IX_CCI_ERR_ASSERTION = IX_CCI_ERR_FIRST,
    IX_CCI_ERR_SEND_FAIL,     /* send-message/packet fn failed. CCI deletes buffer */
    IX_CCI_ERR_INIT_ERR_LOG,              /* Could not init error logging */
    IX_CCI_ERR_RM_INIT,                 /* Could not initialize RM */
    IX_CCI_ERR_SHARED_MEM_ALLOC,        /* Could not allocate shared memory */
    IX_CCI_ERR_OS_MEM_ALLOC,            /* Could not allocate O/S memory */
    IX_CCI_ERR_PROPERTY_CREATE,        /* Could not create registry property */
    IX_CCI_ERR_PROPERTY_SET,           /* Could not set registry property */
    IX_CCI_ERR_GLOB_SEMA_INIT,             /* Could not init global sema */
    IX_CCI_ERR_ENG_THREAD_SPAWN,        /* Could not spawn an engine thread */
    IX_CCI_ERR_ENG_INIT,                /* Could not initialize engine construct */
    IX_CCI_ERR_ENG_THREAD_ID,           /* Could not get thread ID of engine */
    IX_CCI_ERR_ENG_INIT_SEMA_PROP_READ, /* Could not property from registry */
    IX_CCI_ERR_ENG_INIT_SEMA_INIT,      /* Could not initialize engine semaphore */
    IX_CCI_ERR_ENG_INIT_ENGINE_PROP_READ, /* Could not property from registry */
    IX_CCI_ERR_ENG_PROP_CLOSE,          /* Could not close property from registry */
    IX_CCI_ERR_ENG_NO_HANDLERS,         /* No handlers added to execution engine */
    IX_CCI_TOKEN_PROCESSOR_CONTAINER_FULL, /* Non-growable container is full */
    IX_CCI_ERR_TK_NOTIFY_INVALID_HANDLE,    /* Wrong or invalid handle type */
    IX_CCI_ERR_TK_NOTIFY_FINI_STILL_PENDING, /* Token still pending when fini called */
    IX_CCI_ERR_TKP_CONTAINER_MEM_ALLOC, /* Unable to create container due to memory */
    IX_CCI_ERR_CC_ADD_HANDLER_INVALID_HANDLE, /* Invalid CC handle in add handler fn */
    IX_CCI_ERR_EVENT_HANDLER,           /* User-defined event handler returned error */
    IX_CCI_ERR_EVENT_HANDLER_GET_TIME,  /* Couldn't get time for event handler */
    IX_CCI_ERR_INVALID_POLICY_TYPE, /* Invalid policy type passed to create fn */
    IX_CCI_ERR_POLICY_CREATE,   /* Error creating policy */
    IX_CCI_ERR_EVENT_NOT_FOUND, /* Could not find event on active or expired list */
    IX_CCI_ERR_MESSAGE_GET,     /* Error getting message from comm ID */
    IX_CCI_ERR_CC_INIT_INCOMPLETE,   /* CC or engine recvd token before init completed */
    IX_CCI_ERR_MSG_HANDLER,     /* User-registered message handler failed */
    IX_CCI_ERR_DISABLE_MSG_HANDLER,  /* Error disabling message handler in RM */
    IX_CCI_ERR_DISABLE_PKT_HANDLER,  /* Error removing packet handler in RM */
    IX_CCI_ERR_PACKET_GET,      /* Error getting packet from comm ID */
    IX_CCI_ERR_PKT_HANDLER,     /* User-registered packet handler failed */
    IX_CCI_ERR_MSG_PEEK,        /* Error getting number of entries on message queue */
    IX_CCI_ERR_PKT_PEEK,        /* Error getting number of entries on packet queue */
    IX_CCI_ERR_MSG_SET_MODE,    /* Unable to set mode for msg comm ID in RM */
    IX_CCI_ERR_PKT_SET_MODE,    /* Unable to set mode for pkt comm ID in RM */
    IX_CCI_ERROR_CODES_LAST
} ix_cci_err;


/**
 * TYPENAME: ix_ferror_func
 * 
 * DESCRIPTION: Prototype for fatal-error handler that can be registered in the CCI
 *              with function ix_cci_register_fatal_error_handler().
 *
 */
typedef void (* ix_ferror_func)( 
                                 ix_error arg_FatalException, 
                                 ix_exe_handle arg_hEngine, 
                                 void* arg_pContext
                               );


/**
 * NAME: ix_cci_register_fatal_error_handler
 *
 * DESCRIPTION: Allows a control application to register a fatal-error handler.
 *              The CCI will call this handler if an event/message/packet 
 *              handler returns a fatal error. This allows the control 
 *              application to disable any microcode associated with this engine
 *              and shut the engine or whole CCI system down.
 * 
 * @Param:  - IN    arg_Handler: Fatal error handler that will be invoked whenever
 *                      a fatal error occurs in the CCI.
 * @Param:  - IN    arg_pContext: Context to be returned when handler is invoked.
 *
 * @Return: 0 if successful or a valid ix_error token on failure.
 */
IX_EXPORT_FUNCTION 
ix_error ix_cci_register_fatal_error_handler(
                                              ix_ferror_func arg_Handler,
                                              void* arg_pContext
                                            );


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__INTERNAL_CCI_ERROR_H__) */
