/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation.All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 5
 *
 * = FILENAME
 *      internal_cci_error.h
 *
 * = DESCRIPTION
 *      Error macros specifically created for the core-component framework
 *
 * = AUTHOR
 *       Roland Arajs
 *       roland.arajs@intel.com
 *
 * = CHANGE HISTORY
 *      3/11/2002 11:04:12 AM - creation time
 *
 * ============================================================================
 * $Id: internal_cci_error.h,v 1.19 2002/10/23 17:40:25 rlarajs Exp $
 */

#if !defined(__INTERNAL_CCI_ERROR_H__)
#define __INTERNAL_CCI_ERROR_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

#include "ix_ossl.h"
#include "ix_rm.h"
#include "ix_error.h"
#include "ix_macros.h"
#include "cci/cci_error.h"

/* Comment out following line to take out programming logic checks */
#if defined(_DEBUG) || defined(IX_DEBUG)
#define _IX_CCI_DEBUG_  
#endif

#if defined(IX_ERROR_EXTENSIONS)
#define _CC_ERR_TYPE_
#endif

#if defined(_CC_ERR_TYPE_)
#include "ix_cc_error.h"
#endif

#if defined(_CCI_ERROR_PATH_TEST_)

extern int g_vPrimaryErrorCount;
extern int g_vPrimaryErrorTriggerNum;
extern int g_vPrimaryErrorTriggerCode;
extern int g_vSecondaryErrorCount;
extern int g_vSecondaryErrorTriggerNum;


/**
 * Following functions are used to set up the test CCI error system so that when
 * a CCI test is run, an error will be simulated in specific code locations
 */
IX_EXPORT_FUNCTION 
void ix_cci_reset_error_counts( void );

IX_EXPORT_FUNCTION
void ix_cci_set_perror_trig( int arg_TriggerCode, int arg_TriggerCount );

IX_EXPORT_FUNCTION                                 
void ix_cci_set_serror_trig( int arg_TriggerCount );

                                 

/**
 * Following macros are for monitoring RM and OSSL memory. The report and fix
 * macros also effectively act as fini functions.
 */

IX_EXPORT_FUNCTION
void report_and_fix_leaks( void );

IX_EXPORT_FUNCTION
void fix_leaks( void );

IX_EXPORT_FUNCTION
void ix_init_mem_monitor( void );


void* ix_cci_test_osmalloc(
                            ix_ossl_size_t arg_Size,
                            ix_uint32 arg_Line,
                            const char* arg_pFileName
                          );

ix_error ix_cci_test_rmmalloc( 
                               ix_memory_type arg_MemType, 
                               ix_uint32 arg_MemChannel, 
                               ix_uint32 arg_Size, 
                               void** arg_pMemoryAddr,
                               ix_uint32 arg_Line,
                               const char* arg_pFileName
                             );

void ix_cci_test_osfree( void* arg_pMem );

ix_error ix_cci_test_rmfree( 
                             void* arg_pMemory 
                           );

IX_EXPORT_FUNCTION
ix_error ix_cci_test_sem_init(
                               ix_ossl_sem_state arg_StartValue, 
                               ix_ossl_sem_t* arg_pSid,
                               ix_uint32 arg_Line,
                               const char* arg_pFileName
                             );

IX_EXPORT_FUNCTION
ix_error ix_cci_test_sem_fini(
                               ix_ossl_sem_t arg_Sid
                             );

IX_EXPORT_FUNCTION
ix_error ix_cci_test_buffer_alloc( 
                                   ix_buffer_free_list_handle arg_hFreeList, 
                                   ix_buffer_handle* arg_pBufferHandle, 
                                   ix_uint32 arg_Line,
                                   const char* arg_pFileName
                                 );

IX_EXPORT_FUNCTION
ix_error ix_cci_test_buffer_free_chain( 
                                        ix_buffer_handle arg_hBuffer 
                                      );

ix_error ix_cci_test_property_create(
                                      ix_configuration_property_handle arg_hParentProperty, 
                                      const char* arg_pPropertyName,
                                      ix_uint32 arg_PropertyOptions,
                                      ix_configuration_property_handle* arg_pProperty, 
                                      ix_uint32 arg_Line,
                                      const char* arg_pFileName
                                    );

ix_error ix_cci_test_property_delete(
                                      ix_configuration_property_handle arg_hProperty
                                    );

/**
 * Use the following macros instead of directly calling RM/OSSL memory-allocation/free
 * functions so that memory can be monitored in test mode
 */

#define IX_OSSL_MALLOC( arg_Size )  \
    ix_cci_test_osmalloc( arg_Size, __LINE__, __FILE__ )

#define IX_OSSL_FREE( arg_pMem ) \
    ix_cci_test_osfree( arg_pMem )

#define IX_RM_MEM_ALLOC( arg_MemType, arg_MemChannel, arg_Size, arg_pMemoryAddr ) \
    ix_cci_test_rmmalloc( arg_MemType, arg_MemChannel, arg_Size, \
                          arg_pMemoryAddr, __LINE__, __FILE__ )

#define IX_RM_MEM_FREE( arg_pMem )  \
    ix_cci_test_rmfree( arg_pMem )


#define IX_OSSL_SEM_INIT( arg_StartVal, arg_pSema ) \
    ix_cci_test_sem_init( arg_StartVal, arg_pSema, __LINE__, __FILE__ )

#define IX_OSSL_SEM_FINI( arg_Sema ) \
    ix_cci_test_sem_fini( arg_Sema )

#define IX_RM_BUFFER_ALLOC( __arg_hFreeList, __arg_pBufferHandle )      \
    ix_cci_test_buffer_alloc( __arg_hFreeList,                           \
                              __arg_pBufferHandle, __LINE__, __FILE__ )

#define IX_RM_BUFFER_FREE_CHAIN( __arg_hBuf ) \
    ix_cci_test_buffer_free_chain( __arg_hBuf )

/**
 * Following macros have same name as normal CCI error macros but have a test
 * implementation
 */


/**
 * MACRO NAME: IX_CCI_INIT_ERROR_LOGGING
 *
 * DESCRIPTION: Initializes error logging. If there is an error, causes the 
 *              calling function to return error value.
 *
 */
#define IX_CCI_INIT_ERROR_LOGGING()                     \
        ix_ossl_message_log_init()



/**
 * MACRO NAME: IX_CCI_ERROR_DUMP
 *
 * DESCRIPTION: Dumps information about error code, group and severity
 *              Note: ix_cc_err.h should not be using file streams such
 *              as stderr. Change cc error code to use ix_ossl_message_log().
 *
 * @Param:  - IN error  Error of type ix_error to dump
 */
              
#define IX_CCI_ERROR_DUMP( error ) (error)



           

/**
 * MACRO NAME: IX_CCI_IF_ERR_DUMP
 *
 * DESCRIPTION: Acts just like standard CCI error macro.
 *
 */
              
#define IX_CCI_IF_ERR_DUMP( error )  (void)0;



/**
 * MACRO NAME: IX_CCI_ASSERT
 *
 * DESCRIPTION: In debug mode evaluates a condition. If the condition
 *              is not met, dumps the failed condition, file, and line
 *              number, just as with a c assertion. In addition, returns
 *              an ix_error code of IX_CCI_ERROR_ASSERTION.
 *
 * @Param:  - IN arg_Conditon   Expression to evaluate as a condition
 *
 * @Return: 0 if successful or IX_CCI_ERROR_ASSERTION on failure. 
 */
                     
#define IX_CCI_ASSERT(						\
                       arg_Condition		\
                     ) (void)0;



/**
 * MACRO NAME: IX_CCI_ERR_NEW
 *
 * DESCRIPTION: 
 *
 * @Param:  - IN    __arg_ErrorCodeIn: error code
 */
#define IX_CCI_ERR_NEW(                             \
                        __arg_ErrorCodeIn           \
                      )                             \
IX_ERROR_NEW(							            \
	   	      __arg_ErrorCodeIn,   			        \
			  IX_ERROR_LEVEL_LOCAL				    \
			)


/**
 * MACRO NAME: IX_CCI_IXERR_GOTO
 *
 * DESCRIPTION: Evaluates an error condition. If the 
 *              the expression returns an error condition, this condition
 *              is stored in arg_ErrorCodeOut and the code jumps to label
 *              arg_GotoLabel.
 *
 * @Param:  - IN  arg_ErrInOut      ix_error value (not expression or condition)
 * @Param:  - IN  arg_GotoLabel     Where to branch on error
 */
#define IX_CCI_IXERR_GOTO(					                        \
				           arg_GotoLabel,	                        \
                           arg_ErrInOut	                            \
                         )					                        \
    {                                                               \
        if( arg_ErrInOut != IX_SUCCESS )                            \
        {                                                           \
            goto arg_GotoLabel;                                     \
        }                                                           \
    }

/**
 * MACRO NAME: IX_CCI_ERR_GOTO
 *
 * DESCRIPTION: Evaluates an expression or condition. If true (error),
 *              takes an error code, adds CCI group info, and error
 *              level as local. Then branches to arg_GotoLabel.
 *
 * @Param:  - IN arg_Condition       If false generates an error
 * @Param:  - IN arg_ErrorCodeIn     Error code of type ix_cci_err.
 * @Param:  - IN arg_GotoLabel       Where to branch on macro exit
 * @Param:  - OUT arg_ErrOut         Place to store error result
 */
#define IX_CCI_ERR_GOTO(                                                    \
                         arg_Condition,                                     \
                         arg_ErrorCodeIn,                                   \
                         arg_GotoLabel,                                     \
                         arg_ErrOut                                         \
                       )                                                    \
	{													                    \
        if( arg_Condition                                                   \
            || (arg_ErrorCodeIn == g_vPrimaryErrorTriggerCode               \
            && g_vPrimaryErrorTriggerNum == ++g_vPrimaryErrorCount) )       \
        {                                                                   \
            arg_ErrOut = IX_CCI_ERR_NEW( arg_ErrorCodeIn );		            \
		    goto arg_GotoLabel;								                \
        }                                                                   \
	} 

#define IX_CCI_DBG_IXERR_GOTO( arg_Err1, arg_Err2 ) (void)0;

/**
 * MACRO NAME: IX_CCI_ERR_NOCHECK_GOTO
 *
 * DESCRIPTION: Unconditionally takes an error code, adds CCI group info, 
 *              and error level as local. Then branches to arg_GotoLabel.
 *
 * @Param:  - IN arg_ErrorCodeIn     Error code of type ix_cci_err.
 * @Param:  - IN arg_GotoLabel       Where to branch on macro exit
 * @Param:  - OUT arg_ErrOut         Place to store error result
 */
#define IX_CCI_ERR_NOCHECK_GOTO(                                        \
                                 arg_ErrorCodeIn,                       \
                                 arg_GotoLabel,                         \
                                 arg_ErrOut                             \
                               )                                        \
	{													                \
        arg_ErrOut = IX_CCI_ERR_NEW( arg_ErrorCodeIn );		            \
		goto arg_GotoLabel;								                \
	} 



/**
 * MACRO NAME: IX_CCI_IXERR_RET
 *
 * DESCRIPTION: Examines the ix_error value. If non-zero returns.
 *              If in debug mode, an error message is also dumped
 *              by this macro
 *
 * @Param:  - IN arg_Err Error status of IXA SDK function
 */
#define IX_CCI_IXERR_RET( arg_Err )                                 \
    {                                                               \
        if( arg_Err != IX_SUCCESS )                                 \
        {                                                           \
            return arg_Err;                                         \
        }                                                           \
    } 

#define IX_CCI_DBG_IXERR_RET( arg_Err ) (void)arg_Err;


/**
 * Following macro stays the way it is because our test can generate an
 * error (warning or fatal) in a handler
 */

#define IX_CCI_IXERR_LOG_RET_IF_FATAL( arg_CcErr, arg_CciErrCode )      \
    {                                                                   \
        if( arg_CcErr != IX_SUCCESS                                     \
            || (arg_CciErrCode == g_vPrimaryErrorTriggerCode           \
            && g_vPrimaryErrorTriggerNum == ++g_vPrimaryErrorCount) )   \
        {                                                               \
            ix_uint32 ___e = IX_ERROR_GET_LEVEL( arg_CcErr );           \
            if(  ___e != IX_ERROR_LEVEL_WARNING )                       \
                return IX_ERROR_NEW( arg_CciErrCode, ___e );            \
            arg_CcErr = IX_SUCCESS;                                     \
        }                                                               \
    }

/**
 * MACRO NAME: IX_CCI_ERR_RET
 *
 * DESCRIPTION: Checks a value. If value is non-zero, creates an ix_error value
 *              To create the error value, the macro takes an error code arg
 *              adds CCI group info, and error level as local, then causes
 *              calling function to return with new error value
 *
 * @Param:  - IN value              value to be checked
 * @Param:  - IN arg_ErrorCodeIn    Error code of type ix_cci_err
 */
#define IX_CCI_ERR_RET(                                                     \
                        arg_InValue,                                        \
                        arg_ErrorCodeIn                                     \
                      )                                                     \
    {                                                                       \
        if( arg_InValue                                                     \
            || (arg_ErrorCodeIn == g_vPrimaryErrorTriggerCode               \
                && g_vPrimaryErrorTriggerNum == ++g_vPrimaryErrorCount) )   \
        {                                                                   \
            return IX_CCI_ERR_NEW( arg_ErrorCodeIn );                       \
        }                                                                   \
    } 




/**
 * MACRO NAME: IX_CCI_ERR_FWD_OR_DUMP
 *
 * DESCRIPTION: This macro is for handling secondary errors while performing
 *              cleanup/termination functions. If a primary error already
 *              exists, then any errors from the cleanup function are simply
 *              dumped. If there are no preceding errors, the error value from
 *              the cleanup function is copied to the primary error value.
 *              In release mode, the macro simply discards the secondary error
 *              if a primary error is asserted. The macro is compiled out for
 *              code-complexity testing.
 *
 * @Param:  - IN    arg_NewErrStat. An ix_error result from a cleanup fn
 * @Param:  - INOUT  arg_OldErrStat. Primary error status/
 *
 */
#define IX_CCI_ERR_FWD_OR_DUMP( arg_NewErrStat, arg_OldErrStat )            \
        arg_OldErrStat = (arg_OldErrStat) > 0								\
                         ? (arg_OldErrStat)                                 \
                         : ( (g_vSecondaryErrorTriggerNum                    \
                              == ++g_vSecondaryErrorCount)                  \
                            ? IX_CCI_ERR_ASSERTION                          \
                            : (arg_NewErrStat) );

/**
 * MACRO NAME: IX_CCI_ERR_DUMP
 *
 * DESCRIPTION: If an error is detected, dump it if in debug mode and not
 *              compiled out for code-complexity testing.
 *
 * @Param:  - IN    arg_Err. Error to be checked and if necessary dumped
 *
 */
#define IX_CCI_ERR_DUMP( arg_Ignored )    ((void)arg_Ignored)

#else

#define IX_OSSL_MALLOC( arg_Size )  \
    ix_ossl_malloc( arg_Size )

#define IX_OSSL_FREE( arg_pMem ) \
    ix_ossl_free( arg_pMem )

#define IX_RM_MEM_ALLOC( arg_MemType, arg_MemChannel, arg_Size, arg_pMemoryAddr ) \
    ix_rm_mem_alloc( arg_MemType, arg_MemChannel, arg_Size, arg_pMemoryAddr )

#define IX_RM_MEM_FREE( arg_pMem )  \
    ix_rm_mem_free( arg_pMem )


#define IX_OSSL_SEM_INIT( arg_StartVal, arg_pSema ) \
    ix_ossl_sem_init( arg_StartVal, arg_pSema )

#define IX_OSSL_SEM_FINI( arg_Sema ) \
    ix_ossl_sem_fini( arg_Sema )


#define IX_RM_BUFFER_ALLOC( __arg_hFreeList, __arg_pBufferHandle )      \
    ix_rm_buffer_alloc( __arg_hFreeList, __arg_pBufferHandle )

#define IX_RM_BUFFER_FREE_CHAIN( __arg_hBuf ) \
    ix_rm_buffer_free_chain( __arg_hBuf )




/**
 * MACRO NAME: IX_CCI_INIT_ERROR_LOGGING
 *
 * DESCRIPTION: Initializes error logging. If there is an error, causes the 
 *              calling function to return error value.
 *
 */
#define IX_CCI_INIT_ERROR_LOGGING()                     \
        ix_ossl_message_log_init()



/**
 * MACRO NAME: IX_CCI_ERROR_DUMP
 *
 * DESCRIPTION: Dumps information about error code, group and severity
 *              Note: ix_cc_err.h should not be using file streams such
 *              as stderr. Change cc error code to use ix_ossl_message_log().
 *
 * @Param:  - IN error  Error of type ix_error to dump
 */
              
#if defined(_IX_CCI_DEBUG_) && !defined(_CCT_EXCLUDE_) && !defined(_IX_CCI_QUIET_)
#define IX_CCI_ERROR_DUMP( error )							\
    ix_ossl_message_log( "ERROR: File %s\n"                 \
                         "       Line %d\n"                 \
                         "       Code 0x%lx, "              \
                         " Severity: 0x%lx\n",	            \
                         __FILE__,                          \
                         __LINE__,                          \
						 IX_ERROR_GET_CODE( error ),		\
						 IX_ERROR_GET_LEVEL( error )		\
					   )
#elif defined(_IX_CCI_DEBUG_) && defined(_CC_ERR_TYPE_) && !defined(_IX_CCI_QUIET_)
#define IX_CCI_ERROR_DUMP( error )							\
    ix_error_dump( stderr, (error) )
#else
#define IX_CCI_ERROR_DUMP( error ) (error)
#endif



/**
 * Note: Following macro is for internal use within this header only
 */
#if !defined(_CCT_EXCLUDE_) && !defined(_CC_ERR_TYPE_) && !defined(_IX_CCI_QUIET_)
#define IX_CC_ERROR_DUMP( arg_CcErr, arg_CciErrCode )       \
    ix_ossl_message_log( "HANDLER ERROR: File %s\n"         \
                         "       Line %d\n"                 \
                         "       CCI Error Type %s \n"      \
                         "       CC Error Code: 0x%lx\n",   \
                         __FILE__, __LINE__,                \
                         #arg_CciErrCode,                   \
                         arg_CcErr )
#elif defined(_CC_ERR_TYPE_) && !defined(_IX_CCI_QUIET_)
#define IX_CC_ERROR_DUMP( arg_CcErr, arg_CciErrCode, arg_Level )    \
    {                                                               \
        IX_ERROR_CHAIN( arg_CcErr, arg_CciErrCode,                  \
                        arg_Level,                                  \
                        (#arg_CciErrCode) );                        \
        ix_error_dump( stderr, (arg_CcErr) );                       \
    }
#elif defined(_CC_ERR_TYPE_)
#define IX_CC_ERROR_DUMP( arg_CcErr, arg_CciErrCode, arg_Level )    \
    ((void)0)
#else
#define IX_CC_ERROR_DUMP( arg_CcErr, arg_CciErrCode )       \
    ((void)0)
#endif
             

/**
 * MACRO NAME: IX_CCI_IF_ERR_DUMP
 *
 * DESCRIPTION: Dumps information about error code, group and severity
 *
 * @Param:  - IN error  Error of type ix_error to dump
 */
              
#if defined(_IX_CCI_DEBUG_) && !defined(_CCT_EXCLUDE_)
#define IX_CCI_IF_ERR_DUMP( error )         \
    {                                       \
        if( error != IX_SUCCESS )           \
            (void) IX_CCI_ERROR_DUMP( error );     \
    } 
#else
#define IX_CCI_IF_ERR_DUMP( error )  (void)0;
#endif



/**
 * MACRO NAME: IX_CCI_ASSERT
 *
 * DESCRIPTION: In debug mode evaluates a condition. If the condition
 *              is not met, dumps the failed condition, file, and line
 *              number, just as with a c assertion. In addition, returns
 *              an ix_error code of IX_CCI_ERROR_ASSERTION.
 *
 * @Param:  - IN arg_Conditon   Expression to evaluate as a condition
 *
 * @Return: 0 if successful or IX_CCI_ERROR_ASSERTION on failure. 
 */
                     
#if defined(_IX_CCI_DEBUG_) && !defined(_CCT_EXCLUDE_) && !defined(_CC_ERR_TYPE_) && !defined(_IX_CCI_QUIET_)
#define IX_CCI_ASSERT(												\
                       arg_Condition								\
                     )												\
	{																\
		if(!(arg_Condition))										\
		{															\
			ix_ossl_message_log( "ASSERT ERROR: %s\n%s line %d\n",	\
								 #arg_Condition,					\
								 __FILE__, __LINE__ );				\
			return IX_ERROR_NEW(									\
								  IX_CCI_ERR_ASSERTION,			    \
								  IX_ERROR_LEVEL_LOCAL				\
								);									\
		}															\
	} 
#elif defined(_IX_CCI_DEBUG_) && !defined(_CCT_EXCLUDE_) && !defined(_CC_ERR_TYPE_) && defined(_IX_CCI_QUIET_)
#define IX_CCI_ASSERT(												\
                       arg_Condition								\
                     )												\
	{																\
		if(!(arg_Condition))										\
		{															\
			return IX_ERROR_NEW(									\
								  IX_CCI_ERR_ASSERTION,			    \
								  IX_ERROR_LEVEL_LOCAL				\
								);									\
		}															\
	} 
#elif defined(_IX_CCI_DEBUG_) && defined(_CC_ERR_TYPE_) && !defined(_IX_CCI_QUIET_)
#define IX_CCI_ASSERT(												\
                       arg_Condition								\
                     )												\
	{																\
		if(!(arg_Condition))										           \
		{															           \
            ix_error ___e = ix_error_new( 0, IX_CCI_ERR_ASSERTION,             \
                                          IX_ERROR_LEVEL_LOCAL,                \
                                          IX_ERROR_LEAD_FMT("ASSERT ERROR")    \
                                          "\t%s", IX_ERROR_LEAD_ARGS,          \
                                          #arg_Condition );                    \
            IX_CCI_ERROR_DUMP( ___e );                                         \
            return ___e;                                                       \
		}															           \
	} 
#elif defined(_IX_CCI_DEBUG_) && defined(_CC_ERR_TYPE_) && defined(_IX_CCI_QUIET_)
#define IX_CCI_ASSERT(												    \
                       arg_Condition								    \
                     )												    \
	{																    \
		if(!(arg_Condition))										    \
		{															    \
            return ix_error_new( 0, IX_CCI_ERR_ASSERTION,               \
                                    IX_ERROR_LEVEL_LOCAL,               \
                                    IX_ERROR_LEAD_FMT("ASSERT ERROR")   \
                                    "\t%s", IX_ERROR_LEAD_ARGS,         \
                                    #arg_Condition );                   \
		}															    \
	} 
#else
/* No implementation */
#define IX_CCI_ASSERT(						\
                       arg_Condition		\
                     ) (void)0;
#endif  /* _IX_CCI_DEBUG && !_CCT_EXCLUDE_ */




/**
 * MACRO NAME: IX_CCI_ERR_NEW
 *
 * DESCRIPTION: 
 *
 * @Param:  - IN    __arg_ErrorCodeIn: error code
 */
#if defined(_IX_CCI_DEBUG_) 
#define IX_CCI_ERR_NEW(                                             \
                        __arg_ErrorCodeIn                           \
                      )                                             \
(                                                                   \
    (void)IX_CCI_ERROR_DUMP(                                        \
        IX_ERROR_NEW( __arg_ErrorCodeIn, IX_ERROR_LEVEL_LOCAL ) ),  \
    IX_ERROR_NEW(							                        \
	   	          __arg_ErrorCodeIn,   			                    \
			      IX_ERROR_LEVEL_LOCAL				                \
			    )                                                   \
)
#else
#define IX_CCI_ERR_NEW(                             \
                        __arg_ErrorCodeIn           \
                      )                             \
IX_ERROR_NEW(							            \
	   	      __arg_ErrorCodeIn,   			        \
			  IX_ERROR_LEVEL_LOCAL				    \
			)
#endif                      


/**
 * MACRO NAME: IX_CCI_IXERR_GOTO
 *
 * DESCRIPTION: Evaluates an error condition. If the 
 *              the expression returns an error condition, this condition
 *              is stored in arg_ErrorCodeOut and the code jumps to label
 *              arg_GotoLabel.
 *
 * @Param:  - IN  arg_ErrInOut      ix_error value (not expression or condition)
 * @Param:  - IN  arg_GotoLabel     Where to branch on error
 */
#if defined(_CCT_EXCLUDE_)
#define IX_CCI_IXERR_GOTO( arg_Ignored1, arg_Ignored2 ) (void)0;
#else
#define IX_CCI_IXERR_GOTO(					                        \
				           arg_GotoLabel,	                        \
                           arg_ErrInOut	                            \
                         )					                        \
    {                                                               \
	    if( arg_ErrInOut != IX_SUCCESS )	                        \
        {                                                           \
            (void) IX_CCI_ERROR_DUMP( arg_ErrInOut );               \
		    goto arg_GotoLabel;                                     \
        }                                                           \
    }
#endif

/**
 * MACRO NAME: IX_CCI_ERR_GOTO
 *
 * DESCRIPTION: Evaluates an expression or condition. If true (error),
 *              takes an error code, adds CCI group info, and error
 *              level as local. Then branches to arg_GotoLabel.
 *
 * @Param:  - IN arg_Condition       If false generates an error
 * @Param:  - IN arg_ErrorCodeIn     Error code of type ix_cci_err.
 * @Param:  - IN arg_GotoLabel       Where to branch on macro exit
 * @Param:  - OUT arg_ErrOut         Place to store error result
 */
#if defined(_CCT_EXCLUDE_)
#define IX_CCI_ERR_GOTO( ignored1, ignored2, ignored3, ignored4 )   (void)0;
#else
#define IX_CCI_ERR_GOTO(                                                    \
                         arg_Condition,                                     \
                         arg_ErrorCodeIn,                                   \
                         arg_GotoLabel,                                     \
                         arg_ErrOut                                         \
                       )                                                    \
	{													                    \
        if( (arg_Condition) )                                               \
        {                                                                   \
            arg_ErrOut = IX_CCI_ERR_NEW( arg_ErrorCodeIn );		            \
		    goto arg_GotoLabel;								                \
        }                                                                   \
	} 
#endif

#if defined(_IX_CCI_DEBUG_) 
#define IX_CCI_DBG_IXERR_GOTO( arg1, arg2 ) \
    IX_CCI_IXERR_GOTO( arg1, arg2 )
#else
#define IX_CCI_DBG_IXERR_GOTO( arg1, arg2 ) (void)0;
#endif

/**
 * MACRO NAME: IX_CCI_ERR_NOCHECK_GOTO
 *
 * DESCRIPTION: Unconditionally takes an error code, adds CCI group info, 
 *              and error level as local. Then branches to arg_GotoLabel.
 *
 * @Param:  - IN arg_ErrorCodeIn     Error code of type ix_cci_err.
 * @Param:  - IN arg_GotoLabel       Where to branch on macro exit
 * @Param:  - OUT arg_ErrOut         Place to store error result
 */
#define IX_CCI_ERR_NOCHECK_GOTO(                                        \
                                 arg_ErrorCodeIn,                       \
                                 arg_GotoLabel,                         \
                                 arg_ErrOut                             \
                               )                                        \
	{													                \
        arg_ErrOut = IX_CCI_ERR_NEW( arg_ErrorCodeIn );		            \
		goto arg_GotoLabel;								                \
	} 



/**
 * MACRO NAME: IX_CCI_IXERR_RET
 *
 * DESCRIPTION: Examines the ix_error value. If non-zero returns.
 *              If in debug mode, an error message is also dumped
 *              by this macro
 *
 * @Param:  - IN arg_Err Error status of IXA SDK function
 */
#if defined(_CCT_EXCLUDE_)
#define IX_CCI_IXERR_RET( arg_Err )     (void)arg_Err;
#else
#define IX_CCI_IXERR_RET( arg_Err )                         \
    {                                                       \
        if( arg_Err )                                       \
        {                                                   \
            (void) IX_CCI_ERROR_DUMP( arg_Err );            \
		    return arg_Err;                                 \
        }                                                   \
    } 
#endif

#if defined(_IX_CCI_DEBUG_) 
#define IX_CCI_DBG_IXERR_RET( arg_Err ) IX_CCI_IXERR_RET( (arg_Err) )
#else
#define IX_CCI_DBG_IXERR_RET( arg_Err ) (void)arg_Err;
#endif

#if defined(_CCT_EXCLUDE_)
#define IX_CCI_IXERR_LOG_RET_IF_FATAL( arg_CcErr, arg_CciErrCode )      \
            (void)arg_CcErr;
#elif defined(_CC_ERR_TYPE_)
#define IX_CCI_IXERR_LOG_RET_IF_FATAL( arg_CcErr, arg_CciErrCode )      \
    {                                                                   \
        if( arg_CcErr )                                                 \
        {                                                               \
            ix_uint32 ___e = IX_ERROR_GET_LEVEL( arg_CcErr );           \
            IX_CC_ERROR_DUMP( arg_CcErr, arg_CciErrCode, ___e );        \
            if(  ___e != IX_ERROR_LEVEL_WARNING )                       \
                return arg_CcErr;                                       \
            arg_CcErr = IX_SUCCESS;                                     \
        }                                                               \
    }
#else
#define IX_CCI_IXERR_LOG_RET_IF_FATAL( arg_CcErr, arg_CciErrCode )      \
    {                                                                   \
        if( arg_CcErr )                                                 \
        {                                                               \
            ix_uint32 ___e = IX_ERROR_GET_LEVEL( arg_CcErr );           \
            IX_CC_ERROR_DUMP( arg_CcErr, arg_CciErrCode );              \
            if(  ___e != IX_ERROR_LEVEL_WARNING )                       \
                return IX_ERROR_NEW( arg_CciErrCode, ___e );            \
            arg_CcErr = IX_SUCCESS;                                     \
        }                                                               \
    }
#endif

/**
 * MACRO NAME: IX_CCI_ERR_RET
 *
 * DESCRIPTION: Checks a value. If value is non-zero, creates an ix_error value
 *              To create the error value, the macro takes an error code arg
 *              adds CCI group info, and error level as local, then causes
 *              calling function to return with new error value
 *
 * @Param:  - IN value              value to be checked
 * @Param:  - IN arg_ErrorCodeIn    Error code of type ix_cci_err
 */
#if defined(_CCT_EXCLUDE_)
#define IX_CCI_ERR_RET( arg_Ignored1, arg_Ignored2 )    (void)0;
#else
#define IX_CCI_ERR_RET(                                                     \
                        arg_InValue,                                        \
                        arg_ErrorCodeIn                                     \
                      )                                                     \
    {                                                                       \
        if( (arg_InValue) )                                                 \
        {                                                                   \
            return IX_CCI_ERR_NEW( arg_ErrorCodeIn );                       \
        }                                                                   \
    } 
#endif




/**
 * MACRO NAME: IX_CCI_ERR_FWD_OR_DUMP
 *
 * DESCRIPTION: This macro is for handling secondary errors while performing
 *              cleanup/termination functions. If a primary error already
 *              exists, then any errors from the cleanup function are simply
 *              dumped. If there are no preceding errors, the error value from
 *              the cleanup function is copied to the primary error value.
 *              In release mode, the macro simply discards the secondary error
 *              if a primary error is asserted. The macro is compiled out for
 *              code-complexity testing.
 *
 * @Param:  - IN    arg_NewErrStat. An ix_error result from a cleanup fn
 * @Param:  - INOUT  arg_OldErrStat. Primary error status/
 *
 */
#if defined(_CCT_EXCLUDE_)
#define IX_CCI_ERR_FWD_OR_DUMP( arg_Ingored1, arg_Ignored2 )   (void)0;
#elif defined(_IX_CCI_DEBUG_)
#define IX_CCI_ERR_FWD_OR_DUMP( arg_NewErrStat, arg_OldErrStat )            \
        {                                                                   \
            arg_OldErrStat = (arg_OldErrStat) > 0                           \
                ? ( arg_NewErrStat                                          \
                    ? ((void)IX_CCI_ERROR_DUMP(arg_NewErrStat),             \
                      (arg_OldErrStat))                                     \
                    : (arg_OldErrStat) )                                    \
                : arg_NewErrStat;                                           \
        } 
#else
#define IX_CCI_ERR_FWD_OR_DUMP( arg_NewErrStat, arg_OldErrStat )            \
        arg_OldErrStat = (arg_OldErrStat) > 0								\
                         ? (arg_OldErrStat) : arg_NewErrStat;
#endif

/**
 * MACRO NAME: IX_CCI_ERR_DUMP
 *
 * DESCRIPTION: If an error is detected, dump it if in debug mode and not
 *              compiled out for code-complexity testing.
 *
 * @Param:  - IN    arg_Err. Error to be checked and if necessary dumped
 *
 */
#if defined(_IX_CCI_DEBUG_)
#define IX_CCI_ERR_DUMP( arg_Err )              \
    (void)( arg_Err == IX_SUCCESS || IX_CCI_ERROR_DUMP( arg_Err ) )
#else
#define IX_CCI_ERR_DUMP( arg_Ignored )    ((void)arg_Ignored)
#endif

#endif /* defined(_CCI_ERROR_PATH_TEST_) */

/**
 * Following macro is similar to IX_CCI_ASSERT but used specifically for checking
 * public API parameters, whereas IX_CCI_ASSERT is for internal logic errors. In
 * production code, the behavior of the two macros is identical. However when 
 * acquiring coverage metrics for unit testing, the assert macro compiles out, but
 * the parameter-checking macro remains
 */
#if defined(_IX_CCI_DEBUG_) && !defined(_CCT_EXCLUDE_) &&!defined(_CC_ERR_TYPE_) && !defined(_IX_CCI_QUIET_)
#define IX_CCI_PARAM_ASSERT(									    \
                       arg_Condition								\
                     )												\
	{																\
		if(!(arg_Condition))										\
		{															\
			ix_ossl_message_log( "ASSERT ERROR: %s\n%s line %d\n",	\
								 #arg_Condition,					\
								 __FILE__, __LINE__ );				\
			return IX_ERROR_NEW(									\
								  IX_CCI_ERR_ASSERTION,			    \
								  IX_ERROR_LEVEL_LOCAL				\
								);									\
		}															\
	} 
#elif defined(_IX_CCI_DEBUG_) && !defined(_CCT_EXCLUDE_) &&!defined(_CC_ERR_TYPE_) && defined(_IX_CCI_QUIET_)
#define IX_CCI_PARAM_ASSERT(									    \
                       arg_Condition								\
                     )												\
	{																\
		if(!(arg_Condition))										\
		{															\
			return IX_ERROR_NEW(									\
								  IX_CCI_ERR_ASSERTION,			    \
								  IX_ERROR_LEVEL_LOCAL				\
								);									\
		}															\
	} 
#elif defined(_IX_CCI_DEBUG_) && defined(_CC_ERR_TYPE_) && !defined(_IX_CCI_QUIET_)
#define IX_CCI_PARAM_ASSERT(									               \
                       arg_Condition								           \
                     )												           \
	{																           \
		if(!(arg_Condition))										           \
		{															           \
            ix_error ___e = ix_error_new( 0, IX_CCI_ERR_ASSERTION,             \
                                          IX_ERROR_LEVEL_LOCAL,                \
                                          IX_ERROR_LEAD_FMT("ASSERT ERROR")    \
                                          "\t%s", IX_ERROR_LEAD_ARGS,          \
                                          #arg_Condition );                    \
            IX_CCI_ERROR_DUMP( ___e );                                         \
            return ___e;                                                       \
		}															           \
	} 
#elif defined(_IX_CCI_DEBUG_) && defined(_CC_ERR_TYPE_) && defined(_IX_CCI_QUIET_)
#define IX_CCI_PARAM_ASSERT(									            \
                       arg_Condition								        \
                     )												        \
	{																        \
		if(!(arg_Condition))										        \
		{															        \
            return = ix_error_new( 0, IX_CCI_ERR_ASSERTION,                 \
                                      IX_ERROR_LEVEL_LOCAL,                 \
                                      IX_ERROR_LEAD_FMT("ASSERT ERROR")     \
                                      "\t%s", IX_ERROR_LEAD_ARGS,           \
                                      #arg_Condition );                     \
		}															        \
	} 
#else
/* No implementation */
#define IX_CCI_PARAM_ASSERT(				    \
                              arg_Condition		\
                           ) (void)0;

#endif


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__INTERNAL_CCI_ERROR_H__) */
