/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation.All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 5
 *
 * = FILENAME
 *      internal_eci_node.h
 *
 * = DESCRIPTION
 *      Contains structures and functions supporting branch nodes in the 
 *      execution control tree.
 *
 * = AUTHOR
 *       Roland Arajs
 *       roland.arajs@intel.com
 *
 * = CHANGE HISTORY
 *      5/24/2002 7:37:30 AM - creation time
 *
 * ============================================================================
 * $Id: internal_eci_node.h,v 1.3 2002/09/18 17:15:37 lbmiller Exp $
 */

#if !defined(__ECI_NODE_H__)
#define __ECI_NODE_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */


#include "cci/internal/internal_token_proc_container.h"


/**
 * IX_CCI_EVENT_CHILD bit gets set in node's vSelectedChild member when 1st
 * event is added to an engine.
 */

#define IX_CCI_NO_EVENT_PRESENT     0x80000000UL
#define IX_CCI_EVENT_ONLY_PRESENT   ~IX_CCI_NO_EVENT_PRESENT

/**
 * TYPENAME: ix_eci_node
 * 
 * DESCRIPTION: Those execution-control objects that form the branches and
 *              root of the execution control tree must reference other child
 *              exe_ctrl_interface objects. This structure, used by engines
 *              and policies includes a token processor container for its 
 *              children.
 *
 */
typedef struct ix_s_eci_node
{
    ix_cci_exe_ctrl_if          tEci;
    ix_token_processor_container tChildren;
    ix_cci_pending*             pPending;

    /* 
     * The following field indicates the selected child in the tChildren 
     * container. If nothing is selected, this value will be set to
     * IX_CCI_NO_SELECTED_CHILD. In the execution engine, the most significant
     * bit will be cleared when an event is added. This allows an execution
     * engine to handle events even if there are no message or packet handlers
     * attached.
     */
    ix_uint32                    vSelectedChild;
    ix_handle                    hSelf;
} ix_eci_node;

/**
 * NAME: ix_cci_eci_node_init
 *
 * DESCRIPTION:   Structure used by engines and policies
 * 
 * @Param:  - IN arg_pEciNode. Pointer to target structure.
 * @Param:  - IN arg_ProcessTokensFuncPtr. Process function to be associated
 *               with the target structure.
 * @Param:  - IN arg_FiniFuncPtr. Termination function associated with target
 *               structure.
 * @Param:  - IN arg_TkHandle. Specifies the location of the corresponding
 *               child-notify structure in shared memory.
 * @Param   - IN arg_InitSize. Initial capacity of node's container
 * @Param   - IN arg_GrowIncr.  Amount by which container will grow if adding
 *                              a child to a full container.
 *
 * @Return: 0 if successful or a valid ix_error token on failure.
 */
ix_error ix_cci_eci_node_init(
                               ix_eci_node*               arg_pEciNode,
                               ix_cci_process_tokens_ptr  arg_ProcessTokensFuncPtr,
                               ix_cci_eci_fini_ptr        arg_FiniFuncPtr,
                               ix_tk_handle               arg_TkHandle,
                               ix_uint32                  arg_InitSize,
                               ix_uint32                  arg_GrowIncr
                             );


/**
 * NAME: ix_cci_eci_node_fini
 *
 * DESCRIPTION:   
 * 
 * @Param:  - IN arg_pTokenProcessor. Pointer to ix_eci_node. The argument is of
 *                  type ix_cci_exe_ctrl_if* so that its function pointer can be
 *                  called within the execution control tree without the parent
 *                  having to know whether it is calling a fini function for
 *                  an eci node or leaf.
 *
 * @Return: 0 if successful or a valid ix_error token on failure.
 */
ix_error ix_cci_eci_node_fini(
                               ix_cci_exe_ctrl_if* arg_pTokenProcessor
                             );




#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__ECI_NODE_H__) */
