/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation.All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 5
 *
 * = FILENAME
 *      internal_engine.h
 *
 * = DESCRIPTION
 *      Defines the engine execution-control construct. This structure is
 *      hidden from the API: an engine handle provides indirect access to this 
 *      structure via the ix_tk_input_notify construct in RM shared memory.
 *
 * = AUTHOR
 *       Roland Arajs
 *       roland.arajs@intel.com
 *
 * = CHANGE HISTORY
 *      4/26/2002 8:37:52 AM - creation time
 *
 * ============================================================================
 * $Id: internal_engine.h,v 1.7 2003/12/11 22:25:35 ageorges Exp $
 */

#if !defined(__INTERNAL_ENGINE_H__)
#define __INTERNAL_ENGINE_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

#include "cci/engine.h"
#include "cci/cci_error.h"
#include "cci/internal/internal_event.h"
#include "cci/internal/internal_eci_node.h"
#include "cci/internal/internal_cc.h"


/**
 * Following definition allows one byte per engine for shutdown flag and one word
 * for shutdown status
 */

#define IX_CCI_NUM_SHUTDOWN_WORDS   (IX_CCI_MAX_ENGINES + 3) \
                                    / (sizeof(ix_uint32)) + 1

#define IX_CCI_NUM_SHMEM_COUNTS     2

#define IX_ENG_THREAD_AWAKE        'A'
#define IX_ENG_SHUTDOWN_START      'S'
#define IX_ENG_SHUTDOWN_COMPLETE   'C'


extern ix_tk_input_notify*     g_pMsgTkq; 
extern ix_tk_input_notify*     g_pPktTkq;
extern ix_tk_input_notify*     g_pPolicy;
extern ix_tk_input_notify*     g_pEngine;
extern ix_uint8*               g_pShutdownFlags;
extern ix_uint32*              g_pEngShutdownStatus;
extern ix_uint32*              g_pEngineCount;
extern ix_uint32*              g_pPolicyCount;


extern ix_ossl_sem_t* g_pShmemSema;

extern ix_ferror_func* g_pFatalErrorHandler;
extern void** g_ppFatalErrorContext;


/**
 * TYPENAME: ix_cci_eng_thread_args
 * 
 * DESCRIPTION: Structure that encapsulates the args passed into ix_exe_run()
 *
 */
typedef struct ix_s_cci_eng_thread_args
{
    ix_exe_init     pInitFunc;
    ix_exe_fini     pFiniFunc;
} ix_cci_eng_thread_args;



/**
 * TYPENAME: ix_engine_proc_node
 * 
 * DESCRIPTION: Construct to support an engine's processing interface.
 * NOTE:        Typedef of ix_engine_proc_node is in internal_exe_ctrl.h
 *
 */
struct ix_s_engine_proc_node
{
    ix_eci_node         tNode;
    ix_cc_node*         pCCList;
    ix_event_node*      pEventList;
    ix_event_node*      pDueEventList;
    ix_event_node*      pExpiredEventList;
    ix_event_node*      pActiveEvent;
    ix_uint32           vSizeEventList;
    /*
     * Following pointer controls the maximum number of message/packet handlers
     * that can be called before the engine cedes execution control to another
     * task.
     */
    ix_uint32           vGranularity;
    ix_ossl_thread_t    vThreadId;
    /*
     * Following pointer is used if a token is received in an ECI leaf with 
     * higher strict priority than the ECI leaf being currently processed.
     */
    ix_eci_node*        pPriorityChangeUpNode;
    /*
     * Following pointer is initialized from RM shared-memory space and copied
     * into the token-notify interfaces associated with the engine's policy 
     * tree. The referred integer is set when a new message or packet is 
     * received. The WRR process function reads this integer to decide whether
     * to check the tree for a higher priority token queue than the policy is
     * currently processing.
     */
    ix_uint32*          pIsNewToken;
    ix_uint32           vIsUserInitComplete;
    void*               pUserContext;
};


/**
 * NAME: ix_cci_run_engine
 *
 * DESCRIPTION:   Entry point for engine thread.
 * 
 * @Param:  - IN    arg_EngThreadArgs: Pointer to engine args of the
 *                      ix_cci_eng_thread_args. Void pointer used to
 *                      be compatible to ix_ossl_thread_create().
 *
 * @Return: 0 if successful or a valid ix_error token on failure.
 */

ix_error ix_cci_run_engine( void* arg_EngThreadArgs );



/**
 * NAME: ix_cci_exe_remove_leaf
 *
 * DESCRIPTION: Called when a single handler is attached to an engine in place
 *              of a policy. Removes the reference to the leaf as the active
 *              child and removes the handler from the default policy.
 * 
 * @Param:  - IN    arg_pEngine Pointer to engine's notify interface
 *
 */
void ix_cci_exe_remove_leaf( ix_tk_input_notify* arg_pEngine );



#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__INTERNAL_ENGINE_H__) */
