/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation.All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 5
 *
 * = FILENAME
 *      internal_event.h
 *
 * = DESCRIPTION
 *      Engine event construct and related functions
 *
 * = AUTHOR
 *       Roland Arajs
 *       roland.arajs@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CHANGE HISTORY
 *      4/26/2002 2:35:25 PM - creation time
 *
 * ============================================================================
 * $Id: internal_event.h,v 1.6 2002/12/06 23:50:16 rlarajs Exp $
 */

#if !defined(__INTERNAL_EVENT_H__)
#define __INTERNAL_EVENT_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

#include "ix_types.h"
#include "ix_ossl.h"
#include "cci/cc.h"
#include "cci/internal/internal_exe_ctrl_if.h"



/**
 * TYPENAME: ix_event_node
 * 
 * DESCRIPTION: List element in engine's event list. Items are inserted
 *              into the list in expiration sort order.
 *
 */
typedef struct ix_s_event_node
{
    ix_ossl_time_t          vExpiryTime;
    ix_uint32               vPriority;
    ix_event_func           pEventFunc;
    void*                   pEventContext;
    ix_handle               hUserCtxOwner;
    ix_event_handle         hEvent; /* Allows event list to be sorted */
    ix_event_type           vEventType;
    ix_ossl_time_t          vPeriod;    /* For recurring events */
    ix_uint32               vPeriodMilliSec;
    struct ix_s_event_node* pNext;
} ix_event_node;




/**
 * NAME: ix_cci_handle_events
 *
 * DESCRIPTION:   Gets the current time and pops due events off the event
 *                list. Then calls the due events in priority order. As
 *                each handler exits, the associated event node is either
 *                put on the expired list (one-shot events) or put back
 *                on the active event list (periodic events).
 * 
 * @Param:  - INOUT arg_pEngProcNode. Engine where events are to be handled.
 *
 * @Return: 0 if successful or a valid ix_error token on failure.
 */
ix_error ix_cci_handle_events( ix_engine_proc_node* arg_pEngProcNode );



/**
 * NAME: ix_cci_is_event_pending
 *
 * DESCRIPTION:   Examines front of event list and compares with current time
 * 
 * @Param:  - IN    arg_ppEventList: Pointer to event list to check. The function
 *                      takes a pointer to the list rather than the list itself
 *                      so that it can be stored in each execution engine node
 *                      and leaf. The front of the list is subject to change
 *                      as events are added and removed from the list.
 *
 * @Return: 1 if event is pending; 0 otherwise
 */
/*
int ix_cci_is_event_pending( const ix_event_node* const* arg_ppEventList );
*/



#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__INTERNAL_EVENT_H__) */
