/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation.All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 5
 *
 * = FILENAME
 *      internal_policy.h
 *
 * = DESCRIPTION
 *      Constructs for the three types of policy supported by the Core Component
 *      infrastructure: Strict Priority (SP), Weighted Round Robin (WRR), and 
 *      Round Robin (RR). Policy constructs use an ix_eci_node as their first 
 *      member. SP policies can use an ix_eci_node structure alone. The WRR
 *      and round-robin policies need extra data and extend ix_eci_node
 *      accordingly. The structure extensions are defined in this header.
 *
 * = AUTHOR
 *       Roland Arajs
 *       roland.arajs@intel.com
 *
 * = CHANGE HISTORY
 *      5/1/2002 4:03:16 PM - creation time
 *
 * ============================================================================
 * $Id: internal_policy.h,v 1.3 2002/09/18 17:15:38 lbmiller Exp $
 */

#if !defined(__INTERNAL_POLICY_H__)
#define __INTERNAL_POLICY_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */


#include "cci/policy.h"
#include "cci/internal/internal_engine.h"

/**
 * NAME: ix_cci_policy_clear_entry
 *
 * DESCRIPTION: Clears an entry from the selected policy. Note that it does
 *              not free any resources.
 * 
 * @Param:  - IN    arg_pPolicyNotifyIf: Pointer to notify interface 
 *                      corresponding to policy where the entry is located.
 * @Param:  - IN    pEntry: Pointer to entry that must be cleared from policy.
 *
 */
void ix_cci_policy_clear_entry(
                                ix_tk_input_notify* arg_pPolicyNotifyIf,
                                ix_cci_exe_ctrl_if* pEntry
                              );

/**
 * NAME: ix_cci_is_high_priority_override
 *
 * DESCRIPTION: Works up from current notify interface finding corresponding
 *              ECI nodes. For ECI nodes corresponding to strict-priority
 *              policies, the function determines whether the first pending
 *              child has higher priority than the selected child. If an SP
 *              poicy has data pending on a child with higher priority than
 *              the selected child, this function returns true; otherwise
 *              returns false. The new-token indicator is cleared by this
 *              function.
 * 
 * @Param:  - INOUT arg_pNotifyIf: Pointer to a token-notify interface 
 *                      corresponding to an ECI leaf.
 * @Param:  - INOUT arg_pEngProcIf: Pointer to engine node where the function
 *                      will save the reference to the SP policy nearest the 
 *                      root of the tree with higher priority pending than 
 *                      currently selected.
 * @Return: IX_CCI_HIGH_PRIORITY_OVERRIDE if priority override; 0 otherwise
 */
int ix_cci_is_high_priority_override( ix_tk_input_notify* arg_pNotifyIf,
                                      ix_engine_proc_node* pEngProcIf 
                                    );

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__INTERNAL_POLICY_H__) */
