/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation.All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 5
 *
 * = FILENAME
 *      internal_tk_notify_if.h
 *
 * = DESCRIPTION
 *      Functions supporting the token-notify interface. The token-notify
 *      construct is defined in rm/internal_tk_input_notify.h. The functions
 *      in this header are called in the CCI code only (not the RM code).
 *
 * = AUTHOR
 *       Roland Arajs
 *       roland.arajs@intel.com
 *
 * = CHANGE HISTORY
 *      5/2/2002 10:21:36 AM - creation time
 *
 * ============================================================================
 * $Id: internal_tk_notify_if.h,v 1.3 2002/09/18 17:15:38 lbmiller Exp $
 */

#if !defined(__INTERNAL_TK_NOTIFY_IF_H__)
#define __INTERNAL_TK_NOTIFY_IF_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

#include "ix_os_type.h"
#include "ix_symbols.h"
#include "ix_types.h"

#include "rm/internal/internal_tk_input_notify.h"




/* NOTE: Initialization is normally in two stages:
 *       1. Registering the execution-control interface
 *       2. Connecting to parent token-notify interface. 
 *       Of course, with an engine, the second stage is not required.
 */


/**
 * NAME: ix_cci_tk_notify_register_eci
 *
 * DESCRIPTION: Stores the reference to an execution-control interface in the
 *              corresponding token-notify interface. The function also 
 *              initializes the pending flags and semaphore field (if interface
 *              corresponds to an engine), and clears the fields related to 
 *              the token-notify parent. The latter fields should be set up
 *              using ix_cci_connnect_parent().
 * 
 * @Param:  - IN arg_hTkNotify: Handle of token-notify construct in RM shared-
 *                  memory space.
 * @Param:  - IN arg_pEci: Pointer to corresponding execution-control interface
 *
 * @Return: 0 if successful or a valid ix_error token on failure.
 */
ix_error ix_cci_tk_notify_register_eci(
                                        ix_tk_handle    arg_hTkNotify,
                                        void*           arg_pEci
                                      );



/**
 * NAME: ix_cci_connect_parent
 *
 * DESCRIPTION: Stores a pointer to the parent token-notify interface in a
 *              child token-notify interface, and stores the ID that the child
 *              has been assigned by the parent. This ID acts as an index into
 *              the parent's pending flags.
 * 
 * @Param:  - IN arg_pTkNotify. Pointer to structure that has already been
 *                  partly intialized with ix_cci_tk_notify_register_eci().
 * @Param:  - IN arg_pParent. Pointer to parent toke-notify interface
 *
 * @Return: 0 if successful or a valid ix_error token on failure.
 */
ix_error ix_cci_connect_parent(
                                ix_tk_input_notify*   arg_pTkNotify,
                                ix_tk_input_notify*   arg_pParent
                              );




/**
 * NAME: ix_cci_tk_notify_fini
 *
 * DESCRIPTION:   Clears the fields in the ix_tk_notify object
 * 
 * @Param:  - IN arg_pTkNotify. Pointer to structure to be cleared.
 *
 * @Return: 0 if successful or a valid ix_error token on failure.
 */
ix_error ix_cci_tk_notify_fini( ix_tk_input_notify* arg_pTkNotify );


/**
 * NAME: ix_cci_tk_notify_h_to_ptr
 *
 * DESCRIPTION:   Converts a token-notify handle to a shared-memory pointer
 * 
 * @Param:  - IN  arg_hTkNotify Handle to token-notify construct in shared 
 *                memory
 * @Param:  - OUT arg_ppTkNotify Where the function will return the pointer
 *
 * @Return: 0 if successful or a valid ix_error token on failure.
 */
ix_error ix_cci_tk_notify_h_to_ptr(
                                    ix_tk_handle    arg_hTkNotify,
                                    ix_tk_input_notify** arg_ppTkNotify
                                  );



/**
 * NAME: ix_cci_set_tk_notify_id
 *
 * DESCRIPTION: Sets an ID into a token-notify interface, which the ID of
 *              this interface to its parent.
 * 
 * @Param:  - IN arg_pTkNotify: Pointer to token-notify interface to be modified
 * @Param:  - IN arg_Id:        ID to be assigned to this interface.
 *
 * @Return: 0 if successful or a valid ix_error token on failure.
 */
ix_error ix_cci_set_tk_notify_id(
                                  ix_tk_input_notify* arg_pTkNotify,
                                  ix_uint32 arg_Id
                                );




#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__INTERNAL_TK_NOTIFY_IF_H__) */
