/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation.All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 5
 *
 * = FILENAME
 *      internal_token_proc_container.h
 *
 * = DESCRIPTION
 *      Container for ix_common_token_if objects that have a processing function
 *      Typically the container appears in Scheduling Policies and possibly in
 *      Execution Engines. The ix_common_token_if objects that have processing
 *      functions are Scheduling Policies and End-point Extensions.
 *
 * = AUTHOR
 *       Roland Arajs
 *       roland.arajs@intel.com
 *
 * = CHANGE HISTORY
 *      1/30/2002 5:05:12 PM - creation time
 *
 * ============================================================================
 * $Id: internal_token_proc_container.h,v 1.5 2002/09/18 17:15:38 lbmiller Exp $
 */

#if !defined(__INTERNAL_TOKEN_PROC_CONTAINER_H__)
#define __INTERNAL_TOKEN_PROC_CONTAINER_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

#include "ix_os_type.h"
#include "ix_symbols.h"
#include "ix_types.h"
#include "ix_error.h"
#include "cci/internal/internal_exe_ctrl_if.h"





/**
 * TYPENAME: ix_tkp_entry
 * 
 * DESCRIPTION: Element in an ix_token_processor_container.
 *
 */
struct ix_s_tkp_entry
{
    ix_cci_exe_ctrl_if*     pEci;
    ix_uint32               vWeight;
    ix_uint32               vCredits;   /* Only used by WRR policies */
};




/**
 * TYPENAME: ix_token_processor_container
 * 
 * DESCRIPTION: Container for ix_common_token_iinterface objects
 *
 */
typedef struct ix_s_token_processor_container
{
    ix_tkp_entry*   pEntries;
    ix_uint32       vCapacity;
    ix_uint32       vNumEntries;
    ix_uint32       vGrowIncr;
} ix_token_processor_container;


/**
 * NAME: ix_cci_tkproc_container_init
 *
 * DESCRIPTION:   
 * 
 * @Param:  - IN arg_pContainer - a pointer to an uninitialized container
 * @Param:  - IN arg_InitSize - initial capacity of container
 * @Param:  - IN arg_GrowIncr - amount that the container will grow when
 *                              attempting to add an element in a full container
 *
 * @Return: 0 if successful or a valid ix_error token on failure with error code
 *          IX_ERROR_OOM.
 */
ix_error ix_cci_tkproc_container_init(
                                       ix_token_processor_container* arg_pContainer,
                                       ix_uint32    arg_InitSize,
                                       ix_uint32    arg_GrowIncr
                                     );


/**
 * NAME: ix_cci_tkproc_container_fini
 *
 * DESCRIPTION:   
 * 
 * @Param:  - IN arg_pContainer - a pointer to an initialized container
 *
 * @Return: 0 if successful or a valid ix_error token on failure.
 */
ix_error ix_cci_tkproc_container_fini(
                                      ix_token_processor_container* arg_pContainer
                                    );


/**
 * NAME: ix_cci_tkpc_add_child
 *
 * DESCRIPTION:   Adds a ix_token_interface object to an ix_token_processor_container
 *                This function does no synchronization locking
 * 
 * @Param:  - IN arg_pContainer - pointer to container where child is to be 
 *                  added
 * @Param:  - IN arg_pChild - pointer to child.
 * @Param:  - IN arg_Weight - weight or priority to be assigned to child. A high 
 *                  number corresponds to a greater weight or higher priority; a
 *                  low number corresponds to a lower weight or priority.
 *
 * @Return: 0 if successful or a valid ix_error token on failure.
 */
ix_error ix_cci_tkpc_add_child(
                          ix_token_processor_container* arg_pContainer,
                          ix_cci_exe_ctrl_if*           arg_pChild,
                          ix_uint32                     arg_Weight
                        );

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__INTERNAL_TOKEN_PROC_CONTAINER_H__) */
