/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation.All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 5
 *
 * = FILENAME
 *      cci_error.c
 *
 * = DESCRIPTION
 *      
 *
 * = AUTHOR
 *       Roland Arajs
 *       roland.arajs@intel.com
 *
 * = CHANGE HISTORY
 *      7/24/2002 10:03:04 AM - creation time
 *
 * ============================================================================
 * $Id: cci_error.c,v 1.5 2003/08/18 21:48:16 ageorges Exp $
 * $History: $
 */

#include "ix_ossl.h"
#include "ix_rm.h"
#include "cci/internal/internal_cci_error.h"
#include "cci/internal/internal_engine.h"

ix_error ix_cci_register_fatal_error_handler(
                                              ix_ferror_func arg_Handler,
                                              void* arg_pContext
                                            )
{
    ix_configuration_property_handle hShutdownFlagsProperty;
    ix_uint32* uintShmemBlock;
    void** pShmem;
    ix_error err;

    IX_CCI_PARAM_ASSERT( arg_Handler != NULL );
    /*
     * Since the control app could be in a separate memory space from the 
     * execution engine where the error will occur, set up the error-handler
     * and context pointers using the registry to get a shared-memory 
     * reference.
     * NOTE: This function assumes that it will be called just once. Therefore,
     *       no semaphores in CCI are used to set up the shared-memory locations
     *       where the handler and context will be stored.
     */
    err = ix_rm_cp_property_open( IX_CP_CORE_PROPERTY_ROOT,
                                  g_szCciSharedPtrShutdownFlags,
                                  &hShutdownFlagsProperty );
    IX_CCI_ERR_RET( err != IX_SUCCESS,
                     IX_CCI_ERR_ENG_INIT_ENGINE_PROP_READ )
    err = ix_rm_cp_property_get_value_uint32( hShutdownFlagsProperty,
                                              (ix_uint32*)&uintShmemBlock );
    pShmem = (void**)uintShmemBlock 
                     + IX_CCI_NUM_SHUTDOWN_WORDS + IX_CCI_NUM_SHMEM_COUNTS
                     + IX_CCI_NUM_DEFAULT_ENGINES;
    *pShmem++ = (void*)arg_Handler;
    *pShmem = arg_pContext;
    return IX_SUCCESS;
}
