/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation.All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 5
 *
 * = FILENAME
 *      eci_node.c
 *
 * = DESCRIPTION
 *      Execution control construct for engines and policies, based on 
 *      ix_cci_exe_ctrl_if.
 *
 * = AUTHOR
 *       Roland Arajs
 *       roland.arajs@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CHANGE HISTORY
 *      4/28/2002 2:08:15 PM - creation time
 *
 * ============================================================================
 * $Id: eci_node.c,v 1.6 2003/08/18 21:48:16 ageorges Exp $
 */

#include "ix_ossl.h"
#include "ix_rm.h"

#include "cci/internal/internal_eci_node.h"
#include "cci/internal/internal_cci_error.h"
#include "cci/internal/internal_tk_notify_if.h"


ix_error ix_cci_eci_node_init(
                               ix_eci_node*               arg_pEciNode,
                               ix_cci_process_tokens_ptr  arg_ProcessTokensFuncPtr,
                               ix_cci_eci_fini_ptr        arg_FiniFuncPtr,
                               ix_tk_handle               arg_TkHandle,
                               ix_uint32                  arg_InitContainerSize,
                               ix_uint32                  arg_GrowIncr
                             )
{

    /* Initialize the base common_token interface structure */
    ix_error err = ix_cci_eci_init( (ix_cci_exe_ctrl_if*)arg_pEciNode,
                     arg_ProcessTokensFuncPtr, arg_FiniFuncPtr, arg_TkHandle );
    IX_CCI_IXERR_RET( err )

    err = ix_cci_tkproc_container_init( &arg_pEciNode->tChildren,
                                        arg_InitContainerSize,
                                        arg_GrowIncr );
    IX_CCI_IXERR_RET( err )
    
    /**
     * Initialize the pending flags ref through the eci's token-notify pointer
     */
    arg_pEciNode->pPending = &arg_pEciNode->tEci.pTkInNotify->tPending;

    
    /**
     * Store the token-notify handle so that handlers can be added to the 
     * default policy tree.
     */
    arg_pEciNode->hSelf = arg_TkHandle;

    arg_pEciNode->vSelectedChild = 0;
    return IX_SUCCESS;
}

ix_error ix_cci_eci_node_fini(
                               ix_cci_exe_ctrl_if* arg_pTokenProcessor
                             )
{
    ix_error err;
    ix_error cleanup_err;
    ix_eci_node* pNode = (ix_eci_node*)arg_pTokenProcessor;
    ix_uint32 idxChild = pNode->tChildren.vNumEntries;


    /* If the node has a parent */
    ix_tk_input_notify* pParentNotifyIf = pNode->tEci.pTkInNotify->pParent;
    /* If the node has a parent */
    if( pParentNotifyIf )
    {
        ix_eci_node* pParentEciNode;
        ix_tkp_entry* pTkpEntry;
        int numEntries;
        int i;

        /* Get the exe ctrl i/f pointer from token-notify i/f */
        pParentEciNode = (ix_eci_node*)pParentNotifyIf->pProcIfContext;
        /* Find the node pointer in the parent's child container */
        pTkpEntry = pParentEciNode->tChildren.pEntries;
        numEntries = pParentEciNode->tChildren.vNumEntries;

        /* If found */
        for( i=0; i<numEntries; i++ )
        {
            if( pTkpEntry[i].pEci == arg_pTokenProcessor )
            {
                /* Set the entry to NULL */
                pTkpEntry[i].pEci = NULL;
                pTkpEntry[i].vCredits = 0;
                pTkpEntry[i].vWeight = 0;
                break;
            }
        }
    }

    /* Call the fini function in each of this node's children */
    while( idxChild-- )
    {
        ix_cci_exe_ctrl_if* pChild = pNode->tChildren.pEntries[idxChild].pEci;
        if( pChild && pChild->pFiniFunc )
        {
            err = pChild->pFiniFunc( pChild );
            pChild->pFiniFunc = NULL;
            IX_CCI_ERR_DUMP( err );
        }
    }

    /* Close down the token-processor container */
    err = ix_cci_tkproc_container_fini( &pNode->tChildren );
    IX_CCI_ERR_DUMP( err );

    /* Terminate the ECI associated with this node */
    cleanup_err = ix_cci_eci_fini( arg_pTokenProcessor );
    IX_CCI_ERR_FWD_OR_DUMP( cleanup_err, err )

    IX_OSSL_FREE( arg_pTokenProcessor );
    return IX_SUCCESS;
}
