/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation.All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 5
 *
 * = FILENAME
 *      exe_ctrl_if.c
 *
 * = DESCRIPTION
 *      Code supporting the execution control interface (one of two interfaces
 *      used by CCI elements like execution engine and policies.
 *
 * = AUTHOR
 *       Roland Arajs
 *       roland.arajs@intel.com
 *
 * = CHANGE HISTORY
 *      1/29/2002 9:00:56 AM - creation time
 *
 * ============================================================================
 * $Id: exe_ctrl_if.c,v 1.5 2003/01/18 00:02:45 ageorges Exp $
 */
#include "ix_ossl.h"

#if !(_IX_OS_TYPE == _IX_OS_LINUX_KERNEL_)
#include <stddef.h>
#endif /*(_IX_OS_TYPE == _IX_OS_LINUX_KERNEL_) */

#include "ix_os_type.h"
#include "ix_types.h"
#include "ix_symbols.h"
#include "cci/cci_defs.h"
#include "cci/internal/internal_exe_ctrl_if.h"
#include "cci/internal/internal_tk_notify_if.h"
#include "cci/internal/internal_cci_error.h"

#if defined(_UT_COVERAGE_)
#include "cci/cci_engine_stubs.h"
#endif


/**
 * NAME: ix_cci_eci_init
 *
 * DESCRIPTION:   
 * 
 * @Param:  - IN arg_pContext. Pointer to target structure.
 * @Param:  - IN arg_ProcessTokensPtr. Process function to be associated
 *               with the target structure.
 * @Param:  - IN arg_hTkNotify Handle to corresponding token-notify interface.
 *
 * @Return: 0 if successful or a valid ix_error token on failure. 
 */
ix_error ix_cci_eci_init( 
                          ix_cci_exe_ctrl_if*     arg_pContext,
                          ix_cci_process_tokens_ptr      arg_ProcessTokensFuncPtr,
                          ix_cci_eci_fini_ptr            arg_FiniFuncPtr,
                          ix_tk_handle                   arg_hTkNotify
                        )
{
    ix_error err;
    ix_tk_input_notify* pTkNotify = NULL;

#if defined(_UT_COVERAGE_)
    incr_eci_init_counter();
#endif
    IX_CCI_ASSERT( arg_pContext != NULL )

    /* Convert handle passed in 3rd arg to a pointer and store it*/
    err = ix_cci_tk_notify_h_to_ptr( arg_hTkNotify, &pTkNotify );
    IX_CCI_DBG_IXERR_RET( err )

    arg_pContext->pTkInNotify = pTkNotify;
    arg_pContext->pProcessFunc = arg_ProcessTokensFuncPtr;
    arg_pContext->pFiniFunc = arg_FiniFuncPtr;

    return IX_SUCCESS;
}


ix_error ix_cci_eci_fini( ix_cci_exe_ctrl_if* arg_pTokenProcessor )
{
    /* Tell the token-notify interface to close */         
    ix_error err = ix_cci_tk_notify_fini( arg_pTokenProcessor->pTkInNotify );
    IX_CCI_IXERR_RET( err )

#if defined(_UT_COVERAGE_)
    incr_eci_fini_counter();
#endif

    /* Set fini funct to NULL to make sure it isn't called twice */
    arg_pTokenProcessor->pFiniFunc = NULL;
    arg_pTokenProcessor->pProcessFunc = NULL;
    return IX_SUCCESS;
}


ix_error ix_cci_set_notify_id(
                               ix_cci_exe_ctrl_if*   arg_pEci,
                               ix_uint32             arg_Id
                             )
{
    IX_CCI_ASSERT( arg_pEci != NULL )
    IX_CCI_ASSERT( arg_Id <= IX_CCI_POLICY_MAX_FLAGS ) /* ID limited pending flag */


    ix_cci_set_tk_notify_id( arg_pEci->pTkInNotify, arg_Id );
    return IX_SUCCESS;
}

