/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      hash.h
 *
 * = DESCRIPTION
 *      This file defines the types and prototypes required by the Hash API
 *
 * = AUTHOR
 *      Adrian Georgescu
 *      adrian.georgescu@intel.com
 *
 * = CHANGE HISTORY
 *      11/20/2002 4:04:15 PM - creation time 
 * ============================================================================
 * $Id: hash.h,v 1.4 2002/12/21 01:54:40 ageorges Exp $
 */

#if !defined(__HASH_H__)
#define __HASH_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

#if defined(_IX_RM_IMPL_HARDWARE_) 

#if !defined(_IX_RM_IMPL_SOFTWARE_HASH_)

#define _IX_RM_IMPL_HARDWARE_HASH_

#endif /* end !defined(_IX_RM_IMPL_SOFTWARE_HASH_) */

#endif /* end defined(_IX_RM_IMPL_HARDWARE_) */

/**
 * TYPENAME: ix_hash_48
 * 
 * DESCRIPTION: This type represents a 48-bit hash data type
 *
 */
typedef struct ix_s_hash_48
{
    ix_uint32 m_LW0;
    ix_uint32 m_LW1;
} ix_hash_48;


/**
 * TYPENAME: ix_hash_64
 * 
 * DESCRIPTION: This type represents a 64-bit hash data type
 *
 */
typedef struct ix_s_hash_64
{
    ix_uint32 m_LW0;
    ix_uint32 m_LW1;
} ix_hash_64;


/**
 * TYPENAME: ix_hash_128
 * 
 * DESCRIPTION: This type represents a 128-bit hash data type
 *
 */
typedef struct ix_s_hash_128
{
    ix_uint32 m_LW0;
    ix_uint32 m_LW1;
    ix_uint32 m_LW2;
    ix_uint32 m_LW3;
} ix_hash_128;


/**
 * TYPENAME: ix_hash_multiplier_48
 * 
 * DESCRIPTION: This represents the 48-bit hash multiplier type
 *
 */
typedef ix_hash_48 ix_hash_multiplier_48;


/**
 * TYPENAME: ix_hash_multiplier_64
 * 
 * DESCRIPTION: This represents the 64-bit hash multiplier type
 *
 */
typedef ix_hash_64 ix_hash_multiplier_64;


/**
 * TYPENAME: ix_hash_multiplier_128
 * 
 * DESCRIPTION: This represents the 128-bit hash multiplier type
 *
 */
typedef ix_hash_128 ix_hash_multiplier_128;



/**
 * NAME: ix_rm_hash_48_hash
 *
 * DESCRIPTION: This function will perform a 48-bit hash operation
 * 
 * @Param:  - INOUT - arg_pHash48 on input holds the data to be hashed and on
 *      output contains the hashed value.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for failure.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_hash_48_hash(
                             ix_hash_48* arg_pHash48
                           );


/**
 * NAME: ix_rm_hash_48_multiplier_set
 *
 * DESCRIPTION: This function will set the multiplier for the 48-bit hash
 * 
 * @Param:  - IN arg_pHashMultiplier48 - the new value of the 48-bit hash multiplier.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for failure.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_hash_48_multiplier_set(
                                       const ix_hash_multiplier_48* arg_pHashMultiplier48
                                     );


/**
 * NAME: ix_rm_hash_48_multiplier_get
 *
 * DESCRIPTION: This function will retrieves the multiplier for the 48-bit hash
 * 
 * @Param:  - OUT arg_pHashMultiplier48 - the locationwhere the 48-bit hash multiplier
 *      will be stored on successful return
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for failure.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_hash_48_multiplier_get(
                                       ix_hash_multiplier_48* arg_pHashMultiplier48
                                     );


/**
 * NAME: ix_rm_hash_64_hash
 *
 * DESCRIPTION: This function will perform a 64-bit hash operation
 * 
 * @Param:  - INOUT - arg_pHash64 on input holds the data to be hashed and on
 *      output contains the hashed value.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for failure.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_hash_64_hash(
                             ix_hash_64* arg_pHash64
                           );


/**
 * NAME: ix_rm_hash_64_multiplier_set
 *
 * DESCRIPTION: This function will set the multiplier for the 64-bit hash
 * 
 * @Param:  - IN arg_pHashMultiplier64 - the new value of the 64-bit hash multiplier.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for failure.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_hash_64_multiplier_set(
                                       const ix_hash_multiplier_64* arg_pHashMultiplier64
                                     );


/**
 * NAME: ix_rm_hash_64_multiplier_get
 *
 * DESCRIPTION: This function will retrieves the multiplier for the 64-bit hash
 * 
 * @Param:  - OUT arg_pHashMultiplier64 - the locationwhere the 64-bit hash multiplier
 *      will be stored on successful return
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for failure.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_hash_64_multiplier_get(
                                       ix_hash_multiplier_64* arg_pHashMultiplier64
                                     );


/**
 * NAME: ix_rm_hash_128_hash
 *
 * DESCRIPTION: This function will perform a 128-bit hash operation
 * 
 * @Param:  - INOUT - arg_pHash128 on input holds the data to be hashed and on
 *      output contains the hashed value.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for failure.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_hash_128_hash(
                              ix_hash_128* arg_pHash128
                            );


/**
 * NAME: ix_rm_hash_128_multiplier_set
 *
 * DESCRIPTION: This function will set the multiplier for the 128-bit hash
 * 
 * @Param:  - IN arg_pHashMultiplier128 - the new value of the 128-bit hash multiplier.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for failure.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_hash_128_multiplier_set(
                                        const ix_hash_multiplier_128* arg_pHashMultiplier128
                                      );


/**
 * NAME: ix_rm_hash_128_multiplier_get
 *
 * DESCRIPTION: This function will retrieves the multiplier for the 128-bit hash
 * 
 * @Param:  - OUT arg_pHashMultiplier128 - the locationwhere the 128-bit hash multiplier
 *      will be stored on successful return
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for failure.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_hash_128_multiplier_get(
                                        ix_hash_multiplier_128* arg_pHashMultiplier128
                                      );



#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__HASH_H__) */
