/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      internal_hash.h
 *
 * = DESCRIPTION
 *      This file declares all the internal types, symbols and function prototypes
 *      required by the hash API.
 *
 * = AUTHOR
 *      Adrian Georgescu
 *      adrian.georgescu@intel.com
 *
 * = CHANGE HISTORY
 *      11/20/2002 4:21:47 PM - creation time 
 * ============================================================================
 * $Id: internal_hash.h,v 1.9 2003/06/13 20:51:11 ageorges Exp $
 */

#if !defined(__INTERNAL_HASH_H__)
#define __INTERNAL_HASH_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */


#if defined(_IX_RM_IMPL_HARDWARE_) 

#include <halMmap.h>

/**
 * DESCRIPTION: This is the base address of the hash operand registers
 *
 */
#define IX_HASH_OPERAND_BASE_ADDRESS        ((ix_uint32)Hal_xscale_local_hash_virtAddr)
#define IX_HASH_MULTIPLIER_BASE_ADDRESS     ((ix_uint32)Hal_cap_global_ctl_csr_virtAddr + 0x900)
    

/**
 * DESCRIPTION: Offset of long word 0 of the hash 48 operand register
 *
 */
#define IX_HASH_48_OPERAND_LW0_OFFSET   0x00000000U

/**
 * DESCRIPTION: Offset of long word 1 of the hash 48 operand register
 *
 */
#define IX_HASH_48_OPERAND_LW1_OFFSET   0x00000004U


/**
 * DESCRIPTION: Offset of long word 0 of the hash 64 operand register
 *
 */
#define IX_HASH_64_OPERAND_LW0_OFFSET   0x00000010U

/**
 * DESCRIPTION: Offset of long word 1 of the hash 64 operand register
 *
 */
#define IX_HASH_64_OPERAND_LW1_OFFSET   0x00000014U


/**
 * DESCRIPTION: Offset of long word 0 of the hash 128 operand register
 *
 */
#define IX_HASH_128_OPERAND_LW0_OFFSET  0x00000020U

/**
 * DESCRIPTION: Offset of long word 1 of the hash 128 operand register
 *
 */
#define IX_HASH_128_OPERAND_LW1_OFFSET  0x00000024U

/**
 * DESCRIPTION: Offset of long word 0 of the hash 128 operand register
 *
 */
#define IX_HASH_128_OPERAND_LW2_OFFSET  0x00000028U

/**
 * DESCRIPTION: Offset of long word 1 of the hash 128 operand register
 *
 */
#define IX_HASH_128_OPERAND_LW3_OFFSET  0x0000002cU


/**
 * DESCRIPTION: Offset of the hash operation status register
 *
 */
#define IX_HASH_STATUS_OFFSET   0x00000030U


/**
 * DESCRIPTION: Offset of long word 0 of the hash 48 multiplier register
 *
 */
#define IX_HASH_48_MULTIPLIER_LW0_OFFSET   0x00000000U

/**
 * DESCRIPTION: Offset of long word 1 of the hash 48 multiplier register
 *
 */
#define IX_HASH_48_MULTIPLIER_LW1_OFFSET   0x00000004U


/**
 * DESCRIPTION: Offset of long word 0 of the hash 64 multiplier register
 *
 */
#define IX_HASH_64_MULTIPLIER_LW0_OFFSET   0x00000008U

/**
 * DESCRIPTION: Offset of long word 1 of the hash 64 multiplier register
 *
 */
#define IX_HASH_64_MULTIPLIER_LW1_OFFSET   0x0000000cU


/**
 * DESCRIPTION: Offset of long word 0 of the hash 128 multiplier register
 *
 */
#define IX_HASH_128_MULTIPLIER_LW0_OFFSET  0x00000010U

/**
 * DESCRIPTION: Offset of long word 1 of the hash 128 multiplier register
 *
 */
#define IX_HASH_128_MULTIPLIER_LW1_OFFSET  0x00000014U

/**
 * DESCRIPTION: Offset of long word 0 of the hash 128 multiplier register
 *
 */
#define IX_HASH_128_MULTIPLIER_LW2_OFFSET  0x00000018U

/**
 * DESCRIPTION: Offset of long word 1 of the hash 128 multiplier register
 *
 */
#define IX_HASH_128_MULTIPLIER_LW3_OFFSET  0x0000001cU



/**
 * MACRO NAME: IX_RM_HASH_WAIT_FOR_COMPLETION
 *
 * DESCRIPTION: waits for the hash operation to complete
 *
 */
#define IX_RM_HASH_WAIT_FOR_COMPLETION( \
                                      ) \
    while((*((volatile ix_uint32*)(IX_HASH_OPERAND_BASE_ADDRESS + IX_HASH_STATUS_OFFSET)) == 0)) \
    { \
    }
                                                  

#if defined (_IX_HASH_APPLY_SWAP_)

#define IX_HASH_32BIT_SWAP( \
                            arg_Data \
                          ) \
                            (((arg_Data) >> 24) | ((arg_Data) << 24) | \
                            (((arg_Data) & 0x0000ff00) << 8) | \
                            (((arg_Data) & 0x00ff0000) >> 8))


#define IX_HASH_16BIT_SWAP( \
                            arg_Data \
                          ) \
                            (((arg_Data) >> 8) | ((arg_Data) << 8))

#else

#define IX_HASH_32BIT_SWAP( \
                            arg_Data \
                          ) \
                          arg_Data

#define IX_HASH_16BIT_SWAP( \
                            arg_Data \
                          ) \
                          arg_Data

#endif /* defined (_IX_HASH_APPLY_SWAP_) */

#endif /* end defined(_IX_RM_IMPL_HARDWARE_) */

#if !defined(_IX_RM_IMPL_HARDWARE_HASH_)

/**
 * NAME: _ix_rm_hash_48_software_hash
 *
 * DESCRIPTION: This function will perform a software 48 bit hash operation
 * 
 * @Param:  - INOUT - arg_pHash48 on input holds the data to be hashed and on
 *      output contains the hashed value.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for failure.
 */
ix_error _ix_rm_hash_48_software_hash( 
                                       ix_hash_48* arg_pHash48
                                     );


/**
 * NAME: _ix_rm_hash_64_software_hash
 *
 * DESCRIPTION: This function will perform a software 64 bit hash operation
 * 
 * @Param:  - INOUT - arg_pHash64 on input holds the data to be hashed and on
 *      output contains the hashed value.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for failure.
 */
ix_error _ix_rm_hash_64_software_hash( 
                                       ix_hash_64* arg_pHash64
                                     );


/**
 * NAME: _ix_rm_hash_128_software_hash
 *
 * DESCRIPTION: This function will perform a software 128 bit hash operation
 * 
 * @Param:  - INOUT - arg_pHash128 on input holds the data to be hashed and on
 *      output contains the hashed value.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error token for failure.
 */
ix_error _ix_rm_hash_128_software_hash( 
                                        ix_hash_128* arg_pHash128
                                      );



#endif /* end !defined(_IX_RM_IMPL_HARDWARE_HASH_) */

/**
 * TYPENAME: _ix_hash_data_manager
 * 
 * DESCRIPTION: This structure manages the entire hash API
 *
 */
typedef struct _ix_s_hash_data_manager
{
#if !defined(_IX_RM_IMPL_HARDWARE_HASH_)

    /* hash multipliers for the software implementation */
    ix_hash_multiplier_48   m_HashMultiplier48;
    ix_hash_multiplier_64   m_HashMultiplier64;
    ix_hash_multiplier_128  m_HashMultiplier128;

#endif /* end !defined(_IX_RM_IMPL_HARDWARE_HASH_) */

#if defined(_IX_RM_MULTI_THREADED_)

    ix_synchronization_adapter      m_SyncAdapter; /* synchronization adapter */

#endif /* end defined(_IX_RM_MULTI_THREADED_) */

    ix_uint32 m_Reserved;

} _ix_hash_data_manager;



/**
 * NAME: _ix_rm_hash_init
 *
 * DESCRIPTION: This function should be called at the time the resource manager initializer 
 *          function is called. The function will initialize all the Hash API.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 */
ix_error _ix_rm_hash_init(void);


/**
 * NAME: _ix_rm_hash_fini
 *
 * DESCRIPTION: This function should be called at the time the resource manager finalizer function
 *          is called. The function will provide cleanup for Hash API.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 */
ix_error _ix_rm_hash_fini(void);


/**
 * Global Hash API manager
 */
extern _ix_hash_data_manager g_HashAPIManager;


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__INTERNAL_HASH_H__) */
