/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      internal_remote_communication.h
 *
 * = DESCRIPTION
 *      This file defines all internal types and function prototypes required by the remote
 *      communication API.
 *
 * = AUTHOR
 *      Adrian Georgescu
 *      adrian.georgescu@intel.com
 *
 * = CHANGE HISTORY
 *      11/27/2002 3:25:01 PM - creation time 
 * ============================================================================
 * $Id: internal_remote_communication.h,v 1.1 2002/12/05 00:28:12 ageorges Exp $
 */

#if !defined(__INTERNAL_REMOTE_COMMUNICATION_H__)
#define __INTERNAL_REMOTE_COMMUNICATION_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

/**
 * TYPENAME: ix_remote_comm_manager
 * 
 * DESCRIPTION: This structure represents the remote communication manager.
 *
 */
typedef struct ix_s_remote_comm_manager
{
    ix_end_point    m_RemoteCommPacketEndPoint;

    ix_end_point    m_RemoteCommMessageEndPoint;

    ix_remote_comm_service  m_RemoteCommService;

#if defined( _IX_RM_MULTI_THREADED_ )

    ix_synchronization_adapter      m_SyncAdapter; /* synchronization adapter */

#endif /* defined( _IX_RM_MULTI_THREADED_ ) */

} ix_remote_comm_manager;


/**
 * Default handlers declarations
 */
ix_error _ix_rm_remote_comm_default_packet_handler
                              (
                                ix_buffer_handle arg_hBuffer,
                                ix_uint32 arg_UserData,
                                void* arg_pContext,
                                ix_communication_id arg_CommId
                              );

ix_error _ix_rm_remote_comm_default_message_handler
                              (
                                ix_buffer_handle arg_hBuffer,
                                ix_uint32 arg_UserData,
                                void* arg_pContext,
                                ix_communication_id arg_CommId
                              );

/**
 * NAME: _ix_rm_remote_comm_init
 *
 * DESCRIPTION: This function should be called at the time the resource manager initializer 
 *          function is called. The function will initialize all the Remote Communication API.
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 */
ix_error _ix_rm_remote_comm_init(void);


/**
 * NAME: _ix_rm_remote_comm_fini
 *
 * DESCRIPTION: This function should be called at the time the resource manager finalizer function
 *          is called. The function will provide cleanup for Remote Communication API.
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid error token for failure.
 */
ix_error _ix_rm_remote_comm_fini(void);


/**
 * Global remote communication manager
 */
extern ix_remote_comm_manager g_RemoteCommManager;

/**
 * Default remote communication service
 */
extern ix_remote_comm_service g_DefaultRemoteService; 

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__INTERNAL_REMOTE_COMMUNICATION_H__) */
