/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation.All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 5
 *
 * = FILENAME
 *      internal_tk_input_notify.h
 *
 * = DESCRIPTION
 *      Defines the token-input notify interface used in the core component
 *      infrastructure. Access to this structure is required by the RM 
 *      because it will be used by the send-message and send-packet functions.
 *
 * = AUTHOR
 *       Roland Arajs
 *       roland.arajs@intel.com
 *
 * = CHANGE HISTORY
 *      5/1/2002 4:30:18 PM - creation time
 *
 * ============================================================================
 * $Id: internal_tk_input_notify.h,v 1.12 2002/11/06 23:24:28 ageorges Exp $
 */
 
#if !defined(__INTERNAL_TK_INPUT_NOTIFY_H__)
#define __INTERNAL_TK_INPUT_NOTIFY_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */


/* Following #defines should go in some centralized definitions file in 
 * rm directory 
 */
/**
 * IX_CCI_POLICY_MAX_FLAGS: The maximum number of items that an individual 
 * policy can handle
 */
#define IX_CCI_POLICY_MAX_FLAGS     32

#define IX_CCI_POLICY_MAX_QFLAGS    IX_CCI_POLICY_MAX_FLAGS/sizeof(ix_uint32)

#define IX_CCI_NO_SELECTED_CHILD    0xFFFFFFFFUL


/**
 * TYPENAME: ix_cci_pending
 * 
 * DESCRIPTION: Pending flags (1 byte per flag). At first it seems attractive
 *              to use bit flags for detecting pending conditions. With bit
 *              flags, if we limit the number of flags to 32, we can see whether
 *              data is pending on all flags in one operation. However, if flags
 *              are set, ascertaining which flags are set may be slower than
 *              with byte flags. With byte flags we can check 4 flags at a time
 *              and we don't need to do any masking operations. Another 
 *              advantage of byte flags is that if each flag can be set by 
 *              a different thread, we don't have to set a mutex to change the
 *              flag or use special memory that offers atomic flag-set and
 *              flag-clear operations. If we are careful, we can even avoid 
 *              contention issues between a send thread setting a flag and the
 *              process thread clearing it.
 *
 */
typedef union ix_u_cci_pending
{
    ix_uint32   qflag[IX_CCI_POLICY_MAX_QFLAGS];
    ix_uint8    flag[IX_CCI_POLICY_MAX_FLAGS];
} ix_cci_pending;



/**
 * TYPENAME: ix_token_if_type
 * 
 * DESCRIPTION: 
 *
 */
typedef enum ix_e_token_if_type
{
    IX_TOKEN_IF_TYPE_FIRST = 0,
    IX_TOKEN_IF_TYPE_TKQ = IX_TOKEN_IF_TYPE_FIRST,
    IX_TOKEN_IF_TYPE_POLICY,
    IX_TOKEN_IF_TYPE_ENGINE
} ix_token_if_type;


/**
 * TYPENAME: ix_tk_input_notify
 * 
 * DESCRIPTION: The portion of the ix_cci_exe_ctrl_if in shared memory.
 *              Contains data that must be read/written from thread context
 *              of a packet/message sender.
 *
 */
typedef struct ix_s_tk_input_notify
{
    struct ix_s_tk_input_notify* pParent;
    /* ix_handle                    hParent; */ /* Handles can work across mem partitions */
    ix_uint32                    vId; /* used when calling above function */
    ix_cci_pending               tPending; /* used by both above functions in SP */


    /* Semaphore is only set in execution engines */
    ix_ossl_sem_t                vSema; /* synchronization element */
    ix_uint32                    vSemaTimeout;  /* timeout (in ms) for sema take */

    /* The following pointer should only be used in the thread context of the
     * execution engine's control thread; it must not be used in the context
     * of the send-message or send-packet context.
     */
    void*                        pProcIfContext; 
    /*
     * Copy of new-token indicator, initialized in RM shared-mem space in 
     * execution engine.
     */
    ix_uint32*                   pIsNewToken;
} ix_tk_input_notify;

/**
 * Function called from RM (needs to be in RM so that it will
 * compile without the CCI present). This function does not return an error
 * value because there is nothing the RM could do with it.
 */
void ix_cci_notify( void* pContext );

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__INTERNAL_TK_INPUT_NOTIFY_H__) */
