/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2004 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP family of Network Processors
 *
 * = LIBRARY
 *      Resource Manager Library
 *
 * = MODULE
 *      Library Interface
 *
 * = FILENAME
 *      internal_version.h
 *
 * = DESCRIPTION
 *      This file provides all version and build information.
 *
 * = AUTHOR
 *      Adrian Georgescu
 *      adrian.georgescu@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      3/13/2002 10:53:06 AM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: internal_version.h,v 1.8.4.1 2004/03/12 23:01:56 ageorges Exp $
 */

#if !defined(__INTERNAL_VERSION_H__)
#define __INTERNAL_VERSION_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */



/**
 * DESCRIPTION: This symbol defines the copyright string
 *
 */
#define IX_RM_COPYRIGHT_STRING  \
"                INTEL CORPORATION PROPRIETARY INFORMATION\n" \
"    This software is supplied under the terms of a license agreement or\n" \
"    nondisclosure agreement with Intel Corporation and may not be copied\n" \
"    or disclosed except in accordance with the terms in that agreement.\n" \
"       Copyright (C) 2000-2004 Intel Corporation. All rights reserved.\n"


#if (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) 

/**
 * DESCRIPTION: This symbol defines the package name
 *
 */
#define IX_RM_PACKAGE_STRING    "IXA SDK 3.5"


/**
 * DESCRIPTION: This symbol defines the component name
 *
 */
#define IX_RM_PRODUCT_STRING    "Resource Manager Library"


/**
 * DESCRIPTION: This symbol defines the component's major version number
 *
 */
#define IX_RM_VERSION_MAJOR         1


/**
 * DESCRIPTION: This symbol defines the component's minor big version number
 *
 */
#define IX_RM_VERSION_MINOR_BIG     4


/**
 * DESCRIPTION: This symbol defines the component's minor little version number
 *
 */
#define IX_RM_VERSION_MINOR_LITTLE  0


/**
 * DESCRIPTION: This symbol defines the component's version type as follows:
 *          A,a: alpha
 *          B,b: beta
 *          C,c: production
 *          D,d: debug build
 *          E,e: experimental (higher risk/untested code)
 *          P,f: patch build
 *          U,u: unofficial (default for internal builds)
 */
#define IX_RM_VERSION_TYPE              'c'
#define IX_RM_VERSION_TYPE_UNQUOTED      c

#else /* Westport hardware */

/**
 * DESCRIPTION: This symbol defines the package name
 *
 */
#define IX_RM_PACKAGE_STRING    "IXA SDK for WESTPORT"


/**
 * DESCRIPTION: This symbol defines the component name
 *
 */
#define IX_RM_PRODUCT_STRING    "Resource Manager Library - Memory Management API"


/**
 * DESCRIPTION: This symbol defines the component's major version number
 *
 */
#define IX_RM_VERSION_MAJOR         1


/**
 * DESCRIPTION: This symbol defines the component's minor big version number
 *
 */
#define IX_RM_VERSION_MINOR_BIG     0


/**
 * DESCRIPTION: This symbol defines the component's minor little version number
 *
 */
#define IX_RM_VERSION_MINOR_LITTLE  0


/**
 * DESCRIPTION: This symbol defines the component's version type as follows:
 *          A,a: alpha
 *          B,b: beta
 *          C,c: production
 *          D,d: debug build
 *          E,e: experimental (higher risk/untested code)
 *          P,f: patch build
 *          U,u: unofficial (default for internal builds)
 */
#define IX_RM_VERSION_TYPE              'u'
#define IX_RM_VERSION_TYPE_UNQUOTED      u

#endif /* (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) */

/**
 * DESCRIPTION: This symbol defines the component's version increment
 *
 */
#define IX_RM_VERSION_INCREMENT     1


/**
 * MACRO NAME: IX_RM_STRINGIZE
 *
 * DESCRIPTION: Transform the passed token into a string
 *
 * @Param:  - IN arg_Token - any token type
 *
 * @Return: Return a string corresponding to the passed token
 */
#define IX_RM_STRINGIZE( \
                         arg_Token \
                       ) \
                       #arg_Token
                       

/**
 * MACRO NAME: IX_RM_MAKE_VERSION_STRING
 *
 * DESCRIPTION: Builds a version string based on the passed info.
 *
 * @Param:  - IN arg_VersionMajor - an integer representing major version number
 * @Param:  - IN arg_VersionMinorBig - an integer representing minor big version number
 * @Param:  - IN arg_VersionMinorLittle - an integer representing minor little version number
 * @Param:  - IN arg_VersionType - a character representing version type
 * @Param:  - IN arg_VersionIncrement - an integer representing version increment
 *
 * @Return: Returns a string representing the version.
 */
#define IX_RM_MAKE_VERSION_STRING( \
                                   arg_VersionMajor, \
                                   arg_VersionMinorBig, \
                                   arg_VersionMinorLittle, \
                                   arg_VersionType, \
                                   arg_VersionIncrement \
                                  ) \
                                  IX_RM_STRINGIZE(arg_VersionMajor) "." IX_RM_STRINGIZE(arg_VersionMinorBig) "." \
                                  IX_RM_STRINGIZE(arg_VersionMinorLittle) IX_RM_STRINGIZE(arg_VersionType) \
                                  IX_RM_STRINGIZE(arg_VersionIncrement)
                                  
/**
 * DESCRIPTION: This symbol defines the component's version string in the
 *          folowing format:
 *          'VersionMajor'.'VersionMinorBig'.'VersionMinorLittle''VersionType'"VersionIncrement'
 *
 */
#define IX_RM_VERSION_STRING    IX_RM_MAKE_VERSION_STRING(IX_RM_VERSION_MAJOR, IX_RM_VERSION_MINOR_BIG, \
                                    IX_RM_VERSION_MINOR_LITTLE, IX_RM_VERSION_TYPE_UNQUOTED, IX_RM_VERSION_INCREMENT)
  

#if (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) 

/**
 * DESCRIPTION: This symbol defines the component's build month
 *
 */
#define IX_RM_BUILD_MONTH   3


/**
 * DESCRIPTION: This symbol defines the component's build day
 *
 */
#define IX_RM_BUILD_DAY    12


/**
 * DESCRIPTION: This symbol defines the component's build year
 *
 */
#define IX_RM_BUILD_YEAR   2004

#else /* Westport hardware */

/**
 * DESCRIPTION: This symbol defines the component's build month
 *
 */
#define IX_RM_BUILD_MONTH   3


/**
 * DESCRIPTION: This symbol defines the component's build day
 *
 */
#define IX_RM_BUILD_DAY    12


/**
 * DESCRIPTION: This symbol defines the component's build year
 *
 */
#define IX_RM_BUILD_YEAR   2004

#endif /* (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) */


/**
 * MACRO NAME: IX_RM_MAKE_BUILD_DATE_STRING
 *
 * DESCRIPTION: Builds a a build date string based on the passed info.
 *
 * @Param:  - IN arg_Month - an integer representing the build month
 * @Param:  - IN arg_Day - an integer representing the build day
 * @Param:  - IN arg_Year - an integer representing the build year
 *
 * @Return: Return a string representing the build date
 */
#define IX_RM_MAKE_BUILD_DATE_STRING( \
                                      arg_Month, \
                                      arg_Day, \
                                      arg_Year \
                                    ) \
                                    IX_RM_STRINGIZE(arg_Month) "/" \
                                    IX_RM_STRINGIZE(arg_Day) "/" \
                                    IX_RM_STRINGIZE(arg_Year)

/**
 * DESCRIPTION: This symbol defines the component's build date string
 *
 */
#define IX_RM_BUILD_DATE_STRING   IX_RM_MAKE_BUILD_DATE_STRING(IX_RM_BUILD_MONTH, IX_RM_BUILD_DAY, IX_RM_BUILD_YEAR)


/**
 * Global address to the version scheme information string
 */
extern char* g_pVersionSchemeString;



/**
 * NAME: _ix_rm_build_version_scheme_string
 *
 * DESCRIPTION: This function will build the version scheme string for the product and 
 *          will set g_pVersionSchemeString to point to the buffer.
 *
 */
void _ix_rm_build_version_scheme_string(void);

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__INTERNAL_VERSION_H__) */
