/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation.All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = LIBRARY
 *      Resource Manager Library
 *
 * = MODULE
 *      Memory Manager API
 *
 * = FILENAME
 *      memory.h
 *
 * = DESCRIPTION
 *      This file defines all data types required by the Memory Manager library module interface.
 *      The file will declares prototypes for all Memory Manager interface functions.
 *
 * = AUTHOR
 *       Adrian Georgescu
 *       adrian.georgescu@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      10/11/2001 12:13:11 PM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: memory.h,v 1.30 2003/10/03 19:11:18 ageorges Exp $
 */

#if !defined(__MEMORY_H__)
#define __MEMORY_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

#if (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) 

/**
 * TYPENAME: ix_memory_type
 * 
 * DESCRIPTION: This enumerated type defines all types of memory existent in the
 *      system that is managed by the Resource Manager.
 *
 */
typedef enum ix_e_memory_type
{
    IX_MEMORY_TYPE_FIRST = 0,
    IX_MEMORY_TYPE_DRAM = IX_MEMORY_TYPE_FIRST, /* DRAM memory */
    IX_MEMORY_TYPE_SRAM, /* SRAM memory */
    IX_MEMORY_TYPE_SCRATCH, /* Scratch memory */
    IX_MEMORY_TYPE_LOCAL, /* Local memory */

    IX_MEMORY_TYPE_LAST
} ix_memory_type;

#else /* Westport hardware */

/**
 * TYPENAME: ix_memory_type
 * 
 * DESCRIPTION: Defines all types of memory existent in the
 *              system that is managed by the Memory Management API.
 *
 */
typedef enum ix_e_memory_type
{
    IX_MEMORY_TYPE_FIRST = 0,
    IX_MEMORY_TYPE_DRAM_SOUTH = IX_MEMORY_TYPE_FIRST, 
    IX_MEMORY_TYPE_SRAM_MSG_SOUTH, 
    IX_MEMORY_TYPE_DRAM_NORTH,
    IX_MEMORY_TYPE_SRAM_NORTH,
    IX_MEMORY_TYPE_SCRATCH, 
    IX_MEMORY_TYPE_LAST
} ix_memory_type;


/**
 * TYPENAME: ix_memory_bank_type
 * 
 * DESCRIPTION: Defines all memory bank types existent in the system
 *
 */
typedef enum ix_e_memory_bank_type
{
    IX_MEMORY_BANK_TYPE_FIRST = 0,
    IX_MEMORY_BANK_TYPE_SOUTH = IX_MEMORY_BANK_TYPE_FIRST, 
    IX_MEMORY_BANK_TYPE_NORTH, 
    IX_MEMORY_BANK_TYPE_LAST
} ix_memory_bank_type;


#endif /* (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) */


#if (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) 

/**
 * TYPENAME: ix_memory_info
 * 
 * DESCRIPTION: This structure is used to retrieve memory info from resource manager
 *
 */
typedef struct ix_s_memory_info
{
    ix_uint32*      m_pStartAddress; /* pointer to the start of the memory block */
    ix_uint32       m_TotalSize; /* total memory size */
    ix_uint32       m_FreeSize; /* free memory size */
    ix_uint32       m_DefaultAlignmentShift; /* default alignment expressed as a power of 2 */
    ix_uint32       m_ChannelPhysicalOffset; 
        /* physical offset of the start of the memory block inside the memory channel */
    void*           m_pChannelPhysicalAddress;
        /* this is the start physical address of the current channel */
} ix_memory_info;

#else /* Westport hardware */

/**
 * TYPENAME: ix_memory_info
 * 
 * DESCRIPTION: This structure is used to retrieve memory information from 
 *	Memory Management API.
 *
 */
typedef struct ix_s_memory_info
{
    ix_memory_bank_type m_MemoryBankType;  
        /* memory bank this area belongs to */     
    ix_uint32*          m_pStartAddress; 
        /* pointer to the start of the memory block */
    ix_uint32           m_TotalSize; /* total memory size */
    ix_uint32           m_FreeSize; /* free memory size */
        /* default alignment expressed as a power of 2 */
    ix_uint32           m_DefaultAlignmentShift; 
        /* physical offset of the start of the memory block inside the memory channel */
    ix_uint32           m_ChannelPhysicalOffset; 
        /* this is the start physical address of the current channel  as it is aliased in the South bank */
    void*               m_pChannelPhysicalAddress;
        /* physical offset of the start of the non aliased memory block inside the memory channel */
    ix_uint32           m_NonAliasChannelPhysicalOffset; 
        /* this is the start non alias physical address of the current channel in its memory bank */
    void*               m_pNonAliasChannelPhysicalAddress;

} ix_memory_info;

#endif /* (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) */



#if (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) 

/**
 * TYPENAME: ix_memory_alignment_type
 * 
 * DESCRIPTION: This enumerated type describes the possible requests for allocating
 *          or reserving memory with alignment constraints. 
 *          IX_MEMORY_ALIGNMENT_TYPE_VIRTUAL will allocate providing alignment for the 
 *          virtual memory. The physical offset and/or physical address might not
 *          be aligned with the same alignment requirements.
 *          IX_MEMORY_ALIGNMENT_TYPE_PHYSICAL will allocate providing alignment for the 
 *          physical memory. The physical offset and/or virtual address might not
 *          be aligned with the same alignment requirements.
 *          IX_MEMORY_ALIGNMENT_TYPE_PHYSICAL_OFFSET will allocate providing alignment for the 
 *          the physical memory offset for the channel. The virtual address and/or physical 
 *          address might not be aligned with the same alignment requirements.
 *
 */
typedef enum ix_e_memory_alignment_type
{
    IX_MEMORY_ALIGNMENT_TYPE_FIRST = 0,
    IX_MEMORY_ALIGNMENT_TYPE_VIRTUAL = IX_MEMORY_ALIGNMENT_TYPE_FIRST,
    IX_MEMORY_ALIGNMENT_TYPE_PHYSICAL,
    IX_MEMORY_ALIGNMENT_TYPE_PHYSICAL_OFFSET,
    IX_MEMORY_ALIGNMENT_TYPE_LAST
} ix_memory_alignment_type;

#else /* Westport hardware */

/**
 * TYPENAME: ix_memory_alignment_type
 * 
 * DESCRIPTION: This enumerated type describes the possible requests for 
 *	allocating or reserving memory with alignment constraints. 
 *	IX_MEMORY_ALIGNMENT_TYPE_VIRTUAL will allocate providing alignment for 
 *	the virtual memory. The physical offset and/or physical address might 
 *	not be aligned with the same alignment requirements. 
 *	IX_MEMORY_ALIGNMENT_TYPE_PHYSICAL will allocate providing alignment for 
 *	the physical memory. The physical offset and/or virtual address might 
 *	not be aligned with the same alignment requirements. For the aliased 
 *  North memory bank this alignment will be in the aliased South memory 
 *  region.
 *	IX_MEMORY_ALIGNMENT_TYPE_PHYSICAL_OFFSET will allocate providing 
 *	alignment for the physical memory offset for the channel. The 
 *	virtual address and/or physical address might not be aligned with the 
 *	same alignment requirements. For the aliased North memory bank this 
 *  alignment will be in the aliased South memory region.
 *	IX_MEMORY_ALIGNMENT_TYPE_PHYSICAL_NON_ALIAS will allocate providing 
 *  alignment for the physical memory in the owning bank. 
 *	IX_MEMORY_ALIGNMENT_TYPE_PHYSICAL_OFFSET_NON_ALIAS will allocate 
 *  providing alignment for the physical memory offset for the channel in 
 *  the owning bank. 
 *
 */
typedef enum ix_e_memory_alignment_type
{
    IX_MEMORY_ALIGNMENT_TYPE_FIRST = 0,
    IX_MEMORY_ALIGNMENT_TYPE_VIRTUAL = IX_MEMORY_ALIGNMENT_TYPE_FIRST,
    IX_MEMORY_ALIGNMENT_TYPE_PHYSICAL,
    IX_MEMORY_ALIGNMENT_TYPE_PHYSICAL_OFFSET,
    IX_MEMORY_ALIGNMENT_TYPE_PHYSICAL_NON_ALIAS,
    IX_MEMORY_ALIGNMENT_TYPE_PHYSICAL_OFFSET_NON_ALIAS,
    IX_MEMORY_ALIGNMENT_TYPE_LAST
} ix_memory_alignment_type;


/**
 * TYPENAME: ix_memory_offset_type
 * 
 * DESCRIPTION: Defines all memory offset types used by the API. 
 *	IX_MEMORY_OFFSET_TYPE_NON_ALIASED offset it makes sense just for North 
 *	bank memory that is aliased into specific South bank windows.
 *
 */
typedef enum ix_e_memory_offset_type
{
    IX_MEMORY_OFFSET_TYPE_FIRST = 0,
    IX_MEMORY_OFFSET_TYPE_ALIASED = IX_MEMORY_OFFSET_TYPE_FIRST, 
    IX_MEMORY_OFFSET_TYPE_NON_ALIASED, 
    IX_MEMORY_OFFSET_TYPE_LAST
} ix_memory_offset_type;


#endif /* (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) */


/**
 * Prototypes for SRAM, DRAM and SCRATCH memory management functions
 */

#if (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) 

/**
 * NAME: ix_rm_mem_alloc
 *
 * DESCRIPTION: This function allocates common ME/Core memory on behalf of the caller.
 *          On return, the location referred by arg_pMemoryAddr will be set to NULL if failure 
 *          or to the allocated memory block address if successful. The returned address will
 *          aligned according to the default alignment for the type of memory.
 *          If the size does not comply with memory alignment then the actual allocated size will
 *          be rounded up to insure that alignment will be satisfied for further allocations.
 * 
 * @Param:  - IN arg_MemType - represents the type of memory requested.
 * @Param:  - IN arg_MemChannel - represents the channel from which memory is requested.
 * @Param:  - IN arg_Size - represents the size in bytes of the block requested.
 * @Param:  - OUT arg_pMemoryAddr - represents the address where the allocated memory address
 *          should be stored.
 *
 * @Return: IX_ERROR_SUCCESS if successful or a valid ix_error for failure. The location referred by 
 *          arg_pMemoryAddr will be set to NULL if failure, or to the allocated memory block
 *          address if successful.
 */

#else /* Westport hardware */

/**
 * NAME: ix_rm_mem_alloc
 *
 * DESCRIPTION: This function allocates common ME/NPE/Core memory on behalf 
 *	of the caller. On return, the location referred by arg_pMemoryAddr will 
 *	be set to NULL if failure  or to the allocated memory block address if 
 *	successful. The returned address will be aligned according to the 
 *	default virtual alignment for the type of memory. If the size does not 
 *	comply with memory alignment then the actual allocated size will be 
 *	rounded up to insure that alignment will be satisfied for further 
 *	allocations.
 * 
 * @Param:  - IN arg_MemType - represents the type of memory requested.
 * @Param:  - IN arg_MemChannel - represents the channel from which memory is 
 *	requested.
 * @Param:  - IN arg_Size - represents the size in bytes of the block 
 *	requested.
 * @Param:  - OUT arg_pMemoryAddr - represents the address where the 
 *	allocated memory address should be stored.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure. 
 *	The location referred by arg_pMemoryAddr will be set to NULL if 
 *	failure, or to the allocated memory block address if successful.
 */

#endif /* (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) */

IX_EXPORT_FUNCTION 
ix_error ix_rm_mem_alloc( 
                          ix_memory_type arg_MemType, 
                          ix_uint32 arg_MemChannel, 
                          ix_uint32 arg_Size, 
                          void** arg_pMemoryAddr 
                        );


#if (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) 

/**
 * NAME: ix_rm_mem_alloc_aligned
 *
 * DESCRIPTION: This function allocates common ME/Core memory on behalf of the caller.
 *          On return, the location referred by arg_pMemoryAddr will be set to NULL if failure 
 *          or to the allocated memory block address if successful. The returned address will
 *          be alligned according to the arg_AlignmentSfift parameter and  arg_AlignmentType alignment
 *          type. If arg_AlignmentSfift parameter is less than default alignment then the return
 *          memory will be aligned according to the default alignment. 
 *          If the alignment type is not IX_MEMORY_ALIGNMENT_TYPE_VIRTUAL then the resulting virtual
 *          address might not be aligned according to the arg_AlignmentShift but the physiscal offset
 *          or physical address will be properly aligned.
 *          The actual allocated size will be rounded up to accomodate the required alignment.
 * 
 *
 * @Param:  - IN arg_MemType - represents the type of memory requested.
 * @Param:  - IN arg_MemChannel - represents the channel from which memory is requested.
 * @Param:  - IN arg_Size - represents the size in bytes of the block requested.
 * @Param:  - IN arg_AlignmentType - specifies the type of alignment required as explained at 
 *          ix_memory_alignment_type type description
 * @Param:  - IN arg_AlignmentShift - this value specifies the required memory block alignment as the
 *          power of 2 of the alignment. If the specified alignment is less than the
 *          default alignment for the corresonding memory then the default alignment will be used.
 * @Param:  - OUT arg_pMemoryAddr - represents the address where the allocated memory address
 *          should be stored.
 *
 * @Return: IX_ERROR_SUCCESS if successful or a valid ix_error for failure. The location referred by 
 *          arg_pMemoryAddr will be set to NULL if failure, or to the allocated memory block
 *          address if successful.
 */

#else /* Westport hardware */

/**
 * NAME: ix_rm_mem_alloc_aligned
 *
 * DESCRIPTION: This function allocates common ME/NPE/Core memory on behalf 
 *	of the caller. On return, the location referred by arg_pMemoryAddr will 
 *	be set to NULL for failure or to the allocated memory block address if 
 *	successful. The returned address will be aligned according to the 
 *	arg_AlignmentShift parameter and  arg_AlignmentType alignment type. If 
 *	arg_AlignmentShift parameter is less than default alignment then the 
 *	return memory will be aligned according to the default alignment. If 
 *	the alignment type is not IX_MEMORY_ALIGNMENT_TYPE_VIRTUAL then the 
 *	resulting virtual address might not be aligned according to the 
 *	arg_AlignmentShift but the physical offset or physical address will be 
 *	properly aligned. The actual allocated size will be rounded up to 
 *	accommodate the required alignment.
 * 
 * @Param:  - IN arg_MemType - represents the type of memory requested.
 * @Param:  - IN arg_MemChannel - represents the channel from which memory is 
 *	requested.
 * @Param:  - IN arg_Size - represents the size in bytes of the block 
 *	requested.
 * @Param:  - IN arg_AlignmentType - specifies the type of alignment required 
 *	as explained at ix_memory_alignment_type type description
 * @Param:  - IN arg_AlignmentShift - this value specifies the required 
 *	memory block alignment as the power of 2 of the alignment. If the 
 *	specified alignment is less than the default alignment for the 
 *	corresponding memory then the default alignment will be used.
 * @Param:  - OUT arg_pMemoryAddr - represents the address where the 
 *	allocated memory address should be stored.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure. 
 *	The location referred by arg_pMemoryAddr will be set to NULL if 
 *	failure, or to the allocated memory block address if successful.
 */

#endif /* (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) */

IX_EXPORT_FUNCTION 
ix_error ix_rm_mem_alloc_aligned( 
                                  ix_memory_type arg_MemType, 
                                  ix_uint32 arg_MemChannel, 
                                  ix_uint32 arg_Size,
                                  ix_memory_alignment_type arg_AlignmentType,
                                  ix_uint32 arg_AlignmentShift,
                                  void** arg_pMemoryAddr 
                                );

#if (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) 

/**
 * NAME: ix_rm_mem_reserve
 *
 * DESCRIPTION: This function will reserve an area of memory at the specified memory channel byte offset
 *          and of specified size. The function is provided as support for the microC compiler and
 *          linker. This call should be made before any memory allocation has been made in order
 *          to reduce the chance of failure. If the memory channel byte offset is not aligned according
 *          to the default alignment for the memory type then the allocated size will be larger to
 *          accomodate the alignment. In general the byte offset and size should comply with
 *          the memory alignment, otherwise more memory will be reserved to insure the right alignment.
 * 
 * @Param:  - IN arg_MemType - represents the type of memory to be reserved.
 * @Param:  - IN arg_MemChannel - represents the channel from which memory is to be reserved.
 * @Param:  - IN arg_Size - represents the size in bytes of the memory block to be reserved.  
 * @Param:  - INOUT arg_pByteOffset - represents the address of byte offset of the memory block 
 *          requested for reservation. On return this value will be adjusted to accomodate the
 *          the memory type default alignment.
 *
 * @Return: IX_ERROR_SUCCESS if successful or a valid ix_error for failure. If any portion of the memory requested
 *          for reservation has been already allocated then the call will fail.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_mem_reserve( 
                            ix_memory_type arg_MemType, 
                            ix_uint32 arg_MemChannel, 
                            ix_uint32 arg_Size, 
                            ix_uint32* arg_pByteOffset 
                          );

#else /* Westport hardware */

/**
 * NAME: ix_rm_mem_reserve
 *
 * DESCRIPTION: This function will reserve an area of memory at the specified 
 *	memory channel byte offset and type, and of specified size. This call 
 *	should be made before any memory allocation has been made in order to 
 *	reduce the chance of failure. If the memory channel byte offset is not 
 *	aligned according to the default alignment for the memory type, then 
 *	the allocated size will be larger to accommodate the alignment. In 
 *	general the byte offset and size should comply with the memory 
 *	alignment, otherwise, more memory will be reserved to insure the right 
 *	alignment. For the North bank memory, if the offset type is 
 *	IX_MEMORY_OFFSET_TYPE_ALIASED, then it is relative to the South alias 
 *	window, otherwise it is relative to the start of the memory channel in 
 *	the North bank.
 * 
 * @Param:  - IN arg_MemType - represents the type of memory to be reserved.
 * @Param:  - IN arg_MemChannel - represents the channel from which memory is 
 *	to be reserved.
 * @Param:  - IN arg_Size - represents the size in bytes of the memory block 
 *	to be reserved. 
 * @Param:  - IN arg_OffsetType - represents the type of the specified offset 
 * @Param:  - INOUT arg_pByteOffset - represents the address of byte offset 
 *	of the memory block requested for reservation. On return this value 
 *	will be adjusted to accommodate the memory type default alignment.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure. 
 *	If any portion of the memory requested for reservation has been already 
 *	allocated then the call will fail.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_mem_reserve( 
                            ix_memory_type arg_MemType, 
                            ix_uint32 arg_MemChannel, 
                            ix_uint32 arg_Size,
                            ix_memory_offset_type arg_OffsetType, 
                            ix_uint32* arg_pByteOffset 
                          );

#endif /* (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) */


#if (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) 

/**
 * NAME: ix_rm_mem_reserve_aligned
 *
 * DESCRIPTION: This function will reserve an area of memory at the specified memory channel byte 
 *          offset and of specified size. The function is provided as support for the microC compiler and
 *          linker. This call should be made before any memory allocation has been made in order
 *          to reduce the chance of failure. If the byte offset is not be aligned according to 
 *          arg_AlignmentShift or - if that is less than default alignment - to the default alignment
 *          and alignment type then internal alignment will be performed and more data might be 
 *          reserved. 
 *          If the alignment type is not IX_MEMORY_ALIGNMENT_TYPE_VIRTUAL then the resulting virtual
 *          address might not be aligned according to the arg_AlignmentShift but the physiscal offset
 *          or physical address will be properly aligned.
 *          In general the byte offset and size should comply with the memory alignment, 
 *          otherwise more memory will be reserved to insure the right alignment. On return *arg_pByteOffset might
 *          be modified to reflect alignment requirements.
 * 
 * @Param:  - IN arg_MemType - represents the type of memory to be reserved.
 * @Param:  - IN arg_MemChannel - represents the channel from which memory is to be reserved.
 * @Param:  - IN arg_Size - represents the size in bytes of the memory block to be reserved.  
 * @Param:  - IN arg_AlignmentType - specifies the type of alignment required as explained at 
 *          ix_memory_alignment_type type description
 * @Param:  - IN arg_AlignmentShift - this value specifies the required memory block alignment as the
 *          power of 2 of the alignment. If the specified alignment is less than the
 *          default alignment for the corresonding memory then the default alignment will be used.
 * @Param:  - INOUT arg_pByteOffset - represents the address of byte offset of the memory block 
 *          requested for reservation. On return this value will be adjusted to accomodate the
 *          the memory type default alignment.
 *
 * @Return: IX_ERROR_SUCCESS if successful or a valid ix_error for failure. If any portion of the memory requested
 *          for reservation has been already allocated then the call will fail.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_mem_reserve_aligned( 
                                    ix_memory_type arg_MemType, 
                                    ix_uint32 arg_MemChannel, 
                                    ix_uint32 arg_Size,
                                    ix_memory_alignment_type arg_AlignmentType,
                                    ix_uint32 arg_AlignmentShift,
                                    ix_uint32* arg_pByteOffset 
                                  );

#else /* Westport hardware */

/**
 * NAME: ix_rm_mem_reserve_aligned
 *
 * DESCRIPTION: This function will reserve an area of memory at the specified 
 *	memory channel byte offset and type, and of specified size. This call 
 *	should be made before any memory allocation has been made in order to 
 *	reduce the chance of failure. If the byte offset is not aligned 
 *	according to arg_AlignmentShift or - if that is less than default 
 *	alignment - to the default alignment for alignment type, then internal 
 *	alignment will be performed and more data might be reserved. If the 
 *	alignment type is not IX_MEMORY_ALIGNMENT_TYPE_VIRTUAL then the 
 *	resulting virtual address might not be aligned according to the 
 *	arg_AlignmentShift but the physical offset or physical address will be 
 *	properly aligned. In general, the byte offset and size should comply 
 *	with the memory alignment, otherwise more memory will be reserved to 
 *	insure the right alignment. On return, *arg_pByteOffset might be 
 *	modified to reflect alignment requirements. For the North bank memory, 
 *	if the offset type is IX_MEMORY_OFFSET_TYPE_ALIASED, then it is 
 *	relative to the South alias window, otherwise it is relative to the 
 *	start of the memory channel in the North bank.
 * 
 * @Param:  - IN arg_MemType - represents the type of memory to be reserved.
 * @Param:  - IN arg_MemChannel - represents the channel from which memory is 
 *	to be reserved.
 * @Param:  - IN arg_Size - represents the size in bytes of the memory block 
 *	to be reserved.  
 * @Param:  - IN arg_AlignmentType - specifies the type of alignment required 
 *	as explained at ix_memory_alignment_type type description
 * @Param:  - IN arg_AlignmentShift - this value specifies the required 
 *	memory block alignment as the power of 2 of the alignment. If the 
 *	specified alignment is less than the default alignment for the 
 *	corresponding memory then the default alignment will be used.
 * @Param:  - IN arg_OffsetType - represents the type of the specified offset 
 * @Param:  - INOUT arg_pByteOffset - represents the address of byte offset 
 *	of the memory block requested for reservation. On return this value 
 *	will be adjusted to accommodate the memory type default alignment.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure. 
 *	If any portion of the memory requested for reservation has been already 
 *	allocated then the call will fail.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_mem_reserve_aligned( 
                                 ix_memory_type arg_MemType, 
                                 ix_uint32 arg_MemChannel, 
                                 ix_uint32 arg_Size,
                                 ix_memory_alignment_type arg_AlignmentType,
                                 ix_uint32 arg_AlignmentShift,
                                 ix_memory_offset_type arg_OffsetType, 
                                 ix_uint32* arg_pByteOffset 
                                  );

#endif /* (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) */



/**
 * NAME: ix_rm_mem_free
 *
 * DESCRIPTION: This function will free memory previously allocated or reserved. Passing an 
 *          address that has not been allocated or that has already been freed will result in
 *          an error. Passing NULL will result in an no op. Accessing the memory location 
 *          after the block has been freed will result in unpredictable behaviour.
 * 
 * 
 * @Param:  - IN arg_pMemory - represents the address of the memory block to be freed.
 *
 * @Return: IX_ERROR_SUCCESS if successful or a valid ix_error for failure. Passing an address that has 
 *          not been allocated or that has already been freed will result in an error. 
 *          Passing NULL will result in an no op.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_mem_free( 
                         void* arg_pMemory 
                       );



/**
 * NAME: ix_rm_mem_info
 *
 * DESCRIPTION: This function retrieves memory information for the specified memory type
 *          and specified channel. On return the structure pointed by arg_pMemoryInfo will be
 *          filled with the memory information.
 * 
 * @Param:  - IN arg_MemType - represents the type of memory for which we want to get the info.
 * @Param:  - IN arg_MemChannel - represents the channel for which we want to get the info.
 * @Param:  - OUT arg_pMemoryInfo - on return the structure pointed by arg_pMemoryInfo will be
 *          filled with the memory information 
 *
 * @Return: IX_ERROR_SUCCESS if successful or a valid ix_error for failure.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_mem_info( 
                         ix_memory_type arg_MemType, 
                         ix_uint32 arg_MemChannel, 
                         ix_memory_info* arg_pMemoryInfo 
                       );

#if (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) 

/**
 * Prototypes for local memory management functions
 */

/**
 * NAME: ix_rm_mem_local_alloc
 *
 * DESCRIPTION: This function will allocate a block of local memory for the specified ME.  
 *          The location referred by arg_pByteOffset will be set to (ix_uint32)(-1) if 
 *          failure, or to the allocated memory block offset if successful. 
 *          The local memory will not be accessible to the Core applications.
 *          The apps will get the offset and patch it in the uCode for the specific ME. The 
 *          returned offset will be 4 bytes aligned. If the size does not comply with memory alignment
 *          then the actual allocated size will be rounded up to 4 byte boundary.
 * 
 * @Param:  - IN arg_MENumber - represents the ME number for which local memory should be 
 *          allocated.
 * @Param:  - IN arg_Size - represents the size in bytes of the block requested.
 * @Param:  - OUT arg_pByteOffset - represents the address where the allocated memory offset in bytes
 *          should be stored.
 *
 * @Return: IX_ERROR_SUCCESS if successful or a valid ix_error for failure. The location referred by 
 *          arg_pByteOffset will be set to (ix_uint32)(-1) if failure, or to the allocated memory block
 *          offset if successful.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_mem_local_alloc( 
                                ix_uint32 arg_MENumber, 
                                ix_uint32 arg_Size, 
                                ix_uint32* arg_pByteOffset 
                              );

/**
 * NAME: ix_rm_mem_local_reserve
 *
 * DESCRIPTION: This function will reserve an area of memory at the specified byte offset and
 *          of specified size. The function is provided as support for the microC compiler and
 *          linker. This call should be made before any memory allocation has been made in order
 *          to reduce the chance of failure. If the byte offset will not be 4 byte aligned, then 
 *          internal alignment will be performed and more data might be reserved. In general 
 *          the byte offset and size should comply with the memory alignment, otherwise more 
 *          memory will be reserved to insure the right alignment. 
 * 
 * 
 * @Param:  - IN arg_MENumber - represents the ME number for which local memory should be 
 *          reserved.
 * @Param:  - IN arg_Size - represents the size in bytes of the memory block to be reserved.  
 * @Param:  - INOUT arg_pByteOffset - represents the address of byte offset of the memory block 
 *          requested for reservation. On return this value will be adjusted to reflect the
 *          default alignment for the memory block.
 *
 * @Return: IX_ERROR_SUCCESS if successful or a valid ix_error for failure. If any portion of the memory requested
 *          for reservation has been already allocated then the call will fail.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_mem_local_reserve( 
                                  ix_uint32 arg_MENumber, 
                                  ix_uint32 arg_Size, 
                                  ix_uint32* arg_pByteOffset 
                                );


/**
 * NAME: ix_rm_mem_local_free
 *
 * DESCRIPTION: This function will free memory previously allocated or reserved. Passing an 
 *          address that has not been allocated or that has already been freed will result in
 *          an error. Passing NULL will result in an no op. Accessing the memory location 
 *          after the block has been freed will result in unpredictable behaviour.   
 * 
 * 
 * @Param:  - IN arg_MENumber - represents the ME number for which local memory should be 
 *          freed.
 * @Param:  - IN arg_MemoryOffset - represents the memory offset in bytes of the memory 
 *          block to be freed.
 *
 * @Return: IX_ERROR_SUCCESS if successful or a valid ix_error for failure. Passing an address that has 
 *          not been allocated or that has already been freed will result in an error. 
 *          Passing NULL will result in an no op.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_mem_local_free( 
                               ix_uint32 arg_MENumber, 
                               ix_uint32 arg_MemoryOffset 
                             );


/**
 * NAME: ix_rm_mem_local_info
 *
 * DESCRIPTION: This function retrieves memory information for the specified memory type
 *          and specified channel. On return the structure pointed by arg_pMemoryInfo will be
 *          filled with the memory information.
 * 
 * @Param:  - IN arg_MENumber - represents the microegine number for which we want to get the info.
 * @Param:  - OUT arg_pMemoryInfo - on return the structure pointed by arg_pMemoryInfo will be
 *          filled with the memory information 
 *
 * @Return: IX_ERROR_SUCCESS if successful or a valid ix_error for failure.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_mem_local_info( 
                               ix_memory_type arg_MENumber, 
                               ix_memory_info* arg_pMemoryInfo 
                             );

#endif /* (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) */



/**
 * Prototypes for memory READ functions
 */


/**
 * MACRO NAME: IX_RM_MEM_UINT8_READ
 *
 * DESCRIPTION: This macro is provided for high portablity between simulation
 *          code and hardware code. If the user is using this macro instead of
 *          direct memory dereference, then the code written for hardware mode
 *          will compile and run in simulation mode and viceversa. The macro 
 *          will produce a 8 bit unsigned value.
 *
 * @Param:  - IN arg_pMemoryAddr - represents the address of the memory location to be read.
 *          Should be of the type ix_uint8* .
 *
 * @Return: a ix_uint8 representing the value at the specified location.
 */
#if defined(_IX_RM_IMPL_HARDWARE_)

#if (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_)

/* Just dereference the memory pointer */
#define IX_RM_MEM_UINT8_READ( \
                              arg_pMemoryAddr \
                            ) \
                            (*((ix_uint8*)(arg_pMemoryAddr)))

#else /* Westport hardware */

/* Read 32 bit at a time */
#define IX_RM_MEM_UINT8_READ( \
                              arg_pMemoryAddr \
                            ) \
                            ((*(ix_uint32*)((ix_uint32)(arg_pMemoryAddr) & 0xfffffffc)) >> \
                             (((ix_uint32)(arg_pMemoryAddr) & 0x00000003) << 3))


#endif /* (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) */


#else

/**
 * NAME: _ix_rm_mem_uint8_read
 *
 * DESCRIPTION: This function is provided as a IX_RM_MEM_UINT8_READ macro support function.
 *          Returns directly the read value. 
 * 
 * 
 * @Param:  - IN arg_pFromAddr - represents the address where to read from.
 *
 * @Return: The unsigned 8 bit value at the specified memory address. If the operation fails then the return
 *          value will be (ix_uint8)(-1) but there is no guarantee that we get that value because
 *          of an error.
 */
IX_EXPORT_FUNCTION 
ix_uint8 _ix_rm_mem_uint8_read( 
                                const ix_uint8* arg_pFromAddr 
                              );

/* Invoke the internal memory read function */
#define IX_RM_MEM_UINT8_READ( \
                              arg_pMemoryAddr \
                            ) \
                            (_ix_rm_mem_uint8_read((ix_uint8*)(arg_pMemoryAddr)))

#endif /* end defined(_IX_RM_IMPL_HARDWARE_) */



/**
 * MACRO NAME: IX_RM_MEM_UINT16_READ
 *
 * DESCRIPTION: This macro is provided for high portablity between simulation
 *          code and hardware code. If the user is using this macro instead of
 *          direct memory dereference, then the code written for hardware mode
 *          will compile and run in simulation mode and viceversa.
 *          The passed address should be 2 bytes aligned and the macro will produce
 *          a 16 bit unsigned value.
 *
 * @Param:  - IN arg_pMemoryAddr - represents the address of the memory location to be read.
 *          Should be of the type ix_uint16*. The address should be 2 bytes aligned.
 *
 * @Return: a ix_uint16 representing the value at the specified location.
 */
#if defined(_IX_RM_IMPL_HARDWARE_)

#if (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_)

/* Just dereference the memory pointer */
#define IX_RM_MEM_UINT16_READ( \
                               arg_pMemoryAddr \
                             ) \
                             (*((ix_uint16*)(arg_pMemoryAddr)))

#else /* Westport hardware */


/* Read 32 bit at a time */
#define IX_RM_MEM_UINT16_READ( \
                              arg_pMemoryAddr \
                            ) \
                            ((*(ix_uint32*)((ix_uint32)(arg_pMemoryAddr) & 0xfffffffd)) >> \
                             (((ix_uint32)(arg_pMemoryAddr) & 0x00000002) << 3))

#endif /* (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) */

#else

/**
 * NAME: _ix_rm_mem_uint16_read
 *
 * DESCRIPTION: This function is provided as a IX_RM_MEM_UINT16_READ macro support function.
 *          The passed memory address where to read from should be 2 bytes aligned or the
 *          results might be unpredictable. In DEBUG mode the alignment will be checked but
 *          not in RELEASE.
 * 
 * 
 * @Param:  - IN arg_pFromAddr - represents the address where to read from. The address should be
 *          2 bytes aligned or unpredictable behaviour might result.
 *
 * @Return: The value at the specified memory address. If the operation fails then the return
 *          value will be (ix_uint16)(-1) but there is no guarantee that we get that value because
 *          of an error.
 */
IX_EXPORT_FUNCTION 
ix_uint16 _ix_rm_mem_uint16_read( 
                                  const ix_uint16* arg_pFromAddr 
                                );

/* Invoke the internal memory read function */
#define IX_RM_MEM_UINT16_READ( \
                               arg_pMemoryAddr \
                             ) \
                             (_ix_rm_mem_uint16_read((ix_uint16*)(arg_pMemoryAddr)))

#endif /* end defined(_IX_RM_IMPL_HARDWARE_) */



/**
 * MACRO NAME: IX_RM_MEM_UINT32_READ
 *
 * DESCRIPTION: This macro is provided for high portablity between simulation
 *          code and hardware code. If the user is using this macro instead of
 *          direct memory dereference, then the code written for hardware mode
 *          will compile and run in simulation mode and viceversa.
 *          The passed address should be 4 bytes aligned and the macro will produce
 *          a 32 bit unsigned value.
 *
 * @Param:  - IN arg_pMemoryAddr - represents the address of the memory location to be read.
 *          Should be of the type ix_uint32*. The address should be 4 bytes aligned.
 *
 * @Return: a ix_uint32 representing the value at the specified location.
 */
#if defined(_IX_RM_IMPL_HARDWARE_)

/* Just dereference the memory pointer */
#define IX_RM_MEM_UINT32_READ( \
                               arg_pMemoryAddr \
                             ) \
                             (*((ix_uint32*)(arg_pMemoryAddr)))

#else

/**
 * NAME: _ix_rm_mem_uint32_read
 *
 * DESCRIPTION: This function is provided as a IX_RM_MEM_UINT32_READ macro support function.
 *          The passed memory address where to read from should be 4 bytes aligned or the
 *          results might be unpredictable. In DEBUG mode the alignment will be checked but
 *          not in RELEASE.
 * 
 * 
 * @Param:  - IN arg_pFromAddr - represents the address where to read from. The address should be
 *          4 bytes aligned or unpredictable behaviour might result.
 *
 * @Return: The value at the specified memory address. If the operation fails then the return
 *          value will be (ix_uint32)(-1) but there is no guarantee that we get that value because
 *          of an error.
 */
IX_EXPORT_FUNCTION 
ix_uint32 _ix_rm_mem_uint32_read( 
                                  const ix_uint32* arg_pFromAddr 
                                );



/* Invoke the internal memory read function */
#define IX_RM_MEM_UINT32_READ( \
                               arg_pMemoryAddr \
                             ) \
                             (_ix_rm_mem_uint32_read((ix_uint32*)(arg_pMemoryAddr)))

#endif /* end defined(_IX_RM_IMPL_HARDWARE_) */



/**
 * MACRO NAME: IX_RM_MEM_UINT64_READ
 *
 * DESCRIPTION: This macro is provided for high portablity between simulation
 *          code and hardware code. If the user is using this macro instead of
 *          direct memory dereference, then the code written for hardware mode
 *          will compile and run in simulation mode and viceversa.
 *          The passed address should be 8 bytes aligned and the macro will produce
 *          a 64 bit unsigned value.
 *
 * @Param:  - IN arg_pMemoryAddr - represents the address of the memory location to be read.
 *          Should be of the type ix_uint64*. The address should be 8 bytes aligned.
 *
 * @Return: a ix_uint64 representing the value at the specified location.
 */
#if defined(_IX_RM_IMPL_HARDWARE_)

/* Just dereference the memory pointer */
#define IX_RM_MEM_UINT64_READ( \
                               arg_pMemoryAddr \
                             ) \
                             (*((ix_uint64*)(arg_pMemoryAddr)))

#else

/**
 * NAME: _ix_rm_mem_uint64_read
 *
 * DESCRIPTION: This function is provided as a IX_RM_MEM_UINT64_READ macro support function.
 *          The passed memory address where to read from should be 8 bytes aligned or the
 *          results might be unpredictable. In DEBUG mode the alignment will be checked but
 *          not in RELEASE.
 * 
 * @Param:  - IN arg_pFromAddr - represents the address where to read from. The address should be
 *          8 bytes aligned or unpredictable behaviour might result.
 *
 * @Return: The value at the specified memory address. If the operation fails then the return
 *          value will be (ix_uint64)(-1) but there is no guarantee that we get that value because
 *          of an error.
 */
IX_EXPORT_FUNCTION 
ix_uint64 _ix_rm_mem_uint64_read( 
                                  const ix_uint64* arg_pFromAddr 
                                );

/* Invoke the internal memory read function */
#define IX_RM_MEM_UINT64_READ( \
                               arg_pMemoryAddr \
                             ) \
                             (_ix_rm_mem_uint64_read((ix_uint64*)(arg_pMemoryAddr)))

#endif /* end defined(_IX_RM_IMPL_HARDWARE_) */


/**
 * Prototypes for memory WRITE functions
 */



/**
 * MACRO NAME: IX_RM_MEM_UINT8_WRITE
 *
 * DESCRIPTION: This macro is provided for high portablity between simulation
 *          code and hardware code. If the user is using this macro instead of
 *          direct memory dereference, then the code written for hardware mode
 *          will compile and run in simulation mode and viceversa. The macro 
 *          will write a 8 bit unsigned value to the memory location specified.
 *
 * @Param:  - IN arg_pMemoryAddr - represents the address of the memory location to be read.
 *          Should be of the type ix_uint8*.
 * @Param:  - IN arg_Value - The value to be written to the specified location. 
 *          Should be of the type ix_uint8.
 *
 * @Return: a ix_uint8 representing the value written at the specified location.
 */
#if defined(_IX_RM_IMPL_HARDWARE_)

#if (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_)

/* Just dereference the memory pointer and assign the value */
#define IX_RM_MEM_UINT8_WRITE( \
                               arg_pMemoryAddr, \
                               arg_Value \
                             ) \
                             (*((volatile ix_uint8*)(arg_pMemoryAddr)) = (ix_uint8)(arg_Value))


#else /* Westport hardware */

/* Write 32 bits at a time */
#define IX_RM_MEM_UINT8_WRITE( \
                               arg_pMemoryAddr, \
                               arg_Value \
                             ) \
                             (*(volatile ix_uint32*)((ix_uint32)(arg_pMemoryAddr) & 0xfffffffc) = \
                                 ((*(ix_uint32*)((ix_uint32)(arg_pMemoryAddr) & 0xfffffffc)) & \
                                 ((1 << (((ix_uint32)(arg_pMemoryAddr) & 0x00000003) << 3)) - 1)) | \
                                 ((ix_uint32)(arg_Value) << (((ix_uint32)(arg_pMemoryAddr) & 0x00000003) << 3)))


#endif /* (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) */

#else

/**
 * NAME: _ix_rm_mem_uint8_write
 *
 * DESCRIPTION: This function is provided as a IX_RM_MEM_UINT8_WRITE macro support function.
 *          This function will write the 8 bit unsigned arg_Value into the location referred 
 *          by arg_pToAddr. No memory alignment is required.  
 * 
 * 
 * @Param:  - IN arg_pToAddr - represents the memory address where the value should be written.
 * @Param:  - IN arg_Value - represents the value that should be written at the location specified by
 *          arg_pToAddr.
 * 
 * @Return: IX_ERROR_SUCCESS if successful of an error code for failure.
 */
IX_EXPORT_FUNCTION 
ix_error _ix_rm_mem_uint8_write( 
                                 ix_uint8* arg_pToAddr, 
                                 ix_uint8 arg_Value 
                               );

/* Invoke the memory write function */

/**
 * We are using the comma operator in order to have the macro return the 8 bit unsigned value.
 */
#define IX_RM_MEM_UINT8_WRITE( \
                               arg_pMemoryAddr, \
                               arg_Value \
                             ) \
                             (_ix_rm_mem_uint8_write((ix_uint8*)(arg_pMemoryAddr), \
                             (ix_uint8)(arg_Value)), (ix_uint8)(arg_Value))

#endif /* end defined(_IX_RM_IMPL_HARDWARE_) */



/**
 * MACRO NAME: IX_RM_MEM_UINT16_WRITE
 *
 * DESCRIPTION: This macro is provided for high portablity between simulation
 *          code and hardware code. If the user is using this macro instead of
 *          direct memory dereference, then the code written for hardware mode
 *          will compile and run in simulation mode and viceversa. The macro 
 *          will write a 16 bit unsigned value to the memory location specified.
 *          The passed memory address where to write to, should be 2 bytes aligned or the
 *          results might be unpredictable.
 *
 * @Param:  - IN arg_pMemoryAddr - represents the address of the memory location to be read.
 *          Should be of the type ix_uint16*. The address should be 2 bytes aligned.
 * @Param:  - IN arg_Value - The value to be written to the specified location. 
 *          Should be of the type ix_uint16.
 *
 * @Return: a ix_uint16 representing the value written at the specified location.
 */
#if defined(_IX_RM_IMPL_HARDWARE_)

#if (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_)

/* Just dereference the memory pointer and assign the value */
#define IX_RM_MEM_UINT16_WRITE( \
                                arg_pMemoryAddr, \
                                arg_Value \
                              ) \
                              (*((volatile ix_uint16*)(arg_pMemoryAddr)) = (ix_uint16)(arg_Value))

#else /* Westport hardware */

/* Write 32 bits at a time */
#define IX_RM_MEM_UINT16_WRITE( \
                               arg_pMemoryAddr, \
                               arg_Value \
                             ) \
                             (*(volatile ix_uint32*)((ix_uint32)(arg_pMemoryAddr) & 0xfffffffd) = \
                                 ((*(ix_uint32*)((ix_uint32)(arg_pMemoryAddr) & 0xfffffffd)) & \
                                 ((1 << (((ix_uint32)(arg_pMemoryAddr) & 0x00000002) << 3)) - 1)) | \
                                 ((ix_uint32)(arg_Value) << (((ix_uint32)(arg_pMemoryAddr) & 0x00000002) << 3)))

#endif /* (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) */

#else

/**
 * NAME: _ix_rm_mem_uint16_write
 *
 * DESCRIPTION: This function is provided as a IX_RM_MEM_UINT16_WRITE macro support function.
 *          This function will write the 16 bit unsigned arg_Value into the location referred 
 *          by arg_pToAddr. The passed memory address where to write to, should be 2 bytes aligned or the
 *          results might be unpredictable. In DEBUG mode the alignment will be checked but
 *          not in RELEASE.
 * 
 * @Param:  - IN arg_pToAddr - represents the memory address where the value should be written.
 *          The passed memory address where to write to, should be 2 bytes aligned or the
 *          results might be unpredictable.
 *
 * @Param:  - IN arg_Value - represents the value that should be written at the location specified by
 *          arg_pToAddr.
 * 
 * @Return: IX_ERROR_SUCCESS if successful of an error code for failure.
 */
IX_EXPORT_FUNCTION 
ix_error _ix_rm_mem_uint16_write( 
                                  ix_uint16* arg_pToAddr, 
                                  ix_uint16 arg_Value 
                                );
/* Invoke the memory write function */

/**
 * We are using the comma operator in order to have the macro return the 16 bit unsigned value.
 */
#define IX_RM_MEM_UINT16_WRITE( \
                                arg_pMemoryAddr, \
                                arg_Value \
                              ) \
                              (_ix_rm_mem_uint16_write(((ix_uint16*)(arg_pMemoryAddr)), \
                              ((ix_uint16)(arg_Value))), ((ix_uint16)(arg_Value)))

#endif /* end defined(_IX_RM_IMPL_HARDWARE_) */



/**
 * MACRO NAME: IX_RM_MEM_UINT32_WRITE
 *
 * DESCRIPTION: This macro is provided for high portablity between simulation
 *          code and hardware code. If the user is using this macro instead of
 *          direct memory dereference, then the code written for hardware mode
 *          will compile and run in simulation mode and viceversa. The macro 
 *          will write a 32 bit unsigned value to the memory location specified.
 *          The passed memory address where to write to, should be 4 bytes aligned or the
 *          results might be unpredictable.
 *
 * @Param:  - IN arg_pMemoryAddr - represents the address of the memory location to be read.
 *          Should be of the type ix_uint32*. The address should be 4 bytes aligned.
 * @Param:  - IN arg_Value - The value to be written to the specified location. 
 *          Should be of the type ix_uint32.
 *
 * @Return: a ix_uint32 representing the value written at the specified location.
 */
#if defined(_IX_RM_IMPL_HARDWARE_)

/* Just dereference the memory pointer and assign the value */
#define IX_RM_MEM_UINT32_WRITE( \
                                arg_pMemoryAddr, \
                                arg_Value \
                              ) \
                              (*((volatile ix_uint32*)(arg_pMemoryAddr)) = (ix_uint32)(arg_Value))

#else

/**
 * NAME: _ix_rm_mem_uint32_write
 *
 * DESCRIPTION: This function is provided as a IX_RM_MEM_UINT32_WRITE macro support function.
 *          This function will write the 32 bit unsigned arg_Value into the location referred 
 *          by arg_pToAddr. The passed memory address where to write to, should be 4 bytes aligned or the
 *          results might be unpredictable. In DEBUG mode the alignment will be checked but
 *          not in RELEASE.
 * 
 * @Param:  - IN arg_pToAddr - represents the memory address where the value should be written.
 *          The passed memory address where to write to, should be 4 bytes aligned or the
 *          results might be unpredictable.
 * @Param:  - IN arg_Value - represents the value that should be written at the location specified by
 *          arg_pToAddr.
 * 
 * @Return: IX_ERROR_SUCCESS if successful of an error code for failure.
 */
IX_EXPORT_FUNCTION 
ix_error _ix_rm_mem_uint32_write( 
                                  ix_uint32* arg_pToAddr, 
                                  ix_uint32 arg_Value 
                                );


/* Invoke the memory write function */

/**
 * We are using the comma operator in order to have the macro return the 32 bit unsigned value.
 */
#define IX_RM_MEM_UINT32_WRITE( \
                                arg_pMemoryAddr, \
                                arg_Value \
                              ) \
                              (_ix_rm_mem_uint32_write((ix_uint32*)(arg_pMemoryAddr), \
                              (ix_uint32)(arg_Value)), (ix_uint32)(arg_Value))

#endif /* end defined(_IX_RM_IMPL_HARDWARE_) */



/**
 * MACRO NAME: IX_RM_MEM_UINT64_WRITE
 *
 * DESCRIPTION: This macro is provided for high portablity between simulation
 *          code and hardware code. If the user is using this macro instead of
 *          direct memory dereference, then the code written for hardware mode
 *          will compile and run in simulation mode and viceversa. The macro 
 *          will write a 64 bit unsigned value to the memory location specified.
 *          The passed memory address where to write to, should be 8 bytes aligned or the
 *          results might be unpredictable.
 *
 * @Param:  - IN arg_pMemoryAddr - represents the address of the memory location to be read.
 *          Should be of the type ix_uint64*. The address should be 8 bytes aligned.
 * @Param:  - IN arg_Value - The value to be written to the specified location. 
 *          Should be of the type ix_uint64.
 *
 * @Return: a ix_uint64 representing the value written at the specified location.
 */
#if defined(_IX_RM_IMPL_HARDWARE_)

/* Just dereference the memory pointer and assign the value */
#define IX_RM_MEM_UINT64_WRITE( \
                                arg_pMemoryAddr, \
                                arg_Value \
                              ) \
                              (*((volatile ix_uint64*)(arg_pMemoryAddr)) = (ix_uint64)(arg_Value))

#else

/**
 * NAME: _ix_rm_mem_uint64_write
 *
 * DESCRIPTION: This function is provided as a IX_RM_MEM_UINT64_WRITE macro support function.
 *          This function will write the 64 bit unsigned arg_Value into the location referred 
 *          by arg_pToAddr. The passed memory address where to write to, should be 8 bytes aligned or the
 *          results might be unpredictable. In DEBUG mode the alignment will be checked but
 *          not in RELEASE.
 * 
 * @Param:  - IN arg_pToAddr - represents the memory address where the value should be written.
 *          The passed memory address where to write to, should be 8 bytes aligned or the
 *          results might be unpredictable.
 * @Param:  - IN arg_Value - represents the value that should be written at the location specified by
 *          arg_pToAddr.
 * 
 * @Return: IX_ERROR_SUCCESS if successful or avalid error token for failure.
 */
IX_EXPORT_FUNCTION 
ix_error _ix_rm_mem_uint64_write( 
                                 ix_uint64* arg_pToAddr, 
                                 ix_uint64 arg_Value 
                               );

/* Invoke the memory write function */

/**
 * We are using the comma operator in order to have the macro return the 64 bit unsigned value.
 */
#define IX_RM_MEM_UINT64_WRITE( \
                                arg_pMemoryAddr, \
                                arg_Value \
                              ) \
                              (_ix_rm_mem_uint64_write((ix_uint64*)(arg_pMemoryAddr), \
                              (ix_uint64)(arg_Value)), (ix_uint64)(arg_Value))

#endif /* end defined(_IX_RM_IMPL_HARDWARE_) */


#if (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) 

/**
 * NAME: ix_rm_get_phys_offset
 *
 * DESCRIPTION: Get the physical offset into the channel memory in bytes  and the physical
 *          address for the passed virtual memory address. The function also returns the 
 *          type of memory and the channel corresponding to the input address. If any of the
 *          OUT parameters are NULL then that piece of data will not be returned.
 * 
 * @Param:  - IN arg_pMemory - represents the adress of virtual memory for which we want to get
 *          the physical offset.
 * @Param:  - OUT arg_pMemType - represents the location where to store the corresponding memory
 *          type.
 * @Param:  - OUT arg_pMemChannel - represents the location where to store the corresponding
 *          memory channel.
 * @Param:  - OUT arg_pOffset - represents the address where to store the physical offset into the channel.
 * @Param:  - OUT arg_pPhysicalAddress - represents the address where to store the physical address.
 *
 * @Return: IX_ERROR_SUCCESS if successful or a valid ix_error for failure.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_get_phys_offset( 
                                const void* arg_pMemory, 
                                ix_memory_type* arg_pMemType, 
                                ix_uint32* arg_pMemChannel, 
                                ix_uint32* arg_pOffset,
                                void** arg_pPhysicalAddress
                              );

#else /* Westport hardware */

/**
 * NAME: ix_rm_get_phys_offset
 *
 * DESCRIPTION: Get the physical offset into the channel memory in bytes and
 *	the physical address for the passed virtual memory address. The 
 *	function also returns the type of memory and the channel corresponding 
 *	to the input address. If any of the OUT parameters are NULL then that 
 *	piece of data will not be returned. For the North bank memory, 
 *	if the passed offset type is IX_MEMORY_OFFSET_TYPE_ALIASED, then the 
 *	returned offset is relative to the South alias window and the physical 
 *	address is relative to the South bank, otherwise the offset is relative 
 *	to the start of the memory channel in the North bank and the physical 
 *	address is relative to the North bank.
 * 
 * @Param:  - IN arg_pMemory - represents the adress of virtual memory for 
 *	which we want to get the physical offset.
 * @Param:  - IN arg_OffsetType - represents the type of the offset and 
 *	physical address to be returned 
 * @Param:  - OUT arg_pMemType - represents the location where to store the
 *	corresponding memory type.
 * @Param:  - OUT arg_pMemChannel - represents the location where to store 
 *	the corresponding memory channel.
 * @Param:  - OUT arg_pOffset - represents the address where to store the
 *	physical offset into the channel.
 * @Param:  - OUT arg_pPhysicalAddress - represents the address where to 
 *	store the physical address.
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_get_phys_offset( 
                                const void* arg_pMemory,
                                ix_memory_offset_type arg_OffsetType, 
                                ix_memory_type* arg_pMemType, 
                                ix_uint32* arg_pMemChannel, 
                                ix_uint32* arg_pOffset,
                                void** arg_pPhysicalAddress
                              );

#endif /* (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) */


#if (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) 

/**
 * NAME: ix_rm_get_virtual_address
 *
 * DESCRIPTION: The function returns the virtual pointer into memory given the physical offset of the
 *          memory buffer expressed in bytes.
 * 
 * @Param:  - IN arg_MemType - represents the type of memory in question.
 * @Param:  - IN arg_MemChannel - represents the channel for which we want the virtual address.
 * @Param:  - IN arg_Offset - represents the physical offset
 * @Param:  - OUT arg_pMemoryAddr - represents the address where the virtual address should be stored. 
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_get_virtual_address( 
                                    ix_memory_type arg_MemType, 
                                    ix_uint32 arg_MemChannel, 
                                    ix_uint32 arg_Offset, 
                                    ix_uint8** arg_pMemoryAddr 
                                  );

#else /* Westport hardware */

/**
 * NAME: ix_rm_get_virtual_address
 *
 * DESCRIPTION: The function returns the virtual pointer into memory given 
 *	the physical offset of the memory buffer expressed in bytes. 
 *	For the North bank memory, if the passed offset type is 
 *	IX_MEMORY_OFFSET_TYPE_ALIASED, then the offset is considered to be 
 *	relative to the South alias window, otherwise the offset is considered 
 *	to be relative to the start of the memory channel in the North bank.
 * 
 * @Param:  - IN arg_MemType - represents the type of memory in question.
 * @Param:  - IN arg_MemChannel - represents the channel for which we want 
 *	the virtual address.
 * @Param:  - IN arg_Offset - represents the physical offset in bytes
 * @Param:  - IN arg_OffsetType - represents the type of the specified offset 
 * @Param:  - OUT arg_pMemoryAddr - represents the address where the virtual 
 *	address should be stored. 
 *
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_get_virtual_address( 
                                    ix_memory_type arg_MemType, 
                                    ix_uint32 arg_MemChannel, 
                                    ix_uint32 arg_Offset, 
                                    ix_memory_offset_type arg_OffsetType, 
                                    ix_uint8** arg_pMemoryAddr 
                                  );

#endif /* (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) */



#if defined(_IX_RM_DEBUG_)


/**
 * NAME: ix_rm_mem_status_print
 *
 * DESCRIPTION: This function will a print snapshot of a common memory block based on the type and
 *          channel. All allocated and free memory cells will be listed.
 * 
 * @Param:  - IN arg_MemType - represents the type of memory in question.
 * @Param:  - IN arg_MemChannel - represents the channel for which we want to print the status information.
 *
 * @Return: IX_ERROR_SUCCESS if successful or a valid ix_error token for failure.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_mem_status_print(
                                 ix_memory_type arg_MemType, 
                                 ix_uint32 arg_MemChannel 
                               );


#endif /* end defined(_IX_RM_DEBUG_) */

#if (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) 

#if !defined( _IX_RM_IMPL_HARDWARE_ )

/**
 * NAME: ix_rm_get_sram_channel_start_address
 *
 * DESCRIPTION: This function will return the base address of the specified SRAM channel
 *
 * @Param:  - IN arg_Channel - represents the SRAM memory channel in question.
 * 
 */
IX_EXPORT_FUNCTION 
void* ix_rm_get_sram_channel_start_address(ix_uint32 arg_Channel);

#endif /* defined( _IX_RM_IMPL_HARDWARE_ ) */

#endif /* (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) */

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__MEMORY_H__) */
