/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = LIBRARY
 *      Resource Manager Library
 *
 * = MODULE
 *      Library Interface
 *
 * = FILENAME
 *      sync_mechanism.h
 *
 * = DESCRIPTION
 *      This file describes a synchronization mechanism that allows the user to
 *      specify her/his own implementation of the synchronization for the exported
 *      library constructs.
 *
 * = AUTHOR
 *      Adrian Georgescu
 *      adrian.georgescu@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      12/17/2001 4:04:11 PM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: sync_mechanism.h,v 1.3 2002/09/18 17:27:15 lbmiller Exp $
 */

#if !defined(__SYNC_MECHANISM_H__)
#define __SYNC_MECHANISM_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */



/**
 * We provide a generic synchronization adapter, where the users may provide their
 * own synchronization objects. The adapter provides placeholders for two functions: 
 * lock and unlock that have to be provided by the users. These functions must provide
 * the logic of the synchronization mechanism. It is up to the user what mechanism to 
 * use (e.g. semaphore, critical section, mutex, etc.) 
 * The adapter can be associated with some entity that needs to be synchronized and 
 * provides the flexibility that synchronization may be defined externally.
 */

/**
 * TYPENAME: ix_sa_lock_ptr
 * 
 * DESCRIPTION: Defines a lock function for a user-provided synchronization mechanism.
 *
 */
typedef 
void (* ix_sa_lock_ptr)( 
                         void* arg_pContext 
                       );

/**
 * TYPENAME: ix_sa_unlock_ptr
 * 
 * DESCRIPTION: Defines an unlock function for a user-provided synchronization mechanism.
 *
 */
typedef 
void (* ix_sa_unlock_ptr)( 
                           void* arg_pContext 
                         );


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__SYNC_MECHANISM_H__) */
