/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = LIBRARY
 *      Resource Manager Library
 *
 * = MODUE
 *      System API
 *
 * = FILENAME
 *      system.h
 *
 * = DESCRIPTION
 *      This file will define all exported data types used in the System API module. Also
 *      will declare all function prototypes of the exported interface. 
 *
 * = AUTHOR
 *       Adrian Georgescu
 *       adrian.georgescu@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      10/16/2001 2:33:36 PM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: system.h,v 1.38 2003/10/10 00:30:20 ageorges Exp $
 */

#if !defined(__SYSTEM_H__)
#define __SYSTEM_H__

#include "memory.h"

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */


/**
 * TYPENAME: ix_rm_error_code
 * 
 * DESCRIPTION: This enumerated type will describe all internal Resource Manager error codes
 *
 */
typedef enum ix_e_rm_error_code
{
    IX_RM_ERROR_FIRST = IX_ERROR_MAKE_GROUP(RESOURCE_MANAGER),
    IX_RM_ERROR_SUCCESS = IX_RM_ERROR_FIRST,
    /* assertion error code */
    IX_RM_ERROR_ASSERTION,

    /* memory manager error codes */
    IX_RM_ERROR_MEMORY_INIT_FAILED,
    IX_RM_ERROR_MEMORY_FINI_FAILED,
    IX_RM_ERROR_BAD_MEMORY_TYPE,
    IX_RM_ERROR_CANNOT_CREATE_CELL,
    IX_RM_ERROR_DATA_ALREADY_RELEASED,
    IX_RM_ERROR_BAD_MEMORY_ADDRESS,
    IX_RM_ERROR_MEMORY_IN_USE,
    IX_RM_ERROR_NULL_MEMORY_ADDRESS,
    IX_RM_ERROR_CANNOT_ALLOCATE_SIZE,
    IX_RM_ERROR_CANNOT_RESERVE_MEMORY,
    IX_RM_ERROR_CELL_SPLIT_FAILED,
    IX_RM_ERROR_MEMORY_BAD_OFFSET,

    /* general library error codes */
    IX_RM_ERROR_LIB_INIT_FAILED,
    IX_RM_ERROR_LIB_FINI_FAILED,

#if (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) 

    /* communication API error codes */
    IX_RM_ERROR_PKT_DISP_ENG_INIT_FAILED,
    IX_RM_ERROR_MSG_DISP_ENG_INIT_FAILED,
    IX_RM_ERROR_PKT_DISP_ENG_FINI_FAILED,
    IX_RM_ERROR_MSG_DISP_ENG_FINI_FAILED,
    IX_RM_ERROR_PKT_DISP_ENG_START_FAILED,
    IX_RM_ERROR_MSG_DISP_ENG_START_FAILED,
    IX_RM_ERROR_PKT_DISP_ENG_STOP_FAILED,
    IX_RM_ERROR_MSG_DISP_ENG_STOP_FAILED,

    IX_RM_ERROR_COMM_API_INIT_FAILED,
    IX_RM_ERROR_COMM_API_FINI_FAILED,
    IX_RM_ERROR_COMM_QUEUE_FULL,
    IX_RM_ERROR_COMM_QUEUE_EMPTY,
    IX_RM_ERROR_COMM_NOTIFICATION_ELEM_ALLOCATION_FAILED,
    IX_RM_ERROR_COMM_SEM_POOL_INIT_FAILED,
    IX_RM_ERROR_COMM_SEM_NODE_ALLOCATION_FAILED,
    IX_RM_ERROR_COMM_SEM_GET_FAILED,
    IX_RM_ERROR_COMM_ISR_SETUP_FAILED,
    IX_RM_ERROR_COMM_DISPATCH_TH_INIT_FAILED,
    IX_RM_ERROR_COMM_EP_DATA_ALLOCATION_FAILED,
    IX_RM_ERROR_COMM_EP_SEMA_CREATION_FAILED,
    IX_RM_ERROR_COMM_EP_MGR_INIT_FAILED,
    IX_RM_ERROR_COMM_EP_MGR_FINI_FAILED,
    IX_RM_ERROR_COMM_EP_PCI_PROXIES_INIT_FAILED,
    IX_RM_ERROR_COMM_EP_PCI_PROXIES_FINI_FAILED,
    IX_RM_ERROR_COMM_EP_REMOTE,
    IX_RM_ERROR_COMM_EP_UBLOCK,
    IX_RM_ERROR_COMM_EP_CC,
    IX_RM_ERROR_COMM_EP_RECV_CALLBACK,
    IX_RM_ERROR_COMM_EP_RECV_GET_SELECT,
    IX_RM_ERROR_COMM_EP_MULTIPLE_CONSUMER,
    IX_RM_ERROR_COMM_EP_CC_MODE_ENABLED,
    IX_RM_ERROR_COMM_EP_CC_MODE_DISABLED,

    IX_RM_ERROR_REMOTE_COMM_API_INIT_FAILED,
    IX_RM_ERROR_REMOTE_COMM_API_FINI_FAILED,
    IX_RM_ERROR_REMOTE_COMM_SERVICE_ACTIVE,

    IX_RM_ERROR_COMM_PCI_INIT_FAILED,
    IX_RM_ERROR_COMM_PCI_FINI_FAILED,
    IX_RM_ERROR_COMM_PCI_GET_REMOTE_DEV_FAILED,
    IX_RM_ERROR_COMM_PCI_TX_INIT_FAILED,
    IX_RM_ERROR_COMM_PCI_RX_INIT_FAILED,
    IX_RM_ERROR_COMM_PCI_TX_START_FAILED,    
    IX_RM_ERROR_COMM_PCI_RX_START_FAILED,
    IX_RM_ERROR_COMM_PCI_TX_STOP_FAILED,
    IX_RM_ERROR_COMM_PCI_RX_STOP_FAILED,
    IX_RM_ERROR_COMM_PCI_RX_FINI_FAILED,
    IX_RM_ERROR_COMM_PCI_TX_FINI_FAILED,
    IX_RM_ERROR_COMM_PCI_INT_CONNECT_FAILED,
    IX_RM_ERROR_COMM_PCI_INT_ENABLE_FAILED,
    IX_RM_ERROR_COMM_PCI_INT_DISABLE_FAILED,
    IX_RM_ERROR_COMM_PCI_RX_TH_INIT_FAILED,
    IX_RM_ERROR_COMM_PCI_TX_TH_INIT_FAILED,
    IX_RM_ERROR_COMM_PCI_RX_SEMA_CREATION_FAILED,
    IX_RM_ERROR_COMM_PCI_TX_SEMA_CREATION_FAILED,
    IX_RM_ERROR_COMM_PCI_TX_CTRL_INFO_ALLOC_FAILED,
    IX_RM_ERROR_COMM_PCI_TX_CTRL_INFO_GET_PHYS_OFFSET_FAILED,
    IX_RM_ERROR_COMM_PCI_RX_HW_FL_CREATE_FAILED,
    IX_RM_ERROR_COMM_PCI_RX_SW_FL_CREATE_FAILED,
    IX_RM_ERROR_COMM_PCI_TX_CTRL_INFO_FREE_FAILED,
    IX_RM_ERROR_COMM_PCI_TX_SEMA_FINI_FAILED,
    IX_RM_ERROR_COMM_PCI_RX_SW_FL_DELETE_FAILED,
    IX_RM_ERROR_COMM_PCI_RX_HW_FL_DELETE_FAILED,
    IX_RM_ERROR_COMM_PCI_RX_SEMA_FINI_FAILED,
    IX_RM_ERROR_COMM_PCI_UNSUPPORTED_MEMORY_TYPE,

    /* hardware API error codes */
    IX_RM_ERROR_HW_API_INIT_FAILED,
    IX_RM_ERROR_HW_API_FINI_FAILED,
    IX_RM_ERROR_HW_QARRAY_MGR_ALLOC_FAILED,
    IX_RM_ERROR_HW_QARRAY_MGR_DEALLOC_FAILED,
    IX_RM_ERROR_HW_QUEUE_CREATE_FAILED,
    IX_RM_ERROR_HW_RING_CREATE_FAILED,
    IX_RM_ERROR_HW_RING_DELETE_FAILED,
    IX_RM_ERROR_HW_SRAM_RING_PUT_FAILED,
    IX_RM_ERROR_HW_SCRATCH_RING_PUT_FAILED,
    IX_RM_ERROR_HW_SRAM_RING_GET_FAILED,
    IX_RM_ERROR_HW_SCRATCH_RING_GET_FAILED,

    /* buffer API error codes */
    IX_RM_ERROR_BUF_API_INIT_FAILED,
    IX_RM_ERROR_BUF_API_FINI_FAILED,
    IX_RM_ERROR_BUF_QUEUES_ALLOC_FAILED,
    IX_RM_ERROR_BUF_HANDLE_INVALID,
    IX_RM_ERROR_BUF_FREE_LIST_INVALID,
    IX_RM_ERROR_BUF_FREE_LIST_ID_INVALID,
    IX_RM_ERROR_BUF_FREE_LIST_BAD_SIZE,
    IX_RM_ERROR_BUF_FREE_LIST_SIZE_TOO_LARGE,
    IX_RM_ERROR_BUF_FREE_LIST_BAD_META_SIZE,
    IX_RM_ERROR_BUF_FREE_LIST_ALL_IN_USE,
    IX_RM_ERROR_BUF_FREE_LIST_META_ALLOCATION_FAILED,

#if defined(_IX_RM_SPLIT_META_DATA_)

    IX_RM_ERROR_BUF_FREE_LIST_INTERNAL_META_ALLOCATION_FAILED,

#endif /* defined(_IX_RM_SPLIT_META_DATA_) */

#if defined (_IX_RM_BUFFER_DEBUG_)

    IX_RM_ERROR_BUF_FREE_LIST_DEBUG_INFO_ALLOCATION_FAILED,

#endif /* defined (_IX_RM_BUFFER_DEBUG_) */

    IX_RM_ERROR_BUF_FREE_LIST_DATA_ALLOCATION_FAILED,
    IX_RM_ERROR_BUF_FREE_LIST_FINI_FAILED,
    IX_RM_ERROR_BUF_FREE_LIST_CREATION_FAILED,
    IX_RM_ERROR_BUF_NO_FREE_BUFFERS,
    IX_RM_ERROR_BUF_ALREADY_LINKED,

    /* configuration property API error codes */
    IX_RM_ERROR_CP_API_INIT_FAILED,
    IX_RM_ERROR_CP_API_FINI_FAILED,
    IX_RM_ERROR_CP_HANDLE_NULL,
    IX_RM_ERROR_CP_NAME_TOO_LONG,
    IX_RM_ERROR_CP_NAME_NULL,
    IX_RM_ERROR_CP_BAD_NAME,
    IX_RM_ERROR_CP_ALREADY_EXISTS,
    IX_RM_ERROR_CP_CREATION_FAILED,
    IX_RM_ERROR_CP_FAILED_TO_ACCESS_ATTRIBUTES,
    IX_RM_ERROR_CP_CONSTANT_DELETE,
    IX_RM_ERROR_CP_CONSTANT_SET_VALUE,
    IX_RM_ERROR_CP_FAILED_TO_LINK,
    IX_RM_ERROR_CP_DATA_UINT32,
    IX_RM_ERROR_CP_DATA_OPAQUE,
    IX_RM_ERROR_CP_BUFFER_GET_NEXT_FAILED,
    IX_RM_ERROR_CP_BUFFER_TOO_SMALL,
    IX_RM_ERROR_CP_FAILED_TO_ALLOCATE_NOTIFICATION_TOKEN,
    IX_RM_ERROR_CP_NOTIFICATION_ACTIVE,
    IX_RM_ERROR_CP_PROPERTY_NOT_FOUND,

    /* 64 bit counters API error codes */
    IX_RM_ERROR_64BIT_COUNTER_INIT_FAILED,
    IX_RM_ERROR_64BIT_COUNTER_FINI_FAILED,
    IX_RM_ERROR_64BIT_COUNTER_MEMORY_INIT_FAILED,
    IX_RM_ERROR_64BIT_COUNTER_BLOCK_INIT_FAILED,
    IX_RM_ERROR_64BIT_COUNTER_ALL_BLOCKS_IN_USE,
    IX_RM_ERROR_64BIT_COUNTER_INACTIVE,
    IX_RM_ERROR_64BIT_COUNTER_HANDLE_INVALID,
    IX_RM_ERROR_64BIT_COUNTER_UPDATE_TH_INIT_FAILED,
    IX_RM_ERROR_64BIT_COUNTER_UPDATE_SEM_INIT_FAILED,
    IX_RM_ERROR_64BIT_COUNTER_BAD_OVERFLOW_TIME,
    IX_RM_ERROR_64BIT_COUNTER_BAD_INTERNAL_MEMORY,
    IX_RM_ERROR_64BIT_COUNTER_NOT_ENOUGH_COUNTERS,
    IX_RM_ERROR_64BIT_COUNTER_SWAP_MEM_ALLOC_FAILED,

    /* microengine API error codes */
    IX_RM_ERROR_ME_API_INIT_FAILED,
    IX_RM_ERROR_ME_API_FINI_FAILED,
    IX_RM_ERROR_ME_MAP_UCODE_FAILED,
    IX_RM_ERROR_ME_LOAD_UCODE_FAILED,
    IX_RM_ERROR_ME_LOAD_TO_USTORE_FAILED,
    IX_RM_ERROR_ME_LOAD_UCODE_FROM_WB_FAILED,
    IX_RM_ERROR_ME_START_FAILED,
    IX_RM_ERROR_ME_STOP_FAILED,
    IX_RM_ERROR_ME_RESET_FAILED,
    IX_RM_ERROR_ME_ENABLE_CTX_FAILED,
    IX_RM_ERROR_ME_DISABLE_CTX_FAILED,
    IX_RM_ERROR_ME_REGISTER_INTERNAL_SYMBOLS_FAILED,
    IX_RM_ERROR_ME_PATCH_SYMBOLS_FAILED,

    /* hash API error codes */
    IX_RM_ERROR_HASH_API_INIT_FAILED,
    IX_RM_ERROR_HASH_API_FINI_FAILED,

    /* ME\XScale locks codes */
    IX_RM_ERROR_ME_XSCALE_LOCK_INIT_FAILED,
    IX_RM_ERROR_ME_XSCALE_LOCK_FINI_FAILED,
    IX_RM_ERROR_ME_XSCALE_LOCK_MEM_ALLOC_FAILED,
    IX_RM_ERROR_ME_XSCALE_LOCK_OUT_OF_LOCKS,
    IX_RM_ERROR_ME_XSCALE_LOCK_NOT_ACTIVE,
    IX_RM_ERROR_ME_XSCALE_LOCK_IS_LOCKED,
    IX_RM_ERROR_ME_XSCALE_LOCK_BAD_MEMORY,
    IX_RM_ERROR_ME_XSCALE_LOCK_IS_LOCKED_BY_ME,

#endif /* (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) */

    /* unknown error code */
    IX_RM_ERROR_UNKNOWN,
    IX_RM_ERROR_LAST
} ix_rm_error_code;


/**
 * TYPENAME: ix_memory_reserved_area
 * 
 * DESCRIPTION: This structure contains the description of an memory area that needs
 *  to be prereserved at the initialization time. We need this because the microcode
 *  needs memory to be used whithout knowing about the RM existence.
 *
 */
typedef struct ix_s_memory_reserved_area
{
    /* the memory type area that needs to be reserved */
    ix_memory_type  m_Type;
    /* the channel of the memory that needs to be reserved */
    ix_uint32       m_Channel;
    /* the offset in the channel of the memory area to be reserved expressed in bytes */
    ix_uint32       m_StartOffset; 

#if (_IX_HARDWARE_TYPE_ == _IX_HW_WESTPORT_) 

    /* Alaised or non aliased offset to be considered */
    ix_memory_offset_type   m_OffsetType;

#endif /* (_IX_HARDWARE_TYPE_ == _IX_HW_WESTPORT_) */

    /* the size of the memory that needs to be reserved */
    ix_uint32       m_Size;
} ix_memory_reserved_area;



/**
 * Global reference to the prereserved memory areas
 */
extern ix_memory_reserved_area*    g_pReservedMemoryAreas;


/**
 * NAME: ix_rm_version_get_string
 *
 * DESCRIPTION: This function will return the product's version string
 * 
 * @Return: Returns a string representing the product version string.
 */
IX_EXPORT_FUNCTION 
const char* ix_rm_version_get_string(void);



/**
 * NAME: ix_rm_error_get_string
 *
 * DESCRIPTION: This is a support function that retrieves a string represenation of internal
 *          error codes. Internal error codes are passed within ix_error return value for all
 *          interface functions. This function can be used to get a string representation of
 *          that error code.
 * 
 * @Param:  - IN arg_Error - an internal error code for which we want a string representation.
 *
 * @Return: The string representation corresponding to the error code in cause.
 */
IX_EXPORT_FUNCTION 
const char* ix_rm_error_get_string( 
                                    ix_rm_error_code arg_Error 
                                  );


#if (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) 

/**
 * TYPENAME: ix_phy_type
 * 
 * DESCRIPTION: This enumerated type specifies the values for physical layer interfaces
 *          that could be present on the board.
 */
typedef enum ix_e_phy_type
{
    IX_PHY_TYPE_FIRST = 0,
    IX_PHY_TYPE_OTHER = IX_PHY_TYPE_FIRST,
    IX_PHY_TYPE_IX_BUS,
    IX_PHY_TYPE_UTOPIA_3,
    IX_PHY_TYPE_SPI_4,

    IX_PHY_TYPE_LAST
} ix_phy_type;


/**
 * TYPENAME: ix_port_type
 * 
 * DESCRIPTION: This enumerated type specifies the various link layer interfaces present 
 *          on the board.
 */
typedef enum ix_e_port_type
{
    IX_PORT_TYPE_FIRST = 0,
    IX_PORT_TYPE_OTHER = IX_PORT_TYPE_FIRST,
    IX_PORT_TYPE_FAST_ETHERNET,
    IX_PORT_TYPE_GIGABIT_ETHERNET,
    IX_PORT_TYPE_ATM,
    IX_PORT_TYPE_POS,
    IX_PORT_TYPE_FRAME_RELAY,
    IX_PORT_TYPE_CSIX_FABRIC,
    IX_PORT_TYPE_SOFTWARE_LOOPBACK,
    IX_PORT_TYPE_LAST
} ix_port_type;




/**
 * TYPENAME: ix_port_speed
 * 
 * DESCRIPTION: This enumerated type describes the port speed.
 *
 */
typedef enum ix_e_port_speed
{
    IX_PORT_SPEED_FIRST = 0,
    IX_PORT_SPEED_ONE = IX_PORT_SPEED_FIRST,
    IX_PORT_SPEED_TWO,
    IX_PORT_SPEED_THREE,
    IX_PORT_SPEED_LAST
} ix_port_speed;



/**
 * TYPENAME: ix_port
 * 
 * DESCRIPTION: This structure defines a link layer interface on the board.
 *
 */
typedef struct ix_s_port
{
    ix_uint32       m_Number; /* Port number */
    ix_port_type    m_LinkType; /* Link Layer type */
    ix_phy_type     m_PhyType; /* Physical Layer type */
    ix_port_speed   m_PortSpeed; /* Port speed */
} ix_port;


/**
 * DESCRIPTION: This symbol defines the maximum number of physical ports.
 *
 */
#define IX_MAX_NUM_PORTS    265L


/**
 * TYPENAME: ix_subsystem_type
 * 
 * DESCRIPTION: This enumerated type describes all the possible subsystem types
 *
 */
typedef enum ix_e_subsystem_type
{
    IX_SUBSYSTEM_TYPE_FIRST = 0,
    IX_SUBSYSTEM_TYPE_INGRESS = IX_SUBSYSTEM_TYPE_FIRST,
    IX_SUBSYSTEM_TYPE_EGRESS,
    IX_SUBSYSTEM_TYPE_LAST
} ix_subsystem_type;

/**
 * TYPENAME: ix_sys_config
 * 
 * DESCRIPTION: System/Board configuration. Right now it contains the number 
 *	of ports physically present for this board or system. It also contains 
 *	the different types of memory number of channels, the number of 
 *	microengines as well as the chip identifier.
 *
 */
typedef struct ix_s_sys_config
{
    /* This section is constant and can not be modified once initialized */
        /* System identifier into a multi system configuration */
    ix_uint32           m_SystemId; 
    ix_subsystem_type   m_SubsystemType; /* Subsystem type */

        /* The number of microengines in the system */
    ix_uint32           m_NumberOfMicroengines; 
        /* The number of SRAM memory channels */
    ix_uint32           m_NumberOfSramChannels; 
        /* The number of DRAM memory channels */
    ix_uint32           m_NumberOfDramChannels; 
    ix_uint32           m_ChipId; /* The chip identifier */

        /* The actual number of ports present on this system */
    ix_uint32           m_NumberOfPorts; 
        /* Array of physical ports */
    ix_port             m_aPorts[IX_MAX_NUM_PORTS]; 


    /* This is the section that can be modified after initialization */

    ix_uint32           m_Endianess; /* 0 for big and 1 little */
} ix_sys_config;

#endif /* (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) */


/**
 * NAME: ix_rm_init
 *
 * DESCRIPTION: This function initializes the Resource Manager. It should be called before any
 *  other call is made to the Resource Manager. The arg_pReservedAreas parameter is the
 *  beginning of an array with memory areas that need to be prereserved. The end of the array
 *  will be signaled by an entry of size 0. If this parameter is 0 then it signals that there is
 *  no memory area that needs to be prereserved.
 *
 * @Param:  - IN arg_pReservedAreas - points to the beginning of an array of memory areas to be 
 *  prereserved at the initialization time.
 * 
 * @Return: IX_SUCCESS if successful or a valid ix_error for failure.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_init(const ix_memory_reserved_area* arg_pReservedAreas);


/**
 * NAME: ix_rm_term
 *
 * DESCRIPTION: This call should be called when the Resource Manager services are not needed
 *          any more and it will provide all necessary cleanup.
 * 
 * @Return: IX_ERROR_SUCCESS if successful or a valid ix_error for failure.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_term(void);


#if (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) 

/**
 * NAME: ix_rm_sys_config_get
 *
 * DESCRIPTION: This function will retrieve the system specific configuration.  
 * 
 * @Param:  - OUT arg_pSysConfig - this is the location where the system configuration 
 *          will be written.
 *
 * @Return: IX_ERROR_SUCCESS if successful or a valid ix_error for failure.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_sys_config_get( 
                               ix_sys_config* arg_pSysConfig 
                             );


/**
 * NAME: ix_rm_sys_config_set
 *
 * DESCRIPTION: This function will set the system configuration. Only certain members of the
 *          system configuration can be set, the others will be ignored.
 * 
 * @Param:  - IN ix_sys_config - location of the system configuration to be set.
 *
 * @Return: IX_ERROR_SUCCESS if successful or a valid ix_error for failure.
 */
IX_EXPORT_FUNCTION 
ix_error ix_rm_sys_config_set( 
                              const ix_sys_config* arg_pSysConfig 
                            );


/**
 * This array defines the microengine number mapping
 */
extern unsigned char g_aMicroengineNumberMapping[IX_MAX_MICROENGINE_NUMBER];

#endif /* (_IX_HARDWARE_TYPE_ != _IX_HW_WESTPORT_) */

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__SYSTEM_H__) */
