/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation.All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 5
 *
 * = FILENAME
 *      internal_cci_comm.c
 *
 * = DESCRIPTION
 *      End-point code customized to work with CCI so that pending flags and 
 *      engine semaphore are accessed when enqueueing to an empty queue. Also,
 *      ring enqueueing and dequeueing algorithms are changed so that no
 *      locking is required for single-producer/single-consumer operation and
 *      locking is only required for enqueuing for mult-producer/single-
 *      consumer operation. The CCI does not support multi-producer/multi-
 *      consumer operation. This module contains the CCI handler that is 
 *      called by the RM if the CCI has registered the associated comm ID with
 *      the RM.
 *
 * = AUTHOR
 *       Roland Arajs
 *       roland.arajs@intel.com
 *
 * = CHANGE HISTORY
 *      6/5/2002 3:25:15 PM - creation time
 *
 * ============================================================================
 * $Id: internal_cci_comm.c,v 1.12 2002/11/07 23:35:09 ageorges Exp $
 */

#include <stddef.h>
#include "ix_os_type.h"
#include "ix_types.h"
#include "ix_symbols.h"
#include "ix_ossl.h"
#include "ix_hardware_configuration.h"

#include "rm/symbols.h"
#include "rm/sync_mechanism.h"
#include "rm/memory.h"
#include "rm/buffer.h"
#include "rm/hardware.h"
#include "rm/communication.h"
#include "rm/configuration_property.h"


/* CCI support enabled */
#if !defined( _IX_RM_CCI_SUPPORT_EXCLUDE_ )

#include "rm/internal/internal_tk_input_notify.h"


/**
 * Function called from RM (needs to be in RM so that it will
 * compile without the CCI present). This function does not return an error
 * value because there is nothing the RM could do with it. 
 */
void ix_cci_notify( void* pContext )
{
    ix_tk_input_notify* pNotifyNode = (ix_tk_input_notify*)pContext;
    ix_tk_input_notify* pParent = pNotifyNode->pParent;

    /* While not an engine policy (not at root of tree) */
    while( pParent->pParent )
    {
        /* Set byte in pending flag according to ID given by parent */

        pParent->tPending.flag[pNotifyNode->vId] = 1;
        /* Traverse into parent node */
        pNotifyNode = pParent;
        pParent = pNotifyNode->pParent;
    }
    
    /**
     * At this point, we have either reached the root of the execution-control 
     * tree or handle was invalid and a NULL pointer was returned. Check vType
     * to ascertain which.
     */
    /* If type corresponds to an execution engine */
    if( pParent->vSema )
    {
        /* Set new-token indicator in root node */
        *(pParent->pIsNewToken) = 1;

        /* Release semaphore in root (engine) node */
        ix_ossl_sem_give( pParent->vSema );
    }
}

#endif /* !defined( _IX_RM_CCI_SUPPORT_EXCLUDE_ ) */
