/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = LIBRARY
 *      Resource Manager Library
 *
 * = MODULE
 *      Hardware Resource Manager API
 *
 * = FILENAME
 *      internal_hardware.c
 *
 * = DESCRIPTION
 *      This file will define all the internal supporting functions for the 
 *      Hardware Resource Manager API.
 *
 * = AUTHOR
 *       Adrian Georgescu
 *       adrian.georgescu@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      10/26/2001 11:36:23 AM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */

static const char s_aFileIdentifier[] = "$Id: internal_hardware.c,v 1.21 2003/10/10 21:11:07 ageorges Exp $";

#include <ix_ossl.h>

#include "ix_os_type.h"
#include "ix_symbols.h"
#include "ix_hardware_configuration.h"

#include "ix_types.h"
#include "ix_macros.h"
#include "ix_error.h"

#include "rm/symbols.h"
#include "rm/internal/internal_support.h"

#include <hal_scratch.h>

#include "rm/sync_mechanism.h"
#include "rm/internal/internal_sync.h"
#include "rm/system.h"
#include "rm/internal/internal_system.h"

#include "rm/memory.h"

#include "rm/buffer.h"
#include "rm/hardware.h"
#include "rm/internal/internal_hardware.h"



/**
 * This is a global QArray manager
 */
ix_sram_qarray_manager g_QArrayManager;


/**
 * This is a global SCRATCH ring manager
 */
ix_scratch_ring_manager g_SCRATCHRingManager;


#if defined( _IX_RM_MULTI_THREADED_ )

static ix_ossl_sem_t s_QArrayManagerSemaphore = 0;

static ix_ossl_sem_t s_SCRATCHRingManagerSemaphore = 0;

#endif /* defined( _IX_RM_MULTI_THREADED_ ) */



#if defined( _IX_RM_TEST_MACROS_ )

/**
 * This static function is used just for the purpose of testing macros
 * defined in the in this API module.
 */
static void _ix_rm_test_macros(void)
{
    static const char s_aFunctionIdentifier[] = "_ix_rm_test_macros";

    ix_sram_queue_descriptor sramQueueDescriptor = 
        {
            0xf48967ab,
            0x01abf345,
            0x00dea678,
            0x00056789
        };

    ix_sram_ring_descriptor sramRingDescriptor = 
        {
            0xf45678ab,
            0x02345678,
            0x00ccdefa,
            0x00023456
        };

    ix_hw_queue_handle queueHandle = 0;
    ix_hw_ring_handle ringHandle = 0;

    ix_uint32 data;

    (void)s_aFunctionIdentifier;

    /* SRAM queue macros */
    data = IX_RM_GET_QUEUE_DESC_EOP(sramQueueDescriptor.m_HeadInfo);
    data = IX_RM_SET_QUEUE_DESC_EOP(sramQueueDescriptor.m_HeadInfo, 0);

    data = IX_RM_GET_QUEUE_DESC_CELL_COUNT(sramQueueDescriptor.m_HeadInfo);
    data = IX_RM_SET_QUEUE_DESC_CELL_COUNT(sramQueueDescriptor.m_HeadInfo, 100);

    data = IX_RM_GET_QUEUE_DESC_HEAD(sramQueueDescriptor.m_HeadInfo);
    data = IX_RM_SET_QUEUE_DESC_HEAD(sramQueueDescriptor.m_HeadInfo, 0x6789);

    data = IX_RM_GET_QUEUE_DESC_TAIL(sramQueueDescriptor.m_TailInfo);
    data = IX_RM_SET_QUEUE_DESC_TAIL(sramQueueDescriptor.m_TailInfo, 0x6789);

    data = IX_RM_GET_QUEUE_DESC_Q_COUNT(sramQueueDescriptor.m_CountInfo);
    data = IX_RM_SET_QUEUE_DESC_Q_COUNT(sramQueueDescriptor.m_CountInfo, 200);

    data = IX_RM_GET_QUEUE_DESC_SW_PRIVATE(sramQueueDescriptor.m_CountInfo);
    data = IX_RM_SET_QUEUE_DESC_SW_PRIVATE(sramQueueDescriptor.m_CountInfo, 200);

    /* SRAM ring macros */
    data = IX_RM_GET_RING_DESC_RING_SIZE(sramRingDescriptor.m_HeadInfo);
    data = IX_RM_SET_RING_DESC_RING_SIZE(sramRingDescriptor.m_HeadInfo, 100);

    data = IX_RM_GET_RING_DESC_HEAD(sramRingDescriptor.m_HeadInfo);
    data = IX_RM_SET_RING_DESC_HEAD(sramRingDescriptor.m_HeadInfo, 0x6789);

    data = IX_RM_GET_RING_DESC_TAIL(sramRingDescriptor.m_TailInfo);
    data = IX_RM_SET_RING_DESC_TAIL(sramRingDescriptor.m_TailInfo, 0x6789);

    data = IX_RM_GET_RING_DESC_RING_COUNT(sramRingDescriptor.m_CountInfo);
    data = IX_RM_SET_RING_DESC_RING_COUNT(sramRingDescriptor.m_CountInfo, 200);

    /* Queue handle macros */
    data = IX_RM_HW_QUEUE_SET_INDEX(queueHandle,23);
    data = IX_RM_HW_QUEUE_GET_INDEX(queueHandle);

    data = IX_RM_HW_QUEUE_SET_CHANNEL(queueHandle,2);
    data = IX_RM_HW_QUEUE_GET_CHANNEL(queueHandle);

    /* Ring handle macros */
    data = IX_RM_HW_RING_SET_CHANNEL(ringHandle,3);
    data = IX_RM_HW_RING_GET_CHANNEL(ringHandle);

    data = IX_RM_HW_RING_SET_SIZE(ringHandle,35);
    data = IX_RM_HW_RING_GET_SIZE(ringHandle);

    data = IX_RM_HW_RING_SET_MEMORY_TYPE(ringHandle,IX_MEMORY_TYPE_SRAM);
    data = IX_RM_HW_RING_GET_MEMORY_TYPE(ringHandle);

    data = IX_RM_HW_RING_SET_INDEX(ringHandle,15);
    data = IX_RM_HW_RING_GET_INDEX(ringHandle);
}

#endif /* end defined( _IX_RM_TEST_MACROS_ ) */


/**
 * NAME: _ix_rm_hardware_init
 *
 * DESCRIPTION: This function should be called at the time the resource manager initializer 
 *          function is called. The function will initialize all the Hardware API.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 */
ix_error _ix_rm_hardware_init(void)
{
    static const char s_aFunctionIdentifier[] = "_ix_rm_hardware_init";

    ix_error err = 0;
    ix_uint32 i, j;
    
    ix_sram_queue_descriptor *pQue;
    
    IX_RM_UNUSED_VARIABLE(s_aFunctionIdentifier);
    
#if defined( _IX_RM_TEST_MACROS_ )

    /**
     * Debug all macros defined in this API module 
     */
    _ix_rm_test_macros();

#endif /* end defined( _IX_RM_TEST_MACROS_ ) */

    /* Initialize the synchronization mechanism for PM manager */
#if defined( _IX_RM_MULTI_THREADED_ )


    err = ix_ossl_sem_init(IX_OSSL_SEM_AVAILABLE, &s_QArrayManagerSemaphore);
    if(err)
    {
        return IX_RM_ERROR_NEW(IX_RM_ERROR_HW_API_INIT_FAILED, s_aFunctionIdentifier, 
                        "initialization of QArray manager's semaphore failed!", err);
    }

    _ix_rm_sa_sync_adapter_init(&g_QArrayManager.m_SyncAdapter, _ix_rm_sa_default_lock, _ix_rm_sa_default_unlock, 
        (void*)&s_QArrayManagerSemaphore); 

    err = ix_ossl_sem_init(IX_OSSL_SEM_AVAILABLE, &s_SCRATCHRingManagerSemaphore);
    if(err)
    {
        return IX_RM_ERROR_NEW(IX_RM_ERROR_HW_API_INIT_FAILED, s_aFunctionIdentifier, 
                        "initialization of SCRATCH ring manager's semaphore failed!", err);
    }

    _ix_rm_sa_sync_adapter_init(&g_SCRATCHRingManager.m_SyncAdapter, _ix_rm_sa_default_lock, _ix_rm_sa_default_unlock, 
        (void*)&s_SCRATCHRingManagerSemaphore); 

#endif /* defined( _IX_RM_MULTI_THREADED_ ) */

    /* initialize the array of QArray managers pointers */
    for(i = 0; i < IX_SRAM_CHANNELS_NUMBER; i++)
    {
        g_QArrayManager.m_aQArrayManagersPtr[i] = 0;
        ix_ossl_memset(g_QArrayManager.m_aRingOpImplData[i],0,sizeof(g_QArrayManager.m_aRingOpImplData[i]));
    }

    /*
     * Allocate the QArray managers memory from that SRAM channel's memory
     */
    for (i = 0; i < IX_SRAM_CHANNELS_NUMBER; i++)
    {
        /* allocate with 16 byte alignment as required by the qarray descriptors */
        if (ix_rm_mem_alloc_aligned(IX_MEMORY_TYPE_SRAM, i,
               sizeof(ix_sram_qarray_channel_manager), IX_MEMORY_ALIGNMENT_TYPE_PHYSICAL_OFFSET, 4,
               (void**)&g_QArrayManager.m_aQArrayManagersPtr[i]) != IX_SUCCESS)
        {
            ix_memory_info memoryInfo;

            /* get the information about memory on this channel */
            ix_rm_mem_info(IX_MEMORY_TYPE_SRAM, i, &memoryInfo);

            /* check if there is any memory on this channel */
            if(memoryInfo.m_TotalSize != 0)
            /* if it is declare an error */
                return IX_RM_ERROR_NEW(IX_RM_ERROR_HW_QARRAY_MGR_ALLOC_FAILED, s_aFunctionIdentifier, 
                        "the QArray manager for this SRAM channel was not created due to lack of memory!", err);
            /* else just don't use this channel */
            else
                continue; /* no memory in this channel */
        }
        for (j = 0; j < IX_MAX_SRAM_QARRAY_ENTRIES; j++)
        {
            pQue = &g_QArrayManager.m_aQArrayManagersPtr[i]->
                                     m_aQArrayDescriptors[j].m_QueueDescriptor;
            IX_RM_MEM_UINT32_WRITE(&pQue->m_HeadInfo, 0);
            IX_RM_MEM_UINT32_WRITE(&pQue->m_TailInfo, 0);
            IX_RM_MEM_UINT32_WRITE(&pQue->m_CountInfo, 0);
            IX_RM_MEM_UINT32_WRITE(&pQue->m_Reserved, 0);
            /*
             * For each SRAM channel mark the first 16 entries as reserved.
             * They are going to be used by the QueueManager.
             */
            (void)IX_RM_MEM_UINT32_WRITE(&g_QArrayManager.m_aQArrayManagersPtr[i]->
                                   m_aQArrayUsage[j], (j < 16) ? 1 : 0);
        }
    }
    /*
     * Initialize the SCRATCH rings
     */
    for(i = 0; i < IX_MAX_SCRATCH_RINGS; i++)
    {
    
#if defined(_IX_RM_IMPL_HARDWARE_)

        SET_SCRATCH_CSR(RING_X_BASE, i, 0);
        SET_SCRATCH_CSR(RING_X_HEAD, i, 0);
        SET_SCRATCH_CSR(RING_X_TAIL, i, 0);

        /* Added as a fix for a IXP2400 B0 board issue */
        {
            ix_uint32 temp;
            temp = GET_SCRATCH_CSR(RING_X_TAIL, i);
        }

#endif /* defined(_IX_RM_IMPL_HARDWARE_) */

        /* set the rings as being free */
        g_SCRATCHRingManager.m_aRingUsage[i] = 0;
    }

    ix_ossl_memset(g_SCRATCHRingManager.m_aRingOpImplData,0,sizeof(g_SCRATCHRingManager.m_aRingOpImplData));

    return err;
}


/**
 * NAME: _ix_rm_hardware_fini
 *
 * DESCRIPTION: This function should be called at the time the resource manager finalizer function
 *          is called. The function will provide cleanup for Hardware API.
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 */
ix_error _ix_rm_hardware_fini(void)
{
    static const char s_aFunctionIdentifier[] = "_ix_rm_hardware_fini";

    ix_error err = 0;
    ix_uint32 i;

	IX_RM_UNUSED_VARIABLE(s_aFunctionIdentifier);

    /* Reset the SCRATCH rings */
    for(i = 0; i < IX_MAX_SCRATCH_RINGS; i++)
    {

#if defined(_IX_RM_MULTI_THREADED_)

        if(g_SCRATCHRingManager.m_aRingOpImplData[i].m_Semaphore != 0)
        {
            ix_ossl_sem_fini(g_SCRATCHRingManager.m_aRingOpImplData[i].m_Semaphore);
            g_SCRATCHRingManager.m_aRingOpImplData[i].m_Semaphore = 0;
        }

#endif /* end defined(_IX_RM_MULTI_THREADED_) */

        g_SCRATCHRingManager.m_aRingOpImplData[i].m_RingSize = 0;

#if defined(_IX_RM_IMPL_HARDWARE_)

        SET_SCRATCH_CSR(RING_X_BASE, i, 0);
        SET_SCRATCH_CSR(RING_X_HEAD, i, 0);
        SET_SCRATCH_CSR(RING_X_TAIL, i, 0);

        /* Added as a fix for a IXP2400 B0 board issue */
        {
            ix_uint32 temp;
            temp = GET_SCRATCH_CSR(RING_X_TAIL, i);
        }

#endif /* defined(_IX_RM_IMPL_HARDWARE_) */

        /* set the rings as being free */
        g_SCRATCHRingManager.m_aRingUsage[i] = 0;
    }

    /* release the QArray managers memory */
    for(i = 0; i < IX_SRAM_CHANNELS_NUMBER; i++)
    {
        if(g_QArrayManager.m_aQArrayManagersPtr[i] != 0)
        {
            ix_uint32 j;
            for(j = 0; j < IX_MAX_SRAM_QARRAY_ENTRIES; j++)
            {

#if defined(_IX_RM_MULTI_THREADED_)

                if(g_QArrayManager.m_aRingOpImplData[i][j].m_Semaphore != 0)
                {
                    ix_ossl_sem_fini(g_QArrayManager.m_aRingOpImplData[i][j].m_Semaphore);
                    g_QArrayManager.m_aRingOpImplData[i][j].m_Semaphore = 0;
                }

#endif /* end defined(_IX_RM_MULTI_THREADED_) */

                g_QArrayManager.m_aRingOpImplData[i][j].m_RingSize = 0;
            }

            err = ix_rm_mem_free((ix_uint8*)g_QArrayManager.m_aQArrayManagersPtr[i]);
            if(err)
            {
                err = IX_RM_ERROR_NEW(IX_RM_ERROR_HW_QARRAY_MGR_DEALLOC_FAILED, s_aFunctionIdentifier, 
                        "SRAM QArray manager memory release failed!", err);
            }

            g_QArrayManager.m_aQArrayManagersPtr[i] = 0;
        }
    }

    /* Finalize the synchronization mechanism for QArray manager and SCRATCH ring manager */
#if defined( _IX_RM_MULTI_THREADED_ )

    _ix_rm_sa_sync_adapter_init(&g_SCRATCHRingManager.m_SyncAdapter,0, 0, (void*)0); 

    err = ix_ossl_sem_fini(s_SCRATCHRingManagerSemaphore);
    if(err != 0)
    {
        return IX_RM_ERROR_NEW(IX_RM_ERROR_HW_API_FINI_FAILED, s_aFunctionIdentifier, 
                        "finalization of SCRATCH ring manager's semaphore failed!", err);
    }

    s_SCRATCHRingManagerSemaphore = 0;

    _ix_rm_sa_sync_adapter_init(&g_QArrayManager.m_SyncAdapter,0, 0, (void*)0); 

    err = ix_ossl_sem_fini(s_QArrayManagerSemaphore);
    if(err != 0)
    {
        return IX_RM_ERROR_NEW(IX_RM_ERROR_HW_API_FINI_FAILED, s_aFunctionIdentifier, 
                        "finalization of QArray manager's semaphore failed!", err);
    }

    s_QArrayManagerSemaphore = 0;

#endif /* defined( _IX_RM_MULTI_THREADED_ ) */

    return err;
}
