/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = LIBRARY
 *      Resource Manager Library
 *
 * = MODULE
 *      Library Interface
 *
 * = FILENAME
 *      internal_sync.c
 *
 * = DESCRIPTION
 *      This filewill define several functions that will be used for the internal
 *      implementation of the synchronization mechanism.
 *
 * = AUTHOR
 *      Adrian Georgescu
 *      adrian.georgescu@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      1/28/2002 11:52:43 AM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */

static const char s_FileIdentifier[] = "$Id: internal_sync.c,v 1.8 2002/11/07 23:35:10 ageorges Exp $";

#include "ix_ossl.h"
#include "ix_os_type.h"
#include "ix_symbols.h"
#include "ix_hardware_configuration.h"

#include "ix_types.h"
#include "ix_macros.h"
#include "ix_error.h"

#include "rm/symbols.h"
#include "rm/internal/internal_support.h"

#include "rm/system.h"
#include "rm/sync_mechanism.h"
#include "rm/internal/internal_sync.h"
#include "rm/internal/internal_system.h"

/**
 * NAME: _ix_rm_sa_default_lock
 *
 * DESCRIPTION: This function is provided for internal lock support. The function
 *          makes the assumption that the context is a pointer to an OSSL semaphore.
 *          The call will aquire the semaphore, so any subsequent calls to _ix_rm_sa_default_lock
 *          with the same context will block till the semaphore is released.
 * 
 * @Param:  - INOUT arg_pContext - it will be interpreted as a pointer to an OSSL
 *          semaphore of the type ix_ossl_sem_t.
 *
 */
void _ix_rm_sa_default_lock( 
                             void* arg_pContext 
                           )
{
    static const char s_aFunctionIdentifier[] = "_ix_rm_sa_default_lock";

    IX_RM_UNUSED_VARIABLE(s_aFunctionIdentifier);

    ix_ossl_sem_take(*((ix_ossl_sem_t*)arg_pContext), IX_OSSL_WAIT_FOREVER);
}


/**
 * NAME: _ix_rm_sa_default_unlock
 *
 * DESCRIPTION: This function is provided for internal unlock support. The function
 *          makes the assumption that the context is a pointer to an OSSL semaphore.
 *          The call will release the semaphore, so any tasks waiting on that object
 *          will be released and one willa quire the semaphore and continue running.
 * 
 * @Param:  - INOUT it will be interpreted as a pointer to an OSSL
 *          semaphore of the type ix_ossl_sem_t.
 *
 */
void _ix_rm_sa_default_unlock( 
                               void* arg_pContext 
                             )
{
    static const char s_aFunctionIdentifier[] = "_ix_rm_sa_default_unlock";

    IX_RM_UNUSED_VARIABLE(s_aFunctionIdentifier);

    ix_ossl_sem_give(*((ix_ossl_sem_t*)arg_pContext));
}


/**
 * NAME: _ix_rm_sa_sync_adapter_init
 *
 * DESCRIPTION:   Initializes the synchronization adapter object.
 * 
 * @Param:  - IN arg_pAdapter - pointer to adapter to be initialized 
 * @Param:  - IN arg_LockFunction - the lock function
 * @Param:  - IN arg_UnlockFunction - the unlock function
 * @Param:  - IN arg_pContext - context to be passed to the lock and unlock functions
 */
void _ix_rm_sa_sync_adapter_init( 
                                 ix_synchronization_adapter*  arg_pAdapter,
                                 ix_sa_lock_ptr               arg_LockFunction,
                                 ix_sa_unlock_ptr             arg_UnlockFunction,
                                 void*                        arg_pContext
                               )
{
    static const char s_aFunctionIdentifier[] = "_ix_rm_sa_sync_adapter_init";

    IX_RM_UNUSED_VARIABLE(s_aFunctionIdentifier);

    if((arg_LockFunction != 0) && (arg_UnlockFunction != 0))
    {
        arg_pAdapter->m_pContext = arg_pContext;
        arg_pAdapter->m_LockFuncPtr = arg_LockFunction;
        arg_pAdapter->m_UnlockFuncPtr = arg_UnlockFunction;

        /* set the synchronization mechanism in active state */
        arg_pAdapter->m_SyncActive = 1;
    }
    else
    {
        arg_pAdapter->m_SyncActive = 0;
    }
}

