/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor
 *
 * = LIBRARY
 *      Resource Manager Library
 *
 * = MODULE
 *      Library Interface
 *
 * = FILENAME
 *      internal_version.c
 *
 * = DESCRIPTION
 *      This file defines all version and build daa.
 *
 * = AUTHOR
 *      Adrian Georgescu
 *      adrian.georgescu@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      3/13/2002 11:27:02 AM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */

static const char s_FileIdentifier[] = "$Id: internal_version.c,v 1.4 2003/08/18 21:48:53 ageorges Exp $";

#include "ix_ossl.h"

#include "rm/internal/internal_version.h"


/**
 * Static buffer that contains the version scheme information string
 */
static char s_aVersionSchemeString[1024];



/**
 * Global address to the version scheme information string
 */
char* g_pVersionSchemeString = 0;



/**
 * NAME: _ix_rm_build_version_scheme_string
 *
 * DESCRIPTION: This function will build the version scheme string for the product and 
 *          will set g_pVersionSchemeString to point to the buffer.
 *
 */
void _ix_rm_build_version_scheme_string(void)
{
    static const char s_aFunctionIdentifier[] = "_ix_rm_build_version_scheme_string";
    int n;

    (void)s_aFunctionIdentifier;

    n = sprintf(s_aVersionSchemeString,"%s\nVersion %d.%d.%d ", IX_RM_PRODUCT_STRING, 
        IX_RM_VERSION_MAJOR, IX_RM_VERSION_MINOR_BIG, IX_RM_VERSION_MINOR_LITTLE);
    switch(IX_RM_VERSION_TYPE)
    {
        case 'A':
        case 'a': 
            n+=sprintf(s_aVersionSchemeString + n,"alpha release ");
            break;
        case 'B':
        case 'b':
            n+=sprintf(s_aVersionSchemeString + n,"beta release ");
            break;
        case 'C':
        case 'c':
            n+=sprintf(s_aVersionSchemeString + n,"candidate release ");
            n+=sprintf(s_aVersionSchemeString + n,"%d",IX_RM_VERSION_INCREMENT);
            break;
        case 'D':
        case 'd':
            n+=sprintf(s_aVersionSchemeString + n,"debug build ");
            n+=sprintf(s_aVersionSchemeString + n,"%d",IX_RM_VERSION_INCREMENT);
            break;
        case 'E':
        case 'e':
            n+=sprintf(s_aVersionSchemeString + n,"experimental build ");
            break;    
        case 'P':
        case 'p':
            n+=sprintf(s_aVersionSchemeString + n,"patch build ");
            n+=sprintf(s_aVersionSchemeString + n,"%d",IX_RM_VERSION_INCREMENT);
            break;
        default:
            n+=sprintf(s_aVersionSchemeString + n,"internal build ");
            n+=sprintf(s_aVersionSchemeString + n,"%d",IX_RM_VERSION_INCREMENT);
            break;
    }

    n+=sprintf(s_aVersionSchemeString + n," %02d/%02d/%d\n",IX_RM_BUILD_MONTH, IX_RM_BUILD_DAY, IX_RM_BUILD_YEAR);
    n+=sprintf(s_aVersionSchemeString + n,"%s\n",IX_RM_PACKAGE_STRING);
    n+=sprintf(s_aVersionSchemeString + n,"%s\n",IX_RM_COPYRIGHT_STRING);

    g_pVersionSchemeString = s_aVersionSchemeString;
}




