/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      linux_kernel_module.c
 *
 * = DESCRIPTION
 *      Thsi file is used to build the RM library as a Linux Kernel Loadable module
 *
 * = AUTHOR
 *      Adrian Georgescu
 *      adrian.georgescu@intel.com
 *
 * = CHANGE HISTORY
 *      8/18/2003 1:49:26 PM - creation time 
 * ============================================================================
 */

static const char s_FileIdentifier[] = "$Id: linux_kernel_module.c,v 1.7 2003/10/29 21:54:12 ageorges Exp $";


#include "ix_ossl.h"
#include "ix_rm.h"
#include "rm/internal/internal_sync.h"
#include "rm/internal/internal_system.h"

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

#if defined(_IX_RM_DEBUG_)

#include <linux/proc_fs.h>

static const char* s_DebugProcFileName = "rm_debug";

#endif /* defined(_IX_RM_DEBUG_) */

#if defined(MODULE_LICENSE)

MODULE_LICENSE("Proprietary");

#endif /* defined(MODULE_LICENSE) */

#if defined(_IX_RM_DEBUG_)

int ix_rm_kernel_debug_read(char *page, char **start, off_t off,
                           int count, int *eof, void *data)
{
    int len = 0;

    len += sprintf(page+len, "RM debug info usage mode: \n");
    len += sprintf(page+len, "\techo \"command [param]\" > /proc/%s \n", s_DebugProcFileName);
    len += sprintf(page+len, "\t\"command [param]\" could be as follows:\n\n");

    len += sprintf(page+len, "\tix_rm_mem_status_print memoryType memoryChannel\n");
    len += sprintf(page+len, "\t-prints the allocation status for the specified memory type\n");
    len += sprintf(page+len, "\t on the specified channel\n");
    len += sprintf(page+len, "\t\tDRAM:    memoryType = 0\n");
    len += sprintf(page+len, "\t\tSRAM:    memoryType = 1\n");
    len += sprintf(page+len, "\t\tSCRATCH: memoryType = 2\n\n");

    len += sprintf(page+len, "\tix_rm_free_list_print_available_buffers freelistType freelistNumber\n");
    len += sprintf(page+len, "\t-prints the available buffers number in the specified free list\n");
    len += sprintf(page+len, "\t\tHW:    freelistType = 0\n");
    len += sprintf(page+len, "\t\tSW:    freelistType = 1\n\n");

    len += sprintf(page+len, "\tix_rm_free_list_print_buffers_info freelistNumber\n");
    len += sprintf(page+len, "\t-prints the information about buffers belonging to the specified free list\n\n");

    len += sprintf(page+len, "\tix_rm_free_list_print_info freelistType freelistNumber\n");
    len += sprintf(page+len, "\t-prints information about specified free list\n");
    len += sprintf(page+len, "\t\tHW:    freelistType = 0\n");
    len += sprintf(page+len, "\t\tSW:    freelistType = 1\n\n");

    len += sprintf(page+len, "\tix_rm_buffer_print_meta bufferHandle\n");
    len += sprintf(page+len, "\t-prints the meta data for the specified buffer\n\n");

    len += sprintf(page+len, "\tix_rm_buffer_print_data bufferHandle\n");
    len += sprintf(page+len, "\t-prints the data for the specified buffer\n\n");

    len += sprintf(page+len, "\tix_rm_buffer_print_debug_info bufferHandle\n");
    len += sprintf(page+len, "\t-prints the debug information for the specified buffer\n\n");

#if defined (_IX_RM_IMPL_HARDWARE_)

    len += sprintf(page+len, "\tix_rm_scratch_ring_print_info ringNumber\n");
    len += sprintf(page+len, "\t-prints the scratch ring information for the specified ring\n\n");

    len += sprintf(page+len, "\tix_rm_scratch_ring_print_data ringNumber\n");
    len += sprintf(page+len, "\t-prints the scratch ring content for the specified ring\n\n");

    len += sprintf(page+len, "\tix_rm_sram_ring_print_info memory_channel ringNumber\n");
    len += sprintf(page+len, "\t-prints the sram ring information for the specified ring\n\n");

    len += sprintf(page+len, "\tix_rm_scratch_ring_print_data memory_channel ringNumber\n");
    len += sprintf(page+len, "\t-prints the sram ring content for the specified ring\n\n");

#endif /* defined (_IX_RM_IMPL_HARDWARE_) */

    *eof = 1;
    return len;
}

int ix_rm_kernel_debug_write(struct file *file, const char *buffer,
                            unsigned long count, void *data)
{
#define RM_DEBUG_BUFFER_LENGTH	256
    char commandBuffer[RM_DEBUG_BUFFER_LENGTH];
    char command[RM_DEBUG_BUFFER_LENGTH];
    unsigned long param1;
    unsigned long param2;
    int charactersRead;
    int readCount;

    if(g_RMLibInitialization.m_InitCount == 0)
    {
        ix_ossl_message_log("RM library is not initialized!\n");
        goto label_1;
    }

    if(count > RM_DEBUG_BUFFER_LENGTH - 1)
    {
        ix_ossl_message_log("The passed command string is too long. You should pass maximum %d characters!\n", RM_DEBUG_BUFFER_LENGTH - 1);
        goto label_1;
    }

    memcpy(commandBuffer, buffer, count);
    commandBuffer[count] = '\0';

    ix_ossl_message_log("Command received: %s\n", commandBuffer);


    readCount = sscanf(commandBuffer, "%s%n", command, &charactersRead);

    if(readCount < 1)
    {
        ix_ossl_message_log("Bad command string!\n");
        goto label_1;
    }
    if(strcmp(command, "ix_rm_mem_status_print") == 0)
    {
        readCount = sscanf(commandBuffer + charactersRead, "%lu %lu", &param1, &param2);
        if(readCount < 2)
        {
            ix_ossl_message_log("Wrong number of parameters!\n");
            goto label_1;
        }
        ix_rm_mem_status_print(param1, param2);
    }
    else if(strcmp(command,"ix_rm_free_list_print_available_buffers") == 0)
    {
        readCount = sscanf(commandBuffer + charactersRead, "%lu %lu", &param1, &param2);
        if(readCount < 2)
        {
            ix_ossl_message_log("Wrong number of parameters!\n");
            goto label_1;
        }
        ix_rm_free_list_print_available_buffers(param1, param2);
    }
    else if(strcmp(command,"ix_rm_free_list_print_buffers_info") == 0)
    {
        readCount = sscanf(commandBuffer + charactersRead, "%lu", &param1);
        if(readCount < 1)
        {
            ix_ossl_message_log("Wrong number of parameters!\n");
            goto label_1;
        }
        ix_rm_free_list_print_buffers_info(param1);
    }
    else if(strcmp(command,"ix_rm_free_list_print_info") == 0)
    {
        readCount = sscanf(commandBuffer + charactersRead, "%lu %lu", &param1, &param2);
        if(readCount < 2)
        {
            ix_ossl_message_log("Wrong number of parameters!\n");
            goto label_1;
        }
        ix_rm_free_list_print_info(param1, param2);
    }
    else if(strcmp(command,"ix_rm_buffer_print_meta") == 0)
    {
        readCount = sscanf(commandBuffer + charactersRead, "%lu", &param1);
        if(readCount < 1)
        {
            ix_ossl_message_log("Wrong number of parameters!\n");
            goto label_1;
        }
        ix_rm_buffer_print_meta(param1);
    }
    else if(strcmp(command,"ix_rm_buffer_print_data") == 0)
    {
        readCount = sscanf(commandBuffer + charactersRead, "%lu", &param1);
        if(readCount < 1)
        {
            ix_ossl_message_log("Wrong number of parameters!\n");
            goto label_1;
        }
        ix_rm_buffer_print_data(param1);
    }
    else if(strcmp(command,"ix_rm_buffer_print_debug_info") == 0)
    {
        readCount = sscanf(commandBuffer + charactersRead, "%lu", &param1);
        if(readCount < 1)
        {
            ix_ossl_message_log("Wrong number of parameters!\n");
            goto label_1;
        }
        ix_rm_buffer_print_debug_info(param1);
    }

#if defined (_IX_RM_IMPL_HARDWARE_)

    else if(strcmp(command,"ix_rm_scratch_ring_print_info") == 0)
    {
        readCount = sscanf(commandBuffer + charactersRead, "%lu", &param1);
        if(readCount < 1)
        {
            ix_ossl_message_log("Wrong number of parameters!\n");
            goto label_1;
        }
        ix_rm_scratch_ring_print_info(param1);
    }
    else if(strcmp(command,"ix_rm_scratch_ring_print_data") == 0)
    {
        readCount = sscanf(commandBuffer + charactersRead, "%lu", &param1);
        if(readCount < 1)
        {
            ix_ossl_message_log("Wrong number of parameters!\n");
            goto label_1;
        }
        ix_rm_scratch_ring_print_data(param1);
    }
    else if(strcmp(command,"ix_rm_sram_ring_print_info") == 0)
    {
        readCount = sscanf(commandBuffer + charactersRead, "%lu %lu", &param1, &param2);
        if(readCount < 2)
        {
            ix_ossl_message_log("Wrong number of parameters!\n");
            goto label_1;
        }
        ix_rm_sram_ring_print_info(param1, param2);
    }
    else if(strcmp(command,"ix_rm_sram_ring_print_data") == 0)
    {
        readCount = sscanf(commandBuffer + charactersRead, "%lu %lu", &param1, &param2);
        if(readCount < 2)
        {
            ix_ossl_message_log("Wrong number of parameters!\n");
            goto label_1;
        }
        ix_rm_sram_ring_print_data(param1, param2);
    }

#endif /* defined (_IX_RM_IMPL_HARDWARE_) */

    else
    {
        ix_ossl_message_log("Unknown command %s!\n", command);
        goto label_1;
    }

label_1:
    return count;

#undef RM_DEBUG_BUFFER_LENGTH
}

#endif /* defined(_IX_RM_DEBUG_) */


int init_module(void)
{

#if defined(_IX_RM_DEBUG_)

    /* ix_rm_init(0); */

    {
        struct proc_dir_entry *ent;
        
        ent = create_proc_read_entry(
                s_DebugProcFileName, /* file name */
                S_IRUGO | S_IWUGO, /* mode */
                NULL, /* parent directory */
                ix_rm_kernel_debug_read, /* read function */
                NULL /* user data */
            );

        if(ent != 0)
            ent->write_proc = ix_rm_kernel_debug_write;

    }

#endif /* defined(_IX_RM_DEBUG_) */

    return 0;
}

void cleanup_module(void)
{

#if defined(_IX_RM_DEBUG_)

    remove_proc_entry(s_DebugProcFileName, NULL);

    /* ix_rm_term(); */

#endif /* defined(_IX_RM_DEBUG_) */

    return;
}

#endif /* (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)*/


