/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation.All rights reserved.
 *
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 5
 *
 * = FILENAME
 *      ix_cc.h
 *
 * = DESCRIPTION
 *      Data strucutres and types common to all the core components.
 *
 * = AUTHOR
 *       Aaron Luk
 *       aaron.luk@intel.com
 *
 * = CHANGE HISTORY
 *      8/12/2002 2:28:19 PM - creation time
 *
 * ============================================================================
 * $Id: ix_cc.h,v 1.31 2003/12/12 00:40:14 ktseng Exp $
 * $History: $
 */

#if !defined(__IX_CC_H__)
#define __IX_CC_H__

#include "ix_rm.h"
#include "ix_cc_properties.h"
#include "ix_cc_macros.h"

/*ix_cc_handle is defined in L0 system as the init function signature needs it*/
#ifndef IX_EXCLUDE_CCI
    #include "ix_cci.h"
#else
    typedef ix_uint32 ix_cc_handle;
#endif 


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

/* global setting defines  */

#define IX_INGRESS 0
#define IX_EGRESS  1 
#define IX_SWITCH_FABRIC_NO_LOOPBACK    0
#define IX_SWITCH_FABRIC_LOOPBACK       1

/* global CC constants */
#define IX_UBLOCK_IMAGE_NAME_SIZE       128    
#define IX_IMPORTED_SYMBOL_NAME_SIZE    128
#define IX_AF_INET                      2
#define IX_AF_INET6                     23

/**
 * TYPENAME: ix_cc_common_msg_id
 * 
 * DESCRIPTION: Common message IDs for the core components.
 *
 */
typedef enum ix_e_cc_common_msg_id
{
    IX_CC_COMMON_MSG_ID_FIRST = 0,
    IX_CC_COMMON_MSG_ID_PROP_UPDATE = IX_CC_COMMON_MSG_ID_FIRST,
    IX_NEW_DIRECT_HOST_MSG_ID,
    IX_CC_IPV6_MSG_PERFORM_ADDR_RESOLUTION,
    IX_CC_COMMON_MSG_ID_LAST
} ix_cc_common_msg_id;


/**
 * TYPENAME: ix_cc_init_context
 * 
 * DESCRIPTION: Generic context passed into a CC during initialization.
 *
 */
typedef struct ix_s_cc_init_context
{
    /* Handle to shared freelist of buffers for all core components. */
    ix_buffer_free_list_handle hFreeList;

    /**
     * Pointer to head of linked list of interface properties
     * for all ports configured by the system app.
     */
    ix_cc_properties_node* pPropertyList;
    ix_uint32 instanceCC;
} ix_cc_init_context;



/**
 * TYPENAME: ix_cc_eth_rx_statistics_info
 * 
 * DESCRIPTION: Common Ethernet Rx statistics for Ethernet Rx CC and 
 *              Ethernet Tx CC.
 *
 */
typedef enum ix_e_cc_eth_rx_statistics_info
{
    IX_CC_ETH_RX_COUNTER_FIRST = 0, /* Start of Ethernet Rx counter */

#ifdef MICROBLOCK_COUNTERS

    /**
     * Per port Ethernet Rx microblock counters. The order of the
     * counters is defined by microblock code and must not be 
     * altered here. 
     */
    IX_CC_ETH_RX_PACKETS,           /* Rx packet count */
    IX_CC_ETH_RX_DROP_PACKETS,      /* Rx drop packet count */
    IX_CC_ETH_RX_EXCEPTION_PACKETS, /* Rx exception packet count */
    IX_CC_ETH_RX_BYTES,             /* Rx byte count */

    IX_CC_ETH_RX_ALL_MICROBLOCK_COUNTERS, /* All above ublk counters */

#endif /* MICROBLOCK_COUNTERS */

    /**
     * Per port Ethernet device driver Rx counters.
     */
    IX_CC_ETH_RX_OK_OCTETS, /* bytes received in legal frames*/

    IX_CC_ETH_RX_BAD_OCTETS,/*bytes received in all bad frames with legal size */

    IX_CC_ETH_RX_UCAST_PACKETS, /*unicast packets received*/

    IX_CC_ETH_RX_MCAST_PACKETS, /*multicast packets received */

    IX_CC_ETH_RX_BCAST_PACKETS, /*broadcast packets received */

    IX_CC_ETH_RX_PACKETS_64, /*packets received that are 64 octets in length*/

    IX_CC_ETH_RX_PACKETS_65_127, /* packets received that are 65-127 octets in length */

    IX_CC_ETH_RX_PACKETS_128_255, /*packets received that are 128-255 octets in length */

    IX_CC_ETH_RX_PACKETS_256_511, /* packets received that are 256-511 octets in length */

    IX_CC_ETH_RX_PACKETS_512_1023, /* packets received that are 512-1023 octets in length */

    IX_CC_ETH_RX_PACKETS_1024_1518, /*packets received that are 1024-1518 octets in length */

    IX_CC_ETH_RX_PACKETS_1519_MAX, /*packets received that are >1518 octets in length */

    IX_CC_ETH_RX_FCS_ERR, /* frames received with legal size but with wrong CRC field */

    IX_CC_ETH_RX_VLAN_TAG, /*OK frames received with VLAN tag */

    IX_CC_ETH_RX_DATA_ERR, /*frames received with legal length with code violation */

    IX_CC_ETH_RX_ALIGN_ERR, /*frames received with legal frame size but containing less than 8 additional bits */

    IX_CC_ETH_RX_LONG_ERR, /*frames received bigger than maximum allowed but with ok CRC and integral number of octets */

    IX_CC_ETH_RX_JABBER_ERR, /*frames received bigger than maximum allowed with either bad CRC or a non-integral number of octets */

    IX_CC_ETH_RX_PAUSE_MAC_CTL, /*number of Pause MAC control frames received */

    IX_CC_ETH_RX_UNKNOWN_CTL_FRAME, /*number of MAC control frames received with an op code different from 0001 (Pause) */

    IX_CC_ETH_RX_VLONG_ERR, /*frames received bigger than the larger of 2*max frame size and 50000 bits */

    IX_CC_ETH_RX_GET_RUNT_ERR, /*packets received less than 64 octets in length but longer than or equal to 96 bit times */

    IX_CC_ETH_RX_GET_SHORT_ERR, /*packets received that are less than 96 bit times */

    IX_CC_ETH_RX_GET_SEQ_ERR, /* number of sequencing errors that occur in Fiber mode */

    IX_CC_ETH_RX_GET_SYMBOL_ERR, /* number of symbol errors, encounterd by the PHY */

    IX_CC_ETH_RX_ALL_DRIVER_COUNTERS, /* all above driver counters of a given port */

    IX_CC_ETH_RX_COUNTER_LAST       /* End of Ethernet Rx counter definition */

} ix_cc_eth_rx_statistics_info;

/**
 * defines for message types for Ethernet Rx CC
 */
typedef enum ix_e_cc_eth_rx_msg_type{
	IX_CC_ETH_RX_MSG_GET_STATISTICS_INFO = (IX_CC_COMMON_MSG_ID_LAST +1),
	IX_CC_ETH_RX_MSG_GET_INTERFACE_STATE,
    IX_CC_ETH_RX_MSG_ADD_MAC_ADDR,
    IX_CC_ETH_RX_MSG_DELETE_MAC_ADDR,
    IX_CC_ETH_RX_MSG_LOOKUP_PORT,
    IX_CC_ETH_RX_MSG_LAST 
} ix_cc_eth_rx_msg_type;


/**
 * TYPENAME: ix_cc_fpm_direct_host_exception_data
 * 
 * DESCRIPTION: This structure that will be passed by the Interface Tx Core 
 *              component when it receives an exception from the Interface Tx 
 *              micro blocks. This is required for supporting directly 
 *              connected hosts. The exception will be delivered to the CP PDK.
 */
typedef struct ix_s_cc_fpm_direct_host_exception_data
{
    ix_uint32 port_id; /* Port on which the packet had to be sent */
    ix_uint32 dest_ip;  /* Destination IP Address of the packet */
} ix_cc_fpm_direct_host_exception_data;


#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

/**
 * DESCRIPTION: Defines for device number and device name for Linux CC 
 * driver utilities.
 */
/* Device number */
#define DEV_NUMBER_SAUTIL      254
#define DEV_NUMBER_RCONFIG     253
#define DEV_NUMBER_L2CONFIG    252
#define DEV_NUMBER_MCONFIG     250
#define DEV_NUMBER_PCONFIG     249
#define DEV_NUMBER_NATPTCONFIG 240
#define DEV_NUMBER_TNLCONFIG   241
#define DEV_NUMBER_SARCONFIG   242
#define DEV_NUMBER_LLCCONFIG   243

/* Device name */
#define DEV_NAME_SAUTIL        "SaUtil"
#define DEV_NAME_RCONFIG       "RConfig"
#define DEV_NAME_L2CONFIG      "L2Config"
#define DEV_NAME_MCONFIG       "MConfig"
#define DEV_NAME_PCONFIG       "PConfig"
#define DEV_NAME_NATPTCONFIG   "NatptConfig"
#define DEV_NAME_TNLCONFIG     "TunnelConfig"
#define DEV_NAME_SARCONFIG     "SarConfig"
#define DEV_NAME_LLCCONFIG     "LlcConfig"

#endif /* (_ix_os_type_ == _ix_os_LINUX_KERNEL_) */


/*
 * NAME: ix_cc_hw_get_packet_data
 *
 * DESCRIPTION: Returns the pointer to the start of the packet data in a
 * hardware buffer. This is the defined as the start of the data plus the
 * offset obtained from the buffer's meta data.
 *
 * @Param:  - IN    arg_hBuffer
 *                  Handle of a hardware buffer
 *
 * @Param:  - OUT   arg_ppPacketData
 *                  Pointer to the start of packet data.
 *
 * Error codes:
 * IX_CC_ERROR_NULL
 * IX_CC_ERROR_INTERNAL
 * IX_RM_ERROR_BUF_HANDLE_INVALID
 * IX_RM_ERROR_BUF_FREE_LIST_INVALID
 * IX_RM_ERROR_BUF_FREE_LIST_ID_INVALID
 */
ix_error ix_cc_hw_get_packet_data(
    ix_buffer_handle    arg_hBuffer,
    void              **arg_ppPacketData);

/**
 * NAME: ix_rm_cc_calculate_cell_count
 *
 * DESCRIPTION: This function will calculate the required POS cell count expected by hardware
 *  given the cell count and the buffer type: SOP or not. For the SOP buffers there are 8 extra
 *  bytes that need to go into the first cell, so the amount of data that will get into the first
 *  cell will be up to 112 bytes. For non SOP buffers all cells will get up to 120 bytes of data.
 *  The returned cell count would be what hardware expects, which would be the real cell count
 *  less 1.
 * 
 * @Param:  - IN arg_BufferSize - the size of the data buffer
 * @Param:  - IN arg_IsSOP - flag that specifies if the buffer is SOP or not. If the value is different from
 *	0, then the buffer is an SOP buffer.
 *
 * @Return: The cell count expected by hardware which is the reall cell count less 1.
 */
ix_uint32 ix_cc_calculate_cell_count(ix_uint32 arg_BufferSize, ix_uint32 arg_IsSOP);

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_H__) */
