/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      ix_cc_microengines_bindings.h
 *
 * = DESCRIPTION
 *      This file has all the commIds that are defined by the core components and 
 *		needed by the microengines to send up the 
 *		the exception packets. Any microblock that sends packets to the packet Ids defined 
 *		by the core components must include this file.
 *		This file is automatically included in the bindings.h file for core components
 *
 * = CHANGE HISTORY
 *      8/15/2002 - Created
 * $Id: ix_cc_microengines_bindings.h,v 1.20 2003/11/12 01:02:50 rranjeet Exp $
 **/

#ifndef __IX_CC_MICROENGINES_BINDINGS_H__
#define __IX_CC_MICROENGINES_BINDINGS_H__





/* 
 *    Only 255 - 64 ID available for core components. Because of limitations, the core components IDs
 *      are not splitted by range in order to preserve ID space. Some components require one or two input ids, 
 *      and others may requre more.  
 *      The defines below must list all CC input ids, and values must not exceed 253
 *    
 *    Output ids should be defined to input ids of the next component. 
 *    'Next Component' means component, module, or application that receives packets
 *    or messages from input componnet. For example, IPv4 sends some of packets to Queue Manager, so it will 
 *    define output id for these packets to the input id of Queue Manager.
 */



/*
 *     CommIds 64-74 are reserved for shared commIds between core and micro-code
 */

#define START_SHARED_COMM_IDS				64
    
/*IPv4 commIds are the same for high and low priority. One is addded as msg and the other as pkt hdlr */
/* input id for packet handling */
#define IX_CC_IPV4_PKT_MICROBLOCK_ID                    65    /* (START_SHARED_COMM_IDS + 1)          */


/* Pos Rx communication ids  */
#define IX_CC_POS_RX_PKT_ID                             66    /* (IX_CC_IPV4_PKT_MICROBLOCK_ID + 1)   */

/* Ethernet Rx communications ids */
#define IX_CC_ETH_RX_PKT_ID                             67    /* (IX_CC_POS_RX_PKT_ID + 1)            */

/* Ethernet Tx communications ids */
#define IX_CC_ETH_TX_PKT_ID                             68    /* (IX_CC_ETH_RX_PKT_ID + 1)            */

/* 
 * IPv6 commIds are the same for high and low priority. 
 * One is addded as msg and the other as pkt hdlr
 */
/* input id for packet handling */
#define IX_CC_IPV6_PKT_MICROBLOCK_ID                    69    /* (IX_CC_ETH_TX_PKT_ID + 1)            */

/* IPv6/v4 tunneling communication ID */
#define IX_CC_V6V4_ENCAP_PKT_MICROBLOCK_ID                    70    /* (IX_CC_IPV6_PKT_MICROBLOCK_ID + 1)   */
#define IX_CC_V6V4_DECAP_PKT_MICROBLOCK_ID                    71    /* (IX_CC_V6V4_ENCAP_PKT_MICROBLOCK_ID + 1)   */

/* MPLS communication id  */
#define IX_CC_MPLS_ILM_PKT_MICROBLOCK_ID                72    /* (IX_CC_V6V4_DECAP_PKT_MICROBLOCK_ID + 1)   */

/* MPLS communication id  */
#define IX_CC_MPLS_FTN_PKT_MICROBLOCK_ID                73    /* (IX_CC_MPLS_ILM_PKT_MICROBLOCK_ID + 1)   */

/* 6-tuple classifier communication id  */
#define IX_CC_CLASSIFIER_6T_PKT_MICROBLOCK_ID           74    /* (IX_CC_MPLS_FTN_PKT_MICROBLOCK_ID + 1)   */

/* DSCP classifier communication id  */
#define IX_CC_CLASSIFIER_DSCP_PKT_MICROBLOCK_ID         75    /* IX_CC_CLASSIFIER_6T_PKT_MICROBLOCK_ID + 1)   */

/* DSCP Marker microblock communication id */
#define IX_CC_DSCP_MARKER_PKT_MICROBLOCK_ID             76    /* (IX_CC_CLASSIFIER_DSCP_PKT_MICROBLOCK_ID - 1) */

/* TC Meter microblock communication id */
#define IX_CC_TC_METER_PKT_MICROBLOCK_ID                77    /* (IX_CC_DSCP_MARKER_PKT_MICROBLOCK_ID - 1) */

/* WRED microblock communication id */
#define IX_CC_WRED_PKT_MICROBLOCK_ID                    78    /* (IX_CC_TC_METER_PKT_MICROBLOCK_ID + 1) */

/* QM core-to-ME packet communication id  */
#define IX_CC_QM_CORE_TO_MICROBLOCK_ID                  79    /* (IX_CC_WRED_PKT_MICROBLOCK_ID + 1)   */

/* NATPT communication ID */
#define IX_CC_NATPT_PKT_MICROBLOCK_ID                   80    /* (IX_CC_QM_CORE_TO_MICROBLOCK_ID + 1) */

/* AAL5/AAL2 Rx communication ID */
#define IX_CC_ATM_RX_MICROBLOCK_ID                      81     /* (IX_CC_NATPT_PKT_MICROBLOCK_ID + 1) */

#define END_SHARED_COMM_IDS                             82



/**
 * 
 * DESCRIPTION: This defines the IDs for core-to-microcode packet communication rings.
 *          The IDs part of underlaying infrastructure for packet and message communication. 
 *          This ids are defined for microblocks in file dl_system.h of 
 *          ingress and egress pipeline applications as follows: 
 *
 *          #define         EXCEPTION_RING_IN_0			    2
 *          #define         EXCEPTION_RING_IN_1			    3
 *          Right now same numbers are used in both files. the bindings.h looks like best place 
 *          for this defines to be. Other possible place - system app XML file.
 */

#define IX_CORE_TO_UBLOCK_PACKET_CTRL_RING_ID   2
#define IX_CORE_TO_UBLOCK_PACKET_DATA_RING_ID   3

/**
 * Exception codes for Ethernet Rx microblock and Core Component 
 */
#define IX_ARP_PACKET		0x01
#define IX_NON_IP_PACKET	0x02

/**
 * Exception code for Ethernet Tx microblock and Core Component 
 */
#define IX_NOL2_ADDR_IP_PACKET		0x03


/**
 * Exception code for AAL5 Rx and ATM Rx Core Component
 */
#define IX_ATM_OAM_CELL_RECEIVED	0x00		/* OAM Cell Received         */
#define IX_ATM_SIGNALLING_PKTS		0x01		/* Sig. packets rx on VC/CID */


#endif /* #ifndef __IX_CC_MICROENGINES_BINDINGS_H__*/
