/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation.All rights reserved.
 *
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 5
 *
 * = FILENAME
 *      ix_cc_prop_clients.h
 *
 * = DESCRIPTION
 *      Lists of clients subscribed to each interface property.
 *
 * = AUTHOR
 *       Aaron Luk
 *       aaron.luk@intel.com
 *
 * = CHANGE HISTORY
 *      10/3/2002 1:42:09 AM - creation time
 *
 * ============================================================================
 * $Id: ix_cc_prop_clients.h,v 1.8 2003/11/06 20:50:25 ktseng Exp $
 * $History: $
 */

#if !defined(__IX_CC_PROP_CLIENTS_H__)
#define __IX_CC_PROP_CLIENTS_H__

#include "ix_rm.h"
#include "bindings.h"

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

/* Property client lists. */

/* Clients to be notified on updates to IPv4 address. */
ix_communication_id ix_ipv4_addr_clients[] = {IX_CC_STKDRV_MSG_INPUT,IX_CC_IPV4_MSG_INPUT,IX_RM_COMM_MAKE_ID(IX_CC_ETH_TX_PROPERTY_MSG_INPUT, IX_PEER_SUBSYSTEM, 0),0};

/* Clients to be notified on updates to IPv4 gateway. */
ix_communication_id ix_ipv4_gway_clients[] = {IX_CC_STKDRV_MSG_INPUT,0};

/* Clients to be notified on updates to IPv4 broadcast address. */
ix_communication_id ix_ipv4_bcast_clients[] = {IX_CC_STKDRV_MSG_INPUT,0};

#if defined(IX_IPV6_SUPPORT)
/* Clients to be notified on updates to IPv6 address. */
ix_communication_id ix_ipv6_addr_clients[] = {IX_CC_STKDRV_MSG_INPUT,0};

/* Clients to be notified on updates to IPv46 gateway. */
ix_communication_id ix_ipv6_gway_clients[] = {IX_CC_STKDRV_MSG_INPUT,0};

/* Clients to be notified on updates to IPv6 broadcast address. */
ix_communication_id ix_ipv6_bcast_clients[] = {IX_CC_STKDRV_MSG_INPUT,0};
#endif /* defined(IX_IPV6_SUPPORT) */

/* Clients to be notified on updates to MTU. */
ix_communication_id ix_mtu_clients[] = {IX_CC_STKDRV_MSG_INPUT,0};

/* Clients to be notified on updates to MAC address. */
ix_communication_id ix_mac_addr_clients[] = {IX_CC_STKDRV_MSG_INPUT,IX_RM_COMM_MAKE_ID(IX_CC_ETH_TX_PROPERTY_MSG_INPUT, IX_PEER_SUBSYSTEM, 0),0};

/* Clients to be notified on updates to link speed. */
ix_communication_id ix_link_speed_clients[] = {IX_CC_STKDRV_MSG_INPUT,0};

/* Clients to be notified on updates to physical interface status. */
ix_communication_id ix_physical_if_status_clients[] = {IX_CC_STKDRV_MSG_INPUT,IX_CC_IPV4_MSG_INPUT, 
IX_CC_POS_RX_MSG_INPUT, IX_CC_ATM_POS_TX_PROPERTY_MSG_INPUT,0};

/* Clients to be notified on updates to link status. */
/**
 * Ethernet TX is the core component which invokes the property API to
 * update link status, and Ethernet TX is on the Egress subsystem.
 * Therefore the link status client commIDs should be input here as seen
 * from Egress, e.g. the stack driver commID will have the peer subsystem
 * bit set, since the stack driver is on Ingress.
 */
ix_communication_id ix_link_status_clients[] = {IX_RM_COMM_MAKE_ID(IX_CC_STKDRV_MSG_INPUT, IX_PEER_SUBSYSTEM, 0), IX_RM_COMM_MAKE_ID(IX_CC_FP_MODULE_MSG_INPUT, IX_PEER_SUBSYSTEM, 0), IX_RM_COMM_MAKE_ID(IX_CC_ETH_RX_MSG_INPUT, IX_PEER_SUBSYSTEM, 0), 0};


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_CC_PROP_CLIENTS_H__) */
