/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      ix_hardware_configuration.h
 *
 * = DESCRIPTION
 *      This file will define all the symbols related to hardware configuration. All definitions
 *      related to hardware will be kept in this file.
 *
 * = AUTHOR
 *      Adrian Georgescu
 *      adrian.georgescu@intel.com
 *
 * = CHANGE HISTORY
 *      4/22/2002 4:21:58 PM - creation time 
 * ============================================================================
 * $Id: ix_hardware_configuration.h,v 1.14 2003/08/18 21:47:43 ageorges Exp $
 */

#if !defined(__IX_HARDWARE_CONFIGURATION_H__)
#define __IX_HARDWARE_CONFIGURATION_H__


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */


/**
 * Define symbols for each hardware processor and board type.
 * These symbols are used in ix_symbols.h to define _IX_HARDWARE_TYPE_
 * and _IX_BOARD_TYPE_ according to IX_PLATFORM_... symbol definition
 * provided in compilers command line, e.g. -DIX_PLATFORM_2400
 */
#define _IX_HW_2400_        1   /* Sausalito hardware */
#define _IX_HW_2800_        2   /* Castine hardware */
#define _IX_HW_WESTPORT_    3   /* Westport hardware */

#define _IX_IXDP2400_  1   /* IXDP2400 (Angel Island) board */
#define _IX_IXDP2401_  2   /* IXDP2401 (Aleutian Islands) board */
#define _IX_IXDP2800_  3   /* IXDP2800 (Deer Island) board */
#define _IX_IXDP2801_  4   /* IXDP2801 (Aleutian Islands) board */


#if !defined(_IX_HARDWARE_TYPE_)

#error define a hardware type by assigning _IX_HARDWARE_TYPE_ one of the above hardware types

#endif /* !defined(_IX_HARDWARE_TYPE_) */


/**
 * DESCRIPTION: This symbol specifies the number of microengines in this hardware 
 *          configuration.
 */
#if (_IX_HARDWARE_TYPE_ == _IX_HW_2400_)

#define IX_MAX_MICROENGINE_NUMBER       8U 

#elif (_IX_HARDWARE_TYPE_ == _IX_HW_2800_)

#define IX_MAX_MICROENGINE_NUMBER       16U
    
#endif


/**
 * DESCRIPTION: This symbols specifies the number of DRAM controller channels.
 *
 */
#if (_IX_HARDWARE_TYPE_ == _IX_HW_2400_)

#define IX_DRAM_CHANNELS_NUMBER         1U 

#define IX_DRAM_BASE_ADDR_PHYSICAL_CHANNEL_0     0x00000000

#elif (_IX_HARDWARE_TYPE_ == _IX_HW_2800_)


/**
 * Even in fact there are three DRAM channels they are 128 bytes interleaved
 * and the memory space appears as a contiguous memory area. Given that, from
 * RM (and XScale applications point of view) the DRAM will appear as just having
 * one channel
 */
#define IX_DRAM_CHANNELS_NUMBER         1U 

#define IX_DRAM_BASE_ADDR_PHYSICAL_CHANNEL_0     0x00000000

#elif (_IX_HARDWARE_TYPE_ == _IX_HW_WESTPORT_)

#define IX_DRAM_SOUTH_CHANNELS_NUMBER   1U
#define IX_DRAM_NORTH_CHANNELS_NUMBER   1U

#endif


 
/**
 * DESCRIPTION: This symbols specifies the number of SRAM controller channels.
 *
 */
#if (_IX_HARDWARE_TYPE_ == _IX_HW_2400_)

#define IX_SRAM_CHANNELS_NUMBER         2U

#define IX_SRAM_BASE_ADDR_PHYSICAL_CHANNEL_0     0x80000000
#define IX_SRAM_BASE_ADDR_PHYSICAL_CHANNEL_1     0x90000000

#elif (_IX_HARDWARE_TYPE_ == _IX_HW_2800_)

#define IX_SRAM_CHANNELS_NUMBER         4U 

#define IX_SRAM_BASE_ADDR_PHYSICAL_CHANNEL_0     0x80000000
#define IX_SRAM_BASE_ADDR_PHYSICAL_CHANNEL_1     0x90000000
#define IX_SRAM_BASE_ADDR_PHYSICAL_CHANNEL_2     0xa0000000
#define IX_SRAM_BASE_ADDR_PHYSICAL_CHANNEL_3     0xb0000000
    
#elif (_IX_HARDWARE_TYPE_ == _IX_HW_WESTPORT_)

#define IX_SRAM_MSG_SOUTH_CHANNELS_NUMBER   1U
#define IX_SRAM_NORTH_CHANNELS_NUMBER       1U

#endif
    


/**
 * DESCRIPTION: This symbols specifies the number of SCRATCH controller channels.
 *
 */
#if (_IX_HARDWARE_TYPE_ == _IX_HW_2400_)

#define IX_SCRATCH_CHANNELS_NUMBER      1U 

#define IX_SCRATCH_BASE_PHYSICAL_ADDR   0xca000000

#define IX_SCRATCH_MEM_SIZE             16384U /* 16K */

#elif (_IX_HARDWARE_TYPE_ == _IX_HW_2800_)

#define IX_SCRATCH_CHANNELS_NUMBER      1U 

#define IX_SCRATCH_BASE_PHYSICAL_ADDR   0xca000000

#define IX_SCRATCH_MEM_SIZE             16384U /* 16K */

#elif (_IX_HARDWARE_TYPE_ == _IX_HW_WESTPORT_)
    
#define IX_SCRATCH_CHANNELS_NUMBER      1U 

#endif


/**
 * DESCRIPTION: This symbols specifies the size of local memory for each ME.
 *
 */
#if (_IX_HARDWARE_TYPE_ == _IX_HW_2400_)

#define IX_LOCAL_MEM_SIZE      (640 * 4) /* 640 double (long) words */

#elif (_IX_HARDWARE_TYPE_ == _IX_HW_2800_)

#define IX_LOCAL_MEM_SIZE      (640 * 4) 
    
#endif
    

/**
 * DESCRIPTION: This symbol specifies the maximum number of microblocks
 *          allowed in the system.
 *
 */
#define IX_MAX_MICROBLOCKS_NUMBER       64U


/**
 * DESCRIPTION: This symbol defines the maximum number of SRAM QArray entries
 *
 */
#define IX_MAX_SRAM_QARRAY_ENTRIES      64U 


/**
 * DESCRIPTION: This symbol defines the maximum number of SCRATCH rings
 *
 */
#define IX_MAX_SCRATCH_RINGS            16U 


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IX_HARDWARE_CONFIGURATION_H__) */

