/* brh80200.h - header for BRH80200 eval board */

/* Copyright 2001 Wind River Systems, Inc. */

/*
modification history
--------------------
01a,29mar02,vgd  written from brh80200.h, version 01a
*/

/*
DESCRIPTION
This file contains I/O address and related constants for the
IXDP2400 [latform
*/

#ifndef INCIXDP2400h
#define INCIXDP2400h


/*IXP VERSION for IXDP2400*/
#define IXP_VERSION "1.01.001" /* format "major.minor.build" */

/*---------------------------------------------------
 *  Defintion for Bit macro
 * --------------------------------------------------*/
#ifndef BIT
#define BIT(bitNumber)                    (1 << (bitNumber))
#endif /* BIT */

/*---------------------------------------------------
 *IXP2400 Register access Macros
 * --------------------------------------------------*/
#define IXP2400_REG_READ(reg,result) \
                ((result) = *(volatile UINT32 *)(reg))
#define IXP2400_REG_WRITE(reg,data)  \
                (*((volatile UINT32 *)(reg)) = (data))

/*---------------------------------------------------
 * IXDP2400 Constants...
 * --------------------------------------------------*/
#define    SZ_0                           0x00000000
#define    SZ_1                           0x00000001
#define    SZ_2                           0x00000002
#define    SZ_4                           0x00000004
#define    SZ_8                           0x00000008
#define    SZ_16                          0x00000010
#define    SZ_32                          0x00000020
#define    SZ_48                          0x00000030
#define    SZ_64                          0x00000040
#define    SZ_128                         0x00000080
#define    SZ_256                         0x00000100
#define    SZ_512                         0x00000200
#define    SZ_1K                          0x00000400
#define    SZ_2K                          0x00000800
#define    SZ_4K                          0x00001000
#define    SZ_8K                          0x00002000
#define    SZ_16K                         0x00004000
#define    SZ_32K                         0x00008000
#define    SZ_64K                         0x00010000
#define    SZ_128K                        0x00020000
#define    SZ_256K                        0x00040000
#define    SZ_512K                        0x00080000
#define    SZ_1M                          0x00100000
#define    SZ_2M                          0x00200000
#define    SZ_3M                          0x00300000
#define    SZ_4M                          0x00400000
#define    SZ_8M                          0x00800000
#define    SZ_16M                         0x01000000
#define    SZ_32M                         0x02000000
#define    SZ_48M                         0x03000000
#define    SZ_64M                         0x04000000
#define    SZ_128M                        0x08000000
#define	   SZ_224M			              0x0e000000 	
#define    SZ_256M                        0x10000000
#define    SZ_480M                        0x1e000000
#define    SZ_512M                        0x20000000
#define    SZ_1G                          0x40000000
#define    SZ_2G                          0x80000000


/*---------------------------------------------------
 * IXDP2400 global registers
 *---------------------------------------------------
 */

#define IXP2400_CAP_BASE	    0xC0000000
#define IXP2400_GLOBAL_CNTRL	(IXP2400_CAP_BASE + 0x4A00)


#define IXP2400_PROD_ID		    (IXP2400_GLOBAL_CNTRL + 0x00)
#define IXP2400_MISC_CONTROL 	(IXP2400_GLOBAL_CNTRL + 0x04)
#define IXP2400_MSF_ClK_CNTRL	(IXP2400_GLOBAL_CNTRL + 0x08)
#define IXP2400_RESET_1 	    (IXP2400_GLOBAL_CNTRL + 0x10) 
#define IXP2400_RESET_0 	    (IXP2400_GLOBAL_CNTRL + 0x0C) 
#define IXP2400_CLOCK_CONTROL 	(IXP2400_GLOBAL_CNTRL + 0x14) 
#define IXP2400_STRAP_OPTIONS	(IXP2400_GLOBAL_CNTRL + 0x18)

/*IXP2400_STRAP_OPTIONS register bits*/
#define CFG_PCI_BOOT_HOST	    BIT(2) /*(1 << 2)*/
#define CFG_PROM_BOOT		    BIT(1) /*(1 << 1)*/
#define NO_BOOT_ROM		        0x0

/*IXP2400_PROD_ID register bits*/
#define MAJ_PROD_TYPE		0x001F0000
#define MIN_PROD_TYPE 		0x0000FF00	
#define MAJ_REV	      		0x000000F0	
#define MIN_REV	      		0x0000000F
#define PROD_ID			    0xFFFFFFFF

/* IXP2400_MISC_CONTROL register bits*/
#define FLASH_WRITE_ENABLE		BIT(9) /*(1 << 9)*/
#define FLASH_ALIAS_DISABLE		BIT(8) /*(1 << 8) */


/*---------------------------------------------------
 * IXDP2400 GPIO...
 * --------------------------------------------------*/
#define IXP2400_GPIO_BASE	(IXP2400_CAP_BASE  + 0x10000)
#define	IXP2400_GPIO_PLR	(IXP2400_GPIO_BASE + 0x0)  /*Read only register to read the level/data on the pins irrespective of input or output*/
#define	IXP2400_GPIO_PDSR	(IXP2400_GPIO_BASE + 0x08) /*write a one to this register's bit to configure the corressponding pin as output*/
#define IXP2400_GPIO_POPR   (IXP2400_GPIO_BASE + 0x10)
#define IXP2400_GPIO_PDCR	(IXP2400_GPIO_BASE + 0x0c) /*write a one to this register's bit to configure corressponding bit as input*/
#define	IXP2400_GPIO_POSR	(IXP2400_GPIO_BASE + 0x14) /*write to this register's bit to set data high*/
#define IXP2400_GPIO_POCR	(IXP2400_GPIO_BASE + 0x18) /*write to this register's bit to set data low*/


/*------------------------------------------------------
 * IXDP2400 Scratch ..........
 *------------------------------------------------------
*/
#define IXP2400_SCRATCH_BASE		0xCA000000
#define IXP2400_SCRATCH_CSR_BASE 	0xC0004800

#ifndef _ASMLANGUAGE
extern UINT32 ixp2400XtalFreq;
#endif

#define IXP2400_DEFAULT_CLK_RATE	600000000 /*600 MHz*/


/*---------------------------------------------------
 * IXDP2400 UART...
 * --------------------------------------------------*/

#define	IXP2400_UART_XTAL	ixp2400XtalFreq


#define CPU_SPEED_600MHZ


/* UART  base address */
#define IXP2400_UART_ADDR               IXP2400_CAP_BASE+0x30000

/* Byte space between registers */
#define IXP2400_UART_REG_DELTA           1 

/* only one uart per NPU */
#define IXP2400_UART_N_UARTS             1


/*
 *-----------------------------------------------------------
 * IXP2400 SLOWPORT CSRs...
 *-----------------------------------------------------------
 */
#define IXP2400_SP_CSR_BASE		IXP2400_CAP_BASE + 0x80000

#define SP_CCR_OFF				0x0
#define SP_WTC1_OFF				0x4
#define SP_WTC2_OFF				0x8
#define SP_RTC1_OFF				0xC
#define SP_RTC2_OFF				0x10
#define SP_FAC_OFF				0x20
#define SP_FRM_OFF				0x24

#define IXP2400_SP_CCR			(IXP2400_SP_CSR_BASE + SP_CCR_OFF)
#define IXP2400_SP_WTC1			(IXP2400_SP_CSR_BASE + SP_WTC1_OFF)
#define IXP2400_SP_WTC2			(IXP2400_SP_CSR_BASE + SP_WTC2_OFF)
#define IXP2400_SP_RTC1			(IXP2400_SP_CSR_BASE + SP_RTC1_OFF)
#define IXP2400_SP_RTC2			(IXP2400_SP_CSR_BASE + SP_RTC2_OFF)
#define IXP2400_SP_FAC			(IXP2400_SP_CSR_BASE + SP_FAC_OFF)
#define IXP2400_SP_FRM			(IXP2400_SP_CSR_BASE + SP_FRM_OFF)

#define DIVIDE_BY_1			0x0
#define DIVIDE_BY_30		0x24
#define USE_32_BIT		    0x3
#define USE_8_BIT_DATA		0x1
#define USE_32_BIT_DATA		0x0

#define WR_HD				0x1
#define WR_PW				0x2
#define WR_SU				0x0
#define RD_HD				0x1
#define RD_PW				0x2
#define RD_SU				0x3
#define SP_WTC1_VAL			(WR_SU << 6 | WR_PW << 2 | WR_HD)
#define SP_RTC1_VAL			(RD_SU << 6 | RD_PW << 2 | RD_HD)

/* I2C defines */
#define ADUC812_DEV_ID		0x8C
#define TEMP_DEV_ID		    0x31

/*
 *------------------------------------------------------
 *CPLD registers
 *------------------------------------------------------
 */
#define IXP2400_SLOW_PORT_BASE	0xC4000000
#define CPLD_CSR_BASE           (IXP2400_SLOW_PORT_BASE + 0x3000000)

#define SYSTEM_LED_REG_OFF			0x0
#define ALPHANUM_DIS_DATA_OFF		0x4
#define CPLD_INT_OFF				0xC
#define CPLD_REV_OFF				0x10
#define SYS_CLK_M_OFF				0x14
#define SYS_CLK_N_OFF				0x18

#define SYSTEM_LED					(CPLD_CSR_BASE + SYSTEM_LED_REG_OFF)
#define ALPHANUM_DIS_DATA			(CPLD_CSR_BASE + ALPHANUM_DIS_DATA_OFF)
#define CPLD_INT					(CPLD_CSR_BASE + CPLD_INT_OFF)
#define CPLD_REV					(CPLD_CSR_BASE + CPLD_REV_OFF)
#define SYS_CLK_M					(CPLD_CSR_BASE + SYS_CLK_M_OFF)
#define SYS_CLK_N					(CPLD_CSR_BASE + SYS_CLK_N_OFF)


/* CPLD interrupt bits */
#define SLAVE_NPU_INT	(1 << 0)
#define ADUC812_INT		(1 << 7)


#define MEDIA_SLOWPORT_BASE		(IXP2400_SLOW_PORT_BASE + 0x2000000)
#define MSF_SLOWPORT_BASE		(IXP2400_SLOW_PORT_BASE + 0x2800000)	
/*
 *--------------------------------------------------------
 *IXDP2400 4 char Display/LCD Encodings
 *--------------------------------------------------------
 */

#define DISPLAY_NULL	0x20
#define DISPLAY_0		0x30
#define DISPLAY_1		0x31
#define DISPLAY_2		0x32
#define DISPLAY_3		0x33
#define DISPLAY_4		0x34
#define DISPLAY_5		0x35
#define DISPLAY_6		0x36
#define DISPLAY_7		0x37
#define	DISPLAY_8		0x38
#define DISPLAY_9		0x39
#define DISPLAY_A		0x41
#define DISPLAY_B		0x42
#define DISPLAY_C		0x43
#define DISPLAY_D		0x44
#define DISPLAY_E		0x45
#define DISPLAY_F		0x46
#define DISPLAY_G		0x47
#define DISPLAY_H		0x48
#define DISPLAY_I		0x49
#define DISPLAY_J		0x4A
#define DISPLAY_K		0x4B
#define DISPLAY_L		0x4C
#define DISPLAY_M		0x4D
#define DISPLAY_N		0x4E
#define DISPLAY_O		0x4F
#define DISPLAY_P		0x50
#define DISPLAY_Q		0x51
#define DISPLAY_R		0x52
#define DISPLAY_S		0x53
#define DISPLAY_T		0x54
#define DISPLAY_U		0x55
#define DISPLAY_V		0x56
#define DISPLAY_W		0x57
#define DISPLAY_X		0x58
#define DISPLAY_Y		0x59
#define DISPLAY_Z		0x5A

#define DIGIT_0		0x0
#define DIGIT_1		0x1
#define DIGIT_2		0x2
#define DIGIT_3		0x3

#define WRITE_TO_CHAR_RAM	0x48
#define DISPLAY_ASCII		0x18


/*---------------------------------------------------------------
 * IXDP2400 timer................
 *---------------------------------------------------------------
 */
#define IXP2400_TIMER_BASE	(IXP2400_CAP_BASE + 0x20000)

/*Timer Control Register offsets*/
#define T1_CTL	 0x0 
#define T2_CTL   0x04
#define T3_CTL   0x08
#define T4_CTL   0x0C

/*Timer Load Register offsets*/
#define T1_CLD   0x10
#define T2_CLD   0x14
#define T3_CLD   0x18
#define T4_CLD   0x1C

/*Timer Status Register offsets*/
#define T1_CSR   0x20
#define T2_CSR   0x24
#define T3_CSR   0x28
#define T4_CSR   0x2C

/*Timer Interupt Clear Register offsets*/
#define T1_CLR   0x30
#define T2_CLR   0x34
#define T3_CLR   0x38
#define T4_CLR   0x3C

/*Timer Watch dog enable Register offsets*/
#define TWDE     0x40

/*Timer register addreses*/
#define IXP2400_TIMER1_CONTROL	(IXP2400_TIMER_BASE + T1_CTL)
#define IXP2400_TIMER2_CONTROL	(IXP2400_TIMER_BASE + T2_CTL)
#define IXP2400_TIMER3_CONTROL	(IXP2400_TIMER_BASE + T3_CTL)
#define IXP2400_TIMER4_CONTROL	(IXP2400_TIMER_BASE + T4_CTL)

#define IXP2400_TIMER1_LOAD	    (IXP2400_TIMER_BASE + T1_CLD)
#define IXP2400_TIMER2_LOAD	    (IXP2400_TIMER_BASE + T2_CLD)
#define IXP2400_TIMER3_LOAD	    (IXP2400_TIMER_BASE + T3_CLD)
#define IXP2400_TIMER4_LOAD	    (IXP2400_TIMER_BASE + T4_CLD)

#define IXP2400_TIMER1_STATUS	(IXP2400_TIMER_BASE + T1_CSR)
#define IXP2400_TIMER2_STATUS	(IXP2400_TIMER_BASE + T2_CSR)
#define IXP2400_TIMER3_STATUS	(IXP2400_TIMER_BASE + T3_CSR)
#define IXP2400_TIMER4_STATUS	(IXP2400_TIMER_BASE + T4_CSR)


#define IXP2400_TIMER1_COUNTER_CLR	(IXP2400_TIMER_BASE + T1_CLR)
#define IXP2400_TIMER2_COUNTER_CLR	(IXP2400_TIMER_BASE + T2_CLR)
#define IXP2400_TIMER3_COUNTER_CLR	(IXP2400_TIMER_BASE + T3_CLR)
#define IXP2400_TIMER4_COUNTER_CLR	(IXP2400_TIMER_BASE + T4_CLR)

#define IXP2400_WATCHDOGTIMER_ENABLE	(IXP2400_TIMER_BASE + TWDE)

/*timer register bits*/
#define IXP2400_TIMER_INT_CLR	    0x01
#define IXP2400_TIMER_ACTIVATE      0x80
#define IXP2400_TIMER_PSS	        0x0
#define IXP2400_TIMER_PSS_16        0x4
#define IXP2400_TIMER_PSS_256       0x8
#define IXP2400_TIMER_PSS_GPIO      0xc

#define SYS_TIMER_CLK	ixp2400XtalFreq
#define AUX_TIMER_CLK	ixp2400XtalFreq

#define IXP2400_RELOAD_TICKS        10

#define TIMESTAMP_HZ	            SYS_TIMER_CLK		 /* timestamp counter freq */
#define SYS_TIMER_INT_LVL 	        (IXP2400_INT_VEC_T2)
#define AUX_TIMER_INT_LVL	        (IXP2400_INT_VEC_T3)

#define TICKS_PER_USEC				(SYS_TIMER_CLK/1000000)
/* Minimum rate at which the system clock can run */
#define SYS_CLK_RATE_MIN                  0xA

/* 1000 ticks, Maximum rate at which the system clock can run */
#define SYS_CLK_RATE_MAX                  0x3e8

/* Minimum rate at which the auxiliary clock can run */
#define AUX_CLK_RATE_MIN                  0xA

/* 1000 ticks, Maximum rate at which the auxiliary clock can run */
#define AUX_CLK_RATE_MAX                  0x3e8

/*---------------------------------------------------
 * IXDP2400 INTERRUPTS...
 *---------------------------------------------------
 */
#define IXP2400_INT_CTRL_BASE	        0xd6000000

#define IXP2400_IRQ_STATUS_REG          (IXP2400_INT_CTRL_BASE + 0x08)
#define IXP2400_IRQ_ENABLE_SET_REG      (IXP2400_INT_CTRL_BASE + 0x10)
#define IXP2400_IRQ_ENABLE_CLR_REG      (IXP2400_INT_CTRL_BASE + 0x18)

#define IXP2400_INT_VEC_BASE            (0x0)

#define IXP2400_INT_NUM_LEVELS          (32)
#define IXP2400_INT_MAX_LEVELS          (32)
#define IXP2400_INT_MODE                INT_NON_PREEMPT_MODEL

#define IXP2400_MASK_ALL_INT            (0xffffffff)
#define IXP2400_UNMASK_ALL_INT          (0x00000000)

#define IXP2400_INT_ALL_ENABLED 32      /*all levels enabled*/

/*interrupt vectors*/
#define IXP2400_INT_UART	            (2)
#define INT_VEC_PIL		                (15)                  
#define IXP2400_INT_VEC_T3              (6)	/* Timer 3 */ 
#define IXP2400_INT_VEC_T2              (5) /*	 Timer 2 */ 

#define CPLD_INGRESS_NPU_INT			0
#define CPLD_NIC_INT					1
#define CPLD_MEDIA_PCI_INT				2
#define CPLD_MEDIA_SLOWPORT_INT			3
#define CPLD_SF_PCI_INT					4
#define CPLD_SF_SLOWPORT_INT			5
#define CPLD_PMC_INT					6
#define CPLD_ADUC_INT					7

#define MAX_CPLD_INT						8
/*---------------------------------------------------
 * IXDP2400 FLASH...
 *---------------------------------------------------
 */

#define IXDP2400_BOOT_FLASH_ADDR          0x00000000
#define IXDP2400_BOOT_FLASH_SIZE          SZ_128M
#define IXP2400_FLASH_BASE		          0xC4000000	

/*---------------------------------------------------
 *IXDP2400 SDRAM controller...
 *---------------------------------------------------
 */

/*SDRAM Starting Address */
#define IXP2400_SDRAM_BASE_ADDR  0x00000000 /* SDRAM Address */
#define IXP2400_SDRAM_SIZE		 256
#define IXP2400_DDR_CNTRL_BASE	 0xD0000000
#define IXP2400_DU_CONTROL		 (IXP2400_DDR_CNTRL_BASE + 0x000)
#define IXP2400_DU_ECC_TEST		 (IXP2400_DDR_CNTRL_BASE + 0x018)
#define IXP2400_DU_INIT			 (IXP2400_DDR_CNTRL_BASE + 0x020)
#define IXP2400_DU_CONTROL2		 (IXP2400_DDR_CNTRL_BASE + 0x028)


#define IXP2400_DDR_RDDLYSEL_RECEN	 (IXP2400_DDR_CNTRL_BASE +	0x3C8)
#define IXP2400_DDR_RX_DLL			 (IXP2400_DDR_CNTRL_BASE +	0x650)
#define IXP2400_DDR_RX_DESKEW		 (IXP2400_DDR_CNTRL_BASE +	0x688)
#define DDR_RCOMP_IO_CONFIG			 (IXP2400_DDR_CNTRL_BASE +	0x3C0)
#define DDR_RCOMP_IO_CONFIG_VAL		0x312121

/*SDRAM controller register bits*/
#define TCL						(1 << 2)
#define TRAS					(0 << 4)
#define TRC						(1 << 6)
#define TWTR					(1 << 10)
#define REF_EN					(1 << 14)
#define NUM_ROW_COL				(3 << 23)
#define NUM_SIDES				(1 << 26)
#define TWR						(0 << 27)
#define TRRD					(1 << 28)
#define	TRFC					(0 << 29)
#define REF_COUNT				(11 << 15)
#define DU_CONTROL_VAL			(TRRD | NUM_SIDES | NUM_ROW_COL | REF_COUNT | REF_EN | TWTR | TRC | TCL)

/*DRAM initialization register bits*/
#define CKE					(1 << 16)
#define PRECHARGE			(1 << 29)
#define REFRESH				(1 << 30)
#define LD_MODE_REG			(1 << 31)

#define DU_INIT_VAL			(REFRESH | PRECHARGE | CKE)

#define RESET_DLL					0x100
#define LOAD_MODE_NORMAL			0x62
#define PRECHARGE_ALL				(1 << 10)
#define EXT_LD_MODE					(1 << 12)
#define SIDE0						(1 << 14)
#define SIDE1						(1 << 15)
#define CKE							(1 << 16)


#define DDR_RX_DLL_VAL				0x11
#define DDR_RX_DESKEW_VAL			0x11
#define DDR_RDDLYSEL_RECEN_VAL		0x11

/************/

/*DRAM ECC test register bits*/
#define DISABLE_CHK			(1 << 8)


/*---------------------------------------------------
 *IXDP2400 MSF CSRs...
 * --------------------------------------------------
 */

#define IXP2400_MSF_CSR_BASE 			0xC8000000
/*---------------------------------------------------
 *IXDP2400 SRAM controller...
 * --------------------------------------------------
 */
/* sram registers */
#define IXP2400_QDR_CH_CSR_BASE				0xCC010000

#define IXP2400_QDR_CH_CONTROL_OFF			0x0
#define IXP2400_QDR_RX_DLL_OFF				0x228
#define IXP2400_QDR_RD_PTR_OFF				0x240
#define IXP2400_QDR_RX_DESKEW_OFF			0x244

#define IXP2400_QDR_RCMP_SETUP_CONTROL_OFF	0x300
#define IXP2400_QDR_RCOMP_OFF				0x318 /*need to find the right name of the register. for now calling it rcomp*/

#define IXP2400_SRAM_BASE		    0x80000000

#define SRAM_DIVISOR			    0x3
#define SDRAM_DIVISOR			    0x4

#define PIPELINE					(1 << 10)
#define PARITY_ENABLE			    (1 << 3)

#define MIN_SRAM_SIZE			    (4 * 1024 * 1024) /* 4MB SRAM */
#define MAX_SRAM_SIZE			    (32 * 1024 * 1024) /* 32MB SRAM */
#define MAX_SRAM_SIZE_CSR_VAL	    (5 << 7)
#define SRAM_COMPARE_LOCATION	    0x101000
#define SRAM_TEST_LOCATION		    0x1000
#define SRAM_SIZE_LOOPS			    4
#define TEST_PAT				    0x12345678
#define COMPARISON_PAT			    0xAA55AA55
#define MAX_QDR_CHANNEL				2

#define QDR_DLL_FREQ_SEL 0x3
#define QDR_DLL_TAP_SEL 0x2
#define QDR_RD_PTR		0x1

/* Define macros for fixing 8 and 16 bit addresses for big-endian operation */
#if (_BYTE_ORDER == _BIG_ENDIAN)
#define FIX_ADDR_16(x) ((x) ^ 2)
#define FIX_ADDR_32(x) ((x) ^ 3)
#else
#define FIX_ADDR_16(x) (x)
#define FIX_ADDR_32(x) (x)
#endif



/*---------------------------------------------------
 *IXDP2400 PCI unit...
 *---------------------------------------------------
 */

#define IXP2400_VENDOR_ID		0x8086
#define IXP2400_DEVICE_ID		0x9001

#define PCI_CONFIG1_BASE        0xDB000000 /* Config cycle 1 space */
#define PCI_CONFIG0_BASE        0xDA000000 /* Config cycle 0 space */
#define PCI_IO_BASE             0xD8000000 /* I/O cycle space */
#define PCI_MEM_BASE            0xE0000000 /* Mem cycle space */

#define INIT_COMP				(1 << 21)
#define RESET_DRAM0				(1 << 11)
#define RESET_SRAM1				(1 << 4)
#define RESET_SRAM0				(1 << 3)
#define PCIRST					(1 << 2)
#define RESET_PCI				(1 << 1)
#define RESET_XSCALE			1
#define RESET_SDRAM_SRAM		(RESET_DRAM0 | RESET_SRAM0 | RESET_SRAM1)
#define NOT_RESET_PCI			~(PCIRST | RESET_PCI)

#define IXP2400_PCI_CFG_REG_BASE	    0xDE000000 /* config register base */
#define IXP2400_PCI_CSR_BASE		    0xDF000000 /* Base of PCI Unit CSR's */

#define IXP2400_PCI_VENDOR_ID		    FIX_ADDR_16(IXP2400_PCI_CFG_REG_BASE + 0x000)
#define IXP2400_PCI_DEVICE_ID		    FIX_ADDR_16(IXP2400_PCI_CFG_REG_BASE + 0x002)
#define IXP2400_PCI_CMD_STAT			(IXP2400_PCI_CFG_REG_BASE + 0x004)
#define IXP2400_PCI_CACHE_LAT_HDR_BIST	(IXP2400_PCI_CFG_REG_BASE + 0x00C)
#define IXP2400_PCI_CSR_BAR             (IXP2400_PCI_CFG_REG_BASE + 0x010)
#define IXP2400_PCI_SRAM_BAR            (IXP2400_PCI_CFG_REG_BASE + 0x014)
#define IXP2400_PCI_DRAM_BAR            (IXP2400_PCI_CFG_REG_BASE + 0x018)
#define IXP2400_PCI_RCOMP_OVER			(IXP2400_PCI_CFG_REG_BASE + 0x060)
#define IXP2400_OUT_INT_STATUS          (IXP2400_PCI_CSR_BASE + 0x030)
#define IXP2400_OUT_INT_MASK            (IXP2400_PCI_CSR_BASE + 0x034)
#define IXP2400_PCI_INT_LAT             (IXP2400_PCI_CSR_BASE + 0x03c)
#define IXP2400_MAILBOX0                (IXP2400_PCI_CSR_BASE + 0x050)
#define IXP2400_MAILBOX1                (IXP2400_PCI_CSR_BASE + 0x054)
#define IXP2400_MAILBOX2                (IXP2400_PCI_CSR_BASE + 0x058)
#define IXP2400_MAILBOX3                (IXP2400_PCI_CSR_BASE + 0x05c)
#define IXP2400_DOORBELL                (IXP2400_PCI_CSR_BASE + 0x060)
#define IXP2400_DOORBELL_SETUP          (IXP2400_PCI_CSR_BASE + 0x064)
#define IXP2400_CHAN_1_BYTE_COUNT       (IXP2400_PCI_CSR_BASE + 0x080)
#define IXP2400_CHAN_1_PCI_BAR          (IXP2400_PCI_CSR_BASE + 0x084)
#define IXP2400_CHAN_1_DRAM_ADDR        (IXP2400_PCI_CSR_BASE + 0x088)
#define IXP2400_CHAN_1_DESC_PTR         (IXP2400_PCI_CSR_BASE + 0x08C)
#define IXP2400_CHAN_1_CONTROL          (IXP2400_PCI_CSR_BASE + 0x090)
#define IXP2400_DMA_INF_MODE            (IXP2400_PCI_CSR_BASE + 0x09C)
#define IXP2400_CHAN_2_BYTE_COUNT       (IXP2400_PCI_CSR_BASE + 0x0A0)
#define IXP2400_CHAN_2_PCI_BAR          (IXP2400_PCI_CSR_BASE + 0x0A4)
#define IXP2400_CHAN_2_DRAM_ADDR        (IXP2400_PCI_CSR_BASE + 0x0A8)
#define IXP2400_CHAN_2_DESC_PTR         (IXP2400_PCI_CSR_BASE + 0x0AC)
#define IXP2400_CHAN_2_CONTROL          (IXP2400_PCI_CSR_BASE + 0x0B0)
#define IXP2400_PCI_SRAM_BAR_MASK	    (IXP2400_PCI_CSR_BASE + 0x0FC)
#define IXP2400_DRAM_BASE_ADDR_MASK     (IXP2400_PCI_CSR_BASE + 0x100)
#define IXP2400_PCI_CONTROL             (IXP2400_PCI_CSR_BASE + 0x13C)
#define IXP2400_PCI_ADDR_EXT            (IXP2400_PCI_CSR_BASE + 0x140)
#define IXP2400_PCI_XSCALE_INT_ENABLE   (IXP2400_PCI_CSR_BASE + 0x15C)

#define IXP2400_PCI_XSCALE_INT_PILM	    (3 << 26)

#define IXP2400_PCI_CONTROL_XS_INT      (1 << 24) /* Enable the PCI INTA*/
#define IXP2400_PCI_CONTROL_BE_DEO      (1 << 22) /* PCI Big Endian Data Enable Out */
#define IXP2400_PCI_CONTROL_BE_DEI      (1 << 21) /* PCI Big Endian Data Enable In */
#define IXP2400_PCI_CONTROL_BE_BEO      (1 << 20) /* PCI Big Endian Byte Enable Out */
#define IXP2400_PCI_CONTROL_BE_BEI      (1 << 19) /* PCI Big Endian Byte Enable In */



#define IXP2400_PCI_CONTROL_PNR			(1 << 17) /* PCI Not Reset bit of SA_CONTROL */
#define IXP2400_PCI_CONTROL_CFG_RST_DIR (1 << 28) /* PCI Centrl Function bit */

/* 16 bit PCI registers */

#define IXP2400_PCI_COMMAND FIX_ADDR_16(IXP2400_PCI_CSR_BASE + 0x04)

/* Define PCI_COMMAND bits */

#define IO_SPACE_ENABLE				(1 << 0)
#define MEM_SPACE_ENABLE			(1 << 1)
#define BUS_MASTER_ENABLE			(1 << 2)
#define WR_INV_ENABLE				(1 << 4)
#define SERR_ENABLE					(1 << 8)
#define PCI_CMD_STAT_VAL			(BUS_MASTER_ENABLE | MEM_SPACE_ENABLE | IO_SPACE_ENABLE) /*vgd*/

#define XSIM	~(1 << 1)

#ifdef INCLUDE_PCI
/*
 * PCI definitions
 * 
 * First, defines for generic pciIoMapLib.c code
 * 
 * Cache Line Size - in number of 32-bit words
 * It might be thought that we would set this to (_CACHE_ALIGN_SIZE/4), but,
 * in fact, the areas of memory mapped for use either by PCI I/O and
 * PCI Configuration  and the area of (ARM) memory used by the Ethernet
 * controller are marked as non-cacheable, so we should set it to zero.
 */

#define PCI_CLINE_SZ    0

/*
 * Latency Timer value
 * A sensible value for this depends what we are using PCI for.
 * (Rather arbitrarily) leave it at 0 (the reset default value)
 */

#define PCI_LAT_TIMER           0x0

#define SWAP16(x) ((((x) << 8) | ((x) >> 8)) & 0xFFFF)
#define SWAP32(x)               \
  (((x) << 24) |                \
  (((x) & 0x0000FF00) << 8) |   \
  (((x) & 0x00FF0000) >> 8) |   \
  (((unsigned int)(x)) >> 24))

/*
 * Dummy interrupt number for pciIntConnect(): must be passed a real
 * interrupt vector number. This allows for non-linear mappings
 * between interrupts, slots etc.
 */

#define INT_NUM_IRQ0    0

/* memory map as seen by the CPU on the local bus */
#ifdef INCLUDE_EXTRA_PCI_MAPPING
#define CPU_PCI_MEM_ADRS    PCI_MEM_BASE /* PCI memory space base */
#define CPU_PCI_MEM_SIZE    0x20000000   /* 512 Mbytes */
#endif

#define CPU_PCI_IO_ADRS     PCI_IO_BASE  /* PCI I/O space base */
#define CPU_PCI_IO_SIZE     0x2000000    /* 32 Mbytes */

#define CPU_PCI_CNFG_ADRS   PCI_CONFIG0_BASE  /* type 0 PCI config space */
#define CPU_PCI_CNFG_SIZE   0x1000000         /* 16 Mbytes */

#define CPU_PCI_CNFG_1_ADRS PCI_CONFIG1_BASE  /* type 1 PCI config space */
#define CPU_PCI_CNFG_1_SIZE 0x1000000         /* 16 Mbytes */


/* PCI view of PCI I/O space for PCI devices */
#define PCI_IO_ADRS         0x00000000  /* base of PCI I/O address */

/* PCI bus resources */
#define PCI_IO_SIZE         0x80            /* PCI I/O size */

/* PCI device 0: slot I/O 4 */
#define PCI_MEM_ADR0        0x00            /* Memory base for CSR */
#define PCI_IO_ADR0         PCI_IO_ADRS     /* I/O base for CSR 32Bytes */
#define PCI_IO_SIZE0        PCI_IO_SIZE     /* I/O size for CSR */
#define PCI_INT_LVL0        INT_VEC_PIL     /* Interrupt level */
#define PCI_INT_VEC0        IVEC_TO_INUM(PCI_INT_LVL0) /* Interrupt vector */

/* PCI device 1: slot I/O 3 */
#define PCI_MEM_ADR1        0x00
#define PCI_IO_ADR1         PCI_IO_ADRS
#define PCI_IO_SIZE1        PCI_IO_SIZE
#define PCI_INT_LVL1        INT_VEC_PIL
#define PCI_INT_VEC1        IVEC_TO_INUM(PCI_INT_LVL1)

/* PCI device 2: slot I/O 2 */
#define PCI_MEM_ADR2        0x00
#define PCI_IO_ADR2         PCI_IO_ADRS
#define PCI_IO_SIZE2        PCI_IO_SIZE
#define PCI_INT_LVL2        INT_VEC_PIL
#define PCI_INT_VEC2        IVEC_TO_INUM(PCI_INT_LVL2)

/* PCI device 3: slot I/O 1 */
#define PCI_MEM_ADR3        0x00
#define PCI_IO_ADR3         PCI_IO_ADRS
#define PCI_IO_SIZE3        PCI_IO_SIZE
#define PCI_INT_LVL3        INT_VEC_PIL
#define PCI_INT_VEC3        IVEC_TO_INUM(PCI_INT_LVL3)

/*
 * N.B. Only the following values have been tested. In particular, no
 * testing has been performed with more than one PCI bus, i.e. Type 1
 * configuration.
*/

#define PCI_MAX_DEV     8
#define PCI_MAX_BUS     2       /* two bus */


/* Allocate Device Space (Offsets) for PCI devices */
#define LAN_DEV_SPACE       0
#define PCI_IO_LN_ADRS      (PCI_IO_ADRS + LAN_DEV_SPACE)


/* Allocated base address of h/w devices as seen from CPU */
#define LAN_BASE_ADRS       (CPU_PCI_IO_ADRS + LAN_DEV_SPACE)


/* Create configuration address */
#define MAKE_CFG_ADDR(bus, device, func, offset) ((bus) ? \
    ((UINT32)((bus & 0xFF) << 16) | (UINT32)((device & 0x1F) << 11) | \
     (UINT32)((func & 0x7) << 8)  | (UINT32) (offset & 0xFC)) : \
    ((UINT32)(1 << ((device & 0x1F) + 16)) | (UINT32)((func & 0x7) << 8) | \
     (UINT32) (offset & 0xFC)))
#endif /* ifdef INCLUDE_PCI */

 
/* slave's csr as seen from master */
#define SLAVE_GLOBAL_CONTROL_BASE	0x4A00

#define SLAVE_MISC_CONTROL			(SLAVE_GLOBAL_CONTROL_BASE + 0x4)
#define SLAVE_IXP_RESET0			(SLAVE_GLOBAL_CONTROL_BASE + 0xC)
#define SLAVE_CCR					(SLAVE_GLOBAL_CONTROL_BASE + 0x14)
#define SLAVE_STRAP_OPTIONS			(SLAVE_GLOBAL_CONTROL_BASE + 0x18)

#define SLAVE_SLOW_PORT_CSR_BASE	0x80000

#define SLAVE_SCRATCH_BASE			0xF0000

#define SLAVE_QDR_CH_BASE			0xF9000

#define SLAVE_DRAM_CH0_BASE			0xFD800

#define SLAVE_DU_CONTROL			(SLAVE_DRAM_CH0_BASE + 0x0)
#define SLAVE_DU_ECC_TEST			(SLAVE_DRAM_CH0_BASE + 0x18)
#define SLAVE_DU_INIT				(SLAVE_DRAM_CH0_BASE + 0x20)
#define SLAVE_DU_CONTROL2			(SLAVE_DRAM_CH0_BASE + 0x28)
#define SLAVE_DU_RCOMP_IO_CONFIG	(SLAVE_DRAM_CH0_BASE + 0x3C0)
#define SLAVE_DU_RDDLYSEL_RECEN		(SLAVE_DRAM_CH0_BASE + 0x3C8)
#define SLAVE_DU_RX_DLL				(SLAVE_DRAM_CH0_BASE + 0x650)
#define SLAVE_DU_RX_DESKEW			(SLAVE_DRAM_CH0_BASE + 0x688)

#define SLAVE_PCI_CSR_BASE			0xFE000

#define SLAVE_PCI_OUT_INT_MASK		(SLAVE_PCI_CSR_BASE + 0x34)
#define SLAVE_MAILBOX_0				(SLAVE_PCI_CSR_BASE + 0x50)
#define SLAVE_PCI_CONTROL			(SLAVE_PCI_CSR_BASE + 0x13C)
#define SLAVE_PCI_ADDR_EXT			(SLAVE_PCI_CSR_BASE + 0x140)

#define BOOT_THRU_BM		        (1<<31) /*no boot monitor flag*/
#define BOOT_DONT_INIT_PCI          (1 << 30)
#define BOOT_CMP_IMAGE				(1<<29)

#define TB_VENDOR_ID				0x8086
#define TB_DEVICE_ID				0xB154

#define SDRAM_END		            0x20000000
#define SDRAM_HIGH_VIRT		        0x02000000
#define SDRAM_HIGH_PHY		        0x1e000000 

#define CONFIG_DATA_VALID	        0x12341234

/***** UART defines used in romInit****/

#ifdef CPU_SPEED_600MHZ
	#define BaudRateDivisor_115200     0x1B
	#define BaudRateDivisor_57600      0x36 
#else
	#define BaudRateDivisor_115200     0x12
	#define BaudRateDivisor_57600      0x24 
#endif


#define UARTLCR_DivisorLatchAccess 0x80
#define UART_LineControl           0x0C 
#define UART_DivisorLatchLSB       0x00	
#define UART_DivisorLatchMSB       0x04	
#define UARTLCR_CharLength8        0x03  
#define UARTLCR_StopBits1          0x00  
#define UART_InterruptEnable       0x04 
#define UARTFCR_Enable             (1<<0)
#define UART_FIFOControl           0x08
#define UARTFCR_RXReset            (1<<1)
#define ARTFCR_TXReset             (1<<2)
#define UARTFCR_Mode0RXRDYTXRDY    (0<<3)
#define UARTFCR_RXTrigger1         (0<<6)
#define UARTMCR_DTRActive          (1<<0)
#define UART_LineStatus            0x14
#define UART_Transmit              0x0
#define UART_Receive               0x0
#define UARTLSR_TXHoldingEmpty     (1<<5)
#define ROM_FIX_ADDR_MASK			0x2fffffff


#define MMU_Control_M	0x200
#define MMU_Control_R  0x001
#define MMU_Control_C  0x004
#define MMU_Control_BTB 0x800

#define ROM_MMU_TABLE_ADR			0xc4004000

#ifndef _ASMLANGUAGE
UINT32 slave_load_adrs;
#endif


#define SLAVE_LOAD_ADRS 			slave_load_adrs	
#define SLAVE_RAM_MMU_TABLE_ADDR 	(SLAVE_LOAD_ADRS+0x4000)

#define SLAVE_ENTRY_ADRS_LOW	(SDRAM_HIGH_PHY + RAM_LOW_ADRS +0x8000 )
#define SLAVE_ENTRY_ADRS_HI	(SDRAM_HIGH_PHY + RAM_HIGH_ADRS + 0x8000 )


/* Assembly specific defs */
#ifndef _ASMLANGUAGE

#endif
#endif  /* INCIXDP2400h */
