/* ixdp2000.h - header for ixdp2000 board */

/* Copyright 2001 Wind River Systems, Inc. */

/*
modification history
--------------------

01a, 16apr02, eme created from 01a of ixdp2400
01b, 13mar03, tep removed AI CPLD specifics an added DI CPLD defines,
added cpld int reg bit defines, changed PCI_MEM_SPACE define from 768 to 512Mb,
removed ifdef INCLUDE_EXTRA_PCI_MAPPING
01c, 13mar03, tep removed redundant defines for cpld and slowport base
01d,18mar03, tep changed CPLD_INT_EGRNIC to CPLD_INT_NIC
01e,23mar03,tep Changed SRAM_CHANNEL_SIZE to 64Mb
01f,26mar03,tep Changed all "A0_2800" conditionals to "REV_A0_2800"
01e, 27mar03,tep Added clock control reg bitfields and many other
macros for sram controller regs for SRAM init code
01f,apr,03 Deleted define for MEDIA_SLOWPORT_BASE, changed the define
for MSF_SLOWPORT_BASE from 2800000 to 2000000.
01e,22apr03,tep Changed the sram boot location to second sram channel
and increased sram channel size to 8Mb.
01f,27apr03,tep added STANDALONE_BOOT wrapers for MMU_TABLE_ADRS,
BOOTROM_OFFSET, and BOOTROM_TEXT_ADRS.
01G,291pr03,tep Added separate media_slowport_base and fabric_slowport_base
macros
01h,08may03,tep Added defines for SCRATCH bit set/clr/add/enq/rng
01i,13may03,tep Corrected the defice for PCI device ID.  Was 9001 
(2400) is now 9004 (2800).
01j,03jun03,tep Incremented the major rev in ixpversion string to 2.00
*/
/*
DESCRIPTION
This file contains I/O address and related constants for the
ixdp2000 platform.
*/

#ifndef INCixdp2000h
#define INCixdp2000h

#include "config.h"

#ifdef __cplusplus
IMPORT "C" {
#endif

/*IXP VERSION for IXDP2000*/
#define IXP_VERSION "2.00.NoBldNum"

/*
 *  Macro Defintions
 */

#ifndef BIT
#define BIT(bitNumber)                    (1 << (bitNumber))
#endif /* BIT */

/*
 * Register access Macros
 */
#define IXP2000_REG_GET(reg,result) \
                ((result) = *(volatile UINT32 *)(reg))
#define IXP2000_REG_PUT(reg,data)  \
                (*((volatile UINT32 *)(reg)) = (data))

/*
 * Constants
 */
#define    SZ_0                           0x00000000
#define    SZ_1                           0x00000001
#define    SZ_2                           0x00000002
#define    SZ_4                           0x00000004
#define    SZ_8                           0x00000008
#define    SZ_16                          0x00000010
#define    SZ_32                          0x00000020
#define    SZ_48                          0x00000030
#define    SZ_64                          0x00000040
#define    SZ_128                         0x00000080
#define    SZ_256                         0x00000100
#define    SZ_512                         0x00000200
#define    SZ_1K                          0x00000400
#define    SZ_2K                          0x00000800
#define    SZ_4K                          0x00001000
#define    SZ_8K                          0x00002000
#define    SZ_16K                         0x00004000
#define    SZ_32K                         0x00008000
#define    SZ_64K                         0x00010000
#define    SZ_128K                        0x00020000
#define    SZ_256K                        0x00040000
#define    SZ_512K                        0x00080000
#define    SZ_1M                          0x00100000
#define    SZ_2M                          0x00200000
#define    SZ_3M                          0x00300000
#define    SZ_4M                          0x00400000
#define    SZ_8M                          0x00800000
#define    SZ_16M                         0x01000000
#define    SZ_32M                         0x02000000
#define    SZ_48M                         0x03000000
#define    SZ_64M                         0x04000000
#define    SZ_128M                        0x08000000
#define	   SZ_224M			              0x0e000000 	
#define    SZ_256M                        0x10000000
#define    SZ_480M                        0x1e000000
#define    SZ_512M                        0x20000000
#define	   SZ_768M						  0x30000000
#define    SZ_1G                          0x40000000
#define    SZ_2G                          0x80000000


/*
 * IXDP2000 global registers
 */
#define IXP2000_CAP_BASE	    0xC0000000
#define IXP2000_CAP_SIZE	SZ_32M
#define IXP2000_GLOBAL_CNTRL	(IXP2000_CAP_BASE + 0x4A00)

#define IXP2000_PRODUCT_ID		    (IXP2000_GLOBAL_CNTRL + 0x00)
#define IXP2000_MISC_CONTROL 	(IXP2000_GLOBAL_CNTRL + 0x04)
#define IXP2000_MSF_CLK_CNTRL	(IXP2000_GLOBAL_CNTRL + 0x08)
#define IXP2000_RESET_1 	    (IXP2000_GLOBAL_CNTRL + 0x10) 
#define IXP2000_RESET_0 	    (IXP2000_GLOBAL_CNTRL + 0x0C) 
#define IXP2000_CLOCK_CONTROL 	(IXP2000_GLOBAL_CNTRL + 0x14) 
#define IXP2000_STRAP_OPTIONS	(IXP2000_GLOBAL_CNTRL + 0x18)

/* bit defintions within IXP2000_STRAP_OPTIONS */
#define CFG_PLL_MASK		(0x3f << 8)
#define CFG_PCI_BOOT_HOST BIT(2) /* indicates who will configure the PCI devs */
#define CFG_PROM_BOOT	  BIT(1) /* indicates if Boot ROM is present */
#define CFG_RST_DIR	  	  BIT(0) /* the direction of PCI_RST# signal */

/* CLOCK_CONTROL bitfields */
#define CLOCK_CONTROL_Sram0ClkMask           (0xf << 0)
#define CLOCK_CONTROL_Sram1ClkMask           (0xf << 4)
#ifdef IXP2800
#define CLOCK_CONTROL_Sram2ClkMask           (0xf << 8)
#define CLOCK_CONTROL_Sram3ClkMask           (0xf << 12)
#endif
#ifndef IXP2800
#define CLOCK_CONTROL_DramClkMask            (0xf << 16)
#endif
#ifdef IXP2800
#define CLOCK_CONTROL_MsfClkMask             (0xf << 20)
#define CLOCK_CONTROL_ApbClkMask             (0xf << 24)
#define CLOCK_CONTROL_ApbClkShift            24
#endif

/* bit definitions (i.e., masks) within IXP2000_PRODUCT_ID */
#define MAJ_PROD_TYPE     0x001F0000 /* IXP2800 = 0 */
#define MIN_PROD_TYPE     0x0000FF00 /* Castine = 0 */  
#define MAJ_REV           0x000000F0 /* current Maj Revision */ 
#define MIN_REV           0x0000000F /* current Min Revision */
#define PROD_ID			  0xFFFFFFFF

/* bit definitions within IXP2000_MISC_CONTROL */
#define FLASH_WR_EN   		BIT(9) /* provide protection against SW bugs */
#define FLASH_ALIAS_DISABLE     BIT(8) /* allows core to boot from addr 0x0 */

/*
 * GPIO Register Definitions
 */
#define GPIO_BASE_ADRS  (IXP2000_CAP_BASE + 0x10000) /* GPIO Base Address */
#define IXP2000_GPIO_PLR   (GPIO_BASE_ADRS + 0x00)   /* pin level reg */
#define IXP2000_GPIO_PDSR  (GPIO_BASE_ADRS + 0x08)   /* pin direction set reg */
#define IXP2000_GPIO_POPR  (GPIO_BASE_ADRS + 0x10)
#define IXP2000_GPIO_PDCR  (GPIO_BASE_ADRS + 0x0c)   /* pin direction clear reg */
#define IXP2000_GPIO_POSR  (GPIO_BASE_ADRS + 0x14)   /* output data set reg */
#define IXP2000_GPIO_POCR  (GPIO_BASE_ADRS + 0x18)   /* output data clear reg */
#define IXP2000_GPIO_INST  (GPIO_BASE_ADRS + 0x40)   /* Int status reg */

/*
 * Scratchpad Memory and Register Definitions
 */
#define IXP2000_SCRATCH_BASE 		0xCA000000
#define IXP2000_SCRATCH_SET_BASE 	(IXP2000_SCRATCH_BASE+0x00400000)
#define IXP2000_SCRATCH_CLR_BASE 	(IXP2000_SCRATCH_BASE+0x00800000)
#define IXP2000_SCRATCH_ADD_BASE 	(IXP2000_SCRATCH_BASE+0x00C00000)
#define IXP2000_SCRATCH_SUB_BASE 	(IXP2000_SCRATCH_BASE+0x01000000)
#define IXP2000_SCRATCH_RNG_BASE 	(IXP2000_SCRATCH_BASE+0x01400000)

#define IXP2000_SCRATCH_SIZE 		SZ_16K
#define IXP2000_SCRATCH_SET_SIZE 	IXP2000_SCRATCH_SIZE
#define IXP2000_SCRATCH_CLR_SIZE 	IXP2000_SCRATCH_SIZE
#define IXP2000_SCRATCH_ADD_SIZE 	IXP2000_SCRATCH_SIZE
#define IXP2000_SCRATCH_SUB_SIZE 	IXP2000_SCRATCH_SIZE
#define IXP2000_SCRATCH_RNG_SIZE 	IXP2000_SCRATCH_SIZE

#define IXP2000_SCRATCH_CSR_BASE (IXP2000_CAP_BASE + 0x4800) /* Scratchpad mem base */

#define VXWORKS_TEXT_ADRS		0xCA000100

/*
 * UART Register Definitions
 */
#ifndef _ASMLANGUAGE
IMPORT UINT32 ixp2000XtalFreq;
#endif

#define IXP2000_DEFAULT_CLK_RATE	600000000 /*600 MHz*/

#define	IXP2000_UART_XTAL	ixp2000XtalFreq
#define CPU_SPEED_600MHZ

#define IXP2000_UART_BASE      (IXP2000_CAP_BASE+0x30000) /* base address */
#define IXP2000_UART_N   1			 /* UART per NPU */
#define IXP2000_UART_REG_DELTA 1     /* Byte space between registers */

/*
 * SLOWPORT CSRs
 */
#define SLOWPORT_CSR_BASE  	 (IXP2000_CAP_BASE + 0x80000) /* slowport CSR base */
#define IXP2000_SP_CCR   (SLOWPORT_CSR_BASE + 0x00) /* clock configuration reg */
#define IXP2000_SP_WTC1  (SLOWPORT_CSR_BASE + 0x04) /* write timing control reg */
#define IXP2000_SP_WTC2  (SLOWPORT_CSR_BASE + 0x08) 
#define IXP2000_SP_RTC1  (SLOWPORT_CSR_BASE + 0x0c) /* read timing control reg */
#define IXP2000_SP_RTC2  (SLOWPORT_CSR_BASE + 0x10)
#define IXP2000_SP_PCR	 (SLOWPORT_CSR_BASE + 0x18) /* protocol control reg */
#define IXP2000_SP_ADC   (SLOWPORT_CSR_BASE + 0x1c) /* address size control reg */
#define IXP2000_SP_FAC   (SLOWPORT_CSR_BASE + 0x20) /* flash mem size address reg */
#define IXP2000_SP_FRM   (SLOWPORT_CSR_BASE + 0x24) /* flash mem read mode reg */

/* bit definitions within IXP2000_SP_CCR */
#define SP_CCR_DIVIDE_BY_1     0x00 /* divide core circuit by 1 */
#define SP_CCR_DIVIDE_BY_30    0x24 /* divide core circuit by 30 */

/* bit definitions within IXP2000_SP_FAC */
#define SP_FAC_USE_32_BIT    0x03 /* Use 32-bit address space */

/* bit definitions within IXP2000_SP_FRM */
#define SP_FRM_USE_8_BIT_DATA    0x01 
#define SP_FRM_USE_32_BIT_DATA   0x00

/* write/read timing control val definitions */
#define WR_HD     0x1 /* write hold delay */
#define WR_PW     0x2 /* write pulse width */
#define WR_SU     0x0 /* write setup time */
#define RD_HD     0x1 /* read hold delay */
#define RD_PW     0x2 /* read pulse width */
#define RD_SU     0x3 /* read setup time */
#define SP_WTC1_VAL   (WR_SU << 6 | WR_PW << 2 | WR_HD)
#define SP_RTC1_VAL   (RD_SU << 6 | RD_PW << 2 | RD_HD)

/*
 * CPLD registers
 */
#define SLOWPORT_BASE			0xC4000000
#define CPLD_CSR_BASE           (SLOWPORT_BASE + 0x3000000)
#define	CPLD_CSR_INT_STAT		(CPLD_CSR_BASE + 0x0)
#define ALPHANUM_DISPLAY   		(CPLD_CSR_BASE + 0x100)
#define	CPLD_CSR_DEBUG			(CPLD_CSR_BASE + 0x120)
#define	CPLD_CSR_PLDID			(CPLD_CSR_BASE + 0x130)
#define	CPLD_CSR_INT_MASK		(CPLD_CSR_BASE + 0x140)

#define MSF_SLOWPORT_BASE		(SLOWPORT_BASE + 0x2000000)	
#define MEDIA_SLOWPORT_BASE		MSF_SLOWPORT_BASE	
#define FABRIC_SLOWPORT_BASE	(SLOWPORT_BASE + 0x2800000)	

/*
 * TIMERS
 */

#define IXP2000_TIMER_BASE	(IXP2000_CAP_BASE + 0x20000)

/*Timer Control Register offsets*/
#define T1_CTL	 0x00 
#define T2_CTL   0x04
#define T3_CTL   0x08
#define T4_CTL   0x0C

/*Timer Load Register offsets*/
#define T1_CLD   0x10
#define T2_CLD   0x14
#define T3_CLD   0x18
#define T4_CLD   0x1C

/*Timer Status Register offsets*/
#define T1_CSR   0x20
#define T2_CSR   0x24
#define T3_CSR   0x28
#define T4_CSR   0x2C

/*Timer Interupt Clear Register offsets*/
#define T1_CLR   0x30
#define T2_CLR   0x34
#define T3_CLR   0x38
#define T4_CLR   0x3C

/*Timer Watch dog enable Register offsets*/
#define TWDE     0x40

/*Timer register addreses*/
#define IXP2000_TIMER1_CONTROL	(IXP2000_TIMER_BASE + T1_CTL)
#define IXP2000_TIMER2_CONTROL	(IXP2000_TIMER_BASE + T2_CTL)
#define IXP2000_TIMER3_CONTROL	(IXP2000_TIMER_BASE + T3_CTL)
#define IXP2000_TIMER4_CONTROL	(IXP2000_TIMER_BASE + T4_CTL)

#define IXP2000_TIMER1_LOAD	    (IXP2000_TIMER_BASE + T1_CLD)
#define IXP2000_TIMER2_LOAD	    (IXP2000_TIMER_BASE + T2_CLD)
#define IXP2000_TIMER3_LOAD	    (IXP2000_TIMER_BASE + T3_CLD)
#define IXP2000_TIMER4_LOAD	    (IXP2000_TIMER_BASE + T4_CLD)

#define IXP2000_TIMER1_STATUS	(IXP2000_TIMER_BASE + T1_CSR)
#define IXP2000_TIMER2_STATUS	(IXP2000_TIMER_BASE + T2_CSR)
#define IXP2000_TIMER3_STATUS	(IXP2000_TIMER_BASE + T3_CSR)
#define IXP2000_TIMER4_STATUS	(IXP2000_TIMER_BASE + T4_CSR)


#define IXP2000_TIMER1_COUNTER_CLR	(IXP2000_TIMER_BASE + T1_CLR)
#define IXP2000_TIMER2_COUNTER_CLR	(IXP2000_TIMER_BASE + T2_CLR)
#define IXP2000_TIMER3_COUNTER_CLR	(IXP2000_TIMER_BASE + T3_CLR)
#define IXP2000_TIMER4_COUNTER_CLR	(IXP2000_TIMER_BASE + T4_CLR)

#define IXP2000_WATCHDOGTIMER_ENABLE	(IXP2000_TIMER_BASE + TWDE)

/*timer register bits*/
#define IXP2000_TIMER_INT_CLR	    0x01
#define IXP2000_TIMER_ACTIVATE      0x80
#define IXP2000_TIMER_PSS	        0x0
#define IXP2000_TIMER_PSS_16        0x4
#define IXP2000_TIMER_PSS_256       0x8
#define IXP2000_TIMER_PSS_GPIO      0xc

/* 
* Definitions for the ixdp2000 Timer:
* There are four timers. Timer four can be used as a watchdog. We will use
* Timer-1 for the system clock, Timer-2 for the aux clock, and Timer-3 for
* the timer stamp clock. The clocks are run from the system clock, not
* divided.
*/
#define IXP2000_TIMER_SYS_TC_DISABLE (TC_DISABLE)
#define IXP2000_TIMER_SYS_TC_ENABLE  (TC_ENABLE)
#define IXP2000_TIMER_AUX_TC_DISABLE (TC_DISABLE)
#define IXP2000_TIMER_AUX_TC_ENABLE  (TC_ENABLE)

/*timer register bits*/
#define SYS_TIMER_CLK	ixp2000XtalFreq
#define AUX_TIMER_CLK	ixp2000XtalFreq

#undef RELOAD_TICKS
#ifdef RELOAD_TICKS
#define IXP2000_RELOAD_TICKS 10    /* overhead reloading timer*/
#endif

/* Minimum rate at which the system clock can run */
#define SYS_CLK_RATE_MIN                  0x1e

/* 1000 ticks, Maximum rate at which the system clock can run */
#define SYS_CLK_RATE_MAX                  0x3e8

/* Minimum rate at which the auxiliary clock can run */
#define AUX_CLK_RATE_MIN                  0x1e

/* 1000 ticks, Maximum rate at which the auxiliary clock can run */
#define AUX_CLK_RATE_MAX                  0x3e8

/* sys system Clk is timer 1 */
#define SYS_TIMER_CLEAR (IXP2000_TIMER_T1CLEAR(IXP2000_TIMER_BASE)) 
#define SYS_TIMER_CTRL  (IXP2000_TIMER_T1CTRL(IXP2000_TIMER_BASE))
#define SYS_TIMER_LOAD  (IXP2000_TIMER_T1LOAD(IXP2000_TIMER_BASE))
#define SYS_TIMER_STATUS (IXP2000_TIMER_T1VALUE(IXP2000_TIMER_BASE))

#define TICKS_PER_USEC		(SYS_TIMER_CLK/1000000)

#ifdef REV_A0_2800
#define SYS_TIMER_INT_LVL   (INT_LVL_PMU)
#else
#define SYS_TIMER_INT_LVL   (INT_LVL_T1)
#endif	 /*REV_A0_2800*/

#ifdef INCLUDE_AUX_CLK
/* aux auxiliary Clk is timer 4 */
#define AUX_TIMER_CLEAR (IXP2000_TIMER_T4CLEAR(IXP2000_TIMER_BASE)) 
#define AUX_TIMER_CTRL  (IXP2000_TIMER_T4CTRL(IXP2000_TIMER_BASE))
#define AUX_TIMER_LOAD  (IXP2000_TIMER_T4LOAD(IXP2000_TIMER_BASE))
#define AUX_TIMER_STATUS (IXP2000_TIMER_T4VALUE(IXP2000_TIMER_BASE))
  
#define AUX_TIMER_INT_LVL 	(INT_LVL_T4)
#endif /* INCLUDE_AUX_CLK */

#ifdef INCLUDE_TIMESTAMP

#define TIMESTAMP_HZ	            SYS_TIMER_CLK		 /* timestamp counter freq */
/* time stamp Clk is timer*/
#define TIMESTAMP_CLEAR SYS_TIMER_CLEAR
#define TIMESTAMP_CTRL  SYS_TIMER_CTRL
#define TIMESTAMP_LOAD  SYS_TIMER_LOAD
#define TIMESTAMP_STATUS SYS_TIMER_STATUS

#define TIMESTAMP_INT_LVL SYS_TIMER_INT_LVL
#endif /* INCLUDE_TIMESTAMP */

/*
 * Interrupt Controller Definitions
 */
#define IXP2000_INT_CTRL_BASE		(0xd6000000)
#define IXP2000_IRQ_ENABLE_SET      (IXP2000_INT_CTRL_BASE + 0x10)
#define IXP2000_IRQ_ENABLE_CLR      (IXP2000_INT_CTRL_BASE + 0x18)
#define IXP2000_IRQ_STATUS			(IXP2000_INT_CTRL_BASE + 0x08)

#define IXP2000_INT_VEC_BASE            (0x0)

/* bit definitions within IRQ_ENABLE_SET reg */
#define IXP2000_SOFT_INT		(1 << 0)
#define IXP2000_ERRSUM_INT		(1 << 1)
#define IXP2000_UART_INT 		(1 << 2)
#define IXP2000_GPIO_INT 		(1 << 3)
#define IXP2000_PMU_INT	 		(1 << 8)
#define IXP2000_SP_INT   		(1 << 9)
#define IXP2000_DMA1_INT     	(1 << 10)
#define IXP2000_DMA2_INT     	(1 << 11)
#define IXP2000_DMA3_INT     	(1 << 12)
#define IXP2000_PCI_DB_INT 		(1 << 13)
#define IXP2000_ME_ATTN_INT  	(1 << 14)
#define IXP2000_PCI_INT  		(1 << 15)
#define IXP2000_THD96127B_INT	(1 << 27)

/* bit definitions within CPLD_CSR_INT reg */
#define	CPLD_INT_NIC			(0)
#define	CPLD_INT_INGNPU			(1)
#define	CPLD_INT_PMC			(2)
#define	CPLD_INT_FABPCI			(3)
#define	CPLD_INT_FAB			(4)
#define	CPLD_INT_MED			(5)
#define	CPLD_INT_MAX			(6)

#define IXP2000_INT_NUM_LEVELS  (32)
#define IXP2000_INT_NUM_VECTORS (32)
#define IXP2000_INT_MODE        INT_NON_PREEMPT_MODEL

#define IXP2000_MASK_ALL_INT            (0xffffffff)
#define IXP2000_UNMASK_ALL_INT          (0x00000000)

#ifdef REV_A0_2800
#define INT_LVL_THD96127B		27 /**/
#endif	/* REV_A0_2800*/
#define INT_LVL_TCAM			26 /* TCAM Interrupt */
#define INT_LVL_PWR_INT			25 /* Power Interrupt */
#define INT_LVL_I2C_CLK			24 /* I2C Clock */
#define INT_LVL_I2C_DATA		23 /* I2C Datat */
#define INT_LVL_I2C_ARB			22 /* I2C Arbiter */
#define INT_LVL_MEDIA			21 /* Media Int */
#define INT_LVL_FABRIC			20 /* Fabric Int */
#define INT_LVL_GPIO_INGRESS	19 /* Int from Ingress */
#define INT_LVL_GPIO_EGRESS		18 /* Int from Egress */

#define INT_LVL_PIL     		15 /* pci_irq_l */
#define INT_LVL_UENG			14 /* Microengine IRQ */
#define INT_LVL_PCI_DB  		13 /* PCI Doorbell */

#define INT_LVL_DMA3			12 /* DMA channel 3 */
#define INT_LVL_DMA2    		11 /* DMA channel 2 */
#define INT_LVL_DMA1    		10 /* DMA channel 1 */

#define INT_LVL_PMU				8 /* PMU unit IRQ */

#define INT_LVL_T4      		7 /* Timer 4 */
#define INT_LVL_T3				6 /* Timer 3 */ 
#define INT_LVL_T2				5 /* Timer 2 */ 
#define INT_LVL_T1      		4 /* Timer 1 */

#define INT_LVL_GPIO    		3 /* GPIO unit IRQ */
#define INT_LVL_UART    		2 /* UART unit IRQ */
#define INT_LVL_ERR_SUM 		1 /* OR of all interrupts */
#define INT_LVL_SI				0 /* software interrupt */

/*
 * FLASH
 */
#define IXP2000_BOOT_FLASH_ADDR          0x00000000
#define IXP2000_BOOT_FLASH_SIZE          SZ_16M
#define IXP2000_FLASH_BASE     			 0xC4000000  

/*
 * PMU CSRs
 */
#define IXP2000_PMU_BASE	0xc0050000
#define IXP2000_PMUCONTCFG (IXP2000_PMU_BASE + 0x0f00) /* PMU Ctrl Bus Config */
#define IXP2000_PMUSTAT	   (IXP2000_PMU_BASE + 0x0e00) /* PMU Cntr Int status */
#define IXP2000_PMUMASK	   (IXP2000_PMU_BASE + 0x0d00) /* PMU Cntr Int Mask */
#define IXP2000_PMUINTEN   (IXP2000_PMU_BASE + 0x0c00) /* PMU Cntr Int enable */

#define IXP2000_PMU_CHAPCMD0 (IXP2000_PMU_BASE + 0x00) /* CHAP Cntr 0 Cmd */
#define IXP2000_PMU_CHAPCMD1 (IXP2000_PMU_BASE + 0x10) /* CHAP Cntr 1 Cmd */
#define IXP2000_PMU_CHAPCMD2 (IXP2000_PMU_BASE + 0x20) /* CHAP Cntr 2 Cmd */
#define IXP2000_PMU_CHAPCMD3 (IXP2000_PMU_BASE + 0x30) /* CHAP Cntr 3 Cmd */
#define IXP2000_PMU_CHAPCMD4 (IXP2000_PMU_BASE + 0x40) /* CHAP Cntr 4 Cmd */
#define IXP2000_PMU_CHAPCMD5 (IXP2000_PMU_BASE + 0x50) /* CHAP Cntr 5 Cmd */
#define IXP2000_PMU_CHAPEVN0 (IXP2000_PMU_BASE + 0x04) /* CHAP Cntr 0 Event */
#define IXP2000_PMU_CHAPEVN1 (IXP2000_PMU_BASE + 0x14) /* CHAP Cntr 1 Event */
#define IXP2000_PMU_CHAPEVN2 (IXP2000_PMU_BASE + 0x24) /* CHAP Cntr 2 Event */
#define IXP2000_PMU_CHAPEVN3 (IXP2000_PMU_BASE + 0x34) /* CHAP Cntr 3 Event */
#define IXP2000_PMU_CHAPEVN4 (IXP2000_PMU_BASE + 0x44) /* CHAP Cntr 4 Event */
#define IXP2000_PMU_CHAPEVN5 (IXP2000_PMU_BASE + 0x54) /* CHAP Cntr 5 Event */
#define IXP2000_PMU_CHAPSTAT0 (IXP2000_PMU_BASE + 0x08) /* CHAP Cntr 0 status */
#define IXP2000_PMU_CHAPSTAT1 (IXP2000_PMU_BASE + 0x18) /* CHAP Cntr 1 status */
#define IXP2000_PMU_CHAPSTAT2 (IXP2000_PMU_BASE + 0x28) /* CHAP Cntr 2 status */
#define IXP2000_PMU_CHAPSTAT3 (IXP2000_PMU_BASE + 0x38) /* CHAP Cntr 3 status */
#define IXP2000_PMU_CHAPSTAT4 (IXP2000_PMU_BASE + 0x48) /* CHAP Cntr 4 status */
#define IXP2000_PMU_CHAPSTAT5 (IXP2000_PMU_BASE + 0x58) /* CHAP Cntr 5 status */
#define IXP2000_PMU_CHAPDATA0 (IXP2000_PMU_BASE + 0x0C) /* CHAP Cntr 0 Data */
#define IXP2000_PMU_CHAPDATA1 (IXP2000_PMU_BASE + 0x1C) /* CHAP Cntr 1 Data */
#define IXP2000_PMU_CHAPDATA2 (IXP2000_PMU_BASE + 0x2C) /* CHAP Cntr 1 Data */
#define IXP2000_PMU_CHAPDATA3 (IXP2000_PMU_BASE + 0x3C) /* CHAP Cntr 1 Data */
#define IXP2000_PMU_CHAPDATA4 (IXP2000_PMU_BASE + 0x4C) /* CHAP Cntr 1 Data */
#define IXP2000_PMU_CHAPDATA5 (IXP2000_PMU_BASE + 0x5C) /* CHAP Cntr 1 Data */

/*
 * PMU CSR bits and bit fields
 */
#define DESIGN_BLOCK_PMU 	0x1
#define MUX_SEL_NULL 		7

#define CHAP_CC_LESS 		4
#define CHAP_CC_LEQ  		6
#define CHAP_CC_TRUE 		7
#define CHAP_OP_STOP    	0x0
#define CHAP_OP_START   	0x1
#define CHAP_OP_SAMPLE  	0x2
#define CHAP_OP_RESTART 	0x5
#define CHAP_OP_PRELOAD 	0xF
#define CHAP_ACT        	1

#define CHAP_CMD_OUIE   	26
#define CHAP_CMD_CC     	21
#define CHAP_CMD_OPCODE 	16

#define PMU_INT 			8


/* Define macros for fixing 8 and 16 bit addresses for big-endian operation */
#if (_BYTE_ORDER == _BIG_ENDIAN)
#define FIX_ADDR_16(x) ((x) ^ 2)
#define FIX_ADDR_32(x) ((x) ^ 3)
#else
#define FIX_ADDR_16(x) (x)
#define FIX_ADDR_32(x) (x)
#endif

/* 
 * PCI Registers 
 */
#define IXP2000_VENDOR_ID		0x8086
#define IXP2000_DEVICE_ID		0x9004

#define PCI_CONFIG1_BASE  0xDB000000 /* Config cycle 1 space */
#define PCI_CONFIG0_BASE  0xDA000000 /* Config cycle 0 space */
#define PCI_IO_BASE       0xD8000000 /* I/O cycle space */
#define PCI_MEM_BASE      0xE0000000 /* Mem cycle space */
#define PCI_IO_SPACE_SIZE	  SZ_16M
#define PCI_MEM_SPACE_SIZE	  SZ_512M

/* RESET_0 bitfields */
#define RESET_0_XScale                  (1 << 0)
#define RESET_0_Pci                     (1 << 1)
#define RESET_0_AssertPciRst            (1 << 2)
#define RESET_0_Sram0                   (1 << 3)
#define RESET_0_Sram1                   (1 << 4)
#define RESET_0_Sram2                   (1 << 5)
#define RESET_0_Sram3                   (1 << 6)
#define RESET_0_Media                   (1 << 7)
#define RESET_0_Dram0                   (1 << 11)
#define RESET_0_Dram1                   (1 << 12)
#define RESET_0_Dram2                   (1 << 13)
#define RESET_0_ExtRst                  (1 << 15)
#define RESET_0_RstAll                  (1 << 16)
#define RESET_0_Shac                    (1 << 17)
#define RESET_0_DbusArb                 (1 << 18)
#define RESET_0_SbusArb                 (1 << 19)
#define RESET_0_CmdArb                  (1 << 20)
#define RESET_0_InitComplete            (1 << 21)
#define RESET_0_ExtRstEnable            (1 << 22)
#define RESET_0_Res_Sec		            (1 << 23)
#define RESET_0_WatchDogRstEnable       (1 << 24)

#define RESET_RDRAM_SRAM    (RESET_0_Dram0 | RESET_0_Dram1 | RESET_0_Dram2 | \
							 RESET_0_Sram0 | RESET_0_Sram1 | RESET_0_Sram2 | \
							 RESET_0_Sram3)
#define PCI_RESET     		(RESET_0_AssertPciRst | RESET_0_Pci)

/* PCI Configuration registers */
#define PCI_CSR_BASE      			0xDE000000 /* Local config / CSR space */
#define IXP2000_PCI_VENDOR_ID       FIX_ADDR_16(PCI_CSR_BASE + 0x000)
#define IXP2000_PCI_DEVICE_ID       FIX_ADDR_16(PCI_CSR_BASE + 0x002)
#define IXP2000_PCI_CMD_STAT			(PCI_CSR_BASE + 0x004)
#define IXP2000_PCI_CACHE_LAT_HDR_BIST	(PCI_CSR_BASE + 0x00C)
#define IXP2000_PCI_CSR_BAR         	(PCI_CSR_BASE + 0x010)
#define IXP2000_PCI_SRAM_BAR        	(PCI_CSR_BASE + 0x014)
#define IXP2000_PCI_DRAM_BAR        	(PCI_CSR_BASE + 0x018)
#define IXP2000_PCI_INT_LAT         	(PCI_CSR_BASE + 0x03c)
#define IXP2000_PCI_RCOMP_OVER			(PCI_CSR_BASE + 0x060)
#define IXP2000_PCI_RCOMP_STATUS    	(PCI_CSR_BASE + 0x064)
#define IXP2000_PCI_IXP_PARAM       	(PCI_CSR_BASE + 0x078)

/* PCI Controller CSRs 
* registers that are accessed from the PCI by accesses to 
* PCI_CSR_BAR using PCI memory space. These registers are also accessible 
* from the Intel  XScale(TM) processor; the
* addresses are offsets from the base address of 0xdf000000.
*/
#define PCI_CTRL_CSR_BASE   0xDF000000 /* Base of PCI Unit CSR's */

#define IXP2000_OUT_INT_STATUS        (PCI_CTRL_CSR_BASE + 0x030)
#define IXP2000_OUT_INT_MASK          (PCI_CTRL_CSR_BASE + 0x034)
#define IXP2000_MAILBOX0              (PCI_CTRL_CSR_BASE + 0x050)
#define IXP2000_MAILBOX1              (PCI_CTRL_CSR_BASE + 0x054)
#define IXP2000_MAILBOX2              (PCI_CTRL_CSR_BASE + 0x058)
#define IXP2000_MAILBOX3              (PCI_CTRL_CSR_BASE + 0x05c)
#define IXP2000_XSCALE_DOORBELL       (PCI_CTRL_CSR_BASE + 0x060)
#define IXP2000_XSCALE_DOORBELL_SETUP (PCI_CTRL_CSR_BASE + 0x064)
#define IXP2000_PCI_DOORBELL          (PCI_CTRL_CSR_BASE + 0x070)
#define IXP2000_PCI_DOORBELL_SETUP    (PCI_CTRL_CSR_BASE + 0x074)
#define IXP2000_CHAN_1_BYTE_COUNT     (PCI_CTRL_CSR_BASE + 0x080)
#define IXP2000_CHAN_1_PCI_BAR        (PCI_CTRL_CSR_BASE + 0x084)
#define IXP2000_CHAN_1_DRAM_ADDR      (PCI_CTRL_CSR_BASE + 0x088)
#define IXP2000_CHAN_1_DESC_PTR       (PCI_CTRL_CSR_BASE + 0x08C)
#define IXP2000_CHAN_1_CONTROL        (PCI_CTRL_CSR_BASE + 0x090)
#define IXP2000_DMA_INF_MODE          (PCI_CTRL_CSR_BASE + 0x09C)

#define IXP2000_CHAN_2_BYTE_COUNT     (PCI_CTRL_CSR_BASE + 0x0A0)
#define IXP2000_CHAN_2_PCI_BAR        (PCI_CTRL_CSR_BASE + 0x0A4)
#define IXP2000_CHAN_2_DRAM_ADDR      (PCI_CTRL_CSR_BASE + 0x0A8)
#define IXP2000_CHAN_2_DESC_PTR       (PCI_CTRL_CSR_BASE + 0x0AC)
#define IXP2000_CHAN_2_CONTROL        (PCI_CTRL_CSR_BASE + 0x0B0)


#define IXP2000_CHAN_3_BYTE_COUNT     (PCI_CTRL_CSR_BASE + 0x0C0)
#define IXP2000_CHAN_3_PCI_BAR        (PCI_CTRL_CSR_BASE + 0x0C4)
#define IXP2000_CHAN_3_DRAM_ADDR      (PCI_CTRL_CSR_BASE + 0x0C8)
#define IXP2000_CHAN_3_DESC_PTR       (PCI_CTRL_CSR_BASE + 0x0CC)
#define IXP2000_CHAN_3_CONTROL        (PCI_CTRL_CSR_BASE + 0x0D0)

#define IXP2000_SRAM_BASE_ADDR_MASK   (PCI_CTRL_CSR_BASE + 0x0FC)
#define IXP2000_DRAM_BASE_ADDR_MASK   (PCI_CTRL_CSR_BASE + 0x100)
#define IXP2000_PCI_CONTROL           (PCI_CTRL_CSR_BASE + 0x13C)
#define IXP2000_PCI_ADDR_EXT          (PCI_CTRL_CSR_BASE + 0x140)
#define IXP2000_PCI_XSCALE_INT_ENABLE (PCI_CTRL_CSR_BASE + 0x15C)
#define IXP2000_PCI_XSCALE_INT_STATUS  (PCI_CTRL_CSR_BASE + 0x158)

#define XSCALE_INT_STATUS_PIL    	  (BIT(26) | BIT(27))
#define IXP2000_PCI_CONTROL_XS_INT    (1 << 24) /* Enable the PCI INTA*/

#define IXP2000_PCI_CONTROL_BE_DEO (1 << 22) /* PCI Big Endian Data Enable Out */
#define IXP2000_PCI_CONTROL_BE_DEI (1 << 21) /* PCI Big Endian Data Enable In */
#define IXP2000_PCI_CONTROL_BE_BEO (1 << 20) /* PCI Big Endian Byte Enable Out */
#define IXP2000_PCI_CONTROL_BE_BEI (1 << 19) /* PCI Big Endian Byte Enable In */

#define IXP2000_PCI_CONTROL_PNR   (1 << 17) /* PCI Not Reset bit of XScale */
#define IXP2000_PCI_CONTROL_CFG_RST_DIR (1 << 28) /* PCI Centrl Function bit */

/* 16 bit PCI registers */

#define IXP2000_PCI_COMMAND FIX_ADDR_16(PCI_CSR_BASE + 0x04)

/* Define PCI_COMMAND bits */

#define PCI_COMMAND_IOSE (1 << 0) /* I/O Space Enable */
#define PCI_COMMAND_MSE  (1 << 1) /* Mem Space Enable */
#define PCI_COMMAND_ME   (1 << 2) /* Master Enable */
#define PCI_COMMAND_MWI  (1 << 4) /* Master Write And Invalidate Enable */
#define PCI_COMMAND_PEC  (1 << 6) /* Parity Error Control */
#define PCI_COMMAND_SE   (1 << 8) /* SERR# enable */

#define PCI_CMD_STAT_VAL	( \
							PCI_COMMAND_MWI | \
							PCI_COMMAND_ME  | \
							PCI_COMMAND_MSE | \
							PCI_COMMAND_IOSE)

#define XSIM          (1 << 1)

#ifdef INCLUDE_PCI
/*
* PCI definitions
* 
* First, defines for generic pciIoMapLib.c code
*  
* Cache Line Size - in number of 32-bit words
* It might be thought that we would set this to (_CACHE_ALIGN_SIZE/4), but,
* in fact, the areas of memory mapped for use either by PCI I/O and
* PCI Configuration  and the area of (ARM) memory used by the Ethernet
* controller are marked as non-cacheable, so we should set it to zero.
*/

#define PCI_CLINE_SZ    0

/*
* Latency Timer value
* A sensible value for this depends what we are using PCI for.
* (Rather arbitrarily) leave it at 0 (the reset default value)
*/

#define PCI_LAT_TIMER           0x0


#define SWAP16(x) ((((x) << 8) | ((x) >> 8)) & 0xFFFF)
#define SWAP32(x)               \
			  (((x) << 24) |                \
			  (((x) & 0x0000FF00) << 8) |   \
			  (((x) & 0x00FF0000) >> 8) |   \
			  (((unsigned int)(x)) >> 24))

/* memory map as seen by the CPU on the local bus */
#define CPU_PCI_MEM_ADRS    PCI_MEM_BASE /* PCI memory space base */
#define CPU_PCI_MEM_SIZE    PCI_MEM_SPACE_SIZE   

#define CPU_PCI_IO_ADRS     PCI_IO_BASE  /* PCI I/O space base */
#define CPU_PCI_IO_SIZE     0x2000000    /* 32 Mbytes */

#define CPU_PCI_CNFG_ADRS   PCI_CONFIG0_BASE  /* type 0 PCI config space */
#define CPU_PCI_CNFG_SIZE   0x1000000         /* 16 Mbytes */

#define CPU_PCI_CNFG_1_ADRS PCI_CONFIG1_BASE  /* type 1 PCI config space */
#define CPU_PCI_CNFG_1_SIZE 0x1000000         /* 16 Mbytes */


/* PCI view of PCI I/O space for PCI devices */

#define PCI_IO_ADRS         0x00000000  /* base of PCI I/O address */

/* PCI bus resources */

#define PCI_IO_SIZE     0x80            /* PCI I/O size */

/* PCI device 0: slot I/O 4 */

#define PCI_MEM_ADR0            0x00            /* Memory base for CSR */
#define PCI_IO_ADR0             PCI_IO_ADRS     /* I/O base for CSR 32Bytes */
#define PCI_IO_SIZE0            PCI_IO_SIZE     /* I/O size for CSR */
#define PCI_INT_LVL0            INT_LVL_PIL     /* Interrupt level */
#define PCI_INT_VEC0            IVEC_TO_INUM(PCI_INT_LVL0) /* Interrupt vector */

/* PCI device 1: slot I/O 3 */

#define PCI_MEM_ADR1            0x00
#define PCI_IO_ADR1             PCI_IO_ADRS
#define PCI_IO_SIZE1            PCI_IO_SIZE
#define PCI_INT_LVL1            INT_LVL_PIL
#define PCI_INT_VEC1            IVEC_TO_INUM(PCI_INT_LVL1)

/* PCI device 2: slot I/O 2 */

#define PCI_MEM_ADR2            0x00
#define PCI_IO_ADR2             PCI_IO_ADRS
#define PCI_IO_SIZE2            PCI_IO_SIZE
#define PCI_INT_LVL2            INT_LVL_PIL
#define PCI_INT_VEC2            IVEC_TO_INUM(PCI_INT_LVL2)

/* PCI device 3: slot I/O 1 */

#define PCI_MEM_ADR3            0x00
#define PCI_IO_ADR3             PCI_IO_ADRS
#define PCI_IO_SIZE3            PCI_IO_SIZE
#define PCI_INT_LVL3            INT_LVL_PIL
#define PCI_INT_VEC3            IVEC_TO_INUM(PCI_INT_LVL3)

/*
* N.B. Only the following values have been tested. In particular, no
* testing has been performed with more than one PCI bus, i.e. Type 1
* configuration.
*/

#define PCI_MAX_DEV     8       /* 8 PCI slots: 0 to 7 */
#define PCI_MAX_BUS     2       /* two buses */

/* total number of pci interrupts */
#define PCI_MAX_INT	(PCI_MAX_DEV * PCI_MAX_BUS)

/* Allocate Device Space (Offsets) for PCI devices */

#define LAN_DEV_SPACE       0
#define PCI_IO_LN_ADRS      (PCI_IO_ADRS + LAN_DEV_SPACE)


/* Allocated base address of h/w devices as seen from CPU */

#define LAN_BASE_ADRS       (CPU_PCI_IO_ADRS + LAN_DEV_SPACE)


/* Create configuration address */

#define MAKE_CFG_ADDR(bus, device, func, offset) ((bus) ? \
	((UINT32)((bus & 0xFF) << 16) |  (UINT32)((device & 0x1F) << 11) | \
	 (UINT32)((func & 0x7) << 8)  |  (UINT32) (offset & 0xFC)) : \
	((UINT32) (1 << ((device & 0x1F) + 16)) | (UINT32)((func & 0x7) << 8) | \
	 (UINT32) (offset & 0xFC)))

#endif /* ifdef INCLUDE_PCI */
 
#define BOOT_THRU_BM    			(1<<31) /*no boot monitor flag*/
#define BOOT_DONT_INIT_PCI   		(1 << 30)
#define BOOT_CMP_IMAGE				(1<<29)
#define BOOTROM_THRU_BM			    (1<<28)

#define TB_VENDOR_ID				0x8086
#define TB_DEVICE_ID				0xB154

#define CONFIG_DATA_VALID 0x12345678

#ifdef CPU_SPEED_600MHZ
	#define BaudRateDivisor_115200     0x1B
	#define BaudRateDivisor_57600      0x36 
#else
	#define BaudRateDivisor_115200     0x12
	#define BaudRateDivisor_57600      0x24 
#endif

/* MMU */
#define MMU_Control_M		0x200
#define MMU_Control_R  		0x001
#define MMU_Control_C  		0x004
#define MMU_Control_BTB 	0x800

#ifdef SRAM_BOOT
#define ROM_MMU_TABLE_ADR	0x90014000
#else
#ifdef STANDALONE_BOOT
#define ROM_MMU_TABLE_ADR	0xc4004000
#else
#define ROM_MMU_TABLE_ADR	(0xc4004000+SZ_8M)
#endif /*STANDALONE_BOOT*/
#endif /*SRAM_BOOT*/

/*
 * Media Switch Fabric (MSF) CSRs
 */
#define MSF_CSR_BASE		0xc8000000
/* Used to override the drive settings of the Rx IO buffers.*/
#define MSF_Rx_RCOMP_Override	(MSF_CSR_BASE + 0xF8) 
/* Used to override the drive settings of the Tx IO buffers.*/
#define MSF_Tx_RCOMP_Override	(MSF_CSR_BASE + 0xFC) 

#if 0
#define CHAN_CONTROL_OFFSET			0x0
#define RX_DLL_OFFSET				0x228
#define RD_PTR_OFFSET				0x240
#define RX_DESKEW_OFFSET			0x244

#define QDR_RCOMP_OFFSET				0x318 
#endif

/*
 * SRAM QDR controller
 */
/* SRAM Controller */
#define	MAX_QDR_CHANNELS			4
#define IXP2000_SRAM_BASE			0x80000000
#define IXP2000_SRAM_ENQ_BASE		0xCC000100	
#define IXP2000_SRAM_CSR_BASE		0xCC000000	/* MMU map needs 4k booundary*/
#define IXP2000_SRAM_RNG_BASE		0xCE000000

#define SRAM_CHAN0_BASE				(IXP2000_SRAM_BASE + 0x00000000)
#define SRAM_CHAN1_BASE				(IXP2000_SRAM_BASE + 0x10000000)
#define SRAM_CHAN2_BASE				(IXP2000_SRAM_BASE + 0x20000000)
#define SRAM_CHAN3_BASE				(IXP2000_SRAM_BASE + 0x30000000)

#define SRAM_CH0_BIT_SET_OPS_BASE	(SRAM_CHAN0_BASE + 0x04000000)
#define SRAM_CH1_BIT_SET_OPS_BASE	(SRAM_CHAN1_BASE + 0x04000000)
#define SRAM_CH2_BIT_SET_OPS_BASE	(SRAM_CHAN2_BASE + 0x04000000)
#define SRAM_CH3_BIT_SET_OPS_BASE	(SRAM_CHAN3_BASE + 0x04000000)

#define SRAM_CH0_BIT_CLR_OPS_BASE	(SRAM_CHAN0_BASE + 0x08000000)
#define SRAM_CH1_BIT_CLR_OPS_BASE	(SRAM_CHAN1_BASE + 0x08000000)
#define SRAM_CH2_BIT_CLR_OPS_BASE	(SRAM_CHAN2_BASE + 0x08000000)
#define SRAM_CH3_BIT_CLR_OPS_BASE	(SRAM_CHAN3_BASE + 0x08000000)

#define SRAM_CH0_ADD_OPS_BASE		(SRAM_CHAN0_BASE + 0x0C000000)
#define SRAM_CH1_ADD_OPS_BASE		(SRAM_CHAN1_BASE + 0x0C000000)
#define SRAM_CH2_ADD_OPS_BASE		(SRAM_CHAN2_BASE + 0x0C000000)
#define SRAM_CH3_ADD_OPS_BASE		(SRAM_CHAN3_BASE + 0x0C000000)

#define SRAM_CH0_ENQ_BASE			IXP2000_SRAM_ENQ_BASE
#define SRAM_CH1_ENQ_BASE			(IXP2000_SRAM_ENQ_BASE+0x00400000)
#define SRAM_CH2_ENQ_BASE			(IXP2000_SRAM_ENQ_BASE+0x00800000)
#define SRAM_CH3_ENQ_BASE			(IXP2000_SRAM_ENQ_BASE+0x00C00000)

#define SRAM_CH0_ENQ_MMU_BASE		IXP2000_SRAM_ENQ_MMU_BASE
#define SRAM_CH1_ENQ_MMU_BASE		(IXP2000_SRAM_ENQ_MMU_BASE+0x00400000)
#define SRAM_CH2_ENQ_MMU_BASE		(IXP2000_SRAM_ENQ_MMU_BASE+0x00800000)
#define SRAM_CH3_ENQ_MMU_BASE		(IXP2000_SRAM_ENQ_MMU_BASE+0x00C00000)

#define SRAM_CH0_RNG_BASE			IXP2000_SRAM_RNG_BASE
#define SRAM_CH1_RNG_BASE			(IXP2000_SRAM_RNG_BASE+0x00400000)
#define SRAM_CH2_RNG_BASE			(IXP2000_SRAM_RNG_BASE+0x00800000)
#define SRAM_CH3_RNG_BASE			(IXP2000_SRAM_RNG_BASE+0x00C00000)


#define SRAM_CH0_CSR_BASE			0xcc010000
#define SRAM_CH1_CSR_BASE			(SRAM_CH0_CSR_BASE+0x00400000)
#ifdef IXP2800
#define SRAM_CH2_CSR_BASE			(SRAM_CH0_CSR_BASE+0x00800000)
#define SRAM_CH3_CSR_BASE			(SRAM_CH0_CSR_BASE+0x00C00000)
#endif

#define SRAM_CONTROL_OFFSET			0x0
#define SRAM_INT_PIPELINE_OFFSET	0x108
#define SRAM_RX_DLL_OFFSET			0x228
#define SRAM_RD_PTR_OFFSET_OFFSET	0x240
#define SRAM_RX_DESKEW_OFFSET		0x244
#define SRAM_RCMP_SETUP_CTL_OFFSET	0x300
#define SRAM_RCMP_SLEW_INDEX_CTL_OFFSET	0x318

#define SRAM_0_CONTROL	(SRAM_CH0_CSR_BASE + 0x00)
#define SRAM_0_PARITY_STATUS1	(SRAM_CH0_CSR_BASE + 0x04)
#define SRAM_0_PARITY_STATUS2	(SRAM_CH0_CSR_BASE + 0x08)
#define SRAM_0_INT_PIPELINE	(SRAM_CH0_CSR_BASE + 0x108)
#define SRAM_0_RX_DLL		(SRAM_CH0_CSR_BASE + 0x228)
#define SRAM_0_RD_PTR_OFFSET	(SRAM_CH0_CSR_BASE + 0x240)
#define SRAM_0_RX_DESKEW	(SRAM_CH0_CSR_BASE + 0x244)
#define SRAM_0_RCMP_SETUP_CTL	(SRAM_CH0_CSR_BASE + 0x300)
#define SRAM_0_RCMP_SLEW_INDEX_CTL (SRAM_CH0_CSR_BASE + 0x318)

#define SRAM_1_CONTROL	(SRAM_CH1_CSR_BASE + 0x00)
#define SRAM_1_PARITY_STATUS1	(SRAM_CH1_CSR_BASE + 0x04)
#define SRAM_1_PARITY_STATUS2	(SRAM_CH1_CSR_BASE + 0x08)
#define SRAM_1_INT_PIPELINE	(SRAM_CH1_CSR_BASE + 0x108)
#define SRAM_1_RX_DLL		(SRAM_CH1_CSR_BASE + 0x228)
#define SRAM_1_RD_PTR_OFFSET	(SRAM_CH1_CSR_BASE + 0x240)
#define SRAM_1_RX_DESKEW	(SRAM_CH1_CSR_BASE + 0x244)
#define SRAM_1_RCMP_SETUP_CTL	(SRAM_CH1_CSR_BASE + 0x300)
#define SRAM_1_RCMP_SLEW_INDEX_CTL (SRAM_CH1_CSR_BASE + 0x318)
#ifdef IXP2800
#define SRAM_2_CONTROL	(SRAM_CH2_CSR_BASE + 0x00)
#define SRAM_2_PARITY_STATUS1	(SRAM_CH2_CSR_BASE + 0x04)
#define SRAM_2_PARITY_STATUS2	(SRAM_CH2_CSR_BASE + 0x08)
#define SRAM_2_INT_PIPELINE	(SRAM_CH2_CSR_BASE + 0x108)
#define SRAM_2_RX_DLL		(SRAM_CH2_CSR_BASE + 0x228)
#define SRAM_2_RD_PTR_OFFSET	(SRAM_CH2_CSR_BASE + 0x240)
#define SRAM_2_RX_DESKEW	(SRAM_CH2_CSR_BASE + 0x244)
#define SRAM_2_RCMP_SETUP_CTL	(SRAM_CH2_CSR_BASE + 0x300)
#define SRAM_2_RCMP_SLEW_INDEX_CTL (SRAM_CH2_CSR_BASE + 0x318)

#define SRAM_3_CONTROL	(SRAM_CH3_CSR_BASE + 0x00)
#define SRAM_3_PARITY_STATUS1	(SRAM_CH3_CSR_BASE + 0x04)
#define SRAM_3_PARITY_STATUS2	(SRAM_CH3_CSR_BASE + 0x08)
#define SRAM_3_INT_PIPELINE	(SRAM_CH3_CSR_BASE + 0x108)
#define SRAM_3_RX_DLL		(SRAM_CH3_CSR_BASE + 0x228)
#define SRAM_3_RD_PTR_OFFSET	(SRAM_CH3_CSR_BASE + 0x240)
#define SRAM_3_RX_DESKEW	(SRAM_CH3_CSR_BASE + 0x244)
#define SRAM_3_RCMP_SETUP_CTL	(SRAM_CH3_CSR_BASE + 0x300)
#define SRAM_3_RCMP_SLEW_INDEX_CTL (SRAM_CH3_CSR_BASE + 0x318)
#endif

/* SRAM_X_CONTROL bits */
#define SRAM_CONTROL_ParityEnable	(1 << 3)
#define SRAM_CONTROL_PortCtl0		(1 << 4)
#define SRAM_CONTROL_PortCtl1		(1 << 5)
#define SRAM_CONTROL_SizeMask		(7 << 7)
#define SRAM_CONTROL_Size512K		(0 << 7)
#define SRAM_CONTROL_Size1M		(1 << 7)
#define SRAM_CONTROL_Size2M		(2 << 7)
#define SRAM_CONTROL_Size4M		(3 << 7)
#define SRAM_CONTROL_Size8M		(4 << 7)
#define SRAM_CONTROL_Size16M		(5 << 7)
#define SRAM_CONTROL_Size32M		(6 << 7)
#define SRAM_CONTROL_Pipecycles(x)	( ((x) & 0x7) << 10)
#define SRAM_CONTROL_QCIgnSegCnt	(1 << 13)
#define SRAM_CONTROL_QCIgnEOP		(1 << 14)
#define SRAM_CONTROL_Swizzle		(1 << 17)

/* SRAM_X_PARITY_STATUS1 bits */
#define SRAM_PARITY_STATUS1_AddrMask	0xffffff
#define SRAM_PARITY_STATUS1_WWParity0	(1 << 28)
#define SRAM_PARITY_STATUS1_WWParity1	(1 << 29)
#define SRAM_PARITY_STATUS1_WWParity2	(1 << 30)
#define SRAM_PARITY_STATUS1_WWParity3	(1 << 31)

/* SRAM_X_PARITY_STATUS2 bits */
#define SRAM_PARITY_STATUS2_Err0	(1 << 0)
#define SRAM_PARITY_STATUS2_Err1	(1 << 1)
#define SRAM_PARITY_STATUS2_Err2	(1 << 2)
#define SRAM_PARITY_STATUS2_Err3	(1 << 3)
#define SRAM_PARITY_STATUS2_SrcXscalePci	(0 << 16)
#define SRAM_PARITY_STATUS2_SrcME	(1 << 16)
#define SRAM_PARITY_STATUS2_ThdXscale	(0 << 17)
#define SRAM_PARITY_STATUS2_ThdPci	(1 << 17)
#define SRAM_PARITY_STATUS2_ThdMask	(0x7 << 17)
#define SRAM_PARITY_STATUS2_MEMask	(0x1F << 20)
#define SRAM_PARITY_STATUS2_MultiErr	(1 << 31)

/* SRAM_X_INT_PIPELINE bits */
#define SRAM_INT_PIPELINE_Minus1	(0 << 0)
#define SRAM_INT_PIPELINE_Nominal	(1 << 0)
#define SRAM_INT_PIPELINE_Add1		(2 << 0)

/* SRAM_X_RX_DLL bits */
#define SRAM_RX_DLL_SlaveSelMask	0x1f
#define SRAM_RX_DLL_SlaveSel(x)		((x) & SRAM_RX_DLL_SlaveSelMask)

/* SRAM_X_RD_PTR_OFFSET bits */
#define SRAM_RD_PTR_OFFSET_PtrOffsetMask	0x3
#define SRAM_RD_PTR_OFFSET_PtrOffset(x)		((x) & SRAM_RD_PTR_OFFSET_PtrOffsetMask)

/* SRAM_X_RX_DESKEW bits */
#define SRAM_RX_DESKEW_MasterSelMask	0x1f
#define SRAM_RX_DESKEW_MasterSel(x)	((x) & SRAM_RX_DESKEW_MasterSelMask)

/* SRAM_X_RCMP_SETUP_CTL bits */
#define SRAM_RCMP_SETUP_CTL_RoePmos	(1 << 0)
#define SRAM_RCMP_SETUP_CTL_RoeNmos	(1 << 1)
#define SRAM_RCMP_SETUP_CTL_Frcmp	(1 << 2)
#define SRAM_RCMP_SETUP_CTL_RcmpSmDisable	(1 << 3)
#define SRAM_RCMP_SETUP_CTL_BlockRcmpUpdates	(1 << 4)
#define SRAM_RCMP_SETUP_CTL_RcmpPeriod16ms	(0 << 5)
#define SRAM_RCMP_SETUP_CTL_RcmpPeriod8ms	(1 << 5)
#define SRAM_RCMP_SETUP_CTL_RcmpPeriod4ms	(2 << 5)
#define SRAM_RCMP_SETUP_CTL_RcmpPeriod2ms	(3 << 5)
#define SRAM_RCMP_SETUP_CTL_RcmpPeriod1ms	(4 << 5)
#define SRAM_RCMP_SETUP_CTL_RcmpPeriod500us	(5 << 5)
#define SRAM_RCMP_SETUP_CTL_RcmpPeriod250us	(6 << 5)
#define SRAM_RCMP_SETUP_CTL_RcmpPeriod250Clks	(7 << 5)
#define SRAM_RCMP_SETUP_CTL_FilterDisable	(0 << 8)
#define SRAM_RCMP_SETUP_CTL_FilterClamp1	(1 << 8)
#define SRAM_RCMP_SETUP_CTL_FilterClamp2	(2 << 8)
#define SRAM_RCMP_SETUP_CTL_FilterClamp4	(3 << 8)
#define SRAM_RCMP_SETUP_CTL_TablesProgrammed	(1 << 10)
#define SRAM_RCMP_SETUP_CTL_RcmpLock		(1 << 11)
#define SRAM_RCMP_SETUP_CTL_DQRSOverride	(1 << 12)
#define SRAM_RCMP_SETUP_CTL_KRSOverride		(1 << 13)
#define SRAM_RCMP_SETUP_CTL_DRSOverride		(1 << 14)
#define SRAM_RCMP_SETUP_CTL_ARSOverride		(1 << 15)
#define SRAM_RCMP_SETUP_CTL_IncDecInvert	(1 << 16)
#define SRAM_RCMP_SETUP_CTL_SlewIndexSel	(1 << 17)

/* SRAM_X_SLEW_INDEX_CTL bits */
#define SRAM_SLEW_INDEX_CTL_DQStrength(x)	(((x) & 0xf) << 0)
#define SRAM_SLEW_INDEX_CTL_DQSlewClamp		(1 << 4)
#define SRAM_SLEW_INDEX_CTL_KStrength(x)	(((x) & 0xf) << 5)
#define SRAM_SLEW_INDEX_CTL_KSlewClamp		(1 << 9)
#define SRAM_SLEW_INDEX_CTL_DStrength(x)	(((x) & 0xf) << 10)
#define SRAM_SLEW_INDEX_CTL_DSlewClamp		(1 << 14)
#define SRAM_SLEW_INDEX_CTL_AStrength(x)	(((x) & 0xf) << 15)
#define SRAM_SLEW_INDEX_CTL_ASlewClamp		(1 << 19)

#define RDRAM_DIVISOR		0x4

#define PIPELINE					(1 << 10)
#define SRAM_SIZE_MIN		SZ_1M 
#define SRAM_SIZE_MAX		SZ_64M 

/* NOTICE! when executing on the DI board, sram size is 8mb instead of 4mb */
#ifdef SRAM_BOOT
#define SRAM_CHAN_SIZE			SZ_8M
#else
#define SRAM_CHAN_SIZE			SZ_64M
#endif
#define SRAM_SIZE_MAX_CSR_VAL	(5 << 7)
#define SRAM_COMPARE_LOCATION   0x101000
#define SRAM_TEST_LOCATION	0x1000
#define TEST_PAT			0x12345678
#define COMPARISON_PAT		0xAA55AA55

#define QDR_DLL_FREQ_SEL 	0x3
#define QDR_DLL_TAP_SEL 	0x2
#define QDR_RD_PTR			0x1

#ifdef STANDALONE_BOOT
#define BOOTROM_OFFSET		0xc4040000	
#define BOOTROM_TEXT_ADRS	0xc4048000	
#else
#define BOOTROM_OFFSET		(0xc4040000+SZ_8M)	
#define BOOTROM_TEXT_ADRS	(0xc4048000+SZ_8M)	
#endif	/*STANDALONE_BOOT*/

#include "ixdp2800.h" /* IXDP2000 Board specific macro definitions */

/* Assembly specific defs */
#ifndef _ASMLANGUAGE
#endif

#ifdef __cplusplus
}
#endif

#endif  /* INCixdp2000h */
