/*---------------------------------------------------------------------
 *                                                                     
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                       
 *      COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *      THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                       
 *                         INTEL  CORPORATION                            
 *                                                                      
 *                      2200 MISSION COLLEGE BLVD                        
 *                                                                       
 *                SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                       
 *---------------------------------------------------------------------
 *
 *        File Name: dl_system_egress_default.h
 *
 *        Purpose: This file contains default system wide parameters
 *                 that can be overriden for a given application.
 *
 *        History:
 *
 *        Date            Comment                         By
 *        ---------------------------------------------------------------
 *
 *        11/11/2002      Created                         asv
 *
 *-------------------------------------------------------------------*/

#ifndef	__DL_EGRESS_SYSTEM_DEFAULT_H__
#define	__DL_EGRESS_SYSTEM_DEFAULT_H__

/*---------------------------------------------------------------------
 * Include global definition and configuration files
 *-------------------------------------------------------------------*/

/* This file is also used by script files that setup the system when run 
 * in simulation mode. When symbol IXP_SCRIPTS is defined, it indicates
 * that scripts are using this file. Since script files cannot access
 * path definitions, they should not encounter '#include <file>'. Instead
 * they include the <file> using Simulation -> Options -> Startup.
 */
#ifndef IXP_SCRIPTS
#include <definitions.h>
#include <ix_cc_microengines_bindings.h>
#ifndef IXP_MICROCODE
#include <ixp.h> /* for micro-c */
#endif
#endif

/*---------------------------------------------------------------------
 *                             C A U T I O N
 *
 * The code below may not be changed by the user. If any of the below
 * code has to be overwritten, redefine it in a separate file and then
 * include this file.
 *-------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * 1) Miscellaneous settings.
 *-------------------------------------------------------------------*/

/* Memory map base addresses. These are different for xscale and
 * micro-engines. When running this file on xscale, define IXP_XSCALE
 */
#ifndef IXP_XSCALE_RM
#ifdef IXP_XSCALE

#define	SRAM_CH0_BASE	0x80000000
#define	SRAM_CH1_BASE	0x90000000
#define	SRAM_CH2_BASE	0xA0000000
#define	SRAM_CH3_BASE	0xB0000000
#define	SCRATCH_BASE	0xCA000000

#else

#define	SRAM_CH0_BASE	0x00000000
#define	SRAM_CH1_BASE	0x40000000
#define	SRAM_CH2_BASE	0x80000000
#define	SRAM_CH3_BASE	0xC0000000
#define	SCRATCH_BASE	0x00000000

#endif	/* IXP_XSCALE */
#endif	/* IXP_XSCALE_RM */

/* PHY mode for TX device. */
#ifndef		TX_PHY_MODE
#define		TX_PHY_MODE			SPHY_1_32
#endif

#ifndef		IXP_XSCALE_RM
#ifndef		USE_IMPORT_VAR

/* Freelist id that is stored in meta data */
#ifndef		FREE_LIST_ID
#define		FREE_LIST_ID		0
#endif

#ifndef		CHILD_FREELIST_ID	
#define		CHILD_FREELIST_ID	2
#endif

#endif /* USE_IMPORT_VAR */
#endif	/* IXP_XSCALE_RM */

#ifdef		USE_IMPORT_VAR

/* Patched by core */
.import_var DL_REL_BASE
.import_var BUF_FREE_LIST0
.import_var BUF_SRAM_BASE
.import_var L2_TABLE_SRAM_BASE

#endif /* USE_IMPORT_VAR */


/*---------------------------------------------------------------------
 * 2) Microblock specific settings.
 *-------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * i) Queue manager microblock.
 *-------------------------------------------------------------------*/

/* Size of each queue descriptor entry in bytes. */
#ifndef		QD_SIZE
#define		QD_SIZE				16
#endif

/* Queue number for drop queue. (0-1023 are the normal queues used
   in forwarding pkts) */
#ifndef		QM_DROP_QUEUE
#define		QM_DROP_QUEUE		1024
#endif


/*---------------------------------------------------------------------
 * 3) SRAM memory map for channel 0.
 *-------------------------------------------------------------------*/
#ifndef		IXP_XSCALE_RM
#ifndef		USE_IMPORT_VAR

 /* Area of SRAM which is zero'ed out. */
#ifndef		SRAM_ZERO_BLOCK
#define		SRAM_ZERO_BLOCK 	0x0
#endif

#endif /* USE_IMPORT_VAR */
#endif	/* IXP_XSCALE_RM */

/*---------------------------------------------------------------------
 * i) Base address for meta data (buffer descriptors) of packet 
 *    buffers.
 *    LW7 of buffer meta-data is stored here.
 *-------------------------------------------------------------------*/
#ifndef		IXP_XSCALE_RM
#ifndef		USE_IMPORT_VAR

#ifndef		BUF_SRAM_BASE
#define		BUF_SRAM_BASE		(SRAM_CH0_BASE + 0x0)
#endif

#define		BUF_SRAM_BASE_0		BUF_SRAM_BASE

#ifndef		BUF_SRAM_BASE_2
#define		BUF_SRAM_BASE_2		(SRAM_CH0_BASE + 0x300000)
#endif


#endif /* USE_IMPORT_VAR */
#endif	/* IXP_XSCALE_RM */


/* Total SRAM size allocated in bytes for meta data.
   In simulation we use 640 buffers that use 32 bytes of meta-data
   per buffer. */
#ifndef		BUF_SRAM_SIZE
#ifdef 		IXP_SIMULATION
#define		BUF_SRAM_SIZE		(640 * 32)
#else
#define		BUF_SRAM_SIZE		(12800 * 32)
#endif
#endif

/* Size of meta-data (buffer descriptor) in bytes for each buffer.
   Should be a power of 2. */
#ifndef		META_DATA_SIZE
#define		META_DATA_SIZE		32
#endif

#define		META_DATA_SIZE_0	META_DATA_SIZE

#ifndef		META_DATA_SIZE_2
#define		META_DATA_SIZE_2	32
#endif

#ifndef		BUF_SRAM_SIZE_2
#define		BUF_SRAM_SIZE_2		(640 * META_DATA_SIZE_2)
#endif		

/* Allocating 1 MB of SRAM for meta-data. */


/*---------------------------------------------------------------------
 * ii) Base address for queue descriptors.
 *-------------------------------------------------------------------*/
#ifndef		IXP_XSCALE_RM
#ifndef		USE_IMPORT_VAR

#ifndef		QD_SRAM_BASE
#define		QD_SRAM_BASE		(SRAM_CH0_BASE + 0x100000)
#endif

/* Allocating 1 MB of SRAM for queue descriptors. */

#endif /* USE_IMPORT_VAR */
#endif	/* IXP_XSCALE_RM */

/* Total number of queue descriptors. */
#ifndef		QD_TOTAL
#define		QD_TOTAL			65536
#endif

/*---------------------------------------------------------------------
 * 4) SRAM memory map for channel 1.
 *-------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * i) Base address for CSIX RX contexts.
 *-------------------------------------------------------------------*/
#ifndef		IXP_XSCALE_RM
#ifndef		USE_IMPORT_VAR

#ifndef		CSIX_RX_CTX_SRAM_BASE
#define		CSIX_RX_CTX_SRAM_BASE	(SRAM_CH1_BASE + 0x0)
#endif

#endif /* USE_IMPORT_VAR */
#endif	/* IXP_XSCALE_RM */

/*
 * Total number of CSIX Receive Contexts.
 */
#ifndef		CSIX_RX_CTX_TOTAL
#define		CSIX_RX_CTX_TOTAL		1024
#endif

/* Size of a CSIX Receive context in bytes. */
#ifndef		CSIX_RX_CTX_SIZE
#define		CSIX_RX_CTX_SIZE		32
#endif

/*---------------------------------------------------------------------
 * j) Base address for L2 table.
 *-------------------------------------------------------------------*/
#ifndef		IXP_XSCALE_RM
#ifndef		USE_IMPORT_VAR

/*
 * L2 Table (each entry has 32 bytes, bit 31 of first long word is
 * valid bit requires 64K * 32 = 2M 
 */
#ifndef		L2_TABLE_SRAM_BASE
#define 	L2_TABLE_SRAM_BASE			(SRAM_CH1_BASE + 0x100000)
#endif

#endif /* USE_IMPORT_VAR */
#endif	/* IXP_XSCALE_RM */

/* Define number of L2 table entries */
#ifndef		L2_TABLE_NUM_ENTRIES
#define		L2_TABLE_NUM_ENTRIES		65536
#endif

/* The first 256 entries (0-255) are used by IPv4 packets */
/* Entries 256 onwards are used by IPv6 packets */
#ifndef		L2_TABLE_NUM_IPV4_ENTRIES
#define		L2_TABLE_NUM_IPV4_ENTRIES	256
#endif

/* Define size of an L2 table entry in bytes */
#ifndef		L2_TABLE_ENTRY_SIZE
#define		L2_TABLE_ENTRY_SIZE			32
#endif

/*---------------------------------------------------------------------
 * k) Base address for Multicast Output Interface Table
 *-------------------------------------------------------------------*/

#ifdef	MULTICAST_SUPPORT_ENABLE

#ifndef	USE_IMPORT_VAR

		/* no patching */

#ifndef	MOIT_PORT_MASK_TABLE_BASE
#define	MOIT_PORT_MASK_TABLE_BASE				(SRAM_CH1_BASE + 0x300000)
#endif

#ifndef	MOIT_OIF_TABLE_BASE
#define	MOIT_OIF_TABLE_BASE						(SRAM_CH1_BASE + 0x300000 + 0x40000)
#endif

#endif	/* USE_IMPORT_VAR */

#ifndef	MOIT_PORT_MASK_TABLE_ENTRY_SIZE
#define	MOIT_PORT_MASK_TABLE_ENTRY_SIZE			4		/* bytes */
#endif

#ifndef	MOIT_PORT_MASK_TABLE_NUM_ENTRIES
#define	MOIT_PORT_MASK_TABLE_NUM_ENTRIES		65536
#endif

#ifndef	MOIT_OIF_TABLE_ENTRY_SIZE
#define	MOIT_OIF_TABLE_ENTRY_SIZE				4		/* bytes */
#endif

#ifndef	MOIT_OIF_TABLE_NUM_ENTRIES
#define	MOIT_OIF_TABLE_NUM_ENTRIES				16
#endif

#endif	/* MULTICAST_SUPPORT_ENABLE */

/*---------------------------------------------------------------------
 * 5) SRAM memory map for channel 2.
 *-------------------------------------------------------------------*/
#ifndef		IXP_XSCALE_RM
#ifndef		USE_IMPORT_VAR

/* SRAM q-array entry allocated for scheduler queue data structures 
  (SRAM channel 2 -- bits[31:30])*/
#ifndef 	SCHED_QUEUE_STRUCTURES_BASE
#define 	SCHED_QUEUE_STRUCTURES_BASE		SRAM_CH2_BASE
#endif

#ifndef		SCHED_ROUND_COUNTERS_BASE
#define 	SCHED_ROUND_COUNTERS_BASE		(SCHED_QUEUE_STRUCTURES_BASE + \
 											SCHED_QUEUE_STRUCTURES_SIZE)
#endif
 
#endif		/* USE_IMPORT_VAR */
#endif		/* IXP_XSCALE_RM */

#ifndef  	SCHED_QUEUE_STRUCTURES_SIZE		
#define 	SCHED_QUEUE_STRUCTURES_SIZE		0x20000 
#endif

#ifndef 	SCHED_ROUND_COUNTERS_SIZE
#define 	SCHED_ROUND_COUNTERS_SIZE		0x40000
#endif

/* Number of bits used to define port number */
#define 	NUMBER_OF_BITS_FOR_PORTS   	 	4

/* Number of ports */
#define 	NUMBER_OF_PORTS              	(1 << NUMBER_OF_BITS_FOR_PORTS)

/* Number of bits used to specify a DRR round */
#define 	NUMBER_OF_BITS_FOR_ROUNDS	 	12

/* Maximum number of rounds per port */
#define 	NUMBER_OF_ROUNDS_PER_PORT	 	(1 << NUMBER_OF_BITS_FOR_ROUNDS)

/* Number of queues per port - the DRR algorithm can support any number */
#define 	NUMBER_OF_QUEUES_PER_PORT  		256

/*---------------------------------------------------------------------
 * 6) SRAM memory map for channel 3.
 *-------------------------------------------------------------------*/

/* LW 0-6 of buffer meta-data is stored here. */


/*---------------------------------------------------------------------
 * 7) SRAM Q-array memory map.
 *-------------------------------------------------------------------*/

/* SRAM q-array entries 0-15 are used by the queue manager to cache queues */
#ifndef		QM_QARRAY_BASE
#define		QM_QARRAY_BASE			0
#endif

#ifndef		QM_QARRAY_SIZE
#define		QM_QARRAY_SIZE			16
#endif

#ifndef		IXP_XSCALE_RM
#ifndef		USE_IMPORT_VAR

/* SRAM q-array entries allocated for buffer free list. */
#ifndef		BUF_QARRAY_BASE
#define		BUF_QARRAY_BASE			16
#endif

#ifndef		BUF_QARRAY_SIZE
#define		BUF_QARRAY_SIZE			4
#endif

#endif /* USE_IMPORT_VAR */
#endif	/* IXP_XSCALE_RM */

/* SRAM q-array entry allocated for drop queue in queue manager. */
#ifndef		QM_DROP_QUEUE_ENTRY
#define		QM_DROP_QUEUE_ENTRY		20
#endif

#ifndef		CHILD_FREELIST
#define		CHILD_FREELIST			(BUF_QARRAY_BASE + CHILD_FREELIST_ID)
#endif

#ifndef		PKT_COPIER_FREELIST
#define		PKT_COPIER_FREELIST		CHILD_FREELIST
#endif

#ifndef		BUF_FREE_LIST_2
#define		BUF_FREE_LIST_2			CHILD_FREELIST
#endif


/*---------------------------------------------------------------------
 * 8) DRAM memory map.
 *-------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * i) Base address for packet buffers. This is the actual packet data.
 *-------------------------------------------------------------------*/
#ifndef		IXP_XSCALE_RM
#ifndef		USE_IMPORT_VAR

#ifndef		BUF_SDRAM_BASE
#define		BUF_SDRAM_BASE		0
#endif

#define		BUF_SDRAM_BASE_0	BUF_SDRAM_BASE

#endif /* USE_IMPORT_VAR */
#endif	/* IXP_XSCALE_RM */

/* Total DRAM size allocated in bytes for packet buffers.
   In simulation we allocate 640 buffers of size 2048 bytes each. */
#ifndef		BUF_SDRAM_SIZE
#ifdef 		IXP_SIMULATION
#define		BUF_SDRAM_SIZE		(640 * 2048)
#else
#define		BUF_SDRAM_SIZE		(12800 * 2048)
#endif
#endif

/*	Size of one packet buffer in bytes.  */
#ifndef		BUFFER_SIZE
#define		BUFFER_SIZE			2048
#endif

#define		BUFFER_SIZE_0		BUFFER_SIZE

/* Allocating 64 MB of DRAM for packet buffers. */


/*---------------------------------------------------------------------
 * ii) Base address for second packet buffers. 
 *-------------------------------------------------------------------*/

#ifndef		BUF_SDRAM_BASE_2
#define		BUF_SDRAM_BASE_2	0x200000
#endif

/*	Size of one packet buffer in bytes.  */
#ifndef		BUFFER_SIZE_2
#define		BUFFER_SIZE_2		512
#endif

#ifndef		BUF_SDRAM_SIZE_2
#define		BUF_SDRAM_SIZE_2	(640 * 512)
#endif


/*---------------------------------------------------------------------
 * 9) Scratch and scratch ring memory map.
 *-------------------------------------------------------------------*/

	/* 
	 * Scratch ring definitions
	 */

#ifndef	SCR_RING_0_IND
#define	SCR_RING_0_IND
#define	SCR_RING_0				0
#define	SCR_RING_0_BASE			/* reserved */
#define	SCR_RING_0_SIZE			/* reserved */
#endif

#ifndef	SCR_RING_1_IND
#define	SCR_RING_1_IND
#define	SCR_RING_1				1
#define	SCR_RING_1_BASE			/* reserved */
#define	SCR_RING_1_SIZE			/* reserved */
#endif

#ifndef	SCR_RING_2_IND
#define	SCR_RING_2_IND
#define	SCR_RING_2				2
#define	SCR_RING_2_BASE			(SCRATCH_BASE + 8192)
#define	SCR_RING_2_SIZE			512
#endif


#ifndef	SCR_RING_3_IND
#define	SCR_RING_3_IND
#define	SCR_RING_3				3
#define	SCR_RING_3_BASE			/* free */
#define	SCR_RING_3_SIZE			/* free */
#endif


#ifndef	SCR_RING_4_IND
#define	SCR_RING_4_IND
#define	SCR_RING_4				4
#define	SCR_RING_4_BASE			(SCRATCH_BASE + 0)
#define	SCR_RING_4_SIZE			512
#endif


#ifndef	SCR_RING_5_IND
#define	SCR_RING_5_IND
#define	SCR_RING_5				5
#define	SCR_RING_5_BASE			(SCRATCH_BASE + 2048)
#define	SCR_RING_5_SIZE			512
#endif


#ifndef	SCR_RING_6_IND
#define	SCR_RING_6_IND
#define	SCR_RING_6				6
#define	SCR_RING_6_BASE			(SCRATCH_BASE + 4096)
#define	SCR_RING_6_SIZE			512
#endif


#ifndef	SCR_RING_7_IND
#define	SCR_RING_7_IND
#define	SCR_RING_7				7
#define	SCR_RING_7_BASE			(SCRATCH_BASE + 6144)
#define	SCR_RING_7_SIZE			256
#endif	

#ifndef	SCR_RING_8_IND
#define	SCR_RING_8_IND
#define	SCR_RING_8				8
#define	SCR_RING_8_BASE			(SCRATCH_BASE + 7168)
#define	SCR_RING_8_SIZE			256
#endif


#ifndef	SCR_RING_9_IND
#define	SCR_RING_9_IND
#define	SCR_RING_9				9
#define	SCR_RING_9_BASE			(SCRATCH_BASE + 10240)
#define	SCR_RING_9_SIZE			512
#endif


#ifndef	SCR_RING_10_IND
#define	SCR_RING_10_IND
#define	SCR_RING_10				10
#define	SCR_RING_10_BASE		(SCRATCH_BASE + 12288)
#define	SCR_RING_10_SIZE		256
#endif


#ifndef	SCR_RING_11_IND
#define	SCR_RING_11_IND
#define	SCR_RING_11				11
#define	SCR_RING_11_BASE		/* free */
#define	SCR_RING_11_SIZE		/* free */
#endif


/*  There are two exception ring to send pkts up */
#ifndef		EXCEPTION_RING_OUT_0
#define		EXCEPTION_RING_OUT_0						SCR_RING_0
#endif

#ifndef		EXCEPTION_RING_OUT_1
#define		EXCEPTION_RING_OUT_1						SCR_RING_1
#endif

/*  There are two exception ring to send pkts down */
#ifndef		EXCEPTION_RING_IN_0
#define		EXCEPTION_RING_IN_0							SCR_RING_2
#define		EXCEPTION_RING_IN_0_BASE					SCR_RING_2_BASE
#define		EXCEPTION_RING_IN_0_SIZE					SCR_RING_2_SIZE
#endif


/* Scratch ring for communicating between CSIX RX and Statistics block */
#ifndef		CSIX_RX_TO_STATS_SCR
#define		CSIX_RX_TO_STATS_SCR
#define		CSIX_RX_TO_STATS_SCR_RING					SCR_RING_4
#define		CSIX_RX_TO_STATS_SCR_RING_BASE				SCR_RING_4_BASE
#define		CSIX_RX_TO_STATS_SCR_RING_SIZE				SCR_RING_4_SIZE
#endif

/* Scratch ring for communicating between ETHERNET ARP and Statistics block */
#ifndef		ETHERNET_ARP_TO_STATS_SCR
#define		ETHERNET_ARP_TO_STATS_SCR
#define		ETHERNET_ARP_TO_STATS_SCR_RING				SCR_RING_5
#define		ETHERNET_ARP_TO_STATS_SCR_RING_BASE			SCR_RING_5_BASE
#define		ETHERNET_ARP_TO_STATS_SCR_RING_SIZE			SCR_RING_5_SIZE
#endif

/* Scratch ring for communicating within the 3-ME Scheduler block.
   The ME's are: Class Scheduler, Count, Port Scheduler		*/
#ifndef		CLASS_COUNT_PORT_SCHEDULER_SCR
#define		CLASS_COUNT_PORT_SCHEDULER_SCR
#define		CLASS_COUNT_PORT_SCHEDULER_SCR_RING			SCR_RING_6
#define		CLASS_COUNT_PORT_SCHEDULER_SCR_RING_BASE	SCR_RING_6_BASE
#define		CLASS_COUNT_PORT_SCHEDULER_SCR_RING_SIZE	SCR_RING_6_SIZE
#endif


/*
 * First scratch ring for communicating between Queue Manager and Packet TX 
 * microblocks.
 */ 
#ifndef		QM_TO_PACKET_TX_SCR_RING_0       
#define 	QM_TO_PACKET_TX_SCR_RING_0					SCR_RING_7
#define 	QM_TO_PACKET_TX_SCR_RING_0_BASE				SCR_RING_7_BASE
#define 	QM_TO_PACKET_TX_SCR_RING_0_SIZE				SCR_RING_7_SIZE
#endif
/*
 * Second scratch ring for communicating between Queue Manager and Packet TX 
 * microblocks.
 */
#ifndef 	QM_TO_PACKET_TX_SCR_RING_1		
#define 	QM_TO_PACKET_TX_SCR_RING_1					SCR_RING_8
#define 	QM_TO_PACKET_TX_SCR_RING_1_BASE				SCR_RING_8_BASE
#define 	QM_TO_PACKET_TX_SCR_RING_1_SIZE				SCR_RING_8_SIZE
#endif

/*
 * Scratch address required by Packet TX for updating available Transmit Buffers.
 */
#ifndef		PACKET_TX_AVAIL_TBUF_ELE_SCR_ADDR
#define 	PACKET_TX_AVAIL_TBUF_ELE_SCR_ADDR	14336
#endif 

/* These defines are used by the individual microcode to identify their
 * scratch rings. The reason for these defines is to improve the portability
 * of microblocks. User just has to make changes here and the microblocks will
 * remain untouched. For the naming convention, the microblock name goes first,
 * then the _RING_ specifier. Following this is the direction indicator.
 * '_OUT' would imply the microblock is writing to this scratch ring and
 * '_IN' would imply the microblock is reading from this scratch ring.
 * If there is more than one 'IN' or 'OUT' ring the name will have '_<digit>'
 * to differentiate the rings.
 * An '_IND' at the end indicates the compilation switch.
 */

#ifndef		CSIX_RX_RING_OUT_IND
#define		CSIX_RX_RING_OUT_IND
#define 	CSIX_RX_RING_OUT				SCR_RING_4
#define		CSIX_RX_RING_OUT_BASE			SCR_RING_4_BASE
#define		CSIX_RX_RING_OUT_SIZE			SCR_RING_4_SIZE
#endif		/* CSIX_RX_RING_OUT_IND */

#ifndef		STATS_RING_IN_IND
#define		STATS_RING_IN_IND
#define 	STATS_RING_IN					SCR_RING_4
#define		STATS_RING_IN_BASE				SCR_RING_4_BASE
#define		STATS_RING_IN_SIZE				SCR_RING_4_SIZE
#endif		/* STATS_RING_IN_IND */

#ifndef		ETHERNET_ARP_RING_IN_IND
#define		ETHERNET_ARP_RING_IN_IND
#define 	ETHERNET_ARP_RING_IN			SCR_RING_4
#define		ETHERNET_ARP_RING_IN_BASE		SCR_RING_4_BASE
#define		ETHERNET_ARP_RING_IN_SIZE		SCR_RING_4_SIZE
#endif		/* ETHERNET_ARP_RING_IN_IND */

#ifndef		ETHERNET_ARP_RING_OUT_IND
#define		ETHERNET_ARP_RING_OUT_IND
#define 	ETHERNET_ARP_OUT_SCR_RING		SCR_RING_5
#define 	ETHERNET_ARP_RING_OUT			SCR_RING_5
#define		ETHERNET_ARP_RING_OUT_BASE		SCR_RING_5_BASE
#define		ETHERNET_ARP_RING_OUT_SIZE		SCR_RING_5_SIZE
#endif		/* ETHERNET_ARP_RING_OUT_IND */

#ifndef		PACKET_TX_IN_0_IND
#define		PACKET_TX_IN_0_IND
#define		PACKET_TX_IN_0					SCR_RING_7
#endif		/* PACKET_TX_IN_0_IND */

#ifndef		PACKET_TX_IN_1_IND
#define		PACKET_TX_IN_1_IND
#define		PACKET_TX_IN_1					SCR_RING_8	
#endif		/* PACKET_TX_IN_1_IND */


#ifndef		CSIX_RX_RING_OUT_1_IND
#define		CSIX_RX_RING_OUT_1_IND
#define 	CSIX_RX_RING_OUT_1				SCR_RING_5
#define		CSIX_RX_RING_OUT_1_BASE			SCR_RING_5_BASE
#define		CSIX_RX_RING_OUT_1_SIZE			SCR_RING_5_SIZE
#endif		/* CSIX_RX_RING_OUT_IND */

#ifndef		PKT_COPIER_REQ_RING_IND
#define		PKT_COPIER_REQ_RING_IND
#define		PKT_COPIER_REQ_RING				SCR_RING_9
#define		PKT_COPIER_REQ_RING_BASE		SCR_RING_9_BASE
#define		PKT_COPIER_REQ_RING_SIZE		SCR_RING_9_SIZE
#endif

	/* 
	 * packet copier block ring interface 
	 */ 
#define		PKT_COPIER_RING_IN				PKT_COPIER_REQ_RING				
#define		PKT_COPIER_RING_OUT				STATS_RING_IN

	/* 
	 * packet processing dispatch loop ring interface 
	 */ 
#define		PKT_PROC_RING_IN_1				CSIX_RX_RING_OUT_1
#define		PKT_PROC_RING_OUT_1				STATS_RING_IN
#define		PKT_PROC_RING_OUT_2				PKT_COPIER_RING_IN


#ifndef IXP_SCRIPTS
#ifdef IXP_MICROCODE

/* Ring number to use. 0-15. */
#ifndef			DL_RING_NUMBER
#define_eval 	DL_RING_NUMBER			CSIX_RX_TO_STATS_SCR_RING
#endif

#ifndef			DL_DROP_RING
#define_eval	DL_DROP_RING			CSIX_RX_TO_STATS_SCR_RING
#endif

#else	/* Used for micro-c */

/* Ring number to use. 0-15. */
#ifndef			DL_RING_NUMBER
#define		 	DL_RING_NUMBER			CSIX_RX_TO_STATS_SCR_RING
#endif

#endif	/* IXP_MICROCODE */
#endif	/* IXP_SCRIPTS */

/*
 * Valid values for RING_SIZE are 128, 256, 512 or 1024.
 */
#ifndef		DL_RING_SIZE
#define 	DL_RING_SIZE				CSIX_RX_TO_STATS_SCR_RING_SIZE
#endif

#ifndef		DL_RING_BASE
#define 	DL_RING_BASE				CSIX_RX_TO_STATS_SCR_RING_BASE
#endif

/*---------------------------------------------------------------------
 * 10) System-wide signals.
 *-------------------------------------------------------------------*/

/* Signals being a precious resource, we use a single signal for 
 * inter-thread as well as any other inter-me signalling
 * (init over signal).
 */
#ifndef		COMMON_INTER_THD_ME_SIGNAL
#define		COMMON_INTER_THD_ME_SIGNAL		10
#endif

/* We are not able to use a single signal before of limitations in
 * assembler. Until then, we need a separate signal for inter-thread
 * signalling.
 */
#ifndef		INTER_THD_SIGNAL
#define		INTER_THD_SIGNAL				12
#endif

/* ME_INIT_SIGNAL signal is used to let all ME's know that system
 * initialization is over and all blocks in all MEs can start running.
 * This signal will be typically issued by the system initialization code
 * and all other MEs should wait for this signal before starting to run. 
 */
#ifndef		ME_INIT_SIGNAL
#define		ME_INIT_SIGNAL					COMMON_INTER_THD_ME_SIGNAL
#endif

/*
 * Signal number to use in inter thread signalling. 
 */
#ifndef		DL_SIG_WAKE
#define 	DL_SIG_WAKE						INTER_THD_SIGNAL
#endif

/*
 * select maintainence of thread order in dispatch loop functions
 */
#ifndef		DL_THREAD_NO_ORDER
#define		DL_THREAD_NO_ORDER	0x0
#endif

#ifndef		DL_THREAD_ORDER
#define		DL_THREAD_ORDER		0x1
#endif
/*---------------------------------------------------------------------
 * 11) System communication ID's and block ID's.
 *-------------------------------------------------------------------*/

/* All the IX_* constants should be between 0 and 255 (i.e 1 byte)
 * to enable using them as immed operands in instructions.
 *
 * IX_NULL is used to denote a NULL (or empty) packet handle. 
 * It should not be zero, as it would interfere with ring empty (0)
 * condition.
 * Note: IX_NULL is too generic a name and may clash with core
 *  components code. So it's marked for deletion. Start using UC_NULL.
 */
#ifndef		IX_NULL
#define		IX_NULL				0xFF
#endif

#ifndef		IX_EXCEPTION
#define		IX_EXCEPTION		0x1
#endif

#ifndef		IX_DROP
#define		IX_DROP				0x2
#endif

#ifndef		UC_NULL
#define		UC_NULL				0xFF		/* NULL for microcode */
#endif

#ifndef		DL_BUF_NULL
#define		DL_BUF_NULL			UC_NULL 	/* NULL buffer */
#endif

#ifndef		BID_NULL
#define		BID_NULL			UC_NULL		/* NULL block ID */
#endif

#ifndef		BID_EXCEPTION
#define		BID_EXCEPTION		0x1			/* Exception block ID */
#endif

#ifndef		BID_DROP
#define		BID_DROP			0x2			/* Drop block ID */
#endif


/* Block IDs for various blocks. Block IDs Start from 0x20.
 * (0x0 to 0x1f and 0xFF are reserved for IX_*).
 */

#ifndef		BID_MCAST_DISPATCHER
#define		BID_MCAST_DISPATCHER	0x20
#endif

#ifndef		BID_STATS
#define		BID_STATS				0x21
#endif

#ifndef		BID_PKT_COPIER
#define		BID_PKT_COPIER			0x22
#endif


/*---------------------------------------------------------------------
 * 12) Static Bindings (a.k.a targets) for various microblocks.
 *-------------------------------------------------------------------*/

#ifndef		DL_SOURCE_NEXT1
#define		DL_SOURCE_NEXT1			BID_STATS
#endif

#ifndef		DL_SOURCE_NEXT2
#define		DL_SOURCE_NEXT2			BID_MCAST_DISPATCHER
#endif

#ifndef		MCAST_DISPATCHER_NEXT1
#define		MCAST_DISPATCHER_NEXT1	BID_STATS
#endif

#ifndef		MCAST_DISPATCHER_NEXT2
#define		MCAST_DISPATCHER_NEXT2	BID_PKT_COPIER
#endif

#ifndef		DL_SINK_NEXT1
#define		DL_SINK_NEXT1			BID_STATS
#endif

#ifndef		DL_SINK_NEXT2
#define		DL_SINK_NEXT2			BID_PKT_COPIER
#endif



#endif	/* __DL_EGRESS_SYSTEM_DEFAULT_H__ */
