/*---------------------------------------------------------------------
 *                                                                     
 *                  I N T E L   P R O P R I E T A R Y                   
 *                                                                       
 *      COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
 *      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
 *      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
 *      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
 *      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
 *      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
 *      THE PRIOR WRITTEN PERMISSION OF :                                
 *                                                                       
 *                         INTEL  CORPORATION                            
 *                                                                      
 *                      2200 MISSION COLLEGE BLVD                        
 *                                                                       
 *                SANTA  CLARA,  CALIFORNIA  95052-8119                  
 *                                                                       
 *---------------------------------------------------------------------
 *
 *        File Name: dl_system_ingress_default.h
 *
 *        Purpose: This file contains default system wide parameters
 *                 that can be overriden for a given application.
 *
 *        History:
 *
 *        Date            Comment                         By
 *        ---------------------------------------------------------------
 *
 *        11/11/2002      Created                         asv
 *
 *-------------------------------------------------------------------*/

#ifndef	__DL_INGRESS_SYSTEM_DEFAULT_H__
#define	__DL_INGRESS_SYSTEM_DEFAULT_H__

/*---------------------------------------------------------------------
 * Include global definition and configuration files
 *-------------------------------------------------------------------*/

/* This file is also used by script files that setup the system when run 
 * in simulation mode. When symbol IXP_SCRIPTS is defined, it indicates
 * that scripts are using this file. Since script files cannot access
 * path definitions, they should not encounter '#include <file>'. Instead
 * they include the <file> using Simulation -> Options -> Startup.
 */
#ifndef	IXP_SCRIPTS
#include <definitions.h>
#include <ix_cc_microengines_bindings.h>
#ifndef	IXP_MICROCODE
#include <ixp.h> /* for micro-c */
#endif
#endif

/*---------------------------------------------------------------------
 *                             C A U T I O N
 *
 * The code below may not be changed by the user. If any of the below
 * code has to be overwritten, redefine it in a separate file and then
 * include this file.
 *-------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * 1) Miscellaneous settings.
 *-------------------------------------------------------------------*/

/* Memory map base addresses. These are different for xscale and
 * micro-engines. When running this file on xscale, define IXP_XSCALE.
 * Not needed when running on Xscale using RM
 */
#ifndef IXP_XSCALE_RM
#ifdef IXP_XSCALE

#define	SRAM_CH0_BASE	0x80000000
#define	SRAM_CH1_BASE	0x90000000
#define	SRAM_CH2_BASE	0xA0000000
#define	SRAM_CH3_BASE	0xB0000000
#define	SCRATCH_BASE	0xCA000000

#else

#define	SRAM_CH0_BASE	0x00000000
#define	SRAM_CH1_BASE	0x40000000
#define	SRAM_CH2_BASE	0x80000000
#define	SRAM_CH3_BASE	0xC0000000
#define	SCRATCH_BASE	0x00000000

#endif	/* IXP_XSCALE */
#endif	/* IXP_XSCALE_RM */

/* PHY mode for RX device. */
#ifndef		RX_PHY_MODE
#define 	RX_PHY_MODE 		SPHY_1_32
#endif


/* We can optimize SRAM writes in case of EOP_SOP by not flushing meta data in
 * Packet Rx microblock, as we are passing metadata onto scratch ring for 
 * the next block in the pipeline. This next block (for eg IPv4-Fwder block) 
 * will flush the meta data to SRAM at the end using sink block. */
#ifndef		OPTIMIZE_SRAM_WR
#define		OPTIMIZE_SRAM_WR 	
#endif

#ifndef 	IXP_XSCALE_RM
#ifndef		USE_IMPORT_VAR

/* Freelist id that is stored in meta data */
#ifndef		FREE_LIST_ID
#define		FREE_LIST_ID		0x0
#endif

/* This blade's id */
#ifndef		THIS_BLADE_ID
#define		THIS_BLADE_ID		0xF
#endif

#ifdef		MULTICAST_SUPPORT_ENABLE
#ifndef		MCAST_RESERVED_BLADE_ID
#define		MCAST_RESERVED_BLADE_ID	63
#endif
#endif

#endif /* USE_IMPORT_VAR */
#endif	/* IXP_XSCALE_RM */

#ifdef		USE_IMPORT_VAR

/* Patched by core */
.import_var DL_REL_BASE
.import_var BUF_FREE_LIST0
.import_var BUF_SRAM_BASE

#endif /* USE_IMPORT_VAR */


/*---------------------------------------------------------------------
 * 2) Microblock specific settings.
 *-------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * i) Packet Rx microblock.
 *-------------------------------------------------------------------*/

/* Base address for Packet Rx contexts in local memory. */
#ifndef		POS_RX_CTX_LM_BASE
#define		POS_RX_CTX_LM_BASE		0x0
#endif

/* Size of each Packet Rx context in bytes. */
#ifndef		POS_RX_CTX_SIZE
#define		POS_RX_CTX_SIZE			64
#endif

/* Total number of Packet Rx contexts. */
#ifndef		POS_RX_CTX_TOTAL
#define		POS_RX_CTX_TOTAL		16
#endif

/*---------------------------------------------------------------------
 * ii) Queue manager microblock.
 *-------------------------------------------------------------------*/

/* Size of each queue descriptor entry in bytes. */
#ifndef		QD_SIZE
#define		QD_SIZE					16
#endif

/* Queue number for drop queue. (0-1023 are the normal queues used
   in forwarding pkts) */
#ifndef		QM_DROP_QUEUE
#define		QM_DROP_QUEUE			1024
#endif

/*---------------------------------------------------------------------
 * iii) CSIX TX microblock.
 *-------------------------------------------------------------------*/

/*	Size of each CSIX TX context in bytes.  */
#ifndef		CSIX_TX_CTX_SIZE
#define		CSIX_TX_CTX_SIZE		64
#endif

/*	Total number of CSIX TX contexts. */
#ifndef		CSIX_TX_CTX_TOTAL
#define		CSIX_TX_CTX_TOTAL		1024
#endif

/*---------------------------------------------------------------------
 * iv) IPV4 Forwarder microblock.
 *-------------------------------------------------------------------*/

 /* details on where/how to cache dbcast table in local memory */
#ifdef	LM_DBCAST_TABLE

#ifndef	LM_DBCAST_BASE
#define	LM_DBCAST_BASE				256		/* in longwords */
#endif

#ifndef	LM_DBCAST_BLOCK_COUNT
#define	LM_DBCAST_BLOCK_COUNT		32
#endif

#ifndef	LM_DBCAST_BLOCK_SIZE
#define	LM_DBCAST_BLOCK_SIZE		4		/* in longwords */
#endif

#ifndef	LM_DBCAST_INDEX
#define	LM_DBCAST_INDEX				1
#endif

#endif	/* end LM_DBCAST_TABLE */


/*---------------------------------------------------------------------
 * 3) SRAM memory map for channel 0.
 *-------------------------------------------------------------------*/
#ifndef		IXP_XSCALE_RM
#ifndef		USE_IMPORT_VAR

 /* Area of SRAM which is zero'ed out. */
#ifndef		SRAM_ZERO_BLOCK
#define		SRAM_ZERO_BLOCK 	(SRAM_CH0_BASE + 0x0)
#endif

#endif /* USE_IMPORT_VAR */
#endif	/* IXP_XSCALE_RM */

/*---------------------------------------------------------------------
 * i) Base address for meta data (buffer descriptors) of packet 
 *    buffers.
 *-------------------------------------------------------------------*/
#ifndef		IXP_XSCALE_RM
#ifndef		USE_IMPORT_VAR

#ifndef		BUF_SRAM_BASE
#define		BUF_SRAM_BASE		(SRAM_CH0_BASE + 0x0)
#endif

#endif /* USE_IMPORT_VAR */
#endif	/* IXP_XSCALE_RM */

/* Total SRAM size allocated in bytes for meta data.
   In simulation we use 640 buffers that use 32 bytes of meta-data
   per buffer. */
#ifndef		BUF_SRAM_SIZE
#ifdef 		IXP_SIMULATION
#define		BUF_SRAM_SIZE		(640 * 32)
#else
#define		BUF_SRAM_SIZE		(12800 * 32)
#endif
#endif

/* Size of meta-data (buffer descriptor) in bytes for each buffer.
   Should be a power of 2. */
#ifndef		META_DATA_SIZE
#define		META_DATA_SIZE		32
#endif

/* Allocating 1 MB of SRAM for meta-data. */

/*---------------------------------------------------------------------
 * ii) Base address for queue descriptors.
 *-------------------------------------------------------------------*/
#ifndef 	IXP_XSCALE_RM
#ifndef		USE_IMPORT_VAR

#ifndef		QD_SRAM_BASE
#define		QD_SRAM_BASE		(SRAM_CH0_BASE + 0x100000)
#endif

/* Allocating 1 MB of SRAM for queue descriptors. */

#endif /* USE_IMPORT_VAR */
#endif	/* IXP_XSCALE_RM */

/* Total number of queue descriptors. */
#ifndef		QD_TOTAL
#define		QD_TOTAL			1024
#endif
/*---------------------------------------------------------------------
 * iii) Base address for CSIX TX contexts.
 *-------------------------------------------------------------------*/
#ifndef		IXP_XSCALE_RM
#ifndef		USE_IMPORT_VAR

#ifndef		CSIX_TX_CTX_SRAM_BASE
#define		CSIX_TX_CTX_SRAM_BASE	(SRAM_CH0_BASE + 0x200000)
#endif

/* Allocating 1 MB of SRAM for CSIX TX contexts. */

#endif /* USE_IMPORT_VAR */
#endif	/* IXP_XSCALE_RM */

/*---------------------------------------------------------------------
 * 4) SRAM memory map for channel 1.
 *-------------------------------------------------------------------*/

/* LW's 1-7 of buffer meta-data are stored here. Uses 1 MB. */


/*---------------------------------------------------------------------
 * 5) SRAM memory map for channel 2.
 *-------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * i) Base address for directed broadcast table.
 *-------------------------------------------------------------------*/
#ifndef		IXP_XSCALE_RM
#ifndef		USE_IMPORT_VAR

#ifndef		DBCAST_TABLE_SRAM_BASE
#define		DBCAST_TABLE_SRAM_BASE		(SRAM_CH2_BASE + 0x0)
#endif

#endif /* USE_IMPORT_VAR */
#endif	/* IXP_XSCALE_RM */

#ifndef		DBCAST_TABLE_SIZE
#define		DBCAST_TABLE_SIZE			0x2000	
#endif

#ifndef		DBCAST_NUM_BLOCKS
#define		DBCAST_NUM_BLOCKS			256
#endif

/* Size of dbcast table entry in long words */
#ifndef		DBCAST_TABLE_BLOCK_SIZE
#define		DBCAST_TABLE_BLOCK_SIZE		4
#endif

/*---------------------------------------------------------------------
 * ii) Base address for next hop information.
 *-------------------------------------------------------------------*/

/* For better DRAM bank utilization, we put the nexthop info in
 * SRAM at this address. We also have a segment in DRAM reserved
 * to keep this info, should we decide to move back to DRAM. This
 * would be a compile time option for IPV4 forwarder.
 */
#ifndef		IXP_XSCALE_RM
#ifndef		USE_IMPORT_VAR

#ifndef		NEXTHOP_TABLE_SRAM_BASE
#define		NEXTHOP_TABLE_SRAM_BASE		(SRAM_CH2_BASE + 0x2000)
#endif

#endif /* USE_IMPORT_VAR */
#endif	/* IXP_XSCALE_RM */

/*	Each next hop entry is 16 bytes. So 4KB is sufficient */
#ifndef		NEXTHOP_TABLE_SIZE
#define		NEXTHOP_TABLE_SIZE			0x1000
#endif

#ifndef		NUM_NEXTHOPS
#define		NUM_NEXTHOPS				512
#endif

#ifndef		SIZE_OF_NEXTHOP_LW
#define		SIZE_OF_NEXTHOP_LW			2
#endif


/*---------------------------------------------------------------------
 * 6) SRAM memory map for channel 3.
 *-------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * i) Base address for trie table.
 *-------------------------------------------------------------------*/
#ifndef 	IXP_XSCALE_RM
#ifndef		USE_IMPORT_VAR

#ifndef		TRIE_TABLE_SRAM_BASE
#define		TRIE_TABLE_SRAM_BASE	(SRAM_CH3_BASE + 0x02000000)
#endif

#endif /* USE_IMPORT_VAR */
#endif	/* IXP_XSCALE_RM */

/*	Trie table size. 2MB */
#ifndef		TRIE_TABLE_SIZE
#define		TRIE_TABLE_SIZE			0x100000
#endif

#ifndef		SIZE_OF_TRIE_SET
#define		SIZE_OF_TRIE_SET		16
#endif

#ifndef 	NUM_TRIE_SETS
#define		NUM_TRIE_SETS			1024
#endif

#ifdef		MULTICAST_SUPPORT_ENABLE

	/*
	 * for multicast support 
	 */
#ifndef		USE_IMPORT_VAR

#ifndef		IPV4_MFIB_TABLE_BASE
#define		IPV4_MFIB_TABLE_BASE		SRAM_CH3_BASE
#endif

#ifndef		IPV4_MFIB_TABLE_SIZE
#define		IPV4_MFIB_TABLE_SIZE		0x100000
#endif

#ifndef		IPV4_MFIB_TABLE_ENTRY_SIZE
#define		IPV4_MFIB_TABLE_ENTRY_SIZE	16		/* 16 Bytes */
#endif

#ifndef		IPV4_MFIB_TABLE_NUM_ENTRIES
#define		IPV4_MFIB_TABLE_NUM_ENTRIES	(64*1024)
#endif


#endif		/* USE_IMPORT_VAR */

#endif		/* MULTICAST_SUPPORT_ENABLE */

/*---------------------------------------------------------------------
 * 7) SRAM Q-array memory map.
 *-------------------------------------------------------------------*/

/* SRAM q-array entries 0-15 are used by the queue manager to cache queues */
#ifndef		QM_QARRAY_BASE
#define		QM_QARRAY_BASE			0
#endif

#ifndef		QM_QARRAY_SIZE
#define		QM_QARRAY_SIZE			16
#endif

#ifndef		IXP_XSCALE_RM
#ifndef		USE_IMPORT_VAR

/* SRAM q-array entries allocated for buffer free list. */
#ifndef		BUF_QARRAY_BASE
#define		BUF_QARRAY_BASE			16
#endif

#ifndef		BUF_QARRAY_SIZE
#define		BUF_QARRAY_SIZE			4
#endif

#endif /* USE_IMPORT_VAR */
#endif	/* IXP_XSCALE_RM */

/* SRAM q-array entry allocated for drop queue in queue manager. */
#ifndef		QM_DROP_QUEUE_ENTRY
#define		QM_DROP_QUEUE_ENTRY		20
#endif
/*---------------------------------------------------------------------
 * 8) DRAM memory map.
 *-------------------------------------------------------------------*/

/*---------------------------------------------------------------------
 * i) Base address for packet buffers. This is the actual packet data.
 *-------------------------------------------------------------------*/
#ifndef		IXP_XSCALE_RM
#ifndef		USE_IMPORT_VAR

#ifndef		BUF_SDRAM_BASE
#define		BUF_SDRAM_BASE		0
#endif

#endif /* USE_IMPORT_VAR */
#endif	/* IXP_XSCALE_RM */

/* Total DRAM size allocated in bytes for packet buffers.
   In simulation we allocate 640 buffers of size 2048 bytes each. */
#ifndef		BUF_SDRAM_SIZE
#ifdef 		IXP_SIMULATION
#define		BUF_SDRAM_SIZE		(640 * 2048)
#else
#define		BUF_SDRAM_SIZE		(12800 * 2048)
#endif
#endif

/*	Size of one packet buffer in bytes.  */
#ifndef		BUFFER_SIZE
#define		BUFFER_SIZE			2048
#endif

/* Allocating 64 MB of DRAM for packet buffers. */


/*---------------------------------------------------------------------
 * 9) Scratch and scratch ring memory map.
 *-------------------------------------------------------------------*/

/*	Scratch for communicating between Packet RX and IPv4 Blocks */
#ifndef		POS_RX_TO_IPV4_SCR
#define		POS_RX_TO_IPV4_SCR
#define		POS_RX_TO_IPV4_SCR_RING			4
#define		POS_RX_TO_IPV4_SCR_RING_BASE	(SCRATCH_BASE + 0)
#define		POS_RX_TO_IPV4_SCR_RING_SIZE	512
#endif

/*	Scratch for communicating between IPv4 and Statistics Block */
#ifndef		IPV4_TO_QM_SCR
#define		IPV4_TO_QM_SCR
#define		IPV4_TO_QM_SCR_RING				5
#define		IPV4_TO_QM_SCR_RING_BASE		(SCRATCH_BASE + 2048)
#define		IPV4_TO_QM_SCR_RING_SIZE		512
#define		IPV4_TO_QM_SCR_RING_FULL_CSR	inp_state_scr_ring1_full
#endif

/* There are two exception ring to send pkts down */
/* Only defined for now, will be used later */
#ifndef		EXCEPTION_RING_IN_0_SCR
#define		EXCEPTION_RING_IN_0_SCR
#define		EXCEPTION_RING_IN_0				2
#define		EXCEPTION_RING_IN_0_BASE		(SCRATCH_BASE + 4096)
#define		EXCEPTION_RING_IN_0_SIZE		512
#endif

#ifndef		EXCEPTION_RING_IN_1_SCR
#define		EXCEPTION_RING_IN_1_SCR
#define		EXCEPTION_RING_IN_1				3
#define		EXCEPTION_RING_IN_1_BASE		(SCRATCH_BASE + 6144)
#define		EXCEPTION_RING_IN_1_SIZE		512
#endif

/* There are two exception ring to send pkts up */
/* Only defined for now, will be used later */
#ifndef		EXCEPTION_RING_OUT_0_SCR
#define		EXCEPTION_RING_OUT_0			0
#endif

#ifndef		EXCEPTION_RING_OUT_1_SCR
#define		EXCEPTION_RING_OUT_1			1
#endif

#ifdef		FREELIST_MANAGER
#ifndef		DROP_RING_SCR
#define		DROP_RING_SCR
#define		DROP_RING						6
#define		DROP_RING_BASE					(SCRATCH_BASE + 8192)
#define		DROP_RING_SIZE					512
#endif
#endif

/* Base address for IPv4 counters in scratch memory. */
#ifndef		IXP_XSCALE_RM
#ifndef		USE_IMPORT_VAR

#ifdef 		IPV4_COUNTERS_SCRATCH

#ifndef		IPV4_STATS_TABLE_BASE	
#define		IPV4_STATS_TABLE_BASE			(SCRATCH_BASE + 11264)
#endif

#endif  /* IPV4_COUNTERS_SCRATCH */

#endif /* USE_IMPORT_VAR */
#endif	/* IXP_XSCALE_RM */

#ifdef 		IPV4_COUNTERS_SCRATCH
/* Space allocated to maintain these counters */
#ifndef		IPV4_STATS_TABLE_SIZE
#define		IPV4_STATS_TABLE_SIZE			2048
#endif
#endif		/* IPV4_COUNTERS_SCRATCH */

#ifdef	PROCESS_CONTROL_BLOCK
/* Define control block base. Control block is currently unused */
#define		CONTROL_BLOCK_SCRATCH_BASE		(SCRATCH_BASE + 13312)
#endif


/* These defines are used by the individual microcode to identify their
 * scratch rings. The reason for these defines is to improve the portability
 * of microblocks. User just has to make changes here and the microblocks will
 * remain untouched. For the naming convention, the microblock name goes first,
 * then the _RING_ specifier. Following this is the direction indicator.
 * '_OUT' would imply the microblock is writing to this scratch ring and
 * '_IN' would imply the microblock is reading from this scratch ring.
 * If there is more than one 'IN' or 'OUT' ring the name will have '_<digit>'
 * to differentiate the rings.
 * An '_IND' at the end indicates the compilation switch.
 */

/*	The ring on which POS RX sends out msg to next block */
#ifndef		POS_RX_RING_OUT_IND
#define		POS_RX_RING_OUT_IND
#define 	POS_RX_RING_OUT			POS_RX_TO_IPV4_SCR_RING
#define		POS_RX_RING_OUT_BASE	POS_RX_TO_IPV4_SCR_RING_BASE
#define		POS_RX_RING_OUT_SIZE	POS_RX_TO_IPV4_SCR_RING_SIZE
#endif

/*	The ring on which IPV4 receives msgs from prev block */
#ifndef		IPV4_RING_IN_IND
#define		IPV4_RING_IN_IND
#define		IPV4_RING_IN			POS_RX_TO_IPV4_SCR_RING
#define		IPV4_RING_IN_BASE		POS_RX_TO_IPV4_SCR_RING_BASE
#define		IPV4_RING_IN_SIZE		POS_RX_TO_IPV4_SCR_RING_SIZE
#endif

/* The ring on which IPV4 sends out msg to next block */
#ifndef		IPV4_RING_OUT_IND
#define		IPV4_RING_OUT_IND
#define		IPV4_RING_OUT			IPV4_TO_QM_SCR_RING
#define		IPV4_RING_OUT_BASE		IPV4_TO_QM_SCR_RING_BASE
#define		IPV4_RING_OUT_SIZE		IPV4_TO_QM_SCR_RING_SIZE
#endif

#ifndef		STATS_RING_IN_IND
#define		STATS_RING_IN_IND
#define 	STATS_RING_IN			IPV4_TO_QM_SCR_RING
#define		STATS_RING_IN_BASE		IPV4_TO_QM_SCR_RING_BASE
#define		STATS_RING_IN_SIZE		IPV4_TO_QM_SCR_RING_SIZE
#endif		/* STATS_RING_IN_IND */

/* NOTE: DL_RING_* is now obsolete. Will be changed to use 
 * POS_RX_RING_IN, which is defined above.
 */
#ifndef IXP_SCRIPTS
#ifdef	IXP_MICROCODE

/* Ring number to use. 0-15. */
#ifndef			DL_RING_NUMBER
#define_eval 	DL_RING_NUMBER		POS_RX_TO_IPV4_SCR_RING
#endif

#ifndef			DL_DROP_RING
#define_eval	DL_DROP_RING		IPV4_TO_QM_SCR_RING
#endif

#else	/* Used for micro-c */

/* Ring number to use. 0-15. */
#ifndef			DL_RING_NUMBER
#define		 	DL_RING_NUMBER		POS_RX_TO_IPV4_SCR_RING
#endif

#endif	/* IXP_MICROCODE */
#endif	/* IXP_SCRIPTS */

/* Valid values for RING_SIZE are 128, 256, 512 or 1024. */
#ifndef			DL_RING_SIZE
#define 		DL_RING_SIZE		POS_RX_TO_IPV4_SCR_RING_SIZE
#endif

/* Base Address for Scratch Ring is 0x1000. Should be aligned 
 * on a 4 x RING_SIZE byte boundary.
 */
#ifndef			DL_RING_BASE
#define 		DL_RING_BASE		POS_RX_TO_IPV4_SCR_RING_BASE
#endif


/*---------------------------------------------------------------------
 * 10) System-wide signals.
 *-------------------------------------------------------------------*/

/* Signals being a precious resource, we use a single signal for 
 * inter-thread as well as any other inter-me signalling
 * (init over signal).
 */
#ifndef		COMMON_INTER_THD_ME_SIGNAL
#define		COMMON_INTER_THD_ME_SIGNAL		10
#endif

/* We are not able to use a single signal before of limitations in
 * assembler. Until then, we need a separate signal for inter-thread
 * signalling.
 */
#ifndef		INTER_THD_SIGNAL
#define		INTER_THD_SIGNAL				12
#endif

/* ME_INIT_SIGNAL signal is used to let all ME's know that system
 * initialization is over and all blocks in all MEs can start running.
 * This signal will be typically issued by the system initialization code
 * and all other MEs should wait for this signal before starting to run. 
 */
#ifndef		ME_INIT_SIGNAL
#define		ME_INIT_SIGNAL					COMMON_INTER_THD_ME_SIGNAL
#endif

/* Signal number to use in inter thread signalling. */
#ifndef		DL_SIG_WAKE
#define 	DL_SIG_WAKE						INTER_THD_SIGNAL
#endif


/*---------------------------------------------------------------------
 * 11) System communication ID's and block ID's.
 *-------------------------------------------------------------------*/

/* These tell if the executing threads need to work in order.
 * i.e wait for	a signal from prev thread, do some work and signal
 * next thread. */
#ifndef		DL_THREAD_NO_ORDER
#define		DL_THREAD_NO_ORDER	0x0
#endif

#ifndef		DL_THREAD_ORDER
#define		DL_THREAD_ORDER		0x1
#endif

/* All the IX_* constants should be between 0 and 255 (i.e 1 byte)
 * to enable using them as immed operands in instructions.
 *
 * IX_NULL is used to denote a NULL (or empty) packet handle. 
 * It should not be zero, as it would interfere with ring empty (0)
 * condition.
 * Note: IX_NULL is too generic a name and may clash with core
 *  components code. So it's marked for deletion. Start using UC_NULL.
 */

#ifndef		IX_NULL
#define		IX_NULL				0xFF
#endif

#ifndef		IX_EXCEPTION
#define		IX_EXCEPTION		0x1
#endif

#ifndef		IX_DROP
#define		IX_DROP				0x2
#endif

#ifndef		UC_NULL
#define		UC_NULL				0xFF		/* NULL for microcode */
#endif

#ifndef		DL_BUF_NULL
#define		DL_BUF_NULL			UC_NULL 	/* NULL buffer */
#endif

#ifndef		BID_NULL
#define		BID_NULL			UC_NULL		/* NULL block ID */
#endif

#ifndef		BID_EXCEPTION
#define		BID_EXCEPTION		0x1			/* Exception block ID */
#endif

#ifndef		BID_DROP
#define		BID_DROP			0x2			/* Drop block ID */
#endif

/* Block IDs for various blocks. Block IDs Start from 0x20.
 * (0x0 to 0x1f and 0xFF are reserved for IX_*).
 */
#ifndef		BID_POS
#define		BID_POS					IX_CC_POS_RX_PKT_ID
#endif

#ifndef		BID_IPV4
#define		BID_IPV4				IX_CC_IPV4_PKT_MICROBLOCK_ID
#endif

#ifndef		BID_QM
#define		BID_QM					0x23
#endif

#ifndef		BID_ETHER
#define 	BID_ETHER				IX_CC_ETH_RX_PKT_ID
#endif

#ifndef		BID_IPV6
#define		BID_IPV6				IX_CC_IPV6_PKT_MICROBLOCK_ID
#endif

#ifndef		BID_ARP
#define		BID_ARP					IX_EXCEPTION
#endif

#ifndef		BID_ETHER_PPP_DISCOVERY
#define		BID_ETHER_PPP_DISCOVERY	IX_EXCEPTION
#endif

#ifndef		BID_ETHER_PPP_SESSION
#define		BID_ETHER_PPP_SESSION	IX_EXCEPTION
#endif

#ifndef		BID_MPLSILM
#define		BID_MPLSILM				0x29
#endif

#ifndef		BID_MPLSFTN
#define		BID_MPLSFTN				0x2A
#endif

#ifndef		BID_V6V4_TUNNEL_ENCAP
#define		BID_V6V4_TUNNEL_ENCAP	IX_CC_V6V4_ENCAP_PKT_MICROBLOCK_ID
#endif

#ifndef		BID_V6V4_TUNNEL_DECAP
#define		BID_V6V4_TUNNEL_DECAP	IX_CC_V6V4_DECAP_PKT_MICROBLOCK_ID
#endif

#ifdef		MULTICAST_SUPPORT_ENABLE

#define		BID_IPV4_MCAST_FWDER	0x30

#endif		/* MULTICAST_SUPPORT_ENABLE */
	
/*
 * Define Default Targets for PPP microblock. Currently PPP can handle
 * IPv4 and IPv6. Exception packets by default will be sent to Xscale.
 * The targets can be overridden by the user.
 */

#ifndef		TARGET_PPP_IPV4
#define		TARGET_PPP_IPV4			BID_IPV4
#endif

#ifndef		TARGET_PPP_IPV6
#define		TARGET_PPP_IPV6			BID_IPV6
#endif

#ifndef		TARGET_PPP_EXCEPTION
#define		TARGET_PPP_EXCEPTION	IX_EXCEPTION
#endif

/*
 * Define Default Targets for Ethernet microblock.
 */

#ifndef		TARGET_ETHER_DECAP_IPV4
#define     TARGET_ETHER_DECAP_IPV4		BID_IPV4
#endif

#ifndef		TARGET_ETHER_DECAP_IPV6
#define     TARGET_ETHER_DECAP_IPV6		BID_IPV6
#endif

#ifndef		TARGET_ETHER_DECAP_ARP
#define     TARGET_ETHER_DECAP_ARP		BID_ARP
#endif

#ifndef		TARGET_ETHER_DECAP_MPLS
#define     TARGET_ETHER_DECAP_MPLS		BID_ILM_CLASSIFIER
#endif

#ifndef		TARGET_ETHER_DECAP_PPP_DISCOVERY
#define     TARGET_ETHER_DECAP_PPP_DISCOVERY	BID_ETHER_PPP_DISCOVERY
#endif

#ifndef		TARGET_ETHER_DECAP_PPP_SESSION
#define     TARGET_ETHER_DECAP_PPP_SESSION		BID_ETHER_PPP_SESSION
#endif




/*---------------------------------------------------------------------
 * 12) Static Bindings (a.k.a targets) for various microblocks.
 *-------------------------------------------------------------------*/

#ifdef		ETHER_WITHOUT_L2
#define		DL_SOURCE_NEXT1			BID_ETHER
#else
#ifdef		PPP_RECEIVE
#define		DL_SOURCE_NEXT1			BID_POS
#else
#define		DL_SOURCE_NEXT1			BID_IPV4
#endif
#endif

#ifndef		IPV4_NEXT1
#define		IPV4_NEXT1				BID_QM
#endif

#ifndef		IPV4_LOCAL_NEXT1
#define		IPV4_LOCAL_NEXT1		IX_EXCEPTION
#endif

#ifdef		MULTICAST_SUPPORT_ENABLE
#ifndef		BID_IPV4_MCAST_NEXT1	
#define		BID_IPV4_MCAST_NEXT1	BID_IPV4_MCAST_FWDER
#endif

#ifndef		BID_IPV4_MCAST_FWDER_NEXT1
#define		BID_IPV4_MCAST_FWDER_NEXT1	IPV4_NEXT1
#endif

#ifndef		BID_IPV4_MCAST_EXCP_NEXT1
#define		BID_IPV4_MCAST_EXCP_NEXT1	IX_EXCEPTION
#endif	

#endif

/*	To be redefined as an import variable. And its value should 
 *	be one of the above block IDs.
 */

#ifndef		BID_NEXT_BLOCK
#define		BID_NEXT_BLOCK			BID_QM					
#endif

#endif		/* __DL_INGRESS_SYSTEM_DEFAULT_H__ */
