#ifndef	__LLC_ENCAP__H__
#define	__LLC_ENCAP__H__
/*****************************************************************************
 *                            Intel Proprietary
 *
 * Copyright (c) 1998-2002 By Intel Corporation.  All rights reserved.  
 * No part of this program or publication may be reproduced, transmitted,
 * transcribed, stored in a retrieval system, or translated into any language
 * or computer language in any form or by any means, electronic, mechanical,
 * magnetic, optical, chemical, manual, or otherwise, without the prior
 * written permission of:
 *                        Intel Corporation
 *                        2200 Mission College Blvd.
 *                        Santa Clara, CA  95052-8119
 *****************************************************************************/

/**
 *****************************************************************************
 * @ingroup LlcEncap
 * File:  llc_encap.h
 * @description
 *    This is the header file for LLC/SNAP encapsulation. 
 *    It also contains the function declarations. 
 *
 *    Change History:
 *      02/Mar/2003    Created    Shreekanth Patel    Creation
 *****************************************************************************/

#define LLC_SNAP_SIZE	8		
#define LLC_SNAP_TBL_ENTRY_SZ_INLW 	(sizeof (ix_llc_table_entry)/sizeof (unsigned int))
#define NO_ADDN_CELL	0
#define ADDN_CELL		1
#define PKT_PROTO_MPLS	0x8	  /*  MPLS	*/

/*
 * LLC-SNAP Layer-2 Table entry in SRAM
 */
typedef union 
{
	struct {
		unsigned int reserved1:8,		/**< reserved bits */
					 outVpiVci:24;	    /**< LW0: Outgoing VPI-VCI entry in table */

		unsigned int reserved2:16,		/**< reserved bits */
					 vcqNum:16;		    /**< LW1: Output VC queue number */

		unsigned int llcSnapHdr0;		/**< LW2: Bytes 0-3 of LLC/SNAP header */
		unsigned int llcSnapHdr1;		/**< LW3: Bytes 4-7 of LLC/SNAP header */
		unsigned int reserved3[4];
	} llc_hdr;

	unsigned int value [8];
} llc_table_entry_t;

typedef enum 
{
	LLC_ENCAP_SUCCESS,
	LLC_ENCAP_EXCEPTION_INVALID_HDR_VALUE,
} ix_llc_encap_exceptions;


unsigned int 
LlcEncap (
			__declspec(local_mem, aligned(32)) unsigned int *
		  );

extern void 
LlcEncap_AddLlcToPayload (
								__declspec(local_mem, aligned(32)) unsigned int *,
								__declspec(sram_read_reg) llc_table_entry_t*
							  );

extern void
LlcEncap_ReadNextHopEntry (
								__declspec (sram_read_reg) llc_table_entry_t*
							   );

extern void 
LlcEncap_UpdateMetaData (void);	

extern unsigned int
LlcEncap_ComputePacketCellCount(
									 unsigned int
									 );
#endif	/* __LLC_ENCAP__H__ */