///////////////////////////////////////////////////////////////////////////////
//
//
//                  I N T E L   P R O P R I E T A R Y
//
//     COPYRIGHT [c]  2002 BY  INTEL  CORPORATION.  ALL RIGHTS
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT
//     THE PRIOR WRITTEN PERMISSION OF :
//
//                        INTEL  CORPORATION
//
//                     2200 MISSION COLLEGE BLVD
//
//               SANTA  CLARA,  CALIFORNIA  95052-8119
//
///////////////////////////////////////////////////////////////////////////////
//
//
//      Filename: dl_buf.uc
//
//      Purpose: Packet buffer utility macros.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef	__DL_BUF_UC__
#define	__DL_BUF_UC__

#include <buf.uc>
#include <dl_system.h>

// If this flag is defined, the application requires multiple buffer freelists.
// This file has buffer utility macros that support multiple buffer freelists.
#ifdef MULTIPLE_FREELIST_ENABLE

#include <dl_buf_multi.uc>

#endif


//	Number of Packet Buffers

#define_eval	NO_OF_PKT_BUFFERS	BUF_SRAM_SIZE/META_DATA_SIZE

// 	Some Sanity Check. Check if the specified SDRAM memory is of suffcient size 
//	to hold all buffers

#define_eval	DRAM_COMPUTE		BUF_SDRAM_SIZE/BUFFER_SIZE

#if	(DRAM_COMPUTE != NO_OF_PKT_BUFFERS)
#error	"The amount of memory allocated in SRAM for packet buffers doesn't match with\
the SDRAM size. Check BUF_SRAM_SIZE and BUF_SDRAM_SIZE. Also check META_DATA_SIZE and BUFFER_SIZE"
#endif

#undef			DRAM_COMPUTE

#define			AVLBL_PKT_BUFFERS	NO_OF_PKT_BUFFERS

// 	DL_DS_RATIO = Sizeof (SDRAM entry) / Sizeof (SRAM Entry)

#define_eval 	DL_DS_RATIO			BUFFER_SIZE/META_DATA_SIZE

#ifndef  USE_IMPORT_VAR
//	SRAM-QArray ID for enqueing/dequeing pkt buffers. (i.e maintinaing the free list)
// 	Right now there is only one free list. But there may be multiple in the near future.

#define			BUF_FREE_LIST0		BUF_QARRAY_BASE


//	The combination of BUF_SDRAM_BASE, BUFFER_SIZE, BUF_SRAM_BASE, META_DATA_SIZE
//	specify a particular buffer pool. 

//	Specifying a base of address of 0 for SRAM (and DRAM) simplifies the arithmatic required
//	to translate the handle to offset. (it saves one instruction. See dl_buf_get_desc)
//	However, dl_buf_alloc will return 0 for a null buffer (i.e no buffer available) and it may 
//	not be possible to differentiate between a null buffer and a buffer at address 0.
//	So we simply make that one buffer at address 0 unavailable for allocation/freeing, yet
//	taking advantage of using a base address of 0. We end up wasting a little memory (1 buffer)
//	but that's ok.

#if	(BUF_SRAM_BASE == 0 || BUF_SRAM_BASE == 0x40000000 || BUF_SRAM_BASE == 0x80000000 || BUF_SRAM_BASE == 0xC0000000)
#define_eval	BUF_SRAM_BASE		BUF_SRAM_BASE  + META_DATA_SIZE
#define_eval	BUF_SDRAM_BASE		BUF_SDRAM_BASE + BUFFER_SIZE
#define_eval	AVLBL_PKT_BUFFERS	NO_OF_PKT_BUFFERS - 1
#endif

#define			BUF_POOL			BUF_SDRAM_BASE, BUFFER_SIZE, BUF_SRAM_BASE, META_DATA_SIZE	


// DL_REL_BASE = DRAM_BASE - (BUF_SRAM_BASE * DL_DS_RATIO)

// avoid the channel number in the calculation

#define_eval    TEMP_BUF_BASE       (BUF_SRAM_BASE & 0x3fffffff)
#define_eval	DL_REL_BASE			BUF_SDRAM_BASE - (TEMP_BUF_BASE * DL_DS_RATIO)
#undef          TEMP_BUF_BASE    
  
#else /* not simulation */

 #define		BUF_POOL			buf_sram_base_reg, BUFFER_SIZE, buf_sdram_base_reg, META_DATA_SIZE	


#endif /* USE_IMPORT_VAR */

//	ln(DL_DS_RATIO). i.e 2^MULT_FACTOR = DL_DS_RATIO. This value is required to
// 	reduce multiplication in to simple shift operation. The following set of lines
// 	computes MULT_FACTOR (at compile time)

#define MY_COUNT			0
#define	MY_RATIO	 	DL_DS_RATIO
#while (MY_RATIO)
	#define_eval		IS_SET		(MY_RATIO & 0x1)
	#if	(IS_SET == 0)
		#define_eval	MY_COUNT	(MY_COUNT+1)
		#define_eval	MY_RATIO	(MY_RATIO>>1)
	#else
		#define_eval	MULT_FACTOR	MY_COUNT
		#define_eval	MY_RATIO	0
	#endif
#endloop

#undef 	IS_SET
#undef	MY_COUNT
#undef	MY_RATIO


// 	Data = DL_REL_BASE + (HANDLE & 0xFFFFFF) * DL_DS_RATIO
// 	This is how the above imported variables are used to find where data is in dram,
// 	given the buffer handle

#if (DL_DROP_RING > 11)
#error			"For Ring Number Greater than 11, we cannot use the SCR_Ring#_Full input state. Check DL_DROP_RING"
#else
#define_eval	DL_DROP_RING_FULL	SCR_Ring/**/DL_DROP_RING/**/_status
#endif

//	XXX - These are old names. Provided here for backward compatibility only.
//	In the next release these will be removed.

#define	dl_buffer_init			dl_buf_init
#define	dl_buffer_alloc			dl_buf_alloc
#define	dl_buffer_free			dl_buf_free
#define	dl_drop_buffer			dl_buf_drop
#define	dl_drop_buffer_chain	dl_buf_drop_chain


///////////////////////////////////////////////////////////////////////////////
// dl_buffer_init:
//	 	Description: 
//			Initializes the buffering mechanism. This macro should be called once 
//			before any of the other dl_buffer macros can be called.
//			Typically this macro will create all the freelist of buffers.
//
//	 	Outputs:
//
//		Inputs:
//		
//		Size:   				:
//
///////////////////////////////////////////////////////////////////////////////

#macro dl_buf_init[]

#ifdef USE_IMPORT_VAR
	.begin
	.reg buf_sram_base_reg
	.set buf_sram_base_reg		; XXX needs to be removed.

	immed32[buf_sram_base_reg, BUF_SRAM_BASE]
    buf_freelist_create[AVLBL_PKT_BUFFERS, BUF_FREE_LIST0, BUF_POOL]
		
    .end

#else

	buf_freelist_create[AVLBL_PKT_BUFFERS, BUF_FREE_LIST0, BUF_POOL]

#endif

#endm


///////////////////////////////////////////////////////////////////////////////
// dl_buffer_alloc:
//	 	Description: 
//			Allocates a free packet buffer. If no buffer is available will 
//			return 0 in buffer handle. There may be multiple pool of buffers
//			in which case the caller specifies the pool from which this buffer
//			needs to be allocated.
//
//	 	Outputs:
//			buf_handle	:		Handle of the newly allocated buffer. The SOP 
//								and count fields will be set to 0. EOP Field 
//								WILL be set to 1. buf_handle MUST be a SRAM 
//								Transfer Register.
//		Inputs:
//			free_list	:		Specifies the pool from which buffers are to be 
//								allocated.
//			req_sig		:		Signal to use in the I/O operation.
//			sig_action	:		What to do with I/O operation - 
//								- wait for signal(s) as specified by sig_action
//								- do not wait for signal, just return (SIG_NONE)
//		Size:   				:
///////////////////////////////////////////////////////////////////////////////

#macro dl_buf_alloc[buf_handle, free_list, req_sig, sig_action]

#ifdef USE_IMPORT_VAR
	.begin
    .reg sram_qa_addr
    .set sram_qa_addr				; XXX - to be removed.

    
    // initialize the register

    immed[sram_qa_addr, 0]

    // get the channel number
	
    immed_w1[sram_qa_addr, ((BUF_SRAM_BASE>>16) & 0xc000)]

    // add the free list id to which to add the  buffer

	alu_shf[sram_qa_addr, sram_qa_addr, OR, free_list, <<24]

	// get the buffer handle from the free list

	sram[dequeue, buf_handle, sram_qa_addr, 0], sig_done[req_sig]

    // if required swap out and wait for dequeue to complete

	#if (!streq('sig_action', 'SIG_NONE'))
			ctx_arb[sig_action]
	#endif

	.end

#else
	buf_alloc[buf_handle, free_list, BUF_POOL, req_sig, sig_action, --]
#endif


#endm


///////////////////////////////////////////////////////////////////////////////
// dl_buffer_free:
//	 	Description: 
//			Frees the buffer that was previously allocated using dl_buffer_alloc.
//			Only one buffer can be freed at a time by this macro. Chained buffers
//			(for big packets) are not supported by this macro.
//			EOP, SOP and cell count in the buffer handle will all be reset to zero 
//			by this macro. So the user need not bother resetting them to zero before
//			freeing.
//
//	 	Outputs:
//
//		Inputs:
//			buf_handle	:		The Buffer handle.
//			free_list	:		Specifies the pool to which this buffer is to be 
//								released.
//
//		Size:   		:
/////////////////////////////////////////////////////////////////////////////////

#macro dl_buf_free[buf_handle, free_list]

#ifdef USE_IMPORT_VAR
	.begin
    .reg __sram_addr, chan_num
	.set __sram_addr, chan_num
		
	// mask out the cell count and sop and eop bits from 
	// the buffer before enqueuing.

	alu_shf[__sram_addr, buf_handle, AND~, 0xFF, <<24]		; Clear out the upper bits.

	// get the channel number in bits 14 and 15

    immed_w0[chan_num, ((BUF_SRAM_BASE>>16)&0xc000)]
	
	// add the channel number to the sram offset.
	alu[__sram_addr, __sram_addr, OR, chan_num, <<16]	

	// add the free list id to which to add the  buffer

	alu_shf[__sram_addr, __sram_addr, OR, free_list, <<24]

	// finally free the buffer 

	sram[enqueue, --, __sram_addr, 0]
	
	.end

#else

	buf_free[buf_handle, free_list, BUF_POOL]

#endif


#endm

///////////////////////////////////////////////////////////////////////////////
// dl_buf_get_desc:
//	 	Description: 
//			Given an opaq handle, this macro returns the sram address where the 
//			meta data (called the Buffer descriptor) refered to by the buffer 
//			handle is present. The lower 24 bits of the handle indicates the 
//			sram address.
//			Further arithmatic on this 24 bit content may or may not be required 
//			depending onimplementation. (In this case no additional arithmatic 
//			is required. The address returned is an absolute address and not an 
//			offset from a given base address).
//
//	 	Outputs:
//			sram_offset	:		SRAM Byte Address where the meta data for this 
//								buffer is present. 
//		Inputs:
//			buf_handle	:		The Buffer handle.
//		
//		Size:   				:
//			2 or 3 instructions
///////////////////////////////////////////////////////////////////////////////

#macro dl_buf_get_desc[sram_offset, buf_handle]
.begin

	// SRAM offset is in the lower 24 bits. The offset is in Long Words.

#if ( isnum(buf_handle))
	.local _temp_handle
	
	immed32[_temp_handle, buf_handle]
	alu_shf[sram_offset, _temp_handle, AND~, 0xFF, <<24]	; Clear out the upper bits.
	
	.endlocal
#else

	alu_shf[sram_offset, buf_handle, AND~, 0xFF, <<24]		; Clear out the upper bits.

#endif

	//	Buffer handle * 4 will give sram offset within a channel. 
	//	Adding the channel nummber [31:30] will give absolute SRAM address.

	alu_shf[sram_offset, --, b, sram_offset, <<2]			; Convert long word address to byte address

	//	Find the SRAM channel number.

#ifdef	USE_IMPORT_VAR

	.reg	sram_ch

	//	The next two instructions are required only if the buffers are not in Channel 0.
	//	Because we patch symbols (BUF_SRAM_BASE) here, we cannot know this at compile
	//	time. So define BUF_SRAM_CHANNEL_NOT_ZERO if this is the case. This will save
	//	two instructions for channel 0 case.

#ifdef	BUF_SRAM_CHANNEL_NOT_ZERO

	//	BUF_SRAM_BASE is a patched symbol. So an extra instruction (immed_w0) 
	//	is required.

	immed[sram_ch, (BUF_SRAM_BASE>>30)]					; extract channel number [31:30].
	alu[sram_offset, sram_offset, OR, sram_ch, <<30]	; set the channel number on the offset

#endif	//	BUF_SRAM_CHANNEL_NOT_ZERO

#else	//	USE_IMPORT_VAR

	//	BUF_SRAM_BASE is #defined (not a patched symbol).
	//	All #defines are treated as signed integers. So if MSB is set
	//	>>30 will produce -1. Hence & 0x3.

#define_eval  CHANNEL_NUM  ((BUF_SRAM_BASE >> 30) & 0x3)

#if (CHANNEL_NUM != 0)

	alu[sram_offset, sram_offset, OR, CHANNEL_NUM, <<30]	; set the channel number on the offset

#endif

#undef	CHANNEL_NUM

#endif	//	USE_IMPORT_VAR

.end
#endm

///////////////////////////////////////////////////////////////////////////////
// dl_buf_get_data:
//	 	Description: 
//			Given an opaq handle, this macro returns the sdram address where the 
//			packet data for the buffer refered to by this handle  is present. 
//			The lower 24 bits of the buffer handle contains the sram offset for 
//			meta data. From this we calculate the dram offset as follows.
//
//			meta 	= buf_handle & 0xffffff;
//			index 	= (meta - sram base) / sizeof (meta data)
//			data 	= sdram base + index * size of packet buffer (in dram)
//
//			However the above calculation is optimised as follows (grouping all 
//			constants together)
// 			
//			DL_DS_RATIO = Sizeof (Packet buffer) / Sizeof (Meta data)
// 			DL_REL_BASE = BUF_SDRAM_BASE - (BUF_SRAM_BASE * DL_DS_RATIO)
// 			Data = DL_REL_BASE + (HANDLE & 0xFFFFFF) * DL_DS_RATIO
//			
//			DL_DS_RATIO and DL_REL_BASE	are constants which can be defined at 
//			compile/load time so that Data = ...can be coded in two instructions.
//
//	 	Outputs:
//			sdram_offset:		SDRAM Address where the packet buffer for this
//								buffer is present.
//		Inputs:
//			buf_handle	:		The Buffer handle.
//		
//		Size:   				
//			2 to 4 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro dl_buf_get_data[sdram_offset, buf_handle]
.begin

	.reg 	sd

	//	First half of this function is similar to dl_meta_get_desc.

	// SRAM offset is in the lower 24 bits. The offset is in Long Words.

#if ( isnum(buf_handle))
	.local _temp_handle
	
	immed32[_temp_handle, buf_handle]
	alu_shf[sdram_offset, _temp_handle, AND~, 0xFF, <<24]	; Clear out the upper bits.
	
	.endlocal
#else

	alu_shf[sdram_offset, buf_handle, AND~, 0xFF, <<24]		; Clear out the upper bits.

#endif

//	These two lines can be combined into a single instruction. 
//	That's why we unrolled dl_meta_get_desc here.

//	alu_shf[sdram_offset, --, b, sdram_offset, <<2]				; line from dl_buf_get_desc
//	alu_shf[sdram_offset, --, b, sdram_offset, <<MULT_FACTOR]	; line from this macro

#define_eval	MF_PLUS_2	(MULT_FACTOR + 2)
	alu_shf[sdram_offset, --, b, sdram_offset, <<MF_PLUS_2]		; two lines combined.


	// We add DL_REL_BASE only if it is > 0. Saves an instruction.

#ifndef USE_IMPORT_VAR

	//	DL_REL_BASE is #defined. This allows us to save an instruction.

	#define_eval    TEMP_BUF_BASE       (BUF_SRAM_BASE & 0x3fffffff)
	#define_eval	DL_REL_BASE			BUF_SDRAM_BASE - (TEMP_BUF_BASE * DL_DS_RATIO)

#if (DL_REL_BASE > 0)

	move[sd, DL_REL_BASE]
	alu[sdram_offset, sd, +, sdram_offset]

#endif

	#undef TEMP_BUF_BASE
	#undef DL_REL_BASE

#else /* USE IMPORT VAR */

	//	DL_REL_BASE is a patched symbol. So we cannot apply the usual preprocessing.

	#ifdef USE_BUF_FREE_LIST2

 	immed32[sd, DL_REL_BASE2]

	#else

	immed32[sd, DL_REL_BASE]

	#endif 

	alu[sdram_offset, sd, +, sdram_offset]

#endif /* USE_IMPORT_VAR */

#undef	MF_PLUS_2								// clear name space
.end
#endm

///////////////////////////////////////////////////////////////////////////////
// dl_buf_get_data_from_meta:
//	 	Description: 
//			This macro is the same as the dl_buf_get_data. Instead of taking a buffer 
//			handle as an input parameter, it takes the sram offset where the buffer meta
//			data is present (Calling dl_buf_get_desc[buf_handle] will give you that 
//			sram offset). It saves a couple of instruction.
//
//			See dl_buf_get_data for full details.
//
//	 	Outputs:
//			sdram_offset	:	SDRAM Address where the packet buffer for this  
//								buffer is present.
//		Inputs:
//			sram_offset		:	The SRAM offset where the meta data for this 
//								buffer is present.
//		
//		Size:   				
//			1 to 2 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro	dl_buf_get_data_from_meta[sdram_offset, sram_offset]
.begin

	.reg	sd

	//	XXX - We should assume that the caller of this macro will have the 
	//	channel bits cleared to 0!. Revisit here again.

	#if IS_IXPTYPE(__IXP28XX)
	alu_shf[sram_offset, sram_offset, AND~, 0xFF, <<24]		; Clear out the upper bits.
	#endif

	alu_shf[sdram_offset, --, b, sram_offset, <<MULT_FACTOR]

	// We add DL_REL_BASE only if it is > 0. Saves an instruction.

#ifndef USE_IMPORT_VAR

	//	DL_REL_BASE is #defined. This allows us to save an instruction.

	#define_eval    TEMP_BUF_BASE       (BUF_SRAM_BASE & 0x3fffffff)
	#define_eval	DL_REL_BASE			BUF_SDRAM_BASE - (TEMP_BUF_BASE * DL_DS_RATIO)

#if (DL_REL_BASE > 0)

	move[sd, DL_REL_BASE]
	alu[sdram_offset, sd, +, sdram_offset]

#endif

	#undef TEMP_BUF_BASE
	#undef DL_REL_BASE

#else	//	USE_IMPORT_VAR

	//	DL_REL_BASE is a patched symbol. So we cannot apply the usual preprocessing.

	#ifdef USE_BUF_FREE_LIST2

 	immed32[sd, DL_REL_BASE2]

	#else

	immed32[sd, DL_REL_BASE]

	#endif 

	alu[sdram_offset, sd, +, sdram_offset]

#endif	//	USE_IMPORT_VAR

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_drop_buffer:
//	 	Description: 
//			Drop the buffer. This is the same as freeing a buffer.
//
//		NOTE:	
//			Right now, we assume only one free list. When multiple free 
//			lists are used, the free list id may have to be passed in.
//
//	 	Outputs:
//
//		Inputs:
//			buf_handle	: Buffer handle. Buffer that needs to be dropped
//
//		Size:   				:
//	
///////////////////////////////////////////////////////////////////////////////

#macro dl_buf_drop[buf_handle]
.begin

#ifdef FREELIST_MANAGER

	.sig	sig_scr
	.reg	$scr
	.reg	ring

#if (DROP_RING > 11)
#error			"For Ring Number Greater than 11, we cannot use the SCR_Ring#_Full input state. Check DL_DROP_RING"
#else
#define_eval	DROP_RING_FULL	SCR_Ring/**/DROP_RING/**/_status
#endif

check_ring_full#:
 	br_inp_state[DROP_RING_FULL, check_ring_full#]

	// Assume that any inter thread signaling is done after this 

	alu[$scr, --, B, buf_handle]
	alu_shf[ring, --, b, DROP_RING, <<2]
	scratch[put, $scr, 0, ring, 1], ctx_swap[sig_scr]

#else

 	dl_buf_free[buf_handle, BUF_FREE_LIST0]

#endif

.end
#endm

#macro dl_buf_drop[buf_handle, scr, sig_mask, sig_scr]
.begin

#ifdef FREELIST_MANAGER

	.reg	ring

#if (DROP_RING > 11)
#error			"For Ring Number Greater than 11, we cannot use the SCR_Ring#_Full input state. Check DL_DROP_RING"
#else
#define_eval	DROP_RING_FULL	SCR_Ring/**/DROP_RING/**/_status
#endif

check_ring_full#:
 	br_inp_state[DROP_RING_FULL, check_ring_full#]

	alu[scr[0], --, B, buf_handle]
	alu[ring, --, b, DROP_RING, <<2]
	scratch[put, scr[0], 0, ring, 1], sig_done[sig_scr]
	alu[sig_mask, sig_mask, OR, 1, <<&sig_scr]

#else

 	dl_buf_free[buf_handle, BUF_FREE_LIST0]

#endif

.end
#endm

///////////////////////////////////////////////////////////////////////////////
// dl_buf_drop_chain:
// 
// Description: 
//   Drop a chain of buffers. Here we do a ctx_swap[]. So care must
//   be taken when calling this macro to ensure threads do not go out of order
//   after executing the ctx_swap.
//
// Outputs:
//
// Inputs:
//   buf_handle     : The (sop) buffer handle to drop.
//   buf_eop_handle : The eop buffer handle to drop. All buffers in the chain
//                    including these two will be dropped.
//
// Size:
//
///////////////////////////////////////////////////////////////////////////////
#macro dl_buf_drop_chain(buf_handle, buf_eop_handle)
.begin

	#if (DL_DROP_RING > 11)
	#error "For ring number greater than 11, we cannot use the SCR_Ring#_Full \
			input state. Check DL_DROP_RING"
	#else
	#define_eval	DL_DROP_RING_FULL	SCR_Ring/**/DL_DROP_RING/**/_status
	#endif

	.reg	queue_number
	.reg 	ring
	.sig	sig_scr

check_ring_full#:

 	br_inp_state[DL_DROP_RING_FULL, check_ring_full#]

	; At this point ring has enough space. Write the data.

	#ifdef POTS
	xbuf_alloc[$scr, 4, write]
	#else
	xbuf_alloc[$scr, 3, write]
	#endif

	#if IS_IXPTYPE(__IXP28XX)
	
	// For the IXP2800 reference designs, the scratch ring format is different.
	// We add a switch to handle this.

	move($scr[0], QM_DROP_QUEUE)			; LW0 on the ring is drop queue number.
	move($scr[1], buf_handle)				; LW1 on the ring is current buffer handle
	move($scr[2], buf_eop_handle)			; LW2 on the ring is the last buffer in the chain,
											; for big pkts only. Otherwise IX_NULL.

	#else

	move($scr[0], buf_handle)				; LW0 on the ring is current buffer handle
	move($scr[1], buf_eop_handle)			; LW1 on the ring is the last buffer in the chain, 
											; for big pkts only. Otherwise IX_NULL.

	move(queue_number, QM_DROP_QUEUE)		; LW2 on the ring is drop queue number.

	alu[$scr[2], queue_number, OR, 1, <<31]	; The msb is always set to 1, to prevent producing
											; a value of 0 on to the ring.

	#endif	// IS_IXPTYPE(__IXP28XX)

	alu[ring, --, b, DL_DROP_RING, <<2]

	#ifdef POTS
	move($scr[3], seq_num)
	scratch[put, $scr[0], 0, ring, 4], ctx_swap[sig_scr]	; Produce 4 long words on the ring
	#else
	scratch[put, $scr[0], 0, ring, 3], ctx_swap[sig_scr]	; Produce 3 long words on the ring
	#endif

	xbuf_free[$scr]

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_buf_drop_chain:
// 
// Description: 
//   Drop a chain of buffers. This is almost identical to dl_buf_drop_chain().
//   The only is that instead of doing a ctx_swap[] when writing to the 
//   scratch ring, it simply adds the signal to a signal mask. This makes 
//   using the macro safer to use since we do not ctx_swap[].
//
// Outputs:
//
// Inputs:
//   buf_handle     : The (sop) buffer handle to drop.
//   buf_eop_handle : The eop buffer handle to drop. All buffers in the chain
//                    including these two will be dropped.
//   scr            : SRAM transfer registers used for scratch[put]
//   sig_mask       : Signal mask to add the scratch[put] signal.
//   sig_scr        : Signal used for scratch[put].
//
// Size:
//
///////////////////////////////////////////////////////////////////////////////
#macro dl_buf_drop_chain(buf_handle, buf_eop_handle, scr, sig_mask, sig_scr)
.begin

	#if (DL_DROP_RING > 11)
	#error "For ring number greater than 11, we cannot use the SCR_Ring#_Full \
			input state. Check DL_DROP_RING"
	#else
	#define_eval	DL_DROP_RING_FULL	SCR_Ring/**/DL_DROP_RING/**/_status
	#endif

	.reg	queue_number
	.reg 	ring

check_ring_full#:

 	br_inp_state[DL_DROP_RING_FULL, check_ring_full#]

	; At this point ring has enough space. Write the data.

	#if IS_IXPTYPE(__IXP28XX)

	// For the IXP2800 reference designs, the scratch ring format is different.
	// We add a switch to handle this.

	move(scr[0], QM_DROP_QUEUE)				; LW0 on the ring is drop queue number.
	move(scr[1], buf_handle)				; LW1 on the ring is current buffer handle
	move(scr[2], buf_eop_handle)			; LW2 on the ring is the last buffer in the chain,
											; for big pkts only. Otherwise IX_NULL.

	#else

	move(scr[0], buf_handle)				; LW0 on the ring is current buffer handle
	move(scr[1], buf_eop_handle)			; LW1 on the ring is the last buffer in the chain, 
											; for big pkts only. Otherwise IX_NULL.

	move(queue_number, QM_DROP_QUEUE)		; LW2 on the ring is drop queue number.

	alu[scr[2], queue_number, OR, 1, <<31]	; The msb is always set to 1, to prevent producing
											; a value of 0 on to the ring.

	#endif	// IS_IXPTYPE(__IXP28XX)

	alu[ring, --, b, DL_DROP_RING, <<2]

	#ifdef POTS
	move(scr[3], seq_num)
	scratch[put, scr[0], 0, ring, 4], sig_done[sig_scr]	; Produce 4 long words on the ring
	#else
	scratch[put, scr[0], 0, ring, 3], sig_done[sig_scr]	; Produce 3 long words on the ring
	#endif

	alu[sig_mask, sig_mask, OR, 1, <<&sig_scr]

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_set_SOP:
//	 	Description: 
//			Sets the SOP bit in the dl_buf_handle. It indicates that this 
//			buffer has the SOP (Start of Packet)
//
//		Outputs:
//
//	 	Inputs:
//
//		Size:   				
//			1 instruction
///////////////////////////////////////////////////////////////////////////////
#macro dl_buf_set_SOP[buf_handle]
.begin

	alu_shf[buf_handle, buf_handle, OR, 1, <<30]		; bit 30 is SOP bit

.end
#endm

///////////////////////////////////////////////////////////////////////////////
// dl_set_EOP:
//	 	Description: 
//			Sets the EOP bit in the dl_buf_handle. It indicates that this 
//			buffer has the EOP (End of Packet)
//
//		Outputs:
//
//	 	Inputs:
//
//		Size:   				
//			1 instruction
///////////////////////////////////////////////////////////////////////////////

#macro dl_buf_set_EOP[buf_handle]
.begin

	alu_shf[buf_handle, buf_handle, OR, 1, <<31]		; bit 30 is EOP bit

.end
#endm

///////////////////////////////////////////////////////////////////////////////
// dl_set_cell_count:
//	 	Description: 
//			Sets the cellcount in the dl_buf_handle. It indicates how many 
//			cells are there in this buffer.
//
//		Note:
//			We can assume that bits [29:24] in dl_buf_handle will already 
//			be zero [dl_buf_alloc guarantees that]. So we can save one 
//			instruction by avoiding to clear these bits to zero before setting 
//			them to cell count.
//
//		Outputs:
//
//	 	Inputs:
//
//		Size:   				
//			2 instruction
///////////////////////////////////////////////////////////////////////////////
#macro dl_buf_set_cell_count[buf_handle, cell_count, BITS_CLEAR]
.begin


#if (!streq('bits_clear', 'DONT_CLEAR'))
	bits_clr[buf_handle, 24, 0x3F]								; clear bits [29:24]
#endif
								
	alu_shf[buf_handle, buf_handle, OR, cell_count, <<24]		; and then set it in dl_buf_handle

.end
#endm


#endif	//	__DL_BUF_UC__