//------------------------------------------------------------------------------------
//                                                                      
//                   I N T E L   P R O P R I E T A R Y                   
//                                                                       
//      COPYRIGHT (c)  2001 BY  INTEL  CORPORATION.  ALL RIGHTS          
//      RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//      BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//      RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//      LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//      MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//      THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                       
//                         INTEL  CORPORATION                            
//                                                                      
//                      2200 MISSION COLLEGE BLVD                        
//                                                                       
//                SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                       
//------------------------------------------------------------------------------------

#ifndef __DL_MSG_IF_DEFAULTS_UC__
#define __DL_MSG_IF_DEFAULTS_UC__

#include <stdmac.uc>
#include <sig_macros.uc>
#include <dispatch_loop.h>
#include <dl_misc.uc>


/////////////////////////////
//	Some local defines
/////////////////////////////

#if (DL_RING_NUMBER > 11)
#error			"For Ring Number Greater than 11, we cannot use the SCR_Ring#_status input state. Check RING_NUMBER"
#else
#define_eval	DL_RING_FULL			SCR_Ring/**/DL_RING_NUMBER/**/_status	
#endif


/////////////////////////////
// Global variables/Registers
/////////////////////////////


.reg 	global	dl_ring					// scratch ring number for pos rx ==>ipv4. This can be made an absolut register
.reg 	global	dl_buf_handle			// The current Buffer handle.
.reg	global	dl_eop_buf_handle		// For large packets, this is the last buffer in the chain
.reg	global	dl_next_block			// Next block that should process the buffer/packet
.reg    global  dl_exception_reg 		// Exception register used to send pkts to CORE
.set            dl_exception_reg


//#define DEBUG 1


///////////////////////////////////////////////////////////////////////////////
// _dl_pos_sink_init:
//	 	Description: 
//			Initialisation for POS dl_sink. Should be called before using dl_sink.
//			It simply sets a global variable dl_ring to the corresponding ring number
//			This will later save one instruction in dl_sink (for POS).
//
//			[Note: This is an internal macro not meant to be used by the user.
//			This macro will undergo some more changes.]
//
//	 	Outputs:
//			
//		Inputs:
//		
//		Size:   				:
//			1 instruction
//

#macro _dl_pos_sink_init[]
.begin

	alu_shf[dl_ring, --, B, DL_RING_NUMBER, <<2]	; ring number to be used in get/put 

.end
#endm

///////////////////////////////////////////////////////////////////////////////
// dl_rx_from_pkt_rx_init_default:
//	 	Description: 
//
//			Initialisation for dl_source. Should be called before using dl_source.
//			Currently not used/required. Place holder.
//
//	 	Outputs:
//
//		Inputs:
//		
//		Size:   				:
//	
//
#macro dl_rx_from_pkt_rx_init_default[]

//	Global Variable for a Prev/Next thread signaling. 

.sig volatile 	sig_prev				
.addr			sig_prev				DL_SIG_WAKE

#endm


///////////////////////////////////////////////////////////////////////////////
// _dl_eop_buf_handle_init:
//	 	Description: 
//			Initialize the EOP handle to IX_NULL. This is not an exported macro.
//
//	 	Outputs:
//
//		Inputs:
//		
//		Size:   				:
//	
//
#macro _dl_eop_buf_handle_init[]
.begin

	immed32[dl_eop_buf_handle, IX_NULL]

.end
#endm

///////////////////////////////////////////////////////////////////////////////
// dl_pkt_proc_to_qm_sink_init_default:
//	 	Description: 
//
//			Initialisation for dl_sink. Should be called before using dl_sink.
//			Currently not used/required. Place holder.
//
//	 	Outputs:
//
//		Inputs:
//		
//		Size:   				:
//	
//
#macro dl_pkt_proc_to_qm_sink_init_default[]


#endm

///////////////////////////////////////////////////////////////////////////////
// dl_rx_from_pkt_rx_default:
//	 	Description: 
//
//			This macro is used by IPv4 Block to get packets  and other info
//			from the POS RX block. It receives them thro a scratch ring (POS_RX_IPV4_RING)
//			The following info is available from the ring.
//			dl_buf_handle		// The current buffer containing SOP 
//			dl_eop_buf_handle	// for large packets, the buffer containing EOP. Otherwise NULL
//			meta1				// Long Word 1 from the Meta data 
//			meta2				// Long Word 2 from the Meta Data
//			input port			// port from which the packet is rxed.
//			
//
//			THREAD_ORDER means if the execution of the threads need to be in
//			order. i.e wait for a signal from prev. thread, do some work and 
//			signal next thread. (This thread ordering is required, if the 
//			threads access some shared data, in order to get exclusive access 
//			to that shared data).
//
//	 	Outputs:
//
//		Inputs:
//			THREAD_ORDER	:	Specifies if the execution of the threads need to be in
//								order. Possible (constant) values are 
//								DL_THREAD_ORDER
//								DL_THREAD_NO_ORDER.
//			
//			sig_mask		:	Signal mask to wait on. This is the bitmask tobe used in a
//								ctx_arb[--] instrcution. (The bitmask contains a set of signals
//								used in previous I/O on which we want to wait here)
//		Size:   			:
//			N/A	
//		
// 		Notes:
//		dl_source is meant to be used along with dl_qm_sink in the following way (only).
//
//		dl_source[DL_THREAD_ORDER, 0]
//		.while (1)
//			do_what_ever[]
//			dl_qm_sink[...]
//			dl_source[DL_THREAD_NO_ORDER, ...]
//		.endw
//
//		dl_qm_sink is always ordered, meaning, it will always wait for a signal from a previous
//		thread, do its job and then signal the next thread. Because dl_qm_sink is always ordered,
//		we don't need the dl_source just following it to be ordered. (but the first time we call
//		dl_source it needs to be ordered).
//
//		Performance optimisation: 
//		-------------------------
//		original code:
//		dl_qm_sink signals the next thread after producing on to the scratch ring, which was
//		at the end of that macro. Then dl_source is called (no thread order). At the very top
//		we had a scratch[get...] ctx_swap[..]. This ctx_swap happened too soon (because the signal
//		we sent to the next thread takes about 10 cycles to reach) so the next thread is still
//		not ready to execute. This lead to large "Idle" time in the pipeline affecting throughput.
//		
//		The following changes were made to improve performance.
//		1. In dl_qm_sink we signal the next thread at the first available oppurtunity, even
//		before the scratch[put...]. This is ok because the next thread (in the same ME) cannot
//		run until we sleep. So make sure there is no ctx_swap between this signal and scratch_put.
//
//		2. If the next thread is another ME, then 1 is a wrong solution. So if we have to
//		signal another ME (because we are in thread 7), we postpone the signalling until 
//		later (in dl_source macro). Thus, for thread 7, dl_source will always signal the 
//		next ME.
//
//		3. soon after we make a scratch[put...] in dl_qm_sink, we make scratch[get...] in 
//		dl_source. So we wait for the combined signals (in dl_source macro) rather wait
//		for wait for each of them separately. (This is the reason for having 3 paramters
//		to dl_qm_sink, and the second paramter to dl_source).
//
//		For ease of understanding, look at dl_qm_sink and dl_source as one BIG combined 
//		macro in the while loop.

#ifdef POTS
#define	RX_TO_FUNC_MSG_SIZE		6		// 6 longwords
#else
#define	RX_TO_FUNC_MSG_SIZE		5		// 5 longwords
#endif

#macro dl_rx_from_pkt_rx_default[THREAD_ORDER, sig_mask]

.begin
.sig scr_get									; signal for scratch get
.reg ring
.sig	sig_ip

	xbuf_alloc[$rdata, RX_TO_FUNC_MSG_SIZE, read]		; allocate xfer register. No Code generated.

#if	(THREAD_ORDER == DL_THREAD_ORDER)

	// Wait for signal from previous thread to ensure thread ordering

	ctx_arb[sig_prev]

#endif
	
	//	Get Data from Scratch Ring.
	//	Instead of Hardcoding the ring (POS_RX_TO_IPV4_SCR_RING) we should 
	//	use import variable here. ???

	alu_shf[ring, --, B, POS_RX_TO_IPV4_SCR_RING, <<2]		; Ring from which to consume
	scratch[get, $rdata[0], 0, ring, RX_TO_FUNC_MSG_SIZE], sig_done[scr_get]	; Get 5 Long words from ring 

	// 	Signal Next Thread. See Notes as why this code is like the way itis shown here.
#ifdef POTS

	#if	(THREAD_ORDER == DL_THREAD_ORDER)
		br=ctx[7, thread_7#]
		signal_next_ctx[DL_SIG_WAKE]							; !7, so signal the next thread
			
		br[ordering_done#]		

		thread_7#:

		#ifndef IPV4_END_ME
			signal_me[DL_NEXT_ME, DL_SIG_WAKE]						; Thread 7, So Signal Next ME	
		#endif

		ordering_done#:

	#endif

#else 

	br=ctx[7, thread_7#]									; Special case for thread 7.

#if	(THREAD_ORDER == DL_THREAD_ORDER)
	signal_next_ctx[DL_SIG_WAKE]							; !7, so signal the next thread
#endif

	br[wait_for_signals#]

thread_7#:

// If signalling to the first ME, then signal context 1. Context 0 is not
// part of the functional pipeline
;#if (DL_NEXT_ME == 0x1)
#if (defined(START_ME) && (DL_NEXT_ME == START_ME))


	signal_me_ctx1[DL_NEXT_ME, DL_SIG_WAKE]	

#else
	signal_me[DL_NEXT_ME, DL_SIG_WAKE]						; Thread 7, So Signal Next ME

#endif
	//	We wait for the above scratch[get...] and the scratch[put...] in dl_qm_sink.
	//	See Notes above.
#endif		// #ifdef POTS

wait_for_signals#:
.reg tmp

	//	Add the signal to the signal mask on which to wait

	alu_shf[sig_mask, sig_mask, OR, 1, <<(&scr_get)]
	local_csr_wr[active_ctx_wakeup_events, sig_mask]		; tells which signals to wait on.
	ctx_arb[--]

	//	Assembler doesn't know that we actually wait on the I/O (scratch[get..])
.io_completed scr_get

	#ifdef POTS
		#define_eval REG_NO (RX_TO_FUNC_MSG_SIZE - 1)
		alu[seq_num, --, b, $rdata/**/REG_NO]
		#undef REG_NO
	#endif

check_ring_empty#:

	//	Check if Ring is empty. ==> it would have returned zero

	alu[--, $rdata[0], -, 0]									; Check for 0. i.e check if ring empty
	beq[ring_empty#]								

	//	$rdata0...$rdata4 will have valid data now.
	//	Load Meta Data Cache with $rdata2, and $rdata3 << 16
	// 	$rdata2 (buffer size,offset) => LW1 of meta data
	//	$rdata3(pkt size, rx stat, free_list etc) ==> LW2 in meta data. 
	//	LSB16 bits of $rdata4 is inport => LW3, MSB16 of meta data


	dl_meta_init_cache[0, $rdata[2], $rdata[3], $rdata[4], 0, 0, 0, 0]
      dl_meta_set_nexthop_id[0xffff]	// set nexthop ID to -1
	  immed[dl_exception_reg,0x0]		// clear exception register


	// 	We branch here so that we can take advantage of the defer slots here.
	//	This branch with defer, will finally save a branch to the end of this macro 
	//	(where 2 cycles may be wasted)

	br[load_ip_cache#], defer[2]
	move[dl_buf_handle, $rdata[0]]				; LW0 from ring is the current buffer handle (containign SOP)
	move[dl_eop_buf_handle, $rdata[1]]			; LW1 from ring is last buffer handle (for large packets)

	// 	program flow continues in load_ip_cache#:

	//	Handle Ring empty Condition.
ring_empty#:

	//	If ring is empty, just set dl_buf_handle and dl_next_block to IX_NULL;
	// 	Is this the right thing to do ? TBD ???

	move[dl_buf_handle, IX_NULL]				; Note IX_NULL is NOT ZERO.
	move[dl_next_block, IX_NULL]

	br[nothing_to_do#]

load_ip_cache#:

	// Read the IP header of the packet and cache it in some place

	dl_load_iphdr_cache[dl_buf_handle, sig_ip, sig_ip]
	move[dl_next_block, DL_SOURCE_NEXT1]

nothing_to_do#:

	xbuf_free[$rdata]							; Free xfer registers. No Code generated.


	#define_eval i 0
	#while (i < PKT_HDR_CACHE_SIZE)
		.set $$iphdr[i]
		#define_eval i (i + 1)
	#endloop

.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_pkt_rx_sink_default:
//	 	Description: 
//			Called by POS RX to handover the packet to the next block (IPv4) in the
//			ingress pipeline. We do not do thread ordering in this macro. (It is done
//			by POS RX itself because of tight budget)
//			
//			This macro will produce the following on to the scratch ring.
//			dl_buf_handle		// The current buffer containing SOP 
//			dl_eop_buf_handle	// for large packets, the buffer containing EOP. Otherwise NULL
//			meta1				// Long Word 1 from the Meta data (starting from 0)
//			meta2				// Long Word 2 from the Meta Data
//			meta3				// Long Word 3 from the meta data
//
//			This info will be used by the IPV4 block (by calling dl_source)
//
//			Note:
//			sig_mask is a bit mask of signals to wait on. This is passed in a regiter.
//			If there is no need to wait for any signal, pass SIG_NONE. 
//
//	 	Outputs:
//
//		Inputs:
//			wxfer_prefix:		SRAM Write Transfer Register name prefix (using xbuf_alloc).
//			req_sig			:	signal to be used in the I/O (scratch put)
//			sig_mask		:	What to do with I/O operation - 
//								- wait for signal(s) as specified by sig_mask (register)
//								- do not wait for signal, just return (SIG_NONE - constant)
//			The following Global variables will also act as inputs.
//			dl_next_block		- IX_EXCEPTION, for exception packets
//								- BID_NEXT_BLOCK, block ID of the next block to handle this packet.
//			dl_buf_handle		: Buffer handle of the buffer in the start of chain (contianinf SOP)
//			dl_buf_eop_handle	: Buffer handle of the last buffer in the chain
//		Size:   				:
//			N/A
//
#macro dl_pkt_rx_sink_default[wxfer_prefix, req_sig, sig_mask]

.begin

	// We need to act only on EOP. otherwise just return.

	br=byte[dl_next_block, 0, BID_NEXT_BLOCK, sink_it#], defer[3]

	//	Move required data to xfer registers in the defer slot.

		move[wxfer_prefix[0], dl_buf_handle]		; dl_buf_handle -> xfer0
		move[wxfer_prefix[1], dl_eop_buf_handle]	; dl_eop_buf_handle -> xfer1
		move[wxfer_prefix[3], dl_meta[2]]			; LW2 of meta data -> xfer3.


	//	In the case of an exception packet, it needs to be sent to the core (Xscale)
	//	through a different ring.

check_exception#:

	br!=byte[dl_next_block, 0, IX_EXCEPTION, nothing_to_do#]
	
	// Do Exception processing here.
	
	// For now just drop the packet.

drop_pkt#:

	// 	Drop the packet. Two cases to handle. Single buffer or a bufffer chain.
	//	It is determined by looking at the dl_eop_buf_handle

	alu[--, dl_eop_buf_handle, -, IX_NULL]					; if dl_eop_buf_handle == IX_NULL
	beq[drop_single_buffer#]								; then it is single buffer.

	#ifdef POTS
		.reg seq_num

		// store sequence number in variable to be used in dl_buf_drop_chain()
		alu[seq_num, --, b, @global_seq_num]

		// mask msw to make global_seq_num a 16 bit counter
		alu[@global_seq_num, 1, +16, @global_seq_num]
	#endif

	dl_buf_drop_chain[dl_buf_handle, dl_eop_buf_handle]	; otherwise, buffer chain.

	//	XXX - This is hardcoded for port 0 for now. 
	//	To be fixed later for other cases

	_packet_rx_incr_counter[0, PACKET_PKTS_DROPPED]

	br[nothing_to_do#]

drop_single_buffer#:
	dl_buf_drop[dl_buf_handle]
	br[nothing_to_do#]

sink_it#:

	// Ok. We can sink. Check if the ring is full and then produce the data 
	// onto the ring. If ring full, drop the packet and continue.

 	br_inp_state[DL_RING_FULL, drop_pkt#]				; Check if ring is full.

	// Ring Not Full. Put the packet on the ring.

	//	Fillup 2nd LW of scratch data

	move[wxfer_prefix[2], dl_meta[1]]					; buffersize:offset -> xfer2
	ld_field_w_clr[wxfer_prefix[4], 1100, dl_meta[3]]	; [31:16] input port.
														; [15:0] are reserved.

	#ifdef POTS
		move[wxfer_prefix[5], @global_seq_num]
		alu[@global_seq_num, 1, +, @global_seq_num]
		scratch[put, wxfer_prefix[0], 0, dl_ring, 6], sig_done[req_sig]	; Produce 6 Long words on the ring
	#else
		scratch[put, wxfer_prefix[0], 0, dl_ring, 5], sig_done[req_sig]	; Produce 5 Long words on the ring
	#endif

#if (!streq('sig_mask', 'SIG_NONE'))
.reg tmp

	//	Add the signal to the signal mask on which to wait

	alu_shf[sig_mask, sig_mask, OR, 1, <<&req_sig]		; shift 1 by req_sig times

#endif


nothing_to_do#:

#if (!streq('sig_mask', 'SIG_NONE'))
	local_csr_wr[active_ctx_wakeup_events, sig_mask]
	ctx_arb[--]
#endif

	
.end
#endm


///////////////////////////////////////////////////////////////////////////////
// dl_pkt_proc_to_qm_sink_default:
//	 	Description: 
//			Called by IPv4 Block to send the packet to Queue Manager thro' a scratch ring.
//			This macro will do thread ordering always. 
//		
//			This macro will produce the following on to the scratch ring.
//			dl_buf_handle		// The current buffer containing SOP 
//			dl_eop_buf_handle	// for large packets, the buffer containing EOP. Otherwise NULL
//			Queue Number		// Queue number into which this packet is to be queued.
//								// Queue number = 16 * output Fabric id. (available in meta data)
//	 	Outputs:
//			sig_mask			- if the I/O operation is made (scratch put) then the sig_mask
//								  will have the bit corresponding to sig_scr set. (sig_scr is an
//								  input parameter)
//
//		Inputs:
//			sig_scr				- Signal to use in the I/O (scracth put ) operation
//			wxfer				- SRAM write xfer regsiter to be used in I/O.
//								  Should have been allocated using xbuf_alloc. 
//								  Atleast 3 regsiters should be allocated.
//
//			The following Global variables will also act as inputs.
//			dl_next_block		- IX_EXCEPTION, for exception packets
//								- IX_NULL, for null packets.
//								- BID_NEXT_BLOCK, block ID of the next block to handle this packet.
//			dl_buf_handle		: Buffer handle of the buffer in the start of chain (contianinf SOP)
//			dl_buf_eop_handle	: Buffer handle of the last buffer in the chain
//		
//		Size:   				:
//			N/A
//
//		Notes:
//
//		See notes in dl_source macro

#macro dl_pkt_proc_to_qm_sink_default[sig_mask, sig_scr, wxfer]

#if (IPV4_TO_QM_SCR_RING > 11)
#error			"For Ring Number Greater than 11, we cannot use the SCR_Ring#_status input state. Check IPV4_TO_QM_SCR_RING"
#else
#define_eval	DL_IPV4_QM_RING_FULL	SCR_Ring/**/IPV4_TO_QM_SCR_RING /**/_status
#endif

.begin
.sig	sig_meta, sig_ip


	xbuf_alloc[$meta_cache, 8, write]					; This line doesn't produce any code.

	//	This macro can be seen in two parts. The normal packet case (which is the critical
	//	path) and other(drop, null pktetc) cases

	//	First the normal case.

	// compute the cell count

	compute_sop_buf_cframes(dl_buf_handle)

	// now check for next block

	br!=byte[dl_next_block, 0, BID_NEXT_BLOCK, check_exception#]  
	

	//	Write back Meta data. LW0 and LW1 were already written by POS.
	//	So we need to write the rest. (LW2 ...LW7)


//	The 1st LW of the meta data is next buffer pointer and should not be updated
//  when doing dl_meta_flush_cache
#define_eval META_SIZE (META_CACHE_SIZE -1)
	dl_meta_flush_cache[$meta_cache, dl_buf_handle, sig_meta, SIG_NONE, 1, META_SIZE]		; Write back Meta data to SRAM
	dl_flush_iphdr_cache[dl_buf_handle, sig_ip, SIG_NONE]					; Write back IP hdr to DRAM

	//	Wait for the prev. I/O as well as signal from previous thread.

	#ifdef POTS
	ctx_arb[sig_meta, sig_ip]
	#else
	ctx_arb[sig_meta, sig_ip, sig_prev]


	// 	Signal Next Thread. Signaling next thread has a latency of 10 cycles. 
	//  So we signal at the first available oppurtunity. (see notes in dl_source macro).
	//	It is IMPORTANT that there should be not ctx_swap/arb from here to scratch[put...]
	//	below.

		br=ctx[7, thread_7#]							; Special case for thread 7.
		signal_next_ctx[DL_SIG_WAKE]					; !7, so signal the next thread

thread_7#:
	#endif

	// 	If thread 7 we have to signal next ME. But we cannot do that until we have issued
	//	the scratch[put..] command. We'll do it in dl_source macro. (see notes in dl_source).


	// Check if Ring is FUll. If Full, just stall the pipeline. (i.e keep looping)

check_ring_full#:
 	br_inp_state[DL_IPV4_QM_RING_FULL, ring_is_full#]; Check if ring is full and loop

	// At this point, ring has enough space. Sink the data.

	move[wxfer[0], dl_buf_handle]						; LW0 on the ring is current buffer hande
	move[wxfer[1], dl_eop_buf_handle]					; LW1 on the ring is the last buffer in the chain, 
														; for big pkts only. Othereise IX_NULL
.reg	queue_number

	dl_meta_get_fabric_port[queue_number]				; LW2 on the ring is Queue number for Queue Manager
	alu_shf[queue_number, --, B, queue_number, <<4]		; fabric * 16 is the actual queue number
	alu_shf[wxfer[2], queue_number, OR, 1, <<31]		; The msb is always set to 1, to prevent producing a value of 0 on to the ring.

.reg ring

	alu_shf[ring, --, b, IPV4_TO_QM_SCR_RING, <<2]		; ring nos in get/put is always multiple of 4.

#ifdef POTS
	alu[wxfer[3], --, b, seq_num]
	scratch[put, wxfer[0], 0, ring, 4], sig_done[sig_scr]	; Produce 4 Long words on the ring
#else
	scratch[put, wxfer[0], 0, ring, 3], sig_done[sig_scr]	; Produce 3 Long words on the ring
#endif

	//	To improve performance, we do not ctx_swap on scratch[put..] above. We'll wait
	//	on this signal in dl_source macro. (see notes in dl_source macro). However this(dl_qm_sink) 
	//	macro has multiple paths in which we do not issue this scratch[put..] at all. 
	//	e.g exception packets, or NULL packets etc. So if we issue scratch[put..], 
	//	we set the sig_mask to indicate that.

.reg tmp

	//	Add the signal to the signal mask on which to wait
	br[end#], defer[1]
	alu_shf[sig_mask, sig_mask, OR, 1, <<(&sig_scr)]

ring_is_full#:
	nop													; to set break point
	br[check_ring_full#]								; go back and check again

	// 	Check for exception Packets. Exception packets are sent to the core (Xscale)
	//	on a separate ring. 

check_exception#:


	br!=byte[dl_next_block, 0, IX_EXCEPTION, check_drop#]

#ifdef	FLUSH_EXCP_PKT_HDR

	// someone (core) may need packet header. So flush it to memory.
	dl_flush_iphdr_cache[dl_buf_handle, sig_ip, SIG_NONE]					; Write back IP hdr to DRAM
	
#define_eval META_SIZE (META_CACHE_SIZE -1)
	dl_meta_flush_cache[$meta_cache, dl_buf_handle, sig_meta, SIG_NONE, 1, META_SIZE]		; Write back Meta data to SRAM
	//	Wait for the prev. I/O as well as signal from previous thread.

	#ifdef POTS
	ctx_arb[sig_meta, sig_ip]
	#else
	ctx_arb[sig_meta, sig_ip, sig_prev]
	#endif

#else	// FLUSH_EXCP_PKT_HDR

	//
	// do not flush packet header
#define_eval META_SIZE (META_CACHE_SIZE -1)
	dl_meta_flush_cache[$meta_cache, dl_buf_handle, sig_meta, SIG_NONE, 1, META_SIZE]		; Write back Meta data to SRAM
	//	Wait for the prev. I/O as well as signal from previous thread.

	#ifdef POTS
	ctx_arb[sig_meta]
	#else
	ctx_arb[sig_meta, sig_prev]
	#endif


#endif	// end FLUSH_EXCP_PKT_HDR



#ifndef USE_IMPORT_VAR

	//	There are no core components to handle exception packets. So simply
	//	drop the packets. Because we are dropping packets and dl_buf_drop_chain
	//	does a ctx_arb[...] we cannot wakeup next thread yet. We'll do that at the 
	//	end.

	// 	Drop the packet. Two cases to handle. Single buffer or a bufffer chain.
	//	It is determined by looking at the dl_eop_buf_handle

	alu[--, dl_eop_buf_handle, -, IX_NULL]					; if dl_eop_buf_handle == IX_NULL
	beq[drop_single_buffer#]								; then it is single buffer.

	dl_buf_drop_chain[dl_buf_handle, dl_eop_buf_handle]	; otherwise, buffer chain.
	br[signal_next#]

drop_single_buffer#:
	dl_buf_drop[dl_buf_handle]
	br[signal_next#]

#else

    //This code will send pkts to the CORE on two rings based on priority.
    dl_exception_send(dl_buf_handle)

	br[signal_next#]

#endif //USE_IMPORT_VAR
check_drop#:

#ifndef POTS

	// 	Wait for signal from prev thread and then signal next context.

	ctx_arb[sig_prev]

#endif

	br!=byte[dl_next_block, 0, IX_DROP, check_ix_null#]

	// 	Drop the packet. Two cases to handle. Single buffer or a bufffer chain.
	//	It is determined by looking at the dl_eop_buf_handle

	alu[--, dl_eop_buf_handle, -, IX_NULL]					; if dl_eop_buf_handle == IX_NULL
	beq[drop_single_buffer_1#]								; then it is single buffer.

	dl_buf_drop_chain[dl_buf_handle, dl_eop_buf_handle]		; otherwise, buffer chain.
	br[signal_next#]

drop_single_buffer_1#:
	dl_buf_drop[dl_buf_handle]
	br[signal_next#]

check_ix_null#:

	// Nothing to do to handle null packet.
	nop

signal_next#:

#ifndef POTS

    // Signal next context.

	br=ctx[7, end#]										; Special case for thread 7.
	signal_next_ctx[DL_SIG_WAKE]						; !7, so signal the next thread

#endif

end#:
	xbuf_free[$meta_cache]								; free xfer register. No Code generated.

.end

#undef	DL_IPV4_QM_RING_FULL	// cleanup namespace
#endm

/////////////////////////////////////////////////////////////////////////////
// dl_csix_rx_to_qm_sink
//	
//	Description:
//		Send received packet to the qm block by writing scratch ring
//
//	Outputs:	
//		None
//		
//	Inputs:
//		s_hl:		SOP buffer handle register, this register value will be changed
//					after macroc is excuted.  If the caller want to keep this value  
//					the value should be stored some where 
//		e_e_hl:		Encoded EOP buffer handle: NULL for one buffer cframe
//					otherwise will be real EOP handle  
//		cls_n:		Class number
//		o_n:		Output port number
//		p_size:		packet size
//		flow_color:	flow and color information passed as color[24.31] and flow id [0..23]
//		rx_sig:		Signal used to wait receving logid wake up  
//		scr_sig:	Scratch ring write signal
//		rt_lbl:		return lable after waked up by receiving logic
//		qm_ring_num:QM receiving ring number
//	CONSTANTS:	
//		None
//	
//
//	Size: 8 instructions
/////////////////////////////////////////////////////////////////////////////

#macro dl_csix_rx_to_qm_sink[s_hl, e_e_hl, cls_n, o_n, p_size, flow_color_id, rx_sig, scr_sig,\
					 rt_lbl, qm_ring_num]
.begin
/////////////////////////////////////////////////////////////////////////////
// Local register declaration

.reg tmp_0													; temporary gpr 0

	xbuf_alloc[$sink, 3, write]								; alloc $sink
	alu[$sink[0], --, B, s_hl]								; set SOP hndl to $sink0
	alu[$sink[1], --, B, e_e_hl]								; set EOP hndl to $sink1
#ifndef ATM_TX
	alu[tmp_0, cls_n, or, o_n, <<4 ]						; set output port/class num
#else
	alu[tmp_0, --, B, cls_n ]								; Do not set oput port num
#endif
	alu[$sink[2], tmp_0, or,1, <<31 ]							; set MSB in $sink1
	scratch[put, $sink[0], 0, qm_ring_num, 3],\
			sig_done[scr_sig]								; put to SCR ring
	ctx_arb[rx_sig, scr_sig],and,br[rt_lbl]					; ctx swap
	xbuf_free[$sink]										; free $sink
.end
#endm



#endif	// __DL_MSG_IF_DEFAULTS_UC__