///////////////////////////////////////////////////////////////////////////////
//
//
//                  I N T E L   P R O P R I E T A R Y
//
//     COPYRIGHT [c]  2002 BY  INTEL  CORPORATION.  ALL RIGHTS
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT
//     THE PRIOR WRITTEN PERMISSION OF :
//
//                        INTEL  CORPORATION
//
//                     2200 MISSION COLLEGE BLVD
//
//               SANTA  CLARA,  CALIFORNIA  95052-8119
//
///////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: packet_rx.uc
//
//      Purpose: This file provides single ME Rx and Tx support. Rx and Tx run 
//				on 4 therads each.
//
/////////////////////////////////////////////////////////////////////////////

#ifndef	__PACKET_RX_TX_UC__
#define	__PACKET_RX_TX_UC__

// This microblock can run on either one ME or two ME's. When run on two ME's,
// it runs as a context pipeline with data being sent from the first ME to 
// the second ME via the next neighbor ring. So the two ME's have to be allocated
// side-by-side.
// By default the microblock runs on one ME.
// If a 'TWO_ME_PACKET_RX' switch is added, it runs on two ME's.
// The first ME of the context pipeline is identified using the
// 'FIRST_PACKET_RX_ME' switch.
// The second ME of the context pipeline is identified using the
// 'SECOND_PACKET_RX_ME' switch.

// Including needed files.
#include <dl_system.h>
#include <stdmac.uc>
#include <hardware.h>
#include <dispatch_loop.uc>
#include <dl_meta_sme_tx_rx.uc>

//------------------------------------------------
//	Main entry point - Code begins execution here.
//------------------------------------------------

main#:

#ifdef _DEBUG_COUNTERS_
.reg @pkt_rx_num_proper_mpkts_rxed		; number of non-error mpackets received
.reg @pkt_rx_num_err_mpkts_rxed			; number of error mpackets received
.reg @pkt_rx_drop_scr_ring_full			; number packet dropped because of scratch ring full
.reg @pkt_rx_num_pkts					; number of pkts send to next stage
.reg @pkt_rx_drop						; number of single buffer pkts dropped.
.reg @pkt_rx_buf_alloc_fail				; number of buffer alloc failures
#endif // _DEBUG_COUNTERS_



//---------------------------------
//	Packet RX running on one ME.
//---------------------------------

.if((ctx()==0)||(ctx()==1)||(ctx()==2)||(ctx()==3))

#include <packet_rx_one_me_util.uc>
#include <packet_rx.h>
#include <packet_rx_context.h>
#include <packet_rx_common_util.uc>
#include <packet_rx_init.uc>


.begin

	; Initialization.
	; This macro also does system initialization.
	packet_rx_init()

system_init_done#:		

.while(1)

	; Signal used to sink a packet.
	.sig	sig_sink
	; Mask that holds signals to wait on at the end of each iteration.
	.reg	sink_sig_mask

	; Allocate transfer registers to be used by dl_sink
	xbuf_alloc($x_dl_sink, 5, write)	// no code generated

	; Packet processing code. For every mpacket the output of this
	; macro is dl_next_block, dl_buf_handle and dl_eop_buf_handle. 
	packet_pkt_rx()

	; Send the packet to the next block in the pipeline.
	; Even though dl_sink is called for every mpacket, actual
	; work will be done only on receiving EOP.
	dl_sink($x_dl_sink, sig_sink, sink_sig_mask)

	; To avoid warnings. The assembler can't find that we actually
	; waited on this sig_sink because of the use of signal masks.
	.io_completed sig_sink

	; Free the transfer registers
	xbuf_free($x_dl_sink)	// no code generated

.endw

.end


.else //ctx >3

.begin

#define DL_META_DATA_IN_SXFER

#ifdef __DL_META_UC__
#undef __DL_META_UC__
#endif

#include <dl_meta_sme_tx_rx.uc>

#undef DL_META_DATA_IN_SXFER

#include "sphy_mphy4_tx.uc"

	sphy_mphy4_tx_init(port_entry_turnaround, SPHY_2_16)

	sphy_mphy4_tx()

.end
.endif
nop
nop

#endif	//	__PACKET_RX_UC__