/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2X00 Network Processor, Release 6
 *
 * = LIBRARY
 *      Ethernet ARP Library
 *
 *
 * = FILENAME
 *      internal_arp.h
 *
 * = DESCRIPTION
 *      This file will define all internal data types used in the Ethernet ARP library. Also
 *      will declare non-static function prototypes internal to the module . 
  *
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      11/17/2002 10:11:08 AM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: internal_arp.h,v 1.11 2003/04/05 00:54:52 ageorges Exp $ 
 */

/** Ensure the file is only included once. */
#if !defined(__INTERNAL_ARP_H_)
#define __INTERNAL_ARP_H_

#if defined(__cplusplus)
extern "C" {
#endif /* __cplusplus */

#include "ix_rm.h"


/* Internal ARP defines */
/* define max number of hosts per port based on assumption that each port is connected to 24 bit subnet */ 
#define IX_ARP_HOSTS_PER_PORT 256
/* max number of physical ports */
#define IX_ARP_NUM_OF_PORTS     4
#define IX_ARP_CACHE_ENTRIES    (IX_ARP_HOSTS_PER_PORT * IX_ARP_NUM_OF_PORTS)




/*
 * NAME: _IX_CC_ARP_NUM_BUCKETS
 *
 * DESCRIPTION: Number of buckets based on the number of elements provided
 * by the caller. This can be tweaked to get better results.
 */
#define _IX_CC_ARP_NUM_BUCKETS(__num)                                 \
   ((__num) / 4 + 1)
    
/* TODO create ID for arp_cache_entry fields */


typedef struct ix_s_arp_cache_link _ix_arp_cache_link;

/* ix_cc_arp_cache_node */
/**
 * 
 * This structure is an arp cache table. 
 * It is a double link list of the arp cache entries.
  */
struct ix_s_arp_cache_link
{
    _ix_arp_cache_link*  pNext;  /* pointer to next cache link in the linked list */
    _ix_arp_cache_link*  pPrev;  /* pointer to the previous cache link in the linked list */
    ix_cc_arp_cache_entry*  pCacheEntry;      /* pointer to cache entry structure */
};

/* ix_arp_context */
/**
 *
 * This structure is used as a global holder for pointers to structures and functions
 * that needs to be kept persistant from one call to another call.
 * Gets filled up only during initialization
 */
typedef struct ix_s_arp_context
{
    ix_cc_arp_local_ip_lookup       pIpLkupCb;          
    ix_cc_arp_local_port_lookup     pPortLkupCb;        
    ix_cc_arp_send_pkt              pArpSendPkt;        
    ix_buffer_free_list_handle      hFreeList;          
    void*                           pCallerContext;     
    _ix_arp_cache_link*				pCacheHead;     /* start of the ARP cache table */ 
    ix_cc_hash_table                hArpHash;       /* handle of hash table */
    ix_event_handle                 hEventHandle;  /* handle for ARP event */
    ix_cc_l2tm                      hL2tm;          /* handle to L2 Table manager */
    ix_uint32                       numCacheEntries;              
}ix_arp_context;



#define MIN_L2IX    1UL
#define MAX_L2IX    0xFFFFUL
/* intenal macros */
#define IS_L2IDX_IN_RANGE(ix) ((MIN_L2IX < ix) && (ix < MAX_L2IX))
/* dump timeout */
#define IX_ARP_DUMP_TIMEOUT 10000

/**
*  Internal function declarations
*/
ix_error _ix_arp_cache_lookup(ix_uint32 arg_NextHopIp,ix_cc_arp_cache_entry** arg_ppCacheEntry);
ix_error _ix_arp_cache_add(ix_cc_arp_cache_entry* arg_pCacheEntry);
ix_error _ix_arp_cache_entry_alloc(ix_cc_arp_cache_entry** arg_ppCacheEntry);
void _ix_arp_free_entry(ix_cc_arp_cache_entry** arg_ppCacheEntry);
ix_error _ix_arp_cache_remove_entry(ix_cc_arp_cache_entry** arg_ppCacheEntry);
ix_error _ix_arp_cache_l2idx_find(ix_uint16 arg_L2Index,ix_cc_arp_cache_entry** arg_ppCacheEntry);
ix_error _ix_arp_cache_del_entry(ix_uint32 arg_Ip);
/* deletes all entries in arp cache and frees the memory */
ix_error _ix_arp_cache_purge(void); 

#if defined(__cplusplus)
} /* extern "C" */
#endif /* __cplusplus */
#endif /* __INTERNAL_ARP_H_ */
