/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      ix_cc_arp.c
 *
 * = DESCRIPTION
 *         The file contains functions of Ethernet ARP library
 *
 * = AUTHOR
 *      Herman Zaks
 *      Herman.Zaks@intel.com
 *
 **/
#define IX_ERROR_FILE_IDENT "$Id: ix_cc_arp.c,v 1.54 2003/12/12 01:01:36 ktseng Exp $"

#include "ix_cc_error.h"
#include "ix_types.h"
#include "ix_error.h"
#include "ix_rm.h"

#if !defined(IX_EXCLUDE_CCI) 
    #include "ix_cci.h"
#endif  /* end !defined(IX_EXCLUDE_CCI) */


#include "ix_cc.h"
#include "bindings.h"
#include "ix_netmacros.h"
#include "ix_cc_properties.h"
#include "ix_rm.h"
#include "ix_ossl.h"

#include "cc/ix_cc_hash_table.h"
#include "cc/ix_cc_l2tm.h"
#include "cc/ix_cc_arp.h"
#include "cc/internal/internal_arp.h"

/**
 * Variable declarations global to this file only.  Extern are followed
 * by static variables.
 */
PRIVATE ix_uint8 ix_ether_broadcast[IX_ETH_ALEN] = {0xff,0xff,0xff,0xff,0xff,0xff};
 /**
 * Exported variable declarations global to this component only.
 */
ix_arp_context* g_pArpContext = NULL;


/**
 * Pre-processor symbols and macros used in this file.
 */



/**
 * Extern function prototypes
 */


/**
 * Static function prototypes
 */
#if defined(IX_ARP_DEBUG)
PRIVATE ix_error _ix_arp_pkt_dump(ix_buffer_handle arg_hDataToken);
#endif
PRIVATE ix_error _ix_arp_update_l2table(ix_cc_arp_next_hop_info* arg_pArpInfo, ix_ether_hdr* arg_pEthHdr);
PRIVATE ix_error _ix_arp_get_local_port_info(ix_cc_arp_next_hop_info* arg_pArpInfo, ix_ether_hdr* arg_pEthHdr, 
                                     ix_uint32* arg_LocIp );
PRIVATE ix_error _ix_arp_free_context(ix_arp_context* pArpContext);
PRIVATE ix_uint32 _ix_arp_get_host_num(ix_cc_init_context* pCcInitContext);
PRIVATE ix_error _ix_arp_populate_context(ix_arp_init_context* arg_pInitContext, ix_arp_context* pArpContext);
PRIVATE ix_error _ix_arp_create_cache(ix_arp_context* pArpContext);
PRIVATE void _ix_arp_reset_timer(ix_cc_arp_cache_entry* arg_pCacheEntry, ix_uint32 timeout);
PRIVATE ix_error _ix_arp_update_l2tm_entry(ix_uint32 arg_NewIp, ix_uint8* arg_NewMacAddr, ix_uint8* arg_LocMacAddr,
								   ix_uint16* arg_pL2Ix);
ix_error _ix_arp_learn_new_entry(ix_uint32 arg_NewIp, ix_uint8* arg_NewMacAddr, ix_uint8* arg_LocMacAddr);
PRIVATE ix_error _ix_arp_create_unicast_reply(ix_buffer_handle *arg_pRtnPkt, ix_uint8* arg_pBuffData, ix_uint32 arg_LocIp,
                                      ix_uint8* arg_pLocMacAddr, ix_uint32 arg_DstIp, 
                                      ix_uint8* arg_pDstMacAddr,
                                      ix_uint16 arg_OutputPortId, ix_buffer_handle arg_FreeBuf);
PRIVATE ix_error _ix_arp_create_broadcast_request(ix_uint32 arg_NextHopIp,ix_uint32 arg_LocalIp,
                                          ix_ether_hdr* arg_pLocalEthHead,ix_buffer_handle* arg_pRtnPkt, 
                                      ix_uint16 arg_OutputPortId);
PRIVATE void _ix_arp_print_dump_heading(void);
PRIVATE void _ix_arp_print_cache_entry(ix_cc_arp_cache_entry* arg_pCacheEntry);

/**
 * Function definitions.
 */
/* debug dump of the packet */
#if defined(IX_ARP_DEBUG)
PRIVATE ix_error _ix_arp_pkt_dump(
                ix_buffer_handle arg_hDataToken)
{
    void* pData; 
    ix_uint8* pTemp;
    ix_uint32 sizeOfData;
    void* pMeta;
    ix_uint32 i;
    ix_error err;

    /* dump content of the buffer */
    ix_ossl_message_log(" Packet data in ix_arp_pkt_dump() \n");
    err = ix_rm_buffer_get_meta(arg_hDataToken, &pMeta);
    if (IX_SUCCESS != err)
    {
        return IX_ERROR_WARNING(err, ("Failed to get meta data"));
    }

    err = ix_cc_hw_get_packet_data(arg_hDataToken, &pData);
    if (IX_SUCCESS != err)
    {
        return IX_ERROR_WARNING(err, ("Failed to get buffer data"));
    }
    sizeOfData = IX_NTOH16(((ix_hw_buffer_meta*)pMeta)->m_BufferSize); 
    pTemp = ix_ossl_malloc(sizeOfData);
    ix_ossl_memcpy(pTemp, pData, sizeOfData);
    ix_ossl_message_log("Printing the buffer ...\n");
    for ( i=0;i < sizeOfData; i++)
    {
        printf("%02x ", pTemp[i]);
        if ( (i+1) % 4 == 0)
        {
            printf("\n");
        }
    }
    ix_ossl_message_log("\n");
    ix_ossl_free(pTemp);
    /* end dumping of the contents */
    return IX_SUCCESS;
}
#endif /* IX_ARP_DEBUG */
/*_ix_arp_update_l2table(arg_pArpInfo,&etherHdr)*/

PRIVATE ix_error _ix_arp_update_l2table(ix_cc_arp_next_hop_info* arg_pArpInfo, ix_ether_hdr* arg_pEthHdr)
{
    ix_cc_l2tm_entry  l2tm_entry;
    l2tm_entry.l2AddrType =  IX_CC_L2TM_L2ADDR_ETHER;
    l2tm_entry.ipAddrType = IX_CC_L2TM_IPADDR_IPV4;
    l2tm_entry.l3Address.ipv4Address = arg_pArpInfo->nextHopIp;
    ix_ossl_memcpy(&(l2tm_entry.header.ether),arg_pEthHdr, sizeof(ix_ether_hdr));
    /* this call set L2 entry into the valid state */
    IX_ERROR_CR(ix_cc_l2tm_update_entry(g_pArpContext->hL2tm,arg_pArpInfo->l2Idx, &l2tm_entry));
    return IX_SUCCESS;
}

/*_ix_arp_get_local_port_info() */
PRIVATE ix_error _ix_arp_get_local_port_info(ix_cc_arp_next_hop_info* arg_pArpInfo, ix_ether_hdr* arg_pEthHdr, 
                                     ix_uint32* arg_LocIp )
{
    ix_ether_addr lcL2Src;
    ix_uint32     lcIp;
    /* get local src Eth address and ip address based on the egress output port*/
    IX_ERROR_CR(g_pArpContext->pPortLkupCb(g_pArpContext->pCallerContext,arg_pArpInfo->outputPort, &lcL2Src, &lcIp)); 
    /* create new ether hdr */
    ix_ossl_memcpy(arg_pEthHdr->srcAddr.etherAddr, lcL2Src.etherAddr, IX_ETH_ALEN);
    arg_pEthHdr->protFld = IX_ETH_P_IP;
    *arg_LocIp = lcIp;
    return IX_SUCCESS;
}

 /**
 * NAME: _ix_arp_add_event_handler
 *
 * DESCRIPTION: This function will be used to add arp event handler for
 * periodcally cleaning ARP cache           .
 * 
 * @Param:  - IN  none
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 */

ix_error _ix_arp_add_event_handler(void)
{

    ix_uint32 eventInterval;
    /* Calculate event interval in milliseconds */
    eventInterval = IX_CC_ARP_CACHE_TIMEOUT; /* value in milliseconds 5min * 60 * 1000 */
    #if defined(IX_EXCLUDE_CCI) /* compile time flag */
    {
        
        /* Unable to create event handler IX_ERROR_CR(ix_arp_create_event_handler(g_pArpContext)); */
    }
    #else
    {
        ix_uint32       exeNum;
        ix_exe_handle   hExeHandle;
        void*           pExeContext;

        /* get execution engine handle, number, context */
        IX_ERROR_CRT(ix_cci_exe_get_info(&hExeHandle, &exeNum, &pExeContext),IX_CC_ARP_ERROR_CCI,
                                                                          IX_ERROR_LEVEL_WARNING);
        /* add cci event handler */
        IX_ERROR_CRT(ix_cci_cc_add_event_handler_ex(hExeHandle,
                                          eventInterval,
                                          ix_arp_cache_timeout,
                                          IX_EVENT_TYPE_PERIODIC,
                                          IX_CC_ARP_EVENT_PRIORITY,
					  g_pArpContext,	
                                          &g_pArpContext->hEventHandle),
                    IX_CC_ARP_ERROR_CCI,
                    IX_ERROR_LEVEL_WARNING);
    }
    #endif
    return IX_SUCCESS;
}/* end _ix_arp_add_event_handler */


PRIVATE ix_error _ix_arp_free_context(ix_arp_context* pArpContext)
{
    ix_error err = IX_SUCCESS;
    if (NULL != pArpContext->pCacheHead)
	{
		_ix_arp_cache_purge();
	}
	ix_cc_hash_table_fini(pArpContext->hArpHash);
 	
	ix_cc_l2tm_destroy(pArpContext->hL2tm);
	ix_ossl_memset(pArpContext,0,sizeof(ix_arp_context));

    return err;
}

PRIVATE ix_uint32 _ix_arp_get_host_num(ix_cc_init_context* pCcInitContext)
{
    /*
    ix_uint32 numOfEntries = 0;
    ix_uint32 hostNum = 0;
    ix_cc_properties_node* pPropList = pCcInitContext->pPropertyList;
    */
    /* for now return predefined value */
    return IX_ARP_CACHE_ENTRIES;
}
/* create arp context from incoming init context */
PRIVATE ix_error _ix_arp_populate_context(ix_arp_init_context* arg_pInitContext, ix_arp_context* pArpContext)
{
    ix_error err = IX_SUCCESS;
    
    pArpContext->pCacheHead     = NULL;
    /*pArpContext->numCacheEntries = 0;*/
    pArpContext->hFreeList      = arg_pInitContext->pCcInitContext->hFreeList;
    pArpContext->pArpSendPkt    = arg_pInitContext->pArpSendPkt;
    pArpContext->pCallerContext = arg_pInitContext->pContext;
    pArpContext->pIpLkupCb      = arg_pInitContext->pIpLkupCb;
    pArpContext->pPortLkupCb    = arg_pInitContext->pPortLkupCb;
    pArpContext->numCacheEntries = _ix_arp_get_host_num(arg_pInitContext->pCcInitContext);
    return err;
}


PRIVATE ix_error _ix_arp_create_cache(ix_arp_context* pArpContext)
{
    ix_error err = IX_SUCCESS;
    ix_cc_hash_table hHashTable;
    /* TODO  - protect here by mutex */
	/* set pCache Head to NULL */
    pArpContext->pCacheHead = NULL;
    /* create hash table */
    IX_ERROR_CR(ix_cc_hash_table_init(_IX_CC_ARP_NUM_BUCKETS(pArpContext->numCacheEntries),
                                        pArpContext->numCacheEntries,
                                        IX_CC_HASH_TABLE_ALGO_MAGIC,&hHashTable));
    pArpContext->hArpHash = hHashTable;
    /* TODO - set all entries in the ARP cache to 0 values */
    /* TODO  - end of mutex protection */
    return err;
}

PRIVATE void _ix_arp_reset_timer(ix_cc_arp_cache_entry* arg_pCacheEntry, ix_uint32 timeout)
{
	ix_ossl_time_t sysTime = {0, 0};
    ix_ossl_time_get( &sysTime );
	arg_pCacheEntry->expTime.sec = timeout;
    IX_OSSL_TIME_ADD( arg_pCacheEntry->expTime, sysTime );
}


PRIVATE ix_error _ix_arp_update_l2tm_entry(ix_uint32 arg_NewIp, ix_uint8* arg_NewMacAddr, ix_uint8* arg_LocMacAddr,
								   ix_uint16* arg_pL2Ix)
{
	ix_error err = IX_SUCCESS;
	ix_cc_l2tm_ipaddr l2IpAddr;
	ix_cc_l2tm_entry l2Entry;
	l2IpAddr.ipv4Address = arg_NewIp;
	IX_ERROR_CR(ix_cc_l2tm_get_l2_index(g_pArpContext->hL2tm, IX_CC_L2TM_IPADDR_IPV4, &l2IpAddr, arg_pL2Ix));
	IX_ERROR_CRT(ix_cc_l2tm_get_entry(g_pArpContext->hL2tm, *arg_pL2Ix, NULL, &l2Entry),
					IX_CC_ARP_FAIL_UPDATE_L2TM, IX_ERROR_LEVEL_WARNING);
/* update with new MAC address */
	ix_ossl_memcpy(l2Entry.header.ether.destMAC, arg_NewMacAddr,IX_ETH_ALEN);
	if (NULL != arg_LocMacAddr)
	{
		ix_ossl_memcpy(l2Entry.header.ether.sourceMAC, arg_LocMacAddr, IX_ETH_ALEN);
	}
	l2Entry.header.ether.protocol = IX_ETH_P_IP;

	/* need to set the l2 address type or else the update would fail */
	l2Entry.l2AddrType =  IX_CC_L2TM_L2ADDR_ETHER;
	IX_ERROR_CRT(ix_cc_l2tm_update_entry(g_pArpContext->hL2tm,*arg_pL2Ix,&l2Entry),
					IX_CC_ARP_FAIL_UPDATE_L2TM, IX_ERROR_LEVEL_WARNING);
	return err;

}

/* learn hw address for given IP address - this step is done for every ARP packet
 we received */
/* learning algorithm :
* - search L2 table for entry by IP
* - if found update the entry with new dest hw address, and keep l2 idx around
* - lookup ARP cache by IP and if no matching entry exist add new valid entry 
* - if we have valid l2 idx from step 2 set l2ix to the new ARP entry
* - if entry is found update it with new hw address, set flag to valid and reset timers 
*   Note: args are in host order
*/

ix_error _ix_arp_learn_new_entry(ix_uint32 arg_NewIp, ix_uint8* arg_NewMacAddr, ix_uint8* arg_LocMacAddr)
{
    ix_error err = IX_SUCCESS;
	ix_uint16	l2Ix;
	ix_uint8	l2found = 0;
    ix_uint8	newEntry = 0;

	ix_cc_arp_cache_entry* pCacheEntry = NULL;

	err =_ix_arp_update_l2tm_entry(arg_NewIp, arg_NewMacAddr, arg_LocMacAddr, &l2Ix);
	if (IX_SUCCESS == err || IX_CC_ARP_FAIL_UPDATE_L2TM == err)
	{
		/* l2tm entry is found - keep L2 ix around */
		l2found = 1;
	}
	/* update ARP cache */
	err = _ix_arp_cache_lookup(arg_NewIp, &pCacheEntry);
	if (err != IX_SUCCESS) /* no entry lets create new one */
	{
		IX_ERROR_CR(_ix_arp_cache_entry_alloc(&pCacheEntry));
		pCacheEntry->destIp = arg_NewIp;
        newEntry = 1;
	}
	ix_ossl_memcpy(pCacheEntry->destEthAddr.etherAddr, arg_NewMacAddr, IX_ETH_ALEN);
	pCacheEntry->arpFlgs &= ~IX_ARPF_INUSE;
	pCacheEntry->arpFlgs |= IX_ARPF_VALID;
	pCacheEntry->arpXmit = 0;
	if (l2found)
	{
		pCacheEntry->l2Idx = l2Ix;	
	}

	/* reset timer */
	_ix_arp_reset_timer(pCacheEntry,IX_CC_ARP_CACHE_ENTRY_TIMEOUT); 

    if (newEntry == 1)
    {
        IX_ERROR_C(_ix_arp_cache_add(pCacheEntry), err);
        if (IX_SUCCESS != err)
        {
            _ix_arp_free_entry(&pCacheEntry);
        }
    }
    return err;
}

/* creates unicast ARP reply */
PRIVATE ix_error _ix_arp_create_unicast_reply(ix_buffer_handle* arg_pRtnPkt, ix_uint8* arg_pBuffData,ix_uint32 arg_LocIp,
                                      ix_uint8* arg_pLocMacAddr, ix_uint32 arg_DstIp, 
                                      ix_uint8* arg_pDstMacAddr,
                                      ix_uint16 arg_OutputPortId, ix_buffer_handle arg_FreeBuf)
{
    ix_error err = IX_SUCCESS;
	ix_uint16 dataOffset = IX_CC_SOP_HW_BUFFER_PREPEND;
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
	ix_uint16 arp_hdr_field16;
	ix_uint8 arp_hdr_field8;
#else
    ix_ether_hdr* pEthHdr = NULL;
    ix_arp_hdr* pArpHdr = NULL;
    ix_uint8* pArpData = NULL;
#endif
	ix_uint32 lcIp, nhIp;
	ix_buffer_free_list_handle hFreeList;
	ix_buffer_handle hArpBuffer = IX_NULL_BUFFER_HANDLE;
        void* pData;
    /* Used to suppress compile warnings. */
    ix_uint16 temp;
	ix_uint32 cell_count = 0;

    *arg_pRtnPkt = IX_NULL_BUFFER_HANDLE;

    /* we assume that arp request always fit into one buffer  - no accomadation for chains*/
    /* Check if a free buffer has been supplied by the caller */
    if (arg_FreeBuf != IX_NULL_BUFFER_HANDLE)
    {
        hArpBuffer = arg_FreeBuf;
    }
    else
    {
	    hFreeList = g_pArpContext->hFreeList;
        /* get free buffer from the free list */
        err = ix_rm_buffer_alloc(hFreeList, &hArpBuffer);
        if (err != IX_SUCCESS)
            return(IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("Couldn't allocate buffer.\n")));
    }

        /* Calculate the cell count for this buffer. */
    cell_count = ix_cc_calculate_cell_count((ix_uint32)(IX_ARP_PKT_LEN + IX_CC_ARP_PKT_PADDING_LEN), 1);

    /* Write the cell count into the buffer handle */
    ix_rm_buffer_cell_count_set(&hArpBuffer, cell_count);

	/* get buffer's meta */
    IX_ERROR_CGT(ix_rm_buffer_get_meta(hArpBuffer, &pData), err,
            IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_WARNING, label_1);
    temp = IX_RM_MEM_UINT16_WRITE(&((ix_hw_buffer_meta*)pData)->m_PacketSize, IX_HTON16(IX_ARP_PKT_LEN + IX_CC_ARP_PKT_PADDING_LEN));
    temp = IX_RM_MEM_UINT16_WRITE(&((ix_hw_buffer_meta*)pData)->m_BufferSize,
            IX_HTON16(IX_ARP_PKT_LEN + IX_CC_ARP_PKT_PADDING_LEN));
    /* set next hop id in meta to -1 so ublocks don't lookup L2 table */ 
    temp = IX_RM_MEM_UINT16_WRITE(&((ix_hw_buffer_meta*)pData)->m_NextHopID, IX_HTON16(-1));
    
    /* Ensure that the data offset in the meta-data is set. */
    temp = IX_RM_MEM_UINT16_WRITE(&((ix_hw_buffer_meta*)pData)->m_Offset, IX_HTON16(dataOffset));
#if defined (IX_ARP_DEBUG)
    printf("pData->m_HwNext = 0x%08lX\n", IX_NTOH32(((ix_hw_buffer_meta*)pData)->m_HwNext));
#endif
	
    temp = IX_RM_MEM_UINT16_WRITE(&((ix_hw_buffer_meta*)pData)->m_OutputPort, IX_HTON16(arg_OutputPortId));
    
    IX_ERROR_CGT(ix_cc_hw_get_packet_data(hArpBuffer, &pData), err,
            IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_WARNING, label_1);

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

    ix_ossl_memcpy(pData, arg_pDstMacAddr,IX_ETH_ALEN); 
    pData = (ix_uint8 *)pData + IX_ETH_ALEN;

	ix_ossl_memcpy(pData, arg_pLocMacAddr,IX_ETH_ALEN); 
    pData = (ix_uint8 *)pData + IX_ETH_ALEN;

	arp_hdr_field16 = IX_HTON16(IX_ETH_P_ARP);
	ix_ossl_memcpy(pData, &arp_hdr_field16, sizeof(ix_uint16));
    pData = (ix_uint8 *)pData + sizeof(ix_uint16);

    arp_hdr_field16 = IX_HTON16(IX_ARPHRD_ETHER);
	ix_ossl_memcpy(pData, &arp_hdr_field16, sizeof(ix_uint16));
    pData = (ix_uint8 *)pData + sizeof(ix_uint16);

    arp_hdr_field16 = IX_HTON16(IX_ETH_P_IP);
	ix_ossl_memcpy(pData, &arp_hdr_field16, sizeof(ix_uint16));
    pData = (ix_uint8 *)pData + sizeof(ix_uint16);

    arp_hdr_field8 = IX_ETH_ALEN;
	ix_ossl_memcpy(pData, &arp_hdr_field8, sizeof(ix_uint8));
    pData = (ix_uint8 *)pData + sizeof(ix_uint8);

    arp_hdr_field8 = IX_IPV4_ALEN;
	ix_ossl_memcpy(pData, &arp_hdr_field8, sizeof(ix_uint8));
    pData = (ix_uint8 *)pData + sizeof(ix_uint8);

    arp_hdr_field16 = IX_HTON16(IX_ARPOP_REPLY);
	ix_ossl_memcpy(pData, &arp_hdr_field16, sizeof(ix_uint16));
    pData = (ix_uint8 *)pData + sizeof(ix_uint16);

    ix_ossl_memcpy(pData,arg_pLocMacAddr, IX_ETH_ALEN);
    pData += IX_ETH_ALEN;

    lcIp = IX_HTON32(arg_LocIp);
    ix_ossl_memcpy(pData,&lcIp,IX_IPV4_ALEN);
    pData += IX_IPV4_ALEN;

    /* set destination hw addr */
    ix_ossl_memcpy(pData,arg_pDstMacAddr,IX_ETH_ALEN);
    pData += IX_ETH_ALEN;

    nhIp = IX_HTON32(arg_DstIp);
    ix_ossl_memcpy(pData,&nhIp,IX_IPV4_ALEN);
    pData += IX_IPV4_ALEN;

    /* Add padding to make the packet size to be the minimum Ethernet packet size, 64 bytes */
    ix_ossl_memset(pData, 0, IX_CC_ARP_PKT_PADDING_LEN);

#else /* (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_) */

    /* create ethernet header for ARP packet*/
    pEthHdr = (ix_ether_hdr*)(pData);
    /* copy broadcast address into buffer memory */
    ix_ossl_memcpy(pEthHdr->dstAddr.etherAddr, arg_pDstMacAddr,IX_ETH_ALEN); 
    ix_ossl_memcpy(pEthHdr->srcAddr.etherAddr, arg_pLocMacAddr,IX_ETH_ALEN); 

    pEthHdr->protFld = IX_HTON16(IX_ETH_P_ARP);

    /* create ARP header */
    pArpHdr = (ix_arp_hdr*)((ix_uint8*)pData+sizeof(ix_ether_hdr));

    pArpHdr->arHrd = IX_HTON16(IX_ARPHRD_ETHER);
    pArpHdr->arOp =  IX_HTON16(IX_ARPOP_REPLY);
    pArpHdr->arPro = IX_HTON16(IX_ETH_P_IP);
    pArpHdr->arHln = IX_ETH_ALEN;
    pArpHdr->arPln = IX_IPV4_ALEN;

    /* create ARP packet data */
    pArpData = (ix_uint8*)(pArpHdr + 1);

    ix_ossl_memcpy(pArpData,arg_pLocMacAddr,sizeof(ix_ether_addr));
    pArpData += sizeof(ix_ether_addr);
    lcIp = IX_HTON32(arg_LocIp);
    ix_ossl_memcpy(pArpData,&lcIp,IX_IPV4_ALEN);
    pArpData += IX_IPV4_ALEN;
    /* set destination hw addr */
    ix_ossl_memcpy(pArpData,arg_pDstMacAddr,IX_ETH_ALEN);
    pArpData += sizeof(ix_ether_addr);
    nhIp = IX_HTON32(arg_DstIp);
    ix_ossl_memcpy(pArpData,&nhIp,IX_IPV4_ALEN);
    pArpData += IX_IPV4_ALEN;

    /* Add padding to make the packet size to be the minimum Ethernet packet size, 64 bytes */
    ix_ossl_memset(pArpData, 0, IX_CC_ARP_PKT_PADDING_LEN);
    
#endif /* (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_) */

#ifdef IX_ARP_DEBUG
    /* dump packet data */
    _ix_arp_pkt_dump(hArpBuffer);
#endif

    *arg_pRtnPkt = hArpBuffer;
    return IX_SUCCESS;

label_1:
    ix_rm_buffer_free_chain(hArpBuffer);
    return err;
}


PRIVATE ix_error _ix_arp_create_broadcast_request(ix_uint32 arg_NextHopIp,ix_uint32 arg_LocalIp,
                                          ix_ether_hdr* arg_pLocalEthHead,ix_buffer_handle* arg_pRtnPkt,
                                          ix_uint16 arg_OutputPortId)
{
    ix_error err = IX_SUCCESS;
    ix_buffer_free_list_handle hFreeList;
    ix_buffer_handle hArpBuffer = IX_NULL_BUFFER_HANDLE;
    ix_uint16 dataOffset = IX_CC_SOP_HW_BUFFER_PREPEND;
    ix_uint32 nhIp, lcIp; 
    void* pData;
    /* Used to suppress compile warnings. */
    ix_uint32 temp;
    ix_uint32 cell_count = 0;
    
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    ix_uint16 arp_hdr_field16;
    ix_uint8 arp_hdr_field8;
#else
    ix_ether_hdr* pEthHdr = NULL;
    ix_arp_hdr*   pArpHdr = NULL;
    ix_uint8* pArpData = NULL;
#endif
    
    hFreeList = g_pArpContext->hFreeList;
    /* we assume that arp request always fit into one buffer  - no accomadation for chains*/
    /* get free buffer from the free list */
    IX_ERROR_CRT(ix_rm_buffer_alloc(hFreeList, &hArpBuffer),
            IX_CC_ERROR_OOM, IX_ERROR_LEVEL_WARNING);
        /* Calculate the cell count for this buffer. */
    cell_count = ix_cc_calculate_cell_count((ix_uint32)(IX_ARP_PKT_LEN + IX_CC_ARP_PKT_PADDING_LEN), 1);

    /* Write the cell count into the buffer handle */
    ix_rm_buffer_cell_count_set(&hArpBuffer, cell_count);

	/* get buffer's meta */
    IX_ERROR_CGT(ix_rm_buffer_get_meta(hArpBuffer, &pData), err,
            IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_WARNING, label_1);
    temp = IX_RM_MEM_UINT16_WRITE(&((ix_hw_buffer_meta*)pData)->m_PacketSize, IX_HTON16(IX_ARP_PKT_LEN + IX_CC_ARP_PKT_PADDING_LEN));
    temp = IX_RM_MEM_UINT16_WRITE(&((ix_hw_buffer_meta*)pData)->m_BufferSize,
            IX_HTON16(IX_ARP_PKT_LEN + IX_CC_ARP_PKT_PADDING_LEN));
    /* set next hop id in meta to -1 so ublocks don't lookup L2 table */ 
    temp = IX_RM_MEM_UINT16_WRITE(&((ix_hw_buffer_meta*)pData)->m_NextHopID, IX_HTON16(-1));
    
    /* Ensure that the data offset in the meta-data is set. */
    temp = IX_RM_MEM_UINT16_WRITE(&((ix_hw_buffer_meta*)pData)->m_Offset, IX_HTON16(dataOffset));
#if defined (IX_ARP_DEBUG)
    printf("pData->m_HwNext = 0x%08lX\n", IX_NTOH32(((ix_hw_buffer_meta*)pData)->m_HwNext));
#endif
	
    temp = IX_RM_MEM_UINT16_WRITE(&((ix_hw_buffer_meta*)pData)->m_OutputPort, IX_HTON16(arg_OutputPortId));
    
    IX_ERROR_CGT(ix_cc_hw_get_packet_data(hArpBuffer, &pData), err,
            IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_WARNING, label_1);

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

    ix_ossl_memcpy(pData, ix_ether_broadcast,IX_ETH_ALEN); 
    pData = (ix_uint8 *)pData + IX_ETH_ALEN;

    ix_ossl_memcpy(pData, arg_pLocalEthHead->srcAddr.etherAddr,IX_ETH_ALEN); 
    pData = (ix_uint8 *)pData + IX_ETH_ALEN;

    arp_hdr_field16 = IX_HTON16(IX_ETH_P_ARP);
    ix_ossl_memcpy(pData, &arp_hdr_field16, sizeof(ix_uint16));
    pData = (ix_uint8 *)pData + sizeof(ix_uint16);

    arp_hdr_field16 = IX_HTON16(IX_ARPHRD_ETHER);
    ix_ossl_memcpy(pData, &arp_hdr_field16, sizeof(ix_uint16));
    pData = (ix_uint8 *)pData + sizeof(ix_uint16);

    arp_hdr_field16 = IX_HTON16(IX_ETH_P_IP);
	ix_ossl_memcpy(pData, &arp_hdr_field16, sizeof(ix_uint16));
    pData = (ix_uint8 *)pData + sizeof(ix_uint16);

    arp_hdr_field8 = IX_ETH_ALEN;
	ix_ossl_memcpy(pData, &arp_hdr_field8, sizeof(ix_uint8));
    pData = (ix_uint8 *)pData + sizeof(ix_uint8);

    arp_hdr_field8 = IX_IPV4_ALEN;
    ix_ossl_memcpy(pData, &arp_hdr_field8, sizeof(ix_uint8));
    pData = (ix_uint8 *)pData + sizeof(ix_uint8);

    arp_hdr_field16 = IX_HTON16(IX_ARPOP_REQUEST);
    ix_ossl_memcpy(pData, &arp_hdr_field16, sizeof(ix_uint16));
    pData = (ix_uint8 *)pData + sizeof(ix_uint16);

    ix_ossl_memcpy(pData,arg_pLocalEthHead->srcAddr.etherAddr, IX_ETH_ALEN);
    pData += IX_ETH_ALEN;

    lcIp = IX_HTON32(arg_LocalIp);
    ix_ossl_memcpy(pData,&lcIp,IX_IPV4_ALEN);
    pData += IX_IPV4_ALEN;

    /* Set destination hw addr to all 0s */
    ix_ossl_memset(pData,0, IX_ETH_ALEN);
    pData += IX_ETH_ALEN;

    nhIp = IX_HTON32(arg_NextHopIp);
    ix_ossl_memcpy(pData,&nhIp,IX_IPV4_ALEN);
    pData += IX_IPV4_ALEN;

    /* Add padding to make the packet size to be the minimum Ethernet packet size, 64 bytes */
    ix_ossl_memset(pData, 0, IX_CC_ARP_PKT_PADDING_LEN);

#else /* (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_) */

    /* create ethernet header for ARP packet*/
    pEthHdr = (ix_ether_hdr*)(pData);
    /* copy broadcast address into buffer memory */
    ix_ossl_memcpy(pEthHdr->dstAddr.etherAddr,ix_ether_broadcast,IX_ETH_ALEN); 
    pEthHdr->srcAddr = arg_pLocalEthHead->srcAddr;
    pEthHdr->protFld = IX_HTON16(IX_ETH_P_ARP);

    /* create ARP header */
    pArpHdr = (ix_arp_hdr*)((ix_uint8*)pData+sizeof(ix_ether_hdr));
    pArpHdr->arHrd = IX_HTON16(IX_ARPHRD_ETHER);
    pArpHdr->arOp =  IX_HTON16(IX_ARPOP_REQUEST);
    pArpHdr->arPro = IX_HTON16(IX_ETH_P_IP);
    pArpHdr->arHln = IX_ETH_ALEN;
    pArpHdr->arPln = IX_IPV4_ALEN;

    /* create ARP packet data */
    pArpData = (ix_uint8*)(pArpHdr + 1);
    ix_ossl_memcpy(pArpData,arg_pLocalEthHead->srcAddr.etherAddr,sizeof(ix_ether_addr));
    pArpData += sizeof(ix_ether_addr);
    lcIp = IX_HTON32(arg_LocalIp);
    ix_ossl_memcpy(pArpData,&lcIp,IX_IPV4_ALEN);
    pArpData += IX_IPV4_ALEN;
    /* set destination hw addr to all 0s */
    ix_ossl_memset(pArpData,0,sizeof(ix_ether_addr));
    pArpData += sizeof(ix_ether_addr);
    nhIp = IX_HTON32(arg_NextHopIp);
    ix_ossl_memcpy(pArpData,&nhIp,IX_IPV4_ALEN);
    pArpData += IX_IPV4_ALEN;

    /* Add padding to make the packet size to be the minimum Ethernet packet size, 64 bytes */
    ix_ossl_memset(pArpData, 0, IX_CC_ARP_PKT_PADDING_LEN);

#endif /* (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_) */
    
#ifdef IX_ARP_DEBUG
    /* dump packet data */
    _ix_arp_pkt_dump(hArpBuffer);
#endif

    *arg_pRtnPkt = hArpBuffer;
    return IX_SUCCESS;
label_1:
    ix_rm_buffer_free_chain(hArpBuffer);
    return err;
}


PRIVATE void _ix_arp_print_dump_heading(void)
{

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

	ix_ossl_message_log("\n");
	ix_ossl_message_log("%s ","Next Hop IP");
	ix_ossl_message_log("%s ","Next Hop MAC Address");
	ix_ossl_message_log("%s ","L2 Table Index");
	ix_ossl_message_log("%s ","Entry Expiration Time");
	ix_ossl_message_log("%s ","Current System Time");
	ix_ossl_message_log("%s ","Queued buffer handle");
	ix_ossl_message_log("%s ","TX counter");
	ix_ossl_message_log("%s ","ARP Flags");
	ix_ossl_message_log("\n ");
#else
	fprintf(stdout,"\n");
	fprintf(stdout,"%s ","Next Hop IP");
	fprintf(stdout,"%s ","Next Hop MAC Address");
	fprintf(stdout,"%s ","L2 Table Index");
	fprintf(stdout,"%s ","Entry Expiration Time");
	fprintf(stdout,"%s ","Current System Time");
	fprintf(stdout,"%s ","Queued buffer handle");
	fprintf(stdout,"%s ","TX counter");
	fprintf(stdout,"%s ","ARP Flags");
	fprintf(stdout,"\n ");
#endif

}


PRIVATE void _ix_arp_print_cache_entry(ix_cc_arp_cache_entry* arg_pCacheEntry)
{
    ix_uint32 ipAddr;
    char ipAddrString[16];          /* nnn.nnn.nnn.nnn\0 */
	char hwAddrString[18];			/* XX:XX:XX:XX:XX:XX\0 */
	ix_ether_addr hwAddr;
	ix_ossl_time_t sysTime;
	/*int w = 12;*/

	ipAddr = arg_pCacheEntry->destIp;
    sprintf(ipAddrString, "%ld.%ld.%ld.%ld",
        (ipAddr >> 24) & 0xFF,
        (ipAddr >> 16) & 0xFF,
        (ipAddr >>  8) & 0xFF,
        (ipAddr      ) & 0xFF);
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        ix_ossl_message_log("%s", ipAddrString);
#else
	fprintf(stdout,"%s", ipAddrString);
#endif	
	hwAddr = arg_pCacheEntry->destEthAddr;
	sprintf(hwAddrString,"%X:%X:%X:%X:%X:%X",
		hwAddr.etherAddr[0],hwAddr.etherAddr[1],
		hwAddr.etherAddr[2],hwAddr.etherAddr[3],
		hwAddr.etherAddr[4],hwAddr.etherAddr[5]);
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

	ix_ossl_message_log(" %s",hwAddrString);

	ix_ossl_message_log(" %ld",arg_pCacheEntry->l2Idx);

	ix_ossl_message_log(" %ld:%ld,",arg_pCacheEntry->expTime.sec,arg_pCacheEntry->expTime.nsec);
#else
	fprintf(stdout," %s",hwAddrString);

	fprintf(stdout," %ld",arg_pCacheEntry->l2Idx);

	fprintf(stdout," %ld:%ld,",arg_pCacheEntry->expTime.sec,arg_pCacheEntry->expTime.nsec);
#endif	
	ix_ossl_time_get( &sysTime );
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

	ix_ossl_message_log(" %ld:%ld,",sysTime.sec,sysTime.nsec);
	
	ix_ossl_message_log(" %ld",arg_pCacheEntry->arpXmit);

#else
	fprintf(stdout," %ld:%ld,",sysTime.sec,sysTime.nsec);
	
	fprintf(stdout," %ld",arg_pCacheEntry->arpXmit);
#endif	
	/* flags*/
	if (arg_pCacheEntry->arpFlgs & IX_ARPF_VALID)
	{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

		ix_ossl_message_log(" %s","VALID |");		
#else
		fprintf(stdout," %s","VALID |");		
#endif
	}
	else
	{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

		ix_ossl_message_log(" %s","INVALID |");		
#else
		fprintf(stdout," %s","INVALID |");		
#endif
	}
	
	if (arg_pCacheEntry->arpFlgs & IX_ARPF_INUSE)
	{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
               
		ix_ossl_message_log(" INUSE |");		
#else
		fprintf(stdout," INUSE |");		
#endif
	}

	if (arg_pCacheEntry->arpFlgs & IX_ARPF_PERM)
	{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

		ix_ossl_message_log(" PERM. ");		
#else
		fprintf(stdout," PERM. ");		
#endif
	}
	else
	{
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

		ix_ossl_message_log(" DYNAM. ");		
#else
		fprintf(stdout," DYNAM. ");		
#endif
	}

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

	/* end of flags */
	ix_ossl_message_log("\n");			
#else
	/* end of flags */
	fprintf(stdout,"\n");			
#endif
	
}

/**
*	Start of public ARP APIs
*
*
*/


/**
 * NAME: ix_cc_arp_init(()
 *
 * DESCRIPTION: Initialization function invoked by the client of ARP library
 * @Param:  -   IN ix_arp_init_context* arg_pInitContext - pointer to initialization context structure. 
 *
 * @Return: IX_SUCCESS or valid ix_error if code fails ;
 */
/** Pseudo code 
    Initilization of ARP data structures:
    Query local interface table to determine maximum size of the ARP cache: 
    cache_size is size of ARP cache;
    host_size is maxim number of hosts per port = (2 in power of (32- port subnet mask bits)) - 2, for example 255.255.255.0 mask makes host_size = 254;
    calculate all_hosts  for each port  all_hosts += host_size;
    cache_size = all_hosts * sizeof(ix_cc_arp_cache_entry);  
    Use ix_ossl_malloc() call to allocate table of cache_size.  If OSSL calls fails return IX_CC_ERROR_OOM_SYSTEM error code. Store address of the table in the global variable.  
    Call ix_cci_add_event_handler to add an event handler for arp cache refreshing timout with core component infrastructure. This handler will be invoked every 5 minutes If it fails, then return IX_ERROR with error code IX_CC_ARP_ERROR_TIMEOUT. 
    Placeholder for event handler function is invoked in case of L0 system. If ARP Library is ported to L0 system, this event handler needs to be implemented according to event mechanism of that system. 
    3.Copy arg_pInitContext into the global ARP context structure.
    4.Call l2_table_init to get handle to the L2 table manager library
    5.Return IX_SUCCESS.	
**/

ix_error ix_cc_arp_init(ix_arp_init_context* arg_pInitContext)
{
    ix_error err = IX_SUCCESS;
    ix_cc_l2tm  hL2tm;
    /* validate args */
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pInitContext);
    
    /* allocate memory for global context structure */
    g_pArpContext = (ix_arp_context*)ix_ossl_malloc(sizeof(ix_arp_context));
    if (NULL == g_pArpContext)
    {
        return IX_ERROR_WARNING(IX_CC_ERROR_OOM_SYSTEM,("Out of system memory in ix_cc_arp_init"));
    }
    IX_ERROR_CG(_ix_arp_populate_context(arg_pInitContext,g_pArpContext),err, label_1);
    IX_ERROR_CG(_ix_arp_create_cache(g_pArpContext),err, label_1);
    IX_ERROR_CG(_ix_arp_add_event_handler(),err, label_1);
    /* create l2 library handle - need common name for that */
    IX_ERROR_CG(ix_cc_l2tm_create(IX_CC_ARP_L2_TABLE_NAME,&hL2tm),err,label_1);
    g_pArpContext->hL2tm = hL2tm;
    return err;
label_1:
    if (NULL != g_pArpContext)
    {
        ix_ossl_free(g_pArpContext);
    }
    return err;
}


/**
 * NAME: ix_cc_arp_fini(()
 *
 * DESCRIPTION: Termination function invoked by the client of ARP library
 * @Param:  -   NONE
 *
 * @Return: IX_SUCCESS or valid ix_error if code fails ;
 */

ix_error ix_cc_arp_fini()
{
    ix_error err;
    err = ix_cc_arp_purge();
	
	if (IX_SUCCESS != err)
	err = IX_ERROR_CHAIN(err, IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_WARNING,
				("ARP purge fail in arp_fini()"));
	{
        ix_uint32       exeNum;
        ix_exe_handle   hExeHandle;
        void*           pExeContext;

		#if !defined(IX_EXCLUDE_CCI) /* compile time flag */
			/* get execution engine handle, number, context */
			IX_ERROR_CRT(ix_cci_exe_get_info(&hExeHandle, &exeNum, &pExeContext),IX_CC_ARP_ERROR_CCI,
																			  IX_ERROR_LEVEL_WARNING);
	        /* remove cci event handler */
	        err = ix_cci_cc_remove_event_handler(hExeHandle,
	                                          g_pArpContext->hEventHandle);
		#endif

	} 
	err = _ix_arp_free_context(g_pArpContext);

	ix_ossl_free(g_pArpContext);

	g_pArpContext = NULL;

	if (IX_SUCCESS != err)
	err = IX_ERROR_CHAIN(err, IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_WARNING,
				("Freeing context fail in arp_fini()"));
	
    return err;
}

/* arp cache operations */
/**
 * NAME: ix_cc_arp_create_entry()
 *
 * DESCRIPTION: Create Dynamic ARP entry
 * @Param:  -   IN - arg_pArpInfo
 *              OUT - arg_pRtnPkt
 *
 * @Return: IX_SUCCESS or valid ix_error if code fails ;
 */

ix_error ix_cc_arp_create_entry(ix_cc_arp_next_hop_info* arg_pArpInfo,
                                        ix_buffer_handle* arg_pRtnPkt)
{
    ix_error err = IX_SUCCESS;
    ix_error retErr = IX_SUCCESS;
    ix_uint32 nhIp;
    ix_uint32 lcIp;
    ix_ether_hdr  etherHdr;
    ix_cc_arp_cache_entry* pCacheEntry = NULL;
	ix_ossl_time_t sysTime = {0, 0};
        /* validate args */
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pArpInfo);
    IX_ERROR_DCHECK_ARG_NULL(2, arg_pRtnPkt);

    *arg_pRtnPkt = IX_NULL_BUFFER_HANDLE;

    nhIp = arg_pArpInfo->nextHopIp;
    /* get local MAC src address into the etherHdr structure */
    IX_ERROR_CRT(_ix_arp_get_local_port_info(arg_pArpInfo, &etherHdr, &lcIp),
                    IX_CC_ARP_ERROR_NO_LOCAL_MAC_ADDRESS,
                    IX_ERROR_LEVEL_WARNING);
    /* do hash lookup into the ARP cache */
    err = _ix_arp_cache_lookup(nhIp, &pCacheEntry);
    if (IX_SUCCESS == IX_ERROR_GET_CODE(err)) /* found entry - update with the new info  and get dest L2 address*/
    {
        pCacheEntry->l2Idx = arg_pArpInfo->l2Idx;
        ix_ossl_memcpy(etherHdr.dstAddr.etherAddr, pCacheEntry->destEthAddr.etherAddr, IX_ETH_ALEN);

	/* In addition, based on the L2 index,
	   it updates entry in L2 table with L2 header info from
	   the ARP cache and set the entry to valid.*/
	
	/* update l2 table */
        IX_ERROR_CR(_ix_arp_update_l2table(arg_pArpInfo,&etherHdr));

	/* Do we need to add code to remove the permanent flag arg_pCacheEntry->arpFlgs & IX_ARPF_PERM) */

        return(IX_SUCCESS);
    }
    else if (IX_CC_ERROR_ENTRY_NOT_FOUND == IX_ERROR_GET_CODE(err))/* no entry in the ARP cache */
    {
        retErr = err;        
		IX_ERROR_CR(_ix_arp_cache_entry_alloc(&pCacheEntry));
        if (NULL == pCacheEntry)
        {
            return IX_ERROR_WARNING(IX_CC_ERROR_OOM_SYSTEM,("Failed to allocated new cache entry"));
        }
        /* add entry to ARP cache */
        pCacheEntry->arpXmit = 0;
		pCacheEntry->lastSend.sec = 0;
		pCacheEntry->destIp = nhIp;
		pCacheEntry->pktHolder = IX_NULL_BUFFER_HANDLE;
		err = _ix_arp_cache_add(pCacheEntry);
		if (IX_SUCCESS != err )
		{
			_ix_arp_free_entry(&pCacheEntry);
			return IX_ERROR_WARNING(err, ("Failed to add arp cache entry"));
		}
	}
    else
    {
        return(err);
    }

    /* Get current system time */	  
    IX_ERROR_CRT(ix_ossl_time_get( &sysTime ), IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_WARNING);

	if ((pCacheEntry->arpXmit < IX_ARP_MAX_TX) && (pCacheEntry->lastSend.sec < sysTime.sec))
	{
		/* queue original buffer - free one that already was queued*/
		if (pCacheEntry->pktHolder != IX_NULL_BUFFER_HANDLE)
		{
			ix_rm_buffer_free_chain(pCacheEntry->pktHolder);
			pCacheEntry->pktHolder = IX_NULL_BUFFER_HANDLE;
		}
		_ix_arp_reset_timer(pCacheEntry,IX_CC_ARP_CACHE_ENTRY_TIMEOUT);
		pCacheEntry->l2Idx = arg_pArpInfo->l2Idx;
		IX_ERROR_CRT(ix_ossl_time_get( &sysTime ), IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_WARNING);
		pCacheEntry->arpXmit++;
		pCacheEntry->lastSend.sec = sysTime.sec;
		pCacheEntry->arpFlgs |= IX_ARPF_INUSE;
		pCacheEntry->arpFlgs &= ~IX_ARPF_VALID;
		err = _ix_arp_create_broadcast_request(nhIp,lcIp,&etherHdr,arg_pRtnPkt,arg_pArpInfo->outputPort);

        if (IX_SUCCESS != err)
        {
            *arg_pRtnPkt = IX_NULL_BUFFER_HANDLE;
		    return IX_ERROR_WARNING(err, ("Failed to create arp brdcast packet"));
        }
	}

    return retErr;
}

/**
 * NAME: ix_cc_arp_add_entry()
 *
 * DESCRIPTION: Adds static ARP entry
 * @Param:  -   IN - arg_pArpInfo
 *
 * Pseudo code:
 * 1.	Check argument for null. return IX_CC_ERROR_NULL
 * 2.	Next Hop Ip address is used as  input to the hash() function from hash library. Get ARP cache index.
 * 3.	Check if entry already exists and return IX_CC_ARP_ERROR_ENTRY_EXIST.
 * 4.	Fill fields of ix_cc_arp_cache_entry with information from arg_pArpInfo  
 * 5.	Set arp cache expTime to 0 - never expire
 * 6.	Update corresponding entry in L2 Table
 * 7.	return IX_SUCCESS

 * @Return: IX_SUCCESS or valid ix_error if code fails ;
 */
ix_error ix_cc_arp_add_entry(ix_cc_arp_next_hop_info* arg_pArpInfo)
{
    ix_error err = IX_SUCCESS;
    ix_cc_arp_cache_entry* pCacheEntry = NULL;
	ix_uint8 found = 1;
    ix_ether_hdr  etherHdr;
    ix_uint32     lcIp;
    
    IX_ERROR_DCHECK_ARG_NULL(1, arg_pArpInfo);
    
	if (IX_SUCCESS != _ix_arp_cache_lookup(arg_pArpInfo->nextHopIp, &pCacheEntry))
    {
		IX_ERROR_CR(_ix_arp_cache_entry_alloc(&pCacheEntry));
		found = 0;
	}
	if (pCacheEntry != NULL)
	{
        pCacheEntry->arpXmit = 0;
        pCacheEntry->destIp = arg_pArpInfo->nextHopIp;
        _ix_arp_reset_timer(pCacheEntry,IX_CC_ARP_CACHE_ENTRY_TIMEOUT);
        pCacheEntry->l2Idx = arg_pArpInfo->l2Idx;
        pCacheEntry->destEthAddr = arg_pArpInfo->l2Addr;
        if (pCacheEntry->pktHolder != IX_NULL_BUFFER_HANDLE)
		{
			ix_rm_buffer_free_chain(pCacheEntry->pktHolder);
            pCacheEntry->pktHolder = IX_NULL_BUFFER_HANDLE;
		}
        pCacheEntry->arpFlgs &= ~IX_ARPF_INUSE;
        pCacheEntry->arpFlgs |= IX_ARPF_VALID;
		pCacheEntry->arpFlgs |= IX_ARPF_PERM;
		if (!found)
		{
			IX_ERROR_CG(_ix_arp_cache_add(pCacheEntry), err, cache_entry);
		}
        /* update l2 table */
        IX_ERROR_CG(_ix_arp_get_local_port_info(arg_pArpInfo,&etherHdr, &lcIp),err,cache_entry2);
        IX_ERROR_CG(_ix_arp_update_l2table(arg_pArpInfo,&etherHdr), err, cache_entry2);
	}
    
    return err;
cache_entry:
    _ix_arp_free_entry(&pCacheEntry);
    return err;
cache_entry2:
	_ix_arp_cache_remove_entry(&pCacheEntry);
	return err;
}

/**
 * NAME: ix_cc_arp_del_entry()
 *
 * DESCRIPTION: Deletes entry from the ARP cache based on the L2 Index. 
 *              Corresponding entry in the L2 Table is marked as incomplete
 * @Param:  -   IN - arg_L2Index - index into L2 table for entry to be deleted
 *              
 *
 * @Return: IX_SUCCESS or valid ix_error if code fails ;
 */

ix_error ix_cc_arp_del_entry(ix_uint32 arg_L2Index)
{
    ix_error err = IX_SUCCESS;
    ix_cc_arp_cache_entry* pCacheEntry;

    IX_ERROR_DCHECK_ARG_RANGE(1, arg_L2Index, MIN_L2IX, MAX_L2IX, "%ld");

    err = _ix_arp_cache_l2idx_find((ix_uint16)arg_L2Index,&pCacheEntry);
    if (IX_SUCCESS == err) /* entry is found - delete in the ARP cache*/
    {
        _ix_arp_cache_del_entry(pCacheEntry->destIp);
    }
    /* delete from l2table even in case of previous error*/
    IX_ERROR_CR(ix_cc_l2tm_delete_entry(g_pArpContext->hL2tm, (ix_uint16)arg_L2Index));
    return err;
}

/* arp protocol operations */
/**
 * NAME: ix_cc_arp_resolve_l2_addr()
 *
 * DESCRIPTION: This function takes IP packet as an input argument and does one of two things:
 *              1) Looks up ARP cache for matching IP address and 
 *                  if it founds updates L2 table with correct entry end resends packet to the microblocks.
 *                  The situation #1 could happen if ARP cache and L2 Table get out of sync 
 *              2) If entry in ARP cache is not found, creates ARP request packet and broadcasts it over the subnet. 
 *                  Subnet is determined by output port of exception IP packet's meta data
 * @Param:  -   IN - arg_hBuffer -  handle of exception IP packet
 *              OUT - arg_ArpInfo - pointer to the arp_next_hop_info structure. 
 *                      On return it has information about entry in the ARP cache
 *                      This argument should be ignored if function returns an error. 
 *              OUT - arg_RtnPkt - pointer to the buffer handle. Used for the new ARP request packet.
 *    1.Check arguments for null. Return IX_CC_ERROR_NULL in case of error.
 *    2.Extract next hop id from the meta data of input IP packet, return IX_CC_ARP_INVALID_BUFFER in case of error 
 *    3.Use next hop id as L2 index
 *    4.Looku-up L2 table by  L2Index, and extract Next Hop IP address.
 *    5.Lookup ARP cache by Next Hop IP address
 *    6.If entry is found, and entry has hardware address Update L2 table with L2 address
 *    8.Store INPUT buffer handle into OUTPUT parameter and return IX_SUCCESS
 *    9.If entry not found, create ARP request. Since ARP protocol requires to queue  at least one IP packet, INPUT buffer cannot be reused. 
 *    10.Get new ix_buffer from free list. 
 *    11.Get Output port from metadata of IP packet. Lookup port properties from local interface table.
 *    12.Use port hardware and ip address as source hardware and IP address in ARP packet data
 *    13.Set ARP operation field to ARP_REQUEST
 *    14.Set destination hardware address to the broadcast address (FF:FF:FF:FF:FF:FF)
 *    15.Create new entry in ARP cache - set expiration time in 1200 seconds from now, number of tries -1,
 *    16.And store INPUT buffer in the entry's holdBuffer field.
 *    17.Assign new ARP packet buffer handle to the OUTPUT buffer argument
 *    18.Return IX_CC_ERROR_ENTRY_NOT_FOUND.
 *    19.If entry is found and flag indicates that ARP request is in progress, 
 *        number of sent ARP requests are less then 5, and previous ARP request was sent less then a 1 second ego, 
 *        then store INPUT buffer in holdBuffer field only - free the previous buffer from holdBuffer field
 *    20.If previous ARP request was sent > 1 sec. ego and number of sent requests are less then 5, then create new ARP request and update timeout and sent request counter.
 *    21.Assign new ARP packet buffer handle to the OUTPUT buffer argument
 *    22.Return IX_CC_ERROR_ENTRY_NOT_FOUND
 *    23.If entry is found and number of sent requests are more then 5, free input buffer chain. This enrtry will be deleted on next timeout.
 *    24.Return IX_CC_ARP_ERROR_IP_PKT_DROP
 *
 * 
 * @Return: IX_SUCCESS or valid ix_error if code fails ;
 */

ix_error ix_cc_arp_resolve_l2_addr(ix_buffer_handle arg_hBuffer, ix_buffer_handle* arg_pRtnPkt)
{
    ix_error err = IX_SUCCESS;
    ix_cc_arp_cache_entry*  pCacheEntry;
    ix_ether_hdr            etherHdr;
    ix_hw_buffer_meta*      pMeta;
    ix_cc_l2tm_state        state;
    ix_cc_l2tm_entry        l2tmEntry;
    ix_uint16               outPort;    
    ix_uint32               locIp;
    ix_ossl_time_t	    sysTime = {0,0};
    ix_uint16		    l2idx;

    /* argument checking */
#if defined (IX_ARP_DEBUG)
    if (IX_NULL_BUFFER_HANDLE == arg_hBuffer)
    {
        return IX_ERROR_WARNING(IX_CC_ARP_INVALID_HANDLE,("NULL  pkt buffer handle" ));
    }
#endif

    if (arg_pRtnPkt == NULL)
    {
        ix_rm_buffer_free_chain(arg_hBuffer);
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("NULL pkt buffer handle pointer" ));
    }
  
/* get packet meta-data */
    err = ix_rm_buffer_get_meta(arg_hBuffer, (void**) &pMeta);
    if (IX_SUCCESS != err)
    {
        ix_rm_buffer_free_chain(arg_hBuffer);
        return IX_ERROR_WARNING(err, ("Failed to get meta data"));
    }
/* use nhId from the meta to look-up L2 library*/
	l2idx = IX_NTOH16(pMeta->m_NextHopID);
    IX_ERROR_CG(ix_cc_l2tm_get_entry(g_pArpContext->hL2tm,l2idx,
                                    &state,&l2tmEntry), err, l2tm_entry);
	if (state == IX_CC_L2TM_STATE_VALID)
	{
		*arg_pRtnPkt = arg_hBuffer;
		return IX_SUCCESS;
	}
	else 
/* Handle the case if we did not find an entry in the L2TM */
	if (state == IX_CC_L2TM_STATE_IDLE)
	{
	      ix_rm_buffer_free_chain(arg_hBuffer);		
	        return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("No entry found in L2 table, Failed to get meta data"));
	}
    /* get current system time */
	IX_ERROR_CG(ix_ossl_time_get( &sysTime ),err,l2tm_entry); 
	err = _ix_arp_cache_lookup(l2tmEntry.l3Address.ipv4Address,&pCacheEntry);
    if (IX_SUCCESS == err) /* entry is found*/
    {
#if defined (IX_ARP_DEBUG)
		printf("Entry is found");
#endif
        if (pCacheEntry->arpFlgs & IX_ARPF_VALID)
        {
            /* update L2 table and send IP packet out */
            ix_ossl_memcpy(&(l2tmEntry.header.ether), &(pCacheEntry->destEthAddr), sizeof(ix_ether_hdr));
            IX_ERROR_CG(ix_cc_l2tm_update_entry(g_pArpContext->hL2tm,l2idx,
                                        &l2tmEntry), err, l2tm_entry);
            *arg_pRtnPkt = arg_hBuffer; /* store Input Buffer handle in the output handle pointer */

            /* Return IX_SUCCESS for caller to transmit the IP packet */			 
            return(IX_SUCCESS);
        }
		else if (pCacheEntry->arpFlgs & IX_ARPF_INUSE) /* entry in use - at least one request for this IP was sent
													   before
													   */
		{
			/* check if number of tx more then maximum per entry */
			/* compare last time sent for the entry with the current time */
#if defined (IX_ARP_DEBUG)
			printf("pCacheEntry->arpXmit = %ld, pCacheEntry->lastSend.sec = %ld, sysTime.sec = %ld \n",
					pCacheEntry->arpXmit, pCacheEntry->lastSend.sec , sysTime.sec);
#endif
			/*if ((pCacheEntry->arpXmit < IX_ARP_MAX_TX) && (pCacheEntry->lastSend.sec < sysTime.sec))*/
			if (pCacheEntry->lastSend.sec < sysTime.sec)
			{
				if (pCacheEntry->arpXmit < IX_ARP_MAX_TX)
				{
					/* request is ok to send */
					outPort = IX_NTOH16(pMeta->m_OutputPort);
					/* use output port to get local IP and local l2 src address */
					IX_ERROR_CGT(g_pArpContext->pPortLkupCb(g_pArpContext->pCallerContext, 
												(ix_uint32) outPort, &(etherHdr.srcAddr), 
												 &locIp),err,IX_CC_ARP_ERROR_NO_LOCAL_MAC_ADDRESS,
												 IX_ERROR_LEVEL_WARNING,
												 l2tm_entry); 
					/* queue original buffer - free one that already was queued*/
					if (pCacheEntry->pktHolder != IX_NULL_BUFFER_HANDLE)
					{
						ix_rm_buffer_free_chain(pCacheEntry->pktHolder);
					}
					/* assign to the new input buffer */
					pCacheEntry->pktHolder = arg_hBuffer;
					pCacheEntry->arpXmit++;
					pCacheEntry->lastSend.sec = sysTime.sec;
					pCacheEntry->arpFlgs &= ~IX_ARPF_VALID;
					/* create new arp broadcast request - set arg_pRtnPkt there */
					IX_ERROR_CG(_ix_arp_create_broadcast_request(l2tmEntry.l3Address.ipv4Address,locIp, &etherHdr,arg_pRtnPkt, outPort),
									err, cache_entry1);

                    /* Return the error code that caller is expecting for sending the ARP request packet */
                    return(IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND, ("Returning an ARP request packet" )));
				}
				else /* lets wait for 20 seconds before starting transmitting again */
				{
					pCacheEntry->lastSend.sec = sysTime.sec + IX_ARP_XMIT_WAIT;
					pCacheEntry->arpXmit = 0;
					/* queue original buffer - free one that already was queued*/
					if (pCacheEntry->pktHolder != IX_NULL_BUFFER_HANDLE)
					{
						ix_rm_buffer_free_chain(pCacheEntry->pktHolder);
					}
					/* assign to the new input buffer */
					pCacheEntry->pktHolder = arg_hBuffer;
                    /**
                     * Returning IX_CC_ERROR_ENTRY_NOT_FOUND.
                     * To avoid caller from sending the return packet, set the return packet 
                     * pointer to NULL.
                     */
                    *arg_pRtnPkt = IX_NULL_BUFFER_HANDLE;
                    return(IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND, ("No ARP packet returned" )));
				}
			}
            else /* this is the case we are still within the 20 second waiting period */
            {
                /* queue original buffer - free one that already was queued*/
                if (pCacheEntry->pktHolder != IX_NULL_BUFFER_HANDLE)
                {
                    ix_rm_buffer_free_chain(pCacheEntry->pktHolder);
                }
                /* assign to the new input buffer */
                pCacheEntry->pktHolder = arg_hBuffer;

                *arg_pRtnPkt = IX_NULL_BUFFER_HANDLE;
                return(IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND, ("No ARP packet returned" )));
            }
		}
        else
        /* The ARP entry flag is undefined */
        {
            /* Free the input IP packet */					  
            ix_rm_buffer_free_chain(arg_hBuffer);

            _ix_arp_cache_remove_entry(&pCacheEntry);

            *arg_pRtnPkt = IX_NULL_BUFFER_HANDLE;
            return(IX_ERROR_WARNING(IX_CC_ERROR_INTERNAL, ("Detected undefined ARP entry flag" )));
        }
    }
    else if (IX_ERROR_GET_CODE(err) == IX_CC_ERROR_ENTRY_NOT_FOUND) 
    {
#if defined (IX_ARP_DEBUG)		
        printf("New ARP entry \n");
#endif
		/* create ARP request */
        /* get output port from meta data */
        outPort = IX_NTOH16(pMeta->m_OutputPort);
        /* use output port to get local IP and local l2 src address */
        IX_ERROR_CGT(g_pArpContext->pPortLkupCb(g_pArpContext->pCallerContext, 
                                            (ix_uint32) outPort, &(etherHdr.srcAddr), 
                                             &locIp),err,IX_CC_ARP_ERROR_NO_LOCAL_MAC_ADDRESS,
                                             IX_ERROR_LEVEL_WARNING,
                                             l2tm_entry); 
        
        IX_ERROR_CG(_ix_arp_cache_entry_alloc(&pCacheEntry),err, l2tm_entry);

        /* populate rest of entry fields */ 
        /* assign to the new input buffer */
        pCacheEntry->pktHolder = arg_hBuffer;
        pCacheEntry->destIp = l2tmEntry.l3Address.ipv4Address;
        pCacheEntry->l2Idx = IX_NTOH16(pMeta->m_NextHopID);
        pCacheEntry->arpXmit++;
		pCacheEntry->lastSend.sec = sysTime.sec;
        _ix_arp_reset_timer(pCacheEntry,IX_CC_ARP_CACHE_ENTRY_TIMEOUT);
        pCacheEntry->arpFlgs |= IX_ARPF_INUSE;
        pCacheEntry->arpFlgs &= ~IX_ARPF_VALID;
        IX_ERROR_CG(_ix_arp_cache_add(pCacheEntry),err, cache_entry);
        
        /* create new arp broadcast request - set arg_pRtnPkt there */
        IX_ERROR_CG(_ix_arp_create_broadcast_request(l2tmEntry.l3Address.ipv4Address,locIp, &etherHdr,arg_pRtnPkt, outPort),
                        err, cache_entry);

        /* Return the error code that caller is expecting for sending the ARP request packet */
        return(IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND, ("Returning an ARP request packet" )));
    }
	else
	{
#if defined (IX_ARP_DEBUG)
		printf("Unexpected error code %lX\n",IX_ERROR_GET_CODE(err));
#endif
		ix_rm_buffer_free_chain(arg_hBuffer);

        if (*arg_pRtnPkt != (ix_buffer_handle)IX_NULL_BUFFER_HANDLE)
        {
            ix_rm_buffer_free_chain(*arg_pRtnPkt);
            *arg_pRtnPkt = IX_NULL_BUFFER_HANDLE;
        }

        return IX_SUCCESS;
	}
    return err;
cache_entry:
    if (*arg_pRtnPkt != (ix_buffer_handle)IX_NULL_BUFFER_HANDLE)
    {
        ix_rm_buffer_free_chain(*arg_pRtnPkt);
        *arg_pRtnPkt = IX_NULL_BUFFER_HANDLE;
    }

    ix_rm_buffer_free_chain(arg_hBuffer);
    pCacheEntry->pktHolder = IX_NULL_BUFFER_HANDLE;
	_ix_arp_cache_remove_entry(&pCacheEntry);
    return err;

cache_entry1:
    if (*arg_pRtnPkt != (ix_buffer_handle)IX_NULL_BUFFER_HANDLE)
    {
        ix_rm_buffer_free_chain(*arg_pRtnPkt);
        *arg_pRtnPkt = IX_NULL_BUFFER_HANDLE;
    }

    ix_rm_buffer_free_chain(arg_hBuffer);
    pCacheEntry->pktHolder = IX_NULL_BUFFER_HANDLE;
    return err;

l2tm_entry:
    ix_rm_buffer_free_chain(arg_hBuffer);
    return err;
}
/**
 * NAME: ix_cc_arp_process_arp_pkts()
 *
 * DESCRIPTION: This function processes broadcast ARP request and unicast ARP reply packets.
 * @Param:  -   IN - arg_ArpPkt - buffer handle that holds ARP packet
 *				OUT - arg_pRtnPkt - pointer to the buffer handle that is used to store processed packet 
 *              OUT - arg_pPktType - type of the packet that is returned to the caller in arg_pRtnPkt buffer
 *                                  can be ARP_REPLY, ARP_REQUEST, IP packet 
 *
 * @Return: IX_SUCCESS or valid ix_error if code fails ;
 */

ix_error ix_cc_arp_process_arp_pkts(ix_buffer_handle arg_ArpPkt,ix_buffer_handle* arg_pRtnPkt,
                                    ix_uint32* arg_pPktType)
{
    ix_error err = IX_SUCCESS;
    ix_arp_hdr*     pArpHdr;
    ix_uint32       sIp, dIp;
    ix_uint8   *pSrcMacAddr, *pDstMacAddr;
    ix_ether_addr    locAddr;
    ix_uint8* pData = NULL;
    ix_uint8* pArpData;
    ix_hw_buffer_meta *meta;
    ix_uint16 outputPortId;

#if defined (IX_ARP_DEBUG)
    if (IX_NULL_BUFFER_HANDLE == arg_ArpPkt)
    {
        return IX_ERROR_WARNING(IX_CC_ARP_INVALID_HANDLE,("NULL  pkt buffer handle" ));
    }
#endif

    if (arg_pRtnPkt == NULL)
    {
        ix_rm_buffer_free_chain(arg_ArpPkt);
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,("NULL pkt buffer handle pointer" ));
    }

    *arg_pRtnPkt = IX_NULL_BUFFER_HANDLE;

    if (arg_pPktType == NULL)
    {
        ix_rm_buffer_free_chain(arg_ArpPkt);
        return IX_ERROR_WARNING(IX_CC_ERROR_NULL,("NULL pkt type pointer" ));
    }

    /* we assume it is already correct arp pkt */
    /* get packet data */
    IX_ERROR_CT(ix_cc_hw_get_packet_data(arg_ArpPkt, (void**)&pData),err, IX_CC_ERROR_INTERNAL, IX_ERROR_LEVEL_WARNING);
    if (err != IX_SUCCESS)
	    goto label_1;

    /* get source Ethernet and ARP heaer from the packet */

/* Do not need to check if arp, information on the first 14 bytes may be garbage
since microblock does not copy valid data, the arp header has the correct information */

/*****************
    pEtherHdr = (ix_ether_hdr*)pData;
	if (pEtherHdr->protFld != IX_ETH_P_ARP)
	{
		err = IX_ERROR_WARNING(IX_CC_ARP_ERROR_INCORRECT_HEADER,("Not an ARP packet")); 
		goto label_1;
	}
******************/

     pArpHdr = (ix_arp_hdr*)((ix_uint32)(pData + IX_ETH_HLEN));
     pArpData = (ix_uint8*)((ix_uint32)(pData + IX_ETH_HLEN + IX_ARP_HLEN));

    /* process only ARP requests and replies */
    if ((pArpHdr->arOp != IX_HTON16(IX_ARPOP_REPLY)) && (pArpHdr->arOp != IX_HTON16(IX_ARPOP_REQUEST)))
    {
        err = IX_ERROR_WARNING(IX_CC_ARP_ERROR_INCORRECT_HEADER,("ARP operation field is incorrect")); 
        goto label_1;
    }

    /* Extract fields */ 
    pSrcMacAddr = pArpData;
    pArpData += IX_ETH_ALEN;
    ix_ossl_memcpy(&sIp,pArpData,IX_IPV4_ALEN);
    pArpData += IX_IPV4_ALEN;
    pDstMacAddr = pArpData;
    pArpData += IX_ETH_ALEN;
    ix_ossl_memcpy(&dIp,pArpData,IX_IPV4_ALEN);

    /* Check for bad requests - request for 127.x.x.x and multicast addresses */
    if (IX_LOOPBACK(dIp) || IX_MULTICAST(dIp))
    {
        err = IX_ERROR_WARNING(IX_CC_ARP_ERROR_INCORRECT_HEADER,("ARP request for bad IP")); 
        goto label_1;
    }

    /* Get packet meta data */
    IX_ERROR_CG(ix_rm_buffer_get_meta(arg_ArpPkt, (void **)&meta), err, label_1);

    /* Process arp packet */
    /* check if sip is same as local interface IP */
    if (IX_SUCCESS == g_pArpContext->pIpLkupCb(g_pArpContext->pCallerContext,
                                               IX_NTOH32(sIp), &locAddr))
    {
        err = IX_ERROR_WARNING(IX_CC_ARP_ERROR_SIP_CONFLICT,("Duplicate IP address on the network"));
        goto label_1;
    }
    /* check DIP in local IF table and retrieve local MAC Address */

    if (IX_SUCCESS != g_pArpContext->pIpLkupCb(g_pArpContext->pCallerContext,
                                           IX_NTOH32(dIp), &locAddr))
    {
        /* no match - let's drop the packet */
        *arg_pRtnPkt = IX_NULL_BUFFER_HANDLE;
        goto label_1;
    }

	/* We need to learn sender's info since the ARP is destined to us */
	_ix_arp_learn_new_entry(IX_NTOH32(sIp), pSrcMacAddr, locAddr.etherAddr);

	/* check what type of ARP packet we have */
	if (pArpHdr->arOp == IX_HTON16(IX_ARPOP_REQUEST))
	{
		/* we found it - dIp belongs to local IP, lets create a new unicast reply */
		/* Currently unable to reuse since PCI uses freelist 0 and ublock uses freelist 1 */
		/* So free the old buffer														  */
/*		if (arg_ArpPkt != IX_NULL_BUFFER_HANDLE)
		{
			ix_rm_buffer_free_chain(arg_ArpPkt);

		} */
        /* Retrieve input port id */
        outputPortId = IX_NTOH16(meta->m_InputPort);
   
        IX_ERROR_CG(_ix_arp_create_unicast_reply(arg_pRtnPkt, pData, dIp, 
                                                 locAddr.etherAddr, 
                                                 sIp, 
                                                 pSrcMacAddr, 
                                                 outputPortId, 
                                                 (ix_buffer_handle)NULL),
                    err, label_1);

        ix_rm_buffer_free_chain(arg_ArpPkt);

		*arg_pPktType = IX_CC_ARP_REPLY;
		return IX_SUCCESS;            

	}
	else if (pArpHdr->arOp == IX_HTON16(IX_ARPOP_REPLY))
	{
		/* let's update L2 Table and ARP cache */
		/* create new ethernet header */
		ix_cc_arp_cache_entry* pCacheEntry = NULL;

		/* lookup ARP cache */
		err = _ix_arp_cache_lookup(sIp,&pCacheEntry);
		if (NULL != pCacheEntry)
		{
			/* updates to L2Tm and ARP cache is done in learning step */
			/* here we send out only buffered packet */
			if (pCacheEntry->pktHolder != IX_NULL_BUFFER_HANDLE)
			{
				*arg_pRtnPkt = pCacheEntry->pktHolder;
                pCacheEntry->pktHolder = IX_NULL_BUFFER_HANDLE;
				*arg_pPktType = IX_CC_IP;
			}
		}
		/* free original buffer */
		ix_rm_buffer_free_chain(arg_ArpPkt);

		return IX_SUCCESS;            
	}/* reply block */ 
    
	return IX_SUCCESS;            

label_1:
    ix_rm_buffer_free_chain(arg_ArpPkt);

    if ((*arg_pRtnPkt) != IX_NULL_BUFFER_HANDLE)
    {
         ix_rm_buffer_free_chain(*arg_pRtnPkt);
         *arg_pRtnPkt = IX_NULL_BUFFER_HANDLE;
    }
    return err;
}

/**
 * NAME: ix_cc_arp_create_gratuitous_arp()
 *
 * DESCRIPTION: This function broadcast gratuitous ARP packet in response to the changes 
 *              in own interface properties - hardware address or source address. 
 * @Param:      IN - arg_OutputPort - port id of the outgoing interface
 *              OUT - arg_pRtnPkt - pointer to the buffer handle. Gets filled with broadcast ARP 
 *              packet for transmission
 *
 * @Return: IX_SUCCESS or valid ix_error if code fails ;
 */

ix_error ix_cc_arp_create_gratuitous_arp(ix_uint16 arg_OutputPort,
                                        ix_buffer_handle *arg_pRtnPkt)
{
    ix_error err = IX_SUCCESS;
    ix_ether_hdr etherHdr;
    ix_uint32 locIp;
    IX_ERROR_DCHECK_ARG_NULL(2, arg_pRtnPkt);
    
    /** gratuitous arp is the broadcast arp request for own IP address
     *  the fields of ARP packet is filed with src hw address - that of the output port 
     *  dest.hw address =  broadcast, source IP = IP address of output port, dest IP = 
     *  IP address of output port.
     *
     **/
    /* lookup local hw and ip address in interface table */
    IX_ERROR_CRT(g_pArpContext->pPortLkupCb(g_pArpContext->pCallerContext, 
                                            (ix_uint32) arg_OutputPort, &(etherHdr.srcAddr), 
                                             &locIp),IX_CC_ARP_ERROR_NO_LOCAL_MAC_ADDRESS,
                                             IX_ERROR_LEVEL_WARNING);
    IX_ERROR_CR(_ix_arp_create_broadcast_request(locIp,locIp, &etherHdr,arg_pRtnPkt, arg_OutputPort));
    return err;
}

/* event/timeout handling */
 /**
 * NAME: ix_arp_create_event_handler
 *
 * DESCRIPTION: This function is used to create event handler if there is no
 * CCI.This API needs to be implemented by client in LEVEL0 syatem.
 * User needs to call ix_arp_cache_timeout function periodically
 * to check ARP cache for timeouts. 
 * 
 * @Param:  - IN arg_pContext: ARP context
 *
 * 
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 * Error Codes:
 *  
 * IX_CC_ERROR_UNIMPL - not implemented
 * 
 */

ix_error ix_arp_create_event_handler(void* arg_pContext)
{ 

    return IX_ERROR_WARNING(IX_CC_ERROR_UNIMPL,("Time-out Handler for ARP is not implemented\n"));
}/* end _ix_arp_create_event_handler */

 /**
 * NAME: ix_arp_cache_timeout
 *
 * DESCRIPTION: This function is arp event handler - check ARP entry if they are ready to be deleted 
 *              Event handler gets invoked every 5 minutes
 *          .   Each ARP entry gets deleted every 20 minutes;
 * 
 * @Param:  - IN  void* pContext - holds pointer to ARP context
 *
 * @Param:  - OUT none
 *
 * @Return: IX_SUCCESS if successful or a valid error token for failure.
 *
 */
ix_error ix_arp_cache_timeout(void* pContext)
{
    ix_error err = IX_SUCCESS;
	ix_arp_context* pArpContext = (ix_arp_context*)pContext;
	_ix_arp_cache_link* pCacheLink = pArpContext->pCacheHead;
	_ix_arp_cache_link* pLink;
	ix_ossl_time_t sysTime = {0, 0};
	IX_ERROR_CRT(ix_ossl_time_get( &sysTime ),
			IX_CC_ARP_ERROR_EVENT_HANDLER_GET_TIME,IX_ERROR_LEVEL_WARNING);


	/* walk the cache list removing expired entries from the list and from hash */
	/* permanent entries are left in the cache */
	while (pCacheLink)
	{
		pLink = pCacheLink->pNext;
		if ((!IX_OSSL_TIME_LT(sysTime, pCacheLink->pCacheEntry->expTime))
			&& (!(pCacheLink->pCacheEntry->arpFlgs & IX_ARPF_PERM)))
		{ /* entry is good for timeout */
			err = ix_cc_hash_table_delete_entry(pArpContext->hArpHash,pCacheLink->pCacheEntry->destIp);
			if (IX_SUCCESS == err)
			{
                /**
                 * Call the L2tm API to clear the l2 info field.  Note 
                 * that for incomplete L2 entry, there is not a valid l2 info. 
                 * So the return code from L2tm is ignored.
                 */
                ix_cc_l2tm_clear_l2_info(g_pArpContext->hL2tm, pCacheLink->pCacheEntry->l2Idx);

				/* remove queued buffer if entry has one */
				if (pCacheLink->pCacheEntry->pktHolder != IX_NULL_BUFFER_HANDLE)
				{
					ix_rm_buffer_free_chain(pCacheLink->pCacheEntry->pktHolder);
                    pCacheLink->pCacheEntry->pktHolder = IX_NULL_BUFFER_HANDLE;
				}
				_ix_arp_free_entry(&(pCacheLink->pCacheEntry));
				/* delete the link */
				if (pCacheLink->pPrev != NULL)
				{
					pCacheLink->pPrev->pNext = pCacheLink->pNext;
				}
				else /* head of the list */
				{
					pArpContext->pCacheHead = pCacheLink->pNext;
				}

				if (pCacheLink->pNext != NULL)
				{
					pCacheLink->pNext->pPrev = pCacheLink->pPrev;
				}
				ix_ossl_free(pCacheLink);
			}
		}
		pCacheLink = pLink;		
	}
    return IX_SUCCESS;
}



/* arp debugging and management support */
/**
 * NAME: ix_cc_arp_purge()
 *
 * DESCRIPTION: This function deletes all entries from the ARP cache table. 
 *              It marks corresponding entries in L2 table as incomplete , 
 *              
 * @Param:  
 *              NONE
 *
 * @Return: IX_SUCCESS or valid ix_error if code fails ;
 */
ix_error ix_cc_arp_purge(void)
{
    return _ix_arp_cache_purge();
}



/**
 * NAME: ix_cc_arp_dump()
 *
 * DESCRIPTION: Print out content of the ARP cache in the formatted/readeable form form 
 * @Param:  -   NONE
 *
 * @Return: IX_SUCCESS or valid ix_error if code fails ;
 */
ix_error ix_cc_arp_dump(void)
{
    ix_error err = IX_SUCCESS;
	_ix_arp_cache_link* pCacheLink = g_pArpContext->pCacheHead;
	int i = 0;


    while (pCacheLink)
	{
		if(!(i % 8))
		{
			_ix_arp_print_dump_heading();
		}
		_ix_arp_print_cache_entry(pCacheLink->pCacheEntry);
		pCacheLink = pCacheLink->pNext;
		i++;
	}
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        ix_ossl_message_log("\n\n");
#else
	fprintf(stdout,"\n\n");
#endif

    return err;
}





















