/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      ix_cc_arp_cache.c
 *
 * = DESCRIPTION
 *         The file contains functions to perform operations on of Ethernet ARP cache
 *
 * = AUTHOR
 *      Herman Zaks
 *      Herman.Zaks@intel.com
 *
 **/
#define IX_ERROR_FILE_IDENT "$Id: ix_cc_arp_cache.c,v 1.10 2003/07/02 01:46:30 ktseng Exp $"
#include "ix_cc_error.h"
#include "ix_types.h"
#include "ix_error.h"
#include "ix_rm.h"

#if !defined(IX_EXCLUDE_CCI)
    #include "ix_cci.h"
#endif /* end !defined(IX_EXCLUDE_CCI) */

#include "ix_cc.h"
#include "bindings.h"
#include "ix_netmacros.h"
#include "ix_cc_properties.h"
#include "ix_rm.h"
#include "ix_ossl.h"

#include "cc/ix_cc_hash_table.h"
#include "cc/ix_cc_l2tm.h"
#include "cc/ix_cc_arp.h"
#include "cc/internal/internal_arp.h"

/**
 * Variable declarations global to this file only.  Extern are followed
 * by static variables.
 */
extern ix_arp_context* g_pArpContext;
 /**
 * Function definitions.
 */

ix_error _ix_arp_cache_lookup(ix_uint32 arg_NextHopIp,ix_cc_arp_cache_entry** arg_ppCacheEntry)
{
	/* lookup in the hash table */
	return ix_cc_hash_table_lookup(g_pArpContext->hArpHash, arg_NextHopIp, NULL,(void**) arg_ppCacheEntry);
}

/**
*	_ix_arp_cache_add()
*	adds pointer to data to the hash table
*	adds cache entry into the beggining of link list, 
*	allocating new cache_link structure  
**/

ix_error _ix_arp_cache_add(ix_cc_arp_cache_entry* arg_pCacheEntry)
{
    ix_error err = IX_SUCCESS;
	_ix_arp_cache_link* pCacheLink = NULL;
	/* add entry to the hash table*/
	IX_ERROR_CR(ix_cc_hash_table_add_entry(g_pArpContext->hArpHash,arg_pCacheEntry->destIp,arg_pCacheEntry));
	/* add to the head of the link list */
	pCacheLink = ix_ossl_malloc(sizeof(_ix_arp_cache_link));
	if (NULL == pCacheLink)
	{
		ix_cc_hash_table_delete_entry(g_pArpContext->hArpHash,arg_pCacheEntry->destIp);
		return IX_ERROR_NEW(IX_CC_ERROR_OOM_SYSTEM,IX_ERROR_LEVEL_WARNING);
	}
	/* add entry to the link */
	pCacheLink->pCacheEntry = arg_pCacheEntry;
	pCacheLink->pNext = g_pArpContext->pCacheHead;
	pCacheLink->pPrev = NULL;
    if(g_pArpContext->pCacheHead != 0)
        g_pArpContext->pCacheHead->pPrev = pCacheLink;
	g_pArpContext->pCacheHead = pCacheLink;
    return err;
}

ix_error _ix_arp_cache_entry_alloc(ix_cc_arp_cache_entry** arg_ppCacheEntry)
{
	/* for now do ix_ossl_malloc for cache entry */
	/* we'll link and do hash in the add call */
	ix_cc_arp_cache_entry* pCacheEntry = NULL;
	pCacheEntry = ix_ossl_malloc(sizeof(ix_cc_arp_cache_entry));
	if (NULL == pCacheEntry)
	{
		return IX_ERROR_NEW(IX_CC_ERROR_OOM_SYSTEM,IX_ERROR_LEVEL_WARNING);
	}
	/* zero out the entry */
	ix_ossl_memset(pCacheEntry,0,sizeof(ix_cc_arp_cache_entry));
	*arg_ppCacheEntry = pCacheEntry;
    return IX_SUCCESS;
}

/** 
 *	Given the pointer to pointer to the cache entry we free the memory allocated by the entry
 *  and set pointer to NULL 
**/ 
void _ix_arp_free_entry(ix_cc_arp_cache_entry** arg_ppCacheEntry)
{
	ix_ossl_free(*arg_ppCacheEntry);
	*arg_ppCacheEntry = NULL;
}

/** 
 *	Given the pointer to the cache entry we free the memory allocated by the entry
 *  ,set entry pointer to NULL, remove corresponding link from cache link and remove hash entry
 *  it is expected that caller will set L2 index entry in l2 table to the incomplete state, 
 *  if necessary, before caling this function
**/ 

ix_error _ix_arp_cache_remove_entry(ix_cc_arp_cache_entry** arg_ppCacheEntry)
{

	ix_error err = IX_SUCCESS;
	_ix_arp_cache_link* pCacheLink = g_pArpContext->pCacheHead;
	ix_uint8 found = 0;
/* validate args */
    IX_ERROR_DCHECK_ARG_NULL(1, arg_ppCacheEntry);

/* #ifdef IX_ARP_DEBUG */
	if (NULL == g_pArpContext->pCacheHead )
	{
		/* arp cache is empty - let's just free the entry */
        /* remove queued buffer if entry has one */
		if ((*arg_ppCacheEntry)->pktHolder != IX_NULL_BUFFER_HANDLE)
		{
			ix_rm_buffer_free_chain((*arg_ppCacheEntry)->pktHolder);
		}
		_ix_arp_free_entry(arg_ppCacheEntry);
		return IX_ERROR_WARNING(IX_CC_ERROR_NULL, ("ARP cache is empty"));
	}
/* #endif */
	
	while (pCacheLink)
	{
		if (*arg_ppCacheEntry == pCacheLink->pCacheEntry) /* found the entry */
		{
			found = 1;
			if (pCacheLink->pPrev != NULL)
			{
				pCacheLink->pPrev->pNext = pCacheLink->pNext;
			}
			else /* head of the list */
			{
				g_pArpContext->pCacheHead = pCacheLink->pNext;
			}

			if (pCacheLink->pNext != NULL)
			{
				pCacheLink->pNext->pPrev = pCacheLink->pPrev;
			}
			ix_cc_hash_table_delete_entry(g_pArpContext->hArpHash,(*arg_ppCacheEntry)->destIp);

            /* remove queued buffer if entry has one */
		    if ((*arg_ppCacheEntry)->pktHolder != IX_NULL_BUFFER_HANDLE)
		    {
			    ix_rm_buffer_free_chain((*arg_ppCacheEntry)->pktHolder);
		    }
			_ix_arp_free_entry(arg_ppCacheEntry);
			ix_ossl_free(pCacheLink);
			break;
		}
		pCacheLink = pCacheLink->pNext;
	}
	if (!found)
	{
		err = IX_ERROR_NEW(IX_CC_ERROR_ENTRY_NOT_FOUND, IX_ERROR_LEVEL_WARNING);
	}
	return err;
}

ix_error _ix_arp_cache_l2idx_find(ix_uint16 arg_L2Index,ix_cc_arp_cache_entry** arg_ppCacheEntry)
{
    ix_error err = IX_SUCCESS;
	_ix_arp_cache_link* pCacheLink = g_pArpContext->pCacheHead;
	ix_uint8 found = 0;
/* walk through the link list*/ 
	while (pCacheLink)
	{
		if (pCacheLink->pCacheEntry->l2Idx == arg_L2Index)
		{
			*arg_ppCacheEntry = pCacheLink->pCacheEntry;
			found = 1;
			break;
		}
		pCacheLink = pCacheLink->pNext;
	}
	if (!found)
	{
		err = IX_ERROR_WARNING(IX_CC_ERROR_ENTRY_NOT_FOUND, 
				("Could not find an element with the l2 Index 0x%08X", arg_L2Index));
	}
    return err;
}

ix_error _ix_arp_cache_del_entry(ix_uint32 arg_Ip)
{
    ix_error err = IX_SUCCESS;
	ix_cc_arp_cache_entry* pCacheEntry = NULL;
	/* lookup entry in the hash */ 
	IX_ERROR_CR(ix_cc_hash_table_lookup(g_pArpContext->hArpHash, arg_Ip, NULL,(void**) &pCacheEntry));
	if (NULL == pCacheEntry)
	{
		return IX_ERROR_NEW(IX_CC_ERROR_ENTRY_NOT_FOUND, IX_ERROR_LEVEL_WARNING);
	}
	/* to make sure we removing correct entry from the hash */
	pCacheEntry->destIp = arg_Ip;
	IX_ERROR_CR(_ix_arp_cache_remove_entry(&pCacheEntry));
    return err;
}

ix_error _ix_arp_cache_purge()
{
    ix_error err = IX_SUCCESS;
	_ix_arp_cache_link* pCacheLink = g_pArpContext->pCacheHead;	
	/* walk through the list,starting from the head, deleting entries and links, and cleaning hash table 
	 */
	while (pCacheLink)
	{
		IX_ERROR_CR(ix_cc_hash_table_delete_entry(g_pArpContext->hArpHash,pCacheLink->pCacheEntry->destIp));
		/* remove queued buffer if entry has one */
		if (pCacheLink->pCacheEntry->pktHolder != IX_NULL_BUFFER_HANDLE)
		{
			ix_rm_buffer_free_chain(pCacheLink->pCacheEntry->pktHolder);
		}
        /* mark l2 table entry as incomplete */
        ix_cc_l2tm_clear_l2_info(g_pArpContext->hL2tm,(ix_uint16)pCacheLink->pCacheEntry->l2Idx);

		_ix_arp_free_entry(&(pCacheLink->pCacheEntry));
		pCacheLink = pCacheLink->pNext;
		ix_ossl_free(g_pArpContext->pCacheHead);
		g_pArpContext->pCacheHead = pCacheLink;
	}
	g_pArpContext->pCacheHead = NULL;
    return err;
}


