/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = LIBRARY
 *      Fragmentation Library
 *
 * = MODULE
 *      Library Interface
 *
 * = FILENAME
 *      checksum.h
 *
 * = DESCRIPTION
 *      Functions provided for users to manipulate checksum in IP4 IP, TCP, UDP
 *   headers
 *
 * = AUTHOR
 *      Ranjeeta Singh
 *      ranjeeta@intel.com
 *
 * = AKNOWLEDGEMENTS
 *      
 *
 * = CREATION TIME
 *      07/12/2002 8:53:42 PM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: checksum.h,v 1.7 2002/12/13 19:45:04 rjpetri Exp $
 */

#if !defined(__CHECKSUM_H__)
#define __CHECKSUM_H__


   

#include "ix_cc_error.h"
#include "ip4hdr.h"


#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

/**
 * Prototypes for interface functions.
 */

/** Calculate sum -- just return 32-bit accumulator of a number of
                             16-bit integers
    
    @param      arg_ptr	pointer to to the first 16-bit of data stream in 
			Network Byte Order
    @param      arg_len	length of data to be summed up ( measured in octets)
    @param      arg_pSum 	where to return the 32-bit sum
    @returns an error token, IX_SUCCESS for success.
*/
IX_EXPORT_FUNCTION
ix_error ix_cc_fragv4_checksum_sum32( ix_uint16 *arg_pPtr, ix_uint32 arg_Len,
 					ix_uint32 *arg_pSum );

/** Calculate sum -- return 16-bit 1's compliment sum of a number of
                             16-bit integers of data
    
    @param      arg_ptr	pointer to to the first 16-bit of data stream in 
			Network Byte Order
    @param      arg_len	length of data to be summed up ( measured in octets)
    @param      arg_sum 	where to return the 16-bit sum
    @returns an error token, IX_SUCCESS for success.
*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_checksum_compliment_sum16( ix_uint16 *arg_pPtr,
                                        ix_uint32 arg_Len, 
				  ix_uint16 *arg_pSum);




/** Calculate 1's compliment checksum for the basic 20-octet ip4 header
    
    @param     arg_pPtr	pointer to to the ipv4 header
    @param     arg_pChksum where to return the 16-bit 1's complement checksum
    @param     arg_Replace whether to replace the old checksum with the newly
                        calulated checksum.
                        1:  replace old checksum
                        0:  old checksum unchanged

    @returns an error token, IX_SUCCESS for success.
*/

IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_checksum_ip4hsum( ix_cc_fragv4_ip4hdr *arg_pPtr,
                                  ix_uint16 *arg_pChksum,
				 ix_uint32 arg_Replace);



  
#ifdef  __cplusplus
};
#endif

#endif/*__CHECKSUM_H__*/
