/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor, Release 4
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      IPv4 Forwarder Core Component Fragmentation support
 *
 * = FILENAME
 *      ip4hdr.h
 *
 * = DESCRIPTION
 *   This file contains definition of IPv4
 *   helper functions     
 *
 *
 *      
 *
 * = CREATION TIME
 *      7/18/2002 9:18:42 PM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: ip4hdr.h,v 1.8 2002/12/13 19:45:04 rjpetri Exp $
 */
#if !defined(__IP4HDR_H__)
#define __IP4HDR_H__

#include <stdio.h>
#include <ix_ossl.h>
#include <ix_rm.h>
#include "ix_cc_error.h"
#include "ix_netmacros.h"

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */
/** This file is for fragmentation support from TCP/IP extension**/
/** See if you can move definitions to header_defs.h ***/
/**
 * Preprocessor symbol and macro definitions.
 */
/* do not fragment flag, RFC 791 p 13 */
/* These values are defined in Big Endian order */
#define IX_CC_FRAGV4_IP_HEADER_FLAG_DF		0x4000
/* more fragment flag, RFC 791 p 13 */
#define IX_CC_FRAGV4_IP_HEADER_FLAG_MF		0x2000

#define IX_CC_FRAGV4_IP_HEADER_FRAG_OFF_MASK	0x1fff
#define IX_CC_FRAGV4_IP_HEADER_MIN_SIZE	0x14

/*
 * Definitions for options.
 * All defs are from RFC 791, p15-23
 */
#define IX_CC_FRAGV4_IP_HEADER_OPT_COPY(o)           ((o)&0x80)
#define IX_CC_FRAGV4_IP_HEADER_OPT_CLASS(o)          ((o)&0x60)
#define IX_CC_FRAGV4_IP_HEADER_OPT_NUMBER(o)         ((o)&0x1f)

#define IX_CC_FRAGV4_IP_HEADER_OPT_CONTROL           0x00
#define IX_CC_FRAGV4_IP_HEADER_OPT_RESERVED1         0x20
#define IX_CC_FRAGV4_IP_HEADER_OPT_DEBMEAS           0x40
#define IX_CC_FRAGV4_IP_HEADER_OPT_RESERVED2         0x60



#define IX_CC_FRAGV4_IP_HEADER_OPT_EOL               0   /* end of option list */
#define IX_CC_FRAGV4_IP_HEADER_OPT_NOP               1   /* no operation */

 /** Offset of each field (in bytes) from the beginning of IPv4 header

*/
#define IX_CC_FRAGV4_IP_HEADER_VHL		0
#define IX_CC_FRAGV4_IP_HEADER_TOS		1
#define IX_CC_FRAGV4_IP_HEADER_LEN		2
#define IX_CC_FRAGV4_IP_HEADER_ID		4
#define IX_CC_FRAGV4_IP_HEADER_FRAG_OFF	6
#define IX_CC_FRAGV4_IP_HEADER_TTL		8
#define IX_CC_FRAGV4_IP_HEADER_PROTO		9
#define IX_CC_FRAGV4_IP_HEADER_SUM		10
#define IX_CC_FRAGV4_IP_HEADER_SRC		12
#define IX_CC_FRAGV4_IP_HEADER_DEST		16

/**
 * TYPENAME: ix_cc_fragv4_error_code
 * 
 * DESCRIPTION: This enumerated type will describe ipv4 error codes
 *
 */

typedef enum ix_e_cc_fragv4_error_code {

    IX_CC_FRAGV4_ERROR_FIRST = IX_ERROR_MAKE_GROUP(CC_FRAGV4),   
    IX_CC_FRAGV4_ERROR_IP4HDR_READ,
    IX_CC_FRAGV4_ERROR_IP4HDR_WRITE,
    IX_CC_FRAGV4_ERROR_CHECKSUM,
    IX_CC_FRAGV4_ERROR_IP_FRAGMENT
       
    
}ix_cc_fragv4_error_code;



/**
 * Type definitions.
 */


struct ix_s_cc_fragv4_ip4_header;
typedef struct ix_s_cc_ipv4_ip4_header ix_cc_fragv4_ip4hdr;


/**
 * Prototypes for interface functions.
 */
/** Get pointers to the beginning of IP header inside the given buffer

    from a given buffer
    @param      arg_hBuffer          buffer of interest
    @param      arg_ppIPHdr        where to return pointer to beginning of IP 
				header
    @returns    nothing
*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_hdr_get_ip_header_ptr(ix_buffer_handle arg_hBuffer,
					 ix_cc_fragv4_ip4hdr **arg_ppIPHdr); 


/** Print IP4 header field ( for debugging )
    @param      arg_pHeader         pointer to beginning of header
    @returns    nothing
*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_ip4hdr_print(ix_cc_fragv4_ip4hdr *arg_pHeader);

/** Read version number in IPv4 header
    
    @param      arg_pHeader	pointer to beginning of header
    @param      arg_pVersion		where to return IP version number
    @returns an error token, IX_SUCCESS for success.
*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_ip4hdr_version_read(ix_cc_fragv4_ip4hdr *arg_pHeader, ix_uint8 *arg_pVersion);


/** Read header length in IPv4 header
    
    @param      arg_pHeader	pointer to beginning of header
    @param      arg_pHlen		where to return header length in 32-bit words
    @returns an error token, IX_SUCCESS for success.
*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_ip4hdr_read_hlen_in_w32(ix_cc_fragv4_ip4hdr *arg_pHeader,
                                    ix_uint8 *arg_pHlen);

/** Read header length in IPv4 header
    
    @param   arg_pHeader	pointer to beginning of header
    @param   arg_pHlen	where to return header length in the number of octets
    @returns an error token, IX_SUCCESS for success.
*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_ip4hdr_read_hlen_in_octets(ix_cc_fragv4_ip4hdr *arg_pHeader, 
		                               ix_uint16 *arg_pHlen);

/** Write to header length field in IPv4 header
    
    @param      arg_pHeader	pointer to beginning of header
    @param      arg_hLen		header length in 32-bit words
    @returns an error token, IX_SUCCESS for success.
*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_ip4hdr_hlen_write(ix_cc_fragv4_ip4hdr *arg_pHeader,
                                    ix_uint8 arg_hLen);



/** Read total length of datagram in IPv4 header
    
    @param      arg_pHeader	pointer to beginning of header
    @param      arg_pLength	where to return total length of datagram
    @returns an error token, IX_SUCCESS for success.
*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_ip4hdr_len_read(ix_cc_fragv4_ip4hdr *arg_pHeader,
                                      ix_uint16 *arg_pLength); 

/** Assign total length of datagram in IPv4 header
    
    @param      arg_pHeader	pointer to beginning of header
    @param      arg_length total length of datagram
    @returns an error token, zero for success.
*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_ip4hdr_len_write(ix_cc_fragv4_ip4hdr *arg_pHeader,
                                  ix_uint16 arg_length); 

/** Read identification of datagram in IPv4 header
    
    @param      arg_pHeader	pointer to beginning of header
    @param      arg_pId	where to return identification of datagram
    @returns an error token, zero for success.
*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_ip4hdr_id(ix_cc_fragv4_ip4hdr *arg_pHeader,
                              ix_uint16 *arg_pId); 

/**  Read type of service in IPv4 header
    
    @param      arg_pHeader	pointer to beginning of header
    @param      	where to return IP type of service
    @returns an error token, zero for success.
*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_hdr_tos(ix_cc_fragv4_ip4hdr *arg_pHeaderp,
                              ix_uint8 *arg_pTos);

/** Read fragment offset and flags field in IPv4 header
    
    @param      arg_pHeader	pointer to beginning of header
    @param      arg_pOff 	where to return fragment offset and flags field
    @returns an error token, zero for success.
*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_ip4hdr_frag_off_read(ix_cc_fragv4_ip4hdr *arg_pHeader,
                                      ix_uint16 *arg_pOff); 

/** Write fragment offset and flags field in IPv4 header
    
    @param      arg_pHeader	pointer to beginning of header
    @param      arg_off 	offset and flags field of datagram
    @returns an error token, zero for success.
*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_ip4hdr_frag_off_write(ix_cc_fragv4_ip4hdr *arg_pHeader,
                               ix_uint16 arg_off); 

/** Read time to live in IPv4 header
    
    @param      arg_pHeaderp	pointer to beginning of header
    @param      arg_pTtl	where to return time to live
    @returns an error token, zero for success.
*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_ip4hdr_ttl(ix_cc_fragv4_ip4hdr *arg_pHeader,
                               ix_uint8 *arg_pTtl);


/** Read protocol in IPv4 header
    
    @param      arg_pHeader	pointer to beginning of header
    @param      arg_pProto	where to return protocol 
    @returns an error token, IX_SUCCESS for success.
*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_ip4hdr_proto(ix_cc_fragv4_ip4hdr *arg_pHeader, 
                                 ix_uint8 *arg_pProto);



/** Read checksum of datagram in IPv4 header
    
    @param      arg_pHeader	pointer to beginning of header
    @param      arg_pChecksum	where to return checksum of datagram
    @returns an error token, IX_SUCCESS for success.
*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_ip4hdr_checksum_read(ix_cc_fragv4_ip4hdr *arg_pHeaderp,
                                          ix_uint16 *arg_pChecksum); 

/** Change header checksum in IPv4 header
    
    @param      arg_pHeader	pointer to beginning of header
    @param      arg_checksum	checksum of datagram
    @returns an error token, or IX_SUCCESS for success.
*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_ip4hdr_checksum_write( ix_cc_fragv4_ip4hdr *arg_pHeader,
                                           ix_uint16 arg_checksum); 



/** Read 32-bit source address in IPv4 header
    
    @param      arg_pHeader	pointer to beginning of header
    @param      arg_pSrc 	where to return source address of datagram
    @returns an error token, IX_SUCCESS for success.
*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_ip4hdr_src_32(ix_cc_fragv4_ip4hdr *arg_pHeader,
                                  ix_uint32 *arg_pSrc); 



/** Read 32-bit dest address in IPv4 header
    
    @param      arg_pHeader	pointer to beginning of header
    @param      arg_pDest 	where to return dest address of datagram
    @returns an error token,or IX_SUCCESS for success.
*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_ip4hdr_dest_32(ix_cc_fragv4_ip4hdr *arg_pHeader, ix_uint32 *arg_pDest); 


/** calulate the length of data only ( in octets ) in IPv4 datagram
    
    @param    arg_pHeader	pointer to beginning of header
    @param    arg_pDataLen where to return length of data in the number of octets
    @returns an error token, IX_SUCCESS for success.
*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_ip4hdr_data_len(ix_cc_fragv4_ip4hdr *arg_pHeader,
                                    ix_uint16 *arg_pDataLen); 

/** find out where the payload starts
    
    @param      arg_pHeader	pointer to beginning of header
    @param      arg_ppData   where to return the pointer to the first word of
                        payload 
    @returns an error token, or IX_SUCCESS for success.
*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_ip4hdr_payload(ix_cc_fragv4_ip4hdr *arg_pHeader, ix_uint16 **arg_ppData); 


/**  Copy the option part of a ip4 header to the header in a fragment

     Assumption: Destination header belongs to a fragment.

     Only appropriate fields are copied from source header to destination
     header. Users need to pass information via 'isFirst' to specify whether
     the destination header belongs to a fragment that is first in a datagram
     ( first fragment header may carries more options information ).

    @param      arg_pSrc    pointer to beginning of the source header
    @param      arg_pDest   pointer to beginning of the destination header
    @param      arg_isFirst   1: the destination fragment is the first fragment
                             of a datagram
                          0: the destination fragment is NOT the first fragment
                             of a datagram
    @param      arg_pdestHLen  where to return the header length of the 
                destination header
    @returns an error token, zero for success.

*/
IX_EXPORT_FUNCTION 
ix_error ix_cc_fragv4_ip4hdr_copy_options(ix_cc_fragv4_ip4hdr *arg_pSrc,
                             ix_cc_fragv4_ip4hdr* arg_pDest,
                             ix_int32 arg_isFirst, ix_uint16 *arg_pDestHLen);






/**
 * Exported variables.
 */

#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__IP4HDR_H__) */
