/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2003 Intel Corporation. All rights reserved.

 * = PRODUCT
 *      Intel(r) IXA SDK 3.1 for the IXP2000 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      IPv4 Forwarder Core Component Fragmentation Support
 *
 * = FILENAME
 *      fragment.c
 *
 * = DESCRIPTION
 *   This file contains implementation of fragmentation routines      
 *
 *
 *      
 *
 * = CREATION TIME
 *      7/22/2002 9:06:24 PM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */

#define IX_ERROR_FILE_IDENT "$Id: fragment.c,v 1.46 2003/12/12 17:58:31 ttschlue Exp $"
/**
 * System defined include files required.
 */

#include "ix_ossl.h"
#include "ix_cc_error.h"
#include "ix_cc.h"
/**
 * User defined include files required.
 */
#include "cc/fragmentation/ip4hdr.h"
#include "cc/fragmentation/fragment.h"

/**
 * Preprocessor symbols and macros used in this file.
 */
#define IX_CC_FRAGV4_BUFFER_SIZE 0x800
/**
 * Types definitions whose scope is limited to this file.
 */

/**
 * Variable declarations global to this file only.  Externs are followed by
 * static variables.
 */

/**
 * Extern function prototypes.
 */

/**
* Static function prototypes.
 */

/**
 * Function definitions.
 */
ix_error ix_cc_fragv4_fragment_create(ix_uint32 arg_FragSize,
                    ix_uint8 *arg_pData, 
                    ix_hw_buffer_meta *arg_pMeta, 
                    ix_uint16 arg_flagsAndOff, ix_uint16 arg_DataOff,
                    ix_uint16 arg_Length, ix_uint32 arg_IsFirst,
        		    ix_buffer_free_list_handle arg_hHwFreeList,                    
                    ix_cc_fragv4_fragment **arg_ppNewFrag)

{

    
    ix_cc_fragv4_fragment *frag = 0;
    ix_uint16    hLen, fragHLen, sum; 
    ix_uint8    *tempPtr;
    ix_uint8 *dataP;
    void *pTemp;
    
    ix_uint8 *dest;
    ix_uint8 *pDest;
    ix_uint32 pOff;
    ix_cc_fragv4_ip4hdr *hptr;
    ix_hw_buffer_meta *pMetaFirstFrag;
    ix_hw_buffer_meta *pMetaLinkedBuffers;
    ix_hw_buffer_meta *pMeta;
    ix_uint32 count=0;
    ix_uint16 LengthData;    
    ix_uint32 cell_count =0;
    void *pDataInLinkedBuffers;
    ix_uint16 val = IX_CC_FRAGV4_BUFFER_SIZE;
    ix_uint16 payloadInFirstBuffer;
    ix_uint16 dataLeftToCopy; 
    int k=0;    
	ix_error err = IX_SUCCESS;
#if IX_DEBUG    
    if (arg_ppNewFrag == NULL)
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Pointer to new fragment is null\n")); 
    if ((arg_FragSize < IX_CC_FRAGV4_IP_FRAG_MIN_SIZE) ||
     (arg_FragSize > IX_CC_FRAGV4_IP_FRAG_MAX_SIZE))
        return IX_ERROR_LOCAL(IX_CC_ERROR_RANGE,
                            ("Invalid range for arg_FragSize\n"));
    if ((arg_Length < 1) || 
    (arg_Length > (IX_CC_FRAGV4_IP_FRAG_MAX_SIZE - IX_CC_FRAGV4_IP_HEADER_MIN_SIZE)))
        return IX_ERROR_LOCAL(IX_CC_ERROR_RANGE,
                            ("Invalid range for arg_Length"));
#endif
    
    hptr = (ix_cc_fragv4_ip4hdr *)arg_pData;

    frag = ix_ossl_malloc(sizeof(ix_cc_fragv4_fragment));

    

    if (NULL==frag)
    {
        return IX_ERROR_LOCAL(IX_CC_FRAGV4_ERROR_IP_FRAGMENT, ("Null fragment"));
                
    }

    frag->pPrev = frag->pNext = NULL;
    frag->dataLen = arg_Length;
    frag->flagsAndOff = arg_flagsAndOff;
     
    /* fragSize is  used now because we assume all fragments cannot fit in
       1 buffer size 2K. We have the ability to link buffers and allow,
       bigger fragment- fragSize will be used to decide number of buffers
       needed
     */
    /**  Check fragSize.FragSize is in octets
     **   If it is greater than 2K, then buffers have to be linked 
     **/
    /** If buffer chaining is required, then I need to copy this buffer 
    onto multiple buffers **/
    pOff = 0;

    /** Check if fragSize which is in octets is greater than 2k bytes 
    ie 2048 octets **/
    /** If yes, then whole of IP hdr + payload cannot fit in single buffer
    ** and we need to chain multiple buffers **/
    /** Right now, I am assuming that buffers in freelist are of 2K size **/

    if (arg_FragSize > ((ix_uint32)IX_CC_FRAGV4_BUFFER_SIZE - 128))
    {
		
        /** We need to create as many buffer handles based on arg_FragSize **/
        /** arg_hHwFreeList is a handle to hardware freelist 
         **/
        	        
        ix_buffer_handle hHwBuffer[10];
                
        ix_uint32 totalNoOfBuffersNeeded= 0;
 
       
        IX_ERROR_CGT(ix_cc_fragv4_ip4hdr_read_hlen_in_octets(hptr, &hLen), err,
                    IX_CC_FRAGV4_ERROR_IP_FRAGMENT, 0, clean_up_label);
		
		
        if (hLen > IX_CC_FRAGV4_IP_HEADER_MIN_SIZE)
        {
            IX_ERROR_CRT(ix_cc_fragv4_ip4hdr_copy_options(hptr,
               frag->pHeader, arg_IsFirst, &fragHLen), IX_CC_FRAGV4_ERROR_IP_FRAGMENT, 0);
            fragHLen = IX_NTOH16(fragHLen);
	   
        }
        else
        {
            fragHLen = IX_NTOH16(hLen);
        }       
        
        totalNoOfBuffersNeeded=
        1+ (arg_Length+fragHLen + 127)/IX_CC_FRAGV4_BUFFER_SIZE;
                           
        frag->bufferChaining = 1;
      
        
        for (count = 0; count < totalNoOfBuffersNeeded; count++)
        {
            hHwBuffer[count] = IX_NULL_BUFFER_HANDLE;
            err = ix_rm_buffer_alloc(arg_hHwFreeList, &hHwBuffer[count]);
			if (err != IX_SUCCESS)
			{
				for (k=0;k<count;k++)
				{
					ix_rm_buffer_free(hHwBuffer[k]);
				}
				goto clean_up_label;
			}
           
            
        }
		
                
        /** copy meta data from original buffer into first buffer
         in linked list 
         **/
        /** Note: copy meta data info. of original packet to first fragment **/
        /** But each fragment should have its buffer size in meta **/
        IX_ERROR_CGT(ix_rm_buffer_get_meta(hHwBuffer[0],
         (void **)&pMetaFirstFrag), err,
                    IX_CC_ERROR_INTERNAL, 0, clean_up_label);
        /*Copy the whole meta structure so you will never
         * miss anything and there will be no need to change this when the
         * structure layout changes.
         */
        *pMetaFirstFrag = *arg_pMeta;
        /* Set offset to 128 for first buffer in chain */
        pMetaFirstFrag->m_Offset = 128;
        /* Now copy data from original buffer into these linked buffers **/
        /** I need tempPtr so I copy first into first buffer */
        IX_ERROR_CGT(ix_cc_hw_get_packet_data(hHwBuffer[0],
         &pDataInLinkedBuffers), err,
                    IX_CC_ERROR_INTERNAL, 0, clean_up_label);
        
        
        tempPtr = (ix_uint8 *)pDataInLinkedBuffers;
        /* First let me copy IX_CC_FRAGV4_IP_HEADER_MIN_SIZE of header onto first buffer **/
        /** I can copy upto 2K perhaps in first buffer or whatever size is 
        supported in free list**/
        ix_ossl_memcpy(pDataInLinkedBuffers, arg_pData,
             IX_CC_FRAGV4_IP_HEADER_MIN_SIZE );

        frag->pHeader = (ix_cc_fragv4_ip4hdr*) tempPtr;

        /* Set the packetsize for this fragment */
        pMetaFirstFrag->m_PacketSize = IX_HTON16((ix_uint16)(fragHLen + arg_Length));
         
         /* now add data */
        dataP = arg_pData + hLen + arg_DataOff;

        /* Add rest of data from original this_buff **/
        /* We have to calculate amount of data that would fit in this
         buffer - first buffer in the chain **/
         if ((arg_Length + fragHLen) < (IX_CC_FRAGV4_BUFFER_SIZE - 128))
        {
          payloadInFirstBuffer = arg_Length;
        }
        else
        {
            payloadInFirstBuffer = IX_CC_FRAGV4_BUFFER_SIZE - hLen - 128;
        }
       
        /*New offset where payload needs to be copied is at the offset
        from minimum IP header length + options if it is there */
        pDest = tempPtr + fragHLen;
        ix_ossl_memcpy(pDest, (void *)(dataP),
                     payloadInFirstBuffer);

        /* Update buffer size in meta data to the size of first fragment
         hdr+payload **/
        pMetaFirstFrag->m_BufferSize = IX_HTON16((ix_uint16)(fragHLen + payloadInFirstBuffer));
        LengthData = pMetaFirstFrag->m_BufferSize;     
        /* Set cellcount field in buffer */
        cell_count = ix_cc_calculate_cell_count((ix_uint32)LengthData, 1);
                
                    
        /* Write the cell count into the buffer handle */
		IX_ERROR_CRT(ix_rm_buffer_cell_count_set(&hHwBuffer[0], cell_count),
          IX_CC_ERROR_INTERNAL, 0);

        /* Calculate amount of payload left to copy in rest of fragments
         in the chain **/
        dataLeftToCopy = arg_Length - payloadInFirstBuffer;
        
        if (dataLeftToCopy <= IX_CC_FRAGV4_BUFFER_SIZE)
        {
            if (dataLeftToCopy == 0)
            {
                val =0;
                    
            }
            else
            {
                val = dataLeftToCopy;
                dataLeftToCopy = 0;
            }
        }
        else
        {
            val = IX_CC_FRAGV4_BUFFER_SIZE;
                
        }               
                
        dataP +=  payloadInFirstBuffer;
       
        for (count = 1, pDataInLinkedBuffers = NULL,
            pMetaLinkedBuffers = NULL, 
			pDataInLinkedBuffers = NULL; count < totalNoOfBuffersNeeded; count ++)
        {                                    
            
            /** we should first copy meta from thisBuff 
            to this buffer_handle***/
            /** Then we should copy data from thisBuff to this buffer_handle **/
            /* Copy minimum header in this buffer_handle*/
            
            IX_ERROR_CGT(ix_rm_buffer_get_data(hHwBuffer[count],
             &pDataInLinkedBuffers), err,
                             IX_CC_ERROR_INTERNAL, 0, clean_up_label);
            
            /* Copy 2K bytes and we can copy rest into other buffers which 
            will be linked **/
            
            ix_ossl_memcpy(pDataInLinkedBuffers, (void *)dataP, val);
            dataP = dataP + val;
            

            dataLeftToCopy -= val;
            IX_ERROR_CGT(ix_rm_buffer_get_meta(hHwBuffer[count],
             (void **)&pMetaLinkedBuffers), err,
                         IX_CC_ERROR_INTERNAL, 0, clean_up_label);
			
            /* Update Buffer size in meta */
            pMetaLinkedBuffers->m_BufferSize = IX_HTON16(val);
            LengthData = pMetaLinkedBuffers->m_BufferSize;
            /* Set cellcount field in buffer */
            cell_count = ix_cc_calculate_cell_count((ix_uint32)LengthData, 0);

                                   
            /* Write the cell count into the buffer handle */
            IX_ERROR_CGT(ix_rm_buffer_cell_count_set(&hHwBuffer[count], cell_count),
            	 err, IX_CC_ERROR_INTERNAL, 0, clean_up_label);
            /* Update m_Offset to 0 */
            pMetaLinkedBuffers->m_Offset = 0;
            if (dataLeftToCopy <= IX_CC_FRAGV4_BUFFER_SIZE)
            {
                if (dataLeftToCopy == 0)
                {
                    val =0;
                    
                }
                else
                {
                    val = dataLeftToCopy;
                    dataLeftToCopy = 0;
                }
            }
            else
            {
                val = IX_CC_FRAGV4_BUFFER_SIZE;
                
            }     

            /* Link these buffers */
			IX_ERROR_CGT(ix_rm_buffer_link(&hHwBuffer[count - 1], 
                    &hHwBuffer[count]), err,
                    IX_CC_ERROR_INTERNAL, 0, clean_up_label);
        }

		/* Set fragment->buffer to hHwBuffer[0] **/
        frag->hBuffer = hHwBuffer[0];

    }
    else /** NO buffer chaining is required ***/
    {
        ix_buffer_handle hHwBuffer;
        hHwBuffer = IX_NULL_BUFFER_HANDLE;
        
		err = ix_rm_buffer_alloc(arg_hHwFreeList, &hHwBuffer);
		if (err != IX_SUCCESS)
		{

			ix_rm_buffer_free(hHwBuffer);

			goto clean_up_label;
		}
        
        
        frag->bufferChaining = 0;
        /* Get meta data pointer in buffer */
        IX_ERROR_CGT(ix_rm_buffer_get_meta(hHwBuffer, (void **)&pMeta), err,
                    IX_CC_ERROR_INTERNAL, 0, clean_up_label);
        
        /* Enhancement: Copy the whole meta structure so you will never
         * miss anything and ther will be no need to change this when the
         * structure layout changes.
         */
        *pMeta = *arg_pMeta;
        /* Set m_Offset to 128 */
        pMeta->m_Offset = 128;
#if defined(IX_PLATFORM_2401) || defined(IX_PLATFORM_2801)
        /* 
         * Set the m_HwNext pointer for the new buffer.
         */
        pMeta->m_HwNext = 0xff;
#endif /* IX_PLATFORM_2x01 */ 
        
        
        frag->pNext = 0;
        /* now get the location of the start of data in the new fragment*/
	    /* Copy IP header from parent buffer to frag->buffer **/
        
        IX_ERROR_CGT(ix_cc_hw_get_packet_data(hHwBuffer, &pTemp), err,
                     IX_CC_ERROR_INTERNAL, 0, clean_up_label);
        
        ix_ossl_memcpy(pTemp, arg_pData, IX_CC_FRAGV4_IP_HEADER_MIN_SIZE);

        tempPtr = (ix_uint8 *)pTemp;
        frag->pHeader = (ix_cc_fragv4_ip4hdr*) tempPtr;

		
        IX_ERROR_CGT(ix_cc_fragv4_ip4hdr_read_hlen_in_octets(hptr, &hLen), err,
                   IX_CC_FRAGV4_ERROR_IP_FRAGMENT, 0, clean_up_label);

        if (hLen > IX_CC_FRAGV4_IP_HEADER_MIN_SIZE)
        {
            IX_ERROR_CRT(ix_cc_fragv4_ip4hdr_copy_options(hptr, frag->pHeader,
                arg_IsFirst, &fragHLen), IX_CC_FRAGV4_ERROR_IP_FRAGMENT, 0);

		
        }
        else
        {
            fragHLen = hLen;
           
        }
        
        /* now add data */
        dataP = arg_pData + hLen + arg_DataOff;
        
        dest = tempPtr + fragHLen;
        ix_ossl_memcpy(dest, (void *)dataP, arg_Length);
       
        pMeta->m_PacketSize = IX_HTON16((ix_uint16)(fragHLen + arg_Length));
    	pMeta->m_BufferSize = IX_HTON16((ix_uint16)(fragHLen + arg_Length));
        LengthData = pMeta->m_BufferSize;
         
        /* Set cellcount field in buffer */
        cell_count = ix_cc_calculate_cell_count((ix_uint32)LengthData, 1);
      
        /* Write the cell count into the buffer handle */
        IX_ERROR_CGT(ix_rm_buffer_cell_count_set(&hHwBuffer, cell_count), err,
          IX_CC_ERROR_INTERNAL, 0, clean_up_label);

        frag->hBuffer = hHwBuffer;
        
           
    } /* end of else where buffer chaining is not required */
    /* fix offset, total length, and checksum if necessary */
    IX_ERROR_CGT(ix_cc_fragv4_ip4hdr_frag_off_write(frag->pHeader,
                 arg_flagsAndOff), err,
                 IX_CC_FRAGV4_ERROR_IP_FRAGMENT, 0, clean_up_label);
    /* Write the new total length field in IP header of fragment */
    IX_ERROR_CGT(ix_cc_fragv4_ip4hdr_len_write(frag->pHeader,
                 fragHLen+arg_Length), err,
                IX_CC_FRAGV4_ERROR_IP_FRAGMENT, 0, clean_up_label);
   
    
    IX_ERROR_CGT(ix_cc_fragv4_checksum_ip4hsum(frag->pHeader, &sum, 1), err,
                 IX_CC_FRAGV4_ERROR_IP_FRAGMENT, 0, clean_up_label);
    

    *arg_ppNewFrag = frag;

    return IX_SUCCESS;

clean_up_label:

    if(frag != 0)
        ix_ossl_free(frag);

    return err;

}

ix_error ix_cc_fragv4_fragment_create_from_chain(ix_uint32 arg_FragSize,
                                   ix_uint8 *arg_pData, /* CHANGE */
                                   ix_hw_buffer_meta *arg_pMeta, /* CHANGE */
                                   ix_uint16 arg_FlagsAndOff, 
                                   ix_uint16 arg_DataOff,
                                   ix_uint16 arg_Length,
                                   ix_uint32 arg_IsFirst,
                                   ix_buffer_free_list_handle arg_hHwFreeList,
                                   void *arg_pOrigData,
                                   ix_cc_fragv4_fragment **arg_ppNewFrag)
{

    ix_cc_fragv4_fragment *frag = 0;
    ix_uint16    hLen, fragHLen, sum; 
    ix_uint8    *tempPtr;
    ix_uint8 *dataP;
    void *pTemp;
    
    ix_uint8 *dest;
    ix_uint8 *pDest;
    ix_cc_fragv4_ip4hdr *hptr;
       
    ix_hw_buffer_meta *pMetaFirstBuffer;
    ix_hw_buffer_meta *pMetaLinkedBuffers;
    ix_hw_buffer_meta *pMeta;
    ix_uint32 count=0;
    ix_uint16 LengthData=0;   
    ix_uint32 cell_count=0;
    
    void *pDataInLinkedBuffers;
    ix_uint16 dataCopied = IX_CC_FRAGV4_BUFFER_SIZE;
    ix_uint16 payloadInFirstBuffer;
    ix_uint16 dataLeftToCopy=0; 
	int k=0;    
	ix_error err = IX_SUCCESS;
#ifdef IX_DEBUG    
    if (arg_ppNewFrag == NULL)
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Pointer to new fragment is null\n")); 
    if ((arg_FragSize < IX_CC_FRAGV4_IP_FRAG_MIN_SIZE) ||
     (arg_FragSize > IX_CC_FRAGV4_IP_FRAG_MAX_SIZE))
        return IX_ERROR_LOCAL(IX_CC_ERROR_RANGE,
                            ("Invalid range for arg_FragSize\n"));
    if ((arg_Length < 1) || 
    (arg_Length > (IX_CC_FRAGV4_IP_FRAG_MAX_SIZE - IX_CC_FRAGV4_IP_HEADER_MIN_SIZE)))
        return IX_ERROR_LOCAL(IX_CC_ERROR_RANGE,
                            ("Invalid range for arg_Length"));
#endif

    hptr = (ix_cc_fragv4_ip4hdr *)arg_pData;
    frag = ix_ossl_malloc(sizeof(ix_cc_fragv4_fragment));
    if (NULL==frag)
    {
        return IX_ERROR_LOCAL(IX_CC_FRAGV4_ERROR_IP_FRAGMENT, ("Null fragment"));
                
    }
    frag->pPrev = frag->pNext = NULL;
    frag->dataLen = arg_Length;
    frag->flagsAndOff = arg_FlagsAndOff;
     
    /* fragSize is  used now because we assume all fragments cannot fit in
       1 buffer size 2K. We have the ability to link buffers and allow,
       bigger fragment- fragSize will be used to decide number of buffers
       needed
    */
        
    /** Check if fragSize which is in octets is greater than 2k bytes 
     **ie 2048 octets 
     **If yes, then whole of IP hdr + payload cannot fit in single buffer
     ** and we need to chain multiple buffers 
     **/
    if (arg_FragSize > ((ix_uint32)IX_CC_FRAGV4_BUFFER_SIZE - 128))  
    {
		
        /** We need to create as many buffer handles based on arg_FragSize **/
        /** arg_hHwFreeList is a handle to hardware
        freelist that has 2K buffers*/
        ix_buffer_handle hHwBuffer[10];
        ix_uint32 totalNoOfBuffersNeeded= 0;
 
            
        IX_ERROR_CGT(ix_cc_fragv4_ip4hdr_read_hlen_in_octets(hptr, &hLen), err,
                    IX_CC_FRAGV4_ERROR_IP_FRAGMENT, 0, clean_up_label);
		
		
        if (hLen > IX_CC_FRAGV4_IP_HEADER_MIN_SIZE)
        {
            IX_ERROR_CGT(ix_cc_fragv4_ip4hdr_copy_options(hptr,
               frag->pHeader, arg_IsFirst, &fragHLen), err, IX_CC_FRAGV4_ERROR_IP_FRAGMENT, 0, clean_up_label);
	        fragHLen = IX_NTOH16(fragHLen);
        }
        else
        {
            fragHLen = IX_NTOH16(hLen);
        }       
        totalNoOfBuffersNeeded=
        1+ (arg_Length+fragHLen + 127)/IX_CC_FRAGV4_BUFFER_SIZE;
                
        frag->bufferChaining = 1;
      
        
        for (count = 0; count < totalNoOfBuffersNeeded; count++)
        {
            hHwBuffer[count] = IX_NULL_BUFFER_HANDLE;
            
			err = ix_rm_buffer_alloc(arg_hHwFreeList, &hHwBuffer[count]);
			if (err != IX_SUCCESS)
			{
				for (k=0;k<count;k++)
				{
					ix_rm_buffer_free(hHwBuffer[k]);
				}
				
				goto clean_up_label;
			}
             
        }
		                
        /** copy meta data from original buffer into first buffer
         ** in linked list 
         **/
        /** Note: copy meta data info. of original packet to first fragment **/
        /** But each fragment should have its buffer size in meta **/
        IX_ERROR_CGT(ix_rm_buffer_get_meta(hHwBuffer[0],
         (void **)&pMetaFirstBuffer), err,
                    IX_CC_ERROR_INTERNAL, 0, clean_up_label);
 
        /* Enhancement: Copy the whole meta structure so you will never
         * miss anything and there will be no need to change this when the
         * structure layout changes.
         */
        *pMetaFirstBuffer = *arg_pMeta;        
        
        /* Set m_Offset to 128*/
        pMetaFirstBuffer->m_Offset = 128;
        /* For first buffer in chain, we need to leave m_Offset of 128 for
         microblock to prepend header */	
        /* Now copy data from original buffer into these linked buffers **/
        IX_ERROR_CGT(ix_cc_hw_get_packet_data(hHwBuffer[0],
         &pDataInLinkedBuffers), err,
                    IX_CC_ERROR_INTERNAL, 0, clean_up_label);
       
       
        tempPtr = (ix_uint8 *)pDataInLinkedBuffers;
        /* First let me copy IX_CC_FRAGV4_IP_HEADER_MIN_SIZE of header onto first buffer **/
        /** We can copy upto 2K perhaps in first buffer or whatever size is 
        supported in free list**/
        ix_ossl_memcpy(pDataInLinkedBuffers, arg_pData,
             IX_CC_FRAGV4_IP_HEADER_MIN_SIZE );

        frag->pHeader = (ix_cc_fragv4_ip4hdr*) tempPtr;
            
        /* Set the packetsize for this fragment */
        pMetaFirstBuffer->m_PacketSize = IX_HTON16((ix_uint16)(fragHLen+ arg_Length));
        /* now add data */
         dataP = arg_pOrigData + fragHLen + arg_DataOff;

        /* Add rest of data from original this_buff **/
        /* Calculate amount of data that would fit in this
         buffer - first buffer in the chain **/
        if ((arg_Length + fragHLen) < (IX_CC_FRAGV4_BUFFER_SIZE - 128))
        {
            payloadInFirstBuffer = arg_Length;
        }
        else
        {

			payloadInFirstBuffer = IX_CC_FRAGV4_BUFFER_SIZE - hLen -128;
        }
        
        /*New offset where payload needs to be copied is at the offset
        from minimum IP header length + options if it is there */
        pDest = tempPtr + fragHLen;
        ix_ossl_memcpy(pDest, (void *)(dataP),
                     payloadInFirstBuffer);

        /* Update buffer size in meta data for first buffer in chain **/
        pMetaFirstBuffer->m_BufferSize = IX_HTON16(
                (ix_uint16)(fragHLen + payloadInFirstBuffer));
        

        LengthData = pMetaFirstBuffer->m_BufferSize;     
        /* Set cellcount field in buffer */
        cell_count = ix_cc_calculate_cell_count((ix_uint32)LengthData, 1);
          
		
        /* Write the cell count into the buffer handle */
		IX_ERROR_CGT(ix_rm_buffer_cell_count_set(&hHwBuffer[0], cell_count), err,
          IX_CC_ERROR_INTERNAL, 0,clean_up_label);


        /* Calculate amount of payload left to copy in rest of fragments
         in the chain **/
        dataLeftToCopy = arg_Length - payloadInFirstBuffer;
         
        if (dataLeftToCopy <= IX_CC_FRAGV4_BUFFER_SIZE)
        {
            if (dataLeftToCopy == 0)
            {
                dataCopied =0;
                    
            }
            else
            {
                dataCopied = dataLeftToCopy;
                dataLeftToCopy = 0;
            }
        }
        else
        {
            dataCopied = IX_CC_FRAGV4_BUFFER_SIZE;
                
        }     
                  
        dataP +=  payloadInFirstBuffer;
	
        for (count = 1, 
            pMetaLinkedBuffers = NULL, 
	    pDataInLinkedBuffers = NULL; count < totalNoOfBuffersNeeded; count ++)
        {
            
            IX_ERROR_CGT(ix_rm_buffer_get_data(hHwBuffer[count],
             &pDataInLinkedBuffers), err,
                             IX_CC_ERROR_INTERNAL, 0, clean_up_label);
            
            /* Copy 2K bytes and we can copy rest into other buffers which 
            will be linked **/
            ix_ossl_memcpy(pDataInLinkedBuffers, (void *)dataP, dataCopied);

            dataP += dataCopied;
            
            dataLeftToCopy -= dataCopied;
            if (dataLeftToCopy <= 0 )
                  dataLeftToCopy =0;
            
            
            IX_ERROR_CGT(ix_rm_buffer_get_meta(hHwBuffer[count],
             (void **)&pMetaLinkedBuffers), err,
                         IX_CC_ERROR_INTERNAL, 0, clean_up_label);
	        /* Update m_Offset field in meta to 0 */
            pMetaLinkedBuffers->m_Offset = 0;		
            /* Update Buffer size in meta */
            pMetaLinkedBuffers->m_BufferSize = IX_HTON16(dataCopied);
            
            LengthData = pMetaLinkedBuffers->m_BufferSize;

            /* Set cellcount field in buffer */
            cell_count = ix_cc_calculate_cell_count((ix_uint32)LengthData, 0);

            /* Write the cell count into the buffer handle */
            IX_ERROR_CGT(ix_rm_buffer_cell_count_set(&hHwBuffer[count], cell_count),
				 err, IX_CC_ERROR_INTERNAL, 0, clean_up_label);
            if (dataLeftToCopy <= IX_CC_FRAGV4_BUFFER_SIZE)
            {
                if (dataLeftToCopy == 0)
                {
                    dataCopied =0;
                    break;
                }
                else
                {
                    dataCopied = dataLeftToCopy;
                    dataLeftToCopy = 0;
                }
            }
            else
            {
                 dataCopied = IX_CC_FRAGV4_BUFFER_SIZE;
                
            }
   
            /* Link these buffers */
            IX_ERROR_CGT(ix_rm_buffer_link(&hHwBuffer[count - 1], 
                    &hHwBuffer[count]), err,
                    IX_CC_ERROR_INTERNAL, 0, clean_up_label);
           
        }

        /* Set fragment->buffer to hHwBuffer[0] **/
        frag->hBuffer = hHwBuffer[0];

    }

    else /** NO buffer chaining is required ***/
    {
        ix_buffer_handle hHwBuffer;
        hHwBuffer = IX_NULL_BUFFER_HANDLE;
        err = ix_rm_buffer_alloc(arg_hHwFreeList, &hHwBuffer);
        if (err != IX_SUCCESS)
        {
            ix_rm_buffer_free(hHwBuffer);
            goto clean_up_label;
        }

   
        
        frag->bufferChaining = 0;

        
        frag->pNext = 0;
        /* Trying to fill up meta data in buffer */
        IX_ERROR_CGT(ix_rm_buffer_get_meta(hHwBuffer, (void **)&pMeta), err,
                    IX_CC_ERROR_INTERNAL, 0, clean_up_label);
        /* Enhancement: Copy the whole meta structure so you will never
         * miss anything and there will be no need to change this when the
         * structure layout changes.
         */
        *pMeta = *arg_pMeta;
#if defined(IX_PLATFORM_2401) || defined(IX_PLATFORM_2801)
        /* Bug fix
         * Set the m_HwNext pointer for the new buffer.
         */
        pMeta->m_HwNext = 0xff;
#endif /* IX_PLATFORM_2x01 */
        
        /* Need to set m_Offset to 128 for first buffer in chain */
        pMeta->m_Offset = 128;
        /* now get the location of the start of data in the new fragment*/
        /* Copy IP header from parent buffer to frag->buffer **/
        
        IX_ERROR_CGT(ix_cc_hw_get_packet_data(hHwBuffer, &pTemp), err,
                     IX_CC_ERROR_INTERNAL, 0, clean_up_label);
        
    
        ix_ossl_memcpy(pTemp, arg_pData, IX_CC_FRAGV4_IP_HEADER_MIN_SIZE);

        tempPtr = (ix_uint8 *)pTemp;
        frag->pHeader = (ix_cc_fragv4_ip4hdr*) tempPtr;

		
        IX_ERROR_CGT(ix_cc_fragv4_ip4hdr_read_hlen_in_octets(hptr, &hLen), err,
                   IX_CC_FRAGV4_ERROR_IP_FRAGMENT, 0, clean_up_label);

        if (hLen > IX_CC_FRAGV4_IP_HEADER_MIN_SIZE)
        {
            IX_ERROR_CGT(ix_cc_fragv4_ip4hdr_copy_options(hptr, frag->pHeader,
                arg_IsFirst, &fragHLen), err, IX_CC_FRAGV4_ERROR_IP_FRAGMENT, 0, clean_up_label);

		
        }
        else
        {
            fragHLen = hLen;
           
        }
        /* now add data */
  
        dataP = arg_pOrigData + fragHLen + arg_DataOff;
        
        dest = tempPtr + fragHLen;

        ix_ossl_memcpy(dest, (void *)dataP, arg_Length);

       
       
        pMeta->m_PacketSize = IX_HTON16((ix_uint16)(fragHLen + arg_Length));
	    pMeta->m_BufferSize = IX_HTON16((ix_uint16)(fragHLen + arg_Length));
        LengthData = pMeta->m_BufferSize;
        /* Set cellcount field in buffer */
        cell_count = ix_cc_calculate_cell_count((ix_uint32)LengthData, 1);

        /* Write the cell count into the buffer handle */
        IX_ERROR_CGT(ix_rm_buffer_cell_count_set(&hHwBuffer, cell_count), err,
          IX_CC_ERROR_INTERNAL, 0, clean_up_label);

        frag->hBuffer = hHwBuffer;
        /* Enhancement: Copy the whole meta structure so you will never
         * miss anything and ther will be no need to change this when the
         * structure layout changes.
         */
        /* see above *pMeta = *arg_pMeta; */
            
   
    } /* end of else where buffer chaining is not required */
    /* fix offset, total length, and checksum if necessary */
    IX_ERROR_CGT(ix_cc_fragv4_ip4hdr_frag_off_write(frag->pHeader,
                 arg_FlagsAndOff), err,
                 IX_CC_FRAGV4_ERROR_IP_FRAGMENT, 0, clean_up_label);
    /* Write the new total length field in IP header of fragment */
    IX_ERROR_CGT(ix_cc_fragv4_ip4hdr_len_write(frag->pHeader,
                 fragHLen+arg_Length), err,
                IX_CC_FRAGV4_ERROR_IP_FRAGMENT, 0, clean_up_label);
   
    
    IX_ERROR_CGT(ix_cc_fragv4_checksum_ip4hsum(frag->pHeader, &sum, 1), err,
                 IX_CC_FRAGV4_ERROR_IP_FRAGMENT, 0, clean_up_label);
    

    *arg_ppNewFrag = frag;

    return IX_SUCCESS;

clean_up_label:

	if(frag != 0)
        ix_ossl_free(frag);

    return err;

}

ix_error _ix_cc_fragv4_get_data_from_chain(
                          ix_buffer_handle arg_hBuffer, 
                          void **pOrigData)
{
    ix_buffer_handle hCurrBuffer = arg_hBuffer;
    ix_buffer_handle hNextBuffer = IX_NULL_BUFFER_HANDLE;
    ix_hw_buffer_meta *pMeta;
    ix_uint16 pktSize = 0;
    ix_uint8 *pData;
    ix_uint8 *pEntireData;
    ix_uint16 bufSize = 0;
    ix_uint32 isEOP=0;    
    /* Get meta data from original buffer chain */
    IX_ERROR_CRT(ix_rm_buffer_get_meta(arg_hBuffer, (void **)&pMeta),
       IX_CC_ERROR_INTERNAL, 0);
    /* Get packet size contained in entire chain of buffers */
    pktSize = IX_NTOH16(IX_RM_MEM_UINT16_READ(&pMeta->m_PacketSize));
    
    /* Allocate memory for pEntireData */
    pEntireData = ix_ossl_malloc(pktSize);    
    /* Get the next buffer handle in the chain */
    
    *pOrigData = pEntireData;
    IX_ERROR_CRT(ix_cc_hw_get_packet_data(hCurrBuffer,(void **)&pData), 
                     IX_CC_ERROR_INTERNAL, 0);

    /* Get size of data in this buffer */
    bufSize = IX_NTOH16(IX_RM_MEM_UINT16_READ(
                   &pMeta->m_BufferSize));         
    /* Copy data from this buffer */
    ix_ossl_memcpy(pEntireData, pData, bufSize);
    pEntireData += bufSize; 
    
    IX_ERROR_CRT(ix_rm_buffer_is_eop(hCurrBuffer, &isEOP), 
        IX_CC_ERROR_INTERNAL, 0);
    /* traverse the chain to get data contained in each buffer in the 
    chain*/
    while (!isEOP)
    {
        IX_ERROR_CRT(ix_rm_buffer_get_next(hCurrBuffer, &hNextBuffer), 
                     IX_CC_ERROR_INTERNAL, 0);
        hCurrBuffer = hNextBuffer;
        hNextBuffer = IX_NULL_BUFFER_HANDLE;
        IX_ERROR_CRT(ix_cc_hw_get_packet_data(hCurrBuffer,(void **)&pData), 
                     IX_CC_ERROR_INTERNAL, 0);
        /* Get meta for this buffer handle */
        IX_ERROR_CRT(ix_rm_buffer_get_meta(hCurrBuffer, (void **)&pMeta), 
                    IX_CC_ERROR_INTERNAL, 0);

        /* Get size of data in this buffer */
        bufSize = IX_NTOH16(IX_RM_MEM_UINT16_READ(
                   &pMeta->m_BufferSize));
        /* Copy data from this buffer */
        ix_ossl_memcpy(pEntireData, pData, bufSize);
        pEntireData += bufSize; 
        IX_ERROR_CRT(ix_rm_buffer_is_eop(hCurrBuffer, &isEOP), 
          IX_CC_ERROR_INTERNAL, 0);            

    }
    
    return IX_SUCCESS;

}
   



ix_error ix_cc_fragv4_fragment_make_from_buffer(ix_buffer_handle arg_hBuffer,
                                      ix_uint32 arg_HdrOff ,
                                     ix_cc_fragv4_fragment **arg_ppNewFrag)
{
    
    ix_uint16	size;
    ix_uint8     *pData;
    ix_uint16	offset;
    ix_cc_fragv4_fragment *pFrag;
    ix_cc_fragv4_ip4hdr   *hptr;
    ix_hw_buffer_meta *pHwMeta;

    if (arg_ppNewFrag == NULL)
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Null fragment pointer\n")); 
    
    IX_ERROR_CRT(ix_rm_buffer_get_meta(arg_hBuffer,(void **)&pHwMeta),
                IX_CC_ERROR_INTERNAL, 0);
       
    /** Get the size of the current buffer **/
    size = IX_NTOH16(IX_RM_MEM_UINT16_READ(&pHwMeta->m_BufferSize));    

    if (size < IX_CC_FRAGV4_IP_HEADER_MIN_SIZE)
    {
        
        return IX_ERROR_LOCAL(IX_CC_ERROR_FULL,
                            ("Buffer is too small\n"));
                            
    }
    IX_ERROR_CRT(ix_cc_hw_get_packet_data(arg_hBuffer, (void **)&pData),
                IX_CC_ERROR_INTERNAL, 0);
   
    
    hptr = (ix_cc_fragv4_ip4hdr*)(pData+arg_HdrOff);
       
    pFrag = ix_ossl_malloc(sizeof(ix_cc_fragv4_fragment));
    if (NULL == pFrag)
    {
        ix_ossl_message_log("%s, line %d:\n"
         "ix_fragment_make_from_buffer: can't allocate memory for fragment\n",
                __FILE__, __LINE__);
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,("Null fragment "));
               
    }
    pFrag->pPrev = pFrag->pNext = NULL;
    pFrag->pHeader = hptr;
    pFrag->hBuffer = arg_hBuffer;

    IX_ERROR_CRT(ix_cc_fragv4_ip4hdr_frag_off_read(hptr, &offset),
                 IX_CC_FRAGV4_ERROR_IP_FRAGMENT, 0);
    /* flagsAndOff value has to be in host order */
    pFrag->flagsAndOff = IX_NTOH16(offset);

    /* This function expects data length to be returned in host byte order */
    IX_ERROR_CRT(ix_cc_fragv4_ip4hdr_data_len(hptr, &(pFrag->dataLen)),
                 IX_CC_FRAGV4_ERROR_IP_FRAGMENT, 0);
    
    *arg_ppNewFrag = pFrag;

    return IX_SUCCESS;

}

ix_error ix_cc_fragv4_fragment_print( ix_cc_fragv4_fragment *arg_pFragment,
                             ix_uint32 arg_PrintHeader,
                             ix_uint32 arg_PrintData)
{
    ix_error istatus;
    ix_cc_fragv4_ip4hdr *pHeader;
    ix_uint8 *pByte;
    ix_uint16   *pPayload;
    void *pData;
    ix_uint32 i;
    ix_uint8 *dataP;

    pHeader = arg_pFragment->pHeader;

    /* check if this fragment has been 'collasped' into a datagram during 
      datagram reassembly*/
    IX_ERROR_CRT(ix_cc_hw_get_packet_data(arg_pFragment->hBuffer, &pData),
                  IX_CC_ERROR_INTERNAL, 0); 
    
   
    dataP = (ix_uint8 *)pData;
    if ( (dataP > (ix_uint8*)pHeader) && arg_PrintHeader )
    {
        
		
        istatus = IX_ERROR_WARNING(IX_CC_ERROR_RANGE,
                 ("The fragment header is no longer part of fragment\n"));            

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        ix_error_dump(0, istatus);
#else
        ix_error_dump(stdout, istatus);
#endif
        arg_PrintHeader = 0;
    }
    if (arg_PrintHeader)
    {
        istatus = ix_cc_fragv4_ip4hdr_print(pHeader);
        if (istatus)
        {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
            ix_ossl_message_log("Failed to print fragment header\n");
            ix_error_dump (0, istatus);
#else
            fprintf(stdout, "Failed to print fragment header\n");
            ix_error_dump (stdout, istatus);
#endif
        }
    }
    if (arg_PrintData)
    {
        IX_ERROR_CR(ix_cc_fragv4_ip4hdr_payload(pHeader, &pPayload)); 
           
        

        pByte = (ix_uint8*)pPayload;
        for (i=0; i<arg_pFragment->dataLen; i++)
        {
            ix_ossl_message_log("data[%ld] = %x\n", i, *(pByte+i));
        }
    }

    return IX_SUCCESS;
}


   

ix_error ix_cc_fragv4_fragment_divide( ix_cc_fragv4_fragment *arg_pFragment,
                         ix_uint32 arg_Mtu,
                        ix_buffer_free_list_handle arg_hHwFreeList,  
			ix_cc_fragv4_fragment **arg_ppFirst )
{
    ix_cc_fragv4_fragment	*savePrev, *saveNext, *prevFrag;
    ix_cc_fragv4_ip4hdr	*hptr;
    void *pOrigData = 0;
    ix_buffer_handle	buffer;
    ix_uint16	allLen, hdrLen, dataLen;
    ix_uint16    off, fragSize, fragOffset, newOff;
    ix_uint32		first, last;
    ix_uint32 eopFlag = 0; /* To detect if buffer handle has EOP flag set */

    ix_hw_buffer_meta *pMeta;  
    ix_uint8 *pData; 
    ix_error err = IX_SUCCESS;

    
    if (arg_pFragment == NULL)
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Null fragment pointer\n"));
    if (arg_ppFirst == NULL)
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Null first pointer\n")); 
 
    hptr = arg_pFragment->pHeader;
    buffer = arg_pFragment->hBuffer;
    savePrev = (ix_cc_fragv4_fragment*)arg_pFragment->pPrev;
    saveNext = (ix_cc_fragv4_fragment*)arg_pFragment->pNext;

#ifdef IX_DEBUG
    /* check integrity of header pointer*/
    if (hptr == NULL)
    {
        ix_ossl_message_log("%s, line %d:\n"
                            "In ix_fragment_divide: the given fragment has "
							"NULL header pointer\n",
                            __FILE__, __LINE__);
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Null header pointer\n"));
                            
    }
#endif

    IX_ERROR_CRT(ix_cc_fragv4_ip4hdr_frag_off_read( hptr, &fragOffset),
                IX_CC_FRAGV4_ERROR_IP_FRAGMENT, 0);
    
    if (fragOffset & IX_CC_FRAGV4_IP_FRAG_DF)
    { 
        /* don't fragment bit set */
        return IX_ERROR_LOCAL(IX_CC_FRAGV4_ERROR_IP_FRAGMENT, 
                ("Do not fragment bit set\n"));
                            
    }
    /* This allLen is returned in network byte order */
    IX_ERROR_CRT(ix_cc_fragv4_ip4hdr_len_read( hptr, &allLen),
                IX_CC_FRAGV4_ERROR_IP_FRAGMENT, 0);
   
	/** Note MTU passed as parameter is ix_uint32 but fragSize is ix_uint16 **/ 
    fragSize = (ix_uint16)arg_Mtu;

    if ( (IX_NTOH16(allLen)) <= fragSize )
    /* no need to fragment. Print a warning here */
    {
        ix_ossl_message_log("%s, line %d: WARNING: ix_fragment_divide"
        " was called with\n"
        "a datagram that is smaller or equal to the allowed MTU. "
          "No need to fragment\n"
            ,__FILE__, __LINE__);
        *arg_ppFirst = arg_pFragment;
        return IX_SUCCESS;
	}	
    /* This header len is returned in network byte order */
    IX_ERROR_CRT(ix_cc_fragv4_ip4hdr_read_hlen_in_octets( hptr, &hdrLen),
                IX_CC_FRAGV4_ERROR_IP_FRAGMENT, 0);
    /* Check if the buffer handle that came is a list of chained buffers */
    /* If EOP flag is set, then this buffer is not chained else it is chained */
    IX_ERROR_CRT(ix_rm_buffer_is_eop(buffer, &eopFlag),
           IX_CC_ERROR_INTERNAL, 0);   
    
    if (eopFlag == 0)
    {
        /* Get data from chained buffers */
        IX_ERROR_CRT(_ix_cc_fragv4_get_data_from_chain(buffer, &pOrigData),
                        IX_CC_ERROR_INTERNAL, 0);  

    }
        
    /* Get meta data pointer */
    IX_ERROR_CRT(ix_rm_buffer_get_meta(arg_pFragment->hBuffer, (void **)&pMeta),
     IX_CC_ERROR_INTERNAL, 0);
    
    IX_ERROR_CRT(ix_cc_hw_get_packet_data(arg_pFragment->hBuffer, (void **)&pData),
     IX_CC_ERROR_INTERNAL, 0);
   
    first = 1;
    last = 0;
    prevFrag = NULL;

    /* since header length in the first fragment can be longer
      than that in consequence fragments ( ie. when there're options ),
      dataLen chosen for fragments can be some octest shorter the 
      maximum number of bytes available. This is OK as long as
      the length of the fragment in header.len is correct
    */
    /** According to RFc791: The data of the long
    datagram is divided into portions on a 8 octet (64 bit) boundary
    (the second portion might not be an integral multiple of 8 octets,
    but the first must be).***/

    dataLen = (fragSize - (IX_NTOH16(hdrLen)));
    /* since offset field in IP header is a in multiple of 8, dataLen must 
      also be a multiple of 8. If not, the offset field can't represent
      the precise offset of the fragment */
    dataLen = dataLen & ~7;

    for ( off = hdrLen; off < allLen; off += dataLen )
    {
        ix_cc_fragv4_fragment	*newFrag;
        ix_uint16	realLen, dataOff;

        newFrag = NULL;
        newOff = ((off - hdrLen) >> 3) + (fragOffset & ~IX_CC_FRAGV4_IP_FRAG_MF );
        dataOff = off - hdrLen;
        realLen = dataLen; 

        /*if not last fragment */
        if ( ( off + realLen ) < allLen )
        {
            newOff |= IX_CC_FRAGV4_IP_FRAG_MF;
        }
        else
        {
            last = 1;
            realLen = allLen - off; 
            /*if the orginal fragment is the middle fragment of another
        	 datagram, then this one must have MF bit set */
            if ( fragOffset & IX_CC_FRAGV4_IP_FRAG_MF )
            {
                newOff |= IX_CC_FRAGV4_IP_FRAG_MF;
            }
        }
        /* If EOP flag is set, then this buffer is not chained */
        if (eopFlag == 1)
        {

            /* create a fragment */
            IX_ERROR_CG(ix_cc_fragv4_fragment_create(fragSize,
                                 pData, pMeta, newOff, dataOff,
				 realLen, first, arg_hHwFreeList, &newFrag),
                        err, 
                        clean_up_label);
         
        }
        else if (eopFlag == 0)
        {

            /* Create a fragment from the buffer chain */
            IX_ERROR_CG(ix_cc_fragv4_fragment_create_from_chain(fragSize,
                                 pData, pMeta, newOff, dataOff,
		                 realLen, first, arg_hHwFreeList, pOrigData,
                                 &newFrag),
                        err, 
                        clean_up_label);
           
        }
      
        if (first)	
        {
            if (savePrev != NULL)
            {
                newFrag->pPrev = savePrev;
               	savePrev->pNext = newFrag;
            } 
                
        }
        else if (last)
        { 
          
            if (saveNext != NULL)
            {
                newFrag->pNext = saveNext;
          	saveNext->pPrev = newFrag;
            }
          
        }

        if (prevFrag != NULL)
        {
            prevFrag->pNext = (void*)newFrag;
        }	
        newFrag->pPrev = (void*)prevFrag;


        if (first)
        {
            *arg_ppFirst = newFrag;
        }
        first = 0;
        last = 0;
        prevFrag = newFrag;
	
    } /* end for */
    if (eopFlag == 0)
    {
        ix_ossl_free(pOrigData);

    }
    return IX_SUCCESS;


clean_up_label:
    {
        ix_cc_fragv4_fragment *pFragListIterator, *pFragListCur;
        pFragListIterator = *arg_ppFirst;
        *arg_ppFirst = NULL;

        if(pOrigData != 0)
            ix_ossl_free(pOrigData);

        while (pFragListIterator != NULL)
        {
            if (pFragListIterator->hBuffer == IX_NULL_BUFFER_HANDLE)
            {
		        ix_ossl_message_log("Got an fragment element with NULL buffer. \n");
                return(err);
            }

            if (ix_rm_buffer_free_chain(pFragListIterator->hBuffer) != IX_SUCCESS)
            {
                ix_ossl_message_log("Failed to free a buffer in ix_cc_fragv4_fragment_divide. \n");
            }
        
            pFragListCur = pFragListIterator;
            pFragListIterator = (ix_cc_fragv4_fragment *)pFragListIterator->pNext; 

            /* Ignore error from the call to ix_cc_fragv4_fragment_delete_frag_only */
            ix_cc_fragv4_fragment_delete_frag_only(pFragListCur);
        }
    }
    return(err);
}


                      
                         
ix_error ix_cc_fragv4_fragment_delete_frag_and_data_buff(
                                     ix_cc_fragv4_fragment *arg_pFragment)
{
    
    if (arg_pFragment == NULL)
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Null fragment pointer\n")); 
    
	ix_rm_buffer_free_chain(arg_pFragment->hBuffer);
	                   
   
    arg_pFragment->pPrev = NULL; 
    arg_pFragment->pNext = NULL; 
    ix_ossl_free(arg_pFragment); 

    return IX_SUCCESS;

}


ix_error ix_cc_fragv4_fragment_delete_frag_only( ix_cc_fragv4_fragment *arg_pFragment)
{
    if (arg_pFragment == NULL)
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Null fragment pointer\n")); 
    arg_pFragment->pPrev = NULL; 
    arg_pFragment->pNext = NULL; 
    ix_ossl_free(arg_pFragment);

    return IX_SUCCESS;


}
