/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.

 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor
 *
 *
 * = MODULE
 *      Fragmentation library - Fragmentation Handling
 *
 * = FILENAME
 *      ip4hdr.c
 *
 * = DESCRIPTION
 *      This file contains implementation of IPv4 Helper functions
 *
 *
 *      
 *
 * = CREATION TIME
 *      08/14/2002 1:56:24 PM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */

#define IX_ERROR_FILE_IDENT "$Id: ip4hdr.c,v 1.19 2003/05/01 01:11:42 rranjeet Exp $"

/**
 * System defined include files required.
 */

/**
 * User defined include files required.
 */
#include "ix_cc_error.h"
#include "ix_cc.h"
#include "cc/fragmentation/ip4hdr.h"
#include "cc/fragmentation/internal/internal-netrw.h"
/**
 * Preprocessor symbols and macros used in this file.
 */

/**
 * Types definitions whose scope is limited to this file.
 */

/**
 * Variable declarations global to this file only.  Externs are followed by
 * static variables.
 */

/**
 * Extern function prototypes.
 */

/**
 * Static function prototypes.
 */

/**
 * Function definitions.
 */
ix_error ix_cc_fragv4_hdr_get_ip_header_ptr(ix_buffer_handle arg_hBuffer,
					ix_cc_fragv4_ip4hdr **arg_ppIPhdr)
{
      
       
    void *pData;
    
    if (arg_ppIPhdr == NULL)
    {
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Header pointer is NULL\n")); 
    }
    /* find where buffer data starts */
    IX_ERROR_CRT(ix_cc_hw_get_packet_data(arg_hBuffer, &pData),
         IX_CC_ERROR_INTERNAL, 0);
    

    /** I am assuming IP packet **/
    *arg_ppIPhdr = (ix_cc_fragv4_ip4hdr*)pData;
    return IX_SUCCESS;
}

ix_error ix_cc_fragv4_ip4hdr_print(ix_cc_fragv4_ip4hdr *arg_pHeader)
{

    ix_error istatus;
    ix_uint8 oldVal8, hlen;
    ix_uint16   oldVal16, tlen;
    ix_uint32   oldVal32;

    if (arg_pHeader == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("IP header pointer is NULL\n"));
 
    istatus = ix_cc_fragv4_ip4hdr_version_read(arg_pHeader, &oldVal8);
    if (istatus)
    {

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

        ix_ossl_message_log("Fail to read version number\n");
        ix_error_dump(0, istatus);

#else

        fprintf(stdout, "Fail to read version number\n");
        ix_error_dump(stdout, istatus);

#endif /* end (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_) */

    }
    else
    {

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

        ix_ossl_message_log("Version number is 0x%x\n", oldVal8);

#else

        fprintf(stdout, "Version number is 0x%x\n", oldVal8);

#endif /* end (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_) */

    }
    istatus = ix_cc_fragv4_ip4hdr_read_hlen_in_w32(arg_pHeader, &oldVal8);
    if (istatus)
    {

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

        ix_ossl_message_log("%s, line %d:\n"
                            "Fail to read header len\n",
                            __FILE__, __LINE__);
        ix_error_dump(0, istatus);

#else

        fprintf(stdout, "%s, line %d:\n"
                            "Fail to read header len\n",
                            __FILE__, __LINE__);
        ix_error_dump (stdout, istatus);

#endif /* end (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_) */

    }
    else
    {

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

        ix_ossl_message_log("Header length: 0x%x 32-bit words\n", oldVal8);

#else

        fprintf(stdout, "Header length: 0x%x 32-bit words\n", oldVal8);

#endif /* end (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_) */

        hlen = (ix_uint16)(oldVal8 << 2);
    }
    istatus = ix_cc_fragv4_hdr_tos(arg_pHeader, &oldVal8);
    if (istatus)
    {

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

        ix_ossl_message_log("%s, line %d:\n"
                            "Fail to read type of service\n",
                            __FILE__, __LINE__);
        ix_error_dump(0, istatus);

#else

        fprintf(stdout, "%s, line %d:\n"
                            "Fail to read type of service\n",
                            __FILE__, __LINE__);
        ix_error_dump (stdout, istatus);

#endif /* end (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_) */

    }
    else
    {

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

        ix_ossl_message_log("Type of service: 0x%x\n", oldVal8);

#else

        fprintf(stdout, "Type of service: 0x%x\n", oldVal8);

#endif /* end (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_) */

    }
    istatus = ix_cc_fragv4_ip4hdr_len_read(arg_pHeader, &tlen);
    if (istatus)
    {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        ix_ossl_message_log("%s, line %d:\n"
                            "Fail to read type of service\n",
                            __FILE__, __LINE__);
        ix_error_dump (0, istatus); 
#else
        fprintf(stdout, "%s, line %d:\n"
                            "Fail to read type of service\n",
                            __FILE__, __LINE__);
        ix_error_dump (stdout, istatus); 
#endif
    }
    else
    {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

        ix_ossl_message_log("Total length of IP datagram: 0x%x\n", tlen);
#else	    
        fprintf(stdout, "Total length of IP datagram: 0x%x\n", tlen);
#endif
    }
    istatus = ix_cc_fragv4_ip4hdr_id(arg_pHeader, &oldVal16);
    if (istatus)
    {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        ix_ossl_message_log("%s, line %d:\n"
                            "Fail to read ID field\n",
                            __FILE__, __LINE__);
        ix_error_dump (0, istatus); 
#else
        fprintf(stdout, "%s, line %d:\n"
                            "Fail to read ID field\n",
                            __FILE__, __LINE__);
        ix_error_dump (stdout, istatus); 
#endif
    }
    else
    {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

        ix_ossl_message_log("Identification: 0x%x\n", oldVal16);
#else
        fprintf(stdout, "Identification: 0x%x\n", oldVal16);
#endif
    }
    istatus = ix_cc_fragv4_ip4hdr_frag_off_read(arg_pHeader, &oldVal16);
    if (istatus)
    {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        ix_ossl_message_log("%s, line %d:\n"
                            "Failed to read type of service\n",
                            __FILE__, __LINE__);
        ix_error_dump (0, istatus); 
#else
        fprintf(stdout, "%s, line %d:\n"
                            "Failed to read type of service\n",
                            __FILE__, __LINE__);
        ix_error_dump (stdout, istatus); 
#endif
    }
    else
    {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
	
        ix_ossl_message_log("Fragment offset: 0x%x\n", oldVal16);
#else
        fprintf(stdout, "Fragment offset: 0x%x\n", oldVal16);
#endif
    }
    istatus = ix_cc_fragv4_ip4hdr_ttl(arg_pHeader, &oldVal8);
    if (istatus)
    {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        ix_ossl_message_log("%s, line %d:\n"
                            "Failed to read time to live field\n",
                            __FILE__, __LINE__);
        ix_error_dump (0, istatus); 
#else
        fprintf(stdout, "%s, line %d:\n"
                            "Failed to read time to live field\n",
                            __FILE__, __LINE__);
        ix_error_dump (stdout, istatus); 
#endif
    }
    else
    {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

        ix_ossl_message_log("Time to live: 0x%x\n", oldVal8);
#else
        fprintf(stdout, "Time to live: 0x%x\n", oldVal8);
#endif
    }
    istatus = ix_cc_fragv4_ip4hdr_proto(arg_pHeader, &oldVal8);
    if (istatus)
    {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        ix_ossl_message_log("%s, line %d:\n"
                            "Failed to read protocol field\n",
                            __FILE__, __LINE__);
        ix_error_dump (0, istatus);
#else
        fprintf(stdout, "%s, line %d:\n"
                            "Failed to read protocol field\n",
                            __FILE__, __LINE__);
        ix_error_dump (stdout, istatus);
#endif
    }
    else
    {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

        ix_ossl_message_log("Protocol: 0x%x\n", oldVal8);
#else
        fprintf(stdout, "Protocol: 0x%x\n", oldVal8);
#endif
    }
    oldVal16 = 0;
    istatus = ix_cc_fragv4_ip4hdr_checksum_read(arg_pHeader, &oldVal16);
    if (istatus)
    {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        ix_ossl_message_log("%s, line %d:\n"
                            "Failed to read checksum\n",
                            __FILE__, __LINE__);
        ix_error_dump (0, istatus); 
#else
        fprintf(stdout, "%s, line %d:\n"
                            "Failed to read checksum\n",
                            __FILE__, __LINE__);
        ix_error_dump (stdout, istatus); 
#endif

    }
    else
    {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

        ix_ossl_message_log("Checksum: 0x%x\n", oldVal16);
#else
        fprintf(stdout, "Checksum: 0x%x\n", oldVal16);
#endif
    }
    istatus = ix_cc_fragv4_ip4hdr_src_32(arg_pHeader, &oldVal32);
    if (istatus)
    {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        ix_ossl_message_log("%s, line %d:\n"
                            "Failed to read IP source address\n",
                            __FILE__, __LINE__);
        ix_error_dump (0, istatus);
#else
        fprintf(stdout, "%s, line %d:\n"
                            "Failed to read IP source address\n",
                            __FILE__, __LINE__);
        ix_error_dump (stdout, istatus);
#endif
    }
    else
    {
        
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

	ix_ossl_message_log("Src addr: 0x%x\n", (ix_uint16)oldVal32);
#else
	fprintf(stdout, "Src addr: 0x%x\n", (ix_uint16)oldVal32);
#endif
    }
    istatus = ix_cc_fragv4_ip4hdr_dest_32(arg_pHeader, &oldVal32);
    if (istatus)
    {
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
        ix_ossl_message_log("%s, line %d:\n"
                            "Failed to read destination address\n",
                            __FILE__, __LINE__);
        ix_error_dump (0, istatus); 
#else
        fprintf(stdout, "%s, line %d:\n"
                            "Failed to read destination address\n",
                            __FILE__, __LINE__);
        ix_error_dump (stdout, istatus); 
#endif
    }
    else
    {
        
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

	ix_ossl_message_log("Dest addr: 0x%x\n", (ix_uint16)oldVal32);
#else
	fprintf(stdout, "Dest addr: 0x%x\n", (ix_uint16)oldVal32);
#endif
    }
    return IX_SUCCESS;

}

ix_error ix_cc_fragv4_ip4hdr_version_read(ix_cc_fragv4_ip4hdr *arg_pHeader,
                                   ix_uint8 *arg_pVersion)
{
    ix_uint8	*pTemp, temp;
    
    if (arg_pHeader == NULL)
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("IP header pointer is Null\n")); 
    if (arg_pVersion == NULL)
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Version pointe is null\n")); 
    pTemp = (ix_uint8*) arg_pHeader;

    temp = *pTemp;
    *arg_pVersion = temp >> 4;
    return IX_SUCCESS;

}


ix_error ix_cc_fragv4_ip4hdr_read_hlen_in_w32(ix_cc_fragv4_ip4hdr *arg_pHeader,
                                   ix_uint8 *arg_pHlen)
{
    ix_uint8	*pTemp, temp;
   
    if (arg_pHeader == NULL)
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("IP header pointer is null\n")); 
    if (arg_pHlen == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Header length pointer is null\n")); 
    pTemp = (ix_uint8*) arg_pHeader;

    temp = *pTemp;
    *arg_pHlen = temp & 0x0f;
    return IX_SUCCESS;
}

ix_error ix_cc_fragv4_ip4hdr_read_hlen_in_octets(ix_cc_fragv4_ip4hdr *arg_pHeader,
						ix_uint16 *arg_pHlen)
{

    ix_uint8	*pTemp, temp; 

    if (arg_pHeader == NULL)
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("IP header pointer is null\n")); 
    if (arg_pHlen == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Header length pointer is NULL\n")); 
    pTemp = (ix_uint8*) arg_pHeader;

    temp = *pTemp;
    *arg_pHlen = (ix_uint16)((temp & 0x0f) << 2);
    /**arg_pHlen = IX_HTON16((temp & 0x0f) << 2);*/

    return IX_SUCCESS;


}

ix_error ix_cc_fragv4_ip4hdr_hlen_write(ix_cc_fragv4_ip4hdr *arg_pHeader,
                                    ix_uint8 arg_hLen)
{

    ix_uint8	*pTemp, temp;

    if (arg_pHeader == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("IP header pointer is null\n")); 
    pTemp = (ix_uint8*) arg_pHeader;

    temp = *pTemp;
    temp = (temp & 0xf0) | arg_hLen;
    *pTemp = temp;
    return IX_SUCCESS;


}

ix_error ix_cc_fragv4_hdr_tos(ix_cc_fragv4_ip4hdr *arg_pHeader, ix_uint8 *arg_pTos)
{

    ix_uint8	*pTemp;

    if (arg_pHeader == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("IP header pointer is null\n")); 
    if (arg_pTos == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("TOS pointer is null\n")); 
    pTemp = (ix_uint8*)arg_pHeader + IX_CC_FRAGV4_IP_HEADER_TOS;
    *arg_pTos = *pTemp;

    return IX_SUCCESS;

}

/* This fn returns the total length field value in network byte order **/
ix_error ix_cc_fragv4_ip4hdr_len_read(ix_cc_fragv4_ip4hdr *arg_pHeader,
                                   ix_uint16 *arg_pLength)
{

    ix_uint8	*pTemp;
    ix_uint16    temp;
   
    ix_uint32		offset;

    if (arg_pHeader == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("IP header pointer is null\n")); 
    if (arg_pLength == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Length pointer is null\n")); 
    offset = IX_CC_FRAGV4_IP_HEADER_LEN;
    pTemp = (ix_uint8*)arg_pHeader + offset;
    
#ifdef SIM_NT
    IX_ERROR_CRT(_ix_cc_fragv4_netrw_read_int16(pTemp, &temp),
        IX_CC_FRAGV4_ERROR_IP4HDR_READ, 0);
#else
    temp = IX_RM_MEM_UINT16_READ((ix_uint16*)(ix_uint8*)pTemp);
#endif    
    *arg_pLength = temp;
    return IX_SUCCESS;


}


ix_error ix_cc_fragv4_ip4hdr_len_write(ix_cc_fragv4_ip4hdr *arg_pHeader,
                                    ix_uint16 arg_length)
{

    ix_uint8	*pTemp;
    ix_uint32		offset;

    if (arg_pHeader == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("IP header pointer is null\n")); 
    offset = IX_CC_FRAGV4_IP_HEADER_LEN;
    pTemp = (ix_uint8*)arg_pHeader + offset;
#ifdef SIM_NT
    IX_ERROR_CRT(_ix_cc_fragv4_netrw_write_int16(pTemp, arg_length),
      IX_CC_FRAGV4_ERROR_IP4HDR_WRITE, 0);
#else    
    IX_RM_MEM_UINT16_WRITE((ix_uint16*)((ix_uint8*)pTemp), 
       IX_HTON16(arg_length));
#endif    
    return IX_SUCCESS;

}

ix_error ix_cc_fragv4_ip4hdr_id(ix_cc_fragv4_ip4hdr *arg_pHeader,
                           ix_uint16 *arg_pId)
{

    ix_uint8	*pTemp;
    ix_uint16    temp;
    ix_uint32		offset;

    if (arg_pHeader == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("IP header pointer is null\n")); 
    if (arg_pId == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("ID pointer is null\n")); 
    offset = IX_CC_FRAGV4_IP_HEADER_ID;
    pTemp = (ix_uint8*)arg_pHeader + offset;

#ifdef SIM_NT
    IX_ERROR_CRT(_ix_cc_fragv4_netrw_read_int16(pTemp, &temp),
        IX_CC_FRAGV4_ERROR_IP4HDR_READ, 0);
#else    
    temp = IX_RM_MEM_UINT16_READ((ix_uint16*)((ix_uint8*)pTemp));
#endif    
    *arg_pId = temp;

    return IX_SUCCESS;

}

ix_error ix_cc_fragv4_ip4hdr_frag_off_read(ix_cc_fragv4_ip4hdr *arg_pHeader,
                                ix_uint16 *arg_pOff)
{

    ix_uint8	*pTemp;
    ix_uint16    temp;
    ix_int32	offset;

    if (arg_pHeader == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("IP header pointer is null\n")); 
    if (arg_pOff == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Offset pointer is null\n")); 
    offset = IX_CC_FRAGV4_IP_HEADER_FRAG_OFF;
    pTemp = (ix_uint8*)arg_pHeader + offset;
#ifdef SIM_NT
    IX_ERROR_CRT(_ix_cc_fragv4_netrw_read_int16(pTemp, &temp),
     IX_CC_FRAGV4_ERROR_IP4HDR_READ, 0);
#else    
    temp = IX_RM_MEM_UINT16_READ((ix_uint16*)((ix_uint8*)pTemp));
#endif   
    *arg_pOff = temp;
    return IX_SUCCESS;

}

ix_error ix_cc_fragv4_ip4hdr_frag_off_write(ix_cc_fragv4_ip4hdr *arg_pHeader,
                                         ix_uint16 arg_off)
{

    ix_uint8	*pTemp;
    ix_uint16	offset;

    if (arg_pHeader == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("IP header pointer is null\n")); 
    offset = IX_CC_FRAGV4_IP_HEADER_FRAG_OFF;
    pTemp = (ix_uint8*)arg_pHeader + offset;
#ifdef SIM_NT
    IX_ERROR_CRT(_ix_cc_fragv4_netrw_write_int16(pTemp, arg_off),
    IX_CC_FRAGV4_ERROR_IP4HDR_WRITE, 0);
#else    
    IX_RM_MEM_UINT16_WRITE((ix_uint16*)((ix_uint8*)pTemp), 
       IX_HTON16(arg_off)); 
#endif    
    return IX_SUCCESS;

}

ix_error ix_cc_fragv4_ip4hdr_ttl(ix_cc_fragv4_ip4hdr *arg_pHeader, 
                               ix_uint8 *arg_pTtl)
{
    ix_uint8	*pTemp;

    if (arg_pHeader == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("IP header pointer is null\n")); 
    if (arg_pTtl == NULL)
	    return IX_ERROR_NEW(IX_CC_ERROR_NULL,
                            ("TTL pointer is null\n")); 
    pTemp = (ix_uint8*)arg_pHeader + IX_CC_FRAGV4_IP_HEADER_TTL;

    *arg_pTtl = *pTemp;

    return IX_SUCCESS;
}

ix_error ix_cc_fragv4_ip4hdr_proto(ix_cc_fragv4_ip4hdr *arg_pHeader,
                                 ix_uint8 *arg_pProto)
{

    ix_uint8	*pTemp;

    if (arg_pHeader == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("IP header pointer is null\n")); 
    if (arg_pProto == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Protocol pointer is null\n")); 
    pTemp = (ix_uint8*)arg_pHeader + IX_CC_FRAGV4_IP_HEADER_PROTO;

    *arg_pProto = *pTemp;

    return IX_SUCCESS;


}

ix_error ix_cc_fragv4_ip4hdr_checksum_read(ix_cc_fragv4_ip4hdr *arg_pHeader,
						ix_uint16 *arg_pChecksum)
{
    ix_uint8	*pTemp;
    ix_uint16    temp;
    ix_uint32    offset;

    if (arg_pHeader == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("IP header pointer is null\n")); 
    if (arg_pChecksum == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Checksum pointer is null\n")); 
    offset = IX_CC_FRAGV4_IP_HEADER_SUM;
    pTemp = (ix_uint8*)arg_pHeader + offset;
#ifdef SIM_NT
    IX_ERROR_CRT(_ix_cc_fragv4_netrw_read_int16(pTemp, &temp),
     IX_CC_FRAGV4_ERROR_IP4HDR_READ, 0);
#else    
    temp = IX_RM_MEM_UINT16_READ((ix_uint16*)((ix_uint8*)pTemp));
#endif    
    *arg_pChecksum = temp;

    return IX_SUCCESS;



}

ix_error ix_cc_fragv4_ip4hdr_checksum_write( ix_cc_fragv4_ip4hdr *arg_pHeader,
				 ix_uint16 arg_checksum)
{
    ix_uint8	*pTemp;
    ix_uint32   offset;

    if (arg_pHeader == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("IP header pointer is null\n")); 
    offset = IX_CC_FRAGV4_IP_HEADER_SUM;
    pTemp = (ix_uint8 *)arg_pHeader + offset;
#ifdef SIM_NT
    IX_ERROR_CRT(_ix_cc_fragv4_netrw_write_int16(pTemp, arg_checksum),
     IX_CC_FRAGV4_ERROR_IP4HDR_WRITE, 0);
#else
    IX_RM_MEM_UINT16_WRITE((ix_uint16*)((ix_uint8*)pTemp), 
       IX_HTON16(arg_checksum));
#endif   
    return IX_SUCCESS;

}

ix_error ix_cc_fragv4_ip4hdr_src_32(ix_cc_fragv4_ip4hdr *arg_pHeader,
                                  ix_uint32 *arg_pSrc)
{

    ix_uint8	*pTemp;
    ix_uint32    temp;
    ix_uint32         offset;

    if (arg_pHeader == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("IP header pointer is null\n")); 
    if (arg_pSrc == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Src pointer is null\n")); 
    offset = IX_CC_FRAGV4_IP_HEADER_SRC;
    pTemp = (ix_uint8*)arg_pHeader + offset;
    
    IX_ERROR_CRT(_ix_cc_fragv4_netrw_read_int32(pTemp, &temp),
       IX_CC_FRAGV4_ERROR_IP4HDR_READ, 0);
    
    *arg_pSrc = temp;

    return IX_SUCCESS;


}

ix_error ix_cc_fragv4_ip4hdr_dest_32(ix_cc_fragv4_ip4hdr *arg_pHeader,
                                    ix_uint32 *arg_pDest)
{

    ix_uint8	*pTemp;
    ix_uint32    temp;
    ix_uint32 offset;

    if (arg_pHeader == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("IP header pointer is null\n")); 
    if (arg_pDest == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Dest pointer is null\n")); 
    offset = IX_CC_FRAGV4_IP_HEADER_DEST;
    pTemp = (ix_uint8*)arg_pHeader + offset;
    
    IX_ERROR_CRT(_ix_cc_fragv4_netrw_read_int32(pTemp, &temp),
     IX_CC_FRAGV4_ERROR_IP4HDR_READ, 0);
   
    *arg_pDest = temp;

    return IX_SUCCESS;


}

ix_error ix_cc_fragv4_ip4hdr_data_len(ix_cc_fragv4_ip4hdr *arg_pHeader,
                                   ix_uint16 *arg_pDataLen)
{

    ix_uint16    hlen, tlen;
   

    if (arg_pHeader == NULL)
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("IP header pointer is null\n")); 
    if(arg_pDataLen == NULL)
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Data length pointer is null\n")); 

    IX_ERROR_CRT(ix_cc_fragv4_ip4hdr_read_hlen_in_octets(arg_pHeader,&hlen),
                  IX_CC_FRAGV4_ERROR_IP4HDR_READ, 0);
   
    IX_ERROR_CRT(ix_cc_fragv4_ip4hdr_len_read(arg_pHeader,&tlen),
                      IX_CC_FRAGV4_ERROR_IP4HDR_READ, 0);
    
    *arg_pDataLen = tlen - hlen;

    return IX_SUCCESS;


}

ix_error ix_cc_fragv4_ip4hdr_payload(ix_cc_fragv4_ip4hdr *arg_pHeader,
                                   ix_uint16 **arg_ppData)
{
    ix_uint8    hlen;
    ix_uint16     *pTemp;

    if (arg_pHeader == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("IP header pointer is null\n")); 
    if (arg_ppData == NULL)
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Data pointer is null\n"));
 
    IX_ERROR_CRT(ix_cc_fragv4_ip4hdr_read_hlen_in_w32(arg_pHeader,&hlen),
                   IX_CC_FRAGV4_ERROR_IP4HDR_READ, 0);
    
    pTemp = ((ix_uint16*)arg_pHeader) + (hlen << 1);
    *arg_ppData = pTemp;

    return IX_SUCCESS;
}

/* This function returns arg_pDestHLen in n/w byte order */
ix_error ix_cc_fragv4_ip4hdr_copy_options(ix_cc_fragv4_ip4hdr *arg_pSrc,
			 ix_cc_fragv4_ip4hdr* arg_pDest,
                        ix_int32 arg_isFirst, ix_uint16 *arg_pDestHLen)
{

    ix_uint16    len, hlen;
    ix_uint8     *pSrcTmp, *pDestTmp, optCode, optLen;
   

   
    if (arg_pSrc == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Src pointer is null\n")); 
    if (arg_pDest == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Dest pointer is null\n")); 
    if (arg_pDestHLen == NULL)
	    return IX_ERROR_LOCAL(IX_CC_ERROR_NULL,
                            ("Dest Hlen  pointer is null\n")); 

    IX_ERROR_CRT(ix_cc_fragv4_ip4hdr_read_hlen_in_octets(arg_pSrc, &hlen),
                    IX_CC_FRAGV4_ERROR_IP4HDR_READ, 0);
   
    len = hlen - IX_CC_FRAGV4_IP_HEADER_MIN_SIZE;
    pSrcTmp = (ix_uint8*)arg_pSrc +  IX_CC_FRAGV4_IP_HEADER_MIN_SIZE;
    pDestTmp = (ix_uint8*)arg_pDest +  IX_CC_FRAGV4_IP_HEADER_MIN_SIZE;
    optLen = 0;
    for (; len > 0; len -= optLen)
    {
        optCode = *pSrcTmp;	
        if ( optCode == IX_CC_FRAGV4_IP_HEADER_OPT_EOL )
	    break;
        else if ( optCode == IX_CC_FRAGV4_IP_HEADER_OPT_NOP )
	    {
	        *pDestTmp = optCode;
	        pDestTmp++;
            pSrcTmp++;
            optLen = 1;
            continue;
        }
        else 
        {
            optLen = *(pSrcTmp+1);
            if (optLen > len )
            {           
                optLen = len;
            }
            if ( IX_CC_FRAGV4_IP_HEADER_OPT_COPY(optCode) || arg_isFirst )
            {
                ix_ossl_memcpy(pDestTmp, pSrcTmp, optLen);
                pDestTmp += optLen; 
                pSrcTmp += optLen; 
            }
        }
    }
    /*
     * if the dest pointer ends at a non-32-bit-aligned location, pad the
     * header with 0
     */
    while (((ix_uint32)pDestTmp) & 0x03)
    {
        *pDestTmp = IX_CC_FRAGV4_IP_HEADER_OPT_EOL;
        pDestTmp++;
        optLen++;
    }
    *arg_pDestHLen = hlen;
    hlen = (IX_CC_FRAGV4_IP_HEADER_MIN_SIZE / 4) + ((optLen + 3) >> 2);
    
    /* fix destination header length */
    IX_ERROR_CRT(ix_cc_fragv4_ip4hdr_hlen_write(arg_pDest, hlen),
                          IX_CC_FRAGV4_ERROR_IP4HDR_READ, 0);
    return IX_SUCCESS;
} /* ix_cc_fragv4_ip4hdr_copy_options() */

