/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.

 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2000 Network Processor
 *
 * = LIBRARY
 *   Fragmentation    
 *
 *
 *      
 *
 * = FILENAME
 *      netrw.c
 *
 * = DESCRIPTION
 *   This file contains helper functions    
 *
 *
 *
 * = CREATION TIME
 *      06/15/2002 8:00:24 PM
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 */

static const char s_FileIdentifier[] = "$Id: netrw.c,v 1.13 2003/11/04 21:52:02 rranjeet Exp $";


/**
 * System defined include files required.
 */
/**
 * User defined include files required.
 */
#include "ix_ossl.h"
#include "cc/fragmentation/internal/internal-netrw.h"


/** Read 16-bit integer from a location in a network-byte-order 
   (big endian) data block
    
    @param      arg_pInt	pointer to the byte 0 of the integer
    @param      arg_pVal	where to return the 16-bit integer
    @returns an error token, or IX_SUCCESS for success.
*/
ix_error _ix_cc_fragv4_netrw_read_int16( ix_uint8 *arg_pInt, ix_uint16 *arg_pVal)
{
    ix_uint16	temp;

    if (arg_pInt == NULL)
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL, ("Pointer is null\n"));
    if (arg_pVal == NULL)
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL, ("Val pointer is null\n"));

    temp = ((ix_uint16)(*arg_pInt)) << 8;
    temp |= (ix_uint16)(*(arg_pInt+1));
	
    *arg_pVal = temp;

    return IX_SUCCESS;
    
}

/** Read 32-bit integer from a location in a network-byte-order (big endian) 
    data block
    
    @param      arg_pInt	pointer to the byte 0 of the integer
    @param      arg_pVal	where to return the 16-bit integer
    @returns an error token, or IX_SUCCESS for success.
*/
ix_error _ix_cc_fragv4_netrw_read_int32( ix_uint8 *arg_pInt, ix_uint32 *arg_pVal)
{
    ix_uint32	temp;
    ix_uint16 tempVal;
    ix_uint16 tempNext;
    if (arg_pInt == NULL)
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL, ("Pointer is null\n"));
    if (arg_pVal == NULL)
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL, ("Val pointer is null\n"));

#ifdef SIM_NT
    temp =  ((ix_uint32)(*arg_pInt)) << 24;
    temp |= ((ix_uint32)(*(arg_pInt+1))) << 16;
    temp |= ((ix_uint32)(*(arg_pInt+2))) << 8;
    temp |= (ix_uint32)(*(arg_pInt+3));
	
    *arg_pVal = temp;
#else
    
    tempVal = IX_NTOH16(IX_RM_MEM_UINT16_READ(((ix_uint16 *)(ix_uint8 *)arg_pInt)));
    tempNext = IX_NTOH16(IX_RM_MEM_UINT16_READ(((ix_uint16 *)(ix_uint8 *)arg_pInt + 2)));
    temp = (tempVal << 16) | tempNext;
    *arg_pVal = IX_HTON32(temp);
#endif
    return IX_SUCCESS;
    
}

/** Write 16-bit integer in network-byte-order (big endian) 
    
    @param      arg_pInt 		pointer to the byte 0 of the integer
    @param      arg_Val 		value to write to the 16-bit integer
    @returns an error token,or IX_SUCCESS  for success.
*/
ix_error _ix_cc_fragv4_netrw_write_int16( ix_uint8 *arg_pInt, ix_uint16 arg_Val)
{
    ix_uint8	temp;

    if (arg_pInt == NULL)
        return IX_ERROR_LOCAL(IX_CC_ERROR_NULL, ("Pointer is null\n"));
   

    temp = (ix_uint8)(arg_Val >> 8);
    *arg_pInt = temp;
    temp = (ix_uint8)(arg_Val & 0xff );
    *(arg_pInt+1) = temp;

    return IX_SUCCESS;
}


