/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      sw.h
 *
 * = DESCRIPTION
 *      Common header file for implementations of the Software Lookup Library.
 *      Each software algorithm must follow the guidelines in the Lookup
 *      Library document, as well as the guidelines in this file.
 *
 * ============================================================================
 * $Id: sw.h,v 1.4 2002/09/18 17:03:51 lbmiller Exp $
 */

#if !defined(__IX_LKUP_SW_H__)
#define __IX_LKUP_SW_H__

#include "ix_lkup.h"

#if defined(__cplusplus)
extern "C" {
#endif /* __cplusplus */

/*
 * NAME: _IX_LKUP_SW_CURRENT_VERSION
 *
 * DESCRIPTION: The version of the Lookup Library currently supported by the
 * Software library.  When the Software library and its related algorithms 
 * are updated to support new versions, this value should change to reflect
 * the currently supported version.
 */
#define _IX_LKUP_SW_CURRENT_VERSION 1

/*
 * The Lookup Library document requires an implementation to provide a
 * structure of type ix_s_lkup_table.  The shared portion of the software
 * implementations requires that each implementation provides a structure of
 * type ix_s_lkup_sw_table, which contains the requisite ix_s_lkup_table.
 * In addition to the structure as defined by the Lookup Library,
 * ix_s_lkup_sw_table adds a function pointer for destroying a table.
 * Software implementations are responsible for initializing the entire
 * ix_s_lkup_sw_table structure, including the new ltabDestroyTable member.
 * The ltabDestroyTable function is called when the caller invokes
 * IX_LKUP_DESTROY_TABLE().
 */
typedef struct ix_s_lkup_sw_table
{
    struct ix_s_lkup_table      hTable[1];
    ix_lkup_destroy_table       ltabDestroyTable;
} *ix_lkup_sw_table;

#if defined(__cplusplus)
} /* extern "C" */
#endif /* __cplusplus */

#endif /*__IX_LKUP_SW_H__*/
