/*===========================================================================
 * = COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a licence agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 * ===========================================================================
 */

/* =========================================================================*/
/**
 * @file   ix_cc_msup.h
 *
 * Message Support Library Header
 *
 * Contains public API definitions for the message support library.
 **/
/*=========================================================================*/
/* $Id: ix_cc_msup.h,v 1.6 2002/12/12 22:53:07 rjpetri Exp $ */

/** Ensure the file is only included once. */
#if !defined(__IX_CC_MSUP_H_)
#define __IX_CC_MSUP_H_

#if defined(__cplusplus)
extern "C" {
#endif /* __cplusplus */

#include "ix_cc_error.h"
#include "ix_ossl.h"
#include "ix_rm.h"

#if !defined(IX_EXCLUDE_CCI)
/*#include "ix_cci.h"*/
#endif
#include "bindings.h"

/* ix_e_msup_error */
/* ------------------------ */
/* 
 * Enumeration of message support specific errors.
*/
typedef enum ix_e_msup_error
{
    IX_MSUP_ERROR_SHUTDOWN = IX_ERROR_MAKE_GROUP(CC_MSUP),
    IX_MSUP_ERROR_NULL_EE_CONTEXT,
    IX_MSUP_ERROR_NULL_BUFF_DATA,
    IX_MSUP_ERROR_INVAL_BUFF_SIZE,
    IX_MSUP_ERROR_MSG_SIZE_MISMATCH,
    IX_MSUP_ERROR_ENC_INVAL_ERR,
    IX_MSUP_ERROR_ENC_INVAL_BUFFSIZE
} ix_msup_error;

/**
 * Message help library callback signature.
 *
 * arg_Result is the ix_error from the called function.
 * arg_pContext is the user specified context
 * arg_Msg is the message.
 * arg_Msg_Len is the length of the message
 */
typedef ix_error (*ix_cc_msghlp_callback)(
	    ix_error arg_Result,
	    void *arg_pContext,
	    void *arg_Msg,
	    ix_uint32 arg_Msg_Len);

IX_EXPORT_FUNCTION
ix_error
ix_cc_msup_init(ix_buffer_free_list_handle arg_hMsgFreelist,
                ix_uint32 arg_msgBuffSize,
                void **arg_context);

IX_EXPORT_FUNCTION
ix_error
ix_cc_msup_fini(void *arg_context);

IX_EXPORT_FUNCTION
ix_error
ix_cc_msup_send_msg(ix_uint32 arg_commID,
                    ix_uint32 arg_msgType,
                    void *arg_msg,
                    ix_uint32 arg_msgLen);
IX_EXPORT_FUNCTION
ix_error
ix_cc_msup_send_async_msg(ix_uint32 arg_commID,
                          ix_cc_msghlp_callback arg_pCallback,
                          void *arg_pContext,
                          ix_uint32 arg_msgType,
                          void *arg_msg,
                          ix_uint32 arg_msgLen);

IX_EXPORT_FUNCTION
ix_error
ix_cc_msup_send_sync_msg(ix_uint32 arg_commID,
                         ix_uint32 arg_msgType,
                         void *arg_msg,
                         ix_uint32 arg_msgLen,
                         void **arg_retMsg,
                         ix_uint32 *arg_retMsgLen);

IX_EXPORT_FUNCTION
ix_error
ix_cc_msup_send_bcast_msg(ix_uint32 *arg_commIdList,
                          ix_uint32 arg_msgType,
                          void *arg_msg,
                          ix_uint32 arg_msgLen);

IX_EXPORT_FUNCTION
ix_error
ix_cc_msup_send_reply_msg(void *arg_pContext,
                          void *arg_msg,
                          ix_uint32 arg_msgLen,
                          ix_error arg_retError);

IX_EXPORT_FUNCTION
ix_error
ix_cc_msup_extract_msg(ix_buffer_handle arg_mbuf,
                       void **arg_usrMsg,
                       void **arg_context);

#define IX_MSUP_EXTRACT_MSG(__buf, __msg, __ctx) \
     ix_cc_msup_extract_msg(__buf, __msg, __ctx)

#if defined(__cplusplus)
} /* extern "C" */
#endif /* __cplusplus */
#endif /* __MSUP_H_ */
