/*
 * ===========================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      ix_cc_genex.h
 *
 * = DESCRIPTION
 *      Main header file for the generation / index storage mechanism.
 *
 * ===========================================================================
 * $Id: ix_cc_genex.h,v 1.3 2002/09/18 17:01:53 lbmiller Exp $
 */

#ifndef __IX_CC_GENEX_H__
#define __IX_CC_GENEX_H__

#include "ix_cc_error.h"
#include "ix_types.h"

typedef ix_uint32 ix_genex;
typedef struct ix_s_genex_iter* ix_genex_iter;
typedef struct ix_s_genex_pool*	ix_genex_pool;

#define IX_GENEX_NULL 0

enum ix_genex_error {
    IX_CC_GENEX_ERROR_BASE = IX_ERROR_MAKE_GROUP(CC_GENEX),

    /*
     * An invalid parameter was provided to the genex api
     */
    IX_CC_GENEX_ERROR_PARAM,

    IX_CC_GENEX_ERROR_FINGERPRINT,

    /*
     * An invalid operation was performed with an iterator.
     */
    IX_CC_GENEX_ERROR_ITER
};

ix_error 
ix_genex_init(
    ix_uint32 quantity_factor, 
    ix_uint32 element_size, 
    ix_genex_pool *poolp
    );

ix_error
ix_genex_new(
    ix_genex_pool pool,
    ix_genex *genexp,
    void **voidp
    );

ix_error
ix_genex_lookup(
    ix_genex_pool pool,
    ix_genex genex,
    void **voidp
    );

ix_error
ix_genex_bump(
    ix_genex_pool pool,
    ix_genex old,
    ix_genex *newp
    );

ix_error
ix_genex_exists(
    ix_genex_pool pool,
    ix_genex genex,
    int *existsp
    );

/*
 *
 * following is a typical use of an iterator:
 *      ix_genex genex;
 *      void *data;
 *      ix_genex_new_iter(pool, &iter);
 *      ix_genex_get_first(iter, &genex, &data);
 *      while (genex != ix_genex_null) {
 *              // do something...
 *              ix_genex_get_next(iter, &genex, &data);
 *      }
 *      ix_genex_del_iter(iter);
 */

/*
 * ix_genex_new_iter creates a new iterator which will report all the existing
 * items in the pool by means of ix_genex_get_next.
 */
ix_error
ix_genex_new_iter(
    ix_genex_pool pool,
    ix_genex_iter *iterp
    );

/*
 * ix_genex_get_first uses an iterator created from ix_genex_new_iter and
 * reports the first item in the pool.  If there are no items to 
 * report, genexp will contain IX_GENEX_NULL and voidp will contain NULL.  
 */
ix_error
ix_genex_get_first(
    ix_genex_iter iter,
    ix_genex *genexp,
    void **voidp
    );

/*
 * ix_genex_get_next uses an iterator created from ix_genex_new_iter and
 * reports all items in the pool, one per call.  If there are no more items to 
 * report, genexp will contain IX_GENEX_NULL and voidp will contain NULL.  If 
 * the iterator is used again after either ix_genex_get_first or 
 * ix_genex_get_next reports that there are no more items, ix_genex_get_next 
 * will return an error; valid operations at that point are ix_genex_get_first
 * or ix_genex_del_iter.
 */
ix_error
ix_genex_get_next(
    ix_genex_iter iter,
    ix_genex *genexp,
    void **voidp
    );

/*
 * ix_genex_del_iter cleans up an iterator created by ix_genex_new_iter.  An
 * iterator may be deleted at any time, no matter how many times 
 * ix_genex_get_next has been called.  Do not delete the same iterator twice.
 */
ix_error
ix_genex_del_iter(
    ix_genex_iter iter
    );

ix_error
ix_genex_del(
    ix_genex_pool pool,
    ix_genex genex
    );

ix_error
ix_genex_fini(
    ix_genex_pool pool
    );


#endif /* __IX_CC_GENEX_H__ */
