/*
 * ===========================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      ix_cc_genexts.h
 *
 * = DESCRIPTION
 *      Main header file for the threadsafe extensions to the generation / 
 *      index storage mechanism.
 *
 * ===========================================================================
 * $Id: ix_cc_genexts.h,v 1.2 2002/09/18 17:01:53 lbmiller Exp $
 */

#ifndef __IX_CC_GENEX_TS_H__
#define __IX_CC_GENEX_TS_H__

#include "cc/ix_cc_genex.h"

typedef struct ix_s_genex_ts_pool* ix_genex_ts_pool;
typedef struct ix_s_genex_ts_iter* ix_genex_ts_iter;

ix_error 
ix_genex_ts_init(
    unsigned int quantity_factor, 
    unsigned int element_size, 
    ix_genex_ts_pool *poolp
    );

ix_error
ix_genex_ts_new(
    ix_genex_ts_pool pool,
    ix_genex *genexp,
    void **voidp
    );

ix_error
ix_genex_ts_lookup(
    ix_genex_ts_pool pool,
    ix_genex genex,
    void **voidp
    );

ix_error
ix_genex_ts_bump(
    ix_genex_ts_pool pool,
    ix_genex old,
    ix_genex *newp
    );

ix_error
ix_genex_ts_exists(
    ix_genex_ts_pool pool,
    ix_genex genex,
    int *existsp
    );

ix_error
ix_genex_ts_new_iter(
    ix_genex_ts_pool pool,
    ix_genex_ts_iter *iterp
    );

ix_error
ix_genex_ts_get_first(
    ix_genex_ts_iter iter,
    ix_genex *genexp,
    void **voidp
    );

ix_error
ix_genex_ts_get_next(
    ix_genex_ts_iter iter,
    ix_genex *genexp,
    void **voidp
    );

ix_error
ix_genex_ts_del_iter(
    ix_genex_ts_iter iter
    );

ix_error
ix_genex_ts_del(
    ix_genex_ts_pool pool,
    ix_genex genex
    );

ix_error
ix_genex_ts_lock(
    ix_genex_ts_pool pool
    );

ix_error
ix_genex_ts_trylock(
    ix_genex_ts_pool pool,
    int *busyp
    );

ix_error
ix_genex_ts_unlock(
    ix_genex_ts_pool pool
    );

ix_error
ix_genex_ts_fini(
    ix_genex_ts_pool pool
    );

#endif /* __IX_CC_GENEX_TS_H__ */
