/*
**			INTEL CORPORATION PROPRIETARY INFORMATION
**	This software is supplied under the terms of a license agreement or
**	nondisclosure agreement with Intel Corporation and may not be copied
**	or disclosed except in accordance with the terms of that agreement.
**		Copyright (C) 2000,2001 Intel Corporation. All rights Reserved.
*/

#define	IX_ERROR_FILE_IDENT	"$Id: error.c,v 1.13 2003/06/27 20:35:31 ktseng Exp $"

#define	IX_ERROR_DEBUG		0

#if defined(__linux__)

#include "ix_ossl.h"
#include "ix_cc_error.h"
#include "ix_os_type.h"

#else

#include "ix_cc_error.h"
#include "ix_os_type.h"
#include "ix_ossl.h"

#endif /* __linux__ */

/*
 * VxWorks has trouble making dependencies, so we have to revert to using
 * the original #defines rather than the preferred _IX_OS_TYPE_
 */
#if (defined(__linux__) && !defined(__KERNEL__))
#define _IX_ERROR_USE_PTHREADS
#include <pthread.h>
#endif

#include "cc/internal/internal_error.h"
 
#if (_IX_OS_TYPE_ == _IX_OS_WIN32_)
	/*
	 * Windows renamed vsnprintf() to _vsnprintf().
	 */
#define VSNPRINTF(__b, __s, __f, __a) _vsnprintf(__b, __s, __f, __a)
#elif (_IX_OS_TYPE_ == _IX_OS_VXWORKS_)
	/*
	 * VxWorks doesn't have vsnprintf(), so use the closest thing available...
	 */
#define VSNPRINTF(__b, __s, __f, __a) vsprintf(__b, __f, __a)
#else
#define VSNPRINTF(__b, __s, __f, __a) vsnprintf(__b, __s, __f, __a)
#endif

#define	ALIGNS		2
#define	ALIGNM		((1<<ALIGNS)-1)
#define	ALIGNUP(__v)	((((__v)+ALIGNM)>>ALIGNS)<<ALIGNS)

#if IX_ERROR_DEBUG
#define	DPRINTF(a)	_IX_ERROR_WRAP(fprintf a)
#endif

#ifndef	DPRINTF
#define	DPRINTF(a)	_IX_ERROR_WRAP(;)
#endif

#if IX_ERROR_DEBUG>1
#define	DPRINTF2(a)	_IX_ERROR_WRAP(fprintf a)
#endif

#ifndef	DPRINTF2
#define	DPRINTF2(a)	_IX_ERROR_WRAP(;)
#endif

/**
 * MACRO NAME: IX_ERROR_SET_MODE_CHAINED
 *
 * DESCRIPTION: This macro sets the error mode to chained for an error token.
 *
 * @Param:  - IN arg_Error -   error token of type ix_error.
 *
 * @Return: Returns the new value of the error token.
 */
#define IX_ERROR_SET_MODE_CHAINED( \
                            arg_Error \
                          ) \
                          (ix_error)IX_SET_BIT_FIELD32(arg_Error, 1, 31U, 31U)

/**
 * MACRO NAME: IX_ERROR_SET_MODE_FLAT
 *
 * DESCRIPTION: This macro sets the error mode to flat (no chaining).
 *
 * @Param:  - IN arg_Error -   error token of type ix_error.
 *
 * @Return: Returns the new value of the error token.
 */
#define IX_ERROR_SET_MODE_FLAT( \
                            arg_Error \
                          ) \
                          (ix_error)IX_SET_BIT_FIELD32(arg_Error, 0, 31U, 31U)

/** @file source/error.c
    @brief Error Reporting.
*/

#if defined(_IX_ERROR_USE_PTHREADS)
/** Thread-specific Data Key.
 */
pthread_key_t		ix_error_key = 0;

/** Find Thread-Specific Data.
   @returns pointer to thread-specific error area
 */
ix_error_pti           *
ix_error_get_pti(void)
{
    int                     rv;
    size_t                  size;
    ix_error_pti           *pti;

    while (ix_error_key == 0)
    {
        rv = pthread_key_create(&ix_error_key, 0);
        if (rv)
        {
            ix_ossl_message_log(
                    "initialization of error facility failed:\n"
                    "\tunable to create key for per-thread data\n"
                    "\t[%d] %s\n", errno, strerror(errno));
            abort();
        }
    }

    DPRINTF2((stderr, "ix_error: key is %u\n", ix_error_key));

    pti = pthread_getspecific(ix_error_key);
    if (NULL == pti)
    {
        size = sizeof(ix_error_pti);
        pti = (ix_error_pti *) ix_ossl_malloc(size);
        if (NULL == pti)
        {
            ix_ossl_message_log(
                    "initialization of error facility failed:\n"
                    "\tunable to allocate %d bytes for my per-thread data\n"
                    "\t[%d] %s\n", size, errno, strerror(errno));
            abort();
        }

        ix_ossl_memset(pti, 0, size);

        rv = pthread_setspecific(ix_error_key, pti);
        if (rv)
        {
            ix_ossl_message_log(
                    "initialization of error facility failed:\n"
                    "\tunable to set my thread-specific data\n"
                    "\t[%d] %s\n", errno, strerror(errno));
            abort();
        }
    }

    DPRINTF2((stderr, "ix_error: pti is %p\n", pti));
    return pti;
}

#else /*not using pthread support*/

/*
 * sorry, everyone shares, for now.
 */
ix_error_pti           *
ix_error_get_pti(void)
{
    static ix_error_pti *ix_error_the_pti = NULL;
   
    if(ix_error_the_pti == NULL){
        ix_error_the_pti = (ix_error_pti*)ix_ossl_malloc(sizeof(ix_error_pti));
        if(ix_error_the_pti != 0){
            memset(ix_error_the_pti, 0, sizeof(ix_error_pti));
        }
    }
    return ix_error_the_pti;
}

#endif

/** un-initialize error system.
 */
void
ix_error_fini(void)
{
    ix_error_pti           *pti;

    pti = ix_error_get_pti();
    if (NULL == pti)
	return;

#if defined(_IX_ERROR_USE_PTHREADS)
    pthread_setspecific(ix_error_key, 0);
#endif

    DPRINTF((stderr, "ix_error_fini: pti->pool_ptrs is %p\n",
                pti->pool_ptrs));
    if (NULL != pti->pool_ptrs)
    {
        ix_ossl_free(pti->pool_ptrs);
        pti->pool_ptrs = NULL;
    }

    if (pti->tmp_buf != NULL)
    {
        ix_ossl_free(pti->tmp_buf);
        pti->tmp_buf = NULL;
    }

#if defined(_IX_ERROR_USE_PTHREADS)
    ix_ossl_free(pti);
#endif
}

/** initialize error system.
 */
void
ix_error_init(int count, int length)
{
    int                     ix;
    ix_error_hunk         **hunkpp;
    ix_error_hunk          *hunkp;
    ix_error_pti           *pti;
    size_t		    asize;
    size_t		    aincr;
    char		   *alloc;

    DPRINTF2((stderr, "ix_error_init: count is %d length, is %d\n", count, length));

    pti = ix_error_get_pti();
    if (pti == NULL)
    {
        ix_ossl_message_log("Could not get global memory for errors.");
        return;
    }
    DPRINTF2((stderr, "ix_error_init: pti is %p\n", pti));

    /* never reduce pool size */
    if (length < pti->data_size)
        length = pti->data_size;
    if (count < pti->pool_size)
        count = pti->pool_size;

    /* guarantee at least minimums */
    if (length < IX_ERROR_LENGTH_MIN)
        length = IX_ERROR_LENGTH_MIN;
    if (count < IX_ERROR_COUNT_MIN)
        count = IX_ERROR_COUNT_MIN;

    /* prevent going beyond maximums */
    if (count > IX_ERROR_COUNT_MAX)
        count = IX_ERROR_COUNT_MAX;
    if (length > IX_ERROR_LENGTH_MAX)
        length = IX_ERROR_LENGTH_MAX;

    /* see if current pool already big enough */
    if ((pti->pool_ptrs != NULL) &&
        (pti->data_size >= length) &&
        (pti->pool_size >= count))
    {
        DPRINTF2((stderr, "ix_error_init: enough already\n"));
        return;
    }

    DPRINTF2((stderr, "ix_error_init: %sconfigure request received\n"
                "\t%sing maximum chained error count to %d\n"
                "\t%sing maximum error string length to %d\n"
                , (pti->data_size || pti->pool_size) ? "re" : ""
                , pti->pool_size ? "expand" : "sett"
                , count
                , pti->data_size ? "expand" : "sett"
                , length
             ));

    DPRINTF2((stderr, "ix_error_init: count is %d, length is %d\n", count, length));

    /*
     * Free any existing memory.
     */
    if (pti->pool_ptrs != NULL)
    {
        DPRINTF2((stderr, "ix_error_init: hunkpp is %p\n", hunkpp));
        ix_ossl_free(pti->pool_ptrs);
        pti->pool_ptrs = NULL;
    }

    if (pti->tmp_buf != NULL)
    {
        ix_ossl_free(pti->tmp_buf);
        pti->tmp_buf = NULL;
    }

    hunkpp = pti->pool_ptrs;

    /* yes, this could be reduced to math, eliminating
    ** the loop ... but keep the loop anyway, so our
    ** adjustments to asize here exactly match our
    ** adjustments to aincr below.
    */
    asize = count * sizeof (ix_error_hunk *);
    for (ix = 0; ix < count; ix++)
    {
        asize = ALIGNUP(asize);
        asize += sizeof (ix_error_hunk) + length;
    }

    alloc = (char *)ix_ossl_malloc(asize);
    if (NULL == alloc)
    {
        ix_ossl_message_log(
                "ix_error_init: unable to allocate %d bytes of memory\n"
                "\tto store %d errors, each with a %d-byte message buffer."
                , asize
                , count
                , length
                );
        return;			/* someone else picks up the pieces? */
    }
    hunkpp = (ix_error_hunk **)(void *)alloc;
    aincr = count * sizeof(ix_error_hunk *);

    DPRINTF2((stderr, "ix_error_init: hunkpp is %p\n", hunkpp));
    for (ix = 0; ix < count; ix++) {
	aincr = ALIGNUP(aincr);
	hunkp = (ix_error_hunk *)(void *)(alloc + aincr);
	aincr += sizeof(ix_error_hunk) + length;
	if (aincr > asize)
    {
        ix_ossl_message_log(
                "ix_error_init: internal allocation assertion failure\n"
                "\tallocated %d bytes for %d hunks with %d byte messages\n"
                "\tbut it appears that hunk %d extends up to offset %d"
                , asize, count, length
                , ix, aincr
                );
        return;			/* someone else picks up the pieces? */
    }

	hunkp->ref = 0;
	hunkp->self = ix + IX_ERROR_BASE + IX_ERROR_INCR;
	hunkp->level = 0;
	hunkp->code = 0;
	hunkp->buf[0] = 0;

	hunkpp[ix] = hunkp;
	DPRINTF2((stderr, "ix_error_init: hunkpp[%d] is %p\n", ix, hunkp));
	DPRINTF2((stderr, "ix_error_init: index %04X self is %08X\n", ix, hunkp->self));
    }

    pti->tmp_buf = (char*) ix_ossl_malloc(length);

    pti->data_size = length;
    pti->pool_size = count;
    pti->pool_next = 0;
    pti->pool_ptrs = hunkpp;
}

/** set up new error hunk.

    this function locates the error hunk storage, and sets up
    its contents appropriately, up to but not including filling
    in the text string area, which it initializes to a zero
    length string. The length of the longest string that can be
    stored in the hunk's buffer (not including the zero at the
    end of the string) is returned through the size return value.

    this function always returns a nonzero error code that represents
    the error described in its parameters.

    @param	retp	where to return address of error hunk
    @param	sizep	where to return maximum string length
    @param	level	error severity
    @param	code	error number
    @param	ref	subsidiary error
    @returns an error token.
*/
ix_error
ix_error_build(ix_error_hunk **retp,
	       size_t *sizep,
	       ix_error_number code,
	       ix_error_level level,
	       ix_error ref)
{
    ix_error_hunk         **hunkpp;
    ix_error_hunk          *hunkp;
    int                     ix;
    char                   *buf;
    size_t                  size;

    ix_error_pti           *pti = ix_error_get_pti();
    if (pti == NULL)
    {
        goto flat;
    }

    IX_ERROR_DCHECK_ARG_NULL(1, retp);

    DPRINTF2((stderr, "ix_error_build: pti is %p\n", pti));

    if (ref != 0)
    {
        if (level == 0)
        {
            level = IX_ERROR_GET_LEVEL(ref);
        }
        if (code == 0)
        {
            code = IX_ERROR_GET_CODE(ref);
        }
    }

    if (level == 0)
	level = IX_ERROR_LEVEL_WARNING;
    if (code == 0)
	code = IX_OSSL_ERROR_INVALID_OPERATION;

    hunkpp = pti->pool_ptrs;

    if (NULL == hunkpp)
    {
        goto flat;
    }

    DPRINTF2((stderr, "ix_error_build: hunkpp is %p\n", hunkpp));

    ix = pti->pool_next;

    DPRINTF2((stderr, "ix_error_build: ix is %d\n", ix));

    hunkp = hunkpp[ix];

    if (NULL == hunkp) {
        goto flat;
    }

    DPRINTF2((stderr, "ix_error_build: hunkpp[%d] is %p\n", ix, hunkp));

    hunkp->ref = ref;
    hunkp->self += IX_ERROR_INCR;
    IX_ERROR_SET_MODE_CHAINED(hunkp->self);
    hunkp->level = level;
    hunkp->code = code;
    hunkp->buf[0] = 0;

    buf = hunkp->buf;
    size = pti->data_size;

    DPRINTF2((stderr, "ix_error_build: text buf for index %d is %d bytes at %p\n"
	   , ix, size, buf
	));

    DPRINTF2((stderr, "ix_error_build: returning handle %08X for index %04X\n",
	   hunkp->self, ix));

    ix = ix + 1;
    if (ix >= pti->pool_size)
	ix = 0;
    pti->pool_next = ix;

    if (retp) *retp = hunkp;
    if (sizep) *sizep = size;
    return hunkp->self;

flat:
    /*
     * Something is not right in the error system.  Could be corrupted memory,
     * a bug, or perhaps the caller forgot to call ix_error_init().  In any
     * case, simply return a 'flat' error.
     */
    if (retp) *retp = NULL;
    if (sizep) *sizep = 0;
    return IX_ERROR_NEW(code, level);
}

/** construct error.

    this function logs the specified error information into an error
    buffer and returns a token that can be used to retrieve the
    information again later. the error string that is constructed is
    truncated to fit within the storage area, if necessary.

    this function always returns a nonzero error code that represents
    the error described in its parameters.

    @param	ref	subsidiary error
    @param	code	error number
    @param	level	error severity
    @param	fmt	message format string (like printf)
    @param	...	more parameters (like printf)
    @returns an error token.
*/
ix_error
ix_error_new(ix_error ref, ix_error_number code, ix_error_level level,
	     const char *fmt, ...)
{
    ix_error		    err;
    ix_error_hunk          *hunkp;
    char                   *buf;
    size_t                  size;
    va_list                 ap;

    if (ref != 0)
    {
        if (IX_ERROR_GET_MODE(ref) == 0)
        {
            ix_error_level origLevel = IX_ERROR_GET_LEVEL(ref);
            ix_error_number origCode = IX_ERROR_GET_CODE(ref);
            ref = ix_error_new(
                    0, origCode, origLevel,
                    "Generic Error: code = %08lX, level = %02X",
                    code, level);
        }
    }

    err = ix_error_build(&hunkp, &size, code, level, ref);
    if (hunkp == NULL)
    {
        return err;
    }

    buf = hunkp->buf;

    if ((size > 1) && (fmt != NULL))
    {
        va_start(ap, fmt);
        VSNPRINTF(buf, size, fmt, ap);
        va_end(ap);
    }

    buf[size] = 0;

    if (size)
	DPRINTF((stderr, "ix_error_new: message is '%s'\n", buf));

    DPRINTF2((stderr, "ix_error_new: returning handle %08X\n",
	     err));

    return err;
}

/** locate error information.

    this function translates an error handle into a pointer to the
    appropriate matching error hunk. a null pointer is returned if
    the provided handle is zero, or if it refers to a stale hunk.

    @param	err	error token to decode
    @param	retp	where to return hunk pointer
*/
void
ix_error_find(ix_error err,
	      ix_error_hunk **retp,
	      size_t *sizep)
{
    ix_error_hunk         **hunkpp;
    ix_error_hunk          *hunkp;
    int                     ix;
    ix_error_pti           *pti;
    size_t		    size;

    if (0 == err)
    {
        *retp = 0;
        return;
    }

    pti = ix_error_get_pti();
    if (pti == NULL)
    {
        goto flat;
    }

    hunkpp = pti->pool_ptrs;
    size = pti->data_size;

    if (NULL == hunkpp)
    {
        goto flat;
    }

    ix = err & IX_ERROR_MASK;

    if (ix >= pti->pool_size) {
        ix_ossl_message_log(
            "ix_error_scan asked to decode token %08lx,\n"
            "\twhich means data is in hunk %d,\n"
            "\tbut we are only using hunks 0..%d.\n"
            "\tWas that really an error token?"
            , err, ix, pti->pool_size - 1);
        goto flat;
    }

    hunkp = hunkpp[ix];

    if (NULL == hunkp) {
        ix_ossl_message_log(
            "ix_error_scan wanted to use hunk %d,\n"
            "\tbut that index had a NULL pointer.\n", ix);
        goto flat;
    }

    if (hunkp->self != err) {
        *retp = 0;
        return;
    }

    if (retp) *retp = hunkp;
    if (sizep) *sizep = size;
    return;

flat:
    if (retp) *retp = NULL;
    if (sizep) *sizep = 0;
}

/** recover error information.

    this function recovers the error details from an error token into
    the specified areas. if the error token is stale (the details have
    been overwritten), a nonzero flag will be stored into the return
    area for the stale indication; otherwise, a zero will be stored.

    Note that there is NO error path available if someone passes
    a NULL value into stalep, or if the other multibyte return
    locations are misaligned. If a NULL pointer is passed, this
    function will simply refuse to store through it.

    @param	err	error token to decode
    @param	stalep	where to store stale indication (or NULL)
    @param	levelp	where to return error severity (or NULL)
    @param	codep	where to return error number (or NULL)
    @param	refp	where to return subsidiary error (or NULL)
    @param	buf	where to copy formatted error string (or NULL)
    @param	len	maximum size to put into buf
*/
void
ix_error_scan(ix_error err,
	      int *stalep,
	      ix_error_number *codep,
	      ix_error_level *levelp,
	      ix_error *refp,
	      char *buf, size_t len)
{
    ix_error_hunk          *hunkp;
    size_t		    size;

    ix_error_find(err, &hunkp, &size);

    if (0 == hunkp) {
        if (NULL != stalep)	*stalep = 1;
        if (NULL != refp)	*refp = 0;
        if (NULL != levelp)	*levelp = 0;
        if (NULL != codep)	*codep = 0;
        if ((NULL != buf) &&
            (0 < len))		buf[0] = '\0';
        return;
    }

    if (len > size)
        len = size;

    if (NULL != stalep)		*stalep = 0;
    if (NULL != refp)		*refp = hunkp->ref;
    if (NULL != levelp)		*levelp = hunkp->level;
    if (NULL != codep)		*codep = hunkp->code;
    if ((NULL != buf) &&
        (0 < len))		    strncpy(buf, hunkp->buf, len);
}

int
ix_error_is(ix_error err,
	    ix_error_number numb,
	    ix_error_level levl)
{
    int stale;
    ix_error_level elevl;
    ix_error_number enumb;
    if (!err) return 0;
    ix_error_scan(err, &stale, &enumb, &elevl, 0, 0, 0);
    if (stale)	return 0;
    if (levl && (levl != elevl)) return 0;
    if (numb && (numb != enumb)) return 0;
    return 1;
}

/** dump error information to stdio file.
 */
#if (defined (__linux__) && defined (__KERNEL__))

#include <asm/uaccess.h>

/* hopefully this buffer is large enough */
#define ERROR_PRINT_BUFFER_SIZE   2048

static char error_PrintBuffer[ERROR_PRINT_BUFFER_SIZE];

int error_fprintf(struct file* fp, const char *format ,...)
{
    va_list args;
    int i;

    va_start(args, format);
    i=vsnprintf(error_PrintBuffer,ERROR_PRINT_BUFFER_SIZE, format, args);
    va_end(args);

    if(i > 0)
    {
        mm_segment_t fs;
        fs=get_fs();
        set_fs(KERNEL_DS);
        i = fp->f_op->write(fp, error_PrintBuffer, i, &fp->f_pos);
        set_fs(fs);
    }

    return i;
}

void
ix_error_dump(struct file * fp, ix_error prev)
{
    int                     stale;
    ix_error                next;
    ix_error_level          level;
    ix_error_number         code;
    char                    buf[4096];

    if (NULL == fp)
    {
        if (prev == 0)
        {
            ix_ossl_message_log("[zero error token]\n");
            return;
        }

        while (1)
        {
            if (IX_ERROR_GET_MODE(prev) == 0)
            {
                ix_ossl_message_log( "Generic Error: Code = 0x%06lX, Level = %02lX\n",
                    IX_ERROR_GET_LEVEL(prev),
                    IX_ERROR_GET_CODE(prev));
                return;
            }

            ix_error_scan(prev, &stale,
                &code, &level, &next,
                buf, sizeof buf - 1);

            if (stale)
            {
                ix_ossl_message_log( "stale error token %08lx\n", prev);
                return;
            }

            ix_ossl_message_log( "%s\n\tCode = 0x%06lX, Level = %d\n", buf, code, level);

            if (next == 0)
            {
                /*
                 * Separate multiple chain dumps by an empty line.
                 */
                ix_ossl_message_log("\n");
                return;
            }

            prev = next;
        }
    }
    else
    {
        if (prev == 0)
        {
            error_fprintf(fp, "[zero error token]\n");
            return;
        }

        while (1)
        {
            if (IX_ERROR_GET_MODE(prev) == 0)
            {
                error_fprintf(fp, "Generic Error: Code = 0x%06lX, Level = %02lX\n",
                    IX_ERROR_GET_LEVEL(prev),
                    IX_ERROR_GET_CODE(prev));
                return;
            }

            ix_error_scan(prev, &stale,
                &code, &level, &next,
                buf, sizeof buf - 1);

            if (stale)
            {
                error_fprintf(fp, "stale error token %08lx\n", prev);
                return;
            }

            error_fprintf(fp, "%s\n\tCode = 0x%06lX, Level = %d\n", buf, code, level);

            if (next == 0)
            {
                /*
                 * Separate multiple chain dumps by an empty line.
                 */
                error_fprintf(fp, "\n");
                return;
            }

            prev = next;
        }
    }
}
#else

void
ix_error_dump(FILE * fp, ix_error prev)
{
    int                     stale;
    ix_error                next;
    ix_error_level          level;
    ix_error_number         code;
    char                    buf[4096];

    if (NULL == fp)
    {
        fp = stderr;
    }

    if (prev == 0)
    {
        fprintf(fp, "[zero error token]\n");
        return;
    }

    while (1)
    {
        if (IX_ERROR_GET_MODE(prev) == 0)
        {
            fprintf(fp, "Generic Error: Code = 0x%06lX, Level = %02lX\n",
                    IX_ERROR_GET_LEVEL(prev),
                    IX_ERROR_GET_CODE(prev));
            return;
        }

        ix_error_scan(prev, &stale,
                &code, &level, &next,
                buf, sizeof buf - 1);

        if (stale)
        {
            fprintf(fp, "stale error token %08lx\n", prev);
            return;
        }

        fprintf(fp, "%s\n\tCode = 0x%06lX, Level = %d\n", buf, code, level);

        if (next == 0)
        {
            /*
             * Separate multiple chain dumps by an empty line.
             */
            fprintf(fp, "\n");
            return;
        }

        prev = next;
    }
}

#endif /* end (defined (__linux__) && defined (__KERNEL__)) */


ix_error
ix_error_set_code(ix_error err, ix_error_number code)
{
    ix_error_hunk          *hunkp;
    ix_error_find(err, &hunkp, NULL);

    if (0 == hunkp)
    {
        return IX_ERROR_NEW(code, IX_ERROR_LEVEL_LOCAL);
    }

    hunkp->code = code;
    return err;
}

ix_uint32
ix_error_get_code(ix_error err)
{
    ix_error_number code;

    ix_error_scan(err, NULL, &code, NULL, NULL, NULL, 0);
    return code;
}

ix_error_level
ix_error_get_level(ix_error err)
{
    ix_error_level level;

    ix_error_scan(err, NULL, NULL, &level, NULL, NULL, 0);
    return level;
}

ix_error
ix_error_set_level(ix_error err, ix_error_level level)
{
    ix_error_hunk          *hunkp;
    ix_error_find(err, &hunkp, NULL);

    if (0 == hunkp)
    {
        return IX_ERROR_NEW(IX_OSSL_ERROR_INVALID_OPERATION, level);
    }

    hunkp->level = level;
    return err;
}

const char *ix_error_makestr(const char *fmt, ...)
{
    va_list ap;
    ix_error_pti           *pti;

    va_start(ap, fmt);

    pti = ix_error_get_pti();
    if (pti == NULL)
    {
        return "";
    }

    if (pti->tmp_buf == NULL)
    {
        return "";
    }

    (void) VSNPRINTF(pti->tmp_buf, pti->data_size, fmt, ap);

    va_end(ap);
    return pti->tmp_buf;
}
