/**
 * ============================================================================
 * = COPYRIGHT INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      reg_util.c
 *
 * = DESCRIPTION
 *
 * = AUTHOR
 *      Rob Petri
 * ============================================================================
 * $Id:
**/

#include "ix_ossl.h"
#include "ix_rm.h"
#include "ix_cc_error.h"
#include "cc/ix_cc_reg_util.h"

char *
_ix_cc_reg_path_get_next(char *arg_working_path, 
                         char *arg_name_buf);


/*
 * ix_cc_reg_get_parent_from_path
 * This function takes a registry path in the form:
 *  '/prop1/prop2/prop3/prop4'
 * and retrieves the handle to the property pointed to by the path.
 * In the above example, the ix_configuration_property_handle would
 * be provided for 'prop4'.
 *
 * ARGS
 * Input:
 *      arg_path: A string representing the path.
 * Output:
 *      arg_prop: A pointer to an ix_configuration_property_handle to store handle
 */
ix_error 
ix_cc_reg_get_prop_from_path(char *arg_path, 
                               ix_configuration_property_handle *arg_prop)
{
    char namebuf[IX_CP_MAX_PROPERTY_NAME_LENGTH];
    char *wp = NULL;    /* working path */
    ix_configuration_property_handle curparent; 
    ix_configuration_property_handle newparent;

    /* ensure arg_prop is not null */
    IX_ERROR_CHECK_ARG_NULL(2, arg_prop);

    /* if the path is null, then the root is returned. */
    if((arg_path == NULL) || (*arg_path == 0)){
        *arg_prop = ix_rm_cp_get_core_root();
        return IX_SUCCESS;
    }

    /* lookup the first property in the root of the registry*/
    wp = _ix_cc_reg_path_get_next(arg_path, namebuf);
    /*    printf("%s:", namebuf);*/
    IX_ERROR_CR(ix_rm_cp_property_open(ix_rm_cp_get_core_root(), 
                                       namebuf,
                                       &newparent));
    curparent = newparent;
    
    /* lookup each property with the previous property as parent,
       when complete, curparent will be the property that arg_path
       is pointing to */
    while(wp != NULL){
        wp = _ix_cc_reg_path_get_next(wp, namebuf);
    /*        printf("%s:", namebuf);*/
        IX_ERROR_CR(ix_rm_cp_property_open(curparent, namebuf,
                                           &newparent));

        IX_ERROR_CR(ix_rm_cp_property_close(curparent));
        curparent = newparent;
    }
    *arg_prop = curparent;

    return IX_SUCCESS;
}

/*
 * _ix_cc_reg_path_get_next
 * This internal function is used by ix_cc_reg_get_prop_from_path to extract
 * each entry in the path in turn.
 */
char *
_ix_cc_reg_path_get_next(char *arg_working_path, 
                     char *arg_name_buf)
{
    ix_uint32 copy_count = 0;

    if((arg_working_path == NULL) || (arg_name_buf == NULL)){
        return NULL;
    }
    *arg_name_buf = 0;

    /* strip leading '/' */
    while(*arg_working_path == '/'){
        arg_working_path++;
    }

    /* if we are at the end of the string return NULL */
    if(*arg_working_path == 0){
        return NULL;
    }
    
    while((*arg_working_path != 0) && 
          (*arg_working_path != '/') &&
          (copy_count < IX_CP_MAX_PROPERTY_NAME_LENGTH)){
        *arg_name_buf = *arg_working_path;
        arg_name_buf++;
        arg_working_path++;
        copy_count++;
    }
    *arg_name_buf = 0;

    /* if the name was too long truncate and zip to next arg */
    if(copy_count >= IX_CP_MAX_PROPERTY_NAME_LENGTH){
        while((*arg_working_path != 0) && (*arg_working_path != '/')){
            arg_working_path++;
        }
    }
    
    /* if we are at the end of the string return NULL */
    if(*arg_working_path == 0){
        return NULL;
    }
  
    return arg_working_path;
}

