/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.1 for the IXP2000 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      Linux kernel module file.
 *
 * = FILENAME
 *      l2_config_linux_kernel_module.c
 *
 * = DESCRIPTION
 *      The file defines the init and cleanup functions for Linux kernel module.
 *
 * = CREATION TIME
 *      05/06/2003
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: l2_config_linux_kernel_module.c,v 1.8 2003/12/12 19:56:22 ktseng Exp $ 
 */

#include <linux/module.h>
#include "ix_ossl.h"
#include "l2_config_linux_kernel_module.h"
#include "ix_cc.h"

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

MODULE_LICENSE("Proprietary");

int l2_config_open(struct inode *, struct file *); 
int l2_config_close(struct inode *, struct file *);
int l2_config_ioctl(struct inode *, struct file *, unsigned int, unsigned long);

static struct file_operations l2_config_fops = {
    ioctl: l2_config_ioctl,
    open: l2_config_open,
    release: l2_config_close,
};      

/* dynamic allocation of the Major number from the kernel */
ix_int32 g_MajorNumber = DEV_NUMBER_L2CONFIG;



/**
 * This function gets called on the insmod system call.
 */
int init_module(void)
{

#ifdef IX_DEBUG
    ix_ossl_message_log("Loading l2_config utility driver.\n");
#endif

    if (register_chrdev(g_MajorNumber, DEV_NAME_L2CONFIG, &l2_config_fops) == 0)
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("l2_config utility driver is registered with major:%ld\n",g_MajorNumber);
        ix_ossl_message_log("USAGE:\n");
        ix_ossl_message_log("mknod /dev/%s c %ld <minor>\n", DEV_NAME_L2CONFIG, g_MajorNumber);
        ix_ossl_message_log("with different minor numbers.\n\n");
#endif
    }
    else
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("l2_config utility driver cannot be registered!!!!.\n");
#endif
        return -1;
    }

    return 0;
} /* init_module */



/**
 * This function gets called on the rmmod system call.
 */
void cleanup_module(void)
{

#ifdef IX_DEBUG
    ix_ossl_message_log("Unloading l2_config utility driver.\n");
#endif

    if (unregister_chrdev(g_MajorNumber, DEV_NAME_L2CONFIG) == 0)
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("l2_config utility driver cleaned successfully...\n");
#endif
    }
    else
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("l2_config utility driver COULD NOT BE CLEANED UP !!!\n");
#endif
    }

    return;

} /* cleanup_module */



/**
 * This function is called internally, when the user application calls the open
 * function.
 */
int l2_config_open(
    struct inode *l2_config_inode,
    struct file *l2_config_file)
{
#ifdef IX_DEBUG
    ix_ossl_message_log("l2_config_open called.\n");
#endif

    return 0;

} /* l2_config_open */



/**
 * This function is invoked, when the user application calls the close 
 * funtion.
 */
int l2_config_close(
    struct inode *l2_config_inode,
    struct file *l2_config_file)
{
#ifdef IX_DEBUG
    ix_ossl_message_log("l2_config_close called.\n");
#endif

    return 0;

} /* l2_config_close */



/**
 * This function is called by the ioctl command.
 */
int l2_config_ioctl(
    struct inode *l2_config_inode,
    struct file *l2_config_file,
    unsigned int cmd,
    unsigned long arg_IoctlPtr)
{
    ix_uint32 s_Result;
    ix_uint8 cmdString[MAX_NUM_CHAR_CMD_STRING];

#ifdef IX_DEBUG
    ix_ossl_message_log("l2_config_ioctl called.\n");
#endif

    if (l2_config_inode == NULL)
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("l2_config_inode is NULL.\n");
#endif
        return -1;
    }
  
    if (l2_config_file == NULL)
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("l2_config_file is NULL.\n");
#endif
        return -1;
    }        

    /* Copy argument string to kernel space */
    if ((void *)arg_IoctlPtr != NULL)
    {
        strcpy((char *)cmdString, (char *)arg_IoctlPtr);
    }

    switch(cmd)
    {
        case ADD_V4_ETH_ENTRY:
            s_Result = addV4EthEntry(cmdString);
            break;
        case ADD_V6_ETH_ENTRY:
            s_Result = addV6EthEntry(cmdString);
            break;
        case ADD_V4_L3_INFO:
            s_Result = addV4L3Info(cmdString);
            break;
        case ADD_V4_ATM_ENTRY:
            s_Result = addV4AtmEntry(cmdString);
            break;
        case DELETE_L2_ENTRY:
            s_Result = deleteL2Entry(cmdString);
            break;
        case CLEAR_L2_INFO:
            s_Result = clearL2Info(cmdString);
            break;
        case DUMP_L2TM:
            s_Result = dumpL2tm(cmdString);
            break;
        default:
#ifdef IX_DEBUG
            ix_ossl_message_log("Undefined ioctl command %d.\n", cmd);
#endif
            s_Result = -1;
            break;
    }

    if (s_Result != 0)
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("l2_config_ioctl failed.\n");
#endif
    }
    else
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("l2_config_ioctl succeeded.\n");
#endif
    }

    return s_Result;

} /* l2_config_ioctl */

#endif
