/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2X00 Network Processor, Release 6
 *
 * = FILENAME
 *    l2_config_linux_user_mode_util.c
 *
 * = DESCRIPTION
 *    This file contains linux user mode application 'l2config' that 
 *    can be used to in target shell for configuring the route table.
 *
 * = CHANGE HISTORY
 *      05/06/2003 - Created.
 *
 * ============================================================================
 * $Id: l2_config_linux_user_mode_util.c,v 1.2 2003/07/22 21:48:16 ktseng Exp $
 */
#include <fcntl.h>
#include "l2_config_linux_kernel_module.h"

void l2config_usage(void);

int main(int argc, char *argv[])
{

    int fd;
    int cmd;
    char *cmdArgStr;

    if (argc == 1)
    {
        l2config_usage();
        exit(0);
    }

    if (argc != 3)
    {
        l2config_usage();
        exit(1);
    }

    if (strcmp(argv[1],"addV4EthEntry") == 0)
    {
        cmd = ADD_V4_ETH_ENTRY;
        cmdArgStr = argv[2];
    }
    else if (strcmp(argv[1],"addV6EthEntry") == 0)
    {
        cmd = ADD_V6_ETH_ENTRY;
        cmdArgStr = argv[2];
    }
    else if (strcmp(argv[1],"addV4L3Info") == 0)
    {
        cmd = ADD_V4_L3_INFO;
        cmdArgStr = argv[2];
    }
    else if (strcmp(argv[1],"addV4AtmEntry") == 0)
    {
        cmd = ADD_V4_ATM_ENTRY;
        cmdArgStr = argv[2];
    }
    else if (strcmp(argv[1],"deleteL2Entry") == 0)
    {
        cmd = DELETE_L2_ENTRY;
        cmdArgStr = argv[2];
    }
    else if (strcmp(argv[1],"clearL2Info") == 0)
    {
        cmd = CLEAR_L2_INFO;
        cmdArgStr = argv[2];
    }
    else if (strcmp(argv[1],"dumpL2tm") == 0)
    {
        cmd = DUMP_L2TM;
        cmdArgStr = argv[2];
    }
    else
    {
        printf("Unrecognized command\n");
        exit(1);
    }

    fd = open("/dev/L2Config", O_CREAT | O_RDWR);

    ioctl(fd, cmd, cmdArgStr);

    close(fd);

    return 0;

} /* main */



void l2config_usage(void)
{
    printf("\nUSAGE: l2config <cmd> <\"args\">\n\n");
    printf(" where <cmd> <\"args\"> pair can be one of the followings: \n");
    printf("  addV4EthEntry \"l2Index ipAddr DestMAC SourceMAC name\"\n");
    printf("  addV6EthEntry \"l2Index IPv6Addr DestMAC SrcMAC name\"\n");
    printf("  addV4L3Info \"l2Index ipAddr name\"\n");
    printf("  addV4AtmEntry \"l2Index ipAddr vpiVci vcq llcSnap name\"\n");
    printf("  deleteL2Entry \"l2Index name\"\n");
    printf("  clearL2Info \"l2Index name\"\n");
    printf("  dumpL2tm \"name\"\n");
} /* l2config_usage */
