/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2X00 Network Processor
 *
 * = FILENAME
 *    mac_config_linux_user_app.c
 *
 * = DESCRIPTION
 *    This file contains linux user mode application that 
 *    can be used to in target shell for configuring the Ethernet MAC filter
 *    table.
 *
 * = CHANGE HISTORY
 *      05/08/2003 - Created.
 *
 * ============================================================================
 * $Id: mac_config_linux_user_app.c,v 1.1 2003/05/08 21:23:00 rranjeet Exp $
 */
#include <fcntl.h>
#include "mac_config_linux_kernel_module.h"

void mac_config_usage(void);

int main(int argc, char *argv[])
{

    int fd;
    int cmd;
    char *cmdArgStr;

    if (argc == 1)
    {
        mac_config_usage();
        exit(0);
    }


    if (strcmp(argv[1],"addMac") == 0)
    {
        cmd = ADD_MAC;
        cmdArgStr = argv[2];
    }
    else if (strcmp(argv[1],"delMac") == 0)
    {
        cmd = DEL_MAC;
        cmdArgStr = argv[2];
    }
    else if (strcmp(argv[1],"lookupPort") == 0)
    {
        cmd = LOOKUP_PORT;
        cmdArgStr = argv[2];
    }
    else
    {
        printf("Unrecognized command\n");
        exit(1);
    }

    fd = open("/dev/MConfig", O_CREAT | O_RDWR);

    ioctl(fd, cmd, cmdArgStr);

    close(fd);

    return 0;

} /* main */



void mac_config_usage(void)
{
    printf("\nUSAGE: mac_config <cmd> <\"args\">\n\n");
    printf(" where <cmd> <\"args\"> pair can be one of the followings: \n");
    printf("  addMac \"mac-addr port\"\n");
    printf("  delMac \"mac-addr\"\n");
    printf("  lookupPort \"mac-addr\"\n");
}
