/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      print_freelist.c
 *
 * = AUTHORS
 *     Adrian Georgescu and Ranjeeta Singh
 * = DESCRIPTION
 *      Utility that prints the number of buffers in the freelist
 *
 * ============================================================================
 */
#define IX_ERROR_FILE_IDENT "$Id: print_freelist.c,v 1.3 2003/07/11 02:57:24 rranjeet Exp $"

#include "ix_cc_error.h"
#include "ix_cc.h"
#include <ix_ossl.h>
#include <ix_rm.h>



#define IX_BUF_FL_HANDLE_SIGNATURE  0xfa319cd1

/**
 * MACRO NAME: IX_RM_BUF_FL_MAKE_HANDLE
 *
 * DESCRIPTION: This macro will create a FL handle based on the passed FL ID. 
 *
 * @Param:  - IN arg_FLID - this is a ix_uint32 value representing the FL ID 
 *          to build a FL handle upon.
 *
 * @Return: returns a ix_buffer_free_list_handle coresponding to the passed ID.
 */
#define IX_RM_BUF_FL_MAKE_HANDLE( \
                                 arg_FLID \
                               ) \
                               ((ix_buffer_free_list_handle) /* typecast */ \
                               (IX_BIT_FIELD_MASK32(IX_BUF_FL_ID_BITS, 31U) & IX_BUF_FL_HANDLE_SIGNATURE)| \
                               IX_MAKE_BIT_FIELD32(arg_FLID, 0U, IX_BUF_FL_ID_BITS - 1U))                   
                               


typedef struct ix_s_buffer_element ix_buffer_element;

struct ix_s_buffer_element
{
    ix_buffer_handle 	hBuffer;
    ix_buffer_element*	pNext; 
};

void print_available_buffers(ix_uint32 freeListId)
{
	ix_error err = IX_SUCCESS;
    ix_buffer_element *pElement, *pHead = 0;
    ix_buffer_handle hElementBuffer = 0;
    ix_uint32 countElement = 0;

    ix_buffer_free_list_handle arg_hFreeList = IX_RM_BUF_FL_MAKE_HANDLE(freeListId);

    for(;;)
    {
		err = ix_rm_buffer_alloc(arg_hFreeList, &hElementBuffer);
        if(err != IX_SUCCESS)
			break;

        pElement = ix_ossl_malloc(sizeof(ix_buffer_element));
		if(pElement == 0)
        {
			ix_rm_buffer_free(hElementBuffer);
            break;
        }

		pElement->hBuffer = hElementBuffer;

        if(pHead == 0)
			pElement->pNext = 0;
		else
			pElement->pNext = pHead;

	    pHead = pElement;

		countElement++;
    }
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

	ix_ossl_message_log( "!!!\nThere are %lu buffers in the free list %lu! \n!!!\n",
        countElement, freeListId);
#else
	fprintf(stderr, "!!!\nThere are %lu buffers in the free list %lu! \n!!!\n",
        countElement, freeListId);
#endif
	for(pElement = pHead; pElement != 0; pHead=pElement->pNext, ix_ossl_free(pElement), pElement = pHead)
	{
		ix_rm_buffer_free(pElement->hBuffer);
	}
}
