/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2003 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.1 for the IXP2000 Network Processor
 *
 * = LIBRARY
 *      
 *
 * = MODULE
 *      Linux kernel module file.
 *
 * = FILENAME
 *      print_freelist_linux_kernel_module.c
 *
 * = DESCRIPTION
 *      The file defines the init and cleanup functions for Linux kernel module.
 *
 * = CREATION TIME
 *      05/08/2003
 *
 * = CHANGE HISTORY
 *
 * ============================================================================
 * $Id: print_freelist_linux_kernel_module.c,v 1.3 2003/10/28 21:53:32 ttschlue Exp $
 */

#include <linux/module.h>
#include "ix_ossl.h"
#include "print_freelist_linux_kernel_module.h"
#include "ix_cc.h"

#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)

MODULE_LICENSE("Proprietary");

int print_freelist_open(struct inode *, struct file *); 
int print_freelist_close(struct inode *, struct file *);
int print_freelist_ioctl(struct inode *, struct file *, ix_uint32, unsigned long);

static struct file_operations print_freelist_fops = {
    ioctl: print_freelist_ioctl,
    open: print_freelist_open,
    release: print_freelist_close,
};      

/* dynamic allocation of the Major number from the kernel */
ix_int32 g_MajorNumber = DEV_NUMBER_PCONFIG;



/**
 * This function gets called on the insmod system call.
 */
int init_module(void)
{

#ifdef IX_DEBUG
    ix_ossl_message_log("Loading print_freelist utility driver.\n");
#endif

    if (register_chrdev(g_MajorNumber, DEV_NAME_PCONFIG, &print_freelist_fops) == 0)
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("print_freelist utility driver is registered with major:%ld\n",g_MajorNumber);
        ix_ossl_message_log("USAGE:\n");
        ix_ossl_message_log("mknod /dev/%s c %ld <minor>\n", DEV_NAME_PCONFIG, g_MajorNumber);
        ix_ossl_message_log("with different minor numbers.\n\n");
#endif
    }
    else
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("print freelist utility driver cannot be registered!!!!.\n");
#endif
        return -1;
    }

    return 0;
} /* init_module */



/**
 * This function gets called on the rmmod system call.
 */
void cleanup_module(void)
{

#ifdef IX_DEBUG
    ix_ossl_message_log("Unloading print freelist utility driver.\n");
#endif

    if (unregister_chrdev(g_MajorNumber, DEV_NAME_PCONFIG) == 0)
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("print freelist utility driver cleaned successfully...\n");
#endif
    }
    else
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("print freelist utility driver COULD NOT BE CLEANED UP !!!\n");
#endif
    }

    return;

} /* cleanup_module */



/**
 * This function is called internally, when the user application calls the open
 * function.
 */
int print_freelist_open(
    struct inode *print_freelist_inode,
    struct file *print_freelist_file)
{
#ifdef IX_DEBUG
    ix_ossl_message_log("print_freelist_open called.\n");
#endif

    return 0;

} /* print_freelist_open */



/**
 * This function is invoked, when the user application calls the close 
 * funtion.
 */
int print_freelist_close(
    struct inode *print_freelist_inode,
    struct file *print_freelist_file)
{
#ifdef IX_DEBUG
    ix_ossl_message_log("print_freelist_close called.\n");
#endif

    return 0;

} /* print_freelist_close */



/**
 * This function is called by the ioctl command.
 */
int print_freelist_ioctl(
    struct inode *print_freelist_inode,
    struct file *print_freelist_file,
    ix_uint32 cmd,
    unsigned long arg_IoctlPtr)
{
    ix_uint32 s_Result;
    ix_uint8 cmdString[MAX_NUM_CHAR_CMD_STRING];

#ifdef IX_DEBUG
    ix_ossl_message_log("print_freelist_ioctl called.\n");
#endif

    if (print_freelist_inode == NULL)
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("print_freelist_inode is NULL.\n");
#endif
        return -1;
    }
  
    if (print_freelist_file == NULL)
    {
#ifdef IX_DEBUG
        ix_ossl_message_log("print_freelist_file is NULL.\n");
#endif
        return -1;
    }        


    print_available_buffers(cmd);

    return 0;

} /* print_freelist_ioctl */

#endif
