/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2002 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK for the IXP2X00 Network Processor
 *
 * = FILENAME
 *      route_config_linux_kernel_module.h
 *
 * = DESCRIPTION
 *      This file will describe the ioctl commands exported to Linux user 
 *      mode application. 
 *
 * = CHANGE HISTORY
 *      05/06/2003 - Created.
 *
 * ============================================================================
 * $Id: route_config_linux_kernel_module.h,v 1.6 2003/10/15 23:42:13 ktseng Exp $
 */


#if !defined(__ROUTE_CONFIG_LINUX_KERNEL_MODULE_H__)
#define __ROUTE_CONFIG_LINUX_KERNEL_MODULE_H__



/**
 * System defined include files required.
 */

#if defined(__cplusplus)
extern "C"
{
#endif /* end defined(__cplusplus) */

/* maximum number of characters in the string followed after command */
#define MAX_NUM_CHAR_CMD_STRING    1024

/* defines for ioctl commands */
#define ADD_NEXT_HOP               0x0
#define DEL_NEXT_HOP               0x1
#define ADD_ROUTE                  0x2
#define DEL_ROUTE                  0x3
#define LOOKUP_ROUTE               0x4
#define DUMP_RTM                   0x5
#define PURGE_ROUTES               0x6
#define ROUTE_CONFIG               0x7

#if defined(IX_CONFIGURATION_oc48_ethernet_ipv6_ingress) || defined(IX_CONFIGURATION_oc192_ethernet_ipv6_ingress)
#define ADD_NEXT_HOP_V6               0x8
#define DEL_NEXT_HOP_V6               0x9
#define ADD_ROUTE_V6                  0xa
#define DEL_ROUTE_V6                  0xb
#define DUMP_RTM_V6                   0xc
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
#define GET_STAT_V6                   0xe
#endif 
#endif



int addNextHop(const char *argv);
int addRoute(const char *argv);
int delRoute(const char *argv);
int delNextHop(const char *argv);
int lookupRoute(const char *argv);
int dumpRtm(void);
int route_config(void);
int purgeRoutes(void);

#if defined(IX_CONFIGURATION_oc48_ethernet_ipv6_ingress) || defined(IX_CONFIGURATION_oc192_ethernet_ipv6_ingress)
int addNextHopV6(const char *argv);
int addRouteV6(const char *argv);
int delRouteV6(const char *argv);
int delNextHopV6(const char *argv);
int dumpRtmV6(void);

int getStatsV6(void);

#endif


#if defined(__cplusplus)
}
#endif /* end defined(__cplusplus) */

#endif /* end !defined(__ROUTE_CONFIG_LINUX_KERNEL_MODULE_H__) */

