/**
 * ============================================================================
 * = COPYRIGHT
 *              INTEL CORPORATION PROPRIETARY INFORMATION
 *   This software is supplied under the terms of a license agreement or
 *   nondisclosure agreement with Intel Corporation and may not be copied 
 *   or disclosed except in accordance with the terms in that agreement.
 *      Copyright (C) 2000-2001 Intel Corporation. All rights reserved.
 *
 * = PRODUCT
 *      Intel(r) IXA SDK 3.0 for the IXP2X00 Network Processor, Release 6
 *
 * = FILENAME
 *    route_config_linux_user_mode_util.c
 *
 * = DESCRIPTION
 *    This file contains linux user mode application 'rconfig' that 
 *    can be used to in target shell for configuring the route table.
 *
 * = CHANGE HISTORY
 *      05/06/2003 - Created.
 *
 * ============================================================================
 * $Id: route_config_linux_user_mode_util.c,v 1.4 2003/10/15 23:42:14 ktseng Exp $
 */


#include <fcntl.h>
#include "route_config_linux_kernel_module.h"

void rconfig_usage(void);

int main(int argc, char *argv[])
{

    int fd;
    int cmd;
    char *cmdArgStr;

    if (argc == 1)
    {
        rconfig_usage();
        exit(0);
    }

    if ((argc == 2) && 
        !(((strcmp(argv[1],"dumpRtm") == 0)) ||
          ((strcmp(argv[1],"dumpRtmV6") == 0)) ||
	  ((strcmp(argv[1],"getStatsV6") == 0)) ||
	  ((strcmp(argv[1],"purgeRoutes") == 0))))
    {
        rconfig_usage();
        exit(1);
    }


    if (strcmp(argv[1],"addNextHop") == 0)
    {
        cmd = ADD_NEXT_HOP;
        cmdArgStr = argv[2];
    }
    else if (strcmp(argv[1],"delNextHop") == 0)
    {
        cmd = DEL_NEXT_HOP;
        cmdArgStr = argv[2];
    }
    else if (strcmp(argv[1],"addRoute") == 0)
    {
        cmd = ADD_ROUTE;
        cmdArgStr = argv[2];
    }
    else if (strcmp(argv[1],"delRoute") == 0)
    {
        cmd = DEL_ROUTE;
        cmdArgStr = argv[2];
    }
    else if (strcmp(argv[1],"dumpRtm") == 0)
    {
        cmd = DUMP_RTM;
        cmdArgStr = 0;
    }
    else if (strcmp(argv[1],"lookupRoute") == 0)
    {
        cmd = LOOKUP_ROUTE;
        cmdArgStr = argv[2];
    }
    else if (strcmp(argv[1],"purgeRoutes") == 0)
    {
        cmd = PURGE_ROUTES;
        cmdArgStr = 0;
    }

#if defined(IX_CONFIGURATION_oc48_ethernet_ipv6_ingress) || defined(IX_CONFIGURATION_oc192_ethernet_ipv6_ingress)
    else if (strcmp(argv[1],"addNextHopV6") == 0)
    {
        cmd = ADD_NEXT_HOP_V6;
        cmdArgStr = argv[2];
    }
    else if (strcmp(argv[1],"delNextHopV6") == 0)
    {
        cmd = DEL_NEXT_HOP_V6;
        cmdArgStr = argv[2];
    }
    else if (strcmp(argv[1],"addRouteV6") == 0)
    {
        cmd = ADD_ROUTE_V6;
        cmdArgStr = argv[2];
    }
    else if (strcmp(argv[1],"delRouteV6") == 0)
    {
        cmd = DEL_ROUTE_V6;
        cmdArgStr = argv[2];
    }
    else if (strcmp(argv[1],"dumpRtmV6") == 0)
    {
        cmd = DUMP_RTM_V6;
        cmdArgStr = 0;
    }
#if (_IX_OS_TYPE_ == _IX_OS_LINUX_KERNEL_)
    else if (strcmp(argv[1],"getStatsV6") == 0)
    {
        printf("INSIdE user mode Util \n");
        cmd = GET_STAT_V6;
        cmdArgStr = 0;
    }
#endif /* _IX_OS_LINUX_KERNEL_ */
#endif

    
    else
    {
        printf("Unrecognized command\n");
        exit(1);
    }

    fd = open("/dev/RConfig", O_CREAT | O_RDWR);

    ioctl(fd, cmd, cmdArgStr);

    close(fd);

    return 0;

} /* main */



void rconfig_usage(void)
{
    printf("\nUSAGE: rconfig <cmd> <\"args\">\n\n");
    printf(" where <cmd> <\"args\"> pair can be one of the followings: \n");
    printf("  addNextHop \"nextHopID bladeID l2Index portID mtu flags ipAddr l2IndexType\"\n");
    printf("  delNextHop \"nextHopID\"\n");
    printf("  addRoute \"prefix netmask nextHopID\"\n");
    printf("  delRoute \"prefix netmask\"\n");
    printf("  dumpRtm\n");
    printf("  lookupRoute \"ipaddr\"\n");
    printf("  purgeRoutes\n\n");

#if defined(IX_CONFIGURATION_oc48_ethernet_ipv6_ingress) || defined(IX_CONFIGURATION_oc192_ethernet_ipv6_ingress)
    printf("  addNextHopV6 \"nextHopID bladeID l2Index portID mtu flags l2IndexType\"\n");
    printf("  delNextHopV6 \"nextHopID\"\n");
    printf("  addRouteV6 \"prefix prefixlen nextHopID\"\n");
    printf("  delRouteV6 \"prefix prefixlen\"\n");
    printf("  dumpRtmV6\n");
    printf("  getStatsV6\n");
#endif
}
