///////////////////////////////////////////////////////////////////////////////
//                                                                     
//                  I N T E L   P R O P R I E T A R Y                  
//                                                                      
//     COPYRIGHT (c)  2001-2003 BY  INTEL  CORPORATION.  ALL RIGHTS          
//     RESERVED.   NO  PART  OF THIS PROGRAM  OR  PUBLICATION  MAY      
//     BE  REPRODUCED,   TRANSMITTED,   TRANSCRIBED,   STORED  IN  A    
//     RETRIEVAL SYSTEM, OR TRANSLATED INTO ANY LANGUAGE OR COMPUTER    
//     LANGUAGE IN ANY FORM OR BY ANY MEANS, ELECTRONIC, MECHANICAL,    
//     MAGNETIC,  OPTICAL,  CHEMICAL, MANUAL, OR OTHERWISE,  WITHOUT    
//     THE PRIOR WRITTEN PERMISSION OF :                                
//                                                                      
//                        INTEL  CORPORATION                            
//                                                                     
//                     2200 MISSION COLLEGE BLVD                        

//               SANTA  CLARA,  CALIFORNIA  95052-8119                  
//                                                                      
///////////////////////////////////////////////////////////////////////////////
//
//
//      File Name: pkt_tx_16p_util.uc
//
//      Purpose: Utility macros for for mulitport (up to 16 ports) PACKET 
//				 transmitter for Egress IXP24XX/IXP28XX
//
///////////////////////////////////////////////////////////////////////////////


#ifndef	__PKT_TX_P16_UTIL_UC__
#define	__PKT_TX_P16_UTIL_UC__


#if IS_IXPTYPE(__IXP24XX)
///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_init_port_phy_depth()
//
// Description:				initializ flow control related credit for specific
//							port in local memory 
//
// Outputs:							
//							None 
//
// Inputs:  
//      in_port_no:			port to initialize PHY Tx FIFO depth value
//      in_phy_depth_val:	PHY Tx FIFO depth value
//
// Constants:
//						 	None 
//
// Size:
//		8 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro 	_pkt_tx_16p_init_port_phy_depth(in_port_no, in_phy_depth_val)

.begin 

	.reg	queue_status_lm_base

 	// get the base address of the queue status info of the port in local mem

 	_pkt_tx_16p_get_queue_status_lm_base(queue_status_lm_base, in_port_no) ;(2)
	local_csr_wr[active_lm_addr_1, queue_status_lm_base]

	// waiting for local_csr_wr to settle

	nop
	nop
	nop
	alu[lm1_fc_credit_remaining, --, b, in_phy_depth_val]	; set allowed_mpkts
	alu[lm1_fc_ready_credit, --, b, in_phy_depth_val]		; set phy_depth

.end 
 
#endm	// end of #macro _pkt_tx_16p_init_port_phy_depth[]

#endif	// #if IS_IXPTYPE(__IXP24XX)

#if IS_IXPTYPE(__IXP28XX)
///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_init_port_phy_depth()
//
// Description:				initializ flow control related credit for specific
//							port in local memory 
//
// Outputs:							
//							None 
//
// Inputs:  
//      in_port_no:			port to initialize PHY Tx FIFO depth value
//      in_hungry_depth:	PHY Tx FIFO depth value for "HUNGRY"
//      in_starving_depth:	PHY Tx FIFO depth value for "STARVING
//
// Constants:
//						 	None 
//
// Size:
//		9 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro 	_pkt_tx_16p_init_port_phy_depth(in_port_no, in_hungry_depth, \
					in_starving_depth)
	.begin 
	.reg	queue_info_lm

  	// get the base address of the queue status info of the port in local mem

 	_pkt_tx_16p_get_queue_status_lm_base(queue_info_lm, in_port_no)	; (2)

	local_csr_wr[active_lm_addr_1, queue_info_lm]

	// waiting for local_csr_wr to settle

	nop
	nop
	nop


	alu[lm1_fc_credit_remaining, --, b, in_starving_depth]	; set allowed_mpkts
	alu[lm1_fc_hungry_credit, --, b, in_hungry_depth]	; set phy_depth_hungry
	alu[lm1_fc_starving_credit, --, b, in_starving_depth] 

.end 

#endm	// end of #macro _pkt_tx_16p_init_port_phy_depth()

#endif	// #if IS_IXPTYPE(__IXP28XX)


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_init_constants_in_lm()
//
// Description:					initialize constants in local memory to 
//								resolve	insufficient GPR problem
//
// Outputs:							
//		None 
//
// Inputs:  
//      in_port_no:				port to initialize PHY Tx FIFO depth value
//		in_default_sigmask_1	default signal mask in phase 1
//		in_default_sigmask_2	default signal mask in phase 2
//
// Constants:
//		None 
//
///////////////////////////////////////////////////////////////////////////////

#macro 	_pkt_tx_16p_init_constants_in_lm(in_port_no, in_default_sigmask_1, \
					in_default_sigmask_2)

	.begin 

	.reg	queue_info_lm
	.reg	tmp

   	// get the base address of the queue status info of the port in local mem

 	_pkt_tx_16p_get_queue_status_lm_base(queue_info_lm, in_port_no)	; (2)

	local_csr_wr[active_lm_addr_1, queue_info_lm]


	// waiting for local_csr_wr to settle

	nop
	nop
	nop

	// starting address of TBUF 

	move(tmp, RBUF_TBUF_BASE)		
	alu[lm1_tbuf_base, --, B, tmp]		

	// starting address of TBUF_ELEMENT_CTRL_V_# 

	move(tmp, TBUF_ELEMENT_CTRL_VALID)
	alu[lm1_tbuf_valid_base, --, B, tmp]

	// scratch ring to get tx request

	alu_shf[tmp, --, B,  PACKET_TX_SCR_IN_RING, <<2]
	alu[lm1_tx_req_ring,--, B, tmp]

	// base to calculate the address field in indirect ref of 
	// dram[tbuf_wr....] instruction

	move(tmp, DRAM_INDIREF_BASE_LW)
	alu[lm1_indiref_base, --, B, tmp]

	//  base for tcw0 for sop_eop mpaket

	move(tmp, BASE_TCW0_SOP_EOP)	
	alu[lm1_tcw0_sop_eop_base, --, B, tmp]

.end 

#endm	// _pkt_tx_16p_init_constants_in_lm()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_set_16bit_msf_reg()
//
// Description: Use msf[fast_wr...] to set MSF register 
//
// Outputs: 
//								None
//
// Inputs:  
//		in_msf_reg_addr:		MSF register address to set
//		in_msf_reg_val:			MSF register value to set 
//
// Constants:
//								None 
//
// Size: 
//
//		3 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _pkt_tx_16p_set_16bit_msf_reg(in_msf_reg_addr, in_msf_reg_val)

	.begin

	.reg	tmp_addr
	.reg	tmp_val
	
	alu[tmp_addr, --, B, in_msf_reg_addr]			; get 16 bit msf address
	alu_shf[tmp_val, --, B, in_msf_reg_val, <<16]	; get value							
	msf[fast_wr, --, tmp_addr, tmp_val] 			; setting with fast_wr

.end 

#endm    // _pkt_tx_16p_set_16bit_msf_reg()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_get_queue_status_lm_base()
//
// Description: get the base address for queue status info of the port in 
//				local memory 
//
// Outputs: 
//		out_queue_status_lm_base:	base address for queue status info of the 
//                                  port in local memory 
//
// Inputs:
//		in_port:					port number  
//
// Constants:
//		None
//
// Size: 
//		2 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_get_queue_status_lm_base(out_queue_status_lm_base, in_port) 

.begin 

	.reg	tmp

	// the first port_status info block is for FIRST_PORT_IN_THIS_ME, then
	// the next block for next bloack, and so on

	alu[tmp, in_port, -, FIRST_PORT_IN_THIS_ME]		; get port index in this ME
	alu_shf[out_queue_status_lm_base, queue_stat_base, or, tmp, \
													<<QUEUE_STATUS_SHFT]

.end 

#endm	// _pkt_tx_16p_get_queue_status_lm_base()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_get_tx_port()
//
// Description: get tx port number and bit pattern for tx port from tx request
//
// Outputs: 
//			out_tx_port:		port in the tx request
//			out_tx_port_bit:	bit corresponding to port = 1 << out_tx_port
//
// Inputs:
//			in_tx_request:		tx request  
//
// Constants:
//			None
//
// Size: 
//			3 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_get_tx_port(out_tx_port, out_tx_port_bit, in_tx_request)

.begin

	// get port number
		 
	alu_shf[out_tx_port, PORT_MASK_IN_TXR, and, in_tx_request, >>24]	  
	alu[--, out_tx_port, or, 0]
	alu_shf[out_tx_port_bit, --, b, 1, <<indirect]

.end

#endm	// _pkt_tx_16p_get_tx_port()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_validate_first_mpkt()
//
// Description: validate first mpacket
//
// Outputs: 
//		None
//
// Inputs/Outputs: 
//			io_sigmask:         sigmask to add signal  
//
// Inputs:
//			in_sig:  	 		signal for msf[write....]
//			in_tbuf_ele_1:		tbuf element number for first mpacket 
//			in_tcw0_1:			tcw0 for first mpacket
//			in_port:			port to transmit
//
// Constants:
//		None
//
// Size: 
//			6 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_validate_first_mpkt(io_sigmask, in_sig, in_tbuf_ele_1, \
								in_tcw0_1, in_port) 
.begin

	.reg	tmp

	alu[$w6, --, b, in_tcw0_1]		; element trnsmit control word

#if IS_IXPTYPE(__IXP24XX)

	// workaround for HW-bug#1249, copy channel # into tcw1 (reserved in PRM)
	// $w7 has been initialized to 0, so no change necessary for IXP28XX here

	// HW-bug#1249 already fixed in IXP24XX chip rev. B0 and beyond
	// alu[$w7, --, b, in_port]

#endif	//#if IS_IXPTYPE(__IXP24XX)

	alu_shf[tmp, --, B, in_tbuf_ele_1, <<TCW_VALID_ADDR_SHFT]

	alu[tmp, tmp, +, lm1_tbuf_valid_base]
	msf[write, $w6, tmp, 0, 2], sig_done[in_sig]
	_pkt_tx_16p_set_signal(io_sigmask, in_sig)	; add signal in_sig

	_pkt_tx_16p_debug_incr_counter(@pkt_tx_16p_mpkts_txed)

.end

#endm	// _pkt_tx_16p_validate_first_mpkt()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_validate_second_mpkt()
//
// Description: validate second mpacket
//
// Outputs: 
//		None
//
// Inputs/Outputs: 
//			io_sigmask:         sigmask to add signal  
//
// Inputs:
//			in_sig:  	 		signal for msf[write....]
//			in_tbuf_ele_1:		tbuf element number for first mpacket 
//			in_tcw0_2:			tcw0 for second mpacket
//			in_port:			port to transmit
//
// Constants:
//		None
//
// Size: 
//			7 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_validate_second_mpkt(io_sigmask, in_sig, in_tbuf_ele_1, \
								in_tcw0_2, in_port) 

.begin

	.reg	tmp

	alu[$w2, --, b, in_tcw0_2]		; element trnsmit control word

#if IS_IXPTYPE(__IXP24XX)

	// workaround for HW-bug#1249, copy channel # into tcw1 (reserved in PRM)
	// HW-bug#1249 already fixed in IXP24XX chip rev. B0 and beyond

	// alu[$w3, --, b, in_port]
	alu[$w3, --, b, 0]

#else	// not IXP24XX

	alu[$w3, --, b, 0]

#endif 	// #if IS_IXPTYPE(__IXP24XX)

	alu[tmp, in_tbuf_ele_1, +, 1]
	alu[tmp, tmp, AND, TBUF_ELE_BITS_MASK]  	; tbuf_ele_2

	alu_shf[tmp, --, B, tmp, <<TCW_VALID_ADDR_SHFT]

	alu[tmp, tmp, +, lm1_tbuf_valid_base]
	msf[write, $w2, tmp, 0, 2], sig_done[in_sig]
	_pkt_tx_16p_set_signal(io_sigmask, in_sig)	; add signal in_sig

	_pkt_tx_16p_debug_incr_counter(@pkt_tx_16p_mpkts_txed)

.end

#endm	// _pkt_tx_16p_validate_second_mpkt()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_validate_third_mpkt()
//
// Description: validate third mpacket
//
// Outputs: 
//		None
//
// Inputs/Outputs: 
//			io_sigmask:         sigmask to add signal  
//
// Inputs:
//			in_sig:  	 		signal for msf[write....]
//			in_tbuf_ele_1:		tbuf element number for first mpacket 
//			in_tcw0_3:			tcw0 for third mpacket
//			in_port:			port to transmit
//
// Constants:
//		None
//
// Size: 
//			7 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_validate_third_mpkt(io_sigmask, in_sig, in_tbuf_ele_1, \
								in_tcw0_3, in_port) 

.begin

	.reg	tmp

	alu[$w4, --, b, in_tcw0_3]		; element trnsmit control word

 #if IS_IXPTYPE(__IXP24XX)

	// workaround for HW-bug#1249, copy channel # into tcw1 (reserved in PRM)
	// HW-bug#1249 already fixed in IXP24XX chip rev. B0 and beyond

	// alu[$w5, --, b, in_port]
	alu[$w5, --, b, 0]

#else	// not IXP24XX

	alu[$w5, --, b, 0]

#endif 	// #if IS_IXPTYPE(__IXP24XX)

	alu[tmp, in_tbuf_ele_1, +, 2]
	alu[tmp, tmp, AND, TBUF_ELE_BITS_MASK]  	; tbuf_ele_3

	alu_shf[tmp, --, B, tmp, <<TCW_VALID_ADDR_SHFT]

	alu[tmp, tmp, +, lm1_tbuf_valid_base]
	msf[write, $w4, tmp, 0, 2], sig_done[in_sig]
	_pkt_tx_16p_set_signal(io_sigmask, in_sig)	; add signal in_sig

	_pkt_tx_16p_debug_incr_counter(@pkt_tx_16p_mpkts_txed)

.end

#endm	// _pkt_tx_16p_validate_third_mpkt()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_write_critcal_tcw_for_prev_iter()
//
// Description: write transit control word for one mpacket case inr previous
//				iteration, otherwise branchout to save instruction cycles
//
// Outputs: 
//		None
//
// Inputs/Outputs:
//			io_sigmask:         sigmask to add or clear signals  
//
// Inputs:
//			in_mpkt_numb:		number of mpacket whose transmit control word
//								need to be validated
//			in_port:			port to transmit
//
// Constants:
//		None
//
//	Labels:
//		NOT_CRITICAL_TCW_WRITE_LABEL:	symbolic address to go when 
//										in_mpkt_numb is not one								    one mpacket's required
//
// Size: 
//			7 instructions 
//
///////////////////////////////////////////////////////////////////////////////

#macro 	_pkt_tx_16p_write_critcal_tcw_for_prev_iter(io_sigmask, \
						in_mpkt_numb, in_port, NOT_CRITICAL_TCW_WRITE_LABEL) 

.begin 
	
	br!=byte[in_mpkt_numb, 0, 1, NOT_CRITICAL_TCW_WRITE_LABEL]

	_pkt_tx_16p_validate_first_mpkt(io_sigmask, sig_mpkt_1_tcw, tbuf_ele_1, \
								tcw0_1, in_port) 		; (6)

.end 

#endm	// _pkt_tx_16p_write_critcal_tcw_for_prev_iter()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_write_non_critcal_tcws_for_prev_iter()
//
// Description: write transit control words for more than one mpacket case in
// 				previous iteration
//
// Outputs: 
//		None
//
// Inputs/Outputs:
//			io_sigmask:         sigmask to add or clear signals  
//
// Inputs:
//			in_mpkt_numb:		number of mpacket whose transmit control word
//								need to be validated
//			in_port:			port to transmit
//
// Constants:
//		None
//
// Size: 
//			5 instructions (no mpacket)
//			17 instructions (2 mpackets)
//			25 instructions (3 mpackets)
//
///////////////////////////////////////////////////////////////////////////////

#macro 	_pkt_tx_16p_write_non_critcal_tcws_for_prev_iter(io_sigmask, \
														in_mpkt_numb, in_port)

.begin 
	
	br=byte[in_mpkt_numb, 0, 2, validate_two_mpkts#]
	br=byte[in_mpkt_numb, 0, 0, no_need_to_write_tcws#]

validate_three_mpkts#: 		; for in_mpkt_num = 3 or 4 (still 3 mpackets)

	_pkt_tx_16p_validate_first_mpkt(io_sigmask, sig_mpkt_1_tcw, tbuf_ele_1, \
								tcw0_1, in_port) 		; (6)

	_pkt_tx_16p_validate_second_mpkt(io_sigmask, sig_mpkt_2_tcw, tbuf_ele_1, \
								tcw0_2, in_port) 		; (7)
	
	_pkt_tx_16p_validate_third_mpkt(io_sigmask, sig_mpkt_3_tcw, tbuf_ele_1, \
								tcw0_3, in_port) 		; (7)
	
	br[write_tcws_done#]

validate_two_mpkts#:

	_pkt_tx_16p_validate_first_mpkt(io_sigmask, sig_mpkt_1_tcw, tbuf_ele_1, \
								tcw0_1, in_port) 		; (6)

	_pkt_tx_16p_validate_second_mpkt(io_sigmask, sig_mpkt_2_tcw, tbuf_ele_1, \
								tcw0_2, in_port) 		; (7)

no_need_to_write_tcws#:
write_tcws_done#:

.end 

#endm	// _pkt_tx_16p_write_non_critcal_tcws_for_prev_iter()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_write_tcws_for_prev_iter()
//
// Description: write transit control word/words for previous iteration
//
// Outputs: 
//		None
//
// Inputs/Outputs:
//			io_sigmask:         sigmask to add or clear signals  
//
// Inputs:
//			in_mpkt_numb:		number of mpacket whose transmit control word
//								need to be validated
//			in_tx_port:			port to transmit
//
// Constants:
//		None
//
// Size: 
//			6 instructions (0 mpacket)
//			11 instructions (1 mpacket)
//			21 instructions (2 mpacket)
//			31 instructions (3 mpacket)
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_write_tcws_for_prev_iter(io_sigmask, in_mpkt_numb, \
																	in_tx_port)

.begin 
	
	br=byte[in_mpkt_numb, 0, 1, validate_one_mpkt#]
	br=byte[in_mpkt_numb, 0, 2, validate_two_mpkts#]
	br=byte[in_mpkt_numb, 0, 0, no_need_to_write_tcws#]

validate_three_mpkts#: 		; for in_mpkt_num = 3 or 4

	_pkt_tx_16p_validate_first_mpkt(io_sigmask, sig_mpkt_1_tcw, tbuf_ele_1, \
								tcw0_1, in_tx_port) 		; (6)

	_pkt_tx_16p_validate_second_mpkt(io_sigmask, sig_mpkt_2_tcw, tbuf_ele_1, \
								tcw0_2, in_tx_port) 		; (7)
	
	_pkt_tx_16p_validate_third_mpkt(io_sigmask, sig_mpkt_3_tcw, tbuf_ele_1, \
								tcw0_3, in_tx_port) 		; (7)
	
	br[write_tcws_done#]

validate_two_mpkts#:

	_pkt_tx_16p_validate_first_mpkt(io_sigmask, sig_mpkt_1_tcw, tbuf_ele_1, \
								tcw0_1, in_tx_port) 		; (6)

	_pkt_tx_16p_validate_second_mpkt(io_sigmask, sig_mpkt_2_tcw, tbuf_ele_1, \
								tcw0_2, in_tx_port) 		; (7)

	br[write_tcws_done#]

validate_one_mpkt#:

	_pkt_tx_16p_validate_first_mpkt(io_sigmask, sig_mpkt_1_tcw, tbuf_ele_1, \
								tcw0_1, in_tx_port) 		; (6)

no_need_to_write_tcws#:
write_tcws_done#:

.end 

#endm	// _pkt_tx_16p_write_tcws_for_prev_iter()


///////////////////////////////////////////////////////////////////////////////
//
// 	_pkt_tx_16p_update_flow_control_on_transmit() 
//
// Description: update flow control related info after the port is scheduled 
//              to transmit	certain number of mapcket 
//
// Outputs: 
//		None 
//
// Inputs:
//			in_mpkt:		number of mpacket to transmit  
//			in_port:		port to trnsmit mpacket/mpackets  
//			in_port_bit:	port bit for port to trnsmit mpacket/mpackets  
//
//
// Constants:
//		None
//
// Size: 
//			11 (IXP24XX) instructions
//			13 (IXP28XX) instructions
// 
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_update_flow_control_on_transmit(in_mpkt, in_port, \
																in_port_bit) 

#if IS_IXPTYPE(__IXP24XX)
.begin

	.reg	tmp

	// decrease allowed_mpkts by in_mpkt

	alu[lm1_fc_credit_remaining, lm1_fc_credit_remaining, -, in_mpkt]

	alu[--, lm1_fc_credit_remaining, -, FC_LOW_WATERMARK]
    bge[update_busy_port_done#]

 	// add port to @busy_port, following tmp used to resolve odd-sided gprs
	// allocation conflict

	alu[tmp, --, B, in_port_bit]
	alu[@busy_port, @busy_port, OR, tmp]	

update_busy_port_done#:

	alu[--, lm1_fc_credit_remaining, -, MAX_MPKT_PER_THREAD]
	bge[update_flow_control_info_done#]

	// following temporary storage of in_port_bit to tmp is used to resolve
	// odd-sided GPR allocation conflict

	alu[tmp, --, B, in_port_bit]
	alu[@bfcp, @bfcp, OR, tmp]		
	alu[@rp, @ep,  AND~, @bfcp]
	alu[@vpp, @vap, AND~ , @bfcp]

update_flow_control_info_done#:	

.end 
#endif	 // #if IS_IXPTYPE(__IXP24XX)

#if IS_IXPTYPE(__IXP28XX)
.begin

	.reg	mport
	.reg	tmp

	// decrease allowed_mpkts by in_mpkt

	alu[lm1_fc_credit_remaining, lm1_fc_credit_remaining, -, in_mpkt]
	alu_shf[mport, 1, OR, in_port, <<1]
	alu[--, lm1_fc_credit_remaining, -, FC_LOW_WATERMARK]

    bge[update_busy_port_done#]
	alu[--, mport, OR, 0]
	alu_shf[tmp, --, B, 1, <<indirect]

 	// add port to @busy_port

	alu[@busy_port, @busy_port, OR, tmp]	

update_busy_port_done#:

	alu[--, lm1_fc_credit_remaining, -, MAX_MPKT_PER_THREAD]
	bge[update_flow_control_info_done#]

	// following temporary storage of in_port_bit to tmp is used to resolve
	// odd-sided GPR allocation conflict

	alu[tmp, --, B, in_port_bit]
	alu[@bfcp, @bfcp, OR, tmp]		
	alu[@rp, @ep,  AND~, @bfcp]
	alu[@vpp, @vap, AND~ , @bfcp]

update_flow_control_info_done#:	

.end 
#endif	 // #if IS_IXPTYPE(__IXP28XX)

#endm	// _pkt_tx_16p_update_flow_control_on_transmit()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_get_port_queue_entry_base()
//
// Description: get queue entry base for port
//
// Prerequisite:				local memory index 1 alrady set to queue status 
//								of the in_port
//
// Outputs: 
//		out_port_queue_entry_base: queue entry base for port
//
// Inputs:  
//      in_port:				port 
//
// Constants:
//		None
//
//	Size: 3 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_get_port_queue_entry_base(out_port_queue_entry_base, \
								in_port)
 
.begin
	
	.reg	tmp
	
	alu[tmp, in_port, -, FIRST_PORT_IN_THIS_ME]		; get port index in this ME
	alu_shf[out_port_queue_entry_base, --, b, tmp, <<PORT_QUEUE_ENRY_SHFT]
	#if (PORT_QUEUE_ENTRY_BASE !=0)
	alu[out_port_queue_entry_base, out_port_queue_entry_base, +, \
					PORT_QUEUE_ENTRY_BASE]	; port queue entry base in lm
	#endif 	//#if (PORT_QUEUE_ENTRY_BASE !=0)

.end

#endm	//  _pkt_tx_16p_get_port_queue_entry_base


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_set_lm_index0_to_port_status_info()
//
// Description: set local memory index 0 to the beginning of port status 
//
// Outputs: 
//		None 
//
// Inputs:
//		in_port:			port number  
//
// Constants:
//		None
//
// Size: 
//		3 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_set_lm_index0_to_port_status_info(in_port) 

.begin 

	.reg	queue_staus_lm

 	_pkt_tx_16p_get_queue_status_lm_base(queue_staus_lm, in_port)	; (2)

	local_csr_wr[active_lm_addr_0, queue_staus_lm]

.end 

#endm	// _pkt_tx_16p_set_lm_index0_to_port_status_info()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_set_lm_index_1_to_port_status_info()
//
// Description: set local memory index 1 to the beginning of port status 
//
// Outputs: 
//		None 
//
// Inputs:
//		in_port:			port number  
//
// Constants:
//		None
//
// Size: 
//		3 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_set_lm_index_1_to_port_status_info(in_port) 

.begin 

	.reg	queue_status_lm_base

  	// get the base address of the queue status info of the port in local mem

 	_pkt_tx_16p_get_queue_status_lm_base(queue_status_lm_base, in_port)	; (2)

	local_csr_wr[active_lm_addr_1, queue_status_lm_base]

.end 

#endm	// _pkt_tx_16p_set_lm_index_1_to_port_status_info()


///////////////////////////////////////////////////////////////////////////////
//
//	_pkt_tx_16p_update_flow_control_from_status() 
//
// Description: update flow control info based on latest port PHY status
//
// Outputs: 
//		None
//
// Inputs:  
//		None
//
// Constants:
//		None
//
//	Size:
//      4  instructions for thread 0 to 6
//		20/26  instructions for thtrad 7 in IXP24XX/ IXP28XX. (Worst case cycle 
//			count, should not happen every time)
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_update_flow_control_from_status()

#if IS_IXPTYPE(__IXP24XX)
.begin 
	.reg	tmp
	.reg	tx_stat
	.reg	ready_ports
	.reg	need_update
	.reg	cur_port
	.reg	queue_stat_lm

	ld_field_w_clr[tmp, 0011, $mphy_fc_stat, >>16]		; get tx_pending
	ld_field_w_clr[tx_stat, 0011, $mphy_fc_stat]		; get tx_status
	alu[ready_ports, tx_stat, AND~, tmp]
	 
	// @busy_port already identify what ports need to check, then following
	// need_update identify what ports need to update it's credit and 
	// corresponding bit in @nfc 

	alu[need_update, @busy_port, AND, ready_ports]
	beq[update_flow_control_info_done#]

	// remove the port/ports from @busy_port and @bfcp

	alu[@busy_port, @busy_port, AND~, need_update]
	alu[@bfcp, @bfcp, AND~, need_update]

check_again#:

	ffs[cur_port, need_update]
	beq[update_flow_control_info_done#]	

	//set local memory index 1 to port status (3)

	_pkt_tx_16p_set_lm_index_1_to_port_status_info(cur_port)

	//remove current port from need_update

	alu[--, cur_port, OR, 0]
	alu_shf[tmp, --, B, 1, <<indirect]
	alu[need_update, need_update, AND~, tmp]

	// update allowed_mpkts
	
	alu[lm1_fc_credit_remaining	, --, B, lm1_fc_ready_credit]

	br[check_again#]
	
update_flow_control_info_done#:

	// update @rp and @vpp

	alu[@rp, @ep,  AND~, @bfcp]
	alu[@vpp, @vap, AND~ , @bfcp]

.end
#endif	 // #if IS_IXPTYPE(__IXP24XX)

#if IS_IXPTYPE(__IXP28XX)
.begin 

	.reg	ready_ports
	.reg	need_update
	.reg	cur_port
	.reg	curm_port
	.reg	stat_bit
	.reg	tmpm
	.reg	qinfo_lm
	.reg	port_index_diff
	.reg	tmp


	br_bclr[thread_specific_flag, TSF_UPDATE_FC_BIT, \
							update_flow_control_info_done#] 

	// get port/ports with "hungry" or "starving" status

	alu[tmp, $mphy_fc_stat, XOR, @fc_check_pattern]
	alu[ready_ports, tmp, and, @fc_check_pattern]


	// @busy_port already identify what ports need to check, then following
	// need_update identify what ports need to update it's credit and 
	// corresponding bit in @bfcp

	alu[need_update, @busy_port, and, ready_ports]
	beq[update_flow_control_info_done#]

	// remove the ports from busy_port

	alu[@busy_port, @busy_port, and~, need_update]

check_again#:

	ffs[curm_port, need_update]
	beq[update_flow_control_info_done#]
	
	// the cur_port found is (port_numb * 2) + 1
	//remove current port from need_update

	// the first port_status info block is for FIRST_PORT_IN_THIS_ME, then
	// the next block for next bloack, and so on

	alu[cur_port, --, B, curm_port, >>1] 
	alu[--, cur_port, OR, 0]
	alu_shf[tmp, --, b, 1, <<indirect]
	alu[@bfcp, @bfcp, AND~, tmp]

	alu[port_index_diff, cur_port, -, FIRST_PORT_IN_THIS_ME]
	alu_shf[qinfo_lm, queue_stat_base, or, port_index_diff, \
			<<QUEUE_STATUS_SHFT]

	//set local memory index 1

	local_csr_wr[active_lm_addr_1, qinfo_lm]
	alu[stat_bit, curm_port, -, 1]
	alu[--, curm_port, OR, 0]
	alu_shf[tmpm, --, b, 1, <<indirect]
	alu[need_update, need_update, and~, tmpm]  /*reset bit */

	// update fc_credit_remaining to starving credit

	alu[lm1_fc_credit_remaining, --, b, lm1_fc_starving_credit]
	alu[--, $mphy_fc_stat, AND, tmpm]		; check LSB of port status
	beq[check_again#]

	// update fc_credit_remaining to hungry credit

	alu[lm1_fc_credit_remaining, --, B, lm1_fc_hungry_credit]
	br[check_again#]

update_flow_control_info_done#:

	// update @rp and @vpp

	alu[@rp, @ep,  AND~, @bfcp]
	alu[@vpp, @vap, AND~ , @bfcp]

.end 
#endif	 // #if IS_IXPTYPE(__IXP28XX)

#endm	// _pkt_tx_16p_update_flow_control_from_status


///////////////////////////////////////////////////////////////////////////////
// _pkt_tx_16p_notify_scheduler()
//
// Description: notify scheduler that one more packet has been transmitted  
//				from this port 
//
// Prerequisite:			local memory index 1 alrady set to queue status 
//							of the in_port
//
// Outputs:	
//      out_txd_port:		total packets transmitted by the port (sram 
//                          transfer register)
//
// Inputs/Outputs:  
//      io_sigmask:			signal mask 
//
// Inputs:  
//      in_port:			port to transmit the current mpacket 
//		in_sig:				signal used for reflect[write....]
//
// Constants:
//		None 
//
// Size:
//		2 instructions for SPHY_1X32 mode, and 
//      10 instructions for other modes (jump table) 
//      5 instructions for other modes (calculated addressing, 
//        except IXP2400 rev. A0/A1
//
///////////////////////////////////////////////////////////////////////////////

#macro 	_pkt_tx_16p_notify_scheduler(out_txd_port, io_sigmask, in_sig, in_port)

#ifndef DISABLE_TX2SCHED_FEEDBACK

#if IS_IXPTYPE(__IXP24XX)
	.reg remote	$$txd_p0 $$txd_p1 $$txd_p2 $$txd_p3 
	.reg remote $$txd_p4 $$txd_p5 $$txd_p6 $$txd_p7 
	.reg remote $$txd_p8 $$txd_p9 $$txd_p10 $$txd_p11 
	.reg remote $$txd_p12 $$txd_p13 $$txd_p14 $$txd_p15 
.begin 
	.reg	jmp_offset
	alu_shf[jmp_offset, --, b, in_port, <<1]    ; get jmp_offset by port times two
	// jump to different port
	jump[jmp_offset, jmp_tbl#],defer[3], targets[xp0#,xp1#,xp2#,xp3#,xp4#,xp5#, \
						xp6#,xp7#,xp8#,xp9#,xp10#,xp11#,xp12#,xp13#,xp14#,xp15#] 
	alu[out_txd_port, --, B, lm1_pkt_txed] ; get txed pkts
	nop
	nop

//	Start of jump table. 
jmp_tbl#:
  
xp0#:	
	// update scheduler with updated counts of packets transmitted for port 0
	cap[write, out_txd_port, SCHEDULER_ME, $$txd_p0, 0, 1], \
			sig_done[in_sig]	; reflect write
  	br[notify_schlr_done#]

xp1#:
	// update scheduler with updated counts of packets transmitted for port 1
	cap[write, out_txd_port, SCHEDULER_ME, $$txd_p1, 0, 1], \
			sig_done[in_sig]	; reflect write
  	br[notify_schlr_done#]

xp2#:
	// update scheduler with updated counts of packets transmitted for port 2
	cap[write, out_txd_port, SCHEDULER_ME, $$txd_p2, 0, 1], \
			sig_done[in_sig]	; reflect write
  	br[notify_schlr_done#]

xp3#:
	// update scheduler with updated counts of packets transmitted for port 3
	cap[write, out_txd_port, SCHEDULER_ME, $$txd_p3, 0, 1], \
			sig_done[in_sig]	; reflect write
  	br[notify_schlr_done#]

xp4#:  
	// update scheduler with updated counts of packets transmitted for port 4
	cap[write, out_txd_port, SCHEDULER_ME, $$txd_p4, 0, 1], \
			sig_done[in_sig]	; reflect write
  	br[notify_schlr_done#]

xp5#:
	// update scheduler with updated counts of packets transmitted for port 5
	cap[write, out_txd_port, SCHEDULER_ME, $$txd_p5, 0, 1], \
			sig_done[in_sig]	; reflect write
  	br[notify_schlr_done#]

xp6#:
  
	// update scheduler with updated counts of packets transmitted for port 6
	cap[write, out_txd_port, SCHEDULER_ME, $$txd_p6, 0, 1], \
			sig_done[in_sig]	; reflect write
  	br[notify_schlr_done#]

xp7#:  
	// update scheduler with updated counts of packets transmitted for port 7
	cap[write, out_txd_port, SCHEDULER_ME, $$txd_p7, 0, 1], \
			sig_done[in_sig]	; reflect write
  	br[notify_schlr_done#]

xp8#:  
	// update scheduler with updated counts of packets transmitted for port 8
	cap[write, out_txd_port, SCHEDULER_ME, $$txd_p8, 0, 1], \
			sig_done[in_sig]	; reflect write
  	br[notify_schlr_done#]

xp9#:  
	// update scheduler with updated counts of packets transmitted for port 9
	cap[write, out_txd_port, SCHEDULER_ME, $$txd_p9, 0, 1], \
			sig_done[in_sig]	; reflect write
   	br[notify_schlr_done#]

xp10#:
	// update scheduler with updated counts of packets transmitted for port 10
	cap[write, out_txd_port, SCHEDULER_ME, $$txd_p10, 0, 1], \
			sig_done[in_sig]	; reflect write
   	br[notify_schlr_done#]

xp11#: 
	// update scheduler with updated counts of packets transmitted for port 11
	cap[write, out_txd_port, SCHEDULER_ME, $$txd_p11, 0, 1], \
			sig_done[in_sig]	; reflect write
   	br[notify_schlr_done#]

xp12#: 
	// update scheduler with updated counts of packets transmitted for port 12
	cap[write, out_txd_port, SCHEDULER_ME, $$txd_p12, 0, 1], \
			sig_done[in_sig]	; reflect write
   	br[notify_schlr_done#]

xp13#: 
	// update scheduler with updated counts of packets transmitted for port 13
	cap[write, out_txd_port, SCHEDULER_ME, $$txd_p13, 0, 1], \
			sig_done[in_sig]	; reflect write
    	br[notify_schlr_done#]

xp14#:
	// update scheduler with updated counts of packets transmitted for port 14
	cap[write, out_txd_port, SCHEDULER_ME, $$txd_p14, 0, 1], \
			sig_done[in_sig]	; reflect write
  	br[notify_schlr_done#]

xp15#:
	// update scheduler with updated counts of packets transmitted for port 15
	cap[write, out_txd_port, SCHEDULER_ME, $$txd_p15, 0, 1], \
			sig_done[in_sig]	; reflect write

notify_schlr_done#:
	_pkt_tx_16p_set_signal(io_sigmask, in_sig)

.end
#endif	// #if IS_IXPTYPE(__IXP24XX)

#if IS_IXPTYPE(__IXP28XX)
	.reg remote $$txd_p0 

.begin 

	.reg	tmp

	alu[out_txd_port, --, B, lm1_pkt_txed] ; get txed pkts

	alu[tmp, (&remote($$txd_p0, SCHEDULER_ME)), +, in_port]
 	alu_shf[tmp, cap_reflect_addr_base, OR, tmp, <<2] 
	cap[write, out_txd_port, tmp, 0, 1], sig_done[in_sig]	
 	_pkt_tx_16p_set_signal(io_sigmask, in_sig)

.end 
#endif	// #if IS_IXPTYPE(__IXP28XX)

#endif	// #ifndef DISABLE_TX2SCHED_FEEDBACK

#endm	// _pkt_tx_16p_notify_scheduler()


/////////////////////////////////////////////////////////////////////////////// 
//
// 	_pkt_tx_16p_update_pkt_txed_info()
//
// Description: update packet trnsmitted count in local memory and also notify 
//				scheduler 
//
// Prerequisite:			local memory index 1 alrady set to queue status 
//							of the in_port
//
// Outputs:	
//      out_txd_port:		total packets transmitted by the port (sram 
//                          transfer register)
//
// Inputs/Outputs:  
//      io_sigmask:			signal mask 
//
// Inputs:  
//      in_port:			port to transmit the current mpacket 
//		in_sig:				signal used for reflect[write....]
//
// Constants:
//		None 
//
// Size:
//      12 instructions for jump table (for IXP2400 rev. A0/A1)
//      6 instructions for culated addressing mode 
//        except IXP2400 rev. A0/A1
//
///////////////////////////////////////////////////////////////////////////////

#macro 	_pkt_tx_16p_update_pkt_txed_info(out_txd_port, io_sigmask, in_sig, \
					in_port)

.begin

	alu[lm1_pkt_txed, lm1_pkt_txed, +, 1]  ; update pkt_txed for port

	_pkt_tx_16p_debug_incr_counter(@pkt_tx_16p_pkt_txed)

	//  notify scheduler (11 (ixp2400 A0/A1) or 5 (others)) 

	_pkt_tx_16p_notify_scheduler(out_txd_port, io_sigmask, in_sig, in_port)
		
.end

#endm	// _pkt_tx_16p_update_pkt_txed_info()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_advance_queue_head()
//
// Description: advance queue head after the last mpcket of one packet is sent
//              to the port 
//
// Prerequisite:			local memory index 1 alrady set to queue status 
//							of the in_port
// Outputs:							
//		None
//
// Inputs:  
//      None:		
//
// Constants:
//						 	None 
//
// Size:
//		6 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_advance_queue_head()

.begin 

	.reg	head_offset

	alu_shf[head_offset, HEAD_MASK, AND, lm1_port_queue_stat, \
				>>QUEUE_HEAD_SHFT]					; head_offset

	// advance queue head_offset

	alu[head_offset, head_offset, +, QUEUE_ENTRY_SIZE]	
	alu[--, head_offset, -, MAX_QUEUE_HEAD_TAIL]	; compare with upper bound
	ble[wrap_around_done#]							
	alu[head_offset, --, B, 0]						; wrap around
wrap_around_done#:

	ld_field[lm1_port_queue_stat, 0001, head_offset]

.end 

#endm	//_pkt_tx_16p_advance_queue_head()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_decrement_pkt_in_queue()
//
// Description: decrement packets in queue for the port, if necessary, also 
//              update global flag @ep, @vap
//
// Prerequisite:			local memory index 1 alrady set to queue status 
//							of the in_port
// Outputs:							
//		None
//
// Inputs/Outputs:  
//      io_empty_ports:		global empty ports flag	
//		io_virtual_ports:	global vitual queue available ports flag
//
// Inputs:  
//      in_port_bit:		bit for the port	
//
// Constants:
//						 	None 
//
// Size:
//		8 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_decrement_pkt_in_queue(io_empty_ports, io_virtual_ports, \
						in_port_bit)

.begin 

	.reg	tmp

	alu[tmp, PKT_IN_QUEUE_MASK, AND, lm1_port_queue_stat, \
														>>PKT_IN_QUEUE_SHFT]	 

	alu[tmp, tmp, -, 1]
	bne[update_empty_port_flag_done#]

	alu[io_empty_ports, io_empty_ports, OR, in_port_bit]		; add port
	alu[io_virtual_ports, io_virtual_ports, AND~, in_port_bit]	; remove port
	alu[io_virtual_ports, io_virtual_ports, AND, @port_mask]

update_empty_port_flag_done#:
	ld_field[lm1_port_queue_stat, 0100, tmp, <<PKT_IN_QUEUE_SHFT]	 

.end  

#endm	// _pkt_tx_16p_decrement_pkt_in_queue()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_update_rmnd_paylo_offset()
//
// Description: update remaining payload in this buffer and offset of the 
//              remaining payload in local memory
//
// Prerequisite:			local memory index 0 alrady set to queue entry
//							(queue head) being processed 
//
// Outputs:							
//		None
//
// Inputs/Outputs:  
//      io_empty_ports:		global empty ports flog	
//
// Inputs:  
//      in_buf_size:		buffer size before mpacket 1, 2, and 3
//		in_mpkt_1_size:		mpacket 1 size
//		in_mpkt_2_size:		mpacket 2 size
//		in_mpkt_3_size:		mpacket 3 size
//		in_offset:			buffer offset
//
// Constants:
//		None 
//
// Size:
//		5 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_update_rmnd_paylo_offset(in_buf_size, in_mpkt_1_size, \
					in_mpkt_2_size, in_mpkt_3_size, in_offset)
.begin
 
	.reg	tmp

	alu[tmp, in_mpkt_1_size, +, in_mpkt_2_size]
	alu[tmp, tmp, +, in_mpkt_3_size] ; total bytes transmitted by this thread

	alu[lm0_cur_buf_1, in_offset, +, tmp]	; offset of remainning payload

	alu[tmp, in_buf_size, -, tmp]		; payload remaining
	ld_field[lm0_cur_buf_1, 1100, tmp, <<PAYLO_RMND_LOC] 

.end  

#endm	// _pkt_tx_16p_update_rmnd_paylo_offset()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_get_dram_addr()
//
// Description: get the starting address of this buffer in dram
//
// Outputs:							
// 		out_dram_addr:	starting address of this buffer in dram
//
// Inputs:
//  	in_buf_hndl:	buffer handle
//
// Constants:
//		None 
//
// Size:
//		2 instruction
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_get_dram_addr(out_dram_addr, in_buf_hndl)

.begin

	.reg true_buf_hndl

	// get buffer handle 

	alu_shf[true_buf_hndl, --, b, in_buf_hndl, <<2]

    // use following dl_buf_get_data_from_meta macro from dispatch_loop.uc to
	// get dram address at the beginning of the buffer from true_dl_buf_hdl
	// now it takes one instruction

    dl_buf_get_data_from_meta[out_dram_addr, true_buf_hndl]

.end 

#endm	//  _pkt_tx_16p_get_dram_addr


///////////////////////////////////////////////////////////////////////////////
// 
// _pkt_tx_16p_check_wait_tbuf_not_full()
//
// Description:
// 	
//		Check whether tbuf is full to avoid overwritting tbuf, if full, wait 
//      till not full in not sop-eop mpkt case
//
// Outputs: 
//		None
//
// Inputs/Outputs:
//		None
//
// Inputs:  
//		in_tbufele:		next available tbuf element number allocated from 
//						scratch memory through unit operation
//		in_sig:			siganl used to read tx_sequence
//
// Constants:
//		None
//
// Size: 
//		8 instructions in not full case, otherwise indefinite 
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_check_wait_tbuf_not_full(in_tbufele, in_sig)

.begin

	.reg	tbuf_ele_for_check
	.reg	tmp

#if 1	

	
	alu[tbuf_ele_for_check, in_tbufele, AND, CHECK_TBUF_ELE_MASK]

	alu[tmp, tbuf_ele_for_check, -, $mpkts_sent]	; get difference between
													; current tbuf_ele and txed

	alu[tmp, tmp, and, CHECK_TBUF_ELE_MASK]			; mask out overflow value

	alu[--, tmp, -, TBUF_WAIT_THRESHOLD]    		; compare the difference 
													; with threshold 

	blt[packet_tx_tbuf_not_full#]					; tbuf not full, return

	// when the control goes here, it means tbuf is near full

read_tx_sequencer_loop#:

	// read the actual number of mpkts out of tbuf

	msf[read, $mpkts_sent, tx_seq_addr, 0, 1], sig_done[in_sig]	

wait_tx_seq_read_done#:

	br_!signal[in_sig, wait_tx_seq_read_done#]	


	alu[tmp, tbuf_ele_for_check, -, $mpkts_sent]	; get difference between 
													; current tbuf_ele and txed

	alu[tmp, tmp, and, CHECK_TBUF_ELE_MASK]			; mask out overflow value

	alu[--, tmp, -, TBUF_WAIT_THRESHOLD]   			; compare the difference 
													; with threshold 

	bge[read_tx_sequencer_loop#] 					; loop again

packet_tx_tbuf_not_full#:
end_of_tbuf_full_check#:

#endif // #if 0

.end		

#endm // end of _pkt_tx_16p_check_wait_tbuf_not_full


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_read_msf_reg_for_flow_control()
//
// Description: read PHY status for updating port flow control related info
//
// Outputs:							
//		None 
//
// Inputs:
//		None 
//
// Constants:
//		None 
//
// Size:
//		1 instruction
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_read_msf_reg_for_flow_control()

.begin

	// read MSF flow control status register for use in next phase 
	msf[read, $mphy_stat_0, tx_mphy_stat_addr,  0, 1], \
												sig_done[sig_msf_access_2]	

.end		

#endm	// _pkt_tx_16p_read_msf_reg_for_flow_control()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_get_tx_request()
//
// Description: get tx_request from scratch ring
//
// Outputs:							
//		out_tx_req:				tx request in sram xfer reg
//
// Inputs:
//		in_ring:				scratch ring for tx_request 
//		in_signal:				signal		
//
// Constants:
//		None 
//
// Size:
//		1 instruction
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_get_tx_request(out_tx_req, in_ring, in_signal)

.begin

	scratch[get, out_tx_req, in_ring, 0, 1], sig_done[in_signal]

.end

#endm	// _pkt_tx_16p_get_tx_request() 


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_prepare_tcw0_with_base()
//
// Description: prepare first transmit control word with a input base
//
// Outputs:							
//		out_tcw0:			first transmit control word 
//
// Inputs:
//		in_offsetmod8:		offset mod by 8
//		in_paylo_len:		payload length
//		in_port:			port to transmit
//		in_buf_handle:		buffer handle to free 
//  	in_tcw_base:		base for tcw
//
// Constants:
//		None 
//
//	Labels:
//		None
//
// Size:
//		3 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_prepare_tcw0_with_base(out_tcw0, in_offsetmod8, \
			in_paylo_len, in_port, in_tcw_base)

.begin

	alu[out_tcw0, in_tcw_base, OR, in_offsetmod8, \
											<<TCW0_PAYLOAD_OFFSET_LSB_LOC]
	alu[out_tcw0, out_tcw0, OR, in_paylo_len, <<TCW0_PAYLOAD_LEN_LSB_LOC]
	alu[out_tcw0, out_tcw0, OR, in_port]

.end

#endm	// _pkt_tx_16p_prepare_tcw0_with_base()

///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_free_buffer()
//
// Description: free buffer handle by dispatch loop macro dl_buf_free
//
// Outputs:							
// 		None
//
// Inputs:
//		in_buf_handle:				buffer handle to free 
//
// Constants:
//		None   
//
// Size:
//		4 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_free_buffer(in_buf_handle)

.begin

/*
 * Left this code to allow disabling buffers reuse
 */
#ifdef DISABLE_BUFFERS_REUSE_BY_CSIX
    
	dl_buf_free(in_buf_handle, BUF_FREE_LIST0)

#else //DISABLE_BUFFERS_REUSE_BY_CSIX    

    .reg	tmp
				
#ifndef UNIT_TEST

#if IS_IXPTYPE(__IXP28XX)

	// if possible, pass buffer to CSIX RX microblocks to enhance performance, 
	// else use dl_buf_free in dl_buf.uc to free buffer

 	br_inp_state[nn_full, default_buf_free#]
	alu_shf[tmp, in_buf_handle, AND~, 0xFF, <<24] 
	alu[*n$index++, tmp, or, 0xC0, <<24]

	br[free_buf_done#]

default_buf_free#:    
#endif	// #if IS_IXPTYPE(__IXP28XX)
	
	// use dl_buf_free in dl_buf.uc to free buffer handle
	
	dl_buf_free(in_buf_handle, BUF_FREE_LIST0)				; (4) 

free_buf_done#:

#else //#ifndef UNIT_TEST

	alu[tmp, --, B, 0]
	alu[tmp, tmp, +, 1]
	alu[tmp, tmp, +, 1]
	alu[tmp, tmp, +, 1]

#endif //#ifndef UNIT_TEST

#endif //DISABLE_BUFFERS_REUSE_BY_CSIX
    
.end

	_pkt_tx_16p_debug_incr_counter(@pkt_tx_16p_buffer_freed)

#endm	// _pkt_tx_16p_free_buffer()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_wakeup_next_thread()
//
// Description: wake up next thread
//
// Outputs:							
// 		None
//
// Inputs:
//  	in_sig_context_gpr:	next context gpr to write to SAME_ME_SIGAL to wake 
//							up next thread
// Constants:
//		None   
//
// Size:
//		1 instruction
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_wakeup_next_thread(in_sig_context_gpr)

.begin

	local_csr_wr[SAME_ME_SIGNAL, in_sig_context_gpr]

.end

#endm	// _pkt_tx_16p_wakeup_next_thread()

///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_set_lm_index_0_to_queue_head()
//
// Description: set local memory index 0 to the head of the queue of the port 
//
// Prerequisite:				local memory index 1 alrady set to queue status 
//								of the in_port
//
// Outputs: 
//		None
//
// Inputs:  
//      in_port:				port to save
//
// Constants:
//								None
//
//	Size: 6 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_set_lm_index_0_to_queue_head(in_port)
 
.begin
	
	.reg	tmp
	.reg	queue_entry_base_port

	_pkt_tx_16p_get_port_queue_entry_base(queue_entry_base_port, in_port); (3)
	
	alu_shf[tmp, HEAD_MASK, AND, lm1_port_queue_stat, \
				>>QUEUE_HEAD_SHFT]				; head_offset
	alu[tmp, queue_entry_base_port, +, tmp]	

	// get the pointer to the head of the queue, write active_lm_addr0 
	
	local_csr_wr[active_lm_addr_0, tmp]

.end 

#endm // _pkt_tx_16p_set_lm_index_0_to_queue_head()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_set_lm_index_0_to_queue_tail()
//
// Description: set local memory index 0 to tail of the queue of the port 
//
// Prerequisite:				local memory index 1 alrady set to queue status 
//								of the in_port
//
// Outputs: 
//		None
//
// Inputs:  
//      in_port:				port to save
//
// Constants:
//								None
//
//	Size: 6 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_set_lm_index_0_to_queue_tail(in_port)

.begin
	
	.reg	queue_entry_base_port
	.reg	tmp

	_pkt_tx_16p_get_port_queue_entry_base(queue_entry_base_port, in_port) ; (3)

	alu_shf[tmp, TAIL_MASK, AND, lm1_port_queue_stat, \
				>>QUEUE_TAIL_SHFT]			; tail_offset
	alu[tmp, tmp, +, queue_entry_base_port]

	// get the pointer to the tail of the queue, write active_lm_addr0 
	
	local_csr_wr[active_lm_addr_0, tmp]

.end 

#endm // _pkt_tx_16p_set_lm_index_0_to_queue_tail()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_increment_pkts_in_queue()
//
// Description: increments packets in queue
//
// Prerequisite:			local memory index 1 alrady set to queue status 
//							of the in_port
//
// Outputs:					
//		None 
//
// Inputs:  
//		None
//
//	Size:
//		2 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_increment_pkts_in_queue()

.begin

	.reg	 tmp

	alu[tmp, --, B, 1, <<PKT_IN_QUEUE_SHFT]	 
	alu[lm1_port_queue_stat, lm1_port_queue_stat, +, tmp]

.end

#endm	// _pkt_tx_16p_increment_pkts_in_queue() 



///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_advance_queue_tail()
//
// Description: advance queue tail
//
// Prerequisite:			local memory index 1 alrady set to queue status 
//							of the in_port
//
// Outputs:					
//		None 
//
// Inputs:  
//		None
//
//	Size:
//		6 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_advance_queue_tail()

.begin

	.reg 	tmp
	
	// advance queue tail, get current queue tail offset first  

	alu[tmp, TAIL_MASK, AND, lm1_port_queue_stat, >>QUEUE_TAIL_SHFT] 
	alu[tmp, tmp, +, QUEUE_ENTRY_SIZE]			; new queue tail offset
	alu[--, tmp, -, MAX_QUEUE_HEAD_TAIL]		; compare with upper bound
	ble[wrap_around_done#]							
	alu[tmp, --, B, 0]							; wrap around
wrap_around_done#:
	// update queue tail offset
	ld_field[lm1_port_queue_stat, 0010, tmp, <<QUEUE_TAIL_SHFT]

.end

#endm 	// _pkt_tx_16p_advance_queue_tail()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_increment_virtual_queue_count()
//
// Description: increment virtual queue count
//
// Prerequisite:			local memory index 1 alrady set to queue status 
//							of the in_port
//
// Outputs:					
//		None 
//
// Inputs:  
//		None
//
//	Size:
//		2 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro 	_pkt_tx_16p_increment_virtual_queue_count()

.begin

	.reg tmp
	
	// increase virtual count queue cunt by one
	alu[tmp, --, B, 1, <<VIRTUAL_QUEUE_SHFT]
	alu[lm1_port_queue_stat, lm1_port_queue_stat, +, tmp]

.end 
#endm	// _pkt_tx_16p_increment_virtual_queue_count()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_decrement_virtual_queue_count_lm0()
//
// Description: update virtual_queue_count and related global flags
//
// Prerequisite:			local memory index 0 alrady set to queue status 
//							of the port
//
// Outputs:					
//							None 
//
// Inputs:  
//		in_port:			port 
//
// Constants:
//		None 
//
//	Size:
//		11 instructions
//
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_decrement_virtual_queue_count_lm0(in_port)

.begin

	.reg	vq_cnt
	.reg	tmp
	
	alu[vq_cnt, VIRTUAL_QUEUE_MASK, AND, lm0_port_queue_stat, \
				>>VIRTUAL_QUEUE_SHFT]

	alu[vq_cnt, vq_cnt, -, 1]
	bne[decrement_done#]

	alu[--, in_port, or, 0]
	alu_shf[tmp, --, b, 1, <<indirect]

	// remve port from @vap
	alu[@vap, @vap, AND~, tmp]
	alu[@vap, @vap, AND, @port_mask]
	alu[@vpp, @vap, AND~ , @bfcp]
	alu[@vpp, @vpp, AND~ , @blocked_by_read_leftover]					
	alu[@vpp, @vpp, AND~ , @blocked_by_read_next_buf]

decrement_done#:

	ld_field[lm0_port_queue_stat, 1000, vq_cnt, <<VIRTUAL_QUEUE_SHFT]

.end 

#endm	// _pkt_tx_16p_decrement_virtual_queue_count_lm0()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_increment_virtual_queue_count_lm0()
//
// Description: update virtual_queue_count and related global flags
//
// Prerequisite:			local memory index 0 alrady set to queue status 
//							of the port
//
// Outputs:					
//							None 
//
// Inputs:  
//		in_port:			port 
//
// Constants:
//		None 
//
//	Size:
//		13 instructions
//
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_increment_virtual_queue_count_lm0(in_port)

.begin

	.reg	vq_cnt
	.reg	tmp
	
	alu[vq_cnt, VIRTUAL_QUEUE_MASK, AND, lm0_port_queue_stat, \
				>>VIRTUAL_QUEUE_SHFT]

	alu[vq_cnt, vq_cnt, +, 1]

	alu[--, in_port, or, 0]
	alu_shf[tmp, --, b, 1, <<indirect]

	// remve port from @vap
	alu[tmp, tmp, AND~, @ep]
	bne[increment_done#]

	alu[vq_cnt, --, B, 0]  	; since this port is empty, clear virtual count

	// remve port from @vap
	alu[@vap, @vap, AND~, tmp]
	alu[@vap, @vap, AND, @port_mask]
	alu[@vpp, @vap, AND~ , @bfcp]
	alu[@vpp, @vpp, AND~ , @blocked_by_read_leftover]					
	alu[@vpp, @vpp, AND~ , @blocked_by_read_next_buf]

increment_done#:

	ld_field[lm0_port_queue_stat, 1000, vq_cnt, <<VIRTUAL_QUEUE_SHFT]

.end 

#endm	// _pkt_tx_16p_increment_virtual_queue_count_lm0()

  
///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_set_port_not_empty()
//
// Description: set the port not empty in global flag
//
// Outputs:					
//		None 
//
// Inputs:  
//		in_port_bit:	bit for port
//
//	Size:
//		1 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_set_port_not_empty(in_port_bit)

.begin

	.reg	tmp

	// remove port from @ep and add port to @vap, also upadte @rp and @vpp
	// following temporary storage of in_port_bit to tmp is used to resolve
	// odd-sided GPR allocation conflict

	alu[tmp, --, B, in_port_bit] 			; in_port_bit
	alu[@ep, @ep, AND~, tmp]
	alu[@rp, @ep,  AND~, @bfcp]
	alu[@vap, @vap, OR, tmp]
	alu[@vpp, @vap, AND~ , @bfcp]
	alu[@vpp, @vpp, AND~ , @blocked_by_read_leftover]						
	alu[@vpp, @vpp, AND~ , @blocked_by_read_next_buf]		

.end 
#endm	// _pkt_tx_16p_set_port_not_empty()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_save_sop_to_lm()
//
// Description: save the whole packet from tx_request (no transimission yet)
//				to the tail of the queue entry for the port
//
// Prerequisite:			local memory index 1 alrady set to queue status 
//							of the in_port, local memory index 0 already set
//							to the tail of the port
//
// Outputs:					
//		None 
//
// Inputs:  
//		in_tx_request:		transmit request
//
// Constants:
//		None 
//
//	Size:
//		20 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _pkt_tx_16p_save_sop_to_lm(in_tx_request)

.begin 
	 
	.reg	tmp

	// following 3 macros just use local memory index 1, so guanrantee local 
	// memory index 0 will be set when it is used in this macro
	
	_pkt_tx_16p_increment_pkts_in_queue()			; (2)

	_pkt_tx_16p_advance_queue_tail()					; (6)

	_pkt_tx_16p_increment_virtual_queue_count()		; (2)

	dl_meta_get_offset[lm0_cur_buf_1]				; save offset
	dl_meta_get_buffer_size[tmp]
	ld_field[lm0_cur_buf_1, 1100, tmp, <<PAYLO_RMND_LOC] 

	ld_field_w_clr[lm0_cur_buf_0, 0111,  in_tx_request]
	
	dl_meta_get_buffer_next(tmp)					; next_buf_handle
	ld_field_w_clr[lm0_next_buf_0, 0111, tmp]
	 
 	alu[--, lm0_next_buf_0, -, IX_NULL]				; is null (0xFF)?
	beq[update_rd_next_buf_bit_done#]

	// set RD_NEXT_BUF_BIT

	alu_shf[lm0_cur_buf_0, lm0_cur_buf_0, OR, 1, <<RD_NEXT_BUF_BIT]		

update_rd_next_buf_bit_done#:
 
	// this is a complete new packet

 	alu_shf[lm0_cur_buf_0, lm0_cur_buf_0, OR, 1, <<SOP_FLAG_BIT]
		
#ifdef ADD_L2_HEADER
	dl_meta_get_nexthop_id[lm0_next_buf_1]			; save nexthop_id
#endif // #ifdef ADD_L2_HEADER

.end 
#endm // _pkt_tx_16p_save_sop_to_lm()

///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_save_not_sop_to_lm()
//
// Description: save the rest of the packet queue tail of the queue entry for 
//				the port aftrer trnsmission 3 mpackets
//
// Prerequisite:			local memory index 1 alrady set to queue status 
//							of the in_port, local memory index 0 already set
//							to the tail of the port
//
// Outputs:					
//							None 
//
// Inputs:  
//		in_port:			port 
//		in_buf_hndl:		buffer handle
//		in_next_buf_hndl:   next buffer handle
//      in_buf_size:		buffer size
//		in_mpkt_1_size:		mpacket 1 size
//		in_mpkt_2_size:		mpacket 2 size
//		in_mpkt_3_size:		mpacket 3 size
//		in_offset:			buffer offset
//
// Constants:
//		None 
//
//	Size:
//		13 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _pkt_tx_16p_save_not_sop_to_lm(in_port, in_buf_hndl, in_next_buf_hndl, \
		in_buf_size, in_mpkt_1_size, in_mpkt_2_size, in_mpkt_3_size, in_offset)

.begin 
	 
	.reg	tmp

	// following 3 macros just use local memory index 1, so guanrantee local 
	// memory index 0 will be set when it is used in this macro

	_pkt_tx_16p_increment_virtual_queue_count()		; (2)

   	_pkt_tx_16p_increment_pkts_in_queue()			; (2)

	_pkt_tx_16p_advance_queue_tail()					; (6)


	alu[tmp, in_mpkt_1_size, +, in_mpkt_2_size]
	alu[tmp, tmp, +, in_mpkt_3_size] ; total bytes transmitted by this thread

	alu[lm0_cur_buf_1, in_offset, +, tmp] ; offset of remainning payload

	alu[tmp, in_buf_size, -, tmp]		; payload remaining
	ld_field[lm0_cur_buf_1, 1100, tmp, <<PAYLO_RMND_LOC] 

	ld_field_w_clr[lm0_cur_buf_0, 0111,  in_buf_hndl]
	
	ld_field_w_clr[lm0_next_buf_0, 0111, in_next_buf_hndl]
	 
.end 
#endm // _pkt_tx_16p_save_not_sop_to_lm()

	 
///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_find_port_to_tx_from_lm_queue()
//
// Description: find first availbel port to tx, otherwise branch 
//											NO_PORT_AVAILABLE_LABEL 
//
// Outputs:					
//			out_tx_port:		port in the tx request
//			out_tx_port_bit:	bit corresponding to port = 1 << out_tx_port
//
// Inputs:  
//		None
//
// Labels:
//		NO_PORT_AVAILABLE_LABEL: 	//
//	Size:
//		8 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_find_port_to_tx_from_lm_queue(out_tx_port, \
									out_tx_port_bit, NO_PORT_AVAILABLE_LABEL)

.begin

	.reg	tmp
	.reg	tmp1
	
	// check whether any port is ready to transmit
	alu[tmp, @port_mask, AND~, @ep]				; not empty in lm_queue
	alu[tmp, tmp, AND~, @bfcp]
	alu[tmp, tmp, AND~ , @blocked_by_read_leftover]								
	alu[tmp, tmp, AND~ , @blocked_by_read_next_buf]	
	alu[tmp1, tmp, AND, @rr_vector]
	
	ffs[out_tx_port, tmp1]
	bne[port_found#]	
	
	ffs[out_tx_port, tmp]	
	beq[NO_PORT_AVAILABLE_LABEL]		; no port ready to tx

port_found#:
	alu[--, out_tx_port, or, 0]
	alu_shf[out_tx_port_bit, --, b, 1, <<indirect]

	//	updating round robin vecror (@rr_vector)
	_pkt_tx_16p_update_round_robin_vector(@rr_vector, out_tx_port)

.end

#endm	// _pkt_tx_16p_find_port_to_tx_from_lm_queue() 

///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_next_buffer_become_current_buffer()
//
// Description: replace curent buffer data with next buffer data in local 
//              memory
//
// Prerequisite:			local memory index 1 alrady set to queue status 
//							of the in_port, local memory index 0 already set 
//							to the head of the port
//
// Outputs:							
//							None
//
// Inputs:  
//							None
//
// Constants:
//							None 
//
// Size: 
//		5 instructions. (Worst case cycle count)
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_next_buffer_become_current_buffer()

.begin
	
	.reg	tmp

	alu[lm0_cur_buf_0, --, b, lm0_next_buf_0]
	alu[lm0_cur_buf_1, --, b, lm0_next_buf_1]
	alu[tmp, --, B, lm1_lov_stat_next_next_buf]
	alu[lm0_next_buf_0, tmp, AND~, 0xFF, <<24]	; clear MSF byte		
	alu[lm0_next_buf_1, --, b, 0]				; reset

.end

#endm // end of macro 	_pkt_tx_16p_next_buffer_become_current_buffer()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_process_critcal_tx_request_phase_1()
//
// Description: process tx request for critical case (sop-eop mpacket) in 
//				phase 1, including setting lm index 1 and updating flow control
//				to assure not overflooding PHY, branch out for other cases to 
//				reduce instruction for critical case
//
// Outputs:							
// 		out_mpkt:			mpacket to transmit in this iteration
//  	out_buf_handle:		buffer handle
//  	out_offset:			buffer offset
//		out_offset_mod_8:	offset mod by 8
//
// Inputs:
//		in_port:		port to transmit
//		in_port_bit:	port bit for port to transmit
//		in_tx_request:	tx request
//
// Constants:
//		None   
//
//	Labels:
//		NOT_SOP_EOP_LABEL:	symbolic address to go when tx_request is not
//							sop-eop mpacket
//
// Size:
//		(36 for ixp2400 A0/A1 / 30 for others) instructions for one mpacket
//
///////////////////////////////////////////////////////////////////////////////

#macro	_pkt_tx_16p_process_critcal_tx_request_phase_1(out_mpkt, \
			out_buf_handle, out_offset, out_offset_mod_8, in_port, \
			in_port_bit, in_tx_request, NOT_SOP_EOP_LABEL) 

.begin

	.reg	tmp

	_pkt_tx_16p_set_lm_index_1_to_port_status_info(in_port)	; (3)
	ld_field_w_clr[out_buf_handle, 0111, in_tx_request]
	dl_meta_get_offset[out_offset]

#ifdef ADD_L2_HEADER
	dl_meta_get_nexthop_id[nexthop_id]
#ifdef ETHERNET_TX
	immed32(tmp, L2_TABLE_SRAM_BASE)

	// read it even L2 header exists to reduce instruction to check in 
	// critical path 

	_pkt_tx_16p_read_l2_table_entry(sig_mask_1, tmp, nexthop_id, \
													sig_l2_table)

#endif // ETHERNET_TX
#endif	// #ifdef ADD_L2_HEADER

	alu[out_offset_mod_8, out_offset, AND, CONST_07]

	// use mpkt_1_size to keep packet_size, if this tx_request is not sop-eop
	// mpacket, mpkt_1_size will be updated to right value later	

	dl_meta_get_packet_size[mpkt_1_size]		
	alu[tmp, out_offset_mod_8, +, mpkt_1_size]
#ifndef STRIP_4BYTES_CRC
	alu[--, tmp, -, SINGLE_MPKT_MAX_SIZE]
#else	// STRIP_4BYTES_CRC
	alu[--, tmp, -, SINGLE_MPKT_PLUS_CRC_MAX_SIZE]
#endif	// STRIP_4BYTES_CRC


	bgt[NOT_SOP_EOP_LABEL]

#ifdef STRIP_4BYTES_CRC
	alu[mpkt_1_size, mpkt_1_size, -, 4] 	; truncate last 4 bytes CRC
#endif	// #ifdef STRIP_4BYTES_CRC

	alu[out_mpkt, --,  B, 1]
	scratch[test_and_incr, $avail_tbuf_ele_1, tbuf_scr_addr, 0], \
			sig_done[sig_scratch_access_2]	    	; allocate one tbuf element
	alu[exe_stat_flag, --, B, (ESF_FREE_BUF + ESF_EOP_MPKT_THREAD)]

	_pkt_tx_16p_update_flow_control_on_transmit(1, in_port, in_port_bit) ; (13)

	// update pkt txed for port and notify scheduler (12 (ixp2400 A0/A1) or 6
	// (others)) 

	_pkt_tx_16p_update_pkt_txed_info($txed_port, sig_mask_1, \
													sig_reflect_write, in_port)

.end 										

#endm	// _pkt_tx_16p_process_critcal_tx_request_phase_1()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_process_non_critcal_tx_request_phase_1()
//
// Description: process tx request for non-critical case (not sop-eop mpacket) 
//				in phase 1, decide how many mpacket will be transmitted in this
//				iteratio
//
// Outputs:							
// 		out_mpkt:			mpacket to transmit in this iteration
//
// Inputs:
//		in_buf_handle:		buffer handle
//		in_off_set:			packet offset
//  	in_offset_mod_8:	packet offset mod by 8
//		in_port:			port to transmit
//		in_port_bit:		port bit for port to transmit
//
// Constants:
//		None
//
// Size:
//		instructions
//
//
///////////////////////////////////////////////////////////////////////////////

#macro	_pkt_tx_16p_process_non_critcal_tx_request_phase_1(out_mpkt, \
			in_buf_handle, in_off_set, in_offset_mod_8, in_port, in_port_bit) 
.begin	

	.reg	buf_size
	.reg	next_buf_handle
	.reg	tmp

	dl_meta_get_buffer_size(buf_size)
	dl_meta_get_buffer_next(next_buf_handle) ; next_buf_handle

	alu[--, next_buf_handle, -, IX_NULL]	; is next_buf_handle null (0xFF)?
	bne[not_last_buffer_in_this_packet#]

#ifdef STRIP_4BYTES_CRC
	alu[buf_size, buf_size, -, 4] 	; truncate last 4 bytes CRC
#endif	// #ifdef STRIP_4BYTES_CRC

	
	_pkt_tx_16p_decide_sop_mpkt_size_last_buf(mpkt_1_size, buf_size, \
					in_offset_mod_8)

	// decide whether fitting into 2 mpacket and packet 2 size

	_pkt_tx_16p_decide_mpkt_2_size_last_buf(mpkt_2_size, buf_size, \
		mpkt_1_size, in_off_set, check_three_mpkts_fit_last_buffer#)	; (8)
				 
	alu[out_mpkt, --,  B, 2]
	alu[$avail_tbuf_ele_1, --, B, 2]		; alloacte 2 tbuf elements
	_pkt_tx_16p_allocate_tbuf_from_scratch_memory($avail_tbuf_ele_1, \
			sig_mask_1, sig_scratch_access_2)  ; (1)

	// (10 (ixp2400 A0/A1) or 4 (others)) 

	alu[exe_stat_flag, --, B, (ESF_FREE_BUF + ESF_EOP_MPKT_THREAD)]
	_pkt_tx_16p_update_flow_control_on_transmit(2, in_port, in_port_bit) ; (12)

	// update pkt txed for port and notify scheduler (11 (ixp2400 A0/A1) or 5
	// (others)) 

	_pkt_tx_16p_update_pkt_txed_info($txed_port, sig_mask_1, \
												sig_reflect_write, in_port)
	br[done#]

check_three_mpkts_fit_last_buffer#:
	alu[$avail_tbuf_ele_1, --, B, 3]		; alloacte 3 tbuf elements
	_pkt_tx_16p_allocate_tbuf_from_scratch_memory($avail_tbuf_ele_1, \
			sig_mask_1, sig_scratch_access_2)  ; (1)
	_pkt_tx_16p_update_flow_control_on_transmit(3, in_port, in_port_bit) ; (12)

	// decide whether fitting into 3 mpacket and packet 3 size

	_pkt_tx_16p_decide_mpkt_3_size_last_buf(mpkt_3_size, mpkt_2_size, \
			buf_size, mpkt_1_size, in_off_set, \
			more_than_3_mpkts_last_buffer#)					; (10)

	alu[out_mpkt, --,  B, 3]
	alu[exe_stat_flag, --, B, (ESF_FREE_BUF + ESF_EOP_MPKT_THREAD)]

	// update pkt txed for port and notify scheduler (11 (ixp2400 A0/A1) or 5
	// (others)) 

	_pkt_tx_16p_update_pkt_txed_info($txed_port, sig_mask_1, \
												sig_reflect_write, in_port)
	br[done#]

more_than_3_mpkts_last_buffer#: 
	alu[out_mpkt, --,  B, 4]
	alu[exe_stat_flag, --, B, 0]
	
  	// set local memory index 0 to tail of the queue

	_pkt_tx_16p_set_lm_index_0_to_queue_tail(in_port)		; (6)

	// save remaining packet to loacl memory
	 
	_pkt_tx_16p_save_not_sop_to_lm(in_port, in_buf_handle, next_buf_handle, \
		buf_size, mpkt_1_size, mpkt_2_size, mpkt_3_size, in_off_set)

	br[update_global_flags#]

	// the buffer being handled is not last buffer

not_last_buffer_in_this_packet#:

	//  read meta data for next buffer and set flag 

	_pkt_tx_16p_read_next_buf_meta_data($dl_meta, sig_mask_1, \
								next_buf_handle, sig_sram_access_1)

	alu[exe_stat_flag, --, B, 1, <<ESF_READ_NEXT_BUF_META_BIT]

	alu[@blocked_by_read_next_buf, @blocked_by_read_next_buf, OR, in_port_bit]

	_pkt_tx_16p_decide_sop_mpkt_size_not_last_buf(mpkt_1_size, \
				leftover_len, leftover_offset, buf_size, in_off_set, \
				in_offset_mod_8, check_two_mpkts_fit_not_last_buffer#)	

	alu[out_mpkt, --,  B, 1]
	alu[exe_stat_flag, --, B, 0]
	scratch[test_and_incr, $avail_tbuf_ele_1, tbuf_scr_addr, 0], \
			sig_done[sig_scratch_access_2]	    	; allocate one tbuf element
	_pkt_tx_16p_update_flow_control_on_transmit(1, in_port, in_port_bit) ; (12)
		
	br=byte[leftover_len, 0, 0, sop_one_mpkt_read_leftover_done#]

	_pkt_tx_16p_read_leftover_from_dram(sig_mask_1, in_buf_handle, \
			leftover_len, leftover_offset, sig_dram_access_1)	
	alu_shf[exe_stat_flag, exe_stat_flag, or, 1, <<ESF_READ_LEFTOVER_BIT]
	alu[@blocked_by_read_leftover, @blocked_by_read_leftover, OR, in_port_bit]

sop_one_mpkt_read_leftover_done#:
	br[done#]

check_two_mpkts_fit_not_last_buffer#:

	_pkt_tx_16p_decide_mpkt_2_size_not_last_buf(mpkt_2_size, leftover_len, \
		leftover_offset, buf_size, mpkt_1_size, in_off_set, \
		check_three_mpkts_fit_not_last_buffer#)	

	alu[out_mpkt, --,  B, 2]
	alu[$avail_tbuf_ele_1, --, B, 2]		; alloacte 2 tbuf elements
	_pkt_tx_16p_allocate_tbuf_from_scratch_memory($avail_tbuf_ele_1, \
			sig_mask_1, sig_scratch_access_2)  ; (1)
	alu[exe_stat_flag, exe_stat_flag, OR, 1, <<ESF_FREE_BUF_BIT]
	_pkt_tx_16p_update_flow_control_on_transmit(2, in_port, in_port_bit) ; (12)
		
	br=byte[leftover_len, 0, 0, two_mpkts_read_leftover_done#]

	_pkt_tx_16p_read_leftover_from_dram(sig_mask_1, in_buf_handle, \
			leftover_len, leftover_offset, sig_dram_access_1)	
	alu_shf[exe_stat_flag, exe_stat_flag, or, 1, <<ESF_READ_LEFTOVER_BIT]
	alu[@blocked_by_read_leftover, @blocked_by_read_leftover, OR, in_port_bit]

two_mpkts_read_leftover_done#:
	br[done#]

check_three_mpkts_fit_not_last_buffer#:

	alu[$avail_tbuf_ele_1, --, B, 3]		; alloacte 3 tbuf elements
	_pkt_tx_16p_allocate_tbuf_from_scratch_memory($avail_tbuf_ele_1, \
			sig_mask_1, sig_scratch_access_2)  ; (1)
	_pkt_tx_16p_update_flow_control_on_transmit(3, in_port, in_port_bit) ; (12)

	// decide whether fitting into 3 mpacket and packet 3 size

	_pkt_tx_16p_decide_mpkt_3_size_not_last_buf(mpkt_3_size, leftover_len, \
			leftover_offset, buf_size, mpkt_1_size, in_off_set, mpkt_2_size, \
			more_than_3_mpkts_not_last_buffer#)

	alu[out_mpkt, --,  B, 3]
	alu[exe_stat_flag, exe_stat_flag, OR, 1, <<ESF_FREE_BUF_BIT]

	br=byte[leftover_len, 0, 0, three_mpkts_read_leftover_done#]

	_pkt_tx_16p_read_leftover_from_dram(sig_mask_1, in_buf_handle, \
			leftover_len, leftover_offset, sig_dram_access_1)	
	alu_shf[exe_stat_flag, exe_stat_flag, or, 1, <<ESF_READ_LEFTOVER_BIT]
	alu[@blocked_by_read_leftover, @blocked_by_read_leftover, OR, in_port_bit]

three_mpkts_read_leftover_done#:

	br[done#]

more_than_3_mpkts_not_last_buffer#: 
	alu[out_mpkt, --,  B, 4]
	
  	// set local memory index 0 to tail of the queue

	_pkt_tx_16p_set_lm_index_0_to_queue_tail(in_port)		; (6)

	// save remaining packet to loacl memory
	 
	_pkt_tx_16p_save_not_sop_to_lm(in_port, in_buf_handle, next_buf_handle, \
		buf_size, mpkt_1_size, mpkt_2_size, mpkt_3_size, in_off_set)

update_global_flags#:

	// remove port from @ep and add port to @vap, also upadte @rp and @vpp
	// following temporary storage of in_port_bit to tmp is used to resolve
	// odd-sided GPR allocation conflict

	alu[tmp, --, B, in_port_bit] 			; in_port_bit
	alu[@ep, @ep, AND~, tmp]
	alu[@rp, @ep,  AND~, @bfcp]
	alu[@vap, @vap, OR, tmp]
	alu[@vpp, @vap, AND~ , @bfcp]
	alu[@vpp, @vpp, AND~ , @blocked_by_read_leftover]						
	alu[@vpp, @vpp, AND~ , @blocked_by_read_next_buf]			

done#:

.end  

#endm	// _pkt_tx_16p_process_non_critcal_tx_request_phase_1()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_read_next_buf_meta_data_from_sram()  
//
// Description: read the first to third longwords of meta data for next buffer
//				from sram
//
// Outputs:							
//		out_xmeta:			the sram transfer registers to keep meta data 
//
// Inputs/Outputs:  
//		io_sigmask:			signal mask to be added with sig_sram_read_1
// 
// Inputs:  
//		in_buf_handle:		buffer handle to get meta data
//		in_sig:				sigal for sram read
//
// Constants:
//		none   
//
// Size:
//		4 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _pkt_tx_16p_read_next_buf_meta_data(out_xmeta, io_sigmask, \
														in_buf_handle, in_sig)
.begin

	// read meta data by dispatch_loop macro (3)

	dl_meta_load_cache(in_buf_handle, out_xmeta, in_sig, 0, 3)
	_pkt_tx_16p_set_signal(io_sigmask, in_sig)

.end
#endm	// _pkt_tx_16p_read_next_buf_meta_data()

	
///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_save_next_buf_meta_data()  
//
// Description: save meta data for next buffer to local memory
//
// Prerequisite:			local memory index 1 alrady set to queue status 
//							of the in_port, local memory index 0 alrady set to 
//							queue_tail
// Outputs:							
//		None
//
// Inputs:  
//		None
//
// Constants:
//		none   
//
// Size:
//		8 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _pkt_tx_16p_save_next_buf_meta_data()
.begin
	
	.reg	tmp
	.reg	nn_buf_hndl								; next_buf_hndl of next buf
	.reg    next_buf_size

#ifndef STRIP_4BYTES_CRC
	// hold refernce to local memory index 0 after 3 instructions to let local 
	// memory index to settle down

	dl_meta_get_buffer_size(next_buf_size)			; buffer_size of next buf

	dl_meta_get_buffer_next(nn_buf_hndl)			; next_buf_hndl of next buf
	alu[--, nn_buf_hndl, -, IX_NULL]
	beq[update_read_next_buf_flag_done#]
	alu_shf[lm0_next_buf_0, lm0_next_buf_0, OR, 1, <<RD_NEXT_BUF_BIT] 
update_read_next_buf_flag_done#:

	dl_meta_get_offset(lm0_next_buf_1)				; offset of next buf
	ld_field[lm0_next_buf_1, 1100, next_buf_size, <<PAYLO_RMND_LOC] 

	// keep leftover_status in bit 31 to bit 24

	alu[tmp, --, B, lm1_lov_stat_next_next_buf]
	ld_field[tmp, 0111, nn_buf_hndl]		; replace next next buffer handle 
	alu[lm1_lov_stat_next_next_buf, --, B, tmp]

#else	// STRIP_4BYTES_CRC

	// hold refernce to local memory index 0 after 3 instructions to let local 
	// memory index to settle down

	dl_meta_get_buffer_size(next_buf_size)			; buffer_size of next buf

	dl_meta_get_buffer_next(nn_buf_hndl)			; next_buf_hndl of next buf
	alu[--, nn_buf_hndl, -, IX_NULL]
	bne[update_read_next_buf_flag#]
	
	// next bufer handle of next buffer handle is null,  now check the buffer 
	// size of next buffer is greater than 4 bytes (CRC), if it is greater than
	// 4 bytes, just trucate 4 bytes CRC from next_buf_size

	alu[--, next_buf_size, -, 4]
	ble[less_than_4_bytes#]
	alu[next_buf_size, next_buf_size, -, 4]		; truncate last 4 bytes CRC
	br[update_read_next_buf_flag_done#]

less_than_4_bytes#:

	// buffer size of next buffer less  or equal to 4 bytes, then truncation 
	// of the last 4 bytes CRC in this packet, will reduce the remaining buffer
	// size of this buffer by (4 -  next_buf_size), and we need to change the 
	// next buffer handle of current_buf_handle to IX_NULL and free next buffer
	// handle, since all threads runs in sequential folding phases, and further
	// processing of current buffer is bloacked by this 
	// _pkt_tx_16p_save_next_buf_meta_data(), this update in phase 2 should be
	// safe

	ld_field_w_clr[tmp, 0011, lm0_cur_buf_1, >>PAYLO_RMND_LOC]	
	alu[tmp, tmp, +, next_buf_size]
	alu[tmp, tmp, -, 4]
	ld_field[lm0_cur_buf_1, 1100, tmp, <<PAYLO_RMND_LOC]
	
	// free  next buffer handle
	 
	ld_field_w_clr[nn_buf_hndl, 0111, lm0_next_buf_0]
	_pkt_tx_16p_free_buffer(nn_buf_hndl) 

	alu[tmp, --, B, IX_NULL] 
	ld_field[lm0_next_buf_0, 0111, tmp]
	br[save_next_buf_meta_data_done#]

update_read_next_buf_flag#:
	alu_shf[lm0_next_buf_0, lm0_next_buf_0, OR, 1, <<RD_NEXT_BUF_BIT] 
update_read_next_buf_flag_done#:

	dl_meta_get_offset(lm0_next_buf_1)				; offset
	ld_field[lm0_next_buf_1, 1100, next_buf_size, <<PAYLO_RMND_LOC] 

	// keep leftover_status in bit 31 to bit 24

	alu[tmp, --, B, lm1_lov_stat_next_next_buf]
	ld_field[tmp, 0111, nn_buf_hndl]		; replace next next buffer handle 
	alu[lm1_lov_stat_next_next_buf, --, B, tmp]

save_next_buf_meta_data_done#:

#endif	// #ifndef STRIP_4BYTES_CRC

.end	
#endm // _pkt_tx_16p_save_next_buf_meta_data()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_process_pkt_in_lm_queue_phase_1()
//
// Description: process packet in local memory queue in phase 1, including 
//				setting lm index 1 to port status, deciding how many mpacket
//				to transmit, and updating flow control to assure not 
//				overflooding PHY
//
// Outputs:							
// 		out_mpkt:		mpacket to transmitted in this iteration
//  	out_buf_handle:	buffer handle
//  	out_offset:		buffer offset
//  	out_offset:		buffer offset mod by 8
//
// Inputs:
//		in_port:		port to transmit
//		in_port_bit:	port bit for port to transmit
//
// Constants:
//		None   
//
// Size:
//		(41 for ixp2400 A0/A1 / 35 for others) instructions  for one mpacket
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_process_pkt_in_lm_queue_phase_1(out_mpkt, out_buf_handle, \
			out_offset, out_offset_mod_8, in_port, in_port_bit)

.begin

	.reg 	tmp
	.reg 	leftover_to_tx_len		; length of leftover to tx first 
	.reg	buf_size
	.reg 	next_buf_handle

	_pkt_tx_16p_set_lm_index_1_to_port_status_info(in_port)	; (3)
	nop
	nop
	nop

	// set local memory index 0 to queue head of the port

	_pkt_tx_16p_set_lm_index_0_to_queue_head(in_port)			; (6)
	nop
	nop
	nop

	ld_field_w_clr[out_buf_handle, 0111, lm0_cur_buf_0]
	ld_field_w_clr[buf_size, 0011, lm0_cur_buf_1, >>PAYLO_RMND_LOC]
	ld_field_w_clr[out_offset, 0011, lm0_cur_buf_1]
	alu[out_offset_mod_8, out_offset, AND, CONST_07]

	ld_field_w_clr[next_buf_handle, 0111, lm0_next_buf_0]
	
	//check whether need to read next buffer meta data

	br_bclr[lm0_cur_buf_0, RD_NEXT_BUF_BIT, read_next_buf_done#]

	_pkt_tx_16p_read_next_buf_meta_data($dl_meta, sig_mask_1, \
								next_buf_handle, sig_sram_access_1)	

	// clear rd_next_buf_bit in *l$index0[CUR_BUF_0]

	alu[lm0_cur_buf_0, lm0_cur_buf_0, AND~, 1, <<RD_NEXT_BUF_BIT]

	alu[exe_stat_flag, exe_stat_flag, OR , 1, <<ESF_READ_NEXT_BUF_META_BIT]

	alu[@blocked_by_read_next_buf, @blocked_by_read_next_buf, OR, in_port_bit]

read_next_buf_done#:

	alu[--, next_buf_handle, -, IX_NULL]
	bne[last_buf_bit_flag_set_done#]

	alu[exe_stat_flag, exe_stat_flag, OR , 1, <<ESF_LAST_BUF_BIT]

last_buf_bit_flag_set_done#:


	br_bclr[lm0_cur_buf_0, SOP_FLAG_BIT, not_sop#]

	// clear sop_flag in *l$index0[CUR_BUF_0] and set ESF_SOP_MPKT_THREAD_BIT
	// in exe_stat_flag

	alu[lm0_cur_buf_0, lm0_cur_buf_0, AND~, 1, <<SOP_FLAG_BIT]
	alu[exe_stat_flag, exe_stat_flag, OR , 1, <<ESF_SOP_MPKT_THREAD_BIT]

#ifdef ADD_L2_HEADER
	ld_field_w_clr[nexthop_id, 0011, lm0_next_buf_1]
#ifdef ETHERNET_TX
	immed32(tmp, L2_TABLE_SRAM_BASE)

	// read it even L2 header exists to reduce instruction to check in 
	// critical path 

	_pkt_tx_16p_read_l2_table_entry(sig_mask_1, tmp, nexthop_id, \
													sig_l2_table)
#endif // ETHERNET_TX
#endif	// #ifdef ADD_L2_HEADER

	br_bclr[exe_stat_flag, ESF_LAST_BUF_BIT, sop_mpkt_not_last_buffer#]

sop_mpkt_in_only_one_buffer_packet#:

	// since only one buffer, so buffer_size is equql to packet_size

	// instead of "alu[tmp, out_offset_mod_8, +, buf_size]", following two 
	// instructions are used to resolve odd-sided gpr allocation conflict

	alu[tmp, --, B, out_offset_mod_8]
	alu[tmp, tmp, +, buf_size]

#ifndef STRIP_4BYTES_CRC
	alu[--, tmp, -, SINGLE_MPKT_MAX_SIZE]
#else	// STRIP_4BYTES_CRC
	alu[--, tmp, -, SINGLE_MPKT_PLUS_CRC_MAX_SIZE]
#endif	// #ifndef STRIP_4BYTES_CRC
	ble[it_is_sop_eop_mkpkt#]

#ifdef STRIP_4BYTES_CRC
	alu[buf_size, buf_size, -, 4] 	; truncate last 4 bytes CRC
#endif	// #ifdef STRIP_4BYTES_CRC


	_pkt_tx_16p_decide_sop_mpkt_size_last_buf(mpkt_1_size, buf_size, \
														out_offset_mod_8)
	bgt[check_two_mpkts_fit_last_buffer#]

it_is_sop_eop_mkpkt#:
	alu[out_mpkt, --,  B, 1]
#ifndef STRIP_4BYTES_CRC
	alu[mpkt_1_size, --, B, buf_size]		; packet_size for sop_eop_mpkt 
#else	// STRIP_4BYTES_CRC
	alu[mpkt_1_size, buf_size, -, 4]		; packet_size for sop_eop_mpkt and
											; truncate last 4 bytes
#endif	// #ifndef STRIP_4BYTES_CRC
	scratch[test_and_incr, $avail_tbuf_ele_1, tbuf_scr_addr, 0], \
			sig_done[sig_scratch_access_2]	    	; allocate one tbuf element
	alu[exe_stat_flag, exe_stat_flag, OR, 1, <<ESF_FREE_BUF_BIT]
	alu[exe_stat_flag, exe_stat_flag, OR, 1, <<ESF_EOP_MPKT_THREAD_BIT]

	_pkt_tx_16p_update_flow_control_on_transmit(1, in_port, in_port_bit) ; (12)

	// update pkt txed for port and notify scheduler (11 (ixp2400 A0/A1) or 5
	// (others)) 

	_pkt_tx_16p_update_pkt_txed_info($txed_port, sig_mask_1, \
												sig_reflect_write, in_port)
	br[advance_queue_head_update_global_flags#]

sop_mpkt_not_last_buffer#:

	_pkt_tx_16p_decide_sop_mpkt_size_not_last_buf(mpkt_1_size, \
				leftover_len, leftover_offset, buf_size, out_offset, \
				out_offset_mod_8, check_two_mpkts_fit_not_last_buffer#)	

	alu[out_mpkt, --,  B, 1]
	alu[exe_stat_flag, --, B, 0]
	scratch[test_and_incr, $avail_tbuf_ele_1, tbuf_scr_addr, 0], \
			sig_done[sig_scratch_access_2]	    	; allocate one tbuf element
	_pkt_tx_16p_update_flow_control_on_transmit(1, in_port, in_port_bit) ; (12)
		
	br=byte[leftover_len, 0, 0, sop_one_mpkt_read_leftover_done#]

	_pkt_tx_16p_read_leftover_from_dram(sig_mask_1, out_buf_handle, \
			leftover_len, leftover_offset, sig_dram_access_1)	
	alu_shf[exe_stat_flag, exe_stat_flag, or, 1, <<ESF_READ_LEFTOVER_BIT]
	alu[@blocked_by_read_leftover, @blocked_by_read_leftover, OR, in_port_bit]

sop_one_mpkt_read_leftover_done#:

	br[update_global_flags#]

// not sop - sop mpacket already transmitted

not_sop#:

	alu[leftover_to_tx_len, --, B, 0]		; reset to 0

	alu[tmp, --, B, lm1_lov_stat_next_next_buf]
	br_bclr[tmp, LM_LEFTOVER_FLAG_BIT, no_leftover#]

	alu[exe_stat_flag, exe_stat_flag, OR , 1, <<ESF_LEFTOVER_TO_TX_BIT]

	// clear lm_leftover_flag_bit in local memory

	alu[tmp, --, B, lm1_lov_stat_next_next_buf]
	alu[tmp, tmp, AND~, 1, <<LM_LEFTOVER_FLAG_BIT]
	alu[lm1_lov_stat_next_next_buf, --, B, tmp]

	// get leftover length

	_pkt_tx_16p_get_leftover_to_tx_len(leftover_to_tx_len)

no_leftover#:

	br_bclr[exe_stat_flag, ESF_LAST_BUF_BIT, not_last_buffer_in_this_packet#]

last_buffer_in_this_packet#:

	br_bclr[exe_stat_flag, ESF_LEFTOVER_TO_TX_BIT, \
										mpkt1_size_no_leftover_last_buffer#]

	_pkt_tx_16p_decide_mpkt_1_size_with_leftover_last_buf(mpkt_1_size, \
		leftover_to_tx_len, buf_size, out_offset, out_offset_mod_8, \
		check_two_mpkts_fit_last_buffer#)

		br[only_one_mpkt_last_buffer#]

mpkt1_size_no_leftover_last_buffer#:

	_pkt_tx_16p_decide_mpkt_1_size_no_leftov_last_buf(mpkt_1_size, buf_size, \
			out_offset, out_offset_mod_8, check_two_mpkts_fit_last_buffer#)

only_one_mpkt_last_buffer#:

	alu[out_mpkt, --,  B, 1]
	scratch[test_and_incr, $avail_tbuf_ele_1, tbuf_scr_addr, 0], \
			sig_done[sig_scratch_access_2]	    	; allocate one tbuf element
	alu[exe_stat_flag, exe_stat_flag, OR, 1, <<ESF_FREE_BUF_BIT]
	alu[exe_stat_flag, exe_stat_flag, OR, 1, <<ESF_EOP_MPKT_THREAD_BIT]

	_pkt_tx_16p_update_flow_control_on_transmit(1, in_port, in_port_bit) ; (12)

	// update pkt txed for port and notify scheduler (11 (ixp2400 A0/A1) or 5
	// (others)) 

	_pkt_tx_16p_update_pkt_txed_info($txed_port, sig_mask_1, \
												sig_reflect_write, in_port)
	br[advance_queue_head_update_global_flags#]

check_two_mpkts_fit_last_buffer#: 


	// decide whether fitting into 2 mpacket and packet 2 size

	_pkt_tx_16p_decide_mpkt_2_size_last_buf(mpkt_2_size, buf_size, \
		mpkt_1_size, out_offset, check_three_mpkts_fit_last_buffer#)	; (8)
				 
	alu[out_mpkt, --,  B, 2]
	alu[$avail_tbuf_ele_1, --, B, 2]		; alloacte 2 tbuf elements
	_pkt_tx_16p_allocate_tbuf_from_scratch_memory($avail_tbuf_ele_1, \
			sig_mask_1, sig_scratch_access_2)  ; (1)

	// (10 (ixp2400 A0/A1) or 4 (others)) 

	alu[exe_stat_flag, exe_stat_flag, OR, 1, <<ESF_FREE_BUF_BIT]
	alu[exe_stat_flag, exe_stat_flag, OR, 1, <<ESF_EOP_MPKT_THREAD_BIT]
	_pkt_tx_16p_update_flow_control_on_transmit(2, in_port, in_port_bit) ; (12)

	// update pkt txed for port and notify scheduler (11 (ixp2400 A0/A1) or 5
	// (others)) 

	_pkt_tx_16p_update_pkt_txed_info($txed_port, sig_mask_1, \
												sig_reflect_write, in_port)
	br[advance_queue_head_update_global_flags#]

check_three_mpkts_fit_last_buffer#:
	alu[$avail_tbuf_ele_1, --, B, 3]		; alloacte 3 tbuf elements
	_pkt_tx_16p_allocate_tbuf_from_scratch_memory($avail_tbuf_ele_1, \
			sig_mask_1, sig_scratch_access_2)  ; (1)
	_pkt_tx_16p_update_flow_control_on_transmit(3, in_port, in_port_bit) ; (12)

	// decide whether fitting into 3 mpacket and packet 3 size

	_pkt_tx_16p_decide_mpkt_3_size_last_buf(mpkt_3_size, mpkt_2_size, \
			buf_size, mpkt_1_size, out_offset, \
			more_than_3_mpkts_last_buffer#)					; (10)

	alu[out_mpkt, --,  B, 3]
	alu[exe_stat_flag, exe_stat_flag, OR, 1, <<ESF_FREE_BUF_BIT]
	alu[exe_stat_flag, exe_stat_flag, OR, 1, <<ESF_EOP_MPKT_THREAD_BIT]

	// update pkt txed for port and notify scheduler (11 (ixp2400 A0/A1) or 5
	// (others)) 

	_pkt_tx_16p_update_pkt_txed_info($txed_port, sig_mask_1, \
												sig_reflect_write, in_port)
advance_queue_head_update_global_flags#:

	_pkt_tx_16p_advance_queue_head()

	// decrement packets in queue, if necessary, also update @ep & @vap

	_pkt_tx_16p_decrement_pkt_in_queue(@ep, @vap, in_port_bit)
	
	br[update_global_flags#]

more_than_3_mpkts_last_buffer#: 
	alu[out_mpkt, --,  B, 4]
	
  	// local memory index 0 already point to the queue head entry, update 
	// remaining payload in this buffer and offset of the remaining payload

	_pkt_tx_16p_update_rmnd_paylo_offset(buf_size, mpkt_1_size, mpkt_2_size, \
						mpkt_3_size, out_offset)

	br[update_global_flags#]

	// the buffer being handled is not last buffer

not_last_buffer_in_this_packet#:

	br_bclr[exe_stat_flag, ESF_LEFTOVER_TO_TX_BIT, \
									mpkt1_size_no_leftov_not_last_buffer#]

	_pkt_tx_16p_decide_mpkt_1_size_with_leftov_not_last_buf(mpkt_1_size, \
		leftover_len, leftover_offset, leftover_to_tx_len, buf_size, \
		out_offset, out_offset_mod_8, check_two_mpkts_fit_not_last_buffer#)

		br[only_one_mpkt_not_last_buffer#]

mpkt1_size_no_leftov_not_last_buffer#:

	_pkt_tx_16p_decide_mpkt_1_size_no_leftov_not_last_buf(mpkt_1_size, \
			leftover_len, leftover_offset, buf_size, out_offset, \
			out_offset_mod_8, check_two_mpkts_fit_not_last_buffer#)

only_one_mpkt_not_last_buffer#:

	alu[out_mpkt, --,  B, 1]
	scratch[test_and_incr, $avail_tbuf_ele_1, tbuf_scr_addr, 0], \
			sig_done[sig_scratch_access_2]	    	; allocate one tbuf element
	_pkt_tx_16p_update_flow_control_on_transmit(1, in_port, in_port_bit) ; (12)

	br=byte[leftover_len, 0, 0, one_mpkt_read_leftover_done#]

	_pkt_tx_16p_read_leftover_from_dram(sig_mask_1, out_buf_handle, \
			leftover_len, leftover_offset, sig_dram_access_1)	
	alu_shf[exe_stat_flag, exe_stat_flag, or, 1, <<ESF_READ_LEFTOVER_BIT]
	alu[@blocked_by_read_leftover, @blocked_by_read_leftover, OR, in_port_bit]

one_mpkt_read_leftover_done#:

	// since this buffer is done, set flag to free this buffer in the begining 
	// of the next iteration, but packet not done, so change the next buffer 
	// to be current buffer for continue processing of this packet
	br[set_flag_to_free_this_buffer_and_change_next_buffer_to_current_buffer#]

check_two_mpkts_fit_not_last_buffer#:

	_pkt_tx_16p_decide_mpkt_2_size_not_last_buf(mpkt_2_size, leftover_len, \
		leftover_offset, buf_size, mpkt_1_size, out_offset, \
		check_three_mpkts_fit_not_last_buffer#)	

	alu[out_mpkt, --,  B, 2]
	alu[$avail_tbuf_ele_1, --, B, 2]		; alloacte 2 tbuf elements
	_pkt_tx_16p_allocate_tbuf_from_scratch_memory($avail_tbuf_ele_1, \
			sig_mask_1, sig_scratch_access_2)  ; (1)
	_pkt_tx_16p_update_flow_control_on_transmit(2, in_port, in_port_bit) ; (12)
		
	br=byte[leftover_len, 0, 0, two_mpkts_read_leftover_done#]

	_pkt_tx_16p_read_leftover_from_dram(sig_mask_1, out_buf_handle, \
			leftover_len, leftover_offset, sig_dram_access_1)	
	alu_shf[exe_stat_flag, exe_stat_flag, or, 1, <<ESF_READ_LEFTOVER_BIT]
	alu[@blocked_by_read_leftover, @blocked_by_read_leftover, OR, in_port_bit]

two_mpkts_read_leftover_done#:

	// since this buffer is done, set flag to free this buffer in the begining 
	// of the next iteration, but packet not done, so change the next buffer 
	// to be current buffer for continue processing of this packet
	br[set_flag_to_free_this_buffer_and_change_next_buffer_to_current_buffer#]

check_three_mpkts_fit_not_last_buffer#:

	alu[$avail_tbuf_ele_1, --, B, 3]		; alloacte 3 tbuf elements
	_pkt_tx_16p_allocate_tbuf_from_scratch_memory($avail_tbuf_ele_1, \
			sig_mask_1, sig_scratch_access_2)  ; (1)
	_pkt_tx_16p_update_flow_control_on_transmit(3, in_port, in_port_bit) ; (12)

	// decide whether fitting into 3 mpacket and packet 3 size

	_pkt_tx_16p_decide_mpkt_3_size_not_last_buf(mpkt_3_size, leftover_len, \
			leftover_offset, buf_size, mpkt_1_size, out_offset, mpkt_2_size, \
			more_than_3_mpkts_not_last_buffer#)

	alu[out_mpkt, --,  B, 3]
	br=byte[leftover_len, 0, 0, three_mpkts_read_leftover_done#]

	_pkt_tx_16p_read_leftover_from_dram(sig_mask_1, out_buf_handle, \
			leftover_len, leftover_offset, sig_dram_access_1)	
	alu_shf[exe_stat_flag, exe_stat_flag, or, 1, <<ESF_READ_LEFTOVER_BIT]
	alu[@blocked_by_read_leftover, @blocked_by_read_leftover, OR, in_port_bit]

three_mpkts_read_leftover_done#:

	// since this buffer is done, set flag to free this buffer in the begining 
	// of the next iteration, but packet not done, so change the next buffer 
	// to be current buffer for continue processing of this packet
set_flag_to_free_this_buffer_and_change_next_buffer_to_current_buffer#: 

	alu[exe_stat_flag, exe_stat_flag, OR, 1, <<ESF_FREE_BUF_BIT]
	_pkt_tx_16p_next_buffer_become_current_buffer()

	br[update_global_flags#]

more_than_3_mpkts_not_last_buffer#: 
	alu[out_mpkt, --,  B, 4]
	
  	// local memory index 0 already point to the queue head entry, update 
	// remaining payload in this buffer and offset of the remaining payload

	_pkt_tx_16p_update_rmnd_paylo_offset(buf_size, mpkt_1_size, mpkt_2_size, \
						mpkt_3_size, out_offset)


update_global_flags#:

	alu[@rp, @ep,  AND~, @bfcp]
	alu[@vpp, @vap, AND~ , @bfcp]
	alu[@vpp, @vpp, AND~ , @blocked_by_read_leftover]							
	alu[@vpp, @vpp, AND~ , @blocked_by_read_next_buf]			

.end  

#endm	// _pkt_tx_16p_process_pkt_in_lm_queue_phase_1()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_decide_mpkt_1_size_with_leftover_last_buf()
//
// Description: decide the payload size of mpkt 1 in the buffer which is
//				the last buffer of the packet with some leftover from 
//				previous buffer to transmit
//
// Outputs:							
// 		out_mpkt_1_size:		mpacket 1 size
//
// Inputs:
//      in_leftover_to_tx_len:  length of leftover from previous buffer
//  	in_buf_size:			buffer size left before mpacket 1
//		in_offset:				offset for mpacket 1 
//		in_offset_mod8:			offset mod by 8
//
// Constants:
//		None   
//
//	Labels:
//		MORE_THAN_ONE_MPKTS_LABEL:	symbolic address to go when more than
//									one mpacket required
//
// Size:
//		7 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_decide_mpkt_1_size_with_leftover_last_buf(out_mpkt_1_size, \
			in_leftover_to_tx_len, in_buf_size, in_offset, in_offset_mod8, \
			MORE_THAN_ONE_MPKTS_LABEL)

.begin

	.reg	tmp
	.reg	max_mop_lov					; maximum mop including leftover_to_tx

	alu[tmp, LEFTOVER_MPKT_MAX_LENGTH, -, in_offset_mod8]	; maximum eop mpkt 
	alu[max_mop_lov, tmp, +, in_leftover_to_tx_len]
	alu[max_mop_lov, max_mop_lov, AND, BUS_WIDTH_MASK]

	// following out_mpkt_1_size represent maximum allowable mop mpkt len

	alu[out_mpkt_1_size, max_mop_lov, -, in_leftover_to_tx_len] 

	alu[--, in_buf_size, -, tmp]
	bgt[MORE_THAN_ONE_MPKTS_LABEL]

	alu[out_mpkt_1_size, --, B, in_buf_size]

.end 

#endm	// _pkt_tx_16p_decide_mpkt_1_size_with_leftover_last_buf() 


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_decide_mpkt_1_size_no_leftov_last_buf()
//
// Description: decide the payload size of mpkt 1 in the buffer which is
//				the last buffer of the packet without any leftover from 
//				previous buffer to transmit
//
// Outputs:							
// 		out_mpkt_1_size:		mpacket 1 size
//
// Inputs:
//  	in_buf_size:			buffer size left before mpacket 1
//		in_offset:				offset for mpacket 1 
//		in_offset_mod8:			offset mod by 8
//
// Constants:
//		None   
//
//	Labels:
//		MORE_THAN_ONE_MPKTS_LABEL:	symbolic address to go when more than
//									one mpacket required
//
// Size:
//		5 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_decide_mpkt_1_size_no_leftov_last_buf(out_mpkt_1_size, \
			in_buf_size, in_offset, in_offset_mod8, MORE_THAN_ONE_MPKTS_LABEL)

.begin

	.reg	tmp


	alu[tmp, TBUF_ELE_SIZE, -, in_offset_mod8]	; maximum eop mpkt

	// following out_mpkt_1_size represent maximum allowable mop mpkt len

	alu[out_mpkt_1_size, tmp,  AND, BUS_WIDTH_MASK] 

	alu[--, in_buf_size, -, tmp]
	bgt[MORE_THAN_ONE_MPKTS_LABEL]

	alu[out_mpkt_1_size, --, B, in_buf_size]

.end 

#endm	// _pkt_tx_16p_decide_mpkt_1_size_no_leftov_last_buf() 


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_decide_mpkt_1_size_with_leftov_not_last_buf()
//
// Description: decide the payload size of mpkt 1 in the buffer which is
//				not the last buffer of the packet with some leftover from 
//				previous buffer to transmit
//
// Outputs:							
// 		out_mpkt1_len:			mpacket 1 size
//		out_leftover_len:		payload leftover to be transmitted before
//								next buffer due to bus width limitation
//		out_leftover_offset:	offset of the payload leftover
//
//
// Inputs:
//      in_leftover_to_tx_len:  length of leftover from previous buffer
//  	in_buf_size:			buffer size left before mpacket 1
//		in_offset:				offset for mpacket 1 
//		in_offset_mod8:			offset mod by 8
//
// Constants:
//		None   
//
//	Labels:
//		MORE_THAN_ONE_MPKTS_LABEL:	symbolic address to go when more than
//									one mpacket required
//
// Size:
//		12 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_decide_mpkt_1_size_with_leftov_not_last_buf(out_mpkt1_len, \
			out_leftover_len, out_leftover_offset, in_leftover_to_tx_len, \
			in_buf_size, in_offset, in_offset_mod8, MORE_THAN_ONE_MPKTS_LABEL)

.begin

	.reg	tmp

	// following local_leftover_to_tx_len is used to resolve odd-sided
	// gpr allocation conflict

	.reg 	local_leftover_to_tx_len

	alu[local_leftover_to_tx_len, --, B, in_leftover_to_tx_len]
	alu[tmp, LEFTOVER_MPKT_MAX_LENGTH, -, in_offset_mod8]
	alu[tmp, tmp, +, local_leftover_to_tx_len]
	alu[tmp, tmp, AND, BUS_WIDTH_MASK]

	// following out_mpkt1_len is maximum allowable mob mpkt 1 size 

	alu[out_mpkt1_len, tmp, -, local_leftover_to_tx_len] 
	alu[tmp, out_mpkt1_len, +, BUS_WIDTH]
	
	alu[--, in_buf_size, -, tmp]
	bge[MORE_THAN_ONE_MPKTS_LABEL]

	alu[tmp, in_buf_size, +, local_leftover_to_tx_len]
	alu[tmp, tmp, AND, BUS_WIDTH_MASK]
	alu[out_mpkt1_len, tmp, -, local_leftover_to_tx_len]
		 
	alu[out_leftover_len, in_buf_size, -, out_mpkt1_len]
	alu[out_leftover_offset, in_offset, +,  out_mpkt1_len]

.end 

#endm	// _pkt_tx_16p_decide_mpkt_1_size_with_leftov_not_last_buf() 


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_decide_mpkt_1_size_no_leftov_not_last_buf()
//
// Description: decide the payload size of mpkt 1 in the buffer which is
//				not the last buffer of the packet without some leftover from 
//				previous buffer to transmit
//
// Outputs:							
// 		out_mpkt1_len:			mpacket 1 size
//		out_leftover_len:		payload leftover to be transmitted before
//								next buffer due to bus width limitation
//		out_leftover_offset:	offset of the payload leftover
//
//
// Inputs:
//  	in_buf_size:			buffer size left before mpacket 1
//		in_offset:				offset for mpacket 1 
//		in_offset_mod8:			offset mod by 8
//
// Constants:
//		None   
//
//	Labels:
//		MORE_THAN_ONE_MPKTS_LABEL:	symbolic address to go when more than
//									one mpacket required
//
// Size:
//		8 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_decide_mpkt_1_size_no_leftov_not_last_buf(out_mpkt1_len, \
			out_leftover_len, out_leftover_offset, in_buf_size, in_offset, \
			in_offset_mod8, MORE_THAN_ONE_MPKTS_LABEL)

.begin

	.reg	tmp

	
	alu[out_mpkt1_len, TBUF_ELE_SIZE, -, in_offset_mod8]

	// following out_mpkt1_len is maximum allowable mob mpkt 1 size 

	alu[out_mpkt1_len, out_mpkt1_len, AND, BUS_WIDTH_MASK] 

	// following tmp is maximum allowable eob mpkt 1 size with
	// maximum (BUS_WIDTH -1) in leftover

	alu[tmp, out_mpkt1_len, +, BUS_WIDTH]
	
	alu[--, in_buf_size, -, tmp]
	bge[MORE_THAN_ONE_MPKTS_LABEL]

	alu[out_mpkt1_len, in_buf_size, AND, BUS_WIDTH_MASK]

	alu[out_leftover_len, in_buf_size, -, out_mpkt1_len]
	alu[out_leftover_offset, in_offset, +,  out_mpkt1_len]

.end 

#endm	// _pkt_tx_16p_decide_mpkt_1_size_no_leftov_not_last_buf() 


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_get_leftover_to_tx_len()
//
// Description: get length of leftover from previous buffer to transmit before
//				payload of this buffer
//
// Prerequisite:				local memory index 1 alrady set to queue status 
//								of the in_port
//
// Outputs:							
// 		out_leftover_to_tx_len:	len of leftover from previous buffer to 
//								be transmitted before payload of this buffer
// Inputs:
//		None   
//
// Constants:
//		None   
//
// Size:
//		2 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_get_leftover_to_tx_len(out_leftover_to_tx_len)

.begin

	.reg	tmp

	alu[tmp, --, B, lm1_lov_stat_next_next_buf]
	alu[out_leftover_to_tx_len, LM_LEFTOVER_LEN_MASK, AND, tmp, \
												>>LM_LEFTOVER_LEN_BIT]

.end 

#endm	// _pkt_tx_16p_get_leftover_to_tx_len() 


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_process_critical_tx_request_phase_2()
//
// Description: process sop-mop mpacket directly from tx request in phase 2, 
//              branch out to optimize performance in critical case
//
// Outputs:							
// 		None
//
// Inputs/Outputs:
// 		io_sig_mask:		signal mask
//
// Inputs:
// 		in_mpkt:			mpacket to transmitted in this iteration
//  	in_offset:			packet offset
//		in_offset_mod_8:	packet offset mod by 8
//		in_port:			port to transmit
//		in_port_bit:		bit corresponding to port = 1 << in_port
//		in_buf_handle:		buffer handle
//
// Constants:	
//		None  
//
//	Labels:
//		NOT_SOP_EOP_LABEL:	symbolic address to go when it is not sop-eop 
//							mpacket
//
// Size:
//		24 instructions 
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_process_critical_tx_request_phase_2(io_sig_mask, in_mpkt, \
			in_offset, in_offset_mod_8, in_port, in_port_bit, in_buf_handle, \
			NOT_SOP_EOP_LABEL) 							;

.begin

	.reg	dram_addr
	.reg	tmp

	br!=byte[in_mpkt, 0, 1, NOT_SOP_EOP_LABEL]
	alu[tbuf_ele_1, $avail_tbuf_ele_1, AND, TBUF_ELE_BITS_MASK]	

	_pkt_tx_16p_get_dram_addr(dram_addr, in_buf_handle)		; (2)

	// check tbuf full, 8 instructions in not full case, otherwise indefinite

	_pkt_tx_16p_check_wait_tbuf_not_full($avail_tbuf_ele_1, sig_msf_access_1)

	_pkt_tx_16p_load_sop_eop_mpkt(tcw0_1, io_sig_mask, sig_dram_access_1, \
			dram_addr,  tbuf_ele_1, in_offset, mpkt_1_size, \
			in_offset_mod_8, in_port) 						; (12)

.end

#endm	// _pkt_tx_16p_process_critical_tx_request_phase_2()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_process_non_critical_tx_request_phase_2()
//
// Description: process not sop-eop mpacket directly from tx request in phase 2
//
// Outputs:							
// 		None
//
// Inputs/Outputs:
// 		io_sig_mask:		signal mask
//
// Inputs:
// 		in_mpkt:			mpacket to transmitted in this iteration
//  	in_offset:			packet offset
//		in_offset_mod_8:	packet offset mod by 8
//		in_port:			port to transmit
//		in_port_bit:		bit corresponding to port = 1 << in_port
//		in_buf_handle:		buffer handle
//
// Constants:	
//		None   
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_process_non_critical_tx_request_phase_2(io_sig_mask, \
				in_mpkt, in_offset, in_offset_mod_8, in_port, in_port_bit, \
				in_buf_handle) 							;

.begin

	.reg	dram_addr
	.reg	tbuf_ele_2
	.reg	tbuf_ele_3
	.reg	tmp

	alu[tbuf_ele_1, $avail_tbuf_ele_1, AND, TBUF_ELE_BITS_MASK]	

	br!=byte[in_mpkt, 0, 0, two_or_more_mpackets#]

	// just for tbuf full check


	// no mapcket to transmit, clear signals

	_pkt_tx_16p_clear_signal(io_sig_mask, sig_dram_access_1)	
	_pkt_tx_16p_clear_push_signal(io_sig_mask, sig_dram_access_1)	

	br[done#]

two_or_more_mpackets#:

	// process result of next buffer meta read and leftover read, this doesn't
	// apply to to sop_eop mpacket, and mapcket with ESF_FREE_BUF_BIT set
	
	// for safe, set local memory index 0 to the queue head of the port again,
	// since it may be changed for seeking packet to transmit from @vpp,
	// instruction budget should not a problem in this case

	_pkt_tx_16p_set_lm_index_0_to_queue_head(in_port)		; (6)

	br_bset[exe_stat_flag, ESF_EOP_MPKT_THREAD_BIT, nextbuf_leftvoer_done#]

	br_bclr[exe_stat_flag, ESF_READ_NEXT_BUF_META_BIT, \
													nextbuf_meta_save_done#]

	_pkt_tx_16p_save_next_buf_meta_data()	; save meta data  to lm

	// clear port from @blocked_by_read_next_buf

	alu[@blocked_by_read_next_buf, @blocked_by_read_next_buf, AND~, \
																in_port_bit]

nextbuf_meta_save_done#:

	br_bclr[exe_stat_flag, ESF_READ_LEFTOVER_BIT, update_global_flags#]

	_pkt_tx_16p_process_and_save_leftover(leftover_len, leftover_offset)

	// remove port from @blocked_by_read_leftover

	alu[@blocked_by_read_leftover, @blocked_by_read_leftover, AND~, \
																in_port_bit]
update_global_flags#:
	alu[@vpp, @vap, AND~ , @bfcp]												
	alu[@vpp, @vpp, AND~ , @blocked_by_read_leftover]							
	alu[@vpp, @vpp, AND~ , @blocked_by_read_next_buf]			


nextbuf_leftvoer_done#:

	br=byte[in_mpkt, 0, 2, two_mpackets#]
	br=byte[in_mpkt, 0, 3, three_mpackets#]
	br=byte[in_mpkt, 0, 4, buf_not_finished#]

two_mpackets#:

	alu[tbuf_ele_2, tbuf_ele_1, +, 1]
	alu[tbuf_ele_2, tbuf_ele_2, AND, TBUF_ELE_BITS_MASK]

	_pkt_tx_16p_get_dram_addr(dram_addr, in_buf_handle)		; (2)

	_pkt_tx_16p_check_wait_tbuf_not_full($avail_tbuf_ele_1, sig_msf_access_1)

	_pkt_tx_16p_load_sop_mpkt(tcw0_1, io_sig_mask, sig_dram_access_1, \
			dram_addr, tbuf_ele_1, in_offset, mpkt_1_size, in_offset_mod_8, \
			in_port) 

	alu[in_offset, in_offset, +, mpkt_1_size] 	; offset_2
	alu[in_offset_mod_8, in_offset, AND, CONST_07]

	_pkt_tx_16p_set_signal(io_sig_mask, sig_dram_access_2)	
	_pkt_tx_16p_set_push_signal[io_sig_mask, sig_dram_access_2] ; add push signal

	br_bclr[exe_stat_flag, ESF_EOP_MPKT_THREAD_BIT, mpkt_2_not_eop_mpkt#]

	_pkt_tx_16p_load_eop_mpkt_no_leftover(tcw0_2, io_sig_mask, \
			sig_dram_access_2, dram_addr, tbuf_ele_2, in_offset, mpkt_2_size, \
			in_offset_mod_8, in_port)

	br[done#]

mpkt_2_not_eop_mpkt#:

	_pkt_tx_16p_load_mop_mpkt_no_leftover(tcw0_2, io_sig_mask, \
			sig_dram_access_2, dram_addr, tbuf_ele_2, in_offset, mpkt_2_size, \
			in_offset_mod_8, in_port)

	br[done#]

three_mpackets#:

	alu[tbuf_ele_2, tbuf_ele_1, +, 1]
	alu[tbuf_ele_2, tbuf_ele_2, AND, TBUF_ELE_BITS_MASK]
	alu[tbuf_ele_3, tbuf_ele_1, +, 2]
	alu[tbuf_ele_3, tbuf_ele_3, AND, TBUF_ELE_BITS_MASK]

	_pkt_tx_16p_get_dram_addr(dram_addr, in_buf_handle)		; (2)

	_pkt_tx_16p_check_wait_tbuf_not_full($avail_tbuf_ele_1, sig_msf_access_1)

	_pkt_tx_16p_load_sop_mpkt(tcw0_1, io_sig_mask, sig_dram_access_1, \
			dram_addr,  tbuf_ele_1, in_offset, mpkt_1_size, \
			in_offset_mod_8, in_port) 

	alu[in_offset, in_offset, +, mpkt_1_size] 	; offset_2
	alu[in_offset_mod_8, in_offset, AND, CONST_07]
	_pkt_tx_16p_load_mop_mpkt_no_leftover(tcw0_2, io_sig_mask, \
			sig_dram_access_2, dram_addr, tbuf_ele_2, in_offset, mpkt_2_size, \
			in_offset_mod_8, in_port)
	_pkt_tx_16p_set_signal(io_sig_mask, sig_dram_access_2)	
	_pkt_tx_16p_set_push_signal[io_sig_mask, sig_dram_access_2]	; push signal

	// following temporary storage of mpkt_2_size to tmp is used to resolve
	// odd-sided GPR allocation conflict

	alu[tmp, --, B, mpkt_2_size]
	alu[in_offset, in_offset, +, tmp] 			; offset_3
	alu[in_offset_mod_8, in_offset, AND, CONST_07]

	_pkt_tx_16p_set_signal(io_sig_mask, sig_dram_access_3)	
	_pkt_tx_16p_set_push_signal[io_sig_mask, sig_dram_access_3]	; push signal

	br_bclr[exe_stat_flag, ESF_EOP_MPKT_THREAD_BIT, mpkt_3_not_eop_mpkt#]

	_pkt_tx_16p_load_eop_mpkt_no_leftover(tcw0_3, io_sig_mask, \
			sig_dram_access_3, dram_addr, tbuf_ele_3, in_offset, mpkt_3_size, \
			in_offset_mod_8, in_port)

	br[done#]

mpkt_3_not_eop_mpkt#:

	_pkt_tx_16p_load_mop_mpkt_no_leftover(tcw0_3, io_sig_mask, \
			sig_dram_access_3, dram_addr, tbuf_ele_3, in_offset, mpkt_3_size, \
			in_offset_mod_8, in_port)

	br[done#]

buf_not_finished#:	; buffer can't be transmitted completely in this iteration

	alu[tbuf_ele_2, tbuf_ele_1, +, 1]
	alu[tbuf_ele_2, tbuf_ele_2, AND, TBUF_ELE_BITS_MASK]
	alu[tbuf_ele_3, tbuf_ele_1, +, 2]
	alu[tbuf_ele_3, tbuf_ele_3, AND, TBUF_ELE_BITS_MASK]

	_pkt_tx_16p_get_dram_addr(dram_addr, in_buf_handle)		; (2)

	_pkt_tx_16p_check_wait_tbuf_not_full($avail_tbuf_ele_1, sig_msf_access_1)

	_pkt_tx_16p_load_sop_mpkt(tcw0_1, io_sig_mask, sig_dram_access_1, \
			dram_addr,  tbuf_ele_1, in_offset, mpkt_1_size, \
			in_offset_mod_8, in_port) 

	alu[in_offset, in_offset, +, mpkt_1_size] 	; offset_2
	alu[in_offset_mod_8, in_offset, AND, CONST_07]
	_pkt_tx_16p_load_mop_mpkt_no_leftover(tcw0_2, io_sig_mask, \
			sig_dram_access_2, dram_addr, tbuf_ele_2, in_offset, mpkt_2_size, \
			in_offset_mod_8, in_port)
	_pkt_tx_16p_set_signal(io_sig_mask, sig_dram_access_2)	
	_pkt_tx_16p_set_push_signal[io_sig_mask, sig_dram_access_2]	; push signal

	// following temporary storage of mpkt_2_size to tmp is used to resolve
	// odd-sided GPR allocation conflict

	alu[tmp, --, B, mpkt_2_size]
	alu[in_offset, in_offset, +, tmp] 			; offset_3
	alu[in_offset_mod_8, in_offset, AND, CONST_07]
	_pkt_tx_16p_load_mop_mpkt_no_leftover(tcw0_3, io_sig_mask, \
			sig_dram_access_3, dram_addr, tbuf_ele_3, in_offset, mpkt_3_size, \
			in_offset_mod_8, in_port)
	_pkt_tx_16p_set_signal(io_sig_mask, sig_dram_access_3)	
	_pkt_tx_16p_set_push_signal[io_sig_mask, sig_dram_access_3]	; push signal

done#:

.end

#endm	// _pkt_tx_16p_process_non_critical_tx_request_phase_2()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_process_pkt_in_lm_queue_phase_2()
//
// Description: process packet in local memory queue in phase 2
//
// Outputs:							
// 		None
//
// Inputs/Outputs:
// 		io_sig_mask:		signal mask
//
// Inputs:
// 		in_mpkt:			mpacket to transmitted in this iteration
//  	in_offset:			packet offset
//		in_offset_mod_8:	packet offset mod by 8
//		in_port:			port to transmit
//		in_port_bit:		bit corresponding to port = 1 << in_port
//		in_buf_handle:		buffer handle
//
// Constants:	
//		None   
//
// Size:
//		29/32 instructions for one mpacket for ixp2400 A0&A1 / others
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_process_pkt_in_lm_queue_phase_2(io_sig_mask, in_mpkt, \
			in_offset, in_offset_mod_8, in_port, in_port_bit, in_buf_handle) 							;



.begin

	.reg	dram_addr
	.reg	tbuf_ele_2
	.reg	tbuf_ele_3
	.reg	tmp
	.reg 	leftov_to_tx_len		; size of leftover to be transmitted
									; before payload of this buffer	

	br=byte[in_mpkt, 0, 0, no_mpacket_to_tx#]

	alu[tbuf_ele_1, $avail_tbuf_ele_1, AND, TBUF_ELE_BITS_MASK]	

	// for safe, set local memory index 0 to the queue head of the port again,
	// since it may be changed for seeking packet to transmit from @vpp,
	// instruction budget should not a problem in this case

	_pkt_tx_16p_set_lm_index_0_to_queue_head(in_port)		; (6)

	// process result of next buffer meta read 
		
	br_bclr[exe_stat_flag, ESF_READ_NEXT_BUF_META_BIT, \
													nextbuf_meta_save_done#]

	_pkt_tx_16p_save_next_buf_meta_data()	; save meta data  to lm

	// clear port from @blocked_by_read_next_buf

	alu[@blocked_by_read_next_buf, @blocked_by_read_next_buf, AND~, \
																in_port_bit]

nextbuf_meta_save_done#:

	br=byte[in_mpkt, 0, 1, one_mpacket#]
	br=byte[in_mpkt, 0, 3, three_mpackets#]
	br=byte[in_mpkt, 0, 4, buf_not_finished#]

two_mpackets#:

	alu[tbuf_ele_2, tbuf_ele_1, +, 1]
	alu[tbuf_ele_2, tbuf_ele_2, AND, TBUF_ELE_BITS_MASK]

	_pkt_tx_16p_get_dram_addr(dram_addr, in_buf_handle)		; (2)

	_pkt_tx_16p_check_wait_tbuf_not_full($avail_tbuf_ele_1, sig_msf_access_1)

	br_bset[exe_stat_flag, ESF_SOP_MPKT_THREAD_BIT, sop_mpkt_in_2_mpkts#]
	
	br_bclr[exe_stat_flag, ESF_LEFTOVER_TO_TX_BIT, two_mpkts_no_leftov_to_tx#]
	
	_pkt_tx_16p_get_leftover_to_tx_len(leftov_to_tx_len)

	// mop mpkt with leftover to transmit before payload of this buffer, 

	_pkt_tx_16p_load_mop_mpkt_with_leftover(tcw0_1, io_sig_mask, \
			sig_dram_access_1, dram_addr, tbuf_ele_1, in_offset, \
			mpkt_1_size, in_offset_mod_8, in_port, leftov_to_tx_len, \
			sig_msf_access_2)
	br[continue_two_mpackets#]

two_mpkts_no_leftov_to_tx#:

	// mop mpkt without leftover to transmit before payload of this buffer

	_pkt_tx_16p_load_mop_mpkt_no_leftover(tcw0_1, io_sig_mask, \
			sig_dram_access_1, dram_addr, tbuf_ele_1, in_offset, \
			mpkt_1_size, in_offset_mod_8, in_port)
	br[continue_two_mpackets#]

sop_mpkt_in_2_mpkts#:

	// sop only mpacket

	_pkt_tx_16p_load_sop_mpkt(tcw0_1, io_sig_mask, sig_dram_access_1, \
			dram_addr, tbuf_ele_1, in_offset, mpkt_1_size, \
			in_offset_mod_8, in_port) 

continue_two_mpackets#:
	alu[in_offset, in_offset, +, mpkt_1_size] 	; offset_2
	alu[in_offset_mod_8, in_offset, AND, CONST_07]

	_pkt_tx_16p_set_signal(io_sig_mask, sig_dram_access_2)	
	_pkt_tx_16p_set_push_signal[io_sig_mask, sig_dram_access_2]	; push signal

	br_bclr[exe_stat_flag, ESF_EOP_MPKT_THREAD_BIT, mpkt_2_not_eop_mpkt#]

	_pkt_tx_16p_load_eop_mpkt_no_leftover(tcw0_2, io_sig_mask, \
		sig_dram_access_2, dram_addr, tbuf_ele_2, in_offset, mpkt_2_size, \
		in_offset_mod_8, in_port)

	br[check_to_process_and_save_letover_read_in_phase_1#]

mpkt_2_not_eop_mpkt#:

	_pkt_tx_16p_load_mop_mpkt_no_leftover(tcw0_2, io_sig_mask, \
		sig_dram_access_2, dram_addr, tbuf_ele_2, in_offset, mpkt_2_size, \
		in_offset_mod_8, in_port)

	br[check_to_process_and_save_letover_read_in_phase_1#]

three_mpackets#:

	alu[tbuf_ele_2, tbuf_ele_1, +, 1]
	alu[tbuf_ele_2, tbuf_ele_2, AND, TBUF_ELE_BITS_MASK]
	alu[tbuf_ele_3, tbuf_ele_1, +, 2]
	alu[tbuf_ele_3, tbuf_ele_3, AND, TBUF_ELE_BITS_MASK]

	_pkt_tx_16p_get_dram_addr(dram_addr, in_buf_handle)		; (2)

	_pkt_tx_16p_check_wait_tbuf_not_full($avail_tbuf_ele_1, sig_msf_access_1)

	br_bset[exe_stat_flag, ESF_SOP_MPKT_THREAD_BIT, sop_mpkt_in_3_mpkts#]
	
	br_bclr[exe_stat_flag, ESF_LEFTOVER_TO_TX_BIT, \
												three_mpkts_no_leftov_to_tx#]
	
	_pkt_tx_16p_get_leftover_to_tx_len(leftov_to_tx_len)

	// mop mpkt with leftover to transmit before payload of this buffer, 

	_pkt_tx_16p_load_mop_mpkt_with_leftover(tcw0_1, io_sig_mask, \
			sig_dram_access_1, dram_addr, tbuf_ele_1, in_offset, \
			mpkt_1_size, in_offset_mod_8, in_port, leftov_to_tx_len, \
			sig_msf_access_2)
	br[continue_three_mpackets#]

three_mpkts_no_leftov_to_tx#:

	// mop mpkt without leftover to transmit before payload of this buffer

	_pkt_tx_16p_load_mop_mpkt_no_leftover(tcw0_1, io_sig_mask, \
			sig_dram_access_1, dram_addr, tbuf_ele_1, in_offset, \
			mpkt_1_size, in_offset_mod_8, in_port)
	br[continue_three_mpackets#]

sop_mpkt_in_3_mpkts#:

	// sop only mpacket

	_pkt_tx_16p_load_sop_mpkt(tcw0_1, io_sig_mask, sig_dram_access_1, \
			dram_addr, tbuf_ele_1, in_offset, mpkt_1_size, \
			in_offset_mod_8, in_port) 

continue_three_mpackets#:
	alu[in_offset, in_offset, +, mpkt_1_size] 	; offset_2
	alu[in_offset_mod_8, in_offset, AND, CONST_07]
	_pkt_tx_16p_load_mop_mpkt_no_leftover(tcw0_2, io_sig_mask, \
		sig_dram_access_2, dram_addr, tbuf_ele_2, in_offset, mpkt_2_size, \
		in_offset_mod_8, in_port)
	_pkt_tx_16p_set_signal(io_sig_mask, sig_dram_access_2)	
	_pkt_tx_16p_set_push_signal[io_sig_mask, sig_dram_access_2]	; push signal

	// following temporary storage of mpkt_2_size to tmp is used to resolve
	// odd-sided GPR allocation conflict

	alu[tmp, --, B, mpkt_2_size]
	alu[in_offset, in_offset, +, tmp] 			; offset_3
	alu[in_offset_mod_8, in_offset, AND, CONST_07]

	_pkt_tx_16p_set_signal(io_sig_mask, sig_dram_access_3)	
	_pkt_tx_16p_set_push_signal[io_sig_mask, sig_dram_access_3]	; push signal

	br_bclr[exe_stat_flag, ESF_EOP_MPKT_THREAD_BIT, mpkt_3_not_eop_mpkt#]

	_pkt_tx_16p_load_eop_mpkt_no_leftover(tcw0_3, io_sig_mask, \
		sig_dram_access_3, dram_addr, tbuf_ele_3, in_offset, mpkt_3_size, \
			in_offset_mod_8, in_port)

	br[check_to_process_and_save_letover_read_in_phase_1#]

mpkt_3_not_eop_mpkt#:

	_pkt_tx_16p_load_mop_mpkt_no_leftover(tcw0_3, io_sig_mask, \
			sig_dram_access_3, 	dram_addr, tbuf_ele_3, in_offset, \
			mpkt_3_size, in_offset_mod_8, in_port)

	br[check_to_process_and_save_letover_read_in_phase_1#]

buf_not_finished#:	; buffer can't be transmitted completely in this iteration

	alu[tbuf_ele_2, tbuf_ele_1, +, 1]
	alu[tbuf_ele_2, tbuf_ele_2, AND, TBUF_ELE_BITS_MASK]
	alu[tbuf_ele_3, tbuf_ele_1, +, 2]
	alu[tbuf_ele_3, tbuf_ele_3, AND, TBUF_ELE_BITS_MASK]

	_pkt_tx_16p_get_dram_addr(dram_addr, in_buf_handle)		; (2)

	_pkt_tx_16p_check_wait_tbuf_not_full($avail_tbuf_ele_1, sig_msf_access_1)


	br_bset[exe_stat_flag, ESF_SOP_MPKT_THREAD_BIT, sop_mpkt_buf_not_finished#]
	
	br_bclr[exe_stat_flag, ESF_LEFTOVER_TO_TX_BIT, \
											buf_not_finished_no_leftov_to_tx#]
	
	_pkt_tx_16p_get_leftover_to_tx_len(leftov_to_tx_len)

	// mop mpkt with leftover to transmit before payload of this buffer, 

	_pkt_tx_16p_load_mop_mpkt_with_leftover(tcw0_1, io_sig_mask, \
			sig_dram_access_1, dram_addr, tbuf_ele_1, in_offset, \
			mpkt_1_size, in_offset_mod_8, in_port, leftov_to_tx_len, \
			sig_msf_access_2)
	br[continue_buf_not_finished#]

buf_not_finished_no_leftov_to_tx#:

	// mop mpkt without leftover to transmit before payload of this buffer

	_pkt_tx_16p_load_mop_mpkt_no_leftover(tcw0_1, io_sig_mask, \
			sig_dram_access_1, dram_addr, tbuf_ele_1, in_offset, \
			mpkt_1_size, in_offset_mod_8, in_port)
	br[continue_buf_not_finished#]

sop_mpkt_buf_not_finished#:

	// sop only mpacket

	_pkt_tx_16p_load_sop_mpkt(tcw0_1, io_sig_mask, sig_dram_access_1, \
			dram_addr, tbuf_ele_1, in_offset, mpkt_1_size, \
			in_offset_mod_8, in_port) 

continue_buf_not_finished#:
	alu[in_offset, in_offset, +, mpkt_1_size] 	; offset_2
	alu[in_offset_mod_8, in_offset, AND, CONST_07]
	_pkt_tx_16p_load_mop_mpkt_no_leftover(tcw0_2, io_sig_mask, \
		sig_dram_access_2, dram_addr, tbuf_ele_2, in_offset, mpkt_2_size, \
		in_offset_mod_8, in_port)
	_pkt_tx_16p_set_signal(io_sig_mask, sig_dram_access_2)	
	_pkt_tx_16p_set_push_signal[io_sig_mask, sig_dram_access_2]	; push signal

	// following temporary storage of mpkt_2_size to tmp is used to resolve
	// odd-sided GPR allocation conflict

	alu[tmp, --, B, mpkt_2_size]
	alu[in_offset, in_offset, +, tmp] 			; offset_3
	alu[in_offset_mod_8, in_offset, AND, CONST_07]
	_pkt_tx_16p_load_mop_mpkt_no_leftover(tcw0_3, io_sig_mask, \
		sig_dram_access_3, dram_addr, tbuf_ele_3, in_offset, mpkt_3_size, \
		in_offset_mod_8, in_port)
	_pkt_tx_16p_set_signal(io_sig_mask, sig_dram_access_3)	
	_pkt_tx_16p_set_push_signal[io_sig_mask, sig_dram_access_3]	; push signal

check_to_process_and_save_letover_read_in_phase_1#:
	
	// process result of leftover read

	br_bclr[exe_stat_flag, ESF_READ_LEFTOVER_BIT, update_global_flags#]

	_pkt_tx_16p_process_and_save_leftover(leftover_len, leftover_offset) 		

	// remove port from @blocked_by_read_leftover

	alu[@blocked_by_read_leftover, @blocked_by_read_leftover, AND~, \
																in_port_bit]

update_global_flags#:
	alu[@vpp, @vap, AND~ , @bfcp]											
	alu[@vpp, @vpp, AND~ , @blocked_by_read_leftover]							
	alu[@vpp, @vpp, AND~ , @blocked_by_read_next_buf]			

	br[process_pkt_in_lm_queue_phase_2_done#]

no_mpacket_to_tx#:

	// no mapcket to transmit, clear signals

	_pkt_tx_16p_clear_signal(io_sig_mask, sig_dram_access_1)	
	_pkt_tx_16p_clear_push_signal(io_sig_mask, sig_dram_access_1)	

	br[process_pkt_in_lm_queue_phase_2_done#]

one_mpacket#:

	alu[tbuf_ele_1, $avail_tbuf_ele_1, AND, TBUF_ELE_BITS_MASK]
	_pkt_tx_16p_get_dram_addr(dram_addr, in_buf_handle)		; (2)

	_pkt_tx_16p_check_wait_tbuf_not_full($avail_tbuf_ele_1, sig_msf_access_1)

	br_bset[exe_stat_flag, ESF_SOP_MPKT_THREAD_BIT, sop_eop_mpkt#]
	
	br_bclr[exe_stat_flag, ESF_LEFTOVER_TO_TX_BIT, one_mpkt_no_leftov_to_tx#]
	
	_pkt_tx_16p_get_leftover_to_tx_len(leftov_to_tx_len)

	br_bclr[exe_stat_flag, ESF_EOP_MPKT_THREAD_BIT, one_mpkt_mop_with_leftov#]

	// eop mpkt with leftover to transmit before payload of this buffer, 

	_pkt_tx_16p_load_eop_mpkt_with_leftover(tcw0_1, io_sig_mask, \
			sig_dram_access_1, dram_addr, tbuf_ele_1, in_offset, \
			mpkt_1_size, in_offset_mod_8, in_port, leftov_to_tx_len, \
			sig_msf_access_2)
	br[check_to_process_and_save_letover_read_in_phase_1#]

one_mpkt_mop_with_leftov#:

	// mop mpkt with leftover to transmit before payload of this buffer, 

	_pkt_tx_16p_load_mop_mpkt_with_leftover(tcw0_1, io_sig_mask, \
			sig_dram_access_1, dram_addr, tbuf_ele_1, in_offset, \
			mpkt_1_size, in_offset_mod_8, in_port, leftov_to_tx_len, \
			sig_msf_access_2)
	br[check_to_process_and_save_letover_read_in_phase_1#]

one_mpkt_no_leftov_to_tx#:

	br_bclr[exe_stat_flag, ESF_EOP_MPKT_THREAD_BIT, one_mpkt_mop_no_leftov#]

	// eop mpkt without leftover to transmit before payload of this buffer

	_pkt_tx_16p_load_eop_mpkt_no_leftover(tcw0_1, io_sig_mask, \
			sig_dram_access_1, dram_addr, tbuf_ele_1, in_offset, \
			mpkt_1_size, in_offset_mod_8, in_port)
	br[check_to_process_and_save_letover_read_in_phase_1#]

one_mpkt_mop_no_leftov#:

	// mop mpkt without leftover to transmit before payload of this buffer

	_pkt_tx_16p_load_mop_mpkt_no_leftover(tcw0_1, io_sig_mask, \
			sig_dram_access_1, dram_addr, tbuf_ele_1, in_offset, \
			mpkt_1_size, in_offset_mod_8, in_port)
	br[check_to_process_and_save_letover_read_in_phase_1#]

sop_eop_mpkt#:

	// this is sop_eop mpacket, so there is no next buffer meta data and 
	// leftover read in phase 1, so after finishing following routine,
	// it can exit directly to save instruction cycles

	_pkt_tx_16p_load_sop_eop_mpkt(tcw0_1, io_sig_mask, \
			sig_dram_access_1, dram_addr, tbuf_ele_1, in_offset, \
			mpkt_1_size, in_offset_mod_8, in_port) 

process_pkt_in_lm_queue_phase_2_done#:

.end

#endm	// _pkt_tx_16p_process_pkt_in_lm_queue_phase_2()


/////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_decide_sop_mpkt_size_last_buf() 
//
// Description: decide sop mpacket size for SOP buffer which is EOP buffer 
//				too
//
// Outputs:							
// 		out_mpkt_1_size:		payload size for mpkt_1
//
// Inputs:  
//		in_buf_size:		buffer size
//		in_offsetmod8:		payload offset mod by 8 
//
// Constants:
//		None
//
// Size:
//		12 instructions (no L2_HEADER)
//
///////////////////////////////////////////////////////////////////////////////
 
#macro _pkt_tx_16p_decide_sop_mpkt_size_last_buf(out_mpkt_1_size, \
													in_buf_size, in_offsetmod8)
 
// basic assumption for thsi macro is that in_buf_size is greater than 
// TBUF_ELE_SIZE, this is a valid assumption, since here is beginning of a 
// buffer

.begin

	.reg	tmp

	#ifndef ADD_L2_HEADER

	alu[out_mpkt_1_size, TBUF_ELE_SIZE, -, in_offsetmod8]
	alu[out_mpkt_1_size, out_mpkt_1_size, AND, BUS_WIDTH_MASK]

	#else	// ADD_L2_HEADER
	
	// is L2 haeder already exist?

	move[tmp, L2_HEADER_ALREADY_EXIST_ID]
    alu[--, nexthop_id, -, tmp]
	beq[l2_header_already_exist#]

	alu[out_mpkt_1_size, SINGLE_MPKT_MAX_SIZE, -, in_offsetmod8]
	alu[out_mpkt_1_size, out_mpkt_1_size, +, L2_HEADER_SIZE]
	alu[out_mpkt_1_size, out_mpkt_1_size, AND, BUS_WIDTH_MASK]
	alu[out_mpkt_1_size, out_mpkt_1_size, -, L2_HEADER_SIZE]

	br[done#]

l2_header_already_exist#:

	alu[out_mpkt_1_size, TBUF_ELE_SIZE, -, in_offsetmod8]
	alu[out_mpkt_1_size, out_mpkt_1_size, AND, BUS_WIDTH_MASK]
	alu[--, in_buf_size, -, out_mpkt_1_size]
	bgt[done#]

	// make sure something in mpkt_2

	alu[out_mpkt_1_size, out_mpkt_1_size, -, BUS_WIDTH]	 

done#:

#endif	// #ifndef ADD_L2_HEADER

.end 

#endm	// _pkt_tx_16p_decide_sop_mpkt_size_last_buf()



///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_decide_sop_mpkt_size_not_last_buf()
//
// Description: decide the payload size of sop mpacket in the buffer which is
//				not the last buffer of the packet
//
// Outputs:							
// 		out_mpkt_1_size:		mpacket 1 size
//		out_leftover_len:		payload leftover to be transmitted before
//								next buffer due to bus width limitation
//		out_leftover_offset:	offset of the payload leftover
//
// Inputs:
//  	in_buf_size:			buffer size left before mpacket 1
//		in_offset:				offset for mpacket 1 
//		in_offset_mod8:			offset mod by 8
//
// Constants:
//		None   
//
//	Labels:
//		MORE_THAN_ONE_MPKTS_LABEL:	symbolic address to go when more than
//									one mpacket required
//
// Size:
//		12 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_decide_sop_mpkt_size_not_last_buf(out_mpkt_1_size, \
			out_leftover_len, out_leftover_offset, in_buf_size, \
			in_offset, in_offset_mod8, MORE_THAN_ONE_MPKTS_LABEL)

.begin

	.reg	tmp

	// in this macros, out_leftover_len is first used to present 
	// maximum_allowable_mpkt1_size to save gpr usage
		
	#ifndef ADD_L2_HEADER

	alu[tmp, TBUF_ELE_SIZE, -, in_offset_mod8]
	alu[out_mpkt_1_size, tmp, AND, BUS_WIDTH_MASK]	 
	alu[tmp, out_mpkt_1_size, +, BUS_WIDTH]
	
	// comparison of in_buf_size with (maximum allowable mpk_1_size + 
	// BUS_WIDTH) will garrantee no less than BUS_WIDTH bytes in mpacket_2, if
	// mpacket_2 is required, so garantees that SKIP bit will not be needed for
	// mpacket_2

	alu[--, in_buf_size, -, tmp]
	bge[MORE_THAN_ONE_MPKTS_LABEL]

	alu[out_mpkt_1_size, in_buf_size, AND, BUS_WIDTH_MASK]	 
	alu[out_leftover_len, in_buf_size, -, out_mpkt_1_size]
	alu[out_leftover_offset, in_offset, +,  out_mpkt_1_size]

	#else	// ADD_L2_HEADER
	
	// is L2 haeder already exist?

	move[tmp, L2_HEADER_ALREADY_EXIST_ID]
    alu[--, nexthop_id, -, tmp]
	beq[l2_header_already_exist#]

	alu[tmp, SINGLE_MPKT_MAX_SIZE, -, in_offset_mod8]
	alu[tmp, tmp, +, L2_HEADER_SIZE]
	alu[tmp, tmp, AND, BUS_WIDTH_MASK]
	alu[out_mpkt_1_size, tmp, -, L2_HEADER_SIZE] ;max allowable mop mpkt1
	alu[leftover_len, out_mpkt_1_size, +, BUS_WIDTH]
	
	alu[--, in_buf_size, -, leftover_len]
	bge[MORE_THAN_ONE_MPKTS_LABEL]

	alu[tmp, in_buf_size, +, L2_HEADER_SIZE]
	alu[tmp, tmp, AND, BUS_WIDTH_MASK]
	alu[out_mpkt_1_size, tmp, -, L2_HEADER_SIZE]
		 
	alu[out_leftover_len, in_buf_size, -, out_mpkt_1_size]
	alu[out_leftover_offset, in_offset, +,  out_mpkt_1_size]
	br[done#]

l2_header_already_exist#:

	alu[tmp, TBUF_ELE_SIZE, -, in_offset_mod8]
	alu[out_mpkt_1_size, tmp, AND, BUS_WIDTH_MASK]	 
	alu[tmp, out_mpkt_1_size, +, BUS_WIDTH]
	
	alu[--, in_buf_size, -, tmp]
	bge[MORE_THAN_ONE_MPKTS_LABEL]

	alu[out_mpkt_1_size, in_buf_size, AND, BUS_WIDTH_MASK]	 
	alu[out_leftover_len, in_buf_size, -, out_mpkt_1_size]
	alu[out_leftover_offset, in_offset, +,  out_mpkt_1_size]

done#:

#endif	// #ifndef ADD_L2_HEADER

.end 

#endm	// _pkt_tx_16p_decide_sop_mpkt_size_not_last_buf() 

///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_decide_mpkt_2_size_last_buf()
//
// Description: decide the payload size of mpacket 2 for the buffer which is 
//				the last buffer of the packet  
//
// Outputs:							
// 		out_mpkt_2_size:				mpacket 2 size
//		out_max_allowable_mpkt_payload:	maximum allowable payload if mpacket 2
//										is an EOP mpacket 
//
// Inputs:
//  	in_buf_size:			buffer size left before mpacket 1
//		in_mpkt_1_size:			size for mpacket 1
//		in_mpkt_1_offset:		offset for mpacket 1 
//
// Constants:
//		None   
//
//	Labels:
//		MORE_THAN_TWO_MPKTS_LABEL:	symbolic address to go when more than
//									two mpacket required
//
// Size:
//		8 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_decide_mpkt_2_size_last_buf(out_mpkt_2_size, in_buf_size, \
				in_mpkt_1_size, in_mpkt_1_offset, MORE_THAN_TWO_MPKTS_LABEL)

.begin

	.reg	tmp
	.reg	tmp1

	alu[tmp, in_mpkt_1_offset, +, in_mpkt_1_size]
	alu[tmp, tmp, AND, CONST_07]				; offset_mod_8 for mpkt_2

	// payload in this buffer left after mpkt_1

	alu[tmp1, in_buf_size, -, in_mpkt_1_size]	; payload_after_mpkt_1	

	// maximum allowable mpkt_2_size

	alu[tmp, TBUF_ELE_SIZE, -, tmp] 	; maximum allowable eop mpkt_2_size
	alu[out_mpkt_2_size, tmp, AND, BUS_WIDTH_MASK]	; mpkt_2_size for not eop	

	alu[--, tmp1, -, tmp]
	bgt[MORE_THAN_TWO_MPKTS_LABEL]
	alu[out_mpkt_2_size, --, B, tmp1]	; mpkt_2_size for eop mpkt

.end 

#endm	// _pkt_tx_16p_decide_mpkt_2_size_last_buf() 


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_decide_mpkt_3_size_last_buf()
//
// Description: decide the payload size of mpacket 3 for the buffer which is 
//				the last buffer of the packet  
//
// Outputs:							
// 		out_mpkt_3_size:		mpacket 3 size
//
// Inputs:
//		in_mpkt_2_size:			size for mpkt_2
//  	in_buf_size:			buffer size left before mpacket 1
//		in_mpkt_1_size:			size for mpacket 1
//		in_mpkt_1_offset:		offset for mpacket 1 
//
// Constants:
//		None   
//
//	Labels:
//		MORE_THAN_THREE_MPKTS_LABEL:	symbolic address to go when more than
//										three mpackets required
//
// Size:
//		10 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_decide_mpkt_3_size_last_buf(out_mpkt_3_size, \
						in_mpkt_2_size, in_buf_size, in_mpkt_1_size, \
						in_mpkt_1_offset, MORE_THAN_THREE_MPKTS_LABEL)

.begin

	.reg	tmp
	.reg	tmp1

	alu[tmp, in_mpkt_1_offset, +, in_mpkt_1_size]
	alu[tmp, tmp, +, in_mpkt_2_size]
	alu[tmp, tmp, AND, CONST_07]					; offset_mod_8 for mpkt_3
	alu[tmp1, in_buf_size, -, in_mpkt_1_size]
	alu[tmp1, tmp1, -, in_mpkt_2_size]	; payload after mpkt_2

	// maximum allowable mpkt_3_size

	alu[tmp, TBUF_ELE_SIZE, -, tmp] 	; maximum allowable eop mpkt_3_size 
	alu[out_mpkt_3_size, tmp, AND, BUS_WIDTH_MASK]	; mpkt_3_size for not eop
	alu[--, tmp1, -, tmp]
	bgt[MORE_THAN_THREE_MPKTS_LABEL]
	alu[out_mpkt_3_size, --, B, tmp1]	; mpkt_3_size for eop mpkt
.end 

#endm	// _pkt_tx_16p_decide_mpkt_3_size_last_buf() 


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_decide_mpkt_2_size_not_last_buf()
//
// Description: decide the payload size of mpacket 2 in the buffer which is
//				not the last buffer of the packet
//
// Outputs:							
// 		out_mpkt_2_size:		mpacket 2 size
//		out_leftover_len:		payload leftover to be transmitted before
//								next buffer due to bus width limitation
//		out_leftover_offset:	offset of the payload leftover
//
// Inputs:
//  	in_buf_size:					buffer size left before mpacket 1
//		in_mpkt_1_size:					size for mpacket 1
//		in_mpkt_1_offset:				offset for mpacket 1 
//
// Constants:
//		None   
//
//	Labels:
//		MORE_THAN_TWO_MPKTS_LABEL:	symbolic address to go when more than
//									two mpacket required
//
// Size:
//		12 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_decide_mpkt_2_size_not_last_buf(out_mpkt_2_size, \
			out_leftover_len, out_leftover_offset, in_buf_size, \
			in_mpkt_1_size, in_mpkt_1_offset, MORE_THAN_TWO_MPKTS_LABEL)

.begin
		
	.reg	tmp

	// in this macros, out_leftover_offset is first used to present 
	// offset_mpkt_2 and  out_leftover_len is first used to present 
	// payload_after_mpkt_1 to save gpr usage

	alu[out_leftover_offset, in_mpkt_1_offset, +, in_mpkt_1_size]	
	alu[tmp, out_leftover_offset, AND, CONST_07]	; offset_mod_8 for mpkt_2

	// payload in this buffer left after mpkt_1

	alu[out_leftover_len, in_buf_size, -, in_mpkt_1_size] ; paylo_after_mpkt_1

	// maximum allowable mpkt_2_size

	alu[tmp, TBUF_ELE_SIZE, -, tmp]
	alu[tmp, tmp, AND, BUS_WIDTH_MASK]		; applies bus width restriction 
	alu[out_mpkt_2_size, --, B, tmp]		; maximum allowable mpk_2_size		
			
	alu[tmp, tmp, +, BUS_WIDTH]		; maximum allowable mpk_2_size + BUS_WIDTH

	// comparison of paylo_after_mpkt_1_in_this_buf with (maximum allowable 
	// mpk_2_size + BUS_WIDTH) will garrantee no less than BUS_WIDTH bytes 
	// left in mpacket_3, if mpacket_3 is needed, so SKIP bit will never be
	// needed for mpacket_3

	alu[--, out_leftover_len, -, tmp]
	bge[MORE_THAN_TWO_MPKTS_LABEL]

	alu[out_mpkt_2_size, out_leftover_len, AND, BUS_WIDTH_MASK]	
	alu[out_leftover_len, out_leftover_len, -, out_mpkt_2_size]
	alu[out_leftover_offset, out_leftover_offset, +,  out_mpkt_2_size]

.end 

#endm	// _pkt_tx_16p_decide_mpkt_2_size_not_last_buf() 


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_decide_mpkt_3_size_not_last_buf()
//
// Description: decide the payload size of mpacket 3 in the buffer which is
//				not the last buffer of the packet
//
// Outputs:							
// 		out_mpkt_3_size:		mpacket 3 size
//		out_leftover_len:		payload leftover to be transmitted before
//								next buffer due to bus width limitation
//		out_leftover_offset:	offset of the payload leftover
//
//
// Inputs:
//  	in_buf_size:					buffer size left before mpacket 1
//		in_mpkt_1_size:					size for mpacket 1
//		in_mpkt_1_offset:				offset for mpacket 1 
//		in_mpkt_2_size:					mpacket 2 payload size 
//
// Constants:
//		None   
//
//	Labels:
//		MORE_THAN_THREE_MPKTS_LABEL:	symbolic address to go when more than
//										three mpackets required
//
// Size:
//		13 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_decide_mpkt_3_size_not_last_buf(out_mpkt_3_size, \
				out_leftover_len, out_leftover_offset, in_buf_size, \
				in_mpkt_1_size, in_mpkt_1_offset, in_mpkt_2_size, \
				MORE_THAN_THREE_MPKTS_LABEL)

.begin

	.reg	tmp

	// in this macros, out_leftover_offset is first used to present 
	// offset_mpkt_3 and  out_leftover_len is first used to present 
	// payload_after_mpkt_2 to save gpr usage

	alu[out_leftover_offset, in_mpkt_1_offset, +, in_mpkt_1_size]
	alu[out_leftover_offset, out_leftover_offset, +, in_mpkt_2_size]
	alu[tmp, out_leftover_offset, AND, CONST_07] ; offset_mod_8 for mpkt_3
	alu[out_leftover_len, in_buf_size, -, in_mpkt_1_size]

	// payload_after_mpkt_2

	alu[out_leftover_len, out_leftover_len, -, in_mpkt_2_size]

	alu[tmp, TBUF_ELE_SIZE, -, tmp]
	alu[tmp, tmp, AND, BUS_WIDTH_MASK]	; applies bus width restriction 
	alu[out_mpkt_3_size, --, B, tmp]	; maximum allowable mpkt_3_size		
			
	alu[tmp, tmp, +, BUS_WIDTH]	; maximum allowable mpkt_3_size	+ leftover 

	// comparison of paylo_after_mpkt_2_in_this_buf with (maximum allowable 
	// mpk_3_size + BUS_WIDTH) will garrantee no less than BUS_WIDTH bytes 
	// left in this buf and saved back to local memory, so garantee that SKIP
	// bit will never be needed for mpacket_1 when the buffer be processed
	// again in future, if the remaining payload of this buffer from last
	// process can not be fitted into current three mpacket with leftover 
	// less than BUS_WIDTH

	alu[--, out_leftover_len, -, tmp]
	bge[MORE_THAN_THREE_MPKTS_LABEL]

	alu[out_mpkt_3_size, out_leftover_len, AND, BUS_WIDTH_MASK]	
	alu[out_leftover_len, out_leftover_len, -, out_mpkt_3_size]
	alu[out_leftover_offset, out_leftover_offset, +,  out_mpkt_3_size]
.end 

#endm	// _pkt_tx_16p_decide_mpkt_3_size_not_last_buf() 


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_read_leftover_from_dram()
//
// Description: read leftover payload from dram to be kept in local memory and 
//				transmitted before next buffer payload
//
// Outputs:	
//		None 
//						
// Inputs/Outputs:					
// 		io_sig_mask:			signal mask
//

// Inputs:
//		in_buf_handle:			buffer handle
//		in_leftover_len:		payload leftover to be transmitted before
//								next buffer due to bus width limitation
//		in_leftover_offset:		offset of the payload leftover
//		in_sig:					signal 
//
// Constants:
//		None   
//
// Size:
//		5 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_read_leftover_from_dram(io_sig_mask, in_buf_handle, \
			in_leftover_len, in_leftover_offset, in_sig_dram)	

.begin
		
	.reg	tmp

	_pkt_tx_16p_get_dram_addr(tmp, in_buf_handle)	; get dram_addr (2)
	
	dram[read, $$dw0, tmp, in_leftover_offset, LEFTOVER_READ_LENGTH], \
				sig_done[in_sig_dram]	
	_pkt_tx_16p_set_signal(io_sig_mask, in_sig_dram)	
	_pkt_tx_16p_set_push_signal(io_sig_mask, in_sig_dram)	; add push signal

.end 

#endm	// _pkt_tx_16p_read_leftover_from_dram() 


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_read_tx_sequencer()
//
// Description: get mpackets actually sent out of the tbuf for use in next 
//				phase to avoid overwritting existing mpackets which have not 
// 				been transmitted out yet
//
// Outputs:							
// 		out_mpkts_sent:		mpacket sent from tx_sequence_0 reg
//
// Inputs/Ouputs:
//		io_sigmask			signal mask
//
// Inputs:
// 		in_signal:			signal for msf read
//
// Constants:	
//		None   
//
// Size:
//		2 instruction
//
///////////////////////////////////////////////////////////////////////////////
#macro	_pkt_tx_16p_read_tx_sequencer(out_mpkts_sent, io_sigmask, in_signal)

.begin							

	msf[read, out_mpkts_sent, tx_seq_addr, 0, 1], sig_done[in_signal]	

.end 

#endm	// _pkt_tx_16p_read_tx_sequencer()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_read_meta_data()  
//
// Description: read the first to third longwords of meta data from sram
//
// Outputs:							
//		out_xmeta:			the sram transfer registers to keep meta data 
//
// Inputs:  
//		io_sigmask:			signal mask
//
// Inputs:  
//		in_sig:				signal used to read meta data
//		in_buf_handle:		buffer handle to get meta data
//
// Constants:
//		None   
//
// Size:
//		4 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _pkt_tx_16p_read_meta_data(out_xmeta, io_sigmask, in_sig, in_buf_handle)

.begin

	// read meta data by dispatch_loop macro, signal (sig_sram_read_1)
	// is included in sigmask_phs2_default to save one instruction cycle 
	// in the critical POS min packet case

	dl_meta_load_cache(in_buf_handle, out_xmeta, in_sig, 0, \
							META_DATA_LW_TO_READ)

.end 

#endm	 // _pkt_tx_16p_read_meta_data()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_clear_signal()
//
// Description:
//		Clear a signal from a signal mask 
//
// Outputs: 
//		None 
//
// Inputs/Outputs:  
//		io_sigmask:			signal mask to be cleared with in_sig
//
// Inputs:  
//		in_sig:				signal number to clear
//
// Constants
//
//		None
//
// Size: 
//		1 instructions
// 
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_clear_signal(io_sigmask, in_sig)

.begin

#if ( isnum(in_sig) )
	
	#define_eval SIG_SHIFT sig
	alu_shf[io_sigmask, io_sigmask, AND~, 1, <<SIG_SHIFT ]
	#undef SIG_SHIFT

#else

	alu[io_sigmask, io_sigmask, AND~, 1, <<(&in_sig)]

#endif

.end

#endm	// _pkt_tx_16p_clear_signal()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_clear_push_signal()
//
// Description:
//		Clear a signal from a signal mask 
//
// Outputs: 
//		None 
//
// Inputs/Outputs:  
//		io_sigmask:			signal mask to be cleared with in_sig
//
// Inputs:  
//		in_sig:				signal number to clear
//
// Constants
//
//		None
//
// Size: 
//
//		1 instructions
// 
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_clear_push_signal(io_sigmask, in_sig)

.begin

	.reg	tmp

#if ( isnum(in_sig) )

	#define_eval SIG_SHIFT (in_sig +1)
	alu_shf[io_sigmask, io_sigmask, AND~, 1, <<SIG_SHIFT]
	#undef SIG_SHIFT

#else

	alu[tmp, --, B, &in_sig]
	alu[tmp, tmp, +, 1]
	alu[--, tmp, OR, 0]										
	alu_shf[io_sigmask, io_sigmask, AND~, 1, <<indirect]

#endif

.end

#endm	// _pkt_tx_16p_clear_push_signal()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_set_signal()
//
// Description:
// 	
//		Set a signal in a signal mask 
//
// Outputs: 
//		None 
//
// Inputs/Outputs:  
//		io_sigmask:			signal mask to be added with in_sig
//
// Inputs:  
//		in_sig:				signal number to set
//
// Constants
//		None
//
// Size: 
//		1 instruction
// 
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_set_signal(io_sigmask, in_sig)

.begin

#if ( isnum(in_sig) )

	#define_eval SIG_SHIFT in_sig
	alu_shf[io_sigmask, io_sigmask, OR, 1, <<SIG_SHIFT]
	#undef SIG_SHIFT

#else

	alu[io_sigmask, io_sigmask, OR, 1, <<(&in_sig)]

#endif

.end

#endm	// _pkt_tx_16p_set_signal()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_set_push_signal()
//
// Description:
// 	
//		Add push signal (i.e. dram access has puu and push signal) in a signal 
//		mask 
//
// Outputs:
//		None 
//
// Inputs/Outputs:  
//		io_sigmask:			signal mask to be added with push signal of in_sig
//
// Inputs:  
//		in_sig:				signal number to set
//
// Constants
//		None
//
// Size: 
//		1-4 instructions
// 
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_set_push_signal(io_sigmask, in_sig)

.begin

	.reg	tmp

#if ( isnum(in_sig) )

	#define_eval SIG_SHIFT (in_sig +1)
	alu_shf[io_sigmask, io_sigmask, OR, 1, <<SIG_SHIFT]
	#undef SIG_SHIFT

#else

	alu[tmp, --, B, &in_sig]
	alu[tmp, tmp, +, 1]
	alu[--, tmp, OR, 0]
	alu_shf[io_sigmask, io_sigmask, OR, 1, <<indirect]

#endif

.end

#endm	 // _pkt_tx_16p_set_push_signal()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_allocate_tbuf_from_scratch_memory()
//	
//	Description:
//		allocate next available tbuf element through the scratch memory 
//		atomic operation SCRACTH[test_and_incr..]
//	Outputs:	
//		None
//		
//	Inputs/Outputs:
//		io_avail_tbuf_ele:			next available tbuf element
//		io_sig_mask					signal mask
//
//	Inputs:
//		in_sig_scratch:				signal for scratch atomic operation
//
//	CONSTANTS:	
//		None
//	
// Size: 
//		1 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_allocate_tbuf_from_scratch_memory(io_avail_tbuf_ele, \
			io_sig_mask, in_sig_scratch) 

.begin

	scratch[test_and_add, io_avail_tbuf_ele, tbuf_scr_addr, 0], \
			sig_done[in_sig_scratch]
	_pkt_tx_16p_set_push_signal(io_sig_mask, in_sig_scratch)


.end

#endm 	// _pkt_tx_16p_allocate_tbuf_from_scratch_memory() 


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_read_msf_flow_control()
//	
//	Description:
//		read flow control related MSF register
//	Outputs:	
//		out_mphy_fc_stst:	status of MSF register for flow control
//		
//	Inputs:
//		in_addr:			address of MSF register for flow control
//		in_signal:			signal for msf read 
//
//	CONSTANTS:	
//		None
//	
// Size: 
//		1 instructions
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_read_msf_flow_control(out_mphy_fc_stst, in_addr, in_signal)

.begin

	msf[read, out_mphy_fc_stst, in_addr,  0, 1], sig_done[in_signal]	
.end

#endm	// _pkt_tx_16p_read_msf_flow_control()		


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_read_tbuf_alloc_scratch_memory()
//	
//	Description:
//		get next available tbuf element through the scratch memory read for
//		tbuf full check
//	Outputs:	
//		None
//		
//	Inputs:
//		in_avail_tbuf_ele:			next available tbuf element
//		in_sig_scratch:				signal for scratch read operation
//      in_sig_mask:				signal mask to add in_sig_scratch
//
//	CONSTANTS:	
//		None
//	
// Size: 
//		2 instruction
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_read_tbuf_alloc_scratch_memory(out_avail_tbuf_ele, \
											in_sig_scratch, in_sig_mask)
.begin

	scratch[read, out_avail_tbuf_ele, tbuf_scr_addr, 0, 1], 
			sig_done[in_sig_scratch]
	_pkt_tx_16p_set_signal(in_sig_mask, in_sig_scratch)

.end

#endm	// _pkt_tx_16p_read_tbuf_alloc_scratch_memory() 


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_load_sop_eop_mpkt() 
//
// Description: move sop_eop mpacket to tbuf
//
// Outputs:							
// 		out_tcw0:			first transmit control word
//
// Inputs/Outputs:							
//		io_sig_mask:		signal mask
//
// Inputs:  
//		in_sig_dram:		signal number for dram[tbuf_wr...] instruction
//		in_dramaddr:		staring address of the buffer in dram
//		in_tbuf_ele:		the tbuf element number 
//      in_off_set: 		the starting address of the payload to copy
//		in_paylo_len:			payload length
//		in_offsetmod8:		payload offset
//		in_port:			port to transmit
//
// Constants:
//		None
//
// Size:
//		10 instructions (no L2_HEADER)
//
///////////////////////////////////////////////////////////////////////////////
 
#macro 	_pkt_tx_16p_load_sop_eop_mpkt(out_tcw0, io_sig_mask, in_sig_dram, \
			in_dramaddr, in_tbuf_ele, in_off_set, in_paylo_len, \
			in_offsetmod8, in_port)
 
.begin

	.reg	tbuf_addr

#ifdef ADD_L2_HEADER
	.reg tbuf_paylo_addr
	.reg no_header_tcw0_sop_eop_base
#endif	// #ifndef ADD_L2_HEADER
	
 	alu_shf[tbuf_addr, lm1_tbuf_base, or, in_tbuf_ele, <<TBUF_ADDR_SHF]

#ifndef ADD_L2_HEADER

	_pkt_tx_16p_move_payload_to_tbuf(in_dramaddr, tbuf_addr, in_off_set, \
			in_paylo_len, in_offsetmod8, in_sig_dram)	; (6)

	_pkt_tx_16p_prepare_tcw0_with_base(out_tcw0, in_offsetmod8, in_paylo_len, \
				in_port, lm1_tcw0_sop_eop_base) ; (3)

#else	// ADD_L2_HEADER
	
	// is L2 haeder already exist?

	move[tmp, L2_HEADER_ALREADY_EXIST_ID]
    alu[--, nexthop_id, -, tmp]
	beq[l2_header_already_exist#]

	alu[tbuf_paylo_addr, tbuf_addr, +, BYTES_FOR_L2_HEADER]

	_pkt_tx_16p_move_payload_to_tbuf(in_dramaddr, tbuf_paylo_addr, in_off_set, \
			in_paylo_len, in_offsetmod8, in_sig_dram)	; (8)

	_pkt_tx_16p_write_l2_header(io_sig_mask, sig_l2_header, tbuf_addr)

	_pkt_tx_16p_prepare_tcw0_with_base(out_tcw0, in_offsetmod8, in_paylo_len, \
				in_port, lm1_tcw0_sop_eop_base) ; (3)

	br[done#]

l2_header_already_exist#:

	_pkt_tx_16p_move_payload_to_tbuf(in_dramaddr, tbuf_addr, in_off_set, \
			in_paylo_len, in_offsetmod8, in_sig_dram)	; (8)

	_pkt_tx_16p_clear_signal(io_sig_mask, sig_msf_access_1)	

	// mask out prepend offset and length
	alu[tmp, --, B, lm1_tcw0_sop_eop_base]
	ld_field_w_clr[no_header_tcw0_sop_eop_base, 0011, tmp]

	_pkt_tx_16p_prepare_tcw0_with_base(out_tcw0, in_offsetmod8, in_paylo_len, \
				in_port, no_header_tcw0_sop_eop_base) ; (3)

done#:

#endif	// #ifndef ADD_L2_HEADER

.end 

#endm	// _pkt_tx_16p_load_sop_eop_mpkt()

///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_load_sop_mpkt() 
//
// Description: move sop mpacket to tbuf
//
// Outputs:							
// 		out_tcw0:			first transmit control word
//
// Inputs/Outputs:							
//		io_sig_mask:		signal mask
//
// Inputs:  
//		in_sig_dram:		signal number for dram[tbuf_wr...] instruction
//		in_dramaddr:		staring address of the buffer in dram
//		in_tbuf_ele:		the tbuf element number 
//      in_off_set: 		the starting address of the payload to copy
//		in_paylo_len:			payload length
//		in_offsetmod8:		payload offset
//		in_port:			port to transmit
//
// Constants:
//		None
//
// Size:
//		12 instructions (no L2_HEADER)
//
///////////////////////////////////////////////////////////////////////////////
 
#macro 	_pkt_tx_16p_load_sop_mpkt(out_tcw0, io_sig_mask, in_sig_dram, \
			in_dramaddr, in_tbuf_ele, in_off_set, in_paylo_len, \
			in_offsetmod8, in_port)
 
.begin

	.reg	tbuf_addr
	.reg	sop_only_tcw0_base

	#ifdef ADD_L2_HEADER
	.reg tbuf_paylo_addr
	#endif	// #ifndef ADD_L2_HEADER
	
 	alu_shf[tbuf_addr, lm1_tbuf_base, or, in_tbuf_ele, <<TBUF_ADDR_SHF]

	// mask out EOP bit

	alu_shf[sop_only_tcw0_base, lm1_tcw0_sop_eop_base, AND~, 1, \
						<<TCW0_EOP_BIT_SHFT]

	#ifndef ADD_L2_HEADER
	
	_pkt_tx_16p_move_payload_to_tbuf(in_dramaddr, tbuf_addr, in_off_set, \
			in_paylo_len, in_offsetmod8, in_sig_dram)	; (8)

	_pkt_tx_16p_prepare_tcw0_with_base(out_tcw0, in_offsetmod8, in_paylo_len, \
				in_port, sop_only_tcw0_base,) ; (3)

	#else	// ADD_L2_HEADER
	
	// is L2 haeder already exist?

	move[tmp, L2_HEADER_ALREADY_EXIST_ID]
    alu[--, nexthop_id, -, tmp]
	beq[l2_header_already_exist#]

	alu[tbuf_paylo_addr, tbuf_addr, +, BYTES_FOR_L2_HEADER]

	_pkt_tx_16p_move_payload_to_tbuf(in_dramaddr, tbuf_paylo_addr, in_off_set, \
			in_paylo_len, in_offsetmod8, in_sig_dram)	; (8)

	_pkt_tx_16p_write_l2_header(io_sig_mask, sig_l2_header, tbuf_addr)

	_pkt_tx_16p_prepare_tcw0_with_base(out_tcw0, in_offsetmod8, in_paylo_len, \
				in_port, sop_only_tcw0_base,) ; (3)

	br[done#]

l2_header_already_exist#:

	_pkt_tx_16p_move_payload_to_tbuf(in_dramaddr, tbuf_addr, in_off_set, \
			in_paylo_len, in_offsetmod8, in_sig_dram)	; (8)


	// mask out prepend offset and length
	alu[tmp, --, B, sop_only_tcw0_base]
	ld_field_w_clr[sop_only_tcw0_base, 0011, tmp]

	_pkt_tx_16p_prepare_tcw0_with_base(out_tcw0, in_offsetmod8, in_paylo_len, \
				in_port, sop_only_tcw0_base,) ; (3)

done#:

#endif	// #ifndef ADD_L2_HEADER

.end 

#endm	// _pkt_tx_16p_load_sop_mpkt()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_load_mop_mpkt_no_leftover() 
//
// Description: move mop mpacket to tbuf without leftover from previous 
//				buffer to transmit before payload of this buffer 
//
// Outputs:							
// 		out_tcw0:			first transmit control word
//
// Inputs/Outputs:							
//		io_sig_mask:		signal mask
//		
// Inputs:  
//		in_sig_dram:		signal number for dram[tbuf_wr...] instruction
//		in_dramaddr:		staring address of the buffer in dram
//		in_tbuf_ele:		the tbuf element number 
//      in_off_set: 		the starting address of the payload to copy
//		in_paylo_len:			payload length
//		in_offsetmod8:		payload offset
//		in_port:			port to transmit
//
// Constants:
//		None
//
// Size:
//		11 instructions (no L2_HEADER)
//
///////////////////////////////////////////////////////////////////////////////
 
#macro 	_pkt_tx_16p_load_mop_mpkt_no_leftover(out_tcw0, io_sig_mask, \
			in_sig_dram, in_dramaddr, in_tbuf_ele, in_off_set, in_paylo_len, \
			in_offsetmod8, in_port)
.begin

	.reg	tbuf_addr
	.reg 	tmp_port

 	alu_shf[tbuf_addr, lm1_tbuf_base, or, in_tbuf_ele, <<TBUF_ADDR_SHF]

	_pkt_tx_16p_move_payload_to_tbuf(in_dramaddr, tbuf_addr, in_off_set, \
			in_paylo_len, in_offsetmod8, in_sig_dram)	; (8)

	// following tmp_port used to resolved odd-sided gpr allocation conflict 

	alu[tmp_port, --, B, in_port]
	alu[out_tcw0, tmp_port, OR, in_offsetmod8, <<TCW0_PAYLOAD_OFFSET_LSB_LOC]
	alu[out_tcw0, out_tcw0, OR, in_paylo_len, <<TCW0_PAYLOAD_LEN_LSB_LOC]

.end 

#endm	// _pkt_tx_16p_load_mop_mpkt_no_leftover()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_load_mop_mpkt_with_leftover() 
//
// Description: move mop mpacket to tbuf with leftover from previous 
//				buffer to transmit before payload of this buffer 
//
// Outputs:							
// 		out_tcw0:			first transmit control word
//
// Inputs/Outputs:							
//		io_sig_mask:		signal mask
//		
// Inputs:  
//		in_sig_dram:		signal number for dram[tbuf_wr...] instruction
//		in_dramaddr:		staring address of the buffer in dram
//		in_tbuf_ele:		the tbuf element number 
//      in_off_set: 		the starting address of the payload to copy
//		in_paylo_len:			payload length
//		in_offsetmod8:		payload offset
//		in_port:			port to transmit
//      in_leftover_to_tx:	length of leftover from previous buffer to transmit
//		in_sig_leftover:	signal for msf[write....] leftover
// Constants:
//		None
//
// Size:
//		 instructions 
//
///////////////////////////////////////////////////////////////////////////////
 
#macro 	_pkt_tx_16p_load_mop_mpkt_with_leftover(out_tcw0, io_sig_mask, \
			in_sig_dram, in_dramaddr, in_tbuf_ele, in_off_set, in_paylo_len, \
			in_offsetmod8, in_port, in_leftover_to_tx, in_sig_leftover)
.begin

	.reg	tbuf_addr
	.reg 	tbuf_paylo_addr

 	alu_shf[tbuf_addr, lm1_tbuf_base, or, in_tbuf_ele, <<TBUF_ADDR_SHF]

	alu[tbuf_paylo_addr, tbuf_addr, +, LEFTOVER_TBUF_SPACE]

#if IS_IXPTYPE(__IXP28XX)

	// if leftover length less or equal to 8, then payload will be load 
	// from tbuf_addr + 8

	alu[--, in_leftover_to_tx, -, 8]
	bgt[payload_start_address_done#]

	alu[tbuf_paylo_addr, tbuf_addr, +, 8]
payload_start_address_done#:

#endif 	//#if IS_IXPTYPE(__IXP28XX)

	_pkt_tx_16p_move_payload_to_tbuf(in_dramaddr, tbuf_paylo_addr, \
				in_off_set, in_paylo_len, in_offsetmod8, in_sig_dram)	; (8)

	_pkt_tx_16p_load_leftover(io_sig_mask, in_sig_leftover, \
												in_leftover_to_tx, tbuf_addr)

	alu[out_tcw0, in_port, OR, in_offsetmod8, <<TCW0_PAYLOAD_OFFSET_LSB_LOC]
	alu[out_tcw0, out_tcw0, OR, in_paylo_len, <<TCW0_PAYLOAD_LEN_LSB_LOC]
	alu[out_tcw0, out_tcw0, OR, in_leftover_to_tx, <<TCWO_PREPEND_LEN_LSB_LOC]

.end 

#endm	// _pkt_tx_16p_load_mop_mpkt_with_leftover()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_load_eop_mpkt_no_leftover() 
//
// Description: move eop mpacket to tbuf without leftover from previous 
//				buffer to transmit before payload of this buffer 
//
// Outputs:							
// 		out_tcw0:			first transmit control word
//
// Inputs/Outputs:							
//		io_sig_mask:		signal mask
//		
// Inputs:  
//		in_sig_dram:		signal number for dram[tbuf_wr...] instruction
//		in_dramaddr:		staring address of the buffer in dram
//		in_tbuf_ele:		the tbuf element number 
//      in_off_set: 		the starting address of the payload to copy
//		in_paylo_len:			payload length
//		in_offsetmod8:		payload offset
//		in_port:			port to transmit
//
// Constants:
//		None
//
// Size:
//		12 instructions 
//
///////////////////////////////////////////////////////////////////////////////
 
#macro 	_pkt_tx_16p_load_eop_mpkt_no_leftover(out_tcw0, io_sig_mask, \
			in_sig_dram, in_dramaddr, in_tbuf_ele, in_off_set, in_paylo_len, \
			in_offsetmod8, in_port)
.begin

	.reg	tbuf_addr

 	alu_shf[tbuf_addr, lm1_tbuf_base, or, in_tbuf_ele, <<TBUF_ADDR_SHF]

	_pkt_tx_16p_move_payload_to_tbuf(in_dramaddr, tbuf_addr, in_off_set, \
			in_paylo_len, in_offsetmod8, in_sig_dram)	; (8)

	alu_shf[out_tcw0, in_port, OR, 1, <<TCW0_EOP_BIT_SHFT]
	alu[out_tcw0, out_tcw0, OR, in_offsetmod8, <<TCW0_PAYLOAD_OFFSET_LSB_LOC]
	alu[out_tcw0, out_tcw0, OR, in_paylo_len, <<TCW0_PAYLOAD_LEN_LSB_LOC]

.end 

#endm	// _pkt_tx_16p_load_eop_mpkt_no_leftover()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_load_eop_mpkt_with_leftover() 
//
// Description: move eop mpacket to tbuf with leftover from previous 
//				buffer to transmit before payload of this buffer 
//
// Outputs:							
// 		out_tcw0:			first transmit control word
//
// Inputs/Outputs:							
//		io_sig_mask:		signal mask
//		
// Inputs:  
//		in_sig_dram:		signal number for dram[tbuf_wr...] instruction
//		in_dramaddr:		staring address of the buffer in dram
//		in_tbuf_ele:		the tbuf element number 
//      in_off_set: 		the starting address of the payload to copy
//		in_paylo_len:			payload length
//		in_offsetmod8:		payload offset
//		in_port:			port to transmit
//      in_leftover_to_tx:	length of leftover from previous buffer to transmit
//		in_sig_leftover:	signal for msf[write....] leftover
//
// Constants:
//		None
//
// Size:
//		 instructions
//
///////////////////////////////////////////////////////////////////////////////
 
#macro 	_pkt_tx_16p_load_eop_mpkt_with_leftover(out_tcw0, io_sig_mask, \
			in_sig_dram, in_dramaddr, in_tbuf_ele, in_off_set, in_paylo_len, \
			in_offsetmod8, in_port, in_leftover_to_tx, in_sig_leftover)
.begin


	.reg	tbuf_addr
	.reg 	tbuf_paylo_addr

 	alu_shf[tbuf_addr, lm1_tbuf_base, or, in_tbuf_ele, <<TBUF_ADDR_SHF]

	alu[tbuf_paylo_addr, tbuf_addr, +, LEFTOVER_TBUF_SPACE]

#if IS_IXPTYPE(__IXP28XX)

	// if leftover length less or equal to 8, then payload will be load 
	// from tbuf_addr + 8

	alu[--, in_leftover_to_tx, -, 8]
	bgt[payload_start_address_done#]

	alu[tbuf_paylo_addr, tbuf_addr, +, 8]
payload_start_address_done#:

#endif 	//#if IS_IXPTYPE(__IXP28XX)

	_pkt_tx_16p_move_payload_to_tbuf(in_dramaddr, tbuf_paylo_addr, \
			in_off_set, in_paylo_len, in_offsetmod8, in_sig_dram)	; (8)

	_pkt_tx_16p_load_leftover(io_sig_mask, in_sig_leftover, \
										in_leftover_to_tx, tbuf_addr)

	alu_shf[out_tcw0, in_port, OR, 1, <<TCW0_EOP_BIT_SHFT]
	alu[out_tcw0, out_tcw0, OR, in_offsetmod8, <<TCW0_PAYLOAD_OFFSET_LSB_LOC]
	alu[out_tcw0, out_tcw0, OR, in_paylo_len, <<TCW0_PAYLOAD_LEN_LSB_LOC]
	alu[out_tcw0, out_tcw0, OR, in_leftover_to_tx, <<TCWO_PREPEND_LEN_LSB_LOC]

.end 

#endm	// _pkt_tx_16p_load_eop_mpkt_with_leftover()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_process_and_save_leftover()
//
// Description: align leftover read from dram, save them in local memory, and 
//				set flag in local memory
//
// Prerequisite:			local memory index 1 alrady set to queue status 
//							of the in_port
//
// Outputs:							
// 		None
//
// Inputs:
//		in_leftov_len:		leftover length decided in previous phase
//		in_leftov_offset:	leftover offset decided in previous phase
//
// Constants:	
//		None   
//
// Size:
//		instructions 
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_process_and_save_leftover(in_leftov_len, in_leftov_offset)

.begin

	.reg	offset_mod_8
	.reg	tmp

	alu[offset_mod_8, in_leftov_offset, AND, CONST_07]

	// jump based on offset_mod_8

	jump[offset_mod_8, jmp_tbl#], defer[3], targets[oset_0#, oset_1#, \
		oset_2#, oset_3#, oset_4#, oset_5#, oset_6#, oset_7#] 
	nop
	nop
	nop

//	Start of jump table
 
jmp_tbl#:

oset_0#:
	br[offset_mod_8_eq_0#]
			
oset_1#:
	br[offset_mod_8_eq_1#]

oset_2#:
	br[offset_mod_8_eq_2#]

oset_3#:
	br[offset_mod_8_eq_3#]

oset_4#:
	br[offset_mod_8_eq_4#]

oset_5#:
	br[offset_mod_8_eq_5#]

oset_6#:
	br[offset_mod_8_eq_6#]

oset_7#:
	br[offset_mod_8_eq_7#]

offset_mod_8_eq_0#:					; (8)
	alu[lm1_leftover_lw0, --, B, $$dw0]
#if IS_IXPTYPE(__IXP28XX)
	alu[lm1_leftover_lw1, --, B, $$dw1]
	alu[lm1_leftover_lw2, --, B, $$dw2]
	alu[lm1_leftover_lw3, --, B, $$dw3]
#endif	 // #if IS_IXPTYPE(__IXP28XX)
	br[align_save_done#]

offset_mod_8_eq_1#:					; (16)
	ld_field_w_clr[tmp, 1110, $$dw0, <<8]
	ld_field[tmp, 0001, $$dw1, >>24]
	alu[lm1_leftover_lw0, --, B, tmp]

#if IS_IXPTYPE(__IXP28XX)
	ld_field_w_clr[tmp, 1110, $$dw1, <<8]
	ld_field[tmp, 0001, $$dw2, >>24]
	alu[lm1_leftover_lw1, --, B, tmp]

	ld_field_w_clr[tmp, 1110, $$dw2, <<8]
	ld_field[tmp, 0001, $$dw3, >>24]
	alu[lm1_leftover_lw2, --, B, tmp]

	ld_field_w_clr[tmp, 1110, $$dw3, <<8]
	ld_field[tmp, 0001, $$dw4, >>24]
	alu[lm1_leftover_lw3, --, B, tmp]
#endif	 // #if IS_IXPTYPE(__IXP28XX)

	br[align_save_done#]

offset_mod_8_eq_2#:					; (16)
	ld_field_w_clr[tmp, 1100, $$dw0, <<16]
	ld_field[tmp, 0011, $$dw1, >>16]
	alu[lm1_leftover_lw0, --, B, tmp]

#if IS_IXPTYPE(__IXP28XX)
	ld_field_w_clr[tmp, 1100, $$dw1, <<16]
	ld_field[tmp, 0011, $$dw2, >>16]
	alu[lm1_leftover_lw1, --, B, tmp]

	ld_field_w_clr[tmp, 1100, $$dw2, <<16]
	ld_field[tmp, 0011, $$dw3, >>16]
	alu[lm1_leftover_lw2, --, B, tmp]

	ld_field_w_clr[tmp, 1100, $$dw3, <<16]
	ld_field[tmp, 0011, $$dw4, >>16]
	alu[lm1_leftover_lw3, --, B, tmp]
#endif	 // #if IS_IXPTYPE(__IXP28XX)

	br[align_save_done#]

offset_mod_8_eq_3#:					; (16)
	ld_field_w_clr[tmp, 1000, $$dw0, <<24]
	ld_field[tmp, 0111, $$dw1, >>8]
	alu[lm1_leftover_lw0, --, B, tmp]

#if IS_IXPTYPE(__IXP28XX)
	ld_field_w_clr[tmp, 1000, $$dw1, <<24]
	ld_field[tmp, 0111, $$dw2, >>8]
	alu[lm1_leftover_lw1, --, B, tmp]

	ld_field_w_clr[tmp, 1000, $$dw2, <<24]
	ld_field[tmp, 0111, $$dw3, >>8]
	alu[lm1_leftover_lw2, --, B, tmp]

	ld_field_w_clr[tmp, 1000, $$dw3, <<24]
	ld_field[tmp, 0111, $$dw4, >>8]
	alu[lm1_leftover_lw3, --, B, tmp]
#endif	 // #if IS_IXPTYPE(__IXP28XX)

	br[align_save_done#]

offset_mod_8_eq_4#:					; (8)
	alu[lm1_leftover_lw0, --, B, $$dw1]
#if IS_IXPTYPE(__IXP28XX)
	alu[lm1_leftover_lw1, --, B, $$dw2]
	alu[lm1_leftover_lw2, --, B, $$dw3]
	alu[lm1_leftover_lw3, --, B, $$dw4]
#endif	 // #if IS_IXPTYPE(__IXP28XX)
	br[align_save_done#]

offset_mod_8_eq_5#:					; (16)
	ld_field_w_clr[tmp, 1110, $$dw1, <<8]
	ld_field[tmp, 0001, $$dw2, >>24]
	alu[lm1_leftover_lw0, --, B, tmp]

#if IS_IXPTYPE(__IXP28XX)
	ld_field_w_clr[tmp, 1110, $$dw2, <<8]
	ld_field[tmp, 0001, $$dw3, >>24]
	alu[lm1_leftover_lw1, --, B, tmp]

	ld_field_w_clr[tmp, 1110, $$dw3, <<8]
	ld_field[tmp, 0001, $$dw4, >>24]
	alu[lm1_leftover_lw2, --, B, tmp]

	ld_field_w_clr[tmp, 1110, $$dw4, <<8]
	ld_field[tmp, 0001, $$dw5, >>24]
	alu[lm1_leftover_lw3, --, B, tmp]
#endif	 // #if IS_IXPTYPE(__IXP28XX)

	br[align_save_done#]

offset_mod_8_eq_6#:					; (16)
	ld_field_w_clr[tmp, 1100, $$dw1, <<16]
	ld_field[tmp, 0011, $$dw2, >>16]
	alu[lm1_leftover_lw0, --, B, tmp]

#if IS_IXPTYPE(__IXP28XX)
	ld_field_w_clr[tmp, 1100, $$dw2, <<16]
	ld_field[tmp, 0011, $$dw3, >>16]
	alu[lm1_leftover_lw1, --, B, tmp]

	ld_field_w_clr[tmp, 1100, $$dw3, <<16]
	ld_field[tmp, 0011, $$dw4, >>16]
	alu[lm1_leftover_lw2, --, B, tmp]

	ld_field_w_clr[tmp, 1100, $$dw4, <<16]
	ld_field[tmp, 0011, $$dw5, >>16]
	alu[lm1_leftover_lw3, --, B, tmp]
#endif	 // #if IS_IXPTYPE(__IXP28XX)

	br[align_save_done#]


offset_mod_8_eq_7#:					; (16)
	ld_field_w_clr[tmp, 1000, $$dw1, <<24]
	ld_field[tmp, 0111, $$dw2, >>8]
	alu[lm1_leftover_lw0, --, B, tmp]

#if IS_IXPTYPE(__IXP28XX)
	ld_field_w_clr[tmp, 1000, $$dw2, <<24]
	ld_field[tmp, 0111, $$dw3, >>8]
	alu[lm1_leftover_lw1, --, B, tmp]

	ld_field_w_clr[tmp, 1000, $$dw3, <<24]
	ld_field[tmp, 0111, $$dw4, >>8]
	alu[lm1_leftover_lw2, --, B, tmp]

	ld_field_w_clr[tmp, 1000, $$dw4, <<24]
	ld_field[tmp, 0111, $$dw5, >>8]
	alu[lm1_leftover_lw3, --, B, tmp]
#endif	 // #if IS_IXPTYPE(__IXP28XX)

align_save_done#:

	// save leftover length and set flag in local memory

	alu[tmp, --, B, lm1_lov_stat_next_next_buf]
	alu[tmp, tmp, AND~, 0xFF, <<24]		; clear MSF byte (leftover info)
	alu[tmp, tmp, OR, in_leftov_len, <<LM_LEFTOVER_LEN_BIT]
	alu[tmp, tmp, OR, 1, <<LM_LEFTOVER_FLAG_BIT]	; set flag
	alu[lm1_lov_stat_next_next_buf, --, B,  tmp]

.end

#endm	// _pkt_tx_16p_process_and_save_leftover()


/////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_load_leftover() 
//
// Description: write leftover to tbuf through msf[write...]
//
//
// Prerequisite:			local memory index 1 alrady set to queue status 
//							of the in_port
//
// Outputs:							
// 		None
//
// Inputs/Otputs:  
//		io_sig_mask:			signal mask
//
// Inputs:  
//      in_sig:				signal for prepend msf write
//		in_leftover_to_tx:	leftover length
//		in_tbuf_addr:		the tbuf element address 
//
// Constants:
//		None
//
// Size:
//		 instructions 
//
///////////////////////////////////////////////////////////////////////////////

#macro 	_pkt_tx_16p_load_leftover(io_sig_mask, in_sig, in_leftover_to_tx, \
					in_tbuf_addr)
.begin

	alu[$w0, --, B,lm1_leftover_lw0]

#if IS_IXPTYPE(__IXP24XX)

    // write leftover to tbuf

	msf[write, $w0, in_tbuf_addr, 0, 1], sig_done[in_sig]
#endif	 // #if IS_IXPTYPE(__IXP24XX)

#if IS_IXPTYPE(__IXP28XX)

	alu[--, in_leftover_to_tx, -, 4]
	bgt[more_than_4_bytes#]

	msf[write, $w0, in_tbuf_addr, 0, 1], sig_done[in_sig]
	br[write_to_tbuf_done#]

more_than_4_bytes#:
	alu[$w1, --, B,lm1_leftover_lw1]

	alu[--, in_leftover_to_tx, -, 8]
	bgt[more_than_8_bytes#]

	msf[write, $w0, in_tbuf_addr, 0, 2], sig_done[in_sig]
	br[write_to_tbuf_done#]

more_than_8_bytes#:
	alu[$w2, --, B,lm1_leftover_lw2]

	alu[--, in_leftover_to_tx, -, 12]
	bgt[more_than_12_bytes#]

	msf[write, $w0, in_tbuf_addr, 0, 3], sig_done[in_sig]
	br[write_to_tbuf_done#]

more_than_12_bytes#:

	alu[$w3, --, B,lm1_leftover_lw3]
	msf[write, $w0, in_tbuf_addr, 0, 4], sig_done[in_sig]

write_to_tbuf_done#:

#endif	 // #if IS_IXPTYPE(__IXP28XX)

	_pkt_tx_16p_set_signal[io_sig_mask, in_sig]

.end 

#endm	// _pkt_tx_16p_load_leftover()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_move_payload_to_tbuf() 
//
// Description: move payload of sop or sop_eop mpacket to tbuf
//
// Outputs:							
//		None
//
// Inputs:  
//		in_dramaddr:		staring address of the buffer in dram
//		int_tuf_addr:		tbuf address for in_tbuf
//      in_offset:			the starting address of the payload to copy
//		in_paylo:			payload length
//		in_offsetmod8:		payload offset
///		in_sig_dram:		signal number for dram[tbuf_wr...] instruction
//
// Constants:
//		None
//
// Size:
//		6 instructions
//
///////////////////////////////////////////////////////////////////////////////

#macro 	_pkt_tx_16p_move_payload_to_tbuf(in_dramaddr, in_tuf_addr, in_offset, \
		in_paylo, in_offsetmod8, in_sig_dram) 

.begin

	.reg	ref_indirect

	// prepare tbuf address for  dram[tbuf_wr......]

	alu[ref_indirect, lm1_indiref_base, or, in_tuf_addr, <<5]

	// total we need to fecth (offsetmod8 + paylo) bytes to tbuf

	alu[tmp, in_offsetmod8, +, in_paylo]

    // since ref count 0 will fectch 8 bytes, and ref count 1 will fetch
	// 16 bytes......, and ref count 15 will fetch 128 bytes, 
	// the right way to do it is followinhg:
	//    a) decrease tm_payload_len by one
	//    b) then right shift the result by 3
	// We will do step a first, then combine the right shift in the shift into
	// the ref count in indir_ref.  In this way, we make sure that we fetch 
	// enough and closet to 8 byte alignment of packet data	

	alu[tmp, tmp, -, 0x01]	

	// right shift tm_payload_len to get rid of any bits set in bit 0 to 3 to 
	// get ref_cnt for indir_ref 

    alu_shf[tmp, --, b, tmp, >>3]

    // left shift 21 bit to ref_cnt location

    alu[--, ref_indirect, or, tmp, <<21] 

    // how many bytes of packet payload are copied to tbuf from dram is 
	// controlled by Ref Cnt field in indir_ref, following max_16 is
	// dummy, offset is added to dramaddr to get the actual address to start 
	// fetch, and the lower 3 bits of the address is ignored by dram channel	 

    dram[tbuf_wr, --, in_dramaddr, in_offset, max_16], indirect_ref, \
		sig_done[in_sig_dram]

.end 

#endm	// _pkt_tx_16p_move_payload_to_tbuf()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_debug_init_counters()
//	
//	Description:
//		Initialize the debug counters to zero.
//
//	Outputs:	
//	
//		
//	Inputs:
//
//
//	CONSTANTS:	
//		None
//	
//	Globale variables:
//		The debug counters in absolute registers.
//
///////////////////////////////////////////////////////////////////////////////
#macro _pkt_tx_16p_debug_init_counters()

#ifdef	_DEBUG_COUNTERS_
.begin

	alu[@pkt_tx_16p_tx_request_rxed, --, B, 0]
	alu[@pkt_tx_16p_pkt_txed, --, B, 0]
	alu[@pkt_tx_16p_buffer_freed, --, B, 0]
	alu[@pkt_tx_16p_mpkts_txed, --, B, 0]

.end
#endif	// #ifdef	_DEBUG_COUNTERS_
#endm	// _pkt_tx_16p_debug_init_counters()


///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_debug_incr_counter()
//	
//	Description:
//		Increment the debug counter by one
//
//	Outputs:	
//		None
//		
//	Inputs:
//		io_counter:		counter to increment
//
//	Inputs:
//		None
//
//	CONSTANTS:	
//		None
//	
//	Instructions:		1
//
///////////////////////////////////////////////////////////////////////////////
#macro _pkt_tx_16p_debug_incr_counter(io_counter)

#ifdef	_DEBUG_COUNTERS_
.begin

	alu[io_counter,io_counter,+,1]

.end
#endif	// #ifdef	_DEBUG_COUNTERS_
#endm	//	_pkt_tx_16p_debug_incr_counter()


#ifdef ADD_L2_HEADER

///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_write_l2_header() 
//
// Description: write L2 header to tbuf through msf[write...]
//
// Outputs:							
// 		None
//
// Inputs/Otputs:  
//		io_sigmask:		signal mask
//
// Inputs:  
//      in_sig:			signal for prepend msf write
//		in_tbuf_addr:	the tbuf element address for L2 header 
//
// Constants:
//		None
//
// Size:
//		2 instructions (POS_TX)
//		5 instructions (ETHERNET_TX)
//
///////////////////////////////////////////////////////////////////////////////

#macro _pkt_tx_16p_write_l2_header(io_sigmask, in_sig, in_tbuf_addr)

.begin

#ifdef POS_TX

    // write  PPP header for SOP mpacket
	alu_shf[$prepend_0, --, b, PPP_IP_PROTOCOL, <<IP_PROTOCOL_SHFT]

    // write PPP header to tbuf

	msf[write, $prepend_0, in_tbuf_addr, 0, 1], sig_done[in_sig]
	_pkt_tx_16p_set_signal[io_sigmask, in_sig]

#endif	// POS_TX

#ifdef ETHERNET_TX

	// not valid L2 table entry case has already been handled in ethernet_arp 
	// microblock, so here all L2 table entry are valid

	alu[$l2_entry_lw0, --, b, $l2_entry_lw0]
	alu[$l2_entry_lw1, --, b, $l2_entry_lw1]
	alu[$l2_entry_lw2, --, b, $l2_entry_lw2]
	alu[$l2_entry_lw3, --, b, $l2_entry_lw3]

    // write Ethernet L2 header to tbuf

	msf[write, $l2_entry_lw0, in_tbuf_addr, 0, 4], sig_done[in_sig]
	_pkt_tx_16p_set_signal[io_sigmask, in_sig]
#endif	// ETHERNET_TX

.end 

#endm	// _pkt_tx_16p_write_l2_header()


#ifdef ETHERNET_TX
///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_read_l2_table_entry()  
//
// Description: read the L2 table entry data from sram
//
// Inputs/Outputs:  
//		io_sigmask:			signal mask to be added with sig_sram_read_2
// 
// Inputs:  
//		in_l2_table_base:	L2 table base
//		in_nexthop_id:		nexthop_id
//		in_sig:				signal for reading l2_table_entry
//
// Constants:
//		none   
//
// Size:
//		3 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro _pkt_tx_16p_read_l2_table_entry(io_sigmask, in_l2_table_base, \
											in_nexthop_id, in_sig)
.begin
.reg l2_entry_addr

	alu_shf[l2_entry_addr, --, B, in_nexthop_id, <<L2_TABLE_ENTRY_SHFT]
	sram[read, $l2_entry_lw0, in_l2_table_base, l2_entry_addr, \
			L2_TABLE_ENTRY_SIZE_IN_LW], sig_done[in_sig]
	_pkt_tx_16p_set_signal(io_sigmask, in_sig)	; add in_sig
	
.end // l2_entry_addr

#endm // end of macro _pkt_tx_16p_read_l2_table_entry()

#endif // #ifdef ETHERNET_TX
#endif // #ifdef ADD_L2_HEADER


#ifdef CHECK_PORT_QUEUE_OVERFLOW

///////////////////////////////////////////////////////////////////////////////
//
// _pkt_tx_16p_check_discard()
//
// Description: check whether the queue for the port is already full, if it is 
//				full, just discard this tx request, otherwise will overflood 
//				the queue
//
// Prerequisite:				local memory index 1 alrady set to queue status 
//								of the in_port
//
// Outputs: 
//		None
//
// Inputs:
//		None
//
// Constants:
//		None
//
//	Labels:
//		END_OF_SAVE_SOP_TO_LM_LABEL:	symbolic address to go when the packets 
//										in the queue is already full								    one mpacket's required
//
// Size: 
//			6 instructions 
//
///////////////////////////////////////////////////////////////////////////////

#macro 	_pkt_tx_16p_check_discard(END_OF_SAVE_SOP_TO_LM_LABEL) 

.begin 
	
	.reg	tmp

	// get current packets in queue for the port

	alu[tmp, PKT_IN_QUEUE_MASK, AND, lm1_port_queue_stat, \
														>>PKT_IN_QUEUE_SHFT]

	alu[--, tmp, -, QUEUE_DEPTH]
	blt[port_queue_not_full#]


	// maintain counters

	alu[lm1_tx_requests_discarded, lm1_tx_requests_discarded, +, 1]

	br[END_OF_SAVE_SOP_TO_LM_LABEL]

port_queue_not_full#:
															 
.end 

#endm	// _pkt_tx_16p_check_discard()


#endif	// #ifdef CHECK_PORT_QUEUE_OVERFLOW


///////////////////////////////////////////////////////////////////////////////
// _pkt_tx_16p_update_round_robin_vector()
//
// Description: updating round robin vector (@rr_vector)
//
// Outputs:					
//							None 
//
// Inputs/Outputs:  
//      io_rr_vector:		round robin vector
//
// Inputs:  
//		in_port:			port to transmit the current mpacket in processing
//
// Constants:			
//							None
//
//	Size:
//		4 instructions
//
///////////////////////////////////////////////////////////////////////////////
#macro 	_pkt_tx_16p_update_round_robin_vector(io_rr_vector, in_port)

.begin
 
	.reg	tmp
	.reg	minus_two

	alu[minus_two, --, ~b, 1]	; set to 0xFFFFFFFE
	alu[--, in_port, or, 0]
	alu_shf[tmp, --, B, minus_two, <<indirect]
	alu[io_rr_vector, --, b, tmp]
.end 
 
#endm	// _pkt_tx_16p_update_round_robin_vector()


#endif	// __PKT_TX_P16_UTIL_UC__
